/*************************************************************************
 *
 *  $RCSfile: registrycache.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:07 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __FRAMEWORK_REGISTRYCACHE_HXX_
#define __FRAMEWORK_REGISTRYCACHE_HXX_

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XREGISTRYKEY_HPP_
#include <com/sun/star/registry/XRegistryKey.hpp>
#endif

#ifndef _COM_SUN_STAR_UNO_EXCEPTION_HPP_
#include <com/sun/star/uno/Exception.hpp>
#endif

//_________________________________________________________________________________________________________________
//	other includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_
#include <com/sun/star/uno/Sequence.h>
#endif

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

#define	XSIMPLEREGISTRY				::com::sun::star::registry::XSimpleRegistry
#define	XREGISTRYKEY				::com::sun::star::registry::XRegistryKey

#define	EXCEPTION					::com::sun::star::uno::Exception

#define	MUTEX						::osl::Mutex
#define	OUSTRING					::rtl::OUString
#define	REFERENCE					::com::sun::star::uno::Reference
#define	SEQUENCE					::com::sun::star::uno::Sequence

//_________________________________________________________________________________________________________________
//	switches
//	Use these to de/activate some features of this implementation.
//	(for debugging or testing!)
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	exported const
//_________________________________________________________________________________________________________________

#define	VALID						sal_True		// Cache is correct inialized.
#define	INVALID						sal_False		// CAche is not ready for use.

//_________________________________________________________________________________________________________________
//	exported definitions
//_________________________________________________________________________________________________________________

// Define a dynamical stringlist with ::rtl::OUString as elementtype.
typedef ::std::vector< OUSTRING >	tIMPLStringList;

// Define a structure for ONE cache-item.
struct tIMPLRegistryEntry
{
    OUSTRING			sName			;		// The name of the entry.
    tIMPLStringList		aPatternList	;		// A stringlist of patterns for this entry.
    tIMPLStringList		aExtensionList	;		// A stringlist of extensions for this entry.
    tIMPLStringList		aMimeTypeList	;		// A stringlist of mimetypes for this entry.
};

// Define the cache himself as list of entries WITH a count.
typedef ::std::vector< tIMPLRegistryEntry >	tIMPLRegistryCache;

/*-************************************************************************************************************//**
	@short		cache of important registry entries for FrameLoaderFactory
	@descr		This class is not threadsafe!

	@implements	-
	@base		-
*//*-*************************************************************************************************************/

class RegistryCache
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard constructor
			@descr		This will not initialize the cache automaticly! Use method "fill()" to do that.

			@seealso	method fill()

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	 	RegistryCache();

		/*-****************************************************************************************************//**
			@short		standard destructor to delete instance
			@descr		This will clear the cache, if programmer forget this.

			@seealso	method clear()

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		virtual ~RegistryCache();

		/*-****************************************************************************************************//**
			@short		get the current state of the cache
			@descr		Call this method to get information about the state of the current cache.
						I make this method inline for better performance!

			@seealso	-

			@param		-
			@return		sal_True  = if cache is initialzed and work correct<BR>
						sal_False = other way.<BR>

			@onerror	The return value is sal_False.
		*//*-*****************************************************************************************************/

		sal_Bool isValid();

		/*-****************************************************************************************************//**
			@short		initialize the cache with new values
			@descr		This will fill the cache with new values. We make an clear() automaticly!
						The registry will not changed at runtime ... don't call this method again, if
						this cache is valid. We will do nothing, if it so.

			@seealso	method clear()

			@param		"xRegistry" is the registry to cache.
			@return		-

			@onerror	The cache state is set to INVALID!
		*//*-*****************************************************************************************************/

		void fill( const REFERENCE< XSIMPLEREGISTRY >& xRegistry );

		/*-****************************************************************************************************//**
			@short		clear the cache and free memory
			@descr		This will clear the cache. If you call this method again and the cache is empty, we do nothing!

			@seealso	method fill()

			@param		-
			@return		-

			@onerror	The cache state is set to INVALID!
		*//*-*****************************************************************************************************/

		void clear();

		sal_uInt32 getEntryCount();
		sal_uInt32 getPatternCount( sal_uInt32 nEntryNumber );
		sal_uInt32 getExtensionCount( sal_uInt32 nEntryNumber );
		sal_uInt32 getMimeTypeCount( sal_uInt32 nEntryNumber );

		OUSTRING getEntryName( sal_uInt32 nEntryNumber );
		OUSTRING getPattern(	sal_uInt32	nEntryNumber	,
								sal_uInt32	nPatternNumber	);
		OUSTRING getExtension(	sal_uInt32	nEntryNumber	,
								sal_uInt32	nExtensionNumber);
		OUSTRING getMimeType(	sal_uInt32	nEntryNumber	,
								sal_uInt32	nMimeTypeNumber	);

	//-------------------------------------------------------------------------------------------------------------
	//	protected methods
	//-------------------------------------------------------------------------------------------------------------

	protected:

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

	private:

		/*-****************************************************************************************************//**
			@short		set the value or the values of one registrykey on right stringlist of an cache-entry
			@descr		If the value a normal string or ascii-value, we will set this as the end of the given list.
						Is the value a sequence, we will set all values to the right list.

			@seealso	method fill()

			@param		"xKey"  is a reference to a valid registrykey (We will not control this!)
			@param		"aList" is a reference to a valid stringlist to append te value(s)
			@return		-

			@onerror	An exception can be thrown.
		*//*-*****************************************************************************************************/

		void impl_setKeyValueOnList(const	REFERENCE< XREGISTRYKEY >&	xKey	,
			 								tIMPLStringList&			aList	) throw ( EXCEPTION );

	//-------------------------------------------------------------------------------------------------------------
	//	debug methods
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	#ifdef DBG_UTIL

	private:

		sal_Bool impldbg_checkParameter_implsetKeyValueOnList(const	REFERENCE< XREGISTRYKEY >&	xKey	,
			 														tIMPLStringList&			aList	);

	#endif	//	#ifdef DBG_UTIL

	//-------------------------------------------------------------------------------------------------------------
	//	private variables
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	private:

		tIMPLRegistryCache	   					m_aRegistryCache	;

};		//	class RegistryCache

}		//	namespace framework

#endif	//	#ifndef __FRAMEWORK_REGISTRYCACHE_HXX_
