/*************************************************************************
 *
 *  $RCSfile: servicemanager.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __FRAMEWORK_SERVICEMANAGER_HXX_
#define __FRAMEWORK_SERVICEMANAGER_HXX_

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_OMUTEXMEMBER_HXX_
#include <helper/omutexmember.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

//_________________________________________________________________________________________________________________
//	other includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

#define	REFERENCE					::com::sun::star::uno::Reference
#define	XMULTISERVICEFACTORY		::com::sun::star::lang::XMultiServiceFactory
#define	OUSTRING					::rtl::OUString

//_________________________________________________________________________________________________________________
//	switches
//	Use these to de/activate some features of this implementation.
//	(for debugging or testing!)
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	exported definitions
//_________________________________________________________________________________________________________________

/*-************************************************************************************************************//**
	@short		-

	@descr		-

	@implements	-

	@base		OMutexMember
*//*-*************************************************************************************************************/

class ServiceManager	:	public OMutexMember						// Struct for right initalization of mutex member!
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard constructor

			@descr		This method don't initialize the new global servicemanager!
						Use getManager() function to do this.

			@seealso	-

			@param		-

			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	 	ServiceManager();

		/*-****************************************************************************************************//**
			@short		standard destructor to delete instance

			@descr		Here is a good place to destroy the global manager instance!

			@seealso	-

			@param		-

			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		virtual ~ServiceManager();

		/*-****************************************************************************************************//**
			@short		return a reference to the global servicemanager instance

			@descr		This method create a new manager only at first call.
						And then you can call this method everytime to get a reference to the manager.

			@seealso	-

			@param		-

			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		REFERENCE< XMULTISERVICEFACTORY > getManager();

	//-------------------------------------------------------------------------------------------------------------
	//	protected methods
	//-------------------------------------------------------------------------------------------------------------

	protected:

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

	private:

		/*-****************************************************************************************************//**
			@short		create a new global servicemanager instance

			@descr		-

			@seealso	method getManager()

			@param		-

			@return		A reference to a new initialized servicemanager with a valid registry.

			@onerror	Return a NULL-reference.
		*//*-*****************************************************************************************************/

		REFERENCE< XMULTISERVICEFACTORY > impl_createManager();

		/*-****************************************************************************************************//**
			@short		return the name of the system registry

			@descr		Normaly the name is "...applicat.rdb". We use UNC-notation for filename!

			@seealso	method impl_createManager()
			@seealso	osl::FileBase:: ... UNC-notation

			@param		-

			@return		A filename of system registry in unc-notation.

			@onerror	An empty string.
		*//*-*****************************************************************************************************/

		OUSTRING impl_getNameSystemRegistry();

		/*-****************************************************************************************************//**
			@short		return the name of the local (user) registry

			@descr		Normaly the name is "...user.rdb". We use UNC-notation for filename!

			@seealso	method impl_createManager()
			@seealso	osl::FileBase:: ... UNC-notation

			@param		-

			@return		A filename of local registry in unc-notation.

			@onerror	An empty string.
		*//*-*****************************************************************************************************/

		OUSTRING impl_getNameLocalRegistry();

		/*-****************************************************************************************************//**
			@short		return the name of the default local registry

			@descr		Normaly the name is "...user.rdb". We use UNC-notation for filename!

			@seealso	method impl_createManager()
			@seealso	osl::FileBase:: ... UNC-notation

			@param		-

			@return		A filename of system registry in unc-notation.

			@onerror	An empty string.
		*//*-*****************************************************************************************************/

		OUSTRING impl_getNameDefaultLocalRegistry();

		/*-****************************************************************************************************//**
			@short		check state of a registry file

			@descr		We test, if file exist and can be read. We use a filename in unc-notation.

			@seealso	method impl_createManager()
			@seealso	method impl_getNameSystemRegistry()
			@seealso	method impl_getNameLocalRegistry()
			@seealso	method impl_getNameDefaultLocalRegistry()
			@seealso	osl::FileBase:: ... UNC-notation

			@param		"sUNCName" is the filename of the registry to check.

			@return		sal_True  = if registry exist and can be read<BR>
						sal_False = other way

			@onerror	Return sal_False
		*//*-*****************************************************************************************************/

		sal_Bool impl_checkRegistry( const OUSTRING& sUNCName );

	//-------------------------------------------------------------------------------------------------------------
	//	debug methods
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	#ifdef DBG_UTIL

	private:

	#endif	//	#ifdef DBG_UTIL

	//-------------------------------------------------------------------------------------------------------------
	//	private variables
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	private:

};		//	class ServiceManager

}		//	namespace framework

#endif	//	#ifndef __FRAMEWORK_SERVICEMANAGER_HXX_
