/*************************************************************************
 *
 *  $RCSfile: urltransformer.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_URLTRANSFORMER_HXX_
#include <baeh_services/urltransformer.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl							;
using namespace ::rtl							;
using namespace ::cppu							;
using namespace ::com::sun::star::uno			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::util			;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
URLTransformer::URLTransformer( const Reference< XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      OMutexMember is a struct with a mutex as member. We can't use a mutex as member, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember	(			)
		,	OWeakObject		(			)
		// Init member
		,	m_xFactory		( xFactory	)
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
URLTransformer::~URLTransformer()
{
	// Reset instance, free memory ....
	impl_resetObject();
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************

DEFINE_XINTERFACE_3					(	URLTransformer						,
										OWeakObject					  		,
										DIRECT_INTERFACE(XTypeProvider		),
										DIRECT_INTERFACE(XServiceInfo		),
										DIRECT_INTERFACE(XURLTransformer	)
									)

DEFINE_XTYPEPROVIDER_3				(	URLTransformer	,
										XTypeProvider	,
										XServiceInfo	,
										XURLTransformer
									)

DEFINE_XSERVICEINFO_MULTISERVICE	(	URLTransformer								,
										"com.sun.star.util.URLTransformer"			,
										"com.sun.star.comp.framework.URLTransformer"
									)

//*****************************************************************************************************************
//	XURLTransformer
//*****************************************************************************************************************
sal_Bool SAL_CALL URLTransformer::parseStrict( URL& aURL ) throw( RuntimeException )
{
	INetURLObject aObj( aURL.Complete );

	aURL.Protocol	= INetURLObject::GetScheme( aObj.GetProtocol() );
	aURL.User		= aObj.GetUser( INetURLObject::NO_DECODE );
	aURL.Password	= aObj.GetPass( INetURLObject::NO_DECODE );
	aURL.Server		= aObj.GetHost( INetURLObject::NO_DECODE );
	aURL.Port		= aObj.GetPort();
	aURL.Path		= aObj.GetURLPath( INetURLObject::NO_DECODE );
//	aURL.Name = StringToOUString( aObj.GetName(), CHARSET_SYSTEM );
	aURL.Arguments	= aObj.GetParam( INetURLObject::NO_DECODE );
	aURL.Mark		= aObj.GetMark( INetURLObject::NO_DECODE );
	aObj.SetMark	( OUString() );
	aObj.SetParam	( OUString() );
	aURL.Main		= aObj.GetMainURL( INetURLObject::NO_DECODE );

	return sal_True;
}

//*****************************************************************************************************************
//	XURLTransformer
//*****************************************************************************************************************
sal_Bool SAL_CALL URLTransformer::parseSmart(			URL&		aURL			,
												const	OUString&	sSmartProtocol	) throw( RuntimeException )
{
	String aProt( aSmartProt );
	String aSmartURL = INetURLObject::SmartRelToAbs( aURL.Complete );
	INetURLObject aObj;
	aObj.SetSmartProtocol( INetURLObject::CompareProtocolScheme( aProt ) );
	aObj.SetSmartURL( aSmartURL );

	aURL.Protocol = INetURLObject::GetScheme( aObj.GetProtocol() );
	aURL.User = aObj.GetUser( INetURLObject::NO_DECODE );
	aURL.Password = aObj.GetPass( INetURLObject::NO_DECODE );
	aURL.Server = aObj.GetHost( INetURLObject::NO_DECODE );
	aURL.Port = aObj.GetPort();
	aURL.Path = aObj.GetURLPath( INetURLObject::NO_DECODE );
//	aURL.Name = StringToOUString( aObj.GetName(), CHARSET_SYSTEM );
	aURL.Arguments = aObj.GetParam( INetURLObject::NO_DECODE );
	aURL.Mark = aObj.GetMark( INetURLObject::NO_DECODE );
	aObj.SetMark( String() );
	aObj.SetParam( String() );
	aURL.Main = aObj.GetMainURL( INetURLObject::NO_DECODE );
	return sal_True;
}

//*****************************************************************************************************************
//	XURLTransformer
//*****************************************************************************************************************
sal_Bool SAL_CALL URLTransformer::assemble( URL& aURL ) throw( RuntimeException )
{
	INetURLObject aObj;

	aObj.ConcatData( INetURLObject::CompareProtocolScheme( aURL.Protocol ),
					 aURL.User, aURL.Password, aURL.Server,
					 aURL.Port, aURL.Path );

	// Zun"achst URL ohne Parameter und Mark
	aURL.Main = aObj.GetMainURL( INetURLObject::NO_DECODE );

	// Dann Parameter und Mark dazu
	aObj.SetParam( aURL.Arguments );
	aObj.SetMark( aURL.Mark );
	aURL.Complete = aObj.GetMainURL( INetURLObject::NO_DECODE );

	return sal_True;
}

//*****************************************************************************************************************
//	XURLTransformer
//*****************************************************************************************************************
OUString SAL_CALL URLTransformer::getPresentation(	const	URL&		aURL			,
															sal_Bool	bWithPassword	) throw( RuntimeException )
{
	return aURL.Complete;
}

//*****************************************************************************************************************
//	proteced method
//*****************************************************************************************************************
void URLTransformer::impl_resetObject()
{
	// Attention:
	// Write this for multiple calls - NOT AT THE SAME TIME - but for more then one call again)!
	// It exist two ways to call this method. From destructor and from disposing().
	// I can't say, which one is the first. Normaly the disposing-call - but other way ....
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact Andreas Schluens [as] or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef DBG_UTIL
/*
//*****************************************************************************************************************
sal_Bool URLTransformer::impldbg_checkParameter_URLTransformer( const Sequence< Reference< XInterface > >& seqTasks )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &seqTasks				==	NULL	)	||
			( seqTasks.getLength()	<=	0		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}
*/
#endif	//	#ifdef DBG_UTIL

}		//	namespace framework
