/*************************************************************************
 *
 *  $RCSfile: registrycache.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_REGISTRYCACHE_HXX_
#include <classes/registrycache.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl						;
using namespace ::rtl						;
using namespace ::com::sun::star::uno		;
using namespace ::com::sun::star::registry	;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

#define	REGISTRYKEY_LOADER								OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader"))
#define	REGISTRYKEY_IMPLEMENTATIONS						OUString(RTL_CONSTASCII_USTRINGPARAM("/IMPLEMENTATIONS/"))
#define	REGISTRYKEY_LOADERPATTERN						OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/Pattern"))
#define	REGISTRYKEY_LOADEREXTENSIONS					OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/Extension"))
#define	REGISTRYKEY_LOADERMIMETYPE						OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/MimeType"))

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
RegistryCache::RegistryCache()
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
RegistryCache::~RegistryCache()
{
	// Don't forget to free memory!
	if ( isValid() == sal_True )
	{
		clear();
	}
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_Bool RegistryCache::isValid()
{
	// If we have some elements in cache ... cache is valid.
	// Other way ... invalid.
	return ( m_aRegistryCache.size() > 0 );
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
void RegistryCache::fill( const Reference< XSimpleRegistry >& xRegistry )
{
	try
	{

	// Do nothing if cache already filled!
	if ( isValid() == INVALID )
	{
		// Exist a registry?
        if ( xRegistry.is() == sal_True )
        {
			// YES; the get access to the entries.
            Reference< XRegistryKey > xRootKey	= xRegistry->getRootKey();
            Reference< XRegistryKey > xKey		= xRootKey->openKey( REGISTRYKEY_LOADER );

			// Warn programmer, if no loader registered in applicat.rdb or user.rdb!
			OSL_ENSHURE( !(xKey.is() == sal_False), "RegistryCache::fill()\nNo loader found in registry!\nCan't fill cache with values.\n" );

            if ( xKey.is() == sal_True )
			{
				if ( xKey->getValueType() == RegistryValueType_ASCIILIST )
            	{
					// Get names of all subkeys of "/Loader". These are the names of all well known loader's.
                	Sequence< OUString >	seqSubkeys	= xKey->getAsciiListValue();
                	const OUString*			pArray		= seqSubkeys.getConstArray();
					sal_Int32				nPosition	= 0;
					sal_Int32				nLength		= seqSubkeys.getLength();

					// Cache all important informations of this subkeys!
                	for ( nPosition=0; nPosition<nLength; ++nPosition )
                	{
						// Create a new entry for cache.
						// We don't must use the heap! The stl-classes allocate his own memory.
                    	tIMPLRegistryEntry aEntry;

						// Set name of cached entry.
						aEntry.sName = pArray[nPosition];

						// Build full entryname in registry.
						// ("/IMPLEMENTATIONS/<LoaderName>")
                    	OUString sBaseKeyName = REGISTRYKEY_IMPLEMENTATIONS;
                    	sBaseKeyName += pArray[nPosition];

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/Pattern" in cache
                    	OUString sPatternKeyName = sBaseKeyName;
                    	sPatternKeyName += REGISTRYKEY_LOADERPATTERN;

                    	xKey = xRootKey->openKey( sPatternKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnList( xKey, aEntry.aPatternList );
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/Extensions" in cache.
                    	OUString sExtensionsKeyName = sBaseKeyName;
                    	sExtensionsKeyName += REGISTRYKEY_LOADEREXTENSIONS;

                    	xKey = xRootKey->openKey( sExtensionsKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnList( xKey, aEntry.aExtensionList );
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/MimeType" in cache.
                    	OUString sMimeTypeKeyName = sBaseKeyName;
                    	sMimeTypeKeyName += REGISTRYKEY_LOADERMIMETYPE;

                    	xKey = xRootKey->openKey( sMimeTypeKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnList( xKey, aEntry.aMimeTypeList );
                    	}

						// Insert entry in cache.
	                    m_aRegistryCache.push_back( aEntry );
                	}
            	}
        	}
    	}
	}

	// If something wrong ... delete cache and set state to INVALID!
	// (INVALID => cache is empty)
	}
	catch( Exception& )
	{
		clear();
	}
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
void RegistryCache::clear()
{
	// Clear entrylist.
	m_aRegistryCache.clear();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getEntryCount()
{
	// Return the current size of entrylist.
	return (sal_uInt32)m_aRegistryCache.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getPatternCount( sal_uInt32 nEntryNumber )
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getPatternCount()\nThis entry not exist!\n" );
	// Return size of pattern list.
	return (sal_uInt32)m_aRegistryCache[nEntryNumber].aPatternList.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getExtensionCount( sal_uInt32 nEntryNumber )
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getExtensionCount()\nThis entry not exist!\n" );
	// Return size of extension list.
	return (sal_uInt32)m_aRegistryCache[nEntryNumber].aExtensionList.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getMimeTypeCount( sal_uInt32 nEntryNumber )
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getMimeTypeCount()\nThis entry not exist!\n" );
	// Return size of mimetype list.
	return (sal_uInt32)m_aRegistryCache[nEntryNumber].aMimeTypeList.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getEntryName( sal_uInt32 nEntryNumber )
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getEntryName()\nThis entry not exist!\n" );
	// Return name of entry.
	return m_aRegistryCache[nEntryNumber].sName;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getPattern(	sal_uInt32	nEntryNumber	,
		 							sal_uInt32	nPatternNumber	)
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber  >=m_aRegistryCache.size()							 ), "RegistryCache::getPattern()\nThis entry not exist!\n"	);
	OSL_ENSHURE( !(nPatternNumber>=m_aRegistryCache[nEntryNumber].aPatternList.size()), "RegistryCache::getPattern()\nThis pattern not exist!\n" );
	// Return value of pattern.
	return m_aRegistryCache[nEntryNumber].aPatternList[nPatternNumber];
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getExtension(	sal_uInt32	nEntryNumber	,
		 								sal_uInt32	nExtensionNumber)
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber    >=m_aRegistryCache.size()							 ), "RegistryCache::getExtension()\nThis entry not exist!\n"	 );
	OSL_ENSHURE( !(nExtensionNumber>=m_aRegistryCache[nEntryNumber].aExtensionList.size()), "RegistryCache::getExtension()\nThis extension not exist!\n" );
	// Return value of extension.
	return m_aRegistryCache[nEntryNumber].aExtensionList[nExtensionNumber];
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUSTRING RegistryCache::getMimeType(	sal_uInt32	nEntryNumber	,
		 								sal_uInt32	nMimeTypeNumber	)
{
	// Safe impossible cases
	OSL_ENSHURE( !(nEntryNumber   >=m_aRegistryCache.size()							   ), "RegistryCache::getMimeType()\nThis entry not exist!\n"	 );
	OSL_ENSHURE( !(nMimeTypeNumber>=m_aRegistryCache[nEntryNumber].aMimeTypeList.size()), "RegistryCache::getMimeType()\nThis mimetype not exist!\n" );
	// Return value of mimetype.
	return m_aRegistryCache[nEntryNumber].aMimeTypeList[nMimeTypeNumber];
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_setKeyValueOnList(	const	Reference< XRegistryKey >&	xKey	,
													tIMPLStringList&			aList	) throw( Exception )
{
	// Safe impossible cases
	// We need correct parameter to work.
	OSL_ENSHURE( impldbg_checkParameter_implsetKeyValueOnList( xKey, aList ), "RegistryCache::impl_setKeyValueOnList()\nInvalid parameter detected!\n" );

	// Get the right type of the keyvalue, to use right read-method!
    RegistryValueType aType = xKey->getValueType();
    switch ( aType )
    {
        case RegistryValueType_ASCII		:	{
													// Get the value as Ascii from registry and append it on right list.
													aList.push_back( xKey->getAsciiValue() );
												}
                               					break;
        case RegistryValueType_STRING		:	{
													// Get the value as String from registry and append it on right list.
													aList.push_back( xKey->getStringValue() );
												}
                                				break;
        case RegistryValueType_ASCIILIST	:	{
													// Get the value's as ASCII-list from registry and append it on right list.
													Sequence< OUString >	seqValues	= xKey->getAsciiListValue();
        											const OUString*			pArray		= seqValues.getConstArray();
        											sal_Int32	   			nLength		= seqValues.getLength();
													sal_Int32	   			nCounter	= 0;

        											for ( nCounter=0; nCounter<nLength; ++nCounter )
													{
            											aList.push_back( pArray[nCounter] );
													}
												}
                               					break;
        case RegistryValueType_STRINGLIST	:	{
													// Get the value's as String-list from registry and append it on right list.
													Sequence< OUString >	seqValues	= xKey->getAsciiListValue();
        											const OUString*			pArray		= seqValues.getConstArray();
        											sal_Int32	   			nLength		= seqValues.getLength();
													sal_Int32	   			nCounter	= 0;

        											for ( nCounter=0; nCounter<nLength; ++nCounter )
													{
            											aList.push_back( pArray[nCounter] );
													}
												}
                               					break;
    }
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact Andreas Schluens [as] or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef DBG_UTIL

//*****************************************************************************************************************
sal_Bool RegistryCache::impldbg_checkParameter_implsetKeyValueOnList(	const	Reference< XRegistryKey >&	xKey	,
																				tIMPLStringList&			aList	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xKey		==	NULL		)	||
			( xKey.is()	==	sal_False	)	||
			( &aList	==	NULL		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef DBG_UTIL

}		//	namespace framework
