/*************************************************************************
 *
 *  $RCSfile: servicemanager.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_SERVICEMANAGER_HXX_
#include <classes/servicemanager.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_LANG_XINITIALIZATION_HPP_
#include <com/sun/star/lang/XInitialization.hpp>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _CPPUHELPER_SERVICEFACTORY_HXX_
#include <cppuhelper/servicefactory.hxx>
#endif

#ifndef _VOS_PROCESS_HXX_
#include <vos/process.hxx>
#endif

#ifndef _VOS_PROFILE_HXX_
#include <vos/profile.hxx>
#endif

#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	includes of my own project
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl									;
using namespace ::rtl									;
using namespace ::vos									;
using namespace ::com::sun::star::uno					;
using namespace ::com::sun::star::lang					;
using namespace ::com::sun::star::registry				;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

#define	SERVICENAME_XSIMPLEREGISTRY						OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.registry.XSimpleRegistry" ))

#define	UNC_PATHDELEMITTER								sal_Unicode('/')
#define	REGISTRY_KEYDELIMITTER							'/'

#define	NAME_SYSTEMREGISTRY								OUString( RTL_CONSTASCII_USTRINGPARAM( "applicat.rdb" ))
#define NAME_LOCALREGISTRY								OUString( RTL_CONSTASCII_USTRINGPARAM( "user52.rdb" ))

#define ENVIRONMENTNAME_SYSTEMREGISTRY					"STAR_REGISTRY"
#define ENVIRONMENTNAME_USERREGISTRY	 				"STAR_USER_REGISTRY"

#define	REGISTRY_VERSIONKEY								"UserRegistry 5.2"
#define REGISTRY_SECTION								"Registry"

#define SVERSION_NAME									OUString( RTL_CONSTASCII_USTRINGPARAM( "sversion" ))
#define SVERSION_LOCATION								OUString( RTL_CONSTASCII_USTRINGPARAM( "?^" ))
#define SVERSION_FALLBACK								OUString( RTL_CONSTASCII_USTRINGPARAM( "?$" ))

#define LOCALREGISTRY_DEFAULTLOCATION					OUString( RTL_CONSTASCII_USTRINGPARAM( "?^" ))

#define	MAXLENGTH_INILINE								1024

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
ServiceManager::ServiceManager()
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      tIMPLMutexContainer is a struct with a mutex as member. We can't use a mutex as member, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember			(			)
		// Init member
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
ServiceManager::~ServiceManager()
{
}

//*****************************************************************************************************************
//	static method
//*****************************************************************************************************************
Reference< XMultiServiceFactory > ServiceManager::getManager()
{
	// Create one manager for all instances of this class.
	// Optimize this method
	// We initialize a static variable only one time. And we don't must use a mutex at every call!
	// For the first call; pManager is NULL - for the second call it is different from NULL and can be use directly!
	static Reference< XMultiServiceFactory >* pManager = NULL ;
	if ( pManager == NULL )
	{
		// Ready for multithreading;
		// use global mutex for first call of this method only! see before
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		// Control these pointer again ... it can be, that another instance will be faster then these!
		// (But i think, it will exist one instance of THIS class only in one application ....)
		if ( pManager == NULL )
		{
			// Create a new static Manager ...
			static Reference< XMultiServiceFactory > xManager = impl_createManager();
			// ... and set his address to static pointer!
			pManager = &xManager;
		}
	}
	// Return reference to static manager instance.
	return (*pManager);
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
Reference< XMultiServiceFactory > ServiceManager::impl_createManager()
{
	// Set default return value, if method failed.
	Reference< XMultiServiceFactory > xReturn = Reference< XMultiServiceFactory >();

	/*
		HACK
		Versuche direkt einen Manager zu bekommen!
	 */

//	OUString sUser		( RTL_CONSTASCII_USTRINGPARAM( "user.rdb"		) );
	OUString sApplicat	( RTL_CONSTASCII_USTRINGPARAM( "applicat.rdb"	) );
//	xReturn = ::cppu::createRegistryServiceFactory( sUser, sApplicat );
	xReturn = ::cppu::createRegistryServiceFactory( sApplicat );
/*
	try
	{
		// Get default manager from cppu.
		Reference< XMultiServiceFactory > xManager = ::cppu::createServiceFactory();
		// Is there a valid manager?
		if ( xManager.is() == sal_True )
		{
			// Yes - but we must init this new instance!
			Reference< XInitialization > xInit( xManager, UNO_QUERY );

			// Safe impossible cases
			// A servicemanager MUST have an XInitialization interface!
			OSL_ENSHURE( !(xInit.is()==sal_False), "ServiceManager::impl_createManager()\nServiceManager has no XInitialization interface!?" );

			// Get the name of the local and the system registry files.
			sal_Bool	bDefaultRegistry	= sal_False;
			OUString	sNameLocalRegistry	= impl_getNameLocalRegistry();
			OUString	sNameSystemRegistry	= impl_getNameSystemRegistry();

			// If no local registry exist ...
			if ( sNameLocalRegistry.getLength() < 1 )
			{
				// ... use a default registry!
				sNameLocalRegistry	= impl_getNameDefaultLocalRegistry();
				bDefaultRegistry	= sal_True;
			}

			// If no names of registry well known, do nothing!
			if	(
					( sNameLocalRegistry.getLength()	>	0	)	&&
					( sNameSystemRegistry.getLength()	>	0	)
				)
			{
				// Create an access to the registries.
				Reference< XSimpleRegistry > xLocalRegistry	( xManager->createInstance( SERVICENAME_XSIMPLEREGISTRY ), UNO_QUERY );
				Reference< XSimpleRegistry > xSystemRegistry( xManager->createInstance( SERVICENAME_XSIMPLEREGISTRY ), UNO_QUERY );

				// Open the local registry.
				if ( xLocalRegistry.is() == sal_True )
				{
					xLocalRegistry->open( sNameLocalRegistry, sal_False, bDefaultRegistry );
					if ( xLocalRegistry->isValid() == sal_False )
					{
						xLocalRegistry->open( sNameLocalRegistry, sal_True, bDefaultRegistry );
					}
				}

				// Open the system registry.
				if ( xSystemRegistry.is() == sal_True )
				{
					xSystemRegistry->open( sNameSystemRegistry, sal_True, sal_False );
				}

				// Initalize manager.
				if	(
						( xLocalRegistry.is()			==	sal_True	)	&&
						( xLocalRegistry->isValid()		==	sal_True	)	&&
			 			( xSystemRegistry.is()			==	sal_True	)	&&
						( xSystemRegistry->isValid()	==	sal_True	)
					)
				{
					Sequence< Any > seqAnys(2);
					seqAnys[0] <<= xLocalRegistry	;
					seqAnys[1] <<= xSystemRegistry	;
					xInit->initialize( seqAnys );

					// At this point we have a valid global servicemanager!
					// Set reference as result of this operation.
					xReturn = xManager;
				}
			}
		}
	}
	catch( Exception& )
	{
	}
*/
	// Safe impossible cases
	OSL_ENSHURE( !(xReturn.is() == sal_False), "ServiceManager::impl_createManager()I have no valid manager created!\nMethod getManager() will not return a valid reference.\n" );

	// Return result of this operation.
	return xReturn;
}
//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
OUString ServiceManager::impl_getNameSystemRegistry()
{
	/*-************************************************************************************************************
		HACK

		Da OStartUpInfo eigentlich mit normalisierten UNC-Pfaden arbeiten sollte, das aber noch nicht tut :-(
		mu hier noch explizit gewandelt werden! Wenn die Umstellung dann perfekt sein sollte, kann alles, was
		sich um die Variable "sHack" dreht geaendert werden.

		HACK
	 *************************************************************************************************************/
	// Set default return value.
	OUString	sUNCReturnName	;

	// Search in the directory of the executable.
	OUString	sHack				;
	OUString	sUNCApplicationName	;
	// Get the name of the application.
	if ( OStartupInfo().getExecutableFile( sHack ) == OStartupInfo::E_None )
	{
		FileBase::normalizePath( sHack, sUNCApplicationName );

		// Work on temp name. If registry ok, set this value to return variable!
		OUString sUNCRegistryName;

		// Get directory of application.
		// ( zB. c|//test/app.exe => c|//test )
		sal_Int32 nLastIndex = sUNCApplicationName.lastIndexOf( UNC_PATHDELEMITTER );
		if ( nLastIndex > 0 )
		{
			sUNCRegistryName = sUNCApplicationName.copy( 0, nLastIndex );
		}

		// Add delemitter and name of registry to directory.
		// ( zB. c|//test => c|//test/applicat.rdb )
		sUNCRegistryName += OUString::valueOf(UNC_PATHDELEMITTER);
		sUNCRegistryName += NAME_SYSTEMREGISTRY;

		// Check, if registry exist and valid.
		if ( impl_checkRegistry( sUNCRegistryName ) == sal_False )
		{
			// Registry not exist ...
			// ... search the environment STAR_REGISTRY.
			OUString sSystemName = OUString::createFromAscii( getenv( ENVIRONMENTNAME_SYSTEMREGISTRY ) );
			FileBase::normalizePath( sSystemName, sUNCRegistryName );
			// Set value to return variable, if registry was found!
  			if ( impl_checkRegistry( sUNCRegistryName ) == sal_True )
			{
				sUNCReturnName = sUNCRegistryName;
			}
		}
	}

	// Return result of this operation.
	return sUNCReturnName;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
OUString ServiceManager::impl_getNameLocalRegistry()
{
	sal_Bool bFound = sal_False;

	// Set default return value.
	// First search the environment STAR_USER_REGISTRY.
	OUString sSystemName = OUString::createFromAscii( getenv( ENVIRONMENTNAME_USERREGISTRY ) );
	OUString sUNCRegistryName;
	FileBase::normalizePath( sSystemName, sUNCRegistryName );

	bFound = impl_checkRegistry( sUNCRegistryName );
	if ( bFound == sal_False )
	{
		// We have not found a valid registry.
		// Search for entry in sversion.ini in config directory.
		OUString sINIFileName;
		if	( OProfile::getProfileName( sINIFileName, SVERSION_NAME, SVERSION_LOCATION ) == sal_True )
		{
			// Open sversion.ini ...
			OProfile aProfile;
			if ( aProfile.open( sINIFileName ) == sal_True )
			{
				// Initialize buffer to get version key.
				// cBuffer = "" ... It must be! There are unicode fragments in memory!!!
				sal_Char cBuffer[MAXLENGTH_INILINE]	= "";

				// Build keyname for sversion.ini.
				// zB. "UserRegistry 5.2/583"
				OString	sUserRegistryKey( REGISTRY_VERSIONKEY );
				sUserRegistryKey += OString::valueOf( REGISTRY_KEYDELIMITTER );
				sUserRegistryKey += OString::valueOf((sal_Int32)SUPD);

				// Try to read key.
				if ( aProfile.readString( REGISTRY_SECTION, sUserRegistryKey.getStr(), cBuffer, MAXLENGTH_INILINE, NULL ) == sal_True )
				{
					// If it exist - set return value and breakk follow searching!
					sSystemName			= OUString::createFromAscii( cBuffer );
					sUNCRegistryName	= OUString();
					FileBase::normalizePath( sSystemName, sUNCRegistryName );
					bFound = impl_checkRegistry( sUNCRegistryName );
				}

				// Registry don't exist ...
				if ( bFound == sal_False )
				{
					// ... try to read another key.
					if ( aProfile.readString( REGISTRY_SECTION, REGISTRY_VERSIONKEY, cBuffer, MAXLENGTH_INILINE, NULL ) == sal_True )
					{
						// If it exist - set return value and breakk follow searching!
						sSystemName			= OUString::createFromAscii( cBuffer );
						sUNCRegistryName	= OUString();
						FileBase::normalizePath( sSystemName, sUNCRegistryName );
						bFound = impl_checkRegistry( sUNCRegistryName );
					}
				}
			}
		}
	}

	if ( bFound == sal_False )
	{
		// We have not found a valid registry.
		// Search for entry in sversion.ini in system directory.
		OUString sINIFileName;
		if	( OProfile::getProfileName( sINIFileName, SVERSION_NAME, SVERSION_FALLBACK ) == sal_True )
		{
			// Open sversion.ini ...
			OProfile aProfile;
			if ( aProfile.open( sINIFileName ) == sal_True )
			{
				// Initialize buffer to get version key.
				// cBuffer = "" ... It must be! There are unicode fragments in memory!!!
				sal_Char cBuffer[MAXLENGTH_INILINE]	= "";

				// Build keyname for sversion.ini.
				// zB. "UserRegistry 5.2/583"
				OString	sUserRegistryKey( REGISTRY_VERSIONKEY );
				sUserRegistryKey += OString::valueOf(REGISTRY_KEYDELIMITTER);
				sUserRegistryKey += OString::valueOf((sal_Int32)SUPD);

				// Try to read key.
				if ( aProfile.readString( REGISTRY_SECTION, sUserRegistryKey.getStr(), cBuffer, MAXLENGTH_INILINE, NULL ) == sal_True )
				{
					// If it exist - set return value and breakk follow searching!
					sSystemName			= OUString::createFromAscii( cBuffer );
					sUNCRegistryName	= OUString();
					FileBase::normalizePath( sSystemName, sUNCRegistryName );
					bFound = impl_checkRegistry( sUNCRegistryName );
				}

				// Registry don't exist ...
				if ( bFound == sal_False )
				{
					// ... try to read another key.
					if ( aProfile.readString( REGISTRY_SECTION, REGISTRY_VERSIONKEY, cBuffer, MAXLENGTH_INILINE, NULL ) == sal_True )
					{
						// If it exist - set return value and breakk follow searching!
						sSystemName			= OUString::createFromAscii( cBuffer );
						sUNCRegistryName	= OUString();
						FileBase::normalizePath( sSystemName, sUNCRegistryName );
						bFound = impl_checkRegistry( sUNCRegistryName );
					}
				}
			}
		}
	}

	if ( bFound == sal_False )
	{
		// We have not found a valid registry.
		// Use default security of logged in user.
		OSecurity	aSecurity	;
		OUString	sConfigDir	;

		if ( aSecurity.getConfigDir( sConfigDir ) == sal_True )
		{
			// Search without dot.
			// zB. c|//user/user.rdb
			sUNCRegistryName = OUString();
			FileBase::normalizePath( sConfigDir, sUNCRegistryName );
			sUNCRegistryName += OUString::valueOf(UNC_PATHDELEMITTER);
			sUNCRegistryName += NAME_LOCALREGISTRY;
			bFound = impl_checkRegistry( sUNCRegistryName );
		}

		if ( bFound == sal_False )
		{
			// Search with dot.
			// zB. c|//user/.user.rdb
			sUNCRegistryName = OUString();
			FileBase::normalizePath( sConfigDir, sUNCRegistryName );
			sUNCRegistryName += OUString::valueOf(UNC_PATHDELEMITTER);
			sUNCRegistryName += OUString::valueOf(sal_Unicode('.'));
			sUNCRegistryName += NAME_LOCALREGISTRY;
			bFound = impl_checkRegistry( sUNCRegistryName );
		}
	}

	if ( bFound == sal_False )
	{
		// We have not found a registry.
		// Reset return value!
		sUNCRegistryName = OUString();
	}

	// Return result of this operation.
	return sUNCRegistryName;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
OUString ServiceManager::impl_getNameDefaultLocalRegistry()
{
	// set default return value.
	OUString sUNCRegistryName;
/*
	// Try to get registry name.
	OUString sBuffer;
	if	( OProfile::getProfileName( sBuffer, NAME_LOCALREGISTRY, LOCALREGISTRY_DEFAULTLOCATION ) == sal_True )
	{
		// We have a registry found ... set return value.
		sUNCRegistryName = sBuffer;
	}
*/
	// Return result of this operation.
	return sUNCRegistryName;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
sal_Bool ServiceManager::impl_checkRegistry( const OUString& sUNCName )
{
	// Set default return value.
	sal_Bool bExist = sal_False;

/*ASMUSS
	File::open und close() sind noch nicht implementiert!? :-(
	Erstmal sich mit einem DirectoryItem behelfen...

	// Open the registry to check his state.
	// If we can the file open readonly ... it must be exist and can be read!

	// Attention:	File::File() do not allow relativ paths!
	//				Use absolute values.
	File aRegistry( sUNCName );
	if ( aRegistry.open( OpenFlag_Read ) == File::E_None )
	{
		// Set return value for existing file.
		bExist = sal_True;
	}
	aRegistry.close();
*/
	DirectoryItem aItem;
	if ( DirectoryItem::get( sUNCName, aItem ) == FileBase::E_None )
	{
		bExist = sal_True  ;
	}

	// Return result of this operation.
	return bExist;
}

}	//	namespace framework
