/*************************************************************************
 *
 *  $RCSfile: frame.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_FRAME_HXX_
#include <services/frame.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_AWT_XDEVICE_HPP_
#include <com/sun/star/awt/XDevice.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_XTOPWINDOW_HPP_
#include <com/sun/star/awt/XTopWindow.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_POSSIZE_HPP_
#include <com/sun/star/awt/PosSize.hpp>
#endif

#ifndef _COM_SUN_STAR_FRAME_FRAMESEARCHFLAG_HPP_
#include <com/sun/star/frame/FrameSearchFlag.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif

#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif

#ifndef _CPPUHELPER_FACTORY_HXX_
#include <cppuhelper/factory.hxx>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl										;
using namespace ::rtl										;
using namespace ::cppu										;
using namespace ::com::sun::star::uno						;
using namespace ::com::sun::star::lang						;
using namespace ::com::sun::star::frame						;
using namespace ::com::sun::star::task						;
using namespace ::com::sun::star::awt						;
using namespace ::com::sun::star::util						;
using namespace ::com::sun::star::beans						;
using namespace ::com::sun::star::container					;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

#define	SERVICENAME								"com.sun.star.framework.Frame"					// service-name of this class
#define	IMPLEMENTATIONNAME						"framework.Frame"								// implementation-name of this class

#define	FRAMETYPE_SELF							"_self"
#define	FRAMETYPE_PARENT						"_parent"
#define	FRAMETYPE_TOP							"_top"
#define	FRAMETYPE_BLANK							"_blank"
#define	FRAMETYPE_DOCUMENT						"_document"
#define	FRAMETYPE_BEAMER						"_beamer"
#define	FRAMETYPE_EXPLORER						"_explorer"
#define	FRAMETYPE_PARTWINDOW					"_partwindow"

#define	EQUAL									0

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
Frame::Frame( const Reference< XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      tIMPLMutexContainer is a struct with a mutex as member. We can't use a mutex as member, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember			(			)
		,	OWeakObject				(			)
		// Init member
		,	m_xFactory				( xFactory	)
		,	m_aListenerContainer	( m_aMutex	)
		,	m_aChildFramesArray		( m_aMutex	)
		,	m_bIsFrameActive		( sal_True	)
		,	m_bIsFrameUIActive		( sal_True	)
		,	m_bSearchingAllowed		( sal_True	)
		,	m_bIsFrameTop			( sal_True	)
		,	m_bAlreadyDisposed		( sal_True	)
		,	m_bIAmWindowOwner		( sal_True	)
		,	m_bHasNewComponent		( sal_True	)
		,	m_bDontUpdateShells		( sal_True	)
		,	m_bConnected			( sal_True	)
		,	m_bLoading				( sal_True	)
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
Frame::~Frame()
{
	// Reset instance, free memory ....
	impl_resetObject();
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************

DEFINE_XINTERFACE_16				(	Frame												,
										OWeakObject											,
										DIRECT_INTERFACE(XTypeProvider						),
										DIRECT_INTERFACE(XServiceInfo						),
										DIRECT_INTERFACE(XFrames							),
										DIRECT_INTERFACE(XIndexAccess						),
										DIRECT_INTERFACE(XElementAccess						),
										DIRECT_INTERFACE(XFramesSupplier					),
										DIRECT_INTERFACE(XFrame								),
										DIRECT_INTERFACE(XComponent							),
										DIRECT_INTERFACE(XStatusIndicatorSupplier			),
										DIRECT_INTERFACE(XDispatchProvider					),
										DIRECT_INTERFACE(XDispatchProviderInterception		),
										DIRECT_INTERFACE(XLoadEventListener					),
										DERIVED_INTERFACE(XEventListener,XLoadEventListener	),
										DIRECT_INTERFACE(XWindowListener					),
										DIRECT_INTERFACE(XBrowseHistoryRegistry				),
										DIRECT_INTERFACE(XTopWindowListener					)
									)

DEFINE_XTYPEPROVIDER_16				(	Frame								,
										XTypeProvider						,
										XServiceInfo						,
										XFrames								,
										XIndexAccess						,
										XElementAccess						,
										XFramesSupplier						,
										XFrame								,
										XComponent							,
										XStatusIndicatorSupplier			,
										XDispatchProvider					,
										XDispatchProviderInterception		,
										XLoadEventListener					,
										XEventListener						,
										XWindowListener						,
										XBrowseHistoryRegistry				,
										XTopWindowListener
									)

DEFINE_XSERVICEINFO_MULTISERVICE	(	Frame								,
										"com.sun.star.framework.Frame"		,
										"com.sun.star.comp.framework.Frame"
									)

//*****************************************************************************************************************
//	 XFrames
//*****************************************************************************************************************
void SAL_CALL Frame::append( const Reference< XFrame >& xFrame ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_append( xFrame ), "Frame::append()\nInvalid parameter detected.\n" );

	m_aChildFramesArray.addInterface( xFrame );
	/*ASMUSS
	if ( pOwner )
	{
		// get Implementation of xComponents ...
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel > xTunnel( rRef, ::com::sun::star::uno::UNO_QUERY );
        if ( xTunnel.is() )
        {
            sal_Int64 nImplementation = xTunnel->getSomething(SfxUnoFrame::impl_getStaticIdentifier());
            SfxUnoFrame* p = (SfxUnoFrame*)(nImplementation);

            if ( p && p->pOwner )
                pOwner->InsertChildFrame_Impl( p->pOwner, pOwner->GetChildFrameCount() );
        }
	}
	*/
	xFrame->setCreator( this );
}

//*****************************************************************************************************************
//	 XFrames
//*****************************************************************************************************************
Sequence< Reference< XFrame > > SAL_CALL Frame::queryFrames( sal_Int32 nSearchFlags ) throw( RuntimeException)
{
	// Ready fo multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_queryFrames( nSearchFlags ), "Frame::queryFrames()\nInvalid parameter detected.\n" );

	// Copy all frames from container to return sequence.
	Sequence< Reference< XFrame > >	seqFrames	( m_aChildFramesArray.getLength() );
	Reference< XFrame >*			pArray		= seqFrames.getArray();
	sal_Int32						nPosition	= 0;
	OInterfaceIteratorHelper		aIterator	( m_aChildFramesArray );

	while ( aIterator.hasMoreElements() == sal_True )
	{
		pArray[nPosition] = Reference< XFrame >( aIterator.next(), UNO_QUERY );
		++nPosition;
	}

	return seqFrames;
}

//*****************************************************************************************************************
//	 XFrames
//*****************************************************************************************************************
void SAL_CALL Frame::remove( const Reference< XFrame >& xFrame ) throw( RuntimeException )
{
	// Ready for multithreading.
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_remove( xFrame ), "Frame::remove()\nInvalid parameter detected.\n" );

	// Remove frame from list.
    OInterfaceIteratorHelper aIterator( m_aChildFramesArray );

    while( aIterator.hasMoreElements() == sal_True )
	{
        Reference< XFrame > xSearchFrame( aIterator.next(), UNO_QUERY );
		if ( xFrame == xSearchFrame )
		{
			// Reset Creator property
			xFrame->setCreator( Reference< XFramesSupplier >() );
			// Remove from array and delete reference
			m_aChildFramesArray.removeInterface( xFrame );
			break;
		}
	}

	// If removed frame the old active frame -> reset member.
	if ( xFrame == m_xActiveFrame )
	{
		m_xActiveFrame = Reference< XFrame > ();
	}

	/*ASMUSS
	if ( pOwner )
	{
		// get Implementation of xComponents ...
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel > xTunnel( aObject, ::com::sun::star::uno::UNO_QUERY );
        if ( xTunnel.is() )
        {
            sal_Int64 nImplementation = xTunnel->getSomething(SfxUnoFrame::impl_getStaticIdentifier());
            SfxUnoFrame* p = (SfxUnoFrame*)(nImplementation);

            if ( p && p->pOwner )
                pOwner->RemoveChildFrame_Impl( p->pOwner );
        }
	}
	*/
}

//*****************************************************************************************************************
//	 XFramesSupplier
//*****************************************************************************************************************
Reference< XFrames > SAL_CALL Frame::getFrames() throw( RuntimeException )
{
	return Reference< XFrames >( (OWeakObject*)this, UNO_QUERY );
}

//*****************************************************************************************************************
//	 XFramesSupplier
//*****************************************************************************************************************
Reference< XFrame > SAL_CALL Frame::getActiveFrame() throw( RuntimeException )
{
	// Read for multithreading
	MutexGuard aGuard( m_aMutex );

	return m_xActiveFrame;
}

//*****************************************************************************************************************
//	 XFramesSupplier
//*****************************************************************************************************************
void SAL_CALL Frame::setActiveFrame( const Reference< XFrame >& xFrame ) throw( RuntimeException )
{
	// Read for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_setActiveFrame( xFrame ), "Frame::setActiveFrame()\nInvalid parameter detected.\n" );

	// Do the following only, if frame will be changed!
	if ( m_xActiveFrame != xFrame )
	{
		if	(
				( m_xActiveFrame.is()	== sal_True )	&&
				( m_bIsFrameActive		== sal_True	)
			)
		{
			// If this is an active frame, the old active child Frame must be deactivated.
			m_xActiveFrame->deactivate();
		}

		// Safe new active frame.
		m_xActiveFrame = xFrame;

        if ( m_xActiveFrame.is() == sal_True )
		{
            if ( m_bIsFrameUIActive == sal_True )
            {
                // If this is the UI-active Frame, it loses the UI control because this is transferred to the new
                // active frame
                //ASMUSS DBG_ASSERT( bActive, "Wrong activate status!" );

                m_bIsFrameUIActive = sal_False;
				impl_sendFrameActionEvent( FrameAction_FRAME_UI_DEACTIVATING );

				/*ASMUSS
                if ( pOwner )
                {
					// get Implementation of xComponents ...
                    ::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel > xTunnel( m_xActiveFrame, ::com::sun::star::uno::UNO_QUERY );
                    if ( xTunnel.is() )
                    {
                        sal_Int64 nImplementation = xTunnel->getSomething(SfxUnoFrame::impl_getStaticIdentifier());
                        SfxUnoFrame* pUnoFrame = (SfxUnoFrame*)(nImplementation);
                        if ( pUnoFrame )
                        {
                            SfxViewFrame* pActive = pUnoFrame->pOwner->GetCurrentViewFrame();

                            // demn"aechst auch das "uber FrameActionListener, denn der ViewFrame wird der Controller
                            SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
                            if ( pViewFrame )
                                pViewFrame->SetActiveChildFrame_Impl( pActive );
                        }
                    }
                }
				*/
            }

			// Activate frame, if there a difference between internal BOOL and current
            if	(
					( m_bIsFrameActive				==	sal_True	)	&&
					( m_xActiveFrame->isActive()	==	sal_False	)
				)
			{
				m_xActiveFrame->activate();
			}
		}
        else
		if ( m_bIsFrameActive == sal_True )
		{
			// If this frame is active and has no active subframe anymore it is UI active too
			//ASMUSS DBG_ASSERT( !bUIActive, "Wrong activate status!" );

			m_bIsFrameUIActive = sal_True;
			impl_sendFrameActionEvent( FrameAction_FRAME_UI_ACTIVATED );

			/*ASMUSS
            if ( pOwner )
            {
                // demn"aechst auch das "uber FrameActionListener, denn der ViewFrame wird der Controller
                SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
                if ( pViewFrame )
                {
                    pViewFrame->SetActiveChildFrame_Impl( NULL );
                    pViewFrame->MakeActive_Impl();
                    pViewFrame->GetBindings().SetActiveFrame( ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > () );
                }
            }
			*/
		}
	}
}

//*****************************************************************************************************************
//	 XStatusIndicatorSupplier
//*****************************************************************************************************************
Reference< XStatusIndicator > SAL_CALL Frame::getStatusIndicator() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// If no indicator exist ...
	if ( m_xIndicator.is() == sal_False )
	{
		// ... create a new one.
		/*ASMUSS
		if ( pOwner )
			xIndicator = new SfxStatusIndicator( pOwner );
		else
		{
			Window* pComp = VCLUnoHelper::GetWindow(xWindow) ;
			if ( pComp )
			{
 				SfxPopupStatusIndicator* pTmp = new SfxPopupStatusIndicator( pComp );
				xIndicator = pTmp->GetInterface();
				pIndicator = pTmp;		// remember indicator window
			}
		}
		*/
	}

	// Return indicator reference.
	return m_xIndicator;
}

//*****************************************************************************************************************
//	 XDispatchProvider
//*****************************************************************************************************************
Reference< XDispatch > SAL_CALL Frame::queryDispatch(	const	URL&	 	aURL			,
															const	OUString&	sTargetFrameName,
																	sal_Int32	nSearchFlags	) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_queryDispatch( aURL, sTargetFrameName, nSearchFlags ), "Frame::queryDispatch()\nInvalid parameter detected.\n" );

	Reference< XDispatch > xReturn;
	if ( m_xInterceptor.is() == sal_True )
	{
		xReturn = m_xInterceptor->queryDispatch( aURL, sTargetFrameName, nSearchFlags );
	}
	else
	{
		xReturn = impl_queryDispatch( aURL, sTargetFrameName, nSearchFlags );
	}

	return xReturn;
}

//*****************************************************************************************************************
//	 XDispatchProvider
//*****************************************************************************************************************
Sequence< Reference< XDispatch > > SAL_CALL Frame::queryDispatches( const Sequence< DispatchDescriptor >& seqDescriptor ) throw( RuntimeException )
{
	return Sequence< Reference< XDispatch > >();
}

//*****************************************************************************************************************
//	 XDispatchProviderInterception
//*****************************************************************************************************************
void SAL_CALL Frame::registerDispatchProviderInterceptor( const Reference< XDispatchProviderInterceptor >& xInterceptor ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_registerDispatchProviderInterceptor( xInterceptor ), "Frame::registerDispatchProviderInterceptor()\nInvalid parameter detected.\n" );

	// Do the follow only, if a correct interceptor will be set.
	if ( xInterceptor.is() == sal_True )
	{
		if ( m_xInterceptor.is() )
		{
			// There is already an interceptor; the new one will become its master.
			xInterceptor->setSlaveDispatchProvider( Reference< XDispatchProvider >( m_xInterceptor, UNO_QUERY ) );
			m_xInterceptor->setMasterDispatchProvider( Reference< XDispatchProvider >( xInterceptor, UNO_QUERY ) );
		}
		else
		{
			// It is the first interceptor; pass own dispatch provider as a slave.
			/*ASMUSS pOwnDispatch kommt aus dem Sfx!
			xInterceptor->setSlaveDispatchProvider( pOwnDispatch );
			*/
		}

		// frame is the master of the first interceptor
		m_xInterceptor = xInterceptor;
		m_xInterceptor->setMasterDispatchProvider( this );

		/*ASMUSS
		if ( pOwner )
		{
			SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
			if ( pViewFrame )
			{
				SfxBindings& rBindings = pViewFrame->GetBindings();
				if ( rBindings.GetDispatcher_Impl() == pViewFrame->GetDispatcher() )
					rBindings.SetDispatchProvider_Impl( this );
			}
		}
		*/
	}
}

//*****************************************************************************************************************
//	 XDispatchProviderInterception
//*****************************************************************************************************************
void SAL_CALL Frame::releaseDispatchProviderInterceptor( const Reference< XDispatchProviderInterceptor >& xInterceptor ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_releaseDispatchProviderInterceptor( xInterceptor ), "Frame::releaseDispatchProviderInterceptor()\nInvalid parameter detected.\n" );

	// Do the follow only, if a correct interceptor will be given.
	if ( xInterceptor.is() == sal_True )
	{
		// Get slave and master.
		Reference< XDispatchProvider >				xSlave( xInterceptor->getSlaveDispatchProvider(), UNO_QUERY );
		Reference< XDispatchProviderInterceptor >	xMaster( xInterceptor->getMasterDispatchProvider(), UNO_QUERY );

		if ( xMaster.is() == sal_True )
		{
			// Old master may be an interceptor too, must be reconnected.
			if ( xSlave.is() == sal_True )
			{
				xMaster->setSlaveDispatchProvider( xSlave );
			}
			else
			{
				/*ASMUSS
				 pOwnDispatch kommt aus dem sfx
				 xMaster->setSlaveDispatchProvider( pOwnDispatch );
				*/
			}
		}

		// Unchain the interceptor that has to be removed.
		xInterceptor->setSlaveDispatchProvider( Reference< XDispatchProvider >() );
		xInterceptor->setMasterDispatchProvider( Reference< XDispatchProvider >() );

		if ( m_xInterceptor == xInterceptor )
		{
			// First interceptor was removed; its old slave will become the new interceptor.
			m_xInterceptor = Reference< XDispatchProviderInterceptor >( xSlave, UNO_QUERY );
		}

		/*ASMUSS
		if ( pOwner )
		{
			SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
			if ( pViewFrame )
			{
				SfxBindings& rBindings = pViewFrame->GetBindings();
				if ( rBindings.GetDispatchProvider_Impl() == Reference< XDispatchProvider > ( this ) )
				{
					rBindings.InvalidateAll( sal_True );
					rBindings.InvalidateUnoControllers_Impl();
					if ( !m_xInterceptor.is() )
						rBindings.SetDispatchProvider_Impl( Reference< XDispatchProvider > () );
				}
			}
		}
		*/
	}
}

//*****************************************************************************************************************
//	 XBrowseHistoryRegistry
//*****************************************************************************************************************
void SAL_CALL Frame::updateViewData( const Any& aValue ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_updateViewData( aValue ), "Frame::updateViewData()\nInvalid parameter detected.\n" );

	/*ASMUSS
	::vos::OGuard aGuard( Application::GetSolarMutex() );
	if ( pOwner )
		pOwner->UpdatePickEntries( Value );
	*/
}

//*****************************************************************************************************************
//	 XBrowseHistoryRegistry
//*****************************************************************************************************************
void SAL_CALL Frame::createNewEntry(	const	OUString&				  	sURL		,
			  								const	Sequence< PropertyValue >&	seqArguments,
											const	OUString&				  	sTitle		) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_createNewEntry( sURL, seqArguments, sTitle ), "Frame::createNewEntry()\nInvalid parameter detected.\n" );

	/*ASMUSS
	::vos::OGuard aGuard( Application::GetSolarMutex() );
	if ( pOwner )
		pOwner->UpdateHistory( aURL, aArgs, rTitle );
	*/
}

//*****************************************************************************************************************
//	 XLoadEventListener
//*****************************************************************************************************************
void SAL_CALL Frame::loadFinished( const Reference< XFrameLoader >& xLoader ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_loadFinished( xLoader ), "Frame::loadFinished()\nInvalid parameter detected.\n" );

	m_bLoading = sal_False;

	if ( m_xLoader == xLoader )
	{
		// Release old loader
		m_xLoader = Reference< XFrameLoader >();

		if	(
				( m_xComponentWindow.is()	==	sal_False	)	&&
				( m_xController.is()		==	sal_False	)
			)
		{
			/*ASMUSS
			// Controller was suspended but load failed
			// KANN NIE PASSIEREN !?
			if ( m_xController.is() == sal_True )
			{
				xController->suspend( sal_False );
			}
			*/

			/*ASMUSS
			if ( pOwner )
			{
				pOwner->SetItemSet_Impl( NULL );
				if ( !bLoadStarted )
				{
					// loadFinished wurde asynchron gerufen
					LoadEnvironment_Impl *pEnv = pOwner->GetLoadEnvironment_Impl();
					if ( pEnv )
						pEnv->DocumentDetected( NULL, ERRCODE_ABORT );

					if ( bIsTop && ( !pEnv || !pEnv->OwnsFrame() ) && !pOwner->GetCurrentDocument() )
						pOwner->DoClose();
				}
			}
			*/
		}
//ASMUSS		else if ( pOwner && !bDontUpdateShells && bHasNewComponent )
		else
		if	(
				( m_bDontUpdateShells	==	sal_False	)	&&
				( m_bHasNewComponent	==	sal_False	)
			)
		{
			// New component in old sfx-frame
			// controller has taken new model
			// shell update has not been done in setComponent because setComponent has not been called
			OUString sURL = m_sLoadURL;

			if ( m_xController.is() == sal_True )
			{
				Reference< XModel > xModel = m_xController->getModel();
				if ( xModel.is() == sal_True )
				{
					sURL = xModel->getURL();
				}
			}

			/*ASMUSS
			SfxAllItemSet aSet( SFX_APP()->GetPool() );
			if ( pOwner->GetItemSet_Impl() )
				aSet.Put( *pOwner->GetItemSet_Impl() );
			pOwner->SetItemSet_Impl( NULL );
			aSet.Put( SfxStringItem( SID_FILE_NAME, aURL ), SID_FILE_NAME );

			SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
			if ( pViewFrame )
			{
				SfxComponentViewShell* pShell = PTR_CAST( SfxComponentViewShell, pViewFrame->GetViewShell() );
				if ( pShell )
				{
					pShell->NewModelConnected( aSet );
					if ( !bLoadStarted )
						pOwner->GetLoadEnvironment_Impl()->DocumentDetected( pShell->GetObjectShell(), ERRCODE_NONE );
				}

				pViewFrame->GetBindings().InvalidateAll( sal_True );
				pViewFrame->GetBindings().InvalidateUnoControllers_Impl();
			}
			*/

			if	(
					( m_xController.is()		==	sal_True	)	||
					( m_xComponentWindow.is()	==	sal_True	)
				)
			{
				impl_sendFrameActionEvent( FrameAction_COMPONENT_REATTACHED );
				//ASMUSS SfxDesktop::DoFrameAction_Impl( this, ::com::sun::star::frame::FrameAction_COMPONENT_REATTACHED );
			}

			//ASMUSS SfxTask *pActive = SfxTaskManager::GetActiveTask();
			if	(
					( m_bIsFrameTop		==	sal_True	)	&&
					( m_bIsFrameActive	==	sal_True	)
				)
			{
				// Explorer, beamer ... has focus.
				if	(
						( m_xActiveFrame.is()						==	sal_True	)	&&
						( m_xActiveFrame->getComponentWindow().is()	==	sal_True	)
					)
				{
					m_xActiveFrame->getComponentWindow()->setFocus();
				}
				else
				{
					m_xComponentWindow->setFocus();
				}
			}
		}

		m_sLoadURL = OUString();
	}
}

//*****************************************************************************************************************
//	 XLoadEventListener
//*****************************************************************************************************************
void SAL_CALL Frame::loadCancelled( const Reference< XFrameLoader >& xLoader ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_loadCancelled( xLoader ), "Frame::loadCancelled()\nInvalid parameter detected.\n" );

	// Break loading only, if this the correct loader!
	if ( m_xLoader == xLoader )
	{
		// Reset member variables.
		m_bLoading	= sal_False;
		m_sLoadURL	= OUString();
		m_xLoader	= Reference< XFrameLoader >();

		// Controller was suspended but load failed.
		if ( m_xController.is() == sal_True )
		{
			m_xController->suspend( sal_False );
		}

		/*ASMUSS
		if ( pOwner )
		{
			pOwner->SetItemSet_Impl( NULL );
			if ( !bLoadStarted )
			{
				// loadCanceled was called asynchronously
				LoadEnvironment_Impl *pEnv = pOwner->GetLoadEnvironment_Impl();
				if ( pEnv )
					// pOwner may be deleted in this call
					pEnv->DocumentDetected( NULL, ERRCODE_ABORT );

				if ( pOwner && bIsTop && ( !pEnv || !pEnv->OwnsFrame() ) && !pOwner->GetCurrentDocument() )
					pOwner->DoClose();
			}
		}
		*/
	}
}

//*****************************************************************************************************************
//	 XWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowResized( const WindowEvent& aEvent ) throw( RuntimeException )
{
	// Ready fo multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_windowResized( aEvent ), "Frame::windowResized()\nInvalid parameter detected.\n" );

	// Get reference to device.
	Reference< XDevice > xDevice( m_xContainerWindow, UNO_QUERY );

	// Convert relativ size to output size.
	Rectangle	aRectangle	= m_xContainerWindow->getPosSize();
	DeviceInfo	aInfo		= xDevice->getInfo();

	aRectangle.X		+= aInfo.LeftInset	;
	aRectangle.Y		+= aInfo.TopInset	;
	aRectangle.Width	-= aInfo.RightInset	;
	aRectangle.Height	-= aInfo.BottomInset;

	Size aSize( aRectangle.Width-aRectangle.X, aRectangle.Height-aRectangle.Y );

	/*ASMUSS
	if ( pIndicator && pIndicator->IsVisible() )
	{
		// substract height of indicator window
		Point aPos( aRectangleObject.TopLeft() );
		long nHeight = pIndicator->GetSizePixel().Height();
		aSize.Height() -= nHeight;

		// reposition indicatorwindow at the bottom its parent
		pIndicator->SetPosSizePixel( 0, aPos.Y() + aSize.Height(), aSize.Width(), nHeight, WINDOW_POSSIZE_POSSIZE );
	}
	*/

	/*ASMUSS
	if ( pOwner )
	{
		if ( pOwner->OwnsBindings_Impl() )
		{
			SfxWorkWindow *pWork = pOwner->GetWorkWindow_Impl();
			pWork->ArrangeChilds_Impl();
			pWork->ShowChilds_Impl();
		}
		else
			pOwner->GetWindow().SetSizePixel( aSize );
	}
	else
	*/
	if ( m_xComponentWindow.is() == sal_True )
	{
		m_xComponentWindow->setPosSize( 0, 0, aSize.Width, aSize.Height, PosSize::POSSIZE );
	}
}

//*****************************************************************************************************************
//	 XWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowMoved( const WindowEvent& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowShown( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowHidden( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowOpened( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowClosing( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowClosed( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowMinimized( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowNormalized( const EventObject& aEvent ) throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowActivated( const EventObject& aEvent ) throw( RuntimeException )
{
	// Ready fo multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_windowActivated( aEvent ), "Frame::windowActivated()\nInvalid parameter detected.\n" );

/*ASMUSS
 	if ( m_bIsFrameActive == sal_False && pOwner && pOwner->IsFocusLocked_Impl() )
		return;
*/

	if ( m_bIsFrameActive == sal_False )
	{
		setActiveFrame( Reference< XFrame >() );
		activate();
	}

/*ASMUSS
    if	(
			( m_bIsFrameUIActive	==	sal_True	)//	&&
//			( pOwner )
		)
    {
        // demn"aechst auch das "uber FrameActionListener, denn der ViewFrame wird der Controller
        SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
        if ( pViewFrame )
            pViewFrame->GetBindings().SetActiveFrame( ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > () );
    }
*/
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Frame::windowDeactivated( const EventObject& aEvent ) throw( RuntimeException )
{
	// Ready fo multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_windowDeactivated( aEvent ), "Frame::windowDeactivated()\nInvalid parameter detected.\n" );

	if ( m_bIsFrameActive == sal_True )
	{
		// deactivation is always done implicitely by activation of another frame
		// only if no activation is done, deactivations have to be processed if the activated window
		// is a parent window of the last active Window

		/*ASMUSS
		Window *pNewWindow = Application::GetFocusWindow();
        if ( xWindow.is() && pNewWindow && xCreator.is() )
		{
			Reference< XWindow > xCreatorWindow( xCreator, UNO_QUERY );
            if ( pNewWindow->IsChild( VCLUnoHelper::GetWindow(xWindow) ) && (!xCreatorWindow.is() || !pNewWindow->IsChild( VCLUnoHelper::GetWindow(xCreatorWindow) ) ) )
				xCreator->setActiveFrame( ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > () );
		}
		*/
	}
}

//*****************************************************************************************************************
//	 XIndexAccess
//*****************************************************************************************************************
sal_Int32 SAL_CALL Frame::getCount() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Return count of childframes.
	return m_aChildFramesArray.getLength();
}

//*****************************************************************************************************************
//	 XIndexAccess
//*****************************************************************************************************************
Any SAL_CALL Frame::getByIndex( sal_Int32 nIndex ) throw(	IndexOutOfBoundsException	,
			 													WrappedTargetException		,
																RuntimeException			)
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Throw an exception, if index out of range.
	if	(
			( nIndex	<	0			)	||
			( nIndex	>=	getCount()	)
		)
	{
		throw( IndexOutOfBoundsException() );
	}

	/*ASMUSS Index wird gar nicht beruecksichtigt!*/

	// Safe THIS as XFrame reference in an any ...
	Any					aAny;
	Reference< XFrame >	xThis( (OWeakObject*)this, UNO_QUERY );
	aAny <<= xThis ;

	// ... and return it.
	return aAny;
}

//*****************************************************************************************************************
//	 XElementAccess
//*****************************************************************************************************************
Type SAL_CALL Frame::getElementType() throw( RuntimeException )
{
	// The type of childs is frame ... is frame ... is frame.
	return ::getCppuType( ( const Reference< XFrame >* ) NULL );
}

//*****************************************************************************************************************
//	 XElementAccess
//*****************************************************************************************************************
sal_Bool SAL_CALL Frame::hasElements() throw( RuntimeException )
{
	return ( getCount() > 0 );
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::initialize( const Reference< XWindow >& xWindow ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_initialize( xWindow )	, "Frame::initiallize()\nInvalid parameter detected.\n"				);
	OSL_ENSHURE( !(m_xContainerWindow.is() == sal_True )		, "Frame::initialize()\nMethod already called! Don't do it again.\n");

	// Protection against more then o ne calls.
	if ( m_xContainerWindow.is() == sal_False )
	{
		impl_setContainerWindow( xWindow );

		/*ASMUSS
		if ( !pOwner )
		{
			Window*	pComp = VCLUnoHelper::GetWindow(rWindow) ;
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame >  xParent( xCreator, ::com::sun::star::uno::UNO_QUERY );
			if ( !xParent.is() )
			{
				// Frame was not inserted into a container; must be a external TopFrame
				if ( pComp )
					pOwner = new SfxExternalTopFrame( pComp );
				bIsTop = sal_True;
			}
			else
			{
				// get Implementation of xComponents ...
				::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel > xTunnel( xParent, ::com::sun::star::uno::UNO_QUERY );
        		if ( xTunnel.is() )
        		{
            		sal_Int64 nImplementation = xTunnel->getSomething(SfxUnoFrame::impl_getStaticIdentifier());
            		SfxUnoFrame* pUnoFrame = (SfxUnoFrame*)(nImplementation);
            		if ( pUnoFrame && pComp )
            		{
                		// create frame for given window
                		//ASDBG Window *pWindow = pComp->GetWindow();
                		pOwner = pUnoFrame->CreateFrame( pComp );
            		}
        		}

				bIsTop = sal_False;
			}

			if ( pOwner )
			{
				SfxUnoFrame *pOld = pOwner->pUnoImp;
				pOwner->pUnoImp = this;
				if ( pOld )
				{
					pOld->pOwner = NULL;
					if ( xCreator.is() )
						xCreator->getFrames()->remove( (::com::sun::star::frame::XFrame*) pOld );
					pOld->release();
				}
			}

			acquire();
		}
		*/
	}
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
Reference< XWindow > SAL_CALL Frame::getContainerWindow() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Return reference to my own window - if it exist!
	return m_xContainerWindow;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::setCreator( const Reference< XFramesSupplier >& xCreator ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_setCreator( xCreator ), "Frame::setCreator()\nInvalid parameter detected.\n" );

	// Safe new reference to different parent.
	m_xParent = xCreator;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
Reference< XFramesSupplier > SAL_CALL Frame::getCreator() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Return reference to my creator - It's my parent too.
	return m_xParent;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
OUString SAL_CALL Frame::getName() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Return name of this frame.
	return m_sFrameName;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::setName( const OUString& sName ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_setName( sName ), "Frame::setName()\nInvalid parameter detected.\n" );

	// Take the new one.
	m_sFrameName = sName;
	/*ASMUSS
	if ( pOwner )
		pOwner->SetFrameName( rName );
	*/
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
Reference< XFrame > SAL_CALL Frame::findFrame(	const	OUString&	sTargetFrameName	,
							 								sal_Int32	nSearchFlags		) throw( RuntimeException )
{

	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_findFrame( sTargetFrameName, nSearchFlags ), "Frame::findFrame()\nInvalid parameter detected.\n" );

	// Set default return Value, if method failed
	Reference< XFrame > xReturn = Reference< XFrame >();

	// Define some local variables
	sal_Bool			bIsDocument		= ( sTargetFrameName.compareToAscii( FRAMETYPE_DOCUMENT ) == EQUAL );
	Reference< XFrame >	xParentFrame	( m_xParent, UNO_QUERY );

	// protect against recursion while searching in parent frames
	if ( m_bSearchingAllowed == sal_False )
	{
		//*************************************************************************************************************
		if	(
				( sTargetFrameName.getLength()						<	1		)	||
				( sTargetFrameName.compareToAscii( FRAMETYPE_SELF )	==	EQUAL	)
			)
		{
			// If no special name searched or name="self" ...
			// ... return THIS frame as reference.
			xReturn = Reference< XFrame >( (OWeakObject*)this, UNO_QUERY );
		}
		else
		//*************************************************************************************************************
		if	(
				( sTargetFrameName.compareToAscii( FRAMETYPE_PARENT )	==	EQUAL		)	&&
				( xParentFrame.is()										==	sal_True	)
			)
		{
			// Return the XFrame interface from the parent container.
			xReturn = xParentFrame;
		}
		else
		//*************************************************************************************************************
		if	(
				( sTargetFrameName.compareToAscii( FRAMETYPE_TOP )	==	EQUAL	)	||
				(
					( bIsDocument	==	sal_True	)//	&&
//					( pOwner		!=	NULL		)
				)
			)
		{
			// Return the XFrame interface of the uppermost XFrameContainer.
			// If there no parent - we are the topframe himself!
			if ( m_bIsFrameTop == sal_True )
			{
				xReturn = Reference< XFrame >( (OWeakObject*)this, UNO_QUERY );
			}
			else
			if ( xParentFrame.is() == sal_True )
			{
				xReturn = xParentFrame->findFrame( sTargetFrameName, nSearchFlags );
			}
		}
		else
		//*************************************************************************************************************
		if	( bIsDocument == sal_True )
		{
			//ASMUSS xReturn = SfxViewFrame::Current()->GetFrame()->GetTopFrame()->GetFrameInterface();
		}
		else
		//*************************************************************************************************************
		if	(
				( sTargetFrameName.compareToAscii( FRAMETYPE_BEAMER		)	==	EQUAL	)	||
				( sTargetFrameName.compareToAscii( FRAMETYPE_EXPLORER	)	==	EQUAL	)	||
				( sTargetFrameName.compareToAscii( FRAMETYPE_PARTWINDOW	)	==	EQUAL	)
			)
		{
			/*ASMUSS
			sal_uInt16 nID;

			if	(	sTargetFrameName.compareToAscii( "_beamer" ) == COMPARE_EQUAL )
				nId = SID_BROWSER;
			else if ( aName.CompareToAscii("_explorer") == COMPARE_EQUAL )
				nId = SID_EXPLORER;
			else if ( aName.CompareToAscii("_partwindow") == COMPARE_EQUAL )
				nId = SID_PARTWIN;

			SfxApplication *pApp = SFX_APP();
			if ( !pApp->HasChildWindow( nId ) )
			{
				// Beamer nicht da
				if ( ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::CREATE ) == 0 )
					return ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > ();

				pApp->SetChildWindow( nId, sal_True );
				pApp->GetBindings().Invalidate( nId );
			}

			SfxChildWindow *pChild = pApp->GetChildWindow( nId );
			if ( pChild )
			{
				// SBI: #62242 der _explorer hat kein SfxExplorerHorizDockWnd_Impl - ChildWindow
				// sodern ein SfxExplorerDockWnd_Impl
				if ( SID_BROWSER == nId )
				{
					// Einen versteckten Beamer gebe ich nur raus, wenn ich zum Erzeugen
					// gen"otigt werde
					SfxExplorerHorizDockWnd_Impl *pBeamer = (SfxExplorerHorizDockWnd_Impl*) pChild->GetWindow();
					if ( pBeamer->IsAutoHide_Impl() && ( ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::CREATE ) == 0  ) )
						return ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > ();
				}

				return pChild->GetFrame();
			}
			else
				return ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > ();
			*/
		}
		else
		//*************************************************************************************************************
		// Get a new TaskFrame and a TopFrame inside the task.
		if	( sTargetFrameName.compareToAscii( FRAMETYPE_BLANK ) == EQUAL )
		{
			/*ASMUSS
			SfxTask *pTask = SfxTaskManager::CreateTask();
			SfxFrame *pFrame = pTask->CreateFrame();
			return pFrame->GetFrameInterface();
			*/
		}
		else
		//*************************************************************************************************************
		// Check own name.
		if	(
				( nSearchFlags		&	FrameSearchFlag::SELF	)	&&
				( sTargetFrameName	==	m_sFrameName			)
			)
		{
			xReturn = Reference< XFrame >( (OWeakObject*)this, UNO_QUERY );
		}
		else
		//*************************************************************************************************************
		// Continue search with subframes of this frame.
		if	(
				( nSearchFlags						&	FrameSearchFlag::CHILDREN	)	&&
				( m_aChildFramesArray.getLength()	>	0							)
			)
		{
			// Search own container of childframes if allowed.
			// First search only for direct subframes.
    		OInterfaceIteratorHelper aIterator1( m_aChildFramesArray );

    		while	(
						( xReturn.is()					==	sal_False	)	&&
						( aIterator1.hasMoreElements()	==	sal_True	)
					)
			{
        		xReturn = ((XFrame*)aIterator1.next())->findFrame( sTargetFrameName, FrameSearchFlag::SELF );
			}

			// If no direct subframes found, search now subframes of subframes.
			if ( xReturn.is() == sal_False )
			{
    			OInterfaceIteratorHelper aIterator2( m_aChildFramesArray );
    			while	(
							( xReturn.is()					==	sal_False	)	&&
							( aIterator2.hasMoreElements()	==	sal_True	)
						)
				{
        			xReturn = ((XFrame*)aIterator2.next())->findFrame( sTargetFrameName, FrameSearchFlag::CHILDREN );
				}
			}
		}
		else
		//*************************************************************************************************************
		// Check direct parent if allowed ( no searching at parent ).
		if	(
				( nSearchFlags		&	FrameSearchFlag::PARENT	)	&&
				( xParentFrame.is()	==	sal_True				)	&&
				( m_bIsFrameTop		==	sal_False				)
			)
		{
			xReturn = xParentFrame->findFrame( sTargetFrameName, FrameSearchFlag::SELF );
		}
		else
		//*************************************************************************************************************
		// Continue search on brothers ( subframes of parent ) but don't let them search their brothers too ...
		// If FrameSearchFlag_CHILDREN is set, the children of the brothers will be searched also, otherwise not.
		// Search on parents will be done later.
		if	(
				( nSearchFlags		&	FrameSearchFlag::SIBLINGS	)	&&
				( m_xParent.is()	==	sal_True					)
			)
		{
			Reference< XFrame >				xSearchFrame;
			Reference< XFrames >			xFrames		= m_xParent->getFrames();
			Sequence< Reference< XFrame > >	seqFrames	= xFrames->queryFrames( FrameSearchFlag::SIBLINGS );
			Reference< XFrame >*			pArray		= seqFrames.getArray();
			Reference< XFrame >				xThis		( (OWeakObject*)this, UNO_QUERY );
			sal_uInt16						nCount		= (sal_uInt16)seqFrames.getLength();

			sal_uInt16 nPosition = 0;
			while	(
						( xReturn.is()	==	sal_False	)	&&
						( nPosition		<	nCount		)
					)
			{
				if ( pArray[nPosition] != xThis )
				{
					xReturn = pArray[nPosition]->findFrame( sTargetFrameName, FrameSearchFlag::SELF );
				}
				++nPosition;
			}

			if ( xReturn.is() == sal_False )
			{
				nPosition = 0;
				while	(
							( xReturn.is()	==	sal_False	)	&&
							( nPosition		<	nCount		)
						)
				{
					if ( pArray[nPosition] != xThis )
					{
						xReturn = pArray[nPosition]->findFrame( sTargetFrameName, FrameSearchFlag::CHILDREN );
					}
					++nPosition;
				}
			}
		}
		else
		//*************************************************************************************************************
		// Search parent if allowed; parent may create a new frame.
		if	(
				( nSearchFlags		&	FrameSearchFlag::PARENT	)	&&
				( xParentFrame.is()	==	sal_True				)	&&
				(
					( m_bIsFrameTop	==	sal_True				)	||
					( nSearchFlags	&	FrameSearchFlag::TASKS	)
				)
			)
		{
			m_bSearchingAllowed = sal_True ;
			xReturn				= xParentFrame->findFrame( sTargetFrameName, nSearchFlags | FrameSearchFlag::CHILDREN | FrameSearchFlag::SIBLINGS );
			m_bSearchingAllowed = sal_False;
		}
		else
		//*************************************************************************************************************
		// Create a new TopFrame.
		if	(
				( m_bIsFrameTop	==	sal_True												)	&&
				( nSearchFlags	&	( FrameSearchFlag::CREATE | FrameSearchFlag::TASKS )	)
			)
		{
			/*ASMUSS
			if ( pOwner )
			{
				SfxTask *pTask = pOwner->GetTask();
				if ( pTask )
				{
					SfxFrame *pFrame = pTask->CreateFrame();
					pFrame->SetFrameName( aTargetframename );
					return pFrame->GetFrameInterface();
				}
			}
			else if ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::TASKS )
			{
				SfxTask* pTask = SfxTaskManager::CreateTask();
				SfxFrame *pFrame = pTask->CreateFrame();
				pFrame->SetFrameName( aTargetframename );
				return pFrame->GetFrameInterface();
			}
			*/
		}

		//*************************************************************************************************************
		// We only search in children, don't create a new Frame.
		// Return default NULL-value!
	}

	return xReturn;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
sal_Bool SAL_CALL Frame::isTop() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Return state of this instance.
	return m_bIsFrameTop;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::activate() throw( RuntimeException )
{
	//Read for multithreading
	MutexGuard aGuard( m_aMutex );

	/* War schon deaktivierter Quellcode!
	MutexGuard aGuard( Application::GetSolarMutex() );
	if ( pOwner && pOwner->GetCurrentViewFrame() )
		pOwner->Appear();
	*/

	if ( m_bIsFrameActive == sal_False )
	{
		//ASMUSS DBG_ASSERT( !bUIActive, "Wrong activate status!" );
		m_bIsFrameActive = sal_True;

		// Activate notification must be done from out to in, so first go up!
		if ( m_xParent.is() == sal_True )
		{
			m_xParent->setActiveFrame( this );
            if ( m_xParent->isActive() == sal_False )
			{
                m_xParent->activate();
			}
		}

		// Send message to listener
		impl_sendFrameActionEvent( FrameAction_FRAME_ACTIVATED );
	}

    if ( m_xActiveFrame.is() == sal_True )
    {
        // if activation is done "from outside", now go down!
        if( m_xActiveFrame->isActive() == sal_False )
		{
            m_xActiveFrame->activate();
		}
    }
    else
	if ( m_bIsFrameUIActive == sal_False )
	{
		// If this frame is active and has no active subframe it is UI active too
		m_bIsFrameUIActive = sal_True;

		/*ASMUSS
		if ( pOwner )
		{
			// demn"aechst auch das "uber FrameActionListener, denn der ViewFrame wird der Controller
			SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
			if ( pViewFrame )
			{
				pViewFrame->SetActiveChildFrame_Impl( NULL );
				pViewFrame->MakeActive_Impl();
				pViewFrame->GetBindings().SetActiveFrame( ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > () );
			}
		}
		*/

		impl_sendFrameActionEvent( FrameAction_FRAME_UI_ACTIVATED );
	}
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::deactivate() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Deactivate notification must be done from in to out.
	if ( m_xActiveFrame.is() == sal_True )
	{
		m_xActiveFrame->deactivate();
	}

	// If a UI-active frame is deactivated it must be UI-deactivated.
	if ( m_bIsFrameUIActive == sal_True )
	{
		//DBG_ASSERT( bActive, "Wrong activate status!" );
		m_bIsFrameUIActive = sal_False;
		impl_sendFrameActionEvent( FrameAction_FRAME_UI_DEACTIVATING );
	}

	if ( m_bIsFrameActive == sal_True )
	{
        m_bIsFrameActive = sal_False;
		impl_sendFrameActionEvent( FrameAction_FRAME_UI_DEACTIVATING );
	}
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
sal_Bool SAL_CALL Frame::isActive() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	return m_bIsFrameActive;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
sal_Bool SAL_CALL Frame::setComponent(	const	Reference< XWindow >&		xComponentWindow	,
				  							const	Reference< XController >&	xController			) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_setComponent( xComponentWindow, xController ), "Frame::setComponent()\nInvalid parameter detected.\n" );

	// Become frame a new component?
	if	(
			( xController.is()		==	sal_True	)	||
			( xComponentWindow.is()	==	sal_True	)
		)
	{
		m_bHasNewComponent = xComponentWindow.is();
	}

	// Release current component, if any exist.
	if	(
			( m_xController.is()		==	sal_True	)	||
			( m_xComponentWindow.is()	==	sal_True	)
		)
	{
		// Send FrameAction-event to all listener.
		impl_sendFrameActionEvent( FrameAction_COMPONENT_DETACHING );
		//ASMUSS SfxDesktop::DoFrameAction_Impl( this, ::com::sun::star::frame::FrameAction_COMPONENT_DETACHING );
	}

	// Delete controller only, if they will do something with the window.
	impl_setController( Reference< XController >() );
	impl_setComponentWindow( xComponentWindow );
	impl_setController( xController );

	/*ASMUSS
	if ( pOwner && bHasNewComponent && xComponentWindow.is() )
	{
		// new component in old sfx-frame
		SfxComponentViewShell *pShell = NULL;
		SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
		if ( pViewFrame )
			pShell = PTR_CAST( SfxComponentViewShell, pViewFrame->GetViewShell() );

		String aURL( aLoadURL );
		if ( xController.is() )
		{
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XModel >  xModel = rController->getModel();
			if ( xModel.is() )
				aURL = String( xModel->getURL() );
		}

		SfxAllItemSet aSet( SFX_APP()->GetPool() );
		if ( pOwner->GetItemSet_Impl() )
			aSet.Put( *pOwner->GetItemSet_Impl() );
		pOwner->SetItemSet_Impl( NULL );
		aSet.Put( SfxStringItem( SID_FILE_NAME, aURL ), SID_FILE_NAME );

		if ( pShell )
		{
			// former component was a new component too
			pShell->NewModelConnected( aSet );
			if ( bLoading && !bLoadStarted )
				// the Loader has already returned from its load() method, setComponent() has been called after that
				// ("asynchronous" loading)
				pOwner->GetLoadEnvironment_Impl()->DocumentDetected( pShell->GetObjectShell(), ERRCODE_NONE );
		}
		else
		{
			// former component was an old style sfx component
			SfxComponentObjectShell *pDoc = new SfxComponentObjectShell;
			pDoc->DoInitNew(0);
			pDoc->GetMedium()->GetItemSet()->Put( aSet );
			SFX_ITEMSET_ARG( &aSet, pItem, SfxBoolItem, SID_DOC_READONLY, sal_False );
			if ( pItem && pItem->GetValue() )
				pDoc->GetMedium()->SetOpenMode( SFX_STREAM_READONLY, sal_True );
			pDoc->GetMedium()->SetName( aURL, sal_True );

			if ( !pOwner->GetTask() || !pOwner->GetTask()->IsDesktop() )
            {
                SFX_ITEMSET_ARG( &aSet, pFilterItem, SfxStringItem, SID_FILTER_NAME, sal_False );
                const SfxFilter *pFilter = NULL;
                if ( pFilterItem )
                    pFilter = SFX_APP()->GetFilterMatcher().GetFilter( pFilterItem->GetValue() );
                else
                    pFilter = SfxExecutableFilterContainer::GetComponentFilter();
                pDoc->GetMedium()->SetFilter( pFilter );
            }

			pOwner->InsertDocument( pDoc );
			if ( bLoading && !bLoadStarted )
				pOwner->GetLoadEnvironment_Impl()->DocumentDetected( pDoc, ERRCODE_NONE );
			pViewFrame = pOwner->GetCurrentViewFrame();
		}

		if ( pViewFrame )
		{
			pViewFrame->GetBindings().InvalidateAll( sal_True );
		 	pViewFrame->GetBindings().InvalidateUnoControllers_Impl();
		}
	}
	*/

	m_bDontUpdateShells = sal_True;

	// Send FrameActionEvent to all listener
	if	(
			( m_xController.is()		==	sal_True	)	||
			( m_xComponentWindow.is()	==	sal_True	)
		)
	{
		if ( m_bConnected == sal_True )
		{
			impl_sendFrameActionEvent( FrameAction_COMPONENT_REATTACHED	);
		}
		else
		{
			impl_sendFrameActionEvent( FrameAction_COMPONENT_ATTACHED	);
		}
		//ASMUSS	SfxDesktop::DoFrameAction_Impl( this, bConnected ? ::com::sun::star::frame::FrameAction_COMPONENT_REATTACHED : ::com::sun::star::frame::FrameAction_COMPONENT_ATTACHED );
	}

	m_bConnected = sal_True;

	return sal_True;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
Reference< XWindow > SAL_CALL Frame::getComponentWindow() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	/*ASMUSS
	if ( pOwner && !bHasNewComponent )
	{
		SfxViewShell *pViewShell = pOwner->GetCurrentViewFrame()->GetViewShell();
		if ( pViewShell )
			return VCLUnoHelper::GetInterface(pViewShell->GetWindow());
	}
	*/

	return m_xComponentWindow;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
Reference< XController > SAL_CALL Frame::getController() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Return current controller.
	return m_xController;
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::contextChanged() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	impl_sendFrameActionEvent( FrameAction_CONTEXT_CHANGED );

	/*ASMUSS
	if ( !m_bLoading && pOwner && !pOwner->IsClosing_Impl() )
	{
		SfxViewFrame *pView = pOwner->GetCurrentViewFrame();
		if ( pView )
		{
			SfxBindings &rBindings = pView->GetBindings();
			rBindings.InvalidateUnoControllers_Impl();
			rBindings.InvalidateAll( sal_True );
			SfxComponentViewShell* pShell = PTR_CAST( SfxComponentViewShell, pView->GetViewShell() );
			if ( pShell )
				pShell->InvalidateAll();
		}
	}
	*/
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::addFrameActionListener( const Reference< XFrameActionListener >& xListener ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_addFrameActionListener( xListener ), "Frame::addFrameActionListener()\nInvalid parameter detected.\n" );

	// Add listener to container
	m_aListenerContainer.addInterface( ::getCppuType( ( const Reference< XFrameActionListener >* ) NULL ), xListener );
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
void SAL_CALL Frame::removeFrameActionListener( const Reference< XFrameActionListener >& xListener ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_removeFrameActionListener( xListener ), "Frame::removeFrameActionListener()\nInvalid parameter detected.\n" );

	// Rmove listener from container
	m_aListenerContainer.removeInterface( ::getCppuType( ( const Reference< XFrameActionListener >* ) NULL ), xListener );
}

//*****************************************************************************************************************
//	 XComponent
//*****************************************************************************************************************
void SAL_CALL Frame::dispose() throw( RuntimeException )
{
	// Ready for multithreading
//	MutexGuard aGuard( Application::GetSolarMutex() );
	MutexGuard aGuard( Mutex::getGlobalMutex() );

	// Protection against recursive disposing!
	if ( m_bAlreadyDisposed == sal_False )
	{
		// Set flag against recursive or following calls.
		m_bAlreadyDisposed = sal_True ;

		m_sLoadURL = OUString();

		// Cancel loading.
		if ( m_xLoader.is() == sal_True )
		{
			m_xLoader->cancel();
		}

		// Delete component.
		setComponent( Reference< XWindow >(), Reference< XController >() );

		// Tell all listeners to release this object.
		Reference< XFrame >	xThis( (OWeakObject*)this, UNO_QUERY );

		EventObject aEvent;
		aEvent.Source = xThis;
		m_aListenerContainer.disposeAndClear( aEvent );

		// Force parent container to forget this frame.
		// ( It's contained in m_xParent and so no XEventListener for m_xParent! )
		if ( m_xParent.is() == sal_True )
		{
			m_xParent->getFrames()->remove( xThis );
		}

		// Release interceptor.
		while ( m_xInterceptor.is() == sal_True )
		{
			releaseDispatchProviderInterceptor( m_xInterceptor );
		}

		// Release indicator.
		m_xIndicator = Reference< XStatusIndicator >();
/*ASMUSS		if ( pIndicator )
		{
			delete pIndicator;
			pIndicator = NULL;
		}

		// Prepare Containerwindow for closing.
		if ( pOwner )
		{
			// pOwner must release this UnoFrame
			// Containerwindow is closed by pOwner
			release();
			pOwner->pUnoImp = NULL;					//! MBA
			if ( !pOwner->pImp->bClosing )
				pOwner->DoClose();
			pOwner = NULL;
		}*/

		// If we have our own window ... release it!
		if ( m_xContainerWindow.is() == sal_True )
		{
			if ( m_bIAmWindowOwner == sal_True )
			{
				m_xContainerWindow->dispose();
			}
			impl_setContainerWindow( Reference< XWindow >() );
		}
	}
}

//*****************************************************************************************************************
//	 XComponent
//*****************************************************************************************************************
void SAL_CALL Frame::addEventListener( const Reference< XEventListener >& xListener ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_addEventListener( xListener ), "Frame::addEventListener()\nInvalid parameter detected.\n" );

	// Add listener to container.
	m_aListenerContainer.addInterface( ::getCppuType( ( const Reference< XEventListener >* ) NULL ), xListener );
}

//*****************************************************************************************************************
//	 XComponent
//*****************************************************************************************************************
void SAL_CALL Frame::removeEventListener( const Reference< XEventListener >& xListener ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_removeEventListener( xListener ), "Frame::removeEventListener()\nInvalid parameter detected.\n" );

	// Add listener to container.
	m_aListenerContainer.removeInterface( ::getCppuType( ( const Reference< XEventListener >* ) NULL ), xListener );
}

//*****************************************************************************************************************
//	 XEventListener
//*****************************************************************************************************************
void SAL_CALL Frame::disposing( const EventObject& aEvent ) throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_disposing( aEvent ), "Frame::disposing()\nInvalid parameter detected.\n" );

	// This instance is forced to release references to the specified interfaces by event-source.
	if ( aEvent.Source == m_xContainerWindow )
	{
		impl_setContainerWindow( Reference< XWindow >() );
	}
}

//*****************************************************************************************************************
//	protected method
//*****************************************************************************************************************
void Frame::impl_resetObject()
{
	// Free references to other objects
	m_xFactory				= Reference< XMultiServiceFactory >();
	m_xIndicator			= Reference< XStatusIndicator >();
	m_xContainerWindow		= Reference< XWindow >();
	m_xActiveFrame			= Reference< XFrame >();
	m_xParent				= Reference< XFramesSupplier >();
	m_xLoader				= Reference< XFrameLoader >();
	m_xInterceptor			= Reference< XDispatchProviderInterceptor >();
	m_xComponentWindow		= Reference< XWindow >();
	m_xController			= Reference< XController >();

	m_bIsFrameActive		= sal_True;
	m_bIsFrameUIActive		= sal_True;
	m_bSearchingAllowed		= sal_True;
	m_bIsFrameTop			= sal_True;
	m_bAlreadyDisposed		= sal_True;
	m_bIAmWindowOwner		= sal_True;
	m_bHasNewComponent		= sal_True;
	m_bDontUpdateShells		= sal_True;
	m_bConnected			= sal_True;
	m_bLoading				= sal_True;

	m_sFrameName			= OUString();
	m_sLoadURL				= OUString();

//ASMUSS disposeAndClear() !?
//	m_aListenerContainer
//	m_aChildFramesArray
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void Frame::impl_setContainerWindow( const Reference< XWindow >& xWindow )
{
	// Remove this instance himself from "old" window-listener-container.
	if ( m_xContainerWindow.is() == sal_True )
	{
		m_xContainerWindow->removeWindowListener( this );
	}

	// Safe new window reference.
	m_xContainerWindow = xWindow;

	// Register this instance himself as new listener.
	if ( m_xContainerWindow.is() == sal_True )
	{
		m_xContainerWindow->addWindowListener( this );
	}

	// If new window a on top, register this instance a listener to.
	Reference< XTopWindow > xTopWindow( m_xContainerWindow, UNO_QUERY );
	if ( xTopWindow.is() == sal_True )
	{
		xTopWindow->addTopWindowListener( this );
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void Frame::impl_setComponentWindow( const Reference< XWindow >& xWindow )
{
	// Remember old component; dispose later to avoid flickering.
	Reference< XWindow > xOld = m_xComponentWindow;
	// Take the new one.
	m_xComponentWindow = xWindow;

	if	(
			( m_bHasNewComponent		==	sal_True	)	&&
			( m_xComponentWindow.is()	==	sal_True	)
		)
	{
		// Set correct size before showing the window.
		/*ASMUSS
		Window *pWindow = VCLUnoHelper::GetWindow(xWindow);
		Size aSize = pWindow->GetOutputSizePixel();
		xComponentWindow->setPosSize( 0, 0, aSize.Width(), aSize.Height() , POSSIZE );
		*/
		m_xComponentWindow->setVisible( sal_True );
	}

	if ( xOld.is() == sal_True )
	{
		// All VclComponents are XComponents; so call dispose before discarding
		// a Reference< XVclComponent >, because this frame is the owner of the Component.
		xOld->setVisible( sal_False );
		xOld->dispose();
	}

	// Transfer window title to ViewFrame.
	/*ASMUSS
	if ( pOwner && pOwner->GetCurrentViewFrame() )
		pOwner->GetCurrentViewFrame()->UpdateTitle();
	*/
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void Frame::impl_setController( const Reference< XController >& xController )
{
	// Safe old value for disposing AFTER set of new controller!
	Reference< XController > xOld = m_xController;
	// Take the new one.
	m_xController = xController;
	// Dispose old instance.
	if ( xOld.is() == sal_True )
	{
		xOld->dispose();
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void Frame::impl_sendFrameActionEvent( const FrameAction& aAction )
{
	// Send FrameAction event to all listener.
	// Get container for right listener.
	OInterfaceContainerHelper* pContainer = m_aListenerContainer.getContainer( ::getCppuType( ( const Reference< XFrameActionListener >*) NULL ) );

	if ( pContainer != NULL )
	{
		// Build action event.
		FrameActionEvent aFrameActionEvent( (OWeakObject*)this, this, aAction );

		// Get iterator for access to listener.
		OInterfaceIteratorHelper aIterator( *pContainer );
		// Send message to all listener.
		while ( aIterator.hasMoreElements() == sal_True )
		{
			((XFrameActionListener *)aIterator.next())->frameAction( aFrameActionEvent );
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
Reference< XDispatch > Frame::impl_queryDispatch(	const	URL&	 	aURL				,
														const	OUString&	sTargetFrameName	,
																sal_Int32	nSearchFlags		)
{
	// Set default return value.
	Reference< XDispatch > xReturn = Reference< XDispatch >();

	/*ASMUSS
	if ( aURL.Complete.compareToAscii( ".uno:LifeTime" ) == EQUAL )
	{
		//ASMUSS xReturn = SFX_APP()->Get_Impl()->pAppDispatch;
	}

//ASMUSS???	OUString aName = aTargetFrameName;

    if ( m_sFrameName.getLength() > 0 && pOwner &&
		 aURL.Protocol.compareToAscii( ".uno:" ) != COMPARE_EQUAL &&
		 aURL.Protocol.compareToAscii( "slot:" ) != COMPARE_EQUAL &&
         pOwner->GetParentFrame() &&
		 pOwner->GetParentFrame()->GetCurrentViewFrame()->GetViewShell()->IsImplementedAsFrameset_Impl() )
		aName = DEFINE_CONST_UNICODE( "_parent" );

	if ( aName.len() )
	{
		// Name was given; try to find the corresponding frame
		String aStr( aName );
		if ( pOwner && ( aStr.CompareToAscii( "_tool:", 6 ) == COMPARE_EQUAL ) )
		{
			// the name belongs to a tool frame
			if ( pOwner->GetCurrentViewFrame() )
			{
				SfxComponentViewShell* pShell = PTR_CAST( SfxComponentViewShell,
					pOwner->GetCurrentViewFrame()->GetViewShell() );
				if ( pShell )
					xReturn = pShell->GetDispatch( aStr.GetToken( 1,':' ) );
			}

			if ( xReturn.is() )
				return xReturn;

			if ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::PARENT )
			{
				// maybe the tool frame lies above
				::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider >  xParent( xCreator ,::com::sun::star::uno::UNO_QUERY );
				if ( xParent.is() )
					return xParent->queryDispatch( aURL, aName, nSearchFlags );
			}

			return xReturn;
		}
		else
		{
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider >  xThis( this );
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame >  aRef( (aName.equals(aFrameName) ) ? (::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > )this : findFrame( aName, nSearchFlags ) );
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider >  xProv( aRef, ::com::sun::star::uno::UNO_QUERY );
			if ( xProv != xThis )
			{
				if ( xProv.is() )
					xReturn = xProv->queryDispatch( aURL, aRef->getName(), ::com::sun::star::frame::FrameSearchFlag::SELF );
				return xReturn;
			}
		}
	}

	if ( !aName.len() || ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::SELF ) )
	{
		// First ask the controller
		::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider >  xProv( xController, ::com::sun::star::uno::UNO_QUERY );
		if ( xProv.is() )
			xReturn = xProv->queryDispatch( aURL, String(), 0 );
		if ( xReturn.is() )
			return xReturn;

		xReturn = pOwnDispatch;
	}

	if ( aURL.Protocol.compareToAscii( ".uno:" ) == COMPARE_EQUAL ||
		 aURL.Protocol.compareToAscii( "slot:" ) == COMPARE_EQUAL )
	{
		if ( !aName.len() || ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::SELF ) )
		{
			// Now try by myself
			if ( pOwner )
			{
				SfxViewFrame *pViewFrame = pOwner->GetCurrentViewFrame();
				sal_uInt16 nId=0;
				if ( aURL.Protocol.compareToAscii( ".uno:" ) == COMPARE_EQUAL )
				{
					SfxSlotPool& rPool = SFX_APP()->GetSlotPool( pViewFrame );
					const SfxSlot* pSlot = rPool.GetUnoSlot( aURL.Path );
					nId = pSlot ? pSlot->GetSlotId() : 0;
				}
				else
				{
					nId = (sal_uInt16) aURL.Path.toInt32();
				}

				if ( nId )
				{
                	SfxDispatcher* pDispatcher = NULL;
                	if ( pViewFrame && pViewFrame->GetInterface()->GetSlot( nId ) )
                	{
                    	SfxBindings* pBindings = &pViewFrame->GetBindings();
                    	pBindings->ENTERREGISTRATIONS();
                    	SfxOfficeDispatch* pDisp = new SfxOfficeDispatch( *pBindings, pViewFrame->GetDispatcher(), nId, aURL );
                    	pBindings->LEAVEREGISTRATIONS();
                    	return pDisp;
                	}
				}
			}
		}

		if ( !aName.len() || ( nSearchFlags & ::com::sun::star::frame::FrameSearchFlag::PARENT ) )
		{
			// try parent
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider >  xParent( xCreator, ::com::sun::star::uno::UNO_QUERY );
			if( xParent.is() )
        		return xParent->queryDispatch( aURL, aTargetFrameName, nSearchFlags );
		}

		return ::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatch > ();
	}
	else
		// component ::com::sun::star::util::URL; all component URLs may be dispatched
		xReturn = pOwnDispatch;

	*/

	return xReturn;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact Andreas Schluens [as] or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef DBG_UTIL

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_append( const Reference< XFrame >& xFrame )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xFrame		==	NULL		)	||
			( xFrame.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_queryFrames( sal_Int32 nSearchFlags )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( nSearchFlags != FrameSearchFlag::AUTO		)	&&
			( nSearchFlags != FrameSearchFlag::PARENT	)	&&
			( nSearchFlags != FrameSearchFlag::SELF		)	&&
			( nSearchFlags != FrameSearchFlag::CHILDREN	)	&&
			( nSearchFlags != FrameSearchFlag::CREATE	)	&&
			( nSearchFlags != FrameSearchFlag::SIBLINGS	)	&&
			( nSearchFlags != FrameSearchFlag::TASKS	)	&&
			( nSearchFlags != FrameSearchFlag::ALL		)	&&
			( nSearchFlags != FrameSearchFlag::GLOBAL	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_remove( const Reference< XFrame >& xFrame )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xFrame		==	NULL		)	||
			( xFrame.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_setActiveFrame( const Reference< XFrame >& xFrame )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xFrame		==	NULL		)	||
			( xFrame.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_queryDispatch(	const	URL&	 	aURL			,
													  		const	OUString&	sTargetFrameName,
													  				sal_Int32	nSearchFlags	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &aURL							==	NULL		)	||
			( aURL.Complete.getLength()		<	1			)	||
			( &sTargetFrameName				==	NULL		)	||
			( sTargetFrameName.getLength()	<	1			)	||
			(
				( nSearchFlags != FrameSearchFlag::AUTO		)	&&
				( nSearchFlags != FrameSearchFlag::PARENT	)	&&
				( nSearchFlags != FrameSearchFlag::SELF		)	&&
				( nSearchFlags != FrameSearchFlag::CHILDREN	)	&&
				( nSearchFlags != FrameSearchFlag::CREATE	)	&&
				( nSearchFlags != FrameSearchFlag::SIBLINGS	)	&&
				( nSearchFlags != FrameSearchFlag::TASKS	)	&&
				( nSearchFlags != FrameSearchFlag::ALL		)	&&
				( nSearchFlags != FrameSearchFlag::GLOBAL	)
			)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_registerDispatchProviderInterceptor( const Reference< XDispatchProviderInterceptor >& xInterceptor )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xInterceptor		==	NULL		)	||
			( xInterceptor.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_releaseDispatchProviderInterceptor( const Reference< XDispatchProviderInterceptor >& xInterceptor )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xInterceptor		==	NULL		)	||
			( xInterceptor.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_updateViewData( const Any& aValue )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &aValue			==	NULL		)	||
			( aValue.hasValue()	==	sal_False	)
			//ASMUSS Wenn der Typ noch bekannt ist, dann auch den abfragen!
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_createNewEntry(	const	OUString&				  	sURL		,
			  												const	Sequence< PropertyValue >&	seqArguments,
															const	OUString&				  	sTitle		)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &sURL						==	NULL	)	||
			( sURL.getLength()			<	1		)	||
			( &seqArguments				==	NULL	)	||
			( seqArguments.getLength()	<	1		)	||
			( &sTitle					==	NULL	)	||
			( sTitle.getLength()		<	1		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_loadFinished( const Reference< XFrameLoader >& xLoader )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xLoader		==	NULL		)	||
			( xLoader.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_loadCancelled( const Reference< XFrameLoader >& xLoader )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xLoader		==	NULL		)	||
			( xLoader.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_windowResized( const WindowEvent& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &aEvent == NULL )
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_windowActivated( const EventObject& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &aEvent == NULL )
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_windowDeactivated( const EventObject& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &aEvent == NULL )
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_initialize( const Reference< XWindow >& xWindow )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xWindow		==	NULL		)	||
			( xWindow.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_setCreator( const Reference< XFramesSupplier >& xCreator )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xCreator		==	NULL		)	||
			( xCreator.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_setName( const OUString& sName )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &sName			==	NULL	)	||
			( sName.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_findFrame(	const	OUString&	sTargetFrameName	,
								 								sal_Int32	nSearchFlags		)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &sTargetFrameName		==	NULL	)	||
			// sTargetFrameName can be ""!
			(
				( nSearchFlags != FrameSearchFlag::AUTO		)	&&
				( nSearchFlags != FrameSearchFlag::PARENT	)	&&
				( nSearchFlags != FrameSearchFlag::SELF		)	&&
				( nSearchFlags != FrameSearchFlag::CHILDREN	)	&&
				( nSearchFlags != FrameSearchFlag::CREATE	)	&&
				( nSearchFlags != FrameSearchFlag::SIBLINGS	)	&&
				( nSearchFlags != FrameSearchFlag::TASKS	)	&&
				( nSearchFlags != FrameSearchFlag::ALL		)	&&
				( nSearchFlags != FrameSearchFlag::GLOBAL	)
			)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_setComponent(	const	Reference< XWindow >&		xComponentWindow	,
				  											const	Reference< XController >&	xController			)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xComponentWindow		==	NULL		)	||
			( xComponentWindow.is()	==	sal_False	)	||
			( &xController			==	NULL		)	||
			( xController.is()		==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_addFrameActionListener( const Reference< XFrameActionListener >& xListener )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xListener			==	NULL		)	||
			( xListener.is()		==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_removeFrameActionListener( const Reference< XFrameActionListener >& xListener )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xListener			==	NULL		)	||
			( xListener.is()		==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_addEventListener( const Reference< XEventListener >& xListener )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xListener			==	NULL		)	||
			( xListener.is()		==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_removeEventListener( const Reference< XEventListener >& xListener )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xListener			==	NULL		)	||
			( xListener.is()		==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool Frame::impldbg_checkParameter_disposing( const EventObject& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &aEvent				==	NULL		)	||
			( aEvent.Source.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

#endif	// #ifdef DBG_UTIL

}	// namespace framework
