/*************************************************************************
 *
 *  $RCSfile: frameloaderfactory.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_FRAMELOADERFACTORY_HXX_
#include <services/frameloaderfactory.hxx>
#endif

#ifndef __FRAMEWORK_WILDCARD_HXX_
#include <classes/wildcard.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_XFRAMELOADER_HPP_
#include <com/sun/star/frame/XFrameLoader.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl									;
using namespace ::rtl									;
using namespace ::cppu									;
using namespace ::com::sun::star::uno					;
using namespace ::com::sun::star::lang					;
using namespace ::com::sun::star::beans					;
using namespace ::com::sun::star::frame					;
using namespace ::com::sun::star::registry				;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

#define	SERVICENAME										"com.sun.star.framework.FrameLoaderFactory"
#define	IMPLEMENTATIONNAME								"com.sun.star.comp.framework.FrameLoaderFactory"

#define	PROPERTY_REGISTRY								OUString(RTL_CONSTASCII_USTRINGPARAM("Registry"))

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
FrameLoaderFactory::FrameLoaderFactory( const Reference< XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      OMutexMember is a struct with a mutex as member. We can't use a mutex as member directly, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember	(			)
		,	OWeakObject		(			)
		// Init member
		,	m_xFactory		( xFactory	)
		,	m_aRegistryCache(			)
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
FrameLoaderFactory::~FrameLoaderFactory()
{
	m_aRegistryCache.clear();
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************
DEFINE_XINTERFACE_3						(	FrameLoaderFactory						,
											OWeakObject								,
											DIRECT_INTERFACE(XTypeProvider			),
											DIRECT_INTERFACE(XServiceInfo			),
											DIRECT_INTERFACE(XMultiServiceFactory	)
										)

DEFINE_XTYPEPROVIDER_3					(	FrameLoaderFactory	,
											XTypeProvider		,
											XServiceInfo		,
											XMultiServiceFactory
										)

DEFINE_XSERVICEINFO_ONEINSTANCESERVICE	(	FrameLoaderFactory	,
											SERVICENAME			,
											IMPLEMENTATIONNAME
										)

//*****************************************************************************************************************
//	XMultiServiceFactory
//*****************************************************************************************************************
Reference< XInterface > SAL_CALL FrameLoaderFactory::createInstance( const OUString& sServiceSpecifier ) throw(	Exception		,
							  																					RuntimeException)
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Create a new FrameLoader-instance.
	return impl_createFrameLoader( sServiceSpecifier, Sequence< PropertyValue >() );
}

//*****************************************************************************************************************
//	XMultiServiceFactory
//*****************************************************************************************************************
Reference< XInterface > SAL_CALL FrameLoaderFactory::createInstanceWithArguments(	const	OUString&			sServiceSpecifier	,
																					const	Sequence< Any >&	seqArguments		) throw(	Exception		,
																																				RuntimeException)
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	// Copy arguments from "ANY"-structures to structures of "PropertyValue".
	sal_Int32					nArgumentCount	= seqArguments.getLength();
    Sequence< PropertyValue >	seqProperties	( nArgumentCount );
	const Any*					pAnys			= seqArguments.getConstArray();
	PropertyValue*				pProperties		= seqProperties.getArray();

    for ( sal_Int32 nPosition=0; nPosition<nArgumentCount; ++nPosition )
    {
		pAnys[nPosition] >>= pProperties[nPosition];
    }

	// Create a new FrameLoader-instance and return it.
	return impl_createFrameLoader( sServiceSpecifier, seqProperties );
}

//*****************************************************************************************************************
//	XMultiServiceFactory
//*****************************************************************************************************************
Sequence< OUString > SAL_CALL FrameLoaderFactory::getAvailableServiceNames() throw( RuntimeException )
{
	// Ready for multithreading
	MutexGuard aGuard( m_aMutex );

	return Sequence< OUString >();
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
Reference< XInterface > FrameLoaderFactory::impl_createFrameLoader(	const	OUString&					sURL			,
																	const	Sequence< PropertyValue >&	seqArguments	) throw(	Exception		,
																											 						RuntimeException)
{
	// Set default return value.
	Reference< XInterface > xReturn = Reference< XInterface >();

	// Have we a valid registry?
	if ( m_aRegistryCache.isValid() == INVALID )
	{
		// ... NO; Then init a new one!
		// Ask servicemanager for XPropertySet-interface.
		Reference< XPropertySet >		xPropertys( m_xFactory, UNO_QUERY );
		Reference< XSimpleRegistry >	xRegistry;
		// Get the value of the propety "Registry".
		Any aValue = xPropertys->getPropertyValue( PROPERTY_REGISTRY );
		aValue >>= xRegistry;
		// Fill cache with values from right registry.
		m_aRegistryCache.fill( xRegistry );
		// Was operation successful?
		if ( m_aRegistryCache.isValid() == INVALID )
		{
			// ... NO; Handle this error.
			throw( Exception() );
		}
	}

	// First search a matching pattern OR right extension at all entries of cache.
    sal_uInt32 nEntryCount	= m_aRegistryCache.getEntryCount();
    sal_uInt32 nEntryNumber	= 0;
	// First get count of cache entries...
	// .. and loop over all entries.
    while	(
				( nEntryNumber	<	nEntryCount	)	&&
				( xReturn.is()	==	sal_False	)
			)
    {
		// Loop over all pattern of current entry.
        sal_uInt32 nPatternCount	= m_aRegistryCache.getPatternCount( nEntryNumber );
		sal_uInt32 nPatternNumber	= 0;
		while	(
					( nPatternNumber	<	nPatternCount	)	&&
					( xReturn.is()		==	sal_False		)
				)
        {
			// If pattern match the url ...
            if ( Wildcard::match( sURL, m_aRegistryCache.getPattern( nEntryNumber, nPatternNumber ) ) == sal_True )
			{
				// ... try to get a frameloader for this pattern.
				OUString sLoaderName = m_aRegistryCache.getEntryName( nEntryNumber );
				Reference< XFrameLoader > xLoader( m_xFactory->createInstance( sLoaderName ), UNO_QUERY );
				if ( xLoader.is() == sal_True )
				{
					// We have a valid loader. Set it for return and break loop!
					xReturn = Reference< XInterface >( xLoader, UNO_QUERY );
				}
			}
			// Else; step to next pattern ...
			++nPatternNumber;
        }
		// .. or step to next entry.
		++nEntryNumber;
    }

	// We have no valid loader! Try it again with extension as search-parameter.
	if ( xReturn.is() == sal_False )
	{
		// Get the extensions of the given URL.
		// Take first a default value.
		OUString sExtension;
		// Search for last occurence of '.'
		sal_Int32	nPointPosition = sURL.lastIndexOf( '.' );
		if ( nPointPosition >= 0 )
		{
			// We have found a '.' in URL.
			// But if character before this position a '/' ... this is not a '.' of an extensions!
			if ( sURL.lastIndexOf( '.', nPointPosition ) != (nPointPosition-1) )
			{
				// We have found a valid extensions. Copy value to new variable (but without dot!).
				sExtension = sURL.copy( nPointPosition+1 );
			}
		}

		// If extension of URL well known ...
    	if ( sExtension.getLength() > 0 )
    	{
			// ... search for extension at all entries of cache.
			// ATTENTION: nEntryCount is the same, like before!
			// The cache will not refereshed at runtime.
    		sal_uInt32 nEntryNumber	= 0;
			// First get count of cache entries...
			// .. and loop over all entries.
    		while	(
						( nEntryNumber	<	nEntryCount	)	&&
						( xReturn.is()	==	sal_False	)
					)
    		{
				// Loop over all extension of current entry.
		        sal_uInt32 nExtensionCount	= m_aRegistryCache.getExtensionCount( nEntryNumber );
				sal_uInt32 nExtensionNumber	= 0;
				while	(
							( nExtensionNumber	<	nExtensionCount	)	&&
							( xReturn.is()		==	sal_False		)
						)
        		{
					// If extension is in current extension-list ...
            		if ( m_aRegistryCache.getExtension( nEntryNumber, nExtensionNumber ) == sExtension )
					{
						// ... try to get a frameloader for this extension.
						OUString sLoaderName = m_aRegistryCache.getEntryName( nEntryNumber );
						Reference< XFrameLoader > xLoader( m_xFactory->createInstance( sLoaderName ), UNO_QUERY );
						if ( xLoader.is() == sal_True )
						{
							// If we have a valid loader - set it for return.
							// (A valid return value is the break condition for the loops below!)
							xReturn = Reference< XInterface >( xLoader, UNO_QUERY );
						}
					}
					// Else; steop to next extension ...
					++nExtensionNumber;
        		}
				// .. or step to next entry.
				++nEntryNumber;
    		}
    	}
	}

	// Return result of this operation.
    return xReturn;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact Andreas Schluens [as] or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef DBG_UTIL
/*
//*****************************************************************************************************************
sal_Bool FrameLoaderFactory::impldbg_checkParameter_XXX(...)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xFrame		==	NULL		)	||
			( xFrame.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}
*/
#endif	//	#ifdef DBG_UTIL

}		//	namespace framework
