/*************************************************************************
 *
 *  $RCSfile: tasksaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:11:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_TASKSACCESS_HXX_
#include <services/tasksaccess.hxx>
#endif

#ifndef __FRAMEWORK_TASKSENUMERATION_HXX_
#include <services/tasksenumeration.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl							;
using namespace ::rtl							;
using namespace ::cppu							;
using namespace ::com::sun::star::uno			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::container		;
using namespace ::com::sun::star::frame			;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
TasksAccess::TasksAccess(	const	Reference< XDesktop >&		xDesktop	,
									OInterfaceContainerHelper*	pTasks		)
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      tIMPLMutexContainer is a struct with a mutex as member. We can't use a mutex as member, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember	(			)
		,	OWeakObject		(			)
		// Init member
		,	m_xDesktop		( xDesktop	)
		,	m_pTasks		( pTasks	)
{
	// Safe impossible cases
	OSL_ENSHURE( impldbg_checkParameter_TasksAccess( xDesktop, pTasks ), "TasksAccess::TasksAccess()\nInvalid parameter detected!\n" );
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
TasksAccess::~TasksAccess()
{
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider
//*****************************************************************************************************************

DEFINE_XINTERFACE_3		(	TasksAccess							,
							OWeakObject					  		,
							DIRECT_INTERFACE(XTypeProvider		),
							DIRECT_INTERFACE(XEnumerationAccess	),
							DIRECT_INTERFACE(XElementAccess		)
						)

DEFINE_XTYPEPROVIDER_3	(	TasksAccess			,
							XTypeProvider		,
							XEnumerationAccess	,
							XElementAccess
						)

//*****************************************************************************************************************
//	XEnumerationAccess
//*****************************************************************************************************************
Reference< XEnumeration > SAL_CALL TasksAccess::createEnumeration() throw( RuntimeException )
{
	// Read for multithreading
	MutexGuard aGUard( m_aMutex );

	// Set default return value, if method failed.
	// If no desktop exist and there is no taskcontainer - return an empty enumeration!
	Reference< XEnumeration > xReturn = Reference< XEnumeration >();

	// Try to "lock" the desktop for access to taskscontainer.
	Reference< XInterface > xLock = m_xDesktop.get();
	if ( xLock.is() == sal_True )
	{
		// Desktop exist => pointer to taskscontainer must be valid.
		// Initialize a new enumeration ... if some tasks exist!
		// (TasksEnumeration will make an assert, if we initialize the new instance without valid values!)
		if ( m_pTasks->getLength() > 0 )
		{
			// Create new instance in memory (user of this metod will be owner of this new object!)
			TasksEnumeration* pEnumeration = new TasksEnumeration( m_pTasks->getElements() );
			// Cast it to right reference.
			xReturn = Reference< XEnumeration >( (OWeakObject*)pEnumeration, UNO_QUERY );
		}
	}

	// Return result of this operation.
	return xReturn;
}

//*****************************************************************************************************************
//	XElementAccess
//*****************************************************************************************************************
Type SAL_CALL TasksAccess::getElementType() throw( RuntimeException )
{
	// Elements in list an enumeration are tasks!
	// Return the uno-type of XTask.
	return ::getCppuType((const Reference< XTask >*)NULL);
}

//*****************************************************************************************************************
//	XElementAccess
//*****************************************************************************************************************
sal_Bool SAL_CALL TasksAccess::hasElements() throw( RuntimeException )
{
	// Read for multithreading
	MutexGuard aGUard( m_aMutex );

	// Set default return value, if method failed.
	sal_Bool bReturn = sal_False;

	// Try to "lock" the desktop for access to taskscontainer.
	Reference< XInterface > xLock = m_xDesktop.get();
	if ( xLock.is() == sal_True )
	{
		// Desktop exist => pointer to taskscontainer must be valid.
		if ( m_pTasks->getLength() > 0 )
		{
			bReturn = sal_True;
		}
	}

	// Return result of this operation.
	return bReturn;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact Andreas Schluens [as] or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef DBG_UTIL

//*****************************************************************************************************************
sal_Bool TasksAccess::impldbg_checkParameter_TasksAccess(	const	Reference< XDesktop >&		xDesktop	,
																	OInterfaceContainerHelper*	pTasks		)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xDesktop		==	NULL		)	||
			( xDesktop.is()	==	sal_False	)	||
			( pTasks		==	NULL		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef DBG_UTIL

}		//	namespace framework
