/*************************************************************************
 *
 *  $RCSfile: isetbrw.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:34 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _MyISETBRW_HXX
#define _MyISETBRW_HXX

#ifndef _SVTREEBX_HXX
#include <svtools/svtreebx.hxx>
#endif
#ifndef _SV_BUTTON_HXX
#include <vcl/button.hxx>
#endif
#ifndef _SHL_HXX
#include <tools/shl.hxx>
#endif


#define ITEMFLAG_DOTBUTTON			0x0001
// bei Enum- & Stringlist-Items werden die entsprechenden Item-Daten
// in der DropDownlistbox angezeigt. Fuer alle anderen Items, die
// dieses Flags gesetzt haben, wird beim Aufklappen der Liste der Handler
// GetItemDropDownData gerufen. Die einzelnen Zeilen im String muessen durch
// '\n' voneinander getrennt sein. Nach Auswahl eines Wertes wird
// NotifyConvertToItemValue gerufen. Hierdurch ist es moeglich, z.B.
// SfxStringItems als DropDown-Comboboxen editieren zu lassen.
#define ITEMFLAG_COMBOBUTTON    	0x0002
#define ITEMFLAG_STRINGEDIT			0x0004
#define ITEMFLAG_NUMBEREDIT			0x0008
#define ITEMFLAG_NOTEXPANDABLE		0x0010
#define ITEMFLAG_HIDE_IN_MULTISEL	0x0020
#define ITEMFLAG_HIDE				0x0040

#define ITEMTYPE_DONTKNOW			0
#define ITEMTYPE_INT16				1
#define ITEMTYPE_INT32				2
#define ITEMTYPE_UINT16				3
#define ITEMTYPE_UINT32				4
#define ITEMTYPE_BOOL				5
#define ITEMTYPE_ENUM				6
#define ITEMTYPE_STRINGLIST			7
#define ITEMTYPE_RANGE				8
#define ITEMTYPE_COLOR				9
#define ITEMTYPE_STRING				10

class ImpNodeOpenList;
class ImplEdCtrl;
class ImplMultiLineEdCtrl;
class ImplListBoxCtrl;
class ImplColorWindow;
class SfxStringListItem;
class ImpBrwString;
class SfxPoolItem;
class SfxItemSet;

class BrwListEntry : public SvLBoxEntry
{
public:
	USHORT nItemFlags;

	BrwListEntry( USHORT nItmFlags = 0) { nItemFlags = nItmFlags; }
	~BrwListEntry();

	BOOL HasDotButton() const { return (BOOL)(nItemFlags&ITEMFLAG_DOTBUTTON)!=0 ;}
	BOOL HasComboButton() const	{ return (BOOL)(nItemFlags&ITEMFLAG_COMBOBUTTON)!=0;}
	void SetItemFlags( USHORT nFlags )	{	nItemFlags = nFlags; }
	USHORT GetItemFlags() const	{ return nItemFlags; }
	SfxPoolItem* GetItem() const { return (SfxPoolItem*)GetUserData(); }
	void SetItem( const SfxPoolItem* pItem ) { SetUserData((void*)pItem); }
};

class ItemSetBrowser : public SvTreeListBox
{
	friend class ImpBrwString;
	friend class ImplListBoxCtrl;
private:

#ifdef _ISETBRW_CXX
	BOOL				IsNodeOpen( USHORT nWhich );
	void				SetNodeOpen( USHORT nWhich, BOOL bOpen );
	void				PaintTabEmphasis( long nX );
	DECL_LINK( DotClickHdl, ImageButton * );
	DECL_LINK( ComboClickHdl, ImageButton * );
	DECL_LINK( CtrlSelectHdl, Window * );
	DECL_LINK( CtrlFinishedHdl, Window * );
	DECL_LINK( CtrlKeyActionHdl, Window * );
	void				DeleteButtons();
	void				CheckPos( Point&, const Size& );
#endif

	int					bSvLook;
	Color				aLineColor;
	SfxItemSet*			pCurSet;
	List				aItemSetList;
	String				aCurInsertedValueText; // fuer CreateEntry
	ImpNodeOpenList*	pNodeOpenList;
	long				nValueTab;
	USHORT				nCurrentWhich;
	long				nCurLine;
	BrwListEntry*		pCurSelected;
	ImageButton*		pCurDotButton;
	ImageButton*		pCurComboButton;
	Link				aDotClickHdl;
	Link				aComboClickHdl;
	Link				aImpListBoxSelectHdl;
	Link				aModifyHdl;
	USHORT				nModifyId;
	Image				aComboBmp;

	BOOL				bSettingTab;
	BOOL				bTabSettingDisabled;
	BOOL				bStartTabSetting;
	Pointer				aOldPtr;

	Window*				pCurCtrl;
	ImplEdCtrl*			pCurEdit;
	ImplMultiLineEdCtrl* pCurMEdit;
	ImplListBoxCtrl*	 pCurLBox;
	ImplColorWindow*	pCurColWin;
	USHORT				nCurCtrlId;
	USHORT				GetItemType( const SfxPoolItem* pItem );
	USHORT				nCurItemType;  // StringItem, BoolItem, ...
	BOOL				bComboListExpanded;

	Color				aCurSelectedColor;
	USHORT				nCurSelectedListPos;
	Range				aWhichRange;


protected:

	long				nLineLength;
	Font				aValueFont;		// Font fuer Item-Werte

	void				Clear();
	void				FillTree();
	void				ShowButtons( BrwListEntry* pEntry );
	void				HideButtons();
	BOOL 				IsInAllItemSets( USHORT nWhich );
	BOOL				IsDontCare( const SfxPoolItem* pItem );
	void				IntersectSets();
	String				GetDontCareValueText( USHORT nWhich, BrwListEntry* );

	virtual void		SetTabs();
	virtual SvLBoxEntry* CreateEntry() const;
	virtual void 		InitEntry(SvLBoxEntry*,const String&,const Image&,const Image&);
	virtual void		NotifyBeginScroll();
	virtual void		NotifyEndScroll();
	virtual void		NotifyInvalidating();
	virtual void		Paint( const Rectangle& );
	virtual void		Resize();
	// zeichnet ueber & unter dem Entry eine Linie
	void				PaintBorderLines( BrwListEntry* );
	virtual void		MouseButtonDown( const MouseEvent& );
	virtual void		MouseButtonUp( const MouseEvent& );
	virtual void		MouseMove( const MouseEvent& );
	virtual void		KeyInput( const KeyEvent& );
	virtual void		LoseFocus();
	virtual void		ExpandedHdl();

	virtual USHORT		GetItemFlags( const SfxPoolItem* );
	virtual String		GetItemTitle( USHORT nWhichId ) const;
	virtual BOOL  		GetItemBitmap( USHORT nWhichId, Image& );
	virtual String		GetItemDropDownData( USHORT nWhichId );
	virtual BOOL		GetRangeBitmaps( USHORT nWhichId,
							Image& rCollapsedBmp,
							Image& rExpandedBmp );

	virtual void		ComboButtonClick( BrwListEntry* );
	virtual void		DotButtonClick( BrwListEntry* );
	virtual void		ValueClick( BrwListEntry*, const MouseEvent& );

	BOOL				bOpeningCtrl; // fuer LoseFocus
	void				ShowStringEdit( BrwListEntry* );
	void				ShowNumberEdit( BrwListEntry* );
	void				ShowMultiEdit( BrwListEntry* );
	void				ShowListBox( BrwListEntry* );
	void				ShowColorWindow( BrwListEntry* );
	void				EvaluateAndHideCtrl();
	BOOL				SetItemValue(USHORT nWhichId, const SfxPoolItem* pNew,
							BrwListEntry* );
	BOOL				ToggleItemValue( BrwListEntry* pEntry );

	virtual BOOL		NotifyConvertToItemValue( SfxPoolItem*,
							const String& rValue,
							USHORT nItemType );
	virtual BOOL		NotifyChangingItem(USHORT nWhichId, const SfxPoolItem* pNew);

	virtual Rectangle 	GetFocusRect( SvLBoxEntry*, long nLine );

	void				DrawTabLine( long nY );
	BrwListEntry* 		GetListEntry( USHORT nWhich );
	void				CloseCurCtrl( BOOL bCancel );

public:
						ItemSetBrowser( Window* pParent,
							WinBits nWinStyle = WB_BORDER );
						ItemSetBrowser( Window* pParent, const ResId& );

	virtual	   			~ItemSetBrowser();

	virtual void		SetItemSet( SfxItemSet* );
	virtual void		AddItemSet( SfxItemSet* );
	virtual void		RemoveAllItemSets();
	virtual void		RemoveItemSet( SfxItemSet* );
	ULONG				GetItemSetCount() const;
	SfxItemSet*			GetItemSet( ULONG nPos ) const;
	void				SetValueText( BrwListEntry*, const String& );
	String				GetValueText( BrwListEntry* );

	void SetNodeBitmaps( const Image& rPlus, const Image& rMinus )
		{ SvTreeListBox::SetNodeBitmaps( rPlus, rMinus ); }

	void				SelectHdl();

	void				SetModifyHdl( const Link& rLink ){ aModifyHdl = rLink;}
	const Link&			GetModifyHdl() const { return aModifyHdl; }
	USHORT				GetModifyId() const { return nModifyId; }
	virtual void		Modify();
	void            	Update() { Invalidate(); }
	void				Update( USHORT nWhich );

	virtual void		UpdateRange( const SfxItemSet* pNewSet,
									 USHORT nFromWhich, USHORT nToWhich );
	virtual void		UpdateRange( const SfxItemSet* pNewSet,
									 USHORT *nRangeList );

	BOOL				IsEntryInView( BrwListEntry* ) const;
	BOOL				IsEntryInView( BrwListEntry*, Rectangle& ) const;

	void				SetValueTab( long nTab );
	long				GetValueTab() const { return nValueTab; }

	void				SetValueFont( const Font& rFont) {aValueFont=rFont; Invalidate();}
	Font				GetValueFont() const { return aValueFont; }
	void				SetWhichRange( const Range& rRange );
	Range				GetWhichRange() const { return aWhichRange; }
};



#endif

