/*************************************************************************
 *
 *  $RCSfile: vcont.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:35 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#if __PRIVATE
#else
#pragma message("Do not include obsolete VControl-Headers!")
#endif

#ifndef _SVX_VCONT_HXX
#define _SVX_VCONT_HXX

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif
#ifndef _LINK_HXX //autogen
#include <tools/link.hxx>
#endif
#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#ifndef _RTTI_HXX //autogen
#include <tools/rtti.hxx>
#endif

class VCControl;
class VCRadioButton;
class VCGroupBox;
class VCItemPool;
class Window;
class MouseEvent;
class KeyEvent;
class Rectangle;

#define VCCONT_IMPORTING 		0x0001
#define VCCONT_UPDATING_LINKS 	0x0002

//-------------------
// class VCContainer
//-------------------

class VCContainer
{
	friend class VCControl;
	friend class VCButton;
	friend class VCGroupBox;
	friend class VCRadioButton;
	friend class ImpSjJScriptRadioObject;

	USHORT			nFlags;
	USHORT			nUniqueId;	// Hilfszaehler beim Importieren

protected:
	List			aChildList;
	Window*			pWindow;
	String			aLinkData;
	String			aLinkName;
	Link			aGetLinkDataHdlLink;
	Link			aSetLinkDataHdlLink;

	BOOL			HasName( const String& rStr ) const;
	String			GetUniqueName( VCControl* pChild );

	VCControl*		GetGroupBox( VCControl* pActControl );
	void			UncheckRadios( VCRadioButton* pActRadio );
	void			UncheckButFirstRadios( VCGroupBox* pGroup );

public:
	TYPEINFO();
					VCContainer();
					~VCContainer();

	void			InsertChild( VCControl* pChild, ULONG nPos = LIST_APPEND );
	void			RemoveChild( VCControl* pChild );
	void			MoveChild( VCControl* pChild, ULONG nNewPos );
	void			UpdateAllTabPos();

	ULONG			GetChildCount() const { return aChildList.Count(); }
	ULONG			GetChildPos( VCControl* pChild ) const;
	VCControl*		GetChildByPos( ULONG nPos );
	VCControl*		GetChildByName( const String& rName, BOOL bCase=FALSE );

	Window*			GetCtrlParent();
	void			SetSVLook( BOOL bSVLook=TRUE );

	void			SetCurrentLinkData( const String& rLinkData );
	String			GetCurrentLinkData() const { return aLinkData; }
	String			GetCurrentLinkName() const { return aLinkName; }

	void			SetGetLinkDataHdl( const Link& rLink )
						{ aGetLinkDataHdlLink = rLink; }
	const Link&		GetGetLinkDataHdl() const { return aGetLinkDataHdlLink; }
	void			SetSetLinkDataHdl( const Link& rLink )
						{ aSetLinkDataHdlLink = rLink; }
	const Link&		GetSetLinkDataHDl() const { return aSetLinkDataHdlLink; }

	String			GetLinkData( const String& rLinkName );
	void			SetLinkData( const String& rLinkName,
								 const String& rLinkData );
	void			UpdateLinkData( const String& rLinkName,
									const String& rLinkData );
	BOOL			IsUpdatingLinkData() const
					{
						return (nFlags & VCCONT_UPDATING_LINKS) ? TRUE : FALSE;
					}
	// beim Import findet aus Geschwindigkeitsgruenden keine Ueberpruefung
	// statt, ob der Name eines eingefuegten Childs eindeutig ist. Hat
	// das Child einen Namen, so wird dieser als eindeutig angenommen,
	// anderenfalls bekommt es den Namen "Control"+nUniqueId
	void			StartImport() { nFlags |= VCCONT_IMPORTING; }
	void			StopImport() { nFlags &= (~VCCONT_IMPORTING); }
	BOOL			IsInImport() const {return (BOOL)(nFlags & VCCONT_IMPORTING)!=0;}
};


//-----------------
// class VCManager
//-----------------
class List;
class VCForm;
class SjJSbxFormsArray;

class VCManager : public VCContainer
{
protected:
	VCControl*      pCaptureChild;
	List*			pWinList;
	BOOL			bInPaint;

	List			aFormList;
public:
	TYPEINFO();
					VCManager();
					~VCManager();

	void			PaintControl( VCControl* pCtrl, BOOL bPaintNow=FALSE, Window* pNotDrawWin=NULL );
	void			Paint( Window* pWindow, const Rectangle& rRect );
	void			MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt );
	void			MouseButtonDown( Window* pWindow, VCControl* pCtrl, const MouseEvent& rMEvt );
	void    		MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt );
	void    		MouseMove( Window* pWindow, const MouseEvent& rMEvt );
	void    		KeyInput( Window* pWindow, const KeyEvent& rKEvt );
	void			LoseFocus( Window* pWindow );

	void			AddWindow( Window* pWindow );
	void			RemoveWindow( Window* pWindow );


	VCForm* 		GetVCForm( const VCControl* );
	VCForm* 		InsertForm( const String& rURL, USHORT nMethod, const String& rEncryption, String* pName = 0 );
	void			InsertForm( VCForm* );
	void			RemoveAndDeleteForm( VCForm* );
	ULONG			FormCount() const { return aFormList.Count(); }
	VCForm*			GetFormByPos( ULONG nForm ) const { return (VCForm*)aFormList.GetObject(nForm); }
	VCForm*			GetFormByName( const String& rName ) const;

	// erzeugt eine SfxFrame-Liste mit den Frames der aktuellen SfxViewFrame
	static List*	CreateFrameList();
	void			ReleaseCaptureChild() { pCaptureChild = 0; }
};

#endif //_VCONT_HXX
