/*************************************************************************
 *
 *  $RCSfile: edctrl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:38 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/



#define ITEMID_FONT			0
#define ITEMID_POSTURE      0
#define ITEMID_WEIGHT       0
#define ITEMID_FONTHEIGHT   0
#define ITEMID_UNDERLINE    0
#define ITEMID_CROSSEDOUT   0
#define ITEMID_SHADOWED     0
#define ITEMID_CONTOUR      0
#define ITEMID_COLOR        0

//#include <sv.hxx>

#ifndef _ITEMSET_HXX
#include <svtools/itemset.hxx>
#endif

#ifndef _EEITEM_HXX //autogen
#include <svx/eeitem.hxx>
#endif
#ifndef _SV_APP_HXX
#include <vcl/svapp.hxx>
#endif

#ifndef _SFXITEMPOOL_HXX
#include <svtools/itempool.hxx>
#endif
#pragma hdrstop

#include <svx/editview.hxx>
#include "edctrl.hxx"
#include <svx/editeng.hxx>
#ifndef _ISETBRW_HXX
#include "isetbrw.hxx"
#endif
#include <svx/fontitem.hxx>
#include <svx/postitem.hxx>
#include <svx/wghtitem.hxx>
#include <svx/fhgtitem.hxx>
#include <svx/udlnitem.hxx>
#include <svx/crsditem.hxx>
#include <svx/shdditem.hxx>
#include <svx/cntritem.hxx>
#include <svx/colritem.hxx>

ImplEdCtrl::ImplEdCtrl( ItemSetBrowser* pParent, WinBits nWinStyle ) :
		Control( pParent, nWinStyle )
{
	pItemPool = EditEngine::CreatePool();
	SetEditEngineDefaults( pParent->GetValueFont() );
	pEditEngine = new EditEngine( pItemPool );
	pEditEngine->SetRefMapMode( MAP_PIXEL );
	pEditView = new EditView( pEditEngine, this );
	pEditEngine->InsertView( pEditView );
	if ( !pParent->IsSVLook() )
	{
		pEditView->SetBackgroundColor( pParent->GetBackground().GetColor() );
		SetBackground( pParent->GetBackground() );
	}
	else
	{
		Color aColor( COL_WHITE );
		pEditView->SetBackgroundColor( aColor );
		SetBackground( Wallpaper( aColor ) );
	}

	nMaxLines = 1;
	bClosing = FALSE;
	bCancel = FALSE;
	bRdOnly = FALSE;
}

ImplEdCtrl::~ImplEdCtrl()
{
	pEditEngine->RemoveView( pEditView );
	delete pEditView;
	delete pEditEngine;
	pEditView = 0;
	pEditEngine = 0;
	delete pItemPool;
}

void ImplEdCtrl::SetEditEngineDefaults( const Font& rFont )
{
	SfxItemSet aSet( *pItemPool, EE_ITEMS_START, EE_ITEMS_END );

	SvxFontItem aFontItem( rFont.GetFamily(),
		rFont.GetName(),
		String(),
		rFont.GetPitch(),
		rFont.GetCharSet(), EE_CHAR_FONTINFO );
	aSet.Put( aFontItem );

	aSet.Put( SvxFontHeightItem( rFont.GetSize().Height(), 100, EE_CHAR_FONTHEIGHT )  );
	aSet.Put( SvxShadowedItem( rFont.IsShadow(), EE_CHAR_SHADOW )  );
	aSet.Put( SvxWeightItem( rFont.GetWeight(), EE_CHAR_WEIGHT )  );
	aSet.Put( SvxColorItem( rFont.GetColor(), EE_CHAR_COLOR )  );
	aSet.Put( SvxUnderlineItem( rFont.GetUnderline(), EE_CHAR_UNDERLINE )  );
	aSet.Put( SvxCrossedOutItem( rFont.GetStrikeout(), EE_CHAR_STRIKEOUT )  );
	aSet.Put( SvxPostureItem( rFont.GetItalic(), EE_CHAR_ITALIC )  );
	aSet.Put( SvxContourItem( rFont.IsOutline(), EE_CHAR_OUTLINE )  );

	for ( USHORT nWhich = EE_ITEMS_START; nWhich <= EE_ITEMS_END; nWhich++ )
	{
		const SfxPoolItem* pItem;
		SfxItemState eState = aSet.GetItemState( nWhich, TRUE, &pItem );
		if (eState == SFX_ITEM_SET)
			pItemPool->SetPoolDefaultItem( *pItem );
		else
			pItemPool->ResetPoolDefaultItem( nWhich );
	}
}

void ImplEdCtrl::SetText( const String& rText )
{
	pEditEngine->SetText( rText );
	ESelection aSel;
	pEditView->SetSelection( aSel );
}

void ImplEdCtrl::SetText( SfxStringListItem* )
{
/*

*/
}

SfxStringListItem* ImplEdCtrl::CreateListItem()
{
	return 0;
}

String ImplEdCtrl::GetText()
{
	return pEditEngine->GetText();
}

void __EXPORT ImplEdCtrl::MouseButtonDown( const MouseEvent& rMEvt )
{
	pEditView->MouseButtonDown( rMEvt );
}

void __EXPORT ImplEdCtrl::MouseButtonUp( const MouseEvent& rMEvt )
{
	pEditView->MouseButtonUp( rMEvt );
}

void __EXPORT ImplEdCtrl::MouseMove( const MouseEvent& rMEvt )
{
	pEditView->MouseMove( rMEvt );
}

void __EXPORT ImplEdCtrl::Resize()
{
	ItemSetBrowser* pParent = (ItemSetBrowser*)GetParent();
	Size aTextSize( pParent->GetTextWidth('X'), pParent->GetTextHeight() );
	long nEntryHeight	= pParent->GetEntryHeight();
	long nEdCtrlOffs	= (nEntryHeight - aTextSize.Height()) / 2;
	Point aPos;
	aPos.Y() += nEdCtrlOffs;
#ifdef OS2
	aPos.Y()--;
#endif
	aPos.X() += 2;

	aTextSize.Width() = GetSizePixel().Width();
	aTextSize.Height() = nEntryHeight-nEdCtrlOffs-2;

	pEditView->SetOutputArea( Rectangle( aPos, aTextSize ) );

	Control::Resize();
}

void __EXPORT ImplEdCtrl::KeyInput( const KeyEvent& rKEvt )
{
	BOOL bKeyUsed = TRUE;
	aKeyAction = rKEvt.GetKeyCode();
	switch ( aKeyAction.GetCode() )
	{
		case KEY_RETURN:
			if ( !aKeyAction.IsShift() )
				Close();
			break;

		case KEY_ESCAPE:
			if ( !aKeyAction.IsShift() )
				Close( TRUE );
			break;

		case KEY_UP:
			// aKeyActionHdl.Call( this );
			Close();
			break;

		case KEY_DOWN:
			// aKeyActionHdl.Call( this );
			Close();
			break;

		case KEY_TAB:
			break;

		default:
			bKeyUsed = FALSE;
	}

	if ( !bKeyUsed && !bRdOnly )
		pEditView->PostKeyEvent( rKEvt );
}

void __EXPORT ImplEdCtrl::LoseFocus()
{
	Control::LoseFocus();
	if( (!Application::GetFocusWindow()) ||
		 !IsChild( Application::GetFocusWindow()))
	{
		Close();
	}
}

void __EXPORT ImplEdCtrl::GetFocus()
{
	Control::GetFocus();
	pEditView->ShowCursor();
}

void __EXPORT ImplEdCtrl::Paint( const Rectangle& rRect )
{
	pEditView->Paint( rRect );
}

void ImplEdCtrl::Close( BOOL bCanceled )
{
	if ( bClosing )
		return;
	bClosing = TRUE;
	bCancel = bCanceled;
	aClosedHdl.Call( this );
}

/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/isetbrw/edctrl.cxx,v 1.1.1.1 2000/09/29 11:02:38 mh Exp $

	  Source Code Control System - Update

	  $Log: edctrl.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:38  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:44  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:42  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.26  2000/08/22 13:18:32  dl
	  Outliner changes
	
	  Revision 1.25  2000/05/11 08:29:05  pw
	  removed old SV-stuff

	  Revision 1.24  2000/05/09 13:49:53  pw
	  UNICODE

	  Revision 1.23  2000/02/10 19:57:17  hr
	  #70473# changes for unicode ( patched by automated patchtool )

	  Revision 1.22  1998/02/27 16:41:32  OV
	  LoseFocus-Bugfix


	  Rev 1.21   27 Feb 1998 17:41:32   OV
   LoseFocus-Bugfix

	  Rev 1.20   22 Jan 1998 10:18:04   TJ
   chg: header

	  Rev 1.19   21 Nov 1997 17:35:02   OV
   Headerumstellung

	  Rev 1.18   12 Sep 1997 11:49:40   OS
   ITEMID_* definiert

	  Rev 1.17   09 Aug 1997 11:21:26   OS
   frmitems/paraitem/textitem aufgeteilt

	  Rev 1.16   16 Jul 1997 17:31:28   TRI
   includes

	  Rev 1.15   16 Jul 1997 16:29:54   MH
   add: header

	  Rev 1.14   21 Mar 1996 11:03:22   OV
   Clooks

	  Rev 1.13   08 Nov 1995 17:56:54   TRI
   301 Anpassung

	  Rev 1.12   02 Nov 1995 08:57:18   OV
   Anpassungen EditEngine,TreeListBox

	  Rev 1.11   13 Oct 1995 16:24:40   OV
   SetText: Selection setzen; Kosmetik fuer OS/2

	  Rev 1.10   12 Oct 1995 13:10:38   OV
   Kosmetik

	  Rev 1.9   25 Aug 1995 16:30:24   HJS
   header umstellung

	  Rev 1.8   02 Aug 1995 18:49:04   OV
   Pragmas

	  Rev 1.7   11 Jul 1995 19:03:38   OV
   Bugfixes

	  Rev 1.6   05 Jul 1995 13:44:44   OV
   Segmentitis

	  Rev 1.5   03 May 1995 19:37:10   MH
   editview

	  Rev 1.4   17 Mar 1995 16:31:26   OV
   TAB abfangen

	  Rev 1.0   27 Feb 1995 10:29:18   OV
   Initial revision.

*************************************************************************/

#pragma SEG_EOFMODULE

