/*************************************************************************
 *
 *  $RCSfile: lbctrl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:38 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SFXENUMITEM_HXX
#include <svtools/eitem.hxx>
#endif
#ifndef _SFXPOOLITEM_HXX
#include <svtools/poolitem.hxx>
#endif
#ifndef _SFXSLSTITM_HXX
#include <svtools/slstitm.hxx>
#endif
#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif


#pragma hdrstop

#include "lbctrl.hxx"
#include "isetbrw.hxx"

ImplListBoxCtrl::ImplListBoxCtrl( ItemSetBrowser* pPar,WinBits nWinStyle)
	: ListBox( pPar, nWinStyle ),
	pParent(pPar)
{
	bClosing = FALSE;
	bCancel = FALSE;
	SetFont( pParent->GetValueFont() );
}

ImplListBoxCtrl::~ImplListBoxCtrl()
{
}

void ImplListBoxCtrl::Close( BOOL bCanceled )
{
	if ( bClosing )
		return;
	bClosing = TRUE;
	bCancel = bCanceled;
	aClosedHdl.Call( this );
}

void __EXPORT ImplListBoxCtrl::KeyInput( const KeyEvent& rKEvt )
{
	BOOL bKeyUsed = TRUE;
	aKeyAction = rKEvt.GetKeyCode();
	switch ( aKeyAction.GetCode() )
	{
		case KEY_RETURN:
			if ( !aKeyAction.IsShift() && !aKeyAction.IsMod1() )
				Close();
			else
				bKeyUsed = FALSE;
			break;

		case KEY_ESCAPE:
			if ( !aKeyAction.IsShift() )
				Close( TRUE );
			break;

		default:
			bKeyUsed = FALSE;
	}
	if ( !bKeyUsed )
		ListBox::KeyInput( rKEvt );
}

void __EXPORT ImplListBoxCtrl::LoseFocus()
{
	ListBox::LoseFocus();
#ifdef VCL
	if( !Application::GetFocusWindow() || !IsChild( Application::GetFocusWindow() ))
#endif
		Close( FALSE );
}

void ImplListBoxCtrl::SetText( const SfxPoolItem* pItem )
{
	DBG_ASSERT(pItem,"SetText:No Item")
	Clear();
	if ( pItem )
	{
		if ( pItem->ISA(SfxStringListItem) )
			SetText( (const SfxStringListItem&)*pItem );
		else if ( pItem->ISA(SfxBoolItem) )
			SetText( (const SfxBoolItem&)*pItem );
		else if ( pItem->ISA(SfxEnumItem) )
			SetText( (const SfxEnumItem&)*pItem );
		else
		{
			String aText( pParent->GetItemDropDownData( pItem->Which() ));
			USHORT nTokenCount = aText.GetTokenCount( '\n' );
			for( USHORT nCur=0; nCur< nTokenCount; nCur++ )
				InsertEntry( aText.GetToken(nCur,'\n'),LISTBOX_APPEND);
		}
	}
}

void ImplListBoxCtrl::SetText( const SfxStringListItem& rItem )
{
	SfxStringListItem* pItem =  &((SfxStringListItem&)rItem);
	List* pList = pItem->GetList();
	String* pStr = (String*)(pList->First());
	while( pStr )
	{
		InsertEntry( *pStr, LISTBOX_APPEND );
		pStr = (String*)(pList->Next());
	}
}

void ImplListBoxCtrl::SetText( const SfxBoolItem& rItem )
{
	SfxBoolItem aItem( rItem );
	aItem.SetValue( FALSE );
	String aStr;
	aItem.GetPresentation( SFX_ITEM_PRESENTATION_COMPLETE,
		SFX_MAPUNIT_TWIP, SFX_MAPUNIT_TWIP, aStr );
	InsertEntry( aStr );
	aItem.SetValue( TRUE );
	aItem.GetPresentation( SFX_ITEM_PRESENTATION_COMPLETE,
		SFX_MAPUNIT_TWIP, SFX_MAPUNIT_TWIP, aStr );
	InsertEntry( aStr );
	SelectEntryPos( (USHORT)rItem.GetValue() );
}

void ImplListBoxCtrl::SetText( const SfxEnumItem& rItem )
{
	USHORT nValues = rItem.GetValueCount();
	for( USHORT i = 0; i < nValues; i++ )
		InsertEntry( rItem.GetValueTextByPos( i ) );

	SelectEntryPos( (USHORT)rItem.GetValue() );
}


/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/isetbrw/lbctrl.cxx,v 1.1.1.1 2000/09/29 11:02:38 mh Exp $

	  Source Code Control System - Update

	  $Log: lbctrl.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:38  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:44  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:46  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.16  2000/02/10 19:57:27  hr
	  #70473# changes for unicode ( patched by automated patchtool )
	
	  Revision 1.15  1997/11/21 16:35:00  OV
	  Headerumstellung
	

	  Rev 1.14   21 Nov 1997 17:35:00   OV
   Headerumstellung

	  Rev 1.13   06 Nov 1997 12:50:48   OV
   GetFocusWindow==0 abfragen; Includes aufgeraeumt

	  Rev 1.12   22 Sep 1997 11:48:26   OV
   #44027# LoseFocus/VCL: wenn kein IsChildWin dann Close

	  Rev 1.11   05 Jun 1996 12:09:36   OV
   DropDown ComboBoxen

	  Rev 1.10   21 Mar 1996 11:03:20   OV
   Clooks

	  Rev 1.9   05 Aug 1995 12:42:22   CL
   Handling verbessert

	  Rev 1.8   02 Aug 1995 18:48:36   OV
   Pragmas

	  Rev 1.7   11 Jul 1995 19:03:44   OV
   Bugfixes

	  Rev 1.6   05 Jul 1995 13:44:48   OV
   Segmentitis

	  Rev 1.5   28 Jun 1995 15:22:30   TRI
   250b Anpassung

	  Rev 1.4   28 Jun 1995 15:20:34   OV
   Sfx

	  Rev 1.0   24 Feb 1995 18:39:16   OV
   Initial revision.

*************************************************************************/


#pragma SEG_EOFMODULE

