/*************************************************************************
 *
 *  $RCSfile: medctrl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:38 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif
#ifndef _SFXSTRITEM_HXX
#include <svtools/stritem.hxx>
#endif
#ifndef _SFXENUMITEM_HXX
#include <svtools/eitem.hxx>
#endif
#ifndef _SFXSLSTITM_HXX
#include <svtools/slstitm.hxx>
#endif

#pragma hdrstop

#include "medctrl.hxx"
#ifndef _ISETBRW_HXX
#include "isetbrw.hxx"
#endif

ImplMultiLineEdCtrl::ImplMultiLineEdCtrl( ItemSetBrowser* pParent,
	WinBits nWinStyle) : MultiLineEdit( pParent, nWinStyle )
{
	nMaxLines = 0xffff;
	bClosing = FALSE;
	bCancel = FALSE;
	bRdOnly = FALSE;
	Font aFont( pParent->GetValueFont() );
	Color aColor( GetBackgroundBrush().GetColor() );
	aFont.SetFillColor( aColor );
	SetFont( aFont );
}

ImplMultiLineEdCtrl::~ImplMultiLineEdCtrl()
{
}

void ImplMultiLineEdCtrl::Close( BOOL bCanceled )
{
	if ( bClosing )
		return;
	bClosing = TRUE;
	bCancel = bCanceled;
	aClosedHdl.Call( this );
}

void ImplMultiLineEdCtrl::SetText( const SfxPoolItem* pItem )
{
	DBG_ASSERT(pItem,"SetText:No Item")
	if ( pItem )
	{
		if ( pItem->ISA(SfxStringListItem) )
			SetText( (const SfxStringListItem&)*pItem );
		else
		{
			String aStr;
			pItem->GetPresentation(
				SFX_ITEM_PRESENTATION_COMPLETE,
				SFX_MAPUNIT_TWIP, SFX_MAPUNIT_TWIP, aStr );
			MultiLineEdit::SetText( aStr ); // pItem->GetValueText() );
		}
	}
}

void ImplMultiLineEdCtrl::SetText( const SfxStringListItem& rItem )
{
	SfxStringListItem* pItem =  &((SfxStringListItem&)rItem);
	String aStr = pItem->GetString();
	MultiLineEdit::SetText( aStr );
}

SfxStringListItem* ImplMultiLineEdCtrl::CreateListItem()
{
	return 0;
}

BOOL ImplMultiLineEdCtrl::HandleKeyEvent( const KeyEvent& rKEvt )
{
	BOOL bHandled = TRUE;
	aKeyAction = rKEvt.GetKeyCode();
	switch ( aKeyAction.GetCode() )
	{
		case KEY_RETURN:
			if ( !aKeyAction.IsShift() && !aKeyAction.IsMod1() )
				Close();
			else
				bHandled = FALSE;
			break;

		case KEY_ESCAPE:
			if ( !aKeyAction.IsShift() )
				Close( TRUE );
			break;

		case KEY_TAB:
			break;

		default:
			bHandled = FALSE;
	}
	return bHandled;
}


void __EXPORT ImplMultiLineEdCtrl::KeyInput( const KeyEvent& rKEvt )
{
	if ( !HandleKeyEvent(rKEvt) )
		MultiLineEdit::KeyInput( rKEvt );
}

#ifdef VCL
long ImplMultiLineEdCtrl::PreNotify( NotifyEvent& rNEvt )
{
	if( rNEvt.GetType() == EVENT_KEYINPUT )
		return (long)HandleKeyEvent(*rNEvt.GetKeyEvent() );
	return 0; //not handled
}
#endif

void __EXPORT ImplMultiLineEdCtrl::LoseFocus()
{
	MultiLineEdit::LoseFocus();
#ifdef VCL
	if( !IsChild( Application::GetFocusWindow() ))
#endif
		Close( FALSE );
}


/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/isetbrw/medctrl.cxx,v 1.1.1.1 2000/09/29 11:02:38 mh Exp $

	  Source Code Control System - Update

	  $Log: medctrl.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:38  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:44  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:47  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.17  2000/02/10 19:57:37  hr
	  #70473# changes for unicode ( patched by automated patchtool )
	
	  Revision 1.16  1997/11/21 16:35:00  OV
	  Headerumstellung
	

	  Rev 1.15   21 Nov 1997 17:35:00   OV
   Headerumstellung

	  Rev 1.14   17 Sep 1997 11:17:50   OV
   #43866# VCL: PreNotify ueberladen

	  Rev 1.13   17 Sep 1997 10:44:46   OV
   #43866# VCL: Focushandling

	  Rev 1.12   29 May 1996 12:06:08   OV
   Bugfixes, Erweiterungen

	  Rev 1.11   21 Mar 1996 11:03:18   OV
   Clooks

	  Rev 1.10   08 Nov 1995 17:56:32   TRI
   301 Anpassung

	  Rev 1.9   02 Aug 1995 18:47:32   OV
   Pragmas

	  Rev 1.8   14 Jul 1995 15:26:08   OV
   Bugfixes

	  Rev 1.7   11 Jul 1995 19:03:36   OV
   Bugfixes

	  Rev 1.6   05 Jul 1995 13:44:48   OV
   Segmentitis

	  Rev 1.5   28 Jun 1995 15:20:26   OV
   Sfx

	  Rev 1.4   17 Mar 1995 16:31:40   OV
   TAB abfangen

	  Rev 1.3   04 Mar 1995 18:06:46   OV
   SetText

	  Rev 1.2   02 Mar 1995 17:16:06   OV
   SetText

	  Rev 1.1   27 Feb 1995 17:05:04   OV
   Impl.

	  Rev 1.0   24 Feb 1995 18:39:08   OV
   Initial revision.

*************************************************************************/


#pragma SEG_EOFMODULE

