/*************************************************************************
 *
 *  $RCSfile: vceeng.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _VCEENG_HXX
#define _VCEENG_HXX

class Window;
class VCEditEngine;
class SfxPoolItem;
class MouseEvent;
class OutputDevice;
class KeyEvent;
class Font;
class VCScrollBarEngine;

struct ImplActiveEdit
{
	VCEditEngine*	pThis;
	Window*			pWin;
};

#ifndef _SI_NOEDITENGINES

#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _SFX_BINDINGS_HXX
#include <sfx2/bindings.hxx>
#endif
#ifndef _GEN_HXX
#include <tools/gen.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _SV_POINTR_HXX
#include <vcl/pointr.hxx>
#endif
#ifndef _SV_CURSOR_HXX
#include <vcl/cursor.hxx>
#endif

#include "vcengin.hxx"

class SiEditSlotInterceptor : public SfxSlotInterceptor
{
	Link			aCallBackHdl;
public:

					SiEditSlotInterceptor( const Link& rCallBackHdl );

	virtual BOOL	Execute( USHORT nSID, SfxPoolItem**ppArgs );
	virtual 		SfxItemState	QueryState( USHORT nSID, SfxPoolItem* &rpState);
};

class VCEditMapper
{
public:
	virtual void SetText( const String& rText ) = 0;
	virtual String GetText() = 0;
	virtual void SetSelectStart( long nStart ) = 0;
	virtual long GetSelectStart() = 0;
	virtual void SetSelectLen( long nLen ) = 0;
	virtual long GetSelectLen() const = 0;
	virtual String GetSelectText() const = 0;
	virtual void SetSelectText( const String& rText ) = 0;
	virtual void Paint( OutputDevice* pOut ) = 0;
	virtual void MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt ) = 0;
	virtual void MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt ) = 0;
	virtual void MouseMove( Window* pWindow, const MouseEvent& rMEvt ) = 0;
	virtual void KeyInput( Window* pWindow, const KeyEvent& rKEvt ) = 0;
	virtual void PrepareClose() = 0;
	virtual Window* GetEditWin() const = 0;
	virtual void Resize() = 0;
	virtual void Enable( BOOL bEnable ) = 0;
	virtual void SetFont( const Font& rFont ) = 0;
	virtual Size GetOptSize( OutputDevice* ) const = 0;
	virtual Size GetOptSize( const Size& rCharXY, OutputDevice* pOut ) const = 0;
	virtual Size GetCharCountXY( OutputDevice* pOut) const = 0;
	virtual void Focus( Window* pWin ) = 0;
	virtual void Blur() = 0;
	virtual BOOL IsModified() const = 0;
	virtual void SetModifyFlag( BOOL bModified ) = 0;
	virtual void SetWordWrap( USHORT nMode ) = 0;

	VCEditMapper();
	virtual ~VCEditMapper();
	virtual Pointer GetPointer( const Point& ) const = 0;
};


class VCSingleEditEngine : public VCEditMapper
{
	friend class VCSingleEditWin;
	void			Cut();
	void			Copy();
	void			Paste();

	void			InitEditWinPosSizeMap();

protected:
	VCEditEngine*	pOwner;
	Window*   		pEditWin;

	String			aText;
	USHORT			nMaxLen;
	Selection		aSelection;
	Cursor			aCursor;
	USHORT			nCursorPos;
	USHORT 			nAnchorPos;
	long			nTextOfsX;
	String			aPassWordChar;
	Timer			aDestroyEditWinTimer;

	BOOL			bCloseEditWin;
	BOOL			bDrag;
	BOOL			bModified;

	void			NotifyCloseEditWindow();
	void 			DrawBorder( OutputDevice* pOut );
	void			DrawBackground( OutputDevice* pOut );
	Rectangle		GetTextRect( OutputDevice* pOut ) const;
	void			DrawText( OutputDevice* pOut );
	void			DrawSelection( OutputDevice* pOut );
	void			ShowEditWin( Window* pWin );
	String			CutLineFromStart( OutputDevice* pOut, String* pStr, long nMaxWidth );
	void			UpdateCursorPos( Window* pWin );
	void			ScrollToCursor( Window* pWindow );
	long			GetStrPosFromMouse( Window* pWin, const Point& rPt );
	void			TextModified( Window* );
	void			Cleanup( BOOL bAsyncSelfDestroyOfEditWin = TRUE );

	DECL_LINK(DestroyEditWin,Timer*);

public:
	VCSingleEditEngine( VCEditEngine* pOwner );
	~VCSingleEditEngine();

	virtual void SetText( const String& rText );
	virtual String GetText();
	virtual void SetSelectStart( long nStart );
	virtual long GetSelectStart();
	virtual void SetSelectLen( long nLen );
	virtual long GetSelectLen() const;
	virtual String GetSelectText() const;
	virtual void SetSelectText( const String& rText );
	virtual void Paint( OutputDevice* pOut );
	virtual void MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt );
	virtual void MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt );
	virtual void MouseMove( Window* pWindow, const MouseEvent& rMEvt );
	virtual void KeyInput( Window* pWindow, const KeyEvent& rKEvt );
	virtual void PrepareClose();
	virtual Window* GetEditWin() const;
	virtual void Resize();
	virtual void Enable( BOOL bEnable );
	virtual void SetFont( const Font& rFont );
	virtual Size GetOptSize( OutputDevice* ) const;
	virtual Size GetOptSize( const Size& rCharXY, OutputDevice* pOut ) const;
	virtual Size GetCharCountXY( OutputDevice* pOut) const;
	virtual void Focus( Window* pWin );
	virtual void Blur();
	virtual BOOL IsModified() const;
	virtual void SetModifyFlag( BOOL bModified );
	virtual void SetWordWrap( USHORT nMode );
	virtual Pointer GetPointer( const Point& ) const;
};



class EditEngine;
class EditView;
class EditStatus;

class VCMultiEditEngine : public VCEditMapper
{
	friend class VCMultiEditWin;
	void			Cut();
	void			Copy();
	void			Paste();

	void InitEditWinPosSizeMap();
	void PositionScrollBars( OutputDevice* );
	void SetHSBarRange( EditEngine* );
	void SetVSBarRange( EditEngine* );
	void SetLinePageSizes( EditEngine* );

	DECL_LINK(HorScrollHdl,VCScrollBarEngine*);
	DECL_LINK(VerScrollHdl,VCScrollBarEngine*);
	DECL_LINK(StatusEventHdl,EditStatus*);
	DECL_LINK(TimeoutHdl,Timer*);
	DECL_LINK(DestroyEditWin,Timer*);

	VCScrollBarEngine 	aVScroll;
	VCScrollBarEngine 	aHScroll;
	Timer				aCleanupTimer;
	Rectangle			aClientRect;	// in Doc-Koord.
	Rectangle			aResizeRect;	// -- " --
	Rectangle			aHSBarRect;
	Rectangle			aVSBarRect;
	Rectangle			aDocVisArea;	// Vis-Area EditView
	long				nSelStart;
	long				nSelLen;
	Point				aOldVScrollPos;
	Point				aOldHScrollPos;
	Timer				aDestroyEditWinTimer;
	USHORT				nFlags;

	USHORT				nCursorPara;
	USHORT				nCursorPos;

	BOOL				bModified;

protected:
	VCEditEngine*	pOwner;
	Window*   		pEditWin;
	EditEngine*		pEditEngine;	// temporaere Editier/Scroll-Engine
	EditView*		pEditView;		// temporaere Editier/Scroll-View

	Point			aEditPos;
	Size			aEditSize;
	String			aText;
	BOOL			bCloseEditWin;

	void			NotifyCloseEditWindow();
	void 			DrawBorder( OutputDevice* pOut );
	void			DrawBackground( OutputDevice* pOut );
	void 			DrawResizeRect( OutputDevice* pOut );
	Rectangle		GetTextRect( OutputDevice* pOut ) const;
	void			ShowEditWin( Window* pWin );
	void			SetupMultiEdit( OutputDevice* pOut, BOOL bPrint );
	void			Cleanup( BOOL bAsyncSelfDestroyOfEditWin = TRUE );

public:
	VCMultiEditEngine( VCEditEngine* pOwner );
	~VCMultiEditEngine();

	virtual void SetText( const String& rText );
	virtual String GetText();
	virtual void SetSelectStart( long nStart );
	virtual long GetSelectStart();
	virtual void SetSelectLen( long nLen );
	virtual long GetSelectLen() const;
	virtual String GetSelectText() const;
	virtual void SetSelectText( const String& rText );
	virtual void Paint( OutputDevice* pOut );
	virtual void MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt );
	virtual void MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt );
	virtual void MouseMove( Window* pWindow, const MouseEvent& rMEvt );
	virtual void KeyInput( Window* pWindow, const KeyEvent& rKEvt );
	virtual void PrepareClose();
	virtual Window* GetEditWin() const;
	virtual void Resize();
	virtual void Enable( BOOL bEnable );
	virtual void SetFont( const Font& rFont );
	virtual Size GetOptSize( OutputDevice* ) const;
	virtual Size GetOptSize( const Size& rCharXY, OutputDevice* pOut ) const;
	virtual Size GetCharCountXY( OutputDevice* pOut) const;
	virtual void Focus( Window* pWin );
	virtual void Blur();
	virtual BOOL IsModified() const;
	virtual void SetModifyFlag( BOOL bModified );
	virtual void SetWordWrap( USHORT nMode );
	virtual Pointer GetPointer( const Point& ) const;
};

#endif //  _SI_NOEDITENGINES

#endif



