/*************************************************************************
 *
 *  $RCSfile: vcengin.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_SOUND_HXX
#include <vcl/sound.hxx>
#endif

#pragma hdrstop

#define _VCSPINENG_CXX
#define _VCSCROLLENG_CXX
#include "vcengin.hxx"
#include "vctrls.hxx"
#include "vcont.hxx"

IMPL_LINK( VCSpinButtonEngine, Timeout, Timer *, EMPTYARG )
{
	if( !pActWin )
		return 0;

#ifdef VCL
	// Man kann nur einen SpinButton zur Zeit betaetigen
	static BOOL bInCall = FALSE;

	// Timer muss weiterlaufen, damit wir erkennen koennen,
	// ob im Handler das Window disabled wurde
	aTimer.SetTimeout( 50 );
	aTimer.Start();

	// Hat jemand im Spin-Handler einen Dialog hochgezogen?
	if( !pActWin->IsEnabled() || !pActWin->IsInputEnabled() )
	{
		if( bDnIn )
		{
			bDnIn = FALSE;
			DrawDnButton( pActWin );
		}
		else if( bUpIn )
		{
			bUpIn = FALSE;
			DrawUpButton( pActWin );
		}
		bUpHit = bDnHit = FALSE;
		aTimer.Stop();
		pActWin->ReleaseMouse();
		pActWin = NULL;
		((VCManager*)pShell->GetParent())->ReleaseCaptureChild();
		return 0;
	}
	if( bInCall )
		return 0;
#endif


	BOOL bCall=FALSE;

	if( bDnIn )
	{
		if( nAct > nMin )
		{
			SetAct( nAct-nStep );
			bCall = TRUE;
		}
	}
	else
	if( bUpIn )
	{
		if( nAct < nMax )
		{
			SetAct( nAct+nStep );
			bCall = TRUE;
		}
	}

	if( bCall )
	{
#ifdef VCL
		bInCall = TRUE;
#endif
		ULONG nErr = 0;
		if( bUpIn && aSpinUpLink.IsSet() )
			nErr = aSpinUpLink.Call( this );
		if( bDnIn && aSpinDownLink.IsSet() )
			nErr = aSpinDownLink.Call( this );
		if( !nErr )
		{
#ifndef VCL
			aTimer.SetTimeout( 50 );
			aTimer.Start();
#endif
		}
		else
			aTimer.Stop();
	}
#ifdef VCL
	bInCall = FALSE;
#endif
	return 0;
}

Rectangle VCSpinButtonEngine::GetUpRect( OutputDevice* pOut )
{
	Rectangle aUpRect( aPos, aSize );

	aUpRect.Bottom()= aUpRect.Top() + aSize.Height()/2;
	return aUpRect;
}

Rectangle VCSpinButtonEngine::GetDnRect( OutputDevice* pOut )
{
	Rectangle aDnRect( aPos, aSize );

	aDnRect.Top()   = aDnRect.Top() + aSize.Height()/2;

	return aDnRect;
}

VCSpinButtonEngine::VCSpinButtonEngine( VCControl* _pShell ) :
	bEnable(FALSE),
	bUpIn(FALSE),
	bDnIn(FALSE),
	bUpHit(FALSE),
	bDnHit(FALSE),
	nMin(0),
	nMax(0),
	nAct(0),
	nStep(1),
	pActWin(NULL),
	bCalcSizes(TRUE),
	pShell( _pShell )
{
	aTimer.SetTimeoutHdl( LINK( this, VCSpinButtonEngine, Timeout ) );
}

VCSpinButtonEngine::~VCSpinButtonEngine()
{
}

void VCSpinButtonEngine::SetPos( const Point& rPos )
{
	aPos = rPos;

	bCalcSizes = TRUE;
}

void VCSpinButtonEngine::SetSize( const Size& rSize )
{
	aSize = rSize;

	bCalcSizes = TRUE;
}

void VCSpinButtonEngine::Enable( BOOL bFlag )
{
	bEnable = bFlag;
}

void VCSpinButtonEngine::SetMin( long nM )
{
	nMin = nM;
}

void VCSpinButtonEngine::SetMax( long nM )
{
	nMax = nM;
}

void VCSpinButtonEngine::SetAct( long nA )
{
	nAct = nA;

	if( nAct < nMin )
		nAct = nMin;
	if( nAct > nMax )
		nAct = nMax;
}

void VCSpinButtonEngine::SetStepSize( long nS )
{
	nStep = nS;
}

void VCSpinButtonEngine::Paint( OutputDevice* pOut )
{
	if( bCalcSizes )
	{
		aUpRect 	= GetUpRect( pOut );
		aDnRect 	= GetDnRect( pOut );
		bCalcSizes = FALSE;
	}

	DrawSpinButton( pOut );
}

void VCSpinButtonEngine::MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt )
{
	if( !bEnable )
		return;

	pActWin = pWindow;

	BOOL bCall = FALSE;

	Point aPos = rMEvt.GetPosPixel();
	if( aDnRect.IsInside( aPos ) )
	{
		bDnHit = TRUE;
		bDnIn  = TRUE;
		DrawDnButton( pWindow );

		if( nAct > nMin )
		{
			SetAct( nAct-nStep );
			bCall = TRUE;
		}
	}
	else
	if( aUpRect.IsInside( aPos ) )
	{
		bUpHit= TRUE;
		bUpIn = TRUE;
		DrawUpButton( pWindow );

		if( nAct < nMax )
		{
			SetAct( nAct+nStep );
			bCall = TRUE;
		}
	}

	if( bCall )
	{
#ifdef VCL
		aTimer.SetTimeout( 200 );
		aTimer.Start();
#endif
		ULONG nErr = 0;
		if( bUpHit && aSpinUpLink.IsSet() )
			nErr = aSpinUpLink.Call( this );
		if( bDnHit && aSpinDownLink.IsSet() )
			nErr = aSpinDownLink.Call( this );
		if( !nErr )
		{
#ifndef VCL
			aTimer.SetTimeout( 200 );
			aTimer.Start();
#endif
		}
		else
		{
			bDnHit = FALSE;
			bDnIn  = FALSE;
			DrawDnButton( pWindow );
			bUpHit= FALSE;
			bUpIn = FALSE;
			DrawUpButton( pWindow );
			aTimer.Stop();
		}
	}
}

void VCSpinButtonEngine::MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt )
{
	if( !bEnable )
		return;

	if( bDnIn )
	{
		bDnIn = FALSE;
		DrawDnButton( pWindow );
	}
	else
	if( bUpIn )
	{
		bUpIn = FALSE;
		DrawUpButton( pWindow );
	}

	bUpHit = bDnHit = FALSE;

	aTimer.Stop();

	pActWin = NULL;
}

void VCSpinButtonEngine::MouseMove( Window* pWindow, const MouseEvent& rMEvt )
{
	if( !bEnable )
		return;

	if( rMEvt.IsLeft() )
	{
		Point aPos = rMEvt.GetPosPixel();

		if( bDnHit )
		{
			if( aDnRect.IsInside( aPos ) )
			{
				if( !bDnIn )
				{
					bDnIn = TRUE;
					DrawDnButton( pWindow );

					aTimer.Start();
				}
			}
			else
			{
				if( bDnIn )
				{
					bDnIn = FALSE;
					DrawDnButton( pWindow );

					aTimer.Stop();
				}
			}
		}
		else
		if( bUpHit )
		{
			if( aUpRect.IsInside( aPos ) )
			{
				if( !bUpIn )
				{
					bUpIn = TRUE;
					DrawUpButton( pWindow );

					aTimer.Start();
				}
			}
			else
			{
				if( bUpIn )
				{
					bUpIn = FALSE;
					DrawUpButton( pWindow );

					aTimer.Stop();
				}
			}
		}
	}
}

void VCSpinButtonEngine::KeyInput( Window* pWin, const KeyEvent& rKEvt )
{
	KeyCode aCode = rKEvt.GetKeyCode();
	BOOL	bCall = FALSE;
	BOOL	bCallUp = FALSE;

	switch( aCode.GetCode() )
	{
		case KEY_DOWN:
		case KEY_RIGHT:
			if( nAct < nMax )
			{
				SetAct( nAct + nStep );
				bCall = TRUE;
			}
			break;

		case KEY_UP:
		case KEY_LEFT:
			if( nAct > nMin )
			{
				SetAct( nAct-nStep );
				bCallUp = TRUE;
				bCall = TRUE;
			}
			break;

		case KEY_HOME:
			if( nAct > nMin )
			{
				SetAct( nMin );
				bCall = TRUE;
			}
			break;

		case KEY_END:
			if( nAct < nMax )
			{
				SetAct( nMax );
				bCallUp = TRUE;
				bCall = TRUE;
			}
			break;
	}

	if( bCall )
	{
		if( bCallUp && aSpinUpLink.IsSet() )
			aSpinUpLink.Call( this );
		if( !bCallUp && aSpinDownLink.IsSet() )
			aSpinDownLink.Call( this );
	}
}

VCScrollBarEngine::VCScrollBarEngine( BOOL bHorizontal ) :
	bHorz(bHorizontal),
	bEnable(FALSE),
	bUpIn(FALSE),
	bDnIn(FALSE),
	bUpHit(FALSE),
	bDnHit(FALSE),
	bPgUpIn(FALSE),
	bPgDnIn(FALSE),
	bPgUpHit(FALSE),
	bPgDnHit(FALSE),
	bThumbIn(FALSE),
	bThumbHit(FALSE),
	nMin(0),
	nMax(0),
	nAct(0),
	nLineSize(1),
	nPageSize(1),
	pActWin(NULL),
	bCalcSizes(TRUE),
	bCalcThumb(TRUE),
	bInPaint(FALSE),
#ifndef VCL
	bUpDisabled(FALSE),
	bDnDisabled(FALSE)
#else
	bUpDisabled(TRUE),
	bDnDisabled(TRUE)
#endif
{
	aTimer.SetTimeoutHdl( LINK( this, VCScrollBarEngine, Timeout ) );
}

VCScrollBarEngine::~VCScrollBarEngine()
{
}

IMPL_LINK( VCScrollBarEngine, Timeout, Timer *, EMPTYARG )
{
	if( !pActWin )
		return 0;

	BOOL bCall=FALSE;
	BOOL bPaintArrows = FALSE;
	if( bDnIn )
	{
		if( nAct > nMin )
		{
			HideThumb( pActWin );
			bPaintArrows = SetAct( nAct-nLineSize );
			DrawThumb( pActWin );
			bCall = TRUE;
		}
	}
	else
	if( bUpIn )
	{
		if( nAct < nMax )
		{
			HideThumb( pActWin );
			bPaintArrows = SetAct( nAct+nLineSize );
			DrawThumb( pActWin );
			bCall = TRUE;
		}
	}
	else
	if( bPgDnIn )
	{
		if( nAct > nMin && !aThumbRect.IsInside(aMDownPos) )
		{
			bPaintArrows = SetAct( nAct-nPageSize );
			DrawThumb( pActWin );
			DrawPgDnRect( pActWin );
			DrawPgUpRect( pActWin );
			bCall = TRUE;
		}
	}
	else
	if( bPgUpIn && !aThumbRect.IsInside(aMDownPos) )
	{
		if( nAct < nMax )
		{
			bPaintArrows = SetAct( nAct+nPageSize );
			DrawThumb( pActWin );
			DrawPgUpRect( pActWin );
			DrawPgDnRect( pActWin );
			bCall = TRUE;
		}
	}

	if( bCall )
	{
		if( aScrollHdlLink.IsSet() )
			aScrollHdlLink.Call( this );
		aTimer.SetTimeout( 50 );
		aTimer.Start();
	}

#ifdef VCL
	DrawUpButton( pActWin );
	DrawDnButton( pActWin );
#endif

	return 0;
}

Rectangle VCScrollBarEngine::GetUpRect( OutputDevice* pOut )
{
	Rectangle aUpRect;

	if( !bHorz )
	{
		aUpRect.Left()  = aPos.X();
		aUpRect.Top()   = aPos.Y() + (aSize.Height() - aSize.Width())-1;
		aUpRect.Right() = aUpRect.Left()+aSize.Width()-1;
		aUpRect.Bottom()= aPos.Y()+aSize.Height()-1;
	}
	else
	{
		aUpRect.Left()  = aPos.X() + (aSize.Width()-aSize.Height()) - 1;
		aUpRect.Top()   = aPos.Y();
		aUpRect.Right() = aPos.X() + aSize.Width() - 1;
		aUpRect.Bottom()= aUpRect.Top() + aSize.Height()-1;
	}

	return aUpRect;
}

Rectangle VCScrollBarEngine::GetDnRect( OutputDevice* pOut )
{
	Rectangle aDnRect;

	if( !bHorz )
	{
		aDnRect.Left()  = aPos.X();
		aDnRect.Top()   = aPos.Y();
		aDnRect.Right() = aDnRect.Left()+aSize.Width()-1;
		aDnRect.Bottom()= aDnRect.Top()+aSize.Width()-1;
	}
	else
	{
		aDnRect.Left()  = aPos.X();
		aDnRect.Top()   = aPos.Y();
		aDnRect.Right() = aDnRect.Left() + aSize.Height() - 1;
		aDnRect.Bottom()= aDnRect.Top() + aSize.Height() - 1;
	}

	return aDnRect;
}

// !bHorz: Unter dem Thumb
Rectangle VCScrollBarEngine::GetPgUpRect( OutputDevice* pOut )
{
	if( bCalcThumb )
	{
		aThumbRect = GetThumbRect( pOut );
		bCalcThumb = FALSE;
	}

	if( aThumbRect.IsEmpty() )
		return Rectangle();

	Size aLogPix = pOut->PixelToLogic( Size( 1, 1 ) );

	Rectangle aPgUpRect;

	if( !bHorz )
	{
		if( (aUpRect.Top() - aThumbRect.Bottom()) < 2*aLogPix.Height() )
			return Rectangle();
		aPgUpRect = Rectangle( aThumbRect.BottomLeft(), aUpRect.TopRight() );
	}
	else
	{
		if( (aUpRect.Left() - aThumbRect.Right()) < 2*aLogPix.Width() )
			return Rectangle();
		aPgUpRect = Rectangle( aThumbRect.TopRight(), aUpRect.BottomLeft() );
	}

#ifndef VCL
	aPgUpRect.Left()  += aLogPix.Width();
	aPgUpRect.Right() -= aLogPix.Width();
	aPgUpRect.Top()   += aLogPix.Height();
	aPgUpRect.Bottom()-= aLogPix.Height();
#else
	if( bHorz )
	{
		aPgUpRect.Left()  += aLogPix.Width();
		aPgUpRect.Right() -= aLogPix.Width();
	}
	else
	{
		aPgUpRect.Top()   += aLogPix.Height();
		aPgUpRect.Bottom()-= aLogPix.Height();
	}
#endif

	return aPgUpRect;
}

// !bHorz: Ueber dem Thumb
Rectangle VCScrollBarEngine::GetPgDnRect( OutputDevice* pOut )
{
	if( bCalcThumb )
	{
		aThumbRect = GetThumbRect( pOut );
		bCalcThumb = FALSE;
	}

	if( aThumbRect.IsEmpty() )
		return Rectangle();

	Size aLogPix = pOut->PixelToLogic( Size( 1, 1 ) );
	Rectangle aPgDnRect;

	if( !bHorz )
	{
		if( (aThumbRect.Top() - aDnRect.Bottom()) < (2*aLogPix.Height()) )
			return Rectangle();
		aPgDnRect = Rectangle( aDnRect.BottomLeft(), aThumbRect.TopRight() );
	}
	else
	{
		if( (aThumbRect.Left() - aDnRect.Right()) < (2*aLogPix.Width() ) )
			return Rectangle();
		aPgDnRect = Rectangle( aDnRect.TopRight(), aThumbRect.BottomLeft() );
	}

#ifndef VCL
	aPgDnRect.Left()  += aLogPix.Width();
	aPgDnRect.Right() -= aLogPix.Width();
	aPgDnRect.Top()   += aLogPix.Height();
	aPgDnRect.Bottom()-= aLogPix.Height();
#else
	if( bHorz )
	{
		aPgDnRect.Left()  += aLogPix.Width();
		aPgDnRect.Right() -= aLogPix.Width();
	}
	else
	{
		aPgDnRect.Top()   += aLogPix.Height();
		aPgDnRect.Bottom()-= aLogPix.Height();
	}
#endif

	return aPgDnRect;
}

void VCScrollBarEngine::SetPos( const Point& rPos )
{
	aPos = rPos;

	bCalcSizes = TRUE;
	bCalcThumb = TRUE;
}

void VCScrollBarEngine::SetSize( const Size& rSize )
{
	aSize = rSize;

	bCalcSizes = TRUE;
	bCalcThumb = TRUE;
}

void VCScrollBarEngine::Enable( BOOL bFlag )
{
	if( bEnable != bFlag ) {
		bEnable = bFlag;
		bCalcThumb = TRUE;
	}
}

void VCScrollBarEngine::SetMin( long nM )
{
	if( nMin == nM )
		return;

	nMin = nM;
	if( nAct < nMin )
		nAct = nMin;
	bCalcThumb = TRUE;
	DisableUpDnButton();
}

void VCScrollBarEngine::SetMax( long nM )
{
	if( nMax == nM )
		return;

	nMax = nM;
	if( nAct > nMax )
	{
		nAct = nMax;
	}
	bCalcThumb = TRUE;
	DisableUpDnButton();
}

BOOL VCScrollBarEngine::DisableUpDnButton()
{
	BOOL bChanged = FALSE;
	if( nAct == nMin )
	{
		if( !bDnDisabled )
		{
			bDnDisabled = TRUE;
			bChanged = TRUE;
		}
	}
	else
	{
		if( bDnDisabled )
		{
			bDnDisabled = FALSE;
			bChanged = TRUE;
		}
	}
	if( nAct == nMax )
	{
		if( !bUpDisabled )
		{
			bUpDisabled = TRUE;
			bChanged = TRUE;
		}
	}
	else
	{
		if( bUpDisabled )
		{
			bUpDisabled = FALSE;
			bChanged = TRUE;
		}
	}
	return bChanged;
}

BOOL VCScrollBarEngine::SetAct( long nA )
{
	if( nAct == nA )
		return FALSE;

	nAct = nA;

	if( nAct < nMin )
		nAct = nMin;
	if( nAct > nMax )
		nAct = nMax;

	bCalcThumb = TRUE;
#ifdef VCL
	return DisableUpDnButton();
#else
	return FALSE;
#endif
}

void VCScrollBarEngine::SetLineSize( long nLine )
{
	nLineSize = nLine;
}

void VCScrollBarEngine::SetPageSize( long nPage )
{
	nPageSize = nPage;
}

void VCScrollBarEngine::Paint( OutputDevice* pOut )
{
	bInPaint = TRUE;

	if( pOut->GetOutDevType() == OUTDEV_PRINTER )
	{
		// Fuers drucken muss alles neu durchgerechnet werden, da die
		// Aufloesung anders ist
		bCalcThumb = TRUE;
	}

	if( bCalcSizes )
	{
		aUpRect 	= GetUpRect( pOut );
		aDnRect 	= GetDnRect( pOut );
		bCalcSizes = FALSE;
	}
	if( bCalcThumb )
	{
		aThumbRect = GetThumbRect( pOut );
		aPgUpRect  = GetPgUpRect( pOut );
		aPgDnRect  = GetPgDnRect( pOut );
		bCalcThumb = FALSE;
	}

	DrawScrollBar( pOut );
	DrawBorder( pOut );

	if( pOut->GetOutDevType() == OUTDEV_PRINTER )
	{
		// Nach dem Drucken muss vor dem naechsten Paint nochmal durchgerechnet werden
		bCalcThumb = TRUE;
	}

	bInPaint = FALSE;
}

void VCScrollBarEngine::MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt )
{
	if( !bEnable )
		return;

	pActWin = pWindow;

	BOOL bPaintArrows = FALSE;
	BOOL bCall = FALSE;

	if( bCalcThumb )
	{
		aPgUpRect = GetPgUpRect( pWindow );
		aPgDnRect = GetPgDnRect( pWindow );
		aThumbRect= GetThumbRect( pWindow );
		bCalcThumb = FALSE;
	}

	aMDownPos = rMEvt.GetPosPixel();
	if( aDnRect.IsInside( aMDownPos ) )
	{
		if( bDnDisabled )
		{
			Sound::Beep();
			return;
		}

		bDnHit = TRUE;
		bDnIn  = TRUE;
		DrawDnButton( pWindow );

		if( nAct > nMin )
		{
			HideThumb( pWindow );
			bPaintArrows = SetAct( nAct-nLineSize );
			DrawThumb( pWindow );
			bCall = TRUE;
		}
	}
	else
	if( aUpRect.IsInside( aMDownPos ) )
	{
		if( bUpDisabled )
		{
			Sound::Beep();
			return;
		}

		bUpHit= TRUE;
		bUpIn = TRUE;
		DrawUpButton( pWindow );

		if( nAct < nMax )
		{
			HideThumb( pWindow );
			bPaintArrows = SetAct( nAct+nLineSize );
			DrawThumb( pWindow );
			bCall = TRUE;
		}
	}
	else
	if( aPgDnRect.IsInside( aMDownPos ) )
	{
		bPgDnHit = TRUE;
		bPgDnIn  = TRUE;

		if( nAct > nMin )
		{
			bPaintArrows = SetAct( nAct-nPageSize );
			DrawThumb( pWindow );
			DrawPgDnRect( pWindow );
			DrawPgUpRect( pWindow );
			bCall = TRUE;
		}
	}
	else
	if( aPgUpRect.IsInside( aMDownPos ) )
	{
		bPgUpHit = TRUE;
		bPgUpIn  = TRUE;

		if( nAct < nMax )
		{
			bPaintArrows = SetAct( nAct+nPageSize );
			DrawThumb( pWindow );
			DrawPgUpRect( pWindow );
			DrawPgDnRect( pWindow );
			bCall = TRUE;
		}
	}
	else
	if( aThumbRect.IsInside( aMDownPos ) )
	{
		bThumbHit = TRUE;
		bThumbIn  = TRUE;
		if( bHorz )
			nMDownThumbPos = aThumbRect.Left();
		else
			nMDownThumbPos = aThumbRect.Top();
	}

	if( bPaintArrows )
	{
		DrawUpButton( pWindow );
		DrawDnButton( pWindow );
	}

	if( bCall )
	{
		if( aScrollHdlLink.IsSet() )
			aScrollHdlLink.Call( this );

		if( !bEnable )
		{
			bDnIn = bUpIn = bPgDnIn = bPgUpIn = bThumbIn = FALSE;
			bUpHit = bDnHit = bThumbHit = FALSE;
			bCalcSizes = bCalcThumb = TRUE;
			Paint( pWindow );
		}
		else
		if( bCalcThumb )
			Paint( pWindow );

		aTimer.SetTimeout( 200 );
		aTimer.Start();
	}
}

void VCScrollBarEngine::MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt )
{
	if( !bEnable )
		return;

	pActWin = NULL;

	if( bDnIn )
	{
		bDnIn = FALSE;
		DrawDnButton( pWindow );
	}
	else
	if( bUpIn )
	{
		bUpIn = FALSE;
		DrawUpButton( pWindow );
	}
	else
	if( bPgDnIn )
	{
		bPgDnIn = FALSE;
		DrawPgDnRect( pWindow );
	}
	else
	if( bPgUpIn )
	{
		bPgUpIn = FALSE;
		DrawPgUpRect( pWindow );
	}

	bThumbIn = FALSE;
	bUpHit = bDnHit = bThumbHit = FALSE;

	aTimer.Stop();
}

void VCScrollBarEngine::MouseMove( Window* pWindow, const MouseEvent& rMEvt )
{
	if( !bEnable )
		return;

	BOOL bCall = FALSE;
	BOOL bPaintArrows = FALSE;

	if( rMEvt.IsLeft() )
	{
		Point aPos = rMEvt.GetPosPixel();

		if( bThumbHit )
		{
			long nMinPos;
			long nMaxPos;
			long nMPos;

			if( bHorz )
			{
				nMinPos = aDnRect.Right();
				nMaxPos = aUpRect.Left() - aThumbRect.GetSize().Width();
				nMPos = nMDownThumbPos + (aPos.X() - aMDownPos.X());
			}
			else
			{
				nMinPos = aDnRect.Bottom();
				nMaxPos = aUpRect.Top() - aThumbRect.GetSize().Height();
				nMPos = nMDownThumbPos + (aPos.Y() - aMDownPos.Y());
			}

			long nDeltaPos = nMaxPos - nMinPos;
			long nDeltaVal = nMax - nMin;
			long nVal      = (nDeltaVal * (nMPos - nMinPos) / nDeltaPos) + nMin;

			if( nVal != nAct )
			{
				HideThumb( pWindow );
				bPaintArrows = SetAct( nVal );
				DrawThumb( pWindow );
				bCall = TRUE;
			}

		}
		else
		if( bDnHit )
		{
			if( aDnRect.IsInside( aPos ) )
			{
				if( !bDnIn )
				{
					bDnIn = TRUE;
					DrawDnButton( pWindow );

					aTimer.Start();
				}
			}
			else
			{
				if( bDnIn )
				{
					bDnIn = FALSE;
					DrawDnButton( pWindow );
					aTimer.Stop();
				}
			}
		}
		else
		if( bUpHit )
		{
			if( aUpRect.IsInside( aPos ) )
			{
				if( !bUpIn )
				{
					bUpIn = TRUE;
					DrawUpButton( pWindow );
					aTimer.Start();
				}
			}
			else
			{
				if( bUpIn )
				{
					bUpIn = FALSE;
					DrawUpButton( pWindow );
					aTimer.Stop();
				}
			}
		}
		else
		if( bPgDnHit )
		{
			if( aPgDnRect.IsInside( aPos ) )
			{
				if( !bPgDnIn )
				{
					bPgDnIn = TRUE;
					DrawPgDnRect( pWindow );
					aTimer.Start();
				}
			}
			else
			{
				if( bPgDnIn )
				{
					bPgDnIn = FALSE;
					DrawPgDnRect( pWindow );
					aTimer.Stop();
				}
			}
		}
		else
		if( bPgUpHit )
		{
			if( aPgUpRect.IsInside( aPos ) )
			{
				if( !bPgUpIn )
				{
					bPgUpIn = TRUE;
					DrawPgUpRect( pWindow );
					aTimer.Start();
				}
			}
			else
			{
				if( bPgUpIn )
				{
					bPgUpIn = FALSE;
					DrawPgUpRect( pWindow );
					aTimer.Stop();
				}
			}
		}
	}
#ifdef VCL
	if( bPaintArrows )
	{
		DrawUpButton( pWindow );
		DrawDnButton( pWindow );
	}
#endif

	if( bCall )
	{
		if( aScrollHdlLink.IsSet() )
			aScrollHdlLink.Call( this );

		if( !bEnable )
		{
			bDnIn = bUpIn = bPgDnIn = bPgUpIn = bThumbIn = FALSE;
			bUpHit = bDnHit = bThumbHit = FALSE;
			bCalcSizes = bCalcThumb = TRUE;
			Paint( pWindow );
		}
		else
		if( bCalcThumb )
			Paint( pWindow );
	}
}

void VCScrollBarEngine::KeyInput( Window* pWin, const KeyEvent& rKEvt )
{
	KeyCode aCode = rKEvt.GetKeyCode();
	BOOL	bCall = FALSE;

	if( !bEnable )
		return;

	BOOL bPaintArrows = FALSE;

	switch( aCode.GetCode() )
	{
		case KEY_DOWN:
		case KEY_RIGHT:
			if( nAct < nMax )
			{
				HideThumb( pWin );
				bPaintArrows =SetAct( nAct + nLineSize );
				DrawThumb( pWin );
				bCall = TRUE;
			}
			break;

		case KEY_PAGEDOWN:
			if( nAct < nMax )
			{
				HideThumb( pWin );
				bPaintArrows =SetAct( nAct + nPageSize );
				DrawThumb( pWin );
				bCall = TRUE;
			}
			break;

		case KEY_UP:
		case KEY_LEFT:
			if( nAct > nMin )
			{
				HideThumb( pWin );
				bPaintArrows =SetAct( nAct-nLineSize );
				DrawThumb( pWin );
				bCall = TRUE;
			}
			break;

		case KEY_PAGEUP:
			if( nAct > nMin )
			{
				HideThumb( pWin );
				bPaintArrows =SetAct( nAct - nPageSize );
				DrawThumb( pWin );
				bCall = TRUE;
			}
			break;

		case KEY_HOME:
			if( nAct > nMin )
			{
				HideThumb( pWin );
				bPaintArrows =SetAct( nMin );
				DrawThumb( pWin );
				bCall = TRUE;
			}
			break;

		case KEY_END:
			if( nAct < nMax )
			{
				HideThumb( pWin );
				bPaintArrows =SetAct( nMax );
				bCalcThumb = TRUE;
				DrawThumb( pWin );
				bCall = TRUE;
			}
			break;
	}

#ifdef VCL
	if( bPaintArrows )
	{
		DrawUpButton( pWin );
		DrawDnButton( pWin );
	}
#endif

	if( bCall )
	{
		if( aScrollHdlLink.IsSet() )
			aScrollHdlLink.Call( this );

		if( !bEnable )
		{
			bCalcSizes = bCalcThumb = TRUE;
			Paint( pWin );
		}
		else
		if( bCalcThumb )
			Paint( pWin );
	}
}

/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/si/vcengin.cxx,v 1.1.1.1 2000/09/29 11:02:39 mh Exp $

	  Source Code Control System - Update

	  $Log: vcengin.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:39  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:44  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:48  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.44  2000/02/10 20:01:02  hr
	  #70473# changes for unicode ( patched by automated patchtool )
	
	  Revision 1.43  1998/08/11 14:30:38  HR
	  #54781#: Umbennungen fuer GCC
	

	  Rev 1.42   11 Aug 1998 16:30:38   HR
   #54781#: Umbennungen fuer GCC

	  Rev 1.41   21 Nov 1997 17:02:16   OV
   Headerumstellung

	  Rev 1.40   13 Nov 1997 17:06:34   OV
   #42205# Bei gedruecktem SpinButton zyklisch den Status des Fensters pruefen, ggf. Abbruch des Spins

	  Rev 1.39   06 Nov 1997 19:12:04   OV
   weitere VCL-Anpassungen

	  Rev 1.38   06 Nov 1997 12:46:00   OV
   VCL-Umstellungen

	  Rev 1.37   19 Aug 1997 17:53:34   OV
   #42205# SpinnedUp/Down: Fehlercode abfragen

	  Rev 1.36   10 Dec 1996 14:17:22   OV
   Umstellung auf SvxMacroItem

	  Rev 1.35   05 Jun 1996 13:24:48   OV
   Clooks, Bugfixes, Erweiterungen

	  Rev 1.34   21 Mar 1996 10:56:24   OV
   Clooks & Umstellung SV311

	  Rev 1.33   23 Jan 1996 15:41:42   OV
   #23601# Doc-ScrollBars PageUp/Down: Thumb nur bis Maus

	  Rev 1.32   22 Jan 1996 18:49:56   OV
   #23600# ScrollBar: nMax setzen -> nCur korrigieren

*************************************************************************/

#pragma SEG_EOFMODULE

