/*************************************************************************
 *
 *  $RCSfile: vcengin1.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef VCL
#ifndef _SYSDEPEN_HXX
#include <vcl/sysdep.hxx>
#endif
#endif
#ifndef _SHL_HXX
#include <tools/shl.hxx>
#endif
#ifndef _SV_CLIP_HXX //autogen
#include <vcl/clip.hxx>
#endif
#ifndef _ACCEL_HXX
#include <vcl/accel.hxx>
#endif
#ifndef _COLOR_HXX
#include <vcl/color.hxx>
#endif
#ifndef _WRKWIN_HXX
#include <vcl/wrkwin.hxx>
#endif
#ifndef _SFX_SFXSIDS_HRC
#include <sfx2/sfxsids.hrc>
#endif
#pragma hdrstop

#include "sidll.hxx"
#include "vcedit.hxx"
#include "vcengin.hxx"
#include "vceeng.hxx"
#include "vctrls.hxx"
#include "vcform.hxx"
#include "vcattr.hxx"

DBG_NAME(VCSingleEditEngine);
DBG_NAME(VCEditMapper);

static void HideWindow( Window* pWindow )
{
#ifndef VCL

#if defined(WIN) || defined(WNT)
	Window* pParent = pWindow->GetParent();
	if( pParent->GetBackgroundBrush().GetStyle() == BRUSH_NULL )
	{
		pParent->Update();
		pWindow->Hide();
		HWND hWin = Sysdepen::GethWnd( *pParent );
		ValidateRect( hWin, 0 );
	}
	else
		pWindow->Hide();
#else
	pWindow->Hide();
#endif

#else

	pWindow->Hide();

#endif

}

VCEditMapper::VCEditMapper()
{
	DBG_CTOR(VCEditMapper,0);
}

VCEditMapper::~VCEditMapper()
{
	DBG_DTOR(VCEditMapper,0);
}

class VCSingleEditWin : public Window
{
protected:
	VCSingleEditEngine* pEditEngine;
	SiEditSlotInterceptor* pInterceptor;

	void	GetFocus();
	void	LoseFocus();
	void	Paint( const Rectangle& rRect );
	void	MouseButtonDown( const MouseEvent& rMEvt );
	void	MouseButtonUp( const MouseEvent& rMEvt );
	void	MouseMove( const MouseEvent& rMEvt );
	void	KeyInput( const KeyEvent& rKEvt );
			DECL_LINK(InterceptorHdl, void*);

			DECL_LINK(VCSingleEdEvent, void* );

public:
	BOOL	bParentAlreadyClosing;
	BOOL 	bKeyInputCallBaseClass;

			VCSingleEditWin( VCSingleEditEngine* pEngine, Window* pParent, WinBits nStyle );
			~VCSingleEditWin();
	void	DestroyYourself();
};

VCSingleEditWin::VCSingleEditWin( VCSingleEditEngine* pEngine, Window* pParent, WinBits nStyle ) :
	Window( pParent, nStyle ),
	pEditEngine( pEngine ),
	pInterceptor(0)
{
	SetPointer( Pointer( POINTER_BEAM ) );
	bParentAlreadyClosing = FALSE;
	bKeyInputCallBaseClass = FALSE;
}

VCSingleEditWin::~VCSingleEditWin()
{
	DELETEZ(pInterceptor);
	if( pEditEngine && pEditEngine->pEditWin == this )
		pEditEngine->pEditWin = 0;
}

void VCSingleEditWin::DestroyYourself()
{
	pEditEngine = 0;
	Application::PostUserEvent(LINK(this,VCSingleEditWin,VCSingleEdEvent),(void*)0);
}

IMPL_LINK(VCSingleEditWin, VCSingleEdEvent, void*, EMPTY_ARG )
{
	delete this;
	return 0;
}

void __EXPORT VCSingleEditWin::LoseFocus()
{
	if( pEditEngine )
	{
		DELETEZ(pInterceptor);
		if( !bParentAlreadyClosing )
		{
			bParentAlreadyClosing = TRUE;
			pEditEngine->NotifyCloseEditWindow();
		}
	}
}

void VCSingleEditWin::GetFocus()
{
	if( pEditEngine && !bParentAlreadyClosing )
		pInterceptor = new SiEditSlotInterceptor( LINK(this, VCSingleEditWin, InterceptorHdl));
}

IMPL_LINK(VCSingleEditWin, InterceptorHdl, void*, pPtr)
{
	if( pEditEngine && !bParentAlreadyClosing )
	{
		USHORT nSID = (USHORT)(ULONG)pPtr;
		switch( nSID )
		{
			case SID_CUT: pEditEngine->Cut(); break;
			case SID_COPY: pEditEngine->Copy(); break;
			case SID_PASTE: pEditEngine->Paste(); break;
		}
	}
	return 0;
}

//------------------------------------------------------------------------
void __EXPORT VCSingleEditWin::Paint( const Rectangle& rRect )
{
	if( pEditEngine )
		pEditEngine->Paint( this );
}

//------------------------------------------------------------------------
void __EXPORT VCSingleEditWin::MouseButtonDown( const MouseEvent& rMEvt )
{
	if( pEditEngine )
	{
		Point aPos = rMEvt.GetPosPixel();
		aPos = PixelToLogic( aPos );
		MouseEvent aEvt( aPos, rMEvt.GetClicks(), rMEvt.GetMode(),
						 rMEvt.GetButtons(), rMEvt.GetModifier() );
		pEditEngine->MouseButtonDown( this, aEvt );
	}
}


//------------------------------------------------------------------------
void __EXPORT VCSingleEditWin::MouseButtonUp( const MouseEvent& rMEvt )
{
	if( pEditEngine )
	{
		Point aPos = rMEvt.GetPosPixel();
		aPos = PixelToLogic( aPos );
		MouseEvent aEvt( aPos, rMEvt.GetClicks(), rMEvt.GetMode(),
						 rMEvt.GetButtons(), rMEvt.GetModifier() );
		pEditEngine->MouseButtonUp( this, aEvt );
	}
}


//------------------------------------------------------------------------
void __EXPORT VCSingleEditWin::MouseMove( const MouseEvent& rMEvt )
{
	if( pEditEngine )
	{
		Point aPos = rMEvt.GetPosPixel();
		aPos = PixelToLogic( aPos );
		MouseEvent aEvt( aPos, rMEvt.GetClicks(), rMEvt.GetMode(),
						 rMEvt.GetButtons(), rMEvt.GetModifier() );
		pEditEngine->MouseMove( this, aEvt );
	}
}

//------------------------------------------------------------------------
void __EXPORT VCSingleEditWin::KeyInput( const KeyEvent& rKEvt )
{
	if( pEditEngine )
	{
		bKeyInputCallBaseClass = TRUE;
		pEditEngine->KeyInput( this, rKEvt );
		if( bKeyInputCallBaseClass )
			Window::KeyInput( rKEvt );
	}
}

void VCSingleEditEngine::NotifyCloseEditWindow()
{
	DBG_CHKTHIS(VCSingleEditEngine,0);

	if( pEditWin )
	{
		aCursor.Hide();
		SetSelectLen( 0 );
		::HideWindow( pEditWin );
		aDestroyEditWinTimer.Start();
	}
}

Rectangle VCSingleEditEngine::GetTextRect( OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	Size aPixSize = pOut->PixelToLogic( Size( 1, 1 ) );
	Rectangle aTextRect( pOwner->aPos, pOwner->aSize );
	// bei Aenderungen GetOptSize anpassen!
	long nFact = 2;
	if( pOwner->bBorder )
		nFact++;

	aTextRect.Left()  += aPixSize.Width()  * nFact;
	aTextRect.Top()   += aPixSize.Height() * nFact;
	aTextRect.Right() -= aPixSize.Width()  * nFact;
	aTextRect.Bottom()-= aPixSize.Height() * nFact;

	return aTextRect;
}

void VCSingleEditEngine::DrawBackground( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( pOwner->bTransparent )
		return;

	Brush aBrush( pOwner->aBackColor );
	Brush aOldBrush = pOut->GetFillInBrush();
	pOut->SetFillInBrush( aBrush );
	Pen aOldPen( pOut->GetPen() );
	pOut->SetPen( Pen( PEN_NULL ) );
	pOut->DrawRect( Rectangle( pOwner->aPos, pOwner->aSize ) );
	pOut->SetPen( aOldPen );
	pOut->SetFillInBrush( aOldBrush );
}

#ifndef VCL

void VCSingleEditEngine::DrawBorder( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( !pOwner->bBorder )
		return;
#if defined UNX
	Pen aOldPen = pOut->GetPen();

	Color a3DShadowColor( COL_BLACK ); //COL_3DSHADOW );
	Color a3DLightColor( COL_BLACK ); //COL_3DLIGHT );
	Pen a3DShadowPen( a3DShadowColor );
	Pen a3DLightPen( a3DLightColor );
	Size aSize = pOwner->aSize;
	Point aPos = pOwner->aPos;
	pOut->SetPen( a3DShadowPen );
	// vertical shadow
	pOut->DrawLine( aPos,
					Point( aPos.X(), aPos.Y()+aSize.Height() ) );
	// horizontal shadow
	pOut->DrawLine( aPos,
					Point( aPos.X()+aSize.Width(), aPos.Y() ) );
	pOut->SetPen( a3DLightPen );
	// vertical anti-shadow
	pOut->DrawLine( Point( aPos.X()+aSize.Width(), aPos.Y()+1 ),
					Point( aPos.X()+aSize.Width(),
						   aPos.Y()+aSize.Height() ) );
	// horizontal anti-shadow
	pOut->DrawLine( Point( aPos.X()+1, aPos.Y()+aSize.Height() ),
					Point( aPos.X()+aSize.Width(), aPos.Y()+aSize.Height() ) );

	pOut->SetPen( aOldPen );
#else
#ifdef OS2
	Color aCol( COL_3DSHADOW );
#else
	Color aCol;
#endif
	Pen   aPen ( aCol );
	Pen   aOldPen	= pOut->GetPen( );
	pOut->SetPen( aPen );
	Brush aOldBrush = pOut->GetFillInBrush( );
	pOut->SetFillInBrush( Brush( BRUSH_NULL ) );

	pOut->DrawRect( Rectangle( pOwner->aPos, pOwner->aSize ) );

	pOut->SetPen( aOldPen );
	pOut->SetFillInBrush( aOldBrush );
#endif
}
#else
void VCSingleEditEngine::DrawBorder( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( pOwner->bBorder )
		SiDrawBorder_Impl( pOut, pOwner->aPos, pOwner->aSize );
}
#endif

void VCSingleEditEngine::DrawText( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	Rectangle aTextRect = GetTextRect( pOut );

	BOOL   bOldClip = pOut->IsClipRegion();
	Region aOldClip = pOut->GetClipRegion();

	Rectangle aTempRect( aTextRect );
#ifndef VCL
#ifdef WNT
	{
	// HACK, bei kleinen Fonts wird am linken Rand des
	// Textes ein Pixel abgeschnitten (erstmal nur fuer DropDown-
	// Listboxen (->bActive=0)), da sonst Paintprobleme (Schmutz bleibt stehen)
	if( !pOwner->bActive )
	{
		Size aPix(1,0);
		aPix = pOut->PixelToLogic(aPix);
		aTempRect.Left() -= aPix.Width();
	}
	}
#endif
#endif

	pOut->IntersectClipRegion( aTempRect );

	Font   aTempFont = pOwner->aFont;
	aTempFont.SetColor( pOwner->aForeColor );
	aTempFont.SetFillColor( pOwner->aBackColor );
	pOwner->aFont.SetTransparent( pOwner->bTransparent );
	Font aOldFont = pOut->GetFont( );
	pOut->SetFont( aTempFont );

	String aStr = aText;
	if( pOwner->bMnemonic && !pOwner->bActive && (pOut->GetOutDevType() != OUTDEV_PRINTER) )
		VCEditEngine::ConvertAmpersand( &aStr );
	else
	if( pOwner->bMnemonic && !pOwner->bActive && (pOut->GetOutDevType() == OUTDEV_PRINTER) )
		VCEditEngine::DeleteAmpersand( &aStr );

	if( pOwner->aPassWordChar.Len() )
		aStr.Fill( aStr.Len(), pOwner->aPassWordChar[(USHORT)0] );

	Point aTextPos = aTextRect.TopLeft();
	aTextPos.X() += nTextOfsX;

	Size aTextSize = pOut->GetTextSize( aStr );

	long nAlignOfsX = 0;
	if( !pOwner->bMultiLine && pOwner->nAlignment && (!aCursor.IsVisible() || pEditWin!=pOut))
	{
		long nVisWidth = aTextRect.GetSize().Width();
		// Alignment fuer Single-Edit
		// 1 == Rechtsbuendig
		// 2 == Zentriert
		nAlignOfsX = nVisWidth - aTextSize.Width();
		if( pOwner->nAlignment == 2 )
			nAlignOfsX /= 2;
		aTextPos.X() += nAlignOfsX;
	}

	if( pOwner->bCenterVert )
		aTextPos.Y() += (aTextRect.GetSize().Height()>>1) - (aTextSize.Height()>>1);

	if( (pOut->GetOutDevType() != OUTDEV_PRINTER) && pOwner->bMnemonic )
	{
		USHORT nFlags = !pOwner->bEnabled ? CTRLTEXT_DRAW_DISABLE : CTRLTEXT_DRAW_ENABLE;
		Brush aOldBack = pOut->GetBackgroundBrush( );
		pOut->SetBackgroundBrush( Brush( pOwner->aBackColor ) );
		((Window*)pOut)->DrawCtrlText( aTextPos, aStr, 0, STRING_LEN, nFlags );
		pOut->SetBackgroundBrush( aOldBack );
	}
	else if( (pOut==pEditWin) )
	{
		BOOL bCurVis = aCursor.IsVisible();
		if( bCurVis )
			aCursor.Hide();
		Point aPos = aTextPos;
		Size  aPortSize;
		pOut->DrawText( aPos, aStr, 0, (USHORT)aSelection.Min() );
		aPortSize = pOut->GetTextSize( aStr, 0, (USHORT)aSelection.Min() );
		aPos.X() += aPortSize.Width();

#ifndef VCL
		pOwner->aFont.SetColor( Color( COL_HIGHLIGHTTEXT ) );
		pOwner->aFont.SetFillColor( Color( COL_HIGHLIGHT ) );
#else
		const StyleSettings& rSettings = pOut->GetSettings().GetStyleSettings();
		pOwner->aFont.SetColor( rSettings.GetHighlightTextColor() );
		pOwner->aFont.SetFillColor( rSettings.GetHighlightColor() );
#endif

		pOut->SetFont( pOwner->aFont );
		pOut->DrawText( aPos, aStr, (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
		aPortSize = pOut->GetTextSize( aStr, (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
		aPos.X() += aPortSize.Width();

		pOwner->aFont.SetColor( pOwner->aForeColor );
		pOwner->aFont.SetFillColor( pOwner->aBackColor );
		pOut->SetFont( pOwner->aFont );
		pOut->DrawText( aPos, aStr, (USHORT)aSelection.Min()+(USHORT)aSelection.Len(), STRING_LEN );

		// Bis Zeilenende loeschen
		Rectangle aRect( aTextPos.X() + pOut->GetTextSize( aStr ).Width(), aTextPos.Y(),
						aTextRect.Right(), aTextRect.Bottom() );
		Pen   aOldPen	= pOut->GetPen( );
		pOut->SetPen( Pen( PEN_NULL ) );
		Brush aOldBrush = pOut->GetFillInBrush( );
		pOut->SetFillInBrush( Brush( pOwner->aBackColor ) );
		pOut->DrawRect( aRect );
		pOut->SetPen( aOldPen );
		pOut->SetFillInBrush( aOldBrush );

		if( bCurVis )
			aCursor.Show();
	}
	else
	{
		if( !pOwner->bEnabled )
		{
			Color aDisableColor( COL_LIGHTGRAY );
			if( pOwner->aBackColor == aDisableColor )
				aDisableColor = Color( COL_GRAY );
			aTempFont.SetColor( aDisableColor );
			pOut->SetFont( aTempFont );
		}
		pOut->DrawText( aTextPos, aStr );
	}

	pOut->SetFont( aOldFont );
	if( bOldClip )
		pOut->SetClipRegion( aOldClip );
	else
		pOut->SetClipRegion();
}

void VCSingleEditEngine::ShowEditWin( Window* pWin )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( !pEditWin )
	{
		pEditWin = new VCSingleEditWin( this, pWin, 0 );

		// aktives Window im SiDLL-Objekt verankern, damit die
		// VCDrawView beim Scrollen dessen Position anpassen kann
		ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
		if( pImplEdit )
		{
			delete pImplEdit->pWin;
			delete pImplEdit;
		}
		pImplEdit = new ImplActiveEdit;
		pImplEdit->pThis = pOwner;
		pImplEdit->pWin = pEditWin;
		SI_DLL()->SetActiveEdit( pImplEdit );

		InitEditWinPosSizeMap();

		// Bitte nicht flimmern
		Font aTempFont = pOwner->aFont;
		aTempFont.SetColor( pOwner->aForeColor );
		aTempFont.SetFillColor( pOwner->aBackColor );
		pOwner->aFont.SetTransparent( pOwner->bTransparent );
		pEditWin->SetFont( aTempFont );
		Brush aOldBrush = pEditWin->GetBackgroundBrush();
		pEditWin->SetBackgroundBrush( Brush( BRUSH_NULL ) );
		aCursor.Show();
		pEditWin->Show();
		pEditWin->Update();
		pEditWin->SetBackgroundBrush( aOldBrush );

		pEditWin->SetCursor( &aCursor );
		UpdateCursorPos( pEditWin );
		pEditWin->GrabFocus();
		pOwner->GotFocus();
	}

}

String VCSingleEditEngine::CutLineFromStart( OutputDevice* pOut, String* pStr, long nMaxWidth )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( !pStr->Len() )
		return String();

	USHORT nBreakPos = pOut->GetTextBreak( *pStr, nMaxWidth );

	USHORT nHardBreak = pStr->Search( '\n' );
	if( nHardBreak < nBreakPos )
		nHardBreak = nBreakPos;

	// Mindestens 1 Zeichen pro Zeile
	if( !nBreakPos )
		nBreakPos++;

	// Wenns denn passt...
	if( nBreakPos == STRING_LEN )
		return pStr->Cut( 0, nBreakPos );

	// Leezeichen als erstes Zeichen der naechsten Zeile beseitigen
	if( ((*pStr)[nBreakPos] == ' ') ||
		((*pStr)[nBreakPos] == '\t')||
		((*pStr)[nBreakPos] == '\n')   )
	{
		pStr->Cut( nBreakPos, 1 );
		return pStr->Cut( 0, nBreakPos );
	}

	USHORT nIndex = nBreakPos-1;
	while( nIndex > 0 )
	{
		if( ((*pStr)[nIndex] == ' ') ||
			((*pStr)[nIndex] == '\t')||
			((*pStr)[nIndex] == '\n')   )
		{
			pStr->Cut( nIndex, 1 );
			return pStr->Cut( 0, nIndex );
		}
		else
			nIndex--;
	}

	return pStr->Cut( 0, nBreakPos );
}

void VCSingleEditEngine::UpdateCursorPos( Window* pWin )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	Font aOldFont = pWin->GetFont();
	pWin->SetFont( pOwner->aFont );
	Size aSize = pWin->GetTextSize( aText, 0, (USHORT)nCursorPos );
	pWin->SetFont( aOldFont );

	Rectangle aTextRect = GetTextRect( pWin );
	Point aPos( aTextRect.Left() + aSize.Width() + nTextOfsX, aTextRect.Top() );
	if( pOwner->bCenterVert )
		aPos.Y() += (aTextRect.GetSize().Height()>>1) - (aSize.Height()>>1);

	long nHeight = aSize.Height();
	if( nHeight > aTextRect.GetSize().Height() )
		nHeight = aTextRect.GetSize().Height();
	aCursor.SetSize( Size( 0, nHeight ) );
	aCursor.SetPos( aPos );
}

void VCSingleEditEngine::ScrollToCursor( Window* pWindow )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( !aCursor.IsVisible() || (pWindow != pEditWin) )
		return;

	Rectangle aRect = GetTextRect( pWindow );
	if( aCursor.GetPos().X() >= aRect.Right() )
	{
		nTextOfsX -= aCursor.GetPos().X() - aRect.Right();
		UpdateCursorPos( pWindow );
		DrawText( pWindow );
	}
	else
	if( aCursor.GetPos().X() < aRect.Left() )
	{
		nTextOfsX += aRect.Left() - aCursor.GetPos().X();
		if( nTextOfsX > 0 )
			nTextOfsX = 0;
		UpdateCursorPos( pWindow );
		DrawText( pWindow );
	}
}

long VCSingleEditEngine::GetStrPosFromMouse( Window* pWin, const Point& rPos )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	long nWidth = rPos.X() - GetTextRect( pWin ).Left() - nTextOfsX;
	if( nWidth <= 0 )
		return 0;

	String aStr = aText;
	if( aPassWordChar.Len() )
		aStr.Fill( aStr.Len(), aPassWordChar[(USHORT)0] );

	long nPos = pWin->GetTextBreak( aStr, nWidth );
	if( nPos > aText.Len() )
		nPos = aText.Len();
	return nPos;
}

VCSingleEditEngine::VCSingleEditEngine( VCEditEngine* pTheOwner ) :
	pOwner(pTheOwner),
	pEditWin(NULL),
	bCloseEditWin(FALSE),
	bDrag(FALSE),
	nCursorPos(0),
	nAnchorPos(STRING_LEN),
	nTextOfsX(0),
	nMaxLen(0),
	bModified( FALSE )
{
	DBG_CTOR(VCSingleEditEngine,0);
	aDestroyEditWinTimer.SetTimeoutHdl(LINK(this,VCSingleEditEngine,DestroyEditWin));
	aDestroyEditWinTimer.SetTimeout(0);
}

VCSingleEditEngine::~VCSingleEditEngine()
{
	if( pEditWin )
	{
		ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
		// wenn die Fenster unterschiedlich sind, wurde pEditWin bereits
		// von draussen geloescht
		if(pImplEdit && pImplEdit->pWin == pEditWin)
		{
			delete pEditWin;
			delete pImplEdit;
			SI_DLL()->SetActiveEdit( 0 );
		}
	}
	DBG_DTOR(VCSingleEditEngine,0);
}

void VCSingleEditEngine::SetText( const String& rText )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	aText = rText;
	aText.ConvertLineEnd( LINEEND_LF );
	if( pOwner->nMaxLen && (aText.Len() > pOwner->nMaxLen) )
			aText = aText.Cut( 0, pOwner->nMaxLen );

	nCursorPos = 0;
	nAnchorPos = 0;
	aSelection.Min() = 0;
	aSelection.Max() = 0;
	bModified = FALSE;
	if( pEditWin )
		pEditWin->Invalidate();
}

void VCSingleEditEngine::SetSelectStart( long nStart )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( nStart > aText.Len() )
		nStart = aText.Len();

	if( nStart < aSelection.Min() )
	{
		aSelection.Min() = nStart;
		nCursorPos = (USHORT)aSelection.Max();
	}
	else
	{
		aSelection.Min() = nStart;
		aSelection.Max() = nStart;
		nCursorPos = (USHORT)nStart;
	}

	if( pEditWin )
	{
		DrawText( pEditWin );
		UpdateCursorPos( pEditWin );
		ScrollToCursor( pEditWin );
	}
}

void VCSingleEditEngine::SetSelectLen( long nLen )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	aSelection.Max() = aSelection.Min() + nLen;
	if( aSelection.Max() > aText.Len() )
		aSelection.Max() = aText.Len();

	if( pEditWin )
	{
		DrawText( pEditWin );
		UpdateCursorPos( pEditWin );
		ScrollToCursor( pEditWin );
	}
}

String VCSingleEditEngine::GetSelectText() const
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	return aText.Copy( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
}

void VCSingleEditEngine::SetSelectText( const String& rText )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( aSelection.Len() )
		aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );

	USHORT nPasteLen = rText.Len();
	if( pOwner->nMaxLen && ((aText.Len() + rText.Len()) > pOwner->nMaxLen ) )
		nPasteLen = pOwner->nMaxLen - aText.Len();

	aText.Insert( rText, 0, nPasteLen, (USHORT)aSelection.Min() );
	nCursorPos = (USHORT)aSelection.Min() + nPasteLen;

	aSelection.Min() = nCursorPos;
	aSelection.Max() = nCursorPos;
	nAnchorPos = STRING_LEN;
	if( pEditWin )
	{
		DrawText( pEditWin );
		UpdateCursorPos( pEditWin );
		ScrollToCursor( pEditWin );
	}
	pOwner->Modified();
}

void VCSingleEditEngine::Paint( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( bCloseEditWin )
	{
		aCursor.Hide();
		DELETEZ(pEditWin);
		bCloseEditWin = FALSE;
	}

	DrawBackground( pOut );
	DrawBorder( pOut );
	DrawText( pOut );
}

void VCSingleEditEngine::MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( aDestroyEditWinTimer.IsActive())
		DestroyEditWin( &aDestroyEditWinTimer );

	if( !pOwner->bActive )
		return;

	BOOL bCreatedEditWin = FALSE;
	if( !pEditWin )
	{
		ShowEditWin( pWindow );
		bCreatedEditWin = TRUE;
		pWindow = pEditWin;
	}
	nCursorPos = (USHORT)GetStrPosFromMouse( pWindow, rMEvt.GetPosPixel() );
	if( !rMEvt.IsShift() || bCreatedEditWin )
		nAnchorPos = nCursorPos;

	Selection aOldSelection = aSelection;

	if( nAnchorPos < nCursorPos )
	{
		aSelection.Min() = nAnchorPos;
		aSelection.Max() = nCursorPos;
	}
	else
	{
		aSelection.Min() = nCursorPos;
		aSelection.Max() = nAnchorPos;
	}

	if( aOldSelection != aSelection )
		DrawText( pWindow );

		aCursor.Show();
		UpdateCursorPos( pWindow );
	if( !bCreatedEditWin )
	{
		bDrag = TRUE;
		pWindow->CaptureMouse();
		if( aOldSelection != aSelection )
			pOwner->Selected();
	}
}

void VCSingleEditEngine::MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( bDrag )
	{
		pWindow->ReleaseMouse();
		bDrag = FALSE;
	}
}

void VCSingleEditEngine::MouseMove( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	if( bDrag )
	{
		Selection aOldSelection( aSelection );
		USHORT nPos = (USHORT)GetStrPosFromMouse( pWindow, rMEvt.GetPosPixel() );
		if( nPos < nAnchorPos )
		{
			aSelection.Min() = nPos;
			aSelection.Max() = nAnchorPos;
		}
		else
		{
			aSelection.Min() = nAnchorPos;
			aSelection.Max() = nPos;
		}

		if( nCursorPos != nPos )
		{
			nCursorPos = nPos;
			DrawText( pWindow );
			UpdateCursorPos( pWindow );
			ScrollToCursor( pWindow );
		}
		if( aOldSelection != aSelection )
			pOwner->Selected();
	}
}

void VCSingleEditEngine::KeyInput( Window* pWindow, const KeyEvent& rKEvt )
{
	DBG_CHKTHIS(VCSingleEditEngine,0);
	KeyCode aCode = rKEvt.GetKeyCode();
	if( pEditWin )
		((VCSingleEditWin*)pEditWin)->bKeyInputCallBaseClass = FALSE;
/* geht nicht, da sonst auch AltGr ausgefiltert wird
	if( aCode.IsMod2())
	{
		if( pEditWin )
			((VCSingleEditWin*)pEditWin)->bKeyInputCallBaseClass = TRUE;
		return;
	}
*/
	if( aCode.IsShift() && (nAnchorPos==STRING_LEN) )
		nAnchorPos = nCursorPos;
	else
	if( !aCode.IsShift() )
		nAnchorPos = STRING_LEN;

	BOOL bTravel = FALSE;
	BOOL bModify = FALSE;

	if( aCode.GetFunction() != KEYFUNC_DONTKNOW )
	{
		switch( aCode.GetFunction() )
		{
			// der SlotInterceptor schafft das nicht!
			case KEYFUNC_CUT:
				if( aSelection.Len() && !pOwner->bReadOnly )
				{
					Clipboard::CopyString( aText.Copy( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() ) );
					aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
					nCursorPos = (USHORT)aSelection.Min();
					bModify = TRUE;
				}
				break;
			case KEYFUNC_COPY:
				if( aSelection.Len() )
					Clipboard::CopyString( aText.Copy( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() ) );
				break;
			case KEYFUNC_PASTE:
				if( Clipboard::HasFormat( FORMAT_STRING ) && !pOwner->bReadOnly )
				{
					String aClip = Clipboard::PasteString();
					aClip.EraseAllChars( '\n' );
					aClip.EraseAllChars( '\r' );

					if( aSelection.Len() )
						aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );

					USHORT nPasteLen = aClip.Len();
					if( pOwner->nMaxLen && ((aText.Len() + aClip.Len()) > pOwner->nMaxLen ) )
						nPasteLen = pOwner->nMaxLen - aText.Len();

					aText.Insert( aClip, 0, nPasteLen, (USHORT)aSelection.Min() );
					nCursorPos = (USHORT)aSelection.Min() + nPasteLen;

					bModify = TRUE;
				}
				break;
			case KEYFUNC_DELETE:
				if( aSelection.Len() && !pOwner->bReadOnly )
				{
					aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
					nCursorPos = (USHORT)aSelection.Min();
					bModify = TRUE;
				}
				else
				{
					aText.Erase( nCursorPos, 1 );
					bModify = TRUE;
				}
				break;
			default:
				return;
		}
	}
	else
	{
		switch( aCode.GetCode() )
		{
			case KEY_LEFT:
				if( nCursorPos )
					nCursorPos--;
				if( aCode.IsMod1() )
				{
					while( nCursorPos && (aText[(USHORT)(nCursorPos-1)] != ' ') )
						nCursorPos--;
					while( nCursorPos && (aText[(USHORT)(nCursorPos-1)] == ' ') )
						nCursorPos--;
				}
				bTravel = TRUE;
				break;
			case KEY_RIGHT:
				if( nCursorPos < aText.Len() )
					nCursorPos++;
				if( aCode.IsMod1() )
				{
					while( (nCursorPos < aText.Len()) && (aText[nCursorPos] != ' ') )
						nCursorPos++;
					while( (nCursorPos < aText.Len()) && (aText[nCursorPos] == ' ') )
						nCursorPos++;
				}
				bTravel = TRUE;
				break;
			case KEY_HOME:
				if( nCursorPos )
					nCursorPos = 0;
				bTravel = TRUE;
				break;
			case KEY_END:
				if( nCursorPos < aText.Len() )
					nCursorPos = aText.Len();
				bTravel = TRUE;
				break;
			case KEY_BACKSPACE:
				if( aSelection.Len() && (nCursorPos > (USHORT)aSelection.Max()) &&
					!pOwner->bReadOnly )
				{
					aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
					nCursorPos = (USHORT)aSelection.Min();
					bModify = TRUE;
				}
				else
				if( !pOwner->bReadOnly )
				{
					if( nCursorPos )
					{
						nCursorPos--;
						aText.Erase( nCursorPos, 1 );
						bModify = TRUE;
					}
				}
				break;
			case KEY_ESCAPE:
				NotifyCloseEditWindow();
				return;

			case KEY_TAB:
				{
				BOOL bReverse = rKEvt.GetKeyCode().IsShift();
				VCEdit*	pEdit = pOwner->FindNext( bReverse );
				Window* pParent = pEditWin->GetParent();
				((VCSingleEditWin*)pEditWin)->bParentAlreadyClosing = TRUE;
				NotifyCloseEditWindow();
				// verhindern, dass Focus() das EditWin synchron loescht
				Cleanup();	// !!! pEditWin ist jetzt NULL !!!
				if( pEdit )
					pEdit->Focus();
				}
				return;

			case KEY_RETURN:
				{
					if( pOwner->pShell )
					{
						VCForm* pForm = pOwner->pShell->GetVCForm();
						if( pForm )
						{
							NotifyCloseEditWindow();
							pForm->ExecuteDefault();
						}
					}
				}
				return;

			default:
				if( rKEvt.GetCharCode() && !pOwner->bReadOnly )
				{
					if( aSelection.Len() )
					{
						aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
						bModify = TRUE;
					}

					if( (pOwner->nMaxLen && (aText.Len() < pOwner->nMaxLen)) || !pOwner->nMaxLen )
					{
						aText.Insert( rKEvt.GetCharCode(), (USHORT)aSelection.Min() );
						nCursorPos = (USHORT)aSelection.Min() + 1;
						bModify = TRUE;
					}

				}
		}
	}

	Selection aOldSelection( aSelection );
	if( bTravel )
	{
		if( (nAnchorPos==STRING_LEN) )
		{
			aSelection.Min() = nCursorPos;
			aSelection.Max() = nCursorPos;
		}
		else
		{
			if( nAnchorPos < nCursorPos )
			{
				aSelection.Min() = nAnchorPos;
				aSelection.Max() = nCursorPos;
			}
			else
			{
				aSelection.Min() = nCursorPos;
				aSelection.Max() = nAnchorPos;
			}
		}
		DrawText( pWindow );
		UpdateCursorPos( pWindow );
		ScrollToCursor( pWindow );
	}
	else if( bModify )
	{
		TextModified( pWindow );
	}
	if( aOldSelection != aSelection && (aSelection.Len() || aOldSelection.Len() ))
		pOwner->Selected();
}

Window* VCSingleEditEngine::GetEditWin() const
{
	return pEditWin;
}

void VCSingleEditEngine::Resize()
{
	InitEditWinPosSizeMap();
}

void VCSingleEditEngine::InitEditWinPosSizeMap()
{
	if( pEditWin )
	{
		Window* pWin = pEditWin->GetParent();
		Point aPosPixel = pWin->LogicToPixel( pOwner->aPos );
		Size  aSizePixel= pWin->LogicToPixel( pOwner->aSize );
		aSizePixel.Width()++;
		aSizePixel.Height()++;
		pEditWin->SetPosSizePixel( aPosPixel, aSizePixel );

		MapMode aMap = pWin->GetMapMode();
		aMap.SetOrigin( Point( -pOwner->aPos.X(), -pOwner->aPos.Y() ) );
		pEditWin->SetMapMode( aMap );
	}
}

void VCSingleEditEngine::PrepareClose()
{
}

long VCSingleEditEngine::GetSelectLen() const
{
	return aSelection.Len();
}

long VCSingleEditEngine::GetSelectStart()
{
	return aSelection.Min();
}

String VCSingleEditEngine::GetText()
{
	return aText;
}

void VCSingleEditEngine::Enable( BOOL )
{
}

void VCSingleEditEngine::SetFont( const Font& )
{
}

Size VCSingleEditEngine::GetOptSize( OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCSingleEditEngine,0);

	Font aTmpFont( pOut->GetFont() );
	pOut->SetFont( pOwner->aFont );
	Size aSize( pOut->GetTextSize( aText ));
	pOut->SetFont( aTmpFont );
	Size aPix( 2,3 );
	aPix = pOut->PixelToLogic( aPix );

	aSize.Width() += aPix.Width();
	aSize.Height() += aPix.Height();
	return aSize;
}

Size VCSingleEditEngine::GetOptSize( const Size& rCharXY, OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCSingleEditEngine,0);

	String aStr( "n");
	Font aTmpFont( pOut->GetFont() );
	pOut->SetFont( pOwner->aFont );
	Size aSize( pOut->GetTextSize(aStr ));

	aSize.Height() *= rCharXY.Height();

	if( rCharXY.Width() == 0 )
	{
		// max. Breite des vorhandenen Textes nehmen
		Size aMaxSize( pOut->GetTextSize( ((VCMultiEditEngine*)this)->GetText()));
		aSize.Width() = aMaxSize.Width();
	}
	else
	{
		/* bei Prop-Fonts 'n' als Referenzbreite benutzen
		FontPitch ePitch = pOwner->aFont.GetPitch();
		if( ePitch == PITCH_DONTKNOW )
		{
			FontMetric aMetric = pOut->GetFontMetric();
			ePitch = aMetric.GetPitch();
		}
		if( ePitch != PITCH_FIXED )
			aSize.Width() = aSize.Height();
		*/
		aSize.Width() *= rCharXY.Width();
	}

	Size aPix( 12,6 );
	aPix = pOut->PixelToLogic( aPix );

	aSize.Width() += aPix.Width();
	aSize.Height() += aPix.Height();

	pOut->SetFont( aTmpFont );

	return aSize;
}

Size VCSingleEditEngine::GetCharCountXY( OutputDevice* pOut) const
{
	DBG_CHKTHIS(VCSingleEditEngine,0);

	Size aSize( pOwner->aSize );
	Size aPix( 12,4 );
	aPix = pOut->PixelToLogic( aPix );
	aSize.Width() -= aPix.Width();
	aSize.Height() -= aPix.Height();

	String aStr("n");
	Font aTmpFont( pOut->GetFont() );
	pOut->SetFont( pOwner->aFont );
	Size aCharSize( pOut->GetTextSize( aStr ));

	/* 31.5.96: Bei Prop-Fonts 'n' als Referenzbreite benutzen
	   wg. HTML-Export
	FontPitch ePitch = pOwner->aFont.GetPitch();
	if( ePitch == PITCH_DONTKNOW )
	{
		FontMetric aMetric = pOut->GetFontMetric();
		ePitch = aMetric.GetPitch();
	}
	if( ePitch != PITCH_FIXED )
		// bei propfonts die hoehe als breite benutzen
		aSize.Width() /= aCharSize.Height();
	else
	*/
	aSize.Width() /= aCharSize.Width();
	aSize.Height() /= aCharSize.Height();

	pOut->SetFont( aTmpFont );
	return aSize;
}

void VCSingleEditEngine::Focus( Window* pWin )
{
	if( aDestroyEditWinTimer.IsActive())
		DestroyEditWin( &aDestroyEditWinTimer );
	if( !pEditWin )
		ShowEditWin( pWin );
}

void VCSingleEditEngine::Blur()
{
}

IMPL_LINK(VCSingleEditEngine,DestroyEditWin,Timer*,pTimer)
{
	pTimer->Stop(); // falls Handler direkt gerufen wurde
	ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
	// wurde es vielleicht schon vom Dtor der View geloescht?
	if( pImplEdit && pImplEdit->pWin == pEditWin )
	{
		DELETEZ(pEditWin);
		delete pImplEdit;
		SI_DLL()->SetActiveEdit(0);
	}
	pOwner->LostFocus();
	return 0;
}

BOOL VCSingleEditEngine::IsModified() const
{
	return bModified;
}

void VCSingleEditEngine::SetModifyFlag( BOOL bModify )
{
	bModified = bModify;
}

void VCSingleEditEngine::Cut()
{
	if( aSelection.Len() && !pOwner->bReadOnly )
	{
		Clipboard::CopyString( aText.Copy( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() ) );
		aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );
		nCursorPos = (USHORT)aSelection.Min();
		TextModified( pEditWin );
	}
}

void VCSingleEditEngine::Copy()
{
	if( aSelection.Len() )
		Clipboard::CopyString( aText.Copy( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() ) );
}

void VCSingleEditEngine::Paste()
{
	if( Clipboard::HasFormat( FORMAT_STRING ) && !pOwner->bReadOnly )
	{
		String aClip = Clipboard::PasteString();
		aClip.EraseAllChars( '\n' );
		aClip.EraseAllChars( '\r' );

		if( aSelection.Len() )
			aText.Erase( (USHORT)aSelection.Min(), (USHORT)aSelection.Len() );

		USHORT nPasteLen = aClip.Len();
		if( pOwner->nMaxLen && ((aText.Len() + aClip.Len()) > pOwner->nMaxLen ) )
			nPasteLen = pOwner->nMaxLen - aText.Len();

		aText.Insert( aClip, 0, nPasteLen, (USHORT)aSelection.Min() );
		nCursorPos = (USHORT)aSelection.Min() + nPasteLen;
		TextModified( pEditWin );
	}
}

void VCSingleEditEngine::TextModified( Window* pWindow )
{
	aSelection.Min() = nCursorPos;
	aSelection.Max() = nCursorPos;
	nAnchorPos = STRING_LEN;
	DrawText( pWindow );
	UpdateCursorPos( pWindow );
	ScrollToCursor( pWindow );
	pOwner->Modified();
	bModified = TRUE;
}

void VCSingleEditEngine::SetWordWrap( USHORT  )
{
}

Pointer VCSingleEditEngine::GetPointer( const Point& ) const
{
	return Pointer( POINTER_BEAM );
}

void VCSingleEditEngine::Cleanup( BOOL bAsyncSelfDestroyOfEditWin )
{
	aDestroyEditWinTimer.Stop();
	ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
	// wurde es vielleicht schon vom Dtor der View geloescht?
	if( pImplEdit && pImplEdit->pWin == pEditWin )
	{
		delete pImplEdit;
		SI_DLL()->SetActiveEdit(0);
	}
	if( !bAsyncSelfDestroyOfEditWin )
	{
		DELETEZ(pEditWin);
	}
	else
	{
		((VCSingleEditWin*)pEditWin)->DestroyYourself();
		pEditWin = 0;
	}
	pOwner->LostFocus();
}


/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/si/vcengin1.cxx,v 1.1.1.1 2000/09/29 11:02:39 mh Exp $

	  Source Code Control System - Update

	  $Log: vcengin1.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:39  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:45  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:48  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.78  2000/02/10 20:01:11  hr
	  #70473# changes for unicode ( patched by automated patchtool )
	
	  Revision 1.77  1998/01/22 11:13:22  PB
	  chg: includes
	

	  Rev 1.76   22 Jan 1998 12:13:22   PB
   chg: includes

	  Rev 1.75   21 Nov 1997 17:02:24   OV
   Headerumstellung

	  Rev 1.74   11 Nov 1997 17:35:32   OV
   #45459# Tab-Steuerung: Aktuelles Fenster zerstoert sich asynchron

	  Rev 1.73   06 Nov 1997 19:11:54   OV
   weitere VCL-Anpassungen

	  Rev 1.72   06 Nov 1997 12:45:56   OV
   VCL-Umstellungen

	  Rev 1.71   31 Oct 1997 17:14:22   OV
   VCL: Windows-Zeichenfunktionen auf allen Plattformen (364-inkompat); diverse VCL-Anpassungen

	  Rev 1.70   31 Oct 1997 10:51:56   OV
   #45259# Systemabhaengiger Code zum flackerfreien Hide des EditWins unter VCL deaktiviert

	  Rev 1.69   28 Oct 1997 09:56:52   OV
   #44276# KeyInput: Basisklasse rufen, wenn nicht verarbeitet

	  Rev 1.68   16 Oct 1997 18:26:30   OV
   #44716# Anpassungen UNIX

	  Rev 1.67   28 Jul 1997 16:47:06   MH
   chg: header

	  Rev 1.66   02 Jul 1997 14:07:08   OV
   #41064# OutputDevice auf Printer abfragen

	  Rev 1.65   30 Jun 1997 18:25:08   OV
   #41049# SetPos: EditWindow ebenfalls mitverschieben

	  Rev 1.64   25 Jun 1997 13:40:48   OV
   #40988# Aktives Edit async loeschen; richtiger Pointer; Cursor initial richtig positionieren

	  Rev 1.63   16 Jun 1997 14:12:54   OV
   #40726# GetOptSize: Rahmen staerker beruecksichtigen

	  Rev 1.62   29 May 1997 16:06:44   OV
   #35122# Wordwrap in Doc-Multiline-Edits

	  Rev 1.61   29 May 1997 09:11:24   TRI
   ICC brauchte cast

*************************************************************************/

#pragma SEG_EOFMODULE

