/*************************************************************************
 *
 *  $RCSfile: vcengin3.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SFX_SFXSIDS_HRC
#include <sfx2/sfxsids.hrc>
#endif

#pragma hdrstop

#define _SI_NOVCSPINBTENG
#define _SI_NOVCLBOXENG
#define _SI_NOVCDDWIN
#define _SI_NOVCCOMBOENG

#include "vcedit.hxx"
#include "vcengin.hxx"
#include "vceeng.hxx"
#include "vcdrwobj.hxx"
#include "vcdrview.hxx"
#ifndef _VCONT_HXX
#define _SFXHINT_HXX
#define _SFXSMPLHINT_HXX
#define _SFXBRDCST_HXX
#include "vcont.hxx"
#endif
#ifndef _VCFORM_HXX
#include "vcform.hxx"
#endif
#include "vctrls.hxx"
#include "vcictrls.hxx"
#include "vcattr.hxx"

#ifndef _SVX_SIVCDRMOD_HXX //autogen
#include <vcdrmod.hxx>
#endif

DBG_NAME(VCEditEngine);
DBG_NAMEEX(VCEditMapper);

SiEditSlotInterceptor::SiEditSlotInterceptor( const Link& rCallBackHdl )
	: SfxSlotInterceptor( &SFX_BINDINGS() ),
	aCallBackHdl( rCallBackHdl )
{
}


BOOL SiEditSlotInterceptor::Execute( USHORT nSID, SfxPoolItem**ppArgs )
{
	switch( nSID )
	{
		case SID_CUT:
		case SID_COPY:
		case SID_PASTE:
			aCallBackHdl.Call( (void*)nSID );
			return TRUE;
	}
	return FALSE;
}

SfxItemState SiEditSlotInterceptor::QueryState( USHORT nSID, SfxPoolItem* &rpState)
{
	switch( nSID )
	{
		case SID_CUT:
		case SID_COPY:
		case SID_PASTE:
			return SFX_ITEM_AVAILABLE;
	}
	return SfxSlotInterceptor::QueryState( nSID, rpState );
}

VCEditEngine::~VCEditEngine()
{
	DBG_DTOR(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->PrepareClose();
	delete pCurEdit;
}

VCEditEngine::VCEditEngine( VCControl* pSh, BOOL bMultiLineEdit ) :
	pShell(pSh),
	bBorder(FALSE),
	aBackColor(COL_WHITE),
	aForeColor( COL_BLACK ),
	bTransparent(FALSE),
	bMultiLine( bMultiLineEdit ),
	bMnemonic(FALSE),
	nAlignment(0),
	bActive(TRUE),
	bCenterVert(FALSE),
	bEnabled(TRUE),
	nMaxLen(0),
	bReadOnly(FALSE),
	nWordWrap( VCWORDWRAP_OFF )
{
	DBG_CTOR(VCEditEngine,0);
	pCurEdit = CreateEditEngine();
}

VCEditMapper* VCEditEngine::CreateEditEngine()
{
	if( bMultiLine )
		return new VCMultiEditEngine( this );
	else
		return new VCSingleEditEngine( this );
}

// static
void VCEditEngine::ConvertAmpersand( String* pStr )
{
	USHORT i;
	for (i=0; i < pStr->Len(); i++ )
	{
		if( (*pStr)[i] == '&' )
		{
			if( ((i+1) < pStr->Len()) && ((*pStr)[(USHORT)(i+1)] != '&') )
				(*pStr)[i] = '~';
			else
			if( ((i+1) < pStr->Len()) )
				pStr->Erase( i+1, 1 );
		}
	}
}

// static
void VCEditEngine::DeleteAmpersand( String* pStr )
{
	USHORT i;
	for(i=0; i < pStr->Len();)
	{
		if( ((*pStr)[i] == '&') )
		{
			pStr->Erase( i, 1 );
			if( (i < pStr->Len()) && ((*pStr)[i] == '&') )
				i++;
		}
		else
		if( ((*pStr)[i] == '~') )
		{
			pStr->Erase( i, 1 );
			if( (i < pStr->Len()) && ((*pStr)[i] == '~') )
				i++;
		}
		else
			i++;
	}
}

void VCEditEngine::Modified()
{
	DBG_CHKTHIS(VCEditEngine,0);
	if( aModifyHdlLink.IsSet() )
		aModifyHdlLink.Call( this );
}

void VCEditEngine::GotFocus()
{
	DBG_CHKTHIS(VCEditEngine,0);
	if( aGotFocusHdlLink.IsSet() )
		aGotFocusHdlLink.Call( this );
}

void VCEditEngine::LostFocus()
{
	DBG_CHKTHIS(VCEditEngine,0);
	if( aLostFocusHdlLink.IsSet() )
		aLostFocusHdlLink.Call( this );
}

void VCEditEngine::Selected()
{
	DBG_CHKTHIS(VCEditEngine,0);
	if( aSelectedHdlLink.IsSet() )
		aSelectedHdlLink.Call( this );
}

void VCEditEngine::SetMultiLine( BOOL bMulti )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	if( bMultiLine != bMulti )
	{
		bMultiLine = bMulti;
		String aText;
		aText = pCurEdit->GetText();
		pCurEdit->PrepareClose();
		delete pCurEdit;
		pCurEdit = CreateEditEngine();
		pCurEdit->SetText( aText );
	}
}

void VCEditEngine::SetText( const String& rText )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->SetText( rText );
}

String VCEditEngine::GetText()
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetText();
}

void VCEditEngine::SetSelectStart( long nStart )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->SetSelectStart( nStart );
}

long VCEditEngine::GetSelectStart()
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetSelectStart();
}

void VCEditEngine::SetSelectLen( long nLen )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->SetSelectLen( nLen );
}

long VCEditEngine::GetSelectLen() const
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetSelectLen();
}

String VCEditEngine::GetSelectText() const
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetSelectText();
}

void VCEditEngine::SetSelectText( const String& rText )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->SetSelectText( rText );
}

void VCEditEngine::Paint( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->Paint( pOut );
}

void VCEditEngine::MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->MouseButtonDown(pWindow, rMEvt );
}

void VCEditEngine::MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->MouseButtonUp(pWindow, rMEvt );
}

void VCEditEngine::MouseMove( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->MouseMove(pWindow, rMEvt );
}

void VCEditEngine::KeyInput( Window* pWindow, const KeyEvent& rKEvt )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	pCurEdit->KeyInput(pWindow, rKEvt );
}

Window* VCEditEngine::GetEditWin() const
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetEditWin();
}

void VCEditEngine::SetPos( const Point& rPos )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	if( aPos != rPos )
	{
		aPos = rPos;
		pCurEdit->Resize();
	}
}

void VCEditEngine::SetSize( const Size& rSize )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	if( aSize != rSize )
	{
		aSize = rSize;
		pCurEdit->Resize();
	}
}

void VCEditEngine::Enable( BOOL bEnable )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	if( bEnabled != bEnable )
	{
		bEnabled = bEnable;
		pCurEdit->Enable( bEnable );
	}
}

void VCEditEngine::SetFont( const Font& rFont )
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	if( rFont != aFont )
	{
		aFont = rFont;
		pCurEdit->SetFont( rFont );
	}
}

Size VCEditEngine::GetOptSize( OutputDevice* pOut) const
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetOptSize( pOut );
}

Size VCEditEngine::GetOptSize( const Size& rCharXY, OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetOptSize( rCharXY, pOut );
}

Size VCEditEngine::GetCharCountXY( OutputDevice* pOut) const
{
	DBG_CHKTHIS(VCEditEngine,0);
	DBG_CHKOBJ(pCurEdit,VCEditMapper,0);
	return pCurEdit->GetCharCountXY( pOut );
}

VCEdit* VCEditEngine::FindNext( BOOL bReverse )
{
	DBG_CHKTHIS(VCEditEngine,0);
	if( !pShell ) // ComboBox?
		return 0;
	if( !pShell->IsEnabled() ) // Endlosschleife verhindern
		return 0;
	VCManager* pManager = (VCManager*)(pShell->GetParent());
	ULONG nPos = pManager->GetChildPos( pShell );
	ULONG nCount = pManager->GetChildCount();
	VCControl* pResult = 0;
	if( nPos == LIST_ENTRY_NOTFOUND )
		return 0;
	do
	{
		if( bReverse )
		{
			if( nPos )
				nPos--;
			else
				nPos = nCount - 1;
		}
		else
		{
			nPos++;
			if( nPos == nCount )
				nPos = 0;
		}
		pResult = pManager->GetChildByPos( nPos );
	} while( (!pResult->IsEnabled()) ||
			 (pResult->Type() != pShell->Type() && pResult != pShell ) );
	if( pResult == pShell )
		return 0;
	return (VCEdit*)pResult;
}

void VCEditEngine::Focus()
{
	DBG_CHKTHIS(VCEditEngine,0);
	MakeVisible( (VCEdit*)pShell );
	SdrObject* pSdrObj = (SdrObject*)pShell->GetDrawObject();
	DBG_ASSERT(pSdrObj,"SdrObject not set");
	SdrModel* pModel = pSdrObj->GetModel();
	DBG_ASSERT(pModel,"Model not set");
	if( ((VCDrawModel*)pModel)->GetCurDocViewWin() )
		pCurEdit->Focus( ((VCDrawModel*)pModel)->GetCurDocViewWin() );
}

void VCEditEngine::Blur()
{
	DBG_CHKTHIS(VCEditEngine,0);
	pCurEdit->Blur();
}

//static
void VCEditEngine::MakeVisible( VCEdit* pEdit )
{
	DBG_ASSERT(pEdit,"Edit not set");
	SdrObject* pSdrObj = (SdrObject*)pEdit->GetDrawObject();
	DBG_ASSERT(pSdrObj,"SdrObject not set");
	SdrModel* pModel = pSdrObj->GetModel();
	DBG_ASSERT(pModel,"Model not set");
	Window* pWin = ((VCDrawModel*)pModel)->GetCurDocViewWin();

	USHORT nLsAnz=pModel->GetListenerCount();
	for (USHORT nLsNum=0; nLsNum<nLsAnz; nLsNum++)
	{
		SfxListener* pLs=pModel->GetListener(nLsNum);
		VCDrawView* pView=PTR_CAST(VCDrawView,pLs);
		if( pView != NULL )
		{
			USHORT nPvAnz=pView->GetPageViewCount();
			for( USHORT nPvNum=0; nPvNum<nPvAnz; nPvNum++ )
			{
				USHORT nOutAnz = pView->GetWinCount();
				for( USHORT nOutNum=0; nOutNum<nOutAnz; nOutNum++)
				{
					OutputDevice* pOut = pView->GetWin(nOutNum);
					if( pOut == pWin )
					{
						pView->MakeVisible( pSdrObj->GetBoundRect(),*pWin );
						return;
					}
				}
			}
		}
	}
}


BOOL VCEditEngine::IsModified() const
{
	DBG_CHKTHIS(VCEditEngine,0);
	return pCurEdit->IsModified();
}

void VCEditEngine::SetModifyFlag( BOOL bModified )
{
	DBG_CHKTHIS(VCEditEngine,0);
	pCurEdit->SetModifyFlag( bModified );
}

void VCEditEngine::SetWordWrap( USHORT nMode )
{
	DBG_CHKTHIS(VCEditEngine,0);
	if( nMode != nWordWrap )
	{
		nWordWrap = nMode;
		pCurEdit->SetWordWrap( nMode );
	}
}

Pointer VCEditEngine::GetPointer( const Point& rPos ) const
{
	return pCurEdit->GetPointer( rPos );
}


/***************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/si/vcengin3.cxx,v 1.1.1.1 2000/09/29 11:02:39 mh Exp $

	  Source Code Control System - Update

	  $Log: vcengin3.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:39  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:45  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:48  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.28  1998/02/05 12:04:44  HJS
	  includes
	

	  Rev 1.27   05 Feb 1998 13:04:44   HJS
   includes

	  Rev 1.26   12 Jan 1998 11:22:32   OV
   46509# GetCurDocViewWindow auf 0 testen

	  Rev 1.25   01 Dec 1997 09:57:28   TJ
   incude

	  Rev 1.24   24 Nov 1997 16:08:38   OV
   #45698# Disable Edits nicht per Tab anspringbar

	  Rev 1.23   21 Nov 1997 17:01:44   OV
   Headerumstellung

	  Rev 1.22   28 Jul 1997 16:43:12   MH
   chg: header

	  Rev 1.21   25 Jun 1997 13:40:10   OV
   #40988# Aktives Edit async loeschen; richtiger Pointer; Cursor initial richtig positionieren

	  Rev 1.20   29 May 1997 16:06:48   OV
   #35122# Wordwrap in Doc-Multiline-Edits

	  Rev 1.19   28 May 1997 12:23:42   OV
   #38688# SlotInterceptor fuer Clipboard-Events, wenn Ctrl Fokus hat

	  Rev 1.18   26 May 1997 14:08:00   OV
   #40170# FindNext: pShell auf 0 testen (fuer SimpleComboBox)

	  Rev 1.17   02 Apr 1997 21:54:32   MH
   chg: headerfile umstellungen

	  Rev 1.16   25 Mar 1997 16:13:10   OV
   Cursortravelling jetzt unabhaengig von aktueller Form

	  Rev 1.15   20 Jan 1997 17:10:36   OV
   Neu: IsModidied, SetModifyFlag

	  Rev 1.14   08 Jan 1997 17:31:04   OV
   Diverse Bugfixes

	  Rev 1.13   13 Dec 1996 17:54:06   AB
   JavaScript-Probleme

	  Rev 1.12   04 Dec 1996 11:30:42   AB
   JavaScript

	  Rev 1.11   03 Dec 1996 14:45:24   OV
   Anpassungen JavaScript

	  Rev 1.10   13 Nov 1996 12:31:54   OV
   Mussaenderungen 345

	  Rev 1.9   28 Oct 1996 16:32:14   OV
   Edits bei Aktivierung sichtbar machen

	  Rev 1.8   30 May 1996 12:50:22   TRI
   CLOOKS

	  Rev 1.7   29 May 1996 12:04:36   OV
   Erweiterungen,Bugfixes,Aufraeumarbeiten

	  Rev 1.6   23 May 1996 13:39:16   OV
   nochmal Clooks

	  Rev 1.5   23 May 1996 13:24:38   OV
   Clooks

	  Rev 1.4   10 May 1996 12:53:06   OV
   Erweiterungen,Bugfixes

	  Rev 1.3   09 May 1996 19:26:10   OV
   Tabsteuerung

	  Rev 1.2   09 May 1996 15:19:10   OV
   Tabsteuerung,Bugfixes

	  Rev 1.1   07 Mar 1996 16:49:36   OV
   NEU: GetOptSize u.a

	  Rev 1.0   06 Mar 1996 12:56:32   OV
   Initial revision.

*************************************************************************/

#pragma SEG_EOFMODULE
