/*************************************************************************
 *
 *  $RCSfile: vcengin4.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef VCL
#ifndef _SYSDEPEN_HXX
#include <vcl/sysdep.hxx>
#endif
#endif
#ifndef _SHL_HXX
#include <tools/shl.hxx>
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _ACCEL_HXX
#include <vcl/accel.hxx>
#endif
#ifndef _COLOR_HXX
#include <vcl/color.hxx>
#endif
#ifndef _WRKWIN_HXX
#include <vcl/wrkwin.hxx>
#endif
#ifndef _SFX_SFXSIDS_HRC
#include <sfx2/sfxsids.hrc>
#endif
#ifndef _SFXITEMSET_HXX
#include <svtools/itemset.hxx>
#endif
#ifndef _SV_METRIC_HXX //autogen
#include <vcl/metric.hxx>
#endif
#pragma hdrstop

#include "vcedit.hxx"
#include "vcengin.hxx"
#include "vceeng.hxx"
#include "sidll.hxx"
#include <svx/editeng.hxx>
#include <svx/editview.hxx>
#include <svx/editdata.hxx>
#include <svx/editstat.hxx>
#include "vctrls.hxx"
#include "vcattr.hxx"

DBG_NAME(VCMultiEditEngine);
DBG_NAMEEX(VCEditMapper);
DBG_NAMEEX(EditEngine);
DBG_NAMEEX(EditView);

#define FLAG_FIRST_PAINT	0x0001
#define FLAG_VISAREA_VALID	0x0002
#define FLAG_MEVT_ISPIXEL	0x0004
#define FLAG_SCROLL_PAINT	0x0008
#define FLAG_CAPTURED_HSBAR 0x0010
#define FLAG_CAPTURED_VSBAR 0x0020
#define FLAG_CURSOR_POS_VALID 0x0040
#define FLAG_IN_MOUSEDOWN 	0x0080

class VCMultiEditWin : public Window
{
protected:
	VCMultiEditEngine* pEditEngine;
	SiEditSlotInterceptor* pInterceptor;
	void	LoseFocus();
	void	GetFocus();
	void	Paint( const Rectangle& rRect );
	void	MouseButtonDown( const MouseEvent& rMEvt );
	void	MouseButtonUp( const MouseEvent& rMEvt );
	void	MouseMove( const MouseEvent& rMEvt );
	void	KeyInput( const KeyEvent& rKEvt );
			DECL_LINK(InterceptorHdl, void*);

			DECL_LINK(VCMultiEdEvent, void* );

public:
	BOOL	bParentAlreadyClosing;
	BOOL 	bKeyInputCallBaseClass;

	VCMultiEditWin( VCMultiEditEngine*, Window* pParent, WinBits nStyle );
	virtual ~VCMultiEditWin();
	void	DestroyYourself();
};


//------------------------------------------------------------------------

VCMultiEditWin::VCMultiEditWin( VCMultiEditEngine* pEngine, Window* pParent, WinBits nStyle ) :
	Window( pParent, nStyle ),
	pEditEngine( pEngine ),
	pInterceptor(0)
{
	bParentAlreadyClosing = FALSE;
	Color aColor( pParent->GetBackgroundBrush().GetColor() );
	Font aFont( GetFont());
	aFont.SetFillColor(aColor );
	SetFont( aFont );
}

VCMultiEditWin::~VCMultiEditWin()
{
	DELETEZ(pInterceptor);
	if( pEditEngine && pEditEngine->pEditWin == this )
		pEditEngine->pEditWin = 0;
}

void VCMultiEditWin::DestroyYourself()
{
	pEditEngine = 0;
	Application::PostUserEvent(LINK(this,VCMultiEditWin,VCMultiEdEvent),(void*)0);
}

IMPL_LINK(VCMultiEditWin, VCMultiEdEvent, void*, EMPTY_ARG )
{
	delete this;
	return 0;
}

void __EXPORT VCMultiEditWin::LoseFocus()
{
	DELETEZ(pInterceptor);
	if( pEditEngine && !bParentAlreadyClosing )
	{
		bParentAlreadyClosing = TRUE;
		pEditEngine->NotifyCloseEditWindow();
	}
}

void __EXPORT VCMultiEditWin::GetFocus()
{
	if( pEditEngine && !bParentAlreadyClosing )
	{
		pInterceptor = new SiEditSlotInterceptor( LINK(this, VCMultiEditWin, InterceptorHdl));
	}
}

IMPL_LINK(VCMultiEditWin, InterceptorHdl, void*, pPtr)
{
	if( pEditEngine && !bParentAlreadyClosing )
	{
		USHORT nSID = (USHORT)(ULONG)pPtr;
		switch( nSID )
		{
			case SID_CUT: pEditEngine->Cut(); break;
			case SID_COPY: pEditEngine->Copy(); break;
			case SID_PASTE: pEditEngine->Paste(); break;
		}
	}
	return 0;
}

//------------------------------------------------------------------------
void __EXPORT VCMultiEditWin::Paint( const Rectangle& rRect )
{
	if( bParentAlreadyClosing )
		return;
	if( pEditEngine )
	{
		DBG_ASSERT(pEditEngine->pEditView,"No EditView");
		if( pEditEngine->aResizeRect.IsOver( rRect))
			pEditEngine->DrawResizeRect( this );
		if( pEditEngine->aClientRect.IsOver( rRect ))
			pEditEngine->pEditView->Paint( rRect );
		if( pEditEngine->aHSBarRect.IsOver( rRect ))
			pEditEngine->aHScroll.Paint( this );
		if( pEditEngine->aVSBarRect.IsOver( rRect ))
			pEditEngine->aVScroll.Paint( this );
		pEditEngine->DrawBorder( this );
	}
}


//------------------------------------------------------------------------
void __EXPORT VCMultiEditWin::MouseButtonDown( const MouseEvent& rMEvt )
{
	if( pEditEngine )
	{
		pEditEngine->nFlags |= FLAG_MEVT_ISPIXEL;
		pEditEngine->MouseButtonDown( this, rMEvt );
		pEditEngine->nFlags &= (~FLAG_MEVT_ISPIXEL);
	}
}


//------------------------------------------------------------------------
void __EXPORT VCMultiEditWin::MouseButtonUp( const MouseEvent& rMEvt )
{
	if( pEditEngine )
	{
		pEditEngine->nFlags |= FLAG_MEVT_ISPIXEL;
		pEditEngine->MouseButtonUp( this, rMEvt );
		pEditEngine->nFlags &= (~FLAG_MEVT_ISPIXEL);
	}
}


//------------------------------------------------------------------------
void __EXPORT VCMultiEditWin::MouseMove( const MouseEvent& rMEvt )
{
	if( pEditEngine )
	{
		pEditEngine->nFlags |= FLAG_MEVT_ISPIXEL;
		pEditEngine->MouseMove( this, rMEvt );
		pEditEngine->nFlags &= (~FLAG_MEVT_ISPIXEL);
	}
}

//------------------------------------------------------------------------
void __EXPORT VCMultiEditWin::KeyInput( const KeyEvent& rKEvt )
{
	if( pEditEngine )
	{
		bKeyInputCallBaseClass = FALSE;
		pEditEngine->KeyInput( this, rKEvt );
		if( bKeyInputCallBaseClass )
			Window::KeyInput( rKEvt );
	}
}

static void HideWindow( Window* pWindow )
{
#ifndef VCL

#if defined(WIN) || defined(WNT)
	Window* pParent = pWindow->GetParent();
	if( pParent->GetBackgroundBrush().GetStyle() == BRUSH_NULL )
	{
		pParent->Update();
		pWindow->Hide();
		HWND hWin = Sysdepen::GethWnd( *pParent );
		ValidateRect( hWin, 0 );
	}
	else
		pWindow->Hide();
#else
	pWindow->Hide();
#endif

#else

	pWindow->Hide();

#endif
}

//------------------------------------------------------------------------
String GetTextFromEditEngine( const EditEngine* pEditEng, USHORT nWordWrap )
{
	return pEditEng->GetText( "\n" );
}


//------------------------------------------------------------------------
VCMultiEditEngine::VCMultiEditEngine( VCEditEngine* pTheOwner ) :
	pOwner( pTheOwner ),
	pEditWin(NULL),
	bCloseEditWin(FALSE),
	aVScroll(),
	aHScroll( TRUE ),
	pEditEngine(0),
	pEditView(0),
	bModified( FALSE )
{
	DBG_CTOR(VCMultiEditEngine,0);
	nFlags = FLAG_FIRST_PAINT;
	aHScroll.SetScrollHdl( LINK(this,VCMultiEditEngine,HorScrollHdl));
	aVScroll.SetScrollHdl( LINK(this,VCMultiEditEngine,VerScrollHdl));
	aCleanupTimer.SetTimeout( 15000 );
	aCleanupTimer.SetTimeoutHdl(LINK(this,VCMultiEditEngine,TimeoutHdl));
	aDestroyEditWinTimer.SetTimeoutHdl(LINK(this,VCMultiEditEngine,DestroyEditWin));
	aDestroyEditWinTimer.SetTimeout(0);
}

//------------------------------------------------------------------------
VCMultiEditEngine::~VCMultiEditEngine()
{
	aCleanupTimer.Stop();
	ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
	if( pEditWin )
	{
		if(pImplEdit && pImplEdit->pWin == pEditWin)
			delete pEditWin;
	}
	if( pImplEdit )
	{
		delete pImplEdit;
		SI_DLL()->SetActiveEdit(0);
	}
	if( pEditView )
		pEditView->SetWindow(0);
	aCleanupTimer.Stop();
	DBG_DTOR(VCMultiEditEngine,0);
	if( pEditEngine && pEditView )
	{
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		DBG_CHKOBJ(pEditView,EditView,0);
	}
	delete pEditView;
	delete pEditEngine;
}

//------------------------------------------------------------------------
#ifdef VCL

void VCMultiEditEngine::PositionScrollBars(OutputDevice* pOut )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	long nScrollWidth = VCScrollBarEngine::GetOptWidth( pOut, TRUE );
	long nScrollHeight = VCScrollBarEngine::GetOptWidth( pOut, FALSE );
	Size aPix = pOut->PixelToLogic( Size(1,1) );
	BOOL bHorScrollBar = pOwner->GetWordWrap() == VCWORDWRAP_OFF ? TRUE : FALSE;
	Rectangle aAllRect( pOwner->aPos, pOwner->aSize );
	long nBorder = aPix.Width() * 2;

	aAllRect.Left() += nBorder;
	aAllRect.Right() -= nBorder;
	aAllRect.Top() += nBorder;
	aAllRect.Bottom() -= nBorder;

	aClientRect = aAllRect;

	// Offsets fuer die EditView
	aClientRect.Right() -= aPix.Width();
	aClientRect.Bottom() -= aPix.Height();

	// ScrollBars abziehen
	aClientRect.Right() -= nScrollWidth;
	if( bHorScrollBar )
		aClientRect.Bottom() -= nScrollHeight;

	// das von den Scrollbars in die Zange genommene Rechteck
	if( bHorScrollBar )
	{
		aResizeRect = aAllRect;
		aResizeRect.Left() = aResizeRect.Right() - nScrollWidth + aPix.Width();
		aResizeRect.Top() = aResizeRect.Bottom() - nScrollHeight + aPix.Height();
	}
	else
		aResizeRect.SetSize( Size(0,0) );

	// horizontale Scrollbar
	if( bHorScrollBar )
	{
		aHSBarRect = aAllRect;
		aHSBarRect.Top() = aHSBarRect.Bottom() - nScrollHeight;
		aHSBarRect.Right() -= nScrollWidth;
	}
	else
		aHSBarRect.SetSize( Size(0,0) );
	aHScroll.SetPos( aHSBarRect.TopLeft() );
	aHScroll.SetSize( aHSBarRect.GetSize() );

	// vertikale Scrollbar
	aVSBarRect = aAllRect;
	aVSBarRect.Left() = aVSBarRect.Right() - nScrollWidth;
	if( bHorScrollBar )
		aVSBarRect.Bottom() -= nScrollHeight;
	aVScroll.SetPos( aVSBarRect.TopLeft() );
	aVScroll.SetSize( aVSBarRect.GetSize() );

}

#else

void VCMultiEditEngine::PositionScrollBars(OutputDevice* pOut )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	long nScrollWidth = VCScrollBarEngine::GetOptWidth( pOut, TRUE );
	long nScrollHeight = VCScrollBarEngine::GetOptWidth( pOut, FALSE );
	Size aSizePix( 1,1 );
	aSizePix = pOut->PixelToLogic( aSizePix );
	Point aPix( aSizePix.Width(), aSizePix.Height());
	BOOL bHorScrollBar = pOwner->GetWordWrap() == VCWORDWRAP_OFF ? TRUE : FALSE;

	Rectangle aAllRect( pOwner->aPos, pOwner->aSize );

	// hier wird die EditView reingepackt
	aClientRect = aAllRect;
	// die View soll nicht in den Rahmen malen
	aClientRect.Top() += aPix.Y();
	aClientRect.Left() += aPix.X();
	// Platz fuer Scrollbars lassen (zusaetzliche Offsets wg. EditView)
	aClientRect.Right() -= (nScrollWidth+1*aPix.X());
	if( bHorScrollBar )
		aClientRect.Bottom() -= (nScrollHeight+1*aPix.Y());
	else
		aClientRect.Bottom() -= 1*aPix.Y();

	// das von den Scrollbars in die Zange genommene Rechteck
	if( bHorScrollBar )
	{
		aResizeRect = aAllRect;
		aResizeRect.Left() = aResizeRect.Right() - nScrollWidth + aPix.X();
		aResizeRect.Top() = aResizeRect.Bottom() - nScrollHeight + aPix.Y();
		aResizeRect.Bottom() -= aPix.Y();
	}
	else
		aResizeRect.SetSize( Size(0,0) );

	// horizontale Scrollbar
	if( bHorScrollBar )
	{
		aHSBarRect = aAllRect;
		aHSBarRect.Top() = aHSBarRect.Bottom() - nScrollHeight;
		aHSBarRect.Right() -= nScrollWidth;
	}
	else
		aHSBarRect.SetSize( Size(0,0) );
	aHScroll.SetPos( aHSBarRect.TopLeft() );
	aHScroll.SetSize( aHSBarRect.GetSize() );

	// vertikale Scrollbar
	aVSBarRect = aAllRect;
	aVSBarRect.Left() = aVSBarRect.Right() - nScrollWidth;
	if( bHorScrollBar )
		aVSBarRect.Bottom() -= nScrollHeight;
	aVScroll.SetPos( aVSBarRect.TopLeft() );
	aVScroll.SetSize( aVSBarRect.GetSize() );
}

#endif

//------------------------------------------------------------------------
void VCMultiEditEngine::NotifyCloseEditWindow()
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_ASSERT(pEditWin,"No edit window");
	DBG_CHKOBJ(pEditView,EditView,0);
	DBG_CHKOBJ(pEditEngine,EditEngine,0);

	ESelection aSel( pEditView->GetSelection() );
	aSel.Adjust();
	nCursorPara = aSel.nEndPara;
	nCursorPos = aSel.nEndPos;
	nFlags |= FLAG_CURSOR_POS_VALID;

	ESelection aEmptySel;
	ULONG nCtrlWord = pEditView->GetControlWord();
	ULONG nNewCtrlWord = nCtrlWord;
	nNewCtrlWord &= ~EV_CNTRL_AUTOSCROLL;
	pEditView->SetControlWord( nNewCtrlWord );
	pEditView->SetSelection( aEmptySel );
	pEditView->SetControlWord( nCtrlWord );

	aText = GetTextFromEditEngine( pEditEngine, pOwner->GetWordWrap() );
	aDocVisArea = pEditView->GetVisArea();
	nFlags |= FLAG_VISAREA_VALID;
	pEditView->HideCursor();

	// Bei/Nach SetWindow(0) nimmt die EditView nicht ihren Cursor aus
	// dem Fenster. Nach dem Loeschen der EditView enthaelt das Fenster
	// also einen geloeschten Cursor
	if( pEditView->GetWindow() &&
		pEditView->GetWindow()->GetCursor() == pEditView->GetCursor() )
	{
		pEditView->GetWindow()->SetCursor( 0 );
	}
	pEditView->SetWindow(0);
	DBG_ASSERT(pEditEngine->GetViewCount()==1,"bad view count");

	::HideWindow( pEditWin );
	// das Fenster koennte im LoseFocus stehen, deshalb asynchron loeschen
	aDestroyEditWinTimer.Start();
}

//------------------------------------------------------------------------
Rectangle VCMultiEditEngine::GetTextRect( OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_ERROR("GetTextRect not impl");

	Size aPixSize = pOut->PixelToLogic( Size( 1, 1 ) );
	Rectangle aTextRect( pOwner->aPos, pOwner->aSize );
	long nFact = 2;
	if( pOwner->bBorder )
		nFact++;
	aTextRect.Left()  += aPixSize.Width()  * nFact;
	aTextRect.Top()   += aPixSize.Height() * nFact;
	aTextRect.Right() -= aPixSize.Width()  * nFact;
	aTextRect.Bottom()-= aPixSize.Height() * nFact;
	return aTextRect;
}

//------------------------------------------------------------------------
void VCMultiEditEngine::DrawBackground( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( pOwner->bTransparent )
		return;
	Brush aBrush( pOwner->aBackColor );
	Brush aOldBrush = pOut->GetFillInBrush();
	pOut->SetFillInBrush( aBrush );
	Pen aOldPen	= pOut->GetPen( );
	pOut->SetPen( Pen( PEN_NULL ) );
	pOut->DrawRect( aClientRect );
	pOut->SetPen( aOldPen );
	pOut->SetFillInBrush( aOldBrush );
}

//------------------------------------------------------------------------

#ifndef VCL

void VCMultiEditEngine::DrawBorder( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( !pOwner->bBorder )
		return;

#if defined UNX
	Pen aOldPen = pOut->GetPen();

	Color a3DShadowColor( COL_BLACK ); //COL_3DSHADOW );
	Color a3DLightColor( COL_BLACK ); //COL_3DLIGHT );
	Pen a3DShadowPen2( a3DShadowColor, 1 );
	Pen a3DLightPen2( a3DLightColor, 1 );

	Size aSize = pOwner->aSize; // aClientRect.GetSize();
	Point aPos = pOwner->aPos; // aClientRect.TopLeft();
/*
	Size aSize1 = pOut->PixelToLogic( Size( 1, 1 ) );
	Size aSize2 = pOut->PixelToLogic( Size( 2, 2 ) );
	aSize.Width() += aSize2.Width();
	aSize.Height() += aSize2.Height();
	aPos.X() -= aSize1.Width();
	aPos.Y() -= aSize1.Height();
*/

	pOut->SetPen( a3DShadowPen2 );
	// vertical shadow
	pOut->DrawLine( aPos,
					Point( aPos.X(), aPos.Y()+aSize.Height() ) );
	// horizontal shadow
	pOut->DrawLine( aPos,
					Point( aPos.X()+aSize.Width(), aPos.Y() ) );
	pOut->SetPen( a3DLightPen2 );
	// vertical anti-shadow
	pOut->DrawLine( Point( aPos.X()+aSize.Width(), aPos.Y()+1 ),
					Point( aPos.X()+aSize.Width(),
						   aPos.Y()+aSize.Height() ) );
	// horizontal anti-shadow
	pOut->DrawLine( Point( aPos.X()+1, aPos.Y()+aSize.Height() ),
					Point( aPos.X()+aSize.Width(), aPos.Y()+aSize.Height() ) );

	pOut->SetPen( aOldPen );
#else
#ifdef OS2
		Color aCol( COL_3DSHADOW );
#else
		Color aCol;
#endif

	Pen   aPen ( aCol );
	Pen   aOldPen	= pOut->GetPen( );
	pOut->SetPen( aPen );
	Brush aOldBrush = pOut->GetFillInBrush( );
	pOut->SetFillInBrush( Brush( BRUSH_NULL ) );

	pOut->DrawRect( Rectangle( pOwner->aPos, pOwner->aSize ) );

	pOut->SetPen( aOldPen );
	pOut->SetFillInBrush( aOldBrush );
#endif
}

#else

void VCMultiEditEngine::DrawBorder( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( !pOwner->bBorder )
		return;

	SiDrawBorder_Impl( pOut, pOwner->aPos, pOwner->aSize );
/*
	Color aCol;
	Pen aPen( aCol );
	Pen aOldPen( pOut->GetPen() );
	pOut->SetPen( aPen );
	Brush aOldBrush = pOut->GetFillInBrush( );
	pOut->SetFillInBrush( Brush( BRUSH_NULL ) );
	pOut->DrawRect( Rectangle( pOwner->aPos, pOwner->aSize ) );
	pOut->SetPen( aOldPen );
	pOut->SetFillInBrush( aOldBrush );
*/
}
#endif

//------------------------------------------------------------------------
void VCMultiEditEngine::DrawResizeRect( OutputDevice* pOut )
{
#if !defined(UNX) || defined(VCL)
	DBG_CHKTHIS(VCMultiEditEngine,0);
	Color aCol( COL_SCROLLBAR );
	Brush aBrush( aCol );
	Brush aOldBrush = pOut->GetFillInBrush();
	pOut->SetFillInBrush( aBrush );
	Pen aOldPen	= pOut->GetPen( );
	pOut->SetPen( Pen( PEN_NULL ) );
	pOut->DrawRect( aResizeRect );
	pOut->SetPen( aOldPen );
	pOut->SetFillInBrush( aOldBrush );
#endif
}

//------------------------------------------------------------------------
void VCMultiEditEngine::ShowEditWin( Window* pWin )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	// wir muessen schon mal gepaintet haben, da sonst nix initialisiert ist
	DBG_ASSERT(!(nFlags&FLAG_FIRST_PAINT),"Not painted!");
	if( !pEditWin )
	{
		aCleanupTimer.Stop();
		pEditWin = new VCMultiEditWin( this, pWin, 0 );
		pEditWin->SetBackgroundBrush( pOwner->GetBackColor() );
		// aktives Window im SiDLL-Objekt verankern, damit die
		// VCDrawView beim Scrollen dessen Position anpassen kann
		ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
		if( pImplEdit )
		{
			delete pImplEdit->pWin;
			delete pImplEdit;
		}
		pImplEdit = new ImplActiveEdit;
		pImplEdit->pThis = pOwner;
		pImplEdit->pWin = pEditWin;
		SI_DLL()->SetActiveEdit( pImplEdit );

		InitEditWinPosSizeMap();

		// Multiline-Edit mit Maltes EditEngine
		SetupMultiEdit( pEditWin, FALSE );

		// nix flimmern
		Brush aOldBrush = pEditWin->GetBackgroundBrush();
		pEditWin->SetBackgroundBrush( Brush( BRUSH_NULL ) );
		pEditWin->Show();
		pEditWin->Update();
		pEditWin->SetBackgroundBrush( aOldBrush );

		ULONG nOldWord, nWord;
		if( nFlags & FLAG_IN_MOUSEDOWN )
		{
			nOldWord = pEditView->GetControlWord();
			nWord = nOldWord;
			nWord &= ~EV_CNTRL_AUTOSCROLL;
			pEditView->SetControlWord( nWord );
		}
		if( nFlags & FLAG_CURSOR_POS_VALID )
		{
			ESelection aSel( nCursorPara, nCursorPos, nCursorPara, nCursorPos );
			pEditView->SetSelection( aSel );
		}
		pEditView->ShowCursor();
		if( nFlags & FLAG_IN_MOUSEDOWN )
			pEditView->SetControlWord( nOldWord );


		pEditWin->GrabFocus();
		pOwner->GotFocus();
	}
}

//------------------------------------------------------------------------
void VCMultiEditEngine::SetupMultiEdit( OutputDevice* pOut, BOOL bPrint )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_ASSERT(pOut,"No output device");
	// wir muessen schon mal gepaintet haben, da sonst nix initialisiert ist
	DBG_ASSERT(!(nFlags&FLAG_FIRST_PAINT),"Not painted!");

	SiDLL* pDll = SI_DLL();
	// ein globaler Pool fuer alle EditEngines (Pool wird nicht gespeichert)
	if( !pDll->pEditPool )
		pDll->pEditPool = EditEngine::CreatePool();

	if( bPrint )
	{
		// das Multi-Edit soll nur kurz gepaintet werden
		// -> die Paint-Engine nehmen
		EditEngine* pCurEditEngine = pDll->pPaintEngine;
		if( !pCurEditEngine )
		{
			pCurEditEngine = new EditEngine;
			ULONG nWord = pCurEditEngine->GetControlWord();
			nWord &= ~(EE_CNTRL_DOIDLEFORMAT|EE_CNTRL_PASTESPECIAL);
			pCurEditEngine->SetControlWord( nWord );
			pCurEditEngine->SetPool( pDll->pEditPool );
			pDll->pPaintEngine = pCurEditEngine;
		}
		else
		{
			DBG_CHKOBJ(pCurEditEngine,EditEngine,0);
			DBG_ASSERT(pCurEditEngine->GetViewCount()==0,"bad view count");
		}
		if( pCurEditEngine->GetRefDevice() != pOut )
			pCurEditEngine->SetRefDevice( pOut );
		SfxItemSet aSet( pCurEditEngine->GetEmptyItemSet() );
		EditEngine::SetFontInfoInItemSet( aSet, pOwner->aFont );
		pCurEditEngine->SetDefaults( aSet );
		if( pOwner->GetWordWrap() == VCWORDWRAP_OFF )
		{
			Size aPaperSizePixel( 1280, 1024 );
			pCurEditEngine->SetPaperSize( pOut->PixelToLogic( aPaperSizePixel ) );
		}
		else
		{
			pCurEditEngine->SetPaperSize( aClientRect.GetSize() );
		}
		pCurEditEngine->SetText( aText );
	}
	else
	{
		// jemand will im Multi-Edit rumschreiben oder das nichtaktive
		// Edit wird ueber die ScrollBars gescrollt
		if( !pEditView || !pEditEngine )
		{
			DBG_ASSERT(!pEditView && !pEditEngine,"Edit inkonsistent");
			pEditEngine = new EditEngine;
			ULONG nWord = pEditEngine->GetControlWord();
			nWord &= ~(EE_CNTRL_DOIDLEFORMAT|EE_CNTRL_PASTESPECIAL);
			pEditEngine->SetControlWord( nWord );
			pEditEngine->SetPool( pDll->pEditPool );
			pEditView = new EditView( pEditEngine, (Window*)pOut );
			Brush aBrush( pOwner->aBackColor );
			pEditView->SetBackgroundBrush( aBrush );
			pEditEngine->SetRefDevice( pOut );
			SfxItemSet aSet( pEditEngine->GetEmptyItemSet() );
			EditEngine::SetFontInfoInItemSet( aSet, pOwner->aFont );
			pEditEngine->SetDefaults( aSet );

			if( pOwner->GetWordWrap() == VCWORDWRAP_OFF )
			{
				Size aPaperSizePixel( 1280, 1024 );
				pEditEngine->SetPaperSize( pOut->PixelToLogic( aPaperSizePixel ) );
			}
			else
			{
				pEditEngine->SetPaperSize( aClientRect.GetSize() );
			}

			pEditEngine->SetText( aText );
			pEditEngine->InsertView( pEditView );
			Size aNewSize( aClientRect.GetSize() );
			pEditView->SetOutputArea( aClientRect );
			if( nFlags & FLAG_VISAREA_VALID )
				pEditView->SetVisArea( aDocVisArea );
			pEditEngine->SetStatusEventHdl(LINK(this,VCMultiEditEngine,StatusEventHdl));
		}
		else
		{
			DBG_CHKOBJ(pEditEngine,EditEngine,0);
			DBG_CHKOBJ(pEditView,EditView,0);
			DBG_ASSERT(pEditEngine->GetViewCount()==1,"bad view count");
			pEditView->SetWindow( (Window*)pOut );
			pEditView->SetOutputArea( aClientRect );
			if( nFlags & FLAG_VISAREA_VALID )
				pEditView->SetVisArea( aDocVisArea );
			if( pEditEngine->GetRefDevice() != pOut )
				pEditEngine->SetRefDevice( pOut );
		}
	}
}

//------------------------------------------------------------------------
void VCMultiEditEngine::SetText( const String& rText )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	nFlags &= ~FLAG_CURSOR_POS_VALID;
	if( (pEditEngine || pEditView) && pEditView->GetWindow() )
	{
		DBG_ASSERT(pEditEngine&&pEditView,"Edit inkonsistent");
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		DBG_CHKOBJ(pEditView,EditView,0);
		DBG_ASSERT(pEditEngine->GetViewCount()==1,"bad view count");
		pEditEngine->SetText( rText );
		// TimeoutHdl( &aCleanupTimer );
	}
	aText = rText;
	nFlags &= (~FLAG_VISAREA_VALID);
	nFlags |= FLAG_FIRST_PAINT;
	bModified = FALSE;
}

//------------------------------------------------------------------------
void VCMultiEditEngine::SetSelectStart( long nStart )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
}

//------------------------------------------------------------------------
void VCMultiEditEngine::SetSelectLen( long nLen )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
}

//------------------------------------------------------------------------
String VCMultiEditEngine::GetSelectText() const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_ERROR("GetSelectText not impl");
	String aStr;
	return aStr;
}

//------------------------------------------------------------------------
void VCMultiEditEngine::SetSelectText( const String& rText )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_ERROR("SetSelectText not impl");
}

//------------------------------------------------------------------------
void VCMultiEditEngine::Paint( OutputDevice* pOut )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( pOut->GetOutDevType() == OUTDEV_PRINTER )
		nFlags |= FLAG_FIRST_PAINT;
	else if( pEditWin && pEditWin != pOut )
	{
		//
		// Dieser Fall tritt ein, wenn ein von einem Fenster ueberlagertes
		// Edit durch Klicken in eine nicht ueberlappte Flaeche aktiviert
		// wird. Das Edit muss dann komplett neu gepaintet werden, da aufgrund
		// der Flackerminimierung dies beim initialen Erzeugen des
		// Edit-Windows unterdrueckt wird (->ShowEditWin). Der von der
		// Applikation einlaufende Paint-Event bezieht sich aber auf das
		// Fenster unter! pEditWin und wird somit geclippt.
		// pEditWin->GetWindow() ist Null, wenn wir bereits
		// NotifyCloseEditWindow durchlaufen haben (z.B. durch <Escape>)
		if( pEditView->GetWindow() )
		{
			pEditView->HideCursor();
			Paint( pEditWin );
			pEditWin->Paint( aClientRect );
			pEditView->ShowCursor();
			return;
		}
	}
	if( nFlags & FLAG_FIRST_PAINT )
	{
		PositionScrollBars( pOut );
		nFlags &= (~FLAG_FIRST_PAINT);
		SetupMultiEdit( pOut, TRUE );
		EditEngine* pEngine = SI_DLL()->pPaintEngine;
		SetHSBarRange( pEngine );
		SetVSBarRange( pEngine );
		SetLinePageSizes( pEngine );
	}
	else
	{
		SetupMultiEdit( pOut, TRUE );
	}

	// wenn gescrollt wurde, dann nur den Text neu painten
	if( !(nFlags & FLAG_SCROLL_PAINT) )
		DrawBackground( pOut );
	else
	{
		Brush aOldFillBrush( pOut->GetFillInBrush() );
		Brush aBrush( pOwner->aBackColor );
		pOut->SetFillInBrush( aBrush );
		pOut->DrawRect( Rectangle( pOwner->GetPos(), pOwner->GetSize()));
		pOut->SetFillInBrush( aOldFillBrush );
	}

	EditEngine* pPaintEngine = SI_DLL()->pPaintEngine;
	// feststellen, wo wir hinmalen muessen
	Point aVisTopLeft;
	if( nFlags & FLAG_VISAREA_VALID )
		aVisTopLeft = aDocVisArea.TopLeft();
	else if( pEditView )
		// wir haben noch ne View, also deren Topleft nehmen
		aVisTopLeft = pEditView->GetVisArea().TopLeft();

	pPaintEngine->Draw( pOut, aClientRect, aVisTopLeft );

	if( !(nFlags & FLAG_SCROLL_PAINT ))
	{
		aVScroll.Paint( pOut );
		aHScroll.Paint( pOut );
		DrawResizeRect( pOut );
		DrawBorder( pOut );
	}
	if( pOut->GetOutDevType() == OUTDEV_PRINTER )
		nFlags |= FLAG_FIRST_PAINT;
}

//------------------------------------------------------------------------
void VCMultiEditEngine::MouseButtonDown( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( aDestroyEditWinTimer.IsActive() )
		DestroyEditWin( &aDestroyEditWinTimer );

	nFlags &= ~FLAG_CURSOR_POS_VALID;
	nFlags |= FLAG_IN_MOUSEDOWN;

	if( nFlags & FLAG_FIRST_PAINT )
	{
		PositionScrollBars( pWindow );
		nFlags &= (~FLAG_FIRST_PAINT);
		SetupMultiEdit( pWindow, TRUE );
		EditEngine* pEngine = SI_DLL()->pPaintEngine;
		SetHSBarRange( pEngine );
		SetVSBarRange( pEngine );
		SetLinePageSizes( pEngine );
	}
	if( pOwner->bActive )
	{

		Point aPosLog( rMEvt.GetPosPixel() );
		if( nFlags & FLAG_MEVT_ISPIXEL )
		{
			// Pixel-Events gibts nur vom EditWin
			DBG_ASSERT(pEditWin,"No EditWin");
			aPosLog = pEditWin->PixelToLogic( aPosLog );
		}
		MouseEvent aMEvtLog( aPosLog,1,rMEvt.GetMode(),rMEvt.GetButtons(),rMEvt.GetModifier());

		if( aClientRect.IsInside( aPosLog ) )
		{
			if( !pEditWin )
				ShowEditWin( pWindow );
			if( !pEditWin )
			{
				nFlags &= ~FLAG_IN_MOUSEDOWN;
				return; // falls wir im GotFocus-Callback wieder den Fokus verlieren
			}
			pEditWin->SetPointer( Pointer( POINTER_BEAM ) );
			DBG_ASSERT(pEditWin,"ShowEditWin mailed");
			// die EditView wills in Pixeln haben
			Point aPosPix( rMEvt.GetPosPixel() );
			if( !(nFlags & FLAG_MEVT_ISPIXEL))
			{
				// sind noch keine Pixel, deshalb konvertieren
				aPosPix = pWindow->LogicToPixel( aPosPix );
				aPosPix = pWindow->OutputToScreenPixel( aPosPix );
				aPosPix = pEditWin->ScreenToOutputPixel( aPosPix );
			}
			MouseEvent aMEvtPix(aPosPix,1,rMEvt.GetMode(),rMEvt.GetButtons(),rMEvt.GetModifier());
			DBG_CHKOBJ(pEditView,EditView,0);
			ESelection aOldSelection( pEditView->GetSelection() );
			aOldSelection.Adjust();
			pEditView->MouseButtonDown( aMEvtPix );
			ESelection aCurSelection( pEditView->GetSelection() );
			aCurSelection.Adjust();
			if( !aOldSelection.IsEqual(aCurSelection) && (
				aOldSelection.HasRange() || aCurSelection.HasRange() ))
				pOwner->Selected();
		}
		else if( aHSBarRect.IsInside( aPosLog) )
		{
			if( pEditWin )
				pEditWin->SetPointer( Pointer( POINTER_ARROW ) );
			aHScroll.MouseButtonDown( pWindow, aMEvtLog );
			nFlags |= FLAG_CAPTURED_HSBAR;
		}
		else if( aVSBarRect.IsInside( aPosLog) )
		{
			if( pEditWin )
				pEditWin->SetPointer( Pointer( POINTER_ARROW ) );
			aVScroll.MouseButtonDown( pWindow, aMEvtLog );
			nFlags |= FLAG_CAPTURED_VSBAR;
		}
		// Im ersten MouseButtonDown (das EditWindow wurde erzeugt...),
		// darf die Maus nicht vom EditWindow gecaptured werden, da dadurch
		// die MouseEvent-Statemachine in den Applikationen und im VCManager
		// aufgrund des fehlenden MouseButtonUp nicht mehr richtig funktionieren
		// wuerde.
		if( pEditWin && pEditWin == pWindow )
			pEditWin->CaptureMouse();
		else if( pWindow )
		{
			// Mouse capturen, da die Selection-Engine im MouseButtonDown,
			// das ihr geschickt wird, um die initiale Cursorposition richtig
			// zu setzen, die Maus ebenfalls captured.
			pWindow->CaptureMouse();
		}
	}
	nFlags &= ~FLAG_IN_MOUSEDOWN;
}

//------------------------------------------------------------------------
void VCMultiEditEngine::MouseButtonUp( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	BOOL bMouseCaptured = FALSE;
	if( pEditWin )
	{
		bMouseCaptured = pEditWin->IsMouseCaptured();
		pEditWin->ReleaseMouse();
		if( ((VCMultiEditWin*)pEditWin)->bParentAlreadyClosing )
			return;
	}
	else
		pWindow->ReleaseMouse();

	nFlags &= ~( FLAG_CAPTURED_HSBAR | FLAG_CAPTURED_VSBAR );

	Point aPosLog( rMEvt.GetPosPixel() );
	if( nFlags & FLAG_MEVT_ISPIXEL )
	{
		// Pixel-Events gibts nur vom EditWin
		DBG_ASSERT(pEditWin,"No EditWin");
		aPosLog = pEditWin->PixelToLogic( aPosLog );
	}
	MouseEvent aMEvtLog( aPosLog,1,rMEvt.GetMode(),rMEvt.GetButtons(),rMEvt.GetModifier());

	if( aHSBarRect.IsInside( aPosLog) )
	{
		if( pEditWin )
			pEditWin->SetPointer( Pointer( POINTER_ARROW ) );
		aHScroll.MouseButtonUp( pWindow, aMEvtLog );
	}
	else if( aVSBarRect.IsInside( aPosLog ) )
	{
		if( pEditWin )
			pEditWin->SetPointer( Pointer( POINTER_ARROW ) );
		aVScroll.MouseButtonUp( pWindow, aMEvtLog );
	}
	// sonst fuers Edit-Window
	else if( pEditWin && (bMouseCaptured || aClientRect.IsInside( aPosLog )))
	{
		pEditWin->SetPointer( Pointer( POINTER_BEAM ) );
		Point aPosPix( rMEvt.GetPosPixel() );
		if( !(nFlags & FLAG_MEVT_ISPIXEL))
		{
			// sind noch keine Pixel, deshalb konvertieren
			aPosPix = pWindow->LogicToPixel( aPosPix );
			aPosPix = pWindow->OutputToScreenPixel( aPosPix );
			aPosPix = pEditWin->ScreenToOutputPixel( aPosPix );
		}
		MouseEvent aMEvtPix(aPosPix,1,rMEvt.GetMode(),rMEvt.GetButtons(),rMEvt.GetModifier());
		DBG_CHKOBJ(pEditView,EditView,0);
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		ESelection aOldSelection( pEditView->GetSelection() );
		aOldSelection.Adjust();
		pEditView->MouseButtonUp( aMEvtPix );
		ESelection aCurSelection( pEditView->GetSelection() );
		aCurSelection.Adjust();
		if( !aOldSelection.IsEqual(aCurSelection) && (
			aOldSelection.HasRange() || aCurSelection.HasRange() ))
			pOwner->Selected();
	}
}

//------------------------------------------------------------------------
void VCMultiEditEngine::MouseMove( Window* pWindow, const MouseEvent& rMEvt )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( !pEditWin )
		return;

	if( pEditWin && ((VCMultiEditWin*)pEditWin)->bParentAlreadyClosing )
		return;

	Point aPosLog( rMEvt.GetPosPixel() );
	if( nFlags & FLAG_MEVT_ISPIXEL )
	{
		// Pixel-Events gibts nur vom EditWin
		DBG_ASSERT(pEditWin,"No EditWin");
		aPosLog = pEditWin->PixelToLogic( aPosLog );
	}
	MouseEvent aMEvtLog( aPosLog,1,rMEvt.GetMode(),rMEvt.GetButtons(),rMEvt.GetModifier());

	if( (nFlags & FLAG_CAPTURED_HSBAR) || aHSBarRect.IsInside( aPosLog) )
	{
		if( pEditWin )
			pEditWin->SetPointer( Pointer( POINTER_ARROW ) );
		aHScroll.MouseMove( pWindow, aMEvtLog );
		return;
	}
	if( (nFlags & FLAG_CAPTURED_VSBAR) || aVSBarRect.IsInside( aPosLog) )
	{
		if( pEditWin )
			pEditWin->SetPointer( Pointer( POINTER_ARROW ) );
		aVScroll.MouseMove( pWindow, aMEvtLog );
		return;
	}

	if( aClientRect.IsInside( aPosLog ) && pEditWin )
	{
		if( pEditWin )
			pEditWin->SetPointer( Pointer( POINTER_BEAM ) );
		Point aPosPix( rMEvt.GetPosPixel() );
		if( !(nFlags & FLAG_MEVT_ISPIXEL))
		{
			// sind noch keine Pixel, deshalb konvertieren
			aPosPix = pWindow->LogicToPixel( aPosPix );
			aPosPix = pWindow->OutputToScreenPixel( aPosPix );
			aPosPix = pEditWin->ScreenToOutputPixel( aPosPix );
		}
		MouseEvent aMEvtPix(aPosPix,1,rMEvt.GetMode(),rMEvt.GetButtons(),rMEvt.GetModifier());
		DBG_CHKOBJ(pEditView,EditView,0);
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		ESelection aOldSelection( pEditView->GetSelection() );
		aOldSelection.Adjust();
		pEditView->MouseMove( aMEvtPix );
		ESelection aCurSelection( pEditView->GetSelection() );
		aCurSelection.Adjust();
		if( !aOldSelection.IsEqual(aCurSelection) && (
			aOldSelection.HasRange() || aCurSelection.HasRange() ))
			pOwner->Selected();
	}
}

//------------------------------------------------------------------------
void VCMultiEditEngine::KeyInput( Window* pWindow, const KeyEvent& rKEvt )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	KeyCode aCode = rKEvt.GetKeyCode();
	switch( aCode.GetCode() )
	{
		case KEY_ESCAPE:
			if( !aCode.IsShift())
			{
				// LoseFocus-Handler abklemmen
				((VCMultiEditWin*)pEditWin)->bParentAlreadyClosing = TRUE;
				NotifyCloseEditWindow();
				return;
			}
			break;

		case KEY_TAB:
			{
				BOOL bReverse = rKEvt.GetKeyCode().IsShift();
				VCEdit*	pEdit = pOwner->FindNext( bReverse );
				Window* pParent = pEditWin->GetParent();
				// LoseFocus-Handler abklemmen
				((VCMultiEditWin*)pEditWin)->bParentAlreadyClosing = TRUE;
				NotifyCloseEditWindow();
				Cleanup(); // !!!! pEditWin ist jetzt 0 !!!!!
				if( pEdit )
					pEdit->Focus();
			}
			return;


	}

	if( pOwner->IsReadOnly() && EditEngine::DoesKeyChangeText( rKEvt ))
	{
		if( pEditWin )
			((VCMultiEditWin*)pEditWin)->bKeyInputCallBaseClass = FALSE;
		return;
	}

	DBG_CHKOBJ(pEditView,EditView,0);
	ESelection aOldSelection( pEditView->GetSelection() );
	aOldSelection.Adjust();
	if( !pEditView->PostKeyEvent( rKEvt ) )
	{
		if( pEditWin )
			((VCMultiEditWin*)pEditWin)->bKeyInputCallBaseClass = TRUE;
		return;
	}
	else
	{
		if( pEditWin )
			((VCMultiEditWin*)pEditWin)->bKeyInputCallBaseClass = FALSE;
	}

	if( pEditEngine->IsModified() )
	{
		pEditEngine->ClearModifyFlag();
		bModified = TRUE;
		pOwner->Modified();
	}
	else
		bModified = FALSE;
	ESelection aCurSelection( pEditView->GetSelection() );
	aCurSelection.Adjust();
	if( !aOldSelection.IsEqual(aCurSelection) && (
		aOldSelection.HasRange() || aCurSelection.HasRange() ))
		pOwner->Selected();
}

//------------------------------------------------------------------------
Window* VCMultiEditEngine::GetEditWin() const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	return pEditWin;
}

//------------------------------------------------------------------------
void VCMultiEditEngine::Resize()
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	if( pEditWin )
	{
		InitEditWinPosSizeMap();
		PositionScrollBars( pEditWin->GetParent() );
		if( pEditView && pEditView->GetWindow() )
			pEditView->SetOutputArea( aClientRect );
	}
	else
	{
		nFlags |= FLAG_FIRST_PAINT;
		nFlags &= (~FLAG_VISAREA_VALID);
	}
}

//------------------------------------------------------------------------
void VCMultiEditEngine::InitEditWinPosSizeMap()
{
	if( pEditWin )
	{
		Window* pWin = pEditWin->GetParent();
		Point aPos( pOwner->aPos );
		Point aPosPixel = pWin->LogicToPixel( aPos );
		Size aSizePixel= pWin->LogicToPixel( pOwner->aSize );
		aSizePixel.Width()++;
		aSizePixel.Height()++;
		pEditWin->SetPosSizePixel( aPosPixel, aSizePixel );

		MapMode aMap = pWin->GetMapMode();
		aMap.SetOrigin( Point( -aPos.X(), -aPos.Y() ) );
		pEditWin->SetMapMode( aMap );
	}
}

//------------------------------------------------------------------------
void VCMultiEditEngine::PrepareClose()
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( pEditWin )
	{
		DBG_CHKOBJ(pEditView,EditView,0);
		aText = GetTextFromEditEngine( pEditEngine, pOwner->GetWordWrap() );
	}
}

//------------------------------------------------------------------------
long VCMultiEditEngine::GetSelectLen() const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( pEditWin )
	{
		DBG_CHKOBJ(pEditView,EditView,0);
		String aStr( pEditView->GetSelected() );
		return aStr.Len();
	}
	return 0;
}

//------------------------------------------------------------------------
long VCMultiEditEngine::GetSelectStart()
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	ULONG nPos = 0;
	if( pEditWin )
	{
		DBG_CHKOBJ(pEditView,EditView,0);
		ESelection aSel( pEditView->GetSelection() );
		aSel.Adjust();
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		for( ULONG nCurPara=0; nCurPara < aSel.nStartPara; nCurPara++ )
		{
			nPos += (ULONG)(pEditEngine->GetTextLen( (USHORT)nCurPara ));
			nPos++; // Absatztrenner
		}
		nPos += pEditEngine->GetTextLen( aSel.nStartPara );
	}
	return (long)nPos;
}

//------------------------------------------------------------------------
String VCMultiEditEngine::GetText()
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	if( pEditWin && pEditEngine )
	{
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		aText = GetTextFromEditEngine( pEditEngine, pOwner->GetWordWrap() );
	}
	return aText;
}

//------------------------------------------------------------------------
IMPL_LINK(VCMultiEditEngine,HorScrollHdl,VCScrollBarEngine*,pSBar)
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	BOOL bRetrigger = FALSE;
	if( aCleanupTimer.IsActive())
		bRetrigger = TRUE;
	aCleanupTimer.Stop();

	BOOL bEditing = FALSE;
	if( pEditView && (pEditView->GetWindow() != 0))
		bEditing = TRUE;

	if( !bEditing )
	{
		DBG_ASSERT(pSBar->GetActWin(),"Scrollbar has no win");
		SetupMultiEdit( pSBar->GetActWin(), FALSE );
		bRetrigger = TRUE; // das Geraffel wieder weghauen
	}

	// Scrollweite berechnen (vom Scroller gibts nur die neue Position
	long nCurLeft = (nFlags & FLAG_VISAREA_VALID) ? aDocVisArea.Left() : 0;
	long nScrollerLeft = pSBar->GetAct();
	// negativ: rechter Pfeil geclickt; Scrollwert muss negativ sein
	long nScroll = nCurLeft - nScrollerLeft;

	nFlags &= ~(FLAG_VISAREA_VALID);
	nFlags |= FLAG_SCROLL_PAINT;

	pEditView->Scroll( nScroll, 0, RGCHK_PAPERSZ1 );
	aDocVisArea = pEditView->GetVisArea();
	nFlags |= FLAG_VISAREA_VALID;
	nFlags &= ~(FLAG_SCROLL_PAINT);
	if( !bEditing )
	{
		// Bei/Nach SetWindow(0) nimmt die EditView nicht ihren Cursor aus
		// dem Fenster. Nach dem Loeschen der EditView enthaelt das Fenster
		// also einen geloeschten Cursor
		if( pEditView->GetWindow() &&
			pEditView->GetWindow()->GetCursor() == pEditView->GetCursor() )
		{
			pEditView->GetWindow()->SetCursor( 0 );
		}
		pEditView->SetWindow( 0 );
	}
	if( bRetrigger )
		aCleanupTimer.Start();
	return 0;
}

//------------------------------------------------------------------------
IMPL_LINK(VCMultiEditEngine,VerScrollHdl,VCScrollBarEngine*,pSBar)
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	BOOL bRetrigger = FALSE;
	if( aCleanupTimer.IsActive())
		bRetrigger = TRUE;
	aCleanupTimer.Stop();

	BOOL bEditing = FALSE;
	if( pEditView && (pEditView->GetWindow() != 0))
		bEditing = TRUE;

	if( !bEditing )
	{
		DBG_ASSERT(pSBar->GetActWin(),"Scrollbar has no win");
		SetupMultiEdit( pSBar->GetActWin(), FALSE );
		bRetrigger = TRUE; // das Geraffel wieder weghauen
	}

	// Scrollweite berechnen (vom Scroller gibts nur die neue Position
	long nCurTop = (nFlags & FLAG_VISAREA_VALID) ? aDocVisArea.Top() : 0;
	long nScrollerTop = pSBar->GetAct();
	// negativ: unterer Pfeil geclickt; Scrollwert muss negativ sein
	long nScroll = nCurTop - nScrollerTop;

	nFlags &= ~(FLAG_VISAREA_VALID);
	nFlags |= FLAG_SCROLL_PAINT;
	pEditView->Scroll( 0, nScroll, RGCHK_PAPERSZ1 );
	aDocVisArea = pEditView->GetVisArea();
	nFlags |= FLAG_VISAREA_VALID;
	nFlags &= ~(FLAG_SCROLL_PAINT);
	if( !bEditing )
	{
		// Bei/Nach SetWindow(0) nimmt die EditView nicht ihren Cursor aus
		// dem Fenster. Nach dem Loeschen der EditView enthaelt das Fenster
		// also einen geloeschten Cursor
		if( pEditView->GetWindow() &&
			pEditView->GetWindow()->GetCursor() == pEditView->GetCursor() )
		{
			pEditView->GetWindow()->SetCursor( 0 );
		}
		pEditView->SetWindow( 0 );
	}
	if( bRetrigger )
		aCleanupTimer.Start();
	return 0;
}

//------------------------------------------------------------------------
IMPL_LINK(VCMultiEditEngine,StatusEventHdl,EditStatus *, pStat)
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_CHKOBJ(pEditView,EditView,0);
	DBG_CHKOBJ(pEditEngine,EditEngine,0);
	if( pEditWin )
	{
		ULONG nStatus = pStat->GetStatusWord();
		if( nStatus & EE_STAT_TEXTWIDTHCHANGED )
			SetHSBarRange( pEditEngine );
		if( nStatus & EE_STAT_TEXTHEIGHTCHANGED )
			SetVSBarRange( pEditEngine );
		if( nStatus & EE_STAT_HSCROLL )
		{
			aHScroll.SetAct( pEditView->GetVisArea().Left() );
			aHScroll.Paint( pEditWin );
			nFlags &=(~FLAG_VISAREA_VALID);
		}
		if( nStatus & EE_STAT_VSCROLL )
		{
			aVScroll.SetAct( pEditView->GetVisArea().Top() );
			aVScroll.Paint( pEditWin );
			nFlags &= (~FLAG_VISAREA_VALID);
		}
	}
	return 0;
}

IMPL_LINK(VCMultiEditEngine,TimeoutHdl,Timer*,pTimer)
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	pTimer->Stop();
	if( pEditEngine && pEditView )
	{
		DBG_CHKOBJ(pEditEngine,EditEngine,0);
		DBG_CHKOBJ(pEditView,EditView,0);
		DBG_ASSERT(pEditEngine->GetViewCount()==1,"bad view count");
		aDocVisArea = pEditView->GetVisArea();
		nFlags |= FLAG_VISAREA_VALID;
		aText = GetTextFromEditEngine( pEditEngine, pOwner->GetWordWrap() );
		pEditView->SetWindow( 0 );
		DELETEZ(pEditView);
		DELETEZ(pEditEngine);
	}
	return 0;
}

void VCMultiEditEngine::SetHSBarRange( EditEngine* pEngine )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_CHKOBJ(pEngine,EditEngine,0);
	DBG_ASSERT(!(nFlags&FLAG_FIRST_PAINT),"Not painted!");
	aHScroll.SetMax( pEngine->GetPaperSize().Width() );
	aHScroll.Enable( TRUE );
}

void VCMultiEditEngine::SetVSBarRange( EditEngine* pEngine )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_CHKOBJ(pEngine,EditEngine,0);
	DBG_ASSERT(!(nFlags&FLAG_FIRST_PAINT),"Not painted!");
	long nTextHeight = pEngine->GetTextHeight();
	long nOutputHeight = aClientRect.GetHeight();
	if( nTextHeight > nOutputHeight )
		nTextHeight -= nOutputHeight;
	aVScroll.SetMax( nTextHeight );
	aVScroll.Enable( TRUE );
}

void VCMultiEditEngine::SetLinePageSizes( EditEngine* pEngine )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	DBG_CHKOBJ(pEngine,EditEngine,0);
	DBG_ASSERT(!(nFlags&FLAG_FIRST_PAINT),"Not painted!");
	long nLineHeight = (long)(pEngine->GetLineHeight(0));
	Size aPixHeight(0,nLineHeight);
	aPixHeight = pEngine->GetRefDevice()->LogicToPixel( aPixHeight );
	// immer mindestens 1 Pixel
	if( aPixHeight.Height() < 1 )
	{
		aPixHeight.Height() = 1;
		aPixHeight = pEngine->GetRefDevice()->PixelToLogic( aPixHeight );
		nLineHeight = aPixHeight.Height();
	}
	aVScroll.SetLineSize( nLineHeight );
	aVScroll.SetPageSize( 5 * nLineHeight );
	aHScroll.SetLineSize( nLineHeight );
	aHScroll.SetPageSize( 5 * nLineHeight );
}

void VCMultiEditEngine::Enable( BOOL bEnable )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
}

void VCMultiEditEngine::SetFont( const Font& rFont )
{
	DBG_CHKTHIS(VCMultiEditEngine,0);
	nFlags |= FLAG_FIRST_PAINT;
	nFlags &= (~FLAG_VISAREA_VALID);
}

static Size GetLongestLine( OutputDevice* pOut, const Font& rFont, const String& rLines)
{
	Size aSize;

	String aText( rLines );
	if( aText.Len() == 0 )
		aText = "WWWWWWWWWWWWWW\nWWWWWWWWWW\nWWWWWWWWWW";
	aText.ConvertLineEnd( LINEEND_CR );

	Font aTmpFont( pOut->GetFont());
	pOut->SetFont( rFont );

	USHORT nTokenCount = aText.GetTokenCount( (char)LINEEND_CR );
	for( USHORT nCurToken = 0; nCurToken < nTokenCount; nCurToken++)
	{
		Size aTempSize( pOut->GetTextSize( aText.GetToken( nCurToken,(char)LINEEND_CR )) );
		if( aTempSize.Width() > aSize.Width() )
			aSize = aTempSize;
	}
	pOut->SetFont( aTmpFont );
	return aSize;
}

Size VCMultiEditEngine::GetOptSize( OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	Size aSize( GetLongestLine( pOut,pOwner->aFont, ((VCMultiEditEngine*)this)->GetText()) );
	Size a4Pix( 4,4 );
	a4Pix = pOut->PixelToLogic( a4Pix );
	aSize.Width() += a4Pix.Width(); //  Rand + EditViewOffsets
	aSize.Height() += a4Pix.Height(); //  Rand + EditViewOffsets
	aSize.Height() += VCScrollBarEngine::GetOptWidth( pOut, TRUE );
	aSize.Width() += VCScrollBarEngine::GetOptWidth( pOut, FALSE );
	return aSize;
}

Size VCMultiEditEngine::GetOptSize( const Size& rCharXY, OutputDevice* pOut ) const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	String aStr("-");
	Font aTmpFont( pOut->GetFont() );
	pOut->SetFont( pOwner->aFont );
	Size aSize( pOut->GetTextSize(aStr ));

	aSize.Height() *= rCharXY.Height();

	if( rCharXY.Width() == 0 )
	{
		// max. Breite des vorhandenen Textes nehmen
		Size aMaxSize( GetLongestLine( pOut,pOwner->aFont, ((VCMultiEditEngine*)this)->GetText()) );
		aSize.Width() = aMaxSize.Width();
	}
	else
	{
		FontPitch ePitch = pOwner->aFont.GetPitch();
		if( ePitch == PITCH_DONTKNOW )
		{
			FontMetric aMetric = pOut->GetFontMetric();
			ePitch = aMetric.GetPitch();
		}
		if( ePitch != PITCH_FIXED )
			aSize.Width() = aSize.Height();
		aSize.Width() *= rCharXY.Width();
	}

	Size a4Pix( 4,4 );
	a4Pix = pOut->PixelToLogic( a4Pix );

	aSize.Width() += a4Pix.Width(); //  Rand + EditViewOffsets
	aSize.Height() += a4Pix.Height(); //  Rand + EditViewOffsets
	aSize.Height() += VCScrollBarEngine::GetOptWidth( pOut, TRUE );
	aSize.Width() += VCScrollBarEngine::GetOptWidth( pOut, FALSE );
	pOut->SetFont( aTmpFont );
	return aSize;
}

Size VCMultiEditEngine::GetCharCountXY( OutputDevice* pOut) const
{
	DBG_CHKTHIS(VCMultiEditEngine,0);

	Size aSize( pOwner->aSize );

	aSize.Height() -= VCScrollBarEngine::GetOptWidth( pOut, TRUE );
	aSize.Width() -= VCScrollBarEngine::GetOptWidth( pOut, FALSE );

	String aStr("----");
	Font aTmpFont( pOut->GetFont() );
	pOut->SetFont( pOwner->aFont );
	Size aCharSize( pOut->GetTextSize( aStr ));

	FontPitch ePitch = pOwner->aFont.GetPitch();
	if( ePitch == PITCH_DONTKNOW )
	{
		FontMetric aMetric = pOut->GetFontMetric();
		ePitch = aMetric.GetPitch();
	}
	if( ePitch == PITCH_FIXED )
		aSize.Width() /= ( aCharSize.Width() / 4 );
	else
		aSize.Width() /= aCharSize.Height();
	aSize.Height() /= aCharSize.Height();

	pOut->SetFont( aTmpFont );
	return aSize;
}

void VCMultiEditEngine::Focus( Window* pWin )
{
	if( aDestroyEditWinTimer.IsActive())
		DestroyEditWin( &aDestroyEditWinTimer );
	if( !pEditWin )
		ShowEditWin( pWin );
}

void VCMultiEditEngine::Blur()
{
}

IMPL_LINK(VCMultiEditEngine,DestroyEditWin,Timer*,pTimer)
{
	pTimer->Stop(); // falls Handler direkt gerufen wurde
	ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
	// wurde es vielleicht schon vom Dtor der View oder von einem
	// anderen aktivierten Edit geloescht?
	if( pImplEdit && pImplEdit->pWin == pEditWin )
	{
		DELETEZ(pEditWin);
		delete pImplEdit;
		SI_DLL()->SetActiveEdit(0);
	}
	TimeoutHdl( &aCleanupTimer );
	pOwner->LostFocus();
	return 0;
}

BOOL VCMultiEditEngine::IsModified() const
{
	return bModified;
}

void VCMultiEditEngine::SetModifyFlag( BOOL bModify )
{
	bModified = bModify;
	if( pEditEngine )
	if( bModify )
	{
		// pEditEngine->SetModifyFlag();
	}
	else
		pEditEngine->ClearModifyFlag();
}

void VCMultiEditEngine::Cut()
{
	if( pEditView )
	{
		pEditView->Cut();
	}
}

void VCMultiEditEngine::Copy()
{
	if( pEditView )
	{
		pEditView->Copy();
	}
}

void VCMultiEditEngine::Paste()
{
	if( pEditView )
	{
		pEditView->PasteSpecial();
	}
}

void VCMultiEditEngine::SetWordWrap( USHORT nMode )
{
	nFlags |= FLAG_FIRST_PAINT;
	nFlags &= ~FLAG_VISAREA_VALID;
}

Pointer VCMultiEditEngine::GetPointer( const Point& rPos ) const
{
	if( !aClientRect.IsInside( rPos ))
		return Pointer( POINTER_REFHAND );
	return Pointer( POINTER_BEAM );
}

void VCMultiEditEngine::Cleanup( BOOL bAsyncSelfDestroyOfEditWin )
{
	aDestroyEditWinTimer.Stop();
	ImplActiveEdit* pImplEdit = SI_DLL()->GetActiveEdit();
	// wurde es vielleicht schon vom Dtor der View geloescht?
	if( pImplEdit && pImplEdit->pWin == pEditWin )
	{
		delete pImplEdit;
		SI_DLL()->SetActiveEdit(0);
	}
	if( !bAsyncSelfDestroyOfEditWin )
	{
		DELETEZ(pEditWin);
	}
	else
	{
		((VCMultiEditWin*)pEditWin)->DestroyYourself();
		pEditWin = 0;
	}
	pOwner->LostFocus();
}


