/*************************************************************************
 *
 *  $RCSfile: vcictrl3.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef VCL
#if defined(WIN) || defined(WNT)
#include <svwin.h>
#endif
#if defined(OS2)
#include <svpm.h>
#endif
#endif

#ifndef _SFXENUMITEM_HXX
#include <svtools/eitem.hxx>
#endif
#ifndef _SFXSTRITEM_HXX
#include <svtools/stritem.hxx>
#endif

#pragma hdrstop

#define ITEMID_COLOR 0
#include <svx/colritem.hxx>
#include <svx/impgrf.hxx>
#include "vcspnbtn.hxx"
#include "vcpbttn.hxx"
#include "vcrdbttn.hxx"
#include "vcchkbox.hxx"
#define _SI_VCDRAWOBJ
#include "vctrls.hxx"
#include "vcattr.hxx"
#include "vcattrid.hrc"
#include "vcont.hxx"
#include "vcictrls.hxx"
#include "vcengin2.hxx"
#define _SI_NOVCDRAWPAGE
#define _SI_NOVCDRAWVIEW
#include "vcdrwobj.hxx"
#include "sidll.hxx"

class GraphicFilter;


VCImpVirtButton::VCImpVirtButton( VCControl* pShell, VCButtonEngine* pEngine)
	: VCImpVirtControl( pShell ), pButtonEngine( pEngine )
{
	pEngine->SetClickHdl( LINK(this, VCImpVirtButton, ClickHdl ));
}

VCImpVirtButton::~VCImpVirtButton()
{
	delete pButtonEngine;
}

IMPL_LINK( VCImpVirtButton, ClickHdl, VCButtonEngine*, EMPTYARG )
{
	if( pShell->Type() == TYPE( VCCheckBox ) )
	{
		((VCCheckBox*)pShell)->Check(
			!((VCCheckBox*)pShell)->IsChecked() );
		UpdateValue();
	}
	else if( pShell->Type() == TYPE( VCRadioButton ) )
	{
		if( !((VCRadioButton*)pShell)->IsChecked() )
		{
			((VCRadioButton*)pShell)->Check( TRUE );
			UpdateValue();
		}
	}
	// der Click-Handler braucht auch noch den MouseEvent
	DBG_ASSERT(pActMouseEvent,"Mouseevent not set");
	pShell->Click();
	return 0;
}

Rectangle VCImpVirtButton::GetClientRect( OutputDevice* pOut ) const
{
	Rectangle aRect = pOut->LogicToPixel( aBoundRect, MapMode( MAP_TWIP ) );
	aRect = pOut->PixelToLogic(aRect,MapMode( pOut->GetMapMode().GetMapUnit()));
	if( pShell->IsA( TYPE( VCPushButton ) ) ||
		pShell->IsA( TYPE( VCSpinButton ) )    )
		return aRect;
	if( ((SfxBoolItem&)pShell->GetItemSet()->Get( VCATTR_BORDER )).GetValue() )
	{
		Size aOfs = pOut->PixelToLogic( Size( 1, 1 ) );
		Rectangle aClientRect = aRect;

		aClientRect.Left() 	+= aOfs.Width();
		aClientRect.Top()  	+= aOfs.Height();
		aClientRect.Right()	-= aOfs.Width();
		aClientRect.Bottom()-= aOfs.Height();
		return aClientRect;
	}
	else
		return aRect;
}

Rectangle VCImpVirtButton::GetTextRect( OutputDevice* pOut, const Font& rFont ) const
{
	return pButtonEngine->GetTextRect( pOut, rFont );
}

Size VCImpVirtButton::GetOptSize( OutputDevice* pOut ) const
{
	return pButtonEngine->GetOptSize( pOut );
}

void VCImpVirtButton::MouseButtonDown( Window* pWin, const MouseEvent& rMEvt )
{
	SetActMouseEvent( (MouseEvent*)&rMEvt );
	pActWindow = pWin;
	Point aPos = pWin->LogicToPixel( rMEvt.GetPosPixel(),
					MapMode( MAP_TWIP ) );
	aPos = pWin->PixelToLogic( aPos, MapMode( pWin->GetMapMode().GetMapUnit() ) );

	MouseEvent aEvt( aPos, rMEvt.GetClicks(), rMEvt.GetMode(),
					 rMEvt.GetButtons(), rMEvt.GetModifier() );

	if( pShell->HasOverlayObjects() )
		Paint( pWin );
	pButtonEngine->MouseButtonDown( pWin, aEvt );
	VCImpVirtControl::MouseButtonDown( pWin, rMEvt );
}

void VCImpVirtButton::MouseButtonUp( Window* pWin, const MouseEvent& rMEvt )
{
	SetActMouseEvent( (MouseEvent*)&rMEvt );
	pActWindow = pWin;
	Point aPos = pWin->LogicToPixel( rMEvt.GetPosPixel(),
					MapMode( MAP_TWIP ) );
	aPos = pWin->PixelToLogic( aPos, MapMode(pWin->GetMapMode().GetMapUnit()));

	MouseEvent aEvt( aPos, rMEvt.GetClicks(), rMEvt.GetMode(),
					 rMEvt.GetButtons(), rMEvt.GetModifier() );

	if( pShell->HasOverlayObjects() )
		Paint( pWin );
#if SUPD > 364
	DBG_ASSERT(SiDLL::GetCurControl()==pShell,"Current control not registered");
#endif
	pButtonEngine->MouseButtonUp( pWin, aEvt );
	if( SiDLL::IsCurControlDeleted() )
		return;

	VCImpVirtControl::MouseButtonUp( pWin, rMEvt );
	pShell->InvalidateOverlayObjects( pWin );
}

void VCImpVirtButton::MouseMove( Window* pWin, const MouseEvent& rMEvt )
{
	SetActMouseEvent( (MouseEvent*)&rMEvt );
	pActWindow = pWin;
	Point aPos = pWin->LogicToPixel( rMEvt.GetPosPixel(),
					MapMode( MAP_TWIP ) );
	aPos = pWin->PixelToLogic( aPos, MapMode( pWin->GetMapMode().GetMapUnit() ) );

	MouseEvent aEvt( aPos, rMEvt.GetClicks(), rMEvt.GetMode(),
					 rMEvt.GetButtons(), rMEvt.GetModifier() );

	if( pShell->HasOverlayObjects() )
		Paint( pWin );
	pButtonEngine->MouseMove( pWin, aEvt );
	VCImpVirtControl::MouseMove( pWin, rMEvt );
}

void VCImpVirtButton::KeyInput( Window* pWindow, const KeyEvent& rKEvt )
{
	SetActKeyEvent( (KeyEvent*)&rKEvt );
	pButtonEngine->KeyInput( pWindow, rKEvt );
}

void VCImpVirtButton::Paint( OutputDevice* pOut )
{
	if( pOut->GetOutDevType() == OUTDEV_PRINTER && !pShell->IsPrintable() )
		return;
	pButtonEngine->SetFont( GetFont( pOut ) );
	pButtonEngine->Paint( pOut );
}



VCImpVirtPushButton::VCImpVirtPushButton( VCControl* pShell ) :
	VCImpVirtButton( pShell, new VCPushButtonEngine )
{
	UpdateFromItemSet();
}

VCImpVirtCheckBox::VCImpVirtCheckBox( VCControl* pShell ) :
	VCImpVirtButton( pShell, new VCCheckBoxEngine )
{
	UpdateFromItemSet();
}

VCImpVirtRadioButton::VCImpVirtRadioButton( VCControl* pShell ) :
	VCImpVirtButton( pShell, new VCRadioButtonEngine )
{
	UpdateFromItemSet();
}

void VCImpVirtRadioButton::UpdateValue()
{
	((VCRadioButtonEngine*)pButtonEngine)->Check( ((VCRadioButton*)pShell)->IsChecked() );
	VCContainer* pContainer = pShell->GetParent();
	if( pContainer->IsA( TYPE( VCManager ) ) )
		((VCManager*)pContainer)->PaintControl( pShell, TRUE );
}

void VCImpVirtCheckBox::UpdateValue()
{
	((VCCheckBoxEngine*)pButtonEngine)->Check( ((VCCheckBox*)pShell)->IsChecked() );
	VCContainer* pContainer = pShell->GetParent();
	if( pContainer->IsA( TYPE( VCManager ) ) )
		((VCManager*)pContainer)->PaintControl( pShell, TRUE );
}

void VCImpVirtPushButton::PurgeGraphic()
{
	((VCPushButtonEngine*)pButtonEngine)->PurgeGraphic();
}

void VCImpVirtButton::UpdateFromItemSet( USHORT nWhich )
{
	VCImpVirtControl::UpdateFromItemSet( nWhich );
	pButtonEngine->SetPos( pShell->GetPos() );
	pButtonEngine->SetSize( pShell->GetSize());

	SfxItemSet* pItemSet = pShell->GetItemSet();

	// Enable
	if( !nWhich || (nWhich==VCATTR_ENABLED) )
	{
		BOOL bEnabled = ((SfxBoolItem&)pItemSet->Get( VCATTR_ENABLED )).GetValue();
		pButtonEngine->Enable( bEnabled );
	}

	// Border setzen
	if( !nWhich || (nWhich==VCATTR_BORDER) )
	{
		BOOL bBorder = ((SfxBoolItem&)pItemSet->Get( VCATTR_BORDER )).GetValue();
		pButtonEngine->SetBorder( bBorder );
	}

	// Text setzen
	if( !nWhich || (nWhich==VCATTR_CAPTION) )
	{
		String aCaption = ((SfxStringItem&)pItemSet->Get( VCATTR_CAPTION )).GetValue();
		pButtonEngine->SetCaption( aCaption );
	}
	// Farben setzen
	if( !nWhich || (nWhich==VCATTR_FORECOLOR) || (nWhich==VCATTR_BACKCOLOR) )
	{
		Color aForeCol = ((SvxColorItem&)pItemSet->Get(VCATTR_FORECOLOR)).GetValue();
		Color aBackCol = ((SvxColorItem&)pItemSet->Get(VCATTR_BACKCOLOR)).GetValue();
		pButtonEngine->SetForeColor( aForeCol );
		pButtonEngine->SetBackColor( aBackCol );
	}
}

void VCImpVirtRadioButton::UpdateFromItemSet( USHORT nWhich )
{
	VCImpVirtButton::UpdateFromItemSet( nWhich );
	SfxItemSet* pItemSet = pShell->GetItemSet();
	if( !nWhich || (nWhich==VCATTR_RADIOBUTTONVALUE) )
	{
		BOOL bChecked = ((SfxBoolItem&)pItemSet->Get(VCATTR_RADIOBUTTONVALUE)).GetValue();
		((VCRadioButtonEngine*)pButtonEngine)->Check( bChecked );
	}
}

void VCImpVirtCheckBox::UpdateFromItemSet( USHORT nWhich )
{
	VCImpVirtButton::UpdateFromItemSet( nWhich );
	SfxItemSet* pItemSet = pShell->GetItemSet();
	if( !nWhich || (nWhich==VCATTR_CHECKBOXVALUE) )
	{
		BOOL bChecked = ((SfxBoolItem&)pItemSet->Get(VCATTR_CHECKBOXVALUE)).GetValue();
		((VCCheckBoxEngine*)pButtonEngine)->Check( bChecked );
	}
}

void VCImpVirtPushButton::UpdateFromItemSet( USHORT nWhich )
{
	VCImpVirtButton::UpdateFromItemSet( nWhich );
	SfxItemSet* pItemSet = pShell->GetItemSet();
	if( !nWhich || (nWhich==VCATTR_IMAGEURL) )
	{
		String aURL = ((SfxStringItem&)pItemSet->Get( VCATTR_IMAGEURL )).GetValue();
		((VCPushButtonEngine*)pButtonEngine)->SetImageURL( aURL, pShell );
	}
}

void VCImpVirtPushButton::LoadImageSync()
{
	((VCPushButtonEngine*)pButtonEngine)->LoadImageSync();
}

/*************************************************************************
	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/si/vcictrl3.cxx,v 1.1.1.1 2000/09/29 11:02:40 mh Exp $

	  Source Code Control System - Update

	  $Log: vcictrl3.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:40  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:45  hr
	  initial import
	
	  Revision 1.2  2000/09/08 14:03:49  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:03  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.14  1997/11/21 16:02:16  OV
	  Headerumstellung
	

	  Rev 1.13   21 Nov 1997 17:02:16   OV
   Headerumstellung

	  Rev 1.12   31 Oct 1997 17:14:20   OV
   VCL: Windows-Zeichenfunktionen auf allen Plattformen (364-inkompat); diverse VCL-Anpassungen

	  Rev 1.11   12 Sep 1997 10:20:18   OS
   ITEMID_* definiert

	  Rev 1.10   09 Aug 1997 11:09:12   OS
   frmitems/paraitem/textitem aufgeteilt

	  Rev 1.9   30 Jul 1997 13:49:26   MH
   chg: header

	  Rev 1.8   28 Jul 1997 16:43:14   MH
   chg: header

	  Rev 1.7   22 Jul 1997 12:25:28   OV
   Controls koennen im Makrohandler geloescht werden

	  Rev 1.6   16 Jul 1997 18:13:28   MH
   add: header

	  Rev 1.5   21 Feb 1997 16:05:06   OV
   MapUnit des Fensters des benutzen, nicht die des VCControls

	  Rev 1.4   13 Feb 1997 17:51:16   OV
   #35817# Events nur noch am ImpControl

	  Rev 1.3   16 Jan 1997 18:14:56   OV
   Position & Groesse stets in Twips

	  Rev 1.2   08 Jan 1997 17:31:12   OV
   Diverse Bugfixes

	  Rev 1.1   12 Dec 1996 18:26:16   OV
   ClickHandler rufen, RadioButton-Gruppen richtig behandeln

	  Rev 1.0   12 Dec 1996 10:58:30   OV
   Initial revision.

*************************************************************************/

#pragma SEG_EOFMODULE

