/*************************************************************************
 *
 *  $RCSfile: vced.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: mh $ $Date: 2000/09/29 11:02:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SFXAPP_HXX
#include <sfx2/app.hxx>
#endif
#ifndef _SFX_CHILDWIN_HXX
#include <sfx2/childwin.hxx>
#endif
#ifndef _SV_SCRBAR_HXX
#include <vcl/scrbar.hxx>
#endif
#ifndef _SBXCLASS_HXX
#include <svtools/sbx.hxx>
#endif
#ifndef _SHL_HXX //autogen
#include <tools/shl.hxx>
#endif
#ifndef _SVDVIEW_HXX //autogen
#include <svx/svdview.hxx>
#endif
#ifndef _SFXITEMPOOL_HXX
#include <svtools/itempool.hxx>
#endif

#pragma hdrstop

#define _SI_VCDLGOBJ
#define _SI_VCDRAWOBJ

#ifndef _SVXIDS_HRC
#include <svx/svxids.hrc>
#endif
#ifndef _SFXVIEWFRM_HXX
#include <sfx2/viewfrm.hxx>
#endif
#include <svx/svdlayer.hxx>
#include "vcsbxctl.hxx"
#include "vcbrw.hxx"
#include "vcsbxdo.hxx"
#include "vcdrwobj.hxx"
#include "vced.hxx"
#include "vcedfunc.hxx"
#include "vcedfac.hxx"

class VCDlgView : public SdrView
{
protected:

	virtual void MarkListHasChanged()
	{
		SdrView::MarkListHasChanged();

		SfxViewFrame* pCurFrame = SfxViewFrame::Current();
		DBG_ASSERT( pCurFrame != NULL, "No current view frame!" );
		SfxChildWindow* pChildWin = pCurFrame ? pCurFrame->GetChildWindow(SID_SHOW_BROWSER) : NULL;
		if( pChildWin )
			((VCBrowser*)(pChildWin->GetWindow()))->Update( this );
	}

public:
	VCDlgView();

	VCDlgView(SdrModel* pModel1, OutputDevice* pOut=NULL) :
		SdrView( pModel1, pOut )
	{
	}
};

IMPL_LINK( VCDlgEditor, ClipboardCleared, Clipboard *, EMPTYARG )
{
	if( !bClipPrivate )
		return 0;

	SdrModel** ppClipPrivate = (SdrModel**)GetAppData( SHL_VCED );

	if( *ppClipPrivate )
		return 0;

	delete *ppClipPrivate;
	*ppClipPrivate = NULL;

	bClipPrivate = FALSE;
	return 0;
}

BOOL VCDlgEditor::UnmarkDialog()
{
	SdrObject* 		pDlgObj = pSdrModel->GetPage(0)->GetObj(0);
	SdrPageView*    pPgView = pSdrView->GetPageViewPvNum(0);

	BOOL bWasMarked = pSdrView->IsObjMarked( pDlgObj );

	if( bWasMarked )
		pSdrView->MarkObj( pDlgObj, pPgView, FALSE );

	return bWasMarked;
}

void VCDlgEditor::RemarkDialog()
{
	SdrObject* 		pDlgObj = pSdrModel->GetPage(0)->GetObj(0);
	SdrPageView*    pPgView = pSdrView->GetPageViewPvNum(0);

	pSdrView->MarkObj( pDlgObj, pPgView, TRUE );
}

VCDlgEditor::VCDlgEditor( StarBASIC* pBas ) :
	pHScroll(NULL),
	pVScroll(NULL),
	pBasic( pBas ),
	eMode( VCDLGED_SELECT ),
	eActObj( OBJ_DLG_PUSHBUTTON ),
	bFirstDraw(FALSE),
	bGridSnap(FALSE),
	bGridVisible(FALSE),
	bClipPrivate(FALSE),
	bCreateOK(TRUE),
	pSdrView(NULL)
{
	pWindow		= NULL;

	pSdrModel = new VCDrawModel;
	pSdrModel->GetItemPool().FreezeIdRanges();
	pSdrModel->SetScaleUnit( MAP_TWIP );

	SdrLayerAdmin& rAdmin = pSdrModel->GetLayerAdmin();
	rAdmin.NewStandardLayer();
	rAdmin.NewLayer( UniString::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "VCHiddenLayer" ) ) );

	pSdrPage = new VCDlgEdPage( *pSdrModel, pBasic, this );
	pSdrModel->InsertPage( pSdrPage );

	pObjFac = new VCDlgEditFactory( this );

	SdrObjFactory::InsertMakeObjectHdl( LINK( pObjFac, VCDlgEditFactory, MakeObject ) );
	pFunc = new VCDlgEditFuncSelect( this );

	aClip.SetClearedHdl( LINK( this, VCDlgEditor, ClipboardCleared ) );

	aPaintTimer.SetTimeout( 1 );
	aPaintTimer.SetTimeoutHdl( LINK( this, VCDlgEditor, PaintTimeout ) );
}

VCDlgEditor::~VCDlgEditor()
{
	ClipboardCleared( NULL );
	if( pSbxForm.Is() )
		pSbxForm->PrepareDelete();

	pSdrPage->SetDeleteSbxObject( FALSE );
	((VCDrawPage*)pSdrPage)->SetSbxForm( (VCSbxDialog*)NULL );

	SdrObjFactory::RemoveMakeObjectHdl( LINK( pObjFac, VCDlgEditFactory, MakeObject ) );
	delete pObjFac;

	delete pFunc;
	delete pSdrView;
	delete pSdrModel;
}

void VCDlgEditor::SetWindow( Window* pWindow )
{
	VCDlgEditor::pWindow = pWindow;
	pWindow->SetMapMode( MapMode( MAP_TWIP ) );
// was soll das?
//	Brush aBackBrush( BRUSH_NULL );
//	pWindow->SetBackgroundBrush( aBackBrush );
	pSdrPage->SetSize( pWindow->PixelToLogic( Size( 1280, 1024 ) ) );
	pSdrView = new VCDlgView( pSdrModel, pWindow );
	pSdrView->ShowPagePgNum( 0, Point() );
	pSdrView->SetLayerVisible( UniString::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "VCHiddenLayer" ) ), FALSE );
	pSdrView->SetMoveSnapOnlyTopLeft( TRUE );

	Size aGridSize( 60, 60 );  //Twips
	bGridSnap    = TRUE;
	bGridVisible = TRUE;
	pSdrView->SetGridCoarse( aGridSize );
	pSdrView->SetSnapGrid( aGridSize );
	pSdrView->SetGridSnap( bGridSnap );
	pSdrView->SetGridVisible( FALSE );
	pSdrView->SetDragStripes( FALSE );
}

void VCDlgEditor::SetScrollBars( ScrollBar* pHS, ScrollBar* pVS )
{
	pHScroll = pHS;
	pVScroll = pVS;

	Size aOutSize = pWindow->GetOutputSize();
	Size aPgSize  = pSdrPage->GetSize();

	pHScroll->SetRange( Range( 0, aPgSize.Width()  ));
	pVScroll->SetRange( Range( 0, aPgSize.Height() ));
	pHScroll->SetVisibleSize( (ULONG)aOutSize.Width() );
	pVScroll->SetVisibleSize( (ULONG)aOutSize.Height() );

	pHScroll->SetLineSize( aOutSize.Width() / 10 );
	pVScroll->SetLineSize( aOutSize.Height() / 10 );
	pHScroll->SetPageSize( aOutSize.Width() / 2 );
	pVScroll->SetPageSize( aOutSize.Height() / 2 );

	DoScroll( pHScroll );
	DoScroll( pVScroll );
}

void VCDlgEditor::DoScroll( ScrollBar* pActScroll )
{
	if( !pHScroll || !pVScroll )
		return;

	MapMode aMap = pWindow->GetMapMode();
	Point aOrg = aMap.GetOrigin();

	Size  aScrollPos( pHScroll->GetThumbPos(), pVScroll->GetThumbPos() );
	aScrollPos = pWindow->LogicToPixel( aScrollPos );
	aScrollPos = pWindow->PixelToLogic( aScrollPos );

	long  nX   = aScrollPos.Width() + aOrg.X();
	long  nY   = aScrollPos.Height() + aOrg.Y();

	if( !nX && !nY )
		return;

	pWindow->Update();

	Brush aOldBrush( pWindow->GetBackgroundBrush() );
	Brush aBrush( BRUSH_NULL );
	pWindow->SetBackgroundBrush( aBrush );
	pWindow->Scroll( -nX, -nY, SCROLL_NOCHILDREN );
	aMap.SetOrigin( Point( -aScrollPos.Width(), -aScrollPos.Height() ) );
	pWindow->SetMapMode( aMap );
	pWindow->Update();

	pWindow->SetBackgroundBrush( aOldBrush );
}

void VCDlgEditor::SetVCSbxForm( VCSbxDialogRef pForm )
{
	DBG_ASSERT( pForm.Is(), "Kein Dialog -> und tschuess..." );
	pSbxForm = pForm;

	((VCDrawPage*)pSdrModel->GetPage(0))->SetName( pSbxForm->GetName() );
	((VCDrawPage*)pSdrModel->GetPage(0))->SetSbxForm( (VCSbxDialog*)pSbxForm );

	// Drawobject draufklemmen und in Drawing einfuegen
	VCSbxDialogObject* pObj = new VCSbxDialogObject( pSbxForm );
	pSdrModel->GetPage(0)->InsertObject( pObj );

	// Durch das folgende Update wird SVLook am Container richtig
	// gesetzt
	pSbxForm->UpdateItem( 0 );

	pObj->SendRepaintBroadcast();

	// Childs einfuegen
	// und Zeichenobject draufklemmen
	USHORT nCount = pSbxForm->GetObjects()->Count();
	USHORT i;
	for( i=0; i < nCount; i++ )
	{
		SbxVariable* pCtl = pSbxForm->GetObjects()->Get( i );
		DBG_ASSERT( pCtl->IsA( TYPE( VCSbxControl ) ), "Is sich nix Control !!!" );
		if( (pCtl->GetSbxId() >= SBXID_FIRSTCONTROL) &&
			(pCtl->GetSbxId() <= SBXID_LASTCONTROL)      )
		{
			VCSbxDrawObject* pDrawObj = new VCSbxDrawObject( (VCSbxControl*)pCtl );
			pSdrModel->GetPage(0)->InsertObject( pDrawObj );
			pDrawObj->GetControl()->UpdateStep( pSbxForm->GetCurStep() );
			pDrawObj->SendRepaintBroadcast();
		}
	}

	bFirstDraw = TRUE;

	pSdrModel->SetChanged( FALSE );
}

StarBASIC* VCDlgEditor::GetBasic() const
{
	return pBasic;
}

void VCDlgEditor::MouseButtonDown( const MouseEvent& rMEvt )
{
	if( pWindow )
		pWindow->GrabFocus();
	pFunc->MouseButtonDown( rMEvt );
}

void VCDlgEditor::MouseButtonUp( const MouseEvent& rMEvt )
{
	BOOL bRet = pFunc->MouseButtonUp( rMEvt );

	if( (eMode == VCDLGED_INSERT) )
		bCreateOK = bRet;
}

void VCDlgEditor::MouseMove( const MouseEvent& rMEvt )
{
	pFunc->MouseMove( rMEvt );
}

BOOL VCDlgEditor::KeyInput( const KeyEvent& rKEvt )
{
	return pFunc->KeyInput( rKEvt );
}

void VCDlgEditor::Paint( const Rectangle& rRect )
{
	aPaintRect = rRect;
	PaintTimeout( &aPaintTimer );
}

IMPL_LINK( VCDlgEditor, PaintTimeout, Timer *, EMPTYARG )
{
	static int nInPaint = FALSE;
	if( !pSdrView )
		return 0;

	nInPaint = TRUE;

	Size aMacSize;
	if( bFirstDraw &&
		pWindow->IsVisible() &&
		(pWindow->GetOutputSize() != aMacSize) )
	{
		bFirstDraw = FALSE;

		if( pSbxForm->GetSize() == aMacSize )
		{
			Size   aSize = pWindow->PixelToLogic( Size( 400, 300 ) );

			// Am Grid ausrichten
			Size  aGridSize = pSdrView->GetSnapGrid();
			aSize.Width()  -= aSize.Width()  % aGridSize.Width();
			aSize.Height() -= aSize.Height() % aGridSize.Height();

			Point  aPos;
			Size   aOutSize = pWindow->GetOutputSize();
			aPos.X() = (aOutSize.Width()>>1)  -  (aSize.Width()>>1);
			aPos.Y() = (aOutSize.Height()>>1) -  (aSize.Height()>>1);

			// Am Grid ausrichten
			aPos.X() -= aPos.X() % aGridSize.Width();
			aPos.Y() -= aPos.Y() % aGridSize.Height();

			// Bitte nicht in die Ecke quetschen
			Point aMinPos = pWindow->PixelToLogic( Point( 30, 20 ) );
			if( (aPos.X() < aMinPos.X()) || (aPos.Y() < aMinPos.Y()) )
			{
				aPos = aMinPos;
				aPos.X() -= aPos.X() % aGridSize.Width();
				aPos.Y() -= aPos.Y() % aGridSize.Height();
			}

			// Durch diese Groesse werden beim SetPos alle Controls
			// mitgenommen
			pSbxForm->SetSize( Size( 0x7FFFFFFF, 0x7FFFFFFF ) );
			pSbxForm->SetPos( aPos );
			pSbxForm->SetSize( aSize );
		}
	}

	pSdrView->InitRedraw( pWindow, Region( aPaintRect ) );

	nInPaint = FALSE;


	DBG_ASSERT(pWindow,"Window not set");
	return 0;
}

void VCDlgEditor::SetMode( VCDlgMode eNewMode )
{
	if( eMode != eNewMode )
	{
		if( pFunc )
			delete pFunc;
	}

	eMode = eNewMode;
	if( eMode == VCDLGED_INSERT )
		pFunc = new VCDlgEditFuncInsert( this );
	else
		pFunc = new VCDlgEditFuncSelect( this );

	if( eMode == VCDLGED_TEST )
		pSbxForm->EnableTestMode();
}

void VCDlgEditor::SetInsertObj( USHORT eObj )
{
	eActObj = eObj;
	if( pSdrView )
		pSdrView->SetCurrentObj( eActObj, VCSbxInventor );
}

USHORT VCDlgEditor::GetInsertObj() const
{
	return eActObj;
}

void VCDlgEditor::Cut()
{
	if( !pSdrView->HasMarkedObj() )
		return;

	pSdrView->BrkAction();

	BOOL bDlgMarked = UnmarkDialog();

	SdrModel* pMarked = pSdrView->GetAllMarkedModel();

	aClip.Clear();
	aClip.CopyPrivateData( pMarked );

	SdrModel** ppClipPrivate = (SdrModel**)GetAppData( SHL_VCED );
	*ppClipPrivate = pMarked;
	bClipPrivate = TRUE;

	Delete();

	if( bDlgMarked )
		RemarkDialog();
}

void VCDlgEditor::Copy()
{
	if( !pSdrView->HasMarkedObj() )
		return;

	pSdrView->BrkAction();

	BOOL bDlgMarked = UnmarkDialog();

	SdrModel* pMarked = pSdrView->GetAllMarkedModel();

	aClip.Clear();
	aClip.CopyPrivateData( pMarked );

	SdrModel** ppClipPrivate = (SdrModel**)GetAppData( SHL_VCED );
	*ppClipPrivate = pMarked;
	bClipPrivate   = TRUE;

	if( bDlgMarked )
		RemarkDialog();
}

void VCDlgEditor::Paste()
{
	SdrModel** ppClipPrivate = (SdrModel**)GetAppData( SHL_VCED );

	pSdrView->BrkAction();

	if( !*ppClipPrivate )
		return;

	static Point aDefPoint;
	Rectangle aRect( aDefPoint,
					 pSbxForm->GetSize() );
	pSdrView->Paste( **ppClipPrivate, aRect.Center() );
}

void VCDlgEditor::Delete()
{
	if( !pSdrView->HasMarkedObj() )
		return;

	// Sicherstellen, dass nicht ein Dialog geloescht wird. So machts man
	// richtig (inkompat.,ab 369): SdrView::CheckPossibilities() ueberladen
	ULONG nMark = pSdrView->GetMarkList().GetMarkCount();
	for( ULONG i = 0; i < nMark; i++ )
	{
		if(pSdrView->GetMarkList().GetMark(i)->GetObj()->ISA(VCSbxDialogObject))
			return;
	}

	pSdrView->BrkAction();

	BOOL bDlgMarked = UnmarkDialog();

	pSdrView->DeleteMarked();

	if( bDlgMarked )
		RemarkDialog();
}

BOOL VCDlgEditor::IsModified() const
{
	return pSdrModel->IsChanged();
}

void VCDlgEditor::ClearModifyFlag()
{
	pSdrModel->SetChanged( FALSE );
}

#define LMARGPRN		1700
#define RMARGPRN		 900
#define TMARGPRN    	2000
#define BMARGPRN    	1000
#define BORDERPRN		300

void lcl_PrintHeader( Printer* pPrinter, const String& rTitle )
{
	short nLeftMargin 	= LMARGPRN;
	Size aSz = pPrinter->GetOutputSize();
	short nBorder = BORDERPRN;

	Pen aOldPen( pPrinter->GetPen() );
	Brush aOldBrush( pPrinter->GetFillInBrush() );
	Font aOldFont( pPrinter->GetFont() );

	pPrinter->SetPen( Pen( (PenStyle)PEN_SOLID ) );
	pPrinter->SetFillInBrush( Brush( BRUSH_NULL ) );

	Font aFont( aOldFont );
	aFont.SetWeight( WEIGHT_BOLD );
	aFont.SetAlign( ALIGN_BOTTOM );
	pPrinter->SetFont( aFont );

	long nFontHeight = pPrinter->GetTextHeight();

	// 1.Border => Strich, 2+3 Border = Freiraum.
	long nYTop = TMARGPRN-3*nBorder-nFontHeight;

	long nXLeft = nLeftMargin-nBorder;
	long nXRight = aSz.Width()-RMARGPRN+nBorder;

	pPrinter->DrawRect( Rectangle(
		Point( nXLeft, nYTop ),
		Size( nXRight-nXLeft, aSz.Height() - nYTop - BMARGPRN + nBorder ) ) );

	long nY = TMARGPRN-2*nBorder;
	Point aPos( nLeftMargin, nY );
	pPrinter->DrawText( aPos, rTitle );

	nY = TMARGPRN-nBorder;

	pPrinter->DrawLine( Point( nXLeft, nY ), Point( nXRight, nY ) );

	pPrinter->SetPen( aOldPen );
	pPrinter->SetFont( aOldFont );
	pPrinter->SetFillInBrush( aOldBrush );
}

void VCDlgEditor::PrintData( Printer* pPrinter, const String& rTitle )
{
	if( pSdrView )
	{
		MapMode aOldMap( pPrinter->GetMapMode());
		Font aOldFont( pPrinter->GetFont() );

		MapMode aMap( MAP_100TH_MM );
		pPrinter->SetMapMode( aMap );
		Font aFont;
#ifdef OS2
		aFont.SetName( System::GetStandardFont( STDFONT_SWISS ).GetName() );
#endif
		aFont.SetAlign( ALIGN_BOTTOM );
		aFont.SetSize( Size( 0, 360 ));
		pPrinter->SetFont( aFont );

		Size aPaperSz = pPrinter->GetOutputSize();
		aPaperSz.Width() -= (LMARGPRN+RMARGPRN);
		aPaperSz.Height() -= (TMARGPRN+BMARGPRN);

		pPrinter->StartPage();

		lcl_PrintHeader( pPrinter, rTitle );

		Bitmap aDlg;
#ifdef OS2
		Bitmap* pDlg = new Bitmap;
		pSbxForm->SnapShot( *pDlg );
		SvMemoryStream* pStrm = new SvMemoryStream;
		*pStrm << *pDlg;
		delete pDlg;
		pStrm->Seek(0);
		*pStrm >> aDlg;
		delete pStrm;
#else
		pSbxForm->SnapShot( aDlg );
#endif
		Size aBmpSz( pPrinter->PixelToLogic( aDlg.GetSizePixel() ) );
		double nPaperSzWidth = aPaperSz.Width();
		double nPaperSzHeight = aPaperSz.Height();
		double nBmpSzWidth = aBmpSz.Width();
		double nBmpSzHeight = aBmpSz.Height();
		double nScaleX = (nPaperSzWidth / nBmpSzWidth );
		double nScaleY = (nPaperSzHeight / nBmpSzHeight );

		Size aOutputSz;
		if( nBmpSzHeight * nScaleX <= nPaperSzHeight )
		{
			aOutputSz.Width() = (long)(((double)nBmpSzWidth) * nScaleX);
			aOutputSz.Height() = (long)(((double)nBmpSzHeight) * nScaleX);
		}
		else
		{
			aOutputSz.Width() = (long)(((double)nBmpSzWidth) * nScaleY);
			aOutputSz.Height() = (long)(((double)nBmpSzHeight) * nScaleY);
		}

		Point aPosOffs(
			(aPaperSz.Width() / 2) - (aOutputSz.Width() / 2),
			(aPaperSz.Height()/ 2) - (aOutputSz.Height() / 2));

		aPosOffs.X() += LMARGPRN;
		aPosOffs.Y() += TMARGPRN;

		pPrinter->DrawBitmap( aPosOffs, aOutputSz, aDlg );

		pPrinter->EndPage();

		pPrinter->SetMapMode( aOldMap );
		pPrinter->SetFont( aOldFont );
	}
}



/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/script/basctl/source/vcdlged/vced.cxx,v 1.1.1.1 2000/09/29 11:02:41 mh Exp $

	  Source Code Control System - Update

	  $Log: vced.cxx,v $
	  Revision 1.1.1.1  2000/09/29 11:02:41  mh
	  initial import
	
	  Revision 1.1.1.1  2000/09/18 16:11:46  hr
	  initial import
	
	  Revision 1.3  2000/09/12 14:43:47  ab
	  GetChildWindow -> SfxViewFrame()
	
	  Revision 1.2  2000/09/08 14:03:50  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.1.1.1  2000/09/04 07:26:04  tbe
	  basicide, isetbrw, si, vcdlged moved from svx to basctl
	
	  Revision 1.85  2000/05/09 13:34:00  pw
	  UNICODE
	
	  Revision 1.84  2000/02/10 20:08:37  hr
	  #70473# changes for unicode ( patched by automated patchtool )
	
	  Revision 1.83  1999/06/09 13:22:32  MBA
	  SFX_APP() verwenden
	

	  Rev 1.82   09 Jun 1999 15:22:32   MBA
   SFX_APP() verwenden

	  Rev 1.81   05 Nov 1998 18:10:40   OV
   #58891# FreezeIdRanges am Pool des DrawModels rufen

	  Rev 1.80   16 Oct 1998 11:55:22   OV
   Keine Null-Brush setzen; Aufraeumarbeiten

	  Rev 1.79   02 Apr 1998 15:54:00   OV
   #49166# MouseButtonDown: Focus grabben

	  Rev 1.78   27 Feb 1998 18:24:02   OV
   #46884# PrepareDelete aufrufen

	  Rev 1.77   28 Jan 1998 17:40:46   TJ
   include

	  Rev 1.76   01 Dec 1997 10:56:42   TJ
   include

	  Rev 1.75   26 Nov 1997 16:33:38   OV
   #45735# KeyEvents routen

	  Rev 1.74   21 Nov 1997 17:18:50   OV
   Headerumstellung

	  Rev 1.73   18 Nov 1997 13:48:02   OV
   Bei synchronem Paint kuemmert Joe sich um die Handles

	  Rev 1.72   07 Nov 1997 13:09:06   OV
   weitere VCL-Umstellungen

	  Rev 1.71   03 Nov 1997 12:50:50   OV
   #44256# selektierter Dialog nicht loeschbar

	  Rev 1.70   31 Oct 1997 09:54:44   OV
   Anpassungen VCL (asynchrones Paint und Umsetzen der BackgroundBrush im Scroll raus) (364-kompat)

	  Rev 1.69   08 Aug 1997 17:03:06   HJS
   includes

	  Rev 1.68   27 Jun 1997 10:58:50   NF
   OS2-Anpassung

	  Rev 1.67   26 Jun 1997 16:01:16   OV
   #40056# Dialog-Druck: Anpassungen OS/2; Querformat

	  Rev 1.66   25 Jun 1997 19:14:12   OV
   Drucken von Dialogen

	  Rev 1.65   25 Jun 1997 16:55:14   OV
   Drucken von Dialogen

	  Rev 1.64   28 Apr 1997 16:26:56   HJS
   includes

	  Rev 1.63   25 Apr 1997 13:05:36   MH
   chg: header

	  Rev 1.62   04 Apr 1997 15:55:50   MH
   chg: header

	  Rev 1.61   03 Apr 1997 15:23:04   MH

	  Rev 1.60   01 Apr 1997 17:19:54   MH
   chg: header

	  Rev 1.59   08 Jan 1997 18:11:20   OV
   Paint-Bug behoben

	  Rev 1.58   22 Nov 1996 15:09:58   OV
   Paint: Invalidates fuehren zu keinem erneuten Paint

	  Rev 1.57   22 Jul 1996 14:45:58   MT
   ClearModifyFlag

	  Rev 1.56   21 Jun 1996 11:41:36   OV
   #28657# Dtor: Geloeschte View nicht mehr benutzen

*************************************************************************/

#pragma SEG_EOFMODULE

