/*************************************************************************
 *
 *  $RCSfile: basmgr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:12:07 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
//
#ifndef _BASMGR_HXX
#define _BASMGR_HXX

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif
#ifndef _SFXBRDCST_HXX //autogen
#include <svtools/brdcst.hxx>
#endif

#ifndef _SB_SBSTAR_HXX //autogen
#include <sbstar.hxx>
#endif

class SvStorage;

#define BASERR_ID_STDLIBOPEN			ERRCODE_BASMGR_STDLIBOPEN
#define BASERR_ID_STDLIBSAVE            ERRCODE_BASMGR_STDLIBSAVE
#define BASERR_ID_LIBLOAD				ERRCODE_BASMGR_LIBLOAD
#define BASERR_ID_LIBCREATE				ERRCODE_BASMGR_LIBCREATE
#define BASERR_ID_LIBSAVE               ERRCODE_BASMGR_LIBSAVE
#define BASERR_ID_LIBDEL				ERRCODE_BASMGR_LIBDEL
#define BASERR_ID_MGROPEN               ERRCODE_BASMGR_MGROPEN
#define BASERR_ID_MGRSAVE				ERRCODE_BASMGR_MGRSAVE
#define BASERR_ID_REMOVELIB				ERRCODE_BASMGR_REMOVELIB
#define BASERR_ID_UNLOADLIB				ERRCODE_BASMGR_UNLOADLIB

#define BASERR_REASON_OPENSTORAGE       0x0001
#define BASERR_REASON_OPENLIBSTORAGE    0x0002
#define BASERR_REASON_OPENMGRSTREAM     0x0004
#define BASERR_REASON_OPENLIBSTREAM     0x0008
#define BASERR_REASON_LIBNOTFOUND		0x0010
#define BASERR_REASON_STORAGENOTFOUND	0x0020
#define BASERR_REASON_BASICLOADERROR	0x0040
#define BASERR_REASON_NOSTORAGENAME		0x0080

#define BASERR_REASON_STDLIB			0x0100

class BasicError
{
private:
	ULONG	nErrorId;
	USHORT	nReason;
	String	aErrStr;

public:
			BasicError();
			BasicError( const BasicError& rErr );
			BasicError( ULONG nId, USHORT nR, const String& rErrStr );

	ULONG 	GetErrorId() const					{ return nErrorId; }
	USHORT	GetReason() const					{ return nReason; }
	String	GetErrorStr()						{ return aErrStr; }

	void	SetErrorId( ULONG n )				{ nErrorId = n; }
	void	SetReason( USHORT n )				{ nReason = n; }
	void	SetErrorStr( const String& rStr)	{ aErrStr = rStr; }
};


//

class BasicLibs;
class ErrorManager;
class BasicLibInfo;
class BasicErrorManager;

#define LIB_NOTFOUND	0xFFFF

class BasicManager : public SfxBroadcaster
{
public:
					TYPEINFO();
	virtual			~BasicManager();

	virtual void	SetStorageName( const String& rName ) = 0;
	virtual String  GetStorageName() const = 0;
	virtual void	SetName( const String& rName ) = 0;
	virtual String	GetName() const = 0;


//	static BOOL     HasBasicManager( const SvStorage& rStorage );
//	static BOOL     CopyBasicData( SvStorage* pFrom, const String& rSourceURL, SvStorage* pTo);

	virtual void	Merge(  SvStorage& rFromStorage ) = 0;

	virtual USHORT		GetLibCount() const = 0;
	virtual StarBASIC*	GetStdLib() const = 0;
	virtual StarBASIC*	GetLib( USHORT nLib ) const = 0;
	virtual StarBASIC*	GetLib( const String& rName ) const = 0;
	virtual USHORT		GetLibId( const String& rName ) const = 0;
	virtual USHORT		GetLibId( StarBASIC* pLib ) const = 0;
	virtual BOOL		HasLib( const String& rName ) const = 0;

	virtual void		Store( SvStorage& rStorage ) = 0;
	virtual void		Store( SvStorage& rStorage, BOOL bStoreLibs ) = 0;

	virtual BOOL		SetLibName( USHORT nLib, const String& rName ) = 0;
	virtual String		GetLibName( USHORT nLib ) = 0;

	virtual BOOL		SetLibStorageName( USHORT nLib, const String& rName ) = 0;
	virtual String		GetLibStorageName( USHORT nLib ) = 0;
	virtual String		GetRelLibStorageName( USHORT nLib ) = 0;

	virtual BOOL		IsLibLoaded( USHORT nLib ) const = 0;
	virtual BOOL		LoadLib( USHORT nLib ) = 0;
	virtual BOOL		UnloadLib( USHORT nLib ) = 0;
	virtual BOOL		StoreLib( USHORT nLib ) const = 0;
	virtual BOOL		RemoveLib( USHORT nLib ) = 0;
	virtual BOOL		RemoveLib( USHORT nLib, BOOL bDelBasicFromStorage ) = 0;

	virtual BOOL		IsReference( USHORT nLib ) = 0;
	virtual BOOL		IsExtern( USHORT nLib ) = 0;

	virtual StarBASIC*  CreateLib( const String& rLibName ) = 0;
	virtual StarBASIC*  AddLib( SvStorage& rStorage, const String& rLibName, BOOL bReference ) = 0;
	virtual void		AddLib( StarBASIC* pLib ) = 0;
	virtual BOOL		MoveLib( USHORT nLib, USHORT nNewPos ) = 0;

	virtual BOOL		HasPassword( USHORT nLib ) const = 0;
	virtual String		GetPassword( USHORT nLib ) const = 0;
	virtual void		SetPassword( USHORT nLib, const String& rNewPassword ) = 0;

	// Der BasicManager gibt die Basics auch raus, wenn das Passwort nicht
	// geprueft wurde, da man auch ohne Passwort mit der Lib arbeiten kann.
	// Es ist Sache der App, was Sie dem Anwender ohne Passwort nicht gestattet,
	// also z.B. das Betrachten der Source oder das Anzeigen/Loeschen von Modulen.
	virtual BOOL		IsPasswordVerified( USHORT nLib ) const = 0;
	virtual void		SetPasswordVerified( USHORT nLib ) = 0;


	// Modify-Flag wird nur beim Speichern zurueckgesetzt.
	virtual BOOL		IsModified() const = 0;
	virtual BOOL		IsBasicModified() const = 0;
	virtual BOOL		IsManagerModified() const = 0;

	virtual BOOL		HasErrors() = 0;
	virtual void		ClearErrors() = 0;
	virtual BasicError* GetFirstError() = 0;
	virtual BasicError* GetNextError() = 0;
};

#endif	//_BASMGR_HXX
