/*************************************************************************
 *
 *  $RCSfile: id.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:02 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/**************************************************************************
								TODO
 **************************************************************************

 *************************************************************************/

#pragma hdrstop

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _CNTAPI_HXX
#include "cntapi.hxx"
#endif
#ifndef _CNTVNODE_HXX /* CntViewBase */
#include "cntvnode.hxx"
#endif
#include "id.hxx"

using namespace rtl;
using namespace com::sun::star::uno;
using namespace com::sun::star::lang;
using namespace com::sun::star::ucb;

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

//=========================================================================
//
// ChaosContentIdentifier Implementation.
//
//=========================================================================

#define CHAOSID_INIT()  \
    m_xSMgr( rxSMgr ), \
	m_bRoot( sal_False )

//=========================================================================

ChaosContentIdentifier::ChaosContentIdentifier(
	const Reference< XMultiServiceFactory >& rxSMgr, const OUString& aURL )
: CHAOSID_INIT()
{
	CntAnchorRef xAnchor( new CntAnchor( NULL, UniString( aURL ) ) );
	init( *xAnchor );
}

//=========================================================================
ChaosContentIdentifier::ChaosContentIdentifier(
	const Reference< XMultiServiceFactory >& rxSMgr, const CntAnchor& rAnchor )
: CHAOSID_INIT()
{
	init( rAnchor );
}

//=========================================================================
// virtual
ChaosContentIdentifier::~ChaosContentIdentifier()
{
}

//=========================================================================
void ChaosContentIdentifier::init( const CntAnchor& rAnchor )
{
	if ( !SAL_CONST_CAST( CntAnchor*, &rAnchor )->GetError() )
	{
		String aServiceURL( rAnchor.GetServiceURL() );
		m_aServiceURL = aServiceURL;
		CntURLTransformer::ToExternalURL( aServiceURL );
		m_aContentId  = aServiceURL;
		m_aViewId     = rAnchor.GetRootViewURL( TRUE );

		if ( m_aViewId.getLength() )
			m_aViewURL = rAnchor.GetViewURL( TRUE );

		m_bRoot = rAnchor.IsRootAnchor();
	}
}

//=========================================================================
//
// XInterface methods.
//
//=========================================================================

XINTERFACE_IMPL_3( ChaosContentIdentifier,
				   XTypeProvider,
				   XContentIdentifier, /* base of XViewIdentifier */
				   XViewIdentifier );

//=========================================================================
//
// XTypeProvider methods.
//
//=========================================================================

XTYPEPROVIDER_IMPL_2( ChaosContentIdentifier,
					  XTypeProvider,
				   	  XViewIdentifier );

//=========================================================================
//
// XContentIdentifier methods.
//
//=========================================================================

// virtual
OUString SAL_CALL ChaosContentIdentifier::getContentIdentifier()
	throw( RuntimeException )
{
	return m_aContentId;
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getContentProviderScheme()
	throw( RuntimeException )
{
	if ( !m_aProviderScheme.getLength() && m_aContentId.getLength() )
	{
		// The content provider scheme is the part before the first ':'
		// within the content id.
		sal_Int32 nPos = m_aContentId.indexOf( ':', 0 );
		if ( nPos != -1 )
		{
			OUString aScheme( m_aContentId.copy( 0, nPos ) );
			m_aProviderScheme = aScheme.toLowerCase();
		}
	}

	return m_aProviderScheme;
}

//=========================================================================
//
// XViewIdentifier methods.
//
//=========================================================================

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getViewIdentifier()
	throw( RuntimeException )
{
	return m_aViewId;
}

//=========================================================================
// virtual
sal_Bool SAL_CALL ChaosContentIdentifier::isRootIdentifier()
	throw( RuntimeException )
{
	return m_bRoot;
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getServiceURL()
	throw( RuntimeException )
{
	return m_aServiceURL;
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getParentServiceURL()
	throw( RuntimeException )
{
	getURLs();
	return m_aParentServiceURL;
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getRootServiceURL()
	throw( RuntimeException )
{
	getURLs();
	return m_aRootServiceURL;
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getViewURL( sal_Bool bStrict )
	throw( RuntimeException )
{
	if ( m_aViewURL.getLength() )
		return m_aViewURL;

	if ( bStrict )
		return OUString();

	return getServiceURL();
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getParentViewURL(	sal_Bool bStrict )
	throw( RuntimeException )
{
	const OUString& rURL = m_aViewURL.getLength() ? m_aViewURL : m_aServiceURL;

	CntAnchorRef xAnchor( new CntAnchor( NULL, UniString( rURL ) ) );
	if ( !xAnchor->GetError() )
		return OUString( UniString( xAnchor->GetParentViewURL( bStrict ) ) );

	if ( bStrict )
		return OUString();

	return getParentServiceURL();
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getRootViewURL( sal_Bool bStrict )
	throw( RuntimeException )
{
	const OUString& rURL = m_aViewURL.getLength() ? m_aViewURL : m_aServiceURL;

	CntAnchorRef xAnchor( new CntAnchor( NULL, UniString( rURL ) ) );
	if ( !xAnchor->GetError() )
		return OUString( UniString( xAnchor->GetRootViewURL( bStrict ) ) );

	if ( bStrict )
		return OUString();

	return getRootServiceURL();
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getPresentationURL()
	throw( RuntimeException )
{
	getURLs();
	return m_aPresentationURL;
}

//=========================================================================
// virtual
OUString SAL_CALL ChaosContentIdentifier::getCasePreservedURL()
	throw( RuntimeException )
{
	if ( !m_aCasePreservedURL.getLength() )
	{
		CntAnchorRef xAnchor( new CntAnchor( NULL, m_aServiceURL ) );
		if ( !xAnchor->GetError() )
			m_aCasePreservedURL =
				OUString( UniString( xAnchor->GetCaseSensitiveFileURL() ) );
		else
			m_aCasePreservedURL = getPresentationURL();
	}
	return m_aCasePreservedURL;
}

//=========================================================================
void ChaosContentIdentifier::getURLs()
{
	if ( !m_aPresentationURL.getLength() ) // pres. URL's len must always > 0
	{
		CntAnchorRef xAnchor( new CntAnchor( NULL, m_aServiceURL ) );
		if ( !xAnchor->GetError() )
		{
			// Once we have an anchor for our content id, it's very
			// 'cheap' to get most URL's provided by it.

			m_aParentServiceURL =
				OUString( UniString( xAnchor->GetParentServiceURL() ) );
			m_aRootServiceURL =
				OUString( UniString( xAnchor->GetRootServiceURL() ) );
			m_aPresentationURL =
				OUString( UniString( xAnchor->GetPresentationURL() ) );

			// Note: m_aCasePreservedURL should not be initilaized here,
			//       because xAnchor->GetCaseSensitiveFileURL() is really
			//       expensive and most probably the value will not be
			//       needed during this ChaosContentIdentifier's lifetime.
		}
	}
}

