/*************************************************************************
 *
 *  $RCSfile: taskbase.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:02 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CHAOS_TASKBASE_HXX
#define CHAOS_TASKBASE_HXX

#ifndef _COM_SUN_STAR_UCB_CONTENTTASKSTATUS_HPP_
#include <com/sun/star/ucb/ContentTaskStatus.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_HXX_
#include <com/sun/star/uno/Reference.hxx>
#endif
#ifndef _SFXCANCEL_HXX
#include <svtools/cancel.hxx>
#endif
#ifndef _SFXLSTNER_HXX
#include <svtools/lstner.hxx>
#endif
#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _VOS_CONDITN_HXX
#include <vos/conditn.hxx>
#endif

namespace com { namespace sun { namespace star {
	namespace lang { class XEventListener; }
	namespace ucb {
		class XContent;
		class XContentIdentifier;
		class XContentIdentifierFactory;
		class XContentTask;
		class XContentTaskEnvironment;
	}
	namespace uno { class Any; }
} } }
namespace cppu { class OInterfaceContainerHelper; }
namespace rtl { class OUString; }
class SfxBroadcaster;
class SfxHint;
class SfxPoolItem;

#if defined TF_UCB
namespace chaos {
#endif // TF_UCB

class ChaosContent;
struct CntErrorData;

//============================================================================
class ChaosTaskBase : public SfxListener
{
	enum { PROGRESS_NONE = 0x10000, PROGRESS_TEXT };

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentTaskEnvironment > m_xEnv;
	vos::OCondition                      m_aFinished;
	com::sun::star::ucb::ContentTaskStatus m_eState;
	ChaosContent*                        m_pOwner;
	com::sun::star::ucb::XContentTask*   m_pTask;
	SfxCancellable*                      m_pCancellable;
	cppu::OInterfaceContainerHelper* 	 m_pDisposeEventListeners;
	com::sun::star::uno::Any             m_aException;
	sal_uInt32							 m_nProgressState;
	sal_Bool							 m_bStarted  : 1;
	sal_Bool							 m_bExecSync : 1;
	sal_Bool							 m_bThrowException : 1;

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentIdentifierFactory > m_xIDFactory;

private:
	DECL_LINK( ErrorHandler, const CntErrorData* );

	void initIDFactory();
	void waitForCompletion();

public:
	ChaosTaskBase( ChaosContent* pOwner,
				   com::sun::star::ucb::XContentTask* pTask,
			       const com::sun::star::uno::Reference< com::sun::star::ucb::XContentTaskEnvironment >& xEnv );
	virtual ~ChaosTaskBase();

	sal_Bool isSynchronous() const { return m_bExecSync; }
	sal_Bool queryRun() const { return !m_bStarted; }
	com::sun::star::uno::Any run( const rtl::OUString& rName,
								  sal_Int32 nHandle,
								  const com::sun::star::uno::Any& rValue,
								  sal_Bool bSynchronous );
	com::sun::star::uno::Any run( const SfxPoolItem& rItem,
								  sal_Bool bSynchronous );
    void abort() { if ( m_pCancellable ) m_pCancellable->Cancel(); }

    com::sun::star::ucb::ContentTaskStatus getStatus() { return m_eState; }
	sal_Bool setStatus( const com::sun::star::ucb::ContentTaskStatus& rNewState );

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentTask > getContentTask();
    com::sun::star::uno::Reference< com::sun::star::ucb::XContentTaskEnvironment > getEnvironment() { return m_xEnv; }
	ChaosContent* getOwner() const { return m_pOwner; }
	com::sun::star::uno::Reference< com::sun::star::ucb::XContent > getContent() const;
	SfxCancellable* getCancellable() const { return m_pCancellable; }

    void dispose();
    void addEventListener( const com::sun::star::uno::Reference< com::sun::star::lang::XEventListener >& Listener );
    void removeEventListener( const com::sun::star::uno::Reference< com::sun::star::lang::XEventListener >& Listener );

	void notifyContentEvent( sal_Int32 nAction,
							 const com::sun::star::uno::Reference< com::sun::star::ucb::XContent >& rContent,
							 const com::sun::star::uno::Reference< com::sun::star::ucb::XContentIdentifier >& rId );

	// SfxListener
	virtual void Notify( SfxBroadcaster& rBC, const SfxHint& rHint );
};

#if defined TF_UCB
}
#endif // TF_UCB

#endif // CHAOS_TASKBASE_HXX

