/*************************************************************************
 *
 *  $RCSfile: chttpdog.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _CHTTPDOG_CXX "$Revision: 1.1.1.1 $"

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _DATETIME_HXX
#include <tools/datetime.hxx>
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _VOS_MACROS_HXX
#include <vos/macros.hxx>
#endif
#ifndef _VOS_THREAD_HXX
#include <vos/thread.hxx>
#endif
#ifndef _VOS_CONDITN_HXX
#include <vos/conditn.hxx>
#endif
#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif

#ifndef _SFXBRDCST_HXX
#include <svtools/brdcst.hxx>
#endif
#ifndef _SFXLSTNER_HXX
#include <svtools/lstner.hxx>
#endif
#ifndef _SFXSMPLHINT_HXX
#include <svtools/smplhint.hxx>
#endif
#ifndef _SVTOOLS_CINTITEM_HXX
#include <svtools/cintitem.hxx>
#endif
#ifndef _CNTWIDS_HRC
#include <svtools/cntwids.hrc>
#endif

#ifndef _INETMSG_HXX
#include <svtools/inetmsg.hxx>
#endif

#ifndef _CNTHTTPIMP_HXX
#include "chttpimp.hxx"
#endif
#ifndef _CNTHTTPDOG_HXX
#include "chttpdog.hxx"
#endif

#ifndef _CNTSTGND_HXX
#include "cntstgnd.hxx"
#endif
#ifndef _DHDRITEM_HXX
#include "dhdritem.hxx"
#endif
#ifndef _CHAOS_STORITEM_HXX
#include "storitem.hxx"
#endif

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

#ifdef _CHAOS_STORITEM_REF
SV_IMPL_REF(CntStoreItemSet);
#endif

/*========================================================================
 *
 * CntHTTPWatchDog Internals.
 *
 *======================================================================*/
#define CNTHTTP_CACHE_LOCK()   if (m_pLock) m_pLock->acquire()
#define CNTHTTP_CACHE_UNLOCK() if (m_pLock) m_pLock->release()

/*
 * CntStoreItemSet ranges.
 */
CHTTPDOC_DIR_RANGES_IMPL(aDocDirRanges_Impl);

/*
 * CntHTTPCacheEntry.
 */
struct CntHTTPCacheEntry
{
	/** Representation.
	*/
	String m_aUrl;
	UINT32 m_nAttrib;
	ULONG  m_nSize;
	double m_nBadness;
	double m_nCachedAge;
	double m_nOriginAge;

	/** Construction.
	*/
	CntHTTPCacheEntry (const String& rUrl, UINT32 nAttrib, UINT32 nSize)
		: m_aUrl       (rUrl),
		  m_nAttrib    (nAttrib),
#if 1
		  m_nSize      ((((nSize + 1024) >> 10) << 10)),
#else
		  m_nSize      (nSize),
#endif
		  m_nCachedAge (0.0),
		  m_nOriginAge (0.0),
		  m_nBadness   (0.0)
	{}

	/** Comparison.
	*/
	BOOL operator< (const CntHTTPCacheEntry& rOther) const
	{
		return (m_nBadness < rOther.m_nBadness);
	}

	/** age [age = (DateTime(A) - DateTime(B)) [d]].
	*/
	static double age (
		const DateTime& rDTA, const DateTime& rDTB);

	/** analyze.
	*/
	void analyze (
		const CntDocHeader& rHdr, BOOL bKeepExpired = FALSE);
};

/*
 * age.
 */
double CntHTTPCacheEntry::age (
	const DateTime& rDTA, const DateTime& rDTB)
{
	const long nMilliSecPerDay = 24 * 60 * 60 * 1000;

	double nDays = (double)(Date(rDTA.GetDate()) - Date(rDTB.GetDate()));
	Time   aTime = Time(rDTA.GetTime()) - Time(rDTB.GetTime());

	nDays += ((double)(aTime.GetMSFromTime())) / ((double)nMilliSecPerDay);
	if (nDays < 0.0) nDays *= -1.0;
	return nDays;
}

/*
 * analyze.
 */
void CntHTTPCacheEntry::analyze (
	const CntDocHeader& rHdr, BOOL bKeepExpired)
{
	DateTime aDateDT (0, 0);
	DateTime aExpiresDT (0, 0);
	DateTime aLastModifiedDT (0, 0);

	ULONG i, n = rHdr.GetHeaderCount();
	for (i = 0; i < n; i++)
	{
		if (rHdr.GetHeaderName(i).CompareIgnoreCaseToAscii (
			"Cache-Control") == COMPARE_EQUAL)
		{
			String aCacheControl (rHdr.GetHeaderValue(i));
			aCacheControl.EraseLeadingChars (' ');
			if ((aCacheControl.CompareIgnoreCaseToAscii("no-cache") == 0) ||
				(aCacheControl.CompareIgnoreCaseToAscii("private")  == 0)    )
			{
				m_nAttrib |= CNTDIRENTRY_ATTRIB_MARKEDFORDELETE;
				return;
			}
			continue;
		}

		if (rHdr.GetHeaderName(i).CompareIgnoreCaseToAscii (
			"Date") == COMPARE_EQUAL)
		{
			INetRFC822Message::ParseDateField (
				rHdr.GetHeaderValue(i), aDateDT);
			continue;
		}

		if (rHdr.GetHeaderName(i).CompareIgnoreCaseToAscii (
			"Expires") == COMPARE_EQUAL)
		{
			INetRFC822Message::ParseDateField (
				rHdr.GetHeaderValue(i), aExpiresDT);
			continue;
		}

		if (rHdr.GetHeaderName(i).CompareIgnoreCaseToAscii (
			"Last-Modified") == COMPARE_EQUAL)
		{
			INetRFC822Message::ParseDateField (
				rHdr.GetHeaderValue(i), aLastModifiedDT);
			continue;
		}
	}

	DateTime aCurDT;
	aCurDT.ConvertToUTC();

	if (aDateDT.IsValid())
	{
		// Evaluate CachedAge (CurDT - DateDT).
		m_nCachedAge = age (aCurDT, aDateDT);

		if (aLastModifiedDT.IsValid())
		{
			// Evaluate OriginAge (DateDT - LastModifiedDT).
			m_nOriginAge = age (aDateDT, aLastModifiedDT);
		}
	}

	if (!bKeepExpired)
	{
		// Check for expired entry.
		if (aExpiresDT.IsValid() && (aExpiresDT <= aCurDT))
			m_nAttrib |= CNTDIRENTRY_ATTRIB_MARKEDFORDELETE;

		// Check for stale entry.
		if ((0.0 < m_nOriginAge) && (m_nOriginAge < m_nCachedAge))
			m_nAttrib |= CNTDIRENTRY_ATTRIB_MARKEDFORDELETE;
	}
}

/*
 * downheap.
 */
static void downheap (CntHTTPCacheList &rList, ULONG n, ULONG k)
{
	CntHTTPCacheEntry *ei = NULL, *ej = NULL, *ek = rList.GetObject(k);
	while (k < n / 2)
	{
		ULONG i = k + k + 1;
		ei = rList.GetObject(i);
		if ((i + 1) < n)
		{
			ej = rList.GetObject(i + 1);
			if (*ei < *ej)
			{
				ei = ej;
				i += 1;
			}
		}
		if (!(*ek < *ei)) break;
		rList.Replace (ei, k);
		k = i;
	}
	rList.Replace (ek, k);
}

/*
 * heapsort.
 */
static void heapsort (CntHTTPCacheList &rList)
{
	CntHTTPCacheEntry *entry = NULL;
	ULONG i = 0, n = rList.Count();

	for (i = (n - 1) / 2 + 1; i > 0; i--)
		downheap (rList, n, i - 1);

	while (n > 0)
	{
		entry = rList.Replace (rList.GetObject(n - 1), (ULONG)0);
		rList.Replace (entry, n - 1);
		downheap (rList, --n, 0);
	}
}

/*========================================================================
 *
 * CntHTTPWatchDog Implementation.
 *
 *======================================================================*/
/*
 * CntHTTPWatchDog.
 */
CntHTTPWatchDog::CntHTTPWatchDog (void)
	: m_pCache       (NULL),
	  m_pLock        (NULL),
	  m_nSize        (0),
	  m_nSizeLimit   (CHTTPIMP_SIZELIMIT_DEFAULT),
	  m_bKeepExpired (FALSE)
{
	// Create thread.
	m_aCond.reset();
	create();
}

/*
 * ~CntHTTPWatchDog.
 */
CntHTTPWatchDog::~CntHTTPWatchDog (void)
{
	// Cleanup thread.
	terminate();
	m_aCond.set();

	if (getIdentifier() != getCurrentIdentifier())
		join();

	// Release lock and cache.
	CNTHTTP_CACHE_UNLOCK();
	m_pCache = NULL;

	// Cleanup.
	ULONG i = 0, n = m_aList.Count();
	while (i < n)
		delete m_aList.GetObject(i++);
	m_aList.Clear();
}

/*
 * activate.
 */
void CntHTTPWatchDog::activate (
	CntStorageNode        *pCache,
	NAMESPACE_VOS(OMutex) *pLock,
	ULONG                  nSizeLimit,
	BOOL                   bKeepExpired)
{
	if (!m_aCond.check())
	{
		if (pCache)
		{
			m_pCache       = pCache;
			m_pLock        = pLock;
			m_nSizeLimit   = nSizeLimit;
			m_bKeepExpired = bKeepExpired;

			StartListening (*m_pCache);
			m_aCond.set();
		}
	}
}

/*
 * run.
 */
void SAL_CALL CntHTTPWatchDog::run (void)
{
	setPriority (TPriority_BelowNormal);

	while (schedule())
	{
		if (m_pCache)
		{
			collect();
			cleanup();
			m_pCache = NULL;
		}

		if (schedule())
		{
			// Wait for next activation.
			m_aCond.reset();
			m_aCond.wait();
		}
	}
}

/*
 * collect.
 *
 * Removal candidates (in order of preference):
 * (1) entry is expired : ExpiresDT <= CurrentDT  (when not keeping expires)
 * (2) entry is stale   : nCachedAge > nOriginAge (when not keeping expires)
 * (3) entry badness    : nSize * nCachedAge      (always)
 */
void CntHTTPWatchDog::collect (void)
{
	// Check context.
	if (m_pCache == NULL)
		return;

	// Cleanup.
	ULONG n = m_aList.Count();
	while (n > 0)
		delete m_aList.GetObject(--n);
	m_aList.Clear();

	// Initialize iteration.
	CntStorageIterator it (0, CNTDIRENTRY_ATTRIB_HIDDEN);
	CntHTTPCacheEntry *entry = NULL;
	m_nSize = 0;

	CNTHTTP_CACHE_LOCK();
	if (m_pCache)
		m_pCache->iter(it);
	CNTHTTP_CACHE_UNLOCK();

	// Collect.
	while (m_pCache && !it.eof() && schedule())
	{
		entry = new CntHTTPCacheEntry (
			(*it).m_pszName, (*it).m_nAttrib, (*it).m_nSize);
		m_aList.Insert (entry, LIST_APPEND);

		if (!(entry->m_nAttrib & CNTDIRENTRY_ATTRIB_MARKEDFORDELETE))
		{
			CNTHTTP_CACHE_LOCK();
			if (m_pCache)
			{
				CntStoreItemSetRef xDirSet = m_pCache->openItemSet (
					aDocDirRanges_Impl, entry->m_aUrl, STREAM_STD_READ);
				if (xDirSet.Is())
				{
					ULONG nDocSize = ITEMSET_VALUE(
						xDirSet, CntUInt32Item, WID_DOCUMENT_SIZE);
					entry->m_nSize += (((nDocSize + 1024) >> 10) << 10);

					CntDocHeaderRef xHdr (ITEMSET_VALUE(
						xDirSet, CntDocHeaderItem, WID_DOCUMENT_HEADER));
					if (xHdr.Is())
						entry->analyze (*xHdr, m_bKeepExpired);

					xDirSet.Clear();
				}
			}
			CNTHTTP_CACHE_UNLOCK();
		}

		if (!(entry->m_nAttrib & CNTDIRENTRY_ATTRIB_MARKEDFORDELETE))
		{
			// Add to remaining size.
			m_nSize += entry->m_nSize;
		}

		CNTHTTP_CACHE_LOCK();
		if (m_pCache)
			m_pCache->iter(it);
		CNTHTTP_CACHE_UNLOCK();
	}

	// Check SizeLimit.
	if ((m_nSize > m_nSizeLimit) && schedule())
	{
		// Evaluate badness (Size * CachedAge).
		n = m_aList.Count();
		while (n > 0)
		{
			entry = m_aList.GetObject(--n);
			entry->m_nBadness = entry->m_nSize * entry->m_nCachedAge;

			if (entry->m_nAttrib & CNTDIRENTRY_ATTRIB_MARKED)
				entry->m_nBadness *= 0.1;
		}

		// Sort into increasing badness.
		heapsort (m_aList);

		// Mark entries until below SizeLimit.
		n = m_aList.Count();
		while ((n > 0) && (m_nSize > m_nSizeLimit))
		{
			entry = m_aList.GetObject(--n);

			// Mark for delete.
			entry->m_nAttrib &= ~CNTDIRENTRY_ATTRIB_MARKED;
			entry->m_nAttrib |=  CNTDIRENTRY_ATTRIB_MARKEDFORDELETE;

			// Remove from remaining size.
			m_nSize -= entry->m_nSize;
		}
	}

	// Store remaining size.
	if (m_pCache)
	{
		CntNodeJob *pJob = new CntNodeJob (
			NULL, m_pCache, m_pCache, CntUInt32Item (WID_SIZE, m_nSize));
		m_pCache->InsertJob (pJob);
	}
}

/*
 * cleanup.
 */
void CntHTTPWatchDog::cleanup (void)
{
	// Check context.
	if (m_pCache == NULL)
		return;

	// Iterate.
	while (m_aList.Count() && schedule())
	{
		CntHTTPCacheEntry *entry = m_aList.Remove();

		UINT32 nAttrib = entry->m_nAttrib;
		if ((nAttrib & CNTDIRENTRY_ATTRIB_MARKEDFORDELETE) &&
			(!(nAttrib & CNTDIRENTRY_ATTRIB_MARKED)))
		{
			CNTHTTP_CACHE_LOCK();
			if (m_pCache)
			{
				ErrCode eErrCode = m_pCache->attrib (
					entry->m_aUrl,
					CNTDIRENTRY_ATTRIB_MARKED,
					CNTDIRENTRY_ATTRIB_MARKEDFORDELETE);
				if (eErrCode == ERRCODE_NONE)
				{
					// Setup body URL.
					String aBodyUrl (S2U ("body:"));
					aBodyUrl += entry->m_aUrl.Copy(5);

					// Remove body stream.
					eErrCode = m_pCache->remove (aBodyUrl);
					if ((eErrCode == ERRCODE_NONE        ) ||
						(eErrCode == ERRCODE_IO_NOTEXISTS)    )
					{
						// Remove head itemset.
						m_pCache->remove (entry->m_aUrl);
					}
				}
			}
			CNTHTTP_CACHE_UNLOCK();
		}
		delete entry;
	}
}

/*
 * Notify.
 */
void CntHTTPWatchDog::Notify (SfxBroadcaster &rBC, const SfxHint &rHint)
{
	SfxBroadcaster *pCache = m_pCache;
	if ((pCache != NULL) && (pCache == &rBC))
	{
		const SfxSimpleHint *pHint = PTR_CAST(SfxSimpleHint, &rHint);
		if (pHint && (pHint->GetId() == SFX_HINT_DYING))
		{
			CNTHTTP_CACHE_LOCK();

			EndListening (*m_pCache);
			m_pCache = NULL;

			CNTHTTP_CACHE_UNLOCK();
		}
	}
}

