/*************************************************************************
 *
 *  $RCSfile: cnewsmsg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _EINF_HXX
#include <tools/errinf.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef _CNTWIDS_HRC
#include <svtools/cntwids.hrc>
#endif
#ifndef _DATETIMEITEM_HXX
#include <svtools/dateitem.hxx>
#endif
#ifndef _SVTOOLS_CINTITEM_HXX
#include <svtools/cintitem.hxx>
#endif

#ifndef _CNTNEWSIMP_HXX
#include "cnewsimp.hxx"
#endif
#ifndef _CNTRNMGR_HXX
#include "cntrnmgr.hxx"
#endif
#ifndef _CNTMMITM_HXX
#include "cntmmitm.hxx"
#endif
#ifndef _CNTMBITM_HXX
#include "cntmbitm.hxx"
#endif
#ifndef _CNTVWITM_HXX
#include "cntvwitm.hxx"
#endif
#ifndef _CNTXREF_HXX
#include "cntxref.hxx"
#endif
#ifndef _CSTRITEM_HXX
#include "cstritem.hxx"
#endif
#ifndef _ILSTITEM_HXX
#include "ilstitem.hxx"
#endif

#ifndef _CNTRESID_HXX
#include "cntresid.hxx"
#endif
#ifndef _PROCHAOS_HRC
#include "prochaos.hrc"
#endif

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

#ifdef _USE_NAMESPACE
using namespace inet;
#endif

#ifdef _CHAOS_STORITEM_REF
SV_IMPL_REF(CntStoreItemSet);
#endif

/*=======================================================================
 *
 * CntNewsMsg... Internals.
 *
 *=====================================================================*/
/*
 * CntDirtyItemSet Ranges.
 */
CNEWSGRP_USER_RANGES_IMPL (aGrpUserRanges_Impl);
CNEWSMSG_DIR_RANGES_IMPL (aMsgDirRanges_Impl);

/*=======================================================================
 *
 * CntNewsMsgOpenJob_Impl implementation.
 *
 *=====================================================================*/
/*
 * CntNewsMsgOpenJob_Impl.
 */
CntNewsMsgOpenJob_Impl::CntNewsMsgOpenJob_Impl (
	CntNodeJob *pJob, CntNewsBoxNode_Impl *pParent)
	: CntNewsJob_Impl (pJob, pParent),
	  m_pCache         (NULL),
	  m_pDocNode       (NULL),
	  m_pMailer        (NULL),
	  m_pMessage       (NULL),
	  m_aXref          (WID_NEWS_XREF),
	  m_nHintId        (0),
	  m_eState         (STATE_NONE),
	  m_ePrev          (STATE_NONE)
{
}

/*
 * ~CntNewsMsgOpenJob_Impl.
 */
CntNewsMsgOpenJob_Impl::~CntNewsMsgOpenJob_Impl (void)
{
	delete m_pMessage;
}

/*
 * Execute.
 */
const SfxPoolItem* CntNewsMsgOpenJob_Impl::Execute (void)
{
	// Ensure clean destruction.
	CntNewsJob_ImplRef xThis (this);

	// Check context.
	CntNodeJob *pJob = GetJob();
	DBG_ASSERT (pJob, "CntNewsMsgOpenJob_Impl::Execute(): no Job");
	if (pJob == NULL)
		return NULL;

	// Check request.
	const SfxPoolItem *pReq = pJob->GetRequest();
	DBG_ASSERT (pReq, "CntNewsMsgOpenJob_Impl::Execute(): no Request");
	if (pReq == NULL)
	{
		// Failure.
		pJob->Cancel();
		return NULL;
	}

	// Initialize.
	if (m_eState == STATE_NONE)
	{
		// Obtain cache node.
		m_pCache = pJob->GetCacheNode();
		DBG_ASSERT(
			m_pCache, "CntNewsMsgOpenJob_Impl::Execute(): no Cache");
		if (m_pCache == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Obtain subject.
		m_pDocNode = pJob->GetSubject();
		DBG_ASSERT(
			m_pDocNode, "CntNewsMsgOpenJob_Impl::Execute(): no Subject");
		if (m_pDocNode == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Obtain CrossReference list.
		const CntItemListItem &rXrefList =
			(const CntItemListItem&)(m_pDocNode->Get (WID_NEWS_XREFLIST));
		if (rXrefList.Count())
		{
			// Obtain first cross reference.
			const CntCrossReferenceItem &rXref =
				(const CntCrossReferenceItem&)(rXrefList[0]);

			m_aXref.SetGroup (rXref.GetGroup());
			m_aXref.SetIndex (rXref.GetIndex());
		}

		// Check for Offline mode.
		if (GetParent()->GetConnMode() == CNT_CONN_MODE_OFFLINE)
		{
			if (pJob->SetError (ERRCODE_CHAOS_OFFLINE))
			{
				// Job cancelled.
				return NULL;
			}
		}

		// Check for Transacted mode.
		if (GetParent()->GetConnMode() == CNT_CONN_MODE_OFFLINE_TRANSACTED)
		{
			// Insert job into log and finish.
			pJob->Log();
			pJob->Done();
			return NULL;
		}

		// Obtain mailer.
		m_pMailer = GetParent()->GetMailer();
		DBG_ASSERT(
			m_pMailer, "CntNewsMsgOpenJob_Impl::Execute(): no Mailer");
		if (m_pMailer == NULL)
		{
			// Failure.
			pJob->Done();
			return NULL;
		}

		// Initiate article download.
		m_ePrev = m_eState;
		if (m_pMailer->NewsIsOpen())
			m_eState = STATE_OPEN;
		else
			m_eState = STATE_CLOSED;
	}

	// Jump into state machine.
	ExecuteHandler (m_pMailer, SERVICE_READY, NULL);
	return NULL;
}

/*
 * ExecuteHandler.
 */
void CntNewsMsgOpenJob_Impl::ExecuteHandler (
	INetCoreMailer *pMailer,
	sal_Int32 nReplyCode, const sal_Char *pReplyText)
{
	// Check context.
	CntNodeJob *pJob = GetJob();
	DBG_ASSERT(
		pJob, "CntNewsMsgOpenJob_Impl::ExecuteHandler(): no Job");
	if (pJob == NULL)
		return;

	// Check request.
	const SfxPoolItem *pReq = pJob->GetRequest();
	DBG_ASSERT(
		pReq, "CntNewsMsgOpenJob_Impl::ExecuteHandler(): no Request");
	if (pReq == NULL)
	{
		pJob->Cancel();
		return;
	}

	// Jump into state machine.
	while (1)
	{
		switch (m_eState)
		{
			case STATE_CLOSED:
				if (nReplyCode/100 == 2)
				{
					// Next state.
					m_ePrev  = m_eState;
					m_eState = STATE_OPEN;

					// Setup OpenJob.
					CntNodeJobRef xSubJob (new CntNodeJob (
						pJob, pJob->GetClient(), m_pDocNode, *pReq));

					CntNewsOpenJob_Impl *pJobImpl =
						new CntNewsOpenJob_Impl (xSubJob, GetParent());
					pJobImpl->SetCallback (
						CntNewsJob_Impl::ExecuteCallback, this);

					// Start OpenJob.
					GetRootNode()->RescheduleJob (xSubJob);
					return;
				}
				else
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_DONE;
				}
				break;

			case STATE_AUTH:
				if (nReplyCode/100 == 2)
				{
					// Resume previous state.
					m_eState = m_ePrev;
					m_ePrev  = STATE_AUTH;

					// Try again.
					GetRootNode()->RescheduleJob (pJob);
					return;
				}
				else if (nReplyCode == AUTHINFO_REQUIRED)
				{
					// Setup AuthJob.
					CntNodeJobRef xSubJob (new CntNodeJob (
						pJob, pJob->GetClient(), m_pDocNode, *pReq));

					CntNewsAuthJob_Impl *pJobImpl =
						new CntNewsAuthJob_Impl (xSubJob, GetParent());
					pJobImpl->SetReplyText (pReplyText);
					pJobImpl->SetCallback (
						CntNewsJob_Impl::ExecuteCallback, this);

					// Start AuthJob.
					GetRootNode()->RescheduleJob (xSubJob);
					return;
				}
				else if (nReplyCode == PERMISSION_DENIED)
				{
					// Previous authentication failed.
					CntNodeRef xRoot (GetRootNode());

					String aAccount (ITEMSET_VALUE_STRING(xRoot, WID_ACCOUNT));
					if (!(aAccount == m_aXref.GetGroup()))
					{
						aAccount = m_aXref.GetGroup();
						xRoot->Put (CntStringItem (WID_ACCOUNT, aAccount));

						CNT_RNM()->RemoveLoginInfo (OWN_URL (xRoot));
						xRoot->Put (CntStringItem (WID_USERNAME, String()));
						xRoot->Put (CntStringItem (WID_PASSWORD, String()));
					}

					// Authenticate.
					nReplyCode = AUTHINFO_REQUIRED;
				}
				else
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_DONE;
				}
				break;

			case STATE_OPEN:
				if (nReplyCode/100 == 2)
				{
					// Connection established. Retrieve article...
					if (!m_aXref.GetGroup().Len() || !m_aXref.GetIndex())
					{
						// ...by MessageID.
						m_ePrev  = m_eState;
						m_eState = STATE_ARTICLE;

						initDownload (*pMailer, m_pMessage);

						if (!pMailer->NewsArticle (
							ITEMSET_VALUE_STRING (m_pDocNode, WID_MESSAGE_ID),
							*m_pMessage,
							CntNewsJob_Impl::ExecuteCallback, this))
						{
							// Failure.
							clearDownload (*pMailer, *m_pMessage);

							// Notify caller.
							if (!pJob->SetError (ERRCODE_IO_UNKNOWN))
							{
								// Try again.
								m_eState = STATE_NONE;
								GetRootNode()->RescheduleJob (pJob);
							}
						}

						// Wait for next callback.
						return;
					}
					else
					{
						// ...by ArticleNumber. Select group.
						m_ePrev  = m_eState;
						m_eState = STATE_SELECT;

						m_aGroupEntry.initialize();
						m_aGroupEntry.m_aGroupName = m_aXref.GetGroup();

						if (!pMailer->NewsSelectGroup (
							m_aGroupEntry,
							CntNewsJob_Impl::ExecuteCallback, this))
						{
							// Failure. Notify caller.
							if (!pJob->SetError (ERRCODE_IO_UNKNOWN))
							{
								// Try again.
								m_eState = STATE_NONE;
								GetRootNode()->RescheduleJob (pJob);
							}
						}

						// Wait for next callback.
						return;
					}
				}
				else
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_DONE;
				}
				break;

			case STATE_SELECT:
				if (nReplyCode/100 == 2)
				{
					// Retrieve article by ArticleNumber.
					m_ePrev  = m_eState;
					m_eState = STATE_ARTICLE;

					initDownload (*pMailer, m_pMessage);

					if (!pMailer->NewsArticle (
						m_aXref.GetIndex(), *m_pMessage,
						CntNewsJob_Impl::ExecuteCallback, this))
					{
						// Failure.
						clearDownload (*pMailer, *m_pMessage);

						// Notify caller.
						if (!pJob->SetError (ERRCODE_IO_UNKNOWN))
						{
							// Try again.
							m_eState = STATE_NONE;
							GetRootNode()->RescheduleJob (pJob);
						}
					}

					// Wait for next callback.
					return;
				}
				else if ((nReplyCode == PERMISSION_DENIED) ||
						 (nReplyCode == AUTHINFO_REQUIRED)    )
				{
					// Setup state to resume and Authenticate.
					m_ePrev  = STATE_OPEN;
					m_eState = STATE_AUTH;
				}
				else
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_DONE;
				}
				break;

			case STATE_ARTICLE:
				if (nReplyCode/100 == 2)
				{
					// Show article download status.
					showDownload (*pMailer);

					// Save article.
					saveDownload (*pMailer, *m_pMessage, m_aXref);

					// Done.
					m_ePrev  = m_eState;
					m_eState = STATE_DONE;
				}
				else if (nReplyCode == TRANSFER_WAIT)
				{
					// Show article download status.
					showDownload (*pMailer);

					// Wait for next callback.
					return;
				}
				else if ((nReplyCode == NO_SUCH_ARTICLE       ) ||
						 (nReplyCode == NO_SUCH_ARTICLE_NUMBER)    )
				{
					// Cleanup.
					clearDownload (*pMailer, *m_pMessage);

					// Check cross reference.
					if (m_aXref.GetGroup().Len() && m_aXref.GetIndex())
					{
						// Retrieval by ArticleNumber didn't work.
						m_aXref.SetGroup (String());
						m_aXref.SetIndex (0);

						// Try retrieval by MessageID.
						m_ePrev  = m_eState;
						m_eState = STATE_OPEN;

						nReplyCode = SERVICE_READY;
					}
					else
					{
						// Both retrieval methods failed.
						m_ePrev  = m_eState;
						m_eState = STATE_DONE;
					}
				}
				else if ((nReplyCode == PERMISSION_DENIED) ||
						 (nReplyCode == AUTHINFO_REQUIRED)    )
				{
					// Cleanup.
					clearDownload (*pMailer, *m_pMessage);

					// Setup state to resume and Authenticate.
					m_ePrev  = STATE_OPEN;
					m_eState = STATE_AUTH;
				}
				else
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_DONE;
				}
				break;

			case STATE_DONE:
				if (nReplyCode/100 == 2)
				{
					// Finished.
					m_ePrev  = m_eState;
					m_eState = STATE_NONE;
					pJob->Done();
				}
				else if ((nReplyCode == NO_SUCH_ARTICLE       ) ||
						 (nReplyCode == NO_SUCH_ARTICLE_NUMBER)    )
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_NONE;

					// Notify caller.
					pJob->SetError (ERRCODE_CHAOS_MESSAGE_NOT_FOUND);
					pJob->Cancel();
				}
				else
				{
					// Failure.
					m_ePrev  = m_eState;
					m_eState = STATE_NONE;
					pJob->Cancel();
				}
				break;

			default: // STATE_NONE.
				return;
		}
	}
}

/*
 * initDownload.
 */
void CntNewsMsgOpenJob_Impl::initDownload (
	INetCoreMailer       &rMailer,
	INetCoreNewsMessage *&rpMessage)
{
	// Initialize download stream.
	CntNewsMessage_Impl::initDownload (rMailer, rpMessage, m_pCache);

	// Set transfer callback.
	rMailer.NewsSetTransferCallback (
		CntNewsJob_Impl::ExecuteCallback, this);
}

/*
 * clearDownload.
 */
void CntNewsMsgOpenJob_Impl::clearDownload (
	INetCoreMailer      &rMailer,
	INetCoreNewsMessage &rMessage)
{
	// Clear transfer callback.
	rMailer.NewsSetTransferCallback (NULL, NULL);

	// Cleanup download stream.
	CntNewsMessage_Impl::cleanupDownload (rMessage, m_pCache);
}

/*
 * queryDownload.
 */
BOOL CntNewsMsgOpenJob_Impl::queryDownload (const String &rNewsgroups)
{
	// Check Newsgroups header.
	if (rNewsgroups.Len())
	{
		// Initialize.
		BOOL bKnown = FALSE, bLocal = FALSE;

		// Check newsgroup settings.
		xub_StrLen i, n = rNewsgroups.GetTokenCount (',');
		for (i = 0; i < n; i++)
		{
			String aDirName (rNewsgroups.GetToken (i, ','));

			CntStorageNodeRef xDirNode (CntNewsBox_Impl::queryDirectory (
				m_pCache, aDirName, FALSE));
			if (xDirNode.Is())
			{
				const SfxPoolItem *pItem = NULL;
				SfxItemState eState = xDirNode->GetItemState (
					WID_MESSAGE_STOREMODE, TRUE, &pItem);
				if (eState >= SFX_ITEM_SET)
				{
					USHORT eMode = ITEM_VALUE (CntMsgStoreModeItem, *pItem);
					bLocal |= (eMode == CNT_MESSAGE_STOREMODE_LOCAL);
					bKnown  = TRUE;
				}
			}
		}

		// Check result.
		if (bKnown)
			return bLocal;
	}

	// Check global setting.
	USHORT eMode = ITEMSET_VALUE(
		m_pCache, CntMsgStoreModeItem, WID_MESSAGE_STOREMODE);
	return (eMode == CNT_MESSAGE_STOREMODE_LOCAL);
}

/*
 * saveDownload.
 */
BOOL CntNewsMsgOpenJob_Impl::saveDownload (
	INetCoreMailer              &rMailer,
	INetCoreNewsMessage         &rMessage,
	const CntCrossReferenceItem &rXref)
{
	// Clear transfer callback.
	rMailer.NewsSetTransferCallback (NULL, NULL);

	// Check MessageID.
	if (rMessage.GetMessageID().Len() == 0)
	{
		// Failure. Cleanup download stream.
		CntNewsMessage_Impl::cleanupDownload (rMessage, m_pCache);
		return FALSE;
	}

	// Update message properties.
	CntNewsMessage_Impl::updateProperties (
		*m_pDocNode, rMessage, rXref, m_pCache);

	// Setup message body URL.
	String aBodyUrl (OWN_URL (m_pDocNode));
	aBodyUrl.AppendAscii (".body");

	// Store message body stream.
	if (CntNewsMessage_Impl::storeDownload (rMessage, aBodyUrl, m_pCache))
	{
		// Check for persistent storage.
		BOOL bPersistent = queryDownload (rMessage.GetNewsgroups());

		// Create message body item.
		CntMessageBodyItem aBodyItem (WID_MESSAGEBODY, aBodyUrl);
		if (aBodyItem.Get (m_pCache, &rMailer, bPersistent))
		{
			// Deliver message body.
			m_pDocNode->Put (aBodyItem);
			return TRUE;
		}
	}
	return FALSE;
}

/*
 * showDownload.
 */
void CntNewsMsgOpenJob_Impl::showDownload (INetCoreMailer &rMailer)
{
	// Setup StatusText.
	String aStatusText (CntResId (
		RID_PROT_TRANSFERING_MESSAGE));
	aStatusText.SearchAndReplace (
		S2U("$(ARG1)"),
		UniString::CreateFromInt32 (rMailer.NewsGetTransferCount()));

	// Notify caller (ShowStatusText).
	m_nHintId = (USHORT)(~0);
	Broadcast (CntStatusBarHint (aStatusText));
}

/*
 * Notify.
 */
void CntNewsMsgOpenJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	CntNodeJob *pJob = PTR_CAST (CntNodeJob, &rBC);
	if ((pJob != NULL) && (pJob == GetJob()))
	{
		if (pJob->IsCancelled() || pJob->IsDone())
		{
			// Job finished.
			EndListening (*pJob);
			if (pJob->IsCancelled())
			{
				if (m_eState != STATE_NONE)
				{
					// Cancelled while busy. Close connection.
					if (m_pMailer)
						m_pMailer->NewsAbortConnection();

					// Force into idle state.
					m_ePrev  = m_eState;
					m_eState = STATE_NONE;
				}
			}

			if (m_pMailer && m_pMessage)
				clearDownload (*m_pMailer, *m_pMessage);

			if (m_nHintId)
			{
				// Notify caller (RemoveStatusText).
				Broadcast (CntStatusBarHint (String()));
				m_nHintId = 0;
			}
		}
	}
	CntNewsJob_Impl::Notify (rBC, rHint);
}

/*=======================================================================
 *
 * CntNewsMsgMarkJob_Impl implementation.
 *
 *=====================================================================*/
/*
 * CntNewsMsgMarkJob_Impl.
 */
CntNewsMsgMarkJob_Impl::CntNewsMsgMarkJob_Impl (
	CntNodeJob *pJob, CntNewsBoxNode_Impl *pParent)
	: CntNewsJob_Impl (pJob, pParent),
	  m_pUserData     (NULL),
	  m_eState        (STATE_NONE)
{
}

/*
 * ~CntNewsMsgMarkJob_Impl.
 */
CntNewsMsgMarkJob_Impl::~CntNewsMsgMarkJob_Impl (void)
{
}

/*
 * updateGroup.
 */
void CntNewsMsgMarkJob_Impl::updateGroup (
	const CntCrossReferenceItem &rXref,
	const CntBoolItem           &rProp)
{
	// Obtain UserData itemset.
	CntStoreItemSetRef xUserSet (m_pUserData->openItemSet (
		aGrpUserRanges_Impl, rXref.GetGroup(), STREAM_READWRITE));
	if (xUserSet.Is())
	{
		ULONG nCount = 0;

		// Check for 'Read' property.
		if (rProp.Which() == WID_IS_READ)
		{
			// Obtain current SeenRanges.
			CntRangesItem aSeenRanges (
				(const CntRangesItem&)(xUserSet->Get(WID_READ_RANGES)));

			// Update SeenRanges.
			ULONG nIndex = rXref.GetIndex();
			if (rProp.GetValue())
				aSeenRanges.InsertRange (nIndex, nIndex);
			else
				aSeenRanges.RemoveRange (nIndex, nIndex);
			xUserSet->Put (aSeenRanges);

			// Update SeenCount.
			nCount = aSeenRanges.TotalCount();
			xUserSet->Put (CntUInt32Item (
				WID_SEENCONTENTCOUNT, nCount));
		}

		// Check for 'Marked' property.
		if (rProp.Which() == WID_IS_MARKED)
		{
			// Obtain current MarkRanges.
			CntRangesItem aMarkRanges (
				(const CntRangesItem&)(xUserSet->Get(WID_MARK_RANGES)));

			// Update MarkRanges.
			ULONG nIndex = rXref.GetIndex();
			if (rProp.GetValue())
				aMarkRanges.InsertRange (nIndex, nIndex);
			else
				aMarkRanges.RemoveRange (nIndex, nIndex);
			xUserSet->Put (aMarkRanges);

			// Update MarkCount.
			nCount = aMarkRanges.TotalCount();
			xUserSet->Put (CntUInt32Item (
				WID_MARKED_DOCUMENT_COUNT, nCount));
		}

		// Setup GroupNode URL.
		String aGroupURL (OWN_URL (GetRootNode()));
		CNEWSIMP_CONCAT_URLPATH (aGroupURL, rXref.GetGroup());

		// Obtain GroupNode instance.
		CntNodeRef xGroup (GetRootNode()->Query (aGroupURL, FALSE));
		if (xGroup.Is())
		{
			// Update GroupNode instance.
			ULONG nKnown = ITEMSET_VALUE_UINT32(
				xGroup, WID_TOTALCONTENTCOUNT);
			if (nKnown)
				xGroup->Put (CntBoolItem (rProp.Which(), (nCount == nKnown)));
			xGroup->Put (*xUserSet);
		}
	}
}

/*
 * Execute.
 */
const SfxPoolItem* CntNewsMsgMarkJob_Impl::Execute (void)
{
	// Ensure clean destruction.
	CntNewsJob_ImplRef xThis (this);

	// Check context.
	CntNodeJob *pJob = GetJob();
	DBG_ASSERT (pJob, "CntNewsMsgMarkJob_Impl::Execute(): no Job");
	if (pJob == NULL)
		return NULL;

	// Check request.
	const CntBoolItem* pReq = PTR_CAST (CntBoolItem, pJob->GetRequest());
	DBG_ASSERT (pReq, "CntNewsMsgMarkJob_Impl::Execute(): no Request");
	if (pReq == NULL)
	{
		// Failure.
		pJob->Cancel();
		return NULL;
	}

	// Initialize.
	if (m_eState == STATE_NONE)
	{
		// Obtain subject.
		CntNode *pSubject = pJob->GetSubject();
		DBG_ASSERT(
			pSubject, "CntNewsMsgMarkJob_Impl::Execute(): no Subject");
		if (pSubject == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Check for attribute change.
		if (*pReq == pSubject->Get (pReq->Which()))
		{
			// Nothing to do.
			pJob->Done();
			return NULL;
		}

		// Obtain UserData node.
		m_pUserData = pJob->GetUserDataNode();
		DBG_ASSERT(
			m_pUserData, "CntNewsMsgMarkJob_Impl::Execute(): no UserData");
		if (m_pUserData == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Ensure nonblocking bulk operation.
		m_eState = STATE_MARK;
		GetRootNode()->RescheduleJob (pJob);
		return NULL;
	}

	if (m_eState == STATE_MARK)
	{
		// Obtain subject.
		CntNode *pSubject = pJob->GetSubject();

		// Obtain CrossReference list.
		const CntItemListItem &rXrefList =
			(const CntItemListItem&)(pSubject->Get (WID_NEWS_XREFLIST));

		// Process CrossReference list.
		USHORT i, n = rXrefList.Count();
		for (i = 0; i < n; i++)
		{
			const CntCrossReferenceItem &rXref =
				(const CntCrossReferenceItem&)(rXrefList[i]);
			updateGroup (rXref, *pReq);
		}

		// Update message node.
		pSubject->Put (*pReq);

		// Finish.
		m_eState = STATE_NONE;
	}

	// Done.
	pJob->Done();
	return NULL;
}

/*
 * Notify.
 */
void CntNewsMsgMarkJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	CntNewsJob_Impl::Notify (rBC, rHint);
}

