/*************************************************************************
 *
 *  $RCSfile: cnewsprt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _CNEWSPRT_CXX "$Revision: 1.1.1.1 $"

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _DATETIMEITEM_HXX
#include <svtools/dateitem.hxx>
#endif
#ifndef _SVTOOLS_CINTITEM_HXX
#include <svtools/cintitem.hxx>
#endif

#ifndef _CNTNEWSIMP_HXX
#include "cnewsimp.hxx"
#endif
#ifndef _CASTMACS_HXX
#include "castmacs.hxx"
#endif
#ifndef _CHAOS_IMPORT_HXX
#include "import.hxx"
#endif
#ifndef _CHAOS_EXPORT_HXX
#include "export.hxx"
#endif
#ifndef _CHAOS_STRMITEM_HXX
#include "strmitem.hxx"
#endif
#ifndef _CNTAPI_HXX
#include "cntapi.hxx"
#endif
#ifndef _CSTRITEM_HXX
#include "cstritem.hxx"
#endif
#ifndef _CNTXREF_HXX
#include "cntxref.hxx"
#endif
#ifndef _ILSTITEM_HXX
#include "ilstitem.hxx"
#endif
#ifndef _CNTRESID_HXX
#include "cntresid.hxx"
#endif
#ifndef _CNTRNMGR_HXX
#include "cntrnmgr.hxx"
#endif
#ifndef _PROCHAOS_HRC
#include "prochaos.hrc"
#endif

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

#ifdef _CHAOS_STORITEM_REF
SV_IMPL_REF(CntStoreItemSet);
#endif

/*=======================================================================
 *
 * CntNews... internals.
 *
 *=====================================================================*/
/*
 * CntDirtyItemSet Ranges.
 */
CNEWSGRP_USER_RANGES_IMPL (aGrpUserRanges_Impl);
CNEWSMSG_DIR_RANGES_IMPL (aMsgDirRanges_Impl);

/*=======================================================================
 *
 * CntNewsBoxMessageCachingPolicy.
 *
 *=====================================================================*/
class CntNewsBoxMessageCachingPolicy : public CntMessageCachingPolicy
{
	String           m_aBoxURL;
	CntStorageNode * m_pCache;

public:
	CntNewsBoxMessageCachingPolicy (
		const String & rTheBoxURL, CntStorageNode * pTheCache)
		: m_aBoxURL (rTheBoxURL), m_pCache (pTheCache)
	{}

	virtual BOOL doCache (
		const CntMBXVersion * pVersion,
		SfxItemSet          & rItems,
		String              & rName);

	virtual CntStorageNode * getCacheNode() { return m_pCache; }
};

/*
 * doCache.
 */
BOOL CntNewsBoxMessageCachingPolicy::doCache (
	const CntMBXVersion * pVersion,
	SfxItemSet          & rItems,
	String              & rName)
{
	if (pVersion && !pVersion->getVersion())
	{
		const CntMBXVersion0 * pVersion0 = PTR_CAST(CntMBXVersion0, pVersion);
		if (pVersion0 && pVersion0->getType() != CntMBXVersion0::TYPE_MESSAGE)
			return FALSE;
	}

	const SfxPoolItem * pItem = NULL;
	if (rItems.GetItemState(WID_MESSAGE_ID, TRUE, &pItem) < SFX_ITEM_SET)
		return FALSE;

	rName.Assign (m_aBoxURL);
	rName.Append (ITEM_VALUE(CntStringItem, *pItem));
	rName.AppendAscii (RTL_CONSTASCII_STRINGPARAM(".body"));
	return TRUE;
}

/*=======================================================================
 *
 * CntNewsBoxImportJob_Impl implementation.
 *
 *=====================================================================*/
/*
 * CntNewsBoxImportJob_Impl.
 */
CntNewsBoxImportJob_Impl::CntNewsBoxImportJob_Impl (
	CntNodeJob *pJob, CntNewsBoxNode_Impl *pParent)
	: CntNewsJob_Impl   (pJob, pParent),
	  m_pStream         (0),
	  m_pImport         (0),
	  m_nHintID         (0),
	  m_pPolicy         (0),
	  m_pNewsboxScanner (0),
	  m_eState			(STATE_LIST),
	  m_pGroupList		(0),
	  m_pLoadJob		(0)
{
	finishLastGroup();
}

/*
 * ~CntNewsBoxImportJob_Impl.
 */
CntNewsBoxImportJob_Impl::~CntNewsBoxImportJob_Impl (void)
{
	finishLastGroup();

	delete m_pPolicy;
	delete m_pImport;
	INSECURE_DYNAMIC_CAST(const CntInStreamItem *, GetJob()->GetRequest())->
	 releaseStream();
}

/*
 * Execute.
 */
const SfxPoolItem* CntNewsBoxImportJob_Impl::Execute (void)
{
	CntNewsJob_ImplRef xThis (this);

	CntNodeJob *pJob = GetJob();
	if (pJob == NULL)
		return NULL;

	if (m_eState == STATE_LIST)
	{
		m_pGroupList = &GetParent()->GetGroupList();
		m_eState = STATE_INIT;

		if (!m_pGroupList->Count())
		{
			// Setup CntNewsLoadListJob.
			m_pLoadJob = new CntNodeJob (
				pJob, pJob->GetClient(), pJob->GetSubject(),
				*(pJob->GetRequest()));

			CntNewsLoadListJob_Impl *pLoadJob =
			 new CntNewsLoadListJob_Impl (m_pLoadJob, GetParent());

			// Start child job.
			StartListening (*m_pLoadJob);
			GetRootNode()->RescheduleJob (m_pLoadJob);
			return NULL;
		}
	}

	if (m_eState == STATE_INIT)
	{
		const CntInStreamItem *pReq =
		 INSECURE_DYNAMIC_CAST(const CntInStreamItem *, pJob->GetRequest());
		if (pReq == NULL)
		{
			pJob->Cancel();
			return NULL;
		}

		m_pStream = pReq->tryToAcquireStream();
		if (!m_pStream)
		{
			pJob->Cancel();
			return NULL;
		}

		m_pImport = new CntImport(*m_pStream);

		if (m_pStream->IsA() == ID_FILESTREAM)
			m_sFileName
				= static_cast< SvFileStream * >(m_pStream)->GetFileName();
		m_bMessageFound = false;

		String aStatusText;
		if (m_sFileName.Len() > 0)
		{
			aStatusText = CntResId(RID_STATUS_IMPORT_FROM);
			aStatusText.SearchAndReplaceAscii("%1", m_sFileName);
		}
		else
			aStatusText = CntResId(RID_STATUS_IMPORT);

		m_nHintID = CntStatusBarHint::CreateHintId();
		Broadcast (CntStatusBarHint (
			m_nHintID, 0, m_pImport->getProgressMax(), aStatusText));

		m_aHostName = CntMBXFormat::getLocalServerName (
			ITEMSET_VALUE_STRING (GetRootNode(), WID_SERVERNAME));

		m_aBoxURL = OWN_URL(GetRootNode());
		if (!m_aBoxURL.Len() || m_aBoxURL.GetChar(m_aBoxURL.Len() - 1) != '/')
			m_aBoxURL += sal_Unicode('/');

		m_pPolicy = new CntNewsBoxMessageCachingPolicy (
			m_aBoxURL, pJob->GetCacheNode());

		m_eState = STATE_RUNNING;
	}

	ULONG nTicks = Time::GetSystemTicks();
	for (;;)
	{
		ULONG nProgress;

		if (m_pNewsboxScanner)
		{
			CntMBXToken aToken;
			ErrCode nError
			 = m_pNewsboxScanner->
				scan(aToken, CntMBXChars::aCharsRFC822,
					 CntMBXScanner::ScanMode
					  (CntMBXScanner::CANONIC_ATOM | CntMBXScanner::HASH_ATOM
					    | CntMBXScanner::SKIP_LWSP
					    | CntMBXScanner::SKIP_COMMENTS));
			if (nError)
			{
				pJob->SetError(nError);
				pJob->Cancel();
				return NULL;
			}

			if (aToken.getType() == CntMBXToken::TYPE_END)
			{
				nProgress = m_pNewsboxScanner->getInputProcessed();
				m_pImport->releaseBodyScanner();
				m_pNewsboxScanner = 0;
				m_aGroupName.Erase();
			}
			else
			{
				if (aToken.getType() == CntMBXToken::TYPE_ATOM)
					switch (aToken.getAtom()->getKey())
					{
						case CntMBXAtomTable::ATOM_GROUP:
							nError = handleNewsboxGroup(aToken);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return NULL;
							}
							break;

						case CntMBXAtomTable::ATOM_READ:
							nError = handleNewsboxReadMarked(TRUE, aToken);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return NULL;
							}
							break;

						case CntMBXAtomTable::ATOM_MARKED:
							nError = handleNewsboxReadMarked(FALSE, aToken);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return NULL;
							}
							break;

						case CntMBXAtomTable::ATOM_SUBSCRIBED:
							nError = handleNewsboxSubscribed(aToken);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return NULL;
							}
							break;
					}

				while (aToken.getType() != CntMBXToken::TYPE_CRLF
					   && aToken.getType() != CntMBXToken::TYPE_END)
				{
					nError
					 = m_pNewsboxScanner->
						scan(aToken, CntMBXChars::aCharsRFC822,
							 CntMBXScanner::ScanMode
							  (CntMBXScanner::SKIP_LWSP
							    | CntMBXScanner::SKIP_COMMENTS));
					if (nError)
					{
						pJob->SetError(nError);
						pJob->Cancel();
						return NULL;
					}
				}
			}
		}
		else
		{
			BOOL bFound;
			CntMBXVersion * pVersion = 0;
			static const USHORT aRanges[]
				= { WID_CHAOS_START, WID_INTERIM_URL,
					WID_OWN_URL, WID_CHAOS_END, 0 };
			SfxItemSet aItems(*(GetRootNode()->GetPool()), aRanges);

			ErrCode nError = m_pImport->getMessage(
				*m_pPolicy, bFound, pVersion, aItems, nProgress, &m_aHostName);
			if (nError)
			{
				delete pVersion;

				pJob->SetError(nError);
				pJob->Cancel();
				return NULL;
			}
			if (!bFound)
			{
				delete pVersion;
				break;
			}
			m_bMessageFound = true;

			CntMBXVersion0::MessageType eType = CntMBXVersion0::TYPE_MESSAGE;
			if (pVersion && !pVersion->getVersion())
			{
				const CntMBXVersion0 * pVersion0
				 = PTR_CAST(CntMBXVersion0, pVersion);
				if (pVersion0)
					eType = pVersion0->getType();
			}
			delete pVersion;

			switch (eType)
			{
				case CntMBXVersion0::TYPE_MESSAGE:
				{
					const SfxPoolItem * pItem;
					if (aItems.GetItemState(WID_MESSAGE_ID, FALSE, &pItem)
						 >= SFX_ITEM_SET)
						{
							String aMessageID(
								ITEM_VALUE(CntStringItem, *pItem));

							String aMessageURL(m_aBoxURL);
							aMessageURL += aMessageID;

							CntNodeRef xMessageNode (
								GetRootNode()->Query (aMessageURL));
							if (xMessageNode.Is())
								handleMessage(aItems, aMessageID);
						}
					break;
				}

				case CntMBXVersion0::TYPE_NEWSBOX:
					finishLastGroup();
					m_pNewsboxScanner = m_pImport->aquireBodyScanner();
					break;
			}
		}

		if (Time::GetSystemTicks() - nTicks > 250)
		{
			if (m_pNewsboxScanner)
				nProgress = m_pNewsboxScanner->getInputProcessed();
			else
				updateLastGroup();

			Broadcast (CntStatusBarHint(m_nHintID, nProgress));
			GetRootNode()->RescheduleJob (pJob);
			return NULL;
		}
	}

	if (!m_bMessageFound
		&& pJob->SetError(m_sFileName.Len() == 0 ?
						      ERRCODE_CHAOS_NOTHING_IMPORTED :
						      *new StringErrorInfo(
								       ERRCODE_CHAOS_NOTHING_IMPORTED_FROM,
									   m_sFileName)))
		return 0;

	pJob->Done();
	return NULL;
}

/*
 * Notify.
 */
void CntNewsBoxImportJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	CntNodeJob *pJob = PTR_CAST (CntNodeJob, &rBC);
	if ((pJob != NULL) && (pJob == m_pLoadJob))
	{
		if (pJob->IsCancelled() || pJob->IsDone())
		{
			// LoadListJob finished.
			EndListening (*m_pLoadJob);

			// Restart parent job.
			GetRootNode()->RescheduleJob (GetJob());
		}
		return;
	}
	if ((pJob != NULL) && (pJob == GetJob()))
	{
		if (pJob->IsCancelled() || pJob->IsDone())
		{
			EndListening (*pJob);

			if (m_nHintID)
				Broadcast(CntStatusBarHint(m_nHintID));
		}
	}
	CntNewsJob_Impl::Notify (rBC, rHint);
}

/*
 * updateLastGroup.
 */
void CntNewsBoxImportJob_Impl::updateLastGroup (void)
{
	if (!m_aLastGroupName.Len())
		return;

	if (m_bLastGroupKnownRangesModified
		|| m_bLastGroupReadRangesModified || m_bLastGroupMarkedRangesModified
		&& !m_xLastGroupNode.Is())
	{
		String aLastGroupURL(m_aBoxURL);
		aLastGroupURL += m_aLastGroupName;
		m_xLastGroupNode = GetRootNode()->Query(aLastGroupURL, FALSE);
	}

	if (m_bLastGroupKnownRangesModified)
	{
		m_xLastGroupDir->Put(m_aLastGroupKnownRanges);
		m_xLastGroupDir->Put(CntUInt32Item (
			WID_TOTALCONTENTCOUNT, m_aLastGroupKnownRanges.TotalCount()));
		if (m_xLastGroupNode.Is())
		{
			m_xLastGroupNode->Put(m_aLastGroupKnownRanges);
			m_xLastGroupNode->Put(CntUInt32Item (
				WID_TOTALCONTENTCOUNT, m_aLastGroupKnownRanges.TotalCount()));
		}
	}

	if (m_bLastGroupReadRangesModified)
	{
		m_xLastGroupUserSet->Put(m_aLastGroupReadRanges);
		m_xLastGroupUserSet->Put(CntUInt32Item (
			WID_SEENCONTENTCOUNT, m_aLastGroupReadRanges.TotalCount()));
		if (m_xLastGroupNode.Is())
		{
			m_xLastGroupNode->Put(m_aLastGroupReadRanges);
			m_xLastGroupNode->Put(CntUInt32Item (
				WID_SEENCONTENTCOUNT, m_aLastGroupReadRanges.TotalCount()));
		}
	}

	if (m_bLastGroupKnownRangesModified || m_bLastGroupReadRangesModified
		&& m_bLastGroupKnownRangesSet)
	{
		BOOL bRead = m_aLastGroupReadRanges.TotalCount()
			>= m_aLastGroupKnownRanges.TotalCount();
		m_xLastGroupUserSet->Put(CntBoolItem(WID_IS_READ, bRead));
		if (m_xLastGroupNode.Is())
			m_xLastGroupNode->Put(CntBoolItem(WID_IS_READ, bRead));
	}

	if (m_bLastGroupMarkedRangesModified)
	{
		m_xLastGroupUserSet->Put(m_aLastGroupMarkedRanges);
		if (m_xLastGroupNode.Is())
			m_xLastGroupNode->Put(m_aLastGroupMarkedRanges);
	}

	m_bLastGroupKnownRangesModified = FALSE;
	m_bLastGroupReadRangesModified = FALSE;
	m_bLastGroupMarkedRangesModified = FALSE;
}

/*
 * finishLastGroup.
 */
void CntNewsBoxImportJob_Impl::finishLastGroup (void)
{
	updateLastGroup();

	m_aLastGroupName.Erase();
	m_xLastGroupDir.Clear();
	m_xLastGroupNode.Clear();
	m_xLastGroupUserSet.Clear();
	m_bLastGroupKnownRangesSet = FALSE;
	m_bLastGroupKnownRangesModified = FALSE;
	m_bLastGroupReadRangesSet = FALSE;
	m_bLastGroupReadRangesModified = FALSE;
	m_bLastGroupMarkedRangesSet = FALSE;
	m_bLastGroupMarkedRangesModified = FALSE;
}

/*
 * handleNewsboxGroup.
 */
ErrCode CntNewsBoxImportJob_Impl::handleNewsboxGroup (CntMBXToken & rToken)
{
	// Parse rest of line into group name:
	m_aGroupName.Erase();
	ByteString aTheGroupName;
	for (;;)
	{
		ErrCode nError = m_pNewsboxScanner->scan(
			rToken, CntMBXChars::aCharsRFC822,
			CntMBXScanner::ScanMode(CntMBXScanner::CANONIC_ATOM
									| CntMBXScanner::SKIP_LWSP
									| CntMBXScanner::SKIP_COMMENTS));
		if (nError)
			return nError;

		if (rToken.getType() != CntMBXToken::TYPE_ATOM)
			return ERRCODE_NONE;

		aTheGroupName += rToken.getCanonic();

		nError = m_pNewsboxScanner->scan(
			rToken, CntMBXChars::aCharsRFC822,
			CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
									| CntMBXScanner::SKIP_COMMENTS));
		if (nError)
			return nError;

		if (rToken.getType() == CntMBXToken::TYPE_CRLF
			|| rToken.getType() == CntMBXToken::TYPE_END)
			break;
		if (rToken.getType() != CntMBXToken::TYPE_SPECIAL
			|| rToken.getValue() != ".")
			return ERRCODE_NONE;
		aTheGroupName += '.';
	}
	m_aGroupName = String(aTheGroupName, RTL_TEXTENCODING_UTF8);

	// Create a 'cache' storage entry for the group:
	CntStorageNode * pDirectory = GetJob()->GetDirectoryNode();
	if (pDirectory)
	{
		CntNodeRef xDirNode (CntNewsBox_Impl::queryDirectory (
			pDirectory, m_aGroupName, TRUE));
	}

	// If the group was previously unknown, append it to the group list, and
	// update the box's group count:
	BOOL bKnown = FALSE;
	for (ULONG i = m_pGroupList->Count(); i;)
		if (String(m_pGroupList->GetObject(--i)->m_aGroupName)
			    == m_aGroupName)
		{
			bKnown = TRUE;
			break;
		}
	if (!bKnown)
	{
		CntNewsGroupListEntry * pEntry = new CntNewsGroupListEntry;
		pEntry->m_aGroupName = m_aGroupName;
		pEntry->m_nArticleCount = 0;
		pEntry->m_nFirstArticleNumber = 0;
		pEntry->m_nLastArticleNumber = 0;
		pEntry->m_nFlags = CNTDIRENTRY_ATTRIB_NEW;

		m_pGroupList->Insert(pEntry, LIST_APPEND);

		CntStorageNode * pCache = GetJob()->GetCacheNode();
		if (pCache)
		{
			SvStream * pListStream
				= pCache->openStream(S2U("NEWSBOXLIST"), STREAM_WRITE);
			if (pListStream)
			{
				pListStream->Seek(STREAM_SEEK_TO_END);
				*pListStream << *pEntry;
				delete pListStream;

				// If creating a group list from scratch, pretend it has not
				// been updated for a very long time (to get all the remaining
				// groups on the server as new ones when updating/
				// synchronizing):
				CntStoreItemSetRef
				 xBoxSet(GetParent()->GetBoxDirItemSet(pCache));
				if (xBoxSet.Is())
				{
					if (xBoxSet->GetItemState(WID_LAST_UPDATE, FALSE)
						 != SFX_ITEM_SET)
						xBoxSet->Put(SfxDateTimeItem(
							WID_LAST_UPDATE, DateTime(Date(1, 1, 1970))));
					xBoxSet.Clear();
				}

				CntStorageNode * pDirectory = GetJob()->GetDirectoryNode();
				if (pDirectory)
				{
					CntStoreItemSetRef
					 xDirSet(GetParent()->GetBoxDirItemSet(pDirectory));
					if (xDirSet.Is())
					{
						CntUInt32Item
						 aTotal(WID_TOTALCONTENTCOUNT,
								ITEMSET_VALUE_UINT32(xDirSet,
													 WID_TOTALCONTENTCOUNT)
								 + 1);
						xDirSet->Put(aTotal);
						GetRootNode()->Put(aTotal);
					}
				}
			}
		}
	}

	return ERRCODE_NONE;
}

/*
 * handleNewsboxReadMarked.
 */
ErrCode CntNewsBoxImportJob_Impl::handleNewsboxReadMarked (
	BOOL bRead, CntMBXToken & rToken)
{
	// Only process 'read'/'marked' lines of valid group entries:
	if (!m_aGroupName.Len())
		return ERRCODE_NONE;

	// Parse rest of line into ranges:
	ErrCode nError = m_pNewsboxScanner->scan(
		rToken, CntMBXChars::aCharsRFC822,
		CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
								| CntMBXScanner::SKIP_COMMENTS));
	if (nError)
		return nError;

	if (rToken.getType() != CntMBXToken::TYPE_SPECIAL
		|| rToken.getValue() != ":")
		return ERRCODE_NONE;

	CntRangesItem aRanges(bRead ? WID_READ_RANGES : WID_MARK_RANGES);
	do
	{
		nError = m_pNewsboxScanner->scan(
			rToken, CntMBXChars::aCharsRanges,
			CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
									| CntMBXScanner::SKIP_COMMENTS));
		if (nError)
			return nError;

		if (!aRanges.Count()
			&& (rToken.getType() == CntMBXToken::TYPE_CRLF
				|| rToken.getType() == CntMBXToken::TYPE_END))
			break;
		if (rToken.getType() != CntMBXToken::TYPE_ATOM
			|| !rToken.getValue().IsNumericAscii())
			return ERRCODE_NONE;

		ULONG nLower = rToken.getValue().ToInt32();
		nError = m_pNewsboxScanner->scan(
			rToken, CntMBXChars::aCharsRanges,
			CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
									| CntMBXScanner::SKIP_COMMENTS));
		if (nError)
			return nError;

		ULONG nUpper;
		if (rToken.getType() == CntMBXToken::TYPE_SPECIAL
			&& rToken.getValue() == "-")
		{
			nError = m_pNewsboxScanner->scan(
				rToken, CntMBXChars::aCharsRanges,
				CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
										| CntMBXScanner::SKIP_COMMENTS));
			if (nError)
				return nError;

			if (rToken.getType() != CntMBXToken::TYPE_ATOM
				|| !rToken.getValue().IsNumericAscii())
				return ERRCODE_NONE;

			nUpper = ULONG(rToken.getValue().ToInt64());
			nError = m_pNewsboxScanner->scan(
				rToken, CntMBXChars::aCharsRanges,
				CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
										| CntMBXScanner::SKIP_COMMENTS));
			if (nError)
				return nError;
		}
		else
			nUpper = nLower;
		aRanges.InsertRange(nLower, nUpper);
	}
	while (rToken.getType() == CntMBXToken::TYPE_SPECIAL
		   && rToken.getValue() == ",");

	if (rToken.getType() != CntMBXToken::TYPE_CRLF
		&& rToken.getType() != CntMBXToken::TYPE_END)
		return ERRCODE_NONE;

	// Update group's 'user' storage:
	BOOL bRangesModified = FALSE;
	CntRangesItem * pNewRanges = 0;
	CntStorageNode * pUserData = GetJob()->GetUserDataNode();
	if (pUserData)
	{
		CntStoreItemSetRef xGroupItems (
			pUserData->openItemSet(aGrpUserRanges_Impl, m_aGroupName));
		if (xGroupItems.Is())
		{
			CntRangesItem aOldRanges (
				(const CntRangesItem&)
				(xGroupItems->Get(bRead ?
								   WID_READ_RANGES : WID_MARK_RANGES)));
			pNewRanges = aRanges.GetSubtractRanges(aOldRanges);
			aRanges.AddRanges(aOldRanges);
			bRangesModified = TRUE;

			xGroupItems->Put(aRanges);
			xGroupItems->Put(CntUInt32Item(bRead ?
										    WID_SEENCONTENTCOUNT :
										    WID_MARKED_DOCUMENT_COUNT,
										   aRanges.TotalCount()));
		}
	}

	// Update group node:
	String aGroupURL(m_aBoxURL);
	aGroupURL += m_aGroupName;
	CntNodeRef xGroupNode(GetRootNode()->Query(aGroupURL));
	if (xGroupNode.Is())
	{
		xGroupNode->Put(CntStringItem(WID_TITLE, m_aGroupName));

		if (!bRangesModified)
		{
			CntRangesItem aOldRanges (
				(const CntRangesItem&)
				(xGroupNode->Get(bRead ? WID_READ_RANGES : WID_MARK_RANGES)));
			pNewRanges = aRanges.GetSubtractRanges(aOldRanges);
			aRanges.AddRanges(aOldRanges);
		}

		xGroupNode->Put(aRanges);
		xGroupNode->Put(CntUInt32Item(bRead ?
									   WID_SEENCONTENTCOUNT :
									   WID_MARKED_DOCUMENT_COUNT,
									  aRanges.TotalCount()));

		ULONG nTotal = ITEMSET_VALUE_UINT32(
			xGroupNode, WID_TOTALCONTENTCOUNT);
		xGroupNode->Put (CntBoolItem (
			bRead ? WID_IS_READ : WID_IS_MARKED,
			((bRead || nTotal) && (aRanges.TotalCount() == nTotal))));
	}

	// All currently instantiaded messages nodes, for which the newly added
	// read/marked ranges have implicitly changed the read/marked state, must
	// be updated accordingly:
	if (pNewRanges)
	{
		CntStorageNode * pDirectory = GetJob()->GetDirectoryNode();
		CntNodeRef xGroupDirectory (CntNewsBox_Impl::queryDirectory (
			pDirectory, m_aGroupName, FALSE));
		CntStorageNode * pUserData = GetJob()->GetUserDataNode();
		if (xGroupDirectory.Is() && pUserData)
		{
			for (USHORT i = 0; i < pNewRanges->Count(); ++i)
			{
				ULONG j;
				ULONG nUpper;
				pNewRanges->GetRange(i, j, nUpper);
				for (; j <= nUpper; ++j)
				{
					CntStorageNode *pGrpDir =
					 STATIC_CAST(CntStorageNode *, &xGroupDirectory);

					String aXrefName('M');
					aXrefName += j;

					CntStoreItemSetRef xXrefItems (pGrpDir->openItemSet (
						aMsgDirRanges_Impl, aXrefName, STREAM_STD_READ));
					if (xXrefItems.Is())
					{
						const CntItemListItem & rList =
						 (const CntItemListItem&)
						 (xXrefItems->Get(WID_NEWS_XREFLIST));
						for (USHORT k = 0; k < rList.Count(); ++k)
						{
							if (rList[k].Which() == WID_NEWS_XREF)
							{
								const CntCrossReferenceItem & rXref =
									(const CntCrossReferenceItem&)rList[k];

								String aXrefGroupName(rXref.GetGroup());
								ULONG nXrefIndex = rXref.GetIndex();

								CntStoreItemSetRef xXrefGroupSet (
									pUserData->openItemSet (
										aGrpUserRanges_Impl, aXrefGroupName));
								if (!xXrefGroupSet.Is())
									continue;

								CntRangesItem aXrefGroupRanges (
									*INSECURE_DYNAMIC_CAST
									(const CntRangesItem *,
									 &xXrefGroupSet->
									 Get(bRead ?
										 WID_READ_RANGES : WID_MARK_RANGES)));
								aXrefGroupRanges.InsertRange(nXrefIndex,
															 nXrefIndex);
								xXrefGroupSet->Put(aXrefGroupRanges);

								UINT32 nXrefGroupCount
								 = aXrefGroupRanges.TotalCount();
								if (bRead)
									xXrefGroupSet->Put(CntUInt32Item(
										WID_SEENCONTENTCOUNT,
										nXrefGroupCount));

								String aXrefGroupURL(m_aBoxURL);
								aGroupURL += aXrefGroupName;

								CntNodeRef xXrefGroupNode (
									GetRootNode()->Query(aXrefGroupURL,
														 FALSE));
								if (xXrefGroupNode.Is())
								{
									xXrefGroupNode->Put(aXrefGroupRanges);
									if (bRead)
										xXrefGroupNode->Put(CntUInt32Item(
											WID_SEENCONTENTCOUNT,
											nXrefGroupCount));

									ULONG nTotal = ITEMSET_VALUE_UINT32(
										xXrefGroupNode,
										WID_TOTALCONTENTCOUNT);
									xXrefGroupNode->Put(CntBoolItem (
										bRead ? WID_IS_READ : WID_IS_MARKED,
										(bRead || nXrefGroupCount) &&
										(nXrefGroupCount == nTotal)));
								}

								CntNodeRef xXrefGroupDirectory (
									CntNewsBox_Impl::queryDirectory (
										pDirectory, aXrefGroupName, FALSE));
								if (xXrefGroupDirectory.Is())
								{
									CntStorageNode *pXrefGrpDir = STATIC_CAST(
										CntStorageNode *,
										&xXrefGroupDirectory);

									String aXrefMessageName('M');
									aXrefMessageName += nXrefIndex;

									CntStoreItemSetRef xXrefMessageSet (
										pXrefGrpDir->openItemSet (
											aMsgDirRanges_Impl,
											aXrefMessageName,
											STREAM_STD_READ));
									if (xXrefMessageSet.Is())
									{
										String aXrefMessageURL(m_aBoxURL);
										aXrefMessageURL
										 += ITEMSET_VALUE_STRING(
											 xXrefMessageSet, WID_MESSAGE_ID);
										xXrefMessageSet.Clear();

										CntNodeRef xXrefMessageNode (
											GetRootNode()->Query (
												aXrefMessageURL, FALSE));
										if (xXrefMessageNode.Is())
											xXrefMessageNode->Put(CntBoolItem(
												bRead ?
												WID_IS_READ : WID_IS_MARKED,
												TRUE));
									}
								}
							}
						}
					}
				}
			}
		}

		delete pNewRanges;
	}

	return ERRCODE_NONE;
}

/*
 * handleNewsboxSubscribed.
 */
ErrCode CntNewsBoxImportJob_Impl::handleNewsboxSubscribed (
	CntMBXToken & rToken)
{
	// Only process 'subscribed' lines of valid group entries:
	if (!m_aGroupName.Len())
		return ERRCODE_NONE;

	CntStorageNode * pViewData = GetJob()->GetViewDataNode();
	if (!pViewData)
		return ERRCODE_NONE;

	// Parse rest of line into 'yes'/'no':
	ErrCode nError = m_pNewsboxScanner->scan(
		rToken, CntMBXChars::aCharsRFC822,
		CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP
								| CntMBXScanner::SKIP_COMMENTS));
	if (nError)
		return nError;

	if (rToken.getType() != CntMBXToken::TYPE_SPECIAL
		|| rToken.getValue() != ":")
		return ERRCODE_NONE;

	nError = m_pNewsboxScanner->scan(
		rToken, CntMBXChars::aCharsRFC822,
		CntMBXScanner::ScanMode(CntMBXScanner::CANONIC_ATOM
								| CntMBXScanner::HASH_ATOM
								| CntMBXScanner::SKIP_LWSP
								| CntMBXScanner::SKIP_COMMENTS));
	if (nError)
		return nError;
	if (rToken.getType() != CntMBXToken::TYPE_ATOM)
		return ERRCODE_NONE;

	BOOL bSubscribed;
	switch (rToken.getAtom()->getKey())
	{
		case CntMBXAtomTable::ATOM_YES:
			bSubscribed = TRUE;
			break;

		case CntMBXAtomTable::ATOM_NO:
			bSubscribed = FALSE;
			break;

		default:
			return ERRCODE_NONE;
	}

	// Update view data:
	int nDelta = 0;
	if (bSubscribed)
	{
		UINT32 nAttrib = 0;
		pViewData->attrib(m_aGroupName, 0, 0, nAttrib);
		if (!(nAttrib & CNTDIRENTRY_ATTRIB_SUBSCRIBED))
		{
			nAttrib  =  CNTDIRENTRY_ATTRIB_SUBSCRIBED;
			nAttrib |= CNTDIRENTRY_ATTRIB_CREATE;
			pViewData->attrib(m_aGroupName, 0, nAttrib);
			nDelta = 1;
		}
	}
	else
	{
		pViewData->remove(m_aGroupName);
		nDelta = -1;
	}
	if (nDelta)
	{
		String aGroupURL(m_aBoxURL);
		aGroupURL += m_aGroupName;

		CntNodeRef xGroupNode(GetRootNode()->Query(aGroupURL));
		if (xGroupNode.Is())
		{
			xGroupNode->Put(CntStringItem(WID_TITLE, m_aGroupName));

			GetJob()->Result(xGroupNode);

			CntAnchor *pBoxAnchor = PTR_CAST(CntAnchor, GetJob()->GetClient());
			if (pBoxAnchor)
			{
				for (USHORT i = 0; i < pBoxAnchor->SubAnchorCount(); ++i)
				{
					CntAnchor * pGroupAnchor = pBoxAnchor->GetSubAnchor(i);
					if (pGroupAnchor->GetNode()->GetMostReferedNode()
						 == &xGroupNode)
						GetJob()->GetViewDataTarget(pGroupAnchor)->
						 CntInterface::Put(CntBoolItem(WID_FLAG_SUBSCRIBED,
													   nDelta > 0),
										   WID_FLAG_SUBSCRIBED);
				}
			}
		}

		CntStoreItemSetRef xViewSet (
			GetParent()->GetBoxViewItemSet(pViewData));
		if (xViewSet.Is())
		{
			UINT32 nCount = ITEMSET_VALUE(
				xViewSet, CntUInt32Item, WID_SUBSCRNEWSGROUPCOUNT);
			if (nDelta == 1)
				++nCount;
			else if (nCount)
				--nCount;

			CntUInt32Item aItem (WID_SUBSCRNEWSGROUPCOUNT, nCount);
			xViewSet->Put(aItem);
			GetJob()->GetViewDataTarget()->CntInterface::Put(*xViewSet);
		}
	}

	return ERRCODE_NONE;
}

/*
 * handleMessage.
 */
void CntNewsBoxImportJob_Impl::handleMessage (
	const SfxItemSet & rItems,
	const String     & rMessageID)
{
	CntStorageNode * pCacheData = GetJob()->GetCacheNode();
	CntStorageNode * pUserData  = GetJob()->GetUserDataNode();

	const CntItemListItem & rXrefList =
		(const CntItemListItem&)(rItems.Get(WID_NEWS_XREFLIST));
	for (USHORT i = 0; i < rXrefList.Count(); ++i)
	{
		if (rXrefList[i].Which() == WID_NEWS_XREF)
		{
			const CntCrossReferenceItem & rXrefItem =
				(const CntCrossReferenceItem &)rXrefList[i];

			String aGroup(rXrefItem.GetGroup());
			ULONG nIndex(rXrefItem.GetIndex());

			if (aGroup != m_aLastGroupName)
			{
				finishLastGroup();
				m_aLastGroupName = aGroup;
				m_xLastGroupDir =
					CntNewsBox_Impl::queryDirectory(pCacheData, aGroup);
			}
			if (m_xLastGroupDir.Is())
			{
				CntStorageNode *pLastGroupDir =
					STATIC_CAST(CntStorageNode *, &m_xLastGroupDir);

				CntStoreItemSetRef xMsgItems (pLastGroupDir->openItemSet (
					aMsgDirRanges_Impl, rMessageID));
				if (xMsgItems.Is())
				{
					xMsgItems->Put(rItems);

					String aRefID('M');
					aRefID += nIndex;
					pLastGroupDir->link(rMessageID, aRefID);

					if (pCacheData && m_xLastGroupDir.Is())
					{
						if (!m_bLastGroupKnownRangesSet)
						{
							m_bLastGroupKnownRangesSet = TRUE;
							m_aLastGroupKnownRanges
							 = *INSECURE_DYNAMIC_CAST
							     (const CntRangesItem *,
								  &m_xLastGroupDir->Get(WID_KNOWN_RANGES));
						}

						if (m_aLastGroupKnownRanges.InsertRange(nIndex,
																nIndex))
							m_bLastGroupKnownRangesModified = TRUE;
					}

					if (pUserData)
					{
						const SfxPoolItem * pItem;
						BOOL bReadFlagPresent
							= rItems.GetItemState(WID_IS_READ, TRUE, &pItem)
							>= SFX_ITEM_SET;
						if (bReadFlagPresent
							|| m_bLastGroupKnownRangesModified)
						{
							if (!m_xLastGroupUserSet.Is())
								m_xLastGroupUserSet = pUserData->openItemSet(
									aGrpUserRanges_Impl, aGroup);
							if (m_xLastGroupUserSet.Is())
							{
								if (!m_bLastGroupReadRangesSet)
								{
									m_bLastGroupReadRangesSet = TRUE;
									m_aLastGroupReadRanges
									 = *INSECURE_DYNAMIC_CAST
									     (const CntRangesItem *,
										  &m_xLastGroupUserSet->
										    Get(WID_READ_RANGES));
								}

								if (bReadFlagPresent)
									if (ITEM_VALUE(CntBoolItem, *pItem))
									{
										if (m_aLastGroupReadRanges.
											 InsertRange(nIndex, nIndex))
											m_bLastGroupReadRangesModified
											 = TRUE;
									}
									else if (m_aLastGroupReadRanges.
											  RemoveRange(nIndex, nIndex))
										m_bLastGroupReadRangesModified = TRUE;
							}
						}

						if (rItems.GetItemState(WID_IS_MARKED, TRUE, &pItem)
							 >= SFX_ITEM_SET)
						{
							if (!m_xLastGroupUserSet.Is())
								m_xLastGroupUserSet
								 = pUserData->openItemSet(aGrpUserRanges_Impl,
														  aGroup);
							if (m_xLastGroupUserSet.Is())
							{
								if (!m_bLastGroupMarkedRangesSet)
								{
									m_bLastGroupMarkedRangesSet = TRUE;
									m_aLastGroupMarkedRanges
									 = *INSECURE_DYNAMIC_CAST
								         (const CntRangesItem *,
										  &m_xLastGroupUserSet->
										    Get(WID_MARK_RANGES));
								}

								if (ITEM_VALUE(CntBoolItem, *pItem))
								{
									if (m_aLastGroupMarkedRanges.
										 InsertRange(nIndex, nIndex))
										m_bLastGroupMarkedRangesModified
										 = TRUE;
								}
								else if (m_aLastGroupMarkedRanges.
										  RemoveRange(nIndex, nIndex))
									m_bLastGroupMarkedRangesModified = TRUE;
							}
						}
					}
				}
			}
		}
	}
}

/*=======================================================================
 *
 *  CntNewsBoxExportJob_Impl implementation.
 *
 *=====================================================================*/

IMPL_LINK(CntNewsBoxExportJob_Impl, errorHandler, CntErrorData *, pData)
{
	switch (pData->_nError)
	{
		case ERRCODE_CHAOS_OFFLINE:
		{
			if (m_bIgnoreOffline)
				return ERRCODE_BUTTON_NO;
			USHORT nResult
			 = CNT_RNM()->HandleError(ERRCODE_CHAOS_OFFLINE, GetJob(),
									  pData->_pErrorStr, pData->_pData);
			if (nResult == ERRCODE_BUTTON_NO)
				m_bIgnoreOffline = TRUE;
			return nResult;
		}

		case ERRCODE_CHAOS_LOGIN_FAILURE_RECEIVE:
			return CNT_RNM()->HandleError(ERRCODE_CHAOS_LOGINFAIL_NOEXCH,
										  GetJob(), pData->_pErrorStr,
										  SAL_STATIC_CAST(CntLoginErrorData *,
														  pData->_pData)->
										   getTarget());
	}
	return ERRCODE_CHAOS_NOT_HANDLED;
}

/*
 * Execute.
 */
const SfxPoolItem* CntNewsBoxExportJob_Impl::Execute (void)
{
	CntNewsJob_ImplRef xThis (this);

	CntNodeJob *pJob = GetJob();
	if (pJob == NULL)
		return NULL;

	const CntExpStreamItem *pReq
	 = (const CntExpStreamItem *)(pJob->GetRequest());
	if (pReq == NULL || !pReq->hasStream())
	{
		pJob->Cancel();
		return NULL;
	}

	BOOL bAllFolders = ITEMSET_VALUE(pJob->GetClient(), CntFolderViewModeItem,
									 WID_FOLDERVIEW_MODE)
	                    == CNT_VIEW_ALL_FOLDERS;
	CntStorageNode * pIterationStorage
	 = bAllFolders ? pJob->GetDirectoryNode() :pJob->GetViewDataNode();
	if (!pIterationStorage)
	{
		pJob->Cancel();
		return 0;
	}

	if (!m_pStream)
	{
		m_pStream = pReq->tryToAcquireStream();
		if (!m_pStream)
		{
			/* @@@ BUSY WAITING @@@ */
			GetRootNode()->RescheduleJob(pJob);
			return NULL;
		}

		m_nHintID = CntStatusBarHint::CreateHintId();
		ULONG nMaxProgress = 0;
		if (bAllFolders)
			nMaxProgress
			 = 2 * ITEMSET_VALUE_UINT32(GetRootNode(), WID_TOTALCONTENTCOUNT);
		else
		{
			CntStoreItemSetRef
			 xViewSet(GetParent()->GetBoxViewItemSet(pIterationStorage));
			nMaxProgress
			 = xViewSet.Is() ?
			    2 * ITEMSET_VALUE_UINT32(xViewSet, WID_SUBSCRNEWSGROUPCOUNT) :
			    0;
		}
		String aStatusText;
		String aFileName;
		if (m_pStream->IsA() == ID_FILESTREAM)
			aFileName = INSECURE_DYNAMIC_CAST(SvFileStream *, m_pStream)->
			             GetFileName();
		if (aFileName.Len())
		{
			aStatusText = CntResId(RID_STATUS_EXPORT_TO);
			aStatusText.SearchAndReplaceAscii("%1", aFileName);
		}
		else
			aStatusText = CntResId(RID_STATUS_EXPORT);
		Broadcast(CntStatusBarHint(m_nHintID, 0, nMaxProgress, aStatusText));

		m_pExport = new CntExport(*m_pStream);

		m_bWriteNewsbox = TRUE;
		m_pIterator = bAllFolders ?
		               new CntStorageIterator(CNTSTORE_ATTRIB_ISDIR, 0) :
		               new CntStorageIterator(0,
											  CNTDIRENTRY_ATTRIB_HIDDEN
											   | CNTSTORE_ATTRIB_ISDIR);
		ErrCode nError = m_pExport->writeHeader(*GetRootNode());
		if (nError)
		{
			pJob->SetError(nError);
			pJob->Cancel();
			return 0;
		}
	}

	ULONG nTicks = Time::GetSystemTicks();

	if (m_bWriteNewsbox)
	{
		for (;;)
		{
			String aGroupName(pIterationStorage->iter(*m_pIterator));
			if (!aGroupName.Len())
				break;

			ByteString aLine(RTL_CONSTASCII_STRINGPARAM("group "));
			aLine += ByteString(aGroupName, RTL_TEXTENCODING_UTF8);
			ErrCode nError = m_pExport->writeBody(aLine);
			if (nError)
			{
				pJob->SetError(nError);
				pJob->Cancel();
				return 0;
			}

			CntStorageNode * pUserData = pJob->GetUserDataNode();
			if (pUserData)
			{
				CntStoreItemSetRef xGroupItems (
					pUserData->openItemSet(aGroupName, STREAM_STD_READ));
				if (xGroupItems.Is())
				{
					const SfxPoolItem * pItem = NULL;
					if (xGroupItems->GetItemState(
						WID_READ_RANGES, TRUE, &pItem) >= SFX_ITEM_SET)
					{
						const CntRangesItem & rRanges
						 = *INSECURE_DYNAMIC_CAST(const CntRangesItem *,
												  pItem);
						if (rRanges.Count())
						{
							aLine.
								Assign(RTL_CONSTASCII_STRINGPARAM(" read: "));
							aLine += CntMBXFormat::translateRanges(rRanges);
							nError = m_pExport->writeBody(aLine);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return NULL;
							}
						}
					}

					if (xGroupItems->GetItemState(
						WID_MARK_RANGES, TRUE, &pItem) >= SFX_ITEM_SET)
					{
						const CntRangesItem & rRanges
						 = *INSECURE_DYNAMIC_CAST(const CntRangesItem *,
												  pItem);
						if (rRanges.Count())
						{
							aLine.Assign(RTL_CONSTASCII_STRINGPARAM(
								             " marked: "));
							aLine += CntMBXFormat::translateRanges(rRanges);
							nError = m_pExport->writeBody(aLine);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return NULL;
							}
						}
					}
				}
			}

			CntStorageNode * pViewData = pJob->GetViewDataNode();
			if (pViewData)
			{
				UINT32 nAttrib = 0;
				pViewData->attrib(aGroupName, 0, 0, nAttrib);
				aLine.Assign(RTL_CONSTASCII_STRINGPARAM(" subscribed: "));
				aLine += CntMBXFormat::translateBoolean(
					         (nAttrib & CNTDIRENTRY_ATTRIB_SUBSCRIBED) != 0);
				nError = m_pExport->writeBody(aLine);
				if (nError)
				{
					pJob->SetError(nError);
					pJob->Cancel();
					return NULL;
				}
			}

			++m_nProgress;

			if (Time::GetSystemTicks() - nTicks > 250)
			{
				Broadcast(CntStatusBarHint(m_nHintID, m_nProgress));
				GetRootNode()->RescheduleJob(pJob);
				return 0;
			}
		}

		m_bWriteNewsbox = FALSE;
		delete m_pIterator;
		m_pIterator = bAllFolders ?
		               new CntStorageIterator(CNTSTORE_ATTRIB_ISDIR, 0) :
		               new CntStorageIterator(0,
											  CNTDIRENTRY_ATTRIB_HIDDEN
											   | CNTSTORE_ATTRIB_ISDIR);
	}

	String aBoxURL(OWN_URL(GetRootNode()));
	if (!aBoxURL.Len() || aBoxURL.GetChar(aBoxURL.Len() - 1) != '/')
		aBoxURL += '/';
	String
	 aLocalServerName(CntMBXFormat::getLocalServerName
	                                 (ITEMSET_VALUE_STRING(GetRootNode(),
														   WID_SERVERNAME)));
	for (;;)
	{
		if (!m_xGroupDirectory.Is())
		{
			m_aGroupName = pIterationStorage->iter(*m_pIterator);
			if (!m_aGroupName.Len())
				break;

			m_xGroupDirectory = CntNewsBox_Impl::queryDirectory(
				pJob->GetDirectoryNode(), m_aGroupName);
			if (m_xGroupDirectory.Is())
				m_pGroupIterator = new CntStorageIterator;
		}

		if (m_xGroupDirectory.Is())
		{
			for (;;)
			{
				if (!m_xMessageNode)
				{
					String aMessageName(STATIC_CAST(CntStorageNode *,
													&m_xGroupDirectory)->
										 iter(*m_pGroupIterator));
					if (!aMessageName.Len())
						break;

					String aMessageURL(aBoxURL);
					aMessageURL += aMessageName;
					m_xMessageNode = GetRootNode()->Query(aMessageURL);
					if (m_xMessageNode.Is())
					{
						if (!m_xMessageNode->IsInserted())
						{
							CntStoreItemSetRef
							 xMessageSet(STATIC_CAST(CntStorageNode *,
													 &m_xGroupDirectory)->
										  openItemSet(aMsgDirRanges_Impl,
													  aMessageName,
													  STREAM_STD_READ));
							if (xMessageSet.Is())
							{
								m_xMessageNode->Put(*xMessageSet);
								xMessageSet.Clear();
							}
						}

						GetParent()->
						 LoadMessageBodyItem(m_xMessageNode,
											 pJob->GetCacheNode());
						if (pReq->doForceBodies()
							&& m_xMessageNode->GetItemState(WID_MESSAGEBODY)
						        != SFX_ITEM_SET)
						{
							m_xOpenJob
							 = new
							    CntNodeJob(pJob, m_xMessageNode,
										   m_xMessageNode,
										   CntOpenModeItem(WID_OPEN,
														   CNT_OPEN_MESSAGE));
							m_xOpenJob->
							 SetErrorHandler(LINK(this,
												  CntNewsBoxExportJob_Impl,
												  errorHandler));
							StartListening(*m_xOpenJob);
							GetRootNode()->ExecuteJob(m_xOpenJob);
							return 0;
						}
					}
				}

				if (m_xMessageNode.Is())
				{
					ErrCode nError
					 = m_pExport->writeMessage(*m_xMessageNode,
											   &aLocalServerName);
					m_xMessageNode.Clear();
					if (nError)
					{
						pJob->SetError(nError);
						pJob->Cancel();
						return NULL;
					}
				}

				if (Time::GetSystemTicks() - nTicks > 250)
				{
					Broadcast(CntStatusBarHint(m_nHintID, m_nProgress));
					GetRootNode()->RescheduleJob(pJob);
					return NULL;
				}
			}

			m_xGroupDirectory.Clear();
			DELETEZ(m_pGroupIterator);
		}

		++m_nProgress;

		if (Time::GetSystemTicks() - nTicks > 250)
		{
			Broadcast(CntStatusBarHint(m_nHintID, m_nProgress));
			GetRootNode()->RescheduleJob(pJob);
			return NULL;
		}
	}

	pJob->Done();
	return NULL;
}

/*
 * Notify.
 */
void CntNewsBoxExportJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	if (&rBC == &m_xOpenJob)
	{
		CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
		if (pStatusHint
			&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
				|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
				   && pStatusHint->GetError() == ERRCODE_ABORT))
		{
			EndListening(*m_xOpenJob);
			m_xOpenJob.Clear();
			GetRootNode()->RescheduleJob(GetJob());
		}
	}
	else
	{
		if (&rBC == GetJob())
		{
			CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
			if (pStatusHint
				&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
					|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
					   && pStatusHint->GetError() == ERRCODE_ABORT))
			{
				if (m_xOpenJob.Is())
					EndListening(*m_xOpenJob);
				delete m_pGroupIterator;
				delete m_pIterator;
				delete m_pExport;
				if (m_pStream)
					INSECURE_DYNAMIC_CAST(const CntExpStreamItem *,
										  GetJob()->GetRequest())->
					 releaseStream();
				if (m_nHintID)
					Broadcast(CntStatusBarHint(m_nHintID));
			}
		}
		CntNewsJob_Impl::Notify (rBC, rHint);
	}
}

/*=======================================================================
 *
 * CntNewsMsgExportJob_Impl implementation.
 *
 *=====================================================================*/

/*
 * Execute.
 */
const SfxPoolItem* CntNewsMsgExportJob_Impl::Execute (void)
{
	CntNewsJob_ImplRef xThis (this);

	CntNodeJob *pJob = GetJob();
	if (pJob == NULL)
		return NULL;

	const CntExpStreamItem *pReq
	 = (const CntExpStreamItem *)(pJob->GetRequest());
	if (pReq == NULL || !pReq->hasStream())
	{
		pJob->Cancel();
		return NULL;
	}

	if (!m_pStream)
	{
		m_pStream = pReq->tryToAcquireStream();
		if (!m_pStream)
		{
			/* @@@ BUSY WAITING */
			GetRootNode()->RescheduleJob(pJob);
			return NULL;
		}

		m_nHintID = CntStatusBarHint::CreateHintId();
		String aStatusText;
		String aFileName;
		if (m_pStream->IsA() == ID_FILESTREAM)
			aFileName = INSECURE_DYNAMIC_CAST(SvFileStream *, m_pStream)->
			             GetFileName();
		if (aFileName.Len())
		{
			aStatusText = CntResId(RID_STATUS_EXPORT_TO);
			aStatusText.SearchAndReplaceAscii("%1", aFileName);
		}
		else
			aStatusText = CntResId(RID_STATUS_EXPORT);
		Broadcast(CntStatusBarHint(/* m_nHintID, */ aStatusText));

		m_pExport = new CntExport(*m_pStream);

		CntNode & rMessageNode = *pJob->GetSubject();
		GetParent()->LoadMessageBodyItem(&rMessageNode, pJob->GetCacheNode());
		if (pReq->doForceBodies()
			&& rMessageNode.GetItemState(WID_MESSAGEBODY) != SFX_ITEM_SET)
		{
			m_xOpenJob
			 = new CntNodeJob(pJob, &rMessageNode, &rMessageNode,
							  CntOpenModeItem(WID_OPEN, CNT_OPEN_MESSAGE));
			StartListening(*m_xOpenJob);
			GetRootNode()->ExecuteJob(m_xOpenJob);
			return 0;
		}
	}

	String
	 aLocalServerName(CntMBXFormat::getLocalServerName
	                                 (ITEMSET_VALUE_STRING(GetRootNode(),
														   WID_SERVERNAME)));
	ErrCode nError
	 = m_pExport->writeMessage(*pJob->GetSubject(), &aLocalServerName);
	if (nError == ERRCODE_NONE)
		pJob->Done();
	else
	{
		pJob->SetError(nError);
		pJob->Cancel();
	}
	return 0;
}

/*
 * Notify.
 */
void CntNewsMsgExportJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	if (&rBC == &m_xOpenJob)
	{
		CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
		if (pStatusHint
			&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
				|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
				   && pStatusHint->GetError() == ERRCODE_ABORT))
		{
			EndListening(*m_xOpenJob);
			m_xOpenJob.Clear();
			GetRootNode()->RescheduleJob(GetJob());
		}
	}
	else
	{
		if (&rBC == GetJob())
		{
			CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
			if (pStatusHint
				&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
					|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
					   && pStatusHint->GetError() == ERRCODE_ABORT))
			{
				if (m_xOpenJob.Is())
					EndListening(*m_xOpenJob);
				delete m_pExport;
				if (m_pStream)
					INSECURE_DYNAMIC_CAST(const CntExpStreamItem *,
										  GetJob()->GetRequest())->
					 releaseStream();
				if (m_nHintID)
					Broadcast(CntStatusBarHint(m_nHintID));
			}
		}
		CntNewsJob_Impl::Notify(rBC, rHint);
	}
}

