/*************************************************************************
 *
 *  $RCSfile: coutvim.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _COUTVIM_CXX "$Revision: 1.1.1.1 $"

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef _INETCOREMSG_HXX
#include <inet/inetmsg.hxx>
#endif

#ifndef _CNTOUTIMP_HXX
#include <coutimp.hxx>
#endif

#ifndef _CNTMBITM_HXX
#include <cntmbitm.hxx>
#endif

#ifndef _CHAOS_CVIMBOX_HXX
#include <cvimbox.hxx>
#endif

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

/*========================================================================
 *
 * CntOutVIMJob_Impl implementation.
 *
 *======================================================================*/
/*
 * CntOutVIMJob_Impl.
 */
CntOutVIMJob_Impl::CntOutVIMJob_Impl (
	CntNodeJob *pJob, CntOutTrayNode_Impl *pParent)
	: CntOutJob_Impl (pJob, pParent),
	  m_pMessage     (NULL),
	  m_pRcptInfo    (NULL),
	  m_eState       (STATE_NONE)
{
}

/*
 * ~CntOutVIMJob_Impl.
 */
CntOutVIMJob_Impl::~CntOutVIMJob_Impl (void)
{
}

/*
 * Execute.
 */
const SfxPoolItem* CntOutVIMJob_Impl::Execute (void)
{
	// Ensure clean destruction.
	CntOutJob_ImplRef xThis (this);

	// Check context.
	CntNodeJob *pJob = GetJob();
	DBG_ASSERT (pJob, "CntOutVIMJob_Impl::Execute(): no Job");
	if (pJob == NULL)
		return NULL;

	// Check request.
	const SfxPoolItem *pReq = pJob->GetRequest();
	DBG_ASSERT (pReq, "CntOutVIMJob_Impl::Execute(): no Request");
	if (pReq == NULL)
	{
		// Failure.
		pJob->Cancel();
		return NULL;
	}

	// Initialize.
	if (m_eState == STATE_NONE)
	{
		// Obtain RecipientInfo.
		m_pRcptInfo = ITEM_VALUE (CntRecipientInfoItem, *pReq);
		DBG_ASSERT(
			m_pRcptInfo, "CntOutVIMJob_Impl::Execute(): no Recipient");
		if (m_pRcptInfo == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Check State.
		USHORT eState = m_pRcptInfo->GetState();
		if (!((eState == CNTOUT_ISTATE_WRITTEN) ||
			  (eState == CNTOUT_ISTATE_ERROR  )    ))
		{
			// Nothing to do.
			pJob->Done();
			return NULL;
		}

		// Check SendTries.
		USHORT nSendTries = m_pRcptInfo->GetSendTries();
		if (!(nSendTries < COUTIMP_SENDTRIES_LIMIT))
		{
			// Mark undeliverable.
			m_pRcptInfo->SetState (CNTOUT_ISTATE_FATALERROR);

			// Done.
			pJob->Done();
			return NULL;
		}

		// Obtain subject.
		CntNode *pSubject = pJob->GetSubject();
		DBG_ASSERT(
			pSubject, "CntOutVIMJob_Impl::Execute(): no Subject");
		if (pSubject == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Obtain MessageBody.
		const CntMessageBodyItem &rBody =
			(const CntMessageBodyItem&)(pSubject->Get(WID_MESSAGEBODY));
		m_pMessage = rBody.Get();
		DBG_ASSERT(
			m_pMessage, "CntOutVIMJob_Impl::Execute(): no Message");
		if (m_pMessage == NULL)
		{
			// Failure.
			pJob->Cancel();
			return NULL;
		}

		// Initialize protocol error.
		m_pRcptInfo->SetProtocolError (0);
		m_pRcptInfo->SetProtocolErrorStr (String());

		// Next state.
		m_eState = STATE_SEND;
	}

	if (m_eState == STATE_SEND)
	{
		// Initialize VIM mailbox.
		CntVIMMailbox aMailbox;

		CntVIMMailbox::InitStatus eStatus = aMailbox.initialize (
			INetURLObject::decode (m_pRcptInfo->GetVIMPOPath(), '=',
								   INetURLObject::DECODE_WITH_CHARSET),
			INetURLObject::decode (m_pRcptInfo->GetUsername(), '=',
								   INetURLObject::DECODE_WITH_CHARSET),
			INetURLObject::decode (m_pRcptInfo->GetPassword(), '=',
								   INetURLObject::DECODE_WITH_CHARSET));
		if (eStatus == CntVIMMailbox::OK)
		{
			// Setup recipient header fields.
			m_pMessage->SetTo (
				m_pRcptInfo->GetToRecipient());
			m_pMessage->SetCC (
				m_pRcptInfo->GetCcRecipient());
			m_pMessage->SetBCC (
				m_pRcptInfo->GetBccRecipient());
			m_pMessage->SetNewsgroups (
				m_pRcptInfo->GetNewsRecipient());

			// Increment SendTries.
			USHORT nSendTries = m_pRcptInfo->GetSendTries();
			m_pRcptInfo->SetSendTries (nSendTries + 1);

			// Send message.
			aMailbox.sendMessage (*m_pMessage, pJob);
		}
		else if (eStatus == CntVIMMailbox::LOGIN_ERROR)
		{
		}
		else
		{
		}

		// Set recipient info.
		const CntVIMStatus &rStatus = aMailbox.getStatus();
		if (rStatus.isOK())
			m_pRcptInfo->SetState (CNTOUT_ISTATE_SENT);
		else
			m_pRcptInfo->SetState (CNTOUT_ISTATE_FATALERROR);

		m_pRcptInfo->SetProtocolError    (rStatus.getStatusValue());
		m_pRcptInfo->SetProtocolErrorStr (rStatus.getStatusText (TRUE));

		// Next state.
		m_eState = STATE_DONE;
	}

	// Done.
	pJob->Done();
	return NULL;
}

/*
 * Notify.
 */
void CntOutVIMJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	CntOutJob_Impl::Notify (rBC, rHint);
}

