/*************************************************************************
 *
 *  $RCSfile: cpop3prt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#ifndef _DATETIME_HXX
#include <tools/datetime.hxx>
#endif

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _SVTOOLS_CINTITEM_HXX
#include <svtools/cintitem.hxx>
#endif

#ifndef _CNTPOP3IMP_HXX
#include "cpop3imp.hxx"
#endif

#ifndef _CASTMACS_HXX
#include "castmacs.hxx"
#endif

#ifndef _CHAOS_IMPORT_HXX
#include "import.hxx"
#endif

#ifndef _CHAOS_EXPORT_HXX
#include "export.hxx"
#endif

#ifndef _CHAOS_STRMITEM_HXX
#include "strmitem.hxx"
#endif

#ifndef _CHAOS_STORITEM_HXX
#include "storitem.hxx"
#endif

#ifndef _CNTDATA_HXX
#include "cntdata.hxx"
#endif

#ifndef _CNTRNMGR_HXX
#include "cntrnmgr.hxx"
#endif

#ifndef _CNTSTGND_HXX
#include "cntstgnd.hxx"
#endif

#ifndef _CSTRITEM_HXX
#include "cstritem.hxx"
#endif

#ifndef _CNTRESID_HXX
#include "cntresid.hxx"
#endif

#ifndef _PROCHAOS_HRC
#include "prochaos.hrc"
#endif

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

#ifdef _CHAOS_STORITEM_REF
SV_IMPL_REF (CntStoreItemSet);
#endif

/*=======================================================================
 *
 * CntPOP3... internals.
 *
 *=====================================================================*/
/*
 * CntDirtyItemSet ranges.
 */
CPOP3MSG_DIR_RANGES_IMPL(aMsgDirRanges_Impl);

/*=======================================================================
 *
 * CntPOP3BoxMessageCachingPolicy.
 *
 *=====================================================================*/
class CntPOP3BoxMessageCachingPolicy : public CntMessageCachingPolicy
{
	String          m_aBoxURL;
	CntStorageNode *m_pCache;
	BOOL            m_bCreateMessageID;

public:
	CntPOP3BoxMessageCachingPolicy (
		const String   &rTheBoxURL,
		CntStorageNode *pTheCache,
		BOOL            bTheCreateMessageID)
		: m_aBoxURL          (rTheBoxURL),
		  m_pCache           (pTheCache),
		  m_bCreateMessageID (bTheCreateMessageID)
	{}

	virtual BOOL doCache (
		const CntMBXVersion * pVersion,
		SfxItemSet          & rItems,
		String              & rName);

	virtual CntStorageNode * getCacheNode (void) { return m_pCache; }

	static String createUniqueMessageID (void);
};

/*
 * doCache.
 */
BOOL CntPOP3BoxMessageCachingPolicy::doCache (
	const CntMBXVersion * pVersion,
	SfxItemSet          & rItems,
	String              & rName)
{
	if (pVersion && !pVersion->getVersion())
	{
		const CntMBXVersion0 * pVersion0 = PTR_CAST(CntMBXVersion0, pVersion);
		if (pVersion0 && pVersion0->getType() != CntMBXVersion0::TYPE_MESSAGE)
			return FALSE;
	}

	String aMessageID;
	const SfxPoolItem * pItem = NULL;
	if (rItems.GetItemState(WID_MESSAGE_ID, TRUE, &pItem) < SFX_ITEM_SET)
	{
		if (!m_bCreateMessageID)
			return FALSE;

		aMessageID = createUniqueMessageID();
		rItems.Put(CntStringItem(WID_MESSAGE_ID, aMessageID));
	}
	else
		aMessageID = ITEM_VALUE(CntStringItem, *pItem);

	rName = m_aBoxURL;
	rName += aMessageID;
	rName.AppendAscii(RTL_CONSTASCII_STRINGPARAM(".body"));
	return TRUE;
}

/*
 * createUniqueMessageID.
 */
String CntPOP3BoxMessageCachingPolicy::createUniqueMessageID (void)
{
	DateTime aCurrDateTime;
	String aMessageID('<');
	aMessageID += aCurrDateTime.GetDate();
	String aTime(aCurrDateTime.GetTime());
	for (xub_StrLen i = aTime.Len(); i < 8; ++i)
		aMessageID += '0';
	aMessageID += aTime;
	aMessageID += CNT_DATA()->_nInterimURL++;
	aMessageID.AppendAscii(RTL_CONSTASCII_STRINGPARAM("@IMPORT>"));
	return aMessageID;
}

/*=======================================================================
 *
 * CntPOP3BoxImportJob_Impl implementation.
 *
 *=====================================================================*/
/*
 * CntPOP3BoxImportJob_Impl.
 */
CntPOP3BoxImportJob_Impl::CntPOP3BoxImportJob_Impl (
	CntNodeJob * pJob, CntPOP3Node_Impl *pParent)
	: CntPOP3Job_Impl   (pJob, pParent),
	  m_pStream         (0),
	  m_pImport         (0),
	  m_nHintID         (0),
	  m_pPolicy         (0),
	  m_pPOP3BoxScanner (0)
{
}

/*
 * ~CntPOP3BoxImportJob_Impl.
 */
CntPOP3BoxImportJob_Impl::~CntPOP3BoxImportJob_Impl (void)
{
	delete m_pPolicy;
	delete m_pImport;
	INSECURE_DYNAMIC_CAST(CntInStreamItem *, GetJob()->GetRequest())->
	 releaseStream();
}

/*
 * handlePOP3BoxHidden.
 */
ErrCode CntPOP3BoxImportJob_Impl::handlePOP3BoxHidden (CntMBXToken & rToken)
{
	ErrCode nError
	 = m_pPOP3BoxScanner->
		scan(rToken, CntMBXChars::aCharsText,
			 CntMBXScanner::ScanMode(CntMBXScanner::SKIP_LWSP));
	if (nError)
		return nError;

	if (rToken.getType() != CntMBXToken::TYPE_ATOM)
		return ERRCODE_NONE;
	String aMessageID(rToken.getValue(), RTL_TEXTENCODING_UTF8);

	CntStorageNode * pDirectory = GetJob()->GetDirectoryNode();
	CntStorageNode * pUserData  = GetJob()->GetUserDataNode();
	CntNode        * pBox       = GetRootNode();

	BOOL bNewMessage = TRUE;
	if (pDirectory)
	{
		UINT32 nAttrib;
		if (pDirectory->attrib(aMessageID, 0, 0, nAttrib) == ERRCODE_NONE)
			if (nAttrib & CNTDIRENTRY_ATTRIB_MARKEDFORDELETE)
				return ERRCODE_NONE;
			else
				bNewMessage = FALSE;
		pDirectory->attrib(aMessageID, 0,
						   CNTDIRENTRY_ATTRIB_CREATE
						    | CNTDIRENTRY_ATTRIB_MARKEDFORDELETE);
	}

	if (pUserData)
		pUserData->attrib(aMessageID, 0, CNTDIRENTRY_ATTRIB_MARKEDFORDELETE);

	if (!bNewMessage)
	{
		BOOL bWasRead = FALSE;
		if (pUserData)
		{
			UINT32 nAttrib = 0;
			pUserData->attrib(aMessageID, 0, 0, nAttrib);
			bWasRead = !(nAttrib & CNTDIRENTRY_ATTRIB_UNSEEN);
		}

		UINT32 nTotal = ITEMSET_VALUE(
			pBox, CntUInt32Item, WID_TOTALCONTENTCOUNT);
		--nTotal;
		CntUInt32Item aItem(WID_TOTALCONTENTCOUNT, nTotal);

		CntStoreItemSetRef xBoxItems(GetParent()->GetBoxItemSet(pDirectory));
		if (xBoxItems.Is())
			xBoxItems->Put(aItem);
		pBox->Put(aItem);

		UINT32 nSeen = ITEMSET_VALUE(
			pBox, CntUInt32Item, WID_SEENCONTENTCOUNT);
		if (bWasRead)
		{
			--nSeen;
			CntUInt32Item aItem(WID_SEENCONTENTCOUNT, nSeen);

			CntStoreItemSetRef xBoxItems(
				GetParent()->GetUserItemSet(pUserData));
			if (xBoxItems.Is())
				xBoxItems->Put(aItem);
			pBox->Put(aItem);
		}
		pBox->Put(CntBoolItem(WID_IS_READ, nSeen >= nTotal));

		String aMessageURL(m_aBoxURL);
		aMessageURL += aMessageID;

		CntNodeRef xMessageNode (pBox->Query (aMessageURL, FALSE));
		if (xMessageNode.Is())
			Result (xMessageNode, CNT_ACTION_DELETED);
	}

	return ERRCODE_NONE;
}

/*
 * handleMessage.
 */
void CntPOP3BoxImportJob_Impl::handleMessage (
	CntNode          * pMessageNode,
	const SfxItemSet & rItems,
	const String     & rMessageID)
{
	pMessageNode->Put(rItems);

	CntStorageNode * pDirectory = GetJob()->GetDirectoryNode();
	CntStorageNode * pUserData  = GetJob()->GetUserDataNode();
	CntNode        * pBox       = GetRootNode();

	BOOL bNewMessage = TRUE;
	if (pDirectory)
	{
		UINT32 nAttrib;
		if (pDirectory->attrib(rMessageID, 0, 0, nAttrib) == ERRCODE_NONE)
			if (nAttrib & CNTDIRENTRY_ATTRIB_MARKEDFORDELETE)
				pDirectory->
				 attrib(rMessageID, CNTDIRENTRY_ATTRIB_MARKEDFORDELETE, 0);
			else
				bNewMessage = FALSE;

		/* @@@ ??? @@@ */
		CntStoreItemSetRef xDirectoryItems(
			pDirectory->openItemSet(rMessageID, STREAM_STD_READ));
		if (xDirectoryItems.Is())
			xDirectoryItems.Clear();
	}

	BOOL bWasRead = FALSE;
	BOOL bWasMarked = FALSE;
	if (!bNewMessage && pUserData)
	{
		UINT32 nAttrib = 0;
		pUserData->attrib(rMessageID, 0, 0, nAttrib);
		bWasRead = !(nAttrib & CNTDIRENTRY_ATTRIB_UNSEEN);
		bWasMarked = CAST_TO_BOOL(nAttrib & CNTDIRENTRY_ATTRIB_MARKED);
	}

	const SfxPoolItem * pItem;
	BOOL bIsRead
	 = rItems.GetItemState(WID_IS_READ, TRUE, &pItem) >= SFX_ITEM_SET ?
		INSECURE_DYNAMIC_CAST(const CntBoolItem *, pItem)->GetValue() :
		bWasRead;
	BOOL bIsMarked
	 = rItems.GetItemState(WID_IS_MARKED, TRUE, &pItem) >= SFX_ITEM_SET ?
		INSECURE_DYNAMIC_CAST(const CntBoolItem *, pItem)->GetValue() :
		bWasMarked;

	if (pUserData)
	{
		UINT32 nClear = CNTDIRENTRY_ATTRIB_MARKEDFORDELETE;
		UINT32 nSet = 0;
		if (bIsRead)
			nClear |= CNTDIRENTRY_ATTRIB_UNSEEN;
		else
			nSet |= CNTDIRENTRY_ATTRIB_CREATE | CNTDIRENTRY_ATTRIB_UNSEEN;
		if (bIsMarked)
			nSet |= CNTDIRENTRY_ATTRIB_CREATE | CNTDIRENTRY_ATTRIB_MARKED;
		else
			nClear |= CNTDIRENTRY_ATTRIB_MARKED;
		pUserData->attrib(rMessageID, nClear, nSet);
	}

	UINT32 nTotal = ITEMSET_VALUE(
		pBox, CntUInt32Item, WID_TOTALCONTENTCOUNT);
	if (bNewMessage)
	{
		++nTotal;
		CntUInt32Item aItem(WID_TOTALCONTENTCOUNT, nTotal);

		CntStoreItemSetRef xBoxItems(GetParent()->GetBoxItemSet(pDirectory));
		if (xBoxItems.Is())
			xBoxItems->Put(aItem);
		pBox->Put(aItem);
	}

	UINT32 nSeen = ITEMSET_VALUE(
		pBox, CntUInt32Item, WID_SEENCONTENTCOUNT);
	if (bIsRead)
	{
		if (bNewMessage || !bWasRead)
		{
			++nSeen;
			CntUInt32Item aItem(WID_SEENCONTENTCOUNT, nSeen);

			CntStoreItemSetRef xBoxUserSet(
				GetParent()->GetUserItemSet(pUserData));
			if (xBoxUserSet.Is())
			{
				xBoxUserSet->Put(aItem);
				xBoxUserSet.Clear();
			}
			pBox->Put(aItem);
		}
	}
	else if (!bNewMessage && bWasRead && nSeen)
	{
		--nSeen;
		CntUInt32Item aItem(WID_SEENCONTENTCOUNT, nSeen);

		CntStoreItemSetRef xBoxUserSet (
			GetParent()->GetUserItemSet(pUserData));
		if (xBoxUserSet.Is())
		{
			xBoxUserSet->Put(aItem);
			xBoxUserSet.Clear();
		}
		pBox->Put(aItem);
	}
	pBox->Put(CntBoolItem(WID_IS_READ, nSeen >= nTotal));

	if (pDirectory)
	{
		CntStoreItemSetRef xDirectoryItems(
			pDirectory->openItemSet(aMsgDirRanges_Impl, rMessageID));
		if (xDirectoryItems.Is())
		{
			xDirectoryItems->Put(pMessageNode->Get(WID_MESSAGE_ID));
			xDirectoryItems->Put(pMessageNode->Get(WID_SUBJECT));
			xDirectoryItems->Put(pMessageNode->Get(WID_FROM));
			xDirectoryItems->Put(pMessageNode->Get(WID_TO));
			xDirectoryItems->Put(pMessageNode->Get(WID_CC));
			xDirectoryItems->Put(pMessageNode->Get(WID_DATE_CREATED));
			xDirectoryItems->Put(pMessageNode->Get(WID_REFERENCES));
			xDirectoryItems->Put(pMessageNode->Get(WID_IN_REPLY_TO));
			xDirectoryItems->Put(pMessageNode->Get(WID_PRIORITY));
		}
	}

	Result (pMessageNode);
}

/*
 * Execute.
 */
const SfxPoolItem* CntPOP3BoxImportJob_Impl::Execute (void)
{
	// Check context.
	CntNodeJob *pJob = GetJob();
	if (pJob == NULL)
		return NULL;

	const CntInStreamItem *pReq = STATIC_CAST(
		const CntInStreamItem *, pJob->GetRequest());
	if (pReq == NULL)
	{
		pJob->Cancel();
		return NULL;
	}

	// Initialize.
	if (!pJob->IsRescheduled())
	{
		m_pStream = pReq->tryToAcquireStream();
		if (!m_pStream)
		{
			pJob->Cancel();
			return 0;
		}
		m_pImport = new CntImport(*m_pStream);

		CntMBXScanner::SourceFormat eFormat;
		ErrCode nError = m_pImport->determineSourceFormat(eFormat);
		if (nError)
		{
			pJob->SetError(nError);
			pJob->Cancel();
			return 0;
		}
		m_bCreateMessageID = eFormat == CntMBXScanner::FORMAT_OUTLOOK_EXPRESS;

		if (m_pStream->IsA() == ID_FILESTREAM)
			m_sFileName
				= static_cast< SvFileStream * >(m_pStream)->GetFileName();
		m_bMessageFound = false;

		String aStatusText;
		if (m_sFileName.Len() > 0)
		{
			aStatusText = CntResId(RID_STATUS_IMPORT_FROM);
			aStatusText.SearchAndReplaceAscii("%1", m_sFileName);
		}
		else
			aStatusText = CntResId(RID_STATUS_IMPORT);

		m_nHintID = CntStatusBarHint::CreateHintId();
		Broadcast (CntStatusBarHint (
			m_nHintID, 0, m_pImport->getProgressMax(), aStatusText));

		m_aBoxURL = OWN_URL(GetRootNode());
		if (m_aBoxURL.Len() == 0
			|| m_aBoxURL.GetChar(m_aBoxURL.Len() - 1) != '/')
			m_aBoxURL += '/';

		m_pPolicy = new CntPOP3BoxMessageCachingPolicy (
			m_aBoxURL, pJob->GetCacheNode(), m_bCreateMessageID);
	}

	ULONG nTicks = Time::GetSystemTicks();
	for (;;)
	{
		ULONG nProgress;

		if (m_pPOP3BoxScanner)
		{
			CntMBXToken aToken;
			ErrCode nError
			 = m_pPOP3BoxScanner->
				scan(aToken, CntMBXChars::aCharsRFC822,
					 CntMBXScanner::ScanMode
					  (CntMBXScanner::CANONIC_ATOM | CntMBXScanner::HASH_ATOM
					    | CntMBXScanner::SKIP_LWSP
					    | CntMBXScanner::SKIP_COMMENTS));
			if (nError)
			{
				pJob->SetError(nError);
				pJob->Cancel();
				return 0;
			}

			if (aToken.getType() == CntMBXToken::TYPE_END)
			{
				nProgress = m_pPOP3BoxScanner->getInputProcessed();
				m_pImport->releaseBodyScanner();
				m_pPOP3BoxScanner = 0;
			}
			else
			{
				if (aToken.getType() == CntMBXToken::TYPE_ATOM)
					switch (aToken.getAtom()->getKey())
					{
						case CntMBXAtomTable::ATOM_HIDDEN:
							nError = handlePOP3BoxHidden(aToken);
							if (nError)
							{
								pJob->SetError(nError);
								pJob->Cancel();
								return 0;
							}
							break;
					}

				while (aToken.getType() != CntMBXToken::TYPE_CRLF
					   && aToken.getType() != CntMBXToken::TYPE_END)
				{
					nError
					 = m_pPOP3BoxScanner->
						scan(aToken, CntMBXChars::aCharsRFC822,
							 CntMBXScanner::ScanMode
							  (CntMBXScanner::SKIP_LWSP
							    | CntMBXScanner::SKIP_COMMENTS));
					if (nError)
					{
						pJob->SetError(nError);
						pJob->Cancel();
						return 0;
					}
				}
			}
		}
		else
		{
			static const USHORT aRanges[]
			 = { WID_CHAOS_START, WID_INTERIM_URL,
				 WID_OWN_URL, WID_CHAOS_END, 0 };
			SfxItemSet aItems(*(GetRootNode()->GetPool()), aRanges);

			CntMBXVersion * pVersion = 0;
			BOOL            bFound = FALSE;

			ErrCode nError = m_pImport->getMessage (
				*m_pPolicy, bFound, pVersion, aItems, nProgress);
			if (nError)
			{
				delete pVersion;

				pJob->SetError(nError);
				pJob->Cancel();
				return 0;
			}
			if (!bFound)
			{
				delete pVersion;
				break;
			}
			m_bMessageFound = true;

			String aMessageID;
			BOOL bProcess;

			const SfxPoolItem * pItem;
			if (aItems.GetItemState(WID_MESSAGE_ID, FALSE, &pItem)
				 < SFX_ITEM_SET)
				if (m_bCreateMessageID)
				{
					aMessageID = CntPOP3BoxMessageCachingPolicy::
						createUniqueMessageID();
					aItems.Put(CntStringItem(WID_MESSAGE_ID, aMessageID));
					bProcess = TRUE;
				}
				else
					bProcess = FALSE;
			else
			{
				aMessageID = ITEM_VALUE(CntStringItem, *pItem);
				bProcess = TRUE;
			}

			if (bProcess)
			{
				CntMBXVersion0::MessageType eType
				 = CntMBXVersion0::TYPE_MESSAGE;
				if (pVersion && !pVersion->getVersion())
				{
					const CntMBXVersion0 * pVersion0
					 = PTR_CAST(CntMBXVersion0, pVersion);
					if (pVersion0)
						eType = pVersion0->getType();
				}

				switch (eType)
				{
					case CntMBXVersion0::TYPE_MESSAGE:
					{
						String aMessageURL(m_aBoxURL);
						aMessageURL += aMessageID;

						CntNodeRef xMessageNode (
							GetRootNode()->Query(aMessageURL));
						if (xMessageNode.Is())
							handleMessage(xMessageNode, aItems, aMessageID);
						break;
					}

					case CntMBXVersion0::TYPE_MESSAGEBOX:
						m_pPOP3BoxScanner = m_pImport->aquireBodyScanner();
						break;
				}
			}
			delete pVersion;
		}

		if (Time::GetSystemTicks() - nTicks > 250)
		{
			if (m_pPOP3BoxScanner)
				nProgress = m_pPOP3BoxScanner->getInputProcessed();

			Broadcast(CntStatusBarHint(m_nHintID, nProgress));
			GetRootNode()->RescheduleJob(pJob);
			return 0;
		}
	}

	if (!m_bMessageFound
		&& pJob->SetError(m_sFileName.Len() == 0 ?
						      ERRCODE_CHAOS_NOTHING_IMPORTED :
						      *new StringErrorInfo(
								       ERRCODE_CHAOS_NOTHING_IMPORTED_FROM,
									   m_sFileName)))
		return 0;

	pJob->Done();
	return 0;
}

/*
 * Notify.
 */
void CntPOP3BoxImportJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	CntNodeJob *pJob = PTR_CAST (CntNodeJob, &rBC);
	if ((pJob != NULL) && (pJob == GetJob()))
	{
		if (pJob->IsCancelled() || pJob->IsDone())
		{
			if (m_nHintID)
			{
				USHORT nHintID = m_nHintID;
				m_nHintID = 0;
				Broadcast (CntStatusBarHint (nHintID));
			}
		}
	}
	CntPOP3Job_Impl::Notify (rBC, rHint);
}

/*=======================================================================
 *
 * CntPOP3BoxExportJob_Impl implementation.
 *
 *=====================================================================*/

IMPL_LINK(CntPOP3BoxExportJob_Impl, errorHandler, CntErrorData *, pData)
{
	switch (pData->_nError)
	{
		case ERRCODE_CHAOS_OFFLINE:
		{
			if (m_bIgnoreOffline)
				return ERRCODE_BUTTON_NO;
			USHORT nResult
			 = CNT_RNM()->HandleError(ERRCODE_CHAOS_OFFLINE, GetJob(),
									  pData->_pErrorStr, pData->_pData);
			if (nResult == ERRCODE_BUTTON_NO)
				m_bIgnoreOffline = TRUE;
			return nResult;
		}

		case ERRCODE_CHAOS_LOGIN_FAILURE_RECEIVE:
			return CNT_RNM()->HandleError(ERRCODE_CHAOS_LOGINFAIL_NOEXCH,
										  GetJob(), pData->_pErrorStr,
										  SAL_STATIC_CAST(CntLoginErrorData *,
														  pData->_pData)->
										   getTarget());
	}
	return ERRCODE_CHAOS_NOT_HANDLED;
}

/*
 * Execute.
 */
const SfxPoolItem* CntPOP3BoxExportJob_Impl::Execute (void)
{
	// Check context.
	CntNodeJob *pJob = GetJob();
	if (pJob == NULL)
		return NULL;

	const CntExpStreamItem *pReq = INSECURE_DYNAMIC_CAST(
		const CntExpStreamItem *, pJob->GetRequest());
	if (pReq == NULL || !pReq->hasStream())
	{
		pJob->Cancel();
		return NULL;
	}

	CntStorageNode * pDirectory = pJob->GetDirectoryNode();
	if (!pDirectory)
	{
		pJob->Cancel();
		return 0;
	}

	// Initialize.
	if (!m_pStream)
	{
		m_pStream = pReq->tryToAcquireStream();
		if (!m_pStream)
		{
			GetRootNode()->RescheduleJob (pJob);
			return NULL;
		}

		m_nHintID = CntStatusBarHint::CreateHintId();
		String aStatusText;
		String aFileName;
		if (m_pStream->IsA() == ID_FILESTREAM)
			aFileName = INSECURE_DYNAMIC_CAST(SvFileStream *, m_pStream)->
				         GetFileName();
		if (aFileName.Len())
		{
			aStatusText = CntResId(RID_STATUS_EXPORT_TO);
			aStatusText.SearchAndReplaceAscii("%1", aFileName);
		}
		else
			aStatusText = CntResId(RID_STATUS_EXPORT);
		Broadcast(CntStatusBarHint(m_nHintID, 0,
								   ITEMSET_VALUE(GetRootNode(), CntUInt32Item,
												 WID_TOTALCONTENTCOUNT)
								    + 1,
								   aStatusText));

		m_pExport = new CntExport(*m_pStream);

		m_bWritePOP3Box = TRUE;
		m_pIterator = new CntStorageIterator(
			CNTDIRENTRY_ATTRIB_MARKEDFORDELETE,
			CNTDIRENTRY_ATTRIB_NOTITER & ~CNTDIRENTRY_ATTRIB_MARKEDFORDELETE);
		ErrCode nError = m_pExport->writeHeader(*GetRootNode());
		if (nError)
		{
			pJob->SetError(nError);
			pJob->Cancel();
			return 0;
		}
	}

	ULONG nTicks = Time::GetSystemTicks();

	if (m_bWritePOP3Box)
	{
		for (;;)
		{
			String aMessageName(pDirectory->iter(*m_pIterator));
			if (aMessageName.Len() == 0)
				break;

			ByteString aLine(RTL_CONSTASCII_STRINGPARAM("hidden "));
			aLine += ByteString(aMessageName, RTL_TEXTENCODING_UTF8);
			ErrCode nError = m_pExport->writeBody(aLine);
			if (nError)
			{
				pJob->SetError(nError);
				pJob->Cancel();
				return 0;
			}

			if (Time::GetSystemTicks() - nTicks > 250)
			{
				Broadcast(CntStatusBarHint(m_nHintID, m_nProgress));
				GetRootNode()->RescheduleJob(pJob);
				return 0;
			}
		}

		++m_nProgress;
		m_bWritePOP3Box = FALSE;
		delete m_pIterator;
		m_pIterator = new CntStorageIterator;
	}

	String aBoxURL(OWN_URL(GetRootNode()));
	if (aBoxURL.Len() == 0 || aBoxURL.GetChar(aBoxURL.Len() - 1) != '/')
		aBoxURL += '/';
	for (;;)
	{
		if (!m_xMessageNode)
		{
			String aMessageName(pDirectory->iter(*m_pIterator));
			if (aMessageName.Len() == 0)
				break;

			String aMessageURL(aBoxURL);
			aMessageURL += aMessageName;
			m_xMessageNode = GetRootNode()->Query(aMessageURL);
			if (m_xMessageNode.Is())
			{
				if (!m_xMessageNode->IsInserted())
				{
					CntStoreItemSetRef
					 xMessageSet(pDirectory->openItemSet(aMessageName,
														 STREAM_STD_READ));
					if (xMessageSet.Is())
					{
						m_xMessageNode->Put(*xMessageSet);
						xMessageSet.Clear();
					}

					BOOL bRead = FALSE;
					BOOL bMarked = FALSE;
					CntStorageNode * pUserData = pJob->GetUserDataNode();
					if (pUserData)
					{
						UINT32 nAttrib = 0;
						pUserData->attrib(aMessageName, 0, 0, nAttrib);
						bRead = !(nAttrib & CNTDIRENTRY_ATTRIB_UNSEEN);
						bMarked =
						 CAST_TO_BOOL(nAttrib & CNTDIRENTRY_ATTRIB_MARKED);
					}
					m_xMessageNode->Put(CntBoolItem(WID_IS_READ, bRead));
					m_xMessageNode->Put(CntBoolItem(WID_IS_MARKED, bMarked));
				}

				GetParent()->LoadMessageBodyItem(m_xMessageNode,
												 pJob->GetCacheNode());
				if (pReq->doForceBodies()
					&& m_xMessageNode->GetItemState(WID_MESSAGEBODY)
				        != SFX_ITEM_SET)
				{
					m_xOpenJob
					 = new CntNodeJob(pJob, m_xMessageNode, m_xMessageNode,
									  CntOpenModeItem(WID_OPEN,
													  CNT_OPEN_MESSAGE));
					m_xOpenJob->SetErrorHandler(LINK(this,
													 CntPOP3BoxExportJob_Impl,
													 errorHandler));
					StartListening(*m_xOpenJob);
					GetRootNode()->ExecuteJob(m_xOpenJob);
					return 0;
				}
			}
		}

		if (m_xMessageNode.Is())
		{
			ErrCode nError = m_pExport->writeMessage(*m_xMessageNode);
			m_xMessageNode.Clear();
			if (nError)
			{
				pJob->SetError(nError);
				pJob->Cancel();
				return 0;
			}
		}

		++m_nProgress;

		if (Time::GetSystemTicks() - nTicks > 250)
		{
			Broadcast(CntStatusBarHint(m_nHintID, m_nProgress));
			GetRootNode()->RescheduleJob(pJob);
			return 0;
		}
	}

	pJob->Done();
	return 0;
}

/*
 * Notify.
 */
void CntPOP3BoxExportJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	if (&rBC == &m_xOpenJob)
	{
		CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
		if (pStatusHint
			&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
				|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
				   && pStatusHint->GetError() == ERRCODE_ABORT))
		{
			EndListening(*m_xOpenJob);
			m_xOpenJob.Clear();
			GetRootNode()->RescheduleJob(GetJob());
		}
	}
	else
	{
		if (&rBC == GetJob())
		{
			CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
			if (pStatusHint
				&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
					|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
					   && pStatusHint->GetError() == ERRCODE_ABORT))
			{
				if (m_xOpenJob.Is())
					EndListening(*m_xOpenJob);
				delete m_pIterator;
				delete m_pExport;
				if (m_pStream)
					INSECURE_DYNAMIC_CAST(const CntExpStreamItem *,
										  GetJob()->GetRequest())->
					 releaseStream();
				if (m_nHintID)
					Broadcast(CntStatusBarHint(m_nHintID));
			}
		}
		CntPOP3Job_Impl::Notify(rBC, rHint);
	}
}

/*=======================================================================
 *
 * CntPOP3MsgExportJob_Impl implementation.
 *
 *=====================================================================*/

/*
 * Execute.
 */
const SfxPoolItem * CntPOP3MsgExportJob_Impl::Execute()
{
	// Check context.
	CntNodeJob *pJob = GetJob();
	if (pJob == NULL)
		return NULL;

	const CntExpStreamItem *pReq = INSECURE_DYNAMIC_CAST(
		const CntExpStreamItem *, pJob->GetRequest());
	if (pReq == NULL || !pReq->hasStream())
	{
		pJob->Cancel();
		return NULL;
	}

	// Initialize.
	if (!m_pStream)
	{
		m_pStream = pReq->tryToAcquireStream();
		if (!m_pStream)
		{
			GetRootNode()->RescheduleJob (pJob);
			return NULL;
		}

		String aStatusText;
		String aFileName;
		if (m_pStream->IsA() == ID_FILESTREAM)
			aFileName = INSECURE_DYNAMIC_CAST(SvFileStream *, m_pStream)->
				         GetFileName();
		if (aFileName.Len())
		{
			aStatusText = CntResId(RID_STATUS_EXPORT_TO);
			aStatusText.SearchAndReplaceAscii("%1", aFileName);
		}
		else
			aStatusText = CntResId(RID_STATUS_EXPORT);
		Broadcast (CntStatusBarHint(aStatusText));
		m_bProgress = TRUE;

		m_pExport = new CntExport(*m_pStream);

		CntNode & rMessageNode = *pJob->GetSubject();
		GetParent()->LoadMessageBodyItem(&rMessageNode, pJob->GetCacheNode());
		if (pReq->doForceBodies()
			&& rMessageNode.GetItemState(WID_MESSAGEBODY) != SFX_ITEM_SET)
		{
			m_xOpenJob
			 = new CntNodeJob(pJob, &rMessageNode, &rMessageNode,
							  CntOpenModeItem(WID_OPEN, CNT_OPEN_MESSAGE));
			StartListening(*m_xOpenJob);
			GetRootNode()->ExecuteJob(m_xOpenJob);
			return 0;
		}
	}

	ErrCode nError = m_pExport->writeMessage(*pJob->GetSubject());
	if (nError == ERRCODE_NONE)
		pJob->Done();
	else
	{
		pJob->SetError(nError);
		pJob->Cancel();
	}
	return 0;
}

/*
 * Notify.
 */
void CntPOP3MsgExportJob_Impl::Notify (
	SfxBroadcaster& rBC, const SfxHint& rHint)
{
	if (&rBC == &m_xOpenJob)
	{
		CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
		if (pStatusHint
			&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
				|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
				   && pStatusHint->GetError() == ERRCODE_ABORT))
		{
			EndListening(*m_xOpenJob);
			m_xOpenJob.Clear();
			GetRootNode()->RescheduleJob(GetJob());
		}
	}
	else
	{
		if (&rBC == GetJob())
		{
			CntStatusHint * pStatusHint = PTR_CAST(CntStatusHint, &rHint);
			if (pStatusHint
				&& (pStatusHint->GetStatus() == CNT_STATUS_DONE
					|| pStatusHint->GetStatus() == CNT_STATUS_ERROR
					   && pStatusHint->GetError() == ERRCODE_ABORT))
			{
				if (m_xOpenJob.Is())
					EndListening(*m_xOpenJob);
				delete m_pExport;
				if (m_pStream)
					INSECURE_DYNAMIC_CAST(const CntExpStreamItem *,
										  GetJob()->GetRequest())->
					 releaseStream();
				if (m_bProgress)
					Broadcast(CntStatusBarHint(String()));
			}
		}
		CntPOP3Job_Impl::Notify(rBC, rHint);
	}
}

