/*************************************************************************
 *
 *  $RCSfile: cvimbox.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _INETCOREMSG_HXX
#include <inet/inetmsg.hxx>
#endif
#ifndef _RTL_MEMORY_H_
#include <rtl/memory.h>
#endif
#ifndef _FSYS_HXX
#include <tools/fsys.hxx>
#endif
#ifndef TOOLS_INETMIME_HXX
#include <tools/inetmime.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _VOS_MODULE_HXX_
#include <vos/module.hxx>
#endif
#ifndef _VOS_PROCESS_HXX_
#include <vos/process.hxx>
#endif

#ifndef _CASTMACS_HXX
#include <castmacs.hxx>
#endif
#ifndef _CNTIDS_HRC
#include <cntids.hrc>
#endif
#ifndef _CNTJOB_HXX
#include <cntjob.hxx>
#endif
#ifndef _CNTRESID_HXX
#include <cntresid.hxx>
#endif
#ifndef CHAOS_FLSTITEM_HXX
#include <flstitem.hxx>
#endif
#ifndef _PROCHAOS_HRC
#include <prochaos.hrc>
#endif

#ifndef _CHAOS_CVIMBOX_HXX
#include <cvimbox.hxx>
#endif
#ifndef _CHAOS_VIMWRAP_HXX
#include <vimwrap.hxx>
#endif

#if defined TF_UCB
using namespace chaos;
#endif // TF_UCB

//============================================================================
/// A read-only SvMemoryStream that is initialized with a String.
class CntVIMStringStream: public SvMemoryStream
{
public:
	inline CntVIMStringStream(const ByteString & rBuffer);
};

inline CntVIMStringStream::CntVIMStringStream(const ByteString & rBuffer):
	SvMemoryStream(new sal_Char[rBuffer.Len()], rBuffer.Len(),
				   STREAM_STD_READ)
{
	rtl_copyMemory(pBuf, rBuffer.GetBuffer(), rBuffer.Len());
	ObjectOwnsMemory(TRUE);
}

//============================================================================
/// A private-access SvFileStream that kills the underlying file when the
/// stream is destroyed.
class CntVIMTempFileStream: public SvFileStream
{
public:
	inline CntVIMTempFileStream();

	inline CntVIMTempFileStream(const String & rFilename);

	virtual ~CntVIMTempFileStream();
};

inline CntVIMTempFileStream::CntVIMTempFileStream():
	SvFileStream(DirEntry(String::CreateFromAscii(RTL_CONSTASCII_STRINGPARAM(
		                                              "vim*.tmp"))).
				     TempName().GetFull(),
				 STREAM_READ | STREAM_WRITE | STREAM_TRUNC
				     | STREAM_SHARE_DENYALL)
{}

inline CntVIMTempFileStream::CntVIMTempFileStream(const String & rFilename):
	SvFileStream(rFilename,
				 STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYALL)
{}

//============================================================================
class CntVIMSendBufferNode
{
public:
	CntVIMSendBufferNode * m_pNext;
	sal_Char * m_pBuffer;
	CntVIMTempFileStream * m_pTempFile;

	CntVIMSendBufferNode(CntVIMSendBufferNode * pTheNext,
						 sal_Char * pTheBuffer):
		m_pNext(pTheNext), m_pBuffer(pTheBuffer), m_pTempFile(0) {}

	CntVIMSendBufferNode(CntVIMSendBufferNode * pTheNext,
						 CntVIMTempFileStream * pTheTempFile):
		m_pNext(pTheNext), m_pBuffer(0), m_pTempFile(pTheTempFile) {}

	inline ~CntVIMSendBufferNode();
};

inline CntVIMSendBufferNode::~CntVIMSendBufferNode()
{
	delete[] m_pBuffer;
	delete m_pTempFile;
}

//============================================================================
union CntVIMPointer
{
	void * CNT_VIM_PTR m_pPointer;
#if defined CNT_VIM_ALT_LINKAGE
	void * CNT_VIM_ALT_PTR m_pAltPointer;
#endif // CNT_VIM_ALT_LINKAGE

	CntVIMPointer(const CntVIMLibrary & rLibrary) { clear(rLibrary); }

	inline CntVIMPointer(const CntVIMLibrary & rLibrary,
						 const CntVIMPointer & rOther);

	inline void clear(const CntVIMLibrary & rLibrary);

	inline BOOL isSet(const CntVIMLibrary & rLibrary) const;
};

//============================================================================
union CntVIMAttrDescriptor
{
	CntVIMAttrDesc m_aDesc;
#if defined CNT_VIM_ALT_LINKAGE
	CntVIMAltAttrDesc m_aAltDesc;
#endif // CNT_VIM_ALT_LINKAGE

	inline void setAttr(const CntVIMLibrary & rLibray, ULONG nAttr);

	inline void setSize(const CntVIMLibrary & rLibray, ULONG nSize);

	inline void setBuffer(const CntVIMLibrary & rLibray, void * pBuffer);
};

//============================================================================
union CntVIMBFDescriptor
{
	CntVIMBuffFileDesc m_aDesc;
#if defined CNT_VIM_ALT_LINKAGE
	CntVIMAltBuffFileDesc m_aAltDesc;
#endif // CNT_VIM_ALT_LINKAGE

	inline CntVIMBFDescriptor();

	inline ~CntVIMBFDescriptor();

	inline ULONG getSize(const CntVIMLibrary & rLibrary);

	inline void * getBuffer(const CntVIMLibrary & rLibrary);

	inline void setSize(const CntVIMLibrary & rLibrary, ULONG nSize);

	inline void setBuffer(const CntVIMLibrary & rLibrary, void * pBuffer);

	inline void setFileName(const CntVIMLibrary & rLibrary,
							const String & rFileName);
};

inline CntVIMBFDescriptor::CntVIMBFDescriptor()
{
	m_aDesc.Size = 0;
	m_aDesc.Buffer = 0;
	m_aDesc.Offset = 0;
	m_aDesc.FileName = 0;
#if defined CNT_VIM_ALT_LINKAGE
	m_aAltDesc.Size = 0;
	m_aAltDesc.Buffer = 0;
	m_aAltDesc.Offset = 0;
	m_aAltDesc.FileName = 0;
#endif // CNT_VIM_ALT_LINKAGE
}

inline CntVIMBFDescriptor::~CntVIMBFDescriptor()
{
	delete[] m_aDesc.FileName;
#if defined CNT_VIM_ALT_LINKAGE
	delete[] m_aAltDesc.FileName;
#endif // CNT_VIM_ALT_LINKAGE
}

//============================================================================
class CntVIMLibrary
{
	enum InitState { NONE, INITIALIZED, DONE };

	vos::OModule m_aModule;
	CntVIMStatus & m_rStatus;
	void * m_pFnCloseMessage;
	void * m_pFnCloseMessageContainer;
	void * m_pFnCloseSession;
	void * m_pFnCreateMessage;
	void * m_pFnEnumerateMessageHeaderAttrs;
	void * m_pFnEnumerateMessageItems;
	void * m_pFnEnumerateMessageRecipients;
	void * m_pFnEnumerateMessages;
	void * m_pFnGetDefaultSessionInfo;
	void * m_pFnGetMessageHeader;
	void * m_pFnGetMessageItem;
	void * m_pFnInitialize;
	void * m_pFnMarkMessageAsRead;
	void * m_pFnOpenMessage;
	void * m_pFnOpenMessageContainer;
	void * m_pFnOpenSession;
	void * m_pFnQueryCapability;
	void * m_pFnQueryNewMessages;
	void * m_pFnRemoveMessage;
	void * m_pFnSendMessage;
	void * m_pFnSetMessageHeader;
	void * m_pFnSetMessageItem;
	void * m_pFnSetMessageRecipient;
	void * m_pFnStatusText;
	void * m_pFnTerminate;
	ULONG m_nCharSet;
	ULONG m_nMaxSubjectLen;
	InitState m_eInitState;
#if defined CNT_VIM_ALT_LINKAGE
	BOOL m_bAltLinkage;
#endif // CNT_VIM_ALT_LINKAGE
	BOOL m_bCCMail;
	BOOL m_bGetItemThroughFile;

public:
	CntVIMLibrary(CntVIMStatus & rTheStatus):
		m_rStatus(rTheStatus), m_eInitState(NONE), m_nCharSet(0) {}

	~CntVIMLibrary();

	BOOL initialize(CntNodeJob * pJob);

	BOOL isInitialized() const { return m_eInitState != NONE; }

#if defined CNT_VIM_ALT_LINKAGE
	BOOL useAltLinkage() const { return m_bAltLinkage; }
#endif // CNT_VIM_ALT_LINKAGE

	ULONG getCharSet() const { return m_nCharSet; }

	rtl_TextEncoding getTextEncoding() const;

	BOOL isCCMail() const { return m_bCCMail; }

	BOOL getItemThroughFile() const { return m_bGetItemThroughFile; }

	ULONG getMaxSubjectLen() const { return m_nMaxSubjectLen; }

	ULONG getMaxMemorySetItemSize() const
	{ return isCCMail() ? 65000 : 4000000000; }

	inline ULONG fnCloseMessage(CntVIMPointer & rMessage) const;

	inline ULONG fnCloseMessageContainer(CntVIMPointer & rContainer) const;

	inline ULONG fnCloseSession(CntVIMPointer & rSession) const;

	inline ULONG fnCreateMessage(CntVIMPointer & rSession, sal_Char * sType,
								 CntVIMPointer & rMessage) const;

	inline ULONG fnEnumerateMessageHeaderAttrs(CntVIMPointer & rMessage,
											   ULONG nAttr,
											   CntVIMEnumRef * pPos,
											   long nSkip, USHORT nEntrySize,
											   sal_Char * sBuff,
											   USHORT * pnCount,
											   USHORT * pnMore) const;

	inline ULONG fnEnumerateMessageItems(CntVIMPointer & rMessage,
										 CntVIMEnumRef * pPos, long nSkip,
										 USHORT * pnCount,
										 CntVIMItemDesc * pDesc,
										 ULONG nFilter, void * pData,
										 USHORT * pnMore) const;

	inline ULONG fnEnumerateMessageRecipients(CntVIMPointer & rMessage,
											  ULONG nType,
											  CntVIMEnumRef * pPos,
											  long nSkip, USHORT nAttrCount,
											  CntVIMAttrDescriptor * pAttr,
											  USHORT * pnCount,
											  USHORT * pnMore) const;

	inline ULONG fnEnumerateMessages(CntVIMPointer & rContainer,
									 CntVIMEnumRef * pPos, long nSkip,
									 USHORT nAttribCount,
									 CntVIMAttrDescriptor * pAttr,
									 USHORT * pnCount, ULONG nFilter,
									 void * pData, ULONG nFlags,
									 USHORT * pnMore) const;

	inline ULONG fnGetDefaultSessionInfo(USHORT nSizePath, sal_Char * sPath,
										 USHORT nSizeName, sal_Char * sName)
		const;

	inline ULONG fnGetMessageHeader(CntVIMPointer & rMessage,
									USHORT nAttrCount,
									CntVIMAttrDescriptor * pAttr) const;

	inline ULONG fnGetMessageItem(CntVIMPointer & rMessage,
								  const CntVIMRef & rItem,
								  sal_Char * sConvert, ULONG nFlags,
								  CntVIMBFDescriptor * pDesc)
		const;

	inline ULONG fnInitialize() const;

	inline ULONG fnMarkMessageAsRead(CntVIMPointer & rContainer,
									 const CntVIMRef & rRef) const;

	inline ULONG fnOpenMessage(CntVIMPointer & rContainer,
							   const CntVIMRef & rRef, sal_Char * sKey,
							   CntVIMPointer & rMessage) const;

	inline ULONG fnOpenMessageContainer(CntVIMPointer & rSession,
										sal_Char * sName,
										CntVIMPointer & rContainer) const;

	inline ULONG fnOpenSession(sal_Char * sPath, sal_Char * sUser,
							   sal_Char * sPassword, ULONG nVersion,
							   ULONG nCharSet, CntVIMPointer & rSession)
		const;

	inline ULONG fnQueryCapability(ULONG nType, USHORT nSize, void * pData)
		const;

	inline ULONG fnQueryNewMessages(CntVIMPointer & rContainer,
									USHORT * pnNew) const;

	inline ULONG fnRemoveMessage(CntVIMPointer & rContainer,
								 const CntVIMRef & rRef) const;

	inline ULONG fnSendMessage(CntVIMPointer & rMessage) const;

	inline ULONG fnSetMessageHeader(CntVIMPointer & rMessage, ULONG nAttr,
									USHORT nSize, void * pData) const;

	inline ULONG fnSetMessageItem(CntVIMPointer & rMessage, ULONG nClass,
								  sal_Char * sType, ULONG nFlags,
								  sal_Char * sName,
								  CntVIMBFDescriptor * pDesc) const;

	inline ULONG fnSetMessageRecipient(CntVIMPointer & rMessage, ULONG nClass,
									   CntVIMRecipient * pRecipient) const;

	inline ULONG fnStatusText(CntVIMPointer & rSession, ULONG nStatus,
							  USHORT nTextSize, sal_Char * sText,
							  USHORT nExtSize, sal_Char * sExt) const;

	inline ULONG fnTerminate() const;
};

inline CntVIMPointer::CntVIMPointer(const CntVIMLibrary & rLibrary,
									const CntVIMPointer & rOther)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_pAltPointer = rOther.m_pAltPointer;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_pPointer = rOther.m_pPointer;
}

inline void CntVIMPointer::clear(const CntVIMLibrary & rLibrary)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_pAltPointer = 0;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_pPointer = 0;
}

inline BOOL CntVIMPointer::isSet(const CntVIMLibrary & rLibrary) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		return CAST_TO_BOOL(m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return CAST_TO_BOOL(m_pPointer);
}

inline void CntVIMAttrDescriptor::setAttr(const CntVIMLibrary & rLibrary,
										  ULONG nAttr)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_aAltDesc.Attr = nAttr;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_aDesc.Attr = nAttr;
}

inline void CntVIMAttrDescriptor::setSize(const CntVIMLibrary & rLibrary,
										  ULONG nSize)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_aAltDesc.Size = nSize;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_aDesc.Size = nSize;
}

inline void CntVIMAttrDescriptor::setBuffer(const CntVIMLibrary & rLibrary,
											void * pBuffer)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_aAltDesc.Buffer = pBuffer;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_aDesc.Buffer = pBuffer;
}

inline ULONG CntVIMBFDescriptor::getSize(const CntVIMLibrary & rLibrary)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		return m_aAltDesc.Size;
	else
#endif // CNT_VIM_ALT_LINKAGE
		return m_aDesc.Size;
}

inline void * CntVIMBFDescriptor::getBuffer(const CntVIMLibrary & rLibrary)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		return m_aAltDesc.Buffer;
	else
#endif // CNT_VIM_ALT_LINKAGE
		return m_aDesc.Buffer;
}

inline void CntVIMBFDescriptor::setSize(const CntVIMLibrary & rLibrary,
										ULONG nSize)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_aAltDesc.Size = nSize;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_aDesc.Size = nSize;
}

inline void CntVIMBFDescriptor::setBuffer(const CntVIMLibrary & rLibrary,
										  void * pBuffer)
{
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
		m_aAltDesc.Buffer = pBuffer;
	else
#endif // CNT_VIM_ALT_LINKAGE
		m_aDesc.Buffer = pBuffer;
}

inline void CntVIMBFDescriptor::setFileName(const CntVIMLibrary & rLibrary,
											const String & rFileName)
{
	ByteString aTheFileName(rFileName, osl_getThreadTextEncoding());
	sal_Char * sFileNameBuffer = new sal_Char[aTheFileName.Len() + 1];
	rtl_copyMemory(sFileNameBuffer, aTheFileName.GetBuffer(),
				   aTheFileName.Len() + 1);
#if defined CNT_VIM_ALT_LINKAGE
	if (rLibrary.useAltLinkage())
	{
		delete m_aAltDesc.FileName;
		m_aAltDesc.FileName = sFileNameBuffer;
	}
	else
#endif // CNT_VIM_ALT_LINKAGE
	{
		delete m_aDesc.FileName;
		m_aDesc.FileName = sFileNameBuffer;
	}
}

inline ULONG CntVIMLibrary::fnCloseMessage(CntVIMPointer & rMessage) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnCloseMessage, m_pFnCloseMessage)
		        (rMessage.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return
		 STATIC_CAST(CntVIMFnCloseMessage, m_pFnCloseMessage)(rMessage.
															   m_pPointer);
}

inline ULONG CntVIMLibrary::fnCloseMessageContainer(CntVIMPointer &
													 rContainer) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnCloseMessageContainer,
						   m_pFnCloseMessageContainer)
		        (rContainer.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnCloseMessageContainer,
						   m_pFnCloseMessageContainer)
		        (rContainer.m_pPointer);
}

inline ULONG CntVIMLibrary::fnCloseSession(CntVIMPointer & rSession) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnCloseSession, m_pFnCloseSession)
		        (rSession.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return
		 STATIC_CAST(CntVIMFnCloseSession, m_pFnCloseSession)(rSession.
															   m_pPointer);
}

inline ULONG CntVIMLibrary::fnCreateMessage(CntVIMPointer & rSession,
											sal_Char * sType,
											CntVIMPointer & rMessage) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnCreateMessage, m_pFnCreateMessage)
		        (rSession.m_pAltPointer, sType, &rMessage.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnCreateMessage, m_pFnCreateMessage)
		        (rSession.m_pPointer, sType, &rMessage.m_pPointer);
}

inline ULONG CntVIMLibrary::fnEnumerateMessageHeaderAttrs(CntVIMPointer &
														   rMessage,
														  ULONG nAttr,
														  CntVIMEnumRef *
														   pPos,
														  long nSkip,
														  USHORT nEntrySize,
														  sal_Char * sBuff,
														  USHORT * pnCount,
														  USHORT * pnMore)
	const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnEnumerateMessageHeaderAttrs,
						   m_pFnEnumerateMessageHeaderAttrs)
		        (rMessage.m_pAltPointer, nAttr, pPos, nSkip, nEntrySize,
				 sBuff, pnCount, pnMore);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnEnumerateMessageHeaderAttrs,
						   m_pFnEnumerateMessageHeaderAttrs)
		        (rMessage.m_pPointer, nAttr, pPos, nSkip, nEntrySize, sBuff,
				 pnCount, pnMore);
}

inline ULONG CntVIMLibrary::fnEnumerateMessageItems(CntVIMPointer & rMessage,
													CntVIMEnumRef * pPos,
													long nSkip,
													USHORT * pnCount,
													CntVIMItemDesc * pDesc,
													ULONG nFilter,
													void * pData,
													USHORT * pnMore) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnEnumerateMessageItems,
						   m_pFnEnumerateMessageItems)
		        (rMessage.m_pAltPointer, pPos, nSkip, pnCount, pDesc, nFilter,
				 pData, pnMore);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnEnumerateMessageItems,
						   m_pFnEnumerateMessageItems)
		        (rMessage.m_pPointer, pPos, nSkip, pnCount, pDesc, nFilter,
				 pData, pnMore);
}

inline
ULONG CntVIMLibrary::fnEnumerateMessageRecipients(CntVIMPointer & rMessage,
												  ULONG nType,
												  CntVIMEnumRef * pPos,
												  long nSkip,
												  USHORT nAttrCount,
												  CntVIMAttrDescriptor *
												   pAttr,
												  USHORT * pnCount,
												  USHORT * pnMore) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnEnumerateMessageRecipients,
						   m_pFnEnumerateMessageRecipients)
		        (rMessage.m_pAltPointer, nType, pPos, nSkip, nAttrCount,
				 pAttr ? &pAttr->m_aAltDesc : 0, pnCount, pnMore);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnEnumerateMessageRecipients,
						   m_pFnEnumerateMessageRecipients)
		        (rMessage.m_pPointer, nType, pPos, nSkip, nAttrCount,
				 pAttr ? &pAttr->m_aDesc : 0, pnCount, pnMore);
}

inline ULONG CntVIMLibrary::fnEnumerateMessages(CntVIMPointer & rContainer,
												CntVIMEnumRef * pPos,
												long nSkip,
												USHORT nAttribCount,
												CntVIMAttrDescriptor * pAttr,
												USHORT * pnCount,
												ULONG nFilter, void * pData,
												ULONG nFlags, USHORT * pnMore)
	const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnEnumerateMessages,
						   m_pFnEnumerateMessages)
		        (rContainer.m_pAltPointer, pPos, nSkip, nAttribCount,
				 pAttr ? &pAttr->m_aAltDesc : 0, pnCount, nFilter, pData,
				 nFlags, pnMore);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnEnumerateMessages, m_pFnEnumerateMessages)
		        (rContainer.m_pPointer, pPos, nSkip, nAttribCount,
				 pAttr ? &pAttr->m_aDesc : 0, pnCount, nFilter, pData, nFlags,
				 pnMore);
}

inline ULONG CntVIMLibrary::fnGetDefaultSessionInfo(USHORT nSizePath,
													sal_Char * sPath,
													USHORT nSizeName,
													sal_Char * sName) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnGetDefaultSessionInfo,
						   m_pFnGetDefaultSessionInfo)
		        (nSizePath, sPath, nSizeName, sName);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnGetDefaultSessionInfo,
						   m_pFnGetDefaultSessionInfo)
		        (nSizePath, sPath, nSizeName, sName);
}

inline ULONG CntVIMLibrary::fnGetMessageHeader(CntVIMPointer & rMessage,
											   USHORT nAttrCount,
											   CntVIMAttrDescriptor * pAttr)
	const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnGetMessageHeader, m_pFnGetMessageHeader)
		        (rMessage.m_pAltPointer, nAttrCount,
				 pAttr ? &pAttr->m_aAltDesc : 0);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnGetMessageHeader, m_pFnGetMessageHeader)
		        (rMessage.m_pPointer, nAttrCount,
				 pAttr ? &pAttr->m_aDesc : 0);
}

inline ULONG CntVIMLibrary::fnGetMessageItem(CntVIMPointer & rMessage,
											 const CntVIMRef & rItem,
											 sal_Char * sConvert,
											 ULONG nFlags,
											 CntVIMBFDescriptor * pDesc) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnGetMessageItem, m_pFnGetMessageItem)
		        (rMessage.m_pAltPointer, rItem, sConvert, nFlags,
				 pDesc ? &pDesc->m_aAltDesc : 0);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnGetMessageItem, m_pFnGetMessageItem)
		        (rMessage.m_pPointer, rItem, sConvert, nFlags,
				 pDesc ? &pDesc->m_aDesc : 0);
}

inline ULONG CntVIMLibrary::fnInitialize() const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnInitialize, m_pFnInitialize)();
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnInitialize, m_pFnInitialize)();
}

inline ULONG CntVIMLibrary::fnMarkMessageAsRead(CntVIMPointer & rContainer,
												const CntVIMRef & rRef) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnMarkMessageAsRead,
						   m_pFnMarkMessageAsRead)
		        (rContainer.m_pAltPointer, rRef);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnMarkMessageAsRead, m_pFnMarkMessageAsRead)
		        (rContainer.m_pPointer, rRef);
}

inline ULONG CntVIMLibrary::fnOpenMessage(CntVIMPointer & rContainer,
										  const CntVIMRef & rRef,
										  sal_Char * sKey,
										  CntVIMPointer & rMessage) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnOpenMessage, m_pFnOpenMessage)
		        (rContainer.m_pAltPointer, rRef, sKey,
				 &rMessage.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnOpenMessage, m_pFnOpenMessage)
		        (rContainer.m_pPointer, rRef, sKey, &rMessage.m_pPointer);
}

inline ULONG CntVIMLibrary::fnOpenMessageContainer(CntVIMPointer & rSession,
												   sal_Char * sName,
												   CntVIMPointer & rContainer)
	const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnOpenMessageContainer,
						   m_pFnOpenMessageContainer)
		        (rSession.m_pAltPointer, sName, &rContainer.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnOpenMessageContainer,
						   m_pFnOpenMessageContainer)
		        (rSession.m_pPointer, sName, &rContainer.m_pPointer);
}

inline ULONG CntVIMLibrary::fnOpenSession(sal_Char * sPath, sal_Char * sUser,
										  sal_Char * sPassword,
										  ULONG nVersion, ULONG nCharSet,
										  CntVIMPointer & rSession) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnOpenSession, m_pFnOpenSession)
		        (sPath, sUser, sPassword, nVersion, nCharSet,
				 &rSession.m_pAltPointer);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnOpenSession, m_pFnOpenSession)
		        (sPath, sUser, sPassword, nVersion, nCharSet,
				 &rSession.m_pPointer);
}

inline ULONG CntVIMLibrary::fnQueryCapability(ULONG nType, USHORT nSize,
											  void * pData) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnQueryCapability, m_pFnQueryCapability)
		        (nType, nSize, pData);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnQueryCapability, m_pFnQueryCapability)
		        (nType, nSize, pData);
}

inline ULONG CntVIMLibrary::fnQueryNewMessages(CntVIMPointer & rContainer,
											   USHORT * pnNew) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnQueryNewMessages, m_pFnQueryNewMessages)
		        (rContainer.m_pAltPointer, pnNew);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnQueryNewMessages, m_pFnQueryNewMessages)
		        (rContainer.m_pPointer, pnNew);
}

inline ULONG CntVIMLibrary::fnRemoveMessage(CntVIMPointer & rContainer,
											const CntVIMRef & rRef) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnRemoveMessage, m_pFnRemoveMessage)
		        (rContainer.m_pAltPointer, rRef);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnRemoveMessage, m_pFnRemoveMessage)
		        (rContainer.m_pPointer, rRef);
}

inline ULONG CntVIMLibrary::fnSendMessage(CntVIMPointer & rMessage) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnSendMessage, m_pFnSendMessage)
		        (rMessage.m_pAltPointer, 0, 0);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnSendMessage, m_pFnSendMessage)
		        (rMessage.m_pPointer, 0, 0);
}

inline ULONG CntVIMLibrary::fnSetMessageHeader(CntVIMPointer & rMessage,
											   ULONG nAttr, USHORT nSize,
											   void * pData) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnSetMessageHeader, m_pFnSetMessageHeader)
		        (rMessage.m_pAltPointer, nAttr, nSize, pData);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnSetMessageHeader, m_pFnSetMessageHeader)
		        (rMessage.m_pPointer, nAttr, nSize, pData);
}

inline ULONG CntVIMLibrary::fnSetMessageItem(CntVIMPointer & rMessage,
											 ULONG nClass, sal_Char * sType,
											 ULONG nFlags, sal_Char * sName,
											 CntVIMBFDescriptor * pDesc) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnSetMessageItem, m_pFnSetMessageItem)
		        (rMessage.m_pAltPointer, nClass, sType, nFlags, sName,
				 pDesc ? &pDesc->m_aAltDesc : 0);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnSetMessageItem, m_pFnSetMessageItem)
		        (rMessage.m_pPointer, nClass, sType, nFlags, sName,
				 pDesc ? &pDesc->m_aDesc : 0);
}

inline ULONG CntVIMLibrary::fnSetMessageRecipient(CntVIMPointer & rMessage,
												  ULONG nClass,
												  CntVIMRecipient *
												   pRecipient) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnSetMessageRecipient,
						   m_pFnSetMessageRecipient)
		        (rMessage.m_pAltPointer, nClass, pRecipient);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnSetMessageRecipient,
						   m_pFnSetMessageRecipient)
		        (rMessage.m_pPointer, nClass, pRecipient);
}

inline ULONG CntVIMLibrary::fnStatusText(CntVIMPointer & rSession,
										 ULONG nStatus, USHORT nTextSize,
										 sal_Char * sText, USHORT nExtSize,
										 sal_Char * sExt) const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnStatusText, m_pFnStatusText)
		        (rSession.m_pAltPointer, nStatus, nTextSize, sText, nExtSize,
				 sExt);
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnStatusText, m_pFnStatusText)
		        (rSession.m_pPointer, nStatus, nTextSize, sText, nExtSize,
				 sExt);
}

inline ULONG CntVIMLibrary::fnTerminate() const
{
#if defined CNT_VIM_ALT_LINKAGE
	if (useAltLinkage())
		return STATIC_CAST(CntVIMAltFnTerminate, m_pFnTerminate)();
	else
#endif // CNT_VIM_ALT_LINKAGE
		return STATIC_CAST(CntVIMFnTerminate, m_pFnTerminate)();
}

//============================================================================
class CntVIMMessageTextItem: public CntVIMMessageItem
{
public:
	enum CharSet { ISO88591, CP850 = VIMSEL_CP850, CP1252 = VIMSEL_CP1252,
				   CP437 = VIMSEL_CP437 };

private:
	String m_aTitle;
	CharSet m_eCharSet;

public:
	inline CntVIMMessageTextItem(const String & rTheTitle,
								 SvStream * pDataStream);

	inline CntVIMMessageTextItem(const String & rTheTitle,
								 SvStream * pDataStream,
								 ULONG nTheCharSet);

	virtual inline String getContentType() const;

	virtual String getName() const { return m_aTitle; }
};

inline CntVIMMessageTextItem::CntVIMMessageTextItem(const String & rTheTitle,
													SvStream * pDataStream):
	CntVIMMessageItem(pDataStream),
	m_aTitle(rTheTitle),
	m_eCharSet(ISO88591)
{}

inline CntVIMMessageTextItem::CntVIMMessageTextItem(const String & rTheTitle,
													SvStream * pDataStream,
													ULONG nTheCharSet):
	CntVIMMessageItem(pDataStream),
	m_aTitle(rTheTitle),
	m_eCharSet(CharSet(nTheCharSet))
{}

// virtual
inline String CntVIMMessageTextItem::getContentType() const
{
	return
		m_eCharSet == ISO88591 || m_eCharSet == CP1252 ?
		    String::CreateFromAscii(RTL_CONSTASCII_STRINGPARAM(
				                        "text/plain; charset=iso-8859-1")) :
		m_eCharSet == CP850 ?
		    String::CreateFromAscii(RTL_CONSTASCII_STRINGPARAM(
				                        "text/plain; charset=IBM850")) :
		    String::CreateFromAscii(RTL_CONSTASCII_STRINGPARAM(
				                        "text/plain; charset=IBM437"));
		// cf. ftp://ftp.isi.edu/in-notes/iana/assignments/character-sets
}

//============================================================================
class CntVIMMessageRTFItem: public CntVIMMessageItem
{
	String m_aTitle;

public:
	CntVIMMessageRTFItem(const String & rTheTitle, SvStream * pDataStream):
		CntVIMMessageItem(pDataStream), m_aTitle(rTheTitle) {}

	virtual inline String getContentType() const;

	virtual String getName() const { return m_aTitle; }
};

// virtual
inline String CntVIMMessageRTFItem::getContentType() const
{
	return String::CreateFromAscii(RTL_CONSTASCII_STRINGPARAM(
		                               "application/rtf"));
}

//============================================================================
class CntVIMMessageFileItem: public CntVIMMessageItem
{
	String m_aFilename;

public:
	CntVIMMessageFileItem(const String & rTheFilename,
						  SvStream * pDataStream):
		CntVIMMessageItem(pDataStream), m_aFilename(rTheFilename) {}

	virtual String getContentType() const;

	virtual String getName() const { return m_aFilename; }
};

//============================================================================
class CntVIMInboxMessageBody: public CntVIMMessageBody
{
	CntVIMPointer m_aMsg;
	CntVIMEnumRef m_aPos;
	USHORT m_nMore;

public:
	inline CntVIMInboxMessageBody(CntVIMStatus & rTheStatus,
								  const CntVIMLibrary & rTheLibrary,
								  const CntVIMPointer & rTheMsg);

	virtual ~CntVIMInboxMessageBody();

	virtual BOOL startItemIteration(ULONG & rnItems, CntNodeJob * pJob);

	virtual CntVIMMessageItem * getNextItem(CntNodeJob * pJob);
};

inline CntVIMInboxMessageBody::CntVIMInboxMessageBody(CntVIMStatus &
													   rTheStatus,
													  const CntVIMLibrary &
													   rTheLibrary,
													  const CntVIMPointer &
													   rTheMsg):
	CntVIMMessageBody(rTheStatus, rTheLibrary),
	m_aMsg(rTheLibrary, rTheMsg)
{}

//============================================================================
class CntVIMInbox: public CntVIMMessageContainer
{
	enum IterateDirection { START, END, UNKNOWN };

	CntVIMMailbox * pMailbox;
	CntVIMPointer aContainer;
	BOOL bJustOpened;
	IterateDirection eDirection;
	CntVIMEnumRef aPos;
	long nSkip;
	USHORT bMore;

	String createFakedUniqueID(sal_Char const * pFrom,
							   CntVIMDate const & rDate, CntNodeJob * pJob);

	BOOL getRecipients(CntVIMPointer & rMsg, ULONG nType,
					   String & rRecipients, CntNodeJob * pJob);

public:
	inline CntVIMInbox(CntVIMStatus & rTheStatus,
					   const CntVIMLibrary & rTheLibrary,
					   CntVIMMailbox * pTheMailbox);

	virtual ~CntVIMInbox();

	virtual BOOL open(CntNodeJob * pJob);

	virtual BOOL canSeekForMsgRefID() { return FALSE; }

	virtual BOOL canIterateFromEnd(CntNodeJob * pJob);

	virtual BOOL queryNewMessages(BOOL bTryHard, CntNodeJob * pJob);

	virtual ExecuteState startHeaderIteration(UINT32, UINT32, CntNodeJob *);

	virtual ExecuteState startHeaderIteration(BOOL bFromEnd);

	virtual CntVIMMessageHeader * getNextHeader(CntNodeJob * pJob,
												ULONG nFilterType,
												const void * pFilterData,
												ULONG nFilterDataLen);

	virtual BOOL findMessage(const String & rUniqueID, UINT32 & rRefIDPart1,
							 UINT32 & rRefIDPart2, CntNodeJob * pJob,
							 BOOL bNotFoundIsOK);

	virtual ExecuteState openMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
									 CntNodeJob * pJob,
									 CntVIMMessageBody *& rpBody);

	virtual BOOL notifyMessageRead(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
								   CntNodeJob * pJob);

	virtual BOOL deleteMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
							   CntNodeJob * pJob);

	virtual void cancelExecution(BOOL) {}
};

inline CntVIMInbox::CntVIMInbox(CntVIMStatus & rTheStatus,
								const CntVIMLibrary & rTheLibrary,
								CntVIMMailbox * pTheMailbox):
	CntVIMMessageContainer(rTheStatus, rTheLibrary),
	pMailbox(pTheMailbox),
	aContainer(rTheLibrary),
	eDirection(UNKNOWN)
{}

//============================================================================
class CntVIMBBoardMessageBodyItemList
{
public:
	enum ItemKeyword { FIRST_ITEM_KEYWORD, TEXT_ITEM = FIRST_ITEM_KEYWORD,
					   FILE_ITEM, FAX_ITEM, GRAPHICS_ITEM,
					   NUM_ITEM_KEYWORDS };

private:
	ItemKeyword m_eType;
	String m_aTitle;
	ULONG m_nStart;
	ULONG m_nSize;
	CntVIMBBoardMessageBodyItemList * m_pNext;

public:
	inline CntVIMBBoardMessageBodyItemList(ItemKeyword eTheType,
										   const String & rTheTitle,
										   ULONG nTheStart, ULONG nTheSize);

	ItemKeyword getType() const { return m_eType; }

	const String & getTitle() const { return m_aTitle; }

	ULONG getStart() const { return m_nStart; }

	ULONG getSize() const { return m_nSize; }

	CntVIMBBoardMessageBodyItemList * getNext() const { return m_pNext; }

	CntVIMBBoardMessageBodyItemList * append(CntVIMBBoardMessageBodyItemList *
											  pTheNext)
	{ return m_pNext = pTheNext; }
};

inline CntVIMBBoardMessageBodyItemList::CntVIMBBoardMessageBodyItemList
        (ItemKeyword eTheType, const String & rTheTitle, ULONG nTheStart,
		 ULONG nTheSize):
	m_eType(eTheType),
	m_aTitle(rTheTitle),
	m_nStart(nTheStart),
	m_nSize(nTheSize),
	m_pNext(0)
{}

//============================================================================
class CntVIMBBoardMessageBody: public CntVIMMessageBody
{
	SvStream * m_pStream;
	CntVIMBBoardMessageBodyItemList * m_pHead;
	CntVIMBBoardMessageBodyItemList * m_pTail;

public:
	inline CntVIMBBoardMessageBody(CntVIMStatus & rStatus,
								   const CntVIMLibrary & rLibrary,
								   SvStream * pTheStream);

	virtual ~CntVIMBBoardMessageBody();

	virtual BOOL startItemIteration(ULONG & rnItems, CntNodeJob * pJob);

	virtual CntVIMMessageItem * getNextItem(CntNodeJob * pJob);
};

inline CntVIMBBoardMessageBody::CntVIMBBoardMessageBody(CntVIMStatus &
														 rStatus,
														const CntVIMLibrary &
														 rLibrary,
														SvStream *
														 pTheStream):
	CntVIMMessageBody(rStatus, rLibrary),
	m_pStream(pTheStream),
	m_pHead(0)
{}

//============================================================================
class CntVIMBulletinBoard: public CntVIMMessageContainer
{
	/// The maximum time in ms to wait for the cc:Mail export program to
	/// terminate.
	enum { EXPORT_MAX_WAIT = 30000 };

	enum FieldKeyword { FIRST_FIELD_KEYWORD,
						FIELD_KEY_MESSAGE = FIRST_FIELD_KEYWORD,
						FIELD_KEY_MSG_NUM, FIELD_KEY_FROM, FIELD_KEY_DATE,
						FIELD_KEY_CC, FIELD_KEY_BCC, FIELD_KEY_STARED_TO,
						FIELD_KEY_STARED_CC, FIELD_KEY_PRIORITY,
						FIELD_KEY_SUBJECT, NUM_FIELD_KEYWORDS,
						ANY_FIELD_KEYWORD = NUM_FIELD_KEYWORDS };

	CntVIMMailbox * m_pMailbox;
	String m_aName;
	BOOL m_bJustOpened;
	CntVIMTempFileStream * m_pHeaderStream;
	BOOL m_bEOF;
	FieldKeyword m_eNextField;
	UINT32 m_nCurrentRefID;
	vos::OProcess * m_pProcess;
	ULONG m_nProcessWaitingCount;
	String m_aProcessTempFileName;

public:
	inline CntVIMBulletinBoard(CntVIMStatus & rStatus,
							   const CntVIMLibrary & rLibrary,
							   CntVIMMailbox * pTheMailbox,
							   const String & rTheName);

	virtual ~CntVIMBulletinBoard();

	virtual BOOL open(CntNodeJob *) { return TRUE; }

	virtual BOOL canSeekForMsgRefID() { return TRUE; }

	virtual BOOL canIterateFromEnd(CntNodeJob *) { return FALSE; }

	virtual BOOL queryNewMessages(BOOL bTryHard, CntNodeJob *)
	{ return m_bJustOpened || bTryHard; }

	virtual ExecuteState startHeaderIteration(UINT32 nRefIDPart1,
											  UINT32 nRefIDPart2,
											  CntNodeJob * pJob);

	virtual ExecuteState startHeaderIteration(BOOL);

	virtual CntVIMMessageHeader * getNextHeader(CntNodeJob * pJob,
												ULONG nFilterType,
												const void * pFilterData,
												ULONG nFilterDataLen);

	virtual BOOL findMessage(const String & rUniqueID, UINT32 & rRefIDPart1,
							 UINT32 & rRefIDPart2, CntNodeJob * pJob, BOOL);

	virtual ExecuteState openMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
									 CntNodeJob * pJob,
									 CntVIMMessageBody *& rpBody);

	virtual BOOL notifyMessageRead(UINT32, UINT32, CntNodeJob *);

	virtual BOOL deleteMessage(UINT32, UINT32, CntNodeJob *);

	virtual void cancelExecution(BOOL bTerminate = TRUE);
};

inline CntVIMBulletinBoard::CntVIMBulletinBoard(CntVIMStatus & rStatus,
												const CntVIMLibrary &
												 rLibrary,
												CntVIMMailbox * pTheMailbox,
												const String & rTheName):
	CntVIMMessageContainer(rStatus, rLibrary),
	m_pMailbox(pTheMailbox),
	m_aName(rTheName),
	m_bJustOpened(TRUE),
	m_pHeaderStream(0),
	m_pProcess(0)
{}

//============================================================================
//
//  class CntVIMStatus
//
//============================================================================

void CntVIMStatus::handleError(ErrCode nSpecificCode, CntNodeJob & rJob)
{
	switch (ERRCODE_TOERRID(nSpecificCode))
	{
		case ERRCODE_CHAOS_CCMAIL_EXPORT_ERROR:
		case ERRCODE_CHAOS_NO_CCMAIL_EXPORT_FILE:
		case ERRCODE_CHAOS_ILLEGAL_CCMAIL_EXPORT_FILE:
		case ERRCODE_CHAOS_BAD_CCMAIL_EXPORT_PASSWORD:
		case ERRCODE_CHAOS_CCMAIL_EXPORT_TOO_LONG:
		case ERRCODE_CHAOS_CCMAIL_EXPORT_NOT_TERMINATING:
			rJob.Broadcast(CntVIMStatusHint());
			break;
	}
	rJob.SetError(nSpecificCode);
	rJob.Cancel(); // every error is fatal (maybe this should be changed)
}

//============================================================================
BOOL CntVIMStatus::setStatus(ErrCode nTheCode, CntNodeJob * pJob)
{
	DBG_ASSERT(nTheCode != ERRCODE_CHAOS_VIM_LIBRARY_ERROR,
			   "CntVIMStatus::setStatus(): Call setVIMStatus() instead");

	m_nCode = nTheCode;
	if (m_nCode == ERRCODE_NONE)
		return TRUE;
	if (pJob)
		handleError(m_nCode, *pJob);
	return FALSE;
}

//============================================================================
BOOL CntVIMStatus::setVIMStatus(ULONG nTheVIMStatus, CntNodeJob * pJob)
{
	m_nCode = ERRCODE_CHAOS_VIM_LIBRARY_ERROR;
	m_nVIMStatus = nTheVIMStatus;
	if (VIM_STATUS(m_nVIMStatus) == VIMSTS_SUCCESS)
		return TRUE;
	if (pJob)
		handleError(*new StringErrorInfo(m_nCode, getStatusText()), *pJob);
	return FALSE;
}

//============================================================================
BOOL CntVIMStatus::isOK() const
{
	return m_nCode == ERRCODE_NONE
		   || m_nCode == ERRCODE_CHAOS_VIM_LIBRARY_ERROR
		      && VIM_STATUS(m_nVIMStatus) == VIMSTS_SUCCESS;
}

//============================================================================
USHORT CntVIMStatus::getStatusValue() const
{
	return m_nCode == ERRCODE_CHAOS_VIM_LIBRARY_ERROR ?
		    VIM_STATUS(m_nVIMStatus) :
		   m_nCode == ERRCODE_NONE ?
		    0 : 1;
}

//============================================================================
String CntVIMStatus::getStatusText(BOOL bSendStatus) const
{
	if (isOK())
	{
		DBG_ASSERT(bSendStatus,
				   "CntVIMStatus::getStatusText(): Called w/o error");

		return CntResId(RID_PROT_MESSAGE_SENT);
	}

	if (m_nCode == ERRCODE_CHAOS_VIM_LIBRARY_ERROR)
	{
		if (!m_rMailbox.getLibrary().isInitialized())
			return CntResId(RID_PROT_UNKNOWN_VIM_ERROR);

		const USHORT nTextSize = 1024; // can be changed
		sal_Char sStatusText[nTextSize];
		sal_Char sExtStatusText[nTextSize];
		if (VIM_STATUS(m_rMailbox.
					    getLibrary().fnStatusText(m_rMailbox.getSession(),
												  m_nVIMStatus,
												  sizeof sStatusText,
												  sStatusText,
												  sizeof sExtStatusText,
												  sExtStatusText))
			 != VIMSTS_SUCCESS
			|| !sStatusText[0])
			return CntResId(RID_PROT_UNKNOWN_VIM_ERROR);

		String aText(sStatusText, m_rMailbox.getTextEncoding());
		if (sExtStatusText[0])
		{
			aText += bSendStatus ? ' ' : '\n';
			String aExtText(CntResId(RID_PROT_VIM_EXTENDED_ERROR));
			aExtText.SearchAndReplaceAscii("%1",
										   String(sExtStatusText,
												  m_rMailbox.
												      getTextEncoding()));
			aText += aExtText;
		}
		return aText;
	}

	return CntResId(RID_PROT_ERROR_OCCURED);
}

//============================================================================
//
//  class CntVIMStatusHint
//
//============================================================================

TYPEINIT1(CntVIMStatusHint, SfxHint);

//============================================================================
//
//  class CntVIMMessageItem
//
//============================================================================

// virtual
CntVIMMessageItem::~CntVIMMessageItem()
{
	delete m_pDataStream;
}

//============================================================================
//
//  class CntVIMMailbox
//
//============================================================================

BOOL CntVIMMailbox::transformMessage(const INetCoreMIMEMessage & rMIMEMsg,
									 CntVIMPointer & rVIMMsg,
									 CntVIMSendBufferNode *& rpBuffers,
									 BOOL & rbAddNotePart, CntNodeJob * pJob)
{
	// Determine the kind of part to add for this MIME message:
	enum PartType { PART_NONE, PART_NOTE, PART_ATTACHMENT };
	PartType ePart = PART_ATTACHMENT;

	String aType;
	String aSubType;
	INetContentTypeParameterList aParameters;
	if (INetContentTypes::parse(rMIMEMsg.GetContentType(), aType, aSubType,
								&aParameters))
		if (aType.EqualsAscii("message"))
		{
			if (aSubType.EqualsAscii("rfc822"))
				ePart = PART_NONE;
		}
		else if (aType.EqualsAscii("multipart"))
		{
			if (rMIMEMsg.GetDocumentFilename().Len() > 0
				|| rMIMEMsg.GetDocumentStream() != 0)
			{
				if (rbAddNotePart)
					ePart = PART_NOTE;
			}
			else
				ePart = PART_NONE;
		}
		else if (aType.EqualsAscii("text"))
		{
			if (rbAddNotePart && aSubType.EqualsAscii("plain"))
			{
				const INetContentTypeParameter * pCharsetParameter
					= aParameters.find(ByteString(RTL_CONSTASCII_STRINGPARAM(
						                              "charset")));
				if (pCharsetParameter)
					switch (INetMIME::getCharsetEncoding(
						        pCharsetParameter->m_sValue.GetBuffer(),
								pCharsetParameter->m_sValue.GetBuffer()
								    + pCharsetParameter->m_sValue.Len()))
					{
						case RTL_TEXTENCODING_ASCII_US:
							ePart = PART_NOTE;
							break;

						case RTL_TEXTENCODING_ISO_8859_1:
						case RTL_TEXTENCODING_MS_1252:
							if (m_pLibrary->getCharSet() == VIMSEL_CP1252)
								ePart = PART_NOTE;
							break;

						case RTL_TEXTENCODING_IBM_437:
							if (m_pLibrary->getCharSet() == VIMSEL_CP437)
								ePart = PART_NOTE;
							break;

						case RTL_TEXTENCODING_IBM_850:
							if (m_pLibrary->getCharSet() == VIMSEL_CP850)
								ePart = PART_NOTE;
							break;
					}
			}
		}

	// Add either a note part or an attachment:
	switch (ePart)
	{
		case PART_NOTE:
		{
			rbAddNotePart = FALSE;

			CntVIMBFDescriptor aDesc;
			if (rMIMEMsg.GetDocumentStream())
			{
				SvStream * pStream = rMIMEMsg.GetDocumentStream();
				ULONG nSize = pStream->Seek(STREAM_SEEK_TO_END);
				pStream->Seek(STREAM_SEEK_TO_BEGIN);

				if (nSize > m_pLibrary->getMaxMemorySetItemSize())
				{
					if (pStream->IsA() == ID_FILESTREAM)
						// Use available file:
						aDesc.
						 setFileName(*m_pLibrary,
									 INSECURE_DYNAMIC_CAST(SvFileStream *,
														   pStream)->
									  GetFileName());
					else
					{
						// Store stream into temporary file:
						rpBuffers
						 = new
						    CntVIMSendBufferNode(rpBuffers,
												 new CntVIMTempFileStream());
						*rpBuffers->m_pTempFile << *pStream;
						rpBuffers->m_pTempFile->Close();
						if (!m_pStatus->setStatus(rpBuffers->m_pTempFile->
												   GetErrorCode(),
												  pJob))
							return FALSE;
						aDesc.setFileName(*m_pLibrary,
										  rpBuffers->
										   m_pTempFile->GetFileName());
					}
				}
				else
				{
					// Store stream into memory buffer:
					rpBuffers = new CntVIMSendBufferNode(rpBuffers,
														 new sal_Char[nSize]);
					pStream->Read(rpBuffers->m_pBuffer, nSize);
					if (!m_pStatus->setStatus(pStream->GetErrorCode(), pJob))
						return FALSE;
					aDesc.setSize(*m_pLibrary, nSize);
					aDesc.setBuffer(*m_pLibrary, rpBuffers->m_pBuffer);
				}
			}
			else if (rMIMEMsg.GetDocumentFilename().Len())
				// Use available file:
				aDesc.setFileName(*m_pLibrary,
								  rMIMEMsg.GetDocumentFilename());
			else
			{
				// Create dummy memory buffer:
				rpBuffers = new CntVIMSendBufferNode(rpBuffers, new sal_Char);
				*rpBuffers->m_pBuffer = ' ';
				aDesc.setSize(*m_pLibrary, 1);
				aDesc.setBuffer(*m_pLibrary, rpBuffers->m_pBuffer);
			}

			sal_Char sTypeBuffer[] = VIM_TEXT;
#if !defined OS2
			sal_Char sNameBuffer[] = "";
#endif // OS2
			if (!m_pStatus->setVIMStatus(m_pLibrary->
										  fnSetMessageItem(rVIMMsg,
														   VIMSEL_NOTE_PART,
														   sTypeBuffer, 0,
#if defined OS2
														   0,
#else // OS2
														   sNameBuffer,
#endif // OS2
														   &aDesc),
										 pJob))
				return FALSE;
			break;
		}

		case PART_ATTACHMENT:
		{
			// If the content-description is of the form
			// <"FILENAME" "=" value>, use the contents of value as name,
			// otherwise the whole content-description <*text>:
			String aName(rMIMEMsg.GetContentDescription());
			INetContentTypeParameterList aParameters;
			sal_Unicode const * p = aName.GetBuffer();
			sal_Unicode const * pEnd = p + aName.Len();
			if (INetMIME::scanParameters(p, pEnd, &aParameters) == pEnd)
			{
				INetContentTypeParameter const * pParameter
					= aParameters.find(ByteString(RTL_CONSTASCII_STRINGPARAM(
						                              "filename")));
				if (pParameter && pParameter->m_bConverted)
					aName = pParameter->m_sValue;
			}

			CntVIMBFDescriptor aDesc;
			if (rMIMEMsg.GetDocumentFilename().Len())
				// Use available file:
				aDesc.setFileName(*m_pLibrary,
								  rMIMEMsg.GetDocumentFilename());
			else if (rMIMEMsg.GetDocumentStream())
			{
				SvStream * pStream = rMIMEMsg.GetDocumentStream();
				if (pStream->IsA() == ID_FILESTREAM)
					// Use available file:
					aDesc.setFileName(*m_pLibrary,
									  INSECURE_DYNAMIC_CAST(SvFileStream *,
															pStream)->
									   GetFileName());
				else
				{
					// Store stream into temporary file:
					rpBuffers
					 = new CntVIMSendBufferNode(rpBuffers,
												new CntVIMTempFileStream());
					pStream->Seek(STREAM_SEEK_TO_BEGIN);
					*rpBuffers->m_pTempFile << *pStream;
					rpBuffers->m_pTempFile->Close();
					if (!m_pStatus->
						  setStatus(rpBuffers->m_pTempFile->GetErrorCode(),
									pJob))
						return FALSE;
					aDesc.setFileName(*m_pLibrary,
									  rpBuffers->m_pTempFile->GetFileName());
				}
			}
			else
			{
				// Create dummy temporary file:
				rpBuffers
				 = new CntVIMSendBufferNode(rpBuffers,
											new CntVIMTempFileStream());
				rpBuffers->m_pTempFile->Close();
				if (!m_pStatus->
					  setStatus(rpBuffers->m_pTempFile->GetErrorCode(), pJob))
					return FALSE;
				aDesc.setFileName(*m_pLibrary,
								  rpBuffers->m_pTempFile->GetFileName());
			}

			ByteString aTheName(aName, getTextEncoding());
			xub_StrLen nNameLen = aTheName.Len();
			BOOL bSuccess
				= m_pStatus->
				      setVIMStatus(m_pLibrary->
								       fnSetMessageItem(rVIMMsg,
														VIMSEL_ATTACH, 0,
														VIMSEL_NATIVE,
														aTheName.
														    GetBufferAccess(),
														&aDesc),
								   pJob);
			aTheName.ReleaseBufferAccess(nNameLen);
			if (!bSuccess)
				return FALSE;
			break;
		}
	}

	// Handle possible MIME children:
	ULONG nCount = rMIMEMsg.GetChildCount();
	for (ULONG i = 0; i < nCount; ++i)
	{
		const INetCoreMIMEMessage * pSubMsg = rMIMEMsg.GetChild(i);
		if (pSubMsg
			&& !transformMessage(*pSubMsg, rVIMMsg, rpBuffers, rbAddNotePart,
								 pJob))
			return FALSE;
	}
	return TRUE;
}

//============================================================================
CntVIMMailbox::CntVIMMailbox()
{
	m_pStatus = new CntVIMStatus(*this);
	m_pLibrary = new CntVIMLibrary(*m_pStatus);
	m_pSession = new CntVIMPointer(*m_pLibrary);
}

//============================================================================
CntVIMMailbox::~CntVIMMailbox()
{
	if (isInitialized())
		m_pLibrary->fnCloseSession(*m_pSession);
	delete m_pSession;
	delete m_pLibrary;
	delete m_pStatus;
}

//============================================================================
CntVIMMailbox::InitStatus CntVIMMailbox::initialize(const String & rThePath,
													const String &
													 rTheUsername,
													const String &
													 rThePassword,
													CntNodeJob * pJob,
													BOOL bHandleLoginErrors)
{
	// Initialize the VIM library:
	if (!m_pLibrary->initialize(pJob))
		return OTHER_ERROR;

	// Save path, username, and password for later use (RestartSession() for
	// cc:Mail, cc:Mail bulletin boards):
	m_aPath = ByteString(rThePath, osl_getThreadTextEncoding());
	m_aUsername = ByteString(rTheUsername, osl_getThreadTextEncoding());
	m_aPassword = ByteString(rThePassword, osl_getThreadTextEncoding());

	// If no path specified, try to get a default one:
	if (m_aPath.Len() == 0)
	{
		sal_Char sDefaultPath[256];
		sal_Char sDummyUsername[1024];
		if (VIM_STATUS(m_pLibrary->
					    fnGetDefaultSessionInfo(sizeof sDefaultPath,
												sDefaultPath,
												sizeof sDummyUsername,
												sDummyUsername))
			 == VIMSTS_SUCCESS)
			m_aPath = sDefaultPath;
	}

	// Open a VIM session:
	sal_Char * sPathBuffer;
	if (m_aPath.Len() == 0)
		sPathBuffer = 0;
	else
	{
		sPathBuffer = new sal_Char[m_aPath.Len() + 1];
		strcpy(sPathBuffer, m_aPath.GetBuffer());
	}
	sal_Char * sUsernameBuffer = new sal_Char[m_aUsername.Len() + 1];
	strcpy(sUsernameBuffer, m_aUsername.GetBuffer());
	sal_Char * sPasswordBuffer = new sal_Char[m_aPassword.Len() + 1];
	strcpy(sPasswordBuffer, m_aPassword.GetBuffer());
	BOOL bSuccess =
	 m_pStatus->
	  setVIMStatus(m_pLibrary->fnOpenSession(sPathBuffer, sUsernameBuffer,
											 sPasswordBuffer,
											 CNT_VIM_VERSION_REQUIRED,
											 m_pLibrary->getCharSet(),
											 *m_pSession),
				   bHandleLoginErrors ? pJob : 0);
	delete[] sPathBuffer;
	delete[] sUsernameBuffer;
	delete[] sPasswordBuffer;
	if (bSuccess)
		if (m_pSession->isSet(*m_pLibrary))
			return OK;
		else if (bHandleLoginErrors)
			m_pStatus->setStatus(*new StringErrorInfo(
				                          ERRCODE_CHAOS_VIM_LIBRARY_CORRUPTED,
										  String::CreateFromAscii(
											  RTL_CONSTASCII_STRINGPARAM(
												  CNT_VIM_LIBRARY_RES))),
								 pJob);
		else
			m_pStatus->setStatus(*new StringErrorInfo(
				                          ERRCODE_CHAOS_VIM_LIBRARY_CORRUPTED,
										  String::CreateFromAscii(
											  RTL_CONSTASCII_STRINGPARAM(
												  CNT_VIM_LIBRARY_RES))),
								 0);
	else
		m_pSession->clear(*m_pLibrary);
	return LOGIN_ERROR;
}

//============================================================================
BOOL CntVIMMailbox::isInitialized() const
{
	return m_pSession->isSet(*m_pLibrary);
}

//============================================================================
rtl_TextEncoding CntVIMMailbox::getTextEncoding() const
{
	return m_pLibrary->getTextEncoding();
}

//============================================================================
BOOL CntVIMMailbox::isCCMail() const
{
	return m_pLibrary->isCCMail();
}

//============================================================================
BOOL CntVIMMailbox::restartSession(CntNodeJob * pJob)
{
	DBG_ASSERT(isInitialized(), "CntVIMMailbox::restartSession(): Not open");

	if (!m_pStatus->setVIMStatus(m_pLibrary->fnCloseSession(*m_pSession),
								 pJob))
		return FALSE;
	m_pSession->clear(*m_pLibrary);

	sal_Char * sPathBuffer;
	if (m_aPath.Len() == 0)
		sPathBuffer = 0;
	else
	{
		sPathBuffer = new sal_Char[m_aPath.Len() + 1];
		strcpy(sPathBuffer, m_aPath.GetBuffer());
	}
	sal_Char * sUsernameBuffer = new sal_Char[m_aUsername.Len() + 1];
	strcpy(sUsernameBuffer, m_aUsername.GetBuffer());
	sal_Char * sPasswordBuffer = new sal_Char[m_aPassword.Len() + 1];
	strcpy(sPasswordBuffer, m_aPassword.GetBuffer());
	BOOL bSuccess =
	 m_pStatus->
	  setVIMStatus(m_pLibrary->fnOpenSession(sPathBuffer, sUsernameBuffer,
											 sPasswordBuffer,
											 CNT_VIM_VERSION_REQUIRED,
											 m_pLibrary->getCharSet(),
											 *m_pSession),
				   pJob);
	delete[] sPathBuffer;
	delete[] sUsernameBuffer;
	delete[] sPasswordBuffer;
	if (bSuccess)
		if (m_pSession->isSet(*m_pLibrary))
			return TRUE;
		else
			m_pStatus->setStatus(*new StringErrorInfo(
				                          ERRCODE_CHAOS_VIM_LIBRARY_CORRUPTED,
										  String::CreateFromAscii(
											  RTL_CONSTASCII_STRINGPARAM(
												  CNT_VIM_LIBRARY_RES))),
								 pJob);
	return FALSE;
}

//============================================================================
BOOL CntVIMMailbox::sendMessage(const INetCoreNewsMessage & rMsg,
								CntNodeJob * pJob)
{
	DBG_ASSERT(isInitialized(), "CntVIMMailbox::sendMessage(): Not open");

	// Create a new VIM message:
	sal_Char sTypeBuffer[] = VIM_MAIL;
	CntVIMPointer aVIMMsg(*m_pLibrary);
	if (!m_pStatus->setVIMStatus(m_pLibrary->fnCreateMessage(*m_pSession,
															 sTypeBuffer,
															 aVIMMsg),
								 pJob))
		return FALSE;

	// Initialize clean up list:
	CntVIMSendBufferNode * pBuffers = 0;

	// Set the message subject:
	ByteString aSubject(rMsg.GetSubject(), getTextEncoding());
	sal_Char * sSubjectBuffer = new sal_Char[aSubject.Len() + 1];
	strcpy(sSubjectBuffer, aSubject.GetBuffer());
	BOOL bSuccess
	 = m_pStatus->
	    setVIMStatus(m_pLibrary->fnSetMessageHeader(aVIMMsg, VIMSEL_SUBJECT,
													aSubject.Len() + 1,
													sSubjectBuffer),
					 pJob);
	delete[] sSubjectBuffer;
	if (!bSuccess)
		goto cleanup;

	// Try to set the message priority (maybe not supported by all VIM
	// implementations):
	{
		String aPriorityStr(rMsg.GetXPriority());
		USHORT nVIMPriority = VIM_NORMAL_PRIORITY;
		if (aPriorityStr.Len())
			if (aPriorityStr.GetChar(0) == '1'
				|| aPriorityStr.GetChar(0) == '2')
				nVIMPriority = VIM_HIGH_PRIORITY;
			else if (aPriorityStr.GetChar(0) == '4'
					 || aPriorityStr.GetChar(0) == '5')
				nVIMPriority = VIM_LOW_PRIORITY;
		ULONG nVIMStatus
		 = m_pLibrary->fnSetMessageHeader(aVIMMsg, VIMSEL_PRIORITY,
										  sizeof nVIMPriority, &nVIMPriority);
		if (VIM_STATUS(nVIMStatus) != VIMSTS_SUCCESS
			&& VIM_STATUS(nVIMStatus) != VIMSTS_NOT_SUPPORTED)
		{
			m_pStatus->setVIMStatus(nVIMStatus, pJob);
			goto cleanup;
		}
	}

	// Try to set the request for a (non-)delivery report, and specify that in
	// the case of non-delivery reports, the original message is not included
	// (these options are not supported by all VIM implementations); also, set
	// the request for a return receipt:
	{
		USHORT nReport
		 = rMsg.GetReturnReceiptTo().Len() ? VIM_TRUE : VIM_FALSE;
		USHORT nReportBuffer = nReport;
		ULONG nVIMStatus
		 = m_pLibrary->fnSetMessageHeader(aVIMMsg, VIMSEL_DELIVERY_REPORT,
										  sizeof nReportBuffer,
										  &nReportBuffer);
		if (VIM_STATUS(nVIMStatus) != VIMSTS_SUCCESS
			&& VIM_STATUS(nVIMStatus) != VIMSTS_NOT_SUPPORTED)
		{
			m_pStatus->setVIMStatus(nVIMStatus, pJob);
			goto cleanup;
		}
		nReportBuffer = nReport;
		nVIMStatus
		 = m_pLibrary->fnSetMessageHeader(aVIMMsg, VIMSEL_NONDELIVERY_REPORT,
										  sizeof nReportBuffer,
										  &nReportBuffer);
		if (VIM_STATUS(nVIMStatus) != VIMSTS_SUCCESS
			&& VIM_STATUS(nVIMStatus) != VIMSTS_NOT_SUPPORTED)
		{
			m_pStatus->setVIMStatus(nVIMStatus, pJob);
			goto cleanup;
		}
		USHORT nContents = VIM_FALSE;
		nVIMStatus
		 = m_pLibrary->fnSetMessageHeader(aVIMMsg,
										  VIMSEL_NONDELIVERY_CONTENTS,
										  sizeof nContents, &nContents);
		if (VIM_STATUS(nVIMStatus) != VIMSTS_SUCCESS
			&& VIM_STATUS(nVIMStatus) != VIMSTS_NOT_SUPPORTED)
		{
			m_pStatus->setVIMStatus(nVIMStatus, pJob);
			goto cleanup;
		}
		nVIMStatus
		 = m_pLibrary->fnSetMessageHeader(aVIMMsg, VIMSEL_RETURN_RECEIPT,
										  sizeof nReport, &nReport);
		if (VIM_STATUS(nVIMStatus) != VIMSTS_SUCCESS
			&& VIM_STATUS(nVIMStatus) != VIMSTS_NOT_SUPPORTED)
		{
			m_pStatus->setVIMStatus(nVIMStatus, pJob);
			goto cleanup;
		}
	}

	// Try to specify not to save sent messages inside of VIM (maybe not
	// supported by all VIM implementations):
	{
		USHORT nSave = VIM_FALSE;
		ULONG nVIMStatus
		 = m_pLibrary->fnSetMessageHeader(aVIMMsg, VIMSEL_SAVE, sizeof nSave,
										  &nSave);
		if (VIM_STATUS(nVIMStatus) != VIMSTS_SUCCESS
			&& VIM_STATUS(nVIMStatus) != VIMSTS_NOT_SUPPORTED)
		{
			m_pStatus->setVIMStatus(nVIMStatus, pJob);
			goto cleanup;
		}
	}

	// Set the To (Newsgroup), CC, BCC recipients:
	{
		CntVIMRecipient aRecipient;
		aRecipient.Type = VIMSEL_UNKNOWN_RECIP_TYPE;
		aRecipient.DName.Type = VIMSEL_NATIVE;
		aRecipient.DName.AddressBook[0] = '\0';
		aRecipient.Address.Type = VIMSEL_NATIVE;
		aRecipient.Address.Value[0] = '\0';

		String aRecipientList(rMsg.GetTo());
		if (aRecipientList.Len())
		{
			USHORT nRecipients = aRecipientList.GetTokenCount(',');
			for (USHORT i = 0; i < nRecipients; ++i)
			{
				strncpy(aRecipient.DName.Value,
						ByteString(aRecipientList.GetToken(i, ','),
								   getTextEncoding()).
						    GetBuffer(),
						1023);
				aRecipient.DName.Value[1023] = '\0';
				if (!m_pStatus->
					  setVIMStatus(m_pLibrary->
								    fnSetMessageRecipient(aVIMMsg, VIMSEL_TO,
														  &aRecipient),
								   pJob))
					goto cleanup;
			}
		}

		aRecipientList = rMsg.GetNewsgroups();
		if (aRecipientList.Len())
		{
			USHORT nRecipients = aRecipientList.GetTokenCount(',');
			for (USHORT i = 0; i < nRecipients; ++i)
			{
				strncpy(aRecipient.DName.Value,
						ByteString(aRecipientList.GetToken(i, ','),
								   getTextEncoding()).
						    GetBuffer(),
						1023);
				aRecipient.DName.Value[1023] = '\0';
				if (!m_pStatus->
					  setVIMStatus(m_pLibrary->
								    fnSetMessageRecipient(aVIMMsg, VIMSEL_TO,
														  &aRecipient),
								   pJob))
					goto cleanup;
			}
		}

		aRecipientList = rMsg.GetCC();
		if (aRecipientList.Len())
		{
			USHORT nRecipients = aRecipientList.GetTokenCount(',');
			for (USHORT i = 0; i < nRecipients; ++i)
			{
				strncpy(aRecipient.DName.Value,
						ByteString(aRecipientList.GetToken(i, ','),
								   getTextEncoding()).
						    GetBuffer(),
						1023);
				aRecipient.DName.Value[1023] = '\0';
				if (!m_pStatus->
					  setVIMStatus(m_pLibrary->
								    fnSetMessageRecipient(aVIMMsg, VIMSEL_CC,
														  &aRecipient),
								   pJob))
					goto cleanup;
			}
		}

		aRecipientList = rMsg.GetBCC();
		if (aRecipientList.Len())
		{
			USHORT nRecipients = aRecipientList.GetTokenCount(',');
			for (USHORT i = 0; i < nRecipients; ++i)
			{
				strncpy(aRecipient.DName.Value,
						ByteString(aRecipientList.GetToken(i, ','),
								   getTextEncoding()).
						    GetBuffer(),
						1023);
				aRecipient.DName.Value[1023] = '\0';
				if (!m_pStatus->
					  setVIMStatus(m_pLibrary->
								    fnSetMessageRecipient(aVIMMsg, VIMSEL_BCC,
														  &aRecipient),
								   pJob))
					goto cleanup;
			}
		}
	}

	// Transform the hierarchical MIME message into one note part and
	// additional attachments:
	{
		BOOL bAddNotePart = TRUE;
		if (!transformMessage(rMsg, aVIMMsg, pBuffers, bAddNotePart, pJob))
			goto cleanup;
		if (bAddNotePart)
		{
			// If no note part was added, create an empty one:
			sal_Char sTypeBuffer[] = VIM_TEXT;
#if !defined OS2
			sal_Char sNameBuffer[] = "";
#endif // OS2
			pBuffers = new CntVIMSendBufferNode(pBuffers, new sal_Char);
			*pBuffers->m_pBuffer = ' ';
			CntVIMBFDescriptor aDesc;
			aDesc.setSize(*m_pLibrary, 1);
			aDesc.setBuffer(*m_pLibrary, pBuffers->m_pBuffer);
			if (!m_pStatus->
				  setVIMStatus(m_pLibrary->fnSetMessageItem(aVIMMsg,
															VIMSEL_NOTE_PART,
															sTypeBuffer, 0,
#if defined OS2
															0,
#else // OS2
															sNameBuffer,
#endif // OS2
															&aDesc),
							   pJob))
				goto cleanup;
		}
	}

	// Send the message:
	m_pStatus->setVIMStatus(m_pLibrary->fnSendMessage(aVIMMsg), pJob);

cleanup:
	if (!m_pStatus->isOK())
		m_pLibrary->fnCloseMessage(aVIMMsg);
	while (pBuffers)
	{
		CntVIMSendBufferNode * pNode = pBuffers;
		pBuffers = pNode->m_pNext;
		delete pNode;
	}
	return m_pStatus->isOK();
}

//============================================================================
CntVIMMessageContainer * CntVIMMailbox::getInbox()
{
	return new CntVIMInbox(*m_pStatus, *m_pLibrary, this);
}

//============================================================================
CntVIMMessageContainer * CntVIMMailbox::getBulletinBoard(const String & rName)
{
	return new CntVIMBulletinBoard(*m_pStatus, *m_pLibrary, this, rName);
}

//============================================================================
// static
CntFolderList * CntVIMMailbox::parseBBoardList(const String & rFilename,
											   const DirEntry & rPOPath)
{
	// Scan lines of the form
	//
	//   ows 'NAME' ows ':' ows 'BBOARD' ows
	//    '"' <value of chars >= ' ' and != '"'> '"' any-junk
	//
	// where ows means 'optional white space' (i.e., ' ' or '\t'), and 'NAME'
	// and 'BBOARD' can be in any combination of upper and lower case. Lines
	// are seperated by any non-empty sequence of '\n's and '\r's. The value
	// between '"' and '"' is the (case sensitive) name of a bboard.
	//
	// Lines of the form
	//
	//   ows '[' <value of chars >= ' '> ']' any-junk-not-containing-']'
	//
	// start named sections. There can be a unnamed section before the first
	// named section. Only those bboard lines in sections named with the
	// correct po path as value are considered (and there may be multiple
	// sections with the same po path as value). If there are no sections with
	// the correct po path as value, only the bboard lines in the unnamed
	// section (if any) are considered.
	//
	// Both bboard and po path values are only considered valid if they are
	// shorter than a given max len. Lines that do not match either of the
	// above are ignored. Multiple mentions of the same bboard are harmless.

	enum ScanerState { BEGIN_OF_LINE, SCAN_NA, SCAN_NAM, SCAN_NAME,
					   SCAN_COLON, SCAN_B, SCAN_BB, SCAN_BBO, SCAN_BBOA,
					   SCAN_BBOAR, SCAN_BBOARD, SCAN_QUOTE, SCAN_THE_NAME,
					   SCAN_POPATH, SKIP_TO_EOL };

	SvFileStream aStream(rFilename, STREAM_STD_READ);
	if (aStream.GetError())
		return 0;
	CntFolderList * pList = new CntFolderList();

	BOOL bScanBBoards = TRUE;
	BOOL bNamedSections = FALSE;

	const USHORT nMaxValueLen = 2000;
	 // max len of bboard names and po paths; can be changed
	ByteString aValue;
	xub_StrLen nLen = STRING_NOTFOUND;

	ScanerState eState = BEGIN_OF_LINE;
	for (;;)
	{
		sal_Char cChar;
		aStream >> cChar;
		if (aStream.IsEof())
			return pList;
		if (aStream.GetError())
		{
			delete pList;
			return 0;
		}

		switch (eState)
		{
			case BEGIN_OF_LINE:
				if (cChar == 'N' || cChar == 'n')
					eState = bScanBBoards ? SCAN_NA : SKIP_TO_EOL;
				else if (cChar == '[')
				{
					aValue.Erase();
					nLen = STRING_NOTFOUND;
					eState = SCAN_POPATH;
				}
				else if (cChar != '\n' && cChar != '\r' && cChar != '\t'
						 && cChar != ' ')
					eState = SKIP_TO_EOL;
				break;

			case SCAN_NA:
				eState
				 = cChar == 'A' || cChar == 'a' ? SCAN_NAM : SKIP_TO_EOL;
				break;

			case SCAN_NAM:
				eState
					= cChar == 'M' || cChar == 'm' ? SCAN_NAME : SKIP_TO_EOL;
				break;

			case SCAN_NAME:
				eState
					= cChar == 'E' || cChar == 'e' ? SCAN_COLON : SKIP_TO_EOL;
				break;

			case SCAN_COLON:
				if (cChar == ':')
					eState = SCAN_B;
				else if (cChar != '\t' && cChar != ' ')
					eState = SKIP_TO_EOL;
				break;

			case SCAN_B:
				if (cChar == 'B' || cChar == 'b')
					eState = SCAN_BB;
				else if (cChar != '\t' && cChar != ' ')
					eState = SKIP_TO_EOL;
				break;

			case SCAN_BB:
				eState
				 = cChar == 'B' || cChar == 'b' ? SCAN_BBO : SKIP_TO_EOL;
				break;

			case SCAN_BBO:
				eState
				 = cChar == 'O' || cChar == 'o' ? SCAN_BBOA : SKIP_TO_EOL;
				break;

			case SCAN_BBOA:
				eState
				 = cChar == 'A' || cChar == 'a' ? SCAN_BBOAR : SKIP_TO_EOL;
				break;

			case SCAN_BBOAR:
				eState
				 = cChar == 'R' || cChar == 'r' ? SCAN_BBOARD : SKIP_TO_EOL;
				break;

			case SCAN_BBOARD:
				eState
				 = cChar == 'D' || cChar == 'd' ? SCAN_QUOTE : SKIP_TO_EOL;
				break;

			case SCAN_QUOTE:
				if (cChar == '"')
				{
					aValue.Erase();
					eState = SCAN_THE_NAME;
				}
				else if (cChar != '\t' && cChar != ' ')
					eState = SKIP_TO_EOL;
				break;

			case SCAN_THE_NAME:
				if (cChar == '"')
					if (aValue.Len() == 0)
						eState = SKIP_TO_EOL;
					else
					{
						pList->
							Insert(new CntFolderEntry(
								           String(
											   aValue,
											   osl_getThreadTextEncoding())));
						eState = SKIP_TO_EOL;
					}
				else if (cChar == '\n' || cChar == '\r')
					eState = BEGIN_OF_LINE;
				else if (cChar >= '\0' && cChar < ' '
						 || aValue.Len() >= nMaxValueLen)
					eState = SKIP_TO_EOL;
				else
					aValue += cChar;
				break;

			case SCAN_POPATH:
				if (cChar == ']')
					nLen = aValue.Len();
				else if (cChar >= '\0' && cChar < ' '
						 || aValue.Len() >= nMaxValueLen)
					if (nLen == STRING_NOTFOUND)
						eState = cChar == '\n' || cChar == '\r' ?
							      BEGIN_OF_LINE : SKIP_TO_EOL;
					else
					{
						if (DirEntry(String(aValue.Copy(0, nLen),
											osl_getThreadTextEncoding()))
							    == rPOPath)
						{
							bScanBBoards = TRUE;
							if (!bNamedSections)
							{
								for (ULONG i = pList->Count(); i;)
									delete pList->Remove(--i);
								bNamedSections = TRUE;
							}
						}
						else
							bScanBBoards = FALSE;
						eState = BEGIN_OF_LINE;
					}
				else
					aValue += cChar;
				break;

			case SKIP_TO_EOL:
				if (cChar == '\n' || cChar == '\r')
					eState = BEGIN_OF_LINE;
				break;
		}
	}
}

//============================================================================
//
//  class CntVIMTempFileStream
//
//============================================================================

// virtual
CntVIMTempFileStream::~CntVIMTempFileStream()
{
	Close();
	String aFilename(GetFileName());
	DirEntry(aFilename).Kill();
}

//============================================================================
//
//  class CntVIMLibrary
//
//============================================================================

CntVIMLibrary::~CntVIMLibrary()
{
	if (isInitialized())
		fnTerminate();
}

//============================================================================
BOOL CntVIMLibrary::initialize(CntNodeJob * pJob)
{
	if (m_eInitState == NONE)
	{
		// Load VIM module:
		if (m_aModule.
			    load(rtl::OUString::createFromAscii(CNT_VIM_LIBRARY_NAME)))
		{
#if defined CNT_VIM_ALT_LINKAGE
			m_bAltLinkage = FALSE;
#endif // CNT_VIM_ALT_LINKAGE
		}
#if defined CNT_VIM_ALT_LINKAGE
		else if (m_aModule.load(rtl::OUString::createFromAscii(
			                        CNT_VIM_ALT_LIBRARY_NAME)))
			m_bAltLinkage = TRUE;
#endif // CNT_VIM_ALT_LINKAGE
		else
			goto fail1;

		// Load exported functions:
		m_pFnCloseMessage = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_CLOSEMESSAGE));
		if (!m_pFnCloseMessage)
			goto fail2;
		m_pFnCloseMessageContainer
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_CLOSEMESSAGECONTAINER));
		if (!m_pFnCloseMessageContainer)
			goto fail2;
		m_pFnCloseSession = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_CLOSESESSION));
		if (!m_pFnCloseSession)
			goto fail2;
		m_pFnCreateMessage = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_CREATEMESSAGE));
		if (!m_pFnCreateMessage)
			goto fail2;
		m_pFnEnumerateMessageHeaderAttrs
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_ENUMERATEMESSAGEHEADERATTRS));
		if (!m_pFnEnumerateMessageHeaderAttrs)
			goto fail2;
		m_pFnEnumerateMessageItems
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_ENUMERATEMESSAGEITEMS));
		if (!m_pFnEnumerateMessageItems)
			goto fail2;
		m_pFnEnumerateMessageRecipients
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_ENUMERATEMESSAGERECIPIENTS));
		if (!m_pFnEnumerateMessageRecipients)
			goto fail2;
		m_pFnEnumerateMessages
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_ENUMERATEMESSAGES));
		if (!m_pFnEnumerateMessages)
			goto fail2;
		m_pFnGetDefaultSessionInfo
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_GETDEFAULTSESSIONINFO));
		if (!m_pFnGetDefaultSessionInfo)
			goto fail2;
		m_pFnGetMessageHeader
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_GETMESSAGEHEADER));
		if (!m_pFnGetMessageHeader)
			goto fail2;
		m_pFnGetMessageItem = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_GETMESSAGEITEM));
		if (!m_pFnGetMessageItem)
			goto fail2;
		m_pFnInitialize = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_INITIALIZE));
		if (!m_pFnInitialize)
			goto fail2;
		m_pFnMarkMessageAsRead
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_MARKMESSAGEASREAD));
		if (!m_pFnMarkMessageAsRead)
			goto fail2;
		m_pFnOpenMessage = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_OPENMESSAGE));
		if (!m_pFnOpenMessage)
			goto fail2;
		m_pFnOpenMessageContainer
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_OPENMESSAGECONTAINER));
		if (!m_pFnOpenMessageContainer)
			goto fail2;
		m_pFnOpenSession = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_OPENSESSION));
		if (!m_pFnOpenSession)
			goto fail2;
		m_pFnQueryCapability
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_QUERYCAPABILITY));
		if (!m_pFnQueryCapability)
			goto fail2;
		m_pFnQueryNewMessages
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_QUERYNEWMESSAGES));
		if (!m_pFnQueryNewMessages)
			goto fail2;
		m_pFnRemoveMessage = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_REMOVEMESSAGE));
		if (!m_pFnRemoveMessage)
			goto fail2;
		m_pFnSendMessage = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_SENDMESSAGE));
		if (!m_pFnSendMessage)
			goto fail2;
		m_pFnSetMessageHeader
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_SETMESSAGEHEADER));
		if (!m_pFnSetMessageHeader)
			goto fail2;
		m_pFnSetMessageItem = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_SETMESSAGEITEM));
		if (!m_pFnSetMessageItem)
			goto fail2;
		m_pFnSetMessageRecipient
		 = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_SETMESSAGERECIPIENT));
		if (!m_pFnSetMessageRecipient)
			goto fail2;
		m_pFnStatusText = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_STATUSTEXT));
		if (!m_pFnStatusText)
			goto fail2;
		m_pFnTerminate = m_aModule.getSymbol(rtl::OUString::createFromAscii(CNT_VIM_FN_TERMINATE));
		if (!m_pFnTerminate)
			goto fail2;

		// Initialize library:
		if (!m_rStatus.setVIMStatus(fnInitialize(), pJob))
			goto fail3;
		m_eInitState = INITIALIZED;

		// Find a suitable character set that is supported by the library:
		const ULONG anCharSetList[] = { CNT_VIM_CHARSETS, 0 };
		for (const ULONG * pnSelector = anCharSetList;; ++pnSelector)
		{
			if (!*pnSelector)
				goto fail5;
			USHORT nSupported;
			if (!m_rStatus.setVIMStatus(fnQueryCapability(*pnSelector,
														  sizeof nSupported,
														  &nSupported),
										pJob))
				goto fail4;
			if (nSupported)
			{
				m_nCharSet = *pnSelector;
				break;
			}
		}

		// Find out if using a cc:Mail library:
		sal_Char sProduct[16];
		if (!m_rStatus.setVIMStatus(fnQueryCapability(VIMSEL_PRODUCT,
													  sizeof sProduct,
													  sProduct),
									pJob))
			goto fail4;
		m_bCCMail = !strcmp(sProduct, "CCMAIL");

		// The cc:Mail VIM DLL implementation version 6.02 breaks when
		// using VIMGetMessageItem with a memory buffer:
		if (m_bCCMail)
		{
			ULONG nVersion;
			if (!m_rStatus.setVIMStatus
				            (fnQueryCapability(VIMSEL_IMPLEMENTATION_VERSION,
											   sizeof nVersion, &nVersion),
							 pJob))
				goto fail4;
			m_bGetItemThroughFile = nVersion == 602;
		}
		else
			m_bGetItemThroughFile = FALSE;

		// Query max. subject length:
		if (!m_rStatus.setVIMStatus(fnQueryCapability(VIMSEL_MAX_SUBJECT_LEN,
													  sizeof m_nMaxSubjectLen,
													  &m_nMaxSubjectLen),
									pJob))
			goto fail4;
		if (m_nMaxSubjectLen == ULONG(-1))
			m_nMaxSubjectLen = 256;

		m_eInitState = DONE;
	}
	return TRUE;

 fail2:
	m_aModule.unload();
 fail1:
	m_rStatus.setStatus(*new StringErrorInfo(ERRCODE_CHAOS_NO_VIM_LIBRARY,
											 String::CreateFromAscii(
												 RTL_CONSTASCII_STRINGPARAM(
													 CNT_VIM_LIBRARY_RES))),
						pJob);
	return FALSE;

 fail5:
	m_rStatus.setStatus(*new StringErrorInfo(ERRCODE_CHAOS_NO_VIM_LIBRARY,
											 String::CreateFromAscii(
												 RTL_CONSTASCII_STRINGPARAM(
													 CNT_VIM_LIBRARY_RES))),
						pJob);
 fail4:
	fnTerminate();
 fail3:
	m_aModule.unload();
	return FALSE;
}

//============================================================================
rtl_TextEncoding CntVIMLibrary::getTextEncoding() const
{
	switch (m_nCharSet)
	{
		case VIMSEL_CP850:
			return RTL_TEXTENCODING_IBM_850;

		case VIMSEL_CP437:
			return RTL_TEXTENCODING_IBM_437;

		case VIMSEL_CP1252:
			return RTL_TEXTENCODING_MS_1252;

		default:
			DBG_ERROR("CntVIMLibrary::getTextEncoding(): Bad charset");
			return osl_getThreadTextEncoding();
	}
}

//============================================================================
//
//  class CntVIMMessageFileItem
//
//============================================================================

// virtual
String CntVIMMessageFileItem::getContentType() const
{
	return INetContentTypes::GetContentType
	                          (INetContentTypes::GetContentType4Extension
							                      (DirEntry(m_aFilename).
												    GetExtension()));
}

//============================================================================
//
//  class CntVIMInboxMessageBody
//
//============================================================================

// virtual
CntVIMInboxMessageBody::~CntVIMInboxMessageBody()
{
	getLibrary().fnCloseMessage(m_aMsg);
}

//============================================================================
// virtual
BOOL CntVIMInboxMessageBody::startItemIteration(ULONG & rnItems,
												CntNodeJob * pJob)
{
	rnItems = 0;
	const USHORT nSimultItems = 16; // can be changed
	CntVIMItemDesc aDescs[nSimultItems];
	CntVIMEnumRef aItemsPos;
	VIM_UND_REF(aItemsPos);
	USHORT nMoreItems;
	do
	{
		USHORT nCount = nSimultItems;
		if (!getStatus().
			  setVIMStatus(getLibrary().
						    fnEnumerateMessageItems(m_aMsg, &aItemsPos, 1,
													&nCount, aDescs,
													VIMSEL_NO_FILTER, 0,
													&nMoreItems),
						   pJob))
			return FALSE;

		for (USHORT i = 0; i < nCount; ++i)
			if (aDescs[i].Class == VIMSEL_NOTE_PART
				&& (ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_TEXT)).
					    EqualsIgnoreCaseAscii(aDescs[i].Type)
					|| ByteString(RTL_CONSTASCII_STRINGPARAM(
						              VIM_UNWRAPPED_TEXT)).
					       EqualsIgnoreCaseAscii(aDescs[i].Type)
					|| ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_RTF)).
					       EqualsIgnoreCaseAscii(aDescs[i].Type))
				|| aDescs[i].Class == VIMSEL_ATTACH
				|| aDescs[i].Class == VIMSEL_APP_DEFINED
				   && ByteString(RTL_CONSTASCII_STRINGPARAM(
					                 "NOTES_TYPE_TEXT_LIST")).
				          EqualsIgnoreCaseAscii(aDescs[i].Type)
				   && ByteString(RTL_CONSTASCII_STRINGPARAM(
					                 "IntendedRecipient")).
				          Equals(aDescs[i].Name))
				++rnItems;
	}
	while (nMoreItems);

	VIM_UND_REF(m_aPos);
	m_nMore = VIM_TRUE;
	return TRUE;
}

//============================================================================
// virtual
CntVIMMessageItem * CntVIMInboxMessageBody::getNextItem(CntNodeJob * pJob)
{
	while (m_nMore)
	{
		CntVIMItemDesc aDesc;
		USHORT nCount = 1;
		if (!getStatus().
			  setVIMStatus(getLibrary().
						    fnEnumerateMessageItems(m_aMsg, &m_aPos, 1,
													&nCount, &aDesc,
													VIMSEL_NO_FILTER, 0,
													&m_nMore),
						   pJob)
			|| !nCount)
			return 0;

		switch (aDesc.Class)
		{
			case VIMSEL_NOTE_PART:
			{
				enum PartType { TYPE_TEXT, TYPE_RTF, TYPE_OTHER };
				PartType eType;
				if (ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_TEXT)).
					    EqualsIgnoreCaseAscii(aDesc.Type)
					|| ByteString(RTL_CONSTASCII_STRINGPARAM(
						              VIM_UNWRAPPED_TEXT)).
					       EqualsIgnoreCaseAscii(aDesc.Type))
					eType = TYPE_TEXT;
				else if (ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_RTF)).
						     EqualsIgnoreCaseAscii(aDesc.Type))
					eType = TYPE_RTF;
				else
					eType = TYPE_OTHER;
				if (eType <= TYPE_RTF)
				{
					const ULONG nMaxMemoryItemSize = 16384; // can be changed
					const ULONG nBufferIncrement = 1024; // can be changed
					SvStream * pStream;
					if (aDesc.Size < nMaxMemoryItemSize
						&& !getLibrary().getItemThroughFile())
					{
						// Don't trust aDesc.Size, try a growing buffer
						// instead (and if the buffer grows too large, use a
						// file):
						CntVIMBFDescriptor aBuffDesc;
						ULONG nVIMStatus;
						for (aBuffDesc.setSize(getLibrary(), aDesc.Size);;)
						{
							aBuffDesc.
								setBuffer(
									getLibrary(),
									new sal_Char[aBuffDesc.
												     getSize(getLibrary())]);
							nVIMStatus
							 = getLibrary().fnGetMessageItem(m_aMsg,
															 aDesc.Ref,
															 aDesc.Type, 0,
															 &aBuffDesc);
							if (VIM_STATUS(nVIMStatus)
								 != VIMSTS_BUF_TOO_SMALL)
								break;
							delete[] aBuffDesc.getBuffer(getLibrary());
							aBuffDesc.setSize(getLibrary(),
											  aBuffDesc.getSize(getLibrary())
											   + nBufferIncrement);
							if (aBuffDesc.getSize(getLibrary())
								 > nMaxMemoryItemSize)
								goto use_file;
						}
						if (!getStatus().setVIMStatus(nVIMStatus, pJob))
						{
							delete[] aBuffDesc.getBuffer(getLibrary());
							return 0;
						}
						SvMemoryStream * pTheStream
						 = new SvMemoryStream(STATIC_CAST(sal_Char *,
														  aBuffDesc.
														   getBuffer
														    (getLibrary())),
											  aBuffDesc.getSize(getLibrary()),
											  STREAM_READ);
						pTheStream->ObjectOwnsMemory(TRUE);
						pStream = pTheStream;
					}
					else
					{
					use_file:
						// Save the item into a temporary file:
						DirEntry
							aTempFile(String::CreateFromAscii(
								                   RTL_CONSTASCII_STRINGPARAM(
								  					   "vim*.tmp")));
						aTempFile = aTempFile.TempName();
						String aTempName(aTempFile.GetFull());
						CntVIMBFDescriptor aFileDesc;
						aFileDesc.setFileName(getLibrary(), aTempName);
						if (!getStatus().
							  setVIMStatus(getLibrary().
										    fnGetMessageItem(m_aMsg,
															 aDesc.Ref,
															 aDesc.Type, 0,
															 &aFileDesc),
										   pJob))
						{
							aTempFile.Kill();
							return 0;
						}
						CntVIMTempFileStream * pTheStream
						 = new CntVIMTempFileStream(aTempName);
						if (!getStatus().setStatus(pTheStream->GetErrorCode(),
												   pJob))
						{
							delete pTheStream;
							return 0;
						}
						pStream = pTheStream;
					}

					if (eType == TYPE_RTF)
						return new CntVIMMessageRTFItem(
							           String(aDesc.Name,
											  getLibrary().getTextEncoding()),
									   pStream);
					else
						return new CntVIMMessageTextItem(
							           String(aDesc.Name,
											  getLibrary().getTextEncoding()),
									   pStream, getLibrary().getCharSet());
				}
				break;
			}

			case VIMSEL_ATTACH:
			{
				DirEntry
					aTempFile(String::CreateFromAscii(
						                   RTL_CONSTASCII_STRINGPARAM(
						  					   "vim*.tmp")));
				aTempFile = aTempFile.TempName();
				String aTempName(aTempFile.GetFull());
				CntVIMBFDescriptor aFileDesc;
				aFileDesc.setFileName(getLibrary(), aTempName);
				if (!getStatus().
					  setVIMStatus(getLibrary().
								    fnGetMessageItem(m_aMsg, aDesc.Ref,
													 aDesc.Type[0] == '\0' ?
													  0 : aDesc.Type,
													 VIMSEL_NATIVE,
													 &aFileDesc),
								   pJob))
				{
					aTempFile.Kill();
					return 0;
				}
				CntVIMTempFileStream * pStream
				 = new CntVIMTempFileStream(aTempName);
				if (!getStatus().setStatus(pStream->GetErrorCode(), pJob))
				{
					delete pStream;
					return 0;
				}
				return
					new CntVIMMessageFileItem(String(aDesc.Name,
													 getLibrary().
													     getTextEncoding()),
											  pStream);
			}

			case VIMSEL_APP_DEFINED:
			{
				// For Lotus Notes, turn the names of intended recipients of
				// (non-)delivery reports into text items:
				if (ByteString(RTL_CONSTASCII_STRINGPARAM(
					               "NOTES_TYPE_TEXT_LIST")).
					    EqualsIgnoreCaseAscii(aDesc.Type)
					&& ByteString(RTL_CONSTASCII_STRINGPARAM(
						              "IntendedRecipient")).
					       Equals(aDesc.Name))
				{
					CntVIMBFDescriptor aBuffDesc;
					aBuffDesc.setSize(getLibrary(), aDesc.Size);
					sal_Char * sBuffer
						= new sal_Char[aBuffDesc.getSize(getLibrary())];
					aBuffDesc.setBuffer(getLibrary(), sBuffer);
					if (!getStatus().
						  setVIMStatus(getLibrary().
									    fnGetMessageItem(m_aMsg, aDesc.Ref, 0,
														 0, &aBuffDesc),
									   pJob))
					{
					error:
						delete[] sBuffer;
						return
							new CntVIMMessageTextItem(
								    CntResId(RID_PROT_INTENDREC_TITLE),
									new CntVIMStringStream(
										    ByteString(
												CntResId(
												  RID_PROT_INTENDREC_UNKNOWN),
												getLibrary().
												    getTextEncoding())));
					}
					if (aBuffDesc.getSize(getLibrary()) < 4
						|| sBuffer[0] != '\x01' || sBuffer[1] != '\0')
						goto error;
					ULONG nSize
						= ULONG(sal_uChar(sBuffer[2]));
//						      + (ULONG(sal_uChar(sBuffer[3])) << 8) + 4;
// Changed by OBO, 5.6.2000: Above line can't be parsed by linux compiler
					ULONG nDummy = ULONG(sal_uChar(sBuffer[3]));
					nDummy <<= 8;
					nDummy += 4;
					nSize += nDummy;
//End of changes
					if (nSize != aBuffDesc.getSize(getLibrary()))
						goto error;

					// Strip the initial 'KEYWORD=':
					ULONG i;
					for (i = 4;;)
						if (i == nSize)
							goto error;
						else if (sBuffer[i++] == '=')
							break;

					// The recipient string obviously follows X.500, so try to
					// make it more readable by replacing any substring of the
					// form '/KEYWORD=' by a single '/', and by changing the
					// character set from CP850 (which is used for this text
					// even if another character set is chosen for
					// VIMOpenSession---at least under Windows NT):
					ByteString aReadable;
					for (; i < nSize; ++i)
					{
						aReadable += sBuffer[i];
						if (sBuffer[i] == '/')
							do
								if (++i == nSize)
									goto error;
							while (sBuffer[i] != '=');
					}
					delete[] sBuffer;

					String aText(CntResId(RID_PROT_INTENDREC_TEXT));
					aText.SearchAndReplaceAscii(
						      "%1",
							  String(aReadable, RTL_TEXTENCODING_IBM_850));
					return
						new CntVIMMessageTextItem(
							    CntResId(RID_PROT_INTENDREC_TITLE),
								new CntVIMStringStream(
									    ByteString(aText,
												   getLibrary().
												       getTextEncoding())));
				}
				break;
			}
		}
	}

	return 0;
}

//============================================================================
//
//  class CntVIMInbox
//
//============================================================================

String CntVIMInbox::createFakedUniqueID(sal_Char const * pFrom,
										CntVIMDate const & rDate,
										CntNodeJob * pJob)
{
	String aUniqueID;
	if (*pFrom != '\0' && rDate.Date != VIM_UNKNOWN_DATE
		&& rDate.Time != VIM_UNKNOWN_DATE)
	{
		aUniqueID = INetURLObject::encode(pFrom, INetURLObject::PART_VIM,
										  '=', INetURLObject::ENCODE_ALL);
		aUniqueID += ':';
		aUniqueID += rDate.Zone;
		aUniqueID += '.';
		aUniqueID += rDate.Date;
		aUniqueID += '.';
		aUniqueID += rDate.Time;
	}
	else
		getStatus().setStatus(ERRCODE_CHAOS_VIM_NO_FAKE_MESSAGE_ID, pJob);
	return aUniqueID;
}

//============================================================================
BOOL CntVIMInbox::getRecipients(CntVIMPointer & rMsg, ULONG aType,
								String & rRecipients, CntNodeJob * pJob)
{
	const USHORT nSimultNames = 16;
	 // the number of recipient names processed at once; can be changed
	sal_Char aNames[nSimultNames][1024];
	CntVIMAttrDescriptor aAttrib;
	aAttrib.setAttr(getLibrary(), VIMSEL_NAME);
	aAttrib.setSize(getLibrary(), sizeof aNames[0]);
	aAttrib.setBuffer(getLibrary(), aNames);
	CntVIMEnumRef aNamesPos;
	VIM_UND_REF(aNamesPos);
	USHORT bMoreNames;
	do
	{
		USHORT nCount = nSimultNames;
		if (!getStatus().
			  setVIMStatus(getLibrary().
						    fnEnumerateMessageRecipients(rMsg, aType,
														 &aNamesPos, 1, 1,
														 &aAttrib, &nCount,
														 &bMoreNames),
						   pJob))
			return FALSE;
		if (nCount == 0)
			break;
		for (USHORT i = 0; i < nCount; ++i)
		{
			if (rRecipients.Len() > 0)
				rRecipients.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
			rRecipients += String(aNames[i], getLibrary().getTextEncoding());
		}
	}
	while (bMoreNames);
	return TRUE;
}

//============================================================================
// virtual
CntVIMInbox::~CntVIMInbox()
{
	if (aContainer.isSet(getLibrary()))
		getLibrary().fnCloseMessageContainer(aContainer);
}

//============================================================================
// virtual
BOOL CntVIMInbox::open(CntNodeJob * pJob)
{
	if (aContainer.isSet(getLibrary()))
		return TRUE;

	if (getStatus().
		 setVIMStatus(getLibrary().fnOpenMessageContainer(pMailbox->
														   getSession(),
														  0, aContainer),
					  pJob))
		if (aContainer.isSet(getLibrary()))
		{
			bJustOpened = TRUE;
			return TRUE;
		}
		else
			getStatus().
				setStatus(*new StringErrorInfo(
					               ERRCODE_CHAOS_VIM_LIBRARY_CORRUPTED,
								   String::CreateFromAscii(
									   RTL_CONSTASCII_STRINGPARAM(
										   CNT_VIM_LIBRARY_RES))),
						  pJob);
	else
		aContainer.clear(getLibrary());
	return FALSE;
}

//============================================================================
// virtual
BOOL CntVIMInbox::queryNewMessages(BOOL bTryHard, CntNodeJob * pJob)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::queryNewMessages(): Not open");

	// If the message container has only just been opened, it has to be
	// iterated through to find out if there are any new messages:
	if (bJustOpened)
	{
		bJustOpened = FALSE;
		return TRUE;
	}

	// If VIM has to be terminated to find out about new messages (which seems
	// to be true for cc:Mail), and if trying hard, terminate now and start
	// over again:
	if (getLibrary().isCCMail())
		if (bTryHard)
			return getStatus().
			        setVIMStatus(getLibrary().
								  fnCloseMessageContainer(aContainer),
								 pJob)
				   && pMailbox->restartSession(pJob)
				   && getStatus().
			           setVIMStatus(getLibrary().
									 fnOpenMessageContainer(pMailbox->
															 getSession(),
															0, aContainer),
									pJob);
		else
		{
			getStatus().setStatus(ERRCODE_NONE, pJob);
			return FALSE;
		}

	USHORT bNewMsgs;
	if (!getStatus().
		  setVIMStatus(getLibrary().fnQueryNewMessages(aContainer, &bNewMsgs),
					   pJob))
		return FALSE;
	getStatus().setStatus(ERRCODE_NONE, pJob);
	return bNewMsgs == VIM_TRUE;
}

//============================================================================
// virtual
BOOL CntVIMInbox::canIterateFromEnd(CntNodeJob * pJob)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::canIterateFromEnd(): Not open");

	if (eDirection == UNKNOWN)
	{
		// Check if iterating from end produces no error:
		CntVIMEnumRef aDummyPos;
		VIM_UND_REF(aDummyPos);
		CntVIMRef aDummyRefID;
		CntVIMAttrDescriptor aDummyAttrib;
		aDummyAttrib.setAttr(getLibrary(), VIMSEL_REF);
		aDummyAttrib.setSize(getLibrary(), sizeof aDummyRefID);
		aDummyAttrib.setBuffer(getLibrary(), &aDummyRefID);
		USHORT nDummyMsgs = 1;
		USHORT bDummyMore;
		ULONG nVIMStatus
		 = getLibrary().fnEnumerateMessages(aContainer, &aDummyPos, -1, 1,
											&aDummyAttrib, &nDummyMsgs,
											VIMSEL_NO_FILTER, 0, 0,
											&bDummyMore);
		if (VIM_STATUS(nVIMStatus) == VIMSTS_NOT_SUPPORTED)
		{
			getStatus().setStatus(ERRCODE_NONE, pJob);
			eDirection = START;
		}
		else if (getStatus().setVIMStatus(nVIMStatus, pJob))
			eDirection = END;
	}

	return eDirection == END;
}

//============================================================================
// virtual
CntVIMMessageContainer::ExecuteState
 CntVIMInbox::startHeaderIteration(UINT32, UINT32, CntNodeJob *)
{
	DBG_ERROR("CntVIMInbox::startHeaderIteration(): Bad method");
	return EXECUTE_ERROR;
}

//============================================================================
// virtual
CntVIMMessageContainer::ExecuteState
CntVIMInbox::startHeaderIteration(BOOL bFromEnd)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::startHeaderIteration(): Not open");

	VIM_UND_REF(aPos);
	nSkip = bFromEnd ? - 1 : 1;
	bMore = VIM_TRUE;

	return EXECUTE_DONE;
}

//============================================================================
// virtual
CntVIMMessageHeader * CntVIMInbox::getNextHeader(CntNodeJob * pJob,
												 ULONG aFilterType,
												 const void * pFilterData,
												 ULONG nFilterDataLen)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::getNextHeader(): Not open");

	if (!bMore)
	{
		getStatus().setStatus(ERRCODE_NONE, pJob);
		return 0;
	}

	// Get the next VIM message header:
	CntVIMRef aRefID;
	sal_Char sType[32];
	USHORT bUnread;
	sal_Char sFromName[1024];
	USHORT nPriority;
	ULONG nMaxSubjectLen = getLibrary().getMaxSubjectLen();
	sal_Char * sSubjectBuf = new sal_Char[nMaxSubjectLen];
	CntVIMDate aVIMDate;
	const USHORT nAttribCount = 7;
	CntVIMAttrDescriptor aAttribs[nAttribCount];
	aAttribs[0].setAttr(getLibrary(), VIMSEL_REF);
	aAttribs[0].setSize(getLibrary(), sizeof aRefID);
	aAttribs[0].setBuffer(getLibrary(), &aRefID);
	aAttribs[1].setAttr(getLibrary(), VIMSEL_TYPE);
	aAttribs[1].setSize(getLibrary(), sizeof sType);
	aAttribs[1].setBuffer(getLibrary(), sType);
	aAttribs[2].setAttr(getLibrary(), VIMSEL_UNREAD_MAIL);
	aAttribs[2].setSize(getLibrary(), sizeof bUnread);
	aAttribs[2].setBuffer(getLibrary(), &bUnread);
	aAttribs[3].setAttr(getLibrary(), VIMSEL_FROM_NAME);
	aAttribs[3].setSize(getLibrary(), sizeof sFromName);
	aAttribs[3].setBuffer(getLibrary(), sFromName);
	aAttribs[4].setAttr(getLibrary(), VIMSEL_PRIORITY);
	aAttribs[4].setSize(getLibrary(), sizeof nPriority);
	aAttribs[4].setBuffer(getLibrary(), &nPriority);
	aAttribs[5].setAttr(getLibrary(), VIMSEL_SUBJECT);
	aAttribs[5].setSize(getLibrary(), nMaxSubjectLen);
	aAttribs[5].setBuffer(getLibrary(), sSubjectBuf);
	aAttribs[6].setAttr(getLibrary(), VIMSEL_DATE);
	aAttribs[6].setSize(getLibrary(), sizeof aVIMDate);
	aAttribs[6].setBuffer(getLibrary(), &aVIMDate);
	USHORT nCount = 1;
	sal_Char * pFilterDataBuffer = new sal_Char[nFilterDataLen];
	rtl_copyMemory(pFilterDataBuffer, pFilterData, nFilterDataLen);
	BOOL bSuccess
	 = getStatus().
	    setVIMStatus(getLibrary().fnEnumerateMessages(aContainer, &aPos,
													  nSkip, nAttribCount,
													  aAttribs, &nCount,
													  aFilterType,
													  pFilterDataBuffer, 0,
													  &bMore),
					 pJob);
	delete[] pFilterDataBuffer;
	if (!bSuccess || !nCount)
	{
		delete[] sSubjectBuf;
		return 0;
	}

	// Turn the subject into a string:
	String aSubject;
	if (ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_RTRC)).
		    EqualsIgnoreCaseAscii(sType))
		aSubject = CntResId(RID_PROT_RETURN_RECEIPT);
	else if (ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_DLR)).
			     EqualsIgnoreCaseAscii(sType))
		aSubject = CntResId(RID_PROT_DELIVERY_REPORT);
	else if (ByteString(RTL_CONSTASCII_STRINGPARAM(VIM_NDLR)).
			     EqualsIgnoreCaseAscii(sType))
		aSubject = CntResId(RID_PROT_NONDELIVERY_REPORT);
	aSubject += String(sSubjectBuf, getLibrary().getTextEncoding());
	delete[] sSubjectBuf;

	// Translate VIM message date into our internal format:
	BOOL bDatePresent;
	DateTime aDate;
	if (aVIMDate.Date == VIM_UNKNOWN_DATE)
		bDatePresent = FALSE;
	else
	{
		bDatePresent = TRUE;
		aDate.SetDay(STATIC_CAST(USHORT, aVIMDate.Date & 0xFF));
		aDate.SetMonth(STATIC_CAST(USHORT, aVIMDate.Date >> 8 & 0xFF));
		aDate.SetYear(STATIC_CAST(USHORT, aVIMDate.Date >> 16 & 0xFFFF));
		if (aVIMDate.Time == VIM_UNKNOWN_TIME)
			aDate.SetTime(0);
		else
		{

			aDate.SetHour(STATIC_CAST(USHORT, aVIMDate.Time >> 24 & 0xFF));
			aDate.SetMin(STATIC_CAST(USHORT, aVIMDate.Time >> 16 & 0xFF));
			aDate.SetSec(STATIC_CAST(USHORT, aVIMDate.Time >> 8 & 0xFF));
			aDate.Set100Sec(STATIC_CAST(USHORT, aVIMDate.Time & 0xFF));
			if (aVIMDate.Zone == VIMSEL_GMT)
				aDate.ConvertToLocalTime();
		}
	}

	// Open the VIM message (to get the unique ID, recipients and keywords):
	CntVIMPointer aMsg(getLibrary());
	if (!getStatus().setVIMStatus(getLibrary().fnOpenMessage(aContainer,
															 aRefID, 0, aMsg),
								  pJob))
		return 0;

	// Try to get the unique ID (not supported by cc:Mail, for which a 'fake'
	// ID of the form '(quoted from):(date integer)' is used):
	String aUniqueID;
	if (getLibrary().isCCMail())
	{
		aUniqueID = createFakedUniqueID(sFromName, aVIMDate, pJob);
		if (aUniqueID.Len() == 0)
		{
			getLibrary().fnCloseMessage(aMsg);
			return 0;
		}
	}
	else
	{
		sal_Char sMsgID[33];
		CntVIMAttrDescriptor aIDAttrib;
		aIDAttrib.setAttr(getLibrary(), VIMSEL_UNIQUE_MSG_ID);
		aIDAttrib.setSize(getLibrary(), sizeof sMsgID);
		aIDAttrib.setBuffer(getLibrary(), sMsgID);
		if (!getStatus().
			  setVIMStatus(getLibrary().fnGetMessageHeader(aMsg, 1,
														   &aIDAttrib),
						   pJob))
		{
			getLibrary().fnCloseMessage(aMsg);
			return 0;
		}
		aUniqueID = INetURLObject::encode(sMsgID, INetURLObject::PART_VIM,
										  '=', INetURLObject::ENCODE_ALL);
	}

	// Get the 'to', 'CC', and 'BCC' recipients:
	String aTo;
	String aCC;
	String aBCC;
	if (!(getRecipients(aMsg, VIMSEL_TO, aTo, pJob)
		  && getRecipients(aMsg, VIMSEL_CC, aCC, pJob)
		  && getRecipients(aMsg, VIMSEL_BCC, aBCC, pJob)))
	{
		getLibrary().fnCloseMessage(aMsg);
		return 0;
	}

	// Get any keywords:
	const USHORT nSimultKeywords = 16;
	 // the number of keywords processed at once; can be changed
	sal_Char aKeywordBuf[nSimultKeywords][64];
	CntVIMEnumRef aPosKeywords;
	VIM_UND_REF(aPosKeywords);
	USHORT bMoreKeywords;
	String aKeywords;
	do
	{
		USHORT nCount = nSimultKeywords;
		if (!getStatus().
			  setVIMStatus
			   (getLibrary().
				 fnEnumerateMessageHeaderAttrs(aMsg, VIMSEL_KEYWORD,
											   &aPosKeywords, 1,
											   sizeof aKeywordBuf[0],
											   aKeywordBuf[0], &nCount,
											   &bMoreKeywords),
				pJob))
		{
			getLibrary().fnCloseMessage(aMsg);
			return 0;
		}
		if (!nCount)
			break;
		for (USHORT i = 0; i < nCount; ++i)
		{
			if (aKeywords.Len() != 0)
				aKeywords.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
			aKeywords
				+= String(aKeywordBuf[i], getLibrary().getTextEncoding());
		}
	}
	while (bMoreKeywords);

	// Return a new message header object:
	return new CntVIMMessageHeader(aRefID.Ref[0], aRefID.Ref[1], aUniqueID,
								   bUnread != VIM_FALSE,
								   sFromName[0] == '\0' ?
								       String(CntResId(RID_PROT_VIM_MAILER)) :
								       String(sFromName,
											  getLibrary().getTextEncoding()),
								   nPriority == VIM_HIGH_PRIORITY ?
								       CNT_PRIO_HIGH :
								   nPriority == VIM_LOW_PRIORITY ?
								       CNT_PRIO_LOW : CNT_PRIO_NORMAL,
								   aSubject, bDatePresent, aDate, aTo, aCC,
								   aBCC, aKeywords);
}

//============================================================================
// virtual
BOOL CntVIMInbox::findMessage(const String & rUniqueID, UINT32 & rRefIDPart1,
							  UINT32 & rRefIDPart2, CntNodeJob * pJob,
							  BOOL bNotFoundIsOK)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::findMessage(): Not open");

	if (getLibrary().isCCMail())
	{
		// The rUniqueID is a 'fake' ID of the form
		// '(quoted from):(date integer)', so search the message container for
		// a matching pair:
		USHORT nColon = rUniqueID.Search(':');
		if (nColon == STRING_NOTFOUND)
		{
			getStatus().setStatus(ERRCODE_CHAOS_ILLEGAL_MESSAGE_ID, pJob);
			return FALSE;
		}
		String aFromName(INetURLObject::decode(
			                 rUniqueID.Copy(0, nColon), '=',
							 INetURLObject::DECODE_WITH_CHARSET));
		CntVIMDate aDate;
		String
			aDateString(rUniqueID, nColon + 1, rUniqueID.Len() - nColon - 1);
		if (aDateString.GetTokenCount('.') != 3)
		{
			getStatus().setStatus(ERRCODE_CHAOS_ILLEGAL_MESSAGE_ID, pJob);
			return FALSE;
		}
		String aNumber(aDateString.GetToken(0, '.'));
		sal_Unicode const * p = aNumber.GetBuffer();
		sal_Unicode const * pEnd = p + aNumber.Len();
		sal_uInt32 nValue;
		if (!INetMIME::scanUnsigned(p, pEnd, true, nValue) || p != pEnd)
		{
			getStatus().setStatus(ERRCODE_CHAOS_ILLEGAL_MESSAGE_ID, pJob);
			return FALSE;
		}
		aDate.Zone = nValue;
		aNumber = aDateString.GetToken(1, '.');
		p = aNumber.GetBuffer();
		pEnd = p + aNumber.Len();
		if (!INetMIME::scanUnsigned(p, pEnd, true, nValue) || p != pEnd)
		{
			getStatus().setStatus(ERRCODE_CHAOS_ILLEGAL_MESSAGE_ID, pJob);
			return FALSE;
		}
		aDate.Date = nValue;
		aNumber = aDateString.GetToken(2, '.');
		p = aNumber.GetBuffer();
		pEnd = p + aNumber.Len();
		if (!INetMIME::scanUnsigned(p, pEnd, true, nValue) || p != pEnd)
		{
			getStatus().setStatus(ERRCODE_CHAOS_ILLEGAL_MESSAGE_ID, pJob);
			return FALSE;
		}
		aDate.Time = nValue;
		const USHORT nSimultMsgs = 256;
		 // the number of messages processed at once; can be changed
		CntVIMRef aRefIDs[nSimultMsgs];
		CntVIMDate aDates[nSimultMsgs];
		const USHORT nAttribCount = 2;
		CntVIMAttrDescriptor aAttribs[nAttribCount];
		aAttribs[0].setAttr(getLibrary(), VIMSEL_REF);
		aAttribs[0].setSize(getLibrary(), sizeof aRefIDs[0]);
		aAttribs[0].setBuffer(getLibrary(), aRefIDs);
		aAttribs[1].setAttr(getLibrary(), VIMSEL_DATE);
		aAttribs[1].setSize(getLibrary(), sizeof aDates[0]);
		aAttribs[1].setBuffer(getLibrary(), aDates);
		CntVIMEnumRef aPos;
		VIM_UND_REF(aPos);
		long nSkip = canIterateFromEnd(pJob) ? -1 : 1;
		 // the used heuristic is that in most cases messages from the end of
		 // the list are processed
		USHORT bMore;
		do
		{
			USHORT nCount = nSimultMsgs;
			ByteString
				aTheFromName(aFromName, getLibrary().getTextEncoding());
			xub_StrLen nFromNameLen = aTheFromName.Len();
			// The following function call tries to filter out only those
			// messages that have the specific 'from' name, but this need not
			// work in all VIM implementations (it does work, though, in the
			// case of cc:Mail, which is---hopefully---the only implementation
			// that uses 'fake' message IDs anyway):
			BOOL bSuccess
				= getStatus().
			          setVIMStatus(getLibrary().
								       fnEnumerateMessages(
										   aContainer, &aPos, nSkip,
										   nAttribCount, aAttribs, &nCount,
										   VIMSEL_FROM,
										   aTheFromName.GetBufferAccess(), 0,
										   &bMore),
								   pJob);
			aTheFromName.ReleaseBufferAccess(nFromNameLen);
			if (!bSuccess)
				return FALSE;

			// Compare the dates for each message:
			for (USHORT i = 0; i < nCount; ++i)
				if (aDate.Zone == aDates[i].Zone
					&& aDate.Date == aDates[i].Date
					&& aDate.Time == aDates[i].Time)
				{
					rRefIDPart1 = aRefIDs[i].Ref[0];
					rRefIDPart2 = aRefIDs[i].Ref[1];
					return TRUE;
				}
		}
		while (bMore);
	}
	else
	{
		// The unique ID is indeed a (quoted) VIM unique message ID, so search
		// all messages in the container for this ID:
		String aVIMUniqueID(INetURLObject::decode(
			                    rUniqueID, '=',
								INetURLObject::DECODE_WITH_CHARSET));
		const USHORT nSimultMsgs = 256;
		 // the number of messages processed at once; can be changed
		CntVIMRef aRefIDs[nSimultMsgs];
		CntVIMAttrDescriptor aRefIDAttrib;
		aRefIDAttrib.setAttr(getLibrary(), VIMSEL_REF);
		aRefIDAttrib.setSize(getLibrary(), sizeof aRefIDs[0]);
		aRefIDAttrib.setBuffer(getLibrary(), aRefIDs);
		CntVIMEnumRef aPos;
		VIM_UND_REF(aPos);
		long nSkip = canIterateFromEnd(pJob) ? -1 : 1;
		 // the used heuristic is that in most cases messages from the end of
		 // the list are processed
		USHORT bMore;
		do
		{
			USHORT nCount = nSimultMsgs;
			if (!getStatus().
				  setVIMStatus(getLibrary().
							    fnEnumerateMessages(aContainer, &aPos, nSkip,
													1, &aRefIDAttrib, &nCount,
													VIMSEL_NO_FILTER, 0, 0,
													&bMore),
							   pJob))
				return FALSE;

			// Open each message to compare the unique IDs:
			for (USHORT i = 0; i < nCount; ++i)
			{
				CntVIMPointer aMsg(getLibrary());
				if (!getStatus().
					  setVIMStatus(getLibrary().fnOpenMessage(aContainer,
															  aRefIDs[i], 0,
															  aMsg),
								   pJob))
					return FALSE;

				sal_Char sMsgUniqueID[33];
				CntVIMAttrDescriptor aUniqueIDAttrib;
				aUniqueIDAttrib.setAttr(getLibrary(), VIMSEL_UNIQUE_MSG_ID);
				aUniqueIDAttrib.setSize(getLibrary(), sizeof sMsgUniqueID);
				aUniqueIDAttrib.setBuffer(getLibrary(), sMsgUniqueID);
				if (!getStatus().
					  setVIMStatus(getLibrary().
								    fnGetMessageHeader(aMsg, 1,
													   &aUniqueIDAttrib),
								   pJob))
				{
					getLibrary().fnCloseMessage(aMsg);
					return FALSE;
				}

				getLibrary().fnCloseMessage(aMsg);

				if (rUniqueID == String(sMsgUniqueID,
										getLibrary().getTextEncoding()))
				{
					rRefIDPart1 = aRefIDs[i].Ref[0];
					rRefIDPart2 = aRefIDs[i].Ref[1];
					return TRUE;
				}
			}
		}
		while (bMore);
	}

	getStatus().setStatus(bNotFoundIsOK ?
						   ERRCODE_NONE : ERRCODE_CHAOS_MESSAGE_NOT_FOUND,
						  pJob);
	return FALSE;
}

//============================================================================
// virtual
CntVIMMessageContainer::ExecuteState
CntVIMInbox::openMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
						 CntNodeJob * pJob, CntVIMMessageBody *& rpBody)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::openMessage(): Not open");

	rpBody = 0;

	CntVIMRef aRefID = { { nRefIDPart1, nRefIDPart2 } };
	CntVIMPointer aMsg(getLibrary());
	if (!getStatus().setVIMStatus(getLibrary().fnOpenMessage(aContainer,
															 aRefID, 0, aMsg),
								  pJob))
		return EXECUTE_ERROR;

	rpBody = new CntVIMInboxMessageBody(getStatus(), getLibrary(), aMsg);
	return EXECUTE_DONE;
}

//============================================================================
// virtual
BOOL CntVIMInbox::notifyMessageRead(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
									CntNodeJob * pJob)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::notifyMessageRead(): Not open");

	CntVIMRef aRefID = { { nRefIDPart1, nRefIDPart2 } };
	return
	 getStatus().setVIMStatus(getLibrary().fnMarkMessageAsRead(aContainer,
															   aRefID),
							  pJob);
}

//============================================================================
// virtual
BOOL CntVIMInbox::deleteMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
								CntNodeJob * pJob)
{
	DBG_ASSERT(aContainer.isSet(getLibrary()),
			   "CntVIMInbox::deleteMessage(): Not open");

	CntVIMRef aRefID = { { nRefIDPart1, nRefIDPart2 } };
	return getStatus().setVIMStatus(getLibrary().fnRemoveMessage(aContainer,
																 aRefID),
									pJob);
}

//============================================================================
//
//  class CntVIMBBoardMessageBody
//
//============================================================================

// virtual
CntVIMBBoardMessageBody::~CntVIMBBoardMessageBody()
{
	for (CntVIMBBoardMessageBodyItemList * pItem = m_pHead; pItem;)
	{
		CntVIMBBoardMessageBodyItemList * pNext = pItem->getNext();
		delete pItem;
		pItem = pNext;
	}
	delete m_pStream;
}

//============================================================================
// (1) Skip all lines that do not start an item:
//
//      (BEGIN_OF_LINE SCAN_KEY+ SKIP_TO_CR* SCAN_NONMATCHED_LF)+
//
// (2) Scan a line consisting of any of the keywords 'text item:', 'file
//     item:', 'fax item:', or 'graphics item:' in any combination of upper-/
//     lowercase, one optional space, an optional title, and a CR/LF pair:
//
//      BEGIN_OF_LINE SCAN_KEY+ SCAN_KEY_SPACE SCAN_TITLE+ SCAN_TITLE_LF
//
// (3) Scan a line consisting of the item size, any number of spaces, and a
//     CR/LF pair.
//
//      SCAN_SIZE_FIRST SCAN_SIZE_REST+ SCAN_SIZE_SPACE* SCAN_SIZE_LF
//
// (4) Record and skip an item of the stated size.
//
// (5) If not at end of file, go back to step (2).
//
// virtual
BOOL CntVIMBBoardMessageBody::startItemIteration(ULONG & rnItems,
												 CntNodeJob * pJob)
{
	rnItems = 0;

	enum ScanerState { BEGIN_OF_LINE, SCAN_KEY, SCAN_KEY_SPACE, SCAN_TITLE,
					   SCAN_TITLE_LF, SCAN_SIZE_FIRST, SCAN_SIZE_REST,
					   SCAN_SIZE_SPACE, SCAN_SIZE_LF, SKIP_TO_CR,
					   SCAN_NONMATCHED_LF };
	static ByteString const
	aKeyStrings[CntVIMBBoardMessageBodyItemList::NUM_ITEM_KEYWORDS]
		= { ByteString(RTL_CONSTASCII_STRINGPARAM("text item")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("file item")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("fax item")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("graphics item")) };
	const int nMaxKeyLen = 13;
	 // the maximum len of any of the item keywords to match

	sal_Char sKey[nMaxKeyLen + 1];
	int nKeyLen;
	CntVIMBBoardMessageBodyItemList::ItemKeyword eMatchedKey;
	String aTitle;
	ULONG nSize;
	BOOL bSkipHeader = TRUE;
	ScanerState eState = BEGIN_OF_LINE;
	for (;;)
	{
	next:
		sal_Char cChar;
		*m_pStream >> cChar;
		if (m_pStream->IsEof())
			if (eState == BEGIN_OF_LINE)
				return TRUE;
			else
				goto error;
		if (!getStatus().setStatus(m_pStream->GetErrorCode(), pJob))
			return FALSE;

		switch (eState)
		{
			case BEGIN_OF_LINE:
				if (cChar == '\r')
				{
					eState = SCAN_NONMATCHED_LF;
					break;
				}
				nKeyLen = 0;
				eState = SCAN_KEY;
			case SCAN_KEY:
				if (cChar == ' ' || cChar >= 'a' && cChar <= 'z')
				{
					if (nKeyLen == nMaxKeyLen)
						if (bSkipHeader)
							eState = SKIP_TO_CR;
						else
							goto error;
					else
						sKey[nKeyLen++] = cChar;
					break;
				}
				else if (cChar >= 'A' && cChar <= 'Z')
				{
					if (nKeyLen == nMaxKeyLen)
						if (bSkipHeader)
							eState = SKIP_TO_CR;
						else
							goto error;
					else
						sKey[nKeyLen++] = cChar + ('a' - 'A');
					break;
				}
				else if (cChar == ':')
					if (nKeyLen == 0)
						goto error;
					else
					{
						sKey[nKeyLen] = '\0';
						for (eMatchedKey
							  = CntVIMBBoardMessageBodyItemList::
								 FIRST_ITEM_KEYWORD;
							 eMatchedKey
							  < CntVIMBBoardMessageBodyItemList::
								 NUM_ITEM_KEYWORDS;
							 eMatchedKey
							  = CntVIMBBoardMessageBodyItemList::ItemKeyword
								 (eMatchedKey + 1))
							if (aKeyStrings[eMatchedKey].Equals(sKey))
							{
								bSkipHeader = FALSE;
								eState = SCAN_KEY_SPACE;
								goto next;
							}
					}
				if (bSkipHeader)
					eState = cChar == '\r' ? SCAN_NONMATCHED_LF : SKIP_TO_CR;
				else
					goto error;
				break;

			case SCAN_KEY_SPACE:
				aTitle.Erase();
				eState = SCAN_TITLE;
				if (cChar == ' ')
					break;
			case SCAN_TITLE:
				if (cChar == '\r')
					eState = SCAN_TITLE_LF;
				else
					aTitle += cChar;
				break;

			case SCAN_TITLE_LF:
				if (cChar != '\n')
					goto error;
				eState = SCAN_SIZE_FIRST;
				break;

			case SCAN_SIZE_FIRST:
				if (cChar < '0' || cChar > '9')
					goto error;
				nSize = cChar - '0';
				eState = SCAN_SIZE_REST;
				break;

			case SCAN_SIZE_REST:
				if (cChar >= '0' && cChar <= '9')
					nSize = 10 * nSize + (cChar - '0');
				else if (cChar == '\r')
					eState = SCAN_SIZE_LF;
				else if (cChar == ' ')
					eState = SCAN_SIZE_SPACE;
				else
					goto error;
				break;

			case SCAN_SIZE_SPACE:
				if (cChar == '\r')
					eState = SCAN_SIZE_LF;
				else if (cChar != ' ')
					goto error;
				break;

			case SCAN_SIZE_LF:
				if (cChar != '\n')
					goto error;
				else
				{
					++rnItems;
					CntVIMBBoardMessageBodyItemList * pItem
					 = new CntVIMBBoardMessageBodyItemList(eMatchedKey,
														   aTitle,
														   m_pStream->Tell(),
														   nSize);
					m_pTail = m_pHead ?
					           m_pTail->append(pItem) : (m_pHead = pItem);
					m_pStream->SeekRel(nSize);
					eState = BEGIN_OF_LINE;
				}
				break;

			case SKIP_TO_CR:
				if (cChar == '\r')
					eState = SCAN_NONMATCHED_LF;
				break;

			case SCAN_NONMATCHED_LF:
				if (cChar != '\n')
					goto error;
				eState = BEGIN_OF_LINE;
				break;
		}
	}

error:
	getStatus().setStatus(*new StringErrorInfo(
		                           ERRCODE_CHAOS_ILLEGAL_CCMAIL_EXPORT_FILE,
								   String::CreateFromAscii(
									   RTL_CONSTASCII_STRINGPARAM(
										   CNT_VIM_EXPORT_PROGRAM))),
						  pJob);
	return FALSE;
}

//============================================================================
// virtual
CntVIMMessageItem * CntVIMBBoardMessageBody::getNextItem(CntNodeJob * pJob)
{
	if (!m_pHead)
	{
		getStatus().setStatus(ERRCODE_NONE, pJob);
		return 0;
	}

	CntVIMBBoardMessageBodyItemList * pItem = m_pHead;
	m_pHead = pItem->getNext();

	switch (pItem->getType())
	{
		case CntVIMBBoardMessageBodyItemList::TEXT_ITEM:
		{
			ULONG nSize = pItem->getSize();
			sal_Char * sBuffer;
			if (nSize == 0)
				sBuffer = 0;
			else
			{
				sBuffer = new sal_Char[nSize];
				m_pStream->Seek(pItem->getStart());
				m_pStream->Read(sBuffer, nSize);
				if (!getStatus().setStatus(m_pStream->GetErrorCode(), pJob))
				{
					delete[] sBuffer;
					return 0;
				}
			}
			SvMemoryStream * pItemStream = new SvMemoryStream(sBuffer, nSize,
															  STREAM_READ);
			pItemStream->ObjectOwnsMemory(TRUE);
			return new CntVIMMessageTextItem(pItem->getTitle(), pItemStream,
#if defined WNT
											 CntVIMMessageTextItem::CP437
#elif defined OS2
											 CntVIMMessageTextItem::CP850
#else // WNT, OS2
											 CntVIMMessageTextItem::ISO88591
#endif // WNT, OS2
											 );
		}

		case CntVIMBBoardMessageBodyItemList::FILE_ITEM:
		case CntVIMBBoardMessageBodyItemList::FAX_ITEM:
		{
			// Calculate the number of bytes by removing any CR/LF pair after
			// 80 columns from the size, and then half the size, for two
			// digits make up one byte:
			ULONG nSize = pItem->getSize();
			nSize = nSize / 2
				     - (nSize % 82 == 0 ? nSize / 82 : nSize / 82 + 1);

			if (nSize == 0)
				return new CntVIMMessageFileItem(pItem->getTitle(),
												 new SvMemoryStream
												      (0, 0, STREAM_READ));

			// Uncode the hex digit pairs into a temp file stream:
			m_pStream->Seek(pItem->getStart());
			CntVIMTempFileStream * pItemStream = new CntVIMTempFileStream();
			if (!getStatus().setStatus(pItemStream->GetErrorCode(), pJob))
				goto io_error;
			{ // wrap nWritten for old compilers...
				for (ULONG nWritten = 0; nWritten < nSize;)
				{
					sal_Char cChar;
					*m_pStream >> cChar;
					if (m_pStream->IsEof())
						goto format_error;
					if (!getStatus().setStatus(m_pStream->GetErrorCode(),
											   pJob))
						goto io_error;
					if (cChar == '\r')
					{
						*m_pStream >> cChar;
						if (m_pStream->IsEof())
							goto format_error;
						if (!getStatus().setStatus(m_pStream->GetErrorCode(),
												   pJob))
							goto io_error;
						if (cChar != '\n')
							goto format_error;
					}
					else
					{
						sal_Char cByte;
						if (cChar >= '0' && cChar <= '9')
							cByte = (cChar - '0') << 4;
						else if (cChar >= 'A' && cChar <= 'F')
							cByte = ((cChar - 'A') + 10) << 4;
						else if (cChar >= 'a' && cChar <= 'f')
							cByte = ((cChar - 'a') + 10) << 4;
						else
							goto format_error;
						*m_pStream >> cChar;
						if (m_pStream->IsEof())
							goto format_error;
						if (!getStatus().setStatus(m_pStream->GetErrorCode(),
												   pJob))
							goto io_error;
						if (cChar >= '0' && cChar <= '9')
							cByte |= cChar - '0';
						else if (cChar >= 'A' && cChar <= 'F')
							cByte |= (cChar - 'A') + 10;
						else if (cChar >= 'a' && cChar <= 'f')
							cByte |= (cChar - 'a') + 10;
						else
							goto format_error;
						*pItemStream << cByte;
						if (!getStatus().setStatus(pItemStream->
												    GetErrorCode(),
												   pJob))
							goto io_error;
						++nWritten;
					}
				}
			}
			pItemStream->Seek(STREAM_SEEK_TO_BEGIN);
			return new CntVIMMessageFileItem(pItem->getTitle(), pItemStream);

		format_error:
			getStatus().
				setStatus(*new StringErrorInfo(
					               ERRCODE_CHAOS_ILLEGAL_CCMAIL_EXPORT_FILE,
								   String::CreateFromAscii(
									   RTL_CONSTASCII_STRINGPARAM(
										   CNT_VIM_EXPORT_PROGRAM))),
						  pJob);
		io_error:
			delete pItemStream;
			return 0;
		}

		default: // GRAPHICS_ITEM
			if (!pItem->getSize())
				return new CntVIMMessageTextItem(
					           pItem->getTitle(),
							   new CntVIMStringStream(
								       ByteString(
										   CntResId(
											   RID_PROT_CCMAIL_GRAPHICS_ITEM),
										   getLibrary().getTextEncoding())));

			getStatus().
				setStatus(*new StringErrorInfo(
					               ERRCODE_CHAOS_ILLEGAL_CCMAIL_EXPORT_FILE,
								   String::CreateFromAscii(
									   RTL_CONSTASCII_STRINGPARAM(
										   CNT_VIM_EXPORT_PROGRAM))),
						  pJob);
			return 0;
	}
}

//============================================================================
//
//  class CntVIMBulletinBoard
//
//============================================================================

// virtual
CntVIMBulletinBoard::~CntVIMBulletinBoard()
{
	cancelExecution();
	delete m_pHeaderStream;
}

//============================================================================
// virtual
CntVIMMessageContainer::ExecuteState
CntVIMBulletinBoard::startHeaderIteration(UINT32 nRefIDPart1,
										  UINT32 nRefIDPart2,
										  CntNodeJob * pJob)
{
	DBG_ASSERT(nRefIDPart1 == SEEK_BEGIN && nRefIDPart2 == SEEK_BEGIN
			   || nRefIDPart2 == 0,
			   "CntVIMBulletinBoard::startHeaderIteration(): Bad ID");

	delete m_pHeaderStream;
	m_pHeaderStream = 0;

	if (m_pProcess)
	{
		vos::OProcess::TProcessInfo aInfo;
		if (m_pProcess->getInfo(vos::OProcess::TData_ExitCode, &aInfo)
			    != vos::OProcess::E_None
			|| !(aInfo.Fields & vos::OProcess::TData_ExitCode)
#if defined WNT
			|| aInfo.Code == 259 // STILL_ACTIVE
#endif // WNT
		   )
		{
			ULONG nTicks = Time::GetSystemTicks();
			if (nTicks - m_nProcessWaitingCount > EXPORT_MAX_WAIT)
			{
				cancelExecution();
				getStatus().
					setStatus(
						*new StringErrorInfo(
							     ERRCODE_CHAOS_CCMAIL_EXPORT_NOT_TERMINATING,
								 String::CreateFromAscii(
									 RTL_CONSTASCII_STRINGPARAM(
										 CNT_VIM_EXPORT_PROGRAM))),
						pJob);
				return EXECUTE_ERROR;
			}
			return EXECUTE_WAITING;
		}
		delete m_pProcess;
		m_pProcess = 0;
	}
	else
	{
		// Check password for double quote character ("):
		String aPassword(m_pMailbox->getPassword(),
						 getLibrary().getTextEncoding());
		if (aPassword.Search('"') != STRING_NOTFOUND)
		{
			getStatus().
				setStatus(*new StringErrorInfo(
					               ERRCODE_CHAOS_BAD_CCMAIL_EXPORT_PASSWORD,
								   String::CreateFromAscii(
									   RTL_CONSTASCII_STRINGPARAM(
										   CNT_VIM_EXPORT_PROGRAM))),
						  pJob);
			return EXECUTE_ERROR;
		}

		// Call export to get a file with all headers:
		DirEntry
			aTempFile(String::CreateFromAscii(
				                   RTL_CONSTASCII_STRINGPARAM(
				  					   "vim*.tmp")));
		aTempFile = aTempFile.TempName();
		m_aProcessTempFileName = aTempFile.GetFull();
		String aFileArg('@');
		aFileArg += m_aProcessTempFileName;
		String aBBoardArg(RTL_CONSTASCII_USTRINGPARAM("BBOARD/\""));
		aBBoardArg += m_aName;
		aBBoardArg += '"';
		m_pProcess = new vos::OProcess(rtl::OUString::createFromAscii(
			                               CNT_VIM_EXPORT_PROGRAM));
		rtl::OUString aArguments[7];
		aArguments[0] = String(m_pMailbox->getUsername(),
							   getLibrary().getTextEncoding());
		aArguments[1] = aPassword;
		aArguments[2]
			= String(m_pMailbox->getPath(), getLibrary().getTextEncoding());
		aArguments[3] = aFileArg;
		aArguments[4] = rtl::OUString::createFromAscii("NAME/1");
		aArguments[5] = rtl::OUString::createFromAscii("HEADINGS/ALL");
		aArguments[6] = aBBoardArg;
		if (m_pProcess->execute(vos::OProcess::TProcessOption(
			                        vos::OProcess::TOption_SearchPath
									    | vos::OProcess::TOption_Hidden),
								vos::OArgumentList(aArguments, 7))
			    == vos::OProcess::E_None)
		{
			m_nProcessWaitingCount = Time::GetSystemTicks();
			return EXECUTE_WAITING;
		}

		m_nProcessWaitingCount = 0;
		cancelExecution(FALSE);
		getStatus().
		 setStatus
		  (*new StringErrorInfo(
#if defined WNT || defined OS2
			                    RTL_CONSTASCII_LENGTH(CNT_VIM_EXPORT_PROGRAM)
								   + aArguments[0].getLength()
								   + aArguments[1].getLength()
								   + aArguments[2].getLength()
								   + aArguments[3].getLength()
								   + aArguments[4].getLength()
								   + aArguments[5].getLength()
								   + aArguments[6].getLength()
								  > 100 ?
								 ERRCODE_CHAOS_CCMAIL_EXPORT_TOO_LONG :
#endif // WNT, OS2
								 ERRCODE_CHAOS_CCMAIL_EXPORT_ERROR,
								String::CreateFromAscii(
									RTL_CONSTASCII_STRINGPARAM(
										CNT_VIM_EXPORT_PROGRAM))),
		   pJob);
		return EXECUTE_ERROR;
	}

	m_pHeaderStream = new CntVIMTempFileStream(m_aProcessTempFileName);
	if (m_pHeaderStream->GetError())
	{
		getStatus().
		 setStatus(*new
				     TwoStringErrorInfo(ERRCODE_CHAOS_NO_CCMAIL_EXPORT_FILE,
										m_aProcessTempFileName,
										String::CreateFromAscii(
											RTL_CONSTASCII_STRINGPARAM(
												CNT_VIM_EXPORT_PROGRAM))),
				   pJob);
		m_aProcessTempFileName.Erase();
		delete m_pHeaderStream;
		m_pHeaderStream = 0;
		return EXECUTE_ERROR;
	}
	m_aProcessTempFileName.Erase();

	if (nRefIDPart2 == SEEK_BEGIN)
	{
		m_bJustOpened = FALSE;
		m_bEOF = FALSE;
		m_eNextField = FIELD_KEY_MESSAGE;
		return EXECUTE_DONE;
	}

	// Scan the stream at high speed for a line that consists of the keyword
	// 'msg#:' in any combination of upper-/lowercase, one optional space, a
	// decimal number representation matching nRefIDPart1, and a CR/LF pair:
	enum ScanerState { BEGIN_OF_LINE, SCAN_KEY, SCAN_SPACE, SCAN_NUMBER_FIRST,
					   SCAN_NUMBER_REST, SCAN_MATCHED_LF, SKIP_TO_CR,
					   SCAN_NONMATCHED_LF };
	const sal_Char * pKey;
	ULONG nParsedNumber;
	ScanerState eState = BEGIN_OF_LINE;
	for (;;)
	{
		sal_Char cChar;
		*m_pHeaderStream >> cChar;
		if (m_pHeaderStream->IsEof())
			if (eState == BEGIN_OF_LINE)
			{
				delete m_pHeaderStream;
				m_pHeaderStream = 0;
				m_bJustOpened = FALSE;
				m_bEOF = TRUE;
				m_eNextField = FIELD_KEY_MESSAGE;
				return EXECUTE_DONE;
			}
			else
				goto format_error;
		if (!getStatus().setStatus(m_pHeaderStream->GetErrorCode(), pJob))
			goto io_error;

		switch (eState)
		{
			case BEGIN_OF_LINE:
				pKey = "msg#:";
				eState = SCAN_KEY;
			case SCAN_KEY:
				if (cChar == *pKey
					|| cChar >= 'A' && cChar <= 'Z'
					   && cChar + ('a' - 'A') == *pKey)
				{
					if (*++pKey == '\0')
					  eState = SCAN_SPACE;
				}
				else
					eState = cChar == '\r' ? SCAN_NONMATCHED_LF : SKIP_TO_CR;
				break;

			case SCAN_SPACE:
				if (cChar == ' ')
				{
					eState = SCAN_NUMBER_FIRST;
					break;
				}
			case SCAN_NUMBER_FIRST:
				if (cChar < '0' || cChar > '9')
					goto format_error;
				nParsedNumber = cChar - '0';
				eState = SCAN_NUMBER_REST;
				break;

			case SCAN_NUMBER_REST:
				if (cChar == '\r')
					eState = SCAN_MATCHED_LF;
				else if (nParsedNumber == 0 || nParsedNumber >= 100000000
						 || cChar < '0' || cChar > '9')
					goto format_error;
				else
					nParsedNumber = nParsedNumber * 10 + (cChar - '0');
				break;

			case SCAN_MATCHED_LF:
				if (cChar != '\n')
					goto format_error;
				if (nParsedNumber > nRefIDPart1)
				{
					m_bJustOpened = FALSE;
					m_bEOF = FALSE;
					m_eNextField = ANY_FIELD_KEYWORD;
					m_nCurrentRefID = nParsedNumber;
					return EXECUTE_DONE;
				}
				eState = BEGIN_OF_LINE;
				break;

			case SKIP_TO_CR:
				if (cChar == '\r')
					eState = SCAN_NONMATCHED_LF;
				break;

			case SCAN_NONMATCHED_LF:
				if (cChar != '\n')
					goto format_error;
				eState = BEGIN_OF_LINE;
				break;
		}
	}

format_error:
	getStatus().
	 setStatus(*new StringErrorInfo(ERRCODE_CHAOS_ILLEGAL_CCMAIL_EXPORT_FILE,
									String::CreateFromAscii(
										RTL_CONSTASCII_STRINGPARAM(
											CNT_VIM_EXPORT_PROGRAM))),
			   pJob);
io_error:
	delete m_pHeaderStream;
	m_pHeaderStream = 0;
	return EXECUTE_ERROR;
}

//============================================================================
// virtual
CntVIMMessageContainer::ExecuteState
 CntVIMBulletinBoard::startHeaderIteration(BOOL)
{
	DBG_ERROR("CntVIMBulletinBoard::startHeaderIteration(): Bad method");
	return EXECUTE_ERROR;
}

//============================================================================
// virtual
CntVIMMessageHeader * CntVIMBulletinBoard::getNextHeader(CntNodeJob * pJob,
														 ULONG aFilterType,
														 const void *
														  pFilterData,
														 ULONG nFilterDataLen)
{
	DBG_ASSERT(aFilterType == VIMSEL_NO_FILTER && !pFilterData
			   && !nFilterDataLen,
			   "CntVIMBulletinBoard::getNextHeader(): Bad filter");

	if (m_bEOF)
	{
		getStatus().setStatus(ERRCODE_NONE, pJob);
		return 0;
	}

	DBG_ASSERT(m_pHeaderStream,
			   "CntVIMBulletinBoard::getNextHeader(): Not open");

	enum ScanerState { BEGIN_OF_LINE, SCAN_KEY, SCAN_SPACE, SCAN_VALUE,
					   SCAN_MATCHED_LF, SKIP_TO_CR, SCAN_NONMATCHED_LF };
	static ByteString const aKeyStrings[NUM_FIELD_KEYWORDS]
		= { ByteString(RTL_CONSTASCII_STRINGPARAM("message")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("msg#")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("from")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("date")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("cc")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("bcc")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("*to")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("*cc")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("priority")),
			ByteString(RTL_CONSTASCII_STRINGPARAM("subject")) };
	const int nMaxKeyLen = 8;
	 // the maximum len of any of the field keywords to match

	BOOL bFromField = FALSE;
	String aFrom;
	BOOL bDatePresent = FALSE;
	DateTime aDate;
	String aCC;
	String aBCC;
	String aTo; // save the *to field (the name of the bulletin board) as 'to'
	String aStaredCC;
	CntPriority ePriority = CNT_PRIO_NORMAL;
	BOOL bSubjectField = FALSE;
	String aSubject;
	sal_Char sKey[nMaxKeyLen + 1];
	int nKeyLen;
	FieldKeyword eMatchedKey;
	String aValue;
	ScanerState eState = BEGIN_OF_LINE;
	for (;;)
	{
	next:
		sal_Char cChar;
		*m_pHeaderStream >> cChar;
		if (m_pHeaderStream->IsEof())
			if (eState == BEGIN_OF_LINE)
			{
				delete m_pHeaderStream;
				m_pHeaderStream = 0;
				m_bEOF = TRUE;
				if (m_eNextField == FIELD_KEY_MESSAGE)
				{
					// SEEK_BEGIN w/ empty bb:
					getStatus().setStatus(ERRCODE_NONE, pJob);
					return 0;
				}
				goto done;
			}
			else
				goto format_error;
		if (!getStatus().setStatus(m_pHeaderStream->GetErrorCode(), pJob))
			goto io_error;

		switch (eState)
		{
			case BEGIN_OF_LINE:
				nKeyLen = 0;
				eState = SCAN_KEY;
			case SCAN_KEY:
				if (cChar == '#' || cChar == '*'
					|| cChar >= 'a' && cChar <= 'z')
				{
					if (nKeyLen == nMaxKeyLen)
						eState = SKIP_TO_CR;
					else
						sKey[nKeyLen++] = cChar;
					break;
				}
				else if (cChar >= 'A' && cChar <= 'Z')
				{
					if (nKeyLen == nMaxKeyLen)
						eState = SKIP_TO_CR;
					else
						sKey[nKeyLen++] = cChar + ('a' - 'A');
					break;
				}
				else if (cChar == ':')
					if (nKeyLen == 0)
						goto format_error;
					else
					{
						sKey[nKeyLen] = '\0';
						for (eMatchedKey = FIRST_FIELD_KEYWORD;
							 eMatchedKey < NUM_FIELD_KEYWORDS;
							 eMatchedKey = FieldKeyword(eMatchedKey + 1))
							if (aKeyStrings[eMatchedKey].Equals(sKey))
							{
								eState = SCAN_SPACE;
								goto next;
							}
					}
				if (m_eNextField != ANY_FIELD_KEYWORD)
					goto format_error;
				eState = cChar == '\r' ? SCAN_NONMATCHED_LF : SKIP_TO_CR;
				break;

			case SCAN_SPACE:
				aValue.Erase();
				eState = SCAN_VALUE;
				if (cChar == ' ')
					break;
			case SCAN_VALUE:
				if (cChar == '\r')
					eState = SCAN_MATCHED_LF;
				else
					aValue += cChar;
				break;

			case SCAN_MATCHED_LF:
				if (cChar != '\n')
					goto format_error;
				switch (eMatchedKey)
				{
					case FIELD_KEY_MESSAGE:
						if (aValue.Len() > 0)
							goto format_error;
						if (m_eNextField != FIELD_KEY_MESSAGE)
							goto done;
						m_eNextField = FIELD_KEY_MSG_NUM;
						break;

					case FIELD_KEY_MSG_NUM:
						if (m_eNextField != FIELD_KEY_MSG_NUM
							|| aValue.Len() == 0 || aValue.Len() > 9
							|| aValue.Len() > 1 && aValue.GetChar(0) == '0')
							goto format_error;
						m_nCurrentRefID = 0;
						{ // wrap i for old compilers...
							for (USHORT i = 0; i < aValue.Len(); ++i)
							{
								if (aValue.GetChar(i) < '0'
									|| aValue.GetChar(i) > '9')
									goto format_error;
								m_nCurrentRefID = 10 * m_nCurrentRefID
								                   + (aValue.GetChar(i)
													   - '0');
							}
						}
						m_eNextField = ANY_FIELD_KEYWORD;
						break;

					case FIELD_KEY_FROM:
						if (m_eNextField != ANY_FIELD_KEYWORD || bFromField)
							goto format_error;
						aFrom = aValue;
						bFromField = TRUE;
						break;

					case FIELD_KEY_DATE:
					{
						if (m_eNextField != ANY_FIELD_KEYWORD || bDatePresent)
							goto format_error;
						USHORT nPos = 0;
						if (nPos == aValue.Len() || aValue.GetChar(nPos) < '1'
							|| aValue.GetChar(nPos) > '9')
							goto format_error;
						USHORT nMonth = aValue.GetChar(nPos++) - '0';
						if (nPos < aValue.Len() && nMonth == 1
							&& aValue.GetChar(nPos) >= '0'
							&& aValue.GetChar(nPos) <= '2')
							nMonth = 10 + (aValue.GetChar(nPos++) - '0');
						if (nPos == aValue.Len()
							|| aValue.GetChar(nPos++) != '/')
							goto format_error;
						if (nPos == aValue.Len() || aValue.GetChar(nPos) < '1'
							|| aValue.GetChar(nPos) > '9')
							goto format_error;
						USHORT nDay = aValue.GetChar(nPos++) - '0';
						if (nPos < aValue.Len() && nDay <= 3
							&& aValue.GetChar(nPos) >= '0'
							&& aValue.GetChar(nPos) <= '9')
							nDay = 10 * nDay + (aValue.GetChar(nPos++) - '0');
						if (nPos == aValue.Len()
							|| aValue.GetChar(nPos++) != '/')
							goto format_error;
						USHORT nYearStartPos = nPos;
						USHORT nYear = 0;
						while (nPos < aValue.Len()
							   && aValue.GetChar(nPos) >= '0'
							   && aValue.GetChar(nPos) <= '9')
							nYear
								= 10 * nYear + (aValue.GetChar(nPos++) - '0');
						if (nPos - nYearStartPos < 2)
							goto format_error;
						if (nPos - nYearStartPos == 2)
							nYear += 1900;
						if (nPos == aValue.Len()
							|| aValue.GetChar(nPos++) != ' ')
							goto format_error;
						if (nPos == aValue.Len() || aValue.GetChar(nPos) < '0'
							|| aValue.GetChar(nPos) > '9')
							goto format_error;
						USHORT nHour = aValue.GetChar(nPos++) - '0';
						if (nPos < aValue.Len() && nHour == 1
							&& aValue.GetChar(nPos) >= '0'
							&& aValue.GetChar(nPos) <= '2')
						{
							nHour = 10 + (aValue.GetChar(nPos++) - '0');
							if (nHour == 12)
								nHour = 0;
						}
						if (nPos == aValue.Len()
							|| aValue.GetChar(nPos++) != ':')
							goto format_error;
						if (nPos + 1 >= aValue.Len()
							|| aValue.GetChar(nPos) < '0'
							|| aValue.GetChar(nPos) > '5'
							|| aValue.GetChar(nPos + 1) < '0'
							|| aValue.GetChar(nPos + 1) > '9')
							goto format_error;
						USHORT nMinute = 10 * (aValue.GetChar(nPos) - '0')
							              + (aValue.GetChar(nPos + 1) - '0');
						nPos += 2;
						if (nPos + 2 != aValue.Len()
							|| aValue.GetChar(nPos) != 'A'
							   && aValue.GetChar(nPos) != 'P'
							|| aValue.GetChar(nPos + 1) != 'M')
							goto format_error;
						if (aValue.GetChar(nPos) == 'P')
							nHour += 12;
						aDate = DateTime(Date(nDay, nMonth, nYear),
										 Time(nHour, nMinute));
						bDatePresent = TRUE;
						break;
					}

					case FIELD_KEY_CC:
						if (m_eNextField != ANY_FIELD_KEYWORD
							|| aValue.Len() == 0)
							goto format_error;
						if (aCC.Len() > 0)
							aCC.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
						aCC += aValue;
						break;

					case FIELD_KEY_BCC:
						if (m_eNextField != ANY_FIELD_KEYWORD
							|| aValue.Len() == 0)
							goto format_error;
						if (aBCC.Len() > 0)
							aBCC.AppendAscii(RTL_CONSTASCII_STRINGPARAM(
								                 ", "));
						aBCC += aValue;
						break;

					case FIELD_KEY_STARED_TO:
						if (m_eNextField != ANY_FIELD_KEYWORD
							|| aValue.Len() == 0)
							goto format_error;
						if (aTo.Len() > 0)
							aTo.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
						aTo += aValue;
						break;

					case FIELD_KEY_STARED_CC:
						if (m_eNextField != ANY_FIELD_KEYWORD
							|| aValue.Len() == 0)
							goto format_error;
						if (aStaredCC.Len() > 0)
							aStaredCC.AppendAscii(RTL_CONSTASCII_STRINGPARAM(
								                      ", "));
						aStaredCC += aValue;
						break;

					case FIELD_KEY_PRIORITY:
						if (m_eNextField != ANY_FIELD_KEYWORD
							|| ePriority != CNT_PRIO_NORMAL)
							goto format_error;
						if (aValue.EqualsIgnoreCaseAscii("low"))
							ePriority = CNT_PRIO_LOW;
						else if (aValue.EqualsIgnoreCaseAscii("urgent"))
							ePriority = CNT_PRIO_HIGH;
						else
							goto format_error;
						break;

					case FIELD_KEY_SUBJECT:
						if (m_eNextField != ANY_FIELD_KEYWORD
							|| bSubjectField)
							goto format_error;
						aSubject = aValue;
						bSubjectField = TRUE;
						break;
				}
				eState = BEGIN_OF_LINE;
				break;

			case SKIP_TO_CR:
				if (cChar == '\r')
					eState = SCAN_NONMATCHED_LF;
				break;

			case SCAN_NONMATCHED_LF:
				if (cChar != '\n')
					goto format_error;
				eState = BEGIN_OF_LINE;
				break;
		}
	}

done:
	if (m_eNextField == FIELD_KEY_MSG_NUM || aTo.Len() == 0)
		goto format_error;
	if (aStaredCC.Len() > 0)
		if (aCC.Len() == 0)
			aCC = aStaredCC;
		else
		{
			aCC.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
			aCC += aStaredCC;
		}
	m_eNextField = FIELD_KEY_MSG_NUM;
	return new CntVIMMessageHeader(m_nCurrentRefID, 0, m_nCurrentRefID, FALSE,
								   aFrom, ePriority, aSubject, bDatePresent,
								   aDate, aTo, aCC, aBCC, String());

format_error:
	getStatus().
	 setStatus(*new StringErrorInfo(ERRCODE_CHAOS_ILLEGAL_CCMAIL_EXPORT_FILE,
									String::CreateFromAscii(
										RTL_CONSTASCII_STRINGPARAM(
											CNT_VIM_EXPORT_PROGRAM))),
			   pJob);
io_error:
	delete m_pHeaderStream;
	m_pHeaderStream = 0;
	m_bJustOpened = TRUE; // try it again
	return 0;
}

//============================================================================
// virtual
BOOL CntVIMBulletinBoard::findMessage(const String & rUniqueID,
									  UINT32 & rRefIDPart1,
									  UINT32 & rRefIDPart2, CntNodeJob * pJob,
									  BOOL)
{
	sal_Unicode const * p = rUniqueID.GetBuffer();
	sal_Unicode const * pEnd = p + rUniqueID.Len();
	sal_uInt32 nValue;
	if (INetMIME::scanUnsigned(p, pEnd, true, nValue) && p == pEnd)
	{
		rRefIDPart1 = nValue;
		rRefIDPart2 = 0;
		return TRUE;
	}
	else
	{
		getStatus().setStatus(ERRCODE_CHAOS_ILLEGAL_MESSAGE_ID, pJob);
		return FALSE;
	}
}

//============================================================================
// virtual
CntVIMMessageContainer::ExecuteState
CntVIMBulletinBoard::openMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
								 CntNodeJob * pJob,
								 CntVIMMessageBody *& rpBody)
{
	DBG_ASSERT(nRefIDPart2 == 0,
			   "CntVIMBulletinBoard::openMessage(): Bad ID");

	rpBody = 0;

	if (m_pProcess)
	{
		vos::OProcess::TProcessInfo aInfo;
		if (m_pProcess->getInfo(vos::OProcess::TData_ExitCode, &aInfo)
			    != vos::OProcess::E_None
			|| !(aInfo.Fields & vos::OProcess::TData_ExitCode)
#if defined WNT
			|| aInfo.Code == 259 // STILL_ACTIVE
#endif // WNT
		   )
		{
			ULONG nTicks = Time::GetSystemTicks();
			if (nTicks - m_nProcessWaitingCount > EXPORT_MAX_WAIT)
			{
				cancelExecution();
				getStatus().
				 setStatus(*new StringErrorInfo
						         (ERRCODE_CHAOS_CCMAIL_EXPORT_NOT_TERMINATING,
								  String::CreateFromAscii(
									  RTL_CONSTASCII_STRINGPARAM(
										  CNT_VIM_EXPORT_PROGRAM))),
						   pJob);
				return EXECUTE_ERROR;
			}
			return EXECUTE_WAITING;
		}
		delete m_pProcess;
		m_pProcess = 0;
	}
	else
	{
		// Check password for double quote character ("):
		String aPassword(m_pMailbox->getPassword(),
						 getLibrary().getTextEncoding());
		if (aPassword.Search('"') != STRING_NOTFOUND)
		{
			getStatus().
			 setStatus
			  (*new StringErrorInfo(ERRCODE_CHAOS_BAD_CCMAIL_EXPORT_PASSWORD,
									String::CreateFromAscii(
										RTL_CONSTASCII_STRINGPARAM(
											CNT_VIM_EXPORT_PROGRAM))),
			   pJob);
			return EXECUTE_ERROR;
		}

		// Call export to get a file containing the message:
		DirEntry
			aTempFile(String::CreateFromAscii(
				                   RTL_CONSTASCII_STRINGPARAM(
		  					   "vim*.tmp")));
		aTempFile = aTempFile.TempName();
		m_aProcessTempFileName = aTempFile.GetFull();
		String aFileArg('@');
		aFileArg += m_aProcessTempFileName;
		String aBBoardArg(RTL_CONSTASCII_USTRINGPARAM("BBOARD/\""));
		aBBoardArg += m_aName;
		aBBoardArg += '"';
		String aReadArg(RTL_CONSTASCII_USTRINGPARAM("READ/"));
		aReadArg += nRefIDPart1;
		m_pProcess = new vos::OProcess(String::CreateFromAscii(
			                               RTL_CONSTASCII_STRINGPARAM(
											   CNT_VIM_EXPORT_PROGRAM)));
		rtl::OUString aArguments[9];
		aArguments[0] = String(m_pMailbox->getUsername(),
							   getLibrary().getTextEncoding());
		aArguments[1] = aPassword;
		aArguments[2]
			= String(m_pMailbox->getPath(), getLibrary().getTextEncoding());
		aArguments[3] = aFileArg;
		aArguments[4] = rtl::OUString::createFromAscii("NAME/1");
		aArguments[5] = rtl::OUString::createFromAscii("FILES/HEX");
		aArguments[6] = rtl::OUString::createFromAscii("MSGSIZE");
		aArguments[7] = aBBoardArg;
		aArguments[8] = aReadArg;
		if (m_pProcess->execute(vos::OProcess::TProcessOption(
			                        vos::OProcess::TOption_SearchPath
									    | vos::OProcess::TOption_Hidden),
								vos::OArgumentList(aArguments, 9))
			    == vos::OProcess::E_None)
		{
			m_nProcessWaitingCount = Time::GetSystemTicks();
			return EXECUTE_WAITING;
		}

		m_nProcessWaitingCount = 0;
		cancelExecution(FALSE);
		getStatus().
		 setStatus
		  (*new StringErrorInfo(
#if defined WNT || defined OS2
			                    RTL_CONSTASCII_LENGTH(CNT_VIM_EXPORT_PROGRAM)
								   + aArguments[0]
								   + aArguments[1]
								   + aArguments[2]
								   + aArguments[3]
								   + aArguments[4]
								   + aArguments[5]
								   + aArguments[6]
								   + aArguments[7]
								   + aArguments[8]
								  > 100 ?
								 ERRCODE_CHAOS_CCMAIL_EXPORT_TOO_LONG :
#endif // WNT, OS2
								 ERRCODE_CHAOS_CCMAIL_EXPORT_ERROR,
								String::CreateFromAscii(
									RTL_CONSTASCII_STRINGPARAM(
										CNT_VIM_EXPORT_PROGRAM))),
		   pJob);
		return EXECUTE_ERROR;
	}

	CntVIMTempFileStream * pStream
	 = new CntVIMTempFileStream(m_aProcessTempFileName);
	if (pStream->GetError())
	{
		getStatus().
		 setStatus(*new
				     TwoStringErrorInfo(ERRCODE_CHAOS_NO_CCMAIL_EXPORT_FILE,
										m_aProcessTempFileName,
										String::CreateFromAscii(
											RTL_CONSTASCII_STRINGPARAM(
												CNT_VIM_EXPORT_PROGRAM))),
				   pJob);
		m_aProcessTempFileName.Erase();
		delete pStream;
		return EXECUTE_ERROR;
	}
	m_aProcessTempFileName.Erase();

	rpBody = new CntVIMBBoardMessageBody(getStatus(), getLibrary(), pStream);
	return EXECUTE_DONE;
}

//============================================================================
// virtual
BOOL CntVIMBulletinBoard::notifyMessageRead(UINT32, UINT32, CntNodeJob *)
{
	DBG_ERROR("CntVIMBulletionBoard::notifyMessageRead(): Bad method");
	return FALSE;
}

//============================================================================
// virtual
BOOL CntVIMBulletinBoard::deleteMessage(UINT32, UINT32, CntNodeJob *)
{
	DBG_ERROR("CntVIMBulletionBoard::deleteMessage(): Bad method");
	return FALSE;
}

//============================================================================
// virtual
void CntVIMBulletinBoard::cancelExecution(BOOL bTerminate)
{
	if (m_pProcess)
	{
		if (bTerminate)
			m_pProcess->terminate();
		delete m_pProcess;
		m_pProcess = 0;
		if (m_aProcessTempFileName.Len())
			DirEntry(m_aProcessTempFileName).Kill();
	}
	m_aProcessTempFileName.Erase();
}

