/*************************************************************************
 *
 *  $RCSfile: cvimbox.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CHAOS_CVIMBOX_HXX
#define _CHAOS_CVIMBOX_HXX

#ifndef _SFXHINT_HXX //autogen
#include <svtools/hint.hxx>
#endif
#ifndef _DATETIME_HXX //autogen
#include <tools/datetime.hxx>
#endif
#ifndef _ERRCODE_HXX //autogen
#include <tools/errcode.hxx>
#endif
#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif

#ifndef _CNTVWITM_HXX
#include <cntvwitm.hxx>
#endif

class CntVIMLibrary;
class CntVIMSendBufferNode;
union CntVIMPointer;
class DirEntry;
class INetCoreMIMEMessage;
class INetCoreNewsMessage;
class SvStream;

#if defined TF_UCB
namespace chaos {
#endif // TF_UCB

class CntFolderList;
class CntNodeJob;
class CntVIMMailbox;

//============================================================================
class CntVIMStatus
{
	CntVIMMailbox & m_rMailbox;
	ErrCode m_nCode;
	ULONG m_nVIMStatus;

	void handleError(ErrCode nSpecificCode, CntNodeJob & rJob);

public:
	CntVIMStatus(CntVIMMailbox & rTheMailbox):
		m_rMailbox(rTheMailbox), m_nCode(ERRCODE_NONE) {}

	BOOL setStatus(ErrCode nTheCode, CntNodeJob * pJob);

	BOOL setVIMStatus(ULONG nTheVIMStatus, CntNodeJob * pJob);

	BOOL isOK() const;

	USHORT getStatusValue() const;

	String getStatusText(BOOL bSendStatus = FALSE) const;
};

//============================================================================
class CntVIMStatusHint: public SfxHint
{
public:
	TYPEINFO();
};

//============================================================================
class CntVIMMessageItem
{
	SvStream * m_pDataStream;

public:
	CntVIMMessageItem(SvStream * pTheDataStream):
		m_pDataStream(pTheDataStream) {}

	virtual ~CntVIMMessageItem();

	inline SvStream * takeDataStream();

	virtual String getContentType() const = 0;

	virtual String getName() const = 0;
};

inline SvStream * CntVIMMessageItem::takeDataStream()
{
	SvStream * pTheDataStream = m_pDataStream;
	m_pDataStream = 0;
	return pTheDataStream;
}

//============================================================================
class CntVIMMessageBody
{
	CntVIMStatus & m_rStatus;
	const CntVIMLibrary & m_rLibrary;

protected:
	CntVIMStatus & getStatus() { return m_rStatus; }

	const CntVIMLibrary & getLibrary() const { return m_rLibrary; }

public:
	CntVIMMessageBody(CntVIMStatus & rTheStatus,
					  const CntVIMLibrary & rTheLibrary):
		m_rStatus(rTheStatus), m_rLibrary(rTheLibrary) {}

	virtual ~CntVIMMessageBody() {}

	virtual BOOL startItemIteration(ULONG & rnItems, CntNodeJob * pJob) = 0;

	virtual CntVIMMessageItem * getNextItem(CntNodeJob * pJob) = 0;
};

//============================================================================
class CntVIMMessageHeader
{
	UINT32 m_nRefIDPart1;
	UINT32 m_nRefIDPart2;
	String m_aUniqueID;
	BOOL m_bInternallyUnread;
	String m_aFrom;
	CntPriority m_ePriority;
	String m_aSubject;
	BOOL m_bDatePresent;
	DateTime m_aDate;
	String m_aTo;
	String m_aCC;
	String m_aBCC;
	String m_aKeywords;

public:
	inline CntVIMMessageHeader(UINT32 nTheRefIDPart1, UINT32 nTheRefIDPart2,
							   const String & rTheUniqueID,
							   BOOL bTheInternallyUnread,
							   const String & rTheFrom,
							   CntPriority eThePriority,
							   const String & rTheSubject,
							   BOOL bTheDatePresent,
							   const DateTime & rTheDate,
							   const String & rTheTo, const String & rTheCC,
							   const String & rTheBCC,
							   const String & rTheKeywords);

	UINT32 getRefIDPart1() const { return m_nRefIDPart1; }

	UINT32 getRefIDPart2() const { return m_nRefIDPart2; }

	const String & getUniqueID() const { return m_aUniqueID; }

	BOOL getInternallyUnread() const { return m_bInternallyUnread; }

	const String & getFrom() const { return m_aFrom; }

	CntPriority getPriority() const { return m_ePriority; }

	const String & getSubject() const { return m_aSubject; }

	BOOL isDatePresent() const { return m_bDatePresent; }

	const DateTime & getDate() const { return m_aDate; }

	const String & getTo() const { return m_aTo; }

	const String & getCC() const { return m_aCC; }

	const String & getBCC() const { return m_aBCC; }

	const String & getKeywords() const { return m_aKeywords; }
};

inline CntVIMMessageHeader::CntVIMMessageHeader(UINT32 nTheRefIDPart1,
												UINT32 nTheRefIDPart2,
												const String & rTheUniqueID,
												BOOL bTheInternallyUnread,
												const String & rTheFrom,
												CntPriority eThePriority,
												const String & rTheSubject,
												BOOL bTheDatePresent,
												const DateTime & rTheDate,
												const String & rTheTo,
												const String & rTheCC,
												const String & rTheBCC,
												const String & rTheKeywords):
	m_nRefIDPart1(nTheRefIDPart1),
	m_nRefIDPart2(nTheRefIDPart2),
	m_aUniqueID(rTheUniqueID),
	m_bInternallyUnread(bTheInternallyUnread),
	m_aFrom(rTheFrom),
	m_ePriority(eThePriority),
	m_aSubject(rTheSubject),
	m_bDatePresent(bTheDatePresent),
	m_aDate(rTheDate),
	m_aTo(rTheTo),
	m_aCC(rTheCC),
	m_aBCC(rTheBCC),
	m_aKeywords(rTheKeywords)
{}

//============================================================================
class CntVIMMessageContainer
{
public:
	enum ExecuteState { EXECUTE_ERROR, EXECUTE_WAITING, EXECUTE_DONE };

	enum { SEEK_BEGIN = 1 };

private:
	CntVIMStatus & m_rStatus;
	const CntVIMLibrary & m_rLibrary;

protected:
	const CntVIMLibrary & getLibrary() const { return m_rLibrary; }

public:
	CntVIMMessageContainer(CntVIMStatus & rTheStatus,
						   const CntVIMLibrary & rTheLibrary):
		m_rStatus(rTheStatus), m_rLibrary(rTheLibrary) {}

	virtual ~CntVIMMessageContainer() {}

	CntVIMStatus & getStatus() { return m_rStatus; }

	const CntVIMStatus & getStatus() const { return m_rStatus; }

	virtual BOOL open(CntNodeJob * pJob) = 0;

	virtual BOOL canSeekForMsgRefID() = 0;

	virtual BOOL canIterateFromEnd(CntNodeJob * pJob) = 0;

	virtual BOOL queryNewMessages(BOOL bTryHard, CntNodeJob * pJob) = 0;

	virtual ExecuteState startHeaderIteration(UINT32 nRefIDPart1,
											  UINT32 nRefIDPart2,
											  CntNodeJob * pJob) = 0;

	virtual ExecuteState startHeaderIteration(BOOL bFromEnd) = 0;

	virtual CntVIMMessageHeader * getNextHeader(CntNodeJob * pJob,
												ULONG nFilterType,
												const void * pFilterData,
												ULONG nFilterDataLen) = 0;

	virtual BOOL findMessage(const String & rUniqueID, UINT32 & rRefIDPart1,
							 UINT32 & rRefIDPart2, CntNodeJob * pJob,
							 BOOL bNotFoundIsOK) = 0;

	virtual ExecuteState openMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
									 CntNodeJob * pJob,
									 CntVIMMessageBody *& rpBody) = 0;

	virtual BOOL notifyMessageRead(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
								   CntNodeJob * pJob) = 0;

	virtual BOOL deleteMessage(UINT32 nRefIDPart1, UINT32 nRefIDPart2,
							   CntNodeJob * pJob) = 0;

	virtual void cancelExecution(BOOL bTerminate = TRUE) = 0;
};

//============================================================================
class CntVIMMailbox
{
public:
	enum InitStatus { OK, LOGIN_ERROR, OTHER_ERROR };

private:
	CntVIMStatus * m_pStatus;
	CntVIMLibrary * m_pLibrary;
	CntVIMPointer * m_pSession;
	ByteString m_aPath;
	ByteString m_aUsername;
	ByteString m_aPassword;

	BOOL transformMessage(const INetCoreMIMEMessage & rMIMEMsg,
						  CntVIMPointer & rVIMMsg,
						  CntVIMSendBufferNode *& rpBuffers,
						  BOOL & rbAddNotePart, CntNodeJob * pJob);

public:
	CntVIMMailbox();

	~CntVIMMailbox();

	InitStatus initialize(const String & rThePath,
						  const String & rTheUsername,
						  const String & rThePassword, CntNodeJob * pJob = 0,
						  BOOL bHandleLoginErrors = TRUE);

	BOOL isInitialized() const;

	const CntVIMStatus & getStatus() const { return *m_pStatus; }

	const CntVIMLibrary & getLibrary() const { return *m_pLibrary; }

	rtl_TextEncoding getTextEncoding() const;

	CntVIMPointer & getSession() { return *m_pSession; }

	const ByteString & getPath() const { return m_aPath; }

	const ByteString & getUsername() const { return m_aUsername; }

	const ByteString & getPassword() const { return m_aPassword; }

	BOOL isCCMail() const;

	BOOL restartSession(CntNodeJob * pJob);

	BOOL sendMessage(const INetCoreNewsMessage & rMsg, CntNodeJob * pJob = 0);

	CntVIMMessageContainer * getInbox();

	CntVIMMessageContainer * getBulletinBoard(const String & rName);

	static CntFolderList * parseBBoardList(const String & rFilename,
										   const DirEntry & rPOPath);
};

#if defined TF_UCB
}
#endif // TF_UCB

#endif // _CHAOS_CVIMBOX_HXX

