/*************************************************************************
 *
 *  $RCSfile: cvimimp.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CHAOS_CVIMIMP_HXX
#define _CHAOS_CVIMIMP_HXX

#ifndef _CASTMACS_HXX
#include <castmacs.hxx>
#endif
#ifndef _CNTBASE_HXX
#include <cntbase.hxx>
#endif

namespace inet { class INetCoreMailer; }
class CntBoolItem;
class DateTime;

#if defined TF_UCB
namespace chaos {
#endif // TF_UCB

class CntActionListItem;
class CntMessageNode;
class CntNodeItem;
class CntNodeJob;
class CntStorageNode;
class CntStoreItemSet;
class CntVIMBBoardBoxFlagReadTask;
class CntVIMBBoardBoxOpenTask;
class CntVIMContainerFlagTask;
class CntVIMContainerOpenTask;
class CntVIMMailbox;
class CntVIMMessageContainer;

//============================================================================
// An attribute used for bulletin board entries in view storages:
#define CNTDIRENTRY_ATTRIB_VIM_SUBSCRIBED CNTDIRENTRY_ATTRIB_USER1

//============================================================================
class CntVIMNodeImp
{
public:
	enum OnlineMode { ONLINE_NO, ONLINE_YES, ONLINE_RETRY };

private:
	CntNode * pParent;
	CntVIMMailbox * pMailbox;
	BOOL bOnline;
	BOOL bInitialized;

	CntStoreItemSet * OpenDirectoryItemSet(CntStorageNode * pDirectory);

	BOOL OpenVIMSession(CntNodeJob * pJob, BOOL bAutoRetry) const;

	static BOOL hasOpenJob(CntNode & rNode);

public:
	CntVIMNodeImp(CntNode * pTheParent);

	~CntVIMNodeImp();

	void Initialize(CntNodeJob & rJob);

	CntVIMMailbox * GetMailbox() const { return pMailbox; }

	OnlineMode CheckOnline(BOOL bMustGoOnline, CntNodeJob * pJob,
						   BOOL bAutoRetry = TRUE);

	void Open(USHORT nWhich, CntNodeJob * pJob);

	void SetConnMode(CntConnMode eConnMode)
	{ bOnline = eConnMode == CNT_CONN_MODE_ONLINE; }

	static void createNewMessage(CntNodeItem * pMsgNodeItem,
								 CntNodeJob * pJob, BOOL bNews,
								 const String & rNewsGroups);

	BOOL getUpdateOnOpenMode(const CntNodeJob & rJob) const;
};

//============================================================================
class CntVIMContainerNodeImp
{
	enum FindResult { FOUND, NOT_FOUND, ERROR };

public:
	class Change
	{
		long nValue;

	protected:
		Change(long nTheValue): nValue(nTheValue) {}

	public:
		virtual BOOL IsAbsolute() const = 0;

		virtual BOOL HasChanged() const = 0;

		long GetValue() const { return nValue; }
	};

	class AbsoluteChange: public Change
	{
	public:
		AbsoluteChange(long nValue): Change(nValue) {}

		virtual BOOL IsAbsolute() const { return TRUE; }

		virtual BOOL HasChanged() const { return TRUE; }
	};

	class RelativeChange: public Change
	{
	public:
		RelativeChange(long nValue): Change(nValue) {}

		virtual BOOL IsAbsolute() const { return FALSE; }

		virtual BOOL HasChanged() const { return CAST_TO_BOOL(GetValue()); }
	};

private:
	inet::INetCoreMailer * pSavedMailer;

protected:
	CntNode * pParent;
	BOOL bInitialized;
	CntVIMNodeImp * pVIMBoxNodeImp;
	CntVIMMessageContainer * pContainer;
	String aUserDataItemSetName;

private:
	static String DateString(const DateTime & rDate);

	inet::INetCoreMailer * GetMailer();

	FindResult GetMsgRefID(SfxItemSet * pItems, BOOL bPutIntoItems,
						   const String & rMsgID, UINT32 & rMsgRefIDPart1,
						   UINT32 & rMsgRefIDPart2, CntNodeJob * pJob,
						   BOOL bMustFind);

protected:
	inline CntVIMContainerNodeImp(CntNode * pTheParent);

	virtual ~CntVIMContainerNodeImp();

	void DeleteMessageFromStorage(BOOL bDeletePhysical, BOOL bKeepFlag,
								  CntStorageNode * pCache,
								  CntStorageNode * pDirectory,
								  CntStorageNode * pUserData,
								  const String & rMsgID,
								  const String & rMsgURL);
	 // does part of DeleteMsg

public:
	virtual BOOL IsInbox() const = 0;

	virtual void Initialize(CntNodeJob & rJob,
							CntVIMNodeImp * pTheVIMBoxNodeImp);

	CntStoreItemSet * OpenDirectoryItemSet(CntStorageNode * pDirectory);

	CntStoreItemSet * OpenUserDataItemSet(CntStorageNode * pUserData);

	static CntStoreItemSet * OpenMessageDirectoryItemSet(CntStorageNode *
														  pDirectory,
														 const String &
														  rMsgID,
														 CntNode * pMsgNode
														  = 0);
	 // if pMsgNode is 0 and no item set can be found in the directory storage
	 // (maybe because the storage is corrupted), 0 is returned

	static String GetMsgIDFromURL(const String & rMsgURL);

	virtual BOOL DoDeleteMessagesFromServer() const = 0;

	virtual BOOL DoCacheMessageBodies() const = 0;

	virtual UINT16 GetMessageExpirationLimit() const = 0; // 0 means no limit

	void UpdateMessageCounts(CntNodeJob * pJob, const Change & rTotalChange,
							 const Change & rSeenChange,
							 const Change & rMarkedChange);

	virtual void SeenStatusChanged(CntNodeJob * pJob, BOOL bIsSeen) = 0;
	 // called after the seen status of this node changed from seen to unseen
	 // or vice versa

	void SetMessageFlag(USHORT nWhich, BOOL bFlag, CntStorageNode * pUserData,
						const String & rMsgURL);

	BOOL NotifyMessageRead(CntNodeJob * pJob, const String & rMsgID,
						   CntNode * pMsgNode);
	 // returns FALSE if pJob is canceled

	virtual CntVIMMessageContainer * OpenContainer(CntNodeJob * pJob) = 0;

	CntVIMMessageContainer * GetContainer() { return pContainer; }

	void Open(CntNodeJob * pJob);

	void Flag(CntNodeJob * pJob);

	virtual BOOL IsOpenSubJob(CntNodeJob * pJob) = 0;

	virtual CntNodeJob * GetUpdateJob(CntNodeJob * pJob) = 0;

	void InitMessage(CntNodeJob * pJob, CntNode * pMsgNode);

	void setMsgBodyState(CntNode & rMsg, CntStorageNode * pCache);

	void OpenMsg(CntMessageNode * pMsgNode, CntNodeJob * pJob);

	BOOL createMessageBodyItem(CntNodeJob & rJob,
							   CntStoreItemSet * pMsgDirSet,
							   CntNode * pMsgNode, BOOL bCacheBody,
							   BOOL bMustLoad);
	 // a return value of TRUE means 'retry'

	void MarkMsg(CntMessageNode * pMsg, const CntBoolItem & rFlagItem,
				 CntNodeJob * pJob);

	void SetMsgActionList(CntMessageNode & rMsg,
						  const CntActionListItem & rActionList,
						  CntNodeJob & rJob);

	void DeleteMsg(CntMessageNode * pMsg, CntNodeJob * pJob);

	void UndeleteMessage(CntMessageNode * pMsgNode, CntNodeJob * pJob);

	void changeMsgStoreMode(CntNodeJob & rJob);
};

inline CntVIMContainerNodeImp::CntVIMContainerNodeImp(CntNode * pTheParent):
	pParent(pTheParent),
	pContainer(0),
	bInitialized(FALSE),
	pSavedMailer(0)
{}

//============================================================================
class CntVIMInboxNodeImp: public CntVIMContainerNodeImp
{
public:
	CntVIMInboxNodeImp(CntNode * pTheParent);

	virtual BOOL IsInbox() const { return TRUE; }

	virtual void Initialize(CntNodeJob & rJob);

	virtual BOOL DoDeleteMessagesFromServer() const;

	virtual BOOL DoCacheMessageBodies() const;

	virtual UINT16 GetMessageExpirationLimit() const { return 0; }

	virtual void SeenStatusChanged(CntNodeJob *, BOOL) {}

	virtual CntVIMMessageContainer * OpenContainer(CntNodeJob * pJob);

	virtual BOOL IsOpenSubJob(CntNodeJob * pJob);

	virtual CntNodeJob * GetUpdateJob(CntNodeJob * pJob);
};

//============================================================================
class CntVIMBBoardNodeImp: public CntVIMContainerNodeImp
{
	String m_aQuotedName;

public:
	CntVIMBBoardNodeImp(CntNode * pTheParent);

	virtual BOOL IsInbox() const { return FALSE; }

	virtual void Initialize(CntNodeJob & rJob);

	virtual BOOL DoDeleteMessagesFromServer() const { return FALSE; }

	virtual BOOL DoCacheMessageBodies() const;

	virtual UINT16 GetMessageExpirationLimit() const;

	virtual void SeenStatusChanged(CntNodeJob * pNodeJob, BOOL bIsSeen);

	virtual CntVIMMessageContainer * OpenContainer(CntNodeJob *);

	virtual BOOL IsOpenSubJob(CntNodeJob * pJob);

	virtual CntNodeJob * GetUpdateJob(CntNodeJob * pJob);

	void SubscribedStatusChange(CntNodeJob * pJob);

	void CleanCache(CntNodeJob * pJob);
};

//============================================================================
class CntVIMBBoardBoxNodeImp
{
public:
	enum ChangeState { INCREMENT, DECREMENT, STAY };

private:
	CntNode & m_rBBoardBoxNode;
	BOOL m_bInitialized;

public:
	inline CntVIMBBoardBoxNodeImp(CntNode & rTheBBoardBoxNode);

	void Initialize(CntNodeJob & rJob, BOOL bView);

	CntStoreItemSet * OpenDirectoryItemSet(CntStorageNode * pDirectory);

	CntStoreItemSet * OpenUserDataItemSet(CntStorageNode * pUserData);

	static CntStoreItemSet * OpenViewDataItemSet(CntStorageNode * pViewData);

	void UpdateBBoardCounts(CntNodeJob * pJob, ChangeState eTotalChange,
							BOOL bIsSeen, BOOL bWasSeen, BOOL bIsSubscribed,
							BOOL bWasSusbscribed);

	CntFolderList * LoadBBoardList(); // create list from external file

	void SupplyBBoardList(CntNodeJob * pJob);
	 // assemble list and send to 'subscribed' tab page

	void UpdateBBoardList(CntFolderList * pList, CntNodeJob * pJob);
	 // process list received from 'subscribed' tab page

	void Open(CntNodeJob * pJob);

	void FlagRead(CntNodeJob * pJob);

	void ViewModeChange(CntNodeJob * pJob);

	void changeMsgStoreMode(CntNodeJob & rJob);
};

inline CntVIMBBoardBoxNodeImp::CntVIMBBoardBoxNodeImp(CntNode &
													   rTheBBoardBoxNode):
	m_rBBoardBoxNode(rTheBBoardBoxNode),
	m_bInitialized(FALSE)
{}

#if defined TF_UCB
}
#endif // TF_UCB

#endif // _CHAOS_CVIMIMP_HXX

