/*************************************************************************
 *
 *  $RCSfile: ucbcfg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:20:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/**************************************************************************
								TODO
 **************************************************************************

 Configuration_Impl::save()
 Configuration_Impl::restore()
	- den richtigen key aus createComponentFactory bis hierher transportieren
		-> kein support -> MM.

 *************************************************************************/

#ifndef _UCBCFG_HXX
#include <ucbcfg.hxx>
#endif

#include <cstddef>
#include <functional>
#include <hash_map>
#include <list>

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif
#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif
#ifndef _VOS_DIAGNOSE_H_
#include <vos/diagnose.hxx>
#endif

using namespace com::sun::star::beans;
using namespace com::sun::star::lang;
using namespace com::sun::star::registry;
using namespace com::sun::star::ucb;
using namespace com::sun::star::uno;
using namespace cppu;
using namespace rtl;

//============================================================================
//
// class ContentProviderServiceInfo_Impl.
//
//============================================================================

typedef std::list
<
	ContentProviderServiceInfo2*
>
ContentProviderServiceInfo_Impl;

//============================================================================
//
// class Configuration_Impl.
//
//============================================================================

class Configuration_Impl :
				public OWeakObject,
				public XContentProviderConfiguration
{
public:
	Configuration_Impl( const Reference< XMultiServiceFactory >& rxServiceFactory, const OUString& rKey );
	virtual ~Configuration_Impl();

	// XInterface
	XINTERFACE_DECL()

    // XContentProviderConfiguration
    virtual sal_Bool SAL_CALL addContentProviderService( const OUString& Template, const OUString& Service, const OUString& Arguments, sal_Bool ReplaceExisting )
		throw( RuntimeException );
    virtual void SAL_CALL removeContentProviderService( const OUString& Template, const OUString& Service )
		throw( RuntimeException );
    virtual Sequence< ContentProviderServiceInfo2 > SAL_CALL queryContentProviderServiceInfo( const OUString& Template )
		throw( RuntimeException );

private:
	void init();
	void restore();
	void save();

	// members
	Reference< XMultiServiceFactory > m_xSMgr;
	OUString m_aKey;
	ContentProviderServiceInfo_Impl* m_pInfo;
};

//=========================================================================
//
// Configuration_Impl Implementation.
//
//=========================================================================

Configuration_Impl::Configuration_Impl(
				const Reference< XMultiServiceFactory >& rxServiceFactory,
				const OUString& rKey)
: m_xSMgr( rxServiceFactory ),
  m_aKey( rKey ),
  m_pInfo( NULL )
{
}

//=========================================================================
// virtual
Configuration_Impl::~Configuration_Impl()
{
	if ( m_pInfo )
	{
		ContentProviderServiceInfo_Impl::iterator it  = m_pInfo->begin();
		ContentProviderServiceInfo_Impl::iterator end = m_pInfo->end();

		while ( it != end )
		{
			delete (*it);
			it++;
		}

		delete m_pInfo;
	}
}

//=========================================================================
//
// XInterface methods.
//
//=========================================================================

XINTERFACE_IMPL_1( Configuration_Impl,
				   XContentProviderConfiguration );

//=========================================================================
//
// XContentProviderConfiguration methods.
//
//=========================================================================

// virtual
sal_Bool SAL_CALL Configuration_Impl::addContentProviderService(
											const OUString& Template,
											const OUString& Service,
											const OUString& Arguments,
											sal_Bool ReplaceExisting )
	throw( RuntimeException )
{
	// No duplicate <Service,Template> pairs allowed!!!

	if ( !Service.getLength() )
		return sal_False;

	if ( !Template.getLength() )
		return sal_False;

	init();

	ContentProviderServiceInfo_Impl::iterator it  = m_pInfo->begin();
	ContentProviderServiceInfo_Impl::iterator end = m_pInfo->end();

	// Entry for <Service,Template> pair to remove.
	while ( it != end )
	{
		ContentProviderServiceInfo2* pInfo = (*it);
		if ( pInfo->Scheme.equalsIgnoreCase( Template ) &&
			 pInfo->Service.equals( Service ) )
		{
			// Already in list - just merge arguments and replace-flag.
			if ( pInfo->Arguments != Arguments
				 || pInfo->ReplaceExisting != ReplaceExisting )
			{
				pInfo->Arguments = Arguments;
				pInfo->ReplaceExisting = ReplaceExisting;

				// Write data to registry.
				save();
			}
			return sal_True;
		}

		it++;
	}

	// Add new entry to list....
	ContentProviderServiceInfo2* pInfo = new ContentProviderServiceInfo2;
	pInfo->Service 		   = Service;
	pInfo->Scheme		   = Template;
	pInfo->Arguments	   = Arguments;
	pInfo->ReplaceExisting = ReplaceExisting;

	m_pInfo->push_back( pInfo );

	// Write data to registry.
	save();

	return sal_True;
}

//=========================================================================
// virtual
void SAL_CALL Configuration_Impl::removeContentProviderService(
												const OUString& Template,
												const OUString& Service )
	throw( RuntimeException )
{
	// Service == "" -> remove all entries for Template

	if ( !Template.getLength() )
		return;

	init();

	ContentProviderServiceInfo_Impl::iterator it  = m_pInfo->begin();
	ContentProviderServiceInfo_Impl::iterator end = m_pInfo->end();

	if ( Service.getLength() )
	{
		// Entry for <Service,Template> pair to remove.
		while ( it != end )
		{
			ContentProviderServiceInfo_Impl::iterator next = it;
			++next;

			ContentProviderServiceInfo* pInfo = (*it);
			if ( pInfo->Scheme.equalsIgnoreCase( Template ) &&
				 pInfo->Service.equals( Service ) )
			{
				m_pInfo->erase( it );

				// Write data to registry.
				save();

				// List must(!) not contain duplicate
				// <Service,Template> pairs. We're done.
				return;
			}
			it = next;
		}
	}
	else
	{
		// All entries for Template to remove...
		sal_Bool bDirty = sal_False;
		while ( it != end )
		{
			ContentProviderServiceInfo_Impl::iterator next = it;
			++next;

			ContentProviderServiceInfo2* pInfo = (*it);
			if ( pInfo->Scheme.equalsIgnoreCase( Template ) )
			{
				m_pInfo->erase( it );
				bDirty = sal_True;
			}
			it = next;
		}

		if ( bDirty )
		{
			// Write data to registry.
			save();
		}
	}
}

//=========================================================================
// virtual
Sequence< ContentProviderServiceInfo2 > SAL_CALL
Configuration_Impl::queryContentProviderServiceInfo( const OUString& Template )
	throw( RuntimeException )
{
	// Template == "" -> info for all schemes

	init();

	sal_uInt32 nCount = m_pInfo->size();
	if ( nCount )
	{
		Sequence< ContentProviderServiceInfo2 > aSeq( nCount );
		ContentProviderServiceInfo2* pInfos = aSeq.getArray();

		ContentProviderServiceInfo_Impl::iterator it  = m_pInfo->begin();
		ContentProviderServiceInfo_Impl::iterator end = m_pInfo->end();

		if ( Template.getLength() )
		{
			// Info for one scheme requested...

			sal_uInt32 nArrPos = 0;
			while ( it != end )
			{
				if ( (*it)->Scheme.equalsIgnoreCase( Template ) )
				{
					pInfos[ nArrPos ] = *(*it);
					nArrPos++;
				}
				it++;
			}

			if ( nArrPos != nCount )
				aSeq.realloc( nArrPos );
		}
		else
		{
			// Info for all schemes requested...

			sal_uInt32 nArrPos = 0;
			while ( it != end )
			{
				pInfos[ nArrPos ] = *(*it);
				nArrPos++;
				it++;
			}
		}

		return aSeq;
	}

	return Sequence< ContentProviderServiceInfo2 >();
}

//=========================================================================
//
// Other methods.
//
//=========================================================================

void Configuration_Impl::init()
{
	if ( !m_pInfo )
	{
		m_pInfo = new ContentProviderServiceInfo_Impl;
		restore();
	}
}

//=========================================================================
void Configuration_Impl::restore()
{
	// Read data from registry into m_pInfo.

	init();

	if ( m_xSMgr.is() )
	{
		Reference< XPropertySet > xSet(	m_xSMgr, UNO_QUERY );
		if ( xSet.is() )
		{
			Any aValue;

			try
			{
				aValue = xSet->getPropertyValue(
					 			OUString::createFromAscii( "Registry" ) );
			}
			catch ( UnknownPropertyException& )
			{
				VOS_ENSURE(
					sal_False,
					"UcbConfiguration::restore - UnknownPropertyException!" );
				return;
			}
			catch ( WrappedTargetException& )
			{
				VOS_ENSURE(
					sal_False,
					"UcbConfiguration::restore - WrappedTargetException!" );
				return;
			}

			Reference< XSimpleRegistry > xReg(
				*(Reference< XInterface >*)aValue.getValue(), UNO_QUERY );

			if ( xReg.is() )
			{
				try
				{
					Reference< XRegistryKey > xRootKey( xReg->getRootKey() );
					if ( xRootKey.is() )
					{
						Reference< XRegistryKey > xImplKey(
							xRootKey->openKey(
								OUString::createFromAscii(
									"/IMPLEMENTATIONS/UcbConfiguration" ) ) );
						if ( xImplKey.is() )
						{
							Reference< XRegistryKey > xKey(
								xImplKey->openKey( m_aKey ) );
							if ( xKey.is() )
							{
								Sequence< OUString > aSeq =
												xKey->getStringListValue();
								const OUString* pInfos = aSeq.getConstArray();

								sal_uInt32 nCount = aSeq.getLength();
								if ( nCount )
								{
									// Version 1 had no Arguments:
									int nVersion
										= pInfos[0].compareToAscii("1") == 0 ?
										      1 :
										  pInfos[0].compareToAscii("2") == 0 ?
										      2 : 0;
									if (nVersion >= 1)
									{
										sal_uInt32 nInc
											= nVersion == 1 ? 3 : 4;
										for ( sal_uInt32 n = 1;
										  	n + nInc <= nCount;
										  	n += nInc )
										{
											ContentProviderServiceInfo2* pInfo =
												new ContentProviderServiceInfo2;

											pInfo->Service = pInfos[ n ];
											pInfo->Scheme  = pInfos[ n + 1 ];
											pInfo->ReplaceExisting =
												pInfos[ n + 2 ].
												    compareToAscii("0") != 0;
											if (nVersion == 2)
												pInfo->Arguments
													= pInfos[n + 3];

											m_pInfo->push_back( pInfo );
										}
									}
									else
									{
										VOS_ENSURE(
											sal_False,
											"UcbConfiguration::restore - "
											"wrong version!");
									}
								}
							}
						}
					}
				}
				catch( InvalidRegistryException& )
				{
					VOS_ENSURE(	sal_False,
								"UcbConfiguration::restore - "
						   	    "InvalidRegistryException!" );
				}

				return;
			}
		}
	}

	VOS_ENSURE( sal_False, "UcbConfiguration::restore - failed!" );
}

//=========================================================================
void Configuration_Impl::save()
{
	// Write data from m_pInfo to registry.

	init();

	if ( m_xSMgr.is() )
	{
		Reference< XPropertySet > xSet(	m_xSMgr, UNO_QUERY );
		if ( xSet.is() )
		{
			Any aValue;

			try
			{
				aValue = xSet->getPropertyValue(
					 			OUString::createFromAscii( "Registry" ) );
			}
			catch ( UnknownPropertyException& )
			{
				VOS_ENSURE(
					sal_False,
					"UcbConfiguration::save - UnknownPropertyException!" );
				return;
			}
			catch ( WrappedTargetException& )
			{
				VOS_ENSURE(
					sal_False,
					"UcbConfiguration::save - WrappedTargetException!" );
				return;
			}

			Reference< XSimpleRegistry > xReg(
				*(Reference< XInterface >*)aValue.getValue(), UNO_QUERY );

			if ( xReg.is() )
			{
				try
				{
					Reference< XRegistryKey > xRootKey( xReg->getRootKey() );
					if ( xRootKey.is() )
					{
						Reference< XRegistryKey > xImplKey(
							xRootKey->createKey(
								OUString::createFromAscii(
									"/IMPLEMENTATIONS/UcbConfiguration" ) ) );
						if ( xImplKey.is() )
						{
							Reference< XRegistryKey > xKey(
								xImplKey->createKey( m_aKey ) );
							if ( xKey.is() )
							{
								sal_uInt32 nCount = m_pInfo->size();

								Sequence< OUString > aSeq( 4 * nCount + 1 );
								OUString* pInfos = aSeq.getArray();

								sal_uInt32 nArrPos = 0;

								// Header... ( version number )
								pInfos[ nArrPos ]
									= OUString::createFromAscii( "2" );
								nArrPos++;

								ContentProviderServiceInfo_Impl::iterator it =
															m_pInfo->begin();
								ContentProviderServiceInfo_Impl::iterator end =
															m_pInfo->end();
								// Data...
								while ( it != end )
								{
									ContentProviderServiceInfo2* pInfo = (*it);

									pInfos[ nArrPos ] = pInfo->Service;
									nArrPos++;
									pInfos[ nArrPos ] = pInfo->Scheme;
									nArrPos++;
									pInfos[ nArrPos ] =
										( pInfo->ReplaceExisting == sal_True )
										? OUString::createFromAscii( "1" )
										: OUString::createFromAscii( "0" );
									nArrPos++;
									pInfos[ nArrPos ] = pInfo->Arguments;
									nArrPos++;

									it++;
								}

								// Write data.
								xKey->setStringListValue( aSeq );
							}
						}
					}
				}
				catch( InvalidRegistryException& )
				{
					VOS_ENSURE(
						sal_False,
						"UcbConfiguration::save - InvalidRegistryException!" );
				}
				catch( InvalidValueException& )
				{
					VOS_ENSURE(
						sal_False,
						"UcbConfiguration::save - InvalidValueException!" );
				}

				return;
			}
		}
	}

	VOS_ENSURE( sal_False, "UcbConfiguration::save - failed!" );
}

//=========================================================================
//
// UcbConfigurationManager Implementation.
//
//=========================================================================

struct StringHash_Impl: public std::unary_function< OUString const, size_t >
{
	inline size_t operator ()(OUString const & rString) const
	{ return size_t(rString.hashCode()); }
};

class UcbConfigurationManager::Map:
	public std::hash_map< OUString,
						  Reference< XContentProviderConfiguration >,
						  StringHash_Impl >
{};

//=========================================================================
UcbConfigurationManager::UcbConfigurationManager(
				const Reference< XMultiServiceFactory >& rxServiceFactory )
: m_xSMgr( rxServiceFactory ),
  m_pMap( new Map )
{
}

//=========================================================================
// virtual
UcbConfigurationManager::~UcbConfigurationManager()
{
	delete m_pMap;
}

//=========================================================================
//
// XInterface methods.
//
//=========================================================================

XINTERFACE_IMPL_3( UcbConfigurationManager,
				   XTypeProvider,
				   XServiceInfo,
				   XContentProviderConfigurationManager );

//=========================================================================
//
// XTypeProvider methods.
//
//=========================================================================

XTYPEPROVIDER_IMPL_3( UcbConfigurationManager,
					  XTypeProvider,
				   	  XServiceInfo,
				   	  XContentProviderConfigurationManager );

//=========================================================================
//
// XServiceInfo methods.
//
//=========================================================================

XSERVICEINFO_IMPL_1( UcbConfigurationManager,
					 OUString::createFromAscii( "UcbConfigurationManager" ),
					 OUString::createFromAscii( UCB_CONFIG_SERVICE_NAME ) );

//=========================================================================
//
// Service factory implementation.
//
//=========================================================================

ONE_INSTANCE_SERVICE_FACTORY_IMPL( UcbConfigurationManager );

//=========================================================================
//
// XContentProviderConfigurationManager methods.
//
//=========================================================================

// virtual
Reference< XContentProviderConfiguration > SAL_CALL
UcbConfigurationManager::queryContentProviderConfiguration( const OUString&
															    Key )
	throw( RuntimeException )
{
	osl::MutexGuard aGuard(m_aMutex);

	Map::iterator aIt(m_pMap->find(Key));
	if (aIt == m_pMap->end())
		aIt = m_pMap->insert(Map::value_type(Key,
											 new Configuration_Impl(m_xSMgr,
																	Key))).
			              first;
	return aIt->second;
}

