/*************************************************************************
 *
 *  $RCSfile: remoteaccessmanager.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:07 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CHAOS_UCB_REMOTEACCESSMANAGER_HXX_
#include <remoteaccessmanager.hxx>
#endif

#if defined SOLARIS // must include <string> before <algorithm>...
#include <string>
#endif // SOLARIS

#include <algorithm>

#ifndef _VOS_DIAGNOSE_H_ 
#include <vos/diagnose.hxx>
#endif

using namespace chaos_ucb;
using namespace com::sun::star;

//============================================================================
//
//  RemoteAccessManager
//
//============================================================================

// virtual
uno::Any SAL_CALL RemoteAccessManager::queryInterface(uno::Type const & rType)
	throw (uno::RuntimeException)
{
	uno::Any
		aRet(cppu::queryInterface(
			     rType,
				 static_cast< ucb::XContentProviderManager * >(this)));
	return aRet.hasValue() ? aRet : OWeakObject::queryInterface(rType);
}

//============================================================================
// virtual
void SAL_CALL RemoteAccessManager::acquire() throw ()
{
	OWeakObject::acquire();
}

//============================================================================
// virtual
void SAL_CALL RemoteAccessManager::release() throw ()
{
	OWeakObject::release();
}

//============================================================================
// virtual
uno::Reference< ucb::XContentProvider > SAL_CALL
RemoteAccessManager::registerContentProvider(
	uno::Reference< ucb::XContentProvider > const & rProvider,
	rtl::OUString const & rScheme,
	sal_Bool bReplaceExisting)
	throw (ucb::DuplicateProviderException, uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	HashMap::data_type & rEntry = m_aMap[rScheme];
	if (!(rEntry.empty() || bReplaceExisting))
		throw ucb::DuplicateProviderException();

	uno::Reference< ucb::XContentProvider > xPrevious;
	if (!rEntry.empty())
		xPrevious = rEntry.back();

	rEntry.push_back(rProvider);

	return xPrevious;
}

//============================================================================
// virtual
void SAL_CALL
RemoteAccessManager::deregisterContentProvider(
	uno::Reference< ucb::XContentProvider > const & rProvider,
	rtl::OUString const & rScheme)
	throw (uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	HashMap::iterator aIt1(m_aMap.find(rScheme));
	if (aIt1 != m_aMap.end())
	{
		HashMap::data_type & rEntry = aIt1->second;

		HashMap::data_type::iterator aEnd(rEntry.end());
		HashMap::data_type::iterator
			aIt2(std::find(rEntry.begin(), aEnd, rProvider));
		if (aIt2 != aEnd)
		{
			rEntry.erase(aIt2);

			if (rEntry.empty())
				m_aMap.erase(aIt1);
		}
	}
}

//============================================================================
// virtual
uno::Sequence< ucb::ContentProviderInfo > SAL_CALL
RemoteAccessManager::queryContentProviders()
	throw (uno::RuntimeException)
{
	VOS_ASSERT(false);
	return uno::Sequence< ucb::ContentProviderInfo >();
}

//============================================================================
// virtual
uno::Reference< ucb::XContentProvider > SAL_CALL
RemoteAccessManager::queryContentProvider(rtl::OUString const & rScheme)
	throw (uno::RuntimeException)
{
	VOS_ASSERT(false);
	return 0;
}

