/*************************************************************************
 *
 *  $RCSfile: rmtconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _UCPRMT_RMTCONN_HXX_
#include <rmtconn.hxx>
#endif

#ifndef _COM_SUN_STAR_BRIDGE_XUNOURLRESOLVER_HPP_
#include <com/sun/star/bridge/XUnoUrlResolver.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_XCONTENTPROVIDER_HPP_
#include <com/sun/star/ucb/XContentProvider.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_XREMOTECONTENTPROVIDERSUPPLIER_HPP_
#include <com/sun/star/ucb/XRemoteContentProviderSupplier.hpp>
#endif
#ifndef _RTL_USTRBUF_HXX_
#include <rtl/ustrbuf.hxx>
#endif

#ifndef _UCPRMT_RMTCONT_HXX_
#include <rmtcont.hxx>
#endif
#ifndef _UCPRMT_RMTLID_HXX_
#include <rmtlid.hxx>
#endif

// Definitions for RegexpToBoolMap and RegexpToStringMap (Solaris wouldn't
// find explicit template instantiations for these in another compilation
// unit...):
#ifndef _UCB_REGEXPMAP_TPT_
#include <regexpmap.tpt>
#endif

using namespace com::sun;
using namespace com::sun::star;
using namespace ucprmt;

//============================================================================
//
//  ConnectionMap
//
//============================================================================

void ConnectionMap::releaseConnection(Connection * pConnection) throw ()
{
	VOS_ASSERT(pConnection);

	osl::MutexGuard aGuard(m_aMutex);

	if (osl_decrementInterlockedCount(&pConnection->m_nRefCount) == 0)
	{
		HashMap::iterator aEnd(m_aProviderToConnectionMap.end());
		for (HashMap::iterator aIt(m_aProviderToConnectionMap.begin());
			 aIt != aEnd; ++aIt)
			if (aIt->second == pConnection)
			{
				m_aProviderToConnectionMap.erase(aIt);
				break;
			}
		delete pConnection;
	}
}

//============================================================================
ConnectionMap::ConnectionMap(
	uno::Reference< lang::XMultiServiceFactory > const & rTheFactory):
	m_xFactory(rTheFactory)
{
	VOS_ASSERT(m_xFactory.is());
}

//============================================================================
vos::ORef< Connection > ConnectionMap::getConnection(rtl::OUString const &
													     rRemoteProviderURL)
{
	VOS_ASSERT(rRemoteProviderURL.getLength() != 0);

	HashMap::const_iterator
		aIt(m_aProviderToConnectionMap.find(rRemoteProviderURL));
	if (aIt == m_aProviderToConnectionMap.end())
		aIt = m_aProviderToConnectionMap.
			          insert(HashMap::value_type(rRemoteProviderURL,
												 new Connection(
													     m_xFactory, this,
														 rRemoteProviderURL,
														 m_aUUIDSupplier))).
			      first;
	return aIt->second;
}

//============================================================================
vos::ORef< Connection >
ConnectionMap::find(uno::Reference< star::ucb::XContentIdentifier > const &
					    rIdentifier)
	throw (uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	// Create from rIdentifier the uno URL of a remote UCB (or an empty
	// string, denoting bad rIdentifier):
	rtl::OUString aRemoteProviderURL;
	if (rIdentifier.is()
		&& rIdentifier->getContentProviderScheme().
		                    compareToAscii("vnd.sun.star.ucb")
		       == 0)
	{
		rtl::OUString aURI(rIdentifier->getContentIdentifier());
		VOS_ASSERT(aURI.compareToAscii(RTL_CONSTASCII_STRINGPARAM(
			                               "vnd.sun.star.ucb"))
				       == 0);
		sal_Int32 nDelim
			= aURI.indexOf(';', RTL_CONSTASCII_LENGTH("vnd.sun.star.ucb:"));
		if (nDelim > 0)
		{
			aRemoteProviderURL = rtl::OUString::createFromAscii("uno:");
			aRemoteProviderURL
				+= aURI.
				       copy(RTL_CONSTASCII_LENGTH("vnd.sun.star.ucb:"),
							nDelim
							    - RTL_CONSTASCII_LENGTH("vnd.sun.star.ucb:"));
			aRemoteProviderURL
				+= rtl::OUString::createFromAscii(";urp;UCB.Factory");
			//@@@ normalize aRemoteProviderURL
		}
	}

	// Map from uno URL to appropriate Connection object:
	return aRemoteProviderURL.getLength() == 0 ?
		       vos::ORef< Connection >() : getConnection(aRemoteProviderURL);
}

//============================================================================
//
//  Connection
//
//============================================================================

uno::Reference< star::ucb::XContentProvider > Connection::getRemoteProvider()
	throw (connection::NoConnectException,
		   connection::ConnectionSetupException,
		   lang::IllegalArgumentException, uno::RuntimeException)
{
	if (!m_xRemoteProvider.is())
	{
		uno::Reference< lang::XMultiServiceFactory > xRemoteFactory;
		sal_Int32 nIdentifierEnd;
		if (m_aRemoteProviderURL.compareToAscii("uno:SELF;urp;UCB.Factory")
			    == 0)
			xRemoteFactory = m_xFactory;
		else if (m_aRemoteProviderURL.compareToAscii("uno:FACTORY,id=",
													 RTL_CONSTASCII_LENGTH(
														 "uno:FACTORY,id="))
				     == 0
				 && (nIdentifierEnd
					         = m_aRemoteProviderURL.
					               indexOf(';',
										   RTL_CONSTASCII_LENGTH(
											   "uno:FACTORY,id=")))
				        >= 0
				 && m_aRemoteProviderURL.
				            copy(nIdentifierEnd).
				                compareToAscii(";urp;UCB.Factory")
				        == 0)
		{
			rtl::OUString
				aIdentifier(m_aRemoteProviderURL.
							    copy(RTL_CONSTASCII_LENGTH("uno:FACTORY,id="),
									 nIdentifierEnd
									     - RTL_CONSTASCII_LENGTH(
											 "uno:FACTORY,id=")));
				//@@@ decode aIdentifier

			uno::Reference< star::ucb::XRemoteContentProviderSupplier >
				xSupplier;
			try
			{
				xSupplier
					= uno::Reference<
					          star::ucb::XRemoteContentProviderSupplier >(
						  m_xFactory->
					          createInstance(
								  rtl::OUString::createFromAscii(
									  "com.sun.star.ucb."
									      "RemoteContentProviderAcceptor")),
						  uno::UNO_QUERY);
			}
			catch (uno::RuntimeException const &) { throw; }
			catch (uno::Exception const &) {}
			if (!xSupplier.is())
				throw uno::RuntimeException();
					// no service, or service lacks interface

			xRemoteFactory
				= xSupplier->queryRemoteContentProvider(aIdentifier);
			if (!xRemoteFactory.is())
				throw connection::NoConnectException();
		}
		else
		{
			uno::Reference< bridge::XUnoUrlResolver > xResolver;
			try
			{
				xResolver
					= uno::Reference< bridge::XUnoUrlResolver >(
						  m_xFactory->
					          createInstance(
								  rtl::OUString::createFromAscii(
									  "com.sun.star.bridge.UnoUrlResolver")),
						  uno::UNO_QUERY);
			}
			catch (uno::RuntimeException const &) { throw; }
			catch (uno::Exception const &) {}
			if (!xResolver.is())
				throw uno::RuntimeException();
					// no service, or service lacks interface

			xRemoteFactory = uno::Reference< lang::XMultiServiceFactory >(
				                 xResolver->resolve(m_aRemoteProviderURL),
								 uno::UNO_QUERY);
			if (!xRemoteFactory.is())
				throw uno::RuntimeException();
					// bad resolve() result, or service lacks interface

			try
			{
				xRemoteFactory
					= uno::Reference< lang::XMultiServiceFactory >(
						  xRemoteFactory->
					          createInstance(
								  rtl::OUString::createFromAscii(
									  "com.sun.star.lang.ServiceManager")),
						  uno::UNO_QUERY);
			}
			catch (uno::RuntimeException const &) { throw; }
			catch (uno::Exception const &) {}
			if (!xRemoteFactory.is())
				throw uno::RuntimeException();
					// no service, or service lacks interface
		}

		try
		{
			//@@@ The remote broker service should be created using
			// createInstanceWithArguments(), specifing whether and how the
			// broker should be configured.  Not supplying these arguments
			// implies that the remote broker service must already be
			// instantiated and configured when this call is made:
			m_xRemoteProvider
				= uno::Reference< star::ucb::XContentProvider >(
					  xRemoteFactory->
					      createInstance(
							  rtl::OUString::createFromAscii(
								  "com.sun.star.ucb.UniversalContentBroker")),
					  uno::UNO_QUERY);
		}
		catch (uno::RuntimeException const &) { throw; }
		catch (uno::Exception const &) {}
		if (!m_xRemoteProvider.is())
			throw uno::RuntimeException();
				// no service, or service lacks interface
	}

	return m_xRemoteProvider;
}

//============================================================================
uno::Reference< star::ucb::XContentIdentifier >
Connection::mapToRemoteIdentifier(
	uno::Reference< star::ucb::XContentIdentifier > const & rIdentifier) const
	throw (uno::RuntimeException)
{
	if (isLocalIdentifier(rIdentifier))
		return static_cast< LocalContentIdentifier * >(rIdentifier.get())->
			       getRemoteIdentifier();
	else if (rIdentifier.is())
	{
		rtl::OUString aURI(rIdentifier->getContentIdentifier());
		if (aURI.getLength() != 0)
		{
			bool bTranslated;
			if (m_aLocalToRemoteURIs.map(aURI, &aURI, &bTranslated)
				&& bTranslated)
				return new ::ucb::ContentIdentifier(aURI);
		}
	}
	return rIdentifier;
}

//============================================================================
uno::Reference< star::ucb::XContentIdentifier >
Connection::normalizeLocalIdentifier(
	uno::Reference< star::ucb::XContentIdentifier > const & rIdentifier) const
	throw (uno::RuntimeException)
{
	if (rIdentifier.is())
	{
		rtl::OUString aURI(rIdentifier->getContentIdentifier());
		if (aURI.compareTo(m_aLocalURIPrefix, m_aLocalURIPrefix.getLength())
			    == 0
			&& m_aRemoteToLocalURIs.map(aURI.copy(m_aLocalURIPrefix.
												      getLength()),
										&aURI))
			return new ::ucb::ContentIdentifier(aURI);
	}
	return rIdentifier;
}

//============================================================================
bool
Connection::isLocalIdentifier(
	uno::Reference< star::ucb::XContentIdentifier > const & rIdentifier) const
	throw (uno::RuntimeException)
{
	uno::Reference< lang::XUnoTunnel > xTunnel(rIdentifier, uno::UNO_QUERY);
	return xTunnel.is()
		   && xTunnel->getSomething(uno::Sequence< sal_Int8 >(m_aUUID.
															      getData(),
															  UUID::LENGTH))
		          == 1;
}

//============================================================================
bool Connection::isLocalContent(uno::Reference< star::ucb::XContent > const &
								    rContent) const
	throw (uno::RuntimeException)
{
	uno::Reference< lang::XUnoTunnel > xTunnel(rContent, uno::UNO_QUERY);
	return xTunnel.is()
		   && xTunnel->getSomething(uno::Sequence< sal_Int8 >(m_aUUID.
															      getData(),
															  UUID::LENGTH))
		          == 1;
}

//============================================================================
uno::Reference< star::ucb::XContent >
Connection::getLocalContent(
	uno::Reference< star::ucb::XContentIdentifier > const & rLocalIdentifier,
	uno::Reference< star::ucb::XContentIdentifier > const & rRemoteIdentifier)
	throw (connection::NoConnectException,
		   connection::ConnectionSetupException,
		   lang::IllegalArgumentException,
		   star::ucb::IllegalIdentifierException, uno::RuntimeException)
{
	return Content::create(m_xFactory, this, rLocalIdentifier,
						   getRemoteContent(rRemoteIdentifier));
}

//============================================================================
uno::Reference< star::ucb::XContent >
Connection::getRemoteContent(
	uno::Reference< star::ucb::XContentIdentifier > const & rRemoteIdentifier)
	throw (connection::NoConnectException,
		   connection::ConnectionSetupException,
		   lang::IllegalArgumentException,
		   star::ucb::IllegalIdentifierException, uno::RuntimeException)
{
	return getRemoteProvider()->queryContent(rRemoteIdentifier);
}

//============================================================================
Connection::Connection(uno::Reference< lang::XMultiServiceFactory > const &
						   rTheFactory,
					   vos::ORef< ConnectionMap > const & rTheConnectionMap,
					   rtl::OUString const & rTheRemoteProviderURL,
					   UUIDSupplier & rUUIDSupplier):
	m_xFactory(rTheFactory),
	m_xConnectionMap(rTheConnectionMap),
	m_aRemoteProviderURL(rTheRemoteProviderURL),
	m_nRefCount(0),
	m_aUUID(rUUIDSupplier)
{
	VOS_ASSERT(m_xFactory.is());

	// Create m_aLocalURIPrefix from m_aRemoteProviderURL.  If
	// m_aRemoteProviderURL (which must be canonic) is of the form
	//
	//   "uno:" connection ";" protocol ";UCB.Factory"
	//
	// then m_aLocalURIPrefix will be of the form
	//
	//   "vnd.sun.star.ucb:" connection ";"
	VOS_ASSERT(m_aRemoteProviderURL.
			           compareToAscii(RTL_CONSTASCII_STRINGPARAM("uno:"))
			       == 0);
	sal_Int32 nDelim
		= m_aRemoteProviderURL.indexOf(';', RTL_CONSTASCII_LENGTH("uno:"));
	VOS_ASSERT(nDelim > 0);
	m_aLocalURIPrefix = rtl::OUString::createFromAscii("vnd.sun.star.ucb:");
	m_aLocalURIPrefix
		+= m_aRemoteProviderURL.copy(RTL_CONSTASCII_LENGTH("uno:"),
								nDelim + 1 - RTL_CONSTASCII_LENGTH("uno:"));

	{
		rtl::OUStringBuffer aTemplate;
		aTemplate.append(sal_Unicode('"'));
		aTemplate.append(m_aLocalURIPrefix);
		aTemplate.appendAscii(RTL_CONSTASCII_STRINGPARAM("\"(.*)->\\1"));
		m_aLocalToRemoteURIs.add(aTemplate.makeStringAndClear(), true, false);
	}

	{
		rtl::OUStringBuffer aTemplate;
		aTemplate.appendAscii(RTL_CONSTASCII_STRINGPARAM("(.*)->\""));
		aTemplate.append(m_aLocalURIPrefix);
		aTemplate.appendAscii(RTL_CONSTASCII_STRINGPARAM("\"\\1"));
		m_aRemoteToLocalURIs.add(aTemplate.makeStringAndClear(), true, false);
	}

	{
		rtl::OUStringBuffer aTemplate;
		aTemplate.append(sal_Unicode('"'));
		aTemplate.append(m_aLocalURIPrefix);
		aTemplate.appendAscii(RTL_CONSTASCII_STRINGPARAM("\".*"));
		m_aRemoteToLocalURIs.add(aTemplate.makeStringAndClear(), true, false);
	}
}

//============================================================================
// virtual
uno::Any SAL_CALL Connection::queryInterface(uno::Type const & rType)
	throw (uno::RuntimeException)
{
	return cppu::queryInterface(rType,
								static_cast< uno::XInterface * >(this),
								static_cast< star::ucb::XContentProvider * >(
									this));
}

//============================================================================
// virtual
void SAL_CALL Connection::acquire() throw ()
{
	osl_incrementInterlockedCount(&m_nRefCount);
}

//============================================================================
// virtual
void SAL_CALL Connection::release() throw ()
{
	m_xConnectionMap->releaseConnection(this);
}

//============================================================================
// virtual
uno::Reference< star::ucb::XContent > SAL_CALL
Connection::queryContent(
	uno::Reference< star::ucb::XContentIdentifier > const & rIdentifier)
	throw (star::ucb::IllegalIdentifierException, uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	try
	{
		return getLocalContent(normalizeLocalIdentifier(rIdentifier),
							   mapToRemoteIdentifier(rIdentifier));
	}
	catch (connection::NoConnectException const &)
	{
		return 0;
	}
	catch (connection::ConnectionSetupException const &)
	{
		return 0;
	}
	catch (lang::IllegalArgumentException const &)
	{
		throw star::ucb::IllegalIdentifierException();
	}
}

//============================================================================
// virtual
sal_Int32 SAL_CALL
Connection::compareContentIds(
	uno::Reference< star::ucb::XContentIdentifier > const & rId1,
	uno::Reference< star::ucb::XContentIdentifier > const & rId2)
	throw (uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	try
	{
		return getRemoteProvider()->
			       compareContentIds(mapToRemoteIdentifier(rId1),
									 mapToRemoteIdentifier(rId2));
	}
	catch (connection::NoConnectException const &) {}
	catch (connection::ConnectionSetupException const &) {}
	catch (lang::IllegalArgumentException const &) {}

	rtl::OUString aURI1;
	if (rId1.is())
		aURI1 = rId1->getContentIdentifier();
	rtl::OUString aURI2;
	if (rId2.is())
		aURI2 = rId2->getContentIdentifier();
	return aURI1.compareTo(aURI2);
}

//============================================================================
bool Connection::addLocalURITemplate(rtl::OUString const & rTemplate,
									 bool bOverwrite)
	throw (lang::IllegalArgumentException, uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	//@@@ no commit-or-rollback semantics:
	rtl::OUString aReverse;
	bool bAdded
		= m_aLocalToRemoteURIs.add(rTemplate, true, bOverwrite, &aReverse);
	if (bAdded)
		m_aRemoteToLocalURIs.add(aReverse, true, true);
	return bAdded;
}

//============================================================================
bool Connection::removeLocalURITemplate(rtl::OUString const & rTemplate)
	throw (lang::IllegalArgumentException, uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	rtl::OUString aReverse;
	RegexpToBoolMap::iterator
		aIt(m_aLocalToRemoteURIs.find(rTemplate, &aReverse));
	if (aIt == m_aLocalToRemoteURIs.end())
		return false;

	m_aLocalToRemoteURIs.erase(aIt);
	aIt = m_aRemoteToLocalURIs.find(rTemplate);
	if (aIt != m_aRemoteToLocalURIs.end())
		m_aRemoteToLocalURIs.erase(aIt);
	return true;
}

//============================================================================
rtl::OUString Connection::mapToLocalURI(rtl::OUString const & rRemoteURI)
	throw (uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	if (rRemoteURI.getLength() != 0)
	{
		rtl::OUString aLocalURI;
		bool bTranslated;
		if (m_aRemoteToLocalURIs.map(rRemoteURI, &aLocalURI, &bTranslated)
			&& bTranslated)
			return aLocalURI;
	}
	return rRemoteURI;
}

//============================================================================
uno::Reference< star::ucb::XContentIdentifier >
Connection::mapToLocalIdentifier(
	uno::Reference< star::ucb::XContentIdentifier > const & rRemoteIdentifier)
	throw (uno::RuntimeException)
{
	if (!rRemoteIdentifier.is())
		return rRemoteIdentifier;

	rtl::OUString aURI(rRemoteIdentifier->getContentIdentifier());
	if (aURI.getLength())
	{
		osl::MutexGuard aGuard(m_aMutex);
		m_aRemoteToLocalURIs.map(aURI, &aURI);
	}

	return new LocalContentIdentifier(this, aURI, rRemoteIdentifier);
}

//============================================================================
uno::Reference< star::ucb::XContent >
Connection::mapToLocalContent(uno::Reference< star::ucb::XContent > const &
							      rRemoteContent)
	throw (uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	uno::Reference< star::ucb::XContent > xLocalContent;
	if (rRemoteContent.is())
	{
		uno::Reference< star::ucb::XContentIdentifier >
			xRemoteIdentifier(rRemoteContent->getIdentifier());
		if (xRemoteIdentifier.is())
		{
			rtl::OUString aURI(xRemoteIdentifier->getContentIdentifier());
			if (aURI.getLength())
				m_aRemoteToLocalURIs.map(aURI, &aURI);
			uno::Reference< star::ucb::XContentIdentifier >
				xLocalIdentifier(new LocalContentIdentifier(
					                     this, aURI, xRemoteIdentifier));

			try
			{
				xLocalContent
					= getLocalContent(xLocalIdentifier, xRemoteIdentifier);
			}
			catch (connection::NoConnectException const &) {}
			catch (connection::ConnectionSetupException const &) {}
			catch (lang::IllegalArgumentException const &) {}
			catch (star::ucb::IllegalIdentifierException const &) {}
		}
	}
	return xLocalContent.is() ? xLocalContent : rRemoteContent;
}

//============================================================================
uno::Reference< star::ucb::XContent >
Connection::mapToRemoteContent(uno::Reference< star::ucb::XContent > const &
							       rContent)
	throw (uno::RuntimeException)
{
	osl::MutexGuard aGuard(m_aMutex);

	uno::Reference< star::ucb::XContent > xRemoteContent(rContent);
	if (isLocalContent(rContent))
		try
		{
			xRemoteContent
				= getRemoteContent(mapToRemoteIdentifier(
					                   rContent->getIdentifier()));
		}
		catch (connection::NoConnectException const &) {}
		catch (connection::ConnectionSetupException const &) {}
		catch (lang::IllegalArgumentException const &) {}
		catch (star::ucb::IllegalIdentifierException const &) {}
	return xRemoteContent;
}
