/*************************************************************************
 *
 *  $RCSfile: rmtconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _UCPRMT_RMTCONN_HXX_
#define _UCPRMT_RMTCONN_HXX_

#include <cstddef>
#include <functional>
#include <hash_map>
#include <list>

#ifndef _COM_SUN_STAR_UCB_XCONTENTPROVIDER_HPP_
#include <com/sun/star/ucb/XContentProvider.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_ 
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_ 
#include <com/sun/star/uno/Sequence.h>
#endif
#ifndef _OSL_INTERLOCK_H_
#include <osl/interlck.h>
#endif
#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif
#ifndef _RTL_USTRING_HXX 
#include <rtl/ustring.hxx>
#endif
#ifndef _SAL_TYPES_H_ 
#include <sal/types.h>
#endif
#ifndef _VOS_REF_HXX_ 
#include <vos/ref.hxx>
#endif
#ifndef _VOS_REFERNCE_HXX_
#include <vos/refernce.hxx>
#endif

#ifndef _UCPRMT_RMTRMAP_HXX_
#include <rmtrmap.hxx>
#endif
#ifndef _UCPRMT_RMTUUID_HXX_
#include <rmtuuid.hxx>
#endif

namespace com { namespace sun { namespace star {
	namespace connection {
		class ConnectionSetupException;
		class NoConnectException;
	}
	namespace lang {
		class IllegalArgumentException;
		class XMultiServiceFactory;
	}
	namespace ucb {
		class IllegalIdentifierException;
		class XContent;
		class XContentIdentifier;
	}
	namespace uno { class RuntimeException; }
} } }

namespace ucprmt {

class Connection;
class ContentProvider;

//============================================================================
class ConnectionMap: public vos::OReference
{
	friend Connection; // to access releaseConnection()

public:
	ConnectionMap(com::sun::star::uno::Reference<
				          com::sun::star::lang::XMultiServiceFactory > const &
				      rTheFactory);

	vos::ORef< Connection > getConnection(rtl::OUString const &
										      rRemoteProviderURL);

	vos::ORef< Connection >
	find(com::sun::star::uno::Reference<
		         com::sun::star::ucb::XContentIdentifier > const &
		     rIdentifier)
		throw (com::sun::star::uno::RuntimeException);

	osl::Mutex & getMutex() { return m_aMutex; }

private:
	struct StringHash:
		public std::unary_function< rtl::OUString const, size_t >
	{
		inline size_t operator ()(rtl::OUString const & rString) const
		{ return size_t(rString.hashCode()); }
	};

	typedef std::hash_map< rtl::OUString, Connection *, StringHash > HashMap;

	com::sun::star::uno::Reference<
	        com::sun::star::lang::XMultiServiceFactory >
	    m_xFactory;
	UUIDSupplier m_aUUIDSupplier;
	osl::Mutex m_aMutex;
	RegexpToStringMap m_aURIToProviderMap;
	HashMap m_aProviderToConnectionMap;

	void releaseConnection(Connection * pConnection) throw ();
};

//============================================================================
class Connection: public com::sun::star::ucb::XContentProvider
{
	friend ConnectionMap; // to access m_nRefCount

public:
	Connection(com::sun::star::uno::Reference<
			           com::sun::star::lang::XMultiServiceFactory > const &
			       rTheFactory,
			   vos::ORef< ConnectionMap > const & rTheConnectionMap,
			   rtl::OUString const & rTheRemoteProviderURL,
			   UUIDSupplier & rUUIDSupplier);

	inline ~Connection() throw () {}

	//========================================================================
	// com::sun::star::uno::XInterface:

	virtual com::sun::star::uno::Any SAL_CALL
	queryInterface(com::sun::star::uno::Type const & rType)
		throw (com::sun::star::uno::RuntimeException);

    virtual void SAL_CALL acquire() throw ();

    virtual void SAL_CALL release() throw ();

	//========================================================================
	// com::sun::star::ucb::XContentProvider:

	virtual
	com::sun::star::uno::Reference< com::sun::star::ucb::XContent > SAL_CALL
	queryContent(com::sun::star::uno::Reference<
				         com::sun::star::ucb::XContentIdentifier > const &
				     rIdentifier)
		throw (com::sun::star::ucb::IllegalIdentifierException,
			   com::sun::star::uno::RuntimeException);

	virtual sal_Int32 SAL_CALL
	compareContentIds(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rId1,
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rId2)
		throw (com::sun::star::uno::RuntimeException);

	//========================================================================
	// Implementation:

	com::sun::star::uno::Reference<
	    com::sun::star::lang::XMultiServiceFactory >
	getFactory() const { return m_xFactory; }

	bool addLocalURITemplate(rtl::OUString const & rTemplate, bool bOverwrite)
		throw (com::sun::star::lang::IllegalArgumentException,
			   com::sun::star::uno::RuntimeException);

	bool removeLocalURITemplate(rtl::OUString const & rTemplate)
		throw (com::sun::star::lang::IllegalArgumentException,
			   com::sun::star::uno::RuntimeException);

	rtl::OUString mapToLocalURI(rtl::OUString const & rRemoteURI)
		throw (com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentIdentifier >
	mapToLocalIdentifier(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rRemoteIdentifier)
		throw (com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContent >
	mapToLocalContent(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContent > const &
		    rRemoteContent)
		throw (com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContent >
	mapToRemoteContent(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContent > const &
		    rContent)
		throw (com::sun::star::uno::RuntimeException);

	inline bool isUUID(com::sun::star::uno::Sequence< sal_Int8 > const &
					       rIdentifier) const
		throw (com::sun::star::uno::RuntimeException)
	{ return m_aUUID.equals(rIdentifier); }

private:
	com::sun::star::uno::Reference<
	        com::sun::star::lang::XMultiServiceFactory >
	    m_xFactory;
	vos::ORef< ConnectionMap > m_xConnectionMap;
	com::sun::star::uno::Reference< com::sun::star::ucb::XContentProvider >
	    m_xRemoteProvider;
	rtl::OUString m_aRemoteProviderURL;
	rtl::OUString m_aLocalURIPrefix;
	RegexpToBoolMap m_aLocalToRemoteURIs;
	RegexpToBoolMap m_aRemoteToLocalURIs;
	osl::Mutex m_aMutex;
	oslInterlockedCount m_nRefCount;
	UUID m_aUUID;

	Connection(Connection const &); // not implemented
	Connection & operator =(Connection const &); // not implemented

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentProvider >
	getRemoteProvider()
		throw (com::sun::star::connection::NoConnectException,
			   com::sun::star::connection::ConnectionSetupException,
			   com::sun::star::lang::IllegalArgumentException,
			   com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentIdentifier >
	mapToRemoteIdentifier(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rIdentifier) const
		throw (com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContentIdentifier >
	normalizeLocalIdentifier(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rIdentifier) const
		throw (com::sun::star::uno::RuntimeException);

	bool isLocalIdentifier(
		     com::sun::star::uno::Reference<
			         com::sun::star::ucb::XContentIdentifier > const &
			     rIdentifier) const
		throw (com::sun::star::uno::RuntimeException);

	bool isLocalContent(com::sun::star::uno::Reference<
						        com::sun::star::ucb::XContent > const &
						    rContent) const
		throw (com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContent >
	getLocalContent(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rLocalIdentifier,
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rRemoteIdentifier)
		throw (com::sun::star::connection::NoConnectException,
			   com::sun::star::connection::ConnectionSetupException,
			   com::sun::star::lang::IllegalArgumentException,
			   com::sun::star::ucb::IllegalIdentifierException,
			   com::sun::star::uno::RuntimeException);

	com::sun::star::uno::Reference< com::sun::star::ucb::XContent >
	getRemoteContent(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentIdentifier > const &
		    rRemoteIdentifier)
		throw (com::sun::star::connection::NoConnectException,
			   com::sun::star::connection::ConnectionSetupException,
			   com::sun::star::lang::IllegalArgumentException,
			   com::sun::star::ucb::IllegalIdentifierException,
			   com::sun::star::uno::RuntimeException);
};

}

#endif // _UCPRMT_RMTCONN_HXX_

