/*************************************************************************
 *
 *  $RCSfile: distributor.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <exception>
#include <new>

#ifndef _COM_SUN_STAR_BRIDGE_XUNOURLRESOLVER_HPP_
#include <com/sun/star/bridge/XUnoUrlResolver.hpp>
#endif
#ifndef _COM_SUN_STAR_CONNECTION_CONNECTIONSETUPEXCEPTION_HPP_
#include <com/sun/star/connection/ConnectionSetupException.hpp>
#endif
#ifndef _COM_SUN_STAR_CONNECTION_NOCONNECTEXCEPTION_HPP_
#include <com/sun/star/connection/NoConnectException.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_XREMOTECONTENTPROVIDERDISTRIBUTOR_HPP_
#include <com/sun/star/ucb/XRemoteContentProviderDistributor.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_EXCEPTION_HPP_
#include <com/sun/star/uno/Exception.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_HXX_
#include <com/sun/star/uno/Reference.hxx>
#endif
#ifndef _CPPUHELPER_SERVICEFACTORY_HXX_
#include <cppuhelper/servicefactory.hxx>
#endif
#ifndef _RTL_STRING_HXX_
#include <rtl/string.hxx>
#endif
#ifndef _RTL_TEXTENC_H_
#include <rtl/textenc.h>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _UNOTOOLS_REGPATHHELPER_HXX_
#include <unotools/regpathhelper.hxx>
#endif

using namespace com::sun::star;

//============================================================================
//
//  ProgramException
//
//============================================================================

namespace {

class ProgramException: public std::exception
{
public:
	inline ProgramException(rtl::OString const & rWhat) throw ()
	{ try { m_aWhat = rWhat; } catch (...) {} }
		// 'try: m_aWhat(rWhat) {} catch (...) {}' won't work on MSVC++6

	inline ProgramException(ProgramException const & rOther) throw ()
	{ try { m_aWhat = rOther.m_aWhat; } catch (...) {} }
		// 'try: m_aWhat(rOther.m_aWhat) {} catch (...) {}' won't work on
		// MSVC++6

	virtual inline ~ProgramException() throw ()
	{} // 'try {} catch (...) {}' won't work on MSVC++6

	inline ProgramException &
	operator =(ProgramException const & rOther) throw ()
	{ try { m_aWhat = rOther.m_aWhat; } catch (...) {} return *this; }
		// 'try { m_aWhat = rOther.m_aWhat; return *this; } catch (...) {}'
		// won't work on MSVC++6

	virtual inline char const * what() const throw ()
	{ return m_aWhat.getStr(); }
		// 'try { return m_aWhat.getStr(); } catch (...) {}' won't work on
		// MSVC++6

private:
	rtl::OString m_aWhat;
};

}

//============================================================================
//
//  Distributor
//
//============================================================================

namespace {

class Distributor
{
public:
	Distributor(rtl::OUString const & rUrl, rtl::OUString const & rIdentifier)
		throw (ProgramException, uno::RuntimeException, std::bad_alloc);

	~Distributor()
		throw (ProgramException, uno::RuntimeException, std::bad_alloc);

private:
	Distributor(Distributor const &); // not implemented
	Distributor & operator =(Distributor const &); // not implemented

	rtl::OUString m_aUrl;
	uno::Reference< ucb::XRemoteContentProviderDistributor > m_xDistributor;
};

}

//============================================================================
Distributor::Distributor(rtl::OUString const & rUrl,
						 rtl::OUString const & rIdentifier)
	throw (ProgramException, uno::RuntimeException, std::bad_alloc):
	m_aUrl(rUrl)
{
	uno::Reference< lang::XMultiServiceFactory > xLocalFactory;
	try
	{
		xLocalFactory = cppu::createRegistryServiceFactory(
			                utl::getPathToSystemRegistry(),
							utl::getPathToUserRegistry());
	}
	catch (uno::RuntimeException const &) { throw; }
	catch (uno::Exception const &) {}
	if (!xLocalFactory.is())
		throw ProgramException(
			      rtl::OString(
					  RTL_CONSTASCII_STRINGPARAM(
						  "bad createRegistryServiceFactory() result")));

	uno::Reference< bridge::XUnoUrlResolver > xResolver;
	try
	{
		xResolver
			= uno::Reference< bridge::XUnoUrlResolver >(
				  xLocalFactory->
				      createInstance(
						  rtl::OUString(
							  RTL_CONSTASCII_USTRINGPARAM(
								  "com.sun.star.bridge.UnoUrlResolver"))),
				  uno::UNO_QUERY);
	}
	catch (uno::RuntimeException const &) { throw; }
	catch (uno::Exception const &) {}
	if (!xResolver.is())
		throw ProgramException(
			      rtl::OString(
					  RTL_CONSTASCII_STRINGPARAM(
						  "no service, or service lacks interface")));

	uno::Reference< lang::XMultiServiceFactory > xUcbFactory;
	try
	{
		xUcbFactory
			= uno::Reference< lang::XMultiServiceFactory >(
				  uno::Reference< lang::XMultiServiceFactory >(
					  xResolver->
					  	  resolve(
							  rtl::OUString(
								  RTL_CONSTASCII_USTRINGPARAM(
									  "uno:socket,host=localhost,port=8121;"
									  	  "urp;UCB.Factory"))),
					  uno::UNO_QUERY));
	}
	catch (connection::NoConnectException const &)
	{
		throw ProgramException(rtl::OString(RTL_CONSTASCII_STRINGPARAM(
			                                    "NoConnectException")));
	}
	catch (connection::ConnectionSetupException const &)
	{
		throw ProgramException(rtl::OString(RTL_CONSTASCII_STRINGPARAM(
			                                    "ConnectionSetupException")));
	}
	catch (lang::IllegalArgumentException const &)
	{
		throw ProgramException(rtl::OString(RTL_CONSTASCII_STRINGPARAM(
			                                    "IllegalArgumentException")));
	}
	if (!xUcbFactory.is())
		throw
			ProgramException(
				rtl::OString(
					RTL_CONSTASCII_STRINGPARAM(
						"bad resolve() result, or service lacks interface")));

	try
	{
		xUcbFactory
			= uno::Reference< lang::XMultiServiceFactory >(
				  xUcbFactory->
				      createInstance(
						  rtl::OUString(
							  RTL_CONSTASCII_USTRINGPARAM(
								  "com.sun.star.lang.ServiceManager"))),
				  uno::UNO_QUERY);
	}
	catch (uno::RuntimeException const &) { throw; }
	catch (uno::Exception const &) {}
	if (!xUcbFactory.is())
		throw ProgramException(
			      rtl::OString(
					  RTL_CONSTASCII_STRINGPARAM(
						  "no service, or service lacks interface")));

	try
	{
		m_xDistributor
			= uno::Reference< ucb::XRemoteContentProviderDistributor >(
				  xUcbFactory->
				      createInstance(
						  rtl::OUString(
							  RTL_CONSTASCII_USTRINGPARAM(
								  "com.sun.star.ucb."
								  	  "RemoteContentProviderDistributor"))),
				  uno::UNO_QUERY);
	}
	catch (uno::RuntimeException const &) { throw; }
	catch (uno::Exception const &) {}
	if (!m_xDistributor.is())
		throw ProgramException(
			      rtl::OString(
					  RTL_CONSTASCII_STRINGPARAM(
						  "no service, or service lacks interface")));

	bool bSuccess;
	try
	{
		bSuccess
			= m_xDistributor->connectToRemoteAcceptor(m_aUrl, rIdentifier)
			      != false;
	}
	catch (connection::NoConnectException const &)
	{
		throw ProgramException(rtl::OString(RTL_CONSTASCII_STRINGPARAM(
			                                    "NoConnectException")));
	}
	catch (connection::ConnectionSetupException const &)
	{
		throw ProgramException(rtl::OString(RTL_CONSTASCII_STRINGPARAM(
			                                    "ConnectionSetupException")));
	}
	catch (lang::IllegalArgumentException const &)
	{
		throw ProgramException(rtl::OString(RTL_CONSTASCII_STRINGPARAM(
			                                    "IllegalArgumentException")));
	}
	if (!bSuccess)
		throw ProgramException(
			      rtl::OString(
					  RTL_CONSTASCII_STRINGPARAM(
						  "bad connectToRemoteAcceptor() result")));
}

//============================================================================
Distributor::~Distributor()
	throw (ProgramException, uno::RuntimeException, std::bad_alloc)
{
	if (!m_xDistributor->disconnectFromRemoteAcceptor(m_aUrl))
		throw ProgramException(
			      rtl::OString(
					  RTL_CONSTASCII_STRINGPARAM(
						  "bad disconnectFromRemoteAcceptor() result")));
}

//============================================================================
//
//  main
//
//============================================================================

int
#if defined WNT
__cdecl
#endif // WNT
main()
{
	try
	{
		for (;;)
		{
			fputs("Enter URL (empty to quit): ", stdout);
			sal_Char aUrl[1024];
			if (!gets(reinterpret_cast< char * >(aUrl)) || aUrl[0] == '\0')
				break;

			fputs("Enter Identifier: ", stdout);
			sal_Char aIdentifier[1024];
			gets(reinterpret_cast< char * >(aIdentifier));

			try
			{
				Distributor
					aDistributor(rtl::OUString::createFromAscii(aUrl),
								 rtl::OUString::createFromAscii(aIdentifier));

				fputs("Connected... press return to disconnect:", stdout);
				char aLine[1024];
				gets(aLine);
			}
			catch (ProgramException const & rException)
			{
				fprintf(stderr, "Program exception: %s\n", rException.what());
			}
		}
		return EXIT_SUCCESS;
	}
	catch (uno::RuntimeException const & rException)
	{
		fprintf(stderr, "UNO runtime exception: %s\n",
				rtl::OString(rException.Message.getStr(),
							 rException.Message.getLength(),
							 RTL_TEXTENCODING_ISO_8859_1).
				    getStr());
	}
	catch (std::bad_alloc const & rException)
	{
		fprintf(stderr, "Bad allocation exception: %s\n", rException.what());
	}
	catch (uno::Exception const & rException)
	{
		fprintf(stderr, "Unexpected UNO exception: %s\n",
				rtl::OString(rException.Message.getStr(),
							 rException.Message.getLength(),
							 RTL_TEXTENCODING_ISO_8859_1).
				    getStr());
	}
	catch (std::exception const & rException)
	{
		fprintf(stderr, "Unexpected standard C++ exception: %s\n",
				rException.what());
	}
	catch (...)
	{
		fprintf(stderr, "Unexpected unknown exception: %s\n");
	}
	return EXIT_FAILURE;
}
