/*************************************************************************
 *
 *  $RCSfile: perfopen.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif

#ifndef _TOOLS_TIME_HXX //autogen
#include <tools/time.hxx>
#endif

#ifndef _TOOLS_RESMGR_HXX
#include <tools/resmgr.hxx>
#endif

#ifndef _SV_WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif
#ifndef _SV_TOOLBOX_HXX //autogen
#include <vcl/toolbox.hxx>
#endif
#ifndef _SV_EDIT_HXX //autogen
#include <vcl/edit.hxx>
#endif
#ifndef _SVEDIT_HXX //autogen
#include <svtools/svmedit.hxx>
#endif
#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif
#ifndef _SV_HELP_HXX //autogen
#include <vcl/help.hxx>
#endif

#ifndef _SFXINIMGR_HXX
#include <svtools/iniman.hxx>
#endif

#ifndef _SFXLSTNER_HXX //autogen
#include <svtools/lstner.hxx>
#endif

#ifndef _CNTSYS_HXX
#include <chaos/cntsys.hxx>
#endif

#ifndef _CNTAPI_HXX
#include <chaos/cntapi.hxx>
#endif

#ifndef _CNTVWITM_HXX
#include <chaos/cntvwitm.hxx>
#endif

#if defined( WNT ) && defined( PROFILE )

extern "C" {
	void StartCAP();
	void StopCAP();
	void DumpCAP();
};

#define S_CAP	StartCAP();
#define E_CAP	StopCAP(); DumpCAP();

#endif

/*========================================================================
 *
 * CntDemoClient.
 *
 *=======================================================================*/

class CntDemoClient :
	public virtual SvRefBase,
	public SfxListener,
	public SfxBroadcaster
{
	CntAnchorRef    m_xAnchor;
	MultiLineEdit*	m_pOutEdit;

protected:
	CntDemoClient(CntAnchorRef &rAnchor, MultiLineEdit* pOutEdit );
	virtual ~CntDemoClient();

public:
	void Output( const String& rText );
	static CntDemoClient* create(
		CntDemoClient *pParent,
		const String  &rURL,
		MultiLineEdit *pOutEdit);

	void dispose(void);

	void open(void);

	void started (void);
	void done    (void);
	void canceled(void);
	void result  (INT32 nAction, const String& rResult);
	virtual void Notify(
		SfxBroadcaster& rBC,
		const SfxHint& rHint);
};

/*========================================================================
 *
 * CntDemoClientRef.
 *
 *=======================================================================*/

SV_DECL_IMPL_REF( CntDemoClient );

/*========================================================================
 *
 * CntDemoClient implementation.
 *
 *=======================================================================*/

//-------------------------------------------------------------------------
CntDemoClient::CntDemoClient(
	CntAnchorRef& rAnchor,
	MultiLineEdit* pOutEdit)
: m_xAnchor( rAnchor ),
  m_pOutEdit( pOutEdit )
{
}

//-------------------------------------------------------------------------
// virtual
CntDemoClient::~CntDemoClient()
{
}

//-------------------------------------------------------------------------
// static
CntDemoClient* CntDemoClient::create(
	CntDemoClient* pParent,
	const String&  rURL,
	MultiLineEdit* pOutEdit )
{
	if ( !rURL.Len() )
		return NULL;

	CntAnchorRef xAnchor = new CntAnchor(NULL, rURL);
	if (!xAnchor.Is())
		return NULL;

	CntDemoClient* pNew = new CntDemoClient( xAnchor, pOutEdit );

	String aText( "Created Anchor: " );
	aText += xAnchor->GetServiceURL();
	aText += '\n';
	pNew->Output( aText );

	return pNew;
}

//-------------------------------------------------------------------------
void CntDemoClient::dispose()
{
	if ( !m_xAnchor.Is() )
	{
		Output( "No Anchor.\n" );
		return;
	}
	m_xAnchor.Clear();
}

//----------------------------------------------------------------------------
void CntDemoClient::open()
{
	if ( !m_xAnchor.Is() )
	{
		Output( "No Anchor.\n" );
		return;
	}

	String aText( "Executing: 'open'...\n" );
	Output( aText );

	StartListening(*m_xAnchor);
	m_xAnchor->MakeVisible( TRUE );
	m_xAnchor->Put(CntOpenModeItem(WID_OPEN, CNT_OPEN_ALL));
}

//-------------------------------------------------------------------------
void CntDemoClient::started(void)
{
	String aText( "open started" );
	aText += "'\n";
	Output( aText );
}

//-------------------------------------------------------------------------
void CntDemoClient::done()
{
	String aText( "open done" );
	aText += "'\n'";
	Output( aText );
}

//-------------------------------------------------------------------------
void CntDemoClient::canceled()
{
	String aText( "open canceled" );
	aText += "'\n";
	Output( aText );
}

//-------------------------------------------------------------------------
void CntDemoClient::result(INT32 nAction, const String& rResult )
{
	Output( "Got 'result'.\n" );
}

//-------------------------------------------------------------------------
void CntDemoClient::Output( const String& rText )
{
	if ( m_pOutEdit )
	{
		String aText( m_pOutEdit->GetText() );
		aText += rText;
		m_pOutEdit->SetText( aText );
	}
}

//virtual
void CntDemoClient::Notify(SfxBroadcaster& rBC, const SfxHint& rHint)
{
	if (rHint.ISA(CntStatusHint))
	{
		const CntStatusHint* pStatusHint =
			(const CntStatusHint*)&rHint;
		const CntStatus eStatus =
			pStatusHint->GetStatus();

		switch (eStatus)
		{
			case CNT_STATUS_STARTED:
				started();
				break;
			case CNT_STATUS_DONE   :
				done();
				EndListening(*m_xAnchor);
				break;
			case CNT_STATUS_ERROR  :
				canceled();
				EndListening(*m_xAnchor);
				break;
		}
	}
}

/*========================================================================
 *
 * MyWin.
 *
 *=======================================================================*/

#define MYWIN_ITEMID_CLEAR    	1
#define MYWIN_ITEMID_CREATE   	2
#define MYWIN_ITEMID_RELEASE   	3
#define MYWIN_ITEMID_COMMANDS 	4
#define MYWIN_ITEMID_PROPS 		5
#define MYWIN_ITEMID_GET_PROP	6
#define MYWIN_ITEMID_SET_PROP	7
#define MYWIN_ITEMID_SET_OPEN	8
#define MYWIN_ITEMID_VIEWS		9

//-------------------------------------------------------------------------
class MyWin : public WorkWindow
{
private:
	ToolBox*			m_pTool;
	Edit*				m_pCmdEdit;
	MultiLineEdit*		m_pOutEdit;
	CntDemoClientRef	m_xClient;

public:
	MyWin( Window *pParent, WinBits nWinStyle );
	virtual ~MyWin();

	void Resize( void );
	DECL_LINK ( ToolBarHandler, ToolBox* );
};

//-------------------------------------------------------------------------
MyWin::MyWin( Window *pParent, WinBits nWinStyle )
: WorkWindow( pParent, nWinStyle ),
  m_pTool( NULL ),
  m_pOutEdit( NULL )
{
	// ToolBox.
	m_pTool = new ToolBox( this, WB_SVLOOK | WB_BORDER );

	m_pTool->InsertItem ( MYWIN_ITEMID_CLEAR, "Clear" );
	m_pTool->SetHelpText( MYWIN_ITEMID_CLEAR, "Clear the Output Window" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_CREATE, "Create" );
	m_pTool->SetHelpText( MYWIN_ITEMID_CREATE, "Create a new anchor" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_RELEASE, "Release" );
	m_pTool->SetHelpText( MYWIN_ITEMID_RELEASE, "Release current anchor" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_VIEWS, 	"Views" );
	m_pTool->SetHelpText( MYWIN_ITEMID_VIEWS,
						  "Get Views known to the system" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_COMMANDS, "Commands" );
	m_pTool->SetHelpText( MYWIN_ITEMID_COMMANDS,
						  "Get Commands supported by anchor" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_PROPS, "Properties" );
	m_pTool->SetHelpText( MYWIN_ITEMID_PROPS,
						  "Get Properties supported by anchor" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_GET_PROP, "getProperty" );
	m_pTool->SetHelpText( MYWIN_ITEMID_GET_PROP,
						  "Get a Property from anchor" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_SET_PROP, "setProperty" );
	m_pTool->SetHelpText( MYWIN_ITEMID_SET_PROP,
						  "Set a Property in anchor" );
	m_pTool->InsertSeparator();
	m_pTool->InsertItem ( MYWIN_ITEMID_SET_OPEN, "Open" );
	m_pTool->SetHelpText( MYWIN_ITEMID_SET_OPEN, "Open the anchor" );
	m_pTool->InsertSeparator();

	m_pTool->SetClickHdl( LINK( this, MyWin, ToolBarHandler ) );
	m_pTool->Show();

	// Edit.
	m_pCmdEdit = new Edit( this );
	m_pCmdEdit->SetReadOnly( FALSE );
	m_pCmdEdit->SetText(
			"file:///f|/office50/explorer/arbeitsplatz.scc" );
	m_pCmdEdit->Show();

	// MultiLineEdit.
	m_pOutEdit = new MultiLineEdit( this, WB_HSCROLL | WB_VSCROLL | WB_BORDER );
	m_pOutEdit->SetReadOnly( TRUE );
	m_pOutEdit->Show();
}

//-------------------------------------------------------------------------
// virtual
MyWin::~MyWin()
{
	if ( m_xClient.Is() )
		m_xClient->dispose();

	delete m_pTool;
	delete m_pCmdEdit;
	delete m_pOutEdit;
}

//-------------------------------------------------------------------------
void MyWin::Resize()
{
	Size aWinSize = GetOutputSizePixel();
	int nWinW = aWinSize.Width();
	int nWinH = aWinSize.Height();
	int nBoxH = m_pTool->CalcWindowSizePixel().Height();

	m_pTool->SetPosSizePixel   (
		Point( 0, 0 ), Size ( nWinW, nBoxH ) );
	m_pCmdEdit->SetPosSizePixel(
		Point( 0, nBoxH ), Size( nWinW, nBoxH ) );
	m_pOutEdit->SetPosSizePixel(
		Point( 0, nBoxH + nBoxH ), Size ( nWinW, nWinH - ( nBoxH + nBoxH ) ) );
}

//-------------------------------------------------------------------------
IMPL_LINK( MyWin, ToolBarHandler, ToolBox*, pToolBox )
{
	USHORT nItemId  = pToolBox->GetCurItemId();
	String aCmdLine = m_pCmdEdit->GetText();

	switch( nItemId )
	{
		case MYWIN_ITEMID_CLEAR:
			m_pOutEdit->SetText( '\0' );
			break;

		case MYWIN_ITEMID_CREATE:
			if ( m_xClient.Is() )
				m_xClient->dispose();

			m_xClient = CntDemoClient::create( NULL, aCmdLine, m_pOutEdit );
			break;

		case MYWIN_ITEMID_RELEASE:
			if ( m_xClient.Is() )
				m_xClient->dispose();

			m_xClient = CntDemoClientRef();
			break;

		case MYWIN_ITEMID_COMMANDS:
			break;

		case MYWIN_ITEMID_PROPS:
			break;

		case MYWIN_ITEMID_GET_PROP:
			break;

		case MYWIN_ITEMID_SET_PROP:
			break;

		case MYWIN_ITEMID_SET_OPEN:
			if ( m_xClient.Is() )
			{
				ULONG nOldTicks = Time::GetSystemTicks();
				String aText("Ticks started ");
				aText += nOldTicks;
				aText += "'\n'";
				m_xClient->Output(aText);

#if defined( WNT ) && defined( PROFILE )
				S_CAP;
#endif
				m_xClient->open();
#if defined( WNT ) && defined( PROFILE )
				E_CAP;
#endif

				ULONG nNewTicks = Time::GetSystemTicks();
				aText = "Ticks ended ";
				aText += nNewTicks;
				aText += "'\n'";
				m_xClient->Output(aText);
				aText = "Difference ";
				aText += nNewTicks - nOldTicks;
				aText += "'\n'";
				m_xClient->Output(aText);
			}
			break;

		case MYWIN_ITEMID_VIEWS:
			break;
			
		default: // Ignored.
			break;
	}

	return 0;
}

/*========================================================================
 *
 * MyApp.
 *
 *=======================================================================*/
class MyApp : public Application
{
public:
	virtual void Main();
};

MyApp aMyApp;

//-------------------------------------------------------------------------
// virtual
void MyApp::Main()
{
	//////////////////////////////////////////////////////////////////////
	// Initialize CHAOS... ( will no longer be necessary in the future )
	//////////////////////////////////////////////////////////////////////

	Help::EnableBalloonHelp();

	SfxIniManager aIniMgr ("perfopen", "perfopen", NULL);
	aIniMgr.Set ("$(prog)/config", SFX_KEY_CONFIG_DIR);
	aIniMgr.Set ("$(prog)/store" , SFX_KEY_STORAGE_DIR);

	ResMgr aResMgr ("workben.res");

	CntSystem::SetIniManager (&aIniMgr);
	CntSystem::SetResManager (&aResMgr);

	CntSystem::Initialize();

	//////////////////////////////////////////////////////////////////////
	// Create Application Window...
	//////////////////////////////////////////////////////////////////////

	MyWin *pMyWin = new MyWin( NULL, WB_APP | WB_STDWORK );
	pMyWin->SetText( "CHAOS Open Performance Test" );
	pMyWin->Show();

	//////////////////////////////////////////////////////////////////////
	// Go...
	//////////////////////////////////////////////////////////////////////

	EnterMultiThread( TRUE );
	Execute();
	EnterMultiThread( FALSE );

	//////////////////////////////////////////////////////////////////////
	// De-Initialize CHAOS... ( will no longer be necessary in the future )
	//////////////////////////////////////////////////////////////////////

	CntSystem::Deinitialize();

	//////////////////////////////////////////////////////////////////////
	// Destroy Application Window...
	//////////////////////////////////////////////////////////////////////

	delete pMyWin;
}

