/*************************************************************************
 *
 *  $RCSfile: documaker.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:25:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <direct.h>
#include <string.h>
#ifdef OS2
#include <wchar.h>
#endif

#include "registry/registry.hxx"
#include "registry/reflread.hxx"
#include "registry/reflwrit.hxx"


#ifndef	_VOS_MODULE_HXX
#include <vos/module.hxx>
#endif

#ifndef	_VOS_DIAGNOSE_HXX
#include <vos/diagnose.hxx>
#endif

#ifndef _RTL_ALLOC_H_
#include	<rtl/alloc.h>
#endif

#include "wellhtml.hxx"

class BufferIO: public WellformHTML
{
	char *pOutBuf;
	const char *pInBuf;
	int nOut, nIn, nMax;

public:
	BufferIO( const char *pSourceIdent, const char *pInBuf );
	~BufferIO() { delete pOutBuf; }

	const char *getInBuffer() const { return pInBuf; }
	const char *getOutBuffer() const { return pOutBuf; }

	virtual int getCurrentRow() const;
	virtual int getCurrentCol() const;
	
	virtual int getChar();
	virtual void putChar( char c );
};

BufferIO::BufferIO( const char *pSourceIdent, const char *p )
:	WellformHTML( pSourceIdent ),
	pOutBuf( new char[2*strlen(p)+1] ),
	pInBuf( p ),
	nMax( 2*strlen(p) ),
	nIn( 0 ),
	nOut( 0 )
{
	*pOutBuf = 0;
};

int BufferIO::getCurrentRow() const
{
	int nRow = 0;
	for ( int n = 0; n < nIn; ++n )
		if ( pInBuf[n] == '\n' )
			++nRow;
	return nRow;
}

int BufferIO::getCurrentCol() const
{
	int n = 0;
	while ( (nIn-n) >= 0 && pInBuf[nIn-n] == '\n' )	
		++n;
	return n;
}

int BufferIO::getChar()
{
	int c = pInBuf[nIn++];
	if ( !c )
		c = EOF;
	return c;
}

void BufferIO::putChar( char c )
{
	if ( isLocked() || ( c < ' ' && c != '\n' && c != '\t' ) )
		return;

	if ( nOut >= nMax )
	{
		nMax += 16;
		char *pNew = new char[ nMax+1 ];
		strcpy( pNew, pOutBuf );
		delete pOutBuf;
		pOutBuf = pNew;
	}

	pOutBuf[nOut++] = c;
	pOutBuf[nOut] = 0;
}

const char *replace( const char* p, char cFrom, char *pTo, char * const pBuf )
{
	char *pTarget = pBuf;
	while ( *p != 0 )
	{
		if ( *p == cFrom )
		{
			for ( char *pTemp = pTo; *pTemp; ++pTemp )
				*pTarget++ = *pTemp;
		}
		else
			*pTarget++ = *p;
		++p;
	}
	*pTarget = *p;
	return pBuf;
}

const char *scopeOf( const char *p, char * const pBuf )
{
	const char *pTemp;
	char *pLast, *pCopy;
	for ( pTemp = p, pCopy = pBuf, pLast = pBuf; *pTemp; ++pTemp )
	{
		if ( *pTemp == '/' )
		{
			pLast = pCopy;
			*pCopy++ = ':';
			*pCopy++ = ':';
		}
		else
			*pCopy++ = *pTemp;
	}
	*pLast = 0;
	*pCopy = 0;
	return pBuf;
}

const char *nameOf( const char *p, char * const pBuf )
{
	const char *pTemp, *pLast;
	for ( pTemp = p, pLast = p; *pTemp; ++pTemp )
	{
		if ( *pTemp == '/' )
			pLast = pTemp;
	}
	strcpy( pBuf, pLast+1 );
	return pBuf;
}

const char *makeType( const char *p, char * const pBuf )
{
	char sScope[256], sName[64];
	scopeOf( p, sScope );

	strcpy( pBuf, "<type" );

	if ( *sScope )
	{
		strcat( pBuf, " scope=\"" );
		strcat( pBuf, sScope );
		strcat( pBuf, "\">" );
		strcat( pBuf, nameOf( p, sName ) );
	}
	else
	{
		strcat( pBuf, ">" );
		strcat( pBuf, p );
	}

	strcat( pBuf, "</type>" );

	return pBuf;
}

void printXMLHeader( FILE *pFile )
{
	// XML header
	fprintf( pFile, "<?xml version=\"1.0\"?>\n<!DOCTYPE book [\n" );
	fprintf( pFile, "\t<!ENTITY Auml \"&#246;\">\n" );
	fprintf( pFile, "\t<!ENTITY Ouml \"&#246;\">\n" );
	fprintf( pFile, "\t<!ENTITY Uuml \"&#246;\">\n" );
	fprintf( pFile, "\t<!ENTITY auml \"&#246;\">\n" );
	fprintf( pFile, "\t<!ENTITY ouml \"&#246;\">\n" );
	fprintf( pFile, "\t<!ENTITY uuml \"&#246;\">\n" );
	fprintf( pFile, "\t<!ENTITY szlig \"&#246;\">\n" );
	fprintf( pFile, "]>\n\n" );
}

FILE *pFile = 0;
char *pOutPath = 0;

void makeDoku( FILE *pFile, const char *pSourceIdent, const char *pDoku, 
				const char *pTagFilter = 0 )
{
	if ( pDoku && *pDoku )
	{
		BufferIO aWellformer( pSourceIdent, pDoku );
		aWellformer.wellform( pTagFilter );
		fputs( aWellformer.getOutBuffer(), pFile );
	}
}

void dumpKey( RegistryTypeReaderLoader* pReaderLoader, RegistryKey &rKey,
			  FILE *pModuleFile )
{
	char sFileName[256], sFilePath[256], sTemp[256];
	RegValueType aType;
	sal_uInt32 nSize;
	rKey.getValueInfo( 0, &aType, &nSize );

	// determine filename
	if ( pOutPath && strlen( rKey.getName() ) > 5 ) 
	{
		strcpy( sFileName, pOutPath );
		strcat( sFileName, rKey.getName() + 4 );
		strcat( sFileName, ".xml" );
	}
	else
		sFileName[0] = 0;

	// create single output files?
	if ( strlen( sFileName ) > 5 )
	{
		// create directory 
		for ( int i = 0; sFileName[i]; ++i )
		{
			if ( i > 3 && ( '/' == sFileName[i] || '/' == sFileName[i] ) )
				_mkdir( sFilePath );
			sFilePath[i] = sFileName[i];
			sFilePath[i+1] = 0;
		}
	}

	if ( aType == RG_VALUETYPE_BINARY )
	{
		fprintf( stderr, "dumping %s\n", rKey.getName() );

		sal_uInt8* pBuf = new sal_uInt8[2*nSize];
		rKey.getValue( 0, pBuf );

		RegistryTypeReader aTypeReader(*pReaderLoader, pBuf, nSize, sal_False);
		char *pTypeClass = 0, *pFieldClass = 0, *pPostfix = 0;
		switch ( aTypeReader.getTypeClass() )
		{
			case RT_TYPE_INTERFACE:	
				pTypeClass = "interface"; pFieldClass = "method"; pPostfix = "()"; 
				break;
			case RT_TYPE_SERVICE:	
				pTypeClass = "service"; pFieldClass = "property"; pPostfix = "";
				break;
			case RT_TYPE_EXCEPTION:
				pTypeClass = "exception"; pFieldClass = "field"; pPostfix = ""; 
				break;
			case RT_TYPE_STRUCT:
				pTypeClass = "struct"; pFieldClass = "field"; pPostfix = ""; 
				break;
			case RT_TYPE_CONSTANTS:
				pTypeClass = "consts"; pFieldClass = "const"; pPostfix = ""; 
				break;
			case RT_TYPE_ENUM:
				pTypeClass = "enum"; pFieldClass = "value"; pPostfix = ""; 
				break;
			case RT_TYPE_MODULE:
			{
				// extract module name
				const char *pQualifiedName = rKey.getName() + 5;
				const char *pModuleName = pQualifiedName + strlen(pQualifiedName);
				while ( '/' != pModuleName[-1] )
					--pModuleName;

				// parent module entry
				if ( pModuleFile )
				{
					char sModuleScope[256];
					fprintf( pModuleFile, "\t<module scope=\"%s\"><name>%s</name></module>\n", 
							 scopeOf( pQualifiedName, sModuleScope ), pModuleName );
					}

				{
					FILE *pModuleFile = 0;
					
					// module header
					fprintf( stderr, "MODULE: " );
					if ( pOutPath && strlen( sFileName ) > 3 )
					{
						fprintf( stderr, "fopen( \"%s\" )\n", sFileName );
						pModuleFile = fopen( sFileName, "w" );
						printXMLHeader( pModuleFile );

						char sTypeScope[256];
						fprintf( pModuleFile, "<module scope=\"%s\">\n\t<name>%s</name>\n",
								 scopeOf( pQualifiedName, sTypeScope ), pModuleName );
					}

					// module contents
					RegistryKeyArray aKeys;
					rKey.openSubKeys( 0, aKeys );
					for ( sal_uInt32 n = 0; n < aKeys.getLength(); ++n )
						dumpKey( pReaderLoader, aKeys.getElement(n), pModuleFile );
					rKey.closeSubKeys( aKeys );

					// module footer
					if ( pModuleFile )
					{
						fprintf( pModuleFile, "</module>\n" );
						fclose( pModuleFile );
					}
				}
			}
		}
		if ( pTypeClass )
		{
			char sTypeScope[256], sTypeName[256], sSuperType[256];

			if ( pOutPath )
				pFile = fopen( sFileName, "w" );
			else
			{
				pFile = stdout;
				printf( "fopen( \"%s\" )\n", sFileName );
			}

			// known type?
			const char *pSuperTypeName = aTypeReader.getSuperTypeName();
			if ( pTypeClass )
			  /* aTypeReader.getTypeClass() == RT_TYPE_SERVICE ||	
				 aTypeReader.getTypeClass() == RT_TYPE_INTERFACE ||
				 aTypeReader.getTypeClass() == RT_TYPE_EXCEPTION ) */
			{
				// module entry
				if ( pModuleFile )
				{
					// header
					fprintf( pModuleFile, "\t<%s>\n\t\t<name scope=\"%s\">%s</name>\n",
						pTypeClass, scopeOf(aTypeReader.getTypeName(), sTypeScope ), 
						nameOf( aTypeReader.getTypeName(), sTypeName ) );

					// summary
					makeDoku( pModuleFile, rKey.getName(), aTypeReader.getDoku(), "summary" );
					fprintf( pModuleFile, "\n" );

					// footer
					fprintf( pModuleFile, "\t</%s>\n", pTypeClass );
				}

				// header
				printXMLHeader( pFile );
				fprintf( pFile, "<%s scope=\"%s\">\n\t<name>%s</name>\n",
					pTypeClass, scopeOf(aTypeReader.getTypeName(), sTypeScope ), 
					nameOf( aTypeReader.getTypeName(), sTypeName ) );
				if ( pSuperTypeName != 0 && strlen(pSuperTypeName) > 0 )
					fprintf( pFile, "\t<superclass>%s</superclass>\n", 
							makeType( aTypeReader.getSuperTypeName(), sSuperType ) );
				if ( aTypeReader.getTypeClass() == RT_TYPE_INTERFACE ) 
				{
					RTUik aUIK;
					aTypeReader.getUik( aUIK );
					fprintf( pFile, "\t<uik>%08lX-%04X-%04X-%08lX-%08lX</uik>\n", 
						aUIK.m_Data1, aUIK.m_Data2, aUIK.m_Data3, aUIK.m_Data4, aUIK.m_Data5 );
				}
				
				// documentation
				makeDoku( pFile, rKey.getName(), aTypeReader.getDoku() );
				fprintf( pFile, "\n" );

				// properties
				{
					for ( sal_uInt32 n = 0; n < aTypeReader.getFieldCount(); ++n )
					{
						fprintf( pFile, "\t<%s>\n\t\t<name>%s</name>\n",
								pFieldClass, aTypeReader.getFieldName(n) );

						// documentation
						makeDoku( pFile, rKey.getName(), aTypeReader.getFieldDoku(n) );

						fprintf( pFile, "\n\t</%s>\n", pFieldClass );
					}
				}

				// methods
				{
					for ( sal_uInt32 n = 0; n < aTypeReader.getMethodCount(); ++n )
					{
						fprintf( pFile, "\t<method>\n\t\t<name>%s</name>\n", 
								aTypeReader.getMethodName(n) );
						char sReturnType[256];
						fprintf( pFile, "\t\t<return>%s</return>\n",
								makeType( aTypeReader.getMethodReturnType(n), sReturnType ) );

						// documentation
						makeDoku( pFile, rKey.getName(), aTypeReader.getMethodDoku(n) );
						fprintf( pFile, "\n" );

						for ( sal_uInt32 i = 0; i < aTypeReader.getMethodParamCount(n); ++i )
						{
							char sParamType[256];
							fprintf( pFile, "\t\t<param>\n\t\t\t<name>%s</name>\n\t\t\t%s",
									aTypeReader.getMethodParamName(n, i),
									makeType( aTypeReader.getMethodParamType(n, i), sParamType ) );
							fprintf( pFile, "\n\t\t</param>\n" );
						};

						fprintf( pFile, "\t</method>\n" );
					}
				}

				// services and interfaces
				{
					for ( sal_uInt32 n = 0; n < aTypeReader.getReferenceCount(); ++n )
					{
						char *pRefType = 
								aTypeReader.getReferenceType(n) == RT_REF_SUPPORTS ? "service" :
								aTypeReader.getReferenceType(n) == RT_REF_EXPORTS ? "interface" :
								0;
						char sRefName[256];
						fprintf( pFile, "\t<superclass>%s</superclass>\n", 
								replace( aTypeReader.getReferenceName(n), '/', "::", sRefName ) );
					}
				}
			}

			// footer
			fprintf( pFile, "</%s>\n\n", pTypeClass );
			if ( pOutPath )
				fclose( pFile );
		}

		delete pBuf;
	}
	else
	{
		// module contents
		RegistryKeyArray aKeys;
		rKey.openSubKeys( 0, aKeys );
		for ( sal_uInt32 n = 0; n < aKeys.getLength(); ++n )
			dumpKey( pReaderLoader, aKeys.getElement(n), pModuleFile );
		rKey.closeSubKeys( aKeys );
	}
}

#if (defined UNX) || (defined OS2)
int main( int argc, char * argv[] )
#else
int _cdecl main( int argc, char * argv[] )
#endif
{
	RegistryLoader* pLoader = new RegistryLoader();

	if (!pLoader->isLoaded())
	{
		delete pLoader;
		return 1;
	}
	
	RegistryTypeReaderLoader* pReaderLoader = new RegistryTypeReaderLoader();
	if (!pReaderLoader->isLoaded())
	{
		delete pReaderLoader;
		return 2;
	}

	RegistryTypeWriterLoader* pWriterLoader = new RegistryTypeWriterLoader();

	if (!pWriterLoader->isLoaded())
	{
		delete pWriterLoader;
		return 3;
	}
	
	Registry *myRegistry = new Registry(*pLoader);
	delete pLoader;
	
	RegistryKey rootKey, startKey;

	myRegistry->open( argv[1], REG_READONLY);
	myRegistry->openRootKey(rootKey);
	if ( argc > 2 )
		pOutPath = argv[2];
	rootKey.openKey( argc > 3 ? argv[3] : "/UCR", startKey );

	dumpKey( pReaderLoader, startKey, 0 );

	delete pReaderLoader;
	delete pWriterLoader;
	delete myRegistry;

	return 0;
};

