/*************************************************************************
 *
 *  $RCSfile: dump.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:25:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#ifdef OS2
#include <wchar.h>
#endif

#include "registry/registry.hxx"
#include "registry/reflread.hxx"
#include "registry/reflwrit.hxx"


#ifndef	_VOS_MODULE_HXX
#include <vos/module.hxx>
#endif

#ifndef	_VOS_DIAGNOSE_HXX
#include <vos/diagnose.hxx>
#endif

#ifndef _RTL_ALLOC_H_
#include	<rtl/alloc.h>
#endif

const char *replace( const char* p, char cFrom, char *pTo, char * const pBuf )
{
	char *pTarget = pBuf;
	while ( *p != 0 )
	{
		if ( *p == cFrom )
		{
			for ( char *pTemp = pTo; *pTemp; ++pTemp )
				*pTarget++ = *pTemp;
		}
		else
			*pTarget++ = *p;
		++p;
	}
	*pTarget = *p;
	return pBuf;
}

const char *scopeOf( const char *p, char * const pBuf )
{
	const char *pTemp;
	char *pLast, *pCopy;
	for ( pTemp = p, pCopy = pBuf, pLast = pBuf; *pTemp; ++pTemp )
	{
		if ( *pTemp == '/' )
		{
			pLast = pCopy;
			*pCopy++ = ':';
			*pCopy++ = ':';
		}
		else
			*pCopy++ = *pTemp;
	}
	*pLast = 0;
	*pCopy = 0;
	return pBuf;
}

const char *nameOf( const char *p, char * const pBuf )
{
	const char *pTemp, *pLast;
	for ( pTemp = p, pLast = p; *pTemp; ++pTemp )
	{
		if ( *pTemp == '/' )
			pLast = pTemp;
	}
	strcpy( pBuf, pLast+1 );
	return pBuf;
}

const char *makeType( const char *p, char * const pBuf )
{
	char sScope[256], sName[64];
	scopeOf( p, sScope );

	strcpy( pBuf, "<type" );

	if ( *sScope )
	{
		strcat( pBuf, " scope=\"" );
		strcat( pBuf, sScope );
		strcat( pBuf, "\">" );
		strcat( pBuf, nameOf( p, sName ) );
	}
	else
	{
		strcat( pBuf, ">" );
		strcat( pBuf, p );
	}

	strcat( pBuf, "</type>" );

	return pBuf;
}

FILE *pFile = 0;

void dumpKey( RegistryTypeReaderLoader* pReaderLoader, RegistryKey &rKey )
{
	RegValueType aType;
	sal_uInt32 nSize;
	rKey.getValueInfo( 0, &aType, &nSize );

	if ( aType == RG_VALUETYPE_BINARY )
	{
		sal_uInt8* pBuf = new sal_uInt8[2*nSize];
		rKey.getValue( 0, pBuf );
		RegistryTypeReader aTypeReader(*pReaderLoader, pBuf, nSize, sal_False);
		char *pTypeClass = 0, *pFieldClass = 0, *pPostfix = 0;
		switch ( aTypeReader.getTypeClass() )
		{
			case RT_TYPE_INTERFACE:	
				pTypeClass = "interface"; pFieldClass = "method"; pPostfix = "()"; 
				break;
			case RT_TYPE_SERVICE:	
				pTypeClass = "service"; pFieldClass = "property"; pPostfix = "";
				break;
		};
		if ( pTypeClass )
		{
			char sTypeScope[256], sTypeName[256], 
				 sFileName[256], sSuperType[256];

			//pFile = fopen( sFileName, "w" );

			const char *pSuperTypeName = aTypeReader.getSuperTypeName();
			if ( aTypeReader.getTypeClass() == RT_TYPE_INTERFACE &&
				 pSuperTypeName != 0 && strlen(pSuperTypeName) > 0 )
			{
				// header
				printf( "<%s scope=\"%s\">\n\t<name>%s</name>\n\t<superclass>%s</superclass>\n", 
					pTypeClass, scopeOf(aTypeReader.getTypeName(), sTypeScope ), 
					nameOf( aTypeReader.getTypeName(), sTypeName ), 
					makeType( aTypeReader.getSuperTypeName(), sSuperType ) );
				RTUik aUIK;
				aTypeReader.getUik( aUIK );
				printf( "\t<uik>%08lX-%04X-%04X-%08lX-%08lX</uik>\n", 
					aUIK.m_Data1, aUIK.m_Data2, aUIK.m_Data3, aUIK.m_Data4, aUIK.m_Data5 );

				// properties
				{
					for ( sal_uInt32 n = 0; n < aTypeReader.getFieldCount(); ++n )
						printf( "\t<%s><name>%s</name></%s>\n",
								pFieldClass, aTypeReader.getFieldName(n), pFieldClass );
				}

				// methods
				{
					for ( sal_uInt32 n = 0; n < aTypeReader.getMethodCount(); ++n )
					{
						printf( "\t<method>\n\t\t<name>%s</name>\n", 
								aTypeReader.getMethodName(n) );
						char sReturnType[256];
						printf( "\t\t<return>%s</return>\n",
								makeType( aTypeReader.getMethodReturnType(n), sReturnType ) );

						for ( sal_uInt32 i = 0; i < aTypeReader.getMethodParamCount(n); ++i )
						{
							char sParamType[256];
							printf( "\t\t<param>\n\t\t\t<name>%s</name>\n\t\t\t%s",
									aTypeReader.getMethodParamName(n, i),
									makeType( aTypeReader.getMethodParamType(n, i), sParamType ) );
							printf( "</param>\n" );
						};

						printf( "\t</method>\n" );
					}
				}

				// services and interfaces
				{
					for ( sal_uInt32 n = 0; n < aTypeReader.getReferenceCount(); ++n )
					{
						char *pRefType = 
								aTypeReader.getReferenceType(n) == RT_REF_SUPPORTS ? "service" :
								aTypeReader.getReferenceType(n) == RT_REF_EXPORTS ? "interface" :
								0;
						char sRefName[256];
						printf( "%s %s includes %s %s.\n", 
								pTypeClass, sTypeName, pRefType,
								replace( aTypeReader.getReferenceName(n), '/', "::", sRefName ) );
					}
				}

				// footer
				printf( "</class>\n\n" );
			}
		}

		delete pBuf;
	}

	RegistryKeyArray aKeys;
	rKey.openSubKeys( 0, aKeys );
	for ( sal_uInt32 n = 0; n < aKeys.getLength(); ++n )
		dumpKey( pReaderLoader, aKeys.getElement(n) );
	rKey.closeSubKeys( aKeys );
}

#if (defined UNX) || (defined OS2)
int main( int argc, char * argv[] )
#else
int _cdecl main( int argc, char * argv[] )
#endif
{
	RegistryLoader* pLoader = new RegistryLoader();

	if (!pLoader->isLoaded())
	{
		delete pLoader;
		return 1;
	}
	
	RegistryTypeReaderLoader* pReaderLoader = new RegistryTypeReaderLoader();
	if (!pReaderLoader->isLoaded())
	{
		delete pReaderLoader;
		return 2;
	}

	RegistryTypeWriterLoader* pWriterLoader = new RegistryTypeWriterLoader();

	if (!pWriterLoader->isLoaded())
	{
		delete pWriterLoader;
		return 3;
	}
	
	Registry *myRegistry = new Registry(*pLoader);
	delete pLoader;
	
	RegistryKey rootKey;

	myRegistry->open("g:\\src548\\codemaker\\source\\testmaker\\applicat.rdb", REG_READONLY);
	myRegistry->openRootKey(rootKey);

	dumpKey( pReaderLoader, rootKey );

	delete pReaderLoader;
	delete pWriterLoader;
	delete myRegistry;

	return 0;
};

