/*************************************************************************
 *
 *  $RCSfile: wellhtml.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:25:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "wellhtml.hxx"

const char *aEndTags4P[] =
{
	"p", "ul", "ol", "table", "li", "/ul", "/ol", "/dd", "dt", "blockquote", 0 
};

const char *aEndTags4LI[] =
{
	"li", "/ul", "/ol", 0
};

const char *aEndTags4DD[] =
{
	"dd", "dt", "/dl", 0
};

const char *aEndTags4at[] =
{
	"@see", "@param", "@version", "@author", "@return", "@deprecated", "@guarantees", 0
};

const char *aEmptyTags[] =
{
	"br", "hr", "true", "false", "null", "void", "@deprecated", 
	"br/", "hr/", "true/", "false/", "null/", "void/", "@deprecated/", 0
};

const char *aNoArgTags[] =
{
	"@returns", 0,
};

const char *aDeadTags[] =
{
	"@version", "@author", 0,
};

class Char
{
	char *p;

public:
	Char( int nSize ) { p = new char[ nSize ]; }
	~Char() { delete p; }

	operator char*() { return p; }
};

void toLower( char *pBuf )
{
	for ( int n = 0; pBuf[n]; ++n )
		pBuf[n] = tolower( pBuf[n] );
}

int inTags( const char *pTag, const char *pGroup[] )
{
	char sTag[32];
	int n;

	strcpy( sTag, pTag );
	toLower( sTag );

	for ( n = 0; pGroup[n]; ++n )
		if ( 0 == strcmp( sTag, pGroup[n] ) )
			return 1;
	return 0;
}

class LogCounter
{
static int nLogLevel;
public:
	LogCounter() { ++nLogLevel; }
	~LogCounter() { --nLogLevel; }

	static int getCount() { return nLogLevel; }
};

int LogCounter::nLogLevel = 0;

// #define DBG( s ) LogCounter aLogCounter; { for ( int n = 1; n < LogCounter::getCount(); ++n ) putchar(' '); printf s; }
#define DBG( s ) 

WellformHTML::WellformHTML( const char *pSource )
:	pSourceIdent( pSource ),
	nLocks(0)
{}

const char *WellformHTML::getSourceDesc() const
{
	static char sBuf[1024];
	sprintf( sBuf, "%s [row: %d, col: %d]", getSourceIdent(), getCurrentRow(), getCurrentCol() );
	return sBuf;
}

void WellformHTML::putChars( const char *pBuf )
{
	for ( const char *p = pBuf; *p; ++p )
		putChar( *p );
}

class SAX
{
	WellformHTML *pIO;
	const char *pTagFilter;
	char *pStack[16];
	int nStack, nTagCount, bEnableOutput;

private:
	void push( const char *pTag );
	const char *pop( int bSkipAtSign = 0 );

public:
	SAX( WellformHTML *pIO, const char *pTagFilter );

	const char *top( int bSkipAtSign = 0 ) const;

	int getTagCount() const { return nTagCount; }

	void startTag( const char *pTag, const char *pArgs );
	void closeTag( const char *pTag );
	void closeTag();
	void closeTags();
	void closeTagsFor( const char *pTag );
	void emptyTag( const char *pTag );
	void printChar( char c );
};

SAX::SAX( WellformHTML *p, const char *pFilter )
	:	pIO( p ),
		nStack(0),
		nTagCount(0),
		pTagFilter(pFilter)
{
	for ( int n = 0; n < 16; ++n )
		pStack[n] = 0;
	if ( pTagFilter )
		pIO->lock();
}

void SAX::push( const char *pTag )
{
	if ( nStack < 16 )
	{
		if ( pStack[nStack] != 0 )
			delete pStack[nStack];
		pStack[nStack] = new char[ strlen( pTag ) + 1 ];	
		strcpy( pStack[nStack], pTag );
		toLower( pStack[nStack] );
	}
	else
	{
		fprintf( stderr, "%s\nERROR: tag stack overflow with <%s>\n", 
				 pIO->getInBuffer(), pTag, pIO->getSourceDesc() );
		EXIT(1);
	}
	++nStack;
}

const char *SAX::top( int bSkipAtSign ) const
{
	if ( nStack <= 0 )
	{
		fprintf( stderr, "%s\nWARNING: tag stack underflow\n", 
				 pIO->getInBuffer(), pIO->getSourceDesc() );
		return "";
	}
	if ( nStack > 16 )
	{
		fprintf( stderr, "%s\nERROR: tag stack overflowed\n", 
				 pIO->getInBuffer(), pIO->getSourceDesc() );
		return "";
	}
	const char *pRet = pStack[nStack-1];
	if ( bSkipAtSign && *pRet == '@' )
		++pRet;
	return pRet;
}

const char *SAX::pop( int bSkipAtSign )
{
	DBG( ( "pop(%d)->%s\n", nStack, top() ) ); 
	if ( nStack <= 0 )
	{
		fprintf( stderr, "%s\nWARNING: tag stack underflow\nin: %s\n", 
				 pIO->getInBuffer(), pIO->getSourceDesc() );
		return "";
	}
	if ( nStack > 16 )
	{
		fprintf( stderr, "%s\nERROR: tag stack overflowed\nin: %s\n", 
				 pIO->getInBuffer(), pIO->getSourceDesc() );
		EXIT(1);
	}
	const char *pRet = pStack[--nStack];
	if ( bSkipAtSign && *pRet == '@' )
		++pRet;
	return pRet;
}

void SAX::startTag( const char *pTag, const char *pArgs )
{
	DBG( ( "startTag(%s)\n", pTag ) );
	if ( 0 == strcmp( pTag, "@" ) )
		return;
	if ( pTagFilter && 0 == strcmp( pTag, pTagFilter ) )
		pIO->unlock();
	if ( inTags( pTag, aDeadTags ) )
		pIO->lock();
	char *pLang = 0;
	if ( 0 == strncmp( pTag, "@example:", 9 ) )
	{
		pLang = strstr( pTag, ":" );
		*pLang = 0;
		++pLang;
	}

	push( pTag );
	pIO->putChars( "<" );
	pIO->putChars( top(1) );
	while ( pArgs && *pArgs > 0 && *pArgs <= ' ' )
		++pArgs;	
	if ( pArgs && pArgs[0] )
	{
		pIO->putChars( " " );
		pIO->putChars( pArgs );
	};
	while ( pLang && *pLang > 0 && *pLang <= ' ' )
		++pLang;	
	if ( pLang && pLang[0] )
	{
		pIO->putChars( " lang=\"" );
		pIO->putChars( pLang );
		pIO->putChars( "\"" );
	};
	pIO->putChars( ">" );

	++nTagCount;
}

void SAX::closeTag()
{
	DBG( ( "closeTag()->%s\n", top() ) );

	int bStartDesc = ( 0 == strcmp( top(), "summary" ) );

	char sTag[256];
	strcpy( sTag, pop(1) );
	int bDead = inTags( sTag, aDeadTags );
	Char sBuf( strlen(sTag) + 4 );
	sprintf( sBuf, "</%s>", sTag );
	pIO->putChars( sBuf );

	if ( bDead )
		pIO->unlock();
	if ( pTagFilter && 0 == strcmp( sTag, pTagFilter ) )
		pIO->lock();

	if ( bStartDesc )
		startTag( "description", "" );
}

void SAX::closeTag( const char *pTag )
{
	DBG( ( "closeTag(%s)\n", pTag ) );

	Char sTag( strlen(pTag) + 1 );
	strcpy( sTag, pTag );
	toLower( sTag );

	const char *pTop = top(1); 
	if ( 0 == strcmp( pTop, sTag ) )
		closeTag();
	else
	{
		fprintf( stderr, "%s\nERROR: misplaced </%s>, </%s> expected\nin: %s\n", 
				 pIO->getInBuffer(), pTag, pTop, pIO->getSourceDesc() );
		EXIT(1);
	}
}

void SAX::closeTagsFor( const char *pTag )
{
	while ( nStack &&
		    ( ( 0 == strcmp( "p", top() ) && inTags( pTag, aEndTags4P ) ) ||
		      ( 0 == strcmp( "dd", top() ) && inTags( pTag, aEndTags4DD ) ) ||
		      ( 0 == strcmp( "dt", top() ) && inTags( pTag, aEndTags4DD ) ) ||
		      ( 0 == strcmp( "li", top() ) && inTags( pTag, aEndTags4LI ) ) ||
		      ( 0 == strcmp( "summary", top() ) && inTags( pTag, aEndTags4P ) ) ||
		      ( 0 == strcmp( "description", top() ) && '@' == *pTag ) ||
		      ( '@' == top()[0] && inTags( pTag, aEndTags4at ) ) ) ) 
		closeTag();
}

void SAX::closeTags()
{
	while ( nStack )
		closeTag();
}

void SAX::emptyTag( const char *pTag )
{
	DBG( ( "emptyTag(%s)\n", pTag ) );

	char sTag[32];
	strcpy( sTag, '@' == *pTag ? pTag+1 : pTag );
	toLower( sTag );
	if ( sTag[ strlen(sTag) - 1 ] == '/' )
		sTag[ strlen(sTag) - 1 ] = 0;

	char sBuf[256];
	sprintf( sBuf, "<%s/>", sTag );
	pIO->putChars( sBuf );
}

void SAX::printChar( char c )
{
	if ( '&' == c )
		pIO->putChars( "&amp;" );
	else if ( '"' == c )
		pIO->putChars( "&quot;" );
	else if ( '>' == c )
		pIO->putChars( "&gt;" );
	else
		pIO->putChar( c );
}

void WellformHTML::wellform( const char *pTagFilter )
{
	int c;
	SAX aSAX( this, pTagFilter );
	char sLastWord[256];
	int nLastWord = 0;
	sLastWord[0] = 0;

	while ( EOF != (c = getChar()) )
	{
		if ( '<' == c && 0 == strcmp( "sequence", sLastWord ) )
		{
			fprintf( stderr, "%s\nWARNING: 'sequence' with unquoted '<' detected, replacing by &lt\nin: %s\n", 
					 getInBuffer(), getSourceDesc() );
			putChars( "&lt;" );
			
		}
		else if ( '<' == c || ( '@' == c  && 0 == nLastWord ) ) 
		{
			char sTag[32], sArgs[256];
			int n = 0;

			if ( '@' == c )
			{
				aSAX.closeTags();
				sTag[n++] = c;
			}

			for ( ; (c = getChar()), ( EOF != c && ' ' < c && '>' != c ); ++n )
				if ( n < 31 )
					sTag[n] = c;
			if ( n <= 31 )
				sTag[n] = 0;
			else
			{
				sTag[31] = 0;
				fprintf( stderr, "%s\nERROR: tag length exceeded: <%s>\n%s\nin: %s\n", 
						 getInBuffer(), sTag, getSourceDesc() );
				EXIT(1);
			}
			if ( !*sTag )
			{
				fprintf( stderr, "%s\nWARNING: empty tag detected, replacing by &lt\nin: %s\n", 
						 getInBuffer(), getSourceDesc() );
				putChars( "&lt;" );
				sArgs[0] = 0;
			}
			else 
			{
				DBG( ( "%s detected\n", sTag ) );

				if ( !inTags( sTag, aNoArgTags ) )
				{
					if ( '@' == sTag[0] )
						strcpy( sArgs, " ref=\"" );
					else
						sArgs[0] = 0;
					n = strlen(sArgs);

					for ( ; EOF != c && ( ( '@' != sTag[0] ) ? ( '>' != c ) : ( '\n' != c ) ); c = getChar(), ++n )
						sArgs[n] = c;
					sArgs[n] = 0;
					if ( '@' == sTag[0] )
						if ( strlen( sArgs ) < 8 )
							sArgs[0] = 0;
						else
							strcat( sArgs, "\"" );
				}
				else
					sArgs[0] = 0;

				aSAX.closeTagsFor( sTag );
				if ( sTag[0] == '/' )
					aSAX.closeTag( sTag+1 );
				else
				{
					if ( inTags( sTag, aEmptyTags ) )
						aSAX.emptyTag( sTag );
					else
						aSAX.startTag( sTag, sArgs );
				}
			}
		}
		else		
		{
			if ( nLastWord < 255 &&
				 ( ( c >= 'a' && c <= 'z' ) || ( c >= 'A' && c <= 'Z' ) ) )
				sLastWord[nLastWord++] = c, sLastWord[nLastWord] = 0;
			else
				sLastWord[0] = 0, nLastWord = 0;
			if ( nLastWord && aSAX.getTagCount() == 0 )
				aSAX.startTag( "summary", "" );
			aSAX.printChar( c );
		}
	}

	aSAX.closeTags();
}

#if 0

class StdIO: public WellformHTML
{
public:
	virtual int getChar();
	virtual void putChar( char c );
};

int StdIO::getChar()
{
	return getchar();
}

void StdIO::putChar( char c )
{
	putchar( c );
}

int __cdecl main( int nArgs, const char *pArgs[] )
{
	StdIO().wellform();
	return 0;
}

#endif

