/*************************************************************************
 *
 *  $RCSfile: cfgupdatehelper.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_UPDATEHELPER_HXX_
#include "cfgupdatehelper.hxx"
#endif
#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif
#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

namespace configmgr
{
	using uno::Any;
	using uno::Sequence;
	using uno::Type;
	using uno::Reference;

	//-----------------------------------------------------------------------------------
	ValueUpdater::ValueUpdater(	
		ValueNode const& rNode, 
		Any const& rNewValue,
		Mode nMode, 
		Reference< script::XTypeConverter > xConverter)
	: m_rNode(rNode) 
	, m_aNewValue(rNewValue) 
	, m_aMode(nMode)
	, m_xConverter(xConverter)
	{
	}

	//-----------------------------------------------------------------------------------
	bool ValueUpdater::checkDefaults(Any& oldValue)
	{
		if (m_rNode.isDefault())
		{
			switch (m_aMode)
			{
			case ValueChange::changeValue:
				 m_aMode = ValueChange::wasDefault;
				 // fall thru
			case ValueChange::wasDefault:
				OSL_ENSURE(m_aNewValue.hasValue(), "Using <null> to override a default => default will be left unchanged");

				oldValue = m_rNode.getDefault();
				return m_aNewValue.hasValue() != sal_False; // else no change

			case ValueChange::setToDefault: // checks will be done elsewhere
				OSL_TRACE("Setting to default a Node that already is defaulted\n\r");

				oldValue = m_rNode.getDefault();
				return m_aNewValue.hasValue() != sal_False; // else - no change to execute
			}
		}
		if (m_aMode == ValueChange::changeDefault)
			oldValue = m_rNode.getDefault();
		else
			oldValue = m_rNode.getValue();

		return true;
	}
	//-----------------------------------------------------------------------------------
	bool ValueUpdater::convertNullValue()
	{
		OSL_ASSERT(!m_aNewValue.hasValue());

		switch (m_aMode)
		{
		case ValueChange::changeValue:
			OSL_TRACE("Trying to set a configuration value to NULL => Value will be set to default !\n\r");
			m_aMode = ValueChange::setToDefault;
			// fall thru to setDefault case

		case ValueChange::setToDefault:
			if (m_rNode.hasDefault())
			{
				m_aNewValue = m_rNode.getDefault();
				return true;
			}
			// fall thru to case of NULL default

		case ValueChange::changeDefault:
			OSL_ENSHURE(m_rNode.getAttributes().optional, "Cannot set required value to <null>");
			return m_rNode.getAttributes().optional;

		case ValueChange::wasDefault:
			OSL_ENSHURE(m_rNode.isNull(), "Cannot use <null> to override a non-null default");
			return m_rNode.isNull();

		default: 
			OSL_ASSERT(!"Unknown ValueChange::Mode");
			return false;
		}
	}

	//-----------------------------------------------------------------------------------
	sal_Bool ValueUpdater::checkChange( Any const& oldValue) const
	{
		switch(m_aMode)
		{
		case ValueChange::wasDefault:	return m_aNewValue.hasValue();
		case ValueChange::setToDefault: return !m_rNode.isDefault();
		
		default: return oldValue != m_aNewValue;
		}
	}

//-----------------------------------------------------------------------------------
	bool ValueUpdater::convertValue(	uno::Any& rConverted, 
										uno::Any const& rNewValue, 
										uno::Type const& rTargetType)
		throw(uno::RuntimeException)
	{
		if (rTargetType == rNewValue.getValueType())
		{
			rConverted = rNewValue;
			return true;
		}

		if (m_xConverter.is())
		try 
		{
			rConverted = m_xConverter->convertTo(rNewValue,rTargetType);
		}
		catch(uno::RuntimeException&) { throw; }
		catch(lang::IllegalArgumentException&)
		{
			// try to do more conversion here ?!
			return false;
		}
		catch(script::CannotConvertException&)
		{
			// try to do more conversion here ?!
			return false;
		}
		catch(uno::Exception&)
		{
			OSL_ENSHURE(sal_False, "ValueUpdater::convertValue : generic exception ... thought we caught all allowed exceptions !");
			// try to do more conversion here ?!
			return false;
		}
		
		return true;
	}

//-----------------------------------------------------------------------------------
	ValueChange ValueUpdater::prepareChange(
			bool& bDidChange, 
			uno::Reference< uno::XInterface > const& rContext,
			sal_Int32 index 
		)
		throw (lang::IllegalArgumentException)
	{
		Any aOldVal;
		if (!checkDefaults(aOldVal))
		{
			bDidChange = false;
			// leaving at default without a value 
			OSL_ASSERT(m_rNode.isDefault());
			OSL_ASSERT(!m_aNewValue.hasValue());
			return ValueChange(ValueChange::SetToDefault(), m_rNode);
		}

		if (!m_aNewValue.hasValue())
		{
			if (!convertNullValue())
			{
				OUString
				sMessage  = OUString::createFromAscii("Error trying to unset Configuration Entry '");
				sMessage += m_rNode.getName();
				sMessage += OUString::createFromAscii("': The Entry is not optional. !");

				throw lang::IllegalArgumentException(sMessage, rContext,index);
			}
		}
		else 
		{
			Any aNewVal;
			if (!convertValue(aNewVal, m_aNewValue, m_rNode.getValueType())) // could not convert
			{
				OUString
				sMessage  = OUString::createFromAscii("Cannot convert data. ");
				sMessage += OUString::createFromAscii("Error trying to set Configuration Entry '");
				sMessage += m_rNode.getName();
				sMessage += OUString::createFromAscii("' of type ");
				sMessage += m_rNode.getValueType().getTypeName();
				sMessage += OUString::createFromAscii(" to a value of type ");
				sMessage += m_aNewValue.getValueType().getTypeName();

				throw lang::IllegalArgumentException(sMessage, rContext,index);
			}

			if (m_aMode == ValueChange::setToDefault && aNewVal != m_rNode.getDefault())
			{
				OUString
				sMessage  = OUString::createFromAscii("Error trying to set Configuration Entry '");
				sMessage += m_rNode.getName();
				sMessage += OUString::createFromAscii("' to default: ");
				sMessage += OUString::createFromAscii(" The given value is not the default.");

				throw lang::IllegalArgumentException(sMessage, rContext,index);
			}
			m_aNewValue = aNewVal;
		}

		OSL_ASSERT(	m_aNewValue.getValueType() == m_rNode.getValueType() || 
					(!m_aNewValue.hasValue() && m_rNode.getAttributes().optional) );
		bDidChange = (checkChange(aOldVal) != sal_False);


		return ValueChange(m_rNode.getName(), m_aNewValue, m_aMode, aOldVal );
	}


}


