/*************************************************************************
 *
 *  $RCSfile: cfgupdatehelper.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_UPDATEHELPER_HXX_
#define CONFIGMGR_API_UPDATEHELPER_HXX_

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_HXX_
#include <com/sun/star/uno/Reference.hxx>
#endif

#ifndef CONFIGMGR_APITYPES_HXX_
#include "confapitypes.hxx"
#endif
#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif

namespace com { namespace sun { namespace star {
	namespace uno  
	{ 
		class Type; 
		class RuntimeException; 
		class XInterface;
	}

	namespace lang { class IllegalArgumentException; }
	namespace script { class XTypeConverter; }
} } }

namespace configmgr
{
	namespace css = ::com::sun::star;
	namespace uno	= css::uno;
	namespace lang	= css::lang;
	namespace script	= css::script;

	class ValueUpdater
	{
		typedef ValueChange::Mode Mode;

		ValueNode const&	m_rNode; 
		uno::Any			m_aNewValue; 
		Mode				m_aMode;
		uno::Reference< script::XTypeConverter > m_xConverter;
	public:
		ValueUpdater(	ValueNode const& rNode, 
						uno::Any const& rNewValue,					
						Mode nMode,
						uno::Reference< script::XTypeConverter > xConverter);

		/**
		 * Check whether the value changed and can be set. 
		 * If that is the case, set bDidChange and return the change.<BR>
		 */
		ValueChange prepareChange(bool& bDidChange,
						uno::Reference< uno::XInterface> const& aContext,
						sal_Int32 index = -1)	throw (lang::IllegalArgumentException);

		sal_Bool checkChange(uno::Any const& oldValue) const;

		/**
		 * Check whether the value can be converted and set. 
		 * If that is the case, convert it to the right type.<BR>
		 * If the conversion succeeds, return true.
		 */
		bool convertValue(	uno::Any& rConverted, 
							uno::Any const& rNewValue, 
							uno::Type const& rTargetType)
				throw(uno::RuntimeException);
	private:
		bool checkDefaults(uno::Any& oldValue);
		bool convertNullValue();
	private: // not implemented
		ValueUpdater(ValueUpdater const&);
		ValueUpdater&operator=(ValueUpdater const&);
	};


}

#endif // CONFIGMGR_API_UPDATEHELPER_HXX_


