/*************************************************************************
 *
 *  $RCSfile: changenotifier.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include "changenotifier.hxx"

#ifndef _COM_SUN_STAR_UTIL_CHANGESEVENT_HPP_
#include <com/sun/star/util/ChangesEvent.hpp>
#endif

#ifndef _CPPUHELPER_INTERFACECONTAINER_HXX_
#include <cppuhelper/interfacecontainer.hxx>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif

#ifndef CONFIGMGR_API_CHANGESSET_HXX_
#include "confchangesset.hxx"
#endif

#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif
#ifndef __SGI_STL_MEMORY
#include <stl/memory>
#endif

namespace configmgr
{
	using uno::RuntimeException;
	using uno::Reference;
	using uno::Any;
	using uno::Sequence;

//-----------------------------------------------------------------------------------

// Constructors
//-----------------------------------------------------------------------------------
ChangesNotifierImpl::ChangesNotifierImpl(BroadcastHelper& rBHelper_)
: m_rBroadcastHelper(rBHelper_)
, m_pSource(0)
, m_pObservedSource(NULL)
{
}

//-----------------------------------------------------------------------------------
ChangesNotifierImpl::ChangesNotifierImpl(BroadcastHelper& rBHelper_, IConfigBroadcaster* _pSource)
: m_rBroadcastHelper(rBHelper_)
, m_pSource(_pSource)
, m_pObservedSource(NULL)
{
}

//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::Init()
{
}

//-----------------------------------------------------------------------------------
ChangesNotifierImpl::~ChangesNotifierImpl()
{
	stopObserving();
}

//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::observe(IConfigBroadcaster& rSource)
{
	OSL_ENSHURE(m_pObservedSource == 0, "ChangesNotifier should not be rebound. Semantics unclear.");

	if (m_pObservedSource)
		stopObserving();

	m_pSource = &rSource;
	startObserving();

}

//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::startObserving()
{
	if (m_pObservedSource)
	{
		OSL_ENSHURE(sal_False, "ChangesNotifierImpl::startObserving : already observing !");
		return;
	}

	if (m_pSource)
		m_pSource->addListener(this->getHierarchicalName(), this);
	m_pObservedSource = m_pSource;
}

//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::stopObserving()
{
	if (m_pObservedSource)
		m_pObservedSource->removeListener(this);
	m_pObservedSource = NULL;
}

//-----------------------------------------------------------------------------------
// Interface methods
//-----------------------------------------------------------------------------------
	
// XChangesNotifier
//-----------------------------------------------------------------------------------
void SAL_CALL ChangesNotifierImpl::addChangesListener( const uno::Reference< util::XChangesListener >& xListener ) 
	throw(uno::RuntimeException)
{
	::osl::MutexGuard aGuard( m_rBroadcastHelper.rMutex );
	OSL_ENSHURE( !m_rBroadcastHelper.bInDispose, "do not add listeners in the dispose call" );
	OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed" );
	if (!m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed) 
		m_rBroadcastHelper.aLC.addInterface(::getCppuType(&xListener), xListener);
}

//-----------------------------------------------------------------------------------
void SAL_CALL ChangesNotifierImpl::removeChangesListener( const uno::Reference< util::XChangesListener >& xListener ) 
	throw(uno::RuntimeException)
{
	::osl::MutexGuard aGuard( m_rBroadcastHelper.rMutex );
	OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed" );
	if (!m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed) 
		m_rBroadcastHelper.aLC.removeInterface(::getCppuType(&xListener), xListener);
}

// XContainer
//-----------------------------------------------------------------------------------
void SAL_CALL ChangesNotifierImpl::addContainerListener( const uno::Reference< container::XContainerListener >& xListener ) 
	throw(uno::RuntimeException)
{
	::osl::MutexGuard aGuard( m_rBroadcastHelper.rMutex );
	OSL_ENSHURE( !m_rBroadcastHelper.bInDispose, "do not add listeners in the dispose call" );
	OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed" );
	if (!m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed) 
		m_rBroadcastHelper.aLC.addInterface(::getCppuType(&xListener), xListener);
}

//-----------------------------------------------------------------------------------
void SAL_CALL ChangesNotifierImpl::removeContainerListener( const uno::Reference< container::XContainerListener >& xListener ) 
	throw(uno::RuntimeException)
{
	::osl::MutexGuard aGuard( m_rBroadcastHelper.rMutex );
	OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed" );
	if (!m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed) 
		m_rBroadcastHelper.aLC.removeInterface(::getCppuType(&xListener), xListener);
}


//-----------------------------------------------------------------------------------
void SAL_CALL ChangesNotifierImpl::disposing()
{
	if (m_pSource)
		m_pSource->removeListener(this);
}

// Event builder based on Changes sets
//-----------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------
// Change Tree parser for Container event
//-----------------------------------------------------------------------------------
namespace {
	using namespace css::container;

	struct LocalEventDispatcher
	{
		virtual ~LocalEventDispatcher() {}

		virtual bool is() const = 0;
		virtual void dispatchTo(XContainerListener* pListener) const = 0;
	};

	class LocalEventSingleDispatch 
		: public LocalEventDispatcher
		, private ChangeTreeAction
	{
		typedef void (SAL_CALL XContainerListener::*Dispatch)(ContainerEvent const& event);
		ContainerEvent m_event;
		Dispatch m_dispatch;
	public:
		LocalEventSingleDispatch(XContainer* pSource, Change const& rChange)
			: m_event(pSource, uno::makeAny(rChange.getNodeName()), Any(), Any())
			, m_dispatch(0) 
		{ 
			OSL_ENSURE(pSource, "Null Source in Event dispatch");
			this->applyToChange(rChange); 
		}

		bool is() const { return m_dispatch != 0; }

		void dispatchTo(XContainerListener* pListener) const
		{
			OSL_ENSURE(this->is(), "Attempt to dispatch non-event" );
			OSL_ENSURE(pListener, "Null Listener in Event dispatch");

			(pListener->*m_dispatch)(m_event);
		}

	private:
		void handle(ValueChange const& aValueNode)	
		{ 
			m_dispatch = &XContainerListener::elementReplaced; 
			m_event.Element <<= aValueNode.getNewValue();
			m_event.ReplacedElement <<= aValueNode.getOldValue();
		}
		void handle(AddNode const& aAddNode)		
		{ 
			m_dispatch = &XContainerListener::elementInserted;  
			// TO DO:
			// Provide Reference to inserted element
		}
		void handle(RemoveNode const& aRemoveNode)	
		{ 
			m_dispatch = &XContainerListener::elementRemoved; 
			m_event.ReplacedElement = aRemoveNode.getOldNodeValue();
		}
		void handle(SubtreeChange const& )	{} // subtrees are not considered
	};

	class LocalEventMultiDispatch
		: public LocalEventDispatcher
		, private ChangeTreeAction
	{
		typedef std::vector<LocalEventSingleDispatch> DispatchList;
		DispatchList m_events;
		XContainer* m_pSource;
	public:
		LocalEventMultiDispatch(XContainer* pSource, SubtreeChange const& rChanges)
			: m_events(), m_pSource(pSource)
		{ 
			OSL_ENSURE(pSource, "Null Source in Event dispatch");
			this->applyToChildren(rChanges);
		}

		bool is() const { return !m_events.empty(); }

		void dispatchTo(XContainerListener* pListener) const
		{
			OSL_ENSURE(this->is(), "Attempt to dispatch non-event" );
			OSL_ENSURE(pListener, "Null Listener in Event dispatch");

			for(DispatchList::const_iterator iter = m_events.begin(); iter != m_events.end(); ++iter)
				iter->dispatchTo(pListener);
		}

	private:
		void addEvent(Change const& aChange)
		{
			LocalEventSingleDispatch aNewEvent(m_pSource,aChange);
			if (aNewEvent.is())
				m_events.push_back(aNewEvent);
		}
		void handle(ValueChange const& aValueNode)	
		{ 
			addEvent(aValueNode);
		}
		void handle(AddNode const& aAddNode)		
		{ 
			addEvent(aAddNode);
		}
		void handle(RemoveNode const& aRemoveNode)	
		{ 
			addEvent(aRemoveNode);
		}
		void handle(SubtreeChange const& )	
		{
			//addEvent(aChange); // subtrees are not considered
		} 
	};

}
// Event firing
//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::fireChanges(Change const& aChange, OUString const& aPath)
{
	ConfigurationName aFullPath( aPath );
	if ( aFullPath.isRelative() )
		aFullPath = ConfigurationName(this->getHierarchicalName()).composeWith(aPath);

	doFireChanges(aChange,aFullPath); 
}
//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::doFireChanges(Change const& aChange, ConfigurationName const& aPath)
{
	OSL_ASSERT( !aPath.isEmpty() );
	OSL_ASSERT( !aPath.isRelative() );
	OSL_ASSERT( aPath.localName() == aChange.getNodeName() );

	osl::ClearableMutexGuard aCollectChangesGuard( m_rBroadcastHelper.rMutex );

	ConfigurationName myPath(this->getHierarchicalName());
	bool bThisIsRoot = (myPath == aPath);
	OSL_ASSERT( bThisIsRoot || aPath.isNestedIn(myPath) );

	// collect changes deeply
	Reference<util::XChangesListener> const * const selectChanges = 0;
	::cppu::OInterfaceContainerHelper* pChangesListeners = this->m_rBroadcastHelper.aLC.getContainer( ::getCppuType(selectChanges) );

	OChangeCollector aChangeSetBuilder( aPath.getParentName() );

	// collect changes flatly. Possible pessimization: extract this from change set
	Reference<container::XContainerListener> const * const selectContainer = 0;
	::cppu::OInterfaceContainerHelper* pContainerListeners = this->m_rBroadcastHelper.aLC.getContainer( ::getCppuType(selectContainer) );

	typedef std::auto_ptr<LocalEventDispatcher> LocalEventHolder;
	LocalEventHolder aLocalEvents;

	// do collect 'em
	if (pChangesListeners != 0)
		aChangeSetBuilder.applyToChange(aChange);

	if (bThisIsRoot)
	{
		OSL_ENSURE(aChange.isA(SubtreeChange::getStaticType()), "Object change must be subtree change");

		if (pContainerListeners != 0)
			if (aChange.isA(SubtreeChange::getStaticType()))
			{
				SubtreeChange const& rThisSubtree = static_cast<SubtreeChange const&>(aChange);

				aLocalEvents = LocalEventHolder( new LocalEventMultiDispatch(this, rThisSubtree) );
			}
	}
	else
	{
		if (pContainerListeners != 0)
			if (aPath.getParentName() == myPath )
			{
				aLocalEvents = LocalEventHolder( new LocalEventSingleDispatch(this,aChange) );
			}
			// else changes are more deeply nested and therefore not for container listeners
	}

	OSL_ASSERT(pChangesListeners != 0 || aChangeSetBuilder.isEmpty());
	OSL_ASSERT(pContainerListeners != 0 || aLocalEvents.get() == 0);

	// unlock for broadcast
	aCollectChangesGuard.clear(); 

	if (!aChangeSetBuilder.isEmpty())
	{
		::cppu::OInterfaceIteratorHelper aChangesIterator( *pChangesListeners );

		util::ChangesEvent aEvent(
							static_cast<util::XChangesNotifier*>(this), 
							uno::makeAny(aPath.fullName()), 
							aChangeSetBuilder.getChanges());

		while (aChangesIterator.hasMoreElements() )
		{
			util::XChangesListener* pListener = 
				static_cast<util::XChangesListener*>(aChangesIterator.next());
			OSL_ASSERT(pListener);

			pListener->changesOccurred(aEvent);
		}
	}

	if (aLocalEvents.get() && aLocalEvents->is())
	{
		::cppu::OInterfaceIteratorHelper aContainerIterator( *pContainerListeners );

		while (aContainerIterator.hasMoreElements() )
		{
			container::XContainerListener* pListener = 
				static_cast<container::XContainerListener*>(aContainerIterator.next());
			OSL_ASSERT(pListener);

			aLocalEvents->dispatchTo( pListener );
		}
	}
}


// Relaying implementations
// IConfigListener implementation
//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::disposing(IConfigBroadcaster* pSource)
{
	OSL_VERIFY(m_pSource == pSource); // VERIFY to avoid unused variable warnings
	if (m_pSource == pSource)
		m_pSource = 0;

	if (m_pObservedSource == pSource)
		// no need to remove signogff from the source as a listener, 'cause it's beeing disposed currently ...
		m_pObservedSource = NULL;
	this->dispose(); // (?) our root and source has gone :-(
}

// INodeListener implementation
//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::nodeChanged(Change const& aChange, OUString const& aPath, IConfigBroadcaster* pSource)
{
	OSL_VERIFY(m_pSource == pSource); // VERIFY to avoid unused variable warnings

	ConfigurationName aNodePath(aPath);
	OSL_ASSERT( !aNodePath.isEmpty() );
	OSL_ASSERT( !aNodePath.isRelative() );
	OSL_ASSERT( aNodePath.localName() == aChange.getNodeName() );
	
	doFireChanges(aChange, aNodePath);
}

//-----------------------------------------------------------------------------------
void ChangesNotifierImpl::nodeDeleted(OUString const& , IConfigBroadcaster* pSource)
{
	OSL_VERIFY(m_pSource == pSource); // VERIFY to avoid unused variable warnings

	// cannot check whether the path applies to *this - our node may be gone already
	this->dispose();
	
}
//-----------------------------------------------------------------------------------

}


