/*************************************************************************
 *
 *  $RCSfile: confaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_ACCESS_HXX_
#include "confaccess.hxx"
#endif

#ifndef CONFIGMGR_CONFNAME_HXX_
#include "confname.hxx"
#endif
#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif

#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif

#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

namespace configmgr
{

HierarchyReadAccess::HierarchyReadAccess(BroadcastHelper& rBHelper_, IConfigBroadcaster& rSource, ISynchronizedData const* pSync, TreeRef pNode, OUString aBasePath)
: HierarchyAccessImpl(rBHelper_.rMutex, pNode)
, HierarchyElementImpl(rBHelper_.rMutex, aBasePath)
, ChangesNotifierImpl(rBHelper_, &rSource)
, m_pSync(pSync)
{
}

HierarchyReadAccess::HierarchyReadAccess(BroadcastHelper& rBHelper_, IConfigBroadcaster& rSource, ISynchronizedData const* pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath)
: HierarchyAccessImpl(rBHelper_.rMutex, pNode)
, HierarchyElementImpl(rBHelper_.rMutex, pPath,aBasePath)
, ChangesNotifierImpl(rBHelper_, &rSource)
, m_pSync(pSync)
{
}

HierarchyReadAccess::HierarchyReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode)
: HierarchyAccessImpl(rBHelper_.rMutex, pNode)
, HierarchyElementImpl(rBHelper_.rMutex, aAncestorElement,pParentNode)
, ChangesNotifierImpl(rBHelper_, aAncestorElement.changesSource())
, m_pSync(aAncestorElement.getTreeSync())
{
}

HierarchyReadAccess::HierarchyReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath)
: HierarchyAccessImpl(rBHelper_.rMutex, pNode)
, HierarchyElementImpl(rBHelper_.rMutex, aAncestorElement,pPath)
, ChangesNotifierImpl(rBHelper_, aAncestorElement.changesSource())
, m_pSync(aAncestorElement.getTreeSync())
{
}

HierarchyReadAccess::HierarchyReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, ParentElement toParent)
: HierarchyAccessImpl(rBHelper_.rMutex, aAncestorElement.getParentNode())
, HierarchyElementImpl(rBHelper_.rMutex, aAncestorElement,toParent)
, ChangesNotifierImpl(rBHelper_, aAncestorElement.changesSource())
, m_pSync(aAncestorElement.getTreeSync())
{
}

void HierarchyReadAccess::Init()
{
	ChangesNotifierImpl::Init();
	ChangesNotifierImpl::startObserving();
}

// HierarchyAccessImpl & HierarchyElementImpl implementation
HierarchyAccessImpl* HierarchyReadAccess::getSubtreeAccess(TreeRef aSubtree, TreePath const& aPath)
{
	HierarchyReadAccess* pReturn = HierarchyReadAccess::create(*this, aSubtree, aPath);

	// by definition the object to be returned has to be acquired once
	// and do this _before_ the Init !
	static_cast< ::com::sun::star::container::XNameAccess* >(pReturn)->acquire();
	pReturn->Init();

	return pReturn;
}

HierarchyAccessImpl* HierarchyReadAccess::getParentAccess()
{
	HierarchyReadAccess* pReturn = HierarchyReadAccess::create(*this, HierarchyElementImpl::ParentElement());
	// by definition the object to be returned has to be acquired once
	// and do this _before_ the Init !
	static_cast< ::com::sun::star::container::XNameAccess* >(pReturn)->acquire();
	pReturn->Init();

	return pReturn;
}

INode const& HierarchyReadAccess::getThisNode()
{
	ISubtree const* pTree = HierarchyAccessImpl::getThisTree();
	OSL_ENSHURE(pTree != 0, "Invalid NULL configuration Access");

	return *pTree;
}

bool HierarchyReadAccess::normalizeNestedPath(ConfigurationName& aPath)
{
	if (!aPath.isRelative())
	{
		ConfigurationName aMyPath(getBasePath(),getName());
		if (!aPath.isNestedIn(aMyPath))
			return false;

		aPath = aPath.relativeTo(aMyPath);
	}

	return true;
}

// Component behavior

void SAL_CALL HierarchyReadAccess::disposing()
{
	ChangesNotifierImpl::disposing();
}

// UNO 3 boilerplate stuff
	// XTypeProvider implementation

uno::Any SAL_CALL HierarchyReadAccess::queryInterface(uno::Type const& rType) throw (uno::RuntimeException)
{
	uno::Any aRet;

	if (!aRet.hasValue()) aRet = HierarchyAccessImpl::queryInterface(rType);
	if (!aRet.hasValue()) aRet = HierarchyElementImpl::queryInterface(rType);
	if (!aRet.hasValue()) aRet = ChangesNotifierImpl::queryInterface(rType);

	return aRet;
}

uno::Sequence< uno::Type > SAL_CALL HierarchyReadAccess::getTypes() throw (uno::RuntimeException)
{
	uno::Sequence< uno::Type > aTypes;

	aTypes = ::utl::concatSequences(aTypes, HierarchyAccessImpl::getTypes() );
	aTypes = ::utl::concatSequences(aTypes, HierarchyElementImpl::getTypes());
	aTypes = ::utl::concatSequences(aTypes, ChangesNotifierImpl::getTypes());

	return aTypes;
}

} // namespace configmgr



