/*************************************************************************
 *
 *  $RCSfile: confbaseaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "confbaseaccess.hxx"

#ifndef _COM_SUN_STAR_LANG_DISPOSEDEXCEPTION_HPP_
#include <com/sun/star/lang/DisposedException.hpp>
#endif

#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif
#ifndef CONFIGMGR_CONFNAME_HXX_
#include "confname.hxx"
#endif

#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

namespace configmgr
{
	using uno::RuntimeException;
	using uno::Reference;
	using uno::Any;
	using uno::Sequence;
	using namespace ::com::sun::star::lang;
//-----------------------------------------------------------------------------------
	// Constructors
	HierarchyAccessImpl::HierarchyAccessImpl(::osl::Mutex& _rMutex, TreeRef aTree)
		:m_aSubtree(aTree)
		,m_rMutex(_rMutex)
	{
		OSL_ENSHURE(aTree != 0,"Illegal initialization: Null configuration tree");
		if (m_aSubtree == 0)
			throw uno::RuntimeException(OUString(RTL_CONSTASCII_USTRINGPARAM("Hierarchy node not found")), Reference<uno::XInterface>());
	}
/*	void HierarchyAccessImpl::init(TreeRef aTree)
	{
		OSL_ENSHURE(m_aSubtree == 0,"Object already initialized");
		OSL_ENSHURE(aTree != 0,"Illegal initialization: Null configuration tree");
		if (aTree) 
			m_aSubtree = aTree;
	}
*/
//-----------------------------------------------------------------------------------
// Interface methods
//-----------------------------------------------------------------------------------

// XElementAccess, base class of XNameAccess (and XHierarchicalNameAccess ? )
//-----------------------------------------------------------------------------------

// Iterating the elements of m_aSubtree is done using ISubtree::forEachChild
namespace {
	class CountChildTypes : public NodeAction
	{
		sal_uInt16 nValues, nSubtrees;
	public:
		sal_uInt16 subtrees() const { return nSubtrees; }
		sal_uInt16 values() const { return nValues; }
		sal_uInt16 total() const { return nValues+nSubtrees; }

		CountChildTypes() : nValues(0), nSubtrees(0) {}

		void handle(ValueNode const& ) { ++nValues; }
		void handle(ISubtree const&  ) { ++nSubtrees; }
	};
}

//-----------------------------------------------------------------------------------
void SAL_CALL HierarchyAccessImpl::setLocale( const Locale& _rLocale ) throw(RuntimeException)
{
	OSL_ENSHURE(sal_False, "HierarchyAccessImpl::setLocale : not really implemented yet (no server support) !");
	m_aCurrentLocale = _rLocale;
}

//-----------------------------------------------------------------------------------
Locale SAL_CALL HierarchyAccessImpl::getLocale(  ) throw(RuntimeException)
{
	return m_aCurrentLocale;
}

//-----------------------------------------------------------------------------------
uno::Type SAL_CALL HierarchyAccessImpl::getElementType(  ) throw(RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OReadSynchronized aReadGuard( getTreeSync() );
	CountChildTypes counter;
	m_aSubtree->forEachChild(counter);

	if (counter.values() == 0)
		if (counter.subtrees() == 0)	// empty ??
			return ::getVoidCppuType();
		else // only subtrees - report name access container
			return ::getCppuType( static_cast<Reference<css::container::XNameAccess>const*>(0) );

	else // values present - return any element type
		return ::getCppuType( static_cast<Any const*>(0) );
}

//-----------------------------------------------------------------------------------
sal_Bool SAL_CALL HierarchyAccessImpl::hasElements(  ) throw(RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OReadSynchronized aReadGuard( getTreeSync() );
	CountChildTypes counter;
	m_aSubtree->forEachChild(counter);
	
	return counter.total() != 0;
}

//------------------------------------------------------------------------------------------------------------------
struct OSearchExactName : public NodeAction
{
protected:
	ConfigurationName	aSearchName;
	::rtl::OUString		sCurrentModuleName;	// redundant, just for optimization
	sal_Bool			bDoneThisLevel;

	::std::vector< ::rtl::OUString > aRealCompNames;

public:
	OSearchExactName(const ::rtl::OUString& _rLookFor) : aSearchName(_rLookFor), bDoneThisLevel(sal_False) { sCurrentModuleName = *aSearchName.begin(); }

	virtual void handle(ValueNode const&);
	virtual void handle(ISubtree const&);

	ConfigurationName getExactName() const;

protected:
	sal_Bool tryStepInto(const INode& _rNode);
};

//..................................................................................................................
sal_Bool OSearchExactName::tryStepInto(const INode& _rNode)
{
	if (bDoneThisLevel)
		return sal_False;

	::rtl::OUString sNodeName = _rNode.getName();
	if (sNodeName.equalsIgnoreCase(sCurrentModuleName))
	{
		bDoneThisLevel = sal_True;
		aRealCompNames.push_back(sNodeName);

		if (1 == aSearchName.depth())
		{
			aSearchName = ConfigurationName();
			sCurrentModuleName = ::rtl::OUString();
			return sal_False;
				// do not go any further - we reached the end of the search name
		}
		else
		{
			aSearchName = aSearchName.relativeTo(sCurrentModuleName);
			sCurrentModuleName = *aSearchName.begin();
			return sal_True;
		}
	}
	return sal_False;
}

//..................................................................................................................
ConfigurationName OSearchExactName::getExactName() const
{
	ConfigurationName aReturn;
	if (!aSearchName.depth())
	{	// the search was successfull
		for (	::std::vector< ::rtl::OUString >::const_iterator aCollect = aRealCompNames.begin();
				aCollect != aRealCompNames.end();
				++aCollect
			)
			aReturn = aReturn + *aCollect;
	}
	return aReturn;
}

//..................................................................................................................
void OSearchExactName::handle(ValueNode const& _rNode)
{
	tryStepInto(_rNode);
}

//..................................................................................................................
void OSearchExactName::handle(ISubtree const& _rTreeNode)
{
	if (!tryStepInto(_rTreeNode))
		// this subtree's name does not match the currently needed module name
		// OR
		// it does match and we reached the end
		return;

	bDoneThisLevel = sal_False;
	applyToChildren(_rTreeNode);
	bDoneThisLevel = sal_True;
}

//..................................................................................................................
::rtl::OUString SAL_CALL HierarchyAccessImpl::getExactName( const ::rtl::OUString& _rApproximateName ) throw(::com::sun::star::uno::RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkAlive();

	::rtl::OUString sEmpty;
	ConfigurationName aName(_rApproximateName);
	if (0 == aName.depth())
		// empty name (or consisting of slashes only), would require an InvalidArgumentException
		return sEmpty;

	OReadSynchronized aReadGuard( getTreeSync() );
	OSearchExactName aSearch(_rApproximateName);
	aSearch.applyToChildren(*getThisTree());

	ConfigurationName aExactName(aSearch.getExactName());
	if (aExactName.depth())
		return aExactName.fullName();
	else
		return sEmpty;
}

// XNameAccess
//-----------------------------------------------------------------------------------

Any HierarchyAccessImpl::getChildAccess(TreeRef aChildTree) throw(uno::RuntimeException)
{
	OSL_ASSERT(m_aSubtree->getChild(aChildTree->getName()) == aChildTree);
	return getChildAccess(aChildTree, TreePath(1,m_aSubtree));
}

//-----------------------------------------------------------------------------------
Any HierarchyAccessImpl::getChildAccess(TreeRef aChildTree, TreePath const& aPath) throw(uno::RuntimeException)
{
	using namespace ::com::sun::star::container;
	Reference<XNameAccess> xChild( getSubtreeAccess(aChildTree, aPath) );
	if (xChild.is())
		// by definition, the object returned by getSubtreeAccess is acquired once
		xChild->release();
	return uno::makeAny(xChild);
}

//-----------------------------------------------------------------------------------
Any HierarchyAccessImpl::makeElement(NodeRef pFound, TreePath const& aPath) throw(css::container::NoSuchElementException, uno::RuntimeException)
{
	using namespace ::com::sun::star::container;
	if (pFound == 0)
	{
		throw NoSuchElementException( OUString(RTL_CONSTASCII_USTRINGPARAM("Configuration element not available.")), 
										static_cast<XNameAccess*>(this));
	}
	if (ISubtree const* aChildTree = pFound->asISubtree())
	{
		return getChildAccess(aChildTree, aPath);
	}

	ValueNode const* pValue = pFound->asValueNode();
	OSL_ENSHURE(pValue, "Unknown node type in configuration");
	if (!pValue)
	{
		throw uno::RuntimeException(	OUString( RTL_CONSTASCII_USTRINGPARAM("Configuration tree corrupt: Invalid Node found") ),
										static_cast<XNameAccess*>(this) );
	}

	return pValue->getValue();
}


//-----------------------------------------------------------------------------------
Any SAL_CALL HierarchyAccessImpl::getByName( const OUString& aName ) 
	throw(css::container::NoSuchElementException, css::lang::WrappedTargetException, RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OReadSynchronized aReadGuard( getTreeSync() );
	NodeRef pFound = m_aSubtree->getChild(aName);
	if (!pFound) throw css::container::NoSuchElementException();
	return makeElement(pFound, TreePath(1,m_aSubtree));
}
 
//-----------------------------------------------------------------------------------
namespace {
	class CollectNames :  public NodeAction
	{
	public:
		NameList const& list() const { return aList; }

		CollectNames() : aList() {}

		void handle(ValueNode const& aValue)	{ add(aValue); }
		void handle(ISubtree const&  m_aSubtree)	{ add(m_aSubtree); }

		void add(INode const& aNode) 
		{ 
			aList.push_back(aNode.getName());
		}
	private:
		NameList aList;
	};
}

//-----------------------------------------------------------------------------------
Sequence< OUString > SAL_CALL HierarchyAccessImpl::getElementNames(  ) throw( RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OReadSynchronized aReadGuard( getTreeSync() );
	CollectNames aCollect;
	m_aSubtree->forEachChild(aCollect);

	return makeSequence( aCollect.list() );

}

//-----------------------------------------------------------------------------------
// TO DO: optimization - cache found node
//-----------------------------------------------------------------------------------
sal_Bool SAL_CALL HierarchyAccessImpl::hasByName( const OUString& aName ) throw(RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OReadSynchronized aReadGuard( getTreeSync() );
	return m_aSubtree->getChild(aName) != 0;
}

// XHierarchicalNameAccess
//-----------------------------------------------------------------------------------

// if NULL is returned, aPath may have a Partial path appended
NodeRef HierarchyAccessImpl::findDescendant(OUString const& aRelativePath, TreePath& aPath)
{
	checkInitialized();

	ConfigurationName aName( aRelativePath );
	if (!normalizeNestedPath(aName))
		return 0;

	ConfigurationName::Iterator it = aName.begin();	
	ConfigurationName::Iterator const stop	= aName.end();

	for(TreeRef pTree = m_aSubtree; pTree != 0; )
	{
		NodeRef pFound = pTree->getChild(*it);
		aPath.push_back(pTree);

		if (++it == stop || !pFound)
			return pFound;

		pTree = pFound->asISubtree();
	}

	return 0;
}
//-----------------------------------------------------------------------------------

Any SAL_CALL HierarchyAccessImpl::getByHierarchicalName( const OUString& aName ) throw(css::container::NoSuchElementException, RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);

	OReadSynchronized aReadGuard( getTreeSync() );
	TreePath aPath;
	NodeRef pFound = findDescendant(aName,aPath);
	
	OSL_ENSHURE(pFound == 0 || aPath.front() == m_aSubtree, "Inconsistent Tree path handling");
	return makeElement(pFound, aPath);
}

//-----------------------------------------------------------------------------------
// TO DO: optimization - cache found node
//-----------------------------------------------------------------------------------
sal_Bool SAL_CALL HierarchyAccessImpl::hasByHierarchicalName( const OUString& aName ) throw(RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);

	OReadSynchronized aReadGuard( getTreeSync() );

	TreePath aPath;
	return findDescendant(aName,aPath) != 0;
}

//-----------------------------------------------------------------------------------
	// XInitialization
//-----------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------
void HierarchyAccessImpl::checkInitialized()
{
	OSL_ENSHURE(m_aSubtree, "Uninitialized or invalid HierarchyAccessImpl");
	if (m_aSubtree == 0)
		throw css::lang::DisposedException(
							OUString::createFromAscii("No Configuration Node in HierarchyAccess"), 
							static_cast<css::container::XHierarchicalNameAccess*>(this));
	checkAlive();
}

}


