/*************************************************************************
 *
 *  $RCSfile: confbaseelement.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "confbaseelement.hxx"
#include "confbaseaccess.hxx"

#include "cmtreemodel.hxx"

#include <stl/algorithm>	// for count
#include <stl/numeric>	// for accumulate
#include <osl/diagnose.h>

namespace configmgr
{

	using uno::Any;
	using uno::Reference;

	using uno::RuntimeException;
	using css::lang::NoSupportException;


	// Constructors
	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex)
		: m_aBasePath( OUString() )
		, m_aAncestors()
		, m_rMutex(_rMutex)
	{
	}
	//------------------------------------------------------------------------------------------------------------------
	void HierarchyElementImpl::initBasePath(ConfigurationName const& aBasePath)
	{
		OSL_ENSURE(m_aBasePath.empty(),"Reinitializing path not allowed");
		OSL_ENSURE(m_aAncestors.empty(),"Reinitializing path not allowed");
		OSL_ENSURE(!aBasePath.isRelative(),"Base path must be absolute");
		m_aBasePath = aBasePath;
	}
	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex, OUString _aBasePath)
		: m_aBasePath(_aBasePath, ConfigurationName::Absolute())
		, m_aAncestors()
		, m_rMutex(_rMutex)
	{
	}

	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex, TreeRef pParent, OUString _aBasePath)
		: m_aBasePath(_aBasePath, ConfigurationName::Absolute())
		, m_aAncestors(1,pParent)
		, m_rMutex(_rMutex)
	{
		OSL_ENSURE(pParent,"NULL Parent Element not allowed");
		// if (pParent) m_aAncestors.clear();	// the remedy
	}

	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex, TreePath const& aPath, OUString _aBasePath)
		: m_aBasePath(_aBasePath, ConfigurationName::Absolute())
		, m_aAncestors(aPath)
		, m_rMutex(_rMutex)
	{
		OSL_ENSURE(std::count(aPath.begin(),aPath.end(),TreeRef(0)) == 0,"NULL Parent Elements not allowed");
	}

	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex, HierarchyElementImpl const& aElement, TreeRef pParent)
		: m_aBasePath(aElement.m_aBasePath)
		, m_aAncestors(aElement.m_aAncestors)
		, m_rMutex(_rMutex)
	{
		OSL_ENSURE(pParent,"NULL Parent Element not allowed");

		//if (pParent) // the remedy
		m_aAncestors.push_back(pParent);	
	}

	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex, HierarchyElementImpl const& aElement, TreePath const& aPath)
		: m_aBasePath(aElement.m_aBasePath)
		, m_aAncestors(aElement.m_aAncestors)
		, m_rMutex(_rMutex)
	{
		OSL_ENSURE(std::count(aPath.begin(),aPath.end(),TreeRef(0)) == 0,"NULL Parent Elements not allowed");

		m_aAncestors.insert(m_aAncestors.end(), aPath.begin(),aPath.end());	
	}

	//------------------------------------------------------------------------------------------------------------------
	HierarchyElementImpl::HierarchyElementImpl(::osl::Mutex& _rMutex, HierarchyElementImpl const& aElement, ParentElement)
		: m_aBasePath(aElement.m_aBasePath)
		, m_aAncestors(aElement.m_aAncestors)
		, m_rMutex(_rMutex)
	{
		OSL_ENSURE(!m_aAncestors.empty(),"Parent Element not available");

		//if (!m_aAncestors.empty()) // the remedy
		m_aAncestors.pop_back();		
	}


	// Interface methods
	// XChild
	//------------------------------------------------------------------------------------------------------------------
	Reference< uno::XInterface > SAL_CALL HierarchyElementImpl::getParent(  ) throw(RuntimeException)
	{
		::osl::MutexGuard aGuard(m_rMutex);
		checkAlive();
		if (m_aAncestors.empty()) 
			return Reference< uno::XInterface >(0);

		Reference<css::container::XHierarchicalNameAccess> xParentAccess( getParentAccess() );
		if (xParentAccess.is())
			// by definition, the object returned by getParentAccess has to be acquired once
			xParentAccess->release();
		return xParentAccess;
	}

	// XNamed
	//------------------------------------------------------------------------------------------------------------------
	OUString SAL_CALL HierarchyElementImpl::getName(  ) throw(RuntimeException)
	{
		::osl::MutexGuard aGuard(m_rMutex);
		checkAlive();
		return getThisNode().getName();
	}

		
	// XHierarchicalName
	//------------------------------------------------------------------------------------------------------------------
	ConfigurationName appendName(ConfigurationName const& _rBasePath, INode const* pChild) throw ( RuntimeException )
	{
		OSL_ENSURE(pChild != 0, "Unexpected NULL pointer");
		return ConfigurationName(_rBasePath, pChild->getName());
	}

	//------------------------------------------------------------------------------------------------------------------
	OUString SAL_CALL HierarchyElementImpl::getHierarchicalName(  ) throw(RuntimeException)
	{
		::osl::MutexGuard aGuard(m_rMutex);
		checkAlive();
		return appendName(getHierarchicalName(m_aBasePath, m_aAncestors), &getThisNode()).fullName();
	}

	//------------------------------------------------------------------------------------------------------------------
	ConfigurationName HierarchyElementImpl::getHierarchicalName(const ConfigurationName& _rVeryBase, const TreePath& _rPath)
	{
		return std::accumulate(_rPath.begin(), _rPath.end(), _rVeryBase, appendName);
	}

	//------------------------------------------------------------------------------------------------------------------
	OUString SAL_CALL HierarchyElementImpl::composeHierarchicalName( const OUString& aRelativeName ) 
		throw(css::lang::IllegalArgumentException, NoSupportException, RuntimeException)
	{
		::osl::MutexGuard aGuard(m_rMutex);
		checkAlive();
		ConfigurationName aPath = std::accumulate(beginPath(),endPath(),m_aBasePath,appendName);
		aPath = appendName(aPath,&getThisNode());
		return ConfigurationName(aPath,aRelativeName).fullName();
	}

	//------------------------------------------------------------------------------------------------------------------
	//--- Unsupported interface methods. --------------------------------------------------------------------------
	//------------------------------------------------------------------------------------------------------------------

	// Unsupported methods. May be needed for elements to be added to a container node ? If so, override 
	//------------------------------------------------------------------------------------------------------------------

	void SAL_CALL HierarchyElementImpl::setParent( const Reference< uno::XInterface >& Parent ) 
		throw(NoSupportException, RuntimeException)
	{
		throw NoSupportException(
				OUString(RTL_CONSTASCII_USTRINGPARAM("Configuration: cannot move Entry")), 
				static_cast<css::container::XChild*>(this) );
	}

	//------------------------------------------------------------------------------------------------------------------
	void SAL_CALL HierarchyElementImpl::setName( const OUString& aName_ ) throw(RuntimeException)
	{
		checkAlive();
		throw CantRenameException( aName_, static_cast<css::container::XNamed*>(this));
	}

	//------------------------------------------------------------------------------------------------------------------

} // namespace configmgr


