/*************************************************************************
 *
 *  $RCSfile: confchangesset.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#ifndef CONFIGMGR_API_CHANGESSET_HXX_
#include "confchangesset.hxx"
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef _CONFIGMGR_TRACER_HXX_
#include "tracer.hxx"
#endif

namespace configmgr
{

//..........................................................................
void OChangeCounter::handle(ValueChange const& aValueNode)
{
	++nCount;
}

void OChangeCounter::handle(AddNode const& aAddNode)
{
	++nCount;
}

void OChangeCounter::handle(RemoveNode const& aRemoveNode)
{
	++nCount;
}

//..........................................................................
void OChangeCollector::push_back(const util::ElementChange& _rChange)
{
	m_aChanges.push_back(_rChange);
}

//..........................................................................
util::ChangesSet OChangeCollector::getChanges() const
{
	if (m_aChanges.empty()) return util::ChangesSet(0);

	return uno::Sequence<util::ElementChange>( &m_aChanges.front(), m_aChanges.size() );
}

//..........................................................................
void OChangeCollector::handle(ValueChange const& _rValueNode)
{
	ConfigurationName aChangeName(m_aCurrentPath,_rValueNode.getNodeName());

	util::ElementChange aNewChange;
	aNewChange.Accessor <<= aChangeName.fullName();
	aNewChange.Element = _rValueNode.getNewValue();
	aNewChange.ReplacedElement = _rValueNode.getOldValue();

	push_back(aNewChange);
}

//..........................................................................
void OChangeCollector::handle(AddNode const& _rAddNode)
{
	ConfigurationName aChangeName(m_aCurrentPath,_rAddNode.getNodeName());

	util::ElementChange aNewChange;
	aNewChange.Accessor <<= aChangeName.fullName();

	INode* pAddedNode = _rAddNode.getAddedNode();
	if (pAddedNode == 0)
	{
		// unsafe operation: pointer may be dangling if used carelessly
		pAddedNode = _rAddNode.getAddedNode_unsafe();

		OSL_ENSHURE(pAddedNode, "OChangeCollector::handle : invalid 'AddNode' change : no contained node !");
		OSL_TRACE("Warning: notifying 'AddNode' change that does not own its content\n\r");
	}
	
	if (pAddedNode->asValueNode())
	{
		aNewChange.Element = pAddedNode->asValueNode()->getValue();
	}
	else
	{
		OSL_ENSHURE(pAddedNode->asISubtree(), "OChangeCollector::handle : invalid node : no value node, no sub tree !");
//		aNewChange.Element = 
		// TODO : build an element wrapping the ISubtree
	}

	push_back(aNewChange);
}

//..........................................................................
void OChangeCollector::handle(RemoveNode const& _rRemoveNode)
{
	ConfigurationName aChangeName(m_aCurrentPath,_rRemoveNode.getNodeName());

	util::ElementChange aNewChange;
	aNewChange.Accessor <<= aChangeName.fullName();
	aNewChange.ReplacedElement = _rRemoveNode.getOldNodeValue();
		// != VOID only if the node was a value node

	push_back(aNewChange);
}

//..........................................................................
void OChangeCollector::handle(SubtreeChange const& _rSubtree)
{
	m_aCurrentPath = ConfigurationName(m_aCurrentPath, _rSubtree.getNodeName());
	applyToChildren( _rSubtree );
	m_aCurrentPath = m_aCurrentPath.getParentName();
}

//..........................................................................
} // namespace


