/*************************************************************************
 *
 *  $RCSfile: confgroupaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_GROUPACCESS_HXX_
#include "confgroupaccess.hxx"
#endif

//........................................................................
namespace configmgr
{
//........................................................................
//========================================================================
//= HierarchyUpdateElementGroup
//========================================================================
namespace {
	inline
	NodePropertySet::Options updatableProperties(bool bMayHaveDefaults)
	{
		NodePropertySet::Options opt = bMayHaveDefaults	? NodePropertySet::withPropertyState 
														: NodePropertySet::noPropertyState;
		
		return opt | NodePropertySet::withUpdates;
	}
}

HierarchyUpdateElementGroup::HierarchyUpdateElementGroup(
	BroadcastHelper& _rBHelper,
	HierarchyUpdateRoot* _pUpdateRoot,
	SubtreeChange* _pModifications,
	const TreePath& _rRelativeLocation,
	bool bMayHaveDefaults
)
: HierarchyUpdateElement(_rBHelper,_pUpdateRoot,_pModifications,_rRelativeLocation)
, NodePropertySet(_rBHelper, updatableProperties(bMayHaveDefaults) )
{
}

// XInterface
uno::Any SAL_CALL HierarchyUpdateElementGroup::queryInterface(::com::sun::star::uno::Type const& rType) 
	throw (uno::RuntimeException)
{
	uno::Any aRet( HierarchyUpdateElement::queryInterface(rType) );

	if (!aRet.hasValue()) NodePropertySet::queryPropertyInterfaces(aRet,rType);

	return aRet;
}

// XTypeProvider
uno::Sequence< uno::Type > SAL_CALL HierarchyUpdateElementGroup::getTypes() 
	throw (uno::RuntimeException)
{
	using namespace uno;

	Sequence<Type> aTypes = HierarchyUpdateElement::getTypes();

	NodePropertySet::addPropertyTypes(aTypes);

	return aTypes;
}

// misc (Component support)
void SAL_CALL HierarchyUpdateElementGroup::disposing()
{
	NodePropertySet::disposing();
	HierarchyUpdateElement::disposing();
}

void HierarchyUpdateElementGroup::executeLocalChanges(ValueChanges& rChanges)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException)
{
	fireVetoableChanges(rChanges);
	HierarchyUpdateElement::executeLocalChanges(rChanges);
	firePropertyChanges(rChanges);
}


//==========================================================================
//= OFloatableGroup
//==========================================================================
OFloatableGroup::OFloatableGroup(
	BroadcastHelper& _rBHelper,
	const ::rtl::OUString& _rTemplateName,
	const TreePath& _rContainerPath,
	::std::auto_ptr<ISubtree> _pTemplateInstance,
	HierarchyUpdateRoot* _pUpdateRoot
)
: OFloatableNode(_rBHelper,_rTemplateName,_rContainerPath,_pTemplateInstance,_pUpdateRoot)
, NodePropertySet(_rBHelper,updatableProperties(false))
{
}

	// XInterface
uno::Any SAL_CALL OFloatableGroup::queryInterface(uno::Type const& rType) throw (uno::RuntimeException)
{
	uno::Any aRet( OFloatableNode::queryInterface(rType) );

	if (!aRet.hasValue()) NodePropertySet::queryPropertyInterfaces(aRet,rType);

	return aRet;
}

	// XTypeProvider
uno::Sequence< uno::Type > SAL_CALL OFloatableGroup::getTypes() throw (uno::RuntimeException)
{
	using namespace uno;

	Sequence<Type> aTypes = OFloatableNode::getTypes();

	NodePropertySet::addPropertyTypes(aTypes);

	return aTypes;
}


void SAL_CALL OFloatableGroup::disposing()
{
	NodePropertySet::disposing();
	OFloatableNode::disposing();
}


void OFloatableGroup::executeLocalChanges(ValueChanges& rChanges)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException)
{
	fireVetoableChanges(rChanges);
	OFloatableNode::executeLocalChanges(rChanges);
	firePropertyChanges(rChanges);
}



//========================================================================
//= HierarchyUpdateRootGroup
//========================================================================

HierarchyUpdateRootGroup::HierarchyUpdateRootGroup(
	BroadcastHelper& _rbHelper, 
	TreeRef _pTree,	const ConfigurationName& _rName,
	const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
	const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
	IConfigBroadcaster& rEventSource_,
	const ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >& _rxConverter
)
: HierarchyUpdateRoot(_rbHelper,_pTree,_rName,_rUpdater,_rTemplateFactory,rEventSource_,_rxConverter)
, NodePropertySet(_rbHelper,updatableProperties(true))
{
}

	// XInterface
uno::Any SAL_CALL HierarchyUpdateRootGroup::queryInterface(uno::Type const& rType) 
	throw (uno::RuntimeException)
{
	uno::Any aRet( HierarchyUpdateRoot::queryInterface(rType) );

	if (!aRet.hasValue()) NodePropertySet::queryPropertyInterfaces(aRet,rType);

	return aRet;
}

	// XTypeProvider
uno::Sequence< uno::Type > SAL_CALL HierarchyUpdateRootGroup::getTypes() 
	throw (uno::RuntimeException)
{
	using namespace uno;

	Sequence<Type> aTypes = HierarchyUpdateRoot::getTypes();

	NodePropertySet::addPropertyTypes(aTypes);

	return aTypes;
}

void SAL_CALL HierarchyUpdateRootGroup::disposing()
{
	NodePropertySet::disposing();
	HierarchyUpdateRoot::disposing();
}

// NodePropertySet overridables
void HierarchyUpdateRootGroup::executeLocalChanges(ValueChanges& rChanges)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException)
{
	fireVetoableChanges(rChanges);
	HierarchyUpdateRoot::executeLocalChanges(rChanges);
	firePropertyChanges(rChanges);
}

//........................................................................
} // namespace configmgr
//........................................................................


