/*************************************************************************
 *
 *  $RCSfile: confreadaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_READACCESS_HXX_
#include "confreadaccess.hxx"
#endif

#ifndef _COM_SUN_STAR_BEANS_PROPERTYVETOEXCEPTION_HPP_
#include <com/sun/star/beans/PropertyVetoException.hpp>
#endif
#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif

namespace configmgr
{

/////////////////////////////////////////////////////////////////////////////////////////////
namespace {
	inline
	NodePropertySet::Options readonlyOptions(bool bSupportDefaults)
	{
		NodePropertySet::Options opt = NodePropertySet::asReadOnly;
		if (bSupportDefaults)
			opt = opt | NodePropertySet::withPropertyState;
		else
			opt = opt | NodePropertySet::noPropertyState;
		return opt;
	}
}
// Constructors & Destructors
PropsetReadAccess::PropsetReadAccess(BroadcastHelper& rBHelper_, IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, OUString aBasePath, bool withPropertyState)
: HierarchyReadAccess(rBHelper_,rSource, pSync, pNode, aBasePath)
, NodePropertySet(rBHelper_,readonlyOptions(withPropertyState))
{}

PropsetReadAccess::PropsetReadAccess(BroadcastHelper& rBHelper_, IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath, bool withPropertyState)
: HierarchyReadAccess(rBHelper_,rSource, pSync, pNode, pPath, aBasePath)
, NodePropertySet(rBHelper_,readonlyOptions(withPropertyState))
{}

// TODO: In the following withPropertyState should be handled automatically
PropsetReadAccess::PropsetReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode, bool withPropertyState)
: HierarchyReadAccess(rBHelper_, aAncestorElement, pNode, pParentNode)
, NodePropertySet(rBHelper_,readonlyOptions(withPropertyState))
{}

PropsetReadAccess::PropsetReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath, bool withPropertyState)
: HierarchyReadAccess(rBHelper_, aAncestorElement, pNode, pPath)
, NodePropertySet(rBHelper_,readonlyOptions(withPropertyState))
{}

PropsetReadAccess::PropsetReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, ParentElement up, bool withPropertyState)
: HierarchyReadAccess(rBHelper_, aAncestorElement, up)
, NodePropertySet(rBHelper_,readonlyOptions(withPropertyState))
{}

PropsetReadAccess::~PropsetReadAccess()
{}

uno::Any SAL_CALL PropsetReadAccess::queryInterface(uno::Type const& rType) throw (uno::RuntimeException)
{
	uno::Any aRet( HierarchyReadAccess::queryInterface(rType) );

	if (!aRet.hasValue())
		NodePropertySet::queryPropertyInterfaces(aRet,rType);

	return aRet;
}

uno::Sequence< uno::Type > SAL_CALL PropsetReadAccess::getTypes() throw (uno::RuntimeException)
{
	uno::Sequence<uno::Type> aTypes( HierarchyReadAccess::getTypes() );
	
	NodePropertySet::addPropertyTypes(aTypes);

	return aTypes;
}
	
void SAL_CALL PropsetReadAccess::disposing()
{
	NodePropertySet::disposing();
	HierarchyReadAccess::disposing();
}


// NodePropertySet overrideable functions

// Not needed in this implementation - returns null
uno::Reference< css::script::XTypeConverter > PropsetReadAccess::getTypeConverter() const
{
	OSL_ASSERT(!"Read-only configuration access cannot provide a type converter - and should not need one");
	return 0;
}

// Not supported in this implementation - vetoes always
void PropsetReadAccess::executeLocalChanges(ValueChanges& rChange)
{
	OSL_ENSURE(false,"Trying to apply changes in read-only configuration access");
	throw beans::PropertyVetoException(
			OUString::createFromAscii("Trying to change values in read-only configuration access"),
			uno::Reference<beans::XPropertySet>(this)
		);
}

// Not supported in this release :-(
bool PropsetReadAccess::loadDefaults() throw (uno::RuntimeException)
{
	// whom to delegate to ??
	return false;
}

namespace {
	struct UnsychronizedData : ISynchronizedData
	{
		void acquireReadAccess() const;
		void releaseReadAccess() const;
		void acquireWriteAccess();
		void releaseWriteAccess();
	};
	void UnsychronizedData::acquireReadAccess() const {}
	void UnsychronizedData::releaseReadAccess() const {}
	void UnsychronizedData::acquireWriteAccess() {}
	void UnsychronizedData::releaseWriteAccess() {}
}

// Not needed in this implementation - returns a dummy or null
ISynchronizedData * PropsetReadAccess::getUpdateSync()
{
	static UnsychronizedData notAvailable;
	return &notAvailable;
}


uno::Any PropsetReadAccess::getChildAccess(TreeRef aChildNode) throw (uno::RuntimeException)
{
	return HierarchyReadAccess::getChildAccess(aChildNode);
} 

TreeRef PropsetReadAccess::getThisTree() const
{
	return HierarchyReadAccess::getThisTree();
}

//--------------------------------------------------------------------------


}


