/*************************************************************************
 *
 *  $RCSfile: confsetaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_API_SETACCESS_HXX_
#include "confsetaccess.hxx"
#endif
#ifndef _CONFIGMGR_TREEACTIONS_HXX_
#include "treeactions.hxx"
#endif
#ifndef CONFIGMGR_API_ENCODENAME_HXX_
#include "encodename.hxx"
#endif

#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _CPPUHELPER_EXTRACT_HXX_
#include <cppuhelper/extract.hxx>
#endif
#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif
#ifndef _CONFIGMGR_TRACER_HXX_
#include "tracer.hxx"
#endif

#define UNISTRING(s)	::rtl::OUString::createFromAscii(s)
#define THISREF()		static_cast< XSingleServiceFactory* >(this)

//........................................................................
namespace configmgr
{
//........................................................................

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::container;
using namespace ::com::sun::star::util;
using namespace ::com::sun::star::script;
using namespace ::cppu;
using namespace ::rtl;
using namespace ::osl;

//==========================================================================
//= helper
//==========================================================================
//........................................................................
struct CollectAddedNames : public ChangeTreeAction
{
	Sequence< ::rtl::OUString > aNames;

	virtual void handle(ValueChange const& aValueNode) { /* not interested in */ }
	virtual void handle(AddNode const& aAddNode);
	virtual void handle(RemoveNode const& aRemoveNode) { /* not interested in */ }
	virtual void handle(SubtreeChange const& aSubtree) { /* not interested in */ }
};

//........................................................................
void CollectAddedNames::handle(AddNode const& _rAddNode)
{
	sal_Int32 nOldLen = aNames.getLength();
	aNames.realloc(nOldLen + 1);
	aNames[nOldLen] = _rAddNode.getNodeName();
}

//==========================================================================
//= OFloatableNode
//==========================================================================
//--------------------------------------------------------------------------
OFloatableNode::OFloatableNode(BroadcastHelper& _rBHelper, const ::rtl::OUString& _rTemplateName, const TreePath& _rContainerPath, ::std::auto_ptr<ISubtree> _pTemplateInstance, HierarchyUpdateRoot* _pUpdateRoot)
	:HierarchyUpdateElement(_rBHelper, _pUpdateRoot, _rContainerPath, _pTemplateInstance.get())
	,m_sChildTemplateName(_rTemplateName)
	,m_aNode(_pTemplateInstance)
		// this works cause the base class is initialized first, so the above get() is valid (it's invalid after we
		// made this assignment, cause after that _pTemplateInstance is empty).
{
}

//--------------------------------------------------------------------------
sal_Int64 SAL_CALL OFloatableNode::getSomething( const Sequence< sal_Int8 >& _rId ) throw(RuntimeException)
{
	if (_rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  _rId.getConstArray(), 16 ) )
		return reinterpret_cast<sal_Int64>(this);

	return HierarchyUpdateElement::getSomething(_rId);
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OFloatableNode::getUnoTunnelImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

//--------------------------------------------------------------------------
HierarchyAccessImpl* OFloatableNode::getParentAccess()
{
	return isFloating() ? NULL : HierarchyUpdateElement::getParentAccess();
}

//--------------------------------------------------------------------------
::rtl::OUString SAL_CALL OFloatableNode::getName(  ) throw (RuntimeException)
{
	return OFloatableNode::isFloating() ? ::rtl::OUString() : HierarchyUpdateElement::getName();
}

//--------------------------------------------------------------------------
::std::auto_ptr<ISubtree> OFloatableNode::inserted(const TreePath& _rNewAncestors)
{
	// set my new ancestors
	movedNode(_rNewAncestors);
		// TODO : all of my descendants (which HierarchyUpdateBase holds in m_aDescendants) have to be notified
		// of the new path, too. So they can adjust their ancestors, too (if they need it, at the moment I'm not
		// sure about it)
	return m_aNode;
}


//==========================================================================
//= OSetNode
//==========================================================================
//--------------------------------------------------------------------------
OSetNode::OSetNode(::osl::Mutex& _rSafety, const ::vos::ORef< IRefCountedTemplateProvider >& _rNodeFactory, const ::rtl::OUString& _rTemplateName, sal_Bool _bAllowUnescaped)
	:m_aNodeFactory(_rNodeFactory)
	,m_sContainerTemplateName(_rTemplateName)
	,m_rMutex(_rSafety)
	,m_bAllowUnescaped(_bAllowUnescaped)
{
}

//--------------------------------------------------------------------------
std::auto_ptr<AddNode> OSetNode::createAddTransaction(OFloatableNode& rAddedNode_, ::rtl::OUString const& sNewName_)
{
	// the ancestors of the new node
	TreePath aAncestors;
	getNodePath(aAncestors);
	aAncestors.push_back(getThisTree());

	// get the responsibility for the tree of the new node
	::std::auto_ptr<INode> pFloatingNode(rAddedNode_.inserted(aAncestors).release());
	pFloatingNode->setName(sNewName_);

	// build a change
	return std::auto_ptr<AddNode>(new AddNode(pFloatingNode, sNewName_));
}

//--------------------------------------------------------------------------
void SAL_CALL OSetNode::insertByName( const ::rtl::OUString& _rName, const Any& _rElement ) throw(IllegalArgumentException, ElementExistException, WrappedTargetException, RuntimeException)
{
	CFG_TRACE_INFO("folder node: inserting a new child (named %s)", OUSTRING2ASCII(_rName));
	::osl::MutexGuard aGuard(m_rMutex);
	OSL_ENSHURE(getTreeSync() != getChangeTreeSync(), "OSetNode::insertByName : sync objects should not be identical !");
		// below we acquire the read access on the change tree sync object, and afterwards the write access on the tree
		// object. As our current implementation does not allow this to be done for the same object, we need this
		// assertion

//	OReadSynchronized aReadGuard(getTreeSync());
		// TODO : at the moment we don't have tree synchronization
	OWriteSynchronized aWriteGuard(getChangeTreeSync());
		// we're going to write into the changes tree below

	// the tree itself
	TreeRef pTree = getThisTree();
	OSL_ENSHURE(pTree != NULL, "OSetNode::insertByName : invalid tree !");
	if (!pTree)
	{
		CFG_TRACE_ERROR_NI("folder node: I'm invalid. this may crash", OUSTRING2ASCII(_rName));
		throw RuntimeException(UNISTRING("An fatal error occured. The object is invalid."), THISREF());
	}

	// the changes tree
	SubtreeChange* pModifications = getChangesTree();

	// check the arguments
	if (!_rName.getLength())
	{
		CFG_TRACE_WARNING_NI("folder node: invalid name");
		throw IllegalArgumentException(UNISTRING("The name must not be empty."), THISREF(), 1);
	}
	if (!m_bAllowUnescaped)
		escaped_name::validateName(_rName, THISREF(), 1);

	Change* pExistentChange = NULL;
	if (pModifications)
		pExistentChange = pModifications->getChange(_rName);

	NodeRef pExistentNode = pTree->getChild(_rName);
	if (pExistentNode)
	{
		CFG_TRACE_WARNING_NI("folder node: node exists");
		if (pExistentChange && pExistentChange->isA(RemoveNode::getStaticType()))
			throw ElementExistException(UNISTRING("The element has already been removed. You need to commit these removal before you can add another node under this name."), THISREF());

		throw ElementExistException((UNISTRING("There already is an element named ") += _rName) += UNISTRING("."), THISREF());
	}

	if (pExistentChange)
	{
		CFG_TRACE_WARNING_NI("folder node: node change exists");
		if (pExistentChange->isA(AddNode::getStaticType()))
			throw ElementExistException((UNISTRING("You already added a node named ") += _rName) += UNISTRING(", though the changes are not committed yet."), THISREF());
	}

	Reference< XUnoTunnel > xTunnel;
	::cppu::extractInterface(xTunnel, _rElement);
	OFloatableNode* pImplementation = NULL;
	if (xTunnel.is())
		pImplementation = reinterpret_cast< OFloatableNode* >(xTunnel->getSomething(OFloatableNode::getUnoTunnelImplementationId()));

	if (!pImplementation)
		throw IllegalArgumentException(UNISTRING("The object you tried to insert has not been created by this node. Please use com.sun.star.lang.XSingleServiceFactory::createInstance."), THISREF(), 2);

	if (!pImplementation->isFloating())
		throw IllegalArgumentException(UNISTRING("The object you tried to insert is part of a tree already. You can't insert it twice."), THISREF(), 2);

	if (!pImplementation->getTemplateName().equals(m_sContainerTemplateName))
		throw IllegalArgumentException(UNISTRING("The object you tried to insert does not match the template of this node, which means it has not been created by this node."), THISREF(), 2);

	// all right, here we go ....

	// build a change
	AddNode* pNewChange = this->createAddTransaction(*pImplementation,_rName).release();
	::std::auto_ptr<Change> aNewChange(pNewChange);

	if (pNewChange == 0)
	{
		OSL_ENSHURE(false, "Could not create transaction to add the new set node element");
		throw RuntimeException(UNISTRING("Could not create a transaction to add the new set node element '") += OUString(_rName) += UNISTRING("'."), THISREF());
	}

	// put it into the changes tree
	if (pModifications)
	{
		CFG_TRACE_WARNING_NI("folder node: creating a new addNode change");
		// create the change
		pModifications->addChange(aNewChange);
	}
	else
	{
		// just add the tree. We're already part of a sub tree which is to be added on commitment
		const_cast<ISubtree*>(getThisTree())->addChild(pNewChange->releaseAddedNode());
	}

	// Warning ! pNewNode may not be ours any more
	// let derived classes do any knittings and notifications...
	insertedElement(*pNewChange, pImplementation);
}

//--------------------------------------------------------------------------
void SAL_CALL OSetNode::removeByName( const ::rtl::OUString& _rName ) throw(NoSuchElementException, WrappedTargetException, RuntimeException)
{
	CFG_TRACE_INFO("folder node: removing a child (named %s)", OUSTRING2ASCII(_rName));

	::osl::MutexGuard aGuard(m_rMutex);
	OSL_ENSHURE(getTreeSync() != getChangeTreeSync(), "OSetNode::removeByName : sync objects should not be identical !");
		// below we acquire the read access on the change tree sync object, and afterwards the write access on the tree
		// object. As our current implementation does not allow this to be done for the same object, we need this
		// assertion

//	OReadSynchronized aReadGuard(getTreeSync());
		// TODO : at the moment we don't have tree synchronization
	OWriteSynchronized aWriteGuard(getChangeTreeSync());
		// we're going to write into the changes tree below

	// the tree itself
	TreeRef pMeMyselfAndI = getThisTree();
	OSL_ENSHURE(pMeMyselfAndI != NULL, "OSetNode::removeByName : invalid tree !");
	if (!pMeMyselfAndI)
		throw RuntimeException(UNISTRING("An fatal error occured. The object is invalid."), THISREF());

	// the changes tree
	SubtreeChange* pModifications = getChangesTree();

	// check the arguments
	if (!_rName.getLength())
		throw IllegalArgumentException(UNISTRING("The name must not be empty."), THISREF(), 1);

	NodeRef pExistentNode = pMeMyselfAndI->getChild(_rName);
	Change* pExistentChange = NULL;
	if (pModifications)
		pExistentChange = pModifications->getChange(_rName);

	if (pExistentNode)
	{
		CFG_TRACE_INFO_NI("folder node: to-be-removed node exists as real node");
		if (pExistentChange)
		{
			// perhaps the node has already been removed ...
			if (pExistentChange->isA(RemoveNode::getStaticType()))
			{
				CFG_TRACE_INFO_NI("folder node: to-be-removed node has already been removed (locally)");
				throw NoSuchElementException(UNISTRING("The object has already been removed, though you did not commit the changes, yet."), THISREF());
			}
			else if (pExistentChange->isA(SubtreeChange::getStaticType()))
			{
				CFG_TRACE_INFO_NI("folder node: to-be-removed node is has a sub tree change");
#ifdef DEBUG
				OChangeActionCounter aCount;
				aCount.handle(*static_cast<SubtreeChange*>(pExistentChange));
				OSL_ENSHURE(0 == aCount.nValues + aCount.nAdds + aCount.nRemoves,
					"OSetNode::removeByName : You're removing a subtree with existing changes ! Sure you know what you're doing ?");
#endif
				// remember the old SubtreeChange
				auto_ptr<SubtreeChange> aRemovedSubtree(
					static_cast<SubtreeChange*>(pModifications->removeChange(_rName).release()));
				// create a new change saying the node should be removed

				RemoveNode* pThisRemoval = new RemoveNode(_rName);
				::std::auto_ptr<Change> aChangePtr(pThisRemoval);
				pModifications->addChange(aChangePtr);
	
				removedElement(_rName, aRemovedSubtree);

				return;
			}
			else
			{
				CFG_TRACE_ERROR_NI("folder node: unexpected: there is an unrecognized or invalid change for the node (type: %s).", pExistentChange->getType());
				OSL_ENSHURE(sal_False, "OSetNode::removeByName : existent node, but not removed and no subtree !");
				throw RuntimeException(UNISTRING("Fatal error. The object is inconsistent. No changes were made."), THISREF());
			}
		}
		else
		{
			// TODO : guarding of the read tree not implemented yet
//			aReadGuard.clear();
				// TODO : as always ... we need a write guard which can be used _in_addition_ to, rather than _instead_
				// of, a read guard. The current version is not really safe ....
//			OWriteSynchronized aWriteGuard(getTreeSync());

			if (pModifications)
			{	// there is a node, but no change yet, and we're integral part of an update hierarchy
				CFG_TRACE_INFO_NI("folder node: creating a removeNode change");
				// => just create the RemoveNode change ....
				RemoveNode* pThisRemoval = new RemoveNode(_rName);
				::std::auto_ptr<Change> aChangePtr(pThisRemoval);
				pModifications->addChange(aChangePtr);

				removedElement(_rName, auto_ptr<SubtreeChange>());
					// the second param means there was no subtree change before ....
				return;
			}
			else
			{	// there is a node, but no change yet, and we're part of a (still) floating sub tree
				// => just remove the one child node from the ISubtree we represent. This is no problem,
				// as this sub tree does not belong to the overall-structure, but only to the floating tree.
				// So it is just referenced within this floating tree and we can just remove it ....
				CFG_TRACE_INFO_NI("folder node: the to-be-removed node is part of a floating tree");

				auto_ptr<INode> aRemovedNode(const_cast<ISubtree*>(pMeMyselfAndI)->removeChild(_rName));
				removedElement(_rName, aRemovedNode);
				return;
			}
		}
	}
	else	// !pExistentNode
	{
		// there is no such node in the tree, but perhaps in the changes tree
		if (pExistentChange)
		{
			CFG_TRACE_INFO_NI("folder node: the to-be-removed node is not really existent, but has beed added locally");
			OSL_ENSHURE(pExistentChange->isA(AddNode::getStaticType()),
				"OSetNode::removeByName : inconsistence ... should be an added node !");
				// hey, we're a set node, we don't have any changes other than AddNode and RemoveNode
			throw NoSuchElementException(UNISTRING("A node with this name has just been added. You need to commit this addition before removing the node."), THISREF());

			// probably we could just remove the changes of all descendants on that sub tree. But this would be somewhat
			// difficult before we have a real concept for disposing API objects (all the objects below that node
			// would have to be disposed, then).
			// And besides that, it does not make sense to add a node and remove it afterwards before committing the
			// changes, does it ?
		}

		CFG_TRACE_ERROR_NI("folder node: there is no such node ...");
		throw NoSuchElementException((UNISTRING("There is no node named ") += _rName) += UNISTRING("."), THISREF());
	}

	OSL_ENSHURE(sal_False, "OSetNode::removeByName : reached the unreachable !");
		// _nothing_ should survive the above .... There is a throw or a return for every case ...
}

//--------------------------------------------------------------------------
void SAL_CALL OSetNode::replaceByName( const ::rtl::OUString& _rName, const Any& _rElement ) throw(IllegalArgumentException, NoSuchElementException, WrappedTargetException, RuntimeException)
{
	OSL_ENSHURE(sal_False, "OSetNode::replaceByName : invalid call !");
		// normally, this method should not be called. HierarchyUpdateBase has it's own replaceByName, and all updatable
		// object should be derived from HierarchyUpdateBase.

	throw IllegalArgumentException(UNISTRING("Replacing elements is not supported for sub trees."), THISREF(), -1);
}

//--------------------------------------------------------------------------
Reference< XInterface > SAL_CALL OSetNode::createInstance(  ) throw(Exception, RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);

	// create an instance of the template for our children
	::std::auto_ptr<INode> pTemplateInstance;
	if (m_aNodeFactory.isValid() && m_sContainerTemplateName.getLength())
		pTemplateInstance = m_aNodeFactory->createInstance(m_sContainerTemplateName);

	if (!pTemplateInstance.get())
		throw Exception(UNISTRING("Unable to create the template instance."), THISREF());

	ISubtree* pTemplateAsTree = pTemplateInstance->asISubtree();
	if (!pTemplateAsTree)
	{
		OSL_ENSHURE(sal_False, "OSetNode::createInstance : templates for value nodes not supported currently !");
		throw Exception(UNISTRING("Invalid template. Templates for value nodes not supported currently."), THISREF());
	}

	// move the responsibility for the node to the to-be-created object
	::std::auto_ptr<ISubtree> pFloatingTree(pTemplateAsTree);
	pTemplateInstance.release();

	TreePath aCompleteThisPath;
	getNodePath(aCompleteThisPath);
	aCompleteThisPath.push_back(getThisTree());
	Reference< XInterface > xReturn = static_cast< XUnoTunnel* >(createFloatingElement(aCompleteThisPath, pFloatingTree));
	if (xReturn.is())
		// per definitionem, the returned object is acquired once
		xReturn->release();

	return xReturn;
}

//--------------------------------------------------------------------------
Reference< XInterface > SAL_CALL OSetNode::createInstanceWithArguments( const Sequence< Any >& _rArguments ) throw(Exception, RuntimeException)
{
	return createInstance();
}

// XStringEscape
//--------------------------------------------------------------------------
OUString SAL_CALL OSetNode::escapeString( const OUString& aString ) throw(IllegalArgumentException, RuntimeException)
{ 
	return escaped_name::escapeString(aString,this,1); 
}

//--------------------------------------------------------------------------
OUString SAL_CALL OSetNode::unescapeString( const OUString& aEscapedString ) throw(IllegalArgumentException, RuntimeException)
{ 
	return escaped_name::unescapeString(aEscapedString,this,1); 
}

//========================================================================
//= OSetNodeUpdateElement
//========================================================================
//------------------------------------------------------------------------
OSetNodeUpdateElement::OSetNodeUpdateElement(
			BroadcastHelper& _rBHelper, HierarchyUpdateRoot* _pUpdateRoot, SubtreeChange* _pModifications,
			const TreePath& _rRelativeLocation)
	:HierarchyUpdateElement(_rBHelper, _pUpdateRoot, _pModifications, _rRelativeLocation)
	,OSetNode(_rBHelper.rMutex, _pUpdateRoot->getTemplateFactory(), _rRelativeLocation.back()->getChildTemplateName())
{
}

//------------------------------------------------------------------------
Any SAL_CALL OSetNodeUpdateElement::queryInterface(Type const& _rType) throw (RuntimeException)
{
	Any aReturn = HierarchyUpdateElement::queryInterface(_rType);
	if (!aReturn.hasValue())
		aReturn = OSetNode::queryInterface(_rType);
	return aReturn;
}

//------------------------------------------------------------------------
Sequence< Type > SAL_CALL OSetNodeUpdateElement::getTypes() throw (RuntimeException)
{
	return ::utl::concatSequences(
		HierarchyUpdateElement::getTypes(),
		OSetNode::getTypes()
	);
}

//------------------------------------------------------------------------
Any SAL_CALL OSetNodeUpdateElement::getByName( const ::rtl::OUString& _rName ) throw(NoSuchElementException, WrappedTargetException, RuntimeException)
{
	::osl::MutexGuard aGuard(HierarchyUpdateBase::m_rMutex);

	// first ask the base class
	Any aReturn;
	try
	{
		aReturn = HierarchyUpdateElement::getByName(_rName);
		OSL_ENSHURE(aReturn.hasValue(), "OSetNodeUpdateElement::getByName : base class returned nonsense !");
			// should have thrown an exception if there is no such element
	}
	catch (NoSuchElementException&)
	{
	}

	if (!aReturn.hasValue())
	{	// the element does not exist as tree, but perhaps as AddNode change

		NodeRef pNewlyAddedNode = NULL;

		OReadSynchronized aGuard(getChangeTreeSync());
		SubtreeChange* pModifications = getChangesTree();
		if (pModifications)
		{
			Change* pExistentChange = pModifications->getChange(_rName);
			if (pExistentChange && pExistentChange->isA(AddNode::getStaticType()))
			{
				AddNode* pAddChange = static_cast<AddNode*>(pExistentChange);
				pNewlyAddedNode = pAddChange->getAddedNode(); // Do we need the _unsafe version ?
			}
		}

		TreePath aPath(1, getThisTree());
		aGuard.clear();
		aReturn = makeElement(pNewlyAddedNode, aPath);
			// will throw an exception if the node is NULL
	}

	return aReturn;
}

//------------------------------------------------------------------------
Sequence< ::rtl::OUString > SAL_CALL OSetNodeUpdateElement::getElementNames(  ) throw(RuntimeException)
{
	::osl::MutexGuard aObjectGuard(HierarchyUpdateBase::m_rMutex);

	// the original names (which have a representation within the tree)
	Sequence< ::rtl::OUString > aNames = HierarchyUpdateElement::getElementNames();

	// plus the names which at the moment exist just as AddNode-changes
	OReadSynchronized aGuard(getChangeTreeSync());
	SubtreeChange* pModifications = getChangesTree();
	// if the tree is NULL, we're child of a set node itself, so either we're still floating, or we're already
	// inserted into the set node. In both cases the tree completly describes the node (all children can be found
	// in it)

	if (pModifications)
	{
		// the names of the added nodes
		CollectAddedNames aCollector;
		aCollector.applyToChildren(*pModifications);

		sal_Int32 nOldLen = aNames.getLength();
		aNames.realloc(nOldLen + aCollector.aNames.getLength());

		::rtl::OUString* pNames = aNames.getArray() + nOldLen;
		const ::rtl::OUString* pAdded = aCollector.aNames.getConstArray();
		for (sal_Int32 i=0; i<aCollector.aNames.getLength(); ++i, ++pNames, ++pAdded)
			*pNames = *pAdded;
	}

	return aNames;
}

//------------------------------------------------------------------------
sal_Bool SAL_CALL OSetNodeUpdateElement::hasByName( const ::rtl::OUString& _rName ) throw(RuntimeException)
{
	::osl::MutexGuard aObjectGuard(HierarchyUpdateBase::m_rMutex);

	if (HierarchyUpdateElement::hasByName(_rName))
		return sal_True;

	OReadSynchronized aGuard(getChangeTreeSync());
	SubtreeChange* pModifications = getChangesTree();
	return pModifications && (NULL != pModifications->getChange(_rName));
}

//------------------------------------------------------------------------
void OSetNodeUpdateElement::insertedElement(const AddNode& _rChange, OFloatableNode* _pNewElement)
{ 
	HierarchyUpdateElement::insertedNearestDescendant(_rChange.getNodeName(), _pNewElement);
	// tell the new element that it is part of a new hierarchy
	_pNewElement->insertedInto(getUpdateRoot());
	fireLocalChange(_rChange);
}

//------------------------------------------------------------------------
void OSetNodeUpdateElement::removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange)
{
	disposeDescendants(_rName);

	RemoveNode aChange(_rName); // No data here to setOldValue in the RemoveNode
	fireLocalChange(aChange);

	// the given SubtreeChange will be deleted when returning from this method. This should not be a problem,
	// because upon disposing, all maybe-existent API objects which refered to the subtree (or one of it's
	// children or granchildren) should have relesed there reference.
}

//------------------------------------------------------------------------
void OSetNodeUpdateElement::removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode)
{
	OSL_ENSHURE(getChangesTree() && !getChangesTree()->getChange(_rName),
		"OSetNodeUpdateElement::removedElement : invalid call !");
		// this are the preconditions for this method as defined in OSetNode

	// nothing to do here .... As we don't have changes in that sub tree, we don't have any API-objects there, too,
	// so we don't need to dispose them

	RemoveNode aChange(_rName);
	OSL_ASSERT(!_rNode->asValueNode()); // Otherwise use it to setOldValue in the RemoveNode
	fireLocalChange(aChange);
}

//========================================================================
//= OFloatableSetNode
//========================================================================
//------------------------------------------------------------------------
OFloatableSetNode::OFloatableSetNode(
		BroadcastHelper& _rBHelper,
		const ::rtl::OUString& _rChildTemplateName, ::std::auto_ptr<ISubtree> _pChildTemplateInstance,
		const TreePath& _rContainerPath,
		const ::rtl::OUString& _rContainerTemplateName,
		HierarchyUpdateRoot* _pUpdateRoot)
	:OFloatableNode(_rBHelper, _rChildTemplateName, _rContainerPath, _pChildTemplateInstance, _pUpdateRoot)
	,OSetNode(_rBHelper.rMutex, _pUpdateRoot->getTemplateFactory(), _rContainerTemplateName)
{
}

//------------------------------------------------------------------------
Any SAL_CALL OFloatableSetNode::queryInterface(Type const& _rType) throw (RuntimeException)
{
	Any aReturn = OFloatableNode::queryInterface(_rType);
	if (!aReturn.hasValue())
		aReturn = OSetNode::queryInterface(_rType);
	return aReturn;
}

//------------------------------------------------------------------------
Sequence< Type > SAL_CALL OFloatableSetNode::getTypes() throw (RuntimeException)
{
	return ::utl::concatSequences(
		OFloatableNode::getTypes(),
		OSetNode::getTypes()
	);
}

//------------------------------------------------------------------------
Any SAL_CALL OFloatableSetNode::getByName( const ::rtl::OUString& _rName ) throw(NoSuchElementException, WrappedTargetException, RuntimeException)
{
	::osl::MutexGuard aGuard(HierarchyUpdateBase::m_rMutex);

	// first ask the base class
	Any aReturn;
	try
	{
		aReturn = OFloatableNode::getByName(_rName);
		OSL_ENSHURE(aReturn.hasValue(), "OFloatableSetNode::getByName : base class returned nonsense !");
			// should have thrown an exception if there is no such element
	}
	catch (NoSuchElementException&)
	{
	}

	if (!aReturn.hasValue())
	{	// the element does not exist as tree, but perhaps as AddNode change

		NodeRef pNewlyAddedNode = NULL;

		OReadSynchronized aGuard(getChangeTreeSync());
		SubtreeChange* pModifications = getChangesTree();
		if (pModifications)
		{
			Change* pExistentChange = pModifications->getChange(_rName);
			if (pExistentChange && pExistentChange->isA(AddNode::getStaticType()))
			{
				AddNode* pAddChange = static_cast<AddNode*>(pExistentChange);
				pNewlyAddedNode = pAddChange->getAddedNode(); // Do we need the _unsafe version ?
			}
		}

		TreePath aPath(1, getThisTree());
		aGuard.clear();
		aReturn = makeElement(pNewlyAddedNode, aPath);
			// will throw an exception if the node is NULL
	}

	return aReturn;
}

//------------------------------------------------------------------------
Sequence< ::rtl::OUString > SAL_CALL OFloatableSetNode::getElementNames(  ) throw(RuntimeException)
{
	::osl::MutexGuard aObjectGuard(HierarchyUpdateBase::m_rMutex);

	// the original names (which have a representation within the tree)
	Sequence< ::rtl::OUString > aNames = OFloatableNode::getElementNames();

	// plus the names which at the moment exist just as AddNode-changes
	OReadSynchronized aGuard(getChangeTreeSync());
	SubtreeChange* pModifications = getChangesTree();
	// if the tree is NULL, we're child of a set node itself, so either we're still floating, or we're already
	// inserted into the set node. In both cases the tree completly describes the node (all children can be found
	// in it)

	if (pModifications)
	{
		// the names of the added nodes
		CollectAddedNames aCollector;
		aCollector.applyToChildren(*pModifications);

		sal_Int32 nOldLen = aNames.getLength();
		aNames.realloc(nOldLen + aCollector.aNames.getLength());

		::rtl::OUString* pNames = aNames.getArray() + nOldLen;
		const ::rtl::OUString* pAdded = aCollector.aNames.getConstArray();
		for (sal_Int32 i=0; i<aCollector.aNames.getLength(); ++i, ++pNames, ++pAdded)
			*pNames = *pAdded;
	}

	return aNames;
}

//------------------------------------------------------------------------
sal_Bool SAL_CALL OFloatableSetNode::hasByName( const ::rtl::OUString& _rName ) throw(RuntimeException)
{
	::osl::MutexGuard aObjectGuard(HierarchyUpdateBase::m_rMutex);

	if (OFloatableNode::hasByName(_rName))
		return sal_True;

	OReadSynchronized aGuard(getChangeTreeSync());
	SubtreeChange* pModifications = getChangesTree();
	return pModifications && (NULL != pModifications->getChange(_rName));
}

//------------------------------------------------------------------------
void OFloatableSetNode::insertedElement(const AddNode& _rChange, OFloatableNode* _pNewElement)
{ 
	OFloatableNode::insertedNearestDescendant(_rChange.getNodeName(), _pNewElement); 
	// tell the new element that it is part of a new hierarchy
	_pNewElement->insertedInto(getUpdateRoot());
	fireLocalChange(_rChange);
}

//------------------------------------------------------------------------
void OFloatableSetNode::removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange)
{
	disposeDescendants(_rName);

	RemoveNode aChange(_rName); // No data here to setOldValue in the RemoveNode
	fireLocalChange(aChange);


	// the given SubtreeChange will be deleted when returning from this method. This should not be a problem,
	// because upon disposing, all maybe-existent API objects which refered to the subtree (or one of it's
	// children or granchildren) should have relesed there reference.
}

//------------------------------------------------------------------------
void OFloatableSetNode::removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode)
{
	OSL_ENSHURE(getChangesTree() && !getChangesTree()->getChange(_rName),
		"OFloatableSetNode::removedElement : invalid call !");
		// this are the preconditions for this method as defined in OSetNode

	// nothing to do here .... As we don't have changes in that sub tree, we don't have any API-objects there, too,
	// so we don't need to dispose them

	RemoveNode aChange(_rName);
	OSL_ASSERT(!_rNode->asValueNode()); // Otherwise use it to setOldValue in the RemoveNode
	fireLocalChange(aChange);
}

//========================================================================
//= ORootSetNode
//========================================================================
ORootSetNode::ORootSetNode(
		BroadcastHelper& _rBHelper, 
		const ConfigurationName& _rName, TreeRef _pTree,
		const ::rtl::OUString& _rContainerTemplateName, 
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		IConfigBroadcaster& _rEventSource,
		const Reference< XTypeConverter >& _rxConverter
	)
	:HierarchyUpdateRoot(_rBHelper, _pTree, _rName, _rUpdater, _rTemplateFactory, _rEventSource, _rxConverter)
	,OSetNode(_rBHelper.rMutex, _rTemplateFactory, _rContainerTemplateName)
{
}

//------------------------------------------------------------------------
Any SAL_CALL ORootSetNode::queryInterface(Type const& _rType) throw (RuntimeException)
{
	Any aReturn = HierarchyUpdateRoot::queryInterface(_rType);
	if (!aReturn.hasValue())
		aReturn = OSetNode::queryInterface(_rType);
	return aReturn;
}

//------------------------------------------------------------------------
Sequence< Type > SAL_CALL ORootSetNode::getTypes() throw (RuntimeException)
{
	return ::utl::concatSequences(
		HierarchyUpdateRoot::getTypes(),
		OSetNode::getTypes()
	);
}

//------------------------------------------------------------------------
Any SAL_CALL ORootSetNode::getByName( const ::rtl::OUString& _rName ) throw(NoSuchElementException, WrappedTargetException, RuntimeException)
{
	::osl::MutexGuard aGuard(HierarchyUpdateRoot::m_rMutex);

	// first ask the base class
	Any aReturn;
	try
	{
		aReturn = HierarchyUpdateRoot::getByName(_rName);
		OSL_ENSHURE(aReturn.hasValue(), "ORootSetNode::getByName : base class returned nonsense !");
			// should have thrown an exception if there is no such element
	}
	catch (NoSuchElementException&)
	{
	}

	if (!aReturn.hasValue())
	{	// the element does not exist as tree, but perhaps as AddNode change

		NodeRef pNewlyAddedNode = NULL;

		OReadSynchronized aGuard(this);
		Change* pExistentChange = m_aChanges.root.getChange(_rName);
		if (pExistentChange && pExistentChange->isA(AddNode::getStaticType()))
		{
			AddNode* pAddChange = static_cast<AddNode*>(pExistentChange);
			pNewlyAddedNode = pAddChange->getAddedNode(); // Do we need the _unsafe version ?
		}

		TreePath aPath(1, getThisTree());
		aGuard.clear();
		aReturn = makeElement(pNewlyAddedNode, aPath);
			// will throw an exception if the node is NULL
	}

	return aReturn;
}

//------------------------------------------------------------------------
Sequence< ::rtl::OUString > SAL_CALL ORootSetNode::getElementNames(  ) throw(RuntimeException)
{
	::osl::MutexGuard aObjectGuard(HierarchyUpdateRoot::m_rMutex);

	// the original names (which have a representation within the tree)
	Sequence< ::rtl::OUString > aNames = HierarchyUpdateRoot::getElementNames();

	// plus the names which at the moment exist just as AddNode-changes
	OReadSynchronized aGuard(this);
	// the names of the added nodes
	CollectAddedNames aCollector;
	aCollector.applyToChildren(m_aChanges.root);

	sal_Int32 nOldLen = aNames.getLength();
	aNames.realloc(nOldLen + aCollector.aNames.getLength());

	::rtl::OUString* pNames = aNames.getArray() + nOldLen;
	const ::rtl::OUString* pAdded = aCollector.aNames.getConstArray();
	for (sal_Int32 i=0; i<aCollector.aNames.getLength(); ++i, ++pNames, ++pAdded)
		*pNames = *pAdded;

	return aNames;
}

//------------------------------------------------------------------------
sal_Bool SAL_CALL ORootSetNode::hasByName( const ::rtl::OUString& _rName ) throw(RuntimeException)
{
	::osl::MutexGuard aObjectGuard(HierarchyUpdateRoot::m_rMutex);

	if (HierarchyUpdateRoot::hasByName(_rName))
		return sal_True;

	OReadSynchronized aGuard(this);
	return NULL != m_aChanges.root.getChange(_rName);
}

//------------------------------------------------------------------------
void ORootSetNode::insertedElement(const AddNode& _rChange, OFloatableNode* _pNewElement)
{
	HierarchyUpdateRoot::insertedNearestDescendant(_rChange.getNodeName(), _pNewElement); 
	// tell the new element that it is part of a new hierarchy
	_pNewElement->insertedInto(this);
	fireLocalChange(_rChange);
}

//------------------------------------------------------------------------
void ORootSetNode::removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange)
{
	disposeDescendants(_rName);

	RemoveNode aChange(_rName); // No data here to setOldValue in the RemoveNode
	fireLocalChange(aChange);

	// the given SubtreeChange will be deleted when returning from this method. This should not be a problem,
	// because upon disposing, all maybe-existent API objects which refered to the subtree (or one of it's
	// children or granchildren) should have relesed there reference.
}

//------------------------------------------------------------------------
void ORootSetNode::removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode)
{
	OSL_ENSHURE(getChangesTree() && !getChangesTree()->getChange(_rName),
		"OFloatableSetNode::removedElement : invalid call !");
		// this are the preconditions for this method as defined in OSetNode

	// nothing to do here .... As we don't have changes in that sub tree, we don't have any API-objects there, too,
	// so we don't need to dispose them

	RemoveNode aChange(_rName);
	OSL_ASSERT(!_rNode->asValueNode()); // Otherwise use it to setOldValue in the RemoveNode
	fireLocalChange(aChange);
}

//........................................................................
} // namespace configmgr
//........................................................................


