/*************************************************************************
 *
 *  $RCSfile: confupdateaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// Need this to properly wrap them - we know they can occur 
#ifndef _COM_SUN_STAR_BEANS_PROPERTYVETOEXCEPTION_HPP_
#include <com/sun/star/beans/PropertyVetoException.hpp>
#endif
#ifndef _CONFIGMGR_API_UPDATEACCESS_HXX_
#include "confupdateaccess.hxx"
#endif
#include "tracer.hxx"
#ifndef _CONFIGMGR_TREEACCESS_HXX_
#include "treeaccess.hxx"
#endif

#ifndef CONFIGMGR_API_CHANGESSET_HXX_
#include "confchangesset.hxx"
#endif

#ifndef CONFIGMGR_API_BASEELEMENT_HXX_
#include "confbaseelement.hxx"
#endif
#ifndef _CONFIGMGR_SERVER_VERSION_HXX_
#include "server_version.hxx"
#endif
#ifndef _CONFIGMGR_TREEACTIONS_HXX_
#include "treeactions.hxx"
#endif
#ifndef CONFIGMGR_API_UPDATEHELPER_HXX_
#include "cfgupdatehelper.hxx"
#endif

#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif

//..........................................................................
namespace configmgr
{
//..........................................................................

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::container;
using namespace ::com::sun::star::util;
using namespace ::com::sun::star::script;
using namespace ::cppu;
using namespace ::rtl;
using namespace ::osl;

#define UNISTRING(s)	::rtl::OUString::createFromAscii(s)
#define THISREF()		static_cast< XHierarchicalNameAccess* >(this)

//==========================================================================
//= HierarchyUpdateBase
//==========================================================================
//--------------------------------------------------------------------------
HierarchyUpdateBase::HierarchyUpdateBase(::osl::Mutex& _rMutex, TreeRef _pTree)
	:HierarchyAccessImpl(_rMutex, _pTree)
	,m_rMutex(_rMutex)
	,m_pNearestAncestor(NULL)
{
}

//--------------------------------------------------------------------------
HierarchyUpdateBase::~HierarchyUpdateBase()
{
	{
		MutexGuard aGuard(m_aAncestorChainProtection);
		if (m_pNearestAncestor)
		{
			// if we die, our nearest ancestor would have to rebuild the chain for it's subtree where
			// we reside in (all our own nearest descendants will have a new nearest ancestor)
			m_pNearestAncestor->notifyDeath(m_aAncestorRelativePath, this);
		}
	}
}

//--------------------------------------------------------------------------
Any SAL_CALL HierarchyUpdateBase::queryInterface(const Type& _rType) throw (RuntimeException)
{
	Any aReturn = HierarchyAccessImpl::queryInterface(_rType);

	if (!aReturn.hasValue())
		aReturn = HierarchyUpdateBase_Base::queryInterface(_rType);
	return aReturn;
}

//--------------------------------------------------------------------------
Sequence< Type > SAL_CALL HierarchyUpdateBase::getTypes() throw (RuntimeException)
{
	return ::utl::concatSequences(
		HierarchyAccessImpl::getTypes(),
		HierarchyUpdateBase_Base::getTypes()
	);
}

//--------------------------------------------------------------------------
sal_Int64 SAL_CALL HierarchyUpdateBase::getSomething( const Sequence< sal_Int8 >& _rId ) throw(RuntimeException)
{
	if (_rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  _rId.getConstArray(), 16 ) )
		return reinterpret_cast<sal_Int64>(this);

	return 0;
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > HierarchyUpdateBase::getUnoTunnelImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

//--------------------------------------------------------------------------
// Update the Values, do not fire events
void HierarchyUpdateBase::updateLocalValues(ValueChanges const& aChanges)
{
	SubtreeChange* const pModifications = getChangesTree();
	for(ValueChanges::const_iterator it = aChanges.begin(); it != aChanges.end(); ++it)
	{
		// check if we already have a change for that node
		OUString rName = it->getNodeName();

		Change* pNodeChange = pModifications ? pModifications->getChange(rName) : NULL;
		if (pNodeChange && pNodeChange->isA(RemoveNode::getStaticType()))
			throw NoSuchElementException((UNISTRING("The node named ") += rName) += UNISTRING(" has already been removed (though this removal has not been committed, yet)."), THISREF());

		// now we're ready to update the value
		if (!pNodeChange)
		{	
			// we don't have a change for that element, yet
			if (pModifications)
			{
				// -> create one
				pNodeChange = new ValueChange(*it);
				pModifications->addChange(::std::auto_ptr<Change>(pNodeChange));
			}
			else
			{	// we're part of a floating tree or have been floating and been inserted into a set node, _but_not_committed_yet_
				// (this is exactly what pModifications == NULL means)
				// so we can modify the value node directly. We're part of a subtree which is described by a AddNode-change
				// on a higher level in the tree, so this value change is covered by the addNode, too.

				NodeRef pNode = m_aSubtree->getChild(rName);
				OSL_ASSERT(pNode);

				ValueNode* pValue = const_cast<ValueNode*>(pNode->asValueNode());
				OSL_ASSERT(pValue);

				switch (it->getMode())
				{
				case ValueChange::wasDefault:	
				case ValueChange::changeValue:	
					pValue->setValue(it->getNewValue()); 
					break;

				case ValueChange::changeDefault:	
					pValue->changeDefault(it->getNewValue()); 
					break;

				case ValueChange::setToDefault:	
					OSL_ASSERT(it->getNewValue() == pValue->getDefault());
					pValue->setDefault(); 
					break;

				default:
					OSL_ASSERT(!"Unknown kind of Value Change");
					break;
				}

			}
		}
		else if (pNodeChange && pNodeChange->ISA(AddNode))
		{
			INode* pNewNode = static_cast<AddNode*>(pNodeChange)->getAddedNode(); // Do we need the _unsafe version ?

			OSL_ENSHURE(pNewNode, "Added not not available or invalid");

			ValueNode* pNewValue = pNewNode ? pNewNode->asValueNode() : 0;
			OSL_ENSHURE(pNewValue, "Added value not available or invalid");
			if (!pNewValue)
			{
			// -> this node's value can't be changed until all changes are commited
				throw RuntimeException(
					(UNISTRING("The node named ") += rName) += UNISTRING("has been added recently, but  is in an invalid state."),
					THISREF());
			}

			OSL_ENSHURE(!pNewValue->isDefault(), "Newly added nodes dont have defaults");
			OSL_ENSHURE(it->getMode() == ValueChange::changeValue, "Invalid operation on newly added node");

			pNewValue->setValue(it->getNewValue());
		}
		else
		{	// we already had a value change for that node
			OSL_ENSHURE(pNodeChange->isA(ValueChange::getStaticType()),
				"already have a change for that node, but it's the wrong type !");
			ValueChange* pValChange = static_cast<ValueChange*>(pNodeChange);

			ValueChange::Mode aMode = pValChange->getMode();

			switch(it->getMode())
			{
			case ValueChange::wasDefault: 
				OSL_ASSERT(aMode == ValueChange::setToDefault);
				pValChange->setNewValue(it->getNewValue(), ValueChange::changeValue); 
				break;

			case ValueChange::changeValue: 
				OSL_ASSERT(aMode != ValueChange::setToDefault);
				pValChange->setNewValue(it->getNewValue()); 
				break;

			case ValueChange::setToDefault: 
				OSL_ASSERT(aMode != ValueChange::setToDefault);
				OSL_ENSHURE(aMode != ValueChange::wasDefault,"Restoring to default without committing not handled properly now");
				pValChange->setNewValue(it->getNewValue(), ValueChange::setToDefault); 
				break;

			case ValueChange::changeDefault: 
				OSL_ENSURE(aMode == ValueChange::changeDefault,"Cannot handle change of default paralelling value change");
				pValChange->setNewValue(it->getNewValue(), ValueChange::changeDefault); 
				break;

			default:
				OSL_ASSERT(!"Unknown mode for ValueChange");
				break;
			}
		}
	}
}


//--------------------------------------------------------------------------
void SAL_CALL HierarchyUpdateBase::replaceByName( const ::rtl::OUString& _rName, const Any& _rElement ) 
	throw(IllegalArgumentException, NoSuchElementException, WrappedTargetException, RuntimeException)
{
	::osl::ClearableMutexGuard aGuard(m_rMutex);

	OReadSynchronized  aTreeGuard(getTreeSync());
	OWriteSynchronized aUpdateTreeGuard(getUpdateSync());
		// TODO : this tree guard is for our update tree, but we have to lock the read-tree, too

	checkInitialized();

	// check if the name is valid
	NodeRef pFound = m_aSubtree->getChild(_rName);
	if (!pFound)
		throw NoSuchElementException((UNISTRING("There is no node named ") += _rName) += UNISTRING("."), THISREF());

	// only ValueNodes can be replaced
	const ValueNode* pValueElement = pFound->asValueNode();
	if (!pValueElement)
		throw IllegalArgumentException(UNISTRING("Only the content of value nodes (leafs in the configuration tree) can be replaced."),
			THISREF(), 1);

	try
	{
		ValueUpdater anUpdate(*pValueElement,_rElement,ValueChange::changeValue,getTypeConverter());
		ValueChanges aChanges;

		bool bChanged = true;
		aChanges.push_back( anUpdate.prepareChange(bChanged,THISREF(),2) );
		
		if (bChanged)
		{
			// leave read guard locked during notification !
			//aReadGuard.clear();
			// clear the locks for notification
			aGuard.clear();

			executeLocalChanges(aChanges);
		}
	}
	catch (lang::IllegalArgumentException& ) { throw; }
	catch (lang::WrappedTargetException& ) { throw; }
	catch (uno::RuntimeException& ) { throw; }
	catch (css::beans::PropertyVetoException& e)
	{
		Any aWrapped(&e,::getCppuType(&e));
		OUString sMessage =
			UNISTRING("The change was disallowed due to a constraint violation: ")
			+= e.Message;
		throw WrappedTargetException(sMessage, THISREF(), aWrapped);
	}
	catch (uno::Exception& e)
	{
		Any aWrapped(&e,::getCppuType(&e));
		OUString sMessage =
			UNISTRING("An exception occurred updating the value: ")
			+= e.Message;
		throw WrappedTargetException(sMessage, THISREF(), aWrapped);
	}
}

//--------------------------------------------------------------------------
void HierarchyUpdateBase::notifyDeath(const ConfigurationName& _rRelLoc, HierarchyUpdateBase* _pDyingObject)
{
	MutexGuard aGuard(m_rMutex);

	OSL_ENSHURE(_pDyingObject != NULL, "HierarchyUpdateBase::notifyDeath : invalid argument !");

	DescendantsIterator aExistenceCheck = m_aDescendants.find(_rRelLoc);
	OSL_ENSHURE(m_aDescendants.end() != aExistenceCheck,
		"HierarchyUpdateBase::notifyDeath : invalid call !");
		// this method is called from within a dtor of an object which's nearest ancestor we are
		// So if we are consistent, it should be our nearest descendant for it's tree, too.

	if (m_aDescendants.end() == aExistenceCheck)
		return;

	// now loop through all "nearest descendant" of the dying object and hold them ourself
	for	(	ConstDescendantsIterator aLevel2Descs = _pDyingObject->m_aDescendants.begin();
			aLevel2Descs != _pDyingObject->m_aDescendants.end();
			++aLevel2Descs
		)
	{
		// the location of this node relative to ourself
		ConfigurationName sNewRelativeLoc(_rRelLoc, aLevel2Descs->first);

		// the implementation of this descendant
		HierarchyUpdateBase* pLevel2Desc = getSafeDescendant(aLevel2Descs);
		if (pLevel2Desc)
		{
			m_aDescendants[sNewRelativeLoc] = aLevel2Descs->second;
			pLevel2Desc->setNearestAncestor(this, sNewRelativeLoc);
		}
	}

	_pDyingObject->setNearestAncestor(NULL, ConfigurationName());
	m_aDescendants.erase(aExistenceCheck);
}

//--------------------------------------------------------------------------
NodeRef HierarchyUpdateBase::findDescendant(OUString const& _rRelativePath, TreePath& _rPath)
{
	if (!getChangesTree())
		// the base class' implementation if sufficient
		return HierarchyAccessImpl::findDescendant(_rRelativePath, _rPath);

	// no base class call, this would mean doubled costs in case the base classes misses the node
	checkInitialized();

	OReadSynchronized aReadGuard(getUpdateSync());

	// the relative name of the requested node
	ConfigurationName aName( _rRelativePath );
	if (!normalizeNestedPath(aName))
		return NULL;

	ConfigurationName::Iterator it = aName.begin();
	OSL_ENSHURE(aName.depth() > 0, "HierarchyUpdateBase::findDescendant : invalid path !");
	const ConfigurationName::Iterator stop	= aName.end();

	// the tree and it's changes
	TreeRef pTree = getThisTree();
	SubtreeChange* pCurrentTreeMods = getChangesTree();

	// go down
	NodeRef pFound = NULL;
	for ( ;NULL != pTree, it != stop; ++it)
	{
		pFound = pTree->getChild(*it);
		_rPath.push_back(pTree);

		if (!pFound)
		{	// the requested child does not exist one this level, but perhaps there is a (not yet committed)
			// AddNode change for the child
			if (pCurrentTreeMods)
			{
				Change* pChildChange = pCurrentTreeMods->getChange(*it);
				if (pChildChange && pChildChange->ISA(AddNode))
				{	// yes, there is
					pFound = static_cast<AddNode*>(pChildChange)->getAddedNode(); // Do we need the _unsafe version ?

				}
			}
		}

		if (!pFound)
			return NULL;

		// one level down in the modifications tree
		if (pCurrentTreeMods)
		{
#ifdef DEBUG
			Sequence< ::rtl::OUString > aNames = pCurrentTreeMods->elementNames();
			const ::rtl::OUString* pNames = aNames.getConstArray();
			for (sal_Int32 i=0; i<aNames.getLength(); ++i, ++pNames)
			{
				sal_Int32 nDummy = 0;
			}
#endif
			Change* pNextLevelChange = pCurrentTreeMods->getChange(*it);
			if (pNextLevelChange && pNextLevelChange->isA(SubtreeChange::getStaticType()))
				pCurrentTreeMods = static_cast<SubtreeChange*>(pNextLevelChange);
			else
				pCurrentTreeMods = NULL;
		}

		// onve level down with in the tree
		pTree = pFound->asISubtree();
	}

	return pFound;
}

//--------------------------------------------------------------------------
HierarchyAccessImpl* HierarchyUpdateBase::getSubtreeAccess(TreeRef _pSubtree, TreePath const& _rPath)
{
	MutexGuard aGuard(m_rMutex);
		// as we want to access (and write to) m_aDescendants, which may be access from within other threads
		// in notifyDeath, too.

	// TODO : we need a read guard for our tree here

	// build the complete node path to the node where we shall create a new object for
	// (relative to our update root)
	TreePath aCompletePath;
	getThisPath(aCompletePath);
	TreePath::const_iterator aCopy = _rPath.begin();
	OSL_ENSHURE(aCopy != _rPath.end(), "HierarchyUpdateBase::getSubtreeAccess : invalid argument (_rPath) !");
	++aCopy;
	// the first element of the given path is the same as the last one of our own element path, so we
	// don't need to copy it
	for (; aCopy != _rPath.end(); ++aCopy)
		aCompletePath.push_back(*aCopy);
	aCompletePath.push_back(_pSubtree);

	// getRootLocation and aCompletePath are somewhat redundant : they both contain the part for the root of
	// the whole update tree, so we cut aCompletePath to remove this redundance
	TreePath aCompletePathNoRedundance(aCompletePath);
	aCompletePathNoRedundance.pop_front();

	ConfigurationName aRelativeAccessor = HierarchyElementImpl::getHierarchicalName(getRootLocation(), aCompletePathNoRedundance);
	OSL_VERIFY(normalizeNestedPath(aRelativeAccessor));
		// this should never fail, as aRelativeAccessor was build in the line before should always be nested below
		// our node

	// check if we already have a descendant for that node
	ConstDescendantsIterator aExistenceCheck = m_aDescendants.find(aRelativeAccessor);
	if (m_aDescendants.end() != aExistenceCheck)
	{	// yes, we have. At least we had some time ago ...
		HierarchyUpdateBase* pExistentUpdateBase = getSafeDescendant(aExistenceCheck);
		if (pExistentUpdateBase)
		{
			// by definition, the object returned by getSubtreeAccess is acquired once
			pExistentUpdateBase->acquire();
			return pExistentUpdateBase;
		}

		// we're inconsistent in any way, so don't allow any further objects to be created
		return NULL;
	}

	// check if we already have a descendant, which would be an ancestor of the requested node
	if (m_aDescendants.size())
	{
		DescendantsIterator aExistentAncestor = m_aDescendants.lower_bound(aRelativeAccessor);
		// aExistentAncestor is the smallest which is greater or equal, but equality was handled above
		--aExistentAncestor;	// valid as m_aDescendants is not empty
		// now it's the greates which is smaller
		if (aRelativeAccessor.isNestedIn(aExistentAncestor->first))
		{	// yes, we have :
			//  me
			//  |
			//  e
			//  |
			//  r
			// where e is the existent node (aExistentAncestor), and r the requested one

			HierarchyUpdateBase* pAncestor = getSafeDescendant(aExistentAncestor);
			if (pAncestor)
			{
				sal_Int32 nShortenPathBy = aExistentAncestor->first.depth();
				OSL_ENSHURE(nShortenPathBy <= _rPath.size(), "HierarchyUpdateBase::getSubtreeAccess : inconsistence ....");
					// aExistentAncestor can't be an ancestor of the requested node if it's not located above it

				TreePath aRelativeToAncestor(_rPath);
				while (nShortenPathBy--)
					aRelativeToAncestor.pop_front();

				return pAncestor->getSubtreeAccess(_pSubtree, aRelativeToAncestor);
			}
			else
				m_aDescendants.erase(aExistentAncestor);
		}
	}

	// get the changes tree for the new node
	SubtreeChange* pNewObjectsModifications = NULL;
	SubtreeChange* pMyModifications = getChangesTree();
	if (pMyModifications)
	{
		OWriteSynchronized aTreeGuard(getUpdateSync());
			// we are going to write into our changes tree, not just read
			// (this is some kind of brute force, there are scenarios where we get to this point, but are not
			// going to write below. Anyhow.

		OUString sName = _pSubtree->getName();

		// ensure that there is a SubtreeChange for the node given
		pNewObjectsModifications = pMyModifications;
			// seems that this doesn't make any sense ... but it will be changed below

		ConfigurationName aTraveller(aRelativeAccessor);
		ConfigurationName::Iterator aGoDown = aTraveller.begin();
		ConfigurationName::Iterator aStopAt = aTraveller.end();
		OSL_ENSHURE(aGoDown != aStopAt, "HierarchyUpdateBase::getSubtreeAccess : the path should have at least one level !");
		while (aGoDown != aStopAt)
		{
			Change* pExistentChange = pNewObjectsModifications->getChange(*aGoDown);
			if (!pExistentChange)
			{	// at this level we don't have a SubtreeChange yet
				// -> create it
				pExistentChange = new SubtreeChange(*aGoDown);
				pNewObjectsModifications->addChange(::std::auto_ptr<Change>(pExistentChange));
			}
			else
			{
				if (pExistentChange->isA(RemoveNode::getStaticType()))
					throw NoSuchElementException(
						UNISTRING("An ancestor of th requested node has already been removed (though the changes are not committed, yet)."),
						THISREF());

				if (pExistentChange->isA(AddNode::getStaticType()))
				{	// an ancestor of the requested node (or the node itself) has just been added to the tree,
					// but not committed (yet). So the new object won't have a changes tree itself, it is only
					// allowed to do further additions.
					pNewObjectsModifications = NULL;
					break;
				}
			}
			pNewObjectsModifications = static_cast<SubtreeChange*>(pExistentChange);
			++aGoDown;
		}
	}
	else
	{
		// here we're part of a sub tree which just has been added, but not committed yet. So we just have to
		// the combine trees, there are no change trees.
		// That's why there's nothing to do here.
	}

	// create the new element
	HierarchyUpdateBase* pNewElement = createElement(pNewObjectsModifications, aCompletePath);
		// acquired once

	// check if the object is located between an existing descendant and ourself
	DescendantsIterator aUpperBound = m_aDescendants.upper_bound(aRelativeAccessor);
	if (m_aDescendants.end() != aUpperBound)
	{
		// we have an object for the node _under_ or _right_of_ the requested one, e.g.
		//     me
		//  /  |  \
		// a  (b)  c          (1)
		//     |
		//     r
		// where a is any existent node, b a not (yet) existent node, r the requested node, and c the upper bound
		// of r
		if (aUpperBound->first.isNestedIn(aRelativeAccessor))
		{	// it's _under_ the given one -> rebuild the "nearest ancestor"-structures, something like
			//     me
			//  /  |
			// a  (b)         (2)
			//     |
			//     r
			//     |
			//     c
			// with the node names as above in (1). Here c is an upper bound for r, too, so it was found by the
			// above upper_bound call

			HierarchyUpdateBase* pElementBelow = getSafeDescendant(aUpperBound);
			// now recombine the chain ...
			ConfigurationName aBelowRelativeToNew(aUpperBound->first.relativeTo(aRelativeAccessor));
			// the lower element has a new nearest ancestor
			pElementBelow->setNearestAncestor(pNewElement, aBelowRelativeToNew);
			// and the new element has a first descendant
			pNewElement->m_aDescendants[aBelowRelativeToNew] = Reference< XUnoTunnel >(pElementBelow);
			// and we ourself have a new nearest descendant on that sub tree, so delete the old one
			m_aDescendants.erase(aUpperBound);
			// the insertion of the new element into m_aDescendants will be done below
		}
	}
	m_aDescendants[aRelativeAccessor] = Reference< XUnoTunnel >(pNewElement);
	pNewElement->setNearestAncestor(this, aRelativeAccessor);

	// by definition, the object returned by getSubtreeAccess is acquired once
	// but the same counts for createElement which gave us pNewElement, so we don't have to acquire it here
	return pNewElement;
}

//--------------------------------------------------------------------------
void HierarchyUpdateBase::insertedNearestDescendant(const ::rtl::OUString& _rRelativeAccessor, HierarchyUpdateBase* _pNewDesc)
{
	OSL_ENSHURE(m_aDescendants.find(_rRelativeAccessor) == m_aDescendants.end(), "OSetNodeUpdateElement::insertedNearestDescendant : already have a nearest descendant for that tree ... sounds really impossible !");
	// no further check if there is another descendant above the one refered to by _rRelativeAccessor, this is the responsibility of the caller

	_pNewDesc->setNearestAncestor(this, _rRelativeAccessor);
	m_aDescendants[_rRelativeAccessor] = Reference< XUnoTunnel >(_pNewDesc);
}

//--------------------------------------------------------------------------
void HierarchyUpdateBase::committed()
{
	// Just working on my immediate children. They will get the committed, too, thus propagating the event
	// through the whole tree.

	SubtreeChange* pMyChanges = getChangesTree();
	if (!pMyChanges)
	{
		OSL_ENSHURE(sal_False, "HierarchyUpdateBase::committed : how have I been committed without an changes tree ?");
		return;
	}

	// First, all the API objects which, by committing the changes, have been integrated into
	// our existent hierarchy (the ones that where created by an add-operation) have to be given a SubtreeChange
	// for any further changes they may want to do.

	SubtreeChange::ChildIterator aChildren = pMyChanges->begin();
	SubtreeChange::ChildIterator aStopAt = pMyChanges->end();
	while (aChildren != aStopAt)
	{
		const Change* pThisChild = *aChildren;
		if (pThisChild && pThisChild->isA(AddNode::getStaticType()))
		{
			//  replace the AddNode change with a SubtreeChange ....

			OWriteSynchronized aWriteGuard(getUpdateSync());

			::rtl::OUString sNodeName = pThisChild->getNodeName();
			pMyChanges->removeChange(sNodeName);
				// this won't invalidate the iterator as long as we never reach this pos, again

			SubtreeChange* pNewSubtree = new SubtreeChange(sNodeName);
			pMyChanges->addChange(::std::auto_ptr<Change>(pNewSubtree));

			// and tell the descendant that is has been fully integrated into the hierarchy

			// get the descendant which is responsible for that sub tree (such a descendant should always exist,
			// else there should not have been an AddNode change
			ConstDescendantsIterator aDescendant = m_aDescendants.find(ConfigurationName(sNodeName));
			HierarchyUpdateBase* pDescendant = getSafeDescendant(aDescendant);
			if (pDescendant)
				pDescendant->integrated(pNewSubtree);
		}

		++aChildren;
	}

	// and then, forward the "committed" event to all nearest descendants
	for (	ConstDescendantsIterator aDescLoop = m_aDescendants.begin();
			aDescLoop != m_aDescendants.end();
			++aDescLoop
		)
	{
		HierarchyUpdateBase* pDesc = HierarchyUpdateBase::getSafeDescendant(aDescLoop);
		if (pDesc)
			pDesc->committed();
	}
}

//--------------------------------------------------------------------------
HierarchyUpdateBase* HierarchyUpdateBase::getSafeDescendant(const HierarchyUpdateBase::ConstDescendantsIterator& _rPos, const ::com::sun::star::uno::Sequence< sal_Int8 >& _rUnoTunnelImplementationId)
{
	OSL_ENSHURE(16 == _rUnoTunnelImplementationId.getLength(), "HierarchyUpdateBase::getSafeDescendant : invalid tunnel id !");
	HierarchyUpdateBase* pReturn = NULL;
	if (m_aDescendants.end() != _rPos)
	{
		Reference< XUnoTunnel > xTunnel = _rPos->second;
		OSL_ENSHURE(xTunnel.is(), "HierarchyUpdateBase::getSafeDescendant : invalid tunnel !");
			// the object should have notified us upon dying
		if (xTunnel.is())
		{
			pReturn = reinterpret_cast< HierarchyUpdateBase* >(xTunnel->getSomething(_rUnoTunnelImplementationId));
			OSL_ENSHURE(
					NULL != pReturn
				||	0 != rtl_compareMemory(_rUnoTunnelImplementationId.getConstArray(), HierarchyUpdateBase::getUnoTunnelImplementationId().getConstArray(), 16)
				, "HierarchyUpdateBase::getSafeDescendant : invalid implementation !");
				// this reference should never have been inserted if the object is not a HierarchyUpdateBase ...
		}
	}
	return pReturn;
}

//--------------------------------------------------------------------------
void HierarchyUpdateBase::integrated(SubtreeChange* _pNewObjectModifications)
{
	MutexGuard aGuard(m_rMutex);
		// as we want to access m_aDescendants, which may be access from within other threads
		// in notifyDeath, too.
	OSL_ENSHURE(_pNewObjectModifications, "HierarchyUpdateBase::integrated : invalid new change !");
	OSL_ENSHURE(_pNewObjectModifications->size() == 0, "HierarchyUpdateBase::integrated : the new SubtreeChange should be empty !");
		// we ourself are going to create the new children, if necessary

	OSL_ENSHURE(!getChangesTree(), "HierarchyUpdateBase::integrated : I'm already an integral part of a hierarchy !");

	// now that we're a "real" part of an update hierarchy, all of our descendants need a subtree
	// change. With this they're able to do value replaces on leafs or modifications on their own set nodes.

	// loop through all descendants and tell them of the new "integrated" state
	for	(	ConstDescendantsIterator aDescs = m_aDescendants.begin();
			aDescs != m_aDescendants.end();
			++aDescs
		)
	{
		HierarchyUpdateBase* pUpdateBase = getSafeDescendant(aDescs);
		if (pUpdateBase)
		{
			// create a new change (chain) for the descendant
			SubtreeChange* pNewChange = _pNewObjectModifications;

			const ConfigurationName& rAccessor = aDescs->first;
			for (	ConfigurationName::Iterator aLocalAccess = rAccessor.begin();
					aLocalAccess != rAccessor.end();
					++aLocalAccess
				)
			{
				SubtreeChange* pNextLevelChange = new SubtreeChange(*aLocalAccess);
				pNewChange->addChange(::std::auto_ptr<Change>(pNextLevelChange));
				pNewChange = pNextLevelChange;
			}

			// an announce this new change to the descendant
			pUpdateBase->integrated(pNewChange);
		}
	}
}

//--------------------------------------------------------------------------
void HierarchyUpdateBase::disposeDescendants(const ::rtl::OUString& _rPath)
{
	::osl::MutexGuard aGuard(m_rMutex);

	ConfigurationName aSubtreeAccess(_rPath);

	for	(	DescendantsIterator aDisposeLoop = m_aDescendants.lower_bound(_rPath);
			aDisposeLoop != m_aDescendants.end();
		)
	{
		// as we are going to delete the object from the map, we need to iterator from where we can draw on
		// and as we may (implicitly, triggered by notifyDeath triggered by the disposing of the current element)
		// insert new elements after the one we're handling currently, and we want to handle these new ones, too,
		// we can't just get next iterator _after_ aDisposeLoop. We would miss all the inserted ones, then.
		// So we get the previous one and increment it after removing the current one.
		DescendantsIterator aPrevious = aDisposeLoop;
		if (m_aDescendants.begin() == aPrevious)
			aPrevious = m_aDescendants.end();
		else
			--aPrevious;

		if ((aDisposeLoop->first == aSubtreeAccess) || (aDisposeLoop->first.isNestedIn(aSubtreeAccess)))
		{
#ifdef _DEBUG
			ConfigurationName aAccessor = aDisposeLoop->first;
#endif
			HierarchyUpdateBase* pDesc = getSafeDescendant(aDisposeLoop);
			if (pDesc)
				pDesc->dispose();
				// this dispose will cause the descendant to send us a notifyDeath call, which will insert the
				// 2nd-level-descendents into our own map. As they are _below_ the one we're disposing, their
				// ConfigurationName will be greater than the one for pDesc. That's why, when incrementing our
				// iterator, we will visit these "new" nearest descendants, too.

			OSL_ENSHURE(m_aDescendants.end() == m_aDescendants.find(aAccessor),
				"HierarchyUpdateBase::disposeDescendants : object should have been removed !");
				// as stated above : the dispose should result in a call to notifyDeath, which should delete the
				// object from the map
		}
		else
			// if one of the descendants is not nested in aSubtreeAccess anymore, all following won't be, too.
			// So says the order defined on the map via ConfNameCompare :)
			break;

		if (m_aDescendants.end() == aPrevious)
		{
			// the current iterator was the first one in the map
			aDisposeLoop = m_aDescendants.lower_bound(_rPath);
			OSL_ENSHURE(m_aDescendants.begin() == aDisposeLoop, "HierarchyUpdateBase::disposeDescendants : not the beginning !");
				// the new lower bound should be the new first element then, too.
		}
		else
			++aPrevious;
	}
}

//--------------------------------------------------------------------------
void HierarchyUpdateBase::disposing()
{
	// notify our parent that we're dying
	MutexGuard aGuard(m_aAncestorChainProtection);
	if (m_pNearestAncestor)
	{
		// if we die, our nearest ancestor would have to rebuild the chain for it's subtree where
		// we reside in (all our own nearest descendants will have a new nearest ancestor)
		m_pNearestAncestor->notifyDeath(m_aAncestorRelativePath, this);
		OSL_ENSHURE(!m_pNearestAncestor, "HierarchyUpdateBase::disposing : setNearestAncestor should have been called !");
	}
}

//==========================================================================
//= HierarchyUpdateRoot
//==========================================================================
//--------------------------------------------------------------------------

HierarchyUpdateRoot::HierarchyUpdateRoot(BroadcastHelper& _rBHelper,
		TreeRef _pTree, const ConfigurationName& _rName,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		IConfigBroadcaster& rEventSource_,
		const Reference< XTypeConverter >& _rxConverter
	)
	:HierarchyUpdateBase(_rBHelper.rMutex, _pTree)
	,ChangesNotifierImpl(_rBHelper)
	,m_aChanges(_rName.fullName(), _pTree->getName())
	,m_aNotifier(m_aChanges,&rEventSource_)
	,m_xUpdateProvider(_rUpdater)
	,m_xTemplateProvider(_rTemplateFactory)
	,m_pUpdateTreeProtection(new OTreeAccessor)
	,m_xTypeConverter(_rxConverter)
{
	OSL_ENSHURE(m_xUpdateProvider.isValid(), "HierarchyUpdateRoot::HierarchyUpdateRoot : invalid tree update provider !");
	OSL_ENSHURE(!getRootLocation().isRelative(), "HierarchyUpdateRoot::HierarchyUpdateRoot : the root path shouldn't be relative !");
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::Init()
{
	ChangesNotifierImpl::Init();
	// Local Broadcaster observes our own notifier
	ChangesNotifierImpl::observe(m_aNotifier);
}

//--------------------------------------------------------------------------
HierarchyUpdateRoot::~HierarchyUpdateRoot()
{
	ChangesNotifierImpl::stopObserving();
	delete m_pUpdateTreeProtection;
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::disposing()
{
	m_aNotifier.dispose();
	HierarchyUpdateBase::disposing();	
	ChangesNotifierImpl::disposing();
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::acquireReadAccess() const
{
	m_pUpdateTreeProtection->acquireReadAccess();
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::releaseReadAccess() const
{
	m_pUpdateTreeProtection->releaseReadAccess();
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::acquireWriteAccess()
{
	m_pUpdateTreeProtection->acquireWriteAccess();
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::releaseWriteAccess()
{
	m_pUpdateTreeProtection->releaseWriteAccess();
}

//--------------------------------------------------------------------------
bool HierarchyUpdateRoot::normalizeNestedPath(ConfigurationName& _rPath)
{
	if (!_rPath.isRelative())
	{
		ConfigurationName aMyPath(m_aChanges.pathToRoot, m_aChanges.root.getNodeName());
		if (!_rPath.isNestedIn(aMyPath))
			return false;

		_rPath = _rPath.relativeTo(aMyPath);
	}

	return true;
}

//--------------------------------------------------------------------------
Sequence< uno::Type > SAL_CALL HierarchyUpdateRoot::getTypes() throw (RuntimeException)
{
	Sequence< uno::Type > aTypes(HierarchyUpdateBase::getTypes());

	 aTypes = ::utl::concatSequences(aTypes, HierarchyUpdateRoot_Base::getTypes());
	 aTypes = ::utl::concatSequences(aTypes, ChangesNotifierImpl::getTypes());
	 
	 return aTypes;
}

//--------------------------------------------------------------------------
Any SAL_CALL HierarchyUpdateRoot::queryInterface(Type const& _rType) throw (RuntimeException)
{
	Any aReturn = HierarchyUpdateBase::queryInterface(_rType);

	if (!aReturn.hasValue())
		aReturn = HierarchyUpdateRoot_Base::queryInterface(_rType);

	if (!aReturn.hasValue())
		aReturn = ChangesNotifierImpl::queryInterface(_rType);

	return aReturn;
}

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
struct ORemoveChangeLeafs : public ChangeTreeModification
{
protected:
	typedef ::std::vector< ::rtl::OUString >	StringVector;
	StringVector aLeafNames;

public:
	virtual void handle(ValueChange& aValueNode)	{ aLeafNames.push_back(aValueNode.getNodeName()); }
	virtual void handle(AddNode& aAddNode)			{ aLeafNames.push_back(aAddNode.getNodeName()); }
	virtual void handle(RemoveNode& aRemoveNode)	{ aLeafNames.push_back(aRemoveNode.getNodeName()); }
	virtual void handle(SubtreeChange& aSubtree);
};

//..........................................................................
void ORemoveChangeLeafs::handle(SubtreeChange& _rSubtree)
{
	StringVector aThisLevelLeafs = aLeafNames;
	aLeafNames.clear();

	applyToChildren(_rSubtree);

	for (	StringVector::const_iterator aErase = aLeafNames.begin();
			aErase != aLeafNames.end();
			++aErase
		)
	{
		_rSubtree.removeChange(*aErase);
	}

	aLeafNames = aThisLevelLeafs;
}

//..........................................................................
void SAL_CALL HierarchyUpdateRoot::commitChanges(  ) throw(WrappedTargetException, RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OWriteSynchronized aTreeGuard(this);
		// TODO : this tree guard is for our update tree, but we have to lock the read-tree, too

	OChangeActionCounter aChangeCounter;
	aChangeCounter.handle(m_aChanges.root);
	if ((0 == aChangeCounter.nValues) && (0 == aChangeCounter.nRemoves) && (0 == aChangeCounter.nAdds))
		// nothing to do
		return;

	OWriteSynchronized aWriteGuard(getUpdateSync());
	ORemoveChangeLeafs aEraseChangesLeafs;

	try
	{
		m_xUpdateProvider->updateTree(m_aChanges);
	}
	catch(WrappedTargetException&e)
	{
		CFG_TRACE_ERROR("changes batch: caught an WrappedTargetException while updating the tree (message: %s)", OUSTRING2ASCII(e.Message));
		throw;
	}

	committed();

	// all our existent descendants have pointers to their subtree changes, so we cannot just delete our changes
	// tree. But clearly we have to get rid of all changes which are no SubtreeChanges

	aEraseChangesLeafs.handle(m_aChanges.root);
#ifdef DEBUG
	{
		OChangeActionCounter aChangeCounter;
		aChangeCounter.handle(m_aChanges.root);
		OSL_ENSHURE(aChangeCounter.nAdds + aChangeCounter.nRemoves + aChangeCounter.nValues == 0,
			"HierarchyUpdateRoot::commitChanges : the leaf remover did not work correctly !");
	}
#endif

}


//--------------------------------------------------------------------------
sal_Bool SAL_CALL HierarchyUpdateRoot::hasPendingChanges(  ) throw(RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();

	OReadSynchronized aTreeGuard(this);
		// TODO : this tree guard is for our update tree, but we have to lock the read-tree, too

	OChangeCounter aCounter;
	m_aChanges.root.dispatch(aCounter);

	return aCounter.nCount != 0;
}

//--------------------------------------------------------------------------
//..........................................................................
ChangesSet SAL_CALL HierarchyUpdateRoot::getPendingChanges(  ) throw(RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	OReadSynchronized aTreeGuard(this);
		// TODO : this tree guard is for our update tree, but we have to lock the read-tree, too

	OChangeCollector aCollectChanges(m_aChanges.pathToRoot);
	m_aChanges.root.dispatch(aCollectChanges);

	return aCollectChanges.getChanges();
}

//--------------------------------------------------------------------------
::rtl::OUString SAL_CALL HierarchyUpdateRoot::getName(  ) throw(::com::sun::star::uno::RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();
	return m_aChanges.root.getNodeName();
}

//--------------------------------------------------------------------------
void SAL_CALL HierarchyUpdateRoot::setName( const ::rtl::OUString& aName ) throw(::com::sun::star::uno::RuntimeException)
{
	throw ::configmgr::CantRenameException(aName, static_cast<XNamed*>(this));
}

//--------------------------------------------------------------------------
::rtl::OUString SAL_CALL HierarchyUpdateRoot::getHierarchicalName(  ) throw(::com::sun::star::uno::RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();
	return getRootLocation().fullName();
}

//--------------------------------------------------------------------------
::rtl::OUString SAL_CALL HierarchyUpdateRoot::composeHierarchicalName( const ::rtl::OUString& aRelativeName ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::lang::NoSupportException, ::com::sun::star::uno::RuntimeException)
{
	::osl::MutexGuard aGuard(m_rMutex);
	checkInitialized();
	return ConfigurationName(getRootLocation(), aRelativeName).fullName();
}
//--------------------------------------------------------------------------
bool HierarchyUpdateRoot::loadDefaults()
{
	TreePath(aPath);
	getThisPath(aPath);
	return this->loadDefaultsAt(aPath);
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::fireLocalChange(Change const& aChange)
{
	m_aNotifier.broadcast(aChange);
}

//--------------------------------------------------------------------------
void HierarchyUpdateRoot::executeLocalChanges(ValueChanges const& aChanges)
{
	updateLocalValues(aChanges);

	// HACK : fireLocalChanges for multiple (value) changes - should really build a subtree change
	for (ValueChanges::const_iterator it = aChanges.begin(); it != aChanges.end(); ++it)
		m_aNotifier.broadcast(*it);
}
//--------------------------------------------------------------------------
bool HierarchyUpdateRoot::loadDefaultsAt(TreePath const& aPath)
{
	// make the unused .. warning useful:
	TreePath const*const HierarchyUpdateRoot_loadDefaultsAt_notImplemented = &aPath;

	OSL_TRACE("Warning: default handling for configuration trees currently not implemented\n\r");
	return false;
}

//==========================================================================
//= HierarchyUpdateElement
//==========================================================================
//--------------------------------------------------------------------------
HierarchyUpdateElement::HierarchyUpdateElement(BroadcastHelper& _rBHelper,
											   HierarchyUpdateRoot* _pUpdateRoot,
											   const TreePath& _rParentPath,
											   TreeRef _pFloatingNode)
	:HierarchyElementImpl(_rBHelper.rMutex, _rParentPath, _pUpdateRoot->getRootLocation().getParentName().fullName())
	,HierarchyUpdateBase(_rBHelper.rMutex, _pFloatingNode)
	,ChangesNotifierImpl(_rBHelper)
	,m_pUpdateRoot(_pUpdateRoot)
	,m_pModifications(NULL)
{
	OSL_ENSHURE(m_pUpdateRoot != NULL, "HierarchyUpdateElement::HierarchyUpdateElement : invalid update root !");
	m_pUpdateRoot->acquire();
}

//--------------------------------------------------------------------------
static /*inline*/  TreePath extractParentPath(TreePath const& rLocation)
{
#if !(defined(__SUNPRO_CC) && __SUNPRO_CC == 0x500 && defined(DEBUG)) 
	TreePath::const_iterator from = rLocation.begin();
	TreePath::const_iterator to   = rLocation.end();
	--to;
	return TreePath(from,to);
#else
	TreePath aResult(rLocation);
	aResult.pop_back();
	return aResult;
#endif
}

//--------------------------------------------------------------------------
HierarchyUpdateElement::HierarchyUpdateElement(BroadcastHelper& rBHelper_,
											   HierarchyUpdateRoot* pUpdateRoot_,
											   SubtreeChange* pModifications_,
											   const TreePath& rRelativeLocation_)
	:HierarchyElementImpl(rBHelper_.rMutex, extractParentPath(rRelativeLocation_), 
						  pUpdateRoot_->getRootLocation().getParentName().fullName())
	,HierarchyUpdateBase(rBHelper_.rMutex, rRelativeLocation_.back())
	,ChangesNotifierImpl(rBHelper_)
	,m_pUpdateRoot(pUpdateRoot_)
	,m_pModifications(pModifications_)
{
	OSL_ENSHURE(m_pUpdateRoot != NULL, "HierarchyUpdateElement::HierarchyUpdateElement : invalid update root !");
	m_pUpdateRoot->acquire();
}

//--------------------------------------------------------------------------
void HierarchyUpdateElement::Init()
{
	ChangesNotifierImpl::Init();
	ChangesNotifierImpl::observe(m_pUpdateRoot->m_aNotifier);
}

//--------------------------------------------------------------------------
HierarchyUpdateElement::~HierarchyUpdateElement()
{
	ChangesNotifierImpl::stopObserving();
	m_pUpdateRoot->release();
}

//--------------------------------------------------------------------------
Sequence< uno::Type > SAL_CALL HierarchyUpdateElement::getTypes() throw (RuntimeException)
{
	Sequence< uno::Type > aTypes(HierarchyUpdateBase::getTypes());

	 aTypes = ::utl::concatSequences(aTypes, HierarchyElementImpl::getTypes());
	 aTypes = ::utl::concatSequences(aTypes, ChangesNotifierImpl::getTypes());
	 
	 return aTypes;
}

//--------------------------------------------------------------------------
Any SAL_CALL HierarchyUpdateElement::queryInterface( const Type& _rType ) throw(RuntimeException)
{
	Any aReturn = HierarchyUpdateBase::queryInterface(_rType);
	if (!aReturn.hasValue())
		aReturn = HierarchyElementImpl::queryInterface(_rType);
	if (!aReturn.hasValue())
		aReturn = ChangesNotifierImpl::queryInterface(_rType);
	return aReturn;
}

//--------------------------------------------------------------------------
bool HierarchyUpdateElement::normalizeNestedPath(ConfigurationName& _rPath)
{
	if (!_rPath.isRelative())
	{
		ConfigurationName aMyPath(getHierarchicalName());
		if (!_rPath.isNestedIn(aMyPath))
			return false;

		_rPath = _rPath.relativeTo(aMyPath);
	}

	return true;
}

//--------------------------------------------------------------------------
INode const& HierarchyUpdateElement::getThisNode()
{
	OSL_ENSHURE(NULL != m_aSubtree, "HierarchyUpdateElement::getThisNode : invalid subtree node !" );
	return *m_aSubtree;
		// its a member of HierarchyAccessImpl, which is a base of our base HierarchyUpdateBase
}

//--------------------------------------------------------------------------
void HierarchyUpdateElement::getThisPath(TreePath& _rPathToFill)
{
	_rPathToFill = getNodePath();
	_rPathToFill.push_back(m_aSubtree);
}

//--------------------------------------------------------------------------
HierarchyAccessImpl* HierarchyUpdateElement::getParentAccess()
{
	MutexGuard aGuard(HierarchyUpdateBase::m_rMutex);
	MutexGuard aChainGuard(m_aAncestorChainProtection);
	OSL_ENSHURE(m_pNearestAncestor, "HierarchyUpdateElement::getParentAccess : should have an ancestor !");
	OSL_ENSHURE(m_aAncestorRelativePath.depth() > 0, "HierarchyUpdateElement::getParentAccess : senseless path depth !");
	if (!m_pNearestAncestor)
		return NULL;

	if (1 == m_aAncestorRelativePath.depth())
	{
		m_pNearestAncestor->acquire();
		// by definition, the object returned by getParentAccess has to be acquired once
		return m_pNearestAncestor;
	}

	// our nearest ancestor is not our parent, but a grandparent instead
	// So we need to create this in-between element which is our parent

	TreePath aPathFromRoot;
	getThisPath(aPathFromRoot);

	sal_Int32 nDepthBelowRoot = aPathFromRoot.size();
	sal_Int32 nDepthBelowNearestAncestor = m_aAncestorRelativePath.depth();
	OSL_ENSHURE(nDepthBelowRoot >= nDepthBelowNearestAncestor,
		"HierarchyUpdateElement::getParentAccess : inconsistent depths !");

	// plus the very root tree of the update structure (the getSubtreeAccess expects that the first element
	// in the node path is the tree of the object itself)
	aPathFromRoot.push_front(m_pUpdateRoot->getThisTree());

	while (nDepthBelowRoot > nDepthBelowNearestAncestor)
	{
		--nDepthBelowRoot;
		aPathFromRoot.pop_front();
	}

	OSL_ENSHURE(aPathFromRoot.size() >= 3, "HierarchyUpdateElement::getParentAccess : wrong path !");
		// the path should contain at least a) the tree to the root, b) the tree to our parent and
		// c) our own tree

	aPathFromRoot.pop_back();	// this is our own tree, which we do not need
	TreeRef aParentTree = aPathFromRoot.back();
	aPathFromRoot.pop_back();

	return m_pNearestAncestor->getSubtreeAccess(aParentTree, aPathFromRoot);
}

//--------------------------------------------------------------------------
void SAL_CALL HierarchyUpdateElement::disposing()
{
	m_pModifications = NULL;
	HierarchyUpdateBase::disposing();
	ChangesNotifierImpl::disposing();
}

//--------------------------------------------------------------------------
bool HierarchyUpdateElement::loadDefaults()
{
	TreePath(aPath);
	getThisPath(aPath);
	return m_pUpdateRoot->loadDefaultsAt(aPath);
}
//--------------------------------------------------------------------------
void HierarchyUpdateElement::integrated(SubtreeChange* _pNewObjectModifications)
{
	MutexGuard aGuard(HierarchyUpdateBase::m_rMutex);
	HierarchyUpdateBase::integrated(_pNewObjectModifications);
	m_pModifications = _pNewObjectModifications;
}

//--------------------------------------------------------------------------
void HierarchyUpdateElement::fireLocalChange(Change const& aChange)
{
	ConfigurationName aPathFromRoot = HierarchyElementImpl::getHierarchicalName(ConfigurationName(),getNodePath());
	m_pUpdateRoot->m_aNotifier.broadcast(aChange, aPathFromRoot.fullName());
}

//--------------------------------------------------------------------------
void HierarchyUpdateElement::executeLocalChanges(ValueChanges const& aChanges)
{
	updateLocalValues(aChanges);

	// HACK : fireLocalChanges for multiple (value) changes - should really build a subtree change
	ConfigurationName aPathFromRoot = HierarchyElementImpl::getHierarchicalName(ConfigurationName(),getNodePath());
	for (ValueChanges::const_iterator it = aChanges.begin(); it != aChanges.end(); ++it)
		m_pUpdateRoot->m_aNotifier.broadcast(*it, aPathFromRoot.fullName());
}

//--------------------------------------------------------------------------
sal_Int64 SAL_CALL HierarchyUpdateElement::getSomething( const Sequence< sal_Int8 >& _rId ) throw(RuntimeException)
{
	if (_rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  _rId.getConstArray(), 16 ) )
		return reinterpret_cast<sal_Int64>(this);
	return HierarchyUpdateBase::getSomething(_rId);
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > HierarchyUpdateElement::getUnoTunnelImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

//--------------------------------------------------------------------------
void HierarchyUpdateElement::insertedInto(HierarchyUpdateRoot* _pNewAccessRoot)
{
	::osl::ClearableMutexGuard aGuard(HierarchyUpdateBase::m_rMutex);
	OSL_ENSHURE(NULL == getChangesTree(), "HierarchyUpdateElement::insertedInto: not to be called if we have a changes tree !");
		// we could implement this if it's really needed ....

	if (_pNewAccessRoot != m_pUpdateRoot)
	{
		// acquire the new root
		_pNewAccessRoot->acquire();

		// forward the "insertedInto" event to all nearest descendants
		for (	ConstDescendantsIterator aDescLoop = m_aDescendants.begin();
				aDescLoop != m_aDescendants.end();
				++aDescLoop
			)
		{
			HierarchyUpdateElement* pDesc = static_cast<HierarchyUpdateElement*>(HierarchyUpdateBase::getSafeDescendant(aDescLoop, HierarchyUpdateElement::getUnoTunnelImplementationId()));
			if (pDesc)
				pDesc->insertedInto(_pNewAccessRoot);
		}

		// release the old root
		HierarchyUpdateRoot* pOldRoot = m_pUpdateRoot;
		m_pUpdateRoot = _pNewAccessRoot;

		aGuard.clear();	// the one reference we're holding may be the last to our root. And the mutex we're guarding
						// may be the one of exactly this root, so an release of the mutex _after_ the release of
						// the root may be deadly.
		pOldRoot->release();
	}
}

//..........................................................................
}	// namespace configmgr
//..........................................................................


