/*************************************************************************
 *
 *  $RCSfile: nodepropset.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "nodepropset.hxx"

#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif
#ifndef CONFIGMGR_API_UPDATEHELPER_HXX_
#include "cfgupdatehelper.hxx"
#endif

#ifndef _COM_SUN_STAR_BEANS_PROPERTY_HPP_
#include <com/sun/star/beans/Property.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HPP_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYSTATE_HPP_
#include <com/sun/star/beans/PropertyState.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYCHANGEEVENT_HPP_
#include <com/sun/star/beans/PropertyChangeEvent.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XPROPERTIESCHANGELISTENER_HPP_
#include <com/sun/star/beans/XPropertiesChangeListener.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_DISPOSEDEXCEPTION_HPP_
#include <com/sun/star/lang/DisposedException.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_WRAPPEDTARGETEXCEPTION_HPP_
#include <com/sun/star/lang/WrappedTargetException.hpp>
#endif

namespace configmgr
{
	using namespace ::com::sun::star::uno;
	using namespace ::com::sun::star::beans;

//-----------------------------------------------------------------------------------
	namespace {
		inline Type typeOfPropertySet() 
		{ 
			Reference< XPropertySet > const * const p = 0;
			return ::getCppuType(p); 
		}
		inline Type typeOfMultiPropertySet() 
		{ 
			Reference< XMultiPropertySet > const * const p = 0;
			return ::getCppuType(p); 
		}
		inline Type typeOfPropertyState() 
		{ 
			Reference< XPropertyState > const * const p = 0;
			return ::getCppuType(p); 
		}

		inline Reference<XPropertySet>	makePropSetRef(XPropertySet* p)	
		{ return p; }

		inline Reference<XMultiPropertySet> makeMultiPropSetRef(XMultiPropertySet* p) 
		{ return p; }

		inline Reference<XPropertyState> makePropStateRef(XPropertyState* p) 
		{ return p; }
	}
//-----------------------------------------------------------------------------------
// class PropertyEventDispatcher
//-----------------------------------------------------------------------------------

// listener registration
//-----------------------------------------------------------------------------------------------------------------
    void PropertyEventDispatcher::addListener( const OUString& aPropertyName, const ConstraintListener& xListener )
 	{
		osl::MutexGuard aGuard( this->mutex() );

		OSL_ENSHURE( !m_rBroadcastHelper.bInDispose, "do not add Listeners in the dispose call" );
		OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed - the listener will not be added" );
		OSL_ENSHURE( xListener.is(), "Error: Trying to add a NULL Listener" );

		if( !m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed && xListener.is())
		{
			if (aPropertyName.getLength() != 0)
				m_aConstraintListeners.addInterface( aPropertyName, xListener );
			else
				m_rBroadcastHelper.aLC.addInterface( ::getCppuType(&xListener), xListener );
		}
	}

	//-------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::addListener( const OUString& aPropertyName, const BoundListener& xListener )
	{
		osl::MutexGuard aGuard( this->mutex() );

		OSL_ENSHURE( !m_rBroadcastHelper.bInDispose, "do not add Listeners in the dispose call" );
		OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed - the listener will not be added" );
		OSL_ENSHURE( xListener.is(), "Error: Trying to add a NULL Listener" );

		if( !m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed && xListener.is() )
		{
			if (aPropertyName.getLength() != 0)
				m_aBoundListeners.addInterface( aPropertyName, xListener );
			else
				m_rBroadcastHelper.aLC.addInterface( ::getCppuType(&xListener), xListener );
		}
	}

	//-------------------------------------------------------------------------------------------------------------
    void PropertyEventDispatcher::addListener( const OUStringList& /*aPropertyNames*/, const MultiListener& xListener )
	{
		osl::MutexGuard aGuard( this->mutex() );

		OSL_ENSHURE( !m_rBroadcastHelper.bInDispose, "do not add Listeners in the dispose call" );
		OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed - the listener will not be added" );
		OSL_ENSHURE( xListener.is(), "Error: Trying to add a NULL Listener" );

		if( !m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed && xListener.is() )
		{
			// Ignoring the Names
			m_rBroadcastHelper.aLC.addInterface( ::getCppuType(&xListener), xListener );
		}
	}


// listener deregistration
//-----------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::removeListener( const OUString& aPropertyName, const ConstraintListener& xListener ) 
 	{
		osl::MutexGuard aGuard( this->mutex() );
		OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed - there are no listeners to remove" );
		// all listeners are automaticly released in a dispose call
		if( !m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed )
		{
			if (aPropertyName.getLength() != 0)
				m_aConstraintListeners.removeInterface( aPropertyName, xListener );
			else
				m_rBroadcastHelper.aLC.removeInterface( ::getCppuType(&xListener), xListener );
		}
	}

	//-------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::removeListener( const OUString& aPropertyName, const BoundListener& xListener )
 	{
		osl::MutexGuard aGuard( this->mutex() );
		OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed - there are no listeners to remove" );
		// all listeners are automaticly released in a dispose call
		if( !m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed )
		{
			if (aPropertyName.getLength() != 0)
				m_aBoundListeners.removeInterface( aPropertyName, xListener );
			else
				m_rBroadcastHelper.aLC.removeInterface( ::getCppuType(&xListener), xListener );
		}
	}
	//-------------------------------------------------------------------------------------------------------------

	void PropertyEventDispatcher::removeListener( const MultiListener& xListener )
	{
		osl::MutexGuard aGuard( this->mutex() );
		OSL_ENSHURE( !m_rBroadcastHelper.bDisposed, "object is disposed - there are no listeners to remove" );
		// all listeners are automaticly released in a dispose call
		if( !m_rBroadcastHelper.bInDispose && !m_rBroadcastHelper.bDisposed )
		{
			m_rBroadcastHelper.aLC.removeInterface( ::getCppuType(&xListener), xListener );
		}
	}

// event firing
//-----------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::doFireApprovals(beans::PropertyChangeEvent const* pEvents, sal_Int32 nEvents) throw(beans::PropertyVetoException,uno::RuntimeException)
	{
		using namespace cppu;
		// fire single changes
		for (int ix = 0; ix < nEvents; ++ix)
		{
			Reference<XVetoableChangeListener> const * const pSelectListener = 0;

			// first the specific listeners
			OInterfaceContainerHelper* pListeners = m_aConstraintListeners.getContainer(pEvents[ix].PropertyName);
			if (pListeners)
			{
				OInterfaceIteratorHelper aIt( *pListeners);
				while( aIt.hasMoreElements() )
				{
					XInterface * pL = aIt.next();;
					OSL_ENSHURE(pL, "NULL Listener found in event dispatch");
					static_cast<XVetoableChangeListener*>(pL)->vetoableChange( pEvents[ix] );
				}
				
			}

			// then the generic listeners
			pListeners = m_rBroadcastHelper.aLC.getContainer(::getCppuType(pSelectListener));
			if (pListeners)
			{
				OInterfaceIteratorHelper aIt( *pListeners);
				while( aIt.hasMoreElements() )
				{
					XInterface * pL = aIt.next();;
					OSL_ENSHURE(pL, "NULL Listener found in event dispatch");
					static_cast<XVetoableChangeListener*>(pL)->vetoableChange( pEvents[ix] );
				}
				
			}
		}
	}

	//-------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::doFireChanges  (beans::PropertyChangeEvent const* pEvents, sal_Int32 nEvents) throw(uno::RuntimeException)
	{
		using namespace cppu;
		// fire single changes first
		for (int ix = 0; ix < nEvents; ++ix)
		{
			Reference<XPropertyChangeListener> const * const pSelectListener = 0;

			// first the specific listeners
			OInterfaceContainerHelper* pListeners = m_aBoundListeners.getContainer(pEvents[ix].PropertyName);
			if (pListeners)
			{
				OInterfaceIteratorHelper aIt( *pListeners);
				while( aIt.hasMoreElements() )
				{
					XInterface * pL = aIt.next();;
					OSL_ENSHURE(pL, "NULL Listener found in event dispatch");
					static_cast<XPropertyChangeListener*>(pL)->propertyChange( pEvents[ix] );
				}
				
			}

			// then the generic listeners
			pListeners = m_rBroadcastHelper.aLC.getContainer(::getCppuType(pSelectListener));
			if (pListeners)
			{
				OInterfaceIteratorHelper aIt( *pListeners);
				while( aIt.hasMoreElements() )
				{
					XInterface * pL = aIt.next();;
					OSL_ENSHURE(pL, "NULL Listener found in event dispatch");
					static_cast<XPropertyChangeListener*>(pL)->propertyChange( pEvents[ix] );
				}
				
			}
		}

		// fire multiple changes now
		{
			Reference<XPropertiesChangeListener> const * const pSelectMultiListener = 0;
			OInterfaceContainerHelper* pListeners = m_rBroadcastHelper.aLC.getContainer(::getCppuType(pSelectMultiListener));
			if (pListeners)
			{
				Sequence<PropertyChangeEvent> const aEventSequence(pEvents,nEvents);

				OInterfaceIteratorHelper aIt( *pListeners);
				while( aIt.hasMoreElements() )
				{
					XInterface * pL = aIt.next();;
					OSL_ENSHURE(pL, "NULL Listener found in event dispatch");
					static_cast<XPropertiesChangeListener*>(pL)->propertiesChange( aEventSequence );
				}
				
			}
		}
	}


	//-------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::fireApprovals(PropertyChanges const& rEvents) throw(beans::PropertyVetoException,uno::RuntimeException)
	{
		if (!rEvents.empty())
			doFireApprovals(&rEvents[0], rEvents.size());
	}
	void PropertyEventDispatcher::fireChanges  (PropertyChanges const& rEvents) throw(uno::RuntimeException)
	{
		if (!rEvents.empty())
			doFireChanges(&rEvents[0], rEvents.size());
	}

	//-------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::fireApproval(beans::PropertyChangeEvent const& rEvent) throw(beans::PropertyVetoException,uno::RuntimeException)
	{
		doFireApprovals(&rEvent,1);
	}

	//-------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::fireChange  (beans::PropertyChangeEvent const& rEvent) throw(uno::RuntimeException)
	{
		doFireChanges(&rEvent,1);
	}

//-----------------------------------------------------------------------------------------------------------------
	void PropertyEventDispatcher::dispose(XPropertySet* pSource)
	{
	// Create an event with this as sender
		Reference < XPropertySet  > rSource(pSource );
		lang::EventObject aEvt(rSource);

		// inform all listeners to release this object
		// The listener containers are automaticly cleared
		m_aBoundListeners.disposeAndClear( aEvt );
		m_aConstraintListeners.disposeAndClear( aEvt );
	}

	//-------------------------------------------------------------------------------------------------------------
	bool PropertyEventDispatcher::checkAlive(XPropertySet* pSource)
	{
		if( m_rBroadcastHelper.bDisposed )
			throw lang::DisposedException(	OUString::createFromAscii("Object is disposed"), pSource );

		return !m_rBroadcastHelper.bInDispose;
	}

//-----------------------------------------------------------------------------------
// class NodePropertySetInfo
//-----------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------
	namespace {

		class CollectProperties : NodeAction
		{
			typedef sal_Int16 PropAttributes;
			std::vector< Property > m_aProperties;
			NodePropertySet::Options m_options;
		public:
			CollectProperties(NodePropertySet::Options options) : m_aProperties() {}
			CollectProperties(NodePropertySet::Options options, sal_Int32 nCount) : m_aProperties() { m_aProperties.reserve(nCount); }

			Property forNode(NodeRef aNode)
			{ 
				OSL_ASSERT(aNode);
				reset();
				applyToNode(*aNode);
				OSL_ASSERT(m_aProperties.size() == 1);
				return m_aProperties[0];
			}
			Sequence<Property> forChildren(TreeRef aTree) 
			{ 
				OSL_ASSERT(aTree);
				reset();
				applyToChildren(*aTree); 
				return Sequence<Property>(&m_aProperties[0], m_aProperties.size());
			}
		private:
			void reset() { m_aProperties.clear(); }
			void handle(ValueNode const& aValue);
			void handle(ISubtree const& aSubtree);
			void setAttributes(PropAttributes& aPropAttr, INode const& aNode);
		};
	//-----------------------------------------------------------------------------------
		void CollectProperties::setAttributes(PropAttributes& aPropAttr, INode const& aNode)
		{
			using namespace PropertyAttribute;

			NodeAttributes aNodeAttr = aNode.getAttributes();

			if (!aNodeAttr.writable) aPropAttr |= READONLY;
			if (aNodeAttr.optional) aPropAttr |= MAYBEVOID;
			if (aNodeAttr.notified) aPropAttr |= BOUND;
			if (aNodeAttr.constrained) aPropAttr |= CONSTRAINED;

			if (!(m_options & NodePropertySet::withUpdates)) aPropAttr |= READONLY;

			if (!(m_options & NodePropertySet::withPropertyState)) 
				aPropAttr &= ~PropAttributes(MAYBEDEFAULT | MAYBEAMBIGUOUS);
		}
		//-----------------------------------------------------------------------------------
		void CollectProperties::handle(ValueNode const& aValue)
		{
			typedef sal_Int32 Handle;
			using namespace PropertyAttribute;

			PropAttributes baseAttributes = PropAttributes(0);
			if (aValue.hasDefault())
				baseAttributes = MAYBEDEFAULT;

			setAttributes(baseAttributes,aValue);

			m_aProperties.push_back(
					Property (	aValue.getName(),
								Handle(-1),
								aValue.getValueType(),
								baseAttributes
					)		);
		}
		//-----------------------------------------------------------------------------------
		void CollectProperties::handle(ISubtree const& aSubtree)
		{
			typedef sal_Int32 Handle;
			using namespace PropertyAttribute;
			Reference<XInterface> const*const pSelectType = 0;

			PropAttributes baseAttributes = MAYBEDEFAULT | MAYBEAMBIGUOUS;

			setAttributes(baseAttributes,aSubtree);

			m_aProperties.push_back(
					Property (	aSubtree.getName(),
								Handle(-1),
								::getCppuType(pSelectType), 
								baseAttributes
					)		);
		}

	//-----------------------------------------------------------------------------------
	}

//-----------------------------------------------------------------------------------
	Sequence< Property > SAL_CALL NodePropertySetInfo::getProperties(void)
		throw(RuntimeException)
	{
		OReadSynchronized aGuard(m_pLock);
		//sal_Int32 nChildren = m_aTree->getChildCount();

		return CollectProperties(m_options).forChildren(m_aTree);
	}

	//-----------------------------------------------------------------------------------
    Property SAL_CALL NodePropertySetInfo::getPropertyByName(const OUString& sPropertyName) 
		throw(UnknownPropertyException, RuntimeException)
	{
		OReadSynchronized aGuard(m_pLock);
		NodeRef aNode = m_aTree->getChild(sPropertyName);
		if (!aNode)
		{
			OUString sMessage = OUString::createFromAscii("NodePropertySetInfo: getPropertyByName:");
			sMessage += OUString::createFromAscii("No Property named '");
			sMessage += sPropertyName;
			sMessage += OUString::createFromAscii("'");
			throw UnknownPropertyException(sMessage,static_cast<XPropertySetInfo*>(this));
		}

		return CollectProperties(m_options,1).forNode(aNode);
	}

	//-----------------------------------------------------------------------------------
    sal_Bool SAL_CALL NodePropertySetInfo::hasPropertyByName(const OUString& sPropertyName)
		throw(RuntimeException)
	{
		OReadSynchronized aGuard(m_pLock);
		NodeRef aNode = m_aTree->getChild(sPropertyName);
		return aNode != NULL;
	}

//-----------------------------------------------------------------------------------
// class NodePropertySet
//-----------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------
// Constructors
//-----------------------------------------------------------------------------------
	NodePropertySet::NodePropertySet(BroadcastHelper& rBHelper_, Options opt)
		: m_aDispatcher(rBHelper_)
		, m_options(opt)
	{
	}
	NodePropertySet::~NodePropertySet()
	{
	}

//-----------------------------------------------------------------------------------
// XInterface & XTypeProvider helpers
//-----------------------------------------------------------------------------------

	sal_Bool NodePropertySet::queryPropertyInterfaces(Any& xOut, Type const& rType)
	{
		if (xOut.hasValue())
			OSL_ENSURE(false, "The interface was already found");

		else if (rType == typeOfPropertySet())
			xOut <<= makePropSetRef(this);

		else if (rType == typeOfMultiPropertySet())
			xOut <<= makeMultiPropSetRef(this);

		else if (rType == typeOfPropertyState() && (m_options & withPropertyState))
			xOut <<= makePropStateRef(this);

		return xOut.hasValue();
	}

//-----------------------------------------------------------------------------------
	sal_Int32 NodePropertySet::addPropertyTypes(Sequence< Type >& rTypes)
	{
		bool const bHasPropState = ((m_options & withPropertyState) != 0);

		sal_Int32 nAdd = 2;
		if (bHasPropState) ++nAdd;

		sal_Int32 const nLen = rTypes.getLength();
		rTypes.realloc(nLen + nAdd);

		Type* pOut = rTypes.getArray() + nLen;

		*pOut++ = typeOfPropertySet();
		*pOut++ = typeOfMultiPropertySet();
		if (bHasPropState) *pOut++ = typeOfPropertyState();

		OSL_ASSERT(rTypes.getArray() + rTypes.getLength() == pOut);
		return nAdd;
	}

//-----------------------------------------------------------------------------------
// Internal state
//-----------------------------------------------------------------------------------
	void NodePropertySet::checkAlive()
	{		
		OSL_VERIFY(m_aDispatcher.checkAlive(this)); // "This function was called while the object is being disposed");
	}

	void SAL_CALL NodePropertySet::disposing()
	{		
		m_aDispatcher.dispose(this);
	}

	void NodePropertySet::ensureDefaults() throw (uno::RuntimeException)
	{
		// TO DO: might be improved alot
		OSL_VERIFY(loadDefaults()); // Are they implemented yet ? 
	}
//-----------------------------------------------------------------------------------
// Element Nodes
//-----------------------------------------------------------------------------------
	namespace
	{
		OUString missingProperty(OUString const& aName, NodeRef aParent = 0)
		{
			OUString 
			sMessage =	OUString::createFromAscii("Property '");
			sMessage +=	aName;
			sMessage +=	OUString::createFromAscii("' not found");
			if (aParent)
			{
				sMessage +=	OUString::createFromAscii(" in configuration node '");
				sMessage += aParent->getName();
				sMessage += OUString::createFromAscii("'");
			}
			sMessage += OUString::createFromAscii(" !");
			return sMessage;
		}
	}
	inline NodeRef NodePropertySet::getPropertyNode(OUString const& aName)
	{
		TreeRef aTree = getThisTree();
		OSL_ASSERT(aTree);
		return aTree->getChild(aName);
	}
	inline NodeRef NodePropertySet::getValidPropertySetNode(OUString const& aName)
	{
		NodeRef aNode = getPropertyNode(aName);
		if (!aNode)
			throw UnknownPropertyException(	missingProperty(aName, getThisTree()),
											makePropSetRef(this) );
		return aNode;
	}
	inline NodeRef NodePropertySet::getValidPropertyStateNode(OUString const& aName)
	{
		NodeRef aNode = getPropertyNode(aName);
		if (!aNode)
			throw UnknownPropertyException(	missingProperty(aName, getThisTree()),
											makePropStateRef(this) );
		return aNode;
	}

//-----------------------------------------------------------------------------------
// Event firing
//-----------------------------------------------------------------------------------
	sal_Int32 NodePropertySet::buildEvents(PropertyChanges& rEvents, ValueChanges const& rChanges, bool forVeto)
	{
		typedef sal_Int32 PropHandle;

		OSL_ASSERT(rEvents.empty()); // PRE
		rEvents.reserve(rChanges.size());

		osl::MutexGuard aGuard(mutex());
		OReadSynchronized aTreeGuard(getTreeSync());

		for(ValueChanges::const_iterator it = rChanges.begin(); it != rChanges.end(); ++it)
		{
			if (NodeRef aNode = getPropertyNode(it->getNodeName()))
			{
				NodeAttributes attr = aNode->getAttributes();
				OSL_ASSERT(attr.writable);
				if (forVeto ? attr.constrained : attr.notified)
				{
					rEvents.push_back(
						PropertyChangeEvent(
							makePropSetRef(this),
							it->getNodeName(),
							sal_True,
							PropHandle(-1),
							it->getOldValue(),
							it->getNewValue()
					)	);
				}
			}
			else
				OSL_ASSERT("");
		}
		if (!rEvents.empty())
			rEvents.back().Further = sal_False;

		return rEvents.size();
	}

//-----------------------------------------------------------------------------------
	void NodePropertySet::fireVetoableChanges(ValueChanges const& rChanges) 
		throw(beans::PropertyVetoException,uno::RuntimeException)
	{
		if (!(m_options & withUpdates))
		{
			OUString
			sMessage  =	OUString::createFromAscii("Configuration Property Set: Error in update: ");
			sMessage += OUString::createFromAscii("Trying to update values in a readonly Configuration access. ");
			beans::PropertyVetoException aVetoReadOnly(sMessage,makePropSetRef(this));
		}
		PropertyChanges aEvents;
		if (buildEvents(aEvents, rChanges, true))
			m_aDispatcher.fireApprovals(aEvents);
	}

//-----------------------------------------------------------------------------------
	void NodePropertySet::firePropertyChanges(ValueChanges const& rChanges) 
		throw(uno::RuntimeException)
	{
		PropertyChanges aEvents;
		if (buildEvents(aEvents, rChanges, false))
			m_aDispatcher.fireChanges(aEvents);
	}

//-----------------------------------------------------------------------------------
// New overrideable and helper functions
//-----------------------------------------------------------------------------------
	Reference< script::XTypeConverter > NodePropertySet::getTypeConverter() const
	{
		return Reference< script::XTypeConverter >();
	}

//-----------------------------------------------------------------------------------
// Associated helper functions
	/**
	 * Create an any for any child element 
	 */
	uno::Any NodePropertySet::makeChildValue(NodeRef aChildNode)
	{
		OSL_ASSERT(aChildNode);

		if (ValueNode const* pValue = aChildNode->asValueNode())
			return pValue->getValue();

		if (TreeRef aSubtree = aChildNode->asISubtree())
			return getChildAccess(aSubtree);

		OSL_ENSURE(false,"Corrupt registry: Unknown Node type found");
		return Any();
	}

//-----------------------------------------------------------------------------------
// XPropertySet & XMultiPropertySet
//-----------------------------------------------------------------------------------

// PropertySetInfo
//-----------------------------------------------------------------------------------
	Reference< XPropertySetInfo > SAL_CALL NodePropertySet::getPropertySetInfo(  ) 
			throw(RuntimeException)
	{
		return new NodePropertySetInfo(getThisTree(), getTreeSync(), m_options);
	}

// Setting Property Values
//-----------------------------------------------------------------------------------
	// XPropertySet
    void SAL_CALL NodePropertySet::setPropertyValue( const OUString& aPropertyName, const Any& aValue ) 
			throw(UnknownPropertyException, PropertyVetoException, lang::IllegalArgumentException, 
				  lang::WrappedTargetException, RuntimeException)
	{
		OSL_ENSHURE(m_options & withUpdates, "Error: Setting a value in a read-only configuration access");
		osl::ClearableMutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );
		OWriteSynchronized aUpdateTreeGuard(getUpdateSync());

		checkAlive();

		NodeRef aNode = getValidPropertySetNode(aPropertyName);
		if (ValueNode const * pValue = aNode->asValueNode())
		try
		{
			ValueUpdater anUpdate(*pValue,aValue,ValueChange::changeValue,getTypeConverter());

			bool bChanged = true;
			ValueChange aChange = anUpdate.prepareChange(bChanged,makePropSetRef(this),1);
			
			if (bChanged)
			{
				// leave read guard locked during notification !
				//aReadGuard.clear();
				// clear the locks for notification
				aGuard.clear();

				executeLocalChange(aChange);
			}
		}
		catch (PropertyVetoException& ) { throw; }
		catch (lang::IllegalArgumentException& ) { throw; }
		catch (lang::WrappedTargetException& ) { throw; }
		catch (uno::RuntimeException& ) { throw; }
		catch (uno::Exception& e)
		{
			OUString 
			sMessage  = OUString::createFromAscii("Configuration Property Set: An Exception occured while updating: ");
			sMessage += e.Message;
			Any anyE; 
			anyE <<= e;
			throw lang::WrappedTargetException(sMessage,makePropSetRef(this),anyE);
		}
		else
		{
			OSL_ASSERT(!"Replacing non-values is (currently) not supported");
			OUString
			sMessage  = OUString::createFromAscii("Cannot set Property '");
			sMessage += aPropertyName;
			sMessage += OUString::createFromAscii("' in configuration node '");
			sMessage += aNode->getName();
			sMessage += OUString::createFromAscii("': Inner nodes cannot be replaced.");
			throw lang::IllegalArgumentException(sMessage, makePropSetRef(this), 1);
		}
		OSL_ENSHURE(m_options & withUpdates, "Logic Error: Attempt to update did not trigger an Exception");
	}
	//-----------------------------------------------------------------------------------

 	// XMultiPropertySet
	void SAL_CALL NodePropertySet::setPropertyValues( const Sequence< OUString >& aPropertyNames, const Sequence< Any >& aValues ) 
			throw(PropertyVetoException, lang::IllegalArgumentException, 
				  lang::WrappedTargetException, RuntimeException)
	{
		OSL_ENSHURE(m_options & withUpdates, "Error: Setting values in a read-only configuration access");
		if (aPropertyNames.getLength() != aValues.getLength())
		{
			OUString 
			sMessage  = OUString::createFromAscii("Number of properties to set differs from ");
			sMessage += OUString::createFromAscii("the number of values provided");
			throw lang::IllegalArgumentException(sMessage,makePropSetRef(this),-1);
		}

		ValueChanges aChanges;
		aChanges.reserve(aPropertyNames.getLength());

		osl::ClearableMutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );
		OWriteSynchronized aUpdateTreeGuard(getUpdateSync());

		checkAlive();

		for(int ix = 0; ix < aPropertyNames.getLength(); ++ix)
		{
			NodeRef aNode = getPropertyNode(aPropertyNames[ix]);
			// Simply ignoring unknown properties ? Interface spec says so !
			if (!aNode) continue;

			if (ValueNode const * pValue = aNode->asValueNode())
			try
			{
				ValueUpdater anUpdate(*pValue,aValues[ix],ValueChange::changeValue,getTypeConverter());

				bool bChanged = true;
				ValueChange aChange = anUpdate.prepareChange(bChanged,makePropSetRef(this),ix);
				
				if (bChanged)
					aChanges.push_back(aChange);
			}
			catch (PropertyVetoException& ) { throw; }
			catch (lang::IllegalArgumentException& ) { throw; }
			catch (lang::WrappedTargetException& ) { throw; }
			catch (uno::RuntimeException& ) { throw; }
			catch (uno::Exception& e)
			{
				OUString 
				sMessage  = OUString::createFromAscii("Configuration MultiProperty Set: An Exception occured while preparing the update: ");
				sMessage += e.Message;
				Any anyE; 
				anyE <<= e;
				throw lang::WrappedTargetException(sMessage,makeMultiPropSetRef(this),anyE);
			}
			else
			{
				OSL_ASSERT(!"Replacing non-values is (currently) not supported");
				OUString
				sMessage  = OUString::createFromAscii("Cannot set Property '");
				sMessage += aPropertyNames[ix];
				sMessage += OUString::createFromAscii("' in configuration node '");
				sMessage += aNode->getName();
				sMessage += OUString::createFromAscii("': Inner nodes cannot be replaced.");
				throw lang::IllegalArgumentException(sMessage, makePropSetRef(this), 1);
			}
		}
		if (!aChanges.empty())
		try
		{
			// leave read guard locked during notification !
			//aReadGuard.clear();

			// clear the locks for notification
			aGuard.clear();

			executeLocalChanges(aChanges);
		}
		catch (PropertyVetoException& ) { throw; }
		catch (lang::IllegalArgumentException& ) { throw; }
		catch (lang::WrappedTargetException& ) { throw; }
		catch (uno::RuntimeException& ) { throw; }
		catch (uno::Exception& e)
		{
			OUString 
			sMessage  = OUString::createFromAscii("Configuration MultiProperty Set: An Exception occured while updating: ");
			sMessage += e.Message;
			Any anyE; 
			anyE <<= e;
			throw lang::WrappedTargetException(sMessage,makeMultiPropSetRef(this),anyE);
		}
		OSL_ENSHURE(m_options & withUpdates, "Logic Error: Attempt to update did not trigger an Exception");
	}

// Getting Property Values
//-----------------------------------------------------------------------------------
	// XPropertySet
    Any SAL_CALL NodePropertySet::getPropertyValue( const OUString& aPropertyName ) 
			throw(UnknownPropertyException, lang::WrappedTargetException, RuntimeException)
	{
		osl::MutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );

		checkAlive();

		NodeRef aNode = getValidPropertySetNode(aPropertyName);
		return makeChildValue(aNode);
	}
	//-----------------------------------------------------------------------------------

 	// XMultiPropertySet
    Sequence< Any > SAL_CALL NodePropertySet::getPropertyValues( const Sequence< OUString >& aPropertyNames ) 
			throw(RuntimeException)
	{
		sal_Int32 const nLen = aPropertyNames.getLength();
		Sequence<Any> aValues( nLen );

		osl::MutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );

		checkAlive();

		for(sal_Int32 ix = 0; ix < nLen; ++ix)
		{
			NodeRef aNode = getPropertyNode(aPropertyNames[ix]);
			OSL_ENSHURE(aNode, "NodePropertySet: getPropertyValues: Unknown property requested");
			if (aNode)
				aValues[ix] = makeChildValue(aNode);
		}
		return aValues;
	}

	//-----------------------------------------------------------------------------------
   void SAL_CALL NodePropertySet::firePropertiesChangeEvent( const Sequence< OUString >& aPropertyNames, const Reference< XPropertiesChangeListener >& xListener ) 
			throw(RuntimeException)
	{
		OSL_ENSHURE(xListener.is(), "NodePropertySet: firePropertiesChangeEvent: Target is a NULL Listener");
		if (!xListener.is())
			return; // throw NullPointerException(...)

		sal_Int32 const nLen = aPropertyNames.getLength();
		OSL_ENSHURE(nLen != 0, "NodePropertySet: firePropertiesChangeEvent: Firing ZERO Events");
		if (nLen == 0)
			return; 

		Sequence<PropertyChangeEvent> aEvents( nLen );

		// Lock while building the list
		osl::ClearableMutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );

		checkAlive();

		for(sal_Int32 ix = 0; ix < nLen; ++ix)
		{
			NodeRef aNode = getPropertyNode(aPropertyNames[ix]);
			OSL_ENSHURE(aNode, "NodePropertySet: firePropertiesChangeEvent: Unknown property requested");

			if (aNode != 0)
			{ 
				aEvents[ix].PropertyName = aNode->getName(); 
				aEvents[ix].NewValue = makeChildValue(aNode); 

				if (ValueNode const* pValue = aNode->asValueNode())
					aEvents[ix].OldValue = pValue->getDefault();	// Q: What to put here ?
				else
					OSL_ASSERT(!aEvents[ix].OldValue.hasValue());		// ... and here ?
			}
			else  // Q: What to do here ??
			{
				OSL_ASSERT(!aEvents[ix].PropertyName.getLength());
				OSL_ASSERT(!aEvents[ix].NewValue.hasValue());
				OSL_ASSERT(!aEvents[ix].OldValue.hasValue());
			}

			aEvents[ix].Source	= makeMultiPropSetRef(this);
			aEvents[ix].Further	= true;
			aEvents[ix].PropertyHandle	= -1;
		}
		aEvents[nLen-1].Further = false;

		// Unlock mutexes for firing
		aReadGuard.clear(); // do we need this ? - can we safely do it ?
		aGuard.clear();		// clear guard to fire events

		xListener->propertiesChange(aEvents);
   }

// Adding and removing listeners
//-----------------------------------------------------------------------------------
	// XPropertySet
    void SAL_CALL NodePropertySet::addPropertyChangeListener( const OUString& aPropertyName, const Reference< XPropertyChangeListener >& xListener ) 
			throw(UnknownPropertyException, lang::WrappedTargetException, RuntimeException)
	{
		osl::MutexGuard aGuard(this->mutex());

		if (NodeRef aNode = getValidPropertySetNode(aPropertyName))
		{
			if (aNode->getAttributes().notified)
				m_aDispatcher.addListener(aPropertyName,xListener);
			else
				OSL_ENSHURE(false, "Registering listener on non-bound property");
		}
	}

    void SAL_CALL NodePropertySet::removePropertyChangeListener( const OUString& aPropertyName, const Reference< XPropertyChangeListener >& xListener ) 
			throw(UnknownPropertyException, lang::WrappedTargetException, RuntimeException)
	{
		osl::MutexGuard aGuard(this->mutex());

		if (getValidPropertySetNode(aPropertyName) != NULL)
			m_aDispatcher.removeListener(aPropertyName,xListener);
	}

	//-----------------------------------------------------------------------------------
 	// XMultiPropertySet
    void SAL_CALL NodePropertySet::addPropertiesChangeListener( const Sequence< OUString >& aPropertyNames, const Reference< XPropertiesChangeListener >& xListener ) 
			throw(RuntimeException)
	{
		osl::MutexGuard aGuard(this->mutex());

		// does not check for validity of names
		m_aDispatcher.addListener(aPropertyNames,xListener);
	}

    void SAL_CALL NodePropertySet::removePropertiesChangeListener( const Reference< XPropertiesChangeListener >& xListener ) 
			throw(RuntimeException)
	{
		osl::MutexGuard aGuard(this->mutex());

		// does not check for validity of names
		m_aDispatcher.removeListener(xListener);
	}
	//-----------------------------------------------------------------------------------

	// XPropertySet
    void SAL_CALL NodePropertySet::addVetoableChangeListener( const OUString& aPropertyName, const Reference< XVetoableChangeListener >& xListener ) 
			throw(UnknownPropertyException, lang::WrappedTargetException, RuntimeException)
 	{
		osl::MutexGuard aGuard(this->mutex());

		if (NodeRef aNode = getValidPropertySetNode(aPropertyName))
		{
			if (aNode->getAttributes().constrained)
				m_aDispatcher.addListener(aPropertyName,xListener);
			else
				OSL_ENSHURE(false, "Registering veto listener on non-constrained property");
		}
	}
 
	void SAL_CALL NodePropertySet::removeVetoableChangeListener( const OUString& aPropertyName, const Reference< XVetoableChangeListener >& xListener ) 
			throw(UnknownPropertyException, lang::WrappedTargetException, RuntimeException)
	{
		osl::MutexGuard aGuard(this->mutex());

		if (getValidPropertySetNode(aPropertyName) != NULL)
			m_aDispatcher.removeListener(aPropertyName,xListener);
	}

//-----------------------------------------------------------------------------------
// XPropertyState
//-----------------------------------------------------------------------------------

	namespace {
		typedef PropertyState State;
		const State direct = PropertyState_DIRECT_VALUE;
		const State deflt  = PropertyState_DEFAULT_VALUE;
		const State ambig  = PropertyState_AMBIGUOUS_VALUE;

		// unique value as mark
		const PropertyState stateUnknown= PropertyState((direct|deflt|ambig)+1); 

		class CollectState : NodeAction
		{
			State eState;
			void handle(ValueNode const& aValue);
			void handle(ISubtree const& aSubtree);
		public:
			State forNode(INode const& rNode)
			{
				eState = stateUnknown;
				this->applyToNode(rNode);
				OSL_ASSERT(eState != stateUnknown);
				return eState;
			}
		};
		void CollectState::handle(ValueNode const& aValue)
		{
			State eNodeState = aValue.isDefault() ? deflt : direct;
			switch (eState)
			{
			case ambig: 
				break;

			case stateUnknown: 
				eState = eNodeState; 
				break;

			default: 
				if (eState != eNodeState) 
					eState = ambig;
				break;
			}
		}

		void CollectState::handle(ISubtree const& aSubtree)
		{
			if (eState != ambig) // this wont change any more
				this->applyToChildren(aSubtree);
		}

	}

// XPropertyState
//-----------------------------------------------------------------------------------
    PropertyState SAL_CALL NodePropertySet::getPropertyState( const OUString& aPropertyName ) 
			throw(UnknownPropertyException, RuntimeException)
	{
		OSL_ENSHURE(m_options & withPropertyState, "How did you call this ? This object is no XPropertyState");
		ensureDefaults();

		osl::MutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );

		checkAlive();

		NodeRef aNode = getValidPropertyStateNode(aPropertyName);
		return CollectState().forNode(*aNode);
	}

	//-----------------------------------------------------------------------------------
    Sequence< PropertyState > SAL_CALL NodePropertySet::getPropertyStates( const Sequence< OUString >& aPropertyNames ) 
			throw(UnknownPropertyException, RuntimeException)
	{
		OSL_ENSHURE(m_options & withPropertyState, "How did you call this ? This object is no XPropertyState");
		ensureDefaults();

		sal_Int32 const nLen = aPropertyNames.getLength();
		Sequence<PropertyState> aStates( nLen );

		osl::MutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );

		checkAlive();

		CollectState aGetState;
		for(sal_Int32 ix = 0; ix < nLen; ++ix)
		{
			//NodeRef aNode = getPropertyNode(aPropertyNames[ix]);
			//OSL_ENSHURE(aNode, "NodePropertySet: getPropertyValues(), Unknown property requested");
			//if (aNode)

			NodeRef aNode = getValidPropertyStateNode(aPropertyNames[ix]);
			aStates[ix] = aGetState.forNode(*aNode);
		}
		return aStates;

	}

	//-----------------------------------------------------------------------------------
    void SAL_CALL NodePropertySet::setPropertyToDefault( const OUString& aPropertyName ) 
			throw(UnknownPropertyException, RuntimeException)
	{
		OSL_ENSHURE(m_options & withUpdates, "Error: Setting a value in a read-only configuration access");

		osl::ClearableMutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );
		OWriteSynchronized aUpdateTreeGuard(getUpdateSync());

		checkAlive();

		NodeRef aNode = getValidPropertySetNode(aPropertyName);
		if (ValueNode const * pValue = aNode->asValueNode())
		{
			ValueChanges aChanges;
			try
			{
				ValueUpdater anUpdate(*pValue,pValue->getDefault(),ValueChange::setToDefault,getTypeConverter());

				bool bChanged = true;
				ValueChange aChange = anUpdate.prepareChange(bChanged,makePropSetRef(this),-1);

				if (bChanged)
					aChanges.push_back(aChange);
			}
			catch (uno::RuntimeException& ) { throw; }
			catch (uno::Exception& ex)
			{
				OUString
				sMessage  = OUString::createFromAscii("NodePropertySet: SetPropertyToDefault: ");
				sMessage += OUString::createFromAscii("An unexpected exception occurred preparing the action: ");
				sMessage += ex.Message;
				throw uno::RuntimeException(sMessage, makePropStateRef(this));
			}

			if (!aChanges.empty())
			{
				// leave read guard locked during notification !
				//aReadGuard.clear();

				// clear the locks for notification
				aGuard.clear();

				try
				{
					executeLocalChanges(aChanges);
				}
				catch (uno::RuntimeException& ) { throw; }
				catch (uno::Exception& ex)
				{
					OUString
					sMessage  = OUString::createFromAscii("NodePropertySet: SetPropertyToDefault: ");
					sMessage += OUString::createFromAscii("An unexpected exception occurred while updating: ");
					sMessage += ex.Message;
					throw uno::RuntimeException(sMessage, makePropStateRef(this));
				}
			}
		}
		else
		{
			OSL_ASSERT(!"Setting to default whole subtrees is (currently) not supported");
			OUString
			sMessage  = OUString::createFromAscii("Cannot set Property '");
			sMessage += aPropertyName;
			sMessage += OUString::createFromAscii("' in configuration node '");
			sMessage += aNode->getName();
			sMessage += OUString::createFromAscii("' to default: Inner nodes cannot be replaced.");
			throw uno::RuntimeException(sMessage, makePropStateRef(this));
		}
		OSL_ENSHURE(m_options & withUpdates, "Logic Error: Attempt to update did not trigger an Exception");
	}

	//-----------------------------------------------------------------------------------
    Any SAL_CALL NodePropertySet::getPropertyDefault( const OUString& aPropertyName ) 
			throw(UnknownPropertyException, lang::WrappedTargetException, RuntimeException)
	{
		OSL_ENSHURE(m_options & withPropertyState, "How did you call getPropertyDefault() ? This object is no XPropertyState");
		ensureDefaults();

		osl::MutexGuard aGuard(this->mutex());
		OReadSynchronized aReadGuard( getTreeSync() );

		checkAlive();

		NodeRef aNode = getValidPropertyStateNode(aPropertyName);

		if (ValueNode const* pValue = aNode->asValueNode())
			return pValue->getDefault();

		else if (CollectState().forNode(*aNode) == PropertyState_DEFAULT_VALUE)
			return makeChildValue(aNode);

		return Any();
	}
//-----------------------------------------------------------------------------------


//-----------------------------------------------------------------------------------
}


