/*************************************************************************
 *
 *  $RCSfile: readaccessimpl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_READ_ACCESS_IMPL_HXX_
#include "readaccessimpl.hxx"
#endif

#ifndef CONFIGMGR_CONFNAME_HXX_
#include "confname.hxx"
#endif
#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif
#ifndef CONFIGMGR_API_ENCODENAME_HXX_
#include "encodename.hxx"
#endif

#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif

#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

namespace configmgr
{

	static AsciiServiceName const aReadAccessServices[] = 
	{
		"com.sun.star.configuration.ConfigurationAccess", 
		0
	};
	ServiceInfo const aReadSetAccessSI = 
	{ 
		"com.sun.star.configuration.configmgr.OReadOnlySetNode", 
		aReadAccessServices 
	};
	ServiceInfo const aReadGroupAccessSI = 
	{ 
		"com.sun.star.configuration.configmgr.OReadOnlyGroupNode", 
		aReadAccessServices 
	};

	ServiceInfo const& OReadOnlySetNode  ::staticServiceInfo = aReadSetAccessSI;
	ServiceInfo const& OReadOnlyGroupNode::staticServiceInfo = aReadGroupAccessSI;

	// special exported one
	ServiceInfo const aReadAccessSI = 
	{ 
		"com.sun.star.configuration.configmgr.HierarchyReadAccess", 
		aReadAccessServices 
	};

//------------------------------------------------------------------------------------------------------------------------
OReadOnlySetNode::OReadOnlySetNode(IConfigBroadcaster& rSource, ISynchronizedData const* pSync, TreeRef pNode, OUString aBasePath)
: ServiceComponentImpl(&aReadSetAccessSI)
, HierarchyReadAccess(this->rBHelper, rSource, pSync, pNode, aBasePath)
{
}

OReadOnlySetNode::OReadOnlySetNode(IConfigBroadcaster& rSource, ISynchronizedData const* pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath)
: ServiceComponentImpl(&aReadSetAccessSI)
, HierarchyReadAccess(this->rBHelper, rSource, pSync, pNode, pPath, aBasePath)
{
}

OReadOnlySetNode::OReadOnlySetNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode)
: ServiceComponentImpl(&aReadSetAccessSI)
, HierarchyReadAccess(this->rBHelper, aAncestorElement, pNode, pParentNode)
{
}

OReadOnlySetNode::OReadOnlySetNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath)
: ServiceComponentImpl(&aReadSetAccessSI)
, HierarchyReadAccess(this->rBHelper, aAncestorElement, pNode, pPath)
{
}

OReadOnlySetNode::OReadOnlySetNode(HierarchyReadAccess const& aAncestorElement, ParentElement toParent)
: ServiceComponentImpl(&aReadSetAccessSI)
, HierarchyReadAccess(this->rBHelper, aAncestorElement, toParent)
{
}

void OReadOnlySetNode::Init()
{
	osl_incrementInterlockedCount(&m_refCount);
	HierarchyReadAccess::Init();
	osl_decrementInterlockedCount(&m_refCount);
}


//------------------------------------------------------------------------------------------------------------------------
OReadOnlyGroupNode::OReadOnlyGroupNode(IConfigBroadcaster& rSource, ISynchronizedData const* pSync, TreeRef pNode, OUString aBasePath, bool withDefaults)
: ServiceComponentImpl(&aReadGroupAccessSI)
, PropsetReadAccess(this->rBHelper, rSource, pSync, pNode, aBasePath, withDefaults)
{
}

OReadOnlyGroupNode::OReadOnlyGroupNode(IConfigBroadcaster& rSource, ISynchronizedData const* pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath, bool withDefaults)
: ServiceComponentImpl(&aReadGroupAccessSI)
, PropsetReadAccess(this->rBHelper, rSource, pSync, pNode, pPath, aBasePath, withDefaults) 
{
}

OReadOnlyGroupNode::OReadOnlyGroupNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode, bool withDefaults)
: ServiceComponentImpl(&aReadGroupAccessSI)
, PropsetReadAccess(this->rBHelper, aAncestorElement, pNode, pParentNode, withDefaults)  
{
}

OReadOnlyGroupNode::OReadOnlyGroupNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath, bool withDefaults)
: ServiceComponentImpl(&aReadGroupAccessSI)
, PropsetReadAccess(this->rBHelper, aAncestorElement, pNode, pPath, withDefaults)  
{
}

OReadOnlyGroupNode::OReadOnlyGroupNode(HierarchyReadAccess const& aAncestorElement, ParentElement toParent, bool withDefaults)
: ServiceComponentImpl(&aReadGroupAccessSI)
, PropsetReadAccess(this->rBHelper, aAncestorElement, toParent, withDefaults)  
{
}

void OReadOnlyGroupNode::Init()
{
	osl_incrementInterlockedCount(&m_refCount);
	PropsetReadAccess::Init();
	osl_decrementInterlockedCount(&m_refCount);
}

// Component behavior
//------------------------------------------------------------------------------------------------------------------------

void SAL_CALL OReadOnlySetNode::disposing()
{
	HierarchyReadAccess::disposing();
	ServiceComponentImpl::disposing();
}
//------------------------------------------------------------------------------------------------------------------------
void SAL_CALL OReadOnlyGroupNode::disposing()
{
	PropsetReadAccess::disposing();
	ServiceComponentImpl::disposing();
}

//------------------------------------------------------------------------------------------------------------------------
// UNO 3 boilerplate stuff
//------------------------------------------------------------------------------------------------------------------------

// XInterface implementation
//------------------------------------------------------------------------------------------------------------------------
uno::Any SAL_CALL OReadOnlySetNode::queryInterface(uno::Type const& rType) throw (uno::RuntimeException)
{
	uno::Any aRet = ServiceComponentImpl::queryInterface(rType);

	if (!aRet.hasValue()) aRet = HierarchyReadAccess::queryInterface(rType);
	if (!aRet.hasValue()) aRet = ::cppu::queryInterface(rType, static_cast< util::XStringEscape* >(this));

	return aRet;
}

void SAL_CALL OReadOnlySetNode::acquire() throw ()
{
	ServiceComponentImpl::acquire();
}

void SAL_CALL OReadOnlySetNode::release() throw ()
{
	ServiceComponentImpl::release();
}

//------------------------------------------------------------------------------------------------------------------------
uno::Any SAL_CALL OReadOnlyGroupNode::queryInterface(uno::Type const& rType) throw (uno::RuntimeException)
{
	uno::Any aRet = ServiceComponentImpl::queryInterface(rType);

	if (!aRet.hasValue()) aRet = PropsetReadAccess::queryInterface(rType);

	return aRet;
}

void SAL_CALL OReadOnlyGroupNode::acquire() throw ()
{
	ServiceComponentImpl::acquire();
}

void SAL_CALL OReadOnlyGroupNode::release() throw ()
{
	ServiceComponentImpl::release();
}

// XTypeProvider implementation
//------------------------------------------------------------------------------------------------------------------------
uno::Sequence< uno::Type > SAL_CALL OReadOnlySetNode::getTypes() throw (uno::RuntimeException)
{
	uno::Sequence< uno::Type > aTypes( ServiceComponentImpl::getTypes() );

	aTypes = ::utl::concatSequences(aTypes, HierarchyReadAccess::getTypes() );

	uno::Reference<util::XStringEscape> const * const pEscaper = 0;
	return ::cppu::OTypeCollection(::getCppuType(pEscaper), aTypes).getTypes();
}

//------------------------------------------------------------------------------------------------------------------------
uno::Sequence< uno::Type > SAL_CALL OReadOnlyGroupNode::getTypes() throw (uno::RuntimeException)
{
	uno::Sequence< uno::Type > aTypes( ServiceComponentImpl::getTypes() );

	aTypes = ::utl::concatSequences(aTypes, PropsetReadAccess::getTypes() );

	return aTypes;
}

// XStringEscape - set node only
//--------------------------------------------------------------------------
OUString SAL_CALL OReadOnlySetNode::escapeString( const OUString& aString ) throw(lang::IllegalArgumentException, uno::RuntimeException)
{ 
	return escaped_name::escapeString(aString,this,1); 
}

//--------------------------------------------------------------------------
OUString SAL_CALL OReadOnlySetNode::unescapeString( const OUString& aEscapedString ) throw(lang::IllegalArgumentException, uno::RuntimeException)
{ 
	return escaped_name::unescapeString(aEscapedString,this,1); 
}

//------------------------------------------------------------------------------------------------------------------------

} // namespace configmgr



