/*************************************************************************
 *
 *  $RCSfile: changenotifier.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_CHANGESNOTIFIER_HXX_
#define CONFIGMGR_API_CHANGESNOTIFIER_HXX_

#ifndef _COM_SUN_STAR_UTIL_XCHANGESNOTIFIER_HPP_
#include <com/sun/star/util/XChangesNotifier.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XCOMPONENT_HPP_
#include <com/sun/star/lang/XComponent.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XHIERARCHICALNAME_HPP_
#include <com/sun/star/container/XHierarchicalName.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XCONTAINER_HPP_
#include <com/sun/star/container/XContainer.hpp>
#endif

#ifndef CONFIGMGR_API_EVENTS_HXX_
#include "confevents.hxx"
#endif
#ifndef CONFIGMGR_APITYPES_HXX_
#include "confapitypes.hxx"
#endif

#ifndef _CPPUHELPER_IMPLBASE2_HXX_
#include <cppuhelper/implbase2.hxx>
#endif
#ifndef _CPPUHELPER_INTERFACECONTAINER_HXX_
#include <cppuhelper/interfacecontainer.hxx>
#endif

namespace configmgr
{
	namespace css	= ::com::sun::star;
	namespace uno		= css::uno;
	namespace lang		= css::lang;
	namespace util		= css::util;
	namespace container	= css::container;

	class ConfigurationName;

typedef ::cppu::ImplHelper2	
			< util::XChangesNotifier
			, container::XContainer
			> ChangesNotifier_BaseHelper;

/////////////////////////////////////////////////////////////////////////////////////////////
class ChangesNotifierImpl
: public ChangesNotifier_BaseHelper
, private INodeListener
, protected container::XHierarchicalName // used (!) internally to get the name of *this
, protected lang::XComponent // used (!) internally to dispose *this
{
public:
	typedef ::cppu::OBroadcastHelper BroadcastHelper;

// Constructors & Destructors
	ChangesNotifierImpl(BroadcastHelper& rBHelper_);
	ChangesNotifierImpl(BroadcastHelper& rBHelper_, IConfigBroadcaster* _pSource);
	~ChangesNotifierImpl();

	/// late ctor. not really needed within this class, but derived classes my start observing herein
	virtual void Init();

	IConfigBroadcaster* changesSource() const { return m_pSource; }
	void observe(IConfigBroadcaster& rSource);
	void startObserving();
	void stopObserving();

// Interface methods
	// XInterface 
	virtual void SAL_CALL acquire() throw() = 0;
	virtual void SAL_CALL release() throw() = 0;
	virtual uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw(uno::RuntimeException) 
	{ return ChangesNotifier_BaseHelper::queryInterface(rType); }

	// XChangesNotifier
    virtual void SAL_CALL 
		addChangesListener( const uno::Reference< util::XChangesListener >& aListener ) 
			throw(uno::RuntimeException);

    virtual void SAL_CALL 
		removeChangesListener( const uno::Reference< util::XChangesListener >& aListener ) 
			throw(uno::RuntimeException);
 
	// XContainer
	virtual void SAL_CALL 
		addContainerListener( const uno::Reference< container::XContainerListener >& xListener ) 
			throw(uno::RuntimeException);
    virtual void SAL_CALL 
		removeContainerListener( const uno::Reference< container::XContainerListener >& xListener ) 
			throw(uno::RuntimeException);

protected:
	virtual void SAL_CALL disposing() = 0;

// event firing
	void fireChanges(Change const& aChange, OUString const& aPath);
private:
	void doFireChanges(Change const& aChange, ConfigurationName const& aPath);

// Relaying implementations	
	// IConfigListener implementation
	virtual void disposing(IConfigBroadcaster* pSource);

	// INodeListener implementation
	virtual void nodeChanged(Change const& aChange, OUString const& aPath, IConfigBroadcaster* pSource);
	virtual void nodeDeleted(OUString const& aPath, IConfigBroadcaster* pSource);


protected:
	BroadcastHelper& getBroadcastHelper() const { return m_rBroadcastHelper; }

private:
	BroadcastHelper& m_rBroadcastHelper;
	IConfigBroadcaster* m_pSource;
	IConfigBroadcaster* m_pObservedSource;
};

}
#endif // CONFIGMGR_API_CHANGESNOTIFIER_HXX_


