/*************************************************************************
 *
 *  $RCSfile: cmtree.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/* PLEASE DON'T DELETE ANY COMMENT LINES, ALSO IT'S UNNECESSARY. */

// STL set is need!
// because of size and compile time it won't be loaded here.

#ifndef _CMSET_HXX
#define _CMSET_HXX

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _OSL_CONDITN_HXX_
#include<osl/conditn.hxx>
#endif
#ifndef _OSL_MUTEX_HXX_
#include<osl/mutex.hxx>
#endif

#ifndef _COM_SUN_STAR_UNO_ANY_H_
#include <com/sun/star/uno/Any.hxx>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef __SGI_STL_MEMORY
#include <stl/memory>
#endif
#ifndef __SGI_STL_SET
#include <stl/set>
#endif

#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif

#ifndef _CONFIGMGR_SYNCHRONIZE_HXX_
#include "synchronize.hxx"
#endif

namespace configmgr
{

// ---------------------------------- STL set ----------------------------------
	struct ltNode
	{
		bool operator()(const configmgr::INode* n1, const configmgr::INode* n2) const
			{
				// return strcmp(n1->GetName(), n2->GetName()) < 0;
				return (n1->getName().compareTo(n2->getName()) < 0) ? 1 : 0;
			}
	};
	
	
	typedef std::set<INode*, ltNode> ChildList;
	
	class ChildListSet {
		ChildList m_aChildList;
		// ChildListSet operator=(ChildListSet const& aSet){ ChildListSet newCopy(aSet); m_aChildList.swap(newCopy.GetSet()); }
	public:
		ChildList& GetSet() {return m_aChildList;}
		ChildList const& GetSet() const {return m_aChildList;}

		ChildListSet() {}
		ChildListSet(ChildListSet const&);
		~ChildListSet();
	};
	

// Inner Node
	class Subtree : public ISubtree
	{
		//	set<Node*> children;
		ChildListSet		m_aChildren;
		sal_Int16 			m_nLevel;
		sal_Bool  			m_bComplete;				/// true, if Subtree is completely filled

		virtual INode* doGetChild(OUString const& name) const;
	public:
		Subtree();
		Subtree(OUString const& aName);
		virtual ~Subtree();

		INode* createChild(OUString const& name);

		virtual INode* addChild(std::auto_ptr<INode> node);	// takes ownership
		virtual ::std::auto_ptr<INode> removeChild(OUString const& name);

		virtual INode* clone() const;

// Iteration support
		virtual void forEachChild(NodeAction& anAction) const;
		virtual void forEachChild(NodeModification& anAction);

		void setLevel(sal_Int16 _nLevel);
		sal_Int16 getLevel() {return m_nLevel;}

		void setComplete(sal_Bool bComplete) {m_bComplete = bComplete;}
		sal_Bool getComplete() {return m_bComplete;}

		// "rtti"
		RTTI(Subtree, ISubtree);
	};
	


	// to search in ChildListSet a value
	class SearchNode : public INode
	{
		// why you want to search with no names.
		SearchNode();

	public:
		SearchNode(OUString const& aName);
		virtual ~SearchNode();
		virtual INode* clone() const;
		
		OUString getName() const;
		
// double dispatch support
		virtual void dispatch(NodeAction& anAction) const { }
		virtual void dispatch(NodeModification& anAction) { } 
	};

// -----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////

	class OTreeAccessor;
	/** a tree of INode's without to much intelligence. No caching, not automatic loading, just a container.
		<BR>
		Though the object supports the <type>ISynchronizedData</type> interface, no own locking occurs, this is the responsibility
		of the objects using an instance of this class.
	*/
	class Tree
			:public ITreeProvider
	{
		Subtree*		m_pRoot;
		OTreeAccessor*	m_pLock;

	public:
		Tree();
		~Tree();
		
		Subtree * addSubtree(const ConfigurationName& _rLocation, std::auto_ptr<Subtree> pSubtree, sal_Int16 nReliableLevels);

		sal_Int16 getLevelsAvailable(ISubtree const* pSubtree) const;
//or	sal_Int16 getLevelsAvailable(OUString const& aSubtreePath) const;

		virtual ISubtree const*	getSubtree( OUString const& aComponentName ) const;
		virtual ISubtree *		requestSubtree( OUString const& aComponentName, sal_Int16 nMinLevels = ALL_LEVELS );
		
		virtual const INode *	getNode(const OUString& _rPath);

		virtual void updateTree( TreeChangeList& ) throw (starlang::WrappedTargetException, uno::RuntimeException);

		sal_Int32 getNodeId(const OUString& aPath);

		OTreeAccessor* getTreeLock() const { return m_pLock; }
	protected:
		// ISyncronizedData
		virtual void acquireReadAccess() const;
		virtual void releaseReadAccess() const;
		virtual void acquireWriteAccess();
		virtual void releaseWriteAccess();
	};


////////////////////////////////////////////////////////////////////////////////
	
/*
	class Tree
	{
		Subtree const* getSubtree( OUString const& aComponentName );
		void ApplyChangeList( TreeChangeList const& );
	};
*/

////////////////////////////////////////////////////////////////////////////////
	
} // namespace configmgr

#endif

