/*************************************************************************
 *
 *  $RCSfile: cmtreemodel.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
	//**************************************************************************
//
//		$Header: /cvs/oo/util/configmgr/source/inc/cmtreemodel.hxx,v 1.1.1.1 2000/09/18 16:13:40 hr Exp $
//
// =========================================================================
//
//		$Date: 2000/09/18 16:13:40 $
//		$Author: hr $
//		$Revision: 1.1.1.1 $
//
// =========================================================================
//
//		created: 12.06.00 
//		creator: LLA, JB
// Description:
// Configuration Management Object Tree Header
//
//		Copyright (c) 2000 Sun Microsystems Inc. 
//
//**************************************************************************

/* PLEASE DON'T DELETE ANY COMMENT LINES, ALSO IT'S UNNECESSARY. */


#ifndef CONFIGMGR_CMTREEMODEL_HXX
#define CONFIGMGR_CMTREEMODEL_HXX

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _CONFIGMGR_COMMONTYPES_HXX_
#include "commontypes.hxx"
#endif
#ifndef _CONFIGMGR_SYNCHRONIZE_HXX_
#include "synchronize.hxx"
#endif

#ifndef _COM_SUN_STAR_UNO_ANY_H_
#include <com/sun/star/uno/Any.hxx>
#endif
#ifndef _COM_SUN_STAR_UNO_TYPE_HXX_
#include <com/sun/star/uno/Type.hxx>
#endif
#ifndef _COM_SUN_STAR_UNO_SEQUENCE_HXX_
#include <com/sun/star/uno/Sequence.hxx>
#endif
#ifndef _COM_SUN_STAR_LANG_WRAPPEDTARGETEXCEPTION_HPP_
#include <com/sun/star/lang/WrappedTargetException.hpp>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef __SGI_STL_MEMORY
#include <stl/memory>
#endif
#ifndef _MAP_
#include <map>
#endif
#include <string.h>

#ifndef CONFIGMGR_CONFNAME_HXX_
#include "confname.hxx"
#endif

namespace configmgr
{

	namespace uno = com::sun::star::uno;
	namespace starlang = com::sun::star::lang;
	using ::rtl::OUString;

	class INode;
	class ISubtree;
	class ValueNode;

#define RTTI_BASE(classname)	\
	virtual sal_Bool isA(const sal_Char* _pName) const	\
	{	\
		return	0 == strcmp(_pName, getStaticType());	\
	}	\
	const sal_Char* getType() const { return #classname; }	\
	static const sal_Char* getStaticType() { return #classname; }

#define RTTI(classname, baseclassname)	\
	virtual sal_Bool isA(const sal_Char* _pName) const	\
	{	\
		return	0 == strcmp(_pName, getStaticType())	\
			||	baseclassname::isA(_pName);	\
	}	\
	const sal_Char* getType() const { return #classname; }	\
	static const sal_Char* getStaticType() { return #classname; }

#ifndef ISA
#define ISA(classname) isA(classname::getStaticType())
#endif

// Visitors
	//==========================================================================
	//= Visitors
	//==========================================================================
	struct NodeAction
	{
		virtual void handle(ValueNode const&) = 0;
		virtual void handle(ISubtree const&) = 0;

		void applyToNode(INode const&);
		void applyToChildren(ISubtree const&);
	};

	struct NodeModification
	{
		virtual void handle(ValueNode&) = 0;
		virtual void handle(ISubtree&) = 0;

		void applyToNode(INode&);
		void applyToChildren(ISubtree&);
	};
	
// Base INode
	//==========================================================================
	//= Base INode
	//==========================================================================
	struct NodeAttributes
	{
		bool writable : 1;						 // derived (from Parent)
		bool optional : 1;
		bool notified : 1;						 // derived (from Parent)
		bool constrained : 1;
		NodeAttributes() 
				:writable(true),
				 optional(false),notified(true), constrained(false) {}

	};
	

	class INode
	{
		NodeAttributes m_aNodeAttribute;
		OUString       m_aName;

	public:
		INode();
		INode(OUString const& aName);
		virtual ~INode();
		virtual INode* clone() const = 0;
		
		OUString getName() const;
		NodeAttributes getAttributes() const;

		void setName(const OUString& _rNewName) { m_aName = _rNewName; }
			// to be used with caution. If the node is referenced from somewhere else under it's old name,
			// you may have problems with this inconsistence
		
		virtual ValueNode* asValueNode();
		virtual ValueNode const* asValueNode() const;
		virtual ISubtree* asISubtree();
		virtual ISubtree const* asISubtree() const;

		// double dispatch support
		virtual void dispatch(NodeAction&) const = 0;
		virtual void dispatch(NodeModification&) = 0;

		// "rtti"
		RTTI_BASE(INode);
	};



// Inner INode
	//==========================================================================
	//= ISubtree
	//==========================================================================
	class ISubtree : public INode
	{
		::rtl::OUString	m_sId;
		virtual INode* doGetChild(OUString const& name) const = 0;

		::rtl::OUString		m_sTemplateName;			/// path of the template for child instantiation

	public:
		ISubtree() : INode() {}
		ISubtree(OUString const& aName) : INode(aName) {}

		::rtl::OUString	getId() const			{ return m_sId; }
		void			setId(const ::rtl::OUString& _rId)	{ m_sId = _rId; }

//		virtual sal_Int32					size() const = 0;
//		virtual uno::Sequence< OUString >	elementNames() const = 0;

		INode* getChild(OUString const& name)				{ return doGetChild(name); }
		INode const* getChild(OUString const& name) const	{ return doGetChild(name); }
		
		virtual INode* addChild(std::auto_ptr<INode> node) =0;					// takes ownership
		virtual ::std::auto_ptr<INode> removeChild(OUString const& name) =0;	// releases ownership
		
		// Iteration support, stop if (action returns true)
		virtual void forEachChild(NodeAction& anAction) const = 0;
		virtual void forEachChild(NodeModification& anAction) = 0;
		
		ISubtree* asISubtree();
		ISubtree const* asISubtree() const;

		virtual sal_Int16 getLevel() = 0;
		virtual void setLevel(sal_Int16) = 0;

		bool				isSetNode() const { return m_sTemplateName.getLength() != 0; }
		::rtl::OUString		getChildTemplateName() const { return m_sTemplateName; }
		void				setChildTemplateName(const ::rtl::OUString& _rName) { m_sTemplateName = _rName; }

		// double dispatch support
		virtual void dispatch(NodeAction& anAction) const { anAction.handle(*this); }
		virtual void dispatch(NodeModification& anAction) { anAction.handle(*this); } 

		// "rtti"
		RTTI(ISubtree, INode);
	};
	


// Leaf Node
	//==========================================================================
	//= ValueNode
	//==========================================================================
	class ValueNode : public INode
	{
		uno::Type	m_aType;
		uno::Any	m_aValue;
		uno::Any	m_aDefaultValue;
	public:
		ValueNode(OUString const& aName)
			:INode(aName)
		{}
		ValueNode(OUString const& aName,uno::Type const& aType)
			:INode(aName), m_aType(aType) 
		{ check_init(); }
		ValueNode(OUString const& aName,uno::Any const& anAny)
			:INode(aName), m_aValue(anAny)
		{ init(); }
		ValueNode(OUString const& aName,uno::Any const& anAny,uno::Any const& aDefault)
			:INode(aName), m_aValue(anAny), m_aDefaultValue(aDefault) 
		{ init(); }
	
		bool isNull() const;
		bool hasDefault() const;
		bool isDefault() const;
		
		uno::Type getValueType() const;
		uno::Any getValue() const;
		uno::Any getDefault() const;
		
		void setValue(uno::Any aValue);
		void changeDefault(uno::Any aValue);
		void setDefault();
		
		virtual INode* clone() const;

		ValueNode* asValueNode();
		ValueNode const* asValueNode() const;
		// double dispatch support
		virtual void dispatch(NodeAction& anAction) const { anAction.handle(*this); }
		virtual void dispatch(NodeModification& anAction) { anAction.handle(*this); } 

		// "rtti"
		RTTI(ValueNode, INode);
	private:
		void init();
		void check_init();
	};
	
////////////////////////////////////////////////////////////////////////////////

	struct TreeChangeList;

	//==========================================================================
	//= ITreeProvider
	//==========================================================================
	class ITreeProvider : public ISynchronizedData
	{
	public:
		enum { ALL_LEVELS = -1  };

		virtual ISubtree const*	getSubtree( OUString const& aComponentName ) const = 0;
		virtual ISubtree *		requestSubtree( OUString const& aSubtreePath, sal_Int16 nMinLevels = ALL_LEVELS) = 0;

		/** returns the node for the given path. In opposite to <method>getSubtree</method>, this methods covers
			leafs, too.
		*/
		virtual const INode *	getNode(const OUString& _rPath) = 0;

		virtual void updateTree(TreeChangeList& aChanges) throw (starlang::WrappedTargetException, uno::RuntimeException) = 0;
	};

	// a ITreeProvider which can be ref-counted
	class ITreeNotifier;

	//==========================================================================
	//= ITreeListener
	//==========================================================================
	class ITreeListener
	{
	public:
		  virtual void disposing(ITreeNotifier* pNotifier) = 0;
		  virtual void changes(TreeChangeList const& , sal_Bool _bError) = 0;
		  virtual void changes(sal_Int32 _nNotificationId,const ::rtl::OUString& _rNotifyReason) = 0;
	};

	//==========================================================================
	//= ITreeNotifier
	//==========================================================================
	class ITreeNotifier
	{
	public:
		virtual void setListener(ITreeListener* _pListener) = 0;
	};

	//==========================================================================
	//= ITemplateProvider
	//==========================================================================
	class ITemplateProvider
	{
	public:
		virtual ::std::auto_ptr<INode> createInstance(const ::rtl::OUString& _rTemplateName) throw (uno::Exception) = 0;
	};

	//==========================================================================
	//= ref-counted providers
	//==========================================================================
	class IRefCountedTreeProvider : public ITreeProvider, public IInterface
	{
	};

	class IRefCountedTemplateProvider : public ITemplateProvider, public IInterface
	{
	};

////////////////////////////////////////////////////////////////////////////////

	class Change;
	class SubtreeChange;
	class ValueChange;
	class AddNode;
	class RemoveNode;

	//==========================================================================
	//= ChangeTreeAction
	//==========================================================================
	struct ChangeTreeAction
	{
		virtual void handle(ValueChange const& aValueNode) = 0;
		virtual void handle(AddNode const& aAddNode) = 0;
		virtual void handle(RemoveNode const& aRemoveNode) = 0;
		virtual void handle(SubtreeChange const& aSubtree) = 0;

		void applyToChange(Change const& aChange);
		void applyToChildren(SubtreeChange const& aSubtree);
	};

	struct ChangeTreeModification
	{
		virtual void handle(ValueChange& aValueNode) = 0;
		virtual void handle(AddNode& aAddNode) = 0;
		virtual void handle(RemoveNode& aRemoveNode) = 0;
		virtual void handle(SubtreeChange& aSubtree) = 0;

		void applyToChange(Change& aChange);
		void applyToChildren(SubtreeChange& aSubtree);
	};

	//==========================================================================
	//= Change
	//==========================================================================
	class Change
	{
	protected:
		OUString m_aName;
	public:
		Change(OUString const& _rName) : m_aName(_rName){}
		virtual ~Change() {}

		OUString getNodeName() const { return m_aName; } 
		void setNodeName(const OUString &aName) {m_aName = aName;}
		
		Change* getSubChange(OUString const& _rName) { return doGetChild(_rName); }
		Change const* getSubChange(OUString const& _rName) const { return doGetChild(_rName); }

		virtual void dispatch(ChangeTreeAction& anAction) const = 0;
		virtual void dispatch(ChangeTreeModification& anAction) = 0;

		// some kind of simple rtti
		RTTI_BASE(Change);

	private:
		virtual Change* doGetChild(OUString const& ) const { return 0; }
	};

	//==========================================================================
	//= ValueChange
	//==========================================================================
	class ValueChange : public Change
	{
	public:
		typedef uno::Any Any;
		struct SetToDefault {};
		enum Mode { wasDefault, changeValue, setToDefault, changeDefault }; 

	private:
		Any		m_aValue;
		Any		m_aOldValue;
		Mode	m_eMode;
	public:
		ValueChange(OUString const& _rName,Any aNewValue, Mode aMode = changeValue, Any aOldValue = Any());
		ValueChange(Any aNewValue, ValueNode const& aOldValue);
		ValueChange(SetToDefault,  ValueNode const& aOldValue);

		uno::Any	getNewValue() const { return m_aValue; }
		uno::Any	getOldValue() const { return m_aOldValue; }

		void		setNewValue(const uno::Any& _rNewVal) { m_aValue = _rNewVal; }
		void		setNewValue(const uno::Any& _rNewVal, Mode aMode) 
		{ setNewValue(_rNewVal); m_eMode = aMode;}

		Mode getMode() const { return m_eMode; }

		::rtl::OUString getModeAsString() const;
		void setModeAsString(const ::rtl::OUString& _rMode);

		void applyTo(ValueNode& aNode);

		virtual void dispatch(ChangeTreeAction& anAction) const { anAction.handle(*this); }
		virtual void dispatch(ChangeTreeModification& anAction) { anAction.handle(*this); }
		
		// "rtti"
		RTTI(ValueChange, Change);
	};

	//==========================================================================
	//= AddNode
	//==========================================================================
	class AddNode : public Change
	{
		INode*							m_pNewNode;
		std::auto_ptr<INode>			m_aOwnNewNode;

		// don't create CopyCTor automatically
		AddNode(AddNode const&);
		void operator=(AddNode const&);

	public:
		AddNode(std::auto_ptr<INode> aNewNode_,OUString const& _rName);
		~AddNode();

		/** returns the node this change represents; The Node object is owned by this change until
			<method>releaseAddedNode</method> is called.<BR>
			After ownership is lost this method returns NULL.
		*/
		INode*					getAddedNode()	const { return m_aOwnNewNode.get(); }

		/** returns the node this change represents, even if this node does not own the new Node object any more. 
			This is somewhat dangerous if the node referenced by this object dies before the object itself does.<BR>
			In this case all calls to this method will return nonsense. This case can be detected by testing 
			whether <method>getAddedNode</method> returns NULL.
		*/
		INode*					getAddedNode_unsafe()	const { return m_pNewNode; }
		/** returns the node the change represents, and releases ownership of it. This means that 
			afterwards <method>getAddedNode</method> will return NULL. This change object keeps a reference 
			to the node though which can be retrieved using <method>getAddedNode_unsafe</method>.
		*/
		std::auto_ptr<INode>	releaseAddedNode() { return m_aOwnNewNode; }

		virtual void dispatch(ChangeTreeAction& anAction) const { anAction.handle(*this); }
		virtual void dispatch(ChangeTreeModification& anAction) { anAction.handle(*this); }

		// "rtti"
		RTTI(AddNode, Change);
	};

	//==========================================================================
	//= RemoveNode
	//==========================================================================
	class RemoveNode : public Change
	{
	protected:
		uno::Any		m_aOldNodeValue;
			// does make sense only if the change describes a value node
	public:
		RemoveNode(OUString const& _rName) : Change(_rName) {};
		virtual void dispatch(ChangeTreeAction& anAction) const { anAction.handle(*this); }
		virtual void dispatch(ChangeTreeModification& anAction) { anAction.handle(*this); }

		void		setOldNodeValue(const uno::Any& _r) { m_aOldNodeValue = _r; }
		uno::Any	getOldNodeValue() const { return m_aOldNodeValue; }

		// "rtti"
		RTTI(RemoveNode, Change);
	};

	//==========================================================================
	//= SubtreeChange
	//==========================================================================
	class SubtreeChange : public Change
	{	
	protected:
		typedef ::std::map< ::rtl::OUString,Change* > Children;
		Children m_mChanges;
		::rtl::OUString		m_sTemplateName;			/// path of the template for child instantiation

		// don't create CopyCTor automatically
		SubtreeChange(SubtreeChange&);
		void operator=(SubtreeChange&);

	public:
		class ChildIterator;
		ChildIterator begin() const throw();
		ChildIterator end() const throw();

	public:
		SubtreeChange(OUString const& _rName);
		~SubtreeChange();

		bool			isSetNodeChange() const { return m_sTemplateName.getLength() != 0; }
		OUString		getChildTemplateName() const { return m_sTemplateName; }
		void			setChildTemplateName(const OUString& _rName) { m_sTemplateName = _rName; }

		sal_Int32					size() const { return m_mChanges.size(); }
		uno::Sequence< OUString >	elementNames() const;

		void					addChange(std::auto_ptr<Change> aChange);
		::std::auto_ptr<Change>	removeChange(OUString const& _rName);

		Change* getChange(OUString const& _rName);
		Change const* getChange(OUString const& _rName) const;

		virtual void dispatch(ChangeTreeAction& _anAction) const;
		virtual void dispatch(ChangeTreeModification& _anAction);

		void forEachChange(ChangeTreeAction& _anAction) const;
		void forEachChange(ChangeTreeModification& _anAction);

		// "rtti"
		RTTI(SubtreeChange, Change);

	private:
		virtual Change* doGetChild(OUString const& _rName) const;
	};

	/** iterates through all children of a <type>SubtreeChange</type>. Every non-const action on the object
		which is beeing iterated invalidates the iterator.
		<BR>
		beware of the lifetime of the tree change object : it has to live as long as the iterator does (at least) !!
	*/
	class SubtreeChange::ChildIterator
	{
	protected:
		const SubtreeChange*		m_pTree;
		uno::Sequence< OUString >	m_aNames;
		sal_Int32					m_nPos;

		friend class SubtreeChange;
		struct EndPos { };
		ChildIterator(const SubtreeChange* _pTree, struct EndPos);

		inline sal_Bool isValid() const { return m_nPos >= 0 && m_nPos < m_aNames.getLength(); }

	public:
		ChildIterator(const SubtreeChange* _pTree);

		const Change* operator*() const;
		const Change* operator->() const;

		ChildIterator& operator++();
		ChildIterator operator++(int) { ChildIterator ret(*this); ++*this; return ret; }

		ChildIterator& operator--();
		ChildIterator operator--(int) { ChildIterator ret(*this); --*this; return ret; }

		friend bool operator==(ChildIterator const& lhs, ChildIterator const& rhs);
		friend bool operator!=(ChildIterator const& lhs, ChildIterator const& rhs) { return !(lhs == rhs); }
	};

	//==========================================================================
	//= SubtreeChangeReferrer
	//==========================================================================
	/** a specialized SubtreeChange, which, upon desctruction, does not delete the changes
		it holds
		<BR>
		This implies that when using this class, you have to beware of the lifetime of the involved objects
	*/
	class SubtreeChangeReferrer : public SubtreeChange
	{
		// no explicit construction
		SubtreeChangeReferrer() : SubtreeChange(::rtl::OUString()) { }

	public:
		SubtreeChangeReferrer(const SubtreeChange& _rSource);
		~SubtreeChangeReferrer();

		RTTI(SubtreeChangeReferrer, SubtreeChange);
	};

////////////////////////////////////////////////////////////////////////////////

	//==========================================================================
	//= inlines
	//==========================================================================
	inline void NodeAction::applyToNode(INode const& aNode)
	{ aNode.dispatch(*this); }
	inline void NodeAction::applyToChildren(ISubtree const& aSubtree)
	{ aSubtree.forEachChild(*this); }

	inline void NodeModification::applyToNode(INode& aNode)
	{ aNode.dispatch(*this); }
	inline void NodeModification::applyToChildren(ISubtree& aSubtree)
	{ aSubtree.forEachChild(*this); }

	inline void ChangeTreeAction::applyToChange(Change const& aChange)
	{ aChange.dispatch(*this); }
	inline void ChangeTreeAction::applyToChildren(SubtreeChange const& aSubtree)
	{ aSubtree.forEachChange(*this); }

	inline void ChangeTreeModification::applyToChange(Change& aChange)
	{ aChange.dispatch(*this); }
	inline void ChangeTreeModification::applyToChildren(SubtreeChange& aSubtree)
	{ aSubtree.forEachChange(*this); }

////////////////////////////////////////////////////////////////////////////////
	
	//==========================================================================
	//= TreeChangeList
	//==========================================================================
	struct TreeChangeList
	{
		OUString pathToRoot;					 // path to the root of the whole to-be-updated subtree
		SubtreeChange root;						 // changes made within this sub tree
		TreeChangeList(): root(::rtl::OUString()){}

		/** ctor
		@param		_rPathToRoot		path to the root of the whole to-be-updated subtree
		@param		_rLocalName			relative path within the to-be-updated subtree
		*/
		TreeChangeList( const OUString& _rPathToRoot, const OUString& _rLocalName)
				:pathToRoot(_rPathToRoot), root(_rLocalName){}

		// OUString getPath() const {return m_sPathToRoot;}
		// const SubtreeChange* getRoot() const {return m_pRoot;}
		// SubtreeChange* getRoot() {return m_pRoot;}
	};

////////////////////////////////////////////////////////////////////////////////
	
	
} // namespace configmgr

#endif

