/*************************************************************************
 *
 *  $RCSfile: confaccess.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_ACCESS_HXX_
#define CONFIGMGR_API_ACCESS_HXX_

#ifndef CONFIGMGR_API_BASEACCESS_HXX_
#include "confbaseaccess.hxx"
#endif

#ifndef CONFIGMGR_API_BASEELEMENT_HXX_
#include "confbaseelement.hxx"
#endif

#ifndef CONFIGMGR_API_CHANGESNOTIFIER_HXX_
#include "changenotifier.hxx"
#endif

namespace configmgr
{
	class ConfigurationProvider;

class HierarchyReadAccess
: public HierarchyAccessImpl
, public HierarchyElementImpl
, public ChangesNotifierImpl
{
protected:
	HierarchyReadAccess(BroadcastHelper& rBHelper_, IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, OUString aBasePath);
	HierarchyReadAccess(BroadcastHelper& rBHelper_, IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath);
	HierarchyReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode);
	HierarchyReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath);
	HierarchyReadAccess(BroadcastHelper& rBHelper_, HierarchyReadAccess const& aAncestorElement, ParentElement);
public:
	// virtual ctors
	static HierarchyReadAccess* create(IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, OUString aBasePath = OUString(), bool withDefaults=true);
	static HierarchyReadAccess* create(IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath = OUString(), bool withDefaults=true);
	static HierarchyReadAccess* create(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode, bool withDefaults=true);
	static HierarchyReadAccess* create(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath, bool withDefaults=true);
	static HierarchyReadAccess* create(HierarchyReadAccess const& aAncestorElement, ParentElement, bool withDefaults=true);

	/// late ctor helper - not yet final here
	virtual void Init() = 0;

public:
	// XInterface implementation - present but still abstract
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException) = 0;

	// XTypeProvider implementation - present but still abstract
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException) = 0;

	// XHierarchicalName desambuigation
	virtual OUString SAL_CALL getHierarchicalName() 
		throw(uno::RuntimeException)
	{ return HierarchyElementImpl::getHierarchicalName(); }

	virtual OUString SAL_CALL composeHierarchicalName( const OUString& aRelativeName ) 
		throw(css::lang::IllegalArgumentException, css::lang::NoSupportException, uno::RuntimeException)
	{ return HierarchyElementImpl::composeHierarchicalName(aRelativeName); }

protected:
    virtual void SAL_CALL disposing();
		
	virtual ISynchronizedData const * getTreeSync() const
	{ return m_pSync; }

protected:
// HierarchyAccessImpl & HierarchyElementImpl implementation
	virtual HierarchyAccessImpl* getSubtreeAccess(TreeRef aSubtree, TreePath const& aPath);
	virtual HierarchyAccessImpl* getParentAccess();

	virtual INode const& getThisNode();
	virtual bool normalizeNestedPath(ConfigurationName& aRelativeName);
private:
	ISynchronizedData const * m_pSync;
};


} // namespace configmgr


#endif // CONFIGMGR_CONFACCESS_HXX_



