/*************************************************************************
 *
 *  $RCSfile: confbaseaccess.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_BASEACCESS_HXX_
#define CONFIGMGR_API_BASEACCESS_HXX_

#ifndef _COM_SUN_STAR_CONTAINER_XNAMEACCESS_HPP_
#include <com/sun/star/container/XNameAccess.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XHIERARCHICALNAMEACCESS_HPP_
#include <com/sun/star/container/XHierarchicalNameAccess.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XEXACTNAME_HPP_
#include <com/sun/star/beans/XExactName.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XLOCALIZABLE_HPP_
#include <com/sun/star/lang/XLocalizable.hpp>
#endif

#ifndef _CPPUHELPER_IMPLBASE4_HXX_
#include <cppuhelper/implbase4.hxx>
#endif
#ifndef CONFIGMGR_APITYPES_HXX_
#include "confapitypes.hxx"
#endif

namespace configmgr
{

	class ConfigurationName;
	class ISynchronizedData;

class HierarchyAccessImpl
: public ::cppu::ImplHelper4
			< css::container::XNameAccess
			, css::container::XHierarchicalNameAccess
			, css::beans::XExactName
			, css::lang::XLocalizable
			>
{
protected:
	TreeRef				m_aSubtree;
	::osl::Mutex&		m_rMutex;
	css::lang::Locale	m_aCurrentLocale;

public:
// Constructors
	explicit HierarchyAccessImpl(::osl::Mutex& _rMutex, TreeRef aTree);

// Interface methods

	// XElementAccess, base class of XNameAccess (and XHierarchicalNameAccess ? )
    virtual uno::Type SAL_CALL 
		getElementType(  ) 
			throw(uno::RuntimeException);

    virtual sal_Bool SAL_CALL 
		hasElements(  ) 
			throw(uno::RuntimeException);

	// XNameAccess
    virtual uno::Any SAL_CALL 
		getByName( const OUString& aName ) 
			throw(css::container::NoSuchElementException, css::lang::WrappedTargetException, 
					 uno::RuntimeException);
 
	virtual uno::Sequence< OUString > SAL_CALL 
		getElementNames(  ) 
			throw( uno::RuntimeException);

    virtual sal_Bool SAL_CALL 
		hasByName( const OUString& aName ) 
			throw(uno::RuntimeException);

	// XHierarchicalNameAccess
    virtual uno::Any SAL_CALL 
		getByHierarchicalName( const OUString& aName ) 
			throw(css::container::NoSuchElementException, uno::RuntimeException);

    virtual sal_Bool SAL_CALL 
		hasByHierarchicalName( const OUString& aName ) 
			throw(uno::RuntimeException);

	// XExactName
	virtual ::rtl::OUString SAL_CALL getExactName( const ::rtl::OUString& _rApproximateName ) throw(::com::sun::star::uno::RuntimeException);

	// XLocalizable
    virtual void SAL_CALL setLocale( const ::com::sun::star::lang::Locale& eLocale ) throw(::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::lang::Locale SAL_CALL getLocale(  ) throw(::com::sun::star::uno::RuntimeException);

protected: // protected accessors
	TreeRef getThisTree() const { return m_aSubtree; }

	uno::Any getChildAccess(TreeRef aChildTree) throw(uno::RuntimeException);
	uno::Any getChildAccess(TreeRef aChildTree, TreePath const& aPath) throw(uno::RuntimeException);
	
	//	void init(TreeRef aTree);
	void checkInitialized();

	uno::Any makeElement(NodeRef pFound, TreePath const& aPath) throw(css::container::NoSuchElementException, uno::RuntimeException);
	virtual NodeRef findDescendant(OUString const& aRelativePath, TreePath& aPath);

	/** used in every method which is accessible from outside (which means it's part of an interface)
		<BR>
		Will usually be implemented so that it throws a <type scope="com.sun.star.lang">DisposedException</type>
		if the object is already dead.
	*/
	virtual void checkAlive() throw (uno::RuntimeException) = 0;

	/// is used for synchronizing the access to the tree 
	virtual ISynchronizedData const* getTreeSync() const = 0;
private:
	// the object returned by getSubtreeAccess has to be acquired once !!!!
	virtual HierarchyAccessImpl* getSubtreeAccess(TreeRef aSubtree, TreePath const& aPath) = 0;
	virtual bool normalizeNestedPath(ConfigurationName& aRelativeName) = 0;
};

}
#endif // CONFIGMGR_API_BASEACCESS_HXX_


