/*************************************************************************
 *
 *  $RCSfile: confbaseelement.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_BASEELEMENT_HXX_
#define CONFIGMGR_API_BASEELEMENT_HXX_

#include <com/sun/star/container/XChild.hpp>
#include <com/sun/star/container/XNamed.hpp>
#include <com/sun/star/container/XHierarchicalName.hpp>

#include <cppuhelper/implbase3.hxx>

#include "confapitypes.hxx"
#include "confname.hxx"

namespace configmgr
{
	class HierarchyAccessImpl;

	class HierarchyElementImpl
	: public ::cppu::ImplHelper3
				< css::container::XChild
				, css::container::XNamed
				, css::container::XHierarchicalName
				>
	{
	protected:
		::osl::Mutex&		m_rMutex;

	public:
		struct ParentElement {};
	// Constructors
		HierarchyElementImpl(::osl::Mutex& _rMutex);
		HierarchyElementImpl(::osl::Mutex& _rMutex, OUString m_aBasePath);
		HierarchyElementImpl(::osl::Mutex& _rMutex, TreeRef pParent, OUString m_aBasePath = OUString());
		HierarchyElementImpl(::osl::Mutex& _rMutex, TreePath const& aPath, OUString m_aBasePath = OUString());
		HierarchyElementImpl(::osl::Mutex& _rMutex, HierarchyElementImpl const& aElement, TreeRef pParentNode);
		HierarchyElementImpl(::osl::Mutex& _rMutex, HierarchyElementImpl const& aElement, TreePath const& aPath);
		HierarchyElementImpl(::osl::Mutex& _rMutex, HierarchyElementImpl const& aElement, ParentElement);

	// Interface methods
		// XChild
		virtual uno::Reference< uno::XInterface > SAL_CALL 
			getParent(  ) 
				throw(css::uno::RuntimeException);

		virtual void SAL_CALL // currently not supported
			setParent( const uno::Reference< uno::XInterface >& Parent ) 
				throw(css::lang::NoSupportException, css::uno::RuntimeException);

		// XNamed
		virtual OUString SAL_CALL 
			getName(  ) 
				throw(uno::RuntimeException);

		virtual void SAL_CALL  // not supported (!)
			setName( const OUString& aName ) 
				throw(uno::RuntimeException);

		// XHierarchicalName
		virtual OUString SAL_CALL 
			getHierarchicalName(  ) 
				throw(uno::RuntimeException);

		virtual OUString SAL_CALL 
			composeHierarchicalName( const OUString& aRelativeName ) 
				throw(css::lang::IllegalArgumentException, css::lang::NoSupportException, 
						uno::RuntimeException);

		/// a static version of getHierarchicalName - some other may need it
		static ConfigurationName getHierarchicalName(const ConfigurationName& _rVeryBase, const TreePath& _rPath);

	protected:
		// Internal protected accessors
		void initBasePath(ConfigurationName const& aBasePath);
		ConfigurationName const& getBasePath() const { return m_aBasePath; }
		TreePath const& getNodePath() const { return m_aAncestors; }
		TreeRef getParentNode() const { return m_aAncestors.empty() ? 0 : m_aAncestors.back(); }
		// access to path components
		typedef TreePath::const_iterator PathIterator;
		PathIterator beginPath() const { return m_aAncestors.begin(); }
		PathIterator endPath()   const { return m_aAncestors.end(); }

		// Internal implementation interface
		virtual INode const& getThisNode() = 0;
		// the object returned by getParentAccess (if any) has to be acquired once
		virtual HierarchyAccessImpl* getParentAccess() = 0;

		/** used in every method which is accessible from outside (which means it's part of an interface)
			<BR>
			Will usually be implemented so that it throws a <type scope="com.sun.star.lang">DisposedException</type>
			if the object is already dead.
		*/
		virtual void checkAlive() throw (uno::RuntimeException) = 0;

		/// notify the node that it has been moved within it's tree. Use with caution.
		void movedNode(const TreePath& _rNewAncestors) { m_aAncestors = _rNewAncestors; }

	private:

		ConfigurationName m_aBasePath;
		TreePath m_aAncestors;
	};

}
#endif // CONFIGMGR_API_BASEELEMENT_HXX_


