/*************************************************************************
 *
 *  $RCSfile: confgroupaccess.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_GROUPACCESS_HXX_
#define CONFIGMGR_API_GROUPACCESS_HXX_

#ifndef _CONFIGMGR_API_UPDATEACCESS_HXX_
#include "confupdateaccess.hxx"
#endif
#ifndef _CONFIGMGR_API_SETACCESS_HXX_
#include "confsetaccess.hxx"
#endif
#ifndef CONFIGMGR_API_PROPERTIES_HXX_
#include "nodepropset.hxx"
#endif

//........................................................................
namespace configmgr
{
//........................................................................
	namespace script = ::com::sun::star::script;
//========================================================================
//= HierarchyUpdateElementGroup
//========================================================================
/** a class which represents a group node which is non-root element of an update sub tree.
	<BR>
	still abstract and to be concretized with ref-counting and service info.
*/
class HierarchyUpdateElementGroup
			:public HierarchyUpdateElement
			,public NodePropertySet
{
public:
	typedef NodePropertySet::BroadcastHelper BroadcastHelper;
public:
	HierarchyUpdateElementGroup(
		BroadcastHelper& _rBHelper,
		HierarchyUpdateRoot* _pUpdateRoot,
		SubtreeChange* _pModifications,
		const TreePath& _rRelativeLocation,	
		bool bMayHaveDefaults =true
	);

	// XInterface
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);

	// XTypeProvider
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);

protected:
	virtual void SAL_CALL disposing();

	// OSetNode overridables
	virtual ISynchronizedData const*getTreeSync() const { return HierarchyUpdateElement::getTreeSync(); }
	virtual ISynchronizedData *		getUpdateSync()		{ return HierarchyUpdateElement::getUpdateSync(); }
	virtual TreeRef					getThisTree() const	{ return HierarchyUpdateElement::getThisTree(); }
	virtual bool					loadDefaults()		{ return HierarchyUpdateElement::loadDefaults(); }

	uno::Reference<script::XTypeConverter> getTypeConverter() const
	{ return HierarchyUpdateElement::getTypeConverter(); }
	
	virtual void executeLocalChanges(ValueChanges& rChanges)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException);

	uno::Any getChildAccess(TreeRef aChildTree) throw (uno::RuntimeException) 
	{ return HierarchyUpdateElement::getChildAccess(aChildTree); }
};

//==========================================================================
//= OFloatableGroup
//==========================================================================
/** a <type>OFloatableNode</type> that is accessible as property set, as it is a group (as opposed to SET) node.
*/
class OFloatableGroup
		:public OFloatableNode
		,public NodePropertySet
{
public:
	typedef NodePropertySet::BroadcastHelper BroadcastHelper;
public:
	/** ctor
		@param		_rBHelper					notification helper, contains a mutex for thread safety
		@param		_rTemplateName			name of the template which was used to create the node represented
												by the object
		@param		_rContainerPath				the path to the container which created this element,
												relative to _pUpdateRoot, including the container's node itself,
												_not_ including the child template instance (of course, as the node is not part
												of the real tree, yet)
		@param		_pTemplateInstance		the (floating) node which is represented by the object
		@param		_pUpdateRoot				the root to which the will belong (after it is not floating anymore)
	*/
	OFloatableGroup(
		BroadcastHelper& _rBHelper,
		const ::rtl::OUString& _rTemplateName,
		const TreePath& _rContainerPath,
		::std::auto_ptr<ISubtree> _pTemplateInstance,
		HierarchyUpdateRoot* _pUpdateRoot
	);

	// XInterface
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);

	// XTypeProvider
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);

protected:
	virtual void SAL_CALL disposing();

	// NodePropertySet overridables
	virtual ISynchronizedData const*getTreeSync() const { return OFloatableNode::getTreeSync(); }
	virtual ISynchronizedData *		getUpdateSync()		{ return OFloatableNode::getUpdateSync(); }
	virtual TreeRef					getThisTree() const	{ return OFloatableNode::getThisTree(); }
	virtual bool					loadDefaults()		{ return OFloatableNode::loadDefaults(); }

	uno::Reference<script::XTypeConverter> getTypeConverter() const
	{ return OFloatableNode::getTypeConverter(); }

	virtual void executeLocalChanges(ValueChanges& rChanges)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException);

	uno::Any getChildAccess(TreeRef aChildTree) throw (uno::RuntimeException) 
	{ return OFloatableNode::getChildAccess(aChildTree); }
};

//========================================================================
//= HierarchyUpdateRootGroup
//========================================================================
/** a class which represents a root node of an update tree, which is a group node additionally.
	<BR>
	still abstract and to be concretized with ref-counting and service info.
*/
class HierarchyUpdateRootGroup
			:public HierarchyUpdateRoot
			,public NodePropertySet
{
public:
	typedef NodePropertySet::BroadcastHelper BroadcastHelper;
public:
	/** ctor
		@param		_rBHelper					a notification listener container which also contains a mutex for thread safety
		@param		_rName						the location of the parent of the object
		@param		_pTree						the tree represented by the object
		@param		_rContainerTemplateName		name of the template which should be used for creating new children
												of the node.
		@param		_rTemplateFactory			factory which is able to provide the templates described by _rContainerTemplateName
		@param		_rUpdater					the object which implements the tree updates
	*/
	HierarchyUpdateRootGroup(
		BroadcastHelper& _rbHelper, 
		TreeRef _pTree,	const ConfigurationName& _rName,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		IConfigBroadcaster& rEventSource_,
		const uno::Reference< script::XTypeConverter >& _rxConverter
	);

	// XInterface
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);

	// XTypeProvider
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);

protected:
	virtual void SAL_CALL disposing();

	// NodePropertySet overridables
	virtual ISynchronizedData const*getTreeSync() const	{ return HierarchyUpdateRoot::getTreeSync(); }
	virtual ISynchronizedData *		getUpdateSync()		{ return HierarchyUpdateRoot::getUpdateSync(); }
	virtual TreeRef					getThisTree() const	{ return HierarchyUpdateRoot::getThisTree(); }
	virtual bool					loadDefaults()		{ return HierarchyUpdateRoot::loadDefaults(); }
	virtual void executeLocalChanges(ValueChanges& rChanges)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException);

	uno::Reference<script::XTypeConverter> getTypeConverter() const
	{ return HierarchyUpdateRoot::getTypeConverter(); }

	uno::Any getChildAccess(TreeRef aChildTree) throw (uno::RuntimeException) 
	{ return HierarchyUpdateRoot::getChildAccess(aChildTree); }
};

//........................................................................
} // namespace configmgr
//........................................................................

#endif // CONFIGMGR_API_GROUPACCESS_HXX_


