/*************************************************************************
 *
 *  $RCSfile: configsession.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_SESSION_CONFIGSESSION_HXX_
#define _CONFIGMGR_SESSION_CONFIGSESSION_HXX_

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif
#ifndef _CONFIGMGR_COMMONTYPES_HXX_
#include "commontypes.hxx"
#endif
#ifndef _CONFIGMGR_SERVER_VERSION_HXX_
#include "server_version.hxx"
#endif

#ifndef _COM_SUN_STAR_UNO_SEQUENCE_HXX_
#include <com/sun/star/uno/Sequence.hxx>
#endif
#ifndef _COM_SUN_STAR_XML_SAX_XDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XDocumentHandler.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYVALUE_HPP_
#include <com/sun/star/beans/PropertyValue.hpp>
#endif

//..........................................................................
namespace configmgr
{
//..........................................................................

//==========================================================================
//= StatusInfo
//==========================================================================
/** encapsulates the result of a request to the registry server
	@deprecated
*/
struct StatusInfo
{
	sal_Int32			nCode;			/// the error code, at the moment 0 indicates success, != 0 indicates an error
	::rtl::OUString		sMessage;		/// the error message, semantic depends on the specific registry server you're using
};

//==========================================================================
//= IRequestCallback
//==========================================================================
/** interface for transfering the results of asynchronous operations on a config session.
	<BR>
	As most of the methods of <type>IConfigSession</type> are potentially one-way , result
	callbacks are an essential part of the request concept.
	<BR>
	Calling order : If the request could not be forwarded to the server usually fullfilling it,
	<method>failed</method> is called and the object is released afterwards.
	If the request could be forwarded and the server acknowledges the it's queueing, <method>acknowledged</method>
	is invoked. After (successfull or not) finishing the request (as indicated by the server) <method>done</method>
	is called.
	<BR>
	Derived classes may extend this calling order by inserting additonal notifications, but never change
	the relative order of this base class' method calls.
	<BR>
	Usually the caller of <type>IConfigSession</type> methods provides an <type>IRequestCallback</type>
	interface which can handle the results. If so, this object will be acquired immediately (of course)
	and released as soon as the result notification (no matter if it is "success" or "failed") was sent.
	<BR>
	@todo:
	TODO : the current spec says that in case of success, no status will be returned.
	So perhaps our callback interface needs to differ between "done successfully"
	and "done, but had an error"
	after using this interface for a while, I think that it is not very plausible at the moment ...
	"failed" indicates an error, "done" may indicate an error or an success ... that's somewhat unlucky ...
*/
class IRequestCallback : public IInterface
{
public:
	/**	acknowledges the successfull (server-side) queueing of the request. This says nothing about the
		later success of the transaction, it only states that the server accepted the request in a general
		manner.
		@param		_nTransId		the server side transaction id for the request. May be refered to later,
									for instance in listener calls.
	*/
	virtual		void acknowledged(sal_Int32 _nTransId) = 0;

	/** indicates that the request could not be queued. Depending on the object implementing an
		<type>IConfigSession</type> interface, the meaning of this method will be quite different.
		@param			_nErrorCode		the session type specific error code. It may evidence an error
										on the connection to the remote server or an error while working
										on a local configuration file.<BR>
										If this methods gets invoked, no further call to <method>done</method>
										will occur.
	*/
	virtual		void failed(sal_Int32 _nErrorCode) = 0;

	/** called when the request is done (no matter if successfull or not, this information is provided by
		the StatusInfo given).
		@param			_rStatus		status after the request
	*/
	virtual		void done(const StatusInfo& _rStatus) = 0;
};


//==========================================================================
//= IGenericCalback
//==========================================================================
/** a session callback for transactions which return a list of properties not beeing interpreted by the session.
	<BR>
	Simple types are string, integer etc. Especially node data can't be transported through this callback.
*/
class IPropertyRequestCallback : public IRequestCallback
{
public:
	/** forwards the parameters extracted from the response body.
	*/
	void gotParameters(const ::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue >& _rParams);
};

//==========================================================================
//= IDataRequestCallback
//==========================================================================
/** interface for transfering results of asynchronous operations supplying
	node-specific data. May be used for propagating data the user requested
	with an response (e.g. openNode, getNode) or in case of an error while
	executing a request working on node data (e.g. if an updateNode fails,
	a config server may return the current state of the node in request).
	<BR>
	Calling order : the <method>getDataReader</method> gets called between <method>acknowledged</method>
	and <method>done</method>, if ever (if the queueing of and request fails, as indicated by
	<method>failed</method>, the data reader is never needed and thus the method will never be called).
*/
class IDataRequestCallback : public IRequestCallback
{
public:
	/**	get the document handler which will receive the node specific data the
		request produced.
		<BR>
		If the returned interface is NULL, the results of the request will be quashed.
	*/
	virtual		::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XDocumentHandler >
					getDataReader() = 0;
};

//==========================================================================
//= IOpenObjectCallback
//==========================================================================
/** interface for callbacks for operations which open an object which get's a server-side id.
	<BR>
	Calling order : <method>getNodeId</method> gets called between <method>acknowledged</method>
	and <method>getDataReader</method>, if ever.
	
*/
class IOpenObjectCallback : public IDataRequestCallback
{
public:
	/** forwardes the id of the object just opened to the callback object. All later operations
		will probably need this id.
	*/
	virtual	void	gotObjectId(const ::rtl::OUString& _rId) = 0;
};

//==========================================================================
//= IDOMNodeDataProvider
//==========================================================================
/** interface for transfering node data from the configuration proxy to the registry
	client.
*/
class IDOMNodeDataProvider
{
public:
	/** write the data of a DOM node to a given document handler.
		@param		_rHandler		the document handler to write to.
	*/
	virtual void	writeNodeData(
		const ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XDocumentHandler >& _rHandler) = 0;
};

//==========================================================================
//= ISessionListener
//==========================================================================
/** a listener on configuration nodes. able to receive all changes in one or more
	specific registry sub trees.
*/
class ISessionListener : public IInterface
{
public:
	/** called whenever another session modified a node which the listener is registered for
		@param		_rNodePath		the path of the node which was updated. If you're interested in the new
									node values, you should call getNode on the session.
	*/
	virtual void	nodeUpdated(const ::rtl::OUString& _rNodePath) = 0;

	/** called whenever another session deleted a new node (assumed the the listener is registered for an ancestor
		of this new node)
		@param		_rNodePath		the path of the node which was deleted.
	*/
	virtual void	nodeDeleted(const ::rtl::OUString& _rNodePath) = 0;

	/** called whenever another session added a new node (assumed the the listener is registered for an ancestor
		of this new node)
		@param		_rNodePath		the path of the node which was added. If you're interested in the new
									node values, you should call getNode on the session.
	*/
	virtual void	nodeAdded(const ::rtl::OUString& _rNodePath) = 0;
};

//==========================================================================
//= IConfigSession
//==========================================================================
/** abstract interface for configuration access. provides methods for working
	on nodes and listener administration
*/
class IConfigSession
{
public:
	/** possible error reaons
	*/
	enum TSessionError
	{
		E_None,					/// all okay ....
		E_Invalid,				/// the object is invalid in any way
		E_ConnectionError,		/// there was an error on the underlying connection
		E_TimedOut,				/// the underlying connection did not response in time to a vital request
		E_NotConnected,			/// there is no underlying connection
		E_NotOpen,				/// the session is not opened
		E_WrongFormat,			/// the (input or output) XML stream has a wrong format

		E_Unknown				/// an unspecified error occured
	};

	enum { COMPLETE_SUBTREE = 0  };

public:

	virtual ~IConfigSession() { }

// error handling
	/** return the last error occured
		@seealso	getConnectionError
	*/
	virtual TSessionError	getError() const = 0;

	/** return the last error on the underlying connection.
		@return		error code. The meaning of this code depends on the type of connection you're using
					(e.g. whether it is a remote server or a local file)
		@seealso	getError
	*/
	virtual sal_Int32		getConnectionError() const = 0;

// miscellaneous
	/** get the master timeout for the session. This is a session aspect usually collected while
		bootstrapping. All clients waiting for the (maybe asynchronous) callbacks the session does
		should use this timeout.
	*/
	virtual TimeValue*		getMasterTimeout() const { return NULL; }

// listener administration methods
	/** sets a new notification listener for the session. You may have only one listener per session.
		<BR>
		For the format of the notification messages, please refer to the registry server documentation.
		@param			_pListener		the new listener for all session-wide notifications. The new
										listener will be aquired once and the previous one - if any - will
										be released.<BR>
										May be NULL, in this case no notifications will leave the session
										object.
	*/
	virtual	void		setListener(const ::vos::ORef< ISessionListener >& _pListener) = 0;

	/** start listening on a series of nodes. If currently no listener is set, nothing happens.<BR>
		All refered nodes have to be loaded into the server-side cache, i.e. they (or their ancestors)
		have to be opened by using <code>openNode</code>.
		@param			_rNodes			sequence of node paths
		@param			_rHandler		callback for status notifications. May be NULL.
		@seealso		stopListening
		@seealso		setListener
		@seealso		openNode
	*/
	virtual void		startListening(
							const ::com::sun::star::uno::Sequence< ::rtl::OUString >& _rNodes,
							const ::vos::ORef< IRequestCallback >& _rHandler) = 0;

	/** stop listening on a series of nodes.
		@param			_rNodes			sequence of node paths
		@param			_rHandler		callback for status notifications. May be NULL.
		@seealso		stopListening
		@seealso		stopAllListening
		@seealso		setListener
	*/
	virtual void		stopListening(
							const ::com::sun::star::uno::Sequence< ::rtl::OUString >& _rNodes,
							const ::vos::ORef< IRequestCallback >& _rHandler) = 0;

	/** stop listening at all nodes
		@param			_rHandler		callback for status notifications. May be NULL.
		@seealso		stopListening
		@seealso		startListening
		@seealso		setListener
	*/
	virtual void		stopAllListening(
							const ::vos::ORef< IRequestCallback >& _rHandler) = 0;
// node related methods
	/** open (on the server's side) a node given by it's path. Basicly, this means
		that the the server ensures that the node is loaded from the underlying storage
		system into it's cache.<BR>
		For a more detailed description of the execution of a openNode request, please see the
		appropriate registry service documentation.
		@param			_rNodeAccessor	the path to the node in URI notation.
		@param			_nLevel			indicates how many tree levels under the node determined by <arg>_rNodeAccessor</arg>
										should be loaded
		@param			_rHandler		callback for status notifications. May be NULL,
										though this is not recommended, as, without the id
										returned, you're not able to access the node afterwards.
	*/
	virtual void	openNode(const ::rtl::OUString& _rNodeAccessor, sal_Int32 _nLevel, const ::vos::ORef< IOpenObjectCallback >& _rHandler) = 0;

	/** open (on the server's side) a node given by it's path. The registry server tries
		to finish any pending transactions before the close request is fullfilled.<BR>
		For a more detailed description of the execution of a closeNode request, please see the
		appropriate registry service documentation.
		@param			_rNodeId		the id of the node (as supplied to the <type>IOpenObjectCallback</type>
										interface used for opening the node)
		@param			_rHandler		callback for status notifications. May be NULL.
	*/
	virtual void	closeNode(const ::rtl::OUString& _rNodeId, const ::vos::ORef< IRequestCallback >& _rHandler) = 0;

	/** refresh the given node within the registry server's cache. Can't be used to initially load
		a node into the cache. As server-side node-caching is handled on a per-tree basis, you first
		have to call openNode for any ancestor of the node you want to use here.
		@param			_rNodeAccessor	the path to the node in URI notation.
		@param			_rHandler		callback for status notifications.
										May be NULL (if you're sure that the request will succeed and
										not interested in the refresh results ...)
		@seealso		openNode
		@seealso		closeNode
	*/
	virtual void	getNode(const ::rtl::OUString& _rNodeAccessor, const ::vos::ORef< IOpenObjectCallback >& _rHandler) = 0;

	/** update a registry nodes content (maybe partial).
		@param			_rNodeId		the id of a node (as delivered to the <type>IOpenObjectCallback</type>
										interface used for opening the node) of which a small subtree is about
										to be modified. 
		@param			_rNodeAccessor	the subnode which is to be modified, must be a sub node
										of the one identified by _nNodeId.
		@param			_pNodeWriter	a pointer to the object writing the pure node data. Must not
										be NULL. The object pointed to does not need to support ref-counting,
										so the caller has to ensure that it lives until the updateNode
										returns (It does NOT need to live until the request is done).
		@param			_rHandler		callback for status notifications.
										May be NULL (if you're sure that the request will succeed and
										not interested in the results ...)<BR>
										This is an <type>IDataRequestCallback</type> instead of a simple
										<type>IRequestCallback</type>, as the current state of the node
										updated is transfered back to the caller (even if the update did not
										succeed this data will represent the current state without that update).
		@seealso		addNode
		@seealso		deleteNode
	*/
	virtual void	updateNode(const ::rtl::OUString& _rNodeId, const ::rtl::OUString& _rNodeAccessor,
		IDOMNodeDataProvider* _pNodeWriter, const ::vos::ORef< IDataRequestCallback >& _rHandler) = 0;

	/** add a new registry node
		@param			_rNodeId		the id of a node (as delivered to the <type>IOpenObjectCallback</type>
										interface used for opening the node) under which a new node should be added
										<BR>
										Note that this does not mean that you have to insert new nodes directly under
										nodes previously opened with <method>openNode</method>. It is only required
										that, if you want to insert a new node under a existing one, one of
										the <em>ancestors</em> has to be opened previously.
		@param			_rParentNodeAccessor	the path to the parent node, under which the new node should be added.
		@param			_pNodeWriter	a pointer to the object writing the pure node data. Must not
										be NULL. The object pointed to does not need to support ref-counting,
										so the caller has to ensure that it lives until the updateNode
										returns (It does NOT need to live until the request is done).
										<BR>
										The top level element to write is the node name, not the parent node name !
										This means that there is some redundance, as the node name is supplied by
										the writer and by the parameter <arg>_rNewNodeName</arg>.
		@param			_rHandler		callback for status notifications.
										May be NULL (if you're sure that the request will succeed and
										not interested in the results ...)
		@seealso		updateNode
		@seealso		deleteNode
	*/
	virtual void	addNode(const ::rtl::OUString& _rNodeId, const ::rtl::OUString& _rParentNodeAccessor,
		IDOMNodeDataProvider* _pNodeWriter, const ::vos::ORef< IRequestCallback >& _rHandler) = 0;

	/** delete a registry node
		@param			_rNodeId		the id of a node (as delivered to the <type>IOpenObjectCallback</type>
										interface used for opening the node) under which the to-be-deleted node
										is located
		@param			_rNodeAccessor	the path of the node to be deleted. The node given by <arg>_nNodeId</arg>
										must be an ancestor of this node and previously loaded using <method>openNode</method>.
		@param			_rHandler		callback for status notifications.
										May be NULL (if you're sure that the request will succeed and
										not interested in the results ...)
		@seealso		updateNode
		@seealso		addNode
	*/
	virtual void	deleteNode(const ::rtl::OUString& _nNodeId, const ::rtl::OUString& _rNodeAccessor,
		const ::vos::ORef< IRequestCallback >& _rHandler) = 0;


	/** add a new user to the registry registry
		@param			_pNodeWriter	a pointer to the object writing the pure node data. Must not
										be NULL. The object pointed to does not need to support ref-counting,
										so the caller has to ensure that it lives until the updateNode
										returns (It does NOT need to live until the request is done).
										<BR>
										The top level element to write is the node name, not the parent node name !
										This means that there is some redundance, as the node name is supplied by
										the writer and by the parameter <arg>_rNewNodeName</arg>.
		@param			_rHandler		callback for status notifications.
										May be NULL (if you're sure that the request will succeed and
										not interested in the results ...)
		@param			_rNodeId		the id of the 'all users' node (as delivered to the <type>IOpenObjectCallback</type>
										interface used for opening the node) under which the new user should be added
										<p>
										May be omitted if the node hasn't previously be opened.
		@seealso		addNode
	*/
	virtual void	addUser(const ::rtl::OUString& _rUser, IDOMNodeDataProvider* _pNodeWriter, const ::vos::ORef< IRequestCallback >& _rHandler,const ::rtl::OUString& _rNodeId = ::rtl::OUString()) = 0;

// hacks :( ....
#ifdef _OPEN_COMPONENT_NODES_ONLY_
	/** Valid as long as we have to hack around some restrictions of the remote server .
		<BR>
		Called whenever a client of the session wants to do an openNode to confirm the legality of this
		path for an openNode request. The does not need to do any semantical checks (e.g. if the path exists),
		the result is only of syntactical significance.
		@param		_sNodePath		the nodepath the client wants to open. On a rejective return, this is
									used as out parameter an contains the really to-be-opened path, which
									is below the requested one.
		@param		_nDepth			the depth for the openNode request. Will be adjusted on a rejective return, too.
		@return		sal_True if the client is allowed to open the node with the given path. sal_False, if not. In
					the latter case the <arg>_sNodePath</arg> contains the path of the node which should be opened
					instead
	*/
	virtual sal_Bool confirmOpenNode(::rtl::OUString& _sNodePath, sal_Int32& _nDepth) { return sal_True; }
#endif

	/** splits a node accessor (which consists of a node path and a user name) into it's components.
		@param		_rNodeAccessor		the node accessor
		@param		_rNodePath			the path to the node the accessor refers to
		@param		_rUser				the user the accessor refers to
		@return		sal_True	in case the accessor contained a user component (which is optional), sal_False else
		@seealso	composeNodeAccessor
	*/
	static sal_Bool			splitNodeAccessor(const ::rtl::OUString& _rNodeAccessor, ::rtl::OUString& _rNodePath, ::rtl::OUString& _rUser);

	/** composes a node acessor from it's tow components node path and user
		@param		_rNodePath		the path to the node
		@param		_rUser			the user which's tree should be accessed
		@return		the composed node accessor
		@seealso	splitNodeAccessor
	*/
	static ::rtl::OUString	composeNodeAccessor(const ::rtl::OUString& _rNodePath, const ::rtl::OUString& _rUser);

	/** composes a special node acessor for the set of all UserProfiles
		@return		the special node accessor
		@seealso	composeNodeAccessor
	*/
	static ::rtl::OUString	getUserAdminAccessor();
};

//..........................................................................
}	// namespace configmgr
//..........................................................................

#endif // _CONFIGMGR_SESSION_CONFIGSESSION_HXX_


