/*************************************************************************
 *
 *  $RCSfile: confproviderimpl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_PROVIDERIMPL_HXX_
#define CONFIGMGR_API_PROVIDERIMPL_HXX_

#ifndef CONFIGMGR_API_EVENTS_HXX_
#include "confevents.hxx"
#endif
#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

namespace com { namespace sun { namespace star { namespace uno {
	class Any;
	class XInterface;
	template <class> class Sequence;
	template <class> class Reference;
}	}	}	}

namespace rtl { class OUString; }

namespace configmgr
{
	namespace css		= ::com::sun::star;
	namespace uno		= css::uno;
	namespace script	= css::script;
	using ::rtl::OUString;

	class Module;
	class HierarchyReadAccess;
	class HierarchyUpdateRoot;
	class ISubtree;
	class IConfigSession;
	class ConfigurationProvider;
	class TreeManager;

	class ConfigurationProviderImpl
			:public IRefCountedTreeProvider
			,public IRefCountedTemplateProvider
	{
		friend class ConfigurationProvider;

		TreeManager*				m_pConfiguration;	// the tree cache. Will hold a reference to us as long as it lifce
		ConfigChangeBroadcaster		m_aNotifier; // order depedency - this must be after the TreeManager
		ConfigurationProvider*		m_pProvider;		/// used for ref counting

		::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >
									m_xConverter;
	
		ConfigurationProviderImpl(ConfigurationProvider* _pProvider, IConfigSession* _pSession, const uno::Reference< script::XTypeConverter >& _rxTypeConverter);
		~ConfigurationProviderImpl();

		// ITreeProvider
		virtual ISubtree const*	getSubtree( OUString const& aComponentName ) const;
		virtual ISubtree *		requestSubtree( OUString const& aSubtreePath, sal_Int16 nMinLevels = ALL_LEVELS);
		virtual void updateTree(TreeChangeList& aChanges) throw (starlang::WrappedTargetException, uno::RuntimeException);
		virtual const INode *	getNode(const OUString& _rPath);

		// ITemplateProvider
		virtual ::std::auto_ptr<INode> createInstance(const ::rtl::OUString& _rTemplateName) throw (uno::Exception);

		// IRefCountedTreeProvider
	    virtual void SAL_CALL acquire(  ) throw ();
		virtual void SAL_CALL release(  ) throw ();
	protected:
		ISynchronizedData      & getTreeLock();
		ISynchronizedData const& getTreeLock() const;

		// ISyncronizedData
		virtual void acquireReadAccess() const;
		virtual void releaseReadAccess() const;
		virtual void acquireWriteAccess();
		virtual void releaseWriteAccess();

	public:
		// factory methods
		uno::Reference<uno::XInterface>  createReadAccess( uno::Sequence<uno::Any> const& aArgs);
		uno::Reference<uno::XInterface>  createUpdateAccess( uno::Sequence<uno::Any> const& aArgs);
		uno::Reference<uno::XInterface>  createUserAdminAccess( uno::Sequence<uno::Any> const& aArgs);

		// parser methods
		void extractArgs(const uno::Sequence<uno::Any>& _rArgs, ::rtl::OUString& /* [out] */ _rNodeAccessor, sal_Int32& /* [out] */ _nLevels);

		ISubtree* requestSubtree(OUString const& _rAccessor, sal_Int32 nMinLevels );	
		// actual factory methods
		// the retruned object (if any) has to be acquired once)
		HierarchyReadAccess*  buildReadAccess( OUString const& _rAccessor, sal_Int32 nMinLevels);
		// the retruned object (if any) has to be acquired once)
		HierarchyUpdateRoot* buildUpdateAccess(OUString const& _rAccessor, sal_Int32 nMinLevels);
	};


} // namespace configmgr

#endif // CONFIGMGR_API_PROVIDERIMPL_HXX_


