/*************************************************************************
 *
 *  $RCSfile: confsetaccess.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_API_SETACCESS_HXX_
#define _CONFIGMGR_API_SETACCESS_HXX_

#ifndef _CONFIGMGR_API_UPDATEACCESS_HXX_
#include "confupdateaccess.hxx"
#endif

#ifndef _CPPUHELPER_IMPLBASE3_HXX_
#include <cppuhelper/implbase3.hxx>
#endif

#ifndef _COM_SUN_STAR_CONTAINER_XNAMECONTAINER_HPP_
#include <com/sun/star/container/XNameContainer.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XSINGLESERVICEFACTORY_HPP_
#include <com/sun/star/lang/XSingleServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_XSTRINGESCAPE_HPP_
#include <com/sun/star/util/XStringEscape.hpp>
#endif



//........................................................................
namespace configmgr
{
//........................................................................

//==========================================================================
//= OFloatableNode
//==========================================================================
/** an (non-root) element of an updatable sub tree, which is an instance of an configuration scheme template.
	<BR>
	A set element can be floating or fixed. The first means that is has been created without a parent and
	is not part of any tree structure currently. This is the case for newly created nodes which have not
	been inserted into an existing structure yet. If they are inserted (<method>insertInto</method>), they are
	fixed from then on and can't be floating anymore.
	<BR>
	The definition above implies that any child of an floating node is fixed.
*/
class OFloatableNode
		:public HierarchyUpdateElement
{
	friend class OSetNode;
protected:
	::rtl::OUString				m_sChildTemplateName;	/// the name of the template where the object was created from
	TreeChangeList				m_aChanges;				/// the changes the node represents if it is floating
	::std::auto_ptr<ISubtree>	m_aNode;				/// the node we represent (as long as we're floating)

public:
	/** creates a floating node.
			@param		_rBHelper			notification helper, contains a mutex for access safety
			@param		_rTemplateName		the name of the template the node represents
			@param		_rContainerPath		the path to the container which created this element,
											relative to _pUpdateRoot, including the container's node itself,
											_not_ including the template instance (of course, as the node is not part
											of the real tree, yet)
			@param		_pTemplateInstance	a template instance. This node must no be part of any tree.
			@param		_pUpdateRoot		the root to which the will belong (after it is not floating anymore)
	*/
	OFloatableNode(
		BroadcastHelper& _rBHelper,
		const ::rtl::OUString& _rTemplateName,
		const TreePath& _rContainerPath,
		::std::auto_ptr<ISubtree> _pTemplateInstance,
		HierarchyUpdateRoot* _pUpdateRoot
	);

	// XUnoTunnel
	virtual sal_Int64 SAL_CALL getSomething( const ::com::sun::star::uno::Sequence< sal_Int8 >& aIdentifier ) throw(::com::sun::star::uno::RuntimeException);
	static ::com::sun::star::uno::Sequence< sal_Int8 > getUnoTunnelImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// methods to be overwritten because the base class would return nonsense if the object is floating
	// (because of it's faked members describing the location)
	virtual ::rtl::OUString SAL_CALL getName(  ) throw (::com::sun::star::uno::RuntimeException);
	// TODO : the same counts for the XHierarchicalName, me thinks ...

protected:
	// HierarchyElementImpl overridables
	virtual HierarchyAccessImpl* getParentAccess();

	/// returns sal_True if the node does not belong to any tree at the moment
	sal_Bool	isFloating() const { return m_aNode.get() != NULL; }

	::rtl::OUString getTemplateName() const { return m_sChildTemplateName; }

	/** tell the object that it has been inserted into an existing tree and is not floating anymore.
		The returned tree is the node which is represented by the add operation, it's the same as given in
		the ctor. In addition it's in the responsibility of the caller afterwards.
	*/
	::std::auto_ptr<ISubtree> inserted(const TreePath& _rNewAncestors);
};

//==========================================================================
//= OSetNode
//==========================================================================
typedef ::cppu::ImplHelper3	<	 ::com::sun::star::container::XNameContainer
								,::com::sun::star::lang::XSingleServiceFactory
								,::com::sun::star::util::XStringEscape
							>	OSetNode_Base;

/** abstract base class for configuration nodes which are dynamic sets
*/
class OSetNode : public OSetNode_Base
{
protected:
	::vos::ORef< IRefCountedTemplateProvider >
						m_aNodeFactory;
	::rtl::OUString		m_sContainerTemplateName;

	::osl::Mutex&		m_rMutex;
	sal_Bool			m_bAllowUnescaped;
		// this is kind of a HACK ... the only usage of this flag is for user administration, where it is set
		// to false, as we don't require user names to be escaped (they're escaped on a lower level).
		// ...
		// Okay, it's not "some kind of", it IS a HACK

public:
	/** constructs the object.
		@param		_rSafety			mutex to be used for thread safety
		@param		_rNodeFactory		a pointer to the factory which can be used to create new children. Must not be NULL.
		@param		_rTemplateName		the name of the template to be used to create new children. Must be a valid template name for the node factory.
		@param		_bAllowUnescaped	flag if on inserting new children, unescaped names are allowed
	*/
	OSetNode(
		::osl::Mutex& _rSafety,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rNodeFactory,
		const ::rtl::OUString& _rTemplateName,
		sal_Bool _bAllowUnescaped = sal_False
		);

	// XNameContainer
    virtual void SAL_CALL insertByName( const ::rtl::OUString& _rName, const ::com::sun::star::uno::Any& _rElement ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::container::ElementExistException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL removeByName( const ::rtl::OUString& _rName ) throw(::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);

	// XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& _rName, const ::com::sun::star::uno::Any& _rElement ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);

	// XNameAccess
	// abstract, have to be implmented by derived classes
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) throw(::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException) = 0;
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) throw(::com::sun::star::uno::RuntimeException) = 0;
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) throw(::com::sun::star::uno::RuntimeException) = 0;

	// XElementAccess
	// abstract, have to be implmented by derived classes
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) throw(::com::sun::star::uno::RuntimeException) = 0;
    virtual sal_Bool SAL_CALL hasElements(  ) throw(::com::sun::star::uno::RuntimeException) = 0;

	// XSingleServiceFactory
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > SAL_CALL createInstance(  ) throw(::com::sun::star::uno::Exception, ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > SAL_CALL createInstanceWithArguments( const ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Any >& aArguments ) throw(::com::sun::star::uno::Exception, ::com::sun::star::uno::RuntimeException);

	// XStringEscape
	virtual ::rtl::OUString SAL_CALL escapeString( const ::rtl::OUString& aString ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::uno::RuntimeException);
	virtual ::rtl::OUString SAL_CALL unescapeString( const ::rtl::OUString& aEscapedString ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::uno::RuntimeException);

protected:
	// --- changes tree access ---
	/// is used for synchronizing the access to the changes tree
	virtual ISynchronizedData*		getChangeTreeSync() = 0;
	/** get the changes tree of the object.
		The returned pointer is allowed to be NULL if and only if the set node is the descendant of a node which
		was formerly floating and has been inserted (into another set node), but not been committed yet.
	*/
	virtual SubtreeChange*			getChangesTree() = 0;

	// --- tree access ---
	/// is used for synchronizing the access to the tree (not the changes tree)
	virtual ISynchronizedData const *		getTreeSync() const = 0;
	/// get the subtree the object represents. Must not be NULL.
	virtual TreeRef					getThisTree() const = 0;
	/// get the ancestors of the set node
	virtual void getNodePath(TreePath& _rPath) const = 0;

	// --- object creation ---
	/** return an element which represents a floating node. The returned object (if any) has to be acquired once.
		@param			_rCompleteThisPath	the complete path this the object itself. Could be determined in the derived
											class, too, but as we have all information needed for this we collect
											in ourself ....
		@param			_rFloatingSubtree	the sub tree which was build acording the template description. The tree
											is not part of any other tree, it is floating, too.
	*/
	virtual OFloatableNode* createFloatingElement(const TreePath& _rCompleteThisPath, ::std::auto_ptr<ISubtree> _rFloatingSubtree) = 0;

	/** return a change which represents the addition of a floating node. The returned object is owned by the caller.
		@param			pAddedNode_	the floasting node that should be added. This node must be still floating
		@param			sNewName_	the name under which the new node should be added.
	*/
	virtual std::auto_ptr<AddNode> createAddTransaction(OFloatableNode& rAddedNode_, ::rtl::OUString const& sNewName_);

	// --- container notifications ---
	/// will be called after a formerly floating node has been inserted into the set node
	virtual void insertedElement(const AddNode& _rChange, OFloatableNode* _pNewElement) = 0;

	/** called after the removal of a node which is existent in the tree (see <method>getThisTree</method>), while
		the set node itself is an integral part of the update hierarchy.
		<BR>
		Before calling this method, a RemoveNode change will be inserted into the changes tree.
		<BR>
		The responsiblity for the given SubtreeChange may be taken by the callee, else it is deleted automatically
		by the caller afterwards.
		@param		_rName		the name of the removed element
		@param		_rChange	the existent change for that child. May be NULL, if there was no previous change.
								If not NULL, the change has already been removed from the changes tree belonging
								to the set node (see <method>getChangesTree</method>) und replaced by a RemoveNode
								change.
	*/
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange) = 0;

	/** called after the removal of a node existent in the tree (see <method>getThisTree</method>), while the
		set node itself is part of a (still) floating sub tree. In this case there is no changes tree to maintain,
		so the node will just be removed from the tree before calling this method.
		<BR>
		The responsiblity for the given INode may be taken by the callee, else it is automatically
		deleted by the caller afterwards.
		@param		_rChange	the name of the removed element
		@param		_rNode		the existent node for that child. The node has already been removed
								from the tree belonging to the set node (see <method>getThisTree</method>).
	*/
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode) = 0;
};

//========================================================================
//= OSetNodeUpdateElement
//========================================================================
/** a class which represents a set node which is non-root element of an update sub tree.
	<BR>
	still abstract and to be concretized with ref-counting and service info.
*/
class OSetNodeUpdateElement
			:public HierarchyUpdateElement
			,public OSetNode
{
public:
	OSetNodeUpdateElement(
		BroadcastHelper& _rBHelper,
		HierarchyUpdateRoot* _pUpdateRoot,
		SubtreeChange* _pModifications,
		const TreePath& _rRelativeLocation
	);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);

	// XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& _rName, const ::com::sun::star::uno::Any& _rElement ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException)
		{ HierarchyUpdateElement::replaceByName(_rName, _rElement); }
		// the OSetNode does not allow to replace it's sub trees, so we allow it just for the elements which is handled
		// by HierarchyUpdateBase

	// XNameAccess
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) throw(::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) throw(::com::sun::star::uno::RuntimeException);

	// XElementAccess
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyUpdateElement::getElementType(); }
    virtual sal_Bool SAL_CALL hasElements(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyUpdateElement::hasElements(); }

protected:
	// OSetNode overridables
	virtual ISynchronizedData*		getChangeTreeSync() { return HierarchyUpdateElement::getUpdateRoot(); }
	virtual ISynchronizedData const*getTreeSync() const { return HierarchyUpdateElement::getTreeSync(); }
	virtual SubtreeChange*			getChangesTree()	{ return HierarchyUpdateElement::getChangesTree(); }
	virtual TreeRef					getThisTree() const	{ return HierarchyUpdateElement::getThisTree(); }
	virtual void					getNodePath(TreePath& _rPath) const { _rPath = HierarchyUpdateElement::getNodePath(); }

	virtual void insertedElement(const AddNode& _rChange, OFloatableNode* _pNewElement);
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange);
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode);
};

//========================================================================
//= OFloatableSetNode
//========================================================================
/** a class which represents a set node which is non-root floatable element of an update sub tree.
	This means it is a set node which can be created (as floating node) from a parent node, which is
	a set node itself.
	<BR>
	still abstract and to be concretized with ref-counting and service info.
*/
class OFloatableSetNode
			:public OFloatableNode
			,public OSetNode
{
public:
	/** ctor
		@param		_rMutex						access safety
		@param		_rChildTemplateName			name of the template which was used to create the node represented
												by the object
		@param		_pChildTemplateInstance		the (floating) node which is represented by the object
		@param		_rContainerPath				the path to the container which created this element,
												relative to _pUpdateRoot, including the container's node itself,
												_not_ including the child template instance (of course, as the node is not part
												of the real tree, yet)
		@param		_rContainerTemplateName		name of the template which should be used for creating new children
												of the node. For this creating the TemplateFactory provided by
												<method>HierarchyUpdateRoot::getTemplateFactory</method> is used.
		@param		_pUpdateRoot				the root to which the will belong (after it is not floating anymore)
	*/
	OFloatableSetNode(
		BroadcastHelper& _rBHelper,
		const ::rtl::OUString& _rChildTemplateName,
		::std::auto_ptr<ISubtree> _pChildTemplateInstance,
		const TreePath& _rContainerPath,
		const ::rtl::OUString& _rContainerTemplateName,
		HierarchyUpdateRoot* _pUpdateRoot
	);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);

	// XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& _rName, const ::com::sun::star::uno::Any& _rElement ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException)
		{ OFloatableNode::replaceByName(_rName, _rElement); }
		// the OSetNode does not allow to replace it's sub trees, so we allow it just for the elements which is handled
		// by HierarchyUpdateBase

	// XNameAccess
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) throw(::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) throw(::com::sun::star::uno::RuntimeException);

	// XElementAccess
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return OFloatableNode::getElementType(); }
    virtual sal_Bool SAL_CALL hasElements(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return OFloatableNode::hasElements(); }

protected:
	// OSetNode overridables
	virtual ISynchronizedData*		getChangeTreeSync() { return getUpdateRoot(); }
	virtual SubtreeChange*			getChangesTree()	{ return OFloatableNode::getChangesTree(); }
	virtual ISynchronizedData const*getTreeSync() const { return OFloatableNode::getTreeSync(); }
	virtual TreeRef					getThisTree() const	{ return OFloatableNode::getThisTree(); }
	virtual void					getNodePath(TreePath& _rPath) const { _rPath = OFloatableNode::getNodePath(); }

	virtual void insertedElement(const AddNode& _rName, OFloatableNode* _pNewElement);
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange);
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode);
};

//========================================================================
//= ORootSetNode
//========================================================================
/** a class which represents a root node of an update tree, which is a set node additionally.
	<BR>
	still abstract and to be concretized with ref-counting and service info.
*/
class ORootSetNode
			:public HierarchyUpdateRoot
			,public OSetNode
{
protected:
	using HierarchyUpdateRoot::m_rMutex;

public:
	/** ctor
		@param		_rMutex						access safety
		@param		_rName						the location of the parent of the object
		@param		_pTree						the tree represented by the object
		@param		_rContainerTemplateName		name of the template which should be used for creating new children
												of the node.
		@param		_rTemplateFactory			factory which is able to provide the templates described by _rContainerTemplateName
		@param		_rUpdater					the object which implements the tree updates
	*/
	ORootSetNode(
		BroadcastHelper& _rBHelper,
		const ConfigurationName& _rName,
		TreeRef _pTree,
		const ::rtl::OUString& _rContainerTemplateName,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		IConfigBroadcaster& _rEventSource,
		const ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >& _rxConverter
	);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);

	// XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& _rName, const ::com::sun::star::uno::Any& _rElement ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException)
		{ HierarchyUpdateRoot::replaceByName(_rName, _rElement); }
		// the OSetNode does not allow to replace it's sub trees, so we allow it just for the elements which is handled
		// by HierarchyUpdateBase

	// XNameAccess
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) throw(::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) throw(::com::sun::star::uno::RuntimeException);

	// XElementAccess
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyUpdateRoot::getElementType(); }
    virtual sal_Bool SAL_CALL hasElements(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyUpdateRoot::hasElements(); }

protected:
	// OSetNode overridables
	virtual ISynchronizedData*		getChangeTreeSync() { return this; }
	virtual SubtreeChange*			getChangesTree()	{ return &m_aChanges.root; }
	virtual ISynchronizedData const*getTreeSync() const	{ return HierarchyUpdateRoot::getTreeSync(); }
	virtual TreeRef					getThisTree() const	{ return HierarchyUpdateRoot::getThisTree(); }
	virtual void					getNodePath(TreePath& _rPath) const { _rPath.clear(); /* no ancestor */ }

	virtual void insertedElement(const AddNode& _rName, OFloatableNode* _pNewElement);
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<SubtreeChange> _rChange);
	virtual void removedElement(const ::rtl::OUString& _rName, ::std::auto_ptr<INode> _rNode);
};

//........................................................................
} // namespace configmgr
//........................................................................

#endif // _CONFIGMGR_API_SETACCESS_HXX_


