/*************************************************************************
 *
 *  $RCSfile: confupdateaccess.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_API_UPDATEACCESS_HXX_
#define _CONFIGMGR_API_UPDATEACCESS_HXX_

#ifndef CONFIGMGR_API_BASEACCESS_HXX_
#include "confbaseaccess.hxx"
#endif
#ifndef CONFIGMGR_API_BASEELEMENT_HXX_
#include "confbaseelement.hxx"
#endif
#ifndef CONFIGMGR_API_CHANGESNOTIFIER_HXX_
#include "changenotifier.hxx"
#endif

#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif
#ifndef CONFIGMGR_API_EVENTS_HXX_
#include "confevents.hxx"
#endif
#ifndef _CONFIGMGR_SYNCHRONIZE_HXX_
#include "synchronize.hxx"
#endif

#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx>
#endif
#ifndef _CPPUHELPER_IMPLBASE2_HXX_
#include <cppuhelper/implbase2.hxx>
#endif
#ifndef _CPPUHELPER_IMPLBASE3_HXX_
#include <cppuhelper/implbase3.hxx>
#endif

#ifndef _COM_SUN_STAR_CONTAINER_XNAMEREPLACE_HPP_
#include <com/sun/star/container/XNameReplace.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XUNOTUNNEL_HPP_
#include <com/sun/star/lang/XUnoTunnel.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_XCHANGESBATCH_HPP_
#include <com/sun/star/util/XChangesBatch.hpp>
#endif
#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif
#ifndef _CPPUHELPER_WEAKREF_HXX_ 
#include <cppuhelper/weakref.hxx>
#endif
#ifndef _UTL_STLTYPES_HXX_
#include <unotools/stl_types.hxx>
#endif

#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif

//........................................................................
namespace configmgr
{
//........................................................................
	typedef std::vector< ValueChange > ValueChanges;

//========================================================================
//= HierarchyUpdateBase
//========================================================================
class HierarchyUpdateRoot;
typedef ::cppu::ImplHelper2	<	::com::sun::star::container::XNameReplace,
								::com::sun::star::lang::XUnoTunnel
							>	HierarchyUpdateBase_Base;
/** base for updatable elements in the registry tree<BR>
	abstract, has to be concretized with ref counting and service info implementation
	<BR>
	The class is derived from <type scope="com.sun.star.lang">XComponent</type>, but it - intentionally - does not
	make this interface accessible for the public (in particular, this means that queryInterface does not return it).
*/
class HierarchyUpdateBase
	:public HierarchyAccessImpl
	,public HierarchyUpdateBase_Base
	,public ::com::sun::star::lang::XComponent
		// !!! not part of HierarchyUpdateBase_Base, we don't want this interface to be visible from outside)
{
	friend class HierarchyUpdateElement;
protected:
	DECLARE_STL_MAP(	ConfigurationName, ::com::sun::star::uno::WeakReference< ::com::sun::star::lang::XUnoTunnel >,
						ConfNameCompare, Descendants);
	HierarchyUpdateBase*		m_pNearestAncestor;			/// the nearest living ancestor
	ConfigurationName			m_aAncestorRelativePath;	/// our path relative to the nearest living ancestor
	Descendants					m_aDescendants;				/// all objects which's nearest living ancestor this object is

	::osl::Mutex&				m_rMutex;
	::osl::Mutex				m_aAncestorChainProtection;	/// protects the access to m_pNearestAncestor

	// the m_pNearestAncestor is not acquired in any way : if the ancestor dies, it would notify it's own nearest
	// ancestor, which would set a new ancestor for ourself, so this pointer is always valid ....

protected:
	~HierarchyUpdateBase();

public:
	/// ctor
	HierarchyUpdateBase(::osl::Mutex& _rMutex, TreeRef _pTree);

	// XInterface
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() = 0;
	virtual void SAL_CALL release() throw() = 0;

	// XTypeProvider
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);

	// XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& aName, const ::com::sun::star::uno::Any& aElement ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);

	// XNameAccess - base of XNameReplace - just forward to HierarchyAccessImpl
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& _rName ) throw(::com::sun::star::container::NoSuchElementException, ::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException)
		{ return HierarchyAccessImpl::getByName(_rName); }
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyAccessImpl::getElementNames(); }
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& _rName ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyAccessImpl::hasByName(_rName); }

	// XElementAccess - base of XNameAccess - just forward to HierarchyAccessImpl
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyAccessImpl::getElementType(); }
    virtual sal_Bool SAL_CALL hasElements(  ) throw(::com::sun::star::uno::RuntimeException)
		{ return HierarchyAccessImpl::hasElements(); }

	// XUnoTunnel
	virtual sal_Int64 SAL_CALL getSomething( const ::com::sun::star::uno::Sequence< sal_Int8 >& aIdentifier ) throw(::com::sun::star::uno::RuntimeException);
	static ::com::sun::star::uno::Sequence< sal_Int8 > getUnoTunnelImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent
	/// still abstract, to be overwritten by derived classes implementing the XComponent interface
    virtual void SAL_CALL dispose(  ) throw(::com::sun::star::uno::RuntimeException) = 0;
	/// still abstract, to be overwritten by derived classes implementing the XComponent interface
    virtual void SAL_CALL addEventListener( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener >& xListener ) throw(::com::sun::star::uno::RuntimeException) = 0;
	/// still abstract, to be overwritten by derived classes implementing the XComponent interface
    virtual void SAL_CALL removeEventListener( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener >& aListener ) throw(::com::sun::star::uno::RuntimeException) = 0;

	// new overwritables
	/** tell the object that it has been integrated into an existing update hierarchy. To be called only if
		the object does not have a modifications tree, yet (which means that it is part of a floating node).
		This is the case if and only if <method>getChangesTree</method> returns NULL.
		<BR>
		Derived clasess should overwrite this so that after returning from that call their
		<method>getChangesTree</method> won't return NULL anymore.
	*/
	virtual void integrated(SubtreeChange* _pNewObjectModifications);

protected:
// new overwritables (continued)
	virtual ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >
									getTypeConverter() const = 0;

	/// fill in the defaults for this object's children in the source tree
	virtual bool loadDefaults() = 0;

	/// get the syncronization object for the whole update sub tree
	virtual ISynchronizedData*		getUpdateSync() = 0;
	/** get the location of the root of the update tree the object is a part of. The name includes the name of
		the root node itself
	*/
	virtual ConfigurationName		getRootLocation() = 0;
	/// fill the given path with the nodes from the root of the update tree to the object itself
	virtual void getThisPath(TreePath& _rPathToFill) = 0;
	/** return the changes tree responsible for this node<BR>
		If (any only if) the node is part of a subtree jst added (but not committed), it's valid to return NULL here.
	*/
	virtual SubtreeChange* getChangesTree() = 0;
	/** create a new sub tree (non-root) element, which belongs to the same update sub tree the object itself resides in,
		works on a given changes subtree and represents a given path.<BR>
		Not allowed to return NULL unless a fatal system error occures (e.g. out of memory)
		<BR>
		The returned object has to be acquired once.
	*/
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath) = 0;

	/**	execute new value changes locally 
		and notify them to listeners directly listening on the update tree.
	*/
	virtual void executeLocalChanges(ValueChanges const& aChanges) = 0;

	/// A new change has been integrated locally and must be fired to listeners directly listening on the update tree.
	virtual void fireLocalChange(Change const& aChange) = 0;

	/// to be called when the object is being disposed.
	virtual void SAL_CALL disposing();

	// HierarchyAccessImpl overwriteables
	virtual NodeRef findDescendant(OUString const& aRelativePath, TreePath& aPath);

protected:
	/**	integrate new value changes locally 
		do not send any notifications.
		@see executeLocalChanges
	*/
	void updateLocalValues(ValueChanges const& aChanges);

// HierarchyAccessImpl implementation
	/// still to be overwritten
	virtual HierarchyAccessImpl* getSubtreeAccess(TreeRef aSubtree, TreePath const& aPath);
	/// still to be overwritten
	virtual bool normalizeNestedPath(ConfigurationName& aRelativeName) = 0;
	/// still to be overwritten
	virtual ISynchronizedData const * getTreeSync()	const = 0;


	// administration of the "nearest ancestor"
	/// set a new "nearest ancestor" for the object
	inline void setNearestAncestor(HierarchyUpdateBase* _pNewAncestor, const ConfigurationName& _rRelLoc)
	{
		::osl::MutexGuard aGuard(m_aAncestorChainProtection);
		m_pNearestAncestor = _pNewAncestor;
		m_aAncestorRelativePath = _rRelLoc;
	}
	/// notification of an descendant to it's nearest ancestor that it's going to die
	void notifyDeath(const ConfigurationName& _rRelLoc, HierarchyUpdateBase* _pDyingObject);

	/** tells the object that a new "nearest descendant" has been inserted. No checks are made if for the
		given sub tree the object really is the nearest descendant, this is the responsibility of the caller.
	*/
	void insertedNearestDescendant(const ::rtl::OUString& _rRelativeAccessor, HierarchyUpdateBase* pNewDesc);

	/** disposes all descendants under a given relative path.
	*/
	void disposeDescendants(const ::rtl::OUString& _rPath);

	/** tells the object that the tree it belongs to has been committed.
		Will forward this to all descendants and make any adjustments necessary, for instance call the
		<method>integrated</method> method for newly added subtrees.
	*/
	void committed();

	/** return (safely) the descendant which belongs to the given iterator
		<BR>
		@param		_rPos							an iterator, belonging to m_aDescendants
		@param		_rUnoTunnelImplementationId		the id to ask the uno tunnel for.
	*/
	HierarchyUpdateBase* getSafeDescendant(const ConstDescendantsIterator& _rPos,
		const ::com::sun::star::uno::Sequence< sal_Int8 >& _rUnoTunnelImplementationId = HierarchyUpdateBase::getUnoTunnelImplementationId());
		// templated member functions would be a nice feature here .....
};

//========================================================================
//= HierarchyUpdateRoot
//========================================================================
class HierarchyUpdateElement;
class OTreeAccessor;
typedef ::cppu::ImplHelper3	<
								::com::sun::star::util::XChangesBatch
							,	::com::sun::star::container::XNamed
							,	::com::sun::star::container::XHierarchicalName
							> HierarchyUpdateRoot_Base;

/** top level updatable node, not navigatable in the direction of the parent
	@seealso HierarchyUpdateElement
*/
class HierarchyUpdateRoot
		:public HierarchyUpdateBase
		,public HierarchyUpdateRoot_Base
		,public ChangesNotifierImpl
		,public ISynchronizedData
{
	friend class HierarchyUpdateElement;
	friend class HierarchyUpdateBase;

protected:
	TreeChangeList		m_aChanges;					/// the top level changes tree
	// ORDER DEPENDENCY - the notifier must be after the change tree !!
	ScreenedChangeMultiplexer				m_aNotifier;			/// the dispatcher that filters and broadcast events from the main tree
	::vos::ORef< IRefCountedTreeProvider >
						m_xUpdateProvider;			/// the instance which is able to do the commits
	::vos::ORef< IRefCountedTemplateProvider >
						m_xTemplateProvider;		/// the instance which is able to do the commits
	OTreeAccessor*		m_pUpdateTreeProtection;	/// an own protection for m_aChanges

	::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >
						m_xTypeConverter;			/// to ensure the type correctness when updating values

protected:
	~HierarchyUpdateRoot();

public:
	HierarchyUpdateRoot(
		BroadcastHelper& _rbHelper, 
		TreeRef _pTree,	const ConfigurationName& _rName,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		IConfigBroadcaster& rEventSource_,
		const ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >& _rxConverter
		);

	/// late ctor
	virtual void Init();

	// XTypeProvider
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);
		// no getImplementationId as this this class is still abstract

	// XInterface
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() = 0;
	virtual void SAL_CALL release() throw() = 0;

	// XChangesBatch
    virtual void SAL_CALL commitChanges(  ) throw(::com::sun::star::lang::WrappedTargetException, ::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasPendingChanges(  ) throw(::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::util::ChangesSet SAL_CALL getPendingChanges(  ) throw(::com::sun::star::uno::RuntimeException);

	// XNamed
	virtual ::rtl::OUString SAL_CALL getName(  ) throw(::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL setName( const ::rtl::OUString& aName ) throw(::com::sun::star::uno::RuntimeException);

	// XHierarchicalName
	virtual ::rtl::OUString SAL_CALL getHierarchicalName(  ) throw(::com::sun::star::uno::RuntimeException);
	virtual ::rtl::OUString SAL_CALL composeHierarchicalName( const ::rtl::OUString& aRelativeName ) throw(::com::sun::star::lang::IllegalArgumentException, ::com::sun::star::lang::NoSupportException, ::com::sun::star::uno::RuntimeException);

	//
	::vos::ORef< IRefCountedTemplateProvider > getTemplateFactory() const { return m_xTemplateProvider; }
protected:
	// ISynchronizedData
	virtual void acquireReadAccess() const;
	virtual void releaseReadAccess() const;
	virtual void acquireWriteAccess();
	virtual void releaseWriteAccess();

	virtual void SAL_CALL disposing();

	/// HierarchyAccessImpl overridables
	virtual bool normalizeNestedPath(ConfigurationName& aRelativeName);
	virtual ISynchronizedData const	* getTreeSync() const	{ return m_xUpdateProvider.getBodyPtr(); }
	/// returns the same lock as <method>getTreeSync</method>, but enabled for write locks 
	virtual ISynchronizedData		* getTreeUpdateSync()	{ return m_xUpdateProvider.getBodyPtr(); }

	// HierarchyUpdateBase overwritables
	::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >
									getTypeConverter() const { return m_xTypeConverter; }
	virtual ISynchronizedData*		getUpdateSync() { return this; }
	virtual ConfigurationName		getRootLocation() { return ConfigurationName(m_aChanges.pathToRoot, m_aChanges.root.getNodeName()); }
	virtual void getThisPath(TreePath& _rPathToFill) { _rPathToFill.clear(); _rPathToFill.push_back(getThisTree()); }
	virtual SubtreeChange* getChangesTree() { return &m_aChanges.root; }
	virtual bool loadDefaults();

	/**	execute new value changes locally 
		and notify them to listeners directly listening on the update tree.
	*/
	virtual void executeLocalChanges(ValueChanges const& aChanges);

	void fireLocalChange(Change const& aChange);

#ifdef DEBUG
	virtual void integrated(SubtreeChange* _pNewObjectModifications)
	{
		OSL_ENSHURE(sal_False, "HierarchyUpdateRoot::integrated : invalid call for a root object !");
		HierarchyUpdateBase::integrated(_pNewObjectModifications);
	}
#endif
private:
	// used to implement loadDefaults
	bool loadDefaultsAt(TreePath const& aNode);
};

//========================================================================
//= HierarchyUpdateElement
//========================================================================
/** an updateable node, which is not the root if it's update tree
	@seealso HierarchyUpdateRoot
*/
class HierarchyUpdateElement
		:public HierarchyUpdateBase
		,public HierarchyElementImpl
		,public ChangesNotifierImpl
{
	HierarchyUpdateRoot*		m_pUpdateRoot;				/// the parent element
	SubtreeChange*				m_pModifications;			/// the object where we collect modifications done on this node

protected:
	using HierarchyUpdateBase::m_rMutex;

protected:
	/** ctor. to be used by derived classes only. creates an element which is floating, i.e. does not belong
		to a config sub tree.
		@param		_rMutex			access safety
		@param		_pUpdateRoot	the root of the update access to object will belong to
		@param		_rParentPath	the path to the (potential) parent of the (still floating) node, not including
									_pFloatingNode and relative to _pUpdateRoot
		@param		_pFloatingNode	the (floating) node the object represents
	*/
	HierarchyUpdateElement(BroadcastHelper& _rBHelper, HierarchyUpdateRoot* _pUpdateRoot, const TreePath& _rParentPath, TreeRef _pFloatingNode);

public:
	/** ctor.
		@param			_pUpdateRoot		the top level node for the update access. The element will hold a hard ref to
											this  object, so it can't die until the element itself does.<BR>
											Must not be NULL.
		@param			_pModifications		the changes which are represented by the object
		@param			_rRelativeLocation	tree path describing the relative location of the element (including the
											element node itself, which implies that the path must not be empty !)
	*/
	HierarchyUpdateElement(BroadcastHelper& _rBHelper, 
		HierarchyUpdateRoot* _pUpdateRoot, 
		SubtreeChange* _pModifications, 
		const TreePath& _rRelativeLocation);

	/// late ctor
	virtual void Init();

	// XTypeProvider
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);
		// no getImplementationId as this this class is still abstract

	// some methods common in both base classes
    virtual ::com::sun::star::uno::Any SAL_CALL queryInterface( const ::com::sun::star::uno::Type& aType ) throw(::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() = 0;
	virtual void SAL_CALL release() throw() = 0;

	// XHierarchicalName - forward to HierarchyElementImpl
	virtual OUString SAL_CALL getHierarchicalName() 
		throw(uno::RuntimeException)
	{ return HierarchyElementImpl::getHierarchicalName(); }

	virtual OUString SAL_CALL composeHierarchicalName( const OUString& aRelativeName ) 
		throw(css::lang::IllegalArgumentException, css::lang::NoSupportException, uno::RuntimeException)
	{ return HierarchyElementImpl::composeHierarchicalName(aRelativeName); }

	// XUnoTunnel
	virtual sal_Int64 SAL_CALL getSomething( const ::com::sun::star::uno::Sequence< sal_Int8 >& aIdentifier ) throw(::com::sun::star::uno::RuntimeException);
	static ::com::sun::star::uno::Sequence< sal_Int8 > getUnoTunnelImplementationId() throw (::com::sun::star::uno::RuntimeException);

	/** notifies the object that it has been inserted into a hierarchy possibly different from the current one.
		Effectivley, this means that the object will release it's reference to the old root and acquire the new one.
		<BR>
		Not to be called if the element is already an integral part of a tree (i.e. if it already has a modifications tree)
	*/
	void insertedInto(HierarchyUpdateRoot* _pNewAccessRoot);

protected:
	~HierarchyUpdateElement();

	virtual void SAL_CALL disposing();

	// HierarchyAccessImpl overridables
	virtual bool normalizeNestedPath(ConfigurationName& aRelativeName);
	virtual ISynchronizedData const	* getTreeSync() const {  return m_pUpdateRoot->getTreeSync(); }

	// HierarchyElementImpl overridables
	virtual INode const& getThisNode();
	virtual HierarchyAccessImpl* getParentAccess();

	// HierarchyUpdateBase overwritables
	virtual ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >
									getTypeConverter() const { return m_pUpdateRoot->getTypeConverter(); }
	virtual ISynchronizedData*		getUpdateSync() { return m_pUpdateRoot; }
	virtual ConfigurationName		getRootLocation() { return m_pUpdateRoot->getRootLocation(); }
	virtual void getThisPath(TreePath& _rPathToFill);
	virtual SubtreeChange* getChangesTree() { return m_pModifications; }

	virtual bool loadDefaults();
	virtual void integrated(SubtreeChange* _pNewObjectModifications);

	/**	execute new value changes locally 
		and notify them to listeners directly listening on the update tree.
	*/
	virtual void executeLocalChanges(ValueChanges const& aChanges);
	void fireLocalChange(Change const& aChange);

	HierarchyUpdateRoot* getUpdateRoot() { return m_pUpdateRoot; }
};

//........................................................................
} // namespace configmgr
//........................................................................

#endif // CONFIGMGR_CONFACCESS_HXX_


