/*************************************************************************
 *
 *  $RCSfile: confupdateimpl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_API_UPDATEACCESSIMPL_HXX_
#define _CONFIGMGR_API_UPDATEACCESSIMPL_HXX_

#ifndef _CONFIGMGR_API_UPDATEACCESS_HXX_
#include "confupdateaccess.hxx"
#endif
#ifndef _CONFIGMGR_API_SETACCESS_HXX_
#include "confsetaccess.hxx"
#endif
#ifndef CONFIGMGR_API_GROUPACCESS_HXX_
#include "confgroupaccess.hxx"
#endif
#ifndef CONFIGMGR_API_SVCCOMPONENT_HXX_
#include "confsvccomponent.hxx"
#endif

//........................................................................
namespace configmgr
{
//........................................................................

extern const ServiceInfo aUpdateAccessRootSI;
extern const ServiceInfo aSetNodeUpdateRootSI;

//========================================================================
//= OUpdateAccessRoot
//========================================================================
/** concret implementation for the root of an updatable config sub tree
*/
class OUpdateAccessRoot
			:public ServiceComponentImpl
			,public HierarchyUpdateRootGroup
			
{
public:
	/** ctor.
		@param		_pTree				the subtree the object represents
		@param		_rName				the path to the parent of the sub tree (as always, we have no redundance, so the
										this is really the path to the parent, not to the node itself. The name of the node
										itself is implicitly given in <arg>_pTree</arg>.
		@param		_rUpdater			the object which implments the updates on the tree
		@param		_rTemplateFactory	factory for instantiating node templates
	*/
	OUpdateAccessRoot(
		TreeRef _pTree,
		const ConfigurationName& _rName,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		IConfigBroadcaster& rEventSource,
		const ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >& _rxConverter
	);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
	virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);
	::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }

protected:
    virtual void SAL_CALL disposing();

	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

protected:
	// HierarchyUpdateBase overridables
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath);
};

//========================================================================
//= OUpdateAccessElement
//========================================================================
/** concret implementation for (non-root) elements of an updatable config sub tree
*/
class OUpdateAccessElement
			:public ServiceComponentImpl
			,public HierarchyUpdateElementGroup
{
public:
	/** ctor.
		@param			_pUpdateRoot		the top level node for the update access. The element will hold a hard ref to
											this  object, so it can't die until the element itself does.<BR>
											Must not be NULL.
		@param			_pModifications		the changes which are represented by the object
		@param			_rRelativeLocation	tree path describing the relative location of the element (including the
											element node itself, which implies that the path must not be empty !)
	*/
	OUpdateAccessElement(HierarchyUpdateRoot* _pUpdateRoot, SubtreeChange* _pModifications, const TreePath& _rRelativeLocation);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
	virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);
	::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }
protected:
	// ServiceComponentImpl overridables
	virtual void SAL_CALL disposing();

	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

	// HierarchyUpdateBase overridables
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath);
};

//========================================================================
//= OInnerSetNode
//========================================================================
/** a class representing an inner (non-root, non-leaf) node of an update sub tree, which in addition is a set node
*/
class OInnerSetNode
				:public ServiceComponentImpl
				,public OSetNodeUpdateElement
{
public:
	OInnerSetNode(HierarchyUpdateRoot* _pUpdateRoot, SubtreeChange* _pModifications, const TreePath& _rRelativeLocation);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
	virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);
	::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }
protected:
	// ServiceComponentImpl overridables
	virtual void SAL_CALL disposing();

	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

	// HierarchyUpdateBase overridables
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath);

	// OSetNode overridables
	virtual OFloatableNode* createFloatingElement(const TreePath& _rCompleteThisPath, ::std::auto_ptr<ISubtree> _rFloatingSubtree);
};

//========================================================================
//= OInnerSetElement
//========================================================================
/** a class representing an inner (non-root, non-leaf) node of an update tree, and in
	addition is a floatable node, which means it is a direct descendant of a set node.
*/
class OInnerSetElement
		:public ServiceComponentImpl
		,public OFloatableGroup
{
public:
	OInnerSetElement(
		const ::rtl::OUString& _rTemplateName,
		HierarchyUpdateRoot* _pUpdateRoot,
		const TreePath& _rSetPath,
		::std::auto_ptr<ISubtree> _pTemplateInstance
	);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
	virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);
	::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }

protected:
	// ServiceComponentImpl overridables
	virtual void SAL_CALL disposing();

	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

	// HierarchyUpdateBase overridables
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath);
};

//========================================================================
//= OSetElementAndNode
//========================================================================
/** a class representing an inner (non-root, non-leaf) node of an update tree, and in
	addition is a floatable node, which means it is a direct descendant of a set node.
*/
class OSetElementAndNode
		:public ServiceComponentImpl
		,public OFloatableSetNode
{
public:
	/** ctor
		@param		_rElementTemplateName	the name of the template the node was created from
		@param		_pTemplateInstance		the (floating) node represented by the object
		@param		_rSetPath				path to the set the node will belong to (if inserted), including
											the set node itself, not including the template node
		@param		_rContainerTemplateName	template name the container uses for creating it's own children
		@param		_pUpdateRoot			root of the update tree where the (currently floating) node should
											be a part of
	*/
	OSetElementAndNode(
			const ::rtl::OUString& _rElementTemplateName,
			::std::auto_ptr<ISubtree> _pTemplateInstance,
			const TreePath& _rSetPath,
			const ::rtl::OUString& _rContainerTemplateName,
			HierarchyUpdateRoot* _pUpdateRoot
	);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
	virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);
	::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }
protected:
    virtual void SAL_CALL disposing();

	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

	// HierarchyUpdateBase overridables
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath);

	// OSetNode overridables
	virtual OFloatableNode* createFloatingElement(const TreePath& _rCompleteThisPath, ::std::auto_ptr<ISubtree> _rFloatingSubtree);
};

//========================================================================
//= OSetElementAndNode
//========================================================================
/** a concret class representing the root of an update tree, in additionn beeing a set node.
*/
class OSetNodeUpdateRoot
		:public ServiceComponentImpl
		,public ORootSetNode
{
protected:
	/** ctor used by derived classes implementing a specialized service
		@param		pDerivedServiceInfo	service info for a specialized implementation derived from this one
		@param		_pTree				the subtree the object represents
		@param		_rName				the path to the parent of the sub tree (as always, we have no redundance, so the
										this is really the path to the parent, not to the node itself. The name of the node
										itself is implicitly given in <arg>_pTree</arg>.
		@param		_rUpdater			the object which implments the updates on the tree
		@param		_rTemplateFactory	factory for instantiating node templates
		@param		_rContainerTemplateName		name of the template which should be used for creating new children
												of the node.

	*/
	OSetNodeUpdateRoot(
		ServiceInfo const* pDerivedServiceInfo,
		TreeRef _pTree,
		const ConfigurationName& _rName,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		const ::rtl::OUString& _rContainerTemplateName,
		IConfigBroadcaster& rEventSource_,
		const ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >& _rxConverter
		);

public:
	/** ctor
		@param		_pTree				the subtree the object represents
		@param		_rName				the path to the parent of the sub tree (as always, we have no redundance, so the
										this is really the path to the parent, not to the node itself. The name of the node
										itself is implicitly given in <arg>_pTree</arg>.
		@param		_rUpdater			the object which implments the updates on the tree
		@param		_rTemplateFactory	factory for instantiating node templates
		@param		_rContainerTemplateName		name of the template which should be used for creating new children
												of the node.

	*/
	OSetNodeUpdateRoot(
		TreeRef _pTree,
		const ConfigurationName& _rName,
		const ::vos::ORef< IRefCountedTreeProvider >& _rUpdater,
		const ::vos::ORef< IRefCountedTemplateProvider >& _rTemplateFactory,
		const ::rtl::OUString& _rContainerTemplateName,
		IConfigBroadcaster& rEventSource_,
		const ::com::sun::star::uno::Reference< ::com::sun::star::script::XTypeConverter >& _rxConverter
		);

	// XInterface
	::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL acquire() throw() { ServiceComponentImpl::acquire(); }
	virtual void SAL_CALL release() throw() { ServiceComponentImpl::release(); }

	// XTypeProvider
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() throw (::com::sun::star::uno::RuntimeException);
	::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() throw (::com::sun::star::uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }
protected:
    virtual void SAL_CALL disposing();

	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

	// HierarchyUpdateBase overridables
	virtual HierarchyUpdateBase* createElement(SubtreeChange* _pModifications, const TreePath& _rPath);

	// OSetNode overridables
	virtual OFloatableNode* createFloatingElement(const TreePath& _rCompleteThisPath, ::std::auto_ptr<ISubtree> _rFloatingSubtree);
};

//........................................................................
} // namespace configmgr
//........................................................................

#endif // _CONFIGMGR_API_UPDATEACCESSIMPL_HXX_


