/*************************************************************************
 *
 *  $RCSfile: dataimport.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_DATA_IMPORT_HXX_
#define CONFIGMGR_API_DATA_IMPORT_HXX_

#ifndef _CPPUHELPER_IMPLBASE4_HXX_
#include <cppuhelper/implbase4.hxx>
#endif

#ifndef _COM_SUN_STAR_LANG_XINITIALIZATION_HPP_
#include <com/sun/star/lang/XInitialization.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XSERVICEINFO_HPP_
#include <com/sun/star/lang/XServiceInfo.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XDATAIMPORTER_HDL_
#include <com/sun/star/io/XDataImporter.hpp>
#endif

#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XEXTENDEDDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XExtendedDocumentHandler.hpp>
#endif

#ifndef _VOS_THREAD_HXX_
#include <vos/thread.hxx>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XPARSER_HPP_
#include <com/sun/star/xml/sax/XParser.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_SAXPARSEEXCEPTION_HPP_
#include <com/sun/star/xml/sax/SAXParseException.hpp>
#endif

#include <com/sun/star/container/XNameAccess.hpp>

#include "xmlformater.hxx"

namespace configmgr
{
	// borrow from confsvccomponent.hxx
	typedef sal_Char const * AsciiServiceName;
	struct ServiceInfo
	{
		AsciiServiceName implementationName;
		AsciiServiceName const * serviceNames;
	};


	class ConfigurationProvider;
	namespace uno  = ::com::sun::star::uno;
	namespace lang = ::com::sun::star::lang;
	namespace io  = ::com::sun::star::io;
	// namespace util = ::com::sun::star::util;



class ParseThread: public vos::OThread
{
	uno::Reference< sax::XParser > m_xParser;
	uno::Reference< io::XDataTransferEventListener > m_xListener;
	uno::Reference< io::XInputStream > m_xInputStream;
	uno::Reference< io::XOutputStream > m_xOutputStream;
	bool m_bCancel;

	virtual void SAL_CALL run();

public:
	ParseThread( const uno::Reference< sax::XParser > &xParser,
				 const uno::Reference< io::XDataTransferEventListener >& xListener,
				 const uno::Reference< io::XInputStream> &xInputStream,
				 const uno::Reference< io::XOutputStream> &xOutputStream) 
			:m_xParser(xParser), m_xListener(xListener), m_xInputStream(xInputStream),
			 m_xOutputStream(xOutputStream)
		{
		}
	void setCancel(bool bCancel)
		{
			m_bCancel = bCancel;
			m_xOutputStream->closeOutput();
			m_xInputStream->closeInput();
		}

	virtual ~ParseThread()
		{
			delete this;
		}
};


	class ODataImport : public ::cppu::WeakImplHelper4<lang::XInitialization,lang::XServiceInfo,io::XDataImporter, sax::XParser>
	{
		osl::Mutex m_aMutex;

	public:
		static const ServiceInfo staticServiceInfo;
	protected:
		uno::Reference< lang::XMultiServiceFactory > m_xMSF; // the service provider used for creating the instance
		uno::Reference< sax::XParser > m_xParser;
		rtl::OUString m_sPath;
		rtl::OUString m_sFilename;

		uno::Reference<script::XTypeConverter> m_xConverter;
		bool m_bCancel;
		ParseThread *m_pThread;

	public:
		ODataImport(const uno::Reference< lang::XMultiServiceFactory > &xMSF)
				:m_xMSF(xMSF),
				 m_aMutex(),
				 m_bCancel(false)
			{}
		
		// XInitialization
		virtual void SAL_CALL initialize( const uno::Sequence< uno::Any >& aArguments ) throw(uno::Exception, uno::RuntimeException);

		// XServiceInfo
		virtual ::rtl::OUString SAL_CALL getImplementationName(  ) throw(uno::RuntimeException);
		virtual sal_Bool SAL_CALL supportsService( const ::rtl::OUString& ServiceName ) throw(uno::RuntimeException);
		virtual uno::Sequence< ::rtl::OUString > SAL_CALL getSupportedServiceNames(  ) throw(uno::RuntimeException);

		uno::Sequence< rtl::OUString > getServiceNames(ServiceInfo const* pInfo  ) throw(uno::RuntimeException);
		sal_Int32 countServices(ServiceInfo const* aInfo);

		// XDataImporter
		virtual void SAL_CALL importData( const uno::Reference< io::XActiveDataSource >& xActiveSource,
										  const uno::Reference< lang::XComponent >& xComponent,
										  const uno::Reference< io::XDataTransferEventListener >& xListener )
			throw(uno::RuntimeException);
		virtual void SAL_CALL cancel() throw(uno::RuntimeException);

	// XParser
    virtual void SAL_CALL parseStream( const sax::InputSource& strucInputSource ) 
		throw(sax::SAXException, io::IOException, uno::RuntimeException)
		{
			m_xParser->parseStream(strucInputSource);
		}

    virtual void SAL_CALL setDocumentHandler( const uno::Reference< sax::XDocumentHandler >& xHandler )
		throw(uno::RuntimeException)
		{
			m_xParser->setDocumentHandler(xHandler);
		}
	
    virtual void SAL_CALL setErrorHandler( const uno::Reference< sax::XErrorHandler >& xHandler ) 
		throw(uno::RuntimeException)
		{
			m_xParser->setErrorHandler(xHandler);
		}
	
    virtual void SAL_CALL setDTDHandler( const uno::Reference< sax::XDTDHandler >& xHandler ) 
		throw(uno::RuntimeException)
		{
			m_xParser->setDTDHandler(xHandler);
		}
	
    virtual void SAL_CALL setEntityResolver( const uno::Reference< sax::XEntityResolver >& xResolver ) 
		throw(uno::RuntimeException)
		{
			m_xParser->setEntityResolver(xResolver);
		}
	
	virtual void SAL_CALL setLocale( const lang::Locale& locale ) 
		throw(uno::RuntimeException)
		{
			m_xParser->setLocale(locale);
		}
	};

} // namespace configmgr


#endif // CONFIGMGR_API_DATA_IMPORT_HXX_
