/*************************************************************************
 *
 *  $RCSfile: nodepropset.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_PROPERTIES_HXX_
#define CONFIGMGR_API_PROPERTIES_HXX_

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XMULTIPROPERTYSET_HPP_
#include <com/sun/star/beans/XMultiPropertySet.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSTATE_HPP_
#include <com/sun/star/beans/XPropertyState.hpp>
#endif

#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx>
#endif

#ifndef _CPPUHELPER_INTERFACECONTAINER_HXX_
#include <cppuhelper/interfacecontainer.hxx>
#endif

#ifndef CONFIGMGR_APITYPES_HXX_
#include "confapitypes.hxx"
#endif
#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif
#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif

namespace com { namespace sun { namespace star {
	namespace script { class XTypeConverter; }
} } }

namespace configmgr
{
	namespace css	= ::com::sun::star;
	namespace uno		= css::uno;
	namespace lang		= css::lang;
	namespace beans		= css::beans;
	using ::rtl::OUString;

	class ConfigurationName;


/////////////////////////////////////////////////////////////////////////////////////////////
class PropertyEventDispatcher
{
public:
	typedef uno::Reference< beans::XVetoableChangeListener > ConstraintListener;
	typedef uno::Reference< beans::XPropertyChangeListener > BoundListener;
	typedef uno::Reference< beans::XPropertiesChangeListener > MultiListener;
	typedef uno::Sequence< OUString > OUStringList;

	typedef std::vector<beans::PropertyChangeEvent> PropertyChanges;
public:
	typedef cppu::OBroadcastHelper BroadcastHelper;
	PropertyEventDispatcher(BroadcastHelper& rBHelper_) 
		: m_rBroadcastHelper(rBHelper_) 
		, m_aConstraintListeners(rBHelper_.rMutex)
		, m_aBoundListeners(rBHelper_.rMutex)
	{}

public:
	osl::Mutex& mutex() const { return m_rBroadcastHelper.rMutex; }
	// disposing & lifetime
	bool PropertyEventDispatcher::checkAlive(beans::XPropertySet* pSource);
	void dispose(beans::XPropertySet* pSource);
public:
	// listener registration
    void addListener( const OUString& PropertyName, const ConstraintListener& aListener );
    void addListener( const OUString& aPropertyName, const BoundListener& xListener ); 
    void addListener( const OUStringList& aPropertyNames, const MultiListener& xListener );

	// listener deregistration
	void removeListener( const OUString& PropertyName, const ConstraintListener& aListener ); 
    void removeListener( const OUString& aPropertyName, const BoundListener& aListener );
    void removeListener( const MultiListener& Listener ); 

	// event firing
	void fireApprovals(PropertyChanges const& rEvents) throw(beans::PropertyVetoException,uno::RuntimeException);
	void fireChanges  (PropertyChanges const& rEvents) throw(uno::RuntimeException);

	void fireApproval(beans::PropertyChangeEvent const& rEvent) throw(beans::PropertyVetoException,uno::RuntimeException);
	void fireChange  (beans::PropertyChangeEvent const& rEvent) throw(uno::RuntimeException);
private:
	void doFireApprovals(beans::PropertyChangeEvent const* pEvents, sal_Int32 nCount) throw(beans::PropertyVetoException,uno::RuntimeException);
	void doFireChanges  (beans::PropertyChangeEvent const* pEvents, sal_Int32 nCount) throw(uno::RuntimeException);

private:
	struct Yash { // yet another string hash
		sal_Int32 operator() (const OUString& rStr) const {return rStr.hashCode();}
	};
	struct YasEq {  // yet another string-equality
		bool operator() (const OUString& lhs,const OUString& rhs) const {return sal_False != lhs.equals(rhs);}
	};
	typedef cppu::OMultiTypeInterfaceContainerHelperVar< OUString, Yash, YasEq > ListenersByName;

private:
	BroadcastHelper& m_rBroadcastHelper;

	ListenersByName m_aConstraintListeners;
	ListenersByName m_aBoundListeners;
};
/////////////////////////////////////////////////////////////////////////////////////////////

class NodePropertySet 
	: public beans::XPropertySet
	, public beans::XMultiPropertySet
	, public beans::XPropertyState
{
public:
	// Types
	enum Options
	{
		noPropertyState	 = 0,
		asReadOnly		 = 0,
		withPropertyState	= 1,
		withUpdates			= 2
	};
	friend Options operator |(Options l, Options r) { return Options( (l+0) | (r+0) ); }

	typedef cppu::OBroadcastHelper BroadcastHelper;

	typedef std::vector<beans::PropertyChangeEvent> PropertyChanges;
	typedef std::vector<ValueChange>				ValueChanges;
public:
// Constructors & Destructors
	NodePropertySet(BroadcastHelper& rBHelper_, Options opt);
	~NodePropertySet();

// XPropertySet & XMultiPropertySet

	virtual uno::Reference< beans::XPropertySetInfo > SAL_CALL 
		getPropertySetInfo(  ) 
			throw(uno::RuntimeException);

	// XPropertySet
    virtual void SAL_CALL
		setPropertyValue( const OUString& aPropertyName, const uno::Any& aValue ) 
			throw(beans::UnknownPropertyException, beans::PropertyVetoException, lang::IllegalArgumentException, 
				  lang::WrappedTargetException, uno::RuntimeException);

 	// XMultiPropertySet
   virtual void SAL_CALL 
		setPropertyValues( const uno::Sequence< OUString >& PropertyNames, const uno::Sequence< uno::Any >& Values ) 
			throw(beans::PropertyVetoException, lang::IllegalArgumentException, 
				  lang::WrappedTargetException, uno::RuntimeException);

	// XPropertySet
    virtual uno::Any SAL_CALL 
		getPropertyValue( const OUString& PropertyName ) 
			throw(beans::UnknownPropertyException, lang::WrappedTargetException, uno::RuntimeException);

 	// XMultiPropertySet
    virtual uno::Sequence< uno::Any > SAL_CALL 
		getPropertyValues( const uno::Sequence< OUString >& aPropertyNames ) 
			throw(uno::RuntimeException);

	// XPropertySet
    virtual void SAL_CALL 
		addPropertyChangeListener( const OUString& aPropertyName, const uno::Reference< beans::XPropertyChangeListener >& xListener ) 
			throw(beans::UnknownPropertyException, lang::WrappedTargetException, uno::RuntimeException);

 	// XMultiPropertySet
    virtual void SAL_CALL 
		addPropertiesChangeListener( const uno::Sequence< OUString >& aPropertyNames, const uno::Reference< beans::XPropertiesChangeListener >& xListener ) 
			throw(uno::RuntimeException);

	// XPropertySet
    virtual void SAL_CALL 
		removePropertyChangeListener( const OUString& aPropertyName, const uno::Reference< beans::XPropertyChangeListener >& aListener ) 
			throw(beans::UnknownPropertyException, lang::WrappedTargetException, uno::RuntimeException);

 	// XMultiPropertySet
    virtual void SAL_CALL 
		removePropertiesChangeListener( const uno::Reference< beans::XPropertiesChangeListener >& Listener ) 
			throw(uno::RuntimeException);

	// XPropertySet
    virtual void SAL_CALL 
		addVetoableChangeListener( const OUString& PropertyName, const uno::Reference< beans::XVetoableChangeListener >& aListener ) 
			throw(beans::UnknownPropertyException, lang::WrappedTargetException, uno::RuntimeException);
  
	virtual void SAL_CALL 
		removeVetoableChangeListener( const OUString& PropertyName, const uno::Reference< beans::XVetoableChangeListener >& aListener ) 
			throw(beans::UnknownPropertyException, lang::WrappedTargetException, uno::RuntimeException);

 	// XMultiPropertySet
    virtual void SAL_CALL 
		firePropertiesChangeEvent( const uno::Sequence< OUString >& aPropertyNames, const uno::Reference< beans::XPropertiesChangeListener >& xListener ) 
			throw(uno::RuntimeException);

// XPropertyState
    virtual beans::PropertyState SAL_CALL 
		getPropertyState( const OUString& PropertyName ) 
			throw(beans::UnknownPropertyException, uno::RuntimeException);

    virtual uno::Sequence< beans::PropertyState > SAL_CALL 
		getPropertyStates( const uno::Sequence< OUString >& aPropertyName ) 
			throw(beans::UnknownPropertyException, uno::RuntimeException);

    virtual void SAL_CALL 
		setPropertyToDefault( const OUString& PropertyName ) 
			throw(beans::UnknownPropertyException, uno::RuntimeException);

    virtual uno::Any SAL_CALL 
		getPropertyDefault( const OUString& aPropertyName ) 
			throw(beans::UnknownPropertyException, lang::WrappedTargetException, uno::RuntimeException);

protected:
// XInterface and XTypeProvider implementation support

	/** query for the supported interfaces. 
		XFastPropertySet may or may not be available
		XPropertyState may or may not be available
		@returns <B>true</B> if the interface is provided
	*/
	sal_Bool queryPropertyInterfaces(uno::Any& xOut, uno::Type const& rType);
	/** add the supported interface types
		@returns the number of types appended
	*/
	sal_Int32 addPropertyTypes(uno::Sequence< uno::Type >& rTypes);
	
protected:
	virtual void checkAlive();
	virtual void SAL_CALL disposing();
protected:
	/// access to the objects mutex
	osl::Mutex& mutex() const { return m_aDispatcher.mutex(); }

	// working as an event dispatcher
	/**
	 * Fire PropertyVetoEvents for the updates to the local tree. 
	 * The changes should not yet be done at this time.
	 */
	void fireVetoableChanges(ValueChanges const& rChanges) throw(beans::PropertyVetoException,uno::RuntimeException);

	/**
	 * Fire PropertyChangeEvents for the updates to the local tree. 
	 * The changes should already be done at this time.
	 */
	void firePropertyChanges(ValueChanges const& rChanges) throw(uno::RuntimeException);

protected:
// New overrideables functions
	virtual uno::Reference< css::script::XTypeConverter > getTypeConverter() const = 0;
	
	/**
	 * Ensure that the default values for this node and its children are loaded
	 * @return 
	 *    false, if defaults are not available
	 *    true, if defaults have been loaded sucessfully
	 */
	virtual bool loadDefaults() throw (uno::RuntimeException) = 0;

	/**
	 * Create an any for a child element that is not a simple value
	 */
	virtual uno::Any getChildAccess(TreeRef aChildNode) throw (uno::RuntimeException) = 0;

	/**
	 * Execute the updates to the local tree. The Changes are always valid. 
	 * <B>The types are correct you need't test them. </B><BR>
	 * First, approve the changes with any VetoListeners.<BR>
	 * Second, apply the changes locally.<BR>
	 * Then broadcast the changes to local listeners.<BR>
	 */
	virtual void executeLocalChanges(ValueChanges& rChange)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException) = 0;

// Associated helper functions
	void ensureDefaults() throw (uno::RuntimeException);

	/**
	 * Create an any for any child element 
	 */
	uno::Any makeChildValue(NodeRef aChildNode);

	/**
	 * Execute the update to the local tree. The Change is always valid. 
	 * <B>The types are correct you need't test them. </B><BR>
	 * First, approve the change with any VetoListeners.<BR>
	 * Second, apply the change locally.<BR>
	 * Then broadcast the change to local listeners.<BR>
	 *
	 * @see excecuteLocalChanges
	 */
	void executeLocalChange(ValueChange& rChange)
		throw (beans::PropertyVetoException, uno::Exception, uno::RuntimeException)
	{ ValueChanges aChanges(1,rChange); executeLocalChanges( aChanges ); rChange = aChanges.front(); }

	/**
	 * Get the ISubtree represented by this object. 
	 */
	virtual TreeRef getThisTree() const = 0;

	/**
	 * Get the ISynchronizedData protecting this object's master tree. 
	 */
	virtual ISynchronizedData const* getTreeSync() const = 0;
	/**
	 * Get the ISynchronizedData protecting this objects changes. 
	 */
	virtual ISynchronizedData * getUpdateSync() = 0;

private:
	NodeRef getPropertyNode(OUString const& aName);
	NodeRef getValidPropertySetNode(OUString const& aName);
	NodeRef getValidPropertyStateNode(OUString const& aName);

	sal_Int32 buildEvents(PropertyChanges& rEvents, ValueChanges const& rChanges, bool forVeto);
private:
	PropertyEventDispatcher m_aDispatcher;

	Options m_options;
};

//--------------------------------------------------------------------------

class NodePropertySetInfo 
: public cppu::WeakImplHelper1< beans::XPropertySetInfo >
{
	TreeRef m_aTree;
	ISynchronizedData const* m_pLock;
	NodePropertySet::Options m_options;
public:
	NodePropertySetInfo( TreeRef aTree, ISynchronizedData const* pLock, NodePropertySet::Options options)
		: m_aTree(aTree), m_pLock(pLock), m_options(options)
	{}

	~NodePropertySetInfo() {}

	virtual uno::Sequence< beans::Property > SAL_CALL 
		getProperties(void)
			throw(uno::RuntimeException);

    virtual beans::Property SAL_CALL 
		getPropertyByName(const OUString& sPropertyName) 
			throw(beans::UnknownPropertyException, uno::RuntimeException);

    virtual sal_Bool SAL_CALL 
		hasPropertyByName(const OUString& sPropertyName)
			throw(uno::RuntimeException);
};

//--------------------------------------------------------------------------

}
#endif // CONFIGMGR_API_PROPERTIES_HXX_


