/*************************************************************************
 *
 *  $RCSfile: readaccessimpl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef CONFIGMGR_API_READ_ACCESS_IMPL_HXX_
#define CONFIGMGR_API_READ_ACCESS_IMPL_HXX_

#ifndef _COM_SUN_STAR_UTIL_XSTRINGESCAPE_HPP_
#include <com/sun/star/util/XStringEscape.hpp>
#endif

#ifndef CONFIGMGR_API_SVCCOMPONENT_HXX_
#include "confsvccomponent.hxx"
#endif

#ifndef CONFIGMGR_API_ACCESS_HXX_
#include "confaccess.hxx"
#endif
#ifndef CONFIGMGR_API_READACCESS_HXX_
#include "confreadaccess.hxx"
#endif

namespace configmgr
{
	class ConfigurationProvider;
	namespace uno  = ::com::sun::star::uno;
	namespace lang = ::com::sun::star::lang;
	namespace util = ::com::sun::star::util;

extern const ServiceInfo aReadAccessSI;


class OReadOnlySetNode
: public ServiceComponentImpl
, public HierarchyReadAccess
, public util::XStringEscape
{
public:
	static ServiceInfo const& staticServiceInfo;
public:
	OReadOnlySetNode(IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, OUString aBasePath = OUString());
	OReadOnlySetNode(IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath = OUString());
	OReadOnlySetNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode);
	OReadOnlySetNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath);
	OReadOnlySetNode(HierarchyReadAccess const& aAncestorElement, ParentElement);

		/// late ctor
	virtual void Init();

public:
	// XInterface implementation
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);
	void SAL_CALL acquire() throw ();
	void SAL_CALL release() throw ();

	// XTypeProvider implementation
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }

	// XStringEscape
	OUString SAL_CALL escapeString( const OUString& aString ) throw(lang::IllegalArgumentException, uno::RuntimeException);
    OUString SAL_CALL unescapeString( const OUString& aEscapedString ) throw(lang::IllegalArgumentException, uno::RuntimeException);

protected:
    virtual void SAL_CALL disposing();
		
	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

};

class OReadOnlyGroupNode
: public ServiceComponentImpl
, public PropsetReadAccess
{
public:
	static ServiceInfo const& staticServiceInfo;
public:
	OReadOnlyGroupNode(IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, OUString aBasePath = OUString(), bool withDefaults = true);
	OReadOnlyGroupNode(IConfigBroadcaster& rSource, ISynchronizedData const * pSync, TreeRef pNode, TreePath const& pPath, OUString aBasePath = OUString(), bool withDefaults = true);
	OReadOnlyGroupNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreeRef pParentNode, bool withDefaults = true);
	OReadOnlyGroupNode(HierarchyReadAccess const& aAncestorElement, TreeRef pNode, TreePath const& pPath, bool withDefaults = true);
	OReadOnlyGroupNode(HierarchyReadAccess const& aAncestorElement, ParentElement, bool withDefaults = true);

		/// late ctor
	virtual void Init();

public:
	// XInterface implementation
	uno::Any SAL_CALL queryInterface(uno::Type const& rType) throw (uno::RuntimeException);
	void SAL_CALL acquire() throw ();
	void SAL_CALL release() throw ();

	// XTypeProvider implementation
	uno::Sequence< uno::Type > SAL_CALL getTypes() throw (uno::RuntimeException);

	// XComponent desambuigation and forwarding
    virtual void SAL_CALL dispose(  ) throw(uno::RuntimeException)
	{ ServiceComponentImpl::dispose(); }
    virtual void SAL_CALL addEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::addEventListener(xListener); }
    virtual void SAL_CALL removeEventListener( const uno::Reference< lang::XEventListener >& xListener ) 
		throw(uno::RuntimeException)
	{ ServiceComponentImpl::removeEventListener(xListener); }

protected:
    virtual void SAL_CALL disposing();
		
	virtual void checkAlive() throw (uno::RuntimeException)
		{ ServiceComponentImpl::checkAlive(); }

};

} // namespace configmgr


#endif // CONFIGMGR_API_READ_ACCESS_IMPL_HXX_



