/*************************************************************************
 *
 *  $RCSfile: treecache.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_TREECACHE_HXX_
#define _CONFIGMGR_TREECACHE_HXX_

#ifndef CONFIGMGR_CMTREEMODEL_HXX
#include "cmtreemodel.hxx"
#endif
#ifndef _CMSET_HXX
#include "cmtree.hxx"
#endif
#ifndef _CONFIGMGR_SESSION_CONFIGSESSION_HXX_
#include "configsession.hxx"
#endif
#ifndef _COM_SUN_STAR_XML_SAX_XDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XDocumentHandler.hpp>
#endif
#ifndef _CONFIGMGR_XMLTREEBUILDER_HXX
#include "xmltreebuilder.hxx"
#endif
#ifndef _VOS_CONDITN_HXX_
#include <vos/conditn.hxx>
#endif

namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace script
			{
				class XTypeConverter;
			}
		}
	}
}

namespace configmgr
{
	class OTreeAccessor;
	class IConfigSession;

	namespace staruno	= ::com::sun::star::uno;
	namespace xmlsax	= ::com::sun::star::xml::sax;
	namespace script	= ::com::sun::star::script;

	class OUpdateCallback;
	class IConfigSession;
	struct StatusInfo;

	class TreeManager;
	//==========================================================================
	//= OUpdateCallback
	//==========================================================================
	class OUpdateCallback : public IDataRequestCallback
	{
		oslInterlockedCount		m_refCount;
		//	XMLDataReader		m_aDataReader;
		TreeManager*			m_pCacheManager;
		TreeChangeList			m_aChanges;
		::vos::ORef< XMLTreeChangeListBuilder >
								m_xBuilder;
		::vos::OCondition		m_aDoneCond;

		StatusInfo				m_aServerStatus;	/// the status as returned from the server, @seealso <method>done</method>
		sal_Int32				m_nConnError;		/// the error as returned from the session, @seealso <method>failed</method>

	public:
		StatusInfo		getStatus() const { return m_aServerStatus; }
		sal_Int32		getConnError() const { return m_nConnError; }

	protected:
		~OUpdateCallback() { }

	public:
		OUpdateCallback(const ConfigurationName& _rRootLoc, TreeManager* _pCM);

		TreeChangeList* getChanges() { return &m_aChanges; }

		// IInterface
		virtual void SAL_CALL acquire(  ) throw () {osl_incrementInterlockedCount( &m_refCount );}
		virtual void SAL_CALL release(  ) throw () 
		{
			if (! osl_decrementInterlockedCount( &m_refCount ))
				delete this;
		}

		/** wait until the request is not (no matter if successfull or not)
			@param		_pTimeout	time out for the wait operation. If NULL, a default will be used.
			@return		sal_True, if successfull
			@seealso	getStatus
			@seealso	getConnError
		*/
		sal_Bool waitForUpdate(TimeValue* _pTimeout = NULL);

		// IRequestCallback
		virtual		void acknowledged(sal_Int32 _rTransId) {}
		virtual		void failed(sal_Int32 _nErrorCode);
		virtual		void done(const StatusInfo& _rStatus);

		// IDataRequestCallback
		virtual	staruno::Reference< xmlsax::XDocumentHandler > getDataReader();

		XMLTreeChangeListBuilder* getTreeBuilder() { return m_xBuilder.getBodyPtr(); }
	};

	//==========================================================================
	//= TreeManager
	//==========================================================================
	class TreeManager	:public ITreeProvider
						,public ITreeNotifier
						,public ISessionListener
						,public ITemplateProvider
	{
		oslInterlockedCount		m_refCount;
		Tree					m_aTree;
		Tree					m_aTemplates;

		ITreeListener*			m_pListener;

		IConfigSession*			m_pSession;
		::vos::ORef< OUpdateCallback > m_xDataReqCb;
		uno::Reference< script::XTypeConverter > m_xTypeConverter;

	protected:
		// ref counted, that's why no public dtor
		~TreeManager();

	public:
		/** ctor
			@param		_pSession			the session to work on
			@param		_rxTypeConverter	object to be used for type conversion
		*/
		TreeManager(
			IConfigSession*	_pSession,
			const uno::Reference< script::XTypeConverter >& _rxTypeConverter);

		// IInterface
		virtual void SAL_CALL acquire(  ) throw () {osl_incrementInterlockedCount( &m_refCount );}
		virtual void SAL_CALL release(  ) throw (); 

		/** gets a node given by it's path. Basicly, this means
			that the node is fetch from the cache
			system into it's cache.
			@param			_rSubtreePath	the path to the node in URI notation.
		*/
		virtual ISubtree const*	getSubtree( OUString const& _rSubtreePath ) const;
		/** requests a node given by it's path. Basicly, this means
			that the node is fetch from the cache when it contains it else it ask the server
			system into it's cache.
			@param			_rSubtreePath	the path to the node in URI notation.
			@param			_nMinLevels		indicates how many tree levels under the node determined by <arg>_rSubtreePath</arg>
											should be loaded
		*/
		virtual ISubtree *		requestSubtree( OUString const& _rSubtreePath, sal_Int16 _nMinLevels = ALL_LEVELS);
		virtual const INode *	getNode(const OUString& _rPath);

		virtual void updateTree(TreeChangeList& aChanges) throw (starlang::WrappedTargetException, uno::RuntimeException);

		// ITreeNotifier
		virtual void setListener(ITreeListener* _pListener) { m_pListener = _pListener; }

		// ISessionListener
		virtual void	nodeUpdated(const ::rtl::OUString& _rNodePath);
		virtual void	nodeDeleted(const ::rtl::OUString& _rNodePath);
		virtual void	nodeAdded(const ::rtl::OUString& _rNodePath);

		// ITemplateProvider
		virtual ::std::auto_ptr<INode> createInstance(const ::rtl::OUString& _rTemplateName) throw (uno::Exception);

		uno::Reference< script::XTypeConverter > getTypeConverter( ){ return m_xTypeConverter; }

	protected:
		// ISyncronizedData
		virtual void acquireReadAccess() const;
		virtual void releaseReadAccess() const;
		virtual void acquireWriteAccess();
		virtual void releaseWriteAccess();

	protected:
		/** load a subtree of a tree, insert it into the tree. The method will determine if getNode
			can be used or an openNode is neccessary.
			<BR>
			The method has to be called with an acquired read guard for the given tree
			<BR>
			The newly read node will be inserted into the given tree.
			<BR>
			The parameter _rClearBeforeWrite is neccessary as long as we cant nest a write guard into a read guard.
		*/
		Subtree* implLoadSubtree(const OUString& _rSubtreePath, sal_Int16 _nMinLevels, Tree* _pTree, OReadSynchronized& _rClearBeforeWrite, StatusInfo& /* [out] */ _rStatus );

		void ensureTemplate(const ::rtl::OUString& _rName) throw (uno::Exception);

		ConfigurationName completeTemplateLocation(const ::rtl::OUString& _rLogicalTemplateName);

		std::auto_ptr<Subtree> getNodeSynchron( const ::rtl::OUString& _rNodePath, StatusInfo& /* [out] */ _rStatus );
		std::auto_ptr<Subtree> openNodeSynchron( const ::rtl::OUString& _rNodePath, sal_Int32 _nLevel, StatusInfo& /* [out] */ _rStatus );

		void fireChanges(TreeChangeList const& aChangeTree, sal_Bool _bError);
	};
}

#endif // _CONFIGMGR_TREECACHE_HXX_

