/*************************************************************************
 *
 *  $RCSfile: xmlformater.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_XMLFORMATER_HXX
#define _CONFIGMGR_XMLFORMATER_HXX

#include <stl/vector>

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#include <com/sun/star/xml/sax/XExtendedDocumentHandler.hpp>
#include <com/sun/star/io/XOutputStream.hpp>

#include <rtl/ustring.hxx>
#include <com/sun/star/uno/Any.hxx>
#include <com/sun/star/uno/Sequence.hxx>

#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#include "configsession.hxx"
#include "cmtreemodel.hxx"

#ifndef _CONFIGMGR_STRDECL_HXX_
#include "strdecl.hxx"
#endif

namespace configmgr
{
	namespace star = com::sun::star;
	namespace uno = com::sun::star::uno;
	namespace sax = com::sun::star::xml::sax;
	namespace script = com::sun::star::script;

	using ::rtl::OUString;

	class AnyToXMLDocument
	{
		uno::Reference < sax::XDocumentHandler >	m_xHandler;
		uno::Reference< script::XTypeConverter >	m_xTypeConverter;
	public:
		AnyToXMLDocument(){}
		
		void setConverter(const uno::Reference< script::XTypeConverter > &_xConverter)
			{m_xTypeConverter = _xConverter;}
	
		void setHandler(const uno::Reference < sax::XDocumentHandler > &_xHandler)
			{m_xHandler = _xHandler;}
		
		// Value to XML (String) conversations
		// <elementname type="..." [encoding=...] [derived...]>
		void startElement(const rtl::OUString &aName, const uno::Any& aElement);

		// </elementname>
		void endElement(const rtl::OUString &aName);

		// <value>...</value>
		sal_Bool writeValue(const uno::Any& aValue, const OUString& _pElementName = TAG_VALUE);
	};



	class CmXMLFormater
		:public ChangeTreeAction
		,public NodeAction
		,public IDOMNodeDataProvider
	{
	protected:
		uno::Reference < sax::XDocumentHandler >	m_xHandler;
		ISubtree const* m_pSubtree;
		const SubtreeChange*						m_pChanges;
		sal_Bool WriteSubtreeChanges()  throw (sax::SAXException);

		AnyToXMLDocument m_aDocConverter;
	public:
		CmXMLFormater(const SubtreeChange* _pChanges,
					  const uno::Reference< script::XTypeConverter >& _pTypeConverter);
		
		void SetHandler(const uno::Reference< sax::XDocumentHandler > &rHandler) throw (sax::SAXException);
		void SetSubtree(ISubtree const* pTree) {m_pSubtree = pTree;}
		
		static sal_Int32 Write( const uno::Reference< com::sun::star::io::XOutputStream > &aStream, ISubtree const* pTree );
		
	protected:
		// IDOMNodeDataProvider
		virtual void writeNodeData( 
			const uno::Reference< sax::XDocumentHandler >& _rHandler) {SetHandler(_rHandler);}

		// NodeAction
		virtual void handle(ValueNode const&);
		virtual void handle(ISubtree const&);

		// ChangeTreeAction
		virtual void handle(ValueChange const& aValueNode);
		virtual void handle(AddNode const& aAddNode);
		virtual void handle(RemoveNode const& aRemoveNode);
		virtual void handle(SubtreeChange const& aSubtree);
	};
	
} // namespace
#endif /* Header double load protection */

