/*************************************************************************
 *
 *  $RCSfile: configmodule.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include "configmodule.hxx"

#ifndef CONFIGMGR_API_PROVIDER_HXX_
#include "confprovider.hxx"
#endif
#ifndef CONFIGMGR_API_FACTORY_HXX_
#include "confapifactory.hxx"
#endif

#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#ifndef CONFIGMGR_BOOTSTRAP_HXX_
#include "bootstrap.hxx"
#endif
#ifndef _CONFIGMGR_SESSION_CONFIGSESSION_HXX_
#include "configsession.hxx"
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

#ifndef __SGI_STL_MEMORY // for auto_ptr
#include <stl/memory>
#endif

#ifndef __SGI_STL_ALGORITHM // for swap
#include <stl/algorithm>
#endif

namespace configmgr
{
	using ::rtl::OUString;

	const char cTypeConverterService[] = "com.sun.star.script.Converter";

	uno::Reference< uno::XInterface > SAL_CALL instantiateProvider(uno::Reference< lang::XMultiServiceFactory > const& rServiceManager )
	{
		std::auto_ptr<Module> newModule( new Module( rServiceManager ) );

		uno::Reference< lang::XMultiServiceFactory > aRef( new ConfigurationProvider( *newModule ) );

		newModule.release(); // ownership taken by provider/module

		return aRef;
	}

	Module::Module(uno::Reference< lang::XMultiServiceFactory > const& rServiceManager)
		: m_aServiceManager(rServiceManager)
		, m_aConverter(0)
		, m_aSession(0)
		, m_aProvider(0)
	{
		m_aConverter = m_aConverter.query(rServiceManager->createInstance( OUString(RTL_CONSTASCII_USTRINGPARAM( cTypeConverterService )) ));
		OSL_ENSHURE(m_aConverter.is(), "Module::Module : could not create an instance of the type converter !");
	}

	Module::~Module()
	{
		disconnect();

		OSL_ENSHURE(0 == m_aProvider, "Deleting module still having a provider");
	}

	ConnectionSettings Module::getConnectionData() const
	{
		return ConnectionSettings();
	}

	IConfigSession*	Module::connect(ConnectionSettings const& aSettings, SettingsOverride const& _rSettings)
	{
		OSL_ENSHURE(0 == m_aProvider, "Reconnecting module still having a provider");

		IConfigSession*	pSession = aSettings.createConnection(m_aServiceManager, _rSettings);
		std::swap(pSession,m_aSession);
		delete pSession;

		return m_aSession;
	}

	IConfigSession*	Module::connect()
	{
		if (!m_aSession) 
			connect(getConnectionData());
		return m_aSession;
	}

	IConfigSession*	Module::connect(ConnectionSettings const& aSettings)
	{
		if (!m_aSession) 
			connect(aSettings, SettingsOverride());
		return m_aSession;
	}

	void Module::disconnect(IConfigSession* pSession)
	{
		if (pSession == m_aSession)
			m_aSession = 0;
		delete pSession;
	}

	uno::Reference< script::XTypeConverter > Module::getConverter() const
	{
		return m_aConverter;
	}

	lang::XMultiServiceFactory* Module::getServiceManager() const
	{
		return uno::Reference<lang::XMultiServiceFactory>(m_aServiceManager).get();
	}

	void Module::newProvider(ConfigurationProvider* pProvider)
	{
		OSL_ENSHURE(0 == m_aProvider, "Multiple configuration providers");
		m_aProvider = pProvider;
	}

	void Module::disposing(ConfigurationProvider* pProvider)
	{
		OSL_ENSHURE(pProvider == m_aProvider, "Unknown configuration provider");
		if (pProvider == m_aProvider)
		{
			m_aProvider = 0;
			delete this;
		}
	}
} // namespace




