/*************************************************************************
 *
 *  $RCSfile: strconverter.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

// for Converter from old to old or new
#include <stl/vector>
#include <strdecl.hxx>

#include "strconverter.hxx"

//--------------------------------------------------------------------------
#define ASCII(x) ::rtl::OUString::createFromAscii(x)

namespace configmgr 
{
	namespace sax = com::sun::star::xml::sax;
	namespace uno = com::sun::star::uno;
	
	::rtl::OUString changeToComSunStarPath(const ::rtl::OUString &aPath)
	{
		// POST: Replace org.openoffice with com.sun.star

		static ::rtl::OUString aOO  = ASCII("org.openoffice");
		static ::rtl::OUString aCSS = ASCII("com.sun.star");
		::rtl::OUString aNewPath;

		// compare
		sal_Int32 nIdx = aPath.indexOf(aCSS, 0);
		if (nIdx >= 0)
		{
			::rtl::OUString sPre = aPath.copy(0, nIdx);
			::rtl::OUString sPost = aPath.copy(nIdx + aCSS.getLength());

			aNewPath = sPre + aOO + sPost;
			return aNewPath;
		}
		//OSL_ENSHURE(0, "Warning: There is an old com.sun.star... path, please change to org.openoffice...");
		return aPath;
	}

/*
  struct ltString
  {
  bool operator()(const Assoc &n1, const Assoc& n2) const
  {
  // return strcmp(n1->GetName(), n2->GetName()) < 0;
  return (n1.first.compareTo(n2.first) < 0) ? 1 : 0;
  }
  };
*/

	typedef ::std::pair< rtl::OUString, rtl::OUString > Assoc;
// typedef ::std::set<Assoc, ltNode> MappingTable;
	typedef std::vector<Assoc> MappingTable;

// more Speedup:
// create mapTo funcs for
// Element-Tags
// attribute Types
// attribute Parameters

	rtl::OUString mapTo(const rtl::OUString& aFrom, bool bToNew)
	{
		static MappingTable aMap;
		if (aMap.empty())
		{
			// Fill Map old, new

			aMap.push_back(Assoc(ASCII("value"),    TAG_VALUE));
			aMap.push_back(Assoc(ASCII("type"),     ATTR_TYPE));
			aMap.push_back(Assoc(ASCII("instance"), ATTR_INSTANCE));
		}
		if (bToNew)
		{
			// check, if we should convert first to second
			for (std::vector<Assoc>::const_iterator it = aMap.begin();it != aMap.end();++it)
			{
				if ((*it).first.equals(aFrom))
					return (*it).second;
			}
		}
		else
		{
			// check if we should convert second to first
			for (std::vector<Assoc>::const_iterator it = aMap.begin();it != aMap.end();++it)
			{
				if ((*it).second.equals(aFrom))
					return (*it).first;
			}
		}
	
		// do nothing!
		return aFrom;
	}

	rtl::OUString mapToNew(const rtl::OUString& aFrom) {return mapTo(aFrom, true);}
	rtl::OUString mapToOld(const rtl::OUString& aFrom) {return mapTo(aFrom, false);}

	// -----------------------------------------------------------------------------

	OValueConverter::OValueConverter(
		const uno::Reference< sax::XDocumentHandler > &xHandler, bool bConvertToNew)
			:m_xMasterHandler(xHandler),
			 m_bConvertToNew(bConvertToNew)
	{
		m_nElementDepth= 0;
	}

	// -----------------------------------------------------------------------------
	OValueConverter::~OValueConverter()
	{
		// Test only
		volatile int dummy = 0;
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::startDocument() throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->startDocument();
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::endDocument() throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->endDocument();
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::startElement(const ::rtl::OUString& _rName, const uno::Reference< sax::XAttributeList >& _xAttrList ) throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
		{
			sal_Int16 nAttrCount = _xAttrList.is() ? _xAttrList->getLength() : 0;
			if (nAttrCount > 0)
			{
				// create a new AttributList
				AttributeListImpl *pList = new AttributeListImpl;
				uno::Reference< sax::XAttributeList > rList( (sax::XAttributeList *) pList , uno::UNO_QUERY );

				for( sal_Int16 i=0; i < nAttrCount; i++ )
				{
					rtl::OUString aParamName( _xAttrList->getNameByIndex( i ) );
					rtl::OUString aXMLType( _xAttrList->getTypeByIndex( i ) );
					rtl::OUString aParamValue( _xAttrList->getValueByIndex( i ) );
					
					
					pList->addAttribute( mapTo(aParamName, m_bConvertToNew),
										 aXMLType,
										 mapTo(aParamValue, m_bConvertToNew)); // write value to stream	
				}
				m_xMasterHandler->startElement(mapTo(_rName, m_bConvertToNew), rList);
			}
			else
			{
				m_xMasterHandler->startElement(mapTo(_rName, m_bConvertToNew), _xAttrList);
			}
		}
		++m_nElementDepth;
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::endElement(const ::rtl::OUString& _rName) throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->endElement(mapTo(_rName, m_bConvertToNew));
		--m_nElementDepth;
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::characters(const ::rtl::OUString& _rChars) throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->characters(_rChars);
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::ignorableWhitespace(const ::rtl::OUString& _rWhitespaces) throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->ignorableWhitespace(_rWhitespaces);
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::processingInstruction(const ::rtl::OUString& _rTarget, const ::rtl::OUString& _rData ) throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->processingInstruction(_rTarget, _rData);
	}

	//--------------------------------------------------------------------------
	void SAL_CALL OValueConverter::setDocumentLocator(const uno::Reference< sax::XLocator >& _rxLocator ) throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_xMasterHandler.is())
			m_xMasterHandler->setDocumentLocator(_rxLocator);
	}
	
} // namespace configmgr

