/*************************************************************************
 *
 *  $RCSfile: bootstrap.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: fs $ $Date: 2000/09/28 07:58:07 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#ifndef CONFIGMGR_BOOTSTRAP_HXX_
#include "bootstrap.hxx"
#endif
#ifndef CONFIGMGR_SESSIONFACTORY_HXX_
#include "sessionfactory.hxx"
#endif

#ifndef _OSL_PROFILE_HXX_
#include <osl/profile.hxx>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/string.hxx>
#endif
#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#include "tracer.hxx"

namespace configmgr
{
#define PORTAL_SESSION_IDENTIFIER			"portal"
#define REMOTE_SESSION_IDENTIFIER			"remote"
#define LOCAL_SESSION_IDENTIFIER			"local"

// ---------------------------------------------------------------------------------------
	typedef char const * AsciiString;

	struct SessionClass { 
		AsciiString name;
		SessionFactoryFunc create;
	};
	
// ---------------------------------------------------------------------------------------

	AsciiString const cConfigurationProfileName = "sregistry";
		AsciiString const cConfigurationProfileSection = "configuration";
			AsciiString const cConfigurationSessionType = "servertype";

// ---------------------------------------------------------------------------------------
	SessionClass const cSessionClasses[] = 
	{
		{ REMOTE_SESSION_IDENTIFIER,		createRemoteSession },
		{ LOCAL_SESSION_IDENTIFIER,			createLocalSession  },
		{ PORTAL_SESSION_IDENTIFIER,		createPortalSession }
	};
	int const nSessionClasses = sizeof(cSessionClasses)/sizeof(cSessionClasses[0]);

// ---------------------------------------------------------------------------------------
	ConnectionSettings::ConnectionSettings()
		: m_sSettingsFile(OUString::createFromAscii(cConfigurationProfileName))
	{
	}

// ---------------------------------------------------------------------------------------
	ConnectionSettings::ConnectionSettings(OUString const& aSettingsFile)
		: m_sSettingsFile(aSettingsFile)
	{
	}

// ---------------------------------------------------------------------------------------
	namespace { 
		bool locateBootstrapFiles(OUString const& sBaseName, OUString& sFileSystemContext, OUString& sProfilePath);
	}
// ---------------------------------------------------------------------------------------
	IConfigSession* ConnectionSettings::createConnection(uno::Reference<lang::XMultiServiceFactory> const& aServiceMgr, SettingsOverride const& _rSettings) const
	{

		OUString sPath, sContext;
		::osl::Profile* pProfile = NULL;

		if ( locateBootstrapFiles(m_sSettingsFile, sContext, sPath) )
		{
			CFG_TRACE_INFO("bootstrapping: located an ini file, context: %s, path: %s", OUSTRING2ASCII(sContext), OUSTRING2ASCII(sPath));
			try
			{
				pProfile = new ::osl::Profile( sPath );
			}
			catch (::std::exception& e)
			{
				e.what();	// silence warnings
			}
		}

		try
		{
			rtl::OString aSessionType;

			if (pProfile)
				aSessionType = pProfile->readString( cConfigurationProfileSection, cConfigurationSessionType, rtl::OString() /* default ? */ );
			else
			{
				CFG_TRACE_INFO("bootstrapping: defaulting to portal session");
				aSessionType = ::rtl::OString(RTL_CONSTASCII_STRINGPARAM(PORTAL_SESSION_IDENTIFIER));
			}

			// check if there was a settings override for the session type
			SettingsOverride aSettingsOverride(_rSettings);
			const ::rtl::OUString sSessionTypeSpecifier(RTL_CONSTASCII_USTRINGPARAM("servertype"));
			SettingsOverrideIterator aPos = aSettingsOverride.find(sSessionTypeSpecifier);
			if (aSettingsOverride.end() != aPos)
			{
				::rtl::OUString sCheck;
				if (aPos->second >>= sCheck)
					aSessionType = ::rtl::OString(sCheck.getStr(), sCheck.getLength(), RTL_TEXTENCODING_ASCII_US);
				else
					OSL_ENSHURE(sal_False, "ConnectionSettings::createConnection : invalid server type argument !");

#ifdef DBG_UTIL
				aSettingsOverride.erase(aPos);
					// so the session factory may determine if in the parameters left there are some they does not know
#endif
			}

			for(int i= 0; i<nSessionClasses; ++i)
			{
				if (aSessionType.equals(cSessionClasses[i].name))
				{

					if (IConfigSession* pSession = cSessionClasses[i].create(pProfile, sContext, aServiceMgr, aSettingsOverride))
					{
						OSL_ASSERT(pSession);
						return pSession;
					}
				}
			}

		}
		catch (std::exception& e)
		{
			OSL_ENSHURE(!"reading the profile failed !", e.what());
			if (false) e.what();	// to silence warnings
		}

		OSL_ENSHURE(false, "unable to bootstrap the configuration !");
		return 0;
	}

// locateBootstrapFiles
// ---------------------------------------------------------------------------------------

namespace {

	sal_Unicode const cUNCSeparator = '/';

#ifdef _DEBUG
	static bool checkDirectoryFor(OUString const& sPath, OUString const& sName)
	{
		sal_Int32 nPos = sPath.lastIndexOf(cUNCSeparator);

		// relax first check if some platform prefixes INI names

		return sPath.indexOf(sName,nPos+1) == nPos+1 && nPos >= 0; // is there a directory at all ?
	}
#endif

	static bool splitOslPath(OUString const& sPath, OUString& rDirPath, OUString& rFileName)
	{
		sal_Int32 nPos = sPath.lastIndexOf(cUNCSeparator);

		// Use temporaries to avoid aliasing problems
		OUString sDir = (nPos > 0) ? sPath.copy(0,nPos) : OUString();
		OUString sFile = sPath.copy(nPos+1);

		rFileName = sFile;
		rDirPath  = sDir;

		return nPos >= 0;
	}

	inline bool oslOK(::osl::FileBase::RC code) { return code == ::osl::FileBase::E_None; }

	OUString locateUserPath(OUString& _rUserDirPath, OUString _rIniName)
	{
		static const OUString sSofficeName(RTL_CONSTASCII_USTRINGPARAM("soffice"));
		// search the soffice.ini
		OUString sOfficeUserPath;
		sOfficeUserPath = ::osl::Profile::getName(OUString::createFromAscii("?#"), sSofficeName);

		// remove the soffice.ini from the path
		bool hasUserDir = splitOslPath(sOfficeUserPath, _rUserDirPath, _rIniName);
		// check the directory for existence
		osl::DirectoryItem aItem;
		if (!oslOK(aItem.get(sOfficeUserPath, aItem)))
		{
			// HACK HACK HACK:
			// since the RVP office does not have the "StarOffice 5.3 = ..." entry in sversionrc, the above
			// osl profile method does not work in an RVP office environment.
			// The current HACK for the prio 0 bug is to hard code the sversionrc string used to identify an
			// RVP office.
			// In the future, there has to be another solution. The currently prefered one is to hard code
			// the string used in the sversionrc in the executable. This can be done at the time the setup
			// is run. There will be a meeting on this, currently scheduled for the last wekk in august 2000.
			// Let's see how long this HACK here really survives ...
			// 77620 - 08/15/00 - FS
			CFG_TRACE_INFO("bootstrapping: looking for the RVP key (this hack should be removed)");
			::rtl::OUString sSpecialOSLPath = OUString::createFromAscii("?#\"StarOfficeRVP 5.3\"");
			::rtl::OUString sOfficeUserPath2ndTry = ::osl::Profile::getName(sSpecialOSLPath, sSofficeName);
			CFG_TRACE_INFO_NI("bootstrapping: suggested rvp office user path: %s", OUSTRING2ASCII(sOfficeUserPath2ndTry));

			if (sOfficeUserPath2ndTry.getLength())
			{
				sOfficeUserPath = sOfficeUserPath2ndTry;
				hasUserDir = splitOslPath(sOfficeUserPath, _rUserDirPath, _rIniName);
			}
		}

		OUString sReturn = sOfficeUserPath;
		// we looked for the soffice.ini because the osl_getProfileName had a special handling for this file
		// which ensures that the user directory (even in a network installation) is found
		// In reality, we would need the sregistry.ini (resp. sregistryrc), which is located in exactly that
		// directory. So we have to replace the soffice string in the file name with sregistry.
		sal_Int32 nFileNamePos = sOfficeUserPath.lastIndexOf(cUNCSeparator);	// the last separator
		nFileNamePos = sOfficeUserPath.indexOf(sSofficeName, nFileNamePos);	// the first "soffice" after the last separator
		if (nFileNamePos > 0)
		{
			sReturn = sOfficeUserPath.copy(0, nFileNamePos);
			sReturn += OUString::createFromAscii(cConfigurationProfileName);
			sReturn += sOfficeUserPath.copy(nFileNamePos + sSofficeName.getLength());
		}

		return sReturn;
	}

	bool locateBootstrapFiles(OUString const& sBaseName, OUString& rFileSystemContext, OUString& rProfilePath)
	{
		OUString sUserDirPath, sIniName;
		OUString sOfficeUserPath = locateUserPath(sUserDirPath, sIniName);
		osl::DirectoryItem aItem;

		// exists in user dir ?
		if (oslOK( aItem.get(sOfficeUserPath, aItem) ))
		{
			rFileSystemContext	= sUserDirPath;
			rProfilePath		= sOfficeUserPath;
			return true;
		}

		// exists in current dir
		if (oslOK( aItem.get(sIniName, aItem) ))
		{
			rFileSystemContext	= OUString();
			rProfilePath		= sIniName;
			return true;
		}

		// exists in environment/system path
		OUString sFoundPath;
		if ( oslOK(aItem.searchNormalizedPath(sIniName, OUString(), sFoundPath)))
		{
			OUString sName;
			OSL_VERIFY( splitOslPath(sFoundPath, rFileSystemContext, sName) );
			OSL_ASSERT( sName == sIniName );

			OSL_ASSERT( oslOK( aItem.get(sFoundPath, aItem) ) );
			rProfilePath		= sFoundPath;

			return true;
		}

		// Now try plain name - no ",ini"/"rc"
		sIniName = sBaseName;

		// exists in user/environment/system path
		if (oslOK( aItem.searchNormalizedPath( sIniName, sUserDirPath, sFoundPath)))
		{
			OUString sName;
			OSL_VERIFY( splitOslPath(sFoundPath, rFileSystemContext, sName) );
			OSL_ASSERT( sName == sIniName );

			OSL_ASSERT( oslOK( aItem.get(sFoundPath, aItem) ) );
			rProfilePath		= sFoundPath;

			return true;
		}

		// exists in current dir
		if (oslOK( aItem.get(sIniName, aItem) ))
		{
			rFileSystemContext	= OUString();
			rProfilePath		= sIniName;
			return true;
		}

		// no more ideas
		return false;
	}
} // anon namespace
// ---------------------------------------------------------------------------------------
} // namespace configmgr


