/*************************************************************************
 *
 *  $RCSfile: treecache.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include "treecache.hxx"
#ifndef _CONFIGMGR_SESSION_CONFIGSESSION_HXX_
#include "configsession.hxx"
#endif
#ifndef _CONFIGMGR_TREEACCESS_HXX_
#include "treeaccess.hxx"
#endif
#ifndef _CONFIGMGR_SYNCHRONIZE_HXX_
#include "synchronize.hxx"
#endif
#ifndef _CONFIGMGR_TRACER_HXX_
#include "tracer.hxx"
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef _CONFIGMGR_XMLFORMATER_HXX
#include "xmlformater.hxx"
#endif
#ifndef CONFIGMGR_USERADMIN_HXX_
#include "useradmin.hxx"
#endif
#ifndef _COM_SUN_STAR_LANG_XUNOTUNNEL_HPP_
#include <com/sun/star/lang/XUnoTunnel.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_WRAPPEDTARGETEXCEPTION_HPP_
#include <com/sun/star/lang/WrappedTargetException.hpp>
#endif

#ifndef _CONFIGMGR_SERVER_VERSION_HXX_
#include "server_version.hxx"
#endif
#ifndef _CONFIGMGR_TREEACTIONS_HXX_
#include "treeactions.hxx"
#endif

/*	current response formats (02.07.00)

		REQ				REPONSE					ERROR_RESPONSE
	open/getNode		<Tree>					"error"
	update				"success"				<ChangesTree>
	requestNotify		?						?
	cancelNotify		?						?
	<Notification>		<ChangesTree>			n.a.
*/

using namespace configmgr;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;
// =========================================================================
// =========================================================================
struct OIdPropagator : public NodeModification
{
	::rtl::OUString	sId;

	OIdPropagator(const ::rtl::OUString& _rId) : sId(_rId) { }

	virtual void handle(ValueNode& _rValueNode) { /* not interested in value nodes */ }
	virtual void handle(ISubtree& _rSubtree) { _rSubtree.setId(sId); applyToChildren(_rSubtree); }
};


//==========================================================================
//= TreeManager
//==========================================================================
// -------------------------------------------------------------------------
TreeManager::TreeManager(IConfigSession* _pSession, const Reference< script::XTypeConverter >& _rxTypeConverter) 
				:m_refCount(0)
				,m_aTree()
				,m_aTemplates()
				,m_pSession(_pSession)
				,m_xDataReqCb()
				,m_pListener(0)
				,m_xTypeConverter(_rxTypeConverter)
{
	osl_incrementInterlockedCount(&m_refCount);
	{
		m_pSession->setListener(this);
	}
	osl_decrementInterlockedCount(&m_refCount);
}


// -------------------------------------------------------------------------
TreeManager::~TreeManager()
{
	OSL_ASSERT(m_refCount == 0);
	if (m_pListener) 
	{
		CFG_TRACE_WARNING("cache manager: wasn't disposed properly");
		m_pListener->disposing(this);
	}
}

// -------------------------------------------------------------------------
std::auto_ptr<Subtree> TreeManager::getNodeSynchron(const ::rtl::OUString& _rNodePath, StatusInfo& _rStatus)
{
	::vos::ORef< XMLTreeBuilder > xBuilder = new XMLTreeBuilder(m_xTypeConverter);
	m_pSession->getNode(_rNodePath, xBuilder.getBodyPtr());

	std::auto_ptr<Subtree> pReturn = xBuilder->waitToResponse(m_pSession->getMasterTimeout());
	_rStatus = xBuilder->getCallResult();
	return pReturn;
}

// -------------------------------------------------------------------------
std::auto_ptr<Subtree> TreeManager::openNodeSynchron(const ::rtl::OUString& _rNodePath, sal_Int32 _nLevel, StatusInfo& _rStatus)
{

	::vos::ORef< XMLTreeBuilder > xBuilder = new XMLTreeBuilder(m_xTypeConverter);
	if (_nLevel == ITreeProvider::ALL_LEVELS)
		// translate the level parameter if it says "all levels", cause this is represented by different
		// numbers inside and outside the session
		_nLevel = IConfigSession::COMPLETE_SUBTREE;

	m_pSession->openNode(_rNodePath, _nLevel, xBuilder.getBodyPtr());

	std::auto_ptr<Subtree> pReturn = xBuilder->waitToResponse(m_pSession->getMasterTimeout());
	_rStatus = xBuilder->getCallResult();
	return pReturn;
}

// -------------------------------------------------------------------------
void SAL_CALL TreeManager::release(  ) throw () 
{
	if (! osl_decrementInterlockedCount( &m_refCount ))
	{
		if (m_pListener) 
			m_pListener->disposing(this);
		delete this;
	}
}
// -------------------------------------------------------------------------
ISubtree const*	TreeManager::getSubtree( OUString const& _rNodePath ) const
{
	OReadSynchronized aGuard(&m_aTree);
	ISubtree const*	 pSubTree = m_aTree.getSubtree(_rNodePath);
	return pSubTree;
}

// -------------------------------------------------------------------------
Subtree* TreeManager::implLoadSubtree(const OUString& _rSubtreePath, sal_Int16 _nMinLevels, Tree* _pTree, OReadSynchronized& _rClearBeforeWrite, StatusInfo& _rStatus )
{
	// first check if the tree has a parent of the requested node (sounds stupid .... you could think that, if
	// the ancestor is present, the node itself is present, too. But if somebody previously requested an
	// "/a" node with depth 2, and now a node for "/a/b/c/d/e", this is a valid case ...)

	::rtl::OUString sExistentParentId;

	ConfigurationName aParentCheck(_rSubtreePath);
	OSL_ENSHURE(!aParentCheck.empty(), "TreeManager::implLoadSubtree : invalid name !");
	while (aParentCheck.depth() > 1)
	{
		aParentCheck = aParentCheck.getParentName();
		ISubtree* pParentCheckTree = _pTree->requestSubtree(aParentCheck.fullName(), ALL_LEVELS);
		if (pParentCheckTree)
		{
			sExistentParentId = pParentCheckTree->getId();
			break;
		}
	}

	std::auto_ptr<Subtree> pNewSubTree;
	if (sExistentParentId.getLength())
	{
		CFG_TRACE_INFO("cache manager: have a parent of the requested node. node id: %s. using getNode.", OUSTRING2ASCII(sExistentParentId));
		// there already is a parent node in the tree -> use getNode instead of openNode
		pNewSubTree = getNodeSynchron(_rSubtreePath, _rStatus);
		_nMinLevels = ALL_LEVELS;
			// getNode always returns the whole sub tree
			// (TODO : really ? or is the result limited by the previous openNode somebody did and
			// the level specified in this openNode transaction ?)

		OSL_ENSHURE(NULL != pNewSubTree.get(), "TreeManager::implLoadSubtree : getNode failed !");
		if (NULL != pNewSubTree.get())
		{
			// propagate the node id to the new sub tree ....
			OIdPropagator aSetIds(sExistentParentId);
			pNewSubTree->dispatch(aSetIds);
		}
		else
			CFG_TRACE_ERROR("cache manager: something went wrong.");
	}
	else
	{
		CFG_TRACE_INFO("cache manager: opening the node, 'cause I have no parent node in the cache.");
		// no valid parent node in the tree -> open the node
#ifdef _OPEN_COMPONENT_NODES_ONLY_
		// let the session confirm the path
		::rtl::OUString sSubtreePath(_rSubtreePath);
		sal_Int32 nOpenNodeDepth = ALL_LEVELS == _nMinLevels ? IConfigSession::COMPLETE_SUBTREE : _nMinLevels;
		if (!m_pSession->confirmOpenNode(sSubtreePath, nOpenNodeDepth))
		{	// the session says no ...
			nOpenNodeDepth = IConfigSession::COMPLETE_SUBTREE == nOpenNodeDepth ? ALL_LEVELS : nOpenNodeDepth;
			ConfigurationName aAlternative(sSubtreePath);
#ifdef DEBUG
			OSL_ENSHURE(sSubtreePath != _rSubtreePath, "TreeManager::implLoadSubtree : rejecting without giving alternatives is not nice !");
			ConfigurationName aOriginal(_rSubtreePath);
			OSL_ENSHURE(aOriginal.isNestedIn(aAlternative), "TreeManager::implLoadSubtree : invalid node path alternative !");
#endif
			std::auto_ptr<Subtree> pAlternative = openNodeSynchron(aAlternative.fullName(), nOpenNodeDepth, _rStatus);
			if (pAlternative.get())
			{
				CFG_TRACE_INFO("cache manager: got it.");
				_rClearBeforeWrite.clear();
				OWriteSynchronized aWriteGuard(_pTree);
					// TODO : this is not really safe : between freeing the read guard an acquiring the write guard, some other
					// thread may have inserted the new subtree, too.
					// To avoid this, we would need a write access which could be nested within a read access.
					// which is a big TODO
				_pTree->addSubtree(aAlternative, pAlternative, nOpenNodeDepth);
				// now the tree should contain the node for the requested path
				Subtree* pOriginalRequest = static_cast<Subtree*>(_pTree->requestSubtree(_rSubtreePath, nOpenNodeDepth));

					// pOriginalRequest is allowed to be NULL : If the the alternative - which is a parent of the
					// requested node - exists, but not the node we're really interested in. This just means somebody
					// gave us an invalid path.

				return pOriginalRequest;
					// return here, so the sub tree won't be added again below
			}
			else
			{	// maybe the path does not exist on the server .... The StatusInfo will contain more info and will
				// be eveluated by the caller
				CFG_TRACE_ERROR("cache manager: did not get it. server return: %s", OUSTRING2ASCII(_rStatus.sMessage));
				return NULL;
			}
		}
		else
#endif

		pNewSubTree = openNodeSynchron(_rSubtreePath,_nMinLevels, _rStatus);
	}

	Subtree* pSubtree = NULL;
	if (pNewSubTree.get())
	{
		_rClearBeforeWrite.clear();
		OWriteSynchronized aWriteGuard(_pTree);
			// TODO : this is not really safe : between freeing the read guard an acquiring the write guard, some other
			// thread may have inserted the new subtree, too.
			// To avoid this, we would need a write access which could be nested within a read access.
			// which is a big TODO
		CFG_TRACE_INFO("cache manager: adding the node to the cache");
		pSubtree = _pTree->addSubtree(ConfigurationName(_rSubtreePath, ConfigurationName::Absolute()), pNewSubTree, _nMinLevels);
	}

	return pSubtree;
}

// -------------------------------------------------------------------------
ISubtree* TreeManager::requestSubtree( OUString const& aSubtreePath, sal_Int16 nMinLevels)
{
	OReadSynchronized aReadGuard(&m_aTree);
	CFG_TRACE_INFO("cache manager: checking the cache");
	ISubtree*	 pSubTree = m_aTree.requestSubtree(aSubtreePath, nMinLevels);
	if(!pSubTree)
	{
		CFG_TRACE_INFO("cache manager: cache miss. going to load the node");
		// cache miss ...
		StatusInfo aResult;
		pSubTree = implLoadSubtree(aSubtreePath, nMinLevels, &m_aTree, aReadGuard, aResult);
	}
	return pSubTree;
}

// -------------------------------------------------------------------------
ConfigurationName TreeManager::completeTemplateLocation(const ::rtl::OUString& _rLogicalTemplateName)
{
	static const ConfigurationName aTemplateRoot(::rtl::OUString::createFromAscii("com.sun.star.Templates"));
	return ConfigurationName(aTemplateRoot, _rLogicalTemplateName);
}

// -------------------------------------------------------------------------
void TreeManager::ensureTemplate(const ::rtl::OUString& _rName) throw (uno::Exception)
{
	OSL_ENSHURE(_rName.getLength(), "TreeManager::ensureTemplate : invalid template name !");

	OReadSynchronized aReadGuard(&m_aTemplates);

	CFG_TRACE_INFO("cache manager: going to get a template named %s", OUSTRING2ASCII(_rName));

	ConfigurationName aTemplateLocation = completeTemplateLocation(_rName);

	const INode* pNode = m_aTemplates.getNode(aTemplateLocation.fullName());
	if (!pNode)
	{
		CFG_TRACE_INFO("cache manager: cache miss for that template");
		// at the moment, the XML tree builder is not able to supply leafs, it only handles sub trees.
		// as we don't know if the given path describes a inner node or a leaf, we have to ensure that only 
		// a tree is loaded
		OSL_ENSHURE(aTemplateLocation.depth() > 1, "TreeManager::ensureTemplate : invalid template location !");
		ConfigurationName aTemplateParent(aTemplateLocation.getParentName());
		::rtl::OUString sTemplateLocal = aTemplateLocation.localName();

		sal_Int16 nLevels = ALL_LEVELS;
		StatusInfo aResult;
		implLoadSubtree(aTemplateParent.fullName(), nLevels, &m_aTemplates, aReadGuard, aResult);
		if (aResult.nCode)
			throw Exception((::rtl::OUString::createFromAscii("The template description could not be loaded. Error message as got from the session : ")) += aResult.sMessage, NULL);
	}
}

// -------------------------------------------------------------------------
const INode*	TreeManager::getNode(const OUString& _rPath)
{
	OReadSynchronized aGuard(&m_aTree);
	return m_aTree.getNode(_rPath);
}

// -------------------------------------------------------------------------
auto_ptr<INode> TreeManager::createInstance(const ::rtl::OUString& _rTemplateName) throw (uno::Exception)
{
	ensureTemplate(_rTemplateName);
	ConfigurationName aTemplateLocation = completeTemplateLocation(_rTemplateName);

	OReadSynchronized aReadGuard(&m_aTemplates);

	const INode* pTemplate = m_aTemplates.getNode(aTemplateLocation.fullName());
	if (pTemplate)
		return auto_ptr<INode>(pTemplate->clone());

	throw Exception(::rtl::OUString::createFromAscii("The template description could not be loaded. An unknown error occured."), NULL);
}

// -------------------------------------------------------------------------
void TreeManager::fireChanges(TreeChangeList const& aChangeTree, sal_Bool _bError)
{
	if (m_pListener) m_pListener->changes(aChangeTree, _bError);
}

// -------------------------------------------------------------------------
class RemoveEmptySubtrees : public ChangeTreeModification
{
protected:
	typedef ::std::map< ::rtl::OUString, sal_Int32 > LeafNumbers;

	LeafNumbers m_aNumbers;
	sal_Int32	m_nCurrentNum;

public:
	RemoveEmptySubtrees() : m_nCurrentNum(0) { }

	virtual void handle(ValueChange& aValueNode);
	virtual void handle(AddNode& aAddNode);
	virtual void handle(RemoveNode& aRemoveNode);
	virtual void handle(SubtreeChange& aSubtree);
};

// .........................................................................
void RemoveEmptySubtrees::handle(ValueChange& aValueNode)	{ ++m_nCurrentNum; }
void RemoveEmptySubtrees::handle(AddNode& aAddNode)			{ ++m_nCurrentNum; }
void RemoveEmptySubtrees::handle(RemoveNode& aRemoveNode)	{ ++m_nCurrentNum; }

// .........................................................................
void RemoveEmptySubtrees::handle(SubtreeChange& aSubtree)
{
	// push the current state
	LeafNumbers aOldNumbers = m_aNumbers;
	sal_Int32 nOldNum = m_nCurrentNum;

	// initialize the state
	m_aNumbers.clear();
	m_nCurrentNum = 0;

	// handle the children
	aSubtree.forEachChange(*this);

	// remove all subtrees without any leafs
	for (	LeafNumbers::const_iterator aErase = m_aNumbers.begin();
			aErase != m_aNumbers.end();
			++aErase
		)
	{
		if (!aErase->second)
			aSubtree.removeChange(aErase->first);
	}

	// remember the state
	sal_Int32 nMyLeafs = m_nCurrentNum;

	// pop the old state
	m_aNumbers = aOldNumbers;
	m_nCurrentNum = nOldNum;

	// and add my own leaf number to the map of the old state
	m_aNumbers[aSubtree.getNodeName()] += nMyLeafs;
	m_nCurrentNum += nMyLeafs;
}

// .........................................................................
void TreeManager::updateTree(TreeChangeList& aChanges) throw (starlang::WrappedTargetException, uno::RuntimeException)
{ 
	OWriteSynchronized aGuard(&m_aTree);

	CFG_TRACE_INFO("cache manager: updating the tree from a changes list");

#ifdef _NO_MIXED_TRANSACTIONS_
	// do some checks on the current changes tree
	OChangeActionCounter aChangeCounter;
	aChangeCounter.handle(aChanges.root);

	CFG_TRACE_INFO_NI("cache manager: counted changes: additions: %i (%i new users), removes: %i, value changes: %i", aChangeCounter.nAdds, aChangeCounter.nAddUsers, aChangeCounter.nRemoves, aChangeCounter.nValues);

	if	(	0 != (aChangeCounter.nValues * aChangeCounter.nAdds)
		||	0 != (aChangeCounter.nAdds * aChangeCounter.nRemoves)
		||	0 != (aChangeCounter.nRemoves * aChangeCounter.nValues)
		||	((0 != (aChangeCounter.nAddUsers * aChangeCounter.nAdds)) && (aChangeCounter.nAddUsers != aChangeCounter.nAdds))
		)
		throw WrappedTargetException(
			::rtl::OUString::createFromAscii("At the moment, an update transaction must consist of only one type of actions (value-change, add-node, remove-node)."),
			NULL,
			uno::Any()
		);

	if (aChangeCounter.bDifferentSetActionLevels)
		throw WrappedTargetException(
			::rtl::OUString::createFromAscii("At the moment, in an update transaction all set actions (add-node, remove-node) have to be done one the same node."),
			NULL,
			uno::Any()
		);

#ifdef _ONE_SET_ACTION_PER_TRANSACTION_
	if ((aChangeCounter.nAdds > 1) || (aChangeCounter.nRemoves > 1))
	{	// we already know that only on transtype is present, so this means that there is more than one set-action
		throw WrappedTargetException(
			::rtl::OUString::createFromAscii("At the moment, in an update transaction only one set-action (add-node, remove-node) is allowed."),
			NULL,
			uno::Any()
		);
	}
#endif
#endif

	m_aTree.updateTree(aChanges);
	OSL_ENSHURE(m_xDataReqCb.isEmpty(), "TreeManager::updateTree : already doing an update !");
		// this restriction has to be relaxed if this method is asynchronous

	// normalize the update tree. This means that we want a tree with one root which has either more than one child
	// or exactly one non-SubtreeChange-child
	// This prevents us from giving unneccessary items to the session
	ConfigurationName aPathToRoot = aChanges.pathToRoot;
	const SubtreeChange* pRootChanges = &aChanges.root;

	// first, kick off all tree parts which are just empty SubtreeChanges (or SubtreeChanges containing only
	// SubtreeChanges)
	// As the changes tree we got is constant, we have to build our own hierarchy
	SubtreeChangeReferrer aReferrer(*pRootChanges);
		// this is a SubtreeChange, which, in it's dtor, does not delete all it's children, but only the ones which
		// are SubtreeChanges itself. As it replaces all SubtreeChanges with SubtreeChangeReferrers, this means
		// that upon destruction of the object, no Changes are deleted.
	RemoveEmptySubtrees aRemoveDispensables;
	aReferrer.dispatch(aRemoveDispensables);

	// now the root of the changes tree is the "referrer" object
	pRootChanges = &aReferrer;

	// do the normalizing
	SubtreeChange::ChildIterator aChildren = pRootChanges->begin();
	while ((pRootChanges->size() == 1) && (aChildren->isA(SubtreeChange::getStaticType())))
	{
		aPathToRoot = aPathToRoot + pRootChanges->getNodeName();
		pRootChanges = static_cast<const SubtreeChange*>(*aChildren);
		aChildren = pRootChanges->begin();
	}

	ConfigurationName aCompleteChangesName(aPathToRoot.fullName(), pRootChanges->getNodeName());
	m_xDataReqCb = new OUpdateCallback(aCompleteChangesName, this);

	CmXMLFormater aInterpreter(pRootChanges, m_xTypeConverter);
	ISubtree const* pSubTree = m_aTree.getSubtree(aCompleteChangesName.fullName());
	OSL_ENSHURE(pSubTree, "TreeManager::updateTree : getSubtree should always succeed in this situation !");
	if(pSubTree)
	{
		CFG_TRACE_INFO("cache manager: going to send the changes to the session");
#ifdef _NO_MIXED_TRANSACTIONS_
		if (aChangeCounter.nValues)
		{
			m_pSession->updateNode(pSubTree->getId(), aCompleteChangesName.relativeTo(ConfigurationName()).fullName(), &aInterpreter, m_xDataReqCb.getBodyPtr());
		}
		else if (aChangeCounter.nAdds > aChangeCounter.nAddUsers)
		{
			m_pSession->addNode(pSubTree->getId(), aCompleteChangesName.relativeTo(ConfigurationName()).fullName(), &aInterpreter, m_xDataReqCb.getBodyPtr());
		}
		else if (aChangeCounter.nAddUsers)
		{
#ifdef _ONE_SET_ACTION_PER_TRANSACTION_
			OSL_ASSERT(pRootChanges->size() == 1);
			OSL_ASSERT((*pRootChanges->begin())->ISA(AddUser));
			OUString aUserName = (*pRootChanges->begin())->getNodeName();
			m_pSession->addUser(aUserName, &aInterpreter, m_xDataReqCb.getBodyPtr(), pSubTree->getId());
#else
			OSL_ENSHURE(sal_False, "TreeManager::updateTree : implementation missing : adding more than one user !");
			m_xDataReqCb->failed(-1);
				// just to prevent us from waiting too long ....
#endif
		}
		else
		{
			OSL_ENSHURE(aChangeCounter.nRemoves, "TreeManager::updateTree : nothing left to do !");
#ifdef _ONE_SET_ACTION_PER_TRANSACTION_
			// as the changes were already applied to m_aTree, we have to work on the changes tree to get the name
			// of the removed node, not on m_aTree itself
			OSL_ENSHURE(pRootChanges->size() == 1, "TreeManager::updateTree : not supported at the moment : more than one removal !");
				// all other cases should not reach this place here ....
			::rtl::OUString sRemovedOne = (*pRootChanges->begin())->getNodeName();

			ConfigurationName aRemovedTree = ConfigurationName(aCompleteChangesName, sRemovedOne).relativeTo(ConfigurationName());
			m_pSession->deleteNode(pSubTree->getId(), aRemovedTree.fullName(), m_xDataReqCb.getBodyPtr());
#else
			OSL_ENSHURE(sal_False, "TreeManager::updateTree : implementation missing : removal of more than one node !");
			m_xDataReqCb->failed(-1);
				// just to prevent us from waiting too long ....
#endif
		}
#else
		m_pSession->updateNode(pSubTree->getId(), aCompleteChangesName.relativeTo(ConfigurationName()).fullName(), &aInterpreter, m_xDataReqCb.getBodyPtr());
#endif

		// TODO : this should be asynchronous : after doing the update on the session, we should return from here.
		// In case of success, we don't have to do anything else.
		// In case 	of an error, our callback (and thus we) will be notified with the real tree state, which
		// we would have to merge in our tree, then.

		CFG_TRACE_INFO_NI("cache manager: synchron update: waiting for the server response");
		// While we are synchronous we try to emulate that behavior,...
		if (m_xDataReqCb->waitForUpdate())
		{
			m_xDataReqCb = NULL;
			fireChanges(aChanges, false); // deferred for now ...
		}
		else
		{
			CFG_TRACE_ERROR("cache manager: an error occured ...");
			// TODO : change the error informations transported in an IConfigSession, so we don't have to do
			// this, hmm, complex check
			StatusInfo aStatus = m_xDataReqCb->getStatus();
			if (0 == aStatus.nCode)
			{
				aStatus.sMessage = ::rtl::OUString::createFromAscii("an error occured while connecting with the server. code(");
				aStatus.sMessage += ::rtl::OUString::valueOf(m_xDataReqCb->getConnError());
				aStatus.sMessage += ::rtl::OUString::createFromAscii(").");
			}

			// undo the changes
			XMLTreeChangeListBuilder* pBuilder = m_xDataReqCb->getTreeBuilder();
			if(pBuilder)
			{
				if (pBuilder->waitToResponseChange(m_pSession->getMasterTimeout()))
				{
					fireChanges(aChanges, false); // EMULATING async behavior, fire this only now
					TreeChangeList* pChangeList = m_xDataReqCb->getChanges();
					m_aTree.updateTree(*pChangeList);
					fireChanges(*pChangeList, true);
				}
				else
					// TODO
					OSL_ENSHURE(sal_False, "TreeManager::updateTree : no changes tree after the unsuccessfull update !");
			}
			else
				OSL_ENSHURE(sal_False, "TreeManager::updateTree : got no tree builder from the callback !");

			m_xDataReqCb = NULL;

			throw WrappedTargetException(aStatus.sMessage, NULL, Any());
		}
	}
}

// -------------------------------------------------------------------------
void TreeManager::nodeUpdated(const ::rtl::OUString& _rNodePath)
{
}

// -------------------------------------------------------------------------
void TreeManager::nodeDeleted(const ::rtl::OUString& _rNodePath)
{
}

// -------------------------------------------------------------------------
void TreeManager::nodeAdded(const ::rtl::OUString& _rNodePath)
{
}
//==========================================================================
void TreeManager::acquireReadAccess() const
{
	m_aTree.getTreeLock()->acquireReadAccess();
}

void TreeManager::releaseReadAccess() const
{
	m_aTree.getTreeLock()->releaseReadAccess();
}

void TreeManager::acquireWriteAccess()
{
	m_aTree.getTreeLock()->acquireWriteAccess();
}

void TreeManager::releaseWriteAccess()
{
	m_aTree.getTreeLock()->releaseWriteAccess();
}


//==========================================================================
//= OUpdateCallback
//==========================================================================
// -------------------------------------------------------------------------
OUpdateCallback::OUpdateCallback(const ConfigurationName& _rRootLoc, TreeManager* _pCM)
	:m_pCacheManager(_pCM)
	,m_aChanges(_rRootLoc.getParentName().fullName(), _rRootLoc.localName())
	,m_xBuilder(new XMLTreeChangeListBuilder(_pCM->getTypeConverter(), &m_aChanges))
	,m_nConnError(0)
	,m_refCount(0)
{
	m_aDoneCond.reset();
}

// -------------------------------------------------------------------------
staruno::Reference< xmlsax::XDocumentHandler > OUpdateCallback::getDataReader()
{
	return static_cast< xmlsax::XDocumentHandler* >(m_xBuilder.getBodyPtr());
}
// -------------------------------------------------------------------------
sal_Bool OUpdateCallback::waitForUpdate(TimeValue* _pTimeout)
{
	// TODO : if the update on the tree is asnychronous, we don't need this method anymore ...
	TimeValue aTimeout;
	aTimeout.Seconds = 30;
	m_aDoneCond.wait(_pTimeout ? _pTimeout : &aTimeout);

	return (0 == m_nConnError) && (0 == m_aServerStatus.nCode);
		// no connection error and no server error
}
// -------------------------------------------------------------------------
void OUpdateCallback::failed(sal_Int32 _nErrorCode)
{
	OSL_ENSHURE(m_pCacheManager,"OUpdateCallback::done : no cache manager !");
	m_nConnError = _nErrorCode;
	m_aDoneCond.set();
}
// -------------------------------------------------------------------------
void OUpdateCallback::done(const StatusInfo& _rStatus)
{
	OSL_ENSHURE(m_pCacheManager,"OUpdateCallback::done : no cache manager !");
	m_aServerStatus = _rStatus;
	m_aDoneCond.set();
}


