/*************************************************************************
 *
 *  $RCSfile: builddom.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_XML_BUILDDOM_HXX_
#define _CONFIGMGR_XML_BUILDDOM_HXX_

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#include <stl/stack>

#include "updatedom.hxx"
#include "attributes.hxx"
#include "configsession.hxx"

namespace com { namespace sun { namespace star { namespace xml { namespace sax { 
class XAttributeList;
} } } } } 

namespace configmgr
{
	namespace css = com::sun::star;
	namespace sax = css::xml::sax;
	namespace uno = css::uno;
	namespace lang = css::lang;

	class BuilderState;
	struct BuilderContext
	{
		std::stack<BuilderState*> states;
		
		void pushBuilder(BuilderState& rBuilder, const rtl::OUString &aName, const uno::Reference <sax::XAttributeList> & xAttributes);
	};

/////////////////////////////////////////////////////////////////////////////////////////////
	class BuilderState
	{
	protected:
		BuilderState() : m_nStarted(0) {}
		virtual ~BuilderState();
		
	public:
		bool started() const { return m_nStarted > 0; }
		
		void startElement(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				if (start(ctx))
					startSelf(ctx, name, xAttr); 
				else
					startChild(ctx, name, xAttr);
			}
		virtual void characters(BuilderContext& ctx, const rtl::OUString &text)
			{
			}
		void endElement(BuilderContext& ctx, const rtl::OUString &name)
			{
				if (stop(ctx)) 
				{
					endSelf(ctx, name);
					if (!ctx.states.empty())
					{
						OSL_ASSERT(ctx.states.top()->m_nStarted > 1);
						ctx.states.top()->endElement(ctx, name);
					}
					delete this;
				}
				else
					endChild(ctx, name);
			}
		
	protected:
		virtual void startSelf(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
			}
		virtual void startChild(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			= 0;
		
		
		virtual void endChild(BuilderContext& ctx, const rtl::OUString &name)
			{
			}
		virtual void endSelf(BuilderContext& ctx, const rtl::OUString &name)
			{
			}

	private:
		bool start(BuilderContext& ctx)
			{
				OSL_ASSERT(m_nStarted >= 0);
				if (m_nStarted++ != 0)
					return false;

				ctx.states.push(this);
				return true;
			}
		bool stop(BuilderContext& ctx)
			{
				OSL_ASSERT(m_nStarted > 0);
				if (--m_nStarted != 0)
					return false;

				OSL_ASSERT(this == ctx.states.top());
				ctx.states.pop();
				return true;
			}
		int m_nStarted;
	};
	inline BuilderState::~BuilderState() { OSL_ASSERT(!started()); }

///////////////////////////////////////////////////////////////////
	struct BuilderDocumentStateBase : BuilderState
	{
		void startSelf(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				OSL_ASSERT( name.getLength() == 0 );
				OSL_ASSERT( ! xAttr.is() );
			}
		void startChild(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				if (BuilderState* pDocElementBuilder = createChildBuilder(ctx,name,xAttr))
				{
					ctx.pushBuilder(*pDocElementBuilder, name, xAttr);
				}
			}
		void endSelf(BuilderContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT( name.getLength() == 0 );
			}
	protected:
		virtual BuilderState* createChildBuilder(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr) = 0;
	};

///////////////////////////////////////////////////////////////////

	struct Builder : cppu::WeakImplHelper1<sax::XDocumentHandler>
	{
		BuilderContext context;

		Builder() : context() {}

		virtual BuilderState* createDocumentState() = 0;

		virtual void SAL_CALL startDocument()
			throw (sax::SAXException, uno::RuntimeException)
			{ 
				startDocument( *createDocumentState()); 
			}
		void startDocument(BuilderState& rInitialState, const rtl::OUString &name = rtl::OUString()) 
			throw (sax::SAXException, uno::RuntimeException) 
			{ 
			OSL_ASSERT(context.states.empty()); 

			context.pushBuilder( rInitialState, name, 0 );
		}
		virtual void SAL_CALL startElement(const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			throw (sax::SAXException, uno::RuntimeException)
			{
				state().startElement(context,name,xAttr);
			}
		virtual void SAL_CALL endElement(const rtl::OUString &name)
			throw (sax::SAXException, uno::RuntimeException)
			{
				state().endElement(context,name); 
			}
		virtual void SAL_CALL characters(const rtl::OUString &text)
			throw (sax::SAXException, uno::RuntimeException)
			{
				state().characters(context,text);
			}

		virtual void SAL_CALL endDocument()
			throw (sax::SAXException, uno::RuntimeException)
			{ 
				context.states.top()->endElement( context, rtl::OUString() ); 		
				OSL_ASSERT(context.states.empty()); 
			}

		virtual void SAL_CALL ignorableWhitespace(const class rtl::OUString &) {}
		virtual void SAL_CALL processingInstruction(const class rtl::OUString &, const class rtl::OUString &) {}
		virtual void SAL_CALL setDocumentLocator(const uno::Reference< sax::XLocator> &) {}

	private:
		BuilderState& state() { OSL_ASSERT(!context.states.empty()); return *context.states.top(); }

	};

////////////////////////////////////////////////////////////////////////////
	BuilderState* createNodeBuilder(BuilderContext& ctx, InnerNode& rParent, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr);

////////////////////////////////////////////////////////////////////////////
	struct InnerNodeBuilder : BuilderState
	{
		InnerNode* m_pThisNode;
		InnerNode& m_rParentNode;

		InnerNodeBuilder(InnerNode& rParent) : m_pThisNode(0), m_rParentNode(rParent) {}
		~InnerNodeBuilder() { delete m_pThisNode; }

		void startSelf(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				OSL_ASSERT(m_pThisNode == 0);
				m_pThisNode = new InnerNode(name,xAttr);
			}
		void startChild(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				OSL_ASSERT(m_pThisNode != 0);
				ctx.pushBuilder(*createNodeBuilder(ctx,*m_pThisNode,name,xAttr), name, xAttr);
			}

		void characters(BuilderContext& ctx, const rtl::OUString &text)
			{
				OSL_ASSERT(false);
			}
		void endSelf(BuilderContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT(m_rParentNode.getChild(name) == 0);
				m_rParentNode.addChild(m_pThisNode);
				m_pThisNode = 0;
			}
	};

////////////////////////////////////////////////////////////////////////////
	struct LeafNodeBuilder : BuilderState
	{
		LeafNode* m_pThisNode;
		XMLValue* m_pThisText;
		InnerNode& m_rParentNode;
	
	
		LeafNodeBuilder(InnerNode& rParent) 
				: m_pThisNode(0)
			, m_pThisText(0)
			, m_rParentNode(rParent) 
			{}

		void startSelf(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				ElementNode* pNode = m_rParentNode.getChild(name);
				if (pNode)
				{
					m_pThisNode = pNode->asLeafNode();
					OSL_ASSERT(m_pThisNode != 0 );
				}
				else
				{
					m_pThisNode = new LeafNode(name);
					m_rParentNode.addChild(m_pThisNode);
				}
				m_pThisText = &m_pThisNode->appendNew();
			}
		void startChild(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				OSL_ASSERT(false);
			}

		void characters(BuilderContext& ctx, const rtl::OUString &text)
			{
				OSL_ASSERT(m_pThisNode);
				OSL_ASSERT(m_pThisText);
				OSL_VERIFY(m_pThisText == &m_pThisNode->appendCurrent(text));
			}


		void endChild(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				OSL_ASSERT(false);
			}
		void endSelf(BuilderContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT(m_rParentNode.getChild(name) == m_pThisNode);
			}
		
	};
////////////////////////////////////////////////////////////////////////////
	struct ChangeDOMDocumentState : BuilderState
	{
		InnerNode& m_rRootHolder;
		
		ChangeDOMDocumentState(InnerNode& rRootHolder) : m_rRootHolder(rRootHolder) {}
		~ChangeDOMDocumentState() { }
		
		void startSelf(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				OSL_ASSERT(name.getLength() == 0);
				OSL_ASSERT(!xAttr.is());
			}
		void startChild(BuilderContext& ctx, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> & xAttr)
			{
				// OSL_ASSERT(m_aRootHolder.children().size() == 0);
				ctx.pushBuilder(*createNodeBuilder(ctx,m_rRootHolder,name,xAttr), name, xAttr);
			}
		
		void characters(BuilderContext& ctx, const rtl::OUString &text)
			{
				OSL_ASSERT(false);
			}
		void endChild(BuilderContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT(m_rRootHolder.children().size() == 1);
			}
		void endSelf(BuilderContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT(name.getLength() == 0);
				OSL_ASSERT(m_rRootHolder.children().size() <= 1);
			}
	};
	
////////////////////////////////////////////////////////////////////////////
	struct ChangeDOMBuilder : Builder
	{
		InnerNode m_aRootHolder;
		
		ElementNode* getRoot()
			{
				OSL_ASSERT(m_aRootHolder.children().size() <= 1);
				if (m_aRootHolder.children().empty())
					return 0;
				
				return *m_aRootHolder.children().begin();
			}
		virtual BuilderState* createDocumentState() 
			{
				return new ChangeDOMDocumentState(m_aRootHolder);
			}
	};
	

// usage
	

	int doTreeUpdate(const rtl::OUString& aFromURL,
					 const rtl::OUString& aToURL,
					 const rtl::OUString& aBasePath,
					 IDOMNodeDataProvider* pDataProvider,
					 const uno::Reference< lang::XMultiServiceFactory >& _rxServiceProvider,
					 /* OUT */ rtl::OUString &sError);

/*
	{
		ChangeDOMBuilder* pChangeDOM = new ChangeDOMBuilder();
		uno::Reference<sax::XDocumentHandler> xKeepAlive = pChangeDOM;
		
		pChangeDOM->startDocument();
		pDataProvider->writeNodeData(xKeepAlive);
		pChangeDOM->endDocument();
		
		uno::Reference< sax::XParser > xParser;
		xParser = uno::Reference< sax::XParser >(_rxServiceProvider->createInstance( ASCII("com.sun.star.xml.sax.Parser"), UNO_QUERY));

		TreeInfo aChangesInfo = { aBasePath, pChangeDOM->getRoot() };
		mergeTree(xParser, aInput, aChangesInfo, xOutput); 
	}
*/
	
} // namespace configmgr

#endif

