/*************************************************************************
 *
 *  $RCSfile: dataexport.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_BEANS_PROPERTYVALUE_HPP_
#include <com/sun/star/beans/PropertyValue.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif

#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XACTIVEDATASOURCE_HPP_
#include <com/sun/star/io/XActiveDataSource.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HDL_
#include <com/sun/star/io/XOutputStream.hpp>
#endif

#ifndef _UTL_SEQUENCE_HXX_
#include <unotools/sequence.hxx>
#endif

#ifndef CONFIGMGR_API_DATA_EXPORT_IMPL_HXX_
#include "dataexport.hxx"
#endif

#include "attributes.hxx"

#define THISREF() static_cast< ::cppu::OWeakObject* >(this)
#define ASCII(x) OUString::createFromAscii(x)

namespace configmgr
{
	namespace beans  = ::com::sun::star::beans;
	namespace io  = ::com::sun::star::io;
	namespace container  = ::com::sun::star::container;

	using namespace com::sun::star::uno;

	static AsciiServiceName const aDataExportServices[] = 
	{
		"com.sun.star.configuration.DataExport", 
		0
	};
	// special exported one
	ServiceInfo const ODataExport::staticServiceInfo = 
	{ 
		"com.sun.star.configuration.configmgr.DataExport", 
		aDataExportServices 
	};

	const ServiceInfo* getDataExportServiceInfo()
	{
		return &ODataExport::staticServiceInfo;
	}
	
	Reference< XInterface > SAL_CALL instantiateDataExport(Reference< lang::XMultiServiceFactory > const& _rServiceManager )
	{
		return static_cast< ::cppu::OWeakObject* >(new ODataExport(_rServiceManager));
	}

// -----------------------------------------------------------------------------


	sal_Int32 ODataExport::countServices(ServiceInfo const* aInfo)
	{
		AsciiServiceName const* p= aInfo ? aInfo->serviceNames : 0;
		if (p == 0)
			return 0;

		sal_Int32 nCount = 0;
		while (*p != 0)
		{
			++nCount;
			++p;
		}

		return nCount;
	}

// -----------------------------------------------------------------------------

	void ODataExport::initialize( const Sequence< Any >& _rArguments )
		throw(Exception, RuntimeException)
	{
		osl::MutexGuard aGuard(m_aMutex);

		// if (m_pImpl || m_aSecurityOverride.size())
		//	throw Exception(::rtl::OUString::createFromAscii("The configuration provider has already been initialized."), THISREF());

		const Any* pArguments = _rArguments.getConstArray();
		beans::PropertyValue aCurrentArg;
		Reference< script::XTypeConverter > xConverter;
		rtl::OUString sPath;
		for (sal_Int32 i=0; i<_rArguments.getLength(); ++i, ++pArguments)
		{
			if ((*pArguments) >>= aCurrentArg)
			{
				static const ::rtl::OUString sPathParam(RTL_CONSTASCII_USTRINGPARAM("Path"));
				static const ::rtl::OUString sPackageNameParam(RTL_CONSTASCII_USTRINGPARAM("PackageName"));
				static const ::rtl::OUString sConverterParam(RTL_CONSTASCII_USTRINGPARAM("TypeConverter"));

				sal_Bool bExtractSuccess = sal_True;	// defaulted to TRUE, so we skip unknown arguments

				if (aCurrentArg.Name.equalsIgnoreCase(sPathParam))
				{
					bExtractSuccess = (aCurrentArg.Value >>= sPath);
					uno::Sequence<rtl::OUString> aSeq(1);
					aSeq.getArray()[0] = sPath;
					m_sPathSeq = utl::concatSequences(m_sPathSeq, aSeq);
				}
				else if (aCurrentArg.Name.equalsIgnoreCase(sPackageNameParam))
					bExtractSuccess = (aCurrentArg.Value >>= m_sPackageName);

				else if (aCurrentArg.Name.equalsIgnoreCase(sConverterParam))
					bExtractSuccess = (aCurrentArg.Value >>= xConverter);

				if (!bExtractSuccess)
					throw lang::IllegalArgumentException(
						ASCII("The argument ") + aCurrentArg.Name + ASCII(" could not be extracted."),
						Reference<XInterface>(),
						i
						);
			}
			// no check if the argument is known and valid. This would require to much testing
			// m_aSecurityOverride[aCurrentArg.Name] = aCurrentArg.Value;
		}
		
		// create a DocumentHandler
		m_xWriter = Reference< sax::XDocumentHandler >
			(m_xMSF->createInstance(
				ASCII("com.sun.star.xml.sax.Writer")), UNO_QUERY);

		// set value for the TypeConverterHelper
		if (xConverter.is())
		{
			m_aDocConverter.setConverter(xConverter);
		}
		else
		{
			// Create a TypeConverter
			uno::Reference<script::XTypeConverter> aConverter;
			m_aDocConverter.setConverter(
				aConverter.query(m_xMSF->createInstance(ASCII( "com.sun.star.script.Converter" ))));
		}
		m_aDocConverter.setHandler(m_xWriter);
	}
	
	// -----------------------------------------------------------------------------

	// XServiceInfo
	::rtl::OUString ODataExport::getImplementationName(  ) throw(RuntimeException)
	{
		AsciiServiceName p= staticServiceInfo.implementationName;

		return p ? rtl::OUString::createFromAscii(p) : rtl::OUString();
	}
	
	// -----------------------------------------------------------------------------

	sal_Bool ODataExport::supportsService( const ::rtl::OUString& ServiceName ) throw(RuntimeException)
	{
		AsciiServiceName *p= (AsciiServiceName *)&staticServiceInfo.serviceNames;
		if (p == 0)
			return false;

		while (*p != 0)
		{
			if (0 == ServiceName.compareToAscii(*p))
				return true;
			++p;
		}
		return false;
	}

	// -----------------------------------------------------------------------------

	Sequence< rtl::OUString > ODataExport::getServiceNames(ServiceInfo const* pInfo  ) throw(RuntimeException)
	{
		sal_Int32 const nCount = countServices(pInfo);

		Sequence< rtl::OUString > aServices( nCount );

		for(sal_Int32 i= 0; i < nCount; ++i)
			aServices[i] = rtl::OUString::createFromAscii(pInfo->serviceNames[i]);

		return aServices;
	}

	// -----------------------------------------------------------------------------

	Sequence< ::rtl::OUString > ODataExport::getSupportedServiceNames(  ) throw(RuntimeException)
	{
		return getServiceNames( &staticServiceInfo );
	}
	
	// -----------------------------------------------------------------------------

	// XDataExporter
	void ODataExport::exportData( const Reference< io::XOutputStream >& xOutputStream,
								  const Reference< lang::XComponent >& Component,
								  const Reference< io::XDataTransferEventListener >& xListener )
		throw(RuntimeException)
	{
		Reference< sax::XExtendedDocumentHandler> xHandler(m_xWriter, UNO_QUERY);
		Reference< io::XActiveDataSource> xDataSource( xHandler , UNO_QUERY );
		xDataSource->setOutputStream(xOutputStream);
		
		rtl::OUString sError;
		try
		{
			writeStart();
			io::DataTransferEvent aEvent;
			xListener->finished(aEvent);
		}
		catch( sax::SAXException &e )
		{
			sError = e.Message;
			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			
			io::DataTransferEvent aEvent;
			xListener->cancelled(aEvent);
		}
	}
	
	// -----------------------------------------------------------------------------

	void ODataExport::cancel() throw(RuntimeException)
	{
	}

	// -----------------------------------------------------------------------------
	// --------------------------- Export implementation ---------------------------
	// -----------------------------------------------------------------------------
	void ODataExport::writeStart()
		throw (sax::SAXException)
	{
		Reference< lang::XMultiServiceFactory > xCfgProvider( m_xMSF->createInstance(
			::rtl::OUString::createFromAscii("com.sun.star.configuration.ConfigurationProvider")),
														UNO_QUERY);
		
		if (!xCfgProvider.is())
		{
			OSL_ENSHURE(0, "No Configuration Provider");
		}
		
		m_xWriter->startDocument();
		
		AttributeListImpl *pList = new AttributeListImpl;
		Reference< sax::XAttributeList> rList( (sax::XAttributeList *) pList , UNO_QUERY );
		
		m_xWriter->startElement(m_sPackageName, rList);
		m_xWriter->ignorableWhitespace(rtl::OUString());
		
		// call n-time for every Path
		const OUString *pPath = m_sPathSeq.getConstArray();
		for(sal_Int32 i = 0;i<m_sPathSeq.getLength();i++)
		{
			AttributeListImpl *pList = new AttributeListImpl;
			Reference< sax::XAttributeList> rList( (sax::XAttributeList *) pList , UNO_QUERY );
		
			pList->addAttribute( ATTR_PATH,
								 XML_CDATA,
								 pPath[i]);
			
			m_xWriter->startElement(TAG_DATA, rList);
			m_xWriter->ignorableWhitespace(rtl::OUString());
			
			Sequence< Any > aArgs(1);
			aArgs[0] <<= pPath[i];
			
			Reference< XInterface > xCfg = xCfgProvider->createInstanceWithArguments(
				rtl::OUString::createFromAscii("com.sun.star.configuration.ConfigurationAccess"),
				aArgs);
			
			Reference< container::XNameAccess > xNameAccess(xCfg, UNO_QUERY);
			
			if (xNameAccess.is())
				handleNameAccess(xNameAccess);
		
			m_xWriter->endElement(TAG_DATA);
			m_xWriter->ignorableWhitespace(rtl::OUString());
		}
		
		m_xWriter->endElement(m_sPackageName);
		m_xWriter->ignorableWhitespace(rtl::OUString());
		
		m_xWriter->endDocument();
	}

	// -----------------------------------------------------------------------------
	// ------------------------------------ Test ------------------------------------
	// -----------------------------------------------------------------------------
#define ASCII_STRING(rtlOUString)	rtl::OUStringToOString(rtlOUString, RTL_TEXTENCODING_ASCII_US).getStr()
	static ::rtl::OUString	sRegistry = ::rtl::OUString::createFromAscii("applicat.rdb");

	// rtl::OUString XMLFilter::createOSLErrorString(osl::FileBase::RC eError);
	
	
// -----------------------------------------------------------------------------
	
	void ODataExport::handleNameAccess( Reference<container::XNameAccess> &_rxNameAccess)
	{
		Sequence<rtl::OUString> aSeq = _rxNameAccess->getElementNames();
		
		rtl::OUString *pStr = aSeq.getArray();
		for (int i=0;i<aSeq.getLength();i++)
		{
			rtl::OUString aPropertyName = pStr[i];
			Any aAny;
			aAny = _rxNameAccess->getByName(aPropertyName);
			TypeClass aTypeClass = aAny.getValueTypeClass();
			
			Sequence<rtl::OUString> aStrs;
			Reference<lang::XServiceInfo> xInfo(_rxNameAccess, UNO_QUERY);
			if (xInfo.is())
			{
				aStrs = xInfo->getSupportedServiceNames();
				rtl::OUString *pStr = aStrs.getArray();
				for(int i=0;i<aStrs.getLength();i++)
				{
					rtl::OUString aStr;
					aStr = pStr[i];
					volatile int dummy = 0;
				}
			}
			
			// 1. Problem, im ReadAccess gibt es keine Moeglichkeit ein SET zu erkennen.
			// 2. Problem, um an den TemplateNamen zu kommen gibt es z.Z. kein Interface.
			
			m_aDocConverter.startElement(aPropertyName, aAny);
			
			// special treatment for interface
			if (aAny.getValueTypeClass() == TypeClass_INTERFACE)
			{
				Reference< XInterface > xInterface;
				aAny >>= xInterface;
				
				Reference< container::XNameAccess > xNameAccess(xInterface, UNO_QUERY);
				if (xNameAccess.is())
				{
					handleNameAccess(xNameAccess);
				}
			}
			else
			{
				// Output other Types
				m_aDocConverter.writeValue(aAny);
			}
			m_aDocConverter.endElement(aPropertyName);
		}
	}
} // namespace configmgr
