/*************************************************************************
 *
 *  $RCSfile: dataimport.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_BEANS_PROPERTYVALUE_HPP_
#include <com/sun/star/beans/PropertyValue.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif

#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XACTIVEDATASOURCE_HPP_
#include <com/sun/star/io/XActiveDataSource.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HDL_
#include <com/sun/star/io/XOutputStream.hpp>
#endif

#ifndef CONFIGMGR_API_DATA_IMPORT_HXX_
#include "dataimport.hxx"
#endif

#include "xmltreebuilder.hxx"

#define THISREF() static_cast< ::cppu::OWeakObject* >(this)
#define ASCII(x) OUString::createFromAscii(x)

namespace configmgr
{
	namespace beans  = ::com::sun::star::beans;
	namespace io  = ::com::sun::star::io;
	namespace container  = ::com::sun::star::container;

	using namespace com::sun::star::uno;

	static AsciiServiceName const aDataImportServices[] = 
	{
		"com.sun.star.configuration.DataImport", 
		0
	};
	// special Imported one
	ServiceInfo const ODataImport::staticServiceInfo = 
	{ 
		"com.sun.star.configuration.configmgr.DataImport", 
		aDataImportServices 
	};

	const ServiceInfo* getDataImportServiceInfo()
	{
		return &ODataImport::staticServiceInfo;
	}
	
	Reference< XInterface > SAL_CALL instantiateDataImport(Reference< lang::XMultiServiceFactory > const& _rServiceManager )
	{
		return static_cast< ::cppu::OWeakObject* >(new ODataImport(_rServiceManager));
	}

// -----------------------------------------------------------------------------


	sal_Int32 ODataImport::countServices(ServiceInfo const* aInfo)
	{
		AsciiServiceName const* p= aInfo ? aInfo->serviceNames : 0;
		if (p == 0)
			return 0;

		sal_Int32 nCount = 0;
		while (*p != 0)
		{
			++nCount;
			++p;
		}

		return nCount;
	}

// -----------------------------------------------------------------------------

	void ODataImport::initialize( const Sequence< Any >& _rArguments )
		throw(Exception, RuntimeException)
	{
		osl::MutexGuard aGuard(m_aMutex);

		// if (m_pImpl || m_aSecurityOverride.size())
		//	throw Exception(::rtl::OUString::createFromAscii("The configuration provider has already been initialized."), THISREF());

		const Any* pArguments = _rArguments.getConstArray();
		beans::PropertyValue aCurrentArg;
		Reference< script::XTypeConverter > xConverter;

		for (sal_Int32 i=0; i<_rArguments.getLength(); ++i, ++pArguments)
		{
			if ((*pArguments) >>= aCurrentArg)
			{
				static const ::rtl::OUString sConverterParam(RTL_CONSTASCII_USTRINGPARAM("TypeConverter"));

				sal_Bool bExtractSuccess = sal_True;	// defaulted to TRUE, so we skip unknown arguments

				if (aCurrentArg.Name.equalsIgnoreCase(sConverterParam))
					bExtractSuccess = (aCurrentArg.Value >>= xConverter);

				if (!bExtractSuccess)
					throw lang::IllegalArgumentException(
						ASCII("The argument ") + aCurrentArg.Name + ASCII(" could not be extracted."),
						Reference<XInterface>(),
						i
						);
			}
			// no check if the argument is known and valid. This would require to much testing
			// m_aSecurityOverride[aCurrentArg.Name] = aCurrentArg.Value;
		}
		// create a DocumentHandler
		m_xParser = Reference< sax::XParser >
			( m_xMSF->createInstance(
				::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Parser")),
			  UNO_QUERY);
		
		// set DocumentHandler to parser
		if (!xConverter.is())
		{
			// Create a TypeConverter
			xConverter = xConverter.query(m_xMSF->createInstance(ASCII( "com.sun.star.script.Converter" ) ) );
		}
		
		configmgr::XMLDataExtractor *pHandler = new configmgr::XMLDataExtractor(xConverter, m_xMSF);
	
		uno::Reference<sax::XDocumentHandler> xHandler = pHandler;
		m_xParser->setDocumentHandler(xHandler);

		m_xConverter = xConverter;
	}
	
	// -----------------------------------------------------------------------------

	// XServiceInfo
	::rtl::OUString ODataImport::getImplementationName(  ) throw(RuntimeException)
	{
		AsciiServiceName p= staticServiceInfo.implementationName;

		return p ? rtl::OUString::createFromAscii(p) : rtl::OUString();
	}
	
	// -----------------------------------------------------------------------------

	sal_Bool ODataImport::supportsService( const ::rtl::OUString& ServiceName ) throw(RuntimeException)
	{
		AsciiServiceName *p= (AsciiServiceName *)&staticServiceInfo.serviceNames;
		if (p == 0)
			return false;

		while (*p != 0)
		{
			if (0 == ServiceName.compareToAscii(*p))
				return true;
			++p;
		}
		return false;
	}

	// -----------------------------------------------------------------------------

	Sequence< rtl::OUString > ODataImport::getServiceNames(ServiceInfo const* pInfo  ) throw(RuntimeException)
	{
		sal_Int32 const nCount = countServices(pInfo);

		Sequence< rtl::OUString > aServices( nCount );

		for(sal_Int32 i= 0; i < nCount; ++i)
			aServices[i] = rtl::OUString::createFromAscii(pInfo->serviceNames[i]);

		return aServices;
	}

	// -----------------------------------------------------------------------------

	Sequence< ::rtl::OUString > ODataImport::getSupportedServiceNames(  ) throw(RuntimeException)
	{
		return getServiceNames( &staticServiceInfo );
	}
	
	// -----------------------------------------------------------------------------

	// XDataImporter
	void ODataImport::importData( const Reference< io::XActiveDataSource >& xActiveSource,
								  const Reference< lang::XComponent >& xComponent,
								  const Reference< io::XDataTransferEventListener >& xListener )
		throw(RuntimeException)
	{
		// PRE:
		// POST:

		Reference <XInterface> xPipe = m_xMSF->createInstance( ASCII("com.sun.star.io.Pipe") );
		OSL_ENSHURE(xPipe.is(), "there is no pipe");
			
		// ??? OutputStream->Pipe->InputStream
		Reference<io::XOutputStream> xOutputStream(xPipe, UNO_QUERY);
		xActiveSource->setOutputStream(xOutputStream);

		// DocumentHandler *pHandler = new DocumentHandler();

		Reference<io::XInputStream> xPipeInput( xPipe, UNO_QUERY );

		m_pThread = new ParseThread(m_xParser, xListener, xPipeInput, xOutputStream);
		OSL_ENSHURE(m_pThread, "no Thread");
		m_pThread->create();
		// We MUST go back
	}
	
	// -----------------------------------------------------------------------------

	void ODataImport::cancel() throw(RuntimeException)
	{
		// TODO:
		// cancel isn't ready yet because what is with partially imported data
		// we won't create an inconsisten state, so we import all
		m_bCancel = true;
		if (m_pThread)
		{
			m_pThread->setCancel(true);
		}
	}

	// -----------------------------------------------------------------------------
	// --------------------------- Import implementation ---------------------------
	// -----------------------------------------------------------------------------

	void ParseThread::run()
	{
		setPriority(TPriority_BelowNormal);
		
		rtl::OUString sError;
		sax::InputSource aInputSource;
		aInputSource.aInputStream = m_xInputStream;
		
		try
		{
			m_xParser->parseStream(aInputSource);
			io::DataTransferEvent aEvent;
			m_xListener->finished(aEvent);
		}
		catch( sax::SAXParseException &e )
		{
			OUString sLine = OUString::valueOf(e.LineNumber);
			OUString aStr = ASCII("SAXParseException occured in ");
			sError = aStr + ASCII(" Line: (") + sLine + ASCII(")");
			
			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			
			io::DataTransferEvent aEvent;
			m_xListener->cancelled(aEvent);
		}
		catch( sax::SAXException &e )
		{
			sError = e.Message;
			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			
			io::DataTransferEvent aEvent;
			m_xListener->cancelled(aEvent);
		}
		catch( io::IOException &e )
		{
			sError = e.Message;
			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			
			io::DataTransferEvent aEvent;
			m_xListener->cancelled(aEvent);
		}
	}

} // namespace configmgr
