/*************************************************************************
 *
 *  $RCSfile: localsession.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>

#ifndef _CONFIGMGR_LOCAL_LOCAL_HXX_
#include "localsession.hxx"
#endif 

#ifndef _UTL_STREAM_OSLFILEWRAPPER_HXX_
#include "oslstream.hxx"
#endif

#ifndef _CONFIGMGR_XMLFORMATER_HXX
#include "xmltreebuilder.hxx"
#endif

#ifndef CONFIGMGR_CONFNAME_HXX_
#include "confname.hxx"
#endif

#ifndef _CONFIGMGR_SAXTOOLS_HXX_
#include "saxtools.hxx"
#endif

#ifndef _CONFIGMGR_XML_BUILDDOM_HXX_
#include "builddom.hxx"
#endif

#ifndef _CONFIGMGR_FILEHELPER_HXX_
#include "filehelper.hxx"
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XDocumentHandler.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_INPUTSOURCE_HPP_
#include <com/sun/star/xml/sax/InputSource.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_SAXPARSEEXCEPTION_HPP_
#include<com/sun/star/xml/sax/SAXParseException.hpp>
#endif

#ifndef _CONFIGMGR_STRCONVERTER_HXX_
#include "strconverter.hxx"
#endif

#ifndef _CONFIGMGR_TRACER_HXX_
#include "tracer.hxx"
#endif

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::xml;
using namespace ::com::sun::star::xml::sax;
using namespace ::com::sun::star::io;
using namespace ::vos;
using namespace ::std;
using namespace ::osl;
using namespace ::rtl;

#define ASCII(x) rtl::OUString::createFromAscii(x)

#define ENTER()	\
	::osl::MutexGuard aGuard(m_aMutex);	\
	m_eError = E_None;

#define ENTER_FULL_CHECK(methodname, handler)	\
	::osl::MutexGuard aGuard(m_aMutex);	\
	m_eError = E_None;	

namespace configmgr 
{
#pragma warning(disable: 4101)

// -----------------------------------------------------------------------------
	
	LocalSession::LocalSession(const Reference< XMultiServiceFactory >& _rxServiceProvider)
			: m_eError(E_None), m_xServiceProvider(_rxServiceProvider)
	{
		// create the writer and the parser we want to use
	}

// -----------------------------------------------------------------------------
	void LocalSession::cutPath(const OUString &_rPath)
	{
		// cut down name in Modulename and an vector of Path Names
		if (!m_aFilterVector.empty())
			m_aFilterVector.clear();

		ConfigurationName aName(_rPath, ConfigurationName::Absolute() );
		m_aModuleName = aName.moduleName();
		
		for(ConfigurationName::Iterator it = aName.begin();
			it != aName.end();
			++it)
		{
			m_aFilterVector.push_back(*it);
		}
	}
	
// -----------------------------------------------------------------------------
	OUString LocalSession::createFileURL(const OUString& _aModuleName) const
	{
		OUString aFullname = m_aRootPath + ASCII("/") + _aModuleName + ASCII(".xml");

		// convert filename to real URL
		OUString aURL;
		File aConvert(ASCII(""));
		aConvert.normalizePath(aFullname, aURL);

		return aURL;
	}
	

// -----------------------------------------------------------------------------
	void    LocalSession::openNode(const ::rtl::OUString& _rNodeAccessor,
				 sal_Int32 _nLevel,
				 const ::vos::ORef< IOpenObjectCallback >& _rHandler)
	{
		// PRE: 
		ENTER_FULL_CHECK(openNode, _rHandler);

		// convert org.openoffice. to com.sun.star.
		::rtl::OUString rNodeAccessor = changeToComSunStarPath(_rNodeAccessor);

		OSL_ENSHURE(_nLevel >= 0, "ORemoteSession::openNode : invalid level count !");
		OSL_ENSHURE(rNodeAccessor.getLength(), "ORemoteSession::openNode : invalid node path !");

		::rtl::OUString sNodePath, sUser;
		if (splitNodeAccessor(rNodeAccessor, sNodePath, sUser))
			OSL_ENSHURE(0, "LocalSession::openNode : foreign user access not supported !");
		
		// NameConvert 
		cutPath(sNodePath);
		OUString aURL = createFileURL(m_aModuleName);

		// connect parser and filter
		OUString sError;
		sal_Int32 nRet = XMLReadFilter::run(aURL, m_xServiceProvider, m_aFilterVector, _rHandler->getDataReader(), sError);

		m_aStatus.nCode = nRet;
		static const ::rtl::OUString sDummyId(RTL_CONSTASCII_USTRINGPARAM("1"));
		_rHandler->gotObjectId(sDummyId);
		m_aStatus.sMessage = sError;
		_rHandler->done(m_aStatus);
	}
	
// -----------------------------------------------------------------------------
	void    LocalSession::closeNode(const ::rtl::OUString& _rNodeId,
					  const ::vos::ORef< IRequestCallback >& _rHandler)
	
	{
		// POST: nothing todo, because all Files open and closed in his environment
		ENTER_FULL_CHECK(closeNode, _rHandler);
	}
	
// -----------------------------------------------------------------------------
	void    LocalSession::getNode(const ::rtl::OUString& _rNodeAccessor,
					const ::vos::ORef< IOpenObjectCallback >& _rHandler)
	{
		ENTER_FULL_CHECK(getNode, _rHandler);
		openNode(_rNodeAccessor, 4, _rHandler);
	}
	
// -----------------------------------------------------------------------------
	void    LocalSession::updateNode(const ::rtl::OUString& _rNodeId,
					   const ::rtl::OUString& _rNodeAccessor,
					   IDOMNodeDataProvider* _pNodeWriter,
					   const ::vos::ORef< IDataRequestCallback >& _rHandler)
	{
		ENTER_FULL_CHECK(updateNode, _rHandler);

		// convert org.openoffice. to com.sun.star.
		::rtl::OUString rNodeAccessor = changeToComSunStarPath(_rNodeAccessor);


		::rtl::OUString sNodePath, sUser;
		if (splitNodeAccessor(rNodeAccessor, sNodePath, sUser))
			OSL_ENSHURE(0, "LocalSession::updateNode : foreign user access not supported !");
		
		cutPath(sNodePath);
		
		OUString aURL = createFileURL(m_aModuleName);
		// write
		OUString aToURL = createFileURL( m_aModuleName + ASCII("_update"));
		FileHelper::tryToRemoveFile(aToURL);
		OUString sError;
		sal_Int32 nRet = doTreeUpdate(aURL, aToURL, sNodePath, _pNodeWriter, m_xServiceProvider, sError);
		m_aStatus.nCode = nRet;
		m_aStatus.sMessage = sError;
		_rHandler->done(m_aStatus);
	}
	
// -----------------------------------------------------------------------------
	void    LocalSession::addNode(const ::rtl::OUString& _rNodeId,
					const ::rtl::OUString& _rNodeAccessor,
					IDOMNodeDataProvider* _pNodeWriter,
					const ::vos::ORef< IRequestCallback >& _rHandler)
	{
		// PRE: 1. all elements in NodeAccessor must exist,
		//      2. the last element of NodeAccessor MUST be the first element in NodeWriter, it will be removed
		// the 2. invariant won't be checkt. (TODO...)

		ENTER_FULL_CHECK(addNode, _rHandler);

		// convert org.openoffice. to com.sun.star.
		::rtl::OUString rNodeAccessor = changeToComSunStarPath(_rNodeAccessor);


		::rtl::OUString sNodePath, sUser;
		if (splitNodeAccessor(rNodeAccessor, sNodePath, sUser))
			OSL_ENSHURE(sal_False, "LocalSession::addNode : foreign user access not supported !");
		
		cutPath(sNodePath);
		
		OUString aURL = createFileURL(m_aModuleName);

		// write
		OUString aToURL = createFileURL( m_aModuleName + ASCII("_addnode"));
		FileHelper::tryToRemoveFile(aToURL);
		OUString sError;
		sal_Int32 nRet = XMLWriteFilter::run(aURL, aToURL, m_xServiceProvider, m_aFilterVector, _pNodeWriter, sError);
		m_aStatus.nCode = nRet;
		m_aStatus.sMessage = sError;
		_rHandler->done(m_aStatus);
	}
	
// -----------------------------------------------------------------------------
	void    LocalSession::addUser(const ::rtl::OUString& rUser_,
					IDOMNodeDataProvider* _pNodeWriter,
					const ::vos::ORef< IRequestCallback >& _rHandler, const ::rtl::OUString& _rNodeId)
	{
		OSL_ENSHURE(false, "AddUser not supported with local session - local session is single user only. Trying to addNode instead");
		addNode(_rNodeId, getUserAdminAccessor(), _pNodeWriter, _rHandler);
	}
// -----------------------------------------------------------------------------
	void    LocalSession::deleteNode(const ::rtl::OUString& _rNodeId,
					   const ::rtl::OUString& _rNodeAccessor,
					   const ::vos::ORef< IRequestCallback >& _rHandler)
	{
		ENTER_FULL_CHECK(deleteNode, _rHandler);

		::rtl::OUString sNodePath, sUser;
		if (splitNodeAccessor(_rNodeAccessor, sNodePath, sUser))
			OSL_ENSHURE(sal_False, "LocalSession::deleteNode : foreign user access not supported !");
		
		cutPath(sNodePath);
		
		OUString aURL = createFileURL(m_aModuleName);

		// write
		OUString aToURL = createFileURL( m_aModuleName + ASCII("_removenode"));

		FileHelper::tryToRemoveFile(aToURL);
		OUString sError;
		sal_Int32 nRet = XMLRemoveFilter::run(aURL, aToURL, m_xServiceProvider, m_aFilterVector, sError);
		m_aStatus.nCode = nRet;
		m_aStatus.sMessage = sError;
		_rHandler->done(m_aStatus);
	}
	
// -----------------------------------------------------------------------------
	sal_Bool LocalSession::open(const ::rtl::OUString& _rRootPath)
	{
		m_aRootPath = _rRootPath;
		return true;
	}
	
	void     LocalSession::close(const ::vos::ORef< IRequestCallback >& _rHandler)
	{
	}
	
	sal_Bool LocalSession::isOpen()
	{
		sal_Bool nRet = false;
		if (m_aRootPath.getLength() > 0) nRet = true;
		return nRet;
	}
	

// --------------------------------- XMLFilter ---------------------------------
	XMLFilter::XMLFilter(const std::vector< rtl::OUString > &aFilterVector)
			:m_aFilterVector(aFilterVector)
	{
	}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// ------------------------------- XMLReadFilter -------------------------------
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

	XMLReadFilter::XMLReadFilter( const uno::Reference< sax::XDocumentHandler > &xWriter,
								  const std::vector<rtl::OUString> &aFilterVector )
			: m_xWriter(xWriter), XMLFilter(aFilterVector)
	{
	}


// -----------------------------------------------------------------------------
// static
	inline const sal_Char* OUStringToChar(rtl::OUString &aStr)
	{
		return rtl::OUStringToOString(aStr,RTL_TEXTENCODING_ASCII_US).getStr();
	}
	
	sal_Int32 XMLReadFilter::run(const OUString &_aFromURL,
								 const Reference< XMultiServiceFactory >& _rxServiceProvider,
								 const std::vector<rtl::OUString> &aFilterVector,
								 const uno::Reference< sax::XDocumentHandler > &xWriter,
								 /* [OUT] */ OUString &sError)
	{
		File aFile(_aFromURL);
		FileBase::RC eError = aFile.open(OpenFlag_Read);
		if (eError != osl_File_E_None)
		{
			::rtl::OUString aStr(ASCII("Local Loader: Can't open URL: "));
			aStr += _aFromURL;
			CFG_TRACE_WARNING_NI(OUStringToChar(aStr));
			sError = aStr;
			return 1;
		}

		// ------------- sax read --------------
		// create the writer and the parser we want to use
		
		Reference< sax::XParser > xParser;
		xParser = Reference< sax::XParser > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Parser")), UNO_QUERY);
		
		Reference<XInputStream> xInputStream = new configmgr::OSLInputStreamWrapper(aFile);
		
		// connect stream to input stream to the parser
		InputSource aInputSource;
		Reference<XInputStream> xPipeInput( xInputStream, UNO_QUERY );
		aInputSource.aInputStream = xPipeInput;
		

		XMLReadFilter *pTest = new XMLReadFilter(xWriter, aFilterVector);

		// get filter
		Reference<XDocumentHandler> xFilter = pTest;
		
		// convert Files to new format
		// if (_aFromURL.indexOf(ASCII("com.sun.star")) != -1)
		// {
			xFilter = new OValueConverter(xFilter, true /* OValueConverter::CONVERT_TO_NEW */);
		// }
		
		// connect parser and filter
		xParser->setDocumentHandler( xFilter );

		// parse
		sal_Int16 nRet = 0;
		try
		{
			xParser->parseStream( aInputSource );
		}
		catch( SAXParseException &e )
		{
			OUString sLine = OUString::valueOf(e.LineNumber);
			OUString aStr = ASCII("SAXParseException occured in ");
			sError = aStr + ASCII(" Line: (") + sLine + ASCII(")");

			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			nRet = 3;
		}
		catch( SAXException &e )
		{
			sError = e.Message;
			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			nRet = 4;
		}
		catch( IOException &e )
		{
			sError = e.Message;
			OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
			nRet = 5;
		}
		return nRet;
	}

	void XMLReadFilter::startDocument(void)
		throw (sax::SAXException, uno::RuntimeException)
	{
		m_eState = UNKNOWN_ELEMENT;
	}
	
	void XMLReadFilter::endDocument(void)
		throw(sax::SAXException, uno::RuntimeException)
	{
	}
	
	void XMLReadFilter::startElement(const OUString& aName,
									   const uno::Reference< sax::XAttributeList > &xAttrList)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (FORWARDING_DATA == m_eState)
		{
			++m_nElementDepth;
			m_xWriter->startElement(aName, xAttrList);
			// TODO : think about handling the exceptions ....
		}
		else if (m_eState == JUMP_OVER)
		{
			++m_nIgnoreLevel;
		}
		else
		{
			m_aNameStack.push_back(aName);
			sal_Int32 nLevel = 0;
			for(vector<OUString>::const_iterator it = m_aNameStack.begin();
				it != m_aNameStack.end();
				++it)
			{
				OUString aStr = m_aFilterVector[nLevel++];
				if (*it != aStr)
				{
					// the compare goes wrong, all under this will be ignored
					m_eState = JUMP_OVER;
					m_nIgnoreLevel = 1;
				}
			}

			if (nLevel == m_aFilterVector.size() && m_eState != JUMP_OVER)
			{
				// Start Forwarding
				m_ePushedState = m_eState;
				m_eState = FORWARDING_DATA;
				m_nElementDepth = 1;	// 1, not 0 !
				m_xWriter->startDocument();
				m_xWriter->startElement(aName, xAttrList);
			}
		}
	}
	
	void XMLReadFilter::endElement(const OUString& aName)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (FORWARDING_DATA == m_eState)
		{
			if (0 == --m_nElementDepth)
			{	// we left the node description
				m_eState = m_ePushedState;
				// call this method again, this will bring us into the else-branch below
				m_xWriter->endElement(aName);
				endElement(aName);
				m_xWriter->endDocument();
			}
			else
			{	// it's one of the elements of the node value description
				m_xWriter->endElement(aName);
				// TODO : think about handling the exceptions ....
			}
		}
		else if (m_eState == JUMP_OVER)
		{
			--m_nIgnoreLevel;
			if (m_nIgnoreLevel == 0)
			{
				m_eState = UNKNOWN_ELEMENT;
				m_aNameStack.pop_back();
			}
		}
		else
		{
			if (!m_aNameStack.empty())
			{
				m_aNameStack.pop_back();
			}
		}
	}
	
	void XMLReadFilter::characters(const OUString& aChars)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->characters(aChars);
		// TODO : think about handling the exceptions ....
	}
	
	
	void XMLReadFilter::ignorableWhitespace(const OUString& aWhitespaces)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->ignorableWhitespace(aWhitespaces);
		// TODO : think about handling the exceptions ....
	}

	void XMLReadFilter::processingInstruction(const OUString& aTarget,
												const OUString& aData)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->processingInstruction(aTarget, aData);
		// TODO : think about handling the exceptions ....
	}
	
	void XMLReadFilter::setDocumentLocator(const uno::Reference< sax::XLocator > &xLocator)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->setDocumentLocator(xLocator);
		// TODO : think about handling the exceptions ....
	}


// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// ------------------------------- XMLWriteFilter -------------------------------
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

	namespace uno = com::sun::star::uno;
	namespace sax = com::sun::star::xml::sax;

	XMLWriteFilter::XMLWriteFilter(const uno::Reference< sax::XDocumentHandler > &xWriter,
								   const uno::Reference< sax::XParser >          &xReader,
								   const std::vector<rtl::OUString> &aFilterVector,
								   IDOMNodeDataProvider* _pNodeWriter)
			: m_xWriter(xWriter),m_xReader(xReader),
			  m_pNodeWriter(_pNodeWriter),
			  XMLFilter(aFilterVector)
	{
	}
	
	
// -----------------------------------------------------------------------------
// static

	sal_Int32 XMLWriteFilter::run(const OUString &_aFromURL,
								  const OUString &_aToURL,
								  const Reference< XMultiServiceFactory >& _rxServiceProvider,
								  const std::vector<rtl::OUString> &aFilterVector,
								  IDOMNodeDataProvider* _pNodeWriter,
								  /* [OUT] */ OUString &sError)
	{
		// PRE: FromURL, ToURL must be real URL to exists files
		// BACK: 0: no error
		
		sal_Int32 nRet = 0;
		{
			File aFile(_aFromURL);
			FileBase::RC eError = aFile.open(OpenFlag_Read);
			if (eError != osl_File_E_None)
			{
				sError = ASCII("XMLWriteFilter: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aFromURL;

				OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSHURE(0, aStr.getStr());
				return 1;
			}
		
			File aWriteFile(_aToURL);
			eError = aWriteFile.open(OpenFlag_Write | OpenFlag_Create);
			// TODO: E_EXIST
			if (eError != osl_File_E_None)
			{
				sError = ASCII("XMLWriteFilter: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aToURL;

				OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSHURE(0, aStr.getStr());
				return 2;
			}
		
			// ------------- sax read --------------
			// create the writer and the parser we want to use

			Reference< sax::XParser > xParser;
			xParser = Reference< sax::XParser > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Parser")), UNO_QUERY);

			Reference<XInputStream> xInputStream = new configmgr::OSLInputStreamWrapper(aFile);
		
		// connect stream to input stream to the parser
			InputSource aInputSource;
			Reference<XInputStream> xPipeInput( xInputStream, UNO_QUERY );
			aInputSource.aInputStream = xPipeInput;
		
			// ------------- sax write -------------

			Reference<XOutputStream> xOutputStream = new configmgr::OSLOutputStreamWrapper(aWriteFile);
		
			Reference< sax::XDocumentHandler > xWriter;
			xWriter = Reference< sax::XDocumentHandler > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Writer")), UNO_QUERY);

			Reference< XExtendedDocumentHandler> xHandler(xWriter, UNO_QUERY);
			Reference< XActiveDataSource> xDataSource( xHandler , UNO_QUERY );
			xDataSource->setOutputStream(xOutputStream);

			// Write in new Format
			// if (_aToURL.indexOf(ASCII("org.openoffice")) != -1)
			// {
				xWriter = new OValueConverter(xWriter, true);
			// }
			
			XMLWriteFilter *pTest = new XMLWriteFilter(xWriter, xParser, aFilterVector, _pNodeWriter);

			// get filter
			Reference<XDocumentHandler> xFilter = pTest;

			// connect parser and filter
			xParser->setDocumentHandler( xFilter );

			// parse
			try
			{
				xParser->parseStream( aInputSource );
			}
			catch( SAXParseException &e )
			{
				OUString sLine = OUString::valueOf(e.LineNumber);
				OUString aStr = ASCII("SAXParseException occured in ");
				sError = aStr + ASCII(" Line: (") + sLine + ASCII(")");
				OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
				nRet = 3;
			}
			catch( SAXException &e )
			{
				sError = e.Message;
				OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
				nRet = 4;
			}
			catch( IOException &e )
			{
				sError = e.Message;
				OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
				nRet = 5;
			}

			aWriteFile.close();
			aFile.close();

		} // we MUST leave File scope

		if (nRet == 0)
		{
			nRet = FileHelper::createBackupRemoveAndRename(_aFromURL, _aToURL);
		}
		else
		{
			File::remove(_aToURL);
		}

		return nRet;
	}
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::startDocument(void)
		throw (sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->startDocument();
		bInserted = false;
		m_eState = UNKNOWN_ELEMENT;
	}
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::endDocument(void)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->endDocument();
	}
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::startElement(const OUString& aName,
									   const uno::Reference< sax::XAttributeList > &xAttribs)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == JUMP_OVER)
		{
			++m_nIgnoreLevel;
		}
		else
		{
			m_aNameStack.push_back(aName);
			sal_Int32 nLevel = 0;

/* TestCode
#ifdef DEBUG
			// DEBUG ONLY
			OUString aCompleteName;
			for(vector<OUString>::const_iterator it2 = m_aNameStack.begin();
				it2 != m_aNameStack.end();
				++it2)
			{
				aCompleteName += *it2;
				aCompleteName += ASCII("/");
			}
#endif 
#ifdef DEBUG
			// DEBUG ONLY
			OUString aCompleteName2;
			for(vector<OUString>::const_iterator it3 = m_aFilterVector.begin();
				it3 != m_aFilterVector.end();
				++it3)
			{
				aCompleteName2 += *it3;
				aCompleteName2 += ASCII("/");
			}
#endif
*/
			OUString aStr;

			if (m_aNameStack.size() == m_aFilterVector.size())
			{
				sal_Bool bAllEqual = true;
				for(vector<OUString>::const_iterator it = m_aFilterVector.begin();
					it != m_aFilterVector.end();
					++it)
				{
					aStr = m_aNameStack[nLevel++];
					if (*it != aStr)
					{
						// the compare goes wrong, all under this will be ignored
						bAllEqual = false;
						break;
					}
				}
				if (bAllEqual)
				{
					// Insert is possible, look forward.
					m_aInsertFilter.clear();
					for(vector<OUString>::const_iterator it = m_aFilterVector.begin();
						it != m_aFilterVector.end();
						++it)
						m_aInsertFilter.push_back(*it);
				}
			}

			// Check, if in we found a real comparation, so we must throw()
			{
				sal_Bool bAllEqual = true;
				nLevel = 0;
				for(vector<OUString>::const_iterator it = m_aFilterVector.begin();
					it != m_aFilterVector.end();
					++it)
				{
					aStr = m_aNameStack.size() < nLevel ? m_aNameStack[nLevel] : ::rtl::OUString();
					++nLevel;
					if (*it != aStr)
					{
						// the compare goes wrong, all under this will be ignored
						bAllEqual = false;
						break;
					}
				}
				if (bAllEqual)
				{
					// Value is allready set, use updateNode()
					throw SAXException();
				}
			}
		}
		m_xWriter->startElement(aName, xAttribs);
	}
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::endElement(const OUString& aName)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (!m_aNameStack.empty())
		{
/* TestCode
			volatile int size = m_aNameStack.size();
			volatile int size2 = m_aInsertFilter.size();
#ifdef DEBUG
			// DEBUG ONLY
			OUString aCompleteName;
			for(vector<OUString>::const_iterator it2 = m_aNameStack.begin();
				it2 != m_aNameStack.end();
				++it2)
			{
				aCompleteName += *it2;
				aCompleteName += ASCII("/");
			}
#endif 
#ifdef DEBUG
			// DEBUG ONLY
			OUString aCompleteName2;
			for(vector<OUString>::const_iterator it3 = m_aInsertFilter.begin();
				it3 != m_aInsertFilter.end();
				++it3)
			{
				aCompleteName2 += *it3;
				aCompleteName2 += ASCII("/");
			}
#endif
*/
			if (m_aNameStack.size() == m_aInsertFilter.size())
			{
				OUString aStr;
				sal_Bool  bAllEqual = true;
				sal_Int32 nLevel = 0;
				for(vector<OUString>::const_iterator it = m_aNameStack.begin();
					it != m_aNameStack.end();
					++it)
				{
					aStr = m_aInsertFilter[nLevel++];
					if (*it != aStr)
					{
						// the compare goes wrong, all under this will be ignored
						bAllEqual = false;
						break;
					}
				}
				if (bAllEqual)
				{
					// m_xWriter->ignorableWhitespace(ASCII("\x0A"));
					// by definition, the top level element the writer writes is the same as the last one
					// specified in the path for the write, so we have to remove this redundance
					::vos::ORef< OTreeRootTagKiller > xRedundanceKiller = new OTreeRootTagKiller(m_pNodeWriter, aName);
					xRedundanceKiller->writeNodeData(m_xWriter);
				}
			}

			m_aNameStack.pop_back();
		}
		m_xWriter->endElement(aName);
	}
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::characters(const OUString& aChars)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->characters(aChars);
	}
	
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::ignorableWhitespace(const OUString& aWhitespaces)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->ignorableWhitespace(aWhitespaces);
	}

// -----------------------------------------------------------------------------
	void XMLWriteFilter::processingInstruction(const OUString& aTarget,
												const OUString& aData)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->processingInstruction(aTarget, aData);
	}
	
// -----------------------------------------------------------------------------
	void XMLWriteFilter::setDocumentLocator(const uno::Reference< sax::XLocator > &xLocator)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->setDocumentLocator(xLocator);
	}
	
	
	
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// ------------------------------ XMLRemoveFilter ------------------------------
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

	XMLRemoveFilter::XMLRemoveFilter(const uno::Reference< sax::XDocumentHandler > &xWriter,
									 const uno::Reference< sax::XParser >          &xReader,
									 const std::vector<rtl::OUString> &aFilterVector)
			: m_xWriter(xWriter), m_xReader(xReader),
			  XMLFilter(aFilterVector),m_bDropNextCartReturn(false),
			  m_nIgnoreLevel(0)
	{
	}
	
	
// -----------------------------------------------------------------------------
// static

	sal_Int32 XMLRemoveFilter::run(const OUString &_aFromURL,
								   const OUString &_aToURL,
								   const Reference< XMultiServiceFactory >& _rxServiceProvider,
								   const std::vector<rtl::OUString> &aFilterVector,
								   /* [OUT] */ OUString &sError)
	{
		// PRE: FromURL, ToURL must be real URL to exists files
		// BACK: 0: no error
		
		sal_Int32 nRet = 0;
		{
			File aFile(_aFromURL);
			FileBase::RC eError = aFile.open(OpenFlag_Read);
			if (eError != osl_File_E_None)
			{
				sError = ASCII("XMLRemoveFilter: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aFromURL;
				
				OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSHURE(0, aStr.getStr());
				return 1;
			}
			
			File aWriteFile(_aToURL);
			eError = aWriteFile.open(OpenFlag_Write | OpenFlag_Create);
			// TODO: E_EXIST
			if (eError != osl_File_E_None)
			{
				sError = ASCII("XMLRemoveFilter: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aToURL;

				OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSHURE(0, aStr.getStr());
				return 2;
			}
			
			// ------------- sax read --------------
			// create the writer and the parser we want to use
			
			Reference< sax::XParser > xParser;
			xParser = Reference< sax::XParser > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Parser")), UNO_QUERY);
			
			Reference<XInputStream> xInputStream = new configmgr::OSLInputStreamWrapper(aFile);
			
			// connect stream to input stream to the parser
			InputSource aInputSource;
			Reference<XInputStream> xPipeInput( xInputStream, UNO_QUERY );
			aInputSource.aInputStream = xPipeInput;
			
			// ------------- sax write -------------
			
			Reference<XOutputStream> xOutputStream = new configmgr::OSLOutputStreamWrapper(aWriteFile);
			
			Reference< sax::XDocumentHandler > xWriter;
			xWriter = Reference< sax::XDocumentHandler > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Writer")), UNO_QUERY);
			
			Reference< XExtendedDocumentHandler> xHandler(xWriter, UNO_QUERY);
			Reference< XActiveDataSource> xDataSource( xHandler , UNO_QUERY );
			xDataSource->setOutputStream(xOutputStream);
			
			// Write in new Format
			// if (_aToURL.indexOf(ASCII("org.openoffice")) != -1)
			// {
				xWriter = new OValueConverter(xWriter, true);
			// }

			XMLRemoveFilter *pTest = new XMLRemoveFilter(xWriter, xParser, aFilterVector);
			
			// get filter
			Reference<XDocumentHandler> xFilter = pTest;
			
			// connect parser and filter
			xParser->setDocumentHandler( xFilter );
			
			// parse
			try
			{
				xParser->parseStream( aInputSource );
			}
			catch( SAXParseException &e )
			{
				OUString sLine = OUString::valueOf(e.LineNumber);
				sError = ASCII("SAXParseException occured in ") + ASCII(" Line: (") + sLine + ASCII(")");
				// ((((sError += aStr) += ASCII(" Line: (") ) += sLine) += ASCII(")"));
				OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
				nRet = 3;
			}
			catch( SAXException &e )
			{
				sError = e.Message;
				OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
				nRet = 4;
			}
			catch( IOException &e )
			{
				sError = e.Message;
				OSL_ENSHURE(0, rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US).getStr());
				nRet = 5;
			}
			
			aWriteFile.close();
			aFile.close();
		} // we MUST leave File scope
		
		if (nRet == 0)
		{
			nRet = FileHelper::createBackupRemoveAndRename(_aFromURL, _aToURL);
		}
		else
		{
			File::remove(_aToURL);
		}
		return nRet;
	}
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::startDocument(void)
		throw (sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->startDocument();
		m_eState = FORWARDING_DATA;
	}
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::endDocument(void)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_xWriter->endDocument();
	}
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::startElement(const OUString& aName,
									   const uno::Reference< sax::XAttributeList > &xAttribs)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_bDropNextCartReturn = false;
		if (m_eState == JUMP_OVER)
		{
			++m_nIgnoreLevel;
		}
		else
		{
			m_aNameStack.push_back(aName);
			sal_Int32 nLevel = 0;

/*
#ifdef DEBUG
			// DEBUG ONLY
			OUString aCompleteName;
			for(vector<OUString>::const_iterator it2 = m_aNameStack.begin();
				it2 != m_aNameStack.end();
				++it2)
			{
				aCompleteName += *it2;
				aCompleteName += ASCII("/");
			}
#endif 
*/
			OUString aStr;
			// Test:
			// if Stack == FilterVector
			
			if (m_aNameStack.size() == m_aFilterVector.size())
			{
				sal_Bool bAllEqual = true;
				for(vector<OUString>::const_iterator it = m_aNameStack.begin();
					it != m_aNameStack.end();
					++it)
				{
					aStr = m_aFilterVector[nLevel++];
					if (*it != aStr)
					{
						// the compare goes wrong, all under this will be ignored
						bAllEqual = false;
						break;
					}
				}
				if (bAllEqual)
				{
					m_eState = JUMP_OVER;
					m_nIgnoreLevel = 1;
				}
			}
		}
		if (m_eState == FORWARDING_DATA)
			m_xWriter->startElement(aName, xAttribs);
	}
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::endElement(const OUString& aName)
		throw(sax::SAXException, uno::RuntimeException)
	{
		m_bDropNextCartReturn = false;
		if (m_eState == FORWARDING_DATA)
			m_xWriter->endElement(aName);

		if (m_eState == JUMP_OVER)
		{
			--m_nIgnoreLevel;
			if (m_nIgnoreLevel == 0)
			{
				m_eState = FORWARDING_DATA;
				m_bDropNextCartReturn = true;
				m_aNameStack.pop_back();
			}
		}
		else if (!m_aNameStack.empty())
		{
			m_aNameStack.pop_back();
		}
	}
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::characters(const OUString& aChars)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_bDropNextCartReturn)
		{
			m_bDropNextCartReturn = false;
			return;
		}
		if (m_eState == FORWARDING_DATA)
			m_xWriter->characters(aChars);
	}
	
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::ignorableWhitespace(const OUString& aWhitespaces)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->ignorableWhitespace(aWhitespaces);
	}

// -----------------------------------------------------------------------------
	void XMLRemoveFilter::processingInstruction(const OUString& aTarget,
												const OUString& aData)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->processingInstruction(aTarget, aData);
	}
	
// -----------------------------------------------------------------------------
	void XMLRemoveFilter::setDocumentLocator(const uno::Reference< sax::XLocator > &xLocator)
		throw(sax::SAXException, uno::RuntimeException)
	{
		if (m_eState == FORWARDING_DATA)
			m_xWriter->setDocumentLocator(xLocator);
	}

	
} // namespace configmgr


