/*************************************************************************
 *
 *  $RCSfile: mergeupdates.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <stl/vector>
#include <stl/stack>

#ifndef _COM_SUN_STAR_XML_SAX_XPARSER_HPP_
#include <com/sun/star/xml/sax/XParser.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XEXTENDEDDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XExtendedDocumentHandler.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XACTIVEDATASOURCE_HPP_
#include <com/sun/star/io/XActiveDataSource.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_XOUTPUTSTREAM_HPP_
#include <com/sun/star/io/XOutputStream.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_SAXPARSEEXCEPTION_HDL_
#include <com/sun/star/xml/sax/SAXParseException.hdl>
#endif

// header for define OSL_ASSERT
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx>
#endif

#ifndef _CONFIGMGR_XML_UPDATEDOM_HXX_
#include "updatedom.hxx"
#endif
#ifndef _CONFIGMGR_XML_BUILDDOM_HXX_
#include "builddom.hxx"
#endif
#ifndef _CONFIGMGR_OSLSTREAM_HXX_
#include "oslstream.hxx"
#endif
#ifndef CONFIGMGR_CONFNAME_HXX_
#include "confname.hxx"
#endif

#ifndef _CONFIGMGR_LOCAL_LOCAL_HXX_
#include "localsession.hxx"
#endif

#ifndef _CONFIGMGR_FILEHELPER_HXX_
#include "filehelper.hxx"
#endif

#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

#include "strconverter.hxx"

#define ASCII(x) rtl::OUString::createFromAscii(x)

namespace configmgr 
{
	namespace css = com::sun::star;
	namespace uno = css::uno;
	namespace sax = css::xml::sax;
	namespace io = css::io;

	using namespace osl;
	
	class HandlerState;
	struct HandlerContext
	{
		std::stack<HandlerState*> states;
		uno::Reference<sax::XDocumentHandler> xOut;

		HandlerContext(uno::Reference<sax::XDocumentHandler> xOut_) : xOut(xOut_) {}
		void pushHandler(HandlerState& rHandler, const rtl::OUString &aName, const uno::Reference< sax::XAttributeList > &xAttributes);
	};

/////////////////////////////////////////////////////////////////////////////////////////////
	class HandlerState
	{
	protected:
		HandlerState() : m_nStarted(0) {}
		virtual ~HandlerState();

	public:
		bool started() const { return m_nStarted > 0; }

		void startElement(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr);
		void endElement(HandlerContext& ctx, const rtl::OUString &name);

		virtual void characters(HandlerContext& ctx, const rtl::OUString &text)
			{
				ctx.xOut->characters(text);
			}
	protected:
		virtual void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr) = 0;
		virtual void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr) = 0;


		virtual void endChild(HandlerContext& ctx, const rtl::OUString &name) = 0;
		virtual void endSelf(HandlerContext& ctx, const rtl::OUString &name) = 0;

	private:
		bool start(HandlerContext& ctx);
		bool stop(HandlerContext& ctx);
		int m_nStarted;
	};

///////////////////////////////////////////////////////////////////
	inline HandlerState::~HandlerState() { OSL_ASSERT(!started()); }
	inline void HandlerState::startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
		{
			ctx.xOut->startElement(name, xAttr);
		}
	inline void HandlerState::startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
		{
		}


	inline void HandlerState::endChild(HandlerContext& ctx, const rtl::OUString &name)
		{
		}
	inline void HandlerState::endSelf(HandlerContext& ctx, const rtl::OUString &name)
		{
			ctx.xOut->endElement(name);
		}
///////////////////////////////////////////////////////////////////
	void HandlerState::startElement(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
		{
			if (start(ctx))
				startSelf(ctx, name, xAttr); 
			else
				startChild(ctx, name, xAttr);
		}
	void HandlerState::endElement(HandlerContext& ctx, const rtl::OUString &name)
		{
			if (stop(ctx)) 
			{
				endSelf(ctx, name);
				if (!ctx.states.empty())
				{
					OSL_ASSERT(ctx.states.top()->m_nStarted > 1);
					ctx.states.top()->endElement(ctx, name);
				}
				delete this;
			}
			else
				endChild(ctx, name);
		}
	bool HandlerState::start(HandlerContext& ctx)
		{
			OSL_ASSERT(m_nStarted >= 0);
			if (m_nStarted++ != 0)
				return false;

			ctx.states.push(this);
			return true;
		}
	bool HandlerState::stop(HandlerContext& ctx)
		{
			OSL_ASSERT(m_nStarted > 0);
			if (--m_nStarted != 0)
				return false;

			OSL_ASSERT(this == ctx.states.top());
			ctx.states.pop();
			return true;
		}

///////////////////////////////////////////////////////////////////
	struct DocumentStateBase : HandlerState
	{
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT( name.getLength() == 0 );
				OSL_ASSERT( ! xAttr.is() );
				ctx.xOut->startDocument();
			}
		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				if (HandlerState* pDocElementHandler = createChildHandler(ctx,name,xAttr))
				{
					ctx.pushHandler(*pDocElementHandler, name, xAttr);
				}
			}
		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				HandlerState::endChild(ctx,name);			
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT( name.getLength() == 0 );
				ctx.xOut->endDocument();
			}
	protected:
		virtual HandlerState* createChildHandler(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr) = 0;
	};
///////////////////////////////////////////////////////////////////
	struct SilentStateBase : HandlerState
	{
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT( name.getLength() == 0 );
				OSL_ASSERT( ! xAttr.is() );
			}
		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				if (HandlerState* pDocElementHandler = createChildHandler(ctx,name,xAttr))
				{
					ctx.pushHandler(*pDocElementHandler, name, xAttr);
				}
			}
		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				HandlerState::endChild(ctx,name);			
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT( name.getLength() == 0 );
			}
	protected:
		virtual HandlerState* createChildHandler(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr) = 0;
	};
///////////////////////////////////////////////////////////////////
	void HandlerContext::pushHandler(HandlerState& rHandler, const rtl::OUString &aName, const uno::Reference< sax::XAttributeList > &xAttributes)
	{
		OSL_ASSERT(!rHandler.started());
		rHandler.startElement(*this,aName,xAttributes);
	}
///////////////////////////////////////////////////////////////////

	struct Handler : cppu::WeakImplHelper1<sax::XDocumentHandler>
	{
		HandlerContext context;

		Handler(uno::Reference<sax::XDocumentHandler> xOut_) : context(xOut_) {}

		virtual HandlerState* createDocumentState() = 0;

		virtual void SAL_CALL startDocument() throw (sax::SAXException, uno::RuntimeException)
			{ 
				startDocument( *createDocumentState()); 
			}
		void startDocument(HandlerState& rInitialState, const rtl::OUString &name = rtl::OUString()) { 
			OSL_ASSERT(context.states.empty()); 
			context.pushHandler( rInitialState,name, 0 ); 
		}
		virtual void SAL_CALL startElement(const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			throw(sax::SAXException, uno::RuntimeException)
			{
				state().startElement(context,name,xAttr); 
			}
		virtual void SAL_CALL endElement(const rtl::OUString &name)
			throw(sax::SAXException, uno::RuntimeException)
			{
				state().endElement(context,name); 
			}
		virtual void SAL_CALL characters(const rtl::OUString &text)
			throw(sax::SAXException, uno::RuntimeException)
			{
				state().characters(context,text); 
			}

		virtual void SAL_CALL endDocument() 
			throw(sax::SAXException, uno::RuntimeException)
			{ 
				context.states.top()->endElement( context, rtl::OUString() ); 		
				OSL_ASSERT(context.states.empty()); 
			}

		virtual void SAL_CALL ignorableWhitespace(const class rtl::OUString & aWS) 
		{
			context.xOut->ignorableWhitespace(aWS);
		}
		virtual void SAL_CALL processingInstruction(const class rtl::OUString & sTarget, const class rtl::OUString &sData) 
		{
			context.xOut->processingInstruction(sTarget,sData);
		}
		virtual void SAL_CALL setDocumentLocator(const uno::Reference< sax::XLocator> &xLoc) 
		{
			context.xOut->setDocumentLocator(xLoc);
		}

	private:
		HandlerState& state() { OSL_ASSERT(!context.states.empty()); return *context.states.top(); }

	};

////////////////////////////////////////////////////////////////////////////
	HandlerState* createElementHandler(ElementNode* pNode, const rtl::OUString &aName);

////////////////////////////////////////////////////////////////////////////

	struct WriteThruState : HandlerState
	{
		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				ctx.xOut->startElement(name, xAttr);
			}
		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				ctx.xOut->endElement(name);
			}
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				HandlerState::startSelf(ctx,name,xAttr);
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				HandlerState::endSelf(ctx,name);
			}
	};

	struct InnerNodeState : HandlerState
	{
		InnerNode& m_rNode;

		InnerNodeState(InnerNode& pNode) : m_rNode(pNode) {}

		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				ElementNode* pChildNode = m_rNode.getChild(name);
				HandlerState* pNextState = createElementHandler(pChildNode, name);
				ctx.pushHandler(*pNextState, name, xAttr);
			}

		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				HandlerState::endChild(ctx,name);			
			}
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				HandlerState::startSelf(ctx,name,xAttr);
				OSL_ASSERT(!m_rNode.marked());
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				m_rNode.dumpChildren(ctx.xOut,true);
				m_rNode.mark();
				HandlerState::endSelf(ctx,name);
			}
	};

	struct SearchNodeState : HandlerState
	{
		rtl::OUString m_name;
		rtl::OUString m_search;
		rtl::OUString m_remain;
		sal_Int32 m_len;
		ElementNode* const m_root;

		SearchNodeState(TreeInfo const& aTree);
		SearchNodeState(rtl::OUString const& sPath, ElementNode* root);

		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				HandlerState* pNextHandler = 0;
				if (name == m_search)
				{
					if (m_remain.getLength() != 0)
					{
						pNextHandler = new SearchNodeState(name, m_remain, m_root);
					}
					else
					{
						pNextHandler = createElementHandler(m_root, name);
					}
				}
				else
				{
					pNextHandler = new WriteThruState();
				}
				OSL_ASSERT(pNextHandler != 0);
				ctx.pushHandler(*pNextHandler, name, xAttr);
			}

		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				HandlerState::endChild(ctx,name);			
			}
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT(name == m_name);
				HandlerState::startSelf(ctx,name,xAttr);
				if (m_root)
					m_root->clearMarks();
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				if (m_remain.getLength() == 0 && m_root != 0 && !m_root->marked())
				{
					OSL_ENSHURE(false,"Update concerns only non-existing node: Dumping it");
					m_root->dump(ctx.xOut,true);
				}

				OSL_ASSERT(name == m_name);
				HandlerState::endSelf(ctx,name);
			}
	private:
		SearchNodeState(rtl::OUString const& sName, rtl::OUString const& sPath, ElementNode* root);
		static bool split(rtl::OUString const sPath, rtl::OUString& sFirst, rtl::OUString& sRemain);

	};
	bool SearchNodeState::split(rtl::OUString const sPath, rtl::OUString& sFirst,rtl::OUString& sRemain)
	{
		OSL_ASSERT(!sPath.getLength() == 0);
		int nSplit = sPath.indexOf('/');

		if (nSplit >= 0)
		{
			OSL_ENSHURE(nSplit != sPath.getLength()-1,"Path should not end in /");	

			// need temporary (for case of aliasing)
			rtl::OUString sBeforeSplit = sPath.copy(0, nSplit);
			
			sRemain= sPath.copy(nSplit+1);
			sFirst = sBeforeSplit;
		}
		else
		{
			sFirst = sPath;
			OSL_ASSERT(sRemain.getLength() == 0);
		}
		return sFirst.getLength() != 0;
	}

	SearchNodeState::SearchNodeState(TreeInfo const& aTree)
			: m_root(aTree.root())
	{
		OSL_PRECOND(m_root != 0, "SearchNodeState: No tree to find");

		OSL_VERIFY(  split(aTree.getPathToRoot(), m_name, m_search) ); // name should not be empty (did the path start with a slash ?)
		OSL_VERIFY(  split(m_search, m_search, m_remain) );					// need something to search
	}

	SearchNodeState::SearchNodeState(rtl::OUString const& sPath, ElementNode* root)
			: m_root(root)
	{
		OSL_PRECOND(m_root != 0, "SearchNodeState: No tree to find");

		OSL_VERIFY(  split(sPath, m_name, m_search ) ); // name should not be empty (did the path start with a slash ?)
		OSL_VERIFY(  split(m_search, m_search, m_remain) ); // need something to search
	}

	SearchNodeState::SearchNodeState(rtl::OUString const& sName, rtl::OUString const& sPath, ElementNode* root)
			: m_root(root)
			, m_name(sName)
	{
		OSL_PRECOND(m_root != 0, "SearchNodeState: No tree to find");
		OSL_ASSERT(sName.indexOf(ConfigurationName::delimiter) == -1);

		OSL_VERIFY(split(sPath, m_search, m_remain)); // need something to search
	}

	struct LeafNodeState : HandlerState
	{
		LeafNode& m_rNode;

		LeafNodeState(LeafNode& rNode) : m_rNode(rNode) {}
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT(name == XMLValue::sValueName() || name == XMLValue::sDefaultName());
				OSL_ASSERT(name == m_rNode.getName());
				OSL_ASSERT(!m_rNode.marked()); 
				m_rNode.writeValues(ctx.xOut);
			}
		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT(false);
			}
		void characters(HandlerContext& ctx, const rtl::OUString &text)
			{
				// ignore - we have something better to offer
			}

		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT(false);
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				m_rNode.values.contents.clear();
				m_rNode.mark(); // was handled
			}
	};

	struct NullLeafNodeState : HandlerState
	{
		NullLeafNodeState() {}
		void startSelf(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT(name == XMLValue::sValueName() || name == XMLValue::sDefaultName());
				// no output
			}
		void startChild(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				OSL_ASSERT(false);
			}
		void characters(HandlerContext& ctx, const rtl::OUString &text)
			{
				// ignore - we have something better to offer
			}

		void endChild(HandlerContext& ctx, const rtl::OUString &name)
			{
				OSL_ASSERT(false);
			}
		void endSelf(HandlerContext& ctx, const rtl::OUString &name)
			{
				// no output
			}
	};


////////////////////////////////////////////////////////////////////////////
	HandlerState* createElementHandler(ElementNode* pNode, const rtl::OUString &aName)
	{
		if (pNode == 0) 
		{
			if (aName == XMLValue::sValueName())
				return new NullLeafNodeState();
			else
				return new WriteThruState();
		}
		else if (InnerNode* pInner = pNode->asInnerNode())
		{
			OSL_ASSERT(aName != XMLValue::sValueName());
			return new InnerNodeState(*pInner);
		}
		else if (LeafNode* pLeaf = pNode->asLeafNode())
		{
			return new LeafNodeState(*pLeaf);
		}

		OSL_ASSERT(!"Unknown Node type found" );
		return 0;
	}

// Tree merge handler
	template <class Base>
	struct TreeMergerStartState : Base
	{
		TreeInfo& m_rChanges;

		TreeMergerStartState(TreeInfo& rChanges) : m_rChanges(rChanges)
			{
			}
		virtual HandlerState* createChildHandler(HandlerContext& ctx, const rtl::OUString &name, const uno::Reference< sax::XAttributeList > &xAttr)
			{
				if (name == m_rChanges.getPathToRoot())
				{
					ElementNode* pNode = m_rChanges.root();
					OSL_ASSERT(!pNode || name == pNode->getName());
					return createElementHandler(pNode,name);
				}
				else
				{
					return new SearchNodeState(m_rChanges);
				}
			}

	};

	struct TreeMerger : Handler
	{
		TreeInfo& m_rChanges;
		TreeMerger(TreeInfo& rChanges, uno::Reference<sax::XDocumentHandler> xOut_) 
				: Handler(xOut_), m_rChanges(rChanges)
			{
		
			}
		~TreeMerger()
			{
		
			}

		void startWithoutDocument()
			{
				startDocument(*createSilentState() );
			}
		void endWithoutDocument()
			{
				endDocument();
			}
	private:
		virtual HandlerState* createDocumentState() 
			{
				return new TreeMergerStartState<DocumentStateBase>(m_rChanges);
			}

		virtual HandlerState* createSilentState() 
			{
				return new TreeMergerStartState<SilentStateBase>(m_rChanges);
			}

	};


// -----------------------------------------------------------------------------
	void BuilderContext::pushBuilder(BuilderState& rBuilder, const rtl::OUString &aName, const uno::Reference <sax::XAttributeList> & xAttributes)
	{
		OSL_ASSERT(!rBuilder.started());
		rBuilder.startElement(*this,aName,xAttributes);
	}
	
// -----------------------------------------------------------------------------
	BuilderState* createNodeBuilder(BuilderContext&, InnerNode& rParent, const rtl::OUString &name, const uno::Reference <sax::XAttributeList> &)
	{
		if (name == XMLValue::sValueName() || name == XMLValue::sDefaultName())
			return new LeafNodeBuilder(rParent);
		else
			return new InnerNodeBuilder(rParent);
	}
	
// -----------------------------------------------------------------------------
	TreeInfo::TreeInfo(rtl::OUString const& sPathToRoot, ElementNode* pRoot)
		: m_sPathToRoot( sPathToRoot )
		, m_pRoot(pRoot)
	{

	}

// -----------------------------------------------------------------------------
// usage:

	int doTreeUpdate(const rtl::OUString& _aFromURL,
					 const rtl::OUString& _aToURL,
					 const rtl::OUString & aBasePath,
					 IDOMNodeDataProvider* pDataProvider,
					 const uno::Reference< lang::XMultiServiceFactory >& _rxServiceProvider,
					 /* OUT */ rtl::OUString &sError)
	{

		sal_Int32 nRet = 0;
		
		// Filehandling
		{
		
			File aFile(_aFromURL);
			FileBase::RC eError = aFile.open(OpenFlag_Read);
			if (eError != osl_File_E_None)
			{
				sError = ASCII("doTreeUpdate: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aFromURL;
				
				rtl::OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSHURE(0, aStr.getStr());
				return 1;
			}
			
			File aWriteFile(_aToURL);
			eError = aWriteFile.open(OpenFlag_Write | OpenFlag_Create);
			// TODO: E_EXIST
			if (eError != osl_File_E_None)
			{
				sError = ASCII("doTreeUpdate: ") + FileHelper::createOSLErrorString(eError) + ASCII("\n with URL: ") + _aToURL;
				
				rtl::OString aStr = rtl::OUStringToOString(sError,RTL_TEXTENCODING_ASCII_US);
				OSL_ENSHURE(0, aStr.getStr());
				return 1;
			}
			// ------------- sax read --------------
			// create the writer and the parser we want to use

			uno::Reference< sax::XParser > xParser;
			xParser = uno::Reference< sax::XParser > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Parser")), uno::UNO_QUERY);
		
			uno::Reference<io::XInputStream> xInputStream = new configmgr::OSLInputStreamWrapper(aFile);
		
			// connect stream to input stream to the parser
			sax::InputSource aInputSource;
			uno::Reference<io::XInputStream> xPipeInput( xInputStream, uno::UNO_QUERY );
			aInputSource.aInputStream = xPipeInput;
		
			// ------------- sax write -------------
		
			uno::Reference<io::XOutputStream> xOutputStream = new configmgr::OSLOutputStreamWrapper(aWriteFile);
		
			uno::Reference< sax::XDocumentHandler > xWriter;
			xWriter = uno::Reference< sax::XDocumentHandler > (_rxServiceProvider->createInstance( ::rtl::OUString::createFromAscii("com.sun.star.xml.sax.Writer")), uno::UNO_QUERY);
		
			uno::Reference< sax::XExtendedDocumentHandler> xHandler(xWriter, uno::UNO_QUERY);
			uno::Reference< io::XActiveDataSource> xDataSource( xHandler, uno::UNO_QUERY );
			xDataSource->setOutputStream(xOutputStream);

			// Write in new Format
			// if (_aToURL.indexOf(ASCII("org.openoffice")) != -1)
			// {
				xWriter = new OValueConverter(xWriter, true);
			// }
			
			
			ChangeDOMBuilder* pChangeDOM = new ChangeDOMBuilder();
			uno::Reference<sax::XDocumentHandler> xKeepAlive = pChangeDOM;

			// Convert new to old
			xKeepAlive = new OValueConverter(xKeepAlive, false);

			pChangeDOM->startDocument();
			pDataProvider->writeNodeData(xKeepAlive);
			pChangeDOM->endDocument();
		
		// mergeTree(xParser, aInput, aChangesInfo, xOutput); 
		// }

			TreeInfo aChangesInfo( aBasePath,pChangeDOM->getRoot());

		// int mergeTree(sax::XParser xParser,sax::InputStream inFile, TreeInfo& changes, sax::XDocumentHandler xOutput)
		// {

			uno::Reference<sax::XDocumentHandler> xMergeHandler = new TreeMerger(aChangesInfo, xWriter);
			xMergeHandler = new OValueConverter(xMergeHandler, false);

			xParser->setDocumentHandler( xMergeHandler );
			// parse
			try
			{
				xParser->parseStream( aInputSource );
			}
			catch( sax::SAXParseException &e )
			{
				rtl::OUString sLine = rtl::OUString::valueOf(e.LineNumber);
				sError = ASCII("SAXParseException occured in ") + ASCII(" Line: (") + sLine + ASCII(")");
				nRet = 3;
			}
			catch( sax::SAXException &e )
			{
				sError = ASCII("SAXException occured.");
				nRet = 4;
			}
			catch( uno::RuntimeException &e )
			{
				sError = ASCII("IOException occured.");
				nRet = 5;
			}
		
			aWriteFile.close();
			aFile.close();
		} // we MUST leave File scope
		

		if (nRet == 0)
		{
			nRet = FileHelper::createBackupRemoveAndRename(_aFromURL, _aToURL);
		}
		else
		{
			File::remove(_aToURL);
		}

		return nRet;
	}
	
} // namespace configmgr

