/*************************************************************************
 *
 *  $RCSfile: updatedom.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "updatedom.hxx"

#ifndef _CONFIGMGR_MISC_ATTRIBUTES_HXX_
#include "attributes.hxx"
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//#include "msc_msg.hxx"

namespace configmgr
{
	namespace css = com::sun::star;
	namespace sax = css::xml::sax;
	namespace uno = css::uno;

	using rtl::OUString;

// -----------------------------------------------------------------------------
	ElementNode::ElementNode(rtl::OUString sName, const uno::Reference< sax::XAttributeList > &xAttrList)
		:m_sNodeName(sName)
		,m_marked(false) 
	{
		AttributeListImpl *pList = new AttributeListImpl;
		uno::Reference< sax::XAttributeList> rList( (sax::XAttributeList *) pList , uno::UNO_QUERY );

		sal_Int16 nAttrCount = xAttrList.is() ? xAttrList->getLength() : 0;
		for( sal_Int16 i=0; i < nAttrCount; i++ )
		{
			pList->addAttribute( xAttrList->getNameByIndex( i ),
								 xAttrList->getTypeByIndex( i ),
								 xAttrList->getValueByIndex( i )); // write value to stream
		}
		m_attributes = rList;
	}
		
// -----------------------------------------------------------------------------
	void ElementNode::clearMarks() 
	{
		mark(false); 
	}

// -----------------------------------------------------------------------------
	bool ElementNode::dump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly)
	{
		OSL_ASSERT(xOut.is());
		if (marked() && bUnmarkedOnly)
			return false;

		doDump(xOut,bUnmarkedOnly);
		m_marked = marked() || bUnmarkedOnly;
		return true;
	}

// -----------------------------------------------------------------------------
	void ElementNode::doDump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly)
	{
		OSL_ASSERT(xOut.is());
		OSL_ASSERT(!(marked() && bUnmarkedOnly));

		xOut->startElement(m_sNodeName,m_attributes);	
		xOut->endElement(m_sNodeName);
	}

// -----------------------------------------------------------------------------
	
	struct SearchElementNode : ElementNode
	{
		SearchElementNode(const rtl::OUString &aName) : ElementNode(aName) {}

		virtual LeafNode* asLeafNode() { return 0; }
		virtual InnerNode* asInnerNode() { return 0; }
	};
	

// -----------------------------------------------------------------------------
	bool InnerNode::addChild(ElementNode* pNode)
	{
		OSL_ASSERT(pNode);
		rtl::OUString aName = pNode->getName();
		std::pair<Children::iterator, bool> aInserted = m_children.insert(pNode);
		return aInserted.second;
	}
		
// -----------------------------------------------------------------------------
	ElementNode* InnerNode::getChild(rtl::OUString aName)
	{
		SearchElementNode searchObj(aName);
		Children::iterator it = m_children.find(&searchObj);
		if (it == m_children.end())
			return NULL;

		OSL_ASSERT(*it != 0);
		return *it;
	}
		
// -----------------------------------------------------------------------------
	void InnerNode::clearMarks() 
	{
		ElementNode::clearMarks(); 
		for(Children::iterator it = m_children.begin(); it != m_children.end(); ++it)
		{
			OSL_ASSERT(*it != 0);
			(**it).clearMarks();
		}
	}

// -----------------------------------------------------------------------------
	bool InnerNode::dumpChildren(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly)
	{
		OSL_ENSHURE(!(marked() && bUnmarkedOnly),"Warning: Dumping unmarked children of marked node - possible sweep problem");
		bool bRet = false;
		for(Children::iterator it = m_children.begin(); it != m_children.end(); ++it)
		{
			OSL_ASSERT(*it != 0);
			if ((**it).dump(xOut,bUnmarkedOnly))
				bRet = true;
		}
		return bRet; // output anything ?
	}

// -----------------------------------------------------------------------------
	void InnerNode::doDump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly)
	{
		OSL_ASSERT(xOut.is());
		OSL_ASSERT(!(marked() && bUnmarkedOnly));

		xOut->startElement(m_sNodeName,m_attributes);
		xOut->ignorableWhitespace( OUString() );

		if (dumpChildren(xOut,bUnmarkedOnly))
			xOut->ignorableWhitespace( OUString() );

		xOut->endElement(m_sNodeName);
	}
//		ElementNode* removeChild(rtl::OUString aName);

// -----------------------------------------------------------------------------
	
	void XMLValue::write(uno::Reference <sax::XDocumentHandler> &xHandler, rtl::OUString tagName, const uno::Reference< sax::XAttributeList > & attrs) const
	{
		xHandler->startElement(tagName, attrs);
		xHandler->characters(content);
		xHandler->endElement(tagName);
	}

// -----------------------------------------------------------------------------
	void XMLValueList::write(uno::Reference<sax::XDocumentHandler> &xHandler, rtl::OUString tagName, const uno::Reference< sax::XAttributeList > & attrs) const
	{
		for(std::vector<XMLValue>::const_iterator it = contents.begin();
			it != contents.end();
			++it)
			it->write(xHandler,tagName, attrs);
	}

// -----------------------------------------------------------------------------
	LeafNode::LeafNode(rtl::OUString name) : ElementNode(name)
	{
		OSL_ASSERT(name == XMLValue::sDefaultName() || name == XMLValue::sValueName());
	}

// -----------------------------------------------------------------------------
	void LeafNode::writeValues(uno::Reference<sax::XDocumentHandler> &xHandler) const
	{
		values.write(xHandler, m_sNodeName, m_attributes);
	}

// -----------------------------------------------------------------------------
	void LeafNode::doDump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly)
	{
		OSL_ASSERT(xOut.is());
		OSL_ASSERT(!(marked() && bUnmarkedOnly));
		writeValues(xOut);
	}
// -----------------------------------------------------------------------------
} // namespace configmgr
