/*************************************************************************
 *
 *  $RCSfile: updatedom.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:13:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONFIGMGR_XML_UPDATEDOM_HXX_
#define _CONFIGMGR_XML_UPDATEDOM_HXX_

#ifndef _CONFIGMGR_MISC_ATTRIBUTES_HXX_
#include "attributes.hxx"
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef _COM_SUN_STAR_XML_SAX_XDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XDocumentHandler.hpp>
#endif
#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif
#ifndef __SGI_STL_SET
#include <stl/set>
#endif

namespace configmgr
{
	namespace css = com::sun::star;
	namespace sax = css::xml::sax;
	namespace uno = css::uno;

	struct InnerNode;
	struct LeafNode;

// -----------------------------------------------------------------------------
	struct ElementNode
	{
		bool					m_marked;
		rtl::OUString			m_sNodeName;
		uno::Reference< sax::XAttributeList > m_attributes;	//copy of the attributes

		ElementNode(rtl::OUString sName):m_sNodeName(sName),m_marked(false) {}
		ElementNode(rtl::OUString sName, const uno::Reference< sax::XAttributeList > &xAttrList);
		virtual ~ElementNode() {}		

		rtl::OUString getName() const {return m_sNodeName;}

		virtual LeafNode* asLeafNode() = 0;
		virtual InnerNode* asInnerNode() = 0;

		void mark(bool bMark= true) { m_marked = bMark; }
		bool marked() const { return m_marked; }
		virtual void clearMarks();

		bool dump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly);
	private:
		virtual void doDump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly);
	};

// -----------------------------------------------------------------------------
	struct ltNode
	{
		bool operator()(const configmgr::ElementNode* n1, const configmgr::ElementNode* n2) const
			{
				// return strcmp(n1->GetName(), n2->GetName()) < 0;
				return (n1->getName().compareTo(n2->getName()) < 0) ? 1 : 0;
			}
	};
// -----------------------------------------------------------------------------
	struct InnerNode : ElementNode
	{
		InnerNode() : ElementNode(rtl::OUString()) {}

		InnerNode(rtl::OUString sName, const uno::Reference< sax::XAttributeList > &xAttr) : ElementNode(sName, xAttr) {}

		bool addChild(ElementNode* pNode);
		ElementNode* getChild(rtl::OUString aName);
		ElementNode* removeChild(rtl::OUString aName);
		
		virtual void clearMarks();

		bool dumpChildren(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly);

		typedef std::set<ElementNode*, ltNode> Children;
		Children& children() { return m_children; }
	private:
		virtual void doDump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly);

		virtual LeafNode* asLeafNode() { return 0; }
		virtual InnerNode* asInnerNode() { return this; }

	private:
		Children m_children;
	};

// -----------------------------------------------------------------------------
	struct XMLValue
	{
		static rtl::OUString sDefaultName() { return rtl::OUString::createFromAscii("defaultvalue"); }
		static rtl::OUString sValueName()	{ return rtl::OUString::createFromAscii("value"); }

		rtl::OUString content;

		XMLValue() : content() {}
		XMLValue(rtl::OUString const& content_) : content(content_) {}
		XMLValue& operator+=(rtl::OUString const& moreText) { content += moreText; return *this; }
	
		void write(uno::Reference <sax::XDocumentHandler> &xHandler, rtl::OUString tagName, const uno::Reference< sax::XAttributeList > & attrs) const;
	};

// -----------------------------------------------------------------------------
	struct XMLValueList
	{
		std::vector<XMLValue> contents;

		XMLValue& appendNew() 
		{ 
			contents.push_back(XMLValue());
			return contents.back(); 
		}

		XMLValue& appendNew(rtl::OUString const& aText) 
		{ 
			contents.push_back(XMLValue(aText));
			return contents.back();
		}

		XMLValue& appendCurrent(rtl::OUString const& aText) 
		{
			contents.back() += aText;
			return contents.back();
		}

		void write(uno::Reference<sax::XDocumentHandler> &xHandler, rtl::OUString tagName, const uno::Reference< sax::XAttributeList > & attrs) const;
	};

	struct LeafNode : ElementNode
	{
		LeafNode(rtl::OUString name);

		XMLValueList values;

		XMLValue& appendNew()								{ return values.appendNew(); }
		XMLValue& appendNew(rtl::OUString const& aText)		{ return values.appendNew(aText); }
		XMLValue& appendCurrent(rtl::OUString const& aText) { return values.appendCurrent(aText); }

		void writeValues(uno::Reference<sax::XDocumentHandler> &xHandler) const;


	private:
		virtual LeafNode* asLeafNode() { return this; }
		virtual InnerNode* asInnerNode() { return 0; }

		virtual void doDump(uno::Reference<sax::XDocumentHandler>& xOut, bool bUnmarkedOnly);
	};

	class TreeInfo
	{
		rtl::OUString m_sPathToRoot;
		ElementNode* m_pRoot;
	public:
		TreeInfo(rtl::OUString const& sPathToRoot, ElementNode* pRoot);

		rtl::OUString getPathToRoot() const { return m_sPathToRoot; }
		ElementNode* root() const { return m_pRoot; }
	};
} // namespace configmgr

#endif
