/*************************************************************************
 *
 *  $RCSfile: simplepropertycontainer.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:14:19 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _CONNECTIVITY_SIMPLEPROPERTYCONTAINER_HXX_
#include "connectivity/simplepropertycontainer.hxx"
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef _UNO_DATA_H_
#include <uno/data.h>
#endif
#ifndef _COM_SUN_STAR_UNO_GENFUNC_H_
#include <com/sun/star/uno/genfunc.h>
#endif

#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HPP_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::beans;
using namespace connectivity;


//==========================================================================
//= OSimplePropertyContainer
//==========================================================================
//--------------------------------------------------------------------------
OSimplePropertyContainer::OSimplePropertyContainer(::cppu::OBroadcastHelper& _rBHelper)
	:OSimplePropertyContainer_Base(_rBHelper)
	,m_bAlreadyAccessed(sal_False)
{
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::registerProperty(const ::rtl::OUString& _rName, sal_Int32 _nHandle,
		sal_Int32 _nAttributes, void* _pPointerToMember, const Type& _rMemberType)
{
	OSL_ENSHURE(!m_bAlreadyAccessed, "OSimplePropertyContainer::registerProperty : invalid call, the property sequence was already accessed !");
	OSL_ENSHURE((_nAttributes & PropertyAttribute::MAYBEVOID) == 0,
		"OSimplePropertyContainer::registerProperty : don't use this for properties which may be void ! There is a method called \"registerMayBeVoidProperty\" for this !");
	OSL_ENSHURE(!_rMemberType.equals(::getCppuType(static_cast< Any* >(NULL))),
		"OSimplePropertyContainer::registerProperty : don't give my the type of an uno::Any ! Really can't handle this !");
	OSL_ENSHURE(_pPointerToMember,
		"OSimplePropertyContainer::registerProperty : you gave me nonsense : the pointer must be non-NULL");

	PropertyDescription aNewProp;
	aNewProp.sName = _rName;
	aNewProp.nHandle = _nHandle;
	aNewProp.nAttributes = _nAttributes;
	aNewProp.eLocated = PropertyDescription::ltDerivedClassRealType;
	aNewProp.aLocation.pDerivedClassMember = _pPointerToMember;
	aNewProp.aType = _rMemberType;

	implPushBackProperty(aNewProp);
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::registerMayBeVoidProperty(const ::rtl::OUString& _rName, sal_Int32 _nHandle, sal_Int32 _nAttributes,
		Any* _pPointerToMember, const Type& _rExpectedType)
{
	OSL_ENSHURE(!m_bAlreadyAccessed, "OSimplePropertyContainer::registerMayBeVoidProperty : invalid call, the property sequence was already accessed !");
	OSL_ENSHURE((_nAttributes & PropertyAttribute::MAYBEVOID) != 0,
		"OSimplePropertyContainer::registerMayBeVoidProperty : why calling this when the attributes say nothing about may-be-void ?");
	OSL_ENSHURE(!_rExpectedType.equals(::getCppuType(static_cast< Any* >(NULL))),
		"OSimplePropertyContainer::registerMayBeVoidProperty : don't give my the type of an uno::Any ! Really can't handle this !");
	OSL_ENSHURE(_pPointerToMember,
		"OSimplePropertyContainer::registerMayBeVoidProperty : you gave me nonsense : the pointer must be non-NULL");

	_nAttributes &= PropertyAttribute::MAYBEVOID;

	PropertyDescription aNewProp;
	aNewProp.sName = _rName;
	aNewProp.nHandle = _nHandle;
	aNewProp.nAttributes = _nAttributes;
	aNewProp.eLocated = PropertyDescription::ltDerivedClassAnyType;
	aNewProp.aLocation.pDerivedClassMember = _pPointerToMember;
	aNewProp.aType = _rExpectedType;

	implPushBackProperty(aNewProp);
}

	
//--------------------------------------------------------------------------
void OSimplePropertyContainer::registerPropertyNoMember(const ::rtl::OUString& _rName, sal_Int32 _nHandle, sal_Int32 _nAttributes,
		const Type& _rType, void* _pInitialValue)
{
	OSL_ENSHURE(!m_bAlreadyAccessed, "OSimplePropertyContainer::registerMayBeVoidProperty : invalid call, the property sequence was already accessed !");
	OSL_ENSHURE(!_rType.equals(::getCppuType(static_cast< Any* >(NULL))),
		"OSimplePropertyContainer::registerPropertyNoMember : don't give my the type of an uno::Any ! Really can't handle this !");
	OSL_ENSHURE(!_pInitialValue || ((_nAttributes & PropertyAttribute::MAYBEVOID) != 0),
		"OSimplePropertyContainer::registerPropertyNoMember : you should not ommit the initial value if the property can't be void ! This will definitivly crash later !");

	PropertyDescription aNewProp;
	aNewProp.sName = _rName;
	aNewProp.nHandle = _nHandle;
	aNewProp.nAttributes = _nAttributes;
	aNewProp.eLocated = PropertyDescription::ltHoldMyself;
	aNewProp.aLocation.nOwnClassVectorIndex = m_aHoldProperties.size();
	if (_pInitialValue)
		m_aHoldProperties.push_back(Any(_pInitialValue, _rType));
	else
		m_aHoldProperties.push_back(Any());
	aNewProp.aType = _rType;

	implPushBackProperty(aNewProp);
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::implPushBackProperty(const PropertyDescription& _rProp)
{
	sal_Int32 nOldLen = m_aProperties.size();
	m_aProperties.resize(nOldLen + 1);
	PropertyDescription* pProps = m_aProperties.begin() + nOldLen - 1;

	sal_Int32 nPos = nOldLen;
	while (nPos && (_rProp.nHandle < pProps->nHandle))
	{
		*(pProps+1) = *pProps;
		--pProps;
		--nPos;
	}

	m_aProperties[nPos] = _rProp;
}

//--------------------------------------------------------------------------
sal_Bool OSimplePropertyContainer::convertFastPropertyValue(
	Any& _rConvertedValue, Any& _rOldValue, sal_Int32 _nHandle, const Any& _rValue ) throw (IllegalArgumentException)
{
#ifdef DBG_UTIL
	m_bAlreadyAccessed = sal_True;
#endif
	sal_Bool bModified = sal_False;

	PropertiesIterator aPos = searchHandle(_nHandle);
	if (aPos == m_aProperties.end())
	{
		OSL_ASSERT("OSimplePropertyContainer::convertFastPropertyValue : unknown handle !");
		return bModified;
	}

	switch (aPos->eLocated)
	{
		case PropertyDescription::ltHoldMyself:
		case PropertyDescription::ltDerivedClassAnyType:
		{
			sal_Bool bMayBeVoid = ((aPos->nAttributes & PropertyAttribute::MAYBEVOID) != 0);
			// argument check
			if	(	!	(	(bMayBeVoid && !_rValue.hasValue())				// void is allowed if the attribute says so
						||	(_rValue.getValueType().equals(aPos->aType))	// else the types have to be equal
						)
				)
				throw IllegalArgumentException();

			Any* pPropContainer = NULL;
				// the pointer to the any which holds the property value, no matter if located in the derived clas
				// or in out vector

			if (PropertyDescription::ltHoldMyself == aPos->eLocated)
			{
				OSL_ENSHURE(aPos->aLocation.nOwnClassVectorIndex < m_aHoldProperties.size(),
					"OSimplePropertyContainer::convertFastPropertyValue : invalid position !");
				OSL_ENSHURE(m_aHoldProperties[aPos->aLocation.nOwnClassVectorIndex].hasValue(),
					"OSimplePropertyContainer::convertFastPropertyValue : a non-void prop is empty !");
				pPropContainer = m_aHoldProperties.begin() + aPos->aLocation.nOwnClassVectorIndex;
			}
			else
				pPropContainer = reinterpret_cast<Any*>(aPos->aLocation.pDerivedClassMember);

			// check if the new value differs from the current one
			if (!pPropContainer->hasValue() || !_rValue.hasValue())
				bModified = pPropContainer->hasValue() != _rValue.hasValue();
			else
				bModified = !uno_equalData(const_cast<void*>(pPropContainer->getValue()), aPos->aType.getTypeLibType()->pType, const_cast<void*>(_rValue.getValue()), aPos->aType.getTypeLibType()->pType, cpp_queryInterface,cpp_release);

			if (bModified)
			{
				_rOldValue = *pPropContainer;
				_rConvertedValue = _rValue;
			}
		}
		break;
		case PropertyDescription::ltDerivedClassRealType:
			// here we don't allow MAYBEVOID properties, so the type check is simple
			if (!_rValue.getValueType().equals(aPos->aType))
				throw IllegalArgumentException();

			bModified = !uno_equalData(aPos->aLocation.pDerivedClassMember, aPos->aType.getTypeLibType()->pType, const_cast<void*>(_rValue.getValue()), aPos->aType.getTypeLibType()->pType, cpp_queryInterface,cpp_release);
			if (bModified)
			{
				_rOldValue.setValue(aPos->aLocation.pDerivedClassMember, aPos->aType);
				_rConvertedValue = _rValue;
			}
			break;
	}

	return bModified;
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::setFastPropertyValue_NoBroadcast(sal_Int32 _nHandle, const Any& _rValue) throw (Exception)
{
#ifdef DBG_UTIL
	m_bAlreadyAccessed = sal_True;
#endif
	PropertiesIterator aPos = searchHandle(_nHandle);
	if (aPos == m_aProperties.end())
	{
		OSL_ASSERT("OSimplePropertyContainer::setFastPropertyValue_NoBroadcast : unknown handle !");
		return;
	}

	switch (aPos->eLocated)
	{
		case PropertyDescription::ltHoldMyself:
			m_aHoldProperties[aPos->aLocation.nOwnClassVectorIndex] = _rValue;
			break;
		case PropertyDescription::ltDerivedClassAnyType:
			*reinterpret_cast< Any* >(aPos->aLocation.pDerivedClassMember) = _rValue;
			break;
		case PropertyDescription::ltDerivedClassRealType:
			OSL_ENSHURE(_rValue.getValueType().equals(aPos->aType),
				"OSimplePropertyContainer::setFastPropertyValue_NoBroadcast : ooops .... the value is of the wrong type !");
			uno_copyData(aPos->aLocation.pDerivedClassMember, const_cast<void*>(_rValue.getValue()), aPos->aType.getTypeLibType()->pType, cpp_acquire);
			break;
	}
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::getFastPropertyValue(Any& _rValue, sal_Int32 _nHandle) const
{
#ifdef DBG_UTIL
	const_cast<OSimplePropertyContainer*>(this)->m_bAlreadyAccessed = sal_True;
#endif

	PropertiesIterator aPos = const_cast<OSimplePropertyContainer*>(this)->searchHandle(_nHandle);
	if (aPos == m_aProperties.end())
	{
		OSL_ASSERT("OSimplePropertyContainer::getFastPropertyValue : unknown handle !");
		return;
	}

	switch (aPos->eLocated)
	{
		case PropertyDescription::ltHoldMyself:
			OSL_ENSHURE(aPos->aLocation.nOwnClassVectorIndex < m_aHoldProperties.size(),
				"OSimplePropertyContainer::convertFastPropertyValue : invalid position !");
			OSL_ENSHURE(m_aHoldProperties[aPos->aLocation.nOwnClassVectorIndex].hasValue(),
				"OSimplePropertyContainer::convertFastPropertyValue : a non-void prop is empty !");
			_rValue = m_aHoldProperties[aPos->aLocation.nOwnClassVectorIndex];
			break;
		case PropertyDescription::ltDerivedClassAnyType:
			_rValue = *reinterpret_cast<Any*>(aPos->aLocation.pDerivedClassMember);
			break;
		case PropertyDescription::ltDerivedClassRealType:
			_rValue.setValue(aPos->aLocation.pDerivedClassMember, aPos->aType);
			break;
	}
}

//--------------------------------------------------------------------------
OSimplePropertyContainer::PropertiesIterator OSimplePropertyContainer::searchHandle(sal_Int32 _nHandle)
{
	PropertyDescription::PropertyDescriptionCompareByHandle aCompareOp;
	PropertyDescription aSearch;
	aSearch.nHandle = _nHandle;

	// search a lower bound
	PropertiesIterator aLowerBound = ::std::lower_bound(
		m_aProperties.begin(),
		m_aProperties.end(),
		aSearch,
		aCompareOp);

	// check for identity
	if ((aLowerBound != m_aProperties.end()) && aLowerBound->nHandle != _nHandle)
		aLowerBound = m_aProperties.end();

	return aLowerBound;
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::modifyAttributes(sal_Int32 _nHandle, sal_Int32 _nAddAttrib, sal_Int32 _nRemoveAttrib)
{
	OSL_ENSHURE(!m_bAlreadyAccessed, "OSimplePropertyContainer::modifyAttributes : invalid call, the property sequence was already accessed !");

	PropertiesIterator aPos = searchHandle(_nHandle);
	if (aPos == m_aProperties.end())
	{
		OSL_ASSERT("OSimplePropertyContainer::modifyAttributes : invalid handle !");
		return;
	}
	aPos->nHandle |= _nAddAttrib;
	aPos->nHandle &= ~_nRemoveAttrib;
}

//--------------------------------------------------------------------------
void OSimplePropertyContainer::describeProperties(Sequence< Property >& _rProps) const
{
	Sequence< Property > aOwnProps(m_aProperties.size());
	Property* pOwnProps = aOwnProps.getArray();

	for	(	ConstPropertiesIterator aLoop = m_aProperties.begin();
			aLoop != m_aProperties.end();
			++aLoop, ++pOwnProps
		)
	{
		pOwnProps->Name = aLoop->sName;
		pOwnProps->Handle = aLoop->nHandle;
		pOwnProps->Attributes = aLoop->nAttributes;
		pOwnProps->Type = aLoop->aType;
	}

	// as our property vector is sorted by handles, not by name, we have to sort aOwnProps
	qsort(aOwnProps.getArray(), aOwnProps.getLength(), sizeof(Property), PropertyCompare);

	// and our own props into the given sequence
	mergePropertySequencesByName(_rProps, aOwnProps);
}


