/*************************************************************************
 *
 *  $RCSfile: FValue.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:14:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONNECTIVITY_FILE_VALUE_HXX_
#include "file/FValue.hxx"
#endif
#ifndef _CONNECTIVITY_COMMONTOOLS_HXX_
#include "connectivity/CommonTools.hxx"
#endif

using namespace connectivity;
using namespace connectivity::file;

OFileValue::OFileValue(const OFileValue& _rRH) 
	: m_bBound(_rRH.m_bBound)
	, m_bNull(_rRH.m_bNull)
	, m_eType(_rRH.m_eType)
{ 
	if(!m_bNull)
	{
		switch(_rRH.m_eType)
		{
		case ::com::sun::star::sdbc::DataType::CHAR:
		case ::com::sun::star::sdbc::DataType::VARCHAR:
			m_aValue.pString = _rRH.m_aValue.pString;
			rtl_uString_acquire(m_aValue.pString);
			break;
		case ::com::sun::star::sdbc::DataType::DATE:
		case ::com::sun::star::sdbc::DataType::FLOAT:
		case ::com::sun::star::sdbc::DataType::REAL:
		case ::com::sun::star::sdbc::DataType::DOUBLE:
		case ::com::sun::star::sdbc::DataType::NUMERIC:
		case ::com::sun::star::sdbc::DataType::DECIMAL:
			m_aValue.aDouble = _rRH.m_aValue.aDouble;
			break;
		case ::com::sun::star::sdbc::DataType::BIT:
			m_aValue.aBool	= _rRH.m_aValue.aBool;
			break;
		case ::com::sun::star::sdbc::DataType::INTEGER:
			m_aValue.aInt32	= _rRH.m_aValue.aInt32;
			break;
		case ::com::sun::star::sdbc::DataType::LONGVARCHAR:
			m_aValue.aInt32 = _rRH.m_aValue.aInt32;
		default:
			OSL_ASSERT("Invalid type!");
		}
	}
}

OFileValue::OFileValue(const ::com::sun::star::uno::Any& _rVal) 
	: m_bBound(sal_True)
	, m_bNull(!_rVal.hasValue())
{ 
	if(!m_bNull)
	{
		switch(_rVal.getValueTypeClass())
		{
		case ::com::sun::star::uno::TypeClass_STRING:
			{
				::rtl::OUString aVal;
				_rVal >>= aVal;
				m_aValue.pString = aVal.pData;
				rtl_uString_acquire(m_aValue.pString);
				m_eType = ::com::sun::star::sdbc::DataType::VARCHAR;
			}
				break;
			case ::com::sun::star::uno::TypeClass_DOUBLE:
				m_aValue.aDouble = connectivity::getDouble(_rVal);
				m_eType = ::com::sun::star::sdbc::DataType::DOUBLE;
				break;
			case ::com::sun::star::uno::TypeClass_LONG:
				m_aValue.aInt32 = connectivity::getINT32(_rVal);
				m_eType = ::com::sun::star::sdbc::DataType::INTEGER;
				break;
			case ::com::sun::star::uno::TypeClass_BOOLEAN:
				m_aValue.aBool = connectivity::getBOOL(_rVal);
				m_eType = ::com::sun::star::sdbc::DataType::BIT;
				break;
		}
	}
}

OFileValue& OFileValue::operator=(const OFileValue& _rRH)
{
	if(&_rRH == this)
		return *this;

	freeString();

	m_bBound = _rRH.m_bBound;
	m_bNull	= _rRH.m_bNull;
	m_eType	= _rRH.m_eType;
	if(!m_bNull)
	{
		switch(_rRH.m_eType)
		{
		case ::com::sun::star::sdbc::DataType::CHAR:
		case ::com::sun::star::sdbc::DataType::VARCHAR:
			m_aValue.pString = _rRH.m_aValue.pString;
			rtl_uString_acquire(m_aValue.pString);
			break;
		case ::com::sun::star::sdbc::DataType::DATE:
		case ::com::sun::star::sdbc::DataType::FLOAT:
		case ::com::sun::star::sdbc::DataType::REAL:
		case ::com::sun::star::sdbc::DataType::DOUBLE:
		case ::com::sun::star::sdbc::DataType::NUMERIC:
		case ::com::sun::star::sdbc::DataType::DECIMAL:
			m_aValue.aDouble = _rRH.m_aValue.aDouble;
			break;
		case ::com::sun::star::sdbc::DataType::BIT:
			m_aValue.aBool = _rRH.m_aValue.aBool;
			break;
		case ::com::sun::star::sdbc::DataType::INTEGER:
			m_aValue.aInt32	= _rRH.m_aValue.aInt32;
			break;
		case ::com::sun::star::sdbc::DataType::LONGVARCHAR:
			m_aValue.aInt32 = _rRH.m_aValue.aInt32;
		default:
			OSL_ASSERT("Invalid type!");
		}
	}

	return *this;
}

OFileValue& OFileValue::operator=(const ::rtl::OUString& _rRH)
{

	freeString();
	m_bNull = sal_False;
	m_bBound= sal_True; 

	m_aValue.pString = _rRH.pData;
	rtl_uString_acquire(m_aValue.pString);
	m_eType = ::com::sun::star::sdbc::DataType::VARCHAR;

	return *this;
}

OFileValue& OFileValue::operator=(const double& _rRH)
{
	freeString();
	m_bNull = sal_False;
	m_bBound= sal_True; 

	m_aValue.aDouble = _rRH;
	m_eType = ::com::sun::star::sdbc::DataType::DOUBLE;

	return *this;
}

OFileValue& OFileValue::operator=(const sal_Int32& _rRH)
{
	freeString();
	m_bNull = sal_False;
	m_bBound= sal_True; 

	m_aValue.aInt32 = _rRH;
	m_eType = ::com::sun::star::sdbc::DataType::INTEGER;

	return *this;
}

OFileValue& OFileValue::operator=(const sal_Bool _rRH)
{
	freeString();
	m_bNull = sal_False;
	m_bBound= sal_True; 

	m_aValue.aBool = _rRH;
	m_eType = ::com::sun::star::sdbc::DataType::BIT;

	return *this;
}

OFileValue& OFileValue::operator=(const ::com::sun::star::uno::Any& _rRH)
{
	freeString();

	if(!(m_bNull = !_rRH.hasValue()))
	{
		switch(_rRH.getValueTypeClass())
		{
			case ::com::sun::star::uno::TypeClass_STRING:
			{
				::rtl::OUString aVal;
				_rRH >>= aVal;
				m_aValue.pString = aVal.pData;
				rtl_uString_acquire(m_aValue.pString);
				m_eType = ::com::sun::star::sdbc::DataType::VARCHAR;
				break;
			}
			case ::com::sun::star::uno::TypeClass_DOUBLE:
				m_aValue.aDouble = connectivity::getDouble(_rRH);
				m_eType = ::com::sun::star::sdbc::DataType::DOUBLE;
				break;
			case ::com::sun::star::uno::TypeClass_LONG:
				m_aValue.aInt32 = connectivity::getINT32(_rRH);
				m_eType = ::com::sun::star::sdbc::DataType::INTEGER;
				break;
			case ::com::sun::star::uno::TypeClass_BOOLEAN:
				m_aValue.aBool = connectivity::getBOOL(_rRH);
				m_eType = ::com::sun::star::sdbc::DataType::BIT;
				break;
		}
	}
	m_bBound	= sal_True; 

	return *this;
}

OFileValue::operator ::com::sun::star::uno::Any() const	
{	
	::com::sun::star::uno::Any aRet;
	if(!m_bNull)
	{
		switch(m_eType)
		{
			case ::com::sun::star::sdbc::DataType::VARCHAR:
			case ::com::sun::star::sdbc::DataType::CHAR:
			{
				::rtl::OUString aVal(m_aValue.pString);
				aRet <<= aVal;
				break;
			}
			case ::com::sun::star::sdbc::DataType::DOUBLE:
				aRet <<= m_aValue.aDouble;
				break;
			case ::com::sun::star::sdbc::DataType::INTEGER:
				aRet <<= m_aValue.aInt32;
				break;
			case ::com::sun::star::sdbc::DataType::BIT:
				aRet <<= ::com::sun::star::uno::Any(&m_aValue.aBool,getBooleanCppuType());
				break;
		}
	}
	return aRet;
}

OFileValue::operator==(const OFileValue& _rRH) const
{
	if(m_eType != _rRH.m_eType)
		return sal_False;

	sal_Bool bRet = sal_False;
	switch(m_eType)
	{
		case ::com::sun::star::sdbc::DataType::VARCHAR:
		case ::com::sun::star::sdbc::DataType::CHAR:
		{
			::rtl::OUString aVal1(m_aValue.pString);
			::rtl::OUString aVal2(_rRH.m_aValue.pString);
			bRet = aVal1 == aVal2;
			break;
		}
		case ::com::sun::star::sdbc::DataType::DOUBLE:
			bRet = m_aValue.aDouble == _rRH.m_aValue.aDouble;
			break;
		case ::com::sun::star::sdbc::DataType::INTEGER:
			bRet = m_aValue.aInt32 == _rRH.m_aValue.aInt32;
			break;
		case ::com::sun::star::sdbc::DataType::BIT:
			bRet = m_aValue.aBool == _rRH.m_aValue.aBool;
			break;
	}
	return bRet;
}


