/*************************************************************************
 *
 *  $RCSfile: FValue.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:14:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _CONNECTIVITY_FILE_VALUE_HXX_
#define _CONNECTIVITY_FILE_VALUE_HXX_

#ifndef _COM_SUN_STAR_SDBC_DATATYPE_HPP_
#include <com/sun/star/sdbc/DataType.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif
#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif
#ifndef _UTL_STLTYPES_HXX_
#include <unotools/stl_types.hxx>
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif
#ifndef _CONNECTIVITY_COMMONTOOLS_HXX_
#include "connectivity/CommonTools.hxx"
#endif

namespace connectivity
{
	namespace file
	{
		class OFileValue
		{
			union ValueType
			{
				rtl_uString*	pString;
				double			aDouble;
				sal_Int32		aInt32;
				sal_Bool		aBool;
			};

			ValueType	m_aValue;
			sal_Int32	m_eType;	// possible values are the enums from ::com::sun::star::sdbc::DataType
			sal_Bool	m_bBound;
			sal_Bool	m_bNull;

			void freeString()
			{
				switch(m_eType)
				{
					case ::com::sun::star::sdbc::DataType::CHAR:
					case ::com::sun::star::sdbc::DataType::VARCHAR:
						if(!m_bNull)
						{
							OSL_ENSHURE(m_aValue.pString,"String pointer is null!");
							rtl_uString_release(m_aValue.pString);
						}
				}
			}
		public:
			OFileValue() 
			{ 
				m_aValue.pString	= NULL; 
				m_eType				= ::com::sun::star::sdbc::DataType::VARCHAR;
				m_bBound			= sal_True; 
				m_bNull				= sal_True;
			}

			OFileValue(const OFileValue& _rRH);
			
			OFileValue(const ::rtl::OUString& _rRH)
			{
				operator=(_rRH);
			}

			OFileValue(const double& _rRH)
			{
				operator=(_rRH);
			}
			
			OFileValue(const sal_Int32& _rRH)
			{
				operator=(_rRH);
			}

			OFileValue(const sal_Bool& _rRH)
			{
				operator=(_rRH);
			}

			OFileValue(const ::com::sun::star::uno::Any& _rVal);

			~OFileValue()
			{
				freeString();
			}

			OFileValue& operator=(const OFileValue& _rRH);

			OFileValue& operator=(const ::rtl::OUString& _rRH);
			
			OFileValue& operator=(const double& _rRH);

			OFileValue& operator=(const sal_Int32& _rRH);

			OFileValue& operator=(const sal_Bool _rRH);

			OFileValue& operator=(const ::com::sun::star::uno::Any& _rRH);

			operator ::com::sun::star::uno::Any() const;

			operator sal_Bool() const
			{
				return isNull() ? sal_False : getBool();
			}

			operator sal_Int32() const
			{
				return isNull() ? 0 : getInt32();
			}

			operator double() const
			{
				return isNull() ? 0.0 : getDouble();
			}

			operator ::rtl::OUString() const
			{
				return isNull() ? ::rtl::OUString() : getString();
			}

			operator==(const OFileValue& _rRH) const;

			sal_Bool	isNull() const				{ return m_bNull;	}
			void		setNull()	
			{ 
				freeString();
				m_bNull = sal_True;
				m_aValue.pString = NULL; 
			}

			sal_Bool	isBound() const				{ return m_bBound;		}
			void		setBound(sal_Bool _bBound)	{ m_bBound = _bBound;	}

			sal_Int32	getType() const				{ return m_eType;	}
			void		setType(sal_Int32 _eType)	{ m_eType = _eType;	}

			sal_Bool	getBool() const				{ return m_aValue.aBool; }
			sal_Int32	getInt32() const			{ return m_aValue.aInt32; }
			double		getDouble() const			{ return m_aValue.aDouble; }
			::rtl::OUString getString() const		{ return m_aValue.pString; }
		};

		// ----------------------------------------------------------------------------
		// Vector for file based rows
		// ----------------------------------------------------------------------------
		class  OValueVector : public connectivity::ORowVector< OFileValue > 
		{
			sal_Bool	m_bDeleted;
		public:
			OValueVector()				: connectivity::ORowVector< OFileValue >()	,m_bDeleted(sal_False)	{}
			OValueVector(size_t _st)	: connectivity::ORowVector< OFileValue >(_st),m_bDeleted(sal_False)	{}

			sal_Bool	isDeleted() const				{ return m_bDeleted;		}
			void		setDeleted(sal_Bool _bDeleted)	{ m_bDeleted = _bDeleted;	}
		};

#define SQL_NO_PARAMETER (ULONG_MAX)		
		class OAssignValues : public OValueVector
		{
			::std::vector<sal_Int32> m_nParameterIndexes;
		public:
			OAssignValues() : m_nParameterIndexes(1,SQL_NO_PARAMETER){}
			OAssignValues(size_type n) : OValueVector(n),m_nParameterIndexes(n+1,SQL_NO_PARAMETER){}

			void setParameterIndex(sal_Int32 _nId,sal_Int32 _nParameterIndex) { m_nParameterIndexes[_nId] = _nParameterIndex;}
			sal_Int32 getParameterIndex(sal_Int32 _nId) const { return m_nParameterIndexes[_nId]; }
		};

		typedef ::vos::ORef< OAssignValues > ORefAssignValues;

		typedef ::vos::ORef< OValueVector > OValueRow;
	}
}

#endif // #ifndef _CONNECTIVITY_FILE_VALUE_HXX_


