/*************************************************************************
 *
 *  $RCSfile: ubtest.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:14:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include <stl/hash_map>
#include <stl/map>

#include <osl/diagnose.h>
#include <osl/mutex.hxx>
#include <osl/module.h>
#include <osl/process.h>
#include <osl/thread.h>
#include <osl/conditn.hxx>

#include <rtl/string.hxx>
#include <rtl/strbuf.hxx>
#include <rtl/ustrbuf.hxx>

#include <tools/time.hxx>

#include <uno/environment.hxx>
#include <uno/mapping.hxx>

#include <cppuhelper/factory.hxx>
#include <cppuhelper/implbase2.hxx>

#include <com/sun/star/lang/XServiceInfo.hpp>
#include <com/sun/star/lang/XComponent.hpp>
#include <com/sun/star/lang/XMain.hpp>
#include <com/sun/star/lang/XInitialization.hpp>
#include <com/sun/star/loader/XImplementationLoader.hpp>
#include <com/sun/star/registry/XRegistryKey.hpp>
#include <com/sun/star/bridge/XUnoUrlResolver.hpp>
#include <com/sun/star/container/XSet.hpp>
#include <com/sun/star/test/XPerformanceTest.hpp>

#define NLOOP 500000

using namespace rtl;
using namespace osl;
using namespace cppu;
using namespace com::sun::star::uno;
using namespace com::sun::star::lang;
using namespace com::sun::star::loader;
using namespace com::sun::star::registry;
using namespace com::sun::star::bridge;
using namespace com::sun::star::container;
using namespace com::sun::star::test;

#define SERVICENAME		"com.sun.star.test.PerformanceTest"
#define IMPLNAME		"com.sun.star.comp.PerformanceTest"

namespace benchmark_test
{

//--------------------------------------------------------------------------------------------------
static void out( const sal_Char * pText, FILE * stream = stderr,
				 sal_Int32 nStart = -1, sal_Char cFillchar = ' ' )
{
	static sal_Int32 s_nPos = 0;
	
	sal_Char ar[2] = { cFillchar, 0 };
	while (s_nPos < nStart)
	{
		::fprintf( stream, ar );
		++s_nPos;
	}
	
	::fprintf( stream, pText );
	
	for ( const sal_Char * p = pText; *p; ++p )
	{
		if (*p == '\n')
			s_nPos = 0;
		else
			++s_nPos;
	}
}
//--------------------------------------------------------------------------------------------------
static inline void out( const OUString & rText, FILE * stream = stderr,
						sal_Int32 nStart = -1, sal_Char cFillchar = ' ' )
{
	OString aText( OUStringToOString( rText, RTL_TEXTENCODING_ASCII_US ) );
	out( aText.getStr(), stream, nStart, cFillchar );
}
//--------------------------------------------------------------------------------------------------
static inline void out( double fVal, FILE * stream = stderr,
						sal_Int32 nStart = -1, sal_Char cFillchar = ' ' )
{
	sal_Char ar[128];
	::sprintf( ar, "%1.6g", fVal );
	out( ar, stream, nStart, cFillchar );
}

//==================================================================================================
static Reference< XSingleServiceFactory > loadLibComponentFactory(
	const OUString & rLibName, const OUString & rImplName,
	const Reference< XMultiServiceFactory > & xSF, const Reference< XRegistryKey > & xKey )
{
	Reference< XSingleServiceFactory > xRet;
	
	OUStringBuffer aLibNameBuf( 32 );
#ifdef SAL_UNX
	aLibNameBuf.appendAscii( RTL_CONSTASCII_STRINGPARAM("lib") );
	aLibNameBuf.append( rLibName );
	aLibNameBuf.appendAscii( RTL_CONSTASCII_STRINGPARAM(".so") );
#else
	aLibNameBuf.append( rLibName );
	aLibNameBuf.appendAscii( RTL_CONSTASCII_STRINGPARAM(".dll") );
#endif
	OUString aLibName( aLibNameBuf.makeStringAndClear() );
	oslModule lib = osl_loadModule( aLibName.pData, SAL_LOADMODULE_LAZY | SAL_LOADMODULE_GLOBAL );
	
	if (lib)
	{
		void * pSym;
		
		// ========================= LATEST VERSION =========================
		OUString aGetEnvName( RTL_CONSTASCII_USTRINGPARAM(COMPONENT_GETENV) );
		if (pSym = osl_getSymbol( lib, aGetEnvName.pData ))
		{
			uno_Environment * pCurrentEnv = 0;
			uno_Environment * pEnv = 0;
			const sal_Char * pEnvTypeName = 0;
			(*((component_getImplementationEnvironmentFunc)pSym))( &pEnvTypeName, &pEnv );
			
			sal_Bool bNeedsMapping =
				(pEnv || 0 != rtl_str_compare( pEnvTypeName, CPPU_CURRENT_LANGUAGE_BINDING_NAME ));
			
			OUString aEnvTypeName( OUString::createFromAscii( pEnvTypeName ) );
			
			if (bNeedsMapping)
			{
				if (! pEnv)
					uno_getEnvironment( &pEnv, aEnvTypeName.pData, 0 );
				if (pEnv)
				{
					OUString aCppEnvTypeName( RTL_CONSTASCII_USTRINGPARAM(CPPU_CURRENT_LANGUAGE_BINDING_NAME) );
					uno_getEnvironment( &pCurrentEnv, aCppEnvTypeName.pData, 0 );
					if (pCurrentEnv)
						bNeedsMapping = (pEnv != pCurrentEnv);
				}
			}
			
			OUString aGetFactoryName( RTL_CONSTASCII_USTRINGPARAM(COMPONENT_GETFACTORY) );
			if (pSym = osl_getSymbol( lib, aGetFactoryName.pData ))
			{
				OString aImplName( OUStringToOString( rImplName, RTL_TEXTENCODING_ASCII_US ) );
				
				if (bNeedsMapping)
				{
					if (pEnv && pCurrentEnv)
					{
						Mapping aCurrent2Env( pCurrentEnv, pEnv );
						Mapping aEnv2Current( pEnv, pCurrentEnv );
						
						if (aCurrent2Env.is() && aEnv2Current.is())
						{
							void * pSMgr = aCurrent2Env.mapInterface(
								xSF.get(), ::getCppuType( (const Reference< XMultiServiceFactory > *)0 ) );
							void * pKey = aCurrent2Env.mapInterface(
								xKey.get(), ::getCppuType( (const Reference< XRegistryKey > *)0 ) );
							
							void * pSSF = (*((component_getFactoryFunc)pSym))(
								aImplName.getStr(), pSMgr, pKey );
							
							if (pKey)
								(*pEnv->releaseInterface)( pEnv, pKey );
							if (pSMgr)
								(*pEnv->releaseInterface)( pEnv, pSMgr );
							
							if (pSSF)
							{
								aEnv2Current.mapInterface(
									reinterpret_cast< void ** >( &xRet ),
									pSSF, ::getCppuType( (const Reference< XSingleServiceFactory > *)0 ) );
								(*pEnv->releaseInterface)( pEnv, pSSF );
							}
						}
					}
				}
				else
				{
					XSingleServiceFactory * pRet = (XSingleServiceFactory *)
						(*((component_getFactoryFunc)pSym))(
							aImplName.getStr(), xSF.get(), xKey.get() );
					if (pRet)
					{
						xRet = pRet;
						pRet->release();
					}
				}
			}
			
			if (pEnv)
				(*pEnv->release)( pEnv );
			if (pCurrentEnv)
				(*pCurrentEnv->release)( pCurrentEnv );
		}
		
		// ========================= PREVIOUS VERSION =========================
		else
		{
			OUString aGetFactoryName( RTL_CONSTASCII_USTRINGPARAM(CREATE_COMPONENT_FACTORY_FUNCTION) );
			if (pSym = osl_getSymbol( lib, aGetFactoryName.pData ))
			{
				OUString aCppEnvTypeName( RTL_CONSTASCII_USTRINGPARAM(CPPU_CURRENT_LANGUAGE_BINDING_NAME) );
				OUString aUnoEnvTypeName( RTL_CONSTASCII_USTRINGPARAM(UNO_LB_UNO) );
				Mapping aUno2Cpp( aUnoEnvTypeName, aCppEnvTypeName );
				Mapping aCpp2Uno( aCppEnvTypeName, aUnoEnvTypeName );
				OSL_ENSHURE( aUno2Cpp.is() && aCpp2Uno.is(), "### cannot get uno mappings!" );
				
				if (aUno2Cpp.is() && aCpp2Uno.is())
				{
					uno_Interface * pUComponentFactory = 0;
					
					uno_Interface * pUSFactory = (uno_Interface *)aCpp2Uno.mapInterface(
						xSF.get(), ::getCppuType( (const Reference< XMultiServiceFactory > *)0 ) );
					uno_Interface * pUKey = (uno_Interface *)aCpp2Uno.mapInterface(
						xKey.get(), ::getCppuType( (const Reference< XRegistryKey > *)0 ) );
				
					pUComponentFactory = (*((CreateComponentFactoryFunc)pSym))(
						rImplName.getStr(), pUSFactory, pUKey );
				
					if (pUKey)
						(*pUKey->release)( pUKey );
					if (pUSFactory)
						(*pUSFactory->release)( pUSFactory );
				
					if (pUComponentFactory)
					{
						XSingleServiceFactory * pXFactory =
							(XSingleServiceFactory *)aUno2Cpp.mapInterface(
								pUComponentFactory, ::getCppuType( (const Reference< XSingleServiceFactory > *)0 ) );
						(*pUComponentFactory->release)( pUComponentFactory );
					
						if (pXFactory)
						{
							xRet = pXFactory;
							pXFactory->release();
						}
					}
				}
			}
		}
		
		if (! xRet.is())
			osl_unloadModule( lib );
	}
	
	return xRet;
}
//--------------------------------------------------------------------------------------------------
template< class T >
static void createInstance( Reference< T > & rxOut,
							const Reference< XMultiServiceFactory > & xMgr,
							const OUString & rServiceName )
	throw (RuntimeException)
{
	Reference< XInterface > x( xMgr->createInstance( rServiceName ), UNO_QUERY );
	
	if (! x.is())
	{
		static sal_Bool s_bSet = sal_False;
		if (! s_bSet)
		{
			MutexGuard aGuard( Mutex::getGlobalMutex() );
			if (! s_bSet)
			{
				Reference< XSet > xSet( xMgr, UNO_QUERY );
				if (xSet.is())
				{
					// acceptor
					xSet->insert( makeAny( loadLibComponentFactory(
						OUString( RTL_CONSTASCII_USTRINGPARAM("acceptor") ),
						OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.stoc.Acceptor") ),
						xMgr, Reference< XRegistryKey >() ) ) );
					// connector
					xSet->insert( makeAny( loadLibComponentFactory(
						OUString( RTL_CONSTASCII_USTRINGPARAM("connectr") ),
						OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.stoc.Connector") ),
						xMgr, Reference< XRegistryKey >() ) ) );
					// iiop bridge
					xSet->insert( makeAny( loadLibComponentFactory(
						OUString( RTL_CONSTASCII_USTRINGPARAM("iiopbrdg") ),
						OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.stoc.bridge.iiop") ),
						xMgr, Reference< XRegistryKey >() ) ) );
					// bridge factory
					xSet->insert( makeAny( loadLibComponentFactory(
						OUString( RTL_CONSTASCII_USTRINGPARAM("brdgfctr") ),
						OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.stoc.BridgeFactory") ),
						xMgr, Reference< XRegistryKey >() ) ) );
					// uno url resolver
					xSet->insert( makeAny( loadLibComponentFactory(
						OUString( RTL_CONSTASCII_USTRINGPARAM("uuresolver") ),
						OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.bridge.UnoUrlResolver") ),
						xMgr, Reference< XRegistryKey >() ) ) );
					// java loader
					xSet->insert( makeAny( loadLibComponentFactory(
						OUString( RTL_CONSTASCII_USTRINGPARAM("javaloader") ),
						OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.stoc.JavaComponentLoader") ),
						xMgr, Reference< XRegistryKey >() ) ) );
				}
				s_bSet = sal_True;
			}
		}
		x = xMgr->createInstance( rServiceName );
	}
	
	if (! x.is())
	{
		OUStringBuffer buf( 64 );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("cannot get service instance \"") );
		buf.append( rServiceName );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("\"!") );
		throw RuntimeException( buf.makeStringAndClear(), Reference< XInterface >() );
	}
	
	rxOut = Reference< T >::query( x );
	if (! rxOut.is())
	{
		OUStringBuffer buf( 64 );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("service instance \"") );
		buf.append( rServiceName );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("\" does not support demanded interface \"") );
		const Type & rType = ::getCppuType( (const Reference< T > *)0 );
		buf.append( rType.getTypeName() );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("\"!") );
		throw RuntimeException( buf.makeStringAndClear(), Reference< XInterface >() );
	}
}

//--------------------------------------------------------------------------------------------------
inline static Sequence< OUString > getSupportedServiceNames()
{
	OUString aName( RTL_CONSTASCII_USTRINGPARAM(SERVICENAME) );
	return Sequence< OUString >( &aName, 1 );
}

//==================================================================================================
class TestImpl : public WeakImplHelper2< XServiceInfo, XMain >
{
	Reference< XMultiServiceFactory > _xSMgr;
	
	Mutex							  _aMutex;
	Reference< XInterface >			  _xDirect;
	Reference< XInterface > getDirect() throw (Exception);
	Reference< XInterface > resolveObject( const OUString & rUnoUrl ) throw (Exception);
	
public:
	TestImpl( const Reference< XMultiServiceFactory > & xSMgr );
	virtual ~TestImpl();
	
	// XServiceInfo
	virtual OUString SAL_CALL getImplementationName() throw (RuntimeException);
	virtual sal_Bool SAL_CALL supportsService( const OUString & rServiceName ) throw (RuntimeException);
	virtual Sequence< OUString > SAL_CALL getSupportedServiceNames() throw (RuntimeException);
	
	// XMain
    virtual sal_Int32 SAL_CALL run( const Sequence< OUString > & rArgs ) throw (RuntimeException);
};

//##################################################################################################

//__________________________________________________________________________________________________
TestImpl::TestImpl( const Reference< XMultiServiceFactory > & xSMgr )
	: _xSMgr( xSMgr )
{
}
//__________________________________________________________________________________________________
TestImpl::~TestImpl()
{
}

//==================================================================================================
static Reference< XInterface > TestImpl_create( const Reference< XMultiServiceFactory > & xSMgr )
{
	return Reference< XInterface >( *new TestImpl( xSMgr ) );
}

// XServiceInfo
//__________________________________________________________________________________________________
OUString TestImpl::getImplementationName()
	throw (RuntimeException)
{
	return OUString( RTL_CONSTASCII_USTRINGPARAM(IMPLNAME) );
}
//__________________________________________________________________________________________________
sal_Bool TestImpl::supportsService( const OUString & rServiceName )
	throw (RuntimeException)
{
	const Sequence< OUString > & rSNL = getSupportedServiceNames();
	const OUString * pArray = rSNL.getConstArray();
	for ( sal_Int32 nPos = rSNL.getLength(); nPos--; )
	{
		if (pArray[nPos] == rServiceName)
			return sal_True;
	}
	return sal_False;
}
//__________________________________________________________________________________________________
Sequence< OUString > TestImpl::getSupportedServiceNames()
	throw (RuntimeException)
{
	return benchmark_test::getSupportedServiceNames();
}

//__________________________________________________________________________________________________
Reference< XInterface > TestImpl::getDirect()
	throw (Exception)
{
	if (! _xDirect.is())
	{
		MutexGuard aGuard( _aMutex );
		if (! _xDirect.is())
		{
			Reference< XSingleServiceFactory > xFac( loadLibComponentFactory(
				OUString( RTL_CONSTASCII_USTRINGPARAM("ubobject") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.benchmark.TestObject") ),
				_xSMgr, Reference< XRegistryKey >() ) );
			if (! xFac.is())
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("no test object available!") ), Reference< XInterface >() );
			_xDirect = xFac->createInstance();
		}
	}
	return _xDirect;
}
//--------------------------------------------------------------------------------------------------
Reference< XInterface > TestImpl::resolveObject( const OUString & rUnoUrl )
	throw (Exception)
{
	Reference< XUnoUrlResolver > xResolver;
	createInstance(
		xResolver, _xSMgr,
		OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.bridge.UnoUrlResolver") ) );
	
	Reference< XInterface > xResolvedObject( xResolver->resolve( rUnoUrl ) );
	
	if (! xResolvedObject.is())
	{
		OUStringBuffer buf( 32 );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("cannot resolve object \"") );
		buf.append( rUnoUrl );
		buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("\"!") );
		throw RuntimeException( buf.makeStringAndClear(), Reference< XInterface >() );
	}
	
	return xResolvedObject;
}

//==================================================================================================
class TimeEntry
{
	sal_Int32			nLoop;
	sal_Int32			nTicks;
	
public:
	TimeEntry()
		{}
	TimeEntry( sal_Int32 nLoop_, sal_Int32 nTicks_ )
		: nLoop( nLoop_ )
		, nTicks( nTicks_ )
		{}
	
	inline double secPerCall() const
		{ return (((double)nTicks) / nLoop); }
	
	double ratio( const TimeEntry & rEntry ) const;
};
//__________________________________________________________________________________________________
double TimeEntry::ratio( const TimeEntry & rEntry ) const
{
	double f = rEntry.nTicks * nLoop;
	if (f == 0.0)
	{
		return 0.0;
	}
	else
	{
		return (((double)(nTicks * rEntry.nLoop)) / f);
	}
}

//==================================================================================================
typedef std::map< std::string, TimeEntry > t_TimeEntryMap;

//==================================================================================================
struct TimingSheet
{
	t_TimeEntryMap		_entries;	
	void insert( const sal_Char * pText, sal_Int32 nLoop, sal_Int32 nTicks );
};
//__________________________________________________________________________________________________
void TimingSheet::insert( const sal_Char * pText, sal_Int32 nLoop, sal_Int32 nTicks )
{
	_entries[ pText ] = TimeEntry( nLoop, nTicks );
}

//==================================================================================================
typedef std::hash_map< std::string, TimingSheet > t_TimingSheetMap;

//--------------------------------------------------------------------------------------------------
inline sal_Int32 getSystemTicks()
{
	return Time::GetSystemTicks();
}

//--------------------------------------------------------------------------------------------------
static void benchmark(
	TimingSheet & rSheet, const Reference< XInterface > & xInstance, sal_Int32 nLoop )
	throw (Exception)
{
	Reference< XPerformanceTest > xBench( xInstance, UNO_QUERY );
	if (! xBench.is())
		throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("illegal test object!") ), Reference< XInterface >() );
	
	sal_Int32 i;
	sal_Int32 tStart, tEnd;
	
	const Type & rKnownType = ::getCppuType( (const Reference< XPerformanceTest > *)0 );
	const Type & rUnKnownType = ::getCppuType( (const Reference< XSet > *)0 );
	
	ComplexTypes aDummyStruct;
	
	//------------------------------------
	// oneway calls
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->async();
	xBench->sync();
	tEnd = getSystemTicks();
	rSheet.insert( "1a: simple oneway calls (no params, no return)", nLoop+1, tEnd - tStart );
	// synchron calls
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->sync();
	xBench->sync();
	tEnd = getSystemTicks();
	rSheet.insert( "1b: simple synchron calls (no params no return)", nLoop+1, tEnd - tStart );
	
	// acquire
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->acquire();
	tEnd = getSystemTicks();
	rSheet.insert( "2a: interface acquire() calls", nLoop, tEnd - tStart );
	// release
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->release();
	tEnd = getSystemTicks();
	rSheet.insert( "2b: interface release() calls", nLoop, tEnd - tStart );
	
	// queryInterface() for known type
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->queryInterface( rKnownType );
	tEnd = getSystemTicks();
	rSheet.insert( "2c: interface query for implemented type", nLoop, tEnd - tStart );
	// queryInterface() for unknown type
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->queryInterface( rUnKnownType );
	tEnd = getSystemTicks();
	rSheet.insert( "2d: interface query for unknown type", nLoop, tEnd - tStart );
	
	// create and forget objects
	Reference< XPerformanceTest > xBench2( xBench );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench2 = xBench2->createTestObject();
	tEnd = getSystemTicks();
	rSheet.insert( "3a: create and release test objects", nLoop, tEnd - tStart );
	
	// hold new objects
	Sequence< Reference< XInterface > > aSeq( nLoop );
	Reference< XInterface > * pSeq = aSeq.getArray();
	xBench2 = xBench;
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		pSeq[i] = xBench2 = xBench2->createTestObject();
	tEnd = getSystemTicks();
	rSheet.insert( "3b: create and hold test objects", nLoop, tEnd - tStart );
	
	// structs
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->complex_in( aDummyStruct );
	tEnd = getSystemTicks();
	rSheet.insert( "4a: complex_in() calls (in struct; return struct)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->complex_inout( aDummyStruct );
	tEnd = getSystemTicks();
	rSheet.insert( "4b: complex_inout() calls (inout struct; return struct)", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->complex_oneway( aDummyStruct );
	tEnd = getSystemTicks();
	rSheet.insert( "4c: complex_oneway() oneway calls (in struct)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->complex_noreturn( aDummyStruct );
	tEnd = getSystemTicks();
	rSheet.insert( "4d: complex_noreturn() calls (in struct)", nLoop, tEnd - tStart );
	
	// attributes, get() methods
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getLong();
	tEnd = getSystemTicks();
	rSheet.insert( "5a: getLong() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeLong();
	tEnd = getSystemTicks();
	rSheet.insert( "5b: get long attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setLong( 0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5c: setLong() call", nLoop, tEnd - tStart );
	i = nLoop;	
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeLong( 0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5d: set long attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getHyper();
	tEnd = getSystemTicks();
	rSheet.insert( "5e: getHyper() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeHyper();
	tEnd = getSystemTicks();
	rSheet.insert( "5f: get hyper attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setHyper( 0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5g: setHyper() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeHyper( 0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5h: set hyper attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getFloat();
	tEnd = getSystemTicks();
	rSheet.insert( "5i: getFloat() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeFloat();
	tEnd = getSystemTicks();
	rSheet.insert( "5j: get float attribute",nLoop,  tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setFloat( 0.0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5k: setFloat() call", nLoop, tEnd - tStart );	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeFloat( 0.0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5l: set float attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getDouble();
	tEnd = getSystemTicks();
	rSheet.insert( "5m: getDouble() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeDouble();
	tEnd = getSystemTicks();
	rSheet.insert( "5n: get double attribute", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setDouble( 0.0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5o: setDouble() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeDouble( 0.0 );
	tEnd = getSystemTicks();
	rSheet.insert( "5p: set double attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getString();
	tEnd = getSystemTicks();
	rSheet.insert( "6a: getString() call (empty)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeString();
	tEnd = getSystemTicks();
	rSheet.insert( "6b: get empty string attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	OUString aDummyString;
	tStart = getSystemTicks();
	while (i--)
		xBench->setString( aDummyString );
	tEnd = getSystemTicks();
	rSheet.insert( "6c: setString() call (emtpy)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeString( aDummyString );
	tEnd = getSystemTicks();
	rSheet.insert( "6d: set empty string attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getInterface();
	tEnd = getSystemTicks();
	rSheet.insert( "7a: getInterface() call (null)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeInterface();
	tEnd = getSystemTicks();
	rSheet.insert( "7b: get interface attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	Reference< XInterface > aDummyInterface;
	tStart = getSystemTicks();
	while (i--)
		xBench->setInterface( aDummyInterface );
	tEnd = getSystemTicks();
	rSheet.insert( "7c: setInterface() call (null)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeInterface( Reference< XInterface >() );
	tEnd = getSystemTicks();
	rSheet.insert( "7d: set interface attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAny();
	tEnd = getSystemTicks();
	rSheet.insert( "8a: getAny() call (empty)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeAny();
	tEnd = getSystemTicks();
	rSheet.insert( "8b: get empty any attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	Any aDummyAny;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAny( aDummyAny );
	tEnd = getSystemTicks();
	rSheet.insert( "8c: setAny() call (empty)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeAny( aDummyAny );
	tEnd = getSystemTicks();
	rSheet.insert( "8d: set empty any attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getSequence();
	tEnd = getSystemTicks();
	rSheet.insert( "9a: getSequence() call (empty)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeSequence();
	tEnd = getSystemTicks();
	rSheet.insert( "9b: get empty sequence attribute", nLoop, tEnd - tStart );
	i = nLoop;
	Sequence< Reference< XInterface > > aDummySeq;
	tStart = getSystemTicks();
	while (i--)
		xBench->setSequence( aDummySeq );
	tEnd = getSystemTicks();
	rSheet.insert( "9c: setSequence() call (empty)", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeSequence( aDummySeq );
	tEnd = getSystemTicks();
	rSheet.insert( "9d: set empty sequence attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getStruct();
	tEnd = getSystemTicks();
	rSheet.insert( "Aa: getStruct() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->getAttributeStruct();
	tEnd = getSystemTicks();
	rSheet.insert( "Ab: get struct attribute", nLoop, tEnd - tStart );
	
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setStruct( aDummyStruct );
	tEnd = getSystemTicks();
	rSheet.insert( "Ac: setStruct() call", nLoop, tEnd - tStart );
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
		xBench->setAttributeStruct( aDummyStruct );
	tEnd = getSystemTicks();
	rSheet.insert( "Ad: set struct attribute", nLoop, tEnd - tStart );
	
	// load
//  	i = nLoop;
//  	tStart = getSystemTicks();
//  	while (i--)
//  		xBench->setSequence( aSeq );
//  	tEnd = getSystemTicks();
//  	rSheet.insert( "transfer of exisiting objects", nLoop, tEnd - tStart );
	
	// exceptions
	i = nLoop;
	tStart = getSystemTicks();
	while (i--)
	{
		try
		{
			xBench->raiseRuntimeException();
		}
		catch (RuntimeException &)
		{
		}
	}
	tEnd = getSystemTicks();
	rSheet.insert( "Ba: raising RuntimeException", nLoop, tEnd - tStart );
	
	//------------------------------------
}

//XMain
//__________________________________________________________________________________________________
sal_Int32 TestImpl::run( const Sequence< OUString > & rArgs )
	throw (RuntimeException)
{
	try
	{
		sal_Int32 nLoop = NLOOP;
		
		FILE * stream = stderr;
		if (rArgs.getLength() > 1)
		{
			OString aFileName( OUStringToOString( rArgs[1], RTL_TEXTENCODING_ASCII_US ) );
			stream = ::fopen( aFileName.getStr(), "w" );
			if (! stream)
			{
				OUStringBuffer buf( 32 );
				buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("cannot open file for writing: \"") );
				buf.append( rArgs[1] );
				buf.appendAscii( RTL_CONSTASCII_STRINGPARAM("\"!") );
				throw RuntimeException( buf.makeStringAndClear(), Reference< XInterface >() );
			}
		}
		if (rArgs.getLength() > 2)
		{
			sal_Int64 n = rArgs[2].toInt64();
			if (n > 0)
				nLoop = n;
		}
		
		OUString aArg( RTL_CONSTASCII_USTRINGPARAM("dmsj") ); // default
		if (rArgs.getLength() > 0)
		{
			aArg = rArgs[0];
		}
		else
		{
			out( "\n> no options given, defaulting to \"dmsj\"; nLoop = " );
			out( nLoop );
			out( "; dumping to stderr..." );
		}
		
		t_TimingSheetMap aSheets;
		TimingSheet aDirect;
		
		//------------------------------------------------------------------------------------------
		
		if (aArg.indexOf( 'd' ) >= 0)
		{
			// in process direct test
			benchmark( aDirect, getDirect(), nLoop );
		}
		
		//------------------------------------------------------------------------------------------
		
		if (aArg.indexOf( 'm' ) >= 0)
		{
			// in process uno dispatch
			Environment aCppEnv, aAnoCppEnv;
			OUString aCurrentLanguageBindingName( RTL_CONSTASCII_USTRINGPARAM(CPPU_CURRENT_LANGUAGE_BINDING_NAME) );
			uno_getEnvironment( reinterpret_cast< uno_Environment ** >( &aCppEnv ),
								aCurrentLanguageBindingName.pData, 0 );
			// anonymous
			uno_createEnvironment( reinterpret_cast< uno_Environment ** >( &aAnoCppEnv ),
								   aCurrentLanguageBindingName.pData, 0 );
			
			// pseuod mapping uno<->uno: does nothing!
			Mapping aMapping( aCppEnv.get(), aAnoCppEnv.get(), OUString( RTL_CONSTASCII_USTRINGPARAM("pseudo") ) );
			if (! aMapping.is())
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("no pseudo mapping available!") ), Reference< XInterface >() );
			
			Reference< XInterface > xMapped;
			Reference< XInterface > xDirect( getDirect() );
			aMapping.mapInterface( reinterpret_cast< void ** >( &xMapped ), xDirect.get() );
			if (! xMapped.is())
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("mapping object failed!") ), Reference< XInterface >() );
			
			benchmark( aSheets[ "mapped in process" ], xMapped, nLoop / 10 );
		}
		
		//------------------------------------------------------------------------------------------
		
		if (aArg.indexOf( 's' ) >= 0)
		{
			// start server process
			oslSecurity hSecurity = osl_getCurrentSecurity();
			if (! hSecurity)
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("cannot get current security handle!") ), Reference< XInterface >() );
			
			OUString aArgs[] = {
				OUString( RTL_CONSTASCII_USTRINGPARAM("-c") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.benchmark.TestObject") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("-l") ),
#ifdef SAL_UNX
				OUString( RTL_CONSTASCII_USTRINGPARAM("libubobject.so") ),
#else
				OUString( RTL_CONSTASCII_USTRINGPARAM("ubobject.dll") ),
#endif
				OUString( RTL_CONSTASCII_USTRINGPARAM("-r") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("benchmark.rdb") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("-r") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("ubobject.rdb") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("-u") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("uno:socket,host=localhost,port=6000;iiop;TestRemoteObject") ),
				OUString( RTL_CONSTASCII_USTRINGPARAM("--singleaccept") )
			};
			rtl_uString * pArgs[] = {
				aArgs[0].pData,
				aArgs[1].pData,
				aArgs[2].pData,
				aArgs[3].pData,
				aArgs[4].pData,
				aArgs[5].pData,
				aArgs[6].pData,
				aArgs[7].pData,
				aArgs[8].pData,
				aArgs[9].pData,
				aArgs[10].pData
			};
			
			out( "\n> executing: \"uno" );
			for ( sal_Int32 nPos = 0; nPos < (sizeof(aArgs) / sizeof(OUString)); ++nPos )
			{
				out( " " );
				out( aArgs[nPos] );
			}
			out( "\" ..." );
			
			oslProcess hProcess = 0;
			OUString aUnoExe( RTL_CONSTASCII_USTRINGPARAM("uno") );
			OUString aWorkingDir( RTL_CONSTASCII_USTRINGPARAM(".") );
			osl_executeProcess(
				aUnoExe.pData, pArgs, sizeof(aArgs) / sizeof(OUString),
				osl_Process_SEARCHPATH | osl_Process_DETACHED | osl_Process_NORMAL,
				hSecurity, aWorkingDir.pData, 0, 0, 0, &hProcess );
			
			osl_freeSecurityHandle( hSecurity );
			if (! hProcess)
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("cannot start server process!") ), Reference< XInterface >() );
			osl_freeProcessHandle( hProcess );
			
			// wait three seconds
			TimeValue threeSeconds;
			threeSeconds.Seconds = 3;
			osl_waitThread( &threeSeconds );
			
			// connect and resolve outer process object
			Reference< XInterface > xResolvedObject( resolveObject( OUString(
				RTL_CONSTASCII_USTRINGPARAM( "uno:socket,host=localhost,port=6000;iiop;TestRemoteObject") ) ) );
			
			benchmark( aSheets[ "remote same host" ], xResolvedObject, nLoop / 100 );
		}
		
		//------------------------------------------------------------------------------------------
		
		if (aArg.indexOf( 'r' ) >= 0)
		{
			// remote
			if (rArgs.getLength() < 2)
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("test r(emote) needs additional uno url!") ), Reference< XInterface >() );
			OUString aUnoUrl( rArgs[1] );
			
			// connect and resolve outer process object
			Reference< XInterface > xResolvedObject( resolveObject( aUnoUrl ) );
			
			benchmark( aSheets[ "remote other host" ], xResolvedObject, nLoop / 100 );
		}
		
		//------------------------------------------------------------------------------------------
		
		if (aArg.indexOf( 'j' ) >= 0)
		{
			// java
			Reference< XImplementationLoader > xLoader;
			createInstance(
				xLoader, _xSMgr,
				OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.loader.Java2") ) );
			
			Reference< XSingleServiceFactory > xFac( xLoader->activate(
				OUString( RTL_CONSTASCII_USTRINGPARAM("com.sun.star.comp.benchmark.JavaTestObject") ),
				OUString(),
				OUString( RTL_CONSTASCII_USTRINGPARAM("ubobject.jar") ),
				Reference< XRegistryKey >() ), UNO_QUERY );
			if (! xFac.is())
				throw RuntimeException( OUString( RTL_CONSTASCII_USTRINGPARAM("no java test object available!") ), Reference< XInterface >() );
			
			benchmark( aSheets[ "java in process" ], xFac->createInstance(), nLoop / 100 );
		}
		
		//------------------------------------------------------------------------------------------
		// dump out tables
		
		out( "\nTimes( ratio to direct in process )", stream );
#ifdef DEBUG
		out ("; compiled with DEBUG", stream );
#endif
		out( ":", stream );
		
		sal_Int32 nPos = 60;
		out( "[direct in process]", stream, nPos );
		t_TimingSheetMap::const_iterator iSheets( aSheets.begin() );
		for ( ; iSheets != aSheets.end(); ++iSheets )
		{
			nPos += 20;
			out( "[", stream, nPos );
			out( (*iSheets).first.c_str(), stream );
			out( "]", stream );
		}
		for ( t_TimeEntryMap::const_iterator iTopics( aDirect._entries.begin() );
			  iTopics != aDirect._entries.end(); ++iTopics )
		{
			const std::string & rTopic = (*iTopics).first;
			
			out( "\n", stream );
			out( rTopic.c_str(), stream );
			
			out( ":", stream, 58, '.' );

			sal_Int32 nPos = 60;
			
			double secs = (*iTopics).second.secPerCall();
			if (secs > 0.0)
			{
				out( secs, stream, nPos );
				out( "s", stream );
			}
			else
			{
				out( "NA", stream, nPos );
			}
			
			iSheets = aSheets.begin();
			for ( ; iSheets != aSheets.end(); ++iSheets )
			{
				const t_TimeEntryMap::const_iterator iFind( (*iSheets).second._entries.find( rTopic ) );
				OSL_ENSHURE( iFind != (*iSheets).second._entries.end(), "####" );
				
				nPos += 20;
				
				double secs = (*iFind).second.secPerCall();
				if (secs > 0.0)
				{
					out( secs, stream, nPos );
					out( "s", stream );
					
					out( " (", stream );
					double ratio = (*iFind).second.ratio( (*iTopics).second );
					if (ratio > 0.0)
					{
						out( ratio, stream );
						out( " x)", stream );
					}
					else
					{
						out( "NA)", stream );
					}
				}
				else
				{
					out( "NA", stream, nPos );
				}
			}
		}
		::fclose( stream );
	}
	catch (Exception & rExc)
	{
		throw RuntimeException( rExc.Message, rExc.Context );
	}
	
	out( "\n> done.\n" );
	return 0;
}

}


//##################################################################################################
//##################################################################################################
//##################################################################################################


extern "C"
{
//==================================================================================================
void SAL_CALL component_getImplementationEnvironment(
	const sal_Char ** ppEnvTypeName, uno_Environment ** ppEnv )
{
	*ppEnvTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
}
//==================================================================================================
sal_Bool SAL_CALL component_writeInfo(
	void * pServiceManager, void * pRegistryKey )
{
	if (pRegistryKey)
	{
		try
		{
			Reference< XRegistryKey > xNewKey(
				reinterpret_cast< XRegistryKey * >( pRegistryKey )->createKey(
					OUString( RTL_CONSTASCII_USTRINGPARAM("/" IMPLNAME "/UNO/SERVICES") ) ) );
			xNewKey->createKey( OUString( RTL_CONSTASCII_USTRINGPARAM(SERVICENAME) ) );
			
			return sal_True;
		}
		catch (InvalidRegistryException &)
		{
			OSL_ENSHURE( sal_False, "### InvalidRegistryException!" );
		}
	}
	return sal_False;
}
//==================================================================================================
void * SAL_CALL component_getFactory(
	const sal_Char * pImplName, void * pServiceManager, void * pRegistryKey )
{
	void * pRet = 0;
	
	if (pServiceManager && rtl_str_compare( pImplName, IMPLNAME ) == 0)
	{
		Reference< XSingleServiceFactory > xFactory( createSingleFactory(
			reinterpret_cast< XMultiServiceFactory * >( pServiceManager ),
			OUString( RTL_CONSTASCII_USTRINGPARAM(IMPLNAME) ),
			benchmark_test::TestImpl_create,
			benchmark_test::getSupportedServiceNames() ) );
		
		if (xFactory.is())
		{
			xFactory->acquire();
			pRet = xFactory.get();
		}
	}
	
	return pRet;
}
}


