/*************************************************************************
 *
 *  $RCSfile: querydefinition.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:15:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _DBA_COREAPI_QUERYDEFINITION_HXX_
#include "querydefinition.hxx"
#endif
#ifndef _DBASHARED_STRINGCONSTANTS_HRC_
#include "stringconstants.hrc"
#endif

#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

#ifndef _COM_SUN_STAR_SDBC_XCONNECTION_HPP_
#include <com/sun/star/sdbc/XConnection.hpp>
#endif

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::sdb;
using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::sdbcx;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::util;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::container;
using namespace ::com::sun::star::registry;
using namespace ::utl;
using namespace ::cppu;

//==========================================================================
//= OQueryDefinition - an object implementing the sdb.QueryDefinition service
//==========================================================================
DBG_NAME(OQueryDefinition);
#ifdef DBG_UTIL
//--------------------------------------------------------------------------
OQueryDefinition::OQueryDefinition()
	:OConfigurationFlushable(m_aMutex)
	,m_bColumnsOutOfDate(sal_True)
{
	DBG_CTOR(OQueryDefinition, NULL);
}
#endif

//--------------------------------------------------------------------------
OQueryDefinition::OQueryDefinition(const ::rtl::OUString& _rName, const Reference< XRegistryKey >& _rxConfigLocation)
	:OConfigurationFlushable(m_aMutex)
	,m_bColumnsOutOfDate(sal_True)
	,m_xConfigLocation(_rxConfigLocation)
{
	DBG_CTOR(OQueryDefinition, NULL);

	m_sName = _rName;

	// the "Command" and the "UseEscapeProcessing" properties are readonly here (which did not
	// account for the base class), and additionally we remove the "BOUND" attribute
	modifyAttributes(PROPERTY_ID_NAME, PropertyAttribute::READONLY, PropertyAttribute::BOUND);
	modifyAttributes(PROPERTY_ID_COMMAND, PropertyAttribute::READONLY, PropertyAttribute::BOUND);
	modifyAttributes(PROPERTY_ID_USE_ESCAPE_PROCESSING, PropertyAttribute::READONLY, PropertyAttribute::BOUND);

	initializeFrom(_rxConfigLocation);
}

//--------------------------------------------------------------------------
OQueryDefinition::~OQueryDefinition()
{
	DBG_CTOR(OQueryDefinition, NULL);
}

// XInterface
//--------------------------------------------------------------------------
Any SAL_CALL OQueryDefinition::queryInterface( const Type& _rType ) throw(RuntimeException)
{
	Any aReturn = ::cppu::queryInterface(_rType,
		static_cast< XAlterQuery* >(this),
		static_cast< XDataDescriptorFactory* >(this),
		static_cast< XFlushable* >(this)
		);

	if (!aReturn.hasValue())
		aReturn = OQueryDescriptor::queryInterface(_rType);

	return aReturn;
}

// XAlterCommand
//--------------------------------------------------------------------------
void SAL_CALL OQueryDefinition::alterCommand( const ::rtl::OUString& _rCommand, sal_Bool _bUse ) throw(SQLException, RuntimeException)
{
	setFastPropertyValue_NoBroadcast(PROPERTY_ID_COMMAND, makeAny(_rCommand));
	setFastPropertyValue_NoBroadcast(PROPERTY_ID_USE_ESCAPE_PROCESSING, makeAny(_bUse));
	m_bColumnsOutOfDate = sal_True;
}

//--------------------------------------------------------------------------
Reference< XNameAccess > SAL_CALL OQueryDefinition::getColumns(  ) throw(RuntimeException)
{
	if (m_bColumnsOutOfDate)
	{
		m_aColumns.clearColumns();

		// ask the parent for a statement and a result set for our current statement
//		Reference< XConnection > xConn(HOW_TO_GET_THE_CONN, UNO_QUERY);

		m_bColumnsOutOfDate = sal_False;
	}
	return &m_aColumns;
}

// XServiceInfo
//--------------------------------------------------------------------------
::rtl::OUString SAL_CALL OQueryDefinition::getImplementationName(  ) throw(RuntimeException)
{
	return ::rtl::OUString::createFromAscii("com.sun.star.sdb.OQueryDefinition");
}

//--------------------------------------------------------------------------
Sequence< ::rtl::OUString > SAL_CALL OQueryDefinition::getSupportedServiceNames(  ) throw(RuntimeException)
{
	// don't call the base class here : it would say that it supports the sdb.QueryDescriptor service, but we don't,
	// so we would have to remove this entry from the sequence
	Sequence< ::rtl::OUString > aSupported(2);
	aSupported.getArray()[0] = SERVICE_SDB_DEFINITIONSETTINGS;
	aSupported.getArray()[1] = SERVICE_SDB_QUERY;
	return aSupported;
}

// XDataDescriptorFactory
//--------------------------------------------------------------------------
Reference< XPropertySet > SAL_CALL OQueryDefinition::createDataDescriptor(  ) throw(RuntimeException)
{
	return new OQueryDescriptor(*this);
}

//--------------------------------------------------------------------------
void OQueryDefinition::flush_NoBroadcast()
{
//	storeTo();
	// TODO
}

#ifdef DBG_UTIL
//--------------------------------------------------------------------------
OMultiInstanceAutoRegistration< OQueryDefinition >	OQueryDefinition::m_aAutoRegistration;
//--------------------------------------------------------------------------
Sequence< ::rtl::OUString > OQueryDefinition::getSupportedServiceNames_Static(void) throw( RuntimeException )
{
	Sequence< ::rtl::OUString > aReturn(1);
	aReturn.getArray()[0] = ::rtl::OUString::createFromAscii("com.sun.star.sdb.dbg_temporary.QueryDefinition");
	return aReturn;
}

//--------------------------------------------------------------------------
::rtl::OUString OQueryDefinition::getImplementationName_Static(void) throw( RuntimeException )
{
	return ::rtl::OUString::createFromAscii("com.sun.star.sdb.dbg_temporary.OQueryDefinition");
}

//--------------------------------------------------------------------------
Reference< XInterface > OQueryDefinition::Create(const Reference< XMultiServiceFactory >&)
{
	OQueryDefinition* pNew = new OQueryDefinition;
	pNew->m_sName = ::rtl::OUString::createFromAscii("TemporaryTestObject");
	return *pNew;
}
#endif


