/*************************************************************************
 *
 *  $RCSfile: registryhelper.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:15:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _DBA_CORE_REGISTRYHELPER_HXX_
#include "registryhelper.hxx"
#endif

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::registry;

//==========================================================================
//= ORegistryLevelEnumeration - enumeration access for all sub keys of a
//=								a specified registry key
//==========================================================================
//--------------------------------------------------------------------------
ORegistryLevelEnumeration::ORegistryLevelEnumeration(const Reference< XRegistryKey > & _rxLevel)
	:m_nCurPos(0)
{
	if (_rxLevel.is())
	{
		try
		{
			m_aElements = _rxLevel->openKeys();
			m_nCurPos = -1;
			implSeekNext();
		}
		catch (...)
		{
			m_nCurPos = 0;
			m_aElements.realloc(0);
		}
	}
}

//--------------------------------------------------------------------------
sal_Bool ORegistryLevelEnumeration::hasMoreElements() const
{
	return m_nCurPos < m_aElements.getLength();
}

//--------------------------------------------------------------------------
Reference< XRegistryKey > 	ORegistryLevelEnumeration::nextElement()
{
	Reference< XRegistryKey >  xReturn;
	if (hasMoreElements())
	{
		xReturn = m_aElements[m_nCurPos];
		implSeekNext();
	}
	else
		DBG_ERROR("ORegistryLevelEnumeration::nextElement : no more elements !");
	return xReturn;
}

//--------------------------------------------------------------------------
void ORegistryLevelEnumeration::implSeekNext()
{
	while (++m_nCurPos < m_aElements.getLength())
	{
		Reference< XRegistryKey >  xCheck = m_aElements[m_nCurPos];
		if (xCheck.is())
			break;
	}
}

//==========================================================================
//= helper functions for safe reading writing of single key values
//==========================================================================
//------------------------------------------------------------------------------
sal_Bool readValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, ::rtl::OUString& _rValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (xSubKey.is())
			_rValue = xSubKey->getStringValue();
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	catch (InvalidValueException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool readValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, Sequence< ::rtl::OUString >& _rValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (xSubKey.is())
			_rValue = xSubKey->getStringListValue();
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	catch (InvalidValueException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool readValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Int16& _rValue) throw (RuntimeException)
{
	sal_Int32 nVal;
	sal_Bool bSuccess = readValue(_rxKey, _rKeyName, nVal);
	_rValue = nVal;
	return bSuccess;
}

//------------------------------------------------------------------------------
sal_Bool readValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Int32& _rValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (xSubKey.is())
			_rValue = xSubKey->getLongValue();
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	catch (InvalidValueException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool readValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Bool& _rValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (xSubKey.is())
			_rValue = (xSubKey->getLongValue() != 0);
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	catch (InvalidValueException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool writeValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, const ::rtl::OUString& _rValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (!xSubKey.is())
			xSubKey = _rxKey->createKey(_rKeyName);
		if (xSubKey.is())
			xSubKey->setStringValue(_rValue);
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool writeValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, const Sequence< ::rtl::OUString >& _rValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (!xSubKey.is())
			xSubKey = _rxKey->createKey(_rKeyName);
		if (xSubKey.is())
			xSubKey->setStringListValue(_rValue);
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool writeValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Int16 _nValue) throw (RuntimeException)
{
	return writeValue(_rxKey, _rKeyName, (sal_Int32)_nValue);
}

//------------------------------------------------------------------------------
sal_Bool writeValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Int32 _nValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (!xSubKey.is())
			xSubKey = _rxKey->createKey(_rKeyName);
		if (xSubKey.is())
			xSubKey->setLongValue(_nValue);
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
sal_Bool writeValue(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Bool _nValue) throw (RuntimeException)
{
	try
	{
		Reference< XRegistryKey > xSubKey = _rxKey->openKey(_rKeyName);
		if (!xSubKey.is())
			xSubKey = _rxKey->createKey(_rKeyName);
		if (xSubKey.is())
			xSubKey->setLongValue(_nValue ? 1 : 0);
		else
			return sal_False;
	}
	catch (InvalidRegistryException&)
	{
		return sal_False;
	}
	return sal_True;
}

//------------------------------------------------------------------------------
void deleteKey(const Reference< XRegistryKey >& _rxParent, const ::rtl::OUString& _rKeyName) throw (RuntimeException)
{
	try
	{
		_rxParent->deleteKey(_rKeyName);
	}
	catch(InvalidRegistryException&)
	{
	}
}

//------------------------------------------------------------------------------
sal_Bool openKey(const Reference< XRegistryKey >& _rxParent, const ::rtl::OUString& _rKeyName, Reference< XRegistryKey >& _rxKey, sal_Bool _bCreateIfNeccessary)
{
	try
	{
		_rxKey = _rxParent->openKey(_rKeyName);
	}
	catch(InvalidRegistryException&)
	{
	}
	if (_bCreateIfNeccessary && !_rxKey.is())
		try
		{
			_rxKey = _rxParent->createKey(_rKeyName);
		}
		catch(InvalidRegistryException&)
		{
		}

	return _rxKey.is();
}

//------------------------------------------------------------------
sal_Bool writeOrDelete(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, const ::rtl::OUString& _rValue)
{
	if (_rValue.getLength())
		return writeValue(_rxKey, _rKeyName, _rValue);
	deleteKey(_rxKey, _rKeyName);
	return sal_False;
}

//------------------------------------------------------------------
sal_Bool writeOrDelete(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Bool _bValue)
{
	if (_bValue)
		return writeValue(_rxKey, _rKeyName, _bValue);
	deleteKey(_rxKey, _rKeyName);
	return sal_False;
}

//------------------------------------------------------------------
sal_Bool writeOrDelete(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Int32 _nValue)
{
	if (_nValue)
		return writeValue(_rxKey, _rKeyName, _nValue);
	deleteKey(_rxKey, _rKeyName);
	return sal_False;
}

//------------------------------------------------------------------
sal_Bool writeOrDelete(const Reference< XRegistryKey >& _rxKey, const ::rtl::OUString& _rKeyName, sal_Int16 _nValue)
{
	return writeOrDelete(_rxKey, _rKeyName, (sal_Int32)_nValue);
}

//==========================================================================
//= additional helper functions
//==========================================================================
//------------------------------------------------------------------
// compare strings
// TODO : find a better place for this
static int
#if defined( WNT )
 __cdecl
#endif
#if defined( ICC ) && defined( OS2 )
_Optlink
#endif
	NameCompare(const void* pFirst, const void* pSecond)
{
	return ((::rtl::OUString*)pFirst)->compareTo(*(::rtl::OUString*)pSecond);
}

//--------------------------------------------------------------------------
::rtl::OUString getUniqueKeyName(Reference< XRegistryKey >& _rxParentKey, ::rtl::OUString& _rBase)
{
	DBG_ASSERT(_rxParentKey.is(), "ODatabaseContext::implGetUniqueKeyName : invalid parent key !");
	if (!_rxParentKey.is())
		return _rBase;

	Sequence< ::rtl::OUString > aKeyNames;
	try
	{
		aKeyNames = getShortChildKeyNames(_rxParentKey);
	}
	catch (InvalidRegistryException&)
	{
	}

	// sort
	qsort(aKeyNames.getArray(), aKeyNames.getLength(), sizeof(::rtl::OUString), NameCompare);

	sal_Int32 nPostfix = 0;
	::rtl::OUString sCurrentCheck;
	do
	{
		sCurrentCheck = _rBase + ::rtl::OUString::valueOf(nPostfix);
		::rtl::OUString* pExistent = static_cast< ::rtl::OUString* >(bsearch(&sCurrentCheck, aKeyNames.getArray(), aKeyNames.getLength(), sizeof(::rtl::OUString), NameCompare));

		if (!pExistent)
			return sCurrentCheck;
	}
	while (nPostfix++ < 0x7FFFFFFF);

	return ::rtl::OUString();
}

//--------------------------------------------------------------------------
Sequence<rtl::OUString> getShortChildKeyNames(const Reference< XRegistryKey > & _rxKey)
{
	Sequence<rtl::OUString> aReturn;
	if (_rxKey.is())
	{
		aReturn = _rxKey->getKeyNames();
		rtl::OUString* pReturn = aReturn.getArray();
		for (sal_Int32 i=0; i<aReturn.getLength(); ++i, ++pReturn)
		{
			sal_Int32 nSeparatorPos = pReturn->lastIndexOf(L'/');
			if (nSeparatorPos != -1)
				*pReturn = pReturn->copy(nSeparatorPos + 1);
		}
	}
	return aReturn;
}

//--------------------------------------------------------------------------
::rtl::OUString getShortKeyName(const Reference< XRegistryKey > & _rxKey)
{
	rtl::OUString sReturn;
	if (_rxKey.is())
	{
		sReturn = _rxKey->getKeyName();
		sal_Int32 nSeparatorPos = sReturn.lastIndexOf(L'/');
		if (nSeparatorPos != -1)
			sReturn = sReturn.copy(nSeparatorPos + 1);
	}
	return sReturn;
}

