/*************************************************************************
 *
 *  $RCSfile: javaldx.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:15:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <sys/stat.h>

#include <osl/profile.h>
#include <sal/types.h>
#include <tools/solar.h>

/* avoid linking against libcrypt, since crypt is not
 * really needed here */
char *
crypt(const char *key, const char *salt)
{
	return key;
}

/* separate the leading path component from pFile */
static sal_Char*
getDirnameFromFile (sal_Char* pFile)
{
	const sal_Char *pDot = ".";
	sal_Char *pLastSlash;

	/* Find last '/'.  */
	pLastSlash = strrchr (pFile, '/');

	if ( pLastSlash == pFile )
		/* The last slash is the first char, return "/" */
		++pLastSlash;

	if (pLastSlash != NULL)
		/* Terminate the path.  */
		pLastSlash[0] = '\0';
	else
		strncpy (pFile, pDot, strlen( pDot ) + 1 );

	return pFile;
}

/* check whether pPath exists and is a directory */
static sal_Bool
isDirectory( sal_Char* pPath )
{
	struct stat aStatus;

	if (   (stat( pPath, &aStatus ) == 0)
		&& (S_ISDIR(aStatus.st_mode)) )
		return sal_True;

	return sal_False;
}

/* checks whether pPathOrFile is an existing file or directory. If its
 * a file, it is modified to uncover its path component */
static sal_Bool
getDirectory ( sal_Char* pPathOrFile )
{
	if ( isDirectory( pPathOrFile ) )
		return sal_True;
	else
	if ( isDirectory( getDirnameFromFile ( pPathOrFile ) ) )
		return sal_True;
	else
		return sal_False;
}

int
main ( int argc, char **argv )
{
	sal_Char 	pProfileName[ PATH_MAX ];
	sal_Char    pRuntimeLib[ PATH_MAX ];
	oslProfile	hProfile;
	sal_Int32   nResult;

	rtl_uString * uProfileName = NULL;
	rtl_uString * uStarOffice_SVERSION_Key = NULL;
	rtl_uString * uSOfficeINI = NULL;
	rtl_String * buf = NULL;
	
	rtl_uString_newFromAscii( &uStarOffice_SVERSION_Key, STAROFFICE_SVERSION_KEY );
	rtl_uString_newFromAscii( &uSOfficeINI, "soffice" );

	// rtl_uString2String( &buf, rtl_uString_getStr( uSOfficeINI ), rtl_uString_getLength( uSOfficeINI ), RTL_TEXTENCODING_UTF8, OUSTRING_TO_OSTRING_CVTFLAGS) ;
	// fprintf(stderr, "soffice: %s\n", (*buf).buffer ); 
	// rtl_uString2String( &buf, rtl_uString_getStr( uStarOffice_SVERSION_Key ), rtl_uString_getLength( uStarOffice_SVERSION_Key ), RTL_TEXTENCODING_UTF8, OUSTRING_TO_OSTRING_CVTFLAGS) ;
	// fprintf(stderr, "STAROFFICE_SVERSION_KEY: %s\n", (*buf).buffer ); 
	
	/* get fullpath for sofficerc / soffice.ini */
	if ( !osl_getProfileName ( uStarOffice_SVERSION_Key , uSOfficeINI, &uProfileName ) )
	{
		//fprintf( stderr, "No sofficerc found!" );
		return -1;
	}

	rtl_uString2String( &buf, rtl_uString_getStr( uProfileName ), rtl_uString_getLength( uProfileName ), RTL_TEXTENCODING_UTF8, OUSTRING_TO_OSTRING_CVTFLAGS );   
	//fprintf("sofficerc: %s\n", (*buf).buffer);
	/* create the path to the javarc */

	strncpy( pProfileName, (*buf).buffer, (*buf).length);
	/* extract the office installation path */
	strcpy(pProfileName, getDirnameFromFile( pProfileName ) );
	strcat( pProfileName, "/config/javarc" );

	// fprintf(stderr, "try the access to %s\n", pProfileName);
	if ( access( pProfileName, R_OK ) == -1 )
	{
		/* perhaps it is a network installation - look in share/config */
		strcpy( pProfileName, getDirnameFromFile( argv[0] ) );
		strcpy( pProfileName, getDirnameFromFile( pProfileName) );
		strcat( pProfileName, "/share/config/javarc" );
		// fprintf( stderr, "try to access %s\n", pProfileName );

		if ( access( pProfileName, R_OK ) == -1 )
			return -1;
	}
	/* open the profile */
	hProfile = osl_openProfile( pProfileName, osl_Profile_DEFAULT );
	if ( hProfile == (oslProfile)NULL )
		return -1;
	
	/* read the entry RuntimeLib in section Java, return "/lib"
	   if not found */
	nResult = -1;
	if ( osl_readProfileString( hProfile, "Java", "JavaLibPath",
			pRuntimeLib, sizeof( pRuntimeLib ), "" ) )
	{
		/* convert the pRuntimeLib to a path which may extend the
		 * LD_LIBRARY_PATH */
		//if ( getDirectory ( pRuntimeLib ) )
			nResult = 0;
	}

	if ( nResult == 0 )
		fprintf (stdout, "%s\n", pRuntimeLib );

	osl_closeProfile( hProfile );
	return nResult;
}

