/*************************************************************************
 *
 *  $RCSfile: parser.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:16:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#if STLPORT_VERSION < 321
#include <tools/presys.h>
#include <vector.h>
#include <tools/postsys.h>
#else
#include <stl/vector>
#endif

#include <propset.hxx>
#include <stardiv/one/address/address.hxx>

void yyerror( char* s )
{
}

static const sal_Unicode* pCurChar;
static AddressBookQueryTerm* pTerm = 0;

sal_Unicode getChar()
{
	return *pCurChar++;
}

int yylex();

void parseSubstrings( const sal_Unicode* pStr, NAMESPACE_STD(vector)<UString>& rStrings )
{
	NAMESPACE_STD(vector)<sal_Unicode> aCur;
	sal_Unicode aChar;
	while( aChar = *pStr++ )
	{
		if( aChar == L'\\' && *pStr == L'*' )
		{
			aCur.push_back( L'*' );
			continue;
		}
		if( aChar == L'*' )
		{
			sal_Unicode* pCur = new sal_Unicode[ aCur.size() + 1 ];
			pCur[ aCur.size() ] = 0;
			NAMESPACE_STD(copy)( aCur.begin(), aCur.end(), pCur );
			rStrings.push_back( UString( pCur ) );
			delete[] pCur;
			aCur.erase( aCur.begin(), aCur.end() );
		}
		else aCur.push_back( aChar );
	}
	sal_Unicode* pCur = new sal_Unicode[ aCur.size() + 1 ];
	pCur[ aCur.size() ] = 0;
	NAMESPACE_STD(copy)( aCur.begin(), aCur.end(), pCur );
	rStrings.push_back( UString( pCur ) );
	delete[] pCur;
}

//#define YYDEBUG 1
#include "rscyacc.yxx"

AddressBookQueryTerm* parseQuery( const UString& rQuery )
{
	pCurChar = (const sal_Unicode*)rQuery;
	if( !yyparse() ) return pTerm;
	else return 0;
}

int yylex()
{
//	yydebug = 1;
	NAMESPACE_STD(vector)<sal_Unicode> aToken;
	BOOL            bEscaping = FALSE;
	static BOOL     bInValue = FALSE;
	sal_Unicode         wChar;
	static sal_Unicode  wNextChar = 0;

	BOOL bWasInValue = bInValue;
	while( wChar = wNextChar ? wNextChar : getChar() )
	{
		wNextChar = 0;
		if( bEscaping ) 
		{
			bEscaping = FALSE;
			if(wChar == L'(' || wChar == L')' )
			{
				aToken.push_back( wChar );
				continue;
			}
		}
		switch( wChar )
		{
			case L'=': 
			case L'<':
			case L'>':
			case L'~':
			case L'&':
			case L'|':
			case L'*':
			case L'!':
			case L'(':
				if( !bInValue )
				{
					if( aToken.size() )
					{
						wNextChar = wChar;
						yylval.m_pString = new sal_Unicode[ aToken.size() + 1];
						NAMESPACE_STD(copy)( aToken.begin(), aToken.end(), yylval.m_pString );
						yylval.m_pString[ aToken.size() ] = 0;
						aToken.erase( aToken.begin(), aToken.end() );
						return attrib;
					}
					else 
					{
						if( wChar == L'=' ) bInValue = TRUE;
						return wChar;
					}
				}
				else aToken.push_back( wChar );
				break;
				
			case L')': bInValue = FALSE;
				if( aToken.size() )
				{
					yylval.m_pString = new sal_Unicode[ aToken.size() + 1];
					NAMESPACE_STD(copy)( aToken.begin(), aToken.end(), yylval.m_pString );
					yylval.m_pString[ aToken.size() ] = 0;
					bInValue = FALSE;
					aToken.erase( aToken.begin(), aToken.end() );
					wNextChar = wChar;
					return bWasInValue ? value : attrib;
				}
				else return wChar;
			default:
				aToken.push_back( wChar );
		}
	}
	if( aToken.size() )
	{
		yylval.m_pString = new sal_Unicode[ aToken.size() + 1];
		NAMESPACE_STD(copy)( aToken.begin(), aToken.end(), yylval.m_pString );
		yylval.m_pString[ aToken.size() ] = 0;
		aToken.erase( aToken.begin(), aToken.end() );
		return bWasInValue ? value : attrib;
	}
	return 0;
}


main()
{
	parseQuery( L"(&(objectClass=Person)(|(sn=Jensen)(cn=Babs J*)))" );
}

