/******************************************************************************
#*	$Id: parser.y,v 1.1.1.1 2000/09/18 16:16:43 hr Exp $
******************************************************************************/

%union 
{
	wchar_t*                      m_pString;
    AddressBookQueryTerm*         m_pQueryTerm;
    AddressBookQueryFunction      m_eQueryFunction;
}

/* Token */
%token <m_pString>       value
%token <m_pString>       attrib

%type <m_pQueryTerm>    item
%type <m_pQueryTerm>    and
%type <m_pQueryTerm>    or
%type <m_pQueryTerm>    not

%type <m_pQueryTerm>    filtercomp

%type <m_pQueryTerm>    filter
%type <m_pQueryTerm>    filterlist

%type <m_eQueryFunction>  filtertype

%start filter
%%

filter: '(' filtercomp ')'
{
	$$ = $2;
	pTerm = $$;
};

filtercomp: and | or | not | item
;

and: '&' filterlist
{
	$$ = $2;
	$$->Function = AddressBookQueryFunction_AND;
}
;

or: '|' filterlist
{
	$$ = $2;
	$$->Function = AddressBookQueryFunction_OR;
};

not: '!' filter
{
	$$ = new AddressBookQueryTerm;
	$$->Function = AddressBookQueryFunction_NOT;
	UsrAny aAny( $2, AddressBookQueryTerm_getReflection() );
	$$->Arguments = Sequence<UsrAny>( &aAny, 1 );
	delete $2;
};

filterlist: filter
{
	$$ = new AddressBookQueryTerm;
	UsrAny aAny( $1, AddressBookQueryTerm_getReflection() );
	$$->Arguments = Sequence<UsrAny>( &aAny, 1 );
	delete $1;
}
| filterlist filter 
{
	Sequence<UsrAny> aSeq( $1->Arguments.getLen() + 1 );
	UsrAny* pData = aSeq.getArray();
	pData[ 0 ].set( $2, AddressBookQueryTerm_getReflection() );
	NAMESPACE_STD(copy)( $1->Arguments.getConstArray(), $1->Arguments.getConstArray() + 
		  $1->Arguments.getLen(), pData + 1 );
	$1->Arguments = aSeq;
	$$ = $1;
}
;

item: attrib filtertype value
{
	$$ = new AddressBookQueryTerm;
	if( $2 == AddressBookQueryFunction_EQUALITYMATCH &&
		UString( L"*" ) == $3 )
	{
		$$->Function = AddressBookQueryFunction_PRESENT;
		UsrAny aStr;
		aStr.setString( $1 );
		$$->Arguments = Sequence<UsrAny>( &aStr, 1 );
		delete $3;
		delete $1;
	}
	else
	{
		if( UString( $3 ).search( L'*' ) != USTRING_NOTFOUND )
		{
			$$->Function = AddressBookQueryFunction_SUBSTRINGS;
			NAMESPACE_STD(vector)<UString> aStrings;
			parseSubstrings( $3, aStrings );
			delete[] $3;
			$$->Arguments.realloc( aStrings.size() + 1 );
			UsrAny* pSeq = $$->Arguments.getArray();
			const UsrAny* pEnd = pSeq + $$->Arguments.getLen();
			*pSeq++ <<= UString( $1 );
			delete[] $1;
			NAMESPACE_STD(vector)<UString>::const_iterator aIter = aStrings.begin();
			while( pSeq != pEnd )
				*pSeq++ <<= *aIter++;
		}
		else 
		{
			$$->Function = $2;
			UsrAny aList[ 2 ];
			aList[ 0 ] <<= UString( $1 );
			aList[ 1 ] <<= UString( $3 );
			delete[] $1;
			delete[] $3;
			$$->Arguments = Sequence<UsrAny>( aList, 2 );
		}
	}
};

filtertype: '=' 
{
	$$ = AddressBookQueryFunction_EQUALITYMATCH;
}
| '~' '=' 
{
	$$ = AddressBookQueryFunction_APPROXMATCH;
}
| '>' '=' 
{
	$$ = AddressBookQueryFunction_GREATEROREQUAL;
}
| '<' '='
{
	$$ = AddressBookQueryFunction_LESSOREQUAL;
}
;



/******************************************************************************
	$Log: parser.y,v $
	Revision 1.1.1.1  2000/09/18 16:16:43  hr
	initial import
	
	Revision 1.8  1999/11/08 12:50:19  mh
	changes for VC6 and STL321
	
	Revision 1.7  1999/09/27 19:23:36  tlx
	More ldap fixes
	
	Revision 1.6  1999/09/27 18:28:19  tlx
	ldap fixes
	
	Revision 1.5  1999/09/20 09:19:19  tlx
	Adressbuch
	
	Revision 1.4  1999/08/03 12:36:58  tlx
	Changes
	
	Revision 1.3  1999/07/30 15:43:54  tlx
	Linux
	
	Revision 1.2  1999/07/29 13:45:19  tlx
	Fixes
	
	Revision 1.1  1999/07/28 16:21:47  tlx
	Addressbuch #1
	
******************************************************************************/
