/*************************************************************************
 *
 *  $RCSfile: registrycache.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __FRAMEWORK_CLASSES_REGISTRYCACHE_HXX_
#define __FRAMEWORK_CLASSES_REGISTRYCACHE_HXX_

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_MACROS_DEBUG_HXX_
#include <macros/debug.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XREGISTRYKEY_HPP_
#include <com/sun/star/registry/XRegistryKey.hpp>
#endif

#ifndef _COM_SUN_STAR_UNO_EXCEPTION_HPP_
#include <com/sun/star/uno/Exception.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_PROPERTYVALUE_HPP_
#include <com/sun/star/beans/PropertyValue.hpp>
#endif

//_________________________________________________________________________________________________________________
//	other includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_
#include <com/sun/star/uno/Sequence.h>
#endif

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

#ifndef __SGI_STL_VECTOR
#include <stl/vector>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

#define	EXCEPTION					::com::sun::star::uno::Exception
#define	MUTEX						::osl::Mutex
#define	OUSTRING					::rtl::OUString
#define	PROPERTYVALUE				::com::sun::star::beans::PropertyValue
#define	REFERENCE					::com::sun::star::uno::Reference
#define	SEQUENCE					::com::sun::star::uno::Sequence
#define	XREGISTRYKEY				::com::sun::star::registry::XRegistryKey
#define	XSIMPLEREGISTRY				::com::sun::star::registry::XSimpleRegistry

//_________________________________________________________________________________________________________________
//	exported const
//_________________________________________________________________________________________________________________

// These index value is not allowed in RegistryCache.
// He is a sign for wrong searchresults of findxxx()-methods at this object.
// => The consequence is - a cache can have 0xFFFFFFFD elements as maximum!
#define	INVALID_CACHE_POSITION                              0xFFFFFFFF

// Keynames of registrykeys to find ouer cached values.
#define	REGISTRYKEY_LOADER									OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader"					))
#define	REGISTRYKEY_IMPLEMENTATIONS							OUString(RTL_CONSTASCII_USTRINGPARAM("/IMPLEMENTATIONS/"		))
#define	REGISTRYKEY_LOADER_PATTERN							OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/Pattern"			))
#define	REGISTRYKEY_LOADER_EXTENSIONS						OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/Extension"		))
#define	REGISTRYKEY_LOADER_MIMETYPE							OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/MimeType"			))
#define	REGISTRYKEY_LOADER_FLAGS							OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/Flags"			))
#define	REGISTRYKEY_LOADER_FILTERNAME						OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/FilterName"		))
#define	REGISTRYKEY_LOADER_CLIPBOARDFORMAT					OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/ClipboardFormat"	))
#define	REGISTRYKEY_LOADER_DETECTSERVICE					OUString(RTL_CONSTASCII_USTRINGPARAM("/Loader/DetectService"	))

// There exist a method to return all values of an entry as property sequence.
// We must know count of it for create right sequence!
#define	MAXCOUNT_SUPPORTED_REGISTRYPROPERTIES				7

//_________________________________________________________________________________________________________________
//	exported definitions
//_________________________________________________________________________________________________________________

// Define a dynamical stringlist with ::rtl::OUString as elementtype.
typedef ::std::vector< OUSTRING >	tIMPLStringList;

// Define a structure for one cache-item in STL-format with using vector as list.
// Used as dynamical cache entry for internal cache structure.
// Attention: If you add some elements to these struct and create new MASK_... types for nInvalidMask look for
// right mask value! It exist some masks for non existing items in these structs! (STREAM for example!)
struct tIMPLRegistryEntry_stl
{
    OUSTRING				sImplementationName	;		// The implementationname of a loader service.
    tIMPLStringList			aPatternList		;		// A stringlist of patterns for filter.
    tIMPLStringList			aExtensionList		;		// A stringlist of extensions for filter.
	OUSTRING				sMimeType			;		// MimeType of filter
	sal_Int32				nFlags				;		// Flags for filter
	OUSTRING				sFilterName			;		// Name of filter
	sal_Int32				nClipboardFormat	;		// Format of filter
	OUSTRING				sDetectService		;		// Servicename for expanded filterdetection

	sal_Int32				nValidMask			;		// Statemask to mark a entry value as valid or invalid!
														// We use it to distinguish between emtpy or non existing values of an entry.
														// In the moment these is not neccessary, because default values (like 0 at Flags or "" at FilterName)
														// are not allowed(!) ... but nobody know, when these mechanism is changed!
														// Its better to save this state informations now.
};

// Define a structure for one cache-item in UNO-format with using sequences as list.
// Its neccessary for performant access and return value only! Sequences are to much static and not suitable for dynamic changes!
// See operator[] for THESE structure-type for further informations.
// Attention: If you add some elements to these struct and create new MASK_... types for nInvalidMask look for
// right mask value! It exist some masks for non existing items in these structs! (STREAM for example!)
struct tIMPLRegistryEntry_uno
{
    OUSTRING				sImplementationName	;
    SEQUENCE< OUSTRING >	seqPatternList		;
    SEQUENCE< OUSTRING >	seqExtensionList	;
	OUSTRING				sMimeType			;
	sal_Int32				nFlags				;
	OUSTRING				sFilterName			;
	sal_Int32				nClipboardFormat	;
	OUSTRING				sDetectService		;

	sal_Int32				nValidMask			;
};

// Define the cache himself as list of entries with count.
typedef ::std::vector< tIMPLRegistryEntry_stl >	tIMPLRegistryCache;
typedef ::std::vector< tIMPLRegistryEntry_stl >::iterator	tIMPLRegistryCacheIterator;

/*-************************************************************************************************************//**
	@short          cache for special keys of global registry fro frameloaderfactory
	@descr			The frameloaderfactory need some special keys from ouer global registry. (applicat.rdb+user.rdb)
					These keys are not changed during runtime (I hope it!). For better performance its neccessary to
					cache all needed values.

	@implements		-
	@base			-

	@ATTENTION		This class is not threadsafe.

	@devstatus		deprecated
*//*-*************************************************************************************************************/

class RegistryCache
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard constructor
			@descr		This will not initialize the cache automaticly! Use method "fill()" to do that.
						After creation cache is empty.

			@seealso	method fill()

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	 	RegistryCache();

		/*-****************************************************************************************************//**
			@short		standard destructor to delete instance
			@descr		This will clear the cache, if programmer forget this.

			@seealso	method clear()

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		virtual ~RegistryCache();

		/*-****************************************************************************************************//**
			@short		get the current state of the cache
			@descr		Call this method to get information about the state of the current cache.
						I make this method inline for better performance!

			@seealso	-

			@param		-
			@return		sal_True  ,if cache is initialzed and work correct
			@return		sal_False ,otherwise

			@onerror	The return value is sal_False.
		*//*-*****************************************************************************************************/

		sal_Bool isValid() const;

		/*-****************************************************************************************************//**
			@short		initialize the cache with new values
			@descr		This will fill the cache with new values. We make an clear() automaticly!
						The registry will not changed at runtime ... don't call this method again, if
						this cache is valid. We will do nothing, if it so.

			@seealso	method clear()

			@param		"xRegistry" is the registry to cache.
			@return		-

			@onerror	The cache state is set to INVALID!
		*//*-*****************************************************************************************************/

		void fill( const REFERENCE< XSIMPLEREGISTRY >& xRegistry );

		/*-****************************************************************************************************//**
			@short		clear the cache and free memory
			@descr		This will clear the cache. If you call this method again and the cache is empty, we do nothing!

			@seealso	method fill()

			@param		-
			@return		-

			@onerror	The cache state is set to INVALID!
		*//*-*****************************************************************************************************/

		void clear();

		/*-****************************************************************************************************//**
			@short		get count of current entries in cache
			@descr		You can use this for access to all entries in cache to use it at loops.
						The cache is static and don't referesh his content himself.

			@seealso	method getPatternCount()
			@seealso	method getExtensionCount()

			@param		-
			@return		Count of current cache entries as unsigned long.

			@onerror	We return 0.
		*//*-*****************************************************************************************************/

		sal_uInt32 getEntryCount() const;

		/*-****************************************************************************************************//**
			@short		get pure cache item struct for directly access
			@descr		If you can do all your jobs without a comfortable access to the elements of a cache item -
						use these method.

			@seealso	method getEntryCount()
			@seealso	method getEntryAsUNOVersion()

			@param		"nEntryNumber" is the number of searched entry and must be in range of [ 0 ... count-1 ]!
			@return		We return the specified cache entry as pure struct without a chance for comfortable access to his elements!

			@onerror	If you use this method with correct parameter (get it with call of getCount() before!),
						no error will occurre! Otherwise its your problem and we will throw an ASSERT in debug version.
		*//*-*****************************************************************************************************/

		const tIMPLRegistryEntry_stl& operator[]( sal_uInt32 nEntryNumber ) const;

		/*-****************************************************************************************************//**
			@short		get pure cache item struct for directly access
			@descr		These method do the same like operator[] before. Only the return type is different.
						All items of returned structure are converted to UNO-types. (We use STL internal!)

			@seealso	method getEntryCount()
			@seealso	method operator[]

			@param		"nEntryNumber" is the number of searched entry and must be in range of [ 0 ... count-1 ]!
			@return		We return the specified cache entry as pure struct without a chance for comfortable access to his elements!

			@onerror	If you use this method with correct parameter (get it with call of getCount() before!),
						no error will occurre! Otherwise its your problem and we will throw an ASSERT in debug version.
		*//*-*****************************************************************************************************/

		tIMPLRegistryEntry_uno getEntryAsUNOVersion( sal_uInt32 nEntryNumber ) const;

		/*-****************************************************************************************************//**
			@short		get pure cache item struct for directly access
			@descr		These method do the same like operator[] before. Only the return type is different.
						All items of returned structure are converted to sequence of propertyvalues.
						(We use STL internal!)

			@seealso	method getEntryCount()
			@seealso	method operator[]

			@param		"nEntryNumber" is the number of searched entry and must be in range of [ 0 ... count-1 ]!
			@return		We return the specified cache entry as pure list without a chance for comfortable access to his elements!

			@onerror	If you use this method with correct parameter (get it with call of getCount() before!),
						no error will occurre! Otherwise its your problem and we will throw an ASSERT in debug version.
		*//*-*****************************************************************************************************/

		SEQUENCE< PROPERTYVALUE > getEntryAsPropertySequence( sal_uInt32 nEntryNumber ) const;

		/*-****************************************************************************************************//**
			@short		get values of cache entry converted to UNO-types
			@descr		With these methods, you have access to the elements of the specified cache entry.
						(like int, string, sequences ...)

			@seealso	methods for direct list access below

			@param		"nEntryNumber" is the number of searched entry and must be in range of [ 0 ... count-1 ]!
			@return		Value of entry as UNO-type.

			@onerror	If you use this method with correct parameter (get it with call of getCount() before!),
						no error will occurre! Otherwise its your problem and we will throw an ASSERT in debug version.
		*//*-*****************************************************************************************************/

		OUSTRING				getImplementationName	( sal_uInt32 nEntryNumber ) const;
		SEQUENCE< OUSTRING >	getPatternList			( sal_uInt32 nEntryNumber ) const;
    	SEQUENCE< OUSTRING >	getExtensionList		( sal_uInt32 nEntryNumber ) const;
		OUSTRING				getMimeType				( sal_uInt32 nEntryNumber ) const;
		sal_Int32				getFlags				( sal_uInt32 nEntryNumber ) const;
		OUSTRING				getFilterName			( sal_uInt32 nEntryNumber ) const;
		sal_Int32				getClipboardFormat		( sal_uInt32 nEntryNumber ) const;
		OUSTRING				getDetectService		( sal_uInt32 nEntryNumber ) const;
		sal_Int32				getValidMask			( sal_uInt32 nEntryNumber ) const;

		/*-****************************************************************************************************//**
			@short		get entrynumber of corresponding entry for searched values
			@descr		Sometimes you must step over all cache entries to find right entry with corresponding
						informations. Use these methods to do this for simple types(!) - no listsearch at entry values
						is supported. (=> no search for pattern or extensions!) You can step during ouer cache itemlist
						by give as a startpoint is these list as optional parameter. Otherwise we search from beginning
						of cache. If search succesfull you will get the number of corresponding cache-entry.
						With this value you have full access to his other informations ( use operator[]! )
						If no entry was found - we return INVALID_CACHE_POSITION.

			@seealso	define INVALID_CACHE_POSITION

			@param		First parameter is the value for searchin in cache.
			@param		"nStartEntry" is the startposition for continued search.
			@return		Position of found entry in cache, which correspond with searchparameter.
						We return INVALID_CACHE_POSITION if no matching entry was found.

			@onerror	INVALID_CACHE_POSITION is returned.
		*//*-*****************************************************************************************************/

		sal_uInt32	findMimeType		( const OUSTRING& sSearchMimeType		, sal_uInt32 nStartEntry=0 ) const;
		sal_uInt32	findFlags			( 		sal_Int32 nSearchFlags			, sal_uInt32 nStartEntry=0 ) const;
		sal_uInt32	findFilterName		( const OUSTRING& sSearchFilterName		, sal_uInt32 nStartEntry=0 ) const;
		sal_uInt32	findClipboardFormat	( 		sal_Int32 nSearchClipboardFormat, sal_uInt32 nStartEntry=0 ) const;

		/*-****************************************************************************************************//**
			@short		get count of current patterns or extensions of an entry in cache
			@descr		If you wish to have direct access to the string list for pattern or extension, you can
						use follow methods to get count of lists to step during it after these call.
						The cache is static and don't refresh his content himself.

			@seealso	method getEntryCount()
			@seealso	method getPattern()
			@seealso	method getExtension()

			@param		"nEntryNumber" is the number of searched entry and must be in range of [ 0 ... count-1 ]!
			@return		Count of current patterns or extensions of specified cache entry as unsigned long.

			@onerror	We return 0.
		*//*-*****************************************************************************************************/

		sal_uInt32 getPatternCount	( sal_uInt32 nEntryNumber ) const;
		sal_uInt32 getExtensionCount( sal_uInt32 nEntryNumber ) const;

		/*-****************************************************************************************************//**
			@short		get the searched pattern, extension or mimetype of the specified entry in cache
			@descr		With these methods you have full access directly to the pattern or extensions of specified cache entry.

			@seealso	method getPatternCount()
			@seealso	method getExtensionCount()

			@param		"nEntryNumber"		is the number of searched entry and must be in range of [ 0 ... count-1 ]!
			@param		"nPatternNumber"	is the number of searched pattern and must be in range of [ 0 ... count-1 ]!
			@param		"nExtensionNumber"	is the number of searched extension and must be in range of [ 0 ... count-1 ]!
			@return		Value of pattern or extension of cache entry.

			@onerror	If you use this method with correct parameter (get it with call of get...Count() before!),
						no error will occurre! Otherwise its your problem and we will throw an ASSERT in debug version.
		*//*-*****************************************************************************************************/

		OUSTRING getPattern		(	sal_uInt32	nEntryNumber	,
									sal_uInt32	nPatternNumber	) const;
		OUSTRING getExtension	(	sal_uInt32	nEntryNumber	,
							   		sal_uInt32	nExtensionNumber) const;

	//-------------------------------------------------------------------------------------------------------------
	//	protected methods
	//-------------------------------------------------------------------------------------------------------------

	protected:

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

	private:

		/*-****************************************************************************************************//**
			@short		set the value or the values of one registrykey on a cache-entry
			@descr		A registryentry can have different types. (string, ASCII-value, list ...)
						Follow methods set correct values on cache entry.

			@seealso	method fill()

			@param		"xKey"  is a reference to a valid registrykey (We will not control this!)
			@param		"...Item" is a reference to a valid element of a cache entry to set the value(s).
			@return		-

			@onerror	An exception can be thrown.
		*//*-*****************************************************************************************************/

		void impl_setKeyValueOnCacheEntry(const	REFERENCE< XREGISTRYKEY >&	xKey	,
			 									tIMPLStringList&			aItem	) throw ( EXCEPTION );
		void impl_setKeyValueOnCacheEntry(const	REFERENCE< XREGISTRYKEY >&	xKey	,
			 									OUSTRING&					sItem	) throw ( EXCEPTION );
		void impl_setKeyValueOnCacheEntry(const	REFERENCE< XREGISTRYKEY >&	xKey	,
			 									sal_Int32&					nItem	) throw ( EXCEPTION );

		/*-****************************************************************************************************//**
			@short		fill cache with additional informations from "install.ini"
			@descr		This is a temp. Hack for sfxto use unregistered loader which are described in
						"install.ini". If all these loader are written to normal registry we can delete this method!
						These method search the ini-file, open it, get full access and close it.
						Load of sections and filter values is forwarded to impl_loadFilterSection() and impl_cacheFilterValues().

			@seealso	method fill()
			@seealso	method impl_loadFilterSection()
			@seealso	method impl_cacheFilterValues()

			@param		-
			@return		-

			@onerror	The cache state is set to INVALID or wrong data block is ignored!
		*//*-*****************************************************************************************************/

		void impl_loadInstallINI();

		/*-****************************************************************************************************//**
			@short		load a filter section from install.ini in cache
			@descr		This is a helper method for impl_loadInstallINI() to load ONE section of these file.
						We use impl_cacheFilterValues() to cache values of one filter line of given section.

			@seealso	method fill()
			@seealso	method impl_loadInstallINI()
			@seealso	method impl_cacheFilterValues()

			@param		"xSection"; opened registry key for full access to special section of install.ini.
			@return		-

			@onerror	The cache state is set to INVALID or wrong data block is ignored!
		*//*-*****************************************************************************************************/

		void impl_loadFilterSection( const REFERENCE< XREGISTRYKEY >& xSection );

		/*-****************************************************************************************************//**
			@short		analyze a filter string and cache important values
			@descr		All values of an filter are decoed as one string line seperated by ','.
						These method analyze given line and cache important values only.

			@seealso	method fill()
			@seealso	method impl_loadInstallINI()
			@seealso	method impl_loadFilterSection()

			@param		"sFilterName"; name of caching filter
			@param		"sValues"    ; values of filter as string line
			@return		-

			@onerror	The cache state is set to INVALID or wrong data block is ignored!
		*//*-*****************************************************************************************************/

		void impl_cacheFilterValues(	const	OUSTRING&	sFilterName	,
										const	OUSTRING&	sValues		);

		/*-****************************************************************************************************//**
			@short		return all token of a given string seperated by given letter
			@descr		These is a helper method to parse a string and seperate some values by given seperator.
						We return the results as string sequence for better access.

			@seealso	method impl_cacheFilterValues()

			@param		"sString"	; string for parsing
			@param		"cSperator"	; seperator for parsing
			@return		-

			@onerror	An empty sequence is returned.
		*//*-*****************************************************************************************************/

		SEQUENCE< OUSTRING > impl_getAllToken(	const	OUSTRING&		sString		,
												const	sal_Unicode&	cSeperator	);

	//-------------------------------------------------------------------------------------------------------------
	//	debug methods
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		debug-method to check incoming parameter of some other mehods of this class
			@descr		The following methods are used to check parameters for other methods
						of this class. The return value is used directly for an ASSERT(...).

			@seealso	ASSERT in implementation!

			@param		references to checking variables
			@return		sal_False ,on invalid parameter
			@return		sal_True  ,otherwise

			@onerror	-
		*//*-*****************************************************************************************************/

	#ifdef ENABLE_ASSERTIONS

	private:

	#endif	//	#ifdef ENABLE_ASSERTIONS

		/*-****************************************************************************************************//**
			@short		debug method to log current cache content
			@descr		The stl vector is not suitable enough to show informations about his currnt content.
						To get a overview you can call this special debug method. It will log all important informations
						to a file on disk.

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	#ifdef ENABLE_REGISTRYCACHEDEBUG

	private:

		void impldbg_showCacheContent();

	#endif	//	#ifdef ENABLE_REGISTRYCACHEDEBUG

	//-------------------------------------------------------------------------------------------------------------
	//	private variables
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	private:

		tIMPLRegistryCache	   					m_aRegistryCache			;	/// STL vector as container for caching values.
		sal_Int32								m_nNumberOfPreferedFilters	;	/// Position for insert prefered filter in right order.

};		//	class RegistryCache

}		//	namespace framework

#endif	//	#ifndef __FRAMEWORK_CLASSES_REGISTRYCACHE_HXX_
