/*************************************************************************
 *
 *  $RCSfile: opipeconnection.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __FRAMEWORK_HELPER_OPIPECONNECTION_HXX_
#define __FRAMEWORK_HELPER_OPIPECONNECTION_HXX_

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_MACROS_GENERIC_HXX_
#include <macros/generic.hxx>
#endif

#ifndef __FRAMEWORK_MACROS_XINTERFACE_HXX_
#include <macros/xinterface.hxx>
#endif

#ifndef __FRAMEWORK_MACROS_DEBUG_HXX_
#include <macros/debug.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

#ifndef _COM_SUN_STAR_CONNECTION_XCONNECTION_HPP_
#include <com/sun/star/connection/XConnection.hpp>
#endif

#ifndef _COM_SUN_STAR_IO_IOEXCEPTION_HPP_
#include <com/sun/star/io/IOException.hpp>
#endif

//_________________________________________________________________________________________________________________
//	other includes
//_________________________________________________________________________________________________________________

#ifndef _CPPUHELPER_WEAK_HXX_
#include <cppuhelper/weak.hxx>
#endif

#ifndef _OSL_FILE_H_
#include <osl/file.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

#define	IOEXCEPTION							::com::sun::star::io::IOException
#define	OUSTRING							::rtl::OUString
#define	OWEAKOBJECT							::cppu::OWeakObject
#define	REFERENCE							::com::sun::star::uno::Reference
#define	RUNTIMEEXCEPTION					::com::sun::star::uno::RuntimeException
#define	SEQUENCE							::com::sun::star::uno::Sequence
#define	XCONNECTION							::com::sun::star::connection::XConnection
#define	XMULTISERVICEFACTORY				::com::sun::star::lang::XMultiServiceFactory
#define	MUTEX								::osl::Mutex

//_________________________________________________________________________________________________________________
//	exported const
//_________________________________________________________________________________________________________________

#define	PIPEDESCRIPTION						OUSTRING(RTL_CONSTASCII_USTRINGPARAM("Mozilla parent pipe connection"))

//_________________________________________________________________________________________________________________
//	exported definitions
//_________________________________________________________________________________________________________________

/*-************************************************************************************************************//**
	@short			establish a connection between plugin DLL and EXE by means of pipes
	@descr			The plugin DLL must communicate with ouer EXE to create PlugIn objects.
					They use a bridge; and these bridge use ouer OPipeConnection for communication.

	@implements		XInterface
					XConnection
					[ XDebugging if ENABLE_SERVICEDEBUG is defined! ]
	@base			OWeakObject

	@devstatus		deprecated
*//*-*************************************************************************************************************/

//class OPipeConnection	:   DERIVE_FROM_XSPECIALDEBUGINTERFACE		// => These macro will expand to nothing, if no testmode is set in debug.h!
class OPipeConnection	:	public XCONNECTION					,
							public OWEAKOBJECT
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard ctor
			@descr		These initialize a new instance of ths class with needed informations for work.

			@seealso	using at owner

			@param		"xFactory"; Reference to servicemanager, which has created these instance.
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

	 	OPipeConnection( const REFERENCE< XMULTISERVICEFACTORY >& xFactory );

		//---------------------------------------------------------------------------------------------------------
		//	XInterface
		//---------------------------------------------------------------------------------------------------------

		DECLARE_XINTERFACE
//		DECLARE_XSPECIALDEBUGINTERFACE	// => These macro will expand to nothing, if no testmode is set in debug.h!

		//---------------------------------------------------------------------------------------------------------
		//	XConnection
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short      try to read some bytes from READPIPE
			@descr		We return count of reading bytes and the data himself.
						If we can't read something or pipe is closed we throw an exception or return 0 for count.

			@seealso	method write()

			@param		"seqReadBytes"; list of reading bytes for return.
			@param		"nBytesToRead"; count of bytes to read.
			@return		We return count of reading bytes. It can be in range of [0 ... nBytesToRead].

			@onerror	We throw an IOException or return 0.
		*//*-*****************************************************************************************************/

		virtual sal_Int32 SAL_CALL read(	SEQUENCE< sal_Int8 >&	seqReadBytes	,
											sal_Int32				nBytesToRead	) throw(	IOEXCEPTION		,
																								RUNTIMEEXCEPTION);

		/*-****************************************************************************************************//**
			@short      try to write some bytes to WRITEPIPE
			@descr		We try to write all bytes to pipe. But if an error occured or something else will be wrong ...
						we throw an exception.

			@seealso	method read()

			@param		"seqData"; list of bytes for writing.
			@return		-

			@onerror	We throw an IOException.
		*//*-*****************************************************************************************************/

    	virtual void SAL_CALL write( const SEQUENCE< sal_Int8 >& seqData ) throw(	IOEXCEPTION		,
																					RUNTIMEEXCEPTION);

		/*-****************************************************************************************************//**
			@short      not implemented yet
			@descr		-

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

    	virtual void SAL_CALL flush() throw(	IOEXCEPTION		,
												RUNTIMEEXCEPTION);

		/*-****************************************************************************************************//**
			@short      close all pipes and set internal working state to NOT READY
			@descr		After this call these instance will not work!

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

    	virtual void SAL_CALL close() throw(	IOEXCEPTION		,
												RUNTIMEEXCEPTION);

		/*-****************************************************************************************************//**
			@short      return identifier to describe type of connection
			@descr		We use this object to support a pipe connection for mozilla plugins.
						You can't use this obect in another way!

			@seealso	-

			@param		-
			@return		Description of supported connection.

			@onerror	-
		*//*-*****************************************************************************************************/

    	virtual OUSTRING SAL_CALL getDescription() throw( RUNTIMEEXCEPTION );

	//-------------------------------------------------------------------------------------------------------------
	//	protected methods
	//-------------------------------------------------------------------------------------------------------------

	protected:

		/*-****************************************************************************************************//**
			@short		standard destructor
			@descr		This method destruct an instance of this class and clear some member.
						This method is protected, because its not allowed to use an instance of this class as a member!
						You MUST use a pointer.

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		virtual	~OPipeConnection();

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

	private:

		/*-****************************************************************************************************//**
			@short		-
			@descr		-

			@seealso	-

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

		void impl_closeAllPlugIns();

	//-------------------------------------------------------------------------------------------------------------
	//	debug methods
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		debug-method to check incoming parameter of some other mehods of this class
			@descr		The following methods are used to check parameters for other methods
						of this class. The return value is used directly for an ASSERT(...).

			@seealso	ASSERTs in implementation!

			@param		references to checking variables
			@return		sal_False on invalid parameter<BR>
						sal_True  otherway

			@onerror	-
		*//*-*****************************************************************************************************/

	#ifdef ENABLE_ASSERTIONS

	private:

	 	sal_Bool impldbg_checkParameter_OPipeConnectionCtor	(	const	REFERENCE< XMULTISERVICEFACTORY >&	xFactory		);
		sal_Bool impldbg_checkParameter_read				(			SEQUENCE< sal_Int8 >&				seqReadBytes	,
																		sal_Int32							nBytesToRead	);
    	sal_Bool impldbg_checkParameter_write				(	const	SEQUENCE< sal_Int8 >&				seqData			);

	#endif	// #ifdef ENABLE_ASSERTIONS

	//-------------------------------------------------------------------------------------------------------------
	//	variables
	//	(should be private everyway!)
	//-------------------------------------------------------------------------------------------------------------

	private:

		MUTEX									m_aReadMutex	;	/// We need a mutex for every pipe object in these class! read() and write() are called from different threads!
		MUTEX									m_aWriteMutex	;
		REFERENCE< XMULTISERVICEFACTORY >		m_xFactory		;	/// Reference to servicemanager, which has created these instance.
		oslFileHandle							m_aReadPipe		;	/// Pipe for reading
		oslFileHandle							m_aWritePipe	;	/// Pipe for writing
		sal_Bool								m_bReady		;	/// Flag to prevent against multiple closing or working after it.

};		//	class OPipeConnection

}		//	namespace framework

#endif	//	#ifndef __FRAMEWORK_HELPER_OPIPECONNECTION_HXX_
