/*************************************************************************
 *
 *  $RCSfile: registrycache.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_REGISTRYCACHE_HXX_
#include <classes/registrycache.hxx>
#endif

#ifndef __FRAMEWORK_DEFINES_HXX_
#include <defines.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

/*-HACK FOR SFX ******************************************************************************************/

#ifndef _COM_SUN_STAR_FRAME_XCONFIGMANAGER_HPP_
#include <com/sun/star/frame/XConfigManager.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XMULTISRVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

#ifndef _UNOTOOLS_PROCESSFACTORY_HXX_
#include <unotools/processfactory.hxx>
#endif

#ifndef _VOS_PROCESS_HXX_
#include <vos/process.hxx>
#endif

#ifndef _OSL_FILE_HXX_
#include<osl/file.hxx>
#endif

#ifndef _RTL_USTRBUF_HXX_
#include <rtl/ustrbuf.hxx>
#endif

/*-HACK FOR SFX ******************************************************************************************/

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl						;
using namespace ::rtl						;
using namespace ::com::sun::star::uno		;
using namespace ::com::sun::star::registry	;
using namespace ::com::sun::star::beans		;
/*-HACK FOR SFX ******************************************************************************************/
using namespace ::vos						;
using namespace ::com::sun::star::frame		;
using namespace ::com::sun::star::lang		;
/*-HACK FOR SFX ******************************************************************************************/

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
RegistryCache::RegistryCache()
	: m_nNumberOfPreferedFilters(0)
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
RegistryCache::~RegistryCache()
{
	// Don't forget to free memory!
	if ( m_aRegistryCache.size() > 0 )
	{
		clear();
	}
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_Bool RegistryCache::isValid() const
{
	// If we have some elements in cache ... cache is valid.
	// Other way ... invalid.
	return ( m_aRegistryCache.size() > 0 );
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
void RegistryCache::fill( const Reference< XSimpleRegistry >& xRegistry )
{
	try
	{

	// Do nothing if cache already filled!
	if ( isValid() == INVALID )
	{
		// Exist a registry?
        if ( xRegistry.is() == sal_True )
        {
			// YES; the get access to the entries.
            Reference< XRegistryKey > xRootKey	= xRegistry->getRootKey();
            Reference< XRegistryKey > xKey		= xRootKey->openKey( REGISTRYKEY_LOADER );

			// Warn programmer, if no loader registered in applicat.rdb or user.rdb!
			LOG_ASSERT( !(xKey.is() == sal_False), "RegistryCache::fill()\nNo loader found in registry!\nCan't fill cache with values.\n" )

            if ( xKey.is() == sal_True )
			{
				if ( xKey->getValueType() == RegistryValueType_ASCIILIST )
            	{
					// Get names of all subkeys of "/Loader". These are the names of all well known loader's.
                	Sequence< OUString >	seqSubkeys	= xKey->getAsciiListValue();
					sal_Int32				nPosition	= 0;
					sal_Int32				nLength		= seqSubkeys.getLength();

					// Cache all important informations of this subkeys!
                	for ( nPosition=0; nPosition<nLength; ++nPosition )
                	{
						// Create a new entry for cache.
						// We don't must use the heap! The stl-classes allocate his own memory.
                    	tIMPLRegistryEntry_stl aEntry;
						aEntry.nValidMask =0; // Mark all value states as INVALID!

						// Set name of cached entry.
						aEntry.sImplementationName = seqSubkeys[nPosition];

						// Build full entryname in registry.
						// ("/IMPLEMENTATIONS/<LoaderName>")
                    	OUString sBaseKeyName = REGISTRYKEY_IMPLEMENTATIONS;
                    	sBaseKeyName += seqSubkeys[nPosition];

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/Pattern" in cache
                    	OUString sPatternKeyName = sBaseKeyName;
                    	sPatternKeyName += REGISTRYKEY_LOADER_PATTERN;

                    	xKey = xRootKey->openKey( sPatternKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.aPatternList );
							aEntry.nValidMask |= MASK_PATTERN; // Mark Pattern as valid!
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/Extensions" in cache.
                    	OUString sExtensionsKeyName = sBaseKeyName;
                    	sExtensionsKeyName += REGISTRYKEY_LOADER_EXTENSIONS;

                    	xKey = xRootKey->openKey( sExtensionsKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.aExtensionList );
							aEntry.nValidMask |= MASK_EXTENSION; // Mark Extension as valid!
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/MimeType" in cache.
                    	OUString sMimeTypeKeyName = sBaseKeyName;
                    	sMimeTypeKeyName += REGISTRYKEY_LOADER_MIMETYPE;

                    	xKey = xRootKey->openKey( sMimeTypeKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.sMimeType );
							aEntry.nValidMask |= MASK_MIMETYPE; // Mark MimeType as valid!
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/Flags" in cache.
                    	OUString sFlagsKeyName = sBaseKeyName;
                    	sFlagsKeyName += REGISTRYKEY_LOADER_FLAGS;

                    	xKey = xRootKey->openKey( sFlagsKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.nFlags );
							aEntry.nValidMask |= MASK_FLAGS; // Mark Flags as valid!
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/FilterName" in cache.
                    	OUString sFilterNameKeyName = sBaseKeyName;
                    	sFilterNameKeyName += REGISTRYKEY_LOADER_FILTERNAME;

                    	xKey = xRootKey->openKey( sFilterNameKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.sFilterName );
							aEntry.nValidMask |= MASK_FILTERNAME; // Mark FilterName as valid!
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/ClipboardFormat" in cache.
                    	OUString sClipboardFormatKeyName = sBaseKeyName;
                    	sClipboardFormatKeyName += REGISTRYKEY_LOADER_CLIPBOARDFORMAT;

                    	xKey = xRootKey->openKey( sClipboardFormatKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.nClipboardFormat );
							aEntry.nValidMask |= MASK_FORMAT; // Mark ClipboardFormat as valid!
                    	}

						// Write "/IMPLEMENTATIONS/<LoaderName>/Loader/DetectService" in cache.
                    	OUString sDetectServiceKeyName = sBaseKeyName;
                    	sDetectServiceKeyName += REGISTRYKEY_LOADER_DETECTSERVICE;

                    	xKey = xRootKey->openKey( sDetectServiceKeyName );
                    	if ( xKey.is() == sal_True )
                    	{
							impl_setKeyValueOnCacheEntry( xKey, aEntry.sDetectService );
							aEntry.nValidMask |= MASK_DETECTSERVICE; // Mark DetectService as valid!
                    	}

						// Insert entry in cache.
	                    m_aRegistryCache.push_back( aEntry );
                	}
            	}
        	}
    	}

		/*-*******************************************************************************************************/
		// Hack for SFX
		// Load temp. frame loader information from "install.ini".
		impl_loadInstallINI();
		/*-*******************************************************************************************************/
	}

	// If something wrong ... delete cache and set state to INVALID!
	// (INVALID => cache is empty)
	}
	catch( Exception& )
	{
		clear();
	}

	// In special debug mode only(!) ... log current cache content in file.
	#ifdef	ENABLE_REGISTRYCACHEDEBUG
	impldbg_showCacheContent();
	#endif	// #ifdef ENABLE_REGISTRYCACHEDEBUG
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
void RegistryCache::clear()
{
	// Clear entrylist.
	m_aRegistryCache.clear();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getEntryCount() const
{
	// Return the current size of entrylist.
	return (sal_uInt32)m_aRegistryCache.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
const tIMPLRegistryEntry_stl& RegistryCache::operator[]( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "stl RegistryCache::operator[]\nThis entry not exist!\n" )
	// Return searched cache element to caller.
	return m_aRegistryCache[nEntryNumber];
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
tIMPLRegistryEntry_uno RegistryCache::getEntryAsUNOVersion( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "uno RegistryCache::operator[]\nThis entry not exist!\n" )
	// Build a uno-structure from internal representation of specified cache entry.
	// (convert stl to uno)
	tIMPLRegistryEntry_uno aReturnStructure;

	aReturnStructure.sImplementationName= getImplementationName	( nEntryNumber );
	aReturnStructure.seqPatternList		= getPatternList		( nEntryNumber );
	aReturnStructure.seqExtensionList	= getExtensionList		( nEntryNumber );
	aReturnStructure.sMimeType			= getMimeType			( nEntryNumber );
	aReturnStructure.nFlags				= getFlags				( nEntryNumber );
	aReturnStructure.sFilterName		= getFilterName			( nEntryNumber );
	aReturnStructure.nClipboardFormat	= getClipboardFormat	( nEntryNumber );
	aReturnStructure.nValidMask			= getValidMask			( nEntryNumber );

	return aReturnStructure;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
Sequence< PropertyValue > RegistryCache::getEntryAsPropertySequence( sal_uInt32 nEntryNumber ) const
{
	PropertyValue*	pBuffer = new PropertyValue[MAXCOUNT_SUPPORTED_REGISTRYPROPERTIES];
	sal_uInt32		nCount	= 0;

	// Attention: The implmentation name is not returned as property. Ouer FrameLoaderFactory will create these service!
	// Nobody must know this value ... it's better for all.

	// Copy Pattern to list, if it exist.
	if ( m_aRegistryCache[nEntryNumber].nValidMask & MASK_PATTERN )
	{
		pBuffer[nCount].Name	=	ARGUMENT_PATTERN				;
		pBuffer[nCount].Value	<<=	getPatternList( nEntryNumber )	;
		++nCount;
	}

	// Copy Extension to list, if it exist.
	if ( m_aRegistryCache[nEntryNumber].nValidMask & MASK_EXTENSION )
	{
		pBuffer[nCount].Name	=	ARGUMENT_EXTENSION				;
		pBuffer[nCount].Value	<<=	getExtensionList( nEntryNumber );
		++nCount;
	}

	// Copy MimeType to list, if it exist.
	if ( m_aRegistryCache[nEntryNumber].nValidMask & MASK_MIMETYPE )
	{
		pBuffer[nCount].Name	=	ARGUMENT_MIMETYPE			;
		pBuffer[nCount].Value	<<=	getMimeType( nEntryNumber )	;
		++nCount;
	}

	// Copy Flags to list, if it exist.
	if ( m_aRegistryCache[nEntryNumber].nValidMask & MASK_FLAGS )
	{
		pBuffer[nCount].Name	=	ARGUMENT_FLAGS				;
		pBuffer[nCount].Value	<<=	getFlags( nEntryNumber )	;
		++nCount;
	}

	// Copy FilterName to list, if it exist.
	if ( m_aRegistryCache[nEntryNumber].nValidMask & MASK_FILTERNAME )
	{
		pBuffer[nCount].Name	=	ARGUMENT_FILTERNAME				;
		pBuffer[nCount].Value	<<=	getFilterName( nEntryNumber )	;
		++nCount;
	}

	// Copy Format to list, if it exist.
	if ( m_aRegistryCache[nEntryNumber].nValidMask & MASK_FORMAT )
	{
		pBuffer[nCount].Name	=	ARGUMENT_FORMAT						;
		pBuffer[nCount].Value	<<=	getClipboardFormat( nEntryNumber )	;
		++nCount;
	}

	// Build sequence from buffer.
	Sequence< PropertyValue > seqReturn( pBuffer, nCount );
	// Free buffer!
	delete pBuffer;
	// Retrun result of this operation.
	return seqReturn;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getImplementationName( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getLoaderName()\nThis entry not exist!\n" )

	// Return value of specified item.
	// This value must exist if index is right!
	return m_aRegistryCache[nEntryNumber].sImplementationName;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
Sequence< OUString > RegistryCache::getPatternList( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getPatternList()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_PATTERN)==INVALID), "RegistryCache::getExtensionList()\nPatternlist not exist for specified entry!\n" )

	// Copy value from vector to sequence!
	sal_uInt32 nCount = getPatternCount( nEntryNumber );
	Sequence< OUString > seqReturnList( nCount );
	for ( sal_uInt32 nPosition=0; nPosition<nCount; ++nPosition )
	{
		seqReturnList[nPosition] = getPattern( nEntryNumber, nPosition );
	}

	// Return list as result.
	return seqReturnList;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
Sequence< OUString > RegistryCache::getExtensionList( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getExtensionList()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_EXTENSION)==INVALID), "RegistryCache::getExtensionList()\nExtensionlist not exist for specified entry!\n" )

	// Copy value from vector to sequence!
	sal_uInt32 nCount = getExtensionCount( nEntryNumber );
	Sequence< OUString > seqReturnList( nCount );
	for ( sal_uInt32 nPosition=0; nPosition<nCount; ++nPosition )
	{
		seqReturnList[nPosition] = getExtension( nEntryNumber, nPosition );
	}

	// Return list as result.
	return seqReturnList;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getMimeType( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getMimeType()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_MIMETYPE)==INVALID), "RegistryCache::getMimeType()\nMimeType not exist for specified entry!\n" )

	// Return value of specified item.
	return m_aRegistryCache[nEntryNumber].sMimeType;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_Int32 RegistryCache::getFlags( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getFlags()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_FLAGS)==INVALID), "RegistryCache::getFlags()\nFlags not exist for specified entry!\n" )

	// Return value of specified item.
	return m_aRegistryCache[nEntryNumber].nFlags;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getFilterName( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getFilterName()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_FILTERNAME)==INVALID), "RegistryCache::getFilterName()\nFilterName not exist for specified entry!\n" )

	// Return value of specified item.
	return m_aRegistryCache[nEntryNumber].sFilterName;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_Int32 RegistryCache::getClipboardFormat( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getClipBoardFormat()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_FORMAT)==INVALID), "RegistryCache::getClipboardFormat()\nClipboardFormat not exist for specified entry!\n" )

	// Return value of specified item.
	return m_aRegistryCache[nEntryNumber].nClipboardFormat;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getDetectService( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getDetectService()\nThis entry not exist!\n" )
	// This value is optional for register!
	// Programmer must control valid-state of current entry values BEFORE he call these method!
	// We will support FAST access to entry value ... and will not control all possible casses at release!
	LOG_ASSERT( !((m_aRegistryCache[nEntryNumber].nValidMask & MASK_DETECTSERVICE)==INVALID), "RegistryCache::getDetectService()\nFlags not exist for specified entry!\n" )

	// Return value of specified item.
	return m_aRegistryCache[nEntryNumber].sDetectService;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_Int32 RegistryCache::getValidMask( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getValidMask()\nThis entry not exist!\n" )
	// Return value of specified item.
	return m_aRegistryCache[nEntryNumber].nValidMask;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::findMimeType(	const	OUSTRING&	sSearchMimeType	,
												sal_uInt32	nStartEntry		) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	LOG_ASSERT( !(nStartEntry>=m_aRegistryCache.size()), "RegistryCache::findMimeType()\nThis entry not exist!\n" )

	// Set default return value.
	sal_uInt32 nReturn = INVALID_CACHE_POSITION;
	// Step over all cache entries from given startposition (they can be 0, if default is set!) ...
	sal_uInt32 nEntryCount = ((sal_uInt32)m_aRegistryCache.size());
	for ( sal_uInt32 nPosition=nStartEntry; nPosition<nEntryCount; ++nPosition )
	{
		// ... If right mimetype (exist! and) was found ...
		if	(
				( m_aRegistryCache[nPosition].nValidMask	&	MASK_MIMETYPE	)	&&
				( m_aRegistryCache[nPosition].sMimeType		==	sSearchMimeType	)
			)
		{
			// ... set position of cache entry for return and break loop.
			nReturn = nPosition;
			break;
		}
	}

	// Return result of search.
	return nReturn;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::findFlags(	sal_Int32	nSearchFlags	,
										sal_uInt32	nStartEntry		) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	LOG_ASSERT( !(nStartEntry>=m_aRegistryCache.size()), "RegistryCache::findFlags()\nThis entry not exist!\n" )

	// Set default return value.
	sal_uInt32 nReturn = INVALID_CACHE_POSITION;
	// Step over all cache entries from given startposition (they can be 0, if default is set!) ...
	sal_uInt32 nEntryCount = ((sal_uInt32)m_aRegistryCache.size());
	for ( sal_uInt32 nPosition=nStartEntry; nPosition<nEntryCount; ++nPosition )
	{
		// ... If right flags (exist! and) was found ...

		/* ASMUSS

			Die Flags sind noch in einen Bereich MUSS und KANN unterteilt.
			Diese muessen spter durch verUNDerung genauer untersucht werden,
		 	um eine wirkliche Uebereinstimmung feststellen zu koennen!
		 */

		if	(
				( m_aRegistryCache[nPosition].nValidMask	&	MASK_FLAGS		)	&&
				( m_aRegistryCache[nPosition].nFlags		==	nSearchFlags	)
			)
		{
			// ... set position of cache entry for return and break loop.
			nReturn = nPosition;
			break;
		}
	}

	// Return result of search.
	return nReturn;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::findFilterName(	const	OUSTRING&	sSearchFilterName	,
													sal_uInt32	nStartEntry			) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	LOG_ASSERT( !(nStartEntry>=m_aRegistryCache.size()), "RegistryCache::findFilterName()\nThis entry not exist!\n" )

	// Set default return value.
	sal_uInt32 nReturn = INVALID_CACHE_POSITION;
	// Step over all cache entries from given startposition (they can be 0, if default is set!) ...
	sal_uInt32 nEntryCount = ((sal_uInt32)m_aRegistryCache.size());
	for ( sal_uInt32 nPosition=nStartEntry; nPosition<nEntryCount; ++nPosition )
	{
		// ... If right (exist! and) filtername was found ...
		if	(
				( m_aRegistryCache[nPosition].nValidMask	&	MASK_FILTERNAME		)	&&
				( m_aRegistryCache[nPosition].sFilterName	==	sSearchFilterName	)
			)
		{
			// ... set position of cache entry for return and break loop.
			nReturn = nPosition;
			break;
		}
	}

	// Return result of search.
	return nReturn;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::findClipboardFormat(	sal_Int32	nSearchClipboardFormat	,
												sal_uInt32	nStartEntry				) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	LOG_ASSERT( !(nStartEntry>=m_aRegistryCache.size()), "RegistryCache::findMimeType()\nThis entry not exist!\n" )

	// Set default return value.
	sal_uInt32 nReturn = INVALID_CACHE_POSITION;
	// Step over all cache entries from given startposition (they can be 0, if default is set!) ...
	sal_uInt32 nEntryCount = ((sal_uInt32)m_aRegistryCache.size());
	for ( sal_uInt32 nPosition=nStartEntry; nPosition<nEntryCount; ++nPosition )
	{
		// ... If right (exist! and) clipboardformat was found ...
		if	(
				( m_aRegistryCache[nPosition].nValidMask		&	MASK_FORMAT				)	&&
				( m_aRegistryCache[nPosition].nClipboardFormat	==	nSearchClipboardFormat	)
			)
		{
			// ... set position of cache entry for return and break loop.
			nReturn = nPosition;
			break;
		}
	}

	// Return result of search.
	return nReturn;
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getPatternCount( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getPatternCount()\nThis entry not exist!\n" )
	// Return size of pattern list.
	return (sal_uInt32)m_aRegistryCache[nEntryNumber].aPatternList.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
sal_uInt32 RegistryCache::getExtensionCount( sal_uInt32 nEntryNumber ) const
{
	// Safe impossible cases
	// We can work with valid parameters only!
	// If user of these method called getEntryCount() before and step from 0 to COUNT-1(!), no error cann occurre!
	// Otherwise ... HE HAS A PROBLEM ...
	LOG_ASSERT( !(nEntryNumber>=m_aRegistryCache.size()), "RegistryCache::getExtensionCount()\nThis entry not exist!\n" )
	// Return size of extension list.
	return (sal_uInt32)m_aRegistryCache[nEntryNumber].aExtensionList.size();
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getPattern(	sal_uInt32	nEntryNumber	,
		 							sal_uInt32	nPatternNumber	) const
{
	// Safe impossible cases
	LOG_ASSERT( !(nEntryNumber  >=m_aRegistryCache.size()							 ), "RegistryCache::getPattern()\nThis entry not exist!\n"	 )
	LOG_ASSERT( !(nPatternNumber>=m_aRegistryCache[nEntryNumber].aPatternList.size() ), "RegistryCache::getPattern()\nThis pattern not exist!\n" )
	// Return value of pattern.
	return m_aRegistryCache[nEntryNumber].aPatternList[nPatternNumber];
}

//*****************************************************************************************************************
//	public method
//*****************************************************************************************************************
OUString RegistryCache::getExtension(	sal_uInt32	nEntryNumber	,
		 								sal_uInt32	nExtensionNumber) const
{
	// Safe impossible cases
	LOG_ASSERT( !(nEntryNumber    >=m_aRegistryCache.size()							    ), "RegistryCache::getExtension()\nThis entry not exist!\n"	    )
	LOG_ASSERT( !(nExtensionNumber>=m_aRegistryCache[nEntryNumber].aExtensionList.size()), "RegistryCache::getExtension()\nThis extension not exist!\n" )
	// Return value of extension.
	return m_aRegistryCache[nEntryNumber].aExtensionList[nExtensionNumber];
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_setKeyValueOnCacheEntry(	const	Reference< XRegistryKey >&	xKey	,
															tIMPLStringList&			aItem	) throw( Exception )
{
	// Get the right type of the keyvalue, to use right read-method!
    RegistryValueType aType = xKey->getValueType();
    switch ( aType )
    {
        case RegistryValueType_ASCII		:	{
													// Get the value as Ascii from registry and append it on right list.
													aItem.push_back( xKey->getAsciiValue() );
												}
                               					break;
        case RegistryValueType_STRING		:	{
													// Get the value as String from registry and append it on right list.
													aItem.push_back( xKey->getStringValue() );
												}
                                				break;
        case RegistryValueType_ASCIILIST	:	{
													// Get the value's as ASCII-list from registry and append it on right list.
													Sequence< OUString >	seqValues	= xKey->getAsciiListValue();
													OUString*				pArray		= seqValues.getArray();
        											sal_Int32	   			nLength		= seqValues.getLength();

        											for ( sal_Int32 nCounter=0; nCounter<nLength; ++nCounter )
													{
            											aItem.push_back( pArray[nCounter] );
													}
												}
                               					break;
        case RegistryValueType_STRINGLIST	:	{
													// Get the value's as String-list from registry and append it on right list.
													Sequence< OUString >	seqValues	= xKey->getAsciiListValue();
        											sal_Int32	   			nLength		= seqValues.getLength();

        											for ( sal_Int32 nCounter=0; nCounter<nLength; ++nCounter )
													{
            											aItem.push_back( seqValues[nCounter] );
													}
												}
                               					break;
    }
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_setKeyValueOnCacheEntry(const	Reference< XRegistryKey >&	xKey	,
					 									OUString&					sItem	) throw ( Exception )
{
	// Get the right type of the keyvalue, to use right read-method!
    RegistryValueType aType = xKey->getValueType();
    switch ( aType )
    {
        case RegistryValueType_ASCII        :   // Get the value from registry and set it on item.
                                                sItem = xKey->getAsciiValue();
                                				break;
        case RegistryValueType_STRING		:	// Get the value from registry and set it on item.
												sItem = xKey->getStringValue();
                                				break;
        default								:   // Unknown or wrong type of registrykey detected!
												LOG_ASSERT( sal_False, "RegistryCache::impl_setKeyValueOnCacheEntry( OUString )\nWrong type of registrykey detected!\n" )
                               					break;
    }
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_setKeyValueOnCacheEntry(const	Reference< XRegistryKey >&	xKey	,
					 									sal_Int32&					nItem	) throw ( Exception )
{
	// Get the right type of the keyvalue, to use right read-method!
    RegistryValueType aType = xKey->getValueType();
    switch ( aType )
    {
        case RegistryValueType_LONG		:	// Get the value from registry and set it on item.
											nItem = xKey->getLongValue();
                               				break;
        default							:   // Unknown or wrong type of registrykey detected!
			   								LOG_ASSERT( sal_False, "RegistryCache::impl_setKeyValueOnCacheEntry( sal_Int32 )\nWrong type of registrykey detected!\n" )
											break;
    }
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_loadInstallINI()
{
	// Get name of application to get path of it.
	OStartupInfo	aInfo				;
	OUString		sApplicationName	;
	if ( aInfo.getExecutableFile( sApplicationName ) == OStartupInfo::E_None )
	{
		// Build full qualified URL for ini name.
		// "Install.ini" must exist in the same directory like the application which use these class as object.
		// And we need a URL to open a profile at INIManager services.
		OUString sINIUNC = sApplicationName.copy( 0, sApplicationName.lastIndexOf( '/' ) );
		sINIUNC         += OUString(RTL_CONSTASCII_USTRINGPARAM("/install.ini"));
		OUString sINIURL;
		FileBase::getFileURLFromNormalizedPath( sINIUNC, sINIURL );
		// Instanciate INIManager service for working with install.ini.
		Reference< XSimpleRegistry > xInstallINI( ::utl::getProcessServiceFactory()->createInstance( SERVICENAME_INIMANAGER ), UNO_QUERY );
		LOG_ASSERT( !(xInstallINI.is()==sal_False), "RegistryCache::impl_loadInstallINI()\nServicename of INIManager is unknown!\n" )
		if ( xInstallINI.is()==sal_True )
		{
			// We must protect us against exceptions ...
			try
			{
				// Open "install.ini" for reading only and without creation if it not already exit.
				xInstallINI->open( sINIURL, sal_True, sal_False );
				// Get full access to profile.
				Reference< XRegistryKey > xRootKey = xInstallINI->getRootKey();
				LOG_ASSERT( !(xRootKey.is()==sal_False), "RegistryCache::impl_loadInstallINI()\nCan't get root key ... I can't work so.\n")
				if ( xRootKey.is()==sal_True )
				{
					// Get names of all sections to load these sections seperatly by a helper function.
					Sequence< OUString > seqSections = xRootKey->getKeyNames();
					sal_Int32 nCount = seqSections.getLength();
					for( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
					{
						// Don't load any filter sections!
						// [Graph Filter ... ] are not supported yet.
						if	(
								( seqSections[nPosition] != OUString(RTL_CONSTASCII_USTRINGPARAM("Graphics Filters - Import")) )	&&
								( seqSections[nPosition] != OUString(RTL_CONSTASCII_USTRINGPARAM("Graphics Filters - Export")) )
							)
						{
							// Encode section name to suppress parsing for entries and values!
							OUStringBuffer sSectionName( 1024 );
							sSectionName.append( (sal_Unicode)'"'		);
							sSectionName.append( seqSections[nPosition]	);
							sSectionName.append( (sal_Unicode)'"'		);
							// Open section and load it.
							// Safe impossible cases. Open can't failed - we have a valid sectionname and this section MUST exist in this moment
							// or something is wrong at inimanager!
							Reference< XRegistryKey > xSection = xRootKey->openKey( sSectionName.makeStringAndClear() );
							LOG_ASSERT( !(xSection.is()==sal_False), "RegistryCache::impl_loadInstallINI()\nCan't open section with valid name ...!\n" )
							if ( xSection.is()==sal_True )
							{
								impl_loadFilterSection( xSection );
								// We must close all open keys from a registry.
								// INIManager can't destroyed if we forget this!
								xSection->closeKey();
							}
						}
					}
				}
				// Close ini file and rootkey.
				// We must close all open keys from a registry.
				// INIManager can't destroyed if we forget this!
				xRootKey->closeKey();
				xInstallINI->close();
			}
			catch( InvalidRegistryException& )
			{
				// If an exception was thrown we can clear our cache and set his state to invalid.
				// We don't know which cache items are valid ... which not ... we clear it completely.
				// User can try to fill it again ...!
				clear();
			}
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_loadFilterSection( const Reference< XRegistryKey >& xSection )
{
	// Build filter name.
	// FilterName is the part of section name until first '-' and the name of a section entry seperated by ": ".
	// zB.	section name		= "starwriter/web-Filters-49"
	//		entry name			= "StarWriter/Web 5.0 Vorlage/Template"
	// =>	base filter name	= "starwriter/web: "
	// =>	filter name			= "starwriter/web: StarWriter/Web 5.0 Vorlage/Template"

	// First build the base filter name for this section.
	OUString sBaseFilterName = xSection->getKeyName();
	sal_Int32 nMinusFound = sBaseFilterName.indexOf( '-', 0 );
	if ( nMinusFound != -1 )
	{
		sBaseFilterName = sBaseFilterName.copy( 0, nMinusFound );
	}
	sBaseFilterName += OUString(RTL_CONSTASCII_USTRINGPARAM(": "));

	// Now get names of all section entries and
	// step over all filter to cache values.
	Sequence< OUString > seqEntries = xSection->getKeyNames();
	sal_Int32 nCount = seqEntries.getLength();
	for( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
	{
		// Encode entry name to suppress parsing for values!
		OUStringBuffer sEntryName( 1024 );
		sEntryName.append( (sal_Unicode)'"'		);
		sEntryName.append( seqEntries[nPosition]);
		sEntryName.append( (sal_Unicode)'"'		);
		Reference< XRegistryKey > xFilter = xSection->openKey( sEntryName.makeStringAndClear() );
		LOG_ASSERT( !(xFilter.is()==sal_False), "RegistryCache::impl_loadInstallINI()\nCan't open filter line by using valid name ...?!\n" )
		if ( xFilter.is()==sal_True )
		{
			// Build full filter name.
			OUString sFilterName = sBaseFilterName		;
			sFilterName			+= seqEntries[nPosition];
			// Get values of current filter.
			OUString sValues = xFilter->getStringValue();
			// Parse filter values and set important things in cache.
			impl_cacheFilterValues( sFilterName, sValues );
			// We must close all open keys from a registry.
			// INIManager can't destroyed if we forget this!
			xFilter->closeKey();
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void RegistryCache::impl_cacheFilterValues(	const	OUString&	sFilterName	,
											const	OUString&	sValues		)
{
	// Create a new entry for cache.
	// We don't must use the heap! The stl-classes allocate his own memory.
    tIMPLRegistryEntry_stl aEntry;
	aEntry.nValidMask =0; // Mark all value states as INVALID!

	// Cache filter name.
	aEntry.sFilterName	 = sFilterName		;
	aEntry.nValidMask	|= MASK_FILTERNAME	;
	// Set implmentation name. It's the same value fo ALL loader from install.ini.
	// We don't set it on valid mask ... this value MUST exist!
	aEntry.sImplementationName = IMPLEMENTATIONNAME_GENERICFRAMELOADER;
    aEntry.sDetectService = SERVICENAME_GENERICFRAMELOADER;
	// Set pattern. It's the same value fo ALL loader from install.ini.
	aEntry.aPatternList.push_back( OUString(RTL_CONSTASCII_USTRINGPARAM("private:factory*")) );
	aEntry.nValidMask |= MASK_PATTERN;
    aEntry.nValidMask |= MASK_DETECTSERVICE;

	// Get all token of filter line seperated by ','.
	Sequence< OUString >	seqFilterToken	= impl_getAllToken( sValues, ',' );
	sal_Int32				nTokenCount		= seqFilterToken.getLength();
	LOG_ASSERT( !(nTokenCount<10 || nTokenCount>11), "RegistryCache::impl_loadInstallINI()\nWrong filter format detected.\n" )
	if	(
			( nTokenCount == 10 )	||
			( nTokenCount == 11 )
		)
	{
		// Format of a filter line:
		//		Token  1:	GUIName
		//		Token  2:	MimeType
		//		Token  3:	ClipboardFormat
		//		Token  4:	MacType
		//		Token  5:	OS2TypeName
		//		Token  6:	Wildcards
		//		Token  7:	DocIconID
		//		Token  8:	UserData
		//		Token  9:	Version
		//		Token 10:	Flags
		//		Token 11:	DefaultTemplate [optional]
		// We will cache Token 2,3,6,10.

		// Attention: For access in sequence we must use index based on 0!
		// => We search for token 1,2,5,9!

		// Cache token 2 ... MimeType. [index 1!]
		aEntry.sMimeType	 	 = seqFilterToken[1];
		aEntry.nValidMask		|= MASK_MIMETYPE;

		// Cache token 3 ... ClipboardFormat. [index 2!]
		OUString sFormat		 = seqFilterToken[2];
		aEntry.nClipboardFormat	 = sFormat.toInt32();
		aEntry.nValidMask		|= MASK_FORMAT;

		// Cache token 6 ... Wildcards. [index 5!]
		// Seperate it by ';'!
		Sequence< OUString >	seqWildcards	= impl_getAllToken( seqFilterToken[5], ';' );
		sal_Int32				nWildcardCount	= seqWildcards.getLength();
		for( sal_Int32 nWildcardIndex=0; nWildcardIndex<nWildcardCount; ++nWildcardIndex )
		{
			// We must delete "*."! We save extensions only no wildcard or patterns!
			OUString sWildcard = seqWildcards[nWildcardIndex];
			aEntry.aExtensionList.push_back( sWildcard.copy( 2, sWildcard.getLength()-2 ) );
			aEntry.nValidMask |= MASK_EXTENSION;
		}

		// Cache token 10 ... Flags. [index 9!]
		// Seperate it by '|'!
		aEntry.nFlags = 0;
		Sequence< OUString >	seqFlags	= impl_getAllToken( seqFilterToken[9], '|' );
		sal_Int32				nFlagCount	= seqFlags.getLength();
		for( sal_Int32 nFlagIndex=0; nFlagIndex<nFlagCount; ++nFlagIndex )
		{
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_IMPORT )
				aEntry.nFlags |= FILTERFLAG_IMPORT;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_EXPORT )
				aEntry.nFlags |= FILTERFLAG_EXPORT;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_INTERNAL )
				aEntry.nFlags |= FILTERFLAG_INTERNAL;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_TEMPLATE )
				aEntry.nFlags |= FILTERFLAG_TEMPLATE;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_TEMPLATEPATH )
				aEntry.nFlags |= FILTERFLAG_TEMPLATEPATH;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_OWN )
				aEntry.nFlags |= FILTERFLAG_OWN;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_ALIEN )
				aEntry.nFlags |= FILTERFLAG_ALIEN;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_ASYNCHRON )
				aEntry.nFlags |= FILTERFLAG_ASYNCHRON;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_READONLY )
				aEntry.nFlags |= FILTERFLAG_READONLY;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_USESOPTIONS )
				aEntry.nFlags |= FILTERFLAG_USESOPTIONS;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_NOTINSTALLED )
				aEntry.nFlags |= FILTERFLAG_NOTINSTALLED;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_CONSULTSERVICE )
				aEntry.nFlags |= FILTERFLAG_CONSULTSERVICE;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_NOTINCHOOSER )
				aEntry.nFlags |= FILTERFLAG_NOTINCHOOSER;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_NOTINFILEDIALOG )
				aEntry.nFlags |= FILTERFLAG_NOTINFILEDIALOG;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_PACKED )
				aEntry.nFlags |= FILTERFLAG_PACKED;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_SILENTEXPORT )
				aEntry.nFlags |= FILTERFLAG_SILENTEXPORT;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_BROWSERPREFERED )
				aEntry.nFlags |= FILTERFLAG_BROWSERPREFERED;
			else
			if ( seqFlags[nFlagIndex] == FILTERFLAGNAME_PREFERED )
				aEntry.nFlags |= FILTERFLAG_PREFERED;
			aEntry.nValidMask |= MASK_FLAGS;
		}

		// Insert entry in cache.
		if	(
				( aEntry.nValidMask	&	MASK_FLAGS			)	&&
				( aEntry.nFlags		&	FILTERFLAG_PREFERED	)
			)
		{
			// Insert prefered filter at first in cache ...
			// Look for right order of inserting!
			// First at position 1; second at position 2 ... and so on.
			tIMPLRegistryCacheIterator aIter;
			aIter  = m_aRegistryCache.begin();
			aIter += m_nNumberOfPreferedFilters;
			m_nNumberOfPreferedFilters++;
			m_aRegistryCache.insert( aIter, aEntry );
		}
		else
		{
			// Else; Insert all other filter at the end of cache list.
			m_aRegistryCache.push_back( aEntry );
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
Sequence< OUString > RegistryCache::impl_getAllToken(	const	OUString&		sString		,
														const	sal_Unicode&	cSeperator	)
{
	// Count token of given value line.
	// Dont forget to count element AFTER last seperator!
	sal_Int32 nStart		= 0	;
	sal_Int32 nEnd			= 0	;
	sal_Int32 nTokenCount	= 0	;
	while( nEnd != -1 )
	{
		nEnd = sString.indexOf( cSeperator, nStart );
		if	(
				( nEnd		==	-1					)	&&
				( nStart	<	sString.getLength()	)
			)
		{
			// This case handle an element after last seperator.
			++nTokenCount;
			break;
		}
		else
		if ( nEnd != -1 )
		{
			// This case handle a normal element between two seperator.
			++nTokenCount;
		}
		nStart = nEnd+1;
	}

	// Get right sequence for current token count and copy all token to it.
	Sequence< OUString >	seqToken	( nTokenCount );
	sal_Int32				nToken		= 0;
							nStart		= 0;
							nEnd		= 0;
	while( nEnd != -1 )
	{
		nEnd = sString.indexOf( cSeperator, nStart );
		if	(
				( nEnd		==	-1					)	&&
				( nStart	<	sString.getLength()	)
			)
		{
			// This case handle an element after last seperator.
			seqToken[nToken] = sString.copy( nStart, sString.getLength()-nStart );
			++nToken;
			break;
		}
		else
		if ( nEnd != -1 )
		{
			// This case handle a normal element between two seperator.
			seqToken[nToken] = sString.copy( nStart, nEnd-nStart );
			++nToken;
		}
		nStart = nEnd+1;
	}

	// Return result of this operation.
	return seqToken;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS
/* EXAMPLE!
//*****************************************************************************************************************
sal_Bool RegistryCache::impldbg_checkParameter_implsetKeyValueOnCacheEntry(	const	Reference< XRegistryKey >&	xKey	,
																		  			tIMPLStringList&			aList	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xKey		==	NULL		)	||
			( xKey.is()	==	sal_False	)	||
			( &aList	==	NULL		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}
*/
#endif	//	#ifdef ENABLE_ASSERTIONS

/*-----------------------------------------------------------------------------------------------------------------
	The follow method write important informations about current cache content to log file.
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_REGISTRYCACHEDEBUG

#define	LOGFILE_REGISTRYCACHE	"registrycache.log"

void RegistryCache::impldbg_showCacheContent()
{
	// Check state of cache.
	if( isValid()==sal_False )
	{
		WRITE_LOGFILE( LOGFILE_REGISTRYCACHE, "Cache not valid!\n" )
	}
	else
	{
		// Else; Step over all items and log his values.
		sal_uInt32		nEntryCount	= getEntryCount();
		OUStringBuffer	sBuffer		( 50000 );
		sBuffer.appendAscii	( "EntryCount = "			);
		sBuffer.append		( (sal_Int32)nEntryCount	);
		for( sal_uInt32 nEntry=0; nEntry<nEntryCount; ++nEntry )
		{
			sBuffer.appendAscii	( "\n***********************************************************************************************************************************************" );
			sBuffer.appendAscii	( "\n\tEntry ["									);
			sBuffer.append		( (sal_Int32)nEntry								);
			sBuffer.appendAscii	( "]:\n\t\tImplementationName\t=\t"				);
			sBuffer.append		( m_aRegistryCache[nEntry].sImplementationName	);
			//-----------------------------------------------------------------------------------------------------
			// Log pattern
			if( m_aRegistryCache[nEntry].nValidMask & MASK_PATTERN )
			{
				sBuffer.appendAscii( "\n\t\tPattern\t\t\t\t=\t" );
				sal_uInt32 nPatternCount = getPatternCount( nEntry );
				for( sal_uInt32 nPattern=0; nPattern<nPatternCount; ++nPattern )
				{
					sBuffer.append		( getPattern( nEntry, nPattern ));
					sBuffer.appendAscii	( " ; "							);
				}
			}
			//-----------------------------------------------------------------------------------------------------
			// Log extensions
			if( m_aRegistryCache[nEntry].nValidMask & MASK_EXTENSION )
			{
				sBuffer.appendAscii( "\n\t\tExtensions\t\t\t=\t" );
				sal_uInt32 nExtensionCount = getExtensionCount( nEntry );
				for( sal_uInt32 nExtension=0; nExtension<nExtensionCount; ++nExtension )
				{
					sBuffer.append		( getExtension( nEntry, nExtension ));
					sBuffer.appendAscii	( " ; "								);
				}
			}
			//-----------------------------------------------------------------------------------------------------
			// Log mimetype
			if( m_aRegistryCache[nEntry].nValidMask & MASK_MIMETYPE )
			{
				sBuffer.appendAscii	( "\n\t\tMimeType\t\t\t=\t"				);
				sBuffer.append		( m_aRegistryCache[nEntry].sMimeType);
			}
			//-----------------------------------------------------------------------------------------------------
			// Log flags
			if( m_aRegistryCache[nEntry].nValidMask & MASK_FLAGS )
			{
				sBuffer.appendAscii( "\n\t\tFlags\t\t\t\t=\t" );
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_IMPORT			)
					sBuffer.appendAscii( "IMPORT"				);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_EXPORT			)
					sBuffer.appendAscii( " | EXPORT"			);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_TEMPLATE		)
					sBuffer.appendAscii( " | TEMPLATE"			);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_INTERNAL		)
					sBuffer.appendAscii( " | INTERNAL"			);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_TEMPLATEPATH	)
					sBuffer.appendAscii( " | TEMPLATEPATH"		);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_OWN			)
					sBuffer.appendAscii( " | OWN"				);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_ALIEN			)
					sBuffer.appendAscii( " | ALIEN"				);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_USESOPTIONS	)
					sBuffer.appendAscii( " | USESOPTIONS"		);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_NOTINFILEDIALOG)
					sBuffer.appendAscii( " | NOTINFILEDIALOG"	);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_NOTINCHOOSER	)
					sBuffer.appendAscii( " | NOTINCHOOSER"		);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_ASYNCHRON		)
					sBuffer.appendAscii( " | ASYNCHRON"			);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_READONLY		)
					sBuffer.appendAscii( " | READONLY"			);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_NOTINSTALLED	)
					sBuffer.appendAscii( " | NOTINSTALLED"		);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_CONSULTSERVICE	)
					sBuffer.appendAscii( " | CONSULTSERVICE"	);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_PACKED			)
					sBuffer.appendAscii( " | PACKED"			);
				if( m_aRegistryCache[nEntry].nFlags & FILTERFLAG_SILENTEXPORT	)
					sBuffer.appendAscii( " | SILENTEXPORT"		);
			}
			//-----------------------------------------------------------------------------------------------------
			// Log filtername
			if( m_aRegistryCache[nEntry].nValidMask & MASK_FILTERNAME )
			{
				sBuffer.appendAscii	( "\n\t\tFilterName\t\t\t=\t\""			);
				sBuffer.append		( m_aRegistryCache[nEntry].sFilterName 	);
				sBuffer.appendAscii	( "\""									);
			}
			//-----------------------------------------------------------------------------------------------------
			// Log format
			if( m_aRegistryCache[nEntry].nValidMask & MASK_FORMAT )
			{
				sBuffer.appendAscii	( "\n\t\tFormat\t\t\t\t=\t"								);
				sBuffer.append		( (sal_Int32)(m_aRegistryCache[nEntry].nClipboardFormat));
			}
			//-----------------------------------------------------------------------------------------------------
			// Log detectservice
			if( m_aRegistryCache[nEntry].nValidMask & MASK_DETECTSERVICE )
			{
				sBuffer.appendAscii	( "\n\t\tDetectService\t\t=\t"				);
				sBuffer.append		( m_aRegistryCache[nEntry].sDetectService	);
			}

		}
		WRITE_LOGFILE( LOGFILE_REGISTRYCACHE, U2B( sBuffer.makeStringAndClear() ).getStr() );
	}
}

#endif	//	#ifdef ENABLE_REGISTRYCACHEDEBUG

}		//	namespace framework
