/*************************************************************************
 *
 *  $RCSfile: servicemanager.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_SERVICEMANAGER_HXX_
#include <classes/servicemanager.hxx>
#endif

#ifndef __FRAMEWORK_DEFINES_HXX_
#include <defines.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_LANG_XINITIALIZATION_HPP_
#include <com/sun/star/lang/XInitialization.hpp>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _UNOTOOLS_REGPATHHELPER_HXX_
#include <unotools/regpathhelper.hxx>
#endif

#ifndef _CPPUHELPER_SERVICEFACTORY_HXX_
#include <cppuhelper/servicefactory.hxx>
#endif

#ifndef _VOS_PROCESS_HXX_
#include <vos/process.hxx>
#endif

#ifndef _VOS_PROFILE_HXX_
#include <vos/profile.hxx>
#endif

#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

//_________________________________________________________________________________________________________________
//	includes of my own project
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::osl									;
using namespace ::rtl									;
using namespace	::utl									;
using namespace	::cppu									;
using namespace ::com::sun::star::uno					;
using namespace ::com::sun::star::lang					;
using namespace ::com::sun::star::registry				;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
ServiceManager::ServiceManager()
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      OMutexMember is a struct with a mutex as member. We can't use a mutex as member, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember()
		// Init member
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
ServiceManager::~ServiceManager()
{
}

//*****************************************************************************************************************
//	static method
//*****************************************************************************************************************
Reference< XMultiServiceFactory > ServiceManager::getManager()
{
	// Create one manager for all instances of this class.
	// Optimize this method
	// We initialize a static variable only one time. And we don't must use a mutex at every call!
	// For the first call; pManager is NULL - for the second call it is different from NULL and can be use directly!
	static Reference< XMultiServiceFactory >* pManager = NULL ;
	if ( pManager == NULL )
	{
		// Ready for multithreading;
		// use global mutex for first call of this method only! see before
		LOCK_GLOBALMUTEX( aGuard, "ServiceManager::getManager()" )
		// Control these pointer again ... it can be, that another instance will be faster then these!
		// (But i think, it will exist one instance of THIS class only in one application ....)
		if ( pManager == NULL )
		{
			// Create a new static Manager ...
			static Reference< XMultiServiceFactory > xManager = impl_createManager();
			// If the created manager not valid, we should not set it to pointer!
			// The algorithm with static pointer will to prevent these.
			// With an invalid reference we never try to create a new one!
			// The pointer is different from NULL and we don't initialize it again ...
			if ( xManager.is() == sal_True )
			{
				// ... and set his address to static pointer!
				pManager = &xManager;
			}
		}
	}
	// Return reference to static manager instance.
	return (*pManager);
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
Reference< XMultiServiceFactory > ServiceManager::impl_createManager()
{
	// Set default return value, if method failed.
	// Use standard but uninitialized manager from unotools!
	Reference< XMultiServiceFactory > xReturn = Reference< XMultiServiceFactory >();

	try
	{
		// Get default manager from cppu.
		Reference< XMultiServiceFactory > xManager = createServiceFactory();	// These method come from ::cppu!
		// Is there a valid manager?
		if ( xManager.is() == sal_True )
		{
			// Yes - but we must init this new instance!
			Reference< XInitialization > xInit( xManager, UNO_QUERY );

			// Safe impossible cases
			// A servicemanager MUST have an XInitialization interface!
			LOG_ASSERT( !(xInit.is()==sal_False), "ServiceManager::impl_createManager()\nServiceManager has no XInitialization interface!?" )

			// Get the name of the local and the system registry files.
			sal_Bool	bCreateNewRegistry	= sal_True;
			OUString	sNameUserRegistry	= getPathToUserRegistry();	// These helper methods come from ::utl!
			OUString	sNameSystemRegistry	= getPathToSystemRegistry();

			// If no names of registry well known, do nothing!
			if	(
					( sNameUserRegistry.getLength()		>	0	)	&&
					( sNameSystemRegistry.getLength()	>	0	)
				)
			{
				// Create an access to the registries.
				Reference< XSimpleRegistry > xUserRegistry	( xManager->createInstance( SERVICENAME_SIMPLEREGISTRY ), UNO_QUERY );
				Reference< XSimpleRegistry > xSystemRegistry( xManager->createInstance( SERVICENAME_SIMPLEREGISTRY ), UNO_QUERY );

				// Open the local registry.
				if ( xUserRegistry.is() == sal_True )
				{
					xUserRegistry->open( sNameUserRegistry, sal_False, bCreateNewRegistry );
					if ( xUserRegistry->isValid() == sal_False )
					{
						xUserRegistry->open( sNameUserRegistry, sal_True, bCreateNewRegistry );
					}
				}

				// Open the system registry.
				if ( xSystemRegistry.is() == sal_True )
				{
					xSystemRegistry->open( sNameSystemRegistry, sal_True, sal_False );
				}

				// Initalize manager.
				if	(
						( xUserRegistry.is()			==	sal_True	)	&&
						( xUserRegistry->isValid()		==	sal_True	)	&&
			 			( xSystemRegistry.is()			==	sal_True	)	&&
						( xSystemRegistry->isValid()	==	sal_True	)
					)
				{
					// First initialize a nested registry with both registryfiles.
					Sequence< Any > seqNestedRegistryParameter(2);
					seqNestedRegistryParameter[0] <<= xUserRegistry;
					seqNestedRegistryParameter[1] <<= xSystemRegistry;

                	Reference < XSimpleRegistry > xNestedRegistry( xManager->createInstanceWithArguments( SERVICENAME_NESTEDREGISTRY, seqNestedRegistryParameter ), UNO_QUERY );

					// And if we have a valid nested registry ...
					if ( xNestedRegistry.is() == sal_True )
					{
						// ... initialize the global servicemanager with these reference as parameter.
						Sequence< Any > seqInitParameter(1);
						seqInitParameter[0] <<= xNestedRegistry;

						xInit->initialize( seqInitParameter );

						// At this point we have a valid global servicemanager!
						// Set reference as result of this operation.
						xReturn = xManager;
					}
				}
			}
		}
	}
	catch(...)
	{
		// If an exception is thrown - we set the return value to NULL!
		// Its neccessary, because it can be, that we have overwrite the default from beginning of this method with an "HALF-VALID" value!
		xReturn = Reference< XMultiServiceFactory >();
	}

	// Safe impossible cases
	// Its better to warn programmer, that we don't have any valid manager.
	// The method, who has called us, will return an invalid reference forever.
	// (We initialize a static variable with the return of this method there!)
	LOG_ASSERT( !(xReturn.is() == sal_False), "ServiceManager::impl_createManager()I have no valid manager created!\nMethod getManager() will not return a valid reference.\n" )

	// Return result of this operation.
	return xReturn;
}

}	//	namespace framework
