/*************************************************************************
 *
 *  $RCSfile: odesktopdispatcher.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_HELPER_ODESKTOPDISPATCHER_HXX_
#include <helper/odesktopdispatcher.hxx>
#endif

#ifndef __FRAMEWORK_DEFINES_HXX_
#include <defines.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_FRAMESEARCHFLAG_HPP_
#include <com/sun/star/frame/FrameSearchFlag.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_XTOOLKIT_HPP_
#include <com/sun/star/awt/XToolkit.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_WINDOWATTRIBUTE_HPP_
#include <com/sun/star/awt/WindowAttribute.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_WINDOWDESCRIPTOR_HPP_
#include <com/sun/star/awt/WindowDescriptor.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_POSSIZE_HPP_
#include <com/sun/star/awt/PosSize.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_XWINDOWPEER_HPP_
#include <com/sun/star/awt/XWindowPeer.hpp>
#endif

#ifndef _COM_SUN_STAR_MOZILLA_XPLUGININSTANCE_HPP_
#include <com/sun/star/mozilla/XPluginInstance.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_UNKNOWNPROPERTYEXCEPTION_HPP_
#include <com/sun/star/beans/UnknownPropertyException.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_WRAPPEDTARGETEXCEPTION_HPP_
#include <com/sun/star/lang/WrappedTargetException.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#ifndef _COM_SUN_STAR_FRAME_XTASKSSUPPLIER_HPP_
#include <com/sun/star/frame/XTasksSupplier.hpp>
#endif

#ifndef _COM_SUN_STAR_CONTAINER_XENUMERATION_HPP_
#include <com/sun/star/container/XEnumeration.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::awt			;
using namespace ::com::sun::star::beans			;
using namespace ::com::sun::star::container		;
using namespace ::com::sun::star::frame			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::mozilla		;
using namespace ::com::sun::star::uno			;
using namespace ::com::sun::star::util			;
using namespace ::cppu							;
using namespace ::osl							;
using namespace ::rtl							;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
ODesktopDispatcher::ODesktopDispatcher(	const	Reference< XMultiServiceFactory >&	xFactory	,
										const	Reference< XDesktop >&				xOwner		,
												Mutex&								aMutex		)
		//	Init baseclasses first
		:	OWeakObject				(			)
		// Init member
		,	m_aMutex				( aMutex	)	// We share mutex with ouer owner-class!
		,	m_xOwnerWeak			( xOwner	)
		,	m_xFactory				( xFactory	)
		,	m_aTaskList				(			)
		,	m_aListenerContainer	( aMutex	)
		,	m_aTaskCreator			( xFactory	)
{
	// Safe impossible cases
	// We need valid informations about ouer ownerfor work.
	LOG_ASSERT( impldbg_checkParameter_ODesktopDispatcher( xFactory, xOwner, aMutex ), "ODesktopDispatcher::ODesktopDispatcher()\nInvalid parameter detected!\n" )
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
ODesktopDispatcher::~ODesktopDispatcher()
{
	// Reset instance, free memory ....
	impl_resetObject();
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider
//*****************************************************************************************************************

DEFINE_XINTERFACE_5		(	ODesktopDispatcher									,
							OWeakObject					  						,
							DIRECT_INTERFACE(XTypeProvider						),
							DIRECT_INTERFACE(XDispatch							),
							DIRECT_INTERFACE(XLoadEventListener					),
							DIRECT_INTERFACE(XStatusListener					),
							DERIVED_INTERFACE(XEventListener, XStatusListener	)
						)

DEFINE_XTYPEPROVIDER_5	(	ODesktopDispatcher	,
							XTypeProvider		,
							XDispatch			,
							XLoadEventListener	,
							XEventListener		,
							XStatusListener
						)

// Attention: If you have enabled any testmode different from TEST_NOTHING => you have declared XDebugging-interface automaticly!
// Bhere is no macro to define and implement the right methods. You are the only one, who know - how you will use this mechanism.
// It exist a macro to switch on or off your implementation only.
// SERVICEDEBUG_ENABLED must be defined, to activate your code!

//#ifdef SERVICEDEBUG_ENABLED

	// Debug mechanism for services is not used in this implementation.
	// This define will expand to nothing if SERVICEDEBUG_ENABLED unknown!
	// But if you will implement your own code - activate this define before ... and deactivate follow line.
//	DEFINE_EMPTY_XSPECIALDEBUGINTERFACE( ODesktopDispatcher )

//#endif // #ifdef SERVICEDEBUG_ENABLED

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::dispatch(	const	URL&						aURL			,
											const	Sequence< PropertyValue >&	seqProperties	) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::dispatch()" )
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_dispatch( aURL, seqProperties ), "ODesktopDispatcher::dispatch()\nInvalid parameter detected.\n" )

	Reference< XPropertySet > xDesktop( m_xOwnerWeak.get(), UNO_QUERY );
	if ( xDesktop.is() == sal_True )
	{
		// Ask desktop for plugin state and look for right exception handling.
		// If any exception is detected work as default with system tasks only!
		sal_Bool bPlugged;
		try
		{
			xDesktop->getPropertyValue( DECLARE_ASCII("IsPlugged") ) >>= bPlugged;
		}
		catch( UnknownPropertyException& )
		{
			LOG_ASSERT( sal_False, "UnknownPropertyException detected!\n" )
			bPlugged = sal_False;
		}
		catch( WrappedTargetException& )
		{
			LOG_ASSERT( sal_False, "WrappedTargetException detected!\n" )
			bPlugged = sal_False;
		}

		// Depend from plugin state dispatch URL to browser or create a new system task.
		// Attention: Follow impl methods try to load the URL and can call us back in event handling!
		// We must release our current mutex! (These impl methods use a mutex by himself!)
		// Don't do anything after this calls or try to get a new guard!
		UNLOCK_MUTEX( aGuard, "ODesktopDispatcher::dispatch()" )
		if( bPlugged == sal_True )
		{
			impl_dispatchBrowserTask( aURL, seqProperties );
		}
		else
		{
			impl_dispatchSystemTask( aURL, seqProperties );
		}
	}
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::addStatusListener(	const	Reference< XStatusListener >&	xControl,
														const	URL&							aURL	) throw( RuntimeException )
{
	LOG_ASSERT( sal_False, "ODesktopDispatcher::addStatusListener()\nNot implemented yet!\n" )
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_addStatusListener( xControl, aURL ), "ODesktopDispatcher::addStatusListener()\nInvalid parameter detected.\n" )
	// Add listener to container.
	m_aListenerContainer.addInterface( aURL.Complete, xControl );
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::removeStatusListener(	const	Reference< XStatusListener >&	xControl,
														const	URL&							aURL	) throw( RuntimeException )
{
	LOG_ASSERT( sal_False, "ODesktopDispatcher::removeStatusListener()\nNot implemented yet!\n" )
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_removeStatusListener( xControl, aURL ), "ODesktopDispatcher::removeStatusListener()\nInvalid parameter detected.\n" )
	// Add listener to container.
	m_aListenerContainer.removeInterface( aURL.Complete, xControl );
}

//*****************************************************************************************************************
//	 XLoadEventListener
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::loadFinished( const Reference< XFrameLoader >& xLoader ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::loadFinished()" )
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_loadFinished( xLoader ), "ODesktopDispatcher::loadFinished()\nInvalid parameter detected.\n" )

	// Search for right loader in our list.
	tIMPL_TaskList::iterator aItem = impl_searchLoader( xLoader );
	if( aItem != m_aTaskList.end() )
	{
		// Send mapped load event as status event to all listener.
		impl_sendStatusEvent( aItem->aURL.Complete, sal_True );
		// If we have found the right one ...
		// ... close this loader, forget corresponding task and
		// forget this information struct. We don't need it again.
		aItem->xLoader	= Reference< XFrameLoader >()	;
		aItem->xTask	= Reference< XTask >()			;
		m_aTaskList.erase( aItem );
	}
/*
	if ( m_xLoader == xLoader )
	{
		// Release loader. We don't need it any longer.
		m_xLoader = Reference< XFrameLoader >();
		// We need some informations from the frame, who is the owner of this instance!
		// Try to make the weakreference hard! If this failed ... do nothing.
		Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
		if ( xOwnerFrame.is() == sal_True )
		{
			Reference< XController >	xController			= xOwnerFrame->getController()		;
			Reference< XWindow >		xComponentWindow	= xOwnerFrame->getComponentWindow()	;
			if	(
					( xComponentWindow.is()	==	sal_False	)	&&
					( xController.is()		==	sal_False	)
				)
			{
			}
		}
	}*/
}

//*****************************************************************************************************************
//	 XLoadEventListener
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::loadCancelled( const Reference< XFrameLoader >& xLoader ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::loadCancelled()" )
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_loadCancelled( xLoader ), "ODesktopDispatcher::loadCancelled()\nInvalid parameter detected.\n" )

	// Search for right loader in our list.
	tIMPL_TaskList::iterator aItem = impl_searchLoader( xLoader );
	if( aItem != m_aTaskList.end() )
	{
		// Send mapped load event as status event to all listener.
		impl_sendStatusEvent( aItem->aURL.Complete, sal_False );
		// If we have found the right one ...
		// ... close this loader, CLOSE corresponding task(!) and
		// forget this information struct. We don't need it again.
		aItem->xLoader	= Reference< XFrameLoader >()	;
		aItem->xTask->close();
		aItem->xTask	= Reference< XTask >()			;
		m_aTaskList.erase( aItem );
	}
}

//*****************************************************************************************************************
//	 XEventListener
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::disposing( const EventObject& aEvent ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::disposing()" )
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_disposing( aEvent ), "ODesktopDispatcher::disposing()\nInvalid parameter detected.\n" )

	impl_resetObject();
}

//*****************************************************************************************************************
//	 XStatusListener
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::statusChanged( const FeatureStateEvent& aEvent ) throw( RuntimeException )
{
	/*

		Kann momentan noch nicht richtig untersttzt werden da zu diesem PluginFrame auch kein Loader existiert ...
		Macht aber erstmal auch nichts weiter da diese Methode sowieso nie gerufen wird, da das Notify des Browser
		nicht richtig funktioniert ...
		Also warten auf bessere Zeiten.

	*/
	LOG_ASSERT( sal_False, "ODesktopDispatcher::statusChanged()\nCant be used correctly yet ... but how call this method?!\n" )

	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::statusChanged()" )
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_statusChanged( aEvent ), "ODesktopDispatcher::statusChanged()\nInvalid parameter detected.\n" )

	// If loading failed delete obsolete plugin frame!
	if( aEvent.IsEnabled == sal_False )
	{
		// ...
	}

	// Forward event to our listener!!!
	impl_sendStatusEvent( aEvent.FeatureURL.Complete, aEvent.IsEnabled );
}

//*****************************************************************************************************************
//	proteced method
//*****************************************************************************************************************
void ODesktopDispatcher::impl_resetObject()
{
	// Attention:
	// Write this for multiple calls - NOT AT THE SAME TIME - but for more then one call again)!
	// It exist two ways to call this method. From destructor and from disposing().
	// I can't say, which one is the first. Normaly the disposing-call - but other way ....

	// First clear list of created tasks.
	tIMPL_TaskList::iterator aIterator = m_aTaskList.begin();
	while( aIterator != m_aTaskList.end() )
	{
		aIterator->xLoader	= Reference< XFrameLoader >()	;
		aIterator->xTask	= Reference< XTask >()			;
	}
	m_aTaskList.clear();

	// Forget our factory.
	m_xFactory = Reference< XMultiServiceFactory >();
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::impl_dispatchSystemTask(	const	URL&						aURL			,
								  							const	Sequence< PropertyValue >&	seqProperties	) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::impl_dispatchSystemTask()" )
	// Create a new list item for loader-task-binding.
	// We save this information in a member list to distinguish between different loaders in loadFinished() or loadCancelled()!
	tIMPL_LoaderTaskBinding aTaskListItem;

	// We need some informations from the desktop, who is the owner of this instance!
	// Try to make the weakreference hard! If this failed ... do nothing.
	Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
	if ( xOwnerFrame.is() == sal_True )
	{
		// Search loader for given parameter at a frame loader factory.
        Reference< XMultiServiceFactory > xFrameLoaderFactory( m_xFactory->createInstance( SERVICENAME_FRAMELOADERFACTORY ), UNO_QUERY );
		if( xFrameLoaderFactory.is() == sal_True )
		{
			// Convert parameter to right format.
			sal_Int32		nCount		= seqProperties.getLength();
			Sequence< Any >	seqArguments( nCount );
			for ( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
			{
        		seqArguments[nPosition] <<= seqProperties[nPosition];
			}
			// Try to find the right loader.
        	aTaskListItem.xLoader = Reference< XFrameLoader >( xFrameLoaderFactory->createInstanceWithArguments( aURL.Complete, seqArguments ), UNO_QUERY );
		}

		// If we have a valid loader ...
		if( aTaskListItem.xLoader.is() == sal_True )
		{
 			// ... create a new task to load document in it.
			aTaskListItem.xTask = Reference< XTask >( m_aTaskCreator.createNewSystemTask( OUString() ), UNO_QUERY );
		}

		// If we have a valid task ...
		if( aTaskListItem.xTask.is() == sal_True )
		{
			// ... safe given URL, insert item in list and start loading.
			aTaskListItem.aURL = aURL;
			m_aTaskList.push_back( aTaskListItem );

			/* Hack for some loader! */
			// Some loader call loadFinished() at us without to hold itself ... and we release ouer loader reference!!!
			// We must hold these loader until this method end.
			Reference< XFrameLoader > xHoldReference( aTaskListItem.xLoader );
			/* Hack for some loader! */
			// Clear the guard before you call load().
			// The loader call us back in loadFinished() or loadCancelled()!
			// Attention: Set task as destination of loading but us as listener for load events!
			UNLOCK_MUTEX( aGuard, "ODesktopDispatcher::impl_dispatchSystemTask()" )
			aTaskListItem.xLoader->load(	Reference< XFrame >( aTaskListItem.xTask, UNO_QUERY )				,
											aURL.Complete														,
											seqProperties														,
											Reference< XLoadEventListener >( (OWeakObject*)this, UNO_QUERY )	);
			// Do noything here.
			// We have no guard at this point!
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void SAL_CALL ODesktopDispatcher::impl_dispatchBrowserTask(	const	URL&						aURL			,
															const	Sequence< PropertyValue >&	seqProperties	) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODesktopDispatcher::impl_dispatchBrowserTask()" )
	// Create a new list item for loader-task-binding.
	// We save this information in a member list to distinguish between different load operations in loadFinished() or loadCancelled()!
	tIMPL_LoaderTaskBinding aTaskListItem;

	// We need some informations from the desktop, who is the owner of this instance!
	// Try to make the weakreference hard! If this failed ... do nothing.
	Reference< XTasksSupplier > xDesktop( m_xOwnerWeak.get(), UNO_QUERY );
	if ( xDesktop.is() == sal_True )
	{
		// We use any plugin frame to dispatch this url to browser!
		// Search for a plugin frame at desktop.
		Reference< XPluginInstance > xPlugIn;
		Reference< XEnumeration > xList = xDesktop->getTasks()->createEnumeration();
		while	(
					(	xList->hasMoreElements()	==	sal_True	)	&&
					(	xPlugIn.is()				==	sal_False	)
				)
		{
			Reference< XTask > xTask;
			xList->nextElement() >>= xTask;
			xPlugIn = Reference< XPluginInstance >( xTask, UNO_QUERY );
		}
		// If we found anyone ...
		// ... we can use it to forward dispatch to browser.
		// For better performance :-) ... cast it now to right interface and test it!
		// If xPlugIn is NULL, xProvider will NULL too !
		// We doesnt need 2 compares ...
		Reference< XDispatchProvider > xProvider( xPlugIn, UNO_QUERY );
		if( xProvider.is() == sal_True )
		{
			// Get own dispatcher of pluginframe to dispatch "_blank" on it.
			Reference< XDispatch > xDispatcher = xProvider->queryDispatch( aURL, FRAMETYPE_BLANK, 0 );

//			xDispatcher->addStatusListener( this );
			// Clear the guard before you call dispatch().
			// The loader call us back in loadFinished() or loadCancelled()!
			// Attention: Set task as destination of loading but us as listener for load events!
//			UNLOCK_MUTEX( aGuard, "ODesktopDispatcher::impl_dispatchSystemTask()" )

			xDispatcher->dispatch( aURL, seqProperties );
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void ODesktopDispatcher::impl_sendStatusEvent(	const	OUString&	sURL				,
														sal_Bool	bLoadingSuccessful	)
{
	// Get listener for given URL!
	OInterfaceContainerHelper* pListenerForURL = m_aListenerContainer.getContainer( sURL );
	// Send messages to all listener.
	// Do nothing, if there no listener or "getContainer()" works not correct!
	if( pListenerForURL != NULL )
	{
		// Build event for send to listener.
		FeatureStateEvent aEvent;
    	aEvent.FeatureURL.Complete	=	sURL						;
    	aEvent.FeatureDescriptor	=	FEATUREDESCRIPTOR_LOADSTATE	;
    	aEvent.IsEnabled			=	bLoadingSuccessful			;
    	aEvent.Requery				=	sal_False					;
    	aEvent.State				=	Any()						;

		// Send message to all listener on this URL.
		OInterfaceIteratorHelper aIterator(*pListenerForURL);
		while( aIterator.hasMoreElements() )
		{
			((XStatusListener*)aIterator.next())->statusChanged( aEvent );
		}
	}
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
tIMPL_TaskList::iterator ODesktopDispatcher::impl_searchLoader( const Reference< XFrameLoader >& xLoader )
{
	// Search for right loader in list and return a iterator to his position if it exist.
	// Otherwise the return value is not valid for list!
	tIMPL_TaskList::iterator aSearchIterator	=	m_aTaskList.begin();
	tIMPL_TaskList::iterator aFoundItem			=	m_aTaskList.end();
	while( aSearchIterator != m_aTaskList.end() )
	{
		if( aSearchIterator->xLoader == xLoader )
		{
			aFoundItem = aSearchIterator;
			break;
		}
		++aSearchIterator;
	}
	return aFoundItem;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool ODesktopDispatcher::impldbg_checkParameter_ODesktopDispatcher(	const	Reference< XMultiServiceFactory >&	xFactory	,
																		const	Reference< XDesktop >&				xOwner		,
																				Mutex&								aMutex		)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xFactory		==	NULL		)	||
			( &xOwner		==	NULL		)	||
			( &aMutex		==	NULL		)	||
			( xFactory.is()	==	sal_False	)	||
			( xOwner.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// We don't know anything about right values of aURL and seqArguments!
// Check valid references only.
sal_Bool ODesktopDispatcher::impldbg_checkParameter_dispatch(	const	URL&					  	aURL		,
																const	Sequence< PropertyValue >&	seqArguments)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aURL			==	NULL	)	||
			( &seqArguments	==	NULL	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// We need a valid URL. What is meaning with "register for nothing"?!
// xControl must correct to - nobody can advised otherwise!
sal_Bool ODesktopDispatcher::impldbg_checkParameter_addStatusListener(	const	Reference< XStatusListener >&	xControl,
																		const	URL&							aURL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xControl					==	NULL	)	||
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// The same goes for these case! We have added valid listener for correct URL only.
// We can't remove invalid listener for nothing!
sal_Bool ODesktopDispatcher::impldbg_checkParameter_removeStatusListener(	const	Reference< XStatusListener >&	xControl,
																			const	URL&							aURL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xControl					==	NULL	)	||
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODesktopDispatcher::impldbg_checkParameter_loadFinished( const Reference< XFrameLoader >& xLoader )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xLoader		==	NULL		)	||
			( xLoader.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODesktopDispatcher::impldbg_checkParameter_loadCancelled( const Reference< XFrameLoader >& xLoader )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xLoader		==	NULL		)	||
			( xLoader.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODesktopDispatcher::impldbg_checkParameter_disposing( const EventObject& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aEvent				==	NULL		)	||
			( aEvent.Source.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODesktopDispatcher::impldbg_checkParameter_statusChanged( const FeatureStateEvent& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aEvent				==	NULL		)	||
			( aEvent.Source.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
