/*************************************************************************
 *
 *  $RCSfile: odispatchprovider.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: mba $ $Date: 2000/09/27 13:25:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_HELPER_ODISPATCHPROVIDER_HXX_
#include <helper/odispatchprovider.hxx>
#endif

#ifndef __FRAMEWORK_DEFINES_HXX_
#include <defines.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_FRAMESEARCHFLAG_HPP_
#include <com/sun/star/frame/FrameSearchFlag.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_XWINDOW_HPP_
#include <com/sun/star/awt/XWindow.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::awt			;
using namespace ::com::sun::star::beans			;
using namespace ::com::sun::star::frame			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::uno			;
using namespace ::com::sun::star::util			;
using namespace ::cppu							;
using namespace ::osl							;
using namespace ::rtl							;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
ODispatchProvider::ODispatchProvider(	const	Reference< XMultiServiceFactory >&	xFactory	,
										const	Reference< XFrame >&				xOwner		,
												Mutex&								aMutex		)
		//	Init baseclasses first
		:	OWeakObject				(			)
		// Init member
		,	m_aMutex				( aMutex	)	// We share mutex with ouer owner-class!
		,	m_xOwnerWeak			( xOwner	)
		,	m_xFactory				( xFactory	)
		,	m_aListenerContainer	( aMutex	)
{
	// Safe impossible cases
	// We need valid informations about ouer ownerfor work.
	LOG_ASSERT( impldbg_checkParameter_ODispatchProvider( xFactory, xOwner, aMutex ), "ODispatchProvider::ODispatchProvider()\nInvalid parameter detected!\n" )
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
ODispatchProvider::~ODispatchProvider()
{
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider
//*****************************************************************************************************************

DEFINE_XINTERFACE_4		(	ODispatchProvider					,
							OWeakObject					  		,
							DIRECT_INTERFACE(XTypeProvider		),
							DIRECT_INTERFACE(XDispatchProvider	),
							DIRECT_INTERFACE(XDispatch			),
							DIRECT_INTERFACE(XLoadEventListener	)
						)

DEFINE_XTYPEPROVIDER_4	(	ODispatchProvider	,
							XTypeProvider		,
							XDispatchProvider	,
							XDispatch			,
							XLoadEventListener
						)

// Attention: If you have enabled any testmode different from TEST_NOTHING => you have declared XDebugging-interface automaticly!
// Bhere is no macro to define and implement the right methods. You are the only one, who know - how you will use this mechanism.
// It exist a macro to switch on or off your implementation only.
// SERVICEDEBUG_ENABLED must be defined, to activate your code!

//#ifdef SERVICEDEBUG_ENABLED

	// Debug mechanism for services is not used in this implementation.
	// This define will expand to nothing if SERVICEDEBUG_ENABLED unknown!
	// But if you will implement your own code - activate this define before ... and deactivate follow line.
//	DEFINE_EMPTY_XSPECIALDEBUGINTERFACE( ODispatchProvider )

//#endif // #ifdef SERVICEDEBUG_ENABLED

//*****************************************************************************************************************
//	XDispatchProvider
//*****************************************************************************************************************
Reference< XDispatch > SAL_CALL ODispatchProvider::queryDispatch(	const	URL&		aURL				,
			  														const	OUString&	sTargetFrameName	,
																			sal_Int32	nSearchFlags		) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::queryDispatch()" )
	// Safe impossible case.
	// Method not designed for all incoming parameter!
	LOG_ASSERT( impldbg_checkParameter_queryDispatch( aURL, sTargetFrameName, nSearchFlags ), "ODispatchProvider::queryDispatch()\nInvalid Parameter detected!\n" )

	// Set default return value if query failed.
	Reference< XDispatch > xReturn;

	// We need information from ouer ownerframe!
	// Try to get a "lock" to ouer owner-frame ... another way -> do nothing and return default value.
	Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
	if ( xOwnerFrame.is() == sal_True )
	{
		// Search for full name or right searchflag in frame hierarchy of ouer owner frame.
		Reference< XFrame > xSearchedFrame = xOwnerFrame->findFrame( sTargetFrameName, nSearchFlags );
		// If searched frame ouer owner frame ...
		// ( We can compare owner with searched frame, because xOwnerFrame is different from NULL here everyway ... )
        if( xSearchedFrame == xOwnerFrame )
		{
 			// Don't forward dispatch to it! That will call this instance again and we have a problem!!!
			// (... because, ouer owner use us as helper if no interceptor is well known ... see Frame::queryDispatch() for further informations.)
			// We ask the controller of ouer owner frame to dispatch.
			Reference< XDispatchProvider > xSearchedProvider( xOwnerFrame->getController(), UNO_QUERY );
			if( xSearchedProvider.is() == sal_True )
			{
				xReturn = xSearchedProvider->queryDispatch( aURL, OUString(), 0 );
			}
			// If controller say no ... we can return himself as result!
			if( xReturn.is() == sal_False )
			{
				xReturn = this;
			}
		}
		else
		if( xSearchedFrame.is() == sal_True )
		{
			// If search frame any other frame of our hierarchy we try to get his own dispatcher.
			Reference< XDispatchProvider > xSearchedProvider( xSearchedFrame, UNO_QUERY );
			if( xSearchedProvider.is() == sal_True )
			{
				xReturn = xSearchedProvider->queryDispatch( aURL, OUString(), 0 );
			}
		}
	}

	// Return result of this operation.
	return xReturn;
}
/*
Reference< XDispatch > SAL_CALL ODispatchProvider::queryDispatch(	const	URL&		aURL				,
			  														const	OUString&	sTargetFrameName	,
																			sal_Int32	nSearchFlags		) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::queryDispatch()" )

	// Set default return value.
	// We can handle dispatches by himself!
	Reference< XDispatch > xReturn( (OWeakObject*)this, UNO_QUERY );

	// We need information from ouer ownerframe!
	// Try to get a "lock" to ouer owner-frame ... another way -> do nothing and return default value.
	Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
	if ( xOwnerFrame.is() == sal_True )
	{
		// Any frame is searched!

        if ( !sTargetFrameName.compareToAscii("_blank") )
        {
            Reference < XDispatchProvider > xProvider( xOwnerFrame->getCreator(), UNO_QUERY );
            if ( xProvider.is() )
                return xProvider->queryDispatch( aURL, sTargetFrameName, nSearchFlags );
        }

		// Search for full name or right searchflag in frame hierarchy of ouer owner frame.
		Reference< XFrame > xSearchedFrame = xOwnerFrame->findFrame( sTargetFrameName, nSearchFlags );
		// If searched frame ouer owner frame ...
		// (Attention: xSearchedFrame==NULL => condition==FALSE; because xOwnerFrame!=NULL here!)
        if ( !xSearchedFrame.is() || xOwnerFrame == xSearchedFrame )
		{
			// Don't forward dispatch to it! That will call this instance again and we have a problem!!!
			// (... because, ouer owner use us as helper if no interceptor is well known ... see Frame::queryDispatch() for further informations.)
			// We ask the controller of ouer owner frame to dispatch.
			Reference< XDispatchProvider > xProvider( xOwnerFrame->getController(), UNO_QUERY );
			Reference< XDispatch >			xDispatcher ;
			if ( xProvider.is() == sal_True )
			{
				xDispatcher = xProvider->queryDispatch( aURL, OUString(), 0 );
			}
			if ( xDispatcher.is()==sal_True)
			{
				xReturn = xDispatcher;
			}
		}
		else
		// If another right frame was found ...
		if ( xSearchedFrame.is() == sal_True )
		{
			// ... forward dispatch to it.
			Reference< XDispatchProvider > xProvider( xSearchedFrame, UNO_QUERY );
			if ( xProvider.is() == sal_True )
			{
				xReturn = xProvider->queryDispatch( aURL, xSearchedFrame->getName(), FrameSearchFlag::SELF );
			}
		}
	}

	// Return result of this operation.
	return xReturn;
}
*/
//*****************************************************************************************************************
//	XDispatchProvider
//*****************************************************************************************************************
Sequence< Reference< XDispatch > > SAL_CALL ODispatchProvider::queryDispatches( const Sequence< DispatchDescriptor >& seqDescripts ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::queryDispatches()" )

	// Set default return value if no dispatcher will be found.
	Sequence< Reference< XDispatch > > seqDispatcher;

	// Get count of all given discriptors ...
	sal_Int32 nDescriptorCount = seqDescripts.getLength();
	if ( nDescriptorCount > 0 )
	{
		// ... to get enough memory for expected dispatcher.
		// We can't get more dispatcher as descriptors exist!
		Reference< XDispatch >*	pDispatcher	= new Reference< XDispatch >[nDescriptorCount];
		// We must count all getted dispatcher to create return sequence.
		// There can't be more but fewer dispatcher then given descriptors!
		sal_Int32				nDispatcher	= 0;
		// This is a helper variable to hold current returned dispatcher of query.
		Reference< XDispatch >	xDispatcher;

		// Step over all descriptors and try to get any dispatcher for it.
		for ( sal_Int32 nPosition=0; nPosition<nDescriptorCount; ++nPosition )
		{
			xDispatcher = queryDispatch(	seqDescripts[nPosition].FeatureURL	,
											seqDescripts[nPosition].FrameName	,
											seqDescripts[nPosition].SearchFlags	);
			// If any dispatcher for given parameters found ...
			if ( xDispatcher.is() == sal_True )
			{
				// ... safe it temporaly in "memory list" ...
				pDispatcher[nDispatcher] = xDispatcher;
				// ... and count of found dispatcher.
				++nDispatcher;
			}
		}

		// Safe impossible cases.
		// If we have more dispatcher then descriptors => we have a problem in algorithm!
		LOG_ASSERT( !(nDispatcher>nDescriptorCount), "ODispatchProvider::queryDispatches()\nAlgorithm error. There are more dispatcher as reserved fields in memory!\n" )

		// Copy "memory list" to return sequence.
		seqDispatcher = Sequence< Reference< XDispatch > >( pDispatcher, nDispatcher );
	}

	// Return result of this operation.
	return seqDispatcher;
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL ODispatchProvider::dispatch(	const	URL&						aURL			,
											const	Sequence< PropertyValue >&	seqProperties	) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::dispatch()" )

	// We need information about successfuly start of loading.
	// If no loader was found or something else is wrong we must send a status event
	sal_Bool bLoadSuccessfulStarted = sal_False;

	// We need some informations from the frame, who is the owner of this instance!
	// Try to make the weakreference hard! If this failed ... do nothing.
	Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
	if ( xOwnerFrame.is() == sal_True )
	{
		// If there already a controller or he don't will suspend fom us(!) => do nothing!
		// We can't replace it with a new one.
		Reference< XController > xController = xOwnerFrame->getController();
		if	(
				( xController.is()					==	sal_False	)	||
				( xController->suspend( sal_True )	==	sal_True	)
			)
		{
			if( aURL.Complete.getLength() < 1 )
			{
				// Special mode for empty URLs. Then our owner frame must forget his component.
				if( xController.is() == sal_True )
				{
					xOwnerFrame->setComponent( Reference< XWindow >(), Reference< XController >() );
				}
			}
			else
			{
				// Else; create, load and assign new component.

				// First clear pending loader!
				LOG_ASSERT( !(m_xLoader.is()==sal_True), "ODispatchProvider::dispatch()\nPending loader detected!\n" )
				m_xLoader = Reference< XFrameLoader >();

				// Instanciate a new loader and initialize it.
                sal_Bool bHidden = sal_False;
        		Reference< XMultiServiceFactory > xFrameLoaderFactory( m_xFactory->createInstance( SERVICENAME_FRAMELOADERFACTORY ), UNO_QUERY );
				if ( xFrameLoaderFactory.is() == sal_True )
				{
					// Copy properties in list of any.
					sal_Int32		nCount		= seqProperties.getLength();
					Sequence< Any >	seqArguments( nCount );
                    const ::com::sun::star::beans::PropertyValue* pPropsVal = seqProperties.getConstArray();
					for ( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
					{
                        const ::com::sun::star::beans::PropertyValue& rProp = pPropsVal[nPosition];
                        OUString aName = rProp.Name ;
                        if ( aName == OUString::createFromAscii( "Hidden" ) )
                        {
                            if ( rProp.Value.getValueType() == ::getBooleanCppuType() )
                                bHidden = sal_True;
                        }

        				seqArguments[nPosition] <<= seqProperties[nPosition];
					}

					// Initialize loader with parameter.
        			m_xLoader = Reference< XFrameLoader >( xFrameLoaderFactory->createInstanceWithArguments( aURL.Complete, seqArguments ), UNO_QUERY );
				}

				// It can be, that we can't create a new loader ...
				if ( m_xLoader.is() == sal_True )
				{
					// ... but at his point, we have a valid one.
					// Save URL to send status events to listener at later time.
					m_sURL = aURL.Complete;

					/* Hack for some loader! */
					// Some loader call loadFinished() at us without to hold itself ... and we release ouer loader reference!!!
					// We must hold these loader until this method end.
					Reference< XFrameLoader > xHoldReference( m_xLoader );
					/* Hack for some loader! */

					// We must set this localy state to disable sending of status message
					// to all listener that "loading was cancelled" ... !
					bLoadSuccessfulStarted = sal_True;

					// Clear the guard before you call load().
					// The loader call us back in loadFinished() or loadCancelled()!
					UNLOCK_MUTEX( aGuard, "ODispatchProvider::dispatch()" )
					m_xLoader->load(	xOwnerFrame															,
										aURL.Complete														,
										seqProperties														,
										Reference< XLoadEventListener >( (OWeakObject*)this, UNO_QUERY )	);

                    if ( !bHidden )
                        xOwnerFrame->getContainerWindow()->setVisible( sal_True );
				}
				else
				{
					// Controller was suspended but load failed ...
					// ... reactivate it!
					if ( xController.is() == sal_True )
					{
						xController->suspend( sal_False );
					}
				}
			}
		}
	}

	// Have we loading started successfuly ?
	if( bLoadSuccessfulStarted == sal_False )
	{
		// No; The we must send a "loadCancelled" to our listener.
		impl_sendStatusEvent( aURL.Complete, sal_False );
	}
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL ODispatchProvider::addStatusListener(	const	Reference< XStatusListener >&	xControl,
													const	URL&							aURL	) throw( RuntimeException )
{
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_addStatusListener( xControl, aURL ), "ODispatchProvider::addStatusListener()\nInvalid parameter detected.\n" )
	// Add listener to container.
	m_aListenerContainer.addInterface( aURL.Complete, xControl );
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL ODispatchProvider::removeStatusListener(	const	Reference< XStatusListener >&	xControl,
														const	URL&							aURL	) throw( RuntimeException )
{
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_removeStatusListener( xControl, aURL ), "ODispatchProvider::removeStatusListener()\nInvalid parameter detected.\n" )
	// Add listener to container.
	m_aListenerContainer.removeInterface( aURL.Complete, xControl );
}

//*****************************************************************************************************************
//	 XLoadEventListener
//*****************************************************************************************************************
void SAL_CALL ODispatchProvider::loadFinished( const Reference< XFrameLoader >& xLoader ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::loadFinished()" )
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_loadFinished( xLoader ), "ODispatchProvider::loadFinished()\nInvalid parameter detected.\n" )

	if ( m_xLoader == xLoader )
	{
		// Release loader. We don't need it any longer.
		m_xLoader = Reference< XFrameLoader >();
/*		// We need some informations from the frame, who is the owner of this instance!
		// Try to make the weakreference hard! If this failed ... do nothing.
		Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
		if ( xOwnerFrame.is() == sal_True )
		{
			Reference< XController >	xController			= xOwnerFrame->getController()		;
			Reference< XWindow >		xComponentWindow	= xOwnerFrame->getComponentWindow()	;
			if	(
					( xComponentWindow.is()	==	sal_False	)	&&
					( xController.is()		==	sal_False	)
				)
			{
			}
		}*/
		impl_sendStatusEvent( m_sURL, sal_True );
	}
}

//*****************************************************************************************************************
//	 XLoadEventListener
//*****************************************************************************************************************
void SAL_CALL ODispatchProvider::loadCancelled( const Reference< XFrameLoader >& xLoader ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::loadCancelled()" )
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_loadCancelled( xLoader ), "ODispatchProvider::loadCancelled()\nInvalid parameter detected.\n" )

	// Break loading only, if this the correct loader!
	if ( m_xLoader == xLoader )
	{
		// Release loader. We don't need it any longer.
		m_xLoader = Reference< XFrameLoader >();
		// We need some informations from the frame, who is the owner of this instance!
		// Try to make the weakreference hard! If this failed ... do nothing.
		Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );
		if ( xOwnerFrame.is() == sal_True )
		{
			// Controller was suspended but load failed.
			// Reactivate our current controller.
			Reference< XController > xController = xOwnerFrame->getController();
			if( xController.is() == sal_True )
			{
				xController->suspend( sal_False );
			}
		}
		impl_sendStatusEvent( m_sURL, sal_False );
	}
}

//*****************************************************************************************************************
//	 XEventListener
//*****************************************************************************************************************
void SAL_CALL ODispatchProvider::disposing( const EventObject& aEvent ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::disposing()" )
	// Safe impossible cases
	// This method is not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_disposing( aEvent ), "ODispatchProvider::disposing()\nInvalid parameter detected!\n" )
	// We have an impl method to do this.
	// We need this mechanism if programmer forget to dispose this object and we must reset us himself at destructor!
	impl_resetObject();
}

//*****************************************************************************************************************
//	proteced method
//*****************************************************************************************************************
void ODispatchProvider::impl_resetObject()
{
	// Attention:
	// Write this for multiple calls - NOT AT THE SAME TIME - but for more then one call again)!
	// It exist two ways to call this method. From destructor and from disposing().
	// I can't say, which one is the first. Normaly the disposing-call - but other way ....
	m_xFactory	= Reference< XMultiServiceFactory >();
	m_xLoader	= Reference< XFrameLoader >();
	m_sURL		= OUString();
	EventObject aEvent;
	aEvent.Source = static_cast< OWeakObject* >( this );
	m_aListenerContainer.disposeAndClear( aEvent );
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void ODispatchProvider::impl_sendStatusEvent(	const	OUString&	sURL				,
														sal_Bool	bLoadingSuccessful	)
{
	// Get listener for given URL!
	OInterfaceContainerHelper* pListenerForURL = m_aListenerContainer.getContainer( sURL );
	// Send messages to all listener.
	// Do nothing, if there no listener or "getContainer()" works not correct!
	if( pListenerForURL != NULL )
	{
		// Build event for send to listener.
		FeatureStateEvent aEvent;
    	aEvent.FeatureURL.Complete	=	sURL							;
    	aEvent.FeatureDescriptor	=	FEATUREDESCRIPTOR_LOADSTATE		;
    	aEvent.IsEnabled			=	bLoadingSuccessful				;
    	aEvent.Requery				=	sal_False						;
    	aEvent.State				=	Any()							;

		// Send message to all listener on this URL.
		OInterfaceIteratorHelper aIterator(*pListenerForURL);
		while( aIterator.hasMoreElements() )
		{
			((XStatusListener*)aIterator.next())->statusChanged( aEvent );
		}
	}
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool ODispatchProvider::impldbg_checkParameter_ODispatchProvider(	const	Reference< XMultiServiceFactory >&	xFactory	,
																		const	Reference< XFrame >&				xOwner		,
																				Mutex&								aMutex		)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xFactory		==	NULL		)	||
			( &xOwner		==	NULL		)	||
			( &aMutex		==	NULL		)	||
			( xFactory.is()	==	sal_False	)	||
			( xOwner.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// We should not control values of aURLK and sTargetFrameName. We forward these values to findFrame() of ouer owner!
// And if change his implementation we have a problem. Ouer owner must control these for himself.
// We check some general thinks ... like valid references and right enums.
sal_Bool ODispatchProvider::impldbg_checkParameter_queryDispatch(	const	URL&		aURL			,
																	const	OUString&	sTargetFrameName,
																			sal_Int32	nSearchFlags	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aURL				==	NULL	)	||
			( &sTargetFrameName	==	NULL	)	||
			(
				( 	 nSearchFlags != FrameSearchFlag::AUTO		  )	&&
				( !( nSearchFlags &  FrameSearchFlag::PARENT	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::SELF		) )	&&
				( !( nSearchFlags &  FrameSearchFlag::CHILDREN	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::CREATE	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::SIBLINGS	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::TASKS		) )	&&
				( !( nSearchFlags &  FrameSearchFlag::ALL		) )	&&
				( !( nSearchFlags &  FrameSearchFlag::GLOBAL	) )
			)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODispatchProvider::impldbg_checkParameter_queryDispatches( const Sequence< DispatchDescriptor >& seqDescripts )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if( &seqDescripts == NULL )
	{
		bOK = sal_False ;
	}
	else
	{
		// Check content of parameter!
		sal_Int32 nCount = seqDescripts.getLength();
		for( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
		{
			if( impldbg_checkParameter_queryDispatch(	seqDescripts[nPosition].FeatureURL	,
														seqDescripts[nPosition].FrameName	,
														seqDescripts[nPosition].SearchFlags	) == sal_False )
			{
				bOK = sal_False;
				break;
			}
		}
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// We don't know anything about right values of aURL and seqArguments!
// Check valid references only.
sal_Bool ODispatchProvider::impldbg_checkParameter_dispatch(	const	URL&					  	aURL		,
																const	Sequence< PropertyValue >&	seqArguments)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aURL			==	NULL	)	||
			( &seqArguments	==	NULL	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// We need a valid URL. What is meaning with "register for nothing"?!
// xControl must correct to - nobody can advised otherwise!
sal_Bool ODispatchProvider::impldbg_checkParameter_addStatusListener(	const	Reference< XStatusListener >&	xControl,
																		const	URL&							aURL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xControl					==	NULL	)	||
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// The same goes for these case! We have added valid listener for correct URL only.
// We can't remove invalid listener for nothing!
sal_Bool ODispatchProvider::impldbg_checkParameter_removeStatusListener(	const	Reference< XStatusListener >&	xControl,
																			const	URL&							aURL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xControl					==	NULL	)	||
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODispatchProvider::impldbg_checkParameter_loadFinished( const Reference< XFrameLoader >& xLoader )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xLoader		==	NULL		)	||
			( xLoader.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODispatchProvider::impldbg_checkParameter_loadCancelled( const Reference< XFrameLoader >& xLoader )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xLoader		==	NULL		)	||
			( xLoader.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool ODispatchProvider::impldbg_checkParameter_disposing( const EventObject& aEvent )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aEvent				==	NULL		)	||
			( aEvent.Source.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
