/*************************************************************************
 *
 *  $RCSfile: opluginframedispatcher.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_HELPER_OPLUGINFRAMEDISPATCHER_HXX_
#include <helper/opluginframedispatcher.hxx>
#endif

#ifndef __FRAMEWORK_DEFINES_HXX_
#include <defines.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_FRAMESEARCHFLAG_HPP_
#include <com/sun/star/frame/FrameSearchFlag.hpp>
#endif

#ifndef _COM_SUN_STAR_UTIL_XURLTRANSFORMER_HPP_
#include <com/sun/star/util/XURLTransformer.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::beans			;
using namespace ::com::sun::star::frame			;
using namespace ::com::sun::star::io			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::mozilla		;
using namespace ::com::sun::star::uno			;
using namespace ::com::sun::star::util			;
using namespace ::cppu							;
using namespace ::osl							;
using namespace ::rtl							;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
OPlugInFrameDispatcher::OPlugInFrameDispatcher(	const	Reference< XMultiServiceFactory >&		xFactory			,
														Mutex&									aMutex				,
												const	Reference< XPluginInstance >&			xOwner				,
												const	Reference< XPluginInstancePeer >&		xPlugInDLL			)
		//	Init baseclasses first
		:	OWeakObject			(					)
		// Init member
		,	m_xFactory			( xFactory			)
		,	m_aMutex			( aMutex			)	// We share mutex with ouer owner-class!
		,	m_xOwnerWeak		( xOwner			)
		,	m_xPlugInDLL		( xPlugInDLL		)
		,	m_aListenerContainer( aMutex			)
{
	// Safe impossible cases.
	// These object need valid information for working.
	LOG_ASSERT( impldbg_checkParameter_OPlugInFrameDispatcherCtor( xFactory, aMutex, xOwner, xPlugInDLL ), "OPlugInFrameDispatcher::OPlugInFrameDispatcher()\nInvalid parameter detected!\n" )
}

//*****************************************************************************************************************
//	destructor (protected!)
//*****************************************************************************************************************
OPlugInFrameDispatcher::~OPlugInFrameDispatcher()
{
}

//*****************************************************************************************************************
//	XInterface
//*****************************************************************************************************************

DEFINE_XINTERFACE_3		(	OPlugInFrameDispatcher						,
							OWeakObject				  					,
							DIRECT_INTERFACE(XDispatchProvider			),
							DIRECT_INTERFACE(XDispatch					),
							DIRECT_INTERFACE(XPluginInstanceNotifySink	)
						)

// Attention: If you have enabled any testmode different from TEST_NOTHING => you have declared XDebugging-interface automaticly!
// Bhere is no macro to define and implement the right methods. You are the only one, who know - how you will use this mechanism.
// It exist a macro to switch on or off your implementation only.
// SERVICEDEBUG_ENABLED must be defined, to activate your code!

//#ifdef SERVICEDEBUG_ENABLED

	// Debug mechanism for services is not used in this implementation.
	// This define will expand to nothing if SERVICEDEBUG_ENABLED unknown!
	// But if you will implement your own code - activate this define before ... and deactivate follow line.
//	DEFINE_EMPTY_XSPECIALDEBUGINTERFACE( OPlugInFrameDispatcher )

//#endif // #ifdef SERVICEDEBUG_ENABLED

//*****************************************************************************************************************
//	XDispatchProvider
//*****************************************************************************************************************
Reference< XDispatch > SAL_CALL OPlugInFrameDispatcher::queryDispatch(	const	URL&		aURL				,
			  															const	OUString&	sTargetFrameName	,
																				sal_Int32	nSearchFlags		) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::queryDispatch()" )
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_queryDispatch( aURL, sTargetFrameName, nSearchFlags ), "OPlugInFrameDispatcher::queryDispatch()\nInvalid parameter detected!\n" )

	// Set default return value.
	Reference< XDispatch > xReturn;
	// We need information from ouer ownerframe!
	// Try to get a "lock" to ouer owner-frame.
	Reference< XFrame > xOwnerFrame( m_xOwnerWeak.get(), UNO_QUERY );

	// We must forward _blank, _self to browser.
	// All other requests we can handle by our baseclass implementation. (Task,Frame)
	/*
		Praeprozessor Bug!
		Wenn nach CREATE ein Space steht wird versucht es durch das Define CREATE aus tools/rtti.hxx zu ersetzen
		was fehlschlaegt und die naechsten 3 Klammern ")){" unterschlaegt!
		Dann meckert der Compiler das natuerlich an ...
	 */
	if((sTargetFrameName==FRAMETYPE_BLANK)||(sTargetFrameName==FRAMETYPE_SELF)||( sTargetFrameName.getLength()<1)||(nSearchFlags&FrameSearchFlag::SELF)||(nSearchFlags&FrameSearchFlag::CREATE))
	{
        if( xOwnerFrame.is() == sal_True )
        {
			// First ask controller! If URL an internal URL like "slot:..." or "macro:..."
			// He can dispatch it by himself. Otherwise we can forward it to browser!
            Reference< XDispatchProvider > xProvider( xOwnerFrame->getController(), UNO_QUERY );
            if( xProvider.is() == sal_True )
			{
                xReturn = xProvider->queryDispatch( aURL, OUString(), 0 );
			}
        }

        if	(
				( xReturn.is()									==	sal_False	)	&&
				( aURL.Complete.compareToAscii( "macro:", 6 )	!=	STRINGEQUAL	)
			)
        {
            // Don't forget to save current target frame name!
            // We need it, if a dispatch is called at this instance!
            m_sTargetFrameName = sTargetFrameName;
            xReturn = Reference< XDispatch >( (OWeakObject*)this, UNO_QUERY );
        }
	}

	// Else; we must use normal mechanism of our baseclass!
	if	(
			( xReturn.is()		== sal_False	)	&&
			( xOwnerFrame.is()	== sal_True		)
		)
	{
		// Search for full name or right searchflag in frame hierarchy of ouer owner frame.
		Reference< XFrame > xSearchedFrame = xOwnerFrame->findFrame( sTargetFrameName, nSearchFlags );
		// If another right frame was found ...
		if( xSearchedFrame.is() == sal_True )
		{
			// ... forward dispatch to it.
			Reference< XDispatchProvider > xProvider( xSearchedFrame, UNO_QUERY );
			if ( xProvider.is() == sal_True )
			{
				xReturn = xProvider->queryDispatch( aURL, xSearchedFrame->getName(), FrameSearchFlag::SELF );
			}
		}
	}

	// Return result of this operation.
	return xReturn;
}

//*****************************************************************************************************************
//	XDispatchProvider
//*****************************************************************************************************************
Sequence< Reference< XDispatch > > SAL_CALL OPlugInFrameDispatcher::queryDispatches( const Sequence< DispatchDescriptor >& seqDescripts ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "ODispatchProvider::queryDispatches()" )
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_queryDispatches( seqDescripts ), "OPlugInFrameDispatcher::queryDispatches()\nInvalid parameter detected!\n" )

	// Set default return value if no dispatcher will be found.
	Sequence< Reference< XDispatch > > seqDispatcher;

	// Get count of all given discriptors ...
	sal_Int32 nDescriptorCount = seqDescripts.getLength();
	if ( nDescriptorCount > 0 )
	{
		// ... to get enough memory for expected dispatcher.
		// We can't get more dispatcher as descriptors exist!
		Reference< XDispatch >*	pDispatcher	= new Reference< XDispatch >[nDescriptorCount];
		// We must count all getted dispatcher to create return sequence.
		// There can't be more but fewer dispatcher then given descriptors!
		sal_Int32				nDispatcher	= 0;
		// This is a helper variable to hold current returned dispatcher of query.
		Reference< XDispatch >	xDispatcher;

		// Step over all descriptors and try to get any dispatcher for it.
		for ( sal_Int32 nPosition=0; nPosition<nDescriptorCount; ++nPosition )
		{
			xDispatcher = queryDispatch(	seqDescripts[nPosition].FeatureURL	,
											seqDescripts[nPosition].FrameName	,
											seqDescripts[nPosition].SearchFlags	);
			// If any dispatcher for given parameters found ...
			if ( xDispatcher.is() == sal_True )
			{
				// ... safe it temporaly in "memory list" ...
				pDispatcher[nDispatcher] = xDispatcher;
				// ... and count of found dispatcher.
				++nDispatcher;
			}
		}

		// Safe impossible cases.
		// If we have more dispatcher then descriptors => we have a problem in algorithm!
		LOG_ASSERT( !(nDispatcher>nDescriptorCount), "OPlugInFrameDispatcher::queryDispatch()\nAlgorithm error. There are more dispatcher as reserved fields in memory!\n" )

		// Copy "memory list" to return sequence.
		seqDispatcher = Sequence< Reference< XDispatch > >( pDispatcher, nDispatcher );
	}

	// Return result of this operation.
	return seqDispatcher;
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL OPlugInFrameDispatcher::dispatch(	const	URL&						aURL		,
												const	Sequence< PropertyValue >&	seqArguments) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "OPlugInFrameDispatcher::dispatch()" )
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_dispatch( aURL, seqArguments ), "OPlugInFrameDispatcher::dispatch()\nInvalid parameter detected!\n" )

	// Set default parameter for latter call of postURL.
	Reference< XInputStream > xInputStream;

	// extract all possible arguments for forwarding to browser!
	tIMPLExtractedArguments aArgumentField = impl_extractArguments( seqArguments );

	// We need a valid URL for forwarding to browser.
	if ( aURL.Complete.getLength() > 0 )
	{
		// Convert URL to praesentation URL!
		// It's neccessary for plugin dll and browser. He don't understand private URLs!
		Reference< XURLTransformer > xTransformer( m_xFactory->createInstance( SERVICENAME_URLTRANSFORMER ), UNO_QUERY );
		LOG_ASSERT( !(xTransformer.is()==sal_False), "OPlugInFrameDispatcher::dispatch()\nCan't find service URLTransformer!\n" )
		OUString sURL = xTransformer->getPresentation( aURL, sal_False );

		// Have we enough parameter for postURL() WITH a referrer ?
		if	(
				( aArgumentField.nValidMask & MASK_POSTDATA )	&&
				( aArgumentField.nValidMask & MASK_REFERRER )
			)
		{
			// Yes; then do it.
			// Before log special informations about URL converting and send method. (if right mode is set!)
			LOG_URLSEND( "OPluginFrameDispatcher!", "(referrer) postURL", aURL.Complete, sURL )
			m_xPlugInDLL->postURL(	sURL							,
						 			aArgumentField.xPostDataStream	,
						 			m_sTargetFrameName				,
						 			OUString()						,
						 			aArgumentField.sReferrer		,
						 			Reference< XInputStream >()		,
						 			this							);
		}
		else
		// Have we enough parameter for postURL() WITHOUT a referrer ?
		if	(
				( aArgumentField.nValidMask & MASK_POSTDATA )	&&
				( aArgumentField.nValidMask & MASK_REFERRER )
			)
		{
			// Yes; then do it.
			// Before log special informations about URL converting and send method. (if right mode is set!)
			LOG_URLSEND( "OPluginFrameDispatcher!", "(without referrer) postURL", aURL.Complete, sURL )
			m_xPlugInDLL->postURL(	sURL							,
									aArgumentField.xPostDataStream	,
									m_sTargetFrameName				,
									OUString()						,
									OUString()						,
									Reference< XInputStream >()		,
									this							);
		}
		else
		// Have we enough parameter for getURL() WITH a referrer ?
		if ( aArgumentField.nValidMask & MASK_REFERRER )
		{
			// Yes; then do it.
			// Before log special informations about URL converting and send method. (if right mode is set!)
			LOG_URLSEND( "OPluginFrameDispatcher!", "(referrer) getURL", aURL.Complete, sURL )
			m_xPlugInDLL->getURL(	sURL						,
									m_sTargetFrameName			,
									OUString()					,
									aArgumentField.sReferrer	,
									this						);
		}
		else
		// We have enough parameter for getURL() WITHOUT a referrer !
		{
			// Yes; then do it.
			// Before log special informations about URL converting and send method. (if right mode is set!)
			LOG_URLSEND( "OPluginFrameDispatcher!", "(without referrer) getURL", aURL.Complete, sURL )
			m_xPlugInDLL->getURL(	sURL				,
									m_sTargetFrameName	,
									OUString()			,
									OUString()			,
									this				);
		}
	}
}

//*****************************************************************************************************************
//	XPluginInstanceNotifySink
//*****************************************************************************************************************
void SAL_CALL OPlugInFrameDispatcher::notifyURL( const OUString& sURL ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "OPlugInFrameDispatcher::notifyURL()" )
	// Safe impossible cases
	// Method is not defined for all incoming parameters.
	LOG_ASSERT( impldbg_checkParameter_notifyURL( sURL ), "OPlugInFrameDispatcher::notifyURL()\nInvalid parameter detected.\n" )
	// Send status event to all listener for this URL!
	impl_sendStatusEvent( sURL, sal_True );
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL OPlugInFrameDispatcher::addStatusListener(	const	Reference< XStatusListener >&	xControl,
															const	URL&							aURL	) throw( RuntimeException )
{
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_addStatusListener( xControl, aURL ), "OPlugInFrameDispatcher::addStatusListener()\nInvalid parameter detected.\n" )
	// Add listener to container.
	m_aListenerContainer.addInterface( aURL.Complete, xControl );
}

//*****************************************************************************************************************
//	XDispatch
//*****************************************************************************************************************
void SAL_CALL OPlugInFrameDispatcher::removeStatusListener(	const	Reference< XStatusListener >&	xControl,
															const	URL&							aURL	) throw( RuntimeException )
{
	// Safe impossible cases
	// Method not defined for all incoming parameter
	LOG_ASSERT( impldbg_checkParameter_addStatusListener( xControl, aURL ), "OPlugInFrameDispatcher::removeStatusListener()\nInvalid parameter detected.\n" )
	// Add listener to container.
	m_aListenerContainer.removeInterface( aURL.Complete, xControl );
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
tIMPLExtractedArguments OPlugInFrameDispatcher::impl_extractArguments( const Sequence< PropertyValue >& seqArguments )
{
	// Set default return values
	tIMPLExtractedArguments aReturnStruct;
	aReturnStruct.nValidMask = 0;	// Mark all entry values as invalid!

	// Step over argument list and extract all needed informations.
	sal_Int32 nCount = seqArguments.getLength();
	for ( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
	{
		// If REFERRER found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_REFERRER )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.sReferrer;
			aReturnStruct.nValidMask |= MASK_REFERRER;
		}
		else
		// If POSTDATA found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_POSTDATA )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.xPostDataStream;
			aReturnStruct.nValidMask |= MASK_POSTDATA;
		}
	}

	// Retrn extracted informations.
	return aReturnStruct;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void OPlugInFrameDispatcher::impl_sendStatusEvent(	const	OUString&	sURL				,
															sal_Bool	bLoadingSuccessful	)
{
	// Get listener for given URL!
	OInterfaceContainerHelper* pListenerForURL = m_aListenerContainer.getContainer( sURL );
	// Send messages to all listener.
	// Do nothing, if there no listener or "getContainer()" works not correct!
	if( pListenerForURL != NULL )
	{
		// Build event for send to listener.
		FeatureStateEvent aEvent;
    	aEvent.FeatureURL.Complete	=	sURL							;
    	aEvent.FeatureDescriptor	=	FEATUREDESCRIPTOR_LOADSTATE		;
    	aEvent.IsEnabled			=	bLoadingSuccessful				;
    	aEvent.Requery				=	sal_False						;
    	aEvent.State				=	Any()							;

		// Send message to all listener on this URL.
		OInterfaceIteratorHelper aIterator(*pListenerForURL);
		while( aIterator.hasMoreElements() )
		{
			((XStatusListener*)aIterator.next())->statusChanged( aEvent );
		}
	}
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
// sTargetFrameName can be empty! But we need valid references for working.
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_OPlugInFrameDispatcherCtor(	const	Reference< XMultiServiceFactory >&		xFactory	,
																							Mutex&									aMutex		,
																					const	Reference< XPluginInstance >&			xOwner		,
																					const	Reference< XPluginInstancePeer >&		xPlugInDLL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xFactory			==	NULL		)	||
			( &aMutex			==	NULL		)	||
			( &xOwner			==	NULL		)	||
			( &xPlugInDLL		==	NULL		)	||
			( xFactory.is()		==	sal_False	)	||
			( xOwner.is()		==	sal_False	)	||
			( xPlugInDLL.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_queryDispatch(	const	URL&	 	aURL			,
		 																const	OUString&	sTargetFrameName,
																				sal_Int32	nSearchFlags	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aURL							==	NULL		)	||
			( aURL.Complete.getLength()		<	1			)	||
			( &sTargetFrameName				==	NULL		)	||
			// sTargetFrameName can be ""!
			(
				( 	 nSearchFlags != FrameSearchFlag::AUTO		  )	&&
				( !( nSearchFlags &  FrameSearchFlag::PARENT	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::SELF		) )	&&
				( !( nSearchFlags &  FrameSearchFlag::CHILDREN	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::CREATE	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::SIBLINGS	) )	&&
				( !( nSearchFlags &  FrameSearchFlag::TASKS		) )	&&
				( !( nSearchFlags &  FrameSearchFlag::ALL		) )	&&
				( !( nSearchFlags &  FrameSearchFlag::GLOBAL	) )
			)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_queryDispatches( const Sequence< DispatchDescriptor >& seqDescriptor )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &seqDescriptor			==	NULL	)	||
			( seqDescriptor.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}


//*****************************************************************************************************************
// We don't know anything about right values of aURL and seqArguments!
// Check valid references only.
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_dispatch(	const	URL&					  	aURL		,
																	const	Sequence< PropertyValue >&	seqArguments)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)	||
			( &seqArguments				==	NULL	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// We need a valid URL. What is meaning with "register for nothing"?!
// xControl must correct to - nobody can advised otherwise!
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_addStatusListener(	const	Reference< XStatusListener >&	xControl,
																			const	URL&							aURL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xControl					==	NULL	)	||
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
// The same goes for these case! We have added valid listener for correct URL only.
// We can't remove invalid listener for nothing!
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_removeStatusListener(	const	Reference< XStatusListener >&	xControl,
							   													const	URL&							aURL	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xControl					==	NULL	)	||
			( &aURL						==	NULL	)	||
			( aURL.Complete.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OPlugInFrameDispatcher::impldbg_checkParameter_notifyURL( const OUString& sURL )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &sURL				==	NULL	)	||
			( sURL.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
