/*************************************************************************
 *
 *  $RCSfile: otasksaccess.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_HELPER_OTASKSACCESS_HXX_
#include <helper/otasksaccess.hxx>
#endif

#ifndef __FRAMEWORK_HELPER_OTASKSENUMERATION_HXX_
#include <helper/otasksenumeration.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::container		;
using namespace ::com::sun::star::frame			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::uno			;
using namespace ::cppu							;
using namespace ::osl							;
using namespace ::rtl							;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
OTasksAccess::OTasksAccess(	const	Reference< XDesktop >&		xOwner	,
									FrameContainer*				pTasks	,
									Mutex&						aMutex	)
		//	Init baseclasses first
		:	OWeakObject		(			)
		// Init member
		,	m_aMutex		( aMutex	)
		,	m_xOwner		( xOwner	)
		,	m_pTasks		( pTasks	)
{
	// Safe impossible cases
	LOG_ASSERT( impldbg_checkParameter_OTasksAccessCtor( xOwner, pTasks, aMutex ), "OTasksAccess::OTasksAccess()\nInvalid parameter detected!\n" )
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
OTasksAccess::~OTasksAccess()
{
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider
//*****************************************************************************************************************

DEFINE_XINTERFACE_2(	OTasksAccess							,
						OWeakObject								,
						DIRECT_INTERFACE(XEnumerationAccess)	,
						DIRECT_INTERFACE(XElementAccess)		)

// Attention: If you have enabled any testmode different from TEST_NOTHING => you have declared XDebugging-interface automaticly!
// Bhere is no macro to define and implement the right methods. You are the only one, who know - how you will use this mechanism.
// It exist a macro to switch on or off your implementation only.
// SERVICEDEBUG_ENABLED must be defined, to activate your code!

//#ifdef SERVICEDEBUG_ENABLED

	// Debug mechanism for services is not used in this implementation.
	// This define will expand to nothing if SERVICEDEBUG_ENABLED unknown!
	// But if you will implement your own code - activate this define before ... and deactivate follow line.
//	DEFINE_EMPTY_XSPECIALDEBUGINTERFACE( OTasksAccess )

//#endif // #ifdef SERVICEDEBUG_ENABLED

//*****************************************************************************************************************
//	XEnumerationAccess
//*****************************************************************************************************************
Reference< XEnumeration > SAL_CALL OTasksAccess::createEnumeration() throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "OTasksAccess::createEnumeration()" )

	// Set default return value, if method failed.
	// If no desktop exist and there is no taskcontainer - return an empty enumeration!
	Reference< XEnumeration > xReturn = Reference< XEnumeration >();

	// Try to "lock" the desktop for access to taskscontainer.
	Reference< XInterface > xLock = m_xOwner.get();
	if ( xLock.is() == sal_True )
	{
		// Desktop exist => pointer to taskscontainer must be valid.
		// Initialize a new enumeration ... if some tasks exist!
		// (TasksEnumeration will make an assert, if we initialize the new instance without valid values!)

		// Create new instance in memory (user of this metod will be owner of this new object!)
		OTasksEnumeration* pEnumeration = new OTasksEnumeration( m_pTasks->getAllElements() );
		// Cast it to right reference.
		xReturn = Reference< XEnumeration >( (OWeakObject*)pEnumeration, UNO_QUERY );
	}

	// Return result of this operation.
	return xReturn;
}

//*****************************************************************************************************************
//	XElementAccess
//*****************************************************************************************************************
Type SAL_CALL OTasksAccess::getElementType() throw( RuntimeException )
{
	// Elements in list an enumeration are tasks!
	// Return the uno-type of XTask.
	return ::getCppuType((const Reference< XTask >*)NULL);
}

//*****************************************************************************************************************
//	XElementAccess
//*****************************************************************************************************************
sal_Bool SAL_CALL OTasksAccess::hasElements() throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "OTasksAccess::hasElements()" )

	// Set default return value, if method failed.
	sal_Bool bReturn = sal_False;

	// Try to "lock" the desktop for access to taskscontainer.
	Reference< XInterface > xLock = m_xOwner.get();
	if ( xLock.is() == sal_True )
	{
		// Desktop exist => pointer to taskscontainer must be valid.
		// Ask container for existing elements.
		bReturn = m_pTasks->hasElements();
	}

	// Return result of this operation.
	return bReturn;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool OTasksAccess::impldbg_checkParameter_OTasksAccessCtor(	const	Reference< XDesktop >&		xOwner	,
																		FrameContainer*				pTasks	,
																		Mutex&						aMutex	) const
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xOwner		==	NULL		)	||
			( &aMutex		==	NULL		)	||
			( xOwner.is()	==	sal_False	)	||
			( pTasks		==	NULL		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
