/*************************************************************************
 *
 *  $RCSfile: frameloaderfactory.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:29:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_SERVICES_FRAMELOADERFACTORY_HXX_
#include <services/frameloaderfactory.hxx>
#endif

#ifndef __FRAMEWORK_CLASSES_WILDCARD_HXX_
#include <classes/wildcard.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_XFRAMELOADER_HPP_
#include <com/sun/star/frame/XFrameLoader.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif

#ifndef _COM_SUN_STAR_FRAME_XEXTENDEDFILTERDETECTION_HPP_
#include <com/sun/star/frame/XExtendedFilterDetection.hpp>
#endif

#ifndef _COM_SUN_STAR_LANG_XINITIALIZATION_HPP_
#include <com/sun/star/lang/XInitialization.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::beans					;
using namespace ::com::sun::star::frame					;
using namespace ::com::sun::star::lang					;
using namespace ::com::sun::star::registry				;
using namespace ::com::sun::star::uno					;
using namespace ::cppu									;
using namespace ::osl									;
using namespace ::rtl									;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

#define	PROPERTY_REGISTRY								OUString(RTL_CONSTASCII_USTRINGPARAM("Registry"))
//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
FrameLoaderFactory::FrameLoaderFactory( const Reference< XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      OMutexMember is a struct with a mutex as member. We can't use a mutex as member directly, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	OMutexMember	(			)
		,	OWeakObject		(			)
		// Init member
		,	m_xFactory		( xFactory	)
		,	m_aRegistryCache(			)
{
	// Safe impossible cases
	// We can't work without valid initialization.
	LOG_ASSERT( impldbg_checkParameter_FrameLoaderFactoryCtor( xFactory ), "FrameLoaderFactory::FrameLoaderFactory()\nInvalid parameter detected!\n" )

	// Initialize ouer registrycache.
	// Ask servicemanager for XPropertySet-interface.
	Reference< XPropertySet >		xProperties( m_xFactory, UNO_QUERY );
	Reference< XSimpleRegistry >	xRegistry;
	// Get the value of the property "Registry".
	Any aValue = xProperties->getPropertyValue( PROPERTY_REGISTRY );
	aValue >>= xRegistry;
	// Fill cache with values from right registry.
	m_aRegistryCache.fill( xRegistry );
	// Safe impossible cases
	// We neew this cache for working.
	LOG_ASSERT( !(m_aRegistryCache.isValid()==sal_False), "FrameLoaderFactory::FrameLoaderFactory()\nInitializing of cache failed. Factory can't work so!\n" )
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
FrameLoaderFactory::~FrameLoaderFactory()
{
	// Clear ouer cache.
	m_aRegistryCache.clear();
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************
DEFINE_XINTERFACE_4						(	FrameLoaderFactory						,
											OWeakObject								,
											DIRECT_INTERFACE(XTypeProvider			),
											DIRECT_INTERFACE(XServiceInfo			),
											DIRECT_INTERFACE(XFrameLoaderQuery		),
											DIRECT_INTERFACE(XMultiServiceFactory	)
										)

DEFINE_XTYPEPROVIDER_4					(	FrameLoaderFactory	,
											XTypeProvider		,
											XServiceInfo		,
											XFrameLoaderQuery	,
											XMultiServiceFactory
										)

DEFINE_XSERVICEINFO_ONEINSTANCESERVICE	(	FrameLoaderFactory						,
											SERVICENAME_FRAMELOADERFACTORY			,
											IMPLEMENTATIONNAME_FRAMELOADERFACTORY
										)

// Attention: If you have enabled any testmode different from TEST_NOTHING => you have declared XDebugging-interface automaticly!
// Bhere is no macro to define and implement the right methods. You are the only one, who know - how you will use this mechanism.
// It exist a macro to switch on or off your implementation only.
// SERVICEDEBUG_ENABLED must be defined, to activate your code!

//#ifdef SERVICEDEBUG_ENABLED

	// Debug mechanism for services is not used in this implementation.
	// This define will expand to nothing if SERVICEDEBUG_ENABLED unknown!
	// But if you will implement your own code - activate this define before ... and deactivate follow line.
//	DEFINE_EMPTY_XSPECIALDEBUGINTERFACE( FrameLoaderFactory )

//#endif // #ifdef SERVICEDEBUG_ENABLED

//*****************************************************************************************************************
//	XFrameLoaderQuery
//*****************************************************************************************************************
Sequence< OUString > SAL_CALL FrameLoaderFactory::getAvailableFilterNames() throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "FrameLoaderFactory::getAvailableFilterNames()" )

	// Copy all filternames from ouer registrycache to returnlist.
	// First get enough memory for return list.
	sal_uInt32 nCount = m_aRegistryCache.getEntryCount();
	OUString* pBuffer = new OUString[nCount];

	// Then step over all items and copy valid filternames to list.
	sal_uInt32 nSourcePosition		= 0;
	sal_uInt32 nDestinationPosition = 0;
	for ( nSourcePosition=0; nSourcePosition<nCount; ++nSourcePosition )
	{
		// But take valid filternames only!
		if ( m_aRegistryCache[nSourcePosition].nValidMask & MASK_FILTERNAME )
		{
			pBuffer[nDestinationPosition] = m_aRegistryCache[nSourcePosition].sFilterName;
			++nDestinationPosition;
		}
		++nSourcePosition;
	}

	// Build return sequence.
	Sequence< OUString > seqReturn( pBuffer, nDestinationPosition );
	// free memory!
	delete pBuffer;
	// Return result of these operation.
	return seqReturn;
}

//*****************************************************************************************************************
//	XFrameLoaderQuery
//*****************************************************************************************************************
Sequence< PropertyValue > SAL_CALL FrameLoaderFactory::getLoaderProperties( const OUString& sFilterName ) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "FrameLoaderFactory::getLoaderProperties()" )
	// Safe impossible cases.
	// Method not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_getLoaderProperties( sFilterName ), "FrameLoaderFactory::getLoaderProperties()\nInvalid parameter detected!\n" )

	// Set default return value.
	Sequence< PropertyValue > seqReturn;

	// Search for filtername in registrycache.
	sal_uInt32 nFoundItem = m_aRegistryCache.findFilterName( sFilterName, 0 );
	// If we found someone ...
	if ( nFoundItem != INVALID_CACHE_POSITION )
	{
		// ... ask cache for property list of found entry and set it for return.
		seqReturn = m_aRegistryCache.getEntryAsPropertySequence( nFoundItem );
	}

	// Retrun result of this operation.
	return seqReturn;
}

//*****************************************************************************************************************
//	XFrameLoaderQuery
//*****************************************************************************************************************
OUString SAL_CALL FrameLoaderFactory::searchFilter(	const	OUString&					sURL			,
													const	Sequence< PropertyValue >&	seqArguments	) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "FrameLoaderFactory::searchFilter()" )
	// Safe impossible cases.
	// Method not defined for all incoming parameter.
	LOG_ASSERT( impldbg_checkParameter_searchFilter( sURL, seqArguments ), "FrameLoaderFactory::searchFilter()\nInvalid parameter detected!\n" )

	// Set default return value if method failed.
	OUString sFoundName;

	// Search right loader for given parameter.
	sal_uInt32 nFoundItem = impl_searchFrameLoader( sURL, seqArguments );
	// If we have found any loader ...
	if ( nFoundItem != INVALID_CACHE_POSITION )
	{
		// ... set filtername for return.
		sFoundName = m_aRegistryCache[nFoundItem].sFilterName;
	}

	// Return result of this operation.
    return sFoundName;
}

//*****************************************************************************************************************
//	XMultiServiceFactory
//*****************************************************************************************************************
Reference< XInterface > SAL_CALL FrameLoaderFactory::createInstance( const OUString& sServiceSpecifier ) throw(	Exception		,
							  																					RuntimeException)
{
	// We use other interface method createInstanceWithArguments() to do this. But we call it with empty argument list!
	return createInstanceWithArguments( sServiceSpecifier, Sequence< Any >() );
}

//*****************************************************************************************************************
//	XMultiServiceFactory
//*****************************************************************************************************************
Reference< XInterface > SAL_CALL FrameLoaderFactory::createInstanceWithArguments(	const	OUString&			sServiceSpecifier	,
																					const	Sequence< Any >&	seqArguments		) throw(	Exception		,
																																				RuntimeException)
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "FrameLoaderFactory::createInstanceWithArguments()" )

	// Safe impossible cases
	// Method not defined for all incoming parameter!
	// AND: sServiceSpecifier MUST exist as value of an entry of ouer registry cache!
	LOG_ASSERT( impldbg_checkParameter_createInstanceWithArguments( sServiceSpecifier, seqArguments ), "FrameLoaderFactory::createInstanceWithArguments()\nInvalid parameter detected!\n" )

	// Set default return value.
	Reference< XInterface > xReturn;

	// Copy arguments from "Any"-structures to structures of "PropertyValue".
	sal_Int32					nArgumentCount	= seqArguments.getLength();
    Sequence< PropertyValue >	seqProperties	( nArgumentCount );
    for ( sal_Int32 nPosition=0; nPosition<nArgumentCount; ++nPosition )
    {
		seqArguments[nPosition] >>= seqProperties[nPosition];
    }

	// Try to find right frame loder for given parameter.
	sal_uInt32 nEntryFound = impl_searchFrameLoader( sServiceSpecifier, seqProperties );
	if ( nEntryFound != INVALID_CACHE_POSITION )
	{
        // If loader has registered a detect service ...
        if ( m_aRegistryCache[nEntryFound].nValidMask & MASK_DETECTSERVICE )
        {
            // ... ask it for his agreement.
			OUString sMyFoundFilterName	= m_aRegistryCache[nEntryFound].sFilterName;
            OUString sDetectedFilterName= impl_extendedDetect( m_aRegistryCache[nEntryFound].sDetectService, sMyFoundFilterName, sServiceSpecifier, seqProperties );
            // The detectservice has the following possibilities:
            // 1) He doesn't know anything and returns an empty string ...
            //    => Then we must ask ALL detect services of all cache entries ( done below )
            if( sDetectedFilterName.getLength() < 1 )
            {
                nEntryFound = INVALID_CACHE_POSITION;
			}
			else
			// 2) He disagree with our result. => Then we must use his result to instanciate the frame loader.
			if( sDetectedFilterName != sMyFoundFilterName )
			{
				nEntryFound = m_aRegistryCache.findFilterName( sDetectedFilterName, 0 );
				// Safe impossible cases
				// The detect service can return a filtername, which I don't know ...
				LOG_ASSERT( !(nEntryFound==INVALID_CACHE_POSITION), "FrameLoaderFactory::createInstanceWidthArguments()\nA detect service return a unknown filter name!\n")
        	}
            // 3) He agrees with our result. => We can instanciate the frame loader ( done below )
			//    (In this case we have already the valid cache position to get the implementation name of the frame loader!
			//     but it can be that the detectservice return a wrong value! see before)
        }
        // ... if no detect service for this loader exist ...
        else
        {
            // ... we create it however. His name is the right one and he can't disagree!
            xReturn = Reference< XInterface >( m_xFactory->createInstance( m_aRegistryCache[nEntryFound].sImplementationName ), UNO_QUERY );
        }
    }
    else
//    if ( nEntryFound == INVALID_CACHE_POSITION )
    {
        // no loader found until here or found loader was withdrawn by the extended detect service
        // our only chance: call ALL detect services
        OUString sDetectedFilterName;
        sal_uInt32 nCount       = m_aRegistryCache.getEntryCount();
        sal_uInt32 nPosition    = 0;
        while   (
                    ( nPosition                         <   nCount  )   &&
                    ( sDetectedFilterName.getLength()   <   1       )
                )
        {
            sDetectedFilterName = impl_extendedDetect( m_aRegistryCache[nPosition].sDetectService, m_aRegistryCache[nPosition].sFilterName, sServiceSpecifier, seqProperties );
            if ( sDetectedFilterName.getLength() )
				nEntryFound = m_aRegistryCache.findFilterName( sDetectedFilterName, 0 );
            else
                ++nPosition;
        }
    }

    if( nEntryFound != INVALID_CACHE_POSITION )
    {
        // found one at last : so get it!
        xReturn = Reference< XInterface >( m_xFactory->createInstance( m_aRegistryCache[nEntryFound].sImplementationName ), UNO_QUERY );
    }

    if ( xReturn.is() && m_aRegistryCache[nEntryFound].sImplementationName == IMPLEMENTATIONNAME_GENERICFRAMELOADER )
    {
        // Attention: some loaders have generic frameloader implementation.
		// We must create this loader with special arguments! He must know which special frameloader is searched.
        // Add these informations to the given argument list.
		Sequence< Any > seqMyArguments( seqArguments );
		{
			PropertyValue aSpecialArgument;
			aSpecialArgument.Name	=	ARGUMENT_FILTERNAME							;
			aSpecialArgument.Handle	=	-1											;
			aSpecialArgument.Value	<<=	m_aRegistryCache[nEntryFound].sFilterName	;
			aSpecialArgument.State	=	PropertyState_DIRECT_VALUE					;

			seqMyArguments.realloc( seqArguments.getLength()+1 );
			seqMyArguments[seqMyArguments.getLength()-1] <<= aSpecialArgument;

            // Initialize loader with arguments
            try
            {
                Reference< XInitialization > xInit( xReturn, UNO_QUERY );
                if( xInit.is() == sal_True )
                {
                    xInit->initialize( seqMyArguments );
                }
            }
            catch( IllegalArgumentException& )
            {
            }
        }
	}

	// Return result of this operation.
	return xReturn;
}

//*****************************************************************************************************************
//	XMultiServiceFactory
//*****************************************************************************************************************
Sequence< OUString > SAL_CALL FrameLoaderFactory::getAvailableServiceNames() throw( RuntimeException )
{
	LOG_ASSERT( sal_False, "FrameLoaderFactory::getAvailableServiceName()\nNot implemented!\n" )
	return Sequence< OUString >();
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
sal_uInt32 FrameLoaderFactory::impl_searchFrameLoader(	const	OUString&					sURL			,
		  												const	Sequence< PropertyValue >&	seqArguments	)
{
	// Set default return value.
	sal_uInt32 nReturn = INVALID_CACHE_POSITION;

	// Extract optional parameter from argumentlist.
	tIMPLExtractedArguments aExtractedArguments = impl_extractArguments( seqArguments );

	// Define some help variables for follow operations.
	sal_uInt32	nEntryCount		;
	sal_uInt32	nEntryNumber	;

	// These variable is used very often at next lines.
	// Don't change his value without cogent reason!
	nEntryCount = m_aRegistryCache.getEntryCount();

	//-------------------------------------------------------------------------------------------------------------
	//	1)	Is one of given arguments the filtername?
	//-------------------------------------------------------------------------------------------------------------
	if	(
			( nReturn							==	INVALID_CACHE_POSITION	)	&&
			( aExtractedArguments.nValidMask	&	MASK_FILTERNAME			)
		)
	{
		// YES; Search in cache for right entry with same name.
		nReturn = m_aRegistryCache.findFilterName( aExtractedArguments.sFilterName, 0 );
	}

	//-------------------------------------------------------------------------------------------------------------
	//	2)	Search for matching pattern!
	//-------------------------------------------------------------------------------------------------------------
	if ( nReturn == INVALID_CACHE_POSITION )
	{
		// Loop over all cache entries until we found right frameloader or all items are handled.
    	nEntryNumber = 0;
    	while	(
					( nReturn		==	INVALID_CACHE_POSITION	)	&&
					( nEntryNumber	<	nEntryCount				)
				)
    	{
			// Loop over all pattern of current entry.
        	sal_uInt32 nPatternCount	= m_aRegistryCache.getPatternCount( nEntryNumber );
			sal_uInt32 nPatternNumber	= 0;
			while	(
						( nReturn			==	INVALID_CACHE_POSITION	)	&&
						( nPatternNumber	<	nPatternCount			)
					)
        	{
				// If pattern match the url ...
            	if ( Wildcard::match( sURL, m_aRegistryCache.getPattern( nEntryNumber, nPatternNumber ) ) == sal_True )
				{
					// We have found a loader for this pattern.
					// Set value for return.
					nReturn = nEntryNumber;
				}
				// Else; step to next pattern ...
				++nPatternNumber;
        	}
			// .. or step to next entry.
			++nEntryNumber;
    	}
	}

	//-------------------------------------------------------------------------------------------------------------
	//	3)	Is one of given arguments the mimetype?
	//-------------------------------------------------------------------------------------------------------------
	if	(
			( nReturn							==	INVALID_CACHE_POSITION	)	&&
			( aExtractedArguments.nValidMask	&	MASK_MIMETYPE			)
		)
	{
		// Search for MimeType.
		nReturn = m_aRegistryCache.findMimeType( aExtractedArguments.sMimeType, 0 );
	}

	//-------------------------------------------------------------------------------------------------------------
	//	4)	Is one of given arguments the clipboardformat?
	//-------------------------------------------------------------------------------------------------------------
	if	(
			( nReturn							==	INVALID_CACHE_POSITION	)	&&
			( aExtractedArguments.nValidMask	&	MASK_FORMAT				)
		)
	{
		// YES; Search in cache for right entry with same format.
		nReturn = m_aRegistryCache.findClipboardFormat( aExtractedArguments.nClipboardFormat, 0 );
	}

	//-------------------------------------------------------------------------------------------------------------
	//	5)	Search for matching extension!
	//-------------------------------------------------------------------------------------------------------------
	if ( nReturn == INVALID_CACHE_POSITION )
	{
		// If extension of URL well known ...
		OUString sExtension = impl_getExtensionFromURL( sURL );
    	if ( sExtension.getLength() > 0 )
    	{
			// ... search for extension at all entries of cache.
    		nEntryNumber = 0;
    		while	(
						( nReturn		==	INVALID_CACHE_POSITION	)	&&
						( nEntryNumber	<	nEntryCount				)
					)
    		{
				// Loop over all extension of current entry.
		        sal_uInt32 nExtensionCount	= m_aRegistryCache.getExtensionCount( nEntryNumber );
				sal_uInt32 nExtensionNumber	= 0;
				while	(
							( nReturn			==	INVALID_CACHE_POSITION	)	&&
							( nExtensionNumber	<	nExtensionCount			)
						)
        		{
					// If extension is in current extension-list ...
                    if ( m_aRegistryCache.getExtension( nEntryNumber, nExtensionNumber ).equalsIgnoreCase( sExtension ) )
					{
						// We have found a loader for this extension.
						// Set value for return.
						nReturn = nEntryNumber;
					}
					// Else; steop to next extension ...
					++nExtensionNumber;
        		}
				// .. or step to next entry.
				++nEntryNumber;
    		}
    	}
	}

	// Return result of this operation.
    return nReturn;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
tIMPLExtractedArguments FrameLoaderFactory::impl_extractArguments( const SEQUENCE< PROPERTYVALUE >& seqArguments )
{
	// Set default return values
	tIMPLExtractedArguments aReturnStruct;
	aReturnStruct.nValidMask = 0;	// Mark all entry values as invalid!

	// Step over argument list and extract all informations.
	sal_Int32 nCount = seqArguments.getLength();
	for ( sal_Int32 nPosition=0; nPosition<nCount; ++nPosition )
	{
		// If MIMETYPE found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_MIMETYPE )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.sMimeType;
			aReturnStruct.nValidMask |= MASK_MIMETYPE;
		}
		else
		// If FLAGS found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_FLAGS )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.nFlags;
			aReturnStruct.nValidMask |= MASK_FLAGS;
		}
		else
		// If FILTERNAME found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_FILTERNAME )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.sFilterName;
			aReturnStruct.nValidMask |= MASK_FILTERNAME;
		}
		else
		// If (Clipboard)FORMAT found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_FORMAT )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.nClipboardFormat;
			aReturnStruct.nValidMask |= MASK_FORMAT;
		}
		else
		// If DETECTSERVICE found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_DETECTSERVICE )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.sDetectService;
			aReturnStruct.nValidMask |= MASK_DETECTSERVICE;
		}
		else
		// If INPUTS found ... set state to valid and safe value.
		if ( seqArguments[nPosition].Name == ARGUMENT_INPUTSTREAM )
		{
			seqArguments[nPosition].Value >>= aReturnStruct.xInputStream;
			aReturnStruct.nValidMask |= MASK_INPUTSTREAM;
		}
	}

	// Retrn extracted informations.
	return aReturnStruct;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
OUString FrameLoaderFactory::impl_extendedDetect(	const	OUString&					sDetectService	,
                                                    const   OUString&                   sFilterName     ,
		 											const	OUString&					sURL			,
		 											const	Sequence< PropertyValue >&	seqArguments	)
{
	// Set default return value.
	OUString sReturn;

	// Create the detect service.
	Reference< XExtendedFilterDetection > xDetectService( m_xFactory->createInstance( sDetectService ), UNO_QUERY );
	// Safe impossible cases
	// Somewhere has give us a valid name for a detect service ... but we can't create it ... !?
	LOG_ASSERT( !(xDetectService.is()==sal_False), "FrameLoaderFactory::impl_extendedDetect()\nNo detect service found for given servicename!\n" )
	if ( xDetectService.is()==sal_True )
	{
        if ( sDetectService == SERVICENAME_GENERICFRAMELOADER )
		{
            Sequence< Any > seqMyArguments( 1 );
			PropertyValue aSpecialArgument;
			aSpecialArgument.Name	=	ARGUMENT_FILTERNAME							;
			aSpecialArgument.Handle	=	-1											;
            aSpecialArgument.Value  <<= sFilterName                                 ;
			aSpecialArgument.State	=	PropertyState_DIRECT_VALUE					;

            seqMyArguments[0] <<= aSpecialArgument;
            try
            {
                Reference< XInitialization > xInit( xDetectService, UNO_QUERY );
                if( xInit.is() == sal_True )
                {
                    xInit->initialize( seqMyArguments );
                }
            }
            catch( IllegalArgumentException& )
            {
            }
        }

		// Ask him for his agreement for given URL and arguments and return the result!
		sReturn = xDetectService->detect( sURL, seqArguments );
	}

	// Return result of this operation.
	return sReturn;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
OUString FrameLoaderFactory::impl_getExtensionFromURL( const OUSTRING& sURL )
{
	// Get the extensions of the given URL.
	// Take first a default value.
	OUString sExtension;
	// Search for last occurence of '.'
	sal_Int32 nPointPosition = sURL.lastIndexOf( '.' );
	if ( nPointPosition >= 0 )
	{
		// We have found a '.' in URL.
		// But if character before this position a '/' ... this is not a '.' of an extensions!
		if ( sURL.lastIndexOf( '/', nPointPosition ) != (nPointPosition-1) )
		{
			// We have found a valid extensions. Copy value to new variable (but without dot!).
			sExtension = sURL.copy( nPointPosition+1 );
		}
	}

	// Return result of this operation.
	return sExtension;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool FrameLoaderFactory::impldbg_checkParameter_FrameLoaderFactoryCtor( const Reference< XMultiServiceFactory >& xFactory )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xFactory		==	NULL		)	||
			( xFactory.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool FrameLoaderFactory::impldbg_checkParameter_getLoaderProperties( const OUString& sFilterName )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &sFilterName				==	NULL	)	||
			( sFilterName.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool FrameLoaderFactory::impldbg_checkParameter_searchFilter(	const	OUString&					sURL		,
																	const	Sequence< PropertyValue >&	seqArguments)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &sURL				==	NULL	)	||
			( &seqArguments		==	NULL	)	||
			( sURL.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool FrameLoaderFactory::impldbg_checkParameter_createInstanceWithArguments	(	const	OUSTRING&			sServiceSpecifier	,
																					const	SEQUENCE< ANY >&	seqArguments		)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &sServiceSpecifier			==	NULL	)	||
			( sServiceSpecifier.getLength()	<	1		)	||
			( &seqArguments					==	NULL	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
