/*************************************************************************
 *
 *  $RCSfile: task.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: hjs $ $Date: 2000/09/21 14:58:21 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_SERVICES_TASK_HXX_
#include <services/task.hxx>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_FRAMESEARCHFLAG_HPP_
#include <com/sun/star/frame/FrameSearchFlag.hpp>
#endif

#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HPP_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _CPPUHELPER_PROPTYPEHLP_HXX
#include <cppuhelper/proptypehlp.hxx>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

using namespace ::com::sun::star::awt						;
using namespace ::com::sun::star::beans						;
using namespace ::com::sun::star::container					;
using namespace ::com::sun::star::frame						;
using namespace ::com::sun::star::lang						;
using namespace ::com::sun::star::task						;
using namespace ::com::sun::star::uno						;
using namespace ::com::sun::star::util						;
using namespace ::cppu										;
using namespace ::osl										;
using namespace ::rtl										;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

#define	PROPERTYNAME_ISALWAYSVISIBLE			OUString(RTL_CONSTASCII_USTRINGPARAM("IsAlwaysVisible"	))
#define	PROPERTYNAME_ISFLOATING					OUString(RTL_CONSTASCII_USTRINGPARAM("IsFloating"		))
#define	PROPERTYNAME_ISPLUGIN					OUString(RTL_CONSTASCII_USTRINGPARAM("IsPlugIn"			))
#define	PROPERTYNAME_POSITION					OUString(RTL_CONSTASCII_USTRINGPARAM("Position"			))
#define	PROPERTYNAME_SIZE						OUString(RTL_CONSTASCII_USTRINGPARAM("Size"				))
#define	PROPERTYNAME_TITLE						OUString(RTL_CONSTASCII_USTRINGPARAM("Title"			))

#define PROPERTYHANDLE_ISALWAYSVISIBLE			1
#define PROPERTYHANDLE_ISFLOATING				2
#define PROPERTYHANDLE_ISPLUGIN					3
#define PROPERTYHANDLE_POSITION					4
#define PROPERTYHANDLE_SIZE						5
#define PROPERTYHANDLE_TITLE					6

#define	PROPERTYCOUNT							6

#define	DEFAULT_BISALWAYSVISIBLE				sal_False
#define	DEFAULT_BISFLOATING						sal_False
#define	DEFAULT_BISPLUGIN						sal_False
#define	DEFAULT_APOSITION						Point(0,0)
#define	DEFAULT_ASIZE							Size(0,0)
#define	DEFAULT_STITLE							OUString()

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
Task::Task( const Reference< XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
		//	Attention:
		//		Don't change order of initialization!
		//      tIMPLMutexContainer is a struct with a mutex as member. We can't use a mutex as member, while
		//		we must garant right initialization and a valid value of this! First initialize
		//		baseclasses and then members. And we need the mutex for other baseclasses !!!
		:	Frame					( xFactory																				)
		,	OBroadcastHelper		( ((OMutexMember*)this)->m_aMutex														)
		,	OPropertySetHelper		( *SAL_STATIC_CAST( OBroadcastHelperVar< OMultiTypeInterfaceContainerHelper >*, this )	)
		// Init member
		,	m_bIsAlwaysVisible		( DEFAULT_BISALWAYSVISIBLE																)
		,	m_bIsFloating			( DEFAULT_BISFLOATING																	)
		,	m_bIsPlugIn				( DEFAULT_BISPLUGIN																		)
		,	m_aPosition				( DEFAULT_APOSITION																		)
		,	m_aSize					( DEFAULT_ASIZE																			)
		,	m_sTitle				( DEFAULT_STITLE																		)
        ,   m_aPoster               ( LINK( this, Task, Close_Impl )                                                        )
{
}

//*****************************************************************************************************************
//	destructor
//*****************************************************************************************************************
Task::~Task()
{
}

//*****************************************************************************************************************
//	XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************

DEFINE_XINTERFACE_4						(	Task								,
											Frame								,
											DIRECT_INTERFACE(XTask				),
											DIRECT_INTERFACE(XPropertySet		),
											DIRECT_INTERFACE(XFastPropertySet	),
											DIRECT_INTERFACE(XMultiPropertySet	)
										)

DEFINE_XTYPEPROVIDER_4_WITH_BASECLASS	(	Task				,
											Frame				,
											XTask				,
											XPropertySet		,
											XFastPropertySet	,
											XMultiPropertySet
										)

DEFINE_XSERVICEINFO_MULTISERVICE		(	Task					,
											SERVICENAME_TASK		,
											IMPLEMENTATIONNAME_TASK
										)

//*****************************************************************************************************************
//	XTask
//*****************************************************************************************************************
sal_Bool SAL_CALL Task::close() throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aFirstGuard, m_aMutex, "Task::close()" )

	// Set default result of this question to NO.
	sal_Bool bIAmClosed = sal_False;

	// Now ask ouer current controller.
	Reference< XController > xController = getController();
	sal_Bool bCanClose = sal_True;

	if( xController.is() == sal_True )
	{
		// If we have anyone ...
		bCanClose = xController->suspend( sal_True );
	}

	if( bCanClose == sal_True )
	{
		// ... and he accept closing ... do it.
		dispose();
		bIAmClosed = sal_True;
	}

	// Return result of this operation.
	return bIAmClosed;
}

//*****************************************************************************************************************
//	XTask
//*****************************************************************************************************************
void SAL_CALL Task::tileWindows() throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	XTask
//*****************************************************************************************************************
void SAL_CALL Task::arrangeWindowsVertical() throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	XTask
//*****************************************************************************************************************
void SAL_CALL Task::arrangeWindowsHorizontal() throw( RuntimeException )
{
}

//*****************************************************************************************************************
//	 XFrame
//*****************************************************************************************************************
Reference< XFrame > SAL_CALL Task::findFrame(	const	OUString&	sTargetFrameName	,
														sal_Int32	nSearchFlags		) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "Task::findFrame()" )
	// Log some special informations about search. (Active in debug version only, if special mode is set!)
	LOG_PARAMETER_FINDFRAME( "Task", m_sName, sTargetFrameName, nSearchFlags )

	// Set default return value.
	Reference< XFrame > xReturn = Reference< XFrame >();

	// Protection against recursion while searching in parent frames!
	// See search for PARENT for further informations.
	if ( m_bRecursiveSearchProtection == sal_False )
	{
		//*************************************************************************************************************
		//	1)	Search for "_self", "" and "_top".
		//		Attention:
		//		"_top" search for a topframe in hierarchy and a task is a topframe himself!
		//		I think its OK to do the same like "_self".
		//*************************************************************************************************************
		if	(
				( sTargetFrameName				==	FRAMETYPE_SELF	)	||
				( sTargetFrameName				==	FRAMETYPE_TOP	)	||
				( sTargetFrameName.getLength()	<	1				)
			)
		{
			LOG_TARGETINGSTEP( "Task", m_sName, "react to \"_self\" or \"_top\" or \"\"" )
			xReturn = Reference< XFrame >( static_cast< OWeakObject* >( this ), UNO_QUERY );
		}
		else
		//*************************************************************************************************************
		//	2)	Forward "_blank" to desktop. He can create new task only!
		//		(Look for existing parent!)
		//*************************************************************************************************************
		if( sTargetFrameName == FRAMETYPE_BLANK )
		{
			LOG_TARGETINGSTEP( "Task", m_sName, "react to \"_blank\"" )
			if( m_xParent.is() ==  sal_True )
			{
				LOG_TARGETINGSTEP( "Task", m_sName, "forward \"_blank\" to parent" )
				xReturn = m_xParent->findFrame( FRAMETYPE_BLANK, 0 );
			}
			else
			{
				// Else we cant create this new frame!
				LOG_TARGETINGSTEP( "Task", m_sName, "can create new frame for \"_blank\"" )
			}
		}
		else
		//*************************************************************************************************************
		//	ATTENTION!
		//	We have searched for special targets only ... but now we must search for any named frames and use search
		//	flags to do that!
		//*************************************************************************************************************
		{
			//*********************************************************************************************************
			//	At first we must filter all other special target names!
			//	You can disable this statement if all these cases are handled before ...
			//*********************************************************************************************************
			if	(
					( sTargetFrameName != FRAMETYPE_PARENT )
				)
			{
				//*****************************************************************************************************
				//	3)	If SELF or TASKS searched and given name is the right one, we can return us as result.
				//*****************************************************************************************************
				if	(
						( sTargetFrameName	==	m_sName					)	&&
						(
							( nSearchFlags	&	FrameSearchFlag::SELF	)	||
							( nSearchFlags	&	FrameSearchFlag::TASKS	)
						)
					)
				{
					LOG_TARGETINGSTEP( "Task", m_sName, "react to SELF or TASKS" )
					xReturn = Reference< XFrame >( static_cast< OWeakObject* >( this ), UNO_QUERY );
				}
				//*****************************************************************************************************
				//	4)	If TASKS or SIBLINGS searched we can forward it to our parent the desktop.
				//		He can search for CHILDREN for us!
				//*****************************************************************************************************
				if	(
						( xReturn.is()		==	sal_False					)	&&
						( m_xParent.is()	==	sal_True					)	&&
						(
							( nSearchFlags	&	FrameSearchFlag::SIBLINGS	)	||
							( nSearchFlags	&	FrameSearchFlag::TASKS		)
						)
					)
				{
					// We must protect us against searching from top to bottom!
					m_bRecursiveSearchProtection = sal_True  ;
					LOG_TARGETINGSTEP( "Task", m_sName, "forward TASKS or SIBLINGS to parent" )
					xReturn = m_xParent->findFrame( sTargetFrameName, FrameSearchFlag::SIBLINGS | FrameSearchFlag::TASKS );
					m_bRecursiveSearchProtection = sal_False ;
				}
				//*****************************************************************************************************
				//	5)	If CHILDREN searched we search from here to bottom.
				//*****************************************************************************************************
				if	(
						( xReturn.is()							==	sal_False					)	&&
						( nSearchFlags							&	FrameSearchFlag::CHILDREN	)	&&
						( m_aChildFrameContainer.hasElements()	==	sal_True					)
					)
				{
					LOG_TARGETINGSTEP( "Task", m_sName, "react to CHILDREN" )
					// Search at own container of childframes if allowed.
					// Lock the container. Nobody should append or remove elements during next time.
					// But don't forget to unlock it again!
					m_aChildFrameContainer.lock();

					// First search only for direct subframes.
					// Break loop, if something was found or all container items was compared.
					sal_uInt32 nCount		= m_aChildFrameContainer.getCount();
					sal_uInt32 nPosition	= 0;
    				while	(
								( xReturn.is()	==	sal_False	)	&&
								( nPosition		<	nCount		)
							)
					{
        				xReturn = m_aChildFrameContainer[nPosition]->findFrame( sTargetFrameName, FrameSearchFlag::SELF );
						++nPosition;
					}

					// If no direct subframe was found, search now subframes of subframes.
					nPosition = 0;
    				while	(
								( xReturn.is()	==	sal_False	)	&&
								( nPosition		<	nCount		)
							)
					{
        				xReturn = m_aChildFrameContainer[nPosition]->findFrame( sTargetFrameName, FrameSearchFlag::CHILDREN );
						++nPosition;
					}

					// Don't forget to unlock the container!
					m_aChildFrameContainer.unlock();
				}
				//*************************************************************************************************************
				//	6)	If CREATE is set we must forward call to desktop. He is the only one, who can do that.
				//*************************************************************************************************************
				/*
					Praeprozessor Bug!
					Wenn nach CREATE ein Space steht wird versucht es durch das Define CREATE aus tools/rtti.hxx zu ersetzen
					was fehlschlaegt und die naechsten 3 Klammern ")){" unterschlaegt!
					Dann meckert der Compiler das natuerlich an ...
	 			*/
				if((xReturn.is()==sal_False)&&(nSearchFlags&FrameSearchFlag::CREATE)&&(m_xParent.is()==sal_True))
				{
					LOG_TARGETINGSTEP( "Task", m_sName, "forward CREATE to parent" )
					xReturn	= m_xParent->findFrame( sTargetFrameName, FrameSearchFlag::CREATE );
				}
			}
		}
	}

	// Log some special informations about search. (Active in debug version only, if special mode is set!)
	LOG_RESULT_FINDFRAME( "Task", m_sName, xReturn )
	// Return result of this search.
	return xReturn;
}

/*
Reference< XFrame > SAL_CALL Task::findFrame(	const	OUString&	sTargetFrameName	,
														sal_Int32	nSearchFlags		) throw( RuntimeException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "Task::findFrame()" )
	// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//	LOG_SEARCHRESULT( m_sName, "Search started on Task" )
//	LOG_SEARCHPARAMETER( "Task", m_sName, sTargetFrameName, nSearchFlags )

	// Set default return value.
	Reference< XFrame > xReturn;

	//*************************************************************************************************************
	//	1)	Search for "_self" and "_top".
	//		Attention:
	//		"_top" search for a topframe in hierarchy. But a task is a topframe himself!
	//		I think its OK to do the same like "_self".
	//*************************************************************************************************************
	if	(
			( xReturn.is()		==	sal_False		)	&&
			(
				( sTargetFrameName	==	FRAMETYPE_SELF	)	||
				( sTargetFrameName	==	FRAMETYPE_TOP	)
			)
		)
	{
		xReturn = Reference< XFrame >( static_cast< OWeakObject* >( this ), UNO_QUERY );
		// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//		LOG_SEARCHRESULT( m_sName, xReturn.is() ? "_self or _top found" : "_self or _top failed" )
	}

	//*************************************************************************************************************
	//	2)	Search for "_parent".
	//*************************************************************************************************************
	if	(
			( xReturn.is()		==	sal_False			)	&&
			( sTargetFrameName	==	FRAMETYPE_PARENT	)
		)
	{
		xReturn = Reference< XFrame >( m_xParent, UNO_QUERY );
		// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//		LOG_SEARCHRESULT( m_sName, xReturn.is() ? "_parent found" : "_parent failed" )
	}

	//*************************************************************************************************************
	//	3)	Search for SELF.
	//*************************************************************************************************************
	if	(
			( xReturn.is()					==	sal_False				)	&&
			( nSearchFlags					&	FrameSearchFlag::SELF	)	&&
			( sTargetFrameName				==	m_sName					)
		)
	{
		xReturn = Reference< XFrame >( static_cast< OWeakObject* >( this ), UNO_QUERY );
		// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//		LOG_SEARCHRESULT( m_sName, xReturn.is() ? "SELF found" : "SELF failed" )
	}

	//*************************************************************************************************************
	//	4)	Search for TASKS.
	//		Attention:
	//		Tasks can be ouer siblings only!
	//		But we don't search for SIBLINGS, we search at ouer parent (the Desktop) for TASKS!
	//		He hold the container and we can't work, if no parent exist!
	//		Because; if no parent exist => we have no siblings!!!
	//*************************************************************************************************************
	if	(
			( xReturn.is()		==	sal_False				)	&&
			( m_xParent.is()	==	sal_True				)	&&
			( nSearchFlags		&	FrameSearchFlag::TASKS	)
		)
	{
		Reference< XFrame > xDesktopFrame( m_xParent, UNO_QUERY );
		xReturn = xDesktopFrame->findFrame( sTargetFrameName, FrameSearchFlag::TASKS );
		// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//		LOG_SEARCHRESULT( m_sName, xReturn.is() ? "TASKS found" : "TASKS failed" )
	}

	//*************************************************************************************************************
	//	5)	If no frame was found before ... ask baseclass for searched frame.
	//		Attention:
	//		Unmask TASK-flag. We have handled this feature already before!
	//		unmask CREATE-flag. Only desktop can create new frames!
	//*************************************************************************************************************

	if ( xReturn.is() == sal_False )
	{
		nSearchFlags = nSearchFlags & UNMASK_TASKS	;
		nSearchFlags = nSearchFlags & UNMASK_CREATE	;

		// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//		LOG_SEARCHRESULT( m_sName, "forward search to baseclass" )

		xReturn = Frame::findFrame( sTargetFrameName, nSearchFlags );

		// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//		LOG_SEARCHRESULT( m_sName, xReturn.is() ? "search in subtree ok" : "search in subtree failed" )
	}

	// Log some special informations about search. (Active in debug version only, if special mode in debug.h is set!)
//	LOG_SEARCHRESULT( m_sName, "End of search in Task" )

	// Return result of this search.
	return xReturn;
}
*/
//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Task::windowClosing( const EventObject& aEvent ) throw( RuntimeException )
{
	deactivate();
    m_aPoster.Post( 0 );
}

IMPL_LINK( Task, Close_Impl, void*, pVoid )
{
    close();
    return 0;
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Task::windowActivated( const EventObject& aEvent ) throw( RuntimeException )
{
    Frame::windowActivated( aEvent );
}

//*****************************************************************************************************************
//	 XTopWindowListener
//*****************************************************************************************************************
void SAL_CALL Task::windowDeactivated( const EventObject& aEvent ) throw( RuntimeException )
{
    // always one task must be active
    // transferring focus outside the task windows must not deactivate the task
    // so this event will never be evaluated
}

//*****************************************************************************************************************
//	 XEventListener
//*****************************************************************************************************************
void SAL_CALL Task::disposing( const EventObject& aEvent ) throw( RuntimeException )
{
	// Reset ouer member to default values.
	m_bIsAlwaysVisible	=	DEFAULT_BISALWAYSVISIBLE;
	m_bIsFloating		=	DEFAULT_BISFLOATING		;
	m_bIsPlugIn			=	DEFAULT_BISPLUGIN		;
	m_aPosition			=	DEFAULT_APOSITION		;
	m_aSize				=	DEFAULT_ASIZE			;
	m_sTitle			=	DEFAULT_STITLE			;
	// Don't forget to call baseclass!
	Frame::disposing( aEvent );
}

//*****************************************************************************************************************
//	OPropertySetHelper
//*****************************************************************************************************************
sal_Bool SAL_CALL Task::convertFastPropertyValue(			Any&		aConvertedValue	,
				  											Any&		aOldValue		,
															sal_Int32	nHandle			,
													const	Any&		aValue			) throw( IllegalArgumentException )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "Task::convertFastPropertyValue()" )

	//	Safe impossible cases
//	LOG_ASSERT ( impl_debug_checkParameter ( rConvertedValue, rOldValue, nHandle, rValue ), "Task::convertFastPropertyValue()\nInvalid parameter detected.\n" )

	//	Check, if value of property will changed in method "setFastPropertyValue_NoBroadcast()".
	//	Return TRUE, if changed - else return FALSE.
	//	Attention:
	//		Method "impl_tryToChangeProperty()" can throw the IllegalArgumentException !!!

	//	Initialize state with FALSE !!!
	//	(Handle can be invalid)
	sal_Bool bReturn = sal_False;

	switch ( nHandle )
	{
		case PROPERTYHANDLE_ISPLUGIN			:
		case PROPERTYHANDLE_TITLE				:	bReturn = sal_False; // These variables are readonly(!) and can't change.
													break;
		case PROPERTYHANDLE_ISALWAYSVISIBLE		:	bReturn = impl_tryToChangeProperty( m_bIsAlwaysVisible	, aValue, aOldValue, aConvertedValue );
													break;
		case PROPERTYHANDLE_ISFLOATING	   		:	bReturn = impl_tryToChangeProperty( m_bIsFloating		, aValue, aOldValue, aConvertedValue );
													break;
		case PROPERTYHANDLE_POSITION			:	bReturn = impl_tryToChangeProperty( m_aPosition			, aValue, aOldValue, aConvertedValue );
													break;
		case PROPERTYHANDLE_SIZE				:	bReturn = impl_tryToChangeProperty( m_aSize				, aValue, aOldValue, aConvertedValue );
													break;
	}

	// Return state of operation.
	return bReturn ;
}

//*****************************************************************************************************************
//	OPropertySetHelper
//*****************************************************************************************************************
void SAL_CALL Task::setFastPropertyValue_NoBroadcast(			sal_Int32	nHandle	,
			  											const	Any&		aValue	) throw( Exception )
{
	// Ready for multithreading
	LOCK_MUTEX( aGuard, m_aMutex, "Task::setFastPropertyValue_NoBroadcast()" )

	//	Safe impossible cases
	//	OprpertySetHelper say	: "Handle and type of any value are valid at everyway!".
	//	But I say				: "It's better to control this."
//	LOG_ASSERT ( impl_debug_checkParameter ( nHandle, rValue ), "Task::setFastPropertyValue_NoBroadcast()\nInvalid parameter detected.\n" )

	// Search for right handle ... and try to set property value.
	switch ( nHandle )
	{
		case PROPERTYHANDLE_ISPLUGIN			:
		case PROPERTYHANDLE_TITLE				:	LOG_ASSERT( sal_False, "Task::setFastPropertyValue_NoBroadcast()\nSet of readonly property not allowed.\n" )
													break;
		case PROPERTYHANDLE_ISALWAYSVISIBLE		:	aValue >>= m_bIsAlwaysVisible	;
													break;
		case PROPERTYHANDLE_ISFLOATING	   		:	aValue >>= m_bIsFloating		;
													break;
		case PROPERTYHANDLE_POSITION			:	aValue >>= m_aPosition			;
													break;
		case PROPERTYHANDLE_SIZE				:	aValue >>= m_aSize				;
													break;
	}
}

//*****************************************************************************************************************
//	OPropertySetHelper
//*****************************************************************************************************************
void SAL_CALL Task::getFastPropertyValue(	Any&		aValue	,
			  								sal_Int32	nHandle ) const
{
	// Ready for multithreading
	LOCK_GLOBALMUTEX( aGuard, "Task::getFastPropertyValue()" )

	//	Safe impossible cases
	//	OprpertySetHelper say	: "Handle and type of any value are valid at everyway!".
	//	But I say				: "It's better to control this."
//	LOG_ASSERT ( impl_debug_checkParameter ( nHandle, rValue ), "Task::setFastPropertyValue_NoBroadcast()\nInvalid parameter detected.\n" )

	// Search for right handle ... and try to set property value.
	switch ( nHandle )
	{
		case PROPERTYHANDLE_ISALWAYSVISIBLE		:	aValue <<= m_bIsAlwaysVisible	;
													break;
		case PROPERTYHANDLE_ISFLOATING	   		:	aValue <<= m_bIsFloating		;
													break;
		case PROPERTYHANDLE_ISPLUGIN			:	aValue <<= m_bIsPlugIn			;
													break;
		case PROPERTYHANDLE_POSITION			:	aValue <<= m_aPosition			;
													break;
		case PROPERTYHANDLE_SIZE				:	aValue <<= m_aSize				;
													break;
		case PROPERTYHANDLE_TITLE				:	aValue <<= m_sTitle				;
													break;
	}
}

//*****************************************************************************************************************
//	OPropertySetHelper
//*****************************************************************************************************************
IPropertyArrayHelper& SAL_CALL Task::getInfoHelper()
{
	// Optimize this method !
	// We initialize a static variable only one time. And we don't must use a mutex at every call!
	// For the first call; pInfoHelper is NULL - for the second call pInfoHelper is different from NULL!
	static OPropertyArrayHelper* pInfoHelper = NULL;

	if ( pInfoHelper == NULL )
	{
		// Ready for multithreading
		LOCK_GLOBALMUTEX( aGuard, "Task::getInfoHelper()" )
		// Control this pointer again, another instance can be faster then these!
		if ( pInfoHelper == NULL )
		{
			// Define static member to give structure of properties to baseclass "OPropertySetHelper".
			// "impl_getStaticPropertyDescriptor" is a non exported and static funtion, who will define a static propertytable.
			// "sal_True" say: Table is sorted by name.
			static OPropertyArrayHelper aInfoHelper( impl_getStaticPropertyDescriptor(), sal_True );
			pInfoHelper = &aInfoHelper;
		}
	}

	return (*pInfoHelper);
}

//*****************************************************************************************************************
//	OPropertySetHelper
//*****************************************************************************************************************
Reference< XPropertySetInfo > SAL_CALL Task::getPropertySetInfo ()
{
	// Optimize this method !
	// We initialize a static variable only one time. And we don't must use a mutex at every call!
	// For the first call; pInfo is NULL - for the second call pInfo is different from NULL!
	static Reference< XPropertySetInfo >* pInfo = NULL ;

	if ( pInfo == NULL )
	{
		// Ready for multithreading
		LOCK_GLOBALMUTEX( aGuard, "Task::getPropertySetInfo()" )
		// Control this pointer again, another instance can be faster then these!
		if ( pInfo == NULL )
		{
			// Create structure of propertysetinfo for baseclass "OPropertySetHelper".
			// (Use method "getInfoHelper()".)
			static Reference< XPropertySetInfo > xInfo( createPropertySetInfo( getInfoHelper() ) );
			pInfo = &xInfo;
		}
	}

	return (*pInfo);
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
sal_Bool Task::impl_tryToChangeProperty(			sal_Bool	bProperty		,
											const	Any&		aValue			,
													Any&		aOldValue		,
													Any&		aConvertedValue	) throw( IllegalArgumentException )
{
	// Set default return value.
	sal_Bool bReturn = sal_False;

	// Clear information of return parameter!
	aOldValue.clear();
	aConvertedValue.clear();

	// Get new value from any.
	// IllegalArgumentException() can be thrown!
	sal_Bool bNewValue;
	convertPropertyValue( bNewValue, aValue );

	// If value change ...
	if ( bNewValue != bProperty )
	{
		// ... set information of change.
		// Attention:	Use setValue and getCppuBooleanType to set value in any!
		//				It's necessary, if BOOL can be int or char.
		aOldValue.setValue		( &bProperty, ::getCppuBooleanType() );
		aConvertedValue.setValue( &bNewValue, ::getCppuBooleanType() );
		// Return OK - "value will be change ..."
		bReturn = sal_True;
	}

	return bReturn;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
sal_Bool Task::impl_tryToChangeProperty(	const	OUString&	sProperty		,
											const	Any&		aValue			,
													Any&		aOldValue		,
													Any&		aConvertedValue	) throw( IllegalArgumentException )
{
	// Set default return value.
	sal_Bool bReturn = sal_False;

	// Clear information of return parameter!
	aOldValue.clear();
	aConvertedValue.clear();

	// Get new value from any.
	// IllegalArgumentException() can be thrown!
	OUString sNewValue;
	convertPropertyValue( sNewValue, aValue );

	// If value change ...
	if ( sNewValue != sProperty )
	{
		// ... set information of change.
		aOldValue.setValue		( &sProperty, ::getCppuType((const OUString*)NULL) );
		aConvertedValue.setValue( &sNewValue, ::getCppuType((const OUString*)NULL) );
		// Return OK - "value will be change ..."
		bReturn = sal_True;
	}

	return bReturn;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
sal_Bool Task::impl_tryToChangeProperty(	const	Point&		aProperty		,
											const	Any&		aValue			,
													Any&		aOldValue		,
													Any&		aConvertedValue	) throw( IllegalArgumentException )
{
	// Set default return value.
	sal_Bool bReturn = sal_False;

	// Clear information of return parameter!
	aOldValue.clear();
	aConvertedValue.clear();

	// Get new value from any.
	// IllegalArgumentException() can be thrown!
	Point aNewValue;
	convertPropertyValue( aNewValue, aValue );

	// If value change ...
	if	(
			( aNewValue.X	!=	aProperty.X	)	||
			( aNewValue.Y	!=	aProperty.Y	)
		)
	{
		// ... set information of change.
		aOldValue.setValue		( &aProperty, ::getCppuType((const Point*)NULL) );
		aConvertedValue.setValue( &aNewValue, ::getCppuType((const Point*)NULL) );
		// Return OK - "value will be change ..."
		bReturn = sal_True;
	}

	return bReturn;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
sal_Bool Task::impl_tryToChangeProperty(	const	Size&		aProperty		,
											const	Any&		aValue			,
													Any&		aOldValue		,
													Any&		aConvertedValue	) throw( IllegalArgumentException )
{
	// Set default return value.
	sal_Bool bReturn = sal_False;

	// Clear information of return parameter!
	aOldValue.clear();
	aConvertedValue.clear();

	// Get new value from any.
	// IllegalArgumentException() can be thrown!
	Size aNewValue;
	convertPropertyValue( aNewValue, aValue );

	// If value change ...
	if	(
			( aNewValue.Width	!=	aProperty.Width		)	||
			( aNewValue.Height	!=	aProperty.Height	)
		)
	{
		// ... set information of change.
		aOldValue.setValue		( &aProperty, ::getCppuType((const Size*)NULL) );
		aConvertedValue.setValue( &aNewValue, ::getCppuType((const Size*)NULL) );
		// Return OK - "value will be change ..."
		bReturn = sal_True;
	}

	return bReturn;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
const Sequence< Property > Task::impl_getStaticPropertyDescriptor()
{
	// Create a new static property array to initialize sequence!
	// Table of all predefined properties of this class. Its used from OPropertySetHelper-class!
	// Don't forget to change the defines (see begin of this file), if you add, change or delete a property in this list!!!
	// It's necessary for methods of OPropertySetHelper.
	// ATTENTION:
	//		YOU MUST SORT FOLLOW TABLE BY NAME !!!

	static const Property pPropertys[] =
	{
		Property( PROPERTYNAME_ISALWAYSVISIBLE	, PROPERTYHANDLE_ISALWAYSVISIBLE, ::getBooleanCppuType()				, PropertyAttribute::TRANSIENT									),
		Property( PROPERTYNAME_ISFLOATING		, PROPERTYHANDLE_ISFLOATING		, ::getBooleanCppuType()				, PropertyAttribute::TRANSIENT									),
		Property( PROPERTYNAME_ISPLUGIN			, PROPERTYHANDLE_ISPLUGIN		, ::getBooleanCppuType()				, PropertyAttribute::TRANSIENT | PropertyAttribute::READONLY	),
		Property( PROPERTYNAME_POSITION			, PROPERTYHANDLE_POSITION		, ::getCppuType((const Point*	)NULL)	, PropertyAttribute::TRANSIENT									),
		Property( PROPERTYNAME_SIZE				, PROPERTYHANDLE_SIZE			, ::getCppuType((const Size*	)NULL)	, PropertyAttribute::TRANSIENT									),
		Property( PROPERTYNAME_TITLE			, PROPERTYHANDLE_TITLE			, ::getCppuType((const OUString*)NULL)	, PropertyAttribute::TRANSIENT | PropertyAttribute::READONLY	),
	};
	// Use it to initialize sequence!
	static const Sequence< Property > seqPropertyDescriptor( pPropertys, PROPERTYCOUNT );
	// Return static "PropertyDescriptor"
	return seqPropertyDescriptor ;
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef ENABLE_ASSERTIONS
/*
//*****************************************************************************************************************
sal_Bool Task::impldbg_checkParameter_append( const Reference< XTask >& xTask )
{
	// Set default return value.
	sal_Bool bOK = sal_True;

	// Check parameter.
	if	(
			( &xTask		==	NULL		)	||
			( xTask.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}

	// Return result of check.
	return bOK ;
}
*/
#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
