/*************************************************************************
 *
 *  $RCSfile: agapidll.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif
#include <tools/urlobj.hxx>
#include <svtools/libcall.hxx>

#include "chanapi.hxx"
#include "chlist.hxx"
#include "chagent.hxx"
#include "agapidll.hxx"

////////////////////////////////////////////////////////////////////////////////
//	Channel Api
//

void _SV_CALL ChShutDownAgent(AgentInst pAg)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->ShutDownAgent();
}

void _SV_CALL ChSetLastSuccUpd(AgentInst pAg)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetLastSuccUpd();
}

void _SV_CALL ChGetChannelObj(AgentInst pAg, const char* pURL,
							  AgentStreamType eStreamType, const char* pFileName)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	RequestType eType;

	if( eStreamType == AS_LOCAL_FILE )
		eType = REQTYP_LOCAL_FILE;
	else if( eStreamType == AS_MEMORY )
		eType = REQTYP_MEMORY;

	pAgent->GetChApi()->GetChannelObject(INetURLObject(pURL), eType, pFileName);
}

void _SV_CALL ChAddChannelItem(AgentInst pAg, const char* pName, const char* pTransmitter,
	const char* pChannel, unsigned short nUpdPeriode, const char* pAgentName )
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->AddChannelItem(pName, INetURLObject(pTransmitter),
		pChannel, nUpdPeriode, pAgentName);
}

void _SV_CALL ChDelChannelItem(AgentInst pAg, const char* pChName)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->DelChannelItem(pChName);
}

void _SV_CALL ChSetTransmitter(AgentInst pAg, const char* pChName,
	const char* pURL)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetChTransmitter(pChName, pURL);
}

void _SV_CALL ChSetChannel(AgentInst pAg, const char* pChName,
	const char* pRelURL)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetChannel(pChName, pRelURL);
}

void _SV_CALL ChSetChannelName(AgentInst pAg, const char* pChName,
	const char* pNewChName)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetChannelName(pChName, pNewChName);
}

void _SV_CALL ChSetUpdPeriode(AgentInst pAg, const char* pChName,
	unsigned short nUpdPeriode)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetChUpdPeriode(pChName, nUpdPeriode);
}

void _SV_CALL ChSetChannelAgentName(AgentInst pAg, const char* pChName,
	const char* pAgName)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetChannelAgentName(pChName, pAgName);
}

void _SV_CALL ChSetUpdateTransmitter(AgentInst pAg, const char* pTransmitter )
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	pAgent->GetChApi()->SetUpdateTransmitter(pAgent, INetURLObject(pTransmitter));
}

const char* _SV_CALL ChGetAgentSourceURL(AgentInst pAg)
{
	ChannelAgentItem* pAgent = (ChannelAgentItem*)pAg;
	return pAgent->GetAgentSourceURL().GetMainURL().GetStr();
}


////////////////////////////////////////////////////////////////////////////////
//	Agent Api
//

AgentApiDll::AgentApiDll(ChannelAgentItem* pAgent) :
	AgentApi( pAgent )
{
	pAgentDll = NULL;

	aChannelApiFncs.nVersion 				= AGSDKDLL_VERSION;

	// Ver 1
	aChannelApiFncs.fncShutDownAgent		= (FncChShutDownAgent)
											  ChShutDownAgent;
	aChannelApiFncs.fncSetLastSuccUpd		= (FncChSetLastSuccUpd)
											  ChSetLastSuccUpd;
	aChannelApiFncs.fncGetChannelObj		= (FncChGetChannelObj)
											  ChGetChannelObj;
	aChannelApiFncs.fncAddChannelItem		= (FncChAddChannelItem)
											  ChAddChannelItem;
	aChannelApiFncs.fncDelChannelItem		= (FncChDelChannelItem)
											  ChDelChannelItem;
	aChannelApiFncs.fncSetTransmitter		= (FncChSetTransmitter)
											  ChSetTransmitter;
	aChannelApiFncs.fncSetChannel 			= (FncChSetChannel)
											  ChSetChannel;
	aChannelApiFncs.fncSetChannelName		= (FncChSetChannelName)
											  ChSetChannelName;
	aChannelApiFncs.fncSetUpdPeriode		= (FncChSetUpdPeriode)
											  ChSetUpdPeriode;
	aChannelApiFncs.fncSetChannelAgentName	= (FncChSetChannelAgentName)
											  ChSetChannelAgentName;
	aChannelApiFncs.fncSetUpdateTransmitter = (FncChSetUpdateTransmitter)
											  ChSetUpdateTransmitter;

	// Ver 2
	aChannelApiFncs.fncGetAgentSourceURL	= (FncChGetAgentSourceURL)
											  ChGetAgentSourceURL;
}

AgentApiDll::~AgentApiDll()
{
	if( pAgentDll )
		delete pAgentDll;
	pChannelAgent->SetIsActive(FALSE);
}

BOOL AgentApiDll::StartAgent()
{
	String aFileName = pChannelAgent->GetLocation().PathToFileName();

	DirEntry aOldCWD;
	aOldCWD.ToAbs();
	DirEntry aDllPath( aFileName );

	aDllPath.SetCWD(TRUE);
	pAgentDll = new Library( aFileName );
	aOldCWD.SetCWD(TRUE);

	if( !pAgentDll ) {
		DBG_ASSERT( FALSE, "konnte ChannelAgent DLL nicht laden" );
		return FALSE;
	}

	#if defined(OS2) || defined(WIN)
	fncInitAgent			= (FncInitAgent)
							  pAgentDll->GetFunction( "_InitAgent" );
	fncShutDownAgent		= (FncShutDownAgent)
							  pAgentDll->GetFunction( "_ShutDownAgent" );
	fncNewDataPermission	= (FncNewDataPermission)
							  pAgentDll->GetFunction( "_NewDataPermission" );
	fncNewData				= (FncNewData)
							  pAgentDll->GetFunction( "_NewData" );
	fncNotifyChannelObjFile = (FncNotifyChannelObjFile)
							  pAgentDll->GetFunction( "_NotifyChannelObjFile" );
	fncNotifyChannelObjData	= (FncNotifyChannelObjData)
							  pAgentDll->GetFunction( "_NotifyChannelObjData" );
	fncRegisterChannels		= (FncRegisterChannels)
							  pAgentDll->GetFunction( "_RegisterChannels" );
	fncRegisterUpdateTransmitter = (FncRegisterUpdateTransmitter)
							  pAgentDll->GetFunction( "_RegisterUpdateTransmitter" );
	#else
	fncInitAgent			= (FncInitAgent)
							  pAgentDll->GetFunction( "InitAgent" );
	fncShutDownAgent		= (FncShutDownAgent)
							  pAgentDll->GetFunction( "ShutDownAgent" );
	fncNewDataPermission	= (FncNewDataPermission)
							  pAgentDll->GetFunction( "NewDataPermission" );
	fncNewData				= (FncNewData)
							  pAgentDll->GetFunction( "NewData" );
	fncNotifyChannelObjFile = (FncNotifyChannelObjFile)
							  pAgentDll->GetFunction( "NotifyChannelObjFile" );
	fncNotifyChannelObjData	= (FncNotifyChannelObjData)
							  pAgentDll->GetFunction( "NotifyChannelObjData" );
	fncRegisterChannels		= (FncRegisterChannels)
							  pAgentDll->GetFunction( "RegisterChannels" );
	fncRegisterUpdateTransmitter = (FncRegisterUpdateTransmitter)
							  pAgentDll->GetFunction( "RegisterUpdateTransmitter" );
	#endif


	if( !fncInitAgent || !fncShutDownAgent || !fncNewDataPermission ||
		!fncNewData || !fncNotifyChannelObjFile ||!fncNotifyChannelObjData ||
		!fncRegisterChannels || !fncRegisterUpdateTransmitter )
	{
		if( pAgentDll )
			delete pAgentDll;
		pAgentDll = NULL;
		return FALSE;
	}

	pChannelAgent->SetIsActive(TRUE);
	return TRUE;
}

void AgentApiDll::InitAgent()
{
	DBG_ASSERT( fncInitAgent, "!fncInitAgent AgentApiDLL" );
	fncInitAgent( (AgentInst)pChannelAgent, &aChannelApiFncs );
}

void AgentApiDll::ShutDownAgent()
{
	DBG_ASSERT( fncShutDownAgent, "!fncShutDownAgent AgentApiDLL" );
	fncShutDownAgent();

	delete pAgentDll;
	pAgentDll = NULL;
	pChannelAgent->SetIsActive(FALSE);
}

BOOL AgentApiDll::NewDataPermission(const String& rChannelName)
{
	DBG_ASSERT( fncNewDataPermission, "!fncNewDataPermission AgentApiDLL" );
	BOOL bRet = fncNewDataPermission( rChannelName );
	return bRet;
}

void AgentApiDll::NewData(const String& rChannelName, const INetURLObject& rURL)
{
	DBG_ASSERT( fncNewData, "!fncNewData AgentApiDLL" );
	fncNewData( rChannelName, rURL.GetMainURL() );
}

void AgentApiDll::NotifyChannelObjFile(const INetURLObject& rURL, const String& rFileName)
{
	DBG_ASSERT( fncNotifyChannelObjFile, "!fncNotifyChannelObjFile AgentApiDLL" );
	fncNotifyChannelObjFile(rURL.GetMainURL(), rFileName);
}

void AgentApiDll::NotifyChannelObjData(const INetURLObject& rURL,
	void* pBuffer, long nOffset, long nLen, long nTotalLen)
{
	DBG_ASSERT( fncNotifyChannelObjData, "!fncNotifyChannelObjData AgentApiDLL" );
	fncNotifyChannelObjData(rURL.GetMainURL(), pBuffer, nOffset, nLen, nTotalLen);
}

void AgentApiDll::RegisterChannels()
{
	DBG_ASSERT( fncRegisterChannels, "!fncRegisterChannels AgentApiDLL" );
	fncRegisterChannels((AgentInst)pChannelAgent, &aChannelApiFncs);
}

void AgentApiDll::RegisterUpdateTransmitter()
{
	DBG_ASSERT( fncRegisterUpdateTransmitter, "!fncRegisterUpdateTransmitter AgentApiDLL" );
	fncRegisterUpdateTransmitter((AgentInst)pChannelAgent, &aChannelApiFncs);
}
