/*************************************************************************
 *
 *  $RCSfile: chapi.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <tools/urlobj.hxx>

#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif
#include "channel.hxx"
#include "chlist.hxx"
#include "chagent.hxx"
#include "getchobj.hxx"

#include "tools/agapi.hxx"
#include "chanapi.hxx"

#define MAX_BUF		32000

////////////////////////////////////////////////////////////////////////////////
//
//
ChannelApi::ChannelApi(	ChannelAgentItem* pAgent, INetSession* pISess,
						ChannelList* pChLst )
{
	xINetSession	= pISess;
	pChAgent		= pAgent;
	pChannelList	= pChLst;

	pGetChObjList	= new GetChObjList;
}

////////////////////////////////////////////////////////////////////////////////
//
//
ChannelApi::~ChannelApi()
{
	for( USHORT i = 0; i < pGetChObjList->Count(); ++i )
	{
		GetChannelObj* pObj = pGetChObjList->GetObject(i);
		delete pObj;
	}
	delete pGetChObjList;
	pGetChObjList = NULL;

	if( xINetSession )
		xINetSession.Clear();
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::ShutDownAgent()
{
	Application::PostUserEvent(LINK(this, ChannelApi, CloseAgentEvt));
}

IMPL_LINK(ChannelApi, CloseAgentEvt, void*, pPtr )
{
	ChannelAgentItem* pAgent = pChAgent;  // damit er auf dem Stack steht
										  // wir  loeschen uns gleich selber!!
	pAgent->GetApi()->ShutDownAgent();
	pAgent->SetIsActive( FALSE );
	pAgent->DeleteApis();
	pAgent->GetBTXShutDownCallBack().Call( NULL );

	return 1;
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetLastSuccUpd()
{
	pChannelList->SetLastSuccUpdChannel(pChAgent);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::GetChannelObject( const INetURLObject& rURL, RequestType eStreamType,
	const String& rFileName )
{
	if( (rURL.GetProtocol() != INET_PROT_FIF &&
		rURL.GetProtocol() != INET_PROT_CEPT &&
		rURL.GetProtocol() != INET_PROT_FILE) && !xINetSession )
		return; // keine Verbindung zum Internet

	GetChannelObj* pGetObj = new GetChannelObj(xINetSession);
	pGetChObjList->Insert( pGetObj, LIST_APPEND );

	if( rURL.GetProtocol() == INET_PROT_FIF ||
		rURL.GetProtocol() == INET_PROT_CEPT )
		pGetObj->SetBTXDecoder( pChannelList->GetBTXDecoder() );

	pGetObj->SetCallBackHdl(LINK(this,ChannelApi,GetChannelObjNotify));

	if( eStreamType == REQTYP_LOCAL_FILE )
	{
		if( !rFileName ) {
			DBG_ASSERT( !this, "GetChannelObject: Stream AS_LOCAL_FILE aber kein FileName" );
			return;
		}
		pGetObj->Download(rURL, rFileName, TRUE);
	}
	else if( eStreamType == REQTYP_MEMORY )
	{
		pGetObj->Download(rURL, "", TRUE);
	}
	else
	{
		DBG_ASSERT( !this, "GetChannelObject: unbekannter StreamType" );
	}
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::AddChannelItem( const String& aChName, const INetURLObject& aTransmitter,
	const String& aChannel, USHORT nUpdPeriode, const String& rChAgentName )
{
	if( pChannelList->GetChannelItemByName(aChName) )
		return;

	ChannelItem* pNew = new ChannelItem;

	pNew->SetTransmitter		( aTransmitter	);
	pNew->SetChannel			( aChannel		);
	pNew->SetChannelName		( aChName		);
	pNew->SetChannelAgentName	( rChAgentName	);
	pNew->SetUpdatePeriode 		( nUpdPeriode	);
	pNew->SetLastTimerTick		( 0L );
	pNew->SetLastSuccUpdDate	( 0L );
	pNew->SetLastSuccUpdTime	( 0L );

	pChannelList->RegisterNewChannelItem( pNew );
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::DelChannelItem( const String& aChName )
{
	ChannelItem* pItem = pChannelList->GetChannelItemByName( aChName );
	if( !pItem )
		return;
	pChannelList->DeleteChannelItem(pItem);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetChTransmitter( const String& aChName, const String& rNewVal )
{
	ChannelItem* pItem = pChannelList->GetChannelItemByName( aChName );
	if( !pItem )
		return;
	pItem->SetTransmitter(rNewVal);
	pChannelList->WriteChannelItem(pItem);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetChannel( const String& aChName, const String& rNewVal )
{
	ChannelItem* pItem = pChannelList->GetChannelItemByName( aChName );
	if( !pItem )
		return;
	pItem->SetChannel(rNewVal);
	pChannelList->WriteChannelItem(pItem);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetChannelName( const String& aChName, const String& rNewVal )
{
	ChannelItem* pItem = pChannelList->GetChannelItemByName( aChName );
	if( !pItem )
		return;

	ChannelItem* pNew = new ChannelItem;
	pNew->SetTransmitter		( pItem->GetTransmitter()		);
	pNew->SetChannel			( pItem->GetChannel()			);
	pNew->SetChannelName		( rNewVal						);
	pNew->SetChannelAgentName	( pItem->GetChannelAgentName()	);
	pNew->SetUpdatePeriode 		( pItem->GetUpdatePeriode()		);
	pNew->SetLastTimerTick		( pItem->GetLastTimerTick()	 	);
	pNew->SetLastSuccUpdDate	( pItem->GetLastSuccUpdDate()	);
	pNew->SetLastSuccUpdTime	( pItem->GetLastSuccUpdTime()	);
	pNew->SetAgentUpdChannel	( pItem->GetAgentUpdChannel()	);

	pChannelList->DeleteChannelItem(pItem);
	pChannelList->RegisterNewChannelItem(pNew);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetChUpdPeriode( const String& aChName, USHORT nUpdPeriode )
{
	ChannelItem* pItem = pChannelList->GetChannelItemByName( aChName );
	if( !pItem )
		return;
	pItem->SetUpdatePeriode(nUpdPeriode);
	pChannelList->WriteChannelItem(pItem);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetChannelAgentName( const String& aChName, const String& rNewVal )
{
	ChannelItem* pItem = pChannelList->GetChannelItemByName( aChName );
	if( !pItem )
		return;
	pItem->SetChannelAgentName(rNewVal);
	pChannelList->WriteChannelItem(pItem);
}

////////////////////////////////////////////////////////////////////////////////
//
//
void ChannelApi::SetUpdateTransmitter( ChannelAgentItem* pAgent,
									   const INetURLObject& rTransmitter)
{
	pAgent->SetSource(rTransmitter);
	pChannelList->WriteAgentItem(pAgent);
 }

////////////////////////////////////////////////////////////////////////////////
//
//
IMPL_LINK(ChannelApi, CloseChannelObjEvt, void*, pTmp)
{
	if( !pGetChObjList )
		return 0;

	for( USHORT i = 0; i < pGetChObjList->Count(); ++i )
	{
		GetChannelObj* pObj = pGetChObjList->GetObject(i);
		if( !pObj )
			return 0;

		if( pObj->CanCloseIt() )
		{
			pGetChObjList->Remove(i);
			delete pObj;
			break;
		}
	}
	return 1;
}

IMPL_LINK(ChannelApi, GetChannelObjNotify, GetChannelObj*, pGetChObj)
{
	for( USHORT i = 0; i < pGetChObjList->Count(); ++i )
		if( pGetChObjList->GetObject(i) == pGetChObj )
		{
			GetChannelObj* pObj = pGetChObjList->GetObject(i);

			if( pObj->IsSuccess() )
			{
				pObj->SetCloseIt( TRUE );
				Application::PostUserEvent(LINK(this, ChannelApi, CloseChannelObjEvt));

				// war es ein File- oder MemoryStream
				if( pObj->GetLocalFile().Len() )
				{
					pChAgent->GetApi()->NotifyChannelObjFile(
						pObj->GetURL(),
						pObj->GetLocalFile());
				}
				else
				{
					SvStream aStrm(pObj->GetData());
					aStrm.Seek(STREAM_SEEK_TO_END);
					ULONG nTotalSize = aStrm.Tell();
					aStrm.Seek(STREAM_SEEK_TO_BEGIN);

					char* pBuffer = new char [MAX_BUF];
					ULONG nOffset = 0;

					while( nOffset < nTotalSize )
					{
						memset( pBuffer, 0, MAX_BUF );
						ULONG nRead = aStrm.Read( pBuffer, MAX_BUF );

						pChAgent->GetApi()->NotifyChannelObjData( pObj->GetURL(),
								pBuffer, nOffset, nRead, nTotalSize );

						nOffset += nRead;
					}

					delete pBuffer;
				}
			}
			else
			{
				pChannelList->CancelBTX();
				pChAgent->SetBTXShutDownCallBack(Link());
				ShutDownAgent();

				DBG_ASSERT( !this, "und wer teilt dem Agent den Fehler mit" );
			}
			break;
		}

	return 1;
}

