/*************************************************************************
 *
 *  $RCSfile: chtuner.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef MAC
#ifdef BUILD
#undef BUILD
#endif
#endif
#include <prj.hrc>

#ifndef _TOOLS_RESID_HXX //autogen
#include <tools/resid.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#include <inet/inetmain.hxx>
#include <tools/download.hxx>		            // BTX

#include "chtuner.hxx"
#include "chlist.hxx"
#include "channel.hxx"

#include "strings.hrc"

#define INET_CONN(x)   		((INetConnection*)&x)
#define FTP_CONN(x)			((INetFtpConnection*)&x)
#define HTTP_CONN(x)	   	((INetHttpConnection*)&x)
#define HTTP_GET_REQ(x)   	((INetHttpGetRequest*)&x)

#define DEF_RETRY_INTERVAL	60000 	    // Millisek.
#define DEF_RETRY_DIFF		60 		    // Sek.

ChannelTuner::ChannelTuner(INetSession* pSess, ChannelList* pChList)
{
	pBTXDecoder		= NULL;
	xINetSession	= pSess;
	pChannelList	= pChList;
	pReqList		= new ReqList;
	pRetryList		= NULL;

	aRetryTimer.SetTimeoutHdl( LINK(this,ChannelTuner,RetryCallBack) );
	aRetryTimer.SetTimeout( DEF_RETRY_INTERVAL );
}

ChannelTuner::~ChannelTuner()
{
	if( pRetryList )
	{
		for( USHORT nListIdx = 0; nListIdx < pRetryList->Count(); ++nListIdx )
			delete pRetryList->GetObject( nListIdx );
		delete pRetryList;
	}

	DBG_ASSERT( !pReqList->Count(), "es haengen noch Requests am ChannelTuner");
	for( USHORT i = 0; i < pReqList->Count(); ++i )
	{
		INetRequest* pItem = pReqList->GetObject(i);
		((INetRequestRef)pItem).Clear();
	}
	delete pReqList;

	if( xINetSession )
		xINetSession.Clear();
}

void ChannelTuner::SetINetSession(INetSession* pNewSess)
{
	xINetSession = pNewSess;
}

void ChannelTuner::SetBTXDecoder(Downloader* pDecoder)
{
	pBTXDecoder	= pDecoder;
}

void ChannelTuner::NotifyChannelList(String& rStr)
{
	rStr.EraseTrailingChars( 0x0a );
	rStr.EraseTrailingChars( 0x0d );
	pChannelList->NotifyTransmitterMsg(rStr);
}

BOOL ChannelTuner::RequestChannel(ChannelItem* pItem, const Link& rBTXCallBack)
{
	DBG_ASSERT( pItem, "UUps, kein ChannelItem in RequestChannel" );

	INetURLObject aNew, aObj = pItem->GetTransmitter();
	aObj.GetNewAbsURL( pItem->GetChannel(), &aNew );

	if( aNew.HasError() )
		return FALSE;

	aBTXCallBack = rBTXCallBack;

	switch( aNew.GetProtocol() )
	{
		// Internet
		case INET_PROT_HTTP :
		case INET_PROT_HTTPS :
			SendHttpReq( aNew );
			break;
		case INET_PROT_FTP :
			SendFtpReq( aNew );
			break;

		// btx
		case INET_PROT_FIF :
		case INET_PROT_CEPT :
		{
			if( !pItem->GetAgentUpdChannel() )
			{
				if( !pItem->GetChannel().Len() )
					SendBTXReq( aObj );
				else
					SendBTXReq( aNew );
			} else
			{
				String aHackStr( aNew.GetMainURL() );
				if( aHackStr.ICompare("fif:///", 7) == COMPARE_EQUAL )
					aHackStr.Cut(5, 1);
				INetURLObject aHackURL( aHackStr );
				SendBTXReq( aHackURL );
			}
			break;
		}

		case INET_PROT_FILE :
			GetFileTrans( aNew );
			break;

		default:
			DBG_ASSERT( !this, "protocol ist im ChannelTuner nicht impl." );
			return FALSE;
	}
	return TRUE;
}

void ChannelTuner::GetFileTrans( const INetURLObject& rURL )
{
	SvFileStream aStrm( rURL.PathToFileName(), STREAM_STD_READ );
	aStrm.Seek( STREAM_SEEK_TO_END );
	ULONG nLen = aStrm.Tell();
	aStrm.Seek( STREAM_SEEK_TO_BEGIN );

	char* cBuffer = new char[nLen];
	ULONG numRead = aStrm.Read( cBuffer, nLen );
	String aMsg( cBuffer, (USHORT)numRead);
	delete cBuffer;

	NotifyChannelList(aMsg);
}

void ChannelTuner::SendBTXReq( const INetURLObject& rURL )
{
	if( !pBTXDecoder )
		return;

	String aURLPath 	= rURL.GetMainURL();
	aBTXFileName = rURL.GetLastName();

	// DateiNamen abschneiden.
	aURLPath.Cut( aURLPath.Len() - aBTXFileName.Len() - 1 );

	pBTXDecoder->Download(
		aBTXFileName,
		aURLPath,
		LINK(this,ChannelTuner,BTXCallBack)
	);
}

IMPL_LINK(ChannelTuner, BTXCallBack, void*, pStr)
{
	ULONG nBTXError = (ULONG) pStr;
	if( nBTXError != DOWNLOAD_SUCCESS )
	{
		pChannelList->CancelBTX();
		return 0;
	}

	SvFileStream aStrm;
	aStrm.Open(aBTXFileName, STREAM_STD_READ);

	if( !aStrm.IsOpen() )
	{
		DBG_ASSERT( !this, "BTX-File laesst sich nich oeffnen" );
		return 0;
	}

	String aTransmitterMsg;
	aStrm.ReadLine( aTransmitterMsg );
	aStrm.Close();

	NotifyChannelList( aTransmitterMsg );
	aBTXCallBack.Call( &aBTXFileName );

	return 1;
}

void ChannelTuner::SendHttpReq( const INetURLObject& rURL )
{
	String aParamVersionStr( SYSTEM_ID );
	aParamVersionStr += ",";
	aParamVersionStr += String( ResId(STR_CHA_TUNER_LANGUAGECODE, pChannelList->GetResMgr()) );
	aParamVersionStr += ",";
	aParamVersionStr += UPD;
	aParamVersionStr += ",";
	aParamVersionStr += MINOR;
	aParamVersionStr += ",";
	aParamVersionStr += BUILD;

	INetURLObject aTempURL( rURL );
	aTempURL.SetParam( aParamVersionStr );

	INetConnectionRef xConnection = xINetSession->CreateHttpConnection();

	HTTP_CONN(xConnection)->SetServerName(aTempURL.GetHost());
	HTTP_CONN(xConnection)->SetServerPort(aTempURL.HasPort()?
							aTempURL.GetPort() : INET_DEF_HTTP_PORT);

	INetRequestRef xReq = HTTP_CONN(xConnection)->CreateHttpGetRequest();
	xReq->AddRef();
	HTTP_GET_REQ(xReq)->SetPrio(10);
	HTTP_GET_REQ(xReq)->SetCacheFlag( FALSE );

	if( aTempURL.GetProtocol() == INET_PROT_FTP )
		HTTP_GET_REQ(xReq)->SetProtocol(INET_PROT_FTP);

	if( aTempURL.HasUserData())
	{
		HTTP_GET_REQ(xReq)->SetUsername(aTempURL.GetUser());
		HTTP_GET_REQ(xReq)->SetPassword(aTempURL.GetPass());
	}
	if(aTempURL.HasRealm())
		HTTP_GET_REQ(xReq)->SetRealm( aTempURL.GetRealm() );
	HTTP_GET_REQ(xReq)->SetTarget(aTempURL.GetTarget());

	pReqList->Insert(xReq);
	StartListening(*(HTTP_GET_REQ(xReq)));
	HTTP_CONN(xConnection)->Start();
}

void ChannelTuner::SendFtpReq( const INetURLObject& rURL )
{
	DBG_ASSERT( !this, "Sorry, NIY (FTP)" );
}

void ChannelTuner::RemoveRequest(INetRequest* pReq)
{
	for( USHORT i = 0; i < pReqList->Count(); ++i )
	{
		INetRequest* pLstReq = pReqList->GetObject(i);
		if(pLstReq == (INetRequest*)pReq)
		{
			pReqList->Remove(i);
			pLstReq->ReleaseRef();
		}
	}
}

IMPL_LINK( ChannelTuner, RetryCallBack, void*, pTmp )
{
	DBG_ASSERT( pRetryList, "RetryTimer ohne RetryList" );
	if( !pRetryList )
		return 0;

	if( !xINetSession )
	{
		for( USHORT nListIdx = 0; nListIdx < pRetryList->Count(); ++nListIdx )
			delete pRetryList->GetObject( nListIdx );
		delete pRetryList;
		pRetryList = NULL;
		return 0;
	}

	for( USHORT nListIdx = 0; nListIdx < pRetryList->Count(); ++nListIdx )
	{
		RetryInformation* pRetryInf = pRetryList->GetObject( nListIdx );
		Time aActTime;
		Time aErrorTime = pRetryInf->aErrorTime;
		aErrorTime += DEF_RETRY_DIFF;
		if( aActTime > aErrorTime )
		{
			INetURLObject aObj( pRetryInf->aURL );

			pRetryList->Remove( pRetryInf );
			delete pRetryInf;

			if( !aObj.HasError() )
			{
				switch( aObj.GetProtocol() )
				{
					case INET_PROT_HTTP :
					case INET_PROT_HTTPS :
						SendHttpReq( aObj );
						break;
					case INET_PROT_FTP :
						SendFtpReq( aObj );
						break;
				}
			}
		}
	}

	if( !pRetryList->Count() )
	{
		delete pRetryList;
		pRetryList = NULL;
		aRetryTimer.Stop();
	}
	return 1;
}

void ChannelTuner::SFX_NOTIFY( SfxBroadcaster& rBC, const TypeId& rBCType,
							   const SfxHint& rHint, const TypeId& rHintType )
{
	// HTTPGetRequest
	if(rHint.IsA(TYPE(INetHint)) && rBC.IsA(TYPE(INetHttpGetRequest)))
	{
		INetHttpRequest* pReq = &(INetHttpRequest&)rBC;
		switch(((INetHint&)rHint).GetId())
		{
			case INET_NAME_RESOLVE_ERROR:
			case INET_CONNECT_ERROR:
			case INET_WRITE_ERROR:
			case INET_READ_ERROR:
			case INET_CONNECTION_CANCELED:
			case INET_SESSION_CANCELED:
			case INET_REQUEST_CANCELED: {
				RetryInformation* pRetryInf = new RetryInformation;
				pRetryInf->aURL = "http://";
				pRetryInf->aURL += pReq->GetConnection()->GetServerName();
				pRetryInf->aURL += ":";
				pRetryInf->aURL += pReq->GetConnection()->GetServerPort();
				pRetryInf->aURL += pReq->GetTarget();
				pRetryInf->aErrorTime = Time();

				if( !pRetryList )
				{
					pRetryList = new RetryList;
					aRetryTimer.Start();
				}
				pRetryList->Insert( pRetryInf );

				RemoveRequest(pReq);
			} break;
			case INET_READ_SUCCESS: {
				RemoveRequest(pReq);
				SvStream aStrm(pReq->GetReceiveBuffer());
				char* cBuffer = new char[pReq->GetContentLength()];
				ULONG numRead = aStrm.Read( cBuffer,
								pReq->GetContentLength() );
				String aMsg( cBuffer, (USHORT)numRead);
				delete cBuffer;
				RemoveRequest(pReq);
				NotifyChannelList(aMsg);
			} break;
		}
	}
}

