/*************************************************************************
 *
 *  $RCSfile: getchobj.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdio.h>

#if defined(WNT) || defined(WIN) || defined(OS2)
#include <io.h>
#endif

#if defined(WNT) || defined(WIN)
#include <sys/stat.h>
#endif

#include <fcntl.h>

#if defined(MAC)
#include <mac_start.h>
#include <Files.h>
#include <mac_end.h>
#include <unistd.h>
#endif

#if defined(OS2)
#include <sys\stat.h>
#endif

#if defined(UNX)
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#endif

#ifndef _PLUGMGR_HXX //autogen
#include <inet/plugmgr.hxx>
#endif

#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif
#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif
#include <inet/inetmain.hxx>
#include <tools/download.hxx>

#include "getchobj.hxx"

#define HTTP_PRIO		10
#define MAX_REDIRECT	5

#define	ANON_USER "anonymous"
#define ANON_PASS "channel@stardivision.de"

#define MAX_REQ					100
#define DEF_TIMER_INTERVAL		50

#if defined(OS2) || defined(UNX)  || defined(MAC)
#define _OPEN		  		open
#define _LSEEK				lseek
#define _READ		   		read
#define _WRITE		   		write
#define _CLOSE		  		close
#if defined(UNX)
#define _OPEN_R_FLAG		O_RDONLY
#define _OPEN_W_FLAG		O_TRUNC
#define _OPEN_A_FLAG		O_APPEND
#else
#define _OPEN_R_FLAG		O_RDONLY | O_BINARY
#define _OPEN_W_FLAG		O_CREAT | O_TRUNC | O_WRONLY | O_BINARY
#define _OPEN_A_FLAG		O_APPEND | O_WRONLY | O_BINARY
#endif
#else
#define _OPEN		  		_open
#define _LSEEK		   		_lseek
#define _READ		   		_read
#define _WRITE		   		_write
#define _CLOSE		   		_close
#define _OPEN_R_FLAG 		_O_RDONLY | _O_BINARY
#define _OPEN_W_FLAG 		_O_CREAT | _O_TRUNC | _O_WRONLY | _O_BINARY
#define _OPEN_A_FLAG		_O_APPEND | _O_WRONLY | _O_BINARY
#endif

GetChannelObj::GetChannelObj(INetSession* pSession)
{
	xINetSession	= pSession;
	pBTXDecoder		= NULL;
	nBTXError 		= DOWNLOAD_SUCCESS;

	pHttpCon		= NULL;
	pHttpGetReq		= NULL;

	pFtpCon			= NULL;
	pFtpGetDirReq	= NULL;
	pFtpGetFileReq	= NULL;

	nTotalSize	 	= 0;
	nRedirectCount	= 0;

	bFinished		= FALSE;
	bCloseIt		= FALSE;
	bSuccess		= FALSE;
	bAllowUI		= FALSE;
	bShowStatus		= FALSE;

	aLocalTimer.SetTimeoutHdl( LINK(this,GetChannelObj,LocalDownload) );
}

GetChannelObj::~GetChannelObj()
{
	CloseConnection();
	if( pHttpCon )
	{
		pHttpCon->Cancel();
		pHttpCon->ReleaseRef();
	}
}

void GetChannelObj::SetBTXDecoder(Downloader* pDecoder)
{
	pBTXDecoder	= pDecoder;
}

void GetChannelObj::CloseConnection()
{
	if( pFtpGetDirReq )
	{
		pFtpGetDirReq->ReleaseRef();
		pFtpGetDirReq = NULL;
	}

	if( pFtpGetFileReq )
	{
		pFtpGetFileReq->ReleaseRef();
		pFtpGetFileReq = NULL;
	}

	if( pHttpGetReq )
	{
		EndListening( *pHttpGetReq );
		pHttpGetReq->ReleaseRef();
		pHttpGetReq = NULL;
	}

	if( pHttpCon )
	{
		pHttpCon->Cancel();
		pHttpCon->ReleaseRef();
		pHttpCon = NULL;
	}

	if( pFtpCon )
	{
		EndListening( *pFtpCon );
		pFtpCon->Cancel();
		pFtpCon->ReleaseRef();
		pFtpCon = NULL;
	}
}

BOOL GetChannelObj::Download(const INetURLObject& rURL, String const& rLocalFile,
	BOOL bAsync )
{
	aURL			= rURL;
	aLocalFile		= rLocalFile;
	nTotalSize		= 0;
	nBytesCopied	= 0;
	nRedirectCount	= 0;
	bFinished 		= FALSE;
	bSuccess  		= FALSE;

	if (aURL.HasError())
	{
		String aErr("ERR_WRONG_URL");
		aErr += aURL.GetMainURL();
		if( bAllowUI )
			InfoBox(NULL, aErr).Execute();
		return FALSE;
	}

	if( aURL.GetProtocol() == INET_PROT_HTTP ||
		(xINetSession && aURL.GetProtocol() == INET_PROT_FTP &&
		 xINetSession->ShouldUseHttpProxy(aURL)) )
		CreateNewHttpDownload();
	else if (aURL.GetProtocol() == INET_PROT_FTP)
		CreateNewFtpDownload();
	else if (aURL.GetProtocol() == INET_PROT_FIF ||
			 aURL.GetProtocol() == INET_PROT_CEPT)
		CreateNewBTXDownload();
	else if (aURL.GetProtocol() == INET_PROT_FILE )
	{
		aLocalSrcFile = rURL.PathToFileName();
		aLocalTimer.SetTimeout( DEF_TIMER_INTERVAL );
		aLocalTimer.Start();
		return TRUE;
	}

	if( !bAsync )
	{
		while(!bFinished)
		{
			Application::Reschedule();
		}
		return bSuccess;
	}

	return TRUE;
}

void GetChannelObj::CloseDownload(BOOL bSuc)
{
	BOOL bIsBTX = FALSE; // muss auf dem Stack stehen bleiben.
	if( aURL.GetProtocol() == INET_PROT_FIF ||
		aURL.GetProtocol() == INET_PROT_CEPT )
		bIsBTX = TRUE;

	bSuccess	= bSuc;
	bFinished	= TRUE;

	aCallBackHdl.Call(this);
	if( !bIsBTX )
		CloseConnection();
}

void GetChannelObj::CreateNewBTXDownload()
{
	DirEntry aEntry( aLocalFile );
	aEntry.ToAbs();

	String aURLPath = aURL.GetMainURL();
	String aFileName = aURL.GetLastName();

	// DateiNamen abschneiden.
	aURLPath.Cut( aURLPath.Len() - aFileName.Len() - 1 );

	pBTXDecoder->Download(
		aEntry.GetFull(),
		aURLPath,
		LINK(this,GetChannelObj,BTXCallBack)
	);
}

IMPL_LINK(GetChannelObj, BTXCallBack, void*, pErr)
{
	nBTXError = (ULONG) pErr;
	if( nBTXError != DOWNLOAD_SUCCESS )
		CloseDownload( FALSE );
	else
		CloseDownload( TRUE );
	return 1;
}

void GetChannelObj::CreateNewHttpDownload()
{
	CloseConnection();

	if( bShowStatus )
	{
		if( !xCallBack.Is() )
			xCallBack = new SvBindStatusCallback;
		xCallBack->InitStartTime();
	}

	pHttpCon = xINetSession->CreateHttpConnection();

	pHttpCon->AddRef();
	pHttpCon->SetServerName( aURL.GetHost() );
	pHttpCon->SetServerPort( aURL.HasPort()? aURL.GetPort() :
												 INET_DEF_HTTP_PORT );

	pHttpGetReq = pHttpCon->CreateHttpGetRequest();

	pHttpGetReq->AddRef();
	pHttpGetReq->SetPrio( HTTP_PRIO );
	pHttpGetReq->SetCacheFlag( FALSE );

	if( aURL.GetProtocol() == INET_PROT_FTP )
		pHttpGetReq->SetProtocol(INET_PROT_FTP);

	if(aURL.HasUserData())
	{
		pHttpGetReq->SetUsername(aURL.GetUser());
		pHttpGetReq->SetPassword(aURL.GetPass());
	}

	if(aURL.HasRealm())
		pHttpGetReq->SetRealm(aURL.GetRealm());

	pHttpGetReq->SetTarget(aURL.GetTarget());

	StartListening(*pHttpGetReq);
	pHttpCon->Start();
}

void GetChannelObj::CreateNewFtpDownload()
{
	CloseConnection();

	if( bShowStatus )
	{
		if( !xCallBack.Is() )
			xCallBack = new SvBindStatusCallback;
		xCallBack->InitStartTime();
	}

	// ftp connection
	pFtpCon = xINetSession->CreateFtpConnection();
	pFtpCon->AddRef();

	pFtpCon->SetServerName(aURL.GetHost());
	pFtpCon->SetServerPort(aURL.HasPort()? aURL.GetPort() : INET_DEF_FTP_PORT );

	if(aURL.HasUserData())
	{
		pFtpCon->SetUsername(aURL.GetUser());
		pFtpCon->SetPassword(aURL.GetPass());
	}
	else
	{
		pFtpCon->SetUsername(ANON_USER);
		pFtpCon->SetPassword(ANON_PASS);
	}

	// directory request
	pFtpGetDirReq = pFtpCon->CreateFtpGetDirReq();
	pFtpGetDirReq->AddRef();
	pFtpGetDirReq->SetTarget(aURL.GetURLPath() );

	// getfile request
	pFtpGetFileReq = pFtpCon->CreateFtpGetFileReq();
	pFtpGetFileReq->AddRef();
	pFtpGetFileReq->SetTarget(aURL.GetURLPath());

	StartListening(*pFtpCon);
	pFtpCon->Start();
}

void GetChannelObj::WriteToFile(SvLockBytes* pLockBytes)
{
	if(aLocalFile.Len())
	{
		SvFileStream aDest(aLocalFile, STREAM_STD_WRITE);
		SvStream aSource(pLockBytes);
		aDest << aSource;
		aDest.Close();
	}
	else
		pData = pLockBytes;
}

void GetChannelObj::GetINetError(ULONG nId, INetRequest* pReq )
{
	String aErrText;

	switch(nId)
	{
		case INET_NAME_RESOLVE_ERROR:
			aErrText = String("ERR_INET_NAME_RESOLVE");
			break;

		case INET_CONNECT_ERROR:
			aErrText = String("ERR_INET_CONNECT");
			break;

		case INET_READ_ERROR:
			aErrText = String("ERR_INET_READ");
			aErrText.SearchAndReplace( "$(ARG1)", pReq->GetLastErrorText());
			break;

		case INET_WRITE_ERROR:
			aErrText = String("ERR_INET_WRITE");
			aErrText.SearchAndReplace( "$(ARG1)", pReq->GetLastErrorText());
			break;

		default:
			aErrText = String("ERR_INET_GENERAL");
			if( pReq->IsA(TYPE(INetHttpGetRequest)))
			{
				aErrText += "\n";
				aErrText += ((INetHttpGetRequest*)pReq)->GetStatusCode();
				aErrText += " ";
				aErrText += ((INetHttpGetRequest*)pReq)->GetStatusText();
			}
			break;
	}
	if( bAllowUI )
		InfoBox(NULL, aErrText).Execute();
}

void GetChannelObj::SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType,
							 const SfxHint& rHint, const TypeId& rHintType)
{
	if( rHint.IsA(TYPE(INetHint)) && rBC.IsA(TYPE(INetFtpConnection)) )
	{	// FTP
		ULONG nReqId = ((INetHint&)rHint).GetRequestId();
		INetRequest* pReq = pFtpCon->GetRequest(nReqId);

		if(pReq && pReq->IsA(TYPE(INetFtpGetDirReq)))
		{
			switch ( ((INetFileInfoHint&)rHint).GetId() )
			{
				case INET_READ_STATUS:
					nTotalSize = ((INetFileInfoHint&)rHint).GetFileInfo().nSize;
					break;

				case INET_READ_SUCCESS:
					DBG_ASSERT( pFtpGetDirReq == (INetFtpGetDirReq*)pReq,
								"Das ist nicht mein INetFtpGetDirReq!!!" );
					pFtpGetDirReq->ReleaseRef();
					pFtpGetDirReq = 0;
					break;
			}
		}
		else if(pReq && pReq->IsA(TYPE(INetFtpGetFileReq)))
		{
			switch (((INetStatusHint&) rHint).GetId())
			{
				case INET_REQUEST_CANCELED:
					CloseDownload(FALSE);
					break;

				case INET_READ_ERROR:
				case INET_WRITE_ERROR:
					GetINetError(((INetStatusHint&) rHint).GetId(), pReq);
					CloseDownload(FALSE);
					break;

				case INET_NAME_RESOLVE_START: {
					if( bShowStatus && xCallBack.Is() )
						xCallBack->OnProgress( 0, 0, SVBINDSTATUS_FINDINGRESOURCE,
							aURL.GetMainURL() );
					break;
				}
				case INET_CONNECT_START: {
					if( bShowStatus && xCallBack.Is() )
						xCallBack->OnProgress( 0, 0, SVBINDSTATUS_CONNECTING,
							aURL.GetMainURL() );
					break;
				}
				case INET_READ_START: {
					nBytesCopied = ((INetStatusHint&)rHint).GetData();
					aProgressHdl.Call(this);

					if( bShowStatus && xCallBack.Is() )
						xCallBack->OnProgress( nBytesCopied, nTotalSize, SVBINDSTATUS_BEGINDOWNLOADDATA,
							aURL.GetMainURL() );
					break;
				}

				case INET_READ_STATUS:
					nBytesCopied = ((INetStatusHint&)rHint).GetData();
					aProgressHdl.Call(this);

					if( bShowStatus && xCallBack.Is() )
						xCallBack->OnProgress( nBytesCopied, nTotalSize,
							SVBINDSTATUS_DOWNLOADINGDATA, aURL.GetMainURL() );
					break;

				case INET_READ_SUCCESS:
				{
					nBytesCopied = nTotalSize;
					aProgressHdl.Call(this);
					EndListening( *pFtpCon );
					WriteToFile((SvLockBytes*)(((INetFtpGetFileReq*)pReq)->
								GetReceiveBuffer()));
					CloseDownload(TRUE);
					break;
				}
			}
		}
		else
		{
			switch( ((INetHint&)rHint).GetId() )
			{
				case INET_CONNECTION_CANCELED:
				case INET_REQUEST_CANCELED:
				case INET_SESSION_CANCELED:
					CloseDownload(FALSE);
					break;

				case INET_NAME_RESOLVE_ERROR:
				case INET_CONNECT_ERROR:
				case INET_WRITE_ERROR:
				case INET_READ_ERROR:
					GetINetError(((INetHint&)rHint).GetId(), pReq);
					CloseDownload(FALSE);
					break;
			}
		}
	}
	else
	if( rHint.IsA(TYPE(INetHint)) && rBC.IsA(TYPE(INetHttpRequest)) )
	{	// HTTP
		INetHttpRequest* pReq = &(INetHttpRequest&)rBC;
		switch( ((INetStatusHint&) rHint).GetId() )
		{
			case INET_NAME_RESOLVE_ERROR:
			case INET_CONNECT_ERROR:
			case INET_WRITE_ERROR:
			case INET_READ_ERROR:
			case INET_CONNECTION_CANCELED:
			case INET_SESSION_CANCELED:
			case INET_REQUEST_CANCELED:
			{
				ULONG nStatus = pReq->GetStatusCode();
				if((nStatus == 301 || nStatus == 302) &&
					++nRedirectCount < MAX_REDIRECT)
				{
					INetURLObject aNewUrl;
					aURL.GetNewAbsURL(pReq->GetLocation(), &aNewUrl);
					if(aURL.HasMark())
						aNewUrl.SetMark(aURL.GetMark());
					aURL.SetURL( aURL.GetMainURL());

					DBG_ASSERT( pHttpGetReq == pReq, "falscher Request!" );
					CreateNewHttpDownload();
					return;
				}
				GetINetError(nStatus, pReq);
				CloseDownload(FALSE);
				break;
			}

			case INET_NAME_RESOLVE_START: {
				if( bShowStatus && xCallBack.Is() )
					xCallBack->OnProgress( 0, 0, SVBINDSTATUS_FINDINGRESOURCE,
						aURL.GetMainURL() );
				break;
			}
			case INET_CONNECT_START: {
				if( bShowStatus && xCallBack.Is() )
					xCallBack->OnProgress( 0, 0, SVBINDSTATUS_CONNECTING,
						aURL.GetMainURL() );
				break;
			}
			case INET_READ_START: {
				nTotalSize = pReq->GetContentLength();
				nBytesCopied = ((INetStatusHint&)rHint).GetData();
				aProgressHdl.Call(this);

				if( bShowStatus && xCallBack.Is() )
					xCallBack->OnProgress( nBytesCopied, nTotalSize, SVBINDSTATUS_BEGINDOWNLOADDATA,
						aURL.GetMainURL() );
				break;
			}

			case INET_READ_STATUS:
				nTotalSize = pReq->GetContentLength();
				nBytesCopied = ((INetStatusHint&)rHint).GetData();
				aProgressHdl.Call(this);

				if( bShowStatus && xCallBack.Is() )
					xCallBack->OnProgress( nBytesCopied, nTotalSize,
						SVBINDSTATUS_DOWNLOADINGDATA, aURL.GetMainURL() );
				break;

			case INET_READ_SUCCESS:
			{
				nBytesCopied = nTotalSize;
				aProgressHdl.Call(this);

				if( bShowStatus && xCallBack.Is() )
					xCallBack->OnProgress( nBytesCopied, nTotalSize, SVBINDSTATUS_ENDDOWNLOADDATA,
						aURL.GetMainURL() );
				EndListening( *pHttpGetReq );
				WriteToFile((SvLockBytes*)pHttpGetReq->GetReceiveBuffer());

				CloseDownload(TRUE);
				break;
			}
		}
	}
}

IMPL_LINK( GetChannelObj, LocalDownload, void*, pVoid )
{
	int	hFileHandle = _OPEN( aLocalSrcFile.GetStr(), _OPEN_R_FLAG );
	int hDestHandle;

	if( hFileHandle == -1 ) {
		DBG_ASSERT( !this, "Konnte File nicht oeffnen" );
		return 1;
	}

	if( !nTotalSize ) {
		nTotalSize = _LSEEK( hFileHandle, 0L, SEEK_END );
		hDestHandle = _OPEN( aLocalFile.GetStr(), _OPEN_W_FLAG );
	}
	else
		hDestHandle = _OPEN( aLocalFile.GetStr(), _OPEN_A_FLAG );

	char* pBuffer = new char[MAX_BLOCK];

	_LSEEK( hFileHandle, nBytesCopied, SEEK_SET );

	ULONG nRead = _READ( hFileHandle, pBuffer, MAX_BLOCK );
	_WRITE( hDestHandle, pBuffer, nRead );

	nBytesCopied += nRead;

	delete pBuffer;
	_CLOSE( hDestHandle );
	_CLOSE( hFileHandle );

	if( nBytesCopied == nTotalSize ) {
		aLocalTimer.Stop();
		CloseDownload(TRUE);
	}

	return 1;
}

