/*************************************************************************
 *
 *  $RCSfile: ecgm.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_DRAWING_XMASTERPAGETARGET_HPP_
#include <com/sun/star/drawing/XMasterPageTarget.hpp>
#endif
#ifndef _COM_SUN_STAR_DRAWING_FILLSTYLE_HPP_
#include <com/sun/star/drawing/FillStyle.hpp>
#endif
#ifndef _COM_SUN_STAR_DRAWING_XMASTERPAGETARGET_HPP_
#include <com/sun/star/drawing/XMasterPageTarget.hpp>
#endif
#ifndef _COM_SUN_STAR_DRAWING_POINTSEQUENCE_HPP_
#include <com/sun/star/drawing/PointSequence.hpp>
#endif
#ifndef _COM_SUN_STAR_DRAWING_CIRCLEKIND_HPP_
#include <com/sun/star/drawing/CircleKind.hpp>
#endif
#ifndef _COM_SUN_STAR_DRAWING_POINTSEQUENCESEQUENCE_HPP_
#include <com/sun/star/drawing/PointSequenceSequence.hpp>
#endif
#ifndef _COM_SUN_STAR_DRAWING_LINESTYLE_HPP_
#include <com/sun/star/drawing/LineStyle.hpp>
#endif

#include <ecgm.hxx>
#include <vos/xception.hxx>


using namespace ::vos;
using namespace ::rtl;
using namespace ::com::sun::star;


#define PutString( pString )							\
{														\
	const sal_Int8 nPutStringLen = (sal_Int8) strlen( pString );\
	*mpOut << nPutStringLen; 							\
	if( nPutStringLen ) 								\
		mpOut->Write( pString, nPutStringLen );			\
}

// ---------------------------------------------------------------------------------------------

GroupTable::GroupTable() :
	mpGroupEntry		( NULL ),
	mnMaxGroupEntry		( 0 ),
	mnCurrentGroupEntry	( 0 ),
	mnGroupsClosed		( 0 )
{
	ImplResizeGroupTable( 32 );
}

// ---------------------------------------------------------------------------------------------

GroupTable::~GroupTable()
{
	for ( sal_uInt32 i = 0; i < mnCurrentGroupEntry; delete mpGroupEntry[ i++ ] );
	delete mpGroupEntry;
}

// ---------------------------------------------------------------------------------------------

void GroupTable::ImplResizeGroupTable( sal_uInt32 nEntrys )
{
	if ( nEntrys > mnMaxGroupEntry )
	{
		mnMaxGroupEntry			= nEntrys;
		GroupEntry** pTemp = new GroupEntry*[ nEntrys ];
		for ( sal_uInt32 i = 0; i < mnCurrentGroupEntry; i++ )
			pTemp[ i ] = mpGroupEntry[ i ];
		if ( mpGroupEntry )
			delete mpGroupEntry;
		mpGroupEntry = pTemp;
	}
}

// ---------------------------------------------------------------------------------------------

void GroupTable::EnterGroup( uno::Reference< container::XIndexAccess > & rXIndexAccessRef )
{
	if ( rXIndexAccessRef.is() )
	{
		if ( mnMaxGroupEntry == mnCurrentGroupEntry )
			ImplResizeGroupTable( mnMaxGroupEntry + 8 );
		mpGroupEntry[ mnCurrentGroupEntry++ ] = new GroupEntry( rXIndexAccessRef );
	}
}

// ---------------------------------------------------------------------------------------------

sal_uInt32 GroupTable::GetGroupsClosed()
{
	sal_uInt32 nRet = mnGroupsClosed;
	mnGroupsClosed = 0;
	return nRet;
}

// ---------------------------------------------------------------------------------------------

void GroupTable::ClearGroupTable()
{
	for ( sal_uInt32 i = 0; i < mnCurrentGroupEntry; i++, delete mpGroupEntry );
	mnCurrentGroupEntry = 0;
}

// ---------------------------------------------------------------------------------------------

void GroupTable::ResetGroupTable( sal_uInt32 nCount )
{
	ClearGroupTable();
	mpGroupEntry[ mnCurrentGroupEntry++ ] = new GroupEntry( nCount );
}

// ---------------------------------------------------------------------------------------------

sal_Bool GroupTable::GetNextGroupEntry()
{
	while ( mnCurrentGroupEntry )
	{
		mnIndex = mpGroupEntry[ mnCurrentGroupEntry - 1 ]->mnCurrentPos++;

		if ( mpGroupEntry[ mnCurrentGroupEntry - 1 ]->mnCount > mnIndex )
			return sal_True;

		delete ( mpGroupEntry[ --mnCurrentGroupEntry ] );

		if ( mnCurrentGroupEntry )
			mnGroupsClosed++;
	}
	return sal_False;
}

//============================ CGMWriter ==================================

CGMWriter::CGMWriter( uno::Reference< frame::XModel > & rModel, SvStream& rOut, uno::Reference< task::XStatusIndicator > & rStatusIndicator ) :
	mbStatus					( sal_False ),
	mnCurrentPage				( 0 ),
	mpOut						( new SvMemoryStream ( (sal_uInt32)65536 ) ),
	mXModel						( rModel ),
	mXStatusIndicator			( rStatusIndicator ),
	maPageSize					( awt::Size( 28000, 21000 ) ),
	mXDrawPagesSupplier			( rModel, uno::UNO_QUERY ),
	mXMasterPagesSupplier		( rModel, uno::UNO_QUERY ),
	mXDrawPages					( mXDrawPagesSupplier->getDrawPages() ),
	mXMasterPages				( mXMasterPagesSupplier->getMasterPages() ),
	mnPages						( mXDrawPages->getCount() )
{
	mpFileOut = &rOut;

	mpOut->SetNumberFormatInt( NUMBERFORMAT_INT_BIGENDIAN );

	if( !mXDrawPagesSupplier.is() ) return;
	if( !mXMasterPagesSupplier.is() ) return;
	if ( !mXDrawPages.is() ) return;
	if ( !mXMasterPages.is() ) return;

	if ( mnPages )
	{
		if ( !ImplGetPageByIndex( 0 ) )
			return;
		mbStatus = ImplWriteGlobalHeader();
	}
};

//	-----------------------------------------------------------------------


sal_Bool CGMWriter::ImplGetPageByIndex( sal_uInt32 nIndex, sal_Bool bMaster )
{
	sal_Bool	bStatRet = sal_False;

	while ( sal_True )
	{
		mXDrawPage = *(uno::Reference< drawing::XDrawPage > *)mXDrawPages->getByIndex( nIndex ).getValue();
		if( !mXDrawPage.is() )
			break;
		if ( bMaster )
		{
			uno::Reference< drawing::XMasterPageTarget > aMasterPageTarget( mXDrawPage, uno::UNO_QUERY );

			if(!aMasterPageTarget.is())
				break;

			mXDrawPage = aMasterPageTarget->getMasterPage();
			if ( !mXDrawPage.is() )
				break;
		}
		mXPagePropSet = uno::Reference< beans::XPropertySet >( mXDrawPage, uno::UNO_QUERY );
		if( !mXPagePropSet.is() )
			break;

		mXShapes = uno::Reference< drawing::XShapes >( mXDrawPage, uno::UNO_QUERY );

		if( !mXShapes.is() )
			break;

		bStatRet = sal_True;
		break;
	}
	return bStatRet;
}

//	-----------------------------------------------------------------------

sal_Bool CGMWriter::ImplGetShapeByIndex( sal_uInt32 nIndex, sal_Bool bGroup )
{
	while(sal_True)
	{
		if (  ( bGroup == sal_False ) || ( GetCurrentGroupLevel() == 0 ) )
			mXShape = *(uno::Reference< drawing::XShape > *)mXShapes->getByIndex( nIndex ).getValue();
		else
			mXShape = *(uno::Reference< drawing::XShape > *)GetCurrentGroupAccess()->getByIndex( GetCurrentGroupIndex() ).getValue();
		if ( !mXShape.is() )
			break;

		mXPropSet = uno::Reference< beans::XPropertySet >( mXShape, uno::UNO_QUERY );
		if(!mXPropSet.is())
			break;

		mType = ByteString( UniString( mXShape->getShapeType() ), gsl_getSystemTextEncoding() );
		mType.Erase( 0, 12 );									// "stardiv.one.drawing." entfernen
		sal_uInt16 nPos = mType.Search( (const char*)"Shape" );
		mType.Erase( nPos, 5 );
		maShapeSize = mXShape->getSize();
		maShapePosition = mXShape->getPosition();
		return sal_True;
	}
	return sal_False;
}

// ---------------------------------------------------------------------------------------------

sal_Bool CGMWriter::ImplGetPropertyValue( const UniString& rString )
{
	sal_Bool bRetValue = sal_True;
	try
	{
		mAny = mXPropSet->getPropertyValue( rString );
		if ( !mAny.getValue() )
			bRetValue = sal_False;
	}
	catch(...)
	{
		bRetValue = sal_False;
	}

	return bRetValue;
}

// ---------------------------------------------------------------------------------------------

sal_Bool CGMWriter::ImplGetPropertyValue( const uno::Reference< beans::XPropertySet > & aXPropSet, const UniString& rString )
{
	sal_Bool bRetValue = sal_True;
	try
	{
		mAny = aXPropSet->getPropertyValue( rString );
		if ( !mAny.getValue() )
			bRetValue = sal_False;
	}
	catch(...)
	{
		bRetValue = sal_False;
	}

	return bRetValue;
}

//	-----------------------------------------------------------------------

sal_Bool CGMWriter::ImplWriteLineBundle( sal_Bool bEdge )
{
	sal_Int32		nLineWidth;
	drawing::LineStyle	eLineStyle;
	sal_Int32		nColor;
	sal_Int16		nOutStyle;
	sal_Bool		bEdgeOutVisibility = sal_False;
	if ( ImplGetPropertyValue( String::CreateFromAscii( "LineStyle" ) ) )
	{
		if( (mAny >>= eLineStyle) && eLineStyle != LineStyle_NONE )
		{
			bEdgeOutVisibility = sal_True;
			nColor = 0;
			if( ImplGetPropertyValue( String::CreateFromAscii( "LineColor" ) ) )
				mAny >>= nColor;

			if ( bEdge )
			{
				*mpOut << (sal_Int8)( nColor >> 16 ) << (sal_Int8)( nColor >> 8 ) << (sal_Int8)( nColor );	// EdgeColor
				ImplWriteDesc( 5, 0x1d );
			}
			else
			{
				*mpOut << (sal_Int8)( nColor >> 16 ) << (sal_Int8)( nColor >> 8 ) << (sal_Int8)( nColor );	// LineColor
				ImplWriteDesc( 5, 4 );
			}
			switch ( eLineStyle )
			{
				case LineStyle_DASH :
					nOutStyle = 2;
				break;
				case LineStyle_SOLID :
				default:
					nOutStyle = 1;	// drawing::LineStyle = Solid
				break;
			}
			nLineWidth = 1;
			if( ImplGetPropertyValue( String::CreateFromAscii( "LineWidth" ) ) )
				mAny >>= nLineWidth;
			if ( !nLineWidth )
				nLineWidth++;
			nLineWidth = ( mnMaxLength / ( nLineWidth >> 1 ) ) * ( mnMaxLength / 1000 );
			if ( bEdge )
			{
				*mpOut << nOutStyle;			// EdgeType
				ImplWriteDesc( 5, 0x1b );
				ImplWriteVDCInt( nLineWidth );	// EdgeWidth
				ImplWriteDesc( 5, 28 );
			}
			else
			{
				*mpOut << nOutStyle;			// LineType
				ImplWriteDesc( 5, 2 );
				ImplWriteVDCInt( nLineWidth );	// LineWidth
				ImplWriteDesc( 5, 3 );
			}
		}
	}
	if ( bEdge )
	{
		if ( bEdgeOutVisibility )
			*mpOut << (sal_uInt16)1;
		else
			*mpOut << (sal_uInt16)0;
		ImplWriteDesc( 5, 0x1e );
	}
	return bEdgeOutVisibility;
}

//	-----------------------------------------------------------------------

sal_Bool CGMWriter::ImplWriteFillBundle( sal_Bool bEdgeBundle )
{
	drawing::FillStyle	eFS( drawing::FillStyle_NONE );
	sal_Int32		nColor;
	sal_Int16		nOutStyle;
	if ( ImplGetPropertyValue( String::CreateFromAscii( "FillStyle" ) ) )
	{
		mAny >>= eFS;
		switch( eFS )
		{
			case drawing::FillStyle_NONE :
				nOutStyle = 4;
			break;
			case drawing::FillStyle_HATCH :
				nOutStyle = 3;		// FillStyle drawing::Hatch
			break;
			case drawing::FillStyle_GRADIENT :
			case drawing::FillStyle_BITMAP :
			case drawing::FillStyle_SOLID :
			default:
			{
				nOutStyle =	1;		// Fillstyle Solid
				nColor = 0;
				if( ImplGetPropertyValue( String::CreateFromAscii( "FillColor" ) ) )
					mAny >>= nColor;
				*mpOut << (sal_Int8)( nColor >> 16 ) << (sal_Int8)( nColor >> 8 ) << (sal_Int8)( nColor );	// FillColor
				ImplWriteDesc( 5, 0x17 );
			}
			break;
		}
	}
	*mpOut << nOutStyle;			// FillStyle
	ImplWriteDesc( 5, 0x16 );
	return ( ImplWriteLineBundle( bEdgeBundle ) || ( eFS != drawing::FillStyle_NONE ) );
}

//	-----------------------------------------------------------------------
// viele gedrehte objecte muessen in ein Polygon konvertiert werden
void CGMWriter::ImplWritePolygon( Polygon& rPolygon, sal_Bool bClosed )
{
	int	nPoints = rPolygon.GetSize();

	Point aPoint;
	for ( int i = 0; i < nPoints; i++ )
	{
		aPoint = rPolygon.GetPoint( i );
		*mpOut << (sal_Int32)aPoint.X() << (sal_Int32)aPoint.Y();
	}

	if ( bClosed )
		ImplWriteDesc( 4, 7 );
	else
		ImplWriteDesc( 4, 1 );
};

//	-----------------------------------------------------------------------

void CGMWriter::ImplCreatePolyPolygon( sal_Bool bClosed )
{
	sal_Int32 i, j, nInnerSequenceCount, nPolyPolyCount;
	sal_Bool bPolyVisible;
	if ( bClosed )
		bPolyVisible = ImplWriteFillBundle();
	else
		bPolyVisible = ImplWriteLineBundle();

	if ( bPolyVisible && ImplGetPropertyValue( String::CreateFromAscii( "PolyPolygon" ) ) )
	{
		drawing::PointSequenceSequence* pSourcePolyPolygon = (drawing::PointSequenceSequence*)mAny.getValue();
		sal_Int32 nOuterSequenceCount = pSourcePolyPolygon->getLength();

		// Zeiger auf innere sequences holen
		drawing::PointSequence* pInnerSequence = pSourcePolyPolygon->getArray();

		// ist dies ein Poly oder gar ein Polypolygon ?
		if ( nOuterSequenceCount == 1 )
		{
			// Zeiger auf Arrays holen
			awt::Point* pArray = pInnerSequence->getArray();
			nInnerSequenceCount = pInnerSequence->getLength();
			if ( nInnerSequenceCount >= 2 )
			{
				for( i = 0; i < nInnerSequenceCount; i++ )
					ImplWritePoint( *pArray++ );
				if ( ( nInnerSequenceCount == 2 ) || ( !bClosed ) )
					ImplWriteDesc( 4, 1 );	// PolyLine
				else
					ImplWriteDesc( 4, 7 );	// Polygon
			}
		}
		else
		{
			// dies wird ein Polygon set
			for( nPolyPolyCount = j = 0; j < nOuterSequenceCount; j++, pInnerSequence++ )
			{
				// Zeiger auf Arrays holen
				awt::Point* pArray = pInnerSequence->getArray();
				nInnerSequenceCount = pInnerSequence->getLength();
				if ( nInnerSequenceCount >= 2 )
				{
					nPolyPolyCount += nInnerSequenceCount;
					for( i = 0; i < nInnerSequenceCount; )
					{
						/*
							INVISIBLE		0
							VISIBLE			1
							CLOSE INVISIBLE 2
							CLOSE VISIBLE	3
						*/
						ImplWritePoint( *pArray++ );
						if ( ++i == nInnerSequenceCount )
							*mpOut << (sal_uInt16)3;
						else
							*mpOut << (sal_uInt16)1;
					}
				}
			}
			if ( nPolyPolyCount > 2 )
				ImplWriteDesc( 4, 8 );
		}
	}
}

//	-----------------------------------------------------------------------

void CGMWriter::ImplScanPage()
{
	sal_uInt32	nShapes, nGroups;

	ResetGroupTable( nShapes = mXShapes->getCount() );

	while ( GetNextGroupEntry() )
	{
		nGroups = GetGroupsClosed();
//		for ( sal_uInt32 i = 0; i < nGroups; i++, ImplWriteDesc( 0, 0xfb ) );

		if ( ImplGetShapeByIndex( GetCurrentGroupIndex(), sal_True ) )
		{
			sal_Int32		nOrientation = 0;
			sal_Bool		bPolygonIsClosed = sal_False;
			drawing::FillStyle	eFillStyle;

			nOrientation = 0;
			if( ImplGetPropertyValue( String::CreateFromAscii( "RotateAngle" ) ) )
				mAny >>= nOrientation;

			if ( mType == "drawing.Group" )
			{
				uno::Reference< container::XIndexAccess >  aXIndexAccess( mXShape, uno::UNO_QUERY );

				EnterGroup( aXIndexAccess );
//				ImplWriteDesc( 0, 0xfc );
				continue;
			}
			else if ( mType == "drawing.Rectangle" )
			{
				if ( ImplWriteFillBundle() )
				{
					if ( nOrientation )
					{
						Rectangle aShapeRect( maShapePosition.X, maShapePosition.Y, maShapeSize.Width, maShapeSize.Height );
						Polygon aPolygon( aShapeRect );
						aPolygon.Rotate( Point( maShapePosition.X, maShapePosition.Y ), nOrientation / 10 );
						ImplWritePolygon( aPolygon, sal_True );
					}
					else
					{
						ImplWritePoint( maShapePosition );
						ImplWritePoint( awt::Point( maShapePosition.X + maShapeSize.Width, maShapePosition.Y + maShapeSize.Height ) );
						ImplWriteDesc( 4, 11 );
					}
				}
			}
			else if ( mType == "drawing.Ellipse" )
			{
				drawing::CircleKind	eCircleKind( drawing::CircleKind_FULL );
				if ( ImplGetPropertyValue( String::CreateFromAscii( "CircleKind" ) ) )
					mAny >>= eCircleKind;

				switch ( eCircleKind )
				{
					case drawing::CircleKind_FULL :
					{
						eFillStyle = drawing::FillStyle_NONE;
						if( ImplGetPropertyValue( String::CreateFromAscii( "FillStyle" ) ) )
							mAny >>= eFillStyle;

						if ( eFillStyle != drawing::FillStyle_NONE && eFillStyle != drawing::FillStyle_HATCH )
						{
							ImplWriteFillBundle();
							Polygon	aPolygon( 3 );
							aPolygon.SetPoint( Point( maShapePosition.X + ( maShapeSize.Width >> 1 ),
														maShapePosition.Y + ( maShapeSize.Height >> 1 ) ), 0 );
							aPolygon.SetPoint( Point( maShapePosition.X + maShapeSize.Width,
														maShapePosition.Y + ( maShapeSize.Height >> 1 ) ), 1 );
							aPolygon.SetPoint( Point( maShapePosition.X + ( maShapeSize.Width >> 1 ),
														maShapePosition.Y ), 2 );
							aPolygon.Rotate( Point( maShapePosition.X, maShapePosition.Y ), nOrientation / 10 );

							ImplWritePoint( awt::Point( aPolygon.GetPoint( 0 ).X(), aPolygon.GetPoint( 0 ).Y() ));
							ImplWritePoint( awt::Point( aPolygon.GetPoint( 1 ).X(), aPolygon.GetPoint( 1 ).Y() ));
							ImplWritePoint( awt::Point( aPolygon.GetPoint( 2 ).X(), aPolygon.GetPoint( 2 ).Y() ));
							ImplWriteDesc( 4, 0x11 );	// Ellipse
							break;
						}
					}
					case drawing::CircleKind_SECTION :
					case drawing::CircleKind_CUT :
					case drawing::CircleKind_ARC :
					{
						if ( eCircleKind == drawing::CircleKind_ARC )
							ImplWriteLineBundle();
						else
							ImplWriteFillBundle( ( eCircleKind == drawing::CircleKind_FULL ) ? sal_False : sal_True );

						Polygon	 aPolygon( 3 );
						sal_Int32	 nStartAngle, nEndAngle;

						aPolygon.SetPoint( Point( maShapePosition.X + ( maShapeSize.Width >> 1 ),
													maShapePosition.Y + ( maShapeSize.Height >> 1 ) ), 0 );
						aPolygon.SetPoint( Point( maShapePosition.X + maShapeSize.Width,
													maShapePosition.Y + ( maShapeSize.Height >> 1 ) ), 1 );
						aPolygon.SetPoint( Point( maShapePosition.X + ( maShapeSize.Width >> 1 ),
													maShapePosition.Y ), 2 );
						aPolygon.Rotate( Point( maShapePosition.X, maShapePosition.Y ), nOrientation / 10 );
						ImplWritePoint( awt::Point(aPolygon.GetPoint( 0 ).X(), aPolygon.GetPoint( 0 ).Y()) );
						ImplWritePoint( awt::Point(aPolygon.GetPoint( 1 ).X(), aPolygon.GetPoint( 1 ).Y()) );
						ImplWritePoint( awt::Point(aPolygon.GetPoint( 2 ).X(), aPolygon.GetPoint( 2 ).Y()) );
						if ( eCircleKind != drawing::CircleKind_FULL )
						{
							sal_Int32 nValue = 0;
							mAny >>= nValue;

							nStartAngle = ( ImplGetPropertyValue( String::CreateFromAscii( "CircleStartAngle" ) ) ) ? 36000 - nValue : 0;
							nEndAngle = ( ImplGetPropertyValue( String::CreateFromAscii( "CircleEndAngle" ) ) ) ? 36000 - nValue : 0;

							aPolygon.SetSize( 1 );
							aPolygon.SetPoint( Point( 10000, 0 ), 0 );
							aPolygon.Rotate( Point( 0, 0 ), nStartAngle / 10 );
							ImplWritePoint( awt::Point( aPolygon.GetPoint( 0 ).X(), aPolygon.GetPoint( 0 ).Y() ), sal_False );
							aPolygon.SetPoint( Point( 10000, 0 ), 0 );
							aPolygon.Rotate( Point( 0, 0 ), nEndAngle / 10 );
							ImplWritePoint( awt::Point( aPolygon.GetPoint( 0 ).X(), aPolygon.GetPoint( 0 ).Y() ), sal_False );
						}
						if ( eCircleKind == drawing::CircleKind_FULL )
						{
							ImplWriteDesc( 4, 0x11 );			// Elliptical Arc
						}
						else if ( eCircleKind == drawing::CircleKind_ARC )
						{
							ImplWriteDesc( 4, 0x12 );			// Elliptical Arc
						}
						else
						{
							sal_uInt16 nType = 0;					// SECT
							if ( eCircleKind == drawing::CircleKind_CUT )// CUT, CHORD
								nType++;
							*mpOut << nType;
							ImplWriteDesc( 4, 0x13 );			// Elliptical Arc Close
						}
					}
				}
			}
			else if ( mType == "drawing.Control" )
			{

			}
			else if ( mType == "drawing.Connector" )
			{

			}
			else if ( mType == "drawing.Measure" )
			{

			}
			else if ( mType == "drawing.Line" )
			{
				ImplCreatePolyPolygon( sal_False );
			}
			else if ( ( mType == "drawing.PolyPolygon" ) || ( mType == "drawing.ClosedBezier" ) || ( mType == "drawing.ClosedFreeHand" ) || ( mType == "drawing.PolyPolygonPath" ) )
			{
				ImplCreatePolyPolygon( sal_True );
			}
			else if ( ( mType == "drawing.PolyLine" ) || ( mType == "drawing.OpenBezier" ) || ( mType == "drawing.OpenFreeHand" ) || ( mType == "drawing.PolyLinePath" ) )
			{
				ImplCreatePolyPolygon( sal_False );
			}
			else if ( mType == "drawing.GraphicObject" )
			{
/*
				mAny = mXPropSet->getPropertyValue( L"GraphicObjectFillBitmap" );

				FillBitmap*	pFillBitmap = (FillBitmap*)mAny.get();
				uno::Reference< awt::XBitmap > 	aXBitmap = pFillBitmap->Bitmap;
				BitmapEx	aBmpEx = VCLXBitmap::GetBitmapFrom( *aXBitmap );
				Bitmap		aBmp = aBmpEx.GetBitmap();
				sal_uInt16		nColorCount = 0;
				sal_uInt16 		nBitCount = aBmp.GetBitCount();

				if ( nBitCount <= 8 )
				{
					aBmp.Convert( BMP_CONVERSION_8BIT_TRANS );
					nColorCount = aBmp.GetColorCount() - 1;
					nBitCount = 8;
				}
				Bitmap				aBmpMask;
				BitmapReadAccess*	pAcc = aBmp.AcquireReadAccess();
				BitmapReadAccess*	pMaskAcc = NULL;

				if ( pAcc )
				{
					if ( aBmpEx.IsTransparent() )
					{
						aBmpMask = aBmpEx.GetMask();
						pMaskAcc = aBmpMask.AcquireReadAccess();
					}
				 awt::Size	aSize = aBmp.GetSizePixel();
					if ( pAcc->HasPalette() )
					{
						*mpOut << (sal_uInt16)0;			// Color Mode = INDEXED
						ImplWriteDesc( 2, 2 );
						if ( nColorCount )
						{
							*mpOut << (sal_Int8)1;	// starting index
							ImplWriteDesc( 5, 0x22 );	// color table
							for ( sal_uInt16 i = 0; i < nColorCount; i++ )
							{
								BitmapColor aBitmapColor = pAcc->GetPaletteColor( i );
								*mpOut << (sal_Int8)aBitmapColor.GetRed();
								*mpOut << (sal_Int8)aBitmapColor.GetGreen();
								*mpOut << (sal_Int8)aBitmapColor.GetBlue();
							}
						}
					}

					// each line is word aligned
					int i, nSizeOfScanLine = ( ( ( ( aSize.Width() * nBitCount ) + 7 ) >> 3 ) + 1 ) & ~1;

					sal_uInt32 nLineOut = 0x7fff / nSizeOfScanLine;		// BILD in 32KB Stuecke zerlegen

					Point nP( maShapePosition );
					Point nR( Point( maShapePosition.X + maShapeSize.Width, maShapePosition.Y ) );
					Point nQLast( Point( nR.X(), maShapePosition.Y + maShapeSize.Height ) );

					Point nQCurrent = ( Point( nR.X(), maShapePosition.Y + (sal_Int32)( (double)maShapeSize.Height * nLineOut / aSize.Height() ) ) );

					if ( nOrientation )
					{
						Polygon aPolygon( 4 );
						aPolygon[ 0 ] = nP;
						aPolygon[ 1 ] = nR;
						aPolygon[ 2 ] = nQLast;
						aPolygon[ 3 ] = nQCurrent;
						aPolygon.Rotate( nP, nOrientation / 10 );
						nP = aPolygon[ 0 ];
						nR = aPolygon[ 1 ];
						nQLast = aPolygon[ 2 ];
						nQCurrent = aPolygon[ 3 ];
					}

					for ( int nCurrentLine = 0; nCurrentLine < aSize.Height(); nCurrentLine += nLineOut )	// LOOP der 32K Stuecke
					{
						int nMaxLine = nCurrentLine + nLineOut;
						if ( nMaxLine > aSize.Height() )
							nMaxLine = aSize.Height();

						*mpOut << (sal_Int32)nP.X() << (sal_Int32)nP.Y() << (sal_Int32)nQCurrent.X() << (sal_Int32)nQCurrent.Y() << (sal_Int32)nR.X() << (sal_Int32)nR.Y();
						*mpOut << (sal_Int32)aSize.Width() << (sal_Int32)nMaxLine - nCurrentLine;
						*mpOut << (sal_Int32)nBitCount;
						*mpOut << (sal_uInt16)1;	// mode CCOMP_PACKED

						for ( int ny = nCurrentLine; ny < nMaxLine; ny++ )
						{
							if ( nColorCount )
							{
								for ( i = 0; i < aSize.Width(); i++ )
								{
									*mpOut << (sal_Int8)pAcc->GetPixel( ny, i ) + 1;	// +1 -> die hintergrundfarbe darf unter cgm nicht gesetzt werden
								}
							}
							else
							{
								for ( i = 0; i < aSize.Width(); i++ )
								{
									BitmapColor aBitmapColor = pAcc->GetPixel( ny, i );
									*mpOut << (sal_Int8)aBitmapColor.GetRed();
									*mpOut << (sal_Int8)aBitmapColor.GetGreen();
									*mpOut << (sal_Int8)aBitmapColor.GetBlue();
								}
							}
							if ( aSize.Width() & 1 )	// WORD ALIGNMENT
								*mpOut << (sal_Int8)0;
						}
						ImplWriteDesc( 4, 9 );
						Point nYDiff;

						nYDiff.X() = nQCurrent.X() - nR.X();
						nYDiff.Y() = nQCurrent.Y() - nR.Y();

						nR = nQCurrent;
						nP.X() += nYDiff.X();
						nP.Y() += nYDiff.Y();
						if ( ( ny + nLineOut ) > aSize.Height() )
							nQCurrent = nQLast;
						else
						{
							nQCurrent.X() += nYDiff.X();
							nQCurrent.Y() += nYDiff.Y();
						}
					}
					if ( pAcc && pAcc->HasPalette() )
					{
						*mpOut << (sal_uInt16)1;	// Color Mode = DIRECT
						ImplWriteDesc( 2, 2 );
					}
				}
				if ( pMaskAcc )
					aBmpMask.ReleaseAccess( pMaskAcc );
				if ( pAcc )
					aBmp.ReleaseAccess( pAcc );
*/
			}
			else if ( mType == "drawing.Text" )
			{

			}
			else if ( mType == "drawing.OLE2" )
			{

			}
			else if ( mType == "drawing.Page" )
			{

			}
			else if ( mType == "drawing.Frame" )
			{

			}
			else if ( mType == "drawing.3DSceneObject" )
			{

			}
			else if ( mType == "drawing.3DCubeObject" )
			{

			}
			else if ( mType == "drawing.3DSphereObject" )
			{

			}
			else if ( mType == "drawing.3DLatheObject" )
			{

			}
			else if ( mType == "drawing.3DExtrudeObject" )
			{

			}
			else if ( mType == "drawing.3DPolygonObject" )
			{

			}
		}
	}
	ClearGroupTable();	// gruppierungen wegschreiben, sofern noch irgendwelche offen sind, was eigendlich nicht sein sollte
	nGroups = GetGroupsClosed();
//	for ( sal_uInt32 i = 0; i < nGroups; i++, ImplWriteDesc( 0, 0xfb ) );
};

//	-----------------------------------------------------------------------

sal_Bool CGMWriter::ImplWriteGlobalHeader()
{
	PutString( " " );								// Begin MetaFile
	ImplWriteDesc( 0, 1 );

	*mpOut << (sal_uInt16)3;							// MetaFile Version
	ImplWriteDesc( 1, 1 );

	PutString( "CGM Metafile - by Sun Microsystems, Inc." );	// MetaFile Description
	ImplWriteDesc( 1, 2 );

	*mpOut << (sal_uInt32)0x0001ffff << (sal_uInt16)0x0001;	// MetaFile Element List ( Version 3 )
	ImplWriteDesc( 1, 11 );

	*mpOut << (sal_uInt16)16;							// sdbcx::Index Precision = sal_Int16
	ImplWriteDesc( 1, 6 );

	*mpOut << (sal_uInt16)1;							// Color Model
	ImplWriteDesc( 1, 19 );

	*mpOut << (sal_uInt16)0;							// VDC_Type Integer
	ImplWriteDesc( 1, 3 );

	*mpOut << (sal_Int16)8;								// Color Precision
	ImplWriteDesc( 1, 7 );

	*mpOut << (sal_Int8)0 << (sal_Int8)0 << (sal_Int8)0 << (sal_Int8)0xff << (sal_Int8)0xff << (sal_Int8)0xff;
	ImplWriteDesc( 1, 10 );

	*mpOut << (sal_uInt16)0 << (sal_uInt16)12 << (sal_uInt16)52;// RealPrecision = Double
	ImplWriteDesc( 1, 5 );

	*mpOut << (sal_uInt16)32;							// Integer Precision = sal_Int32;
	ImplWriteDesc( 1, 4 );

	*mpFileOut << (sal_uInt16)( ( 1 << 12 )				// MetaFile Default Replacement
					| ( 0xc << 5 ) | 0x1f ) << (sal_uInt16)0;
	sal_uInt32 nOldPos = mpFileOut->Tell();
	{
		*mpOut << (sal_uInt16)0;							// Scaling Mode Absolute
		ImplWriteDouble( 1 );
		ImplWriteDesc( 2, 1 );

		*mpOut << (sal_uInt32)32;							// VDC Integer Precision = sal_Int32
		ImplWriteDesc( 3, 1 );
		if ( ImplGetPropertyValue( mXPagePropSet, String::CreateFromAscii( "Width" ) ) )
		{
			sal_Int32 nValue;
			if( mAny >>= nValue )
				maPageSize.Width = nValue;
		}

		if ( ImplGetPropertyValue( mXPagePropSet, String::CreateFromAscii( "Height" ) ) )
		{
			sal_Int32 nValue;
			if( mAny >>= nValue )
				maPageSize.Height = nValue;
		}

		mnMaxLength = max( maPageSize.Width, maPageSize.Height );
		*mpOut	<< (sal_Int32)0
				<< (sal_Int32)0
				<< (sal_Int32)maPageSize.Width
				<< (sal_Int32)maPageSize.Height;			// Set VDC Extent
		ImplWriteDesc( 2, 6 );
		*mpOut << (sal_uInt32)0xffffff00;					// Set BackGroundColor
		ImplWriteDesc( 2, 7 );
		*mpOut << (sal_uInt16)0;							// LineWidthSpecMode = Absolute
		ImplWriteDesc( 2, 3 );
		*mpOut << (sal_uInt16)0;							// EdgeWidthSpecMode = Absolute
		ImplWriteDesc( 2, 5 );
	}
	sal_uInt32 nCurrentPos = mpFileOut->Tell();
	mpFileOut->Seek( nOldPos - 2 );
	*mpFileOut << (sal_uInt16)( nCurrentPos - nOldPos );
	mpFileOut->Seek( nCurrentPos );

	return sal_True;
};

// ------------------------------------------------------------------------

void CGMWriter::ImplWriteDesc( int nClass, int nID )
{
	sal_uInt16	nHeader = ( ( nClass << 12 ) | ( nID << 5 ) );
	sal_uInt16	nSize = mpOut->Tell();
	if ( nSize > 0x1e )
		nHeader |= 0x1f;
	else
		nHeader |= nSize;
	*mpFileOut << nHeader;
	if ( nSize > 0x1e )
		*mpFileOut << nSize;
	if ( nSize )
	{
		mpOut->Seek( STREAM_SEEK_TO_BEGIN );
		mpFileOut->Write( mpOut->GetData(), nSize );
		mpOut->Seek( STREAM_SEEK_TO_BEGIN );
		if ( nSize & 1 )
			mpFileOut->SeekRel( 1 );	// WORD drawing::Alignment
	}
};

//	-----------------------------------------------------------------------

void CGMWriter::ImplWriteDouble( const double& rDouble )
{
#ifdef __BIGENDIAN
	mpOut->Write( &rDouble, 8 );
#else
	const sal_Int8 *pSrc = (const sal_Int8*)&rDouble;
	for ( int i = 0; i < 8; i++ )
		*mpOut << (sal_Int8)( pSrc[ i ^ 7 ] );	// Little Endian <-> Big Endian switch
#endif
}

//	-----------------------------------------------------------------------

void CGMWriter::ImplWriteVDCInt( sal_Int32 nNumb )
{
	*mpOut << (sal_Int32)nNumb;
}

//	-----------------------------------------------------------------------

void CGMWriter::ImplWritePoint( const awt::Point& rPoint, sal_Bool bMap )
{
	if ( bMap )
	{
		awt::Point aPoint( ImplMapPoint( rPoint ) );
		*mpOut	<< (sal_Int32)aPoint.X
				<< (sal_Int32)aPoint.Y;
	}
	else
	{
		*mpOut	<< (sal_Int32)rPoint.X
				<< (sal_Int32)rPoint.Y;
	}
}

//	-----------------------------------------------------------------------

awt::Size CGMWriter::ImplMapSize( const awt::Size& rSize )
{
	awt::Size aRetSize( rSize.Width, -rSize.Height );
	return aRetSize;
}

// ------------------------------------------------------------------------

awt::Point CGMWriter::ImplMapPoint( const awt::Point& rPoint )
{
	awt::Point aRetPoint( rPoint.X, maPageSize.Height - rPoint.Y );
	return aRetPoint;
}

// ------------------------------------------------------------------------

CGMWriter::~CGMWriter()
{
	delete mpOut;
};

//	-----------------------------------------------------------------------

void CGMWriter::Write()
{
	while ( mbStatus && ( mnCurrentPage < mnPages ) )
	{
		PutString( " " );			// Begin Picture
		ImplWriteDesc( 0, 3 );
		*mpOut << (sal_uInt16)1;		// Color Mode = DIRECT
		ImplWriteDesc( 2, 2 );
		ImplWriteDesc( 0, 4 );		// Begin Picture Body
		*mpOut << (sal_uInt16)1;
		ImplWriteDesc( 3, 4 );		// Transparency ON
		if ( ImplGetPageByIndex( mnCurrentPage, sal_True ) )	// erst die MasterPage
			ImplScanPage();
		if ( ImplGetPageByIndex( mnCurrentPage++, sal_False ) )	// dann den Slide
			ImplScanPage();

		ImplWriteDesc( 0, 5 );		// End Picture
	}
	if ( mbStatus )
		ImplWriteDesc( 0, 2 );		// End MetaFile;
};

//	-----------------------------------------------------------------------

// ---------------------
// - exported function -
// ---------------------

extern "C" sal_Bool __LOADONCALLAPI ExportCGM( UniString& rFileName, uno::Reference< frame::XModel > & rXModel, uno::Reference< task::XStatusIndicator > & rStatusIndicator, void* pvoid )
{
	CGMWriter*	pCGMWriter;
	sal_Bool bStatus = sal_False;

	if( rXModel.is() )
	{
		SvFileStream	aOut( rFileName, STREAM_WRITE );
		aOut.SetNumberFormatInt( NUMBERFORMAT_INT_BIGENDIAN );
		pCGMWriter = new CGMWriter( rXModel, aOut, rStatusIndicator );
		if ( pCGMWriter && pCGMWriter->IsValid() )
		{
			pCGMWriter->Write();
			bStatus = pCGMWriter->IsValid();
		}
		delete pCGMWriter;
	}
	return bStatus;
}

