/*************************************************************************
 *
 *  $RCSfile: characterclassification.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _I18N_CHARACTERCLASSIFICATION_HXX_
#define _I18N_CHARACTERCLASSIFICATION_HXX_


#ifndef _COM_SUN_STAR_LANG_XCHARACTERCLASSIFICATION_HPP_
#include <com/sun/star/lang/XCharacterClassification.hpp>
#endif

#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx> // helper for implementations
#endif

#include <tools/string.hxx>
#include <com/sun/star/lang/KCharacterType.hpp>


// Determine if the parser uses the default implementation (using class
// International) instead of the ICU I18N implementation.
// Set to 0 for ICU I18N implementation
#define USE_I18N_DEFAULT_IMPLEMENTAION 1

#if USE_I18N_DEFAULT_IMPLEMENTAION
#ifndef _LANG_HXX
#include <tools/lang.hxx>
#endif
#ifndef _ISOLANG_HXX
#include <tools/isolang.hxx>
#endif
class International;
#endif // USE_I18N_DEFAULT_IMPLEMENTAION


typedef sal_uInt32 UPT_FLAG_TYPE;

class CharacterClassification :
	public cppu::WeakImplHelper1 < ::com::sun::star::lang::XCharacterClassification >
{
public:

	CharacterClassification();
	~CharacterClassification();

    virtual ::rtl::OUString SAL_CALL toUpper( const ::rtl::OUString& Text, sal_Int32 nPos, sal_Int32 nCount, const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException);
    virtual ::rtl::OUString SAL_CALL toLower( const ::rtl::OUString& Text, sal_Int32 nPos, sal_Int32 nCount, const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException);
    virtual ::rtl::OUString SAL_CALL toTitle( const ::rtl::OUString& Text, sal_Int32 nPos, sal_Int32 nCount, const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Int16 SAL_CALL getType( const ::rtl::OUString& Text, sal_Int32 nPos )  throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Int16 SAL_CALL getCharacterDirection( const ::rtl::OUString& Text, sal_Int32 nPos ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Int16 SAL_CALL getScript( const ::rtl::OUString& Text, sal_Int32 nPos ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Int32 SAL_CALL getCharacterType( const ::rtl::OUString& text, sal_Int32 nPos, const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Int32 SAL_CALL getStringType( const ::rtl::OUString& text, sal_Int32 nPos, sal_Int32 nCount, const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::lang::ParseResult SAL_CALL parseAnyToken( const ::rtl::OUString& Text, sal_Int32 nPos, const ::com::sun::star::lang::Locale& rLocale, sal_Int32 nStartCharFlags, const ::rtl::OUString& userDefinedCharactersStart, sal_Int32 nContCharFlags, const ::rtl::OUString& userDefinedCharactersCont ) throw(::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::lang::ParseResult SAL_CALL parsePredefinedToken( sal_Int32 nTokenType, const ::rtl::OUString& Text, sal_Int32 nPos, const ::com::sun::star::lang::Locale& rLocale, sal_Int32 nStartCharFlags, const ::rtl::OUString& userDefinedCharactersStart, sal_Int32 nContCharFlags, const ::rtl::OUString& userDefinedCharactersCont ) throw(::com::sun::star::uno::RuntimeException);

// --- parser specific (implemented in characterclassification_parser.cxx) ---

private:

	enum ScanState
	{
		ssGetChar,
		ssGetValue,
		ssGetWord,
		ssGetWordFirstChar,
		ssGetString,
		ssGetBool,
		ssStopBack,
		ssBounce,
		ssStop
	};

	static const sal_uInt8		nDefCnt;
	static const UPT_FLAG_TYPE	pDefaultParserTable[];
	static const sal_Int32		pParseTokensType[];

	/// Flag values of table.
	static const UPT_FLAG_TYPE	TOKEN_ILLEGAL;
	static const UPT_FLAG_TYPE	TOKEN_CHAR;
	static const UPT_FLAG_TYPE	TOKEN_CHAR_BOOL;
	static const UPT_FLAG_TYPE	TOKEN_CHAR_WORD;
	static const UPT_FLAG_TYPE	TOKEN_CHAR_VALUE;
	static const UPT_FLAG_TYPE	TOKEN_CHAR_STRING;
	static const UPT_FLAG_TYPE	TOKEN_CHAR_DONTCARE;
	static const UPT_FLAG_TYPE	TOKEN_BOOL;
	static const UPT_FLAG_TYPE	TOKEN_WORD;
	static const UPT_FLAG_TYPE	TOKEN_WORD_SEP;
	static const UPT_FLAG_TYPE	TOKEN_VALUE;
	static const UPT_FLAG_TYPE	TOKEN_VALUE_SEP;
	static const UPT_FLAG_TYPE	TOKEN_VALUE_EXP;
	static const UPT_FLAG_TYPE	TOKEN_VALUE_SIGN;
	static const UPT_FLAG_TYPE	TOKEN_VALUE_EXP_VALUE;
	static const UPT_FLAG_TYPE	TOKEN_VALUE_DIGIT;
	static const UPT_FLAG_TYPE	TOKEN_NAME_SEP;
	static const UPT_FLAG_TYPE	TOKEN_STRING_SEP;
	static const UPT_FLAG_TYPE	TOKEN_EXCLUDED;

	/// If and where c occurs in pStr
	static	const sal_Unicode*	StrChr( const sal_Unicode* pStr, sal_Unicode c );

#if USE_I18N_DEFAULT_IMPLEMENTAION
	/// used for parser AND upper/lower et al.
	International*				pIntl;
#else
	/// used for parser only
	::com::sun::star::lang::Locale	aParserLocale;
#endif
	String						aStartChars;
	String						aContChars;
	UPT_FLAG_TYPE*				pTable;
	UPT_FLAG_TYPE*				pStart;
	UPT_FLAG_TYPE*				pCont;
	sal_Int32					nStartTypes;
	sal_Int32					nContTypes;
	ScanState					eState;
	sal_Unicode					cGroupSep;
	sal_Unicode					cDecimalSep;

	/// Get corresponding KParseTokens flag for a character
			sal_Int32			getParseTokensType( sal_Unicode c );

	/// Access parser table flags.
			UPT_FLAG_TYPE		getFlags( sal_Unicode c );

	/// Access parser flags via International and special definitions.
			UPT_FLAG_TYPE		getFlagsExtended( sal_Unicode c );

	/// Access parser table flags for user defined start characters.
			UPT_FLAG_TYPE		getStartCharsFlags( sal_Unicode c );

	/// Access parser table flags for user defined continuation characters.
			UPT_FLAG_TYPE		getContCharsFlags( sal_Unicode c );

	/// Setup parser table. Calls initParserTable() only if needed.
			void				setupParserTable(
									const ::com::sun::star::lang::Locale& rLocale,
									sal_Int32 startCharTokenType,
									const String& userDefinedCharactersStart,
									sal_Int32 contCharTokenType,
									const String& userDefinedCharactersCont );

	/// Init parser table.
			void				initParserTable(
									const ::com::sun::star::lang::Locale& rLocale,
									sal_Int32 startCharTokenType,
									const String& userDefinedCharactersStart,
									sal_Int32 contCharTokenType,
									const String& userDefinedCharactersCont );

	/// Destroy parser table.
			void				destroyParserTable();

	/// Parse a text.
			void				parseText( ::com::sun::star::lang::ParseResult& r,
									const ::rtl::OUString& rText, sal_Int32 nPos,
									sal_Int32 nTokenType = 0xffffffff );


#if USE_I18N_DEFAULT_IMPLEMENTAION
	/// Setup International class, new'ed only if different from existing.
			void				setupInternational( LanguageType eLang );

	inline	void				setupInternational(
									const ::com::sun::star::lang::Locale& rLocale )
									{
										setupInternational(
											ConvertIsoNamesToLanguage(
											rLocale.Language, rLocale.Country ) );
									}
#else
	inline	void				setupInternational(
									const ::com::sun::star::lang::Locale& rLocale )
									{
										aParserLocale.Language = rLocale.Language;
										aParserLocale.Country = rLocale.Country;
										aParserLocale.Variant = rLocale.Variant;
									}
#endif

	/// Implementation of getCharacterType() for one single character
			sal_Int32			getCharType( sal_Unicode c );

};



#endif
