/*************************************************************************
 *
 *  $RCSfile: characterclassification_parser.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <characterclassification.hxx>

#ifndef _TOOLS_INTN_HXX
#include <tools/intn.hxx>
#endif
#ifndef _ISOLANG_HXX
#include <tools/isolang.hxx>
#endif
#ifndef _TOOLS_SOLMATH_HXX
#include <tools/solmath.hxx>
#endif

#ifndef _COM_SUN_STAR_LANG_KPARSETOKENS_HPP_
#include <com/sun/star/lang/KParseTokens.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_KPARSETYPE_HPP_
#include <com/sun/star/lang/KParseType.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_UNICODETYPE_HPP_
#include <com/sun/star/lang/UnicodeType.hpp>
#endif

#include <string.h>		// memcpy()

#if USE_I18N_DEFAULT_IMPLEMENTAION
#include <ctype.h>
#else
#include <unicode/uchar.h>
#endif

const UPT_FLAG_TYPE CharacterClassification::TOKEN_ILLEGAL		= 0x00000000;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_CHAR			= 0x00000001;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_CHAR_BOOL	= 0x00000002;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_CHAR_WORD	= 0x00000004;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_CHAR_VALUE	= 0x00000008;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_CHAR_STRING	= 0x00000010;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_CHAR_DONTCARE= 0x00000020;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_BOOL			= 0x00000040;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_WORD			= 0x00000080;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_WORD_SEP		= 0x00000100;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_VALUE		= 0x00000200;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_VALUE_SEP	= 0x00000400;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_VALUE_EXP	= 0x00000800;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_VALUE_SIGN	= 0x00001000;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_VALUE_EXP_VALUE	= 0x00002000;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_VALUE_DIGIT	= 0x00004000;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_NAME_SEP		= 0x20000000;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_STRING_SEP	= 0x40000000;
const UPT_FLAG_TYPE CharacterClassification::TOKEN_EXCLUDED		= 0x80000000;

#define TOKEN_DIGIT_FLAGS (TOKEN_CHAR_VALUE | TOKEN_VALUE | TOKEN_VALUE_EXP | TOKEN_VALUE_EXP_VALUE | TOKEN_VALUE_DIGIT)


// Default identifier/name specification is [A-Za-z_][A-Za-z0-9_]*

const sal_uInt8 CharacterClassification::nDefCnt = 128;
const UPT_FLAG_TYPE CharacterClassification::pDefaultParserTable[ nDefCnt ] =
{
// (...) == Calc formula compiler specific, commented out and modified

	/* \0 */	TOKEN_EXCLUDED,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
	/*  9 \t */	TOKEN_CHAR_DONTCARE | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,		// (TOKEN_ILLEGAL)
				TOKEN_ILLEGAL,
	/* 11 \v */	TOKEN_CHAR_DONTCARE | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,		// (TOKEN_ILLEGAL)
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
				TOKEN_ILLEGAL,
	/*  32   */	TOKEN_CHAR_DONTCARE | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  33 ! */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  34 " */	TOKEN_CHAR_STRING | TOKEN_STRING_SEP,
	/*  35 # */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_WORD_SEP)
	/*  36 $ */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_CHAR_WORD | TOKEN_WORD)
	/*  37 % */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_VALUE)
	/*  38 & */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  39 ' */	TOKEN_NAME_SEP,
	/*  40 ( */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  41 ) */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  42 * */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  43 + */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP | TOKEN_VALUE_EXP | TOKEN_VALUE_SIGN,
	/*  44 , */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_CHAR_VALUE | TOKEN_VALUE)
	/*  45 - */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP | TOKEN_VALUE_EXP | TOKEN_VALUE_SIGN,
	/*  46 . */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_WORD | TOKEN_CHAR_VALUE | TOKEN_VALUE)
	/*  47 / */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	//for ( i = 48; i < 58; i++ )
	/*  48 0 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  49 1 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  50 2 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  51 3 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  52 4 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  53 5 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  54 6 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  55 7 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  56 8 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  57 9 */	TOKEN_DIGIT_FLAGS | TOKEN_WORD,
	/*  58 : */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_WORD)
	/*  59 ; */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  60 < */	TOKEN_CHAR_BOOL | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  61 = */	TOKEN_CHAR | TOKEN_BOOL | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  62 > */	TOKEN_CHAR_BOOL | TOKEN_BOOL | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  63 ? */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_CHAR_WORD | TOKEN_WORD)
	/*  64 @ */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	//for ( i = 65; i < 91; i++ )
	/*  65 A */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  66 B */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  67 C */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  68 D */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  69 E */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  70 F */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  71 G */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  72 H */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  73 I */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  74 J */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  75 K */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  76 L */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  77 M */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  78 N */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  79 O */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  80 P */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  81 Q */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  82 R */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  83 S */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  84 T */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  85 U */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  86 V */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  87 W */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  88 X */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  89 Y */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  90 Z */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  91 [ */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/*  92 \ */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/*  93 ] */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/*  94 ^ */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,
	/*  95 _ */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  96 ` */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	//for ( i = 97; i < 123; i++ )
	/*  97 a */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  98 b */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/*  99 c */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 100 d */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 101 e */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 102 f */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 103 g */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 104 h */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 105 i */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 106 j */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 107 k */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 108 l */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 109 m */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 110 n */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 111 o */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 112 p */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 113 q */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 114 r */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 115 s */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 116 t */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 117 u */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 118 v */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 119 w */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 120 x */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 121 y */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 122 z */	TOKEN_CHAR_WORD | TOKEN_WORD,
	/* 123 { */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/* 124 | */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/* 125 } */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/* 126 ~ */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP,	// (TOKEN_ILLEGAL // UNUSED)
	/* 127   */	TOKEN_CHAR | TOKEN_WORD_SEP | TOKEN_VALUE_SEP	// (TOKEN_ILLEGAL // UNUSED)
};


const sal_Int32 CharacterClassification::pParseTokensType[ nDefCnt ] =
{
	/* \0 */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
	/*  9 \t */	::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
	/* 11 \v */	::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
				::com::sun::star::lang::KParseTokens::ASC_CONTROL,
	/*  32   */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  33 ! */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  34 " */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  35 # */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  36 $ */	::com::sun::star::lang::KParseTokens::ASC_DOLLAR,
	/*  37 % */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  38 & */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  39 ' */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  40 ( */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  41 ) */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  42 * */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  43 + */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  44 , */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  45 - */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  46 . */	::com::sun::star::lang::KParseTokens::ASC_DOT,
	/*  47 / */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	//for ( i = 48; i < 58; i++ )
	/*  48 0 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  49 1 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  50 2 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  51 3 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  52 4 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  53 5 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  54 6 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  55 7 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  56 8 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  57 9 */	::com::sun::star::lang::KParseTokens::ASC_DIGIT,
	/*  58 : */	::com::sun::star::lang::KParseTokens::ASC_COLON,
	/*  59 ; */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  60 < */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  61 = */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  62 > */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  63 ? */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  64 @ */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	//for ( i = 65; i < 91; i++ )
	/*  65 A */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  66 B */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  67 C */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  68 D */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  69 E */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  70 F */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  71 G */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  72 H */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  73 I */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  74 J */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  75 K */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  76 L */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  77 M */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  78 N */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  79 O */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  80 P */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  81 Q */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  82 R */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  83 S */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  84 T */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  85 U */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  86 V */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  87 W */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  88 X */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  89 Y */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  90 Z */	::com::sun::star::lang::KParseTokens::ASC_UPALPHA,
	/*  91 [ */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  92 \ */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  93 ] */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  94 ^ */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/*  95 _ */	::com::sun::star::lang::KParseTokens::ASC_UNDERSCORE,
	/*  96 ` */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	//for ( i = 97; i < 123; i++ )
	/*  97 a */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/*  98 b */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/*  99 c */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 100 d */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 101 e */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 102 f */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 103 g */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 104 h */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 105 i */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 106 j */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 107 k */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 108 l */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 109 m */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 110 n */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 111 o */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 112 p */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 113 q */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 114 r */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 115 s */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 116 t */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 117 u */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 118 v */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 119 w */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 120 x */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 121 y */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 122 z */	::com::sun::star::lang::KParseTokens::ASC_LOALPHA,
	/* 123 { */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/* 124 | */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/* 125 } */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/* 126 ~ */	::com::sun::star::lang::KParseTokens::ASC_OTHER,
	/* 127   */	::com::sun::star::lang::KParseTokens::ASC_OTHER
};


// static
const sal_Unicode* CharacterClassification::StrChr( const sal_Unicode* pStr, sal_Unicode c )
{
	if ( !pStr )
		return NULL;
	while ( *pStr )
	{
		if ( *pStr == c )
			return pStr;
		pStr++;
	}
	return NULL;
}


sal_Int32 CharacterClassification::getParseTokensType( sal_Unicode c )
{
	if ( c < nDefCnt )
		return pParseTokensType[ sal_uInt8(c) ];
	else
	{
		using namespace ::com::sun::star::lang;

#if USE_I18N_DEFAULT_IMPLEMENTAION

		sal_Int32 nCharType = getCharType( c );
		if ( nCharType & KCharacterType::UPPER )
			return KParseTokens::UNI_UPALPHA;
		if ( nCharType & KCharacterType::LOWER )
			return KParseTokens::UNI_LOALPHA;
		if ( nCharType & KCharacterType::DIGIT )
			return KParseTokens::UNI_DIGIT;

#else

		//! all KParseTokens::UNI_... must be matched
		switch ( u_charType( c ) )
		{
			case UnicodeType::UPPERCASE_LETTER :
				return KParseTokens::UNI_UPALPHA;
			break;
			case UnicodeType::LOWERCASE_LETTER :
				return KParseTokens::UNI_LOALPHA;
			break;
			case UnicodeType::TITLECASE_LETTER :
				return KParseTokens::UNI_TITLE_ALPHA;
			break;
			case UnicodeType::MODIFIER_LETTER :
				return KParseTokens::UNI_MODIFIER_LETTER;
			break;
			case UnicodeType::OTHER_LETTER :
				return KParseTokens::UNI_OTHER_LETTER;
			break;
			case UnicodeType::DECIMAL_DIGIT_NUMBER :
				return KParseTokens::UNI_DIGIT;
			break;
			case UnicodeType::LETTER_NUMBER :
				return KParseTokens::UNI_LETTER_NUMBER;
			break;
			case UnicodeType::OTHER_NUMBER :
				return KParseTokens::UNI_OTHER_NUMBER;
			break;
		}

#endif

		return KParseTokens::UNI_OTHER;
	}
}


sal_Int32 CharacterClassification::getCharType( sal_Unicode ch )
{
	sal_Int32 result = 0;

#if USE_I18N_DEFAULT_IMPLEMENTAION

	using namespace ::com::sun::star::lang;

	BYTE nTypes = pIntl->GetCharType( ch );

	if ( nTypes & INTN_CHAR_UPPER )
		result |= KCharacterType::UPPER | KCharacterType::LETTER;

	else if ( nTypes & INTN_CHAR_LOWER )
		result |= KCharacterType::LOWER | KCharacterType::LETTER;

	else if ( nTypes & INTN_CHAR_NUMERIC )
		result |= KCharacterType::DIGIT;

	if ( ch < 128 )
	{
		if ( isprint( (unsigned char) ch ) )
			result |= KCharacterType::PRINTABLE;

		else if ( iscntrl( (unsigned char) ch ) )
			result |= KCharacterType::CONTROL;
	}

#else

	using namespace ::com::sun::star::lang;{

		if(u_isupper(ch))
			result |= KCharacterType::UPPER | KCharacterType::LETTER;

		else if(u_islower(ch))
			result |= KCharacterType::LOWER | KCharacterType::LETTER;

		else if(u_isdigit(ch))
			result |= KCharacterType::DIGIT;

		else if(u_istitle(ch))
			result |= KCharacterType::TITLE_CASE | KCharacterType::LETTER;

		else
		{
			switch ( u_charType( ch ) )
			{
				case UnicodeType::UPPERCASE_LETTER :
				case UnicodeType::LOWERCASE_LETTER :
				case UnicodeType::TITLECASE_LETTER :
				case UnicodeType::MODIFIER_LETTER :
				case UnicodeType::OTHER_LETTER :
					result |= KCharacterType::LETTER;
				break;
			}
		}


		if(u_isprint(ch))
			result |= KCharacterType::PRINTABLE;

		else if(u_iscntrl(ch))
			result |= KCharacterType::CONTROL;


		if(u_isbase(ch))
			result |= KCharacterType::BASE_FORM;
	}

#endif

	return result;
}


#if USE_I18N_DEFAULT_IMPLEMENTAION
void CharacterClassification::setupInternational( LanguageType eLang )
{
	if ( !pIntl )
		pIntl = new International( eLang );
	else if ( eLang != pIntl->GetLanguage() )
	{
		delete pIntl;
		pIntl = new International( eLang );
	}
}
#endif


void CharacterClassification::setupParserTable(
			const ::com::sun::star::lang::Locale& rLocale,
			sal_Int32 startCharTokenType,
			const String& userDefinedCharactersStart,
			sal_Int32 contCharTokenType,
			const String& userDefinedCharactersCont )
{
#if USE_I18N_DEFAULT_IMPLEMENTAION
	LanguageType eLang = ConvertIsoNamesToLanguage(
		rLocale.Language, rLocale.Country );
	BOOL bIntlEqual = pIntl && pIntl->GetLanguage() == eLang;
#else
	BOOL bIntlEqual = (rLocale.Language == aParserLocale.Language &&
		rLocale.Country == aParserLocale.Country &&
		rLocale.Variant == aParserLocale.Variant);
#endif
	if ( !pTable || !bIntlEqual ||
			startCharTokenType != nStartTypes ||
			contCharTokenType != nContTypes ||
			userDefinedCharactersStart != aStartChars ||
			userDefinedCharactersCont != aContChars )
		initParserTable( rLocale, startCharTokenType, userDefinedCharactersStart,
			contCharTokenType, userDefinedCharactersCont );
}


void CharacterClassification::initParserTable(
			const ::com::sun::star::lang::Locale& rLocale,
			sal_Int32 startCharTokenType,
			const String& userDefinedCharactersStart,
			sal_Int32 contCharTokenType,
			const String& userDefinedCharactersCont )
{
	// (Re)Init
	setupInternational( rLocale );
	// Memory of pTable is reused.
	if ( !pTable )
		pTable = new UPT_FLAG_TYPE[nDefCnt];
	memcpy( pTable, pDefaultParserTable, sizeof(UPT_FLAG_TYPE) * nDefCnt );
	// Start and cont tables only need reallocation if different length.
	if ( pStart && userDefinedCharactersStart.Len() != aStartChars.Len() )
	{
		delete [] pStart;
		pStart = NULL;
	}
	if ( pCont && userDefinedCharactersCont.Len() != aContChars.Len() )
	{
		delete [] pCont;
		pCont = NULL;
	}
	nStartTypes = startCharTokenType;
	nContTypes = contCharTokenType;
	aStartChars = userDefinedCharactersStart;
	aContChars = userDefinedCharactersCont;

	// specials
#if USE_I18N_DEFAULT_IMPLEMENTAION
	cGroupSep = pIntl->GetNumThousandSep();
	cDecimalSep = pIntl->GetNumDecimalSep();
#else
//!!!!!
//!TODO: Locale dependent implementation
//!!!!!
	cGroupSep = ',';
	cDecimalSep = '.';
//!!!!!
#endif
	if ( cGroupSep < nDefCnt )
		pTable[cGroupSep] |= TOKEN_VALUE;
	if ( cDecimalSep < nDefCnt )
		pTable[cDecimalSep] |= TOKEN_CHAR_VALUE | TOKEN_VALUE;

	// Modify characters according to KParseTokens definitions.
	{
		using namespace ::com::sun::star::lang::KParseTokens;
		sal_uInt8 i;

		if ( !(nStartTypes & ASC_UPALPHA) )
			for ( i = 65; i < 91; i++ )
				pTable[i] &= ~TOKEN_CHAR_WORD;	// not allowed as start character
		if ( !(nContTypes & ASC_UPALPHA) )
			for ( i = 65; i < 91; i++ )
				pTable[i] &= ~TOKEN_WORD;		// not allowed as cont character

		if ( !(nStartTypes & ASC_LOALPHA) )
			for ( i = 97; i < 123; i++ )
				pTable[i] &= ~TOKEN_CHAR_WORD;	// not allowed as start character
		if ( !(nContTypes & ASC_LOALPHA) )
			for ( i = 97; i < 123; i++ )
				pTable[i] &= ~TOKEN_WORD;		// not allowed as cont character

		if ( nStartTypes & ASC_DIGIT )
			for ( i = 48; i < 58; i++ )
				pTable[i] |= TOKEN_CHAR_WORD;	// allowed as start character
		if ( !(nContTypes & ASC_DIGIT) )
			for ( i = 48; i < 58; i++ )
				pTable[i] &= ~TOKEN_WORD;		// not allowed as cont character

		if ( !(nStartTypes & ASC_UNDERSCORE) )
			pTable[95] &= ~TOKEN_CHAR_WORD;		// not allowed as start character
		if ( !(nContTypes & ASC_UNDERSCORE) )
			pTable[95] &= ~TOKEN_WORD;			// not allowed as cont character

		if ( nStartTypes & ASC_DOLLAR )
			pTable[36] |= TOKEN_CHAR_WORD;		// allowed as start character
		if ( nContTypes & ASC_DOLLAR )
			pTable[36] |= TOKEN_WORD;			// allowed as cont character

		if ( nStartTypes & ASC_DOT )
			pTable[46] |= TOKEN_CHAR_WORD;		// allowed as start character
		if ( nContTypes & ASC_DOT )
			pTable[46] |= TOKEN_WORD;			// allowed as cont character

		if ( nStartTypes & ASC_COLON )
			pTable[58] |= TOKEN_CHAR_WORD;		// allowed as start character
		if ( nContTypes & ASC_COLON )
			pTable[58] |= TOKEN_WORD;			// allowed as cont character

		if ( nStartTypes & ASC_CONTROL )
			for ( i = 1; i < 32; i++ )
				pTable[i] |= TOKEN_CHAR_WORD;	// allowed as start character
		if ( nContTypes & ASC_CONTROL )
			for ( i = 1; i < 32; i++ )
				pTable[i] |= TOKEN_WORD;		// allowed as cont character

		if ( nStartTypes & ASC_ANY_BUT_CONTROL )
			for ( i = 32; i < nDefCnt; i++ )
				pTable[i] |= TOKEN_CHAR_WORD;	// allowed as start character
		if ( nContTypes & ASC_ANY_BUT_CONTROL )
			for ( i = 32; i < nDefCnt; i++ )
				pTable[i] |= TOKEN_WORD;		// allowed as cont character

	}

	// Merge in (positively override with) user defined characters.
	// StartChars
	xub_StrLen nLen = aStartChars.Len();
	if ( nLen )
	{
		if ( !pStart )
			pStart = new UPT_FLAG_TYPE[ nLen ];
		for ( xub_StrLen j=0; j<nLen; j++ )
		{
			pStart[j] = TOKEN_CHAR_WORD;
			sal_Unicode c = aStartChars.GetChar( j );
			if ( c < nDefCnt )
				pTable[c] |= TOKEN_CHAR_WORD;
		}
	}
	// ContChars
	nLen = aContChars.Len();
	if ( nLen )
	{
		if ( !pCont )
			pCont = new UPT_FLAG_TYPE[ nLen ];
		for ( xub_StrLen j=0; j<nLen; j++ )
		{
			pCont[j] = TOKEN_WORD;
			sal_Unicode c = aContChars.GetChar( j );
			if ( c < nDefCnt )
				pTable[c] |= TOKEN_WORD;
		}
	}
}


void CharacterClassification::destroyParserTable()
{
	if ( pCont )
		delete [] pCont;
	if ( pStart )
		delete [] pStart;
	if ( pTable )
		delete [] pTable;
}


UPT_FLAG_TYPE CharacterClassification::getFlags( sal_Unicode c )
{
	UPT_FLAG_TYPE nMask;
	if ( c < nDefCnt )
		nMask = pTable[ sal_uInt8(c) ];
	else
		nMask = getFlagsExtended( c );
	switch ( eState )
	{
		case ssGetChar :
		case ssGetWordFirstChar :
			if ( !(nMask & TOKEN_CHAR_WORD) )
			{
				nMask |= getStartCharsFlags( c );
				if ( nMask & TOKEN_CHAR_WORD )
					nMask &= ~TOKEN_EXCLUDED;
			}
		break;
		case ssGetValue :
		case ssGetWord :
			if ( !(nMask & TOKEN_WORD) )
			{
				nMask |= getContCharsFlags( c );
				if ( nMask & TOKEN_WORD )
					nMask &= ~TOKEN_EXCLUDED;
			}
		break;
		// other cases aren't needed
	}
	return nMask;
}


UPT_FLAG_TYPE CharacterClassification::getFlagsExtended( sal_Unicode c )
{
	if ( c == cGroupSep )
		return TOKEN_VALUE;
	else if ( c == cDecimalSep )
		return TOKEN_CHAR_VALUE | TOKEN_VALUE;
	using namespace ::com::sun::star::lang;
	BOOL bStart = (eState == ssGetChar || eState == ssGetWordFirstChar);
	sal_Int32 nTypes = (bStart ? nStartTypes : nContTypes);

#if USE_I18N_DEFAULT_IMPLEMENTAION

	sal_Int32 nCharType = getCharType( c );
	if ( nCharType & KCharacterType::UPPER )
		return (nTypes & KParseTokens::UNI_UPALPHA) ?
			(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
			TOKEN_ILLEGAL;
	if ( nCharType & KCharacterType::LOWER )
		return (nTypes & KParseTokens::UNI_LOALPHA) ?
			(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
			TOKEN_ILLEGAL;
	if ( nCharType & KCharacterType::DIGIT )
		return ((nTypes & (KParseTokens::UNI_DIGIT |
				KParseTokens::UNI_LETTER_NUMBER |
				KParseTokens::UNI_OTHER_NUMBER)) ?
			(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
			TOKEN_ILLEGAL) | TOKEN_DIGIT_FLAGS;

#else

	//! all KParseTokens::UNI_... must be matched
	switch ( u_charType( c ) )
	{
		case UnicodeType::UPPERCASE_LETTER :
			return (nTypes & KParseTokens::UNI_UPALPHA) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL;
		break;
		case UnicodeType::LOWERCASE_LETTER :
			return (nTypes & KParseTokens::UNI_LOALPHA) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL;
		break;
		case UnicodeType::TITLECASE_LETTER :
			return (nTypes & KParseTokens::UNI_TITLE_ALPHA) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL;
		break;
		case UnicodeType::MODIFIER_LETTER :
			return (nTypes & KParseTokens::UNI_MODIFIER_LETTER) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL;
		break;
		case UnicodeType::OTHER_LETTER :
			return (nTypes & KParseTokens::UNI_OTHER_LETTER) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL;
		break;
		case UnicodeType::DECIMAL_DIGIT_NUMBER :
			return ((nTypes & KParseTokens::UNI_DIGIT) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL) | TOKEN_DIGIT_FLAGS;
		break;
		case UnicodeType::LETTER_NUMBER :
			return ((nTypes & KParseTokens::UNI_LETTER_NUMBER) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL) | TOKEN_DIGIT_FLAGS;
		break;
		case UnicodeType::OTHER_NUMBER :
			return ((nTypes & KParseTokens::UNI_OTHER_NUMBER) ?
				(bStart ? TOKEN_CHAR_WORD : TOKEN_WORD) :
				TOKEN_ILLEGAL) | TOKEN_DIGIT_FLAGS;
		break;
	}

#endif
	return TOKEN_ILLEGAL;
}


UPT_FLAG_TYPE CharacterClassification::getStartCharsFlags( sal_Unicode c )
{
	if ( pStart )
	{
		const sal_Unicode* pStr = aStartChars.GetBuffer();
		const sal_Unicode* p = StrChr( pStr, c );
		if ( p )
			return pStart[ p - pStr ];
	}
	return TOKEN_ILLEGAL;
}


UPT_FLAG_TYPE CharacterClassification::getContCharsFlags( sal_Unicode c )
{
	if ( pCont )
	{
		const sal_Unicode* pStr = aContChars.GetBuffer();
		const sal_Unicode* p = StrChr( pStr, c );
		if ( p )
			return pCont[ p - pStr ];
	}
	return TOKEN_ILLEGAL;
}


void CharacterClassification::parseText( ::com::sun::star::lang::ParseResult& r,
			const ::rtl::OUString& rText, sal_Int32 nPos, sal_Int32 nTokenType )
{
	using namespace ::com::sun::star::lang;
	const sal_Unicode* const pStart = rText.getStr() + nPos;
	const sal_Unicode* pSym = pStart;
	const sal_Unicode* pSrc = pSym;
	String aSymbol;
	sal_Unicode c = *pSrc;
	sal_Unicode cLast = 0;
	BOOL bQuote = FALSE;
	BOOL bMightBeWord = TRUE;
	BOOL bMightBeWordLast = TRUE;
	eState = ssGetChar;
	while ( (c != 0) && (eState != ssStop) )
	{
		pSrc++;
		UPT_FLAG_TYPE nMask = getFlags( c );
		if ( nMask & TOKEN_EXCLUDED )
			eState = ssBounce;
		if ( bMightBeWord )
		{	// only relevant for ssGetValue fall back
			if ( eState == ssGetChar )
				bMightBeWord = ((nMask & TOKEN_CHAR_WORD) != 0);
			else
				bMightBeWord = ((nMask & TOKEN_WORD) != 0);
		}
		sal_Int32 nParseTokensType = getParseTokensType( c );
		switch (eState)
		{
			case ssGetChar :
			{
				if ( nMask & TOKEN_CHAR_VALUE )
				{	//! must be first, may fall back to ssGetWord via bMightBeWord
					eState = ssGetValue;
					if ( (nMask & TOKEN_VALUE_DIGIT) && 128 <= c )
						r.TokenType = KParseType::UNI_NUMBER;
					else
						r.TokenType = KParseType::ASC_NUMBER;
				}
				else if ( nMask & TOKEN_CHAR_WORD )
				{
					eState = ssGetWord;
					r.TokenType = KParseType::IDENTNAME;
				}
				else if ( nMask & TOKEN_NAME_SEP )
				{
					eState = ssGetWordFirstChar;
					bQuote = TRUE;
					pSym++;
					nParseTokensType = 0;	// will be taken of first real character
					r.TokenType = KParseType::SINGLE_QUOTE_NAME;
				}
				else if ( nMask & TOKEN_CHAR_STRING )
				{
					eState = ssGetString;
					pSym++;
					nParseTokensType = 0;	// will be taken of first real character
					r.TokenType = KParseType::DOUBLE_QUOTE_STRING;
				}
				else if ( nMask & TOKEN_CHAR_DONTCARE )
				{
					if ( nStartTypes & KParseTokens::IGNORE_LEADING_WS )
					{
						r.LeadingWhiteSpace++;
						pSym++;
						nParseTokensType = 0;	// wait until real character
						bMightBeWord = TRUE;
					}
					else
						eState = ssBounce;
				}
				else if ( nMask & TOKEN_CHAR_BOOL )
				{
					eState = ssGetBool;
					r.TokenType = KParseType::BOOLEAN;
				}
				else if ( nMask & TOKEN_CHAR )
				{	//! must be last
					eState = ssStop;
					r.TokenType = KParseType::ONE_SINGLE_CHAR;
				}
				else
					eState = ssBounce;		// not known
			}
			break;
			case ssGetValue :
			{
				if ( (nMask & TOKEN_VALUE_DIGIT) && 128 <= c )
					r.TokenType = KParseType::UNI_NUMBER;
				if ( nMask & TOKEN_VALUE )
					;	// keep it going
				else if ( c == 'E' || c == 'e' )
				{
					UPT_FLAG_TYPE nNext = getFlags( *pSrc );
					if ( nNext & TOKEN_VALUE_EXP )
						;	// keep it going
					else if ( bMightBeWord && ((nNext & TOKEN_WORD) || !*pSrc) )
					{	// might be a numerical name (1.2efg)
						eState = ssGetWord;
						r.TokenType = KParseType::IDENTNAME;
					}
					else
						eState = ssStopBack;
				}
				else if ( nMask & TOKEN_VALUE_SIGN )
				{
					if ( (cLast == 'E') || (cLast == 'e') )
					{
						UPT_FLAG_TYPE nNext = getFlags( *pSrc );
						if ( nNext & TOKEN_VALUE_EXP_VALUE )
							;	// keep it going
						else if ( bMightBeWord && ((nNext & TOKEN_WORD) || !*pSrc) )
						{	// might be a numerical name (1.2e+fg)
							eState = ssGetWord;
							r.TokenType = KParseType::IDENTNAME;
						}
						else
							eState = ssStopBack;
					}
					else if ( bMightBeWord )
					{	// might be a numerical name (1.2+fg)
						eState = ssGetWord;
						r.TokenType = KParseType::IDENTNAME;
					}
					else
						eState = ssStopBack;
				}
				else if ( bMightBeWord && (nMask & TOKEN_WORD) )
				{	// might be a numerical name (1995.A1)
					eState = ssGetWord;
					r.TokenType = KParseType::IDENTNAME;
				}
				else
					eState = ssStopBack;
			}
			break;
			case ssGetWordFirstChar :
				eState = ssGetWord;
				// fall thru
			case ssGetWord :
			{
				if ( nMask & TOKEN_WORD )
					;	// keep it going
				else if ( nMask & TOKEN_NAME_SEP )
				{
					if ( bQuote )
					{
						if ( cLast == '\\' )
						{	// escaped
							aSymbol.Append( pSym, pSrc - pSym - 2 );
							aSymbol += c;
						}
						else
						{
							eState = ssStop;
							aSymbol.Append( pSym, pSrc - pSym - 1 );
						}
						pSym = pSrc;
					}
					else
						eState = ssStopBack;
				}
				else if ( bQuote )
					;	// keep it going
				else
					eState = ssStopBack;
			}
			break;
			case ssGetString :
			{
				if ( nMask & TOKEN_STRING_SEP )
				{
					if ( cLast == '\\' )
					{	// escaped
						aSymbol.Append( pSym, pSrc - pSym - 2 );
						aSymbol += c;
					}
					else if ( c == *pSrc )
					{	// "" => literal " escaped
						aSymbol.Append( pSym, pSrc - pSym );
						pSrc++;
					}
					else
					{
						eState = ssStop;
						aSymbol.Append( pSym, pSrc - pSym - 1 );
					}
					pSym = pSrc;
				}
			}
			break;
			case ssGetBool :
			{
				if ( (nMask & TOKEN_BOOL) )
					eState = ssStop;	// maximum 2: <, >, <>, <=, >=
				else
					eState = ssStopBack;
			}
			break;
		}
		if ( !(r.TokenType & nTokenType) )
		{
			if ( (r.TokenType & (KParseType::ASC_NUMBER | KParseType::UNI_NUMBER))
					&& (nTokenType & KParseType::IDENTNAME) && bMightBeWord )
				;	// keep it
			else
				eState = ssBounce;
		}
		if ( eState == ssBounce )
		{
			r.TokenType = 0;
			eState = ssStopBack;
		}
		if ( eState == ssStopBack )
		{	// put back
			pSrc--;
			bMightBeWord = bMightBeWordLast;
			eState = ssStop;
		}
		if ( eState != ssStop )
		{
			if ( !r.StartFlags )
				r.StartFlags |= nParseTokensType;
			else
				r.ContFlags |= nParseTokensType;
		}
		bMightBeWordLast = bMightBeWord;
		cLast = c;
		c = *pSrc;
	}
	r.CharLen = pSrc - pStart;
	r.EndPos = nPos + (pSrc - pStart);
	if ( r.TokenType & KParseType::ASC_NUMBER )
	{
		int nErrno;
		r.Value = SolarMath::StringToDouble( pStart + r.LeadingWhiteSpace,
			cGroupSep, cDecimalSep, nErrno );
		if ( bMightBeWord )
			r.TokenType |= KParseType::IDENTNAME;
	}
	else if ( r.TokenType & KParseType::UNI_NUMBER )
	{
//!!!!!
//!TODO: r.Value = implementation
//!!!!!
		if ( bMightBeWord )
			r.TokenType |= KParseType::IDENTNAME;
	}
	else if ( r.TokenType & (KParseType::SINGLE_QUOTE_NAME | KParseType::DOUBLE_QUOTE_STRING) )
	{
		if ( pSym < pSrc )
		{	//! open quote
			aSymbol.Append( pSym, pSrc - pSym );
			r.TokenType |= KParseType::MISSING_QUOTE;
		}
		r.DequotedNameOrString = aSymbol;
	}
}

