/*************************************************************************
 *
 *  $RCSfile: localedata.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <localedata.hxx>
#include "localedata_nf.hxx"

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _TOOLS_INTN_HXX
#include <tools/intn.hxx>
#endif
#ifndef _ISOLANG_HXX
#include <tools/isolang.hxx>
#endif

#pragma hdrstop


using namespace ::com::sun::star::lang;


LocaleData::LocaleData()
		:
		pIntl( NULL ),
		pFormats( NULL )
{
}


LocaleData::~LocaleData()
{
	delete pFormats;
	delete pIntl;
}


void LocaleData::setupInternational( const ::com::sun::star::lang::Locale& rLocale )
{
	LanguageType eLang = ConvertIsoNamesToLanguage( rLocale.Language,
													rLocale.Country );
	if ( !pIntl )
		pIntl = new International( eLang );
	else if ( eLang != pIntl->GetLanguage() )
	{
		delete pIntl;
		pIntl = new International( eLang );
		if ( pFormats )
			pFormats->scheduleIntlChange( pIntl );
	}
}


::com::sun::star::lang::LocaleDataItem SAL_CALL
LocaleData::getLocaleItem( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
	setupInternational( rLocale );
	LocaleDataItem aItem(
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "generic" ) ),
		String( pIntl->GetDateSep() ),
		String( pIntl->GetNumThousandSep() ),
		String( pIntl->GetNumDecimalSep() ),
		String( pIntl->GetTimeSep() ),
		String( pIntl->GetTime100SecSep() ),
		String( pIntl->GetListSep() ),
		String( pIntl->GetQuotationMarkStart() ),
		String( pIntl->GetQuotationMarkEnd() ),
		String( pIntl->GetDoubleQuotationMarkStart() ),
		String( pIntl->GetDoubleQuotationMarkEnd() ),
		String( pIntl->GetTimeAM() ),
		String( pIntl->GetTimePM() ),
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "metric" ) )
	);
	switch ( pIntl->GetMeasurementSystem() )
	{
		case MEASURE_METRIC :
			// we have that already
		break;
		case MEASURE_US :
			aItem.measurementSystem = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "us" ) );
		break;
		default:
			DBG_ERRORFILE( "unknown measurement system" );
			aItem.measurementSystem = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "unknown" ) );
	}
	return aItem;
}


::com::sun::star::uno::Sequence< ::com::sun::star::lang::Calendar > SAL_CALL
LocaleData::getAllCalendars( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
	setupInternational( rLocale );
	::com::sun::star::uno::Sequence< Calendar > aSeq(1);
	int j;
	::com::sun::star::uno::Sequence< CalendarItem > aDays(7);
	for ( j=0; j<7; j++ )
	{
		aDays[j] = CalendarItem(
			pIntl->GetDayText( DayOfWeek(j) ),			// strID (==FullName)
			pIntl->GetAbbrevDayText( DayOfWeek(j) ),	// strAbbrevName
			pIntl->GetDayText( DayOfWeek(j) )			// strFullName
			);
	}
	::com::sun::star::uno::Sequence< CalendarItem > aMonths(12);
	for ( j=0; j<12; j++ )
	{
		aMonths[j] = CalendarItem(
			pIntl->GetAbbrevMonthText(j), 	// strID (==AbbrevName)
			pIntl->GetAbbrevMonthText(j),	// strAbbrevName
			pIntl->GetMonthText(j)			// strFullName
			);
	}
	::com::sun::star::uno::Sequence< CalendarItem > aEras(2);
	// International doesn't know anything about eras
	aEras[0] = CalendarItem(
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "ad" ) ),
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "AD" ) ),
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "AD" ) )
		);
	aEras[1] = CalendarItem(
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "bc" ) ),
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "BC" ) ),
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "BC" ) )
		);
	aSeq[0] = Calendar(
		aDays,								// sequenceDays
		aMonths,							// sequenceMonths
		aEras,								// sequenceEras
		pIntl->GetDayText( pIntl->GetWeekStart() ),		// strStartOfWeekDay
		4,		// !International			// shortMinimalDaysInFirstWeek
		sal_True,							// boolDefaultCalendar
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "gregorian" ) )		// strCalendarID
		);
	return aSeq;
}


::com::sun::star::uno::Sequence< ::com::sun::star::lang::Currency > SAL_CALL
LocaleData::getAllCurrencies( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
	setupInternational( rLocale );
	String aEmpty;
	::com::sun::star::uno::Sequence< ::com::sun::star::lang::Currency > aSeq(1);
	aSeq[0] = ::com::sun::star::lang::Currency(
		aEmpty,							// strCurrencyID (International doesn't have any)
		pIntl->GetCurrSymbol(),			// strCurrencySymbol
		pIntl->GetCurrBankSymbol(),		// strBankSymbol
		aEmpty,							// strCurrencyName (International doesn't have any)
		sal_True						// boolDefaultCurrency (always the only one)
		);
	return aSeq;
}


::com::sun::star::uno::Sequence< ::com::sun::star::lang::FormatElement > SAL_CALL
LocaleData::getAllFormats( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
	setupInternational( rLocale );
	::com::sun::star::uno::Sequence< ::com::sun::star::lang::FormatElement > aSeq( NF_INDEX_TABLE_ENTRIES );
	if ( !pFormats )
		pFormats = new LocaleDataFormats( pIntl );
	pFormats->getFormats( aSeq );
	return aSeq;
}


::com::sun::star::uno::Sequence< ::com::sun::star::lang::Implementation > SAL_CALL
LocaleData::getCollatorImplementations( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
//	setupInternational( rLocale );
#if 1
	// I18N localedata_... sets the count to zero for "normal" implementation
	::com::sun::star::uno::Sequence< ::com::sun::star::lang::Implementation > aSeq(0);
#else
	// if count should not be zero
	::com::sun::star::uno::Sequence< ::com::sun::star::lang::Implementation > aSeq(1);
	aSeq[0] = ::com::sun::star::lang::Implementation(
		::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "normal" ) ),	// strCollatorID
		sal_True													// boolDefaultCollator
		);
#endif
	return aSeq;
}


::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL
LocaleData::getTransliterations( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
//	setupInternational( rLocale );
	::com::sun::star::uno::Sequence< ::rtl::OUString > aSeq(3);
	aSeq[0] = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "toUpper" ) );
	aSeq[1] = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "toLower" ) );
	aSeq[2] = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "ignoreCase" ) );
	return aSeq;
}


::com::sun::star::lang::LCInfo SAL_CALL
LocaleData::getLCInfo( const ::com::sun::star::lang::Locale& rLocale ) throw(::com::sun::star::uno::RuntimeException)
{
	// We could use the Locale directly, but this way we get what the
	// International really is.
	setupInternational( rLocale );
	String aLanguage, aCountry, aEmpty;
	ConvertLanguageToIsoNames( pIntl->GetLanguage(), aLanguage, aCountry );
	::com::sun::star::lang::LCInfo aInfo(
		aLanguage,		// strLangID
		aEmpty,			// strLangDefaultName
		aCountry,		// strCountryID
		aEmpty,			// strCountryDefaultName
		aEmpty			// strVariant
		);
	return aInfo;
}

