/*************************************************************************
 *
 *  $RCSfile: olecontr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __IE_OLECONTR_HXX__
#define __IE_OLECONTR_HXX__

#include <sal/types.h>

#include <oleidl.h>
#include <docobj.h>
#include <mshtmhst.h>
#include <exdisp.h>

class IExplorerController_Impl;

//---------------------------------------------------------------
//  IOleObject
//---------------------------------------------------------------

enum OLE_SERVER_STATE
{
    OS_PASSIVE,
    OS_LOADED,                          // handler but no server
    OS_RUNNING,                         // server running, invisible
    OS_INPLACE,                         // server running, inplace-active, no U.I.
    OS_UIACTIVE,                        // server running, inplace-active, w/ U.I.
    OS_OPEN                             // server running, open-edited
};


// prototypes for HIMETRIC stuff.
//

void InitPixelsPerInch(VOID);
int HPixFromHimetric( long lHi );
int VPixFromHimetric( long lHi );
long HimetricFromHPix( int iPix );
long HimetricFromVPix( int iPix );


class Container_Impl;

//+---------------------------------------------------------------------------
//
//  Class:      COleContainer ()
//
//  Purpose:    our implementation of IOleContainer.  does nothing.  But the
//              WebBrowser OC needs to see it or frames don't work.  This is
//              their bug.
//
//----------------------------------------------------------------------------
class COleContainer
	: public IOleContainer
{
public:
    // IUnknown stuff
    STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
    STDMETHOD_(ULONG, AddRef)(VOID);
    STDMETHOD_(ULONG, Release)(VOID);

	// IOleContainer
    STDMETHOD(EnumObjects)(DWORD grfFlags, IEnumUnknown **ppenum);
    STDMETHOD(LockContainer)(int fLock); // param is BOOL
    STDMETHOD(ParseDisplayName)(
		IBindCtx *pbc,
		LPOLESTR pszDisplayName,
		ULONG *pchEaten,
		IMoniker **ppmkOut);

    COleContainer( Container_Impl* pCtr )
		: _pContainer( pCtr )
	{}

protected:
	Container_Impl*		_pContainer;
};

//+---------------------------------------------------------------------------
//
//  Class:      COleClientSite ()
//
//  Purpose:    our implementation of IOleClientSite
//
//  Interface:  COleClientSite         -- ctor
//              QueryInterface         -- gimme an interface!
//              AddRef                 -- bump up refcount
//              Release                -- bump down refcount
//              SaveObject             -- returns E_FAIL
//              GetMoniker             -- E_NOTIMPL
//              GetContainer           -- returns our COleContainer impl
//              ShowObject             -- just say OK
//              OnShowWindow           -- just say OK
//              RequestNewObjectLayout -- E_NOTIMPL
//
//  Notes:      probably the most important thing our IOleClientSite
//              implementation does is hand off our IOleContainer
//              implementation when GetContainer() is called.
//
//----------------------------------------------------------------------------
class COleClientSite
	: public IOleClientSite
{
public:
    // IUnknown stuff
    STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
    STDMETHOD_(ULONG, AddRef)(VOID);
    STDMETHOD_(ULONG, Release)(VOID);

    // IOleClientSite stuff
    STDMETHOD(SaveObject)(VOID);
    STDMETHOD(GetMoniker)(
		DWORD dwAssign,
		DWORD dwWhichMoniker,
		LPMONIKER FAR * ppmk );
    STDMETHOD(GetContainer)(LPOLECONTAINER FAR * pOleCtr);
    STDMETHOD(ShowObject)(VOID);
    STDMETHOD(OnShowWindow)(int bShow);
    STDMETHOD(RequestNewObjectLayout)(VOID);

    COleClientSite( Container_Impl* pCtr )
		: _pContainer( pCtr )
	{}

protected:
	Container_Impl*		_pContainer;
};

//  //+---------------------------------------------------------------------------
//  //
//  // Class:      COleControlSite
//  //
//  // Purpose:    our implementation of IOleControlSite
//  //
//  //----------------------------------------------------------------------------
//  class COleControlSite
//  	: public IOleControlSite
//  {
//  public:
//      // IUnknown stuff
//      STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
//      STDMETHOD_(ULONG, AddRef)(VOID);
//      STDMETHOD_(ULONG, Release)(VOID);

//      // IOleControlSite stuff
//  	STDMETHOD(OnControlInfoChanged)(VOID);
//  	STDMETHOD(LockInPlaceActive)( BOOL fLock );
//  	STDMETHOD(GetExtendedControl)( IDispatch __RPC_FAR *__RPC_FAR *ppDisp );
//  	STDMETHOD(TransformCoords)( POINTL __RPC_FAR *pPtlHimetric,
//  								POINTF __RPC_FAR *pPtfContainer,
//  								DWORD dwFlags );
//  	STDMETHOD(TranslateAccelerator)( MSG __RPC_FAR *pMsg,
//  									 DWORD grfModifiers );
//  	STDMETHOD(OnFocus)( BOOL fGotFocus );
//  	STDMETHOD(ShowPropertyFrame)(VOID);

//      COleControlSite( Container_Impl* pCtr )
//  		: _pContainer( pCtr )
//  	{}

//  protected:
//  	Container_Impl*		_pContainer;
//  };

//  //+---------------------------------------------------------------------------
//  //
//  //  Class:      CAdviseSink ()
//  //
//  //  Purpose:    IAdviseSink implementation
//  //
//  //  Notes:      It turns out we're not doing anything with this
//  //              currently, but it would be a bad idea to remove this.
//  //              The WebBrowser OC *does* ask for this, and calls the
//  //              OnViewChange() from time to time, although we've found
//  //              it best to just let the OC handle it's own visual state.
//  //
//  //----------------------------------------------------------------------------
//  class CAdviseSink
//  	: public IAdviseSink
//  {
//  public:
//  	// IUnknown stuff
//  	STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
//  	STDMETHOD_(ULONG, AddRef)(VOID);
//  	STDMETHOD_(ULONG, Release)(VOID);

//  	// IAdviseSink stuff
//  	STDMETHOD_(VOID, OnDataChange)(LPFORMATETC pFE, LPSTGMEDIUM pStgMedium);
//  	STDMETHOD_(VOID, OnViewChange)(DWORD dwAspect, LONG lIndex);
//  	STDMETHOD_(VOID, OnRename)(LPMONIKER pmkNew);
//  	STDMETHOD_(VOID, OnSave)(VOID);
//  	STDMETHOD_(VOID, OnClose)(VOID);

//      CAdviseSink( Container_Impl* pCtr )
//  		: _pContainer( pCtr )
//  	{}

//  protected:
//  	Container_Impl*		_pContainer;
//  };


//+---------------------------------------------------------------------------
//
//  Class:      CInPlaceFrame ()
//
//  Purpose:
//
//  Interface:  CInPlaceFrame        -- ctor
//              QueryInterface       -- gimme an interface!
//              AddRef               -- bump up refcount
//              Release              -- decrement refcount
//              GetWindow            -- from IOleWindow - returns frame hWnd
//              ContextSensitiveHelp -- never implemented by design
//              GetBorder            -- for toolbar negotiation
//              RequestBorderSpace   -- ditto
//              SetBorderSpace       -- ditto
//              SetActiveObject      -- called whenever URL changes
//              InsertMenus          -- menu negotiation
//              SetMenu              -- ditto
//              RemoveMenus          -- ditto
//              SetStatusText        -- called by OC to set status text
//              EnableModeless       -- we have no modeless dlgs.
//              TranslateAccelerator -- calls ::TranslateAccelerator
//
//----------------------------------------------------------------------------
class CInPlaceFrame
	: public IOleInPlaceFrame
{
public:
	// IUnknown stuff
	STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
	STDMETHOD_(ULONG, AddRef)(VOID);
	STDMETHOD_(ULONG, Release)(VOID);

	// IOleWindow stuff
	STDMETHOD(GetWindow)(HWND * phwnd);
	STDMETHOD(ContextSensitiveHelp)(int fEnterMode); // param is BOOL

	// IOleInPlaceUIWindow stuff
	STDMETHOD(GetBorder)(LPRECT lprectBorder);
	STDMETHOD(RequestBorderSpace)(LPCBORDERWIDTHS pborderwidths);
	STDMETHOD(SetBorderSpace)(LPCBORDERWIDTHS pborderwidths);
	STDMETHOD(SetActiveObject)(
        IOleInPlaceActiveObject * pActiveObject,
        LPCOLESTR                 pszObjName);

	// IOleInPlaceFrame stuff
	STDMETHOD(InsertMenus)(
        HMENU                hmenuShared,
        LPOLEMENUGROUPWIDTHS lpMenuWidths);

	STDMETHOD(SetMenu)(
        HMENU    hmenuShared,
        HOLEMENU holemenu,
        HWND     hwndActiveObject);

	STDMETHOD(RemoveMenus)(HMENU hmenuShared);
	STDMETHOD(SetStatusText)(LPCOLESTR pszStatusText);
	STDMETHOD(EnableModeless)(BOOL fEnable); // BOOL
	STDMETHOD(TranslateAccelerator)(LPMSG lpmsg, WORD wID);

    CInPlaceFrame( Container_Impl* pCtr )
		: _pContainer( pCtr )
	{}

protected:
	Container_Impl*		_pContainer;
};

//+---------------------------------------------------------------------------
//
//  Class:      CInPlaceSite ()
//
//  Purpose:    IOleInPlaceSite implementation.
//
//  Interface:  CInPlaceSite         -- ctor
//              QueryInterface       -- get a new interface
//              AddRef               -- bump ref count
//              Release              -- decrement ref count
//              GetWindow            -- returns frame window
//              ContextSensitiveHelp -- never implemented by design
//              CanInPlaceActivate   -- returns S_OK.
//              OnInPlaceActivate    -- caches IOleInPlaceObject ptr
//              OnUIActivate         -- returns S_OK  - sets state
//              GetWindowContext     -- returns IOleInPlaceFrame,
//                                              IOleInPlaceUIWindow,
//                                              PosRect and ClipRect
//              Scroll               -- never implemented by design.
//              OnUIDeactivate       -- obvious
//              OnInPlaceDeactivate  -- releases cached IOleInPlaceObject
//              DiscardUndoState     -- returns S_OK.
//              DeactivateAndUndo    -- deactivates in place active object
//              OnPosRectChange      -- never implemented by design
//
//----------------------------------------------------------------------------
class CInPlaceSite
	: public IOleInPlaceSite
{
public:
	// IUnknown stuff
	STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
	STDMETHOD_(ULONG, AddRef)(VOID);
	STDMETHOD_(ULONG, Release)(VOID);

	// IOleWindow stuff
	STDMETHOD(GetWindow)(HWND * phwnd);
	STDMETHOD(ContextSensitiveHelp)(int fEnterMode); //BOOL

	// IOleInPlaceSite stuff
	STDMETHOD(CanInPlaceActivate)(VOID);
	STDMETHOD(OnInPlaceActivate)(VOID);
	STDMETHOD(OnUIActivate)(VOID);
	STDMETHOD(GetWindowContext)(
		IOleInPlaceFrame** ppFrame,
		IOleInPlaceUIWindow** ppDoc,
		LPRECT prcPosRect,
		LPRECT prcClipRect,
		LPOLEINPLACEFRAMEINFO pFrameInfo );

	STDMETHOD(Scroll)(SIZE scrollExtant);
	STDMETHOD(OnUIDeactivate)(int fUndoable); // BOOL
	STDMETHOD(OnInPlaceDeactivate)(VOID);
	STDMETHOD(DiscardUndoState)(VOID);
	STDMETHOD(DeactivateAndUndo)(VOID);
	STDMETHOD(OnPosRectChange)(LPCRECT lprcPosRect);

    CInPlaceSite( Container_Impl* pCtr )
		: _pContainer( pCtr )
	{}

protected:
	Container_Impl*		_pContainer;
};

//+---------------------------------------------------------------------------
//
//  Class:      CCommandTarget ()
//
//  Purpose:    IOleCommandTarget.  It turns out the WebBrowser OC
//              communicates some of its status messages by calling
//              IOleCommandTarget->Exec(OLECMDID_SETPROGRESSTEXT);
//
//  Interface:  CCommandTarget -- ctor
//              QueryInterface -- gimme an interface
//              AddRef         -- bump up refcount
//              Release        -- release refcount
//              QueryStatus    -- what cmds do you support?
//              Exec           -- execute a command.
//
//----------------------------------------------------------------------------
class CCommandTarget
	: public IOleCommandTarget
{
public:
    // IUnknown stuff
    STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
    STDMETHOD_(ULONG, AddRef)(VOID);
    STDMETHOD_(ULONG, Release)(VOID);

    // IOleCommandTarget stuff
    STDMETHOD(QueryStatus)(const GUID * pguidCmdGroup,
                           ULONG        cCmds,
                           OLECMD       aCmds[],
                           OLECMDTEXT * pCmdText);

    STDMETHOD(Exec)(const GUID *  pguidCmdGroup,
					DWORD         nCmdID,
					DWORD         nCmdexecopt,
					LPVARIANTARG  pVarIn,
					LPVARIANTARG  pVarOut);

    CCommandTarget( Container_Impl* pCtr )
		: _pContainer( pCtr )
	{}

protected:
	Container_Impl*		_pContainer;
};

//-----------------------------------------------------------------------------
//CDocHostUIHandler is only implemented to let the web browser get the external
//dispatch. The external dispatch interface pointer is passed in createContainer
//and receives all DHTML window.external calls.
//The host won't get involved in any UI handling.
//-------------------------------------------------------------------------------
class CDocHostUIHandler: public IDocHostUIHandler
{

public:
	CDocHostUIHandler( Container_Impl* pCtr, IDispatch* pExt );
	~CDocHostUIHandler();
		// IUnknown
    STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR * ppv);
    STDMETHOD_(ULONG, AddRef)(VOID);
    STDMETHOD_(ULONG, Release)(VOID);

	// IDocHostUIHandler
	// This is only implemented for the GetExternal method that is necessary to extend the
	// DHTML model of the explorer.
	STDMETHOD(ShowContextMenu)(DWORD dwID, POINT* pptPosition, IUnknown* pCommandTarget,
		IDispatch* pDispatchObjectHit);
	// Called at initialisation to find UI styles from container
	STDMETHOD(GetHostInfo)(DOCHOSTUIINFO* pInfo);
	// Allows the host to replace the IE4/MSHTML menus and toolbars. 
	STDMETHOD(ShowUI)(DWORD dwID, IOleInPlaceActiveObject* pActiveObject, 
		IOleCommandTarget* pCommandTarget, IOleInPlaceFrame* pFrame, IOleInPlaceUIWindow* pDoc);
	// Called when IE4/MSHTML removes its menus and toolbars. 
	STDMETHOD(HideUI)();
	// Notifies the host that the command state has changed. 
	STDMETHOD(UpdateUI)();
	// Called from the IE4/MSHTML implementation of IOleInPlaceActiveObject::EnableModeless
	STDMETHOD(EnableModeless)(BOOL fEnable); // param is BOOL
	// Called from the IE4/MSHTML implementation of IOleInPlaceActiveObject::OnDocWindowActivate
	STDMETHOD(OnDocWindowActivate)(BOOL fActivate); // param is BOOL
	// Called from the IE4/MSHTML implementation of IOleInPlaceActiveObject::OnFrameWindowActivate. 
	STDMETHOD(OnFrameWindowActivate)(BOOL fActivate); // param is BOOL
	// Called from the IE4/MSHTML implementation of IOleInPlaceActiveObject::ResizeBorder.
	STDMETHOD(ResizeBorder)(LPCRECT prcBorder, IOleInPlaceUIWindow* pUIWindow, BOOL fFrameWindow); // fFrameWindow is BOOL
	// Called by IE4/MSHTML when IOleInPlaceActiveObject::TranslateAccelerator or IOleControlSite::TranslateAccelerator is called. 
	STDMETHOD(TranslateAccelerator)(LPMSG lpMsg, const GUID* pguidCmdGroup, DWORD nCmdID);
	// Returns the registry key under which IE4/MSHTML stores user preferences. 
	// Returns S_OK if successful, or S_FALSE otherwise. If S_FALSE, IE4/MSHTML will default to its own user options.
	STDMETHOD(GetOptionKeyPath)(BSTR* pbstrKey, DWORD dwReserved);
	// Called by IE4/MSHTML when it is being used as a drop target to allow the host to supply an alternative IDropTarget
	STDMETHOD(GetDropTarget)(IDropTarget* pDropTarget, IDropTarget** ppDropTarget);
	// Called by IE4/MSHTML to obtain the host's IDispatch interface
	STDMETHOD(GetExternal)(IDispatch** ppDispatch);
	// Called by IE4/MSHTML to allow the host an opportunity to modify the URL to be loaded
	STDMETHOD(TranslateUrl)(DWORD dwTranslate, OLECHAR* pchURLIn, OLECHAR** ppchURLOut);
	// Called on the host by IE4/MSHTML to allow the host to replace IE4/MSHTML's data object.
	// This allows the host to block certain clipboard formats or support additional clipboard formats. 
	STDMETHOD(FilterDataObject)(IDataObject* pDO, IDataObject** ppDORet);

public:
	IDispatch* m_externalDispatch;
protected:
	Container_Impl* m_pContainer;

};

//+---------------------------------------------------------------------------
//
//  Class:      Container_Impl ()
//
//  Purpose:    This is the one, the big kahuna.  Container_Impl is the
//              MSJ OLE Controls container that contains a single
//              OLE Control, the WebBrowser OC.  It contains
//              sub-objects which implement the various interfaces
//              we have to support (could have used multiple inheritance,
//              but this seemed more straightforward for our needs).
//
//              Conventions:  Interfaces we implement are contained objects
//                            of a class trivially derived from the interface,
//                            e.g., IOleInPlaceFrame is a contained
//                            instance of CInPlaceFrame called m_IPF.
//
//                            Interfaces we hold on the WebBrowser OC
//                            are pointers to the actual OLE interface.
//                            e.g., our pointer to the control's
//                            IOleControl interface is m_pOC.
//
//                            The contained sub-objects are all friends
//                            of the container - they are all conceptually
//                            the same object, but are implemented
//                            separately so as to cause the compiler to
//                            generate the correct vtable.
//
//----------------------------------------------------------------------------
class Container_Impl : public IUnknown
{
friend CInPlaceSite;
friend CInPlaceFrame;
friend COleClientSite;
//friend COleControlSite;
friend CCommandTarget;
friend CDocHostUIHandler;
friend Container_Impl* getContainerFromWindow( HWND hwnd );

public:
	Container_Impl( const HWND hWnd, IExplorerController_Impl* pController, IDispatch* pExt );

	void		shutdown();
	sal_Bool	scrollScreen( sal_Bool bDown );
	void		setPosSize( sal_Int32 nX, sal_Int32 nY, sal_Int32 nWidth, sal_Int32 nHeight );
	void		activate();
	void		deactivate();
	IWebBrowser2 * getWebBrowser();
	
	//====================================================================================
public:

    // IUnknown stuff
    STDMETHOD(QueryInterface)( REFIID riid, LPVOID FAR * ppv );
    STDMETHOD_(ULONG, AddRef)(VOID);
    STDMETHOD_(ULONG, Release)(VOID);

public:
	HRESULT				CreateBrowserOC( IDispatch * pSink );
    HRESULT				GetSize( LPRECT prc );
    HRESULT				SetSize( LPRECT prc, BOOL fMoveFrameWindow = TRUE );
    void				SetFrameWindow( HWND hWndFrame )
	{ _hWndFrame = hWndFrame; }
    HWND				GetFrameWindow()
	{ //_ASSERTE(_hWndFrame && ::IsWindow(_hWndFrame));
	  return _hWndFrame; }
    HWND				GetMainWindow()
	{ //_ASSERTE(_hWndBrowser && ::IsWindow(_hWndBrowser));
	  return _hWndBrowser; }

    //
    //  Whenever we display a modal dialog, we need to let
    //  our embeddings (the WebBrowser OC) know to disable
    //  any modeless dialogs the embedding is displaying.
    //
	void				EnableEmbeddingModelessDlgs( sal_Bool fEnable )
	{ LPOLEINPLACEACTIVEOBJECT pIPAO = GetIPAObject();
	 if (pIPAO)	pIPAO->EnableModeless(fEnable);	}

    HRESULT				NormalizeURL( LPSTR szPathIn, LPSTR szPathOut );

    OLE_SERVER_STATE	GetState()
	{ return _state; }
    VOID             SetState( OLE_SERVER_STATE state )
	{ _state = state; }
    sal_Bool             ModelessEnabled()
	{ return _fModelessEnabled; }

    HRESULT          EnableModeless( BOOL fEnable );
    LRESULT          SetFocus();

    LPOLEINPLACEACTIVEOBJECT GetIPAObject()
	{ return _pActiveObj; }

	//
	static LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK AcceleratorMsgProc( int code, WPARAM wParam, LPARAM lParam );

	static Container_Impl*	 s_pCurrentContainer;
	static HHOOK			 s_hAcceleratorHook;

protected:
    HRESULT			_SetExtent( LPRECT prc );
    void			_AdjustForTools( LPRECT prc );
    HRESULT			_TransAccelerator( LPMSG lpmsg, WORD wID );
    void			_GetDoVerbRect( LPRECT prc );
// Attributes ---------------------------------------------------
public:
	IExplorerController_Impl* m_pController;

private:
    COleClientSite           _CS;          // clientsite
//  	COleControlSite			 _OCS;		   // controlsite
//      CAdviseSink              _AS;          // advise sink
    CInPlaceFrame            _IPF;         // inplace frame
    CInPlaceSite             _IPS;         // inplace site object
    COleContainer            _OCtr;        // IOleContainer
    CCommandTarget           _OCT;         // IOleCommandTarget
	CDocHostUIHandler		m_DocHostUIHandler; // IDocHostUIHandler
    HWND                     _hWndBrowser; // hWnd of main browser
    HWND                     _hWndFrame;   // hWnd that contains OC Site
    LPUNKNOWN                _pUnk;        // the object itself.
    IWebBrowser2*            _pIWebBrowser2;        // pointer to IWebBrowser2 for OC
    LPVIEWOBJECT             _pVO;         // pointer to IViewObject
    LPOLEOBJECT              _pOO;         // pointer to IOleObject
    LPOLEINPLACEOBJECT       _pIPO;        // pointer to InPlaceActiveObject
    LPOLEINPLACEACTIVEOBJECT _pActiveObj;  // current active object
    LPOLECONTROL             _pOC;         // IOleControl interface for OC
	
    ULONG                    _Ref;         // refcount
    OLE_SERVER_STATE         _state;       // current OLE state of OC
    DWORD                    _dwMiscStatus;// misc status bits for OC
    sal_Bool                     _fModelessEnabled; // OC is putting up modal dlg?

	// cookie for Unadvise()
	DWORD					 _aWBCookie;
};

#endif



