/*************************************************************************
 *
 *  $RCSfile: himetric.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:30:50 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <windows.h>
#pragma hdrstop


#define HIMETRIC_PER_INCH 2540L

SIZE g_sizePixelsPerInch;

//+------------------------------------------------------------------------
//
//  Function:   InitPixelsPerInch
//
//  Synopsis:   Initializing coordinate mapping for screen pixels
//
//  Returns:    HRESULT; S_OK on success, E_OUTOFMEMORY otherwise
//
//-------------------------------------------------------------------------

void InitPixelsPerInch(VOID)
{
    HDC hdc;

    hdc = GetDC(NULL);
    if (hdc)
	{
		g_sizePixelsPerInch.cx = GetDeviceCaps( hdc, LOGPIXELSX );
		g_sizePixelsPerInch.cy = GetDeviceCaps( hdc, LOGPIXELSY );
		
		ReleaseDC(NULL, hdc);
	}
	else
	{
		g_sizePixelsPerInch.cx = 96;
		g_sizePixelsPerInch.cy = 96;
	}
}


//+---------------------------------------------------------------
//
//  Function:   HimetricFromHPix
//
//  Synopsis:   Converts horizontal pixel units to himetric units.
//
//----------------------------------------------------------------

long HimetricFromHPix(int iPix)
{
    if (! g_sizePixelsPerInch.cx)
        InitPixelsPerInch();

    return MulDiv( iPix, HIMETRIC_PER_INCH, g_sizePixelsPerInch.cx );
}

//+---------------------------------------------------------------
//
//  Function:   HimetricFromVPix
//
//  Synopsis:   Converts vertical pixel units to himetric units.
//
//----------------------------------------------------------------

long HimetricFromVPix( int iPix )
{
    if (! g_sizePixelsPerInch.cy)
        InitPixelsPerInch();

    return MulDiv( iPix, HIMETRIC_PER_INCH, g_sizePixelsPerInch.cy );
}

//+---------------------------------------------------------------
//
//  Function:   HPixFromHimetric
//
//  Synopsis:   Converts himetric units to horizontal pixel units.
//
//----------------------------------------------------------------

int HPixFromHimetric( long lHi )
{
    if (! g_sizePixelsPerInch.cx)
        InitPixelsPerInch();

    return MulDiv( g_sizePixelsPerInch.cx, lHi, HIMETRIC_PER_INCH );
}

//+---------------------------------------------------------------
//
//  Function:   VPixFromHimetric
//
//  Synopsis:   Converts himetric units to vertical pixel units.
//
//----------------------------------------------------------------

int VPixFromHimetric( long lHi )
{
    if (! g_sizePixelsPerInch.cy)
        InitPixelsPerInch();

    return MulDiv( g_sizePixelsPerInch.cy, lHi, HIMETRIC_PER_INCH );
}

//+---------------------------------------------------------------------------
//
//  Function:   PixelFromHMRect
//
//  Synopsis:   Converts a Himetric RECTL to a Pixel RECT
//
//----------------------------------------------------------------------------

void PixelFromHMRect( RECT *prcDest, RECTL *prcSrc )
{
    prcDest->left		= HPixFromHimetric( prcSrc->left );
    prcDest->top		= VPixFromHimetric( prcSrc->top );
    prcDest->right		= HPixFromHimetric( prcSrc->right );
    prcDest->bottom		= VPixFromHimetric( prcSrc->bottom );
}

//+---------------------------------------------------------------------------
//
//  Function:   HMFromPixelRect
//
//  Synopsis:   Converts a Pixel RECT to a Himetric RECTL
//
//----------------------------------------------------------------------------

void HMFromPixelRect( RECTL *prcDest, RECT *prcSrc )
{
    prcDest->left		= HimetricFromHPix( prcSrc->left );
    prcDest->top		= HimetricFromVPix( prcSrc->top );
    prcDest->right		= HimetricFromHPix( prcSrc->right );
    prcDest->bottom		= HimetricFromVPix( prcSrc->bottom );
}

