/*************************************************************************
 *
 *  $RCSfile: imaptest.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:31:17 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _TOOLS_RESMGR_HXX //autogen
#include <tools/resmgr.hxx>
#endif

#ifndef _SVTREEBOX_HXX //autogen
#include <svtools/svtreebx.hxx>
#endif

#ifndef _SV_WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif

#ifndef _SV_TOOLBOX_HXX //autogen
#include <vcl/toolbox.hxx>
#endif

#ifndef _SVEDIT_HXX //autogen
#include <svtools/svmedit.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _SV_HELP_HXX //autogen
#include <vcl/help.hxx>
#endif

#ifndef _INETMAIL_HXX
#include <inetmail.hxx>
#endif

#ifndef _INETMSG_HXX
#include <inetmsg.hxx>
#endif

#ifndef _INETIMAP_HXX
#include <inetimap.hxx>
#endif

#include <imaptest.hrc>

class MyWin;

// cmd-defines
#define IMAP_LOGIN			1L
#define IMAP_LIST			2L
#define IMAP_LOGOUT			3L

// res-stuff
static ResMgr* pAppResMgr = NULL;

class MyResId : public ResId
{
public:
	MyResId( USHORT nId );
};

MyResId::MyResId( USHORT nId ) : ResId(nId,pAppResMgr)
{
}

#define UNKNOWN -1

// ImapBox (speichert einen Mailbox-Eintrag)
class ImapBox
{
private:
	char 	cSep;
	String 	sName;
	INT8 	nHasChildren;
	BOOL    bSelectable;
	ULONG	nExists;
	ULONG	nNew;

public:
	ImapBox(const INetImapBox* pBox);
	const String GetBasename() const;
	const String& GetFullname() const {	return sName;}
	const String GetFullData() const;
	char GetSep() const {return cSep;}
	void  SetChildrenFlag(BOOL bFlag)
	   {nHasChildren = bFlag ? TRUE : FALSE;}
	INT8 HasChildren() const {return nHasChildren;}
	BOOL IsSelectable() const {return bSelectable;}
	void SetExists(ULONG nP) {nExists = nP;}
	void SetNew(ULONG nP) {nNew = nP;}
	ULONG GetExists() const {return nExists;}
	ULONG GetNew() const {return nNew;}
};

ImapBox::ImapBox(const INetImapBox* pBox)
	: cSep(pBox->GetHierarchySeparator()),
	  sName(pBox->GetName()),
	  nHasChildren(UNKNOWN),
	  bSelectable(pBox->GetIsSelectable()),
	  nExists(0),
	  nNew(0)
{
}

const String ImapBox::GetFullData() const
{
	String sTmp = GetBasename();
	sTmp += " (";
	sTmp += nNew;
	sTmp += "/";
	sTmp += nExists;
	sTmp += ')';
	return sTmp;
}

const String ImapBox::GetBasename() const
{
	USHORT i = sName.GetTokenCount(cSep);
	if(i)
		return sName.GetToken(i-1,cSep);
	else
		return sName;
}

class ImapTree : public SvTreeListBox
{
private:
	MyWin* pOwner;
	
public:
	ImapTree( MyWin* pOwn, Window* pParent, WinBits nWinStyle=0 )
		: pOwner(pOwn), SvTreeListBox( pParent, nWinStyle ) {};
 	virtual void RequestingChilds( SvLBoxEntry* pParent );
	virtual BOOL EditedEntry( SvLBoxEntry* pEntry, const XubString& rNewText );
};

/*
 * CmdEdit Interface.
 */
class CmdEdit : public Edit {
private:
	MyWin *pParentWnd;

public:
	CmdEdit (MyWin *pParent);
	void KeyInput (const KeyEvent& rKeyEvent);
};

// ______________________________________________________________________
// MyWin.
//
enum MyImapCmd {
	IMAPCMD_NONE = 0,
	IMAPCMD_LIST
};

class MyWin : public WorkWindow
{
	friend class ImapTree;
private:
	ToolBox                *pSendTool, *pImapTool;
	CmdEdit                *pCmdEdit;
	Edit				   *pReference;
	String					sReference;
	MultiLineEdit          *pEdit;
	ImapTree			   *pTree;
	SvLBoxEntry			   *pActParent;
	SvLBoxEntry			   *pSelectedEntry;
	
	INetImapMailer*		   pImapMailer;
	INetCoreMailer         aMailer;
	INetCoreMIMEMessage    *pMsg, *pAttMsg;
	List                   *pMsgList;
	SvFileStream           *pDataStream, *pAttStream;

	MyImapCmd              eImapCmd;

	static int SendCallback (
		INetCoreMailer *pMailer,
		int nReplyCode, String *pReplyText, void *pData);

	static int ImapCallback (
		INetCoreMailer *pMailer,
		int nReplyCode, String *pReplyText, void *pData);

public:
	MyWin (Window *pParent, WinBits nWinStyle);
	~MyWin (void);

	void Resize (void);
	void RequestingChildren( SvLBoxEntry* pParent);
	
	DECL_LINK (SendToolBarHnd, ToolBox*);
	DECL_LINK (ImapToolBarHnd, ToolBox*);
	DECL_LINK (ImapStatusLink, INetImapResult*);
	DECL_LINK (ImapResultLink, INetImapResult*);
	DECL_LINK (BoxStatusLink, INetImapResult*);
    DECL_LINK( DoubleClickHdl, SvTreeListBox* );
    DECL_LINK( SelectHdl, SvTreeListBox* );
};

MyWin::MyWin (Window *pParent, WinBits nWinStyle)
	: WorkWindow (pParent, nWinStyle),
	  pImapMailer(0)
{
	// SendTool.
	pSendTool = new ToolBox (this, WB_SVLOOK | WB_BORDER);

	pSendTool->InsertItem  (1, "SendOpen");
	pSendTool->SetHelpText (1, "Open Connection to SMTP Server");
	pSendTool->InsertSeparator ();

	pSendTool->InsertItem  (2, "From");
	pSendTool->SetHelpText (2, "Sender's Mail Address (you@this.org)");
	pSendTool->InsertItem  (3, "To");
	pSendTool->SetHelpText (3, "Recipient's Mail Address (someone@other.org)");
	pSendTool->InsertItem  (4, "Mail");
	pSendTool->SetHelpText (4, "Send your Mail to specified Recipient(s)");
	pSendTool->InsertSeparator ();

	pSendTool->InsertItem  (5, "Close");
	pSendTool->SetHelpText (5, "Quit and disconnect from SMTP Server");
	pSendTool->InsertItem  (6, "Abort");
	pSendTool->SetHelpText (6, "Abort Connection with SMTP Server");

	pSendTool->SetClickHdl (LINK(this, MyWin, SendToolBarHnd));
	pSendTool->Show ();

	// ImapToolBar.
	pImapTool = new ToolBox (this, WB_SVLOOK | WB_BORDER | WB_LINESPACING);

	pImapTool->InsertItem  (BUTTON_LOGIN, "Login");
	pImapTool->SetHelpText (BUTTON_LOGIN,
							"Open connection and login (user:pass@host)");
	pImapTool->InsertSeparator ();
	pImapTool->InsertItem  (BUTTON_LISTSUB, "List Sub",
							TIB_AUTOCHECK | TIB_RADIOCHECK );
	pImapTool->SetHelpText (BUTTON_LISTSUB, "Show Subscribed Mailboxes");
	pImapTool->InsertItem  (BUTTON_LISTALL, "List All",
							TIB_AUTOCHECK | TIB_RADIOCHECK );
	pImapTool->SetHelpText (BUTTON_LISTALL, "Show All Mailboxes");
	pImapTool->CheckItem( BUTTON_LISTSUB );
	pImapTool->InsertSeparator();

	pImapTool->InsertItem  (BUTTON_SUBSCRIBE, "Subscribe");
	pImapTool->SetHelpText (BUTTON_SUBSCRIBE, "Subscribe to selected Mailbox");
	pImapTool->InsertItem  (BUTTON_UNSUBSCRIBE, "Unsubscribe");
	pImapTool->SetHelpText (BUTTON_SUBSCRIBE,
							"Unsubscribe from selected Mailbox");
	pImapTool->InsertSeparator();

	pImapTool->InsertItem  (BUTTON_COPY, "Copy");
	pImapTool->SetHelpText (BUTTON_COPY,
				"Copy a Message to another mailbox (<from>:<to> <targetbox>");
	
	pImapTool->InsertSeparator();

	pImapTool->InsertItem  (BUTTON_CLOSE, "Deselect");
	pImapTool->SetHelpText (BUTTON_CLOSE,
							"Deselects (closes) a Mailbox");
	pImapTool->InsertItem  (BUTTON_EXPUNGE, "Expunge");
	pImapTool->SetHelpText (BUTTON_EXPUNGE,
							"Expunges a Mailbox (erase all del-flagged msgs)");
	pImapTool->InsertSeparator();

	pImapTool->InsertItem  (BUTTON_LOGOUT, "Logout");
	pImapTool->SetHelpText (BUTTON_LOGOUT,
							"Quit and disconnect from IMAP4 Server");
	pImapTool->InsertItem  (BUTTON_ABORT, "Abort");
	pImapTool->SetHelpText (BUTTON_ABORT,
							"Abort Connection with IMAP4 Server");

	pImapTool->SetClickHdl (LINK(this, MyWin, ImapToolBarHnd));
	pImapTool->Show ();

	// Edit for Reference
	pReference = new Edit(this);
//	pReference->SetText("/export/opt/mail/");
	pReference->SetText("");
	pReference->Show();

    // CmdEdit.
	pCmdEdit = new CmdEdit (this);
	pCmdEdit->SetText("imaptest:a12345@asa");
	pCmdEdit->Show ();

	// MultiLineEdit (output work window).
	pEdit = new MultiLineEdit (this, WB_HSCROLL | WB_VSCROLL | WB_BORDER);
	pEdit->SetReadOnly (TRUE);
	pEdit->Show ();

	// TreeListBox
	pTree = new ImapTree (this, this,
						  WB_CLIPCHILDREN 	| WB_VSCROLL |
						  WB_HSCROLL 		| WB_BORDER);
	pTree->SetWindowBits( WB_HASLINES | WB_HASBUTTONS );
	pTree->SetSelectionMode( SINGLE_SELECTION );
// 	edit mit Alt-Mousebuttondown
	pTree->EnableInplaceEditing( TRUE );
	pTree->SetDoubleClickHdl( LINK(this,MyWin,DoubleClickHdl));
	pTree->SetSelectHdl( LINK(this,MyWin,SelectHdl));
	pTree->SetIndent( 19 );
	Color aColor( COL_WHITE );
	Image aImgOpen( Bitmap(ResId( FOLDER_OPEN )), aColor );
	Image aImgClosed( Bitmap(ResId( FOLDER_CLOSED )), aColor );

	pTree->SetDefaultExpandedEntryBmp( aImgOpen  );
    pTree->SetDefaultCollapsedEntryBmp( aImgClosed );

	pTree->SetNodeBitmaps(Bitmap(ResId(PLUSBUTTON)),
						 Bitmap(ResId(MINUSBUTTON)));

	pTree->Show();
	
	// Mailer.
	pMsg = NULL;
	pMsgList = NULL;
	pDataStream = NULL;

	pAttMsg = NULL;
	pAttStream = NULL;

	eImapCmd = IMAPCMD_NONE;
}

MyWin::~MyWin (void)
{
	delete pAttStream;
	delete pAttMsg;
	delete pDataStream;
	delete pMsgList;
	delete pMsg;
	if(pImapMailer)
		pImapMailer->Abort();
	delete pImapMailer;
	delete pEdit;
	delete pTree;
	delete pCmdEdit;
	delete pReference;
	delete pImapTool;
	delete pSendTool;
}

void MyWin::RequestingChildren(SvLBoxEntry* pParent)
{
	if(pParent->GetUserData() &&
	   (((ImapBox*)pParent->GetUserData())->HasChildren() == UNKNOWN))
	{
		pActParent = pParent;
		ImapBox* pBox = (ImapBox*) pParent->GetUserData();
		String sBox = pBox->GetFullname();
		sBox.Cut(0,sReference.Len());
		sBox += pBox->GetSep();
		sBox += '%';
		if (pImapTool->IsItemChecked(BUTTON_LISTSUB))
		{
			if (pImapMailer->ListSubscribedBoxes(sReference, sBox,
												LINK(this,MyWin,ImapResultLink),
												(void*) IMAP_LIST))
			{
				pEdit->SetText ("Getting the subscribed List...");
				pEdit->Show ();
			}
		}
		else
		{
			if (pImapMailer->ListAllBoxes(sReference,
										  sBox, LINK(this,MyWin,ImapResultLink),
										  (void*) IMAP_LIST))
			{
				pEdit->SetText ("Getting the complete List...");
				pEdit->Show ();
			}
		}
	}
}

void MyWin::Resize (void)
{
	Size aWinSize = GetOutputSizePixel ();
	int nWinW = aWinSize.Width ();
	int nWinH = aWinSize.Height ();

	int nSendBoxH = pSendTool->GetSizePixel().Height();
	int nRecvBoxH = pImapTool->GetSizePixel().Height();

	int nTreeWidth = nWinW / 3;
	int nImapWidth = nWinW * 5 / 6;

	pSendTool->SetPosSizePixel (
		Point (0, 0),
		Size (nWinW, nSendBoxH));

	pImapTool->SetPosSizePixel (
		Point (0, nSendBoxH),
		Size  (nImapWidth, nRecvBoxH));

	pReference->SetPosSizePixel (
		Point (nImapWidth, nSendBoxH),
		Size  (nWinW - nImapWidth, nRecvBoxH));

	pCmdEdit->SetPosSizePixel (
		Point (0, nSendBoxH + nRecvBoxH),
		Size  (nWinW, 32   ) );

	pTree->SetPosSizePixel (
		Point (0,nSendBoxH + nRecvBoxH + 32),
		Size  (nTreeWidth, nWinH - (nSendBoxH + nRecvBoxH + 32)));
	
	pEdit->SetPosSizePixel (
		Point (nTreeWidth, nSendBoxH + nRecvBoxH + 32),
		Size (nWinW, nWinH - (nSendBoxH + nRecvBoxH + 32)));
}

/*
 * Send via SMTP.
 */
IMPL_LINK (MyWin, SendToolBarHnd, ToolBox*, pSendTool)
{
	USHORT nItemId  = pSendTool->GetCurItemId ();
	String aCmdLine = pCmdEdit->GetText ();
	String aStr     = pEdit->GetText ();

	switch (nItemId)
	{
		case 1: // Open.
			if (aCmdLine.Len () == 0) return 0;

			if (aMailer.SendOpenConnection (
				aCmdLine, 0, SendCallback, this))
			{
				aStr = ("Connecting to \" ");
				aStr += aCmdLine;
				aStr += "\" ...";
			}
			else
			{
				aStr += "Connect Failure.";
			}
			break;

		case 2: // From.
			if (aCmdLine.Len () == 0) return 0;

			if (pMsg)
				delete pMsg;
			pMsg = aMailer.CreateINetCoreMIMEMessage ();
			pMsg->SetFrom (aCmdLine);
			break;

		case 3: // To.
			if (aCmdLine.Len () == 0) return 0;
			if (pMsg)
			{
				String aStr = pMsg->GetTo ();
				if (aStr.Len ()) aStr += ',';
				aStr += aCmdLine;
				pMsg->SetTo (aStr);
			}
			break;

		case 4: // Mail.
			if (aCmdLine.Len () == 0) return 0;

			if (pDataStream)
				delete pDataStream;
			pDataStream = new SvFileStream;

			pDataStream->Open (aCmdLine, STREAM_READ);
			// pDataStream->Open ("d:/mhu/tmp/internet.jpg", STREAM_READ);

			if (!pDataStream->IsOpen ())
			{
				delete pDataStream;
				pDataStream = NULL;
				return 0;
			}
			else if (pMsg)
			{
				pMsg->SetDocumentStream (pDataStream);
				// pMsg->SetCC  ("asa@stardiv.de");
				// pMsg->SetBCC ("mhu@stardiv.de");
				pMsg->SetSubject ("Hello (Schne Gre) MIME World");

				pMsg->SetMIMEVersion ("1.0");
				pMsg->SetContentType ("text/plain; charset=iso-8859-1");
				// pMsg->SetContentType ("image/jpeg");

				pMsg->SetXMailer ("MHU-MailTest/1.2 (MIME/1.0)");
/**/
				if (pAttMsg)
					delete pAttMsg;
				pAttMsg = new INetCoreMIMEMessage;

				pAttMsg->SetContentType ("image/jpeg");

				if (pAttStream)
					delete pAttStream;
				pAttStream = new SvFileStream ("d:\\attach.jpg", STREAM_READ);
				pAttMsg->SetDocumentStream (pAttStream);

				pMsg->AttachChild(*pAttMsg);
/**/
				if (aMailer.SendMail (*pMsg, SendCallback, this))
				{
				}
				else
				{
/**/
					delete pAttStream;
					pAttStream = NULL;
					delete pAttMsg;
					pAttMsg = NULL;
/**/
					delete pDataStream;
					pDataStream = NULL;
					delete pMsg;
					pMsg = NULL;
				}
			}
			break;

		case 5: // Close.
			if (aMailer.SendCloseConnection (SendCallback, this))
			{
			}
			else
			{
			}
			break;

		case 6: // Abort.
			aMailer.SendAbortConnection ();
			break;

		default: // Ignored.
			break;
	}

	return 0;
}

int MyWin::SendCallback (
	INetCoreMailer *pMailer,
	int nReplyCode, String *pReplyText, void *pData)
{
	MyWin  *pWin = (MyWin *)pData;
	String aStr  = pWin->pEdit->GetText ();

	aStr += "\nSendCallback: ReplyCode: ";
	aStr += nReplyCode;
	aStr += '\n';
	if (pReplyText) aStr += *pReplyText;

	pWin->pEdit->SetText (aStr.ConvertLineEnd ());
	pWin->pEdit->Show ();

	return 1;
}

/*
 * Recv via IMAP4.
 */
IMPL_LINK (MyWin, ImapToolBarHnd, ToolBox*, pImapTool)
{
	USHORT nItemId  = pImapTool->GetCurItemId ();
	String aCmdLine = pCmdEdit->GetText ();
	String aStr     = pEdit->GetText ();

	eImapCmd = IMAPCMD_NONE;

	switch (nItemId)
	{
		case BUTTON_LOGIN: // Open & login.
			if (aCmdLine.Len () == 0) return 0;

			if(pImapMailer)
				delete pImapMailer;
			if (aCmdLine.GetTokenCount (':') == 2)
			{
				String aUser = aCmdLine.GetToken (0, ':');
				if (aUser.Len () == 0) return 0;

				String aPwd  = aCmdLine.GetToken (1, ':');
				if (aPwd.Len () == 0) return 0;

				if(aPwd.GetTokenCount ('@') != 2) return 0;

				String aHost = aPwd.GetToken (1, '@');
				aPwd = aPwd.GetToken (0,'@');
				
				pImapMailer = new INetImapMailer
							  (LINK (this,MyWin,ImapStatusLink),this, aHost);

				if (pImapMailer->Login( aUser, aPwd,
										LINK(this,MyWin,ImapResultLink),
										(void*) IMAP_LOGIN))
				{
					aStr = ("Connecting to \" ");
					aStr += aCmdLine;
					aStr += "\" ...\n";
				}
			}
			else
			{
				aStr = "Syntax: user:pass@host!";
			}
			break;

		case BUTTON_LISTSUB: // Get Listing of subscribed boxes
			if(pImapMailer)
			{
				pTree->Clear();
				pActParent = pTree->InsertEntry(pImapMailer->GetMailerName());
				pActParent->SetUserData(0);
				if(pReference->GetText().Len())
				{
					sReference = pReference->GetText();
					pActParent = pTree->InsertEntry(sReference,pActParent);
					pActParent->SetUserData(0);
				}
				else
					sReference.Erase();
				if (pImapMailer->ListSubscribedBoxes(pReference->GetText(),
													"%",
													LINK(this,MyWin,ImapResultLink),
													(void*) IMAP_LIST))
				{
					aStr += "Getting the subsribed List...";
				}
			}
			break;
		case BUTTON_LISTALL: // List.
			if(pImapMailer)
			{
				pTree->Clear();
				pActParent = pTree->InsertEntry(pImapMailer->GetMailerName());
				pActParent->SetUserData(0);
				if(pReference->GetText().Len())
				{
					sReference = pReference->GetText();
					pActParent = pTree->InsertEntry(sReference,pActParent);
					pActParent->SetUserData(0);
				}
				else
					sReference.Erase();
				if (pImapMailer->ListAllBoxes(pReference->GetText(),
											  "%",LINK(this,MyWin,ImapResultLink),
											  (void*) IMAP_LIST))
				{
					aStr += "Getting the complete List...";
				}
			}
			break;
		case BUTTON_SUBSCRIBE:
			if(pImapMailer && pTree->FirstSelected())
			{
				SvLBoxEntry* pEntry = pTree->FirstSelected();
				ImapBox* pBox = (ImapBox*) pEntry->GetUserData();
				if(pBox && pBox->IsSelectable())
				{
					if(pImapMailer->SubscribeBox(pBox->GetFullname(),
												 LINK(this,MyWin,ImapResultLink),
												 0))
					{
						aStr += "Subscribing to ";
						aStr += pBox->GetFullname();
					}
				}
			}
			break;
		case BUTTON_UNSUBSCRIBE:
			if(pImapMailer && pTree->FirstSelected())
			{
				SvLBoxEntry* pEntry = pTree->FirstSelected();
				ImapBox* pBox = (ImapBox*) pEntry->GetUserData();
				if(pBox && pBox->IsSelectable())
				{
					if(pImapMailer->UnsubscribeBox(pBox->GetFullname(),
												   LINK(this,MyWin,ImapResultLink),
												   0))
					{
						aStr += "Unsubscribing to ";
						aStr += pBox->GetFullname();
					}
				}
			}
			break;
		case BUTTON_COPY:
			if(pImapMailer && pTree->FirstSelected() && aCmdLine.Len())
			{
				if (aCmdLine.GetTokenCount (' ') == 2)
				{
					String sRange = aCmdLine.GetToken (0, ' ');

					String sFrom = sRange.GetToken (0, ':');
					if (sFrom.Len () == 0) return 0;

					String sTo  = sRange.GetToken (1, ':');

					String sBox = aCmdLine.GetToken(1, ' ');
					if (sBox.Len () == 0) return 0;

					if (pImapMailer->CopyMsg( sFrom, sTo, sBox,
										   LINK(this,MyWin,ImapResultLink),
										   (void*) IMAP_LOGIN))
					{
						aStr = ("Copy ");
						aStr += sFrom;
						if(sTo.Len())
						{
							aStr += ' ';
							aStr += sTo;
						}
						aStr += " to ";
						aStr += sBox;
					}
				}
				else
				{
					aStr = "Syntax: <from>:<to> <targetbox>";
				}
			}
			break;
		case BUTTON_EXPUNGE:
			if(pImapMailer && pTree->FirstSelected())
			{
				if(pImapMailer->Expunge(LINK(this,MyWin,ImapResultLink),0))
				{
					aStr += "Expunge current mailbox";
				}
			}
			break;
		case BUTTON_CLOSE:
			if(pImapMailer && pTree->FirstSelected())
			{
				if(pImapMailer->Close(LINK(this,MyWin,ImapResultLink),0))
				{
					aStr += "Close current mailbox";
				}
				pTree->Select(pTree->FirstSelected(), FALSE);
			}
			break;
		case BUTTON_LOGOUT:
			if(pImapMailer)
			{
				if(pImapMailer->Logout(LINK(this,MyWin,ImapResultLink),
									   (void*)IMAP_LOGOUT))
				{
					aStr += "Logging out...";
				}
			}
			break;
		case BUTTON_ABORT:
			if(pImapMailer)
			{
				if(pImapMailer->Abort())
				{
					aStr += "Aborted...";
					delete pImapMailer;
					pImapMailer = 0;
				}
			}
			break;
		default: // Ignored.
			break;
	} // switch (nItemId)

	pEdit->SetText (aStr.ConvertLineEnd ());
	pEdit->Show();

	return 0;
}

IMPL_LINK(MyWin, ImapResultLink, INetImapResult*, pResult)
{
	String(sResult);

	switch(pResult->GetType())
	{
		case INET_IMAP_RESULT_SUCCESS:
			sResult = "Success: ";
			break;
		case INET_IMAP_RESULT_ERROR:
			sResult = "Error: ";
			break;
		case INET_IMAP_RESULT_PREAUTH:
			sResult = "Preauthenticated: ";
			break;
		case INET_IMAP_RESULT_LISTBOX:
			sResult = "Success: ";
			break;
		default:
			sResult = "Unknown result-Type: ";
			break;
	}
	sResult += pResult->GetText();
	sResult += '\n';
	switch ((ULONG)pResult->GetUserData())
	{
		case IMAP_LOGIN:
		{
			pTree->Clear();
			pActParent = pTree->InsertEntry(pImapMailer->GetMailerName());
			pActParent->SetUserData(0);
			if(pReference->GetText().Len())
			{
				sReference = pReference->GetText();
				pActParent = pTree->InsertEntry(sReference,pActParent);
				pActParent->SetUserData(0);
			}
			else
				sReference.Erase();
			pTree->Show();
			if (pImapTool->IsItemChecked(BUTTON_LISTSUB))
			{
				if (pImapMailer->ListSubscribedBoxes(pReference->GetText(),
										   "%",LINK(this,MyWin,ImapResultLink),
											(void*) IMAP_LIST))
				{
					sResult += "Getting the subscribed List...";
				}
			} 
			else
			{
				if (pImapMailer->ListAllBoxes(pReference->GetText(),
											"%",LINK(this,MyWin,ImapResultLink),
											  (void*) IMAP_LIST))
				{
					sResult += "Getting the complete List...";
				}
			} 
			break;
		}
		case IMAP_LIST:
		{
			INetImapListBoxResult* pLRes = (INetImapListBoxResult*) pResult;
			const INetImapBox* pBox = pLRes->GetFirstBox();
			if(pActParent->GetUserData())
				((ImapBox*)pActParent->GetUserData())->SetChildrenFlag(FALSE);
			while(pBox)
			{
				ImapBox* pMyBox = new ImapBox(pBox);
				SvLBoxEntry* pNewEntry;
				String sPrefix(sReference);
				if(pActParent->GetUserData())
				{
					sPrefix = ((ImapBox*)pActParent->GetUserData())->GetFullname();
					sPrefix += ((ImapBox*)pActParent->GetUserData())->GetSep();
				}
				if(pMyBox->GetFullname() == sPrefix)
					delete pMyBox;
				else
				{
					if(pActParent->GetUserData())
						((ImapBox*)pActParent->GetUserData())->SetChildrenFlag(TRUE);
					pNewEntry = pTree->InsertEntry(pMyBox->GetBasename(),
												   pActParent);
					pNewEntry->SetUserData(pMyBox);
					pNewEntry->EnableChildsOnDemand();
				}
				pBox = pLRes->GetNextBox();
			}
			pTree->Expand(pActParent);
			break;
		}
		case IMAP_LOGOUT:
			delete pImapMailer;
			pImapMailer = 0;
			break;
		default:
			break;
	}
	if(sResult.Len())
	{
		pEdit->SetText (sResult.ConvertLineEnd ());
		pEdit->Show ();
	}
	return 0;
}

IMPL_LINK(MyWin, ImapStatusLink, INetImapResult*, pResult)
{
	String(sResult);
	
	switch(pResult->GetType())
	{
		case INET_IMAP_CONNECTION_NAMERESOLVE_SUCCESS:
			sResult = "Name resolved...\n";
			break;
			
		case INET_IMAP_CONNECTION_CONNECT_SUCCESS:
			sResult = "Connected...\n";
			break;

		case INET_IMAP_STATUS_ALERT:
			sResult = "[ALERT]: ";
			sResult += pResult->GetText();
			break;

		case INET_IMAP_GENERAL_ERROR:
			sResult = "General Error...\n";
			break;
		case INET_IMAP_NETWORK_ERROR:
			sResult = "Network Error...\n";
			break;
		case INET_IMAP_NAMERESOLVE_ERROR:
			sResult = "Name Resolve Error...\n";
			break;
		case INET_IMAP_CONNECT_ERROR:
			sResult = "Connect Error...\n";
			break;
	}
	if(pResult->GetType() < 0)
	{
		delete pImapMailer;
		pImapMailer = 0;
	}
	if(sResult.Len())
	{
		pEdit->SetText (sResult.ConvertLineEnd ());
		pEdit->Show ();
	}
	return 0;
}

IMPL_LINK( MyWin, DoubleClickHdl, SvTreeListBox*, pTree )
{
	SvLBoxEntry* pEntry = pTree->FirstSelected();
	ImapBox* pBox = (ImapBox*) pEntry->GetUserData();
	if(pBox && pBox->IsSelectable())
	{
		pSelectedEntry = pTree->FirstSelected();
/*		INetImapMessage* pTestMsg = pImapMailer->CreateINetImapMessage();
		INetImapFlagList* pFlags = pImapMailer->CreateINetImapFlagList();
		pFlags->SetAnswered();
		pFlags->SetFlagged();
		pFlags->SetSeen();
		pFlags->SetDraft();
		INetCoreNewsMessage* pTestCore = new INetCoreNewsMessage;
		pTestMsg->SetFlags(pFlags);
		pTestMsg->SetInternalDate(DateTime());
		pTestMsg->SetMessage(pTestCore);
		pTestCore->SetFrom("asa@stardiv.de");
		pTestCore->SetTo("all@asa.stardiv.de");
		SvMemoryStream*	pMailStream = new SvMemoryStream;
		*pMailStream << "Hallo!\n\nThis is a imap-test.\n"
					 << "That's it!";
		pMailStream->Seek(STREAM_SEEK_TO_BEGIN);
		pTestCore->SetDocumentStream(pMailStream);
		pImapMailer->AppendMsg("testbox",*pTestMsg,
							   LINK(this,MyWin,ImapStatusLink),
							   this);
		delete pTestMsg;*/
		INetImapFlagList* pFlags = pImapMailer->CreateINetImapFlagList();
		pFlags->SetFlagged();
		pImapMailer->ReplaceFlags("1","*",*pFlags,
								  LINK(this,MyWin,ImapStatusLink),
								  this,FALSE);
		delete pFlags;
	}
	return 0;
}

IMPL_LINK( MyWin, SelectHdl, SvTreeListBox*, pTree )
{
	SvLBoxEntry* pEntry = pTree->FirstSelected();
	ImapBox* pBox = (ImapBox*) pEntry->GetUserData();
	if(pBox && pBox->IsSelectable())
	{
		pSelectedEntry = pTree->FirstSelected();
		pImapMailer->SelectBox(pBox->GetFullname(),
							   LINK(this,MyWin,BoxStatusLink),
							   this,
							   LINK(this,MyWin,ImapStatusLink),
							   this);
	}
	return 0;
}

IMPL_LINK(MyWin, BoxStatusLink, INetImapResult*, pResult)
{
	if(pSelectedEntry && pSelectedEntry->GetUserData())
	{
		ImapBox* pBox = (ImapBox*) pSelectedEntry->GetUserData();
		switch(pResult->GetType())
		{
			case INET_IMAP_BOXSTATUS_EXISTS:
			{
				ULONG nExists = pResult->GetNumber();
				pBox->SetExists(nExists);
				break;
			}
			case INET_IMAP_BOXSTATUS_RECENT:
			{
				ULONG nNew = pResult->GetNumber();
				pBox->SetNew(nNew);
				break;
			}
			case INET_IMAP_BOXSTATUS_FLAGS:
				break;
			case INET_IMAP_BOXSTATUS_UNSEEN:
				break;
			case INET_IMAP_BOXSTATUS_PERMANENTFLAGS:
				break;
			default:
				break;
		}
		pTree->SetEntryText(pSelectedEntry,pBox->GetFullData());
	}
	return 0;
}

/*
 * Input window.
 */
CmdEdit::CmdEdit (MyWin *pParent) : Edit (pParent)
{
	pParentWnd = pParent;
	SetReadOnly (FALSE);
}

void CmdEdit::KeyInput (const KeyEvent& rKeyEvent)
{
	if (rKeyEvent.GetKeyCode () == KEY_RETURN)
	{
		// Unused.
	}

	Edit::KeyInput (rKeyEvent);
}

/*
 * MyApp.
 */
class MyApp : public Application {
private:
	MyWin *pMainWin;

public:
	virtual void Main (
#ifndef VCL
		int argc, char *argv[]
#endif
	);
};

MyApp aMyApp;

void MyApp::Main (
#ifndef VCL
	int argc, char *argv[]
#endif
)
{
	EnableSVLook ();
	Help::EnableBalloonHelp ();

	pMainWin = new MyWin (NULL, WB_APP | WB_STDWORK);
	String sRev("$Revision: 1.1.1.1 $");
	sRev.Cut(0,10);
	sRev.Cut(sRev.Len()-1);
	sRev.EraseAllChars(' ');
	if(!sRev)
		sRev = "???";
	pMainWin->SetText (String("ImapTest v.")+sRev);
	pMainWin->Show ();

	Execute ();

	delete pMainWin;
}

// ______________________________________________________________________
// ImapTree

void ImapTree::RequestingChilds( SvLBoxEntry* pParent )
{
	pOwner->RequestingChildren(pParent);
}

BOOL ImapTree::EditedEntry( SvLBoxEntry* pEntry, const XubString& rNewText )
{
	if(!rNewText)
		return FALSE;
	else
		return TRUE;
}

