/*************************************************************************
 *
 *  $RCSfile: plugtest.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:31:17 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#if defined(WNT) || defined (WIN)
#include <svwin.h>
#endif

#include <stdio.h>

#include "inet.hxx"
#include "plugpool.hxx"
#include "urlobj.hxx"

#include "plugtest.hxx"
#include "plugtest.hrc"

#include "plugwin.hxx"

MyApp aMyApp;

//=============================================================================
//	Application
//=============================================================================
void MyApp::Main( int, char* [] )
{
	EnableSVLook();
	Help::EnableBalloonHelp();
	SetAppFont( System::GetStandardFont(STDFONT_APP) );

	MyWin aMainWin( NULL, WB_APP | WB_STDWORK );
	aMainWin.SetText( "SVPlug Debug Studio" );
	aMainWin.Show();

	Execute();
}

//=============================================================================
//	URLEdit
//=============================================================================
URLEdit::URLEdit( MyWin* pWin ) :
	Edit( pWin )
{
	pMaster = pWin;
	String aDefault( "ftp://ftp.informatik.uni-hamburg.de/pub/StarDivision/so3/trybuy/win95/tb2105_1.zip" );
//	String aDefault( "http://rabbit.stardiv.de/plugins/plug/vrml/mtv/mtv.wrl" );
	SetText( aDefault );
	SetReadOnly( FALSE );
}

void URLEdit::KeyInput( const KeyEvent& rKEvent )
{
	if( rKEvent.GetKeyCode() == KEY_RETURN )
	{
		String aURL = GetText();
		pMaster->LoadFromServer( aURL );
	}
	Edit::KeyInput( rKEvent );
}

//=============================================================================
//	SpyWin
//=============================================================================
SpyWin::SpyWin(Window* pParent, WinBits nWinBits, PlugWin* pPlug) :
	DockingWindow(pParent, nWinBits)
{
	SetBackgroundBrush( Brush(Color(COL_WINDOWWORKSPACE)) );

	pEdit = new MultiLineEdit( this, WB_HSCROLL | WB_VSCROLL | WB_BORDER );
	pEdit->SetReadOnly( TRUE );

	pList = new ListBox( this, WB_HSCROLL | WB_VSCROLL | WB_BORDER );
	pList->Show();

	pPlugWin = pPlug;
}

SpyWin::~SpyWin()
{
	if( pEdit )
		delete pEdit;
	pEdit = NULL;

	if( pList )
		delete pList;
	pList = NULL;
}

void SpyWin::Resize()
{
	Size aSz = GetSizePixel();
	pEdit->SetPosSizePixel( Point(0,0), aSz );
	pList->SetPosSizePixel( Point(0,0), aSz );
}

void SpyWin::ShowPlugTrace()
{
	if( pList->IsVisible() )
		return;
	pEdit->Hide();
	pList->Show();
}

void SpyWin::ShowPlugList()
{
	if( pEdit->IsVisible() )
		return;
	pList->Hide();
	pEdit->Show();

	String aInfo;
	String aLastDll;

	const PlugList* pList = pPlugWin->GetManager()->GetPlugList();
	for( USHORT n = 0; n < pList->Count(); ++n )
	{
		if( aLastDll != (*pList)[n]->GetDllName() )
		{
			aInfo += "==============================================\n";
			aInfo += (*pList)[n]->GetDllName();
			aInfo += "\n";
		}
		aInfo += "\t==>";
		aInfo += (*pList)[n]->GetFileOpenName();
		aInfo += "\n\t   ";
		aInfo += (*pList)[n]->GetMIMEType();
		aInfo += "\n\t   ";
		aInfo += (*pList)[n]->GetFileExtents();
		aInfo += "\n";

		aLastDll = (*pList)[n]->GetDllName();
	}
	pEdit->SetText( aInfo.ConvertLineEnd() );
}

void SpyWin::StartNewPlug()
{
	PlugInWindow* pWin = ((FakeWindow*)pPlugWin->GetPlug())->GetPlugWin();
	pWin->SetSpyHdl( LINK(this, SpyWin, SpyHdl) );
	pList->Clear();
}

IMPL_LINK( SpyWin, SpyHdl, char*, pStr )
{
	USHORT nCnt = pList->GetEntryCount();
	if( nCnt > MAX_ENTRYS )
		pList->RemoveEntry(0);

	String aMsg( pStr );
	pList->InsertEntry( aMsg );
	pList->SelectEntryPos( pList->GetEntryCount()-1 );
	return 0;
}

//=============================================================================
//	PlugWin
//=============================================================================
PlugWin::PlugWin(Window* pParent, WinBits nWinBits, Window* pHdlPar) :
	DockingWindow(pParent, nWinBits)
{
	SetBackgroundBrush( Brush(Color(COL_WINDOWWORKSPACE)) );

	pPlug = NULL;
	eMode = PLUGIN_FULL;

	bNoDown = FALSE;
	bAsReadOnly = FALSE;
	bAsync = TRUE;

	pINetWrapper = new INetWrapper;
	if( !pINetWrapper )
		aMyApp.Quit();

	xSession = pINetWrapper->NewINetSession();
	xSession->AddRef();

	String aAppPath = aMyApp.GetAppFileName();
	int nPos = aAppPath.Len() - 1;
	while( nPos > 0 && aAppPath[(USHORT)nPos] != '\\' )
		--nPos;
	aAppPath.Cut( nPos );
	aAppPath += "\\";

	pManager = pINetWrapper->NewPlugInManager(aAppPath);
	if( !pManager )
		aMyApp.Quit();

	pManager->SetSession( &xSession );
	pManager->SetReadStatusHdl( LINK(pHdlPar, MyWin, StatusHdl) );
	pManager->SetInfoStatusHdl( LINK(pHdlPar, MyWin, InfoHdl) );
}

PlugWin::~PlugWin()
{
	if( pPlug )
		pManager->Destroy( pPlug );
	pPlug = NULL;

	if( pManager )
		delete pManager;
	pManager = NULL;

	xSession->DeInitialize();
	xSession.Clear();

	if( pINetWrapper )
		delete pINetWrapper;
	pINetWrapper = NULL;
}

void PlugWin::Resize()
{
	if( pPlug )
		pPlug->SetPosSizePixel( Point(0,0), GetSizePixel() );
}

void PlugWin::StopRunningPlug()
{
	if( pPlug )
	{
		pManager->Destroy( pPlug );
		pPlug = NULL;
	}
}

void PlugWin::LoadPlugIn( INetURLObject& rURL )
{
	if( rURL.HasError() )
	{
		InfoBox aBox( NULL, "Sorry, fehlerhafter URL!" );
		aBox.Execute();
		return;
	}

	BOOL bSucc = FALSE;

	const char* pCmd[2] = { "loop", "autostart" };
	const char* pPar[2] = { "true", "true" };

	if( pPlug )
		pManager->Destroy( pPlug );

	pPlug = pManager->NewPlugInWindow(this);


	if( bNoDown )
		pManager->DontDownload( pPlug );

	if( bAsReadOnly )
		pManager->SetRequestReadOnly( pPlug );
	else
		pManager->ClearRequestReadOnly( pPlug );

	if( !bAsync )
		pManager->SetSyncDownload(pPlug);

	pSpyWin->StartNewPlug();

	if( pPlug )
	{
		Size aSz = GetOutputSizePixel();
		pPlug->SetPosSizePixel( Point(0,0), aSz );
		pPlug->Show();
		pManager->SetURLLoadHdl( pPlug, LINK(this, MyWin, GetURLHdl) );

		bSucc = pManager->RegisterPlugIn( pPlug, rURL, eMode, 2, pCmd, pPar );
	}

	if( !pPlug || !bSucc )
	{
		InfoBox aBox( NULL, "Konnte PlugIn nicht starten!" );
		aBox.Execute();
		if( pPlug )
		{
			pManager->Destroy( pPlug );
			pPlug = NULL;
		}
	}
}

//=============================================================================
//	MyWin
//=============================================================================
MyWin::MyWin( Window* pParent, WinBits nWinStyle ) :
			WorkWindow( pParent, nWinStyle )
{
	// ToolBox with URLEditline
	pURL	= new URLEdit( this );
	pBox	= new ToolBox( this, WB_SVLOOK | WB_BORDER | WB_LINESPACING );

	pBox->SetLineCount( 2 );

	pBox->InsertItem	( BUTTON_LOAD, "Load" );
	pBox->SetHelpText	( BUTTON_LOAD, "Load a PlugIn" );

	pBox->InsertItem	( BUTTON_STOP, "Destroy" );
	pBox->SetHelpText	( BUTTON_STOP, "Destroy a PlugIn" );

	pBox->InsertSeparator();

	pBox->InsertItem	( BUTTON_NODOWN, "NoDownload", TIB_AUTOCHECK );
	pBox->SetHelpText	( BUTTON_NODOWN, "PlugInManager do not download file" );
	pBox->InsertItem	( BUTTON_READONLY, "ReadOnly", TIB_AUTOCHECK );
	pBox->SetHelpText	( BUTTON_READONLY, "request temp. files as read only" );
	pBox->InsertItem	( BUTTON_ASYNC, "Async", TIB_AUTOCHECK );
	pBox->SetHelpText	( BUTTON_ASYNC, "ansynchron download" );
	pBox->CheckItem		( BUTTON_ASYNC );

	pBox->InsertSeparator();

	pBox->InsertItem	( BUTTON_FULL, "Full", TIB_AUTOCHECK | TIB_RADIOCHECK );
	pBox->SetHelpText	( BUTTON_FULL, "start PlugIn in fullscreen mode" );

	pBox->InsertItem	( BUTTON_EMBED, "Embed", TIB_AUTOCHECK | TIB_RADIOCHECK );
	pBox->SetHelpText	( BUTTON_EMBED, "start PlugIn in embeded mode" );

	pBox->CheckItem( BUTTON_FULL );
	pBox->InsertSeparator();

	pBox->InsertItem	( BUTTON_TRACE, "Trace", TIB_AUTOCHECK | TIB_RADIOCHECK );
	pBox->SetHelpText	( BUTTON_TRACE, "trace debug info for running plugin" );

	pBox->InsertItem	( BUTTON_LIST, "List", TIB_AUTOCHECK | TIB_RADIOCHECK );
	pBox->SetHelpText	( BUTTON_LIST, "list all known PlugIns on system" );

	pBox->CheckItem( BUTTON_TRACE );

	pBox->InsertSeparator();
	pBox->InsertBreak();

	Size aSz( pBox->CalcWindowSizePixel() );

	pURL->SetSizePixel( Size(100, pBox->GetItemRect(1).GetHeight()) );
	pBox->InsertWindow( BUTTON_URL, pURL );
	pBox->SetSelectHdl( LINK(this, MyWin, SelectHdl) );

	// StatusBar
	pStatus = new StatusBar( this, WB_BORDER | WB_LEFT | WB_SVLOOK );
	pStatus->InsertItem( 1, 150, SIB_AUTOSIZE | SIB_IN );
	pStatus->InsertItem( 2, 200, SIB_AUTOSIZE | SIB_IN );

	// SplitWindows
	pMainWin	= new SplitWindow(this, WB_SVLOOK);
	pPlugWin	= new PlugWin(pMainWin,0,this);
	pSpyWin 	= new SpyWin(pMainWin,0,pPlugWin);

	pPlugWin->SetSpyWin( pSpyWin );
	pMainWin->SetAlign( BOXALIGN_LEFT );

	pMainWin->InsertItem(0, pPlugWin, 60, SPLITWINDOW_APPEND, 0,
						 SWIB_PERCENTSIZE);

	pMainWin->InsertItem(1, pSpyWin, 40, SPLITWINDOW_APPEND, 0,
						 SWIB_PERCENTSIZE);

	// Show on all Windows
	pBox->Show();
	pStatus->Show();
	pPlugWin->Show();
	pSpyWin->Show();
	pMainWin->Show();
}

//=============================================================================
MyWin::~MyWin()
{
	delete pURL;
	delete pBox;
	delete pStatus;

	delete pSpyWin;
	delete pPlugWin;
	delete pMainWin;
}

// === void MyWin::Resize() ===================================================
void MyWin::Resize()
{
	Size aSz	= GetOutputSizePixel();
	Size aBoxSz = pBox->CalcWindowSizePixel();
	Size aStaSz = pStatus->GetSizePixel();

	pURL->SetSizePixel( Size((aSz.Width()-pURL->GetPosPixel().X()- 7),
						pURL->GetSizePixel().Height()) );

	pBox->SetPosSizePixel( Point(0,0), Size(aSz.Width(), aBoxSz.Height()) );

	pStatus->SetPosSizePixel( Point(0, aSz.Height() - aStaSz.Height()),
						Size( aSz.Width(), aStaSz.Height()) );

	pMainWin->SetPosSizePixel( Point(0, aBoxSz.Height()),
		Size(aSz.Width(), aSz.Height() - aBoxSz.Height() - aStaSz.Height()) );
}

// === IMPL_LINK( MyWin, SelectHdl, ToolBox*, pBox ) ==========================
IMPL_LINK( MyWin, SelectHdl, ToolBox*, pBox )
{
	USHORT nItemId = pBox->GetCurItemId();

	switch ( nItemId )
	{
		case BUTTON_LOAD: {
			FileDialog aDiag( this, WB_OPEN );
			const PlugList* pList = pPlugWin->GetManager()->GetPlugList();
			for( USHORT n = 0; n < pList->Count(); ++n )
			{
				String aExt = (*pList)[n]->GetFileExtents();
				aExt.Insert( "*.", 0 );
				aDiag.AddFilter( (*pList)[n]->GetFileOpenName(), aExt );

				if( (*pList)[n]->GetFileExtents() == "*" )
					aDiag.SetCurFilter( (*pList)[n]->GetFileOpenName() );
			}

			if( aDiag.Execute() )
			{
				String			aFile = aDiag.GetPath();
				INetURLObject	aURL;

				aURL.SetSmartProtocol( INET_PROT_FILE );
				aURL.SetURL( aFile, FALSE );
				pPlugWin->LoadPlugIn( aURL );
			}
			break;
		}

		case BUTTON_STOP: {
			pPlugWin->StopRunningPlug();
			break;
		}

		case BUTTON_ASYNC: {
			if( pBox->IsItemChecked(BUTTON_ASYNC) )
				pPlugWin->SetAsync();
			else
				pPlugWin->ClearAsync();
			break;
		}

		case BUTTON_NODOWN: {
			if( pBox->IsItemChecked(BUTTON_NODOWN) )
				pPlugWin->SetNoDown();
			else
				pPlugWin->ClearNoDown();
			break;
		}

		case BUTTON_READONLY: {
			if( pBox->IsItemChecked(BUTTON_READONLY) )
				pPlugWin->ClearReadOnly();
			else
				pPlugWin->SetReadOnly();
			break;
		}

		case BUTTON_FULL: {
			pPlugWin->SetPlugInMode( PLUGIN_FULL );
			break;
		}

		case BUTTON_EMBED: {
			pPlugWin->SetPlugInMode( PLUGIN_EMBEDED );
			break;
		}

		case BUTTON_TRACE: {
			pSpyWin->ShowPlugTrace();
			break;
		}

		case BUTTON_LIST: {
			pSpyWin->ShowPlugList();
			break;
		}

	}
	return 0;
}

//=============================================================================
void MyWin::KeyInput( const KeyEvent& rKEvt )
{
	WorkWindow::KeyInput( rKEvt );
}

//=============================================================================
void MyWin::LoadFromServer( String& rURLString )
{
	INetURLObject aURL;
	aURL.SetSmartURL( rURLString );
	if( aURL.GetMainURL() != rURLString )
		pURL->SetText( aURL.GetMainURL() );

	pPlugWin->LoadPlugIn( aURL );
	pStatus->SetItemText( 1, String() );
}

//=============================================================================
IMPL_LINK( MyWin, StatusHdl, PlugInLoadStatus*, pIn )
{
	const INetHint* pHint = (const INetHint*)pIn->pHint;

	switch( pHint->GetId() )
	{
		case INET_NAME_RESOLVE_START: {
			pStatus->SetItemText( 1, "Looking for Server" );
			break;
		}

		case INET_CONNECT_START: {
			pStatus->SetItemText( 1, "Connect to Server" );
			break;
		}

		case INET_READ_STATUS: {
			char aBuf[256];
			sprintf( aBuf, "Bytes: %ld", ((INetStatusHint*)pHint)->GetData() );
			pStatus->SetItemText( 1, aBuf );
			break;
		}
	}
	return 0;
}

//=============================================================================
IMPL_LINK( MyWin, InfoHdl, String*, pMsg )
{
	pStatus->SetItemText( 2, *pMsg );
	return 0;
}

//=============================================================================
IMPL_LINK( MyWin, GetURLHdl, String*, pStr )
{
	pURL->SetText( *pStr );
	return 0;
}
