/*************************************************************************
 *
 *  $RCSfile: ConnectionClientSide.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:51 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.comp.rmclient;


import java.io.IOException;

import java.lang.reflect.InvocationTargetException;

import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;

import com.sun.star.bridge.XConnectionClientSide;
import com.sun.star.bridge.XLogin;

import com.sun.star.connection.XConnection;
import com.sun.star.connection.XConnector;

import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.lang.XSingleServiceFactory;

import com.sun.star.lib.uno.environments.java.Proxy;

import com.sun.star.registry.XRegistryKey;

import com.sun.star.uno.IBridge;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XInterface;


public class ConnectionClientSide implements XConnectionClientSide {
	private static boolean DEBUG = true;

    protected final static String serviceName = "com.sun.star.bridge.Connection";

	public static int __instances;

	protected              IBridge _bridge;
	protected               Object _object;
	protected XMultiServiceFactory _xMultiServiceFactory;

	protected ConnectionClientSide(XMultiServiceFactory xMultiServiceFactory, Object arguments[]) {
		++ __instances;
		
		_xMultiServiceFactory = xMultiServiceFactory;
	}

    public boolean connect(String aURL, String user, String passwd) throws com.sun.star.lang.SecurityException, com.sun.star.uno.RuntimeException {
		System.err.println(getClass().getName() + ".connect - connecting to:" + aURL + " user:" + user);

		boolean result = false;

		try {
			String hostName = aURL.substring(0, aURL.indexOf(':'));
			String port     = aURL.substring(aURL.indexOf(':') + 1);

			XConnector  xConnector  = (XConnector)_xMultiServiceFactory.createInstance("com.sun.star.connection.Connector");
			XConnection xConnection = xConnector.connect(hostName + ":" + port);
			
			_bridge = UnoRuntime.getBridgeByName("java", null, "remote", null, new Object[] {"iiop", xConnection, null});
			
			String rootOid = "classic_uno";


			// bootstrapping wird geaendert: erster call ist immer ein queryInterface(XInterface);

//  			XInterface rootObject = (XInterface)_bridge.mapInterfaceFrom(rootOid, XInterface.class);
			XInterface rootObject = (XInterface)Proxy.create((com.sun.star.lib.uno.environments.java.IRequester)com.sun.star.uno.WeakTable.__getObject(_bridge), rootOid, XInterface.class, true);

			XLogin xLogin = (XLogin)UnoRuntime.queryInterface(XLogin.class, rootObject);    //bridge.mapInterfaceFrom(rootOid, XLogin.class);
			rootObject = null;

			_object = xLogin.login(aURL, user, passwd);
			xLogin = null;

			result = true;
		}
		catch(IOException ioException) {
			ioException.printStackTrace();
			throw new com.sun.star.uno.RuntimeException(ioException.toString());
		}
		catch(Exception exception) {
			if(exception instanceof InvocationTargetException) {
				InvocationTargetException invocationTargetException = (InvocationTargetException)exception;
				exception = (Exception)invocationTargetException.getTargetException();
			}

			if(DEBUG) {
				System.err.println("##### " + getClass().getName() + " - exception:" + exception);
				exception.printStackTrace();
			}

			throw new com.sun.star.uno.RuntimeException(exception.toString());
		}

		return result;
	}

    public void close() throws com.sun.star.uno.RuntimeException {
		try {
			_bridge.dispose();
		}
		catch(Exception exception) {
			throw new com.sun.star.uno.RuntimeException(exception.toString());
		}
	}

    public Object getRemoteObject() throws com.sun.star.uno.RuntimeException {
		return _object;
	}

	public void finalize() {
		_bridge = null;
		_object = null;
		-- __instances;
	}


	// the factory
	static class Factory implements XSingleServiceFactory, XServiceInfo {
		protected XMultiServiceFactory _xMultiServiceFactory;
		protected         XRegistryKey _xRegistryKey;

		Factory(XMultiServiceFactory xMultiServiceFactory, XRegistryKey xRegistryKey) {
			_xMultiServiceFactory = xMultiServiceFactory;
			        _xRegistryKey = xRegistryKey;
		}

		// XSingleServiceFactory
		public Object createInstance() 
			throws com.sun.star.uno.Exception, 
				   com.sun.star.uno.RuntimeException
		{
			return createInstanceWithArguments(null);
		}

		public Object createInstanceWithArguments(Object[] arguments) 
			throws com.sun.star.uno.Exception, 
				   com.sun.star.uno.RuntimeException 
		{
			return new ConnectionClientSide(_xMultiServiceFactory, arguments);
		}

		// XServiceInfo
		public String getImplementationName() throws com.sun.star.uno.RuntimeException {
			return ConnectionClientSide.class.getName();
		}

		public boolean supportsService( /*IN*/String ServiceName ) throws com.sun.star.uno.RuntimeException {
			String services[] = getSupportedServiceNames();

			boolean found = false;

			for(int i = 0; i < services.length && !found; ++i)
				found = services[i].equals(serviceName);

			return found;
		}

		public String[] getSupportedServiceNames() throws com.sun.star.uno.RuntimeException {
			return new String[]{ConnectionClientSide.serviceName};
		}
	}

	public static XSingleServiceFactory getServiceFactory(String implName, 
														  XMultiServiceFactory multiFactory, 
														  XRegistryKey regKey)
	{
		XSingleServiceFactory xSingleServiceFactory = null;

	    if (implName.equals(ConnectionClientSide.class.getName()) )
	        xSingleServiceFactory =  new Factory(multiFactory, regKey);
	    
	    return xSingleServiceFactory;
	}
	
	public static boolean writeRegistryServiceInfo(XRegistryKey regKey) {
	    boolean result = false;
	    
  	    try {
	        XRegistryKey newKey = regKey.createKey("/" + Factory.class.getName() + "/UNO/SERVICES");
	       
			newKey.createKey(serviceName);
	        
	        result = true;
  	    }
  	    catch (Exception ex) {
  	        System.err.println(">>>Connection_Impl.writeRegistryServiceInfo " + ex);
  	    }
	    
	    return result;
    }
}

