/*************************************************************************
 *
 *  $RCSfile: ConnectionBroker.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:51 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.comp.rmserver;


import java.io.IOException;

import java.lang.reflect.InvocationTargetException;

import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;

import com.sun.star.bridge.XConnectionBroker;
import com.sun.star.bridge.XConnectionServerSide;
import com.sun.star.bridge.XConnectCallback;
import com.sun.star.bridge.XInstanceProvider;
import com.sun.star.bridge.XLogin;

import java.util.Vector;

import com.sun.star.connection.XAcceptor;
import com.sun.star.connection.XConnection;

import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.lang.XSingleServiceFactory;

import com.sun.star.registry.XRegistryKey;

import com.sun.star.uno.IBridge;
import com.sun.star.uno.UnoRuntime;


public class ConnectionBroker implements XConnectionBroker {
	public static final boolean DEBUG = false;

    protected final static String __serviceName = "com.sun.star.bridge.ConnectionBroker";

	class MyXInstanceProvider implements XInstanceProvider {
		protected XConnection _xConnection;

		MyXInstanceProvider(XConnection xConnection) {
			_xConnection = xConnection;
		}

		public Object getInstance(String oId) {
			if(DEBUG) System.err.println("###### MyXInstanceProvider.find:" + oId);

			return new Login(_xConnection, _connectCallback);
		}
	}

	protected               Thread _dispatchThread;
	protected              boolean _accept;
  	protected     XConnectCallback _connectCallback;
	protected            XAcceptor _xAcceptor;
	protected XMultiServiceFactory _xMultiServiceFactory;
	protected               String _description;

	protected ConnectionBroker(XMultiServiceFactory xMultiServiceFactory, Object arguments[]) {
		_xMultiServiceFactory = xMultiServiceFactory;
		
	}

	protected void dispatch(XConnection xConnection) throws IOException {
		System.err.println("ConnectionBroker.dispatch - connected from:" + xConnection.getDescription());
		
		MyXInstanceProvider myXInstanceProvider = new MyXInstanceProvider(xConnection);

		try {
			UnoRuntime.getBridgeByName("java", null, "remote", myXInstanceProvider, new Object[]{"iiop", xConnection});
		}
		catch(Exception exception) {
			throw new com.sun.star.uno.RuntimeException();
		}
	}

    public boolean acceptConnections(String aURL, XConnectCallback connectCallback) throws com.sun.star.uno.RuntimeException {
		boolean result = false;

		if(_dispatchThread != null)
			throw new com.sun.star.uno.RuntimeException();

		_connectCallback = connectCallback;

		_accept = true;
		_description = aURL;

		try {
		  _xAcceptor = (XAcceptor)_xMultiServiceFactory.createInstance("com.sun.star.connection.Acceptor");
		}
		catch(com.sun.star.uno.Exception exception) {
		  throw new com.sun.star.uno.RuntimeException(getClass().getName() + " - exception occurred:" + exception);
		}
		
		_dispatchThread = new Thread(new Runnable() {
		    public void run() {
				
				while(_accept) {
					try {
						System.err.println(getClass().getName() + ".acceptConnections - waiting for connection");
						XConnection xConnection = _xAcceptor.accept(_description);
						
						dispatch(xConnection);
					}
					catch(Exception exception) {
						System.err.println(getClass().getName() + " - exception occured:" + exception);
					}
				}
			}
				
		}, "ConnectionBroker - dispatcher");
		
		_dispatchThread.start();

		result = true;
		
		return result;
	}

    public void stopAccepting() throws com.sun.star.uno.RuntimeException {
		try {
			_accept = false;

			_xAcceptor.stopAccepting();

			_dispatchThread.interrupt();
			_dispatchThread.join();
		}
		catch(InterruptedException interruptedException) {
			throw new com.sun.star.uno.RuntimeException();
		}
	}


	// the factory
	static class Factory implements XSingleServiceFactory, XServiceInfo {
		protected XMultiServiceFactory _xMultiServiceFactory;
		protected         XRegistryKey _xRegistryKey;

		Factory(XMultiServiceFactory xMultiServiceFactory, XRegistryKey xRegistryKey) {
			_xMultiServiceFactory = xMultiServiceFactory;
			        _xRegistryKey = xRegistryKey;
		}

		// XMultiServiceFactory
		public Object createInstance() 
			throws com.sun.star.uno.Exception, 
				   com.sun.star.uno.RuntimeException
		{
			return createInstanceWithArguments(null);
		}

		public java.lang.Object createInstanceWithArguments(Object[] arguments) 
			throws com.sun.star.uno.Exception, 
				   com.sun.star.uno.RuntimeException 
		{
			return new ConnectionBroker(_xMultiServiceFactory, arguments);
		}

		// XServiceInfo
		public String getImplementationName() throws com.sun.star.uno.RuntimeException {
			return ConnectionBroker.class.getName();
		}

		public boolean supportsService( /*IN*/String ServiceName ) throws com.sun.star.uno.RuntimeException {
			String services[] = getSupportedServiceNames();

			boolean found = false;

			for(int i = 0; i < services.length && !found; ++i)
				found = services[i].equals(__serviceName);

			return found;
		}

		public String[] getSupportedServiceNames() throws com.sun.star.uno.RuntimeException {
			return new String[]{ConnectionBroker.__serviceName};
		}
	}

	public static XSingleServiceFactory getServiceFactory(String implName, 
														 XMultiServiceFactory multiFactory, 
														 XRegistryKey regKey)
	{
		XSingleServiceFactory xSingleServiceFactory = null;

	    if (implName.equals(ConnectionBroker.class.getName()) )
	        xSingleServiceFactory =  new Factory(multiFactory, regKey);
	    
	    return xSingleServiceFactory;
	}
	
	public static boolean writeRegistryServiceInfo(XRegistryKey regKey) {
	    boolean result = false;
	    
  	    try {
	        XRegistryKey newKey = regKey.createKey("/" + Factory.class.getName() + "/UNO/SERVICES");
	       
			newKey.createKey(__serviceName);
	        
	        result = true;
  	    }
  	    catch (Exception ex) {
  	        System.err.println(">>>ConnectionBroker.writeRegistryServiceInfo " + ex);
  	    }
	    
	    return result;
    }
}

