/*************************************************************************
 *
 *  $RCSfile: SimpleRegistry.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.comp.simpleregistry;


import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StreamTokenizer;

import java.net.URL;
import java.net.URLConnection;

import java.util.Enumeration;
import java.util.Hashtable;

import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XSingleServiceFactory;

import com.sun.star.registry.InvalidRegistryException;
import com.sun.star.registry.InvalidValueException;
import com.sun.star.registry.MergeConflictException;
import com.sun.star.registry.RegistryKeyType;
import com.sun.star.registry.RegistryValueType;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.registry.XSimpleRegistry;


import com.sun.star.comp.loader.RegistrationClassFinder;


class RegistryKey implements XRegistryKey {
	private static final boolean DEBUG = false;

	protected            String _name;
	protected           boolean _readonly;
	protected         Hashtable _keys = new Hashtable();
	protected RegistryValueType _registryValueType;
	protected               int _long;
	protected               int _longs[];
	protected            String _string;
	protected            String _strings[];
	protected              byte _byte;
	protected              byte _bytes[];

	RegistryKey(String name, boolean readonly) {
		if(DEBUG) System.err.println("##### " + getClass().getName() + " - init:" + name + " " + readonly);

		    _name = name;
		_readonly = readonly;
	}

	RegistryKeyType getRegistryKeyType() {
		return RegistryKeyType.KEY;
	}

	void read(StreamTokenizer streamTokenizer) throws IOException, InvalidRegistryException {
		if(DEBUG) System.err.println("##### " + getClass().getName() + " - read1 : " + streamTokenizer);

		switch(streamTokenizer.ttype) {
		case StreamTokenizer.TT_EOF:
			break;

		case StreamTokenizer.TT_EOL:
			break;

		case StreamTokenizer.TT_NUMBER:
			_registryValueType = RegistryValueType.fromInt((int)streamTokenizer.nval);

			if(DEBUG) System.err.println("##### " + getClass().getName() + " - read2 : " + streamTokenizer);

			streamTokenizer.nextToken();

			if(_registryValueType == RegistryValueType.NOT_DEFINED)
				throw new RuntimeException(this.getClass().getName() + " read");
			else if(_registryValueType == RegistryValueType.LONG)
				setLongValue((int)streamTokenizer.nval);
			else if(_registryValueType == RegistryValueType.ASCII)
				setAsciiValue(streamTokenizer.sval);
			else if(_registryValueType == RegistryValueType.STRING)
				setStringValue(streamTokenizer.sval);
			else if(_registryValueType == RegistryValueType.BINARY)
				; //setBinaryValue(Byte.parseByte(streamTokenizer.sval));
			else if(_registryValueType == RegistryValueType.LONGLIST)
				;
			else if(_registryValueType == RegistryValueType.ASCIILIST)
				;
			else if(_registryValueType == RegistryValueType.STRINGLIST)
				;

			streamTokenizer.nextToken();

            break;

		case StreamTokenizer.TT_WORD:
			do {
				RegistryKey registryKey = new RegistryKey(streamTokenizer.sval, _readonly);
				_keys.put(streamTokenizer.sval, registryKey);
				streamTokenizer.nextToken();

				registryKey.read(streamTokenizer);
			}
			while(streamTokenizer.ttype != StreamTokenizer.TT_EOL && streamTokenizer.ttype != StreamTokenizer.TT_EOF);

		    break;
		}
	}

	void write(String prefix, OutputStreamWriter outputStreamWriter) throws IOException {
		if(_registryValueType != RegistryValueType.NOT_DEFINED) {
			outputStreamWriter.write(prefix + _name + " " + _registryValueType.getValue() + " ");

			if(_registryValueType == RegistryValueType.LONG)
				outputStreamWriter.write(_long);
			else if(_registryValueType == RegistryValueType.ASCII)
				outputStreamWriter.write(_string);
			else if(_registryValueType == RegistryValueType.STRING)
				outputStreamWriter.write(_string);
			else if(_registryValueType == RegistryValueType.BINARY)
				; //outputStream.write(_byte);
			else if(_registryValueType == RegistryValueType.LONGLIST)
				;
			else if(_registryValueType == RegistryValueType.ASCIILIST)
				;
			else if(_registryValueType == RegistryValueType.STRINGLIST)
				;

			outputStreamWriter.write("\n");
		}

		Enumeration elements = _keys.elements();
		while(elements.hasMoreElements()) {
			RegistryKey registryKey = (RegistryKey)elements.nextElement();
			registryKey.write(prefix + _name + " ", outputStreamWriter);
		}
	}
	
    // Attributes
    public String getKeyName() throws com.sun.star.uno.RuntimeException {
		return _name;
	}

    // Methods
    public boolean isReadOnly() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return _readonly;
	}

    public boolean isValid() throws com.sun.star.uno.RuntimeException {
		return true;
	}

    public RegistryKeyType getKeyType( /*IN*/String rKeyName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return ((RegistryKey)_keys.get(rKeyName)).getRegistryKeyType();
	}

    public RegistryValueType getValueType() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return _registryValueType;
	}

    public int getLongValue() throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _long;
	}

    public void setLongValue( /*IN*/int value ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.LONG;
		_long = value;
	}

    public int[] getLongListValue(  ) throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _longs;
	}

    public void setLongListValue( /*IN*/int[] seqValue ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.LONGLIST;
		_longs = seqValue;
	}

    public String getAsciiValue(  ) throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _string;
	}

    public void setAsciiValue( /*IN*/String value ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.ASCII;
		_string = value;
	}

    public String[] getAsciiListValue(  ) throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _strings;
	}

    public void setAsciiListValue( /*IN*/String[] seqValue ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.ASCIILIST;
		_strings = seqValue;
	}

    public String getStringValue(  ) throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _string;
	}

    public void setStringValue( /*IN*/String value ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.STRING;
		_string = value;
	}

    public String[] getStringListValue(  ) throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _strings;
	}

    public void setStringListValue( /*IN*/String[] seqValue ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.STRINGLIST;
		_strings = seqValue;
	}

    public byte[] getBinaryValue(  ) throws InvalidRegistryException, InvalidValueException, com.sun.star.uno.RuntimeException {
		return _bytes;
	}

    public void setBinaryValue( /*IN*/byte[] value ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_registryValueType = RegistryValueType.BINARY;
		_bytes = value;
	}

    public XRegistryKey openKey( /*IN*/String aKeyName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return (XRegistryKey)_keys.get(aKeyName);
	}

    public XRegistryKey createKey( /*IN*/String aKeyName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		XRegistryKey xRegistryKey = new RegistryKey(aKeyName, _readonly);

		_keys.put(aKeyName, xRegistryKey);
	   
		return xRegistryKey;
	}

    public void closeKey() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
	}

    public void deleteKey( /*IN*/String rKeyName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		_keys.remove(rKeyName);
	}

    public synchronized XRegistryKey[] openKeys() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		XRegistryKey xRegistryKeys[] = new XRegistryKey[_keys.size()];

		Enumeration elements = _keys.elements();
		int i = 0;
		while(elements.hasMoreElements()) {
			xRegistryKeys[i ++] = (XRegistryKey)elements.nextElement();
			System.err.println("##### openKeys:" + ((RegistryKey)xRegistryKeys[i - 1])._name + " " + ((RegistryKey)xRegistryKeys[i - 1])._registryValueType);
		}

		return xRegistryKeys;
	}

    public String[] getKeyNames(  ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return null;
	}

    public boolean createLink( /*IN*/String aLinkName, /*IN*/String aLinkTarget ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return false;
	}

    public void deleteLink( /*IN*/String rLinkName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
	}

    public String getLinkTarget( /*IN*/String rLinkName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return null;
	}

    public String getResolvedName( /*IN*/String aKeyName ) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return null;
	}
}

public class SimpleRegistry implements XSimpleRegistry {
	protected         URL _url;
	protected boolean     _readonly;
	protected RegistryKey _rootKey;
	protected boolean     _valid = false;

    // Attributes
    public String getURL() throws com.sun.star.uno.RuntimeException {
		return _url.toString();
	}

    // Methods
    public synchronized void open(/*IN*/String rURL, /*IN*/boolean bReadOnly, /*IN*/boolean bCreate) throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		if(_url != null)
			throw new InvalidRegistryException("already open");

		try {
			     _url = new URL(rURL);
			_readonly = bReadOnly;
		
			URLConnection urlConnection = _url.openConnection();
			InputStream inputStream = urlConnection.getInputStream();
			
			_rootKey = new RegistryKey("/", false);
			StreamTokenizer streamTokenizer = new StreamTokenizer(inputStream);
			streamTokenizer.nextToken();

			while(streamTokenizer.ttype != StreamTokenizer.TT_EOF) {
				System.err.println("#################### " + streamTokenizer);
				_rootKey.read(streamTokenizer);
			}
			
			inputStream.close();
		}
		catch(IOException exception) {
			throw new com.sun.star.uno.RuntimeException(exception.toString());
		}

		_valid = true;
	}

    public boolean isValid() throws com.sun.star.uno.RuntimeException {
		return _valid;
	}

    public synchronized void close() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		try {
			URLConnection urlConnection = _url.openConnection();
			OutputStream outputStream = urlConnection.getOutputStream();
			
			_rootKey.write("", new OutputStreamWriter(outputStream));
			
			outputStream.close();
		}
		catch(Exception exception) {
			throw new com.sun.star.uno.RuntimeException(exception.toString());
		}

		_valid = false;
	}

    public void destroy() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
	}

    public XRegistryKey getRootKey() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return _rootKey;
	}

    public boolean isReadOnly() throws InvalidRegistryException, com.sun.star.uno.RuntimeException {
		return _readonly;
	}

    public void mergeKey( /*IN*/String aKeyName, /*IN*/String aUrl ) throws InvalidRegistryException, MergeConflictException, com.sun.star.uno.RuntimeException {
	}


	public static XSingleServiceFactory getServiceFactory( String implName, 
	                                                       XMultiServiceFactory multiFactory, 
	                                                       XRegistryKey regKey)
	{	     
		XSingleServiceFactory xSingleServiceFactory = null;

	    if (implName.equals(SimpleRegistry.class.getName()))
	        xSingleServiceFactory = new SimpleRegistryFactory(multiFactory);
	    
	    return xSingleServiceFactory;
	}
}

