/*************************************************************************
 *
 *  $RCSfile: Proxy.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.lib.uno.environments.java;


import java.io.IOException;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;


import com.sun.star.lib.sandbox.generic.Dispatcher;
import com.sun.star.lib.sandbox.generic.DispatcherAdapterBase;
import com.sun.star.lib.sandbox.generic.DispatcherAdapterFactory;

import com.sun.star.lib.util.WeakTable;

import com.sun.star.uno.IQueryInterface;
import com.sun.star.uno.MappingException;
import com.sun.star.uno.Type;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XInterface;

import com.sun.star.lib.uno.typeinfo.MethodTypeInfo;

/**
 * The purpose of this class is to provide a ProxyFactory.
 * Proxies are needed to integrate objects of other
 * environments seamsless into java.
 * A proxy delegates all method calls to provided object.
 * <p>
 * @version 	$Revision: 1.1.1.1 $ $ $Date: 2000/09/18 15:27:52 $
 * @author 	    Kay Ramme
 * @see         com.sun.star.uno.IQueryInterface
 * @since       UDK1.0
 */
public class Proxy extends DispatcherAdapterBase implements IQueryInterface {
	/**
	 * When set to true, enables various debugging output.
	 */
	static public final boolean DEBUG = false;

	/**
	 * Count the instances created of this <code>Proxy</code> class.
	 */
	static public int __instances;

	protected Class      _zInterface;
	protected IRequester _requester;
	protected boolean    _disposed = false;
	protected boolean    _virtuell;

	/**
	 * This constructor is only public due to problems
	 * with java reflection, it does not belong to the provider
	 * <code>api</code>
	 */
	public Proxy() {
		super();

		++ __instances;
	}

	private void acquire() throws Throwable {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".acquire:" + (String)object);
		_requester.sendRequest(object, new Type(_zInterface), "acquire", null, null);
	}

	private void release() throws Throwable {
		if(!_virtuell) {
			if(DEBUG) System.err.println("#####"  + getClass().getName() + ".released:" + (String)object);
			_requester.sendRequest(object, new Type(_zInterface), "release", null, null);
		}
	}

	private void setObject(Object object) {
		this.object = object;
	}

	private void setInterface(Class zInterface, IRequester requester, boolean virtuell) {
		_zInterface = zInterface;
		_requester  = requester;
		_virtuell   = virtuell;
	}

	final public synchronized void __dispose() throws Throwable {
		if(!_disposed) {
			_disposed = true;
			release();
		}
	}

	final public void finalize() throws Throwable {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".finalize - " + _zInterface);

		-- __instances;
		__dispose();
	}


	/**
	 * Returns an object implementing the desired interface.
	 *<p>
	 * @return     the desired interface if available, otherwise null.
	 * @param      zInterface the class of the desired interface.
	 * @see        com.sun.star.uno.UnoRuntime
	 * @see        com.sun.star.uno.IQueryInterface#queryInterface
	 */
	final public Object queryInterface(Class zInterface) throws MappingException {
		Object result = null;

		if(_zInterface == zInterface)
			result = this;
		else {
			try {
				result = _requester.sendRequest(object, new Type(_zInterface), "queryInterface", new Object[]{new Type(zInterface.getName())}, null);
			}
			catch(Throwable exception) {
				if(DEBUG) {
					System.err.println("##### " + getClass().getName() + ".queryInterface exception occured:" + exception);
					exception.printStackTrace();
				}
				throw new MappingException(getClass().getName() + ".queryInterface - exception occured:" + exception);
			}
		}

		return result;
	}

	/** 
	 * Tests if the given object is an interface of the same underlying object.
	 * <p>
	 * @return     true, if the underlying object is the same, otherwise false.
	 * @param      object an object representing an interface.
	 * @see        com.sun.star.uno.UnoRuntime
	 * @see        com.sun.star.uno.IQueryInterface#isSame
	 */
	final public boolean isSame(Object object) throws MappingException {
		boolean result = false;

		object = WeakTable.__getObject(object);

		if((object instanceof Proxy))
			result = ((Proxy)object).object == this.object;

		return result;
	}

	/**
	 * Get the Oid of the underlying object.
	 * <p>
	 * @return    the oid of the underlying object.
	 * @see        com.sun.star.uno.IQueryInterface#getOid
	 */
	final public String getOid() {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".getOid:" + (String)object);
		return (String)object;
	}

	static class IDispatcher implements Dispatcher {
		protected Class      _zInterface;
		protected IRequester _requester;

		IDispatcher(Class zInterface, IRequester requester) {
			_zInterface = zInterface;
			_requester  = requester;
		}

		public Object invoke(Object object, String methodName, Object params[]) throws Exception {
			if(DEBUG) System.err.println("##### " + getClass().getName() + ".invoke:" + methodName);

			Object result = null;
			try {
				result = _requester.sendRequest(object, new Type(_zInterface), methodName, params, null);
			}
			catch(Throwable throwable) {
				if(throwable instanceof Exception) { // can we rethrow the exception?
					throwable.fillInStackTrace();

					throw (Exception)throwable;
				}

				System.err.println(getClass().getName() + ".invoke - unforseen exception:" + throwable);
				throwable.printStackTrace();
			}

			return result;
		}
	}

	/**
	 * Creates a new <code>Proxy</code> for the given interface.
	 * The newly created <code>Proxy</code> delegates all calls
	 * to the <code>sendRequest</code> method of the <code>IRequester</code>
	 * interface.
	 * <p>
	 * @return    the newly created <code>Proxy</code>
	 * @param     requester   the requester to which the calls become delegated
	 * @param     oId         the oid of the proxy.
	 * @param     zInterface  the interface the proxy should implement
	 * @param     virtuell    determines if the proxy should be virtuell, e.g. does not send a release
	 */
  	static public Object create(IRequester requester, String oId, Class zInterface, boolean virtuell) throws MappingException {
  		Class dispatcherClass = DispatcherAdapterFactory.createDispatcherAdapter(zInterface, "com/sun/star/lib/uno/environments/java/Proxy");

  		Proxy javaProxy;

		try {
			javaProxy = (Proxy)dispatcherClass.newInstance();
			javaProxy.setObject(new IDispatcher(zInterface, requester), oId);
			javaProxy.setInterface(zInterface, requester, virtuell);

			if(DEBUG) System.err.println("##### " + Proxy.class.getName() + ".create:" + oId + " " + zInterface);
		}
		catch(java.lang.Exception exception) {
			exception.printStackTrace();

			throw new MappingException();
		}

		return javaProxy;
	}

	static public int getInstances() {
		return __instances;
	}
}

