/*************************************************************************
 *
 *  $RCSfile: Protocol.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.lib.uno.environments.remote;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import java.util.Hashtable;
import java.util.Stack;


import com.sun.star.lib.uno.typeinfo.AttributeTypeInfo;
import com.sun.star.lib.uno.typeinfo.MemberTypeInfo;
import com.sun.star.lib.uno.typeinfo.MethodTypeInfo;
import com.sun.star.lib.uno.typeinfo.ParameterTypeInfo;
import com.sun.star.lib.uno.typeinfo.TypeInfo;

import com.sun.star.uno.Any;
import com.sun.star.uno.MappingException;
import com.sun.star.uno.Type;
import com.sun.star.uno.XInterface;


/**
 * This class implements the complete IIOP protocol
 * from corba. The functionality is reachable through
 * the <code>IProtocol</code> interface. 
 * <p>
 * @version 	$Revision: 1.1.1.1 $ $ $Date: 2000/09/18 15:27:53 $
 * @author 	    Kay Ramme
 * @see         com.sun.star.lib.uno.environments.remote.IProtocol
 * @since       UDK1.0
 */
public abstract class Protocol implements IProtocol  {
	/**
	 * When set to true, enables various debugging output.
	 */
	static public final boolean DEBUG = false;

	static private final Hashtable __classMDDesc = new Hashtable();

	static private Class __anyArray;
	static private Class __xInterfaceArray;
	static private Class __xInterfaceArrayArray;

	static {
		try {
			__anyArray = Class.forName("[Lcom.sun.star.uno.Any;");
			__xInterfaceArray = Class.forName("[Lcom.sun.star.uno.XInterface;");
			__xInterfaceArrayArray = Class.forName("[[Lcom.sun.star.uno.XInterface;");
		}
		catch(ClassNotFoundException classNotFoundException) {
			System.err.println("##### Type - ERORR:" + classNotFoundException);
		}
	}

	static public final class MMDesc {
		public int            _index;
		public String         _name;
		public Method         _method;
		public Class          _in_sig[];
		public Class          _out_sig[];
		public Class          _res;
		public MethodTypeInfo _methodTypeInfo;

		MMDesc(int index, String name, Method method, Class in_sig[], Class out_sig[], Class res, MethodTypeInfo methodTypeInfo) {
			_name = name;
			_index = index;
			_method = method;
			_in_sig = in_sig;
			_out_sig = out_sig;
			_res = res;
			_methodTypeInfo = methodTypeInfo;
		}

		MMDesc(int offset, Method method, MethodTypeInfo methodTypeInfo) {
			Class zClass = method.getDeclaringClass();

			_name = method.getName();
			_index = offset + methodTypeInfo.getIndex();
			_method = method;
			_methodTypeInfo = methodTypeInfo;

			_in_sig  = __getInSig(zClass, _name, method);
			_out_sig = __getOutSig(zClass, _name, method);
			_res     = __getRetSig(zClass, _name, method, methodTypeInfo);
		}

		MMDesc(int offset, Method method, AttributeTypeInfo attributeTypeInfo) {
			_name = method.getName();
			_index = offset + attributeTypeInfo.getIndex();
			_method = method;

			Class zClass = method.getReturnType();
			if(zClass == void.class)
				zClass = method.getParameterTypes()[0];

			if(attributeTypeInfo.isAny())
				if(zClass.isArray())
					zClass = __anyArray;
				else
					zClass= Any.class;

			else if(attributeTypeInfo.isInterface())
				if(!zClass.isInterface())
					if(zClass.isArray())
						zClass = __xInterfaceArray;
					else
						zClass = XInterface.class;


			if(method.getReturnType() == void.class) {
				_in_sig  = new Class[]{zClass};
				_out_sig = new Class[]{null};
				_res = void.class;
			}
			else {
				_in_sig  = new Class[0];
				_out_sig = new Class[0];
				_res = zClass;
			}
		}
	}

	static protected final class MDDesc {
		MDDesc    _super;
		int       _offset;
		MMDesc    _mMDescs[];
		Hashtable _nameToIndex  = new Hashtable();

		MDDesc(Class zClass) {
			if(zClass == XInterface.class) { // special handling for XInterface
				_super = null;
				_offset = 0;
				_mMDescs = new MMDesc[3];
				
				_mMDescs[0] = new MMDesc(0, "queryInterface", null, new Class[]{Type.class}, new Class[]{null}, Any.class, new MethodTypeInfo("queryInterface", 0, 0));
				_mMDescs[1] = new MMDesc(1, "acquire",        null, new Class[0],            new Class[0],      null,      new MethodTypeInfo("acquire", 1, TypeInfo.ONEWAY));
				_mMDescs[2] = new MMDesc(2, "release",        null, new Class[0],            new Class[0],      null,      new MethodTypeInfo("release", 2, TypeInfo.ONEWAY));

				_nameToIndex.put(_mMDescs[0]._name, new Integer(0));
				_nameToIndex.put(_mMDescs[1]._name, new Integer(1));
				_nameToIndex.put(_mMDescs[2]._name, new Integer(2));
			}
			else {
				Class zInterfaces[] = zClass.getInterfaces();
				
				if(zInterfaces != null && zInterfaces.length > 0) {
					_super = getMDDesc(zInterfaces[0]);
					_offset = _super._offset + _super._mMDescs.length;
				}
				
				Method methods[] = zClass.getDeclaredMethods();
				_mMDescs = new MMDesc[methods.length];
				
				for(int i = 0; i < methods.length; ++ i) {
					MethodTypeInfo methodTypeInfo = __findMethodTypeInfo(zClass, methods[i].getName());

					if(methodTypeInfo == null) { // it is an attribute
						AttributeTypeInfo attributeTypeInfo = __findAttributeTypeInfo(zClass, methods[i].getName().substring(3));

						if(methods[i].getName().substring(0, 3).equals("get")) {
							_mMDescs[attributeTypeInfo.getIndex()] = new MMDesc(_offset, methods[i], attributeTypeInfo);
							_nameToIndex.put(methods[i].getName(), new Integer(attributeTypeInfo.getIndex() + _offset));
						}
						else {
							_mMDescs[attributeTypeInfo.getIndex() + 1] = new MMDesc(_offset, methods[i], attributeTypeInfo);
							_nameToIndex.put(methods[i].getName(), new Integer(attributeTypeInfo.getIndex() + 1 + _offset));
						}
					}
					else { // is is an ordinary method
						_mMDescs[methodTypeInfo.getIndex()] = new MMDesc(_offset, methods[i], methodTypeInfo);
						
//    					System.err.println("################### " + methods[i].getName() + " " + methodTypeInfo.getIndex() + " " + _offset);
						_nameToIndex.put(methods[i].getName(), new Integer(methodTypeInfo.getIndex() + _offset));
//  					System.err.println("################### " + methods[i].getName() + " " + i + " " + _offset);
//  					_nameToIndex.put(methods[i].getName(), new Integer(i + _offset));
					}
				}
			}
		}
	};


	static protected final MDDesc getMDDesc(Class zClass) {
		MDDesc mDDesc = (MDDesc)__classMDDesc.get(zClass);

		if(mDDesc == null) {
			mDDesc = new MDDesc(zClass);
			
			__classMDDesc.put(zClass, mDDesc);
		}

		return mDDesc;
	}

	static protected final MMDesc getMMDesc(MDDesc mDDesc, int index) {
		MMDesc mMDesc = null;

		if(index >= mDDesc._offset)
			mMDesc = mDDesc._mMDescs[index - mDDesc._offset];
		else
			mMDesc = getMMDesc(mDDesc._super, index);

		return mMDesc;
	}

	static protected final Integer getMethodIndex(MDDesc mDDesc, String operation) {
		Integer index = (Integer)mDDesc._nameToIndex.get(operation);

		if(index == null)
			index = getMethodIndex(mDDesc._super, operation);

		return index;
	}

	static protected final MMDesc getMMDesc(Class zClass, int index) {
//  		if(DEBUG) System.err.println("urp.getMMDesc:" + zClass + " " + index);

		MMDesc mMDesc = null;

		mMDesc = getMMDesc(getMDDesc(zClass), index);

		return mMDesc;
	}

	static protected final int getMethodIndex(Class zClass, String operation) {
		return getMethodIndex(getMDDesc(zClass), operation).intValue();
	}

	static protected final MMDesc getMMDesc(Class zClass, String operation) {
		MMDesc mMDesc = null;

		mMDesc = getMMDesc(zClass, getMethodIndex(zClass, operation));

		return mMDesc;
	}



	static private Hashtable __methodss = new Hashtable();

	/**
	 * Gets the methods of a class as a hashtable.
	 * Uses reflection for first time and stores the methods
	 * array in hashtable for later use.
	 * <p>
	 * This method does not belong to the provided <code>api</code>
	 * <p>
	 * @return the methods
	 * @param  zClass   the class
	 * @see    java.lang.Class#getMethods
	 */
	static public Hashtable __getMethodsAsTable(Class zClass) {
		Hashtable methods_tab = (Hashtable)__methodss.get(zClass);

		if(methods_tab == null) {
			methods_tab = new Hashtable();

			Method methods[] = zClass.getMethods();

			for(int i = 0; i < methods.length; ++ i) {
				methods_tab.put(methods[i].getName(), methods[i]);
			}

			__methodss.put(zClass, methods_tab);
		}

		return methods_tab;
	}


	static private Hashtable __declaredFields = new Hashtable();

	/**
	 * Gets the fields of a class.
	 * Uses reflection for first time and stores the fields
	 * array in hashtable for later use.
	 * After the first call this method is faster than the original
	 * <p>
	 * This method does not belong to the provided <code>api</code>
	 * <p>
	 * @return the fields
	 * @param  zClass   the class
	 * @see    java.lang.Class#getFields
	 */
	static public Field[] __getDeclaredFields(Class zInterface) {
		Field fields[] = (Field[])__declaredFields.get(zInterface);

		if(fields == null) {
			fields = zInterface.getDeclaredFields();

			int size = 0;
			for(int i = 0; i < fields.length; ++ i)
				if((fields[i].getModifiers() & (Modifier.STATIC | Modifier.TRANSIENT)) == 0 ) // neither static nor transient ?
					++ size;

			Field mFields[] = new Field[size];
			size = 0;
			for(int i = 0; i < fields.length; ++ i)
				if((fields[i].getModifiers() & (Modifier.STATIC | Modifier.TRANSIENT)) == 0 ) // neither static nor transient ?
					mFields[size ++] = fields[i];

			__declaredFields.put(zInterface, mFields);
		}

		return fields;
	}


	static public Method __findMethod(Class zInterface, String methodName) {
		return (Method)__getMethodsAsTable(zInterface).get(methodName);
	}

	/**
	 * Gets the type infos of a class without inherited staff.
	 * <p>
	 * This method does not belong to the provided <code>api</code>
	 * <p>
	 * @return the type infos
	 * @param  zInterface   the class
	 * @see                 com.sun.star.lib.uno.TypeInfo
	 */
	static public TypeInfo[] __getTypeInfos(Class zInterface) {
		TypeInfo typeInfos[] = null;

		try {
			Field field = zInterface.getDeclaredField("UNOTYPEINFO");
			typeInfos = (TypeInfo[])field.get(null);

			if(typeInfos == null) { 
				// This is a workaround for a bug in solaris JDK 1.2, so please do not remove.
				// Interface members are sometimes not fully initialized,
				// reloading helps, who knows why?
				Class.forName(zInterface.getName());
				typeInfos = (TypeInfo[])field.get(null);
			}
		}
		catch(NoSuchFieldException noSuchFieldException) {
			if(DEBUG) System.err.println("__getTypeInfos:" + zInterface + " - exception:" + noSuchFieldException);
		}
		catch(ClassNotFoundException classNotFoundException) {
			if(DEBUG) System.err.println("__getTypeInfos:" + zInterface + " - exception:" + classNotFoundException);
		}
		catch(IllegalAccessException illegalAccessException) {
			System.err.println("__getTypeInfos:" + zInterface + " - exception:" + illegalAccessException);
		}
		
		return typeInfos;
	}

	static private Hashtable __typeTable = new Hashtable();

	/**
	 * Gets the type infos of a class with inherited infos.
	 * <p>
	 * This method does not belong to the provided <code>api</code>
	 * <p>
	 * @return the type infos
	 * @param  zInterface   the interface
	 * @see                 com.sun.star.lib.uno.TypeInfo
	 */
	static public TypeInfo[] getAllTypeInfos(Class zInterface) {
		TypeInfo typeInfos[] = null;

		typeInfos = (TypeInfo[])__typeTable.get(zInterface);

		if(typeInfos == null) {
			TypeInfo type_typeInfos[][] = new TypeInfo[256][];
			int index = 0;
			int count = 0;
			
			Class others[] = zInterface.getInterfaces();
			
			type_typeInfos[index] = __getTypeInfos(zInterface);
			if(type_typeInfos[index] != null)
				count += type_typeInfos[index ++].length;
			
			for(int i = 0; i < others.length; ++ i) {
				type_typeInfos[index] = getAllTypeInfos(others[i]);
				
				if(type_typeInfos[index] != null)
					count += type_typeInfos[index ++].length;
			}
			
			typeInfos = new TypeInfo[count];
			count = 0;
			
			for(int i = 0; i < index; ++ i)
				for(int j = 0; j < type_typeInfos[i].length; ++ j) {
					typeInfos[count ++] = type_typeInfos[i][j];
				}

			__typeTable.put(zInterface, typeInfos);
		}

		return typeInfos;
	}

	/**
	 * Find the member type info for the given interface and member name
	 * <p>
	 * This method does not belong to the provided <code>api</code>
	 * <p>
	 * @return the member type infos
	 * @param  zInterface  the interface
	 * @param  name        the member name
	 * @see                com.sun.star.lib.uno.MemberTypeInfo
	 */
	static public MemberTypeInfo __findMemberTypeInfo(Class zInterface, String name) {
		MemberTypeInfo memberTypeInfo = null;

		TypeInfo typeInfos[] = getAllTypeInfos(zInterface);
		
		if(typeInfos != null) 
			for(int i = 0; i < typeInfos.length; ++ i) {
				if(typeInfos[i] instanceof MemberTypeInfo && typeInfos[i].getName().equals(name)) {
					memberTypeInfo = (MemberTypeInfo)typeInfos[i];
					break;
				}
			}

		return memberTypeInfo;
	}

	/**
	 * Find the attribute type info for the given interface and attribute name
	 * <p>
	 * This method does not belong to the provided <code>api</code>
	 * <p>
	 * @return the attribute type infos
	 * @param  zInterface  the interface
	 * @param  name        the attribute name
	 * @see                com.sun.star.lib.uno.AttributeTypeInfo
	 */
	static public AttributeTypeInfo __findAttributeTypeInfo(Class zInterface, String name) {
		AttributeTypeInfo attributeTypeInfo = null;

		TypeInfo typeInfos[] = getAllTypeInfos(zInterface);
		
		if(typeInfos != null) 
			for(int i = 0; i < typeInfos.length; ++ i) {
				if(typeInfos[i] instanceof AttributeTypeInfo && typeInfos[i].getName().equals(name)) {
					attributeTypeInfo = (AttributeTypeInfo)typeInfos[i];
					break;
				}
			}

		return attributeTypeInfo;
	}

	/**
	 * Find the method type info for the given interface and method name.
	 * <p>
	 * This method does not belong to the provided <code>api</code>.
	 * <p>
	 * @return the method type infos
	 * @param  zInterface  the interface
	 * @param  methodName  the method name
	 * @see                com.sun.star.lib.uno.MethodTypeInfo
	 */
	static public MethodTypeInfo __findMethodTypeInfo(Class zInterface, String methodName) {
		MethodTypeInfo methodTypeInfo = null;
		
		TypeInfo typeInfos[] = getAllTypeInfos(zInterface);
		
		if(typeInfos != null) 
			for(int i = 0; i < typeInfos.length; ++ i) {
				if(typeInfos[i] instanceof MethodTypeInfo && typeInfos[i].getName().equals(methodName)) {
					methodTypeInfo = (MethodTypeInfo)typeInfos[i];
					break;
				}
			}

		return methodTypeInfo;
	}

	/**
	 * Find the parameter type info for the given interface, method name and parameter index.
	 * <p>
	 * This method does not belong to the provided <code>api</code>.
	 * <p>
	 * @return the parameter type infos
	 * @param  zInterface  the interface
	 * @param  methodName  the method name
	 * @param  index       the parameter index
	 * @see                com.sun.star.lib.uno.ParameterTypeInfo
	 */
	static public ParameterTypeInfo __findParameterTypeInfo(Class zInterface, String methodName, int index) { 
		ParameterTypeInfo parameterTypeInfo = null;

		TypeInfo typeInfos[] = getAllTypeInfos(zInterface);

		if(typeInfos != null)
			for(int i = 0; i < typeInfos.length; ++ i) {
				if(typeInfos[i] instanceof ParameterTypeInfo 
				&& (index == ((ParameterTypeInfo)typeInfos[i]).getIndex())
				&& ((ParameterTypeInfo)typeInfos[i]).getMethodName().equals(methodName)) 
				{
					parameterTypeInfo = (ParameterTypeInfo)typeInfos[i];
					break;
				}
			}

		return parameterTypeInfo;
	}

	/**
	 * Gets the return type of the given method of the given interface.
	 * <p>
	 * This method does not belong to the provided <code>api</code>.
	 * <p>
	 * @return the return type
	 * @param  zInterface  the interface
	 * @param  methodName  the method name
	 * @see                java.lang.Class
	 */
	static private Class __getRetSig(Class zInterface, String methodName, Method method, MethodTypeInfo methodTypeInfo) {
		Class zRetInterface = method.getReturnType();

		if(zRetInterface == Object.class && methodTypeInfo != null && methodTypeInfo.isInterface())
			if(zRetInterface.isArray())
				zRetInterface = __xInterfaceArray;
			else
				zRetInterface = XInterface.class;

		return zRetInterface;
	}

	static private Class __takeCareOf_XInterface_and_java_lang_Object_as_in(ParameterTypeInfo typeInfo, Class parameterClass) {
		if(typeInfo != null) { // if there is special typeinfo, use it
			if(typeInfo.isIN()) {
				// blackdown 118 bug workaround
				__getDeclaredFields(parameterClass);

				// see if the typeinfo says, that the parameter is an interface
				// and the parameter is not assignable to xinterface (mapping of xinterface to java.lang.Object)
				// set the parameter to class of xinterface, cause it must be assignable
				if(typeInfo.isInterface() && !parameterClass.isInterface())
					if(parameterClass.isArray())
						parameterClass = __xInterfaceArray;
					else
						parameterClass = XInterface.class;
			}
			else // this is an out parameter, and must not be passed
				parameterClass = null;
		}

		return parameterClass;
	}

	static private Class __takeCareOf_XInterface_and_java_lang_Object_as_out(ParameterTypeInfo typeInfo, Class parameterClass) {
		if(typeInfo != null && typeInfo.isOUT()) { // if there is special typeinfo and it is an outparameter, use it
			// blackdown 118 bug workaround
			__getDeclaredFields(parameterClass);

			// see if the typeinfo says, that the parameter is an interface
			// and the parameter is not assignable to xinterface (mapping of xinterface to java.lang.Object)
			// set the parameter to class of xinterface, cause it must be assignable
			if(typeInfo.isInterface() && !parameterClass.isInterface()) {
				if(parameterClass.isArray())
					parameterClass = __xInterfaceArrayArray;
				else
					parameterClass = __xInterfaceArray;
			}
		}
		else // this is not an out parameter
			parameterClass = null;

		return parameterClass;
	}

	/**
	 * Gets the types of the input parameters in a signature 
	 * corresponding array.
	 * <p>
	 * This method does not belong to the provided <code>api</code>.
	 * <p>
	 * @return the array of input parameter types
	 * @param  zInterface  the interface
	 * @param  methodName  the method name
	 * @see                java.lang.Class
	 */
	static private Class[] __getInSig(Class zInterface, String methodName, Method method) {
		Class signature[] = method.getParameterTypes();
			
		Class in_sig[] = new Class[signature.length];
			
		for(int i = 0; i < in_sig.length; ++i) {
			ParameterTypeInfo typeInfo = __findParameterTypeInfo(zInterface, methodName, i);
			
			in_sig[i] = __takeCareOf_XInterface_and_java_lang_Object_as_in(typeInfo, signature[i]);
		}

		return in_sig;
	}

	/**
	 * Gets the types of the output parameters in a signature 
	 * corresponding array.
	 * <p>
	 * This method does not belong to the provided <code>api</code>.
	 * <p>
	 * @return the array of output parameter types
	 * @param  zInterface  the interface
	 * @param  methodName  the method name
	 * @see                java.lang.Class
	 */
	static private Class[] __getOutSig(Class zInterface, String methodName, Method method) {
		Class signature[] = method.getParameterTypes();
			
		Class out_sig[] = new Class[signature.length];

		for(int i = 0; i < out_sig.length; ++i) {
			ParameterTypeInfo typeInfo = __findParameterTypeInfo(zInterface, methodName, i);
			
			out_sig[i] = __takeCareOf_XInterface_and_java_lang_Object_as_out(typeInfo, signature[i]);
		}

		return out_sig;
	}

	protected int               _requestsSend;
	protected int               _requestsRecieved;

	private Hashtable _pending_replys   = new Hashtable();
	private Hashtable _pending_requests = new Hashtable();

	protected final void putPendingReply(ThreadID threadID, Object object) {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".putPendingReply:" + threadID + " " + object);

		Stack stack = (Stack)_pending_replys.get(threadID);

		if(stack == null) {
			stack = new Stack();

			_pending_replys.put(threadID, stack);
		}

		stack.push(object);
	}

	protected final Object removePendingReply(ThreadID threadID) {
		Stack stack = (Stack)_pending_replys.get(threadID);
		Object object = stack.pop();

		if(stack.empty())
			_pending_replys.remove(threadID);

		if(DEBUG) {
			System.err.println("##### " + getClass().getName() + ".removePendingReply:" + threadID + " " + object);
//  			System.err.println("----- :" + ((MMDesc)((Object[])object)[3])._method);
		}

		
		return object;
	}


	protected final void putPendingRequest(ThreadID threadID, Object object) {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".putPendingRequest:" + threadID + " " + object);

		Stack stack = (Stack)_pending_requests.get(threadID);
		if(stack == null) {
			stack = new Stack();
			_pending_requests.put(threadID, stack);
		}

		stack.push(object);
	}

	protected final Object removePendingRequest(ThreadID threadID) {
		Stack stack = (Stack)_pending_requests.get(threadID);

		Object object = null;

		if(stack == null) 
			throw new com.sun.star.uno.RuntimeException("WARNING! received unexpected reply " + threadID + " !!!");
		else {
			object = stack.pop();
			
			if(stack.empty())
				_pending_requests.remove(threadID);
			
		}

		if(DEBUG) System.err.println("##### " + getClass().getName() + ".removePendingRequest:" + threadID + " " + object);

		return object;
	}

	public int getRequestsSendCount() {
		return _requestsSend;
	}

	public int getRequestsReceivedCount() {
		return _requestsRecieved;
	}


}

