/*************************************************************************
 *
 *  $RCSfile: ThreadID.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.lib.uno.environments.remote;


/**
 * This is the global thread id.
 * <p>
 * @version 	$Revision: 1.1.1.1 $ $ $Date: 2000/09/18 15:27:53 $
 * @author 	    Joerg Budischewski
 * @see         com.sun.star.lib.uno.environments.remote.ThreadPool
 * @see         com.sun.star.lib.uno.environments.remote.IThreadPool
 * @see         com.sun.star.lib.uno.environments.remote.Job
 * @since       UDK1.0
 */
public class ThreadID {
	static protected byte __async_count;

	protected byte   _threadId[];
	protected int    _hashCode;
	protected String _string;


	/**
	 * Constructs a new async thread id from the given thread id
	 * <p>
	 * @param  threadID     the base thread id
	 */
	public ThreadID(ThreadID threadID) {
		byte threadId[] = new byte[threadID._threadId.length + 1];

		System.arraycopy(threadID._threadId, 0, threadId, 0, threadID._threadId.length);
		threadId[threadId.length - 1] = ++ __async_count;

		init(threadId);
	}

	/**
	 * Constructs a new thread id from the given byte array
	 * <p>
	 * @param  threadID     a byte array describing a thread id
	 */
	public ThreadID(byte threadId[]) {
		init(threadId);
	}

	/**
	 * Initializes a thread id with a byte array
	 * <p>
	 * @param  threadID     a byte array describing a thread id
	 */
	private void init(byte threadId[]) {
		_threadId = threadId;
  		_string = new String(threadId);
	}

	/**
	 * Gives a hashcode.
	 * <p>
	 */
	public int hashCode() {
		//  			return _hashCode;
  		return _string.hashCode();
	}

	/**
	 * Gives a thread id described by a byte array
	 * <p>
	 * @return   a byte array
	 */
	public byte[] getBytes() {
		return _threadId;
	}

	/**
	 * Indicates whether two thread ids describe the same threadid
	 * <p>
	 * @return   <code>true</code>, if the thread ids are equal
	 * @param   othreadID    the other thread id
	 */
	public boolean equals(Object othreadID) {
  		return _string.equals(((ThreadID)othreadID)._string);
	}

	/**
	 * Gives a descriptive string
	 * <p>
	 * @return   the descriptive string
	 */
	public String toString() {
		String result = "id:";

		for(int i = 0; i < _threadId.length; ++ i) {
			String tmp = Integer.toHexString(_threadId[i]);
			if(tmp.length() < 2)
				result += "0" + tmp;
			else
				result += tmp.substring(tmp.length() - 2);
		}

		return result;
	}
}


