/*************************************************************************
 *
 *  $RCSfile: Marshal.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.lib.uno.protocols.iiop;


import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import java.io.DataOutputStream;
import java.io.IOException;

import java.util.Hashtable;


import com.sun.star.uno.Any;
import com.sun.star.uno.Ascii;
import com.sun.star.uno.AsciiString;
import com.sun.star.uno.Enum;
import com.sun.star.uno.IBridge;
import com.sun.star.uno.Type;
import com.sun.star.uno.TypeClass;
import com.sun.star.uno.Union;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XInterface;

import com.sun.star.corba.CorbaString8;
import com.sun.star.corba.CorbaUnion;
import com.sun.star.corba.ObjectKey;
import com.sun.star.corba.TCKind;

import com.sun.star.corba.iiop.ProfileBody_1_1;

import com.sun.star.corba.iop.IOR;
import com.sun.star.corba.iop.ProfileIdGroup;
import com.sun.star.corba.iop.TaggedProfile;

import com.sun.star.corba.giop.MessageHeader_1_1;
import com.sun.star.corba.giop.ReplyHeader_1_2;
import com.sun.star.corba.giop.RequestHeader_1_2;

import com.sun.star.uno.Type;

import com.sun.star.lib.uno.environments.remote.IMarshal;
import com.sun.star.lib.uno.environments.remote.Protocol;

import com.sun.star.lib.uno.typeinfo.MemberTypeInfo;


final class Marshal extends CDROutputStream implements IMarshal {
	public static boolean DEBUG = false;

	private void todo() throws com.sun.star.uno.Exception {
		throw new com.sun.star.uno.Exception(getClass().getName() + " - todo: not implemented");
	}

	protected IBridge bridge;

	Marshal(boolean littleEndian, IBridge bridge) {
		super(littleEndian);

		this.bridge = bridge;
//  		if(bridge == null)
//  			throw new NullPointerException();
	}

	/** Return the proper type kind of a class 
		
	 */
	private static int classToIIOPTypeKind( Class JavaClass )
	{
		return Unmarshal.classToIIOPTypeKind( JavaClass );
	};

	private int getStructMemberCount( Class StructClass, Class UntilClass )
	{
		int nMemberCount = 0;
		Class Superclass = StructClass.getSuperclass();
		// superclass must be filled before subclass
		if( Superclass != UntilClass )
			nMemberCount += getStructMemberCount( Superclass, UntilClass );

		// The precondition is, that the fields occure in the same order as 
		// specified in the IDL language, but this is not specified in Java.
		Field [] Fields = Protocol.__getDeclaredFields(StructClass);
//  		Field [] Fields = StructClass.getDeclaredFields();
		int nFieldsLength = Fields.length;
		for( int i = 0; i < nFieldsLength; i++ )
		{
			Field aField = Fields[i];
			if( (aField.getModifiers() & (Modifier.STATIC | Modifier.TRANSIENT)) == 0 )
				nMemberCount++;
		}
		return nMemberCount;
	}

	private void write_structtypemembers( Class StructClass, Class UntilClass ) throws Exception
	{
		Class Superclass = StructClass.getSuperclass();
		// superclass must be filled before subclass
		if( Superclass != UntilClass )
			write_structtypemembers( Superclass, UntilClass );

		// The precondition is, that the fields occure in the same order as 
		// specified in the IDL language, but this is not specified in Java.
		Field [] Fields = Protocol.__getDeclaredFields(StructClass);
		int nFieldsLength = Fields.length;
		for( int i = 0; i < nFieldsLength; i++ )
		{
			Field aField = Fields[i];
			if( (aField.getModifiers() & (Modifier.STATIC | Modifier.TRANSIENT)) == 0 )
			{
				// member name
				write_asciistring( aField.getName() );
				write_typecode( aField.getType() );
			}
		}
	}

	private void write_typecode(Object object) throws Exception {
		if(object instanceof Type)
			write_typecode((Type)object);
		else if(object instanceof Class)
			write_typecode((Class)object);
		else
			throw new com.sun.star.uno.RuntimeException("iiop.Marshal - unknown type object:" + object);
	}

	private int write_typecode(Type type) throws Exception {
		return write_typecode(type.getDescription());
	}

	private void write_typecode(int typecode) {
		write_long((short)typecode);
	}

	private int write_typecode(Class TypeClass) throws Exception {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".write_typecode:" + TypeClass);

		//Class return = null;
		String TypeName = TypeClass.getName();
		int kind = classToIIOPTypeKind( TypeClass );
		write_typecode(kind);

		// java optimize the switch statement to a jump table, only if
		// the case values are subsequent.
		switch(kind) {
			case TCKind.tk_TypeCode_value:
				;
				break;

			case TCKind.tk_Principal_value:
				todo();
				break;

			case TCKind.tk_objref_value:
				write_asciistring( TypeName );
				// dummy
				write_asciistring("");
				break;

			case TCKind.tk_except_value:
			case TCKind.tk_struct_value:
			{
				write_asciistring( TypeName );
				// dummy
				write_asciistring("");
				int nCount = getStructMemberCount( TypeClass, Object.class );
				write_long( nCount );
				if( kind == TCKind.tk_except_value )
					write_structtypemembers( TypeClass, Exception.class );
				else
					write_structtypemembers( TypeClass, Object.class );
				break;
			}

			case TCKind.tk_union_value:
			{
				todo();
				break;
			}

			case TCKind.tk_enum_value:
			{
				write_asciistring( TypeName );
				// dummy
				write_asciistring("");
				write_long( 0 );
//  				todo();
				/*
				// all enum names
				int nCount = write_long();
				for( int i = 0; i < nCount; i++ )
				{
					// member name
					write_asciistring();
				}
				*/
				break;
			}

			case TCKind.tk_string_value:
			{
				// read the bounds of an string
				write_long( 0 );
				break;
			}

			case TCKind.tk_sequence_value:
			{
				// get the element type
				Class ElementTypeCode = TypeClass.getComponentType();
				write_typecode( ElementTypeCode );
				if( ElementTypeCode != null )
				{
					TypeName = "]" + ElementTypeCode.getName();
					write_long(0);
				}
				break;
			}

			case TCKind.tk_array_value:
			case TCKind.tk_alias_value:
			case TCKind.tk_longdouble_value:
				todo();
				break;

			case TCKind.tk_wstring_value:
			{
				// write the bounds of an string
				write_long( 0 );
				break;
			}
		}
		return kind;
	}

	public void writeObject(Class zClass, Object obj) throws Exception  {
		if(DEBUG) System.err.println("##### Marshal - index:" + size + " " + obj + " " + zClass);

		if(zClass == Any.class || zClass == Object.class) {
			write_any(obj);
		}
		else if(zClass == Void.class || zClass == void.class) // void, nothing has to be done
			;
		else
			writeAnyValue(obj, Unmarshal.classToIIOPTypeKind(zClass), zClass);
	}

	private void writeAnyValue(Object obj, int nIIOPTypeKind, Class zInterface) throws Exception {
		switch(nIIOPTypeKind) {
		case TCKind.tk_null_value:
		case TCKind.tk_void_value:															 break;
		case TCKind.tk_short_value:		 write_short( ((Short)obj).shortValue() );           break;
		case TCKind.tk_long_value:		 write_long( ((Integer)obj).intValue() );			 break;
		case TCKind.tk_ushort_value:	 write_short( ((Short)obj).shortValue() );			 break;
		case TCKind.tk_ulong_value:		 write_long( ((Integer)obj).intValue() );			 break;
		case TCKind.tk_float_value:		 write_float( ((Float)obj).floatValue() );			 break;
		case TCKind.tk_double_value:	 write_double( ((Double)obj).doubleValue() );		 break;
		case TCKind.tk_boolean_value:	 write_boolean( ((Boolean)obj).booleanValue() );	 break;
		case TCKind.tk_char_value:		 write_ascii( ((Ascii)obj).ascii );					 break;
		case TCKind.tk_octet_value:		 write_octet( ((Byte)obj).byteValue() );			 break;
  		case TCKind.tk_any_value:		 write_any(obj);									 break;
//    		case TCKind.tk_any_value:		 /* any in an any is a bug */todo();				 break;
		case TCKind.tk_TypeCode_value:	 write_typecode(obj);						         break;
		case TCKind.tk_Principal_value:	 /* what's this */	todo();							 break;
		case TCKind.tk_objref_value:	 write_objref( obj, zInterface );					 break;
		case TCKind.tk_struct_value:	 write_struct(obj);									 break;
		case TCKind.tk_union_value:		 write_union( (CorbaUnion)obj );					 break;
//  		case TCKind.tk_union_value:		 write_union( (Union)obj );							 break;
		case TCKind.tk_enum_value:		 write_enum( (Enum)obj );							 break;
		case TCKind.tk_string_value:	 write_asciistring( ((AsciiString)obj).asciistring );break;
		case TCKind.tk_sequence_value:	 write_sequence( obj );							     break;
		case TCKind.tk_array_value:
		case TCKind.tk_alias_value:		 todo();											 break;
		case TCKind.tk_except_value:	 write_exception( (Exception)obj );					 break;
		case TCKind.tk_longlong_value:	 write_longlong( ((Long)obj).longValue() );			 break;
		case TCKind.tk_ulonglong_value:	 write_longlong( ((Long)obj).longValue() );			 break;
		case TCKind.tk_longdouble_value: todo();											 break;
		case TCKind.tk_wchar_value:		 write_unicode( ((Character)obj).charValue() );		 break;
		case TCKind.tk_wstring_value:	 write_unicodestring( (String)obj );				 break;
		default:						 todo();
        }
	}
	
	private void write_any(Object obj) throws Exception {
		Class zClass = null;
		int nIIOPTypeKind = 0;

		if(obj != null && obj.getClass().equals(Object.class)) // send java.lang.Object as void
			obj = null;

		if(obj == null) {
			nIIOPTypeKind = TCKind.tk_void_value;

			write_typecode(nIIOPTypeKind);
		}
		else {
			if(obj instanceof Any) {
				Any any = (Any)obj;
				zClass = any.getInterface();
				obj = any.getObject();
			}
			else {
				zClass = Unmarshal.anyClassToBaseClass(obj.getClass(), obj);
			}

			nIIOPTypeKind = classToIIOPTypeKind(zClass);

			if(nIIOPTypeKind == TCKind.tk_objref_value && !zClass.isInterface())
				zClass = XInterface.class;

			nIIOPTypeKind = write_typecode(zClass);
		}

		writeAnyValue(obj, nIIOPTypeKind, zClass);
    }

	private void write_enum( Enum enum ) throws Exception {
		int nValue = enum.getValue();
		write_long( nValue );
	}

	private void writeField(Field aField, Object Struct) throws Exception {
		Class FieldType = aField.getType();
		int nIIOPTypeKind = classToIIOPTypeKind( FieldType );
		switch( nIIOPTypeKind )	{
		case TCKind.tk_null_value:
		case TCKind.tk_void_value:       /* error */	todo();		                                break;
		case TCKind.tk_short_value:		 write_short( aField.getShort( Struct ) );					break;
		case TCKind.tk_long_value:		 write_long( aField.getInt( Struct ) );						break;
		case TCKind.tk_ushort_value:	 write_short( aField.getShort( Struct ) );					break;
		case TCKind.tk_ulong_value:		 write_long( aField.getInt( Struct ) );						break;
		case TCKind.tk_float_value:		 write_float( aField.getFloat( Struct ) );					break;
		case TCKind.tk_double_value:	 write_double( aField.getDouble( Struct ) );				break;
		case TCKind.tk_boolean_value:	 write_boolean( aField.getBoolean( Struct) );				break;
		case TCKind.tk_char_value:	{
			Object obj = aField.get( Struct );
			if( obj == null )
				write_ascii( '\0' );
			else
				write_ascii( ((Ascii)obj).ascii );
		}
		break;
		
		case TCKind.tk_octet_value:		 write_octet( aField.getByte( Struct) );				   break;
		case TCKind.tk_any_value:		 write_any( aField.get( Struct) );						   break;
		case TCKind.tk_TypeCode_value:	 write_typecode((Type)aField.get(Struct));				   break;
		case TCKind.tk_Principal_value:	 todo();												   break;
		case TCKind.tk_objref_value:	 write_objref( aField.get( Struct ), aField.getType() );   break;
		case TCKind.tk_struct_value:	 write_struct(aField.get(Struct));						   break;
		case TCKind.tk_union_value:		 write_union( (CorbaUnion)aField.get( Struct ) );			   break;
//  		case TCKind.tk_union_value:		 write_union( (Union)aField.get( Struct ) );			   break;
		case TCKind.tk_enum_value:	     write_enum( (Enum)aField.get( Struct ) );				   break;
		case TCKind.tk_string_value: {
			Object obj = aField.get( Struct );
			if( obj == null )
				write_asciistring( "" );
			else
				write_asciistring( ((AsciiString)obj).asciistring );
		}
		break;

		case TCKind.tk_sequence_value:	 write_sequence( aField.get( Struct ) );				   break;
		case TCKind.tk_array_value:		 todo();												   break;
		case TCKind.tk_alias_value:	     todo();												   break;
		case TCKind.tk_except_value:	 write_exception( (Exception)aField.get( Struct ) );	   break;
		case TCKind.tk_longlong_value:	 write_longlong( aField.getLong( Struct) );				   break;
		case TCKind.tk_ulonglong_value:	 write_longlong( aField.getLong( Struct) );				   break;
		case TCKind.tk_longdouble_value: todo();												   break;
		case TCKind.tk_wchar_value:	     write_unicode( aField.getChar( Struct ) );				   break;
		case TCKind.tk_wstring_value:	 write_unicodestring( (String)aField.get( Struct ) );	   break;
		default:	todo();
		}
	}
	
	private void writeStruct(Class StructClass, Class UntilClass, Object Struct) throws Exception {
		if(StructClass == CorbaString8.class) { // special treatment for CorbaString8
			write_asciistring(((CorbaString8)Struct).theString);
		}
		else {
			Class Superclass = StructClass.getSuperclass();
			// superclass must be filled before subclass
			if( Superclass != UntilClass )
				writeStruct(Superclass, UntilClass, Struct);

			// The precondition is, that the fields occure in the same order as 
			// specified in the IDL language, but this is not specified in Java.
			Field [] Fields = Protocol.__getDeclaredFields(StructClass);
			int nFieldsLength = Fields.length;
			for( int i = 0; i < nFieldsLength; i++ ) {
				Field aField = Fields[i];
				
				if( (aField.getModifiers() & (Modifier.STATIC | Modifier.TRANSIENT)) == 0 ) { // not static or transient ?
					MemberTypeInfo memberTypeInfo = Protocol.__findMemberTypeInfo(StructClass, aField.getName());
					if(memberTypeInfo != null && memberTypeInfo.isAny()) // is the member an any ?
						write_any(aField.get(Struct));
//  						write_any(Struct);
					else if(memberTypeInfo != null && memberTypeInfo.isInterface()) { // is interface ?
						Class zInterface = aField.getType();
						if(zInterface == Object.class)
							zInterface = com.sun.star.uno.XInterface.class;
						write_objref(aField.get(Struct), zInterface);
					}
					else
						writeField( aField, Struct );
				}
			}
		}
	}

	private void write_struct(Object Struct) throws Exception	{
		Class zClass = Struct.getClass();

		if(zClass == Any.class) {
			Any any = (Any)Struct;

			writeObject(any.getInterface(), any.getObject());
		}
		else {
			// write struct to the stream
			writeStruct(Struct.getClass(), Object.class, Struct);
		}
	}

	private void write_exception(Exception exc) throws Exception	{
		if(DEBUG) System.err.println("Marshal.write_exception: " + exc + " " + exc.getClass().getName());

		// type Name of the exception
		write_asciistring(exc.getClass().getName());

		// write the message of the exception
		write_unicodestring( exc.getMessage() );

		// write struct to the stream
		if(exc instanceof java.lang.Exception)
			writeStruct(exc.getClass(), Exception.class, exc);
		else
			writeStruct(exc.getClass(), RuntimeException.class, exc);
	}

	private void write_sequence( Object Sequence ) throws Exception
	{
	    if( null == Sequence ) {
			write_long( 0 );
			return;
		}
		
		Class ComponentType = Sequence.getClass().getComponentType();
		int nIIOPTypeKind = classToIIOPTypeKind( ComponentType );
		        
		switch( nIIOPTypeKind )	{
		case TCKind.tk_null_value:
		case TCKind.tk_void_value: /* error */	todo();	break;
		case TCKind.tk_short_value:   write_short_array( (short[])Sequence );	  break;
		case TCKind.tk_long_value:	  write_long_array( (int[])Sequence );		  break;
		case TCKind.tk_ushort_value:  write_short_array( (short[])Sequence );	  break;
		case TCKind.tk_ulong_value:	  write_long_array( (int[])Sequence );		  break;
		case TCKind.tk_float_value:	  write_float_array( (float[])Sequence );	  break;
		case TCKind.tk_double_value:  write_double_array( (double[])Sequence );	  break;
		case TCKind.tk_boolean_value: write_boolean_array( (boolean[])Sequence ); break;
		case TCKind.tk_char_value:  {
			Ascii[] szAscii = (Ascii[])Sequence;
			int nLen = szAscii.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ ) {
				if( szAscii[i] == null )
					write_ascii( '\0' );
				else
					write_ascii( szAscii[i].ascii );
			}
		}
		break;
				
		case TCKind.tk_octet_value:	write_octet_array( (byte[])Sequence );	break;
		case TCKind.tk_any_value: {
			Object[] szAny = (Object[])Sequence;
			int nLen = szAny.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_any( szAny[i] );
		}
		break;

		case TCKind.tk_TypeCode_value:  {
			Type[] szType = (Type[])Sequence;
			int nLen = szType.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_typecode( szType[i].getDescription() );
		}
		break;

		case TCKind.tk_Principal_value:	todo();	break;
		case TCKind.tk_objref_value: {
			Object[] szObject = (Object[])Sequence;
			int nLen = szObject.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_objref( szObject[i], Sequence.getClass().getComponentType());
		}
		break;

		case TCKind.tk_struct_value: {
			Object[] szObject = (Object[])Sequence;
			int nLen = szObject.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_struct(szObject[i]);
		}
		break;
			
		case TCKind.tk_union_value:  {
			CorbaUnion[] szUnion = (CorbaUnion[])Sequence;
			int nLen = szUnion.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_union( szUnion[i] );
		}
		break;

		case TCKind.tk_enum_value:  {
			Enum[] szEnum = (Enum[])Sequence;
			int nLen = szEnum.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_enum( szEnum[i] );
		}
		break;

		case TCKind.tk_string_value:  {
			AsciiString[] szAsciiString = (AsciiString[])Sequence;
			int nLen = szAsciiString.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ ){
				if( szAsciiString[i] == null )
					write_asciistring( "" );
				else
					write_asciistring( szAsciiString[i].asciistring );
			}
		}
		break;

		case TCKind.tk_sequence_value: {
			Object[] szSequence = (Object[])Sequence;
			int nLen = szSequence.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_sequence( szSequence[i] );
		}
		break;

		case TCKind.tk_array_value:	todo();	break;
		case TCKind.tk_alias_value:	todo();	break;
		case TCKind.tk_except_value:  {
			Exception[] szException = (Exception[])Sequence;
			int nLen = szException.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_exception( szException[i] );
		}
		break;

		case TCKind.tk_longlong_value: write_longlong_array( (long[])Sequence ); break;
		case TCKind.tk_ulonglong_value:	write_longlong_array( (long[])Sequence );break;
		case TCKind.tk_longdouble_value:todo();	break;
		case TCKind.tk_wchar_value:	write_unicode_array( (char[])Sequence );	break;
		case TCKind.tk_wstring_value: {
			String[] szString = (String[])Sequence;
			int nLen = szString.length;
			write_long( nLen );
			for( int i = 0; i < nLen ; i++ )
				write_unicodestring( szString[i] );
		}
		break;
			
		default: todo();
		}
	}

	private void write_objref(Object obj, Class zInterface) throws Exception {
		String sType = zInterface.getName();

		IOR ior = new IOR(new CorbaString8(sType), null);

		if(obj != null) {
			String oId = (String)bridge.mapInterfaceTo(obj, zInterface);
			
			if(DEBUG) System.err.println("##### Marshal.write_objref:" + obj + " " + zInterface + " oId:" + oId + " stype:" + sType);
			
			ObjectKey objectKey = new ObjectKey(new CorbaString8(oId), new CorbaString8(sType));
			Marshal tmpMarshal = new Marshal(littleEndian, bridge);
			tmpMarshal.writeObject(ObjectKey.class, objectKey);
			
			ProfileBody_1_1 profileBody = new ProfileBody_1_1(new com.sun.star.corba.iiop.Version((byte)1, (byte)2), 
															  new CorbaString8("unknown"), 
															  (short)0, 
															  tmpMarshal.reset(), 
															  new com.sun.star.corba.iop.TaggedComponent[0]);

			Marshal marshal = new Marshal(littleEndian, bridge);
			marshal.writeObject(ProfileBody_1_1.class, profileBody);
			
			ior.profiles = new TaggedProfile[]{new TaggedProfile(ProfileIdGroup.TAG_INTERNET_IOP, marshal.reset())};
		}
		else
			ior.profiles = new TaggedProfile[0];

  		writeObject(IOR.class, ior);
	}

	private void write_union(CorbaUnion aUnion) throws Exception {
		Class aUnionClass = aUnion.getClass();
		Field aDescriminatorField = aUnionClass.getField("nDiscriminator");
		
		int index = aDescriminatorField.getInt(aUnion);
		
		Field fields[] = Protocol.__getDeclaredFields(aUnionClass);
		
		writeField(aDescriminatorField, aUnion);
		writeField(fields[index + 1], aUnion);
	}
}



