/*************************************************************************
 *
 *  $RCSfile: Unmarshal.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.lib.uno.protocols.iiop;


import java.io.DataInputStream;
import java.io.IOException;

import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import com.sun.star.corba.CorbaString8;
import com.sun.star.corba.CorbaUnion;
import com.sun.star.corba.ObjectKey;
import com.sun.star.corba.TCKind;

import com.sun.star.uno.Any;
import com.sun.star.uno.Ascii;
import com.sun.star.uno.AsciiString;
import com.sun.star.uno.Enum;
import com.sun.star.uno.IBridge;
import com.sun.star.uno.Type;
import com.sun.star.uno.TypeClass;
import com.sun.star.uno.Union;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XInterface;

import com.sun.star.corba.iiop.ProfileBody_1_1;

import com.sun.star.corba.iop.IOR;


import com.sun.star.corba.giop.ReplyHeader_1_2;


import com.sun.star.lib.uno.environments.remote.IUnmarshal;
import com.sun.star.lib.uno.environments.remote.Protocol;


final class Unmarshal extends CDRInputStream implements IUnmarshal {
	public static boolean DEBUG = false;

	static class I2U {
		TypeClass _unoTypeClass;
		Class	  _javaClass;
		Class	  _anyClass;

		I2U(TypeClass unoTypeClass, Class javaClass, Class anyClass) {
			_unoTypeClass   = unoTypeClass;
			_javaClass      = javaClass;
			_anyClass       = anyClass;
		}
	}

//  		new I2U(TypeClass.UNSIGNED_SHORT.getValue(), Short.TYPE,        Short.class,    null,),	    //ushort -> ushort, 4 -> 5
//  		new I2U(TypeClass.UNSIGNED_LONG.getValue(),  Integer.TYPE,      Integer.class),     //ulong -> ulong, 5 -> 7
//  		new I2U(TypeClass.UNSIGNED_HYPER.getValue(), Long.TYPE,         Long.class),		//ulonglong -> uhyper, 24 -> 9
	final static I2U IIOP2UNOTypeClass[] = {
		new I2U(TypeClass.VOID,           void.class,        null),                       //null -> void, 0 ->0
		new I2U(TypeClass.VOID,           void.class,        null),                       //void -> void, 1 -> 0
		new I2U(TypeClass.SHORT,          Short.TYPE,        Short.class),		                //short -> short, 2 -> 4
		new I2U(TypeClass.LONG,           Integer.TYPE,      Integer.class),		                //long -> long, 3 -> 6
		new I2U(TypeClass.UNSIGNED_SHORT, null,              null),	                    // ushort, 4 -> 5, not supported
		new I2U(TypeClass.UNSIGNED_LONG,  null,              null),                       //ulong -> ulong, 5 -> 7, not supported
		new I2U(TypeClass.FLOAT,          Float.TYPE,        Float.class),		                //float -> float, 6 -> 10
		new I2U(TypeClass.DOUBLE,         Double.TYPE,       Double.class),	                    //double -> double, 7 -> 11
		new I2U(TypeClass.BOOLEAN,        Boolean.TYPE,      Boolean.class),                       //bool -> bool, 8 -> 2
		new I2U(TypeClass.UNKNOWN,        Ascii.class,       null),	                    //ascii -> ???, 9 -> 27
		new I2U(TypeClass.BYTE,           Byte.TYPE,         Byte.class),		                //octet -> byte, 10 -> 3
		new I2U(TypeClass.ANY,            null,              null),	//any -> any, 11 -> 14
		new I2U(TypeClass.TYPE,           Type.class,        Type.class),	//TypeCode -> type, 12 -> 13
		new I2U(TypeClass.UNKNOWN,        null,              null),			            //Principal -> ???, 13 -> 27
		new I2U(TypeClass.INTERFACE,      null,              null),		                //objref -> interface, 14 -> 22
		new I2U(TypeClass.STRUCT,         null,              null),			            //struct -> struct, 15 -> 17
		new I2U(TypeClass.UNION,          null,              null),			            //union -> union, 16 -> 18
		new I2U(TypeClass.ENUM,           null,              null),				        //enum -> enum, 17 -> 15
		new I2U(TypeClass.UNKNOWN,        AsciiString.class, AsciiString.class),	                    //asciistring -> ???, 18 -> 27
		new I2U(TypeClass.SEQUENCE,       null,              null),			            //sequence -> sequence, 19 -> 20
		new I2U(TypeClass.ARRAY,          null,              null),			            //array -> array, 20 -> 21
		new I2U(TypeClass.TYPEDEF,        null,              null),			            //alias -> typedef, 21 -> 16
		new I2U(TypeClass.EXCEPTION,      null,              null),		                //exception -> exception, 22 -> 19
		new I2U(TypeClass.HYPER,          Long.TYPE,         Long.class),		                //longlong -> hyper, 23 -> 8
		new I2U(TypeClass.UNSIGNED_HYPER, null,              null),		                //ulonglong -> uhyper, 24 -> 9, not supported
		new I2U(TypeClass.UNKNOWN,        null,              null),			            //longdouble -> ???, 25 -> 27
		new I2U(TypeClass.CHAR,           Character.TYPE,    Character.class),	                    //unicode -> unicode, 26 -> 1
		new I2U(TypeClass.STRING,         String.class,      String.class),	    //unicodestring-> unicodestring, 27 -> 12
		new I2U(TypeClass.UNKNOWN,        null,              null),			            //fixed -> ???, 28 -> 27
		new I2U(TypeClass.UNKNOWN,        null,              null),			            //value -> ???, 29 -> 27
		new I2U(TypeClass.UNKNOWN,        null,              null),			            //value_box -> ???, 30 -> 27
		new I2U(TypeClass.UNKNOWN,        null,              null),			            //native -> ???, 31 -> 27
		new I2U(TypeClass.UNKNOWN,        null,              null)			            //abstract_interface -> ???, 32 -> 27
	};

	// The last type kind read with the read_typeclass method. Only usefull for spead up.
	private int LastIIOPTypeKind;

	private void todo() throws Exception {
		throw new Exception(getClass().getName() + " - todo: not implemented");
	}

	protected IBridge _bridge;

	Unmarshal(byte [] Buffer, int size, boolean littleEndian, IBridge bridge)	{
		super(Buffer, size, littleEndian);
		_bridge = bridge;
	}

	/** Return the proper type kind of a class 
	 */
	static int classToIIOPTypeKind(Class JavaClass){
	    int nLen = IIOP2UNOTypeClass.length;
		int result = TCKind.tk_struct_value;

	    //strat with 0, so no unsigned types occure
		int i;
		for(i = 0; i != nLen; ++i)
			if( IIOP2UNOTypeClass[i]._javaClass == JavaClass ) {
				result = i;
				break;
			}

		if(i >= nLen) {
			// workaround for: isAssignableFrom bug with jdk blackdown 118 under linux
			Protocol.__getDeclaredFields(JavaClass);

			/*if(JavaClass == com.sun.star.corba.CorbaString8.class)  //////////////////////////////////////////////////
				result = TCKind.tk_string.getValue();
				else */ if( JavaClass.isArray() )
				result = TCKind.tk_sequence.getValue();
			else if( Exception.class.isAssignableFrom( JavaClass ) )
				result = TCKind.tk_except.getValue();
			else if( Enum.class.isAssignableFrom( JavaClass ) )
				result = TCKind.tk_enum.getValue();
			else if( Union.class.isAssignableFrom( JavaClass ) )
				result = TCKind.tk_union.getValue();
			// this is (like write_union) only temporaer
			else if( com.sun.star.corba.CorbaUnion.class.isAssignableFrom( JavaClass ) )
				result = TCKind.tk_union.getValue();
			// any ?
			else if(Object.class == JavaClass
                 || Number.class.isAssignableFrom(JavaClass)
                 || Character.class == JavaClass
                 || Boolean.class == JavaClass) {
				result =  TCKind.tk_any.getValue();
			}
			else if(XInterface.class.isAssignableFrom(JavaClass))
					result = TCKind.tk_objref.getValue();
		}

  		if(DEBUG) System.err.println("##### Unmarshal.classToIIOPTypeKind:" + JavaClass + " " + result);

		return result;
	}

	/** Map from any type to the real type
	 */
	static Class anyClassToBaseClass(Class AnyClass, Object object) {
	    int nLen = IIOP2UNOTypeClass.length;
	    //strat with 0, so no unsigned types occure
		for( int i = 0; i != nLen; i++ )
			if( IIOP2UNOTypeClass[i]._anyClass == AnyClass )
				return IIOP2UNOTypeClass[i]._javaClass;

		return AnyClass;
	}
	
	/*****
	 * unpacks a discrimnative union. (see corba specification)
	 * The discriminative value ( byte or long ) and one value is unpacked,
	 * the unused values are default constructed.
	 *****/
	/*
	private Object unpackUnion( Class pUnionType )
	{

		sal_Int8 *pDest = ( sal_Int8 * ) pVoidDest;

		int nPosToMarshal = 0;
		typelib_TypeDescription *pType = 0;
		TYPELIB_DANGER_GET( &pType , pCompType->ppTypeRefs[0] );
		pUnmarshal->unpack( pDest + pCompType->pMemberOffsets[0] , pType );

		if( typelib_TypeClass_BYTE == pType->eTypeClass ||
			typelib_TypeClass_BOOLEAN == pType->eTypeClass )
		{
			// retrieve the position to marshal
			nPosToMarshal =  (int ) *(sal_Int8 * ) (pDest + pCompType->pMemberOffsets[0]);
		}
		else if( typelib_TypeClass_SHORT    == pType->eTypeClass ||
				 typelib_TypeClass_UNSIGNED_SHORT    == pType->eTypeClass )
		{
			// retrieve the position to marshal
			nPosToMarshal =  (int ) *(sal_Int16 * ) (pDest + pCompType->pMemberOffsets[0]);
		}
		else if( typelib_TypeClass_LONG    == pType->eTypeClass ||
				 typelib_TypeClass_ENUM    == pType->eTypeClass )
		{
			// retrieve the position to marshal
			nPosToMarshal =  (int ) *(int * ) (pDest + pCompType->pMemberOffsets[0]);
		}
		else {
			assert( 0 );
		}
		TYPELIB_DANGER_RELEASE( pType );

		int i;
		for( i = 1 ; i < pCompType->nMembers ; i ++ )
		{
			pType = 0;
			TYPELIB_DANGER_GET( &pType , pCompType->ppTypeRefs[i] );
			if( 1 + nPosToMarshal == i ) {
				pUnmarshal->unpack( pDest + pCompType->pMemberOffsets[i], pType );
			}
			else {
				uno_constructData( pDest + pCompType->pMemberOffsets[i] , pType );
			}
			TYPELIB_DANGER_RELEASE( pType );
		} 
	}
	*/

	private Class read_typeclass() throws Exception {
		return read_type().getDescription();
	}

	private Type read_type() throws Exception {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".read_typeclass");

		//Class return = null;
		String TypeName = null;
		int kind = read_long();

		int nCount = 0;
		int nBound = 0;

		// java optimize the switch statement to a jump table, only if
		// the case values are subsequent.
		switch(kind) {
			case TCKind.tk_TypeCode_value:
				break;
			case TCKind.tk_Principal_value:
				todo();
				break;

			case TCKind.tk_objref_value:
				TypeName = read_asciistring();
				// dummy
				read_asciistring();
				break;

			case TCKind.tk_except_value:
			case TCKind.tk_struct_value:
				TypeName = read_asciistring();
				// dummy
				read_asciistring();
				nCount = read_long();
				for( int i = 0; i < nCount; i++ )
				{
					// member name
					read_asciistring();
					read_typeclass();
				}
				break;

			case TCKind.tk_union_value:
				TypeName = read_asciistring();
				// dummy
				read_asciistring();

				// Descriminator type code
				Class Descreminator = read_typeclass();
				// index of default
				int nDefault = read_long();
				// count of members
				nCount = read_long();
				// add members of the unions
				for( int i = 0; i < nCount; i++ )
				{
					// value of descriminator
					todo();
					// member name
					read_asciistring();
					//typecode of the member
					read_typeclass();
				}
				break;

			case TCKind.tk_enum_value:
				TypeName = read_asciistring();
				// dummy
				read_asciistring();
				// all enum names
				nCount = read_long();
				for( int i = 0; i < nCount; i++ )
				{
					// member name
					read_asciistring();
				}
				break;

			case TCKind.tk_string_value:
				// read the bounds of an string
				nBound = read_long();
				break;

			case TCKind.tk_sequence_value:
//  				Class ElementTypeCode = read_typeclass();
//  				if( ElementTypeCode != null )
//  				{
//  					TypeName = "]" + ElementTypeCode.getName();
//  					nBound = read_long();
//  					if( nBound != 0 )
//  						todo();
//  				}

				Type elementType = read_type();
//  				if(ElementTypeCode.isPrimitive())
//  					TypeName = "[" + ElementTypeCode.getName
//  				TypeName = "L" + ElementTypeCode.getName();
				TypeName = "[]" + elementType.getTypeName();
//  				TypeName = elementType.getArrayTypeName();

				nBound = read_long();
				if( nBound != 0 )
					todo();

				break;

			case TCKind.tk_array_value:
			case TCKind.tk_alias_value:
			case TCKind.tk_longdouble_value:
				break;

			case TCKind.tk_wstring_value:
				// read the bounds of an string
				nBound = read_long();
				break;
		}
		LastIIOPTypeKind = kind;

		if(DEBUG) System.err.println("##### " + getClass().getName() + " - read_typeclass - kind:" + kind);

		Type result = new Type(IIOP2UNOTypeClass[kind]._unoTypeClass, TypeName, IIOP2UNOTypeClass[kind]._javaClass);

		return result;
	}

	public Object readObject(Class zClass) throws Exception {
		if(zClass == Any.class)
			return read_any();

		int kind = classToIIOPTypeKind(zClass);

		if(DEBUG) System.err.println("##### " + getClass().getName() + ".readObject - index:" + index + " " + zClass + " " + kind);

		Object object = null;

		if(zClass == void.class)
			return null;

		switch(kind) {
		case TCKind.tk_null_value:
		case TCKind.tk_void_value:	     todo(); /* error */	object = null;	       break;
		case TCKind.tk_short_value:	     object = new Short(read_short());			   break;
		case TCKind.tk_long_value: 	     object = new Integer(read_long());			   break;
		case TCKind.tk_ushort_value:     object = new Short(read_short());			   break;
		case TCKind.tk_ulong_value:      object = new Integer(read_long());			   break;
		case TCKind.tk_float_value: 	 object = new Float(read_float());			   break;
		case TCKind.tk_double_value:     object = new Double(read_double());		   break;
		case TCKind.tk_boolean_value:    object = new Boolean(read_boolean());		   break;
		case TCKind.tk_char_value:	     object = new Ascii(read_ascii());			   break;
		case TCKind.tk_octet_value:	     object = new Byte(read_octet());			   break;
		case TCKind.tk_any_value:	     object = read_any();						   break;
		case TCKind.tk_TypeCode_value:	 object = read_type();					   break;
//  		case TCKind.tk_TypeCode_value:	 object = read_typeclass();					   break;
		case TCKind.tk_Principal_value:	 object = null;								   break;
		case TCKind.tk_objref_value:     object = read_objref(zClass);				   break;
		case TCKind.tk_struct_value:     object = read_struct(zClass);				   break;
		case TCKind.tk_union_value: 	 object = read_union(zClass);				   break;
		case TCKind.tk_enum_value:	     object = read_enum(zClass);				   break;
		case TCKind.tk_string_value:	 object = new AsciiString(read_asciistring()); break;
		case TCKind.tk_sequence_value:	 object = read_sequence(zClass);			   break;
		case TCKind.tk_array_value:
		case TCKind.tk_alias_value:	     todo();	object = null;					   break;
		case TCKind.tk_except_value:     object = read_exception(zClass);			   break;
		case TCKind.tk_longlong_value:	 object = new Long(read_longlong());		   break;
		case TCKind.tk_ulonglong_value:	 object = new Long(read_longlong());		   break;
		case TCKind.tk_longdouble_value: todo(); object = null;						   break;
		case TCKind.tk_wchar_value: 	 object = new Character(read_unicode());	   break;
		case TCKind.tk_wstring_value:	 object = read_unicodestring();				   break;
		default:  object = null; 
        }

		return object;
	}

	private Object read_any() throws Exception	{
		// Type is acquired with typelib_typedescription_acquire
		Class Type = read_typeclass();

		if(DEBUG) System.err.println("##### Unmarshal.read_any:" +  index + " " + Type);

		// Hack, use the object variable LastIIOPTypeKind of the last read_typeclass method
		switch( LastIIOPTypeKind )  {
		case TCKind.tk_null_value:
		case TCKind.tk_void_value: return null;
		case TCKind.tk_short_value:	return new Short( read_short() );
		case TCKind.tk_long_value:	return new Integer( read_long() );
		case TCKind.tk_ushort_value: return new Short( read_short() );
		case TCKind.tk_ulong_value:	return new Integer( read_long() );
		case TCKind.tk_float_value:	return new Float( read_float() );
		case TCKind.tk_double_value:return new Double( read_double() );
		case TCKind.tk_boolean_value:return new Boolean( read_boolean() );
		case TCKind.tk_char_value:return new Ascii( read_ascii() );
		case TCKind.tk_octet_value:	return new Byte( read_octet() );
		case TCKind.tk_any_value:return read_any();
		case TCKind.tk_TypeCode_value:	return read_typeclass();
		case TCKind.tk_Principal_value:	return null;
		case TCKind.tk_objref_value:return read_objref( Type );
		case TCKind.tk_struct_value:return read_struct( Type );
		case TCKind.tk_union_value:	return read_union( Type );
		case TCKind.tk_enum_value:	return read_enum( Type );
		case TCKind.tk_string_value:	return new AsciiString( read_asciistring() );
		case TCKind.tk_sequence_value:	return read_sequence( Type );
		case TCKind.tk_array_value:
		case TCKind.tk_alias_value:	todo();	return null;
		case TCKind.tk_except_value:return read_exception( Type );
		case TCKind.tk_longlong_value:	return new Long( read_longlong() );
		case TCKind.tk_ulonglong_value:	return new Long( read_longlong() );
		case TCKind.tk_longdouble_value:todo();	return null;
		case TCKind.tk_wchar_value:	return new Character( read_unicode() );
		case TCKind.tk_wstring_value:return read_unicodestring();
		default:return null;
        }
	}

	private Enum read_enum( Class EnumClass ) throws Exception {
		Class [] ParamTypes = { Integer.TYPE };
		Method pMeth = EnumClass.getMethod( "fromInt", ParamTypes );
		
		int nValue = read_long();
		Object [] Params = { new Integer( nValue ) };
		return (Enum)pMeth.invoke( null, Params );
	}

	private void fillField(Field aField, Object Struct) throws Exception {
    	Class FieldType = aField.getType();
		com.sun.star.lib.uno.typeinfo.TypeInfo typeInfo = Protocol.__findMemberTypeInfo(aField.getDeclaringClass(), aField.getName());
    	int nIIOPTypeKind = -1;

		if(typeInfo != null && typeInfo.isInterface()) {
			nIIOPTypeKind = TCKind.tk_objref_value;

			Protocol.__getDeclaredFields(FieldType);
			if(!XInterface.class.isAssignableFrom(FieldType))
				FieldType = XInterface.class;
		}

		if(nIIOPTypeKind == -1)
			nIIOPTypeKind = classToIIOPTypeKind( FieldType );

			
    	switch( nIIOPTypeKind ) {
		case TCKind.tk_null_value:
		case TCKind.tk_void_value:	break;
		case TCKind.tk_short_value:	        aField.setShort( Struct, read_short() );                     break;
		case TCKind.tk_long_value:			aField.setInt( Struct, read_long() );						 break;
		case TCKind.tk_ushort_value:		aField.setShort( Struct, read_short() );					 break;
		case TCKind.tk_ulong_value:			aField.setInt( Struct, read_long() );						 break;
		case TCKind.tk_float_value:			aField.setFloat( Struct, read_float() );					 break;
		case TCKind.tk_double_value:		aField.setDouble( Struct, read_double() );					 break;
		case TCKind.tk_boolean_value:		aField.setBoolean( Struct, read_boolean() );				 break;
		case TCKind.tk_char_value:			aField.set( Struct, new Ascii( read_ascii() ) );			 break;
		case TCKind.tk_octet_value:			aField.setByte( Struct, read_octet() );						 break;
		case TCKind.tk_any_value:			aField.set( Struct, read_any() );							 break;
		case TCKind.tk_TypeCode_value:		aField.set( Struct, read_type() );						 break;
		case TCKind.tk_Principal_value:		todo();														 break;
		case TCKind.tk_objref_value:		aField.set( Struct, read_objref( FieldType ) );				 break;
		case TCKind.tk_struct_value:		aField.set( Struct, read_struct( FieldType ) );				 break;
		case TCKind.tk_union_value:			aField.set( Struct, read_union( FieldType ) );				 break;
		case TCKind.tk_enum_value:			aField.set( Struct, read_enum( FieldType ) );				 break;
		case TCKind.tk_string_value:		aField.set( Struct, new AsciiString( read_asciistring() ) ); break;
		case TCKind.tk_sequence_value:		aField.set( Struct, read_sequence( FieldType ) );		     break;
		case TCKind.tk_array_value:			todo();														 break;
		case TCKind.tk_alias_value:			todo();														 break;
		case TCKind.tk_except_value:		aField.set( Struct, read_exception( FieldType ) );			 break;
		case TCKind.tk_longlong_value:		aField.setLong( Struct, read_longlong() );					 break;
		case TCKind.tk_ulonglong_value:		aField.setLong( Struct, read_longlong() );					 break;
		case TCKind.tk_longdouble_value:	todo();														 break;
		case TCKind.tk_wchar_value:		    aField.setChar( Struct, read_unicode() );					 break;
		case TCKind.tk_wstring_value:		aField.set( Struct, read_unicodestring() );					 break;
		default:todo();
    	}
	}
			
	private void fillStruct(Class StructClass, Class UntilClass, Object Struct) throws Exception {
		Class Superclass = StructClass.getSuperclass();
		// superclass must be filled before subclass
		if( Superclass != UntilClass )
			fillStruct( Superclass, UntilClass, Struct );

		// The precondition is, that the fields occure in the same order as 
		// specified in the IDL language, but this is not specified in Java.
		Field [] Fields = Protocol.__getDeclaredFields(StructClass);
		int nFieldsLength = Fields.length;
		for( int i = 0; i < nFieldsLength; i++ ) {
			Field aField = Fields[i];
			if( (aField.getModifiers() & (Modifier.STATIC | Modifier.TRANSIENT)) == 0 )
				// not static or transient
				fillField( aField, Struct );
		}
	}

	private Object read_struct(Class StructClass) throws Exception {
		if(DEBUG) System.err.println("##### " + getClass().getName() + ".read_struct:" + StructClass);

		Object object = null;

		if(StructClass == CorbaString8.class) { // special treatement for CorbaString8
			String string = read_asciistring();
			if(DEBUG) System.err.println("##### Unmarshal.read_struct - CorbaString8:" + string);
			object = new CorbaString8(string);
		}
		else {
			Object Struct = StructClass.newInstance();
			// fill the struct from the stream
			fillStruct( StructClass, Object.class, Struct );
			object = Struct;
		}

		return object;
	}

  	Exception read_exception( Class ExceptionClass ) throws Exception {
		// type Name of the exception
		//String ExceptionName = read_unicodestring();
		//Class ExceptionClass = Class.forName( ExceptionName ); 
		// read the message of the exception
		String typeName = read_asciistring();
		ExceptionClass = Class.forName(typeName);

		String aMessage = read_unicodestring();
		
		Class [] ParamTypes = { String.class };
		Constructor Ctor = ExceptionClass.getConstructor( ParamTypes );
		Object [] Params = { aMessage };
		Exception Except = (Exception)Ctor.newInstance( Params );
		// fill the exception from the stream

		if(com.sun.star.uno.Exception.class.isAssignableFrom(ExceptionClass))
			fillStruct( ExceptionClass, java.lang.Exception.class, Except );
		else
			fillStruct( ExceptionClass, java.lang.RuntimeException.class, Except );

		return Except;
  	}

	private Object read_sequence( Class SequenceClass ) throws Exception {
		int nLen = -1;
		Class ComponentType = SequenceClass.getComponentType();
		if(ComponentType == Any.class)
			ComponentType = Object.class;

		int nIIOPTypeKind = classToIIOPTypeKind( ComponentType );
		
		switch( nIIOPTypeKind )	{
		case TCKind.tk_null_value:
		case TCKind.tk_void_value: /* error */	todo();	break;
		case TCKind.tk_short_value:	return read_short_array();
		case TCKind.tk_long_value:return read_long_array();
		case TCKind.tk_ushort_value:return read_short_array();
		case TCKind.tk_ulong_value:	return read_long_array();
		case TCKind.tk_float_value:	return read_float_array();
		case TCKind.tk_double_value:return read_double_array();
		case TCKind.tk_boolean_value:	return read_boolean_array();
		case TCKind.tk_char_value: {
			nLen = read_long();
			Ascii[] szAscii = new Ascii[nLen];
			for( int i = 0; i < nLen ; i++ )
				szAscii[i] = new Ascii( read_ascii() );
			return szAscii;
		}
		
		case TCKind.tk_octet_value:	return read_octet_array();
		case TCKind.tk_any_value: {
			nLen = read_long();
			Object[] szAny = new Object[nLen];
			for( int i = 0; i < nLen ; i++ )
				szAny[i] = read_any();
			return szAny;
		}
		
		case TCKind.tk_TypeCode_value: {
			nLen = read_long();
			Type[] szType = new Type[nLen];
			for( int i = 0; i < nLen ; i++ )
				szType[i] = read_type();
			return szType;
		}
		
		case TCKind.tk_Principal_value:	todo();	break;
		case TCKind.tk_objref_value: {
			nLen = read_long();
			Object [] szObject = (Object [])Array.newInstance( ComponentType, nLen );
			for( int i = 0; i < nLen ; i++ )
				szObject[i] = read_objref( ComponentType );
			return szObject;
		}
		
		case TCKind.tk_struct_value: {
			nLen = read_long();
			if(nLen > 16000000)
				nLen = 1;
			Object [] szStruct = (Object [])Array.newInstance( ComponentType, nLen );
//  			Object [] szStruct = (Object [])Array.newInstance( SequenceClass.getComponentType(), nLen );
			for( int i = 0; i < nLen ; i++ )
				szStruct[i] = readObject( ComponentType );
			return szStruct;
		}
		
		case TCKind.tk_union_value: {
			nLen = read_long();
			CorbaUnion[] szUnion = (CorbaUnion[])Array.newInstance( SequenceClass.getComponentType(), nLen );
			for( int i = 0; i < nLen ; i++ )
				szUnion[i] = read_union( ComponentType );
			return szUnion;
		}
		
		case TCKind.tk_enum_value: {
			nLen = read_long();
			Enum[] szEnum = (Enum[])Array.newInstance( SequenceClass.getComponentType(), nLen );
			for( int i = 0; i < nLen ; i++ )
				szEnum[i] = read_enum( ComponentType );
			return szEnum;
		}
		
		case TCKind.tk_string_value: {
			nLen = read_long();
			AsciiString[] szAsciiString = new AsciiString[nLen];
			for( int i = 0; i < nLen ; i++ )
				szAsciiString[i] = new AsciiString( read_asciistring() );
			return szAsciiString;
		}
		
		case TCKind.tk_sequence_value: {
			nLen = read_long();
			Object[] szSequence = (Object[])Array.newInstance( SequenceClass.getComponentType(), nLen );
			for( int i = 0; i < nLen ; i++ )
				szSequence[i] = read_sequence( ComponentType );
			return szSequence;
		}
		
		case TCKind.tk_array_value:todo();	break;
		case TCKind.tk_alias_value:todo();	break;
		case TCKind.tk_except_value: {
			nLen = read_long();
			Exception[] szException = (Exception[])Array.newInstance( SequenceClass.getComponentType(), nLen );
			for( int i = 0; i < nLen ; i++ )
				szException[i] = read_exception( ComponentType );
			return szException;
		}
		
		case TCKind.tk_longlong_value:return read_longlong_array();
		case TCKind.tk_ulonglong_value: return read_longlong_array();
		case TCKind.tk_longdouble_value:todo();	break;
		case TCKind.tk_wchar_value: return read_unicode_array();
		case TCKind.tk_wstring_value: {
			nLen = read_long();
			String[] szString = new String[ nLen ];
			for( int i = 0; i < nLen ; i++ )
				szString[i] = read_unicodestring();
			return szString;
		}
		
		default:todo();
		}
		
		return null;
	}

	private XInterface read_objref(Class zInterface) throws Exception {
		XInterface xInterface = null;

		IOR ior = (IOR)readObject(IOR.class);
		
		// is it the empty ref?
		if(ior.profiles.length != 0) {
			String sType = ior.type_id.theString;
			byte profileBody_bytes[] = ior.profiles[0].profile_data;

			Unmarshal unmarshal = new Unmarshal(profileBody_bytes, 
												profileBody_bytes.length,
												littleEndian, 
												_bridge);
			ProfileBody_1_1 profileBody = (ProfileBody_1_1)unmarshal.readObject(ProfileBody_1_1.class);
		
			byte key_bytes[] = profileBody.object_key;
			
			Unmarshal tmpUnmarshal = new Unmarshal(key_bytes, key_bytes.length, littleEndian, _bridge);
			ObjectKey objectKey = (ObjectKey)tmpUnmarshal.readObject(ObjectKey.class);
			
			xInterface = (XInterface)_bridge.mapInterfaceFrom(objectKey.sOid.theString, zInterface);
			
			if(DEBUG) System.err.println("#### Unmarshal.read_objref - oid:" + objectKey.sOid.theString + " stype:" + objectKey.sType.theString + " xInterface:" + xInterface);
		}

  		return xInterface;
	}

	private CorbaUnion read_union(Class aUnionClass) throws Exception {
		CorbaUnion union = (CorbaUnion)aUnionClass.newInstance();
		
		Field aDescriminatorField = aUnionClass.getDeclaredField("nDiscriminator");
		fillField(aDescriminatorField, union);
		
		int index = aDescriminatorField.getInt(union);
		
		Field fields[] = Protocol.__getDeclaredFields(aUnionClass);
		
		fillField(fields[index + 1], union);

    	return union;
	}
}

