/*************************************************************************
 *
 *  $RCSfile: UnoRuntime.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:27:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.uno;


import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import java.util.Enumeration;
import java.util.Hashtable;


import com.sun.star.lib.util.WeakTable;

import com.sun.star.uno.Uik;

/**
 * The <code>UnoRuntime</code> class is the central class needed for
 * implementing or using UNO components in Java.
 * <p>
 * The Methods <code>queryInterface</code> and <code>isSame</code> delegate
 * calls to the implementing objects and are used instead of casts
 * and identity comparisons.
 * <p>
 * The other methods are used for managing bridges, which enable a user to
 * map objects out to or into java.
 * <p>
 * @version 	$Revision: 1.1.1.1 $ $ $Date: 2000/09/18 15:27:54 $
 * @author 	    Kay Ramme
 * @see         com.sun.star.uno.IEnvironment
 * @see         com.sun.star.uno.IQueryInterface
 * @see         com.sun.star.uno.IBridge
 * @since       UDK1.0
 */
public class UnoRuntime {
	static public final boolean DEBUG = false;

	static protected WeakTable __environments = new WeakTable();
	static protected WeakTable __bridges      = new WeakTable();

	// debug interface begin
	static int getEnvironmentCount() {
		return __environments.size();
	}
	static int getBridgeCount() {
		return __bridges.size();
	}
	// debug interface end

	static String uid = null;
	static {
  		uid = (new java.rmi.server.UID()).toString();
	}

	/**
	 * Generates a worldwide unique object identifier for the given object.
	 * <p>
	 * @return     the generated Oid.
	 * @param      object     the object for which a Oid should be generated.
	 */
	static public synchronized String generateOid(Object object) {
  		String oId =  null;

		object = WeakTable.__getObject(object);

		if(object instanceof IQueryInterface)
			oId = ((IQueryInterface)object).getOid();
		if ( oId == null ) 
  			oId = uid + object.hashCode(); 

  		if(DEBUG) System.err.println("#### UnoRuntime.generateOid:" + oId);

		return oId;
	}
	
	/**
	 * Returns an object implementing the desired interface.
	 * <p>
	 * If the given by object already implements the desired interface,
	 * it is passed back immediatly. Otherwise it is checked if the given
	 * object implments the delegator interface <code>IQueryInterface</code> and
	 * the request is delegated to <code>queryInterface</code> of <code>IQueryInterface</code>.
	 * <p>
	 * @return     the desired interface if available, otherwise null.
	 * @param      zInterface the class of the desired interface.
	 * @param      object     the object to ask for it.
	 * @see        com.sun.star.uno.IQueryInterface
	 */
	static public Object queryInterface(Class zInterface, Object object) throws MappingException {
		object = WeakTable.__getObject(object);

		if (object instanceof IQueryInterface)   // If the object supports the IQueryInterface -> delegate
			object = ((IQueryInterface) object).queryInterface(zInterface);
		else if (!zInterface.isInstance(object)) // Alternatively test, if the object is an instance of zInterface
			object = null;
	    
		return object;
	}


	/** 
	 * Tests if the given objects are interfaces of the same underlying object.
	 * <p>
	 * @return     true, if the underlying object is the same, otherwise false.
	 * @param      object1 an object representing an interface.
	 * @param      object2 an object representing an interface.
	 */
	static public boolean areSame(Object object1, Object object2) throws MappingException {

		object1 = WeakTable.__getObject(object1);
		object2 = WeakTable.__getObject(object2);

		boolean identical = false;

		if(object1 == object2)
			identical = true;
		else if(object1 instanceof IQueryInterface)
			identical = ((IQueryInterface) object1).isSame(object2);
		else if(object2 instanceof IQueryInterface)
			identical = ((IQueryInterface) object2).isSame(object1);

		return identical;
	}

    /**
	 * Retrieves an environment of type <code>name</code> with context <code>context</code>.
	 * Environments are hold weak at the UnoRuntime. If the desired environment already
	 * exits, this methods gives it back.
	 * If the desired environement does not exist, this method looks for it under
	 * <code>com.sun.star.lib.uno.environments.&lt;name&gt;.&lt;name&gt;_environment</code>.
	 * <p>
     * @param      name      the name of the environment.
     * @param      context   the context of the environment.
     * @see        com.sun.star.uno.IEnvironment
     */
	static public synchronized IEnvironment getEnvironment(String name, Object context) throws java.lang.Exception {
		IEnvironment xEnvironment = (IEnvironment)__environments.get(name + context, IEnvironment.class);

		if(DEBUG) System.err.println("##### " + UnoRuntime.class.getName() + ".getEnvironment:" + name + " " + context + " got it:" + xEnvironment);
		if(xEnvironment == null) {
			String className = "com.sun.star.lib.uno.environments." + name + "." + name + "_environment"; 

  			Class zClass = Class.forName(className);
			Constructor constructor = zClass.getConstructor(new Class[]{Object.class});
			xEnvironment = (IEnvironment)constructor.newInstance(new Object[]{context});
 
			xEnvironment = (IEnvironment)__environments.put(name + context, xEnvironment, IEnvironment.class);
		}

		return  xEnvironment;
	}

    /**
	 * Gets a bridge from environment <code>from</code> to environment <code>to</code>.
	 * Creates a new bridge, if the desired bridge does not exist yet and handles the arguments
	 * to the bridge.
	 * <p>
	 * If the desired bridge does not exist, is it searched in <code>com.sun.star.lib.uno.bridges.&lt;from&gt;_&lt;to&gt;.&lt;</code> 
	 * and the root classpath as <code>from&gt;_&lt;to&gt;_bridge</code>.
	 * <p>
	 * @result     the desired bridge.
     * @param      from      the source environment.
     * @param      to        the target environment.
	 * @param      args      the initial arguments for the bridge
     * @see        com.sun.star.uno.IEnvironment
     * @see        com.sun.star.uno.IBridge
     * @see        com.sun.star.uno.UnoRuntime#getBridgeByName
     */
	static public synchronized IBridge getBridge(IEnvironment from, IEnvironment to, Object args[]) throws java.lang.Exception {
		String name = from.getName() + "_" + to.getName();
		String hashName = from.getName() + from.getContext() + "_" + to.getName() + to.getContext();
		
		IBridge bridge = (IBridge)__bridges.get(hashName, IBridge.class);
		if(bridge == null) {
			Class zClass = null;
			
			String className =  name + "_bridge";
			
			try {
				if(DEBUG) System.err.println("#### UnoRuntime.getBridge - looking for:" + className);
				zClass = Class.forName(className);
			}
			catch(ClassNotFoundException classNotFoundException) {
				className = "com.sun.star.lib.uno.bridges." + name + "." + className;

				if(DEBUG) System.err.println("#### UnoRuntime.getBridge - looking for:" + className);
				zClass = Class.forName(className);
			}
			
			Class signature[] = {IEnvironment.class, IEnvironment.class, args.getClass()};
			Constructor constructor = zClass.getConstructor(signature);
			Object iargs[] = {from, to, args};
			
			bridge = (IBridge)constructor.newInstance(iargs);
			bridge = (IBridge)__bridges.put(hashName, bridge, IBridge.class);
		}
		else
			if(DEBUG) System.err.println("#### UnoRuntime.getBridge - found existing bridge:" + bridge);

		return bridge;
	}

    /**
	 * Gets a bridge from environment <code>from</code> to environment <code>to</code>.
	 * Creates a new bridge, if the desired bridge does not exist yet and handles the arguments
	 * to the bridge.
	 * <p>
	 * If the desired bridge does not exist, is it searched in <code>com.sun.star.lib.uno.bridges.&lt;from&gt;_&lt;to&gt;.&lt;</code> 
	 * and the root classpath as <code>from&gt;_&lt;to&gt;_bridge</code>.
	 * The needed environments are retrieved through <code>UnoRuntime#getEnvironment</code>.
	 * <p>
	 * @result     the desired bridge.
     * @param      from         the name of the source environment.
	 * @param      from_context the context for the source environment
     * @param      to           the name of the target environment.
	 * @param      to_context   the context for the source environment
	 * @param      args         the initial arguments for the bridge
     * @see        com.sun.star.uno.IEnvironment
     * @see        com.sun.star.uno.IBridge
     * @see        com.sun.star.uno.UnoRuntime#getBridge
     * @see        com.sun.star.uno.UnoRuntime#getEnvironment
     */
	static public IBridge getBridgeByName(String from, Object from_context, String to, Object to_context, Object args[]) throws java.lang.Exception {
		return getBridge(getEnvironment(from, from_context), getEnvironment(to, to_context), args);
	}

    /**
	 * Gets the desired mapping from <code>from</code> to <code>to</code>.
	 * <p>
	 * Mappings are like bridges, except that with mappings one can only map in one direction.
	 * Mappings are here for <code>api</code> compatibility reasons the binary UNO runtime.
	 * Mappings are implemented as wrappers around bridges.
	 * <p>
	 * @result     the desired bridge.
     * @param      from         the source environment.
     * @param      to           the target environment.
     * @see        com.sun.star.uno.Mapping
     * @see        com.sun.star.uno.UnoRuntime#getMapping
     * @see        com.sun.star.uno.UnoRuntime#getEnvironment
     * @see        com.sun.star.uno.IEnvironment
     */
	static public Mapping getMapping(IEnvironment from, IEnvironment to) throws java.lang.Exception {
  		IBridge bridge = null;

  		try {
  			bridge = getBridge(from, to, null);
  		}
  		catch(ClassNotFoundException classNotFoundException) {
  			bridge = new BridgeTurner(getBridge(to, from, null));
  		}

  		return new MappingWrapper(bridge);
	}

    /**
	 * Gets the desired mapping from <code>from</code> to <code>to</code>.
	 * Retrieves the environments via <code>UnoRuntime#getEnvironment</code>.
	 * <p>
	 * @result     the desired bridge.
     * @param      from         the name of the source environment.
     * @param      to           the name of the target environment.
     * @see        com.sun.star.uno.Mapping
     * @see        com.sun.star.uno.UnoRuntime#getMapping
     * @see        com.sun.star.uno.UnoRuntime#getEnvironment
     */
	static public Mapping getMappingByName(String from, String to) throws java.lang.Exception {
		return getMapping(getEnvironment(from, null), getEnvironment(to, null));
	}

    /**
	 * Resets the UnoRuntime to its initial state.
	 * Releases all references to bridges and environments.
	 * <p>
     * @see        com.sun.star.uno.UnoRuntime#getBridge
     * @see        com.sun.star.uno.UnoRuntime#getBridgeByName
     * @see        com.sun.star.uno.UnoRuntime#getEnvironment
     * @see        com.sun.star.uno.UnoRuntime#getMapping
     * @see        com.sun.star.uno.UnoRuntime#getMappingByName
     */
	static public boolean reset() {
		if(DEBUG) System.err.println("UnoRuntime.dispose");

		if(!__bridges.isEmpty()) {
			System.err.println("\tdisposing bridges:");
			__bridges.reset();
		}

		System.err.println();

		if(!__environments.isEmpty()) {
			System.err.println("\tdisposing environments:");
			__environments.reset();
		}

		boolean passed = __environments.isEmpty() && __bridges.isEmpty();

		if(!passed) {
			System.err.println();
			
			System.err.println("\tenvironments destroyed - passed? " + __environments.isEmpty());
			System.err.println("\bridges destroyed - passed? " + __bridges.isEmpty());
		}

		return passed;
	}
}

