<?xml version="1.0" encoding="utf-8" standalone="yes" ?>

<!-- ****************************************************************************************** -->
<!-- * Transformation of schema description into configuration instance format to XML		*** -->
<!-- ****************************************************************************************** -->
<xsl:transform  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:schema="http://openoffice.org/2000/registry/schema/description"
				xmlns:default="http://openoffice.org/2000/registry/schema/default"
				xmlns:cfg="http://openoffice.org/2000/registry/instance">

<!-- Get the correct format -->
<xsl:output method="xml" indent-result="yes" indent="yes"/>

<!-- parameter for controlling the generation of templates -->
<xsl:param name="templates">false</xsl:param>
<!-- parameter for controlling in which way sets are generated -->
<xsl:param name="useSchema">false</xsl:param>
<!-- parameter for controlling in which way sets are generated -->
<xsl:param name="server">false</xsl:param>
<!-- parameter which contains the path -->
<xsl:param name="path">.</xsl:param>

<!-- variables which depend and the format choosen -->
<xsl:variable name ="derivedBy">
     <xsl:choose>
        <xsl:when test="$useSchema='true'">cfg:derivedBy</xsl:when>
        <xsl:otherwise>derived</xsl:otherwise>
     </xsl:choose>
</xsl:variable>

<xsl:variable name ="value">
     <xsl:choose>
        <xsl:when test="$useSchema='true'">cfg:value</xsl:when>
		<xsl:when test="$server='true'">defaultvalue</xsl:when>
        <xsl:otherwise>value</xsl:otherwise>
     </xsl:choose>
</xsl:variable>

<xsl:variable name ="element-of">
     <xsl:choose>
        <xsl:when test="$useSchema='true'">cfg:element-of</xsl:when>
        <xsl:otherwise>instance</xsl:otherwise>
     </xsl:choose>
</xsl:variable>

<xsl:variable name ="type">
     <xsl:choose>
        <xsl:when test="$useSchema='true'">cfg:type</xsl:when>
        <xsl:otherwise>type</xsl:otherwise>
     </xsl:choose>
</xsl:variable>

<xsl:variable name ="integer">
     <xsl:choose>
        <xsl:when test="$useSchema='true'">int</xsl:when>
        <xsl:otherwise>integer</xsl:otherwise>
     </xsl:choose>
</xsl:variable>



<!-- Transformation into data and template files -->

<!-- ****************************** -->
<!-- * schema:package			*** -->
<!-- ****************************** -->
	<xsl:template match="/schema:package">
        <!-- Deside what to generate -->
		<xsl:element name="{@package-id}">
			<xsl:attribute name="xmlns" namespace="">http://openoffice.org/2000/registry/components/<xsl:value-of select="@package-id"/></xsl:attribute>
			<xsl:attribute name="xmlns:cfg" namespace="">http://openoffice.org/2000/registry/instance</xsl:attribute>
			<xsl:attribute name="xmlns:xsi" namespace="">http://www.w3.org/1999/XMLSchema-instance</xsl:attribute>
			<xsl:choose>
                <xsl:when test="$templates = 'true' ">
	<!-- Templates -->
					<xsl:apply-templates select="schema:templates"/>
				</xsl:when>
	<!-- Data -->
				<xsl:otherwise>
					<xsl:apply-templates select="schema:component"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:documentation		*** -->
<!-- ****************************** -->
    <xsl:template match="schema:documentation">
<!-- nothing to do, as we don't need any documentation in the instance xml -->        
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:type-info			*** -->
<!-- ****************************** -->
    <xsl:template match="schema:type-info">
<!-- nothing to do, as we don't need any type-info in the instance xml -->        
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:templates			*** -->
<!-- ****************************** -->
    <xsl:template match="schema:templates">
        <xsl:apply-templates/>
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:component			*** -->
<!-- ****************************** -->
	<xsl:template match="schema:component">
        <xsl:apply-templates/>
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:instance			*** -->
<!-- ****************************** -->
	<xsl:template match="schema:instance">
		<xsl:choose>
            <xsl:when test="child::default:set"><xsl:apply-templates select="default:set"/></xsl:when>
            <xsl:when test="child::default:group"><xsl:apply-templates select="default:group"/></xsl:when>
            <xsl:when test="child::default:value"><xsl:apply-templates select="default:value"/></xsl:when>
			<xsl:otherwise>
				<xsl:variable name ="instance"><xsl:value-of select="@cfg:instance-of"/></xsl:variable>
				<xsl:choose>					
					<xsl:when test="@template-id">												
						<xsl:variable name ="file"><xsl:value-of select="$path"/>/<xsl:value-of select="@template-id"/>.xml</xsl:variable>																	
						<xsl:message>Using template '<xsl:value-of select="$instance"/> from file <xsl:value-of select="$file"/></xsl:message>
						<xsl:apply-templates select="document($file)/schema:package/schema:templates/schema:set[@cfg:name=$instance]">
							<xsl:with-param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:with-param>
						</xsl:apply-templates>
						<xsl:apply-templates select="document($file)/schema:package/schema:templates/schema:group[@cfg:name=$instance]">
							<xsl:with-param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:with-param>
						</xsl:apply-templates>
						<xsl:apply-templates select="document($file)/schema:package/schema:templates/schema:value[@cfg:name=$instance]">
							<xsl:with-param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:with-param>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:otherwise>												
						<xsl:apply-templates select="/schema:package/schema:templates/schema:set[@cfg:name=$instance]">
							<xsl:with-param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:with-param>
						</xsl:apply-templates>
						<xsl:apply-templates select="/schema:package/schema:templates/schema:group[@cfg:name=$instance]">
							<xsl:with-param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:with-param>
						</xsl:apply-templates>
						<xsl:apply-templates select="/schema:package/schema:templates/schema:value[@cfg:name=$instance]">
							<xsl:with-param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:with-param>
						</xsl:apply-templates>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
        </xsl:choose>        
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:set				*** -->
<!-- ****************************** -->
    <xsl:template match="schema:set">
        <xsl:param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:param>
		<xsl:element name="{$elementName}" >
			<xsl:call-template name="setContent"/>
		</xsl:element>
	</xsl:template>

    <xsl:template match="default:set">
		<xsl:choose>
			<!-- do we use the group in a set -->
            <xsl:when test="../@cfg:element-type and $useSchema='true'">
				<!-- then use the element-type as template name -->
				<xsl:element name="{../@cfg:element-type}" >
				<!-- and add a cfg:name as attribute -->
					<xsl:attribute name="cfg:name" namespace="">
						<xsl:value-of select="@cfg:name"/>
					</xsl:attribute>
					<xsl:call-template name="setContent"/>
				</xsl:element>
			</xsl:when>
			<xsl:when test="../@cfg:instance-of">
				<xsl:element name="{../@cfg:name}" >
					<xsl:call-template name="setContent"/>
				</xsl:element>
			</xsl:when>			
			<xsl:otherwise>
				<xsl:element name="{@cfg:name}" >
					<xsl:call-template name="setContent"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:group				*** -->
<!-- ****************************** -->
	<xsl:template match="schema:group">
        <xsl:param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:param>
		<xsl:element name="{$elementName}">        
            <xsl:if test="$templates = 'true' ">
                <xsl:call-template name="commonAttributes"/>
            </xsl:if>
            <xsl:apply-templates/>
		</xsl:element>
	</xsl:template>

    <xsl:template match="default:group">
		<xsl:choose>
			<!-- do we use the group in a set -->
            <xsl:when test="../@cfg:element-type and $useSchema='true'">
				<!-- then use the element-type as template name -->
				<xsl:element name="{../@cfg:element-type}" >
				<!-- and add a cfg:name as attribute -->
					<xsl:attribute name="cfg:name" namespace="">
						<xsl:value-of select="@cfg:name"/>
					</xsl:attribute>
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:when>			
			<xsl:when test="../@cfg:instance-of">
				<xsl:element name="{../@cfg:name}" >
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="{@cfg:name}" >
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

<!-- ****************************** -->
<!-- * schema:value				*** -->
<!-- ****************************** -->
	<xsl:template match="schema:value">
		<xsl:param name="elementName"><xsl:value-of select="@cfg:name"/></xsl:param>
		<xsl:element name="{$elementName}">        
			<xsl:call-template name="valueContent"/>
		</xsl:element>
	</xsl:template>

<!-- ****************************** -->
<!-- * default:value			*** -->
<!-- ****************************** -->
    <xsl:template match="default:value">
		<xsl:choose>
			<!-- do we use the group in a set -->
            <xsl:when test="../@cfg:element-type and $useSchema='true'">
				<!-- then use the element-type as template name -->
				<xsl:element name="{../@cfg:element-type}" >
				<!-- and add a cfg:name as attribute -->
					<xsl:attribute name="cfg:name" namespace="">
                        <xsl:value-of select="@cfg:name"/>
					</xsl:attribute>
					<xsl:call-template name="valueContent"/>
				</xsl:element>
			</xsl:when>
			<xsl:when test="../@cfg:instance-of">
				<xsl:element name="{../@cfg:name}" >
					<xsl:call-template name="valueContent"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="{@cfg:name}" >
					<xsl:call-template name="valueContent"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

<!-- ****************************** -->
<!-- * default:list				*** -->
<!-- ****************************** -->
    <xsl:template match="default:list">
        <xsl:apply-templates select="default:data"/>
	</xsl:template>

<!-- ****************************** -->
<!-- * default:null				*** -->
<!-- ****************************** -->
    <xsl:template match="default:null">
        <xsl:call-template name="nullContent"/>
	</xsl:template>

<!-- ****************************** -->
<!-- * default:data				*** -->
<!-- ****************************** -->
    <xsl:template match="default:data">
        <xsl:element name="{$value}">
			<xsl:if test="@cfg:encoding">
				<xsl:attribute name="cfg:encoding" namespace=""><xsl:value-of select="@cfg:encoding"/></xsl:attribute>
			</xsl:if>
			<xsl:value-of select="."/>
		</xsl:element>
	</xsl:template>

<!-- ****************************** -->
<!-- * setContent				*** -->
<!-- ****************************** -->
    <xsl:template name="setContent">
        <xsl:attribute name="{$type}" namespace="">set</xsl:attribute>
        <xsl:attribute name="{$element-of}" namespace="">
			<xsl:choose>
					<xsl:when test="@template-id"><xsl:value-of select="@template-id"/>/<xsl:value-of select="@cfg:element-type"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="/schema:package/@package-id"/>/<xsl:value-of select="@cfg:element-type"/></xsl:otherwise>
			</xsl:choose>
        </xsl:attribute>
        <xsl:if test="$templates = 'true' ">
            <xsl:call-template name="commonAttributes"/>
        </xsl:if>
        <xsl:apply-templates select="default:set"/>
        <xsl:apply-templates select="default:group"/>
        <xsl:apply-templates select="default:value"/>
	</xsl:template>

<!-- ****************************** -->
<!-- * valueContent				*** -->
<!-- ****************************** -->
	<xsl:template name="valueContent">
        <xsl:call-template name="typeResolution"/>
        <xsl:if test="$templates = 'true' ">
            <xsl:call-template name="commonAttributes"/>
        </xsl:if>

        <xsl:choose>
            <xsl:when test="child::default:data"><xsl:apply-templates select="default:data"/></xsl:when>
            <xsl:when test="child::default:list"><xsl:apply-templates select="default:list"/></xsl:when>
            <xsl:when test="child::default:null"><xsl:apply-templates select="default:null"/></xsl:when>
            <xsl:when test="$useSchema='false'"><xsl:call-template name="nullContent"/></xsl:when>
        </xsl:choose>
	</xsl:template>

<!-- ****************************** -->
<!-- * nullContent             *** -->
<!-- ****************************** -->
    <xsl:template name="nullContent">
        <xsl:element name="{$value}">
            <xsl:if test="$useSchema = 'true' ">
                <xsl:attribute name="xsi:null" namespace="">true</xsl:attribute>
            </xsl:if>
		</xsl:element>
	</xsl:template>


<!-- ****************************** -->
<!-- * typeResolution			*** -->
<!-- ****************************** -->
    <xsl:template name="typeResolution">
        <!-- printing the type -->		
		<xsl:attribute name="{$type}" namespace="">		
			<xsl:choose>
				<xsl:when test="@cfg:type='int'">
					<xsl:value-of select="$integer"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@cfg:type"/>
				</xsl:otherwise>
			</xsl:choose>				
		</xsl:attribute>

		<!-- printing the encoding if use schema false -->
		<xsl:if test="@cfg:type='binary' and $useSchema='false'">			
			<xsl:attribute name="encoding" namespace="">hex</xsl:attribute>
		</xsl:if>

		<!-- do we have a list? -->
        <xsl:if test="@cfg:derivedBy='list'">
            <xsl:attribute name="{$derivedBy}" namespace="">
                <xsl:value-of select="@cfg:derivedBy"/>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

<!-- ****************************** -->
<!-- * commonAttributes			*** -->
<!-- ****************************** -->
    <xsl:template name="commonAttributes">
        <xsl:if test="@cfg:writable !='true'">
            <xsl:attribute name="cfg:writable" namespace="">
                <xsl:value-of select="@cfg:writable"/>
            </xsl:attribute>
        </xsl:if>
        <xsl:if test="@cfg:nullable != 'false'">
            <xsl:attribute name="cfg:nullable" namespace="">
                <xsl:value-of select="@cfg:nullable"/>
            </xsl:attribute>
        </xsl:if>

		<xsl:if test="@cfg:notified = 'false'">
            <xsl:attribute name="cfg:notified" namespace="">
                <xsl:value-of select="@cfg:notified"/>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

</xsl:transform>

