/*************************************************************************
 *
 *  $RCSfile: ofadbmgr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _OFF_OFADBMGR_HXX
#define _OFF_OFADBMGR_HXX

#ifndef _SDB_ROW_HXX
#include <sdb/row.hxx>
#endif

#ifndef _SFXREQUEST_HXX //autogen
#include <sfx2/request.hxx>
#endif
#ifndef _SBAITEMS_HXX //autogen
#include "sbaitems.hxx"
#endif
#ifndef _DBDATDEF_HXX //autogen
#include "dbdatdef.hxx"
#endif

class OfficeApplication;
class SbaObject;
class SwImportParam;
struct OfaDatabaseRef;
class SdbConnection;
class SdbCursor;
class SdbColumn;
class SdbColumns;
class SbaDBDef;
class SbaColumn;
class Date;
class Time;
class List;

// -----------------------------------------------------------------------

#define DB_DELIM ((BYTE)0xff)			// Datenbank <-> Tabellen-Trenner
										// Ist auch im Writer definiert
										// (swtypes.hxx), daher nicht aendern!

// Fuer spaetere Erweiterungen auf weitere Typen Konstanten verwenden
#define DBMGR_STD	FALSE
#define DBMGR_BASIC	TRUE

// -----------------------------------------------------------------------

class OfaDBParam
{
	friend class OfaDBMgr;

	SbaObject**		pSbaObject;
	SdbCursor* 		pCursor;			// Cursorhandle fuer Datensatzpositionierung
	OfaDatabaseRef*	pDBRef;				// fuer Updates aus der QueryEditSh
	SbaDBDataDefRef	aDBDef;				// Spaltenbeschreibung
	String			sStatement;			// verwendetes SQL-Statement
	String			sDBName;			// Datenbankname + SQL-Statement (falls vorhanden)
	String			sSymDBName;			// Datenbankname
	ULONG	 		nCurrentPos;		// Position des Cursors im Datensatz
	ULONG	 		nCurrentSelPos;		// Position des Cursors in der Selektion des Datensatzes
	BOOL			bTable		: 1;	// Cursor ueber Tabellennamen oder SQL-Statement oeffnen?
	BOOL			bDirectPos	: 1;	// Direkte Positionierung erlaubt?
	BOOL			bBasic		: 1;
	BOOL			bDirty		: 1;	// Beim nchsten Zugriff auf Datensatz 1 positionieren

	OfaDBParam	(const OfaDBParam& r);	//nicht implementiert

public:
	SbaSelectionListRef pSelectionList;	// Liste der selektierten Eintrge (siehe ref.hxx)

	OfaDBParam	(SbaObject** pSbaObj);
	~OfaDBParam	();

	OfaDBParam&				operator=	 (const OfaDBParam& r);
	void					Clear		 ();
	void					SetDBName	 (const String& rDBName);
	inline String&			GetDBName	 () { return sDBName; }
	inline const String&	GetSymDBName () { return sSymDBName; }
	String					GetTableName ();
	inline void				SetDirty	 (const BOOL bFlag = TRUE) { bDirty = bFlag; }
	inline BOOL				GetDirty	 () const { return bDirty; }
	BOOL					HasConnection();

	// Folgende Funktionen bitte NUR in abgeleiteten DB-Managern verwenden:
	inline String&			GetStatement () { return sStatement; }
	inline SdbCursor*		GetCursor    () { return pCursor; }
	inline void				SetCursor    (SdbCursor *pCrsr) { pCursor = pCrsr; }
	inline BOOL				IsBasic      ()	{ return bBasic; }
	inline ULONG&			CurrentPos   ()	{ return nCurrentPos; }
	inline ULONG&			CurrentSelPos()	{ return nCurrentSelPos; }
	inline SbaDBDataDef*	GetDBDef     () { return &aDBDef; }
};

typedef OfaDBParam* DBParamPtr;
SV_DECL_PTRARR_DEL(OfaDBParamArr, DBParamPtr, 0, 5)

/*--------------------------------------------------------------------
	 Beschreibung: Datenbanken verwalten
 --------------------------------------------------------------------*/
class OfaDBMgr
{
	SbaObject*			Impl_SbaObject();
	OfficeApplication*	pOffApp;
	String				aEmptyStr;

protected:
	SbaObject*		pSbaObject;
	OfaDBParamArr	aDBDataArr;
	String			sLastName;		// Zuletzt bearbeiteter Datenbankname
	String			sLastBasicName;	// Zuletzt vom Basic bearbeiteter Datenbankname
	String			sAddressName;	// Name der Adressdatenbank
	USHORT			nCurrParamPos;	// Position fuer GetFirst/NextDBData
	BOOL			bCurrBasic	  : 1;// Basicflag fuer GetFirst/NextDBData
	BOOL 			bInMerge	: 1;	// Fuer Writer Feldbefehle

	String			GetSQLValue(const SdbColumn *pColumn, const String& rFieldVal);
	BOOL			SendSQLCommand(const BOOL bBasic, const String& rStatement);
	BOOL			RestoreOldCursorPos(const BOOL bBasic);
	void			SetNewStatement(const BOOL bBasic, const String& rStatement);
	void			GotoRecord(const BOOL bBasic, ULONG nIndex);
	BOOL			SetCursorOnValidPos(const BOOL bBasic);
//	void			ClearList(List* pList);

public:

	OfaDBMgr();

	inline SbaObject* GetSbaObject() { if (!pSbaObject) pSbaObject = Impl_SbaObject(); return pSbaObject; }

	BOOL			CreateDatabaseEntry(const String& rDBName, const String& rTitle, const String& rArgs);
	BOOL			DeleteDatabaseEntry(const String& rDBName);
	long			GetDatabaseLimits(const String& rDBName, USHORT nType);

	OfaDBParam&		GetDBData(const BOOL bBasic, const String* pDBName = NULL);
	OfaDBParam*		GetFirstDBData(const BOOL bBasic);
	OfaDBParam*		GetNextDBData();

	String			ExtractDBName(const String& rDBName);
	String			ExtractDBStatement(const String& rDBName);
	const String&	GetAddressDBName();
	static String	GetDatabaseTitle(const String& rFileName);

	BOOL 			OpenDB(const BOOL bBasic, BOOL bShowError = FALSE);
	BOOL			OpenDB(const BOOL bBasic, const String& rDBName, BOOL bShowError = FALSE);
	BOOL			IsDBOpen(const BOOL bBasic, const String& rDBName);
	// Alle Datenbankverbindungen loeschen
	void			CloseAll(const BOOL bBasic = DBMGR_STD);

	void			PreInitDBData(const String& rDBName, long nSelStart, long nSelEnd);
	void			GetDBSelection(const String& sDBDesc, long& rSelStart, long& rSelEnd);

	void			SetAllDirty(const BOOL bBasic, const BOOL bDirty = TRUE);

	BOOL			IsSuccessful(const BOOL bBasic);
	BOOL			IsSuccessful(const BOOL bBasic, SdbCursor **ppCursor);

	BOOL			ChangeStatement(const BOOL bBasic, const String &rStatement);

	SbaDBDataDef*	OpenColumnNames(const BOOL bBasic);

	void			Flush(const BOOL bBasic);	// DB-Cache loeschen
	void 			RemoveConnection(const BOOL bBasic);

	String			ImportDBField(UINT16 i, SbaDBDataDef* pDef, const ODbRowRef& xRow, double *pNumber = NULL, ULONG *pFormat = NULL);

	// Selektion auswerten, Navigation
	//
	BOOL			ToFirstSelectedRecord(const BOOL bBasic);
	ODbRowRef		GetCurSelectedRecord(const BOOL bBasic);
	BOOL			ToNextSelectedRecord(const BOOL bBasic);
	BOOL			ToPrevSelectedRecord(const BOOL bBasic);
	BOOL			ToLastSelectedRecord(const BOOL bBasic);

	BOOL			ToSelectedRecord(const BOOL bBasic, ULONG nRecNum);
	ODbRowRef		GetSelectedRecord(const BOOL bBasic, ULONG nRecNum);
	ULONG			GetCurSelectedRecordId(const BOOL bBasic);
	ULONG			GetCurRecordId(const BOOL bBasic);
	BOOL			IsFirstRecord(const BOOL bBasic);
	BOOL			IsLastRecord(const BOOL bBasic);
	ULONG			AbsToRel(const BOOL bBasic, ULONG nAbsPos);

	// Columns
	BOOL			GetColumnCnt(const BOOL bBasic, String sColumnName, ULONG nRecNum, String &sResult, double *pNumber = NULL, ULONG *pFormat = NULL);
	String			GetTypeDescription(const SbaColumn* pCol);
	String			GetSubtypeDescription(const SbaColumn* pCol);
	short			GetColumnNum(const BOOL bBasic);
	int				GetColumnPos(const BOOL bBasic, String sColumnName);
	String			GetColumnType(const BOOL bBasic, UINT16 nColPos, BOOL bSub);
	ULONG			GetColumnFormat(const BOOL bBasic, UINT16 nColPos);
	short			GetColumnFormatType(const BOOL bBasic, UINT16 nColPos);
	ULONG			GetRowCount(const BOOL bBasic);

	// Tabellen
	BOOL			ShowInBeamer(const String& rDBName, const String& rTableName, USHORT nType, const String& rStatement);

	// Nur fuer Writer wichtig:
	inline BOOL		IsInMerge() const				{ return bInMerge; }
	inline void		SetInMerge(BOOL bMerge = TRUE) 	{ bInMerge = bMerge; }

};

#endif


