/*************************************************************************
 *
 *  $RCSfile: apearcfg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PRECOMPILED
#include "ofapch.hxx"
#endif

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif
#pragma hdrstop

#include "app.hrc"
#include "app.hxx"
#include "apearcfg.hxx"
#include "resid.hxx"
#include "optgdlg.hxx"

#define VERSION01 		1
#define VERSION02 		2
#define APPEARANCE_VERSION 	VERSION02

#define DEFAULT_LOOKNFEEL   0
#define DEFAULT_DRAGMODE    2
#define DEFAULT_SNAPMODE    0
#define DEFAULT_SCALEFACTOR 100


#ifndef SFX_ITEMTYPE_APPEARCONFIG
// Must be defined in sfx2/inc/cfgitem.hxx
#define SFX_ITEMTYPE_APPEARCONFIG 17
#endif

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

OfaTabAppearanceCfg::OfaTabAppearanceCfg() :
	SfxConfigItem(SFX_ITEMTYPE_APPEARCONFIG),
	nLookNFeel   ( DEFAULT_LOOKNFEEL ),
	nDragMode    ( DEFAULT_DRAGMODE ),
	nSnapMode    ( DEFAULT_SNAPMODE ),
	nScaleFactor ( DEFAULT_SCALEFACTOR ),
	bMenuMouseFollow(FALSE),
	bMenuMouseFollowValid(FALSE),
	bSingleLineTabCtrl(FALSE),
	bSingleLineTabCtrlValid(FALSE),
	bColoredTabCtrl(FALSE),
	bColoredTabCtrlValid(FALSE)
{
}

Boolean OfaTabAppearanceCfg::bInitialized = False;

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

String	OfaTabAppearanceCfg::GetName() const
{
	return String::CreateFromAscii("(empty)");
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

int  OfaTabAppearanceCfg::Load(SvStream& rStream)
{
	SetDefault(FALSE);
	UINT16 nVersion;
	rStream >> nVersion;

	if(nVersion >= VERSION01)
	{
		USHORT nSet;
		rStream      >> nSet;
		nLookNFeel   =  nSet;
		rStream      >> nSet;
		nDragMode    =  nSet;
		rStream      >> nSet;
		nScaleFactor =  nSet;
		rStream      >> nSet;
		nSnapMode    =  nSet;
	}
	if(nVersion >= VERSION02)
	{
		BOOL bFlag;
		rStream >> bFlag; 	bMenuMouseFollow 		= bFlag;
		rStream >> bFlag; 	bMenuMouseFollowValid	= bFlag;
		rStream >> bFlag; 	bSingleLineTabCtrl		= bFlag;
		rStream >> bFlag; 	bSingleLineTabCtrlValid	= bFlag;
		rStream >> bFlag; 	bColoredTabCtrl			= bFlag;
		rStream >> bFlag; 	bColoredTabCtrlValid	= bFlag;
	}

	return SfxConfigItem::ERR_OK;
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

BOOL    OfaTabAppearanceCfg::Store(SvStream& rStream)
{
	UINT16 nVersion = APPEARANCE_VERSION;
	rStream << nVersion;
	rStream << nLookNFeel;
	rStream << nDragMode;
	rStream << nScaleFactor;
	rStream << nSnapMode;

	BOOL bFlag = bMenuMouseFollow; 	 rStream << bFlag;
	bFlag = bMenuMouseFollowValid;	 rStream << bFlag;
	bFlag = bSingleLineTabCtrl;		 rStream << bFlag;
	bFlag = bSingleLineTabCtrlValid; rStream << bFlag;
	bFlag = bColoredTabCtrl;		 rStream << bFlag;
	bFlag = bColoredTabCtrlValid;	 rStream << bFlag;

	return TRUE;
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void  OfaTabAppearanceCfg::UseDefault()
{
	nLookNFeel   = DEFAULT_LOOKNFEEL;
	nDragMode    = DEFAULT_DRAGMODE;
	nScaleFactor = DEFAULT_SCALEFACTOR;
	nSnapMode    = DEFAULT_SNAPMODE;
	bMenuMouseFollow		  =
	bMenuMouseFollowValid	  =
	bSingleLineTabCtrl		  =
	bSingleLineTabCtrlValid   =
	bColoredTabCtrl			  =
	bColoredTabCtrlValid	  = FALSE;

	SfxConfigItem::UseDefault();
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetLookNFeel ( USHORT nSet )
{
	nLookNFeel = nSet;
	SetDefault(FALSE);
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetDragMode  ( USHORT nSet )
{
	nDragMode = nSet;
	SetDefault(FALSE);
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetScaleFactor ( USHORT nSet )
{
	nScaleFactor = nSet;
	SetDefault(FALSE);
}

/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetSnapMode ( USHORT nSet )
{
	nSnapMode = nSet;
	SetDefault(FALSE);
}


/*--------------------------------------------------------------------
	 Beschreibung:
 --------------------------------------------------------------------*/

void OfaTabAppearanceCfg::SetApplicationDefaults ( Application* pApp )
{
	AllSettings   hAppSettings = pApp->GetSettings();
	StyleSettings hAppStyle    = hAppSettings.GetStyleSettings();

	// Look & Feel

	switch ( nLookNFeel )
	{
	case LookMotif:
		hAppStyle.SetStandardUnixStyles();
		break;
	case LookOSTwo:
		hAppStyle.SetStandardOS2Styles();
		break;
	case LookMacintosh:
		hAppStyle.SetStandardMacStyles();
		break;
	case LookWindows:
		hAppStyle.SetStandardWinStyles();
		break;
	case LookStardivision:
	default:
		hAppStyle.SetStandardStyles();
		break;
	}

	// Screen and ScreenFont Scaling

	hAppStyle.SetScreenZoom( nScaleFactor );
	hAppStyle.SetScreenFontZoom( nScaleFactor );

	// Mouse Snap

	MouseSettings hMouseSettings = hAppSettings.GetMouseSettings();
	ULONG         nMouseOptions  = hMouseSettings.GetOptions();

	nMouseOptions &=  ! (MOUSE_OPTION_AUTOCENTERPOS | MOUSE_OPTION_AUTODEFBTNPOS);

	switch ( nSnapMode )
	{
	case SnapToButton:
		nMouseOptions |= MOUSE_OPTION_AUTODEFBTNPOS;
		break;
	case SnapToMiddle:
		nMouseOptions |= MOUSE_OPTION_AUTOCENTERPOS;
		break;
	case NoSnap:
	default:
		break;
	}

	// Merge and Publish Settings

	hMouseSettings.SetOptions( nMouseOptions );
	hAppSettings.SetMouseSettings( hMouseSettings );

	hAppSettings.SetStyleSettings( hAppStyle );
	pApp->MergeSystemSettings    ( hAppSettings );      // Allow system-settings to apply
	pApp->SystemSettingsChanging ( hAppSettings, NULL );// Allow overruling of system-settings
														//is concerned with window drag

	pApp->SetSettings ( hAppSettings );
}



