/*************************************************************************
 *
 *  $RCSfile: fltrcfg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PRECOMPILED
#include "ofapch.hxx"
#endif

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _SVX_SVXIDS_HRC
#include <svx/svxids.hrc>
#endif

#pragma hdrstop

#include "fltrcfg.hxx"
#include "resid.hxx"
#include "app.hrc"

// -----------------------------------------------------------------------
#define FILTERCFG_WORD_CODE 			0x0001
#define FILTERCFG_WORD_STORAGE 			0x0002
#define FILTERCFG_EXCEL_CODE 			0x0004
#define FILTERCFG_EXCEL_STORAGE 		0x0008
#define FILTERCFG_PPOINT_CODE 			0x0010
#define FILTERCFG_PPOINT_STORAGE 		0x0020
#define FILTERCFG_MATH_LOAD				0x0100
#define FILTERCFG_MATH_SAVE				0x0200
#define FILTERCFG_WRITER_LOAD			0x0400
#define FILTERCFG_WRITER_SAVE			0x0800
#define FILTERCFG_CALC_LOAD				0x1000
#define FILTERCFG_CALC_SAVE				0x2000
#define FILTERCFG_IMPRESS_LOAD			0x4000
#define FILTERCFG_IMPRESS_SAVE			0x8000

#define FILTERCONFG_VERSION01 0x01

#define FILTERCFG_VERSION 	FILTERCONFG_VERSION01


struct OfaFilterOptions_Impl
{
	ULONG nFlags;
	OfaFilterOptions_Impl() : nFlags(0) {}

	void SetFlag( ULONG nFlag, BOOL bSet )
		{
			if( bSet )
				nFlags |= nFlag;
			else
				nFlags &= ~nFlag;
		}
	BOOL IsFlag( ULONG nFlag ) const
		{
			return 0 != (nFlags & nFlag );
		}
};

// -----------------------------------------------------------------------

OfaFilterOptions::OfaFilterOptions()
	: SfxConfigItem( SFX_ITEMTYPE_FILTERCONFIG )
{
	pImp = new OfaFilterOptions_Impl;
	UseDefault();
}

OfaFilterOptions::~OfaFilterOptions()
{
	delete pImp;
}

int OfaFilterOptions::Load( SvStream& rStream )
{
	USHORT nVersion;
	rStream >> nVersion;
	if( FILTERCONFG_VERSION01 <= nVersion )
	{
		rStream >> pImp->nFlags;
		return SfxConfigItem::ERR_OK;
	}
	return SfxConfigItem::WARNING_VERSION;
}

BOOL OfaFilterOptions::Store(SvStream& rStream)
{
	rStream	<< (USHORT)FILTERCFG_VERSION;
	rStream << pImp->nFlags;
	return SVSTREAM_OK == rStream.GetError();
}

void OfaFilterOptions::UseDefault()
{
	pImp->nFlags = FILTERCFG_WORD_CODE		| FILTERCFG_WORD_STORAGE |
					FILTERCFG_EXCEL_CODE	| FILTERCFG_EXCEL_STORAGE |
					FILTERCFG_PPOINT_CODE	| FILTERCFG_PPOINT_STORAGE |
					FILTERCFG_MATH_LOAD		| FILTERCFG_MATH_SAVE |
					FILTERCFG_WRITER_LOAD	| FILTERCFG_WRITER_SAVE |
					FILTERCFG_CALC_LOAD		| FILTERCFG_CALC_SAVE	|
					FILTERCFG_IMPRESS_LOAD	| FILTERCFG_IMPRESS_SAVE
					;

	SfxConfigItem::UseDefault();
}

String OfaFilterOptions::GetName() const
{
	return String( OffResId( RID_STR_NAME_FILTERCFG	));
}

// -----------------------------------------------------------------------

void OfaFilterOptions::SetLoadWordBasicCode( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_WORD_CODE, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsLoadWordBasicCode() const
{
	return pImp->IsFlag( FILTERCFG_WORD_CODE );
}

void OfaFilterOptions::SetLoadWordBasicStorage( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_WORD_STORAGE, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsLoadWordBasicStorage() const
{
	return pImp->IsFlag( FILTERCFG_WORD_STORAGE );
}

// -----------------------------------------------------------------------

void OfaFilterOptions::SetLoadExcelBasicCode( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_EXCEL_CODE, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsLoadExcelBasicCode() const
{
	return pImp->IsFlag( FILTERCFG_EXCEL_CODE );
}

void OfaFilterOptions::SetLoadExcelBasicStorage( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_EXCEL_STORAGE, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsLoadExcelBasicStorage() const
{
	return pImp->IsFlag( FILTERCFG_EXCEL_STORAGE );
}

// -----------------------------------------------------------------------

void OfaFilterOptions::SetLoadPPointBasicCode( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_PPOINT_CODE, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsLoadPPointBasicCode() const
{
	return pImp->IsFlag( FILTERCFG_PPOINT_CODE );
}

void OfaFilterOptions::SetLoadPPointBasicStorage( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_PPOINT_STORAGE, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsLoadPPointBasicStorage() const
{
	return pImp->IsFlag( FILTERCFG_PPOINT_STORAGE );
}

// -----------------------------------------------------------------------

BOOL OfaFilterOptions::IsMathType2StarMath() const
{
	return pImp->IsFlag( FILTERCFG_MATH_LOAD );
}

void OfaFilterOptions::SetMathType2StarMath( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_MATH_LOAD, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsStarMath2MathType() const
{
	return pImp->IsFlag( FILTERCFG_MATH_SAVE );
}

void OfaFilterOptions::SetStarMath2MathType( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_MATH_SAVE, bFlag );
	SetDefault(FALSE);
}


// -----------------------------------------------------------------------
BOOL OfaFilterOptions::IsWinWord2StarWriter() const
{
	return pImp->IsFlag( FILTERCFG_WRITER_LOAD );
}

void OfaFilterOptions::SetWinWord2StarWriter( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_WRITER_LOAD, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsStarWriter2WinWord() const
{
	return pImp->IsFlag( FILTERCFG_WRITER_SAVE );
}

void OfaFilterOptions::SetStarWriter2WinWord( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_WRITER_SAVE, bFlag );
	SetDefault(FALSE);
}


// -----------------------------------------------------------------------
BOOL OfaFilterOptions::IsExcel2StarCalc() const
{
	return pImp->IsFlag( FILTERCFG_CALC_LOAD );
}

void OfaFilterOptions::SetExcel2StarCalc( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_CALC_LOAD, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsStarCalc2Excel() const
{
	return pImp->IsFlag( FILTERCFG_CALC_SAVE );
}

void OfaFilterOptions::SetStarCalc2Excel( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_CALC_SAVE, bFlag );
	SetDefault(FALSE);
}


// -----------------------------------------------------------------------
BOOL OfaFilterOptions::IsPowerPoint2StarImpress() const
{
	return pImp->IsFlag( FILTERCFG_IMPRESS_LOAD );
}

void OfaFilterOptions::SetPowerPoint2StarImpress( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_IMPRESS_LOAD, bFlag );
	SetDefault(FALSE);
}

BOOL OfaFilterOptions::IsStarImpress2PowerPoint() const
{
	return pImp->IsFlag( FILTERCFG_IMPRESS_SAVE );
}

void OfaFilterOptions::SetStarImpress2PowerPoint( BOOL bFlag )
{
	pImp->SetFlag( FILTERCFG_IMPRESS_SAVE, bFlag );
	SetDefault(FALSE);
}




