/*************************************************************************
 *
 *  $RCSfile: column.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <column.hxx>
#include <api.hrc>

#ifndef _COM_SUN_STAR_SDBC_DATATYPE_HPP_
#include <com/sun/star/sdbc/DataType.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_COLUMNVALUE_HPP_
#include <com/sun/star/sdbc/ColumnValue.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEFIELDTYPE_HPP_
#include <com/sun/star/data/DatabaseFieldType.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HDL_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _UNOTOOLS_ENUMHELPER_HXX_
#include <unotools/enumhelper.hxx>
#endif
#ifndef _SFXINTITEM_HXX //autogen
#include <svtools/intitem.hxx>
#endif
#ifndef _SFXENUMITEM_HXX //autogen
#include <svtools/eitem.hxx>
#endif
#ifndef _SFXSTRITEM_HXX //autogen
#include <svtools/stritem.hxx>
#endif
#ifndef _SDB_SDBCOL_HXX //autogen
#include <sdb/sdbcol.hxx>
#endif
#ifndef _SBA_DBDATDEF_HXX
#include <dbdatdef.hxx>
#endif
#ifndef _SBA_DBOBJENU_HXX //autogen
#include <dbobjenu.hxx>
#endif
#ifndef _SDB_TYPECONV_HXX
#include <sdb/typeconv.hxx>
#endif
#ifndef _SBAITEMS_HXX
#include "sbaitems.hxx"
#endif
#ifndef _SBAITEMS_HRC
#include "sbaitems.hrc"
#endif

DBG_NAME(OColumn);

using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::data;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::osl;
using namespace ::cppu;

//------------------------------------------------------------------------------
sal_Int32 SbaType2SDBCType(SbaFieldType eType, SbaNumericSize eNum)
{
	sal_Int32 nType = DataType::OTHER;
	switch (eType)
	{
		case dbText:
			nType = DataType::VARCHAR;
			break;
		case dbChar:
			nType = DataType::CHAR;
			break;
		case dbMemo:
			nType = DataType::LONGVARCHAR;
			break;
		case dbBool:
			nType = DataType::BIT;
			break;
		case dbCurrency:
			nType = DataType::NUMERIC;
			break;
		case dbDateTime:
			nType = DataType::TIMESTAMP;
			break;
		case dbDate:
			nType = DataType::DATE;
			break;
		case dbTime:
			nType = DataType::TIME;
			break;
		case dbDecimal:
			nType = DataType::NUMERIC;
			break;
		case dbVarBinary:
			nType = DataType::VARBINARY;
			break;
		case dbBinary:
			nType = DataType::BINARY;
			break;
		case dbImage:
			nType = DataType::LONGVARBINARY;
			break;
		case dbCounter:
			nType = DataType::INTEGER;
			break;
		case dbNumeric:
			switch (eNum)
			{
				case dbByte:
					nType = DataType::TINYINT;
					break;
				case dbInteger:
					nType = DataType::SMALLINT;
					break;
				case dbLong:
					nType = DataType::INTEGER;
					break;
				case dbSingle:
					nType = DataType::REAL;
					break;
				case dbDouble:
					nType = DataType::DOUBLE;
					break;
				default:
					DBG_ERROR("Unbekanntes ::com::sun::star::awt::Size!");
			}
	}
	return nType;
}

//------------------------------------------------------------------------------
sal_Int32 SdbType2SDBCType(sal_Int16 nSdbType)
{
	sal_Int32 nType = DataType::OTHER;
	switch (nSdbType)
	{
		case DatabaseFieldType::CURRENCY:
			nType = DataType::NUMERIC;
			break;
		default:
			nType = nSdbType;
	}
	return nType;
}

//------------------------------------------------------------------------------
void OColumn::construct()
{
	m_nType = DataType::OTHER;
	m_nPrecision = 0;
	m_nNumberFormat = 0;
	m_nIsNullable = m_nScale = 0;
	m_bAutoIncrement = m_bRowVersion = sal_False;
	m_nIsNullable = ColumnValue::NULLABLE_UNKNOWN;
}

//------------------------------------------------------------------------------
void OColumn::construct(const SdbColumn& rCol)
{
	m_aName = UniString(rCol.GetName());
	m_aDescription = UniString(rCol.GetComment());
	m_aDefaultValue = UniString(rCol.GetDefaultValue());
	m_nPrecision = rCol.GetLength();
	m_nNumberFormat = rCol.GetFormatKey();
	m_nScale = rCol.GetScale();
	m_nType = SdbType2SDBCType(rCol.GetType());
	m_bAutoIncrement = rCol.IsAutoIncrement();
	m_nIsNullable = rCol.IsNullAllowed() ? ColumnValue::NULLABLE : ColumnValue::NO_NULLS;
	m_bRowVersion = (rCol.GetAllFlags() & SDB_FLAGS_VERSIONING) == SDB_FLAGS_VERSIONING;
}

//------------------------------------------------------------------------------
void OColumn::construct(const SbaColumn& rCol)
{
	SFX_ITEMSET_GET(rCol,pName, SbaNameItem, SBA_DEF_FLTNAME, sal_True);
	m_aName = UniString(pName->GetValue());

	SFX_ITEMSET_GET(rCol, pType, SbaDataFieldTypeItem, SBA_DEF_FLTTYPE, sal_True);
	SFX_ITEMSET_GET(rCol, pLength, SfxUInt32Item, SBA_DEF_FLTLENGTH, sal_True);
	m_nType = SbaType2SDBCType((SbaFieldType)pType->GetValue(), (SbaNumericSize)pLength->GetValue());
	m_nPrecision = (sal_Int32)pLength->GetValue();

	SFX_ITEMSET_GET(rCol, pFormatKey, SfxUInt32Item, SBA_DEF_FMTVALUE, sal_True);
	m_nNumberFormat = pFormatKey->GetValue();

	SFX_ITEMSET_GET(rCol, pScale, SfxUInt16Item, SBA_DEF_FLTSCALE, sal_True);
	m_nScale = pScale->GetValue();

	SFX_ITEMSET_GET(rCol, pRequired, SfxBoolItem, SBA_DEF_FLTREQUIRED, sal_True);
	m_nIsNullable = pRequired->GetValue() ? ColumnValue::NO_NULLS : ColumnValue::NULLABLE;

	SFX_ITEMSET_GET(rCol, pComment, SfxStringItem, SBA_DEF_FLTCOMMENT, sal_True);
	m_aDescription = UniString(pComment->GetValue());

	SFX_ITEMSET_GET(rCol, pAuto, SfxBoolItem, SBA_DEF_FLTAUTOINCREMENT, sal_True);
	m_bAutoIncrement = pAuto->GetValue();

	SFX_ITEMSET_GET(rCol, pDefault, SfxStringItem, SBA_DEF_FLTDEFAULT, sal_True);
	m_aDefaultValue = UniString(pDefault->GetValue());

	SFX_ITEMSET_GET(rCol, pSearch, SfxUInt16Item, SBA_DEF_FLTSEARCH, sal_True);
	m_bRowVersion = pSearch && (pSearch->GetValue() & SDB_FLAGS_VERSIONING);
}

//--------------------------------------------------------------------------
OColumn::OColumn()
	    :OColumnBase(m_aMutex)
	    ,OPropertySetHelper(OColumnBase::rBHelper)
{
	DBG_CTOR(OColumn, NULL);
	construct();
}

//--------------------------------------------------------------------------
OColumn::OColumn(const SdbColumn& rCol)
	    :OColumnBase(m_aMutex)
	    ,OPropertySetHelper(OColumnBase::rBHelper)
{
	DBG_CTOR(OColumn, NULL);
	construct(rCol);
}

//--------------------------------------------------------------------------
OColumn::OColumn(const SbaColumn& rCol)
	    :OColumnBase(m_aMutex)
	    ,OPropertySetHelper(OColumnBase::rBHelper)
{
	DBG_CTOR(OColumn, NULL);
	construct(rCol);
}

//--------------------------------------------------------------------------
OColumn::~OColumn()
{
	DBG_DTOR(OColumn, NULL);
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OColumn::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XPropertySet > *)0 ),
							OColumnBase::getTypes() );

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OColumn::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OColumn::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OColumnBase::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XPropertySet * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OColumn::acquire() throw(RuntimeException)
{
	OColumnBase::acquire();
}

//--------------------------------------------------------------------------
void OColumn::release() throw(RuntimeException)
{
	OColumnBase::release();
}

// OComponentHelper
//------------------------------------------------------------------------------
void OColumn::disposing()
{
	OPropertySetHelper::disposing();
}

// OPropertySetHelper
//------------------------------------------------------------------------------
void OColumn::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROPERTY_ID_TYPE:
			rValue <<= m_nType;
			break;
		case PROPERTY_ID_PRECISION:
			rValue <<= m_nPrecision;
			break;
		case PROPERTY_ID_SCALE:
			rValue <<= m_nScale;
			break;
		case PROPERTY_ID_ISNULLABLE:
			rValue <<= m_nIsNullable;
			break;
		case PROPERTY_ID_NUMBERFORMAT:
			rValue <<= m_nNumberFormat;
			break;
		case PROPERTY_ID_TYPENAME:
			rValue <<= m_aTypeName;
			break;
		case PROPERTY_ID_DESCRIPTION:
			rValue <<= m_aDescription;
			break;
		case PROPERTY_ID_DEFAULTVALUE:
			rValue <<= m_aDefaultValue;
			break;
		case PROPERTY_ID_ISAUTOINCREMENT:
		{
			sal_Bool bVal = m_bAutoIncrement;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_ISROWVERSION:
		{
			sal_Bool bVal = m_bRowVersion;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_NAME:
			rValue <<= m_aName;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

//------------------------------------------------------------------------------
sal_Bool OColumn::convertFastPropertyValue(
							Any & rConvertedValue,
							Any & rOldValue,
							sal_Int32 nHandle,
							const Any& rValue )
								throw (IllegalArgumentException)
{
	return sal_False;
}

//------------------------------------------------------------------------------
void OColumn::setFastPropertyValue_NoBroadcast(
								sal_Int32 nHandle,
								const Any& rValue
												 )
												 throw (Exception)
{
}

//------------------------------------------------------------------------------
Reference< XPropertySetInfo > OColumn::getPropertySetInfo() throw (RuntimeException)
{
	return createPropertySetInfo( getInfoHelper() ) ;
}

// utl::OPropertyArrayUsageHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper* OColumn::createArrayHelper( ) const
{
	BEGIN_PROPERTY_HELPER(11)
		DECL_PROP1(DEFAULTVALUE,		::rtl::OUString,	READONLY);
		DECL_PROP1(DESCRIPTION,			::rtl::OUString,	READONLY);
		DECL_PROP1(NUMBERFORMAT,		sal_Int32,			READONLY);
		DECL_PROP1_BOOL(ISAUTOINCREMENT,	 				READONLY);
		DECL_PROP1(ISNULLABLE,			sal_Int32,			READONLY);
		DECL_PROP1_BOOL(ISROWVERSION,						READONLY);
		DECL_PROP1(NAME,				::rtl::OUString,	READONLY);
		DECL_PROP1(PRECISION,			sal_Int32,			READONLY);
		DECL_PROP1(SCALE,				sal_Int32,			READONLY);
		DECL_PROP1(TYPE,				sal_Int32,			READONLY);
		DECL_PROP1(TYPENAME,			::rtl::OUString,	READONLY);
	END_PROPERTY_HELPER();
}
	
// cppu::OPropertySetHelper
//------------------------------------------------------------------------------
IPropertyArrayHelper& OColumn::getInfoHelper()
{
	return *getArrayHelper();
}

// ::com::sun::star::container::XNamed
//------------------------------------------------------------------------------
::rtl::OUString OColumn::getName( ) throw(RuntimeException)
{
	return m_aName;
}

//------------------------------------------------------------------------------
void OColumn::setName( const ::rtl::OUString& aName ) throw(RuntimeException)
{
	// name setting not allowed
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OColumn::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OColumn");
}

//------------------------------------------------------------------------------
sal_Bool OColumn::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OColumn::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< ::rtl::OUString > aSNS( 1 );
	aSNS[0] = SERVICE_SDB_COLUMN;
	return aSNS;
}

//------------------------------------------------------------------------------
void OColumn::adjustTo(const OColumn& rCol)
{
	m_nType = rCol.m_nType;
	m_aDefaultValue = rCol.m_aDefaultValue;
	m_nNumberFormat = rCol.m_nNumberFormat;
}

//************************************************************
//  OColumns
//************************************************************
DBG_NAME(OColumns);

//--------------------------------------------------------------------------
OColumns::OColumns(::cppu::OWeakObject& _rParent, ::osl::Mutex& _rMutex, sal_Bool _bCaseSensitive)
		 :m_rParent(_rParent)
		 ,m_rMutex(_rMutex)
{
	m_pColMap = new OColumnMap(17, ::utl::UStringMixHash(_bCaseSensitive), ::utl::UStringMixEqual(_bCaseSensitive));
}

//--------------------------------------------------------------------------
OColumns::~OColumns()
{
	DELETEZ(m_pColMap);
}

//--------------------------------------------------------------------------
void SAL_CALL OColumns::acquire() throw(RuntimeException)
{
	m_rParent.acquire();
}

//--------------------------------------------------------------------------
void SAL_CALL OColumns::release() throw(RuntimeException)
{
	m_rParent.release();
}

//------------------------------------------------------------------------------
void OColumns::disposing()
{
	MutexGuard aGuard(m_rMutex);
	for (OColumnArray::iterator i = m_aColArray.begin(); m_aColArray.end() != i; i++)
	{
		(*i)->dispose();
		(*i)->release();
	}
	m_aColArray.clear();
	m_pColMap->clear();
}

//------------------------------------------------------------------
void OColumns::append(OColumn* pCol)
{
	pCol->acquire();
	(*m_pColMap)[pCol->m_aName] = pCol;
	m_aColArray.push_back(pCol);
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OColumns::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OColumns");
}

//------------------------------------------------------------------------------
sal_Bool OColumns::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OColumns::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< ::rtl::OUString > aSNS( 1 );
	aSNS[0] = SERVICE_SDBCX_CONTAINER;
	return aSNS;
}

// ::com::sun::star::container::XElementAccess
//------------------------------------------------------------------------------
Type SAL_CALL OColumns::getElementType(  ) throw(RuntimeException)
{
	return::getCppuType(static_cast<Reference<XPropertySet>*>(NULL));
}

//------------------------------------------------------------------------------
sal_Bool OColumns::hasElements(void) throw( RuntimeException )
{
	MutexGuard aGuard(m_rMutex);
	return !m_aColArray.empty();
}

// ::com::sun::star::container::XEnumerationAccess
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XEnumeration >  OColumns::createEnumeration(void) throw( RuntimeException )
{
	MutexGuard aGuard(m_rMutex);
	return new utl::OEnumerationByIndex(this);
}

// ::com::sun::star::container::XIndexAccess
//------------------------------------------------------------------------------
sal_Int32 OColumns::getCount( ) throw( RuntimeException )
{
	MutexGuard aGuard(m_rMutex);
	return m_aColArray.size();
}

//------------------------------------------------------------------------------
Any OColumns::getByIndex( sal_Int32 Index ) throw(IndexOutOfBoundsException,
												  WrappedTargetException,
												  RuntimeException)
{
	MutexGuard aGuard(m_rMutex);
	if (Index < 0 || Index >= m_aColArray.size())
		throw IndexOutOfBoundsException();

	return makeAny(Reference< XPropertySet >(m_aColArray[Index]));
}

// ::com::sun::star::container::XNameAccess
//------------------------------------------------------------------------------
Any OColumns::getByName(const ::rtl::OUString& aName) throw(::com::sun::star::container::NoSuchElementException,
															WrappedTargetException,
															RuntimeException)
{
	MutexGuard aGuard(m_rMutex);
	OColumnMap::const_iterator i = m_pColMap->find(aName);
	if (i == m_pColMap->end())
		throw ::com::sun::star::container::NoSuchElementException();

	return makeAny(Reference< XPropertySet >((*i).second));
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OColumns::getElementNames(void) throw(RuntimeException)
{
	MutexGuard aGuard(m_rMutex);
	Sequence< ::rtl::OUString> aNameList(m_aColArray.size());
	::rtl::OUString* pStringArray = aNameList.getArray();
	for (OColumnArray::const_iterator i = m_aColArray.begin(); m_aColArray.end() != i; i++)
	{
		(*pStringArray) = (*i)->m_aName;
		++pStringArray;
	}
	return aNameList;
}

//------------------------------------------------------------------------------
sal_Bool OColumns::hasByName( const ::rtl::OUString& aName ) throw(RuntimeException)
{
	MutexGuard aGuard(m_rMutex);
	OColumnMap::const_iterator i = m_pColMap->find(aName);
	return i != m_pColMap->end();
}

//------------------------------------------------------------------------------
sal_Int32 OColumns::findColumn(const rtl::OUString& columnName) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_rMutex);
	OColumnMap::const_iterator i = m_pColMap->find(columnName);
	if (i == m_pColMap->end())
		return 0;

	return find(m_aColArray.begin(), m_aColArray.end(), (*i).second) -	m_aColArray.begin() + 1;
}

//------------------------------------------------------------------------------
void OColumns::insertColumns(const SdbColumnsRef& xCols)
{
	disposing();
	sal_uInt32 nCount = xCols->Count();
	for (sal_uInt32 i = 1; i < nCount; i++)
	{
		const SdbColumn* pCol = xCols->Column(i);
		append(new OColumn(*pCol));
	}
}

//------------------------------------------------------------------------------
void OColumns::insertColumns(const SbaColumnList& rCols)
{
	disposing();
	sal_uInt32 nCount = rCols.Count();
	for (sal_uInt32 i = 0; i < nCount; i++)
	{
		const SbaColumn* pCol = rCols.GetObject(i);
		append(new OColumn(*pCol));
	}
}

//------------------------------------------------------------------
const OColumn* OColumns::at(const rtl::OUString& rName) const
{
	OColumnMap::const_iterator i = m_pColMap->find(rName);
	return i == m_pColMap->end() ? NULL : (*i).second;
}

//------------------------------------------------------------------
void OColumns::setCaseSensitive(sal_Bool bCaseSensitive)
{
	DBG_ASSERT(m_pColMap->empty(), "OColumns::setCaseSensitive() there are still existing columns");
	if (bCaseSensitive == m_pColMap->hash_funct().isCaseSensitive())
		return;

	delete(m_pColMap);
	m_pColMap = new OColumnMap(17, ::utl::UStringMixHash(bCaseSensitive), ::utl::UStringMixEqual(bCaseSensitive));
}



