/*************************************************************************
 *
 *  $RCSfile: connection.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <connection.hxx>
#include <statement.hxx>
#include <preparedstatement.hxx>
#include <callablestatement.hxx>
#include <querycomposer.hxx>
#include <api.hrc>

#ifndef _COM_SUN_STAR_UTIL_XNUMBERFORMATSSUPPLIER_HPP_
#include <com/sun/star/util/XNumberFormatsSupplier.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_DISPOSEDEXCEPTION_HPP_
#include <com/sun/star/lang/DisposedException.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_TRANSACTIONISOLATION_HPP_
#include <com/sun/star/sdbc/TransactionIsolation.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEISOLATIONLEVEL_HPP_
#include <com/sun/star/data/DatabaseIsolationLevel.hpp>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

using namespace ::com::sun::star::sdb;
using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::sdbcx;
using namespace ::com::sun::star::data;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::osl;
using namespace ::cppu;

DBG_NAME(OConnection);

//--------------------------------------------------------------------------
OConnection::OConnection(const Reference< XInterface > & xParent,
						 const SdbConnectionRef& xConn)
			:OConnectionBase(xParent)
			,m_xConnection(xConn)
			,m_aMetaData(*this)
			,m_aTables(*this)
{
	DBG_CTOR(OConnection,NULL);
	m_bReadOnly = !m_xConnection->IsCapable(SDB_CAP_UPDATABLE);
}

//--------------------------------------------------------------------------
OConnection::~OConnection()
{
	DBG_DTOR(OConnection,NULL);
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OConnection::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XConnection > *)0 ),
						   ::getCppuType( (const Reference< XServiceInfo > *)0 ),
						   ::getCppuType( (const Reference< XWarningsSupplier > *)0 ),
						   ::getCppuType( (const Reference< XTablesSupplier > *)0 ),
						   ::getCppuType( (const Reference< XSQLQueryComposerFactory > *)0 ),
						   OConnectionBase::getTypes());

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OConnection::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OConnection::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OConnectionBase::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XConnection * >( this ),
					static_cast< XCloseable * >( this ),
					static_cast< XServiceInfo * >( this ),
					static_cast< XWarningsSupplier * >( this ),
					static_cast< XTablesSupplier * >( this ),
					static_cast< XSQLQueryComposerFactory * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OConnection::acquire() throw (RuntimeException)
{
	OConnectionBase::acquire();
}

//--------------------------------------------------------------------------
void OConnection::release() throw (RuntimeException)
{
	OConnectionBase::release();
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OConnection::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OConnection");
}

//------------------------------------------------------------------------------
sal_Bool OConnection::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OConnection::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< ::rtl::OUString > aSNS( 2 );
	aSNS[0] = SERVICE_SDBC_CONNECTION;
	aSNS[1] = SERVICE_SDB_CONNECTION;
	return aSNS;
}

// OComponentHelper
//------------------------------------------------------------------------------
void OConnection::disposing()
{
	MutexGuard aGuard(*m_pMutex);
	OConnectionBase::disposing();

	m_aTables.disposing();
	for (OWeakRefArray::iterator i = m_aStatements.begin(); m_aStatements.end() != i; i++)
	{
		Reference< XComponent > xComp(i->get(), UNO_QUERY);
		if (xComp.is())
			xComp->dispose();
	}
	m_aStatements.clear();

	for (OWeakRefArray::iterator j = m_aComposers.begin(); m_aComposers.end() != j; j++)
	{
		Reference< XComponent > xComp(j->get(), UNO_QUERY);
		if (xComp.is())
			xComp->dispose();
	}
	m_aComposers.clear();
	m_xConnection = NULL;
}

// XCloseable
//------------------------------------------------------------------------------
void OConnection::close(void) throw( SQLException, RuntimeException )
{
	{
		MutexGuard aGuard( *m_pMutex );
		if (OConnectionBase::rBHelper.bDisposed)
			throw DisposedException();
	}
	dispose();
}

//------------------------------------------------------------------------------
sal_Bool OConnection::isClosed(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	return OConnectionBase::rBHelper.bDisposed;
}

// XWarningsSupplier
//------------------------------------------------------------------------------
Any OConnection::getWarnings(void) throw( SQLException, RuntimeException )
{
	return Any();
}

//------------------------------------------------------------------------------
void OConnection::clearWarnings(void) throw( SQLException, RuntimeException )
{
}

// XTablesSupplier
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XNameAccess >  OConnection::getTables() throw( RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	return &m_aTables;
}

// XSQLQueryComposerFactory
//------------------------------------------------------------------------------
Reference< XSQLQueryComposer >  OConnection::createQueryComposer(void) throw( RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed)
		throw DisposedException();

	Reference< XSQLQueryComposer >  xComposer(new OQueryComposer(*this, m_xConnection, NULL));
	m_aComposers.push_back(WeakReferenceHelper(xComposer));
	return xComposer;
}

// XConnection
//------------------------------------------------------------------------------
Reference< XStatement >  OConnection::createStatement(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	SdbCursorRef xCursor = m_xConnection->CreateCursor(SDB_SNAPSHOT, SDB_FORWARDONLY | SDB_READONLY);
	checkResult(m_xConnection->Status(), *this, sal_False);

	Reference< XStatement >  xStmt(new OStatement(*this, xCursor, NULL));
	m_aStatements.push_back(WeakReferenceHelper(xStmt));
	return xStmt;
}

//------------------------------------------------------------------------------
Reference< XPreparedStatement >  OConnection::prepareStatement(const rtl::OUString& sql) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	SdbCursorRef xCursor = m_xConnection->CreateCursor(SDB_SNAPSHOT, SDB_FORWARDONLY | SDB_READONLY | SDB_INDIRECT);
	checkResult(m_xConnection->Status(), *this, sal_False);

	// try it first with parsing
	if (!xCursor->Prepare(sql))
	{
		xCursor = NULL;
		// now without parsing
		xCursor = m_xConnection->CreateCursor(SDB_SNAPSHOT, SDB_FORWARDONLY | SDB_READONLY);
		checkResult(m_xConnection->Status(), *this, sal_False);
		xCursor->Prepare(sql);
		checkResult(xCursor->Status(), *this, sal_False);
	}

	Reference< XPreparedStatement >  xStmt(new OPreparedStatement(*this, xCursor, NULL));
	m_aStatements.push_back(WeakReferenceHelper(xStmt));
	return xStmt;
}

//------------------------------------------------------------------------------
Reference< XPreparedStatement >  OConnection::prepareCall(const rtl::OUString& sql) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	SdbCursorRef xCursor = m_xConnection->CreateCursor(SDB_SNAPSHOT, SDB_FORWARDONLY | SDB_READONLY);
	checkResult(m_xConnection->Status(), *this, sal_False);

	xCursor->Prepare(sql);
	checkResult(xCursor->Status(), *this, sal_False);

	Reference< XPreparedStatement >  xStmt(new OCallableStatement(*this, xCursor, NULL));
	m_aStatements.push_back(WeakReferenceHelper(xStmt));
	return xStmt;
}

//------------------------------------------------------------------------------
rtl::OUString OConnection::nativeSQL(const rtl::OUString& sql) throw( SQLException, RuntimeException )
{
	return sql;
}

//------------------------------------------------------------------------------
void OConnection::setAutoCommit(sal_Bool autoCommit) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	m_xConnection->SetAutoCommit(autoCommit);
	checkResult(m_xConnection->Status(), *this, sal_False);
}

//------------------------------------------------------------------------------
sal_Bool OConnection::getAutoCommit(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	return m_xConnection->IsAutoCommit();
}

//------------------------------------------------------------------------------
void OConnection::commit(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();
	if (!m_xConnection->CommitTrans())
		checkResult(m_xConnection->Status(), *this);
}

//------------------------------------------------------------------------------
void OConnection::rollback(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();
	if (!m_xConnection->RollbackTrans())
		checkResult(m_xConnection->Status(), *this);
}

//------------------------------------------------------------------------------
Reference< XDatabaseMetaData >  OConnection::getMetaData(void) throw( SQLException, RuntimeException )
{
	if (!m_aMetaData.getDescriptor().is())
	{
		MutexGuard aGuard(*m_pMutex);
		m_aMetaData.setDescriptor(getDescriptor());
	}
	return &((OConnection*)this)->m_aMetaData;
}

//------------------------------------------------------------------------------
void OConnection::setReadOnly(sal_Bool readOnly) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();
	m_bReadOnly = readOnly;
}

//------------------------------------------------------------------------------
sal_Bool OConnection::isReadOnly(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	return m_bReadOnly;
}

//------------------------------------------------------------------------------
void OConnection::setCatalog(const rtl::OUString& catalog) throw( SQLException, RuntimeException )
{
}

//------------------------------------------------------------------------------
rtl::OUString OConnection::getCatalog(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();
	return UniString(m_xConnection->GetQualifier());
}

//------------------------------------------------------------------------------
void OConnection::setTransactionIsolation(sal_Int32 level) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	SdbIsolationLevel aSDBIso;
	switch (level)
	{
		case TransactionIsolation::NONE:
			aSDBIso = SdbIsolationLevel(DatabaseIsolationLevel::NONE);
			break;
		case TransactionIsolation::READ_UNCOMMITTED:
			aSDBIso = SdbIsolationLevel(DatabaseIsolationLevel::READ_UNCOMMIT);
			break;
		case TransactionIsolation::READ_COMMITTED:
			aSDBIso = SdbIsolationLevel(DatabaseIsolationLevel::READ_COMMIT);
			break;
		case TransactionIsolation::REPEATABLE_READ:
			aSDBIso = SdbIsolationLevel(DatabaseIsolationLevel::REPEAT_READ);
			break;
		case TransactionIsolation::SERIALIZABLE:
			aSDBIso = SdbIsolationLevel(DatabaseIsolationLevel::SERIALIZABLE);
			break;
		default:
			throw SQLException();
	}
	m_xConnection->SetIsolationLevel(aSDBIso);
	checkResult(m_xConnection->Status(), *this, sal_False);
}

//------------------------------------------------------------------------------
sal_Int32 OConnection::getTransactionIsolation(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed || !m_xConnection.Is())
		throw DisposedException();

	sal_Int32 nLevel = TransactionIsolation::NONE;
	switch (m_xConnection->IsolationLevel())
	{
		case DatabaseIsolationLevel::NONE:
			nLevel = TransactionIsolation::NONE;
			break;
		case DatabaseIsolationLevel::READ_UNCOMMIT:
			nLevel = TransactionIsolation::READ_UNCOMMITTED;
			break;
		case DatabaseIsolationLevel::READ_COMMIT:
			nLevel = TransactionIsolation::READ_COMMITTED;
			break;
		case DatabaseIsolationLevel::REPEAT_READ:
			nLevel = TransactionIsolation::REPEATABLE_READ;
			break;
		case DatabaseIsolationLevel::SERIALIZABLE:
			nLevel = TransactionIsolation::SERIALIZABLE;
			break;
		default:
			DBG_ERROR("Unknown transaction level");
	}
	return nLevel;
}

//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XNameAccess >  OConnection::getTypeMap(void) throw( SQLException, RuntimeException )
{
	return NULL;
}

//------------------------------------------------------------------------------
void OConnection::setTypeMap(const Reference< ::com::sun::star::container::XNameAccess > & typeMap) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(*m_pMutex);
	if (OConnectionBase::rBHelper.bDisposed)
		throw DisposedException();

	throw SQLException();
}

//------------------------------------------------------------------------------
Reference< ::com::sun::star::data::XDatabaseDescriptor >  OConnection::getDescriptor() const throw( SQLException, RuntimeException )
{
	Reference< ::com::sun::star::data::XDatabaseDescriptor >  xDesc;
	{
		MutexGuard aGuard(*m_pMutex);
		if (m_xConnection.Is())
			xDesc = m_xConnection->GetMetaData();
		else
			throw DisposedException();
	}
	return xDesc;
}

//--------------------------------------------------------------------------
SdbObj* OConnection::getByName(DBObject _eObj, const rtl::OUString& _rName) throw( ::com::sun::star::container::NoSuchElementException,
																			       WrappedTargetException,
																				   RuntimeException )
{
	SdbObj* pObj = NULL;
	{
		MutexGuard aGuard( *m_pMutex );
		if (m_xConnection.Is())
			pObj = (SdbObj*)m_xConnection->OpenTable(_rName);
		else
			throw DisposedException();

		if (!pObj)
			throw ::com::sun::star::container::NoSuchElementException();
	}
	return pObj;
}

//--------------------------------------------------------------------------
SdbObj* OConnection::getByIndex(DBObject _eObj, sal_Int32 _nPos) throw( IndexOutOfBoundsException,
																	    WrappedTargetException,
																		RuntimeException )
{
	SdbObj* pObj = NULL;
	{
		MutexGuard aGuard( *m_pMutex );
		if (m_xConnection.Is())
			pObj = (SdbObj*)m_xConnection->OpenTable((sal_uInt16)_nPos);
		else
			throw DisposedException();

		if (!pObj)
			throw IndexOutOfBoundsException();
	}
	return pObj;
}

//--------------------------------------------------------------------------
void OConnection::refresh(DBObject _eObj) throw( RuntimeException )
{
	MutexGuard aGuard( *m_pMutex );
	if (m_xConnection.Is())
		m_xConnection->Refresh();
	else
		throw DisposedException();
}

//--------------------------------------------------------------------------
Sequence<rtl::OUString> OConnection::getElementNames(DBObject _eObj) throw( RuntimeException  )
{
	MutexGuard aGuard(*m_pMutex );
	sal_Int32 nCount = getCount(_eObj);

	Sequence<rtl::OUString> aNameList(nCount);
	rtl::OUString* pStringArray = aNameList.getArray();
	for (sal_uInt32 i = 0; i < nCount; i++)
		pStringArray[i] = UniString(m_xConnection->GetTableName(i));

	return aNameList;
}

//--------------------------------------------------------------------------
sal_Int32 OConnection::getCount(DBObject _eObj) throw( RuntimeException )
{
	sal_Int32 nRet = 0;
	{
		MutexGuard aGuard(*m_pMutex );
		if (m_xConnection.Is())
			nRet = m_xConnection->GetTableCount();
		else
			throw DisposedException();
	}
	return nRet;
}




