/*************************************************************************
 *
 *  $RCSfile: query.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <query.hxx>
#include <api.hrc>
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HDL_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _UNOTOOLS_CONTAINER_HXX_
#include <unotools/container.hxx>
#endif
#ifndef _UNOTOOLS_ENUMHELPER_HXX_
#include <unotools/enumhelper.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::data;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::osl;
using namespace ::cppu;


DBG_NAME(OQuery);
//--------------------------------------------------------------------------
OQuery::OQuery(OConnectionBase* pConnection, SbaQueryDef* pQuery)
	   :OQueryBase(m_aMutex)
	   ,OPropertySetHelper(OQueryBase::rBHelper)
	   ,m_aConnection(*pConnection)
	   ,m_xQuery(pQuery)
	   ,m_aColumns(*this, m_aMutex, pQuery->GetDatabase()->GetIdentifierCase() == SDB_IC_ALL)
{
	DBG_CTOR(OQuery, NULL);

	m_aName = UniString(m_xQuery->Name());
	m_aCommand = UniString(m_xQuery->Statement());
	m_bUseEscapeProcessing = !m_xQuery->IsNative();

	// inserting the columns
	const SbaColumnList& rCols = m_xQuery->GetOriginalColumns();
	sal_uInt32 nCount = rCols.Count();
	for (sal_uInt32 i = 0; i < nCount; i++)
	{
		const SbaColumn* pCol = rCols.GetObject(i);
		m_aColumns.append(new OColumn(*pCol));
	}
}

//--------------------------------------------------------------------------
OQuery::~OQuery()
{
	m_aColumns.disposing();
	DBG_DTOR(OQuery, NULL);
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OQuery::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XPropertySet > *)0 ),
							OQueryBase::getTypes() );

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OQuery::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OQuery::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OQueryBase::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XPropertySet * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OQuery::acquire() throw(RuntimeException)
{
	OQueryBase::acquire();
}

//--------------------------------------------------------------------------
void OQuery::release() throw(RuntimeException)
{
	OQueryBase::release();
}

// OComponentHelper
//------------------------------------------------------------------------------
void OQuery::disposing()
{
	OPropertySetHelper::disposing();

	MutexGuard aGuard(m_aMutex);
	m_aConnection = Reference< XInterface >();
	m_xQuery = NULL;
}

// OPropertySetHelper
//------------------------------------------------------------------------------
Reference< XPropertySetInfo > OQuery::getPropertySetInfo() throw (RuntimeException)
{
	return createPropertySetInfo( getInfoHelper() ) ;
}

// utl::OPropertyArrayUsageHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper* OQuery::createArrayHelper( ) const
{
	BEGIN_PROPERTY_HELPER(3)
		DECL_PROP1(COMMAND,					::rtl::OUString,	READONLY);
		DECL_PROP1_BOOL(USE_ESCAPE_PROCESSING,	READONLY);
		DECL_PROP1(NAME,					::rtl::OUString,	READONLY);
	END_PROPERTY_HELPER();
}

// cppu::OPropertySetHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper& OQuery::getInfoHelper()
{
	return *getArrayHelper();
}

//------------------------------------------------------------------------------
sal_Bool OQuery::convertFastPropertyValue( Any & rConvertedValue, Any & rOldValue, sal_Int32 nHandle, const Any& rValue ) throw( IllegalArgumentException  )
{
	return sal_False;
}

//------------------------------------------------------------------------------
void OQuery::setFastPropertyValue_NoBroadcast( sal_Int32 nHandle, const Any& rValue ) throw(::com::sun::star::uno::Exception)
{
}

//------------------------------------------------------------------------------
void OQuery::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROPERTY_ID_USE_ESCAPE_PROCESSING:
		{
			sal_Bool bVal = m_bUseEscapeProcessing;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_COMMAND:
			rValue <<= m_aCommand;
			break;
		case PROPERTY_ID_NAME:
			rValue <<= m_aName;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OQuery::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OQuery");
}

//------------------------------------------------------------------------------
sal_Bool OQuery::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OQuery::getSupportedServiceNames(  ) throw (RuntimeException)
{
	return Sequence< rtl::OUString > ();
}

// XColumnsSupplier
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XNameAccess >  OQuery::getColumns(void) throw( RuntimeException )
{
	return &const_cast<OQuery*>(this)->m_aColumns;
}

// Helper
//------------------------------------------------------------------------------
SdbObj* OQuery::getImplObj() const
{
	return m_xQuery;
}

//************************************************************
//  OQueries
//************************************************************
DBG_NAME(OQueries);
// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
void OQueries::acquire() throw(RuntimeException)
{
	m_rParent.acquire();
}

//--------------------------------------------------------------------------
void OQueries::release() throw(RuntimeException)
{
	m_rParent.release();
}

//------------------------------------------------------------------------------
void OQueries::disposing()
{
	m_aRefreshListeners.disposeAndClear(EventObject(static_cast<XWeak*>(this)));

	MutexGuard aGuard(m_rMutex);
	for (vector<OQuery*>::const_iterator i = m_aElements.begin(); i != m_aElements.end(); i++)
	{
		(*i)->disposing();
		(*i)->release();
	}
	m_aElements.clear();
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OQueries::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OQueries");
}

//------------------------------------------------------------------------------
sal_Bool OQueries::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OQueries::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< rtl::OUString > aSNS( 1 );
	aSNS[0] = SERVICE_SDBCX_CONTAINER;
	return aSNS;
}

// ::com::sun::star::container::XElementAccess
//------------------------------------------------------------------------------
Type OQueries::getElementType(  ) throw(RuntimeException)
{
	return::getCppuType(static_cast<Reference<XPropertySet>*>(NULL));
}


//------------------------------------------------------------------------------
sal_Bool OQueries::hasElements(void) throw( RuntimeException )
{
	return getCount() != 0;
}

// ::com::sun::star::container::XEnumerationAccess
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XEnumeration >  OQueries::createEnumeration(void) throw( RuntimeException )
{
	MutexGuard aGuard(m_rMutex);
	return new utl::OEnumerationByName( static_cast< ::com::sun::star::container::XNameAccess*>(this));
}

// ::com::sun::star::container::XIndexAccess
//------------------------------------------------------------------------------
sal_Int32 OQueries::getCount(void) throw( RuntimeException )
{
	return static_cast<OConnectionBase&>(m_rParent).getCount(dbQuery);
}

//------------------------------------------------------------------------------
Any OQueries::getByIndex(sal_Int32 Index) throw( IndexOutOfBoundsException, WrappedTargetException, RuntimeException )
{
	SdbObj* pObj = static_cast<OConnectionBase&>(m_rParent).getByIndex(dbQuery, Index);
	return makeAny(Reference< XPropertySet >(get(pObj)));
}

// ::com::sun::star::container::XNameAccess
//------------------------------------------------------------------------------
Any OQueries::getByName(const rtl::OUString& aName) throw( ::com::sun::star::container::NoSuchElementException,
														   WrappedTargetException, RuntimeException )
{
	SdbObj* pObj = static_cast<OConnectionBase&>(m_rParent).getByName(dbQuery, aName);
	return makeAny(Reference< XPropertySet >(get(pObj)));
}

//------------------------------------------------------------------------------
Sequence< rtl::OUString > OQueries::getElementNames(void) throw( RuntimeException )
{
	return static_cast<OConnectionBase&>(m_rParent).getElementNames(dbQuery);
}

//------------------------------------------------------------------------------
sal_Bool OQueries::hasByName(const rtl::OUString& aName) throw( RuntimeException )
{
	sal_Bool bHaveIt = sal_False;
	try
	{
		bHaveIt = static_cast<OConnectionBase&>(m_rParent).getByName(dbQuery, aName) != NULL;
	}
	catch (::com::sun::star::container::NoSuchElementException e)
	{
		bHaveIt = sal_False;
	}
	return bHaveIt;
}

// ::com::sun::star::util::XRefreshable
//------------------------------------------------------------------------------
void OQueries::addRefreshListener(const Reference< ::com::sun::star::util::XRefreshListener > & l) throw( RuntimeException  )
{   m_aRefreshListeners.addInterface(l); }
//------------------------------------------------------------------------------
void OQueries::removeRefreshListener(const Reference< ::com::sun::star::util::XRefreshListener > & l) throw( RuntimeException  )
{   m_aRefreshListeners.removeInterface(l); }

//------------------------------------------------------------------------------
void OQueries::refresh(void) throw( RuntimeException  )
{
	MutexGuard aGuard(m_rMutex);
	for (vector<OQuery*>::const_iterator i = m_aElements.begin(); i != m_aElements.end(); i++)
	{
		(*i)->disposing();
		(*i)->release();
	}
	m_aElements.clear();

	static_cast<OConnectionBase&>(m_rParent).refresh(dbQuery);
	EventObject aEvt(static_cast<XWeak*>(this));
	NOTIFY_LISTENERS(m_aRefreshListeners, ::com::sun::star::util::XRefreshListener, refreshed, aEvt);
}

// helper methods
//------------------------------------------------------------------------------
OQuery* OQueries::get(SdbObj* pObj) const
{
	MutexGuard aGuard(const_cast<OQueries*>(this)->m_rMutex );
	for (vector<OQuery*>::const_iterator i = m_aElements.begin(); i != m_aElements.end(); i++)
	{
		if ((*i)->getImplObj() == pObj)
			return *i;
	}

	OQuery* pQuery = create(pObj);
	pQuery->acquire();
	const_cast<OQueries*>(this)->m_aElements.push_back(pQuery);
	return pQuery;
}

//------------------------------------------------------------------------------
OQuery* OQueries::create(SdbObj* pObj) const
{
	return new OQuery(&static_cast<OConnectionBase&>(m_rParent), static_cast<SbaQueryDef*>(pObj));
}



