/*************************************************************************
 *
 *  $RCSfile: resultcolumn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <resultcolumn.hxx>
#include <api.hrc>

#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#ifndef _COM_SUN_STAR_SDBC_DATATYPE_HPP_
#include <com/sun/star/sdbc/DataType.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_COLUMNVALUE_HPP_
#include <com/sun/star/sdbc/ColumnValue.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEFIELDTYPE_HPP_
#include <com/sun/star/data/DatabaseFieldType.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HDL_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif

#ifndef _UNOTOOLS_TYPES_HXX_
#include <unotools/types.hxx>
#endif

#ifndef _SFXINTITEM_HXX //autogen
#include <svtools/intitem.hxx>
#endif

#ifndef _SFXENUMITEM_HXX //autogen
#include <svtools/eitem.hxx>
#endif

#ifndef _SFXSTRITEM_HXX //autogen
#include <svtools/stritem.hxx>
#endif

#ifndef _SDB_SDBCOL_HXX //autogen
#include <sdb/sdbcol.hxx>
#endif

#ifndef _SBA_DBDATDEF_HXX
#include <dbdatdef.hxx>
#endif

#ifndef _SBA_DBOBJENU_HXX //autogen
#include <dbobjenu.hxx>
#endif

#ifndef _SBAITEMS_HXX
#include "sbaitems.hxx"
#endif

#ifndef _SBAITEMS_HRC
#include "sbaitems.hrc"
#endif

using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::data;

//--------------------------------------------------------------------------
OResultColumn::OResultColumn(const SdbColumn& rCol)
			  :OColumn(rCol)
{
	construct(rCol);
}

//--------------------------------------------------------------------------
OResultColumn::OResultColumn(const SbaColumn& rCol)
			  :OColumn(rCol)
{
	construct(rCol);
}

//--------------------------------------------------------------------------
OResultColumn::OResultColumn(const Reference< XPropertySet > & rCol)
	  		  :OColumn()
{
	construct(rCol);
}

//------------------------------------------------------------------------------
void OResultColumn::construct()
{
	m_bSearchable = m_bSigned = m_bCurrency = m_bReadOnly = sal_False;
	m_nDisplaySize = 0;
}

//------------------------------------------------------------------------------
void OResultColumn::construct(const Reference< XPropertySet > & xSet)
{
	construct();
	if (xSet.is())
	{
		xSet->getPropertyValue(PROPERTY_NAME) >>= m_aName;
		m_bSearchable = utl::getBOOL(xSet->getPropertyValue(PROPERTY_ISSEARCHABLE));
		xSet->getPropertyValue(PROPERTY_TYPE) >>= m_nType;
		xSet->getPropertyValue(PROPERTY_SCALE) >>= m_nScale;
		xSet->getPropertyValue(PROPERTY_PRECISION) >>= m_nPrecision;
		xSet->getPropertyValue(PROPERTY_NUMBERFORMAT) >>= m_nNumberFormat;
	}
}

//------------------------------------------------------------------------------
void OResultColumn::construct(const SdbColumn& rCol)
{
	if (((rCol.GetAllFlags() & SDB_FLAGS_SEARCH_ALL) == SDB_FLAGS_SEARCH_ALL) ||
	    ((rCol.GetAllFlags() & SDB_FLAGS_SEARCH_LIKE) == SDB_FLAGS_SEARCH_LIKE) ||
	    ((rCol.GetAllFlags() & SDB_FLAGS_SEARCH_EXCEPT_LIKE) == SDB_FLAGS_SEARCH_EXCEPT_LIKE))
		m_bSearchable = sal_True;
	else
		m_bSearchable = sal_False;

	m_bSigned	= sal_False;
	m_bCurrency = rCol.GetType() == DatabaseFieldType::CURRENCY;
	m_bReadOnly = !rCol.IsUpdateAllowed() || m_bAutoIncrement;
	m_aLabel	= m_aName;
	m_nDisplaySize = m_nPrecision;
}

//------------------------------------------------------------------------------
void OResultColumn::construct(const SbaColumn& rCol)
{
	m_nDisplaySize = m_nPrecision;
	m_aLabel	= m_aName;
	m_bSigned	= sal_False;

	SFX_ITEMSET_GET(rCol, pType, SbaDataFieldTypeItem, SBA_DEF_FLTTYPE, sal_True);
	m_bCurrency = pType->GetValue() == dbCurrency;

	SFX_ITEMSET_GET(rCol, pUpd, SfxBoolItem, SBA_DEF_FLTUPDATABLE, sal_True);
	m_bReadOnly = !pUpd->GetValue() || m_bAutoIncrement;

	SFX_ITEMSET_GET(rCol, pSearch, SfxUInt16Item, SBA_DEF_FLTSEARCH, sal_True);
	switch (pSearch->GetValue() & SDB_FLAGS_SEARCH_ALL)
	{
		case SDB_FLAGS_SEARCH_ALL:
		case SDB_FLAGS_SEARCH_LIKE:
		case SDB_FLAGS_SEARCH_EXCEPT_LIKE:
			m_bSearchable = sal_True;
			break;
		default:
			m_bSearchable = sal_False;
	}
}

// utl::OPropertyArrayUsageHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper* OResultColumn::createArrayHelper( ) const
{
	BEGIN_PROPERTY_HELPER(24)
		DECL_PROP1(CATALOGNAME,				::rtl::OUString,	READONLY);
		DECL_PROP1(DEFAULTVALUE,			::rtl::OUString,	READONLY);
		DECL_PROP1(DESCRIPTION,				::rtl::OUString,	READONLY);		
		DECL_PROP1(DISPLAYSIZE,				sal_Int32,		READONLY);
		DECL_PROP1(NUMBERFORMAT,			sal_Int32,		READONLY);  //FormatKey
		DECL_PROP1_BOOL(ISAUTOINCREMENT,			READONLY);
		DECL_PROP1_BOOL(ISCASESENSITIVE,			READONLY);
		DECL_PROP1_BOOL(ISCURRENCY,				READONLY);
		DECL_PROP1_BOOL(ISDEFINITELYWRITABLE,	READONLY);
		DECL_PROP1(ISNULLABLE,				sal_Int32,		READONLY);
		DECL_PROP1_BOOL(ISREADONLY,				READONLY);
		DECL_PROP1_BOOL(ISROWVERSION,			READONLY);
		DECL_PROP1_BOOL(ISSEARCHABLE,			READONLY);
		DECL_PROP1_BOOL(ISSIGNED,				READONLY);
		DECL_PROP1_BOOL(ISWRITABLE,				READONLY);
		DECL_PROP1(LABEL,					::rtl::OUString,	READONLY);
		DECL_PROP1(NAME,					::rtl::OUString,	READONLY);		
		DECL_PROP1(PRECISION,				sal_Int32,		READONLY);
		DECL_PROP1(SCALE,					sal_Int32,		READONLY);
		DECL_PROP1(SCHEMANAME,				::rtl::OUString,	READONLY);
		DECL_PROP1(SERVICENAME,				::rtl::OUString,	READONLY);
		DECL_PROP1(TABLENAME,				::rtl::OUString,	READONLY);
		DECL_PROP1(TYPE,					sal_Int32,		READONLY);
		DECL_PROP1(TYPENAME,				::rtl::OUString,	READONLY);
	END_PROPERTY_HELPER();
}
	
// cppu::OPropertySetHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper& OResultColumn::getInfoHelper()
{
	return *static_cast< ::utl::OPropertyArrayUsageHelper<OResultColumn>*>(this)->getArrayHelper();
}

//------------------------------------------------------------------------------
void OResultColumn::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROPERTY_ID_TABLENAME:
			rValue <<= m_aTableName;
			break;
		case PROPERTY_ID_SCHEMANAME:
			rValue <<= m_aSchemaName;
			break;
		case PROPERTY_ID_CATALOGNAME:
			rValue <<= m_aCatalogName;
			break;
		case PROPERTY_ID_ISSIGNED:
		{
			sal_Bool bVal = m_bSigned;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_ISCURRENCY:
		{
			sal_Bool bVal = m_bCurrency;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_ISSEARCHABLE:
		{
			sal_Bool bVal = m_bSearchable;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_ISCASESENSITIVE:
		{
			sal_Bool bVal = sal_True;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_ISREADONLY:
		{
			sal_Bool bVal = m_bReadOnly;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_SERVICENAME:
			rValue <<= rtl::OUString();
			break;
		case PROPERTY_ID_ISWRITABLE:
		case PROPERTY_ID_ISDEFINITELYWRITABLE:
		{
			sal_Bool bVal = !m_bReadOnly;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_LABEL:
			rValue <<= m_aLabel;
			break;
		case PROPERTY_ID_DISPLAYSIZE:
			rValue <<= m_nDisplaySize;
			break;
		default:
			OColumn::getFastPropertyValue(rValue, nHandle);
	}
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OResultColumn::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OResultColumn");
}



