/*************************************************************************
 *
 *  $RCSfile: resultset.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <resultset.hxx>
#include <statement.hxx>
#include <api.hrc>

#ifndef _COM_SUN_STAR_SDBC_FETCHDIRECTION_HPP_
#include <com/sun/star/sdbc/FetchDirection.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_RESULTSETCONCURRENCY_HPP_
#include <com/sun/star/sdbc/ResultSetConcurrency.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_RESULTSETTYPE_HPP_
#include <com/sun/star/sdbc/ResultSetType.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBCX_COMPAREBOOKMARK_HPP_
#include <com/sun/star/sdbcx/CompareBookmark.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HDL_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _UNOTOOLS_PROPERTY_HXX_
#include <unotools/property.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::sdbcx;
using namespace ::com::sun::star::sdb;
using namespace ::com::sun::star::data;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::cppu;
using namespace ::osl;

DBG_NAME(OResultSet);

//--------------------------------------------------------------------------
OResultSet::OResultSet(OStatementBase& rStmt, const SdbCursorRef& xCursor)
		   :OResultSetBase(m_aMutex)
		   ,OPropertySetHelper(OResultSetBase::rBHelper)
		   ,m_aColumns(*this, m_aMutex, xCursor->GetConnection()->GetIdentifierCase() == SDB_IC_ALL)
		   ,m_xCursor(xCursor)
		   ,m_nFetchDirection(FetchDirection::FORWARD)
		   ,m_nFetchSize(1)
		   ,m_nResultSetType(rStmt.m_nResultSetType)
		   ,m_bCanBookmark(rStmt.m_nResultSetType != ResultSetType::FORWARD_ONLY)
		   ,m_aStatement(static_cast<XWeak*>(&rStmt))
{
	DBG_CTOR(OResultSet, NULL);

	if (m_xCursor->IsUpdatable() && rStmt.m_nResultSetConcurrency == ResultSetConcurrency::UPDATABLE)
		m_nResultSetConcurrency = ResultSetConcurrency::UPDATABLE;
	else
		m_nResultSetConcurrency = ResultSetConcurrency::READ_ONLY;

	m_aColumns.insertColumns(xCursor->GetColumns(), ResultSetConcurrency::READ_ONLY == m_nResultSetConcurrency, this);
	m_aColumns.setRow(xCursor->GetRow(), sal_False);
}

//--------------------------------------------------------------------------
OResultSet::~OResultSet()
{
	DBG_DTOR(OResultSet, NULL);
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OResultSet::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XPropertySet > *)0 ),
						   OResultSetBase::getTypes());

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OResultSet::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OResultSet::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OResultSetBase::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XPropertySet * >( this ));

	return aIface;
}

//--------------------------------------------------------------------------
void OResultSet::acquire() throw (RuntimeException)
{
	OResultSetBase::acquire();
}

//--------------------------------------------------------------------------
void OResultSet::release() throw (RuntimeException)
{
	OResultSetBase::release();
}


// OResultSetBase
//------------------------------------------------------------------------------
void OResultSet::disposing()
{
	OPropertySetHelper::disposing();

	MutexGuard aGuard(m_aMutex);
	m_aColumns.disposing();
	m_xCursor = NULL;
	m_aStatement = NULL;
}

// XCloseable
//------------------------------------------------------------------------------
void OResultSet::close(void) throw( SQLException, RuntimeException )
{
	{
		MutexGuard aGuard( m_aMutex );
		if (OResultSetBase::rBHelper.bDisposed)
			throw DisposedException();
	}
	dispose();
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OResultSet::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OResultSet");
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OResultSet::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< ::rtl::OUString > aSNS( 2 );
	aSNS[0] = SERVICE_SDBC_RESULTSET;
	aSNS[1] = SERVICE_SDB_RESULTSET;
	return aSNS;
}

// com::sun::star::beans::XPropertySet
//------------------------------------------------------------------------------
Reference< XPropertySetInfo > OResultSet::getPropertySetInfo() throw (RuntimeException)
{
	return createPropertySetInfo( getInfoHelper() ) ;
}

// utl::OPropertyArrayUsageHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper* OResultSet::createArrayHelper( ) const
{
	BEGIN_PROPERTY_HELPER(6)
		DECL_PROP1(CURSORNAME,				::rtl::OUString,	READONLY);
		DECL_PROP0(FETCHDIRECTION,			sal_Int32);
		DECL_PROP0(FETCHSIZE,				sal_Int32);
		DECL_PROP1_BOOL(ISBOOKMARKABLE,			READONLY);
		DECL_PROP1(RESULTSETCONCURRENCY,	sal_Int32,		READONLY);
		DECL_PROP1(RESULTSETTYPE,			sal_Int32,		READONLY);
	END_PROPERTY_HELPER();
}

// cppu::OPropertySetHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper& OResultSet::getInfoHelper()
{
	return *getArrayHelper();
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::convertFastPropertyValue(Any & rConvertedValue, Any & rOldValue, sal_Int32 nHandle, const Any& rValue ) throw( IllegalArgumentException  )
{
	sal_Bool bModified(sal_False);
	switch (nHandle)
	{
		case PROPERTY_ID_FETCHDIRECTION:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nFetchDirection);
			if (bModified)
			{
				sal_Int32 nNew;
				rConvertedValue >>= nNew;
				if (nNew != FetchDirection::FORWARD &&
					nNew != FetchDirection::REVERSE && nNew != FetchDirection::UNKNOWN)
					throw IllegalArgumentException();
			}
			break;
		case PROPERTY_ID_FETCHSIZE:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nFetchSize);
			break;
		default:
			DBG_ERROR("unknown Property");
	}
	return bModified;
}

//------------------------------------------------------------------------------
void OResultSet::setFastPropertyValue_NoBroadcast( sal_Int32 nHandle, const Any& rValue ) throw (Exception)
{
	switch (nHandle)
	{
		case PROPERTY_ID_FETCHDIRECTION:
			rValue >>= m_nFetchDirection;
			break;
		case PROPERTY_ID_FETCHSIZE:
			rValue >>= m_nFetchSize;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

//------------------------------------------------------------------------------
void OResultSet::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROPERTY_ID_ISBOOKMARKABLE:
		{
			sal_Bool bVal = m_bCanBookmark;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_FETCHDIRECTION:
			rValue <<= m_nFetchDirection;
			break;
		case PROPERTY_ID_FETCHSIZE:
			rValue <<= m_nFetchSize;
			break;
		case PROPERTY_ID_RESULTSETCONCURRENCY:
			rValue <<= m_nResultSetConcurrency;
			break;
		case PROPERTY_ID_RESULTSETTYPE:
			rValue <<= m_nResultSetType;
			break;
		case PROPERTY_ID_CURSORNAME:
			rValue <<= m_aCursorName;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

// XWarningsSupplier
//------------------------------------------------------------------------------
Any OResultSet::getWarnings(void) throw( SQLException, RuntimeException )
{
	return Any();
}

//------------------------------------------------------------------------------
void OResultSet::clearWarnings(void) throw( SQLException, RuntimeException )
{
}

// XResultSetMetaDataSupplier
//------------------------------------------------------------------------------
Reference< XResultSetMetaData >  OResultSet::getMetaData(void) throw( SQLException, RuntimeException )
{
	return &const_cast<OResultSet*>(this)->m_aColumns;
}

// XColumnLocate
//------------------------------------------------------------------------------
sal_Int32 OResultSet::findColumn(const rtl::OUString& columnName) throw( SQLException, RuntimeException )
{
	return m_aColumns.findColumn(columnName);
}

// XColumnsSupplier
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XNameAccess >  OResultSet::getColumns(void) throw( RuntimeException )
{
	return &const_cast<OResultSet*>(this)->m_aColumns;
}

// XRow
//------------------------------------------------------------------------------
sal_Bool OResultSet::wasNull(void) throw( SQLException, RuntimeException )
{
	return m_aColumns.wasNull();
}
//------------------------------------------------------------------------------
rtl::OUString OResultSet::getString(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getString(columnIndex);
}
//------------------------------------------------------------------------------
sal_Bool OResultSet::getBoolean(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getBoolean(columnIndex);
}
//------------------------------------------------------------------------------
sal_Int8 OResultSet::getByte(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getByte(columnIndex);
}
//------------------------------------------------------------------------------
sal_Int16 OResultSet::getShort(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getShort(columnIndex);
}
//------------------------------------------------------------------------------
sal_Int32 OResultSet::getInt(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getInt(columnIndex);
}
//------------------------------------------------------------------------------
sal_Int64 OResultSet::getLong(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getLong(columnIndex);
}
//------------------------------------------------------------------------------
float OResultSet::getFloat(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getFloat(columnIndex);
}
//------------------------------------------------------------------------------
double OResultSet::getDouble(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getDouble(columnIndex);
}
//------------------------------------------------------------------------------
Sequence< sal_Int8 > OResultSet::getBytes(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getBytes(columnIndex);
}
//------------------------------------------------------------------------------
::com::sun::star::util::Date OResultSet::getDate(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getDate(columnIndex);
}
//------------------------------------------------------------------------------
::com::sun::star::util::Time OResultSet::getTime(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getTime(columnIndex);
}
//------------------------------------------------------------------------------
::com::sun::star::util::DateTime OResultSet::getTimestamp(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getTimestamp(columnIndex);
}
//------------------------------------------------------------------------------
Reference< ::com::sun::star::io::XInputStream >  OResultSet::getBinaryStream(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getBinaryStream(columnIndex);
}
//------------------------------------------------------------------------------
Reference< ::com::sun::star::io::XInputStream >  OResultSet::getCharacterStream(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getCharacterStream(columnIndex);
}
//------------------------------------------------------------------------------
Any OResultSet::getObject(sal_Int32 columnIndex, const Reference< ::com::sun::star::container::XNameAccess > & typeMap) throw( SQLException, RuntimeException )
{
	return m_aColumns.getObject(columnIndex, typeMap);
}
//------------------------------------------------------------------------------
Reference< XRef >  OResultSet::getRef(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getRef(columnIndex);
}
//------------------------------------------------------------------------------
Reference< XBlob >  OResultSet::getBlob(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getBlob(columnIndex);
}
//------------------------------------------------------------------------------
Reference< XClob >  OResultSet::getClob(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getClob(columnIndex);
}
//------------------------------------------------------------------------------
Reference< XArray >  OResultSet::getArray(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	return m_aColumns.getArray(columnIndex);
}

// XResultSet
//------------------------------------------------------------------------------
sal_Bool OResultSet::next(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	sal_Bool bResult = m_xCursor->Next();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	Reference<XInterface> xIFace(*this);
	checkResult(m_xCursor->Status(), xIFace, sal_False);
	return bResult;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::isBeforeFirst(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	return m_xCursor->IsBOF();
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::isAfterLast(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	return m_xCursor->IsEOF();
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::isFirst(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	return m_xCursor->IsOnFirst();
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::isLast(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	return m_xCursor->IsOnLast();
}

//------------------------------------------------------------------------------
void OResultSet::beforeFirst(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
		throw SQLException();

	m_xCursor->First();

	if (m_xCursor->IsOnFirst() || m_xCursor->IsEOF())
	{
		m_xCursor->Prev();
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
	else if (!m_xCursor->IsBOF())
	{
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
	else
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
}

//------------------------------------------------------------------------------
void OResultSet::afterLast(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
		throw SQLException();

	m_xCursor->Last();

	if (m_xCursor->IsOnLast() || m_xCursor->IsBOF())
	{
		m_xCursor->Next();
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
	else if (!m_xCursor->IsEOF())
	{
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::first(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
		throw SQLException();

	sal_Bool bResult = m_xCursor->First();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);

	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	return bResult;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::last(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
		throw SQLException();

	sal_Bool bResult = m_xCursor->Last();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);

	return bResult;
}

//------------------------------------------------------------------------------
sal_Int32 OResultSet::getRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_xCursor->IsBOF())
		return 0;
	else if (m_xCursor->IsEOF())
		return m_xCursor->RowCount() + 1;
	else
		return m_xCursor->GetCurrentPos();
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::absolute(sal_Int32 row) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
		throw SQLException();

	sal_Bool bResult = sal_False;
	if (row == 0)
		beforeFirst();
	else if (row > 0)
	{
		bResult = m_xCursor->MoveAbs(row);
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
	else
	{
		afterLast();
		bResult = m_xCursor->MoveRel(row);
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
	return bResult;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::relative(sal_Int32 rows) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY || m_xCursor->IsBOF() || m_xCursor->IsEOF())
		throw SQLException();

	sal_Bool bResult = m_xCursor->MoveRel(rows);
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	return bResult;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::previous(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
		throw SQLException();

	sal_Bool bResult = m_xCursor->Prev();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	return bResult;
}

//------------------------------------------------------------------------------
void OResultSet::refreshRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();
	else if (m_nResultSetType == ResultSetType::FORWARD_ONLY ||
			 m_nResultSetConcurrency == ResultSetConcurrency::READ_ONLY)
		throw SQLException();
	else if (!m_xCursor->IsValid() || m_xCursor->IsModeNew())
		throw SQLException();

	m_xCursor->MoveRel(0);
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::rowUpdated(void) throw( SQLException, RuntimeException )
{
	return sal_False;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::rowInserted(void) throw( SQLException, RuntimeException )
{
	return sal_False;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::rowDeleted(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	return m_xCursor->GetRow()->isDeleted() || m_xCursor->GetRow()->isObsolete();
}

//------------------------------------------------------------------------------
Reference< XInterface >  OResultSet::getStatement(void) throw( SQLException, RuntimeException )
{
	return m_aStatement.get();
}

//------------------------------------------------------------------------------
Any OResultSet::getBookmark(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (!m_bCanBookmark)
		throw SQLException();

	if (!m_xCursor->IsValid())
		return Any();
	else
		return (*m_xCursor->GetRow())[0]->getAny();
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::moveToBookmark(const Any& bookmark) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (!m_bCanBookmark)
		throw SQLException();

	sal_Bool bResult = m_xCursor->MoveBookmark(bookmark);
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	if (SDB_CURRENT_ROW_UNDEFINED == m_xCursor->GetCurrentPos())
	{
		SdbStatus aStatus = m_xCursor->Status();
		try
		{
			afterLast();
		}
		catch (...)
		{
		}
		checkResult(aStatus, static_cast<XWeak*>(this), sal_False);
	}
	return bResult;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::moveRelativeToBookmark(const Any& bookmark, sal_Int32 rows) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	return moveToBookmark(bookmark) && relative(rows);
}

//------------------------------------------------------------------------------
sal_Int32 OResultSet::compareBookmarks(const Any& first, const Any& second) throw( SQLException, RuntimeException )
{
	if (!m_bCanBookmark)
		throw SQLException();

	return OBookmarkEqual()(first, second) ? CompareBookmark::EQUAL : CompareBookmark::NOT_EQUAL;
}

//------------------------------------------------------------------------------
sal_Bool OResultSet::hasOrderedBookmarks(void) throw( SQLException, RuntimeException )
{
	if (!m_bCanBookmark)
		throw SQLException();

	return sal_False;
}

//------------------------------------------------------------------------------
sal_Int32 OResultSet::hashBookmark(const Any& bookmark) throw( SQLException, RuntimeException )
{
	if (!m_bCanBookmark)
		throw SQLException();

	return OBookmarkHash()(bookmark);
}

// IValueListener
//------------------------------------------------------------------------------
void OResultSet::valueWillChange(const ODataColumn&)
{
	MutexGuard aGuard( m_aMutex );
	if (m_xCursor.Is() && !m_xCursor->IsModeEdit() && m_xCursor->IsValid() && !m_xCursor->IsModeNew())
	{
		// start editing and copy the current row
		m_xCursor->EditRow();
		m_aColumns.setRow(m_xCursor->GetRow(), sal_False);
	}
}

//------------------------------------------------------------------------------
void OResultSet::valueHasChanged(const ODataColumn&)
{
}

//************************************************************
//  OUpdateResultSet
//************************************************************
//--------------------------------------------------------------------------
OUpdateResultSet::OUpdateResultSet(OStatementBase& rStmt, const SdbCursorRef& xCursor)
		         :OResultSet(rStmt, xCursor)
{
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OUpdateResultSet::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XResultSetUpdate > *)0 ),
						   ::getCppuType( (const Reference< XRowUpdate > *)0 ),
							OResultSet::getTypes() );

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OUpdateResultSet::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OUpdateResultSet::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OResultSet::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XRowUpdate * >( this ),
					static_cast< XResultSetUpdate * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OUpdateResultSet::acquire() throw (RuntimeException)
{
	OResultSet::acquire();
}

//--------------------------------------------------------------------------
void OUpdateResultSet::release() throw (RuntimeException)
{
	OResultSet::release();
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OUpdateResultSet::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OUpdateResultSet");
}

// XResultSetUpdate
//------------------------------------------------------------------------------
void OUpdateResultSet::insertRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (!m_xCursor->IsModeNew())
		throw SQLException();

	m_xCursor->UpdateRow();
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);

	m_xCursor->AppendRow();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
}

//------------------------------------------------------------------------------
void OUpdateResultSet::updateRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_xCursor->IsModeNew())
		throw SQLException();

	m_xCursor->UpdateRow();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
}

//------------------------------------------------------------------------------
void OUpdateResultSet::deleteRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	m_xCursor->DeleteRow();
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
}

//------------------------------------------------------------------------------
void OUpdateResultSet::cancelRowUpdates(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_xCursor->IsModeNew())
		throw SQLException();
	else if (m_xCursor->IsModeEdit())
	{
		m_xCursor->CancelRow();
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
}

//------------------------------------------------------------------------------
void OUpdateResultSet::moveToInsertRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	m_xCursor->AppendRow();
	m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
	checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
}

//------------------------------------------------------------------------------
void OUpdateResultSet::moveToCurrentRow(void) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OResultSetBase::rBHelper.bDisposed)
		throw DisposedException();

	if (m_xCursor->IsModeNew())
	{
		m_xCursor->CancelRow();
		m_aColumns.setRow(m_xCursor->GetRow(), sal_True);
		checkResult(m_xCursor->Status(), static_cast<XWeak*>(this), sal_False);
	}
}

// XRowUpdate
//------------------------------------------------------------------------------
void OUpdateResultSet::updateNull(sal_Int32 columnIndex) throw( SQLException, RuntimeException )
{
	m_aColumns.updateNull(columnIndex);
}

//------------------------------------------------------------------------------
void OUpdateResultSet::updateBoolean(sal_Int32 columnIndex, sal_Bool x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateBoolean(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateByte(sal_Int32 columnIndex, sal_Int8 x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateByte(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateShort(sal_Int32 columnIndex, sal_Int16 x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateShort(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateInt(sal_Int32 columnIndex, sal_Int32 x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateInt(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateLong(sal_Int32 columnIndex, sal_Int64 x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateLong(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateFloat(sal_Int32 columnIndex, float x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateFloat(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateDouble(sal_Int32 columnIndex, double x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateDouble(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateString(sal_Int32 columnIndex, const rtl::OUString& x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateString(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateBytes(sal_Int32 columnIndex, const Sequence< sal_Int8 >& x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateBytes(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateDate(sal_Int32 columnIndex, const ::com::sun::star::util::Date& x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateDate(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateTime(sal_Int32 columnIndex, const ::com::sun::star::util::Time& x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateTime(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateTimestamp(sal_Int32 columnIndex, const ::com::sun::star::util::DateTime& x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateTimestamp(columnIndex, x);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateBinaryStream(sal_Int32 columnIndex, const Reference< ::com::sun::star::io::XInputStream > & x, sal_Int32 length) throw( SQLException, RuntimeException )
{
	m_aColumns.updateBinaryStream(columnIndex, x, length);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateCharacterStream(sal_Int32 columnIndex, const Reference< ::com::sun::star::io::XInputStream > & x, sal_Int32 length) throw( SQLException, RuntimeException )
{
	m_aColumns.updateCharacterStream(columnIndex, x, length);
}
//------------------------------------------------------------------------------
void OUpdateResultSet::updateNumericObject(sal_Int32 columnIndex, const Any& x, sal_Int32 scale) throw( SQLException, RuntimeException )
{
	m_aColumns.updateNumericObject(columnIndex, x, scale);
}

//------------------------------------------------------------------------------
void OUpdateResultSet::updateObject(sal_Int32 columnIndex, const Any& x) throw( SQLException, RuntimeException )
{
	m_aColumns.updateObject(columnIndex, x);
}



