/*************************************************************************
 *
 *  $RCSfile: statement.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <statement.hxx>
#ifndef _COM_SUN_STAR_SDBC_FETCHDIRECTION_HPP_
#include <com/sun/star/sdbc/FetchDirection.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_RESULTSETCONCURRENCY_HPP_
#include <com/sun/star/sdbc/ResultSetConcurrency.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_RESULTSETTYPE_HPP_
#include <com/sun/star/sdbc/ResultSetType.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HDL_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _UNOTOOLS_PROPERTY_HXX_
#include <unotools/property.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#include <dataaccessconnection.hxx>
#include <resultset.hxx>
#include <api.hrc>

using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::sdbcx;
using namespace ::com::sun::star::sdb;
using namespace ::com::sun::star::data;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::cppu;
using namespace ::osl;

DBG_NAME(OStatementBase);

//--------------------------------------------------------------------------
OStatementBase::OStatementBase(const Reference< XInterface > & xPar, const SdbCursorRef& xCur, ODatabaseAccessConnection* pCon)
			   :OSubComponent(m_aMutex, xPar)
			   ,OPropertySetHelper(OComponentHelper::rBHelper)
			   ,m_xCursor(xCur)
			   ,m_bUseEscapeProcessing(sal_True)
			   ,m_nFetchDirection(FetchDirection::FORWARD)
			   ,m_nFetchSize(1)
			   ,m_nMaxFieldSize(0)
			   ,m_nMaxRows(0)
			   ,m_nQueryTimeOut(0)
			   ,m_nResultSetConcurrency(ResultSetConcurrency::READ_ONLY)
			   ,m_nResultSetType(ResultSetType::FORWARD_ONLY)
			   ,m_bUseBookmarks(sal_False)
			   ,m_pDBConnection(pCon)
{
	DBG_CTOR(OStatementBase, NULL);

	// Do we need more informations about the fields used?
	if (m_pDBConnection)
		m_pDBConnection->acquire();
}

//--------------------------------------------------------------------------
OStatementBase::~OStatementBase()
{
	DBG_DTOR(OStatementBase, NULL);
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OStatementBase::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XPropertySet > *)0 ),
						   ::getCppuType( (const Reference< XWarningsSupplier > *)0 ),
						   ::getCppuType( (const Reference< XCloseable > *)0 ),
						   ::getCppuType( (const Reference< ::com::sun::star::util::XCancellable > *)0 ),
							OSubComponent::getTypes() );

	return aTypes.getTypes();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OStatementBase::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OSubComponent::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XPropertySet * >( this ),
					static_cast< XWarningsSupplier * >( this ),
					static_cast< XCloseable * >( this ),
					static_cast< ::com::sun::star::util::XCancellable * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OStatementBase::acquire() throw (RuntimeException)
{
	OSubComponent::acquire();
}

//--------------------------------------------------------------------------
void OStatementBase::release() throw (RuntimeException)
{
	OSubComponent::release();
}

//------------------------------------------------------------------------------
void OStatementBase::disposeResultSet()
{
	// free the cursor if alive
	Reference< XComponent > xComp(m_aResultSet.get(), UNO_QUERY);
	if (xComp.is())
		xComp->dispose();
	m_aResultSet = NULL;
}

// OComponentHelper
//------------------------------------------------------------------------------
void OStatementBase::disposing()
{
	OPropertySetHelper::disposing();

	MutexGuard aGuard(m_aMutex);
	disposeResultSet();
	if (m_pDBConnection)
		m_pDBConnection->release();

	m_xCursor = NULL;
	OSubComponent::disposing();
}

// XCloseable
//------------------------------------------------------------------------------
void OStatementBase::close(void) throw( SQLException, RuntimeException )
{
	{
		MutexGuard aGuard( m_aMutex );
		if (OComponentHelper::rBHelper.bDisposed)
			throw DisposedException();
	}
	dispose();
}

// OPropertySetHelper
//------------------------------------------------------------------------------
Reference< XPropertySetInfo > OStatementBase::getPropertySetInfo() throw (RuntimeException)
{
	return createPropertySetInfo( getInfoHelper() ) ;
}

// utl::OPropertyArrayUsageHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper* OStatementBase::createArrayHelper( ) const
{
	BEGIN_PROPERTY_HELPER(10)
		DECL_PROP0(CURSORNAME,				::rtl::OUString);
		DECL_PROP0_BOOL(USE_ESCAPE_PROCESSING);
		DECL_PROP0(FETCHDIRECTION,			sal_Int32);
		DECL_PROP0(FETCHSIZE,				sal_Int32);
		DECL_PROP0(MAXFIELDSIZE,			sal_Int32);
		DECL_PROP0(MAXROWS,					sal_Int32);
		DECL_PROP0(QUERYTIMEOUT,			sal_Int32);
		DECL_PROP0(RESULTSETCONCURRENCY,	sal_Int32);
		DECL_PROP0(RESULTSETTYPE,			sal_Int32);
		DECL_PROP0_BOOL(USEBOOKMARKS);
	END_PROPERTY_HELPER();
}

// cppu::OPropertySetHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper& OStatementBase::getInfoHelper()
{
	return *getArrayHelper();
}

//------------------------------------------------------------------------------
sal_Bool OStatementBase::convertFastPropertyValue( Any & rConvertedValue, Any & rOldValue, sal_Int32 nHandle, const Any& rValue ) throw( IllegalArgumentException  )
{
	sal_Bool bModified(sal_False);
	switch (nHandle)
	{
		case PROPERTY_ID_USE_ESCAPE_PROCESSING:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_bUseEscapeProcessing);
			break;
		case PROPERTY_ID_USEBOOKMARKS:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_bUseBookmarks);
			break;
		case PROPERTY_ID_FETCHDIRECTION:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nFetchDirection);
			if (bModified)
			{
				sal_Int32 nNew;
				rConvertedValue >>= nNew;
				if (nNew != FetchDirection::FORWARD &&
					nNew != FetchDirection::REVERSE && nNew != FetchDirection::UNKNOWN)
					throw IllegalArgumentException();
			}
			break;
		case PROPERTY_ID_FETCHSIZE:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nFetchSize);
			break;
		case PROPERTY_ID_MAXFIELDSIZE:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nMaxFieldSize);
			break;
		case PROPERTY_ID_MAXROWS:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nMaxRows);
			break;
		case PROPERTY_ID_QUERYTIMEOUT:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nQueryTimeOut);
			break;
		case PROPERTY_ID_RESULTSETCONCURRENCY:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nResultSetConcurrency);
			if (bModified)
			{
				sal_Int32 nNew;
				rConvertedValue >>= nNew;
				if (nNew != ResultSetConcurrency::READ_ONLY &&
					nNew != ResultSetConcurrency::UPDATABLE)
					throw IllegalArgumentException();
			}
			break;
		case PROPERTY_ID_RESULTSETTYPE:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_nResultSetType);
			if (bModified)
			{
				sal_Int32 nNew;
				rConvertedValue >>= nNew;
				if (nNew != ResultSetType::FORWARD_ONLY &&
					nNew != ResultSetType::SCROLL_INSENSITIVE && nNew != ResultSetType::SCROLL_SENSITIVE)
					throw IllegalArgumentException();
			}
			break;
		case PROPERTY_ID_CURSORNAME:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue, m_aCursorName);
			break;
		default:
			DBG_ERROR("unknown Property");
	}
	return bModified;
}

//------------------------------------------------------------------------------
void OStatementBase::setFastPropertyValue_NoBroadcast( sal_Int32 nHandle, const Any& rValue ) throw (Exception)
{
	switch (nHandle)
	{
		case PROPERTY_ID_USE_ESCAPE_PROCESSING:
		{
			sal_Bool bVal;
			rValue >>= bVal;
			m_bUseEscapeProcessing = bVal;
		}	break;
		case PROPERTY_ID_USEBOOKMARKS:
		{
			sal_Bool bVal;
			rValue >>= bVal;
			m_bUseBookmarks = bVal;
		}	break;
		case PROPERTY_ID_FETCHDIRECTION:
			rValue >>= m_nFetchDirection;
			break;
		case PROPERTY_ID_FETCHSIZE:
			rValue >>= m_nFetchSize;
			break;
		case PROPERTY_ID_MAXFIELDSIZE:
			rValue >>= m_nMaxFieldSize;
			break;
		case PROPERTY_ID_MAXROWS:
			rValue >>= m_nMaxRows;
			break;
		case PROPERTY_ID_QUERYTIMEOUT:
			rValue >>= m_nQueryTimeOut;
			break;
		case PROPERTY_ID_RESULTSETCONCURRENCY:
			rValue >>= m_nResultSetConcurrency;
			break;
		case PROPERTY_ID_RESULTSETTYPE:
			rValue >>= m_nResultSetType;
			break;
		case PROPERTY_ID_CURSORNAME:
			rValue >>= m_aCursorName;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

//------------------------------------------------------------------------------
void OStatementBase::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROPERTY_ID_USE_ESCAPE_PROCESSING:
		{
			sal_Bool bVal = m_bUseEscapeProcessing;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_USEBOOKMARKS:
		{
			sal_Bool bVal = m_bUseBookmarks;
			rValue.setValue(&bVal, getBooleanCppuType());
		}	break;
		case PROPERTY_ID_FETCHDIRECTION:
			rValue <<= m_nFetchDirection;
			break;
		case PROPERTY_ID_FETCHSIZE:
			rValue <<= m_nFetchSize;
			break;
		case PROPERTY_ID_MAXFIELDSIZE:
			rValue <<= m_nMaxFieldSize;
			break;
		case PROPERTY_ID_MAXROWS:
			rValue <<= m_nMaxRows;
			break;
		case PROPERTY_ID_QUERYTIMEOUT:
			rValue <<= m_nQueryTimeOut;
			break;
		case PROPERTY_ID_RESULTSETCONCURRENCY:
			rValue <<= m_nResultSetConcurrency;
			break;
		case PROPERTY_ID_RESULTSETTYPE:
			rValue <<= m_nResultSetType;
			break;
		case PROPERTY_ID_CURSORNAME:
			rValue <<= m_aCursorName;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

// XWarningsSupplier
//------------------------------------------------------------------------------
Any OStatementBase::getWarnings(void) throw( SQLException, RuntimeException )
{
	return Any();
}

//------------------------------------------------------------------------------
void OStatementBase::clearWarnings(void) throw( SQLException, RuntimeException )
{
}

// ::com::sun::star::util::XCancellable
//------------------------------------------------------------------------------
void OStatementBase::cancel(void) throw( RuntimeException )
{
}

// Helper
//------------------------------------------------------------------------------
void OStatementBase::adjustResultColumns(ODataColumns& rResultColumns)
{
	// get more informations about the database fields
	if (m_bUseEscapeProcessing && m_pDBConnection)
	{
		const SdbSqlParseTreeIterator& rIter =	*m_xCursor->SQLParseIterator();
		if (m_xCursor->SQLParseIterator()
			&& rIter.StatementType() == SDB_STATEMENT_SELECT
			&& rIter.GetSelectColumns().is()
			&& rIter.GetSelectColumns()->Count() > 1)
		{
			const OTables& rTables = m_pDBConnection->getImplTables();

			Reference< XInterface >  xTable; // keep the table in that scope
			OTable*  pTable  = NULL;
			String aCurrentTableName;

			const SdbColumnsRef xCols(rIter.GetSelectColumns());
			for (sal_uInt16 i = 1; i < xCols->Count(); i++)
			{
				const SdbColumn* pCol = xCols->Column(i);
				if (!pCol->GetTableName().Len())
					continue;

				if (!aCurrentTableName.Equals(pCol->GetTableName()))
				{
					aCurrentTableName = pCol->GetTableName();

					if (aCurrentTableName.Len() != 0)
					{
						try
						{
							pTable = rTables.get(UniString(aCurrentTableName));
						}
						catch (...)
						{
							pTable = NULL;
						}
					}
					else
						pTable = NULL;

					xTable =  (XPropertySet*)pTable;
				}
				if (pTable)
				{
					const OColumns& m_rColumns = pTable->getImplColumns();
					const OColumn*  pRefCol    = m_rColumns.at(UniString(pCol->GetRealName()));
					if (pRefCol)
					{
						OColumn* pResultCol = rResultColumns[sal_Int32(i-1)];
						pResultCol->adjustTo(*pRefCol);
					}
				}
			}
		}
	}
}

//************************************************************
//  OStatement
//************************************************************
// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OStatement::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XServiceInfo > *)0 ),
						   ::getCppuType( (const Reference< XStatement > *)0 ),
							OStatementBase::getTypes() );

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OStatement::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OStatement::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OStatementBase::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XServiceInfo * >( this ),
					static_cast< XStatement * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OStatement::acquire() throw (RuntimeException)
{
	OStatementBase::acquire();
}

//--------------------------------------------------------------------------
void OStatement::release() throw (RuntimeException)
{
	OStatementBase::release();
}


// XServiceInfo

//------------------------------------------------------------------------------
rtl::OUString OStatement::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OStatement");
}

//------------------------------------------------------------------------------
sal_Bool OStatement::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OStatement::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< ::rtl::OUString > aSNS( 2 );
	aSNS.getArray()[0] = SERVICE_SDBC_STATEMENT;
	aSNS.getArray()[1] = SERVICE_SDB_STATEMENT;
	return aSNS;
}

// XStatement
//------------------------------------------------------------------------------
Reference< XResultSet >  OStatement::executeQuery(const rtl::OUString& sql) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OComponentHelper::rBHelper.bDisposed)
		throw DisposedException();

	disposeResultSet();

	if (m_xCursor->IsOpen())
		m_xCursor->Close();

	sal_uInt16 nSdbOptions = SDB_API_COMPATIBLE;
	SdbCursorType eSdbType = SDB_SNAPSHOT;

	if (m_nResultSetConcurrency == ResultSetConcurrency::READ_ONLY)
		nSdbOptions |= SDB_READONLY;

	if (m_bUseEscapeProcessing)
		nSdbOptions |= SDB_INDIRECT;

	if (m_nResultSetType == ResultSetType::FORWARD_ONLY)
	{
		nSdbOptions |= SDB_FORWARDONLY;
		eSdbType = SDB_SNAPSHOT;
	}
	else if (m_nResultSetType == ResultSetType::SCROLL_INSENSITIVE)
	{
		eSdbType = SDB_SNAPSHOT;
		nSdbOptions |= SDB_SEEDELETED;
	}
	else
	{
		eSdbType = SDB_KEYSET;
		nSdbOptions |= SDB_SEEDELETED;
	}

	m_xCursor->SetMaxFieldLen(m_nMaxFieldSize);
	m_xCursor->SetMaxRows(m_nMaxRows);

	m_xCursor->Open(sql,sal_False, eSdbType, nSdbOptions);
	checkResult(m_xCursor->Status(), *this, sal_False);

	if (!m_xCursor->GetColumns().is() || m_xCursor->GetColumns()->Count() < 2)
	{
		SdbStatus aStatus;
		aStatus.SetInvalidStatement();
		checkResult(aStatus, *this, sal_False);
	}

	Reference< XResultSet >  xResult;
	OResultSet* pSet;
	xResult = pSet = new OUpdateResultSet(*this, m_xCursor);

	adjustResultColumns(pSet->getImplColumns());
	m_aResultSet = xResult;
	return xResult;
}

//------------------------------------------------------------------------------
sal_Int32 OStatement::executeUpdate(const rtl::OUString& sql) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OComponentHelper::rBHelper.bDisposed)
		throw DisposedException();

	disposeResultSet();

	if (m_xCursor->IsOpen())
		m_xCursor->Close();

	m_xCursor->Open(sql,sal_False);
	checkResult(m_xCursor->Status(), *this, sal_False);
	return (sal_Int32) m_xCursor->Status().GetRecordsAffected();
}

//------------------------------------------------------------------------------
sal_Bool OStatement::execute(const rtl::OUString& sql) throw( SQLException, RuntimeException )
{
	MutexGuard aGuard(m_aMutex);
	if (OComponentHelper::rBHelper.bDisposed)
		throw DisposedException();

	disposeResultSet();

	if (m_xCursor->IsOpen())
		m_xCursor->Close();

	m_xCursor->Open(sql, sal_False);
	checkResult(m_xCursor->Status(), *this, sal_False);
	return sal_True;
}

//------------------------------------------------------------------------------
Reference< XConnection > OStatement::getConnection(void) throw( SQLException, RuntimeException )
{
	return Reference< XConnection > (m_xParent, UNO_QUERY);
}



