/*************************************************************************
 *
 *  $RCSfile: table.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <table.hxx>
#include <api.hrc>
#ifndef _COM_SUN_STAR_SDBCX_PRIVILEGE_HPP_
#include <com/sun/star/sdbcx/Privilege.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HDL_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif
#ifndef _UNOTOOLS_ENUMHELPER_HXX_
#include <unotools/enumhelper.hxx>
#endif
#ifndef _UNOTOOLS_CONTAINER_HXX_
#include <unotools/container.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

using namespace ::com::sun::star::sdbc;
using namespace ::com::sun::star::sdbcx;
using namespace ::com::sun::star::data;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::osl;
using namespace ::cppu;


DBG_NAME(OTable);
//--------------------------------------------------------------------------
OTable::OTable(OConnectionBase* pConnection, sal_Bool bCaseSensitive)
	   :OTableBase(m_aMutex)
	   ,OPropertySetHelper(OTableBase::rBHelper)
	   ,m_aConnection(*pConnection)
	   ,m_nPrivileges(0)
	   ,m_aColumns(*this, m_aMutex, bCaseSensitive)
{
	DBG_CTOR(OTable, NULL);
}

//--------------------------------------------------------------------------
OTable::~OTable()
{
	DBG_DTOR(OTable, NULL);
}

// com::sun::star::lang::XTypeProvider
//--------------------------------------------------------------------------
Sequence< Type > OTable::getTypes() throw (RuntimeException)
{
	OTypeCollection aTypes(::getCppuType( (const Reference< XPropertySet > *)0 ),
							OTableBase::getTypes() );

	return aTypes.getTypes();
}

//--------------------------------------------------------------------------
Sequence< sal_Int8 > OTable::getImplementationId() throw (RuntimeException)
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}

// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
Any OTable::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aIface = OTableBase::queryInterface( rType );
	if (!aIface.hasValue())
		aIface = ::cppu::queryInterface(
					rType,
					static_cast< XPropertySet * >( this ));
	return aIface;
}

//--------------------------------------------------------------------------
void OTable::acquire() throw(RuntimeException)
{
	OTableBase::acquire();
}

//--------------------------------------------------------------------------
void OTable::release() throw(RuntimeException)
{
	OTableBase::release();
}


// OComponentHelper
//------------------------------------------------------------------------------
void OTable::disposing()
{
	OPropertySetHelper::disposing();

	MutexGuard aGuard(m_aMutex);
	m_aColumns.disposing();
	m_aConnection = Reference< XInterface > ();
}

// OPropertySetHelper
//------------------------------------------------------------------------------
Reference< XPropertySetInfo > OTable::getPropertySetInfo() throw (RuntimeException)
{
	return createPropertySetInfo( getInfoHelper() ) ;
}

// utl::OPropertyArrayUsageHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper* OTable::createArrayHelper( ) const
{
	BEGIN_PROPERTY_HELPER(6)
		DECL_PROP1(CATALOGNAME,	::rtl::OUString,	READONLY);
		DECL_PROP1(DESCRIPTION,	::rtl::OUString,	READONLY);
		DECL_PROP1(NAME,		::rtl::OUString,	READONLY);
		DECL_PROP1(PRIVILEGES,	sal_Int32,		READONLY);
		DECL_PROP1(SCHEMANAME,	::rtl::OUString,	READONLY);
		DECL_PROP1(TYPE,		::rtl::OUString,	READONLY);
	END_PROPERTY_HELPER();
}

// cppu::OPropertySetHelper
//------------------------------------------------------------------------------
::cppu::IPropertyArrayHelper& OTable::getInfoHelper()
{
	return *getArrayHelper();
}

//------------------------------------------------------------------------------
sal_Bool OTable::convertFastPropertyValue( Any & rConvertedValue, Any & rOldValue, sal_Int32 nHandle, const Any& rValue ) throw( IllegalArgumentException  )
{
	return sal_False;
}

//------------------------------------------------------------------------------
void OTable::setFastPropertyValue_NoBroadcast( sal_Int32 nHandle, const Any& rValue ) throw(::com::sun::star::uno::Exception)
{
}

//------------------------------------------------------------------------------
void OTable::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROPERTY_ID_PRIVILEGES:
			rValue <<= m_nPrivileges;
			break;
		case PROPERTY_ID_CATALOGNAME:
			rValue <<= m_aCatalogName;
			break;
		case PROPERTY_ID_SCHEMANAME:
			rValue <<= m_aSchemaName;
			break;
		case PROPERTY_ID_NAME:
			rValue <<= m_aName;
			break;
		case PROPERTY_ID_DESCRIPTION:
		case PROPERTY_ID_TYPE:
			rValue <<= rtl::OUString();
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

// XServiceInfo
//------------------------------------------------------------------------------
sal_Bool OTable::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OTable::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< rtl::OUString > aSNS( 1 );
	aSNS[0] = SERVICE_SDBCX_TABLE;
	return aSNS;
}

// XColumnsSupplier
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XNameAccess >  OTable::getColumns(void) throw( RuntimeException )
{
	return &const_cast<OTable*>(this)->m_aColumns;
}

//************************************************************
//  OTables
//************************************************************
DBG_NAME(OTables);
// com::sun::star::uno::XInterface
//--------------------------------------------------------------------------
void OTables::acquire() throw(RuntimeException)
{
	m_rParent.acquire();
}

//--------------------------------------------------------------------------
void OTables::release() throw(RuntimeException)
{
	m_rParent.release();
}

//------------------------------------------------------------------------------
void OTables::disposing()
{
	m_aRefreshListeners.disposeAndClear(EventObject(static_cast<XWeak*>(this)));

	MutexGuard aGuard(m_rMutex);
	for (vector<OTable*>::const_iterator i = m_aElements.begin(); i != m_aElements.end(); i++)
	{
		(*i)->disposing();
		(*i)->release();
	}
	m_aElements.clear();
}

// ::com::sun::star::util::XRefreshable
//------------------------------------------------------------------------------
void OTables::addRefreshListener(const Reference< ::com::sun::star::util::XRefreshListener > & l) throw( RuntimeException  )
{   m_aRefreshListeners.addInterface(l); }
//------------------------------------------------------------------------------
void OTables::removeRefreshListener(const Reference< ::com::sun::star::util::XRefreshListener > & l) throw( RuntimeException  )
{   m_aRefreshListeners.removeInterface(l); }

//------------------------------------------------------------------------------
void OTables::refresh(void) throw( RuntimeException  )
{
	MutexGuard aGuard(m_rMutex);
	for (vector<OTable*>::const_iterator i = m_aElements.begin(); i != m_aElements.end(); i++)
	{
		(*i)->disposing();
		(*i)->release();
	}
	m_aElements.clear();

	static_cast<OConnectionBase&>(m_rParent).refresh(dbTable);
	EventObject aEvt(static_cast<XWeak*>(this));
	NOTIFY_LISTENERS(m_aRefreshListeners, ::com::sun::star::util::XRefreshListener, refreshed, aEvt);
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OTables::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OTables");
}

//------------------------------------------------------------------------------
sal_Bool OTables::supportsService( const ::rtl::OUString& ServiceName ) throw (RuntimeException)
{
	return ::hasString(getSupportedServiceNames(), ServiceName);
}

//------------------------------------------------------------------------------
Sequence< ::rtl::OUString > OTables::getSupportedServiceNames(  ) throw (RuntimeException)
{
	Sequence< rtl::OUString > aSNS( 1 );
	aSNS[0] = SERVICE_SDBCX_CONTAINER;
	return aSNS;
}

// ::com::sun::star::container::XElementAccess
//------------------------------------------------------------------------------
Type OTables::getElementType(  ) throw(RuntimeException)
{
	return::getCppuType(static_cast<Reference<XPropertySet>*>(NULL));
}


//------------------------------------------------------------------------------
sal_Bool OTables::hasElements(void) throw( RuntimeException )
{
	return getCount() != 0;
}

// ::com::sun::star::container::XEnumerationAccess
//------------------------------------------------------------------------------
Reference< ::com::sun::star::container::XEnumeration >  OTables::createEnumeration(void) throw( RuntimeException )
{
	MutexGuard aGuard(m_rMutex);
	return new utl::OEnumerationByName( static_cast< ::com::sun::star::container::XNameAccess*>(this));
}

// ::com::sun::star::container::XIndexAccess
//------------------------------------------------------------------------------
sal_Int32 OTables::getCount(void) throw( RuntimeException )
{
	return static_cast<OConnectionBase&>(m_rParent).getCount(dbTable);
}

//------------------------------------------------------------------------------
Any OTables::getByIndex(sal_Int32 Index) throw( IndexOutOfBoundsException, WrappedTargetException, RuntimeException )
{
	SdbObj* pObj = static_cast<OConnectionBase&>(m_rParent).getByIndex(dbTable, Index);
	return makeAny(Reference< XPropertySet >(get(pObj)));
}

// ::com::sun::star::container::XNameAccess
//------------------------------------------------------------------------------
Any OTables::getByName(const rtl::OUString& aName) throw( ::com::sun::star::container::NoSuchElementException,
														   WrappedTargetException, RuntimeException )
{
	SdbObj* pObj = static_cast<OConnectionBase&>(m_rParent).getByName(dbTable, aName);
	return makeAny(Reference< XPropertySet >(get(pObj)));
}

//------------------------------------------------------------------------------
Sequence< rtl::OUString > OTables::getElementNames(void) throw( RuntimeException )
{
	return static_cast<OConnectionBase&>(m_rParent).getElementNames(dbTable);
}

//------------------------------------------------------------------------------
sal_Bool OTables::hasByName(const rtl::OUString& aName) throw( RuntimeException )
{
	sal_Bool bHaveIt = sal_False;
	try
	{
		bHaveIt = static_cast<OConnectionBase&>(m_rParent).getByName(dbTable, aName) != NULL;
	}
	catch (::com::sun::star::container::NoSuchElementException e)
	{
		bHaveIt = sal_False;
	}
	return bHaveIt;
}


// helper methods
//------------------------------------------------------------------------------
OTable* OTables::get(SdbObj* pObj) const
{
	MutexGuard aGuard(const_cast<OTables*>(this)->m_rMutex );
	for (vector<OTable*>::const_iterator i = m_aElements.begin(); i != m_aElements.end(); i++)
	{
		if ((*i)->getImplObj() == pObj)
			return *i;
	}

	OTable* pTable = create(pObj);
	pTable->acquire();
	const_cast<OTables*>(this)->m_aElements.push_back(pTable);
	return pTable;
}

//------------------------------------------------------------------------------
OTable* OTables::get(const rtl::OUString& aName) const throw( ::com::sun::star::container::NoSuchElementException,
															  WrappedTargetException,
															  RuntimeException )
{
	SdbObj* pObj = static_cast<OConnectionBase&>(m_rParent).getByName(dbTable, aName);
	return get(pObj);
}

//************************************************************
//  OUITable
//************************************************************

//--------------------------------------------------------------------------
OUITable::OUITable(OConnectionBase* pConnection, SbaTableDef* pImpl)
		 :OTable(pConnection, pImpl->GetTable()->GetConnection()->GetIdentifierCase() == SDB_IC_ALL)
		 ,m_xTable(pImpl)
{
	sal_Int16 nPrivilege = m_xTable->GetTable()->GetTablePrivileges();
	if (nPrivilege & DatabasePrivilige::READ_DEF)
		m_nPrivileges |= Privilege::READ;
	if (nPrivilege & DatabasePrivilige::ALTER_DEF)
		m_nPrivileges |= Privilege::ALTER;
	if (nPrivilege & DatabasePrivilige::DELETE_DEF)
		m_nPrivileges |= Privilege::DROP;
	if (nPrivilege & DatabasePrivilige::RELATION_DEF)
		m_nPrivileges |= Privilege::REFERENCE;
	if (nPrivilege & DatabasePrivilige::RETRIEVE_DATA)
		m_nPrivileges |= Privilege::SELECT;
	if (nPrivilege & DatabasePrivilige::INSERT_DATA)
		m_nPrivileges |= Privilege::INSERT;
	if (nPrivilege & DatabasePrivilige::UPDATE_DATA)
		m_nPrivileges |= Privilege::UPDATE;
	if (nPrivilege & DatabasePrivilige::DELETE_DATA)
		m_nPrivileges |= Privilege::DELETE;

	m_aName = UniString(m_xTable->GetTable()->Name());
	m_aSchemaName = UniString(m_xTable->GetTable()->OwnerName());
	m_aCatalogName = UniString(m_xTable->GetTable()->QualifierName());
	m_aColumns.insertColumns(m_xTable->GetOriginalColumns());
}

// OComponentHelper
//------------------------------------------------------------------------------
void OUITable::disposing()
{
	OTable::disposing();

	MutexGuard aGuard(m_aMutex);
	m_xTable = NULL;
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OUITable::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OUITable");
}

// Helper
//------------------------------------------------------------------------------
SdbObj* OUITable::getImplObj() const
{
	return m_xTable;
}

//************************************************************
//  OUITables
//************************************************************
OTable* OUITables::create(SdbObj* pObj) const
{
	return new OUITable(&static_cast<OConnectionBase&>(m_rParent), static_cast<SbaTableDef*>(pObj));
}

//************************************************************
//  OCoreTable
//************************************************************

//--------------------------------------------------------------------------
OCoreTable::OCoreTable(OConnectionBase* pConnection, SdbTable* pImpl)
		 :OTable(pConnection, pImpl->GetConnection()->GetIdentifierCase() == SDB_IC_ALL)
		 ,m_xTable(pImpl)
{
	sal_Int16 nPrivilege = m_xTable->GetTablePrivileges();
	if (nPrivilege & DatabasePrivilige::READ_DEF)
		m_nPrivileges |= Privilege::READ;
	if (nPrivilege & DatabasePrivilige::ALTER_DEF)
		m_nPrivileges |= Privilege::ALTER;
	if (nPrivilege & DatabasePrivilige::DELETE_DEF)
		m_nPrivileges |= Privilege::DROP;
	if (nPrivilege & DatabasePrivilige::RELATION_DEF)
		m_nPrivileges |= Privilege::REFERENCE;
	if (nPrivilege & DatabasePrivilige::RETRIEVE_DATA)
		m_nPrivileges |= Privilege::SELECT;
	if (nPrivilege & DatabasePrivilige::INSERT_DATA)
		m_nPrivileges |= Privilege::INSERT;
	if (nPrivilege & DatabasePrivilige::UPDATE_DATA)
		m_nPrivileges |= Privilege::UPDATE;
	if (nPrivilege & DatabasePrivilige::DELETE_DATA)
		m_nPrivileges |= Privilege::DELETE;

	m_aName = UniString(m_xTable->Name());
	m_aSchemaName = UniString(m_xTable->OwnerName());
	m_aCatalogName = UniString(m_xTable->QualifierName());

	m_aColumns.insertColumns(m_xTable->OriginalColumns());
}

// OComponentHelper
//------------------------------------------------------------------------------
void OCoreTable::disposing()
{
	OTable::disposing();

	MutexGuard aGuard(m_aMutex);
	m_xTable = NULL;
}

// XServiceInfo
//------------------------------------------------------------------------------
rtl::OUString OCoreTable::getImplementationName(  ) throw(RuntimeException)
{
	return rtl::OUString::createFromAscii("com.sun.star.sdb.OCoreTable");
}

// Helper
//------------------------------------------------------------------------------
SdbObj* OCoreTable::getImplObj() const
{
	return m_xTable;
}

//************************************************************
//  OCoreTables
//************************************************************
OTable* OCoreTables::create(SdbObj* pObj) const
{
	return new OCoreTable(&static_cast<OConnectionBase&>(m_rParent), static_cast<SdbTable*>(pObj));
}



