/*************************************************************************
 *
 *  $RCSfile: datconv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _ZFORLIST_HXX //autogen
#include <svtools/zforlist.hxx>
#endif

#ifndef _SDB_TYPECONV_HXX
#include <sdb/typeconv.hxx>
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include <sdb/sdbtools.hxx>
#endif

#ifndef _SBA_DATCONV_HXX_
#include "datconv.hxx"
#endif
#ifndef _SBARESID_HRC
#include "sbaresid.hrc"
#endif
#ifndef _SBA_SBARES_HXX
#include "sbares.hxx"
#endif

//------------------------------------------------------------------------------
sal_Bool SbaDbDataConverter::String2Double(const String& rText, sal_uInt32 nFormatKey, double& rRet)
{
	// FS 09.04.99: Bug 64520 - Sonderbehandlung fuer Prozent
	String sCompletedText(rText);
	if (NUMBERFORMAT_PERCENT == m_pFormatter->GetType(nFormatKey))
	{	// es handelt sich um eine Prozent-Formatierung
		sal_uInt32 nTempFormat = 0;
		if (m_pFormatter->IsNumberFormat(sCompletedText, nTempFormat, rRet) &&
			NUMBERFORMAT_NUMBER == m_pFormatter->GetType(nTempFormat))
			// und der String entspricht einer Number-Formatierung, hat also nur kein %
			// -> dranhaengen
			sCompletedText += '%';
	}

	return m_pFormatter->IsNumberFormat(sCompletedText, nFormatKey, rRet);
}

//------------------------------------------------------------------------------
sal_Bool SbaDbDataConverter::String2Var(const String& sText, SbaFieldType aRequestedType, sal_uInt32 nFormatKey, SbaNumericSize aSize, ODbVariant& rVar)
{
	if (m_pFormatter->GetType(nFormatKey) == NUMBERFORMAT_UNDEFINED)
	{
		// wenn ich den Format-Key nicht kenne, behandele ich den String einfach als gueltig
		rVar.setString(sText);
		return sal_True;
	}

	switch (aRequestedType)
	{
		case dbMemo:
		{
			if (sText.Len() == 0)
				rVar.setNull();
			else
				rVar.setString(sText);
		}
		break;
		case dbText:
		case dbChar:
		{
			double fResult;
			if (sText.Len() == 0)
				rVar.setNull();
			else if (m_pFormatter->IsTextFormat(nFormatKey))
				rVar.setString(sText);
			else
			{
				if (String2Double(sText, nFormatKey, fResult))
				{
					switch (m_pFormatter->GetType(nFormatKey))
					{
						case NUMBERFORMAT_DATETIME:
						case NUMBERFORMAT_TIME:
						case NUMBERFORMAT_DATE:
							rVar.setDouble( SdbTools::ToStandardDate(*m_pFormatter, fResult) );
							break;
						default:
							rVar.setDouble( fResult );
					}					
				}
				else
					return sal_False;
			}
		}
		break;

		case dbCurrency:
		case dbCounter:
		case dbDecimal:
		case dbNumeric:
		case dbDateTime:
		case dbDate:
		case dbTime:
		{
			double fResult;

			if (!sText.Len())
				rVar.setNull();
			else
			{
				if (m_pFormatter->IsTextFormat(nFormatKey))
					rVar.setString(sText);
				else if (!String2Double(sText, nFormatKey, fResult))
					return sal_False;
				else
				{
					if (aRequestedType == dbNumeric)
					{
						sal_Bool bValid = sal_False;
						switch (aSize)
						{
							case dbLong:
								if ((double)(sal_Int32)fResult != fResult)
									break;
								bValid = sal_True;
								rVar.setINT32((sal_Int32)(fResult));
								break;

							case dbInteger:
								if ((double)(sal_Int16)fResult != fResult)
									break;
								bValid = sal_True;
								rVar.setINT16((sal_Int16)(fResult));
								break;

							case dbByte:
								if ((double)(sal_Int8)fResult != fResult)
									break;
								bValid = sal_True;
								rVar.setBYTE((sal_Int8)(fResult));
								break;
							case dbSingle:
								bValid = sal_True;
								rVar.setDouble(fResult);
								break;
							default:
								bValid = sal_True;
								switch (m_pFormatter->GetType(nFormatKey))
								{
									case NUMBERFORMAT_DATETIME:
									case NUMBERFORMAT_TIME:
									case NUMBERFORMAT_DATE:
										rVar.setDouble( SdbTools::ToStandardDate(*m_pFormatter, fResult) );
										break;
									default:
										rVar.setDouble( fResult );
								}													
						}
						if (!bValid)
							return sal_False;
					}
					else
					{
						switch (aRequestedType)
						{
							case dbDateTime:
							case dbDate:
							case dbTime:						
								rVar.setDouble( SdbTools::ToStandardDate(*m_pFormatter, fResult) );
								break;
							default:
								rVar.setDouble( fResult );
						}
					}					
				}
			}
		}
		break;					
		default:
			DBG_ERROR("SbaDbDataConverter::String2Var : wrong field type !");
	}

	return sal_True;
}
//------------------------------------------------------------------------------
String SbaDbDataConverter::Var2String(const ODbVariant& rVar, SbaFieldType aOutputType, sal_uInt32 nFormatKey, Color** ppColor,sal_Bool bOutput)
{
	return Var2String(rVar,aOutputType,nFormatKey,ppColor,m_pFormatter,bOutput);
}
//------------------------------------------------------------------------------
String SbaDbDataConverter::Var2String(const ODbVariant& rVar, SbaFieldType aOutputType, sal_uInt32 nFormatKey, Color** ppColor, SvNumberFormatter* _pFormatter,sal_Bool bOutput,sal_Bool _bAutoInc)

{
	String aText;
	if (rVar.isNull())
	{
		switch (aOutputType)
		{
			case dbBinary:
			case dbVarBinary:
			case dbNone:
			case dbImage:
				aText.AssignAscii("<OBJECT>");
				break;
			case dbCounter:
				aText = '(';
				aText += String(SbaResId(STR_FIELD_TYPE_AUTOCOUNT));
				aText += ')';
				break;
			default:
				if(_bAutoInc)
				{
					aText = '(';
					aText += String(SbaResId(STR_FIELD_TYPE_AUTOCOUNT));
					aText += ')';
				}
		}
	}
	else if (rVar.getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence<sal_Int8>*)0))
		aText.AssignAscii("<OBJECT>");
	else if (_pFormatter->GetType(nFormatKey) == NUMBERFORMAT_UNDEFINED)
	{
		// wenn ich den Format-Key nicht kenne, nehme ich den String so, wie er in der Variablen steht
		try
		{
			aText = (const sal_Unicode*)rVar.toString();
		}
		catch(...)
		{
		}
		
	}
	else
	{
		try
		{
			switch (aOutputType)
			{
				case dbMemo:
					aText = (const sal_Unicode*)rVar.toString();
					break;
				case dbBool:
				case dbText:
				case dbChar:
				case dbCurrency:
				case dbDateTime:
				case dbDate:
				case dbTime:
				case dbNumeric:
				case dbDecimal:
				case dbCounter:
				{
					if (_pFormatter->IsTextFormat(nFormatKey))
					{
						if (bOutput)
						{
							String aTemp = rVar.toString();
							_pFormatter->GetOutputString(aTemp, nFormatKey, aText, ppColor );
						}
						else
							aText = (const sal_Unicode*)rVar.toString();
					}
					else
					{
						double nVal = rVar.toDouble();
						switch (_pFormatter->GetType(nFormatKey))
						{
							case NUMBERFORMAT_DATETIME:
							case NUMBERFORMAT_TIME:
							case NUMBERFORMAT_DATE:
								nVal = SdbTools::ToFormatterDate(*_pFormatter, nVal);
								break;					
						}
						if (bOutput || aOutputType != dbCurrency)
							_pFormatter->GetOutputString( nVal, nFormatKey, aText, ppColor );
						else
							_pFormatter->GetInputLineString( nVal, nFormatKey, aText);
					}
				} break;
				case dbBinary:
				case dbVarBinary:
				case dbNone:
				case dbImage:
					aText.AssignAscii("<OBJECT>"); break;
				default: 
				{
					DBG_ERROR("SbaDbDataConverter::Var2String : nicht unterstuetzer Datentyp:");
				}
			}
		}
		catch(...)
		{
		}
		
	}
	return aText;
}

