/*************************************************************************
 *
 *  $RCSfile: dbdefobj.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#if STLPORT_VERSION>=321
#include <cstdarg>		// std::va_list and friends
#endif

#include "sbaexpob.hxx"
#include "dbdefobj.hxx"
#include "sbadb.hxx"
#include "sbahints.hxx"
#include "dbstat.hxx"
#include "dbexch.hxx"
#include "dbstat.hxx"
#include "sbaitems.hrc"

#ifndef _SBA_DBFILEOB_HXX
#include "dbfileob.hxx"
#endif

#ifndef _SFXAPP_HXX
#include <sfx2/app.hxx>
#endif

#ifndef _SFXEVENT_HXX //autogen
#include <sfx2/event.hxx>
#endif

#ifndef _SFX_OBJSH_HXX //autogen
#include <sfx2/objsh.hxx>
#endif

#ifndef _URLOBJ_HXX //autogen
#include <tools/urlobj.hxx>
#endif

#ifndef _SFXDOCFILE_HXX //autogen
#include <sfx2/docfile.hxx>
#endif

#ifndef _SFXENUMITEM_HXX //autogen
#include <svtools/eitem.hxx>
#endif

#ifndef _SFXSTRITEM_HXX //autogen
#include <svtools/stritem.hxx>
#endif

#ifndef _SFXITEMSET_HXX //autogen
#include <svtools/itemset.hxx>
#endif

#ifndef _SFXDISPATCH_HXX //autogen
#include <sfx2/dispatch.hxx>
#endif

#ifndef _SFXSIDS_HRC
#include <sfx2/sfxsids.hrc>
#endif


DBG_NAME( SbaDBDef );

TYPEINIT1(SbaDBDef, SbaDBObj);

//------------------------------------------------------------------------------
SbaDBDef::SbaDBDef(SbaDatabase* pDB,
				   const String& rName,
				   DBObject eObj,
				   BOOL _bNew,
				   BOOL bShow)
		  :SbaDBObj(pDB, rName, TRUE, bShow)
		  ,edbObj(eObj)
		  ,aLockCounter(_bNew ? 1 : 0)
{
	bNew = _bNew;
	//StartListening(*SFX_APP());
}

//------------------------------------------------------------------------------
SbaDBDef::~SbaDBDef()
{
	//if (IsListening(*SFX_APP()))
	//	EndListening(*SFX_APP());
	DBG_ASSERT(!aDBStor.Is(), "SbaFileDef: Fehler in der Transaction");
	aDBStor.Clear();
}

//------------------------------------------------------------------------------
void SbaDBDef::Close()
{
	//if (IsListening(*SFX_APP()))
	//	EndListening(*SFX_APP());
	SbaDBObj::Close();
}

//------------------------------------------------------------------------------
void SbaDBDef::SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType,
						  const SfxHint& rHint, const TypeId& rHintType){}

//------------------------------------------------------------------------------
String SbaDBDef::ToURL(const String& rDBName, DBObject eKind, const String& rName)
{
	INetURLObject aObj;
	aObj.SetSmartProtocol(INET_PROT_FILE);

	aObj.SetSmartURL(rDBName);

	String aParam;
	switch (eKind)
	{
		case dbQuery:
			aParam.AssignAscii(SDB_QUERYMARK_HEADER); break;
		case dbTable:
			aParam.AssignAscii(SDB_TABLEMARK_HEADER); break;
		case dbView:
			aParam.AssignAscii(SDB_TABLEMARK_HEADER);
			aParam += ';';
			break;
		case dbForm:
			aParam.AssignAscii(SDB_FORMMARK_HEADER); break;
		case dbReport:
			aParam.AssignAscii(SDB_REPORTMARK_HEADER); break;
	}
	aParam += rName;

	// besonders kennzeichnen, da es sich um eine View handelt
	aObj.SetMark(aParam);
	return aObj.GetMainURL();
}

//------------------------------------------------------------------------------
String SbaDBDef::ToURL() const
{
	if (!GetDatabase())
		return String();

	return SbaDBDef::ToURL(GetDatabase()->Name(), GetKind(), Name());
}

//------------------------------------------------------------------------------
// Hier fragt das Object, ob irgendjemand etwas dagegen hat,
// wenn das Object freigegeben wird
BOOL SbaDBDef::CanRelease()
{
	SbaDBDefHint aHint(this, SbaDBDefHint::dbRelease);
	SBA_MOD()->Broadcast(aHint);
	return aHint.CanRelease();
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::Drop()
{
	SBA_CHK_OPEN_BOOL();

	if (!CanRelease())
		return FALSE;

	BeginDBAction();
	if (DropImpl())
	{
		SbaDBDefHint aHint(this, SbaDBDefHint::dbDelete);
		SBA_MOD()->Broadcast(aHint);

		// Datenbank benachrichtigen
		EndDBAction();
		GetDatabase()->RemoveChild(this);
		bNew = FALSE;
	}
	else
	{
		CancelDBAction();
		ShowError(dbDeleteError);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::Compare(const SdbObj& _rObj) const
{
	SbaDBDef* pObj = PTR_CAST(SbaDBDef,&_rObj);
	SbaDatabase* pDatabase = GetDatabase();

	return pObj && pDatabase && pObj->IsOpen() &&
		   *pDatabase == *pObj->GetDatabase() && GetKind() == pObj->GetKind() &&
		   aName.Equals(pObj->aName);
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::DropReversibleImpl()
{
	BOOL bSuccess = GetDatabase()->DeleteObject(edbObj, aName);
	aStatus = GetDatabase()->Status();
	return bSuccess;
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::Rename(const String& rNewName)
{
	SBA_CHK_OPEN_BOOL();
	SbaDatabase* pDB = GetDatabase();

	if (GetKind() == dbTable)
	{
		// Tabellen koennen Casesentive behandelt werden
		if ((pDB->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT)
		{
			if (aName.Equals(rNewName))
				return TRUE;
		}
		else if (EqualNamesInsensitive(aName, rNewName))
			return TRUE;
		// Sonderbehandlung fuer Tabellen, da diese
		// vor einem Umbenennen freigegeben werden muessen
		else if (!IsNew() && !CanRelease())
			return FALSE;
	}
	else if (aName.Equals(rNewName))
	{
		return TRUE;
	}

	BeginDBAction();
	BOOL bResult = RenameImpl(rNewName);
	if (!bResult)
	{
		CancelDBAction();
		ShowError(dbRenameError);
	}
	else
	{
		EndDBAction();

//		if (!IsNew())
//		{
			// die Liste in der Datenbank
			// mit den neuen Namen aktualisieren
			pDB->ReplaceInList(this);
			// Office informieren
			SbaDBDefHint aHint(this, SbaDBDefHint::dbRename);
			SBA_MOD()->Broadcast(aHint);
//		}
	}
	return bResult;
}

//------------------------------------------------------------------------------
void SbaDBDef::OnCreate()
{
	// Office informieren, dass ein neues Objekt erzeugt wurde
	SbaDBDefHint aHint(this, SbaDBDefHint::dbCreate);
	SBA_MOD()->Broadcast(aHint);
}

//------------------------------------------------------------------------------
SbaDatabase* SbaDBDef::GetDatabase() const
{
	return PTR_CAST(SbaDatabase,GetParent());
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::OpenDoc(BOOL bRead, const SfxPoolItem *pArgs1, ... )
{
	SBA_CHK_OPEN_BOOL();

	SfxApplication *pSApp = SFX_APP();
	SfxAllItemSet aSet(pSApp->GetPool());
	if (pArgs1)	// argumente sind angegeben
	{
		va_list pVarArgs;
		va_start( pVarArgs, pArgs1 );
		for ( const SfxPoolItem *pArg = pArgs1;
			  pArg;
			  pArg = va_arg( pVarArgs, const SfxPoolItem* ) )
			aSet.Put(*pArg, pArg->Which());
		va_end(pVarArgs);
	}

	if( ISA(SbaFileDef) )
		aSet.Put(SfxBoolItem(SID_DOC_READONLY, bRead),SID_DOC_READONLY);
//	aSet.Put(SfxBoolItem(SID_SBA_READONLY, bRead),SID_SBA_READONLY);
	aSet.Put(SfxStringItem( SID_FILE_NAME, ToURL()),SID_FILE_NAME);
	aSet.Put(SfxStringItem( SID_REFERER, String::CreateFromAscii("file:")), SID_REFERER);
	aSet.Put(SbaDBDefItem(SID_SBA_DEFINITION, this, GetDatabase()),SID_SBA_DEFINITION);
	//	pSApp->GetAppDispatcher().Execute( SID_OPENDOC, SFX_CALLMODE_ASYNCHRON, aSet);

	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::SetLock(BOOL _bLocked)
{
	if (_bLocked)
		++aLockCounter;
	else
		if (aLockCounter)
			--aLockCounter;

	return IsLocked();
}

//------------------------------------------------------------------------------
SbaDataExchObj* SbaDBDef::CreateDataExchObj( USHORT nType, const String& rExchStr )
{
	switch( nType )
	{
		case EXPLORER_EXCHANGE:
			return new SbaExplorerExchObj( this, rExchStr );
		default:
			DBG_ERROR("Unbstimmter Typ");
	}
	return NULL;
}

//------------------------------------------------------------------------------
SbaDataExchObj* SbaDBDef::CreateDataExchObj(USHORT nType, SbaDatabase* pDB, DBObject eKind,
	const String& rName)
{
	switch( nType )
	{
		case EXPLORER_EXCHANGE:
			return new SbaExplorerExchObj( pDB, eKind, rName );
		case RTF_EXCHANGE:
			return new SbaRTFExchObj( pDB );
		case HTML_EXCHANGE:
			return new SbaHTMLExchObj( pDB );
		default:
			DBG_ERROR("Unbstimmter Typ");
	}
	return NULL;
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::DropReversible()
{
	SBA_CHK_OPEN_BOOL();

	if (!CanRelease())
		return FALSE;

	BeginDBAction();
	if (DropReversibleImpl())
	{
		SbaDBDefHint aHint(this, SbaDBDefHint::dbDelete);
		SBA_MOD()->Broadcast(aHint);

		// Datenbank benachrichtigen
		EndDBAction();
		GetDatabase()->RemoveChild(this);
	}
	else
	{
		CancelDBAction();
		ShowError(dbDeleteError);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::BeginTrans(BOOL bForStore)
{
	SBA_CHK_OPEN_BOOL();
	if (aDBStor.Is())		// Storage schon geoeffnet
	{
		aDBStor->AddRef();
		return TRUE;
	}
	else
	{
		aDBStor = GetDatabase()->OpenStorage(edbObj, !bForStore);
		if (!aDBStor.Is())
			aStatus = GetDatabase()->Status();
		return aDBStor.Is();
	}
}

//------------------------------------------------------------------------------
BOOL SbaDBDef::EndTrans(BOOL bCommit)
{
	SBA_CHK_OPEN_BOOL();
	DBG_ASSERT(aDBStor.Is(), "SbaDBDef: Fehler in der Transaction");
	BOOL bResult;
	if (!aDBStor.Is())
		bResult = FALSE;
	else if (aDBStor->GetRefCount() > 2)		// eine Ref haelt immer die Datenbank
	{
		aDBStor->ReleaseReference();
		bResult = TRUE;
	}
	else
	{
		aDBStor.Clear();
		bResult = GetDatabase()->ReleaseStorage(edbObj,bCommit);
		if (!bResult && bCommit)
			aStatus = GetDatabase()->Status();
	}
	return bResult;
}

//-------------------------------------------------------------------------
ULONG SbaDBDef::CopyTo(SvStream& rTarget, SvStream& rDest)
{
	void * pMem = new BYTE[ 8192 ];
	ULONG nError = ERRCODE_NONE;
	ULONG  nRead;
	rDest.Seek(0);
	while( 0 != (nRead = rDest.Read(pMem, 8192 )))
	{
		if (nRead != rTarget.Write(pMem, nRead))
		{
			nError = SVSTREAM_GENERALERROR;
			break;
		}
	}
	delete pMem;
	if (nError == ERRCODE_NONE)
	{
		rTarget.Flush();
		nError = rTarget.GetError();
	}
	return nError;
}

//------------------------------------------------------------------------------
TYPEINIT1(SbaDBDefItem, SfxPoolItem);

//------------------------------------------------------------------------------
SfxPoolItem* SbaDBDefItem::Clone( SfxItemPool *pPool) const
{
	return new SbaDBDefItem(Which(), xDBDef, xDatabase);
}

//------------------------------------------------------------------------------
int SbaDBDefItem::operator==(const SfxPoolItem& rItem) const
{
	if (rItem.Type() == Type())
	{
		return &xDBDef == &PTR_CAST(SbaDBDefItem,&rItem)->xDBDef;
	}
	else
		return 0;
}


















