/*************************************************************************
 *
 *  $RCSfile: dbfileob.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sbaresid.hrc"
#include "dbstat.hxx"
#include "sbaexpob.hxx"
#include "sbadb.hxx"
#include "dbfileob.hxx"
#include "sbasltid.hrc"
#include "dbexch.hxx"
#include "duplname.hxx"

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif

#ifndef _SDB_SDBDOC_HXX //autogen
#include <sdb/sdbdoc.hxx>
#endif


TYPEINIT1(SbaFileDef, SbaDBDef);
TYPEINIT1(SbaReportDef, SbaFileDef);
TYPEINIT1(SbaFormDef, SbaFileDef);

//------------------------------------------------------------------------------
SbaFileDef::SbaFileDef(SbaDatabase* pDB,
					   const String& rName,
					   DBObject eObj,
					   BOOL _bNew,
					   BOOL bShow)
			   :SbaDBDef(pDB, rName, eObj, _bNew, bShow)
{
	if (IsNew())
		pDocument = (SdbDocument*)pDB->CreateObject(eObj, rName);
	else
		pDocument = (SdbDocument*)pDB->OpenObject(eObj, rName);

	if (pDocument)
	{
		pDocument->AddRef();
		bOpen = IsNew();
	}
	else
	{
		aStatus = pDB->Status();
		ShowError(IsNew() ? dbWriteError : dbReadError);
	}
}

//------------------------------------------------------------------------------
SbaFileDef::~SbaFileDef()
{
	if (pDocument)
		pDocument->ReleaseRef();
}

//------------------------------------------------------------------------------
BOOL SbaFileDef::Open()
{
	if (!IsOpen())
		bOpen = pDocument != NULL;

	return bOpen;
}

//------------------------------------------------------------------------------
BOOL SbaFileDef::RenameImpl(const String& rNewName)
{
	if (pDocument->Rename(rNewName))
		aName = rNewName;
	else
		aStatus = pDocument->Status();
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SbaFileDef::DropImpl()
{
	if (pDocument->Drop())
		aStatus = pDocument->Status();
	return aStatus.IsSuccessful();
}

//-------------------------------------------------------------------------
String SbaFileDef::CreateTempFile()
{
	SBA_CHK_OPEN_STRING();

	// erzeugen einer temp Datei, die das Dokument beinhaltet
	BeginDBAction();
	String aFileName;

	DirEntry aFile;
	aFile.SetBase(aName);
	aFile.SetExtension(String::CreateFromAscii("tmp"));

	DirEntry aEntry = DirEntry().TempName().GetPath();
	aEntry.MakeShortName(aFile.GetName(),FSYS_KIND_FILE);
	SvFileStream aFileStream;
	aFileStream.Open(aEntry.GetFull(), STREAM_STD_READWRITE | STREAM_TRUNC);
	if (!pDocument->Get(aFileStream))
	{
		aStatus = pDocument->Status();
		CancelDBAction();
		ShowError(dbReadError);
	}
	else
	{
		EndDBAction();
		aFileName = aEntry.GetFull();
	}
	return aFileName;
}

//-------------------------------------------------------------------------
BOOL SbaFileDef::Update(const String& rFile)
{
	SBA_CHK_OPEN_BOOL();
	BOOL bResult = FALSE;

	BeginDBAction();
	SvFileStream aFileStream;
	aFileStream.Open(rFile, STREAM_STD_READ);

	if (!pDocument->Update(aFileStream))
	{
		aStatus = pDocument->Status();
		CancelDBAction();
		ShowError(dbWriteError);
	}
	else
	{
		// lschen wenn neu angelegt
		if (bNew)
		{
			aFileStream.Close();
			DirEntry aEntry(rFile);
			FileStat::SetReadOnlyFlag(aEntry,FALSE);
			ULONG nError = aEntry.Kill();
			OnCreate();
			bNew = FALSE;

			// new definitions are initially locked, so decrement the lock counter as we aren't new anymore
			UnLock();
		}
		EndDBAction();
	}
	return aStatus.IsSuccessful();
}

//-------------------------------------------------------------------------
BOOL SbaFileDef::Update(SvStream& rFile)
{
	SBA_CHK_OPEN_BOOL();
	BOOL bResult = FALSE;

	BeginDBAction();

	if (!pDocument->Update(rFile))
	{
		aStatus = pDocument->Status();
		CancelDBAction();
		ShowError(dbWriteError);
	}
	else
	{
		// lschen wenn neu angelegt
		if (bNew)
		{
			OnCreate();
			bNew = FALSE;

			// new definitions are initially locked, so decrement the lock counter as we aren't new anymore
			UnLock();
		}
		EndDBAction();
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SbaFileDef::DoRefresh()
{
	BOOL bResult = pDocument->Refresh();
	if (!bResult)
	{
		bOpen = FALSE;
	}
	return bResult;
}

















