/*************************************************************************
 *
 *  $RCSfile: dbqryobj.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sbaresid.hrc"
#include "sbadb.hxx"
#include "dbstat.hxx"
#include "sbahints.hxx"
#include "sbaobj.hxx"
#include "dbqryobj.hxx"
#include "sbastr.hxx"
#include "sbaitems.hrc"
#include "sbaitems.hxx"
#include "duplname.hxx"

#ifndef _SDB_SDBTOOLS_HXX
#include <sdb/sdbtools.hxx>
#endif

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif

#ifndef _SDB_SDBQRY_HXX //autogen
#include <sdb/sdbqry.hxx>
#endif

#ifndef _URLOBJ_HXX //autogen
#include <tools/urlobj.hxx>
#endif

#ifndef _SFXINTITEM_HXX //autogen
#include <svtools/intitem.hxx>
#endif

#ifndef _SFXSTRITEM_HXX //autogen
#include <svtools/stritem.hxx>
#endif

#ifndef _SFXITEMSET_HXX //autogen
#include <svtools/itemset.hxx>
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifndef _SDB_SDBTAB_HXX //autogen
#include <sdb/sdbtab.hxx>
#endif

#ifndef _SBA_DBTABOBJ_HXX
#include "dbtabobj.hxx"
#endif // _SBA_DBTABOBJ_HXX

static char __READONLY_DATA UI_INFO[] = "UI";   // speichern von UI-Abfrageinfo

TYPEINIT1(SbaQueryObj, SbaDataObj);
DBG_NAME(SbaQueryObj);
//------------------------------------------------------------------------------
SbaQueryObj::SbaQueryObj(SbaDBDataDef* pDef,
						 const String& rSQLStatement,
						 BOOL bNative,
						 BOOL bShow)
			:SbaDataObj(pDef, rSQLStatement, bNative, pDef->Name(), bShow)
			,aParseIterator(NULL, pDef->GetDatabase())
			,pParseTree(NULL)
			,pUpdTable(NULL)
			,m_bSettingSpecialConditions(FALSE)
{
	DBG_CTOR(SbaQueryObj,NULL);
}

//------------------------------------------------------------------------------
SbaQueryObj::~SbaQueryObj()
{
	delete pParseTree;

	if (pUpdTable)
	{
		pUpdTable->ReleaseUsage();
		pUpdTable->ReleaseRef();
	}
	DBG_DTOR(SbaQueryObj,NULL);
}

//------------------------------------------------------------------------------
BOOL SbaQueryObj::Open()
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	// nur geoeffnet Objekte sind Listener
	BOOL bRes = SbaDataObj::Open();
	if (bRes && aParseIterator.IsSimple())
	{
		SbaDatabase* pDatabase = GetDatabase();
		pUpdTable = (SbaDBDataDef*)pDatabase->OpenDBDef(dbTable,(*aParseIterator.GetSdbTableRangeMap().GetObject(0))->GetFullName());
		if (pUpdTable)
		{
			pUpdTable->AddRef();
			pUpdTable->AddUsage();
		}
		bRes = pUpdTable != NULL;
	}

	// die original-Strings
	if (bRes && !m_bSettingSpecialConditions)
	{
		m_strOriginalSqlStatement = SQLStatement();
		m_strOriginalWherePart = aParseIterator.BuildPart(SbaParseIterator::WHERE);
		m_strOriginalOrderPart = aParseIterator.BuildPart(SbaParseIterator::ORDER_BY);
	}

	return bRes;
}

//------------------------------------------------------------------------------
BOOL SbaQueryObj::IsUpdateAllowed() const
{
	DBG_CHKTHIS(SbaQueryObj,NULL);

	if (!pCursor || !pCursor->IsUpdateAllowed())
		// bei IsUpdateAllowed ist nur NEIN eine verlaessliche Antwort, aber mehr brauche ich hier nicht ...
		return FALSE;

	// besteht eine Moeglichkeit, ein Object zu updaten
	SbaDBDataDefRef aDef = GetDataDef();
	return (aDef.Is() && pUpdTable && aDef->IsDataUpdateAllowed() && pUpdTable->IsDataUpdateAllowed()) ||
		   (!aDef.Is() && pUpdTable && pUpdTable->IsDataUpdateAllowed());
}

//-----------------------------------------------------------------------------
void FindParameter(SdbSqlParseNode* pNode, String& rCols, String& rParams)
{
	// durchsucht den ParseTree nach columnrefs, kann zur Beschreibung von Parametern herangezogen
	// Parameter Name-Regel gefunden?
	if (SDB_ISRULE(pNode,comparison_predicate) ||
		SDB_ISRULE(pNode,like_predicate) &&
		SDB_ISRULE(pNode->GetChild(0),column_ref))
	{
		// suchen nach Parametern
		for (ULONG i = 1; i < pNode->Count(); i++)
		{
			if (SDB_ISRULE(pNode->GetChild(i),parameter))
			{
				SdbSqlParseNode* pChild = pNode->GetChild(0);
				for (ULONG j = 0; j < pChild->Count(); j++)
					rCols += pChild->GetChild(j)->TokenValue();
				rCols += ';';

				String aFirstToken(pNode->GetChild(i)->GetChild(0)->TokenValue());
				DBG_ASSERT(aFirstToken.EqualsAscii(":") || aFirstToken.EqualsAscii("?") || aFirstToken.EqualsAscii("["),"SdbCursor: Parse Tree fehlerhaft");

				if (aFirstToken.EqualsAscii(":"))
				{
					rParams += pNode->GetChild(i)->GetChild(1)->TokenValue();
				}
				else if (aFirstToken.EqualsAscii("["))
				{
					// Name
					DBG_ASSERT(pNode->GetChild(i)->Count() == 3,"SdbCursor: Parse Tree fehlerhaft");
					DBG_ASSERT(pNode->GetChild(i)->GetChild(1)->GetNodeType() == SDB_SQL_NAME,"SdbCursor: Parse Tree fehlerhaft");

					rParams += pNode->GetChild(i)->GetChild(1)->TokenValue();
				}
				else
				{
					// Fragezeichen
					DBG_ASSERT(pNode->GetChild(i)->Count() == 1,"SdbCursor: Parse Tree fehlerhaft");
					rParams += '?';
				}
				rParams += ';';
			}
		}
	}
	else
	   // Weiter absteigen im Parse Tree
		for (UINT32 i = 0; i < pNode->Count(); i++)
			FindParameter(pNode->GetChild(i), rCols, rParams);
}


//------------------------------------------------------------------------------
BOOL SbaQueryObj::Prepare()
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	SbaDatabase* pDatabase = GetDatabase();
	if (IsPrepared())
		return TRUE;

	if (!pCursor || !pDatabase)
		return FALSE;

	BeginDBAction();

	// nur wenn das SQLStatement nicht aus einem Tabellennamen
	// besteht und Native ist erst einmal Parsen
	// um Parameternamen zu ersetzen
	BOOL bParseFirst = !aSQLStatement.Equals(aName) && !IsNative();
	if (!bParseFirst)
	{
		if (!SbaDataObj::Prepare())
		{
			EndDBAction();
			return FALSE;
		}
	}

	// Jetzt den ParseTree aufbauen
	if (!IsNative())
	{
		SdbSqlParser* pParser = pDatabase->GetSqlParser();
		String aErrorMsg;
		SdbSqlParseNode* pParseNode = pParser->ParseTree(aErrorMsg, aSQLStatement);	// versuche einen ParseTree zu erzeugen

		if (pParseNode == NULL)
		{
			aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("01000"),aErrorMsg,0,String());
			CancelDBAction();
			ShowError(dbReadError);
			return FALSE;
		}
		else
		{
			pParseTree = pParseNode;
			aParseIterator.SetParseTree(pParseTree);

			// werden Parameter verwendet, dann die Strukturbeschreibung nachbessern
			if (pCursor)
			{
				SdbColumnsRef aCols = pCursor->GetParameterColumns();
				String		  aParamNameList, aColNameList;

				// alle Parameter mit Namen
				FindParameter(pParseTree, aColNameList, aParamNameList);
				while (aColNameList.Len())
				{
					String aColName(aColNameList.GetToken(0));
					const SdbColumn*	pColItem = aParseIterator.GetCol(aColName);
					if (pColItem)
					{
						SdbColumn *pClone = pColItem->Clone();
						pClone->SetFlag(SDB_FLAGS_PARAM_IN); // Standard, solange nichts anderes gesagt wird
						aCols->AddColumn(pClone);
						aColNameList.Erase(0,aColName.Len());
						aColNameList.EraseLeadingChars(';');

						aParamNameList.Erase(0, aParamNameList.GetToken(0).Len());
						aParamNameList.EraseLeadingChars(';');
					}
					else
						aColNameList.Erase();
				}
			}
		}
	}

	if (bParseFirst && !SbaDataObj::Prepare())
	{
		EndDBAction();
		return FALSE;
	}

	EndDBAction();
	return IsPrepared();
}

//------------------------------------------------------------------------------
BOOL SbaQueryObj::IsPrepared() const
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	return SbaDataObj::IsPrepared() && (pParseTree != NULL || IsNative());
}

//------------------------------------------------------------------------------
SbaPredicateItem* SbaQueryObj::CreatePredicateItem(const String& rFieldName, const ODbVariantRef& xVar)
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	if (xVar.is())
	{
		const SdbColumn* pCol = aParseIterator.GetCol(rFieldName);
		if (!pCol)
			return NULL;

		SbaPredicateItem* pItem = new SbaPredicateItem;
		SbaDBDataDef* pDef = GetDataDef();

		// Init LogTyp und Feldname
		pItem->aFieldName	= rFieldName;
		pItem->eLogType		= SBA_LOG_INVALID;

		// Init Typ
		if (xVar->isNull())
			pItem->eType = SDB_PRED_ISNULL;
		else if (pCol->GetAllFlags() & SDB_FLAGS_SEARCH_EXCEPT_LIKE)
			pItem->eType = SDB_PRED_EQUAL;
		else if (pCol->GetAllFlags() & SDB_FLAGS_SEARCH_LIKE)
			pItem->eType = SDB_PRED_LIKE;
		else
			pItem->eType = SDB_PRED_ISNOTNULL;

		if (pDef)
		{
			SfxItemSet* pSet = pDef->GetColumn(rFieldName);
			switch (PTR_CAST(SbaDataFieldTypeItem,&pSet->Get(SBA_DEF_FLTTYPE))->GetValue())
			{
				// Fuer Datumswerte Sonderbehandlung
				case dbDateTime:
					pItem->eVarType = SDB_DBTYPE_TIMESTAMP;
					pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, TRUE );
					break;
				case dbDate:
					pItem->eVarType = SDB_DBTYPE_DATE;
					pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, TRUE );
					break;
				case dbTime:
					pItem->eVarType = SDB_DBTYPE_TIME;
					pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, TRUE );
					break;
				default:
					pItem->eVarType	= pCol->GetType();
					pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, FALSE );
			}
		}
		else
		{
			pItem->eVarType	= pCol->GetType();
			pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, FALSE );
		}

		// like nur bis max. 255 Zeichen
		if (pItem->eType == SDB_PRED_LIKE && pItem->aValue.Len() > 255)
		{
			pItem->aValue.Erase(254);
			pItem->aValue += '%';
		}
		return pItem;
	}
	else
		return NULL;

}

//------------------------------------------------------------------------------
void SbaQueryObj::SetAutoFilter(const String& rFieldName, const ODbVariantRef& xVar, BOOL bAppend)
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	if (xVar.is())
	{
		// Aus Parametern neues PredicateItem erzeugen und in Liste einfuegen
		SbaPredicateItem* pItem = CreatePredicateItem( rFieldName, xVar );
		SbaPredicateItemList aList;

		aList.Insert(pItem, LIST_APPEND);
		String aNewWherePart = aParseIterator.BuildWherePart(aList);
		delete pItem;		// Listenelement wieder zerstoeren

		// Alte und neue Kriterien in SQL String zusammenfuegen
		String	aSqlCmd(STR_SELECT);
		aSqlCmd += aParseIterator.BuildColumnPart();
		aSqlCmd += STR_FROM;
		aSqlCmd += aParseIterator.BuildTablePart();
		aSqlCmd += STR_WHERE;

		String sOldWherePart = bAppend ? aParseIterator.BuildPart(SbaParseIterator::WHERE) : m_strOriginalWherePart;
		if (sOldWherePart.Len())
		{
			aSqlCmd += '(';
			aSqlCmd += sOldWherePart;
			aSqlCmd += ')';
			aSqlCmd += STR_AND;
			aSqlCmd += aNewWherePart;
		}
		else
			aSqlCmd += aNewWherePart;

		aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::ORDER_BY, TRUE);
		aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::HAVING, TRUE);
		aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::GROUP_BY, TRUE);

		m_bSettingSpecialConditions = TRUE;
		ReInit(aSqlCmd);
		m_bSettingSpecialConditions = FALSE;
	}
}

//------------------------------------------------------------------------------
void SbaQueryObj::Reset()
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	SbaDataObj::ReInit(m_strOriginalSqlStatement);
}

//------------------------------------------------------------------------------
void SbaQueryObj::SetAutoOrder(const String& rFieldName, const ODbVariantRef& xVar, BOOL bDesc)
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	if (xVar.is())
	{
		SbaOrderItemList	aList;
		SbaOrderItem		aItem;

		aItem.aFieldName = rFieldName;
		aItem.bDirection = bDesc;
		aList.Insert( new SbaOrderItem( aItem ), LIST_APPEND );
		String aNewOrderPart = aParseIterator.BuildOrderPart( aList );

		SbaOrderItem* pItem = aList.First();
		while (pItem)
		{
			delete pItem;
			pItem = aList.Next();
		}
		aList.Clear();

		String	aSqlCmd(STR_SELECT);
		aSqlCmd += aParseIterator.BuildColumnPart();
		aSqlCmd += STR_FROM;
		aSqlCmd += aParseIterator.BuildTablePart();
		aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::WHERE, TRUE);
		aSqlCmd += STR_ORDER_BY;
		aSqlCmd += aNewOrderPart;
		if (m_strOriginalOrderPart.Len())
		{
			aSqlCmd.AppendAscii(", ");
			aSqlCmd += m_strOriginalOrderPart;
		}

		aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::HAVING, TRUE);
		aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::GROUP_BY, TRUE);

		m_bSettingSpecialConditions = TRUE;
		ReInit(aSqlCmd);
		m_bSettingSpecialConditions = FALSE;
	}
}

//------------------------------------------------------------------------------
void SbaQueryObj::Close()
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	DELETEZ(pParseTree);
	aParseIterator.SetParseTree(NULL);

	if (pUpdTable)
	{
		pUpdTable->ReleaseUsage();
		pUpdTable->ReleaseRef();
		pUpdTable = NULL;
	}
	SbaDataObj::Close();
}

//------------------------------------------------------------------------------
void SbaQueryObj::SetFilter(const String& rFilter)
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	String	aSqlCmd(STR_SELECT);
	aSqlCmd += aParseIterator.BuildColumnPart();
	aSqlCmd += STR_FROM;
	aSqlCmd += aParseIterator.BuildTablePart();

	if (rFilter.Len())
	{
		aSqlCmd += STR_WHERE;
		// (alter Filter) AND (neuer Filter)
		if (m_strOriginalWherePart.Len())
		{
			aSqlCmd += '(';
			aSqlCmd += m_strOriginalWherePart;
			aSqlCmd.AppendAscii(") AND (");
		}
		aSqlCmd += rFilter;
		if (m_strOriginalWherePart.Len())
			aSqlCmd += ')';
	}
	else if (m_strOriginalWherePart.Len())
	{
		aSqlCmd += STR_WHERE;
		// nur alter Filter
		aSqlCmd += m_strOriginalWherePart;
	}

	aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::ORDER_BY, TRUE);
	aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::HAVING, TRUE);
	aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::GROUP_BY, TRUE);

	m_bSettingSpecialConditions = TRUE;
	ReInit(aSqlCmd);
	m_bSettingSpecialConditions = FALSE;
}


//------------------------------------------------------------------------------
void SbaQueryObj::SetOrder(const String& rOrder)
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	String	aSqlCmd(STR_SELECT);
	aSqlCmd += aParseIterator.BuildColumnPart();
	aSqlCmd += STR_FROM;
	aSqlCmd += aParseIterator.BuildTablePart();
	aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::WHERE, TRUE);

	if (rOrder.Len())
	{
		aSqlCmd += STR_ORDER_BY;
		aSqlCmd += rOrder;
		if (m_strOriginalOrderPart.Len())
		{
			aSqlCmd.AppendAscii(", ");
			aSqlCmd += m_strOriginalOrderPart;
		}
	}
	else if (m_strOriginalOrderPart.Len())
	{
		aSqlCmd += STR_ORDER_BY;
		aSqlCmd += m_strOriginalOrderPart;
	}

	aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::HAVING, TRUE);
	aSqlCmd += aParseIterator.BuildPart(SbaParseIterator::GROUP_BY, TRUE);

	m_bSettingSpecialConditions = TRUE;
	ReInit(aSqlCmd);
	m_bSettingSpecialConditions = FALSE;
}

//------------------------------------------------------------------------------
BOOL SbaQueryObj::Refresh()
{
	DBG_CHKTHIS(SbaQueryObj,NULL);
	m_bSettingSpecialConditions = TRUE;
	ReInit(aSQLStatement);
	m_bSettingSpecialConditions = FALSE;
	return aStatus.IsSuccessful();
}

TYPEINIT1(SbaQueryDef, SbaDBDataDef);
DBG_NAME(SbaQueryDef);
//------------------------------------------------------------------------------
SbaQueryDef::SbaQueryDef(SbaDatabase* pDB,
						 const String& rName,
						 BOOL _bNew,
						 BOOL bShow)
			:SbaDBDataDef(pDB, rName, dbQuery, _bNew, bShow)
			,pQuery(NULL)
			,bNative(FALSE)
{
	DBG_CTOR(SbaQueryDef,NULL);
	// Name kann auch ein SQLStatement enthalten
	// dann handelt es sich um eine temporaere Definition
	// die nur fuer die Anzeige eines SQL-Statements im Browser
	// gedacht ist, diese temp Definition kann nicht gespeichert werden!
	if (rName.GetTokenCount(SDB_STATEMENT_DELIMETER) == 3)
	{
		String aString(rName);
		aName.Erase();
		bNative	    = (aString.GetToken(1,SDB_STATEMENT_DELIMETER).ToInt32()) != 0 ;
		aStatement  = aString.GetToken(2,SDB_STATEMENT_DELIMETER);
	}
	else
	{	if (IsNew())
			pQuery = (SdbQuery*)pDB->CreateObject(dbQuery, aName);
		else
			pQuery = (SdbQuery*)pDB->OpenObject(dbQuery, aName);

		if (pQuery)
		{
			pQuery->AddRef();
			bOpen = IsNew();
		}
		else
		{
			aStatus = pDB->Status();
			ShowError(IsNew() ? dbWriteError : dbReadError);
		}
	}
}

//------------------------------------------------------------------------------
void SbaQueryDef::QueryDelete()
// temp Objekte werden in keiener liste vom Parent gehalten und koennen
// sofort geloescht werden
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	if (IsTemp())
	{
		RemoveChildren();
		delete this;
	}
	else
		SbaDBDataDef::QueryDelete();
}

//------------------------------------------------------------------------------
SbaDataObj* SbaQueryDef::CreateDataObjImpl()
{
	return new SbaQueryObj(this, aStatement, IsNative(),bShowError);
}

//------------------------------------------------------------------------------
SbaQueryDef::~SbaQueryDef()
{
	DBG_DTOR(SbaQueryDef,NULL);
	if (pQuery)
	{
		pQuery->ReleaseRef();
		pQuery = NULL;
	}
}

//------------------------------------------------------------------------------
BOOL SbaQueryDef::Open()
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	if (IsOpen())
		return TRUE;
	else if (IsTemp())		//nur die Spaltenstruktur setzen
	{
		BeginDBAction();
		if (!DescribeCols(aStatement, IsNative()))
		{
			CancelDBAction();
			ShowError(dbReadError);
			return FALSE;
		}
		else
		{
			EndDBAction();
			bOpen = TRUE;
			return TRUE;
		}
	}
	else
	{
		BeginDBAction();
		if (!Load())
		{
			CancelDBAction();
			ShowError(dbReadError);
			return FALSE;
		}
		// aktualisieren der Spaltenbeschreibung
		DescribeCols(pQuery->SQLStatement(),pQuery->IsNative());
		bOpen = TRUE;
		bNative = pQuery->IsNative();
		aStatement = pQuery->SQLStatement();
		EndDBAction();
	}
	return bOpen;
}

//------------------------------------------------------------------------------
BOOL SbaQueryDef::DoRefresh()
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	if (IsNew() || IsTemp())
		return TRUE;

	BOOL bResult = pQuery->Refresh();
	if (!bResult)
	{
		bOpen = FALSE;
	}
	else
	{
		bOpen = FALSE;
		bResult = Open();
	}
	return bResult;
}

//------------------------------------------------------------------------------
// Auslesen von UI-Informationen zur Abfrage (werden von der QueryObjectschell)
// ausgewertet
BOOL SbaQueryDef::GetUIInfo(SvStream& rTarget)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	DBG_ASSERT(!IsTemp(), "Temporre Abfragedefinition, darf nur geoeffnet werden");
	aStatus.Clear();

	if (!IsOpen() || IsNew() || IsNative() || !BeginTrans(FALSE))
		return FALSE;

	SbaDatabase* pDatabase = GetDatabase();
	SvStorageStreamRef aStream;
	SdbStorageRef aQueryStor = pDatabase->OpenStorage(*aDBStor, aName, STREAM_STD_READ | STREAM_NOCREATE );
	if (!aQueryStor.Is())
	{
		EndTrans(FALSE);
		return FALSE;
	}
	else
	{
		SvStorageStreamRef aStream = pDatabase->OpenStream(*aQueryStor, String::CreateFromAscii(UI_INFO), STREAM_STD_READ);
		if (!aStream.Is())
		{
			EndTrans(FALSE);
			return FALSE;
		}

		ULONG nError = CopyTo(rTarget, *aStream);// Stream uebergeben
		EndTrans(FALSE);
		return (nError == ERRCODE_NONE);
	}
}

//------------------------------------------------------------------------------
BOOL SbaQueryDef::RenameImpl(const String& rNewName)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	DBG_ASSERT(!IsTemp(), "Temporre Abfragedefinition, darf nur geoeffnet werden");
	if (!pQuery->Rename(rNewName))
		aStatus = pQuery->Status();
	else
		aName = rNewName;
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SbaQueryDef::DropImpl()
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	DBG_ASSERT(!IsTemp(), "Temporre Abfragedefinition, darf nur geoeffnet werden");
	if (!pQuery->Drop())
		aStatus = pQuery->Status();
	return aStatus.IsSuccessful();
}

//-------------------------------------------------------------------------
BOOL SbaQueryDef::UpdateImpl(const String& aStmt, BOOL _bNative)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	if (!DescribeCols(aStmt, _bNative))
		return FALSE;

	// Updaten des SQL Statements
	if (!pQuery->Update(aStmt, _bNative))
	{
		aStatus = pQuery->Status();
	}
	else
	{
		// Stream fuer UI-Info aus Abfrageentwurf lschen
		SbaDatabase*  pDatabase = GetDatabase();
		SdbStorageRef aQueryStor = pDatabase->OpenStorage(*aDBStor, aName, STREAM_STD_READWRITE);
		if (!aQueryStor.Is() || !aQueryStor->Remove(String::CreateFromAscii(UI_INFO),TRUE))
			aStatus = pDatabase->Status();
	}
	// Spalten Info Speichern
	return aStatus.IsSuccessful() && Store();
}

//-------------------------------------------------------------------------
BOOL SbaQueryDef::Update(const String& aStmt, BOOL _bNative)
// Speichern eines Statements ohne UI-Info
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	DBG_ASSERT(!IsTemp(), "Temporre Abfragedefinition, darf nur geoeffnet werden");
	aStatus.Clear();

	BeginDBAction();
	if (!BeginTrans(TRUE) || !UpdateImpl(aStmt, _bNative) || !Commit())
	{
		EndTrans(FALSE);
		CancelDBAction();
		ShowError(dbWriteError);
		return FALSE;
	}

	aStatement = aStmt;
	bNative	   = _bNative;

	EndDBAction();
	if (IsNew())
	{
		OnCreate();
		bNew = FALSE;
	}
//	bLocked = FALSE;
	aLockCounter = 0;
	return TRUE;
}

//-------------------------------------------------------------------------
BOOL SbaQueryDef::Update(const String& aStmt, BOOL _bNative, SvStream* pStream)
// Speichern eines Statements mit UI-Info
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	DBG_ASSERT(!IsTemp(), "Temporre Abfragedefinition, darf nur geoeffnet werden");
	aStatus.Clear();

	SbaDatabase* pDatabase = GetDatabase();
	SvStorageStreamRef aStream;
	SdbStorageRef aQueryStor;

	BeginDBAction();
	if (!BeginTrans(TRUE) || !UpdateImpl(aStmt, _bNative))
		;
	// Speichern der UI-Info
	else if (!_bNative && pStream)
	{
		aQueryStor = pDatabase->OpenStorage(*aDBStor, aName, STREAM_STD_READWRITE);
		if (!aQueryStor.Is())
			aStatus = pDatabase->Status();
		else
		{
			aStream = pDatabase->OpenStream(*aQueryStor, String::CreateFromAscii(UI_INFO), STREAM_STD_READWRITE | STREAM_TRUNC);
			if (!aStream.Is())
				aStatus = pDatabase->Status();
			else
			{
				ULONG nError = CopyTo(*aStream, *pStream);		// Stream uebernehmen
				if (nError != ERRCODE_NONE)
					aStatus.SetError(nError,edbObj,aName);
			}
		}
	}

	if (aStatus.IsSuccessful())
		Commit();

	if (aStatus.IsSuccessful())
	{
		aStatement	= aStmt;
		bNative		= _bNative;


		EndDBAction();

		if (IsNew())
		{
			OnCreate();
			bNew = FALSE;
		}
//		bLocked = FALSE;
		aLockCounter = 0;
	}
	else
	{
		EndTrans(FALSE);
		CancelDBAction();
		ShowError(dbWriteError);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SbaQueryDef::DescribeCols(const String& aStmt, BOOL _bNative)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	SbaDatabase* pDatabase = GetDatabase();
	// zunaechst die Beschreibung von der Datenbank besorgen
	SdbColumnsRef xCols = pDatabase->DescribeResult(aStmt,FALSE);
	if (!xCols.Is())
	{
		aStatus = pDatabase->Status();
		return FALSE;
	}
	else
		return SetColumns(xCols, aStmt, _bNative);
}
//------------------------------------------------------------------------------
BOOL SbaQueryDef::SetLock(BOOL _bLocked)
{
	// QueryDefs werden noch nicht gelocked
	return IsLocked();
}
//------------------------------------------------------------------------------
BOOL SbaQueryDef::SetColumns(const SdbColumnsRef& rCols, const String& aStmt, BOOL _bNative)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	if (!rCols.Is())
		return FALSE;

	// dann einen Abgleich mit der bestehenden Beschreibung vornehmen
	// Wenn ein gueltiger ParseTree existiert, dann koennen fehlende Spaltenbeschreibungen
	// uber die Tabellendefinition ersetzt werden, ansonsten werden default Beschreibungen gesetzt
	// Wenn ein native Statement uebergeben wird, wird nur ein Merge vorgenommen und gut ist
	if (_bNative)
	{
		Merge(*rCols);
		return TRUE;
	}

	SbaDatabase* pDatabase = GetDatabase();
	SdbSqlParser* pParser = pDatabase->GetSqlParser();
	String aErrorMsg;
	SdbSqlParseNode* pParseNode = pParser->ParseTree(aErrorMsg, aStmt);	// versuche einen ParseTree zu erzeugen
	if(!pParseNode)
		return FALSE;
	SbaParseIterator aIterator(pParseNode, pDatabase);

	// jetzt mu die Anzahl Spalten uebereinstimmen, sonst ist eine Bestimmung nicht moeglich
//	DBG_ASSERT(rCols->Count() - 1 == (USHORT)aColumnNameList.Count(), "Falsche Spaltenanzahl!");

	if (rCols->Count() != (USHORT)aIterator.GetSelectColumns()->Count())
	{
		xOriginalColumns->Clear();
		xColumns->Clear();
		Merge(*rCols);
	}
	else
	{
		SbaDBDefList  aTableList((pDatabase->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT);
		// zunaechst alle Tabellendefinitionen laden
		ReadTables(aTableList, aIterator.GetSdbTableRangeMap());

		// fill in the table names of the connection columns (if neccessary)
		const SdbColumnsRef aIterCols = aIterator.GetSelectColumns();
		for (UINT16 i=1; i<rCols->Count(); ++i)
		{
			const SdbColumn* pConnCol = rCols->Column(i);
			if (!pConnCol || pConnCol->GetTableName().Len())
				continue;
			const SdbColumn* pIterCol = aIterCols->Column(pConnCol->GetName());
			if (pIterCol)
				const_cast<SdbColumn*>(pConnCol)->SetTableName(pIterCol->GetTableName());
		}

		FillCols(rCols, aTableList, aIterator.GetSdbTableRangeMap());
	}

	delete pParseNode;

	return TRUE;
}

//------------------------------------------------------------------------------
void SbaQueryDef::FillCols(const SdbColumnsRef& rCols, const SbaDBDefList& rTableList,
						   const SdbTableRangeMap& rTableMap)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	SbaDatabase* pDatabase = GetDatabase();
	if (!pDatabase)
		return;

	SbaColumnListRef xMergeList = new SbaColumnList(pDatabase->GetIdentifierCase());
	SbaColumnRef  xSbaColumn;
	String aFullName;
	for (UINT16 i = 1; i < rCols->Count(); i++,aFullName = String())
	{
		const SdbColumn* pCol = rCols->Column(i);
		String sName = pCol->GetName();

		// in der List von SdbColumn wird als Tabellenname der Alias gespeichert
		SbaTableDef* pTableDef = (SbaTableDef*)rTableList.Lookup(pCol->GetTableName());
		SdbTable* pTable = pTableDef ? pTableDef->GetTable() : NULL;

		if (pTable) // gibt es schon eine Beschreibung fr die Spalte
			aFullName =  pTable->GetFullName();

		xSbaColumn = GetColumn(sName, aFullName);
		if (xSbaColumn.Is()) // definition exitiert
		{
			xOriginalColumns->Remove(&xSbaColumn);
			xSbaColumn->Put(SfxUInt16Item(SBA_DEF_FLTPOS, pCol->GetId()));
			xSbaColumn->Put(SfxUInt16Item(SBA_DEF_FLTSEARCH, pCol->GetAllFlags()));
			xMergeList->InsertAt(sName, &xSbaColumn , LIST_APPEND);
		}
		else
		{
			// wird die Spalte nicht gefunden, dann die Tabellenliste auswerten
			if(pTableDef)
			{
				xSbaColumn = pTableDef->GetColumn(sName);

				// Definition aus der Tabelle verwenden
				if (xSbaColumn.Is())
				{
					xSbaColumn = (SbaColumn*)xSbaColumn->Clone(TRUE,GetPool());
					xSbaColumn->Put(SfxStringItem(SBA_DEF_FLTTABLE, aFullName));
					xSbaColumn->Put(SfxUInt16Item(SBA_DEF_FLTPOS, pCol->GetId()));
					xSbaColumn->Put(SfxUInt16Item(SBA_DEF_FLTSEARCH, pCol->GetAllFlags()));
					// Hide ausschalten, sonst bekommen wir die Spalte nicht zu sehen
					xSbaColumn->Put(SfxBoolItem(SBA_DEF_FLTHIDE, (pCol->GetAllFlags() & SDB_FLAGS_VERSIONING)==SDB_FLAGS_VERSIONING));
					xMergeList->InsertAt(sName, xSbaColumn);
				}
				else // keine Def gefunden
				{
					SbaColumn* pNewSbaCol = CreateColumn(*pCol);
					pNewSbaCol->Put(SbaNameItem(SBA_DEF_FLTNAME, sName, pDatabase->GetIdentifierCase() == SDB_IC_ALL));
					xMergeList->InsertAt(sName, pNewSbaCol);
				}
			}
			else // keine Def gefunden
			{
				SbaColumn* pNewSbaCol = CreateColumn(*pCol);
				pNewSbaCol->Put(SbaNameItem(SBA_DEF_FLTNAME, sName, pDatabase->GetIdentifierCase() == SDB_IC_ALL));
				xMergeList->InsertAt(sName, pNewSbaCol);
			}
		}
	}
	xOriginalColumns = xMergeList;
	*xColumns = *xOriginalColumns;
}

//------------------------------------------------------------------------------
void SbaQueryDef::ReadTables(SbaDBDefList& rTableList, const SdbTableRangeMap& rTableItemList)
{
	DBG_CHKTHIS(SbaQueryDef,NULL);
	SbaDatabase* pDatabase = GetDatabase();
	String aTableName;
	for (ULONG i = 0; i < rTableItemList.Count(); i++)
	{
		aTableName = (*rTableItemList.GetObject(i))->GetFullName();
		if (rTableList.Lookup(aTableName))
			continue;
		else
		{
			SbaDBDef* pTable = pDatabase->OpenDBDef(dbTable, aTableName, FALSE);
			if (pTable)
				rTableList.InsertAt(pTable->Name(),pTable);
		}
	}
}

















