/*************************************************************************
 *
 *  $RCSfile: dbstat.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sbaresid.hrc"
#include "dbstat.hxx"
#include "sbares.hxx"

DBG_NAME(SbaStatus);

// Leerstring
const String SbaStatus::aEmptyString;

//------------------------------------------------------------------------------
SbaStatus::SbaStatus() : eResult(SBA_SUCCESS)
{
	DBG_CTOR(SbaStatus,NULL);
}

//------------------------------------------------------------------------------
SbaStatus::SbaStatus(const SbaStatus &rSbaStatus)
{
	DBG_CTOR(SbaStatus,NULL);
	DBG_CHKOBJ(&rSbaStatus,SbaStatus,NULL);

	eResult = rSbaStatus.Result();
	nMessageNumber = rSbaStatus.MessageNumber();
	aMessage = rSbaStatus.Message();
	aSdbStatus = rSbaStatus.SdbStatusObject();
	bSdbStatusAvailable = rSbaStatus.IsSdbStatusAvailable();

	if (IsSdbStatusAvailable())
		aSdbStatus = rSbaStatus.SdbStatusObject();
	else
		aSdbStatus.Clear();	// Ungueltig.
}

//------------------------------------------------------------------------------
SbaStatus::~SbaStatus()
{
	DBG_DTOR(SbaStatus,NULL);
}

//------------------------------------------------------------------------------
const SbaStatus & SbaStatus::operator=(const SbaStatus &rSbaStatus)
{
	DBG_CHKTHIS(SbaStatus,NULL);
	DBG_CHKOBJ(&rSbaStatus,SbaStatus,NULL);

	// Absicherung gegen Objekt-Identitaet nicht erforderlich,
	// da StarView String-Selbstzuweisung erlaubt.

	eResult = rSbaStatus.Result();
	nMessageNumber = rSbaStatus.MessageNumber();
	aMessage = rSbaStatus.Message();
	bSdbStatusAvailable = rSbaStatus.IsSdbStatusAvailable();

	if (IsSdbStatusAvailable())
		aSdbStatus = rSbaStatus.SdbStatusObject();
	else
		aSdbStatus.Clear();	// Ungueltig.

	return *this;
}

//------------------------------------------------------------------------------
sal_Bool SbaStatus::operator==(const SbaStatus &rSbaStatus) const
{
	DBG_CHKTHIS(SbaStatus,NULL);
	DBG_CHKOBJ(&rSbaStatus,SbaStatus,NULL);

	if (IsInformationAvailable()) {
		return eResult == rSbaStatus.Result() &&
			   nMessageNumber == rSbaStatus.MessageNumber() &&
			   aMessage == rSbaStatus.Message() &&
			   bSdbStatusAvailable == rSbaStatus.IsSdbStatusAvailable();
			   aSdbStatus == rSbaStatus.aSdbStatus;
	} else {
		return eResult == rSbaStatus.Result();
	}
}

//------------------------------------------------------------------------------
void SbaStatus::Set(SbaStatusCode eNewResult, DBErrorContext eContext,
					  const char * pParam1, const char * pParam2, const char * pParam3)
{
	DBG_CHKTHIS(SbaStatus,NULL);

	DBG_ASSERT(eNewResult == SBA_SUCCESS || eNewResult == SBA_SUCCESS_WITH_INFO ||
			   eNewResult == SBA_NO_DATA_FOUND || eNewResult == SBA_ERROR, "SbaStatus: invalid Result Code");

	eResult = eNewResult;
	switch (eContext)
	{
		case dbSyntaxError:
			nMessageNumber = SBA_STAT_SYNTAXERROR;
			break;
		case dbWriteError:
			nMessageNumber = SBA_STAT_WRITEERROR;
			break;
		case dbDeleteError:
			nMessageNumber = SBA_STAT_DELETEERROR;
			break;
		case dbRenameError:
			nMessageNumber = SBA_STAT_RENAMEERROR;
			break;
		case dbWarning:
			nMessageNumber = SBA_STAT_WARNING;
			break;
		default:
			nMessageNumber = SBA_STAT_READERROR;
	}
	aMessage = String(SbaResId(nMessageNumber));
// nyi: Parameter einsetzen!

	bSdbStatusAvailable = sal_False;
	aSdbStatus.Clear();		// Der Sdb-Status ist in diesem Fall ohnehin ungueltig.

	aStatusChangeHdl.Call(this);
}

//------------------------------------------------------------------------------
void SbaStatus::Set(SbaStatusCode eNewResult, const SdbStatus & rNewSdbStatus, DBErrorContext eContext,
					  const char * pParam1, const char * pParam2, const char * pParam3)
{
	DBG_CHKTHIS(SbaStatus,NULL);
	DBG_ASSERT(eNewResult == SBA_SUCCESS || eNewResult == SBA_SUCCESS_WITH_INFO ||
			   eNewResult == SBA_NO_DATA_FOUND || eNewResult == SBA_ERROR, "SbaStatus: invalid Result Code");


	eResult = eNewResult;
	switch (eContext)
	{
		case dbSyntaxError:
			nMessageNumber = SBA_STAT_SYNTAXERROR;
			break;
		case dbWriteError:
			nMessageNumber = SBA_STAT_WRITEERROR;
			break;
		case dbDeleteError:
			nMessageNumber = SBA_STAT_DELETEERROR;
			break;
		case dbRenameError:
			nMessageNumber = SBA_STAT_RENAMEERROR;
			break;
		case dbWarning:
			nMessageNumber = SBA_STAT_WARNING;
			break;
		default:
			nMessageNumber = SBA_STAT_READERROR;
	}
	aMessage = String(SbaResId(nMessageNumber));

// nyi: Parameter einsetzen!

	bSdbStatusAvailable = sal_True;
	aSdbStatus = rNewSdbStatus;

	aStatusChangeHdl.Call(this);
}

//------------------------------------------------------------------------------
void SbaStatus::Set(SbaStatusCode eNewResult, const ::com::sun::star::data::DatabaseDetailedException& rUnoDescription)
{
	DBG_CHKTHIS(SbaStatus,NULL);
	DBG_ASSERT(eNewResult == SBA_SUCCESS || eNewResult == SBA_SUCCESS_WITH_INFO ||
			   eNewResult == SBA_NO_DATA_FOUND || eNewResult == SBA_ERROR, "SbaStatus: invalid Result Code");

	eResult = eNewResult;
	switch (rUnoDescription.ExceptionContext)
	{
		case ::com::sun::star::data::DatabaseExceptionContext_WRITE:
			nMessageNumber = SBA_STAT_WRITEERROR;
			break;
		case ::com::sun::star::data::DatabaseExceptionContext_SYNTAX:
			nMessageNumber = SBA_STAT_SYNTAXERROR;
			break;
		case ::com::sun::star::data::DatabaseExceptionContext_DELETE:
			nMessageNumber = SBA_STAT_DELETEERROR;
			break;
		case ::com::sun::star::data::DatabaseExceptionContext_CONNECT:
			nMessageNumber = SBA_STAT_CONNECT_ERROR;
			break;
		default:
			nMessageNumber = SBA_STAT_READERROR;
	}
	aMessage = String(SbaResId(nMessageNumber));

	aSdbStatus.Set(SDB_STAT_ERROR, rUnoDescription.SQLState, rUnoDescription.Message,
		rUnoDescription.ErrorCode, rUnoDescription.Description
	);
	bSdbStatusAvailable = sal_True;

	aStatusChangeHdl.Call(this);
}


