/*************************************************************************
 *
 *  $RCSfile: sbadbit.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sbares.hxx"
#include "sbaresid.hrc"
#include "sbadbit.hxx"
#include "dbtabobj.hxx"
#include "sbastr.hxx"

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifndef _SDB_SDBTAB_HXX //autogen
#include <sdb/sdbtab.hxx>
#endif

#ifndef _SDB_SDBDB_HXX //autogen
#include <sdb/sdbdb.hxx>
#endif

const String STR_ORDER_BY = String::CreateFromAscii(" ORDER BY ");
const String STR_WHERE = String::CreateFromAscii(" WHERE ");
const String STR_GROUP_BY = String::CreateFromAscii(" GROUP BY ");
const String STR_HAVING = String::CreateFromAscii(" HAVING ");
const String STR_AND = String::CreateFromAscii(" AND ");
const String STR_SELECT = String::CreateFromAscii("SELECT ");
const String STR_FROM = String::CreateFromAscii(" FROM ");

//------------------------------------------------------------------------------
void ReplaceChars( String& rStr, sal_Unicode nOrg, sal_Unicode nNew )
{
	xub_StrLen	nLen = rStr.Len();

	for (xub_StrLen i=0; i<nLen; ++i)
	{
		if (rStr.GetChar(i) == nOrg)
			rStr.SetChar(i, nNew);
	}
}

//------------------------------------------------------------------------------
String QuoteField( const String& rValue, SdbDatabaseType aType )
{
	String rNewValue;
	switch (rValue.GetChar(0))
	{
		case '?':
			if (rValue.Len() != 1)
				break;
		case '\'':	// String Quotierung oder Datum
		//case '#':	// Datumsquotierung // jetengine
		case ':':	// Parameter
		case '[':	// Parameter
			return rValue;
	}

	switch( aType )
	{
		case SDB_DBTYPE_DATE:
		case SDB_DBTYPE_TIME:
		case SDB_DBTYPE_TIMESTAMP:
			if (rValue.GetChar(0) != '{') // nur quoten, wenn kein Access Datum
				rNewValue = SdbConnection::StringQuotation(rValue);
			else
				rNewValue = rValue;
			break;
		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_LONGVARCHAR:
			rNewValue = SdbConnection::StringQuotation(rValue);
			break;
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
		case SDB_DBTYPE_TINYINT:
		case SDB_DBTYPE_SMALLINT:
		case SDB_DBTYPE_INTEGER:
		case SDB_DBTYPE_BIGINT:
		case SDB_DBTYPE_REAL:
		case SDB_DBTYPE_DOUBLE:
		case SDB_DBTYPE_BOOKMARK:
		case SDB_DBTYPE_BINARY:
		case SDB_DBTYPE_VARBINARY:
		case SDB_DBTYPE_LONGVARBINARY:
		case SDB_DBTYPE_CURRENCY:
			rNewValue = rValue;
			break;
		case SDB_DBTYPE_BOOLEAN:
			if (SbaDBObj::EqualNamesInsensitive(rValue, SbaResId(SBA_STR_TRUE)))
				rNewValue.AssignAscii("TRUE");
			else if (SbaDBObj::EqualNamesInsensitive(rValue, SbaResId(SBA_STR_FALSE)))
				rNewValue.AssignAscii("FALSE");
			else
				rNewValue = rValue;
			break;
		default:
			DBG_ERROR( "QuoteField: illegal type" );
		break;
	}
	return rNewValue;
}

DBG_NAME(SbaParseIterator);
//------------------------------------------------------------------------------
SbaParseIterator::SbaParseIterator()
				 :pWhereTree(0)
				 ,pOrderTree(0)
				 ,pTempCriteriaList(0)
				 ,pTempOrderList(0)
{
	DBG_CTOR(SbaParseIterator,NULL);
}

//------------------------------------------------------------------------------
SbaParseIterator::SbaParseIterator(const SdbSqlParseNode* pRootNode, SdbDatabase* pDb)
					:SdbSqlParseTreeIterator(pDb,pRootNode)
				 ,pWhereTree(0)
				 ,pOrderTree(0)
				 ,pTempCriteriaList(0)
				 ,pTempOrderList(0)
{
	DBG_CTOR(SbaParseIterator,NULL);
	SetParseTree(pRootNode);
}

//------------------------------------------------------------------------------
SbaParseIterator::SbaParseIterator(const SdbSqlParseNode* pRootNode, SdbConnection* pConn)
				 :SdbSqlParseTreeIterator(pConn,pRootNode)
				 ,pWhereTree(0)
				 ,pOrderTree(0)
				 ,pGroupByTree(0)
				 ,pHavingTree(0)
				 ,pTempCriteriaList(0)
				 ,pTempOrderList(0)
{
	DBG_CTOR(SbaParseIterator,NULL);
	SetParseTree(pRootNode);
}

//------------------------------------------------------------------------------
SbaParseIterator::~SbaParseIterator()
{
	DBG_DTOR(SbaParseIterator,NULL);
}

//------------------------------------------------------------------------------
void SbaParseIterator::SetParseTree( const SdbSqlParseNode* pRootNode)
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	pWhereTree = pOrderTree = pHavingTree = pGroupByTree = NULL;
	SdbSqlParseTreeIterator::SetParseTree( pRootNode );
	if (IsValid())
	{
		TraverseTableNames();
		if(!aIteratorStatus.IsSuccessful())
			return;

		const SdbSqlParseNode* pSelectNode = pRootNode;
		TraverseSelectColumnNames(pSelectNode);
		if (! aIteratorStatus.IsSuccessful())
			return;

		if (StatementType() == SDB_STATEMENT_SELECT)
		{
			SdbSqlParseNode * pTableExp;
			if(SDB_ISRULE(pRootNode,select_statement))
				pTableExp = pRootNode->GetChild(3);
			else
			{
				pWhereTree = NULL;
				pOrderTree = NULL;
				pHavingTree = NULL;
				pGroupByTree = NULL;
				return;
			}

			DBG_ASSERT(pTableExp != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
			DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
			DBG_ASSERT(pTableExp->Count() == 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

			pWhereTree = pTableExp->GetChild(1);
			pGroupByTree = pTableExp->GetChild(2);
			pHavingTree = pTableExp->GetChild(3);
			pOrderTree	= pTableExp->GetChild(4);

			// ohne " where "
			if (pWhereTree && pWhereTree->Count())
				pWhereTree = pWhereTree->GetChild(1);
			else
				pWhereTree = NULL;

			if (pOrderTree->Count() == 3)
				pOrderTree = pOrderTree->GetChild(2); // ohne order by
			else
				pOrderTree = NULL;

			if (pHavingTree && pHavingTree->Count())
				pHavingTree = pHavingTree->GetChild(1);
			else
				pHavingTree = NULL;

			if (pGroupByTree->Count() == 3)
				pGroupByTree = pGroupByTree->GetChild(2);
			else
				pGroupByTree = NULL;
		}
	}
}

//------------------------------------------------------------------------------
const SdbColumn* SbaParseIterator::GetCol(const String& rColumnName) const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	return GetSelectColumns()->Column( rColumnName );
}

//------------------------------------------------------------------------------
String SbaParseIterator::GetOperator( const SdbPredicateType eType ) const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	static const char* arrOp[] =
	{ "=", "<>", "<", "<=", ">", ">=", "LIKE", "IS NULL" , "IS NOT NULL", "NOT LIKE" };
	String aResult;
	int nIdx = (int)eType -(int)SDB_PRED_EQUAL;

	if( nIdx >= 0 )
		aResult.AssignAscii(arrOp[nIdx]);

	return aResult;
}

//------------------------------------------------------------------------------
BOOL SbaParseIterator::FillCriteriaList(SbaPredicateItemList& rList)
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	// Versucht aus dem ParseTree eine Liste mit Where-Kriterien zusammenzustellen
	// Dieses funktioniert nur bei einfachen Statements -> gedacht fuer den einfachen FilterDialog
	// siehe auch SetPredicate
	for (USHORT i = 0 ; i < (USHORT)rList.Count(); i++)
		delete rList.GetObject( i );
	rList.Clear();

	// durchlaufen der selektionskriterien
	pTempCriteriaList = &rList;
	TraverseSelectionCriteria(ParseTree());
	pTempCriteriaList = NULL;
	if (!IteratorStatus().IsSuccessful())
	{
		for (USHORT i = 0 ; i < (USHORT)rList.Count(); i++)
			delete rList.GetObject( i );
		rList.Clear();
	}
	return IteratorStatus().IsSuccessful();
}

//------------------------------------------------------------------------------
void SbaParseIterator::SetPredicate(const String & rColumnName,
									const String & rTableRange,
									SdbPredicateType ePredicateType,
									const String & rValue,
									const String & rName)
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	// Beschreibung eines einfachen Predicats
	SbaPredicateItem* pItem = new SbaPredicateItem;
	if (!pItem)
		return;

	pItem->eType	= ePredicateType;

	pItem->aName	= rName;
	pItem->aValue	= rValue;


	SdbTableRef* pTableRef=NULL;
	if (GetSdbTableRangeMap().Count() > 1 && rTableRange.Len())
		pTableRef = GetSdbTableRangeMap().Lookup(rTableRange);
	else if(rTableRange.Len())
		pTableRef = GetSdbTableRangeMap().GetObject(0);
	else
	{
		for(ULONG i=0;i<GetSdbTableRangeMap().Count();i++)
		{
			SdbTableRef* pSearch = GetSdbTableRangeMap().GetObject(i);
			if (pSearch && pSearch->Is())
			{
				const SdbColumnsRef rColumns = (*pSearch)->Columns();
				const SdbColumn* pColumn = rColumns->Column(rColumnName);
				if (pColumn)
				{
					pTableRef = pSearch;
					break;
				}
			}
		}
	}

	DBG_ASSERT(pTableRef != NULL, "SbaParseIterator::SetPredicate : ungueltiges Feld (keine Tabelle gefunden) !");
	if(pTableRef)
	{
		pItem->aTableRange = rTableRange;
		pItem->eVarType	 = (*pTableRef)->Columns()->Column(rColumnName)->GetType();
	}
	pItem->aFieldName = rColumnName;
	pTempCriteriaList->Insert(pItem, LIST_APPEND);
}

//------------------------------------------------------------------------------
BOOL SbaParseIterator::FillOrderList(SbaOrderItemList& rList)
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	// nur fuer max. 3 Order spalten
	for (USHORT i = 0 ; i < (USHORT)rList.Count(); i++)
		delete rList.GetObject( i );
	rList.Clear();

	// durchlaufen der selektionskriterien
	pTempOrderList = &rList;
	TraverseOrderByColumnNames(ParseTree());
	pTempOrderList = NULL;
	if (!IteratorStatus().IsSuccessful())
	{
		for (USHORT i = 0 ; i < (USHORT)rList.Count(); i++)
			delete rList.GetObject( i );
		rList.Clear();
	}
	return IteratorStatus().IsSuccessful();
}

//------------------------------------------------------------------------------
void SbaParseIterator::SetOrderByColumnName(const String & rColumnName, const String & rTableRange, BOOL bAscending)
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	SbaOrderItem* pItem = new SbaOrderItem;
	pItem->aTableRange = rTableRange;
	pItem->aFieldName = rColumnName;
	pItem->bDirection = ! bAscending;
	pTempOrderList->Insert(pItem, LIST_APPEND );
}

//------------------------------------------------------------------------------
String SbaParseIterator::BuildColumnPart() const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	String	aResult;
	const SdbSqlParseNode* pRoot = ParseTree();
	DBG_ASSERT(pRoot,"Der Parsetree mu\"s gesetzt sein");
	pRoot->GetChild(1)->ParseNodeToStr(aResult,pConnection->GetMetaData()); // DISTINCT/ALL
	pRoot->GetChild(2)->ParseNodeToStr(aResult,pConnection->GetMetaData()); // Spalten
	return aResult;
}

//------------------------------------------------------------------------------
String SbaParseIterator::BuildTablePart() const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	String	aResult;
	const SdbSqlParseNode* pRoot = ParseTree();
	pRoot->GetChild(3)->GetChild(0)->GetChild(1)->ParseNodeToStr(aResult,pConnection->GetMetaData());
	return aResult;
}

//------------------------------------------------------------------------------
String SbaParseIterator::BuildPart(SQLOptionalPart ePart, BOOL bPrependKeyword) const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	String aResult;
	const SdbSqlParseNode* pParseTree = ParseTree();
	if (!pConnection)
		return aResult;

	const SdbSqlParseNode* pNode = NULL;
	switch (ePart)
	{
		case WHERE:
			pNode = pWhereTree;
			break;
		case HAVING:
			pNode = pHavingTree;
			break;
		case ORDER_BY:
			pNode = pOrderTree;
			break;
		case GROUP_BY:
			pNode = pGroupByTree;
			break;
	}

	if (!pNode)
		return aResult;

	String aClause;
	// Jetzt die Klausel ermitteln
	pNode->ParseNodeToStr(aClause,pConnection->GetMetaData());
	aClause.EraseLeadingChars(' ');
	aClause.EraseTrailingChars(' ');

	if (bPrependKeyword && aClause.Len())
	{
		switch (ePart)
		{
			case WHERE:
				aResult = STR_WHERE;
				break;
			case HAVING:
				aResult = STR_HAVING;
				break;
			case ORDER_BY:
				aResult = STR_ORDER_BY;
				break;
			case GROUP_BY:
				aResult = STR_GROUP_BY;
				break;
		}
	}

	aResult += aClause;
	return aResult;
}

//------------------------------------------------------------------------------
BOOL SbaParseIterator::HasPart(SQLOptionalPart ePart) const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);

	const SdbSqlParseNode* pNode = NULL;
	switch (ePart)
	{
		case WHERE:
			pNode = pWhereTree;
			break;
		case HAVING:
			pNode = pHavingTree;
			break;
		case ORDER_BY:
			pNode = pOrderTree;
			break;
		case GROUP_BY:
			pNode = pGroupByTree;
			break;
	}
	return pNode != NULL;
}

//------------------------------------------------------------------------------
String SbaParseIterator::BuildWherePart(SbaPredicateItemList& rList)
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	String	aResult;
	for (USHORT j=0 ; j<(USHORT)rList.Count(); j++)
	{
		SbaPredicateItem* pItem = rList.GetObject(j);
		if (pItem->eLogType != SBA_LOG_INVALID)
			aResult.AppendAscii( pItem->eLogType == SBA_LOG_OR ? " OR ": " AND " );

		if (pItem->aTableRange.Len())
		{
			aResult += IdentifierQuotation(pItem->aTableRange);
			aResult += '.';
		}
		aResult += IdentifierQuotation(pItem->aFieldName);
		aResult += ' ';
		aResult += GetOperator( pItem->eType );
		aResult += ' ';

		if (pItem->aName.Len())
		{
			// Parameter
			if (pItem->aName.GetChar(0) == '?')
				aResult += pItem->aValue;
			else if (pItem->aName.GetChar(0) == ':')
			{
				aResult += pItem->aName;
				aResult += pItem->aValue;
			}
			else if (pItem->aName.GetChar(0) == '[')
			{
				aResult += '[';
				aResult += pItem->aValue;
				aResult += ']';
			}
			else
			{
				if (pItem->aValue.Len())			// kann den Tabellennamen enthalten
				{
					aResult += IdentifierQuotation(pItem->aValue);
					aResult += '.';
				}
				aResult += IdentifierQuotation(pItem->aName);
			}
		}
		else
		{
			switch( pItem->eType )
			{
				case SDB_PRED_ISNULL:
				case SDB_PRED_ISNOTNULL:
					break;
				default:
					aResult += QuoteField( pItem->aValue, pItem->eVarType );
			}
		}
	}
	aResult.EraseLeadingChars(' ');
	aResult.EraseTrailingChars(' ');
	return aResult;
}

//------------------------------------------------------------------------------
String SbaParseIterator::BuildOrderPart( SbaOrderItemList& rList )
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	USHORT i;
	String	aResult;

	if (rList.Count())
	{
		for (i=0 ; i<(USHORT)rList.Count() ; i++)
		{
			if (i>0) aResult += ',';
			SbaOrderItem* pItem = rList.GetObject( i );

			if (pItem->aTableRange.Len())
			{
				aResult += IdentifierQuotation(pItem->aTableRange);
				aResult += '.';
			}
			aResult += IdentifierQuotation(pItem->aFieldName);
			aResult.AppendAscii(pItem->bDirection ? " DESC" : " ASC");
		}
		aResult.EraseLeadingChars(' ');
		aResult.EraseTrailingChars(' ');
	}

	return aResult;
}

//------------------------------------------------------------------------------
BOOL SbaParseIterator::IsSimple() const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	return IsValid() && GetSdbTableRangeMap().Count()==1 && GetSelectColumns()->Count() > 0;
}

//------------------------------------------------------------------------------
String SbaParseIterator::IdentifierQuotation(const String& rText) const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	if (pConnection)
		return pConnection->IdentifierQuotation(rText);
	else
		return rText;
}

//------------------------------------------------------------------------------
BOOL SbaParseIterator::HasFilterOrSort() const
{
	DBG_CHKTHIS(SbaParseIterator,NULL);
	return IsValid() && (pWhereTree || pOrderTree);
}

