/*************************************************************************
 *
 *  $RCSfile: derivedimplbase.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <sbautl/derivedimplbase.hxx>

#include <memory.h>

#ifndef _RTL_UUID_H_
#include <rtl/uuid.h>
#endif

namespace sbautl
{
	namespace staruno = ::com::sun::star::uno;

	void TypeProviderInfo::allocId() throw()
	{
		::osl::MutexGuard aGuard(getMutex());
		if (!nState)
		{
			rtl_createUuid(reinterpret_cast<sal_uInt8*>(uuid),0,sal_True);
			::osl_incrementInterlockedCount(&nState);
		}
		OSL_ENSHURE(nState == 1,"TypeProviderInfo: ImplementationId not created or overwritten"); 
	}

	void TypeProviderInfo::createTypes(const ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type >& rTypes_) 
		throw(::com::sun::star::uno::RuntimeException)
	{
		OSL_ENSHURE(instanceCount() > 0,"TypeProviderInfo: Type collections supported only if instances exist"); 
			::osl::MutexGuard aGuard(getMutex());
		if (!pTypes)
		{
			// allocate our cached sequence
			pTypes = new ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type >(rTypes_);
		}
	}

	// getImplementationId support
	staruno::Sequence< sal_Int8 > TypeProviderInfo::getImplementationId() 
		throw(::com::sun::star::uno::RuntimeException)
	{
		if (!nState)
			allocId();


		if (instanceCount() > 0)
		{
			// should allocate/use our cached sequence
			::osl::MutexGuard aGuard(getMutex());
			if (instanceCount() > 0 && !pIdSeq)
			{
				pIdSeq = new staruno::Sequence< sal_Int8 >(uuid, sizeof(uuid));
			}

			if (pIdSeq)
				return *pIdSeq;
		}

		// No instances ? create one on the fly
		return staruno::Sequence< sal_Int8 >(uuid, sizeof(uuid));
	}

	// UnoTunnel support
	sal_Bool TypeProviderInfo::isImplementationId(::com::sun::star::uno::Sequence< sal_Int8 > const& rIdentifier_) 
		throw()
	{
		if (!nState)
			allocId();

		return	rIdentifier_.getLength() == sizeof(uuid) && 
				::memcmp(rIdentifier_.getConstArray(),&uuid,sizeof(uuid)) == 0;
	}

	void TypeProviderInfo::cleanup() throw()
	{
		::osl::MutexGuard aGuard(getMutex());
		if (instanceCount() == 0)
		{
			delete pTypes, pTypes = 0;
			delete pIdSeq, pIdSeq = 0;
		}
	}



} // namespace sbautl

