/*************************************************************************
 *
 *  $RCSfile: sbadbcon.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sbadbcon.hxx"

#ifndef _NUMUNO_HXX
#include <svtools/numuno.hxx>
#endif

using staruno::Any;
using staruno::Sequence;
using staruno::Reference;

//------------------------------------------------------------------------------
SbaXdbConnection::SbaXdbConnection()
				 :SbaXdbObjectCreator(sal_True)
				 ,m_aStatements(*this)
				 ,m_aTables(*this)
				 ,m_aRelations(*this)
				 ,m_aParsers(*this)
{
}

//------------------------------------------------------------------------------
SbaXdbConnection::SbaXdbConnection(SbaXdbCollection& rCollection, SdbConnection* pConn)
				 :SbaXdbObjectCreator(rCollection, pConn->GetIdentifierCase() == SDB_IC_ALL)
				 ,m_xConnection(pConn)
				 ,m_aStatements(*this)
				 ,m_aTables(*this)
				 ,m_aRelations(*this)
				 ,m_aParsers(*this)
{
}

//------------------------------------------------------------------------------
SbaXdbConnection::~SbaXdbConnection()
{
	if (!ComponentHelper::rBHelper.bDisposed)
	{
		acquire();
		dispose();
	}

	if (m_xAgg.is())
	{
		Reference< staruno::XInterface >  xInt;
		m_xAgg->setDelegator(xInt);
	}
}

//------------------------------------------------------------------------------
Sequence<staruno::Type> SbaXdbConnection::collectTypes(SbaXdbConnection* p) throw(staruno::RuntimeException)
{
	// hard-coded: Aggregate provides XNumberFormatsSupplier
	using starutil::XNumberFormatsSupplier;
	staruno::Type const& rExtraType = ::getCppuType(static_cast<Reference<XNumberFormatsSupplier>*>(0));

	DBG_ASSERT(getAggregation()->queryAggregation(rExtraType).hasValue(),"Missing type from aggregated object");
	return ::cppu::OTypeCollection(rExtraType, DerivationHelper4::collectTypes(p) ).getTypes();
}

//------------------------------------------------------------------------------
Any SAL_CALL SbaXdbConnection::queryInterface(staruno::Type const& rType) throw(staruno::RuntimeException)
{
	Any aRet(DerivationHelper4::queryInterface(rType));
	return aRet.hasValue() ? aRet : getAggregation()->queryAggregation(rType);
}
using namespace ::cppu;
using namespace ::osl;

//------------------------------------------------------------------------------
sal_Int64 SAL_CALL SbaXdbConnection::getSomething(staruno::Sequence<sal_Int8> const& rId)throw( staruno::RuntimeException )
{
	if (rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  rId.getConstArray(), 16 ) )
		return (sal_Int64)this;

	return SbaXdbObjectCreator::getSomething(rId);
}
//--------------------------------------------------------------------------
Sequence< sal_Int8 > SbaXdbConnection::getUnoTunnelImplementationId() 
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}


//------------------------------------------------------------------------------
const Reference< staruno::XAggregation > & SbaXdbConnection::getAggregation()
{
	::osl::MutexGuard aGuard( m_aMutex );
	if (!m_xAgg.is())
	{
		SvNumberFormatsSupplierObj* pNum = new SvNumberFormatsSupplierObj(m_xConnection->GetEnvironment()->GetStandardFormatter());
		m_xAgg = pNum;
		m_xAgg->setDelegator(*this);
	}
	return m_xAgg;
}

// ComponentHelper
//------------------------------------------------------------------------------
void SbaXdbConnection::disposing()
{
	SbaXdbObject::disposing();
	m_aStatements.disposing();
	m_aTables.disposing();
	m_aRelations.disposing();
	m_aParsers.disposing();
	m_xConnection = NULL;
}

// stardata::XDatabaseConnection
//------------------------------------------------------------------------------
Reference< stardata::XDatabaseDescriptor > SAL_CALL SbaXdbConnection::getMetaData(void) 
	throw( staruno::RuntimeException )
{
	Reference< stardata::XDatabaseDescriptor >  xMData;
	if (m_xConnection)
		xMData = m_xConnection->GetMetaData();
	return xMData;
}

//------------------------------------------------------------------------------
Reference< starcontainer::XNameContainer > SAL_CALL SbaXdbConnection::getTables(void) 
	throw( staruno::RuntimeException )
{
	return &m_aTables;
}

//------------------------------------------------------------------------------
Reference< starcontainer::XNameContainer > SAL_CALL SbaXdbConnection::getRelations(void) 
	throw( staruno::RuntimeException )
{
	return &m_aRelations;
}

//------------------------------------------------------------------------------
::rtl::OUString SAL_CALL SbaXdbConnection::getCatalog(void) 
	throw( staruno::RuntimeException )
{
	return isOpen() ? ::rtl::OUString(m_xConnection->GetQualifier()) : ::rtl::OUString();
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbConnection::setCatalog(const ::rtl::OUString& aCatalog)
	throw( stardata::DatabaseException, staruno::RuntimeException )
{
}

//------------------------------------------------------------------------------
Reference< stardata::XDatabaseStatement > 
	SAL_CALL  SbaXdbConnection::createStatement(const ::rtl::OUString& aStatement, sal_Bool bPassThrough) 
		throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();
	::osl::MutexGuard aGuard( m_aMutex );

	SdbCursorType eSdbType = SDB_SNAPSHOT;
	sal_uInt16 nSdbOptions = SDB_FORWARDONLY | SDB_READONLY;

	if (!bPassThrough)
		nSdbOptions |= SDB_INDIRECT;

	SdbCursorRef xSdbCursor = m_xConnection->CreateCursor(eSdbType, nSdbOptions);
	checkDBResult(m_xConnection->Status(), *this, sal_False);

	DBG_ASSERT(xSdbCursor.Is(), "SbaXdbConnection::createStatement: kein Cursor erzeugt");

	if (xSdbCursor.Is())
		return new SbaXdbStatement(m_aStatements, *xSdbCursor, aStatement, bPassThrough);
	else
		return Reference< stardata::XDatabaseStatement > ();
}

//------------------------------------------------------------------------------
Reference< stardata::XPreparedDatabaseStatement > 
	SAL_CALL SbaXdbConnection::createPreparedStatement(const ::rtl::OUString& aStatement, sal_Bool bPassThrough) 
		throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();
	::osl::MutexGuard aGuard( m_aMutex );

	SdbCursorType eSdbType = SDB_SNAPSHOT;
	sal_uInt16 nSdbOptions = SDB_FORWARDONLY | SDB_READONLY;

	if (!bPassThrough)
		nSdbOptions |= SDB_INDIRECT;

	SdbCursorRef xSdbCursor = m_xConnection->CreateCursor(eSdbType, nSdbOptions);
	checkDBResult(m_xConnection->Status(), *this, sal_False);

	DBG_ASSERT(xSdbCursor.Is(), "SbaXdbConnection::createStatement: kein Cursor erzeugt");

	if (xSdbCursor.Is())
	{
		xSdbCursor->Prepare(aStatement);
		checkDBResult(xSdbCursor->Status(), *this, sal_False);
		return new SbaXdbStatement(m_aStatements, *xSdbCursor, aStatement, bPassThrough);
	}
	else
		return Reference< stardata::XPreparedDatabaseStatement > ();
}

//------------------------------------------------------------------------------
Reference< stardata::XPreparedDatabaseStatement > 
	SAL_CALL SbaXdbConnection::createProcedureStatement(const ::rtl::OUString& aStatement, sal_Bool bPassThrough) 
		throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();
	::osl::MutexGuard aGuard( m_aMutex );

	SdbCursorType eSdbType = SDB_SNAPSHOT;
	sal_uInt16 nSdbOptions = SDB_FORWARDONLY | SDB_READONLY;

	if (!bPassThrough)
		nSdbOptions |= SDB_INDIRECT;

	SdbCursorRef xSdbCursor = m_xConnection->CreateCursor(eSdbType, nSdbOptions);
	checkDBResult(m_xConnection->Status(), *this, sal_False);

	DBG_ASSERT(xSdbCursor.Is(), "SbaXdbConnection::createStatement: kein Cursor erzeugt");

	if (xSdbCursor.Is())
	{
		xSdbCursor->Prepare(aStatement);
		checkDBResult(xSdbCursor->Status(), *this, sal_False);
		return new SbaXdbStatement(m_aStatements, *xSdbCursor, aStatement, bPassThrough);
	}
	else
		return Reference< stardata::XPreparedDatabaseStatement > ();
}

// stardata::XDatabaseTransactionSupplier
//------------------------------------------------------------------------------
sal_Bool SAL_CALL SbaXdbConnection::isAutoCommit(void) 
	throw( staruno::RuntimeException )
{
	if (m_xConnection.Is())
		return m_xConnection->IsAutoCommit();
	else
		return sal_True;
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbConnection::setAutoCommit(sal_Bool bEnable) 
	throw( stardata::DatabaseException, starlang::IllegalAccessException, staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	m_xConnection->SetAutoCommit(bEnable);
	checkDBResult(m_xConnection->Status(), *this);
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbConnection::commit(void) 
	throw( stardata::DatabaseException, starlang::IllegalAccessException, staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	if (!m_xConnection->CommitTrans())
		checkDBResult(m_xConnection->Status(), *this);
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbConnection::rollback(void) 
	throw( stardata::DatabaseException, starlang::IllegalAccessException,  staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	if (!m_xConnection->RollbackTrans())
		checkDBResult(m_xConnection->Status(), *this);
}

//------------------------------------------------------------------------------
sal_Int16 SAL_CALL SbaXdbConnection::getIsolationLevel(void) throw( staruno::RuntimeException )
{
	return m_xConnection ? m_xConnection->IsolationLevel() : stardata::DatabaseIsolationLevel::NONE;
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbConnection::setIsolationLevel(sal_Int16 Level) 
	throw( stardata::DatabaseException,  staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	m_xConnection->SetIsolationLevel(SdbIsolationLevel(Level));
	checkDBResult(m_xConnection->Status(), *this);
}

//------------------------------------------------------------------------------
SdbObj* SbaXdbConnection::createObject(DBObject _eObj, const ::rtl::OUString& _rName) 
	throw( stardata::DatabaseException  )
{
	::osl::MutexGuard aGuard( m_aMutex );
	SdbObj* pObj = NULL;
	if (m_xConnection.Is())
	{
		switch (_eObj)
		{
			case dbTable:
				pObj = m_xConnection->CreateTable(_rName);
				break;
			case dbRelation:
				pObj = m_xConnection->CreateRelation(_rName);
				break;
		}
		if (!pObj)
			checkDBResult(m_xConnection->Status(), *this);
	}
	return pObj;
}

//------------------------------------------------------------------------------
SdbObj* SbaXdbConnection::openObject(DBObject _eObj, const ::rtl::OUString& _rName)
{
	::osl::MutexGuard aGuard( m_aMutex );
	SdbObj* pObj = NULL;
	if (m_xConnection.Is())
	{
		switch (_eObj)
		{
			case dbTable:
				pObj = m_xConnection->OpenTable(_rName);
				break;
			case dbRelation:
				pObj = m_xConnection->OpenRelation(_rName);
				break;
		}
	}
	return pObj;
}

//------------------------------------------------------------------------------
SdbObj* SbaXdbConnection::openObject(DBObject _eObj, sal_Int32 _nPos)
{
	::osl::MutexGuard aGuard( m_aMutex );
	SdbObj* pObj = NULL;
	if (m_xConnection.Is())
	{
		switch (_eObj)
		{
			case dbTable:
				pObj = m_xConnection->OpenTable((sal_uInt16)_nPos);
				break;
			case dbRelation:
				pObj = m_xConnection->OpenRelation((sal_uInt16)_nPos);
				break;
		}
	}
	return pObj;
}

//------------------------------------------------------------------------------
::rtl::OUString SbaXdbConnection::getObjectName(DBObject _eObj, sal_Int32 _nPos) const
{
	String sName;
	if (m_xConnection.Is())
	{
		switch (_eObj)
		{
			case dbTable:
				sName = m_xConnection->GetTableName((sal_uInt16)_nPos);
				break;
			case dbRelation:
				sName = m_xConnection->GetRelationName((sal_uInt16)_nPos);
				break;
		}
	}
	return sName;
}

//------------------------------------------------------------------------------
sal_Int32 SbaXdbConnection::getObjectCount(DBObject _eObj) const
{
	sal_Int32 nCount = 0;
	if (m_xConnection.Is())
	{
		switch (_eObj)
		{
			case dbTable:
				nCount = m_xConnection->GetTableCount();
				break;
			case dbRelation:
				nCount = m_xConnection->GetRelationCount();
				break;
		}
	}
	return nCount;
}

//------------------------------------------------------------------------------
void SbaXdbConnection::refresh(DBObject _eObj) throw( stardata::DatabaseException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	// Cursor werden am Leben gehalten, da diese von einem Refresh nicht betroffen sind
	m_aTables.disposing();
	m_aRelations.disposing();

	m_xConnection->Refresh();
}

// stardata::XDatabaseQueryComposerFactory
//------------------------------------------------------------------------------
Reference< stardata::XDatabaseQueryComposer > SAL_CALL SbaXdbConnection::createDatabaseQueryComposer(void) 
	throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();

	::osl::MutexGuard aGuard( m_aMutex );

	return new SbaXdbParser(m_aParsers, m_xConnection);
}


















