/*************************************************************************
 *
 *  $RCSfile: sbadbidx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SBA_SBADBIDX_HXX
#include "sbadbidx.hxx"
#endif

#ifndef _SDB_SDBTAB_HXX //autogen
#include <sdb/sdbtab.hxx>
#endif

#ifndef _UTL_PROPERTY_HXX_
#include <unotools/property.hxx>
#endif

#ifndef _SBA_UNOPROPMACROS_HXX
#include "sbautl/unopropertymacros.hxx"
#endif

using staruno::Any;
using staruno::Sequence;
using staruno::Reference;
using staruno::UNO_QUERY;

//------------------------------------------------------------------------------
SbaXdbIndex::SbaXdbIndex(sal_Bool bCase)
			:SbaXdbObject()
			,PropertySetHelper(ComponentHelper::rBHelper)
			,m_aFields(*this, bCase)
			,m_bUnique(sal_False)
			,m_bClustered(sal_False)
			,m_bPrimaryKey(sal_False)
{}

//------------------------------------------------------------------------------
SbaXdbIndex::SbaXdbIndex(SbaXdbCollection& rCollection, SdbIndex* pIndexDef, sal_Bool bCase)
			:SbaXdbObject(rCollection)
			,PropertySetHelper(ComponentHelper::rBHelper)
			,m_aFields(*this, bCase)
			,m_xIndex(pIndexDef)
			,m_bUnique(pIndexDef->IsUnique())
			,m_bClustered(sal_False)
			,m_bPrimaryKey(sal_False)
{
	if (m_xIndex)
	{
		SdbColumnsRef xCols = m_xIndex->OriginalColumns();
		m_aFields.setDefinitions(xCols);
	}
}

//------------------------------------------------------------------------------
SbaXdbIndex::~SbaXdbIndex()
{
	if (!ComponentHelper::rBHelper.bDisposed)
	{
		acquire();
		dispose();
	}
}
// -------------------------------------------------------------------------
using namespace ::cppu;
using namespace ::osl;

// XUnoTunnel
::com::sun::star::uno::Sequence< sal_Int8 > SbaXdbIndex::getUnoTunnelImplementationId()
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}
//------------------------------------------------------------------------------
sal_Int64 SAL_CALL SbaXdbIndex::getSomething(staruno::Sequence<sal_Int8> const& rId)throw( staruno::RuntimeException )
{
	if (rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  rId.getConstArray(), 16 ) )
		return (sal_Int64)this;

	return SbaXdbObject::getSomething(rId);
}

// ComponentHelper
//------------------------------------------------------------------------------
void SbaXdbIndex::disposing()
{
	SbaXdbObject::disposing();
	PropertySetHelper::disposing();
	m_aFields.disposing();
	m_xIndex = NULL;
}

// starcontainer::XNamed
//------------------------------------------------------------------------------
::rtl::OUString SAL_CALL SbaXdbIndex::getName(void)
	throw( staruno::RuntimeException  )
{
	::osl::MutexGuard aGuard( m_aMutex );
	return isOpen() ? ::rtl::OUString(m_xIndex->GetFullName()) : ::rtl::OUString();
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbIndex::setName(const ::rtl::OUString& _rName)
	throw( staruno::RuntimeException  )
{
	// Not Allowed: throw starlang::NoSupportException() ;
	throw staruno::RuntimeException();
}

// stardata::XDatabaseFieldsSupplier
//------------------------------------------------------------------------------
Reference< starcontainer::XNameAccess > SAL_CALL SbaXdbIndex::getFields(void)
	throw( staruno::RuntimeException  )
{
	return &m_aFields;
}

//------------------------------------------------------------------------------
void SbaXdbIndex::fillProperties( Sequence<starbeans::Property>& aProperties) const
	throw( staruno::RuntimeException )
{
	using starbeans::PropertyAttribute::READONLY;
	SBA_BEGIN_PROPERTY_HELPER( aProperties, 4)
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(ISCLUSTERED),	BOOL_PROP(),		0);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(PRIMARYKEY),	BOOL_PROP(),		READONLY);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(ISUNIQUE),		BOOL_PROP(),		0),
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(NAME),			PROP_TYPE(OUString),READONLY);
	SBA_END_PROPERTY_HELPER();
}

//------------------------------------------------------------------------------
sal_Bool SbaXdbIndex::convertFastPropertyValue(
								Any & rConvertedValue, Any & rOldValue,
								sal_Int32 nHandle, const Any& rValue )
	throw( starlang::IllegalArgumentException  )
{
	sal_Bool bModified(sal_False);
	switch (nHandle)
	{
		case PROP_ISCLUSTERED:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue,
															m_bClustered);
			break;
		case PROP_ISUNIQUE:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue,
															m_bUnique);
			break;
		default:
			DBG_ERROR("unknown Porperty");
	}
	return bModified;
}

//------------------------------------------------------------------------------
void SbaXdbIndex::setFastPropertyValue_NoBroadcast( sal_Int32 nHandle, const Any& rValue )
	throw (staruno::Exception)
{
	switch (nHandle)
	{
		case PROP_ISCLUSTERED:
			DBG_ASSERT(rValue.getValueTypeClass() == staruno::TypeClass_BOOLEAN, "invalid type" );

			m_bClustered = ::utl::getBOOL(rValue);
			break;
		case PROP_ISUNIQUE:
			DBG_ASSERT(rValue.getValueTypeClass() == staruno::TypeClass_BOOLEAN, "invalid type" );

			m_bUnique = ::utl::getBOOL(rValue);
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

//------------------------------------------------------------------------------
void SbaXdbIndex::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROP_ISCLUSTERED:
			::utl::setBOOL(rValue, m_bClustered);
			break;
		case PROP_ISUNIQUE:
			::utl::setBOOL(rValue, m_bUnique);
			break;
		case PROP_PRIMARYKEY:
			::utl::setBOOL(rValue, m_bPrimaryKey);
			break;
		case PROP_NAME:
			rValue <<= const_cast<SbaXdbIndex*>(this)->getName();
			break;
		default:
			DBG_ERROR("unknown Porperty");
	}
}

//------------------------------------------------------------------------------
void SbaXdbIndex::setObject(const SdbIndexRef& xDef)
					throw( stardata::DatabaseException, starlang::IllegalArgumentException, staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );

	if (m_bUnique)
		xDef->SetStyle(SDB_IDX_UNIQUE);

	// setzen der Columns
	SdbTableRef xTable(xDef->GetTable());

	sal_uInt32 nCount = m_aFields.getCount();
	for (sal_uInt32 i = 0; i < nCount; i++)
	{
		const SbaXdbField* pField = m_aFields[i];

		// gehoert die Column zur Tabelle ?
		if (xTable->OriginalColumns()->ColumnNumber(pField->m_aName) == SDB_COLUMN_NOTFOUND)
			throw starlang::IllegalArgumentException();

		SdbColumn* pCol = pField->createColumn(xTable->GetConnection());
		xDef->Columns()->AddColumn(pCol);
	}

	// keine Spalten -> Fehler
	if (!nCount)
	{
		SdbStatus aStat;
		aStat.Set(SDB_STAT_ERROR);
		checkDBResult(aStat, *this);
	}

	if (xDef->Update())
		m_xIndex = xDef;
	else
		checkDBResult(xDef->Status(), *this);
}

//------------------------------------------------------------------------------
SbaXdbIndexes::SbaXdbIndexes(SbaXdbObjectCreator& _rParent)
: SbaXdbNamedCollection(_rParent, dbIndex,
						::getCppuType(static_cast<const Reference< starbeans::XPropertySet >*>(0)) )
{
}


// stardata::XDatabaseIndexFactory
//------------------------------------------------------------------------------
Reference< starbeans::XPropertySet > SAL_CALL SbaXdbIndexes::create(void)
			throw( starlang::NoSupportException, staruno::RuntimeException )
{
	checkOpen();
	return new SbaXdbIndex(getXdbCreator()->isCaseSensitive());
}

// helper methods
//------------------------------------------------------------------------------
SbaXdbObject* SbaXdbIndexes::doGet(SdbObj* pObj) const throw( stardata::DatabaseException, staruno::RuntimeException )
{
	SbaXdbIndex* pXObj = NULL;

	// Dokument schon geoeffnet
	sal_uInt32 nCount = size();
	sal_uInt32 i;
	for (i = 0; i < nCount; i++)
	{
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel((::cppu::OWeakObject*)getObject(i),::com::sun::star::uno::UNO_QUERY);
		if(xTunnel.is())
		{
			pXObj = (SbaXdbIndex*)xTunnel->getSomething(SbaXdbIndex::getUnoTunnelImplementationId());
		}
		//	utl::getImplementation(pXObj, getObject(i));

		if (pXObj && pObj == &pXObj->m_xIndex)
			break;
	}

	if (i == nCount)
		pXObj = new SbaXdbIndex(*(SbaXdbIndexes*)this, (SdbIndex*)pObj, getXdbCreator()->isCaseSensitive());

	return pXObj;
}

//------------------------------------------------------------------------------
sal_Bool SbaXdbIndexes::doRemove(SbaXdbObject* pObj) throw( stardata::DatabaseException, staruno::RuntimeException )
{
	SbaXdbIndex* pXObj = (SbaXdbIndex*)pObj;

	if (!pXObj->m_xIndex->Drop())
		checkDBResult(pXObj->m_xIndex->Status(), *this, sal_True);
	return sal_True;
}

//------------------------------------------------------------------------------
SbaXdbObject* SbaXdbIndexes::doInsert(const ::rtl::OUString& Name, const Any& Element)
				throw( stardata::DatabaseException, starlang::IllegalArgumentException, staruno::RuntimeException )
{
	// Ist es ein bekanntes objekt
	SbaXdbIndex* pXObj = 0;
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel;
	Element >>= xTunnel;
	if(xTunnel.is())
	{
		pXObj = (SbaXdbIndex*)xTunnel->getSomething(SbaXdbIndex::getUnoTunnelImplementationId());
	}
	//	utl::getImplementation(pXObj,Element);

	DBG_ASSERT(pXObj, "SbaXdbIndexes::doInsert for invalid element");

	// bereits in einem Container enthalten?
	if (!pXObj || pXObj->getXdbParent())
	{
		throw starlang::IllegalArgumentException();
	}

	SdbObj* pObj = NULL;

	pObj = getXdbCreator()->createObject(m_eObj, Name);

	SdbIndexRef xIdx = (SdbIndex*)pObj;
	pXObj->setObject(xIdx);
	return pXObj;
}

















