/*************************************************************************
 *
 *  $RCSfile: sbadbqry.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SBA_SBADBQRY_HXX
#include "sbadbqry.hxx"
#endif
#ifndef _SBAITEMS_HRC
#include "sbaitems.hrc"
#endif
#ifndef _SBA_SBADBDAT_HXX
#include <sbadbdat.hxx>
#endif

#ifndef _SFXSTRITEM_HXX //autogen wg. SfxStringItem
#include <svtools/stritem.hxx>
#endif

#ifndef _UTL_CONTAINER_HXX_
#include <unotools/container.hxx>
#endif

#ifndef _UTL_PROPERTY_HXX_
#include <unotools/property.hxx>
#endif

#ifndef _SBA_UNOPROPMACROS_HXX
#include "sbautl/unopropertymacros.hxx"
#endif

using staruno::Reference;
using staruno::Sequence;
using staruno::Any;

//------------------------------------------------------------------

//------------------------------------------------------------------------------
SbaXdbQuery::SbaXdbQuery(sal_Bool bCase)
			:SbaXdbObject()
			,PropertySetHelper(ComponentHelper::rBHelper)
			,m_aRefreshListeners(m_aMutex)
			,m_aFields(*this, sal_True)
			,m_bPassThrough(sal_False)
{}

//------------------------------------------------------------------------------
SbaXdbQuery::SbaXdbQuery(SbaXdbCollection& rCollection, SbaQueryDef* pQueryDef, sal_Bool bCase)
			:SbaXdbObject(rCollection)
			,PropertySetHelper(ComponentHelper::rBHelper)
			,m_aRefreshListeners(m_aMutex)
			,m_aFields(*this, bCase)
			,m_xQuery(pQueryDef)
			,m_bPassThrough(sal_False)
{
	if (m_xQuery)
	{
		const SbaColumnList& rCols = m_xQuery->GetOriginalColumns();
		m_aFields.setDefinitions(rCols);
		m_aQuery = m_xQuery->Statement();
		m_bPassThrough = m_xQuery->IsNative();

		const SfxStringItem* pOrderFilter = (const SfxStringItem*)m_xQuery->GetObjAttrs()->GetItem(SBA_DEF_FILTER);
		m_aFilter = pOrderFilter ? pOrderFilter->GetValue() : String();

		pOrderFilter = (const SfxStringItem*)m_xQuery->GetObjAttrs()->GetItem(SBA_DEF_SORT);
		m_aSort = pOrderFilter ? pOrderFilter->GetValue() : String();
	}
}

//------------------------------------------------------------------------------
SbaXdbQuery::~SbaXdbQuery()
{
	if (!ComponentHelper::rBHelper.bDisposed)
	{
		acquire();
		dispose();
	}
}

// XUnoTunnel
using namespace ::cppu;
using namespace ::osl;
// -------------------------------------------------------------------------
::com::sun::star::uno::Sequence< sal_Int8 > SbaXdbQuery::getUnoTunnelImplementationId()
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}
//------------------------------------------------------------------------------
sal_Int64 SAL_CALL SbaXdbQuery::getSomething(staruno::Sequence<sal_Int8> const& rId)throw( staruno::RuntimeException )
{
	if (rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  rId.getConstArray(), 16 ) )
		return (sal_Int64)this;

	return SbaXdbObject::getSomething(rId);
}

// ComponentHelper
//------------------------------------------------------------------------------
void SbaXdbQuery::disposing()
{
	SbaXdbObject::disposing();
	OPropertySetHelper::disposing();

	starlang::EventObject aEvt(*this);
	m_aRefreshListeners.disposeAndClear(aEvt);

	m_aFields.disposing();
	m_xQuery = NULL;
}

// starcontainer::XNamed
//------------------------------------------------------------------------------
::rtl::OUString SAL_CALL SbaXdbQuery::getName(void)
	throw( staruno::RuntimeException  )
{
	::osl::MutexGuard aGuard( m_aMutex );
	return isOpen() ? ::rtl::OUString(m_xQuery->Name()) : ::rtl::OUString();
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbQuery::setName(const ::rtl::OUString& _rName)
	throw( staruno::RuntimeException  )
{
	try
	{
		::osl::MutexGuard aGuard( m_aMutex );
		checkOpen();

		m_xQuery->Rename(_rName);

		checkDBResult(m_xQuery->Status(), *this, sal_True);
	}
	catch(staruno::RuntimeException&)
	{
		throw;
	}
	catch(staruno::Exception& e)
	{
		throw staruno::RuntimeException(e.Message,*this);
	}
}

// stardata::XDatabaseQuery
//------------------------------------------------------------------------------
Reference< stardata::XPreparedDatabaseStatement >
	SAL_CALL SbaXdbQuery::createPreparedStatement(void)
		throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();
	::osl::MutexGuard aGuard( m_aMutex );

	SdbCursorType eSdbType = SDB_KEYSET;
	sal_uInt16 nSdbOptions = SDB_SEEDELETED;

	if (!m_bPassThrough)
		nSdbOptions |= SDB_INDIRECT;

	SbaXdbDatabase* pDatabase = 0;
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel((::cppu::OWeakObject*)getXdbParent(),::com::sun::star::uno::UNO_QUERY);
	if(xTunnel.is())
	{
		pDatabase = (SbaXdbDatabase*)xTunnel->getSomething(SbaXdbDatabase:: getUnoTunnelImplementationId());
	}
	//	utl::getImplementation(pDatabase, getXdbParent());

	DBG_ASSERT(pDatabase, "SbaXdbQuery::createPreparedStatement : keine Datenbank");
	SbaDatabaseRef xDatabaseImpl = pDatabase->getDatabase();

	SdbCursorRef xSdbCursor = xDatabaseImpl->CreateCursor(eSdbType, nSdbOptions);
	checkDBResult(xDatabaseImpl->Status(), *this, sal_False);

	DBG_ASSERT(xSdbCursor.Is(), "SbaXdbQuery::createPreparedStatement: kein Cursor erzeugt");

	if (xSdbCursor.Is())
	{
		xSdbCursor->Prepare(m_aQuery);
		checkDBResult(xSdbCursor->Status(), *this, sal_False);
		return new SbaXdbQueryStatement(pDatabase->getXdbStatements(), *xSdbCursor, *this);
	}
	else
		return Reference< stardata::XPreparedDatabaseStatement > ();
}

// starutil::XRefreshable
//------------------------------------------------------------------------------
void SAL_CALL SbaXdbQuery::addRefreshListener(const Reference< starutil::XRefreshListener > & l)
	throw( staruno::RuntimeException  )
{   m_aRefreshListeners.addInterface(l); }
//------------------------------------------------------------------------------
void SAL_CALL SbaXdbQuery::removeRefreshListener(const Reference< starutil::XRefreshListener > & l)
	throw( staruno::RuntimeException  )
{   m_aRefreshListeners.removeInterface(l); }

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbQuery::refresh(void)
	throw( staruno::RuntimeException  )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	m_xQuery->Refresh();

	const SbaColumnList& rCols = m_xQuery->GetOriginalColumns();
	m_aFields.setDefinitions(rCols);

	m_aQuery = m_xQuery->Statement();
	m_bPassThrough = m_xQuery->IsNative();

	starlang::EventObject aEvt(*this);
	NOTIFY_LISTENERS(m_aRefreshListeners, starutil::XRefreshListener, refreshed, aEvt);
}

// stardata::XDatabaseFieldsSupplier
//------------------------------------------------------------------------------
Reference< starcontainer::XNameAccess > SAL_CALL SbaXdbQuery::getFields(void)
	throw( staruno::RuntimeException  )
{
	return &((SbaXdbQuery*)this)->m_aFields;
}

//------------------------------------------------------------------------------
// PropertySetHelper
//------------------------------------------------------------------------------
void SbaXdbQuery::fillProperties( Sequence<starbeans::Property>& aProperties) const
	throw( staruno::RuntimeException )
{
	using starbeans::PropertyAttribute::TRANSIENT;
	using starbeans::PropertyAttribute::READONLY;
	SBA_BEGIN_PROPERTY_HELPER(aProperties,6)
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(FILTER),		PROP_TYPE(OUString), 0);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(ISPASSTHROUGH),	BOOL_PROP(),		 0);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(NAME),			PROP_TYPE(OUString), READONLY);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(QUERY),			PROP_TYPE(OUString), 0);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(SORT),			PROP_TYPE(OUString), 0);
		NEXT_PROPERTY() = Property(SBA_OLD_PROPERTY(URL),			PROP_TYPE(OUString), TRANSIENT | READONLY);
	SBA_END_PROPERTY_HELPER();
}

//------------------------------------------------------------------------------
// OPropertySetHelper
//------------------------------------------------------------------------------
sal_Bool SbaXdbQuery::convertFastPropertyValue( Any & rConvertedValue, Any & rOldValue, sal_Int32 nHandle, const Any& rValue ) throw( starlang::IllegalArgumentException  )
{
	sal_Bool bModified(sal_False);
	switch (nHandle)
	{
		case PROP_ISPASSTHROUGH:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue,
															m_bPassThrough);
			break;
		case PROP_QUERY:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue,
															m_aQuery);
			break;
		case PROP_SORT:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue,
															m_aSort);
			break;
		case PROP_FILTER:
			bModified = ::utl::tryPropertyValue(rConvertedValue, rOldValue, rValue,
															m_aFilter);
			break;
		default:
			DBG_ERROR("unknown Porperty");
	}
	return bModified;
}

//------------------------------------------------------------------------------
void SbaXdbQuery::setFastPropertyValue_NoBroadcast( sal_Int32 nHandle, const Any& rValue )
throw (staruno::Exception)
{
	switch (nHandle)
	{
		case PROP_ISPASSTHROUGH:
			DBG_ASSERT(rValue.getValueTypeClass() == staruno::TypeClass_BOOLEAN, "invalid type" );

			m_bPassThrough = ::utl::getBOOL(rValue);
			break;
		case PROP_FILTER:
			DBG_ASSERT(rValue.getValueTypeClass() == staruno::TypeClass_STRING, "invalid type" );

			rValue >>= m_aFilter;
			break;
		case PROP_SORT:
			DBG_ASSERT(rValue.getValueTypeClass() == staruno::TypeClass_STRING, "invalid type" );

			rValue >>= m_aSort;
			break;
		case PROP_QUERY:
			DBG_ASSERT(rValue.getValueTypeClass() == staruno::TypeClass_STRING, "invalid type" );

			rValue >>= m_aQuery;
			break;
		default:
			DBG_ERROR("unknown Property");
	}
}

//------------------------------------------------------------------------------
void SbaXdbQuery::getFastPropertyValue( Any& rValue, sal_Int32 nHandle ) const
{
	switch (nHandle)
	{
		case PROP_ISPASSTHROUGH:
			::utl::setBOOL(rValue,m_bPassThrough);
			break;
		case PROP_FILTER:
			rValue <<= m_aFilter;
			break;
		case PROP_SORT:
			rValue <<= m_aSort;
			break;
		case PROP_QUERY:
			rValue <<= m_aQuery;
			break;
		case PROP_URL:
			rValue <<= (isOpen() ? ::rtl::OUString(m_xQuery->ToURL()) : ::rtl::OUString());
			break;
		case PROP_NAME:
			rValue <<= ::rtl::OUString(const_cast<SbaXdbQuery*>(this)->getName());
			break;
		default:
			DBG_ERROR("unknown Porperty");
	}
}

// stardata::XDatabaseUpdate
//------------------------------------------------------------------------------
void SAL_CALL SbaXdbQuery::update(void)
	throw( starlang::NoSupportException, stardata::DatabaseException, staruno::RuntimeException  )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	if (!m_xQuery->Update(m_aQuery, m_bPassThrough))
		checkDBResult(m_xQuery->Status(), *this);
}

//------------------------------------------------------------------------------
void SbaXdbQuery::setObject(const SbaQueryDefRef& xDef) throw( stardata::DatabaseException, staruno::RuntimeException  )
{
	::osl::MutexGuard aGuard( m_aMutex );

	if (xDef->Update(m_aQuery, m_bPassThrough))
		m_xQuery = xDef;
	else
		checkDBResult(xDef->Status(), *this);
}

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
SbaXdbQueries::SbaXdbQueries(SbaXdbObjectCreator& _rParent)
			  :SbaXdbNamedCollection(_rParent, dbQuery,::getCppuType(static_cast<const Reference< stardata::XDatabaseQuery >*>(0)) )
{
}

// stardata::XDatabaseQueryFactory
//------------------------------------------------------------------------------
Reference< stardata::XDatabaseQuery > SAL_CALL SbaXdbQueries::create(void)
			throw( starlang::NoSupportException, staruno::RuntimeException )
{
	checkOpen();
	return new SbaXdbQuery(getXdbCreator()->isCaseSensitive());
}

// helper methods
//------------------------------------------------------------------------------
SbaXdbObject* SbaXdbQueries::doGet(SdbObj* pObj) const
{
	SbaXdbQuery* pXObj = NULL;

	// Dokument schon geoeffnet
	sal_uInt32 nCount = size();
	sal_uInt32 i;
	for (i = 0; i < nCount; i++)
	{
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel((::cppu::OWeakObject*)getObject(i),::com::sun::star::uno::UNO_QUERY);
		if(xTunnel.is())
		{
			pXObj = (SbaXdbQuery*)xTunnel->getSomething(SbaXdbQuery::getUnoTunnelImplementationId());
		}
		//	utl::getImplementation(pXObj,getObject(i));

		if (pXObj && pObj == &pXObj->m_xQuery)
			break;
	}

	if (i == nCount)
		pXObj = new SbaXdbQuery(*(SbaXdbQueries*)this, (SbaQueryDef*)pObj, getXdbCreator()->isCaseSensitive());

	return pXObj;
}

//------------------------------------------------------------------------------
sal_Bool SbaXdbQueries::doRemove(SbaXdbObject* pObj)
{
	SbaXdbQuery* pXObj = (SbaXdbQuery*)pObj;

	if (!pXObj->m_xQuery->Drop())
		checkDBResult(pXObj->m_xQuery->Status(), *this, sal_True);
	return sal_True;
}

//------------------------------------------------------------------------------
SbaXdbObject* SbaXdbQueries::doInsert(const ::rtl::OUString& Name, const Any& Element)
{
	// Ist es ein bekanntes objekt
	SbaXdbQuery* pXObj = 0;
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel;
	Element >>= xTunnel;
	if(xTunnel.is())
	{
		pXObj = (SbaXdbQuery*)xTunnel->getSomething(SbaXdbQuery::getUnoTunnelImplementationId());
	}
	//	utl::getImplementation(pXObj,Element);

	DBG_ASSERT(pXObj, "SbaXdbQueries::doInsert for invalid element");

	// bereits in einem Container enthalten?
	if (!pXObj || pXObj->getXdbParent())
	{
		throw starlang::IllegalArgumentException();
	}

	SdbObj* pObj = NULL;

	try
	{
		pObj = getXdbCreator()->createObject(m_eObj, Name);
	}
	catch (stardata::DatabaseException& e)
	{
		throw starlang::WrappedTargetException(::rtl::OUString(), *this, staruno::makeAny(e));
	}

	SbaQueryDefRef xQry = (SbaQueryDef*)pObj;
	pXObj->setObject(xQry);
	return pXObj;
}

















