/*************************************************************************
 *
 *  $RCSfile: sbadbwsp.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:25:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SBA_SBADBWSP_HXX
#include "sbadbwsp.hxx"
#endif
#ifndef _SDB_SDBTOOLS_HXX
#include <sdb/sdbtools.hxx>
#endif

#ifndef _COM_SUN_STAR_UTIL_XLOCALIZEDALIASES_HPP_
#include <com/sun/star/util/XLocalizedAliases.hpp>
#endif

#ifndef _UTL_ENUMHELPER_HXX_
#include <unotools/enumhelper.hxx>
#endif

#ifndef _SBA_SBADBENG_HXX
#include "sbadbeng.hxx"
#endif

using staruno::Any;
using staruno::Sequence;
using staruno::Reference;
using staruno::UNO_QUERY;


//------------------------------------------------------------------------------
SbaXdbWorkspace::SbaXdbWorkspace()
				:m_aDatabases(*this)
				,m_bDefault(sal_False)
{
}

//------------------------------------------------------------------------------
SbaXdbWorkspace::SbaXdbWorkspace(SbaXdbCollection& rCollection, SdbWorkspace* pWs, sal_Bool bDefault)
				:SbaXdbObject(rCollection)
				,m_aDatabases(*this)
				,m_xWorkspace(pWs)
				,m_bDefault(bDefault)
{
}

//------------------------------------------------------------------------------
SbaXdbWorkspace::~SbaXdbWorkspace()
{
	if (!ComponentHelper::rBHelper.bDisposed)
	{
		acquire();
		dispose();
	}
}

// XUnoTunnel
using namespace ::cppu;
using namespace ::osl;
// -------------------------------------------------------------------------
::com::sun::star::uno::Sequence< sal_Int8 > SbaXdbWorkspace::getUnoTunnelImplementationId()
{
	static OImplementationId * pId = 0;
	if (! pId)
	{
		MutexGuard aGuard( Mutex::getGlobalMutex() );
		if (! pId)
		{
			static OImplementationId aId;
			pId = &aId;
		}
	}
	return pId->getImplementationId();
}
//------------------------------------------------------------------------------
sal_Int64 SAL_CALL SbaXdbWorkspace::getSomething(staruno::Sequence<sal_Int8> const& rId)throw( staruno::RuntimeException )
{
	if (rId.getLength() == 16 && 0 == rtl_compareMemory(getUnoTunnelImplementationId().getConstArray(),  rId.getConstArray(), 16 ) )
		return (sal_Int64)this;

	return SbaXdbObject::getSomething(rId);
}

// starlang::XComponent
//------------------------------------------------------------------------------
void SAL_CALL SbaXdbWorkspace::dispose(void) throw( staruno::RuntimeException )
{
	if (!m_bDefault)
		SbaXdbObject::dispose();
}

// ComponentHelper
//------------------------------------------------------------------------------
void SbaXdbWorkspace::disposing(void)
{
	SbaXdbObject::disposing();
	m_aDatabases.disposing();
	m_xWorkspace = NULL;
}

// starcontainer::XNamed
//------------------------------------------------------------------------------
::rtl::OUString SAL_CALL SbaXdbWorkspace::getName(void) throw( staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );

	return isOpen() ? ::rtl::OUString(m_xWorkspace->GetName()) : ::rtl::OUString();
}

//------------------------------------------------------------------------------
void SAL_CALL SbaXdbWorkspace::setName(const ::rtl::OUString& _rName) throw( staruno::RuntimeException )
{
	try
	{
		::osl::MutexGuard aGuard( m_aMutex );
		checkOpen();

		m_xWorkspace->Rename(_rName);

		checkDBResult(m_xWorkspace->Status(), *this, sal_True);
	}
	catch (staruno::RuntimeException& )
	{
		throw;
	}
	catch (...)
	{
		throw staruno::RuntimeException();
	}
}

// stardata::XDatabaseWorkspace
//------------------------------------------------------------------------------
Reference< stardata::XDatabase >
	SAL_CALL SbaXdbWorkspace::create(
				const ::rtl::OUString& DatabaseName,
				const ::rtl::OUString& Title,
				const Sequence< starbeans::PropertyValue >& ConnectInfo
			)
		throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();

	String aConnectStr(SbaXdbDatabase::toConnectionStr(ConnectInfo));
	Reference< stardata::XDatabase >  xDatabase;

	if (m_xWorkspace->CreateDatabase(DatabaseName, Title, aConnectStr))
	{
		SbaDatabase* pDB = (SbaDatabase*)m_xWorkspace->OpenDatabase(DatabaseName);
		if (pDB)
			xDatabase = new SbaXdbDatabase(m_aDatabases, pDB);
	}
	else
		checkDBResult(m_xWorkspace->Status(), *this, sal_False);

	return xDatabase;
}

//------------------------------------------------------------------------------
Reference< stardata::XDatabase > SAL_CALL SbaXdbWorkspace::open(const ::rtl::OUString& DatabaseName)
	throw( stardata::DatabaseException, staruno::RuntimeException )
{
	checkOpen();

//	String aRealName = SbaDatabase::GetDatabaseFilename(DatabaseName);
	// don't use the SbaDatabase::GetDatabaseFilename, this is obsolete
	// use the stardata::XDatabaseFavorites instead, though they are obsolete, too :)
	// FS - 24.01.00 - 71571
	String aRealName;
	Reference< stardata::XDatabaseFavorites >  xFavorites(
		getXdbContext().getServiceManager()->createInstance(::rtl::OUString::createFromAscii("com.sun.star.data.DatabaseEngine")), UNO_QUERY);
	if (xFavorites.is())
		aRealName = (const sal_Unicode*)xFavorites->getLocation(DatabaseName);
	if (!aRealName.Len())
		aRealName = (const sal_Unicode*)DatabaseName;

	Reference< stardata::XDatabase >  xDatabase;
	SbaDatabase* pDB = (SbaDatabase*)m_xWorkspace->OpenDatabase(aRealName);

	if (pDB)
	{
		// Datenbank bereits geoeffnet
		sal_Int32 nCount = m_aDatabases.size();
		for (sal_Int32 i = 0; i < nCount; i++)
		{
			SbaXdbDatabase* pXObj = 0;
			//	utl::getImplementation(pXObj,m_aDatabases.getObject(i));
			::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel((::cppu::OWeakObject*)m_aDatabases.getObject(i),::com::sun::star::uno::UNO_QUERY);
			if(xTunnel.is())
			{
				pXObj = (SbaXdbDatabase*)xTunnel->getSomething(SbaXdbDatabase:: getUnoTunnelImplementationId());
			}

			if (pXObj && pDB == &pXObj->m_xDatabase)
			{
				xDatabase = pXObj;
				break;
			}
		}
		if (!xDatabase.is())
		{
			SbaXdbDatabase* pXObj = new SbaXdbDatabase(m_aDatabases, pDB);
			xDatabase = pXObj;
		}
	}
	else
		checkDBResult(m_xWorkspace->Status(), *this, sal_False);

	return xDatabase;
}

// starcontainer::XElementAccess
//------------------------------------------------------------------------------
sal_Bool SAL_CALL SbaXdbWorkspace::hasElements(void) throw( staruno::RuntimeException )
{
	return !m_aDatabases.empty();
}

//------------------------------------------------------------------------------
staruno::Type SAL_CALL SbaXdbWorkspace::getElementType(void) throw( staruno::RuntimeException )
{
	Reference<stardata::XDatabase> const*const pXDatabase = 0;
	return ::getCppuType(pXDatabase);
}

// starcontainer::XEnumerationAccess
//------------------------------------------------------------------------------
Reference< starcontainer::XEnumeration > SAL_CALL SbaXdbWorkspace::createEnumeration(void)
	throw( staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	checkOpen();

	return new ::utl::OEnumerationByIndex( static_cast<starcontainer::XIndexAccess*>(this));
}

// starcontainer::XIndexAccess
//------------------------------------------------------------------------------
Any SAL_CALL SbaXdbWorkspace::getByIndex(sal_Int32 Index)
	throw( staruno::RuntimeException,  starlang::WrappedTargetException, starlang::IndexOutOfBoundsException )
{
	checkOpen();
	checkIndex(this, Index);

	Reference< staruno::XInterface > xObj(*m_aDatabases.getObject(Index));
	Reference< stardata::XDatabase > xDatabase( xObj, UNO_QUERY);
	return staruno::makeAny(xDatabase);
}

// helper
//------------------------------------------------------------------------------
void SbaXdbWorkspace::setObject(const SdbWorkspaceRef& xWs)
	throw( stardata::DatabaseException, staruno::RuntimeException )
{
	::osl::MutexGuard aGuard( m_aMutex );
	m_xWorkspace = xWs;
}

//------------------------------------------------------------------------------
SbaXdbWorkspaces::SbaXdbWorkspaces(SbaXdbEngine& rParent)
: SbaXdbNamedCollection(rParent, dbWorkspace, ::getCppuType(static_cast<const Reference< stardata::XDatabaseWorkspace >*>(0)) )
{
	m_xEnvironment = rParent.m_xEnvironment;
}


//------------------------------------------------------------------------------
void SbaXdbWorkspaces::disposing()
{
	SbaXdbNamedCollection::disposing();
	m_xEnvironment = NULL;
}

// starcontainer::XIndexAccess
//------------------------------------------------------------------------------
Any SAL_CALL SbaXdbWorkspaces::getByIndex(sal_Int32 Index)
	throw( starlang::IndexOutOfBoundsException, starlang::WrappedTargetException, staruno::RuntimeException )
{
	checkIndex(this, Index);

	// Datenbank bereits geoeffnet
	Reference< staruno::XInterface > xObj( *getObject(Index) );
	Reference< stardata::XDatabaseWorkspace >  xWsp(xObj , UNO_QUERY);
	return staruno::makeAny ( xWsp );
}

// stardata::XDatabaseWorkspaceFactory
//------------------------------------------------------------------------------
Reference< stardata::XDatabaseWorkspace > SAL_CALL SbaXdbWorkspaces::create(void)
	throw( staruno::RuntimeException )
{
	return new SbaXdbWorkspace();
}

// helper methods
//------------------------------------------------------------------------------
SbaXdbObject* SbaXdbWorkspaces::doGet(SdbObj* pObj) const
{
	SbaXdbWorkspace* pXObj = NULL;

	// Dokument schon geoeffnet
	sal_Int32 nCount = size();
	sal_Int32 i;
	for (i = 0; i < nCount; i++)
	{
		//	utl::getImplementation(pXObj,getObject(i));
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel((::cppu::OWeakObject*)getObject(i),::com::sun::star::uno::UNO_QUERY);
		if(xTunnel.is())
		{
			pXObj = (SbaXdbWorkspace*)xTunnel->getSomething(SbaXdbWorkspace::getUnoTunnelImplementationId());
		}

		if (pXObj && pObj == &pXObj->m_xWorkspace)
			break;
	}

	if (i == nCount)
		pXObj = new SbaXdbWorkspace(*const_cast<SbaXdbWorkspaces*>(this), static_cast<SdbWorkspace*>(pObj));

	return pXObj;
}

//------------------------------------------------------------------------------
sal_Bool SbaXdbWorkspaces::doRemove(SbaXdbObject* pObj)
{
	SbaXdbWorkspace* pXObj = (SbaXdbWorkspace*)pObj;
	if (pXObj->m_xWorkspace->GetName().EqualsAscii(DEFAULT_WORKSPACENAME))
	{
		throw starlang::IllegalArgumentException();
	}

	if (!m_xEnvironment->RemoveWorkspace(pXObj->m_xWorkspace))
		checkDBResult(m_xEnvironment->Status(), *this, sal_True);

	return sal_True;
}

//------------------------------------------------------------------------------
SbaXdbObject* SbaXdbWorkspaces::doInsert(const ::rtl::OUString& Name, const Any& Element)
{
	// Ist es ein bekannter Workspace
	SbaXdbWorkspace* pXObj = 0;
	//	utl::getImplementation(pXObj,Element);
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XUnoTunnel> xTunnel;
	Element >>= xTunnel;
	if(xTunnel.is())
	{
		pXObj = (SbaXdbWorkspace*)xTunnel->getSomething(SbaXdbWorkspace::getUnoTunnelImplementationId());
	}

	DBG_ASSERT(pXObj, "SbaXdbWorkspaces::doInsert for invalid element");

	// bereits in einem Container enthalten?
	if (!pXObj || pXObj->getXdbParent())
	{
		throw starlang::IllegalArgumentException() ;
	}

	SdbWorkspaceRef xObj;
	try
	{
		xObj = (SdbWorkspace*)getXdbCreator()->createObject(dbWorkspace, Name);
	}
	catch(stardata::DatabaseException& e)
	{
		throw starlang::WrappedTargetException(::rtl::OUString(), *this, staruno::makeAny(e) );
	}

	m_xEnvironment->AppendWorkspace(xObj);
	checkDBResult(m_xEnvironment->Status(), *this, sal_True);
	pXObj->setObject(xObj);

	return pXObj;
}

















