/*************************************************************************
 *
 *  $RCSfile: dbstat.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBA_SBASTAT_HXX
#define _SBA_SBASTAT_HXX 1

#ifndef _SBA_SBAOBJ_HXX
#include "sbaobj.hxx"
#endif

#ifndef _COM_SUN_STAR_DATA_DATABASEDETAILEDEXCEPTION_HPP_
#include <com/sun/star/data/DatabaseDetailedException.hpp>
#endif

// Statuscodes:
enum SbaStatusCode {
	SBA_SUCCESS = 20,			// keine weiteren Informationen verfuegbar
	SBA_SUCCESS_WITH_INFO,		// erfolgreich, aber mit Info/Warnung
	SBA_NO_DATA_FOUND,			// alle Rows der Ergebnismenge wurden abgeholt,
								// keine weiteren Daten verfuegbar. Keine weitere Info.
	SBA_ERROR					// Funktionsaufruf gescheitert. Weitere Info vorhanden.
};

// Class SbaStatus
//
// Beschreibung:
//	Status der letzten Sba-Operation:
//	(SbaStatus-Objekt als Member in jedem Document)
//
class SbaStatus
{
  private:
	// Leerstring (wird manchmal fuer return benoetigt):
	static const String aEmptyString;

	// Ergebnis der letzten Operation:
	enum SbaStatusCode eResult;

	// Statuscode (identisch mit der Resource Id des
	// zugehoerigen Textes). 0 wenn ! IsInformationAvailable.
	sal_uInt16 nMessageNumber;

	// Statustext zur letzten Operation
	// (leer, wenn ! IsInformationAvailable):
	String aMessage;

	// Der zuletzt aufgetretene Sba-Status ("low level"-DB):
	SdbStatus aSdbStatus;

	// Registrierte Callback-Funktion
	Link aStatusChangeHdl;

	// Ist der Sdb-Status ("low level"-DB-Status) verfuegbar?
	sal_Bool bSdbStatusAvailable;

  public:
	SbaStatus();

	// Callback-Funktion wird NICHT mitkopiert.
	// Beim Kopieren erfolgt KEIN Callback beim Zielobjekt.
	SbaStatus(const SbaStatus &right);

	const SbaStatus & operator=(const SbaStatus &rSbaStatus);

	sal_Bool operator==(const SbaStatus &rSbaStatus) const;
	sal_Bool operator!=(const SbaStatus &rSbaStatus) const;

	~SbaStatus();

	// Zugriff auf Elemente (Beschreibung siehe oben):
	const enum SbaStatusCode Result() const;

	const String & Message() const;

	const sal_uInt16 MessageNumber() const;

	// Sind zusdtzliche Informationen zum Resultat vorhanden
	// (Message und ggf. SbaStatus)
	// (nur bei SUCCESS_WITH_INFO oder ERROR)?
	const sal_Bool IsInformationAvailable() const;

	// Ist der "low level"-DB-Status zur Abfrage verfuegbar?
	// Nur dann darf der SdbStatus abgefragt werden.
	const sal_Bool IsSdbStatusAvailable() const;
	const SdbStatus & SdbStatusObject() const;


	// Ist die letzte Datenbankoperation erfolgreich
	// durchgelaufen (SUCCESS oder SUCCESS_WITH_INFO)?
	const sal_Bool IsSuccessful() const;

	// Hat die letzte Datenbankoperation einen Fehler
	// ergeben (ERROR)? NO_DATA_FOUND und
	// SUCCESS_WITH_INFO zaehlen nicht als Fehler.
	const sal_Bool IsError() const;

	// Registrierung eines Links, ueber den der Aufrufer
	// benachrichtigt wird, wenn sich der Status aendert.
	void SetStatusChangeHdl(const Link& rNewLink);
	const Link & StatusChangeHdl() const;

	// Status der letzten Operation setzen (geschieht nur in
	// den Dokumentklassen).

	// Durch Setzen des Status wird ein etwaiger registrierter Callback zur
	// Benachrichtigung ueber den Ausgang des Statements
	// ausgefuehrt.

	// Der Text zur MessageNumber kommt immer aus einer Ressource-Datei (nMessageNumber
	// ist die betreffende Resource Id). Bei Meldungen ohne Text
	// (! IsInformationAvailable) darf keine MessageNumber angegeben werden.
	// Optional mit Angabe des SbaStatus ("low level"-DB-Status).
	// Optional Parameter, die in den String aus der Ressource-Datei
	// eingesetzt werden, und zwar der erste Parameter dort, wo im
	// String "%1" auftritt, der zweite bei "%2" und der dritte bei "%3".
	// Fuer ein Prozent-Zeichen ist "%%" einzusetzen.
	void Set(SbaStatusCode eNewResult, DBErrorContext eContext,
			 const char * pParam1 = "", const char * pParam2 = "", const char * pParam3 = "");
	void Set(SbaStatusCode eNewResult, const SdbStatus & rNewSdbStatus, DBErrorContext eContext,
			 const char * pParam1 = "", const char * pParam2 = "", const char * pParam3 = "");
	void Set(SbaStatusCode eNewResult, const ::com::sun::star::data::DatabaseDetailedException& rUnoDescription);

	// Beim Beginn einer neuen Datenbankoperation wird
	// Clear aufgerufen, um die vorher gesetzten
	// Fehlermeldungen zu loeschen und den Status auf
	// STILL_EXECUTING zu setzen.
	void Clear();

};


// Inline-Implementierungen:

inline sal_Bool SbaStatus::operator!=(const SbaStatus &rSbaStatus) const
{
	return ! (*this == rSbaStatus);
}

inline const sal_Bool SbaStatus::IsInformationAvailable() const
{
	return eResult == SBA_SUCCESS_WITH_INFO || eResult == SBA_ERROR;
}

inline const sal_Bool SbaStatus::IsSuccessful() const
{
	return eResult == SBA_SUCCESS || eResult == SBA_SUCCESS_WITH_INFO;
}

inline const sal_Bool SbaStatus::IsError() const
{
	return eResult == SBA_ERROR;
}

inline const enum SbaStatusCode SbaStatus::Result() const
{
	return eResult;
}

inline const sal_uInt16 SbaStatus::MessageNumber() const
{
	return nMessageNumber;
}

inline const String & SbaStatus::Message() const
{
	return IsInformationAvailable() ? aMessage : aEmptyString;
}

inline const sal_Bool SbaStatus::IsSdbStatusAvailable() const
{
	return bSdbStatusAvailable;
}

inline const SdbStatus & SbaStatus::SdbStatusObject() const
{
	return aSdbStatus;
}

inline const Link & SbaStatus::StatusChangeHdl() const
{
	return aStatusChangeHdl;
}

inline void SbaStatus::Clear()
{
	eResult = SBA_SUCCESS;
}


inline void SbaStatus::SetStatusChangeHdl(const Link& rNewLink)
{
	aStatusChangeHdl = rNewLink;
}


#endif



