/*************************************************************************
 *
 *  $RCSfile: impexp.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBA_IMPEXP_HXX
#define _SBA_IMPEXP_HXX

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifndef _SBA_DBDEFOBJ_HXX //autogen
#include "dbdefobj.hxx"
#endif

#ifndef _SV_FONT_HXX //autoge
#include <vcl/font.hxx>
#endif

// =========================================================================
// SbaImportExport Basisklasse f"ur Import/Export
// =========================================================================
class SbaParser;
class SbaImportExport
{
protected:
	SvStream&			m_aStream;
	SbaDBDefRef			m_rDBDef; // gerade aktuelles Objekt
	SbaDatabaseRef		m_rDatabase;
	Font				m_aFont;

#if defined UNX || defined MAC
	static const char __FAR_DATA sNewLine;
#else
	static const char __FAR_DATA sNewLine[];
#endif
	SbaParser*			m_pReader;
	ULONG*				m_pRowMarker; // wenn gesetzt, dann nur diese Rows kopieren

	SbaImportExport(SvStream &rStream,SbaDBDef& rDBDef,const String& rExchange = String());
	SbaImportExport(SvStream &rStream,SbaDatabase& rDatabase) : m_aStream(rStream),m_rDatabase(&rDatabase),m_pReader(NULL),m_pRowMarker(NULL) {};
public:
	virtual ~SbaImportExport();

	virtual BOOL Write()	= 0; // Export
	virtual BOOL Read()		= 0; // Import

};

// =========================================================================
// RTF Im- und Export
// =========================================================================

class SbaRTFImportExport : public SbaImportExport
{

public:
	SbaRTFImportExport(SvStream &rStream,SbaDBDef& rDBDef,const String& rExchange = String()) : SbaImportExport(rStream,rDBDef,rExchange) {};
	SbaRTFImportExport(SvStream &rStream,SbaDatabase& rDatabase) : SbaImportExport(rStream,rDatabase) {};

	virtual BOOL Write();
	virtual BOOL Read();
};
// =========================================================================
// HTML Im- und Export
// =========================================================================
#define SBA_HTML_FONTSIZES 7
const short nIndentMax = 23;
class SbaColumn;

class SbaHTMLImportExport : public SbaImportExport
{
	// default HtmlFontSz[1-7]
	static const USHORT	nDefaultFontSize[SBA_HTML_FONTSIZES];
	// HtmlFontSz[1-7] in s*3.ini [user]
	static USHORT		nFontSize[SBA_HTML_FONTSIZES];
	static const USHORT	nCellSpacing;
	static const char __FAR_DATA sIndentSource[];
	char				sIndent[nIndentMax+1];
	short				nIndent;
#if DBG_UTIL
	BOOL				m_bCheckFont;
#endif

	void WriteHeader();
	void WriteBody();
	void WriteTables();
	void WriteCell( ULONG nFormat,USHORT nWidthPixel,USHORT nHeightPixel,const char* pChar,const String& rValue,const char* pHtmlTag);
	void IncIndent( short nVal );
	const char*			GetIndentStr() { return sIndent; }
	void FontOn();
	inline void FontOff();

public:
	SbaHTMLImportExport(SvStream &rStream,SbaDBDef& rDBDef,const String& rExchange = String());
	SbaHTMLImportExport(SvStream &rStream,SbaDatabase& rDatabase) : SbaImportExport(rStream,rDatabase) {};

	virtual BOOL Write();
	virtual BOOL Read();

};


#endif // _SBA_IMPEXP_HXX


