/*************************************************************************
 *
 *  $RCSfile: qryconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SBA_QRYCONN_HXX
#define _SBA_QRYCONN_HXX

#ifndef _SBA_TABCONN_HXX
#include "tabconn.hxx"
#endif

#ifndef _SBA_QRYDGN1_HXX
#include "qrydgn1.hxx"
#endif // _SBA_QRYDGN1_HXX

//==================================================================
class QueryTabWinContainer;
class QueryTabWin;
// beschreibt die Verbindung zwischen zwei Tabellen im Abfrageentwurf
enum SbaJoinType
{
	SBA_FULL_JOIN,
	SBA_LEFT_JOIN,
	SBA_RIGHT_JOIN,
	SBA_NATURAL_JOIN,
	SBA_UNION_JOIN,
	SBA_CROSS_JOIN,
	SBA_INNER_JOIN
};
class QueryTabConnData : public SbaJoinTabConnData
{
	ULONG			nFromEntryIndex;
	ULONG			nDestEntryIndex;
	SbaJoinType		eJoinType;

	SbaTabFieldType	eFromType;
	SbaTabFieldType	eDestType;

	// meine Basisklasse merkt sich leider nur die Fensternamen (was hier das selbe wie die Aliasnamen ist), ich aber brauche
	// auch Zugriff auf die realen Tabellennamen. Damit ich mir nicht die Fenster dazu merken muss (das waere schmutzig, da die
	// TabConDatas normalerweise vom Dokument verwaltet werden und voellig losgeloest von der Repraesentation der Tabellen als
	// Fenster sein sollten), habe ich zwei String-Member
	String	m_strSourceTableName;
	String	m_strDestTableName;

protected:
	// fuer das Anlegen und Duplizieren von Lines vom eigenen Typ
	virtual SbaJoinConnLineData* CreateLineDataObj();
	virtual SbaJoinConnLineData* CreateLineDataObj( const SbaJoinConnLineData& rConnLineData );

public:
	TYPEINFO();
	QueryTabConnData();
	QueryTabConnData( const QueryTabConnData& rConnData );
	QueryTabConnData( const String& strSourceTable, const String& strDestTable,
		const String& strSourceAlias, const String& strDestAlias, const String& rConnName=String());
	virtual ~QueryTabConnData();

	virtual void CopyFrom(const SbaJoinTabConnData& rSource);
	virtual SbaJoinTabConnData* NewInstance() const { return new QueryTabConnData(); }

	QueryTabConnData& operator=( const QueryTabConnData& rConnData );

	String			GetTableName(JoinTabConnSide nWhich) const;
	String			GetAliasName(JoinTabConnSide nWhich) const;

	ULONG			GetFieldIndex(JoinTabConnSide nWhich) const { return nWhich==JTCS_TO ? nDestEntryIndex : nFromEntryIndex; }
	void			SetFieldIndex(JoinTabConnSide nWhich, ULONG nVal) { if (nWhich==JTCS_TO) nDestEntryIndex=nVal; else nFromEntryIndex=nVal; }

	SbaTabFieldType	GetFieldType(JoinTabConnSide nWhich) const { return nWhich==JTCS_TO ? eDestType : eFromType; }
	void			SetFieldType(JoinTabConnSide nWhich, SbaTabFieldType eType) { if (nWhich==JTCS_TO) eDestType=eType; else eFromType=eType; }

	void			InitFromDrag(const TabFieldDesc& rDragLeft, const TabFieldDesc& rDragRight);

	SbaJoinType		GetJoinType() const { return eJoinType; };
	void			SetJoinType(const SbaJoinType& eJT) { eJoinType = eJT; };

};


//==================================================================
class QueryTabConn : public SbaJoinTabConn
{
	BOOL m_bVisited;	// is true if the conn was already visited through the join algorithm
protected:
	SbaJoinConnLine* CreateConnLine( const SbaJoinConnLine& rConnLine );

public:
	TYPEINFO();
	QueryTabConn(QueryTabWinContainer* pContainer, QueryTabConnData* pTabConnData);
	QueryTabConn(const QueryTabConn& rConn);
	virtual ~QueryTabConn();

	QueryTabConn& operator=(const QueryTabConn& rConn);
	BOOL operator==(const QueryTabConn& rCompare);

	String	GetAliasName(JoinTabConnSide nWhich) const { return ((QueryTabConnData*)GetData())->GetAliasName(nWhich); }

	BOOL	IsVisited() const { return m_bVisited; }
	void	SetVisited(BOOL bVisited) { m_bVisited = bVisited; }

};

#endif


