/*************************************************************************
 *
 *  $RCSfile: qryundo.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// Definitionen in qryds.cxx
#ifndef _SBA_QRYUNDO_HXX
#define _SBA_QRYUNDO_HXX

#ifndef _SBA_QRYTXTVS_HXX
#include "qrytxtvs.hxx"
#endif

#ifndef _SBA_QRYDGN_HXX
#include "qrydgn.hxx"
#endif

#ifndef _SBA_QRYDGN1_HXX
#include "qrydgn1.hxx"
#endif

#ifndef _SBARESID_HRC
#include "sbaresid.hrc"
#endif

#ifndef _UNDO_HXX //autogen
#include <svtools/undo.hxx>
#endif

#ifndef _SBA_QRYTBWIN_HXX
#include "qrytbwin.hxx"
#endif	// _SBA_QRYTBWIN_HXX

#ifndef _SBA_QRYCONN_HXX
#include "qryconn.hxx"
#endif // _SBA_QRYCONN_HXX

#ifndef _SBA_QRYWINCT_HXX
#include "qrywinct.hxx"
#endif // _SBA_QRYWINCT_HXX

#ifndef _SBA_SBARES_HXX
#include "sbares.hxx"
#endif // _SBA_SBARES_HXX

#include "sbaresid.hrc"

// ================================================================================================
// SbaCommentUndoAction - Undo-Basisklasse fuer Aktionen, deren GetComment einen aus einer Sba-Ressource
// geladenen String liefert

class SbaCommentUndoAction : public SfxUndoAction
{
protected:
	String					m_strComment;

public:
	SbaCommentUndoAction(USHORT nCommentID) { m_strComment = String(SbaResId(nCommentID)); }

	String	GetComment() const { return m_strComment; }
};

// ================================================================================================
// QueryDesignUndoAction - Undo-Basisklasse fuer Aktionen im graphischen Abfrageentwurf (ohne Feldliste)

class QueryDesignUndoAction : public SbaCommentUndoAction
{
protected:
	QueryTabWinContainer*	m_pOwner;		// in diesem Container spielt sich alles ab

public:
	QueryDesignUndoAction(QueryTabWinContainer* pOwner, USHORT nCommentID) : SbaCommentUndoAction(nCommentID), m_pOwner(pOwner) { }
};

// ================================================================================================
// QueryTabConnUndoAction - Undo-Basisklasse fuer alles, was mit Einfuegen/Entfernen von Connections zu tun hat
DBG_NAMEEX(QueryTabConnUndoAction);
class QueryTabConnUndoAction : public QueryDesignUndoAction
{
protected:
	QueryTabConn*			m_pConnection;
	BOOL					m_bOwnerOfConn;
		// bin ich alleiniger Eigentuemer der Connection ? (aendert sich mit jedem Redo oder Undo)

public:
	QueryTabConnUndoAction(QueryTabWinContainer* pOwner, USHORT nCommentID);
	virtual ~QueryTabConnUndoAction();

	virtual void Undo() = 0;
	virtual void Redo() = 0;

	void SetConnection(QueryTabConn* pConn) { m_pConnection = pConn; }
		// anschliessend bitte SetOwnership
	void SetOwnership(BOOL bTakeIt) { m_bOwnerOfConn = bTakeIt; }
};

// ------------------------------------------------------------------------------------------------
inline QueryTabConnUndoAction::~QueryTabConnUndoAction()
{
	DBG_DTOR(QueryTabConnUndoAction,NULL);
	if (m_bOwnerOfConn)
	{	// ich besitze die Connection -> loeschen
		delete m_pConnection->GetData();
		delete m_pConnection;
	}
}

// ------------------------------------------------------------------------------------------------
inline QueryTabConnUndoAction::QueryTabConnUndoAction(QueryTabWinContainer* pOwner, USHORT nCommentID)
	:QueryDesignUndoAction(pOwner, nCommentID)
	,m_pConnection(NULL)
	,m_bOwnerOfConn(FALSE)
{
	DBG_CTOR(QueryTabConnUndoAction,NULL);
}

// ================================================================================================
// QueryAddTabConnUndoAction - Undo-Klasse fuer Einfuegen einer Connection

class QueryAddTabConnUndoAction : public QueryTabConnUndoAction
{
public:
	QueryAddTabConnUndoAction(QueryTabWinContainer* pOwner) : QueryTabConnUndoAction(pOwner, STR_QUERY_UNDO_INSERTCONNECTION) { }

	virtual void Undo() { m_pOwner->DropConnection(m_pConnection); SetOwnership(TRUE); }
	virtual void Redo() { m_pOwner->GetConnection(m_pConnection); SetOwnership(FALSE); }
};

// ================================================================================================
// QueryDelTabConnUndoAction - Undo-Klasse fuer Einfuegen einer Connection

class QueryDelTabConnUndoAction : public QueryTabConnUndoAction
{
public:
	QueryDelTabConnUndoAction(QueryTabWinContainer* pOwner) : QueryTabConnUndoAction(pOwner, STR_QUERY_UNDO_REMOVECONNECTION) { }

	virtual void Undo() { m_pOwner->GetConnection(m_pConnection); SetOwnership(FALSE); }
	virtual void Redo() { m_pOwner->DropConnection(m_pConnection); SetOwnership(TRUE); }
};

// ================================================================================================
// QueryTabWinUndoAct - Undo-Basisklasse fuer alles, was mit Einfuegen/Entfernen von TabWIns zu tun hat zu tun hat

class QueryTabWinUndoAct : public QueryDesignUndoAction
{
protected:
	SbaJoinTabConnList		m_lstConnections;
	QueryTabWin*			m_pTabWin;
	BOOL					m_bOwnerOfObjects;
		// bin ich alleiniger Eigentuemer der verwalteten Objekte ? (aendert sich mit jedem Redo oder Undo)

public:
	QueryTabWinUndoAct(QueryTabWinContainer* pOwner, USHORT nCommentID);
	virtual ~QueryTabWinUndoAct();

	void SetOwnership(BOOL bTakeIt) { m_bOwnerOfObjects = bTakeIt; }


	virtual void Undo() = 0;
	virtual void Redo() = 0;

	// Zugriff auf das TabWin
	void SetTabWin(QueryTabWin* pTW) { m_pTabWin = pTW; }
		// anschliessend sollte das SetOwnership aufgerufen werden

	// Zugriff auf die verwalteten Connections
	USHORT	ConnCount() { return m_lstConnections.Count(); }

	SbaJoinTabConn* FirstConn() { return m_lstConnections.First(); }
	SbaJoinTabConn* NextConn() { return m_lstConnections.Next(); }

	void InsertConnection( SbaJoinTabConn* pConnection ) { m_lstConnections.Insert(pConnection, LIST_APPEND); }
	void RemoveConnection( SbaJoinTabConn* pConnection ) { m_lstConnections.Remove(pConnection); }
};

// ------------------------------------------------------------------------------------------------
inline QueryTabWinUndoAct::QueryTabWinUndoAct(QueryTabWinContainer* pOwner, USHORT nCommentID)
	:QueryDesignUndoAction(pOwner, nCommentID)
	,m_pTabWin(NULL)
	,m_bOwnerOfObjects(FALSE)
{
}

// ================================================================================================
// QueryTabWinShowUndoAct - Undo-Klasse fuer Anzeigen eines TabWins

class QueryTabWinShowUndoAct : public QueryTabWinUndoAct
{
public:
	QueryTabWinShowUndoAct(QueryTabWinContainer* pOwner) : QueryTabWinUndoAct(pOwner, STR_QUERY_UNDO_TABWINSHOW) { }

	virtual void	Undo() { m_pOwner->HideTabWin(m_pTabWin, this); SetOwnership(TRUE); }
	virtual void	Redo() { m_pOwner->ShowTabWin(m_pTabWin, this); SetOwnership(FALSE); }
};

// ================================================================================================
// QueryTabWinDelUndoAct - Undo-Klasse fuer Loeschen eines TabWins

class QueryTabWinDelUndoAct : public QueryTabWinUndoAct
{
public:
	QueryTabWinDelUndoAct(QueryTabWinContainer* pOwner) : QueryTabWinUndoAct(pOwner, STR_QUERY_UNDO_TABWINDELETE) { }

	virtual void	Undo() { m_pOwner->ShowTabWin( m_pTabWin, this ); SetOwnership(FALSE); }
	virtual void	Redo() { m_pOwner->HideTabWin( m_pTabWin, this ); SetOwnership(TRUE); }
};

// ================================================================================================
// QueryMoveTabWinUndoAct - Undo-Klasse fuer Verschieben eines TabWins

class QueryMoveTabWinUndoAct : public QueryDesignUndoAction
{
	Point					m_ptNextPosition;
	QueryTabWin*			m_pTabWin;

protected:
	void TogglePosition();

public:
	QueryMoveTabWinUndoAct(QueryTabWinContainer* pOwner, const Point& ptOriginalPosition, QueryTabWin* pTabWin);

	virtual void	Undo() { TogglePosition(); }
	virtual void	Redo() { TogglePosition(); }
};

// ------------------------------------------------------------------------------------------------
inline QueryMoveTabWinUndoAct::QueryMoveTabWinUndoAct(QueryTabWinContainer* pOwner, const Point& ptOriginalPosition, QueryTabWin* pTabWin)
	:QueryDesignUndoAction(pOwner, STR_QUERY_UNDO_MOVETABWIN)
	,m_ptNextPosition(ptOriginalPosition)
	,m_pTabWin(pTabWin)
{
}

// ================================================================================================
// QuerySizeTabWinUndoAct - Undo-Klasse fuer Groessenveraenderung eines TabWins

class QuerySizeTabWinUndoAct : public QueryDesignUndoAction
{
	Point					m_ptNextPosition;
	Size					m_szNextSize;
	QueryTabWin*			m_pTabWin;

protected:
	inline void ToggleSizePosition();

public:
	QuerySizeTabWinUndoAct(QueryTabWinContainer* pOwner, const Point& ptOriginalPos, const Size& szOriginalSize, QueryTabWin* pTabWin);
	// Nebenbedingung : es darf nicht gescrollt worden sein, waehrend die neue Groesse/Position ermittelt wurde, das heisst, die Position
	// hier sind physische, nicht logische Koordinaten
	// (im Gegensatz zur QueryMoveTabWinUndoAct)

	virtual void	Undo() { ToggleSizePosition(); }
	virtual void	Redo() { ToggleSizePosition(); }
};

//------------------------------------------------------------------------------
inline QuerySizeTabWinUndoAct::QuerySizeTabWinUndoAct(QueryTabWinContainer* pOwner, const Point& ptOriginalPos, const Size& szOriginalSize, QueryTabWin* pTabWin)
	:QueryDesignUndoAction(pOwner, STR_QUERY_UNDO_SIZETABWIN)
	,m_ptNextPosition(ptOriginalPos)
	,m_szNextSize(szOriginalSize)
	,m_pTabWin(pTabWin)
{
}

//------------------------------------------------------------------------------
inline void QuerySizeTabWinUndoAct::ToggleSizePosition()
{
	Point ptNext = m_pTabWin->GetPosPixel();
	Size szNext = m_pTabWin->GetSizePixel();

	m_pOwner->Invalidate(INVALIDATE_NOCHILDREN);
	m_pTabWin->SetPosSizePixel(m_ptNextPosition, m_szNextSize);
	m_pOwner->Invalidate(INVALIDATE_NOCHILDREN);

	m_ptNextPosition = ptNext;
	m_szNextSize = szNext;
}

// ================================================================================================
// SbaQueryDesignFieldUndoAct - Basisklasse fuer Undos in der Feldauflistung im Abfrageentwurf

class SelectionBrowseBox;
class SbaQueryDesignFieldUndoAct : public SbaCommentUndoAction
{
protected:
	SelectionBrowseBox*		pOwner;
	long					m_nColId;

	virtual void	Undo() = 0;
	virtual void	Redo() = 0;

public:
	SbaQueryDesignFieldUndoAct(SelectionBrowseBox* pSelBrwBox, USHORT nCommentID) : SbaCommentUndoAction(nCommentID), pOwner(pSelBrwBox), m_nColId(-1) { }

	void SetColId(long nCol) { m_nColId = nCol; }
};

// ================================================================================================
// SbaTabFieldCellModifiedUndoAct - Undo-Klasse fuer Aendern einer Zelle einer Spaltenbeschreibung

class SbaTabFieldCellModifiedUndoAct : public SbaQueryDesignFieldUndoAct
{
protected:
	String		m_strNextCellContents;
	USHORT		m_nCellIndex;

public:
	SbaTabFieldCellModifiedUndoAct(SelectionBrowseBox* pSelBrwBox) : SbaQueryDesignFieldUndoAct(pSelBrwBox, STR_QUERY_UNDO_MODIFY_CELL) { }

	void SetCellContents(const String& str) { m_strNextCellContents = str; }
	void SetCellIndex(USHORT nIndex) { m_nCellIndex = nIndex; }

	virtual void Undo();
	virtual void Redo() { Undo(); }
};

// ================================================================================================
// SbaTabFieldSizedUndoAct - Undo-Klasse fuer Aendern einer Spaltenbreite

class SbaTabFieldSizedUndoAct : public SbaQueryDesignFieldUndoAct
{
protected:
	long		m_nNextWidth;

public:
	SbaTabFieldSizedUndoAct(SelectionBrowseBox* pSelBrwBox) : SbaQueryDesignFieldUndoAct(pSelBrwBox, STR_QUERY_UNDO_SIZE_COLUMN), m_nNextWidth(0) { }

	void SetOriginalWidth(long nWidth) { m_nNextWidth = nWidth; }

	virtual void Undo();
	virtual void Redo() { Undo(); }
};

// ================================================================================================
// SbaTabFieldUndoAct - Basisklasse fuer Undos in der Feldauflistung im Abfrageentwurf, die mit Veraendern einer kompletten Feldbeschreibung zu tun haben

class SbaTabFieldUndoAct : public SbaQueryDesignFieldUndoAct
{
protected:
	TabFieldDesc*			pDescr;		// geloeschte Spaltenbeschreibung
	BOOL					m_bOwnerOfDescription;

public:
	SbaTabFieldUndoAct(SelectionBrowseBox* pSelBrwBox, USHORT nCommentID) : SbaQueryDesignFieldUndoAct(pSelBrwBox, nCommentID) { }
	virtual ~SbaTabFieldUndoAct() { if (m_bOwnerOfDescription) delete pDescr; }

	void SetTabFieldDescr(TabFieldDesc* pDescription) { pDescr = pDescription; }
		// anschliessend bitte SetOwnership
	void SetOwnership(BOOL bTakeIt) { m_bOwnerOfDescription = bTakeIt; }
};

// ================================================================================================
// SbaTabFieldDelUndoAct - Undo-Klasse fuer Loeschen eines Feldes

class SbaTabFieldDelUndoAct : public SbaTabFieldUndoAct
{
protected:
	virtual void Undo() { pOwner->InsertColumn(pDescr, m_nColId); SetOwnership(FALSE); }
	virtual void Redo() { pOwner->RemoveColumn((USHORT)m_nColId); SetOwnership(TRUE); }

public:
	SbaTabFieldDelUndoAct(SelectionBrowseBox* pSelBrwBox) : SbaTabFieldUndoAct(pSelBrwBox, STR_QUERY_UNDO_TABFIELDDELETE) { }
};

// ================================================================================================
// SbaTabFieldDelUndoAct - Undo-Klasse fuer Anlegen eines Feldes

class SbaTabFieldCreateUndoAct : public SbaTabFieldUndoAct
{
protected:
	virtual void Undo() { pOwner->RemoveColumn((USHORT)m_nColId); SetOwnership(TRUE); }
	virtual void Redo() { pOwner->InsertColumn(pDescr, m_nColId); SetOwnership(FALSE); }

public:
	SbaTabFieldCreateUndoAct(SelectionBrowseBox* pSelBrwBox) : SbaTabFieldUndoAct(pSelBrwBox, STR_QUERY_UNDO_TABFIELDCREATE) { }
};

// ================================================================================================
// SbaSqlEditUndoAct - Undo-Klasse fuer direktes Aendern des SQL-Statements
//------------------------------------------------------------------------
class SbaSqlEditUndoAct : public SbaCommentUndoAction
{
protected:
	SqlEdit* pOwner;
	String	m_strNextText;

	virtual void	Undo() { ToggleText(); }
	virtual void	Redo() { ToggleText(); }

	void ToggleText();

public:
	SbaSqlEditUndoAct(SqlEdit* pEdit) : SbaCommentUndoAction(STR_QUERY_UNDO_MODIFYSQLEDIT), pOwner(pEdit) { }

	void SetOriginalText(const String& strText) { m_strNextText = strText; }
};
//------------------------------------------------------------------------
// ================================================================================================
// SbaSqlNativeUndoAct - Undo-Klasse for changing the native flag
// ================================================================================================
class SbaSqlNativeUndoAct : public SbaSqlEditUndoAct
{
protected:
	BOOL m_bNative;

	virtual void	Undo() { ToggleNative();ToggleText(); }
	virtual void	Redo() { ToggleNative();ToggleText(); }

	void ToggleNative();

public:
	SbaSqlNativeUndoAct(SqlEdit* pEdit,BOOL bNative=FALSE) : SbaSqlEditUndoAct(pEdit) , m_bNative(bNative) { }

	void SetNative(BOOL bNative) { m_bNative = bNative; }
};

#endif

