/*************************************************************************
 *
 *  $RCSfile: qrywinct.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:53 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBA_QRYWINCT_HXX
#define _SBA_QRYWINCT_HXX

#ifndef _SBA_TWCONT_HXX
#include "twcont.hxx"
#endif	// _SBA_TWCONT_HXX

#ifndef _SBA_QRYCONN_HXX
#include "qryconn.hxx"
#endif

//========================================================================
struct TabWinsChangeNotification
{
	enum ACTION_TYPE	{ AT_ADDED_WIN, AT_REMOVED_WIN };
	ACTION_TYPE			atActionPerformed;
	String				strAffectedTable;

	TabWinsChangeNotification(ACTION_TYPE at, const String& str) : atActionPerformed(at), strAffectedTable(str) { }
};

//========================================================================
class SbaJoinViewShExt;
class QueryTabWinUndoAct;
class QueryTabConnUndoAction;
class QueryTabConn;
class SbaQueryDocSh;
class QueryTabWin;
class QueryTabWinContainer : public SbaJoinTabWinCont
{
	Link	m_lnkTabWinsChangeHandler;

protected:
	virtual void ConnDoubleClicked(SbaJoinTabConn* pConnection);
	virtual void KeyInput(const KeyEvent& rEvt);

	SbaQueryDocSh* GetQueryDocShell();

	// wird nach Verschieben/Groessenaenderung der TabWins aufgerufen (die Standardimplementation reicht die neuen Daten einfach
	// an die Daten des Wins weiter)
	virtual void TabWinMoved(SbaJoinTabWin* pWhich, const Point& ptOldPosition);
	virtual void TabWinSized(SbaJoinTabWin* ptWhich, const Point& ptOldPosition, const Size& szOldSize);

public:
	TYPEINFO();
	QueryTabWinContainer(Window* pParent, SbaJoinViewShExt* pView);
	virtual ~QueryTabWinContainer();

	// Basisklasse ueberschrieben : Fenster kreieren und loeschen
	// (eigentlich nicht wirklich LOESCHEN, es geht in die Verantwortung einer UNDO-Action ueber)
	virtual void AddTabWin(const String& strDatabase, const String& strTableName, BOOL bNewTable = FALSE);
	virtual void RemoveTabWin(SbaJoinTabWin* pTabWin);

	// und ein AddTabWin, das einen Alias vorgibt
	void	AddTabWin(const String& strDatabase, const String& strTableName, const String& strAlias, BOOL bNewTable = FALSE);
	// Fenster an Hand des AliasNamen loeschen (zieht sich natuerlich auf voriges RemoveTabWin zurueck)
	BOOL	RemoveTabWin(const String& rAliasName);

	// TabWin suchen
	QueryTabWin*	FindTable(const String& rAliasName);
	BOOL			FindTableFromField(const String& rFieldName, TabFieldDesc& rInfo, USHORT& rCnt);

	// Basisklasse ueberschrieben : Connections kreieren und loeschen
	virtual void AddConnection(const SbaJoinExchangeData& jxdSource, const SbaJoinExchangeData& jxdDest);
	virtual BOOL RemoveConnection(SbaJoinTabConn* pConn);

	// Transfer von Connections von/zu einer UndoAction
	void GetConnection(QueryTabConn* pConn);
		// Einfuegen einer Connection in meine Struktur
	void DropConnection(QueryTabConn* pConn);
		// Entfernen einer Connection aus meiner Struktur

	// alle TabWins verstecken (NICHT loeschen, sie werden in eine Undo-Action gepackt)
	void	HideTabWins();
		// das resultiert effektiv in einem voelligen Leeren des Abfrageentwurfs, da alle Fenster versteckt werden, und dabei
		// natuerlich alle Connections an diesen Fenstern und alle Abfrage-Spalten, die auf diesen Tabellen basierten.

	// TabWin anzeigen oder verstecken (NICHT kreieren oder loeschen)
	BOOL	ShowTabWin(QueryTabWin* pTabWin, QueryTabWinUndoAct* pUndoAction);
	void	HideTabWin(QueryTabWin* pTabWin, QueryTabWinUndoAct* pUndoAction);

	// Sichbarkeit eines TabWins sicherstellen (+ Invalidieren der Connections)
	void	EnsureVisible(const SbaJoinTabWin* _pWin);

	// wieviel Tabellen mit einem bestimmten Namen habe ich schon ?
	USHORT	CountTableAlias(const String& rName, USHORT& rMax);

	// ein Feld einfuegen (wird einfach an das Elter weitergereicht
	void InsertField(const TabFieldDesc& rInfo);

	// alles (TabWins, Connections) neu aufbauen (PRECONDITION : vorher wurde ClearAll gerufen)
	virtual void ReSync();
	// alles (TabWins, Connections) loeschen, und zwar hart, es erfolgen also keinerlei Notifications
	virtual void ClearAll();

	// wird vom AddTabDlg benutzt, um festzustellen, ob noch Tabellen hinzugefuegt werden duerfen
	//virtual BOOL IsAddAllowed();

	// eine neu Connection bekanntgeben und einfuegen lassen, wenn nicht schon existent
	void NotifyTabConnection(const QueryTabConn& rNewConn, BOOL _bCreateUndoAction = TRUE);

	// fuer Droppen von Tabellen
	BOOL Drop(const DropEvent& rEvt);
	BOOL QueryDrop(DropEvent& rEvt);

	Link	SetTabWinsChangeHandler(const Link& lnk) { Link lnkRet = m_lnkTabWinsChangeHandler; m_lnkTabWinsChangeHandler = lnk; return lnkRet; }
		// der Handler bekommt einen Zeiger auf eine TabWinsChangeNotification-Struktur

	BOOL ExistsAVisitedConn(const QueryTabWin* pFrom) const;
};

#endif


