/*************************************************************************
 *
 *  $RCSfile: derivedimplbase.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBA_UTL_DERIVEDIMPLBASE_HXX
#define _SBA_UTL_DERIVEDIMPLBASE_HXX

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_HXX_
#include <com/sun/star/uno/Reference.hxx>
#endif
#ifndef _COM_SUN_STAR_UNO_SEQUENCE_HXX_
#include <com/sun/star/uno/Sequence.hxx>
#endif

#ifndef _COM_SUN_STAR_LANG_XTYPEPROVIDER_HPP_
#include <com/sun/star/lang/XTypeProvider.hpp>
#endif

#ifndef _CPPUHELPER_TYPEPROVIDER_HXX_
#include <cppuhelper/typeprovider.hxx>
#endif

#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif

#ifndef _UNOTOOLS_QUERYDEEPINTERFACE_HXX
#include <unotools/querydeep.hxx>
#endif

#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif

#ifndef _OSL_INTERLOCK_H_
#include <osl/interlck.h>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

namespace sbautl
{

//-------------------------------------------------------------------------------------------
struct TypeProviderInfo
{
// private: // cant do this, as it should be an aggregate
	sal_Int8 uuid[16];
	::osl::Mutex* pMutex;	// not owned - should be some static Mutex
	::com::sun::star::uno::Sequence< sal_Int8 > const* pIdSeq;
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type >* pTypes;
	::oslInterlockedCount nInstances; 
	::oslInterlockedCount nState; 

	void allocId() throw();
	void cleanup() throw();
public:
	// caching support
	void registerInstance() throw() 
	{ ::osl_incrementInterlockedCount(&nInstances); }

	void revokeInstance() throw() 
	{ if (::osl_decrementInterlockedCount(&nInstances) == 0) cleanup(); }

	oslInterlockedCount instanceCount() const throw() 
	{ return nInstances; }

	
	::osl::Mutex& getMutex() const throw() 
	{ OSL_ENSHURE(pMutex != 0, "TypeProviderInfo: no mutex provided"); return *pMutex; }

	// getTypes support
	sal_Bool hasTypes() const throw() 
	{ 
		OSL_ENSHURE(instanceCount() > 0,"TypeProviderInfo: Type collections supported only if instances exist"); 
		return pTypes != 0; 
	}

	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > getTypes() const throw() 
	{ 
		OSL_ENSHURE(instanceCount() > 0,"TypeProviderInfo: Type collections supported only if instances exist"); 
		OSL_ENSHURE(pTypes,"TypeProviderInfo: Forgot to createTypes ?"); 
		return *pTypes; 
	}

	void createTypes(::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > const& rTypes_) 
		throw(::com::sun::star::uno::RuntimeException); 

	// getImplementationId support
	::com::sun::star::uno::Sequence< sal_Int8 > getImplementationId() 
		throw(::com::sun::star::uno::RuntimeException); 

	// UnoTunnel support
	sal_Bool isImplementationId(::com::sun::star::uno::Sequence< sal_Int8 > const& rIdentifier_) 
		throw(); 
};


//-------------------------------------------------------------------------------------------
template <class Classname>
class TypeProviderImplHelper : public ::com::sun::star::lang::XTypeProvider
{
protected:
	static TypeProviderInfo	s_aInfo;
	static ::osl::Mutex		s_aMutex;

protected:
	TypeProviderImplHelper() throw()
	{ s_aInfo.registerInstance(); }

	TypeProviderImplHelper(TypeProviderImplHelper const&) throw() 
	{ s_aInfo.registerInstance(); }

	virtual ~TypeProviderImplHelper() throw() 
	{ s_aInfo.revokeInstance(); }

protected:
	Classname const* This() const throw() { return static_cast<Classname const*>(this); }
	Classname* This() throw() { return static_cast<Classname*>(this); }

	typedef TypeProviderImplHelper<Classname> TypeProviderHelper;

	// The parameter makes this unique per most derived class
	virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > 
		collectTypes(Classname* p) 
			throw(::com::sun::star::uno::RuntimeException) 
		= 0; 

public:
	// ::com::sun::star::uno::XInterface
	virtual ::com::sun::star::uno::Any 
		SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) 
			throw(::com::sun::star::uno::RuntimeException) 
	{
		return ::cppu::queryInterface(rType,
						static_cast< ::com::sun::star::uno::XInterface*>(this), 
						static_cast< ::com::sun::star::lang::XTypeProvider*>(this));
	}
	// ::com::sun::star::lang::XTypeProvider
	virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > 
		SAL_CALL getTypes() 
			throw(::com::sun::star::uno::RuntimeException) 
	{
		if (!s_aInfo.hasTypes())
			s_aInfo.createTypes(this->collectTypes(This()));
		return s_aInfo.getTypes();
	}
	
	virtual ::com::sun::star::uno::Sequence< sal_Int8 > 
		SAL_CALL getImplementationId() 
			throw(::com::sun::star::uno::RuntimeException) 
	{
		return s_aInfo.getImplementationId();
	}

	// static access 
	static ::com::sun::star::uno::Sequence< sal_Int8 > 
		getStaticImplementationId() 
			throw(::com::sun::star::uno::RuntimeException) 
	{
		return s_aInfo.getImplementationId();
	}

	// minimal support for implementing XUnoTunnel 
	static sal_Bool 
		isImplementationId(::com::sun::star::uno::Sequence< sal_Int8 > const& rIdentifier_) 
			throw(::com::sun::star::uno::RuntimeException) 
	{
		return s_aInfo.isImplementationId(rIdentifier_);
	}
};
template <class Classname>
::osl::Mutex TypeProviderImplHelper<Classname>::s_aMutex;
template <class Classname>
TypeProviderInfo TypeProviderImplHelper<Classname>::s_aInfo = { { 0 }, &s_aMutex, 0,0, 0,0 };


//-------------------------------------------------------------------------------------------
template <class Baseclass>
class DerivedImplHelperBase
{
protected:
	// XInterface support
	void acquireBase(Baseclass* pThis_)
		throw(::com::sun::star::uno::RuntimeException) 
	{ pThis_->Baseclass::acquire(); }

	void releaseBase(Baseclass* pThis_)
		throw(::com::sun::star::uno::RuntimeException) 
	{ pThis_->Baseclass::release(); }

	::com::sun::star::uno::Any queryBase(Baseclass* pThis_,::com::sun::star::uno::Type const& rType)
		throw(::com::sun::star::uno::RuntimeException) 
	{ return pThis_->Baseclass::queryInterface(rType); }

	// XTypeProvider support
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type> getBaseTypes(Baseclass* pThis_)
		throw(::com::sun::star::uno::RuntimeException)
	{ return pThis_->Baseclass::getTypes(); }
};

//-------------------------------------------------------------------------------------------
template <class Classname, class Baseclass>
class DerivedImplHelper 
:	public TypeProviderImplHelper<Classname>
,	public DerivedImplHelperBase<Baseclass> 
{
protected:
	typedef DerivedImplHelper<Classname,Baseclass> DerivationHelper;

public:
	// ::com::sun::star::uno::XInterface
	virtual void SAL_CALL acquire() throw(::com::sun::star::uno::RuntimeException) 
	{ acquireBase(This()); }

	virtual void SAL_CALL release() throw(::com::sun::star::uno::RuntimeException) 
	{ releaseBase(This()); }

	virtual ::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) throw(::com::sun::star::uno::RuntimeException) 
	{ 
		::com::sun::star::uno::Any aRet(TypeProviderHelper::queryInterface(rType));
		return aRet.hasValue() ? aRet : queryBase(This(),rType); 
	}
};
//-------------------------------------------------------------------------------------------

} // namespace sbautl
//-------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------
//-- MACRO WORLD ----------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------
#define SBA_UTL_IFC_TYPE(N) ::getCppuType(static_cast<const ::com::sun::star::uno::Reference< Ifc##N >*>(0))
#define SBA_UTL_IFC_TYPELIST1						  SBA_UTL_IFC_TYPE(1)
#define SBA_UTL_IFC_TYPELIST2 SBA_UTL_IFC_TYPELIST1 , SBA_UTL_IFC_TYPE(2)
#define SBA_UTL_IFC_TYPELIST3 SBA_UTL_IFC_TYPELIST2 , SBA_UTL_IFC_TYPE(3)
#define SBA_UTL_IFC_TYPELIST4 SBA_UTL_IFC_TYPELIST3 , SBA_UTL_IFC_TYPE(4)
#define SBA_UTL_IFC_TYPELIST5 SBA_UTL_IFC_TYPELIST4 , SBA_UTL_IFC_TYPE(5)
#define SBA_UTL_IFC_TYPELIST6 SBA_UTL_IFC_TYPELIST5 , SBA_UTL_IFC_TYPE(6)
#define SBA_UTL_IFC_TYPELIST7 SBA_UTL_IFC_TYPELIST6 , SBA_UTL_IFC_TYPE(7)
#define SBA_UTL_IFC_TYPELIST8 SBA_UTL_IFC_TYPELIST7 , SBA_UTL_IFC_TYPE(8)
#define SBA_UTL_IFC_TYPELIST9 SBA_UTL_IFC_TYPELIST8 , SBA_UTL_IFC_TYPE(9)

//-------------------------------------------------------------------------------------------
#define SBA_UTL_IFC_QUERYARG(N) static_cast< Ifc##N *>(this)
#define SBA_UTL_IFC_QUERYLIST1							SBA_UTL_IFC_QUERYARG(1)
#define SBA_UTL_IFC_QUERYLIST2 SBA_UTL_IFC_QUERYLIST1 , SBA_UTL_IFC_QUERYARG(2)
#define SBA_UTL_IFC_QUERYLIST3 SBA_UTL_IFC_QUERYLIST2 , SBA_UTL_IFC_QUERYARG(3)
#define SBA_UTL_IFC_QUERYLIST4 SBA_UTL_IFC_QUERYLIST3 , SBA_UTL_IFC_QUERYARG(4)
#define SBA_UTL_IFC_QUERYLIST5 SBA_UTL_IFC_QUERYLIST4 , SBA_UTL_IFC_QUERYARG(5)
#define SBA_UTL_IFC_QUERYLIST6 SBA_UTL_IFC_QUERYLIST5 , SBA_UTL_IFC_QUERYARG(6)
#define SBA_UTL_IFC_QUERYLIST7 SBA_UTL_IFC_QUERYLIST6 , SBA_UTL_IFC_QUERYARG(7)
#define SBA_UTL_IFC_QUERYLIST8 SBA_UTL_IFC_QUERYLIST7 , SBA_UTL_IFC_QUERYARG(8)
#define SBA_UTL_IFC_QUERYLIST9 SBA_UTL_IFC_QUERYLIST8 , SBA_UTL_IFC_QUERYARG(9)

//-------------------------------------------------------------------------------------------
#define SBA_UTL_DEF_DERIVED_IMPLHELPER( N ) \
	namespace sbautl {												\
		template <class Classname, class Baseclass, __CLASS_IFC##N>	\
		class DerivedImplHelper##N : __PUBLIC_IFC##N				\
		,	public TypeProviderImplHelper< Classname >			 	\
		,	public DerivedImplHelperBase< Baseclass >				\
		{															\
		protected:													\
			typedef DerivedImplHelper##N DerivationHelper##N;		\
			virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type >						\
				collectTypes(Classname*) throw(::com::sun::star::uno::RuntimeException)					\
			{ return ::cppu::OTypeCollection( SBA_UTL_IFC_TYPELIST##N, getBaseTypes(This())).getTypes(); }	\
		public:																										\
			virtual void SAL_CALL acquire() throw(::com::sun::star::uno::RuntimeException) { acquireBase(This()); }	\
			virtual void SAL_CALL release() throw(::com::sun::star::uno::RuntimeException) { releaseBase(This()); }	\
			virtual ::com::sun::star::uno::Any SAL_CALL queryInterface( const ::com::sun::star::uno::Type & rType )	\
				throw(::com::sun::star::uno::RuntimeException)									\
			{																					\
				::com::sun::star::uno::Any aRet( TypeProviderHelper::queryInterface(rType) );	\
				if (!aRet.hasValue())															\
					aRet = ::utl::queryDeepInterface( rType, SBA_UTL_IFC_QUERYLIST##N );		\
				return (aRet.hasValue() ? aRet : queryBase(This(),rType));						\
			}																					\
		};																						\
	} // namespace sbautl

// Macros to be used in the derived class
#define DECLARE_DERIVED_IMPL_NOQUERY(classname, implclass)	\
	virtual void SAL_CALL acquire() throw (::com::sun::star::uno::RuntimeException) \
	{ implclass::acquire(); } \
	virtual void SAL_CALL release() throw (::com::sun::star::uno::RuntimeException) \
	{ implclass::release(); } \
	virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes() \
		throw (::com::sun::star::uno::RuntimeException) \
	{ return implclass::getTypes(); } \
	virtual ::com::sun::star::uno::Sequence< sal_Int8 > SAL_CALL getImplementationId() \
		throw (::com::sun::star::uno::RuntimeException) \
	{ return implclass::getImplementationId(); } \
	static ::com::sun::star::uno::Sequence< sal_Int8 > getStaticImplementationId() \
		throw (::com::sun::star::uno::RuntimeException) \
	{ return implclass::getStaticImplementationId(); } \

#define DECLARE_DERIVED_IMPL(classname, implclass)	\
	DECLARE_DERIVED_IMPL_NOQUERY(classname, implclass)	\
	virtual ::com::sun::star::uno::Any SAL_CALL queryInterface(const ::com::sun::star::uno::Type& rType) \
		throw (::com::sun::star::uno::RuntimeException) \
	{ return implclass::queryInterface(rType); } \

#endif // _SBA_UTL_DERIVEDIMPLBASE_HXX

