/*************************************************************************
 *
 *  $RCSfile: derivedpropimpl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBA_UTL_DERIVEDPROPIMPL_HXX
#define _SBA_UTL_DERIVEDPROPIMPL_HXX

#ifndef _CPPUHELPER_PROPSHLP_HXX
#include <cppuhelper/propshlp.hxx>
#endif

#ifndef _SBA_UTL_DERIVEDIMPLBASE_HXX
#include "derivedimplbase.hxx"
#endif

namespace sbautl
{

//-------------------------------------------------------------------------------------------
struct PropertySetHelperInfo
{
// private: // cant do this, as it should be an aggregate
	::osl::Mutex* pMutex;	// not owned - should be some static Mutex
	::cppu::IPropertyArrayHelper* pArrayHelper;
	::com::sun::star::beans::XPropertySetInfo* pPropSetInfo;
	::oslInterlockedCount nInstances; 

	void cleanup() throw();
public:
	// caching support
	void registerInstance() throw() 
	{ ::osl_incrementInterlockedCount(&nInstances); }

	void revokeInstance() throw() 
	{ if (::osl_decrementInterlockedCount(&nInstances) == 0) cleanup(); }

	oslInterlockedCount instanceCount() const throw() 
	{ return nInstances; }

	::osl::Mutex& getMutex() const throw() 
	{ OSL_ENSHURE(pMutex != 0, "PropertySetHelperInfo: no mutex provided"); return *pMutex; }

	sal_Bool hasArrayHelper() const throw() 
	{
		OSL_ENSHURE(instanceCount() > 0,"PropertySetHelperInfo: Array Helper supported only if instances exist"); 
		return pArrayHelper != 0;
	}

	::cppu::IPropertyArrayHelper& getArrayHelper() throw()
	{
		OSL_ENSHURE(instanceCount() > 0,"PropertySetHelperInfo: Array Helper supported only if instances exist"); 
		OSL_ENSHURE(pArrayHelper,"PropertySetHelperInfo: Forgot to createArrayHelper ?"); 
		return *pArrayHelper;
	}

	void createArrayHelper(::com::sun::star::uno::Sequence< ::com::sun::star::beans::Property > const & rProperties) 
		throw(::com::sun::star::uno::RuntimeException);

	::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySetInfo > getPropertySetInfo() 
		throw(::com::sun::star::uno::RuntimeException);
};

//-------------------------------------------------------------------------------------------
template <class Classname>
class PropertySetHelperBase 
:	public TypeProviderImplHelper<Classname>
,	public ::cppu::OPropertySetHelper
{
protected:
	static PropertySetHelperInfo s_aPropInfo;
protected:
	typedef PropertySetHelperBase<Classname> PropertySetHelper;

	PropertySetHelperBase(::cppu::OBroadcastHelper & rBHelper)
		: OPropertySetHelper(rBHelper)
	{ s_aPropInfo.registerInstance(); }

	~PropertySetHelperBase()
	{ s_aPropInfo.revokeInstance(); }

	// support for implementing TypeProviderImplHelper::collectTypes
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > 
		addPropertyTypes(::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > const& baseTypes) 
			throw(::com::sun::star::uno::RuntimeException)
	{
		return::cppu::OTypeCollection(
							::getCppuType(static_cast< ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > const*>(0)),
							::getCppuType(static_cast< ::com::sun::star::uno::Reference< ::com::sun::star::beans::XMultiPropertySet > const*>(0)),
							::getCppuType(static_cast< ::com::sun::star::uno::Reference< ::com::sun::star::beans::XFastPropertySet > const*>(0)),
							baseTypes
						).getTypes();
	}

	// support for implementing OPropertySetHelper::getInfoHelper
	virtual void fillProperties(
		::com::sun::star::uno::Sequence< ::com::sun::star::beans::Property >& /* [out] */ rProperties_
		) const 
		throw(::com::sun::star::uno::RuntimeException) = 0;

	void createArrayHelper() throw(::com::sun::star::uno::RuntimeException)
	{
		::com::sun::star::uno::Sequence< ::com::sun::star::beans::Property > aProperties;
		fillProperties( aProperties );
		s_aPropInfo.createArrayHelper(aProperties);
	}

	// cppu::OPropertySetHelper
	virtual ::cppu::IPropertyArrayHelper& SAL_CALL getInfoHelper()
	{
		if (!s_aPropInfo.hasArrayHelper())
			createArrayHelper( );
		return s_aPropInfo.getArrayHelper();
	}

public:
	// ::com::sun::star::uno::XInterface
	virtual ::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) 
		throw(::com::sun::star::uno::RuntimeException) 
	{
		::com::sun::star::uno::Any aRet(TypeProviderHelper::queryInterface(rType));
		return aRet.hasValue() ? aRet : OPropertySetHelper::queryInterface(rType);
	}

	// ::com::sun::star::beans::XPropertySet
	virtual ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySetInfo > SAL_CALL getPropertySetInfo() 
		throw(::com::sun::star::uno::RuntimeException)
	{
		if (!s_aPropInfo.hasArrayHelper())
			createArrayHelper( );
		return s_aPropInfo.getPropertySetInfo();
	}

};
template <class Classname>
PropertySetHelperInfo PropertySetHelperBase<Classname>::s_aPropInfo = { &s_aMutex, 0,0, 0 };

//-------------------------------------------------------------------------------------------
template <class Classname, class Baseclass>
class DerivedPropertySetHelper 
:	public PropertySetHelperBase<Classname>
,	public DerivedImplHelperBase<Baseclass>
{ 
protected:
	typedef DerivedPropertySetHelper<Classname,Baseclass> PropertySetHelper;

	DerivedPropertySetHelper(::cppu::OBroadcastHelper & rBHelper)
		: PropertySetHelperBase<Classname>(rBHelper)
	{}

	// TypeProviderImplHelper::collectTypes
	virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type >
		collectTypes(Classname*) 
			throw(::com::sun::star::uno::RuntimeException) 
	{
		return addPropertyTypes(getBaseTypes(This()));
	}
public:
	virtual void SAL_CALL acquire() throw(::com::sun::star::uno::RuntimeException) 
	{ acquireBase(This()); }

	virtual void SAL_CALL release() throw(::com::sun::star::uno::RuntimeException) 
	{ releaseBase(This()); }

	virtual ::com::sun::star::uno::Any SAL_CALL queryInterface(::com::sun::star::uno::Type const& rType) 
		throw(::com::sun::star::uno::RuntimeException) 
	{ 
		::com::sun::star::uno::Any aRet(PropertySetHelperBase<Classname>::queryInterface(rType));
		return aRet.hasValue() ? aRet : queryBase(This(),rType); 
	}
};
//-------------------------------------------------------------------------------------------

} // namespace sbautl

//-------------------------------------------------------------------------------------------

#define SBA_UTL_DEF_DERIVED_PROP_IMPLHELPER( N ) \
	namespace sbautl {													\
		template <class Classname, class Baseclass, __CLASS_IFC##N>	\
		class DerivedPropertySetHelper##N : __PUBLIC_IFC##N			\
		,	public PropertySetHelperBase<Classname>					\
		,	public DerivedImplHelperBase<Baseclass>					\
		{															\
		protected:													\
			typedef DerivedPropertySetHelper##N PropertySetHelper;													\
			DerivedPropertySetHelper##N(::cppu::OBroadcastHelper & rBHelper)										\
				: PropertySetHelperBase<Classname>(rBHelper) {}														\
			virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > collectTypes(Classname*)			\
				throw(::com::sun::star::uno::RuntimeException)														\
			{																										\
				return ::cppu::OTypeCollection( SBA_UTL_IFC_TYPELIST##N,											\
												addPropertyTypes(getBaseTypes(This()))).getTypes();					\
			}																										\
		public:																										\
			virtual void SAL_CALL acquire() throw(::com::sun::star::uno::RuntimeException) { acquireBase(This()); }	\
			virtual void SAL_CALL release() throw(::com::sun::star::uno::RuntimeException) { releaseBase(This()); }	\
			virtual ::com::sun::star::uno::Any SAL_CALL queryInterface( const ::com::sun::star::uno::Type & rType ) \
				throw(::com::sun::star::uno::RuntimeException)														\
			{																										\
				::com::sun::star::uno::Any aRet( PropertySetHelperBase<Classname>::queryInterface(rType) );			\
				if (!aRet.hasValue())																				\
					aRet = ::utl::queryDeepInterface( rType, SBA_UTL_IFC_QUERYLIST##N );							\
				return (aRet.hasValue() ? aRet : queryBase(This(),rType));											\
			}																										\
		};																											\
	} // namespace sbautl
//-------------------------------------------------------------------------------------------

#endif // _SBA_UTL_DERIVEDPROPIMPL_HXX

