/*************************************************************************
 *
 *  $RCSfile: tabconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SBA_TABCONN_HXX
#define _SBA_TABCONN_HXX

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif

#ifndef _SVLBOX_HXX //autogen
#include <svtools/svlbox.hxx>
#endif

#ifndef _SBA_SBADB_HXX
#include "sbadb.hxx"
#endif

#ifndef _SBA_DBTABOBJ_HXX
#include "dbtabobj.hxx"
#endif

//==================================================================
// ConnData		---------->*	ConnLineData
//    ^1							^1
//    |								|
//	Conn		---------->*	ConnLine
//==================================================================


//==================================================================
enum JoinTabConnSide { JTCS_FROM, JTCS_TO };

class SbaJoinTabConn;
class SbaJoinConnLineData
{
	String aSourceFieldName;
	String aDestFieldName;

public:
	SbaJoinConnLineData();
	SbaJoinConnLineData( const String& rSourceFieldName, const String& rDestFieldName );
	SbaJoinConnLineData( const SbaJoinConnLineData& rConnLineData );
	virtual ~SbaJoinConnLineData();

	// eine Kopie der eigenen Instanz liefern (das ist mir irgendwie angenehmer als ein virtueller Zuweisungsoperator)
	virtual void CopyFrom(const SbaJoinConnLineData& rSource);

	// Memberzugriff (schreiben)
	void SetFieldName(JoinTabConnSide nWhich, const String& strFieldName) { if (nWhich==JTCS_FROM) aSourceFieldName = strFieldName; else aDestFieldName = strFieldName; }
	void SetSourceFieldName( const String& rSourceFieldName ){ SetFieldName(JTCS_FROM, rSourceFieldName); }
	void SetDestFieldName( const String& rDestFieldName ){ SetFieldName(JTCS_TO, rDestFieldName); }

	// Memberzugriff (lesen)
	String GetFieldName(JoinTabConnSide nWhich) const { return (nWhich == JTCS_FROM) ? aSourceFieldName : aDestFieldName; }
	String GetSourceFieldName() const { return GetFieldName(JTCS_FROM); }
	String GetDestFieldName() const { return GetFieldName(JTCS_TO); }

	virtual BOOL IsValid();
	virtual void Reset();
	virtual SbaJoinConnLineData& operator=( const SbaJoinConnLineData& rConnLineData );
};

DECLARE_LIST( SbaJoinConnLineDataList, SbaJoinConnLineData* );

//==================================================================
class SbaJoinTabConn;

class SbaJoinConnLine
{
	SbaJoinTabConn* pTabConn;
	SbaJoinConnLineData* pData;

	SvLBoxEntry* pSourceEntry;
	SvLBoxEntry* pDestEntry;

	Point aSourceConnPos, aDestConnPos;
	Point aSourceDescrLinePos, aDestDescrLinePos;

public:
	SbaJoinConnLine( SbaJoinTabConn* pConn, const String& rSourceFieldName = String(),
					 const String& rDestFieldName = String() );
	SbaJoinConnLine( SbaJoinTabConn* pConn, SbaJoinConnLineData* pLineData );
	SbaJoinConnLine( const SbaJoinConnLine& rLine );
	virtual ~SbaJoinConnLine();

	virtual SbaJoinConnLine& operator=( const SbaJoinConnLine& rLine );

	Rectangle			GetBoundingRect();
	BOOL				RecalcLine();
	void				Draw( OutputDevice* pOutDev );
	BOOL				CheckHit( const Point& rMousePos );
	String				GetSourceFieldName() const { return pData->GetSourceFieldName(); }
	String				GetDestFieldName() const { return pData->GetDestFieldName(); }

	void				SetSourceFieldName( const String& rSourceFieldName );
	void				SetDestFieldName( const String& rDestFieldName );
	BOOL				Connect( const String& rSourceFieldName, const String& rDestFieldName );
	BOOL				IsValid();

	Rectangle			GetSourceTextPos();
	Rectangle			GetDestTextPos();

	SbaJoinConnLineData*	GetData() const { return pData; }

};

DECLARE_LIST( SbaJoinConnLineList, SbaJoinConnLine* );

//==================================================================
class SbaJoinTabConnData
{
protected:
	String aSourceWinName;
	String aDestWinName;
	String aConnName;

	SbaJoinConnLineDataList aConnLineDataList;

	void	Init();
	void	Init(const String& rSourceWinName, const String& rDestWinName, const String& rConnName = String() );
		// Das erste Init baut darauf, dass die 3 String-Members schon korrekt gesetzt sind und aConnLineDataList leer ist.
		// Das zweite stellt genau diesen Zustand her

	virtual SbaJoinConnLineData* CreateLineDataObj();
	virtual SbaJoinConnLineData* CreateLineDataObj( const SbaJoinConnLineData& rConnLineData );

public:
	TYPEINFO();
	SbaJoinTabConnData();
	SbaJoinTabConnData( const String& rSourceWinName, const String& rDestWinName, const String& rConnName = String() );
	SbaJoinTabConnData( const SbaJoinTabConnData& rConnData );
	virtual ~SbaJoinTabConnData();

	// sich aus einer Quelle initialisieren (das ist mir irgendwie angenehmer als ein virtueller Zuweisungsoperator)
	virtual void CopyFrom(const SbaJoinTabConnData& rSource);

	// eine neue Instanz meines eigenen Typs liefern (braucht NICHT initialisiert sein)
	virtual SbaJoinTabConnData* NewInstance() const { return new SbaJoinTabConnData(); }
		// (von SbaJoinTabConnData abgeleitete Klasse muessen entsprechend eine Instanz ihrer Klasse liefern)

	SbaJoinTabConnData& operator=( const SbaJoinTabConnData& rConnData );

	BOOL SetConnLine( USHORT nIndex, const String& rSourceFieldName, const String& rDestFieldName );
	BOOL AppendConnLine( const String& rSourceFieldName, const String& rDestFieldName );
	void ResetConnLines( BOOL bUseDefaults = TRUE );
		// loescht die Liste der ConnLines, bei bUseDefaults == TRUE werden danach MAX_CONN_COUNT neue Dummy-Linien eingefuegt

	SbaJoinConnLineDataList* GetConnLineDataList(){ return &aConnLineDataList; }

	String GetSourceWinName() const { return aSourceWinName; }
	String GetDestWinName() const { return aDestWinName; }
	String GetConnName() const { return aConnName; }

	virtual void SetSourceWinName( const String& rSourceWinName ){ aSourceWinName = rSourceWinName; }
	virtual void SetDestWinName( const String& rDestWinName ){ aDestWinName = rDestWinName; }
	virtual void SetConnName( const String& rConnName ){ aConnName = rConnName; }
};

DECLARE_LIST( SbaJoinTabConnDataList, SbaJoinTabConnData* );

//==================================================================
const USHORT MAX_CONN_COUNT		= 2;

class SbaJoinTabWin;
class SbaJoinTabWinCont;

class SbaJoinTabConn
{
	SbaJoinConnLineList aConnLineList;
	SbaJoinTabConnData* pData;
	SbaJoinTabWinCont*	pParent;

	BOOL bSelected;

	void Init();

protected:
	SbaJoinConnLine* CreateConnLine( const SbaJoinConnLine& rConnLine );

public:
	TYPEINFO();
	SbaJoinTabConn( SbaJoinTabWinCont* pContainer, SbaJoinTabConnData* pTabConnData );
	SbaJoinTabConn( const SbaJoinTabConn& rConn );
		// WICHTIG : normalerweise bekomme ich von aussen einen Zeiger auf SbaJoinTabConnData mitgegeben, hier aber muss ich
		// mir (ueber SbaJoinTabConnData::NewInstance) selber eine INstanz anlegen, die ich aber - wie in allen anderen Faellen auch -
		// NIE loesche. Der Aufrufer ist also bei Benutzung dieses Constructors dafuer verantwortlich, meine Daten abzufragen und
		// sich irgendwo zu merken, um sie dann irgendwann zu loeschen.
	virtual ~SbaJoinTabConn();

	SbaJoinTabConn& operator=( const SbaJoinTabConn& rConn );

	void		Select();
	void		Deselect();
	BOOL		IsSelected() const { return bSelected; }
	BOOL		CheckHit( const Point& rMousePos );
	void		Invalidate();
	void		UpdateLineList();

	SbaJoinTabWin* GetSourceWin();
	SbaJoinTabWin* GetDestWin();

	void RecalcLines();

	Rectangle	GetBoundingRect();

	SbaJoinTabConnData*		GetData() const { return pData; }
	SbaJoinConnLineList*	GetConnLineList() const { return &((SbaJoinTabConn*)this)->aConnLineList; }
	SbaJoinTabWinCont*		GetParent(){ return pParent; }
	virtual void			Draw( const Rectangle& rRect );
};

DECLARE_LIST( SbaJoinTabConnList, SbaJoinTabConn* );

#endif


