/*************************************************************************
 *
 *  $RCSfile: dlgindex.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:55 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/* =======================================================================
Beschreibung:

Der Dialog sucht sich alle .dbf-Dateien aus dem DBase-Verzeichnis.
und fuellt damit die Liste aTableInfoList. Jedes TableInfo-Object
enthaelt den Namen der Tabelle und eine Liste aller Indizes (aIndexList),
die die Indizes aus der .inf-Datei repraesentiert. Indizes werden durch
TableIndex-Objekte "gewrappt".
Die FreeIndexList enthaelt alle Indizes, die an keine Tabelle gebunden sind.
======================================================================= */


#ifndef _SBA_SBARES_HXX
#include "sbares.hxx"
#endif
#ifndef SBA_DLGINDEX_HRC
#include "dlgindex.hrc"
#endif
#ifndef _DLGINDEXES_HXX
#include "dlgindex.hxx"
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _CONFIG_HXX //autogen
#include <vcl/config.hxx>
#endif

#ifndef _SFXAPP_HXX //autogen
#include <sfx2/app.hxx>
#endif

#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif

const ByteString aGroupIdent("dbase III");

//////////////////////////////////////////////////////////////////////////
// Klasse DlgIndexes
DlgIndexes::DlgIndexes( Window * pParent, String aDataSrcName )
	: ModalDialog( pParent, SbaResId(DLG_INDEXES) ),
	aPB_OK(				this, ResId( PB_OK ) ),
	aPB_CANCEL(			this, ResId( PB_CANCEL ) ),
	aPB_HELP(			this, ResId( PB_HELP ) ),
	m_FT_Tables(		this, ResId( FT_TABLES ) ),
	aCB_Tables(			this, ResId( CB_TABLES ) ),
	m_GB_Indexes(		this, ResId( GB_INDEXES ) ),
	m_FT_TableIndexes(	this, ResId( FT_TABLEINDEXES ) ),
	aLB_TableIndexes(	this, ResId( LB_TABLEINDEXES ) ),
	m_FT_AllIndexes(	this, ResId( FT_ALLINDEXES ) ),
	aLB_FreeIndexes(	this, ResId( LB_FREEINDEXES ) ),
	aPB_Add(			this, ResId( PB_ADD ) ),
	aPB_Remove(			this, ResId( PB_REMOVE ) ),
	aPB_AddAll(			this, ResId( PB_ADDALL ) ),
	aPB_RemoveAll(		this, ResId( PB_REMOVEALL ) ),
	aDSN(aDataSrcName)
{
	aCB_Tables.SetSelectHdl( LINK(this, DlgIndexes, TableSelectHdl) );
	aPB_Add.SetClickHdl( LINK(this, DlgIndexes, AddClickHdl) );
	aPB_Remove.SetClickHdl( LINK(this, DlgIndexes, RemoveClickHdl) );
	aPB_AddAll.SetClickHdl( LINK(this, DlgIndexes, AddAllClickHdl) );
	aPB_RemoveAll.SetClickHdl( LINK(this, DlgIndexes, RemoveAllClickHdl) );
	aPB_OK.SetClickHdl( LINK(this, DlgIndexes, OKClickHdl) );

	Init();
	SetCtrls();
	FreeResource();
}

DlgIndexes::~DlgIndexes()
{
	sal_Int32 i;
	for(i=0;i<aTableInfoList.Count();i++)
		delete aTableInfoList.GetObject(i);
	aTableInfoList.Clear();

	for(i=0;i<aFreeIndexList.Count();i++)
		delete aFreeIndexList.GetObject(i);
	aFreeIndexList.Clear();
}

TableInfo* DlgIndexes::GetTable( const String& rName )
{
	TableInfo* pTabInfo = aTableInfoList.First();
	while( pTabInfo )
	{
		if (m_bCaseSensitiv)
		{
			if (pTabInfo->aTableName.Equals(rName))
				break;
		}
		else
		{
			if (pTabInfo->aTableName.EqualsIgnoreCaseAscii(rName))
				break;
		}

		pTabInfo = aTableInfoList.Next();
	}
	return pTabInfo;


}

TableIndex* DlgIndexes::RemoveFreeIndex( const String& rName )
{
	TableIndex* pIndex = aFreeIndexList.First();
	sal_uInt32 nPos;
	while( pIndex )
	{
		if ( m_bCaseSensitiv ? pIndex->GetIndexFileName().Equals(rName) : pIndex->GetIndexFileName().EqualsIgnoreCaseAscii(rName) )
		{
			nPos = aFreeIndexList.GetPos( pIndex );
			aFreeIndexList.Remove( pIndex );
			aLB_FreeIndexes.RemoveEntry( rName );

			// ::com::sun::star::awt::Selection setzen
			if( nPos == aFreeIndexList.Count() )
				aLB_FreeIndexes.SelectEntryPos( (sal_uInt16)nPos-1 );
			else
				aLB_FreeIndexes.SelectEntryPos( (sal_uInt16)nPos );
			return pIndex;
		}
		pIndex = aFreeIndexList.Next();
	}
	return NULL;

}

void DlgIndexes::InsertFreeIndex( TableIndex* pIndex )
{
	if( !pIndex )
		return;
	aFreeIndexList.Insert( pIndex );
	aLB_FreeIndexes.InsertEntry( pIndex->GetIndexFileName() );
	aLB_FreeIndexes.SelectEntryPos(0);
}

TableIndex* DlgIndexes::RemoveTableIndex( const String& rTableName, const String& rIndexName )
{
	TableInfo* pTabInfo = GetTable( rTableName );
	if( !pTabInfo )
		return NULL;
	TableIndex* pIndex = pTabInfo->aIndexList.First();
	sal_uInt32 nPos;
	while( pIndex )
	{
		if (m_bCaseSensitiv ? pIndex->GetIndexFileName().Equals(rIndexName) : pIndex->GetIndexFileName().EqualsIgnoreCaseAscii(rIndexName) )
		{
			nPos = pTabInfo->aIndexList.GetPos( pIndex );
			pTabInfo->aIndexList.Remove( pIndex );
			aLB_TableIndexes.RemoveEntry( rIndexName );

			// ::com::sun::star::awt::Selection setzen
			if( nPos==pTabInfo->aIndexList.Count() )
				aLB_TableIndexes.SelectEntryPos( (sal_uInt16)nPos-1 );
			else
				aLB_TableIndexes.SelectEntryPos( (sal_uInt16)nPos );

			return pIndex;
		}
		pIndex = pTabInfo->aIndexList.Next();
	}
	return NULL;

}

void DlgIndexes::InsertTableIndex( const String& rTableName, TableIndex* pIndex )
{
	if( !pIndex )
		return;
	TableInfo* pTabInfo = GetTable( rTableName );
	pTabInfo->aIndexList.Insert( pIndex );
	aLB_TableIndexes.InsertEntry( pIndex->GetIndexFileName() );
	aLB_TableIndexes.SelectEntryPos(0);
}

IMPL_LINK( DlgIndexes, OKClickHdl, PushButton*, pButton )
{
	// Alle Tabellen schreiben ihr INF-File
	TableInfo* pTabInfo = aTableInfoList.First();

	while( pTabInfo )
	{
		pTabInfo->WriteInfFile( aDSN );
		pTabInfo = aTableInfoList.Next();
	}

	EndDialog();

	return 0;
}

IMPL_LINK( DlgIndexes, AddClickHdl, PushButton*, pButton )
{
	String aSelection = aLB_FreeIndexes.GetSelectEntry();
	String aTableName = aCB_Tables.GetText();
	TableIndex* pIndex = RemoveFreeIndex( aSelection );
	InsertTableIndex( aTableName, pIndex );
	return 0;
}

IMPL_LINK( DlgIndexes, RemoveClickHdl, PushButton*, pButton )
{
	String aSelection = aLB_TableIndexes.GetSelectEntry();
	String aTableName = aCB_Tables.GetText();
	TableIndex* pIndex = RemoveTableIndex( aTableName, aSelection );
	InsertFreeIndex( pIndex );

	return 0;
}

IMPL_LINK( DlgIndexes, AddAllClickHdl, PushButton*, pButton )
{
	sal_uInt16 nCnt = aLB_FreeIndexes.GetEntryCount();
	String aTableName = aCB_Tables.GetText();
	String aEntry;
	TableIndex* pIndex;

	for( sal_uInt16 nPos = 0; nPos < nCnt; nPos++ )
	{
		aEntry = aLB_FreeIndexes.GetEntry(0);
		pIndex = RemoveFreeIndex( aEntry );
		InsertTableIndex( aTableName, pIndex );
	}

	return 0;
}

IMPL_LINK( DlgIndexes, RemoveAllClickHdl, PushButton*, pButton )
{
	sal_uInt16 nCnt = aLB_TableIndexes.GetEntryCount();
	String aTableName = aCB_Tables.GetText();
	String aEntry;
	TableIndex* pIndex;

	for( sal_uInt16 nPos = 0; nPos < nCnt; nPos++ )
	{
		aEntry = aLB_TableIndexes.GetEntry(0);
		pIndex = RemoveTableIndex( aTableName, aEntry );
		InsertFreeIndex( pIndex );
	}

	return 0;
}

IMPL_LINK( DlgIndexes, TableSelectHdl, ComboBox*, pComboBox )
{
	// Richtige Tabelle suchen
	TableInfo* pTabInfo = aTableInfoList.First();
	while( pTabInfo )
	{
		if ( m_bCaseSensitiv ? pTabInfo->aTableName.Equals( pComboBox->GetText() ): pTabInfo->aTableName.EqualsIgnoreCaseAscii( pComboBox->GetText() ) )
		{
			// ListBox der Tabellenindizes aufbauen
			aLB_TableIndexes.Clear();
			TableIndex* pIndex = pTabInfo->aIndexList.First();
			while( pIndex )
			{
				aLB_TableIndexes.InsertEntry( pIndex->GetIndexFileName() );
				pIndex = pTabInfo->aIndexList.Next();
			}
			if( pTabInfo->aIndexList.Count() )
				aLB_TableIndexes.SelectEntryPos(0);

			break;
		}
		pTabInfo = aTableInfoList.Next();
	}

	return 0;
}

void DlgIndexes::Init()
{
	///////////////////////////////////////////////////////////////////////////
	// Alle Indizes werden erst einmal zur Liste der freien Indizes hinzugefuegt.
	// Dann wird fuer jede Tabelle in der Inf-Datei nachgeschaut, welche Indizes sie besitzt.
	// Diese Indizes werden aus der Liste der freien Indizes entfernt
	// und in die Indexliste der Tabelle eingetragen

	///////////////////////////////////////////////////////////////////////////
	// $(INST) im Pfad ersetzen
	aDSN = SFX_APP()->GetIniManager()->SubstPathVars( aDSN );

	///////////////////////////////////////////////////////////////////////////
	// Wenn der String keinen Verzeichnispfad enthaelt, String kuerzen
	DirEntry aInitEntry( aDSN );
	FileStat aFileStat(aInitEntry); 	// nicht zusammenfassen - Bug im VC++ 2.0!

	if (aFileStat.IsKind(FSYS_KIND_WILD))
		aDSN = (DirEntry(aDSN).GetPath()).GetFull();
	else if (aFileStat.IsKind(FSYS_KIND_DIR))
		aDSN = DirEntry(aDSN).GetFull();
	else if (aFileStat.IsKind(FSYS_KIND_FILE))
		aDSN = (DirEntry(aDSN).GetPath()).GetFull();

	///////////////////////////////////////////////////////////////////////////
	// Alle Indizes erst einmal in die Liste der freien Indizes eintragen
	DirEntry aDirEntry( aDSN );

	m_bCaseSensitiv = aDirEntry.IsCaseSensitive();

	aDirEntry += String::CreateFromAscii("*.ndx");
	Dir* pDir = new Dir( aDirEntry, FSYS_KIND_FILE,FSYS_SORT_NAME | FSYS_SORT_ASCENDING | FSYS_SORT_END );

	sal_uInt16 nCount = pDir->Count();
	String aEntry;

	for( sal_uInt16 nCurPos = 0; nCurPos < nCount; nCurPos++ )
	{
		aEntry = (*pDir)[nCurPos].GetName();
		TableIndex* pIndex = new TableIndex(aEntry);
		aFreeIndexList.Insert( pIndex );
	}

	///////////////////////////////////////////////////////////////////////////
	// Jetzt die Liste der Tabellen fuellen
	aDirEntry.SetExtension(String::CreateFromAscii("dbf"));
	delete pDir;
	pDir = new Dir( aDirEntry, FSYS_KIND_FILE,FSYS_SORT_NAME | FSYS_SORT_ASCENDING | FSYS_SORT_END );

	nCount = pDir->Count();
	if(!nCount)
	{
		aPB_OK.Disable(sal_True);
		m_GB_Indexes.Disable(sal_True);
		m_FT_TableIndexes.Disable(sal_True);
		aLB_TableIndexes.Disable(sal_True);
		m_FT_AllIndexes.Disable(sal_True);
		aLB_FreeIndexes.Disable(sal_True);
		aPB_Add.Disable(sal_True);
		aPB_Remove.Disable(sal_True);
		aPB_AddAll.Disable(sal_True);
		aPB_RemoveAll.Disable(sal_True);
	}

	for( nCurPos = 0; nCurPos < nCount; nCurPos++ )
	{
		// Tabelle in Liste einfuegen
		aEntry = (*pDir)[nCurPos].GetName();
		TableInfo* pTabInfo = new TableInfo(aEntry);
		aTableInfoList.Insert( pTabInfo );

		// INF-Datei oeffnen
		String aTableName = (*pDir)[nCurPos].GetBase();
		String aFileName = aDSN;
		aFileName += DirEntry::GetAccessDelimiter();
		aFileName += aTableName;
		aFileName.AppendAscii(".inf");
		Config aInfFile( aFileName );
		aInfFile.SetGroup( aGroupIdent );

		///////////////////////////////////////////////////////////////////////////
		// Liste Tabellenindizes fuellen
		ByteString aNDX;
		sal_uInt16 nKeyCnt = aInfFile.GetKeyCount();
		ByteString aKeyName;

		for( sal_uInt16 nKey = 0; nKey < nKeyCnt; nKey++ )
		{
			// Verweist der Key auf ein Indexfile?...
			aKeyName = aInfFile.GetKeyName( nKey );
			aNDX = aKeyName.Copy(0,3);

			//...wenn ja, Indexliste der Tabelle hinzufuegen
			if (aNDX == "NDX" )
			{
				aEntry = String(aInfFile.ReadKey(aKeyName), gsl_getSystemTextEncoding());
				TableIndex* pIndex = new TableIndex( aEntry );
				pTabInfo->aIndexList.Insert( pIndex );

				// Index aus der freien Indexliste entfernen
				RemoveFreeIndex( aEntry );
			}

		} // end for Indizes

	} // end for Tabellen
	delete pDir;

}

void DlgIndexes::SetCtrls()
{
	// ComboBox Tabellen
	TableInfo* pTabInfo = aTableInfoList.First();
	while( pTabInfo )
	{
		aCB_Tables.InsertEntry( pTabInfo->aTableName );
		pTabInfo = aTableInfoList.Next();
	}

	// Den ersten Datensatz ins Edit stellen
	if( aTableInfoList.Count() )
	{
		TableInfo* pTabInfo = aTableInfoList.GetObject(0);
		aCB_Tables.SetText( pTabInfo->aTableName );

		// ListBox der Tabellenindizes aufbauen
		TableIndex* pIndex = pTabInfo->aIndexList.First();
		while( pIndex )
		{
			aLB_TableIndexes.InsertEntry( pIndex->GetIndexFileName() );
			pIndex = pTabInfo->aIndexList.Next();
		}
		if( pTabInfo->aIndexList.Count() )
			aLB_TableIndexes.SelectEntryPos( 0 );

	}

	// ListBox freie Indizes
	TableIndex* pIndex = aFreeIndexList.First();
	while( pIndex )
	{
		aLB_FreeIndexes.InsertEntry( pIndex->GetIndexFileName() );
		pIndex = aFreeIndexList.Next();
	}

	if( aFreeIndexList.Count() )
		aLB_FreeIndexes.SelectEntryPos( 0 );

}

//////////////////////////////////////////////////////////////////////////
// Klasse TableInfo
void TableInfo::WriteInfFile( const String& rDSN )
{
	// INF-Datei oeffnen
	xub_StrLen nLen = aTableName.Len();
	String aName = aTableName.Copy( 0,(nLen-4) ); // Extension ausblenden
	String aFileName = rDSN;
	aFileName += '\\';
	aFileName += aName;
	aFileName.AppendAscii(".inf");
	Config aInfFile( aFileName );
	aInfFile.SetGroup( aGroupIdent );

	// Erst einmal alle Tabellenindizes loeschen
	ByteString aNDX;
	sal_uInt16 nKeyCnt = aInfFile.GetKeyCount();
	ByteString aKeyName;
	ByteString aEntry;
	sal_uInt16 nKey = 0;

	while( nKey < nKeyCnt )
	{
		// Verweist der Key auf ein Indexfile?...
		aKeyName = aInfFile.GetKeyName( nKey );
		aNDX = aKeyName.Copy(0,3);

		//...wenn ja, Indexfile loeschen, nKey steht dann auf nachfolgendem Key
		if( aNDX == "NDX" )
		{
			aInfFile.DeleteKey(aKeyName);
			nKeyCnt--;
		}
		else
			nKey++;

	}

	// Jetzt alle gespeicherten Indizes hinzufuegen
	sal_uInt16 nPos = 0;
	TableIndex* pIndex = aIndexList.First();

	while( pIndex )
	{
		aKeyName = "NDX";
		if( nPos > 0 )	// Erster Index erhaelt keine Ziffer
			aKeyName += ByteString::CreateFromInt32( nPos );
		aInfFile.WriteKey( aKeyName, ByteString(pIndex->GetIndexFileName(), gsl_getSystemTextEncoding()) );
		pIndex = aIndexList.Next();
		nPos++;
	}

	aInfFile.Flush();

	// Falls nur noch [dbase] in INF-File steht, Datei loeschen
	DirEntry aDirEntry( aFileName );
	FileStat aFileStat( aDirEntry );
	sal_uInt32 nFileSize = aFileStat.GetSize();
	xub_StrLen nGroupIdLen = aGroupIdent.Len();

	if( (xub_StrLen)nFileSize == (nGroupIdLen+4) )
		aDirEntry.Kill();
}

