/*************************************************************************
 *
 *  $RCSfile: dlgsave.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:56 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _DLGSAVE_HXX
#include "dlgsave.hxx"
#endif
#ifndef _SBARESID_HRC
#include "sbaresid.hrc"
#endif
#ifndef _SBA_SBARES_HXX
#include "sbares.hxx"
#endif
#ifndef _DLGSAVE_HRC
#include "dlgsave.hrc"
#endif
#ifndef _SBA_SBAOBJ_HXX
#include "sbaobj.hxx"
#endif
#ifndef _SBA_DBTABOBJ_HXX
#include "dbtabobj.hxx"
#endif
#ifndef _SBA_DBQRYOBJ_HXX
#include "dbqryobj.hxx"
#endif

#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif

//==================================================================
SbaSaveAsDlg::SbaSaveAsDlg( Window * pParent,
							SbaDBDefRef _xObj,
							const String& rDefault)
			 :ModalDialog( pParent, SbaResId(DLG_SAVE_AS))
			 ,aLabel(this, ResId (FT_TITLE))
			 ,aTitle(this, ResId (ET_TITLE),_xObj->GetDatabase())
			 ,aPB_OK(this, ResId( PB_OK ) )
			 ,aPB_CANCEL(this, ResId( PB_CANCEL ))
			 ,aPB_HELP(this, ResId( PB_HELP))
			 ,aTblLabel(ResId(STR_TBL_LABEL))
			 ,aViewLabel(ResId(STR_VW_LABEL))
			 ,aQryLabel(ResId(STR_QRY_LABEL))
			 ,aFrmLabel(ResId(STR_FRM_LABEL))
			 ,aRptLabel(ResId(STR_RPT_LABEL))
			 ,xObj(_xObj)
			 ,aName(rDefault)
{
	switch (xObj->GetKind())
	{
		case dbQuery:
			aLabel.SetText(aQryLabel);
			aTitle.EnableCharCheck(sal_False);
			break;
		case dbView:
			aLabel.SetText(aViewLabel);
			break;
		case dbForm:
			// the name will be used as stream name within an storage, so limit it's length
			// FS - 68097 - 22.10.99
			aTitle.SetMaxTextLen(32);
			aLabel.SetText(aFrmLabel);
			aTitle.EnableCharCheck(sal_False);
			break;
		case dbReport:
			// the name will be used as stream name within an storage, so limit it's length
			// FS - 68097 - 22.10.99
			aTitle.SetMaxTextLen(32);
			aLabel.SetText(aRptLabel);
			aTitle.EnableCharCheck(sal_False);
			break;
		case dbTable:
			aLabel.SetText(aTblLabel);
			break;
	}

	aTitle.SetText(aName);
	aPB_OK.SetClickHdl(LINK(this,SbaSaveAsDlg,ButtonClickHdl));
	aTitle.SetModifyHdl(LINK(this,SbaSaveAsDlg,EditModifyHdl));
	FreeResource();
}

IMPL_LINK(SbaSaveAsDlg, ButtonClickHdl, Button *, pButton)
{
	if (pButton == &aPB_OK)
	{
		String strNewName = aTitle.GetText();
		// der Objektname ...
		String sObjectName = xObj->Name();
		// ... mit einer Sonderbehandlung fuer Tabellen : xObj liefert mir hier den voll qualifizierten Namen, ich brauche aber
		// nur den eigentlichen Tabellennamen ohne Owner und Schema
		if (xObj->ISA(SbaTableDef))
			sObjectName = ((SbaTableDef*)(SbaDBDef*)xObj)->GetTable()->Name();

		// Tabellen werden direkt in der DB gespeichert, und da haben diese bei der Namensvergabe noch ein Woertchen mitzureden
		if (xObj->GetKind() == dbTable)
		{
			sal_Bool bValidName = xObj->GetDatabase()->IsSQLNameValid(strNewName);
			if (!bValidName)
			{
				ErrorBox(this, SbaResId(ERROR_INVALID_TABLE_NAME)).Execute();
				aTitle.GrabFocus();
				return 0;
			}
		}

		// neues Objekt, Name hat sich nicht verndert
		if (xObj->IsNew() && sObjectName == strNewName)
			EndDialog(RET_OK);
		else if (!xObj->GetDatabase()->HasObject(xObj->GetKind(), strNewName)  )
		{
			aName = strNewName;
			EndDialog(RET_OK);
		}
		else
		{
			SdbStatus aStatus;
			aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,xObj->GetKind(), strNewName);
			SBA_MOD()->ShowDbStatus(aStatus, dbWriteError);
			aTitle.GrabFocus();
		}
	}
	return 0;
}

IMPL_LINK(SbaSaveAsDlg, EditModifyHdl, Edit *, pEdit )
{
	if (pEdit == &aTitle)
		aPB_OK.Enable(aTitle.GetText().Len());
	return 0;
}

