/*************************************************************************
 *
 *  $RCSfile: tabconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:57 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "math.h"
#include "sbaresid.hrc"

#ifndef _WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif

#ifndef _SV_OUTDEV_HXX //autogen
#include <vcl/outdev.hxx>
#endif

#ifndef _SV_POLY_HXX //autogen
#include <vcl/poly.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#include "sbares.hxx"
#include "tabwin.hxx"
#include "tabconn.hxx"
#include "twcont.hxx"
#include "joinds.hxx"
#include "sbaobj.hxx"

#ifdef UNX
#define _cdecl
#define	_CDECLARE_
#endif

#ifdef MAC
#define _cdecl
#endif

const UINT16 QUERY_UI_TABWIN_TAG = 0x0001;
const UINT16 QUERY_UI_TABFIELD_TAG = 0x0002;

const long DESCRIPT_LINE_WIDTH = 15;
const long HIT_SENSITIVE_RADIUS = 5;


TYPEINIT0(SbaJoinTabConn);
TYPEINIT0(SbaJoinTabConnData);

//==================================================================
//class SbaJoinConnLineData
//==================================================================

//------------------------------------------------------------------------
SbaJoinConnLineData::SbaJoinConnLineData()
{
}

//------------------------------------------------------------------------
SbaJoinConnLineData::SbaJoinConnLineData( const String& rSourceFieldName, const String& rDestFieldName )
	:aSourceFieldName( rSourceFieldName )
	,aDestFieldName( rDestFieldName )
{
}

//------------------------------------------------------------------------
SbaJoinConnLineData::SbaJoinConnLineData( const SbaJoinConnLineData& rConnLineData )
{
	*this = rConnLineData;
}

//------------------------------------------------------------------------
SbaJoinConnLineData::~SbaJoinConnLineData()
{
}

//------------------------------------------------------------------------
void SbaJoinConnLineData::CopyFrom(const SbaJoinConnLineData& rSource)
{
	*this = rSource;
	// hier ziehe ich mich auf das (nicht-virtuelle) operator= zurueck, das nur meine Members kopiert
}

//------------------------------------------------------------------------
BOOL SbaJoinConnLineData::IsValid()
{
	if( GetSourceFieldName().Len() && GetDestFieldName().Len() )
		return TRUE;

	return FALSE;
}

//------------------------------------------------------------------------
SbaJoinConnLineData& SbaJoinConnLineData::operator=( const SbaJoinConnLineData& rConnLineData )
{
	if (&rConnLineData == this)
		return *this;

	aSourceFieldName = rConnLineData.GetSourceFieldName();
	aDestFieldName = rConnLineData.GetDestFieldName();

	return *this;
}

//------------------------------------------------------------------------
void SbaJoinConnLineData::Reset()
{
	aSourceFieldName = String();
	aDestFieldName = String();
}


//========================================================================
// class SbaJoinConnLine
//========================================================================

//------------------------------------------------------------------------
SbaJoinConnLine::SbaJoinConnLine( SbaJoinTabConn* pConn, SbaJoinConnLineData* pLineData )
	: pTabConn( pConn )
	 ,pData( pLineData )
	 ,pSourceEntry( NULL )
	 ,pDestEntry( NULL )
{
}

//------------------------------------------------------------------------
SbaJoinConnLine::SbaJoinConnLine( SbaJoinTabConn* pConn, const String& rSourceFieldName,
								  const String& rDestFieldName )
	: pTabConn( pConn )
	 ,pSourceEntry( NULL )
	 ,pDestEntry( NULL )
{
	pData->SetSourceFieldName( rSourceFieldName );
	pData->SetDestFieldName( rDestFieldName );
}

//------------------------------------------------------------------------
SbaJoinConnLine::SbaJoinConnLine( const SbaJoinConnLine& rLine )
{
	pData = new SbaJoinConnLineData( *rLine.GetData() );
	*this = rLine;
}

//------------------------------------------------------------------------
SbaJoinConnLine::~SbaJoinConnLine()
{
}

//------------------------------------------------------------------------
void SbaJoinConnLine::SetSourceFieldName( const String& rSourceFieldName )
{
	pData->SetSourceFieldName( rSourceFieldName );
//	pSourceEntry = pTabConn->GetSourceWin()->GetListBox()->GetEntryFromText( rSourceFieldName );
}

//------------------------------------------------------------------------
void SbaJoinConnLine::SetDestFieldName( const String& rDestFieldName )
{
	pData->SetDestFieldName( rDestFieldName );
//	pDestEntry = pTabConn->GetDestWin()->GetListBox()->GetEntryFromText( rDestFieldName );
}

//------------------------------------------------------------------------
SbaJoinConnLine& SbaJoinConnLine::operator=( const SbaJoinConnLine& rLine )
{
	if( &rLine == this )
		return *this;

	// da mir die Daten nicht gehoeren, loesche ich die alten nicht
	pData->CopyFrom(*rLine.GetData());
		// CopyFrom ist virtuell, damit ist es kein Problem, wenn pData von einem von SbaJoinTabConnData abgeleiteten Typ ist

	pTabConn = rLine.pTabConn;
	pSourceEntry = rLine.pSourceEntry;
	pDestEntry = rLine.pDestEntry;
	aSourceConnPos = rLine.aSourceConnPos;
	aDestConnPos = rLine.aDestConnPos;
	aSourceDescrLinePos = rLine.aSourceDescrLinePos;
	aDestDescrLinePos = rLine.aDestDescrLinePos;

	return *this;
}

//------------------------------------------------------------------------
BOOL SbaJoinConnLine::Connect( const String& rSourceFieldName, const String& rDestFieldName )
{
	//////////////////////////////////////////////////////////////////////
	// Parameter duerfen nicht leer sein
	if( !rSourceFieldName.Len() || !rDestFieldName.Len() )
		return FALSE;

	//////////////////////////////////////////////////////////////////////
	// Feldnamen setzen
	pData->SetSourceFieldName( rSourceFieldName );
	pData->SetDestFieldName( rDestFieldName );

	//////////////////////////////////////////////////////////////////////
	// Entries setzen
/*	pSourceEntry = pTabConn->GetSourceWin()->GetListBox()->GetEntryFromText( rSourceFieldName );
	if( !pSourceEntry )
		return FALSE;
	pDestEntry = pTabConn->GetDestWin()->GetListBox()->GetEntryFromText( rDestFieldName );
	if( !pDestEntry )
		return FALSE;
*/

	return TRUE;
}


//------------------------------------------------------------------------
Rectangle SbaJoinConnLine::GetBoundingRect()
{
	//////////////////////////////////////////////////////////////////////
	// Umgebendes Rechteck bestimmen
	Rectangle aBoundingRect( Point(0,0), Point(0,0) );
	if( !IsValid() )
		return aBoundingRect;

	Point aTopLeft;
	Point aBottomRight;

	if( aSourceDescrLinePos.Y() <= aDestDescrLinePos.Y() )
	{
		aTopLeft.Y() = aSourceDescrLinePos.Y();
		aBottomRight.Y() = aDestDescrLinePos.Y();
	}
	else
	{
		aTopLeft.Y() = aDestDescrLinePos.Y();
		aBottomRight.Y() = aSourceDescrLinePos.Y();
	}

	if( aSourceDescrLinePos.X() <= aDestDescrLinePos.X() )
	{
		aTopLeft.X() = aSourceDescrLinePos.X();
		aBottomRight.X() = aDestDescrLinePos.X();
	}
	else
	{
		aTopLeft.X() = aDestDescrLinePos.X();
		aBottomRight.X() = aSourceDescrLinePos.X();
	}

	//////////////////////////////////////////////////////////////////////
	// Linie verlaeuft in z-Form
	if( Abs(aSourceConnPos.X()-aDestConnPos.X()) > Abs(aSourceDescrLinePos.X()-aDestDescrLinePos.X()) )
	{
		aTopLeft.X() -= DESCRIPT_LINE_WIDTH;
		aBottomRight.X() += DESCRIPT_LINE_WIDTH;
	}

	aBoundingRect = Rectangle( aTopLeft-Point(0,17), aBottomRight+Point(0,2) );

	return aBoundingRect;
}

//------------------------------------------------------------------------
BOOL SbaJoinConnLine::RecalcLine()
{
	//////////////////////////////////////////////////////////////////////
	// Fenster und Entries muessen gesetzt sein
	const SbaJoinTabWin* pSourceWin = pTabConn->GetSourceWin();
	const SbaJoinTabWin* pDestWin = pTabConn->GetDestWin();

	if( !pSourceWin || !pDestWin )
		return FALSE;

	pSourceEntry = pSourceWin->GetListBox()->GetEntryFromText( GetData()->GetSourceFieldName() );
	pDestEntry = pDestWin->GetListBox()->GetEntryFromText( GetData()->GetDestFieldName() );

	if( !pSourceEntry || !pDestEntry )
		return FALSE;

	//////////////////////////////////////////////////////////////////////
	// X-Koordinaten bestimmen
	Point aSourceCenter( 0, 0 );
	Point aDestCenter( 0, 0 );

	aSourceCenter.X() = pSourceWin->GetPosPixel().X() + (long)( 0.5*pSourceWin->GetSizePixel().Width() );
	aDestCenter.X() = pDestWin->GetPosPixel().X() + (long)( 0.5*pDestWin->GetSizePixel().Width() );
	if( aDestCenter.X()>aSourceCenter.X() )
	{
		//////////////////////////////////////////////////////////////////////
		// DestWin liegt rechts vom SourceWin
		aSourceConnPos.X() = pSourceWin->GetPosPixel().X()+pSourceWin->GetSizePixel().Width();
		aSourceDescrLinePos.X() = aSourceConnPos.X();
		aSourceConnPos.X() += DESCRIPT_LINE_WIDTH;

		aDestConnPos.X() = pDestWin->GetPosPixel().X();
		aDestDescrLinePos.X() = aDestConnPos.X();
		aDestConnPos.X() -= DESCRIPT_LINE_WIDTH;
	}
	else
	{
		//////////////////////////////////////////////////////////////////////
		// DestWin liegt links vom SourceWin
		aSourceConnPos.X() = pSourceWin->GetPosPixel().X();
		aSourceDescrLinePos.X() = aSourceConnPos.X();
		aSourceConnPos.X() -= DESCRIPT_LINE_WIDTH;

		aDestConnPos.X() = pDestWin->GetPosPixel().X()+pDestWin->GetSizePixel().Width();
		aDestDescrLinePos.X() = aDestConnPos.X();
		aDestConnPos.X() += DESCRIPT_LINE_WIDTH;
	}

	//////////////////////////////////////////////////////////////////////
	// aSourceConnPosY bestimmen
	SbaJoinTabWinListBox* pListBox = pSourceWin->GetListBox();
	long nRowHeight = pListBox->GetEntryHeight();

	aSourceConnPos.Y() = pSourceWin->GetPosPixel().Y();
	aSourceConnPos.Y() += pListBox->GetPosPixel().Y();
	long nEntryPos = pListBox->GetEntryPos( pSourceEntry ).Y();

	if( nEntryPos >= 0 )
	{
		aSourceConnPos.Y() += nEntryPos;
		aSourceConnPos.Y() += (long)( 0.5 * nRowHeight );
	}
	else
		aSourceConnPos.Y() -= (long)( 0.5 * nRowHeight );

	long nListBoxBottom = pSourceWin->GetPosPixel().Y()
							+pListBox->GetPosPixel().Y()
							+pListBox->GetSizePixel().Height();
	if( aSourceConnPos.Y() > nListBoxBottom )
		aSourceConnPos.Y() = nListBoxBottom + 2;

	aSourceDescrLinePos.Y() = aSourceConnPos.Y();

	//////////////////////////////////////////////////////////////////////
	// aDestConnPosY bestimmen
	pListBox = pDestWin->GetListBox();

	aDestConnPos.Y() = pDestWin->GetPosPixel().Y();
	aDestConnPos.Y() += pListBox->GetPosPixel().Y();
	nEntryPos = pListBox->GetEntryPos( pDestEntry ).Y();
	if( nEntryPos >= 0 )
	{
		aDestConnPos.Y() += nEntryPos;
		aDestConnPos.Y() += (long)( 0.5 * nRowHeight );
	}
	else
		aDestConnPos.Y() -= (long)( 0.5 * nRowHeight );

	nListBoxBottom = pDestWin->GetPosPixel().Y()
						+pListBox->GetPosPixel().Y()
						+pListBox->GetSizePixel().Height();
	if( aDestConnPos.Y() > nListBoxBottom )
		aDestConnPos.Y() = nListBoxBottom + 2;

	aDestDescrLinePos.Y() = aDestConnPos.Y();

	return TRUE;
}

//------------------------------------------------------------------------
Rectangle SbaJoinConnLine::GetSourceTextPos()
{
	const SbaJoinTabWin* pDestWin = pTabConn->GetDestWin();
	SbaJoinTabWinListBox* pListBox = pDestWin ? pDestWin->GetListBox() : NULL;
	DBG_ASSERT(pDestWin && pListBox, "SbaJoinConnLine::GetSourceTextPos : invalid call !");

	long nRowHeight = pListBox->GetEntryHeight();

	Rectangle aReturn;
	aReturn.Top() = aSourceConnPos.Y() - nRowHeight;
	aReturn.Bottom() = aReturn.Top() + nRowHeight;
	if (aSourceDescrLinePos.X() < aSourceConnPos.X())
	{
		aReturn.Left() = aSourceDescrLinePos.X();
		aReturn.Right() = aReturn.Left() + aSourceConnPos.X() - aSourceDescrLinePos.X();
	}
	else
	{
		aReturn.Left() = aSourceConnPos.X();
		aReturn.Right() = aReturn.Left() + aSourceDescrLinePos.X() - aSourceConnPos.X();
	}

	return aReturn;
}

//------------------------------------------------------------------------
Rectangle SbaJoinConnLine::GetDestTextPos()
{
	const SbaJoinTabWin* pSourceWin = pTabConn->GetSourceWin();
	SbaJoinTabWinListBox* pListBox = pSourceWin ? pSourceWin->GetListBox() : NULL;
	DBG_ASSERT(pSourceWin && pListBox, "SbaJoinConnLine::GetSourceTextPos : invalid call !");

	long nRowHeight = pListBox->GetEntryHeight();

	Rectangle aReturn;
	aReturn.Top() = aDestConnPos.Y() - nRowHeight;
	aReturn.Bottom() = aReturn.Top() + nRowHeight;
	if (aDestDescrLinePos.X() < aDestConnPos.X())
	{
		aReturn.Left() = aDestDescrLinePos.X();
		aReturn.Right() = aReturn.Left() + aDestConnPos.X() - aDestDescrLinePos.X();
	}
	else
	{
		aReturn.Left() = aDestConnPos.X();
		aReturn.Right() = aReturn.Left() + aDestDescrLinePos.X() - aDestConnPos.X();
	}

	return aReturn;
}

//------------------------------------------------------------------------
void SbaJoinConnLine::Draw( OutputDevice* pOutDev )
{
	const UINT16 nRectSize = 3;

	//////////////////////////////////////////////////////////////////////
	// Neue Dimensionen berechnen
	if( !RecalcLine() )
		return;

	//////////////////////////////////////////////////////////////////////
	// Zeichnen der Linien
	if (pTabConn->IsSelected())
		pOutDev->SetLineColor(Application::GetSettings().GetStyleSettings().GetHighlightColor());
	else
		pOutDev->SetLineColor(Application::GetSettings().GetStyleSettings().GetWindowTextColor());

	pOutDev->DrawLine( aSourceDescrLinePos, aSourceConnPos );
	pOutDev->DrawLine( aDestDescrLinePos, aDestConnPos );
	pOutDev->DrawLine( aSourceConnPos, aDestConnPos );
	// wenn die  Linie selektiert ist, sollte sie dicker erscheinen
	// da OutputDevice nach meinem Wissen (das ich nur aus dem Headerfile habe) kein
	// SetLineWidth (o..) hat, ein Fake
	if (pTabConn->IsSelected())
	{
		UINT16 xOffset, yOffset;
		if (abs(aSourceConnPos.Y() - aDestConnPos.Y()) > abs(aSourceConnPos.X()-aDestConnPos.X()))
		{
			xOffset = 1;
			yOffset = 0;
		} else
		{
			xOffset = 0;
			yOffset = 1;
		}
		pOutDev->DrawLine(aSourceConnPos + Point(-xOffset, -yOffset), aDestConnPos + Point(-xOffset, -yOffset));
		pOutDev->DrawLine(aSourceConnPos + Point(xOffset, yOffset), aDestConnPos + Point(xOffset, yOffset));
	}


	//////////////////////////////////////////////////////////////////////
	// Zeichnen der Verbindungs-Rechtecke
//	pOutDev->SetLineColor(Application::GetSettings().GetStyleSettings().GetWindowTextColor());
	Rectangle aSourceRect( aSourceDescrLinePos-Point(nRectSize,nRectSize),
						   aSourceDescrLinePos+Point(nRectSize,nRectSize) );

	Rectangle aDestRect( aDestDescrLinePos-Point(nRectSize,nRectSize),
						 aDestDescrLinePos+Point(nRectSize,nRectSize) );
	pOutDev->SetFillColor(Application::GetSettings().GetStyleSettings().GetWindowColor());
	pOutDev->DrawRect( aSourceRect );
	pOutDev->DrawRect( aDestRect );
}

//------------------------------------------------------------------------
BOOL SbaJoinConnLine::IsValid()
{
	return pData->IsValid();
}
//------------------------------------------------------------------------
double dist_Euklid(const Point &p1, const Point& p2,const Point& pM, Point& q)
{
	int vx = p2.X() - p1.X();
	int vy = p2.Y() - p1.Y();
	Point v(p2 - p1);
	Point w(pM - p1);
	double a = sqrt((double)(v.X()*v.X() + v.Y()*v.Y()));
	double l = (v.X() * w.Y() - v.Y() * w.X()) / a;
	double a2 = w.X()*v.X()+w.Y()*v.Y();
	a = a2 / (a * a);
	q.X() = (long)p1.X() + a * v.X(); q.Y() = (long)p1.Y() + a * v.Y();
	return l;
}
//------------------------------------------------------------------------
BOOL SbaJoinConnLine::CheckHit( const Point& rMousePos )
{
	//////////////////////////////////////////////////////////////////////
	/*
		Vorgehensweise beim HitTest:
		Es wird der Abstand nach Euklid berechnet.
	*/
	Point q;
	double l = fabs(dist_Euklid(aSourceConnPos,aDestConnPos,rMousePos,q));
	if( l < HIT_SENSITIVE_RADIUS)
	{
		if(min(aSourceConnPos.X(),aDestConnPos.X()) <= q.X() && min(aSourceConnPos.Y(),aDestConnPos.Y()) <= q.Y()
			&& q.X() <= max(aDestConnPos.X(),aSourceConnPos.X())   && q.Y() <= max(aDestConnPos.Y(),aSourceConnPos.Y()))
			return TRUE;
	}

	return FALSE;
}


//==================================================================
// class SbaJoinTabConnData
//==================================================================
DBG_NAME(SbaJoinTabConnData);
//------------------------------------------------------------------------
SbaJoinTabConnData::SbaJoinTabConnData()
{
	DBG_CTOR(SbaJoinTabConnData,NULL);
	Init();
}

//------------------------------------------------------------------------
SbaJoinTabConnData::SbaJoinTabConnData( const String& rSourceWinName, const String& rDestWinName, const String& rConnName )
	:aSourceWinName( rSourceWinName )
	,aDestWinName( rDestWinName )
	,aConnName( rConnName )
{
	DBG_CTOR(SbaJoinTabConnData,NULL);
	Init();
}

//------------------------------------------------------------------------
void SbaJoinTabConnData::Init()
{
	//////////////////////////////////////////////////////////////////////
	// LineDataList mit Defaults initialisieren
	DBG_ASSERT(GetConnLineDataList()->Count() == 0, "SbaJoinTabConnData::Init() : nur mit leere Linienliste aufzurufen !");
	ResetConnLines(TRUE);
		// das legt Defaults an
}

//------------------------------------------------------------------------
void SbaJoinTabConnData::Init(const String& rSourceWinName, const String& rDestWinName, const String& rConnName)
{
	// erst mal alle LineDatas loeschen
	SbaJoinConnLineData* pLineData = GetConnLineDataList()->First();
	while (pLineData)
	{
		delete pLineData;
		pLineData = GetConnLineDataList()->Next();
	}
	GetConnLineDataList()->Clear();

	// dann die Strings
	aSourceWinName = rSourceWinName;
	aDestWinName = rDestWinName;
	aConnName = rConnName;

	// den Rest erledigt das andere Init
	Init();
}

//------------------------------------------------------------------------
SbaJoinTabConnData::SbaJoinTabConnData( const SbaJoinTabConnData& rConnData )
{
	DBG_CTOR(SbaJoinTabConnData,NULL);
	*this = rConnData;
}

//------------------------------------------------------------------------
void SbaJoinTabConnData::CopyFrom(const SbaJoinTabConnData& rSource)
{
	*this = rSource;
	// hier ziehe ich mich auf das (nicht-virtuelle) operator= zurueck, das nur meine Members kopiert
}

//------------------------------------------------------------------------
SbaJoinTabConnData::~SbaJoinTabConnData()
{
	// LineDataList loeschen
	ResetConnLines(FALSE);
	DBG_DTOR(SbaJoinTabConnData,NULL);
}

//------------------------------------------------------------------------
SbaJoinTabConnData& SbaJoinTabConnData::operator=( const SbaJoinTabConnData& rConnData )
{
	if (&rConnData == this)
		return *this;

	aSourceWinName = rConnData.GetSourceWinName();
	aDestWinName = rConnData.GetDestWinName();
	aConnName = rConnData.GetConnName();

/*	//////////////////////////////////////////////////////////////////////
	// Linienliste loeschen
	SbaJoinConnLineData* pDestConnLineData = GetConnLineDataList()->First();
	while (pDestConnLineData)
	{
		pDestConnLineData->Reset();
		pDestConnLineData = GetConnLineDataList()->Next();
	}

	//////////////////////////////////////////////////////////////////////
	// Linienliste kopieren
	SbaJoinConnLineData* pSourceConnLineData = ((SbaJoinTabConnData*)&rConnData)->GetConnLineDataList()->First();
	pDestConnLineData = GetConnLineDataList()->First();
	while( pSourceConnLineData )
	{
//		pDestConnLineData = CreateLineDataObj( *pSourceConnLineData );
//		GetConnLineDataList()->Insert( pDestConnLineData, nIndex );
		*pDestConnLineData = *pSourceConnLineData;
		pSourceConnLineData = ((SbaJoinTabConnData*)&rConnData)->GetConnLineDataList()->Next();
		pDestConnLineData = GetConnLineDataList()->Next();
	}

//	for( USHORT i=nIndex; i<MAX_CONN_COUNT; i++ )
//		aConnLineDataList.Insert( new SbaJoinConnLineData() );
*/

	// Linienliste loeschen
	ResetConnLines(FALSE);

	// und kopieren
	SbaJoinConnLineDataList* pDestLines = ((SbaJoinTabConnData*)&rConnData)->GetConnLineDataList();
	SbaJoinConnLineData* pLoop = pDestLines->First();
	while (pLoop)
	{
		GetConnLineDataList()->Insert(new SbaJoinConnLineData(*pLoop), LIST_APPEND);
		pLoop = pDestLines->Next();
	}


	return *this;
}

//------------------------------------------------------------------------
BOOL SbaJoinTabConnData::SetConnLine( USHORT nIndex, const String& rSourceFieldName, const String& rDestFieldName )
{
	if (USHORT(GetConnLineDataList()->Count()) < nIndex)
		return FALSE;
		// == ist noch erlaubt, das entspricht einem Append

	if (GetConnLineDataList()->Count() == nIndex)
		return AppendConnLine(rSourceFieldName, rDestFieldName);

	SbaJoinConnLineData* pConnLineData = aConnLineDataList.GetObject(nIndex);
	DBG_ASSERT(pConnLineData != NULL, "SbaJoinTabConnData::SetConnLine : habe ungueltiges LineData-Objekt");

	pConnLineData->SetSourceFieldName( rSourceFieldName );
	pConnLineData->SetDestFieldName( rDestFieldName );

	return TRUE;
}

//------------------------------------------------------------------------
BOOL SbaJoinTabConnData::AppendConnLine( const String& rSourceFieldName, const String& rDestFieldName )
{
	SbaJoinConnLineData* pNew = new SbaJoinConnLineData(rSourceFieldName, rDestFieldName);
	if (!pNew)
		return FALSE;

	GetConnLineDataList()->Insert(pNew, LIST_APPEND);
	return TRUE;
}

//------------------------------------------------------------------------
void SbaJoinTabConnData::ResetConnLines( BOOL bUseDefaults )
{
	for (USHORT i=0; i<GetConnLineDataList()->Count(); ++i)
		delete GetConnLineDataList()->GetObject(i);
	GetConnLineDataList()->Clear();

	if (bUseDefaults)
	{
		for (USHORT i=0; i<MAX_CONN_COUNT; i++)
			aConnLineDataList.Insert( new SbaJoinConnLineData(), LIST_APPEND );
	}
}

//------------------------------------------------------------------------
SbaJoinConnLineData* SbaJoinTabConnData::CreateLineDataObj()
{
	return new SbaJoinConnLineData();
}

//------------------------------------------------------------------------
SbaJoinConnLineData* SbaJoinTabConnData::CreateLineDataObj( const SbaJoinConnLineData& rConnLineData )
{
	return new SbaJoinConnLineData( rConnLineData );
}

//========================================================================
// class SbaJoinTabConn
//========================================================================
DBG_NAME(SbaJoinTabConn);
//------------------------------------------------------------------------
SbaJoinTabConn::SbaJoinTabConn( SbaJoinTabWinCont* pContainer, SbaJoinTabConnData* pTabConnData )
	:bSelected( FALSE )
	,pParent( pContainer )
	,pData( pTabConnData )
{
	DBG_CTOR(SbaJoinTabConn,NULL);
	Init();
}

//------------------------------------------------------------------------
SbaJoinTabConn::SbaJoinTabConn( const SbaJoinTabConn& rConn )
{
	DBG_CTOR(SbaJoinTabConn,NULL);
	pData = rConn.GetData()->NewInstance();
	*this = rConn;
}

//------------------------------------------------------------------------
SbaJoinTabConn::~SbaJoinTabConn()
{
	//////////////////////////////////////////////////////////////////////
	// Linienliste loeschen
	SbaJoinConnLine* pConnLine;
	while( pConnLine = aConnLineList.Remove((ULONG)0) )
		delete pConnLine;
	DBG_DTOR(SbaJoinTabConn,NULL);
}

//------------------------------------------------------------------------
void SbaJoinTabConn::Init()
{
	//////////////////////////////////////////////////////////////////////
	// Linienliste mit Defaults initialisieren
	SbaJoinConnLineDataList* pLinesDatas = GetData()->GetConnLineDataList();

	for ( USHORT i=0; i<pLinesDatas->Count(); ++i )
	{
		SbaJoinConnLine* pConnLine = new SbaJoinConnLine(this, pLinesDatas->GetObject(i));
		aConnLineList.Insert( pConnLine, LIST_APPEND );
	}
}

//------------------------------------------------------------------------
SbaJoinConnLine* SbaJoinTabConn::CreateConnLine( const SbaJoinConnLine& rConnLine )
{
	return new SbaJoinConnLine( rConnLine );
}
//------------------------------------------------------------------------
void SbaJoinTabConn::UpdateLineList()
{
	//////////////////////////////////////////////////////////////////////
	// Linienliste loeschen
	SbaJoinConnLine* pConnLine;
	while( pConnLine = aConnLineList.Remove((ULONG)0) )
		delete pConnLine;

	//////////////////////////////////////////////////////////////////////
	// Linienliste aus der Datenliste neu erstellen
	SbaJoinConnLineData* pConnLineData = GetData()->GetConnLineDataList()->First();
	USHORT nIndex = 0;
	while( pConnLineData )
	{
		pConnLine = new SbaJoinConnLine( this, pConnLineData );
		aConnLineList.Insert( pConnLine, nIndex );
		pConnLineData = GetData()->GetConnLineDataList()->Next();
		nIndex++;
	}

}

//------------------------------------------------------------------------
SbaJoinTabConn& SbaJoinTabConn::operator=( const SbaJoinTabConn& rConn )
{
	if( &rConn == this )
		return *this;

	// Linienliste loeschen
	if( aConnLineList.Count() )
	{
		SbaJoinConnLine* pConnLine;
		while( pConnLine = aConnLineList.Remove((ULONG)0) )
			delete pConnLine;
	}

	// Linienliste kopieren
	if( rConn.GetConnLineList()->Count() )
	{
		SbaJoinConnLine* pDestLine;
		SbaJoinConnLine* pSourceLine = rConn.GetConnLineList()->First();
		USHORT nIndex = 0;
		while( pSourceLine )
		{
			pDestLine = CreateConnLine( *pSourceLine );
			aConnLineList.Insert( pDestLine, nIndex );

			pSourceLine = rConn.GetConnLineList()->Next();
			nIndex++;
		}
	}

	// da mir die Daten nicht gehoeren, loesche ich die alten nicht
	pData->CopyFrom(*rConn.GetData());
		// CopyFrom ist virtuell, damit ist es kein Problem, wenn pData von einem von SbaJoinTabConnData abgeleiteten Typ ist

	bSelected = rConn.bSelected;
	pParent = rConn.pParent;

	return *this;
}


//------------------------------------------------------------------------
void SbaJoinTabConn::RecalcLines()
{
	SbaJoinConnLine* pConnLine = aConnLineList.First();
	while (pConnLine)
	{
		pConnLine->RecalcLine();
		pConnLine = aConnLineList.Next();
	}
}

//------------------------------------------------------------------------
SbaJoinTabWin* SbaJoinTabConn::GetSourceWin()
{
	return pParent->GetWindow( GetData()->GetSourceWinName() );
}

//------------------------------------------------------------------------
SbaJoinTabWin* SbaJoinTabConn::GetDestWin()
{
	return pParent->GetWindow( GetData()->GetDestWinName() );
}

//------------------------------------------------------------------------
void SbaJoinTabConn::Select()
{
	bSelected = TRUE;
	pParent->Invalidate( GetBoundingRect(), INVALIDATE_NOCHILDREN);
}

//------------------------------------------------------------------------
void SbaJoinTabConn::Deselect()
{
	bSelected = FALSE;
	Rectangle rcBounding = GetBoundingRect();
	rcBounding.Bottom() += 1;
	rcBounding.Right() += 1;
	pParent->Invalidate( rcBounding, INVALIDATE_NOCHILDREN );
		// Kommentar siehe ::Invalidate ...
}

//------------------------------------------------------------------------
BOOL SbaJoinTabConn::CheckHit( const Point& rMousePos )
{
	//////////////////////////////////////////////////////////////////////
	// Pruefen, ob auf eine der Linien geclickt worden ist
	SbaJoinConnLine* pConnLine = GetConnLineList()->First();
	while( pConnLine )
	{
		if( pConnLine->IsValid() && pConnLine->CheckHit(rMousePos) )
			return TRUE;
		pConnLine = GetConnLineList()->Next();
	}

	return FALSE;
}

//------------------------------------------------------------------------
void SbaJoinTabConn::Invalidate()
{
	Rectangle rcBounding = GetBoundingRect();
	rcBounding.Bottom() += 1;
	rcBounding.Right() += 1;
		// ich glaube, dass sich Invalidate und Draw(Rectangle) nicht konsistent verhalten : jedenfalls waere dadurch zu
		// erklaeren, warum ohne diesen Fake hier beim Loeschen einer Connection ein Strich an ihrem unteren Ende stehen bleibt :
		// Invalidate erfasst dabei offensichtlich eine Pixelzeile weniger als Draw.
		// Oder alles haengt ganz anders zusammen ... jedenfalls klappt es so ...
	pParent->Invalidate( rcBounding, INVALIDATE_NOCHILDREN );
}

//------------------------------------------------------------------------
Rectangle SbaJoinTabConn::GetBoundingRect()
{
	//////////////////////////////////////////////////////////////////////
	// Aus allen Linien das umgebende Rechteck bestimmen
	Rectangle aBoundingRect( Point(0,0), Point(0,0) );
	Rectangle aTempRect;
	SbaJoinConnLine* pConnLine = GetConnLineList()->First();
	while( pConnLine )
	{
		aTempRect = pConnLine->GetBoundingRect();

		//////////////////////////////////////////////////////////////////////
		// Ist das BoundingRect dieser Linie gueltig?
		if( (aTempRect.GetWidth()!=1) && (aTempRect.GetHeight()!=1) )
		{
			if( (aBoundingRect.GetWidth()==1) && (aBoundingRect.GetHeight()==1) )
				aBoundingRect = pConnLine->GetBoundingRect();
			else
				aBoundingRect.Union( pConnLine->GetBoundingRect() );
		}
		pConnLine = GetConnLineList()->Next();
	}

	return aBoundingRect;
}

//------------------------------------------------------------------------
void SbaJoinTabConn::Draw( const Rectangle& rRect )
{
	//////////////////////////////////////////////////////////////////////
	// Nur neu zeichnen, wenn Connection innerhalb des ungueltigen Bereichs liegt
/*	Rectangle aIntersectRect = GetBoundingRect().GetIntersection( rRect );
	Rectangle aBoundingRect = GetBoundingRect();
	if( !aIntersectRect.GetWidth() && !aIntersectRect.GetHeight() )
		return;
*/

	//////////////////////////////////////////////////////////////////////
	// Linien zeichnen
	SbaJoinConnLineList* pConnLineList = GetConnLineList();
	SbaJoinConnLine* pConnLine = pConnLineList->First();
	while( pConnLine )
	{
		pConnLine->Draw( pParent );
		pConnLine = pConnLineList->Next();
	}
}




