/*************************************************************************
 *
 *  $RCSfile: qryfilt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:57 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <sdb/sdbconn.hxx>
#include <sdb/sdbtools.hxx>

#ifndef _SBA_SBASTR_HXX
#include "sbastr.hxx"
#endif
#ifndef _SBA_SBADBIT_HXX
#include "sbadbit.hxx"
#endif
#ifndef _SBA_SBARES_HXX
#include "sbares.hxx"
#endif
#ifndef _SBA_SBAOBJ_HXX
#include "sbaobj.hxx"
#endif
#ifndef _SBA_SBAQRYFILT_HRC
#include "qryfilt.hrc"
#endif
#ifndef _SBA_QRYFILT_HXX
#include "qryfilt.hxx"
#endif

#ifndef _SDB_SDBCURS_HXX //autogen
#include <sdb/sdbcurs.hxx>
#endif

static const char	aAND[] = "AND";
static const char	aOR[] = "OR";

//------------------------------------------------------------------------------
void Replace_OS_PlaceHolder(String& aString)
{
	while (aString.SearchAndReplace( '*', '%' ) != STRING_NOTFOUND);
	while (aString.SearchAndReplace( '?', '_' ) != STRING_NOTFOUND);
}

//------------------------------------------------------------------------------
void Replace_SQL_PlaceHolder(String& aString)
{
	while (aString.SearchAndReplace( '%', '*' ) != STRING_NOTFOUND);
	while (aString.SearchAndReplace( '_', '?' ) != STRING_NOTFOUND);
}

//------------------------------------------------------------------------------
SbaPredicateItem* CreatePredicateItem(SbaParseIterator& rIter,
									  const String& rFieldName,
									  const ODbVariantRef& xVar )
{
	if (xVar.is())
	{
		SbaPredicateItem* pItem = new SbaPredicateItem;

		// Init LogTyp und Feldname
		pItem->aFieldName	= rFieldName;
		pItem->eLogType		= SBA_LOG_INVALID;

		// Init Typ
		pItem->eType		= xVar->isNull() ? SDB_PRED_ISNULL : SDB_PRED_EQUAL;
		pItem->eVarType		= rIter.GetCol(pItem->aFieldName)->GetType();

		switch (pItem->eVarType)
		{
			case SDB_DBTYPE_TIMESTAMP:
			case SDB_DBTYPE_DATE:
			case SDB_DBTYPE_TIME:
				pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, sal_True );
				break;
			default:
				pItem->aValue	= SdbTools::SQLString( pItem->eVarType, *xVar, sal_False );
		}
		return pItem;
	}
	else
		return NULL;
}
DBG_NAME(DlgFilterCrit);
//------------------------------------------------------------------------------
DlgFilterCrit::DlgFilterCrit(Window * pParent,
							 SbaParseIterator* pIter,
							 const SdbColumnsRef& aCols,
							 const ODbVariantRef& xVar,
							 const String& rFieldName)
			: ModalDialog( pParent, SbaResId( DLG_FILTERCRIT ) ),
			aLB_WHEREFIELD1		( this, ResId( LB_WHEREFIELD1 ) ),
			aLB_WHERECOMP1		( this, ResId( LB_WHERECOMP1 ) ),
			aET_WHEREVALUE1		( this, ResId( ET_WHEREVALUE1 ) ),
			aLB_WHERECOND2		( this, ResId( LB_WHERECOND2 ) ),
			aLB_WHEREFIELD2		( this, ResId( LB_WHEREFIELD2 ) ),
			aLB_WHERECOMP2		( this, ResId( LB_WHERECOMP2 ) ),
			aET_WHEREVALUE2		( this, ResId( ET_WHEREVALUE2 ) ),
			aLB_WHERECOND3		( this, ResId( LB_WHERECOND3 ) ),
			aLB_WHEREFIELD3		( this, ResId( LB_WHEREFIELD3 ) ),
			aLB_WHERECOMP3		( this, ResId( LB_WHERECOMP3 ) ),
			aET_WHEREVALUE3		( this, ResId( ET_WHEREVALUE3 ) ),
			aFT_WHEREFIELD		( this, ResId( FT_WHEREFIELD ) ),
			aFT_WHERECOMP		( this, ResId( FT_WHERECOMP ) ),
			aFT_WHEREVALUE		( this, ResId( FT_WHEREVALUE ) ),
			aFT_WHEREOPER		( this, ResId( FT_WHEREOPER ) ),
			aGB_FIELDS			( this, ResId( GB_FIELDS ) ),
			aBT_OK				( this, ResId( BT_OK ) ),
			aBT_CANCEL			( this, ResId( BT_CANCEL ) ),
			aBT_HELP			( this, ResId( BT_HELP ) ),
			aSTR_NOENTRY		( ResId( STR_NOENTRY ) ),
			aSTR_COMPARE_OPERATORS( ResId( STR_COMPARE_OPERATORS ) ),
			rIterator( *pIter ),
			aColumns(aCols)
{
	DBG_CTOR(DlgFilterCrit,NULL);
	// Den String fuer noEntry in die ListBoxen der Feldnamen schreiben
	aLB_WHEREFIELD1.InsertEntry( aSTR_NOENTRY );
	aLB_WHEREFIELD2.InsertEntry( aSTR_NOENTRY );
	aLB_WHEREFIELD3.InsertEntry( aSTR_NOENTRY );

	// ... sowie auch die restlichen Felder
	String aName;
	for ( sal_uInt16 i=1; i<aColumns->Count(); i++ )
	{
		const SdbColumn* pColumn = aColumns->Column(i);
		sal_uInt16 nSearchFlags = pColumn->GetAllFlags();
		if(!pColumn->IsFunction() && (nSearchFlags & (SDB_FLAGS_SEARCH_ALL|SDB_FLAGS_SEARCH_LIKE|SDB_FLAGS_SEARCH_EXCEPT_LIKE)))
		{
			aName = pColumn->GetName();
			aLB_WHEREFIELD1.InsertEntry( aName );
			aLB_WHEREFIELD2.InsertEntry( aName );
			aLB_WHEREFIELD3.InsertEntry( aName );
		}
	}
	// Die ListBoxen der Feldnamen mit noEntry initialisieren
	aLB_WHEREFIELD1.SelectEntryPos(0);
	aLB_WHEREFIELD2.SelectEntryPos(0);
	aLB_WHEREFIELD3.SelectEntryPos(0);

	// Jetzt die Felder mit den Kriterien des SQL-Strings fuellen
	SbaPredicateItem* pNewItem = CreatePredicateItem(rIterator, rFieldName, xVar );
	if (pNewItem)
		pNewItem->eLogType = SBA_LOG_AND;

	rIterator.FillCriteriaList( aCritList );
	if( pNewItem && (!IsRedundantCrit(pNewItem)) && (aCritList.Count()<3) )
		aCritList.Insert( pNewItem, LIST_APPEND );
	else
		delete pNewItem;

	sal_uInt32 nCount(aCritList.Count());
	if( nCount )
	{
		// Eintraege in Dlg setzen
		for (sal_uInt32 i=0 ; i < nCount; i++ )
		{
			SbaPredicateItem* pCritItem = aCritList.GetObject(i);
			SetLine( (sal_uInt16)i, pCritItem );
		}

	}
	else
		SBA_MOD()->ShowDbStatus(rIterator.IteratorStatus(),dbSyntaxError);

	EnableLines();

	aLB_WHEREFIELD1.SetSelectHdl(LINK(this,DlgFilterCrit,ListSelectHdl));
	aLB_WHEREFIELD2.SetSelectHdl(LINK(this,DlgFilterCrit,ListSelectHdl));
	aLB_WHEREFIELD3.SetSelectHdl(LINK(this,DlgFilterCrit,ListSelectHdl));

	aLB_WHERECOMP1.SetSelectHdl(LINK(this,DlgFilterCrit,ListSelectCompHdl));
	aLB_WHERECOMP2.SetSelectHdl(LINK(this,DlgFilterCrit,ListSelectCompHdl));
	aLB_WHERECOMP3.SetSelectHdl(LINK(this,DlgFilterCrit,ListSelectCompHdl));

	FreeResource();
}
//------------------------------------------------------------------------------
DlgFilterCrit::DlgFilterCrit( Window * pParent, SbaQueryObj& rQueryObj, const String& rFieldName, const ODbVariantRef& xVar,
	const SbaPredicateItemList* pItemList )
	:ModalDialog( pParent, SbaResId( DLG_FILTERCRIT ) )
	,aLB_WHEREFIELD1		( this, ResId( LB_WHEREFIELD1 ) )
	,aLB_WHERECOMP1		( this, ResId( LB_WHERECOMP1 ) )
	,aET_WHEREVALUE1		( this, ResId( ET_WHEREVALUE1 ) )
	,aLB_WHERECOND2		( this, ResId( LB_WHERECOND2 ) )
	,aLB_WHEREFIELD2		( this, ResId( LB_WHEREFIELD2 ) )
	,aLB_WHERECOMP2		( this, ResId( LB_WHERECOMP2 ) )
	,aET_WHEREVALUE2		( this, ResId( ET_WHEREVALUE2 ) )
	,aLB_WHERECOND3		( this, ResId( LB_WHERECOND3 ) )
	,aLB_WHEREFIELD3		( this, ResId( LB_WHEREFIELD3 ) )
	,aLB_WHERECOMP3		( this, ResId( LB_WHERECOMP3 ) )
	,aET_WHEREVALUE3		( this, ResId( ET_WHEREVALUE3 ) )
	,aFT_WHEREFIELD		( this, ResId( FT_WHEREFIELD ) )
	,aFT_WHERECOMP		( this, ResId( FT_WHERECOMP ) )
	,aFT_WHEREVALUE		( this, ResId( FT_WHEREVALUE ) )
	,aFT_WHEREOPER		( this, ResId( FT_WHEREOPER ) )
	,aGB_FIELDS			( this, ResId( GB_FIELDS ) )
	,aBT_OK				( this, ResId( BT_OK ) )
	,aBT_CANCEL			( this, ResId( BT_CANCEL ) )
	,aBT_HELP			( this, ResId( BT_HELP ) )
	,aSTR_NOENTRY		( ResId( STR_NOENTRY ) )
	,aSTR_COMPARE_OPERATORS( ResId( STR_COMPARE_OPERATORS ) )
	,rIterator( rQueryObj.Iterator() )
	,aColumns(rQueryObj.GetCursor()->GetColumns())
{
	DBG_CTOR(DlgFilterCrit,NULL);
	// Den String fuer noEntry in die ListBoxen der Feldnamen schreiben
	aLB_WHEREFIELD1.InsertEntry( aSTR_NOENTRY );
	aLB_WHEREFIELD2.InsertEntry( aSTR_NOENTRY );
	aLB_WHEREFIELD3.InsertEntry( aSTR_NOENTRY );

	// ... sowie auch die restlichen Felder
	String aName;
	for ( sal_uInt16 i=1; i<aColumns->Count(); i++ )
	{
		const SdbColumn* pColumn = aColumns->Column(i);
		sal_uInt16 nSearchFlags = pColumn->GetAllFlags();
		if(!pColumn->IsFunction() && (nSearchFlags & (SDB_FLAGS_SEARCH_ALL|SDB_FLAGS_SEARCH_LIKE|SDB_FLAGS_SEARCH_EXCEPT_LIKE)))
		{
			aName = pColumn->GetName();
			aLB_WHEREFIELD1.InsertEntry( aName );
			aLB_WHEREFIELD2.InsertEntry( aName );
			aLB_WHEREFIELD3.InsertEntry( aName );
		}
	}

	// Die ListBoxen der Feldnamen mit noEntry initialisieren
	aLB_WHEREFIELD1.SelectEntryPos(0);
	aLB_WHEREFIELD2.SelectEntryPos(0);
	aLB_WHEREFIELD3.SelectEntryPos(0);

	// Jetzt die Felder mit den Kriterien des SQL-Strings fuellen
	SbaPredicateItem* pNewItem = rQueryObj.CreatePredicateItem( rFieldName, xVar );
	if (pNewItem)
		pNewItem->eLogType = SBA_LOG_AND;

	if (pItemList == NULL)
		rIterator.FillCriteriaList( aCritList );
	else
	{
		DBG_ASSERT(aCritList.Count() == 0, "DlgFilterCrit::DlgFilterCrit : criteria list isn't empty !");
			// wir sind hier im Constructor, da sollte das schon so sein ...
		for (sal_uInt16 i=0; i<pItemList->Count(); ++i)
			aCritList.Insert(new SbaPredicateItem(*pItemList->GetObject(i)), LIST_APPEND);
	}

	if( pNewItem && (!IsRedundantCrit(pNewItem)) && (aCritList.Count()<3) )
		aCritList.Insert( pNewItem, LIST_APPEND );
	else
		delete pNewItem;

	sal_uInt16 nCount = (sal_uInt16)aCritList.Count();
	if( nCount )
	{
		// Eintraege in Dlg setzen
		for (i=0 ; i < nCount; i++ )
		{
			SbaPredicateItem* pCritItem = aCritList.GetObject(i);
			SetLine( i, pCritItem );
		}

	}
	else
		SBA_MOD()->ShowDbStatus(rIterator.IteratorStatus(),dbSyntaxError);

	EnableLines();

	aLB_WHEREFIELD1.SetSelectHdl(LINK(this, DlgFilterCrit, ListSelectHdl));
	aLB_WHEREFIELD2.SetSelectHdl(LINK(this, DlgFilterCrit, ListSelectHdl));
	aLB_WHEREFIELD3.SetSelectHdl(LINK(this, DlgFilterCrit, ListSelectHdl));

	aLB_WHERECOMP1.SetSelectHdl(LINK(this, DlgFilterCrit, ListSelectCompHdl));
	aLB_WHERECOMP2.SetSelectHdl(LINK(this, DlgFilterCrit, ListSelectCompHdl));
	aLB_WHERECOMP3.SetSelectHdl(LINK(this, DlgFilterCrit, ListSelectCompHdl));

	FreeResource();
}

//------------------------------------------------------------------------------
DlgFilterCrit::~DlgFilterCrit()
{
	DBG_DTOR(DlgFilterCrit,NULL);

	// Liste loeschen
	for( sal_uInt16 i=0 ; i < aCritList.Count() ; i++ )
	{
		delete aCritList.GetObject(i);
	}
	aCritList.Clear();
}

#define LbText(x)		((x).GetSelectEntry())
#define LbPos(x)		((x).GetSelectEntryPos())

//------------------------------------------------------------------------------
sal_Bool DlgFilterCrit::IsRedundantCrit( const SbaPredicateItem* pItem )  const
{
	DBG_CHKTHIS(DlgFilterCrit,NULL);
	SbaPredicateItem* pCompItem;
	for( sal_uInt16 i=0 ; i < aCritList.Count() ; i++ )
	{
		pCompItem = aCritList.GetObject(i);
		if( (pItem->aFieldName == pCompItem->aFieldName) &&
			(pItem->eType == pCompItem->eType) &&
			(pItem->aValue == pCompItem->aValue) )
			return sal_True;
	}

	return sal_False;
}
//------------------------------------------------------------------------------
SdbPredicateType DlgFilterCrit::GetSdbPredicateType(sal_uInt16 nPos,sal_uInt16 nCount) const
{
	SdbPredicateType ePreType;

	if(nCount == 10)
	{
		switch(nPos)
		{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			ePreType = (SdbPredicateType)(nPos + SDB_PRED_EQUAL);
			break;
		case 6:
			ePreType = SDB_PRED_LIKE;
			break;
		case 7:
			ePreType = SDB_PRED_NOTLIKE;
			break;
		case 8:
			ePreType = SDB_PRED_ISNULL;
			break;
		case 9:
			ePreType = SDB_PRED_ISNOTNULL;
			break;
		}
	}
	else if(nCount == 8)
	{
		switch(nPos)
		{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			ePreType = (SdbPredicateType)(nPos + SDB_PRED_EQUAL);
			break;
		case 6:
			ePreType = SDB_PRED_ISNULL;
			break;
		case 7:
			ePreType = SDB_PRED_ISNOTNULL;
			break;
		}
	}
	else
	{
		switch(nPos)
		{
		case 0:
			ePreType = SDB_PRED_LIKE;
			break;
		case 1:
			ePreType = SDB_PRED_NOTLIKE;
			break;
		}
	}
	return ePreType;
}
//------------------------------------------------------------------------------
sal_uInt16 DlgFilterCrit::GetSelectionPos(SdbPredicateType eType,const ListBox& rListBox) const
{
	sal_uInt16 nPos;
	switch(eType)
	{
		case SDB_PRED_EQUAL:
			nPos = 0;
			break;
		case SDB_PRED_NOTEQUAL:
			nPos = 1;
			break;
		case SDB_PRED_LESS:
			nPos = 2;
			break;
		case SDB_PRED_LESSOREQUAL:
			nPos = 3;
			break;
		case SDB_PRED_GREATER:
			nPos = 4;
			break;
		case SDB_PRED_GREATEROREQUAL:
			nPos = 5;
			break;
		case SDB_PRED_NOTLIKE:
			nPos = rListBox.GetEntryCount() > 2 ? rListBox.GetEntryCount()-3 : 0;
			break;
		case SDB_PRED_LIKE:
			nPos = rListBox.GetEntryCount() > 2 ? rListBox.GetEntryCount()-4 : 1;
			break;
		case SDB_PRED_ISNULL:
			nPos = rListBox.GetEntryCount()-2;
			break;
		case SDB_PRED_ISNOTNULL:
			nPos = rListBox.GetEntryCount()-1;
			break;
	}
	return nPos;
}

//------------------------------------------------------------------------------
void DlgFilterCrit::GetFilterList(SbaPredicateItemList& rItemList) const
{
	SbaPredicateItem		aItem;
	sal_Bool bSet=sal_False;

	if( LbPos(aLB_WHEREFIELD1) != 0 )
	{
		aItem.aValue		= aET_WHEREVALUE1.GetText();
		aItem.eType			= GetSdbPredicateType(aLB_WHERECOMP1.GetSelectEntryPos(),aLB_WHERECOMP1.GetEntryCount());
		aItem.aFieldName	= aLB_WHEREFIELD1.GetSelectEntry();
		aItem.eLogType		= SBA_LOG_INVALID;
		bSet				= sal_True;

		const SdbColumn* pCol = rIterator.GetCol( aItem.aFieldName );
		if(!pCol)
			pCol = (*aColumns)[aItem.aFieldName];
		aItem.eVarType		= pCol->GetType();

		rItemList.Insert( new SbaPredicateItem( aItem ), LIST_APPEND );
	}

	if( LbPos(aLB_WHEREFIELD2) != 0 )
	{
		aItem.aValue		= aET_WHEREVALUE2.GetText();
		aItem.eType			= GetSdbPredicateType(aLB_WHERECOMP2.GetSelectEntryPos(),aLB_WHERECOMP2.GetEntryCount());
		aItem.aFieldName	= aLB_WHEREFIELD2.GetSelectEntry();
		aItem.eLogType		= bSet ? (SbaLogType)aLB_WHERECOND2.GetSelectEntryPos() : SBA_LOG_INVALID;
		bSet				= sal_True;

		const SdbColumn* pCol = rIterator.GetCol( aItem.aFieldName );
		if(!pCol)
			pCol = (*aColumns)[aItem.aFieldName];
		aItem.eVarType		= pCol->GetType();

		rItemList.Insert( new SbaPredicateItem( aItem ), LIST_APPEND );
	}

	if( LbPos(aLB_WHEREFIELD3) != 0 )
	{
		aItem.aValue		= aET_WHEREVALUE3.GetText();
		aItem.eType			= GetSdbPredicateType(aLB_WHERECOMP3.GetSelectEntryPos(),aLB_WHERECOMP3.GetEntryCount());
		aItem.aFieldName	= aLB_WHEREFIELD3.GetSelectEntry();
		aItem.eLogType		= bSet ? (SbaLogType)aLB_WHERECOND3.GetSelectEntryPos() : SBA_LOG_INVALID;


		const SdbColumn* pCol = rIterator.GetCol( aItem.aFieldName );
		if(!pCol)
			pCol = (*aColumns)[aItem.aFieldName];
		aItem.eVarType		= pCol->GetType();


		rItemList.Insert( new SbaPredicateItem( aItem ), LIST_APPEND );
	}
}

//------------------------------------------------------------------------------
void DlgFilterCrit::SetFilterList( const SbaPredicateItemList* pItemList )
{
	for (sal_uInt16 i=0; i<pItemList->Count(); ++i)
	{
		SbaPredicateItem* pCritItem = pItemList->GetObject(i);
		SetLine(i, pCritItem);
	}

	// die nicht gesetzten auf 'kein' '=' ''
	sal_uInt16 nItemsSet = pItemList ? min((sal_Int32)pItemList->Count(), 3L) : 0;
	if (nItemsSet<3)
	{
		aLB_WHEREFIELD3.SelectEntryPos(0);
		aLB_WHERECOMP3.SelectEntryPos(0);
		aET_WHEREVALUE3.SetText(String());
	}
	if (nItemsSet<2)
	{
		aLB_WHEREFIELD2.SelectEntryPos(0);
		aLB_WHERECOMP2.SelectEntryPos(0);
		aET_WHEREVALUE2.SetText(String());
	}
	if (nItemsSet<1)
	{
		aLB_WHEREFIELD1.SelectEntryPos(0);
		aLB_WHERECOMP1.SelectEntryPos(0);
		aET_WHEREVALUE1.SetText(String());
	}
	EnableLines();
}

//------------------------------------------------------------------------------
void DlgFilterCrit::SetLine( sal_uInt16 nIdx, SbaPredicateItem* pItem )
{
	DBG_CHKTHIS(DlgFilterCrit,NULL);
	DBG_ASSERT( pItem, "SetLine" );
	String	aValue(pItem->aValue);
	::Replace_SQL_PlaceHolder(aValue);

	if (pItem->eVarType == SDB_DBTYPE_BOOLEAN)
	{
		if (aValue.EqualsAscii("sal_True"))	   // uebersetzten von sal_True und sal_False
			aValue = String(SbaResId(SBA_STR_TRUE));
		else if (aValue.EqualsAscii("sal_False"))
			aValue = String(SbaResId(SBA_STR_FALSE));
	}

	switch( nIdx )
	{
		case 0:
		{
			SelectField( aLB_WHEREFIELD1, pItem->aFieldName );
			ListSelectHdl(&aLB_WHEREFIELD1);
			aLB_WHERECOMP1.SelectEntryPos( GetSelectionPos(pItem->eType , aLB_WHERECOMP1));
			aET_WHEREVALUE1.SetText( aValue );
		}
		break;

		case 1:
		{
			aLB_WHERECOND2.SelectEntryPos( pItem->eLogType );
			SelectField( aLB_WHEREFIELD2, pItem->aFieldName );
			ListSelectHdl(&aLB_WHEREFIELD2);
			aLB_WHERECOMP2.SelectEntryPos( GetSelectionPos(pItem->eType , aLB_WHERECOMP2));
			aET_WHEREVALUE2.SetText( aValue );
		}
		break;

		case 2:
		{
			aLB_WHERECOND3.SelectEntryPos( pItem->eLogType );
			SelectField( aLB_WHEREFIELD3, pItem->aFieldName );
			ListSelectHdl(&aLB_WHEREFIELD3);
			aLB_WHERECOMP3.SelectEntryPos( GetSelectionPos(pItem->eType , aLB_WHERECOMP3));
			aET_WHEREVALUE3.SetText( aValue );
		}
		break;
	}
}

//------------------------------------------------------------------------------
void DlgFilterCrit::SelectField( ListBox& rBox, const String& rField )
{
	DBG_CHKTHIS(DlgFilterCrit,NULL);
	sal_uInt16 nCnt = rBox.GetEntryCount();
	sal_Bool bCase = rIterator.TablesAreSensitive();

	for( sal_uInt16 i=0 ; i<nCnt ; i++ )
	{
		if(bCase ? rBox.GetEntry(i) == rField : rBox.GetEntry(i).EqualsIgnoreCaseAscii(rField))
		{
			rBox.SelectEntryPos(i);
			return;
		}
	}

	rBox.SelectEntryPos(0);
}

//------------------------------------------------------------------------------
void DlgFilterCrit::EnableLines()
{
	DBG_CHKTHIS(DlgFilterCrit,NULL);
	// Enablen/Disablen ganzer Zeilen
	if( LbPos(aLB_WHEREFIELD1) == 0 )
	{
		aLB_WHEREFIELD2.Disable();
		aLB_WHERECOND2.Disable();
		aLB_WHERECOMP2.Disable();
		aET_WHEREVALUE2.Disable();

		aLB_WHEREFIELD3.Disable();
		aLB_WHERECOND3.Disable();
		aLB_WHERECOMP3.Disable();
		aET_WHEREVALUE3.Disable();
	}
	else
	{
		aLB_WHEREFIELD2.Enable();
		aLB_WHERECOND2.Enable();
		aLB_WHERECOMP2.Enable();
		aET_WHEREVALUE2.Enable();

		aLB_WHEREFIELD3.Enable();
		aLB_WHERECOND3.Enable();
		aLB_WHERECOMP3.Enable();
		aET_WHEREVALUE3.Enable();
	}

	if( LbPos(aLB_WHEREFIELD2) == 0 )
	{
		aLB_WHEREFIELD3.Disable();
		aLB_WHERECOND3.Disable();
		aLB_WHERECOMP3.Disable();
		aET_WHEREVALUE3.Disable();
	}
	else
	{
		aLB_WHEREFIELD3.Enable();
		aLB_WHERECOND3.Enable();
		aLB_WHERECOMP3.Enable();
		aET_WHEREVALUE3.Enable();
	}

	// Vergleichsfeld gleich NOENTRY
	if( LbPos(aLB_WHEREFIELD1) == 0 )
	{
		aLB_WHERECOMP1.Disable();
		aET_WHEREVALUE1.Disable();
	}
	else
	{
		aLB_WHEREFIELD1.Enable();
		aLB_WHERECOMP1.Enable();
		aET_WHEREVALUE1.Enable();
	}

	if( LbPos(aLB_WHEREFIELD2) == 0 )
	{
		aLB_WHERECOND2.Disable();
		aLB_WHERECOMP2.Disable();
		aET_WHEREVALUE2.Disable();
	}
	else
	{
		aLB_WHERECOND2.Enable();
		aLB_WHEREFIELD2.Enable();
		aLB_WHERECOMP2.Enable();
		aET_WHEREVALUE2.Enable();
	}

	if( LbPos(aLB_WHEREFIELD3) == 0 )
	{
		aLB_WHERECOND3.Disable();
		aLB_WHERECOMP3.Disable();
		aET_WHEREVALUE3.Disable();
	}
	else
	{
		aLB_WHERECOND3.Enable();
		aLB_WHERECOND3.Enable();
		aLB_WHEREFIELD3.Enable();
		aLB_WHERECOMP3.Enable();
		aET_WHEREVALUE3.Enable();
	}

	// Vergleichsoperator gleich ISNULL oder ISNOTNULL
	if(aLB_WHERECOMP1.GetEntryCount() > 2 &&
		((LbPos(aLB_WHERECOMP1) == aLB_WHERECOMP1.GetEntryCount()-1) ||
		 (LbPos(aLB_WHERECOMP1) == aLB_WHERECOMP1.GetEntryCount()-2)) )
		aET_WHEREVALUE1.Disable();

	if(aLB_WHERECOMP2.GetEntryCount() > 2 &&
		((LbPos(aLB_WHERECOMP2) == aLB_WHERECOMP2.GetEntryCount()-1) ||
		 (LbPos(aLB_WHERECOMP2) == aLB_WHERECOMP2.GetEntryCount()-2)) )
		aET_WHEREVALUE2.Disable();

	if(aLB_WHERECOMP3.GetEntryCount() > 2 &&
		((LbPos(aLB_WHERECOMP3) == aLB_WHERECOMP3.GetEntryCount()-1) ||
		 (LbPos(aLB_WHERECOMP3) == aLB_WHERECOMP3.GetEntryCount()-2)) )
		aET_WHEREVALUE3.Disable();

}

//------------------------------------------------------------------------------
IMPL_LINK( DlgFilterCrit, ListSelectHdl, ListBox *, pListBox )
{
	String aName;
	ListBox* pComp;
	if(pListBox == &aLB_WHEREFIELD1)
	{
		aName = LbText(aLB_WHEREFIELD1);
		pComp = &aLB_WHERECOMP1;
	}
	else if(pListBox == &aLB_WHEREFIELD2)
	{
		aName = LbText(aLB_WHEREFIELD2);
		pComp = &aLB_WHERECOMP2;
	}
	else
	{
		aName = LbText(aLB_WHEREFIELD3);
		pComp = &aLB_WHERECOMP3;
	}

	pComp->Clear();

	const SdbColumn* pColumn = aColumns->Column(aName);
	if(pColumn)
	{
		sal_uInt16 nSearchFlags = pColumn->GetAllFlags();
		if((nSearchFlags & SDB_FLAGS_SEARCH_ALL) == SDB_FLAGS_SEARCH_ALL)
		{
			for(xub_StrLen i=0;i<aSTR_COMPARE_OPERATORS.GetTokenCount();i++)
				pComp->InsertEntry(aSTR_COMPARE_OPERATORS.GetToken(i));
		}
		else if((nSearchFlags & SDB_FLAGS_SEARCH_LIKE) == SDB_FLAGS_SEARCH_LIKE)
		{
			pComp->InsertEntry(aSTR_COMPARE_OPERATORS.GetToken(6));
			pComp->InsertEntry(aSTR_COMPARE_OPERATORS.GetToken(7));
		}
		else if((nSearchFlags & SDB_FLAGS_SEARCH_EXCEPT_LIKE) == SDB_FLAGS_SEARCH_EXCEPT_LIKE)
		{
			for(xub_StrLen i=0;i<6;i++)
				pComp->InsertEntry(aSTR_COMPARE_OPERATORS.GetToken(i));
			for(i=8;i<aSTR_COMPARE_OPERATORS.GetTokenCount();i++)
				pComp->InsertEntry(aSTR_COMPARE_OPERATORS.GetToken(i));
		}
		else
		{
			DBG_ASSERT(0,"DlgFilterCrit::ListSelectHdl: Diese Column drfte garnicht vorhanden sein!");
		}
	}
	pComp->SelectEntryPos(0);

	EnableLines();
	return 0;
}



//------------------------------------------------------------------------------
IMPL_LINK_INLINE_START( DlgFilterCrit, ListSelectCompHdl, ListBox *, pListBox )
{
	EnableLines();
	return 0;
}
IMPL_LINK_INLINE_END( DlgFilterCrit, ListSelectCompHdl, ListBox *, pListBox )
//------------------------------------------------------------------------------
String DlgFilterCrit::BuildWherePart()
{
	DBG_CHKTHIS(DlgFilterCrit,NULL);
	String	aResult;

	SbaPredicateItemList	aList;
	GetFilterList(aList);

	if( !aList.Count() )
		return String();

	aResult += rIterator.BuildWherePart( aList );
	for( sal_uInt16 i=0 ; i < aList.Count(); i++ )
		delete aList.GetObject(i);
	aList.Clear();

	::Replace_OS_PlaceHolder(aResult);

	return aResult;
}


