/*************************************************************************
 *
 *  $RCSfile: brwview.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBX_BRWVIEW_HXX
#include "brwview.hxx"
#endif
#ifndef _SBA_GRID_HXX
#include "sbagrid.hxx"
#endif
#ifndef _SBA_API_HRC
#include "api.hrc"
#endif
#ifndef _SFXAPP_HXX //autogen wg. SFX_APP
#include <sfx2/app.hxx>
#endif
#ifndef _SFXIMGMGR_HXX //autogen wg. SFX_IMAGEMANAGER
#include <sfx2/imgmgr.hxx>
#endif
#ifndef _TOOLKIT_HELPER_VCLUNOHELPER_HXX_
#include <toolkit/helper/vclunohelper.hxx>
#endif
#ifndef _UTL_TYPES_HXX_
#include <unotools/types.hxx>
#endif

//==================================================================
//= UnoDataBrowserView
//==================================================================

// -------------------------------------------------------------------------
UnoDataBrowserView::UnoDataBrowserView(Window* pParent, const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& _rFactory)
	:Window(pParent)
	,m_pToolBox(NULL)
	,m_pVclControl(NULL)
	,m_xServiceFactory(_rFactory)
{
}

// -------------------------------------------------------------------------
void UnoDataBrowserView::Construct(const ::com::sun::star::uno::Reference< ::com::sun::star::awt::XControlModel >& xModel)
{
	try
	{
		// our UNO representation
		m_xMe = VCLUnoHelper::CreateControlContainer(this);
		DBG_ASSERT(m_xMe.is(), "UnoDataBrowserView::Construct : no UNO representation");

		// create the (UNO-) control
		m_xGrid = new SbaXGridControl(m_xServiceFactory);
		DBG_ASSERT(m_xGrid.is(), "UnoDataBrowserView::Construct : could not create a grid control !");
		// in design mode (for the moment)
		m_xGrid->setDesignMode(sal_True);

		::com::sun::star::uno::Reference< ::com::sun::star::awt::XWindow >  xGridWindow(m_xGrid, ::com::sun::star::uno::UNO_QUERY);
		xGridWindow->setVisible(sal_True);
		xGridWindow->setEnable(sal_True);

		// introduce the model to the grid
		m_xGrid->setModel(xModel);
		// introduce the container (me) to the grid
		::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet >  xModelSet(xModel, ::com::sun::star::uno::UNO_QUERY);
		m_xMe->addControl(::utl::getString(xModelSet->getPropertyValue(PROPERTY_NAME)), m_xGrid);

		// get the VCL-control
		m_pVclControl = NULL;
		::com::sun::star::uno::Reference< ::com::sun::star::awt::XWindowPeer >  xPeer = m_xGrid->getPeer();
		if (xPeer.is())
		{
			SbaXGridPeer* pPeer = SbaXGridPeer::getImplementation(xPeer);
			if (pPeer)
				m_pVclControl = static_cast<SbaGridControl*>(pPeer->GetWindow());
		}
		DBG_ASSERT(m_pVclControl != NULL, "UnoDataBrowserView::Construct : no real grid control !");
	}
	catch(...)
	{
		::utl::disposeComponent(m_xGrid);
		::utl::disposeComponent(m_xMe);
		throw;
	}
}

// -------------------------------------------------------------------------
UnoDataBrowserView::~UnoDataBrowserView()
{
	setToolBox(NULL);

	::utl::disposeComponent(m_xMe);
	::utl::disposeComponent(m_xGrid);
}

// -------------------------------------------------------------------------
void UnoDataBrowserView::setToolBox(ToolBox* pTB)
{
	if (pTB == m_pToolBox)
		return;

	if (m_pToolBox)
	{
		SFX_IMAGEMANAGER()->ReleaseToolBox(m_pToolBox);
		delete m_pToolBox;
	}

	m_pToolBox = pTB;
	if (m_pToolBox)
	{
		SFX_IMAGEMANAGER()->RegisterToolBox(m_pToolBox, SFX_TOOLBOX_CHANGEOUTSTYLE);

		m_pToolBox->SetParent(this);
		m_pToolBox->Show();
	}

	// rearrange the grid and the TB
	Resize();
}

// -------------------------------------------------------------------------
void UnoDataBrowserView::Resize()
{
	Size aSize(GetOutputSizePixel());
	Size aToolBoxSize(0,0);
	// position the toolbox (if any) at the top
	if (m_pToolBox)
	{
		aToolBoxSize = m_pToolBox->GetSizePixel();
		m_pToolBox->SetPosSizePixel(Point(0 ,0), Size(aSize.Width(), aToolBoxSize.Height()));
	}

	Window::Resize();
	// and let the grid occupy the remaining space ...
	::com::sun::star::uno::Reference< ::com::sun::star::awt::XWindow >  xGridAsWindow(m_xGrid, ::com::sun::star::uno::UNO_QUERY);
	if (xGridAsWindow.is())
		xGridAsWindow->setPosSize(0, aToolBoxSize.Height(), aSize.Width(), aSize.Height() - aToolBoxSize.Height(), ::com::sun::star::awt::PosSize::POSSIZE);
}

//------------------------------------------------------------------
sal_uInt16 UnoDataBrowserView::Model2ViewPos(sal_uInt16 nPos) const
{
	return m_pVclControl ? m_pVclControl->GetViewColumnPos(m_pVclControl->GetColumnIdFromModelPos(nPos)) : -1;
}

//------------------------------------------------------------------
sal_uInt16 UnoDataBrowserView::View2ModelPos(sal_uInt16 nPos) const
{
	return m_pVclControl ? m_pVclControl->GetModelColumnPos(m_pVclControl->GetColumnIdFromViewPos(nPos)) : -1;
}

//------------------------------------------------------------------
sal_uInt16 UnoDataBrowserView::ViewColumnCount() const
{
	return m_pVclControl ? m_pVclControl->GetViewColCount() : 0;
}

//------------------------------------------------------------------
void UnoDataBrowserView::GetFocus()
{
	Window::GetFocus();
	if (m_pVclControl)
		m_pVclControl->GrabFocus();
}

