/*************************************************************************
 *
 *  $RCSfile: tabedund.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:32:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <sdb/tools.hxx>
#include "tabedund.hxx"
#include "editbrw.hxx"
#include "sbatabed.hxx"

#ifndef _SFX_BINDINGS_HXX //autogen
#include <sfx2/bindings.hxx>
#endif

#include "sbares.hxx"
#ifdef UNX
#define _cdecl
#define	_CDECLARE_
#endif

#ifdef MAC
#define _cdecl
#endif

//==============================================================================
// class SbaTabDgnUndoAct
//==============================================================================
SbaTabDgnUndoAct::SbaTabDgnUndoAct( SbaTableDesignCtrl* pOwner ) :
	pTabDgnCtrl(  pOwner )
{
	pTabDgnCtrl->nCurUndoActId++;
}

//-------------------------------------------------------------------------
SbaTabDgnUndoAct::~SbaTabDgnUndoAct()
{
}

//-------------------------------------------------------------------------
void SbaTabDgnUndoAct::Undo()
{
	pTabDgnCtrl->nCurUndoActId--;

	//////////////////////////////////////////////////////////////////////
	// Wenn erstes Undo zurueckgenommen wurde, ist Doc nicht modifiziert worden
	if( pTabDgnCtrl->nCurUndoActId == 0 )
	{
		pTabDgnCtrl->GetDocShell()->SetModified( FALSE );
		SFX_BINDINGS().Invalidate(SID_SAVEDOC);
	}
}

//-------------------------------------------------------------------------
void SbaTabDgnUndoAct::Redo()
{
	pTabDgnCtrl->nCurUndoActId++;

	//////////////////////////////////////////////////////////////////////
	// Wenn Redo fuer erste Undo-Action, muss Modified-Flag wieder gesetzt werden
	if( pTabDgnCtrl->nCurUndoActId > 0 )
	{
		pTabDgnCtrl->GetDocShell()->SetModified( TRUE );
		SFX_BINDINGS().Invalidate(SID_SAVEDOC);
	}
}

//-------------------------------------------------------------------------
String SbaTabDgnUndoAct::GetComment() const
{
	return String();
}

//==============================================================================
// class SbaTabDgnCellUndoAct
//==============================================================================
SbaTabDgnCellUndoAct::SbaTabDgnCellUndoAct( SbaTableDesignCtrl* pOwner, long nRowID, USHORT nColumn ) :
	 SbaTabDgnUndoAct( pOwner )
	,nRow( nRowID )
	,nCol( nColumn )
{
	//////////////////////////////////////////////////////////////////////
	// Text an der Position (nRow, nCol) auslesen
	aOldText = pTabDgnCtrl->GetData( nRow, nCol );
}

//-------------------------------------------------------------------------
SbaTabDgnCellUndoAct::~SbaTabDgnCellUndoAct()
{
}

//-------------------------------------------------------------------------
void SbaTabDgnCellUndoAct::Undo()
{
	//////////////////////////////////////////////////////////////////////
	// Neuen Text der alten Zelle speichern und alten wieder einsetzen
	pTabDgnCtrl->ActivateCell( nRow, nCol );
	aNewText = pTabDgnCtrl->GetData( nRow, nCol );
	pTabDgnCtrl->SetData( nRow, nCol, aOldText );
	//////////////////////////////////////////////////////////////////////
	// Wenn erstes Undo zurueckgenommen wurde, ist Zelle nicht mehr modifiziert
	if (pTabDgnCtrl->GetCurUndoActId() == 1)
	{
		DbCellControllerRef xController = pTabDgnCtrl->Controller();
		if(xController.Is())
			xController->ClearModified();
		pTabDgnCtrl->GetDocShell()->SetModified(FALSE);

	}

	SbaTabDgnUndoAct::Undo();
}

//-------------------------------------------------------------------------
void SbaTabDgnCellUndoAct::Redo()
{
	//////////////////////////////////////////////////////////////////////
	// Neuen Text wieder einseten
	pTabDgnCtrl->ActivateCell( nRow, nCol );
	pTabDgnCtrl->SetData( nRow, nCol, aNewText );

	SbaTabDgnUndoAct::Redo();
}

//-------------------------------------------------------------------------
String SbaTabDgnCellUndoAct::GetComment() const
{
	return String( SbaResId(STR_TABED_UNDO_CELLMODIFIED) );
}

//==============================================================================
// class SbaTabEdUndoAct
//==============================================================================
SbaTabEdUndoAct::SbaTabEdUndoAct( SbaTableEditorCtrl* pOwner ) :
	 SbaTabDgnUndoAct(  pOwner )
{
	pTabEdCtrl = (SbaTableEditorCtrl*)pTabDgnCtrl;
}

//-------------------------------------------------------------------------
SbaTabEdUndoAct::~SbaTabEdUndoAct()
{
}

//==============================================================================
// class SbaTabEdTypeSelUndoAct
//==============================================================================
SbaTabEdTypeSelUndoAct::SbaTabEdTypeSelUndoAct( SbaTableEditorCtrl* pOwner, long nRowID, USHORT nColumn, const String& rOldType ) :
	 SbaTabEdUndoAct( pOwner )
	,nRow( nRowID )
	,nCol( nColumn )
	,aOldType( rOldType )
{
}

//-------------------------------------------------------------------------
SbaTabEdTypeSelUndoAct::~SbaTabEdTypeSelUndoAct()
{
}

//-------------------------------------------------------------------------
void SbaTabEdTypeSelUndoAct::Undo()
{
	//////////////////////////////////////////////////////////////////////
	// Typ zuruecksetzen
	pTabEdCtrl->GoToRow( nRow );
	pTabEdCtrl->GoToColumnId( nCol );
	aNewType = pTabEdCtrl->GetData(nRow,nCol);
	pTabEdCtrl->SetData(nRow,nCol,aOldType);
	pTabEdCtrl->SwitchType( aOldType );

	SbaTabEdUndoAct::Undo();
}

//-------------------------------------------------------------------------
void SbaTabEdTypeSelUndoAct::Redo()
{
	//////////////////////////////////////////////////////////////////////
	// Neuen Typ
	pTabEdCtrl->GoToRow( nRow );
	pTabEdCtrl->GoToColumnId( nCol );
	pTabEdCtrl->SetData(nRow,nCol,aNewType);
	//pTabEdCtrl->SwitchType( aNewType );

	SbaTabEdUndoAct::Redo();
}

//-------------------------------------------------------------------------
String SbaTabEdTypeSelUndoAct::GetComment() const
{
	return String( SbaResId(STR_TABED_UNDO_TYPE_CHANGED) );
}

//==============================================================================
// class SbaTabEdDelUndoAct
//==============================================================================
SbaTabEdDelUndoAct::SbaTabEdDelUndoAct( SbaTableEditorCtrl* pOwner ) :
	 SbaTabEdUndoAct( pOwner )
{
	//////////////////////////////////////////////////////////////////////
	// DeletedRowList fuellen
	SbaTabEdRowList* pOriginalRows = pOwner->GetRowList();
	long nIndex = pOwner->FirstSelectedRow();
	SbaTabEdRow* pOriginalRow;
	SbaTabEdRow* pNewRow;

	while( nIndex >= 0 )
	{
		pOriginalRow = pOriginalRows->GetObject( nIndex );
		pNewRow = new SbaTabEdRow( *pOriginalRow, nIndex );
		aDeletedRows.Insert( pNewRow, LIST_APPEND );

		nIndex = pOwner->NextSelectedRow();
	}
}

//-------------------------------------------------------------------------
SbaTabEdDelUndoAct::~SbaTabEdDelUndoAct()
{
	//////////////////////////////////////////////////////////////////////
	// DeletedRowList loeschen
	SbaTabEdRow* pRow;
	while( pRow=aDeletedRows.Remove((ULONG)0) )
		delete pRow;
	aDeletedRows.Clear();
}

//-------------------------------------------------------------------------
void SbaTabEdDelUndoAct::Undo()
{
	//////////////////////////////////////////////////////////////////////
	// Geloeschte Zeilen wieder einfuegen
	ULONG nPos;
	SbaTabEdRow* pDeletedRow = aDeletedRows.First();
	SbaTabEdRow* pNewOrigRow;
	SbaTabEdRowList* pOriginalRows = pTabEdCtrl->GetRowList();

	while( pDeletedRow )
	{
		pNewOrigRow = new SbaTabEdRow( *pDeletedRow );
		nPos = pDeletedRow->GetPos();
		pOriginalRows->Insert( pNewOrigRow, nPos );

		pDeletedRow = aDeletedRows.Next();
	}

	pTabEdCtrl->Invalidate();
	SbaTabEdUndoAct::Undo();
}

//-------------------------------------------------------------------------
void SbaTabEdDelUndoAct::Redo()
{
	//////////////////////////////////////////////////////////////////////
	// Zeilen wieder loeschen
	ULONG nPos;
	SbaTabEdRow* pDeletedRow = aDeletedRows.First();
	SbaTabEdRowList* pOriginalRows = pTabEdCtrl->GetRowList();

	while( pDeletedRow )
	{
		nPos = pDeletedRow->GetPos();
		delete pOriginalRows->GetObject( nPos );
		pOriginalRows->Remove( nPos );//hack XXXX
		pDeletedRow = aDeletedRows.Next();
	}

	pTabEdCtrl->Invalidate();
	SbaTabEdUndoAct::Redo();
}

//-------------------------------------------------------------------------
String SbaTabEdDelUndoAct::GetComment() const
{
	return String( SbaResId(STR_TABED_UNDO_ROWDELETED) );
}

//==============================================================================
// class SbaTabEdInsUndoAct
//==============================================================================
SbaTabEdInsUndoAct::SbaTabEdInsUndoAct( SbaTableEditorCtrl* pOwner, long nInsertPosition ) :
	 SbaTabEdUndoAct( pOwner )
	,nInsPos( nInsertPosition )
{
	//////////////////////////////////////////////////////////////////////
	// Clipboard-Inhalt in InsertedRowList kopieren
	SbaTabEdRowList* pClipboardList = pOwner->GetClipboardRowList();
	SbaTabEdRow* pClipboardRow = pClipboardList->First();
	SbaTabEdRow* pNewRow;

	while( pClipboardRow )
	{
		pNewRow = new SbaTabEdRow( *pClipboardRow );
		aInsertedRows.Insert( pNewRow, LIST_APPEND );

		pClipboardRow = pClipboardList->Next();
	}
}

//-------------------------------------------------------------------------
SbaTabEdInsUndoAct::~SbaTabEdInsUndoAct()
{
	//////////////////////////////////////////////////////////////////////
	// InsertedRowList loeschen
	SbaTabEdRow* pRow;
	while( pRow=aInsertedRows.Remove((ULONG)0) )
		delete pRow;
	aInsertedRows.Clear();
}

//-------------------------------------------------------------------------
void SbaTabEdInsUndoAct::Undo()
{
	//////////////////////////////////////////////////////////////////////
	// Eingefuegte Zeilen wieder loeschen
	SbaTabEdRowList* pOriginalRows = pTabEdCtrl->GetRowList();

	for( long i=(nInsPos+aInsertedRows.Count()-1); i>(nInsPos-1); i-- )
		delete pOriginalRows->Remove(i);

	pTabEdCtrl->RowRemoved( nInsPos, aInsertedRows.Count(), TRUE );
	pTabEdCtrl->InvalidateHandleColumn();

	SbaTabEdUndoAct::Undo();
}

//-------------------------------------------------------------------------
void SbaTabEdInsUndoAct::Redo()
{
	//////////////////////////////////////////////////////////////////////
	// Zeilen wieder einfuegen
	long nInsertRow = nInsPos;
	SbaTabEdRow* pRow;
	SbaTabEdRow* pInsRow = aInsertedRows.First();
	SbaTabEdRowList* pRowList = pTabEdCtrl->GetRowList();

	while( pInsRow )
	{
		pRow = new SbaTabEdRow( *pInsRow );
		pRowList->Insert( pRow, nInsertRow );

		pInsRow = aInsertedRows.Next();
		nInsertRow++;
	}

	pTabEdCtrl->RowInserted( nInsPos, aInsertedRows.Count(), TRUE );
	pTabEdCtrl->InvalidateHandleColumn();

	SbaTabEdUndoAct::Redo();
}

//-------------------------------------------------------------------------
String SbaTabEdInsUndoAct::GetComment() const
{
	return String( SbaResId(STR_TABED_UNDO_ROWINSERTED) );
}

//==============================================================================
// class SbaTabEdInsNewUndoAct
//==============================================================================
SbaTabEdInsNewUndoAct::SbaTabEdInsNewUndoAct( SbaTableEditorCtrl* pOwner, long nInsertPosition, long nInsertedRows ) :
	 SbaTabEdUndoAct( pOwner )
	,nInsPos( nInsertPosition )
	,nInsRows( nInsertedRows )
{
}

//-------------------------------------------------------------------------
SbaTabEdInsNewUndoAct::~SbaTabEdInsNewUndoAct()
{
}

//-------------------------------------------------------------------------
void SbaTabEdInsNewUndoAct::Undo()
{
	//////////////////////////////////////////////////////////////////////
	// Eingefuegte Zeilen wieder loeschen
	SbaTabEdRowList* pOriginalRows = pTabEdCtrl->GetRowList();

	for( long i=(nInsPos+nInsRows-1); i>(nInsPos-1); i-- )
		delete pOriginalRows->Remove(i);

	pTabEdCtrl->RowRemoved( nInsPos, nInsRows, TRUE );
	pTabEdCtrl->InvalidateHandleColumn();

	SbaTabEdUndoAct::Undo();
}

//-------------------------------------------------------------------------
void SbaTabEdInsNewUndoAct::Redo()
{
	//////////////////////////////////////////////////////////////////////
	// Zeilen wieder einfuegen
	SbaTabEdRowList* pRowList = pTabEdCtrl->GetRowList();

	for( long i=nInsPos; i<(nInsPos+nInsRows); i++ )
		pRowList->Insert( new SbaTabEdRow(), i );

	pTabEdCtrl->RowInserted( nInsPos, nInsRows, TRUE );
	pTabEdCtrl->InvalidateHandleColumn();

	SbaTabEdUndoAct::Redo();
}

//-------------------------------------------------------------------------
String SbaTabEdInsNewUndoAct::GetComment() const
{
	return String( SbaResId(STR_TABED_UNDO_NEWROWINSERTED) );
}

//========================================================================
// class SbaPrimKeyUndoAct
//========================================================================

//-------------------------------------------------------------------------
SbaPrimKeyUndoAct::SbaPrimKeyUndoAct( SbaTableEditorCtrl* pOwner, MultiSelection aDeletedKeys, MultiSelection aInsertedKeys ) :
	 SbaTabEdUndoAct( pOwner )
	,aDelKeys( aDeletedKeys )
	,aInsKeys( aInsertedKeys )
	,pEditorCtrl( pOwner )
{
}

//-------------------------------------------------------------------------
SbaPrimKeyUndoAct::~SbaPrimKeyUndoAct()
{
}

//-------------------------------------------------------------------------
void SbaPrimKeyUndoAct::Undo()
{
	SbaTabEdRowList* pRowList = pEditorCtrl->GetRowList();
	SbaTabEdRow* pRow = NULL;

	//////////////////////////////////////////////////////////////////////
	// Die eingefuegten Keys loeschen
	for( long nIndex = aInsKeys.FirstSelected(); nIndex != SFX_ENDOFSELECTION; nIndex=aInsKeys.NextSelected() )
	{
		pRow =pRowList->GetObject( nIndex );
		pRow->SetPrimaryKey( FALSE );
	}

	//////////////////////////////////////////////////////////////////////
	// Die geloeschten Keys herstellen
	for( nIndex = aDelKeys.FirstSelected(); nIndex != SFX_ENDOFSELECTION; nIndex=aDelKeys.NextSelected() )
	{
		pRow =pRowList->GetObject( nIndex );
		pRow->SetPrimaryKey( TRUE );
	}

	pEditorCtrl->InvalidateHandleColumn();
	SbaTabEdUndoAct::Undo();
}

//-------------------------------------------------------------------------
void SbaPrimKeyUndoAct::Redo()
{
	SbaTabEdRowList* pRowList = pEditorCtrl->GetRowList();
	SbaTabEdRow* pRow = NULL;

	//////////////////////////////////////////////////////////////////////
	// Die geloeschten Keys loeschen
	for( long nIndex = aDelKeys.FirstSelected(); nIndex != SFX_ENDOFSELECTION; nIndex=aDelKeys.NextSelected() )
	{
		pRow =pRowList->GetObject( nIndex );
		pRow->SetPrimaryKey( FALSE );
	}

	//////////////////////////////////////////////////////////////////////
	// Die eingefuegten Keys herstellen
	for( nIndex = aInsKeys.FirstSelected(); nIndex != SFX_ENDOFSELECTION; nIndex=aInsKeys.NextSelected() )
	{
		pRow =pRowList->GetObject( nIndex );
		pRow->SetPrimaryKey( TRUE );
	}

	pEditorCtrl->InvalidateHandleColumn();
	SbaTabEdUndoAct::Redo();
}

//-------------------------------------------------------------------------
String SbaPrimKeyUndoAct::GetComment() const
{
	return String( SbaResId(STR_TABED_UNDO_PRIMKEY) );
}




