/*************************************************************************
 *
 *  $RCSfile: iiop_provider.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:29:57 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>

#include "iiop_bridge.hxx"

#include <osl/diagnose.h>

#include <uno/mapping.hxx>
#include <uno/environment.h>

#include <bridges/remote/remote.h>

using namespace ::rtl;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::bridge;

namespace remotebridges_iiop
{

	OInstanceProviderWrapper::OInstanceProviderWrapper(
		const Reference <XInstanceProvider > & rProvider,
		OIiopBridge * pBridgeCallback ) :
		m_rProvider( rProvider ),
		m_pBridgeCallback( pBridgeCallback ),
		m_nRef( 0 )
	{
		acquire = thisAcuire;
		release = thisRelease;
		getInstance = thisGetInstance;
	}

	OInstanceProviderWrapper::~OInstanceProviderWrapper()
	{
#ifdef VERBOSE
		printf( "Instance Provider Wrapper dies\n" );
#endif		
	}
	
	void OInstanceProviderWrapper::thisAcuire( remote_InstanceProvider *p )
	{
		OInstanceProviderWrapper * m = (OInstanceProviderWrapper *) p;
		osl_incrementInterlockedCount( &(m->m_nRef ) );
	}
	
	void OInstanceProviderWrapper::thisRelease( remote_InstanceProvider *p )
	{
		OInstanceProviderWrapper * m = ( OInstanceProviderWrapper *) p;
		if( ! osl_decrementInterlockedCount( &(m->m_nRef ) ) )
		{
			delete m;
		}
	}

	void OInstanceProviderWrapper::thisGetInstance(
		remote_InstanceProvider * pProvider ,
		uno_Environment *pEnvRemote,
		remote_Interface **ppRemoteI,
		rtl_uString *pInstanceName,
		typelib_InterfaceTypeDescription *pType
		)
	{
		OInstanceProviderWrapper * m = (OInstanceProviderWrapper *)pProvider;

		OSL_ASSERT( ppRemoteI );
		if( *ppRemoteI )
		{
			(*ppRemoteI)->release( *ppRemoteI );
			*ppRemoteI = 0;
		}

		if( OUString( pType->aBase.pTypeName ) ==
			getCppuType( (Reference<XInterface>*)0).getTypeName() )
		{
			try
			{
				Reference< XInterface > r = m->m_rProvider->getInstance(
					OUString( pInstanceName ) );

				OUString sCppuName( RTL_CONSTASCII_USTRINGPARAM( CPPU_CURRENT_LANGUAGE_BINDING_NAME ) );
				uno_Environment *pEnvThis = 0;
				uno_getEnvironment( &pEnvThis ,
									sCppuName.pData ,
									0 );
				
				Mapping map( pEnvThis , pEnvRemote );
				
				*ppRemoteI = (remote_Interface*) map.mapInterface (
					r.get(),
					getCppuType( (Reference< XInterface > *) 0 )
				);
				
				pEnvThis->release( pEnvThis );
				
				if( *ppRemoteI && m->m_pBridgeCallback )
				{
					m->m_pBridgeCallback->objectMappedSuccesfully();
					m->m_pBridgeCallback = 0;
				}
			}
			catch( ::com::sun::star::container::NoSuchElementException & )
			{
				// simply pass a null reference 
			}

		}
	}
}
