/*************************************************************************
 *
 *  $RCSfile: channel.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_CHANNEL_H_
#define _OSL_CHANNEL_H_

#ifndef _RTL_USTRING_H
#	include <rtl/ustring.h>
#endif

#ifndef _OSL_TYPES_H_
#	include <osl/types.h>
#endif

#ifndef _OSL_PIPE_H_
#	include <osl/pipe.h>
#endif

#ifndef _OSL_SOCKET_H_
#	include <osl/socket.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	osl_Channel_Type_Socket,
	osl_Channel_Type_Pipe,
	osl_Channel_TYPE_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslChannelType;

typedef union {
	oslPipe 	Pipe;
	oslSocket 	Socket;
} oslChannelResource;

typedef enum {
	osl_Channel_E_None,						/* no error */
	osl_Channel_E_NoManger,	 				/* channel manager not running */
	osl_Channel_E_InvalidName,	 			/* the resource name is invalid */
	osl_Channel_E_UnsupportedResourceClass,	/* the resource class of the channel is not supported */
	osl_Channel_E_Timeout,					/* the timeout was reached during sendind or accepting a channel */
	osl_Channel_E_ChannelInUse,				/* this channel broker connection is allready in use */
	osl_Channel_E_AuthenticationFailed,		/* the acceptor denies this resource because of the auth. data */
	osl_Channel_E_Unknown,					/* unknown error */
	osl_Channel_E_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslChannelError;

typedef void* oslChannelBroker;

typedef sal_Bool (SAL_CALL *CheckAuthentication_F)
(
	void* pData,
	rtl_uString *strUserIdent,
	rtl_uString *strPassword
);

sal_Bool SAL_CALL osl_startChannelBroker();
sal_Bool SAL_CALL osl_stopChannelBroker();

oslChannelBroker SAL_CALL osl_channelConnectBroker();
void SAL_CALL osl_channelDisconnectBroker(oslChannelBroker Broker);

oslChannelError SAL_CALL osl_channelProvide(oslChannelBroker		Broker,
											oslChannelResource		Resource, 
										   	oslChannelType			Type,	
											rtl_uString*            strResourceName,
											rtl_uString*            strUserIdent,
											rtl_uString*            strPassword,
										   	const TimeValue* 		pTimeout);

oslChannelError SAL_CALL osl_channelAccept(oslChannelBroker			Broker,
										   oslChannelResource*		pResource, 
										   oslChannelType* 			pType,	
										   rtl_uString*    		    strResourceName,
										   CheckAuthentication_F	checkAuthFunc,
										   void* 					pAuthFuncData,
										   const TimeValue* 		pTimeout);


#ifdef __cplusplus
}
#endif

#endif	/* _OSL_CHANNEL_H_ */

