/*************************************************************************
 *
 *  $RCSfile: daemon.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_DAEMON_H_
#define _OSL_DAEMON_H_

#ifndef _RTL_USTRING_H
#     include <rtl/ustring.h>
#endif

#ifndef _OSL_TYPES_H_
#	include <osl/types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define OSL_DAEMON_IDENTIFIER_LEN	64
#define OSL_DAEMON_NAME_LEN			64
#define OSL_DAEMON_MANUFACTOR_LEN	64
#define OSL_DAEMON_PATH_LEN			512
#define	OSL_DAEMON_DEPENDENCIES_LEN 1024
#define OSL_DAEMON_PROP_NAME_LEN	64
#define OSL_DAEMON_PROP_ERROR_LEN	64

/**
	Opaque data type for daemons in execution. As with all other osl-handles
	you can initialize and/or test it to/for 0.
*/
typedef void* oslDaemonHandle;

/** command codes for daemons.
*/
typedef enum
{
	osl_Daemon_CommandStart,
	osl_Daemon_CommandStop,
	osl_Daemon_CommandPause,
	osl_Daemon_CommandContinue,
	osl_Daemon_CommandShutdown,
	osl_Daemon_CommandUnknown,				/* don't use to set */
    osl_Daemon_Command_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslDaemonCommand;


/** status codes for daemons.
*/
typedef enum
{
	osl_Daemon_StatusStartPending,
	osl_Daemon_StatusStopPending,
	osl_Daemon_StatusPausePending,
	osl_Daemon_StatusRunning,
	osl_Daemon_StatusStopped,
	osl_Daemon_StatusPaused,
	osl_Daemon_StatusUnknown,				/* don't use to set */
    osl_Daemon_Status_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslDaemonStatus;

/** start up modes for daemons.
*/
typedef enum
{
	osl_Daemon_Automatic,
	osl_Daemon_OnDemand,
    osl_Daemon_StartupMode_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslDaemonStartupMode;

/** property types for daemons
*/
typedef enum
{
	osl_Daemon_PropertyType_Boolean,
	osl_Daemon_PropertyType_uLong,
	osl_Daemon_PropertyType_String,
	osl_Daemon_PropertyType_BooleanList,
	osl_Daemon_PropertyType_uLongList,
	osl_Daemon_PropertyType_StringList,
	osl_Daemon_PropertyType_Invalid,			/* don't use to set */
    osl_Daemon_PropertyType_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslDaemonPropertyType;

#ifdef SAL_W32
#	pragma pack(push, 8)
#elif defined(SAL_OS2)
#	pragma pack(1)
#endif

/** representation for a property value for daemons.
	string and list values have to be allocated and freed
	manually.
*/
typedef union
{
	sal_Bool 	 aBool;
	sal_uInt32	 aULong;
	sal_Unicode* aString;

	struct
	{
		sal_uInt32 Count;

		union
		{
			sal_Bool* 	  pBool;
			sal_uInt32*	  pULong;
			sal_Unicode** pString;
		} Data;
	} aList;

} oslDaemonPropertyValue;

/** representation for a property for daemons.
*/
typedef struct 
{
	sal_uInt32				Size;
	oslDaemonPropertyType 	Type;	
	rtl_uString	*			Name;
	oslDaemonPropertyValue	Value;		

} oslDaemonProperty;


/** all needed informations for a daemon.
	the identifier of the depend daemons are given as string list.
*/
typedef struct
{
	sal_uInt32				Size;
	rtl_uString *			Identifier;
	rtl_uString *			DisplayName;
	rtl_uString	*			Manufactor;
	rtl_uString	*			Path;
	sal_Unicode 			Dependencies[OSL_DAEMON_DEPENDENCIES_LEN];
	sal_uInt32 			   	Version;
	oslDaemonStartupMode 	Mode;
} oslDaemonInfo;

/** error codes returned by following functions
*/
typedef enum
{
	osl_Daemon_ManagerError_NoError,
	osl_Daemon_ManagerError_NotInstalled,
	osl_Daemon_ManagerError_AllreadyInstalled,
	osl_Daemon_ManagerError_AllreadyRunning,
	osl_Daemon_ManagerError_NotRunning,
	osl_Daemon_ManagerError_NotPaused,
	osl_Daemon_ManagerError_NoManager,
	osl_Daemon_ManagerError_Timeout,
	osl_Daemon_ManagerError_Security,
	osl_Daemon_ManagerError_CircularDep,
	osl_Daemon_ManagerError_ParamError,
	osl_Daemon_ManagerError_Unknown,
    osl_Daemon_ManagerError_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslDaemonManagerError;

/** callback functions to access daemon properties.
*/
typedef oslDaemonManagerError (SAL_CALL * oslPropertyGetCount) 		(void* pThis, sal_uInt32* pCount);
typedef oslDaemonManagerError (SAL_CALL * oslPropertyGetProperties)  	(void* pThis, oslDaemonProperty** ppProp, sal_uInt32* pCount);
typedef oslDaemonManagerError (SAL_CALL * oslPropertySetProperties)  	(void* pThis, oslDaemonProperty* pProp, sal_uInt32 count, sal_uInt32* pErrorText, rtl_uString** strError);

typedef struct 
{
	oslPropertyGetCount			fGetCount;
	oslPropertyGetProperties	fGetProperties;
	oslPropertySetProperties	fSetProperties;

} oslDaemonPropertyAccess;

#ifdef SAL_W32
#	pragma pack(pop)
#elif  defined(SAL_OS2)
#	pragma pack()
#endif

/** the function-ptr. representing the daemon control-function.
*/
typedef void (SAL_CALL * oslDaemonControl)(void*, oslDaemonCommand);

/** try to access the daemon manager.
	returns False, if the manager is not running.
*/
sal_Bool SAL_CALL osl_accessDaemonManager();

/** query the number of installed daemons.
*/
oslDaemonManagerError SAL_CALL osl_queryDaemonCount(sal_uInt32* pCount);

/** query the identifier of a daemon.
*/
oslDaemonManagerError SAL_CALL osl_queryDaemonIdentifier(sal_uInt32 index, rtl_uString** strIdentifier);

/** query if the daemon is installed.
*/
oslDaemonManagerError SAL_CALL osl_isDaemonInstalled(rtl_uString* strIdentifier, sal_Bool* pRet);

/** query the status of a daemon.
*/
oslDaemonManagerError SAL_CALL osl_queryDaemonStatus(rtl_uString* strIdentifier, oslDaemonStatus* pStatus);

/** query the informations of a daemon.
*/
oslDaemonManagerError SAL_CALL osl_queryDaemonInfo(rtl_uString* strIdentifier, oslDaemonInfo* pInfo);

/** Start the daemon.
*/
oslDaemonManagerError SAL_CALL osl_startDaemon(rtl_uString* strIdentifier);

/** Stop the daemon.
*/
oslDaemonManagerError SAL_CALL osl_stopDaemon(rtl_uString* strIdentifier);

/** Start the daemon.
*/
oslDaemonManagerError SAL_CALL osl_pauseDaemon(rtl_uString* strIdentifier);

/** Stop the daemon.
*/
oslDaemonManagerError SAL_CALL osl_continueDaemon(rtl_uString* strIdentifier);

/** install a new daemon witch given informations.
*/
oslDaemonManagerError SAL_CALL osl_installDaemon(const oslDaemonInfo* pInfo);

/** uninstall a installed daemon.
*/
oslDaemonManagerError SAL_CALL osl_uninstallDaemon(rtl_uString* strIdentifier);

/** prepare executing this process as the specified daemon.
	This functions returns immediately the handle for the daemon control,
	which has to be freed by osl_endDaemon() after execution.
	It doesn't execute it.
*/
oslDaemonManagerError SAL_CALL osl_createDaemon(rtl_uString*                strIdentifier,
                                                oslDaemonControl            pControl,
                                                void*                       pThisControl,
                                                oslDaemonPropertyAccess*    pPropertyAccess,
                                                void*                       pThisAccess,
                                                oslDaemonHandle*            pHandle);

/** start executing this process as the specified daemon.
*/
oslDaemonManagerError SAL_CALL osl_executeDaemon(oslDaemonHandle pHandle);

/** start executing this process as a set of daemons.
*/
oslDaemonManagerError SAL_CALL osl_executeDaemons(oslDaemonHandle* pHandle, sal_uInt32 numOfDaemons);

/** Set daemon status.
*/
oslDaemonManagerError SAL_CALL osl_setDaemonStatus(oslDaemonHandle Daemon, oslDaemonStatus Status);

/** stop executing this process as daemon and frees the handle.
*/
oslDaemonManagerError SAL_CALL osl_endDaemon(oslDaemonHandle handle);

/** Create an array of daemon properties.
	each property has to be initialized via osl_initDaemonProperty() before 
	using it. After using the properties they have to be freed via
	osl_destroyDaemonProperties().
*/
oslDaemonProperty* SAL_CALL osl_createDaemonProperties(sal_uInt32 count);

/** Destroy an array of daemon properties.
	if the properties hold dynamic allocated storage like string and list 
	values, they are freed to.
*/
void SAL_CALL osl_destroyDaemonProperties(oslDaemonProperty* list, sal_uInt32 count);

/** Initialize on daemon property with the given type and name.
	if type is an list type and listSize is greater 0 the list value is allocated.
*/
void SAL_CALL osl_initDaemonProperty(oslDaemonProperty*    pProp, 
                                     oslDaemonPropertyType type, 
                                     rtl_uString*          strName,
                                     sal_uInt32            listSize);

/** Get the count of properties of the denoted daemon.
*/
oslDaemonManagerError SAL_CALL osl_getDaemonPropertyCount(rtl_uString* strIdentifier, sal_uInt32* pCount);

/** Get the property of the denoted daemon.
	If this function succeed, a array of properties is allocated and the pointer 
	is returned by the out parameter ppProps. pCount returns the count of properties 
	which where returned.
*/
oslDaemonManagerError SAL_CALL osl_getDaemonProperties(rtl_uString*         strIdentifier, 
                                                       oslDaemonProperty**  ppProps, 
                                                       sal_uInt32*          pCount);

/** Set the properties to the denoted daemon.
	If the daemon daes not accept the properties, a detailed error message is written
	to the buffer errorBuffer.
*/
oslDaemonManagerError SAL_CALL osl_setDaemonProperties(rtl_uString*        strIdentifier, 
                                                       oslDaemonProperty*  pProps, 
                                                       sal_uInt32          count, 
                                                       sal_uInt32*         pErrorCode, 
                                                       rtl_uString**       strErrorText);


oslDaemonManagerError SAL_CALL osl_getDaemonProfileSections(rtl_uString*   strIdentifier, 
                                                            sal_Unicode*   buffer, 
                                                            sal_uInt32     bufferSize, 
                                                            sal_uInt32*    pRequiredSize);

oslDaemonManagerError SAL_CALL osl_getDaemonProfileEntries(rtl_uString* strIdentifier, 
                                                           rtl_uString* strSection, 
                                                           sal_Unicode* buffer, 
                                                           sal_uInt32   bufferSize,
                                                           sal_uInt32*  pRequiredSize);

oslDaemonManagerError SAL_CALL osl_readDaemonProfileString(rtl_uString*  strIdentifier, 
                                                           rtl_uString*  strSection, 
                                                           rtl_uString*  strEntry,
                                                           rtl_uString** strValue, 
                                                           rtl_uString*  strDefaultValue);

oslDaemonManagerError SAL_CALL osl_readDaemonProfileBool(rtl_uString* strIdentifier, 
                                                         rtl_uString* strSection, 
                                                         rtl_uString* strEntry,
                                                         sal_Bool*    pValue, 
                                                         sal_Bool     defaultValue);

oslDaemonManagerError SAL_CALL osl_readDaemonProfileIdent(rtl_uString* strIdentifier, 
                                                          rtl_uString* strSection, 
                                                          rtl_uString* strEntry,
                                                          sal_uInt32*  pValue, 
                                                          sal_uInt32   firstId, 
                                                          rtl_uString* strStrings[],
                                                          sal_uInt32   defaultValue);

oslDaemonManagerError SAL_CALL osl_writeDaemonProfileString(rtl_uString* strIdentifier, 
                                                            rtl_uString* strSection, 
                                                            rtl_uString* strEntry,
                                                            rtl_uString* strValue);

oslDaemonManagerError SAL_CALL osl_writeDaemonProfileBool(rtl_uString* strIdentifier, 
                                                          rtl_uString* strSection, 
                                                          rtl_uString* strEntry,
                                                          sal_Bool     value);

oslDaemonManagerError SAL_CALL osl_writeDaemonProfileIdent(rtl_uString* strIdentifier, 
                                                           rtl_uString* strSection, 
                                                           rtl_uString* strEntry,
                                                           sal_uInt32   value, 
                                                           sal_uInt32   firstId, 
                                                           rtl_uString* strStrings[]);

oslDaemonManagerError SAL_CALL osl_removeDaemonProfileEntry(rtl_uString* strIdentifier, 
                                                            rtl_uString* strSection, 
                                                            rtl_uString* strEntry);

oslDaemonManagerError SAL_CALL osl_queryDaemonChanges(TimeValue* pSince, sal_Bool* pBChanged);

#ifdef __cplusplus
}
#endif

#endif	/* _OSL_DAEMON_H_ */

