/*************************************************************************
 *
 *  $RCSfile: isocket.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_ISOCKET_H_
#define _OSL_ISOCKET_H_ "$Revision: 1.1.1.1 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _OSL_SOCKET_H_
#include <osl/socket.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/** oslSocketLevel
 */
#define osl_Socket_LevelSocketLayer (osl_Socket_LevelInvalid + 1)


/** Data types.
 */
typedef struct oslSocketDescriptor  oslSocketDescriptor;
typedef struct oslSocketMethods     oslSocketMethods;
typedef void                        oslSocketPrivate;


/** oslSocketDescriptor.
 */
struct oslSocketDescriptor
{
	const oslSocketMethods *m_methods;
	oslSocketPrivate       *m_private;

	oslSocketDescriptor    *m_lower;
	oslSocketDescriptor    *m_upper;
};


/** Create socket layer.
 */
oslSocketDescriptor* SAL_CALL osl_socket_createSocketLayer (
	const oslSocketMethods *methods);


/** Push socket layer.
 */
void SAL_CALL osl_socket_pushSocketLayer (
	oslSocketDescriptor *socket,
	oslSocketDescriptor *stack);


/** Pop socket layer.
 */
void SAL_CALL osl_socket_popSocketLayer (
	oslSocketDescriptor *socket);


/** Delete socket layer.
 */
void SAL_CALL osl_socket_deleteSocketLayer (
	oslSocketDescriptor *socket);


/** create.
 */
typedef oslSocketResult (SAL_CALL osl_socket_create_t) (
	oslSocketDescriptor *socket,
	oslAddrFamily        family,
	oslSocketType        type,
	oslProtocol          protocol);

oslSocketResult SAL_CALL osl_socket_create (
	oslSocketDescriptor *socket,
	oslAddrFamily        family,
	oslSocketType        type,
	oslProtocol          protocol);


/** delete.
 */
typedef void (SAL_CALL osl_socket_delete_t) (
	oslSocketDescriptor *socket);

void SAL_CALL osl_socket_delete (
	oslSocketDescriptor *socket);


/** getLastError.
 */
typedef oslSocketError (SAL_CALL osl_socket_getlasterror_t) (
	oslSocketDescriptor *socket);

oslSocketError SAL_CALL osl_socket_getLastError (
	oslSocketDescriptor *socket);


/** getOption.
 */
typedef oslSocketResult (SAL_CALL osl_socket_getoption_t) (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	void                 *buffer,
	sal_uInt32            length);

oslSocketResult SAL_CALL osl_socket_getOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	void                 *buffer,
	sal_uInt32            length);


/** setOption.
 */
typedef oslSocketResult (SAL_CALL osl_socket_setoption_t) (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	const void           *buffer,
	sal_uInt32            length);

oslSocketResult SAL_CALL osl_socket_setOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	const void           *buffer,
	sal_uInt32            length);


/** handshake.
 */
typedef oslSocketResult (SAL_CALL osl_socket_handshake_t) (
	oslSocketDescriptor *socket,
	oslSocketAddr        peeraddr);


/** accept.
 */
typedef oslSocketDescriptor* (SAL_CALL osl_socket_accept_t) (
	oslSocketDescriptor *socket,
	oslSocketAddr       *fromaddr);

oslSocketDescriptor* SAL_CALL osl_socket_accept (
	oslSocketDescriptor *socket,
	oslSocketAddr       *fromaddr);

oslSocketResult SAL_CALL osl_socket_accept_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        fromaddr);


/** bind.
 */
typedef oslSocketResult (SAL_CALL osl_socket_bind_t) (
    oslSocketDescriptor *socket,
	oslSocketAddr        addr);

oslSocketResult SAL_CALL osl_socket_bind (
	oslSocketDescriptor *socket,
	oslSocketAddr        addr);


/** connect.
 */
typedef oslSocketResult (SAL_CALL osl_socket_connect_t) (
    oslSocketDescriptor *socket,
	oslSocketAddr        toaddr);

oslSocketResult SAL_CALL osl_socket_connect (
	oslSocketDescriptor *socket,
	oslSocketAddr        toaddr);

oslSocketResult SAL_CALL osl_socket_connect_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        toaddr);


/** close.
 */
typedef void (SAL_CALL osl_socket_close_t) (
	oslSocketDescriptor *socket);

void SAL_CALL osl_socket_close (
	oslSocketDescriptor *socket);


/** listen.
 */
typedef oslSocketResult (SAL_CALL osl_socket_listen_t) (
	oslSocketDescriptor *socket,
	sal_Int32            pending);

oslSocketResult SAL_CALL osl_socket_listen (
	oslSocketDescriptor *socket,
	sal_Int32            pending);


/** recv.
 */
typedef sal_Int32 (SAL_CALL osl_socket_recv_t) (
	oslSocketDescriptor *socket,
	void                *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag);

sal_Int32 SAL_CALL osl_socket_recv (
	oslSocketDescriptor *socket,
	void                *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag);


/** send.
 */
typedef sal_Int32 (SAL_CALL osl_socket_send_t) (
	oslSocketDescriptor *socket,
	const void          *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag);

sal_Int32 SAL_CALL osl_socket_send (
	oslSocketDescriptor *socket,
	const void          *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag);


/** shutdown.
 */
typedef void (SAL_CALL osl_socket_shutdown_t) (
    oslSocketDescriptor *socket,
	oslSocketDirection   direction);

void SAL_CALL osl_socket_shutdown (
    oslSocketDescriptor *socket,
	oslSocketDirection   direction);


/** oslSocketMethods.
 */
struct oslSocketMethods
{
	osl_socket_create_t       *m_create;
	osl_socket_delete_t       *m_delete;

	osl_socket_getlasterror_t *m_getlasterror;

	osl_socket_getoption_t    *m_getoption;
	osl_socket_setoption_t    *m_setoption;

	osl_socket_accept_t       *m_accept;
	osl_socket_handshake_t    *m_accept_handshake;

	osl_socket_bind_t         *m_bind;

	osl_socket_connect_t      *m_connect;
	osl_socket_handshake_t    *m_connect_handshake;

	osl_socket_close_t        *m_close;
	osl_socket_listen_t       *m_listen;

	osl_socket_recv_t         *m_recv;
	osl_socket_send_t         *m_send;

	osl_socket_shutdown_t     *m_shutdown;
};


/** osl_socket_getDefaultSocketMethods.
 *
 *  @return The default socket layer methods.
 *  @see    osl_socket_createSocketLayer.
 */
const oslSocketMethods* SAL_CALL osl_socket_getDefaultSocketMethods (void);


/** osl_socket_getSocketMethods.
 *
 *  @return The bottom socket layer methods.
 *  @see    osl_socket_createSocketLayer.
 */
const oslSocketMethods* SAL_CALL osl_socket_getSocketMethods (void);


/*========================================================================
 *
 * The End.
 *
 *======================================================================*/
#ifdef __cplusplus
}
#endif

#endif /* !_OSL_ISOCKET_H_ */

