/*************************************************************************
 *
 *  $RCSfile: ports.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _OSL_PORTS_H_
#define _OSL_PORTS_H_

#ifndef _RTL_USTRING_H
#	include <rtl/ustring.h>
#endif

#ifndef _OSL_TYPES_H_
#	include <osl/types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void* oslPort;
typedef void* oslPortEnum;

typedef void (SAL_CALL * osl_Port_Callback)( sal_uInt32 nEventMask, oslPort hPort, sal_uInt32 nUserData );


#define OSL_PORT_TYPE_UNKNOWN		    ((sal_uInt32)0x00000000)
#define OSL_PORT_TYPE_RS232			    ((sal_uInt32)0x00000001)
#define OSL_PORT_TYPE_PARALLELPORT		((sal_uInt32)0x00000002)

/*
// Access Modes
*/
#define OSL_PORT_AC_NONE			0x0000	
	/** Specifies device query access to the object. 
		An application can query device attributes without accessing the device.*/
#define OSL_PORT_AC_READ			0x0001  
	/** Specifies read access to the object. Data can be read from the file and the file pointer can be moved.
		Combine with OSL_PORT_AC_WRITE for read-write access.  */
#define OSL_PORT_AC_WRITE           0x0002  
	/** Specifies write access to the object. Data can be written to the file and the file pointer 
		can be moved. Combine with OSL_PORT_AC_READ for read-write access.  */

/*
// Events
*/

#define OSL_PORT_EV_RXCHAR           0x0001
	  /** A character was received and placed in the input buffer. */
#define OSL_PORT_EV_RXFLAG           0x0002
	  /** The event character was received and placed in the 
		 input buffer. The event character is specified in the device's 
		 oslSerialPortControlBlock structure, which is applied to a serial 
		 port by using the oslSetPortState function. */
#define OSL_PORT_EV_TXEMPTY          0x0004  
	/** The last character in the output buffer was sent. */
#define OSL_PORT_EV_CTS              0x0008  
	/** The CTS (clear-to-send) signal changed state. */
#define OSL_PORT_EV_DSR              0x0010  
	/** The DSR (data-set-ready) signal changed state. */
#define OSL_PORT_EV_RLSD             0x0020  
	/** The RLSD (receive-line-signal-detect) signal changed state. */
#define OSL_PORT_EV_BREAK            0x0040  
	/** A break was detected on input. */
#define OSL_PORT_EV_ERR              0x0080  
	/** Line-status errors are OSL_PORT_E_FRAME, OSL_PORT_E_OVERRUN, and OSL_PORT_E_RXPARITY.
		To find the cause of the error, oslGetAndClearPortError should be called. */					 
#define OSL_PORT_EV_RING             0x0100  
	/** A ring indicator was detected. */
#define OSL_PORT_EV_PERR             0x0200  
	/** A printer error occured.  */
#define OSL_PORT_EV_RX80FULL         0x0400  
	/** The receive buffer is 80 percent full. */
#define OSL_PORT_EV_SHUTDOWN         0x1000  

/*
// Error Flags
*/

#define OSL_PORT_E_NONE				0x0000
	/** No Error occurred. */
#define OSL_PORT_E_RXOVER           0x0001
	/** An input buffer overflow has occurred. 
		There is either no room in the input buffer, 
		or a character was received after the end-of-file (EOF) character. */
#define OSL_PORT_E_OVERRUN          0x0002  
	/** A character-buffer overrun has occurred. The next character is lost.  */
#define OSL_PORT_E_RXPARITY         0x0004  // Receive Parity Error
	/** The hardware detected a parity error.  */
#define OSL_PORT_E_FRAME            0x0008
	/** The hardware detected a framing error.  */
#define OSL_PORT_E_BREAK            0x0010  
	/** The hardware detected a break condition.*/
#define OSL_PORT_E_TXFULL           0x0100  
	/** The application tried to transmit a character, but the output buffer was full.  */
#define OSL_PORT_E_PTO              0x0200  
	/** A time-out occurred on a parallel device. */
#define OSL_PORT_E_IOE              0x0400  
	/** An I/O error occurred during communications with the device.  */
#define OSL_PORT_E_DNS              0x0800  
	/** A parallel device is not selected. */
#define OSL_PORT_E_OOP              0x1000
	/** A parallel device signaled that it is out of paper. */
#define OSL_PORT_E_UNKNOWN          0x8000  

/*
// Port capabilities flags.
*/

#define OSL_PORT_PCF_DTRDSR        ((sal_uInt32)0x0001)	/** DTR (data-terminal-ready)/DSR (data-set-ready) supported */
#define OSL_PORT_PCF_INTTIMEOUTS   ((sal_uInt32)0x0002)	/**	Interval time-outs supported  */
#define OSL_PORT_PCF_RTSCTS        ((sal_uInt32)0x0002)	/**	(request-to-send)/CTS (clear-to-send) supported*/
#define OSL_PORT_PCF_RLSD          ((sal_uInt32)0x0004)	/**	(receive-line-signal-detect) supported  */
#define OSL_PORT_PCF_PARITY_CHECK  ((sal_uInt32)0x0008)	/**  Parity checking supported */
#define OSL_PORT_PCF_XONXOFF       ((sal_uInt32)0x0010)	/** flow control supported */
#define OSL_PORT_PCF_SETXCHAR      ((sal_uInt32)0x0020)	/**	Settable XON/XOFF supported */
#define OSL_PORT_PCF_TOTALTIMEOUTS ((sal_uInt32)0x0040)	/**	Total (elapsed) time-outs supported  */
#define OSL_PORT_PCF_SPECIALCHARS  ((sal_uInt32)0x0100)	/** Special character support provided */
#define OSL_PORT_PCF_16BITMODE     ((sal_uInt32)0x0200)	/** Special 16-bit mode supported */

/*
// settable port parameters.
*/

#define OSL_PORT_SP_PARITY         ((sal_uInt32)0x0001)		/** Parity */
#define OSL_PORT_SP_BAUD           ((sal_uInt32)0x0002)		/** Baud rate */
#define OSL_PORT_SP_DATABITS       ((sal_uInt32)0x0004)		/** Data bits */
#define OSL_PORT_SP_STOPBITS       ((sal_uInt32)0x0008)		/** Stopbits */
#define OSL_PORT_SP_HANDSHAKING    ((sal_uInt32)0x0010)		/** Handshaking (flow control) */
#define OSL_PORT_SP_PARITY_CHECK   ((sal_uInt32)0x0020)		/** Parity checking */
#define OSL_PORT_SP_RLSD           ((sal_uInt32)0x0040)		/**	(receive-line-signal-detect) */
#define OSL_PORT_SP_BINARY         ((sal_uInt32)0x0080)		/**	Binarymode settable*/


/*
// Settable Data Bits
*/
#define OSL_PORT_DATABITS_5        ((sal_uInt16)0x0001)	/** 5 data bits */
#define OSL_PORT_DATABITS_6        ((sal_uInt16)0x0002)	/** 6 data bits */
#define OSL_PORT_DATABITS_7        ((sal_uInt16)0x0004)	/** 7 data bits */
#define OSL_PORT_DATABITS_8        ((sal_uInt16)0x0008)	/** 8 data bits */
#define OSL_PORT_DATABITS_16       ((sal_uInt16)0x0010)	/** 16 data bits */
#define OSL_PORT_DATABITS_16X      ((sal_uInt16)0x0020)	/** Special wide path through serial hardware lines  */

/*
// Settable Stop and Parity bits.
*/

#define OSL_PORT_STOPBITS_10       ((sal_uInt16)0x0001)	/** 1 stop bit */
#define OSL_PORT_STOPBITS_15       ((sal_uInt16)0x0002)	/** 1.5 stop bits */
#define OSL_PORT_STOPBITS_20       ((sal_uInt16)0x0004)	/** 2 stop bits */

#define OSL_PORT_PARITY_NONE       ((sal_uInt16)0x0100)	/**	No parity */
#define OSL_PORT_PARITY_ODD        ((sal_uInt16)0x0200)	/** Odd parity */
#define OSL_PORT_PARITY_EVEN       ((sal_uInt16)0x0400)	/** Even parity */
#define OSL_PORT_PARITY_MARK       ((sal_uInt16)0x0800)	/** Mark parity */
#define OSL_PORT_PARITY_SPACE      ((sal_uInt16)0x1000)	/** Space parity */

/*
// DTR Control Flow Values.
*/
#define OSL_PORT_DTR_CONTROL_DISABLE    0x00
	/* Disables the DTR line when the device is opened and leaves it disabled. */
#define OSL_PORT_DTR_CONTROL_ENABLE     0x01
	/* Enables the DTR line when the device is opened and leaves it on. */
#define OSL_PORT_DTR_CONTROL_HANDSHAKE  0x02
	/* Enables DTR handshaking */

/*
// RTS Control Flow Values
*/
#define OSL_PORT_RTS_CONTROL_DISABLE    0x00
	/** Disables the RTS line when the device is opened and leaves it disabled.*/
#define OSL_PORT_RTS_CONTROL_ENABLE     0x01
	/** Enables the RTS line when the device is opened and leaves it on */
#define OSL_PORT_RTS_CONTROL_HANDSHAKE  0x02
	/** Enables RTS handshaking. */
#define OSL_PORT_RTS_CONTROL_TOGGLE     0x03
	/** Specifies that the RTS line will be high if bytes are available 
		for transmission. After all buffered bytes have been sent, the RTS line will be low. */

#define OSL_PORT_BAUD_075		   ((sal_uInt32)0x00000001)
#define OSL_PORT_BAUD_110          ((sal_uInt32)0x00000002)
#define OSL_PORT_BAUD_134_5        ((sal_uInt32)0x00000004)
#define OSL_PORT_BAUD_150          ((sal_uInt32)0x00000008)
#define OSL_PORT_BAUD_300          ((sal_uInt32)0x00000010)
#define OSL_PORT_BAUD_600          ((sal_uInt32)0x00000020)
#define OSL_PORT_BAUD_1200         ((sal_uInt32)0x00000040)
#define OSL_PORT_BAUD_1800         ((sal_uInt32)0x00000080)
#define OSL_PORT_BAUD_2400         ((sal_uInt32)0x00000100)
#define OSL_PORT_BAUD_4800         ((sal_uInt32)0x00000200)
#define OSL_PORT_BAUD_7200         ((sal_uInt32)0x00000400)
#define OSL_PORT_BAUD_9600         ((sal_uInt32)0x00000800)
#define OSL_PORT_BAUD_14400        ((sal_uInt32)0x00001000)
#define OSL_PORT_BAUD_19200        ((sal_uInt32)0x00002000)
#define OSL_PORT_BAUD_38400        ((sal_uInt32)0x00004000)
#define OSL_PORT_BAUD_56000        ((sal_uInt32)0x00008000)
#define OSL_PORT_BAUD_128000       ((sal_uInt32)0x00010000)
#define OSL_PORT_BAUD_115200       ((sal_uInt32)0x00020000)
#define OSL_PORT_BAUD_57600        ((sal_uInt32)0x00040000)
#define OSL_PORT_BAUD_USER         ((sal_uInt32)0x10000000)

/*
// Flags for oslPurgePort
// Specifies the action to take on oslPurge. 
// This parameter can be a combination of the following values:
*/

#define OSL_PORT_PURGE_TXABORT       0x0001  
	/** Terminates all outstanding overlapped write operations and  
		returns immediately, even if the write operations have not been completed. */
#define OSL_PORT_PURGE_RXABORT       0x0002
	/** Terminates all outstanding overlapped read operations and returns immediately, 
		even if the read operations have not been completed. */
#define OSL_PORT_PURGE_TXCLEAR       0x0004  
	/** Clears the output buffer (if the device driver has one). */
#define OSL_PORT_PURGE_RXCLEAR       0x0008  
	/** Clears the input buffer (if the device driver has one).  */

#ifdef SAL_W32
#	pragma pack(push, 8)
#elif defined(SAL_OS2)
#	pragma pack(1)
#endif

typedef struct osl_PortTimeouts
{
	sal_uInt32	Size;
	sal_uInt32 ReadIntervalTimeout;
		/** Specifies the maximum time, in milliseconds, allowed to elapse between the arrival 
			of two characters on the communications line. During a ReadFile operation, the time 
			period begins when the first character is received. If the interval between the arrival 
			of any two characters exceeds this amount, the ReadFile operation is completed and any 
			buffered data is returned. A value of zero indicates that interval time-outs are not used. 
			A value of MAXDWORD, combined with zero values for both the ReadTotalTimeoutConstant
			and ReadTotalTimeoutMultiplier members, specifies that the read operation is to return
			immediately with the characters that have already been received, even if no characters
			have been received. 
		*/
	sal_uInt32 ReadTotalTimeoutMultiplier; 
		/** Specifies the multiplier, in milliseconds, used to calculate the total 
			time-out period for read operations. For each read operation, this value is 
			multiplied by the requested number of bytes to be read. 
		*/
    sal_uInt32 ReadTotalTimeoutConstant;
		/** Specifies the constant, in milliseconds, used to calculate the total time-out 
			period for read operations. For each read operation, this value is added to the 
			product of the ReadTotalTimeoutMultiplier member and the requested number of bytes. 
			A value of zero for both the ReadTotalTimeoutMultiplier and ReadTotalTimeoutConstant
			members indicates that total time-outs are not used for read operations. 
		*/
	sal_uInt32 WriteTotalTimeoutMultiplier; 
		/** Specifies the multiplier, in milliseconds, used to calculate the total time-out 
			period for write operations. For each write operation, this value is multiplied by 
			the number of bytes to be written.
		*/
    sal_uInt32 WriteTotalTimeoutConstant; 
		/** Specifies the constant, in milliseconds, used to calculate the total time-out period
			for write operations. For each write operation, this value is added to the product of 
			the WriteTotalTimeoutMultiplier member and the number of bytes to be written. 
			A value of zero for both the WriteTotalTimeoutMultiplier and WriteTotalTimeoutConstant
			members indicates that total time-outs are not used for write operations. 
		*/

} oslPortTimeouts;


typedef struct osl_PortState
{
	sal_uInt32	Size;

    sal_Bool CTSHold;
		/** Specifies whether transmission is waiting for the CTS (clear-to-send) signal to be sent.
			If this member is TRUE, transmission is waiting. */
    sal_Bool DSRHold;
		/** Specifies whether transmission is waiting for the DSR (data-set-ready) 
			signal to be sent. If this member is TRUE, transmission is waiting. */
    sal_Bool RLSDHold;
		/**	Specifies whether transmission is waiting for the RLSD 
			(receive-line-signal-detect) signal to be sent. If this member is TRUE, transmission is waiting. */
    sal_Bool XOFFHold;
		/** Specifies whether transmission is waiting because the XOFF character was received. 
			If this member is TRUE, transmission is waiting. **/
    sal_Bool XOFFSent;
		/**	Specifies whether transmission is waiting because the XOFF character was transmitted. 
			If this member is TRUE, transmission is waiting. Transmission halts when the XOFF character is 
			transmitted to a system that takes the next character as XON, regardless of the actual character. */
	sal_Bool Txim;
		/** If this member is TRUE, there is a character queued for transmission that has come 
			to the communications device by way of the oslTransmitPortChar function. The communications 
			device transmits such a character ahead of other characters in the device's output buffer. 
		*/
    sal_Bool Eof;
		/**	Specifies whether the end-of-file (EOF) character has been received. If this member is TRUE, 
			the EOF character has been received. */
	sal_uInt32 BytesInQueue;
		/** Specifies the number of bytes received by the serial provider but not yet read by a receive operation. */
    sal_uInt32 BytesOutQueue;
		/**	Specifies the number of bytes of user data remaining to be transmitted for all write operations. */

} oslPortState;

typedef struct osl_PortControlBlock
{
	sal_uInt32	Size;

	sal_uInt32	PortType;

} oslPortControlBlock;

typedef struct osl_SerialPortControlBlock
{
	oslPortControlBlock PortControlBlock;

    sal_uInt32 BaudRate;				
		/** Specifies the baud rate at which the communications device operates.
			Note that this is NOT an an OSL_PORT_BAUD_xxx constant, but the real value like 9600 baud.
		*/
    sal_Bool Binary;
		/** Specifies whether binary mode is enabled. For Win32 this member can only be TRUE */
	sal_Bool Parity;
		/** Specifies whether parity checking is enabled. If this member is TRUE, parity checking 
			is performed and errors are reported. */
    sal_Bool OutxCTSFlow;
		/** Specifies whether the CTS (clear-to-send) signal is monitored for output flow control. 
			If this member is TRUE and CTS is turned off, output is suspended until CTS is sent again. */
    sal_Bool OutxDSRFlow;
		/** Specifies whether the DSR (data-set-ready) signal is monitored for output flow control. 
			If this member is TRUE and DSR is turned off, output is suspended until DSR is sent again. */
    sal_uInt32 DTRControl;
		/** Specifies the DTR (data-terminal-ready) flow control. 
			This member can be one of OSL_PORT_DTR_CONTROL_XXX constants. */
    sal_Bool DSRSensitivity;
		/** Specifies whether the communications driver is sensitive to the state of the DSR signal. 
			If this member is TRUE, the driver ignores any bytes received, unless the DSR modem input line is high.*/ 
    sal_Bool TXContinueOnXoff;
		/** Specifies whether transmission stops when the input buffer is full and the driver has 
			transmitted the XoffChar character. If this member is TRUE, transmission continues after 
			the input buffer has come within XoffLim bytes of being full and the driver has transmitted 
			the XoffChar character to stop receiving bytes. If this member is FALSE, transmission does 
			not continue until the input buffer is within XonLim bytes of being empty and the driver has 
			transmitted the XonChar character to resume reception. */

    sal_Bool OutX;
		/** Specifies whether XON/XOFF flow control is used during transmission. If this member is TRUE,
			transmission stops when the XoffChar character is received and starts again when the XonChar 
			character is received. */
    sal_Bool InX;
		/** Specifies whether XON/XOFF flow control is used during reception. If this member is TRUE, 
			the XoffChar character is sent when the input buffer comes within XoffLim bytes of being full, 
			and the XonChar character is sent when the input buffer comes within XonLim bytes of being empty. */
    sal_Bool UseErrorChar;
		/** Specifies whether bytes received with parity errors are replaced with the character 
			specified by the ErrorChar member. If this member is TRUE and the 
			Parity member is TRUE, replacement occurs. */
    sal_Bool Null;
		/** Specifies whether null bytes are discarded. If this member is TRUE, 
			null bytes are discarded when received. */
    sal_uInt32 RTSControl;	
		/** Specifies the RTS (request-to-send) flow control. 
			If this value is zero, the default is OSL_PORT_RTS_CONTROL_HANDSHAKE. 
			This member can be one of OSL_PORT_RTS_XXX */
    sal_Bool AbortOnError;
		/** Specifies whether read and write operations are terminated if an error occurs.
			If this member is TRUE, the driver terminates all read and write operations with an error 
			status if an error occurs. The driver will not accept any further communications operations 
			until the application has acknowledged the error by calling the ClearCommError function. */
    sal_uInt16 XonLim;
		/** Specifies the minimum number of bytes allowed in the input buffer before the XON character is sent. */
    sal_uInt16 XoffLim;
		/** Specifies the maximum number of bytes allowed in the input buffer before the XOFF character is sent. 
			The maximum number of bytes allowed is calculated by subtracting this value from the size, in bytes, 
			of the input buffer. */
    sal_uInt8 ByteSize;
		/** Specifies the number of bits in the bytes transmitted and received.
			Remark: The number of data bits must be 5 to 8 bits. The use of 5 data bits with 2 stop 
					bits is an invalid combination, as is 6, 7, or 8 data bits with 1.5 stop bits. 

		*/
    sal_uInt16 ParityMode;
		/** Specifies the parity scheme to be used. This member can be one of OSL_PORT_PARITY_XXX constants. */
    sal_uInt16 StopBits;
		/** Specifies the number of stop bits to be used.  
			Remark: The number of data bits must be 5 to 8 bits. The use of 5 data bits with 2 stop 
					bits is an invalid combination, as is 6, 7, or 8 data bits with 1.5 stop bits. 
		*/
    sal_Char XonChar;
		/** Specifies the value of the XON character for both transmission and reception. */
    sal_Char XoffChar;
		/** Specifies the value of the XOFF character for both transmission and reception. */
    sal_Char ErrorChar;
		/** Specifies the value of the character used to replace bytes received with a parity error. */
    sal_Char EofChar;
		/** Specifies the value of the character used to signal the end of data. */
    sal_Char EvtChar;
		/** Specifies the value of the character used to signal an event. */
    void* pData;
		/** Reserved. Do not use */

} oslSerialPortControlBlock; 


typedef struct osl_PortProperties
 { 
	sal_uInt32 Size;

    sal_uInt32 MaxTxQueue;
		/** Specifies the maximum size, in bytes, of the driver's internal output buffer. 
			A value of zero indicates that no maximum value is imposed by the serial provider. */
    sal_uInt32 MaxRxQueue;        /* max Rx bufsize, in bytes */
		/** Specifies the maximum size, in bytes, of the driver's internal input buffer. 
			A value of zero indicates that no maximum value is imposed by the serial provider. */
    sal_uInt32 MaxBaud;           
		/** Specifies the maximum allowable baud rate, in bits per second (bps).
			This member can be one of OSL_PORT_BAUDXXX constants. */
    sal_uInt32 PortType;
		/** Specifies the specific communications port type.
			This member can be one of OSL_PORT_TYPE_XXX constants. */
    sal_uInt32 PortCapabilities;  
		/** Specifies a bitmask indicating the capabilities offered by the port.
			This member can be one of OSL_PORT_PCF_XXX constans. */
    sal_uInt32 SettableParams;
		/** Specifies a bitmask indicating the communications parameter that can be changed. 
			This member can be one of OSL_PORT_SP_XXX constants. */
    sal_uInt32 SettableBaud; 
		/** Specifies a bitmask indicating the baud rates that can be used.
			For values, see the dwMaxBaud member. */
    sal_uInt16  SettableData;       
		/** Specifies a bitmask indicating the number of data bits that can be set.
			This member can be one of OSL_PORT_DATABITS_XXX constants. */
    sal_uInt16  SettableStopParity; /* stop bits/parity allowed */
		/** Specifies a bitmask indicating the stop bit and parity settings that can be selected.
			This member can be one of OSL_PORT_STOPBITS_XXX OSL_PORT_PARITY_XXX or constants. */
    sal_uInt32 CurrentTxQueue;
		/**	Specifies the size, in bytes, of the driver's internal output buffer. 
			A value of zero indicates that the value is unavailable. */
    sal_uInt32 CurrentRxQueue;    
		/** Specifies the size, in bytes, of the driver's internal input buffer. 
			A value of zero indicates that the value is unavailable. */

} oslPortProperties; 

#ifdef SAL_W32
#	pragma pack(pop)
#elif defined(SAL_OS2)
#	pragma pack()
#endif

typedef enum
{
	osl_Port_E_None,			/** No Error */
	osl_Port_E_NotAvailable,
	osl_Port_E_PortNotFound,
	osl_Port_E_InvalidHandle,
	osl_Port_E_DifferentStructSize,
	osl_Port_E_InvalidType,
	osl_Port_E_InvalidArgument,
	osl_Port_E_NoMemory,		/** No system memory available */
	osl_Port_E_Unknown,			/** An unmapped Error occurred. 
									This is allways the last entry in enum oslPortError. */
	osl_Port_E_FORCE_EQUAL_SIZE = SAL_MAX_ENUM
} oslPortError;


/*********************************************************

  port functions

**********************************************************/

/** @return a port enumeration structure.
*/
oslPortEnum SAL_CALL osl_openPortEnum();

/** Resets a enumeration pointer to the start pos.

    @param portEnum [in] a port enumeration opened with osl_openPortEnum.
*/
void SAL_CALL osl_resetPortEnum(oslPortEnum portEnum);


/** Returns the next port name in a enumeration.

    @param portEnum [in] a port enumeration opened with osl_openPortEnum.
    @param strPortName [out] the string that receives the port name.
    @return sal_False if the end of the list is already reached, sal_True otherwise.
*/
sal_Bool SAL_CALL osl_getNextPortName( oslPortEnum portEnum, rtl_uString **strPortName );

/** Release the enumeration data structure

    @param portEnum [in] a port enumeration opened with osl_openPortEnum.
 */
void SAL_CALL osl_closePortEnum( oslPortEnum portEnum );

/**
	The getPortTypeByName function returns the port type of the port with the name pName.
	The return value is one of the OSL_PORT_TYPE_XXX constants.

*/
sal_uInt32 SAL_CALL osl_getPortTypeByName( rtl_uString *strName );

/** The openPort function opens a port for a connection.
	@param strPortName [in]
		The name of the port, like COM1 (win) or /dev/term/a (unx).
	@param nAccessMode [in]
		Specifies the type of access to the object. An application can obtain read access,
		write access, read-write access, or device query access. This parameter can be any
		combination of the OSL_PORT_AC_XXX constants.
	@param hPort [out]
		If the requested port is open, hPort is a valid Porthandle. Otherwise the handle is NULL.
	@return Error value.
*/
oslPortError SAL_CALL osl_openPort( rtl_uString *strPortName, sal_uInt32 nAccessMode, oslPort *hPort );

/** 
	The closePort function closes an openPort and frees all resources.
*/
oslPortError SAL_CALL osl_closePort( oslPort hPort );

/** The osl_getPortProperties function fills a buffer with information about the 
	communications properties for a specified communications device. 
	
	 @param hPortProp [out]
		Structure of a oslPortProperties filled with properties for a specified communications device.

*/
oslPortError SAL_CALL osl_getPortProperties( oslPort hPort, oslPortProperties *hPortProp );

oslPortError SAL_CALL osl_setEventHandler( oslPort hPort, osl_Port_Callback aCallback, sal_uInt32 nUserData );
oslPortError SAL_CALL osl_removeEventHandler( oslPort hPort );


/** The osl_clearPortBreak function restores character transmission for a specified communications 
	device and places the transmission line in a nonbreak state.
*/
oslPortError SAL_CALL osl_clearPortBreak( oslPort hPort );

/** The osl_setPortBreak function suspends character transmission for a specified communications 
	device and places the transmission line in a break state until the ClearCommBreak function is called.
*/
oslPortError SAL_CALL osl_setPortBreak( oslPort hPort );


/** The GetCommState function fills in a port-control block (a oslPortControlBlock structure) 
	with the current control settings for a specified communications port. 
*/
oslPortError SAL_CALL osl_getPortControlBlock( oslPort hPort, oslPortControlBlock *pPCB );

/** The osl_getPortState function configures a communications device according to the specifications 
	in a port-control block (a oslPortControlBlock structure). The function reinitializes all hardware and control 
	settings, but it does not empty output or input queues. 
*/
oslPortError SAL_CALL osl_setPortControlBlock( oslPort hPort, const oslPortControlBlock *pPCB );

/** The getAndClearPortError retrieves information about a communications error and reports the 
	current status of a communications device. The function is called when a communications error occurs, 
	and it clears the device's error flag to enable additional input and output (I/O) operations. 
	
	@param pErrorValueMask [out]
		Specifies the error that occurred. This parameter can be a combination of 
		the OSL_PORT_E_XXX constants.

	@param pPortState	[out]
		Pointer to a oslPortState structure in which the device's status information is returned.
		If pPortState is NULL, no status information is returned. 

*/
oslPortError SAL_CALL osl_getAndClearPortError( oslPort hPort, sal_uInt32* pErrorValueMask, oslPortState *pPortState );

/** The purgePort function can discard all characters from the output or input 
	buffer of a specified communications resource. It can also terminate pending 
	read or write operations on the resource. 

	@param nFlags [in]
		Specifies the action to take. This parameter can be a combination of 
		the OSL_PORT_PURGE_XXX constants.
*/
oslPortError SAL_CALL osl_purgePort( oslPort hPort, sal_uInt32 nFlags );


/** The osl_getPortEventMask function specifies a set of events to be monitored for a communications device. 
*/
oslPortError SAL_CALL osl_getPortEventMask( oslPort hPort, sal_uInt32 *nEventMask );

/** The osl_getPortEventMask function specifies a set of events to be monitored for a communications device. 
*/
oslPortError SAL_CALL osl_setPortEventMask( oslPort hPort, sal_uInt32 nEventMask );

/** The TransmitPortChar function transmits a specified character ahead of 
	any pending data in the output buffer of the specified communications device.
	This function is useful for sending an interrupt character (such as a ctrl+c) to a host system. 
 */
oslPortError SAL_CALL osl_transmitPortChar( oslPort hPort, sal_Char cChar );

/** The GetPortTimeout function returned the current time-out parameters for all read 
	and write operations on a specified communications device. 
*/
oslPortError SAL_CALL osl_getPortTimeout( oslPort hPort, oslPortTimeouts *pPortTimeouts );


/** The SetPortTimeout function set the time-out parameters for all read 
	and write operations on a specified communications device. 
*/
oslPortError SAL_CALL osl_setPortTimeout( oslPort hPort, const oslPortTimeouts* pPortTimeouts );


/** The function sendToPort tries to send the contents of pBuffer synchronous to the port.
	The function will block until all bytes are transmitted, or if a timeout is set
	(with setPortTimeout) until the timeout has reached. If no timeout is set, the function
	may be block forever.

	@return The number of bytes written.
*/
sal_uInt32 SAL_CALL osl_sendToPort( oslPort hPort, void* pBuffer, sal_uInt32 nNumberOfBytesToWrite );

/** The function receiveFromPort tries to fill the contents of pBuffer synchronous from the port.
	The function will block until nNumberOfBytesToRead bytes received, or if a timeout is set
	(with setPortTimeout) until the timeout has reached. If no timeout is set, the function
	may be block forever.

	@return The number of bytes received.
*/
sal_uInt32 SAL_CALL osl_receiveFromPort( oslPort hPort, void* pBuffer, sal_uInt32 nNumberOfBytesToRead );


#ifdef __cplusplus
}
#endif

#endif	/* _OSL_PORT_ */

      
/*************************************************************************
*
*    $Log: ports.h,v $
*    Revision 1.1.1.1  2000/09/18 15:17:13  hr
*    initial import
*
*    Revision 1.6  2000/09/18 14:28:49  willem.vandorp
*    OpenOffice header added.
*
*    Revision 1.5  2000/05/29 16:45:16  hro
*    SRC591: Explicite SAL_CALL calling convention
*
*    Revision 1.4  2000/05/17 14:30:14  mfe
*    comments upgraded
*
*    Revision 1.3  2000/03/16 16:43:52  obr
*    Unicode API changes
*
*    Revision 1.2  1999/10/27 15:02:08  mfe
*    Change of Copyright, removed compiler warnings, code clean up, ...
*
*    Revision 1.1  1999/08/05 10:18:19  jsc
*    verschoben aus osl
*
*    Revision 1.4  1999/01/20 18:53:40  jsc
*    #61011# Typumstellung
*
*    Revision 1.3  1998/12/14 14:08:04  jru
*    #60220# aenderung an SerialPortDeviceBlock
*
*    Revision 1.1  1998/12/08 15:55:20  jru
*    #60220# initial revision
*
*************************************************************************/

