/*************************************************************************
 *
 *  $RCSfile: char.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _RTL_CHAR_HXX_
#define _RTL_CHAR_HXX_

#ifndef _RTL_CHAR_H_
#include <rtl/char.h>
#endif
#ifndef _RTL_MACROS_HXX_
#include <rtl/macros.hxx>
#endif

#ifdef __cplusplus

#ifdef _USE_NAMESPACE
namespace rtl
{     
#endif

/**
 * The Character class wraps a value of the primitive type <code>char</code> 
 * in an object. An object of type <code>Character</code> contains a 
 * single field whose type is <code>char</code>. 
 * <p>
 * In addition, this class provides several methods for determining 
 * the type of a character and converting characters from uppercase 
 * to lowercase and vice versa. 
 * <p>
 * Many of the methods of class <code>Character</code> are defined 
 * in terms of a "Unicode attribute table" that specifies 
 * a name for every defined Unicode code point. The table also 
 * includes other attributes, such as a decimal value, an uppercase 
 * equivalent, a lowercase equivalent, and/or a titlecase equivalent. 
 * The Unicode attribute table is available on the World Wide Web as 
 * the file:
 * <ul><code>
 *   ftp://unicode.org/pub/MappingTables/UnicodeData1.1.5.txt
 * </code></ul>
 * <p>
 * For a more detailed specification of the <code>Character</code> 
 * class, one that encompasses the exact behavior of methods such as 
 * <code>isDigit</code>, <code>isLetter</code>, 
 * <code>isLowerCase</code>, and <code>isUpperCase</code> over the 
 * full range of Unicode values, see Gosling, Joy, and Steele, <i>The 
 * Java Language Specification</i>. 
 *
 * @author  Lee Boynton
 * @author  Guy Steele
 * @author  Akira Tanaka
 * @version 1.42 01/30/97
 * @since   JDK1.0
 */
class OCharacter
{
public:
/**
 * Returns a value indicating a character category.
 *
 * @param   ch      the character to be tested.
 * @return  a value of type int, the character category.
 * @see     java.lang.Character#COMBINING_SPACING_MARK
 * @see     java.lang.Character#CONNECTOR_PUNCTUATION
 * @see     java.lang.Character#CONTROL
 * @see     java.lang.Character#CURRENCY_SYMBOL
 * @see     java.lang.Character#DASH_PUNCTUATION
 * @see     java.lang.Character#DECIMAL_DIGIT_NUMBER
 * @see     java.lang.Character#ENCLOSING_MARK
 * @see     java.lang.Character#END_PUNCTUATION
 * @see     java.lang.Character#FORMAT
 * @see     java.lang.Character#LETTER_NUMBER
 * @see     java.lang.Character#LINE_SEPARATOR
 * @see     java.lang.Character#LOWERCASE_LETTER
 * @see     java.lang.Character#MATH_SYMBOL
 * @see     java.lang.Character#MODIFIER_LETTER
 * @see     java.lang.Character#MODIFIER_SYMBOL
 * @see     java.lang.Character#NON_SPACING_MARK
 * @see     java.lang.Character#OTHER_LETTER
 * @see     java.lang.Character#OTHER_NUMBER
 * @see     java.lang.Character#OTHER_PUNCTUATION
 * @see     java.lang.Character#OTHER_SYMBOL
 * @see     java.lang.Character#PARAGRAPH_SEPARATOR
 * @see     java.lang.Character#PRIVATE_USE
 * @see     java.lang.Character#SPACE_SEPARATOR
 * @see     java.lang.Character#START_PUNCTUATION
 * @see     java.lang.Character#SURROGATE
 * @see     java.lang.Character#TITLECASE_LETTER
 * @see     java.lang.Character#UNASSIGNED
 * @see     java.lang.Character#UPPERCASE_LETTER
 * @since   JDK1.1
 */
static sal_Int32 getType(sal_Unicode ch) { return RTL_CHAR_GETTYPE(ch); }

/**
 * Determines if the specified character is a lowercase character. 
 * A character is lowercase if it is not in the range 
 * <code>'&#92;u2000'</code> through <code>'&#92;u2FFF'</code>, the Unicode 
 * attribute table does not specify a mapping to lowercase for the 
 * character, and at least one of the following is true: 
 * <ul>
 * <li>The attribute table specifies a mapping to uppercase for the 
 *     character. 
 * <li>The name for the character contains the words "<code>SMALL 
 *     LETTER</code>". 
 * <li>The name for the character contains the words "<code>SMALL 
 *     LIGATURE</code>". 
 * </ul>
 * <p> A character is considered to be lowercase if and only if
 * it is specified to be lowercase by the Unicode 2.0 standard
 * (category "Ll" in the Unicode specification data file).
 * <p>
 * Of the ISO-LATIN-1 characters (character codes 0x0000 through 0x00FF),
 * the following are lowercase:
 * <p><blockquote><pre>
 * a b c d e f g h i j k l m n o p q r s t u v w x y z
 * &#92;u00DF &#92;u00E0 &#92;u00E1 &#92;u00E2 &#92;u00E3 &#92;u00E4 &#92;u00E5 &#92;u00E6 &#92;u00E7
 * &#92;u00E8 &#92;u00E9 &#92;u00EA &#92;u00EB &#92;u00EC &#92;u00ED &#92;u00EE &#92;u00EF &#92;u00F0
 * &#92;u00F1 &#92;u00F2 &#92;u00F3 &#92;u00F4 &#92;u00F5 &#92;u00F6 &#92;u00F8 &#92;u00F9 &#92;u00FA
 * &#92;u00FB &#92;u00FC &#92;u00FD &#92;u00FE &#92;u00FF
 * </pre></blockquote>
 * <p> Many other Unicode characters are lowercase, too.
 *
 * @param   ch   the character to be tested.
 * @return  <code>true</code> if the character is lowercase;
 *          <code>false</code> otherwise.
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isTitleCase(char)
 * @see     java.lang.Character#toLowerCase(char)
 * @since   JDK1.0
 */
static sal_Bool isLowerCase( sal_Unicode ch ) { return RTL_CHAR_ISLOWERCASE(ch); }

/**
 * Determines if the specified character is an uppercase character. 
 * A character is uppercase if it is not in the range 
 * <code>'&#92;u2000'</code> through <code>'&#92;u2FFF'</code>, the Unicode 
 * attribute table does not specify a mapping to uppercase for the 
 * character, and at least one of the following is true: 
 * <ul>
 * <li>The attribute table specifies a mapping to lowercase for the
 *     character. 
 * <li>The name for the character contains the words 
 *     "<code>CAPITAL LETTER</code>".
 * <li>The name for the character contains the words
 *     "<code>CAPITAL LIGATURE</code>".
 * </ul>
 * <p>
 * Of the ISO-LATIN-1 characters (character codes 0x0000 through 0x00FF),
 * the following are uppercase:
 * <p><blockquote><pre>
 * A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
 * &#92;u00C0 &#92;u00C1 &#92;u00C2 &#92;u00C3 &#92;u00C4 &#92;u00C5 &#92;u00C6 &#92;u00C7
 * &#92;u00C8 &#92;u00C9 &#92;u00CA &#92;u00CB &#92;u00CC &#92;u00CD &#92;u00CE &#92;u00CF &#92;u00D0
 * &#92;u00D1 &#92;u00D2 &#92;u00D3 &#92;u00D4 &#92;u00D5 &#92;u00D6 &#92;u00D8 &#92;u00D9 &#92;u00DA
 * &#92;u00DB &#92;u00DC &#92;u00DD &#92;u00DE
 * </pre></blockquote>
 * <p> Many other Unicode characters are uppercase, too.
 *
 * @param   ch   the character to be tested.
 * @return  <code>true</code> if the character is uppercase;
 *          <code>false</code> otherwise.
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isTitleCase(char)
 * @see     java.lang.Character#toUpperCase(char)
 * @since   1.0
 */
static sal_Bool isUpperCase(sal_Unicode ch) { return RTL_CHAR_ISUPPERCASE( ch ); }

/**
 * Determines if the specified character is a titlecase character.
 * A character is considered to be titlecase if and only if
 * it is specified to be titlecase by the Unicode 2.0 standard
 * (category "Lt" in the Unicode specification data file).
 * <p>
 * The printed representations of four Unicode characters look like 
 * pairs of Latin letters. For example, there is an uppercase letter 
 * that looks like "LJ" and has a corresponding lowercase letter that 
 * looks like "lj". A third form, which looks like "Lj", 
 * is the appropriate form to use when rendering a word in lowercase 
 * with initial capitals, as for a book title.
 * <p>
 * These are the Unicode characters for which this method returns 
 * <code>true</code>: 
 * <ul>
 * <li><code>LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON</code> 
 * <li><code>LATIN CAPITAL LETTER L WITH SMALL LETTER J</code> 
 * <li><code>LATIN CAPITAL LETTER N WITH SMALL LETTER J</code> 
 * <li><code>LATIN CAPITAL LETTER D WITH SMALL LETTER Z</code> 
 * </ul>
 *
 * @param   ch   the character to be tested.
 * @return  <code>true</code> if the character is titlecase;
 *          <code>false</code> otherwise.
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isUpperCase(char)
 * @see     java.lang.Character#toTitleCase(char)
 * @since   JDK1.0.2
 */
static sal_Bool isTitleCase(sal_Unicode ch) { return RTL_CHAR_ISTITLECASE(ch); }

/**
 * Determines if the specified character is a digit.
 * A character is considered to be a digit if it is not in the range 
 * <code>'&#92;u2000'&nbsp;&lt;=&nbsp;ch&nbsp;&lt;=&nbsp;'&#92;u2FFF'</code>
 * and its Unicode name contains the word 
 * "<code>DIGIT</code>". For a more complete 
 * specification that encompasses all Unicode characters that are 
 * defined as digits, see Gosling, Joy, and Steele, <i>The Java 
 * Language Specification</i>.
 * <p>
 * These are the ranges of Unicode characters that are considered digits:
 * <table>
 * <tr><td>0x0030 through 0x0039</td>
 *                        <td>ISO-LATIN-1 digits ('0' through '9')</td></tr>
 * <tr><td>0x0660 through 0x0669</td>  <td>Arabic-Indic digits</td></tr>
 * <tr><td>0x06F0 through 0x06F9</td>
 *                                <td>Extended Arabic-Indic digits</td></tr>
 * <tr><td>0x0966 through 0x096F</td>  <td>Devanagari digits</td></tr>
 * <tr><td>0x09E6 through 0x09EF</td>  <td>Bengali digits</td></tr>
 * <tr><td>0x0A66 through 0x0A6F</td>  <td>Gurmukhi digits</td></tr>
 * <tr><td>0x0AE6 through 0x0AEF</td>  <td>Gujarati digits</td></tr>
 * <tr><td>0x0B66 through 0x0B6F</td>  <td>Oriya digits</td></tr>
 * <tr><td>0x0BE7 through 0x0BEF</td>  <td>Tamil digits</td></tr>
 * <tr><td>0x0C66 through 0x0C6F</td>  <td>Telugu digits</td></tr>
 * <tr><td>0x0CE6 through 0x0CEF</td>  <td>Kannada digits</td></tr>
 * <tr><td>0x0D66 through 0x0D6F</td>  <td>Malayalam digits</td></tr>
 * <tr><td>0x0E50 through 0x0E59</td>  <td>Thai digits</td></tr>
 * <tr><td>0x0ED0 through 0x0ED9</td>  <td>Lao digits</td></tr>
 * <tr><td>0x0F20 through 0x0F29</td>  <td>Tibetan digits</td></tr>
 * <tr><td>0xFF10 through 0xFF19</td>  <td>Fullwidth digits</td></tr>
 * </table>
 *
 * @param   ch   the character to be tested.
 * @return  <code>true</code> if the character is a digit;
 *          <code>false</code> otherwise.
 * @see     java.lang.Character#digit(char, int)
 * @see     java.lang.Character#forDigit(int, int)
 * @since   JDK1.0
 */
static sal_Bool isDigit(sal_Unicode ch){ return RTL_CHAR_ISDIGIT(ch); }

/**
 * Determines if a character has a defined meaning in Unicode.
 * A character is defined if at least one of the following is true: 
 * <ul>
 * <li>It has an entry in the Unicode attribute table. 
 * <li>Its value is in the range 
 *     <code>
 *     '&#92;u3040'&nbsp;&lt;=&nbsp;ch&nbsp;&lt;=&nbsp;'&#92;u9FA5'</code>. 
 * <li>Its value is in the range 
 *     <code>
 *     '&#92;uF900'&nbsp;&lt;=&nbsp;ch&nbsp;&lt;=&nbsp;'&#92;uFA2D'</code>. 
 * </ul>
 * 
 * @param   ch   the character to be tested
 * @return  <code>true</code> if the character has a defined meaning
 *          in Unicode; <code>false</code> otherwise.
 * @see     java.lang.Character#isDigit(char)
 * @see     java.lang.Character#isLetter(char)
 * @see     java.lang.Character#isLetterOrDigit(char)
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isTitleCase(char)
 * @see     java.lang.Character#isUpperCase(char)
 * @since   JDK1.0.2
 */
static sal_Bool isDefined(sal_Unicode ch){ return RTL_CHAR_ISDEFINED(ch); }

/**
 * Determines if the specified character is a letter. For a 
 * more complete specification that encompasses all Unicode 
 * characters, see Gosling, Joy, and Steele, <i>The Java Language 
 * Specification</i>. 
 *
 * <p> A character is considered to be a letter if and only if
 * it is specified to be a letter by the Unicode 2.0 standard
 * (category "Lu", "Ll", "Lt", "Lm", or "Lo" in the Unicode
 * specification data file).
 *
 * <p> Note that most ideographic characters are considered
 * to be letters (category "Lo") for this purpose.
 *
 * <p> Note also that not all letters have case: many Unicode characters are
 * letters but are neither uppercase nor lowercase nor titlecase.
 * 
 * @param   ch   the character to be tested.
 * @return  <code>true</code> if the character is a letter;
 *          <code>false</code> otherwise.
 * @see     java.lang.Character#isDigit(char)
 * @see     java.lang.Character#isJavaIdentifierStart(char)
 * @see     java.lang.Character#isJavaLetter(char)
 * @see     java.lang.Character#isJavaLetterOrDigit(char)
 * @see     java.lang.Character#isLetterOrDigit(char)
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isTitleCase(char)
 * @see     java.lang.Character#isUnicodeIdentifierStart(char)
 * @see     java.lang.Character#isUpperCase(char)
 * @since   JDK1.0
 */
static sal_Bool isLetter(sal_Unicode ch){ return RTL_CHAR_ISLETTER(ch); }

/**
 * Determines if the specified character is a letter or digit. 
 * For a more complete specification that encompasses all Unicode 
 * characters, see Gosling, Joy, and Steele, <i>The Java Language 
 * Specification</i>. 
 *
 * <p> A character is considered to be a letter if and only if
 * it is specified to be a letter or a digit by the Unicode 2.0 standard
 * (category "Lu", "Ll", "Lt", "Lm", "Lo", or "Nd" in the Unicode
 * specification data file).  In other words, isLetterOrDigit is true
 * of a character if and only if either isLetter is true of the character
 * or isDigit is true of the character.
 * 
 * @param   ch   the character to be tested.
 * @return  <code>true</code> if the character is a letter or digit;
 *          <code>false</code> otherwise.
 * @see     java.lang.Character#isDigit(char)
 * @see     java.lang.Character#isJavaIdentifierPart(char)
 * @see     java.lang.Character#isJavaLetter(char)
 * @see     java.lang.Character#isJavaLetterOrDigit(char)
 * @see     java.lang.Character#isLetter(char)
 * @see     java.lang.Character#isUnicodeIdentifierPart(char)
 * @since   JDK1.0.2
 */
static sal_Bool isLetterOrDigit(sal_Unicode ch){ return RTL_CHAR_ISLETTERORDIGIT(ch); }

/**
 * Determines if the specified character is
 * permissible as the first character in a Java identifier.
 * A character may start a Java identifier if and only if
 * it is one of the following:
 * <ul>
 * <li>  a letter
 * <li>  a currency symbol (such as "$")
 * <li>  a connecting punctuation character (such as "_").
 * </ul>
 *
 * @param   ch	the character to be tested.
 * @return  true if the character may start a Java identifier;
 *          false otherwise.
 * @see     java.lang.Character#isJavaIdentifierPart(char)
 * @see     java.lang.Character#isLetter(char)
 * @see     java.lang.Character#isUnicodeIdentifierStart(char)
 * @since   JDK1.1
 */
static sal_Bool isJavaIdentifierStart(sal_Unicode ch){ return RTL_CHAR_ISJAVAIDENTIFIERSTART(ch); }

/**
 * Determines if the specified character may be part of a Java
 * identifier as other than the first character.
 * A character may be part of a Java identifier if and only if
 * it is one of the following:
 * <ul>
 * <li>  a letter
 * <li>  a currency symbol (such as "$")
 * <li>  a connecting punctuation character (such as "_").
 * <li>  a digit
 * <li>  a numeric letter (such as a Roman numeral character)
 * <li>  a combining mark
 * <li>  a non-spacing mark
 * <li>  an ignorable control character
 * </ul>
 * 
 * @param   ch	the character to be tested.
 * @return  true if the character may be part of a Unicode identifier; 
 *          false otherwise.
 * @see     java.lang.Character#isIdentifierIgnorable(char)
 * @see     java.lang.Character#isJavaIdentifierStart(char)
 * @see     java.lang.Character#isLetterOrDigit(char)
 * @see     java.lang.Character#isUnicodeIdentifierPart(char)
 * @since   JDK1.1
 */
static sal_Bool isJavaIdentifierPart(sal_Unicode ch){ return RTL_CHAR_ISJAVAIDENTIFIERPART(ch); }

/**
 * Determines if the specified character is
 * permissible as the first character in a Unicode identifier.
 * A character may start a Unicode identifier if and only if
 * it is a letter.
 *
 * @param   ch	the character to be tested.
 * @return  true if the character may start a Unicode identifier;
 *          false otherwise.
 * @see     java.lang.Character#isJavaIdentifierStart(char)
 * @see     java.lang.Character#isLetter(char)
 * @see     java.lang.Character#isUnicodeIdentifierPart(char)
 * @since   JDK1.1
 */
static sal_Bool isUnicodeIdentifierStart(sal_Unicode ch){ return RTL_CHAR_ISUNICODEIDENTIFIERSTART(ch); }

/**
 * Determines if the specified character may be part of a Unicode
 * identifier as other than the first character.
 * A character may be part of a Unicode identifier if and only if
 * it is one of the following:
 * <ul>
 * <li>  a letter
 * <li>  a connecting punctuation character (such as "_").
 * <li>  a digit
 * <li>  a numeric letter (such as a Roman numeral character)
 * <li>  a combining mark
 * <li>  a non-spacing mark
 * <li>  an ignorable control character
 * </ul>
 * 
 * @param   ch	the character to be tested.
 * @return  true if the character may be part of a Unicode identifier;
 *          false otherwise.
 * @see     java.lang.Character#isIdentifierIgnorable(char)
 * @see     java.lang.Character#isJavaIdentifierPart(char)
 * @see     java.lang.Character#isLetterOrDigit(char)
 * @see     java.lang.Character#isUnicodeIdentifierStart(char)
 * @since   JDK1.1
 */
static sal_Bool isUnicodeIdentifierPart(sal_Unicode ch){ return RTL_CHAR_ISUNICODEIDENTIFIERPART(ch); }

/**
 * Determines if the specified character should be regarded as
 * an ignorable character in a Java identifier or a Unicode identifier.
 * The following Unicode characters are ignorable in a Java identifier
 * or a Unicode identifier:
 * <table>
 * <tr><td>0x0000 through 0x0008,</td>
 *                                 <td>ISO control characters that</td></tr>
 * <tr><td>0x000E through 0x001B,</td> <td>are not whitespace</td></tr>
 * <tr><td>and 0x007F through 0x009F</td></tr>
 * <tr><td>0x200C through 0x200F</td>  <td>join controls</td></tr>
 * <tr><td>0x200A through 0x200E</td>  <td>bidirectional controls</td></tr>
 * <tr><td>0x206A through 0x206F</td>  <td>format controls</td></tr>
 * <tr><td>0xFEFF</td>               <td>zero-width no-break space</td></tr>
 * </table>
 * 
 * @param   ch	the character to be tested.
 * @return 	true if the character may be part of a Unicode identifier;
 *          false otherwise.
 * @see     java.lang.Character#isJavaIdentifierPart(char)
 * @see     java.lang.Character#isUnicodeIdentifierPart(char)
 * @since   JDK1.1
 */
static sal_Bool isIdentifierIgnorable(sal_Unicode ch){ return RTL_CHAR_ISIDENTIFIERIGNORABLE(ch); }

/**
 * The given character is mapped to its lowercase equivalent; if the 
 * character has no lowercase equivalent, the character itself is 
 * returned. 
 * <p>
 * A character has a lowercase equivalent if and only if a lowercase 
 * mapping is specified for the character in the Unicode attribute 
 * table. 
 * <p>
 * Note that some Unicode characters in the range 
 * <code>'&#92;u2000'</code> to <code>'&#92;u2FFF'</code> have lowercase 
 * mappings; this method does map such characters to their lowercase 
 * equivalents even though the method <code>isUpperCase</code> does 
 * not return <code>true</code> for such characters. 
 *
 * @param   ch   the character to be converted.
 * @return  the lowercase equivalent of the character, if any;
 *          otherwise the character itself.
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isUpperCase(char)
 * @see     java.lang.Character#toTitleCase(char)
 * @see     java.lang.Character#toUpperCase(char)
 * @since   JDK1.0
 */
static sal_Unicode toLowerCase(sal_Unicode ch)
{
	return rtl_char_toLowerCase( ch ); 
	//return RTL_CHAR_TOLOWERCASE( ch ); 
}

/**
 * Converts the character argument to uppercase. A character has an 
 * uppercase equivalent if and only if an uppercase mapping is 
 * specified for the character in the Unicode attribute table. 
 * <p>
 * Note that some Unicode characters in the range 
 * <code>'&#92;u2000'</code> to <code>'&#92;u2000FFF'</code> have uppercase 
 * mappings; this method does map such characters to their titlecase 
 * equivalents even though the method <code>isLowerCase</code> does 
 * not return <code>true</code> for such characters. 
 *
 * @param   ch   the character to be converted.
 * @return  the uppercase equivalent of the character, if any;
 *          otherwise the character itself.
 * @see     java.lang.Character#isLowerCase(char)
 * @see     java.lang.Character#isUpperCase(char)
 * @see     java.lang.Character#toLowerCase(char)
 * @see     java.lang.Character#toTitleCase(char)
 * @since   JDK1.0
 */
static sal_Unicode toUpperCase(char ch)
{
	return rtl_char_toUpperCase( ch ); 
//	return RTL_CHAR_TOUPPERCASE( ch ); 
}

/**
 * Converts the character argument to titlecase. A character has a 
 * titlecase equivalent if and only if a titlecase mapping is 
 * specified for the character in the Unicode attribute table. 
 * <p>
 * Note that some Unicode characters in the range 
 * <code>'&#92;u2000'</code> through <code>'&#92;u2FFF'</code> have titlecase 
 * mappings; this method does map such characters to their titlecase 
 * equivalents even though the method <code>isTitleCase</code> does 
 * not return <code>true</code> for such characters.
 * <p>
 * There are only four Unicode characters that are truly titlecase forms
 * that are distinct from uppercase forms.  As a rule, if a character has no
 * true titlecase equivalent but does have an uppercase mapping, then the
 * Unicode 2.0 attribute table specifies a titlecase mapping that is the
 * same as the uppercase mapping.
 *
 * @param   ch   the character to be converted.
 * @return  the titlecase equivalent of the character, if any;
 *          otherwise the character itself.
 * @see     java.lang.Character#isTitleCase(char)
 * @see     java.lang.Character#toLowerCase(char)
 * @see     java.lang.Character#toUpperCase(char)
 * @since   JDK1.0.2
 */
static sal_Unicode toTitleCase(sal_Unicode ch)
{
	return rtl_char_toTitleCase( ch ); 
//	return RTL_CHAR_TOTITLECASE( ch ); 
}

/**
 * Returns the Unicode numeric value of the character as a
 * nonnegative integer.
 * If the character does not have a numeric value, then -1 is returned.
 * If the character has a numeric value that cannot be represented as a
 * nonnegative integer (for example, a fractional value), then -2
 * is returned.
 *
 * @param   ch	the character to be converted.
 * @param   radix 	the radix.
 * @return  the numeric value of the character, as a nonnegative int value;
 *          -2 if the character has a numeric value that is not a
 *          nonnegative integer; -1 if the character has no numeric value.
 * @see     java.lang.Character#forDigit(char)
 * @see     java.lang.Character#isDigit(char)
 * @since   JDK1.1
 */
static sal_Int32 getNumericValue(sal_Unicode ch){ return rtl_char_getNumericValue(ch); }

/**
 * Determines if the specified character is a Unicode space character.
 * A character is considered to be a space character if and only if
 * it is specified to be a space character by the Unicode 2.0 standard
 * (category "Zs", "Zl, or "Zp" in the Unicode specification data file).
 * 
 * @param   ch	the character to be tested.
 * @return 	true if the character is a space character; false otherwise.
 * @see     java.lang.Character#isWhitespace(char)
 * @since   JDK1.1
 */
static sal_Bool isSpaceChar(sal_Unicode ch){ return RTL_CHAR_ISSPACECHAR(ch); }

/**
 * Determines if the specified character is white space according to Java.
 * A character is considered to be a Java whitespace character if and only
 * if it satisfies one of the following criteria:
 * <ul>
 * <li> It is a Unicode space separator (category "Zs"), but is not
 *      a no-break space (&#92;u00A0 or &#92;uFEFF).
 * <li> It is a Unicode line separator (category "Zl").
 * <li> It is a Unicode paragraph separator (category "Zp").
 * <li> It is &#92;u0009, HORIZONTAL TABULATION.
 * <li> It is &#92;u000A, LINE FEED.
 * <li> It is &#92;u000B, VERTICAL TABULATION.
 * <li> It is &#92;u000C, FORM FEED.
 * <li> It is &#92;u000D, CARRIAGE RETURN.
 * <li> It is &#92;u001C, FILE SEPARATOR.
 * <li> It is &#92;u001D, GROUP SEPARATOR.
 * <li> It is &#92;u001E, RECORD SEPARATOR.
 * <li> It is &#92;u001F, UNIT SEPARATOR.
 * </ul>
 *
 * @param   ch	the character to be tested.
 * @return  true if the character is a Java whitespace character;
 *          false otherwise.
 * @see     java.lang.Character#isSpaceChar(char)
 * @since   JDK1.1
 */
static sal_Bool isWhitespace(sal_Unicode ch){ return RTL_CHAR_ISWHITESPACE(ch); }

/**
 * Determines if the specified character is an ISO control character.
 * A character is considered to be an ISO control character if its
 * code is in the range &#92;u0000 through &#92;u001F or in the range
 * &#92;u007F through &#92;u009F.
 *
 * @param   ch	the character to be tested.
 * @return  true if the character is an ISO control character;
 *          false otherwise.
 *
 * @see     java.lang.Character#isSpaceChar(char)
 * @see     java.lang.Character#isWhitespace(char)
 * @since   JDK1.1
 */
static sal_Bool isISOControl(sal_Unicode ch){ return RTL_CHAR_ISISOCONTROL(ch); }
};

#ifdef _USE_NAMESPACE
}
#endif

#endif	/* __cplusplus */
#endif	/* _RTL_CHAR_HXX */

