/*************************************************************************
 *
 *  $RCSfile: object.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _RTL_OBJECT_H_
#define _RTL_OBJECT_H_

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Increment a reference counter on the object This.
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @see     rtl_object_acquire
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
typedef void (*RTL_CALL rtl_object_acquire_Fct)( void * This );

/**
 * Decrement a reference counter on the object This. If the reference count
 * goes to zero, the object is deleted.
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @see     rtl_object_release
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
typedef void (*RTL_CALL rtl_object_release_Fct)( void * This );

/**
 * Return the type of the object This.
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @returns a memory address that in this process that identify
 *					the type.
 * @see     rtl_object_getType
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
typedef void * (*RTL_CALL rtl_object_getType_Fct)( void * This, void * Type );

/**
 * Returns a hash code value for the object. This method is 
 * supported for the benefit of hashtables. If the equal function
 * only test the identity of the object, return the low 4 bytes of the
 * memory address.
 * <p>
 * The general contract of <code>hashCode</code> is: 
 * <ul>
 * <li>Whenever it is invoked on the same object more than once during 
 *     an execution of a Java application, the <code>hashCode</code> method 
 *     must consistently return the same integer. This integer need not 
 *     remain consistent from one execution of an application to another 
 *     execution of the same application. 
 * <li>If two objects are equal according to the <code>equals</code> 
 *     method, then calling the <code>hashCode</code> method on each of the 
 *     two objects must produce the same integer result. 
 * </ul>
 *
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @return  a hash code value for this object.
 * @see     rtl_object_hashCode
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since   RTL1.0
 */
typedef Int32 (*RTL_CALL rtl_object_hashCode_Fct)( void * This );

/**
 * Compares two Objects for equality.
 * <p>
 * The <code>equals</code> method implements an equivalence relation: 
 * <ul>
 * <li>It is <i>reflexive</i>: for any reference value <code>x</code>, 
 *     <code>x.equals(x)</code> should return <code>true</code>. 
 * <li>It is <i>symmetric</i>: for any reference values <code>x</code> and 
 *     <code>y</code>, <code>x.equals(y)</code> should return 
 *     <code>true</code> if and only if <code>y.equals(x)</code> returns 
 *     <code>true</code>. 
 * <li>It is <i>transitive</i>: for any reference values <code>x</code>, 
 *     <code>y</code>, and <code>z</code>, if <code>x.equals(y)</code>
 *     returns  <code>true</code> and <code>y.equals(z)</code> returns 
 *     <code>true</code>, then <code>x.equals(z)</code> should return 
 *     <code>true</code>. 
 * <li>It is <i>consistent</i>: for any reference values <code>x</code> 
 *     and <code>y</code>, multiple invocations of <code>x.equals(y)</code> 
 *     consistently return <code>true</code> or consistently return 
 *     <code>false</code>. 
 * <li>For any reference value <code>x</code>, <code>x.equals(null)</code> 
 *     should return <code>false</code>.
 * </ul>
 * <p>
 * The most discriminating possible equivalence relation on objects is, 
 * for any reference values <code>x</code> and <code>y</code>, this 
 * function returns <code>true</code> if and only if <code>x</code> and 
 * <code>y</code> refer to the same object (<code>x==y</code> has the 
 * value <code>true</code>). 
 *
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @param   obj   the reference object with which to compare.
 * @return  <code>true</code> if this object is equal to the obj
 *          argument; <code>false</code> otherwise.
 * @see     rtl_object_equals
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
typedef sal_Bool (*RTL_CALL rtl_object_equals_Fct)( void * This, void * obj );

/**
 * Creates a new object of the same class as this object. It then 
 * initializes each of the new object's fields by assigning it the 
 * same value as the corresponding field in this object.
 * <p>
 * If the function entry in the table is null, than it is not possible
 * to clone the object.
 *
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @return	a clone of this instance.
 * @see		rtl_object_newClone
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
typedef void * (*RTL_CALL rtl_object_newClone_Fct)( void * This );


/**
 * Increment a reference counter on the object This. Call the function no. 0 in the function table.
 * @param This	a pointer to a structure with the same memory layout as rtl_object.
 * @see		rtl_object_acquire_Fct
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
void RTL_CALL rtl_object_acquire( void * This );

/**
 * Decrement a reference counter on the object This. If the reference count
 * goes to zero, the object is deleted. Call the function no. 1 in the function table.
 * @param This	a pointer to a structure with the same memory layout as rtl_object.
 * @see		rtl_object_release_Fct
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
void RTL_CALL rtl_object_release( void * This );

/**
 * Return the type of the object This. Call the function no. 2 in the function table.
 * @param This	a pointer to a structure with the same memory layout as rtl_object.
 * @returns the exact type of the return is specified bei the UniqueId.
 *			Null is returned, if the implementation does not recognize the UniqueId.
 * @see		rtl_object_release_Fct
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
void * RTL_CALL rtl_object_getType( void * This );

/**
 * Returns a hash code value for the object. Call the function no. 3 in the function table.
 * @see     rtl_object_hashCode_Fct
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since   RTL1.0
 */
Int32 RTL_CALL rtl_object_hashCode( void * This );

/**
 * Compares two Objects for equality. Call the function no. 4 in the function table.
 * @see     rtl_object_equals_Fct
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
sal_Bool RTL_CALL rtl_object_equals( void * This, void * obj );

/**
 * Creates a new object of the same class as this object. Call the function 
 * no. 5 in the function table. The entry in the function table may by null.
 * @see		rtl_object_newClone_Fct
 * @see		rtl_Object
 * @see		rtl_Object_VTable
 * @since	RTL1.0
 */
typedef void * (*RTL_CALL rtl_object_newClone_Fct)( void * This );

/**
 * It is a dummy type for a any rtl function.
 */
typedef void (*RTL_CALL rtl_object_Fct)();

#ifdef SAL_W32
#	pragma pack(push, 8)
#elif defined(SAL_OS2)
#	pragma pack(1)
#endif

typedef struct _rtl_Object_VTable
{
	rtl_object_acquire_Fct	acquire;
	rtl_object_release_Fct	release;
	rtl_object_getType_Fct	getType;
	rtl_object_hashCode_Fct	hashCode;
	rtl_object_equals_Fct	equals;
	rtl_object_newClone_Fct	newClone;
} rtl_Object_VTable;

/**
 * The implementation structur of an object. All struct that emulate objects in
 * c must start with this memory layout.
 */
typedef struct _rtl_Object
{
	/**
	 * The virtual function table to operate on this object.
	 */
	rtl_Object_VTable *	VTable;
} rtl_Object;

#ifdef SAL_W32
#	pragma pack(pop)
#elif defined(SAL_OS2)
#	pragma pack()
#endif

/**
 * Return the length of the sequence.
 *
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @return	the count of elements in the sequence.
 * @see		rtl_Sequence_VTable
 * @since	RTL1.0
 */
typedef Int32 (*RTL_CALL rtl_sequence_getLength_Fct)( void * This );

/**
 * Return the element at index Index.
 *
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @return	a pointer to the element.
 * @see		rtl_Sequence_VTable
 * @since	RTL1.0
 */
typedef void * (*RTL_CALL rtl_sequence_get_Fct)( void * This, Int32 Index );

/**
 * Return the type of the elements in the sequence.
 *
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @return	a pointer to the element.
 * @see		rtl_Sequence_VTable
 * @see		rtl_object_getType_Fct
 * @since	RTL1.0
 */
typedef void * (*RTL_CALL rtl_sequence_getElementType_Fct)( void * This );

#ifdef SAL_W32
#	pragma pack(push, 8)
#elif defined(SAL_OS2)
#	pragma pack(1)
#endif

/**
 * The function table of a sequence.
 */
typedef struct _rtl_Sequence_VTable
{
	/* rtl_Object */
	rtl_object_acquire_Fct			acquire;
	rtl_object_release_Fct			release;
	rtl_object_getType_Fct			getType;
	rtl_object_hashCode_Fct			hashCode;
	rtl_object_equals_Fct			equals;
	rtl_object_newClone_Fct			newClone;
	/* rtl_Sequence */
	rtl_sequence_getLength_Fct		getLength;
	rtl_sequence_get_Fct			get;
	rtl_sequence_getElementType_Fct	getElementType;
} rtl_Sequence_VTable;

/**
 * The structure of a sequence.
 */
typedef struct _rtl_Sequence
{
	/**
	 * The virtual function table to operate on this object.
	 */
	rtl_Sequence_VTable *	VTable;
} rtl_Sequence;

#ifdef SAL_W32
#	pragma pack(pop)
#elif defined(SAL_OS2)
#	pragma pack()
#endif


#ifdef __cplusplus
}
#endif
#endif


