/*************************************************************************
 *
 *  $RCSfile: rb.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _RTL_RB_H_
#define _RTL_RB_H_

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef _RTL_LOCALE_HXX_
#include <rtl/locale.hxx>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Get an object from a ResourceBundle.
 * <BR>Convenience method to save casting.
 * @param value	a in/out parameter. If the *Value is set than the string is released by the
 *				callee. The resource found under Key is returned. *Value is null, if the 
 *				resource is not found.
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @param key see class description.
 * @return false, if the resource is not found.
 */
typedef sal_Bool (*RTL_CALL rtl_resourcebundle_getString_Fct)
				( void * This, rtl_uString ** value, const sal_Unicode * key, Int32 keyLen );

/**
 * Get an object from a ResourceBundle.
 * <BR>Convenience method to save casting.
 * @param value	a in/out parameter. If the *Value is set than the rtl_Sequence is released by the
 *				callee. The resource found under Key is returned, if it is a string sequence. 
 *				*Value is null, if the resource is not found or not the right type.
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @param key see class description.
 * @return false, if the resource is not found.
 */
typedef sal_Bool (*RTL_CALL rtl_resourcebundle_getStringArray_Fct)
				( void * This, rtl_Sequence ** value, const sal_Unicode * key, Int32 keyLen );

/**
 * Get an object from a ResourceBundle.
 * @param value	a in/out parameter. If the *Value is set than the rtl_object is released by the
 *				callee. The resource found under Key is returned. *Value is null, if the 
 *				resource is not found.
 * @param This	a pointer to a structure which first element is a pointer to
 *				the function table that include this function.
 * @param key see class description.
 * @return false, if the resource is not found.
 */
typedef sal_Bool (*RTL_CALL rtl_resourcebundle_getObject_Fct)
				( void * This, void ** value, const sal_Unicode * key, Int32 keyLen );

#ifdef SAL_W32
#	pragma pack(push, 8)
#elif defined(SAL_OS2)
#	pragma pack(1)
#endif

/**
 * The implementation structur of a resource bundle. 
 */
typedef struct _rtl_ResourceBundle_VTable
{
	/* rtl_Object */
	rtl_object_acquire_Fct					acquire;
	rtl_object_release_Fct					release;
	rtl_object_getType_Fct					getType;
	rtl_object_hashCode_Fct					hashCode;
	rtl_object_equals_Fct					equals;
	rtl_object_newClone_Fct					newClone;
	/* rtl_ResourceBundle */
	rtl_resourcebundle_getString_Fct		getString;
	rtl_resourcebundle_getStringArray_Fct	getStringArray;
	rtl_resourcebundle_getObject_Fct		getObject;
} rtl_ResourceBundle_VTable;

/**
 * The function table of a resource bundle. 
 */
typedef struct _rtl_ResourceBundle
{
	/**
	 * The virtual function table to operate on this object.
	 */
	rtl_ResourceBundle_VTable *		VTable;
} rtl_ResourceBundle;

#ifdef SAL_W32
#	pragma pack(pop)
#elif defined(SAL_OS2)
#	pragma pack()
#endif


/**
 * Get the appropriate ResourceBundle.
 * @param baseName see class description.
 * @param baseNameLen the length of the 
 * @param locale   see class description.
 */
typedef sal_Bool (*RTL_CALL rtl_resourcebundle_getBundle_Fct)( rtl_ResourceBundle ** value,
								const sal_Unicode * baseName, Int32 baseNameLen, rtl_Locale * locale );

/**
 * Set the function to access resource bundles.
 * @param bundler the function called to get rtl_ResourceBundle objects.
 */
sal_Bool RTL_CALL rtl_resourcebundle_setBundler( rtl_resourcebundle_getBundle_Fct * bundler );

#ifdef __cplusplus
}
#endif

#endif /* _RTL_RB_H_ */


