/*************************************************************************
 *
 *  $RCSfile: channman.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <osl/diagnose.h>
#include <osl/mutex.h>
#include <osl/thread.h>

#include <string.h>
#include <stdio.h>

#include "channman.h"
#include "chanpool.h"

#define GLOBAL_RESOURCE_PIPE_NAME "osl_resource_pipe"

#define RESOURCE_REQUEST_TYPE_ACCEPT 		0x00000001
#define RESOURCE_REQUEST_TYPE_PROVIDE 		0x00000002
#define RESOURCE_REQUEST_TYPE_PING 			0x00000003
#define RESOURCE_REQUEST_TYPE_SHUTDOWN		0x00000004
#define RESOURCE_REQUEST_TYPE_QUERY_AUTH	0x00000005

#define RESOURCE_REQUEST_FLAG_USE_AUTH 		0x00000010
#define RESOURCE_REQUEST_FLAG_USE_TIMEOUT 	0x00000020

#define PATH_MAX 512

typedef sal_uInt32 RequestType;

typedef struct 
{
	ResourcePipeApi* 	pApi;
	oslResourcePipe		hPipe;
	oslMutex			mutex;
			
} BrokerProxy;


typedef struct _ClientThread
{
	ResourcePipeApi* 	pApi;
	oslResourcePipe		hPipe;
	oslThread			clientThread;
	struct _BrokerStub* pBrokerStub;

} ClientThread;


typedef struct _BrokerStub
{
	ResourcePipeApi* 	pApi;
	oslResourcePipe		hPipe;
	oslThread			acceptorThread;
	sal_Bool			bEnd;
	oslChannelPool		hPool;
			
} BrokerStub;


/*****************************************************************************/
/* Static Module Functions */
/*****************************************************************************/

static sal_Bool checkClientAuthentication( void* pData, rtl_uString* ustrUserIdent, rtl_uString* ustrPassword );
static void     channelBrokerClient_Function( ClientThread* pThis );
static void     channelBrokerAcceptor_Function( BrokerStub* pThis );

static BrokerStub* channelBrokerStub = NULL;

/*****************************************************************************
 	accept protocol:
	================
	
	client 						manager

			type -> 
			resourceNameLen ->
			resourceName -> 

*			timeout ->

*			<- errorCode
*			<- userIdentLen
*			<- passwordLen
*			<- userIdent
*			<- password

			<- errorCode
			<- resource

			errorCode -> 
					
******************************************************************************

 	provide protocol:
	================
	
	client 						manager

			type -> 
			resourceNameLen ->
			resourceName -> 
*			timeout ->
			userIdentLen -> 
			passwordLen -> 
*			userIdent -> 
*			password -> 

			resource ->

			<- errorCode

*****************************************************************************/


/*****************************************************************************/
/* osl_osl_default_startChannelBroker */
/*****************************************************************************/

sal_Bool SAL_CALL osl_default_startChannelBroker(ResourcePipeApi* pApi, rtl_uString *PipeName)
{
	sal_Bool ret = sal_False;

	if (channelBrokerStub == NULL)
	{
        rtl_uString* ustrPipeName=0;
        oslResourcePipe hAcceptPipe=0;
        
        rtl_uString_newFromAscii(&ustrPipeName,GLOBAL_RESOURCE_PIPE_NAME);

        hAcceptPipe = pApi->listenOnResourcePipe(ustrPipeName);
        
        rtl_uString_release(ustrPipeName);
        
		if (hAcceptPipe != NULL)
		{
			channelBrokerStub = (BrokerStub*) malloc(sizeof(BrokerStub));

			channelBrokerStub->pApi = pApi;
			channelBrokerStub->hPipe = hAcceptPipe;
			channelBrokerStub->bEnd = sal_False;
			channelBrokerStub->hPool = createChannelPool_Impl();


			channelBrokerStub->acceptorThread = 
            osl_createSuspendedThread(
                (oslWorkerFunction)channelBrokerAcceptor_Function, 
                channelBrokerStub);

            osl_resumeThread(channelBrokerStub->acceptorThread);

			ret	= sal_True;
		}
	}

	return(ret);
}	


/*****************************************************************************/
/* osl_default_stopChannelBroker 											 */
/*****************************************************************************/

sal_Bool SAL_CALL osl_default_stopChannelBroker()
{
	sal_Bool ret = sal_False;

	if (channelBrokerStub != NULL)
	{
        rtl_uString* ustrPipeName=0;
        
		oslResourcePipe hPipe = NULL;

		channelBrokerStub->bEnd = sal_True;

        rtl_uString_newFromAscii(&ustrPipeName,GLOBAL_RESOURCE_PIPE_NAME);
        
		hPipe = channelBrokerStub->pApi->connectToResourcePipe(ustrPipeName);

        rtl_uString_release(ustrPipeName);
        
		if (hPipe != NULL)
		{
			RequestType type = RESOURCE_REQUEST_TYPE_SHUTDOWN;
			oslChannelError error = osl_Channel_E_None;

			channelBrokerStub->pApi->sendBuffer(hPipe, &type, sizeof(type));
			channelBrokerStub->pApi->receiveBuffer(hPipe, &error, sizeof(error));

			channelBrokerStub->pApi->closeResourcePipe(hPipe);

			ret	= sal_True;
		}

		osl_joinWithThread(channelBrokerStub->acceptorThread);

		channelBrokerStub->pApi->closeResourcePipe(channelBrokerStub->hPipe);

		osl_destroyThread(channelBrokerStub->acceptorThread);
		destroyChannelPool_Impl(channelBrokerStub->hPool);
	
		free(channelBrokerStub);

		channelBrokerStub = NULL;

		ret = sal_True;				
	}

	
	return(ret);
}	


/*****************************************************************************/
/* osl_default_channelConnectBroker */
/*****************************************************************************/

oslChannelBroker SAL_CALL osl_default_channelConnectBroker(ResourcePipeApi* pApi, rtl_uString *PipeName)
{
	BrokerProxy* pBroker = NULL;
	oslResourcePipe hPipe=0;
    rtl_uString* ustrPipeName=0;
    
    rtl_uString_newFromAscii(&ustrPipeName,GLOBAL_RESOURCE_PIPE_NAME);
    
    hPipe = pApi->connectToResourcePipe(ustrPipeName);

    rtl_uString_release(ustrPipeName);
    
	if (hPipe != NULL)
	{
		pBroker = (BrokerProxy*) malloc(sizeof(BrokerProxy));
		pBroker->pApi = pApi;
		pBroker->hPipe = hPipe;
		pBroker->mutex = osl_createMutex();
	}

	return((oslChannelBroker)pBroker);
}	


/*****************************************************************************/
/* osl_default_channelDisconnectBroker */
/*****************************************************************************/

void SAL_CALL osl_default_channelDisconnectBroker(oslChannelBroker Broker)
{
	BrokerProxy* pBroker = (BrokerProxy*)Broker;

	if (pBroker != NULL)	
	{
		pBroker->pApi->closeResourcePipe(pBroker->hPipe);
		osl_destroyMutex(pBroker->mutex);
		free(pBroker);
	}
}	


/*****************************************************************************/
/* osl_default_channelProvide */
/*****************************************************************************/

oslChannelError SAL_CALL osl_default_channelProvide
(
	oslChannelBroker		Broker,
	oslChannelResource		Resource, 
	oslChannelType			Type,	
	rtl_uString*            strResourceName,
	rtl_uString*            strUserIdent,
	rtl_uString*            strPassword,
	const TimeValue* 		pTimeout
)
{
	oslChannelError error = osl_Channel_E_Unknown;

	BrokerProxy* pBroker = (BrokerProxy*)Broker;

	if (pBroker != NULL)	
	{
		RequestType type = RESOURCE_REQUEST_TYPE_PROVIDE;
		sal_Int32 nameLen = rtl_uString_getLength(strResourceName);
		sal_Int32 userIdentLen = 0;
		sal_Int32 passwordLen = 0;

		if (strUserIdent != NULL)
			userIdentLen = rtl_uString_getLength(strUserIdent);

		if (strPassword != NULL)
			passwordLen = rtl_uString_getLength(strPassword);

		if (pTimeout != NULL)
			type |= RESOURCE_REQUEST_FLAG_USE_TIMEOUT;

		osl_acquireMutex(pBroker->mutex);

		if (
			(pBroker->pApi->sendBuffer(pBroker->hPipe, &type, sizeof(type)) == sizeof(type)) &&
			(pBroker->pApi->sendBuffer(pBroker->hPipe, &nameLen, sizeof(nameLen)) == sizeof(nameLen)) &&
			(pBroker->pApi->sendBuffer(pBroker->hPipe, rtl_uString_getStr(strResourceName), sizeof( sal_Unicode ) * nameLen) == (sal_Int32)( sizeof( sal_Unicode ) * nameLen)) &&
			(
			 (pTimeout == NULL) ||
			 (pBroker->pApi->sendBuffer(pBroker->hPipe, pTimeout, sizeof(*pTimeout)) == sizeof(*pTimeout))
			) &&
			(pBroker->pApi->sendBuffer(pBroker->hPipe, &userIdentLen, sizeof(userIdentLen)) == sizeof(userIdentLen)) &&
			(pBroker->pApi->sendBuffer(pBroker->hPipe, &passwordLen, sizeof(passwordLen)) == sizeof(passwordLen)) &&
			(
			 (userIdentLen == 0) ||
			 (pBroker->pApi->sendBuffer(pBroker->hPipe, rtl_uString_getStr(strUserIdent), sizeof( sal_Unicode ) * userIdentLen ) == (sal_Int32)( sizeof( sal_Unicode ) * userIdentLen ))
			) &&
			(
			 (passwordLen == 0) ||
			 (pBroker->pApi->sendBuffer(pBroker->hPipe, rtl_uString_getStr(strPassword), sizeof( sal_Unicode ) * passwordLen) == (sal_Int32)( sizeof( sal_Unicode ) * passwordLen ))
			) &&
			(pBroker->pApi->sendResource(pBroker->hPipe, Resource, Type, sal_False)) &&
			(pBroker->pApi->receiveBuffer(pBroker->hPipe, &error, sizeof(error)) == sizeof(error))
		   )	
		{
/*			if (
				(error == osl_Channel_E_None) &&
				(Type == osl_Channel_Type_Socket)
			   )
				osl_closeSocket(Resource.Socket);*/
		}

		osl_releaseMutex(pBroker->mutex);
	}

	return(error);
}	


/*****************************************************************************/
/* osl_default_channelAccept */
/*****************************************************************************/

oslChannelError SAL_CALL osl_default_channelAccept
(
	oslChannelBroker		Broker,
	oslChannelResource*		pResource, 
    oslChannelType* 		pType,	
	rtl_uString*            strResourceName,
	CheckAuthentication_F	checkAuthFunc,
	void* 					pAuthFuncData,
	const TimeValue* 		pTimeout
)
{
	oslChannelError error = osl_Channel_E_Unknown;

	BrokerProxy* pBroker = (BrokerProxy*)Broker;

	if (pBroker != NULL)	
	{
		RequestType type = RESOURCE_REQUEST_TYPE_ACCEPT;
		sal_uInt32 nameLen = rtl_uString_getLength(strResourceName);

		if (pTimeout != NULL)
			type |= RESOURCE_REQUEST_FLAG_USE_TIMEOUT;

		if (checkAuthFunc != NULL)
			type |= RESOURCE_REQUEST_FLAG_USE_AUTH;

		osl_acquireMutex(pBroker->mutex);

		if (
			(pBroker->pApi->sendBuffer(pBroker->hPipe, &type, sizeof(type)) == sizeof(type)) &&
			(pBroker->pApi->sendBuffer(pBroker->hPipe, &nameLen, sizeof(nameLen)) == sizeof(nameLen)) &&
			(pBroker->pApi->sendBuffer(pBroker->hPipe, rtl_uString_getStr(strResourceName), sizeof( sal_Unicode ) * nameLen) == (sal_Int32)( sizeof( sal_Unicode ) * nameLen )) &&
			(
			 (pTimeout == NULL) ||
			 (pBroker->pApi->sendBuffer(pBroker->hPipe, pTimeout, sizeof(*pTimeout)) == sizeof(*pTimeout))
			) &&
			(pBroker->pApi->receiveBuffer(pBroker->hPipe, &error, sizeof(error)) == sizeof(error))
		   )	
		{
			if (type & RESOURCE_REQUEST_FLAG_USE_AUTH)
			{
				sal_uInt32 userIdentLen;
				sal_uInt32 passwordLen;
				error = osl_Channel_E_AuthenticationFailed;

				while (error == osl_Channel_E_AuthenticationFailed)
				{
					if (
						(pBroker->pApi->receiveBuffer(pBroker->hPipe, &userIdentLen, sizeof(userIdentLen)) == sizeof(userIdentLen)) &&
						(pBroker->pApi->receiveBuffer(pBroker->hPipe, &passwordLen, sizeof(passwordLen)) == sizeof(passwordLen))
					   )
					{
						sal_Unicode* pUserIdent = (sal_Unicode*)calloc(userIdentLen+1, sizeof(sal_Unicode));
						sal_Unicode* pPassword = (sal_Unicode*)calloc(passwordLen+1, sizeof(sal_Unicode));

						if (
							(
							 (userIdentLen == 0) ||
							 (pBroker->pApi->receiveBuffer(pBroker->hPipe, pUserIdent, ( sizeof(sal_Unicode) * userIdentLen )) == (sal_Int32)( sizeof(sal_Unicode) * userIdentLen ))
							) &&
							(
							 (passwordLen == 0) ||
							 (pBroker->pApi->receiveBuffer(pBroker->hPipe, pPassword, ( sizeof(sal_Unicode) * passwordLen )) == ( sizeof(sal_Unicode) * sizeof(passwordLen)))
							)
						   )
						{
                            rtl_uString* ustrPassword=0;
                            rtl_uString* ustrUserIdent=0;
                            
                            rtl_uString_newFromStr(&ustrPassword,pPassword);
                            rtl_uString_newFromStr(&ustrUserIdent,pUserIdent);
                            
							if (checkAuthFunc(pAuthFuncData, ustrUserIdent, ustrPassword))
                            {    
								error = osl_Channel_E_None;
                            }
                            
                            rtl_uString_release(ustrPassword);
                            rtl_uString_release(ustrUserIdent);
						}

						free(pUserIdent);
						free(pPassword);
                        
					}

					if (
						(pBroker->pApi->sendBuffer(pBroker->hPipe, &error, sizeof(error)) != sizeof(error)) ||
						(pBroker->pApi->receiveBuffer(pBroker->hPipe, &error, sizeof(error)) != sizeof(error))
					   )
						error = osl_Channel_E_Unknown;
				}
			}

			if (error == osl_Channel_E_None)
				if (!pBroker->pApi->receiveResource(pBroker->hPipe, pResource, pType, sal_False))
					error = osl_Channel_E_Unknown;

			if (pBroker->pApi->sendBuffer(pBroker->hPipe, &error, sizeof(error)) != sizeof(error))
				error = osl_Channel_E_Unknown;
		}

		osl_releaseMutex(pBroker->mutex);
	}

	return(error);
}	



/*****************************************************************************/
/* osl_checkClientAuthentication */
/*****************************************************************************/

static sal_Bool checkClientAuthentication(void* pData, rtl_uString* ustrUserIdent, rtl_uString* ustrPassword)
{
	ClientThread* pThis = (ClientThread*)pData;
	sal_Bool bOk = sal_False;	
	oslChannelError error = osl_Channel_E_AuthenticationFailed;
	sal_Int32 userIdentLen = 0;
	sal_Int32 passwordLen = 0;

	if (ustrUserIdent != NULL)
		userIdentLen = rtl_uString_getLength(ustrUserIdent);

	if (ustrPassword != NULL)
		passwordLen = rtl_uString_getLength(ustrPassword);

	if (
		(pThis->pApi->sendBuffer(pThis->hPipe, &error, sizeof(error)) == sizeof(error)) &&
		(pThis->pApi->sendBuffer(pThis->hPipe, &userIdentLen, sizeof(userIdentLen)) == sizeof(userIdentLen)) &&
		(pThis->pApi->sendBuffer(pThis->hPipe, &passwordLen, sizeof(passwordLen)) == sizeof(passwordLen)) &&
		(pThis->pApi->sendBuffer(pThis->hPipe, rtl_uString_getStr(ustrUserIdent), (sizeof( sal_Unicode ) * userIdentLen)) == (sal_Int32)(sizeof( sal_Unicode ) * userIdentLen)) &&
		(pThis->pApi->sendBuffer(pThis->hPipe, rtl_uString_getStr(ustrPassword), (sizeof( sal_Unicode ) * passwordLen)) == (sal_Int32)(sizeof( sal_Unicode ) * passwordLen)) &&
		(pThis->pApi->receiveBuffer(pThis->hPipe, &error, sizeof(error)) == sizeof(error)) &&
		(error == osl_Channel_E_None)
	   )	
	   bOk = sal_True;

	return(bOk);
}	


/*****************************************************************************/
/* osl_channelBrokerClient_Function */
/*****************************************************************************/

static void channelBrokerClient_Function(ClientThread* pThis)
{
	sal_Bool bEnd = sal_False;

	while (!bEnd)
	{
		RequestType type;
						 
		if (pThis->pApi->receiveBuffer(pThis->hPipe, &type, sizeof(type)) != sizeof(type))
			bEnd = sal_True;
		else
		{
			if (type & RESOURCE_REQUEST_TYPE_ACCEPT)
			{
				oslChannelError error = osl_Channel_E_None;
				sal_Unicode* resourceName = NULL;
				rtl_uString* ustrResourceName = 0;
				sal_uInt32 nameLen;
				TimeValue timeout;
				TimeValue* pTimeout = NULL;

				oslChannelType resourceType;
				oslChannelResource resource;

				if (pThis->pApi->receiveBuffer(pThis->hPipe, &nameLen, sizeof(nameLen)) == sizeof(nameLen))
				{
					resourceName = (sal_Unicode*)calloc(nameLen + 1, sizeof(sal_Unicode));
					if (pThis->pApi->receiveBuffer(pThis->hPipe, resourceName, ( sizeof(sal_Unicode) * nameLen)) != (sal_Int32)( sizeof(sal_Unicode) * nameLen))
					{
						free(resourceName);
						resourceName = NULL;
					}
                    else
                    {    
                        rtl_uString_newFromStr(&ustrResourceName,resourceName);
                    }
				}

				if (resourceName == NULL)
					error = osl_Channel_E_InvalidName;
				
				if (
					(error == osl_Channel_E_None) &&
					(type & RESOURCE_REQUEST_FLAG_USE_TIMEOUT)
				   )
				{
					if (pThis->pApi->receiveBuffer(pThis->hPipe, &timeout, sizeof(timeout)) != sizeof(timeout))
						error = osl_Channel_E_Unknown;
					pTimeout = &timeout;
				}

				if (error == osl_Channel_E_None)
				{
					CheckAuthentication_F check_f = NULL;

					if (type & RESOURCE_REQUEST_FLAG_USE_AUTH)
						check_f = (CheckAuthentication_F) checkClientAuthentication;

					error = channelPool_pop_Impl(
								pThis->pBrokerStub->hPool, 	
								&resourceType,
								&resource,
								ustrResourceName,
								check_f,
								pThis,
								pTimeout);
				}

				pThis->pApi->sendBuffer(pThis->hPipe, &error, sizeof(error));

				if (error == osl_Channel_E_None)
				{
					if (!pThis->pApi->sendResource(pThis->hPipe, resource, resourceType, sal_True))
						error = osl_Channel_E_Unknown;

					switch (resourceType)
					{
						case osl_Channel_Type_Socket:
							osl_destroySocket(resource.Socket);
							break;
						case osl_Channel_Type_Pipe:
							osl_destroyPipe(resource.Pipe);
							break;
						default:
							break;
					}
				}

				pThis->pApi->receiveBuffer(pThis->hPipe, &error, sizeof(error));

				if (resourceName != NULL)
					free(resourceName);

                if ( ustrResourceName != 0 )
                {
                    rtl_uString_release(ustrResourceName);
                }
                
			}
			else if (type & RESOURCE_REQUEST_TYPE_PROVIDE)
			{
				oslChannelError error = osl_Channel_E_None;
				sal_Unicode* resourceName = NULL;
				rtl_uString* ustrResourceName = 0;
				sal_uInt32 nameLen;
				TimeValue timeout;
				TimeValue* pTimeout = NULL;

				oslChannelType resourceType;
				oslChannelResource resource;

				if (pThis->pApi->receiveBuffer(pThis->hPipe, &nameLen, sizeof(nameLen)) == sizeof(nameLen))
				{
					resourceName = (sal_Unicode*)calloc(nameLen + 1, sizeof(sal_Unicode));
					if (pThis->pApi->receiveBuffer(pThis->hPipe, resourceName, ( sizeof(sal_Unicode) * nameLen)) != (sal_Int32)( sizeof(sal_Unicode) * nameLen ))
					{
						free(resourceName);
						resourceName = NULL;
					}
                    else
                    {
                        rtl_uString_newFromStr(&ustrResourceName,resourceName);
                    }
                    
				}

				if (resourceName == NULL)
					error = osl_Channel_E_InvalidName;
				
				if (
					(error == osl_Channel_E_None) &&
					(type & RESOURCE_REQUEST_FLAG_USE_TIMEOUT)
				   )
				{
					if (pThis->pApi->receiveBuffer(pThis->hPipe, &timeout, sizeof(timeout)) != sizeof(timeout))
						error = osl_Channel_E_Unknown;
					pTimeout = &timeout;
				}
				
				if (error == osl_Channel_E_None)
				{
					sal_uInt32 userIdentLen;
					sal_uInt32 passwordLen;
					sal_Unicode* pUserIdent = NULL;
					sal_Unicode* pPassword = NULL;
					rtl_uString* ustrUserIdent = 0;
					rtl_uString* ustrPassword = 0;

					if (
						(pThis->pApi->receiveBuffer(pThis->hPipe, &userIdentLen, sizeof(userIdentLen)) == sizeof(userIdentLen)) &&
						(pThis->pApi->receiveBuffer(pThis->hPipe, &passwordLen, sizeof(passwordLen)) == sizeof(passwordLen))
					   )
					{
						if (userIdentLen > 0)
						{
							pUserIdent = (sal_Unicode*) calloc(userIdentLen+1, sizeof(sal_Unicode));
                            pThis->pApi->receiveBuffer(pThis->hPipe, pUserIdent, ( sizeof(sal_Unicode) * userIdentLen));

                            rtl_uString_newFromStr(&ustrUserIdent,pUserIdent);
						}

						if (passwordLen > 0)
						{
							pPassword = (sal_Unicode*) calloc(passwordLen+1, sizeof(sal_Unicode));
							pThis->pApi->receiveBuffer(pThis->hPipe, pPassword, ( sizeof(sal_Unicode) * passwordLen));

                            rtl_uString_newFromStr(&ustrPassword,pPassword);                            
						}
					}
					
					if (!pThis->pApi->receiveResource(pThis->hPipe, &resource, &resourceType, sal_True))
						error = osl_Channel_E_Unknown;

					if (error == osl_Channel_E_None)
					{
						error = channelPool_push_Impl(
									pThis->pBrokerStub->hPool, 	
									resourceType,
									resource,
									ustrResourceName,
									ustrUserIdent,
									ustrPassword,
									pTimeout);

						if (error != osl_Channel_E_None)
						{
							switch (resourceType)
							{
								case osl_Channel_Type_Socket:
									osl_destroySocket(resource.Socket);
									break;
								case osl_Channel_Type_Pipe:
									osl_destroyPipe(resource.Pipe);
									break;
								default:
									break;
							}
						}
					}

					pThis->pApi->sendBuffer(pThis->hPipe, &error, sizeof(error));

					if (resourceName != NULL)
						free(resourceName);

                    if ( ustrResourceName != 0 )
                    {
                        rtl_uString_release(ustrResourceName);
                    }
                    if ( ustrUserIdent != 0 )
                    {
                        rtl_uString_release(ustrUserIdent);
                    }
                    if ( ustrPassword != 0 )
                    {
                        rtl_uString_release(ustrPassword);
                    }
                    
					if (pUserIdent != NULL)
						free(pUserIdent);

					if (pPassword != NULL)
						free(pPassword);
				}
			}
			else if (type & RESOURCE_REQUEST_TYPE_PING)
			{
				oslChannelError error = osl_Channel_E_None;
				pThis->pApi->sendBuffer(pThis->hPipe, &error, sizeof(error));
			}
			else if (type & RESOURCE_REQUEST_TYPE_SHUTDOWN)
			{
				oslChannelError error = osl_Channel_E_None;
				pThis->pBrokerStub->bEnd = sal_True;
				pThis->pApi->sendBuffer(pThis->hPipe, &error, sizeof(error));
			}
			else
				bEnd = sal_True;
		}
	}

	pThis->pApi->closeResourcePipe(pThis->hPipe);
	osl_freeThreadHandle(pThis->clientThread);
	free(pThis);
}	


/*****************************************************************************/
/* osl_channelBrokerAcceptor_Function */
/*****************************************************************************/

static void channelBrokerAcceptor_Function(BrokerStub* pThis)
{
	while (!pThis->bEnd)
	{
		oslResourcePipe hClientPipe = pThis->pApi->acceptOnResourcePipe(pThis->hPipe);

		if (hClientPipe != NULL)
		{
			ClientThread* pClientThread = (ClientThread*) malloc(sizeof(ClientThread));			
		
			pClientThread->pApi = pThis->pApi;
			pClientThread->hPipe = hClientPipe;
			pClientThread->pBrokerStub = pThis;

			pClientThread->clientThread = 
				osl_createSuspendedThread(
					(oslWorkerFunction)channelBrokerClient_Function, 
					pClientThread);

			osl_resumeThread(pClientThread->clientThread);
		}
		else
			pThis->bEnd = sal_True;
	}
}	


