/*************************************************************************
 *
 *  $RCSfile: daemclie.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <osl/diagnose.h>
#include <osl/daemon.h>
#include <osl/thread.h>
#include <osl/conditn.h>

#include "../all/daemclie.h"
#include "../all/daemprot.h"

#include <rtl/alloc.h>
#include <rtl/memory.h>

#include <string.h>
#include <stdio.h>

#define MAX_DWORD_2_STRING_BUFFER 64

/*
	Daemon-data structure hidden behind oslDaemonHandle:    
*/
typedef struct _osl_TDaemonImpl 
{
	oslDaemonControl        	m_DaemonControl;
	void*                   	m_pControlData;
	oslDaemonPropertyAccess* 	m_pPropertyAccess;
	void*                   	m_pAccessData;
	oslDaemonInfo				m_info;
	oslDaemonStatus				m_status;
	oslCondition				m_stopCond;
} osl_TDaemonImpl;


/*****************************************************************************/
/* osl_executeDaemonRequest */
/*****************************************************************************/

/* returns False, if the message was a reply */
static sal_Bool osl_executeDaemonRequest(oslDaemonMgrMsg* pMsg, osl_TDaemonImpl* pDaemon)
{
	sal_Bool ret = sal_False;

	if (pDaemon)
	{
		OSL_ENSHURE(
			rtl_ustr_compare(pMsg->Identifier->buffer, pDaemon->m_info.Identifier->buffer) == 0, 
			"error in osl_executeDaemonRequest");

		if (pMsg->Type == osl_DaemonMgrMsgType_Request)	
		{
			switch (pMsg->Subject)
			{
				case osl_DaemonMgrMsgSubject_CheckConnection:
					break;
				case osl_DaemonMgrMsgSubject_Command:
					pDaemon->m_DaemonControl(pDaemon->m_pControlData, pMsg->Body.Command);
					break;
				case osl_DaemonMgrMsgSubject_StatusGet:
					pMsg->Body.Status = pDaemon->m_status;
					break;
				case osl_DaemonMgrMsgSubject_PropertyCountGet:
					if (pDaemon->m_pPropertyAccess == NULL)
						pMsg->Body.PropertyCount = 0;
					else
					{
						pMsg->Error = 
							pDaemon->m_pPropertyAccess->fGetCount(
								pDaemon->m_pAccessData, 
								&(pMsg->Body.PropertyCount));
					}
					break;
				case osl_DaemonMgrMsgSubject_PropertiesGet:
					if (pDaemon->m_pPropertyAccess == NULL)
					{
						pMsg->Body.Properties.Count = 0;
					}
					else
					{
						pMsg->Error = 
							pDaemon->m_pPropertyAccess->fGetProperties(
								pDaemon->m_pAccessData, 
								&(pMsg->Body.Properties.List),
								&(pMsg->Body.Properties.Count));
					}
					break;
				case osl_DaemonMgrMsgSubject_PropertiesSet:
					if (pDaemon->m_pPropertyAccess == NULL)
					{
						pMsg->Body.Properties.Count = 0;
					}
					else
					{
						rtl_uString *strErrorText = NULL;

						pMsg->Error = 
							pDaemon->m_pPropertyAccess->fSetProperties(
								pDaemon->m_pAccessData, 
								pMsg->Body.Properties.List,
								pMsg->Body.Properties.Count,
								&(pMsg->Body.Properties.ErrorCode),
								&strErrorText);

						if (strErrorText->length)
						{
							rtl_uString_assign(&pMsg->Body.Properties.ErrorText, strErrorText);
						}
					}
					break;
				default:
					pMsg->Error = osl_Daemon_ManagerError_Unknown;
					break;
			}

			pMsg->Type = osl_DaemonMgrMsgType_Reply;
			ret = sal_True;
		}
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_daemonControlFunction */
/*****************************************************************************/

static oslDaemonManagerError osl_daemonControlFunction(osl_TDaemonImpl** pDaemons,	
									  				   sal_uInt32 			 daemonCount)
{
	oslDaemonConnection	conn = NULL;
	oslDaemonManagerError error = osl_Daemon_ManagerError_NoError;
	sal_Bool ready = sal_False;
	TimeValue pollingTime = {1, 0};
	sal_uInt32 i = 0;
	oslDaemonMgrMsg* pRequest = NULL;

	while (!ready)
	{
		conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

		if (conn == NULL)
		{
			/* retry in a few  seconds */
			osl_waitThread(&pollingTime);
		}
		else
		{
			error = osl_Daemon_ManagerError_NoError;

			/* try to connect all services to mananger */
			for (i = 0; 
				 (error == osl_Daemon_ManagerError_NoError) && (i < daemonCount); 
				 i++)
			{
				/* send initial request to connect */
				oslProcessInfo processInfo;
				memset(&processInfo, 0, sizeof(oslProcessInfo));
				processInfo.Size = sizeof(oslProcessInfo);

				osl_getProcessInfo(NULL, osl_Process_IDENTIFIER, &processInfo);

				pRequest = osl_createDaemonMgrMsg(pDaemons[i]->m_info.Identifier, osl_DaemonMgrMsgSubject_Connect);

				pRequest->Type 						= osl_DaemonMgrMsgType_Request;
				pRequest->Body.ProcessIdentifier	= processInfo.Ident;

				error = osl_sendDaemonMgrMsg(conn, pRequest);
				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;

				if (error == osl_Daemon_ManagerError_NoError)
					error = osl_recvDaemonMgrMsg(conn, &pRequest);

				if (error == osl_Daemon_ManagerError_NoError)
				{
					error = pRequest->Error;
					osl_destroyDaemonMgrMsg(pRequest);
					pRequest = NULL;
				}
			}

			/* wait for incoming requests */

			while ((!ready) && (error == osl_Daemon_ManagerError_NoError))
			{
				error = osl_recvDaemonMgrMsg(conn, &pRequest);

				if (error == osl_Daemon_ManagerError_NoError)
				{
					if (pRequest->Subject == osl_DaemonMgrMsgSubject_CloseConnection)
					{
						error = osl_Daemon_ManagerError_NoManager;
					}
					else
					{
						for (
							 i = 0; 
							 (i < daemonCount) && 
							 (rtl_ustr_compare(pRequest->Identifier->buffer, pDaemons[i]->m_info.Identifier->buffer) != 0);
							 i++
							)
						{
						}
						if (i < daemonCount)
						{
							osl_executeDaemonRequest(pRequest, pDaemons[i]);
						}
						else
						{
							pRequest->Error = osl_Daemon_ManagerError_Unknown;
							pRequest->Type = osl_DaemonMgrMsgType_Reply;
						}
						error = osl_sendDaemonMgrMsg(conn, pRequest);
					}

					osl_destroyDaemonMgrMsg(pRequest);
					pRequest = NULL;
				}


				/* check for ready conditions */

				ready = sal_True;

				for (i = 0; ready & (i < daemonCount); i++)
				{
					ready &= osl_checkCondition(pDaemons[i]->m_stopCond);
				}
			}
			
			osl_disconnectDaemonMgrConn(conn);
		}
	}
	
	return(osl_Daemon_ManagerError_NoError);
}	

/*****************************************************************************/
/* osl_default_accessDaemonManager */
/*****************************************************************************/
sal_Bool SAL_CALL osl_default_accessDaemonManager()
{
	oslDaemonConnection conn = NULL;
/*	oslDaemonManagerError error = osl_Daemon_ManagerError_NoError;*/
	sal_Bool ret = sal_False;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn)
	{
		osl_disconnectDaemonMgrConn(conn);
		ret	= sal_True;
	}

	return(ret);
		
}	

/*****************************************************************************/
/* osl_default_queryDaemonCount */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_queryDaemonCount(sal_uInt32* pCount)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_QueryDaemonCount);

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest->Type = osl_DaemonMgrMsgType_Request;
		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
				*pCount = pRequest->Body.DaemonCount;

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
		conn = NULL;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_default_queryDaemonIdentifier */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_queryDaemonIdentifier(sal_uInt32 index, rtl_uString **strIdentifier)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_QueryDaemonIdentifier);

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest->Type 				= osl_DaemonMgrMsgType_Request;
		pRequest->Body.DaemonIndex 	= index;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				rtl_uString_assign(strIdentifier, pRequest->Identifier);
			}
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_isDaemonInstalled */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_isDaemonInstalled(rtl_uString* strIdentifier, sal_Bool* pRet)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_QueryDaemonInfo);
		pRequest->Type = osl_DaemonMgrMsgType_Request;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);

		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		if (ret == osl_Daemon_ManagerError_NoError)
			*pRet = sal_True;

		osl_disconnectDaemonMgrConn(conn);
	}

	return(ret);
}

/*****************************************************************************/
/* osl_default_queryDaemonStatus */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_queryDaemonStatus(rtl_uString* strIdentifier, 
													oslDaemonStatus* pStatus)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_StatusGet);
		pRequest->Type = osl_DaemonMgrMsgType_Request;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);

		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				*pStatus = pRequest->Body.Status;
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}


		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_queryDaemonInfo */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_queryDaemonInfo(rtl_uString* strIdentifier, 
										  		  oslDaemonInfo* pInfo)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_QueryDaemonInfo);
		pRequest->Type = osl_DaemonMgrMsgType_Request;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
				
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;
			if (ret == osl_Daemon_ManagerError_NoError)
			{
				memcpy(pInfo, pRequest->Body.pInfo, sizeof(oslDaemonInfo));
				if (pInfo->Identifier)
					rtl_uString_acquire(pInfo->Identifier);
				if (pInfo->DisplayName)
					rtl_uString_acquire(pInfo->DisplayName);
				if (pInfo->Manufactor)
					rtl_uString_acquire(pInfo->Manufactor);
				if (pInfo->Path)
					rtl_uString_acquire(pInfo->Path);
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}

	return(ret);
}	

/*****************************************************************************/
/* sendDaemonCommand */
/*****************************************************************************/

static oslDaemonManagerError sendDaemonCommand(rtl_uString* strIdentifier, 
											   oslDaemonCommand command)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_Command);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		pRequest->Body.Command = command;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}

/*****************************************************************************/
/* osl_default_startDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_startDaemon(rtl_uString* strIdentifier)
{
	return(sendDaemonCommand(strIdentifier, osl_Daemon_CommandStart));
}	

/*****************************************************************************/
/* osl_default_stopDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_stopDaemon(rtl_uString* strIdentifier)
{
	return(sendDaemonCommand(strIdentifier, osl_Daemon_CommandStop));
}	

/*****************************************************************************/
/* osl_default_pauseDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_pauseDaemon(rtl_uString* strIdentifier)
{
	return(sendDaemonCommand(strIdentifier, osl_Daemon_CommandPause));
}	

/*****************************************************************************/
/* osl_default_pauseDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_continueDaemon(rtl_uString* strIdentifier)
{
	return(sendDaemonCommand(strIdentifier, osl_Daemon_CommandContinue));
}	

/*****************************************************************************/
/* osl_default_installDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_installDaemon(const oslDaemonInfo* pInfo)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_Install);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		memcpy(pRequest->Body.pInfo, pInfo, sizeof(oslDaemonInfo));

		if (pInfo->Identifier)
			rtl_uString_acquire(pInfo->Identifier);
		if (pInfo->DisplayName)
			rtl_uString_acquire(pInfo->DisplayName);
		if (pInfo->Manufactor)
			rtl_uString_acquire(pInfo->Manufactor);
		if (pInfo->Path)
			rtl_uString_acquire(pInfo->Path);

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_default_uninstallDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_uninstallDaemon(rtl_uString* strIdentifier)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_Uninstall);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_createDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_createDaemon(rtl_uString* strIdentifier,
								   		  	   oslDaemonControl pControl, 
								  		  	   void* pControlData,
											   oslDaemonPropertyAccess* pPropertyAccess,
								  		  	   void* pAccessData,
								  		  	   oslDaemonHandle* pHandle)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	osl_TDaemonImpl* pDaemonImpl = (osl_TDaemonImpl*)rtl_allocateZeroMemory(sizeof(osl_TDaemonImpl));

	OSL_ASSERT(pDaemonImpl);

	pDaemonImpl->m_DaemonControl  = pControl;
	pDaemonImpl->m_pPropertyAccess  = pPropertyAccess;
	pDaemonImpl->m_pControlData = pControlData;
	pDaemonImpl->m_pAccessData = pAccessData;
	pDaemonImpl->m_status = osl_Daemon_StatusUnknown;
	pDaemonImpl->m_stopCond = osl_createCondition();

	pDaemonImpl->m_info.Size = sizeof(pDaemonImpl->m_info);

	ret = osl_queryDaemonInfo(strIdentifier, &(pDaemonImpl->m_info));
	
	if (ret == osl_Daemon_ManagerError_NoError)
	{
		*pHandle = pDaemonImpl;
	}
	else
	{
		rtl_freeMemory(pDaemonImpl);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_default_executeDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_executeDaemon(oslDaemonHandle handle)
{
	osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)handle;

	OSL_ASSERT(pDaemonImpl != NULL);

	pDaemonImpl->m_status = osl_Daemon_StatusStopped;
	
	return(osl_daemonControlFunction(&pDaemonImpl, 1));
}	

/*****************************************************************************/
/* osl_default_executeDaemons */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_executeDaemons(oslDaemonHandle* pHandle, sal_uInt32 numOfDaemons)
{
	osl_TDaemonImpl** ppDaemonImpl= (osl_TDaemonImpl**)pHandle;
	sal_uInt32 i = 0;

	OSL_ASSERT(ppDaemonImpl != NULL);

	for (i = 0; i < numOfDaemons; i++) 
	{
		ppDaemonImpl[i]->m_status = osl_Daemon_StatusStopped;
	}

	return(osl_daemonControlFunction(ppDaemonImpl, numOfDaemons));
}	


/*****************************************************************************/
/* osl_default_setDaemonStatus */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_setDaemonStatus(oslDaemonHandle handle, oslDaemonStatus Status)
{
	osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)handle;

	OSL_ASSERT(pDaemonImpl != NULL);

	pDaemonImpl->m_status = Status;

	if (Status == osl_Daemon_StatusStopped)
		osl_setCondition(pDaemonImpl->m_stopCond);
	else
		osl_resetCondition(pDaemonImpl->m_stopCond);


	return(osl_Daemon_ManagerError_NoError);
}	

/*****************************************************************************/
/* osl_default_endDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_endDaemon(oslDaemonHandle handle)
{
	osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)handle;

	OSL_ASSERT(pDaemonImpl != NULL);

	osl_destroyCondition(pDaemonImpl->m_stopCond);
	free(pDaemonImpl);

	return(osl_Daemon_ManagerError_NoError);
}	

/*****************************************************************************/
/* osl_default_getDaemonPropertyCount */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_getDaemonPropertyCount(rtl_uString* strIdentifier, sal_uInt32* pCount)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_PropertyCountGet);

		pRequest->Type = osl_DaemonMgrMsgType_Request;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				*pCount = pRequest->Body.PropertyCount;
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_default_getDaemonProperties */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_getDaemonProperties(rtl_uString* strIdentifier, 
													  oslDaemonProperty** ppProps, 
													  sal_uInt32* pCount)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_PropertiesGet);

		pRequest->Type = osl_DaemonMgrMsgType_Request;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				*pCount = pRequest->Body.Properties.Count;
				*ppProps = pRequest->Body.Properties.List;

				/* do not destroy recieved property list */
				pRequest->Body.Properties.Count = 0;
				pRequest->Body.Properties.List = NULL;
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_setDaemonProperties */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_setDaemonProperties(rtl_uString* strIdentifier, 
													  oslDaemonProperty* pProps, 
													  sal_uInt32 count, 
													  sal_uInt32* pErrorCode, 
													  rtl_uString** strErrorText)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_PropertiesSet);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		pRequest->Body.Properties.List = pProps;
		pRequest->Body.Properties.Count = count;
				
		ret = osl_sendDaemonMgrMsg(conn, pRequest);

		/* do not destroy property list */
		pRequest->Body.Properties.List = NULL;
		pRequest->Body.Properties.Count = 0;
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			*pErrorCode = pRequest->Body.Properties.ErrorCode;

			if (pRequest->Body.Properties.ErrorText != NULL)
			{
				rtl_uString_assign(strErrorText, pRequest->Body.Properties.ErrorText);
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	


/*****************************************************************************/
/* osl_default_getDaemonProfileSections */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_getDaemonProfileSections(rtl_uString* strIdentifier, 
														   sal_Unicode* buffer, 
														   sal_uInt32 bufferSize,
														   sal_uInt32* pRequiredSize)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_ProfileGetSections);

		pRequest->Type = osl_DaemonMgrMsgType_Request;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				if (bufferSize >= 2)
				{
					buffer[0] = 0;
					buffer[1] = 0;
				}
				else
					*pRequiredSize = 2;

				if (pRequest->Body.Profile.Section != NULL)
				{
					sal_uInt32 charCount = 0;

					while (pRequest->Body.Profile.Section[charCount] != 0)
						charCount += rtl_ustr_getLength(pRequest->Body.Profile.Section + charCount) + 1;

					if (bufferSize >= 2)
					{
						if (charCount++)
							rtl_copyMemory(buffer, pRequest->Body.Profile.Section, charCount * sizeof(sal_Unicode));
					}
					else
						*pRequiredSize = ++charCount;
				}
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_getDaemonProfileEntries */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_getDaemonProfileEntries(rtl_uString* strIdentifier, 
														  rtl_uString* strSection, 
														  sal_Unicode* buffer, 
														  sal_uInt32 bufferSize,
														  sal_uInt32* pRequiredSize)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_ProfileGetEntries);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		pRequest->Body.Profile.Section = strSection->buffer;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				if (bufferSize >= 2)
				{
					buffer[0] = '\0';
					buffer[1] = '\0';
				}
				else
					*pRequiredSize = 2;
					
				if (pRequest->Body.Profile.Entry != NULL)
				{
					sal_uInt32 charCount = 0;

					while (pRequest->Body.Profile.Entry[charCount] != 0)
						charCount += rtl_ustr_getLength(pRequest->Body.Profile.Entry + charCount) + 1;

					if (bufferSize >= 2)
					{
						if (charCount++)
							rtl_copyMemory(buffer, pRequest->Body.Profile.Entry, charCount * sizeof(sal_Unicode));
					}
					else
						*pRequiredSize = ++charCount;
				}
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_readDaemonProfileString */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_readDaemonProfileString(rtl_uString* strIdentifier, 
														  rtl_uString* strSection, 
														  rtl_uString* strEntry, 
														  rtl_uString** strValue, 
														  rtl_uString* strDefaultValue)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_ProfileReadEntry);

		pRequest->Type = osl_DaemonMgrMsgType_Request;

		pRequest->Body.Profile.Section = strSection->buffer;
		pRequest->Body.Profile.Entry = strEntry->buffer;
		pRequest->Body.Profile.Default = strDefaultValue->buffer;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				if (pRequest->Body.Profile.Value == NULL)
					rtl_uString_assign(strValue, strDefaultValue);
				else
					rtl_uString_newFromStr(strValue, pRequest->Body.Profile.Value);
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_readDaemonProfileBool */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_readDaemonProfileBool(rtl_uString* strIdentifier, 
													    rtl_uString* strSection, 
														rtl_uString* strEntry,
														sal_Bool* pValue, 
														sal_Bool defaultValue)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;
	rtl_uString *strValue = NULL;
	rtl_uString *strDefaultValue = NULL;

	rtl_uString_new_WithLength(&strDefaultValue, RTL_USTR_MAX_VALUEOFBOOLEAN);
	rtl_ustr_valueOfBoolean(strDefaultValue->buffer, defaultValue);

	ret = osl_default_readDaemonProfileString(strIdentifier,
											  strSection,
											  strEntry,
											  &strValue,
											  strDefaultValue);

	if (ret == osl_Daemon_ManagerError_NoError)
	{
		rtl_uString_new_WithLength(&strDefaultValue, RTL_USTR_MAX_VALUEOFBOOLEAN);
		rtl_ustr_valueOfBoolean(strDefaultValue->buffer, sal_True);

		*pValue = rtl_ustr_compare(strDefaultValue->buffer, strValue->buffer) == 0;
	}

	rtl_uString_release(strValue);
	rtl_uString_release(strDefaultValue);
	return(ret);
}	

/*****************************************************************************/
/* osl_default_readDaemonProfileIdent */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_readDaemonProfileIdent(rtl_uString* strIdentifier, 
													     rtl_uString* strSection, 
														 rtl_uString* strEntry,
														 sal_uInt32* pValue, 
														 sal_uInt32 firstId, 
														 rtl_uString* strings[], 
														 sal_uInt32 defaultValue)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;
	rtl_uString *strValue = NULL;

	ret = osl_default_readDaemonProfileString(strIdentifier, 
											  strSection, 
											  strEntry, 
											  &strValue, 
											  strings[defaultValue - firstId]);

	if (ret == osl_Daemon_ManagerError_NoError)
	{
		sal_uInt32 i = 0;

		while ((strings[i] != NULL) && (rtl_ustr_compare(strings[i]->buffer, strValue->buffer) != 0))
		{
			i++;
		}

		if (strings[i] != NULL)
		{
			*pValue = firstId + i;
		}
		else
		{
			*pValue = defaultValue;
		}
	}

	rtl_uString_release(strValue);
	return(ret);
}	

/*****************************************************************************/
/* osl_default_writeDaemonProfileString */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_writeDaemonProfileString(rtl_uString* strIdentifier, 
													       rtl_uString* strSection, 
														   rtl_uString* strEntry,
														   rtl_uString* strValue)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_ProfileWriteEntry);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		pRequest->Body.Profile.Section = strSection->buffer;
		pRequest->Body.Profile.Entry = strEntry->buffer;
		pRequest->Body.Profile.Value = strValue->buffer;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_default_writeDaemonProfileBool */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_writeDaemonProfileBool(rtl_uString* strIdentifier, 
													       rtl_uString* strSection, 
														   rtl_uString* strEntry,
														   sal_Bool value)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;
	rtl_uString *strValue = NULL;

	rtl_uString_new_WithLength(&strValue, RTL_USTR_MAX_VALUEOFBOOLEAN);
	rtl_ustr_valueOfBoolean(strValue->buffer, value);

	ret = osl_default_writeDaemonProfileString(strIdentifier, 
											   strSection, 
											   strEntry, 
											   strValue);

	rtl_uString_release(strValue);
	return(ret);
}	

/*****************************************************************************/
/* osl_default_writeDaemonProfileIdent */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_writeDaemonProfileIdent(rtl_uString* strIdentifier, 
													       rtl_uString* strSection, 
														   rtl_uString* strEntry, 
														  sal_uInt32 value, 
														  sal_uInt32 firstId, 
														  rtl_uString* strings[])
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;

	ret = osl_default_writeDaemonProfileString(strIdentifier, 
											   strSection, 
											   strEntry, 
											   strings[value + firstId]);

	return(ret);
}	

/*****************************************************************************/
/* osl_default_removeDaemonProfileEntry */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_default_removeDaemonProfileEntry(rtl_uString* strIdentifier, 
													       rtl_uString* strSection, 
														   rtl_uString* strEntry)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_ProfileRemoveEntry);

		pRequest->Type = osl_DaemonMgrMsgType_Request;
		pRequest->Body.Profile.Section = strSection->buffer;
		pRequest->Body.Profile.Entry = strEntry->buffer;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

/*****************************************************************************/
/* osl_default_queryDaemonChanges */
/*****************************************************************************/

oslDaemonManagerError SAL_CALL osl_default_queryDaemonChanges(TimeValue* pSince, sal_Bool* pBChanged)
{
	oslDaemonConnection conn = NULL;
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;
	oslDaemonMgrMsg* pRequest = NULL;

	conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_QueryChanges);

		pRequest->Type = osl_DaemonMgrMsgType_Request;

		pRequest->Body.Changes.since = *pSince;

		ret = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
		
		if (ret == osl_Daemon_ManagerError_NoError)
			ret = osl_recvDaemonMgrMsg(conn, &pRequest);

		if (ret == osl_Daemon_ManagerError_NoError)
		{
			ret = pRequest->Error;

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				*pSince = pRequest->Body.Changes.since;
				*pBChanged = pRequest->Body.Changes.bChanged;
			}

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	
	return(ret);
}	

