/*************************************************************************
 *
 *  $RCSfile: daemman.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string.h>
#include <stdio.h>

#ifndef _OSL_THREAD_H_
#   include <osl/thread.h>
#endif
#ifndef _OSL_TIME_H
#   include <osl/time.h>
#endif
#ifndef _OSL_SIGNAL_H_
#   include <osl/signal.h>
#endif	 
#ifndef _OSL_MUTEX_H_
#   include <osl/mutex.h>
#endif
#ifndef _OSL_PROCESS_H_
#   include <osl/process.h>
#endif
#ifndef _OSL_DAEMON_H_
#   include <osl/daemon.h>
#endif
#ifndef _OSL_DEAMMAN_H_
#   include "daemman.h"
#endif
#ifndef _OSL_DAEMPROT_H_
#   include "daemprot.h"
#endif
#ifndef _OSL_DAEMREG_H_
#   include "daemreg.h"
#endif

#include <osl/diagnose.h>
#include <rtl/alloc.h>

#define DAEMON_MAX_ENTRY_SIZE			1024

#define DAEMON_MAX(a,b)            (((a) > (b)) ? (a) : (b))

#define OSL_DAEMON_MANAGER_PIPE "SDAEMON"
#define DAEMON_MAGIC 			0xdaef37b1
#define DAEMON_VERSION 			1
#define HEAD_LEN				sizeof(sal_uInt32) + sizeof(sal_uInt32) + sizeof(sal_uInt32)
#define INIT_SIZE 				128

#if defined( WNT ) || defined( OS2 )
#define PATH_DELEMITTER	"\\"
#elif defined( UNX )
#define PATH_DELEMITTER	"/"
#elif defined( MAC )
#define PATH_DELEMITTER	":"
#else
#define PATH_DELEMITTER	"/"
#endif

typedef struct
{
	sal_uInt32 	size;
	sal_uInt32	pos;
	sal_uInt8*	pData;	
} Buffer;
	

#define DAEMON_COMAND_MAX_RETRY 		10

static TimeValue pollingTime = {1, 0};
static TimeValue startupTime = {5, 0};

typedef struct
{
	oslThread 			hThread;
	oslDaemonConnection	hConnection;
		
} oslDaemonManagerThread;

/*****************************************************************************/
/* ADT active daemon list declarations */
/*****************************************************************************/

typedef struct _activeDaemonEntry
{
	rtl_uString	*				Identifier;
	oslDaemonConnection			Connection;
	oslProcessIdentifier		ProcessIdent;
	oslMutex					Mutex;
} activeDaemonEntry;
	
static oslMutex 			globalDaemonMutex = NULL;
static DaemonRegistryApi* 	pGlobalRegistry = NULL;

static activeDaemonEntry** 	activeDaemonList = NULL;
static sal_uInt32				activeDaemonSize = 0;

static activeDaemonEntry* 	getDaemonEntry(rtl_uString* Identifier);
static activeDaemonEntry* 	addDaemonEntry(rtl_uString* Identifier, oslDaemonConnection Conn, oslProcessIdentifier procIdent);
static void 				releaseDaemonEntry(activeDaemonEntry* pEntry);
static void 				shutdownAllConnections();
static void 				killAllConnections();


/*****************************************************************************/
/* helper methods */
/*****************************************************************************/

static oslSignalHandler hSignalHandler = NULL;

static oslSignalAction SAL_CALL DaemonManagerSignalHandler(void* pData, oslSignalInfo* pInfo)
{
	if (
		(pInfo->Signal == osl_Signal_Terminate)	||
		(pInfo->Signal == osl_Signal_AccessViolation) ||
		(pInfo->Signal == osl_Signal_IntegerDivideByZero) || 
		(pInfo->Signal == osl_Signal_FloatDivideByZero)
	   )	
	{
		killAllConnections();
		osl_deinitDaemonManager();
	}

	return (osl_Signal_ActCallNextHdl);
}

static oslDaemonManagerError getDaemonInfo(rtl_uString* Identifier, oslDaemonInfo* pInfo);
static oslDaemonManagerError queryDaemonCount(sal_uInt32* pCount);
static oslDaemonManagerError queryDaemonIdentifier(sal_uInt32 Index, rtl_uString** Identifier);
static oslDaemonManagerError connectFromDaemon(rtl_uString* Identifier, oslDaemonConnection Conn, oslProcessIdentifier procIdent);
static oslDaemonManagerError install(oslDaemonInfo* pInfo);
static oslDaemonManagerError uninstall(rtl_uString* Identifier);
static oslDaemonManagerError startDaemon(oslDaemonInfo* pInfo);
static oslDaemonManagerError stopDaemon(oslDaemonInfo* pInfo);
static oslDaemonManagerError pauseDaemon(oslDaemonInfo* pInfo);
static oslDaemonManagerError continueDaemon(oslDaemonInfo* pInfo);
static oslDaemonManagerError shutdownDaemon(oslDaemonInfo* pInfo);
static oslDaemonManagerError command(rtl_uString* Identifier, oslDaemonCommand Command);
static oslDaemonManagerError getStatus(rtl_uString* Identifier, oslDaemonStatus* pStatus);
static oslDaemonManagerError getStatusByEntry(activeDaemonEntry* pEntry, oslDaemonStatus* pStatus);
static oslDaemonManagerError sendCommandByEntry(activeDaemonEntry* pEntry, oslDaemonCommand command, oslDaemonStatus statusPre, oslDaemonStatus statusPost, oslDaemonManagerError preFailedError);

/*****************************************************************************/
/* queryChanges */
/*****************************************************************************/

static TimeValue lastChanges = {0, 0};

static void setChangedTime()
{
	osl_acquireMutex(globalDaemonMutex);

	osl_getSystemTime(&lastChanges);

	osl_releaseMutex(globalDaemonMutex);
}	

static void queryChanges(oslDaemonMgrMsg* pRequest)
{
	osl_acquireMutex(globalDaemonMutex);

	if ((lastChanges.Seconds == 0) && (lastChanges.Nanosec == 0))
	{
		osl_getSystemTime(&lastChanges);
		pRequest->Body.Changes.bChanged = sal_True;
		pRequest->Body.Changes.since = lastChanges;
	}
	else
	{
		pRequest->Body.Changes.bChanged = sal_False;

		if (lastChanges.Seconds == pRequest->Body.Changes.since.Seconds)
		{
			if (lastChanges.Nanosec > pRequest->Body.Changes.since.Nanosec)
				pRequest->Body.Changes.bChanged = sal_True;
		}
		else if (lastChanges.Seconds > pRequest->Body.Changes.since.Seconds)
			pRequest->Body.Changes.bChanged = sal_True;

		pRequest->Body.Changes.since = lastChanges;
	}

	osl_releaseMutex(globalDaemonMutex);
}

/*****************************************************************************/
/* ADT active daemon list implementation */
/* compareStringWithDaemonEntry */
/*****************************************************************************/

#if 0
static int compareStringWithDaemonEntry( const sal_Char** pSearchString, const activeDaemonEntry** pSearchElement )
{
	return(strcmp(*pSearchString, (*pSearchElement)->Identifier));
}	
#endif

/*****************************************************************************/
/* ADT active daemon list implementation */
/* checkDaemonConnection */
/*****************************************************************************/

static sal_Bool checkDaemonConnection(activeDaemonEntry* pEntry)
{
	/* try to send an control request to an existing connection */
	sal_Bool ret = sal_False;
	if (pEntry->Connection)
	{
		oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(pEntry->Identifier, osl_DaemonMgrMsgSubject_CheckConnection);
/*		oslDaemonManagerError error = osl_Daemon_ManagerError_NoError;*/

		if (osl_sendDaemonMgrMsg(pEntry->Connection, pRequest) == osl_Daemon_ManagerError_NoError)
		{
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
			
			if (osl_recvDaemonMgrMsg(pEntry->Connection, &pRequest) == osl_Daemon_ManagerError_NoError)
			{
				ret = (pRequest->Error == osl_Daemon_ManagerError_NoError);

				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}
		}
		else
		{
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
		}
	}

	return(ret);
}	

/*****************************************************************************/
/* ADT active daemon list implementation */
/* getDaemonEntry */
/*****************************************************************************/

static activeDaemonEntry* getDaemonEntry(rtl_uString* Identifier)
{
	activeDaemonEntry* pRet= NULL;
	sal_uInt32 i = 0;

	osl_acquireMutex(globalDaemonMutex);

	if (activeDaemonSize)
	{
/*		pRet = *(activeDaemonEntry**) 
			bsearch(&Identifier, 
				     activeDaemonList, 
					 activeDaemonSize, 
					 sizeof(activeDaemonEntry*), 
					 (int (*)(const void*, const void*))compareStringWithDaemonEntry);*/

		for (i = 0;
			 (i < activeDaemonSize) && (rtl_ustr_compare(Identifier->buffer, activeDaemonList[i]->Identifier->buffer) != 0);
			 i++)
		{
		}
	
		if (i < activeDaemonSize)	
			pRet = activeDaemonList[i];
	}

	osl_releaseMutex(globalDaemonMutex);

	if (pRet) osl_acquireMutex(pRet->Mutex);

	return(pRet);
}	

/*****************************************************************************/
/* ADT active daemon list implementation */
/* addDaemonEntry */
/*****************************************************************************/

static activeDaemonEntry* addDaemonEntry(rtl_uString* Identifier, oslDaemonConnection Conn, oslProcessIdentifier procIdent)
{
	activeDaemonEntry* pEntry= NULL;
	sal_uInt32 i = 0;
	activeDaemonEntry** tmpList = NULL;

	osl_acquireMutex(globalDaemonMutex);

	/* find the first entry in sorted list which identifier
		is greater or equal the given identifier */
	for (i = 0;
		 (i < activeDaemonSize) && (rtl_ustr_compare(Identifier->buffer, activeDaemonList[i]->Identifier->buffer) < 0);
		 i++)
	{
	}

 	if ((i < activeDaemonSize) && (rtl_ustr_compare(Identifier->buffer, activeDaemonList[i]->Identifier->buffer) == 0))
 	{
		pEntry = activeDaemonList[i];

		osl_acquireMutex(pEntry->Mutex);

		/* found an existing entry -> check for validity */
		if (!checkDaemonConnection(pEntry))
		{
			/* only reuse this entry if the connection is down 
				and no other thread is using this entry */
			if (pEntry->Connection) osl_disconnectDaemonMgrConn(pEntry->Connection);
			pEntry->Connection = Conn;
			pEntry->ProcessIdent = procIdent;
		}
		else
		{
			osl_releaseMutex(pEntry->Mutex);
			pEntry = NULL;
		}
	}
	else
	{
		/* create a new entry */
		pEntry = (activeDaemonEntry*) rtl_allocateZeroMemory(sizeof(activeDaemonEntry));
		rtl_uString_assign(&pEntry->Identifier, Identifier);
		pEntry->Connection = Conn;
		pEntry->ProcessIdent = procIdent;
		pEntry->Mutex = osl_createMutex();

		/* add entry to list */
		activeDaemonSize++;
		tmpList = (activeDaemonEntry**) rtl_allocateMemory(activeDaemonSize * sizeof(activeDaemonEntry*));
		memcpy(tmpList, activeDaemonList, i * sizeof(activeDaemonEntry*));
		tmpList[i] = pEntry;
		memcpy(&(tmpList[i + 1]), &(activeDaemonList[i]), (activeDaemonSize - (i + 1)) * sizeof(activeDaemonEntry*));
		activeDaemonList = tmpList;	 

		osl_acquireMutex(pEntry->Mutex);
	}

	osl_releaseMutex(globalDaemonMutex);
		
	return(pEntry);
}	

/*****************************************************************************/
/* ADT active daemon list implementation */
/* releaseDaemonEntry */
/*****************************************************************************/

static void releaseDaemonEntry(activeDaemonEntry* pEntry)
{
	osl_releaseMutex(pEntry->Mutex);
}	

/*****************************************************************************/
/* killAllConnections */
/*****************************************************************************/

static void killAllConnections()
{
	sal_uInt32 i = 0;

	osl_acquireMutex(globalDaemonMutex);

	for (i = 0; i < activeDaemonSize; i++)
	{
		oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_CloseConnection);

		osl_acquireMutex(activeDaemonList[i]->Mutex);

		osl_sendDaemonMgrMsg(activeDaemonList[i]->Connection, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		osl_disconnectDaemonMgrConn(activeDaemonList[i]->Connection);
		activeDaemonList[i]->Connection = NULL;

		osl_releaseMutex(activeDaemonList[i]->Mutex);
	}

	osl_releaseMutex(globalDaemonMutex);
}	

/*****************************************************************************/
/* shutdownAllConnections */
/*****************************************************************************/

static void shutdownAllConnections()
{
	sal_uInt32 i = 0;

	osl_acquireMutex(globalDaemonMutex);

	for (i = 0; i < activeDaemonSize; i++)
	{
		oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(activeDaemonList[i]->Identifier, osl_DaemonMgrMsgSubject_Command);
		oslDaemonMgrMsg* pReply = NULL;

		osl_acquireMutex(activeDaemonList[i]->Mutex);

		pRequest->Body.Command = osl_Daemon_CommandShutdown;
		if (osl_sendDaemonMgrMsg(activeDaemonList[i]->Connection, pRequest) == osl_Daemon_ManagerError_NoError)
		{
			if (osl_recvDaemonMgrMsg(activeDaemonList[i]->Connection, &pReply) == osl_Daemon_ManagerError_NoError)
			{
				osl_destroyDaemonMgrMsg(pReply);
				pReply = NULL;
			}
		}

		osl_releaseMutex(activeDaemonList[i]->Mutex);

		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
	}

	osl_releaseMutex(globalDaemonMutex);

	killAllConnections();
}	

/*****************************************************************************/
/* getDaemonInfo */
/*****************************************************************************/

static oslDaemonManagerError getDaemonInfo(rtl_uString* Identifier, oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	sal_Unicode	buffer[2048];

	osl_acquireMutex(globalDaemonMutex);

	if (pGlobalRegistry->hasDaemon(Identifier))
    {
		rtl_uString *strEntry = NULL;
		rtl_uString *strDefault = NULL;

		/* identifier */
		rtl_uString_newFromAscii(&strEntry, SERVICES_IDENTIFIER_NAME);
		rtl_uString_newFromAscii(&strDefault, "");
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              buffer,
                              OSL_DAEMON_IDENTIFIER_LEN,
                              strDefault->buffer);
		rtl_uString_newFromStr(&pInfo->Identifier, buffer);
		/* display name */
		rtl_uString_newFromAscii(&strEntry, SERVICES_DISPLAY_NAME);
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              buffer,
                              OSL_DAEMON_NAME_LEN,
                              strDefault->buffer);
		rtl_uString_newFromStr(&pInfo->DisplayName, buffer);

		/* manufactor */
		rtl_uString_newFromAscii(&strEntry, SERVICES_MANUFACTOR_NAME);
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              buffer,
                              OSL_DAEMON_MANUFACTOR_LEN,
                              strDefault->buffer);
		rtl_uString_newFromStr(&pInfo->Manufactor, buffer);

		/* path */
		rtl_uString_newFromAscii(&strEntry, SERVICES_PATH_NAME);
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              buffer,
                              OSL_DAEMON_PATH_LEN,
                              strDefault->buffer);
		rtl_uString_newFromStr(&pInfo->Path, buffer);

		/* dependencies */
		rtl_uString_newFromAscii(&strEntry, SERVICES_DEPENDENCIES_NAME);
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              pInfo->Dependencies,
                              OSL_DAEMON_DEPENDENCIES_LEN,
                              strDefault->buffer);

		/* version */
		rtl_uString_newFromAscii(&strEntry, SERVICES_VERSION_NAME);
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              buffer,
                              sizeof(buffer),
                              strDefault->buffer);
		pInfo->Version = rtl_ustr_toInt32(buffer, 10);

		/* mode */
		rtl_uString_newFromAscii(&strEntry, SERVICES_MODE_NAME);
		pGlobalRegistry->readDaemonEntry(
                              Identifier, 
							  NULL,
                              strEntry, 
                              buffer,
                              sizeof(buffer),
                              strDefault->buffer);

		rtl_uString_newFromAscii(&strDefault, "automatic");
		if (rtl_ustr_compare(buffer, strDefault->buffer) == 0)
			pInfo->Mode = osl_Daemon_Automatic;
		else
			pInfo->Mode = osl_Daemon_OnDemand;

		rtl_uString_release(strEntry);
		rtl_uString_release(strDefault);
	}
	else
		ret = osl_Daemon_ManagerError_NotInstalled;

	osl_releaseMutex(globalDaemonMutex);
	
	return(ret);
}	

/*****************************************************************************/
/* queryDaemonCount */
/*****************************************************************************/

static oslDaemonManagerError queryDaemonCount(sal_uInt32* pCount)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	sal_uInt32 max = 0;
	sal_Unicode* entries = NULL;
	sal_Unicode* pChar = NULL;

	osl_acquireMutex(globalDaemonMutex);

        *pCount = 0;
        
	max = pGlobalRegistry->getDaemons(NULL, 0);

        if (max > 0)
        {
            entries = (sal_Unicode*)rtl_allocateMemory(max * sizeof(sal_Unicode));

            max = pGlobalRegistry->getDaemons(entries, max);

            pChar = entries;

            while (*pChar != 0)
            {
				pChar += (rtl_ustr_getLength(pChar) + 1);
				(*pCount)++;
            }
        }

	rtl_freeMemory(entries);

	osl_releaseMutex(globalDaemonMutex);

	return(ret);
}	

/*****************************************************************************/
/* queryDaemonIdentifier */
/*****************************************************************************/

static oslDaemonManagerError queryDaemonIdentifier(sal_uInt32 Index, rtl_uString** Identifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	sal_uInt32 i = 0, max = 0;
	sal_Unicode* entries = NULL;
	sal_Unicode* pChar = NULL;		

	osl_acquireMutex(globalDaemonMutex);

	max = pGlobalRegistry->getDaemons(NULL, 0);

	if (max <= 0)
		ret = osl_Daemon_ManagerError_NotInstalled;
	else
	{
		entries = (sal_Unicode*)rtl_allocateMemory(max * sizeof(sal_Unicode));		

		max = pGlobalRegistry->getDaemons(entries, max);

		i = 0;
		pChar = entries;

		while ((i < Index) && (*pChar != 0))
		{
			pChar += (rtl_ustr_getLength(pChar) + 1);
			i++;
		}

		if (rtl_ustr_getLength(pChar) > 0)
		{
			rtl_uString_newFromStr(Identifier, pChar);
		}

		rtl_freeMemory(entries);
	}

	osl_releaseMutex(globalDaemonMutex);

	return(ret);
}	

/*****************************************************************************/
/* connectFromDaemon */
/*****************************************************************************/

static oslDaemonManagerError connectFromDaemon(rtl_uString* Identifier, oslDaemonConnection Conn, oslProcessIdentifier procIdent)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	activeDaemonEntry* pEntry = addDaemonEntry(Identifier, Conn, procIdent);

	if (!pEntry)
		ret = osl_Daemon_ManagerError_AllreadyRunning;
	else
	{
		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
		
	return(ret);
}	

/*****************************************************************************/
/* install */
/*****************************************************************************/

static oslDaemonManagerError install(oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;

	osl_acquireMutex(globalDaemonMutex);

	if (!pGlobalRegistry->hasDaemon(pInfo->Identifier))
    {
		sal_Unicode buffer[64];
		rtl_uString *strEntry = NULL;
		rtl_uString *strValue = NULL;

		pGlobalRegistry->addDaemon(pInfo->Identifier, pInfo->DisplayName);

		/* identifier */
		rtl_uString_newFromAscii(&strEntry, SERVICES_IDENTIFIER_NAME);
		pGlobalRegistry->writeDaemonEntry(
                               pInfo->Identifier, 
							   NULL,
                               strEntry, 
							   pInfo->Identifier->buffer);								

		/* display name */
		rtl_uString_newFromAscii(&strEntry, SERVICES_DISPLAY_NAME);
		pGlobalRegistry->writeDaemonEntry(
                               pInfo->Identifier, 
							   NULL,
                               strEntry, 
							   pInfo->DisplayName->buffer);								

		/* manufactor */
		rtl_uString_newFromAscii(&strEntry, SERVICES_MANUFACTOR_NAME);
		pGlobalRegistry->writeDaemonEntry(
                               pInfo->Identifier, 
							   NULL,
                               strEntry, 
							   pInfo->Manufactor->buffer);								

		/* path */
		rtl_uString_newFromAscii(&strEntry, SERVICES_PATH_NAME);
		pGlobalRegistry->writeDaemonEntry(
                               pInfo->Identifier, 
							   NULL,
                               strEntry, 
							   pInfo->Path->buffer);								

		/* dependencies */
		rtl_uString_newFromAscii(&strEntry, SERVICES_DEPENDENCIES_NAME);
		pGlobalRegistry->writeDaemonEntry(
                               pInfo->Identifier, 
							   NULL,
                               strEntry, 
							   pInfo->Dependencies);								

		/* version */
		rtl_uString_newFromAscii(&strEntry, SERVICES_VERSION_NAME);
		rtl_ustr_valueOfInt32(buffer, pInfo->Version, 10);
		pGlobalRegistry->writeDaemonEntry(
                               pInfo->Identifier, 
							   NULL,
                               strEntry, 
							   buffer);								

		/* mode */
		rtl_uString_newFromAscii(&strEntry, SERVICES_MODE_NAME);
		rtl_uString_newFromAscii(&strValue, (pInfo->Mode == osl_Daemon_Automatic) ? SERVICES_MODE_AUTOMATIC : SERVICES_MODE_ONDEMAND);
		pGlobalRegistry->writeDaemonEntry(
			pInfo->Identifier, 
			NULL,
	        strEntry, 
			strValue->buffer);								

		rtl_uString_release(strEntry);
		rtl_uString_release(strValue);
	}
	else
		ret = osl_Daemon_ManagerError_AllreadyInstalled;


	osl_releaseMutex(globalDaemonMutex);

	setChangedTime();

	return(ret);
}	

/*****************************************************************************/
/* uninstall */
/*****************************************************************************/

static oslDaemonManagerError uninstall(rtl_uString* Identifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;

	osl_acquireMutex(globalDaemonMutex);

	if (pGlobalRegistry->hasDaemon(Identifier))
    {
		pGlobalRegistry->removeDaemon(Identifier);
	}
	else
		ret = osl_Daemon_ManagerError_NotInstalled;

	osl_releaseMutex(globalDaemonMutex);

	setChangedTime();
	
	return(ret);
}	
/*****************************************************************************/
/* sendCommandByEntry */
/*****************************************************************************/

static oslDaemonManagerError sendCommandByEntry(activeDaemonEntry* pEntry, 
					  					  		oslDaemonCommand command, 
										  		oslDaemonStatus statusPre, 
										  		oslDaemonStatus statusPost, 
										  		oslDaemonManagerError preFailedError)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	oslDaemonMgrMsg* pRequest = NULL;
	oslDaemonStatus status = osl_Daemon_StatusUnknown;
	sal_uInt32 retry = DAEMON_COMAND_MAX_RETRY;

	setChangedTime();

	/* got connection -> get actual status */
	ret = getStatusByEntry(pEntry, &status);

	if (ret == osl_Daemon_ManagerError_NoError)
	{
		if (status != statusPre)
		{
			ret = preFailedError;
		}
		else
		{
			/* send request */
			pRequest = osl_createDaemonMgrMsg(pEntry->Identifier, osl_DaemonMgrMsgSubject_Command);
			pRequest->Body.Command = command;

			ret = osl_sendDaemonMgrMsg(pEntry->Connection, pRequest);

			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;

			if (ret == osl_Daemon_ManagerError_NoError)
				ret = osl_recvDaemonMgrMsg(pEntry->Connection, &pRequest);

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				ret = pRequest->Error;
				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				/* polling for actual status changed */
				retry = DAEMON_COMAND_MAX_RETRY;
				ret = getStatusByEntry(pEntry, &status);

				while (
					   (ret == osl_Daemon_ManagerError_NoError) && 
					   (--retry > 0) &&
					   (status != statusPost)
					  )
				{
					osl_waitThread(&pollingTime);
					ret = getStatusByEntry(pEntry, &status);
				}

			   	if (
			   		(ret == osl_Daemon_ManagerError_NoError) && 
			   		(status != statusPost)
				   )
				{
					/* timeout reached */
					ret = osl_Daemon_ManagerError_Timeout;
				}

				if (
					(ret != osl_Daemon_ManagerError_NoError) &&
					(statusPost == osl_Daemon_StatusStopped)
				   )
				{
					/* ignore connection lost while waiting fot stop status */
					ret = osl_Daemon_ManagerError_NoError;
				}
			}
		}
	}

	setChangedTime();
		
	return(ret);
}	

/*****************************************************************************/
/* startDaemon */
/*****************************************************************************/

static oslDaemonManagerError startDaemon(oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	oslProcessError procError;
	oslProcess process = NULL;
	activeDaemonEntry* pEntry = NULL;
	sal_uInt32 retry = DAEMON_COMAND_MAX_RETRY;

	/* try to get connection */
	pEntry = getDaemonEntry(pInfo->Identifier);

	if (pEntry && !checkDaemonConnection(pEntry))
	{
		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}

	if (!pEntry)
	{
		rtl_uString* path = NULL;
		rtl_uString* delimiter = NULL;
		sal_Int32    Index;
		sal_Char*     c = PATH_DELEMITTER;

		rtl_uString_newFromAscii(&delimiter, c);
		Index = rtl_ustr_lastIndexOfStr(pInfo->Path->buffer, delimiter->buffer);
		if (Index != -1)
        {
			rtl_uString_newFromStr_WithLength(&path, pInfo->Path->buffer, Index+1);
        }
		else
        {    
			rtl_uString_newFromStr(&path, pInfo->Path->buffer);
        }
        

		rtl_uString_release(delimiter);

		/* try to start process once */
		procError = osl_executeProcess(pInfo->Path,
	                                   NULL,
									   0,
	                                   osl_Process_DETACHED | osl_Process_HIDDEN,
	                                   NULL,
	                                   path,
	                                   NULL,
									   0,
	                                   NULL,
	                                   &process);


		rtl_uString_release(path);
		if (procError == osl_Process_E_None)
		{
			while ((!pEntry) && (--retry > 0))
			{
				osl_waitThread(&pollingTime);
				pEntry = getDaemonEntry(pInfo->Identifier);

				if (pEntry && !checkDaemonConnection(pEntry))
				{
					releaseDaemonEntry(pEntry);
					pEntry = NULL;
				}
			}
		}
		else
		{
			ret = osl_Daemon_ManagerError_Unknown;
		}
	}

	if (pEntry)
	{
		ret = sendCommandByEntry(pEntry, 
								 osl_Daemon_CommandStart,
								 osl_Daemon_StatusStopped,
								 osl_Daemon_StatusRunning,
								 osl_Daemon_ManagerError_AllreadyRunning);
	}

	if (pEntry)
	{
		if (!checkDaemonConnection(pEntry))
		{
			if (pEntry->Connection) osl_disconnectDaemonMgrConn(pEntry->Connection);
			pEntry->Connection = NULL;
			pEntry->ProcessIdent = 0;
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
		
	return(ret);
}

/*****************************************************************************/
/* stopDaemon */
/*****************************************************************************/

static oslDaemonManagerError stopDaemon(oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	activeDaemonEntry* pEntry = getDaemonEntry(pInfo->Identifier);

	if (!pEntry || !checkDaemonConnection(pEntry))
	{
		ret = osl_Daemon_ManagerError_NotRunning;
	}
	else
	{
		ret = sendCommandByEntry(pEntry, 
								 osl_Daemon_CommandStop,
								 osl_Daemon_StatusRunning,
								 osl_Daemon_StatusStopped,
								 osl_Daemon_ManagerError_NotRunning);
	}

	if (pEntry)
	{
		if (!checkDaemonConnection(pEntry))
		{
			if (pEntry->Connection) osl_disconnectDaemonMgrConn(pEntry->Connection);
			pEntry->Connection = NULL;
			pEntry->ProcessIdent = 0;
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
		
	return(ret);
}

/*****************************************************************************/
/* pauseDaemon */
/*****************************************************************************/

static oslDaemonManagerError pauseDaemon(oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	activeDaemonEntry* pEntry = getDaemonEntry(pInfo->Identifier);

	if (!pEntry || !checkDaemonConnection(pEntry))
	{
		ret = osl_Daemon_ManagerError_NotRunning;
	}
	else
	{
		ret = sendCommandByEntry(pEntry, 
								 osl_Daemon_CommandPause,
								 osl_Daemon_StatusRunning,
								 osl_Daemon_StatusPaused,
								 osl_Daemon_ManagerError_NotRunning);
	}

	if (pEntry)
	{
		if (!checkDaemonConnection(pEntry))
		{
			if (pEntry->Connection) osl_disconnectDaemonMgrConn(pEntry->Connection);
			pEntry->Connection = NULL;
			pEntry->ProcessIdent = 0;
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
		
	return(ret);
}

/*****************************************************************************/
/* continueDaemon */
/*****************************************************************************/

static oslDaemonManagerError continueDaemon(oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	activeDaemonEntry* pEntry = getDaemonEntry(pInfo->Identifier);

	if (!pEntry || !checkDaemonConnection(pEntry))
	{
		ret = osl_Daemon_ManagerError_NotRunning;
	}
	else
	{
		ret = sendCommandByEntry(pEntry, 
								 osl_Daemon_CommandContinue,
								 osl_Daemon_StatusPaused,
								 osl_Daemon_StatusRunning,
								 osl_Daemon_ManagerError_NotPaused);
	}

	if (pEntry)
	{
		if (!checkDaemonConnection(pEntry))
		{
			if (pEntry->Connection) osl_disconnectDaemonMgrConn(pEntry->Connection);
			pEntry->Connection = NULL;
			pEntry->ProcessIdent = 0;
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
		
	return(ret);
}

/*****************************************************************************/
/* shutdownDaemon */
/*****************************************************************************/

static oslDaemonManagerError shutdownDaemon(oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	activeDaemonEntry* pEntry = getDaemonEntry(pInfo->Identifier);

	if (!pEntry || !checkDaemonConnection(pEntry))
	{
		ret = osl_Daemon_ManagerError_NotRunning;
	}
	else
	{
		ret = sendCommandByEntry(pEntry, 
								 osl_Daemon_CommandShutdown,
								 osl_Daemon_StatusRunning,
								 osl_Daemon_StatusStopped,
								 osl_Daemon_ManagerError_NotRunning);
	}

	if (pEntry)
	{
		if (!checkDaemonConnection(pEntry))
		{
			if (pEntry->Connection) osl_disconnectDaemonMgrConn(pEntry->Connection);
			pEntry->Connection = NULL;
			pEntry->ProcessIdent = 0;
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
		
	return(ret);
}

/*****************************************************************************/
/* command */
/*****************************************************************************/

static oslDaemonManagerError command(rtl_uString* Identifier, oslDaemonCommand Command)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	oslDaemonInfo info;

	memset(&info, 0, sizeof(info));
	info.Size = sizeof(info);

	ret = getDaemonInfo(Identifier, &info);
	
	if (ret == osl_Daemon_ManagerError_NoError)
	{
		switch (Command)
		{
			case osl_Daemon_CommandStart:
				ret = startDaemon(&info);
				break;
			case osl_Daemon_CommandStop:
				ret = stopDaemon(&info);
				break;
			case osl_Daemon_CommandPause:
				ret = pauseDaemon(&info);
				break;
			case osl_Daemon_CommandContinue:
				ret = continueDaemon(&info);
				break;
			case osl_Daemon_CommandShutdown:
				ret = shutdownDaemon(&info);
				break;
			default:
				ret = osl_Daemon_ManagerError_Unknown;
				break;
		}		
	}

	return(ret);
}	

/*****************************************************************************/
/* getStatusByEntry */
/*****************************************************************************/

static oslDaemonManagerError getStatusByEntry(activeDaemonEntry* pEntry, oslDaemonStatus* pStatus)
{
	oslDaemonManagerError error = osl_Daemon_ManagerError_NoError;
	oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(pEntry->Identifier, osl_DaemonMgrMsgSubject_StatusGet);

	error = osl_sendDaemonMgrMsg(pEntry->Connection, pRequest);

	osl_destroyDaemonMgrMsg(pRequest);
	pRequest = NULL;

	if (error == osl_Daemon_ManagerError_NoError)
		error = osl_recvDaemonMgrMsg(pEntry->Connection, &pRequest);

	if (error == osl_Daemon_ManagerError_NoError)
	{
		error = pRequest->Error;

		if (error == osl_Daemon_ManagerError_NoError)
			*pStatus = pRequest->Body.Status;

		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;
	}


	return(error);
}

/*****************************************************************************/
/* getStatus */
/*****************************************************************************/

static oslDaemonManagerError getStatus(rtl_uString* Identifier, oslDaemonStatus* pStatus)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;
	activeDaemonEntry* pEntry = getDaemonEntry(Identifier);
	oslDaemonInfo info;

	if (pEntry != NULL)
	{
		if (!checkDaemonConnection(pEntry))
		{
			ret = getDaemonInfo(Identifier, &info);

			if (ret == osl_Daemon_ManagerError_NoError)
				*pStatus = osl_Daemon_StatusStopped;
		}
		else
		{
			ret = getStatusByEntry(pEntry, pStatus);
		}
			
		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}

	if (ret == osl_Daemon_ManagerError_Unknown)
	{
		ret = getDaemonInfo(Identifier, &info);

		if (ret == osl_Daemon_ManagerError_NoError)
			*pStatus = osl_Daemon_StatusStopped;
	}
	
	return(ret);
}	

/*****************************************************************************/
/* getPropertyCount */
/*****************************************************************************/

static void getPropertyCount(oslDaemonMgrMsg* pRequest)
{
	activeDaemonEntry* 		pEntry = getDaemonEntry(pRequest->Identifier);
	oslDaemonMgrMsg* 		pReply = NULL;

	pRequest->Error = osl_Daemon_ManagerError_NotRunning;

	if (pEntry != NULL)
	{
		if (pEntry->Connection)
		{
			if (osl_sendDaemonMgrMsg(pEntry->Connection, pRequest) == osl_Daemon_ManagerError_NoError)
			{
				if (osl_recvDaemonMgrMsg(pEntry->Connection, &pReply) == osl_Daemon_ManagerError_NoError)
				{
					pRequest->Error = pReply->Error;
					pRequest->Body.PropertyCount = pReply->Body.PropertyCount;
		
					osl_destroyDaemonMgrMsg(pReply);
					pReply = NULL;
				}
			}
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
}	

/*****************************************************************************/
/* getProperties */
/*****************************************************************************/

static void getProperties(oslDaemonMgrMsg* pRequest)
{
	activeDaemonEntry* 	pEntry = getDaemonEntry(pRequest->Identifier);
	oslDaemonMgrMsg* 	pReply = NULL;

	if (pEntry != NULL)
	{
		if (pEntry->Connection)
		{
			if (osl_sendDaemonMgrMsg(pEntry->Connection, pRequest) == osl_Daemon_ManagerError_NoError)
			{
				if (osl_recvDaemonMgrMsg(pEntry->Connection, &pReply) == osl_Daemon_ManagerError_NoError)
				{
					pRequest->Error = pReply->Error;
					pRequest->Body.Properties = pReply->Body.Properties;

					/* do not destroy property list and error message */
					pReply->Body.Properties.List = NULL;
					pReply->Body.Properties.Count = 0;
					pReply->Body.Properties.ErrorText = NULL;

					osl_destroyDaemonMgrMsg(pReply);
					pReply = NULL;
				}
			}
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
}	

/*****************************************************************************/
/* setProperties */
/*****************************************************************************/

static void setProperties(oslDaemonMgrMsg* pRequest)
{
	activeDaemonEntry* 	pEntry = getDaemonEntry(pRequest->Identifier);
	oslDaemonMgrMsg* 	pReply = NULL;

	pRequest->Body.Properties.ErrorCode = 0;
	rtl_uString_newFromAscii(&pRequest->Body.Properties.ErrorText, "daemon is not accessable");
	pRequest->Error = osl_Daemon_ManagerError_NotRunning;

	if (pEntry != NULL)
	{
		if (pEntry->Connection)
		{
			if (osl_sendDaemonMgrMsg(pEntry->Connection, pRequest) == osl_Daemon_ManagerError_NoError)
			{
				if (osl_recvDaemonMgrMsg(pEntry->Connection, &pReply) == osl_Daemon_ManagerError_NoError)
				{
					pRequest->Error = pReply->Error;
					pRequest->Body.Properties.ErrorCode = pReply->Body.Properties.ErrorCode;

					if (pReply->Body.Properties.ErrorText != NULL)
					{
						rtl_freeMemory(pRequest->Body.Properties.ErrorText);
						rtl_uString_assign(&pRequest->Body.Properties.ErrorText, pReply->Body.Properties.ErrorText);
					}

					osl_destroyDaemonMgrMsg(pReply);
					pReply = NULL;
				}
			}
		}

		releaseDaemonEntry(pEntry);
		pEntry = NULL;
	}
}	

/*****************************************************************************/
/* getProfileSections */
/*****************************************************************************/

static void getProfileSections(oslDaemonMgrMsg* pRequest)
{
	osl_acquireMutex(globalDaemonMutex);

	if (!pGlobalRegistry->hasDaemon(pRequest->Identifier))
	{
		pRequest->Error = osl_Daemon_ManagerError_NotInstalled;
	}
	else
	{
		sal_uInt32 size = pGlobalRegistry->getSections(pRequest->Identifier, NULL, 0);

		if (size > 0)
		{
			sal_Unicode* list = (sal_Unicode*) rtl_allocateMemory(size * sizeof(sal_Unicode));

			pGlobalRegistry->getSections(pRequest->Identifier, list, size);

			pRequest->Body.Profile.Section = list;
		}
	}

	osl_releaseMutex(globalDaemonMutex);
}

/*****************************************************************************/
/* getProfileEntries */
/*****************************************************************************/

static void getProfileEntries(oslDaemonMgrMsg* pRequest)
{
	osl_acquireMutex(globalDaemonMutex);

	if (!pGlobalRegistry->hasDaemon(pRequest->Identifier))
	{
		pRequest->Error = osl_Daemon_ManagerError_NotInstalled;
	}
	else
	{
		sal_uInt32  size;
		rtl_uString *strSection = NULL;

		rtl_uString_newFromStr(&strSection, pRequest->Body.Profile.Section);
		size = pGlobalRegistry->getEntries(	pRequest->Identifier, strSection, NULL,  0);

		if (size > 0)
		{
			sal_Unicode* list = (sal_Unicode*) rtl_allocateMemory(size * sizeof(sal_Unicode));

			pGlobalRegistry->getEntries(pRequest->Identifier, 
												  strSection, 
												  list, 
												  size);

			pRequest->Body.Profile.Entry = list;
		}

		rtl_uString_release(strSection);
	}

	osl_releaseMutex(globalDaemonMutex);
}

/*****************************************************************************/
/* readProfileEntry */
/*****************************************************************************/

static void readProfileEntry(oslDaemonMgrMsg* pRequest)
{
	osl_acquireMutex(globalDaemonMutex);

	if (!pGlobalRegistry->hasDaemon(pRequest->Identifier))
	{
		pRequest->Error = osl_Daemon_ManagerError_NotInstalled;
	}
	else
	{
		sal_Unicode *buffer = NULL;
		rtl_uString *strSection = NULL;
		rtl_uString *strEntry = NULL;
		
		buffer = (sal_Unicode*) rtl_allocateMemory(DAEMON_MAX_ENTRY_SIZE);
		rtl_uString_newFromStr(&strSection, pRequest->Body.Profile.Section);
		rtl_uString_newFromStr(&strEntry, pRequest->Body.Profile.Entry);

		pGlobalRegistry->readDaemonEntry(pRequest->Identifier, 
										 strSection, 
										 strEntry, 
										 buffer,
										 sizeof(buffer),
										 pRequest->Body.Profile.Default); 

		rtl_uString_release(strSection);
		rtl_uString_release(strEntry);
	 	pRequest->Body.Profile.Value = buffer;
	}

	osl_releaseMutex(globalDaemonMutex);
}

/*****************************************************************************/
/* writeProfileEntry */
/*****************************************************************************/

static void writeProfileEntry(oslDaemonMgrMsg* pRequest)
{
	osl_acquireMutex(globalDaemonMutex);

	if (!pGlobalRegistry->hasDaemon(pRequest->Identifier))
	{
		pRequest->Error = osl_Daemon_ManagerError_NotInstalled;
	}
	else
	{
		rtl_uString *strSection = NULL;
		rtl_uString *strEntry = NULL;

		rtl_uString_newFromStr(&strSection, pRequest->Body.Profile.Section);
		rtl_uString_newFromStr(&strEntry, pRequest->Body.Profile.Entry);

		pGlobalRegistry->writeDaemonEntry(pRequest->Identifier, 
										  strSection, 
										  strEntry, 
										  pRequest->Body.Profile.Value); 
		rtl_uString_release(strSection);
		rtl_uString_release(strEntry);
	}

	osl_releaseMutex(globalDaemonMutex);
}

/*****************************************************************************/
/* writeProfileEntry */
/*****************************************************************************/

static void removeProfileEntry(oslDaemonMgrMsg* pRequest)
{
	osl_acquireMutex(globalDaemonMutex);

	if (!pGlobalRegistry->hasDaemon(pRequest->Identifier))
	{
		pRequest->Error = osl_Daemon_ManagerError_NotInstalled;
	}
	else
	{
		rtl_uString *strSection = NULL;
		rtl_uString *strEntry = NULL;

		rtl_uString_newFromStr(&strSection, pRequest->Body.Profile.Section);
		rtl_uString_newFromStr(&strEntry, pRequest->Body.Profile.Entry);

		pGlobalRegistry->removeDaemonEntry(pRequest->Identifier, 
										   strSection, 
										   strEntry);

		rtl_uString_release(strSection);
		rtl_uString_release(strEntry);
	}

	osl_releaseMutex(globalDaemonMutex);
}


/*****************************************************************************/
/* oslDaemonManager_Connection_WorkerFunction */
/*****************************************************************************/

static void oslDaemonManager_Connection_WorkerFunction(oslDaemonManagerThread* pThread)
{
	oslDaemonMgrMsg* pMsg = NULL;
	oslDaemonManagerError error = osl_Daemon_ManagerError_NoError;
	sal_Bool bExit = sal_False;
	sal_Bool bClose = sal_True;
	sal_Bool bShutdown = sal_False;


	while (
			!bExit &&
			(osl_recvDaemonMgrMsg(pThread->hConnection, &pMsg) == osl_Daemon_ManagerError_NoError)
		  )
	{
		if (pMsg->Type == osl_DaemonMgrMsgType_Request)	
		{
			if (pMsg->Subject == osl_DaemonMgrMsgSubject_CloseConnection)
			{
			}
			else
			{
				switch (pMsg->Subject)
				{
					case osl_DaemonMgrMsgSubject_Shutdown:
						bExit = sal_True;
						bShutdown = sal_True;
						shutdownAllConnections();
						pMsg->Error = osl_Daemon_ManagerError_NoError;
						break;
					case osl_DaemonMgrMsgSubject_QueryDaemonInfo:
						pMsg->Error = getDaemonInfo(pMsg->Identifier, pMsg->Body.pInfo);
						break;
					case osl_DaemonMgrMsgSubject_QueryDaemonCount:
						pMsg->Error = queryDaemonCount(&(pMsg->Body.DaemonCount));
						break;
					case osl_DaemonMgrMsgSubject_QueryDaemonIdentifier:
						pMsg->Error = queryDaemonIdentifier(pMsg->Body.DaemonIndex, 
															&pMsg->Identifier);
						break;
					case osl_DaemonMgrMsgSubject_Connect:
						pMsg->Error = connectFromDaemon(pMsg->Identifier, 
														pThread->hConnection, 
														pMsg->Body.ProcessIdentifier);
						bClose = sal_False;
						bExit = sal_True;
						break;
					case osl_DaemonMgrMsgSubject_Install:
						pMsg->Error = install(pMsg->Body.pInfo);
						break;
					case osl_DaemonMgrMsgSubject_Uninstall:
						pMsg->Error = uninstall(pMsg->Identifier);
						break;
					case osl_DaemonMgrMsgSubject_Command:
						pMsg->Error = command(pMsg->Identifier, pMsg->Body.Command);
						break;
					case osl_DaemonMgrMsgSubject_StatusGet:
						pMsg->Error = getStatus(pMsg->Identifier, &(pMsg->Body.Status));
						break;
					case osl_DaemonMgrMsgSubject_PropertyCountGet:
						getPropertyCount(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_PropertiesGet:
						getProperties(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_PropertiesSet:
						setProperties(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_ProfileGetSections:
						getProfileSections(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_ProfileGetEntries:
						getProfileEntries(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_ProfileReadEntry:
						readProfileEntry(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_ProfileWriteEntry:
						writeProfileEntry(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_ProfileRemoveEntry:
						removeProfileEntry(pMsg);
						break;
					case osl_DaemonMgrMsgSubject_QueryChanges:
						queryChanges(pMsg);
						break;
					default:
						pMsg->Error = osl_Daemon_ManagerError_Unknown;
						break;
				}

				pMsg->Type = osl_DaemonMgrMsgType_Reply;

				error = osl_sendDaemonMgrMsg(pThread->hConnection, pMsg);
			}

			osl_destroyDaemonMgrMsg(pMsg);
			pMsg = NULL;
		}
	}
	

	if (bClose)
		osl_disconnectDaemonMgrConn(pThread->hConnection);

	/* destroy thread handle and this object */
	osl_freeThreadHandle(pThread->hThread);
	rtl_freeMemory(pThread);

	if (bShutdown)
	{
		osl_deinitDaemonManager();
		exit(0);
	}
}

/*****************************************************************************/
/* oslDaemonManager_Startup_WorkerFunction */
/*****************************************************************************/

static void oslDaemonManager_Startup_WorkerFunction(oslDaemonManagerThread* pThread)
{
	sal_uInt32 i = 0;
	sal_uInt32 daemonCount = 0;
	rtl_uString* Identifier = 0;
	oslDaemonInfo* pInfo = NULL;

	osl_acquireMutex(globalDaemonMutex);

	if (
		(queryDaemonCount(&daemonCount) == osl_Daemon_ManagerError_NoError)	&&
		(daemonCount > 0)
	   )
	{
		pInfo = (oslDaemonInfo*) rtl_allocateMemory(daemonCount * sizeof(oslDaemonInfo));
        memset(pInfo,0,sizeof(oslDaemonInfo));
        
		for (i = 0; i < daemonCount; i++)
		{
			if ((queryDaemonIdentifier(i, &Identifier) == osl_Daemon_ManagerError_NoError))
			{
				getDaemonInfo(Identifier, &(pInfo[i]));
			}
		}
	}

	osl_releaseMutex(globalDaemonMutex);

	if (daemonCount > 0)
	{
		osl_waitThread(&startupTime);

		for (i = 0; i < daemonCount; i++)
		{
			if (pInfo[i].Mode == osl_Daemon_Automatic)
			{
				startDaemon(&(pInfo[i]));
			}
		}

		rtl_freeMemory(pInfo);
		pInfo = NULL;
	}

	/* destroy thread handle and this object */
	osl_freeThreadHandle(pThread->hThread);
	rtl_freeMemory(pThread);
}	

/*****************************************************************************/
/* osl_initDaemonManager */
/*****************************************************************************/
oslDaemonManagerStartupError SAL_CALL osl_initDaemonManager(DaemonRegistryApi* pRegistry)
{
	oslDaemonManagerStartupError ret = osl_oslDaemonManagerStartupError_NoError;

	/* check if daemon manager is allready running */
	oslDaemonConnection conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_CloseConnection);
		osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		osl_disconnectDaemonMgrConn(conn);
		ret = osl_oslDaemonManagerStartupError_AllreadyStarted;
		conn = NULL;
	}
	else
	{
		pGlobalRegistry	= pRegistry;
		globalDaemonMutex = osl_createMutex();
		hSignalHandler = osl_addSignalHandler(DaemonManagerSignalHandler, NULL);
	}

	return(ret);
}

void SAL_CALL osl_deinitDaemonManager()
{
	pGlobalRegistry	= NULL;

	osl_destroyMutex(globalDaemonMutex);
	globalDaemonMutex = NULL;

	osl_removeSignalHandler(hSignalHandler);
}	

/*****************************************************************************/
/* osl_stopDaemonManager */
/*****************************************************************************/
sal_Bool SAL_CALL osl_shutdownDaemonManager()
{
/*	oslDaemonManagerStartupError error = osl_oslDaemonManagerStartupError_NoError;*/
	sal_Bool ret = sal_False;
	
	/* check if daemon manager is allready running */
	oslDaemonConnection conn = osl_openDaemonMgrConnA(OSL_DAEMON_MANAGER_PIPE, osl_DaemonConnection_OPEN);

	if (conn != NULL)
	{
		oslDaemonMgrMsg* pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_Shutdown);
		if (osl_sendDaemonMgrMsg(conn, pRequest) == osl_Daemon_ManagerError_NoError)
		{
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;

			if (osl_recvDaemonMgrMsg(conn, &pRequest) == osl_Daemon_ManagerError_NoError)
			{
				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}
		}

		osl_disconnectDaemonMgrConn(conn);
		ret = sal_True;
		conn = NULL;
	}

	return(ret);
	
}	

/*****************************************************************************/
/* osl_handleConnection */
/*****************************************************************************/
void SAL_CALL osl_handleConnection(oslDaemonConnection conn)
{
	oslDaemonManagerThread* pChildThread = (oslDaemonManagerThread*) rtl_allocateMemory(sizeof(oslDaemonManagerThread));

	pChildThread->hConnection = conn;
	pChildThread->hThread = 
		osl_createSuspendedThread(
				(oslWorkerFunction)oslDaemonManager_Connection_WorkerFunction, 
				(void*)pChildThread);

	osl_resumeThread(pChildThread->hThread);							             
}	

/*****************************************************************************/
/* osl_startupAutomaticDaemons */
/*****************************************************************************/
void SAL_CALL osl_startupAutomaticDaemons()
{
	oslDaemonManagerThread* pStartUpThread = NULL;

	pStartUpThread = (oslDaemonManagerThread*) rtl_allocateMemory(sizeof(oslDaemonManagerThread));
	pStartUpThread->hThread = 
		osl_createSuspendedThread(
				(oslWorkerFunction)oslDaemonManager_Startup_WorkerFunction, 
				(void*)pStartUpThread);
	pStartUpThread->hConnection = NULL;
	osl_resumeThread(pStartUpThread->hThread);							             
}	

