/*************************************************************************
 *
 *  $RCSfile: daemprot.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string.h>
#include <stdio.h>

#ifndef _OSL_DIAGNOSE_H_
	#include <osl/diagnose.h>
#endif
#ifndef _OSL_PROCESS_H_
	#include <osl/process.h>
#endif
#ifndef _OSL_SECURITY_H_
	#include <osl/security.h>
#endif
#ifndef _OSL_MUTEX_H_
	#include <osl/mutex.h>
#endif
#ifndef _OSL_PIPE_H_
	#include <osl/pipe.h>
#endif
#ifndef _OSL_DAEMPROT_H_
#   include "daemprot.h"
#endif
#include <rtl/alloc.h>
#include <rtl/memory.h>

#define DAEMON_MAX(a,b)            (((a) > (b)) ? (a) : (b))

#define DAEMON_MAGIC 			0xdaef37b1
#define DAEMON_VERSION 			1
#define HEAD_LEN				sizeof(sal_uInt32) + sizeof(sal_uInt32) + sizeof(sal_uInt32)
#define INIT_SIZE 				128

#define MAX_PIPE_NAME			512

typedef struct _oslConnection_Impl
{
	oslPipe 					pipe;
	oslDaemonConnectionOptions 	options;
	sal_Unicode					name[MAX_PIPE_NAME];
	struct _oslConnection_Impl* next;
} oslConnection_Impl;

typedef struct
{
	sal_uInt32 	size;
	sal_uInt32	pos;
	sal_uInt8*	pData;	
} Buffer;
	
static oslConnection_Impl*  connectionList = NULL;
static oslMutex 			connectionMutex = NULL;
static oslSecurity 			connectionSecurity = NULL;
static sal_Bool				bInitialized = sal_False;

static sal_Bool initModule()
{
	rtl_uString *strEnvVar = NULL;
	rtl_uString *strValue = NULL;

	rtl_uString_newFromAscii(&strEnvVar, "OSL_DAEMON_DEBUG_ENV");

	connectionList = NULL;
	connectionMutex = osl_createMutex();

	if (osl_getEnvironment(strEnvVar, &strValue) == osl_Process_E_None)
	{
		if (rtl_ustr_ascii_compare(strValue->buffer, "user") == 0)
		{
			connectionSecurity = osl_getCurrentSecurity();
		}
	}

	if (strEnvVar)
		rtl_uString_release(strEnvVar);
	if (strValue)
		rtl_uString_release(strValue);

	return(sal_True);
}	

/*****************************************************************************/
/* createNewBuffer */
/*****************************************************************************/

static Buffer* createNewBuffer()
{
	Buffer* buffer = (Buffer*) rtl_allocateMemory(sizeof(sal_uInt32) + sizeof(sal_uInt32) + sizeof(sal_uInt8*));

	buffer->size = INIT_SIZE;
	buffer->pos = 0;

	buffer->pData = (sal_uInt8*) rtl_allocateMemory(INIT_SIZE);

	return(buffer);
}	

/*****************************************************************************/
/* createBuffer */
/*****************************************************************************/

static Buffer* createBuffer(sal_uInt32 size)
{
	Buffer* buffer = (Buffer*) rtl_allocateMemory(sizeof(sal_uInt32) + sizeof(sal_uInt32) + sizeof(sal_uInt8*));

	buffer->size = size;
	buffer->pos = 0;

	buffer->pData = (sal_uInt8*) rtl_allocateMemory(size);

	return(buffer);
}	

/*****************************************************************************/
/* deleteBuffer */
/*****************************************************************************/

static void deleteBuffer(Buffer* pBuffer)
{
	rtl_freeMemory(pBuffer->pData);
	rtl_freeMemory(pBuffer);		
}	

/*****************************************************************************/
/* appendDWORD */
/*****************************************************************************/

static void appendDWORD(Buffer* pBuffer, sal_uInt32 data)
{
	if ((pBuffer->pos + sizeof(sal_uInt32)) > pBuffer->size)
	{
		pBuffer->size = DAEMON_MAX(pBuffer->size * 2, pBuffer->size + sizeof(sal_uInt32));
		pBuffer->pData = (sal_uInt8*) rtl_reallocateMemory(pBuffer->pData, pBuffer->size);
	}

	pBuffer->pData[pBuffer->pos++] = (sal_uInt8) ((data >> 24)  & 0xFF);
	pBuffer->pData[pBuffer->pos++] = (sal_uInt8) ((data >> 16)  & 0xFF);
	pBuffer->pData[pBuffer->pos++] = (sal_uInt8) ((data >> 8)   & 0xFF);
	pBuffer->pData[pBuffer->pos++] = (sal_uInt8) ((data >> 0)   & 0xFF);
}	

/*****************************************************************************/
/* appendSTRING */
/*****************************************************************************/

static void appendSTRING(Buffer* pBuffer, const sal_Unicode* data)
{
	sal_uInt32 size = 0;

	if (data != 0) size = rtl_ustr_getLength(data) * sizeof(sal_Unicode);

	appendDWORD(pBuffer, size);

	if ((pBuffer->pos + size) > pBuffer->size)
	{
		pBuffer->size  = DAEMON_MAX(pBuffer->size * 2, pBuffer->size + size);
		pBuffer->pData = (sal_uInt8*) rtl_reallocateMemory(pBuffer->pData, pBuffer->size);
	}

	if (size > 0)
		rtl_copyMemory(pBuffer->pData + pBuffer->pos, data, size);

	pBuffer->pos += size;
}
/*****************************************************************************/
/* appendUSTRING */
/*****************************************************************************/

static void appendUSTRING(Buffer* pBuffer, rtl_uString* data)
{
	sal_uInt32 size = 0;

    if ( data == 0 )
    {
        appendDWORD(pBuffer, 0);
        return;
    }

	if (data->length != 0) size = data->length * sizeof(sal_Unicode);

    appendDWORD(pBuffer, size);

	if ((pBuffer->pos + size) > pBuffer->size)
	{
		pBuffer->size  = DAEMON_MAX(pBuffer->size * 2, pBuffer->size + size);
		pBuffer->pData = (sal_uInt8*) rtl_reallocateMemory(pBuffer->pData, pBuffer->size);
	}

	if (size > 0)
		rtl_copyMemory(pBuffer->pData + pBuffer->pos, data->buffer, size);

	pBuffer->pos += size;
}

/*****************************************************************************/
/* appendSTRINGLIST */
/*****************************************************************************/

static void appendSTRINGLIST(Buffer* pBuffer, const sal_Unicode* data)
{
	sal_uInt32 charCount = 0;

	if (data != NULL)
	{
		do
		{
			charCount += (rtl_ustr_getLength(data + charCount) + 1) * sizeof(sal_Unicode);			
		}
		while(data[charCount] != 0);
	}

	appendDWORD(pBuffer, charCount);

	if ((pBuffer->pos + charCount) > pBuffer->size)
	{
		pBuffer->size  = DAEMON_MAX(pBuffer->size * 2, pBuffer->size + charCount);
		pBuffer->pData = (sal_uInt8*) rtl_reallocateMemory(pBuffer->pData, pBuffer->size);
	}

	if (charCount > 0)
		rtl_copyMemory(pBuffer->pData + pBuffer->pos, data, charCount);

	pBuffer->pos += charCount;
}	

/*****************************************************************************/
/* appendINFO */
/*****************************************************************************/

static void appendINFO(Buffer* pBuffer, oslDaemonInfo* pInfo)
{
	appendUSTRING(pBuffer, pInfo->Identifier);
	appendUSTRING(pBuffer, pInfo->DisplayName);
	appendUSTRING(pBuffer, pInfo->Manufactor);
	appendUSTRING(pBuffer, pInfo->Path);
	appendSTRINGLIST(pBuffer, pInfo->Dependencies);
	appendDWORD(pBuffer, pInfo->Version);
	appendDWORD(pBuffer, pInfo->Mode);
}	

/*****************************************************************************/
/* appendPROPERTY */
/*****************************************************************************/

static void appendPROPERTY(Buffer* pBuffer, oslDaemonProperty* pProp)
{
	sal_uInt32 i = 0;

	appendDWORD(pBuffer, pProp->Type);
	appendUSTRING(pBuffer, pProp->Name);

	switch (pProp->Type)
	{
		case osl_Daemon_PropertyType_Boolean:
			appendDWORD(pBuffer, pProp->Value.aBool);
			break;
		case osl_Daemon_PropertyType_uLong:
			appendDWORD(pBuffer, pProp->Value.aULong);
			break;
		case osl_Daemon_PropertyType_String:
			appendSTRING(pBuffer, pProp->Value.aString);
			break;
		case osl_Daemon_PropertyType_BooleanList:
			appendDWORD(pBuffer, pProp->Value.aList.Count);
			for (i = 0; i < pProp->Value.aList.Count; i++)
			{
				appendDWORD(pBuffer, pProp->Value.aList.Data.pBool[i]);
			}
			break;
		case osl_Daemon_PropertyType_uLongList:
			appendDWORD(pBuffer, pProp->Value.aList.Count);
			for (i = 0; i < pProp->Value.aList.Count; i++)
			{
				appendDWORD(pBuffer, pProp->Value.aList.Data.pULong[i]);
			}
			break;
		case osl_Daemon_PropertyType_StringList:
			appendDWORD(pBuffer, pProp->Value.aList.Count);
			for (i = 0; i < pProp->Value.aList.Count; i++)
			{
				appendSTRING(pBuffer, pProp->Value.aList.Data.pString[i]);
			}
			break;
		default:
			break;
	}
}	

/*****************************************************************************/
/* readDWORD */
/*****************************************************************************/

static sal_Bool readDWORD(Buffer* pBuffer, sal_uInt32* pData)
{
	sal_Bool ret	= sal_False;

	if ((pBuffer->pos + sizeof(sal_uInt32)) <= pBuffer->size)
	{
		ret = sal_True;

		*pData = ((sal_uInt32)pBuffer->pData[pBuffer->pos + 0] << 24) | 
				 ((sal_uInt32)pBuffer->pData[pBuffer->pos + 1] << 16) | 
				 ((sal_uInt32)pBuffer->pData[pBuffer->pos + 2] << 8)  | 
				 ((sal_uInt32)pBuffer->pData[pBuffer->pos + 3] << 0);

		pBuffer->pos += 4;
	}

	return(ret);
}	

/*****************************************************************************/
/* readSTRING */
/*****************************************************************************/

static sal_Bool readSTRING(Buffer* pBuffer, sal_Unicode* buffer, sal_uInt32 bufferSize)
{
	sal_Bool ret	= sal_False;
	sal_uInt32 len = 0;

	if (readDWORD(pBuffer, &len) && (len < bufferSize))
	{
		if ((pBuffer->pos + len) <= pBuffer->size)
		{
			ret	= sal_True;
			rtl_copyMemory(buffer, pBuffer->pData + pBuffer->pos, len);
			buffer[len] = '\0';
			pBuffer->pos += len;
		}
	}

	return(ret);
}	
/*****************************************************************************/
/* readSTRING */
/*****************************************************************************/

static sal_Bool readUSTRING(Buffer* pBuffer, rtl_uString** pString)
{
	sal_Bool ret	= sal_False;
	sal_uInt32 size = 0;

	if (readDWORD(pBuffer, &size))
	{
		sal_uInt32 len = size / sizeof(sal_Unicode);
		rtl_uString_new_WithLength(pString, len);

		if ((pBuffer->pos + len) <= pBuffer->size)
		{
			ret	= sal_True;
			rtl_copyMemory((*pString)->buffer, pBuffer->pData + pBuffer->pos, len);
			(*pString)->length = len;
			pBuffer->pos += size;
		}
	}

	return(ret);
}	

/*****************************************************************************/
/* readSTRINGLIST */
/*****************************************************************************/

static sal_Bool readSTRINGLIST(Buffer* pBuffer, sal_Unicode* buffer, sal_uInt32 bufferSize)
{
	sal_Bool ret	= sal_False;
	sal_uInt32 charCount = 0;

	if (readDWORD(pBuffer, &charCount) && ((charCount + 1) < bufferSize))
	{
		if ((pBuffer->pos + charCount) <= pBuffer->size)
		{
			ret	= sal_True;
			rtl_copyMemory(buffer, pBuffer->pData + pBuffer->pos, charCount);
			buffer[charCount] = 0;
			pBuffer->pos += charCount;
		}
	}

	return(ret);
}	

/*****************************************************************************/
/* read_and_alloc_STRING */
/*****************************************************************************/

static sal_Bool read_and_alloc_STRING(Buffer* pBuffer, sal_Unicode** pNewString)
{
	sal_Bool ret	= sal_False;
	sal_uInt32 len = 0;

	if (readDWORD(pBuffer, &len))
	{
		if ((pBuffer->pos + len) <= pBuffer->size)
		{
			ret	= sal_True;

			*pNewString = (sal_Unicode *) rtl_allocateMemory((len + 1) * sizeof(sal_Unicode));
			rtl_copyMemory(*pNewString, pBuffer->pData + pBuffer->pos, len);
			(*pNewString)[len] = 0;
			pBuffer->pos += len;
		}
	}

	return(ret);
}	

/*****************************************************************************/
/* read_and_alloc_STRINGLIST */
/*****************************************************************************/

static sal_Bool read_and_alloc_STRINGLIST(Buffer* pBuffer, sal_Unicode** pNewStringList)
{
	sal_Bool ret	= sal_False;
	sal_uInt32 charCount = 0;

	if (readDWORD(pBuffer, &charCount))
	{
		if ((pBuffer->pos + charCount) <= pBuffer->size)
		{
			ret	= sal_True;

			if (charCount == 0)
			{
				*pNewStringList = (sal_Unicode *) rtl_allocateMemory(2 * sizeof(sal_Unicode));
				(*pNewStringList)[0] = 0;
				(*pNewStringList)[1] = 0;
			}
			else
			{
				*pNewStringList = (sal_Unicode *) rtl_allocateMemory((charCount + 1) * sizeof(sal_Unicode));
				rtl_copyMemory(*pNewStringList, pBuffer->pData + pBuffer->pos, charCount);
				(*pNewStringList)[charCount] = 0;
				pBuffer->pos += charCount;
			}
		}
	}

	return(ret);
}	

/*****************************************************************************/
/* readINFO */
/*****************************************************************************/

static sal_Bool readINFO(Buffer* pBuffer, oslDaemonInfo* pInfo)
{
	sal_Bool ret	= sal_True;

	if (ret) ret = readUSTRING(pBuffer, &pInfo->Identifier);
	if (ret) ret = readUSTRING(pBuffer, &pInfo->DisplayName);
	if (ret) ret = readUSTRING(pBuffer, &pInfo->Manufactor);
	if (ret) ret = readUSTRING(pBuffer, &pInfo->Path);
	if (ret) ret = readSTRINGLIST(pBuffer, pInfo->Dependencies, OSL_DAEMON_DEPENDENCIES_LEN);
	if (ret) ret = readDWORD(pBuffer, &(pInfo->Version));
	if (ret) ret = readDWORD(pBuffer, (sal_uInt32*) &pInfo->Mode);

	return(ret);
	
}	

/*****************************************************************************/
/* readPROPERTY */
/*****************************************************************************/

static sal_Bool readPROPERTY(Buffer* pBuffer, oslDaemonProperty* pProp)
{
	sal_Bool ret	= sal_True;
	sal_uInt32 i = 0;
	sal_uInt32 listSize = 0;
	rtl_uString* strPropName = NULL;
	oslDaemonPropertyType propType = osl_Daemon_PropertyType_Invalid;

	if (ret) ret = readDWORD(pBuffer, (sal_uInt32*) &propType);
	if (ret) ret = readUSTRING(pBuffer, &strPropName);

	if (ret)
	{
		switch (propType)
		{
			case osl_Daemon_PropertyType_Boolean:
				{
					sal_uInt32 aBool = 0;
					osl_initDaemonProperty(pProp, propType, strPropName, 0);
					ret = readDWORD(pBuffer, &aBool);
					pProp->Value.aBool = aBool ? sal_True : sal_False;
				}
				break;
			case osl_Daemon_PropertyType_uLong:
				osl_initDaemonProperty(pProp, propType, strPropName, 0);
				ret = readDWORD(pBuffer, &(pProp->Value.aULong));
				break;
			case osl_Daemon_PropertyType_String:
				osl_initDaemonProperty(pProp, propType, strPropName, 0);
				ret = read_and_alloc_STRING(pBuffer, &(pProp->Value.aString));
				break;
			case osl_Daemon_PropertyType_BooleanList:
				ret = readDWORD(pBuffer, &listSize);
				if (ret)
				{
					osl_initDaemonProperty(pProp, propType, strPropName, listSize);
					for (i = 0; ret && (i < listSize); i++)
					{
						sal_uInt32 aBool = 0;
						ret = readDWORD(pBuffer, &aBool);
						pProp->Value.aList.Data.pBool[i] = aBool ? sal_True : sal_False;
					}
				}
				break;
			case osl_Daemon_PropertyType_uLongList:
				ret = readDWORD(pBuffer, &listSize);
				if (ret)
				{
					osl_initDaemonProperty(pProp, propType, strPropName, listSize);
					for (i = 0; ret && (i < listSize); i++)
					{
						ret = readDWORD(pBuffer, &(pProp->Value.aList.Data.pULong[i]));
					}
				}
				break;
			case osl_Daemon_PropertyType_StringList:
				ret = readDWORD(pBuffer, &listSize);
				if (ret)
				{
					osl_initDaemonProperty(pProp, propType, strPropName, listSize);
					for (i = 0; ret && (i < listSize); i++)
					{
						ret = read_and_alloc_STRING(pBuffer, &(pProp->Value.aList.Data.pString[i]));
					}
				}
				break;
			default:
				break;
		}

		rtl_uString_release(strPropName);
	}

	return(ret);
}	

/*****************************************************************************/
/* openConnection */
/*****************************************************************************/

static oslConnection_Impl* openConnection(rtl_uString * strName, oslDaemonConnectionOptions options)
{
	oslPipe pipe 					= NULL;
	oslConnection_Impl* pConn 		= NULL;
	oslConnection_Impl** ppPrevLink = &connectionList;

	if (!bInitialized) bInitialized = initModule();

	osl_acquireMutex(connectionMutex);

	for (pConn = connectionList; 
		 (pConn != NULL) &&
		 (
		   	(pConn->options != options) ||
		   	(rtl_ustr_compare(pConn->name, strName->buffer) != 0)
		 ); 
		 pConn = pConn->next)
	{
		ppPrevLink = &(pConn->next);
		/*pConn->next = NULL;*/
	}

	if (pConn != NULL)
	{
		*ppPrevLink = pConn->next;
		pConn->next = NULL;
	}

	osl_releaseMutex(connectionMutex);

	if (pConn == NULL)
	{
		pipe = osl_createPipe(strName, (oslPipeOptions)options, connectionSecurity);

		if (pipe != NULL)
		{
			pConn = (oslConnection_Impl*) rtl_allocateMemory(sizeof(oslConnection_Impl));

			pConn->pipe = pipe;
			pConn->options = options;
			rtl_copyMemory(pConn->name, strName, strName->length);
			pConn->name[strName->length] = '\0';
			pConn->next = NULL;
		}
	}

	return(pConn);
}	

/*****************************************************************************/
/* osl_openDaemonMgrConn */
/*****************************************************************************/

oslDaemonConnection SAL_CALL osl_openDaemonMgrConn(rtl_uString* name, oslDaemonConnectionOptions options)
{
	return openConnection(name, options);
}

oslDaemonConnection SAL_CALL osl_openDaemonMgrConnA(sal_Char* name, oslDaemonConnectionOptions options)
{
	oslDaemonConnection con;
	rtl_uString* strPipe = NULL;

	rtl_uString_newFromAscii(&strPipe, name);
	con = openConnection(strPipe, options);
	rtl_uString_release(strPipe);

	return(con);
}

/*****************************************************************************/
/* osl_acceptOnDaemonMgrConn */
/*****************************************************************************/
oslDaemonConnection SAL_CALL osl_acceptOnDaemonMgrConn(oslDaemonConnection conn)
{
	oslConnection_Impl* pConn = (oslConnection_Impl*) conn;
	oslConnection_Impl* pNewConn = NULL;

	if (pConn)
	{
		oslPipe newPipe = osl_acceptPipe(pConn->pipe);

		if (newPipe != NULL)
		{
			pNewConn = (oslConnection_Impl*) rtl_allocateMemory(sizeof(oslConnection_Impl));

			pNewConn->pipe = newPipe;
			pNewConn->options = osl_DaemonConnection_ACCEPT;
			rtl_copyMemory(pNewConn->name, pConn->name, (rtl_ustr_getLength(pConn->name) + 1) * sizeof(sal_Unicode));
			pNewConn->next = NULL;
		}
	}

	return(pNewConn);
}	

/*****************************************************************************/
/* osl_disconnectDaemonMgrConn */
/*****************************************************************************/
void SAL_CALL osl_disconnectDaemonMgrConn(oslDaemonConnection conn)
{
	oslConnection_Impl* pConn = (oslConnection_Impl*) conn;

	if (pConn)
	{
		if (
			(pConn->options & osl_DaemonConnection_ACCEPT) ||
			(pConn->pipe == NULL)
		   )
		{
			if (pConn->pipe != NULL) osl_destroyPipe(pConn->pipe);
			rtl_freeMemory(pConn);
		}
		else
		{
			osl_acquireMutex(connectionMutex);

			pConn->next = connectionList;
			connectionList = pConn;

			osl_releaseMutex(connectionMutex);
		}
	}
}	

/*****************************************************************************/
/* osl_sendDaemonMgrMsg */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_sendDaemonMgrMsg(oslDaemonConnection conn, oslDaemonMgrMsg* pMsg)
{
	sal_uInt32 i = 0;
	oslConnection_Impl* pConn = (oslConnection_Impl*) conn;

	if (pConn)
	{
		oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
		Buffer* pBufferMsg = createNewBuffer();
		Buffer* pBufferHeader = createBuffer(HEAD_LEN);

        appendUSTRING(pBufferMsg, pMsg->Identifier);
		appendDWORD(pBufferMsg, pMsg->Type);
		appendDWORD(pBufferMsg, pMsg->Subject);
		appendDWORD(pBufferMsg, pMsg->Error);

		switch (pMsg->Subject)
		{
			case osl_DaemonMgrMsgSubject_CheckConnection:
				break;
			case osl_DaemonMgrMsgSubject_QueryDaemonInfo:
				if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
					appendINFO(pBufferMsg, pMsg->Body.pInfo);
				break;
			case osl_DaemonMgrMsgSubject_QueryDaemonCount:
				appendDWORD(pBufferMsg, pMsg->Body.DaemonCount);
				break;
			case osl_DaemonMgrMsgSubject_QueryDaemonIdentifier:
				appendDWORD(pBufferMsg, pMsg->Body.DaemonIndex);
				break;
			case osl_DaemonMgrMsgSubject_Connect:
				appendDWORD(pBufferMsg, pMsg->Body.ProcessIdentifier);
				break;
			case osl_DaemonMgrMsgSubject_Install:
				if (pMsg->Type == osl_DaemonMgrMsgType_Request)
					appendINFO(pBufferMsg, pMsg->Body.pInfo);
				break;
			case osl_DaemonMgrMsgSubject_Uninstall:
				break;
			case osl_DaemonMgrMsgSubject_Command:
				appendDWORD(pBufferMsg, pMsg->Body.Command);
				break;
			case osl_DaemonMgrMsgSubject_StatusGet:
				appendDWORD(pBufferMsg, pMsg->Body.Status);
				break;
			case osl_DaemonMgrMsgSubject_PropertyCountGet:
				if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
					appendDWORD(pBufferMsg, pMsg->Body.PropertyCount);
				break;
			case osl_DaemonMgrMsgSubject_PropertiesGet:
				appendDWORD(pBufferMsg, pMsg->Body.Properties.Count);
				if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
				{
					for (i = 0; i < pMsg->Body.Properties.Count; i++)
					{
						appendPROPERTY(pBufferMsg, &(pMsg->Body.Properties.List[i]));
					}
				}
				break;
			case osl_DaemonMgrMsgSubject_PropertiesSet:
				if (pMsg->Type == osl_DaemonMgrMsgType_Request)
				{
					appendDWORD(pBufferMsg, pMsg->Body.Properties.Count);
					for (i = 0; i < pMsg->Body.Properties.Count; i++)
					{
						appendPROPERTY(pBufferMsg, &(pMsg->Body.Properties.List[i]));
					}
				}
				else
				{
					appendDWORD(pBufferMsg, pMsg->Body.Properties.ErrorCode);
					appendUSTRING(pBufferMsg, pMsg->Body.Properties.ErrorText);
				}
				break;
			case osl_DaemonMgrMsgSubject_ProfileGetSections:
				if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
				{
					appendSTRINGLIST(pBufferMsg, pMsg->Body.Profile.Section);
				}
				break;
			case osl_DaemonMgrMsgSubject_ProfileGetEntries:
				if (pMsg->Type == osl_DaemonMgrMsgType_Request)
				{
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Section);
				}
				else
				{
					appendSTRINGLIST(pBufferMsg, pMsg->Body.Profile.Entry);
				}
				break;
			case osl_DaemonMgrMsgSubject_ProfileReadEntry:
				if (pMsg->Type == osl_DaemonMgrMsgType_Request)
				{
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Section);
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Entry);
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Default);
				}
				else
				{
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Value);
				}
				break;
			case osl_DaemonMgrMsgSubject_ProfileWriteEntry:
				if (pMsg->Type == osl_DaemonMgrMsgType_Request)
				{
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Section);
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Entry);
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Value);
				}
				break;
			case osl_DaemonMgrMsgSubject_ProfileRemoveEntry:
				if (pMsg->Type == osl_DaemonMgrMsgType_Request)
				{
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Section);
					appendSTRING(pBufferMsg, pMsg->Body.Profile.Entry);
				}
				break;
			case osl_DaemonMgrMsgSubject_QueryChanges:
				appendDWORD(pBufferMsg, pMsg->Body.Changes.since.Seconds);
				appendDWORD(pBufferMsg, pMsg->Body.Changes.since.Nanosec);
				if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
				{
					appendDWORD(pBufferMsg, pMsg->Body.Changes.bChanged);
				}

				break;
			default:
				break;
		}		


		appendDWORD(pBufferHeader, DAEMON_MAGIC);
		appendDWORD(pBufferHeader, DAEMON_VERSION);
		appendDWORD(pBufferHeader, pBufferMsg->pos);

		if (
			(pConn->pipe == NULL) ||
			(osl_sendPipe(pConn->pipe, pBufferHeader->pData, pBufferHeader->pos) != pBufferHeader->pos)
		   )
		{
			ret = osl_Daemon_ManagerError_NoManager;

			if (pConn->options == osl_DaemonConnection_OPEN)
			{
				/* try to reconnect */
				oslConnection_Impl* pReConn = NULL;
				rtl_uString *strPipe = NULL;

				rtl_uString_newFromStr(&strPipe, pConn->name);

				if (pConn->pipe != NULL)
				{
					osl_destroyPipe(pConn->pipe);			
					pConn->pipe = NULL;
				}

				pReConn = openConnection(strPipe, osl_DaemonConnection_OPEN);
				rtl_uString_release(strPipe);

				if (pReConn)
				{
					pConn->pipe = pReConn->pipe;
					pReConn->pipe = NULL;
					rtl_freeMemory(pReConn);

					/* try to re-send if connect succeed */
					if (osl_sendPipe(pConn->pipe, pBufferHeader->pData, pBufferHeader->pos) == pBufferHeader->pos)
					{
						ret = osl_Daemon_ManagerError_NoError;
					}
				}
			}
		}

		if (
			(ret == osl_Daemon_ManagerError_NoError) &&
			(
			 (pConn->pipe == NULL) ||
			 (osl_sendPipe(pConn->pipe, pBufferMsg->pData, pBufferMsg->pos) != pBufferMsg->pos)
			)
		   )
		{
			if (pConn->pipe != NULL)
			{
				osl_destroyPipe(pConn->pipe);			
				pConn->pipe = NULL;
			}

			ret = osl_Daemon_ManagerError_NoManager;
		}
					
		deleteBuffer(pBufferMsg);
		pBufferMsg = NULL;
		deleteBuffer(pBufferHeader);
		pBufferHeader = NULL;

		return(ret);
	}
	else
		return(osl_Daemon_ManagerError_NoManager);
		
}	

/*****************************************************************************/
/* osl_recvDaemonMgrMsg */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_recvDaemonMgrMsg(oslDaemonConnection conn, oslDaemonMgrMsg** ppMsg)
{
	oslConnection_Impl* pConn = (oslConnection_Impl*) conn;

	if (pConn)
	{
		oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
		Buffer* pBufferMsg = NULL;
		Buffer* pBufferHeader = createBuffer(HEAD_LEN);
		sal_uInt32 	msgLen = 0;
		sal_uInt32 	magic = 0;
		sal_uInt32 	version = 0;
		oslDaemonMgrMsgSubject 	subject = osl_DaemonMgrMsgSubject_Unknown;
		oslDaemonMgrMsgType 	type = osl_DaemonMgrMsgType_Unknown;
		oslDaemonManagerError 	error = osl_Daemon_ManagerError_NoError;
		rtl_uString* strIdentifier = NULL;
		oslDaemonMgrMsg* pMsg = NULL;
		sal_uInt32 i = 0;

		if (
			(pConn->pipe == NULL) ||
			(osl_receivePipe(pConn->pipe, pBufferHeader->pData, HEAD_LEN) != HEAD_LEN)
		   )
		{
			if (pConn->pipe != NULL)
			{
				osl_destroyPipe(pConn->pipe);			
				pConn->pipe = NULL;
			}

			ret = osl_Daemon_ManagerError_NoManager;
		}
		else
		{
			if (
				readDWORD(pBufferHeader, &magic) &&
				readDWORD(pBufferHeader, &version) && 
				readDWORD(pBufferHeader, &msgLen) &&
				(magic == DAEMON_MAGIC) &&
				(version == DAEMON_VERSION)
			   )
			{
				pBufferMsg = createBuffer(msgLen);
				if (
					(pConn->pipe == NULL) ||
					(osl_receivePipe(pConn->pipe, pBufferMsg->pData, msgLen) != msgLen)
				   )
				{
					if (pConn->pipe != NULL)
					{
						osl_destroyPipe(pConn->pipe);			
						pConn->pipe = NULL;
					}

					ret = osl_Daemon_ManagerError_NoManager;
				}
				else
				{
					if (!readUSTRING(pBufferMsg, &strIdentifier))
						ret = osl_Daemon_ManagerError_Unknown;
					else if (!readDWORD(pBufferMsg, (sal_uInt32*) &(type)))
						ret = osl_Daemon_ManagerError_Unknown;
					else if (!readDWORD(pBufferMsg, (sal_uInt32*) &(subject)))
						ret = osl_Daemon_ManagerError_Unknown;
					else if (!readDWORD(pBufferMsg, (sal_uInt32*) &(error)))
						ret = osl_Daemon_ManagerError_Unknown;
					else
					{
						pMsg = osl_createDaemonMgrMsg(strIdentifier, subject);
						pMsg->Type = type;
						pMsg->Error = error;

						switch (pMsg->Subject)
						{
							case osl_DaemonMgrMsgSubject_CheckConnection:
								break;
							case osl_DaemonMgrMsgSubject_QueryDaemonInfo:
								{
									if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
									{
										if (!readINFO(pBufferMsg, pMsg->Body.pInfo))
											ret = osl_Daemon_ManagerError_Unknown;
									}
								}
								break;
							case osl_DaemonMgrMsgSubject_QueryDaemonCount:
								if (!readDWORD(pBufferMsg, &(pMsg->Body.DaemonCount)))
									ret = osl_Daemon_ManagerError_Unknown;
								break;
							case osl_DaemonMgrMsgSubject_QueryDaemonIdentifier:
								if (!readDWORD(pBufferMsg, &(pMsg->Body.DaemonIndex)))
									ret = osl_Daemon_ManagerError_Unknown;
								break;
							case osl_DaemonMgrMsgSubject_Connect:
								if (!readDWORD(pBufferMsg, &(pMsg->Body.ProcessIdentifier)))
									ret = osl_Daemon_ManagerError_Unknown;
								break;
							case osl_DaemonMgrMsgSubject_Install:
								{
									if (pMsg->Type == osl_DaemonMgrMsgType_Request)
									{
										if (!readINFO(pBufferMsg, pMsg->Body.pInfo))
											ret = osl_Daemon_ManagerError_Unknown;
									}
								}
								break;
							case osl_DaemonMgrMsgSubject_Uninstall:
								break;
							case osl_DaemonMgrMsgSubject_Command:
								if (!readDWORD(pBufferMsg, (sal_uInt32*) &(pMsg->Body.Command)))
									ret = osl_Daemon_ManagerError_Unknown;
								break;
							case osl_DaemonMgrMsgSubject_StatusGet:
								if (!readDWORD(pBufferMsg, (sal_uInt32*) &(pMsg->Body.Status)))
									ret = osl_Daemon_ManagerError_Unknown;
								break;
							case osl_DaemonMgrMsgSubject_PropertyCountGet:
								if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
									readDWORD(pBufferMsg, &(pMsg->Body.PropertyCount));
								break;
							case osl_DaemonMgrMsgSubject_PropertiesGet:
								if (!readDWORD(pBufferMsg, &(pMsg->Body.Properties.Count)))
									ret = osl_Daemon_ManagerError_Unknown;
								else
								{
									if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
									{
										pMsg->Body.Properties.List = osl_createDaemonProperties(pMsg->Body.Properties.Count);
										for (i = 0; (ret == osl_Daemon_ManagerError_NoError) && (i < pMsg->Body.Properties.Count); i++)
										{
											if (!readPROPERTY(pBufferMsg, &(pMsg->Body.Properties.List[i])))
												ret = osl_Daemon_ManagerError_Unknown;
										}

										if (ret != osl_Daemon_ManagerError_NoError) 
										{
											osl_destroyDaemonProperties(pMsg->Body.Properties.List, pMsg->Body.Properties.Count);
											pMsg->Body.Properties.List = NULL;
											pMsg->Body.Properties.Count = 0;
										}
									}
								}
								break;
							case osl_DaemonMgrMsgSubject_PropertiesSet:
								if (pMsg->Type == osl_DaemonMgrMsgType_Request)
								{
									if (!readDWORD(pBufferMsg, &(pMsg->Body.Properties.Count)))
										ret = osl_Daemon_ManagerError_Unknown;
									else
									{
										pMsg->Body.Properties.List = osl_createDaemonProperties(pMsg->Body.Properties.Count);
										for (i = 0; (ret == osl_Daemon_ManagerError_NoError) && (i < pMsg->Body.Properties.Count); i++)
										{
											if (!readPROPERTY(pBufferMsg, &(pMsg->Body.Properties.List[i])))
												ret = osl_Daemon_ManagerError_Unknown;
										}

										if (ret != osl_Daemon_ManagerError_NoError) 
										{
											osl_destroyDaemonProperties(pMsg->Body.Properties.List, pMsg->Body.Properties.Count);
											pMsg->Body.Properties.List = NULL;
											pMsg->Body.Properties.Count = 0;
										}
									}
								}
								else
								{
									if (!readDWORD(pBufferMsg, &(pMsg->Body.Properties.ErrorCode)))
										ret = osl_Daemon_ManagerError_Unknown;
									else if (!readUSTRING(pBufferMsg, &pMsg->Body.Properties.ErrorText))
										ret = osl_Daemon_ManagerError_Unknown;
								}
								break;
							case osl_DaemonMgrMsgSubject_ProfileGetSections:
								if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
								{
									if (!read_and_alloc_STRINGLIST(pBufferMsg, &(pMsg->Body.Profile.Section)))
										ret = osl_Daemon_ManagerError_Unknown;
								}
								break;
							case osl_DaemonMgrMsgSubject_ProfileGetEntries:
								if (pMsg->Type == osl_DaemonMgrMsgType_Request)
								{
									if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Section)))
										ret = osl_Daemon_ManagerError_Unknown;
								}
								else
								{
									if (!read_and_alloc_STRINGLIST(pBufferMsg, &(pMsg->Body.Profile.Entry)))
										ret = osl_Daemon_ManagerError_Unknown;
								}
								break;
							case osl_DaemonMgrMsgSubject_ProfileReadEntry:
								if (pMsg->Type == osl_DaemonMgrMsgType_Request)
								{
									if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Section)))
										ret = osl_Daemon_ManagerError_Unknown;
									else
									{
										if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Entry)))
											ret = osl_Daemon_ManagerError_Unknown;
										else
										{
											if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Default)))
												ret = osl_Daemon_ManagerError_Unknown;
										}
									}
								}
								else
								{
									if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Value)))
										ret = osl_Daemon_ManagerError_Unknown;
								}
								break;
							case osl_DaemonMgrMsgSubject_ProfileWriteEntry:
								if (pMsg->Type == osl_DaemonMgrMsgType_Request)
								{
									if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Section)))
										ret = osl_Daemon_ManagerError_Unknown;
									else
									{
										if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Entry)))
											ret = osl_Daemon_ManagerError_Unknown;
										else
										{
											if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Value)))
												ret = osl_Daemon_ManagerError_Unknown;
										}
									}
								}
								break;
							case osl_DaemonMgrMsgSubject_ProfileRemoveEntry:
								if (pMsg->Type == osl_DaemonMgrMsgType_Request)
								{
									if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Section)))
										ret = osl_Daemon_ManagerError_Unknown;
									else
									{
										if (!read_and_alloc_STRING(pBufferMsg, &(pMsg->Body.Profile.Entry)))
											ret = osl_Daemon_ManagerError_Unknown;
									}
								}
								break;
							case osl_DaemonMgrMsgSubject_QueryChanges:
								if (!readDWORD(pBufferMsg, &(pMsg->Body.Changes.since.Seconds)))
									ret = osl_Daemon_ManagerError_Unknown;
								else if (!readDWORD(pBufferMsg, &(pMsg->Body.Changes.since.Nanosec)))
									ret = osl_Daemon_ManagerError_Unknown;
								else if (pMsg->Type == osl_DaemonMgrMsgType_Reply)
								{
									if (!readDWORD(pBufferMsg, (sal_uInt32*) &(pMsg->Body.Changes.bChanged)))
										ret = osl_Daemon_ManagerError_Unknown;
								}
								break;
							default:
								break;
						}		
					}
				}

				deleteBuffer(pBufferMsg);
				pBufferMsg = NULL;
			}
			else
			{
				ret = osl_Daemon_ManagerError_Unknown;
			}
		}

        if ( strIdentifier != 0 )
        {
            rtl_uString_release(strIdentifier);
        }
        
		deleteBuffer(pBufferHeader);
		pBufferHeader = NULL;

		if (ret == osl_Daemon_ManagerError_NoError)
			*ppMsg = pMsg;
		else
			if (pMsg) osl_destroyDaemonMgrMsg(pMsg);

		return(ret);
	}
	else
		return(osl_Daemon_ManagerError_NoManager);
}	



/*****************************************************************************/
/* osl_createDaemonMgrMsg */
/*****************************************************************************/
oslDaemonMgrMsg* SAL_CALL osl_createDaemonMgrMsg(rtl_uString* identifier, oslDaemonMgrMsgSubject subject)
{
	oslDaemonMgrMsg* pMsg = (oslDaemonMgrMsg*) rtl_allocateZeroMemory(sizeof(oslDaemonMgrMsg));

	if (
		(subject == osl_DaemonMgrMsgSubject_Install) ||
		(subject == osl_DaemonMgrMsgSubject_QueryDaemonInfo)
	   )
	{
		pMsg->Body.pInfo = (oslDaemonInfo*) rtl_allocateZeroMemory(sizeof(oslDaemonInfo));
		pMsg->Body.pInfo->Size = sizeof(oslDaemonInfo);
	}

	pMsg->Size		= sizeof(oslDaemonMgrMsg);
	pMsg->Subject 	= subject;
	pMsg->Error 	= osl_Daemon_ManagerError_NoError;

	if (identifier)
		rtl_uString_assign(&pMsg->Identifier, identifier);

	return(pMsg);
}	

/*****************************************************************************/
/* osl_destroyDaemonMgrMsg */
/*****************************************************************************/
void SAL_CALL osl_destroyDaemonMgrMsg(oslDaemonMgrMsg* pMsg)
{
	if (pMsg->Identifier)
		rtl_uString_release(pMsg->Identifier);

	if (
		(pMsg->Subject == osl_DaemonMgrMsgSubject_Install) ||
		(pMsg->Subject == osl_DaemonMgrMsgSubject_QueryDaemonInfo)
	   )
	{
		rtl_freeMemory(pMsg->Body.pInfo);
	}
	else if (
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_PropertiesGet) ||
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_PropertiesSet)
			)
	{
		if (pMsg->Body.Properties.ErrorText)
			rtl_uString_release(pMsg->Body.Properties.ErrorText);

		osl_destroyDaemonProperties(pMsg->Body.Properties.List, pMsg->Body.Properties.Count);
	}
	else if (
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_ProfileGetSections) ||
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_ProfileGetEntries) ||
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_ProfileReadEntry) ||
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_ProfileWriteEntry) ||
			 (pMsg->Subject == osl_DaemonMgrMsgSubject_ProfileRemoveEntry) 
			)
	{
		if (pMsg->Body.Profile.Section)
			rtl_freeMemory(pMsg->Body.Profile.Section);

		if (pMsg->Body.Profile.Entry)
			rtl_freeMemory(pMsg->Body.Profile.Entry);

		if (pMsg->Body.Profile.Value)
			rtl_freeMemory(pMsg->Body.Profile.Value);

		if (pMsg->Body.Profile.Default)
			rtl_freeMemory(pMsg->Body.Profile.Default);
	}

	rtl_freeMemory(pMsg);	
}	

/*****************************************************************************/
/* osl_createDaemonProperties */
/*****************************************************************************/
oslDaemonProperty* SAL_CALL osl_createDaemonProperties(sal_uInt32 count)
{
	oslDaemonProperty*  ret = (oslDaemonProperty*) rtl_allocateMemory(count * sizeof(oslDaemonProperty));

	sal_uInt32 i = 0;
	
	memset(ret, 0, count * sizeof(oslDaemonProperty));

	for (i = 0; i < count; i++)
	{
		ret[i].Size = sizeof(oslDaemonProperty);
	}	

	return(ret);
}	

/*****************************************************************************/
/* osl_destroyDaemonProperties */
/*****************************************************************************/
void SAL_CALL osl_destroyDaemonProperties(oslDaemonProperty* list, sal_uInt32 count)
{
	if (list != NULL)
	{
		sal_uInt32 i = 0, j = 0;

		for (i = 0; i < count; i++)
		{
			rtl_uString_release(list[i].Name);

			switch (list[i].Type)
			{
				case osl_Daemon_PropertyType_Boolean:
					break;
				case osl_Daemon_PropertyType_uLong:
					break;
				case osl_Daemon_PropertyType_String:
					if (list[i].Value.aString != NULL)
						rtl_freeMemory(list[i].Value.aString);			
					break;
				case osl_Daemon_PropertyType_BooleanList:
					rtl_freeMemory(list[i].Value.aList.Data.pBool);
					break;
				case osl_Daemon_PropertyType_uLongList:
					rtl_freeMemory(list[i].Value.aList.Data.pULong);
					break;
				case osl_Daemon_PropertyType_StringList:
					for (j = 0; j < list[i].Value.aList.Count; j++)
					{
						if (list[i].Value.aList.Data.pString[j] != NULL)
							rtl_freeMemory(list[i].Value.aList.Data.pString[j]);
					}				
					rtl_freeMemory(list[i].Value.aList.Data.pString);
					break;
				default:
					break;
			}
		}	

		rtl_freeMemory(list);
	}
}	

/*****************************************************************************/
/* osl_initDaemonProperty */
/*****************************************************************************/
void SAL_CALL osl_initDaemonProperty(oslDaemonProperty* pProp, oslDaemonPropertyType type, rtl_uString* strName, sal_uInt32 listSize)
{
	pProp->Type = type;
	pProp->Name = strName;
	rtl_uString_acquire(strName);

	switch (type)
	{
		case osl_Daemon_PropertyType_Boolean:
			pProp->Value.aBool = sal_False;			
			break;
		case osl_Daemon_PropertyType_uLong:
			pProp->Value.aULong = 0;			
			break;
		case osl_Daemon_PropertyType_String:
			pProp->Value.aString = NULL;			
			break;
		case osl_Daemon_PropertyType_BooleanList:
			pProp->Value.aList.Count = listSize;
			pProp->Value.aList.Data.pBool = (sal_Bool*) rtl_allocateMemory(listSize * sizeof(sal_Bool));
			memset(pProp->Value.aList.Data.pBool, 0, listSize * sizeof(sal_Bool));
			break;
		case osl_Daemon_PropertyType_uLongList:
			pProp->Value.aList.Count = listSize;
			pProp->Value.aList.Data.pULong = (sal_uInt32*) rtl_allocateMemory(listSize * sizeof(sal_uInt32));
			memset(pProp->Value.aList.Data.pULong, 0, listSize * sizeof(sal_uInt32));
			break;
		case osl_Daemon_PropertyType_StringList:
			pProp->Value.aList.Count = listSize;
			pProp->Value.aList.Data.pString = (sal_Unicode**) rtl_allocateMemory(listSize * sizeof(sal_Unicode*));
			memset(pProp->Value.aList.Data.pString, 0, listSize * sizeof(sal_Char*));
			break;
		default:
			break;
	}		
}	

