/*************************************************************************
 *
 *  $RCSfile: daemprot.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_DAEMPROT_H_
#define _OSL_DAEMPROT_H_


#ifndef _OSL_DAEMON_H_
	#include <osl/daemon.h>
#endif
#ifndef _OSL_PROCESS_H_
	#include <osl/process.h>
#endif

#define OSL_DAEMON_MANAGER_PIPE "SDAEMON"

/** type of a daemon message
*/
typedef enum 
{
	osl_DaemonMgrMsgType_Request,
	osl_DaemonMgrMsgType_Reply,
	osl_DaemonMgrMsgType_Unknown = -1

} oslDaemonMgrMsgType;

/** subject of a daemon message
*/
typedef enum
{
	osl_DaemonMgrMsgSubject_Shutdown,	
	osl_DaemonMgrMsgSubject_CloseConnection,	
	osl_DaemonMgrMsgSubject_CheckConnection,	
	osl_DaemonMgrMsgSubject_QueryDaemonInfo,
	osl_DaemonMgrMsgSubject_QueryDaemonCount,
	osl_DaemonMgrMsgSubject_QueryDaemonIdentifier,
	osl_DaemonMgrMsgSubject_Connect,
	osl_DaemonMgrMsgSubject_Install,
	osl_DaemonMgrMsgSubject_Uninstall,
	osl_DaemonMgrMsgSubject_Command,
	osl_DaemonMgrMsgSubject_StatusGet,
	osl_DaemonMgrMsgSubject_PropertyCountGet,
	osl_DaemonMgrMsgSubject_PropertiesGet,
	osl_DaemonMgrMsgSubject_PropertiesSet,
	osl_DaemonMgrMsgSubject_ProfileGetSections,
	osl_DaemonMgrMsgSubject_ProfileGetEntries,
	osl_DaemonMgrMsgSubject_ProfileReadEntry,
	osl_DaemonMgrMsgSubject_ProfileWriteEntry,
	osl_DaemonMgrMsgSubject_ProfileRemoveEntry,
	osl_DaemonMgrMsgSubject_QueryChanges,

	osl_DaemonMgrMsgSubject_Unknown = -1

} oslDaemonMgrMsgSubject;

/**	definition of the body of a daemon message
*/
typedef union
{
	oslDaemonInfo* 			pInfo;
	oslDaemonCommand 		Command;		
	oslDaemonStatus 		Status;		
	sal_uInt32				PropertyCount;

	struct
	{
		sal_uInt32 			Count;
		oslDaemonProperty*  List;
		sal_uInt32			ErrorCode;
		rtl_uString*		ErrorText;
	} Properties;

	struct 
	{
		sal_Unicode*		Section;
		sal_Unicode*		Entry;
		sal_Unicode*		Value;
		sal_Unicode*		Default;
	} Profile;

	struct 
	{
		TimeValue 	since;
		sal_Bool	bChanged;
	} Changes;

	sal_uInt32				DaemonCount;
	sal_uInt32				DaemonIndex;
	oslProcessIdentifier	ProcessIdentifier;
} oslDaemonMgrMsgBody;

/** definition of a daemom message
*/
typedef struct
{
	size_t					Size;
	rtl_uString	*			Identifier;
	oslDaemonMgrMsgType		Type;
	oslDaemonMgrMsgSubject	Subject;
	oslDaemonManagerError 	Error;	
	oslDaemonMgrMsgBody		Body;

} oslDaemonMgrMsg;

typedef void* oslDaemonConnection;
typedef sal_uInt32 oslDaemonConnectionOptions;

#define	osl_DaemonConnection_OPEN        0x0000  	
#define	osl_DaemonConnection_CREATE      0x0001     
#define	osl_DaemonConnection_ACCEPT      0x0002

/**	opens a new pipe to the daemon manager (options == osl_Pipe_OPEN) or 
	creates the pipe on dameon manager side (options == osl_Pipe_CREATE)
*/
oslDaemonConnection SAL_CALL osl_openDaemonMgrConn(rtl_uString* name, oslDaemonConnectionOptions options);
oslDaemonConnection SAL_CALL osl_openDaemonMgrConnA(sal_Char* name, oslDaemonConnectionOptions options);

/**	accept a new connection on the daemon mananger pipe
*/
oslDaemonConnection SAL_CALL osl_acceptOnDaemonMgrConn(oslDaemonConnection conn);

/**	close the connection to the daemon mananger
*/
void SAL_CALL osl_disconnectDaemonMgrConn(oslDaemonConnection conn);

/**	create a new message with given subject
*/
oslDaemonMgrMsg* SAL_CALL osl_createDaemonMgrMsg(rtl_uString* identifier, oslDaemonMgrMsgSubject subject);

/** free the allocated memory of a message
*/
void SAL_CALL osl_destroyDaemonMgrMsg(oslDaemonMgrMsg* pMsg);

/** send the given message 
*/
oslDaemonManagerError SAL_CALL osl_sendDaemonMgrMsg(oslDaemonConnection conn, oslDaemonMgrMsg* pMsg);

/** recieve a new message 
*/
oslDaemonManagerError SAL_CALL osl_recvDaemonMgrMsg(oslDaemonConnection conn, oslDaemonMgrMsg** ppMsg);

#endif /* _OSL_DAEMPROT_H_ */

