/*************************************************************************
 *
 *  $RCSfile: daemreg.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string.h>
#include <stdio.h>

#ifndef _OSL_PROFILE_H_
#   include <osl/profile.h>
#endif
#ifndef _OSL_DEAMREG_H_
#   include "daemreg.h"
#endif

#include <osl/diagnose.h>
#include <rtl/alloc.h>
#include <rtl/memory.h>


#define DAEMON_LIST_SECTION				"daemons"
	
#define MAX_PROFILE_SECTION_LEN 		128
#define MAX_PROFILE_VALUE_LEN			1024

#define PRIVATE_SECTION_PREFIX			"-Section-"
#define PRIVATE_SECTION_LIST_PREFIX		"-Section"
#define PRIVATE_SECTION_LINK_ENTRY		"Sections"


static oslProfile hGlobalProfile = NULL;

/*****************************************************************************/
/* getPrivateDaemonLink */
/*****************************************************************************/

static sal_Bool getPrivateDaemonLink(const sal_Char* identifier, sal_Char* buffer, sal_uInt32 maxLen)
{
	if ((! osl_readProfileString(hGlobalProfile, identifier, PRIVATE_SECTION_LINK_ENTRY,
						         buffer, maxLen, "")) ||
	    (strlen(buffer) == 0))
	{
		strcpy(buffer, identifier);
		strcat(buffer, PRIVATE_SECTION_LIST_PREFIX); 

		osl_writeProfileString(hGlobalProfile, identifier, PRIVATE_SECTION_LINK_ENTRY,
						       buffer);
	}								         						  
	
	return (sal_True);
}


/*****************************************************************************/
/* mapDaemonSection */
/*****************************************************************************/

static sal_Bool mapPrivateDaemonSection(const sal_Char* identifier, const sal_Char* section, sal_Char* buffer, sal_uInt32 maxLen)
{
	sal_Char mapSection[MAX_PROFILE_SECTION_LEN];

	if (getPrivateDaemonLink(identifier, mapSection, sizeof(mapSection)))
	{
		if ((!osl_readProfileString(hGlobalProfile, mapSection, section,
							        buffer, maxLen, "")) || 
			(strlen(buffer) == 0))
		{
			strcpy(buffer, identifier);
			strcat(buffer, PRIVATE_SECTION_PREFIX); 
			strcat(buffer, section); 

			osl_writeProfileString(hGlobalProfile, mapSection, section, buffer);
		}								         						  

		return (sal_True);
	}

	return (sal_False);
}

/*****************************************************************************/
/* getPrivateDaemonSections */
/*****************************************************************************/

static sal_uInt32 getPrivateDaemonSections(const sal_Char* identifier, sal_Char* buffer, sal_uInt32 maxBuffer)
{
	sal_Char mapSection[MAX_PROFILE_SECTION_LEN];
	sal_uInt32 ret = 0;

	if (getPrivateDaemonLink(identifier, mapSection, sizeof(mapSection)))
	{
		ret = osl_getProfileSectionEntries(hGlobalProfile, mapSection, buffer, maxBuffer);
	}

	return(ret);
}

/*****************************************************************************/
/* getPrivateDaemonSections */
/*****************************************************************************/

static sal_uInt32 getPrivateDaemonEntries(const sal_Char* identifier, const sal_Char* section, sal_Char* buffer, sal_uInt32 maxBuffer)
{
	sal_Char mapSection[MAX_PROFILE_SECTION_LEN];
	sal_uInt32 ret = 0;

	if (mapPrivateDaemonSection(identifier, section, mapSection, sizeof(mapSection)))
	{
		ret = osl_getProfileSectionEntries(hGlobalProfile, mapSection, buffer, maxBuffer);
	}

	return(ret);
}

/*****************************************************************************/
/* removePrivateDaemonSection */
/*****************************************************************************/

static void removePrivateDaemonSection(const sal_Char* identifier, const sal_Char* section)
{
	sal_Char mapSection[MAX_PROFILE_SECTION_LEN];
	sal_uInt32 size = 0;
	sal_Char* entries = NULL;
	const sal_Char* entry = NULL;

	if (mapPrivateDaemonSection(identifier, section, mapSection, sizeof(mapSection)))
	{
		size = osl_getProfileSectionEntries(hGlobalProfile, mapSection, NULL, 0);

		if (size > 0)	
		{
			entries = (sal_Char*) rtl_allocateMemory(size);

			osl_getProfileSectionEntries(hGlobalProfile, mapSection, entries, size);

			for (entry = entries; *entry != '\0'; entry += strlen(entry) + 1)
			{
				osl_removeProfileEntry(hGlobalProfile, mapSection, entry);
			}	
						
			rtl_freeMemory(entries);

			if (getPrivateDaemonLink(identifier, mapSection, sizeof(mapSection)))
			{
				osl_removeProfileEntry(hGlobalProfile, mapSection, section);
			}
		}
	}
}


/*****************************************************************************/
/* default_hasDaemon */
/*****************************************************************************/

static sal_Bool SAL_CALL default_hasDaemon(rtl_uString* strIdentifier)
{
	sal_Char buffer[OSL_DAEMON_NAME_LEN];
	rtl_String* Identifier = NULL;
    sal_Char* pszIdentifier=0;
    
    sal_Bool ret;

    if ( strIdentifier != 0 )
    {
        
        rtl_uString2String(&Identifier, 
                           strIdentifier->buffer, 
                           strIdentifier->length, 
                           RTL_TEXTENCODING_UTF8,
                           OUSTRING_TO_OSTRING_CVTFLAGS);

        pszIdentifier = rtl_string_getStr(Identifier);
    }
    

	OSL_ASSERT(hGlobalProfile != NULL);

	ret = osl_readProfileString(hGlobalProfile, 
		DAEMON_LIST_SECTION,
		pszIdentifier, 
		buffer,
		sizeof(buffer),
		"");

	if (ret)
		ret = strlen(buffer) != 0;

    if ( Identifier != 0 )
    {
        rtl_string_release(Identifier);
    }
    
	return ret;
}	

/*****************************************************************************/
/* default_getDaemons */
/*****************************************************************************/

static sal_uInt32 SAL_CALL default_getDaemons(sal_Unicode* pszBuffer, sal_uInt32 MaxLen)
{
	sal_Char   *Buffer = NULL;
	sal_uInt32 ret;

	OSL_ASSERT(hGlobalProfile != NULL);

	if (pszBuffer)
		Buffer = (sal_Char*) rtl_allocateMemory(MaxLen * 3);

	ret =  osl_getProfileSectionEntries(hGlobalProfile, 
										DAEMON_LIST_SECTION,
	    						    	Buffer, 
	    						    	MaxLen * 3);

	if (Buffer)
	{
		sal_Char *pSrc = Buffer;
		sal_Unicode *pDest = pszBuffer;
		rtl_uString *strTmp = NULL;
		sal_Int32 leftSpace = MaxLen - 1;

		while (*pSrc)
		{
			size_t len = strlen(pSrc);
			rtl_string2UString(&strTmp, 
				pSrc, len, 
				RTL_TEXTENCODING_UTF8, 
				OSTRING_TO_OUSTRING_CVTFLAGS);

			pSrc += len + 1;
			if (strTmp->length < leftSpace)
			{
				rtl_copyMemory(pDest, strTmp->buffer, (strTmp->length + 1) * sizeof(sal_Unicode));
				pDest += strTmp->length + 1;
			}
			else
				break;
		}

		*pDest = 0;
		rtl_uString_release(strTmp);
		rtl_freeMemory(Buffer);
	}

	return ret;
}	

/*****************************************************************************/
/* default_addDaemon */
/*****************************************************************************/

static sal_Bool SAL_CALL default_addDaemon(rtl_uString* strIdentifier, rtl_uString* strDisplayName)
{
	sal_Bool ret = sal_False;

	OSL_ASSERT(hGlobalProfile != NULL);

	if (!default_hasDaemon(strIdentifier))
	{
		rtl_String* Identifier = NULL;
		rtl_String* DisplayName = NULL;

		rtl_uString2String(&Identifier, 
			strIdentifier->buffer, 
			strIdentifier->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);
		rtl_uString2String(&DisplayName, 
			strDisplayName->buffer, 
			strDisplayName->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		ret = osl_writeProfileString(hGlobalProfile,
                               		 DAEMON_LIST_SECTION, 
                               		 Identifier->buffer, 
							   		 DisplayName->buffer);

	 	rtl_string_release(Identifier);
		rtl_string_release(DisplayName);
	}

	return(ret);
}	

/*****************************************************************************/
/* default_removeDaemon */
/*****************************************************************************/

static void SAL_CALL default_removeDaemon(rtl_uString* strIdentifier)
{
	sal_uInt32 	size = 0;
	sal_Char* 	entries = NULL;
	const sal_Char* 	entry = NULL;
	sal_Char* 	sections = NULL;
	const sal_Char* 	section = NULL;

	OSL_ASSERT(hGlobalProfile != NULL);

	if (default_hasDaemon(strIdentifier))
	{
		rtl_String* Identifier = NULL;

		rtl_uString2String(&Identifier, 
			strIdentifier->buffer, 
			strIdentifier->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		/* remove private sections */
		size = getPrivateDaemonSections(Identifier->buffer, NULL, 0);

		if (size > 0)
		{
			sections = (sal_Char*) rtl_allocateMemory(size);

			getPrivateDaemonSections(Identifier->buffer, sections, size);

			for (section = sections; *section != '\0'; section += strlen(section) + 1)
			{
				removePrivateDaemonSection(Identifier->buffer, section);
			}	
			
			rtl_freeMemory(sections);
		}

		/* remove entries */
		size = osl_getProfileSectionEntries(hGlobalProfile, Identifier->buffer, NULL, 0);

		if (size > 0)
		{
			entries = (sal_Char*) rtl_allocateMemory(size);

			osl_getProfileSectionEntries(hGlobalProfile, Identifier->buffer, entries, size);

			for (entry = entries; *entry != '\0'; entry += strlen(entry) + 1)
			{
				osl_removeProfileEntry(hGlobalProfile, Identifier->buffer, entry);
			}	

			rtl_freeMemory(entries);
		}

		/* remove list entry */
		osl_removeProfileEntry(hGlobalProfile, DAEMON_LIST_SECTION, Identifier->buffer);
	 	rtl_string_release(Identifier);
	}
}	

/*****************************************************************************/
/* default_readDaemonEntry */
/*****************************************************************************/

static sal_Bool SAL_CALL default_readDaemonEntry(rtl_uString* strIdentifier,	
                                        rtl_uString* strSection, 
                                        rtl_uString* strEntry, 
                                        sal_Unicode* strString, 
                                        sal_uInt32 	   MaxLen, 
                                        const sal_Unicode* strDefault)
{
	sal_Bool ret	= sal_False;

	rtl_String *Identifier = NULL;
	rtl_String *Section = NULL;
	rtl_String *Entry = NULL;
	rtl_String *DefaultValue = NULL;
	sal_Char   *pszIdentifier = 0;
	sal_Char   *pszSection = NULL;
	sal_Char   *pszEntry = NULL;
	sal_Char   *pszDefault = NULL;
	sal_Char   *Buffer = NULL;

	OSL_ASSERT(hGlobalProfile != NULL);

 
    if ( strIdentifier != 0 )
    {
        rtl_uString2String(&Identifier, 
                           strIdentifier->buffer, 
                           strIdentifier->length, 
                           RTL_TEXTENCODING_UTF8,
                           OUSTRING_TO_OSTRING_CVTFLAGS);

        pszIdentifier = Identifier->buffer;
    }
    

	if (strSection)
	{
		rtl_uString2String(&Section, 
			strSection->buffer, 
			strSection->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszSection = Section->buffer;
	}

	if (strEntry)
	{
		rtl_uString2String(&Entry, 
			strEntry->buffer, 
			strEntry->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszEntry = Entry->buffer;
	}

	if (strDefault)
	{
		rtl_uString2String(&DefaultValue, 
			strDefault, 
			rtl_ustr_getLength(strDefault), 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszDefault = DefaultValue->buffer;
	}

	if (strString)
    {    
		Buffer = (sal_Char*) rtl_allocateMemory(MaxLen * 3);
/*        memset(Buffer,0,MaxLen*3);*/
    }
    

	if (pszSection == NULL)	
	{
		ret = osl_readProfileString(hGlobalProfile, 
                              		pszIdentifier, 
                              		pszEntry, 
                              		Buffer,
                              		MaxLen * 3,
                              		pszDefault);
	}
	else
	{
		sal_Char mapSection[MAX_PROFILE_SECTION_LEN];

		if (mapPrivateDaemonSection(Identifier->buffer, pszSection, mapSection, sizeof(mapSection)))
		{
			ret = osl_readProfileString(hGlobalProfile, 
	                              		mapSection, 
	                              		pszEntry, 
	                              		Buffer,
	                              		MaxLen * 3,
	                              		pszDefault);
		}
	}

	if (Buffer)
	{
		sal_Char *pSrc = Buffer;
		sal_Unicode *pDest = strString;
		rtl_uString *strTmp = NULL;
		sal_Int32 leftSpace = MaxLen - 1;

		while (*pSrc)
		{
			size_t len = strlen(pSrc);
  			rtl_string2UString(&strTmp,
  				pSrc, len,
  				RTL_TEXTENCODING_UTF8,
  				OSTRING_TO_OUSTRING_CVTFLAGS);
			pSrc += len;
            
			if (strTmp->length < leftSpace)
			{
				rtl_copyMemory(pDest, strTmp->buffer, (strTmp->length + 1) * sizeof(sal_Unicode));
				pDest += strTmp->length + 1;
			}
			else
				break;
		}

		*pDest = 0;
        if ( strTmp != 0 )
        {    
            rtl_uString_release(strTmp);
        }
        
		rtl_freeMemory(Buffer);
	}

    if ( Identifier != 0 )
    {    
        rtl_string_release(Identifier);
    }
    
	if ( Section != 0 )
    {
        rtl_string_release(Section);
    }

    if ( Entry != 0 )
    {    
        rtl_string_release(Entry);
    }
    
	if ( DefaultValue != 0 )
    {
        rtl_string_release(DefaultValue);
    }
    
	return(ret);
}	

/*****************************************************************************/
/* default_writeDaemonEntry */
/*****************************************************************************/

static sal_Bool SAL_CALL default_writeDaemonEntry(rtl_uString* strIdentifier,	
									   	rtl_uString* strSection, 
									    rtl_uString* strEntry, 
			                            const sal_Unicode* strString)
{
	sal_Bool ret	= sal_False;

	rtl_String *Identifier = NULL;
	rtl_String *Section = NULL;
	rtl_String *Entry = NULL;
	rtl_String *pString = NULL;
	sal_Char   *pszSection = NULL;
	sal_Char   *pszEntry = NULL;
	sal_Char   *pszString = NULL;

	OSL_ASSERT(hGlobalProfile != NULL);

	rtl_uString2String(&Identifier, 
		strIdentifier->buffer, 
		strIdentifier->length, 
		RTL_TEXTENCODING_UTF8,
		OUSTRING_TO_OSTRING_CVTFLAGS);

	if (strSection)
	{
		rtl_uString2String(&Section, 
			strSection->buffer, 
			strSection->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszSection = Section->buffer;
	}

	if (strEntry)
	{
		rtl_uString2String(&Entry, 
			strEntry->buffer, 
			strEntry->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszEntry = Entry->buffer;
	}
	
	if (strString)
	{
		rtl_uString2String(&pString, 
			strString, 
			rtl_ustr_getLength(strString), 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszString = pString->buffer;
	}

	if (pszSection == NULL)	
	{
		ret = osl_writeProfileString(hGlobalProfile, 
                              		 Identifier->buffer, 
                              		 pszEntry, 
                              		 pszString);
	}
	else
	{
		sal_Char mapSection[MAX_PROFILE_SECTION_LEN];

		if (mapPrivateDaemonSection(Identifier->buffer, pszSection, mapSection, sizeof(mapSection)))
		{
			ret = osl_writeProfileString(hGlobalProfile, 
	                              		 mapSection, 
	                              		 pszEntry, 
	                              		 pszString);
		}
	}

	rtl_string_release(Identifier);
	rtl_string_release(Section);
	rtl_string_release(Entry);
	rtl_string_release(pString);
	return(ret);
}	

/*****************************************************************************/
/* default_removeDaemonEntry */
/*****************************************************************************/

static sal_Bool SAL_CALL default_removeDaemonEntry(rtl_uString* strIdentifier,	
									      rtl_uString* strSection, 
									      rtl_uString* strEntry)
{
	sal_Bool ret	= sal_False;

	rtl_String *Identifier = NULL;
	rtl_String *Section = NULL;
	rtl_String *Entry = NULL;
	sal_Char   *pszSection = NULL;
	sal_Char   *pszEntry = NULL;

	OSL_ASSERT(hGlobalProfile != NULL);

	rtl_uString2String(&Identifier, 
		strIdentifier->buffer, 
		strIdentifier->length, 
		RTL_TEXTENCODING_UTF8,
		OUSTRING_TO_OSTRING_CVTFLAGS);

	if (strSection)
	{
		rtl_uString2String(&Section, 
			strSection->buffer, 
			strSection->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszSection = Section->buffer;
	}

	if (strEntry)
	{
		rtl_uString2String(&Entry, 
			strEntry->buffer, 
			strEntry->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszEntry = Entry->buffer;
	}

	if (pszSection == NULL)	
	{
		ret = osl_removeProfileEntry(hGlobalProfile, Identifier->buffer, pszEntry);
	}
	else
	{
		sal_Char mapSection[MAX_PROFILE_SECTION_LEN];

		if (mapPrivateDaemonSection(Identifier->buffer, pszSection, mapSection, sizeof(mapSection)))
		{
			ret = osl_removeProfileEntry(hGlobalProfile, mapSection, pszEntry);
		}
	}

	rtl_string_release(Identifier);
	rtl_string_release(Section);
	rtl_string_release(Entry);
	return(ret);
}	

/*****************************************************************************/
/* default_getSections */
/*****************************************************************************/

static sal_uInt32 SAL_CALL default_getSections(rtl_uString* strIdentifier,
								  sal_Unicode* 	  pszBuffer, 
			    				  sal_uInt32 	  MaxLen)
{
	rtl_String *Identifier = NULL;
	sal_Char   *Buffer = NULL;
	sal_uInt32 ret;

	OSL_ASSERT(hGlobalProfile != NULL);

	rtl_uString2String(&Identifier, 
		strIdentifier->buffer, 
		strIdentifier->length, 
		RTL_TEXTENCODING_UTF8,
		OUSTRING_TO_OSTRING_CVTFLAGS);

	if (pszBuffer)
		Buffer = (sal_Char*) rtl_allocateMemory(MaxLen * 3);

	ret = getPrivateDaemonSections(Identifier->buffer, Buffer, MaxLen * 3);

	if (Buffer)
	{
		sal_Char *pSrc = Buffer;
		sal_Unicode *pDest = pszBuffer;
		rtl_uString *strTmp = NULL;
		sal_Int32 leftSpace = MaxLen - 1;

		while (*pSrc)
		{
			size_t len = strlen(pSrc);
			rtl_string2UString(&strTmp, 
				pSrc, len, 
				RTL_TEXTENCODING_UTF8, 
				OSTRING_TO_OUSTRING_CVTFLAGS);

			pSrc += len + 1;
			if (strTmp->length < leftSpace)
			{
				rtl_copyMemory(pDest, strTmp->buffer, (strTmp->length + 1) * sizeof(sal_Unicode));
				pDest += strTmp->length + 1;
			}
			else
				break;
		}

		*pDest = 0;
		rtl_uString_release(strTmp);
		rtl_freeMemory(Buffer);
	}

	rtl_string_release(Identifier);
	return ret;
}

/*****************************************************************************/
/* default_getEntries */
/*****************************************************************************/

static sal_uInt32 SAL_CALL default_getEntries(rtl_uString* strIdentifier,
								 rtl_uString* strSection,	
								 sal_Unicode*  pszBuffer, 
			    				 sal_uInt32 	 MaxLen)
{
	rtl_String *Identifier = NULL;
	rtl_String *Section = NULL;
	sal_Char   *pszSection = NULL;
	sal_Char   *Buffer = NULL;
	sal_uInt32 ret;

	OSL_ASSERT(hGlobalProfile != NULL);

	rtl_uString2String(&Identifier, 
		strIdentifier->buffer, 
		strIdentifier->length, 
		RTL_TEXTENCODING_UTF8,
		OUSTRING_TO_OSTRING_CVTFLAGS);

	if (strSection)
	{
		rtl_uString2String(&Section, 
			strSection->buffer, 
			strSection->length, 
			RTL_TEXTENCODING_UTF8,
			OUSTRING_TO_OSTRING_CVTFLAGS);

		pszSection = Section->buffer;
	}
	
	if (pszBuffer)
		Buffer = (sal_Char*) rtl_allocateMemory(MaxLen * 3);

	ret = getPrivateDaemonEntries(Identifier->buffer, pszSection, Buffer, MaxLen * 3);

	if (Buffer)
	{
		sal_Char *pSrc = Buffer;
		sal_Unicode *pDest = pszBuffer;
		rtl_uString *strTmp = NULL;
		sal_Int32 leftSpace = MaxLen - 1;

		while (*pSrc)
		{
			size_t len = strlen(pSrc);
			rtl_string2UString(&strTmp, 
				pSrc, len, 
				RTL_TEXTENCODING_UTF8, 
				OSTRING_TO_OUSTRING_CVTFLAGS);

			pSrc += len + 1;
			if (strTmp->length < leftSpace)
			{
				rtl_copyMemory(pDest, strTmp->buffer, (strTmp->length + 1) * sizeof(sal_Unicode));
				pDest += strTmp->length + 1;
			}
			else
				break;
		}

		*pDest = 0;
		rtl_uString_release(strTmp);
		rtl_freeMemory(Buffer);
	}

	rtl_string_release(Section);
	rtl_string_release(Identifier);
	return ret;
}	

/*****************************************************************************/
/* globalApi */
/*****************************************************************************/

static DaemonRegistryApi globalApi =
{
	default_hasDaemon,
	default_getDaemons,
	default_addDaemon,
	default_removeDaemon,
	default_readDaemonEntry,	
	default_writeDaemonEntry,	
	default_removeDaemonEntry,	
	default_getSections,
	default_getEntries,
};

/*****************************************************************************/
/* osl_openDefaultRegistry */
/*****************************************************************************/
DaemonRegistryApi* SAL_CALL osl_openDefaultRegistry()
{
	sal_Char* profileMask = "?$";

	if (hGlobalProfile == NULL)
	{
		rtl_uString* strEnvVar = NULL;
		rtl_uString* strEnvValue = NULL;
		rtl_uString* strProfileName = NULL;
		rtl_uString* strProfileMask = NULL;

		rtl_uString_newFromAscii(&strEnvVar, "OSL_DAEMON_DEBUG_ENV");
		if (osl_getEnvironment(strEnvVar, &strEnvValue) == osl_Process_E_None)
		{
			rtl_uString *strTmp = NULL;
			rtl_uString_newFromAscii(&strTmp, "user");
			if (rtl_ustr_compare(strTmp->buffer, strEnvValue->buffer) == 0)
				profileMask = "?~";

			rtl_uString_release(strTmp);
		}
		rtl_uString_release(strEnvVar);
		rtl_uString_release(strEnvValue);

		/* open profile */
		rtl_uString_newFromAscii(&strProfileMask, profileMask);
		if (osl_getProfileName(strProfileMask, NULL, &strProfileName))
		{
			hGlobalProfile = osl_openProfile(strProfileName, osl_Profile_DEFAULT);
		}

        if ( strProfileName != 0 )
        {
            
            rtl_uString_release(strProfileName);
        }
        if ( strProfileMask != 0 )
        {    
            rtl_uString_release(strProfileMask);
        }
        
	}

	if (hGlobalProfile == NULL)
	{
		return(NULL);
	}
	else
	{
		return(&globalApi);
	}
}

/*****************************************************************************/
/* osl_closeDefaultRegistry */
/*****************************************************************************/
void SAL_CALL osl_closeDefaultRegistry()
{
	if (hGlobalProfile == NULL)
	{
		osl_closeProfile(hGlobalProfile);
		hGlobalProfile = NULL;
	}
}	

