/*************************************************************************
 *
 *  $RCSfile: isocket.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _RTL_ALLOC_H_
#include <rtl/alloc.h>
#endif
#ifndef _OSL_ISOCKET_H_
#include <osl/isocket.h>
#endif

/*========================================================================
 *
 * SocketLayer internals.
 *
 *======================================================================*/
#ifndef SAL_ALLOC
#define SAL_ALLOC(T) ((T*)(rtl_allocateZeroMemory(sizeof(T))))
#endif
#ifndef SAL_FREE
#define SAL_FREE(T, p) rtl_freeZeroMemory((p), sizeof(T))
#endif

/*========================================================================
 *
 * SocketLayer implementation.
 *
 *======================================================================*/
/*
 * osl_socket_createSocketLayer.
 */
oslSocketDescriptor* SAL_CALL osl_socket_createSocketLayer (
	const oslSocketMethods *methods)
{
	oslSocketDescriptor *socket = NULL;
	if (methods)
	{
		socket = SAL_ALLOC(oslSocketDescriptor);
		socket->m_methods = methods;
	}
	return (socket);
}

/*
 * osl_socket_pushSocketLayer.
 */
void SAL_CALL osl_socket_pushSocketLayer (
	oslSocketDescriptor *socket,
	oslSocketDescriptor *stack)
{
	if (stack)
	{
		if (stack->m_upper)
			stack->m_upper->m_lower = socket;
		stack->m_upper  = socket;
	}
	if (socket)
	{
		if (socket->m_lower)
			socket->m_lower->m_upper = stack;
		socket->m_lower = stack;
	}
}

/*
 * osl_socket_popSocketLayer.
 */
void SAL_CALL osl_socket_popSocketLayer (
	oslSocketDescriptor *socket)
{
	if (socket)
	{
		if (socket->m_upper)
			socket->m_upper->m_lower = socket->m_lower;
		if (socket->m_lower)
			socket->m_lower->m_upper = socket->m_upper;
		socket->m_upper = NULL;
		socket->m_lower = NULL;
	}
}

/*
 * osl_socket_deleteSocketLayer.
 */
void SAL_CALL osl_socket_deleteSocketLayer (
	oslSocketDescriptor *socket)
{
	if (socket)
	{
		osl_socket_popSocketLayer (socket);
		SAL_FREE (oslSocketDescriptor, socket);
	}
}

/*========================================================================
 *
 * public layer (former osl/socket.c)
 * indirection to current (top) layer.
 *
 *======================================================================*/
/*
 * osl_socket_create.
 */
oslSocketResult SAL_CALL osl_socket_create (
	oslSocketDescriptor *socket,
	oslAddrFamily        family,
	oslSocketType        type,
	oslProtocol          protocol)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_create)(socket, family, type, protocol);
}

/*
 * osl_socket_delete.
 */
void SAL_CALL osl_socket_delete (oslSocketDescriptor *socket)
{
	if (!socket)
		return;

	(socket->m_methods->m_delete)(socket);
}

/*
 * osl_socket_getLastError.
 */
oslSocketError SAL_CALL osl_socket_getLastError (oslSocketDescriptor *socket)
{
	if (!socket)
		return (osl_Socket_E_NotSocket);

	return (socket->m_methods->m_getlasterror)(socket);
}

/*
 * osl_socket_getOption.
 */
oslSocketResult SAL_CALL osl_socket_getOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	void                 *buffer,
	sal_uInt32            length)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_getoption)(
		socket, level, option, buffer, length);
}

/*
 * osl_socket_setOption.
 */
oslSocketResult SAL_CALL osl_socket_setOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	const void           *buffer,
	sal_uInt32            length)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_setoption)(
		socket, level, option, buffer, length);
}

/*
 * osl_socket_accept.
 */
oslSocketDescriptor* SAL_CALL osl_socket_accept (
	oslSocketDescriptor *socket,
	oslSocketAddr       *fromaddr)
{
	if (!socket)
		return (NULL);

	return (socket->m_methods->m_accept)(socket, fromaddr);
}

/*
 * osl_socket_accept_handshake.
 */
oslSocketResult SAL_CALL osl_socket_accept_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        fromaddr)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_accept_handshake)(socket, fromaddr);
}

/*
 * osl_socket_bind.
 */
oslSocketResult SAL_CALL osl_socket_bind (
	oslSocketDescriptor *socket,
	oslSocketAddr        addr)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_bind)(socket, addr);
}

/*
 * osl_socket_connect.
 */
oslSocketResult SAL_CALL osl_socket_connect (
	oslSocketDescriptor *socket,
	oslSocketAddr        toaddr)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_connect)(socket, toaddr);
}

/*
 * osl_socket_connect_handshake.
 */
oslSocketResult SAL_CALL osl_socket_connect_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        toaddr)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_connect_handshake)(socket, toaddr);
}

/*
 * osl_socket_close.
 */
void SAL_CALL osl_socket_close (oslSocketDescriptor *socket)
{
	if (!socket)
		return;

	(socket->m_methods->m_close)(socket);
}

/*
 * osl_socket_listen.
 */
oslSocketResult SAL_CALL osl_socket_listen (
	oslSocketDescriptor *socket,
	sal_Int32            pending)
{
	if (!socket)
		return (osl_Socket_Error);

	return (socket->m_methods->m_listen)(socket, pending);
}

/*
 * osl_socket_recv.
 */
sal_Int32 SAL_CALL osl_socket_recv (
	oslSocketDescriptor *socket,
	void                *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag)
{
	if (!socket)
		return (-1);

	return (socket->m_methods->m_recv)(socket, buffer, length, flag);
}

/*
 * osl_socket_send.
 */
sal_Int32 SAL_CALL osl_socket_send (
	oslSocketDescriptor *socket,
	const void          *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag)
{
	if (!socket)
		return (-1);

	return (socket->m_methods->m_send)(socket, buffer, length, flag);
}

/*
 * osl_socket_shutdown.
 */
void SAL_CALL osl_socket_shutdown (
	oslSocketDescriptor *socket,
	oslSocketDirection   direction)
{
	if (!socket)
		return;

	(socket->m_methods->m_shutdown)(socket, direction);
}

/*========================================================================
 *
 * default, top layer.
 * indirection to lower layer.
 *
 *======================================================================*/
/*
 * __osl_default_create.
 */
static oslSocketResult SAL_CALL __osl_default_create (
	oslSocketDescriptor *socket,
	oslAddrFamily        family,
	oslSocketType        type,
	oslProtocol          protocol)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_create)(
		socket->m_lower, family, type, protocol);
}

/*
 * __osl_default_delete.
 */
static void SAL_CALL __osl_default_delete (oslSocketDescriptor *socket)
{
	if (!(socket && socket->m_lower))
		return;

	(socket->m_lower->m_methods->m_delete)(socket->m_lower);
	osl_socket_deleteSocketLayer (socket->m_lower);
}

/*
 * __osl_default_getLastError.
 */
static oslSocketError SAL_CALL __osl_default_getLastError (
	oslSocketDescriptor *socket)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_E_NotSocket);

	return (socket->m_lower->m_methods->m_getlasterror)(socket->m_lower);
}

/*
 * __osl_default_getOption.
 */
static oslSocketResult SAL_CALL __osl_default_getOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	void                 *buffer,
	sal_uInt32            length)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	if (level == osl_Socket_LevelSocketLayer)
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_getoption)(
		socket->m_lower, level, option, buffer, length);
}

/*
 * __osl_default_setOption.
 */
static oslSocketResult SAL_CALL __osl_default_setOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	const void           *buffer,
	sal_uInt32            length)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	if (level == osl_Socket_LevelSocketLayer)
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_setoption)(
		socket->m_lower, level, option, buffer, length);
}

/*
 * __osl_default_accept.
 */
static oslSocketDescriptor* SAL_CALL __osl_default_accept (
	oslSocketDescriptor *socket,
	oslSocketAddr       *fromaddr)
{
	oslSocketDescriptor *lower, *upper;

	if (!(socket && socket->m_lower))
		return (NULL);

	lower = (socket->m_lower->m_methods->m_accept)(socket->m_lower, fromaddr);
	if (!lower)
		return (NULL);

	upper = osl_socket_createSocketLayer (socket->m_methods);
	osl_socket_pushSocketLayer (upper, lower);
	return (upper);
}

/*
 * __osl_default_accept_handshake.
 */
static oslSocketResult SAL_CALL __osl_default_accept_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        fromaddr)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_accept_handshake)(
		socket->m_lower, fromaddr);
}

/*
 * __osl_default_bind.
 */
static oslSocketResult SAL_CALL __osl_default_bind (
	oslSocketDescriptor *socket,
	oslSocketAddr        addr)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_bind)(socket->m_lower, addr);
}

/*
 * __osl_default_connect.
 */
static oslSocketResult SAL_CALL __osl_default_connect (
	oslSocketDescriptor *socket,
	oslSocketAddr        toaddr)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_connect)(
		socket->m_lower, toaddr);
}

/*
 * __osl_default_connect_handshake.
 */
static oslSocketResult SAL_CALL __osl_default_connect_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        toaddr)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_connect_handshake)(
		socket->m_lower, toaddr);
}

/*
 * __osl_default_close.
 */
static void SAL_CALL __osl_default_close (oslSocketDescriptor *socket)
{
	if (!(socket && socket->m_lower))
		return;

	(socket->m_lower->m_methods->m_close)(socket->m_lower);
}

/*
 * __osl_default_listen.
 */
static oslSocketResult SAL_CALL __osl_default_listen (
	oslSocketDescriptor *socket,
	sal_Int32            pending)
{
	if (!(socket && socket->m_lower))
		return (osl_Socket_Error);

	return (socket->m_lower->m_methods->m_listen)(socket->m_lower, pending);
}

/*
 * __osl_default_recv.
 */
static sal_Int32 SAL_CALL __osl_default_recv (
	oslSocketDescriptor *socket,
	void                *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag)
{
	if (!(socket && socket->m_lower))
		return (-1);

	return (socket->m_lower->m_methods->m_recv) (
		socket->m_lower, buffer, length, flag);
}

/*
 * __osl_default_send.
 */
static sal_Int32 SAL_CALL __osl_default_send (
	oslSocketDescriptor *socket,
	const void          *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag)
{
	if (!(socket && socket->m_lower))
		return (-1);

	return (socket->m_lower->m_methods->m_send) (
		socket->m_lower, buffer, length, flag);
}

/*
 * __osl_default_shutdown.
 */
static void SAL_CALL __osl_default_shutdown (
	oslSocketDescriptor *socket,
	oslSocketDirection   direction)
{
	if (!(socket && socket->m_lower))
		return;

	(socket->m_lower->m_methods->m_shutdown)(socket->m_lower, direction);
}

/*
 * __osl_default_methods.
 */
static const oslSocketMethods __osl_default_methods =
{
	__osl_default_create,
	__osl_default_delete,

	__osl_default_getLastError,

	__osl_default_getOption,
	__osl_default_setOption,

	__osl_default_accept,
	__osl_default_accept_handshake,

	__osl_default_bind,

	__osl_default_connect,
	__osl_default_connect_handshake,

	__osl_default_close,
	__osl_default_listen,

	__osl_default_recv,
	__osl_default_send,

	__osl_default_shutdown
};

/*
 * osl_socket_getDefaultSocketMethods.
 */
const oslSocketMethods* SAL_CALL osl_socket_getDefaultSocketMethods (void)
{
	return (&__osl_default_methods);
}

/*========================================================================
 *
 * physical, bottom layer (former osl/socket.c)
 *
 *======================================================================*/
/*
 * __osl_socket_create.
 */
static oslSocketResult SAL_CALL __osl_socket_create (
	oslSocketDescriptor *socket,
	oslAddrFamily        family,
	oslSocketType        type,
	oslProtocol          protocol)
{
	if (!socket)
		return (osl_Socket_Error);

	if (!(socket->m_private))
		socket->m_private = osl_createSocket (family, type, protocol);

	if (!(socket->m_private))
		return (osl_Socket_Error);

	return (osl_Socket_Ok);
}

/*
 * __osl_socket_delete.
 */
static void SAL_CALL __osl_socket_delete (oslSocketDescriptor *socket)
{
	if (socket)
	{
		osl_destroySocket ((oslSocket)(socket->m_private));
		socket->m_private = NULL;
	}
}

/*
 * __osl_socket_getLastError.
 */
static oslSocketError SAL_CALL __osl_socket_getLastError (
	oslSocketDescriptor *socket)
{
	if (!(socket && socket->m_private))
		return (osl_Socket_E_NotSocket);

	return osl_getLastSocketError ((oslSocket)(socket->m_private));
}

/*
 * __osl_socket_getOption.
 */
static oslSocketResult SAL_CALL __osl_socket_getOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	void                 *buffer,
	sal_uInt32            length)
{
	oslSocket hSocket;

	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	if (level == osl_Socket_LevelSocketLayer)
		return (osl_Socket_Error);

	hSocket = (oslSocket)(socket->m_private);
	if (osl_getSocketOption (hSocket, level, option, buffer, length))
		return (osl_Socket_Ok);
	else
		return (osl_Socket_Error);
}

/*
 * __osl_socket_setOption.
 */
static oslSocketResult SAL_CALL __osl_socket_setOption (
	oslSocketDescriptor  *socket,
	oslSocketOptionLevel  level,
	oslSocketOption       option,
	const void           *buffer,
	sal_uInt32            length)
{
	oslSocket hSocket;

	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	if (level == osl_Socket_LevelSocketLayer)
		return (osl_Socket_Error);

	hSocket = (oslSocket)(socket->m_private);
	if (osl_setSocketOption (hSocket, level, option, (void*)buffer, length))
		return (osl_Socket_Ok);
	else
		return (osl_Socket_Error);
}

/*
 * __osl_socket_accept.
 */
static oslSocketDescriptor* SAL_CALL __osl_socket_accept (
    oslSocketDescriptor *socket,
    oslSocketAddr       *fromaddr)
{
	oslSocket hSocket, hAccept;

	if (!(socket && socket->m_private))
		return (NULL);

	hSocket = (oslSocket)(socket->m_private);
	hAccept = osl_acceptConnectionOnSocket (hSocket, fromaddr);
	if (hAccept)
	{
		oslSocketDescriptor *result;
		result = osl_socket_createSocketLayer (socket->m_methods);
		if (result)
			result->m_private = hAccept;
		return (result);
	}
	return (NULL);
}

/*
 * __osl_socket_accept_handshake.
 */
static oslSocketResult SAL_CALL __osl_socket_accept_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        fromaddr)
{
	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	return (osl_Socket_Ok);
}

/*
 * __osl_socket_bind.
 */
static oslSocketResult SAL_CALL __osl_socket_bind (
	oslSocketDescriptor *socket,
	oslSocketAddr        addr)
{
	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	if (osl_bindAddrToSocket ((oslSocket)(socket->m_private), addr))
		return (osl_Socket_Ok);
	else
		return (osl_Socket_Error);
}

/*
 * __osl_socket_connect.
 */
static oslSocketResult SAL_CALL __osl_socket_connect (
	oslSocketDescriptor *socket,
	oslSocketAddr        addr)
{
	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	return osl_connectSocketTo (
		(oslSocket)(socket->m_private), addr, NULL);
}

/*
 * __osl_socket_connect_handshake.
 */
static oslSocketResult SAL_CALL __osl_socket_connect_handshake (
	oslSocketDescriptor *socket,
	oslSocketAddr        addr)
{
	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	return (osl_Socket_Ok);
}

/*
 * __osl_socket_close.
 */
static void SAL_CALL __osl_socket_close (
	oslSocketDescriptor *socket)
{
	if (!(socket && socket->m_private))
		return;

	osl_closeSocket ((oslSocket)(socket->m_private));
}

/*
 * __osl_socket_listen.
 */
static oslSocketResult SAL_CALL __osl_socket_listen (
	oslSocketDescriptor *socket,
	sal_Int32            pending)
{
	if (!(socket && socket->m_private))
		return (osl_Socket_Error);

	if (osl_listenOnSocket ((oslSocket)(socket->m_private), pending))
		return (osl_Socket_Ok);
	else
		return (osl_Socket_Error);
}

/*
 * __osl_socket_recv.
 */
static sal_Int32 SAL_CALL __osl_socket_recv (
	oslSocketDescriptor *socket,
	void                *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag)
{
	if (!(socket && socket->m_private))
		return (-1);

	return osl_receiveSocket (
		(oslSocket)(socket->m_private), buffer, length, flag);
}

/*
 * __osl_socket_send.
 */
static sal_Int32 SAL_CALL __osl_socket_send (
	oslSocketDescriptor *socket,
	const void          *buffer,
	sal_uInt32           length,
	oslSocketMsgFlag     flag)
{
	if (!(socket && socket->m_private))
		return (-1);

	return osl_sendSocket (
		(oslSocket)(socket->m_private), buffer, length, flag);
}

/*
 * __osl_socket_shutdown.
 */
static void SAL_CALL __osl_socket_shutdown (
    oslSocketDescriptor *socket,
    oslSocketDirection   direction)
{
	if (!(socket && socket->m_private))
		return;

	osl_shutdownSocket ((oslSocket)(socket->m_private), direction);
}

/*
 * __osl_socket_methods.
 */
static const oslSocketMethods __osl_socket_methods =
{
	__osl_socket_create,
	__osl_socket_delete,

	__osl_socket_getLastError,

	__osl_socket_getOption,
	__osl_socket_setOption,

	__osl_socket_accept,
	__osl_socket_accept_handshake,

	__osl_socket_bind,

	__osl_socket_connect,
	__osl_socket_connect_handshake,

	__osl_socket_close,
	__osl_socket_listen,

	__osl_socket_recv,
	__osl_socket_send,

	__osl_socket_shutdown
};

/*
 * osl_socket_getSocketMethods.
 */
const oslSocketMethods* SAL_CALL osl_socket_getSocketMethods (void)
{
	return (&__osl_socket_methods);
}

