/*************************************************************************
 *
 *  $RCSfile: channel.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:20 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_CHANNEL_H_
#include <osl/channel.h>
#endif

#include <osl/diagnose.h>
#include "../all/channman.h"
#include "system.h"
#include "sockimpl.h"
#include "sockimpl.h"


/*****************************************************************************/
/* Data Type Definition */
/*****************************************************************************/

typedef struct _oslPipeImpl
{
    int       m_Socket;
    sal_Char  m_Name[PATH_MAX + 1];
} oslPipeImpl;


typedef struct
{
    int       fd;
    sal_Char  name[PATH_MAX + 1];
} Pipe;



/*****************************************************************************/
/* Functions of other Modules */
/*****************************************************************************/

oslPipeImpl* __osl_createPipeImpl();
oslSocketImpl* __osl_createSocketImpl(int Socket);


/*****************************************************************************/
/* Static Module Functions Declarations */
/*****************************************************************************/

static void makePipeName(sal_Char* buffer, const sal_Char* pipeName);
static ssize_t writen(int fd, const void *vptr, size_t n);
static ssize_t readn(int fd, void *vptr, size_t n);
static oslResourcePipe SAL_CALL listenOnResourcePipe(rtl_uString* ustrName);
static oslResourcePipe SAL_CALL acceptOnResourcePipe(oslResourcePipe hPipe);
static oslResourcePipe SAL_CALL connectToResourcePipe(rtl_uString* ustrName);
static void SAL_CALL closeResourcePipe(oslResourcePipe hPipe);
static sal_Int32 SAL_CALL sendBuffer(oslResourcePipe hPipe, const void* pBuffer, sal_uInt32 BufferSize);
static sal_Int32 SAL_CALL receiveBuffer(oslResourcePipe hPipe,void* pBuffer,sal_uInt32 BufferSize);
static sal_Bool SAL_CALL sendResource(
    oslResourcePipe hPipe,
    oslChannelResource Resource,
    oslChannelType Type,
    sal_Bool bAmITheBroker);
static sal_Bool SAL_CALL receiveResource(
    oslResourcePipe hPipe,
    oslChannelResource* pResource,
    oslChannelType* pType,
    sal_Bool bAmITheBroker);


/*****************************************************************************/
/* Static Module Functions */
/*****************************************************************************/


#define PIPEDEFAULTPATH             "/tmp" 
#define PIPEALTERNATEPATH           "/var/tmp"
#define PIPENAMEMASK                "OSL_IOT_%s"

#if !defined(IOCHANNEL_TRANSFER_BSD)
#define CONTROLLEN (sizeof(struct cmsghdr) + sizeof(int))
#endif

#define OSL_CHANNEL_MANAGER_TYPE_NAME "OSL_CHANNEL_MANAGER"


#define MAGIC_NUMBER                0xFD2FD2FD

#define MESSAGE_TYPE_RAW_DATA       0x00000001
#define MESSAGE_TYPE_FD             0x00000002
#define MESSAGE_FD_TYPE_SOCKET      0x00000010
#define MESSAGE_FD_TYPE_PIPE        0x00000020


static ResourcePipeApi globalResourcePipeApi =
{
	listenOnResourcePipe,
	acceptOnResourcePipe,
	connectToResourcePipe,
	closeResourcePipe,
	sendBuffer,
	receiveBuffer,
	sendResource,
	receiveResource
};


/*****************************************************************************/
/* Exported Module Functions */
/*****************************************************************************/

sal_Bool SAL_CALL osl_startChannelBroker()
{
    rtl_uString* ustrChannel=0;
    sal_Bool bRet = sal_False;
    
    rtl_uString_newFromAscii(&ustrChannel,OSL_CHANNEL_MANAGER_TYPE_NAME);

    bRet=osl_default_startChannelBroker(&globalResourcePipeApi, ustrChannel);

    rtl_uString_release(ustrChannel);
    
	return bRet;
}	

sal_Bool SAL_CALL osl_stopChannelBroker()
{
	return(osl_default_stopChannelBroker());
}	

oslChannelBroker SAL_CALL osl_channelConnectBroker()
{
    rtl_uString* ustrChannel=0;
    oslChannelBroker pChannel = sal_False;
    
    rtl_uString_newFromAscii(&ustrChannel,OSL_CHANNEL_MANAGER_TYPE_NAME);

    pChannel = osl_default_channelConnectBroker(&globalResourcePipeApi, ustrChannel);

    rtl_uString_release(ustrChannel);
    
    return pChannel;
}	

void SAL_CALL osl_channelDisconnectBroker(oslChannelBroker Broker)
{
	osl_default_channelDisconnectBroker(Broker);
}	

oslChannelError SAL_CALL osl_channelProvide(oslChannelBroker		Broker,
											oslChannelResource		Resource, 
										   	oslChannelType			Type,	
											rtl_uString*            ustrResourceName,
											rtl_uString*            ustrUserIdent,
											rtl_uString*            ustrPassword,
										   	const TimeValue* 		pTimeout)
{
	return(osl_default_channelProvide(Broker, Resource, Type, ustrResourceName, ustrUserIdent, ustrPassword, pTimeout));
}	

/*  oslChannelError SAL_CALL osl_psz_channelProvide */
/*  ( */
/*  	oslChannelBroker		Broker, */
/*  	oslChannelResource		Resource,  */
/*  	oslChannelType			Type,		 */
/*  	const sal_Char*			ResourceName,  */
/*  	const sal_Char*			UserIdent,  */
/*  	const sal_Char*			Password,  */
/*  	const TimeValue* 		pTimeout */
/*  ) */
/*  { */
/*  	return(osl_default_channelProvide(Broker, Resource, Type, ResourceName, UserIdent, Password, pTimeout)); */
/*  }	 */
oslChannelError SAL_CALL osl_channelAccept(oslChannelBroker			Broker,
										   oslChannelResource*		pResource, 
										   oslChannelType* 			pType,	
										   rtl_uString*    		    ustrResourceName,
										   CheckAuthentication_F	checkAuthFunc,
										   void* 					pAuthFuncData,
										   const TimeValue* 		pTimeout)
{
	return(osl_default_channelAccept(Broker, pResource, pType, ustrResourceName, checkAuthFunc, pAuthFuncData, pTimeout));
}	

/*  oslChannelError SAL_CALL osl_channelAccept */
/*  ( */
/*  	oslChannelBroker		Broker, */
/*  	oslChannelResource*		pResource,  */
/*      oslChannelType* 		pType,	 */
/*  	const sal_Char*			ResourceName,  */
/*  	CheckAuthentication_F	checkAuthFunc, */
/*  	void* 					pAuthFuncData, */
/*  	const TimeValue* 		pTimeout */
/*  ) */
/*  { */
/*  	return(osl_default_channelAccept(Broker, pResource, pType, ResourceName, checkAuthFunc, pAuthFuncData, pTimeout)); */
/*  }	 */


/*****************************************************************************/
/* Static Module Functions */
/*****************************************************************************/


/*****************************************************************************/
/* makePipeName */
/*****************************************************************************/

static void makePipeName(sal_Char* buffer, const sal_Char* pipeName)
{
	if (access(PIPEDEFAULTPATH, O_RDWR) == 0)
		strcpy(buffer, PIPEDEFAULTPATH); 
	else
		strcpy(buffer, PIPEALTERNATEPATH); 

	strcat(buffer, "/");

	sprintf(&buffer[strlen(buffer)], PIPENAMEMASK, pipeName);
}	

/* Write "n" bytes to a descriptor. */
static ssize_t writen(int fd, const void *vptr, size_t n)
{
	ssize_t		nleft, nwritten;
	const char	*ptr;

	ptr = (char*) vptr;	/* can't do pointer arithmetic on void* */
	nleft = n;
	while (nleft > 0) {
/*		if ( (nwritten = write(fd, ptr, nleft)) <= 0)*/
		if ( (nwritten = send(fd, ptr, nleft, 0)) <= 0)
			return(nwritten);		/* error */

		nleft -= nwritten;
		ptr   += nwritten;
	}
	return(n);
}

/* Read "n" bytes from a descriptor. */
static ssize_t readn(int fd, void *vptr, size_t n)
{
	ssize_t	nleft, nread;
	char	*ptr;

	ptr = (char*) vptr;
	nleft = n;
	while (nleft > 0) {
/*		if ( (nread = read(fd, ptr, nleft)) < 0)*/
		if ( (nread = recv(fd, ptr, nleft, 0)) < 0)
			return(nread);		/* error, return < 0 */
		else if (nread == 0)
			break;				/* EOF */

		nleft -= nread;
		ptr   += nread;
	}
	return(n - nleft);		/* return >= 0 */
}

/* Implementation resource pipe access function */
/* see: W. R. Stevens, Advanced Programming in the UNIX Environment, ADDISON-WESLEY, 1993 */

static oslResourcePipe SAL_CALL listenOnResourcePipe(rtl_uString* ustrPipeName)
{
	Pipe* pPipeImpl = NULL;
    rtl_String* strPipeName=0;
    sal_Char* pszPipeName=0;
    
#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	pPipeImpl = (Pipe*) malloc(sizeof(Pipe));

    if ( ustrPipeName != 0 )
    {    
        rtl_uString2String( &strPipeName,
                            rtl_uString_getStr(ustrPipeName),
                            rtl_uString_getLength(ustrPipeName),
                            RTL_TEXTENCODING_UTF8,
                            OUSTRING_TO_OSTRING_CVTFLAGS );

        pszPipeName = rtl_string_getStr(strPipeName);
    }
    
    makePipeName(pPipeImpl->name, pszPipeName);

    rtl_string_release(strPipeName);
    
	if ( (pPipeImpl->fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0 )
	{
  		OSL_TRACE("listenOnResourcePipe : can't create socket. Errno: %d; %s\n",errno,strerror(errno));
	}
	
	if (pPipeImpl->fd != -1)
	{
		int				   	len;
		struct sockaddr_un 	addr;
		struct stat 		status;
		int msk;
		
		memset(&addr, 0, sizeof(addr));
	
		addr.sun_family = AF_UNIX;
		strcpy(addr.sun_path, pPipeImpl->name);
		len = sizeof(addr.sun_family) + strlen(addr.sun_path);

			/* check if there exists an orphan filesystem entry */
		if ( (stat(pPipeImpl->name, &status) == 0) && 
			 (S_ISSOCK(status.st_mode) || S_ISFIFO(status.st_mode) ) )
		{
			if ( connect(pPipeImpl->fd,(struct sockaddr *)&addr,len) >= 0  )
			{
				OSL_TRACE("listenOnResourcePipe : Pipe already in use. Errno: %d; %s\n",errno,strerror(errno));
				close(pPipeImpl->fd);
				free(pPipeImpl);
				pPipeImpl = NULL;
				return NULL;
			}
		}
		
		unlink(pPipeImpl->name);

			/* try to create a named file a base for the pipe */
		msk = umask(0);	
		if (bind(pPipeImpl->fd, (struct sockaddr *)&addr, len) >= 0)
		{
			if (listen(pPipeImpl->fd, 5) < 0)
			{
				OSL_TRACE("listenOnResourcePipe : can't listen on socket. Errno: %d; %s\n",errno,strerror(errno));
				pPipeImpl->fd = -1;					
				unlink(pPipeImpl->name);	/* remove filesystem entry */
			}			
		}
		else
		{
			OSL_TRACE("listenOnResourcePipe : can't bind socket. Errno: %d; %s\n",errno,strerror(errno));
		}
		umask(msk);
	}

	if (pPipeImpl->fd < 0)
	{
		free(pPipeImpl);
		pPipeImpl = NULL;
	}

#endif

	return((oslResourcePipe)pPipeImpl);
}	

static oslResourcePipe SAL_CALL acceptOnResourcePipe(oslResourcePipe hPipe)
{
	Pipe* pPipeImpl = NULL;

#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	Pipe* pListenPipeImpl = (Pipe*)hPipe;

	if ((pListenPipeImpl != NULL) && (pListenPipeImpl->fd != 0))
	{
		pPipeImpl = (Pipe*) malloc(sizeof(Pipe));
		pPipeImpl->name[0] = '\0';
		pPipeImpl->fd = accept((int)pListenPipeImpl->fd, 0, 0);

		if (pPipeImpl->fd < 0)
		{
			free(pPipeImpl);
			pPipeImpl = NULL;
		}
	}
	
#endif

	return((oslResourcePipe)pPipeImpl);
}

static oslResourcePipe SAL_CALL connectToResourcePipe(rtl_uString* ustrPipeName)
{
	Pipe* pPipeImpl = NULL;

#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	int 				len;
	struct sockaddr_un	addr;

	pPipeImpl = (Pipe*) malloc(sizeof(Pipe));
	pPipeImpl->name[0] = '\0';
	if ((pPipeImpl->fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0 )
	{
  		OSL_TRACE("socket failed errno == %i , '%s'\n",errno,strerror(errno));
	}

	if (pPipeImpl->fd >= 0)
	{
        rtl_String* strPipeName=0;
		sal_Char tmpName[PATH_MAX + 1] = "";
        sal_Char* pszPipeName=0;

        if ( ustrPipeName != 0 )
        {
            rtl_uString2String( &strPipeName,
                                rtl_uString_getStr(ustrPipeName),
                                rtl_uString_getLength(ustrPipeName),
                                RTL_TEXTENCODING_UTF8,
                                OUSTRING_TO_OSTRING_CVTFLAGS );
            pszPipeName = rtl_string_getStr(strPipeName);
        }
        
        
		makePipeName(tmpName, pszPipeName);

        rtl_string_release(strPipeName);
        
		memset(&addr, 0, sizeof(addr));

		addr.sun_family = AF_UNIX;
		strcpy(addr.sun_path, tmpName);
		len = sizeof(addr.sun_family) + strlen(addr.sun_path);

		/* connect to socket */
		if ( connect(pPipeImpl->fd, (struct sockaddr *)&addr, len) < 0 )
		{
 			OSL_TRACE("connect failed errno == %i , '%s'\n",errno,strerror(errno));
			pPipeImpl->fd = -1;
		}
	}

	if (pPipeImpl->fd < 0)
	{
		free(pPipeImpl);
		pPipeImpl = NULL;
	}

#endif

	return((oslResourcePipe)pPipeImpl);
}	

static void SAL_CALL closeResourcePipe(oslResourcePipe hPipe)
{
#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	Pipe* pPipeImpl = (Pipe*)hPipe;

	if (pPipeImpl != NULL)
	{
		if (pPipeImpl->fd >= 0)
			close(pPipeImpl->fd);

		if (pPipeImpl->name[0] != '\0')
			unlink(pPipeImpl->name);

		free(pPipeImpl);
	}

#endif
}	

static sal_Int32 SAL_CALL sendBuffer(oslResourcePipe hPipe, const void* pBuffer, sal_uInt32 BufferSize)
{
	sal_Int32 bytesSend = 0;

#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	Pipe* pPipeImpl = (Pipe*)hPipe;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd >= 0))
	{
		sal_uInt32 code = MESSAGE_TYPE_RAW_DATA;

		if (
			(writen(pPipeImpl->fd, &code, sizeof(code)) == sizeof(code)) &&
			(writen(pPipeImpl->fd, &BufferSize, sizeof(BufferSize)) == sizeof(BufferSize))
		   )
			bytesSend = writen(pPipeImpl->fd, pBuffer, BufferSize);
	}

#endif

	return(bytesSend);
}	

static sal_Int32 SAL_CALL receiveBuffer
(
	oslResourcePipe hPipe, 
	void* pBuffer, 
	sal_uInt32 BufferSize
)
{
	sal_Int32 bytesRead = 0;

#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	Pipe* pPipeImpl = (Pipe*)hPipe;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd >= 0))
	{
		sal_uInt32 code = 0;
		sal_uInt32 size = 0;

		if (
			(readn(pPipeImpl->fd, &code, sizeof(code)) == sizeof(code)) &&
			(code & MESSAGE_TYPE_RAW_DATA) &&
			(readn(pPipeImpl->fd, &size, sizeof(size)) == sizeof(size)) &&
			(size > 0) &&
			(size <= BufferSize)
		   )
		{
			bytesRead = readn(pPipeImpl->fd, pBuffer, size);
		}
	}

#endif

	return(bytesRead);
}	

static sal_Bool SAL_CALL sendResource
(
	oslResourcePipe hPipe, 
	oslChannelResource Resource, 
	oslChannelType Type,
	sal_Bool bAmITheBroker
)
{
	sal_Bool bRet = sal_False;

#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	Pipe* pPipeImpl = (Pipe*)hPipe;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd >= 0))
	{
		int fd = -1;
		sal_uInt32 code = MESSAGE_TYPE_FD;

		switch (Type)
		{
			case osl_Channel_Type_Socket:
				fd = ((oslSocketImpl *)(Resource.Socket))->m_Socket;
				code |= MESSAGE_FD_TYPE_SOCKET;
				break;
			case osl_Channel_Type_Pipe:
				fd = ((oslPipeImpl *)(Resource.Pipe))->m_Socket;
				code |= MESSAGE_FD_TYPE_PIPE;
				break;
			default:
				break;
		}

		if (writen(pPipeImpl->fd, &code, sizeof(code)) == sizeof(code))
		{
			sal_Int32 commitCode;
			struct iovec	iov[1];
			struct msghdr	msg;
			char			buf[2];	/* send_fd()/recv_fd() 2-byte protocol */
#if defined(IOCHANNEL_TRANSFER_BSD)

			iov[0].iov_base = buf;
			iov[0].iov_len  = 2;
			msg.msg_iov     = iov;
			msg.msg_iovlen  = 1;
			msg.msg_name    = NULL;
			msg.msg_namelen = 0;

			msg.msg_accrights    = (caddr_t) &fd;	/* addr of descriptor */
			msg.msg_accrightslen = sizeof(int);		/* pass 1 descriptor */
			buf[1] = 0;								/* zero status means OK */
			buf[0] = 0;								/* null byte flag to recv_fd() */

			if (sendmsg(pPipeImpl->fd, &msg, 0) >= 0)
			{
				bRet = sal_True;
			}
#else
			struct cmsghdr* cmptr = (struct cmsghdr*)malloc(CONTROLLEN);

			iov[0].iov_base = buf;
			iov[0].iov_len = 2;
			msg.msg_iov = iov;
			msg.msg_iovlen = 1;
			msg.msg_name = NULL;
			msg.msg_namelen = 0;
			msg.msg_control = (caddr_t) cmptr;
			msg.msg_controllen = CONTROLLEN;

			cmptr->cmsg_level = SOL_SOCKET;
			cmptr->cmsg_type = SCM_RIGHTS;
			cmptr->cmsg_len = CONTROLLEN;
			*(int*)CMSG_DATA(cmptr) = fd;

			if (sendmsg(pPipeImpl->fd, &msg, 0) >= 0)
			{
				bRet = sal_True;
			}

			free(cmptr);
#endif

			if (
				(readn(pPipeImpl->fd, &commitCode, sizeof(commitCode)) != sizeof(commitCode)) ||
				(commitCode <= 0) 
			   )
			{
				bRet = sal_False;
			}
		}
	}

#endif
	
	return(bRet);
}	

static sal_Bool SAL_CALL receiveResource(
	oslResourcePipe hPipe,
	oslChannelResource* pResource,
	oslChannelType* pType,
	sal_Bool bAmITheBroker
)
{
	sal_Bool bRet = sal_False;

#if defined(IOCHANNEL_TRANSFER_BSD) || defined(IOCHANNEL_TRANSFER_BSD_RENO)

	Pipe* pPipeImpl = (Pipe*)hPipe;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd >= 0))
	{
		sal_uInt32 code = 0;

		if (
			(readn(pPipeImpl->fd, &code, sizeof(code)) == sizeof(code)) &&
			(code & MESSAGE_TYPE_FD) 
		   )
		{
			sal_Int32 commitCode;
			struct msghdr msghdr;
			struct iovec iov[1];
			char buffer[PATH_MAX];

#if defined(IOCHANNEL_TRANSFER_BSD)
			char *ptr;
			sal_Int32 nread, status;
			int newfd;

			iov[0].iov_base = buffer;
			iov[0].iov_len = sizeof(buffer);
			msghdr.msg_name = NULL;
			msghdr.msg_namelen = 0;
			msghdr.msg_iov = iov;
			msghdr.msg_iovlen = 1;
			msghdr.msg_accrights = (caddr_t) &newfd;/* addr of descriptor */
			msghdr.msg_accrightslen = sizeof(int);	 /* receive 1 descriptor */

			if ((nread = recvmsg(pPipeImpl->fd, &msghdr, 0)) > 0)
			{
				if (newfd >= 0)
				{
					if (code & MESSAGE_FD_TYPE_SOCKET)
					{
				    	pResource->Socket = __osl_createSocketImpl(newfd);
						*pType = osl_Channel_Type_Socket;
						bRet = sal_True;
					}
					else if (code & MESSAGE_FD_TYPE_PIPE)
					{
						oslPipeImpl * pPipeImpl = __osl_createPipeImpl();
				    	pPipeImpl->m_Socket = newfd;
				    	pResource->Pipe = pPipeImpl;

						*pType = osl_Channel_Type_Pipe;
						bRet = sal_True;
					}
				}
			}

#else
			struct cmsghdr* cmptr = (struct cmsghdr*)malloc(CONTROLLEN);
			int newfd;

			iov[0].iov_base = buffer;
			iov[0].iov_len = sizeof(buffer);
			msghdr.msg_name = NULL;
			msghdr.msg_namelen = 0;
			msghdr.msg_iov = iov;
			msghdr.msg_iovlen = 1;

			msghdr.msg_control = (caddr_t) cmptr;
			msghdr.msg_controllen = CONTROLLEN;

			if (
				(recvmsg(pPipeImpl->fd, &msghdr, 0) >= 0) &&
				(msghdr.msg_controllen == CONTROLLEN)
			   )
			{
				newfd = *(int*)CMSG_DATA(cmptr);
				if (code & MESSAGE_FD_TYPE_SOCKET)
				{
			    	pResource->Socket = __osl_createSocketImpl(newfd);
					*pType = osl_Channel_Type_Socket;
					bRet = sal_True;
				}
				else if (code & MESSAGE_FD_TYPE_PIPE)
				{
					oslPipeImpl * pPipeImpl = __osl_createPipeImpl();
			    	pPipeImpl->m_Socket = newfd;
			    	pResource->Pipe = pPipeImpl;

					*pType = osl_Channel_Type_Pipe;
					bRet = sal_True;
				}
			}


			free(cmptr);
#endif

			if (bRet)
				commitCode = 1;
			else
				commitCode = 0;

			writen(pPipeImpl->fd, &commitCode, sizeof(commitCode));
		}
	}

#endif
	
	return(bRet);
}	

