#*************************************************************************
#
#   $RCSfile: sdaemon.sh,v $
#
#   $Revision: 1.1.1.1 $
#
#   last change: $Author: hr $ $Date: 2000/09/18 15:17:21 $
#
#   The Contents of this file are made available subject to the terms of
#   either of the following licenses
#
#          - GNU Lesser General Public License Version 2.1
#          - Sun Industry Standards Source License Version 1.1
#
#   Sun Microsystems Inc., October, 2000
#
#   GNU Lesser General Public License Version 2.1
#   =============================================
#   Copyright 2000 by Sun Microsystems, Inc.
#   901 San Antonio Road, Palo Alto, CA 94303, USA
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License version 2.1, as published by the Free Software Foundation.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#   MA  02111-1307  USA
#
#
#   Sun Industry Standards Source License Version 1.1
#   =================================================
#   The contents of this file are subject to the Sun Industry Standards
#   Source License Version 1.1 (the "License"); You may not use this file
#   except in compliance with the License. You may obtain a copy of the
#   License at http://www.openoffice.org/license.html.
#
#   Software provided under this License is provided on an "AS IS" basis,
#   WITHOUT WARRUNTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
#   WITHOUT LIMITATION, WARRUNTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
#   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
#   See the License for the specific provisions governing your rights and
#   obligations concerning the Software.
#
#   The Initial Developer of the Original Code is: Sun Microsystems, Inc..
#
#   Copyright: 2000 by Sun Microsystems, Inc.
#
#   All Rights Reserved.
#
#   Contributor(s): _______________________________________
#
#
#
#*************************************************************************
#!/bin/sh

# be verbose
# set -x

#
# CONSTANTS 
#

DMGR=sdaemon
DMGRRC=/etc/sdaemonrc
STARTS=S99sdaemon
KILLS=K10sdaemon
SCRIPT=sdaemon.sh

_KILL_DIRS="rc0.d rcS.d rc1.d"
_START_DIRS="rc2.d rc3.d rc5.d"
_BASE_DIRS="/etc /etc/init.d /etc/rc.d"

#
# FUNCTIONS
#
			
usage_error() 
{
	echo ""
	echo "usage:"
	echo "  $0 start   (start $DMGR process)"
	echo "  $0 stop    (stop  $DMGR process)"
	echo "  $0 reset   (reset $DMGR process)"
	echo "  $0 install   (install $DMGR binary and Start/Kill shell scripts)"
	echo "  $0 uninstall (uninstall $DMGR binary and Start/Kill shell scripts)"
	echo " "
}

find_prog() 
{
	if [ -x /usr/bin/${PROG} ]; then
		eval ${PROG}=/usr/bin/${PROG}
	else
		eval ${PROG}=/bin/${PROG}
	fi
}

#
# Find login server pid from the process tree
#
login_server_pid() 
{
	$ps -u 0 -l | $grep $DMGR | $cut -c1-24 | $awk '{print $4 " " $5}' |
  	while read pid ppid; do
    	parent_login_ps=`$ps -p $ppid | $grep $DMGR`
    	if [ -z "$parent_login_ps" ]; then
			echo "$pid"
      		break
    	fi
  	done
}

get_login_server_pid() 
{
	if [ -x /usr/bin/pgrep ]
	then
		/usr/bin/pgrep -u 0 -P 1 -x $DMGR 
	else
		login_server_pid
	fi
}

stop_sdaemon() 
{
	if [ "$1" != "" ]
	then
		# explicit kill with a signal, does not shutdown the clients
		SIGNAL=-$1

		if [ -x /usr/bin/pkill ]; then
			/usr/bin/pkill $SIGNAL -u 0 -P 1 -x $DMGR
		else
        	sdaemon_pid=`login_server_pid`
       		if [ "$sdaemon_pid" != "" ] ; then
      			$kill $SIGNAL $sdaemon_pid
       		fi
		fi
	else
		# this is the canonical way, let the sdaemon shutdown its 
		# children before exiting
		/usr/bin/$DMGR -shutdown
	fi

	$rm /tmp/OSL_*
}

#
# check the root configuration directory for init files
# usually /etc/init.d or /etc/rc.d, see _base_dirs for list
#
get_etcbasedir() 
{
	for _DIR in ${_BASE_DIRS} ; do
		if [ -d ${_DIR}/rc0.d ]; then
			_BASE_DIR=${_DIR}
			break
		fi
	done
	
	_INIT_BASE_DIR=${_BASE_DIR}
	for _DIR in ${_BASE_DIRS} ; do
		if [ -d ${_DIR}/init.d ]; then
			_INIT_BASE_DIR=${_DIR}
			break
		fi
	done
}

#
# copy this file as _target_name into some subdirectories of /etc
# i.e. /etc/_base_dir/_install_dirs
# _base_dir must be set up by get_basedir()
#
install_scripts() 
{

	if [ -d ${_INIT_BASE_DIR}/init.d ]; then
		_COPY_SOURCE="${_INIT_BASE_DIR}/init.d/${SCRIPT}"
		_LINK_SOURCE="../init.d/${SCRIPT}"
	else
		_COPY_SOURCE="${_INIT_BASE_DIR}/${SCRIPT}"
		_LINK_SOURCE="../${SCRIPT}"
	fi

	if [ ${_INIT_BASE_DIR} != ${_BASE_DIR} ]; then
		_LINK_SOURCE="${_COPY_SOURCE}"
	fi

	# copy the script to xxx/init.d/sdaemon.sh	
	echo installing $0 as ${_COPY_SOURCE}
	$cp "$0" "${_COPY_SOURCE}"
	chmod 744 "${_COPY_SOURCE}"
	
	# link the scripts from rc?.d/script to the copied version
	_CURRENT_WORKING_DIRECTORY=`pwd`
	for _DIR in  ${_INSTALL_DIRS} ; do
		if [ -d ${_BASE_DIR}/${_DIR} ]; then
			cd ${_BASE_DIR}/${_DIR}
			echo "${_BASE_DIR}/${_DIR}" ":" linking ${_LINK_SOURCE} as ${_TARGET_NAME}
			$ln "${_LINK_SOURCE}" "${_TARGET_NAME}"
		fi
	done
	cd "${_CURRENT_WORKING_DIRECTORY}"
}

#
# and vice versa, rm all the stuff we've installed by install_scripts()
# from /etc/ directories
#
uninstall_scripts() 
{
	if [ -d ${_INIT_BASE_DIR}/init.d ]; then
		_SOURCE=${_INIT_BASE_DIR}/init.d/${SCRIPT}
	else
		_SOURCE=${_INIT_BASE_DIR}/${SCRIPT}
	fi

	if [ -f ${_SOURCE} -o -h ${_SOURCE} ]; then
		echo removing ${_SOURCE}
		$rm ${_SOURCE}
	fi
	
	for _DIR in  ${_INSTALL_DIRS} ; do
		_TARGET=${_BASE_DIR}/${_DIR}/${_TARGET_NAME}
		if [ -f ${_TARGET} -o -h ${_TARGET} ]; then
			echo removing ${_TARGET}
			$rm ${_TARGET}
			if [ -f ${_TARGET} ]; then
				echo "    " FAILED
			fi
		fi
	done
}

#
# remove the sdaemon and all of its associated scripts from
# etc
#
uninstall_sdaemon() 
{
	# check the root configuration directory for init files
	get_etcbasedir
	
	# uninstall Kill scripts
	_INSTALL_DIRS=${_KILL_DIRS}
	_TARGET_NAME=${KILLS}
	uninstall_scripts
	
	# uninstall Start scripts
	_INSTALL_DIRS=${_START_DIRS}
	_TARGET_NAME=${STARTS}
	uninstall_scripts

	# remove the sdaemon from /usr/bin and its rc file from /etc
	if [ -f /usr/bin/$DMGR ]; then
		echo removing /usr/bin/$DMGR
		$rm /usr/bin/$DMGR
	fi

	if [ -f $DMGRRC ]; then
		echo removing $DMGRRC
		$rm $DMGRRC
	fi
}

#
# install the sdaemon and all of its associated scripts to /etc
#
install_sdaemon() 
{
	if [ ! -f "$INST_DIR/$DMGR" ]; then
		echo $DMGR binary not found, unable to install
		exit 1
	fi

	# install daemon, please note that /usr/bin exists for sure
	$cp "$INST_DIR/$DMGR" /usr/bin/$DMGR
	echo installing $DMGR as /usr/bin/$DMGR
	chmod 744 /usr/bin/$DMGR

	# check the root configuration directory for init files
	get_etcbasedir

	# install Kill scripts
	_INSTALL_DIRS=${_KILL_DIRS}
	_TARGET_NAME=${KILLS}
	install_scripts
	
	# install Start scripts
	_INSTALL_DIRS=${_START_DIRS}
	_TARGET_NAME=${STARTS}
	install_scripts
}

#
# start the daemon
#
start_sdaemon()
{
	if [ -x /usr/bin/$DMGR ] ; then
		/usr/bin/$DMGR
	fi
}


#
#
# MAIN
#
#

mode="$1"
installdir="$2"

if [ "$installdir" != "" ]; then
	INST_DIR="${installdir}/program/service"
else
	INST_DIR=`pwd`
fi

if [ ! -d /usr/bin ]; then
	echo "$0: installation dir /usr/bin not found"
	exit 1
fi

# thats it, must be root to operate 
set `/usr/bin/id`
if [ $1 != "uid=0(root)" ]; then
	echo "$0: must be run as root"
	exit 1
fi

# try to find some of the programs needed to perform properly
_PROGRAMS="cut awk grep ps rm kill cp ln"
for PROG in ${_PROGRAMS} ;  do
	find_prog
done

cp="${cp} -f"
rm="${rm} -f"
ln="${ln} -sf"

# main dispatch routine
case "$mode" in
'start')
	start_sdaemon
	;;
'start_msg')
	start_sdaemon
	;;
'stop') 	
	stop_sdaemon
	;;
'install') 	
	install_sdaemon
	;;
'uninstall') 	
	uninstall_sdaemon
	;;
'reset') 	
	stop_sdaemon HUP
	;;
*)
	usage_error
	exit 1
	;;
esac

# well done
exit 0
