/*************************************************************************
 *
 *  $RCSfile: signal.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


/* system headers */

#include "system.h"

#include <osl/diagnose.h>
#include <osl/mutex.h>
#include <osl/signal.h>

typedef struct _oslSignalHandlerImpl
{
	oslSignalHandlerFunction      Handler;
	void*			        	  pData;
	struct _oslSignalHandlerImpl* pNext;	
} oslSignalHandlerImpl;

static sal_Bool  			  bInitSignal = sal_False;
static oslMutex 			  SignalListMutex;
static oslSignalHandlerImpl*  SignalList;

static WORD Interrupt;

static HINSTANCE	 		  hToolHelp	= 0;
static void (WINAPI *pFnTerminateApp)(HTASK hTask, WORD wFlags) = NULL;
static BOOL (WINAPI *pFnInterruptRegister)(HTASK, FARPROC)		= NULL;
static BOOL (WINAPI *pFnInterruptUnRegister)(HTASK)				= NULL;


static void FAR PASCAL SignalHandlerFunction();

static sal_Bool InitSignal()
{
	SignalListMutex = osl_createMutex();

	hToolHelp = LoadLibrary("TOOLHELP.DLL");
	if (hToolHelp >= HINSTANCE_ERROR)
	{
		pFnTerminateApp = (void (WINAPI*)(HTASK, WORD))GetProcAddress(hToolHelp, "TERMINATEAPP");
		pFnInterruptRegister = (BOOL (WINAPI*)(HTASK, FARPROC))GetProcAddress(hToolHelp, "INTERRUPTREGISTER");
		pFnInterruptUnRegister = (BOOL (WINAPI*)(HTASK))GetProcAddress(hToolHelp, "INTERRUPTUNREGISTER");
		if (pFnInterruptRegister && pFnInterruptUnRegister )
			pFnInterruptRegister(0, MakeProcInstance((FARPROC)SignalHandlerFunction, GetAppInstance()));
		else
		{
			FreeLibrary(hToolHelp);
			pFnInterruptRegister	= NULL;
			pFnInterruptUnRegister  = NULL;
			hToolHelp		        = 0;
		}
	}
	else
		hToolHelp = 0;

	return sal_True;
}

static sal_Bool DeInitSignal()
{
	if (pFnInterruptUnRegister)
	{
		pFnInterruptUnRegister(0);
		FreeLibrary(hToolHelp);
	}

	osl_destroyMutex(SignalListMutex);

	return sal_False;
}

static oslSignalAction CallSignalHandler(oslSignalInfo *pInfo)
{
	oslSignalHandlerImpl* pHandler = SignalList;
	oslSignalAction Action = osl_Signal_ActCallNextHdl;

	while (pHandler != NULL)
	{
		if ((Action = pHandler->Handler(pHandler->pData, pInfo)) != osl_Signal_ActCallNextHdl)
			break;

		pHandler = pHandler->pNext;
	}

	return Action;
}

/*****************************************************************************/
/* SignalHandlerFunction	*/
/*****************************************************************************/
static void FAR PASCAL SignalHandlerFunction()
{
	// save all register
	_asm pusha
	_asm pushf

	// possible ds != ss
	_asm mov	ds,ax

	_asm mov	ax, [bp+8]
	_asm mov	Interrupt, ax


	{
		oslSignalInfo	 Info;
		
		Info.UserSignal = Interrupt;
		Info.UserData   = NULL;

		Info.Signal = osl_Signal_AccessViolation;

		switch (CallSignalHandler(&Info))
		{
			case osl_Signal_ActCallNextHdl:
				return;

			case osl_Signal_ActAbortApp:
				pFnTerminateApp(GetCurrentTask(), 0);
				break;

			case osl_Signal_ActKillApp:
				pFnTerminateApp(GetCurrentTask(), 0);
				break;
		}
	}

	_asm popf
	_asm popa
}

/*****************************************************************************/
/* osl_addSignalHandler */
/*****************************************************************************/
oslSignalHandler SAL_CALL osl_addSignalHandler(oslSignalHandlerFunction Handler, void* pData)
{
	oslSignalHandlerImpl* pHandler;

	OSL_ASSERT(Handler != NULL);

	if (! bInitSignal)
		bInitSignal = InitSignal();

	pHandler = calloc(1, sizeof(oslSignalHandlerImpl));

	if (pHandler != NULL)
	{	
		pHandler->Handler = Handler;
		pHandler->pData   = pData;

		osl_acquireMutex(SignalListMutex);
	
		pHandler->pNext = SignalList;
		SignalList      = pHandler;

		osl_releaseMutex(SignalListMutex);

		return (pHandler);
	}

	return (NULL);
}

/*****************************************************************************/
/* osl_removeSignalHandler */
/*****************************************************************************/
sal_Bool SAL_CALL osl_removeSignalHandler(oslSignalHandler Handler)
{
	oslSignalHandlerImpl *pHandler, *pPrevious = NULL;

	OSL_ASSERT(Handler != NULL);

	if (! bInitSignal)
		bInitSignal = InitSignal();

	osl_acquireMutex(SignalListMutex);

	pHandler = SignalList;

	while (pHandler != NULL)
	{
		if (pHandler == Handler)
		{
			if (pPrevious)
				pPrevious->pNext = pHandler->pNext;
			else
				SignalList = pHandler->pNext;

			osl_releaseMutex(SignalListMutex);

			if (SignalList == NULL)
				bInitSignal = DeInitSignal();
							
			free(pHandler);

			return (sal_True);
		}

		pPrevious = pHandler;
		pHandler  = pHandler->pNext;
	}
			
	osl_releaseMutex(SignalListMutex);

	return (sal_False);
}

/*****************************************************************************/
/* osl_raiseSignal */
/*****************************************************************************/
oslSignalAction SAL_CALL osl_raiseSignal(sal_Int32 UserSignal, void* UserData)
{
	oslSignalInfo   Info;
	oslSignalAction Action;

	if (! bInitSignal)
		bInitSignal = InitSignal();

	osl_acquireMutex(SignalListMutex);

	Info.Signal     = osl_Signal_User;
	Info.UserSignal = UserSignal;
	Info.UserData   = UserData;

	Action = CallSignalHandler(&Info);
			
	osl_releaseMutex(SignalListMutex);

	return (Action);
}


/*************************************************************************
*
*    $Log: signal.c,v $
*    Revision 1.1.1.1  2000/09/18 15:17:22  hr
*    initial import
*
*    Revision 1.8  2000/09/18 14:29:00  willem.vandorp
*    OpenOffice header added.
*
*    Revision 1.7  2000/08/11 17:03:59  willem.vandorp
*    Header and footer replaced
*
*    Revision 1.6  1999/10/27 15:11:03  mfe
*    Change of Copyright, removed compiler warnings, code clean up, ...
*
*    Revision 1.5  1999/01/20 19:07:19  jsc
*    #61011# Typumstellung
*
*    Revision 1.4  1998/03/15 12:51:42  hro
*    Neue enums
*
*    Revision 1.3  1998/03/12 14:27:19  hro
*    Anpassungen fuer Borland Compiler
*
*    Revision 1.2  1998/03/06 18:00:14  hro
*    Syntaxkorrektur
*
*    Revision 1.1  1998/03/06 16:36:27  rh
*    Added signal.c for W16 and deinit-routines
*
*************************************************************************/

 






