/*************************************************************************
 *
 *  $RCSfile: channel.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_CHANNEL_H_
#	include <osl/channel.h>
#endif

#include <osl/interlck.h>
#include <rtl/alloc.h>
#include <rtl/memory.h>

#include "../all/channman.h" 
#include "system.h"
#include "sockimpl.h"
#include "secimpl.h"

#define PIPEPREFIX  "\\\\.\\PIPE\\OSL_IOT_"

#define OSL_CHANNEL_MANAGER_TYPE_NAME "OSL_CHANNEL_MANAGER"

typedef struct _oslPipeImpl {
	oslInterlockedCount m_Reference;
	HANDLE		 		m_File;
} oslPipeImpl;

oslPipeImpl* __osl_createPipeImpl();

typedef struct {
	HANDLE					fd;
	PSECURITY_DESCRIPTOR 	pPipeSD;
	rtl_uString *			name;
} Pipe;


#define MAGIC_NUMBER 			0xFD2FD2FD

#define MESSAGE_TYPE_RAW_DATA 	0x00000001
#define MESSAGE_TYPE_FD 		0x00000002
#define MESSAGE_FD_TYPE_SOCKET	0x00000010
#define MESSAGE_FD_TYPE_PIPE	0x00000020

static void makePipeName(rtl_uString** buffer, rtl_uString* pipeName)
{
	rtl_uString* pipePrefix = NULL;

	rtl_uString_newFromAscii(&pipePrefix, PIPEPREFIX);

	if (pipeName)
		rtl_uString_newConcat(buffer, pipePrefix, pipeName);
	else
		rtl_uString_assign(buffer, pipePrefix);

	rtl_uString_release(pipePrefix);
}

/* Implementation resource pipe access function */
static oslResourcePipe SAL_CALL listenOnResourcePipe(rtl_uString* Name)
{
	SECURITY_ATTRIBUTES saPipeSecurity;

	Pipe* pPipeImpl = (Pipe*) rtl_allocateZeroMemory(sizeof(Pipe));
	makePipeName(&pPipeImpl->name, Name);

	/* security inits */
	rtl_zeroMemory(&saPipeSecurity, sizeof (SECURITY_ATTRIBUTES));

	/* alloc & init SD */
	pPipeImpl->pPipeSD = (PSECURITY_DESCRIPTOR) rtl_allocateMemory(SECURITY_DESCRIPTOR_MIN_LENGTH);

	InitializeSecurityDescriptor(pPipeImpl->pPipeSD, SECURITY_DESCRIPTOR_REVISION);

	/* set NULL DACL on the SD */
	SetSecurityDescriptorDacl (pPipeImpl->pPipeSD, TRUE, (PACL) NULL, FALSE);

	/* now set up the security attributes */
	saPipeSecurity.nLength    = sizeof (SECURITY_ATTRIBUTES);
	saPipeSecurity.bInheritHandle  = TRUE; 
	saPipeSecurity.lpSecurityDescriptor = pPipeImpl->pPipeSD;

	/* can call this funtion directly here because this code will only be called on NT */
    pPipeImpl->fd = CreateNamedPipeW(
		pPipeImpl->name->buffer, 
		PIPE_ACCESS_DUPLEX,
        PIPE_WAIT | PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE,
        PIPE_UNLIMITED_INSTANCES,
		4096, 4096,
		NMPWAIT_WAIT_FOREVER, 
        &saPipeSecurity);

	if (pPipeImpl->fd == INVALID_HANDLE_VALUE)
	{
		rtl_freeMemory(pPipeImpl->pPipeSD);
		rtl_uString_release(pPipeImpl->name);
		rtl_freeMemory(pPipeImpl);
		pPipeImpl = NULL;
	}

	return((oslResourcePipe)pPipeImpl);
}


static oslResourcePipe SAL_CALL acceptOnResourcePipe(oslResourcePipe hPipe)
{
	Pipe* pPipeImpl = (Pipe*)hPipe;
	Pipe* pNewPipeImpl = NULL;
	DWORD error =0;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd != INVALID_HANDLE_VALUE))
	{
		pNewPipeImpl = (Pipe*) rtl_allocateZeroMemory(sizeof(Pipe));
		rtl_uString_assign(&pNewPipeImpl->name, pPipeImpl->name);
	
		if (
			ConnectNamedPipe(pPipeImpl->fd, NULL) ||
			(GetLastError() == ERROR_PIPE_CONNECTED)
		   )
		{
			SECURITY_ATTRIBUTES saPipeSecurity;

			// security inits
			rtl_zeroMemory( &saPipeSecurity, sizeof (SECURITY_ATTRIBUTES));

			// now set up the security attributes
			saPipeSecurity.nLength    = sizeof (SECURITY_ATTRIBUTES);
			saPipeSecurity.bInheritHandle  = TRUE; 
			saPipeSecurity.lpSecurityDescriptor = pPipeImpl->pPipeSD;

			pNewPipeImpl->fd = pPipeImpl->fd;

			/* can call this funtion directly here because this code will only be called on NT */
		    pPipeImpl->fd = CreateNamedPipeW(
				pPipeImpl->name->buffer, 
				PIPE_ACCESS_DUPLEX,
		        PIPE_WAIT | PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE,
		        PIPE_UNLIMITED_INSTANCES,
				4096, 4096,
				NMPWAIT_WAIT_FOREVER, 
		        &saPipeSecurity);
		}
		else
		{
			error = GetLastError();
			rtl_uString_release(pNewPipeImpl->name);
			rtl_freeMemory(pNewPipeImpl);
			pNewPipeImpl = NULL;
		}
	}

	return((oslResourcePipe)pNewPipeImpl);
}


/* mfe : please fix this since I do not know how to write it correctly on windows */
static oslResourcePipe SAL_CALL connectToResourcePipe(rtl_uString* Name)
{
	DWORD error;

	Pipe* pPipeImpl = (Pipe*) rtl_allocateZeroMemory(sizeof(Pipe));
	pPipeImpl->fd = INVALID_HANDLE_VALUE;
	makePipeName(&pPipeImpl->name, Name);

	/* wait until the server do ConnectNamedPipe() */
	while (
		   (pPipeImpl->fd == INVALID_HANDLE_VALUE) &&
		   WaitNamedPipeW(pPipeImpl->name->buffer, NMPWAIT_WAIT_FOREVER)
		  )
	{
	
	    pPipeImpl->fd = CreateFileW(
			pPipeImpl->name->buffer,
			GENERIC_READ|GENERIC_WRITE, 
	        FILE_SHARE_READ | FILE_SHARE_WRITE,
	        NULL,
	        OPEN_EXISTING, 
	        FILE_ATTRIBUTE_NORMAL, 
	        NULL);

		if (pPipeImpl->fd == INVALID_HANDLE_VALUE)
			error = GetLastError();
	}

	if (pPipeImpl->fd == INVALID_HANDLE_VALUE)
	{
		rtl_uString_release(pPipeImpl->name);
		rtl_freeMemory(pPipeImpl);
		pPipeImpl = NULL;
	}

	return((oslResourcePipe)pPipeImpl);
}

static void SAL_CALL closeResourcePipe(oslResourcePipe hPipe)
{
	Pipe* pPipeImpl = (Pipe*)hPipe;

	if (pPipeImpl != NULL)
	{
		if (pPipeImpl->fd != INVALID_HANDLE_VALUE)
		{
			DisconnectNamedPipe(pPipeImpl->fd);
			CloseHandle(pPipeImpl->fd);
		}

		if (pPipeImpl->pPipeSD != NULL)	
			rtl_freeMemory(pPipeImpl->pPipeSD);

		rtl_uString_release(pPipeImpl->name);
		rtl_freeMemory(pPipeImpl);
	}
}	

static sal_Int32 SAL_CALL sendBuffer(oslResourcePipe hPipe, const void* pBuffer, sal_uInt32 BufferSize)
{
	Pipe* pPipeImpl = (Pipe*)hPipe;
	sal_Int32 bytesSend = 0;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd != INVALID_HANDLE_VALUE))
		WriteFile(pPipeImpl->fd, pBuffer, BufferSize, &bytesSend, NULL);

	return(bytesSend);
}	

static sal_Int32 SAL_CALL receiveBuffer(oslResourcePipe hPipe, void* pBuffer, sal_uInt32 BufferSize)
{
	Pipe* pPipeImpl = (Pipe*)hPipe;
	sal_Int32 bytesRead = 0;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd != INVALID_HANDLE_VALUE))
		ReadFile(pPipeImpl->fd, pBuffer, BufferSize, &bytesRead, NULL);

	return(bytesRead);
}	

static sal_Bool SAL_CALL sendResource
(
	oslResourcePipe 	hPipe, 
	oslChannelResource 	Resource, 
	oslChannelType 		Type,
	sal_Bool 			bAmITheBroker
)
{
	Pipe* pPipeImpl = (Pipe*)hPipe;
	sal_Bool bRet = sal_False;
	sal_Int32 bytes = 0;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd != INVALID_HANDLE_VALUE))
	{
		if (bAmITheBroker)
		{
			/* 	duplicate handle on this other side -> 
				receive remote process 
				duplicate handle and send it */
			HANDLE fd = INVALID_HANDLE_VALUE;
			DWORD remoteProcessID = 0;
			sal_uInt32 code = MESSAGE_TYPE_FD;

			switch (Type)
			{
				case osl_Channel_Type_Socket:
					fd = (HANDLE)((oslSocketImpl *)(Resource.Socket))->m_Socket;
					code |= MESSAGE_FD_TYPE_SOCKET;
					break;
				case osl_Channel_Type_Pipe:
					fd = ((oslPipeImpl *)(Resource.Pipe))->m_File;
					code |= MESSAGE_FD_TYPE_PIPE;
					break;
				default:
					break;
			}

			if (ReadFile(pPipeImpl->fd, &remoteProcessID, sizeof(remoteProcessID), &bytes, NULL))
			{
				HANDLE hRemoteProc = OpenProcess(
										PROCESS_DUP_HANDLE, 
										FALSE, 
										remoteProcessID);

				if (hRemoteProc != (HANDLE)NULL)
				{
					HANDLE newFd;

					if (DuplicateHandle(GetCurrentProcess(),
										fd, 
										hRemoteProc, 
										&newFd,
										0, FALSE, DUPLICATE_SAME_ACCESS))
					{
						if (
							WriteFile(pPipeImpl->fd, &code, sizeof(code), &bytes, NULL) &&
							WriteFile(pPipeImpl->fd, &newFd, sizeof(fd), &bytes, NULL)
						   )
							bRet = sal_True;
					}

					CloseHandle(hRemoteProc);
				}
			}
		}
		else
		{
			/* duplicate handle on the other side -> send my process id and my handle */
			HANDLE fd = INVALID_HANDLE_VALUE;
			DWORD myProcessID = GetCurrentProcessId();
			sal_uInt32 code = MESSAGE_TYPE_FD;

			switch (Type)
			{
				case osl_Channel_Type_Socket:
					fd = (HANDLE)((oslSocketImpl *)(Resource.Socket))->m_Socket;
					code |= MESSAGE_FD_TYPE_SOCKET;
					break;
				case osl_Channel_Type_Pipe:
					fd = ((oslPipeImpl *)(Resource.Pipe))->m_File;
					code |= MESSAGE_FD_TYPE_PIPE;
					break;
				default:
					break;
			}

			if (
				WriteFile(pPipeImpl->fd, &myProcessID, sizeof(myProcessID), &bytes, NULL) &&
				WriteFile(pPipeImpl->fd, &code, sizeof(code), &bytes, NULL) &&
				WriteFile(pPipeImpl->fd, &fd, sizeof(fd), &bytes, NULL)
			   )
				bRet = sal_True;
		}
	}

	if (bRet)
	{
		sal_Int32 commitCode;

		if (
			!ReadFile(pPipeImpl->fd, &commitCode, sizeof(commitCode), &bytes, NULL) ||
			(commitCode <= 0)
		   )
			bRet = sal_False;
	}
	
	return(bRet);
}	

static sal_Bool SAL_CALL receiveResource
(
	oslResourcePipe 	hPipe, 
	oslChannelResource* pResource, 
	oslChannelType* 	pType,
	sal_Bool 			bAmITheBroker
)
{
	Pipe* pPipeImpl = (Pipe*)hPipe;
	sal_Bool bRet = sal_False;
	DWORD error;
	sal_Int32 bytes = 0;
	sal_Int32 commitCode;

	if ((pPipeImpl != NULL) && (pPipeImpl->fd != INVALID_HANDLE_VALUE))
	{
		if (bAmITheBroker)
		{
			/* 	duplicate handle on this side -> 
				receive remote process and handle and duplicate handle it */
			HANDLE fd = INVALID_HANDLE_VALUE;
			HANDLE newFd = INVALID_HANDLE_VALUE;
			DWORD remoteProcessID = 0;
			sal_uInt32 code = MESSAGE_TYPE_FD;

			if (
				ReadFile(pPipeImpl->fd, &remoteProcessID, sizeof(remoteProcessID), &bytes, NULL) &&
				ReadFile(pPipeImpl->fd, &code, sizeof(code), &bytes, NULL) &&
				ReadFile(pPipeImpl->fd, &fd, sizeof(fd), &bytes, NULL)
			   )
			{
				HANDLE hRemoteProc = OpenProcess(
										PROCESS_DUP_HANDLE, 
										FALSE, 
										remoteProcessID);

				if (hRemoteProc != (HANDLE)NULL)
				{
					if (DuplicateHandle(hRemoteProc,
										fd, 
										GetCurrentProcess(), 
										&newFd,
										0, FALSE, DUPLICATE_SAME_ACCESS))
					{
						if (code & MESSAGE_FD_TYPE_SOCKET)
						{
					    	pResource->Socket = __osl_createSocketImpl((SOCKET)newFd);
							*pType = osl_Channel_Type_Socket;
							bRet = sal_True;
						}
						else if (code & MESSAGE_FD_TYPE_PIPE)
						{
							oslPipeImpl * pPipeImpl = __osl_createPipeImpl();
					    	pPipeImpl->m_File = newFd;
					    	pResource->Pipe = pPipeImpl;

							*pType = osl_Channel_Type_Pipe;
							bRet = sal_True;
						}
					}

					CloseHandle(hRemoteProc);
				}
				else
				{
					error = GetLastError();
				}
			}
		}
		else
		{
			/* duplicate handle on the other side -> 
			send my process id receive duplicated handle */
			HANDLE fd = INVALID_HANDLE_VALUE;
			DWORD myProcessID = GetCurrentProcessId();
			sal_uInt32 code = MESSAGE_TYPE_FD;

			if (
				WriteFile(pPipeImpl->fd, &myProcessID, sizeof(myProcessID), &bytes, NULL) &&
				ReadFile(pPipeImpl->fd, &code, sizeof(code), &bytes, NULL) &&
				ReadFile(pPipeImpl->fd, &fd, sizeof(fd), &bytes, NULL)
			   )
			{
				if (code & MESSAGE_FD_TYPE_SOCKET)
				{
			    	pResource->Socket = __osl_createSocketImpl((SOCKET)fd);
					*pType = osl_Channel_Type_Socket;
					bRet = sal_True;
				}
				else if (code & MESSAGE_FD_TYPE_PIPE)
				{
					oslPipeImpl * pPipeImpl = __osl_createPipeImpl();
			    	pPipeImpl->m_File = fd;
			    	pResource->Pipe = pPipeImpl;

					*pType = osl_Channel_Type_Pipe;
					bRet = sal_True;
				}
			}
		}

		if (bRet)
			commitCode = 1;
		else
			commitCode = 0;

		WriteFile(pPipeImpl->fd, &commitCode, sizeof(commitCode), &bytes, NULL);
	}
	
		

	return(bRet);
}	

static ResourcePipeApi globalResourcePipeApi =
{
	listenOnResourcePipe,
	acceptOnResourcePipe,
	connectToResourcePipe,
	closeResourcePipe,
	sendBuffer,
	receiveBuffer,
	sendResource,
	receiveResource
};

sal_Bool SAL_CALL osl_startChannelBroker()
{
	sal_Bool	fSuccess=sal_False;
	rtl_uString	*type=NULL;

	rtl_uString_newFromAscii(&type, OSL_CHANNEL_MANAGER_TYPE_NAME);

	if (IS_NT)
		fSuccess=osl_default_startChannelBroker(&globalResourcePipeApi, type);

	rtl_uString_release(type);
	
	return fSuccess;
}	

sal_Bool SAL_CALL osl_stopChannelBroker()
{
	sal_Bool ret = sal_False;

	if (IS_NT)
		ret = osl_default_stopChannelBroker();

	return ret;
}

oslChannelBroker SAL_CALL osl_channelConnectBroker()
{
	oslChannelBroker Broker = NULL;

	if (IS_NT)
	{
		rtl_uString	*type=NULL;

		rtl_uString_newFromAscii(&type, OSL_CHANNEL_MANAGER_TYPE_NAME);
		Broker=osl_default_channelConnectBroker(&globalResourcePipeApi, type);
		rtl_uString_release(type);
	}

	return Broker;
}	

void SAL_CALL osl_channelDisconnectBroker(oslChannelBroker Broker)
{
	if (IS_NT)
		osl_default_channelDisconnectBroker(Broker);
}	

oslChannelError SAL_CALL osl_channelProvide
(
	oslChannelBroker		Broker,
	oslChannelResource		Resource, 
	oslChannelType			Type,
	rtl_uString*            strResourceName,
	rtl_uString*            strUserIdent,
	rtl_uString*            strPassword,
	const TimeValue* 		pTimeout
)
{
	oslChannelError error = osl_Channel_E_NoManger;

	if (IS_NT)
		error = osl_default_channelProvide(Broker, Resource, Type, strResourceName, strUserIdent, strPassword, pTimeout);

	return error;
}

oslChannelError SAL_CALL osl_channelAccept
(
	oslChannelBroker		Broker,
	oslChannelResource*		pResource, 
    oslChannelType* 		pType,	
	rtl_uString*            strResourceName,
	CheckAuthentication_F	checkAuthFunc,
	void* 					pAuthFuncData,
	const TimeValue* 		pTimeout
)
{
	oslChannelError error = osl_Channel_E_NoManger;

	if (IS_NT)
		error = osl_default_channelAccept(Broker, pResource, pType, strResourceName, checkAuthFunc, pAuthFuncData, pTimeout);

	return error;
}

