/*************************************************************************
 *
 *  $RCSfile: daemon.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//#define UNICODE
#include "system.h"

#include <osl/diagnose.h>
#include <osl/daemon.h>
#include <osl/conditn.h>
#include <osl/thread.h>
#include <rtl/alloc.h>

#include "../all/daemprot.h"
#include "../all/daemclie.h"
#include "../all/daemreg.h" 
#include "daemregw.h"

typedef DWORD (WINAPI *REGISTER_SERVICE_PROCESS_FUNCTION)(DWORD, DWORD);

#define MAX_DWORD_2_STRING_BUFFER	64
/*
	Daemon-data structure hidden behind oslDaemonHandle:    
*/
typedef struct _osl_TDaemonImpl 
{
	sal_Bool                 	m_bStopped;
	SERVICE_STATUS          	m_Status;
	SERVICE_STATUS_HANDLE   	m_hStatus;
	oslDaemonControl        	m_DaemonControl;
	void*                   	m_pControlData; 
	oslDaemonPropertyAccess* 	m_pPropertyAccess;
	void*                   	m_pAccessData;
	oslDaemonInfo				m_info;
	oslCondition				m_stopCond;
} osl_TDaemonImpl;

static osl_TDaemonImpl* pService;

static sal_Bool useNTServices()
{
	static sal_Bool ret = sal_False;	
	static sal_Bool isInit = sal_False;	

	if (!isInit)
	{
		OSVERSIONINFO VersionInformation = 
		{
			sizeof(OSVERSIONINFO),
			0, 
	    	0, 
	    	0,
	    	0, 
	    	"",
		};

		if (
			GetVersionEx(&VersionInformation) &&
			(VersionInformation.dwPlatformId == VER_PLATFORM_WIN32_NT)
		   )
		{
			rtl_uString	*buffer=NULL;
			rtl_uString	*sDaemon=NULL;

			rtl_uString_newFromStr(&sDaemon,L"OSL_FORCE_SDAEMON");
			if (
				(osl_getEnvironment(sDaemon, &buffer) != osl_Process_E_None)	||
				(wcscmp(rtl_uString_getStr(buffer), L"yes") != 0)
			   )	
			{
				ret	= sal_True;
			}
			if (buffer)
				rtl_uString_release(buffer);
			rtl_uString_release(sDaemon);
		}

   		isInit = sal_True;
	}

	return(ret);
}	

DWORD W95_RegisterServiceProcess(DWORD dwProcessId, DWORD dwServiceType)
{
	static REGISTER_SERVICE_PROCESS_FUNCTION lpfRegisterServiceProcess = NULL; 
	DWORD ret = 0;
	
	if (lpfRegisterServiceProcess == NULL)
	{
		lpfRegisterServiceProcess = (REGISTER_SERVICE_PROCESS_FUNCTION)
			GetProcAddress(GetModuleHandle("KERNEL32"),
						   "RegisterServiceProcess");
	}

	if (lpfRegisterServiceProcess != NULL)
	{
		ret = ((*lpfRegisterServiceProcess)(dwProcessId, dwServiceType));
	}

	return(ret);
	
}	


/*****************************************************************************/
/* oslDaemonWrapperControl */
/*****************************************************************************/
static void WINAPI oslDaemonWrapperControl(DWORD dwControl) 
{
	osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)pService;

	if (pDaemonImpl)
	{
		switch (dwControl)
		{
			case SERVICE_CONTROL_INTERROGATE:
			    SetServiceStatus(pDaemonImpl->m_hStatus, &pDaemonImpl->m_Status);
				break;

			case SERVICE_CONTROL_PAUSE:
				pDaemonImpl->m_DaemonControl(pDaemonImpl->m_pControlData, osl_Daemon_CommandPause);
				break;

			case SERVICE_CONTROL_CONTINUE:
				pDaemonImpl->m_DaemonControl(pDaemonImpl->m_pControlData, osl_Daemon_CommandContinue);
				break;

			case SERVICE_CONTROL_SHUTDOWN:
				pDaemonImpl->m_DaemonControl(pDaemonImpl->m_pControlData, osl_Daemon_CommandShutdown);
				break;

			case SERVICE_CONTROL_STOP:
				pDaemonImpl->m_DaemonControl(pDaemonImpl->m_pControlData, osl_Daemon_CommandStop);
				pDaemonImpl->m_bStopped = sal_True;
				break;
		}
	}
}

/*****************************************************************************/
/* ConnectionWorkerFunction */
/*****************************************************************************/

static void SAL_CALL ConnectionWorkerFunction(void* pVoid)
{
	osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)pVoid;
	oslDaemonConnection	rootConn = 
		osl_openDaemonMgrConn(pDaemonImpl->m_info.Identifier, osl_DaemonConnection_CREATE);
	oslDaemonConnection childConn = NULL;
	oslDaemonMgrMsg* pRequest = NULL;
	sal_Bool bTerminate = sal_False;

	if (rootConn)
	{
		do
		{
			childConn = osl_acceptOnDaemonMgrConn(rootConn);

			while (
					!bTerminate &&
					(osl_recvDaemonMgrMsg(childConn, &pRequest) == osl_Daemon_ManagerError_NoError)
				  )
			{
				if (pRequest->Type == osl_DaemonMgrMsgType_Request)
				{
					switch (pRequest->Subject)
					{
						case osl_DaemonMgrMsgSubject_Shutdown:
							bTerminate = sal_True;
							break;
						case osl_DaemonMgrMsgSubject_PropertyCountGet:
							if (pDaemonImpl->m_pPropertyAccess == NULL)
								pRequest->Body.PropertyCount = 0;
							else
							{
								pRequest->Error = 
									pDaemonImpl->m_pPropertyAccess->fGetCount(
										pDaemonImpl->m_pAccessData, 
										&(pRequest->Body.PropertyCount));
							}
							break;
						case osl_DaemonMgrMsgSubject_PropertiesGet:
							if (pDaemonImpl->m_pPropertyAccess == NULL)
							{
								pRequest->Body.Properties.Count = 0;
							}
							else
							{
								pRequest->Error = 
									pDaemonImpl->m_pPropertyAccess->fGetProperties(
										pDaemonImpl->m_pAccessData, 
										&(pRequest->Body.Properties.List),
										&(pRequest->Body.Properties.Count));
							}
							break;
						case osl_DaemonMgrMsgSubject_PropertiesSet:
							if (pDaemonImpl->m_pPropertyAccess == NULL)
							{
								pRequest->Body.Properties.Count = 0;
							}
							else
							{
								rtl_uString *strErrorText = NULL;

								pRequest->Error = 
									pDaemonImpl->m_pPropertyAccess->fSetProperties(
										pDaemonImpl->m_pAccessData, 
										pRequest->Body.Properties.List,
										pRequest->Body.Properties.Count,
										&(pRequest->Body.Properties.ErrorCode),
										&strErrorText);

								if (strErrorText && strErrorText->length)
									rtl_uString_assign(&pRequest->Body.Properties.ErrorText, strErrorText);
							}
							break;
						default:
							pRequest->Error = osl_Daemon_ManagerError_Unknown;
							break;
					}					

					pRequest->Type = osl_DaemonMgrMsgType_Reply;
					osl_sendDaemonMgrMsg(childConn, pRequest);
				}

				osl_destroyDaemonMgrMsg(pRequest);
			}

			osl_disconnectDaemonMgrConn(childConn);

		} 
		while ((!bTerminate) && (childConn != NULL));
		
		osl_disconnectDaemonMgrConn(rootConn);
	}
}	

/*****************************************************************************/
/* oslDaemonWrapperFunction */
/*****************************************************************************/
static void WINAPI oslDaemonWrapperFunction(DWORD dwArgc, LPWSTR *lpszArgv) 
{
	osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)pService;
	oslDaemonMgrMsg* pRequest = NULL;
	oslThread pipeThread = NULL;
	oslDaemonConnection	conn = NULL;
	oslDaemonManagerError error;
		
	pipeThread = osl_createThread(ConnectionWorkerFunction, pService);

	pDaemonImpl->m_hStatus = RegisterServiceCtrlHandlerW(pDaemonImpl->m_info.Identifier->buffer, 
													    oslDaemonWrapperControl);

	pDaemonImpl->m_DaemonControl(pDaemonImpl->m_pControlData, osl_Daemon_CommandStart);

	pDaemonImpl->m_bStopped = sal_False;

	osl_waitCondition(pDaemonImpl->m_stopCond, NULL);

	/* shutdown pipe acceptor thread */

	conn = osl_openDaemonMgrConn(pDaemonImpl->m_info.Identifier, osl_DaemonConnection_OPEN);
	
	if (conn != NULL)
	{
		pRequest = osl_createDaemonMgrMsg(NULL, osl_DaemonMgrMsgSubject_Shutdown);

		error = osl_sendDaemonMgrMsg(conn, pRequest);
		osl_destroyDaemonMgrMsg(pRequest);
		pRequest = NULL;

		if (error == osl_Daemon_ManagerError_NoError)		
		{
			error = osl_recvDaemonMgrMsg(conn, &pRequest);

			if (error == osl_Daemon_ManagerError_NoError)
			{
				error = pRequest->Error;
				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}
		}

		osl_disconnectDaemonMgrConn(conn);
	}
	else
		error = osl_Daemon_ManagerError_Unknown;

	if (error == osl_Daemon_ManagerError_NoError)
	{
		osl_terminateThread(pipeThread);
		osl_freeThreadHandle(pipeThread);
	}
	else
	{
		osl_destroyThread(pipeThread);
	}
}

/*****************************************************************************/
/* waitForDaemonReachState */
/*****************************************************************************/

sal_Bool waitForDaemonReachState(SC_HANDLE hService, 
								DWORD desiredState, 
								SERVICE_STATUS* pss, 
								DWORD millisecs)
{
	sal_Bool success = sal_False;

	return(success);
}	

/*****************************************************************************/
/* osl_queryDaemonCount */
/*****************************************************************************/

oslDaemonManagerError SAL_CALL osl_queryDaemonCount(sal_uInt32* pCount)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	sal_uInt32 size = osl_getDaemonWindowsRegistry()->getDaemons(NULL, 0);
	
	*pCount = 0;
	
	if (size > 0)
	{	
		sal_Unicode* daemons = (sal_Unicode*) rtl_allocateMemory(size * sizeof(sal_Unicode));
		const sal_Unicode* daemon = daemons;

		osl_getDaemonWindowsRegistry()->getDaemons(daemons, size);

		while (*daemon != L'\0')
		{
			(*pCount)++;
			daemon += wcslen(daemon) + 1;
		}		

		rtl_freeMemory(daemons);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_queryDaemonIdentifier */
/*****************************************************************************/

oslDaemonManagerError SAL_CALL osl_queryDaemonIdentifier(sal_uInt32 index, rtl_uString** strIdentifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	sal_uInt32 size = osl_getDaemonWindowsRegistry()->getDaemons(NULL, 0);
	sal_uInt32 i = 0;
	
	if (size > 0)
	{
		sal_Unicode* daemons = (sal_Unicode*) rtl_allocateMemory(size * sizeof(sal_Unicode));
		const sal_Unicode* daemon = daemons;

		osl_getDaemonWindowsRegistry()->getDaemons(daemons, size);

		while ((*daemon != L'\0') && (i < index))
		{
			daemon += wcslen(daemon) + 1;
			i++;
		}		

		if (*daemon != L'\0')
		{
			rtl_uString_newFromStr(strIdentifier, daemon);
			ret = osl_Daemon_ManagerError_NoError;
		}

		free(daemons);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_isDaemonInstalled */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_isDaemonInstalled(rtl_uString* strIdentifier, sal_Bool* pRet)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;

	*pRet = osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier);

	return(ret);
}

/*****************************************************************************/
/* osl_queryDaemonStatus */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_queryDaemonStatus(rtl_uString* strIdentifier, oslDaemonStatus* pStatus)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;
		SERVICE_STATUS	ServiceStatus;
		sal_Bool isInstalled;

		OSL_ASSERT(strIdentifier != NULL);

		/* lookup in registry */
		ret = osl_isDaemonInstalled(strIdentifier, &isInstalled);

		if (isInstalled && (ret == osl_Daemon_ManagerError_NoError))
		{
		    // Open the Service Control Manager
		    hSCM = OpenSCManager(NULL, 						// local machine
					 			 NULL, 						// ServicesActive database
					 			 SC_MANAGER_ALL_ACCESS); 	// full access
		    if (hSCM) 
		    {
				// Try to open the service
				hService = OpenServiceW(hSCM,
								       strIdentifier->buffer,
								       SERVICE_QUERY_STATUS);
				if (hService) 
				{
					if (QueryServiceStatus(hService, &ServiceStatus))
					{
						ret = osl_Daemon_ManagerError_NoError;

						switch (ServiceStatus.dwCurrentState)
						{
							case SERVICE_STOPPED:
								*pStatus = osl_Daemon_StatusStopped;
								break;
							case SERVICE_START_PENDING:
								*pStatus = osl_Daemon_StatusStartPending;						
								break;
							case SERVICE_STOP_PENDING:
								*pStatus = osl_Daemon_StatusStopPending;						
								break;
							case SERVICE_RUNNING:
								*pStatus = osl_Daemon_StatusRunning;
								break;
							case SERVICE_CONTINUE_PENDING:
								*pStatus = osl_Daemon_StatusStartPending;						
								break;
							case SERVICE_PAUSE_PENDING:
								*pStatus = osl_Daemon_StatusPausePending;						
								break;
							case SERVICE_PAUSED:
								*pStatus = osl_Daemon_StatusPaused;						
								break;
							default:
								*pStatus = osl_Daemon_StatusUnknown;						
								break;
						}
					}
					else
					{
						ret = osl_Daemon_ManagerError_Security;
					}

			    	CloseServiceHandle(hService);
				}
				else
				{
					ret = osl_Daemon_ManagerError_NotInstalled;
				}

				CloseServiceHandle(hSCM);
		    }
			else
			{
				ret = osl_Daemon_ManagerError_NoManager;
			}
		}
	}
	else
	{
		ret	= osl_default_queryDaemonStatus(strIdentifier, pStatus);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_queryDaemonInfo */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_queryDaemonInfo(rtl_uString* strIdentifier, oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
	sal_Unicode	buffer[2048];

	OSL_ASSERT(strIdentifier != NULL);

	/* lookup in registry */
	if (!osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		ret	= osl_Daemon_ManagerError_NotInstalled;
	}
	else
	{
		rtl_uString *strEntry = NULL;

		rtl_uString_newFromAscii(&strEntry, SERVICES_IDENTIFIER_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														buffer,
														OSL_DAEMON_IDENTIFIER_LEN,
														strIdentifier->buffer);
		rtl_uString_newFromStr(&pInfo->Identifier, buffer);

		rtl_uString_newFromAscii(&strEntry, SERVICES_DISPLAY_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														buffer,
														OSL_DAEMON_NAME_LEN,
														L"");		
		rtl_uString_newFromStr(&pInfo->DisplayName, buffer);

		rtl_uString_newFromAscii(&strEntry, SERVICES_MANUFACTOR_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														buffer,
														OSL_DAEMON_MANUFACTOR_LEN,
														L"");		
		rtl_uString_newFromStr(&pInfo->Manufactor, buffer);

		rtl_uString_newFromAscii(&strEntry, SERVICES_PATH_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														buffer,
														OSL_DAEMON_PATH_LEN,
														L"");		
		rtl_uString_newFromStr(&pInfo->Path, buffer);

		rtl_uString_newFromAscii(&strEntry, SERVICES_DEPENDENCIES_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														pInfo->Dependencies,
														OSL_DAEMON_DEPENDENCIES_LEN,
														L"");		

		rtl_uString_newFromAscii(&strEntry, SERVICES_VERSION_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														buffer,
														MAX_DWORD_2_STRING_BUFFER,
														L"0");		
		pInfo->Version = rtl_ustr_toInt32(buffer, 10);

		rtl_uString_newFromAscii(&strEntry, SERVICES_MODE_NAME);
		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
														NULL, 
														strEntry,
														buffer,
														OSL_DAEMON_NAME_LEN,
														L"on demand");		

		if (wcscmp(buffer, L"automatic") == 0)
			pInfo->Mode = osl_Daemon_Automatic;
		else
			pInfo->Mode = osl_Daemon_OnDemand;

		rtl_uString_release(strEntry);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_startDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_startDaemon(rtl_uString* strIdentifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;
		sal_Bool isInstalled;

		OSL_ASSERT(strIdentifier != NULL);

		/* lookup in registry */
		ret = osl_isDaemonInstalled(strIdentifier, &isInstalled);

		if (isInstalled && (ret == osl_Daemon_ManagerError_NoError))
		{
		    // Open the Service Control Manager
		    hSCM = OpenSCManager(NULL, 						// local machine
					 			 NULL, 						// ServicesActive database
					 			 SC_MANAGER_ALL_ACCESS); 	// full access
		    if (hSCM) 
		    {
				// Try to open the service
				hService = OpenServiceW(hSCM,
								        strIdentifier->buffer,
								        SERVICE_START);
				if (hService) 
				{
					if (StartService(hService, 0, NULL))
					{
						ret = osl_Daemon_ManagerError_NoError;
					}
					else
					{
						DWORD error = GetLastError();

						switch (error)
						{
							case ERROR_SERVICE_ALREADY_RUNNING:
								ret = osl_Daemon_ManagerError_AllreadyRunning;
								break;
							case ERROR_ACCESS_DENIED:
							case ERROR_SERVICE_LOGON_FAILED:
								ret = osl_Daemon_ManagerError_Security;
								break;
							case ERROR_SERVICE_DISABLED:
							case ERROR_SERVICE_MARKED_FOR_DELETE:
								ret = osl_Daemon_ManagerError_NotInstalled;
								break;
							case ERROR_SERVICE_REQUEST_TIMEOUT:
								ret = osl_Daemon_ManagerError_Timeout;
								break;
							default:
								ret = osl_Daemon_ManagerError_Unknown;
								break;
						}
					}

			    	CloseServiceHandle(hService);
				}
				else
				{
					ret = osl_Daemon_ManagerError_NotInstalled;
				}

				CloseServiceHandle(hSCM);
		    }
			else
			{
				ret = osl_Daemon_ManagerError_NoManager;
			}
		}
	}
	else
	{
		ret	= osl_default_startDaemon(strIdentifier);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_stopDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_stopDaemon(rtl_uString* strIdentifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;
		SERVICE_STATUS ServiceStatus;
		sal_Bool isInstalled;

		OSL_ASSERT(strIdentifier != NULL);

		/* lookup in registry */
		ret = osl_isDaemonInstalled(strIdentifier, &isInstalled);

		if (isInstalled && (ret == osl_Daemon_ManagerError_NoError))
		{
		    // Open the Service Control Manager
		    hSCM = OpenSCManager(NULL, 						// local machine
					 			 NULL, 						// ServicesActive database
					 			 SC_MANAGER_ALL_ACCESS); 	// full access
		    if (hSCM) 
		    {
				// Try to open the service
				hService = OpenServiceW(hSCM,
								        strIdentifier->buffer,
								        SERVICE_STOP);
				if (hService) 
				{
					if (ControlService(hService, SERVICE_CONTROL_STOP, &ServiceStatus))
					{
						ret = osl_Daemon_ManagerError_NoError;
					}
					else
					{
						DWORD error = GetLastError();

						switch (error)
						{
							case ERROR_SERVICE_NOT_ACTIVE:
								ret = osl_Daemon_ManagerError_NotRunning;
								break;
							case ERROR_ACCESS_DENIED:
							case ERROR_SERVICE_LOGON_FAILED:
								ret = osl_Daemon_ManagerError_Security;
								break;
							case ERROR_SERVICE_DISABLED:
							case ERROR_SERVICE_MARKED_FOR_DELETE:
								ret = osl_Daemon_ManagerError_NotInstalled;
								break;
							case ERROR_SERVICE_REQUEST_TIMEOUT:
								ret = osl_Daemon_ManagerError_Timeout;
								break;
							default:
								ret = osl_Daemon_ManagerError_Unknown;
								break;
						}
					}

			    	CloseServiceHandle(hService);
				}
				else
				{
					ret = osl_Daemon_ManagerError_NotInstalled;
				}

				CloseServiceHandle(hSCM);
		    }
			else
			{
				ret = osl_Daemon_ManagerError_NoManager;
			}
		}
	}
	else
	{
		ret = osl_default_stopDaemon(strIdentifier);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_pauseDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_pauseDaemon(rtl_uString* strIdentifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;
		SERVICE_STATUS ServiceStatus;
		sal_Bool isInstalled;

		OSL_ASSERT(strIdentifier != NULL);

		/* lookup in registry */
		ret = osl_isDaemonInstalled(strIdentifier, &isInstalled);

		if (isInstalled && (ret == osl_Daemon_ManagerError_NoError))
		{
		    // Open the Service Control Manager
		    hSCM = OpenSCManager(NULL, 						// local machine
					 			 NULL, 						// ServicesActive database
					 			 SC_MANAGER_ALL_ACCESS); 	// full access
		    if (hSCM) 
		    {
				// Try to open the service
				hService = OpenServiceW(hSCM,
								        strIdentifier->buffer,
								        SERVICE_PAUSE_CONTINUE);
				if (hService) 
				{
					if (ControlService(hService, SERVICE_CONTROL_PAUSE, &ServiceStatus))
					{
						ret = osl_Daemon_ManagerError_NoError;
					}
					else
					{
						DWORD error = GetLastError();

						switch (error)
						{
							case ERROR_SERVICE_NOT_ACTIVE:
								ret = osl_Daemon_ManagerError_NotRunning;
								break;
							case ERROR_ACCESS_DENIED:
							case ERROR_SERVICE_LOGON_FAILED:
								ret = osl_Daemon_ManagerError_Security;
								break;
							case ERROR_SERVICE_DISABLED:
							case ERROR_SERVICE_MARKED_FOR_DELETE:
								ret = osl_Daemon_ManagerError_NotInstalled;
								break;
							case ERROR_SERVICE_REQUEST_TIMEOUT:
								ret = osl_Daemon_ManagerError_Timeout;
								break;
							default:
								ret = osl_Daemon_ManagerError_Unknown;
								break;
						}
					}

			    	CloseServiceHandle(hService);
				}
				else
				{
					ret = osl_Daemon_ManagerError_NotInstalled;
				}

				CloseServiceHandle(hSCM);
		    }
			else
			{
				ret = osl_Daemon_ManagerError_NoManager;
			}
		}
	}
	else
	{
		ret	= osl_default_pauseDaemon(strIdentifier);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_pauseDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_continueDaemon(rtl_uString* strIdentifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;
		SERVICE_STATUS ServiceStatus;
		sal_Bool isInstalled;

		OSL_ASSERT(strIdentifier != NULL);

		/* lookup in registry */
		ret = osl_isDaemonInstalled(strIdentifier, &isInstalled);

		if (isInstalled && (ret == osl_Daemon_ManagerError_NoError))
		{
		    // Open the Service Control Manager
		    hSCM = OpenSCManager(NULL, 						// local machine
					 			 NULL, 						// ServicesActive database
					 			 SC_MANAGER_ALL_ACCESS); 	// full access
		    if (hSCM) 
		    {
				// Try to open the service
				hService = OpenServiceW(hSCM,
								        strIdentifier->buffer,
								        SERVICE_PAUSE_CONTINUE);
				if (hService) 
				{
					if (ControlService(hService, SERVICE_CONTROL_CONTINUE, &ServiceStatus))
					{
						ret = osl_Daemon_ManagerError_NoError;
					}
					else
					{
						DWORD error = GetLastError();

						switch (error)
						{
							case ERROR_SERVICE_NOT_ACTIVE:
								ret = osl_Daemon_ManagerError_NotRunning;
								break;
							case ERROR_ACCESS_DENIED:
							case ERROR_SERVICE_LOGON_FAILED:
								ret = osl_Daemon_ManagerError_Security;
								break;
							case ERROR_SERVICE_DISABLED:
							case ERROR_SERVICE_MARKED_FOR_DELETE:
								ret = osl_Daemon_ManagerError_NotInstalled;
								break;
							case ERROR_SERVICE_REQUEST_TIMEOUT:
								ret = osl_Daemon_ManagerError_Timeout;
								break;
							default:
								ret = osl_Daemon_ManagerError_Unknown;
								break;
						}
					}

			    	CloseServiceHandle(hService);
				}
				else
				{
					ret = osl_Daemon_ManagerError_NotInstalled;
				}

				CloseServiceHandle(hSCM);
		    }
			else
			{
				ret = osl_Daemon_ManagerError_NoManager;
			}
		}
	}
	else
	{
		ret	= osl_default_continueDaemon(strIdentifier);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_installDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_installDaemon(const oslDaemonInfo* pInfo)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;

	    // Open the Service Control Manager
	    hSCM = OpenSCManager(NULL, 						// local machine
				 			 NULL, 						// ServicesActive database
				 			 SC_MANAGER_ALL_ACCESS); 	// full access

	    if (hSCM) 
	    {
			DWORD startMode = SERVICE_DEMAND_START;

			if (pInfo->Mode == osl_Daemon_Automatic)
				startMode = SERVICE_AUTO_START;

		    /* Create the service */
		    hService = CreateServiceW(hSCM,
					     pInfo->Identifier->buffer,
					     pInfo->DisplayName->buffer,
					     SERVICE_ALL_ACCESS,
					     SERVICE_WIN32_OWN_PROCESS,
					     startMode,
					     SERVICE_ERROR_NORMAL,
					     pInfo->Path->buffer,
					     NULL, NULL, 
					     pInfo->Dependencies, 
					     NULL, NULL);

		    if (! hService) 
		    {
				DWORD error = GetLastError();
		
				switch (error)
				{
					case ERROR_ACCESS_DENIED:
						ret = osl_Daemon_ManagerError_Security;
						break;
					case ERROR_CIRCULAR_DEPENDENCY:
						ret = osl_Daemon_ManagerError_CircularDep;
						break;						
					case ERROR_DUP_NAME:
						ret = osl_Daemon_ManagerError_AllreadyInstalled;
						break;						
					case ERROR_INVALID_HANDLE:
						ret = osl_Daemon_ManagerError_Unknown;
						break;						
					case ERROR_INVALID_NAME:
						ret = osl_Daemon_ManagerError_Unknown;
						break;						
					case ERROR_INVALID_PARAMETER:
						ret = osl_Daemon_ManagerError_Unknown;
						break;						
					case ERROR_INVALID_SERVICE_ACCOUNT:
						ret = osl_Daemon_ManagerError_Unknown;
						break;						
					case ERROR_SERVICE_EXISTS:
						ret = osl_Daemon_ManagerError_AllreadyInstalled;
						break;						
					default:
						ret = osl_Daemon_ManagerError_Unknown;
						break;
				}
		    }
			else
			{
				sal_Unicode buffer[MAX_DWORD_2_STRING_BUFFER];
				memset( buffer, 0, sizeof( buffer ));

				if (osl_getDaemonWindowsRegistry()->addDaemon(pInfo->Identifier, pInfo->DisplayName))
				{
					rtl_uString *strEntry = NULL;
					rtl_uString *strValue = NULL;

					rtl_uString_newFromAscii(&strEntry, SERVICES_IDENTIFIER_NAME);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																	 NULL,
																	 strEntry,
																	 pInfo->Identifier->buffer);

					rtl_uString_newFromAscii(&strEntry, SERVICES_DISPLAY_NAME);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																	 NULL,
																	 strEntry,
																	 pInfo->DisplayName->buffer);

					rtl_uString_newFromAscii(&strEntry, SERVICES_MANUFACTOR_NAME);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																	 NULL,
																	 strEntry,
																	 pInfo->Manufactor->buffer);

					rtl_uString_newFromAscii(&strEntry, SERVICES_PATH_NAME);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																	 NULL,
																	 strEntry,
																	 pInfo->Path->buffer);

					rtl_uString_newFromAscii(&strEntry, SERVICES_DEPENDENCIES_NAME);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																	 NULL,
																	 strEntry,
																	 pInfo->Dependencies);

					rtl_ustr_valueOfInt32(buffer, pInfo->Version, 10);
					rtl_uString_newFromAscii(&strEntry, SERVICES_VERSION_NAME);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																	 NULL,
																	 strEntry,
																	 buffer);

					rtl_uString_newFromAscii(&strEntry, SERVICES_MODE_NAME);
					rtl_uString_newFromAscii(&strValue, (pInfo->Mode == osl_Daemon_Automatic) ? SERVICES_MODE_AUTOMATIC : SERVICES_MODE_ONDEMAND);
					osl_getDaemonWindowsRegistry()->writeDaemonEntry(pInfo->Identifier, 
																		 NULL,
																		 strEntry,
																		 strValue->buffer);

					rtl_uString_release(strValue);
					rtl_uString_release(strEntry);
				}

			    ret = osl_Daemon_ManagerError_NoError;
			    CloseServiceHandle(hService);
			}

		    CloseServiceHandle(hSCM);
		}
		else
			ret = osl_Daemon_ManagerError_NoManager;
	}
	else
	{
		ret	= osl_default_installDaemon(pInfo);
	}

	return(ret);
	
}	

/*****************************************************************************/
/* osl_uninstallDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_uninstallDaemon(rtl_uString* strIdentifier)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	if (useNTServices())
	{
		SC_HANDLE hSCM, hService;

		OSL_ASSERT(strIdentifier != NULL);


		// Open the Service Control Manager
		hSCM = OpenSCManager(NULL, 						// local machine
			 				 NULL, 						// ServicesActive database
			 				 SC_MANAGER_ALL_ACCESS); 	// full access

		if (hSCM)
		{
			hService = OpenServiceW(hSCM, strIdentifier->buffer, DELETE);

			if (hService) 
			{
				if (DeleteService(hService))
				{
					osl_getDaemonWindowsRegistry()->removeDaemon(strIdentifier);
					ret = osl_Daemon_ManagerError_NoError;
				}
				else
				{
					DWORD error = GetLastError();
				
					switch (error)
					{
						case ERROR_ACCESS_DENIED:
							ret = osl_Daemon_ManagerError_Security;
							break;
						case ERROR_INVALID_HANDLE:
							ret = osl_Daemon_ManagerError_Unknown;
							break;
						case ERROR_SERVICE_MARKED_FOR_DELETE:
							ret = osl_Daemon_ManagerError_NotInstalled;
							break;						
						default:
							ret = osl_Daemon_ManagerError_Unknown;
							break;
					}
				}

				CloseServiceHandle(hService);
    		}
			else
			{
				DWORD error = GetLastError();
			
				switch (error)
				{
					case ERROR_ACCESS_DENIED:
						ret = osl_Daemon_ManagerError_Security;
						break;
					case ERROR_INVALID_HANDLE:
						ret = osl_Daemon_ManagerError_Unknown;
						break;
					case ERROR_INVALID_NAME:
						ret = osl_Daemon_ManagerError_Unknown;
						break;
					case ERROR_SERVICE_DOES_NOT_EXIST:
						ret = osl_Daemon_ManagerError_NotInstalled;
						break;						
					default:
						ret = osl_Daemon_ManagerError_Unknown;
						break;
				}
			}
	    		CloseServiceHandle(hSCM);
		}
	}
	else
	{
		ret	= osl_default_uninstallDaemon(strIdentifier);
	}

    return (ret);
}	

/*****************************************************************************/
/* osl_createDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_createDaemon(rtl_uString* strIdentifier,
						   		  	   oslDaemonControl pControl, 
						  		  	   void* pControlData,
									   oslDaemonPropertyAccess* pPropertyAccess,
						  		  	   void* pAccessData,
						  		  	   oslDaemonHandle* pHandle)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		osl_TDaemonImpl* pDaemonImpl = (osl_TDaemonImpl*)malloc(sizeof(osl_TDaemonImpl));

		OSL_ASSERT(pDaemonImpl);

		memset(pDaemonImpl, 0, sizeof(osl_TDaemonImpl));

		pDaemonImpl->m_DaemonControl  = pControl;
		pDaemonImpl->m_pControlData= pControlData;
		pDaemonImpl->m_pPropertyAccess= pPropertyAccess;
		pDaemonImpl->m_pAccessData= pAccessData;
		pDaemonImpl->m_bStopped = sal_True;
		pDaemonImpl->m_stopCond = osl_createCondition();


	    // set up the initial service status
	    pDaemonImpl->m_hStatus = 0;
	    pDaemonImpl->m_Status.dwServiceType      = SERVICE_WIN32_OWN_PROCESS;
	    pDaemonImpl->m_Status.dwCurrentState     = SERVICE_STOPPED;
	    pDaemonImpl->m_Status.dwControlsAccepted = SERVICE_ACCEPT_STOP |  
						       SERVICE_ACCEPT_PAUSE_CONTINUE | 
						       SERVICE_ACCEPT_SHUTDOWN;
	    pDaemonImpl->m_Status.dwWin32ExitCode    = 0;
	    pDaemonImpl->m_Status.dwServiceSpecificExitCode = 0;
	    pDaemonImpl->m_Status.dwCheckPoint       = 0;
	    pDaemonImpl->m_Status.dwWaitHint         = 0;

		pDaemonImpl->m_info.Size = sizeof(pDaemonImpl->m_info);

		osl_queryDaemonInfo(strIdentifier, &(pDaemonImpl->m_info));

		OSL_ASSERT(pService == NULL);

		*pHandle = pDaemonImpl;

		ret = osl_Daemon_ManagerError_NoError;
	}
	else
	{
		ret	= osl_default_createDaemon(strIdentifier,
						   		  	   pControl, 
						  		  	   pControlData,
									   pPropertyAccess,
						  		  	   pAccessData,
						  		  	   pHandle);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_executeDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_executeDaemon(oslDaemonHandle Daemon)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)Daemon;
	    SERVICE_TABLE_ENTRYW st[] = {
			{pDaemonImpl->m_info.Identifier->buffer, oslDaemonWrapperFunction},
			{NULL, NULL}
	    };

		pService = pDaemonImpl;

	    if (StartServiceCtrlDispatcherW(st))
	    {
			ret	= osl_Daemon_ManagerError_NoError;
		}
		else
		{
			DWORD error = GetLastError();

			switch (error)
			{
				case ERROR_INVALID_DATA:
					ret = osl_Daemon_ManagerError_Unknown;
					break;
				default:
					ret = osl_Daemon_ManagerError_Unknown;
					break;
			}
		}
	}
	else
	{
		W95_RegisterServiceProcess(0, 1);
		ret	= osl_default_executeDaemon(Daemon);
		W95_RegisterServiceProcess(0, 0);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_executeDaemons */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_executeDaemons(oslDaemonHandle* pHandle, sal_uInt32 numOfDaemons)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		ret = osl_Daemon_ManagerError_Unknown;
	}
	else
	{
		W95_RegisterServiceProcess(0, 1);
		ret = osl_default_executeDaemons(pHandle, numOfDaemons);
		W95_RegisterServiceProcess(0, 0);
	}

	return(ret);
	
}	

/*****************************************************************************/
/* osl_setDaemonStatus */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_setDaemonStatus(oslDaemonHandle Daemon, oslDaemonStatus Status)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)Daemon;

		OSL_ASSERT(pDaemonImpl != NULL);

		switch (Status)
		{
			case osl_Daemon_StatusStartPending:
			    pDaemonImpl->m_Status.dwCurrentState = SERVICE_START_PENDING;
				break;

			case osl_Daemon_StatusStopPending:
			    pDaemonImpl->m_Status.dwCurrentState = SERVICE_STOP_PENDING;
				break;

			case osl_Daemon_StatusPausePending:
			    pDaemonImpl->m_Status.dwCurrentState = SERVICE_PAUSE_PENDING;
				break;

			case osl_Daemon_StatusRunning:
			    pDaemonImpl->m_Status.dwCurrentState = SERVICE_RUNNING;
				break;

			case osl_Daemon_StatusStopped:
			    pDaemonImpl->m_Status.dwCurrentState = SERVICE_STOPPED;
				osl_setCondition(pDaemonImpl->m_stopCond);
				break;

			case osl_Daemon_StatusPaused:
			    pDaemonImpl->m_Status.dwCurrentState = SERVICE_PAUSED;
				break;
		}
		    
	    if (SetServiceStatus(pDaemonImpl->m_hStatus, &pDaemonImpl->m_Status))
	    {
			ret	= osl_Daemon_ManagerError_NoError;
		}
		else
		{
			DWORD error = GetLastError();

			switch (error)
			{
				case ERROR_INVALID_DATA:
					ret = osl_Daemon_ManagerError_Unknown;
					break;
				case ERROR_INVALID_HANDLE:
					ret = osl_Daemon_ManagerError_Unknown;
					break;
				default:
					ret = osl_Daemon_ManagerError_Unknown;
					break;
			}
		}
	}
	else
	{
		ret	= osl_default_setDaemonStatus(Daemon, Status);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_endDaemon */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_endDaemon(oslDaemonHandle handle)
{
	if (useNTServices())
	{
		osl_TDaemonImpl* pDaemonImpl= (osl_TDaemonImpl*)handle;

		OSL_ASSERT(pDaemonImpl != NULL);

		osl_destroyCondition(pDaemonImpl->m_stopCond);
		free(pDaemonImpl);

		pService = NULL;

		return(osl_Daemon_ManagerError_NoError);
	}
	else
	{
		return(osl_default_endDaemon(handle));
	}
}	


/*****************************************************************************/
/* osl_getDaemonPropertyCount */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_getDaemonPropertyCount(rtl_uString* strIdentifier, sal_uInt32* pCount)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		oslDaemonConnection	conn = NULL;
		oslDaemonMgrMsg* pRequest = NULL;

		conn = osl_openDaemonMgrConn(strIdentifier, osl_DaemonConnection_OPEN);

		if (conn != NULL)
		{
			pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_PropertyCountGet);

			pRequest->Type = osl_DaemonMgrMsgType_Request;

			ret = osl_sendDaemonMgrMsg(conn, pRequest);
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
			
			if (ret == osl_Daemon_ManagerError_NoError)
				ret = osl_recvDaemonMgrMsg(conn, &pRequest);

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				ret = pRequest->Error;

				if (ret == osl_Daemon_ManagerError_NoError)
				{
					*pCount = pRequest->Body.PropertyCount;
				}

				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}

			osl_disconnectDaemonMgrConn(conn);
		}
	}
	else
	{
		ret	= osl_default_getDaemonPropertyCount(strIdentifier, pCount);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_getDaemonProperties */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_getDaemonProperties(rtl_uString* strIdentifier, 
											  oslDaemonProperty** ppProps, 
											  sal_uInt32* pCount)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		oslDaemonConnection conn = NULL;
		oslDaemonMgrMsg* pRequest = NULL;

		conn = osl_openDaemonMgrConn(strIdentifier, osl_DaemonConnection_OPEN);

		if (conn != NULL)
		{
			pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_PropertiesGet);

			pRequest->Type = osl_DaemonMgrMsgType_Request;

			ret = osl_sendDaemonMgrMsg(conn, pRequest);
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
			
			if (ret == osl_Daemon_ManagerError_NoError)
				ret = osl_recvDaemonMgrMsg(conn, &pRequest);

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				ret = pRequest->Error;

				if (ret == osl_Daemon_ManagerError_NoError)
				{
					*pCount = pRequest->Body.Properties.Count;
					*ppProps = pRequest->Body.Properties.List;

					/* do not destroy recieved property list */
					pRequest->Body.Properties.Count = 0;
					pRequest->Body.Properties.List = NULL;
				}

				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}

			osl_disconnectDaemonMgrConn(conn);
		}
	}
	else
	{
		ret	= osl_default_getDaemonProperties(strIdentifier, 
											  ppProps, 
											  pCount);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_setDaemonProperties */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_setDaemonProperties(rtl_uString* strIdentifier, 
											  oslDaemonProperty* pProps, 
											  sal_uInt32 count, 
											  sal_uInt32* pErrorCode, 
											  rtl_uString** strErrorText)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NoManager;

	if (useNTServices())
	{
		oslDaemonConnection conn = NULL;
		oslDaemonMgrMsg* pRequest = NULL;

		conn = osl_openDaemonMgrConn(strIdentifier, osl_DaemonConnection_OPEN);

		if (conn != NULL)
		{
			pRequest = osl_createDaemonMgrMsg(strIdentifier, osl_DaemonMgrMsgSubject_PropertiesSet);

			pRequest->Type = osl_DaemonMgrMsgType_Request;
			pRequest->Body.Properties.List = pProps;
			pRequest->Body.Properties.Count = count;
					
			ret = osl_sendDaemonMgrMsg(conn, pRequest);

			/* do not destroy property list */
			pRequest->Body.Properties.List = NULL;
			pRequest->Body.Properties.Count = 0;
			osl_destroyDaemonMgrMsg(pRequest);
			pRequest = NULL;
			
			if (ret == osl_Daemon_ManagerError_NoError)
				ret = osl_recvDaemonMgrMsg(conn, &pRequest);

			if (ret == osl_Daemon_ManagerError_NoError)
			{
				ret = pRequest->Error;

				*pErrorCode = pRequest->Body.Properties.ErrorCode;

				if (pRequest->Body.Properties.ErrorText != NULL)
				{
					rtl_uString_assign(strErrorText, pRequest->Body.Properties.ErrorText);
				}

				osl_destroyDaemonMgrMsg(pRequest);
				pRequest = NULL;
			}

			osl_disconnectDaemonMgrConn(conn);
		}
	}
	else
	{
		ret	= osl_default_setDaemonProperties(strIdentifier, 
											  pProps, 
											  count, 
											  pErrorCode, 
											  strErrorText);
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_getDaemonProfileSections */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_getDaemonProfileSections(rtl_uString* strIdentifier, 
															sal_Unicode* buffer, 
															sal_uInt32   bufferSize,
															sal_uInt32*  pRequiredSize)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		*pRequiredSize = osl_getDaemonWindowsRegistry()->getSections(strIdentifier, 
																	 buffer, 
																	 bufferSize);
		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_getDaemonProfileEntries */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_getDaemonProfileEntries(rtl_uString* strIdentifier, 
                                                           rtl_uString* strSection, 
                                                           sal_Unicode* buffer, 
                                                           sal_uInt32   bufferSize,
                                                           sal_uInt32*  pRequiredSize)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		*pRequiredSize = osl_getDaemonWindowsRegistry()->getEntries(strIdentifier, 
																	strSection,
																	buffer, 
																	bufferSize);
		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_readDaemonProfileString */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_readDaemonProfileString(rtl_uString* strIdentifier, 
                                                           rtl_uString*  strSection, 
                                                           rtl_uString*  strEntry,
                                                           rtl_uString** strValue, 
                                                           rtl_uString*  strDefaultValue)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		sal_Unicode buffer[2048];

		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
												   		strSection,
														strEntry,
												   		buffer, 
												   		2048,
												   		strDefaultValue->buffer);
		rtl_uString_newFromStr(strValue, buffer);
		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_readDaemonProfileBool */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_readDaemonProfileBool(rtl_uString* strIdentifier, 
                                                         rtl_uString* strSection, 
                                                         rtl_uString* strEntry,
														 sal_Bool* pValue, 
														 sal_Bool defaultValue)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		sal_Unicode buffer[MAX_DWORD_2_STRING_BUFFER];
		sal_Unicode defaultBuffer[MAX_DWORD_2_STRING_BUFFER];

		rtl_ustr_valueOfInt32(buffer, defaultValue, 10);

		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
												   		strSection,
														strEntry,
												   		buffer, 
												   		MAX_DWORD_2_STRING_BUFFER,
												   		defaultBuffer);

		*pValue = (rtl_ustr_toInt32(buffer, 10) != 0);

		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_readDaemonProfileIdent */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_readDaemonProfileIdent(rtl_uString* strIdentifier, 
                                                          rtl_uString* strSection, 
                                                          rtl_uString* strEntry,
                                                          sal_uInt32*  pValue, 
                                                          sal_uInt32   firstId, 
                                                          rtl_uString* strStrings[],
                                                          sal_uInt32   defaultValue)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		sal_Unicode buffer[MAX_DWORD_2_STRING_BUFFER];
		sal_uInt32 i = 0;

		osl_getDaemonWindowsRegistry()->readDaemonEntry(strIdentifier, 
										   			    strSection,
														strEntry,
												   		buffer, 
												   		MAX_DWORD_2_STRING_BUFFER,
												   		strStrings[defaultValue - firstId]->buffer);

		while ((strStrings[i] != NULL) && (wcscmp(strStrings[i]->buffer, buffer) != 0))
		{
			i++;
		}

		if (strStrings[i] != NULL)
		{
			*pValue = firstId + i;
		}
		else
		{
			*pValue = defaultValue;
		}

		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_writeDaemonProfileString */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_writeDaemonProfileString(rtl_uString* strIdentifier, 
                                                            rtl_uString* strSection, 
                                                            rtl_uString* strEntry,
                                                            rtl_uString* strValue)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		osl_getDaemonWindowsRegistry()->writeDaemonEntry(strIdentifier, 
												   		 strSection,
														 strEntry,
												   		 strValue->buffer);

		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_writeDaemonProfileBool */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_writeDaemonProfileBool(rtl_uString* strIdentifier, 
                                                          rtl_uString* strSection, 
                                                          rtl_uString* strEntry,
														  sal_Bool value)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		sal_Unicode buffer[MAX_DWORD_2_STRING_BUFFER];

		rtl_ustr_valueOfInt32(buffer, value, 10);
		osl_getDaemonWindowsRegistry()->writeDaemonEntry(strIdentifier, 
													   	 strSection,
														 strEntry,
													   	 buffer);
		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_writeDaemonProfileIdent */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_writeDaemonProfileIdent(rtl_uString* strIdentifier, 
                                                           rtl_uString* strSection, 
                                                           rtl_uString* strEntry,
                                                           sal_uInt32   value, 
                                                           sal_uInt32   firstId, 
                                                           rtl_uString* strStrings[])
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_Unknown;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		osl_getDaemonWindowsRegistry()->writeDaemonEntry(strIdentifier, 
													   	 strSection,
														 strEntry,
													   	 strStrings[value + firstId]->buffer);

		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}	

/*****************************************************************************/
/* osl_removeDaemonProfileEntry */
/*****************************************************************************/
oslDaemonManagerError SAL_CALL osl_removeDaemonProfileEntry(rtl_uString* strIdentifier, 
                                                            rtl_uString* strSection, 
                                                            rtl_uString* strEntry)
{
	oslDaemonManagerError ret = osl_Daemon_ManagerError_NotInstalled;

	if (osl_getDaemonWindowsRegistry()->hasDaemon(strIdentifier))
	{
		osl_getDaemonWindowsRegistry()->removeDaemonEntry(strIdentifier, 
												   		  strSection,
														  strEntry);

		ret = osl_Daemon_ManagerError_NoError;
	}

	return(ret);
}


oslDaemonManagerError SAL_CALL osl_queryDaemonChanges(TimeValue* pSince, sal_Bool* pBChanged)
{
	*pBChanged = sal_True;
	return(osl_Daemon_ManagerError_NoError);
}	


