/*************************************************************************
 *
 *  $RCSfile: daemregw.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define UNICODE
#include "system.h"
#include <systools/win32/advapi9x.h>

#ifndef _OSL_DEAMREGW_H_
#   include "daemregw.h"
#endif

#include <osl/diagnose.h>
#include <rtl/alloc.h>

#define SERVICES_ROOT_KEY_NAME 		L"Software\\Sun Microsystems\\Services"

#define SERVICES_MAX_SECTION_LEN	128
#define SERVICES_MAX_ENTRY_LEN		128

/*****************************************************************************/
/* DeleteRegKey */
/*****************************************************************************/

static void DeleteRegKey(HKEY hRoot, LPCTSTR pSubKeyName)   
{
	if (pSubKeyName && wcslen(pSubKeyName))      
	{
		HKEY hKey;      

		if (RegOpenKeyEx(hRoot, pSubKeyName, 0, KEY_ENUMERATE_SUB_KEYS | DELETE, &hKey) 
			== ERROR_SUCCESS)
		{
			TCHAR   SubKey[512]; 
			DWORD   dwSubKey;

			while (TRUE)            
			{
				dwSubKey = sizeof(SubKey);
			
				if (RegEnumKeyEx(hKey, 0, SubKey, &dwSubKey, NULL, NULL, NULL, NULL) 
				    == ERROR_SUCCESS)
					DeleteRegKey(hKey, SubKey);            
				else
				{
					RegDeleteKey(hRoot, pSubKeyName);
					break;               
				}
			 }

			 RegCloseKey(hKey);
		 }
	}
}

/*****************************************************************************/
/* windows_hasDaemon */
/*****************************************************************************/

static sal_Bool SAL_CALL windows_hasDaemon(rtl_uString *strIdentifier)
{
	rtl_uString* strKeyName = NULL;
	sal_Bool	 ret	= sal_False;
	HKEY hServiceKey;

	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) +  
		sizeof(sal_Unicode) * (strIdentifier->length + 1)
	);

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	/* lookup in registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_READ, &hServiceKey) == ERROR_SUCCESS)
	{
		ret = sal_True;
		RegCloseKey(hServiceKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* windows_getDaemons */
/*****************************************************************************/

static sal_uInt32 SAL_CALL windows_getDaemons(sal_Unicode* pszBuffer, sal_uInt32 MaxLen)
{
	sal_uInt32 ret = 2;
	HKEY hRootKey;
	DWORD index = 0;
	sal_Unicode buffer[OSL_DAEMON_IDENTIFIER_LEN];
	DWORD sizeOfBuffer = OSL_DAEMON_IDENTIFIER_LEN;
	sal_Unicode* pChar = pszBuffer;
	sal_uInt32 sizeLeft = MaxLen - (pChar - pszBuffer);

	if (MaxLen >= 2)
	{
		pszBuffer[0] = '\0';		
		pszBuffer[1] = '\0';		
	}

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, SERVICES_ROOT_KEY_NAME, 0, KEY_READ, &hRootKey) == ERROR_SUCCESS)
	{

		while ((RegEnumKeyEx(
		    	hRootKey,		/* handle of key to enumerate */
		    	index,			/* index of subkey to enumerate */
		    	buffer,			/* address of buffer for subkey name */
		    	&sizeOfBuffer,	/* address for size of subkey buffer */
		    	NULL,			/* reserved */
		    	NULL,			/* address of buffer for class string */
		    	NULL,			/* address for size of class buffer */
		    	NULL) 			/* address for time key last written to */
			== ERROR_SUCCESS)
		   )
		{
			ret	+= wcslen(buffer) + 1;

			if (MaxLen >= 2)
			{
				wcsncpy(pChar, buffer, sizeLeft);
				pChar[sizeLeft - 1] = L'\0';
				pChar += wcslen(pChar) + 1;
			}
			
			sizeLeft = MaxLen - (pChar - pszBuffer);

			index++;

			sizeOfBuffer = OSL_DAEMON_IDENTIFIER_LEN;
		}

		RegCloseKey(hRootKey);
	}

	return(ret);
}	

/*****************************************************************************/
/* windows_addDaemon */
/*****************************************************************************/

static sal_Bool SAL_CALL windows_addDaemon(rtl_uString *strIdentifier, rtl_uString* strDisplayName)
{
	sal_Bool	ret	= sal_False;
	HKEY hServiceKey;
	DWORD disposition = 0;
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * (strIdentifier->length + 1)
	);

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	/* lookup in registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_WRITE, &hServiceKey) != ERROR_SUCCESS)
	{
		if (RegCreateKeyEx(
			    HKEY_LOCAL_MACHINE,			/* handle of an open key */
			    keyName,					/* address of subkey name */
			    0,							/* reserved */
			    NULL,						/* address of class string */
			    REG_OPTION_NON_VOLATILE,	/* special options flag */
			    KEY_ALL_ACCESS,				/* desired security access */
			    NULL,						/* address of key security structure */
			    &hServiceKey,				/* address of buffer for opened handle */
			    &disposition 				/* address of disposition value buffer */
			   ) == ERROR_SUCCESS)
		{
			ret = sal_True;
			RegCloseKey(hServiceKey);
		}
	}
	else
	{
		RegCloseKey(hServiceKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* windows_removeDaemon */
/*****************************************************************************/

static void SAL_CALL windows_removeDaemon(rtl_uString *strIdentifier)
{
	HKEY hServiceKey;
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * (strIdentifier->length + 1));

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	/* lookup in registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_WRITE, &hServiceKey) == ERROR_SUCCESS)
	{
		RegCloseKey(hServiceKey);
		DeleteRegKey(HKEY_LOCAL_MACHINE, keyName);
	}

	rtl_freeMemory(keyName);
}	

/*****************************************************************************/
/* windows_readDaemonEntry */
/*****************************************************************************/

static sal_Bool SAL_CALL windows_readDaemonEntry(rtl_uString* strIdentifier,	
									    rtl_uString* strSection, 
										rtl_uString* strEntry, 
										sal_Unicode* pszString, 
										sal_uInt32 	 MaxLen, 
										const sal_Unicode* pszDefault)
{
	sal_Bool	ret	= sal_False;
	HKEY hServiceKey, hSectionKey;
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * (strIdentifier->length + 1)
	);
	DWORD bufferSize = MaxLen * sizeof(sal_Unicode);
	DWORD valueType;

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	/* lookup in registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_READ, &hServiceKey) == ERROR_SUCCESS)
	{
		ret = sal_True;

		wcsncpy(pszString, pszDefault, MaxLen);
		pszString[MaxLen - 1] = '\0';

		if (strSection == NULL)
		{
			if ((RegQueryValueEx(
			    	hServiceKey,				/* handle of key to query */
			    	strEntry->buffer,			/* address of name of value to query */
			    	NULL,						/* reserved */
			    	&valueType,					/* address of buffer for value type */
			    	(LPBYTE) pszString,			/* address of data buffer */
			    	&bufferSize) 				/* address of data buffer size */
				!= ERROR_SUCCESS) ||
				(valueType != REG_SZ)
			   )
			{
				wcsncpy(pszString, pszDefault, MaxLen);
				pszString[MaxLen - 1] = '\0';
			}
		}
		else
		{
			if (RegOpenKeyEx(hServiceKey, strSection->buffer, 0, KEY_READ, &hSectionKey) == ERROR_SUCCESS)
			{
				if ((RegQueryValueEx(
				    	hSectionKey,				/* handle of key to query */
				    	strEntry->buffer,			/* address of name of value to query */
				    	NULL,						/* reserved */
				    	&valueType,					/* address of buffer for value type */
				    	(LPBYTE) pszString,			/* address of data buffer */
				    	&bufferSize) 				/* address of data buffer size */
					!= ERROR_SUCCESS) ||
					(valueType != REG_SZ)
				   )
				{
					wcsncpy(pszString, pszDefault, MaxLen);
					pszString[MaxLen - 1] = '\0';
				}

				RegCloseKey(hSectionKey);
			}
		}
						
		RegCloseKey(hServiceKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* windows_writeDaemonEntry */
/*****************************************************************************/

static sal_Bool SAL_CALL windows_writeDaemonEntry(rtl_uString* strIdentifier,	
									     rtl_uString* strSection, 
									     rtl_uString* strEntry, 
			                             const sal_Unicode* pszString)
{
	sal_Bool	ret	= sal_False;
	HKEY hServiceKey, hSectionKey;
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * (strIdentifier->length + 1)
	);
	DWORD bufferSize = (wcslen(pszString) + 1) * sizeof(sal_Unicode);
	DWORD disposition;

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	/* lookup in registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_WRITE, &hServiceKey) == ERROR_SUCCESS)
	{
		if (strSection == NULL)
		{
			if (RegSetValueEx(
				    hServiceKey,					/* handle of key to set value for */
				    strEntry->buffer,				/* address of value to set */
				    0,								/* reserved */
				    REG_SZ,							/* flag for value type */
				    (PBYTE) pszString,				/* address of value data */
				    bufferSize						/* size of value data */
			   	) == ERROR_SUCCESS)
			{
				ret = sal_True;
			}
		}
		else
		{
			if (RegCreateKeyEx(
				    hServiceKey,				/* handle of an open key */
				    strSection->buffer,			/* address of subkey name */
				    0,							/* reserved */
				    NULL,						/* address of class string */
				    REG_OPTION_NON_VOLATILE,	/* special options flag */
				    KEY_ALL_ACCESS,				/* desired security access */
				    NULL,						/* address of key security structure */
				    &hSectionKey,				/* address of buffer for opened handle */
				    &disposition 				/* address of disposition value buffer */
				   ) == ERROR_SUCCESS)
			{
				if (RegSetValueEx(
					    hSectionKey,					/* handle of key to set value for */
					    strEntry->buffer,				/* address of value to set */
					    0,								/* reserved */
					    REG_SZ,							/* flag for value type */
					    (PBYTE) pszString,				/* address of value data */
					    bufferSize						/* size of value data */
				   	) == ERROR_SUCCESS)
				{
					ret = sal_True;
				}

				RegCloseKey(hSectionKey);
			}
		}
					
		RegCloseKey(hServiceKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* windows_removeDaemonEntry */
/*****************************************************************************/

static sal_Bool SAL_CALL windows_removeDaemonEntry(rtl_uString *strIdentifier,	
									     rtl_uString *strSection, 
									     rtl_uString *strEntry)
{
	sal_Bool	ret	= sal_False;
	HKEY hServiceKey, hSectionKey;
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * (strIdentifier->length + 1));
	DWORD dwValues;
	DWORD dwSubKeys;

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	/* lookup in registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_WRITE, &hServiceKey) == ERROR_SUCCESS)
	{
		if (strSection == NULL)
		{
			ret = (RegDeleteValue(hServiceKey, strEntry->buffer) == ERROR_SUCCESS);
		}
		else
		{
			if (RegOpenKeyEx(hServiceKey, strSection->buffer, 0, KEY_WRITE, &hSectionKey) == ERROR_SUCCESS)
			{
				ret = (RegDeleteValue(hSectionKey, strEntry->buffer) == ERROR_SUCCESS);

				if ((RegQueryInfoKeyA(hSectionKey, NULL, NULL, NULL, 
					                 &dwSubKeys, NULL, NULL, 
									 &dwValues, NULL, NULL, NULL, NULL) == ERROR_SUCCESS) &&
					(dwSubKeys == 0) && (dwValues == 0))
					RegDeleteKey(hServiceKey, strSection->buffer);
				 
				RegCloseKey(hSectionKey);
			}
		}

		if ((RegQueryInfoKeyA(hServiceKey, NULL, NULL, NULL, 
			                 &dwSubKeys, NULL, NULL, 
							 &dwValues, NULL, NULL, NULL, NULL) == ERROR_SUCCESS) &&
			(dwSubKeys == 0) && (dwValues == 0))
			RegDeleteKey(HKEY_LOCAL_MACHINE, keyName);

		RegCloseKey(hServiceKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* windows_getSections */
/*****************************************************************************/

static sal_uInt32 SAL_CALL windows_getSections(rtl_uString* strIdentifier,
								  	  sal_Unicode* pszBuffer, 
									  sal_uInt32   MaxLen)
{
	sal_uInt32 ret = 2;
	HKEY hServiceKey;
	DWORD index = 0;
	sal_Unicode buffer[SERVICES_MAX_SECTION_LEN];
	DWORD sizeOfBuffer = SERVICES_MAX_SECTION_LEN;
	sal_Unicode* pChar = pszBuffer;
	sal_uInt32 sizeLeft = MaxLen - (pChar - pszBuffer);
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * (strIdentifier->length + 1)
	);

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);

	if (MaxLen >= 2)
	{
		pszBuffer[0] = '\0';		
		pszBuffer[1] = '\0';		
	}

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_READ, &hServiceKey) == ERROR_SUCCESS)
	{

		while ((RegEnumKeyEx(
		    	hServiceKey,	/* handle of key to enumerate */
		    	index,			/* index of subkey to enumerate */
		    	buffer,			/* address of buffer for subkey name */
		    	&sizeOfBuffer,	/* address for size of subkey buffer */
		    	NULL,			/* reserved */
		    	NULL,			/* address of buffer for class string */
		    	NULL,			/* address for size of class buffer */
		    	NULL) 			/* address for time key last written to */
			== ERROR_SUCCESS)
		   )
		{
			ret	+= wcslen(buffer) + 1;

			if (MaxLen >= 2)
			{
				wcsncpy(pChar, buffer, sizeLeft);
				pChar[sizeLeft - 1] = L'\0';
				pChar += wcslen(pChar) + 1;
			}
			
			sizeLeft = MaxLen - (pChar - pszBuffer);

			index++;
			
			sizeOfBuffer = OSL_DAEMON_IDENTIFIER_LEN;
		}

		RegCloseKey(hServiceKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* windows_getEntries */
/*****************************************************************************/

static sal_uInt32 SAL_CALL windows_getEntries(rtl_uString* strIdentifier,
								 	 rtl_uString* strSection,	
									 sal_Unicode* pszBuffer, 
									 sal_uInt32   MaxLen)
{
	sal_uInt32 ret = 2;
	HKEY hSectionKey;
	DWORD index = 0;
	sal_Unicode buffer[SERVICES_MAX_SECTION_LEN];
	DWORD sizeOfBuffer = SERVICES_MAX_SECTION_LEN;
	sal_Unicode* pChar = pszBuffer;
	sal_uInt32 sizeLeft = MaxLen - (pChar - pszBuffer);
	sal_Unicode* keyName = (sal_Unicode*) rtl_allocateMemory(
		sizeof(SERVICES_ROOT_KEY_NAME) + 
		sizeof(sal_Unicode) * ( strIdentifier->length + strSection->length + 2)
	);

	wcscpy(keyName, SERVICES_ROOT_KEY_NAME);
	wcscat(keyName, L"\\");
	wcscat(keyName, strIdentifier->buffer);
	wcscat(keyName, L"\\");
	wcscat(keyName, strSection->buffer);

	if (MaxLen >= 2)
	{
		pszBuffer[0] = '\0';		
		pszBuffer[1] = '\0';		
	}

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, KEY_READ, &hSectionKey) == ERROR_SUCCESS)
	{

		while (RegEnumValue(
			    	hSectionKey,		/* handle of key to enumerate */
			    	index,				/* index of subkey to enumerate */
			    	buffer,				/* address of buffer for subkey name */
			    	&sizeOfBuffer,		/* address for size of subkey buffer */
			    	NULL,				/* reserved */
			    	NULL,				/* address of buffer for class string */
			    	NULL,				/* address for size of class buffer */
			    	NULL) 				/* address for time key last written to */
				== ERROR_SUCCESS)
		{
			ret	+= wcslen(buffer) + 1;

			if (MaxLen >= 2)
			{
				wcsncpy(pChar, buffer, sizeLeft);
				pChar[sizeLeft - 1] = L'\0';
				pChar += wcslen(pChar) + 1;
			}
			
			sizeLeft = MaxLen - (pChar - pszBuffer);

			index++;

			sizeOfBuffer = SERVICES_MAX_SECTION_LEN;
		}

		RegCloseKey(hSectionKey);
	}

	rtl_freeMemory(keyName);

	return(ret);
}	

/*****************************************************************************/
/* globalApi */
/*****************************************************************************/

static DaemonRegistryApi globalApi =
{
	windows_hasDaemon,
	windows_getDaemons,
	windows_addDaemon,
	windows_removeDaemon,
	windows_readDaemonEntry,	
	windows_writeDaemonEntry,	
	windows_removeDaemonEntry,	
	windows_getSections,
	windows_getEntries,
};

/*****************************************************************************/
/* osl_openDaemonWindowsRegistry */
/*****************************************************************************/

DaemonRegistryApi* osl_getDaemonWindowsRegistry()
{
	return(&globalApi);
}


