/*************************************************************************
 *
 *  $RCSfile: dir.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef UNICODE
#define CHAR_SLASH		L'/'
#define CHAR_BKSLASH	L'\\'
#define CHAR_SPACE		L' '
#define CHAR_COLON		L':'
#define CHAR_PIPE		L'|'
#define CHAR_POINT		L'.'
#else
#define CHAR_SLASH		'/'
#define CHAR_BKSLASH	'\\'
#define CHAR_SPACE		' '
#define CHAR_COLON		':'
#define CHAR_PIPE		'|'
#define CHAR_POINT		'.'
#endif

#ifdef UNICODE
#define STR_SLASH		L"/"
#define STR_BKSLASH		L"\\"
#define STR_SPACE		L" "
#define STR_COLON		L":"
#define STR_PIPE		L"|"
#define STR_POINT		L"."
#else
#define STR_SLASH		"/"
#define STR_BKSLASH		"\\"
#define STR_SPACE		" "
#define STR_COLON		":"
#define STR_PIPE		"|"
#define STR_POINT		"."
#endif

#define UNICODE
#include <osl/file.h>
#include <rtl/alloc.h>
#include <rtl/tencinfo.h>

#include "system.h"
#include "dirW9X.h"
#include <systools/win32/kernel9x.h>

#ifdef lstrlen
#undef lstrlen
#endif

#ifdef lstrcpy
#undef lstrcpy
#endif

#ifdef UNICODE
#define lstrlen	wcslen
#define lstrcpy	wcscpy
#else
#define lstrlen	strlen
#define lstrcpy	strcpy
#endif

#define MAX_PATH_WIN2000	    32767
#define MAX_PATH_NT4			256

HANDLE WINAPI OpenLogicalDrivesEnum();
BOOL WINAPI EnumLogicalDrives( HANDLE hEnum, LPTSTR lpBuffer );
BOOL WINAPI CloseLogicalDrivesEnum( HANDLE hEnum );

typedef struct {
	UINT	uType;

	union {
		WIN32_FIND_DATA	FindData;
		TCHAR			cDriveString[4];
	};
	TCHAR			szFullPath[MAX_PATH];
	TCHAR			szLocation[MAX_PATH];
	int				nRefCount;
}DirectoryItem_Impl;

#define	DIRECTORYTYPE_LOCALROOT	0
#define	DIRECTORYTYPE_NETROOT		1
#define	DIRECTORYTYPE_NETRESORCE	2
#define	DIRECTORYTYPE_FILESYSTEM	3

#define DIRECTORYITEM_DRIVE		0
#define DIRECTORYITEM_FILE		1
#define DIRECTORYITEM_SERVER	2

typedef struct {
	UINT	uType;
	union {
		HANDLE	hFind;
		HANDLE	hEnumDrives;
	};
	TCHAR	szDirectoryPath[MAX_PATH];
} Directory_Impl;

typedef struct {
	HANDLE	hFile;
	TCHAR	szFileName[MAX_PATH];
} File_Impl;

// Different types of paths
#define PATHTYPE_ERROR		0x00000000
#define PATHTYPE_LOCAL		0x00000001
#define PATHTYPE_SERVER		0x00000002
#define PATHTYPE_WIN2000	0x10000000

typedef enum _PATHTYPE
{
	PATHTYPE_SYNTAXERROR,
	PATHTYPE_NETROOT,
	PATHTYPE_NETSERVER,
	PATHTYPE_VOLUME,
	PATHTYPE_FILE
} PATHTYPE;

/* This is the error table that defines the mapping between OS error
   codes and errno values */

struct errentry {
        unsigned long oscode;	/* OS return value */
        int errnocode;			/* System V error code */
};

static struct errentry errtable[] = {
        {  ERROR_SUCCESS,				 osl_File_E_None     },  /* 0 */
        {  ERROR_INVALID_FUNCTION,       osl_File_E_INVAL    },  /* 1 */
        {  ERROR_FILE_NOT_FOUND,         osl_File_E_NOENT    },  /* 2 */
        {  ERROR_PATH_NOT_FOUND,         osl_File_E_NOENT    },  /* 3 */
        {  ERROR_TOO_MANY_OPEN_FILES,    osl_File_E_MFILE    },  /* 4 */
        {  ERROR_ACCESS_DENIED,          osl_File_E_ACCES    },  /* 5 */
        {  ERROR_INVALID_HANDLE,         osl_File_E_BADF     },  /* 6 */
        {  ERROR_ARENA_TRASHED,          osl_File_E_NOMEM    },  /* 7 */
        {  ERROR_NOT_ENOUGH_MEMORY,      osl_File_E_NOMEM    },  /* 8 */
        {  ERROR_INVALID_BLOCK,          osl_File_E_NOMEM    },  /* 9 */
        {  ERROR_BAD_ENVIRONMENT,        osl_File_E_2BIG     },  /* 10 */
        {  ERROR_BAD_FORMAT,             osl_File_E_NOEXEC   },  /* 11 */
        {  ERROR_INVALID_ACCESS,         osl_File_E_INVAL    },  /* 12 */
        {  ERROR_INVALID_DATA,           osl_File_E_INVAL    },  /* 13 */
        {  ERROR_INVALID_DRIVE,          osl_File_E_NOENT    },  /* 15 */
        {  ERROR_CURRENT_DIRECTORY,      osl_File_E_ACCES    },  /* 16 */
        {  ERROR_NOT_SAME_DEVICE,        osl_File_E_XDEV     },  /* 17 */
        {  ERROR_NO_MORE_FILES,          osl_File_E_NOENT    },  /* 18 */
        {  ERROR_LOCK_VIOLATION,         osl_File_E_ACCES    },  /* 33 */
        {  ERROR_BAD_NETPATH,            osl_File_E_NOENT    },  /* 53 */
        {  ERROR_NETWORK_ACCESS_DENIED,  osl_File_E_ACCES    },  /* 65 */
        {  ERROR_BAD_NET_NAME,           osl_File_E_NOENT    },  /* 67 */
        {  ERROR_FILE_EXISTS,            osl_File_E_EXIST    },  /* 80 */
        {  ERROR_CANNOT_MAKE,            osl_File_E_ACCES    },  /* 82 */
        {  ERROR_FAIL_I24,               osl_File_E_ACCES    },  /* 83 */
        {  ERROR_INVALID_PARAMETER,      osl_File_E_INVAL    },  /* 87 */
        {  ERROR_NO_PROC_SLOTS,          osl_File_E_AGAIN    },  /* 89 */
        {  ERROR_DRIVE_LOCKED,           osl_File_E_ACCES    },  /* 108 */
        {  ERROR_BROKEN_PIPE,            osl_File_E_PIPE     },  /* 109 */
        {  ERROR_DISK_FULL,              osl_File_E_NOSPC    },  /* 112 */
        {  ERROR_INVALID_TARGET_HANDLE,  osl_File_E_BADF     },  /* 114 */
        {  ERROR_INVALID_HANDLE,         osl_File_E_INVAL    },  /* 124 */
        {  ERROR_WAIT_NO_CHILDREN,       osl_File_E_CHILD    },  /* 128 */
        {  ERROR_CHILD_NOT_COMPLETE,     osl_File_E_CHILD    },  /* 129 */
        {  ERROR_DIRECT_ACCESS_HANDLE,   osl_File_E_BADF     },  /* 130 */
        {  ERROR_NEGATIVE_SEEK,          osl_File_E_INVAL    },  /* 131 */
        {  ERROR_SEEK_ON_DEVICE,         osl_File_E_ACCES    },  /* 132 */
        {  ERROR_DIR_NOT_EMPTY,          osl_File_E_NOTEMPTY },  /* 145 */
        {  ERROR_NOT_LOCKED,             osl_File_E_ACCES    },  /* 158 */
        {  ERROR_BAD_PATHNAME,           osl_File_E_NOENT    },  /* 161 */
        {  ERROR_MAX_THRDS_REACHED,      osl_File_E_AGAIN    },  /* 164 */
        {  ERROR_LOCK_FAILED,            osl_File_E_ACCES    },  /* 167 */
        {  ERROR_ALREADY_EXISTS,         osl_File_E_EXIST    },  /* 183 */
        {  ERROR_FILENAME_EXCED_RANGE,   osl_File_E_NOENT    },  /* 206 */
        {  ERROR_NESTING_NOT_ALLOWED,    osl_File_E_AGAIN    },  /* 215 */
        {  ERROR_NOT_ENOUGH_QUOTA,       osl_File_E_NOMEM    }    /* 1816 */
};

/* size of the table */
#define ERRTABLESIZE (sizeof(errtable)/sizeof(errtable[0]))

/* The following two constants must be the minimum and maximum
   values in the (contiguous) range of osl_File_E_xec Failure errors. */
#define MIN_EXEC_ERROR ERROR_INVALID_STARTING_CODESEG
#define MAX_EXEC_ERROR ERROR_INFLOOP_IN_RELOC_CHAIN

/* These are the low and high value in the range of errors that are
   access violations */
#define MIN_EACCES_RANGE ERROR_WRITE_PROTECT
#define MAX_EACCES_RANGE ERROR_SHARING_BUFFER_EXCEEDED


/*******************************************************************************/

static oslFileError _mapError( DWORD dwError )
{
    int i;

    /* check the table for the OS error code */
    for ( i = 0; i < ERRTABLESIZE; ++i )
	{
		if ( dwError == errtable[i].oscode ) 
			return errtable[i].errnocode;
    }

    /* The error code wasn't in the table.  We check for a range of */
    /* osl_File_E_ACCES errors or exec failure errors (ENOEXEC).  Otherwise   */
    /* osl_File_E_INVAL is returned.                                          */

    if ( dwError >= MIN_EACCES_RANGE && dwError <= MAX_EACCES_RANGE)
		return osl_File_E_ACCES;
    else if ( dwError >= MIN_EXEC_ERROR && dwError <= MAX_EXEC_ERROR)
		return osl_File_E_NOEXEC;
    else
		return osl_File_E_INVAL;
}

#define MapError( oserror )	_mapError( oserror )

/*******************************************************************************/
// Tests, whether Windows2000 is operating

static BOOL testWin2000(void)
{
	return FALSE;
}

static PATHTYPE SAL_CALL _osl_splitNormalizedPath(
	rtl_uString*	strPath,
	rtl_uString**	pstrServer,
	rtl_uString**	pstrRootResource,
	rtl_uString**	pstrDirectory,
	rtl_uString**	pstrFile )
{
	sal_Unicode*	pStart = strPath->buffer;
	sal_Unicode*	pStringEnd = strPath->buffer + strPath->length;
	sal_Unicode*	pEnd = pStart;
	sal_Unicode*	pLastSlash = NULL;

	/* First check if the path starts with '//' otherwise return an error */

	if ( strPath->length < 2 || (rtl_ustr_ascii_shortenedCompare_WithLength( strPath->buffer, 2, "//", 2 ) != 0) )
		return PATHTYPE_SYNTAXERROR;
	else
		pEnd += 2;

	/* Now retreive the server part of the path */

	while ( pEnd < pStringEnd && *pEnd != CHAR_SLASH )
		pEnd++;

	/* If we don't reached the end place pointer after the slash */

	if ( pEnd < pStringEnd )
		pEnd++;

	if ( pstrServer )
		rtl_uString_newFromStr_WithLength( pstrServer, pStart, pEnd - pStart );

	/* Retrieve the root directory/drives/resource */

	pStart = pEnd;

	while ( pEnd < pStringEnd && *pEnd != CHAR_SLASH )
		pEnd++;

	if ( pEnd < pStringEnd )
		pEnd++;

	if ( pstrRootResource )
		rtl_uString_newFromStr_WithLength( pstrRootResource, pStart, pEnd - pStart );

	/* Retrieve the rest of the directory path followed by a path separator */

	pStart = pEnd;

	while ( pEnd < pStringEnd )
	{
		if ( *pEnd == CHAR_SLASH )
			pLastSlash = pEnd;
		pEnd++;
	}

	if ( pLastSlash )
		pLastSlash++;

	if ( pstrDirectory )
		rtl_uString_newFromStr_WithLength( pstrDirectory, pStart, pLastSlash ? pLastSlash - pStart : 0 );

	if ( pLastSlash )
		pStart = pLastSlash;

	if ( pstrFile )
		rtl_uString_newFromStr_WithLength( pstrFile, pStart, pEnd - pStart );

	if ( *(*pstrFile)->buffer || *(*pstrDirectory)->buffer )
		return PATHTYPE_FILE;
	else if ( *(*pstrRootResource)->buffer )
		return PATHTYPE_VOLUME;
	else if ( *(*pstrServer)->buffer )
		return PATHTYPE_NETSERVER;
	else
		return PATHTYPE_NETROOT;
}


static sal_Bool SAL_CALL isLocalHost( rtl_uString* strPath )
{
	return 
		strPath->length >= 3 && (rtl_ustr_ascii_shortenedCompare_WithLength( strPath->buffer, 3, "//.", 3 ) == 0) ||
		strPath->length >= 11 && (rtl_ustr_ascii_shortenedCompare_WithLength( strPath->buffer, 11, "//localhost", 11 ) == 0);

}


static PATHTYPE SAL_CALL _osl_getSystemPathFromNormalizedPath( rtl_uString* strPath, rtl_uString** pstrSystemPath )
{
	rtl_uString*	strServer = NULL;
	rtl_uString*	strRoot = NULL;
	rtl_uString*	strDirectory = NULL;
	rtl_uString*	strFile = NULL;

	PATHTYPE	type = _osl_splitNormalizedPath( strPath, &strServer, &strRoot, &strDirectory, &strFile );

	if ( type != PATHTYPE_SYNTAXERROR )
	{
		/* if it is a local path we don't need a server part in the system depended path */

		if ( isLocalHost( strServer ) && strRoot->length )
		{
			rtl_uString_assign( pstrSystemPath, strRoot );
		}
		else
		{
			rtl_uString_assign( pstrSystemPath, strServer );
			rtl_uString_newConcat( pstrSystemPath, *pstrSystemPath, strRoot );
		}

		{
			/* Append the rest */

			rtl_uString_newConcat( pstrSystemPath, *pstrSystemPath, strDirectory );
			rtl_uString_newConcat( pstrSystemPath, *pstrSystemPath, strFile );

			/* Replace '/' with '\' */

			rtl_uString_newReplace( pstrSystemPath, *pstrSystemPath, CHAR_SLASH, CHAR_BKSLASH );

			/* Replace '|' with ':' */
			rtl_uString_newReplace( pstrSystemPath, *pstrSystemPath, CHAR_PIPE, CHAR_COLON );
		}
	}

	/* Release temporary acquired substrings */

	if ( strServer )
		rtl_uString_release( strServer );

	if ( strRoot )
		rtl_uString_release( strRoot );

	if ( strDirectory )
		rtl_uString_release( strDirectory );

	if ( strFile )
		rtl_uString_release( strFile );

	return type;
}

oslFileError SAL_CALL osl_getSystemPathFromNormalizedPath( rtl_uString* strPath, rtl_uString** pstrSystemPath )
{
	PATHTYPE	type = _osl_getSystemPathFromNormalizedPath( strPath, pstrSystemPath );
	return type == PATHTYPE_SYNTAXERROR ? osl_File_E_INVAL : osl_File_E_None;
}


/*******************************************************************************/
/*	Path_Transferring */
/****************************************************************************/

/****************************************************************************/

static BOOL UNC2Local( LPCTSTR pszUNC, LPTSTR pszLocal )
{
	if ( pszUNC[0] != '/' || pszUNC[1] != '/' )
		return FALSE;

	if ( pszUNC[2] == '.' )
	{
		if ( pszUNC[3] != '/' )
			return FALSE;
		else
			pszUNC += 4;
	}
	
	while ( *pszUNC )
	{
		switch ( *pszUNC )
		{
		case '/':
			*pszLocal++ = '\\';
			break;
		default:
			*pszLocal++ = *pszUNC++;
			break;
		}
	}

	return TRUE;
}

/****************************************************************************/
/*	osl_openFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_openFile( rtl_uString *strPath, oslFileHandle *pHandle, sal_uInt32 uFlags )
{
	rtl_uString		*strSysPath = NULL;
	oslFileError	error = osl_getSystemPathFromNormalizedPath( strPath, &strSysPath );

	if ( osl_File_E_None == error )
	{
		DWORD	dwAccess = 0, dwShare = FILE_SHARE_READ, dwCreation = 0, dwAttributes = 0;
		HANDLE	hFile;

		if ( uFlags & osl_File_OpenFlag_Read )
		{
			dwAccess |= GENERIC_READ;
			dwShare  |= FILE_SHARE_WRITE;
		}

		if ( uFlags & osl_File_OpenFlag_Write )
			dwAccess |= GENERIC_WRITE;

		if ( uFlags & osl_File_OpenFlag_Create )
			dwCreation |= CREATE_NEW;
		else
			dwCreation |= OPEN_EXISTING;

		hFile = lpfnCreateFile( rtl_uString_getStr( strSysPath ), dwAccess, dwShare, NULL, dwCreation, dwAttributes, NULL );

		if ( INVALID_HANDLE_VALUE != hFile && ((HANDLE)0) != hFile )
		{
			File_Impl	*pImpl = (File_Impl *)rtl_allocateMemory( sizeof(File_Impl) );

			pImpl->hFile = hFile;
			wcscpy( pImpl->szFileName, rtl_uString_getStr( strSysPath ) );
			*pHandle = (oslFileHandle)pImpl;
		}
		else
		{
			error = MapError( GetLastError() );
			*pHandle = NULL;
		}

		rtl_uString_release( strSysPath );
	}

	return error;
}


/****************************************************************************/
/*	osl_closeFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_closeFile( oslFileHandle Handle )
{
	oslFileError	error;
	File_Impl		*pImpl = (File_Impl *)Handle;

	if ( pImpl )
	{
		error = CloseHandle( pImpl->hFile ) ? osl_File_E_None : MapError( GetLastError() );
		free( pImpl );
	}
	else
		error = osl_File_E_INVAL;

	return error;
}

/****************************************************************************/
/*	osl_setFilePos	*/
/****************************************************************************/

oslFileError SAL_CALL osl_setFilePos( oslFileHandle Handle, sal_uInt32 uHow, sal_uInt64 uPos )
{
	oslFileError	error;
	File_Impl		*pImpl = (File_Impl *)Handle;

	if ( pImpl )
	{
		DWORD	dwMoveMethod;
		LONG	lDistanceToMove, lDistanceToMoveHigh;

		switch ( uHow )
		{
		case osl_Pos_Current:
			dwMoveMethod = FILE_CURRENT;
			break;
		case osl_Pos_End:
			dwMoveMethod = FILE_END;
			break;
		case osl_Pos_Absolut:
		default:
			dwMoveMethod = FILE_BEGIN;
			break;
		}

		lDistanceToMove = (LONG)(uPos & 0xFFFFFFFF);
		lDistanceToMoveHigh = (LONG)(uPos >> 32);


		SetLastError(0);
		SetFilePointer( pImpl->hFile, lDistanceToMove, &lDistanceToMoveHigh, dwMoveMethod );

		error = MapError( GetLastError() );
	}
	else
		error = osl_File_E_INVAL;

	return error;
}

/****************************************************************************/
/*	osl_getFilePos	*/
/****************************************************************************/

oslFileError SAL_CALL osl_getFilePos( oslFileHandle Handle, sal_uInt64 *pPos )
{
	oslFileError	error;
	File_Impl		*pImpl = (File_Impl *)Handle;

	if ( pImpl )
	{
		LONG	lDistanceToMove, lDistanceToMoveHigh;


		lDistanceToMoveHigh = 0;

		lDistanceToMove = SetFilePointer( pImpl->hFile, 0, &lDistanceToMoveHigh, FILE_CURRENT );

		error = MapError( GetLastError() );

		if ( osl_File_E_None == error )
			*pPos = (sal_uInt64)lDistanceToMove + ((sal_uInt64)lDistanceToMoveHigh << 32);
	}
	else
		error = osl_File_E_INVAL;


	return error;
}


/****************************************************************************/
/*	osl_setFileSize	*/
/****************************************************************************/

oslFileError SAL_CALL osl_setFileSize( oslFileHandle Handle, sal_uInt64 uSize )
{
	oslFileError	error = error = osl_setFilePos( Handle, osl_Pos_Absolut, uSize );
	if ( osl_File_E_None == error )
	{
		if ( !SetEndOfFile( ((File_Impl *)Handle)->hFile ) )
			error = MapError( osl_File_E_None );
	}

	return error;
}

/****************************************************************************/
/*	osl_readFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_readFile( oslFileHandle Handle, void *pBuffer, sal_uInt64 uBytesRequested, sal_uInt64 *pBytesRead )
{
	oslFileError	error;
	File_Impl		*pImpl = (File_Impl *)Handle;

	if ( pImpl )
	{
		DWORD	dwBytesToRead = (DWORD)uBytesRequested;
		DWORD	dwBytesRead;

		if ( ReadFile( pImpl->hFile, pBuffer, dwBytesToRead, &dwBytesRead, NULL ) )
		{
			*pBytesRead = (sal_uInt64)dwBytesRead;
			error = osl_File_E_None;
		}
		else
			error = MapError( GetLastError() );
	}
	else
		error = osl_File_E_INVAL;

	return error;
}


/****************************************************************************/
/*	osl_writeFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_writeFile( oslFileHandle Handle, const void *pBuffer, sal_uInt64 uBytesToWrite, sal_uInt64 *pBytesWritten )
{
	oslFileError	error;
	File_Impl		*pImpl = (File_Impl *)Handle;

	if ( pImpl )
	{
		DWORD	dwBytesToWrite = (DWORD)uBytesToWrite;
		DWORD	dwBytesWritten;

		if ( WriteFile( pImpl->hFile, pBuffer, dwBytesToWrite, &dwBytesWritten, NULL ) )
		{
			*pBytesWritten = (sal_uInt64)dwBytesWritten;
			error = osl_File_E_None;
		}
		else
			error = MapError( GetLastError() );
	}
	else
		error = osl_File_E_INVAL;

	return error;
}


/****************************************************************************/
/*	osl_createDirectoryItemFromHandle	*/
/****************************************************************************/

oslFileError SAL_CALL osl_createDirectoryItemFromHandle( oslFileHandle Handle, oslDirectoryItem *pItem )
{
	oslFileError	error;
	File_Impl		*pImpl = (File_Impl *)Handle;

	if ( pImpl )
	{
		HANDLE	hFind;

		*pItem = (oslDirectoryItem)rtl_allocateMemory ( sizeof(DirectoryItem_Impl ) );

		((DirectoryItem_Impl *)*pItem)->uType = DIRECTORYITEM_FILE;
		hFind = lpfnFindFirstFile( pImpl->szFileName, &((DirectoryItem_Impl *)*pItem)->FindData );

		if ( hFind != INVALID_HANDLE_VALUE )
		{
			wcscpy( ((DirectoryItem_Impl *)*pItem)->szFullPath, pImpl->szFileName );
			FindClose( hFind );
			error = osl_File_E_None;
		}
		else
			error = MapError( GetLastError() );
	}
	else
		error = osl_File_E_INVAL;

	return error;
}

/****************************************************************************/
/*	osl_createDirectory	*/
/****************************************************************************/

oslFileError SAL_CALL osl_createDirectory( rtl_uString* strPath )
{
	rtl_uString	*strSysPath = NULL;
	oslFileError	error = osl_getSystemPathFromNormalizedPath( strPath, &strSysPath );

	if ( osl_File_E_None == error )
	{
		if ( lpfnCreateDirectory( rtl_uString_getStr( strSysPath ), NULL ) )
			error = osl_File_E_None;
		else
			error = MapError( GetLastError() );

		rtl_uString_release( strSysPath );
	}

	return error;
}


/****************************************************************************/
/*	osl_removeDirectory	*/
/****************************************************************************/

oslFileError SAL_CALL osl_removeDirectory( rtl_uString* strPath )
{
	rtl_uString	*strSysPath = NULL;
	oslFileError	error = osl_getSystemPathFromNormalizedPath( strPath, &strSysPath );

	if ( osl_File_E_None == error )
	{
		if ( RemoveDirectory( rtl_uString_getStr( strSysPath ) ) )
			error = osl_File_E_None;
		else
			error = MapError( GetLastError() );

		rtl_uString_release( strSysPath );
	}

	return error;
}

/****************************************************************************/
/*	osl_removeFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_removeFile( rtl_uString* strPath )
{
	rtl_uString	*strSysPath = NULL;
	oslFileError	error = osl_getSystemPathFromNormalizedPath( strPath, &strSysPath );

	if ( osl_File_E_None == error )
	{
		if ( DeleteFile( rtl_uString_getStr( strSysPath ) ) )
			error = osl_File_E_None;
		else
			error = MapError( GetLastError() );

		rtl_uString_release( strSysPath );
	}

	return error;
}

/****************************************************************************/
/*	osl_copyFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_copyFile( rtl_uString* strPath, rtl_uString *strDestPath )
{
	rtl_uString	*strSysPath = NULL, *strSysDestPath = NULL;
	oslFileError	error = osl_getSystemPathFromNormalizedPath( strPath, &strSysPath );

	if ( osl_File_E_None == error )
		error = osl_getSystemPathFromNormalizedPath( strDestPath, &strSysDestPath );

	if ( osl_File_E_None == error )
	{
		if ( CopyFile( rtl_uString_getStr( strSysPath ), rtl_uString_getStr( strSysDestPath ), FALSE ) )
			error = osl_File_E_None;
		else
			error = MapError( GetLastError() );
	}

	if ( strSysPath )
		rtl_uString_release( strSysPath );
	if ( strSysDestPath )
		rtl_uString_release( strSysDestPath );

	return error;
}

/****************************************************************************/
/*	osl_moveFile	*/
/****************************************************************************/

oslFileError SAL_CALL osl_moveFile( rtl_uString* strPath, rtl_uString *strDestPath )
{
	rtl_uString	*strSysPath = NULL, *strSysDestPath = NULL;
	oslFileError	error = osl_getSystemPathFromNormalizedPath( strPath, &strSysPath );

	if ( osl_File_E_None == error )
		error = osl_getSystemPathFromNormalizedPath( strDestPath, &strSysDestPath );

	if ( osl_File_E_None == error )
	{
		if ( MoveFileEx( rtl_uString_getStr( strSysPath ), rtl_uString_getStr( strSysDestPath ), MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH ) )
			error = osl_File_E_None;
		else
			error = MapError( GetLastError() );
	}

	if ( strSysPath )
		rtl_uString_release( strSysPath );
	if ( strSysDestPath )
		rtl_uString_release( strSysDestPath );

	return error;
}


/****************************************************************************/
/****************************************************************************/
/*	*/
/*	Directory entry functions */
/*	*/ 
/****************************************************************************/
/****************************************************************************/



/****************************************************************************/
/*	osl_openDirectory	*/
/****************************************************************************/

oslFileError SAL_CALL osl_openLocalRoot(  rtl_uString *strDirectoryPath, oslDirectory *pDirectory )
{
	rtl_uString		*strSysPath = NULL;
	oslFileError	error;

	/* Assume failure */
 
	if ( !pDirectory )
		return osl_File_E_INVAL;

	*pDirectory = NULL;

	error = osl_getSystemPathFromNormalizedPath( strDirectoryPath, &strSysPath );

	if ( osl_File_E_None == error )
	{
		Directory_Impl	*pDirImpl;

		pDirImpl = rtl_allocateMemory( sizeof(Directory_Impl) );
		wcscpy( pDirImpl->szDirectoryPath, rtl_uString_getStr(strSysPath) );
	
		/* Append backslash if neccessary */

		if ( pDirImpl->szDirectoryPath[wcslen(pDirImpl->szDirectoryPath) - 1] != L'\\' )
			wcscat( pDirImpl->szDirectoryPath, L"\\" );

		pDirImpl->uType = DIRECTORYTYPE_LOCALROOT;
		pDirImpl->hEnumDrives = OpenLogicalDrivesEnum();

		if ( pDirImpl->hEnumDrives != INVALID_HANDLE_VALUE )
		{
			*pDirectory = (oslDirectory)pDirImpl;
			error = osl_File_E_None;
		}
		else
		{
			if ( pDirImpl )
				rtl_freeMemory(pDirImpl);

			error = MapError( GetLastError() );
		}

		rtl_uString_release( strSysPath );
	}

	return error;
}

oslFileError SAL_CALL osl_openFileDirectory( rtl_uString *strDirectoryPath, oslDirectory *pDirectory)
{
	TCHAR			lpFileMask[MAX_PATH];
	WIN32_FIND_DATA	wfd;
	rtl_uString		*strSysPath = NULL;
	oslFileError	error;

	/* Assume failure */
 
	if ( !pDirectory )
		return osl_File_E_INVAL;

	*pDirectory = NULL;

	error = osl_getSystemPathFromNormalizedPath( strDirectoryPath, &strSysPath );
	if ( osl_File_E_None == error )
	{
		Directory_Impl	*pDirImpl;

		pDirImpl = rtl_allocateMemory( sizeof(Directory_Impl) );
		wcscpy( pDirImpl->szDirectoryPath, rtl_uString_getStr(strSysPath) );
	
		/* Append backslash if neccessary */

		if ( pDirImpl->szDirectoryPath[wcslen(pDirImpl->szDirectoryPath) - 1] != L'\\' )
			wcscat( pDirImpl->szDirectoryPath, L"\\" );

		wcscpy( lpFileMask, pDirImpl->szDirectoryPath );
		wcscat( lpFileMask, L"*.*" );

		pDirImpl->uType = DIRECTORYTYPE_FILESYSTEM;
		pDirImpl->hFind = lpfnFindFirstFile( lpFileMask, &wfd );

		/* Every Directory has at least two entries (. and ..)", so if the first */
		/* call fails the directory does not exist or is unaccessible. */

		if ( pDirImpl->hFind != INVALID_HANDLE_VALUE )
		{
			*pDirectory = (oslDirectory)pDirImpl;
			error = osl_File_E_None;
		}
		else
		{
			if ( pDirImpl )
				rtl_freeMemory(pDirImpl);

			error = MapError( GetLastError() );
		}

		rtl_uString_release( strSysPath );
	}

	return error;
}


oslFileError SAL_CALL osl_openDirectory( rtl_uString *strDirectoryPath, oslDirectory *pDirectory)
{
	oslFileError	error;
	rtl_uString*	strServer = NULL;
	rtl_uString*	strRoot = NULL;
	rtl_uString*	strDirectory = NULL;
	rtl_uString*	strFile = NULL;

	_osl_splitNormalizedPath( strDirectoryPath, &strServer, &strRoot, &strDirectory, &strFile );

	{
		if ( isLocalHost( strServer ) &&  !strRoot->length )
			error = osl_openLocalRoot( strDirectoryPath, pDirectory );
		else
			error = osl_openFileDirectory( strDirectoryPath, pDirectory );
	}

	/* Release temporary acquired substrings */

	if ( strServer )
		rtl_uString_release( strServer );

	if ( strRoot )
		rtl_uString_release( strRoot );

	if ( strDirectory )
		rtl_uString_release( strDirectory );

	if ( strFile )
		rtl_uString_release( strFile );

	return error;
}

/****************************************************************************/
/*	osl_getNextDirectoryEntry	*/
/****************************************************************************/

oslFileError SAL_CALL osl_getNextDrive( oslDirectory Directory, oslDirectoryItem *pItem, sal_uInt32 uHint )
{
	Directory_Impl		*pDirImpl = (Directory_Impl *)Directory;
	DirectoryItem_Impl	*pItemImpl = NULL;
	BOOL				fSuccess;

	/* Assume failure */

	if ( !pItem )
		return osl_File_E_INVAL;

	*pItem = NULL;

	if ( !pDirImpl )
		return osl_File_E_INVAL;

	pItemImpl = rtl_allocateMemory( sizeof(DirectoryItem_Impl) );
	if ( !pItemImpl )
		return osl_File_E_NOMEM;


	ZeroMemory( pItemImpl, sizeof(DirectoryItem_Impl) );
	pItemImpl->uType = DIRECTORYITEM_DRIVE;
	osl_acquireDirectoryItem( (oslDirectoryItem)pItemImpl );
	fSuccess = EnumLogicalDrives( pDirImpl->hEnumDrives, pItemImpl->cDriveString );

	if ( fSuccess )
	{
		*pItem = pItemImpl;
		return osl_File_E_None;
	}
	else
	{
		rtl_freeMemory( pItemImpl );
		return MapError( GetLastError() );
	}
}

oslFileError SAL_CALL osl_getNextFileItem( oslDirectory Directory, oslDirectoryItem *pItem, sal_uInt32 uHint )
{
	Directory_Impl		*pDirImpl = (Directory_Impl *)Directory;
	DirectoryItem_Impl	*pItemImpl = NULL;
	BOOL				fFound, fValid;

	/* Assume failure */

	if ( !pItem )
		return osl_File_E_INVAL;

	*pItem = NULL;

	if ( !pDirImpl )
		return osl_File_E_INVAL;

	pItemImpl = rtl_allocateMemory( sizeof(DirectoryItem_Impl) );
	if ( !pItemImpl )
		return osl_File_E_NOMEM;

	do {
		memset( pItemImpl, 0, sizeof(DirectoryItem_Impl) );
		fFound = lpfnFindNextFile( pDirImpl->hFind, &pItemImpl->FindData );
		fValid = pItemImpl->FindData.cFileName[0] != '.';
	} while ( fFound && !fValid );

	if ( fFound )
	{
		pItemImpl->uType = DIRECTORYITEM_FILE;
		pItemImpl->nRefCount = 1;
		wcscpy( pItemImpl->szFullPath, pDirImpl->szDirectoryPath );
		wcscat( pItemImpl->szFullPath, pItemImpl->FindData.cFileName );
		*pItem = (oslDirectoryItem)pItemImpl;
		return osl_File_E_None;
	}
	else
	{
		rtl_freeMemory( pItemImpl );
		return MapError( GetLastError() );
	}
}


oslFileError SAL_CALL osl_getNextDirectoryItem( oslDirectory Directory, oslDirectoryItem *pItem, sal_uInt32 uHint )
{
	Directory_Impl		*pDirImpl = (Directory_Impl *)Directory;
	DirectoryItem_Impl	*pItemImpl = NULL;

	/* Assume failure */

	if ( !pItem )
		return osl_File_E_INVAL;

	*pItem = NULL;

	if ( !pDirImpl )
		return osl_File_E_INVAL;

	switch ( pDirImpl->uType )
	{
	case DIRECTORYTYPE_LOCALROOT:
		return osl_getNextDrive( Directory, pItem, uHint );
	case DIRECTORYTYPE_FILESYSTEM:
		return osl_getNextFileItem( Directory, pItem, uHint );
	default:
		return osl_File_E_INVAL;
	}
}


/****************************************************************************/
/*	osl_closeDirectory	*/
/****************************************************************************/

oslFileError SAL_CALL osl_closeDirectory( oslDirectory Directory )
{
	Directory_Impl	*pDirImpl = (Directory_Impl *)Directory;
	oslFileError	eError;

	if ( pDirImpl )
	{
		if ( pDirImpl->uType == DIRECTORYTYPE_FILESYSTEM )
			eError = FindClose( pDirImpl->hFind ) ? osl_File_E_None : MapError( GetLastError() );
		else
			eError = CloseLogicalDrivesEnum( pDirImpl->hEnumDrives ) ? osl_File_E_None : MapError( GetLastError() );
		rtl_freeMemory(pDirImpl);
	}
	else
		eError = osl_File_E_INVAL;

	return eError;
}


/****************************************************************************/
/*	osl_getDirectoryItem */
/****************************************************************************/

oslFileError SAL_CALL osl_getDirectoryItem( rtl_uString *strFilePath, oslDirectoryItem *pItem )
{
	oslFileError	error = osl_File_E_None;
	rtl_uString*	strSysFilePath = NULL;
	PATHTYPE		type;

	/* Assume failure */

	if ( !pItem )
		return osl_File_E_INVAL;

	*pItem = NULL;


	type = _osl_getSystemPathFromNormalizedPath( strFilePath, &strSysFilePath );

	
	switch ( type )
	{
	case PATHTYPE_NETSERVER:
		{
			DirectoryItem_Impl*	pItemImpl = rtl_allocateMemory( sizeof(DirectoryItem_Impl) );


			if ( !pItemImpl )
				error = osl_File_E_NOMEM;

			if ( osl_File_E_None == error )
			{
				ZeroMemory( pItemImpl, sizeof(DirectoryItem_Impl) );
				pItemImpl->uType = DIRECTORYITEM_SERVER;

				osl_acquireDirectoryItem( (oslDirectoryItem)pItemImpl );

				lstrcpy( pItemImpl->szFullPath, strSysFilePath->buffer );

				*pItem = pItemImpl;
			}
		}
		break;
	case PATHTYPE_VOLUME:
		{
			DirectoryItem_Impl*	pItemImpl = rtl_allocateMemory( sizeof(DirectoryItem_Impl) );


			if ( !pItemImpl )
				error = osl_File_E_NOMEM;

			if ( osl_File_E_None == error )
			{
				ZeroMemory( pItemImpl, sizeof(DirectoryItem_Impl) );
				pItemImpl->uType = DIRECTORYITEM_DRIVE;

				osl_acquireDirectoryItem( (oslDirectoryItem)pItemImpl );

				lstrcpy( pItemImpl->cDriveString, strSysFilePath->buffer );

				if ( pItemImpl->cDriveString[lstrlen(pItemImpl->cDriveString) - 1] != '\\' )
					lstrcat( pItemImpl->cDriveString, TEXT( "\\" ) );

				*pItem = pItemImpl;
			}
		}
		break;
	case PATHTYPE_FILE:
		{
			HANDLE				hFind;
			WIN32_FIND_DATA		aFindData;

			hFind = lpfnFindFirstFile( rtl_uString_getStr(strSysFilePath), &aFindData );

			if ( hFind != INVALID_HANDLE_VALUE )
			{
				DirectoryItem_Impl	*pItemImpl = rtl_allocateMemory( sizeof(DirectoryItem_Impl) );

				ZeroMemory( pItemImpl, sizeof(DirectoryItem_Impl) );
				osl_acquireDirectoryItem( (oslDirectoryItem)pItemImpl );

				CopyMemory( &pItemImpl->FindData, &aFindData, sizeof(WIN32_FIND_DATA) );
				wcscpy( pItemImpl->szFullPath, rtl_uString_getStr(strSysFilePath) );

				pItemImpl->uType = DIRECTORYITEM_FILE;
				*pItem = pItemImpl;
				FindClose( hFind );

				return osl_File_E_None;
			}
			else
				error = MapError( GetLastError() );
		}
		break;

	case PATHTYPE_SYNTAXERROR:
	default:
		error = osl_File_E_INVAL;
		break;
	}

	if ( strSysFilePath )
		rtl_uString_release( strSysFilePath );

	return error;
}


/****************************************************************************/
/*	osl_releaseDirectoryItem */
/****************************************************************************/

oslFileError SAL_CALL osl_acquireDirectoryItem( oslDirectoryItem Item )
{
	DirectoryItem_Impl	*pItemImpl = (DirectoryItem_Impl *)Item;

	if ( !pItemImpl )
		return osl_File_E_INVAL;

	pItemImpl->nRefCount++;
	return osl_File_E_None;
}

oslFileError SAL_CALL osl_releaseDirectoryItem( oslDirectoryItem Item )
{
	DirectoryItem_Impl	*pItemImpl = (DirectoryItem_Impl *)Item;

	if ( !pItemImpl )
		return osl_File_E_INVAL;

	if ( ! --pItemImpl->nRefCount )
		rtl_freeMemory( pItemImpl );
	return osl_File_E_None;
}


/****************************************************************************/
/*	osl_getFileStatus */
/****************************************************************************/

BOOL FileTimeToTimeValue( const FILETIME *cpFTime, TimeValue *pTimeVal )
{
	SYSTEMTIME	BaseSysTime;
	FILETIME	BaseFileTime;
	BOOL		fSuccess = FALSE;	/* Assume failure */

	BaseSysTime.wYear         = 1970;
	BaseSysTime.wMonth        = 1;
    BaseSysTime.wDayOfWeek    = 0;
    BaseSysTime.wDay          = 1;
    BaseSysTime.wHour         = 0;
    BaseSysTime.wMinute       = 0;
    BaseSysTime.wSecond       = 0;
    BaseSysTime.wMilliseconds = 0;

	if ( SystemTimeToFileTime(&BaseSysTime, &BaseFileTime) )
	{
		__int64		Value;

		fSuccess = 0 <= (Value = *((__int64 *)cpFTime) - *((__int64 *)&BaseFileTime));

		if ( fSuccess )
		{
			pTimeVal->Seconds  = (unsigned long) (Value / 10000000L);
			pTimeVal->Nanosec  = (unsigned long)((Value % 10000000L) * 100);
		}
	}

	return fSuccess;
}


/****************************************************************************/

static oslFileError SAL_CALL osl_getDriveInfo( oslDirectoryItem Item, oslFileStatus *pStatus, uLong uFieldMask )
{
	DirectoryItem_Impl	*pItemImpl = (DirectoryItem_Impl *)Item;
	TCHAR				cDrive[3] = L"A:";

	if ( !pItemImpl )
		return osl_File_E_INVAL;

	pStatus->uValidFields = 0;

	cDrive[0] = pItemImpl->cDriveString[0];

	rtl_uString_newFromStr( &pStatus->strFileName, cDrive );
	pStatus->uValidFields |= osl_FileStatus_Mask_FileName;

	pStatus->eType = osl_File_Type_Volume;
	pStatus->uValidFields |= osl_FileStatus_Mask_Type;

	if ( (uFieldMask & osl_FileStatus_Mask_NativePath) && pStatus->pstrNativePath != NULL )
	{
		rtl_uString_newFromStr( pStatus->pstrNativePath, pItemImpl->cDriveString );
		pStatus->uValidFields |= osl_FileStatus_Mask_NativePath;
	}

	if ( (uFieldMask & osl_FileStatus_Mask_FilePath) && pStatus->pstrFilePath != NULL )
	{
		rtl_uString	*strNativePath = NULL;

		rtl_uString_newFromStr( &strNativePath, pItemImpl->cDriveString );
		osl_normalizePath( strNativePath, pStatus->pstrFilePath );
		rtl_uString_release( strNativePath );
		pStatus->uValidFields |= osl_FileStatus_Mask_FilePath;
	}

	return osl_File_E_None;
}

/****************************************************************************/

static oslFileError SAL_CALL osl_getServerInfo( oslDirectoryItem Item, oslFileStatus *pStatus, uLong uFieldMask )
{
	DirectoryItem_Impl	*pItemImpl = (DirectoryItem_Impl *)Item;

	if ( !pItemImpl )
		return osl_File_E_INVAL;

	pStatus->uValidFields = 0;

	pStatus->uValidFields |= osl_FileStatus_Mask_FileName;
	rtl_uString_newFromStr( &pStatus->strFileName, pItemImpl->FindData.cFileName );

	pStatus->eType = osl_File_Type_Directory;
	pStatus->uValidFields |= osl_FileStatus_Mask_Type;

	if ( (uFieldMask & osl_FileStatus_Mask_NativePath) && pStatus->pstrNativePath != NULL )
	{
		rtl_uString_newFromStr( pStatus->pstrNativePath, pItemImpl->szFullPath );
		pStatus->uValidFields |= osl_FileStatus_Mask_NativePath;
	}

	if ( (uFieldMask & osl_FileStatus_Mask_FilePath) && pStatus->pstrFilePath != NULL )
	{
		rtl_uString_newFromStr( pStatus->pstrFilePath, pItemImpl->szFullPath );
		pStatus->uValidFields |= osl_FileStatus_Mask_FilePath;
	}

	return osl_File_E_None;
}

/****************************************************************************/

oslFileError SAL_CALL osl_getFileStatus( oslDirectoryItem Item, oslFileStatus *pStatus, uLong uFieldMask )
{
	DirectoryItem_Impl	*pItemImpl = (DirectoryItem_Impl *)Item;

	if ( !pItemImpl )
		return osl_File_E_INVAL;

	switch ( pItemImpl->uType  )
	{
	case DIRECTORYITEM_DRIVE:
		return osl_getDriveInfo( Item, pStatus, uFieldMask );
	case DIRECTORYITEM_SERVER:
		return osl_getServerInfo( Item, pStatus, uFieldMask );
	default:
		break;
	}

	if ( uFieldMask & osl_FileStatus_Mask_Validate )
	{
		HANDLE	hFind = lpfnFindFirstFile( pItemImpl->szFullPath, &pItemImpl->FindData );

		if ( hFind != INVALID_HANDLE_VALUE )
			FindClose( hFind );
		else
			return MapError( GetLastError() );

		uFieldMask &= ~	osl_FileStatus_Mask_Validate;
	}

	/* If no fields to retrieve left ignore pStatus */
	if ( !uFieldMask )
		return osl_File_E_None;

	/* Otherwise, this must be a valid pointer */
	if ( !pStatus )
		return osl_File_E_INVAL;

	if ( pStatus->uStructSize != sizeof(oslFileStatus) )
		return osl_File_E_INVAL;

	pStatus->uValidFields = 0;

	/* File time stamps */

	if ( (uFieldMask & osl_FileStatus_Mask_ModifyTime) && 
		FileTimeToTimeValue( &pItemImpl->FindData.ftLastWriteTime, &pStatus->aModifyTime ) )
		pStatus->uValidFields |= osl_FileStatus_Mask_ModifyTime;

	if ( (uFieldMask & osl_FileStatus_Mask_AccessTime) && 
		FileTimeToTimeValue( &pItemImpl->FindData.ftLastAccessTime, &pStatus->aAccessTime ) )
		pStatus->uValidFields |= osl_FileStatus_Mask_AccessTime;

	if ( (uFieldMask & osl_FileStatus_Mask_CreationTime) && 
		FileTimeToTimeValue( &pItemImpl->FindData.ftCreationTime, &pStatus->aCreationTime ) )
		pStatus->uValidFields |= osl_FileStatus_Mask_CreationTime;

	/* Most of the fields are already set, regardless of requiered fields */

	rtl_uString_newFromStr( &pStatus->strFileName, pItemImpl->FindData.cFileName );
	pStatus->uValidFields |= osl_FileStatus_Mask_FileName;

	pStatus->eType = pItemImpl->FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? osl_File_Type_Directory : osl_File_Type_Regular;
	pStatus->uValidFields |= osl_FileStatus_Mask_Type;

	pStatus->uAttributes = pItemImpl->FindData.dwFileAttributes;
	pStatus->uValidFields |= osl_FileStatus_Mask_Attributes;

	pStatus->uFileSize = (sal_uInt64)pItemImpl->FindData.nFileSizeLow + ((sal_uInt64)pItemImpl->FindData.nFileSizeHigh << 32);
	pStatus->uValidFields |= osl_FileStatus_Mask_FileSize;

	if ( (uFieldMask & osl_FileStatus_Mask_NativePath) && pStatus->pstrNativePath != NULL )
	{
		rtl_uString_newFromStr( pStatus->pstrNativePath, pItemImpl->szFullPath );
		pStatus->uValidFields |= osl_FileStatus_Mask_NativePath;
	}

	if ( (uFieldMask & osl_FileStatus_Mask_FilePath) && pStatus->pstrFilePath != NULL )
	{
		rtl_uString	*strNativePath = NULL;

		rtl_uString_newFromStr( &strNativePath, pItemImpl->szFullPath );
		osl_normalizePath( strNativePath, pStatus->pstrFilePath );
		rtl_uString_release( strNativePath );
		pStatus->uValidFields |= osl_FileStatus_Mask_FilePath;
	}

	return osl_File_E_None;
}

/****************************************************************************/
/*	osl_getVolumeInformation */
/****************************************************************************/

BOOL WINAPI GetVolumeInformationW9x(
	LPCWSTR	lpRootPathName,
	LPWSTR	lpVolumeNameBuffer,
	DWORD	nVolumeNameSize,  
	LPDWORD	lpVolumeSerialNumber, 
	LPDWORD	lpMaximumComponentLength,
	LPDWORD	lpFileSystemFlags,
	LPWSTR	lpFileSystemName,
	DWORD	nFileSystemNameSize )
{
	BOOL	fSuccess;
	LPSTR	lpVolumeNameBufferA = lpVolumeNameBuffer ? calloc( nVolumeNameSize, sizeof(CHAR) ) : NULL;
	LPSTR	lpFileSystemNameA = lpFileSystemName ? calloc( nFileSystemNameSize, sizeof(CHAR) ) : NULL;

	int		nRootPathLen = WideCharToMultiByte( CP_THREAD_ACP, 0, lpRootPathName, -1, NULL, 0, NULL, NULL );
	LPSTR	lpRootPathNameA = calloc(nRootPathLen, sizeof(CHAR) );

	WideCharToMultiByte( CP_THREAD_ACP, 0, lpRootPathName, -1, lpRootPathNameA, nRootPathLen, NULL, NULL );

	fSuccess = GetVolumeInformationA( lpRootPathNameA, lpVolumeNameBufferA, nVolumeNameSize, 
		lpVolumeSerialNumber, lpMaximumComponentLength, lpFileSystemFlags,
		lpFileSystemNameA, nFileSystemNameSize );

	free( lpRootPathNameA );

	if ( lpVolumeNameBufferA )
		free( lpVolumeNameBufferA );
	if ( lpFileSystemNameA )
		free( lpFileSystemNameA );

	return fSuccess;
}

oslFileError SAL_CALL osl_getVolumeInformation( rtl_uString *strDirectory, oslVolumeInfo *pInfo, sal_uInt32 uFieldMask )
{
	rtl_uString	*strSystemPath = NULL;
	WCHAR	szVolumeNameBuffer[MAX_PATH];
	WCHAR	szFileSystemName[MAX_PATH];
	DWORD	dwSerial;
	DWORD	dwMaxComponentLength;
	DWORD	dwFlags;
	oslFileError	error =osl_File_E_INVAL;


	if (osl_getSystemPathFromNormalizedPath( strDirectory, &strSystemPath )==osl_File_E_INVAL)
	{
		return osl_File_E_INVAL;
	}

	if ( GetVolumeInformation( rtl_uString_getStr( strSystemPath), szVolumeNameBuffer, MAX_PATH, &dwSerial, &dwMaxComponentLength, &dwFlags,
		 szFileSystemName, MAX_PATH ) )
	{
		pInfo->uValidFields = 0xFF;
		pInfo->uAttributes = 0;
		pInfo->uTotalSpace = 0;
		pInfo->uUsedSpace = 0;
		pInfo->uFreeSpace = 0;
		pInfo->uMaxNameLength = dwMaxComponentLength;
		pInfo->uMaxPathLength = MAX_PATH;
		rtl_uString_newFromStr( pInfo->pstrFileSystemName, szFileSystemName );

		rtl_uString_acquire( strDirectory );
		pInfo->pDeviceHandle = (oslVolumeDeviceHandle)strDirectory;

		error = osl_File_E_None;
	}

	rtl_uString_release( strSystemPath );

	return error;
}

oslFileError SAL_CALL osl_unmountVolumeDevice( oslVolumeDeviceHandle Handle )
{
	return osl_File_E_None;
}

oslFileError SAL_CALL osl_automountVolumeDevice( oslVolumeDeviceHandle Handle )
{
	return osl_File_E_None;
}

oslFileError SAL_CALL osl_acquireVolumeDeviceHandle( oslVolumeDeviceHandle Handle )
{
	if ( Handle )
		rtl_uString_acquire( (rtl_uString *)Handle );
	return osl_File_E_None;
}

oslFileError SAL_CALL osl_releaseVolumeDeviceHandle( oslVolumeDeviceHandle Handle )
{
	if ( Handle )
		rtl_uString_release( (rtl_uString *)Handle );
	return osl_File_E_None;
}

oslFileError SAL_CALL osl_getVolumeDeviceMountPath( oslVolumeDeviceHandle Handle, rtl_uString **pstrPath )
{
	rtl_uString_newFromStr( pstrPath, (LPCWSTR)Handle );
	return osl_File_E_None;
}

/****************************************************************************/
/*	osl_normalizePath */
/****************************************************************************/

// Tests, whether the character is legal for pathnames
static BOOL testChar(sal_Unicode character)
{
	if (character<L' ')
		return FALSE;
	
	// Forbidden characters 
	switch (character)
	{
		case L'?':
		case L'<':
		case L'>':
		case L'|':
		case L'*':
		case L'"':
		case L':':
			return FALSE;
			break;

		default:
			break;
	}

	return TRUE;
}

// Tests, whether a character is a slash or a backslash and converts a backslash into a slash
static BOOL slash(sal_Unicode *character)
{
	if (*character==L'/')
		return TRUE;
	
	if (*character==L'\\') 
	{
		*character=L'/';
		return TRUE;
	}
	
	return FALSE;	
}

static sal_Bool isColon( sal_Unicode c )
{
	switch ( c )
	{
	case L':':
	case L'|':
		return sal_True;
	default:
		return sal_False;
	}
}

//	Tests, whether the string starts with a drive-letter
static BOOL testDrive(sal_Unicode *urlLocal)
{
	if(isalpha(urlLocal[0]) && isColon(urlLocal[1]) && slash(&urlLocal[2]))
		return TRUE;
	else 
		return FALSE;
}

// Tests, whether the string starts with "./" to determine the localhost
static BOOL testLocalPoint(sal_Unicode *urlLocal, sal_Int32 nLength, sal_Int32 *pBeginServer, sal_Int32 *pBeginPath)
{
	if (nLength>=2)
	{
		if (urlLocal[0]==L'.' && slash(&urlLocal[1]))
		{
			if (testDrive(urlLocal+2))
			{	
				*pBeginServer=2;
				*pBeginPath=4;
				return TRUE;
			}			   
		}
	}		
	return FALSE;
}


// Tests, whether the string starts with "localhost/" 
static BOOL testLocalHost(sal_Unicode *urlLocal, sal_Int32 nLength, sal_Int32 *pBeginServer, sal_Int32 *pBeginPath)
{
	const sal_Unicode *hostName=L"localhost";

	if (nLength<9)
		return FALSE;

	if (rtl_ustr_equalsIgnoreCase_WithLength( urlLocal, 9, hostName, 9 ))
	{
		if (!slash(&urlLocal[9]))
			return FALSE;

		if (testDrive(urlLocal+10))
		{
			*pBeginServer=10;
			*pBeginPath=13;
			return TRUE;
		}
	}

	return FALSE;
}

// Tests, whether the string starts with "127.0.0.1" to determine the localhost
static BOOL testLocalIP(sal_Unicode *urlLocal, sal_Int32 nLength, sal_Int32 *pBeginServer, sal_Int32 *pBeginPath)
{
	sal_Int32 hostIP[4]={127,0,0,1};
	sal_Int32 serverIP[4];
	sal_Int32 nCount=0;
	sal_Int32 c_loop=0;
	sal_Int32 c_IP=0;

	// Tests, whether the characters are digits and extracts the four components of the IP-Address
	while (nCount<nLength && (!slash(&urlLocal[nCount])))
	{
		if (iswdigit(urlLocal[nCount]))
			c_IP=10*c_IP+(urlLocal[nCount]-L'0');
		else if (urlLocal[nCount]==L'.')
			{
				serverIP[c_loop]=c_IP;
				c_loop++;
				c_IP=0;	  			
			}
			else 
			{
				// No digit and no point
				return FALSE;
			}
		
		nCount++;			 
	}
	
	// Consists the IP-Addres of four components 
	if (c_loop!=3)
		return FALSE;

	serverIP[c_loop]=c_IP;

	// Compare the four components with 127.0.0.1
	for(c_loop=0;c_loop<4;c_loop++)
	{
		if (serverIP[c_loop]!=hostIP[c_loop])
		return FALSE;
	}

	nCount++;

	// Tests whether the IP-Address is followed by a drive-letter
	if (testDrive(urlLocal+nCount))
	{
		*pBeginServer=nCount;
		*pBeginPath=nCount+3;
		return TRUE;
	}

  	return FALSE;
}


// Examination of the server-name
static sal_uInt64 examineServer(sal_Unicode *urlLocal, sal_Int32 nLength, sal_Int32 *pBeginServer, sal_Int32 *pBeginPath)
{
	sal_Int32 nCount;
	
	// CASE: server= ./ (localhost)								   
	if (testLocalPoint(urlLocal, nLength, pBeginServer, pBeginPath))
		return PATHTYPE_LOCAL;
	
	// CASE: server= localhost/									   	
	if (testLocalHost(urlLocal, nLength, pBeginServer, pBeginPath))
		return PATHTYPE_LOCAL;
		
	// CASE: server= 127.0.0.1/
	if (testLocalIP(urlLocal, nLength, pBeginServer, pBeginPath))
		return PATHTYPE_LOCAL;

	// CASE: other server
	nCount=0;
	if (nLength>0 && (isalpha(urlLocal[0]) || isdigit(urlLocal[0])))
	{
		while(nCount<nLength && (!slash(&urlLocal[nCount])))
		{
			if (!testChar(urlLocal[nCount]))
				return PATHTYPE_ERROR;
			nCount++;
		}

		if(nCount>MAX_PATH)
			return PATHTYPE_ERROR;

		*pBeginServer=0;
		*pBeginPath=nCount;
		return PATHTYPE_SERVER;
	}
		
	return PATHTYPE_ERROR;
}	
			
// Examination of the beginning of the PathURL
sal_Int64 examineDrvSrv(sal_Unicode *urlPath, sal_Int32 nLength, sal_Int32 *pBeginServer, sal_Int32 *pBeginPath)
{
	sal_Unicode *uncName=L"UNC";
	sal_uInt64 nType;
	
	// CASE: Localhost	
	if (testDrive(urlPath))
	{
		*pBeginServer=0;
		*pBeginPath=2;

		return PATHTYPE_LOCAL;
	}

	// CASE: Path starts with "//" (but not "//?/")
	if (nLength>2 && slash(&urlPath[0]) && slash(&urlPath[1]))
	{
		nType=examineServer(urlPath+2,nLength-2,pBeginServer,pBeginPath);
		if (nType!=PATHTYPE_ERROR)
		{
			*pBeginServer+=2;
			*pBeginPath+=2;
			return nType;
		}
	}

	// CASE: Path starts with "//?/" (Only Windows2000)
	if (nLength>4 && slash(&urlPath[0]) && slash(&urlPath[1]) && urlPath[2]==L'?' && slash(&urlPath[3]))
	{
		if (testWin2000())
		{
			// CASE: Path starts with "//?/UNC/" (Only Windows2000)
			if (nLength>8 && rtl_ustr_equalsIgnoreCase_WithLength( urlPath+4, 3, uncName, 3 ))
			{	 
				if (slash(&urlPath[7]))
				{
					nType=examineServer(urlPath+8,nLength-8,pBeginServer,pBeginPath);
					if (nType!=PATHTYPE_ERROR)
					{
						*pBeginServer+=8;
						*pBeginPath+=8;
						nType|=PATHTYPE_WIN2000;
						return nType;
					}
				}
			}				  
			else if (testDrive(urlPath+4))
			{	
				*pBeginServer=4;
				*pBeginPath=6;
				return PATHTYPE_LOCAL | PATHTYPE_WIN2000;
			}
		}

		return PATHTYPE_ERROR;	
	}

	return PATHTYPE_ERROR;
}

// Tests the token and delivers a pointer to the next token
static BOOL testToken(sal_Unicode **pToken)
{
	sal_Int32 nCount=1;
	sal_Int32 nLength;
	const sal_Int32 CMAXToken=MAX_PATH_NT4;

	sal_Int32 nFirstAlpha=0;
	sal_Int32 nLastAlpha=0;
	sal_Int32 nFirstPoint=0;
	sal_Int32 nLastPoint=0;
	sal_Int32 nFirstSpace=0;
	sal_Int32 nLastSpace=0;
	
	nLength=rtl_ustr_getLength(*pToken);
	if (nLength==1)
	{
		*pToken+=1;
		return TRUE;
	}

	while (nCount<nLength && (!slash(*pToken+nCount)))
	{
		// Forbidden character ?
		if (*(*pToken+nCount)==L':' || (!testChar(*(*pToken+nCount))))
			return FALSE;

		// CASE: '.'
		if (*(*pToken+nCount)==L'.')
		{
			if (nFirstPoint==0)
			{
				nFirstPoint=nCount;
				nLastPoint=nCount;
			}
			else 
				nLastPoint=nCount;
		}

		// CASE: ' '
		if (*(*pToken+nCount)==L' ')
		{
			if (nFirstSpace==0)
			{
				nFirstSpace=nCount;
				nLastSpace=nCount;
			}
			else 
				nLastSpace=nCount;
		}

		// CASE: A-Z 0-9
		if (iswalpha(*(*pToken+nCount)) || iswdigit(*(*pToken+nCount)))
		{
			if (nFirstAlpha==0)
			{
				nFirstAlpha=nCount;
				nLastAlpha=nCount;
			}
			else		
				nLastAlpha=nCount;
		}
		nCount++;
	}

//	if (nCount>=CMAXToken || nCount==1)
//		return FALSE;

//	if ((nFirstAlpha==0) || (nLastPoint>nLastAlpha) || (nLastSpace>nLastAlpha) || (nFirstSpace==1) )
//		return FALSE;		

	*pToken+=nCount;
 	return TRUE;
}

// Analyzes the path
static BOOL analyzePath(sal_Unicode *strPath, sal_Int32 nLength, sal_uInt64 nType)
{
	sal_Unicode *pToken;
	sal_Int32 MAXIMPATH=MAX_PATH_NT4;

	if (nType & PATHTYPE_WIN2000)
		MAXIMPATH=MAX_PATH_WIN2000;
  
	if (nLength>MAXIMPATH || nLength==0)
		return FALSE;

	// Examines the tokens of the path
	pToken=strPath;	 	
	while ((pToken-strPath)<nLength)
	{
		if (!testToken(&pToken))
			return FALSE;		
	}

	return TRUE;
}

// Converts a systempath into a normalized path
oslFileError SAL_CALL osl_normalizePath( rtl_uString *strSysPath, rtl_uString **strPath )
{
	rtl_uString *sysStr=NULL;
	rtl_uString	*dirStr=NULL;
	sal_Unicode *urlPath;
	sal_Int32 nBeginServer=0;
	sal_Int32 nBeginPath=0;

	sal_uInt64 system_Unc_Mask=0x0000000f;
	sal_uInt64 nType=PATHTYPE_ERROR;
	
	sal_Int32 nLength;
	sal_Int32 MAXIMPATH=MAX_PATH_NT4;

	if ((nLength=rtl_uString_getLength(strSysPath))<1) 
		return osl_File_E_INVAL;

	rtl_uString_newFromString(&sysStr,strSysPath);
	urlPath=rtl_uString_getStr(sysStr);

	// Analyzing of the server- / drive-name
	nType=examineDrvSrv(urlPath, nLength, &nBeginServer, &nBeginPath);
	if (nType==PATHTYPE_ERROR)
	{
		rtl_uString_release(sysStr);
		return osl_File_E_INVAL;
	}

	// Windows2000 ?
	if (nType & PATHTYPE_WIN2000)
		MAXIMPATH=MAX_PATH_WIN2000;

	// Analyzing of the path
	if((nBeginPath-nBeginServer)>MAXIMPATH || (!analyzePath(urlPath+nBeginPath, nLength-nBeginPath, nType)))
	{
		rtl_uString_release(sysStr);
		return osl_File_E_INVAL;
	}

	switch (nType & system_Unc_Mask)
	{
		// Localhost
		case PATHTYPE_LOCAL:
			rtl_uString_new_WithLength( &dirStr, nLength-nBeginServer+5);
			wcscpy(rtl_uString_getStr(dirStr),L"//./");
			wcscpy(rtl_uString_getStr(dirStr)+4,urlPath+nBeginServer);
			break;

		// other server
		case PATHTYPE_SERVER:
			rtl_uString_new_WithLength( &dirStr, nLength-nBeginServer+2);
			wcscpy(rtl_uString_getStr(dirStr),L"//");
			wcscpy(rtl_uString_getStr(dirStr)+2,urlPath+nBeginServer);
			break;
		
		default:
			return osl_File_E_INVAL;
	}

	rtl_uString_release(sysStr);
	dirStr->length=rtl_ustr_getLength(rtl_uString_getStr(dirStr));
	rtl_uString_assign(strPath, dirStr);
	rtl_uString_release(dirStr);

	return osl_File_E_None;
}

/****************************************************************************/
/*	osl_getNormalizedPathFromFileURL */
/****************************************************************************/

// Converts a hexadecimal value of a given string into a sal_Int32-value
static sal_Int32 getHexFromString(sal_Unicode *str, sal_Int32 digit)
{
	sal_Int32 nHex=0;		
	sal_Int32 nCount;
	
	for(nCount=0;nCount<digit;nCount++)
	{
		if (iswdigit(str[nCount]))
			nHex=16*nHex+(str[nCount]-L'0');
		else if (str[nCount]>=L'a' && str[nCount]<=L'f')
			nHex=16*nHex+(str[nCount]-('f'-0x0f));
		else if (str[nCount]>=L'A' && str[nCount]<=L'F')
			nHex=16*nHex+(str[nCount]-('F'-0x0F));
		else return -1;
	}

	return nHex;
}											 
	
// Converts a FILE-URL into a normalized path														  
oslFileError SAL_CALL osl_getNormalizedPathFromFileURL( rtl_uString *urlPath, rtl_uString **strPath) 
{
	oslFileError error=osl_File_E_INVAL;		
	rtl_uString *filePath=NULL;
	sal_Unicode *strFile=L"file://";
	sal_Unicode *urlLocal;
	sal_Unicode *dirLocal;
	sal_uInt64 nType;
	sal_Int32 nBeginServer;
	sal_Int32 nBeginPath;
	sal_Int32 nCount=0;
	sal_Int32 nLength;
	sal_Int32 nPath=0;
	sal_Int32 nHex=0;
															 
	if ((nLength=rtl_uString_getLength(urlPath))<7)
		return osl_File_E_INVAL;
	if (nLength>MAX_PATH+5)
		return osl_File_E_INVAL;

	urlLocal=rtl_uString_getStr(urlPath);			   													   
	if (!rtl_ustr_equalsIgnoreCase_WithLength(urlLocal, 7, strFile, 7))
		return osl_File_E_INVAL;

	rtl_uString_new_WithLength( &filePath, nLength-2 );
	wcscpy(rtl_uString_getStr(filePath),L"//");
	nPath=2;
	nCount=7;

	// Path start with "///" (localhost)
	if (urlLocal[nCount]==L'/')
	{
		wcscpy(rtl_uString_getStr(filePath)+nPath, L"./");
		nPath+=2;
		nCount++;
	}

	dirLocal=rtl_uString_getStr(filePath);
	while(nCount<nLength)
	{	
		// converts a '|' into a ':'
		if (urlLocal[nCount]==L'|')
		{
			dirLocal[nPath]=L':';
			nPath++;
			nCount++;
		}
		else if (urlLocal[nCount]==L'%')
		{
			// converts all hexadecimal codes into characters
			if ((nHex=getHexFromString(urlLocal+nCount+1,2))==-1)
				return error;
			dirLocal[nPath]=(sal_Unicode)nHex;
			nPath++;
			nCount+=3;
		}
		else 
		{
			dirLocal[nPath]=urlLocal[nCount];
			nPath++;
			nCount++;
		}
	}
	dirLocal[nPath]=L'\0';

	nLength=nPath;
	filePath->length=nLength;
	
	// Examines the path
	nType=examineDrvSrv(dirLocal, nLength, &nBeginServer, &nBeginPath);
	if (nType!=PATHTYPE_ERROR && (analyzePath(dirLocal+nBeginPath, nLength-nBeginPath, nType)))
	{
		rtl_uString_assign(strPath,filePath);	
		error=osl_File_E_None;
	}
	
	rtl_uString_release(filePath);
	return error;
}

/****************************************************************************/
/*	osl_getSystemPathFromNormalizedPath */
/****************************************************************************/

// Converts a normalized path into a system path
/*
oslFileError SAL_CALL osl_getSystemPathFromNormalizedPath( rtl_uString *normPath, rtl_uString **strPath)
{
	rtl_uString *dirPath=NULL;

	sal_Unicode *dirLocal;	

	sal_Int32 nLength;
	sal_Int32 nCount=0;
	
	sal_uInt64 nType;
	
	sal_Int32 nBeginServer;
	sal_Int32 nBeginPath;
	
	sal_Int32 MAXIMPATH=MAX_PATH_NT4;

	if (testWin2000())
		MAXIMPATH=MAX_PATH_WIN2000;

	nLength=rtl_uString_getLength(normPath);
	dirLocal=rtl_uString_getStr(normPath);

	// Analyzes the server- / drive-name
	nType=examineDrvSrv(dirLocal, nLength, &nBeginServer, &nBeginPath);
			
	// localhost
	if (nType==PATHTYPE_LOCAL)
	{
		if (nBeginServer==0)
			return osl_File_E_INVAL;
	}
	// other server
	else if (nType!=PATHTYPE_SERVER)
	{
		return osl_File_E_INVAL;
	}

	// Analyzing the path
	if((nBeginPath-nBeginServer)>MAXIMPATH+5 || (!analyzePath(dirLocal+nBeginPath, nLength-nBeginPath, nType)))
	{
		return osl_File_E_INVAL;
	}

	if (dirLocal[2]==L'.' && dirLocal[3]==L'/')
	{
		if (testWin2000())
		{
			// Let a systempath begin with \\?\ to cancel parsing (Only Windows2000)
			rtl_uString_newFromString(&dirPath, normPath);
			*(rtl_uString_getStr(dirPath)+2)=L'?';
		}
		else
		{
			// Delete the //./ of the normalized path
			rtl_uString_newFromStr(&dirPath, dirLocal+4);
		}
	}
	else
	{
		if (testWin2000())
		{
			// Place \\?\UNC\ at the beginning of the UNC-path to cancel parsing (Only Windows2000)
			nLength=rtl_uString_getLength(normPath);
			
			rtl_uString_new_WithLength(&dirPath, nLength+7);
			wcscpy(rtl_uString_getStr(dirPath),L"\\\\?\\UNC\\");
			wcscpy(rtl_uString_getStr(dirPath)+8,dirLocal+2);
			dirPath->length=nLength+8-2;	// 8 : Lenght of "\\?\UNC\"
											// 2 : Starting Point in the path
		}
		else
		{
			rtl_uString_newFromString(&dirPath, normPath);
		}
	}
	
	dirLocal=rtl_uString_getStr(dirPath);
	nLength=rtl_uString_getLength(dirPath);

	// Convert all '/' into '\' 
	// and '|' into ':'
	while (nCount<nLength)
	{
		if (dirLocal[nCount]==L'/')
			dirLocal[nCount]=L'\\';

		if (dirLocal[nCount]==L'|')
			dirLocal[nCount]=L':';
		nCount++;
	}

	rtl_uString_assign(strPath, dirPath);
	rtl_uString_release(dirPath);
	return osl_File_E_None;
}
*/

/****************************************************************************/
/*	osl_getFileURLFromNormalizedPath */
/****************************************************************************/

// Converts a normalized path into a FILE-URL
oslFileError SAL_CALL osl_getFileURLFromNormalizedPath(rtl_uString *normPath, rtl_uString **strPath)
{
	rtl_uString *dirPath=NULL;
	rtl_uString *urlPath=NULL;
	sal_Unicode *urlLocal;
	sal_Unicode *dirLocal;

	sal_Char *cTemp;
	sal_uInt64 nType;
	
	sal_Int32 nBeginServer;
	sal_Int32 nBeginPath;
	
	sal_Int32 nCount=0;		// Counter of the source string
	sal_Int32 nLength;
	sal_Int32 nUrl=0;		// Couner of the destination string
	
	sal_Int32 MAXIMPATH=MAX_PATH_NT4;

	if (testWin2000())
		MAXIMPATH=MAX_PATH_WIN2000;

	rtl_uString_newFromString(&dirPath,normPath);
	nLength=rtl_uString_getLength(dirPath);
	dirLocal=rtl_uString_getStr(dirPath);

	// Analyzes the server- / drive-name
	nType=examineDrvSrv(dirLocal, nLength, &nBeginServer, &nBeginPath);
			
	// localhost
	if (nType==PATHTYPE_LOCAL)
	{
		if (nBeginServer==0)
			return osl_File_E_INVAL;
		
		rtl_uString_new_WithLength( &urlPath, MAXIMPATH+5 );
		wcscpy(rtl_uString_getStr(urlPath),L"file:///");
		nUrl=8;
		nCount=nBeginServer;
	}
	// other server
	else if (nType==PATHTYPE_SERVER)
		{
			rtl_uString_new_WithLength( &urlPath, MAXIMPATH+5 );
			wcscpy(rtl_uString_getStr(urlPath),L"file://");
			nUrl=7;
			nCount=nBeginServer;
		}
		else
			return osl_File_E_INVAL;

	// Analyzing the path
	if((nBeginPath-nBeginServer)>MAXIMPATH+5 || (!analyzePath(dirLocal+nBeginPath, nLength-nBeginPath, nType)))
	{
		rtl_uString_release(urlPath);
		return osl_File_E_INVAL;
	}

	urlLocal=rtl_uString_getStr(urlPath);
	cTemp=rtl_allocateMemory(2*sizeof(char));
	while (nCount<nLength)
	{
		// converts all ':' into '|'
		if (dirLocal[nCount]==L':')
		{
			urlLocal[nUrl]=L'|';
			nUrl++;
		}
		// converts all characters into the hexadecimal value (apart from 'a' - 'z' , 'A' - 'Z' , '0' - '9', '.' , '/' , '\'
		else if ((!slash(dirLocal+nCount)) && (dirLocal[nCount]!=L'.') &&
				 (!iswdigit(dirLocal[nCount])) && (!iswalpha(dirLocal[nCount])))
			{
				// Uses the system encoding table
				WideCharToMultiByte(CP_ACP,0,  dirLocal+nCount, 1, cTemp, 2, NULL, NULL);
				swprintf( &urlLocal[nUrl], L"%%%02X", (unsigned char)*cTemp);
				
				if (nUrl+3>MAXIMPATH+5)
				{
					rtl_uString_release(urlPath);
					rtl_freeMemory(cTemp);
					return osl_File_E_INVAL;
				}
				nUrl+=3;
			}
		else
		{
			//'a' - 'z' , 'A' - 'Z' , '0' - '9', '.' , '/' , '\'
			urlLocal[nUrl]=dirLocal[nCount];
			nUrl++;
		}	 
		nCount++;
	}

	urlLocal[nUrl]=L'\0';
	urlPath->length=nUrl;

	rtl_uString_assign(strPath, urlPath);
	rtl_uString_release(urlPath);

	rtl_freeMemory(cTemp);	
	return osl_File_E_None;
}


/****************************************************************************/
/*	osl_searchNormalizedPath */
/****************************************************************************/

// Converts slashes into backslashes in a given path
static void convertSlashToBackslash(rtl_uString *slashPath, rtl_uString **backslashPath)
{
	rtl_uString *dirPath=NULL;
	sal_Unicode *localPath;
	sal_Int32 nCount=0;
	sal_Int32 nLength=0;

	rtl_uString_newFromString(&dirPath, slashPath);
	localPath=rtl_uString_getStr(dirPath);
	nLength=rtl_uString_getLength(dirPath);
	
	while (nCount<nLength)
	{
		if (localPath[nCount]==L'/')
			localPath[nCount]=L'\\';
		nCount++;
	}

	rtl_uString_assign(backslashPath, dirPath);
	rtl_uString_release(dirPath);

	return;
}

// Serches a File in an given searchpath
static oslFileError searchFile(rtl_uString *filePath, rtl_uString *searchPath, rtl_uString **strPath, BOOL fQualPath)
{
	rtl_uString *dirPath=NULL;
	rtl_uString *normPath=NULL;
	sal_Unicode *foundPath;
	sal_Unicode *foundFile;
	sal_Int32 nLength;
	sal_Int32 nTotalLength=0;
	sal_Int32 MAXIMPATH=MAX_PATH_NT4;

	if (fQualPath)
	{
		// CASE: full qualified path or FileURL
		nLength=rtl_uString_getLength(filePath);
		foundPath=rtl_allocateMemory((nLength+1)*sizeof(sal_Unicode));
		nTotalLength=lpfnSearchPath(L"", rtl_uString_getStr(filePath), NULL, nLength+1, foundPath, &foundFile);
	}
	else
	{
		// CASE: other
		if (testWin2000())
			MAXIMPATH=MAX_PATH_WIN2000;
	
		nLength=MAXIMPATH;
		foundPath=rtl_allocateMemory((MAXIMPATH+1)*sizeof(sal_Unicode));

		if (searchPath!=NULL)
			nTotalLength=lpfnSearchPath(rtl_uString_getStr(searchPath), rtl_uString_getStr(filePath), NULL, MAXIMPATH+1, foundPath, &foundFile);

		if (nTotalLength==0)
			nTotalLength=lpfnSearchPath(NULL, rtl_uString_getStr(filePath), NULL, MAXIMPATH+1, foundPath, &foundFile);
	}

	if (nTotalLength!=0 && nTotalLength<=nLength)
	{
		rtl_uString_newFromStr(&dirPath, foundPath);
		osl_normalizePath(dirPath,&normPath);
		rtl_uString_release(dirPath);

		rtl_uString_assign(strPath, normPath);
		rtl_uString_release(normPath);
		
		rtl_freeMemory(foundPath);
		return osl_File_E_None;
	}
			
	rtl_freeMemory(foundPath);		
	return osl_File_E_NOENT;
}

// Searches a file or directory and delivers the normalized path
oslFileError SAL_CALL osl_searchNormalizedPath( rtl_uString *filePath, rtl_uString *searchPath, rtl_uString **strPath)
{
	oslFileError error=osl_File_E_INVAL;
	rtl_uString *tempPath=NULL;
	rtl_uString *dirPath=NULL;
											 
	if (rtl_uString_getLength(filePath)<1) 
		return osl_File_E_INVAL;	
	
	if (osl_getNormalizedPathFromFileURL(filePath, &tempPath)!=osl_File_E_INVAL)
	{
		// CASE: FILE-URL
		error=osl_getSystemPathFromNormalizedPath(tempPath, &dirPath);
		rtl_uString_release(tempPath);
	}
	else 
	{
		// CASE: full qualified path
		error=osl_getSystemPathFromNormalizedPath(filePath, &dirPath);
	}

	if (error==osl_File_E_None)
	{
		error=searchFile(dirPath, NULL, strPath, TRUE);
	}
	else
	{
		// CASE: other
		convertSlashToBackslash(filePath, &dirPath);
		error=searchFile(dirPath, searchPath, strPath, FALSE);
	}

	rtl_uString_release(dirPath);
	return error;
}

/****************************************************************************/
/*	osl_getAbsolutePath */
/****************************************************************************/

// remove the last token of the directory-path
void removeLastToken(rtl_uString* strDirPath, rtl_uString **strPath)
{
	sal_Unicode*	pToken = strDirPath->buffer + strDirPath->length;

	if ( strDirPath->length == 0 )
	{
		rtl_uString_new( strPath );
		return;
	}

	if ( *(--pToken) == CHAR_SLASH )
		pToken--;

	while ( pToken > strDirPath->buffer && *pToken != CHAR_SLASH )
		pToken--;

	if ( pToken == strDirPath->buffer )
		rtl_uString_new( strPath );
	else 
		rtl_uString_newFromStr_WithLength( strPath , strDirPath->buffer , pToken - strDirPath->buffer+1 );	

	return;
}

oslFileError SAL_CALL osl_getAbsolutePath( rtl_uString* strDirBase, rtl_uString *strRelative, rtl_uString **strAbsolute )
{
	oslFileError	error = osl_File_E_INVAL;

	rtl_uString*	strServer = NULL;
	rtl_uString*	strRoot = NULL;
	rtl_uString*	strDirectory = NULL;
	rtl_uString*	strFile = NULL;
	rtl_uString*	strNormPath = NULL;
	rtl_uString*	strPath = NULL;
	rtl_uString*	strHelpPath = NULL;
	rtl_uString*	strConcatPath = NULL;

	sal_Unicode*	pStrRelEnd;
	sal_Unicode*	pToken;
	sal_Unicode*	pTokenStart;

	if ( !strDirBase || strDirBase->length == 0 )
		return osl_File_E_INVAL;

	if ( !strRelative || strRelative->length == 0 )
	{
		rtl_uString_newFromString( strAbsolute, strDirBase );
		return osl_File_E_None;
	}

	pStrRelEnd= strRelative->buffer + strRelative->length-1;
	pToken=strRelative->buffer;
	pTokenStart=pToken;

	// strRelative starts with "/"
	if ( *pToken == CHAR_SLASH )
		return osl_File_E_INVAL;

	if ( osl_normalizePath( strDirBase , &strNormPath ) != osl_File_E_None )
		return osl_File_E_INVAL;

	// Split the Path of the base-directory
	_osl_splitNormalizedPath( strNormPath, &strServer, &strRoot, &strDirectory, &strFile );

	rtl_uString_release( strNormPath );

	// strPath = Directory / File /
	rtl_uString_newConcat( &strPath, strDirectory, strFile );
	rtl_uString_newFromStr( &strHelpPath, L"/" );
	rtl_uString_newConcat( &strPath, strPath, strHelpPath );

	rtl_uString_release( strHelpPath );
	strHelpPath=NULL;

	// Analyze relative path
	while ( pToken <= pStrRelEnd )
	{
		// get next token
		while( *pToken != CHAR_SLASH   && pToken <= pStrRelEnd)
			pToken++;

		// analyze token
		if ( *pTokenStart == CHAR_POINT )
		{
			if ( pTokenStart+1 >= pStrRelEnd )
			{
				// Token: "." (Last token of the relative Path)
				pToken++;
			}
			else if ( pTokenStart+1 == pToken ) 
			{
				// Token: "./"
				pToken++;
			}

			if ( *(pTokenStart+1) == CHAR_POINT )
			{
				if ( pTokenStart+2 >= pStrRelEnd )
				{
					// Token: ".." (Last token of the relative Path)
					removeLastToken( strPath, &strHelpPath );

					rtl_uString_release( strPath );
					strPath = NULL;

					rtl_uString_newFromString( &strPath, strHelpPath );

					pToken++;
				}
				else if ( pTokenStart+2 == pToken )
				{
					// Token: "../"
					removeLastToken( strPath, &strHelpPath );

					rtl_uString_release( strPath );
					strPath = NULL;

					rtl_uString_newFromString( &strPath, strHelpPath );

					pToken++;
				}
			}
		}
		else 
		{
			// add the token to the temporary path
			if ( pToken != pTokenStart )
			{
				rtl_uString_newFromStr_WithLength( &strHelpPath, pTokenStart, pToken - pTokenStart+1 );
				rtl_uString_newConcat( &strPath, strPath, strHelpPath );

				rtl_uString_release( strHelpPath );
				strHelpPath = NULL;
			}

			pToken++;
		}

		pTokenStart=pToken;
	}
	
	// Concat all parts to a new normalized path
	rtl_uString_newFromString( &strConcatPath, strServer );

	rtl_uString_newConcat(&strConcatPath, strConcatPath, strRoot);

	if ( strPath->length )
	{
		rtl_uString_newConcat( &strConcatPath, strConcatPath, strPath );

		if ( *( strConcatPath->buffer + strConcatPath->length-1 ) == CHAR_SLASH )
			rtl_uString_newFromStr_WithLength( &strHelpPath, strConcatPath->buffer, strConcatPath->length-1 );
		else
			rtl_uString_newFromString( &strHelpPath, strConcatPath );
	}
	else 
		rtl_uString_newFromString( &strHelpPath, strConcatPath );

	error = osl_searchNormalizedPath( strHelpPath, NULL , strAbsolute );

	// Release temporary acquired strings

	if ( strConcatPath )
		rtl_uString_release( strConcatPath );

	if ( strHelpPath )
		rtl_uString_release( strHelpPath );

	if ( strPath )
		rtl_uString_release( strPath );

	if ( strServer )
		rtl_uString_release( strServer );

	if ( strRoot )
		rtl_uString_release( strRoot );

	if ( strDirectory )
		rtl_uString_release( strDirectory );

	if ( strFile )
		rtl_uString_release( strFile );
	
	return error;
}


/****************************************************************************/
/*	osl_getCanonicalName */
/****************************************************************************/

oslFileError SAL_CALL osl_getCanonicalName( rtl_uString *strRequested, rtl_uString **strValid )
{
	rtl_uString_newFromString(strValid, strRequested);

	return osl_File_E_None;
}


/****************************************************************************/
/*	osl_setFileAttributes */
/****************************************************************************/

// Sets attributes to a given file
oslFileError SAL_CALL osl_setFileAttributes( rtl_uString *filePath, sal_uInt64 uAttributes )
{
	oslFileError error;
	rtl_uString *sysPath=NULL;	
	DWORD dwFileAttributes=0;
	BOOL fSuccess=FALSE;
	
	if (uAttributes & osl_File_Attribute_ReadOnly) 
		dwFileAttributes=FILE_ATTRIBUTE_READONLY;
	if (uAttributes & osl_File_Attribute_Hidden)
		dwFileAttributes|=FILE_ATTRIBUTE_HIDDEN;
		
	// Converts the normalized path into a systempath
	error=osl_getSystemPathFromNormalizedPath(filePath, &sysPath);

	if (error==osl_File_E_INVAL)
		return error;
	
	fSuccess=lpfnSetFileAttributes(rtl_uString_getStr(sysPath), dwFileAttributes);
	
	rtl_uString_release(sysPath);
	
	if (!fSuccess)
		return osl_File_E_INVAL;
	else
		return osl_File_E_None;
}

/****************************************************************************/
/*	osl_setFileTime */
/****************************************************************************/

// Converts a TimeValue into a FILETIME
BOOL TimeValueToFileTime(const TimeValue *cpTimeVal, FILETIME *pFTime)
{
	SYSTEMTIME	BaseSysTime;
	FILETIME	BaseFileTime;
	FILETIME	FTime;
	__int64		localTime;
	BOOL		fSuccess = FALSE;

	BaseSysTime.wYear         = 1970;
	BaseSysTime.wMonth        = 1;
    BaseSysTime.wDayOfWeek    = 0;
    BaseSysTime.wDay          = 1;
    BaseSysTime.wHour         = 0;
    BaseSysTime.wMinute       = 0;
    BaseSysTime.wSecond       = 0;
    BaseSysTime.wMilliseconds = 0;

	if (cpTimeVal==NULL)			  
		return fSuccess;

	if ( SystemTimeToFileTime(&BaseSysTime, &BaseFileTime) )
	{
		__int64 timeValue;
		localTime=cpTimeVal->Seconds*(__int64)10000000+cpTimeVal->Nanosec/100;
		*(__int64 *)&FTime=localTime;
		fSuccess = 0 <= (timeValue= *((__int64 *)&BaseFileTime) + *((__int64 *) &FTime));
		if (fSuccess)
			*(__int64 *)pFTime=timeValue;
	}			
			 
	return fSuccess;  
}

// Sets the time parameters to a given file
oslFileError SAL_CALL osl_setFileTime( rtl_uString *filePath, TimeValue *aCreationTime, TimeValue *aLastAccessTime, TimeValue *aLastWriteTime)
{
	oslFileError error;
	rtl_uString *sysPath=NULL;	
	FILETIME *lpCreationTime=NULL;
	FILETIME *lpLastAccessTime=NULL;
	FILETIME *lpLastWriteTime=NULL;
	FILETIME ftCreationTime;
	FILETIME ftLastAccessTime;
	FILETIME ftLastWriteTime;
	HANDLE hFile;
	BOOL fSuccess;	

	
	error=osl_getSystemPathFromNormalizedPath(filePath, &sysPath);

	if (error==osl_File_E_INVAL)
		return error;

	hFile=lpfnCreateFile(rtl_uString_getStr(sysPath), GENERIC_WRITE, 0, NULL , OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	rtl_uString_release(sysPath);

	if (hFile==INVALID_HANDLE_VALUE)
		return osl_File_E_NOENT;
		
	if (TimeValueToFileTime(aCreationTime, &ftCreationTime))
		lpCreationTime=&ftCreationTime;

	if (TimeValueToFileTime(aLastAccessTime, &ftLastAccessTime))
		lpLastAccessTime=&ftLastAccessTime;

	if (TimeValueToFileTime(aLastWriteTime, &ftLastWriteTime))
		lpLastWriteTime=&ftLastWriteTime;

	fSuccess=SetFileTime(hFile, lpCreationTime, lpLastAccessTime, lpLastWriteTime);	

	CloseHandle(hFile);

	if (!fSuccess)
		return osl_File_E_INVAL;
	else 
		return osl_File_E_None;
}


/* create a new oslFileHandle from a native handle */
oslFileHandle importFileHandle(HANDLE hFile, LPCWSTR pszFileName)
{
      File_Impl * ret = (File_Impl *) rtl_allocateZeroMemory(sizeof(File_Impl));

      if (ret)
      {
              ret->hFile = hFile;
              if (pszFileName)
                      wcscpy(ret->szFileName, pszFileName);
      }

      return ret;
}


/****************************************************************************/

typedef struct tagDRIVEENUM {
	LPCTSTR	lpIdent;
	TCHAR	cBuffer[/*('Z' - 'A' + 1) * sizeof("A:\\") + 1*/256];
	LPCTSTR	lpCurrent;
} DRIVEENUM, * PDRIVEENUM, FAR * LPDRIVEENUM;

/****************************************************************************/

HANDLE WINAPI OpenLogicalDrivesEnum()
{
	LPDRIVEENUM	pEnum = NULL;

	pEnum = (LPDRIVEENUM)HeapAlloc( GetProcessHeap(), 0, sizeof(DRIVEENUM) );

	if ( pEnum )
	{
		DWORD	dwNumCopied = GetLogicalDriveStrings( sizeof(pEnum->cBuffer) - 1 / sizeof(TCHAR), pEnum->cBuffer );
		
		if ( dwNumCopied && dwNumCopied < sizeof(pEnum->cBuffer) / sizeof(TCHAR) )
		{
			pEnum->lpCurrent = pEnum->cBuffer;
			pEnum->lpIdent = L"tagDRIVEENUM";
		}
		else
		{
			HeapFree( GetProcessHeap(), 0, pEnum );
			pEnum = NULL;
		}
	}

	return pEnum ? (HANDLE)pEnum : INVALID_HANDLE_VALUE;
}

/****************************************************************************/

BOOL WINAPI EnumLogicalDrives( HANDLE hEnum, LPTSTR lpBuffer )
{
	BOOL		fSuccess = FALSE;
	LPDRIVEENUM	pEnum = (LPDRIVEENUM)hEnum;

	if ( pEnum )
	{
		int	nLen = lstrlen( pEnum->lpCurrent );

		if ( nLen )
		{
			CopyMemory( lpBuffer, pEnum->lpCurrent, (nLen + 1) * sizeof(TCHAR) );
			pEnum->lpCurrent += nLen + 1;
			fSuccess = TRUE;
		}
		else
			SetLastError( ERROR_NO_MORE_FILES );
	}
	else
		SetLastError( ERROR_INVALID_HANDLE );

	return fSuccess;
}

/****************************************************************************/

BOOL WINAPI CloseLogicalDrivesEnum( HANDLE hEnum )
{
	BOOL		fSuccess = FALSE;
	LPDRIVEENUM	pEnum = (LPDRIVEENUM)hEnum;

	if ( pEnum )
	{
		HeapFree( GetProcessHeap(), 0, pEnum );
		fSuccess = TRUE;
	}
	else
		SetLastError( ERROR_INVALID_HANDLE );

	return fSuccess;
}


