/*************************************************************************
 *
 *  $RCSfile: ports.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include "system.h"

#include <rtl/alloc.h>
#include <osl/diagnose.h>
#include <osl/thread.h>
#include <osl/ports.h>
#include "dirW9X.h"

#define OSL_PORT_PURGE_ALL		(PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR)


/*********************************************************
  globals
**********************************************************/
#define NUM_OF_PORTS_ENTRIES	14
#define TO_OSL		1
#define TO_WIN32	2

static sal_Char * PortNames[] =
{
	"COM1",
	"COM2",
	"COM3",
	"COM4",
	"COM5",
	"COM6",
	"COM7",
	"LPT1",
	"LPT2",
	"LPT3",
	"LPT4",
	"LPT5",
	"LPT6",
	"LPT7"
};
	

struct _osl_Port_Impl
{
	HANDLE				Porthandle;
	DWORD				PortType;
	DWORD 				AccessMode;
	oslThread			EventThread;
	sal_uInt32				Userdata;
	osl_Port_Callback	UserCallback;
};
typedef struct _osl_Port_Impl oslPortImpl;

struct _osl_Port_Enum_Impl
{
	sal_uInt32 nCurrent;
};

typedef struct _osl_Port_Enum_Impl *oslPortEnumImpl;

static void SAL_CALL eventThreadRun(void * pData)
{
	DWORD dwEvent;
	oslPortImpl* pPort = (oslPortImpl*) pData;

	while(TRUE)
	{
		if( ! WaitCommEvent( pPort->Porthandle, &dwEvent, NULL) )
		{
			// error while waiting for commEvent
			break;
		}
		else
		{
			pPort->UserCallback( dwEvent, (oslPort) pPort, pPort->Userdata);
		}
	}
}

oslPortError mapError( DWORD dwError )
{
	oslPortError retVal;
	retVal = osl_Port_E_Unknown;

	switch( dwError )
	{
	case 0:
		retVal = osl_Port_E_None;
		break;
	case ERROR_FILE_NOT_FOUND:
		retVal = osl_Port_E_PortNotFound;
		break;
	default:
		retVal = osl_Port_E_Unknown;
		break;
	}
	return retVal;
}

sal_uInt16 mapParityMode( sal_uInt16 dwPmode, sal_uInt8 nMapTo)
{
	sal_uInt16 retVal;

	if( nMapTo == TO_WIN32 )
	{
		switch( dwPmode )
		{
		case OSL_PORT_PARITY_EVEN:
			retVal = EVENPARITY;
			break;
		case OSL_PORT_PARITY_MARK:
			retVal = MARKPARITY;
			break;
		case OSL_PORT_PARITY_NONE:
			retVal = NOPARITY;
			break;
		case OSL_PORT_PARITY_ODD:
			retVal = ODDPARITY;
			break;
		case OSL_PORT_PARITY_SPACE:
			retVal = SPACEPARITY;
			break;
		default:
			retVal = NOPARITY;
			break;
	}

	}
	else
	{
		switch( dwPmode )
		{
		case EVENPARITY:
			retVal = OSL_PORT_PARITY_EVEN;
			break;
		case MARKPARITY:
			retVal = OSL_PORT_PARITY_MARK;
			break;
		case NOPARITY:
			retVal = OSL_PORT_PARITY_NONE;
			break;
		case ODDPARITY:
			retVal = OSL_PORT_PARITY_ODD;
			break;
		case SPACEPARITY:
			retVal = OSL_PORT_PARITY_SPACE;
			break;
		default:
			retVal = OSL_PORT_PARITY_NONE;
			break;
		}

	}
	return retVal;
}

sal_uInt16 mapStopBits( sal_uInt16 dwStopBit, sal_uInt8 nMapTo)
{
	sal_uInt16 retVal;
	if( nMapTo == TO_WIN32 )
	{
		switch( dwStopBit )
		{
		case OSL_PORT_STOPBITS_10:
			retVal = ONESTOPBIT;
			break;
		case OSL_PORT_STOPBITS_15:
			retVal = ONE5STOPBITS;
			break;
		case OSL_PORT_STOPBITS_20:
			retVal = TWOSTOPBITS;
			break;
		default:
			retVal = ONESTOPBIT;
			break;
		}
	}
	else
	{
		switch( dwStopBit )
		{
		case ONESTOPBIT:
			retVal = OSL_PORT_STOPBITS_10;
			break;
		case ONE5STOPBITS:
			retVal = OSL_PORT_STOPBITS_15;
			break;
		case TWOSTOPBITS:
			retVal = OSL_PORT_STOPBITS_20;
			break;
		default:
			retVal = OSL_PORT_STOPBITS_10;
			break;
		}
	}
	return retVal;
}

/*********************************************************
  port functions implementation
**********************************************************/

/** @return a port enumeration structure.
*/

oslPortEnum SAL_CALL osl_openPortEnum()
{
	oslPortEnumImpl pImpl = rtl_allocateMemory( sizeof( struct _osl_Port_Enum_Impl ) );
	pImpl->nCurrent = 0;

	return (oslPortEnum) pImpl;
}

/** Resets a enumeration pointer to the start pos.

    @param portEnum [in] a port enumeration opened with osl_openPortEnum.
*/
void SAL_CALL osl_resetPortEnum(oslPortEnum portEnum)
{
	((oslPortEnumImpl) portEnum)->nCurrent = 0;
}


/** Returns the next port name in a enumeration.

    @param portEnum [in] a port enumeration opened with osl_openPortEnum.
    @param strPortName [out] the string that receives the port name.
    @return sal_False if the end of the list is already reached, sal_True otherwise.
*/
sal_Bool SAL_CALL osl_getNextPortName( oslPortEnum portEnum, rtl_uString **strPortName )
{
	oslPortEnumImpl pImpl = (oslPortEnumImpl) portEnum;
	rtl_uString * strCurrentPort = NULL;

	while (pImpl->nCurrent <= NUM_OF_PORTS_ENTRIES)
	{
		oslPort aPort;

		rtl_uString_newFromAscii(&strCurrentPort, PortNames[pImpl->nCurrent]);
		
		if (osl_Port_E_None == osl_openPort(strCurrentPort, OSL_PORT_AC_NONE, &aPort))
		{
			osl_closePort(aPort);
			rtl_uString_release(strCurrentPort);
			strCurrentPort = NULL;
			break;
		}

		pImpl->nCurrent++;
	}

	if (strCurrentPort)
	{
		*strPortName = strCurrentPort;
		pImpl->nCurrent++;
		
		return sal_True;
	}

	return sal_False;
}

/** Release the enumeration data structure

    @param portEnum [in] a port enumeration opened with osl_openPortEnum.
 */

void SAL_CALL osl_closePortEnum( oslPortEnum portEnum )
{
	rtl_freeMemory(portEnum);
}

/** The openPort function opens a port for a connection.
	@param pPortName [in]
		The name of the port, like COM1 (win) or /dev/term/a (unx). This must be a null terminated string.
	@param nAccessMode [in]
		Specifies the type of access to the object. An application can obtain read access,
		write access, read-write access, or device query access. This parameter can be any
		combination of the OSL_PORT_AC_XXX constants.
	@param hPort [out]
		If the requested port is open, hPort is a valid Porthandle. Otherwise the handle is NULL.
	@return Error value.
*/
sal_uInt32 SAL_CALL osl_getPortTypeByName( rtl_uString *strName )
{
	if ( wcsstr( rtl_uString_getStr(strName) , L"COM") ) return OSL_PORT_TYPE_RS232;
	if ( wcsstr( rtl_uString_getStr(strName) , L"LPT") ) return OSL_PORT_TYPE_PARALLELPORT;

	return OSL_PORT_TYPE_UNKNOWN;
}

/** 
	The getPortTypeByName function returns the port type of the port with the name pName.
	The return value is one of the OSL_PORT_TYPE_XXX constants.

*/
oslPortError SAL_CALL osl_openPort( rtl_uString *strPortName , sal_uInt32 nAccessMode, oslPort *hPort )
{
	HANDLE			hComm = NULL;
	oslPortImpl*	pPort;
	COMMPROP		CommProp;

	CommProp.wPacketLength = sizeof(COMMPROP);

	pPort = NULL;
	
	pPort = malloc( sizeof(oslPortImpl) );
	if(! pPort ) return osl_Port_E_NoMemory;

	pPort->AccessMode = nAccessMode;
	pPort->Userdata = 0;
	pPort->UserCallback = NULL;
	pPort->EventThread = NULL;

	hComm = lpfnCreateFile( 
					rtl_uString_getStr(strPortName),
					nAccessMode,
                    0,
					NULL,
					OPEN_EXISTING,
                    0,				// NON-OVERLAPPED
                    NULL);

	if (hComm == INVALID_HANDLE_VALUE)
	{
		free(pPort);
		pPort = NULL;
		return mapError (GetLastError());
	}
	
	pPort->Porthandle = hComm;
	if( GetCommProperties( pPort->Porthandle, &CommProp) )
	{
		pPort->PortType = CommProp.dwProvSubType;
	}
	else
	{
		free(pPort);
		pPort = NULL;
		return mapError (GetLastError());  // error opening port; abort
	}
	(*hPort) = (oslPort) pPort;

	return osl_Port_E_None;
}

/** 
	The closePort function closes an openPort and frees all resources.
*/
oslPortError SAL_CALL osl_closePort( oslPort hPort )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;

	if (pPort->EventThread)
	{
		osl_terminateThread(pPort->EventThread);
		osl_destroyThread(pPort->EventThread);
	}
	
	PurgeComm(pPort->Porthandle, OSL_PORT_PURGE_ALL);
	CloseHandle(pPort->Porthandle);
	return osl_Port_E_None;
}


oslPortError SAL_CALL osl_getPortProperties( oslPort hPort, oslPortProperties *pPortProp )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	COMMPROP	CommProp;
	CommProp.wPacketLength = sizeof(COMMPROP);
	
	if( pPortProp->Size != sizeof( oslPortProperties ) )
		return osl_Port_E_DifferentStructSize;

	if( GetCommProperties( pPort->Porthandle, &CommProp) )
	{
		pPortProp->MaxTxQueue		= CommProp.dwMaxTxQueue;
		pPortProp->MaxRxQueue		= CommProp.dwMaxRxQueue;
		pPortProp->MaxBaud			= CommProp.dwMaxBaud;
		pPortProp->PortType			= CommProp.dwProvSubType;
		pPortProp->PortCapabilities = CommProp.dwProvCapabilities;
		pPortProp->SettableParams	= CommProp.dwSettableParams;
		pPortProp->SettableBaud		= CommProp.dwSettableBaud;
		pPortProp->SettableData		= CommProp.wSettableData;
		pPortProp->SettableStopParity = CommProp.wSettableStopParity ;
		pPortProp->CurrentTxQueue	= CommProp.dwCurrentTxQueue;
		pPortProp->CurrentRxQueue	= CommProp.dwCurrentRxQueue;
	}
	else
	{
		return mapError (GetLastError());
	}
	return osl_Port_E_None;
}


/** The osl_getPortProperties function fills a buffer with information about the 
	communications properties for a specified communications device. 
	
	 @param hPortProp [out]
		Structure of a oslPortProperties filled with properties for a specified communications device.

*/
oslPortError SAL_CALL osl_setEventHandler( oslPort hPort, osl_Port_Callback aCallback, sal_uInt32 nUserData )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;

	pPort->UserCallback = aCallback;
	pPort->Userdata = nUserData;
	pPort->EventThread = osl_createThread( eventThreadRun, (void*) pPort);

	if( ! pPort->EventThread ) 
		return osl_Port_E_Unknown;

	osl_resumeThread( pPort->EventThread );
	return osl_Port_E_None;
}

oslPortError SAL_CALL osl_removeEventHandler( oslPort hPort )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;

	if (pPort->EventThread)
	{
		osl_terminateThread(pPort->EventThread);
		osl_destroyThread(pPort->EventThread);
		pPort->EventThread = NULL;
	}
	pPort->UserCallback = NULL;
	pPort->Userdata = 0;

	return osl_Port_E_None;
}



oslPortError SAL_CALL osl_clearPortBreak( oslPort hPort )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	if( !ClearCommBreak( pPort->Porthandle ))
		return mapError (GetLastError());

	return osl_Port_E_None;
}

/** The osl_clearPortBreak function restores character transmission for a specified communications 
	device and places the transmission line in a nonbreak state. */

oslPortError SAL_CALL osl_setPortBreak( oslPort hPort )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	if ( !SetCommBreak( pPort->Porthandle ) )
		return 	mapError (GetLastError());

	return osl_Port_E_None;
}

/** The osl_setPortBreak function suspends character transmission for a specified communications 
	device and places the transmission line in a break state until the ClearCommBreak function is called. */


oslPortError SAL_CALL osl_getPortControlBlock( oslPort hPort, oslPortControlBlock *pPCB )
{
	DCB dcb = {0};
	oslSerialPortControlBlock * pSPCB = NULL;

	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if( pPort->PortType != pPCB->PortType)
		return osl_Port_E_InvalidType;

	switch( pPort->PortType )
	{
	case	OSL_PORT_TYPE_RS232:
		if( pPCB->Size != sizeof( oslSerialPortControlBlock ))
			return osl_Port_E_DifferentStructSize;

		pSPCB = (oslSerialPortControlBlock *) pPCB;
		
		if ( !GetCommState( pPort->Porthandle , &dcb) )
			return mapError( GetLastError() );
		
			pSPCB->BaudRate			=	dcb.BaudRate;           // current baud rate 
			pSPCB->Binary			=	dcb.fBinary ? sal_True : sal_False;			// binary mode, no EOF check 
			pSPCB->Parity			=	dcb.fParity ? sal_True : sal_False;			// enable parity checking 
			pSPCB->OutxCTSFlow		=	dcb.fOutxCtsFlow ? sal_True : sal_False;		// CTS output flow control 
			pSPCB->OutxDSRFlow		=	dcb.fOutxDsrFlow ? sal_True : sal_False;		// DSR output flow control 
			pSPCB->DTRControl		=	dcb.fDtrControl;		// DTR flow control type 
			pSPCB->DSRSensitivity	=	dcb.fDsrSensitivity ? sal_True : sal_False;	// DSR sensitivity 
			pSPCB->TXContinueOnXoff	=	dcb.fTXContinueOnXoff ? sal_True : sal_False;	// XOFF continues Tx 
			pSPCB->OutX				=	dcb.fOutX ? sal_True : sal_False;				// XON/XOFF out flow control 
			pSPCB->InX				=	dcb.fInX ? sal_True : sal_False;				// XON/XOFF in flow control 
			pSPCB->UseErrorChar		=	dcb.fErrorChar ? sal_True : sal_False;			// enable error replacement 
			pSPCB->Null				=	dcb.fNull ? sal_True : sal_False;				// enable null stripping 
			pSPCB->RTSControl		=	dcb.fRtsControl;		// RTS flow control 
			pSPCB->AbortOnError		=	dcb.fAbortOnError ? sal_True : sal_False;		// abort reads/writes on error 
			pSPCB->XonLim			=	dcb.XonLim;             // transmit XON threshold 
			pSPCB->XoffLim			=	dcb.XoffLim;            // transmit XOFF threshold 
			pSPCB->ByteSize			=	dcb.ByteSize;           // number of bits/byte, 4-8 
			pSPCB->ParityMode		=	mapParityMode( dcb.Parity, TO_OSL );	// 0-4=no,odd,even,mark,space 
			pSPCB->StopBits			=	mapStopBits( dcb.StopBits, TO_OSL );	// 0,1,2 = 1, 1.5, 2 
			pSPCB->XonChar			=	dcb.XonChar;            // Tx and Rx XON character 
			pSPCB->XoffChar			=	dcb.XoffChar;           // Tx and Rx XOFF character 
			pSPCB->ErrorChar		=	dcb.ErrorChar;          // error replacement character 
			pSPCB->EofChar			=	dcb.EofChar;            // end of input character 
			pSPCB->EvtChar			=	dcb.EvtChar;            // received event character 
			pSPCB->pData			=	NULL;

		break;

	case	OSL_PORT_TYPE_PARALLELPORT:
		pPCB->Size = sizeof( oslPortControlBlock );
		pPCB->PortType = OSL_PORT_TYPE_PARALLELPORT;
		break;
	default:
		return osl_Port_E_InvalidType;
	}

	return osl_Port_E_None;
}



oslPortError SAL_CALL osl_setPortControlBlock( oslPort hPort, const oslPortControlBlock *pPCB )
{
	DCB dcb = {0};
	oslSerialPortControlBlock * pSPCB = NULL;

	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if( pPort->PortType != pPCB->PortType)
		return osl_Port_E_InvalidType;

	switch( pPort->PortType )
	{
	case	OSL_PORT_TYPE_RS232:
		if( pPCB->Size != sizeof( oslSerialPortControlBlock ))
			return osl_Port_E_DifferentStructSize;

		pSPCB = (oslSerialPortControlBlock *) pPCB;

		dcb.BaudRate			=	pSPCB->BaudRate;		// current baud rate 
		dcb.fBinary				=	pSPCB->Binary ? 1 : 0;			// binary mode, no EOF check 
		dcb.fParity				=	pSPCB->Parity ? 1 : 0;			// enable parity checking 
		dcb.fOutxCtsFlow		=	pSPCB->OutxCTSFlow ? 1 : 0;		// CTS output flow control 
		dcb.fOutxDsrFlow		=	pSPCB->OutxDSRFlow ? 1 : 0;		// DSR output flow control 
		dcb.fDtrControl			=	pSPCB->DTRControl;		// DTR flow control type 
		dcb.fDsrSensitivity		=	pSPCB->DSRSensitivity ? 1 : 0;	// DSR sensitivity 
		dcb.fTXContinueOnXoff	=	pSPCB->TXContinueOnXoff ? 1 : 0;	// XOFF continues Tx 
		dcb.fOutX				=	pSPCB->OutX ? 1 : 0;				// XON/XOFF out flow control 
		dcb.fInX				=	pSPCB->InX ? 1 : 0;				// XON/XOFF in flow control 
		dcb.fErrorChar			=	pSPCB->UseErrorChar ? 1 : 0;		// enable error replacement 
		dcb.fNull				=	pSPCB->Null ? 1 : 0;				// enable null stripping 
		dcb.fRtsControl			=	pSPCB->RTSControl;		// RTS flow control 
		dcb.fAbortOnError		=	pSPCB->AbortOnError ? 1 : 0;		// abort reads/writes on error 
		dcb.XonLim				=	pSPCB->XonLim;			// transmit XON threshold 
		dcb.XoffLim				=	pSPCB->XoffLim;			// transmit XOFF threshold 
		dcb.ByteSize			=	pSPCB->ByteSize;			// number of bits/byte, 4-8 
		dcb.Parity				=	(BYTE) mapParityMode( pSPCB->ParityMode, TO_WIN32 );	// 0-4=no,odd,even,mark,space 
		dcb.StopBits			=	(BYTE) mapStopBits( pSPCB->StopBits, TO_WIN32 );		// 0,1,2 = 1, 1.5, 2 
		dcb.XonChar				=	pSPCB->XonChar;			// Tx and Rx XON character 
		dcb.XoffChar			=	pSPCB->XoffChar;			// Tx and Rx XOFF character 
		dcb.ErrorChar			=	pSPCB->ErrorChar;		// error replacement character 
		dcb.EofChar				=	pSPCB->EofChar;			// end of input character 
		dcb.EvtChar				=	pSPCB->EvtChar;			// received event character 

		if ( !SetCommState( pPort->Porthandle, &dcb ) )
			return mapError( GetLastError() );
		break;

	case	OSL_PORT_TYPE_PARALLELPORT:

		// Not implemented

		break;
	default:
		return osl_Port_E_InvalidType;
	}

	return osl_Port_E_None;
}


oslPortError SAL_CALL osl_getAndClearPortError( oslPort hPort, sal_uInt32* pErrorValueMask, oslPortState *pPortState )
{
	COMSTAT aComStat;	
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if ( !ClearCommError( pPort->Porthandle, pErrorValueMask, &aComStat ) )
		return mapError( GetLastError() );

	if( pPortState )
	{
		if( pPortState->Size != sizeof( oslPortState ))
			return osl_Port_E_DifferentStructSize;

		pPortState->CTSHold			=	aComStat.fCtsHold ? sal_True : sal_False;  // Tx waiting for CTS signal 
		pPortState->DSRHold			=	aComStat.fDsrHold ? sal_True : sal_False;  // Tx waiting for DSR signal  
		pPortState->RLSDHold		=	aComStat.fRlsdHold ? sal_True : sal_False; // Tx waiting for RLSD signal
		pPortState->XOFFHold		=	aComStat.fXoffHold ? sal_True : sal_False; // Tx waiting, XOFF char received 
		pPortState->XOFFSent		=	aComStat.fXoffSent ? sal_True : sal_False; // Tx waiting, XOFF char sent 
		pPortState->Eof				=	aComStat.fEof ? sal_True : sal_False;	  // EOF character sent 
		pPortState->Txim			=	aComStat.fTxim ? sal_True : sal_False;	  // character waiting for Tx 
		pPortState->BytesInQueue	=	aComStat.cbInQue;	  // bytes in input buffer 
		pPortState->BytesOutQueue	=	aComStat.cbOutQue;  // bytes in output buffer 
	}
	
	return osl_Port_E_None;
}

/** The getAndClearPortError function retrieves information about a communications error.
	The function is called when a communications error occurs, and it clears the device's error 
	flag to enable additional input and output (I/O) operations. 
	
	@param pErrorValueMask [out]
		Specifies the error that occurred. This parameter can be a combination of 
		the OSL_PORT_E_XXX constants.
*/
oslPortError SAL_CALL osl_purgePort( oslPort hPort, sal_uInt32 nFlags )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if ( !PurgeComm( pPort->Porthandle, nFlags ) )
		return mapError( GetLastError() );

	return osl_Port_E_None;
}

/** The purgePort function can discard all characters from the output or input 
	buffer of a specified communications resource. It can also terminate pending 
	read or write operations on the resource. 

	@param nFlags [in]
		Specifies the action to take. This parameter can be a combination of 
		the OSL_PORT_PURGE_XXX constants.
*/


oslPortError SAL_CALL osl_getPortEventMask( oslPort hPort, sal_uInt32 *nEventMask)
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if ( !GetCommMask( pPort->Porthandle, nEventMask ) )
		return mapError( GetLastError() );


	return osl_Port_E_None;
}

/** The osl_getPortEventMask function specifies a set of events to be monitored for a communications device. 
*/

oslPortError SAL_CALL osl_setPortEventMask( oslPort hPort, sal_uInt32 nEventMask )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if ( !SetCommMask( pPort->Porthandle, nEventMask ) )
		return mapError( GetLastError() );

	return osl_Port_E_None;
}

/** The osl_getPortEventMask function specifies a set of events to be monitored for a communications device. 
*/


oslPortError SAL_CALL osl_transmitPortChar( oslPort hPort, sal_Char cChar )
{
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	if ( !TransmitCommChar( pPort->Porthandle, cChar ) )
		return mapError( GetLastError() );

	return osl_Port_E_None;
}

/** The TransmitPortChar function transmits a specified character ahead of 
	any pending data in the output buffer of the specified communications device.
	This function is useful for sending an interrupt character (such as a ctrl+c) to a host system. 
 */

oslPortError SAL_CALL osl_setPortTimeout( oslPort hPort, const oslPortTimeouts* pPortTimeouts )
{
	COMMTIMEOUTS aCommTimeOut;

	oslPortImpl *pPort = (oslPortImpl *) hPort;

	if( pPortTimeouts->Size != sizeof( oslPortTimeouts ))
		return osl_Port_E_DifferentStructSize;

	aCommTimeOut.ReadIntervalTimeout			= pPortTimeouts->ReadIntervalTimeout;
	aCommTimeOut.ReadTotalTimeoutMultiplier		= pPortTimeouts->ReadTotalTimeoutMultiplier; 
    aCommTimeOut.ReadTotalTimeoutConstant		= pPortTimeouts->ReadTotalTimeoutConstant;
	aCommTimeOut.WriteTotalTimeoutMultiplier	= pPortTimeouts->WriteTotalTimeoutMultiplier; 
    aCommTimeOut.WriteTotalTimeoutConstant		= pPortTimeouts->WriteTotalTimeoutConstant; 

	if ( !SetCommTimeouts( pPort->Porthandle, &aCommTimeOut ) )
		return mapError( GetLastError() );

	return osl_Port_E_None;
}
/**	The SetPortTimeout function set the time-out parameters for all read 
	and write operations on a specified communications device. 
*/

oslPortError SAL_CALL osl_getPortTimeout( oslPort hPort, oslPortTimeouts *pPortTimeouts )
{
	COMMTIMEOUTS aCommTimeOut;

	oslPortImpl *pPort = (oslPortImpl *) hPort;

	if( pPortTimeouts->Size != sizeof( oslPortTimeouts ))
		return osl_Port_E_DifferentStructSize;

	if ( !GetCommTimeouts( pPort->Porthandle, &aCommTimeOut ) )
		return mapError( GetLastError() );

	pPortTimeouts->ReadIntervalTimeout			= aCommTimeOut.ReadIntervalTimeout;
	pPortTimeouts->ReadTotalTimeoutMultiplier	= aCommTimeOut.ReadTotalTimeoutMultiplier;	
    pPortTimeouts->ReadTotalTimeoutConstant     = aCommTimeOut.ReadTotalTimeoutConstant;
	pPortTimeouts->WriteTotalTimeoutMultiplier  = aCommTimeOut.WriteTotalTimeoutMultiplier;
    pPortTimeouts->WriteTotalTimeoutConstant    = aCommTimeOut.WriteTotalTimeoutConstant;	

	return osl_Port_E_None;
}

/** The GetPortTimeout function returned the current time-out parameters for all read 
	and write operations on a specified communications device. 
*/


sal_uInt32 SAL_CALL osl_sendToPort( oslPort hPort, void* pBuffer, sal_uInt32 nNumberOfBytesToWrite )
{
	DWORD dwBytesWritten = 0;
	BOOL test;
	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	WriteFile( pPort->Porthandle, pBuffer, nNumberOfBytesToWrite, &dwBytesWritten, NULL);

	test = FlushFileBuffers(pPort->Porthandle);
		
	return dwBytesWritten;
}

/** The function sendToPort tries to send the contents of pBuffer synchronous to the port.
	The function will block until all bytes are transmitted, or if a timeout is set
	(with setPortTimeout) until the timeout has reached. If no timeout is set, the function
	may be block forever.

	@return The number of bytes written.
*/

void msgtest( DWORD err )
{
	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		err,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL );// Process any inserts in lpMsgBuf.

	MessageBox( NULL, (LPCTSTR)lpMsgBuf, "Error", MB_OK | MB_ICONINFORMATION );
	LocalFree( lpMsgBuf );
}




sal_uInt32 SAL_CALL osl_receiveFromPort( oslPort hPort, void* pBuffer, sal_uInt32 nNumberOfBytesToRead )
{
	DWORD dwBytesRead = 0;

	oslPortImpl *pPort = (oslPortImpl *) hPort;
	
	ReadFile( pPort->Porthandle,  pBuffer, nNumberOfBytesToRead, &dwBytesRead, NULL);


			
	return dwBytesRead;
}

/** The function receiveFromPort tries to fill the contents of pBuffer synchronous from the port.
	The function will block until nNumberOfBytesToRead bytes received, or if a timeout is set
	(with setPortTimeout) until the timeout has reached. If no timeout is set, the function
	may be block forever.

	@return The number of bytes received.
*/

/*************************************************************************
*
*    $Log: ports.c,v $
*    Revision 1.1.1.1  2000/09/18 15:17:23  hr
*    initial import
*
*    Revision 1.9  2000/09/18 14:29:02  willem.vandorp
*    OpenOffice header added.
*
*    Revision 1.8  2000/05/29 16:34:34  hro
*    SRC591: Explicite SAL_CALL calling convention
*
*    Revision 1.7  2000/04/07 13:30:28  obr
*    Unicode changes
*
*    Revision 1.6  2000/03/31 15:48:46  rs
*    UNICODE-Changes
*
*    Revision 1.5  1999/10/27 15:13:35  mfe
*    Change of Copyright, removed compiler warnings, code clean up, ...
*
*    Revision 1.4  1999/01/20 19:09:50  jsc
*    #61011# Typumstellung
*
*    Revision 1.3  1998/12/14 14:10:08  jru
*    #60220# aenderung an SerialPortDeviceBlock
*
*    Revision 1.1  1998/12/08 15:55:47  jru
*    #60220# initial revision
*
*************************************************************************/


