/*************************************************************************
 *
 *  $RCSfile: sdaemon.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "system.h"

#ifndef _OSL_THREAD_H_
#   include <osl/thread.h>
#endif
#ifndef _OSL_DAEMON_H_
#   include <osl/daemon.h>
#endif

#include "../all/daemprot.h"
#include "../all/daemman.h"

#include "daemregw.h"

#define SHUTDOWN_COMMAND "-shutdown"

DWORD W95_RegisterServiceProcess(DWORD dwProcessId, DWORD dwServiceType);

int PASCAL WinMain( HINSTANCE hInst, HINSTANCE hPrevInst,
					LPSTR lpszCmdLine, int nCmdShow )
{
	if ((lpszCmdLine != NULL) && (strncmp(lpszCmdLine, SHUTDOWN_COMMAND, strlen(SHUTDOWN_COMMAND)) == 0))
	{
		if (osl_shutdownDaemonManager())
			fprintf(stderr, "shutdown request send to daemon manager\n");
		else
			fprintf(stderr, "daemon manager is not running\n");
	}
	else
	{
		rtl_uString* ustrPipeName=0;
		oslDaemonConnection rootConn = NULL;
		oslDaemonConnection childConn = NULL;
		DaemonRegistryApi* pRegApi = osl_getDaemonWindowsRegistry();

		oslDaemonManagerStartupError startupError = osl_initDaemonManager(pRegApi);

		switch (startupError)
		{
			case osl_oslDaemonManagerStartupError_NoError:
				break;
			case osl_oslDaemonManagerStartupError_AllreadyStarted:
				fprintf(stderr, "daemon manager allready running\n");
				break;
			case osl_oslDaemonManagerStartupError_CouldNotReadProfile:
				fprintf(stderr, "could not read profile\n");
				break;
			default:
				fprintf(stderr, "unknown error occured\n");
				break;
		}

		if (startupError != osl_oslDaemonManagerStartupError_NoError)
		{
			return(-1);
		}

		rtl_uString_newFromAscii(&ustrPipeName,OSL_DAEMON_MANAGER_PIPE);

		rootConn = osl_openDaemonMgrConn(ustrPipeName, osl_DaemonConnection_CREATE);

		rtl_uString_release(ustrPipeName);

		if (rootConn == NULL)
		{
			osl_deinitDaemonManager();
			fprintf(stderr, "unknown error occured\n");
			return(-1);
		}
		else
		{
			W95_RegisterServiceProcess(0, 1);
			osl_startupAutomaticDaemons();

			/* wait for connections */
			do
			{
				childConn = osl_acceptOnDaemonMgrConn(rootConn);

				if (childConn != NULL)
					osl_handleConnection(childConn);
			} 
			while (childConn != NULL);
			W95_RegisterServiceProcess(0, 0);
		}

		osl_deinitDaemonManager();
	}

	return 0;
}

/*************************************************************************
*
*    $Log: sdaemon.c,v $
*    Revision 1.1.1.1  2000/09/18 15:17:23  hr
*    initial import
*
*    Revision 1.9  2000/09/18 14:29:02  willem.vandorp
*    OpenOffice header added.
*
*    Revision 1.8  2000/05/15 08:35:14  mfe
*    Unicode adaptions
*
*    Revision 1.7  1999/10/27 15:13:36  mfe
*    Change of Copyright, removed compiler warnings, code clean up, ...
*
*    Revision 1.6  1999/04/28 14:30:46  ts
*    #65507# sdaemon.exe unter WINDOWS als GUI Applikation, damit er beim Systemabschuschluss runtergefahren werden
*
*    Revision 1.5  1999/03/19 10:32:24  ts
*    #63572# Win95: logon user geht jetz immer, sdaemon hidden starten
*
*    Revision 1.4  1999/03/15 10:02:09  ts
*    #63377# W95: daemons mit RegisterServiceProcess() registrieren
*
*    Revision 1.3  1999/01/20 19:09:50  jsc
*    #61011# Typumstellung
*
*    Revision 1.2  1998/12/04 15:11:51  ts
*    #52712# pipes
*
*    Revision 1.1  1998/12/04 08:22:24  ts
*    #52712# Profile eingebunden
*
*************************************************************************/
