/*************************************************************************
 *
 *  $RCSfile: Security.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:24:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
package stardiv.security;

import java.io.File;
import java.io.IOException;
import java.io.FileDescriptor;
import java.net.URL;
import java.net.InetAddress;
import java.net.UnknownHostException;

import java.text.MessageFormat;

import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

public class Security extends java.lang.SecurityManager {
	static final private boolean DEBUG = false;
	static final private boolean TRACE = false;

	final static private int PRIVELEGED_PORT = 1024;

	static private final Object checkPermission_Trusts[] = {
		"stardiv.security.PermissionCollection", new Integer(7), new Integer(8), new Integer(5),
		"stardiv.controller.PropertyEditor", new Integer(4), new Integer(7),
		"sun.audio.AudioPlayer", new Integer(9), // <init>
		"com.sun.star.lib.awt.peer.EmbeddedFrame", new Integer(10), /* <init */ new Integer(7), /* getEventQueue */
		"stardiv.applet.AppletExecutionContext", new Integer(6), new Integer(10), // init
		"sun.net.www.http.HttpClient", new Integer(4), new Integer(8), new Integer(7), new Integer(6),// <init>
	};

	static private final Object checkAccess_thread_Trusts[] = {
		"sun.awt.ScreenUpdater", new Integer(4), new Integer(6),
		"sun.audio.AudioPlayer", new Integer(4), new Integer(6),
		"sun.awt.image.ImageFetcher", new Integer(6), new Integer(4), new Integer(8), // fetchloop
		"sun.net.www.http.HttpClient", new Integer(5), new Integer(6), new Integer(7), // put
		"java.awt.EventQueue", new Integer(7), // <init>
		"com.sun.star.lib.util.ExecutionContext", new Integer(4), new Integer(6), // init
	};

	static private final Object checkAccess_threadGroup_Trusts[] = {
		"sun.awt.ScreenUpdater", new Integer(4), new Integer(5),
		"sun.audio.AudioPlayer", new Integer(4), new Integer(5), new Integer(6), // <init>
		"sun.awt.image.ImageFetcher", new Integer(4), new Integer(5),
		"java.awt.EventQueue", new Integer(6), // <init>
		"sun.net.www.http.HttpClient", new Integer(6), // put
		"com.sun.star.lib.util.ExecutionContext", new Integer(4), new Integer(5), // init
	};

	private boolean inInheritedTrustedClass(Object dcpAr[]) {
		int i = 0;
		boolean granted = false;
		String trustedClassName = null;
		int methDepth = 0;

		while(i < dcpAr.length && !granted) {
			trustedClassName = (String)dcpAr[i++];
			
			methDepth = classDepth(trustedClassName);
			while(i < dcpAr.length && !granted && dcpAr[i] instanceof Integer) {
				granted = granted || ((Integer)dcpAr[i]).intValue() == methDepth;

				i++;
			}
//  			System.err.println("XXX:" + trustedClassName + " " + methDepth + " " + granted);
		}

		if(TRACE && !granted) {
			System.err.println("#### Security.inInheritedTrustedMethod - ScreenUpdater:" + classDepth("sun.awt.ScreenUpdater") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - AudioPlayer:" + classDepth("sun.audio.AudioPlayer") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - ImageFetcher:" + classDepth("sun.awt.image.ImageFetcher") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - HttpClient:" + classDepth("sun.net.www.http.HttpClient") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - PermissionCollection:" + classDepth("stardiv.security.PermissionCollection") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - EventQueue:" + classDepth("java.awt.EventQueue") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - PropertyEditor:" + classDepth("stardiv.controller.PropertyEditor") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - ExecutionContext:" + classDepth("com.sun.star.lib.util.ExecutionContext") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - AppletExecutionContext:" + classDepth("stardiv.applet.AppletExecutionContext") + " " + granted);
			System.err.println("#### Security.inInheritedTrustedMethod - EmbeddedFrame:" + classDepth("com.sun.star.lib.awt.peer.EmbeddedFrame") + " " + granted);
			Thread.dumpStack();
		}

		return granted;
	}

	private void throwException(String descriptor, Object arg0, Object arg1, Object arg2) {
		Object args[] = null;
		String txt = null;
		
		try {
			txt = resourceBundle.getString("appletsecurityexception." + descriptor.toLowerCase());
			if(arg2 != null) {
				args = new Object[3];
				args[0] = arg0;
				args[1] = arg1;
				args[2] = arg2;
			}
			else if(arg1 != null) {
				args = new Object[2];
				args[0] = arg0;
				args[1] = arg1;
			}
			else if(arg0 != null) {
				args = new Object[1];
				args[0] = arg0;
			}
		}
		catch(MissingResourceException emr) {
			txt = descriptor;
		}

		if(DEBUG) {
			System.err.println("#### Throwing security exception!!!!");
			Thread.dumpStack();
		}
//    		throw new SecurityException((args != null) ? MessageFormat.format(txt, args) : txt);
	}

	private ProtectionDomain getProtectionDomain(Class loadedClass) {
		ProtectionDomain protectionDomain = null;

		Object xx[] = loadedClass.getSigners();
		protectionDomain = (ProtectionDomain)xx[0];
		if(protectionDomain == null)
			throw new NullPointerException("Loaded class without protectionDomain!!!");

		return protectionDomain;
	}

    private void checkPermission(ProtectionDomain protectionDomain, Permission permission) {
  		if(!inInheritedTrustedClass(checkPermission_Trusts)) {
			if(!protectionDomain.implies(permission)) {
				throwException("no " + permission + " permission", null, null, null);
			}
  		}
    }

    private void checkPermission(Permission permission) {
		Class loadedClass = currentLoadedClass();
		if(loadedClass != null) {
			ProtectionDomain protectionDomain = getProtectionDomain(loadedClass);
			checkPermission(protectionDomain, permission);
		}
    }

	
	private void checkTrusted(String descriptor, Object arg0, Object arg1, Object arg2) {
  		if(!inInheritedTrustedClass(checkPermission_Trusts)) {
			Class loadedClass = currentLoadedClass();
			if(loadedClass != null) {
				ProtectionDomain protectionDomain = getProtectionDomain(loadedClass);
			
				if(!protectionDomain.isTrusted()) {
					throwException(descriptor, arg0, arg1, arg2);
				}
			}
  		}
	}

	private void checkTrusted(String descriptor, Object arg0, Object arg1) {
		checkTrusted(descriptor, arg0, arg1, null);
	}

	private void checkTrusted(String descriptor, Object arg0) {
		checkTrusted(descriptor, arg0, null);
	}

	private void checkTrusted(String descriptor) {
		checkTrusted(descriptor, null);
	}

	private ResourceBundle resourceBundle;

    public Security() {
		resourceBundle = ResourceBundle.getBundle("sun.applet.resources.MsgAppletViewer");
    }

	public void reset() {
	}


    public void checkAccept(String host, int port) {
		if(inClassLoader() && port > 0 && port < PRIVELEGED_PORT)
//  			throw new SecurityException("checkaccept");
			throwException("checkaccept", null, null, null);

		checkConnect(host, port);
    }

    public void checkAccess(Thread thread) {
		if(!(Thread.currentThread().equals(thread) || Thread.currentThread().getThreadGroup().parentOf(thread.getThreadGroup())))
			if(!inInheritedTrustedClass(checkAccess_thread_Trusts))
				checkPermission(new RuntimePermission("modifyThread", thread));
    }

    public void checkAccess(ThreadGroup threadGroup) {
		if(!threadGroup.parentOf(Thread.currentThread().getThreadGroup())) // every thread can access its threadgroup
			if(!inInheritedTrustedClass(checkAccess_threadGroup_Trusts))
				checkPermission(new RuntimePermission("modifyThreadGroup", threadGroup));
    }

    public void checkAwtEventQueueAccess() {
		checkTrusted("checkAwtEventQueueAccess");
    }

    public void checkConnect(String s, int i) {
//  		if(inClass("sun.net.www.http.HttpClient")) return;

        if(i == -1)
            checkPermission(new SocketPermission(s, "resolve", i));
        else
            checkPermission(new SocketPermission(s, "connect", i));
    }

    public void checkConnect(String s, int i, Object protectionDomain) {
		Permission permission = null;
		if(i == -1)
			checkPermission(new SocketPermission(s, "resolve", i));
		else
			checkPermission(new SocketPermission(s, "connect", i));

		checkPermission((ProtectionDomain)protectionDomain, permission);
    }

    public void checkCreateClassLoader() {
		checkTrusted("checkCreateClassLoader");
    }

    public void checkDelete(String file) {
		checkTrusted("checkDelete", file);
    }

    public void checkExec(String file) {
		checkTrusted("checkExec", file);
    }

    public void checkExit(int i) {
		checkTrusted("checkExit");
    }

    public void checkLink(String file) {
//  		if(inClass("sun.audio.AudioDevice")) return;
		checkTrusted("checkLink", file);
    }

    public void checkListen(int port) {
		if(inClassLoader() && port > 0 && port < PRIVELEGED_PORT)
//  			throw new SecurityException("checklisten");
			throwException("checklisten", null, null, null);
    }

    public void checkMemberAccess(Class clazz, int which) {
		if(which != java.lang.reflect.Member.PUBLIC) {
			ClassLoader classLoader = currentClassLoader();
		
			if(classLoader != null && clazz.getClassLoader() != classLoader)
				checkTrusted("checkMemberAccess", clazz, new Integer(which));
		}
    }

    public void checkMulticast(InetAddress inetaddress) {
		checkTrusted("checkMulticast", inetaddress);
    }

    public void checkMulticast(InetAddress inetaddress, byte byte0) {
		checkTrusted("checkMulticast", inetaddress);
    }

    public void checkPackageAccess(String pkg) {
		if(inClassLoader()) {
			int i = pkg.indexOf(".");

			while(i > 0) {
				String subpkg = pkg.substring(0, i);
				if(Boolean.getBoolean("package.restrict.access." + subpkg))
					checkTrusted("checkPackageAccess", pkg);

				i = pkg.indexOf(".", i + 1);
			}
		}
    }

    public void checkPackageDefinition(String pkg) {
		if(inClassLoader()) {
			int i = pkg.indexOf(".");

			while(i > 0) {
				String subpkg = pkg.substring(0, i);
				if(Boolean.getBoolean("package.restrict.definition." + subpkg))
					checkTrusted("checkPackageDefinition", pkg);

				i = pkg.indexOf(".", i + 1);
			}
		}
    }

    public void checkPrintJobAccess() {
		checkTrusted("checkPrintJobAccess");
    }

    public void checkPropertiesAccess() {
		checkTrusted("checkPropertiesAccess");
    }

    public void checkPropertyAccess(String key) {
		if(classLoaderDepth() == 2) {
			boolean allow = Boolean.getBoolean(key + ".applet");
			if(!allow) {
				//  		if(inClass("java.io.BufferedWriter")) return;
				//  		if(inClass("java.net.PlainSocketImpl")) return;
				checkTrusted("checkPropertyAccess", key);
			}
		}
    }

    public void checkRead(String file) {
		checkPermission(new FilePermission(file, "read"));
    }

    public void checkRead(String file, Object protectionDomain) {
		checkPermission((ProtectionDomain)protectionDomain, new FilePermission(file, "read"));
    }

    public void checkSecurityAccess(String s) {
		checkTrusted("checkSecurityAccess", s);
    }

    public void checkSetFactory() {
		if(inClassLoader() && !inClass("sun.net.www.MimeTable"))
			checkTrusted("checkSetFactory");
    }

    public void checkSystemClipboardAccess() {
		checkTrusted("checkSystemClipboardAccess");
    }

    public boolean checkTopLevelWindow(java.lang.Object obj) {
        try {
			checkTrusted("checkTopLevelWindow");
            return true;
        }
        catch(SecurityException ex) {
            return false;
        }
    }

    /**
     * Applets are not allowed to open file descriptors unless
     * it is done through a socket, in which case other access
     * restrictions still apply.
     */
    public synchronized void checkRead(FileDescriptor fd) {
		if ((inClassLoader() && !inClass("java.net.SocketInputStream"))
			|| (!fd.valid())) {
//              throw new SecurityException("Applet: checkread.fd");
			throwException("Applet: checkread.fd", null, null, null);
		}
    }

    /**
     * Applets are not allowed to open file descriptors unless
     * it is done through a socket, in which case other access
     * restrictions still apply.
     */
    public synchronized void checkWrite(FileDescriptor fd) {
		if ((inClassLoader() && !inClass("java.net.SocketOutputStream")) 
			|| (!fd.valid()) ) {
//              throw new SecurityException("Applet: checkwrite.fd");
			throwException("Applet: checkwrite.fd", null, null, null);
		}
    }

    public void checkWrite(String file) {
		String path = null;

		try {
			path = (new File(file)).getCanonicalPath();
		}
		catch(IOException eio) {
			throwException("checkWrite.exeception", eio.getMessage(), file, null);
		}

		checkTrusted("checkWrite", file, path);
    }

	public Object getSecurityContext() {
		ProtectionDomain protectionDomain = null;

		Class loadedClass = currentLoadedClass();
		if(loadedClass != null) {
			protectionDomain = getProtectionDomain(loadedClass);
		}

		return protectionDomain;
	}

	public synchronized boolean getInCheck() {
		return super.getInCheck();
	}
}
