/*************************************************************************
 *
 *  $RCSfile: SocketPermission.java,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:24:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

package com.sun.star.lib.sandbox;

import java.net.InetAddress;
import java.net.UnknownHostException;

public final class SocketPermission implements Permission {
	private static final boolean DEBUG = false;

    private String host, action;
    private int modus;

    public final static int NETWORK_HOST = 1;
    public final static int NETWORK_UNRESTRICTED = 2;
    public final static int NETWORK_APPLET = 3;

    public SocketPermission(int modus, String host) {
		this.modus = modus;
		this.host = host;
    }

    public SocketPermission(String host, String action, int port) {
		this.host   = host;
		this.action = action;
    }

	private boolean checkToHost(String toHost) {
		boolean granted = false;

		granted = host.equals(toHost);

		if (!granted) {
			try {
				InetAddress toHostAddr, fromHostAddr;
				// the only time we allow non-matching strings
				// is when IPs and the IPs match.
				toHostAddr = InetAddress.getByName(toHost);
				InetAddress hostAddr = InetAddress.getByName(host);

				if (hostAddr.equals(toHostAddr))
					granted = true;
				else
					granted = Boolean.getBoolean("trustProxy");
			} 
			catch (UnknownHostException e) {
				System.err.println(" Unknown:" + e);
			}  
		} 

		return granted;
	}

    public boolean implies(Permission permission) {
		boolean granted = false;

		if(permission instanceof SocketPermission && !granted) {
			SocketPermission socketPermission = (SocketPermission)permission;
			String toHost = socketPermission.host;

			switch(modus) {
			case NETWORK_HOST: 
				granted = checkToHost(toHost);

				break;

			case NETWORK_UNRESTRICTED:
				granted = true;
				break;
			
			case NETWORK_APPLET:
				String securityMode = System.getProperty("appletviewer.security.mode");
				if(securityMode != null) {
					if(securityMode.equals("unrestricted"))
						granted = true;
					else if(securityMode.equals("host"))
						granted = checkToHost(toHost);
				}

				break;
			}
			if(DEBUG)
				if(!granted)
					System.err.println("#### SocketPermission.implies:" + this + " " + permission);
		}

		return granted;
    }

	public String toString() {
		return "SocketPermission:" + host + " " + action + " " + modus;
	}
}

