/*************************************************************************
 *
 *  $RCSfile: scmatrix.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:16:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef SC_MATRIX_HXX
#define SC_MATRIX_HXX


#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif
#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#define SC_MAXMATSIZE 30

#define SC_MATVAL_STRING	0x01
#define SC_MATVAL_EMPTY		0x03	// empty ist Leer-String plus Flag

union MatValue
{
	double fVal;
	String* pS;
};

class ScMatrix
{
	USHORT nAnzCol;
	USHORT nAnzRow;
	MatValue* pMat;
	BYTE* bIsString;

	void ResetIsString();
	void CreateMatrix(USHORT nC, USHORT nR);

public:
	ScMatrix(USHORT nC, USHORT nR) 					 { CreateMatrix(nC, nR); }
	~ScMatrix();
	ScMatrix* Clone() const;

	ScMatrix(SvStream& rStream);
	void Store(SvStream& rStream) const;

	void GetDimensions(USHORT& rC, USHORT& rR) const { rC = nAnzCol; rR = nAnzRow; };
	void PutDouble(double fVal, USHORT nC, USHORT nR);
	void PutDouble(double fVal, USHORT nIndex) 	 	 { pMat[nIndex].fVal = fVal; }
	void PutString(const String& rStr, USHORT nC, USHORT nR);
	void PutString(const String& rStr, USHORT nIndex);
	void PutEmpty(USHORT nC, USHORT nR);
	void PutEmpty(USHORT nIndex);
	void FillDouble( double fVal, USHORT nC1, USHORT nR1, USHORT nC2, USHORT nR2 );
	void FillDoubleLowerLeft( double fVal, USHORT nC2 );	// unteres linkes Dreieck
	double GetDouble(USHORT nC, USHORT nR) const;
	double GetDouble(USHORT nIndex) const			 { return pMat[nIndex].fVal; }
	const String& GetString(USHORT nC, USHORT nR) const;
	const String& GetString(USHORT nIndex) const	 { return *(pMat[nIndex].pS); }
	const MatValue* Get(USHORT nC, USHORT nR, BOOL& bString) const;
	BOOL IsString(USHORT nC, USHORT nR) const
		{ return bIsString && bIsString[nC*nAnzRow+nR]; }
	BOOL IsEmpty(USHORT nC, USHORT nR) const
		{ return bIsString && bIsString[nC*nAnzRow+nR] == SC_MATVAL_EMPTY; }
	BOOL IsEmpty(USHORT nIndex) const
		{ return bIsString && bIsString[nIndex] == SC_MATVAL_EMPTY; }
	BOOL IsValue(USHORT nIndex) const
		{ return !bIsString || !bIsString[nIndex]; }
	BOOL IsNumeric() const							 { return bIsString == NULL; }
	void MatTrans(ScMatrix& mRes) const;
	void MatCopy (ScMatrix& mRes) const;
	void MatCopyUpperLeft(ScMatrix& mRes) const;

	// Werte von ScInterpreter::CompareMat (-1,0,1) in logische Werte wandeln
	void CompareEqual();
	void CompareNotEqual();
	void CompareLess();
	void CompareGreater();
	void CompareLessEqual();
	void CompareGreaterEqual();

	BOOL And();		// logisches UND aller Matrixwerte
	BOOL Or();		// logisches ODER aller Matrixwerte

// alle anderen Matrixfunktionen  MatMult, MInv,...finden in ScInterpreter
// statt, damit sie numerisch (matherr ...) abgesichert sind !!
};


#endif


