/*************************************************************************
 *
 *  $RCSfile: spstring.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//------------------------------------------------------------------------

#include <tools/debug.hxx>
#include <tools/intn.hxx>
#include <tools/string.hxx>
#include <unotools/charclass.hxx>

#include "global.hxx"
#include "compiler.hxx"

#include "spstring.hxx"


// 'Auszug' aus ansitab.hxxx, da nicht includierbar
#define C_WORD			0x0080


String ExcelNameToScName( const String& rStr )
{
	String						aRet;
	if( !rStr.Len() )
		return aRet;

	static const sal_Unicode	cUnderSc = '_';
	static const sal_Unicode	cBlank = ' ';

	const sal_Unicode*			pAkt = rStr.GetBuffer();

	BOOL						bSpace = FALSE;
	sal_Unicode					cAkt = *pAkt;

	while( cAkt )
	{
		if( !( ScGlobal::pCharClass->isLetterNumeric( rStr, pAkt - rStr.GetBuffer() ) || cAkt == cUnderSc || cAkt == cBlank ) )
		{
			aRet += cUnderSc;
			bSpace = FALSE;
		}
		else if( cAkt == cBlank )
		{
			if( bSpace )
				aRet += cUnderSc;
			else
				bSpace = TRUE;
		}
		else
		{
			aRet += cAkt;
			bSpace = FALSE;
		}

		pAkt++;
		cAkt = *pAkt;
	}

	return aRet;
}


/*void ExcelNameToScName( ByteString& r )
{
	ByteString					aRet;
	if( !rStr.Len() )
		return aRet;

	static const sal_Char		cUnderSc = '_';
	static const sal_Char		cBlank = ' ';

	const sal_Char*				pAkt = rStr.GetBuffer();

	BOOL						bSpace = FALSE;
	sal_Char					cAkt = *pAkt;

	while( cAkt )
	{
		if( !( ScGlobal::pCharClass->isLetterNumeric( rStr, pAkt - rStr.GetBuffer() ) || cAkt == cUnderSc || cAkt == cBlank ) )
		{
			aRet += cUnderSc;
			bSpace = FALSE;
		}
		else if( cAkt == cBlank )
		{
			if( bSpace )
				aRet += cUnderSc;
			else
				bSpace = TRUE;
		}
		else
		{
			aRet += cAkt;
			bSpace = FALSE;
		}

		pAkt++;
		cAkt = *pAkt;
	}

	r = aRet;
}
*/

String ReadString( SvStream& r, const UINT16 n, CharSet eSrc )
{
	return String( ReadString( r, n ), eSrc );
}


String ReadCString( SvStream& r, CharSet eSrc )
{
	return String( ReadCString( r ), eSrc );
}


String ReadCString( SvStream& r, INT32& rBytesLeft, CharSet eSrc )
{
	return String( ReadCString( r, rBytesLeft ), eSrc );
}


ByteString ReadString( SvStream& r, const UINT16 n )
{
	ByteString	aRet;

	if( n )
	{
		sal_Char*	p = new sal_Char[ n + 1 ];
		r.Read( p, n );
		p[ n ] = 0x00;
		aRet = p;
		delete[] p;
	}

	return aRet;
}


ByteString ReadCString( SvStream& r )
{
	ByteString	aRet;
	sal_Char	c;

	r >> c;
	while( c )
	{
		aRet += c;
		r >> c;
	}

	return aRet;
}


ByteString ReadCString( SvStream& r, INT32& rBytesLeft )
{
	ByteString	aRet;
	sal_Char	c;

	r >> c;
	rBytesLeft--;
	while( c )
	{
		aRet += c;
		r >> c;
		rBytesLeft--;
	}

	return aRet;
}


void AppendString( SvStream& r, String& rStr, const UINT16 n, CharSet eSrc )
{
	ByteString	aByteString;
	AppendString( r, aByteString, n );
	rStr += String( aByteString, eSrc );
}


void AppendCString( SvStream& r, String& rStr, CharSet eSrc )
{
	ByteString	aByteString;
	AppendCString( r, aByteString );
	rStr += String( aByteString, eSrc );
}


void AppendCString( SvStream& r, String& rStr, INT32& rBytesLeft, CharSet eSrc )
{
	ByteString	aByteString;
	AppendCString( r, aByteString, rBytesLeft );
	rStr += String( aByteString, eSrc );
}


void AppendString( SvStream& r, ByteString& rStr, const UINT16 n )
{
	if( n )
	{
		sal_Char*	p = new sal_Char[ n + 1 ];
		r.Read( p, n );
		p[ n ] = 0x00;
		rStr += p;
		delete[] p;
	}
}


void AppendCString( SvStream& r, ByteString& rAddAt )
{
	sal_Char	c;

	r >> c;
	while( c )
	{
		rAddAt += c;
		r >> c;
	}
}


void AppendCString( SvStream& r, ByteString& rAddAt, INT32& rBytesLeft )
{
	sal_Char	c;

	r >> c;
	rBytesLeft--;
	while( c )
	{
		rAddAt += c;
		r >> c;
		rBytesLeft--;
	}
}


