/*************************************************************************
 *
 *  $RCSfile: vfbuff.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//------------------------------------------------------------------------

#include <svtools/zforlist.hxx>
#include <tools/debug.hxx>

#include "vfbuff.hxx"
#include "flttools.hxx"



const sal_Char* ValueFormBuffer::pBuiltinFormats[] = {
	"General",								// 0
	"0",									// 1
	"0.00",									// 2
	"#,##0",								// 3
	"#,##0.00",								// 4
	"",										// 5	-> Defined 0
	"",										// 6	-> Defined 1
	"",										// 7	-> Defined 2
	"",										// 8	-> Defined 3
	"0%",									// 9
	"0.00%",								// 10
	"0.00E+00",								// 11
	"# ?/?",								// 12
	"# ?""?/??",							// 13
	"DD.MM.YY",								// 14
	"DD. MMM YY",							// 15
	"DD. MMM",								// 16
	"MMM YY",								// 17
	"h:mm AM/PM",							// 18
	"h:mm:ss AM/PM",						// 19
	"hh:mm",								// 20
	"hh:mm:ss",								// 21
	"DD.MM.YYYY hh:mm",						// 22
	"",									// 23	?
	"",									// 24	?
	"",									// 25	?
	"",									// 26	?
	"",									// 27	?
	"",									// 28	?
	"",									// 29	?
	"",									// 30	?
	"",									// 31	?
	"",									// 32	?
	"",									// 33	?
	"",									// 34	?
	"",									// 35	?
	"",									// 36	?
	"#,##0 _$;-#,##0 _$",					// 37
	"#,##0 _$;[Red]-#,##0 _$",				// 38
	"#,##0.00 _$;-#,##0.00 _$",				// 39
	"#,##0.00 _$;[Red]-#,##0.00 _$",		// 40
	"",										// 41	-> Defined 5
	"",										// 42	-> Defined 4
	"",										// 43	-> Defined 7
	"",										// 44	-> Defined 6
	"mm:ss",								// 45
	"[h]:mm:ss",							// 46
	"mm:ss,0",								// 47
	"##0.0E+0",								// 48
	"@"										// 49
};

const UINT16	ValueFormBuffer::nAnzBuiltin = 50;
const UINT16	ValueFormBuffer::nNewFormats = 164;



ValueFormBuffer::ValueFormBuffer( RootData* pRD, UINT16 nSize ) : ExcRoot( pRD )
{
	nMax = nSize;

	pHandles = new UINT32[ nSize ];

	// ACHTUNG: nCount wird in Init() sinnvoll gesetzt!

	_NewValueFormat = &ValueFormBuffer::__NewValueFormat;
	_GetValueFormat = &ValueFormBuffer::__GetValueFormat;
}


ValueFormBuffer::~ValueFormBuffer()
{
	delete[] pHandles;
}


void ValueFormBuffer::__NewValueFormat( String& rFormString )
{
	Init();
	( this->*_NewValueFormat )( rFormString );
}


void ValueFormBuffer::_NewValueFormatX( String& rFormString )
{
	DBG_ASSERT( nCount < nMax, "+ValueFormBuffer::NewValueFormat(): Value-Form-Puffer voll!" );

	if( nCount >= nMax ) return;

	xub_StrLen	nDummy;
	INT16		nTyp = NUMBERFORMAT_DEFINED;

	pExcRoot->pFormTable->PutandConvertEntry( rFormString, nDummy, nTyp,
		pHandles[ nCount ], LANGUAGE_ENGLISH_US, pExcRoot->eDefLanguage );

	nCount++;
}


void ValueFormBuffer::_NewValueFormat5( String &rFormString )
{
	DBG_ASSERT( nCount < nMax, "+ValueFormBuffer::NewValueFormat(): Value-Form-Puffer voll!" );

	if( nCount >= nMax ) return;

	xub_StrLen	nDummy;
	INT16		nTyp = NUMBERFORMAT_DEFINED;

	pExcRoot->pFormTable->PutandConvertEntry( rFormString, nDummy, nTyp,
		pHandles[ nCount ], LANGUAGE_ENGLISH_US, pExcRoot->eDefLanguage );

	// jetzt wird's merkwuerdig!
	switch( nCount )
	{
		case 5:		nCount++;				break;	// 0 |
		case 6:		nCount++;				break;	// 1 |--|
		case 7:		nCount++;				break;	// 2    |--|
		case 8:		nCount = 42;			break;	// 3       |--|
		case 41:	nCount = 44;			break;	// 5          |  |--|
		case 42:	nCount--;				break;	// 4          |--|  |
		case 43:	nCount = nNewFormats;	break;	// 7                |  |--|
		case 44:	nCount--;				break;	// 6                |--|  |
		default:	nCount++;						//                        |-...
	}
}


UINT32 ValueFormBuffer::__GetValueFormat( UINT16 nExcIndex )
{
	Init();
	return ( this->*_GetValueFormat )( nExcIndex );
}


UINT32 ValueFormBuffer::_GetValueFormatX5( UINT16 nExcIndex )
{
#ifdef DBG_UTIL
	if( nExcIndex >= nCount )
	{
		DBG_WARNING(  "+ValueFormBuffer::GetValueFormat(): Da kann ich auch nichts fuer!" );
	}
#endif
	if( nExcIndex >= nCount )
		return nDefaultHandle;

	return pHandles[ nExcIndex ];
}


void ValueFormBuffer::Reset( void )
{
	nCount = 0;
	_NewValueFormat = &ValueFormBuffer::__NewValueFormat;
	_GetValueFormat = &ValueFormBuffer::__GetValueFormat;
}


void ValueFormBuffer::Init( void )
{
	switch( pExcRoot->eGlobalDateiTyp )
	{
		case Biff5W:
		case Biff8W:
		{// Sonderwurst!
			UINT16		nC;
			xub_StrLen	nDummy;
			String		aString;

			INT16		nTyp = NUMBERFORMAT_DEFINED;
			pExcRoot->pFormTable->PutEntry( aString, nDummy, nTyp,
				nDefaultHandle,	pExcRoot->eDefLanguage );

			// Builtin belegen
			pHandles[ 0 ] = pExcRoot->pFormTable->
				GetStandardIndex( pExcRoot->eDefLanguage );

			for( nC = 1 ; nC < nAnzBuiltin ; nC++ )
			{
				pExcRoot->pFormTable->PutandConvertEntry( _STRING( pBuiltinFormats[ nC ] ), nDummy, nTyp,
					pHandles[ nC ], LANGUAGE_ENGLISH_US, pExcRoot->eDefLanguage );
		}

			// Rest defaulten
			for( nC = nAnzBuiltin ; nC < nMax ; nC++ )
				pHandles[ nC ] = nDefaultHandle;

			_NewValueFormat = &ValueFormBuffer::_NewValueFormat5;
			nCount = 5;			// !!!ACHTUNG!!! erster aus Format-Record!
		}
			break;
		default:
			_NewValueFormat = &ValueFormBuffer::_NewValueFormatX;
			nCount = 0;
	}

	_GetValueFormat = &ValueFormBuffer::_GetValueFormatX5;
}


