/*************************************************************************
 *
 *  $RCSfile: xfbuff.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

#include "scitems.hxx"
#include <svx/algitem.hxx>
#include <svx/boxitem.hxx>
#include <svx/brshitem.hxx>
#include <svx/colritem.hxx>
#include <svx/rotmodit.hxx>

#include "document.hxx"
#include "patattr.hxx"
#include "docpool.hxx"
#include "attrib.hxx"
#include "stlsheet.hxx"
#include "stlpool.hxx"

#include "fontbuff.hxx"
#include "vfbuff.hxx"
#include "xfbuff.hxx"



const UINT16 XF_Buffer::nIndentFact = 10 * 20;



XF_Data::XF_Data( void )
{
	pFill = NULL;
	pBord = NULL;
	bFormValid = bFontValid = bHidden = bProtValid = FALSE;
	bLocked = TRUE;		// SC- / Excel-Default
	nValForm = nParent = nFont = 0;
	eWrap = EWT_Parent;			// nichts gesetzt -> Parent
	eHoriz = EHA_Parent;
	eVert = EVA_Parent;
	eTextOrient = ETO_Parent;
	nRotation = 0;
	nIndent = 0;
}


XF_Data::~XF_Data()
{
	if( pFill )
		delete pFill;
	if( pBord )
		delete pBord;
}


void XF_Data::SetFill( CellFill* pNew )
{
	// PREC: Rufender newed pNew, deleted wird in XF_Data!
	//			!Es wird keine Kopie angelegt!
	if( pFill )
		delete pFill;
	pFill = pNew;
}


void XF_Data::SetFill( const CellFill& rNew )
{
	if( !pFill )
		pFill = new CellFill;

	pFill->nPattern = rNew.nPattern;
	pFill->nForeColor = rNew.nForeColor;
	pFill->nBackColor = rNew.nBackColor;
}


void XF_Data::SetFill( BYTE nPattern, UINT16 nForeColor, UINT16 nBackColor )
{
	if( !pFill )
		pFill = new CellFill;

	pFill->nPattern = nPattern;
	pFill->nForeColor = nForeColor;
	pFill->nBackColor = nBackColor;
}


void XF_Data::SetBorder( CellBorder* pNew )
{
	// PREC: Rufender newed pNew, deleted wird in XF_Data!
	//			!Es wird keine Kopie angelegt!
	if( pBord )
		delete pBord;
	pBord = pNew;
}


void XF_Data::SetBorder( const CellBorder& rNew )
{
	if( !pBord )
		pBord = new CellBorder;

	pBord->nTopLine = rNew.nTopLine;
	pBord->nLeftLine = rNew.nLeftLine;
	pBord->nBottomLine = rNew.nBottomLine;
	pBord->nRightLine = rNew.nRightLine;
	pBord->nTopColor = rNew.nTopColor;
	pBord->nLeftColor = rNew.nLeftColor;
	pBord->nBottomColor = rNew.nBottomColor;
	pBord->nRightColor = rNew.nRightColor;
}


void XF_Data::SetBorder( BYTE nTopLine, BYTE nLeftLine, BYTE nBottomLine,
	BYTE nRightLine, UINT16 nTopColor, UINT16 nLeftColor,
	UINT16 nBottomColor, UINT16 nRightColor )
{
	if( !pBord )
		pBord = new CellBorder;

	pBord->nTopLine = nTopLine;
	pBord->nLeftLine = nLeftLine;
	pBord->nBottomLine = nBottomLine;
	pBord->nRightLine = nRightLine;
	pBord->nTopColor = nTopColor;
	pBord->nLeftColor = nLeftColor;
	pBord->nBottomColor = nBottomColor;
	pBord->nRightColor = nRightColor;
}




XF_Buffer::XF_Buffer( RootData* pRD ) :
	aDefEntry(
		*( new ScPatternAttr( pRD->pDoc->GetPool() ) ),
		*( new XF_Data ) ),
	ExcRoot( pRD )
{
}


XF_Buffer::~XF_Buffer()
{
	register ENTRY *pDel = ( ENTRY * ) List::First();

	while( pDel )
	{
		delete pDel;
		pDel = ( ENTRY * ) List::Next();
	}
}


ScPatternAttr& XF_Buffer::GetPattern( UINT16 nIndex )
{
	ScPatternAttr*			pRet;
	ENTRY*					pEntry = ( ENTRY* ) GetObject( nIndex );

	DBG_ASSERT( pEntry || pExcRoot->eDateiTyp == Biff2,
							"+XF_Buffer::GetPattern(): das ist zuviel des Guten!" );

	if( !pEntry )
		pRet = aDefEntry.pPattern;
	else
	{
		if( !pEntry->pPattern )
			pEntry->pPattern = CreatePattern( nIndex );
		pRet = pEntry->pPattern;
	}

	return *pRet;
}


void XF_Buffer::ExcToScBorderLine( BYTE nLine, UINT16 nColor, SvxBorderLine*& rpBorderLine, ColorBuffer& rColBuff )
{
	if( !nLine )
	{
		rpBorderLine = NULL;
		return;
	}
	else if( nLine > 7 )
		nLine = 1;		// Unbekanntes auf normale Linie abbilden

	if( nColor == 64 )
		// auto for line -> black
		nColor = 0;

	const SvxColorItem*		pColItem = rColBuff.GetColor( nColor, FALSE );
										// accepting no defaults

	if( !pColItem )
	{
		rpBorderLine = NULL;
		return;
	}

	const static UINT16 nPara[ 8 ][ 3 ] = {
		// OutWidth, InWidth, Distance
	{ 0, 0, 0 },		// (0)
	{ DEF_LINE_WIDTH_1, 0, 0 },		// 1, duenn
	{ DEF_LINE_WIDTH_2, 0, 0 },		// 2, mittel
	{ DEF_LINE_WIDTH_1, 0, 0 },		// 3, gestrichelt lang
	{ DEF_LINE_WIDTH_0, 0, 0 },		// 4, gestrichelt kurz
	{ DEF_LINE_WIDTH_3, 0, 0 },		// 5, dick
	{ DEF_LINE_WIDTH_1, DEF_LINE_WIDTH_1,
				DEF_LINE_WIDTH_1 },		// 6, doppelt
	{ DEF_LINE_WIDTH_0, 0, 0 } };	// 7, gepunktet	(Haarlinie...)

	rpBorderLine = new SvxBorderLine;	// ACHTUNG: delete im Caller!!!!!!

	rpBorderLine->SetColor( pColItem->GetValue() );

	rpBorderLine->SetOutWidth( nPara[ nLine ][ 0 ] );
	rpBorderLine->SetInWidth( nPara[ nLine ][ 1 ] );
	rpBorderLine->SetDistance( nPara[ nLine ][ 2 ] );
}


void XF_Buffer::Reset()
{
	register ENTRY *pDel = ( ENTRY * ) List::First();

	while( pDel )
	{
		delete pDel;
		pDel = ( ENTRY * ) List::Next();
	}

	Clear();
}




static inline UINT8 GetMixedCol( const UINT8 nB, const UINT8 nF, const UINT16 nFak )
{
	INT32		nT = nB - nF;
				nT *= ( INT32 ) nFak;
				nT /= 0xFFFF;
				nT += nF;
	return ( UINT8 ) nT;
}




ScPatternAttr* XF_Buffer::CreatePattern( UINT16 nIndex )
{
	SfxItemSet*			pItemSet;
	XF_Data*			pD;

	ENTRY*				pEntry = ( ENTRY* ) GetObject( nIndex );

	DBG_ASSERT( pEntry, "+XF_Buffer::CreatePattern(): XF nicht im Puffer!" );

	if( !pEntry )
		return aDefEntry.pPattern;

	DBG_ASSERT( !pEntry->pPattern, "*XF_Buffer::CreatePattern(): Schlingel! Pattern existiert schon!" );

	DBG_ASSERT( pEntry->pData, "-XF_Buffer::CreatePattern(): Wiiiiiiiiiiiiiie bitte?!" );

	// neues Pattern anlegen
	ScPatternAttr*		pPatt = new ScPatternAttr( pExcRoot->pDoc->GetPool() );

	pItemSet = &pPatt->GetItemSet();
	pEntry->pPattern = pPatt;		// ... und merken
	pD = pEntry->pData;

	const BOOL			bCellXF = pD->IsCellXF();

//	if( !pD->IsCellXF() )
//	{
//		DBG_WARNING( "+XF_Buffer::CreatePattern(): Und was soll ich mit 'nem Style?" );
//
//		return pPatt;
//	}

	ColorBuffer&		rColorBuff = *pExcRoot->pColor;

	// ab hier zeigt pD und pP auf gueltige Daten
	// Basteln der XF-Pattern

	// Value-Format -------------------------------------------------
	if( bCellXF || pD->HasValueFormat() )
		pItemSet->Put( SfxUInt32Item( ATTR_VALUE_FORMAT, (UINT32) pD->ValueFormat() ) );

	// Font ---------------------------------------------------------
	if( bCellXF || pD->HasFont() )
		pExcRoot->pFontBuffer->Fill( pD->Font(), *pItemSet );

	// Lock / Hidden ------------------------------------------------
	if( bCellXF || pD->HasProt() )
	{
		ScProtectionAttr	aAttr;

		if( !pD->IsLocked() )
			aAttr.SetProtection( FALSE );

		if( pD->IsHidden() )
			aAttr.SetHideFormula( TRUE );

		pItemSet->Put( aAttr );
	}

	// Bordereinstellungen ------------------------------------------
	if( bCellXF || pD->HasBorder() )
	{
		const CellBorder*	pBorder = pD->Border();

		if( pBorder )
			SetBorder( *pItemSet, rColorBuff,
						pBorder->nLeftLine, pBorder->nLeftColor,
						pBorder->nRightLine, pBorder->nRightColor,
						pBorder->nTopLine, pBorder->nTopColor,
						pBorder->nBottomLine, pBorder->nBottomColor );
	}

	// Hintergrund- / Fill-Einstellungen ----------------------------
	if( bCellXF || pD->HasFill() )
	{
		const CellFill*		pFill = pD->Fill();

		if( pFill )
			SetFill( *pItemSet, rColorBuff, pFill->nPattern, pFill->nForeColor, pFill->nBackColor );
	}

	// Ausrichtungsattribute ----------------------------------------
	// Horizontal
	if( pD->HasIndent() )
		pItemSet->Put( SfxUInt16Item( ATTR_INDENT, pD->GetIndent() * nIndentFact ) );

	if( bCellXF || pD->HasHorizAlign() )
	{
		SvxCellHorJustify	e;

		switch( pD->HorizAlign() )
		{
			case EHA_Parent:
			case EHA_General:		e = SVX_HOR_JUSTIFY_STANDARD;	break;
			case EHA_Left:			e = SVX_HOR_JUSTIFY_LEFT;		break;
			case EHA_CentAcrSel:
			case EHA_Center:		e = SVX_HOR_JUSTIFY_CENTER;		break;
			case EHA_Right:			e = SVX_HOR_JUSTIFY_RIGHT;		break;
			case EHA_Fill:			e = SVX_HOR_JUSTIFY_REPEAT;		break;
			case EHA_Justify:		e = SVX_HOR_JUSTIFY_BLOCK;		break;
			default:
				DBG_ERROR( "+XF_Buffer::NewXF(): Attribut Vert fehlt" );
				e = SVX_HOR_JUSTIFY_STANDARD;
		}

		pItemSet->Put( SvxHorJustifyItem( e ) );
	}

	// Wrap or not
	if( bCellXF || pD->HasWrap() )
	{
		SfxBoolItem aWrap( ATTR_LINEBREAK );
		ExcWrapText eWrap;

		eWrap = pD->Wrap();

		aWrap.SetValue( eWrap == EWT_Wrap );

		pItemSet->Put( aWrap );
	}

	// Vertical
	if( bCellXF || pD->HasVertAlign() )
	{
		SvxCellVerJustify	e;

		switch( pD->VertAlign() )
		{
			case EVA_Parent:
			case EVA_Top:			e = SVX_VER_JUSTIFY_TOP;		break;
			case EVA_Center:		e = SVX_VER_JUSTIFY_CENTER;		break;
			case EVA_Bottom:		e = SVX_VER_JUSTIFY_BOTTOM;		break;
			case EVA_Justify:		e = SVX_VER_JUSTIFY_TOP;		break;
			default:
				DBG_WARNING( "+XF_Buffer::NewXF(): Attribut Horiz fehlt" );
				e = SVX_VER_JUSTIFY_STANDARD;
		}


		pItemSet->Put( SvxVerJustifyItem( e ) );
	}

	if( bCellXF || pD->HasTextOrient() )
	{
		if( pD->TextOrient() == ETO_Rot && pD->GetTextRotation() != 255 )
		{// neumodscher Kram
			INT32			n = pD->GetTextRotation();	// 8-Bit-Wert

			DBG_ASSERT( n <= 180, "*XF_Buffer::CreatePattern(): Iiiiiiii, was is'n dat fuer'n Winkel?" );

			if( n > 90 )
				n = 90 - n;

			n *= 100;

			if( n < 0 )
				n += 36000;

			if( n == 9000 )
				pItemSet->Put( SvxOrientationItem( SVX_ORIENTATION_BOTTOMTOP ) );
			else if( n == 27000 )
				pItemSet->Put( SvxOrientationItem( SVX_ORIENTATION_TOPBOTTOM ) );

			pItemSet->Put( SfxInt32Item( ATTR_ROTATE_VALUE, n ) );

			pItemSet->Put( SvxRotateModeItem( SVX_ROTATE_MODE_BOTTOM, ATTR_ROTATE_MODE ) );
		}
		else
		{// traditionell und erprobt
			SvxCellOrientation	e;

			switch( pD->TextOrient() )
			{
				case ETO_Parent:
				case ETO_NoRot:			e = SVX_ORIENTATION_STANDARD;	break;
				case ETO_TopBottom:		e = SVX_ORIENTATION_STACKED;	break;
				case ETO_Rot:
					DBG_ASSERT( pD->GetTextRotation() == 255,
						"*XF_Buffer::CreatePattern(): 255 bricht mit allen Traditionen" );
				case ETO_90ccw:			e = SVX_ORIENTATION_BOTTOMTOP;	break;
				case ETO_90cw:			e = SVX_ORIENTATION_TOPBOTTOM;	break;
				default:
					DBG_ERROR( "+XF_Buffer::NewXF(): Attribut TextRot fehlt" );
					e = SVX_ORIENTATION_STANDARD;
			}

			pItemSet->Put( SvxOrientationItem( e ) );
		}
	}

	return pPatt;
}


BOOL XF_Buffer::HasAttribSuperOrSubscript( const UINT16 n ) const
{
	const ENTRY*			pEntry = ( const ENTRY* ) GetObject( n );

	if( pEntry && pEntry->pData )
		return pExcRoot->pFontBuffer->HasSuperOrSubscript( pEntry->pData->Font() );
	else
		return FALSE;
}


void XF_Buffer::SetBorder( SfxItemSet& rItemSet, ColorBuffer& rColBuff,
					const UINT8 nLL, const UINT16 nCL,
					const UINT8 nLR, const UINT16 nCR,
					const UINT8 nLT, const UINT16 nCT,
					const UINT8 nLB, const UINT16 nCB )
{
	SvxBoxItem		aBox( ATTR_BORDER );
	SvxBorderLine	*pT, *pL, *pB, *pR;

	ExcToScBorderLine( nLL, nCL, pL, rColBuff );
	ExcToScBorderLine( nLR, nCR, pR, rColBuff );
	ExcToScBorderLine( nLT, nCT, pT, rColBuff );
	ExcToScBorderLine( nLB, nCB, pB, rColBuff );

	aBox.SetLine( pL, BOX_LINE_LEFT );
	aBox.SetLine( pR, BOX_LINE_RIGHT );
	aBox.SetLine( pT, BOX_LINE_TOP );
	aBox.SetLine( pB, BOX_LINE_BOTTOM );

	rItemSet.Put( aBox );

	if( pB )
		delete pB;
	if( pT )
		delete pT;
	if( pR )
		delete pR;
	if( pL )
		delete pL;
}


void XF_Buffer::SetFill( SfxItemSet& rItemSet, ColorBuffer& rColorBuff,
				const UINT8 nPat, const UINT16 nFore, const UINT16 nBack )
{
	static const UINT16		nFakt[ 19 ] = {
		0xFFFF, 0xFFFF, 0x7FFF, 0x3FFF,			// 00 - 03
		0xBFFF, 0xB332, 0xB332, 0xB851,			// 04 - 07
		0xB851, 0xB851, 0x3FFF, 0xDC28,			// 08 - 11
		0xD709, 0xD709, 0xD709, 0xB851,			// 12 - 15
		0xB851, 0xDFFF, 0xEFFF };				// 16 -	18

	// !no background -> set nothing!
	if( nPat != 0 )
	{
		Color				aFore;
		if( nFore < 64 )
			aFore = rColorBuff.GetColor( nFore )->GetValue();
		else if( nFore == 64 )
			aFore = COL_BLACK;
		else
			aFore = COL_WHITE;

		Color				aBack;
		if( nBack < 64 )
			aBack = rColorBuff.GetColor( nBack )->GetValue();
		else if( nBack == 64 )
			aBack = COL_BLACK;
		else
			aBack = COL_WHITE;

		Color				aResCol;

		if( nPat < 19 )
		{
			if( nPat == 1 )
			{
				Color		aTemp = aBack;
				aBack = aFore;
				aFore = aTemp;
			}

			const UINT16	n = nFakt[ nPat ];

			aResCol = Color(	GetMixedCol( aBack.GetRed(), aFore.GetRed(), n ),
								GetMixedCol( aBack.GetGreen(), aFore.GetGreen(), n ),
								GetMixedCol( aBack.GetBlue(), aFore.GetBlue(), n ) );
		}
		else
			aResCol = aFore;

		rItemSet.Put( SvxBrushItem( aResCol ) );
	}
}


void XF_Buffer::SetStyle( UINT16 nXF, const String& rName )
{
	ENTRY*					pEntry = ( ENTRY* ) GetObject( nXF );

	if( pEntry && !pEntry->pData->IsCellXF() )
	{
		ScPatternAttr*		pPattAttr = CreatePattern( nXF );

		if( pPattAttr )
		{
//			ScPatternAttr*	pNewStyle = new ScPatternAttr( pPattAttr, rName );

			SfxItemSet*		pItemSet = &pExcRoot->pDoc->GetStyleSheetPool()->Make(
											rName, SFX_STYLE_FAMILY_PARA,
											SFXSTYLEBIT_USERDEF ).GetItemSet();

			pItemSet->Put( pPattAttr->GetItemSet() );
		}
	}
}




