/*************************************************************************
 *
 *  $RCSfile: excchart.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _EXCCHART_HXX
#define _EXCCHART_HXX


#ifndef _GEN_HXX //autogen
#include <tools/gen.hxx>
#endif
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _STACK_HXX
#include <tools/stack.hxx>
#endif
#ifndef _COM_SUN_STAR_LANG_XCOMPONENT_HPP_
#include <com/sun/star/lang/XComponent.hpp>
#endif
#ifndef SC_SCGLOB_HXX
#include <global.hxx>
#endif
#ifndef SC_RANGELST_HXX
#include <rangelst.hxx>
#endif
#ifndef _ROOT_HXX
#include "root.hxx"
#endif
#ifndef _XFBUFF_HXX
#include "xfbuff.hxx"
#endif
#ifndef _FONTBUFF_HXX
#include "fontbuff.hxx"
#endif
#ifndef _CHRTDEFS_HXX
#include "chrtdefs.hxx"
#endif


#define XCOMPONENT	com::sun::star::uno::Reference< com::sun::star::lang::XComponent >


//___________________________________________________________________

enum ExcChartState		// internal import state
{
	csUnknown,			// unknown rec groups
	csGlobal,			// lowest level
	csChart,			// chart base info
	csChartFrame,		// chart shape (line/area)
	csSeries,			// a series with formats
	csPointFormat,		// format of a data point or a series
	csDefText,			// a default text
	csDefTextFrame,		// shape of a default text
	csAxisParent,		// an axes set
	csAxis,				// axis (X, Y or Z)
	csFormat,			// chart format
	csLegend,			// legend
	csLegendText,		// text in a legend
	csLegendFrame,		// shape of the legend
	csDropbar,			// dropbar (stock charts)
	csStockFormat,		// data format (stock charts)
	csTitle,			// text of chart title
	csText,				// unknown text (becomes title or data caption)
	csTextFrame			// shape of an unknown text
};



class ExcStateStack : protected Stack
{
public:
	Stack::Clear;
	Stack::Count;

	inline void					Push( ExcChartState eState )	{ Stack::Push( (void*) eState ); }
	inline ExcChartState		Pop()							{ return (ExcChartState)(ULONG) Stack::Pop(); }
	inline ExcChartState		Top() const						{ return (ExcChartState)(ULONG) Stack::Top(); }
};


//___________________________________________________________________

struct ExcChart_Lineformat
{
	ColorData					nLineColor;
	UINT16						nPattern;				// 0=solid, 1=dash, 2=dot, 3=dash-dot,
														// 4=dash-dot-dot, 5=none, 6=dark gray,
														// 7=medium gray, 8=light gray
	INT16						nWeight;				// -1=hairline, 0=single, 1=double, 2=triple
	BOOL						bAuto			: 1;	// Autoformat
	BOOL						bDrawTick 		: 1;

								ExcChart_Lineformat( SvStream& rStrm );
								ExcChart_Lineformat( const ExcChart_Lineformat& rCpy );
};


//___________________________________________________________________

struct ExcChart_Areaformat
{
	ColorData					nForeColor;
	ColorData					nBackColor;
	UINT16						nPattern;
	BOOL						bAuto			: 1;
	BOOL						bInvertIfNeg	: 1;	// switch backgr and foregr

								ExcChart_Areaformat( SvStream& rIn );
								ExcChart_Areaformat( const ExcChart_Areaformat& rCpy );
};



//___________________________________________________________________

struct ExcChart_MarkerFormat
{
	ColorData					nForeColor;
	ColorData					nBackColor;
	UINT16						nMarkerType;		// 0=no marker, 1=square, 2=diamond, 3=triangle
													// 4=X, 5=star, 6=Dow-Jones, 7=standard deviation
													// 8=circle, 9=plus sign
	UINT32						nMarkerSize;
	BOOL						bAutoColor : 1;
	BOOL						bBackgroundNone : 1;
	BOOL						bForegroundNone : 1;

								ExcChart_MarkerFormat( SvStream& rIn );
								ExcChart_MarkerFormat( const ExcChart_MarkerFormat& rCpy );

	inline BOOL					IsDowJonesMarker( void ) const;
};


inline BOOL ExcChart_MarkerFormat::IsDowJonesMarker( void ) const
{
	return nMarkerType == EXC_CHART_MARKF_DOWJ;
}


//___________________________________________________________________

struct ExcChart_3DDataFormat
{
	UINT8						nBase;
	UINT8						nTop;
	inline						ExcChart_3DDataFormat( UINT8 nB, UINT8 nT );
	inline						ExcChart_3DDataFormat( const ExcChart_3DDataFormat& rCpy );
};


inline ExcChart_3DDataFormat::ExcChart_3DDataFormat( UINT8 nB, UINT8 nT ) :
		nBase( nB ),
		nTop( nT )
{	}

inline ExcChart_3DDataFormat::ExcChart_3DDataFormat( const ExcChart_3DDataFormat& rCpy ) :
		nBase( rCpy.nBase ),
		nTop( rCpy.nTop )
{	}


//___________________________________________________________________

struct ExcChart_Text
{
	String*						pString;
	const ScExcFont*			pFont;

	ExcChart_Lineformat*		pLine;				// shape border
	ExcChart_Areaformat*		pArea;				// shape background

	UINT32						nPosX;				// position/size in 1/4000 of chart area
	UINT32						nPosY;
	UINT32						nSizeX;
	UINT32						nSizeY;

	ExcTextOrient				eRot;				// -> xfbuff.hxx
	UINT16						nRot;				// 0...90 = 0...90, 91...180 = -1...-90, 255 = stacked

	UINT8						nAlignHor;			// 1=left, 2=center, 3=right, 4=justify
	UINT8						nAlignVert;			// 1=top, 2=center, 3=bottom, 4=justify

	UINT16						nMode;				// 1=transparent, 2=opaque

	UINT8						nDataLabelPlace;	// 1=outside (bar, pie), 2=inside (bar, pie)
													// 3=center (bar, line, pie), 4=axis (bar)
													// 5=above (line), 6=below (line)
													// 7=left (line), 8=right (line)
													// 9=auto (pie), 10=moved by user
													// 0=auto (pie), right (line), outside (bar),
													//		center (stacked bar)

	ColorData					nTextColor;

	UINT16						nLinkObj;			// 1 = chart title, 2 = Y axis, 3 = X axis
													// 4 = data point, 7 = Z axis
	UINT16						nLinkSer;			// number of series
	UINT16						nLinkPoint;			// number of data point

	BOOL						bAutoColor	: 1;
	BOOL						bShowKey	: 1;	// draw data label with legend key
	BOOL						bShowVal	: 1;	// text of label is the value of the data point
	BOOL						bVertical	: 1;	// text is vertical
	BOOL						bAutoText	: 1;	// generated
	BOOL						bGenerated	: 1;	// default or unmodified
	BOOL						bDeleted	: 1;	// automatic text label has been deleted by user
	BOOL						bAutoBackgr	: 1;	// background is set automaticaly
	BOOL						bShowLabPrct: 1;	// Pie-Chart: show category label and value as percentage
	BOOL						bShowPrct	: 1;	// Pie-Chart: show value as percentage
	BOOL						bShowBSize	: 1;	// show bubble sizes
	BOOL						bShowLabel	: 1;	// text of label is the category name

								ExcChart_Text( SvStream& rIn );
								ExcChart_Text( const ExcChart_Text& rCpy );
								~ExcChart_Text();
};


//___________________________________________________________________

enum ExcChartDirState
{
	dsNull,
	dsFirst,
	dsRows,
	dsCols
};


class ExcChart_LinkedData
{
private:
	ExcChartDirState			eState;
	BOOL						bBaeh;
	ScRangeListRef				xValList;
	ScRangeListRef				xTitleList;

	UINT16						nMinColVal;
	UINT16						nMinRowVal;

	UINT16						nSerCnt;

	ScRange						aLastRange;

public:
								ExcChart_LinkedData( const ExcChart_LinkedData& rCpy );
								ExcChart_LinkedData( const ScRange& rRange, BOOL bGuessDir, BOOL bIsTitle );
	inline						~ExcChart_LinkedData()	{}

	void						AppendData( const ScRange& rAdd, BOOL bGuessDir, BOOL bIsTitle );

	inline BOOL					GetDir() const			{ return eState != dsRows; }
	void						GetDir( const ScRange& rRange );

	inline const ScRangeListRef&
								GetRangeList() const	{ return xValList; }

	inline BOOL					IsBaeh( const ScRange& rRange, BOOL bIsTitle ) const
														{ return FALSE; }
	inline BOOL					IsBaeh() const			{ return bBaeh; }

	BOOL						Close();
								//  Baeh-Flag wird entsprechend gesetzt, Ergebnis in xValList
								// Return = TRUE -> Titel nicht mehr vorhanden weil ungueltig
	inline void					EndSer()				{ nSerCnt++; }
};


//___________________________________________________________________

struct ExcChart_Point
{
	const UINT32				nPointNum;

	ExcChart_Lineformat*		pLine;
	ExcChart_Areaformat*		pArea;
	ExcChart_Text*				pText;
	ExcChart_MarkerFormat*		pMarker;
	ExcChart_3DDataFormat*		p3DData;
	UINT16*						pPieDist;

								ExcChart_Point( UINT32 nPointNum );
								ExcChart_Point( const ExcChart_Point& rCpy );
								~ExcChart_Point();

	inline BOOL					HasStockFormat() const;
};


inline BOOL ExcChart_Point::HasStockFormat() const
{
	return (pMarker && pMarker->IsDowJonesMarker());
}


//___________________________________________________________________

class ExcChart_PointList : protected List
{
private:
	ExcChart_Point*				Find( UINT16 nPoint );
protected:
public:
								ExcChart_PointList()	{}
								ExcChart_PointList( const ExcChart_PointList& rCpy );
	virtual						~ExcChart_PointList();

								List::Count;

	inline ExcChart_Point*		First()	{ return (ExcChart_Point*) List::First(); }
	inline ExcChart_Point*		Next()	{ return (ExcChart_Point*) List::Next(); }
	inline const ExcChart_Point*
								Get( UINT32 nIndex ) const;

	inline void					Append( ExcChart_Point* pNew );
	ExcChart_Point&				GetPoint( UINT16 nPoint );
};


inline const ExcChart_Point* ExcChart_PointList::Get( UINT32 nIndex ) const
{
	return (const ExcChart_Point*) List::GetObject( nIndex );
}

inline void ExcChart_PointList::Append( ExcChart_Point* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}


//___________________________________________________________________

struct ExcChart_Series
{
	const UINT16				nSer;		// number of series
	ExcChart_Point*				pAll;		// formats for series
	ExcChart_PointList*			pSpec;		// list of different points
	BOOL						bSecondary : 1;

								ExcChart_Series( UINT16 nSerNum );
								ExcChart_Series( const ExcChart_Series& rCpy );
								~ExcChart_Series();

	ExcChart_Point&				GetPoint( UINT16 nPointNum );
};


//___________________________________________________________________

class ExcChart_SeriesList : protected List
{
private:
	ExcChart_Series*			pCurrSer;

	ExcChart_Series*			Find( UINT16 nSerNum );

protected:
public:
								ExcChart_SeriesList();
								ExcChart_SeriesList( const ExcChart_SeriesList& rCpy );
	virtual						~ExcChart_SeriesList();

								List::Count;

	inline ExcChart_Series*		First()	{ return (ExcChart_Series*) List::First(); }
	inline ExcChart_Series*		Next()	{ return (ExcChart_Series*) List::Next(); }
	inline const ExcChart_Series*
								Get( UINT32 nIndex ) const;

	inline void					Append( ExcChart_Series* pSer );

	ExcChart_Series&			GetSeries( UINT16 nSerNum );
	ExcChart_Point&				GetPoint( UINT16 nSerNum, UINT16 nPointNum );

	void						SetSecondaryFlag( UINT16 nSerNum );

};


inline const ExcChart_Series* ExcChart_SeriesList::Get( UINT32 nIndex ) const
{
	return (const ExcChart_Series*) List::GetObject( nIndex );
}

inline void ExcChart_SeriesList::Append( ExcChart_Series* pSer )
{
	List::Insert( pSer, LIST_APPEND );
}


//___________________________________________________________________























struct LEGEND
{
	UINT32						nPosX;				// 1/4000 of the chart area
	UINT32						nPosY;
	UINT32						nSizeX;
	UINT32						nSizeY;

	ExcChart_Lineformat*		pLine;
	ExcChart_Areaformat*		pArea;
	ExcChart_Text*				pText;

	UINT8						nType;				// 0=bottom, 1=corner, 2=top, 3=right, 4=left,
													// 7=not docked or inside the plot area
	UINT8						nSpacing;			// 0=close, 1=medium, 2=open
	BOOL						bDocked : 1;
	BOOL						bAutoSeries : 1;	// automatic series distribution
	BOOL						bAutoPosX : 1;
	BOOL						bAutoPosY : 1;
	BOOL						bVert : 1;			// =TRUE -> single column of entries,
													// =FALSE -> horizontal legend,
													//  manual sized: always FALSE
	BOOL						bWasDataTable : 1;

								LEGEND( void ){}
								LEGEND( const LEGEND& rCpy );
};


struct DATA3D
{
	UINT16						nRot;				// 0...360
	INT16						nElev;				// -90...+90
	UINT16						nDist;				// 0...100
	UINT16						nHeight;			// Height of plot volume relative to width and depth
	UINT16						nDepth;				// Depth of points relative to width
	UINT16						nGap;				// Space between series
	BOOL						bPerspective : 1;	// perspective transform
	BOOL						bCluster : 1;		// Clustered or stacked
	BOOL						b3DScaling : 1;		// use auto scaling
	BOOL						b2DWalls : 1;		// use 2D walls and gridlines

								DATA3D( void ){}
								DATA3D( const DATA3D& rCpy );
};


struct VALUERANGE
{
	double						fMin;				// Minimum auf Achse
	double						fMax;				// Maximum ~
	double						fMajorInc;			// Haupt-Inkrement
	double						fMinorInc;			// Unter ~
	double						fCross;				// Achsenabschnitt
	BOOL						bAutoMin : 1;
	BOOL						bAutoMax : 1;
	BOOL						bAutoMajor : 1;
	BOOL						bAutoMinor : 1;
	BOOL						bAutoCross : 1;
	BOOL						bLogScale : 1;
	BOOL						bReverse : 1;
	BOOL						bMaxCross : 1;		// Achsenabschnitt auf max. Wert

								VALUERANGE( SvStream& rIn );
								VALUERANGE( const VALUERANGE& rCopy );
};



struct TICK
{
	UINT8						nMajor;				// Tick marks
	UINT8						nMinor;
	UINT8						nLabel;

	ExcTextOrient				eRot;				// Ausrichtung Text
	UINT16						nRot;				// in Grad?

	ColorData					nTextColor;

	BOOL						bTransparent	: 1;	// background opaque / transparent
	BOOL						bAutoColor		: 1;	// Textfarbe automatisch
	BOOL						bAutoBack		: 1;	// Texthintergrund automatisch
	BOOL						bAutoRot		: 1;
								TICK( SvStream& rIn );
								TICK( const TICK& rCopy );
};


#define	INVALIDNUMFORMAT		0xFFFFFFFF


struct AXIS
{
	VALUERANGE*					pValueRange;
	TICK*						pTick;

	ExcChart_Lineformat*		pAxisLine;			// axis line
	ExcChart_Lineformat*		pMajorLine;			// major grid
	ExcChart_Lineformat*		pMinorLine;			// minor grid
	ExcChart_Lineformat*		pWallLine;			// X axis = wall, Y axis = floor
	ExcChart_Areaformat*		pWallArea;			// X axis = wall, Y axis = floor

	ExcChart_Text*				pTitle;				// axis title
	ExcChart_Text*				pCaptionText;		// axis caption

	UINT32						nNumForm;

								AXIS( void );
								AXIS( const AXIS& rCopy );
								~AXIS();

	inline void					SetTitle( ExcChart_Text* pTitle );
};



struct AXIS_PACK
{
	AXIS*						pCatAxis;
	AXIS*						pValueAxis;
	AXIS*						pSeriesAxis;

	UINT32						nAxisX;
	UINT32						nAxisY;
	UINT32						nAxisDX;
	UINT32						nAxisDY;

								AXIS_PACK( void );
								AXIS_PACK( const AXIS_PACK& rCopy );
								~AXIS_PACK();
};


struct DROPBAR
{
	UINT16						nGap;		// 0...100%
	ExcChart_Lineformat*		pLineForm;
	ExcChart_Areaformat*		pAreaForm;

								DROPBAR( void );
								DROPBAR( SvStream& rIn );
								DROPBAR( const DROPBAR& rCopy );
								~DROPBAR();

	inline void					ReadLineForm( SvStream& rIn );
	inline void					ReadAreaForm( SvStream& rIn );
};








class ExcChart : public ExcRoot
{
private:
protected:
	ExcStateStack*				pStateStack;
	ExcChartState				eNewState;

	ExcChart_LinkedData*		pSourceData;

	UINT32						nPosX;				// chart shape
	UINT32						nPosY;
	UINT32						nSizeX;
	UINT32						nSizeY;

	ExcChart_Lineformat*		pLine;
	ExcChart_Areaformat*		pArea;

	ExcChart_Text*				pChartTitle;

	ExcChart_SeriesList*		pSeries;
	UINT16						nCurSeries;
	ExcChart_Point*				pLastDataPoint;

	AXIS_PACK*					pPrimAP;
	AXIS_PACK*					pSecAP;
	AXIS_PACK*					pActAP;
	AXIS*						pActAxis;

	LEGEND*						pLegend;
	DATA3D*						p3D;

	DROPBAR*					pUpBar;
	DROPBAR*					pDownBar;
	DROPBAR*					pActDropBar;

	INT16						nCurrDefText;		// number of default text
	ExcChart_Text*				pDefText0;			// data labels as categories
	ExcChart_Text*				pDefText1;			// data labels as value/perc.
	ExcChart_Text*				pDefText2;			// all text
	ExcChart_Text*				pNewText;			// unlinked text

	// pointers to current struct pointers
	ExcChart_Lineformat**		ppCurrLine;
	ExcChart_Areaformat**		ppCurrArea;
	ExcChart_Text**				ppCurrText;

public:
	BOOL						bPrimaryAndSecondary	: 1;	// FALSE -> primary only, TRUE -> ~

	BOOL						bHasSeriesNames			: 1;
	BOOL						bHasCategoryNames		: 1;

	BOOL						bDataPointMarker		: 1;
	BOOL						bStock					: 1;
	BOOL						bSpline					: 1;

protected:
	virtual void				ApplyExt( XCOMPONENT ) const;		// for derived classes
	virtual void				ApplyExtPost( XCOMPONENT ) const;	// after Apply(), before unlock
	virtual UINT16				GetDataRowOffset() const;			// offset for getDataRowProperties()

	void						UpdateState( ExcChartState eState );
	void						MoveNewTextTo( ExcChart_Text*& rpDest );
	ExcChart_Point&				GetPoint( UINT16 nSerNum, UINT16 nPointNum );

public:
	UINT16						nBaseTab;

								ExcChart( RootData* pRoot );
								ExcChart( ExcChart& rCpy );			// flat copy, deletes pointers in rCpy
	virtual						~ExcChart();

	void						Apply( XCOMPONENT xComponent, const Rectangle& rAnchor ) const;	// common

	inline ExcChartState		GetState() const	{ return pStateStack->Top(); }

	void						ReadChartBegin();
	void						ReadChartEnd();

	void						ReadChartChart( SvStream& rIn );

	void						ReadChartFrame();
	void						ReadChartLineformat( SvStream& rIn );
	void						ReadChartAreaformat( SvStream& rIn );

	void						ReadChartText( SvStream& rIn );
	void						ReadChartFontx( SvStream& rIn );
	void						SetChartSeriestext( const String& rString );
	void						ReadChartObjectlink( SvStream& rIn );
	void						ReadChartDefaulttext( SvStream& rIn );
	void						LinkNewText();

	void						ReadChartSeries();
	void						AddSourceData( const ScRange& rRange, BOOL bValues, BOOL bIsTitle = FALSE );
	inline const ExcChart_LinkedData*
								GetSourceData() const	{ return pSourceData; }
	void						EndSerValues();

	void						ReadChartDataformat( SvStream& rIn );
	void						ReadChartMarkerformat( SvStream& rIn );
	void						ReadChart3DDataformat( SvStream& rIn );
	void						ReadChartPieformat( SvStream& rIn );
	void						ReadChartSerfmt( SvStream& rIn );








	void						ReadChartLegend( SvStream& rIn );
	void						ReadChart3D( SvStream& rIn );

	void						ReadChartAxislineformat( SvStream& rIn );
	void						SetChartAxis( UINT16 n );
	void						ReadChartTick( SvStream& rIn );
	void						ReadValuerange( SvStream& rIn );
	void						SetAxisNumberFormat( UINT32 nSvNumIndex );
	void						ReadChartAxisparent( SvStream& rIn );

	inline void					FoundEmptyDataPointMarker()	{ bDataPointMarker = FALSE; }

	void						StartNewDropBar( SvStream& rIn );

	void						Reset();

	Rectangle					GetFrame() const;	// pos / size

	static inline long			Points_16_16_ToHmm( UINT32 nFixPoint16_16 );

	void						CloseSourceData();

	void						SetChartGroupIndex( UINT16 nIndex );
};

inline long ExcChart::Points_16_16_ToHmm( const UINT32 n )
{
	static const double f = HMM_PER_TWIPS * TWIPS_PER_POINT / 65536.0;
	return (long)((double) n * f);
}



class ExcChartLine : public ExcChart
{
private:
protected:
	BOOL						bAreaChart		: 1;
	BOOL						bStacked		: 1;
	BOOL						bCatAsPercent	: 1;
	BOOL						bHasShadow		: 1;

	virtual void				ApplyExt( XCOMPONENT xComponent ) const;
	virtual void				ApplyExtPost( XCOMPONENT xComponent  ) const;
public:
								ExcChartLine( ExcChart& rChart, SvStream& rIn, BOOL bArea );
	virtual						~ExcChartLine();
};



class ExcChartBar : public ExcChart
{
private:
protected:
	INT16						nSpaceBeetwBar;			// % of bar width
	INT16						nSpaceBeetwCat;			// % of bar width
	BOOL						bHorizontal		: 1;	// FALSE = column, TRUE = bar
	BOOL						bStacked		: 1;
	BOOL						bCatAsPercent	: 1;
	BOOL						bHasShadow		: 1;

	virtual void				ApplyExt( XCOMPONENT xComponent ) const;
public:
								ExcChartBar( ExcChart& rChart, SvStream& rIn );
	virtual						~ExcChartBar();
};



class ExcChartPie : public ExcChart
{
private:
protected:
	UINT16						nAngleFirstSlice;	// in degrees
	UINT16						nHoleSize;			// size of hole in %
	BOOL						bHasShadow			: 1;
	BOOL						bShowLeaderLines	: 1;

	virtual void				ApplyExt( XCOMPONENT xComponent ) const;
public:
								ExcChartPie( ExcChart& rChart, SvStream& rIn );
	virtual						~ExcChartPie();
};



class ExcChartRadar : public ExcChart
{
private:
protected:
	virtual void				ApplyExt( XCOMPONENT xComponent ) const;
public:
								ExcChartRadar( ExcChart& rChart );
	virtual						~ExcChartRadar();
};



class ExcChartSurface : public ExcChart
{
private:
protected:
	virtual void				ApplyExt( XCOMPONENT xComponent ) const;
public:
								ExcChartSurface( ExcChart& rChart );
	virtual						~ExcChartSurface();
};



class ExcChartScatter : public ExcChart
{
private:
protected:
	UINT16						nBubbleSizeRatio;		// % of largest bubble comp. to chart in general
	UINT16						nBubbleSize;			// 1 = bubble size is area, 2 = ~ is width
	BOOL						bBubbles		: 1;	// bubble series
	BOOL						bShowNegBubbles	: 1;
	BOOL						bHasShadows		: 1;

	virtual void				ApplyExt( XCOMPONENT xComponent ) const;
	virtual UINT16				GetDataRowOffset() const;		// here 1!
public:
								ExcChartScatter( ExcChart& rChart, SvStream& rIn );
	virtual						~ExcChartScatter();
};
















inline void AXIS::SetTitle( ExcChart_Text* p )
{
	if( pTitle )
		delete pTitle;

	pTitle = p;
}






inline void DROPBAR::ReadLineForm( SvStream& r )
{
	if( pLineForm )
		delete pLineForm;
	pLineForm = new ExcChart_Lineformat( r );
}


inline void DROPBAR::ReadAreaForm( SvStream& r )
{
	if( pAreaForm )
		delete pAreaForm;
	pAreaForm = new ExcChart_Areaformat( r );
}


#endif

