/*************************************************************************
 *
 *  $RCSfile: excpivot.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/



#ifndef _EXCPIVOT_HXX
#define _EXCPIVOT_HXX

#ifndef SC_SCGLOB_HXX
#include "global.hxx"
#endif
#ifndef _ROOT_HXX
#include "root.hxx"
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
//_________________________________________________________

class UINT16List;
class ScDPSaveData;
//_________________________________________________________

enum PT_SOURCETYPE
{
	PTST_ExcelOrDB,					// M$ Excel list or database
	PTST_extern,					// external data source"
	PTST_MultConsoldRngs,			// multiple consolidation ranges
	PTST_PivTab,					// pivot table
	PTST_ScenMan,					// scenario manager summary report
	PTST_unknown
};



//_________________________________________________________

class PivotCacheItem
{
private:
protected:
public:
	virtual						~PivotCacheItem();
	virtual BOOL				IsString() const;
	virtual BOOL				IsDouble() const;
};



class PivotCacheStringItem : public PivotCacheItem
{
private:
	String						aText;

protected:
public:
	inline						PivotCacheStringItem( const String& rText );
	virtual BOOL				IsString() const;

	inline const String&		GetCont() const		{ return aText; }
};

inline PivotCacheStringItem::PivotCacheStringItem( const String& rText ) :
		aText( rText )
{	}



class PivotCacheDoubleItem : public PivotCacheItem
{
private:
	double						fVal;

protected:
public:
	inline						PivotCacheDoubleItem( double fNewVal );
	virtual BOOL				IsDouble() const;

	inline double				GetCont() const		{ return fVal; }
};

inline PivotCacheDoubleItem::PivotCacheDoubleItem( double fNewVal ) :
		fVal( fNewVal )
{	}



//_________________________________________________________

class PivotCacheField : private List
{
private:
	String						aName;
	BOOL						bReadLater;

	inline PivotCacheItem*		_First()	{ return (PivotCacheItem*) List::First(); }
	inline PivotCacheItem*		_Next()		{ return (PivotCacheItem*) List::Next(); }

protected:
public:
	inline						PivotCacheField( const String& rName );
	virtual						~PivotCacheField();

	inline const PivotCacheItem* Get( UINT16 nIndex ) const;

	inline const String*		GetName() const				{ return &aName; }
	inline BOOL					GetReadState() const		{ return bReadLater; }
	inline void					SetReadState( BOOL bRL )	{ bReadLater = bRL; }

	inline void					Append( PivotCacheItem* pNew );
};

inline PivotCacheField::PivotCacheField( const String& rName ) :
		aName( rName ),
		bReadLater( FALSE )
{	}

inline const PivotCacheItem* PivotCacheField::Get( UINT16 nIndex ) const
{
	return (const PivotCacheItem*) List::GetObject( nIndex );
}

inline void PivotCacheField::Append( PivotCacheItem* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}



//_________________________________________________________

class PivotCache : private List
{
private:
	UINT16						nStreamId;
	PT_SOURCETYPE				eSrcType;
	ScRange						aSrcRange;
	String						aFileName;
	String						aTabName;
	BOOL						bSelf;

	inline PivotCacheField*		_First()	{ return (PivotCacheField*) List::First(); }
	inline PivotCacheField*		_Next()		{ return (PivotCacheField*) List::Next(); }
	PivotCacheField*			GetFirstReadLater();
	PivotCacheField*			GetNextReadLater();

	inline void					Append( PivotCacheField* pNew );

protected:
public:
								PivotCache( RootData* pExcRoot, UINT16 nStreamId );
	virtual						~PivotCache();

	void						SetSourceType( UINT16 nType );
	void						SetSource( UINT8 nC1, UINT16 nR1, UINT8 nC2, UINT16 nR2,
											const String& rFName, const String& rTName,
											BOOL bSelfRef );

								List::Count;
	inline PT_SOURCETYPE		GetSourceType() const	{ return eSrcType; }
	inline const ScRange&		GetSourceRange() const	{ return aSrcRange; }
	inline const String&		GetFileName() const		{ return aFileName; }
	inline const String&		GetTabName() const		{ return aTabName; }
	inline BOOL					IsSelfRef() const		{ return bSelf; }
	const PivotCacheItem*		GetItem( UINT16 nFieldId, UINT16 nItemId ) const;
	const String*				GetFieldName( UINT16 nFieldId ) const;
};

inline void PivotCache::Append( PivotCacheField* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}



//_________________________________________________________

class PivotCacheList : private List
{
private:
	inline PivotCache*			_First()	{ return (PivotCache*) List::First(); }
	inline PivotCache*			_Next()		{ return (PivotCache*) List::Next(); }

protected:
public:
	virtual						~PivotCacheList();

	inline void					Append( PivotCache* pNew );
	inline PivotCache*			Get( UINT16 nIndex );
};

inline void PivotCacheList::Append( PivotCache* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}

inline PivotCache* PivotCacheList::Get( UINT16 nIndex )
{
	return (PivotCache*) List::GetObject( nIndex );
}



//_________________________________________________________

class PivotDataItem
{
private:
	UINT16						nIndex;
	UINT16						nFunction;
	UINT16						nDisplay;
	UINT16						nSXVDInd;
	UINT16						pSXVIInd;
	UINT16						nFormat;
	String*						pLayoutName;

protected:
public:
								PivotDataItem( UINT16 nI, UINT16 nFunc, UINT16 nD,
												UINT16 nF, String* pName );
	virtual						~PivotDataItem();

	UINT16						GetIndex() const		{ return nIndex; }
	UINT16						GetFunction() const		{ return nFunction; }
	const String*				GetLayoutName() const	{ return pLayoutName; }
};



//_________________________________________________________

class PivotDataItemList : protected List
{
private:
	inline PivotDataItem*		_First()	{ return (PivotDataItem*) List::First(); }
	inline PivotDataItem*		_Next()		{ return (PivotDataItem*) List::Next(); }

protected:
public:
	virtual						~PivotDataItemList();

	inline void					Append( PivotDataItem* pNew );

								List::Count;
	inline const PivotDataItem*	First()		{ return (const PivotDataItem*) List::First(); }
	inline const PivotDataItem*	Next()		{ return (const PivotDataItem*) List::Next(); }
};

inline void PivotDataItemList::Append( PivotDataItem* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}



//_________________________________________________________

class PivotFieldItem
{
private:
	UINT16						nType;		// -> SXVI
	UINT16						nCache;
	BOOL						bHidden;
	BOOL						bHideDetail;
	BOOL						bFormula;
	BOOL						bMissing;

protected:
public:
								PivotFieldItem( UINT16 nT, UINT16 nC, UINT16 nGrBit );
								PivotFieldItem( PivotFieldItem& r );

	inline UINT16				GetType() const			{ return nType; }
	inline UINT16				GetCacheIndex() const	{ return nCache; }
	inline BOOL					IsHidden() const		{ return bHidden; }
	inline BOOL					IsHideDetail() const	{ return bHideDetail; }
};



//_________________________________________________________

class PivotFieldItemList : protected List
{
private:
	inline PivotFieldItem*		_First()	{ return (PivotFieldItem*) List::First(); }
	inline PivotFieldItem*		_Next()		{ return (PivotFieldItem*) List::Next(); }

protected:
public:
	inline						PivotFieldItemList()	{	}
								PivotFieldItemList( PivotFieldItemList& r );
	virtual						~PivotFieldItemList();

	inline void					Append( PivotFieldItem* pNew );

								List::Count;
	inline const PivotFieldItem* First()	{ return (const PivotFieldItem*) List::First(); }
	inline const PivotFieldItem* Next()		{ return (const PivotFieldItem*) List::Next(); }
	inline const PivotFieldItem* Get( UINT32 nIndex ) const;
};

inline void PivotFieldItemList::Append( PivotFieldItem* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}

inline const PivotFieldItem* PivotFieldItemList::Get( UINT32 nIndex ) const
{
	return (const PivotFieldItem*) List::GetObject( nIndex );
}



//_________________________________________________________

class PivotTabField
{
private:
	PivotFieldItemList			aItemList;

	String*						pLayoutName;		// NULL = use name in cache
	UINT16						nCacheIndex;		// index to field name

	UINT16						nIndex;				// row/col = order of fields in list
													// page    = index to SXVI
	UINT16						nNumOfSubtotals;
	UINT16						nNumOfItems;
	USHORT						nAxis;
	UINT16						nSubType;
	BOOL						bShowEmpty;

protected:
public:
								PivotTabField( UINT16 nAxis, UINT16 nNOSubt,
												UINT16 nSType, UINT16 nNOItems,
												UINT16 nCI, const String* pName );
								PivotTabField( PivotTabField& r, UINT16 nSubT, const String* pName );
	virtual						~PivotTabField();

	inline void					AppendItem( UINT16 nItemType, UINT16 nCache, UINT16 nGrBit );

	inline void					SetIndex( UINT16 nInd )		{ nIndex = nInd; }
	inline void					SetShowEmpty( BOOL bShowE )	{ bShowEmpty = bShowE; }
	inline UINT16				GetIndex() const			{ return nIndex; }
	inline UINT16				GetSubType() const		   	{ return nSubType; }
	inline const PivotFieldItemList& GetItemList() const	{ return aItemList; }
	inline UINT16				GetCacheIndex() const		{ return nCacheIndex; }
	inline BOOL					GetShowEmpty() const		{ return bShowEmpty; }
	inline const String*		GetLayoutName() const		{ return pLayoutName; }
	const String*				GetCacheName( PivotCache* pCache ) const;
};

inline void PivotTabField::AppendItem( UINT16 nItemType, UINT16 nCache, UINT16 nGrBit )
{
	aItemList.Append( new PivotFieldItem( nItemType, nCache, nGrBit ) );
}

inline const String* PivotTabField::GetCacheName( PivotCache* pCache ) const
{
	return pCache ? pCache->GetFieldName( nCacheIndex ) : NULL;
}



//_________________________________________________________

class PivotTabFieldList : protected List
{
private:
	UINT16List*					pIndexList;
	UINT16						nIndexCnt;

	inline PivotTabField*		_First()	{ return (PivotTabField*) List::First(); }
	inline PivotTabField*		_Next()		{ return (PivotTabField*) List::Next(); }
	inline PivotTabField*		_Get( UINT32 nIndex ) const;
	PivotTabField*				_FindIndex( UINT16 nIndex );

protected:
public:
	inline						PivotTabFieldList();
	virtual						~PivotTabFieldList();

								List::Count;
	inline const PivotTabField*	First()	{ return (const PivotTabField*) List::First(); }
	inline const PivotTabField*	Next()	{ return (const PivotTabField*) List::Next(); }
	inline const PivotTabField*	Get( UINT32 nIndex ) const;

	const PivotTabField*		FirstInOrder();	// uses pIndexList if present
	const PivotTabField*		NextInOrder();

	inline void					Append( PivotTabField* pNew );
	inline void					SetIndex( UINT32 nField, UINT16 nIndex );
	void						AddIndex( UINT16 nIndex );

	PivotTabField*				CreateDuplicate( UINT16 nCache, UINT16 nSubT, const String* pName );
};

inline PivotTabFieldList::PivotTabFieldList() :
		pIndexList( NULL )
{	}

inline PivotTabField* PivotTabFieldList::_Get( UINT32 nIndex ) const
{
	return (PivotTabField*) List::GetObject( nIndex );
}

inline const PivotTabField* PivotTabFieldList::Get( UINT32 nIndex ) const
{
	return (const PivotTabField*) List::GetObject( nIndex );
}

inline void PivotTabFieldList::Append( PivotTabField* pNew )
{
	List::Insert( pNew, LIST_APPEND );
}

inline void PivotTabFieldList::SetIndex( UINT32 nField, UINT16 nIndex )
{
	PivotTabField* pField = _Get( nField );
	if( pField )
		pField->SetIndex( nIndex );
}



//_________________________________________________________

class PivotTable : public ExcRoot
{
// impl -> exctools.cxx
private:
	ScRange						aViewRange;
	ScRange						aOutputRange;

	PivotTabField*				pActField;
	PivotTabField*				pActDataField;

	PivotTabFieldList*			pColFieldList;
	PivotTabFieldList*			pRowFieldList;
	PivotTabFieldList*			pPageFieldList;
	PivotTabFieldList*			pDataFieldList;
	PivotTabFieldList*			pFullDataList;
	UINT16						nSxIvdTurn;

	UINT16						nFlags;
	UINT16						nDimCol;
	UINT16						nDimRow;
	UINT16						nDimPage;
	UINT16						nDimData;
	UINT16						nSxLiInd;		// current SXLI record

	PivotDataItemList*			pDataItemList;

	PivotCache*					pCache;
	UINT16						nCacheIndexCnt;

	void						Init();
	void						CreateFullDataList();
	void						DecodeSubTotals( long& rCount, USHORT* pFuncs, UINT16 nFlags );
	void						SetFields( PivotTabFieldList& rList, USHORT nOrient,
											ScDPSaveData& rSaveData );

protected:
public:
								PivotTable( SvStream& rIn, RootData* pRD,
											UINT8 nTab, INT32 nBytesLeft );
	virtual						~PivotTable();

	void						AddViewField( SvStream& rIn, INT32 nBytesLeft );
	inline void					SetShowEmpty( BOOL bShowEmpty );
	void						AddViewItem( UINT16 nItemType, UINT16 nCache, UINT16 nGrBit );
	void						ReadRCFieldIDs( SvStream& rIn, INT32 nBytesLeft );
	void						AddPageItemInfo( UINT16 nSXVD, UINT16 nSXVI );
	void						AddDataItem( SvStream& rIn, INT32 nBytesLeft );

	inline BOOL					IsInPivotRange( const ScAddress& rAddr ) const;

	void						Apply();
};

inline void PivotTable::SetShowEmpty( BOOL bShowE )
{
	if( pActField )
		pActField->SetShowEmpty( bShowE );
}

inline BOOL PivotTable::IsInPivotRange( const ScAddress& rAddr ) const
{
	return aOutputRange.In( rAddr );
}


//_________________________________________________________

class PivotTableList : protected List
{
private:
protected:
public:
	virtual						~PivotTableList();

								List::Count;
	inline PivotTable*			First()		{ return (PivotTable*) List::First(); }
	inline PivotTable*			Next()		{ return (PivotTable*) List::Next(); }
	inline PivotTable*			Get( UINT16 nIndex ) const;

	inline void					Append( PivotTable* pNewPivTab );

	BOOL						IsInPivotRange( UINT16 nCol, UINT16 nRow, UINT16 nTab ) const;

	void						Apply();
};

inline PivotTable* PivotTableList::Get( UINT16 n ) const
{
	return ( PivotTable* ) List::GetObject( n );
}

inline void PivotTableList::Append( PivotTable* p )
{
	List::Insert( p, LIST_APPEND );
}


#endif
