/*************************************************************************
 *
 *  $RCSfile: xcl97exp.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _XCL97EXP_HXX
#define _XCL97EXP_HXX


#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#include "flttools.hxx"


// wenn mehr (8228 inkl. ID und Len) => CONTINUE Records schreiben
#define SC_XCL_RECLENMAX 8224

// --- class XclContinue ---------------------------------------------
// Daten schreiben und evtl. CONTINUE Records erzeugen,
// wenn RecordOffset > SC_XCL_RECORDLENMAX wird.
// Dtor schreibt auf jeden Fall die Record-Laenge neu.

class XclContinue
{
private:
		SvStream&			rStrm;
		ULONG				nRecLenPos;
		UINT16				nPredictLen;

			void				WriteRecLen();

public:
								XclContinue( SvStream& rStrm, UINT16 nRecOff,
									UINT16 nPredictLen = 0 );
								~XclContinue();

	inline	UINT16				GetRecOff() const;

	inline	SvStream&			GetStream() const	{ return rStrm; }

			BOOL				Slice( UINT32& nElems, UINT32& nStop, UINT16 nFactor );

			void				NextRecord( UINT16 nPredictLen = 0 );

			void				Write( const UINT8* pData, UINT32 nElems );

			void				Write( const UINT16* pData, UINT32 nElems );

			void				WriteUnicode( const UINT16* pData, UINT32 nElems );
			void				WriteUnicodeLow( const UINT16* pData, UINT32 nElems );

			void				Write( UINT16List* pData, UINT32 nElems );

			void				Write( const UINT8* pData, UINT16 nStructSize,
									UINT32 nElems );

			void				WriteStream( SvStream& rIn, UINT32 nBytes );

			void				WriteMemStream( SvMemoryStream& rMem,
									UINT16 nStructSize );
};


inline UINT16 XclContinue::GetRecOff() const
{
	return (UINT16) (rStrm.Tell() - nRecLenPos - sizeof(UINT16));
}


// --- class XclRawUnicodeString -------------------------------------

class XclRawUnicodeString
{
								// not implemented
								XclRawUnicodeString( const XclRawUnicodeString& );
	XclRawUnicodeString&		operator=( const XclRawUnicodeString& );

private:
			UINT16*				pBuf;
			UINT16				nLen;
			BOOL				bHigh;

			void				Build( const sal_Unicode* pMem, UINT16 nLen, BOOL bForce16bit );

public:
								XclRawUnicodeString();
								XclRawUnicodeString( const String& rStr,
									UINT16 nMax = 0xFFFF,
									BOOL bForce16bit = FALSE );
								XclRawUnicodeString( const rtl::OUString& rStr,
									UINT16 nMax = 0xFFFF,
									BOOL bForce16bit = FALSE );
	virtual						~XclRawUnicodeString();

			void				Assign( const String& rStr,
									UINT16 nMax = 0xFFFF,
									BOOL bForce16bit = FALSE );
			void				Assign( const rtl::OUString& rStr,
									UINT16 nMax = 0xFFFF,
									BOOL bForce16bit = FALSE );

	inline	BOOL				HasHighByte() const		{ return bHigh; }
	inline	UINT32				GetByteFactor() const	{ return bHigh ? 2 : 1; }

	virtual	UINT8				GetGrbit() const;

								// character count
	inline	UINT16				GetLen() const			{ return nLen; }

								// number of bytes to be saved in 8/16 mode
	inline	UINT32				GetByteCount() const	{ return GetByteFactor() * GetLen(); }

								// no Grbit, String only, 8-Bit if not High, else 16-Bit.
			void				Write( XclContinue& rCont ) const;

								// no Grbit, GetByteCount must fit into memory
			void				WriteToMem( void* pMem ) const;

								// no Grbit, direct write, no CONTINUE records
			void				WriteToStream( SvStream& rStrm ) const;
};


// --- class XclUnicodeString ----------------------------------------

class XclSstList;

class XclUnicodeString : public XclRawUnicodeString
{
								// not implemented
								XclUnicodeString( const XclUnicodeString& );
	XclUnicodeString&			operator=( const XclUnicodeString& );

public:
								XclUnicodeString();
								XclUnicodeString( const String& rStr,
									UINT16 nMax = 0xFFFF );
	virtual						~XclUnicodeString() {}

	virtual	UINT16				GetFixLen() const;

								// number of bytes to be saved
	virtual	UINT32				GetByteCount() const;

								// cch, grbit, String 8/16, no RichString
	virtual	void				Write( XclContinue& rCont ) const;
};


// --- class XclRichString -------------------------------------------

class ExcRichStr;

class XclRichString : public XclUnicodeString
{
								// not implemented
								XclRichString( const XclRichString& );
	XclRichString&				operator=( const XclRichString& );

private:
		ExcRichStr*			pRichStr;

public:
								XclRichString();
								XclRichString( const String& rStr,
									UINT16 nMax = 0xFFFF );
	virtual						~XclRichString();

	inline	void				SetRichStr( ExcRichStr* pRS )
									{ pRichStr = pRS; }

	virtual	UINT8				GetGrbit() const;

	virtual	UINT16				GetFixLen() const;

								// number of bytes to be saved
	virtual	UINT32				GetByteCount() const;

								// cch, grbit, [FormatCnt], String 8/16, [Formats]
	virtual	void				Write( XclContinue& rCont ) const;
};



#endif // _XCL97EXP_HXX
