/*************************************************************************
 *
 *  $RCSfile: xcl97rch.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _XCL97RCH_HXX
#define _XCL97RCH_HXX

#ifndef _CHRTDEFS_HXX
#include "chrtdefs.hxx"
#endif
#ifndef _XCL97REC_HXX
#include "xcl97rec.hxx"
#endif
#ifndef _EXCUPN_HXX
#include "excupn.hxx"
#endif

#ifndef _COM_SUN_STAR_CHART_XCHARTDOCUMENT_HPP_
#include <com/sun/star/chart/XChartDocument.hpp>
#endif
#ifndef _COM_SUN_STAR_CHART_XDIAGRAM_HPP_
#include <com/sun/star/chart/XDiagram.hpp>
#endif
#ifndef _COM_SUN_STAR_CHART_X3DDISPLAY_HPP_
#include <com/sun/star/chart/X3DDisplay.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XSHAPE_HPP_
#include <com/sun/star/drawing/XShape.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif


#include "rangelst.hxx"
#ifndef _REF_HXX //autogen wg. SV_DECL_REF_LIST
#include <tools/ref.hxx>
#endif
SV_DECL_REF_LIST( ScRangeList, ScRangeList* );


// --- struct XclPosSize ---------------------------------------------

struct XclPosSize
{
	INT32						nPosX;
	INT32						nPosY;
	INT32						nWidth;
	INT32						nHeight;
	UINT16						nAngle;
};


// --- class XclChartDataFormat --------------------------------------
// contains global properties of data series

class XclChartDataFormat
{
private:
	BOOL						bDiff;

public:
	// line format
	UINT16						nLineStyle;
	INT16						nLineWidth;
	Color						aLineColor;
	UINT16						nLineIndex;
	// area format
	UINT16						nAreaStyle;
	Color						aAreaColor;
	UINT16						nAreaIndex;
	// pie segment offset
	UINT16						nSegment;
	// line symbols
	BOOL						bHasSymbol;
	// data captions
	UINT16						nFontx;
	Color						aTextColor;
	UINT16						nTextIndex;
	UINT16						nTextFlags;
	UINT16						nLabelFlags;
	// 3D bar style
	UINT8						nDataFormatBase;
	UINT8						nDataFormatTop;

								XclChartDataFormat();
								XclChartDataFormat( const XclChartDataFormat& rSrc );

	void						SetLineFormat( UINT16 nStyle, INT16 nWidth, const Color& rColor, UINT16 nIndex );
	void						SetAreaFormat( UINT16 nStyle, const Color& rColor, UINT16 nIndex );
	inline void					SetSegmentOffset( UINT16 nSeg );
	inline void					SetSymbol( BOOL bHas );
	void						SetTextFormat( UINT16 nFx, const Color& rColor, UINT16 nIndex,
												UINT16 nTFl, UINT16 nLFl );
	void						SetSolidType( UINT8 nBase, UINT8 nTop );

	UINT16						GetSymbol( UINT16 nSeries ) const;

	inline BOOL					IsDifferent() const		{ return bDiff; }
};


inline void XclChartDataFormat::SetSegmentOffset( UINT16 nSeg )
{
	bDiff |= (nSegment != nSeg);
	nSegment = nSeg;
}

inline void XclChartDataFormat::SetSymbol( BOOL bHas )
{
	bDiff |= (bHasSymbol && !bHas);
	bHasSymbol &= bHas;
}


// --- struct XclChartText -------------------------------------------
// contains data for a TEXT record

struct XclChartText
{
	UINT16						nSeries;
	UINT16						nPoint;
	UINT16						nFontx;
	ColorData					nRGB;
	UINT16						nIndex;
	UINT16						nFlags;

								XclChartText( UINT16 nSer, UINT16 nPt,
											const XclChartDataFormat& rFormats );
};


// --- class XclChartTextList ----------------------------------------
// list of XclChartText, write them at end of chart

class XclChartTextList : private List
{
private:
	inline XclChartText*		_First()	{ return (XclChartText*) List::First(); }
	inline XclChartText*		_Next()		{ return (XclChartText*) List::Next(); }

protected:
public:
	virtual						~XclChartTextList();

	inline const XclChartText*	First()		{ return (const XclChartText*) List::First(); }
	inline const XclChartText*	Next()		{ return (const XclChartText*) List::Next(); }

	inline void					Append( XclChartText* pText )
											{ List::Insert( pText, LIST_APPEND ); }
};


// --- class XclObjChart ---------------------------------------------

class ScChartArray;
//class ExcUPN;

class XclObjChart : public XclObj
{
private:
	enum ChartType
	{
		ctUnknown,
		ctLine,
		ctArea,
		ctBar,
		ctColumn,			// vertical Bar chart
		ctPie,
		ctDonut,			// Pie sub type
		ctScatter,			// XY
		ctNet,				// Radar
		ctStock
	};

	enum TextType			// for WriteGroupText(), WriteText(), WritePos()
	{
		ttNone,				// no POS record
		ttCommon,			// POS record
		ttTitle,			// POS record chart relative
		ttLegend,			// no POS record
		ttXAxisTitle,		// POS record X axis relative
		ttYAxisTitle,		// POS record Y axis relative
		ttZAxisTitle,		// POS record Z axis relative
		ttDataCaption		// caption of data points
	};

	enum DefaultAlign		// for WriteText()
	{
		daAboveTop,
		daBelowTop,
		daCenter,
		daAboveBottom,
		daBelowBottom
	};

	com::sun::star::uno::Reference< com::sun::star::drawing::XShape >		xChartShape;
	com::sun::star::uno::Reference< com::sun::star::chart::XChartDocument >	xChartDoc;
	com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >	xDocProp;
	com::sun::star::uno::Reference< com::sun::star::chart::XDiagram >		xDiagram;
	com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >	xDiaProp;
	com::sun::star::uno::Reference< com::sun::star::chart::X3DDisplay >		xDiagr3D;
	com::sun::star::uno::Any	aAny;				// used for Get...PropValue
	ScRangeListMemberList		aSeriesYRangeList;	// Y-values
	ScRangeListMemberList		aSeriesXRangeList;	// X-categories
	List						aSeriesYUpnList;	// Y-UPN values
	List						aSeriesXUpnList;	// X-UPN categories
	List						aSeriesNUpnList;	// N-UPN names, title or text
	List						aSerieslistList;	// list of SvUShort arrays
	XclChartTextList			aTextList;			// write TEXT groups at end of chart
	double						fX4000;
	double						fY4000;
	RootData&					rRootData;
	const SdrObject&			rChartObj;
	ExcPalette2&				rPal;
	UsedFontList&				rFontList;
	ScChartArray*				pChartArray;
	SvStream*					pStrm;
	ChartType					eChartType;
	XclPosSize					aDiaPosSize;
	XclChartDataFormat			aDiaFormats;
	UINT16						nMaxSegOffset;		// correct diagram area for pie chart
	USHORT						nLevel;				// BEGIN, END
	USHORT						nChartDataColumns;
	UINT16						nChartCategoriesDataType;
	UINT16						nChartGroup;
	UINT32						nCurrentAxis;
	BOOL						bInitOk			: 1;
	BOOL						bColHeaders		: 1;
	BOOL						bRowHeaders		: 1;
	BOOL						bXAxisDescr		: 1;
	BOOL						bYAxisDescr		: 1;
	BOOL						bChartStacked	: 1;
	BOOL						bChartPercent	: 1;
	BOOL						bChartLine		: 1;
	BOOL						bChartSpline	: 1;
	BOOL						bChart3D		: 1;
	BOOL						bChartDeep		: 1;
	BOOL						bStockUpDown	: 1;
	BOOL						bStockVolume	: 1;
	BOOL						bChartTypeRound	: 1;	// ctPie, ctDonut
	BOOL						bChartTypeBars	: 1;	// ctBar, ctColumn
	BOOL						bWriteMode		: 1;


	// properties
			BOOL				GetPropValue(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									const rtl::OUString& rString
									);
			BOOL				GetPropBool(	// FALSE = either property value is FALSE or property not found
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									const rtl::OUString& rString
									);
			BOOL				GetPropBool(	// property in rValue
									sal_Bool& rValue,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									const rtl::OUString& rString
									);
			BOOL				GetPropInt32(
									sal_Int32& rValue,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									const rtl::OUString& rString
									);
			BOOL				GetPropDouble(
									double& rValue,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									const rtl::OUString& rString
									);

			void				GetChartType();

			ColorData			GetColor(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									const rtl::OUString& rString
									);
			void				GetTextColor(
									Color& rColor, UINT16& rIndex,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);

			ExcFont*			NewExcFont(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);

			UINT16				GetFontx(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			void				GetTextFormat(
									UINT16& rFontx,
									Color& rColor, UINT16& rColIndex,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);

			void				GetLineformat(
									UINT16& rStyle, INT16& rWidth,
									Color& rColor, UINT16& rColIndex,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			void				GetAreaformat(
									UINT16& rStyle, Color& rColor, UINT16& rColIndex,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);

			BOOL				GetDataCaption(
									UINT16& rTextFlags, UINT16& rLabelFlags,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);

			void				GetFormats(
									XclChartDataFormat& rFormats,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									BOOL bGetLineArea, BOOL bGetText
									);

	// initialisation, global functions
			void				InitChartFormats();
	virtual	void				SaveCont( SvStream& );

								// FALSE	= find formats (colors & fonts)
								// TRUE		= save the chart
			void				GoThroughChart( BOOL bSetWriteMode );

			BOOL				InitInterface();
			void				InitValues();
			void				AppendToSeriesNUpnList( const ScAddress* pA );
			void				BuildSeriesList();

	// writing records...
			void				WriteBeginLevel();
			void				WriteEndLevel();

			void				WriteSheetHeaderSetup();
			void				WriteChart();
			void				WriteShtprops();

			inline void			WriteRGB( ColorData nRGB );

	// SERIES
			void				WriteTheSeries();
			void				WriteGroupSeries(
									UINT16 nSeriesIndex,
									UINT16 nSeriesNumber,
									UINT16 nXCatCount,
									UINT16 nYValCount,
									const ExcUPN& rYUPN,
									const ExcUPN* pXUPN,
									const ExcUPN* pNUPN
									);
			void				WriteSeries( UINT16 nXValCount, UINT16 nYValCount );
			void				WriteGroupDataformat(
									UINT16 nSeriesIndex,
									UINT16 nSeriesNumber,
									UINT16 nSeriesPoint,
									const XclChartDataFormat& rFormats
									);
			void				WriteDataformat(
									UINT16 nSeriesIndex,
									UINT16 nSeriesNumber,
									UINT16 nSeriesPoint
									);
			void				Write3DDataFormat( const XclChartDataFormat& rFormats );
			void				WriteMarkerformat( UINT16 nType, UINT16 nBordInd, UINT16 nFillInd );
			void				WriteSerfmt( UINT16 nFlags );
			void				WriteAttachedLabel( UINT16 nFlags );
			void				WriteSertocrt();
			void				WriteAI( const ExcUPN* pUPN, UINT8 nType, UINT8 nRef, UINT16 nFmt );

	// FRAME group
			void				WriteGroupFrame(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape,
									BOOL bAutoSize = TRUE
									);
			void				WriteGroupFrame(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									BOOL bAutoSize = TRUE
									);
			void				WriteFrame( BOOL bAutoSize );

			void				WriteLineformat(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			void				WriteLineformat( const XclChartDataFormat& rF );
			void				WriteLineformat(
									ColorData nRGB, UINT16 nIndex,
									UINT16 nStyle, INT16 nWidth, UINT16 nGrbit
									);

			void				WriteAreaformat(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			void				WriteAreaformat( const XclChartDataFormat& rF );
			void				WriteAreaformat(
									ColorData nForeRGB, ColorData nBackRGB,
									UINT16 nForeIndex, UINT16 nBackIndex,
									UINT16 nStyle, UINT16 nGrbit
									);

	// TEXT group
			void				WriteTheText();
			void				WriteDefaulttext( UINT16 nId );
			void				WriteGroupText(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xTextShape,
									TextType eType
									);
			void				WriteGroupText(
									UINT16 nSeries, UINT16 nPoint, UINT16 nFontx,
									ColorData nRGB, UINT16 nIndex, UINT16 nFlags
									);

			void				WriteText(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xTextShape,
									const XclPosSize& rPosSize,
									TextType eType
									);
			void				WriteText(
									const XclPosSize& rPosSize,
									ColorData nRGB, UINT16 nIndex,
									TextType eType,
									UINT16 nFlags
									);

			void				WriteFontx(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			void				WriteFontx( UINT16 nFontx );
			void				WriteAI();
			void				WriteSeriestext(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xTextShape
									);
			void				WriteSeriestext( const String& rStr );
			void				WriteSeriestext( const XclRawUnicodeString& rStr );
			void				WriteObjectlink( UINT16 nLinkObj, UINT16 nLinkVar1, UINT16 nLinkVar2 );

			void				WriteTextList();

	// POS record - position, size, rotation
			void				WritePos(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape,
									TextType eType
									);
			void				WritePos( const XclPosSize& rPos, TextType eType );

			void				WritePosData(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape,
									TextType eType
									);
			void				WritePosData( const XclPosSize& rPos, TextType eType );

			INT32				GetDefRelPos(
									INT32 nPos, INT32 nSize,
									INT32 nParPos, INT32 nParSize,
									DefaultAlign eDefAlign, INT32 nDist,
									INT32 nScaling = 4000
									);

			void				GetPosSize(
									XclPosSize& rP,
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape
									);

			BOOL				GetRotation(
									UINT16& nRealAngle,
									UINT16& nExcAngle,
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			UINT16				GetRotationFlags( UINT16 nExcAngle );

	// AXIS group
			void				SetCurrentAxesSet( UINT16 nAxesSet );
			void				SetCurrentAxis( UINT16 nAxis );
	inline	UINT16				GetCurrentAxesSet()	{ return UINT16(nCurrentAxis >> 16); }
	inline	UINT16				GetCurrentAxis()	{ return UINT16(nCurrentAxis & 0xFFFF); }

			void				WriteAllAxes();
			void				WriteAxesused( UINT16 nNumber );
			void				WriteTheAxes( UINT16 nAxesSet );
			void				WriteAxisparent();
			void				WritePlotArea();
			void				WriteGroupAxis( UINT16 nAxis );
			void				WriteFirstXAxis(
									com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xTitle,
									TextType& rType
									);
			void				WriteSecondXAxis();
			void				WriteFirstYAxis(
									com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xTitle,
									TextType& rType,
									BOOL bWriteExtras
									);
			void				WriteSecondYAxis();
			void				WriteFirstZAxis(
									com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xTitle,
									TextType& rType
									);
			void				WriteAxis();
			void				WriteXZRange();
			void				WriteCatserrange();
			void				WriteAxcext();
			void				WriteTick(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp,
									BOOL bHasDescr
									);
			void				WriteValuerange(
									const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xProp
									);
			void				WriteAxislineformat( UINT16 nId );

	// chart type formats
			void				WriteGroupChartformat( UINT16 nAxesSet );
			void				WriteChartformat( UINT16 nZOrder );
			void				WriteSerieslist( UINT16 nAxesSet );

	// chart types
			inline void			WriteLine()		{ WriteLineArea( 0x1018 ); }
			inline void			WriteArea()		{ WriteLineArea( 0x101A ); }
			void				WriteLineArea( UINT16 nRecId );
			void				WriteBar();
			void				WritePie();
			void				WritePieformat( UINT16 nPercent );
			void				WriteScatter();
			void				WriteRadar();
			void				Write3D();
			void				WriteChartline( UINT16 nId );
			void				WriteGroupDropbar( int nBar );
			void				WriteDropbar();

	// main title, legend
			void				WriteTheMainTitle();

			void				WriteGroupLegend();
			void				WriteLegend(
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape
									);
	// SIINDEX
			void				WriteTheSiindex();
			void				WriteSiindex( UINT16 nIndex );

public:
								XclObjChart( RootData& rRoot,
									const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& rShape
									);
	virtual						~XclObjChart();

//	virtual	UINT16				GetNum() const;		// done by XclObj
	virtual	UINT16				GetLen() const;
};


inline void XclObjChart::WriteRGB( ColorData nRGB )
{
	*pStrm	<< COLORDATA_RED( nRGB )
			<< COLORDATA_GREEN( nRGB )
			<< COLORDATA_BLUE( nRGB )
			<< (UINT8)0;
}




#endif // _XCL97RCH_HXX
