/*************************************************************************
 *
 *  $RCSfile: xcl97exp.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:45:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop


#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif


#include "xcl97exp.hxx"
#include "excrecds.hxx"


// --- class XclContinue ---------------------------------------------

XclContinue::XclContinue( SvStream& rStrmP, UINT16 nRecOff, UINT16 nPredictLenP )
		:
		rStrm( rStrmP ),
		nPredictLen( nPredictLenP )
{
	nRecLenPos = rStrm.Tell() - nRecOff - sizeof(UINT16);
}


XclContinue::~XclContinue()
{
	WriteRecLen();
}


void XclContinue::WriteRecLen()
{
	ULONG nPos = rStrm.Tell();
	DBG_ASSERT( nPos - nRecLenPos - sizeof(UINT16) <= SC_XCL_RECLENMAX,
		"XclContinue::WriteRecLen: record overflow" );
	UINT16 nLen = (UINT16) (nPos - nRecLenPos - sizeof(UINT16));
	if ( nLen != nPredictLen )
	{
		rStrm.Seek( nRecLenPos );
		rStrm << nLen;
		rStrm.Seek( nPos );
	}
}


void XclContinue::NextRecord( UINT16 nPredictLenP )
{
	WriteRecLen();
	rStrm << (UINT16) 0x003c << nPredictLenP;
	nRecLenPos = rStrm.Tell() - sizeof(UINT16);
	nPredictLen = nPredictLenP;
}


BOOL XclContinue::Slice( UINT32& nElems, UINT32& nStop, UINT16 nFactor )
{
	BOOL bNew;
	UINT16 nBytes;
	UINT16 nRecOff = GetRecOff();
	if ( nRecOff + nFactor > SC_XCL_RECLENMAX )
	{
		nBytes = (UINT16) Min( nElems * nFactor, (UINT32) SC_XCL_RECLENMAX );
		UINT16 nMod = nBytes % nFactor;
		if ( nMod != 0 )
			nBytes -= nMod;
		NextRecord( nBytes );
		bNew = TRUE;
	}
	else
	{
		nBytes = (UINT16) Min( nElems * nFactor, (UINT32) SC_XCL_RECLENMAX - nRecOff );
		UINT16 nMod = nBytes % nFactor;
		if ( nMod != 0 )
			nBytes -= nMod;
		bNew = FALSE;
	}
	UINT16 nCnt = nBytes / nFactor;
	nStop += nCnt;
	nElems -= nCnt;
	return bNew;
}


void XclContinue::Write( const UINT8* pData, UINT32 nElems )
{
	UINT32 nHere = 0;
	UINT32 nStop = 0;
	while ( nElems )
	{
		Slice( nElems, nStop, sizeof(UINT8) );
		UINT32 nCnt = nStop - nHere;
		rStrm.Write( pData, nCnt );
		pData += nCnt;
		nHere += nCnt;
	}
}


void XclContinue::Write( const UINT16* pData, UINT32 nElems )
{
	UINT32 nHere = 0;
	UINT32 nStop = 0;
	while ( nElems )
	{
		Slice( nElems, nStop, sizeof(UINT16) );
		while ( nHere < nStop )
		{
			rStrm << pData[nHere++];
		}
	}
}


void XclContinue::WriteUnicode( const UINT16* pData, UINT32 nElems )
{
	UINT32 nHere = 0;
	UINT32 nStop = 0;
	while ( nElems )
	{
		if ( Slice( nElems, nStop, sizeof(UINT16) ) )
		{
			rStrm << (UINT8) 0x01;		// 16-Bit Flag
			nElems++;
			nStop--;
		}
		while ( nHere < nStop )
		{
			rStrm << pData[nHere++];
		}
	}
}


void XclContinue::WriteUnicodeLow( const UINT16* pData, UINT32 nElems )
{
	UINT32 nHere = 0;
	UINT32 nStop = 0;
	while ( nElems )
	{
		if ( Slice( nElems, nStop, sizeof(UINT8) ) )
		{
			rStrm << (UINT8) 0x00;		// 8-Bit Flag
			nElems++;
			nStop--;
		}
		while ( nHere < nStop )
		{
			rStrm << (UINT8) pData[nHere++];
		}
	}
}


void XclContinue::Write( UINT16List* pList, UINT32 nElems )
{
	UINT32 nHere = 0;
	UINT32 nStop = 0;
	while ( nElems )
	{
		Slice( nElems, nStop, sizeof(UINT16) );
		if ( nHere == 0 )
		{
			rStrm << pList->First();
			nHere++;
		}
		while ( nHere < nStop )
		{
			rStrm << pList->Next();
			nHere++;
		}
	}
}


void XclContinue::Write( const UINT8* pData, UINT16 nStructSize, UINT32 nElems )
{
	UINT32 nHere = 0;
	UINT32 nStop = 0;
	while ( nElems )
	{
		Slice( nElems, nStop, nStructSize );
		UINT32 nCnt = nStop - nHere;
		UINT32 nBytes = nCnt * nStructSize;
		rStrm.Write( pData, nBytes );
		pData += nBytes;
		nHere += nCnt;
	}
}


void XclContinue::WriteStream( SvStream& rIn, UINT32 nBytes )
{
	UINT8 aBuf[256];
	UINT8* pBuf = ( nBytes <= 256 ? &aBuf[0] : new UINT8[nBytes] );
	rIn.Read( pBuf, nBytes );
	Write( pBuf, nBytes );
	if ( pBuf != &aBuf[0] )
		delete [] pBuf;
}


void XclContinue::WriteMemStream( SvMemoryStream& rMem,	UINT16 nStructSize )
{
	UINT32 nElems = rMem.Seek( STREAM_SEEK_TO_END ) / nStructSize;
	const UINT8* pData = (const UINT8*) rMem.GetData();
	Write( pData, nStructSize, nElems );
}


// --- class XclRawUnicodeString -------------------------------------

XclRawUnicodeString::XclRawUnicodeString()
		:
		pBuf( NULL ),
		nLen( 0 ),
		bHigh( FALSE )
{
}


XclRawUnicodeString::XclRawUnicodeString( const String& rStr, UINT16 nMax, BOOL bForce16bit )
{
	Build( rStr.GetBuffer(), Min( rStr.Len(), ( xub_StrLen ) nMax ), bForce16bit );
}


XclRawUnicodeString::XclRawUnicodeString( const rtl::OUString& rStr, UINT16 nMax, BOOL bForce16bit )
{
	Build( rStr.getStr(), ( UINT16 ) min(	max( rStr.len(), ( xub_StrLen ) 0 ),
											( xub_StrLen ) nMax )
						, bForce16bit );
}


XclRawUnicodeString::~XclRawUnicodeString()
{
	if ( pBuf )
		delete [] pBuf;
}


void XclRawUnicodeString::Assign( const String& rStr, UINT16 nMax, BOOL bForce16bit )
{
	if ( pBuf )
		delete [] pBuf;
	Build( rStr.GetBuffer(), Min( rStr.Len(), ( xub_StrLen ) nMax ), bForce16bit );
}


void XclRawUnicodeString::Assign( const rtl::OUString& rStr, UINT16 nMax, BOOL bForce16bit )
{
	if ( pBuf )
		delete [] pBuf;
	Build( rStr.getStr(), UINT16(Min( Max( rStr.len(), INT32(0) ), INT32(nMax) )), bForce16bit );
}


void XclRawUnicodeString::Build( const sal_Unicode* pMem, UINT16 nLenP, BOOL bForce16bit )
{
	nLen = nLenP;
	pBuf = new UINT16[nLen+1];
	bHigh = bForce16bit;
// ER: 16.05.00 16:37  This looks so funny that I just comment it out instead of deleting it :)
// A good example of what automatisms may create (either by script or in your head).
//	UINT16 wcAlien = ( UINT16 ) *String::CreateFromAscii( "?" ).GetBuffer();		// Null-Ersatz
	const UINT16 wcAlien = '?';
	for ( USHORT j=0; j<nLen; j++ )
	{
		pBuf[j] = UINT16( pMem[j] );
		if ( !pBuf[j] )
			pBuf[j] = wcAlien;
		if ( pBuf[j] & 0xFF00 )
			bHigh = TRUE;
	}
	pBuf[nLen] = 0;
}


UINT8 XclRawUnicodeString::GetGrbit() const
{
	return bHigh ? 0x01 : 0x00;
}


void XclRawUnicodeString::Write( XclContinue& rCont ) const
{
	DBG_ASSERT( pBuf, "XclRawUnicodeString::Write: no String" );
	if ( !pBuf )
		return;
	if ( bHigh )
		rCont.WriteUnicode( pBuf, nLen );
	else
		rCont.WriteUnicodeLow( pBuf, nLen );
}


void XclRawUnicodeString::WriteToMem( void* pMem ) const
{
	DBG_ASSERT( pBuf, "XclRawUnicodeString::WriteToMem: no String" );
	if ( !pBuf )
		return;
	if ( bHigh )
	{
		UINT16* pDst = (UINT16*) pMem;
		for ( USHORT j=0; j<nLen; j++ )
		{
			pDst[j] = pBuf[j];
		}
	}
	else
	{
		UINT8* pDst = (UINT8*) pMem;
		for ( USHORT j=0; j<nLen; j++ )
		{
			pDst[j] = (UINT8) pBuf[j];
		}
	}
}


void XclRawUnicodeString::WriteToStream( SvStream& rStrm ) const
{
	DBG_ASSERT( pBuf, "XclRawUnicodeString::WriteToStream: no String" );
	if ( !pBuf )
		return;
	if ( bHigh )
	{
		DBG_ASSERT( nLen <= SC_XCL_RECLENMAX/2-4,
			"XclRawUnicodeString::WriteToStream: record overflow, use Write(XclContinue&) instead!" );
		for ( USHORT j=0; j<nLen; j++ )
		{
			rStrm << UINT16( pBuf[j] );
		}
	}
	else
	{
		DBG_ASSERT( nLen <= SC_XCL_RECLENMAX-4,
			"XclRawUnicodeString::WriteToStream: record overflow, use Write(XclContinue&) instead!" );
		for ( USHORT j=0; j<nLen; j++ )
		{
			rStrm << UINT8( pBuf[j] );
		}
	}
}


// --- class XclUnicodeString ----------------------------------------

XclUnicodeString::XclUnicodeString()
		:
		XclRawUnicodeString()
{
}


XclUnicodeString::XclUnicodeString( const String& rStr, UINT16 nMax )
		:
		XclRawUnicodeString( rStr, nMax, FALSE )
{
}


UINT16 XclUnicodeString::GetFixLen() const
{
	return 2 + 1;
}


UINT32 XclUnicodeString::GetByteCount() const
{
	return GetFixLen() + XclRawUnicodeString::GetByteCount();
}


void XclUnicodeString::Write( XclContinue& rCont ) const
{
	// #71367# Xcl has an obscure sense of whether starting a new record or not,
	// and crashes if it encounters the string header at the very end of a record.
	// Thus we add 1 to give some room, seems like they do it that way but with another count (10?)
	if ( rCont.GetRecOff() + GetFixLen() + 1 > SC_XCL_RECLENMAX )
		rCont.NextRecord();		// occurs in SST
	rCont.GetStream() << GetLen() << GetGrbit();
	XclRawUnicodeString::Write( rCont );
}


// --- class XclRichString -------------------------------------------

XclRichString::XclRichString()
		:
		XclUnicodeString(),
		pRichStr( NULL )
{
}


XclRichString::XclRichString( const String& rStr, UINT16 nMax )
		:
		XclUnicodeString( rStr, nMax ),
		pRichStr( NULL )
{
}


XclRichString::~XclRichString()
{
	delete pRichStr;
}


UINT16 XclRichString::GetFixLen() const
{
	return pRichStr ? (2 + 1 + 2) : (2 + 1);
}


UINT32 XclRichString::GetByteCount() const
{
	if ( pRichStr )
		return GetFixLen() + XclRawUnicodeString::GetByteCount() + pRichStr->GetByteCount();
	else
		return GetFixLen() + XclRawUnicodeString::GetByteCount();
}


UINT8 XclRichString::GetGrbit() const
{
	if ( pRichStr )
		return XclUnicodeString::GetGrbit() | 0x08;
	else
		return XclUnicodeString::GetGrbit();
}


void XclRichString::Write( XclContinue& rCont ) const
{
	// #71367# Xcl has an obscure sense of whether starting a new record or not,
	// and crashes if it encounters the string header at the very end of a record.
	// Thus we add 1 to give some room, seems like they do it that way but with another count (10?)
	if ( rCont.GetRecOff() + GetFixLen() + 1 > SC_XCL_RECLENMAX )
		rCont.NextRecord();		// occurs in SST
	SvStream& rStrm = rCont.GetStream();
	rStrm << GetLen() << GetGrbit();
	if ( pRichStr )
	{
		rStrm << pRichStr->GetFormCount();
		XclRawUnicodeString::Write( rCont );
		pRichStr->Write( rCont );
	}
	else
		XclRawUnicodeString::Write( rCont );
}


