/*************************************************************************
 *
 *  $RCSfile: fuedipo.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:44:56 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "ui_pch.hxx"
#endif

#pragma hdrstop

#ifndef PCH
#include <sbx.hxx>
#include <sfxitems.hxx>
#include <sfx.hxx>
#include "segmentc.hxx"
#endif

#include <dialogs.hrc>
#include <svdraw.hxx>

#include "tabvwsh.hxx"
#include "fuedipo.hxx"
#include "sc.hrc"


SEG_EOFGLOBALS()

/*************************************************************************
|*
|* Konstruktor
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_01)

FuEditPoints::FuEditPoints(ScTabViewShell* pViewSh, Window* pWin, SdrView* pView,
							   SdrModel* pDoc, SfxRequest& rReq) :
	FuDraw(pViewSh, pWin, pView, pDoc, rReq)
{
	pView->SetFrameDragSingles(FALSE);
}

/*************************************************************************
|*
|* Destruktor
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_02)

FuEditPoints::~FuEditPoints()
{
	pView->SetFrameDragSingles(TRUE);
}

/*************************************************************************
|*
|* MouseButtonDown-event
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_03)

BOOL __EXPORT FuEditPoints::MouseButtonDown(const MouseEvent& rMEvt)
{
	if ( pView->IsAction() )
	{
		if ( rMEvt.IsRight() )
			pView->BckAction();
		return TRUE;
	}

	BOOL bReturn = FALSE;

	aMDPos = pWindow->PixelToLogic( rMEvt.GetPosPixel() );

	if ( rMEvt.IsLeft() )
	{
		pWindow->CaptureMouse();
		SdrHdl* pHdl = pView->HitHandle(aMDPos, *pWindow);

		if ( pHdl || pView->IsMarkedHit(aMDPos) )
		{
			pView->BegDragObj(aMDPos, (OutputDevice*) NULL, pHdl, 1);
			bReturn = TRUE;
		}
		else
		{
			pView->UnmarkAll();
			if ( pView->MarkObj(aMDPos, -2, FALSE, rMEvt.IsMod1()) )
				pView->BegDragObj(aMDPos, (OutputDevice*) NULL, pHdl, 1);
			bReturn = TRUE;
		}
	}
	ForcePointer(&rMEvt);

	return bReturn;
}

/*************************************************************************
|*
|* MouseMove-event
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_04)

BOOL __EXPORT FuEditPoints::MouseMove(const MouseEvent& rMEvt)
{
	Point aPix(rMEvt.GetPosPixel());
	Point aPnt( pWindow->PixelToLogic(aPix) );

	if ( pView->IsAction() )
	{
		ForceScroll(aPix);
		pView->MovAction(aPnt);
	}
	ForcePointer(&rMEvt);

	return TRUE;
}

/*************************************************************************
|*
|* MouseButtonUp-event
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_05)

BOOL __EXPORT FuEditPoints::MouseButtonUp(const MouseEvent& rMEvt)
{
	BOOL bReturn = TRUE;

	Point aPnt( pWindow->PixelToLogic( rMEvt.GetPosPixel() ) );

	if ( pView->IsDragObj() )
		pView->EndDragObj( rMEvt.IsMod1() );

	else bReturn = FALSE;

	if ( !pView->IsAction() )
	{
		pWindow->ReleaseMouse();

		if ( pView->HasMarkedObj() )
		{
/*			if ( pView->GetMarkList().GetMark(0)->GetObj()->ISA(SdrPathObj) )
				SFX_DISPATCHER().Execute( SID_BEZIER_EDIT, SFX_CALLMODE_ASYNCHRON | SFX_CALLMODE_RECORD);
*/
		}
	}
	ForcePointer(&rMEvt);

	return bReturn;
}

/*************************************************************************
|*
|* Tastaturereignisse bearbeiten
|*
|* Wird ein KeyEvent bearbeitet, so ist der Return-Wert TRUE, andernfalls
|* FALSE.
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_06)

BOOL __EXPORT FuEditPoints::KeyInput(const KeyEvent& rKEvt)
{
	return FuDraw::KeyInput(rKEvt);
}

/*************************************************************************
|*
|* Funktion aktivieren
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_07)

void FuEditPoints::Activate()
{
	FuDraw::Activate();
}



/*************************************************************************
|*
|* Funktion deaktivieren
|*
\************************************************************************/

#pragma SEG_FUNCDEF(fuedipo_08)

void FuEditPoints::Deactivate()
{
	FuDraw::Deactivate();
}


/*************************************************************************

	  Source Code Control System - History

	  $Log: fuedipo.cxx,v $
	  Revision 1.1.1.1  2000/09/18 16:44:56  hr
	  initial import
	
	  Revision 1.15  2000/09/17 14:09:02  willem.vandorp
	  OpenOffice header added.
	
	  Revision 1.14  2000/08/31 16:38:24  willem.vandorp
	  Header and footer replaced
	
	  Revision 1.13  1998/07/03 20:52:40  NN
	  Parameter bDeep bei MarkObj angeben
	

	  Rev 1.12   03 Jul 1998 22:52:40   NN
   Parameter bDeep bei MarkObj angeben

	  Rev 1.11   08 Jan 1997 19:01:58   NN
   MouseEvent als Parameter bei ForcePointer

	  Rev 1.10   31 Jul 1996 18:43:22   NN
   Hit/Drag-Toleranz bei SdrView-Methoden raus

	  Rev 1.9   17 Jul 1996 10:56:54   NN
   327: UnMark -> Unmark

	  Rev 1.8   27 Jun 1996 11:50:52   NN
   Dispatcher::Execute Umstellung

	  Rev 1.7   01 Apr 1996 14:57:20   NN
   include svdraw.hxx statt drawview.hxx

	  Rev 1.6   15 Jun 1995 12:02:56   TRI
   EXPORTS

	  Rev 1.5   15 Jun 1995 10:24:50   TRI
   EXPORT

	  Rev 1.4   07 Jun 1995 08:44:30   MO
   SvxIds fuer DrawUI

	  Rev 1.3   04 Apr 1995 16:44:34   NN
   Execute-Aufruf raus

	  Rev 1.2   08 Feb 1995 13:31:28   MO
   * SID-Umbenennung


	  Rev 1.1   08 Feb 1995 11:16:10   MO
   * SID_EDIT_POINTS jetzt SID_TOOL_POINTS (gleicher Wert)


	  Rev 1.0   01 Feb 1995 18:21:08   NN
   Initial revision.

*************************************************************************/

#pragma SEG_EOFMODULE

