/*************************************************************************
 *
 *  $RCSfile: memchrt.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SCH_MEMCHRT_HXX
#define _SCH_MEMCHRT_HXX

#ifndef _SFXDEFS_HXX
#include <sfx2/sfxdefs.hxx>
#endif
#ifndef _SFXMODULE_HXX
#include <sfx2/module.hxx>
#endif
#ifndef _SV_COLOR_HXX
#include <vcl/color.hxx>
#endif
#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _COM_SUN_STAR_CHART_CHARTSERIESADDRESS_HPP_
#include <com/sun/star/chart/ChartSeriesAddress.hpp>
#endif

class SvStream;
class SfxItemSet;
class ChartModel;
class SvNumberFormatter;

#define SCH_COPY_HACK

enum ChartDataId
{
	CHDATAID_NONE,
	CHDATAID_MEMCHART,
	CHDATAID_DYNCHART,
	CHDATAID_MEMCHART_PLUS
};

// This struct contains information about selection in the chart, so that
// other apps (esp. calc) know what is currently selected

struct ChartSelectionInfo
{
	ChartSelectionInfo() : nRow(0), nCol(0), fValue(0.0), nValue(0), nSelection(0) {}
	ChartSelectionInfo(long sel) : nRow(0), nCol(0), fValue(0.0), nValue(0), nSelection(sel) {}
	BOOL operator == (const ChartSelectionInfo& s) const
	{
		return nRow				== s.nRow
			&& nCol				== s.nCol
			&& nSelection		== s.nSelection
			&& fValue			== s.fValue
			&& nValue			== s.nValue
			&& aSelectionColor	== s.aSelectionColor;
	}
	long	nRow;			 // Datenzeilen-Koordinate
	long	nCol;			 // Datenspaltenkoordinate
	double	fValue;			 // neuer Wert fuer die Zelle (nCol,nRow)
	long	nValue;			 // reserviert, evtl. fuer Numberformat und andere Informationen
	long	nSelection;      // s.u. , 31-Bit-Register (also besser Int32? -> return ist long)
	Color	aSelectionColor; // Farbe des Selektions-Rahmens
};

// Flags fuer nSelection UND den Rueckgabewert der CallBackFunktion
// long ChartSelectionCallback(struct ChartSelectionInfo *pInfo), die mittels
// SchMemChart::SetSelectionCallback(...) am Chart angemeldet wird

#define CHART_SEL_NONE			1	// bestehende Selektion aufheben (derzeit default, da im Chart keine Multiselektion besteht)
#define CHART_SEL_ALL			2	// den gesamten MemChart-Datenbereich selektieren
#define CHART_SEL_ROW			4	// die DatenReihe nRow selektieren
#define CHART_SEL_COL			8	// die DatenSpalte nCol Selektieren
#define CHART_SEL_POINT			16	// den Datenpunkt selektieren
#define CHART_SEL_SETCURSOR		32	// Cursor an die Stelle (nCol,nRow) setzen
#define CHART_SEL_CHANGEVALUE	64	// Wert in der Tabelle aendern (auf fValue)
#define CHART_SEL_COLOR			128	// Farbinformation fr farbigen Rahmen um den Selektionsbereich herum ist gesetzt
#define CHART_SEL_QUERYSUPPORT  256
#define CHART_SEL_QUERYPOSSIBLE 512

/*
 In der Callback-routine soll wie folgt verfahren werden:
 Kann eine Aktion (z.B. Aenderung des Wertes) ausgefuehrt werden, so MUSS das
 entsprechende Flag im Rueckgabewert gesetzt werden, kann die Aktion nicht ausgefuehrt
 werden, so DARF das Flag NICHT gesetzt werden.

Prioritaet der Aktionen:

 Prio 0 : CHART_SEL_QUERYSUPPORT
 Prio 1 : CHART_SEL_NONE   ; wichtig fuer (spaetere) Multiselektion.
 Prio 2 : CHART_SEL_BORDERONLY ; Selektionsmodus, wird vorerst immer gesetzt sein
 Prio n : der Rest....

CHART_SEL_QUERYSUPPORT:
 Ist dieses Flag gesetzt, DARF die Aktion NICHT ausgefuehrt werden, sondern nur
 zurueckgegeben werden, ob es moeglich waere!

CHART_SEL_NONE:

  Aufheben der Selektion, bei Multiselektion ist dieses Flag NICHT gesetzt, die neue Selektion
  wird zusaetzlich angewand. (derzeit im Chart nicht implementiert, daher immer gesetzt)

*/

/*************************************************************************
|*
|* SchMemChart: class for chart data
|*
\************************************************************************/

#define TRANS_NONE	0
#define TRANS_COL	1
#define TRANS_ROW	2
#define TRANS_ERROR 3

class SchMemChart
{
private:
	long			nLastSelInfoReturn;
	ULONG           nRefCount;
	mutable String*	mpColNameBuffer;
	mutable String*	mpRowNameBuffer;

protected :
	long			nTranslated;
	short			nRowCnt;
	short			nColCnt;
	String			aMainTitle;
	String			aSubTitle;
	String			aXAxisTitle;
	String			aYAxisTitle;
	String			aZAxisTitle;
	short			eDataType;
	String          aSomeData1;
	String          aSomeData2;
	String          aSomeData3;
	String          aSomeData4;
	double          *pData;
	String          *pColText;
	String          *pRowText;
	ChartDataId     myID;

	void QuickSortTableCols (long l,
							 long r,
							 long nRow);
	void QuickSortTableRows (long l,
							 long r,
							 long nCol);
	void QuickSortCols (long l,
						long r,
						long nRow);
	void QuickSortRows (long l,
						long r,
						long nCol);

	// number formatter and format id
	SvNumberFormatter*  mpNumFormatter;
	long *pRowNumFmtId;
	long *pColNumFmtId;

	// translation table for row-/column reordering (internal)
	long *pRowTable;
	long *pColTable;
	ChartSelectionInfo aSelectionInfo;

	// is not copied in copy contrustor (?)
	Link aAppLink;
	BOOL bReadOnly;

	rtl::OUString maCategoriesRangeAddress;
	com::sun::star::uno::Sequence<
		com::sun::star::chart::ChartSeriesAddress > maSeriesAddresses;

public:

	void SetReadOnly(BOOL b) { bReadOnly = b; }
	BOOL IsReadOnly() { return bReadOnly; }

	SchMemChart();
	SchMemChart(ChartDataId myID);
	SchMemChart(short nCols, short nRows);
	SchMemChart(const SchMemChart& rMemChart);
	virtual ~SchMemChart();

	// initialize number format, -1 means unset
	void InitNumFmt();

	// transfer chart selection to calc or writer
	const ChartSelectionInfo& GetSelectionInfo() const { return aSelectionInfo; }
	// calc/writer have to set this link
	long SubmitSelection(const ChartSelectionInfo& aInfo);


	// methods for translation of data
	BOOL TransCol(long nCol,BOOL bUp=TRUE);
	BOOL TransRow(long nRow,BOOL bUp=TRUE);
	void ResetTranslation(long *pTable,long nCnt);
	BOOL VerifyTranslation();
	long GetTranslation() const { return nTranslated; }
	void UpdateTranslation(long *pTable,long nCnt);
	// for acces always use these Get-methods !
	double GetTransData(long nCol,long nRow);
	double GetTransDataInPercent(long nCol ,long nRow,BOOL bRowData) const;
	const String& GetTransColText(long nCol) const;
	const String& GetTransRowText(long nRow) const;

	const long *GetRowTranslation() const { return pRowTable; }
	const long *GetColTranslation() const { return pColTable; }

#ifdef SCH_COPY_HACK
	const long* GetRowNumFmtTable() const { return pRowNumFmtId; }
	const long* GetColNumFmtTable() const { return pColNumFmtId; }

	// ********************
	// BM: Fix for #68864#
	// Use these four methods with care! The arrays MUST have the correct size!
	// ********************
	void SetRowTranslation( const long* pTransTable )
	{
		if( !pRowTable ) return;
		for( long i=0; i<nRowCnt; i++ )
			pRowTable[ i ] = pTransTable[ i ];
	}
	void SetColTranslation( const long* pTransTable )
	{
		if( !pColTable ) return;
		for( long i=0; i<nColCnt; i++ )
			pColTable[ i ] = pTransTable[ i ];
	}
	void SetRowNumFmtTable( const long* pNumFmtTable )
	{
		if( !pRowNumFmtId ) return;
		for( long i=0; i<nRowCnt; i++ )
			pRowNumFmtId[ i ] = pNumFmtTable[ i ];
	}
	void SetColNumFmtTable( const long* pNumFmtTable )
	{
		if( !pColNumFmtId ) return;
		for( long i=0; i<nColCnt; i++ )
			pColNumFmtId[ i ] = pNumFmtTable[ i ];
	}
	// this is only valid if the corresponding translation table was set correctly!
	void SetTranslation( long nTrans ) { nTranslated = nTrans; }
	// ********************
#endif
	
	long GetTableIndexCol(long nCol) const;
	long GetTableIndexRow(long nRow) const;

	BOOL SwapRowTranslation(long n1, long n2);
	BOOL SwapColTranslation(long n1, long n2);

	
	// number format

	void SetNumberFormatter(SvNumberFormatter* pNumFormatter) { mpNumFormatter = pNumFormatter; }
	SvNumberFormatter*  GetNumberFormatter() const { return mpNumFormatter;	}

	void SetNumFormatIdRow( const long nRow, const long nFmtId )	{ if( pRowNumFmtId && nRow < nRowCnt ) pRowNumFmtId[ nRow ] = nFmtId; }
	void SetNumFormatIdCol( const long nCol, const long nFmtId )	{ if( pColNumFmtId && nCol < nColCnt ) pColNumFmtId[ nCol ] = nFmtId; }
	long GetNumFormatIdRow( const long nRow ) const					{ return ( pRowNumFmtId && nRow < nRowCnt )? pRowNumFmtId[ nRow ]: -1; }
	long GetNumFormatIdCol( const long nCol ) const					{ return ( pColNumFmtId && nCol < nColCnt )? pColNumFmtId[ nCol ]: -1; }
	long GetTransNumFormatIdRow( const long nRow ) const;
	long GetTransNumFormatIdCol( const long nCol ) const;

	
	void SetSelectionHdl(const Link& rLink) { aAppLink=rLink; }

	void SetNonNumericData(const SchMemChart &rMemChart);

	void SetDataType(short eType) { eDataType = eType; }
	short GetDataType() const { return eDataType; }

	short GetColCount() const { return nColCnt;	}
	short GetRowCount() const {	return nRowCnt;	}

	const String& GetMainTitle() const	{ return aMainTitle; }
	const String& GetSubTitle() const	{ return aSubTitle; }

	void SetMainTitle(const String& rText) { aMainTitle = rText; }
	void SetSubTitle(const String& rText) { aSubTitle = rText; }

	const String& GetXAxisTitle() const	{ return aXAxisTitle; }
	const String& GetYAxisTitle() const	{ return aYAxisTitle; }
	const String& GetZAxisTitle() const	{ return aZAxisTitle; }
	
	void SetXAxisTitle(const String& rText) { aXAxisTitle = rText; }
	void SetYAxisTitle(const String& rText) { aYAxisTitle = rText; }
	void SetZAxisTitle(const String& rText) { aZAxisTitle = rText; }

	friend SvStream& operator << (SvStream& rOut, const SchMemChart& rMemChart);
	friend SvStream& operator >> (SvStream& rIn, SchMemChart& rMemChart);

	const String& GetColText(short nCol) const { return pColText[nCol]; }
	const String& GetRowText(short nRow) const { return pRowText[nRow]; }

	ChartDataId GetId() const { return myID; }
	double GetData(short nCol, short nRow) const { return pData[nCol * nRowCnt + nRow]; }
	double GetDataInPercent(const short nCol , const short nRow, const BOOL bRowData) const;

	virtual void InsertCols(short nAtCol, short nCount);
	virtual void RemoveCols(short nAtCol, short nCount);
	virtual void InsertRows(short nAtRow, short nCount);
	virtual void RemoveRows(short nAtRow, short nCount);
	virtual void SwapCols(int nAtCol1, int nAtCol2);
	virtual void SwapRows(int nAtRow1, int nAtRow2);

	void SetData(short nCol, short nRow, const double& rVal)
	{ pData[nCol * nRowCnt + nRow] = rVal; }

	String& SomeData1() { return aSomeData1; }
	String& SomeData2() { return aSomeData2; }
	String& SomeData3() { return aSomeData3; }
	String& SomeData4() { return aSomeData4; }

	void SetColText(short nCol, const String& rText) { pColText[nCol] = rText; }
	void SetRowText(short nRow, const String& rText) { pRowText[nRow] = rText; }

	void SortTableCols (long nRow = 0) 
	{
		QuickSortTableCols (0, nColCnt - 1, nRow);
		ResetTranslation(pColTable, nColCnt);
	}
	
	void SortTableRows (long nCol = 0)
	{
		QuickSortTableRows (0, nRowCnt - 1, nCol);
		ResetTranslation(pRowTable, nRowCnt);
	}
	
	void SortCols (long nRow = 0)
	{
		QuickSortCols (0, nColCnt - 1, nRow);
		ResetTranslation(pColTable, nColCnt);
	}

	void SortRows (long nCol = 0)
	{
		QuickSortRows (0, nRowCnt - 1, nCol);
		ResetTranslation(pRowTable, nRowCnt);
	}

	friend class ChartModel;

	void IncreaseRefCount() { nRefCount++; }
	BOOL DecreaseRefCount()
	{
		DBG_ASSERT( nRefCount, "SchMemChart::DecreaseRefCount: illegal decrement of RefCount");
		if(nRefCount) nRefCount--;
		return (nRefCount==0);	// true iff object can be deleted
	}

	// get a string for default (column|row) labels
	// the indexes start at 0, but the text starts with "(Column|Row) 1"
	String GetDefaultColumnText( sal_Int32 nCol ) const;
	String GetDefaultRowText( sal_Int32 nRow ) const;

	// methods for getting/setting information about cell references
	void SetCategoriesRangeAddress( const rtl::OUString& rCellRangeAddress )
		{ maCategoriesRangeAddress = rCellRangeAddress; }
	void SetSeriesAddresses( const com::sun::star::uno::Sequence<
							 	com::sun::star::chart::ChartSeriesAddress >& rCellRangeAddresses )
		{ maSeriesAddresses = rCellRangeAddresses; }

	const rtl::OUString& GetCategoriesRangeAddress()
		{ return maCategoriesRangeAddress; }
	const com::sun::star::uno::Sequence< com::sun::star::chart::ChartSeriesAddress >&
		GetSeriesAddresses()
		{ return maSeriesAddresses; }
};

#endif	// _SCH_MEMCHRT_HXX

