/*************************************************************************
 *
 *  $RCSfile: calculat.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:29:21 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// header for Point, Rectangle
#ifndef _SV_GEN_HXX
#include <tools/gen.hxx>
#endif
// header for Polygon
#ifndef _SV_POLY_HXX
#include <vcl/poly.hxx>
#endif
// header for DBG_ASSERT
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
// header for XPolygon, XPolyPolygon
#ifndef _XPOLY_HXX
#include <svx/xpoly.hxx>
#endif
// header for Line
#ifndef _SV_LINE_HXX
#include <vcl/line.hxx>
#endif

#include "calculat.hxx"

BOOL SchCalculationHelper::IsPointInsideRectangle( const Point& rPoint, const Rectangle& rRect )
{
	BOOL bResult = rRect.IsInside( rPoint );

	if( bResult )
	{
		if( rPoint.X() == rRect.Left() ||
			rPoint.X() == rRect.Right() ||
			rPoint.Y() == rRect.Top() ||
			rPoint.Y() == rRect.Bottom() )
			return FALSE;
	}

	return bResult;
}

void SchCalculationHelper::IntersectPolygonWithRectangle( const XPolygon& rPolygon, const Rectangle& rRectangle, XPolyPolygon& aResult )
{
	aResult.Clear();

	if( rRectangle.IsInside( rPolygon.GetBoundRect() ) )
	{
		aResult.Insert( rPolygon );
		return;
	}

	XPolygon aRectPoly( rRectangle );
	Line aRectLines[ 4 ];

	aRectLines[ 0 ] = Line( aRectPoly[ 0 ], aRectPoly[ 1 ] );
	aRectLines[ 1 ] = Line( aRectPoly[ 1 ], aRectPoly[ 2 ] );
	aRectLines[ 2 ] = Line( aRectPoly[ 2 ], aRectPoly[ 3 ] );
	aRectLines[ 3 ] = Line( aRectPoly[ 3 ], aRectPoly[ 0 ] );

	const Point* pStart = &(rPolygon[ 0 ]);
	const Point* pEnd;

	Point aIntersection, aIntersection2;
	XPolygon aSubPoly;
	BOOL bStartInside, bEndInside;
	USHORT i;
	USHORT nLast = rPolygon.GetPointCount() - 1;

	for( i = 1; i <= nLast; i++ )
	{
		pEnd = &(rPolygon[ i ]);

		bStartInside = IsPointInsideRectangle( *pStart, rRectangle );
		bEndInside	 = IsPointInsideRectangle( *pEnd, rRectangle );

		if( bStartInside != bEndInside )		// => one intersection
		{
			Line aLine( *pStart, *pEnd );
			if( aLine.Intersection( aRectLines[ 0 ], aIntersection ) ||
				aLine.Intersection( aRectLines[ 1 ], aIntersection ) ||
				aLine.Intersection( aRectLines[ 2 ], aIntersection ) ||
				aLine.Intersection( aRectLines[ 3 ], aIntersection ) )
			{
				if( bStartInside )
				{
					aSubPoly[ aSubPoly.GetPointCount() ] = *pStart;
					aSubPoly[ aSubPoly.GetPointCount() ] = aIntersection;
					aResult.Insert( aSubPoly );
					aSubPoly.SetPointCount( 0 );		// clear polygon
				}
				else
				{
					aSubPoly[ aSubPoly.GetPointCount() ] = aIntersection;

					if( i == nLast )
					{
						aSubPoly[ aSubPoly.GetPointCount() ] = *pEnd;
						aResult.Insert( aSubPoly );
					}
				}
			}
			else
				DBG_TRACE( "SchCalculationHelper::IntersectPolygonWithRectangle: "
							"No intersection where one was expected" );
		}
		else if( !bStartInside )				// => both are outside => 0 or 2 intersections
		{
			long nLine = -1;
			Line aLine( *pStart, *pEnd );

			if( aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) ||
				aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) ||
				aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) ||
				aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) )
			{
				if( nLine < 3 && aLine.Intersection( aRectLines[ ++nLine ], aIntersection2 ) ||
					nLine < 3 && aLine.Intersection( aRectLines[ ++nLine ], aIntersection2 ) ||
					nLine < 3 && aLine.Intersection( aRectLines[ ++nLine ], aIntersection2 ) )
				{
					aSubPoly[ aSubPoly.GetPointCount() ] = aIntersection;
					aSubPoly[ aSubPoly.GetPointCount() ] = aIntersection2;

					aResult.Insert( aSubPoly );
					aSubPoly.SetPointCount( 0 );
				}
				else
				{
					DBG_TRACE( "SchCalculationHelper::IntersectPolygonWithRectangle: "
								"Only one intersection where two were expected" );
				}
			}
			// else no intersection
		}
		else 									// both are inside => no intersection
		{
			aSubPoly[ aSubPoly.GetPointCount() ] = *pStart;
			if( i == nLast )
			{
				aSubPoly[ aSubPoly.GetPointCount() ] = *pEnd;
				aResult.Insert( aSubPoly );
			}
		}

		pStart = pEnd;
	}
}

BOOL SchCalculationHelper::ClipLineAtRectangle( Line& aLine, const Rectangle& rRectangle )
{
	BOOL bStartInside = IsPointInsideRectangle( aLine.GetStart(), rRectangle ),
		bEndInside = IsPointInsideRectangle( aLine.GetEnd(), rRectangle );

	if( bStartInside && bEndInside )		 // line stays unclipped
		return TRUE;

	Polygon aRectPoly( rRectangle );
	Line aRectLines[ 4 ];

	aRectLines[ 0 ] = Line( aRectPoly[ 0 ], aRectPoly[ 1 ] );
	aRectLines[ 1 ] = Line( aRectPoly[ 1 ], aRectPoly[ 2 ] );
	aRectLines[ 2 ] = Line( aRectPoly[ 2 ], aRectPoly[ 3 ] );
	aRectLines[ 3 ] = Line( aRectPoly[ 3 ], aRectPoly[ 0 ] );

	Point aIntersection, aIntersection2;

	if( bStartInside != bEndInside )	// => one intersection
	{
		if( aLine.Intersection( aRectLines[ 0 ], aIntersection ) ||
			aLine.Intersection( aRectLines[ 1 ], aIntersection ) ||
			aLine.Intersection( aRectLines[ 2 ], aIntersection ) ||
			aLine.Intersection( aRectLines[ 3 ], aIntersection ) )
		{
			if( bStartInside )
				aLine.SetEnd( aIntersection );
			else
				aLine.SetStart( aIntersection );
		}
		else
		{
			DBG_TRACE( "SchCalculationHelper::ClipLineAtRectangle: "
						"No intersection where one was expected" );
		}
	}
	else		// bStartInside == bEndInside == FALSE  =>  0 or 2 intersections
	{
		long nLine = -1;

		if( aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) ||
			aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) ||
			aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) ||
			aLine.Intersection( aRectLines[ ++nLine ], aIntersection ) )
		{
			if( nLine < 3 && aLine.Intersection( aRectLines[ ++nLine ], aIntersection2 ) ||
				nLine < 3 && aLine.Intersection( aRectLines[ ++nLine ], aIntersection2 ) ||
				nLine < 3 && aLine.Intersection( aRectLines[ ++nLine ], aIntersection2 ) )
			{
				aLine.SetStart( aIntersection );
				aLine.SetEnd( aIntersection2 );
			}
			else
			{
				DBG_TRACE( "SchCalculationHelper::ClipLineAtRectangle: "
							"Only one intersection where two were expected" );
			}
		}
		else
			return FALSE;		// result is empty
	}

	return TRUE;
}

