/*************************************************************************
 *
 *  $RCSfile: SchXMLWrapper.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SFXDOCFILE_HXX
#include <sfx2/docfile.hxx>
#endif

#include "SchXMLWrapper.hxx"

#include <xmloff/SchXMLExport.hxx>
#include <xmloff/SchXMLImport.hxx>

#ifndef _XMLOFF_XMLKYWD_HXX
#include <xmloff/xmlkywd.hxx>
#endif

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#ifndef _UNO_MAPPING_HXX_
#include <uno/mapping.hxx>	//! only necessary until SfxMedium has a uno3 DataSource / DataSink
#endif

#ifndef _UNOTOOLS_PROCESSFACTORY_HXX_
#include <unotools/processfactory.hxx>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_XERRORHANDLER_HPP_
#include <com/sun/star/xml/sax/XErrorHandler.hpp>
#endif
#ifndef _COM_SUN_STAR_XML_SAX_XENTITYRESOLVER_HPP_
#include <com/sun/star/xml/sax/XEntityResolver.hpp>
#endif
#ifndef _COM_SUN_STAR_XML_SAX_INPUTSOURCE_HPP_
#include <com/sun/star/xml/sax/InputSource.hpp>
#endif
#ifndef _COM_SUN_STAR_XML_SAX_XDTDHANDLER_HPP_
#include <com/sun/star/xml/sax/XDTDHandler.hpp>
#endif
#ifndef _COM_SUN_STAR_XML_SAX_XPARSER_HPP_
#include <com/sun/star/xml/sax/XParser.hpp>
#endif
#ifndef _COM_SUN_STAR_IO_XACTIVEDATASOURCE_HPP_
#include <com/sun/star/io/XActiveDataSource.hpp>
#endif
#ifndef _COM_SUN_STAR_IO_XACTIVEDATACONTROL_HPP_
#include <com/sun/star/io/XActiveDataControl.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

#ifndef _COM_SUN_STAR_XML_SAX_SAXPARSEEXCEPTION_HPP_
#include <com/sun/star/xml/sax/SAXParseException.hpp>
#endif

using namespace com::sun::star;

SchXMLWrapper::SchXMLWrapper( uno::Reference< frame::XModel >& xModel,
							  SfxMedium& rMedium,
							  sal_Bool bShowProgress ) :
		mxModel( xModel ),
		mrMedium( rMedium ),
		mbShowProgress( bShowProgress )
{}

sal_Bool SchXMLWrapper::Import()
{
	if( ! mxModel.is())
	{
		DBG_ERROR( "got no Model for XML import" );
		return sal_False;
	}
	uno::Reference< lang::XServiceInfo > xServiceInfo( mxModel, uno::UNO_QUERY );
	if( ! xServiceInfo.is() ||
		! xServiceInfo->supportsService( rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.chart.ChartDocument"))))
	{
		DBG_ERROR( "Model for XML import is no chart::ChartDocument" );
		return sal_False;
	}
	uno::Reference< lang::XMultiServiceFactory > xServiceFactory( ::utl::getProcessServiceFactory());
	if( ! xServiceFactory.is())
	{
		DBG_ERROR( "got no service manager" );
		return sal_False;
	}
	// get data source
	uno::Reference< io::XActiveDataSource > xSource = mrMedium.GetDataSource();

	if( ! xSource.is())
	{
		DBG_ERROR( "data source missing" );
		return sal_False;
	}
	// get parser
	uno::Reference< uno::XInterface > xXMLParser( xServiceFactory->createInstance(
		rtl::OUString::createFromAscii( "com.sun.star.xml.sax.Parser")));
	if( ! xXMLParser.is())
	{
		DBG_ERROR( "com.sun.star.xml.sax.Parser service missing" );
		return sal_False;
	}
	// get a pipe for connecting the data source to the parser
	uno::Reference< uno::XInterface > xPipe( xServiceFactory->createInstance(
		rtl::OUString::createFromAscii( "com.sun.star.io.Pipe")));
	if( ! xPipe.is())
	{
		DBG_ERROR( "com.sun.star.io.Pipe service missing" );
		return sal_False;
	}
	
	sal_Bool bLoadDoc = sal_True;

	// get filter
  	uno::Reference< xml::sax::XDocumentHandler > xFilter =
  		new SchXMLImport( mxModel, bLoadDoc, mbShowProgress );

	// connect pipe's output stream to the data source
	uno::Reference< io::XOutputStream > xPipeOutput( xPipe, uno::UNO_QUERY );
	xSource->setOutputStream( xPipeOutput );

	// connect pipe's input stream to the parser
	xml::sax::InputSource aParserInput;
	uno::Reference< io::XInputStream > xPipeInput( xPipe, uno::UNO_QUERY );
	aParserInput.aInputStream = xPipeInput;
	aParserInput.sSystemId = mrMedium.GetName();

	// connect parser and filter
	uno::Reference< xml::sax::XParser > xParser( xXMLParser, uno::UNO_QUERY );
  	xParser->setDocumentHandler( xFilter );

	// parse
	uno::Reference< io::XActiveDataControl > xSourceControl( xSource, uno::UNO_QUERY );
	xSourceControl->start();
	sal_Bool bRetval = sal_True;

	try
	{
		xParser->parseStream( aParserInput );
	}
	catch( xml::sax::SAXParseException )
	{
		bRetval = sal_False;
	}
	catch( xml::sax::SAXException )
	{
		bRetval = sal_False;
	}
	catch( io::IOException )
	{
		bRetval = sal_False;
	}

	return bRetval;
}

sal_Bool SchXMLWrapper::Export()
{
	if( ! mxModel.is())
	{
		DBG_ERROR("Got no Model for XML export");
		return sal_False;
	}
	uno::Reference< lang::XServiceInfo > xServiceInfo( mxModel, uno::UNO_QUERY );
	if( ! xServiceInfo.is() ||
		! xServiceInfo->supportsService( rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.chart.ChartDocument" ))))
	{
		DBG_ERROR( "Model for XML export is no chart::ChartDocument" );
		return sal_False;
	}
	uno::Reference< lang::XMultiServiceFactory > xServiceFactory( utl::getProcessServiceFactory() );
	if( ! xServiceFactory.is())
	{
		DBG_ERROR("got no service manager");
		return sal_False;
	}
	uno::Reference< uno::XInterface > xWriter( xServiceFactory->createInstance(
		rtl::OUString::createFromAscii( "com.sun.star.xml.sax.Writer" )));
	if( ! xWriter.is())
	{
		DBG_ERROR( "com.sun.star.xml.sax.Writer service missing" );
		return sal_False;
	}

	// smart -> uno3 conversion only until SfxMedium has a uno3 DataSink
	uno::Reference< io::XOutputStream > xOut = mrMedium.GetDataSink();
	uno::Reference< io::XActiveDataSource > xSrc( xWriter, uno::UNO_QUERY );
	xSrc->setOutputStream( xOut );

	uno::Reference< xml::sax::XDocumentHandler > xHandler( xWriter, uno::UNO_QUERY );
	rtl::OUString sFileName = mrMedium.GetName();
	SchXMLExport *pExp = new SchXMLExport( mxModel, sFileName, xHandler, mbShowProgress, sal_True );

	// give string descriptor as parameter for doc type
	sal_Bool bRet = ( 0 == pExp->exportDoc( sXML_chart ) );

	delete pExp;

	return bRet;
}

