/*************************************************************************
 *
 *  $RCSfile: dragserv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <sot/exchange.hxx>

#ifndef _SVDOGRAF_HXX //autogen
#include <svx/svdograf.hxx>
#endif
#ifndef _SVDOBJ_HXX //autogen
#include <svx/svdobj.hxx>
#endif
#ifndef _EMBOBJ_HXX //autogen
#include <so3/embobj.hxx>
#endif
#ifndef SO2_DECL_SVEMBEDDEDOBJECT_DEFINED
#define SO2_DECL_SVEMBEDDEDOBJECT_DEFINED
SO2_DECL_REF(SvEmbeddedObject)
#endif
#ifndef _SOT_FORMATS_HXX //autogen
#include <sot/formats.hxx>
#endif
#ifndef _SV_GRAPH_HXX //autogen
#include <vcl/graph.hxx>
#endif
#include "dragserv.hxx"
#include "docshell.hxx"
#include "schview.hxx"
#include "schmod.hxx"
#include "app.hxx"


/*************************************************************************
|*
|* Ctor
|*
\************************************************************************/

SchDragServer::SchDragServer(SchView* pView)
	: SvDataMemberObject(),
	pSchViewIntern(pView),
	pSchView(pView),
	pChartModelIntern(NULL),
	pChartModel(NULL),
	pObjectDescriptor(NULL),
	pVDev(NULL),
	pGraphic(NULL),
	bInternalMove(FALSE),
	bOwnView(FALSE),
	bOwnDocument(FALSE)
{
	CHART_TRACE( "SchDragServer::SchDragServer( SchView* )" );
	if ( pSchView )
	{
		const SdrMarkList&	rMarkList = pSchView->GetMarkList();

		if ( rMarkList.GetMarkCount() == 1 )
		{
			const SdrObject* pObj = rMarkList.GetMark( 0 )->GetObj();

			if ( pObj && pObj->ISA( SdrGrafObj ) )
				pGraphic = new Graphic( ( (SdrGrafObj*) pObj )->GetGraphic() );
		}
	}
}


/*************************************************************************
|*
|* Ctor
|*
\************************************************************************/

SchDragServer::SchDragServer(ChartModel* pModel,
							BOOL bHoleDocumentP )
	: SvDataMemberObject(),
	pSchViewIntern(NULL),
	pSchView(NULL),
	pChartModelIntern(pModel),
	pChartModel(pModel),
	pObjectDescriptor(NULL),
	pVDev(NULL),
	pGraphic(NULL),
	bInternalMove(FALSE),
	bOwnView(FALSE),
	bOwnDocument(TRUE),
	bHoleDocument(bHoleDocumentP)
{

	CHART_TRACE1( "SchDragServer::SchDragServer (ChartModel=%lx)", (long)pModel );

	if ( !bHoleDocument ) //Dokument mit Lchern ? ;)
	{
		const Point		aPos;
		const SdrPage*	pPage = pModel->GetPage( 0 );
		const ULONG		nObjCount = pPage->GetObjCount();

		if ( nObjCount == 1 )
		{
			SdrObject* pObj = pPage->GetObj( 0 );

			if ( pObj->ISA( SdrGrafObj ) )
				pGraphic = new Graphic( ( (SdrGrafObj*) pObj )->GetGraphic() );
		}

		pVDev = new VirtualDevice( *( (OutputDevice*)Application::GetDefaultDevice() ) );	// #67730#
		MapMode aMap = pVDev->GetMapMode();
		aMap.SetMapUnit( pModel->GetScaleUnit() );
		aMap.SetScaleX( pModel->GetScaleFraction() );
		aMap.SetScaleY( pModel->GetScaleFraction() );
		pVDev->SetMapMode( aMap );

		bOwnView = TRUE;
		pSchViewIntern = new SchView( pModel, pVDev );
		pSchViewIntern->SetActualWin ( pVDev );
		pSchViewIntern->SetMarkHdlHidden(TRUE);

		SdrPageView* pPageView = pSchViewIntern->ShowPagePgNum( 0, aPos );
		pSchViewIntern->MarkAll();
	}
}


/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/

SchDragServer::~SchDragServer()
{
	CHART_TRACE( "SchDragServer::~SchDragServer" );

	if ( this == SCH_MOD1()->GetDragData() )
	{
				SCH_MOD1()->SetDragData(NULL);
	}

	if (this == SCH_MOD1()->GetClipboardData() )
	{
				SCH_MOD1()->SetClipboardData(NULL);
	}

	if (bOwnView)
	{
		delete pSchViewIntern;
		delete pVDev;
	}

	if (bOwnDocument)
	{
		delete pChartModelIntern;
	}

	aTypeList.Clear();

	if ( aDocShellRef.Is() )
	{
		SvEmbeddedObject* pObj = aDocShellRef;
		SchChartDocShell* pDocSh = (SchChartDocShell*) pObj;
//		pDocSh->Close();
		pDocSh->DoClose();
	}

	aDocShellRef.Clear();

	delete pObjectDescriptor;

	if ( pGraphic )
		delete pGraphic;
}


/*************************************************************************
|*
|*
|*
\************************************************************************/
void SchDragServer::SetObjectDescriptor(SvObjectDescriptor* pObjDescr)
{
	CHART_TRACE( "SchDragServer::SetObjectDescriptor" );
	pObjectDescriptor = pObjDescr;
}

BOOL SchDragServer::GetData(SvData* pData)
{
	CHART_TRACE2( "SchDragServer::GetData (pModel=%lx) Format=%lu ", (long)pChartModel, pData->GetFormat());

	BOOL bOK = FALSE;

	if (!GetTypeList().Get(*pData))
		return(bOK);

	if ( pChartModel && !pSchViewIntern )
	{
		const SdrPage*	pPage = pChartModel->GetPage( 0 );
		const ULONG		nObjCount = pPage->GetObjCount();

		if ( nObjCount == 1 )
		{
			SdrObject* pObj = pPage->GetObj( 0 );

			if ( pObj->ISA( SdrGrafObj ) )
			{
				if ( pGraphic )
					delete pGraphic;

				pGraphic = new Graphic( ( (SdrGrafObj*) pObj )->GetGraphic() );
			}
		}

		/**********************************************************************
		* Dokument vorhanden, View muss erzeugt werden
		**********************************************************************/
		pVDev = new VirtualDevice( *( (OutputDevice*)Application::GetDefaultDevice() ) );	// #67730#
		MapMode aMap = pVDev->GetMapMode();
		aMap.SetMapUnit(pChartModelIntern->GetScaleUnit());
		aMap.SetScaleX(pChartModelIntern->GetScaleFraction());
		aMap.SetScaleY(pChartModelIntern->GetScaleFraction());
		pVDev->SetMapMode(aMap);

		bOwnView = TRUE;
		pSchViewIntern = new SchView(pChartModelIntern, pVDev);
		pSchViewIntern->SetActualWin (pVDev);
		pSchViewIntern->SetMarkHdlHidden(TRUE);

		Point aPos;
		SdrPageView* pPageView = pSchViewIntern->ShowPagePgNum(0, aPos);
		pSchViewIntern->MarkAll();
	}
	else if (pSchView && !pChartModelIntern)
	{
		/**********************************************************************
		* View vorhanden, Dokument muss erzeugt werden
		**********************************************************************/
		bOwnDocument = TRUE;
		pChartModelIntern = (ChartModel*) pSchView->GetAllMarkedModel();
	}

	ULONG nFormat = pData->GetFormat();

//	if (pObjectDescriptor && (nFormat == pObjectDescriptor->GetLinkFormat() ||
//							  nFormat == pObjectDescriptor->GetFormat()))
	if (pObjectDescriptor && (nFormat == SOT_FORMATSTR_ID_OBJECTDESCRIPTOR ||
							  nFormat == SOT_FORMATSTR_ID_LINKSRCDESCRIPTOR))
	{
		SvMemoryStream aStream;
		aStream << *pObjectDescriptor;
		pData->SetData((void*)aStream.GetData(), aStream.GetSize());
		bOK = TRUE;
	}
	else if (nFormat == FORMAT_GDIMETAFILE)
	{
		GDIMetaFile aMtf(pSchViewIntern->GetAllMarkedMetaFile());
		pData->SetData(&aMtf);
		bOK = TRUE;
	}
	else if (nFormat == FORMAT_BITMAP)
	{
		Bitmap aBmp(pSchViewIntern->GetAllMarkedBitmap());
		pData->SetData(&aBmp);
		bOK = TRUE;
	}
	else if (nFormat == FORMAT_STRING)
	{
		pData->SetData( String() );
		bOK = TRUE;
	}
	else if ( ( nFormat == SOT_FORMATSTR_ID_SVXB ) && pGraphic )
	{
		DBG_ASSERT( pGraphic, "Graphic's missing!" );

		pData->SetData( pGraphic );
		bOK = TRUE;
	}
	else
	{
		if (!aDocShellRef.Is())
		{
			aDocShellRef = new SchChartDocShell(pChartModelIntern,SFX_CREATE_MODE_EMBEDDED);// SFX_CREATE_MODE_INTERNAL);
			bOwnDocument = FALSE;	   // Eigentuemeruebergang
			aDocShellRef->DoInitNew(NULL);
			Rectangle aVisArea = pSchViewIntern->GetAllMarkedRect();
			if( !aVisArea.IsEmpty() )
				aDocShellRef->SetVisArea(aVisArea);
		}

		bOK = aDocShellRef->GetData(pData); //Hier wird SaveAs in der Basis gerufen
	}

	return bOK;
}

/*************************************************************************
|*
|*
|*
\************************************************************************/

BOOL SchDragServer::GetDataHere(SvData* pData)
{
	CHART_TRACE2( "SchDragServer::GetDataHere (pModel=%lx) Format=%lu ", (long)pChartModel, pData->GetFormat());

	BOOL bOK = FALSE;

	ULONG nFormat = pData->GetFormat();

	if (nFormat == FORMAT_GDIMETAFILE ||
		nFormat == FORMAT_BITMAP ||
		nFormat == FORMAT_STRING ||
		nFormat == SOT_FORMATSTR_ID_SVXB )
	{
		bOK = SvDataObject::GetDataHere(pData);
	}
	else
	{
		if (pChartModel && !pSchViewIntern)
		{
			/**********************************************************************
			* Dokument vorhanden, View muss erzeugt werden
			**********************************************************************/
			pVDev = new VirtualDevice((const class OutputDevice &) *Application::GetDefaultDevice());	// #67730#
			MapMode aMap = pVDev->GetMapMode();
			aMap.SetMapUnit(pChartModelIntern->GetScaleUnit());
			aMap.SetScaleX(pChartModelIntern->GetScaleFraction());
			aMap.SetScaleY(pChartModelIntern->GetScaleFraction());
			pVDev->SetMapMode(aMap);

			bOwnView = TRUE;
			pSchViewIntern = new SchView(pChartModelIntern, pVDev);
			pSchViewIntern->SetActualWin (pVDev);
			Point aPos;
			SdrPageView* pPageView = pSchViewIntern->ShowPagePgNum(0, aPos);
			pSchViewIntern->SetMarkHdlHidden(TRUE);
			pSchViewIntern->MarkAll();
		}
		if (!aDocShellRef.Is())
		{
			aDocShellRef = new SchChartDocShell(pChartModelIntern,SFX_CREATE_MODE_EMBEDDED);// SFX_CREATE_MODE_INTERNAL);
			bOwnDocument = FALSE;	   // Eigentuemeruebergang
			aDocShellRef->DoInitNew(NULL);
			Rectangle aVisArea = pSchViewIntern->GetAllMarkedRect();
			if( !aVisArea.IsEmpty() )
				aDocShellRef->SetVisArea(aVisArea);
		}
		bOK = aDocShellRef->GetDataHere(pData);
	}

	return bOK;
}

/*************************************************************************
|*
|*
|*
\************************************************************************/

const SvDataTypeList& SchDragServer::GetTypeList() const
{
	if ( !aTypeList.Count() )
	{
		SchDragServer* pThis = (SchDragServer*) this;

		if( bHoleDocument )
			pThis->aTypeList = SvEmbeddedObject::GetStdFormatList();
		else if ( pGraphic )
		{
			const GraphicType	eType = pGraphic->GetType();

			if ( eType == GRAPHIC_BITMAP )
			{
				if ( pGraphic->IsAnimated() || pGraphic->IsTransparent() )
				{
					const ULONG nGraphicFormat = SOT_FORMATSTR_ID_SVXB;
					pThis->aTypeList.Append( SvDataType( nGraphicFormat, MEDIUM_STREAM | MEDIUM_MEMORY ) );
				}
				else
					pThis->aTypeList.Append(SvDataType(FORMAT_BITMAP));
			}
			else
				pThis->aTypeList.Append(SvDataType(FORMAT_GDIMETAFILE));
		}

		if ( !pThis->aTypeList.HasEqualType( FORMAT_GDIMETAFILE ) )
			pThis->aTypeList.Insert( SvDataType(FORMAT_GDIMETAFILE), LIST_APPEND );

		if ( !pThis->aTypeList.HasEqualType( FORMAT_BITMAP ) )
			pThis->aTypeList.Insert( SvDataType(FORMAT_BITMAP), LIST_APPEND );

//		  SdrTextObj* pTextObj = pSchViewIntern->GetTextObj();
//
//		  if (pTextObj)
//		  {
//			  ((SchDragServer*)this)->aTypeList.Insert(SvDataType(FORMAT_STRING), LIST_APPEND);
//		  }
	}

	return aTypeList;
}



