/*************************************************************************
 *
 *  $RCSfile: schopt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// header for SvStream
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
// header for SFX_INIMANAGER
#ifndef _SFX_INIMGR_HXX
#include <sfx2/inimgr.hxx>
#endif
// header for SAL_STATIC_CAST
#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#include "schopt.hxx"

#include "cfgids.hxx"
#include "defines.hxx"			// for ROW_COLOR_COUNT
#include "schresid.hxx"
#include "strings.hrc"


#define SCH_OPTIONS_VERSION_001		USHORT( 1 )

TYPEINIT1( SchColorTableItem, SfxPoolItem );

// --------------------
// class SchColorTable
// --------------------
SchColorTable::SchColorTable()
{
}

SchColorTable::SchColorTable( const SchColorTable& rOther ) :
		Table( rOther )
{
}

BOOL SchColorTable::Insert( ULONG nKey, XColorEntry* pEntry )
{
	return Table::Insert( nKey, (void*)pEntry );
}

BOOL SchColorTable::RemoveAndDestroy( ULONG nKey )
{
	XColorEntry* pOldEntry = SAL_STATIC_CAST( XColorEntry*, Table::Remove( nKey ) );
	if( pOldEntry )
	{
		delete pOldEntry;
		return TRUE;
	}
	return FALSE;
}

BOOL SchColorTable::ReplaceAndDestroy( ULONG nKey, XColorEntry* pEntry )
{
	XColorEntry* pOldEntry = SAL_STATIC_CAST( XColorEntry*, Table::Replace( nKey, (void*)pEntry ) );
	if( pOldEntry )
	{
		delete pOldEntry;
		return TRUE;
	}
	return FALSE;
}

void SchColorTable::ClearAndDestroy()
{
	for( ULONG i = Count(); i; )
		delete Get( --i );
	Clear();
}

XColorEntry* SchColorTable::Get( ULONG nKey ) const
{
	return SAL_STATIC_CAST( XColorEntry*, Table::Get( nKey ) );
}

Color SchColorTable::GetColor( ULONG nKey ) const
{
	XColorEntry* pEntry = Get( nKey );
	if( pEntry )
		return pEntry->GetColor();

	return COL_BLACK;
}

ColorData SchColorTable::GetColorData( ULONG nKey ) const
{
	return GetColor( nKey ).GetRGBColor();
}

void SchColorTable::UseDefault()
{
	ColorData aColors[ 12 ] = {
		RGB_COLORDATA( 0x99,  0x99, 0xff ),
		RGB_COLORDATA( 0x99,  0x33, 0x66 ),
		RGB_COLORDATA( 0xff,  0xff, 0xcc ),
		RGB_COLORDATA( 0xcc,  0xff, 0xff ),
		RGB_COLORDATA( 0x66,  0x00, 0x66 ),
		RGB_COLORDATA( 0xff,  0x80, 0x80 ),
		RGB_COLORDATA( 0x00,  0x66, 0xcc ),
		RGB_COLORDATA( 0xcc,  0xcc, 0xff ),
		RGB_COLORDATA( 0x00,  0x00, 0x80 ),
		RGB_COLORDATA( 0xff,  0x00, 0xff ),
		RGB_COLORDATA( 0x00,  0xff, 0xff ),
		RGB_COLORDATA( 0xff,  0xff, 0x00 )
	};

	ClearAndDestroy();

	String aResName( SchResId( STR_DIAGRAM_ROW ));
	String aPrefix, aPostfix, aName;
	xub_StrLen nPos = aResName.SearchAscii( "$(ROW)" );
	if( nPos != STRING_NOTFOUND )
	{
		aPrefix = String( aResName, 0, nPos );
		aPostfix = String( aResName, nPos + sizeof( "$(ROW)" ) - 1, STRING_LEN );
	}
	else
		aPrefix = aResName;

	for( sal_Int32 i=0; i<ROW_COLOR_COUNT; i++ )
	{
		aName = aPrefix;
		aName.Append( String::CreateFromInt32( i + 1 ));
		aName.Append( aPostfix );

		Insert( i, new XColorEntry( aColors[ i%12 ], aName ) );
	}
}

// ====================
// class SchOptions
// ====================

SchOptions::SchOptions() :
		SfxConfigItem( SCHCFG_CHART )
{
}

SchOptions::~SchOptions()
{
	aDefColors.ClearAndDestroy();
}

int __EXPORT SchOptions::Load( SvStream& rStream )
{
	USHORT nVersion;
	rStream >> nVersion;
	
	if( nVersion != SCH_OPTIONS_VERSION_001 )
		return SfxConfigItem::WARNING_VERSION;


	// default colors: aDefColors
	aDefColors.ClearAndDestroy();

	ULONG nCount;
	ColorData aData;
	Color aCol;

	rStream >> nCount;

	String aResName( SchResId( STR_DIAGRAM_ROW ));
	String aPrefix, aPostfix, aName;
	xub_StrLen nPos = aResName.SearchAscii( "$(ROW)" );
	if( nPos != STRING_NOTFOUND )
	{
		aPrefix = String( aResName, 0, nPos );
		aPostfix = String( aResName, nPos + sizeof( "$(ROW)" ) - 1, STRING_LEN );
	}
	else
		aPrefix = aResName;

	for( sal_Int32 i=0; i < nCount; i++ )
	{
		rStream >> aData;
		aCol.SetColor( aData );

		aName = aPrefix;
		aName.Append( String::CreateFromInt32( i + 1 ));
		aName.Append( aPostfix );

		aDefColors.Insert( i, new XColorEntry( aCol, aName ) );
	}

	return SfxConfigItem::ERR_OK;;
}

BOOL __EXPORT SchOptions::Store( SvStream& rStream )
{
	USHORT nVersion = SCH_OPTIONS_VERSION_001;

	rStream << nVersion;

	// default colors: aDefColors
	ULONG nCount = aDefColors.Count();

	rStream << nCount;

	for( ULONG i=0; i<nCount; i++ )
	{
		rStream << aDefColors.GetColorData( i );
	}
	
	return SVSTREAM_OK == rStream.GetError();
}

void SchOptions::UseDefault()
{
	aDefColors.UseDefault();
	SfxConfigItem::UseDefault();
}

// --------------------
// class SchColorTableItem
// --------------------

SchColorTableItem::SchColorTableItem( USHORT nWhich, const SchColorTable& aTable ) :
		SfxPoolItem( nWhich ),
		SchColorTable( aTable )
{
}

SchColorTableItem::SchColorTableItem( const SchColorTableItem& rOther ) :
		SfxPoolItem( rOther ),
		SchColorTable( rOther )
{
}

SfxPoolItem* __EXPORT SchColorTableItem::Clone( SfxItemPool *pPool ) const
{
	return new SchColorTableItem( *this );
}

int __EXPORT SchColorTableItem::operator==( const SfxPoolItem& rAttr ) const
{
	DBG_ASSERT( SfxPoolItem::operator==( rAttr ), "SchColorTableItem::operator== : types differ" );

	return SchColorTable::operator==( (const SchColorTable&)(const SchColorTableItem&)rAttr );
}

void __EXPORT SchColorTableItem::SetOptions( SchOptions* pOpts ) const
{
	if( pOpts )
	{
		pOpts->SetDefaultColors( *this );
	}
}


