/*************************************************************************
 *
 *  $RCSfile: ChXChartDocument.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "ChXChartDocument.hxx"
#include "ChXDiagram.hxx"
#include "ChXChartData.hxx"
#include "ChXChartObject.hxx"
#include "ChXChartDataChangeEventListener.hxx"

#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif

#ifndef SCH_UNOPMAP_HXX
#include "mapprov.hxx"
#endif
#ifndef _SCHATTR_HXX
#include "schattr.hxx"
#endif

// for access to table addresses
#include "memchrt.hxx"

// header for SvNumberFormatsSupplierObj
#ifndef _NUMUNO_HXX 
#include <svtools/numuno.hxx>
#endif

// header for class OGuard
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
// header for class Application
#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif
#ifndef _SVX_UNOSHAPE_HXX
#include <svx/unoshape.hxx>
#endif

#ifndef _COM_SUN_STAR_CHART_CHARTSERIESADDRESS_HPP_
#include <com/sun/star/chart/ChartSeriesAddress.hpp>
#endif

extern SchUnoPropertyMapProvider aSchMapProvider;

using namespace vos;
using namespace com::sun::star;

ChXChartDocument::ChXChartDocument( SchChartDocShell* pShell ) :
		SfxBaseModel( pShell ),
		m_pDocShell( pShell ),
		m_pModel( NULL ),
		m_aPropSet( aSchMapProvider.GetMap( CHMAP_DOC, NULL ))
{
	if( m_pDocShell )
	{
		m_pModel = m_pDocShell->GetModelPtr();
		m_aPropSet = SvxItemPropertySet( aSchMapProvider.GetMap( CHMAP_DOC, m_pModel ));
		m_xChartData = (chart::XChartDataArray *)(new ChXChartDataArray( m_pDocShell ));	// XChartData is inherited twice
	}
	else
		DBG_WARNING( "Parameter pShell is NULL." );
}

ChXChartDocument::~ChXChartDocument()
{
	OGuard aGuard( Application::GetSolarMutex());

   if( m_rXDiagram.is() )
   {
	   m_pDiagram->SetDocShell( NULL );
	   m_rXDiagram = NULL;
   }
}

// private methods
uno::Reference< drawing::XShape > ChXChartDocument::getShapeById( long nId )
{
	OGuard aGuard( Application::GetSolarMutex());

	if( m_pModel )
	{
		return SAL_STATIC_CAST( drawing::XShape*, m_pModel->GetShape( nId ));
	}
	else
		return uno::Reference< drawing::XShape >();
}

uno::Reference< beans::XPropertySet > ChXChartDocument::getObjectByMapAndId( const long nMapId, const long nObjId )
{
	OGuard aGuard( Application::GetSolarMutex());

	ChXChartObject* xSet = new ChXChartObject( nMapId, m_pModel, nObjId );
	return SAL_STATIC_CAST( beans::XPropertySet*, xSet );
}

// public methods
void ChXChartDocument::setDiagramType( const ::rtl::OUString& aType ) throw()
{
	OGuard aGuard( Application::GetSolarMutex());

	// never show sorting dialog
	m_pModel->SetChartStatusFlag( CHS_USER_NOQUERY );			

	// all chart types start with the prefix com.sun.star.chart
	if( 0 == aType.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_NAMESPACE )))
	{
		// get postfix using copy( beginIndex )
		rtl::OUString aPostfix = aType.copy( RTL_CONSTASCII_LENGTH( SCH_X_STR_CHTYPE_NAMESPACE ));

		if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_LINE )) == 0 )
			m_pModel->SetBaseType( CHTYPE_LINE );
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_AREA )) == 0 )
			m_pModel->SetBaseType( CHTYPE_AREA );
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_PIE )) == 0 )
			m_pModel->SetBaseType( CHTYPE_CIRCLE);
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_BAR )) == 0 )
			m_pModel->SetBaseType( CHTYPE_BAR);
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_XY )) == 0 )
			m_pModel->SetBaseType( CHTYPE_XY );
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_NET )) == 0 )
			m_pModel->SetBaseType( CHTYPE_NET );
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_DONUT )) == 0 )
			m_pModel->SetBaseType( CHTYPE_DONUT );
		else if( aPostfix.compareToAscii( RTL_CONSTASCII_STRINGPARAM( SCH_X_STR_CHTYPE_STOCK )) == 0 )
			m_pModel->SetBaseType( CHTYPE_STOCK );
	}
}

::rtl::OUString ChXChartDocument::getDiagramType() const throw()
{
	if( m_rXDiagram.is() )
		return m_pDiagram->getDiagramType();

	return ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "UnknownChartType" ));
}

void ChXChartDocument::SetChartModel( ChartModel* pModel ) throw()
{
	OGuard aGuard( Application::GetSolarMutex());
	m_pModel = pModel;
}

void ChXChartDocument::RefreshData( const chart::ChartDataChangeEvent& aEvent ) throw()
{
	OGuard aGuard( Application::GetSolarMutex());

	// copy new data ( in member m_xChartData )
	if( m_xChartData.is() )
	{
		// get XChartDataArray representation of m_xChartData
		uno::Reference< chart::XChartDataArray > rDataArray;
		uno::Any aAny = m_xChartData->queryInterface( ::getCppuType((const uno::Reference< chart::XChartDataArray >*)0));
		aAny >>= rDataArray;

		if( rDataArray.is() )
		{
			// new data object using own doc shell
			ChXChartDataArray* pNewDataArray = new ChXChartDataArray( m_pDocShell );
			chart::XChartDataArray* pSrcArray = rDataArray.get();

			// the get-methods re-retrieve the (changed) data from the SchMemChart
			pNewDataArray->setData( pSrcArray->getData() );
			pNewDataArray->setRowDescriptions( pSrcArray->getRowDescriptions() );
			pNewDataArray->setColumnDescriptions( pSrcArray->getColumnDescriptions() );

			// remember the new data as member
			m_xChartData = SAL_STATIC_CAST( ChXChartData*, pNewDataArray );
		}
	}
}


// XInterface
uno::Any SAL_CALL ChXChartDocument::queryInterface( const uno::Type & rType )
	throw( uno::RuntimeException )
{
	uno::Any aAny = SfxBaseModel::queryInterface( rType );
	if( aAny.hasValue() )
		return aAny;

	return ::cppu::queryInterface(
		rType,
		( lang::XMultiServiceFactory* )this,
		( beans::XPropertySet* )this,
		( chart::XChartDocument* )this,
		( lang::XServiceInfo* )this,
		( util::XNumberFormatsSupplier* )this );
}

void SAL_CALL ChXChartDocument::acquire() throw()
{
	// #71736# get a reference to DocShell, so that it cannot be
	// disposed until no external references are held to this object
	OGuard aGuard( Application::GetSolarMutex());

	if( m_refCount == 1 && ! IsDisposed() )
	{
		m_rDocShell = m_pDocShell;
	}

	SfxBaseModel::acquire();
}

void SAL_CALL ChXChartDocument::release() throw()
{
	// #71736# last reference apart from the one in SchDocShell
	OGuard aGuard( Application::GetSolarMutex());

	if( m_refCount == 2 )
	{
		m_rDocShell = NULL;			// release Reference to DocShell => DocShell may be disposed now
	}

	SfxBaseModel::release();
}


// XMultiServiceFactory
uno::Reference< uno::XInterface > SAL_CALL ChXChartDocument::createInstance( const ::rtl::OUString& aServiceSpecifier )
	throw( uno::Exception, uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	ChXDiagram* pDiagram = new ChXDiagram( m_pDocShell, FALSE );
	pDiagram->SetServiceName( aServiceSpecifier );

	return uno::Reference< uno::XInterface > ( SAL_STATIC_CAST( cppu::OWeakObject*, pDiagram ));
}

uno::Reference< uno::XInterface > SAL_CALL ChXChartDocument::createInstanceWithArguments(
	const ::rtl::OUString& ServiceSpecifier, const uno::Sequence< uno::Any >& Arguments )
	throw( uno::Exception, uno::RuntimeException )
{
	if( Arguments.getLength() )
		throw lang::IllegalArgumentException();
	else
		return createInstance( ServiceSpecifier );
}

uno::Sequence< ::rtl::OUString > SAL_CALL ChXChartDocument::getAvailableServiceNames()
	throw( uno::RuntimeException )
{
	uno::Sequence< ::rtl::OUString >* pSequence = new uno::Sequence< ::rtl::OUString >( 8 );
	::rtl::OUString* pString = pSequence->getArray();

	pString[ 0 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_LINE );
	pString[ 1 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_AREA );
	pString[ 2 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_BAR );
	pString[ 3 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_PIE );
	pString[ 4 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_XY );
	pString[ 5 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_NET );
	pString[ 6 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_DONUT );
	pString[ 7 ].createFromAscii( SCH_X_STR_CHTYPE_NAMESPACE SCH_X_STR_CHTYPE_STOCK );

	return *pSequence;
}


// XPropertySet
uno::Reference< beans::XPropertySetInfo > SAL_CALL ChXChartDocument::getPropertySetInfo()
	throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	return m_aPropSet.getPropertySetInfo();
}

void SAL_CALL ChXChartDocument::setPropertyValue( const ::rtl::OUString& aPropertyName, const uno::Any& aValue )
	throw( beans::UnknownPropertyException,
		   beans::PropertyVetoException,
		   lang::IllegalArgumentException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	if( m_pModel )
	{
		const SfxItemPropertyMap* pMap = m_aPropSet.getPropertyMapEntry( aPropertyName );
		if( pMap && pMap->nWID )
		{
			long nWID = pMap->nWID;

			switch( nWID )
			{
				case CHATTR_DIAGRAM_START:		// dummy id for "HasLegend"
					{
						sal_Bool bVal;
						aValue >>= bVal;
						m_pModel->SetShowLegend( bVal );
						m_pModel->SetLegendHasBeenMoved( FALSE );
						m_pModel->BuildChart( FALSE );
					}
					break;

				case CHATTR_ADDR_CATEGORIES:
					{
						SchMemChart* pData = m_pModel->GetChartData();
						if( pData )
						{
							rtl::OUString aCatAddr;
							aValue >>= aCatAddr;
							pData->SetCategoriesRangeAddress( aCatAddr );
						}
						else
							DBG_ERROR( "invalid SchMemChart" );
					}
					break;

				case CHATTR_ADDR_SERIES:
					{
						SchMemChart* pData = m_pModel->GetChartData();
						if( pData )
						{
							uno::Sequence< chart::ChartSeriesAddress > aSeriesAddresses;
							aValue >>= aSeriesAddresses;
							pData->SetSeriesAddresses( aSeriesAddresses );
						}
						else
							DBG_ERROR( "invalid SchMemChart" );
					}
					break;

				default:
					{
						SfxItemSet aSet( m_pModel->GetPool(), nWID, nWID );
						m_pModel->GetAttr( aSet );

						if( ! aSet.Count() )
						{
							// get default from ItemPool
							if( m_pModel->GetItemPool().IsWhich( nWID ) )
							{
								aSet.Put( m_pModel->GetItemPool().GetDefaultItem( pMap->nWID ) );
							}
							else
							{
#ifdef DBG_UTIL
								String aTmpString( aPropertyName );
								ByteString aProp( aTmpString, RTL_TEXTENCODING_ASCII_US );
								DBG_ERROR2( "Diagram: Property %s has an invalid ID (%d)", aProp.GetBuffer(), nWID );
#endif
							}
						}
						if( aSet.Count())
						{
							m_aPropSet.setPropertyValue( pMap, aValue, aSet );
							m_pModel->PutAttr( aSet );
							m_pModel->BuildChart( FALSE );
						}
						else
						{
							throw beans::UnknownPropertyException();
						}
					}
					break;
			}
		}
		else
		{
			throw beans::UnknownPropertyException();
		}
	}
	else
		DBG_WARNING( "No Model" );
}

uno::Any SAL_CALL ChXChartDocument::getPropertyValue( const ::rtl::OUString& aPropertyName )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	uno::Any aAny;
	if( m_pModel )
	{
		const SfxItemPropertyMap* pMap = m_aPropSet.getPropertyMapEntry( aPropertyName );
		if( pMap && pMap->nWID )
		{
			long nWID = pMap->nWID;

			switch( nWID )
			{
				case CHATTR_DIAGRAM_START:		// dummy id for "HasLegend"
					{
						SfxItemSet rSet( m_pModel->GetFullLegendAttr() );
						const SfxPoolItem *pPoolItem = NULL;
						SvxChartLegendPos ePos = CHLEGEND_RIGHT;
						if( rSet.GetItemState( SCHATTR_LEGEND_POS, TRUE, &pPoolItem ) == SFX_ITEM_SET )
						{
							ePos = ((const SvxChartLegendPosItem*)pPoolItem)->GetValue();
						}
						aAny <<= sal_Bool( ePos != CHLEGEND_NONE );
					}
					break;

				case CHATTR_ADDR_CATEGORIES:
					{
						SchMemChart* pData = m_pModel->GetChartData();
						if( pData )
						{
							aAny <<= pData->GetCategoriesRangeAddress();
						}
						else
							DBG_ERROR( "invalid SchMemChart" );
					}
					break;

				case CHATTR_ADDR_SERIES:
					{
						SchMemChart* pData = m_pModel->GetChartData();
						if( pData )
						{
							aAny <<= pData->GetSeriesAddresses();
						}
						else
							DBG_ERROR( "invalid SchMemChart" );
					}
					break;

				default:
					{
						SfxItemSet aSet( m_pModel->GetPool(), nWID, nWID );
						m_pModel->GetAttr( aSet );

						if( ! aSet.Count() )
						{
							// get default from ItemPool
							if( m_pModel->GetItemPool().IsWhich( nWID ) )
							{
								aSet.Put( m_pModel->GetItemPool().GetDefaultItem( pMap->nWID ) );
							}
							else
							{
#ifdef DBG_UTIL
								String aTmpString( aPropertyName );
								ByteString aProp( aTmpString, RTL_TEXTENCODING_ASCII_US );
								DBG_ERROR2( "Diagram: Property %s has an invalid ID (%d)", aProp.GetBuffer(), nWID );
#endif
							}
						}
						if( aSet.Count() )
							aAny = m_aPropSet.getPropertyValue( pMap, aSet );
						else
							throw beans::UnknownPropertyException();
					}
			}
		}
		else
			throw beans::UnknownPropertyException();
	}
	else
		DBG_WARNING( "No Model" );

	return aAny;
}

void SAL_CALL ChXChartDocument::addPropertyChangeListener( const ::rtl::OUString& aPropertyName,
														   const uno::Reference< beans::XPropertyChangeListener >& xListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXChartDocument::removePropertyChangeListener( const ::rtl::OUString& aPropertyName,
															  const uno::Reference< beans::XPropertyChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXChartDocument::addVetoableChangeListener( const ::rtl::OUString& PropertyName,
														   const uno::Reference< beans::XVetoableChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXChartDocument::removeVetoableChangeListener( const ::rtl::OUString& PropertyName,
															  const uno::Reference< beans::XVetoableChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

// XChartDocument
uno::Reference< drawing::XShape > SAL_CALL ChXChartDocument::getTitle() throw( uno::RuntimeException )
{
	return getShapeById( CHOBJID_TITLE_MAIN );
}

uno::Reference< drawing::XShape > SAL_CALL ChXChartDocument::getSubTitle() throw( uno::RuntimeException )
{
	return getShapeById( CHOBJID_TITLE_SUB );
}

uno::Reference< drawing::XShape > SAL_CALL ChXChartDocument::getLegend() throw( uno::RuntimeException )
{
	return getShapeById( CHOBJID_LEGEND );
}

uno::Reference< beans::XPropertySet > SAL_CALL ChXChartDocument::getArea() throw( uno::RuntimeException )
{
	uno::Reference< beans::XPropertySet > xProp( getShapeById( CHOBJID_DIAGRAM_AREA ), uno::UNO_QUERY );
	return xProp;
}

uno::Reference< chart::XDiagram > SAL_CALL ChXChartDocument::getDiagram() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	if( ! m_rXDiagram.is() )
	{
		m_pDiagram = new ChXDiagram( m_pDocShell );
		m_rXDiagram = m_pDiagram;	 // keep reference to implementation
	}

	return m_rXDiagram;
}

void SAL_CALL ChXChartDocument::setDiagram( const uno::Reference< chart::XDiagram >& _NewDiagram )
	throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	
	if( m_rXDiagram.is() )
	{
		m_pDiagram->SetDocShell( NULL );
		m_rXDiagram = NULL;
		m_pDiagram = NULL;
	}

	if( _NewDiagram.is() )
	{
		m_pDiagram = ChXDiagram::getImplementation( _NewDiagram );
		if( m_pDiagram )
		{
			if( m_pDocShell )
			{
				if( m_pDiagram->SetDocShell( m_pDocShell, sal_True ))		// keep model (is copied)
				{
					// update local model
					m_pModel = m_pDocShell->GetModelPtr();
				}
				else
				{
					// model was invalid
					// so set correct chart type to current model
					setDiagramType( m_pDiagram->getServiceName());
				}
			}
		}
		m_rXDiagram = _NewDiagram;	 // keep reference to implementation
	}
}

uno::Reference< chart::XChartData > SAL_CALL ChXChartDocument::getData() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	return m_xChartData;
}

void SAL_CALL ChXChartDocument::attachData( const uno::Reference< chart::XChartData >& _Data )
	throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	if( ! _Data.is() )
		return;

	if( ! m_xEventListener.is() )
	{
		ChXChartDataChangeEventListener *pEvtListener = new ChXChartDataChangeEventListener;
		if( pEvtListener )
		{
			pEvtListener->Reset();				// Reset not implemented yet
			pEvtListener->SetOwner( this ); 	// become owner of event listener
			m_xEventListener = pEvtListener;	// remember Listener as member
		}
	}
	
	// add listener to XChartDataArray
	uno::Any aAny = _Data->queryInterface( ::getCppuType((const uno::Reference< chart::XChartDataArray >*)0));
	uno::Reference< chart::XChartDataArray > rChDataArray;
	aAny >>= rChDataArray;

	if( rChDataArray.is() )
		rChDataArray->addChartDataChangeEventListener( m_xEventListener );

	// copy new Data
	m_xChartData = _Data;

	chart::ChartDataChangeEvent aEvent;

	aEvent.Type			 = chart::ChartDataChangeType_ALL;
	aEvent.StartColumn	 = 0;
	aEvent.EndColumn	 = 0;
	aEvent.StartRow		 = 0;
	aEvent.EndRow		 = 0;

	RefreshData( aEvent ); // set initial data
}


// XModel ( ::SfxBaseModel )
sal_Bool SAL_CALL ChXChartDocument::attachResource( const ::rtl::OUString& aURL,
													const uno::Sequence< beans::PropertyValue >& aArgs )
	throw( uno::RuntimeException )
{
	return SfxBaseModel::attachResource( aURL, aArgs );
}

::rtl::OUString SAL_CALL ChXChartDocument::getURL() throw( uno::RuntimeException )
{
	return SfxBaseModel::getURL();
}

uno::Sequence< beans::PropertyValue > SAL_CALL ChXChartDocument::getArgs() throw( uno::RuntimeException )
{
	return SfxBaseModel::getArgs();
}

void SAL_CALL ChXChartDocument::connectController( const uno::Reference< frame::XController >& xController )
	throw( uno::RuntimeException )
{
	SfxBaseModel::connectController( xController );
}

void SAL_CALL ChXChartDocument::disconnectController( const uno::Reference< frame::XController >& xController )
	throw( uno::RuntimeException )
{
	SfxBaseModel::disconnectController( xController );
}

uno::Reference< frame::XController > SAL_CALL ChXChartDocument::getCurrentController() throw( uno::RuntimeException )
{
	return SfxBaseModel::getCurrentController();
}

void SAL_CALL ChXChartDocument::setCurrentController( const uno::Reference< frame::XController >& xController )
	throw( container::NoSuchElementException, uno::RuntimeException )
{
	SfxBaseModel::setCurrentController( xController );
}

uno::Reference< uno::XInterface > SAL_CALL ChXChartDocument::getCurrentSelection() throw( uno::RuntimeException )
{
	return SfxBaseModel::getCurrentSelection();
}

void SAL_CALL ChXChartDocument::lockControllers() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	if( m_pModel )
		m_pModel->LockBuild();
	else
		throw uno::RuntimeException();
}

void SAL_CALL ChXChartDocument::unlockControllers() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	if( m_pModel )
		m_pModel->UnlockBuild();
	else
		throw uno::RuntimeException();
}

sal_Bool SAL_CALL ChXChartDocument::hasControllersLocked() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	if( m_pModel )
		return m_pModel->IsLockedBuild();
	else
		return sal_False;
}

// XTypeProvider ( ::SfxBaseModel )
uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL ChXChartDocument::getTypes()
	throw( uno::RuntimeException )
{
	if( maTypeSequence.getLength() == 0 )
	{
		const uno::Sequence< uno::Type > aBaseTypes( SfxBaseModel::getTypes() );
		const sal_Int32 nBaseTypes = aBaseTypes.getLength();
		const uno::Type* pBaseTypes = aBaseTypes.getConstArray();

		maTypeSequence.realloc( nBaseTypes + 5 );		// Note: Keep the size updated !!
		uno::Type* pTypes = maTypeSequence.getArray();

		*pTypes++ = ::getCppuType((const uno::Reference< lang::XMultiServiceFactory >*)0);
		*pTypes++ = ::getCppuType((const uno::Reference< beans::XPropertySet >*)0);
		*pTypes++ = ::getCppuType((const uno::Reference< chart::XChartDocument >*)0);
		*pTypes++ = ::getCppuType((const uno::Reference< lang::XServiceInfo >*)0);
		*pTypes++ = ::getCppuType((const uno::Reference< util::XNumberFormatsSupplier >*)0);

		for( sal_Int32 nType = 0; nType < nBaseTypes; nType++ )
			*pTypes++ = *pBaseTypes++;
	}

	return maTypeSequence;
}

uno::Sequence< sal_Int8 > SAL_CALL ChXChartDocument::getImplementationId()
	throw( uno::RuntimeException )
{
	static uno::Sequence< sal_Int8 > aId;
	if( aId.getLength() == 0 )
	{
		aId.realloc( 16 );
		rtl_createUuid( (sal_uInt8 *)aId.getArray(), 0, sal_True );
	}
	return aId;
}

// XComponent ( ::XModel ::SfxModel )
void SAL_CALL ChXChartDocument::dispose() throw( uno::RuntimeException )
{
/* CHECKME: How to fix this bug?
	OGuard aGuard( Application::GetSolarMutex());

	m_rDocShell = NULL;
	m_pDocShell = NULL;
	m_rXDiagram = NULL;
	m_pDiagram	= NULL;
	m_pModel	= NULL;

	SfxBaseModel::dispose();
*/
}

void SAL_CALL ChXChartDocument::addEventListener( const uno::Reference< lang::XEventListener >& xListener )
	throw( uno::RuntimeException )
{
	SfxBaseModel::addEventListener( xListener );
}

void SAL_CALL ChXChartDocument::removeEventListener( const uno::Reference< lang::XEventListener >& aListener )
	throw( uno::RuntimeException )
{
	SfxBaseModel::removeEventListener( aListener );
}

// XNumberFormatsSupplier
void ChXChartDocument::InitNumberFormatter() throw( uno::RuntimeException )
{
	if( ! rNumberFormatter.is())
	{
		if( m_pModel )
			rNumberFormatter = new SvNumberFormatsSupplierObj( m_pModel->GetNumFormatter() );
		else
			rNumberFormatter = new SvNumberFormatsSupplierObj();
	}

	if( ! rNumberFormatter.is())
		throw uno::RuntimeException();
}

uno::Reference< beans::XPropertySet > SAL_CALL ChXChartDocument::getNumberFormatSettings() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );
	if( ! rNumberFormatter.is())
		InitNumberFormatter();

	return rNumberFormatter.get()->getNumberFormatSettings();
}

uno::Reference< util::XNumberFormats > SAL_CALL ChXChartDocument::getNumberFormats() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );
	if( ! rNumberFormatter.is())
		InitNumberFormatter();

	return rNumberFormatter.get()->getNumberFormats();
}

// XServiceInfo
::rtl::OUString SAL_CALL ChXChartDocument::getImplementationName()
	throw( uno::RuntimeException )
{
	return SCH_ASCII_TO_OU( "ChXChartDocument" );
}

sal_Bool SAL_CALL ChXChartDocument::supportsService( const ::rtl::OUString& ServiceName )
	throw( uno::RuntimeException )
{
	return SvxServiceInfoHelper::supportsService( ServiceName, getSupportedServiceNames() );
}

uno::Sequence< rtl::OUString > SAL_CALL ChXChartDocument::getSupportedServiceNames()
	throw( uno::RuntimeException )
{
	uno::Sequence< ::rtl::OUString > aSeq( 2 );
	::rtl::OUString* pStr = aSeq.getArray();
	pStr[ 0 ] = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.chart.ChartDocument" ));
	pStr[ 1 ] = ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.chart.ChartTableAddressSupplier" ));

	return aSeq;
}

