/*************************************************************************
 *
 *  $RCSfile: ChXChartObject.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "ChXChartObject.hxx"
#include "mapprov.hxx"
#include "schattr.hxx"

// header for class OGuard
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
// header for class Application
#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

// header for SvxDoubleItem
#ifndef _SVX_CHRTITEM_HXX
#define ITEMID_DOUBLE 0
#define ITEMID_CHARTTEXTORDER SCHATTR_TEXT_ORDER
#include <svx/chrtitem.hxx>
#endif
// header for SvxUnoTextRangeBase
#ifndef _SVX_UNOTEXT_HXX
#include <svx/unotext.hxx>
#endif
// for OWN_ATTR_...
#ifndef _SVX_UNOSHPRP_HXX
#include <svx/unoshprp.hxx>
#endif
// for SID_ATTR_...
#ifndef _SVX_SVXIDS_HRC
#include <svx/svxids.hrc>
#endif

#ifndef _RTL_UUID_H_
#include <rtl/uuid.h>
#endif
#ifndef _RTL_MEMORY_H_
#include <rtl/memory.h>
#endif

#ifndef _COM_SUN_STAR_CHART_CHARTAXISARRANGEORDERTYPE_HPP_
#include <com/sun/star/chart/ChartAxisArrangeOrderType.hpp>
#endif

#include "mapprov.hxx"
#include "chtmodel.hxx"
#include "chaxis.hxx"
#include "globfunc.hxx"			// for GlobalGenerate3DAttrDefaultItem

extern SchUnoPropertyMapProvider aSchMapProvider;

using namespace vos;
using namespace com::sun::star;

ChXChartObject::ChXChartObject( long _MapId, ChartModel* _Model, long _WhichId, long _Index ) :
	maPropSet( aSchMapProvider.GetMap( _MapId, _Model )),
	maSfxPropSet( aSchMapProvider.GetMap( _MapId, _Model )),
	mpModel( _Model ),
	mnWhichId( _WhichId? _WhichId: CHOBJID_ANY ),
	mnIndex( _Index )
{}

ChXChartObject::~ChXChartObject()
{}

const long ChXChartObject::GetId() const
{
	return mnWhichId;
}

// generate a uniqueId
const uno::Sequence< sal_Int8 > & ChXChartObject::getUnoTunnelId() throw()
{
	static uno::Sequence< sal_Int8 > * pSeq = 0;
	if( !pSeq )
	{
		osl::Guard< osl::Mutex > aGuard( osl::Mutex::getGlobalMutex() );
		if( !pSeq )
		{
			static uno::Sequence< sal_Int8 > aSeq( 16 );
			rtl_createUuid( (sal_uInt8*)aSeq.getArray(), 0, sal_True );
			pSeq = &aSeq;
		}
	}
	return *pSeq;
}

ChXChartObject* ChXChartObject::getImplementation( uno::Reference< uno::XInterface > xData ) throw()
{
	uno::Reference< lang::XUnoTunnel > xUT( xData, uno::UNO_QUERY );
	if( xUT.is() )
		return (ChXChartObject*)xUT->getSomething( ChXChartObject::getUnoTunnelId() );
	else
		return NULL;
}

::rtl::OUString SAL_CALL ChXChartObject::getShapeType()
	throw( ::com::sun::star::uno::RuntimeException )
{
	switch( mnWhichId )
	{
		case CHOBJID_DIAGRAM_AREA:
		case CHOBJID_DIAGRAM_WALL:
		case CHOBJID_DIAGRAM_FLOOR:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartArea" );
		case CHOBJID_DIAGRAM_X_GRID_MAIN:
		case CHOBJID_DIAGRAM_Y_GRID_MAIN:
		case CHOBJID_DIAGRAM_Z_GRID_MAIN:
		case CHOBJID_DIAGRAM_X_GRID_HELP:
		case CHOBJID_DIAGRAM_Y_GRID_HELP:
		case CHOBJID_DIAGRAM_Z_GRID_HELP:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartGrid" );
		case CHOBJID_LINE:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartLine" );
		case CHOBJID_LEGEND:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartLegend" );
		case CHOBJID_DIAGRAM_TITLE_X_AXIS:
		case CHOBJID_DIAGRAM_TITLE_Y_AXIS:
		case CHOBJID_DIAGRAM_TITLE_Z_AXIS:
		case CHOBJID_TITLE_MAIN:
		case CHOBJID_TITLE_SUB:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartTitle" );
		case CHOBJID_DIAGRAM_X_AXIS:
		case CHOBJID_DIAGRAM_Y_AXIS:
		case CHOBJID_DIAGRAM_Z_AXIS:
		case CHOBJID_DIAGRAM_A_AXIS:
		case CHOBJID_DIAGRAM_B_AXIS:
		case CHOBJID_DIAGRAM_C_AXIS:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartAxis" );
		default:
			return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartObject" );
	}
}

// XPropertySet
uno::Reference< beans::XPropertySetInfo > SAL_CALL ChXChartObject::getPropertySetInfo() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	return maPropSet.getPropertySetInfo();
}

void SAL_CALL ChXChartObject::setPropertyValue( const ::rtl::OUString& aPropertyName, const uno::Any& aValue )
	throw( beans::UnknownPropertyException,
		   beans::PropertyVetoException,
		   lang::IllegalArgumentException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	if( mpModel && mnWhichId != CHOBJID_ANY )
	{
		const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( aPropertyName );

		if( pMap && pMap->nWID )
		{
			long nWID = pMap->nWID;

			SfxItemSet* pSet = new SfxItemSet( mpModel->GetItemPool(), nWID, nWID );
			mpModel->GetAttr( mnWhichId, *pSet, mnIndex );

			if(!pSet->Count())
			{
				// get default from ItemPool
				if( mpModel->GetItemPool().IsWhich( nWID ))
				{
					pSet->Put( mpModel->GetItemPool().GetDefaultItem( nWID ));
				}
				else
				{
#ifdef DBG_UTIL
					String aTmpString( aPropertyName );
					ByteString aProp( aTmpString, RTL_TEXTENCODING_ASCII_US );
					DBG_ERROR2( "Diagram: Property %s has an invalid ID (%d)", aProp.GetBuffer(), nWID );
#endif
				}
			}
			// map enum
			if( nWID == SCHATTR_TEXT_ORDER )
			{
				SvxChartTextOrder eTextOrder;
				chart::ChartAxisArrangeOrderType eArrOrder;
				aValue >>= eArrOrder;
				switch( eArrOrder )
				{
					case chart::ChartAxisArrangeOrderType_SIDE_BY_SIDE:
						eTextOrder = CHTXTORDER_SIDEBYSIDE; break;
					case chart::ChartAxisArrangeOrderType_STAGGER_EVEN:
						eTextOrder = CHTXTORDER_DOWNUP; break;
					case chart::ChartAxisArrangeOrderType_STAGGER_ODD:
						eTextOrder = CHTXTORDER_UPDOWN; break;
					case chart::ChartAxisArrangeOrderType_AUTO:
					default:
						eTextOrder = CHTXTORDER_AUTO; break;
				}
				pSet->Put( SvxChartTextOrderItem( eTextOrder ));
			}
			// manual step must be in correct range
			else if( ( nWID == SCHATTR_AXIS_AUTO_STEP_MAIN || nWID == SCHATTR_AXIS_AUTO_STEP_HELP ))
			{
				sal_Bool bVal;
				aValue >>= bVal;
				if( ! bVal )
				{
					long nStepWID = (nWID == SCHATTR_AXIS_AUTO_STEP_MAIN)?
						SCHATTR_AXIS_STEP_MAIN:
					SCHATTR_AXIS_STEP_HELP;
					SfxItemSet* pNewSet = new SfxItemSet( mpModel->GetItemPool(), nWID, nWID,
														  nStepWID, nStepWID, 0 );
					mpModel->GetAttr( mnWhichId, *pNewSet, mnIndex );
					pNewSet->Put( *pSet );
					delete pSet;
					pSet = pNewSet;

					BOOL bLog = FALSE;
					ChartAxis* pAxis = mpModel->GetAxis( mnWhichId );
					if( pAxis )
						bLog = pAxis->IsLogarithm();
					const SfxPoolItem* pPoolItem = NULL;
					if( pSet->GetItemState( nStepWID, TRUE, &pPoolItem ) == SFX_ITEM_SET )
					{
						double fOld = ((const SvxDoubleItem*)pPoolItem)->GetValue();
						if( fOld <= (bLog? 1.0: 0.0) )
							pSet->Put( SvxDoubleItem( bLog? 10.0 : 1.0, nStepWID ) );
					}
				}
			}
			else if( nWID == SCHATTR_AXIS_LOGARITHM	)
			{
				sal_Bool bVal;
				aValue >>= bVal;
				if( bVal )
				{
					SfxItemSet* pNewSet = new SfxItemSet( mpModel->GetItemPool(), nWID, nWID,
														  SCHATTR_AXIS_AUTO_STEP_MAIN, SCHATTR_AXIS_AUTO_STEP_MAIN,
														  SCHATTR_AXIS_MIN, SCHATTR_AXIS_MIN,
														  SCHATTR_AXIS_MAX, SCHATTR_AXIS_MAX, 0 );
					mpModel->GetAttr( mnWhichId, *pNewSet, mnIndex );
					pNewSet->Put( *pSet );
					delete pSet;
					pSet = pNewSet;

					// for logarithmic scale always use automatic steps
					pSet->Put( SfxBoolItem( SCHATTR_AXIS_AUTO_STEP_MAIN, TRUE ) );

					const SfxPoolItem* pPoolItem = NULL;
					double fMin = 0.0;
					if( pSet->GetItemState( SCHATTR_AXIS_MIN, TRUE, &pPoolItem ) == SFX_ITEM_SET )
					{
						fMin = ((const SvxDoubleItem*)pPoolItem)->GetValue();
						if( fMin <= 0.0 )
							pSet->Put( SvxDoubleItem( 1.0, SCHATTR_AXIS_MIN ) );
					}
					if( pSet->GetItemState( SCHATTR_AXIS_MAX, TRUE, &pPoolItem ) == SFX_ITEM_SET )
					{
						if( ((const SvxDoubleItem*)pPoolItem)->GetValue() <= fMin )
							pSet->Put( SvxDoubleItem( fMin * 10.0, SCHATTR_AXIS_MAX ));
					}
				}
			}
			else if( nWID == SCHATTR_AXIS_STEP_MAIN  || nWID == SCHATTR_AXIS_STEP_HELP )
			{
				BOOL bLog = FALSE;
				ChartAxis* pAxis = mpModel->GetAxis( mnWhichId );
				if( pAxis )
					bLog = pAxis->IsLogarithm();
				double fVal;
				aValue >>= fVal;
				if( fVal <= bLog? 1.0: 0.0 )
					throw lang::IllegalArgumentException();
			}
			else if( nWID == SCHATTR_AXIS_MIN || nWID == SCHATTR_AXIS_MAX )
			{
				ChartAxis* pAxis = mpModel->GetAxis( mnWhichId );
				double fVal;
				aValue >>= fVal;
				if( pAxis &&
					pAxis->IsLogarithm() && fVal <= 0.0 )
					throw lang::IllegalArgumentException();
 			}

			if( pSet->Count() )
			{
				// CL: convert special character properties
				if( !SvxUnoTextRangeBase::SetPropertyValueHelper( *pSet, pMap, aValue, *pSet ))
					maPropSet.setPropertyValue( pMap, aValue, *pSet );

				mpModel->ChangeAttr( *pSet, mnWhichId, mnIndex );
			}
			else
			{
				throw beans::UnknownPropertyException();
			}

			delete pSet;
		}
		else
		{
			throw beans::UnknownPropertyException();
		}
	}
}

uno::Any SAL_CALL ChXChartObject::getPropertyValue( const ::rtl::OUString& PropertyName )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());

	uno::Any aAny;

	if( mpModel && mnWhichId != CHOBJID_ANY )
	{
		const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( PropertyName );

		if( pMap && pMap->nWID )
		{
			long nWID = pMap->nWID;
			SfxItemSet aSet( mpModel->GetItemPool(), nWID, nWID );

			mpModel->GetAttr( mnWhichId, aSet, mnIndex );

			// map enum
			if( nWID == SCHATTR_TEXT_ORDER )
			{
				chart::ChartAxisArrangeOrderType eArrOrder;
				SvxChartTextOrder eTextOrder =
					((const SvxChartTextOrderItem&)(aSet.Get( SCHATTR_TEXT_ORDER ))).GetValue();

				switch( eTextOrder )
				{
					case CHTXTORDER_SIDEBYSIDE:
						eArrOrder = chart::ChartAxisArrangeOrderType_SIDE_BY_SIDE; break;
					case CHTXTORDER_DOWNUP:
						eArrOrder = chart::ChartAxisArrangeOrderType_STAGGER_EVEN; break;
					case CHTXTORDER_UPDOWN:
						eArrOrder = chart::ChartAxisArrangeOrderType_STAGGER_ODD; break;
					case CHTXTORDER_AUTO:
					default:
						eArrOrder = chart::ChartAxisArrangeOrderType_AUTO; break;
				}
				aAny <<= eArrOrder;
				return aAny;
			}

			if( !aSet.Count() )
			{
				// Default aus ItemPool holen
				if( SID_ATTR_3D_START <= nWID && nWID <= SID_ATTR_3D_END )
				{
					GlobalGenerate3DAttrDefaultItem( nWID, aSet );
				}
				else if( mpModel->GetItemPool().IsWhich( nWID ) )
				{
					if( ! ( OWN_ATTR_VALUE_START <= nWID && nWID <= OWN_ATTR_VALUE_END ))	// properties from SvxShape
					{
						aSet.Put( mpModel->GetItemPool().GetDefaultItem( nWID ) );
					}
				}
				else
				{
#ifdef DBG_UTIL
					String aTmpString( PropertyName );
					ByteString aProp( aTmpString, RTL_TEXTENCODING_ASCII_US );
					DBG_ERROR2( "Diagram: Property %s has an invalid ID (%d)", aProp.GetBuffer(), nWID );
#endif
				}
			}

			if( aSet.Count() )
			{
				// get value from ItemSet
				aAny = maPropSet.getPropertyValue( pMap, aSet );
			}
			else
			{
				// properties Bitmap and BoundRect go here. ToDo: call SvxShape::getPropertyValue (?)
				throw beans::UnknownPropertyException();
			}
		}
		else
		{
			throw beans::UnknownPropertyException();
		}
	}
	return aAny;
}

void SAL_CALL ChXChartObject::addPropertyChangeListener( const ::rtl::OUString& aPropertyName,
													 const uno::Reference< beans::XPropertyChangeListener >& xListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXChartObject::removePropertyChangeListener( const ::rtl::OUString& aPropertyName,
														const uno::Reference< beans::XPropertyChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXChartObject::addVetoableChangeListener( const ::rtl::OUString& PropertyName,
													 const uno::Reference< beans::XVetoableChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXChartObject::removeVetoableChangeListener( const ::rtl::OUString& PropertyName,
														const uno::Reference< beans::XVetoableChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}


// XPropertyState
beans::PropertyState SAL_CALL ChXChartObject::getPropertyState( const ::rtl::OUString& PropertyName )
	throw( beans::UnknownPropertyException, uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( PropertyName );

	if( mpModel &&
		pMap && pMap->nWID )
	{
		SfxItemSet aSet( mpModel->GetItemPool(), pMap->nWID, pMap->nWID );
		mpModel->GetAttr( mnWhichId, aSet, mnIndex );

		return maSfxPropSet.getPropertyState( PropertyName, aSet );
	}

	return beans::PropertyState_DIRECT_VALUE;
}

uno::Sequence< beans::PropertyState > SAL_CALL ChXChartObject::getPropertyStates(
		const uno::Sequence< ::rtl::OUString >& aPropertyName )
		throw( beans::UnknownPropertyException,
			   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	const sal_Int32 nCount = aPropertyName.getLength();
	const rtl::OUString* pNames = aPropertyName.getConstArray();

	uno::Sequence< beans::PropertyState > aRet( nCount );
	beans::PropertyState* pState = aRet.getArray();

	for( sal_Int32 nIdx = 0; nIdx < nCount; nIdx++ )
		pState[ nIdx ] = getPropertyState( pNames[ nIdx ] );

	return aRet;	
}

void SAL_CALL ChXChartObject::setPropertyToDefault( const ::rtl::OUString& PropertyName )
		throw( beans::UnknownPropertyException,
			   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( PropertyName );

	if( mpModel )
	{
		SfxItemSet aSet( mpModel->GetItemPool(), pMap->nWID, pMap->nWID );
		mpModel->GetAttr( mnWhichId, aSet, mnIndex );
		aSet.ClearItem( pMap->nWID );
		mpModel->ChangeAttr( aSet, mnWhichId, mnIndex );
	}
}

uno::Any SAL_CALL ChXChartObject::getPropertyDefault( const ::rtl::OUString& aPropertyName )
		throw( beans::UnknownPropertyException,
			   lang::WrappedTargetException,
			   uno::RuntimeException )
{
	const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( aPropertyName );

	if( ! mpModel->GetItemPool().IsWhich( pMap->nWID ))
		throw beans::UnknownPropertyException();

	SfxItemSet aSet( mpModel->GetItemPool(), pMap->nWID, pMap->nWID );
	aSet.Put( mpModel->GetItemPool().GetDefaultItem( pMap->nWID ));

	uno::Any aAny = maPropSet.getPropertyValue( pMap, aSet );

	return aAny;
}


// XServiceInfo

::rtl::OUString SAL_CALL ChXChartObject::getImplementationName()
	throw( uno::RuntimeException )
{
	return SCH_ASCII_TO_OU( "ChXObject" );
}

uno::Sequence< rtl::OUString > SAL_CALL ChXChartObject::getSupportedServiceNames()
	throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	uno::Sequence< rtl::OUString > aSeq;
	switch( mnWhichId )
	{
		case CHOBJID_DIAGRAM_A_AXIS:
		case CHOBJID_DIAGRAM_B_AXIS:
		case CHOBJID_DIAGRAM_C_AXIS:
		case CHOBJID_DIAGRAM_X_AXIS:
		case CHOBJID_DIAGRAM_Y_AXIS:
		case CHOBJID_DIAGRAM_Z_AXIS:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 4,
				"com.sun.star.chart.ChartAxis",
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor",
				"com.sun.star.style.CharacterProperties" );
			break;
		case CHOBJID_DIAGRAM_X_GRID_MAIN_GROUP:
		case CHOBJID_DIAGRAM_Y_GRID_MAIN_GROUP:
		case CHOBJID_DIAGRAM_Z_GRID_MAIN_GROUP:
		case CHOBJID_DIAGRAM_X_GRID_HELP_GROUP:
		case CHOBJID_DIAGRAM_Y_GRID_HELP_GROUP:
		case CHOBJID_DIAGRAM_Z_GRID_HELP_GROUP:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 3,
				"com.sun.star.chart.ChartGrid",
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor" );
			break;
		case CHOBJID_DIAGRAM_WALL:
		case CHOBJID_DIAGRAM_FLOOR:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 4,
				"com.sun.star.chart.ChartArea",
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.AreaShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor" );
			break;
		case CHOBJID_DIAGRAM_STOCKLINE_GROUP:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 2,
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor" );
			break;
		case CHOBJID_DIAGRAM_STOCKLOSS_GROUP:
		case CHOBJID_DIAGRAM_STOCKPLUS_GROUP:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 4,
				"com.sun.star.chart.ChartArea",
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.AreaShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor" );
			break;
		case CHOBJID_DIAGRAM_AVERAGEVALUE:
		case CHOBJID_DIAGRAM_ERROR:
		case CHOBJID_DIAGRAM_REGRESSION:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 3,
				"com.sun.star.chart.ChartLine",
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor" );
			break;
		case CHOBJID_DIAGRAM_AREA:
			SvxServiceInfoHelper::addToSequence(
				aSeq, 4,
				"com.sun.star.chart.ChartArea",
				"com.sun.star.drawing.AreaShapeDescriptor",
				"com.sun.star.drawing.LineShapeDescriptor",
				"com.sun.star.drawing.ShapeDescriptor" );
			break;
	}
	return aSeq;
}

sal_Bool SAL_CALL ChXChartObject::supportsService( const ::rtl::OUString& ServiceName )
	throw( uno::RuntimeException )
{
	return SvxServiceInfoHelper::supportsService( ServiceName, getSupportedServiceNames() );
}

// XUnoTunnel
sal_Int64 SAL_CALL ChXChartObject::getSomething( const uno::Sequence< sal_Int8 >& aIdentifier )
	throw( uno::RuntimeException )
{
	if( aIdentifier.getLength() == 16 && 0 == rtl_compareMemory( getUnoTunnelId().getConstArray(),
																 aIdentifier.getConstArray(), 16 ) )
	{
		return (sal_Int64)this;
	}
	return 0;
}

