/*************************************************************************
 *
 *  $RCSfile: ChXDataPoint.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:46:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "ChXDataPoint.hxx"
#include "mapprov.hxx"

// header for SvxServiceInfoHelper
#ifndef SVX_UNOPROV_HXX
#include <svx/unoprov.hxx>
#endif
// header for class OGuard
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
// header for class Application
#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif
// header for SvxUnoTextRangeBase
#ifndef _SVX_UNOTEXT_HXX
#include <svx/unotext.hxx>
#endif
// for OWN_ATTR_...
#ifndef _SVX_UNOSHPRP_HXX
#include <svx/unoshprp.hxx>
#endif
// for SID_ATTR_...
#ifndef _SVX_SVXIDS_HRC
#include <svx/svxids.hrc>
#endif
#ifndef _RTL_UUID_H_
#include <rtl/uuid.h>
#endif
#ifndef _RTL_MEMORY_H_
#include <rtl/memory.h>
#endif

#ifndef _COM_SUN_STAR_CHART_CHARTDATACAPTION_HPP_
#include <com/sun/star/chart/ChartDataCaption.hpp>
#endif

#include "schattr.hxx"

// header for SvxChartDataDescrItem
#ifndef _SVX_CHRTITEM_HXX
#define ITEMID_CHARTDATADESCR   SCHATTR_DATADESCR_DESCR
#include <svx/chrtitem.hxx>
#endif

#include "chtmodel.hxx"
#include "globfunc.hxx"			// for GlobalGenerate3DAttrDefaultItem


extern SchUnoPropertyMapProvider aSchMapProvider;

using namespace vos;
using namespace com::sun::star;

ChXDataPoint::ChXDataPoint( sal_Int32 _Col, sal_Int32 _Row, ChartModel* _Model ) :
		maPropSet( aSchMapProvider.GetMap( _Model? CHMAP_DATAPOINT: CHMAP_NONE, _Model )),
		mpModel( _Model ),
		mnCol( _Col ),
		mnRow( _Row )
{}

ChXDataPoint::~ChXDataPoint()
{}


// generate a uniqueId
const uno::Sequence< sal_Int8 > & ChXDataPoint::getUnoTunnelId() throw()
{
	static uno::Sequence< sal_Int8 > * pSeq = 0;
	if( !pSeq )
	{
		osl::Guard< osl::Mutex > aGuard( osl::Mutex::getGlobalMutex() );
		if( !pSeq )
		{
			static uno::Sequence< sal_Int8 > aSeq( 16 );
			rtl_createUuid( (sal_uInt8*)aSeq.getArray(), 0, sal_True );
			pSeq = &aSeq;
		}
	}
	return *pSeq;
}

ChXDataPoint* ChXDataPoint::getImplementation( uno::Reference< uno::XInterface > xData ) throw()
{
	uno::Reference< lang::XUnoTunnel > xUT( xData, uno::UNO_QUERY );
	if( xUT.is() )
		return (ChXDataPoint*)xUT->getSomething( ChXDataPoint::getUnoTunnelId() );
	else
		return NULL;
}


// XPropertySet
uno::Reference< beans::XPropertySetInfo > SAL_CALL ChXDataPoint::getPropertySetInfo() throw( uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );
	return maPropSet.getPropertySetInfo();
}

void SAL_CALL ChXDataPoint::setPropertyValue( const ::rtl::OUString& aPropertyName, const uno::Any& aValue )
	throw( beans::UnknownPropertyException,
		   beans::PropertyVetoException,
		   lang::IllegalArgumentException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	if( mpModel )
	{
		const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( aPropertyName );

		if( pMap && pMap->nWID )
		{
			long nWID = pMap->nWID;
			SfxItemSet* pSet;

			if( nWID == SCHATTR_DATADESCR_DESCR || nWID == SCHATTR_DATADESCR_SHOW_SYM )
			{
				pSet = new SfxItemSet( mpModel->GetItemPool(),
									   SCHATTR_DATADESCR_DESCR, SCHATTR_DATADESCR_DESCR,
									   SCHATTR_DATADESCR_SHOW_SYM, SCHATTR_DATADESCR_SHOW_SYM, 0 );
			}
			else if( nWID == CHATTR_PIE_SEGMENT_OFFSET )	// member of ChartModel rather than property
			{
				sal_Int32 nVal;
				aValue >>= nVal;
				short nValToSet = (short)(abs( nVal ) % 101);		// cast long->short is ok (range 0..100)

				mpModel->PieSegOfs( mnCol ) = nValToSet;
				SvxChartStyle& rStyle = mpModel->ChartStyle();
				if( rStyle == CHSTYLE_2D_PIE_SEGOF1 ||
					rStyle == CHSTYLE_2D_PIE_SEGOFALL )
					rStyle = CHSTYLE_2D_PIE;

				mpModel->BuildChart( FALSE );
				return;				// RETURN
			}
			else
			{
				pSet = new SfxItemSet( mpModel->GetItemPool(), nWID, nWID );
			}

			pSet->Put( mpModel->GetFullDataPointAttr( mnCol, mnRow ));

			if( ! pSet->Count() )
			{
				// get default from ItemPool
				if( SID_ATTR_3D_START <= nWID && nWID <= SID_ATTR_3D_END )
				{
					GlobalGenerate3DAttrDefaultItem( nWID, *pSet );
				}
				else if( mpModel->GetItemPool().IsWhich( nWID ))
				{
					if( ! ( OWN_ATTR_VALUE_START <= nWID && nWID <= OWN_ATTR_VALUE_END ))
					{
						pSet->Put( mpModel->GetItemPool().GetDefaultItem( nWID ));
					}
				}
				else
				{
#ifdef DBG_UTILS
					String aTmpString( aPropertyName );
					ByteString aProp( aTmpString, RTL_TEXTENCODING_ASCII_US );
					DBG_ERROR2( "Diagram: Property %s has an invalid ID (%d)", aProp.GetBuffer(), nWID );
#endif
				}
			}

			if( pSet->Count() )
			{
				sal_Int32 nVal;
				switch( nWID )
				{
					case SCHATTR_STAT_KIND_ERROR:
					case SCHATTR_STAT_INDICATE:
					case SCHATTR_STAT_REGRESSTYPE:
						// property is enum, but item is INT32
						aValue >>= nVal;
						pSet->Put( SfxInt32Item( nWID, nVal ) );
						break;
					// BM: theese two SCHATTRs are merged into one INT32, so extract them from aValue
					case SCHATTR_DATADESCR_DESCR:
					case SCHATTR_DATADESCR_SHOW_SYM:
						{
							// symbol
							aValue >>= nVal;
							pSet->Put( SfxBoolItem( SCHATTR_DATADESCR_SHOW_SYM,
													(nVal & chart::ChartDataCaption::SYMBOL)) );

							// display text, percent or both or value
							SvxChartDataDescr eDescr = CHDESCR_NONE;
							if( nVal & chart::ChartDataCaption::TEXT )
							{
								if( nVal & chart::ChartDataCaption::PERCENT )
									eDescr = CHDESCR_TEXTANDPERCENT;
								else if( (nVal & chart::ChartDataCaption::VALUE) )
									eDescr = CHDESCR_TEXTANDVALUE;
								else
									eDescr = CHDESCR_TEXT;
							}
							else if( (nVal & chart::ChartDataCaption::VALUE) )
							{
								eDescr = CHDESCR_VALUE;
							}
							else
							{
								if( (nVal & chart::ChartDataCaption::PERCENT) )
									eDescr = CHDESCR_PERCENT;
							}

							// chart::ChartDataCaption::FORMAT (missing)

							pSet->Put( SvxChartDataDescrItem( eDescr ) );
						}
						break;
					default:
						// CL: convert special character properties
						if( ! SvxUnoTextRangeBase::SetPropertyValueHelper( *pSet, pMap, aValue, *pSet ))
							maPropSet.setPropertyValue( pMap, aValue, *pSet );
						break;
				}

				mpModel->PutDataPointAttr( mnCol,mnRow, *pSet );

				mpModel->BuildChart( FALSE );
			}
			else
			{
				throw beans::UnknownPropertyException();
			}
			if( pSet )
				delete pSet;
		}
		else
		{
			throw beans::UnknownPropertyException();
		}
	}
	else
	{
		DBG_WARNING( "No Model" );
	}
}

uno::Any SAL_CALL ChXDataPoint::getPropertyValue( const ::rtl::OUString& PropertyName )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex());
	
	uno::Any aAny;

	if( mpModel )
	{
		const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( PropertyName );

		if( pMap && pMap->nWID )
		{
			SfxItemSet* pSet;
			long nWID = pMap->nWID;

			if( nWID == SCHATTR_DATADESCR_DESCR || nWID == SCHATTR_DATADESCR_SHOW_SYM )
			{
				pSet = new SfxItemSet( mpModel->GetItemPool(),
					SCHATTR_DATADESCR_DESCR, SCHATTR_DATADESCR_DESCR,
					SCHATTR_DATADESCR_SHOW_SYM, SCHATTR_DATADESCR_SHOW_SYM, 0 );
			}
			else if( nWID == CHATTR_PIE_SEGMENT_OFFSET )
			{
				sal_Int32 nVal = mpModel->PieSegOfs( mnCol );
				aAny <<= nVal;
				return aAny;								// RETURN
			}
			else
			{
				pSet = new SfxItemSet( mpModel->GetItemPool(), nWID, nWID );
			}
			pSet->Put( mpModel->GetFullDataPointAttr( mnCol, mnRow ));

			if( ! pSet->Count() )
			{
				// get default from pool
				if( SID_ATTR_3D_START <= nWID && nWID <= SID_ATTR_3D_END )
				{
					GlobalGenerate3DAttrDefaultItem( nWID, *pSet );
				}
				else if( mpModel->GetItemPool().IsWhich( nWID ))
				{
					if( OWN_ATTR_VALUE_START > nWID || nWID > OWN_ATTR_VALUE_END )
					{
						pSet->Put( mpModel->GetItemPool().GetDefaultItem( nWID ) );
					}
				}
				else
				{
#ifdef DBG_UTIL
					String aTmpString( PropertyName );
					ByteString aProp( aTmpString, RTL_TEXTENCODING_ASCII_US );
					DBG_ERROR2( "Diagram: Property %s has an invalid ID (%d)", aProp.GetBuffer(), nWID );
#endif
				}
			}

			if( pSet->Count() )
			{
				// get value from ItemSet

				// theese two SCHATTRs have to be merged into one INT32 in aAny
				if( nWID == SCHATTR_DATADESCR_DESCR || nWID == SCHATTR_DATADESCR_SHOW_SYM )
				{
					BOOL bShowSymbol = ((const SfxBoolItem&)
						(pSet->Get(SCHATTR_DATADESCR_SHOW_SYM))).GetValue();
					SvxChartDataDescr eDescr = ((const SvxChartDataDescrItem&)
						(pSet->Get(SCHATTR_DATADESCR_DESCR))).GetValue();

					sal_Int32 nVal = 0;
					switch( eDescr )
					{
						case CHDESCR_NONE:
							nVal = chart::ChartDataCaption::NONE;
							break;
						case CHDESCR_VALUE:
							nVal = chart::ChartDataCaption::VALUE;
							break;
						case CHDESCR_PERCENT:
							nVal = chart::ChartDataCaption::PERCENT;
							break;
						case CHDESCR_TEXT:
							nVal = chart::ChartDataCaption::TEXT;
							break;
						case CHDESCR_TEXTANDPERCENT:
							nVal = chart::ChartDataCaption::PERCENT | chart::ChartDataCaption::TEXT;
							break;
						case CHDESCR_TEXTANDVALUE:
							nVal = chart::ChartDataCaption::VALUE | chart::ChartDataCaption::TEXT;
					// chart::ChartDataCaption::FORMAT (missing)
					}
					if( bShowSymbol ) nVal |= chart::ChartDataCaption::SYMBOL;

					aAny <<= nVal;
				}
				else
				{
					aAny = maPropSet.getPropertyValue( pMap, *pSet );
				}
			}
			else
			{
				throw beans::UnknownPropertyException();
			}
			if( pSet )
				delete pSet;
		}
		else
		{
			throw beans::UnknownPropertyException();
		}
	}

	return aAny;
}

void SAL_CALL ChXDataPoint::addPropertyChangeListener( const ::rtl::OUString& aPropertyName,
													   const uno::Reference< beans::XPropertyChangeListener >& xListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXDataPoint::removePropertyChangeListener( const ::rtl::OUString& aPropertyName,
														  const uno::Reference< beans::XPropertyChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXDataPoint::addVetoableChangeListener( const ::rtl::OUString& PropertyName,
													   const uno::Reference< beans::XVetoableChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

void SAL_CALL ChXDataPoint::removeVetoableChangeListener( const ::rtl::OUString& PropertyName,
														  const uno::Reference< beans::XVetoableChangeListener >& aListener )
	throw( beans::UnknownPropertyException,
		   lang::WrappedTargetException,
		   uno::RuntimeException )
{}

// XPropertyState
beans::PropertyState SAL_CALL ChXDataPoint::getPropertyState( const ::rtl::OUString& PropertyName )
	throw( beans::UnknownPropertyException, uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( PropertyName );

	if( mpModel &&
		pMap && pMap->nWID )
	{
		if( pMap->nWID == CHATTR_PIE_SEGMENT_OFFSET )
		{
			sal_Int32 nOffset;
			uno::Any aAny = getPropertyValue( PropertyName );
			aAny >>= nOffset;
			if( nOffset )
				return beans::PropertyState_DIRECT_VALUE;
			else
				return beans::PropertyState_DEFAULT_VALUE;
		}
		else if( pMap->nWID == SCHATTR_DATADESCR_DESCR )
		{
			SfxItemSet aSet( mpModel->GetItemPool(),
							 SCHATTR_DATADESCR_DESCR, SCHATTR_DATADESCR_DESCR,
							 SCHATTR_DATADESCR_SHOW_SYM, SCHATTR_DATADESCR_SHOW_SYM, 0 );
			aSet.Put( mpModel->GetFullDataPointAttr( mnCol, mnRow ));

			SfxItemState eState1 = aSet.GetItemState( SCHATTR_DATADESCR_DESCR );
			SfxItemState eState2 = aSet.GetItemState( SCHATTR_DATADESCR_SHOW_SYM );
			if( eState1 == SFX_ITEM_DEFAULT &&
				eState2 == SFX_ITEM_DEFAULT )
				return beans::PropertyState_DEFAULT_VALUE;
			else if( eState1 < SFX_ITEM_DEFAULT &&
					 eState2 < SFX_ITEM_DEFAULT )
				return beans::PropertyState_AMBIGUOUS_VALUE;
			else
				return beans::PropertyState_DIRECT_VALUE;
		}
		else
		{
			SfxItemSet aSet( mpModel->GetItemPool(), pMap->nWID, pMap->nWID );
			aSet.Put( mpModel->GetFullDataPointAttr( mnCol, mnRow ));

			SfxItemState eState = aSet.GetItemState( pMap->nWID );
			if( eState == SFX_ITEM_DEFAULT )
				return beans::PropertyState_DEFAULT_VALUE;
			else if( eState < SFX_ITEM_DEFAULT )
				return beans::PropertyState_AMBIGUOUS_VALUE;
			else
				return beans::PropertyState_DIRECT_VALUE;
		}
	}

	return beans::PropertyState_DIRECT_VALUE;
}

uno::Sequence< beans::PropertyState > SAL_CALL ChXDataPoint::getPropertyStates(
		const uno::Sequence< ::rtl::OUString >& aPropertyName )
		throw( beans::UnknownPropertyException,
			   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	const sal_Int32 nCount = aPropertyName.getLength();
	const rtl::OUString* pNames = aPropertyName.getConstArray();

	uno::Sequence< beans::PropertyState > aRet( nCount );
	beans::PropertyState* pState = aRet.getArray();

	for( sal_Int32 nIdx = 0; nIdx < nCount; nIdx++ )
		pState[ nIdx ] = getPropertyState( pNames[ nIdx ] );

	return aRet;
}

void SAL_CALL ChXDataPoint::setPropertyToDefault( const ::rtl::OUString& PropertyName )
		throw( beans::UnknownPropertyException,
			   uno::RuntimeException )
{
	OGuard aGuard( Application::GetSolarMutex() );

	const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( PropertyName );

	if( mpModel && pMap && pMap->nWID )
	{
		if( pMap->nWID == CHATTR_PIE_SEGMENT_OFFSET )
		{
				mpModel->PieSegOfs( mnCol ) = 0;
		}
		else if( pMap->nWID == SCHATTR_DATADESCR_DESCR )
		{
			SfxItemSet aSet( mpModel->GetItemPool(),
							 SCHATTR_DATADESCR_DESCR, SCHATTR_DATADESCR_DESCR,
							 SCHATTR_DATADESCR_SHOW_SYM, SCHATTR_DATADESCR_SHOW_SYM, 0 );

			aSet.ClearItem( SCHATTR_DATADESCR_DESCR );
			aSet.ClearItem( SCHATTR_DATADESCR_SHOW_SYM );
			mpModel->PutDataPointAttr( mnCol, mnRow, aSet );
		}
		else
		{
			SfxItemSet aSet( mpModel->GetItemPool(), pMap->nWID, pMap->nWID );

			aSet.ClearItem( pMap->nWID );
			mpModel->PutDataPointAttr( mnCol, mnRow, aSet );
		}
	}
}

uno::Any SAL_CALL ChXDataPoint::getPropertyDefault( const ::rtl::OUString& aPropertyName )
		throw( beans::UnknownPropertyException,
			   lang::WrappedTargetException,
			   uno::RuntimeException )
{
	uno::Any aAny;
	const SfxItemPropertyMap* pMap = maPropSet.getPropertyMapEntry( aPropertyName );

	if( mpModel && ! mpModel->GetItemPool().IsWhich( pMap->nWID ))
		throw beans::UnknownPropertyException();

	if( mpModel && pMap && pMap->nWID )
	{
		if( pMap->nWID == CHATTR_PIE_SEGMENT_OFFSET )
		{
			aAny <<= (sal_Int32)0;
		}
		else if( pMap->nWID == SCHATTR_DATADESCR_DESCR )
		{
			SfxItemSet aSet( mpModel->GetItemPool(),
							 SCHATTR_DATADESCR_DESCR, SCHATTR_DATADESCR_DESCR,
							 SCHATTR_DATADESCR_SHOW_SYM, SCHATTR_DATADESCR_SHOW_SYM, 0 );

			aSet.Put( mpModel->GetItemPool().GetDefaultItem( SCHATTR_DATADESCR_DESCR ));
			aSet.Put( mpModel->GetItemPool().GetDefaultItem( SCHATTR_DATADESCR_SHOW_SYM ));
			aAny = maPropSet.getPropertyValue( pMap, aSet );
		}
		else
		{
			SfxItemSet aSet( mpModel->GetItemPool(), pMap->nWID, pMap->nWID );
			aSet.Put( mpModel->GetItemPool().GetDefaultItem( pMap->nWID ));
			aAny = maPropSet.getPropertyValue( pMap, aSet );
		}
	}

	return aAny;
}


// XShapeDescriptor ( ::XShape ::XDiagram )
::rtl::OUString SAL_CALL ChXDataPoint::getShapeType() throw( uno::RuntimeException )
{
	return SCH_ASCII_TO_OU( "com.sun.star.chart.ChartDataPoint" );
}


// XServiceInfo
::rtl::OUString SAL_CALL ChXDataPoint::getImplementationName() throw( uno::RuntimeException )
{
	return SCH_ASCII_TO_OU( "ChXDataPoint" );
}

sal_Bool SAL_CALL ChXDataPoint::supportsService( const ::rtl::OUString& ServiceName ) throw( uno::RuntimeException )
{
	return SvxServiceInfoHelper::supportsService( ServiceName, getSupportedServiceNames() );	
}

uno::Sequence< rtl::OUString > SAL_CALL ChXDataPoint::getSupportedServiceNames() throw( uno::RuntimeException )
{
	uno::Sequence< rtl::OUString > aSeq;
	SvxServiceInfoHelper::addToSequence(
		aSeq, 5,
		"com.sun.star.chart.ChartDataPointProperties",
		"com.sun.star.drawing.ShapeDescriptor",
		"com.sun.star.drawing.LineShapeDescriptor",
		"com.sun.star.drawing.AreaShapeDescriptor",
		"com.sun.star.style.CharacterProperties" );

	if( mpModel )
	{
		long nType = mpModel->GetBaseType();
		if( mpModel->Is3DChart() && (nType == CHTYPE_BAR || nType == CHTYPE_COLUMN) )
			SvxServiceInfoHelper::addToSequence( aSeq, 1, "com.sun.star.chart.Chart3DBarProperties" );
		else if( ! mpModel->Is3DChart() && nType == CHTYPE_CIRCLE )
			SvxServiceInfoHelper::addToSequence( aSeq, 1, "com.sun.star.chart.ChartPieSegmentProperties" );
	}

	return aSeq;
}

// XUnoTunnel
sal_Int64 SAL_CALL ChXDataPoint::getSomething( const uno::Sequence< sal_Int8 >& aIdentifier )
	throw( uno::RuntimeException )
{
	if( aIdentifier.getLength() == 16 && 0 == rtl_compareMemory( getUnoTunnelId().getConstArray(),
																 aIdentifier.getConstArray(), 16 ) )
	{
		return (sal_Int64)this;
	}
	return 0;
}

