/*************************************************************************
 *
 *  $RCSfile: director.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:47:19 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _DIRECTOR_HXX
#define _DIRECTOR_HXX

#ifndef _VOS_THREAD_HXX_
#include <vos/thread.hxx>
#endif

#ifndef _VOS_PROCESS_HXX_
#include <vos/process.hxx>
#endif

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _DATETIME_HXX
#include <tools/datetime.hxx>
#endif

#ifndef _FSYS_HXX
#include <tools/fsys.hxx>
#endif

#ifndef _SV_CONFIG_HXX
#include <vcl/config.hxx>
#endif

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

#ifndef _SV_TIMER_HXX
#include <vcl/timer.hxx>
#endif

#include "ziplst.hxx"


#define PRG_NAME			"zipDirector"
#define PRG_VERSION			"v1.23 (unicode)"

class MainWin;
class ResMgr;

class PersistentInformationBroadcaster;
class GenericInformationList;
class BroadcastMessage;
class InformationClient;
class GenericInformation;

/////////////////////////////////////////////////////////////////////

#define KEY_SEC_DIRECTOR		"director"
#define KEY_MAX_PROCESSES		"maxprocesses"
#define KEY_VOLUMEWARN			"volumewarn"

/////////////////////////////////////////////////////////////////////

DECLARE_LIST(PrjList, Project*)

class ProjectList : public IServerZipList
{
	PrjList					m_aList;

	void 					DeleteList();
	Project*				SearchInList(const ByteString& rName);
	ByteString 				GetValue(GenericInformationList* pGenList, ByteString& rPath);
public:
		ProjectList();
		~ProjectList();

	BOOL 					Update();
	PrjList&				GetList() { return m_aList; }
};


/////////////////////////////////////////////////////////////////////

class ZipJob : public NAMESPACE_VOS(OThread)
{
	USHORT 								m_nPrio;
	char**								m_ppCmd;
	USHORT 								m_nCmd;
	NAMESPACE_VOS(OProcess)*			m_pProcess;

	BOOL								m_bKilled;
	BOOL								m_bOverrun;

	PersistentInformationBroadcaster*	m_pIBroadcaster;

	BOOL 								m_bRunning;
	BOOL 								m_bDoSuccess;
	Project* 							m_pPrj;

	ByteString							m_aExecutable;
	ByteString							m_aBroadcastMsg;
	ByteString							m_aStatusBroadcastMsg;
	ByteString							m_aLogFilename;

	ByteString 							m_aWorkstamp;
	ByteString 							m_aEnvironment;
	ByteString 							m_aMinor;
	ByteString 							m_aMinorSub;
	ByteString							m_aOSDef;
	ByteString							m_aDestination;
	ByteString							m_aProtocol;
	ByteString							m_aLanguages;

	BOOL 								m_bSpecialPrjJob;

	void 						ScanSuccess();
public:
		ZipJob( USHORT nPrio );
		~ZipJob();

	ULONG 					m_nRePostCnt;
	ULONG 					m_nProcessID;

	void					SAL_CALL run();
	void					SAL_CALL onTerminated();
	void					SAL_CALL kill();

	BOOL 					isRunning() const { return m_bRunning; }
	BOOL					isKilled() const  { return m_bKilled; }
	BOOL 					isOverrun() const { return m_bOverrun; }

	void 					setRunning(BOOL b) { m_bRunning = b; }
	void 					setKilled(BOOL b) { m_bKilled = b; }
	void 					setOverrun() { m_bOverrun = TRUE; }
	BOOL 					IsEqualTo( ZipJob* pJob );

	void 					SetIBroadcaster( PersistentInformationBroadcaster* pNew ) { m_pIBroadcaster = pNew; }
	void					SendStatus( const ByteString& aKey, const ByteString& aValue );

	BOOL 					IsSuccess() const { return m_bDoSuccess; }

	void 					SetExecutable( const ByteString& rExec) { m_aExecutable = rExec; }

	void 					SetBroadcastMsg( const ByteString& rNew ) { m_aBroadcastMsg = rNew; }
	const ByteString& 		GetBroadcastMsg() const { return m_aBroadcastMsg; }

	void 					SetStatusBroadcastMsg( const ByteString& rNew ) { m_aStatusBroadcastMsg = rNew; }
	const ByteString& 		GetStatusBroadcastMsg() const { return m_aStatusBroadcastMsg; }

	void 					SetLogFilename( const ByteString& rNew ) { m_aLogFilename = rNew; }
	const ByteString& 		GetLogFilename() const { return m_aLogFilename; }

	void 					SetWorkstamp( const ByteString& rNew ) { m_aWorkstamp = rNew; }
	const ByteString&		GetWorkstamp() const { return m_aWorkstamp; }

	void 					SetEnvironment( const ByteString& rNew ) { m_aEnvironment = rNew; }
	const ByteString&	 	GetEnvironment() const { return m_aEnvironment; }

	void 					SetMinor( const ByteString& rNew ) { m_aMinor = rNew; }
	const ByteString&	 	GetMinor() const { return m_aMinor; }

	void 					SetMinorSub( const ByteString& rNew ) { m_aMinorSub = rNew; }
	const ByteString&		GetMinorSub() const { return m_aMinorSub; }

	void 					SetOSDef( const ByteString& rNew ) { m_aOSDef = rNew; }
	const ByteString&		GetOSDef() const { return m_aOSDef; }

	void 					SetDestination( const ByteString& rNew ) { m_aDestination = rNew; }
	const ByteString&		GetDestination() const { return m_aDestination; }

	void 					SetProtocol( const ByteString& rNew ) { m_aProtocol = rNew; }
	const ByteString&	 	GetProtocol() const { return m_aProtocol; }

	void 					SetLanguages( const ByteString& rNew ) { m_aLanguages = rNew; }
	const ByteString&	 	GetLanguages() const { return m_aLanguages; }

	BOOL 					IsSpecialPrj() const { return m_bSpecialPrjJob; }
	void					SetSpecialPrj(BOOL bNew) { m_bSpecialPrjJob = bNew; }

	USHORT 					GetPrio() const { return m_nPrio; }

	void 					SetPrj(Project* p) { m_pPrj = p; }
	Project*				GetPrj() const { return m_pPrj; }

	void 					AddCmd( const ByteString& rCmd );
	ByteString				GetCmdStr();
	char**					GetCmd() const { return m_ppCmd; }
	USHORT 					GetCmdCnt() const { return m_nCmd; }
};

DECLARE_LIST(JobQueue, ZipJob*)

/////////////////////////////////////////////////////////////////////

#define EVT_JOB_KILLED			1UL
#define EVT_JOB_DONE			2UL
#define EVT_SEND_SUCCESS		3UL
#define EVT_SEND_FAIL			4UL

DECLARE_LIST(BroadcastStack, ByteString*)

class ZipDirector : public Application
{
	ResMgr*								m_pResMgr;
	MainWin*  							m_pMainWin;

	InformationClient*					m_pIClient;
	Timer								m_aIServerReconnectTimer;
	PersistentInformationBroadcaster*	m_pIBroadcaster;

	Timer								m_aBroadcastReconnectTimer;
	ProjectList							m_aPrjList;

	BroadcastStack						m_aBroadcastStack;
	AutoTimer							m_aBroadcastStackTimer;

	ULONG  								m_nVolumeWarn;
	USHORT 								m_nMaxProcesses;
	JobQueue							m_aJobQueue;
	AutoTimer							m_aJobQueueTimer;
	Timer								m_aJobQueueStatTimer;
	BOOL 								m_bQueuePause;

	void 					Connect2Broadcast();
	void 					DeleteJobQueue();
	void 					StartPendingBroadcasts();

	ByteString 				DoMetaReplacement( MetaReplacement*, Project*, const ByteString&, const ByteString& );

	void 					UpdateQueueWin();
	USHORT 					QueueCnt( BOOL bPending );
	void					InsertQueue( ZipJob* pNew );

	AutoTimer							m_aLowInfoTimer;
	BOOL			 					m_bLastVolume;
	DirEntry 							m_aLastVolume;
	void 					TestDestVolume();
	void 					TestDestVolume(const DirEntry& rVolume);
	void 					SendMailNotification( BOOL bSuccess, ZipJob* pJob );

	DECL_LINK( LowInfo, void* );

	DECL_LINK( IServerTerminate, void* );
	DECL_LINK( IServerReconnectHdl, void* );

	DECL_LINK( BroadcastTerminate, void* );
	DECL_LINK( BroadcastReconnectHdl, void* );
	DECL_LINK( Broadcast, BroadcastMessage* );

	DECL_LINK( BroadcastStackHdl, void* );

	DECL_LINK( JobQueueStat, void* );
	DECL_LINK( JobQueueHdl, void* );
//	DECL_LINK( JobComeBack, ZipJob* );
public:
	virtual void 			Main();
	ResMgr*					GetResMgr() const { return m_pResMgr; }

	void					UserEvent( ULONG nEvent, void* pEventData );

	BOOL					Connect2IServer();
	InformationClient*		GetIClient() const { return m_pIClient; }

	void	  				Log( const ByteString& );
	void 					PrepareJob( const ByteString& );

	void					QueuePause( BOOL bPause );
	USHORT 					QueuePendingCnt();
	USHORT					QueueProcessCnt();

	BOOL 					ReadProjects();
	void 					KillAllJobs();
	void					KillByID( ULONG nID );
};

extern ZipDirector* pApp;

#define RESMGR()	pApp->GetResMgr()
#define LOG(msg)	pApp->Log(msg)

#endif // _DIRECTOR_HXX
