/*************************************************************************
 *
 *  $RCSfile: scpzip.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: ok $ $Date: 2000/09/29 14:09:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdio.h>
#ifdef UNX
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <grp.h>
#endif
#include <time.h>

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif
#ifndef _VOS_PROCESS_HXX
#include <vos/process.hxx>
#endif
#ifndef _FSYS_HXX
#include <tools/fsys.hxx>
#endif
#ifndef _OSL_THREAD_H_
#include <osl/thread.h>
#endif

#include <com/sun/star/setup/OSType.hpp>

#include "setup2/sistream.hxx"
#include "setup2/script.hxx"
#include "setup2/instdb.hxx"
#include "setup2/zipfile.hxx"
#include "setup2/hashtbl.hxx"
#include "setup2/sifsys.hxx"

#include "protocol.hxx"
#include "scplink.hxx"

#include "searchcache.hxx"

// ============================================================================

using namespace com::sun::star::setup;

// ============================================================================

#define NORMAL_DIRNAME				"normal"
#define MERGE_DIRNAME				"plump"
#define SPLIT_DIRNAME				"splump"
#define WEB_DIRNAME					"web"

#define JUNK_EXTENSION		   		"bin"
#define MASTER_EXTENSION_UNX		"bin"
#define MASTER_EXTENSION	   		"exe"

USHORT 			nGlobalSetupID		= 0;
USHORT 			nGlobalID			= 0;

ByteString		aBasePath;
ByteString		aDestinationPath;
ByteString		aMergeDestinationPath;
ByteString		aSplitDestinationPath;
ByteString		aWebDestinationPath;

ByteString		aMergeFilename;
ByteString		aHTMLFilename;

BOOL			bTestOnly			= FALSE;
BOOL			bOldSetupZIPStyle	= FALSE;

USHORT 			nCompression		= 5;
USHORT 			nProcessorType 		= 0; 			// 0 == INTEL, 1 == SPARC
USHORT 			nTargetSystem		= 0;	        // 0 == WNT, 1 == OS2, 2 == UNX
USHORT 			nUnzipedFiles		= 0;
ULONG 			nUnzipedSize		= 0;
USHORT 			nDefLang			= 0;
ULONG 			nSplitSize			= 0;

Protocol*		pLog				= NULL;

DECLARE_HASHTABLE(DoneList,ByteString,UINT32);
DoneList aDoneList(1017);

#ifdef UNX
gid_t	gr_gid;
#endif

///////////////////////////////////////////////////////////////////////////////

void PrintZipUsage()
{
	fprintf( stderr, "usage:\n" );
	fprintf( stderr, "\t-<1..9> compression (default is 5)\n" );
	fprintf( stderr, "\t-o test only\n" );
	fprintf( stderr, "\t-l <langcode,...> (first langcode is default language)\n" );
	fprintf( stderr, "\t-h <INTEL | SPARC> (INTEL)\n" );
	fprintf( stderr, "\t-t <WNT | OS2 | UNX> (default WNT)\n" );
	fprintf( stderr, "\t-m <filename> merge all files to one file\n" );
	fprintf( stderr, "\t-z <size> split merged file in <size> blocks\n" );
	fprintf( stderr, "\t-s <source script>\n" );
	fprintf( stderr, "\t-d <destination path>\n" );
	fprintf( stderr, "\t-i <include path,...>\n" );
	fprintf( stderr, "\t-e <filename> (write a non HTML error log)\n" );
	fprintf( stderr, "\t-p <filename> (write a HTML protocol)\n" );
	fprintf( stderr, "\t-w generate a webinstall script\n" );
	fprintf( stderr, "\t-u use old setup.zip style\n" );
}

///////////////////////////////////////////////////////////////////////////////

class ScpZip : public Application
{
public:
	virtual void Main() {}
};

ScpZip aApp;

///////////////////////////////////////////////////////////////////////////////

void __EnumFilesCallBack( char *pFile, ULONG nSize, void *pObject )
{
	nUnzipedSize += nSize;
}

///////////////////////////////////////////////////////////////////////////////

void KillingAction()
{
	SiDirEntry aKillfile( aBasePath );
	aKillfile += ByteString("bastard.kil");
	if( aKillfile.Exists() )
	{
		aKillfile.Kill();

		SiDirEntry aDestDir( aDestinationPath );
		Dir aDir( aDestDir, FSYS_KIND_FILE );
		for( USHORT i = 0; i < aDir.Count(); ++i )
		{
			SiDirEntry aEntry( aDir[i] );
			aEntry.Kill();
		}

		aDestDir.Kill();

		aDestDir.SetName( UniString::CreateFromAscii(MERGE_DIRNAME) );
		aDestDir.Kill();

		aDestDir.SetName( UniString::CreateFromAscii(SPLIT_DIRNAME) );
		aDestDir.Kill();

		pLog->Log( "*** KILLED ***" );
		exit( 0 );
	}
}

void ZipFile( SiFile* pFile, USHORT nLang )
{
	ByteString aDoneID( pFile->GetName() );
	aDoneID += ByteString::CreateFromInt32( nLang );
	aDoneID += pFile->GetID();
	if( aDoneList.Find(aDoneID) != 0 )
		return;
	else
		aDoneList.Insert( aDoneID, 1 );

	ByteString aSource( FindSourcefile(pFile, nLang) );
	if( !aSource.Len() )
	{
		ByteString aFilename( pFile->GetName() );
		USHORT nTmp = 0;
		USHORT nPos = STRING_NOTFOUND;
		while( (nTmp = aFilename.Search("/", nTmp)) != STRING_NOTFOUND ) {
			nPos = nTmp;
			nTmp += 1;
		}
		if( nPos != STRING_NOTFOUND )
		{
			ByteString aPath( aFilename );
			aPath.Erase( nPos );
			aPath.Erase( 0, 1 );

			aFilename.Erase( 0, nPos+1 );
			aSource = ExtNOCacheSearch( aFilename, aPath, nLang );
			if( aSource.Len() )
				pFile->SetProperty( "Name", aFilename );
		}
	}

	if( !aSource.Len() )
	{
		ByteString aMsg( "error: source file '" );
		aMsg += pFile->GetName();
		aMsg += "' cannot be found.";
		pLog->Log( aMsg );
		return ;
	}

	if( bTestOnly ) return;

	SiDirEntry aDestination( aDestinationPath );
	if( pFile->IsArchive() || pFile->IsPacked() )
	{
		ByteString aPackedname;
		if( bOldSetupZIPStyle && pFile->IsSetupZip() )
			aPackedname = "f_0000";
		else
		{
			aPackedname = pFile->IsSetupZip() ?
							ByteString::CreateFromInt32( ++nGlobalSetupID ) :
							ByteString::CreateFromInt32( ++nGlobalID );

			USHORT n = pFile->IsSetupZip()? 3 : 4;
			while( aPackedname.Len() < n )
				aPackedname.Insert( "0", 0 );

			if( pFile->IsSetupZip() )
				aPackedname.Insert( "f0_", 0 );
			else
				aPackedname.Insert( "f_", 0 );
		}

		aDestination += aPackedname;
		if( !pFile->IsSetupZip() && aDestination.Exists() )
			aDestination.Kill();

		if( pFile->IsArchive() )
		{
			SiDirEntry aEntry(aSource);
			FileCopier aCpy( aEntry, aDestination );
			aCpy.Execute();
		}
		else
		{
			ByteString aCompressionParam( '-' );
			aCompressionParam += ByteString::CreateFromInt32( nCompression );

			ByteString aSys( "zip -j ");
			aSys += aCompressionParam;
			aSys += " \"";
			aSys += aDestination.GetFull();
			aSys += "\" \"";
			aSys += aSource;
			aSys += "\"";
			pLog->Log( aSys );

#ifndef UNX
			USHORT nOptions = NAMESPACE_VOS(OProcess)::TOption_SearchPath |
								NAMESPACE_VOS(OProcess)::TOption_Wait;

			NAMESPACE_VOS(OProcess)::TProcessOption eOptions =
								(NAMESPACE_VOS(OProcess)::TProcessOption)nOptions;

			::rtl::OUString* pArgs = new ::rtl::OUString [ 4 ];

			pArgs[0] = ::rtl::OUString::createFromAscii( "-j" );
			pArgs[1] = ::rtl::OUString::createFromAscii( aCompressionParam.GetBuffer() );
			pArgs[2] = ::rtl::OUString::createFromAscii( aDestination.GetFull().GetBuffer() );
			pArgs[3] = ::rtl::OUString::createFromAscii( aSource.GetBuffer() );

			NAMESPACE_VOS(OArgumentList) aArgs( pArgs, 4 );

			NAMESPACE_VOS(OProcess) aProcess( ::rtl::OUString::createFromAscii("zip") );
			aProcess.execute( eOptions, aArgs );
			aProcess.join();

			delete [] pArgs;
 #else
 			system( aSys.GetBuffer() );
#endif
		}

		if( (bOldSetupZIPStyle && !pFile->IsSetupZip()) ||
			!bOldSetupZIPStyle )
		{
			// rename destination
			SiDirEntry aExtension( aDestination );
			aExtension.SetExtension( UniString::CreateFromAscii("zip") );
			aExtension.MoveTo( aDestination );
			if( pFile->IsArchive() || pFile->IsPacked() )
				pFile->SetProperty( "PackedName", aPackedname );
		}
	}
	else
	{
		aDestination += pFile->GetName();
		SiDirEntry aSrcEntry( aSource );

		ByteString aMsg( "copy " );
		aMsg += aDestination.GetFull();
		aMsg += " ";
		aMsg += aSrcEntry.GetFull();
		pLog->Log( aMsg );

		FileCopier aCopier( aSrcEntry, aDestination );
		aCopier.Execute();
	}

	// adjust file properties
	SiDirEntry aEntry;
	if( pFile->IsArchive() )
		aEntry = aDestination;
	else
		aEntry = aSource;

	FileStat aFileStat( aEntry );
	long nSz = aFileStat.GetSize();
	pFile->SetProperty( "Size", nSz );

	if( pFile->IsArchive() )
	{
		SiZipFile aZipFile;
		aZipFile.DetectFilesAndSize( aSource );

		pFile->SetProperty( "ArchiveFiles", aZipFile.GetFiles() );
		pFile->SetProperty( "ArchiveSize", aZipFile.GetSize() );
	}
#ifdef UNX
	SiDirEntry aDest( aDestination );
	chown( aDestination.GetFull().GetBuffer(), -1, gr_gid );

	int nOctal	= 0;
	int nRights	= pFile->GetScpUnixRights();

	nOctal 	 = nRights / 100;
	nRights %= 100;
	nOctal  *= 8;
	nOctal  += nRights / 10;
	nRights %= 10;
	nOctal  *= 8;
	nOctal  += nRights;

	chmod( aDestination.GetFull().GetBuffer(), nOctal );
#endif
}

void ZipInstallation( SiModule* pModule )
{
	const SiFileList& rFLst = pModule->GetFileList();
	for( USHORT i = 0; i < rFLst.Count(); ++i )
	{
		KillingAction();

		SiFile* pFile = rFLst.GetObject(i);
		if( pFile->DeleteOnly() )
			continue;

		if( pFile->HasLangRef() )
		{
			for( USHORT xx = 0; xx < aLanguageList.Count(); ++xx )
			{
				USHORT nLang = aLanguageList.GetObject(xx)->nId;
				SiFile* pLangRef = (SiFile*) pFile->GetLangRef( nLang );
				if( pLangRef )
				{
					pLangRef->JoinWithParent();
					ZipFile( pLangRef, nLang );
				}
			}
		}
		else
			ZipFile( pFile, nDefLang );
	}

	const SiModuleList* pModuleList = pModule->GetModuleList();
	for( USHORT x = 0; x < pModuleList->Count(); ++x )
		ZipInstallation( pModuleList->GetObject(x) );
}

///////////////////////////////////////////////////////////////////////////////

void DoScpActions( SiCompiledScript* pCS )
{
	const SiScpActionList& rLst = pCS->GetScpActionList();
	for( USHORT x = 0; x < rLst.Count(); x++ )
	{
		KillingAction();

		SiScpAction* pAction = rLst.GetObject(x);
		ByteString aSource( FindSourcefile(pAction->GetCpySource(),nDefLang) );
		ByteString aMsg;
		if( !aSource.Len() )
		{
			aMsg = "error: source file '";
			aMsg += pAction->GetCpySource();
			aMsg += " ";
			aMsg += "' cannot be found.";
			pLog->Log( aMsg );
			continue;
		}

		if( bTestOnly ) continue;

		SiDirEntry aSrc(aSource);
		SiDirEntry aDst(aDestinationPath);
		aDst += pAction->GetCpyDestination();

		aMsg = "copy ";
		aMsg += aSrc.GetFull();
		aMsg += " ";
		aMsg += aDst.GetFull();
		pLog->Log( aMsg );

		FileCopier aCopier( aSrc, aDst );
		aCopier.Execute();
#ifdef UNX
		chown( aDst.GetFull().GetBuffer(), -1, gr_gid );
		chmod( aDst.GetFull().GetBuffer(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
#endif
	}
}

///////////////////////////////////////////////////////////////////////////////

#define _BUF_LEN 32000

void Splitter()
{
	ByteString aMsg( "\nsplit merged file into " );
	aMsg += ByteString::CreateFromInt32( nSplitSize );
	aMsg += " byte blocks:\n";
	pLog->Log( aMsg );

	ByteString aSourceName	( aMergeFilename );
	ByteString aMergeName	( aMergeFilename );
	aSourceName += ".";
	aMergeName += "-000.";

	if( nTargetSystem == 2 )
	{
		aSourceName += MASTER_EXTENSION_UNX;
		aMergeName += MASTER_EXTENSION_UNX;
	}
	else
	{
		aSourceName += MASTER_EXTENSION;
		aMergeName += MASTER_EXTENSION;
	}

	SiDirEntry aSource( aMergeDestinationPath );
	aSource += ByteString( aSourceName );

	SiDirEntry aNewEXE( aSplitDestinationPath );
	aNewEXE += aMergeName;

	SiDirEntry aFirstEXE( aNewEXE );

	SvFileStream aStrm( aSource.GetFullUni(), STREAM_READ );
	SvFileStream* pSplitterStrm = new SvFileStream( aNewEXE.GetFullUni(), STREAM_WRITE | STREAM_TRUNC );

	aMsg = "split: ";
	aMsg += aNewEXE.GetFull();
	pLog->Log( aMsg );

	char cBuf[_BUF_LEN];
	ULONG nActSplitSize = 0;
	USHORT nSplitPart = 0;

	while( !aStrm.IsEof() )
	{
		ULONG nDoRead = _BUF_LEN;
		memset( cBuf, 0, _BUF_LEN );

		if( nActSplitSize < nSplitSize )
			if( nActSplitSize + _BUF_LEN > nSplitSize )
				nDoRead = nSplitSize - nActSplitSize;

		ULONG nRead = aStrm.Read( cBuf, nDoRead );
		nActSplitSize += nRead;

		pSplitterStrm->Write( cBuf, nRead );
		if( nActSplitSize == nSplitSize || aStrm.IsEof() )
		{
			nActSplitSize = 0;
			pSplitterStrm->Close();
			delete pSplitterStrm;
			pSplitterStrm = NULL;
			if( !aStrm.IsEof() )
			{
				ByteString aExtension( ByteString::CreateFromInt32(++nSplitPart) );
				while( aExtension.Len() < 3 )
					aExtension.Insert( "0", 0 );

				USHORT nPos = aMergeName.SearchBackward( '-' );
				ByteString aNewName( aMergeName.Copy(0, nPos+1) );
				aNewName += aExtension;
				aNewName += '.';
				aNewName += JUNK_EXTENSION;

				aNewEXE = aSplitDestinationPath;
				aNewEXE += aNewName;

				pSplitterStrm = new SvFileStream( aNewEXE.GetFullUni(), STREAM_WRITE | STREAM_TRUNC );

				aMsg = "split: ";
				aMsg += aNewEXE.GetFull();
				pLog->Log( aMsg );
			}
		}
	}

	if( pSplitterStrm )
	{
		pSplitterStrm->Close();
		delete pSplitterStrm;
	}
	aStrm.Close();

#ifdef UNX
	chown( aFirstEXE.GetFull().GetBuffer(), -1, gr_gid );
	chmod( aFirstEXE.GetFull().GetBuffer(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH );
#endif
}


void MergeFiles()
{
	ByteString aMsg( "merge all files to one (" );
	aMsg += aMergeFilename;
	aMsg += "):";

	ByteString aMergeName( aMergeFilename );
	aMergeName += '.';
	if( nTargetSystem == 2 )
		aMergeName += MASTER_EXTENSION_UNX;
	else
		aMergeName += MASTER_EXTENSION;

	pLog->NewLine();
	pLog->Log( aMsg, TRUE );
	pLog->NewLine();

	SiDirEntry aSetupEXE( aDestinationPath );
	if( nTargetSystem == 2 )
		aSetupEXE += ByteString("setup");
	else
		aSetupEXE += ByteString("setup.exe");

	// quick hack beta 5.2
	if( !aSetupEXE.Exists() )
	{
		aSetupEXE = aDestinationPath;
		if( nTargetSystem == 2 )
			aSetupEXE += ByteString("adabas");
		else
			aSetupEXE += ByteString("adabas.exe");
	}

	SiDirEntry aNewEXE( aMergeDestinationPath );
	aNewEXE += aMergeName;

	if( !aSetupEXE.Exists() )
	{
		aMsg = "error: setup (loader) binary  file '";
    	aMsg += aSetupEXE.GetFull();
	  	aMsg += "' cannot be found.";
		pLog->Log( aMsg );
		return ;
	}

	SvFileStream aNewStrm( aNewEXE.GetFullUni(), STREAM_WRITE | STREAM_TRUNC );
	if( nProcessorType == 1 )           // SPARC
		aNewStrm.SetNumberFormatInt( NUMBERFORMAT_INT_BIGENDIAN );

	SvFileStream aOldStrm( aSetupEXE.GetFullUni(), STREAM_READ );

	FileStat aStat( aSetupEXE );
	ULONG nSz = aStat.GetSize();

	// patch size
	char cBuf[_BUF_LEN];
	while( !aOldStrm.IsEof() )
	{
		memset( cBuf, 0, _BUF_LEN );
		ULONG nRead = aOldStrm.Read( cBuf, _BUF_LEN );

		for( USHORT i = 0; i < nRead; ++i )
			if( cBuf[i] == 'B' && (((ULONG)(i + 15)) <= nRead) )
				if( strncmp(&(cBuf[i]), "BIGFILE:", 8) == 0 )
				{
					ByteString aSz = ByteString::CreateFromInt32(nSz);
					strcpy( &(cBuf[i+8]), aSz.GetBuffer() );
				}
		aNewStrm.Write( cBuf, nRead );
	}
	aOldStrm.Close();
	//! aSetupEXE.Kill();

	SiDirEntry aDirEntry( aMergeDestinationPath );
	SiDirEntry aDataEntry( aMergeDestinationPath );

	aDirEntry	+= ByteString( "~dir.tmp" );
	aDataEntry	+= ByteString( "~data.tmp" );

	SvFileStream aDirStrm( aDirEntry.GetFullUni(), STREAM_READ | STREAM_WRITE | STREAM_TRUNC );
	if( nProcessorType == 1 )           // SPARC
		aDirStrm.SetNumberFormatInt( NUMBERFORMAT_INT_BIGENDIAN );

	SvFileStream aDataStrm( aDataEntry.GetFullUni(), STREAM_READ | STREAM_WRITE | STREAM_TRUNC );

	// copy files
	ULONG nOffset = 0;
	ULONG nDirCnt = 0;
	Dir aFiles(aDestinationPath, FSYS_KIND_FILE);
	for( USHORT i = 0; i < aFiles.Count(); ++i )
	{
		KillingAction();

		SiDirEntry aEntry(aFiles[i]);
		ByteString aFilename( aEntry.GetName() );
		if( aFilename.CompareIgnoreCaseToAscii(aMergeName)				!= COMPARE_EQUAL &&
			aFilename.CompareIgnoreCaseToAscii(aDirEntry.GetName())		!= COMPARE_EQUAL &&
			aFilename.CompareIgnoreCaseToAscii(aDataEntry.GetName())	!= COMPARE_EQUAL )
		{
			SvFileStream aFile( aEntry.GetFullUni(), STREAM_READ );
			FileStat aFSt( aEntry );

			aMsg = "merge ";
    		aMsg += aEntry.GetFull();
		  	aMsg += " (offset: ";
		  	aMsg += ByteString::CreateFromInt32( nOffset );
		  	aMsg += " size: ";
		  	aMsg += ByteString::CreateFromInt32( aFSt.GetSize() );
		  	aMsg += ")";
			pLog->Log( aMsg );

			aDirStrm	<< (ULONG) nOffset;
			aDirStrm	<< (ULONG) aFSt.GetSize();
			aDirStrm	<< aEntry.GetName().GetBuffer();
			aDirStrm	<< (char) 0x00;

			aDataStrm	<< aFile;

			nOffset += aFSt.GetSize();
			nDirCnt++;

			aFile.Close();
			//! aEntry.Kill();
		}
	}

	aDirStrm.Flush();
	aDataStrm.Flush();

	aDirStrm.Seek( STREAM_SEEK_TO_BEGIN );
	aDataStrm.Seek( STREAM_SEEK_TO_BEGIN );

	aNewStrm << (ULONG) 1;
	aNewStrm << (ULONG) nDirCnt;
	aNewStrm << aDirStrm;
	aNewStrm << aDataStrm;

	aNewStrm.Close();
	aDirStrm.Close();
	aDataStrm.Close();

	aDirEntry.Kill();
	aDataEntry.Kill();
#ifdef UNX
	chown( aNewEXE.GetFull().GetBuffer(), -1, gr_gid );
	chmod( aNewEXE.GetFull().GetBuffer(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH );
#endif
}

///////////////////////////////////////////////////////////////////////////////

void FillSubFiles( SiModule* pModule )
{
	const SiFileList& rFLst = pModule->GetFileList();
	for( USHORT i = 0; i < rFLst.Count(); ++i )
	{
		SiFile* pFile = rFLst.GetObject(i);

		SiDirEntry aPackednameSource( aDestinationPath );
		aPackednameSource += pFile->GetPackedName();
		if( pFile->IsArchive() )
		{
			SiZipFile aZipFile;
			aZipFile.AddSubfileListTo( aPackednameSource.GetFull(), pFile, "*" );
		}

		if( pFile->IsArchive() || pFile->IsPacked() )
		{
			SiZipFile aZipFile;
			ULONG nCRC = aZipFile.GetCRC32( aPackednameSource.GetFull() );
			pFile->SetProperty( "CRC", nCRC );
		}

		if( pFile->IsReadme() ||
			pFile->GetName().CompareIgnoreCaseToAscii( "readme.txt" ) == COMPARE_EQUAL ||
			pFile->GetName().CompareIgnoreCaseToAscii( "license.txt" ) == COMPARE_EQUAL ||
			pFile->GetName().CompareIgnoreCaseToAscii( "LICENSE" ) == COMPARE_EQUAL ||
			pFile->GetName().CompareIgnoreCaseToAscii( "README" ) == COMPARE_EQUAL )
		{
			SiDirEntry aSrc(aDestinationPath);
			aSrc += UniString( pFile->GetName(), osl_getThreadTextEncoding() );

			SiDirEntry aDest(aWebDestinationPath);
			aDest += UniString( pFile->GetName(), osl_getThreadTextEncoding() );

			FileCopier aCpy( aSrc, aDest );
			aCpy.Execute();
		}
	}

	const SiModuleList* pModuleList = pModule->GetModuleList();
	for( USHORT x = 0; x < pModuleList->Count(); ++x )
		FillSubFiles( pModuleList->GetObject(x) );
}

///////////////////////////////////////////////////////////////////////////////

int __LOADONCALLAPI main( int argv, char** argc )
{
	time_t start_time;
	time_t end_time;
	time( &start_time );

	fprintf( stderr, "\nScript Particel Zip (c) 2000 Sun Microsystems\n" );
	if( argv == 1 ) {
		PrintZipUsage();
		exit( -1 );
	}

#ifdef UNX
	//struct group* pgrp = getgrnam("installer");
	//gr_gid = pgrp->gr_gid;
#endif

	///////////////////////////////////////////////////////////////////////////
	// parameter parse & check
	ByteString	aErrorLogFilename;
	ByteString	aScriptFilename;
	ByteString	aIncParam;
	BOOL		bMerge2One = FALSE;
	BOOL		bSplitBig = FALSE;
	BOOL		bWebScript = FALSE;

	for( USHORT n = 1; n < argv; n++ )
	{
		if( argc[ n ][ 0 ] == '-' )
		{
			switch( argc[ n ][ 1 ] )
			{
				case 'u': bOldSetupZIPStyle = TRUE;
						  break;

				case 'w': bWebScript = TRUE;
						  break;

				case 'o': bTestOnly = TRUE;
						  break;

				case 'h': {
						  ByteString aTmp( argc[++n] );
						  if( aTmp.CompareIgnoreCaseToAscii("INTEL") == COMPARE_EQUAL )
							nProcessorType = 0;
						  else if( aTmp.CompareIgnoreCaseToAscii("SPARC") == COMPARE_EQUAL )
							nProcessorType = 1;
						  } break;

				case 'e': aErrorLogFilename = argc[ ++n ];
						  break;

				case 'p': aHTMLFilename = argc[ ++n ];
						  break;

				case 'i': aIncParam = argc[ ++n ];
						  break;

				case 'l': {
							ByteString aTmp( argc[ ++n ] );
							USHORT nDelimTok = 0;
							USHORT nTokCount = aTmp.GetTokenCount(PARA_DELIM);
							for( USHORT x = 0; x < nTokCount; ++x )
							{
								ByteString aLang = aTmp.GetToken(0, PARA_DELIM, nDelimTok);
								LangId* pNew = new LangId;
								pNew->nId = (USHORT)aLang.ToInt32();
								aLanguageList.Insert( pNew, LIST_APPEND );
							}
						  } break;

				case 'm':	aMergeFilename = argc[ ++n ];
							aMergeFilename.EraseLeadingChars('\"');
							aMergeFilename.EraseTrailingChars('\"');
							// aMergeFilename += "-000.";
							bMerge2One = TRUE;
							break;

				case 'z':   nSplitSize = ByteString(argc[++n]).ToInt32();
							bSplitBig = TRUE;
							break;

				case 'd':	aBasePath = argc[ ++n ];
							break;

				case 's':	aScriptFilename = argc[ ++n ];
							break;

				case 't': {
							ByteString aTmp = argc[ ++n ];
							if( aTmp.CompareIgnoreCaseToAscii("WNT") == COMPARE_EQUAL )
								nTargetSystem = 0;
							else if( aTmp.CompareIgnoreCaseToAscii("OS2") == COMPARE_EQUAL )
								nTargetSystem = 1;
							else if( aTmp.CompareIgnoreCaseToAscii("UNX") == COMPARE_EQUAL )
								nTargetSystem = 2;
							else
								nTargetSystem = USHRT_MAX;
						  } break;
				default: {
							USHORT comp = ByteString(argc[ n ][ 1 ]).ToInt32();
							if( comp >= 1 && comp <= 9 )
								nCompression = comp;
						 } break;
			}
		}
	}

	ByteString aTitle( "ZIP Protocol " );
	DateTime aNow;
	aTitle += ByteString::CreateFromInt32( aNow.GetDay() );
	aTitle += ".";
	aTitle += ByteString::CreateFromInt32( aNow.GetMonth() );
	aTitle += ".";
	aTitle += ByteString::CreateFromInt32( aNow.GetYear() );
	aTitle += " / ";
	aTitle += ByteString::CreateFromInt32( aNow.GetHour() );
	aTitle += ":";
	aTitle += ByteString::CreateFromInt32( aNow.GetMin() );
	aTitle += ":";
	aTitle += ByteString::CreateFromInt32( aNow.GetSec() );

	pLog = new Protocol( aTitle );
	pLog->DoConsole();

	if( !bTestOnly && aErrorLogFilename.Len() )
	{
		SiDirEntry aErrFile( aErrorLogFilename );
		aErrFile.GetPath().MakeDir();
		pLog->DoErrorFile( aErrorLogFilename );
	}

	if( !aLanguageList.Count() )
	{
		LangId* pNew = new LangId;
		pNew->nId = 1;
		aLanguageList.Insert( pNew, LIST_APPEND );
	}

	if( nTargetSystem == USHRT_MAX )
	{
		PrintZipUsage();
		pLog->Log( "error: wrong targetsystem" );
		delete pLog;
		exit( -1 );
	}

	if( !aBasePath.Len() ) {
		PrintZipUsage();
		pLog->Log( "error: no destinationpath" );
		delete pLog;
		exit( -1 );
	}

	if( !aScriptFilename.Len() ) {
		PrintZipUsage();
		pLog->Log( "error: no setup script" );
		delete pLog;
		exit( -1 );
	}


	///////////////////////////////////////////////////////////////////////////

	SiDirEntry aScriptFile( aScriptFilename );
	if( !aScriptFile.Exists() ) {
		ByteString aMsg( "error: source '");
		aMsg += aScriptFile.GetFull();
		aMsg += "' cannot be found.";
		pLog->Log( aMsg );
		delete pLog;
		exit( -1 );
	}
	else
	{
		ByteString aLog( "sourcescript: " );
		aLog += aScriptFile.GetFull();
		pLog->Log( aLog );
	}

	SiDirEntry aEntry( aBasePath );
	aEntry += ByteString(NORMAL_DIRNAME);
	aDestinationPath = aEntry.GetFull();

	if( !bTestOnly && !aEntry.Exists() )
		aEntry.MakeDir();

	if( !bTestOnly && bMerge2One )
	{
		aEntry.SetName( UniString::CreateFromAscii(MERGE_DIRNAME) );
		aEntry.MakeDir();
		aMergeDestinationPath = aEntry.GetFull();

		if( bSplitBig )
		{
			aEntry.SetName( UniString::CreateFromAscii(SPLIT_DIRNAME) );
			aEntry.MakeDir();
			aSplitDestinationPath = aEntry.GetFull();
		}
	}
	if( !bTestOnly && bWebScript )
	{
		aEntry.SetName( UniString::CreateFromAscii(WEB_DIRNAME) );
		aEntry.MakeDir();
		aWebDestinationPath = aEntry.GetFull();
	}

	SiAnsiFileStream aStrm;
    aStrm.Open( aScriptFile.GetFullUni(), STREAM_READ );
	if( !aStrm.IsOpen() ) {
		ByteString aMsg( "error: script '");
		aMsg += aScriptFile.GetFull();
		aMsg += "' cannot be open.";
		pLog->Log( aMsg );
		delete pLog;
		exit( -1 );
	}
	OSType eOS = nTargetSystem == 0 ? OSType_WIN :
				 nTargetSystem == 1 ? OSType_OS2 : OSType_UNIX_SOLS;

	///////////////////////////////////////////////////////////////////////////

	ByteString aDefLangStr;
	ByteString aAllLangStr;
	for( USHORT q = 0; q < aLanguageList.Count(); ++q )
	{
		ByteString n( ByteString::CreateFromInt32(aLanguageList.GetObject(q)->nId) );
		if( aAllLangStr.Search(n) == STRING_NOTFOUND )
		{
			if( n.Len() == 1 )
				n.Insert( "0", 0 );
			if( !q )
			{
				aDefLangStr = n;
				nDefLang = aDefLangStr.ToInt32();
			}
			else
				aAllLangStr += ',';
			aAllLangStr += n;
		}
	}

	SiCompiledScript*	pCS = new SiCompiledScript;
	SiCompilerRef		xCompiler = new SiCompiler( aStrm, eOS );

	xCompiler->NoApplication();
	xCompiler->SetDefDefaultLanguage( aDefLangStr );
	xCompiler->SetDefLanguages( aAllLangStr );

	if( !xCompiler->CompileTo( pCS ) ) {
		pLog->Log( "error: script error." );
		return -1;
	}
	aStrm.Close();

	//////////////////////////////////////////////
	// do main action
	if( !bTestOnly && aHTMLFilename.Len() )
	{
		SiDirEntry aHTMLFile( aHTMLFilename );
		aHTMLFile.GetPath().MakeDir();
		pLog->DoHTML( aHTMLFilename );
	}

	pLog->Log( aTitle, TRUE );
	pLog->NewLine();

	PreCacheIncludes( aIncParam );
	DoScpActions( pCS );
	ZipInstallation( pCS->GetRootModule() );

	if( bTestOnly ) exit( 0 );

	if( bOldSetupZIPStyle )
	{
		SiDirEntry aExtension(aDestinationPath);
		aExtension += ByteString("f_0000.zip");

		SiDirEntry aDestination(aExtension);
		aDestination.SetExtension( UniString::CreateFromAscii("") );
		aExtension.MoveTo(aDestination);
	}

	SiDirEntry aInstallationScript( aDestinationPath );
	if( nTargetSystem == 0 )
		aInstallationScript += ByteString("setup.inf");
	else
		aInstallationScript += ByteString("setup.ins");

	SiFileStream aOutScriptfile;
	aOutScriptfile.Open( aInstallationScript.GetFullUni(), STREAM_STD_WRITE | STREAM_TRUNC );
	SiDatabase aInstDB( aOutScriptfile );
	pCS->SetSecondLevel(TRUE);
	aInstDB.Create( pCS );
	aOutScriptfile.Close();

#ifdef UNX
	chown( aInstallationScript.GetFull().GetBuffer(), -1, gr_gid );
	chmod( aInstallationScript.GetFull().GetBuffer(), S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH );
#endif

	if( bMerge2One )
	{
		MergeFiles();
		if( bSplitBig )
			Splitter();
	}

	if( bWebScript )
	{
		FillSubFiles( pCS->GetRootModule() );

		SiDirEntry aWebScript( aWebDestinationPath );
		aWebScript += ByteString("webinstdb.inf");

		SiFileStream aOutScriptfile;
		aOutScriptfile.Open( aWebScript.GetFullUni(), STREAM_STD_WRITE | STREAM_TRUNC );

		SiDatabase aInstDB( aOutScriptfile );
		pCS->SetSecondLevel(TRUE);
		aInstDB.Create( pCS );

		aOutScriptfile.Close();
	}

	DeleteSearchCache();

	time( &end_time );
	time_t duration = end_time - start_time;
	struct tm* tm_duration = gmtime( &duration );

	ByteString aMsg( "\ntime needed: " );
	aMsg += ByteString::CreateFromInt32( tm_duration->tm_hour );
	aMsg += ":";
	aMsg += ByteString::CreateFromInt32( tm_duration->tm_min );
	aMsg += ":";
	aMsg += ByteString::CreateFromInt32( tm_duration->tm_sec );

	pLog->Log( aMsg );

	delete pCS;
	delete pLog;

	return 0;
}

