/*************************************************************************
 *
 *  $RCSfile: splcfg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:48:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#pragma hdrstop

#ifndef _SFX_HRC
#include <sfx2/sfx.hrc>
#endif
#ifndef _SFXAPP_HXX //autogen
#include <sfx2/app.hxx>
#endif
#ifndef NOOLDSV //autogen
#include <vcl/system.hxx>
#endif
#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif

#include "splcfg.hxx"
#include "cfgids.hxx"

#define SD_VERSION ((USHORT)91)


/*************************************************************************
|*
|* Ctor
|*
\************************************************************************/
SdSpellCheckCfg::SdSpellCheckCfg( const DirEntry& rHMDicPath,
								  const DirEntry& rUsrDicPath )
	:	SpellCheck		( rHMDicPath.GetFull(), rUsrDicPath ),
		SfxConfigItem	( SDCFG_SPELL )
{
}



/*************************************************************************
|*
|*
|*
\************************************************************************/
BOOL __EXPORT SdSpellCheckCfg::Store( SvStream& rStream )
{
	rStream << SD_VERSION;

	rStream << GetOptions();		// fuer SpellCheck Options

	USHORT nCount = GetDicCount();
	USHORT nAct   = 0;
	USHORT i;

	for ( i=0; i<nCount; ++i )
		if ( IsDicActive(i) )
			++nAct;

	rStream << nAct;

	for ( i=0; i<nCount; ++i)
	{
		if ( IsDicActive(i) )
		{
			rStream << GetDicName(i);   // Name rausschreiben

			// folgender Code wird noch gebraucht fuer alte
			// Version 1.x Dictionaries

			rStream << GetDicLanguage(i);	// Sprache
			rStream << IsDicNegativ(i);		// negativ
		}
	}

	// Default Sprache
	rStream << GetDefaultLanguage();

	return SfxConfigItem::ERR_OK;
}



/*************************************************************************
|*
|*
|*
\************************************************************************/
int	__EXPORT SdSpellCheckCfg::Load(SvStream& rStream)
{
	USHORT	nVersion;
	int		rc = SfxConfigItem::ERR_OK;

	SetDefault( FALSE );
	rStream >> nVersion;

	if( nVersion == SD_VERSION )
	{
		USHORT nOptions;
		USHORT nCount;
		USHORT nLang;
		USHORT i;

		rStream >> nOptions;
		SetOptions( nOptions );

		// fuer SpellCheck aktive Woerterbuecher einlesen
		rStream >> nCount;

		for ( i=0; i<nCount; ++i )
		{
			BOOL  	 bNeg;
			String	 aStrFilename;
			DirEntry aTmp( SFX_APP()->GetAppIniManager()->
								Get( SFX_KEY_DICTIONARY_PATH ) );

			rStream >> aStrFilename;
			aTmp += aStrFilename;
			rStream >> nLang;	// Sprache des Woerterbuchs
			rStream >> bNeg;	// Negativ-Flag

			if ( FileStat(aTmp).IsKind(FSYS_KIND_FILE) )
				SetActiveDic( NewDic( aStrFilename, nLang, bNeg ) );
		}

		// Default Sprache aus der Config einlesen
		rStream >> nLang;
		SetDefaultLanguage( nLang );
	}
	else
	{
		rc = SfxConfigItem::WARNING_VERSION;
	}

	return rc;
}



/*************************************************************************
|*
|*
|*
\************************************************************************/
void __EXPORT SdSpellCheckCfg::UseDefault()
{
	LanguageType eLanguage = Application::GetAppInternational().GetLanguage();

	if ( LANGUAGE_SYSTEM == eLanguage )
		eLanguage = System::GetLanguage();

	SetDefaultLanguage( (USHORT)eLanguage );

	SfxConfigItem::UseDefault();
}




