/*************************************************************************
 *
 *  $RCSfile: diadlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:48:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#pragma hdrstop

#ifdef VCL
#include <svtools/filedlg.hxx>
#endif
#ifndef _SFXITEMSET_HXX //autogen
#include <svtools/itemset.hxx>
#endif
#ifndef _SFXAPP_HXX //autogen
#include <sfx2/app.hxx>
#endif
#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif
#ifndef _SFX_IODLG_HXX
#include <sfx2/iodlg.hxx>
#endif
#ifndef _URLOBJ_HXX //autogen
#include <tools/urlobj.hxx>
#endif

#include "sdattr.hxx"
#include "dia_dlg.hxx"
#include "dia_dlg.hrc"
#include "app.hrc"
#include "sdresid.hxx"
#include "strings.hrc"
#include "res_bmp.hrc"
#include "fadedef.h"

/*************************************************************************
|*
\************************************************************************/

SdDiaDlg::SdDiaDlg( Window* pWindow, const SfxItemSet& rInAttrs  ) :
			ModalDialog     ( pWindow, SdResId( DLG_DIA ) ),
		aLbEffect       ( this, SdResId( LB_EFFECT ) ),
		aRbtSlow        ( this, SdResId( RBT_SLOW ) ),
		aRbtMedium      ( this, SdResId( RBT_MEDIUM ) ),
		aRbtFast        ( this, SdResId( RBT_FAST ) ),
		aGrpEffect      ( this, SdResId( GRP_EFFECT ) ),
		aRbtManuel      ( this, SdResId( RBT_MANUEL ) ),
		aRbtSemiAuto	( this, SdResId( RBT_SEMIAUTO ) ),
		aRbtAutomatic   ( this, SdResId( RBT_AUTOMATIC ) ),
		aTmeField       ( this, SdResId( TME_FIELD ) ),
		aGrpChange      ( this, SdResId( GRP_CHANGE ) ),
		aTsbSound       ( this, SdResId( TSB_SOUND ) ),
		aEdtSound       ( this, SdResId( EDT_SOUND ) ),
		aGrpSound       ( this, SdResId( GRP_SOUND ) ),
		aBtnSearch      ( this, SdResId( BTN_SEARCH ) ),
		aCtlPreview     ( this, SdResId( CTL_PREVIEW ) ),
		aBtnOk          ( this, SdResId( BTN_OK ) ),
		aBtnCancel      ( this, SdResId( BTN_CANCEL ) ),
		aBtnHelp        ( this, SdResId( BTN_HELP ) ),
		rOutAttrs		( rInAttrs )
{
	FreeResource();

	aLbEffect.SetSelectHdl( LINK( this, SdDiaDlg, ChangeEffectHdl ) );
	aRbtManuel.SetClickHdl( LINK( this, SdDiaDlg, ClickChangeHdl ) );
	aRbtSemiAuto.SetClickHdl( LINK( this, SdDiaDlg, ClickChangeHdl ) );
	aRbtAutomatic.SetClickHdl( LINK( this, SdDiaDlg, ClickChangeHdl ) );
	aTsbSound.SetClickHdl( LINK( this, SdDiaDlg, ClickSoundHdl ) );
	aBtnSearch.SetClickHdl( LINK( this, SdDiaDlg, ClickSearchHdl ) );

	aTmeField.SetModifyHdl( LINK( this, SdDiaDlg, ModifyTimeHdl ) );
	aTmeField.SetLoseFocusHdl( LINK( this, SdDiaDlg, LoseTimeHdl ) );
	aTmeField.SetGetFocusHdl( LINK( this, SdDiaDlg, GetTimeHdl ) );

	// Diese Befehle dienen als Workaround um einen Bug im TimeField.
	// Nach Behebung des Fehlers kann der direkt folgende Code entfernt werden
	aTmeField.SetMin( 0L );
	aTmeField.SetMax( Time( 12, 00, 00 ) );

	aCtlPreview.Hide();

	aLbEffect.Fill();

	ResetAttrs();
}

// -----------------------------------------------------------------------

BOOL SdDiaDlg::GetAttr( SfxItemSet& rOutAttrs )
{
	if( bEffectInvalide )
		rOutAttrs.InvalidateItem( ATTR_DIA_EFFECT );
	else
		rOutAttrs.Put( DiaEffectItem( (FadeEffect) aLbEffect.GetSelectEntryPos() ) );

	if( aRbtSlow.IsChecked() )
		rOutAttrs.Put( DiaSpeedItem( FADE_SPEED_SLOW ) );
	else if( aRbtMedium.IsChecked() )
		rOutAttrs.Put( DiaSpeedItem( FADE_SPEED_MEDIUM ) );
	else if( aRbtFast.IsChecked() )
		rOutAttrs.Put( DiaSpeedItem( FADE_SPEED_FAST ) );
	else
		rOutAttrs.InvalidateItem( ATTR_DIA_SPEED );

	if( aRbtAutomatic.IsChecked() )
		rOutAttrs.Put( DiaAutoItem( PRESCHANGE_AUTO ) );
	else if( aRbtSemiAuto.IsChecked() )
		rOutAttrs.Put( DiaAutoItem( PRESCHANGE_SEMIAUTO ) );
	else if( aRbtManuel.IsChecked() )
		rOutAttrs.Put( DiaAutoItem( PRESCHANGE_MANUAL ) );
	else
		rOutAttrs.InvalidateItem( ATTR_DIA_AUTO );


	if( bTimeInvalide )
		rOutAttrs.InvalidateItem( ATTR_DIA_TIME );
	else
	{
		Time aTime = aTmeField.GetTime();
		ULONG lTime = aTime.GetSec() + aTime.GetMin() * 60 + aTime.GetHour() * 3600;
		rOutAttrs.Put( DiaTimeItem( lTime ) );
	}


	if( aTsbSound.GetState() == STATE_DONTKNOW )
		rOutAttrs.InvalidateItem( ATTR_DIA_SOUND );
	else if( aTsbSound.GetState() == STATE_CHECK )
		rOutAttrs.Put( SfxBoolItem( ATTR_DIA_SOUND, TRUE ) );
	else
		rOutAttrs.Put( SfxBoolItem( ATTR_DIA_SOUND, FALSE ) );


	if( bSoundFileInvalide )
		rOutAttrs.InvalidateItem( ATTR_DIA_SOUNDFILE );
	else
		rOutAttrs.Put( SfxStringItem( ATTR_DIA_SOUNDFILE, aEdtSound.GetText() ) );

	return( TRUE );
}

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, ChangeEffectHdl, void *, EMPTYARG )
{
#ifdef TF_ONE51
	if( aLbEffect.GetSelectEntryPos() == FadeEffect_NONE )
#else
	if( aLbEffect.GetSelectEntryPos() == FADE_EFFECT_NONE )
#endif
	{
		aRbtSlow.Disable();
		aRbtMedium.Disable();
		aRbtFast.Disable();
	}
	else
	{
		aRbtSlow.Enable();
		aRbtMedium.Enable();
		aRbtFast.Enable();
	}
	bEffectInvalide = FALSE;

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, ClickChangeHdl, void *, EMPTYARG )
{
	if( aRbtManuel.IsChecked() )
		aTmeField.Disable();
	else
		aTmeField.Enable();

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, ClickSoundHdl, void *, EMPTYARG )
{
	if( aTsbSound.GetState() == STATE_NOCHECK)
	{
		aEdtSound.Disable();
		aBtnSearch.Disable();
	}
	else
	{
		aEdtSound.Enable();
		aBtnSearch.Enable();
	}
	bSoundFileInvalide = FALSE;

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, ClickSearchHdl, void *, EMPTYARG )
{
	pFileDialog = new SfxSimpleFileDialog (
						this, WB_OPEN | WB_3DLOOK | WB_STDMODAL );

	pBtnPreview = new PushButton( pFileDialog, SdResId( RID_PREVIEW_BUTTON ) );
	pBtnPreview->SetClickHdl( LINK( this, SdDiaDlg, ClickPreviewHdl ) );
	pBtnPreview->Show();

#if defined MAC
	String aDescr1;
	aDescr1 = String(SdResId(STR_SOUNDFILE1));
	pFileDialog->AddFilter (aDescr1, "*.snd", "sfil0");

	String aDescr2;
	aDescr2 = String(SdResId(STR_SOUNDFILE2));
	pFileDialog->AddFilter (aDescr2, "*.aif", "AIFF0");

	pFileDialog->SetDefaultExt("snd");
#elif defined UNX
   String aDescr;
   aDescr = String(SdResId(STR_AU_FILE));
   pFileDialog->AddFilter( aDescr, "*.au;*.snd" );
   aDescr = String(SdResId(STR_VOC_FILE));
   pFileDialog->AddFilter( aDescr, "*.voc" );
   aDescr = String(SdResId(STR_WAV_FILE));
   pFileDialog->AddFilter( aDescr, "*.wav" );
   aDescr = String(SdResId(STR_AIFF_FILE));
   pFileDialog->AddFilter( aDescr, "*.aiff" );
   aDescr = String(SdResId(STR_SVX_FILE));
   pFileDialog->AddFilter( aDescr, "*.svx" );

   pFileDialog->SetDefaultExt( "au" );
#else
	String aDescr;
	aDescr = String(SdResId(STR_WAV_FILE));
	pFileDialog->AddFilter (aDescr, "*.wav");
	aDescr = String(SdResId(STR_MIDI_FILE));
	pFileDialog->AddFilter (aDescr, "*.mid");

	pFileDialog->SetDefaultExt("wav");
#endif

	String aFile( aEdtSound.GetText() );
	if( !aFile.Len() )
	{
		aFile = SFX_APP()->GetAppIniManager()->Get( SFX_KEY_GRAPHICS_PATH );
	}
	pFileDialog->SetPath( aFile );

	if ( pFileDialog->Execute() )
	{
		aFile = pFileDialog->GetPath();
		aEdtSound.SetText( aFile );
	}

	if( aSound.IsPlaying() )
		aSound.Stop();

	delete pBtnPreview;
	delete pFileDialog;

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, ClickPreviewHdl, void *, EMPTYARG )
{
	if( pFileDialog )
	{
		if( aSound.IsPlaying() )
			aSound.Stop();
		else
		{
			const INetURLObject	aUrl; aUrl.SetSmartURL( pFileDialog->GetPath() );
			String				aSoundFile( aUrl.PathToFileName() ); 

			if( aSoundFile.Len() > 0 )
			{
				aSound.SetNotifyHdl( LINK( this, SdDiaDlg, StopMusicHdl ) );

				aSound.SetSoundName( aSoundFile );

#ifdef VCL
				aSound.Play();
#else
				aSound.Play( TRUE );
#endif

				if( aSound.IsPlaying() )
					pBtnPreview->SetText( SdResId( STR_STOP ) );
			}
		}
	}
	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, StopMusicHdl, void *, EMPTYARG )
{
	if( pBtnPreview )
		pBtnPreview->SetText( SdResId( STR_PLAY ) );

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK_INLINE_START( SdDiaDlg, ModifyTimeHdl, void *, EMPTYARG )
{
	bTimeInvalide = FALSE;

	return( 0L );
}
IMPL_LINK_INLINE_END( SdDiaDlg, ModifyTimeHdl, void *, EMPTYARG )

//------------------------------------------------------------------------

IMPL_LINK_INLINE_START( SdDiaDlg, LoseTimeHdl, void *, EMPTYARG )
{
	if( bTimeInvalide )
		aTmeField.SetText( "" );

	return( 0L );
}
IMPL_LINK_INLINE_END( SdDiaDlg, LoseTimeHdl, void *, EMPTYARG )

//------------------------------------------------------------------------

IMPL_LINK( SdDiaDlg, GetTimeHdl, void *, EMPTYARG )
{
	if( bTimeInvalide )
	{
		aTmeField.SetTime( Time( 0L ) );
		aTmeField.Reformat();
	}

	aTmeField.SetSelection( aTmeField.GetText().Len() );

	return( 0L );
}

//------------------------------------------------------------------------

void SdDiaDlg::ResetAttrs()
{
	FadeEffect eFE;
	FadeSpeed  eFS;

	if( rOutAttrs.GetItemState( ATTR_DIA_EFFECT ) != SFX_ITEM_DONTCARE )
	{
		eFE = (FadeEffect) ( ( const DiaEffectItem& ) rOutAttrs.Get( ATTR_DIA_EFFECT ) ).GetValue();
		aLbEffect.SelectEntryPos( eFE );
		bEffectInvalide = FALSE;
		ChangeEffectHdl( this );
	}
	else
		bEffectInvalide = TRUE;

	if( rOutAttrs.GetItemState( ATTR_DIA_SPEED ) != SFX_ITEM_DONTCARE )
	{
		eFS = (FadeSpeed) ( ( const DiaSpeedItem& ) rOutAttrs.Get( ATTR_DIA_SPEED ) ).GetValue();
		switch( eFS )
		{
			case FADE_SPEED_SLOW:   aRbtSlow.Check(); break;
			case FADE_SPEED_MEDIUM: aRbtMedium.Check(); break;
			case FADE_SPEED_FAST:   aRbtFast.Check(); break;
		}
	}

	if( rOutAttrs.GetItemState( ATTR_DIA_AUTO ) != SFX_ITEM_DONTCARE )
	{
		const PresChange ePresChange = (PresChange) ( ( const DiaAutoItem& ) rOutAttrs.Get( ATTR_DIA_AUTO ) ).GetValue();
		BOOL			 bMan = FALSE;
		BOOL			 bSemi = FALSE;
		BOOL			 bAuto = FALSE;

		switch( ePresChange )
		{
			case( PRESCHANGE_MANUAL ): bMan = TRUE; break;
			case( PRESCHANGE_SEMIAUTO ): bSemi = TRUE; break;
			case( PRESCHANGE_AUTO ): bAuto = TRUE; break;

			default:
			break;
		}

		aRbtAutomatic.Check( bAuto );
		aRbtSemiAuto.Check( bSemi );
		aRbtManuel.Check( bMan );
	}

	if( rOutAttrs.GetItemState( ATTR_DIA_TIME ) != SFX_ITEM_DONTCARE )
	{
		ULONG lTime = ( ( const DiaTimeItem& ) rOutAttrs.Get( ATTR_DIA_TIME ) ).GetValue();
		USHORT  nHour = (USHORT) ( lTime / 3600 );
		USHORT  nMin  = (USHORT) ( ( lTime % 3600 ) / 60 );
		USHORT  nSec  = (USHORT) ( ( lTime % 3600 ) % 60 );
		Time aTime( nHour, nMin, nSec);

		aTmeField.SetTime( aTime );

		bTimeInvalide = FALSE;
	}
	else
	{
		bTimeInvalide = TRUE;
		LoseTimeHdl( this );
	}

	if( rOutAttrs.GetItemState( ATTR_DIA_SOUND ) != SFX_ITEM_DONTCARE )
	{
		aTsbSound.EnableTriState( FALSE );
		if( ( ( const SfxBoolItem& ) rOutAttrs.Get( ATTR_DIA_SOUND ) ).GetValue() )
			aTsbSound.SetState( STATE_CHECK );
		else
			aTsbSound.SetState( STATE_NOCHECK );
	}
	else
		aTsbSound.SetState( STATE_DONTKNOW );

	if( rOutAttrs.GetItemState( ATTR_DIA_SOUNDFILE ) != SFX_ITEM_DONTCARE )
	{
		aEdtSound.SetText( ( ( const SfxStringItem& ) rOutAttrs.Get( ATTR_DIA_SOUNDFILE ) ).GetValue() );
		bSoundFileInvalide = FALSE;
	}
	else
		bSoundFileInvalide = TRUE;

	ClickSoundHdl( this );
	ClickChangeHdl( this );
}

/*************************************************************************

	  Source Code Control System - History

	  $Log: diadlg.cxx,v $
	  Revision 1.1.1.1  2000/09/18 16:48:32  hr
	  initial import
	
	  Revision 1.41  2000/09/17 15:23:34  willem.vandorp
	  OpenOffice header added.
	
	  Revision 1.40  2000/09/06 13:47:51  willem.vandorp
	  Header and footer replaced.
	
	  Revision 1.39  2000/08/31 12:25:52  ka
	  added decode mechanism to INetURLObject
	
	  Revision 1.38  2000/06/08 11:03:09  ka
	  DirEntry removed
	
	  Revision 1.37  2000/03/13 17:43:57  pw
	  Remove old SV-functionality
	
	  Revision 1.36  1999/01/20 16:52:08  CL
	  #52552# TF_ONE51 Umstellung
	

	  Rev 1.35   20 Jan 1999 17:52:08   CL
   #52552# TF_ONE51 Umstellung

	  Rev 1.34   26 Jun 1998 11:28:28   DL
   #51123# SoundFile kann File-URL sein

	  Rev 1.33   21 Jun 1998 18:13:16   KA
   neues Feature: Diawechsel halbautomatisch

	  Rev 1.32   28 Apr 1998 15:08:28   MI
   #49143# Alte FileDialoge ersetzt

	  Rev 1.31   14 Jan 1998 17:41:10   DL
   includes

	  Rev 1.30   09 Dec 1997 11:21:58   KA
   Anpassungen an VCL-Soundumstellungen

	  Rev 1.29   11 Aug 1997 08:46:38   TRI
   VCL: includes

	  Rev 1.28   29 Oct 1996 10:52:08   DL
   Neu: SdResId

	  Rev 1.27   26 Sep 1996 13:44:10   SB
   ClickSearchHdl: Def. Ext. ohne Wildcards; MAC: AddFilter mit 3 Parametern

	  Rev 1.26   05 Aug 1996 16:04:56   SOH
   #29765#,#29797#: Umstellung: SetCWD() -> FileDlg.SetPath()

	  Rev 1.25   24 Jul 1996 16:03:02   SOH
   Pfad fuer Sounds wird gesetzt (Graphic-Pfad)

	  Rev 1.24   10 Jul 1996 09:47:52   TRI
   sdattr.hxx included

	  Rev 1.23   13 Jun 1996 15:41:58   SOH
   ListBox -> FadeEffectLB

	  Rev 1.22   17 May 1996 15:20:00   mda
   Soundfiles unter Unix

	  Rev 1.21   11 Jan 1996 13:50:20   SOH
   #24185#: SoundHdl() wird immer in Reset() gerufen

	  Rev 1.20   08 Jan 1996 10:59:34   SOH
   Um SoundPreview erweitert

	  Rev 1.19   04 Dec 1995 13:32:24   DL
   Neuer Link

	  Rev 1.18   07 Nov 1995 16:15:32   TRI
   Change->Set

	  Rev 1.17   16 Oct 1995 14:44:44   SOH
   Disablen des PreviewControls; TriState ggfs. -> FALSE

	  Rev 1.16   30 Sep 1995 18:13:10   SB
   Midi-Filter im Dialog (Bugfix 20177)

	  Rev 1.15   08 Sep 1995 13:26:20   SB
   ClickSearchHdl: MAC-Anpassung

	  Rev 1.14   20 Jul 1995 13:13:36   SOH
   TimeField wird beim ersten Aufruf nicht auf Null gesetzt

	  Rev 1.13   28 Jun 1995 13:44:56   SB
   Strings fuer Dateifilter jetzt aus Resource

	  Rev 1.12   26 Jun 1995 18:54:32   SB
   neu: ClickSearchHdl()

	  Rev 1.11   23 Mar 1995 15:13:30   TRI
   Segmentierung

	  Rev 1.10   11 Jan 1995 13:00:48   SOH
   Sfx*Items auf Dia*Items geaendert

	  Rev 1.9   10 Jan 1995 12:25:08   SOH
   Bugfix: Time -> long

	  Rev 1.8   26 Oct 1994 09:53:58   SOH
   TimeField erweitert: Cursor steht beim Eintritt auf den Sekunden; leer, bei DONT-CARE-Status

	  Rev 1.7   25 Oct 1994 17:12:46   SOH
   Erweitert um DONT_CARE-Items

	  Rev 1.6   25 Oct 1994 10:26:50   SOH
   Neu: #pragma hdrstop

	  Rev 1.5   20 Oct 1994 16:00:16   SOH
   kleiner Bug beim Setzen des Manuell-Radiobuttons

	  Rev 1.4   20 Oct 1994 14:05:44   SOH
   Time-Handling geaendert (die 2te)

	  Rev 1.3   20 Oct 1994 14:01:18   SOH
   Time-Handling geaendert

	  Rev 1.2   20 Oct 1994 12:24:40   SOH
   Erweitert um Auto-Item

	  Rev 1.1   19 Oct 1994 14:22:06   SOH
   Umgestellt auf nicht-poolbare Items

	  Rev 1.0   13 Oct 1994 15:50:32   SOH
   Initial revision.

*************************************************************************/

#pragma SEG_EOFMODULE

