/*************************************************************************
 *
 *  $RCSfile: docdlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:48:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#pragma hdrstop
#include <svx/impgrf.hxx>

#include "docdlg.hxx"
#include "docdlg.hrc"
#include "sdresid.hxx"
#include "strings.hrc"

/*************************************************************************
|*
|*
|*
\************************************************************************/

SdOpenDialog::SdOpenDialog( Window* pParent, ULONG	nFlags,
							const SfxObjectFactory& rFact,
							const SfxItemSet& rSet ) :
			SfxFileDialog	( pParent, nFlags, rFact, rSet ),
			bOpen			( WB_OPEN == ( nFlags & WB_OPEN ) )
{
	pGraphicFilter = GetGrfFilter();
}

/*************************************************************************
|*
|*
|*
\************************************************************************/

SdOpenDialog::SdOpenDialog( Window* pParent, ULONG nFlags ) :
			SfxFileDialog	( pParent, nFlags ),
			bOpen			( WB_OPEN == ( nFlags & WB_OPEN ) )
{
	pGraphicFilter = GetGrfFilter();
}

/*************************************************************************
|*
|* Ctor I
|*
\************************************************************************/

SdSaveAsDialog::SdSaveAsDialog( Window* pParent, ULONG nFlags,
								const SfxObjectFactory& rFact,
								const SfxItemSet& rSet ) :
			SdOpenDialog		( pParent, nFlags, rFact, rSet ),
			pCbxOnlySelected	( NULL ),
			bShowSelectionBox	( FALSE )
{
	SetFilterSelectHdl( LINK( this, SdSaveAsDialog, SdSelectHdl ) );
}

/*************************************************************************
|*
|* Ctor II
|*
\************************************************************************/

SdSaveAsDialog::SdSaveAsDialog( Window* pParent, ULONG nFlags ) :
			SdOpenDialog		( pParent, nFlags ),
			pCbxOnlySelected	( NULL ),
			bShowSelectionBox	( SDWB_SELECTIONBOX == ( nFlags & SDWB_SELECTIONBOX ) )
{
	SetFilterSelectHdl( LINK( this, SdSaveAsDialog, SdSelectHdl ) );

#if defined( VCL ) && SUPD!=370

	if( bShowSelectionBox )
	{
		pCbxOnlySelected = new CheckBox( this, SdResId( CBX_ONLYSELECTED ) );
		AddControl( pCbxOnlySelected );
		pCbxOnlySelected->Show();
		SdSelectHdl( NULL );
	}

#endif
}


/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/
#pragma SEG_FUNCDEF(docdlg_06)

SdSaveAsDialog::~SdSaveAsDialog()
{
#if !defined( VCL ) || SUPD==370

	if( bShowSelectionBox )
		delete pCbxOnlySelected;

#endif
}


/*************************************************************************
|*
|* Diese Methode wird gerufen, um zusaetzliche Controls hinzuzufuegen. Da
|* die CheckBox ganz nach hinten soll (im Dialog) darf diese erst hier
|* erzeugt werden.
|*
\************************************************************************/

#if !defined( VCL ) || SUPD==370

void __EXPORT SdSaveAsDialog::AddControls( Window* pParent,
				SfxFileDialogTypes eTypes )
{
	SfxFileDialog::AddControls( pParent, eTypes );

	if ( bShowSelectionBox )
	{
		pCbxOnlySelected = new CheckBox( pParent, SdResId( CBX_ONLYSELECTED ) );
		pCbxOnlySelected->Show();

		SdSelectHdl( NULL );
	}
}

#endif

/*************************************************************************
|*
|*
|*
\************************************************************************/

IMPL_LINK( SdSaveAsDialog, SdSelectHdl, void*, pArg )
{
	if( bShowSelectionBox )
	{
		const String aFilter( GetCurFilter() );

		if ( !aFilter.Len() || ( aFilter == String( SdResId( STR_EXPORT_HTML_NAME ) ) ) )
			pCbxOnlySelected->Disable();
		else
			pCbxOnlySelected->Enable();
	}

	return 0L;
}


