/*************************************************************************
 *
 *  $RCSfile: drgrfob.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:48:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/whiter.hxx>
#include <svtools/itempool.hxx>
#ifndef _SFXAPP_HXX
#include <sfx2/app.hxx>
#endif
#ifndef _SFX_SHELL_HXX //autogen
#include <sfx2/shell.hxx>
#endif
#ifndef _SVXIDS_HRC
#include <svx/svxids.hrc>
#endif
#ifndef _SFXREQUEST_HXX //autogen
#include <sfx2/request.hxx>
#endif
#ifndef _SVDOPATH_HXX //autogen
#include <svx/svdograf.hxx>
#endif
#ifndef _AEITEM_HXX //autogen
#include <svtools/aeitem.hxx>
#endif

#pragma hdrstop

#include "app.hrc"
#include "res_bmp.hrc"
#include "glob.hrc"
#include "strings.hrc"
#include "docshell.hxx"
#include "viewshel.hxx"
#include "sdwindow.hxx"
#include "drawview.hxx"
#include "sdresid.hxx"
#include "fugrfflt.hxx"
#include "tbxitem.hxx"
#include "drawdoc.hxx"
#include "drgrfob.hxx"

#define ITEMVALUE(ItemSet,Id,Cast) ((const Cast&)(ItemSet).Get(Id)).GetValue()
#define SdDrawGrafObjectBar

SFX_DECL_TYPE( 13 );
#include "sdslots.hxx"

// -----------------------
// - SdDrawGrafObjectBar -
// -----------------------

SFX_IMPL_INTERFACE( SdDrawGrafObjectBar, SfxShell, SdResId( STR_GRAFOBJECTBARSHELL ) )
{
	SFX_OBJECTBAR_REGISTRATION( SFX_OBJECTBAR_OBJECT, SdResId( RID_DRAW_GRAF_TOOLBOX ) );
}

// -----------------------------------------------------------------------------

TYPEINIT1( SdDrawGrafObjectBar, SfxShell );

// -----------------------------------------------------------------------------

SdDrawGrafObjectBar::SdDrawGrafObjectBar( SdViewShell* pSdViewShell, SdView* pSdView ) :
	SfxShell			( pSdViewShell ),
	pView				( pSdView ),
	pViewSh				( pSdViewShell ),
	nMappedSlotFilter	( SID_GRAFFILTER_INVERT )
{
	SdDrawDocShell* pDocShell = pViewSh->GetDocSh();

	SetPool( &pDocShell->GetPool() );
	SetUndoManager( pDocShell->GetUndoManager() );
	SetRepeatTarget( pView );
	SetHelpId( SD_IF_SDDRAWGRAFOBJECTBAR );
	SetName( String( RTL_CONSTASCII_USTRINGPARAM( "Graphic objectbar" )));
}

// -----------------------------------------------------------------------------

SdDrawGrafObjectBar::~SdDrawGrafObjectBar()
{
	SetRepeatTarget( NULL );
}

// -----------------------------------------------------------------------------

void SdDrawGrafObjectBar::GetAttrState( SfxItemSet& rSet )
{
	SfxItemSet		aAttrSet( GetPool() );
	SfxWhichIter	aIter( rSet );
	USHORT			nWhich = aIter.FirstWhich();

	pView->GetAttributes( aAttrSet );

	while( nWhich )
	{
		USHORT nSlotId = SfxItemPool::IsWhich( nWhich )
							? GetPool().GetSlotId( nWhich )
							: nWhich;

		switch( nSlotId )
		{
			case( SID_ATTR_GRAF_MODE ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFMODE ) )
				{
					rSet.Put( SfxUInt16Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFMODE, SdrGrafModeItem ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_RED ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFRED ) )
				{
					rSet.Put( SfxInt16Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFRED, SdrGrafRedItem ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_GREEN ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFGREEN ) )
				{
					rSet.Put( SfxInt16Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFGREEN, SdrGrafGreenItem ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_BLUE ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFBLUE ) )
				{
					rSet.Put( SfxInt16Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFBLUE, SdrGrafBlueItem ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_LUMINANCE ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFLUMINANCE ) )
				{
					rSet.Put( SfxInt16Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFLUMINANCE, SdrGrafLuminanceItem ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_CONTRAST ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFCONTRAST ) )
				{
					rSet.Put( SfxInt16Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFCONTRAST, SdrGrafContrastItem ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_GAMMA ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFGAMMA ) )
				{
					rSet.Put( SfxUInt32Item( nSlotId,
						ITEMVALUE( aAttrSet, SDRATTR_GRAFGAMMA, SdrGrafGamma100Item ) ) );
				}
			}
			break;

			case( SID_ATTR_GRAF_TRANSPARENCE ):
			{
				if( SFX_ITEM_AVAILABLE <= aAttrSet.GetItemState( SDRATTR_GRAFTRANSPARENCE ) )
				{
					const SdrMarkList&	rMarkList = pView->GetMarkList();
					BOOL				bEnable = TRUE;

					for( USHORT i = 0, nCount = rMarkList.GetMarkCount();
							( i < nCount ) && bEnable; i++ )
					{
						SdrObject* pObj = rMarkList.GetMark( 0 )->GetObj();

						if( !pObj || !pObj->ISA( SdrGrafObj ) ||
							( (SdrGrafObj*) pObj )->HasGDIMetaFile() ||
							( (SdrGrafObj*) pObj )->IsAnimated() )
						{
							bEnable = FALSE;
						}
					}

					if( bEnable )
						rSet.Put( SfxUInt16Item( nSlotId,
							ITEMVALUE( aAttrSet, SDRATTR_GRAFTRANSPARENCE, SdrGrafTransparenceItem ) ) );
					else
						rSet.DisableItem( SID_ATTR_GRAF_TRANSPARENCE );
				}
			}
			break;

			case( SID_ATTR_GRAF_INVERT ):
			{
				rSet.Put( SfxBoolItem( SID_ATTR_GRAF_INVERT,
					ITEMVALUE( aAttrSet, SDRATTR_GRAFINVERT, SdrGrafInvertItem ) ) );
			}
			break;

			default:
			break;
		}

		nWhich = aIter.NextWhich();
	}
}

// -----------------------------------------------------------------------------

void SdDrawGrafObjectBar::Execute( SfxRequest& rReq )
{
	SfxItemSet aSet( GetPool(), SDRATTR_GRAF_FIRST, SDRATTR_GRAF_LAST-1 );

    const SfxItemSet *pArgs = rReq.GetArgs();
    const SfxPoolItem* pItem;
    USHORT nSlot = rReq.GetSlot();
    if( !pArgs || SFX_ITEM_SET != pArgs->GetItemState( nSlot, FALSE, &pItem ))
		pItem = 0;

	switch( nSlot )
	{
	case SID_ATTR_GRAF_RED:
		if( pItem )
			aSet.Put( SdrGrafRedItem( ((SfxInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_GREEN:
		if( pItem )
			aSet.Put( SdrGrafGreenItem( ((SfxInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_BLUE:
		if( pItem )
			aSet.Put( SdrGrafBlueItem( ((SfxInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_LUMINANCE:
		if( pItem )
			aSet.Put( SdrGrafLuminanceItem( ((SfxInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_CONTRAST:
		if( pItem )
			aSet.Put( SdrGrafContrastItem( ((SfxInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_GAMMA:
		if( pItem )
			aSet.Put( SdrGrafGamma100Item( ((SfxUInt32Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_TRANSPARENCE:
		if( pItem )
			aSet.Put( SdrGrafTransparenceItem( ((SfxUInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_MODE:
		if( pItem )
			aSet.Put( SdrGrafModeItem( (GraphicDrawMode)
									((SfxUInt16Item*)pItem)->GetValue() ));
		break;
	case SID_ATTR_GRAF_INVERT:
		if( pItem )
			aSet.Put( SdrGrafInvertItem( ((SfxBoolItem*)pItem)->GetValue() ));
		break;

	default:
		break;
	}
	if( aSet.Count() )
		pView->SetAttributes( aSet );

	Invalidate();
}

// -----------------------------------------------------------------------------

void SdDrawGrafObjectBar::GetFilterState( SfxItemSet& rSet )
{
	rSet.Put( TbxImageItem( SID_GRAFTBX_FILTERS, nMappedSlotFilter ) );

	const SdrMarkList&	rMarkList = pView->GetMarkList();
	BOOL				bEnable = FALSE;

	if( rMarkList.GetMarkCount() == 1 )
	{
		SdrObject* pObj = rMarkList.GetMark( 0 )->GetObj();

		if( pObj && pObj->ISA( SdrGrafObj ) && ( ( (SdrGrafObj*) pObj )->GetGraphicType() == GRAPHIC_BITMAP ) )
			bEnable = TRUE;
	}

	if( !bEnable )
	{
		rSet.DisableItem( SID_GRAFTBX_FILTERS );
		rSet.DisableItem( SID_GRAFFILTER_INVERT );
		rSet.DisableItem( SID_GRAFFILTER_SMOOTH );
		rSet.DisableItem( SID_GRAFFILTER_SHARPEN );
		rSet.DisableItem( SID_GRAFFILTER_REMOVENOISE );
		rSet.DisableItem( SID_GRAFFILTER_SOBEL );
		rSet.DisableItem( SID_GRAFFILTER_MOSAIC );
		rSet.DisableItem( SID_GRAFFILTER_EMBOSS );
		rSet.DisableItem( SID_GRAFFILTER_POSTER );
		rSet.DisableItem( SID_GRAFFILTER_POPART );
		rSet.DisableItem( SID_GRAFFILTER_SEPIA );
		rSet.DisableItem( SID_GRAFFILTER_SOLARIZE );
	}
}

// -----------------------------------------------------------------------------

void SdDrawGrafObjectBar::ExecuteFilter( SfxRequest& rReq )
{
	switch( rReq.GetSlot() )
	{
		case( SID_GRAFTBX_FILTERS ):
		{
			SfxRequest aReq( nMappedSlotFilter, 0, pView->GetDoc()->GetItemPool() );
			ExecuteSlot( aReq );
		}
		break;

		case( SID_GRAFFILTER_INVERT ):
		case( SID_GRAFFILTER_SMOOTH ):
		case( SID_GRAFFILTER_SHARPEN ):
		case( SID_GRAFFILTER_REMOVENOISE ):
		case( SID_GRAFFILTER_SOBEL ):
		case( SID_GRAFFILTER_MOSAIC ):
		case( SID_GRAFFILTER_EMBOSS ):
		case( SID_GRAFFILTER_POSTER ):
		case( SID_GRAFFILTER_POPART ):
		case( SID_GRAFFILTER_SEPIA ):
		case( SID_GRAFFILTER_SOLARIZE ):
		{
			FuPoor* pFuActual = new FuGrafFilter( pViewSh, (SdWindow*) pViewSh->GetWindow(),
												  pView, pView->GetDoc(), rReq );

			pFuActual->Activate();
			pFuActual->Deactivate();
			delete pFuActual;
			nMappedSlotFilter = rReq.GetSlot();
		}
		break;

		default:
		break;
	}

	Invalidate();
}

