/*************************************************************************
 *
 *  $RCSfile: slidview.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:48:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_WRKWIN_HXX
#include <vcl/wrkwin.hxx>
#endif
#ifndef _VIEW3D_HXX
#include <svx/view3d.hxx>
#endif
#ifndef _SFXAPP_HXX
#include <sfx2/app.hxx>
#endif
#ifndef _SFXDISPATCH_HXX //autogen
#include <sfx2/dispatch.hxx>
#endif
#ifndef _SFXINTITEM_HXX //autogen
#include <svtools/intitem.hxx>
#endif

#pragma hdrstop

#include "docshell.hxx"
#include "slidview.hxx"
#include "slidvish.hxx"
#include "viewshel.hxx"
#include "frmview.hxx"
#include "sdpage.hxx"
#include "pres.hxx"
#include "drawdoc.hxx"
#include "sdresid.hxx"
#include "bmcache.hxx"
#include "showview.hxx"
#include "glob.hrc"
#include "fuslsel.hxx"
#include "dragserv.hxx"
#include "sdmod.hxx"
#include "sdresid.hxx"
#include "strings.hrc"
#include "app.hrc"

#ifndef _SV_POLY_HXX //autogen
#include <vcl/poly.hxx>
#endif
#ifndef _SVDPAGV_HXX //autogen
#include <svx/svdpagv.hxx>
#endif
#ifndef _SVDUNDO_HXX //autogen
#include <svx/svdundo.hxx>
#endif
#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif

using namespace ::com::sun::star;

#define NAME_HEIGHT_FRACTION  (ULONG)	   25
#define NAME_OFFSET_FRACTION  (ULONG)		4
#define SELRECT_OFFSET		  (ULONG)		2
#define SELRECT_THICKNESS	  (ULONG)		2
#define BITMAPCACHE_SIZE	  (ULONG) 2097152
#define ZOOM_TOLERANCE		  (long)	  -10
#define DELAYED_PAINT_TIMEOUT (ULONG)	   50

typedef struct							// Struktur fuer verzoegertes Zeichnen
{
	OutputDevice* pOut;
	Rectangle	  aRect;
} DelayedPaintEvent;

TYPEINIT1( SdSlideView, SdView );

/*************************************************************************
|*
|* der Konstruktor setzt den MapMode und arrangiert die einzelnen Seiten
|*
\************************************************************************/

SdSlideView::SdSlideView(SdDrawDocument* pDoc, Window* pWindow,
						 SdSlideViewShell* pSlideVShell)
		   : SdView(pDoc, pWindow, pSlideVShell),
			 nAllowInvalidateSmph(0),
			 nPagesPerRow(4),
			 pCache(NULL),
			 pShowView(NULL),
			 pVDev(NULL),
			 bInDelayedPaint(FALSE),
			 pSlideViewShell(pSlideVShell)

{
	ArrangePages();

	aDelayedPaintTimer.SetTimeout(DELAYED_PAINT_TIMEOUT);
	aDelayedPaintTimer.SetTimeoutHdl(LINK(this,SdSlideView,PaintDelayed));
	aDelayedPaintTimer.Start();

	SetAnimationEnabled( FALSE );
}

/*************************************************************************
|*
|* Destruktor
|*
\************************************************************************/

SdSlideView::~SdSlideView()
{
	delete pCache;
	delete pShowView;
	delete pVDev;
	while (aDelayedPaints.Count() > 0)
	{
		delete (DelayedPaintEvent*)aDelayedPaints.Remove((ULONG)0);
	}
	aDelayedPaintTimer.Stop();
}

/*************************************************************************
|*
|* aendert die Anzahl der Seiten pro Reihe, danach ist ein Aufruf von
|* ArrangePages() noetig
|*
\************************************************************************/

USHORT SdSlideView::ChangePagesPerRow(USHORT nNum)
{
	USHORT nResult = nPagesPerRow;
	nPagesPerRow   = nNum;
	return nResult;
}

/*************************************************************************
|*
|* gibt den Freiraum zwischen den Seiten zurueck (z. Zt. 1/8 der Seitenbreite)
|*
\************************************************************************/

ULONG SdSlideView::GetPageGap() const
{
	ULONG nResult = 0;
	if (pDoc->GetSdPageCount(PK_STANDARD) > 0)
	{
		nResult = pDoc->GetSdPage(0, PK_STANDARD)->GetSize().Width() / 8;
	}
	return nResult;
}

/*************************************************************************
|*
|* Invalidate abfangen
|*
\************************************************************************/

void __EXPORT SdSlideView::InvalidateOneWin(Window& rWin)
{
	if (IsInvalidateAllowed())
	{
		SdrPaintView::InvalidateOneWin(rWin);
	}
}

/*************************************************************************
|*
|* Invalidate abfangen
|*
\************************************************************************/

void __EXPORT SdSlideView::InvalidateOneWin(Window& rWin, const Rectangle& rRect)
{
	if (IsInvalidateAllowed())
	{
		SdrPaintView::InvalidateOneWin(rWin, rRect);
	}
}

/*************************************************************************
|*
|* die Paint-Methode
|*
\************************************************************************/

void __EXPORT SdSlideView::Paint(const Rectangle& rRect, OutputDevice* pOut)
{
	const Color& rOldLineColor = pOut->GetLineColor();
	const Color& rOldFillColor = pOut->GetFillColor();

	SdPage* pPage	   = 0;
	USHORT	nNoOfPages = pDoc->GetSdPageCount(PK_STANDARD);
	USHORT	nPage;
	Point	aPos;					// Position fuer Name
	Font	aOldFont   = pOut->GetFont();
	Font	aNewFont;

	aNewFont.SetFamily(FAMILY_SWISS);

	if (nNoOfPages > 0)
	{
		pPage = pDoc->GetSdPage(0L, PK_STANDARD);
		Size aPageSize	= pPage->GetSize();
		Size aPixelSize = pOut->PixelToLogic(Size(1,1));
		Size aCPageSize(aPageSize.Width() + aPixelSize.Width(),
						aPageSize.Height() + aPixelSize.Height());

		Size aFontSize (0, pPage->GetSize().Height() / NAME_HEIGHT_FRACTION);
		aNewFont.SetSize(aFontSize);
		aNewFont.SetTransparent(TRUE);
		pOut->SetFont(aNewFont);

		if (pShowView)
		{
			// ShowView und VDev existieren schon
			MapMode aMapMode(pOut->GetMapMode());
			aMapMode.SetOrigin(Point(0,0));
			pVDev->SetMapMode(aMapMode);
			pVDev->SetOutputSize(aCPageSize);
			pShowView->HideAllPages();
		}

		for (nPage = 0; nPage < nNoOfPages; nPage++)
		{
			// Seitenbereich getroffen?
			if (rRect.IsOver(GetPageArea(nPage)))
			{
				pPage = pDoc->GetSdPage(nPage, PK_STANDARD);
				Rectangle aPageRect(CalcPagePos(nPage), aPageSize);

				// die Seite selbst getroffen
				if (rRect.IsOver(aPageRect))
				{
					Fraction aScale = pOut->GetMapMode().GetScaleX();
					long	nZoom	= aScale.GetNumerator() * 100L /
									  aScale.GetDenominator();

					// Bitmap im Cache?
					long  nZoomOfBitmap = nZoom;
					const Bitmap* pBitmap = GetFromCache(pPage,
														 nZoomOfBitmap,
														 ZOOM_TOLERANCE);

					// die Seite selbst zeichnen?
					if (bInDelayedPaint || pBitmap != NULL)
					{
						// nicht im Cache, erzeugen und in Cache stellen
						if (pBitmap == NULL)
						{

							// wenn noetig, virtuelles Device und View erzeugen
							if (pShowView == NULL)
							{
								pVDev = new VirtualDevice(*pOut);
								MapMode aMapMode(pOut->GetMapMode());
								aMapMode.SetOrigin(Point(0,0));
								pVDev->SetMapMode(aMapMode);
								pVDev->SetOutputSize(aCPageSize);
								pShowView = new ShowView(pDoc, pVDev,
														 pSlideViewShell);

								pShowView->SetLineDraft( IsLineDraft() );
								pShowView->SetFillDraft( IsFillDraft() );
								pShowView->SetTextDraft( IsTextDraft() );
								pShowView->SetGrafDraft( IsGrafDraft() );
							}

							pShowView->ShowPage(pPage, Point(0,0));

							SdrPageView* pPageView = pShowView->GetPageView(pPage);
							FrameView* pFrameView = pSlideViewShell->GetFrameView();
							pPageView->SetVisibleLayers( pFrameView->GetVisibleLayers() );

							Point aPoint(0,0);
							Region aRegion(Rectangle(aPoint, aCPageSize));
							pShowView->InitRedraw(pVDev,aRegion);
							while (!pShowView->IsRedrawReady()) {}

							pBitmap = new Bitmap(pVDev->GetBitmap(
														Point(0,0),aCPageSize));
							AddToCache(pPage, (Bitmap*)pBitmap, nZoom);
						}

						if (nZoomOfBitmap == nZoom)
						{
							pOut->DrawBitmap(aPageRect.TopLeft(), *pBitmap);
						}
						else
						{
							pOut->DrawBitmap(aPageRect.TopLeft(), aCPageSize,
											 *pBitmap);
						}
					}
					// spaeter zeichnen (kommt Zeit, kommt Paint)
					else
					{
						DelayedPaintEvent* pEvent = new DelayedPaintEvent;
						pEvent->aRect = aPageRect.GetIntersection(rRect);

						pEvent->aRect.Right() += aPixelSize.Width();
						pEvent->aRect.Bottom() += aPixelSize.Height();

						pEvent->pOut  = pOut;
						aDelayedPaints.Insert(pEvent, LIST_APPEND);
					}

					// draw border around page (in every case)
					pOut->SetLineColor( Color( COL_BLACK ) );
					pOut->SetFillColor();
					pOut->DrawRect(aPageRect);
				}

				if (!bInDelayedPaint)
				{
					// Symbol fuer Ueberblendeffekt
					Rectangle aRect = GetFadeIconArea(nPage);
					if (pPage->GetFadeEffect() != presentation::FadeEffect_NONE)

					{
						pOut->SetLineColor( Color( COL_BLACK ) );
						pOut->SetFillColor();
						pOut->DrawRect(aRect);

						pOut->SetLineColor();
						pOut->SetFillColor( Color( COL_BLACK ) );

						Point aTriangle[3];
						aTriangle[0] = aRect.TopLeft();
						aTriangle[1] = aRect.Center();
						aTriangle[2] = aRect.BottomLeft();
						pOut->DrawPolygon(Polygon(3, aTriangle));
					}

					// Name der Seite
					aPos	  = CalcPagePos(nPage);
					Size aPageSize = pPage->GetSize();
					Size aSize(0, SELRECT_OFFSET + SELRECT_THICKNESS + 2);
					aSize = pOut->PixelToLogic(aSize);
					aPos.Y() += aPageSize.Height() + aSize.Height();


					String aName (pPage->GetName());

					if ( aName.Len() == 0 )
					{
						String aStandardName( SdResId(STR_PAGE) );
						aName = aStandardName;
						aName += String::CreateFromInt32( nPage + 1 );
					}

					if (pOut->GetTextWidth(aName) > aPageSize.Width() - 2 * aRect.GetWidth())
					{
						aName.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "..." ));
						while (pOut->GetTextWidth(aName) >
							   aPageSize.Width() - 2 * aRect.GetWidth())
						{
							aName.Erase(aName.Len() - 4, 1);
						}
					}

					// rechtsbuendig unter der Seite
					aPos.X() += aPageSize.Width() - pOut->GetTextWidth(aName);

					// grau hinterlegen, wenn Seite ausgeschlossen
					if (pPage->IsExcluded())
					{
						Size aTextSize	= Size( pOut->GetTextWidth(aName), pOut->GetTextHeight() );

						pOut->SetFillColor( Color( COL_LIGHTGRAY ) );
						pOut->SetLineColor();
						pOut->DrawRect(Rectangle(aPos, aTextSize));
					}
					pOut->DrawText(aPos, aName);

					// Selektionskennung
					DrawSelectionRect (nPage);
				}
			}
		}
	}
	pOut->SetFont(aOldFont);

	pOut->SetFillColor( rOldLineColor );
	pOut->SetFillColor( rOldFillColor );
}

/*************************************************************************
|*
|* zeichnet Selektionsrahmen um die Seite nPage oder entfernt ihn
|* wieder (bShow == FALSE)
|*
\************************************************************************/

void SdSlideView::DrawSelectionRect(USHORT nPage)
{
	pSlideViewShell->DrawSelectionRect(nPage);

}

/*************************************************************************
|*
|* berechnet den Ursprung fuer die Seite nPageNo auf dem "Diatisch"
|*
\************************************************************************/

Point SdSlideView::CalcPagePos(USHORT nPageNo) const
{
	Point  aResult (0, 0);

	if (pDoc->GetSdPageCount(PK_STANDARD) > 0)
	{
		Size   aPageSize = pDoc->GetSdPage(0, PK_STANDARD)->GetSize();

		// Freiraum zwischen Seiten in x und y
		ULONG nGapX = GetPageGap();
		ULONG nGapY = nGapX;

		// sicherstellen, dass die Luecke gross genug ist um Selektionsrechteck,
		// Titel und Ueberblendsymbol aufzunehmen
		OutputDevice* pOut = (OutputDevice*)pSlideViewShell->GetActiveWindow();
		Size aSize(0, SELRECT_OFFSET + SELRECT_THICKNESS + 2);
		aSize = pOut->PixelToLogic(aSize);
		aSize.Height() += aPageSize.Height() / NAME_HEIGHT_FRACTION;
		if (nGapY < aSize.Height())
		{
			nGapY = aSize.Height() * 2;
		}

		aResult.X() = nPageNo % nPagesPerRow * (nGapX + aPageSize.Width()) +
					  nGapX;
		aResult.Y() = nPageNo / nPagesPerRow * (nGapY + aPageSize.Height()) +
					  nGapY;
	}
	return (aResult);
}

/*************************************************************************
|*
|* ordnet Seiten auf dem "Diatisch" an, ohne Redraw
|*
\************************************************************************/

void SdSlideView::ArrangePages()
{
	USHORT nNoOfPages = pDoc->GetSdPageCount(PK_STANDARD);
	USHORT nPage;
	Point aOffset;

	if (nNoOfPages > 0)
	{
		for (nPage = 0; nPage < nNoOfPages; nPage++)
		{
			SetAllowInvalidate(FALSE);
			aOffset = CalcPagePos(nPage);
			SdrPage* pPage = pDoc->GetSdPage(nPage, PK_STANDARD);
			SdrPageView* pPageView = GetPageView(pPage);
			if (pPageView == NULL)
			{
				ShowPage(pPage, aOffset);

				SdrPageView* pPageView = GetPageView(pPage);
				FrameView* pFrameView = pSlideViewShell->GetFrameView();
				pPageView->SetVisibleLayers( pFrameView->GetVisibleLayers() );
			}
			else
			{
				if (pPageView->GetOffset() != aOffset)
					SetPagePos(pPageView, aOffset);
			}
			SetAllowInvalidate(TRUE);
		}
	}
}


/*************************************************************************
|*
|* gibt die von einer Seite beanspruchte Flaeche auf dem Diatisch zurueck
|* (dazu gehoert auch der Platz fuer den Seitennamen, Selektionsrahmen etc.)
|*
\************************************************************************/

Rectangle SdSlideView::GetPageArea(USHORT nPageNo) const
{
	Size aSize(pDoc->GetSdPage(nPageNo, PK_STANDARD)->GetSize());
	Point aPos(CalcPagePos(nPageNo));

	aSize.Height() += aSize.Height() / NAME_HEIGHT_FRACTION +
					  aSize.Height() / NAME_HEIGHT_FRACTION /
									   NAME_OFFSET_FRACTION;

	Rectangle aResult(aPos, aSize);

	// dazu auf jeder Seite SELRECT_OFFSET Pixel Abstand + SELRECT_THICKNESS
	// Pixel Selektionsrahmen + 2 Pixel Toleranz
	OutputDevice* pOut = (OutputDevice*)pSlideViewShell->GetActiveWindow();
	Size aPixelSize = pOut->PixelToLogic(Size(1,1));
	USHORT nFactor = SELRECT_OFFSET + SELRECT_THICKNESS + 2;
	aResult.Left()		-= nFactor * aPixelSize.Width();
	aResult.Right() 	+= nFactor * aPixelSize.Width();
	aResult.Top()		-= nFactor * aPixelSize.Height();
	aResult.Bottom()	+= nFactor * aPixelSize.Height();

	return (aResult);
}

/*************************************************************************
|*
|* gibt die Flaeche eines Diawechsel-Icons zurueck
|*
\************************************************************************/

Rectangle SdSlideView::GetFadeIconArea(USHORT nPageNo) const
{
	SdPage* pPage	  = pDoc->GetSdPage(nPageNo, PK_STANDARD);
	Size	aPageSize = pPage->GetSize();
	Point	aPos	  = CalcPagePos(nPageNo);

	OutputDevice* pOut = (OutputDevice*)pSlideViewShell->GetActiveWindow();
	Size aSize(0, SELRECT_OFFSET + SELRECT_THICKNESS + 2);
	aSize = pOut->PixelToLogic(aSize);

	aPos.Y() += aPageSize.Height() + aSize.Height();

	return Rectangle (aPos,
					  Size(aPageSize.Height() / NAME_HEIGHT_FRACTION,
						   aPageSize.Height() / NAME_HEIGHT_FRACTION));
}

/*************************************************************************
|*
|* fuegt eine Bitmap in den Cache ein
|*
\************************************************************************/

void SdSlideView::AddToCache(SdPage* pPage, Bitmap* pBitmap, long nZoom)
{
	if (pCache == NULL) pCache = new BitmapCache(BITMAPCACHE_SIZE);
	pCache->Add(pPage, pBitmap, nZoom);
}

/*************************************************************************
|*
|* sucht eine Bitmap im Cache (gibt NULL zurueck, wenn nichts gefunden wurde)
|* - auf rZoom wird der Zoomfaktor der Bitmap geschrieben
|* - abs(nZoomTolerance) gibt die maximal erlaubte Abweichung des
|*	 Zoomfaktors an; ist die Toleranz negativ, so wird der "best fit" gesucht,
|*	 sonst den "first fit"
|*
\************************************************************************/

const Bitmap* SdSlideView::GetFromCache(SdPage* pPage, long& rZoom,
										long nZoomTolerance) const
{
	const Bitmap* pBitmap = NULL;
	if (pCache)
	{
		pBitmap = pCache->Get(pPage, rZoom, nZoomTolerance);
	}
	return pBitmap;
}


/*************************************************************************
|*
|* Handler fuer verzoegertes Zeichnen
|*
\************************************************************************/

IMPL_LINK( SdSlideView, PaintDelayed, Timer *, pTimer )
{
	do
	{
		if (aDelayedPaints.Count() > 0)
		{

			FuPoor* pFu = pSlideViewShell->GetActualFunction();
			if (!pFu									||
				(pFu && !pFu->ISA(FuSlideSelection))	||
				(pFu && pFu->ISA(FuSlideSelection) &&
				!((FuSlideSelection*)pFu)->IsShowingEffect()))
			{
				DelayedPaintEvent* pEvent = (DelayedPaintEvent*)
											 aDelayedPaints.Remove((ULONG)0);
				bInDelayedPaint 		  = TRUE;

				if (OUTDEV_WINDOW == pEvent->pOut->GetOutDevType())
				{
					((Window*)pEvent->pOut)->Invalidate(pEvent->aRect);
					((Window*)pEvent->pOut)->Update();
				}
				else
				{
					Paint(pEvent->aRect, pEvent->pOut);
				}

				bInDelayedPaint 		  = FALSE;
				delete pEvent;
			}
		}
	}
	while (!GetpApp()->AnyInput() && aDelayedPaints.Count() > 0);

	pTimer->Start();
	return 0;
}

/*************************************************************************
|*
|* Notify fuer Aenderung der Seitenanordnung
|*
\************************************************************************/

void __EXPORT SdSlideView::SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType,
							   const SfxHint& rHint, const TypeId& rHintType)
{
	SdrHint* pSdrHint = PTR_CAST(SdrHint, &rHint);

	if(pSdrHint)
	{
		SdrHintKind eHint = pSdrHint->GetKind();

		if(eHint == HINT_PAGEORDERCHG)
		{
			ArrangePages();
			InvalidateAllWin();
		}
		else
		{
			const SdrPage* pChangedPage = pSdrHint->GetPage();

			if(pChangedPage)
			{
				if(pChangedPage->IsMasterPage())
				{
					if(pCache)
					{
						delete pCache;
						pCache = NULL;
					}
				}
				else
				{
					SdrPageView* pPageView = GetPageView(pChangedPage);

					if(pPageView)
					{
						if(pCache)
						{
							pCache->Remove((const SdPage*)pChangedPage);
						}
					}
				}
			}
		}
	}

	FmFormView::SFX_NOTIFY(rBC, rBCType, rHint, rHintType);
}

/*************************************************************************
|*
|* Zaehler fuer Invalidierungserlaubnis hoch- oder runter zaehlen
|*
\************************************************************************/

void SdSlideView::SetAllowInvalidate(BOOL bFlag)
{
	if (!bFlag)
	{
		nAllowInvalidateSmph++;
	}
	else if (nAllowInvalidateSmph > 0)
	{
		nAllowInvalidateSmph--;
	}
}

/*************************************************************************
|*
|* ermittelt, ob invalidiert werden darf
|*
\************************************************************************/

BOOL SdSlideView::IsInvalidateAllowed() const
{
	return (nAllowInvalidateSmph == 0);
}


/*************************************************************************
|*
|* Cut object to clipboard
|*
\************************************************************************/

void SdSlideView::DoCut(Window* pWindow)
{
	String aString(SdResId(STR_WARN_DEL_SEL_PAGES));

	if (pDoc->GetSdPageCount(PK_STANDARD) > 1 &&
		QueryBox(pWindow, WB_YES_NO, aString).Execute() == RET_YES)
	{
		List aBookmarkList;
		SdPage* pPage = NULL;
		USHORT nPgCnt = pDoc->GetSdPageCount(PK_STANDARD);

		for (USHORT nPage = 0; nPage < nPgCnt; nPage++)
		{
			pPage = pDoc->GetSdPage(nPage, PK_STANDARD);

			if (pPage->IsSelected())
			{
				aBookmarkList.Insert(new String(pPage->GetName()), LIST_APPEND);
			}
		}

		if (aBookmarkList.Count() != 0)
		{
			BrkAction();
			SdDataObjectRef pDataObject = CreateDataObject();

			SdDrawDocument* pDataDoc = pDataObject->pSdDrawDocument;
			pDataObject->ResetPageView();
			pDataDoc->Clear();
			pDataDoc->CreateFirstPages();
			pDataDoc->InsertBookmarkAsPage(&aBookmarkList, NULL, FALSE, TRUE, 1,
										   TRUE, pDocSh, TRUE, TRUE);
			pDataObject->UpdatePageView();

			DeleteMarked();

			String* pString = (String*) aBookmarkList.First();
			while( pString )
			{
				delete pString;
				pString = (String*) aBookmarkList.Next();
			}
		}
	}
}

/*************************************************************************
|*
|* Copy object to clipboard
|*
\************************************************************************/

void SdSlideView::DoCopy(Window* pWindow)
{
	List aBookmarkList;
	SdPage* pPage = NULL;
	USHORT nPgCnt = pDoc->GetSdPageCount(PK_STANDARD);

	for (USHORT nPage = 0; nPage < nPgCnt; nPage++)
	{
		pPage = pDoc->GetSdPage(nPage, PK_STANDARD);

		if (pPage->IsSelected())
		{
			aBookmarkList.Insert(new String(pPage->GetName()), LIST_APPEND);
		}
	}

	if (aBookmarkList.Count() != 0)
	{
		BrkAction();
		SdDataObjectRef pDataObject = CreateDataObject();

		SdDrawDocument* pDataDoc = pDataObject->pSdDrawDocument;
		pDataObject->ResetPageView();
		pDataDoc->Clear();
		pDataDoc->CreateFirstPages();
		pDataDoc->InsertBookmarkAsPage(&aBookmarkList, NULL, FALSE, TRUE, 1,
									   TRUE, pDocSh, TRUE, TRUE);
		pDataObject->UpdatePageView();

		String* pString = (String*) aBookmarkList.First();
		while( pString )
		{
			delete pString;
			pString = (String*) aBookmarkList.Next();
		}
	}
}


/*************************************************************************
|*
|* Paste object from clipboard
|*
\************************************************************************/

void SdSlideView::DoPaste(Window* pWindow)
{
	pDocSh->SetWaitCursor( TRUE );

	SdView::DoPaste(pWindow);

	SfxUInt16Item aItem( SID_PAGES_PER_ROW, nPagesPerRow);
	SFX_DISPATCHER().Execute( SID_PAGES_PER_ROW, SFX_CALLMODE_SYNCHRON |
							  SFX_CALLMODE_RECORD, &aItem, 0L, 0L );

	pDocSh->SetWaitCursor( FALSE );
}

/*************************************************************************
|*
|* Selektierte Seiten loeschen
|*
\************************************************************************/

void SdSlideView::DeleteMarked()
{
	String aStr(SdResId(STR_UNDO_DELETEPAGES));
	BegUndo(aStr);

	USHORT nPage = 0;
	SdPage* pPage = pDoc->GetSdPage(nPage, PK_STANDARD);

	while (pPage)
	{
		pPage = pDoc->GetSdPage(nPage, PK_STANDARD);

		if (pPage->IsSelected() && pDoc->GetSdPageCount(PK_STANDARD) > 1)
		{
			AddUndo(new SdrUndoDelPage(*pPage));
			pDoc->RemovePage(pPage->GetPageNum());

			pPage = pDoc->GetSdPage(nPage, PK_NOTES);
			AddUndo(new SdrUndoDelPage(*pPage));
			pDoc->RemovePage(pPage->GetPageNum());
		}
		else
		{
			nPage++;
		}

		pPage = pDoc->GetSdPage(nPage, PK_STANDARD);
	}

	EndUndo();
}


/*************************************************************************
|*
|* Drop
|*
\************************************************************************/

BOOL SdSlideView::Drop(const DropEvent& rMEvt, SdWindow* pWin,
					   USHORT nPage, USHORT nLayer)
{
	pDocSh->SetWaitCursor( TRUE );

	BOOL bRet = SdView::Drop(rMEvt, pWin, nPage, nLayer);

	delete pCache;
	pCache = NULL;

//    SfxUInt16Item aItem( SID_PAGES_PER_ROW, nPagesPerRow);
//    SFX_DISPATCHER().Execute( SID_PAGES_PER_ROW, SFX_CALLMODE_SLOT |
//                              SFX_CALLMODE_RECORD, &aItem, 0L, 0L );

	pDocSh->SetWaitCursor( FALSE );

	return bRet;
}


