/*************************************************************************
 *
 *  $RCSfile: row.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_ROW_HXX
#define _SDB_ROW_HXX 1

#include <variant.hxx>
#include <sdbcol.hxx>

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#define ROW_CLEAN		0x00
#define ROW_DELETED		0x01
#define	ROW_OBSOLETE	0x03	// contains deleted

#if __PRIVATE
DBG_NAMEEX(ODbRow);
#endif

//************************************************************
//  ODbRow
//	General Variant which is based on the UsrAny, but it should
//	be possible to change the data holder if neccessary
//************************************************************
class ODbRow : public NAMESPACE_VOS(OReference), private NAMESPACE_STD(vector)<ODbVariantRef>
{		
	friend class ORowCache;

	SdbColumnsRef	m_aColumns;
	INT8			m_nState;	

	ODbRow():m_nState(ROW_CLEAN)
	{
#if __PRIVATE
		DBG_CTOR(ODbRow,NULL);
#endif
	}

protected:
	inline ~ODbRow();	

public:	
	ODbRow(SdbColumns& rColumns, BOOL bDoBind = TRUE);
	ODbRow(const ODbRow& rRow);	
	ODbRow& operator=(const ODbRow& rRow);			 
	BOOL operator==(const ODbRow& rRow) const;	
	BOOL operator!=(const ODbRow& rRow) const {return !((*this) == rRow);}
	
	// vector methods
	vector<ODbVariantRef>::operator[];		
	vector<ODbVariantRef>::begin;
	vector<ODbVariantRef>::end;
	vector<ODbVariantRef>::rbegin;
	vector<ODbVariantRef>::rend;
	vector<ODbVariantRef>::front;
    vector<ODbVariantRef>::back;
	vector<ODbVariantRef>::size;
    vector<ODbVariantRef>::empty;		

	// modified Mimik
	BOOL isModified() const;
	void setModified(BOOL bMod);

	BOOL isDeleted() const {return (m_nState & ROW_DELETED) == ROW_DELETED;}
	BOOL isObsolete() const {return (m_nState & ROW_OBSOLETE) == ROW_OBSOLETE;}

	const SdbColumnsRef& getColumns() const {return m_aColumns;}

	// set all variants to NULL	and not modified and state clean
	void reset();

#if __PRIVATE
public:												
	void setState(INT8 _nState) {m_nState = _nState;}	
#endif
};

//************************************************************
inline ODbRow::~ODbRow()
{
#if __PRIVATE
	DBG_DTOR(ODbRow,NULL);
#endif
}

//************************************************************
//Handles for the Db variants
typedef ODbRef< ODbRow > ODbRowRef;

#endif

