/*************************************************************************
 *
 *  $RCSfile: rowcache.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_ROWCACHE_HXX
#define _SDB_ROWCACHE_HXX 1

#ifndef _SDB_ROW_HXX
#include <row.hxx>			  
#endif

#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif

// Richtung eines Seek oder eines Shift (nur fuer interne Zwecke!)
enum SdbDirection
{
	SDB_DIR_LEFT,
	SDB_DIR_RIGHT
};	  

// Symbolische Konstanten fuer absolute Positionierung
#define ROWCACHE_FIRST (0)
#define ROWCACHE_LAST (ULONG_MAX)

//************************************************************
// ORowCache
//************************************************************
class ORowCache
{
	friend class SdbCursor;

	SdbCursor& m_rCursor;

	// Zeichen fuer "NO DATA FOUND" im Cache:
	ODbRow *OFFLEFT,
		   *OFFRIGHT;

	ODbRowRef* m_pRows;

	// Wieviele Rows enthaelt der Cache maximal?
	const UINT16 m_nSize;
	const UINT16 m_nDefaultOffset;  // Offset um das standardmaessig verschoben wird

	// Die aktuelle Position (wie sie nach aussen sichtbar ist)
	// im Cache (Cache-Offset 0 = erste Row im Cache).
	UINT16 m_nCurrentPos;

	// Die aktuelle Position des zugrundeliegenden Cursors im
	// Cache (Cache-Offset 0 = erste Row im Cache).
	UINT16 m_nCursorPos;	

protected:
	ORowCache(SdbCursor& rCursor, UINT16 nCacheSize, BOOL bFromFirst);
	~ORowCache();

	// Platz im Cache schaffen:
	// Cache-Anfang (und damit alle Eintraege) verschieben.
	// Left: Cache (-Anfangsposition) nach links verschieben.
	//       Die vorhandenen Eintraege verschieben sich dadurch nach rechts,
	//       links werden dadurch Eintraege frei.
	// Right: umgekehrt.
	void shift(SdbDirection eDirection, UINT16 nOffset = 0);
	BOOL fetch(UINT16 eDirection, INT32 nPos = 1);
	void clear(UINT16 eDirection, UINT16 nPos);

	// Positionierung
	BOOL positionRelative(SdbDirection eDirection, UINT32 nOffset);
	BOOL positionAbsolute(UINT32 nPos, BOOL bForceFill = FALSE);
	BOOL positionBookmark(const ::com::sun::star::uno::Any& rBookmark) 
			throw( ::com::sun::star::lang::IllegalArgumentException );

	// Alle Rows im Cache angefangen von der Cursor-Position in
	// die angegebene Richtung fuellen.
	// nRequestedRowsInCache: Anzahl Rows, die (inkl. der aktuellen Row) gefuellt werden
	// sollen. Ohne Argument: alle bis zum Ende des Cache.
	void fill(SdbDirection eDirection, UINT16 nRequestedRowsInCache = 0);

	inline ODbRowRef& at(UINT16 nPos);
	inline const ODbRowRef& at(UINT16 nPos) const;

	inline BOOL isOffLeft(const ODbRowRef& rRow) const {return rRow.getBodyPtr() == OFFLEFT;}
	inline BOOL isOffRight(const ODbRowRef& rRow) const {return rRow.getBodyPtr() == OFFRIGHT;}
	inline BOOL isOffRange(const ODbRowRef& rRow) const {return isOffLeft(rRow) || isOffRight(rRow);}
	
	BOOL isOffLeft() const;
	BOOL isOffRight(BOOL bLookForward = TRUE) const;
	
	inline BOOL isOffRange() const;

	BOOL isOnFirst()  const;
	BOOL isOnLast() const;


private:
	BOOL implPositionRelative(SdbDirection eDirection);
	INT32 getCurrentPos() const;
	BOOL synchronize(BOOL bRefreshData);
	BOOL refresh();	

	// ungueltige Saetze (Vorsicht mit Offrange)
	BOOL isObsolete(const ODbRowRef& rRow) const;
	inline BOOL isObsolete() const;	

	void rowInserted();
	void rowDeleted();
};

//--------------------------------------------------------------------------
inline ODbRowRef& ORowCache::at(UINT16 nPos)
{
	DBG_ASSERT(nPos < m_nSize && m_pRows != NULL, "Fehler im Cache");
	// const cast
	return m_pRows[nPos];
}

//--------------------------------------------------------------------------
inline const ODbRowRef& ORowCache::at(UINT16 nPos) const
{
	DBG_ASSERT(nPos < m_nSize && m_pRows != NULL, "Fehler im Cache");
	// const cast
	return m_pRows[nPos];
}

//--------------------------------------------------------------------------
inline BOOL ORowCache::isOffRange() const {return isOffRange(at(m_nCurrentPos));}

//--------------------------------------------------------------------------
inline BOOL ORowCache::isObsolete() const
{
	return isObsolete(at(m_nCurrentPos));
}	  
#endif

