/*************************************************************************
 *
 *  $RCSfile: sdbcancl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_CANCEL_
#define _SDB_CANCEL_

#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif

class SdbCancelable
{
private:
	UINT16					m_nNestingLevel;			// the execution is within a cancelable part of code ?
	BOOL					m_bExecutionCanceled : 1;	// the execution (since the first BeginCancelable) was canceled ?
	BOOL					m_bCoreCall : 1;			// see SdbCancelableCoreCall

protected:
	NAMESPACE_VOS(OMutex)	m_aCancelAccess;		// thread safety

	// if you instantiate this class before doing a single core call it will
	// upon construction :
	//	a) set m_bCoreCall of the master to TRUE
	//	b) increment m_nNestingLevel of the master
	//	c) acquire the mutex of the master
	// upon destruction :
	//	d) release the mutex
	//	e) decrement m_nNestingLevel of the master
	//	f) set m_bCoreCall of the master to FALSE
	// With this you can distinguish between cancelable calls which are just loops (for instance an MoveToLast
	// implemented by consecutive FetchNext calls) - where a cancel requires an exit from the loop - and core calls,
	// where cancel requires another core call.
	class SdbCancelableCoreCall
	{
		SdbCancelable&			m_rMaster;
		NAMESPACE_VOS(OGuard)	m_aCancelGuard;
	public:
		SdbCancelableCoreCall(SdbCancelable& _rMaster);
		~SdbCancelableCoreCall();
	};
	friend class SdbCancelableCoreCall;

public:
	SdbCancelable();

	BOOL ExecutingCancelable() { NAMESPACE_VOS(OGuard) aGuard(m_aCancelAccess); return (m_nNestingLevel > 0); }
 	virtual void Cancel();

	/**	since the last BeginCancelable - was there a call to Cancel ?
		if bOnlyIfExecuting == TRUE and the execution isn't within a Begin-/EndCancelable block
		(means if you call BeginCancelable(), EndCancelable()and then WasCanceled(TRUE) the latter
		always returns FALSE, no matter if Cancel was called within the block)
	*/
	virtual BOOL WasCanceled(BOOL bOnlyIfExecuting = FALSE);

	/// ask for m_bCoreCall
	BOOL IsCoreCall() const;

protected:
	/**	initialize m_bExecutionCanceled/m_bExecutingCancelable thread-safe
		always call EndCancelable after BeginCancelable (even if you're not interested in the result)
		(best use BEGIN_CANCELABLE und END_CANCELABLE)
	*/
	virtual void BeginCancelable();

	/**	opposite to BeginCancelable, returns TRUE if Cancel was called since BeginCancelable
	*/
	virtual BOOL EndCancelable();
};

#endif // _SDB_CANCEL_


