/*************************************************************************
 *
 *  $RCSfile: sdbcol.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBCOL_HXX
#define _SDB_SDBCOL_HXX 1

#ifndef _COM_SUN_STAR_DATA_DATABASEFIELDTYPE_HPP_
#include <com/sun/star/data/DatabaseFieldType.hpp>
#endif

#ifndef _REF_HXX //autogen
#include <tools/ref.hxx>
#endif

#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#include <tools.hxx>			  
#include <vector>
#include <map>

#ifndef _UTL_STLTYPES_HXX_
#include <unotools/stl_types.hxx>
#endif

class SdbConnection;

// Datentypen in der Datenbank
// L = Length relevant
// P = Length repraesentiert die Precision
// S = Scale relevant
enum SdbDatabaseType
{
	SDB_DBTYPE_CHAR			= ::com::sun::star::data::DatabaseFieldType::CHAR,  			// [L ] String fester Laenge  (1..254 Zeichen)
	SDB_DBTYPE_VARCHAR		= ::com::sun::star::data::DatabaseFieldType::VARCHAR,			// [L ] String variabler Laenge (1..254 Zeichen)
	SDB_DBTYPE_LONGVARCHAR	= ::com::sun::star::data::DatabaseFieldType::LONGVARCHAR,    	// [L ] String variabler Laenge (1..n Zeichen [implementationsabhaengig])
	SDB_DBTYPE_DECIMAL		= ::com::sun::star::data::DatabaseFieldType::DECIMAL,        	// [PS] Exakte, vorzeichenbehaftete Zahl (precision: 1..15, scale: 0..precision)
	SDB_DBTYPE_NUMERIC		= ::com::sun::star::data::DatabaseFieldType::NUMERIC,        	// [PS] Exakte, vorzeichenbehaftete Zahl (precision: 1..15, scale: 0..precision)
	SDB_DBTYPE_TINYINT		= ::com::sun::star::data::DatabaseFieldType::TINYINT,        	// [  ] Exakte Zahl (precision: 3, scale: 0): -128..127 oder 0..255
	SDB_DBTYPE_SMALLINT		= ::com::sun::star::data::DatabaseFieldType::SMALLINT,		// [  ] Exakte Zahl (precision: 5, scale: 0): -32768..32767 oder 0..65535
	SDB_DBTYPE_INTEGER		= ::com::sun::star::data::DatabaseFieldType::INTEGER,			// [  ] Exakte Zahl (precision: 10, scale: 0): -2^31..2^31-1 oder 0..2^32
	SDB_DBTYPE_BIGINT		= ::com::sun::star::data::DatabaseFieldType::BIGINT,			// [  ] Exakte Zahl (precision: 19 oder 20, scale: 0): -2^63..2^63-1 oder 0..2^64
	SDB_DBTYPE_REAL			= ::com::sun::star::data::DatabaseFieldType::REAL,			// [  ] Nicht exakte, vorzeichenbehaftete Zahl (mantissa precision 7 [10^-38..10^38])
	SDB_DBTYPE_DOUBLE		= ::com::sun::star::data::DatabaseFieldType::DOUBLE,			// [  ] Nicht exakte, vorzeichenbehaftete Zahl (mantissa precision 15 [10^-308..10^308])
	SDB_DBTYPE_BINARY		= ::com::sun::star::data::DatabaseFieldType::BINARY,			// [L ] Binaere Daten fester Laenge (1..255 Byte)
	SDB_DBTYPE_VARBINARY	= ::com::sun::star::data::DatabaseFieldType::VARBINARY,		// [L ] Binaere Daten variabler Laenge (1..255 Byte)
	SDB_DBTYPE_LONGVARBINARY = ::com::sun::star::data::DatabaseFieldType::LONGVARBINARY,	// [L ] Binaere Daten variabler Laenge (1..n Byte [implementationsabhaengig])
	SDB_DBTYPE_DATE			= ::com::sun::star::data::DatabaseFieldType::DATE,			// [  ] Datum
	SDB_DBTYPE_TIME			= ::com::sun::star::data::DatabaseFieldType::TIME,			// [  ] Zeit
	SDB_DBTYPE_TIMESTAMP	= ::com::sun::star::data::DatabaseFieldType::DATETIME ,		// [  ] Datum und Zeit
	SDB_DBTYPE_CURRENCY		= ::com::sun::star::data::DatabaseFieldType::CURRENCY,		// [PS] Exakte, vorzeichenbehaftete Zahl (precision: 19, scale: 4)
								//		Wird bei manchen Treibern wie dBase auf 	 10			4 gesetzt
	SDB_DBTYPE_BOOLEAN		= ::com::sun::star::data::DatabaseFieldType::BOOL,			// [  ] Ein Bit (0 oder 1, interpretiert als FALSE oder TRUE)
	SDB_DBTYPE_UNKNOWN      = ::com::sun::star::data::DatabaseFieldType::OTHER,    // [  ] Unbekannter Datentyp, zumeist Benutzer definiert
	SDB_DBTYPE_BOOKMARK		= -20000		// [  ] "Bookmark", d. h. eindeutige Kennzeichnung einer Row in einer Ergebnismenge.
								//      Eine Bookmark kommt nie direkt in der Datenbank vor, sondern aussschliesslich
								//      als Element #0 in einer Row.
};

#define SDB_DBTYPE_SERIAL  SDB_DBTYPE_INTEGER

enum SdbFlags
{
	SDB_FLAGS_NULLALLOWED =		0x0001,	// Darf der Wert in dieser Column NULL sein,
								// d. h. *keinen* Wert enthalten?
	SDB_FLAGS_UPDATEALLOWED =	0x0002,// Bei Selektionen: ist auf diese Column ein
								// UPDATE moeglich?

	SDB_FLAGS_DESCENDING	=	0x0004,	// Nur relevant im  Zusammenhang mit Indices, legt die Sortierung innerhalb eines
										// Indices fest
	SDB_FLAGS_UNIQUE		=	0x0008,	// Ist das Feld unique
	SDB_FLAGS_PRIMARYKEY	=	0x0018,	// Ist das Feld ein primary key (impliziert Unique)
	SDB_FLAGS_FOREIGNKEY	=	0x0020,	// Ist das Feld ein foreign key
	SDB_FLAGS_CASESENSITIVE =	0x0040,	// ist der Name casesensitive
	SDB_FLAGS_SEARCH_LIKE	=	0x0080,	// kann nach der Spalte mit like gesucht werden
	SDB_FLAGS_SEARCH_EXCEPT_LIKE =	0x0100,	// kann nach der Spalte compare gesucht werden
	SDB_FLAGS_SEARCH_ALL	=	0x0180,	// kann nach der Spalte compare gesucht werden
	SDB_FLAGS_AUTOINCREMENT	=	0x0200,	// kann nach der Spalte compare gesucht werden
	SDB_FLAGS_PARAM_IN		=	0x1000,	// IN Parameter
	SDB_FLAGS_PARAM_OUT		=	0x2000,	// OUT Parameter f"ur Stored Procedures
	SDB_FLAGS_VERSIONING	=	0x4000	// Does the column specify a row version?
};
//------------------------------------------------------------------------------------------------
// Erweiterte Columninformationen
struct SdbColumnInfo
{
	BOOL bFunction;		 // Info ob Column eine Funktion ist
	BOOL m_bDbasePrecisionChanged; // a long name says it all
	String aTableName;	 // der zugeh"orige Tabellenname
	

	SdbColumnInfo(String aName,BOOL bFkt=FALSE,BOOL bDbf=FALSE):bFunction(bFkt),m_bDbasePrecisionChanged(bDbf),aTableName(aName){};
	SdbColumnInfo(const SdbColumnInfo& aC):bFunction(aC.bFunction),m_bDbasePrecisionChanged(aC.m_bDbasePrecisionChanged),aTableName(aC.aTableName){};
};
//------------------------------------------------------------------------------------------------
// Class SdbColumn
//
// Beschreibung:
//	Beschreibung einer (Datenbank-)Spalte bestehend aus
//	Spaltenname und Typinformationen.
//
class SdbColumn : public NAMESPACE_VOS(OReference)
{
	friend class SdbColumns;
	friend class SdbConnection;

	// Laenge der Column (Maximale Anzahl von Bytes). Die Zahl
	// bezieht sich auf die Darstellung in der Datenbank.
	// Manche Typen haben eine fest vorgegebene Laenge, in diesem
	// Fall ist nLength 0.

	// Je nach Datentyp kann die Laenge fest (z. B. CHARACTER)
	// oder variable (z. B. VARYING) sein. In letzterem Fall
	// ist die Laenge die Maximallaenge. Null-Bytes als
	// Stringende werden dabei nicht mitgezaehlt.

	// Bei NUMBER, DECIMAL und CURRENCY wird die "Precision" als
	// Length abgelegt.
	UINT32 nLength;

	// Formatierungsinformation fuer eine Spalte, ist zunaechst nur eine temporaere Info
	// erst wenn die neue Datenengine existiert, kann der key direkt aus der Datenbank uebernommen
	// werden. Der Formatkey fliet nicht mit in den Vergleichsoperator mit ein.
	UINT32	nFormatKey;

	
	SdbColumnInfo* pInfo;


	// Name des Feldes. Feldnamen muessen den Richtlinien von
	// SQL gehorchen, d. h. sie muessen mit einem Buchstaben
	// beginnen und duerfen nur aus Buchstaben, Ziffern sowie
	// den Sonderzeichen '$' und '#' bestehen.

	// Fragen: Was ist mit '_'??? '$' und '#' sind in Basic
	// problematisch?
	// aName kann ein Alias Name sein
	String aName;

	// Kommentar zur Column
	String aComment;

	// Default value, set by string
	String aDefaultValue;

	// Richtiger Name der Column, im aName kann auch nur ein Label verborgen
	// sein
	String aRealName;

	// Datentyp in der Datenbank
	SdbDatabaseType eType;

	// Interne Identifikation einer Column.
	// Wird nur benoetigt, um Umbenennungen im Rahmen
	// der Bearbeitung von Strukturen zu erkennen.
	UINT16 nId;

	// Weitere Flags (Bitmaske, siehe unten):
	UINT16 nFlags;

	// Nur ueber eine Connection koennen Columns erzeugt werden, da die Namen
	// einer Column abhaengig von der Connnection case sensitiv sein koennen oder
	// auch nicht.

	// Scale (bei Nicht-Number-Feldern 0).
	// Mu im Bereich 0..nLength liegen, aber auf jeden Fall <255 sein!
	UINT8 nScale;

protected:
	virtual ~SdbColumn();

	// Die Angabe von Length und Scale bei Typen, fuer die diese Angabe nicht
	// relevant ist, ist nicht zulaessig (siehe Tabelle oben)!
	// Fuer die uebrigen Typen muss die Laenge angegeben werden (und > 0 sein).
	// Die Maximallaenge des jeweiligen Typs darf dabei nicht ueberschritten werden
	// (entsprechend bei Scale).
	SdbColumn(const String &_rName, SdbDatabaseType _eType,
			  UINT32 _nLength = 0, UINT16 _nScale = 0,
			  UINT16 _nFlags = 0, UINT32 _nFormatKey = 0);

	SdbColumn(const SdbColumn &rSdbColumn);
	// Kopie anlegen, aber nur die angegebenen Flags mit kopieren:
	SdbColumn(const SdbColumn &rSdbColumn, UINT16 nFlagMask);
	// SdbColumn mit Alias, wird aName zugewiesen
	SdbColumn(const String& rAlias, const SdbColumn& rCol);

public:
	TYPEINFO();
	BOOL operator==(const SdbColumn &rSdbColumn) const;
	BOOL operator!=(const SdbColumn &rSdbColumn) const {return ! (*this == rSdbColumn);}

	// Vergleich, wobei nur die angegebenen Flags beruecksichtigt werden:
	BOOL Compare(const SdbColumn &rSdbColumn, INT16 nFlagMask = 0x7fff) const;
	BOOL CompareWithOutName(const SdbColumn &rSdbColumn, INT16 nFlagMask = 0x7fff) const;

	// Setzen von Flags:
	// Einzelne Flags:
	void SetFlag(SdbFlags eNewFlag) {nFlags |= eNewFlag;}
	void ResetFlag(SdbFlags eNewFlag) {nFlags &= ~eNewFlag;}

	// Alle Flags auf einmal:
	void SetAllFlags(INT16 nNewFlags) {nFlags = (UINT16)nNewFlags;}

	// Alle Flags als Bitmaske:
	UINT16 GetAllFlags() const {return nFlags;}

	// Kommentar
	void SetComment(const String & rNewComment) {aComment = rNewComment;}
	const String& GetComment() const {return aComment;}

	// Default-Value
	void SetDefaultValue(const String& rNewDefaultValue) {aDefaultValue = rNewDefaultValue;};
	const String& GetDefaultValue() const {return aDefaultValue;};

	// Zugriff auf Members (Beschreibung siehe oben):
	// Length und Precision liefert immer den gleichen Wert.
	// Length/Precision und Scale von Datentypen, fuer die dieser Wert nicht
	// relevant ist, sind 0.
	const String		GetName() const     {return aName;}
	SdbDatabaseType				GetType() const     {return eType;}
	UINT32						GetLength() const	 {return nLength;}
	UINT32						GetPrecision() const{return nLength;}
	UINT16						GetScale() const	 {return nScale;}
	UINT32						GetFormatKey() const {return nFormatKey;}
	const String		GetTableName() const {return pInfo->aTableName;}
	const SdbColumnInfo&		GetColumnInfo() const {return *pInfo;}

	void				SetType(SdbDatabaseType _eType) {eType = _eType;}
	void				SetLength(UINT32 _nLength) {nLength = _nLength;}
	void				SetScale(UINT16 _nScale) {nScale = (UINT8)_nScale;}
	void				SetFormatKey(UINT32 _nKey) {nFormatKey = _nKey;}
	void				SetTableName(const String& _aTableName) {pInfo->aTableName = _aTableName;}
	void				SetFunction(BOOL _bFkt) { pInfo->bFunction = _bFkt;}


	// Einzelne Flags:
	BOOL IsAutoIncrement() const {return (nFlags & SDB_FLAGS_AUTOINCREMENT) == SDB_FLAGS_AUTOINCREMENT;}
	BOOL IsNullAllowed() const	{return (nFlags & SDB_FLAGS_NULLALLOWED) == SDB_FLAGS_NULLALLOWED;}
	BOOL IsUpdateAllowed() const {return (nFlags & SDB_FLAGS_UPDATEALLOWED) == SDB_FLAGS_UPDATEALLOWED;}
	BOOL IsPrimaryKey() const {return (nFlags & SDB_FLAGS_PRIMARYKEY) == SDB_FLAGS_PRIMARYKEY;}
	BOOL IsForeignKey() const {return (nFlags & SDB_FLAGS_FOREIGNKEY) == SDB_FLAGS_FOREIGNKEY;}
	BOOL IsUnique() const {return (nFlags & SDB_FLAGS_UNIQUE) == SDB_FLAGS_UNIQUE;}
	BOOL IsCaseSensitive() const {return (nFlags & SDB_FLAGS_CASESENSITIVE) == SDB_FLAGS_CASESENSITIVE;}
	BOOL IsDescending() const {return (nFlags & SDB_FLAGS_DESCENDING) == SDB_FLAGS_DESCENDING;}
	BOOL IsFunction() const {return pInfo->bFunction;}
	BOOL IsInParameter() const { return (nFlags & SDB_FLAGS_PARAM_IN) == SDB_FLAGS_PARAM_IN;}
	BOOL IsOutParameter() const { return (nFlags & SDB_FLAGS_PARAM_OUT) == SDB_FLAGS_PARAM_OUT;}

	// Id (nur zur Feststellung der Identitaet einer Column
	// [zur Erkennung von Umbenennungen bei der Bearbeitung
	// der Struktur in SdbTable]). Fuer die Beschreibung der
	// Struktur belanglos.
	UINT16 GetId() const {return nId;}
	const String& GetRealName() const {return aRealName;}
	virtual SdbColumn* Clone() const;

#if __PRIVATE
protected:
	void SetId(UINT16 nId);
public:
	void SetRealName(const String& rName) {aRealName = rName;}
	void SetAutoIncrement();
	void SetDbasePrecisionChanged(BOOL bChg) { pInfo->m_bDbasePrecisionChanged = bChg; }
	BOOL IsDbasePrecisionChanged() const { return pInfo->m_bDbasePrecisionChanged; }
#endif
};

typedef ODbRef< SdbColumn > SdbColumnRef;

// Class SdbColumns
//
// Beschreibung:
//	Liste der Spalten (Columns), aus der ein Row besteht.
//	Mit dieser Column-Liste wird die Struktur der Rows
//	beschrieben.

//	Der Zugriff auf einzelne Spalten kann ueber die Nr. der
//	Column (Zaehlung beginnt bei Null) in der Liste oder
//	auch ueber den Spaltennamen erfolgen.

//	SdbColumns verwaltet einen Zaehler der Referenzen auf
//	sich (siehe SdbReferenceCountObject).
//	Jedes Objekt, das eine Referenz auf die
//	SdbColumns-Strukturbeschreibung haelt, muss sich mit
//	AddReference bei SdbColumns registrieren und bei
//	Aufgabe der Referenz mit ReleaseReference abmelden.
//	Die Konstruktion eines SdbColumns-Objektes impliziert
//	allerdings eine Referenz (nReferenceCount = 1).

//	Durch die Verwaltung eines Referenzzaehlers wird
//	sichergestellt, dass die Strukturbeschreibung zu
//	potentiell vielen Rows solange erhalten bleibt, bis
//	die letzte Row die Referenz aufloest (und das passiert
//	beim Zerstoeren der Row).

//	Da die Strukturbeschreibung (SdbColumns) sich ausschliesslich
//	selbst loeschen darf (gesteuert ueber den Referenzzaehler),
//	koennen Instanzen von SdbColumns nur mit 'new' (nicht statisch
//	oder auf dem Stack) angelegt werden.
//
class SdbColumns : public NAMESPACE_VOS(OReference), private NAMESPACE_STD(vector)<SdbColumnRef>
{
	friend class SdbRow;
	friend class SdbTable;
	friend class SdbCursor;
	friend class SdbIndex;
	friend class SdbConnection;

	::std::map< String, SdbColumnRef, ::utl::UStringMixLess> m_aMap;

protected:
	SdbColumns(SdbConnection& _rConn);
	// Die Columns-Liste und alle enthaltenen Columns werden kopiert
	// (aber nicht die Anzahl der Referenzen - die wird auf 1 gesetzt!)
	SdbColumns(SdbConnection& _rConn, const SdbColumns &rSdbColumns);
	virtual ~SdbColumns();

public:
	// Wertegleichheit der enthaltenen Columns
	// (unabhaengig von der Anzahl der Referenzen)
	BOOL operator==(const SdbColumns &rSdbColumns) const;
	BOOL operator!=(const SdbColumns &rSdbColumns) const {return !((*this) == rSdbColumns);}

	// Vergleich, wobei nur die angegebenen Flags (siehe Columns)
	// beruecksichtigt werden:
	BOOL Compare(const SdbColumns &rSdbColumns, INT16 nFlagMask = 0x7fff) const;

	// Eine Column hinzufuegen (am Ende der Liste).
	// Die Column geht in die Verantwortung der Struktur-
	// beschreibung (Columns) ueber und wird nur von dieser
	// wieder zerstoert (delete).
	// Die Column MUSS mit new alloziert worden sein!

	// AddColumn, InsertColumn, RemoveColumn und ReplaceColumn
	// duerfen nur im Anschluss an die Erzeugung
	// des SdbColumns-Objektes verwendet werden, um das
	// Objekt EINMAL zu initialisieren. Nach Uebergabe
	// (einer Referenz) des SdbColumns-Objektes an
	// eine oder mehrere Rows duerfen keine Columns mehr
	// hinzugefuegt werden (d. h. nach Verwendung von
	// AddReference etc.)! Die uebergebene Column geht
	// in die Verantwortung von SdbColumns ueber!
	inline void AddColumn(SdbColumn* _pCol);

	// Column vor der angegebenen Column einfuegen.
	inline void InsertColumn(SdbColumn* _pCol, UINT16 _nColNo);
	void InsertColumn(SdbColumn* _pCol, const String& _rName){InsertColumn(_pCol, ColumnNumber(_rName));}

	// Angegebene Column loeschen
	inline void RemoveColumn(UINT16 _nColNo);
	void RemoveColumn(const String& _rName) {RemoveColumn(ColumnNumber(_rName));}

	// Angegebene Column durch neue ersetzen (alte wird "delete"d)
	// Die 'Id' der Column bleibt erhalten (in der neuen Column wird die
	// Id der alten Column gesetzt).
	inline void ReplaceColumn(SdbColumn* _pCol, UINT16 _nColNo);
	void ReplaceColumn(SdbColumn* _pCol, const String& _rName){ReplaceColumn(_pCol, ColumnNumber(_rName));}

	// Columns in der Strukturbeschreibung (Columns) sind
	// geordnet. Auf diese Weise
	// ist es moeglich, die n-te Column in der Liste
	// zuzugreifen. Die erste Column hat die Nr. 0.
	// Liefert NULL bei Zugriff auf nicht existente Column.
	const SdbColumn* Column(UINT16 _nColNo) const {return vector<SdbColumnRef>::operator[](_nColNo).getBodyPtr();}
	const SdbColumn* operator[] (UINT16 _nColNo) const {return Column(_nColNo);}

	// checks if _nColNo < then Count(). If not it return NULL
	const SdbColumn* at(UINT16	_nColNo) const;

	// Alle Columns haben einen Namen. Dieser Name kann
	// als Index fuer den Zugriff der Columninformationen
	// verwendet werden. Die ColumnNumber zu einem Namen
	// kann getrennt erfragt werden.

	// Liefert NULL bei Zugriff auf nicht existente Column.
	inline const SdbColumn* Column(const String& _rName) const;
	inline const SdbColumn* operator[] (const String& _rName) const;
	
	// Liefert SDB_COLUMN_NOTFOUND, wenn nicht gefunden, sonst die Nummer
	// der Column.
#define SDB_COLUMN_NOTFOUND ((UINT16)0xffff)
	inline UINT16 ColumnNumber(const String& _rName) const;

	// Anzahl Columns in der Liste.
	UINT16 Count() const {return (UINT16) vector<SdbColumnRef>::size();}

	// gibt alle Spalten bis auf die BOOKMARKSPALTE frei
	void Reset();

	// give the count without counting the SDB_FLAGS_VERSIONING columns
	UINT16	GetVisibleCount() const;

#if __PRIVATE
	UINT16 GetColumnNumberByRealName(const String& _rName) const;
#endif
};

//------------------------------------------------------------------
inline const SdbColumn* SdbColumns::Column(const String& _rName) const
{
	::std::map<String, SdbColumnRef, ::utl::UStringLess>::const_iterator i = m_aMap.find(_rName);
	if (i != m_aMap.end())
		return (*i).second.getBodyPtr();
	else
		return NULL;
}

//------------------------------------------------------------------
inline const SdbColumn* SdbColumns::operator[](const String& _rName) const
{
	::std::map<String, SdbColumnRef, ::utl::UStringLess>::const_iterator i = m_aMap.find(_rName);
	if (i != m_aMap.end())
		return (*i).second.getBodyPtr();
	else
		return NULL;
}

//------------------------------------------------------------------
inline UINT16 SdbColumns::ColumnNumber(const String& _rName) const
{	
	::std::map<String, SdbColumnRef, ::utl::UStringLess>::const_iterator i = m_aMap.find(_rName);
	if (i != m_aMap.end())
		return (*i).second->nId;

	return SDB_COLUMN_NOTFOUND;
}

//------------------------------------------------------------------------
inline void SdbColumns::AddColumn(SdbColumn* _pCol)
{
	_pCol->nId = size();
	push_back(_pCol);
	m_aMap[_pCol->GetName()] = _pCol;
}

//------------------------------------------------------------------------
inline void SdbColumns::InsertColumn(SdbColumn* _pCol, UINT16 nPos)
{
	_pCol->nId = size();
	insert(begin() + nPos, _pCol);
	m_aMap[_pCol->GetName()] = _pCol;
}

//------------------------------------------------------------------
inline void SdbColumns::RemoveColumn(UINT16 nPos)
{
	// AddColumn ist nur erlaubt, solange nicht mehrere Referenzen existieren
	DBG_ASSERT(nPos != 0,"SdbColumns: Bookmark Column kann nicht geloescht werden");

	SdbColumnRef xOldCol = vector<SdbColumnRef>::operator[](nPos);
	erase(begin() + nPos);
	m_aMap.erase(xOldCol->GetName());
}

//------------------------------------------------------------------
inline void SdbColumns::ReplaceColumn(SdbColumn* _pCol, UINT16 nPos)
{
	// AddColumn ist nur erlaubt, solange nicht mehrere Referenzen existieren
	DBG_ASSERT(_pCol != NULL,"SdbColumns: Ungueltige NewColumn");
	DBG_ASSERT(nPos != 0,"SdbColumns: Bookmark Column kann nicht ersetzt werden");

	SdbColumnRef xOldCol = vector<SdbColumnRef>::operator[](nPos);
	m_aMap.erase(xOldCol->GetName());
	vector<SdbColumnRef>::operator[](nPos) = _pCol;
	m_aMap[_pCol->GetName()] = _pCol;
	_pCol->nId = size();
}	

typedef ODbRef< SdbColumns > SdbColumnsRef;

#endif

