/*************************************************************************
 *
 *  $RCSfile: sdbconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBCONN_HXX
#define _SDB_SDBCONN_HXX 1

#include <sdbcol.hxx>

#ifndef _COM_SUN_STAR_DATA_DATABASEISOLATIONLEVEL_HPP_
#include <com/sun/star/data/DatabaseIsolationLevel.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASESEARCH_HPP_
#include <com/sun/star/data/DatabaseSearch.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEPRIVILIGE_HPP_
#include <com/sun/star/data/DatabasePrivilige.hpp>
#endif

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#ifndef _SDB_CMDPARSE_HXX
#include <cmdparse.hxx>
#endif
#ifndef _SDB_SDBCONT_HXX
#include <sdbcont.hxx>
#endif
#ifndef _SDB_SDBOBJ_HXX
#include <sdbobj.hxx>
#endif
#ifndef _SDB_SDBENV_HXX
#include <sdbenv.hxx>
#endif
#ifndef _SVSTDARR_STRINGSDTOR
#define _SVSTDARR_STRINGSDTOR
#include <svtools/svstdarr.hxx>
#endif

class SdbCursor;
class SdbRelation;

namespace com { namespace sun { namespace star { namespace data {
	class XDatabaseDescriptor;
}}}}

// Isolation Level, mit der die Transaktion ablaeuft:

enum SdbIsolationLevel
{
	SDB_ISO_NONE = ::com::sun::star::data::DatabaseIsolationLevel::NONE,
	SDB_ISO_READ_UNCOMMITTED = ::com::sun::star::data::DatabaseIsolationLevel::READ_UNCOMMIT,
	SDB_ISO_READ_COMMITTED = ::com::sun::star::data::DatabaseIsolationLevel::READ_COMMIT,
	SDB_ISO_REPEATABLE_READ = ::com::sun::star::data::DatabaseIsolationLevel::REPEAT_READ,
	SDB_ISO_SERIALIZABLE = ::com::sun::star::data::DatabaseIsolationLevel::SERIALIZABLE
};

#define DEFAULT_LIST_SEPARATOR	char(11)	// Trenner fuer Listen, da ein Komma oder Semikolon
											// auch anderweitig verwendet werden knnen

// Faehigkeiten des Treibers und des SQL-Dialektes:
enum SdbCapability
{
	// DDL
	SDB_CAP_CREATE_TABLE,			// Tabellen erzeugen
	SDB_CAP_ALTER_TABLE,
	SDB_CAP_RENAME_TABLE,
	SDB_CAP_CREATE_VIEW,			// Views erzeugen

	SDB_CAP_CREATE_INDEX,			// Erzeugen von Indizes

	SDB_CAP_UPDATABLE,				// UPDATE grundsaetzlich moeglich (Insert, Update, Delete)

	// Unterstuetzt der Treiber AUTOCOMMIT bzw. manuelles COMMIT?
	SDB_CAP_COMMIT_AUTO,
	SDB_CAP_COMMIT_MANUAL,

	// Welche Isolation Level unterstuetzt dieser Treiber?
	SDB_CAP_READ_UNCOMMITTED,		// siehe oben ...
	SDB_CAP_READ_COMMITTED,
	SDB_CAP_REPEATABLE_READ,
	SDB_CAP_SERIALIZABLE,

	// only for Oterro this will return TRUE
	SDB_CAP_VERSIONING,
	// Kann ein exclusives Sperren fr Updates vorgenommen werden
	SDB_CAP_JOIN,					// Kann Joins

	SDB_CAP_DESC_INDEX,				// koennen absteigende Indices erzeugt
	SDB_CAP_COMPRESS,				// Kann die Datenbank komprimiert werden?
	SDB_CAP_PRIMARYKEYS,			// koennen pkeys vergeben werden
	SDB_CAP_FOREIGNKEYS,			// koennen fkeys vergeben werden
	SDB_CAP_COL_COMMENT,			// koennen Felddefinitionen einen Kommentar erhalten
	SDB_CAP_BULKOPTIMIZE,			// koennen beim Appendonly Cursor, preparedstatements
									// verwendet werden (interner Gebrauch)
	SDB_CAP_COUNT,					// Letzter enum-Wert = Anzahl der enum-Werte
	SDB_CAP_COMPACT,				// Datenbank reorganisieren

	SDB_CAP_INSERTTABCOLS,			// Tabellen-Spalten im Entwurf wirklich _einfuegen_ oder nur anhaengen
	SDB_CAP_MULTITHREADACCESS,		// connections are thread safe	
	SDB_CAP_USERSETTINGS,			// User information only for adabas
	SDB_CAP_STATIC_PRIOR,			// static cursor supports prior movement
	SDB_CAP_KEYSET_PRIOR,			// keyset/dynamic cursor supports prior movement
	SDB_CAP_BACKUP					// backup of database eq. ADABAS
};

// Typinformationen vom Treiber, Mehrfachnennungen von SdbDatabaseType sind moeglich
// Besonderheit ist Currency: Viele Treiber kenn diesen Datentyp nicht,
// aber es kann hierfuer eine Repraesentation angegen werden

// wie kann nach einer Spalte eines bestimmten Typen gesucht werden

enum SdbTypeSearch
{
	Search_None			= ::com::sun::star::data::DatabaseSearch::NONE,
	Search_Like_Only  	= ::com::sun::star::data::DatabaseSearch::CHAR,
	Search_Except_Like  = ::com::sun::star::data::DatabaseSearch::BASIC,
	Search_All			= ::com::sun::star::data::DatabaseSearch::ALL
};


struct SdbTypeInfo
{
	BOOL			bCurrency		: 1,	// Waehrung
					bAutoIncrement	: 1,	// Ist es ein automatisch incrementierendes Feld
					bNullable		: 1,	// Kann das Feld NULL annehmen
					bCaseSensitive	: 1,	// Ist der Type Casesensitive
					bRowVersion		: 1,	// type can be a row version column
					bUnsigned		: 1,	// Ist der Type Unsigned	
					bEmpty_1		: 1,	// for later use
					bEmpty_2		: 1;
	
	UINT8			nScale;			// Nachkommastellen
	UINT8			nMinimumScale;  // Min Nachkommastellen

	UINT32			nPrecision;		// Laenge des Types

	SdbDatabaseType	eType;			// Datenbanktyp
	SdbTypeSearch	eSearchType;	// kann nach dem Typen gesucht werden
	String	aTypeName;		// Name des Types in der Datenbank
	String			aLiteralPrefix; // Prefix zum Quoten
	String			aLiteralSuffix; // Suffix zum Quoten
	String			aCreateParams;  // Parameter zum Erstellen
	String			aLocalTypeName;
	


	SdbTypeInfo()
			:bCurrency(FALSE)
			,bAutoIncrement(FALSE)
			,bNullable(TRUE)
			,bCaseSensitive(FALSE)
			,bRowVersion(FALSE)
			,bUnsigned(FALSE)
			,nScale(0)
			,nMinimumScale(0)
			,eType(SDB_DBTYPE_UNKNOWN)
			,nPrecision(0)
			,eSearchType(Search_All)
	{}

	SdbTypeInfo(SdbDatabaseType eT, const String& rTypeName)
			:bCurrency(FALSE)
			,bAutoIncrement(FALSE)
			,bNullable(TRUE)
			,bCaseSensitive(FALSE)
			,bUnsigned(FALSE)
			,bRowVersion(FALSE)
			,nScale(0)
			,nMinimumScale(0)
			,nPrecision(0)
			,eType(eT)
			,eSearchType(Search_All)
			,aTypeName(rTypeName)
	{}
};

enum SdbCursorType
{
	SDB_SNAPSHOT = 1,				// Snapshot, haelt alle Daten im Speicher
	SDB_KEYSET	 = 2,				// Erzeugt ein Keyset, optimaler Kompromi zwischen Laufzeit und Speicherverbrauch
	SDB_DYNAMIC	 = 4,				// Dynamisch, langsam aber wenig Speicherverbrauch
	SDB_CURRENT	 = 0xff				// aktuelle Einstellungen werden beibehalten
};

enum SdbOpenOptions				// kombinierbar
{
	SDB_READONLY	= 1,			// Nur lesend
	SDB_FORWARDONLY = 2,			// Nur Forward iterieren
	SDB_APPENDONLY	= 4,			// Nur Einfuegen moeglich, Saetze koennen nicht gelesen werden
	SDB_INDIRECT	= 8,			// Query wird vor der Ausfuehrung geparst
	SDB_SEEDELETED	= 16,			// geloeschte Saetze werden beim Positionieren angezeigt
									// wird ein Satz geloescht, ist dieser nach wie vor sichtbar
	SDB_MINIMUM_DESC= 32,			// Spalten werden nur minimal beschrieben
									// (Name,Type,Laenge,Scale,NULLABLE)
	SDB_CONCURRENCY	= 64,			// concurrent update
	SDB_API_COMPATIBLE = 128,		// cursor stays after open before the result sets, editing a result set could be done
									// without using EditRow	
	SDB_CURRENT_OPT	= 0xff			// aktuelle Einstellungen werden beibehalten
};

enum SdbIdentifierCase
{
	SDB_IC_UPPER	= 0,		// Insensitive, alles gro
	SDB_IC_LOWER	= 1,		// Insensitive, alles klein
	SDB_IC_MIXED	= 2,		// InSensitive, Gro und Kleinschreibung
	SDB_IC_OBJECT	= 4,		// nur Tabellen und Indices sind case sensitive
	SDB_IC_ALL		= 12		// alle identifier sind case sensitive,
								// enthaelt SDB_IC_OBJECT
};

enum SdbTablePriviliges				// kombinierbar
{
	SDB_PR_NONE			= 0x0000,		// der Benutzer darf nichts
	SDB_PR_READ			= ::com::sun::star::data::DatabasePrivilige::RETRIEVE_DATA,		// Darf der Inhalt der Tabelle gelesen werden
	SDB_PR_INSERT		= ::com::sun::star::data::DatabasePrivilige::INSERT_DATA,		// Duerfen Saetze eingefgt
	SDB_PR_UPDATE		= ::com::sun::star::data::DatabasePrivilige::UPDATE_DATA,		// Duerfen Saetze verndert werden
	SDB_PR_DELETE		= ::com::sun::star::data::DatabasePrivilige::DELETE_DATA,		// Duerfen Saetze geloescht werden
	SDB_PR_ALTER		= ::com::sun::star::data::DatabasePrivilige::ALTER_DEF,			// Darf die Tabellenstruktur geaendert werden
	SDB_PR_DROP			= ::com::sun::star::data::DatabasePrivilige::DELETE_DEF,			// Darf eine Tabelle geloescht werden
	SDB_PR_REFERENCES	= ::com::sun::star::data::DatabasePrivilige::RELATION_DEF		// duerfen vom user constraints angelegt werden
};

#define SDB_DATAPRIVILEGES	  (SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_DELETE)
#define SDB_TABLEPRIVILEGES	  (SDB_PR_ALTER | SDB_PR_DROP | SDB_PR_REFERENCES)

DECLARE_LIST(SdbTypeInfoList, SdbTypeInfo*);
DECLARE_SDBKEYCONT(SdbTableList, SdbTable*);
DECLARE_SDBKEYCONT(SdbRelationList, SdbRelation*);
DECLARE_LIST(SdbCursorList, SdbCursor*);

class SdbEnvironment;
class SdbSqlParseNode;
class SdbSqlParser;
class SdbStringStlMap;

// Class SdbConnection
//
// Beschreibung:
//	Um Objekte eines SQL-Servers zu verwenden, ist eine
//	Verbindung ("Connection") zum betreffenden Server
//	erforderlich. SdbConnection repraesentiert diese
//	Verbindung.

//	Mit der Verbindung ist die Anmeldung als ein Benutzer
//	verbunden. Bestandteil einer Verbindung ist auch die
//	"Session", in der die Transaktionen ausgefuehrt werden.

//	Es ist moeglich, gleichzeitig mehrere Verbindungen zur
//	gleichen oder zu verschiedenen Datenbank-Servern zu
//	haben. Die Transaktionen aller Verbindungen sind
//	voneinander unabhaengig. Pro Verbindung kann jeweils
//	nur eine Anweisung zur Zeit ausgefuehrt werden, aber es
//	koennen durchaus Anweisungen in mehreren Verbindungen
//	gleichzeitig zur Ausfuehrung kommen.
//

class SdbConnection : public SdbObj
{
	friend class SdbEnvironment;
	friend class SdbTable;
	friend class SdbView;
	friend class SdbCursor;
	friend class SdbDatabase;
	friend class SdbDatabaseImpl;
	friend class SdbRelation;
	friend class SdbMetaData;

	INT16			nTransCount;    // Zaehler fuer Transaktionen
	SdbTypeInfoList aTypeList;		// Liste aller verfuegbaren Datentypen
	SdbTableList    aTableList;		// Liste zum Cachen von Tabellen
	SdbCursorList   aCursorList;	// Liste, zum Verwalten von Cursoren
	SvStringsDtor	aTableNameList; // Liste aller verfuegbaren Tabellen
	SdbRelationList aRelationList;	// Liste der Relationen
	::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > xDBMetaData;	// Interface um auf die MetaData zuzugreifen

	// Arbeiten auf meiner TableNameList
	void AddTableName(const String& strNew,BOOL bTable = FALSE);
	BOOL ReplaceTableName(const String& strOld, const String& strNew);

protected:
	// In Sdb intern verwendete Optionen ("TYP;OWNER;TABLE;CHARSET;..."),
	// die normalerweise nicht von den einzelnen Treibern interpretiert werden
	static const char * pReservedOptions;

	// Die Angaben zum Verbindungsaufbau werden aufbewahrt,
	// Dadurch ist der Aufbau zusaetzlicher Verbindungen ohne erneute
	// Abfrage der Benutzerdaten moeglich (siehe Copy-Constructur).

	// Optionen zum Verbindungsaufbau (aus einem String im Format
	// "option=wert;option=wert;option=wert").
	// Immer vorhanden:
	//		"TYP=..."		Server-Typ (siehe SdbEnvironment)
	// Weitere haeufig verwendete Optionen:
	//		"DSN=..."		"Data Source Name", Aliasname fuer eine Datenquelle,
	//						unter dem eine Reihe anderer Parameter zusammengefasst
	//						werden koennen (z. B. bei ODBC).
	// 		"UID=..." 		Benutzername
	//		"PWD=..." 		Passwort
	// sowie beliebige treiberspezifische Optionen.
	CommandParser aOptionsParser;

	// Seperatoren fuer Qualifier knnen hchst unterschiedlich aussehen
	// default ist '.'
	char cQualifierSeparator;

	// je nach Treiber werden unterschiedliche Quotes fuer Identifier verwendet,
	// \" ist default
	char cIdentifierQuotation;

	BOOL  bAutoCommit : 1;	// werden transactionen verwendet ?
	UINT32 nFree; 			// wird zur Zeit nicht bentigt

	SdbStringStlMap *pTableNames;
	void*	pVOID2;
	UINT32 nFree2;

	// Aktueller Benutzername
	String aUserName;

	// Name der aktuell verwendeten Datenbank, wird als Qualifier z.b. fuer Tabellen
	// verwendet.
	String aQualifierName;

	// Der Zeichensatz, mit dem die Datenbank arbeitet.
	rtl_TextEncoding eCharacterSet;

	// Isolation Level
	SdbIsolationLevel eIsolationLevel;

	// Special Characters
	String aSpecialCharStr;

	// werden Namen von der Verbindung Case-Sensitive behandelt
	SdbIdentifierCase eIdentifierCase;

public:
	TYPEINFO();

	// Zugriff auf Vater
	virtual SdbEnvironment* GetEnvironment() const;

	// Informationen ueber die Faehigkeiten des Treibers und des SQL-Dialektes:
	virtual BOOL IsCapable(SdbCapability eCapability) const;
	// Informationen ueber die Faehigkeiten des Treibers und des SQL-Dialektes: mit Interface
	virtual ::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > GetMetaData();
	// Information ueber Grenzwerte fuer die Datenbank
	// wie zum Beispiel maximale Anzahl Spalten in einer Tabelle

	// Erzeuge eine Datenbank gemaess dem bei der Erzeugung
	// verwendeten OptionString
	virtual BOOL Create();

	// Loesche die dem OptionString entsprechende Datenbank
	virtual BOOL Drop();

	// Voreingestellt als Owner-Name ist, wie bei den meisten Datenbanken
	// ueblich, der Benutzername. Als Tabellenname ist, je nach Treiber,
	// der Basisname der beim Verbindungsaufbau angegebenen Datei oder
	// ein beim Verbindungsaufbau angegebener Parameter voreinstellt.
	const String& GetUser() const {return aUserName;}
	const String& GetQualifier() const {return aQualifierName;}

	// Zugriff auf Relationen
	SdbRelation* CreateRelation(const String& _rName);
	SdbRelation* OpenRelation(USHORT _nPos);
	SdbRelation* OpenRelation(const String& rName);
	String GetRelationName(USHORT _nPos);

	USHORT GetRelationCount();

	// Welche Datenbanktypen unterstuetzt die jeweilige Implementation?
	// Wird der angegebene Typ unterstuetzt?
	BOOL IsTypeAvailable(SdbDatabaseType eType) const;

	// Uebergibt eine Liste mit allen verfuegbaren Datentypen eines Servers
	// Datentypen koennen mehrfach auftreten, eindeutig sind nur die Typnamen
	const SdbTypeInfoList& GetTypeInfo() const {return aTypeList;}

	// Die Optionen zum Verbindungsaufbau (inkl. "TYP=..."-Option).
	// Mit diesen Angaben kann die Verbindung jederzeit wieder hergestellt
	// werden.
	// (Enthaelt ggf. auch das Passwort fuer den Verbindungsaufbau!)
	String Options() const;

	// Isolation Level:
	virtual void SetIsolationLevel(SdbIsolationLevel eNewIsolationLevel);
	SdbIsolationLevel IsolationLevel() const {return eIsolationLevel;}

	// Schema: das Default-Schema, auf das sich alle Tabellen- und
	// View-Namen, bei denen das Schema nicht explizit angegeben ist,
	// beziehen.

	// Wenn die Datenbank nicht selbst ein "default schema" unterstuetzt
	// (und welche tut das heutzutage schon ...), setzt Translate() jeweils
	// den Default-Schema-Namen ein.

	// Abfragen und Setzen des Zeichensatzes, mit dem die Datenbank
	// arbeitet

	// Bei Fetch-Operationen werden Strings von diesem Zeichensatz in
	// den aktuellen Zeichensatz der Applikation konvertiert. Bei
	// Insert und Update-Operationen entsprechend umgekehrt.

	// Nach dem Oeffnen einer Connection ist zunaechst ein Default-
	// Zeichensatz (je nach Treiber oder ggf. aus der Datenbank selbst
	// abgefragt) eingestellt, der aber jederzeit und beliebig oft bei
	// geoeffneter Connection umgesetzt werden kann.
	inline void SetCharacterSet(CharSet eNewCharacterSet);
	void SetCharacterSet(const String & rNewCharacterSet);
	CharSet GetCharacterSet() const {return eCharacterSet;}

	virtual void SetAutoCommit(BOOL bAuto) = 0;

	// Starten einer Transaction
	BOOL BeginTrans();

	// Committen einer Transaction
	BOOL CommitTrans();

	// Zuruecksetzen einer Transactionn.
	BOOL RollbackTrans();

	// wird gerade eine Transaktion ausgefuehrt
	BOOL IsAutoCommit() const {return bAutoCommit;}

	// Zugriff auf den Parser (vom Environment ...):
	SdbSqlParser* GetSqlParser() const;

	// Zugriff auf den OptionsParser:
	const CommandParser& GetOptionsParser() const { return aOptionsParser; }

	BOOL HasRelation(const String& rRelName) const;

	//-------------------------------------------------------------
	// von der Implementation zu realisieren:

	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).

	// Cursor, mit dem der Status und die Ergebnismenge
	// des angegebenen SQL-Statements zugegriffen werden koennen.
	// Liefert NULL (und Status in SdbConnection), wenn die Connection
	// nicht geoeffnet ist [Function sequence error].
	virtual SdbCursor* CreateCursor(SdbCursorType eType = SDB_KEYSET,
									UINT16 nOptions = SDB_READONLY);


	// Ausfuehren eines SQLStatements
	virtual BOOL ExecuteSQL(const String& aStatement);

	// liefert eine Beschreibung der Ergebnismenge, ist effizienter als Abfragen der
	// Beschreibung nach einem Open auf einen Cursor, wenn man an der Ergebnismenge
	// nicht interessiert ist
	// bParse gibt an, ob das Ergebnis geparst werden kann
	virtual SdbColumns* DescribeResult(const String& aStatement,
										 BOOL bTable = FALSE);

	// Informationen ueber die Namen der Tabellen und Views
	virtual String GetTableName(USHORT _nPos);
	virtual USHORT GetTableCount();
	// Erzeugen eines Tabellenobjekts
	// a.) zum Anlegen einer neuen Tabelle in der Datenbank
	virtual SdbTable*  CreateTable(const String& _rName);
	// b.) zum Anlegen einer neuen View in der Datenbank
	virtual SdbView*   CreateView(const String& _rName);
	// c.) zum Oeffnen einer bestehenden Tabelle/View aus der Datenbank
	virtual SdbTable*  OpenTable(const String& _rName);
	inline  SdbTable*  OpenTable(USHORT _nPos)
		{return OpenTable(GetTableName(_nPos));}


	// Erzeugen von Feldern, kann nur von einer Verbindung gemacht werden
	// da sie wei, ob Felder CaseSensitive behandelt werden oder nicht
	SdbIdentifierCase GetIdentifierCase() const {return eIdentifierCase;}

	char GetQualifierSeparator() const {return cQualifierSeparator;}
	char GetIdentifierQuote() const {return cIdentifierQuotation;}

	// werden Transaktionen verwendet
	BOOL GetAutoCommitMode() const {return bAutoCommit;}

	virtual SdbColumns* CreateColumns();
	virtual SdbColumns* CreateColumns(const SdbColumns& _rColumns);
	virtual SdbColumn*  CreateColumn(const String& _rName,
									 SdbDatabaseType _eType,
									 UINT32 _nLength = 0, UINT16 _nScale = 0,
									 UINT16 _nFlags = 0);
	virtual SdbColumn*  CreateColumn(const SdbColumn& _rColumn);
#if __PRIVATE
	inline SdbColumn*   CreateColumn(const String& aAlias,const SdbColumn& _rColumn);
#endif

	// Treiber abhaengig
	virtual String IdentifierQuotation(const String& rString) const;
	static  String StringQuotation(const String& rString);
	virtual BOOL   IsSQLNameValid(const String& rString) const;
	virtual BOOL IsCharOk(char c) const;
	// Erzeugt einen neuen Namen falls noetig
	virtual String ConvertName(const String& rName) const;
	virtual String QuoteTableName(const String& rName);
	virtual String CompleteTableName(const String& rName);
	// ueberpruefen, ob eine Column eine gueltigen Datentyp besitzt
	virtual const SdbTypeInfo* GetColumnTypeInfo(const SdbColumn& rColumn);
	virtual const SdbTypeInfo* GetColumnTypeInfo(const String& rTypeName);
	virtual const SdbTypeInfo* GetColumnTypeInfo(const SdbDatabaseType eType);
	virtual const SdbTypeInfo* GetColumnTypeInfo(const String& aTypeName,INT32 nLength,UINT8 nScale,BOOL bNullAllowed,INT32 nJavaType);

	// ein Komandointerpreter mit Argument(en)
	virtual BOOL DoCmd(UINT16 nCmd,const String& rArgs);
	const SdbCursorList& GetCursorList() const { return aCursorList;}
	// Konvertierung von Datum/Zeitfeldern in die treiberspezifische Form
	// Strukturen der unterliegenden DB duerfen (koennen) veraendert werden ?
	virtual BOOL StructureIsReadOnly();

	// Aufdroeseln eines voll qualifizierten Namens in sein Bestandteile (return FALSE wenn rQualifiedName ungueltig)
	BOOL QualifiedNameComponents(const String& rQualifiedName, String& rCatalog, String& rSchema, String& rName) const;
	virtual BOOL IsView(const String& rTableName) const ;

protected:
	// Verbindungen gelten als gleich, wenn die Verbindungsparameter
	// gleich sind.
	virtual BOOL Compare(const SdbObj& _rObj) const;
	virtual BOOL DoRefresh();

	// Neue Verbindungen werden nur mit Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbConnection(SdbEnvironment* pEnv, const String& rOptions);
	virtual ~SdbConnection();

	// Implementierung von Methoden die vom SdbObj erwartet werden
	virtual BOOL Open();
	virtual void Close();

	virtual void RemoveChildren();
	virtual void RemoveChild(SdbObj* pObj);

	// Anforderungen an Treiber
	// virtual void Automatisches Commit
	// COMMIT ausfuehren
	virtual void CommitImpl() = 0;

	// ROLLBACK ausfuehren
	virtual void RollbackImpl() = 0;

	// Verbindung aufbauen
	virtual BOOL OpenImpl() = 0;

	// Verbindung schliessen
	virtual BOOL CloseImpl() = 0;

	// Tabellenobject erstellen
	virtual SdbTable*  CreateTableImpl() = 0;

	virtual SdbView* CreateViewImpl();

	// Cursor erstellen
	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType,UINT16 nOptions) = 0;

	// TypInformation fuellen
	virtual void FillTypeList(SdbTypeInfoList& _rList) = 0;
	virtual void FillTableNames(SdbStringStlMap* _pList) = 0;
	virtual void FillRelations(SdbRelationList& _rList, const String& rName) = 0;

	virtual BOOL IsCapableImpl(SdbCapability eCapability) const;
	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl() = 0;

	// Information ueber Grenzwerte fuer die Datenbank
	// wie zum Beispiel maximale Anzahl Spalten in einer Tabelle
	// Hierbei steht 0 fuer Unbeschraenktheit oder fuer einen
	// unbekannten Grenzwert
	virtual void SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel);

	// Erzeugen von Relationen, default liefert NULL
	virtual SdbRelation* CreateRelationImpl(const String& _rName);
	// ein Komandointerpreter mit Argument(en) : mu von den abgeleiteten Klassen definiert werdem
	virtual BOOL DoCmdImpl(UINT16 nCmd,const String &rArgs);

	// Anzahl der Relationen, ggf. Relationenliste fuellen - fuer alle Tabellen
	virtual void FillAllRelations(SdbRelationList& _rList);

#if __PRIVATE
public:
		// Links, die *zu Beginn* bzw. *am Ende* einer laenger andauernden Ausfuehrung
	// von Datenbankoperationen gerufen werden:
	// wichtig: sicherstellen, da die Umgebung ueberhaupt noch vorhanden ist!!!
	const Link& BusyHdl(){return GetEnvironment()->m_aBusyHdl;}
	const Link& BeginHdl(){return GetEnvironment()->m_aBeginHdl;}
	const Link& EndHdl(){return GetEnvironment()->m_aEndHdl;}

#endif

};

SV_DECL_IMPL_REF(SdbConnection);

// Inline implementation
//------------------------------------------------------------------
inline void SdbConnection::SetCharacterSet(CharSet eNewCharacterSet)
{
	aStatus.Clear();
	eCharacterSet = eNewCharacterSet;
};
#if __PRIVATE

inline SdbColumn*  SdbConnection::CreateColumn(const String& aAlias,const SdbColumn& _rColumn)
{
	SdbColumn *pColumn = CreateColumn(aAlias,_rColumn.GetType(),_rColumn.GetLength(),
									  _rColumn.GetScale(),_rColumn.GetAllFlags());
	pColumn->SetRealName(_rColumn.GetName());
	return pColumn;
};
#endif
#endif


