/*************************************************************************
 *
 *  $RCSfile: sdbcont.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBCONT_HXX
#define _SDB_SDBCONT_HXX 1

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#ifndef _CONTNR_HXX //autogen
#include <tools/contnr.hxx>
#endif

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif

#ifndef _SDB_SDBHASH_HXX
#include "sdbhash.hxx"
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

DECLARE_LIST(SdbFileList, DirEntry*);


#define SDB_CONT_APPEND           CONTAINER_APPEND
#define SDB_CONT_ENTRY_NOTFOUND   CONTAINER_ENTRY_NOTFOUND

class SvRefBase;

//==================================================================
// SdbContainer
//==================================================================
class SdbContainer : private Container
{
public:
	SdbContainer(USHORT _nInitSize = 16, USHORT _nReSize = 16)
		:Container(1024, _nInitSize, _nReSize) {}
	SdbContainer(USHORT _nBlockSize, USHORT _nInitSize, USHORT _nReSize)
		:Container( _nBlockSize, _nInitSize, _nReSize ) {}

public:
	Container::Insert;
	Container::Remove;
	Container::Replace;
	Container::Clear;
	Container::Count;
	Container::GetCurObject;
	Container::GetCurPos;
	Container::GetObject;
	Container::GetPos;
};

//==================================================================
// SdbKeyContainer:
//	Container, der Zugriff sowohl ueber einen Key (String) von Objecten
//	als auch ueber die Position erlaubt
//==================================================================
#ifdef MAC
class SdbKeyContainer : public SdbContainer
#else
class SdbKeyContainer : protected SdbContainer
#endif
{
private:
	SdbStringMap	aMap;

public:
	SdbKeyContainer(BOOL _bCaseSensitive = FALSE,
					 USHORT _nInitSize = 16, USHORT _nReSize = 16)
		:SdbContainer(1024, _nInitSize, _nReSize)
		{ SetCaseSensitive(_bCaseSensitive); }

	SdbKeyContainer(USHORT _nBlockSize, USHORT _nInitSize, USHORT _nReSize, BOOL _bCaseSensitive = FALSE)
		:SdbContainer(_nBlockSize, _nInitSize, _nReSize)
		{ SetCaseSensitive(_bCaseSensitive); }

	BOOL IsCaseSensitive() const { return aMap.IsCaseSensitive(); }
	inline void SetCaseSensitive(BOOL _bCaseSensitive);

public:
	SdbContainer::Count;
	SdbContainer::GetObject;
	SdbContainer::GetPos;

	inline void Clear();
	BOOL InsertAt(const String& _rKey, void* _pObj, ULONG _nIndex = SDB_CONT_APPEND);

	BOOL ReplaceAt(const String& _rOldKey,const String& _rKey,void* _pObj);
	BOOL ReplaceAt(ULONG _nOldIndex, const String& _rKey, void* _pObj);
	BOOL Replace(void* _pOldObj, const String& _rKey, void* _pObj);

	// werden die Objekte case insensitive abgelegt, ist der Returnwert immer UPPER-CASE
	inline String GetKey(void* _pObj) const;
	inline void* Lookup(const String& _rKey) const;

	BOOL RemoveAt(const String& _rKey);
	BOOL RemoveAt(ULONG nIndex);
	BOOL Remove(void* _pObj);

protected:
	virtual void* Find(const String& _rKey) const;
};

#define DECLARE_SDBKEYCONT( ClassName, Type )							\
class ClassName : public SdbKeyContainer								\
{																		\
public:																	\
	SdbKeyContainer::Clear;												\
	SdbKeyContainer::Count;												\
	SdbKeyContainer::SetCaseSensitive;									\
	SdbKeyContainer::IsCaseSensitive;									\
																		\
    ClassName( BOOL _bCaseSensitive = FALSE,							\
			   USHORT _nInitSize = 16,									\
               USHORT _nReSize = 16 ) 									\
		:SdbKeyContainer(_bCaseSensitive, _nInitSize, _nReSize ) {}		\
																		\
	ClassName( USHORT _nBlockSize, USHORT _nInitSize,				    \
               USHORT _nReSize, BOOL _bCaseSensitive = FALSE)			\
		:SdbKeyContainer(_nBlockSize,									\
						 _nInitSize,_nReSize,_bCaseSensitive) {}		\
                                                                        \
    Type GetObject(ULONG _nIndex) const									\
		{return (Type) SdbKeyContainer::GetObject(_nIndex);}			\
																		\
	ULONG GetPos(Type _pObj) const										\
		{return SdbKeyContainer::GetPos((void*)_pObj);}					\
																		\
	BOOL InsertAt(const String& _rKey,									\
			      Type _pObj,											\
				  ULONG _nIndex = SDB_CONT_APPEND)						\
		{return SdbKeyContainer::InsertAt(_rKey, (void*) _pObj,	\
												 _nIndex);}				\
	BOOL ReplaceAt(const String& _rOldKey,const String& _rKey,			\
				   Type _pObj)											\
		{return SdbKeyContainer::ReplaceAt(_rOldKey,					\
												  _rKey,				\
												  (void*) _pObj);}		\
	BOOL ReplaceAt(ULONG _nIndex, const String& _rKey, Type _pObj)		\
		{return SdbKeyContainer::ReplaceAt(_nIndex,				\
												  _rKey,				\
												  (void*) _pObj);}		\
	BOOL Replace(Type _pOldObj, const String& _rKey, Type _pObj)		\
		{return SdbKeyContainer::Replace((void*) _pOldObj,		\
												_rKey, (void*) _pObj);}	\
																		\
	String GetKey(Type _pObj) const										\
		{return SdbKeyContainer::GetKey((void*)_pObj);}					\
	Type Lookup(const String& _rKey) const								\
		{return (Type) SdbKeyContainer::Lookup(_rKey);}					\
																		\
	BOOL RemoveAt(const String& _rKey)									\
		{return SdbKeyContainer::RemoveAt(_rKey);}						\
	BOOL RemoveAt(ULONG _nIndex)										\
		{return SdbKeyContainer::RemoveAt(_nIndex);}					\
	BOOL Remove(Type _pObj)												\
		{return SdbKeyContainer::Remove((void*)_pObj);}					\
};

#define DECL_SDBKEYCONT_VIRT( ClassName, Type )							\
class ClassName : public SdbKeyContainer								\
{																		\
public:																	\
	SdbKeyContainer::Clear;												\
	SdbKeyContainer::Count;												\
	SdbKeyContainer::SetCaseSensitive;									\
	SdbKeyContainer::IsCaseSensitive;									\
																		\
    ClassName( BOOL _bCaseSensitive = FALSE,							\
			   USHORT _nInitSize = 16,									\
               USHORT _nReSize = 16 ) 									\
		:SdbKeyContainer(_bCaseSensitive, _nInitSize, _nReSize ) {}		\
																		\
	ClassName( USHORT _nBlockSize, USHORT _nInitSize,				    \
               USHORT _nReSize, BOOL _bCaseSensitive = FALSE)			\
		:SdbKeyContainer(_nBlockSize,									\
						 _nInitSize,_nReSize,_bCaseSensitive) {}		\
                                                                        \
    Type GetObject(ULONG _nIndex) const									\
		{return (Type) SdbKeyContainer::GetObject(_nIndex);}			\
																		\
	ULONG GetPos(Type _pObj) const										\
		{return SdbKeyContainer::GetPos((void*)_pObj);}					\
																		\
	virtual BOOL InsertAt(const String& _rKey,							\
			      Type _pObj,											\
				  ULONG _nIndex = SDB_CONT_APPEND);						\
	virtual BOOL ReplaceAt(const String& _rOldKey,const String& _rKey,	\
				   Type _pObj);										\
	virtual BOOL ReplaceAt(ULONG _nIndex, const String& _rKey, Type _pObj);		\
	virtual BOOL Replace(Type _pOldObj, const String& _rKey, Type _pObj);		\
																		\
	String GetKey(Type _pObj) const										\
		{return SdbKeyContainer::GetKey((void*)_pObj);}					\
	Type Lookup(const String& _rKey) const								\
		{return (Type) SdbKeyContainer::Lookup(_rKey);}					\
																		\
	virtual BOOL RemoveAt(const String& _rKey);							\
	virtual BOOL RemoveAt(ULONG _nIndex);								\
	virtual BOOL Remove(Type _pObj);									\
};

#define IMPL_SDBKEYCONT_VIRT( ClassName, Type )							\
	BOOL ClassName::InsertAt(const String& _rKey,						\
			      Type _pObj,											\
				  ULONG _nIndex)										\
		{return SdbKeyContainer::InsertAt(_rKey, (void*) _pObj,			\
												 _nIndex);}				\
	BOOL ClassName::ReplaceAt(const String& _rOldKey,const String& _rKey,\
				   Type _pObj)											\
		{return SdbKeyContainer::ReplaceAt(_rOldKey,					\
												  _rKey,				\
												  (void*) _pObj);}		\
	BOOL ClassName::ReplaceAt(ULONG _nIndex, const String& _rKey, Type _pObj)		\
		{return SdbKeyContainer::ReplaceAt(_nIndex,						\
												  _rKey,				\
												  (void*) _pObj);}		\
	BOOL ClassName::Replace(Type _pOldObj, const String& _rKey, Type _pObj)		\
		{return SdbKeyContainer::Replace((void*) _pOldObj,				\
												_rKey, (void*) _pObj);}	\
																		\
	BOOL ClassName::RemoveAt(const String& _rKey)						\
		{return		SdbKeyContainer::RemoveAt(_rKey);}					\
	BOOL ClassName::RemoveAt(ULONG _nIndex)								\
		{return		SdbKeyContainer::RemoveAt(_nIndex);}				\
	BOOL ClassName::Remove(Type _pObj)									\
		{return		SdbKeyContainer::Remove((void*)_pObj);}


//==================================================================
// SdbKeyRefContainer:
//	Container, der Zugriff sowohl ueber einen Key (String) von Objecten
//	als auch ueber die Position erlaubt, Objecte muessen von SvRefBase
//  abgeleitet sein
//==================================================================
#ifdef MAC
class SdbKeyRefContainer : public SdbContainer
#else
class SdbKeyRefContainer : protected SdbContainer
#endif
{
private:
	SdbStringRefMap	aMap;
#if SUPD<540
	BOOL			bCaseSensitive : 1;		// werden die Objekte CaseSensitive abgelegt
#endif // SUPD<540

public:
	SdbKeyRefContainer(BOOL _bCaseSensitive = FALSE,
					USHORT _nInitSize = 16, USHORT _nReSize = 16)
		:SdbContainer(1024, _nInitSize, _nReSize)
#if SUPD<540
		,bCaseSensitive(_bCaseSensitive)
#endif // SUPD>=540
		{ SetCaseSensitive(_bCaseSensitive); }

	SdbKeyRefContainer(USHORT _nBlockSize, USHORT _nInitSize, USHORT _nReSize, BOOL _bCaseSensitive = FALSE)
		:SdbContainer(_nBlockSize, _nInitSize, _nReSize)
#if SUPD<540
		,bCaseSensitive(_bCaseSensitive)
#endif // SUPD>=540
		{ SetCaseSensitive(_bCaseSensitive); }

	BOOL IsCaseSensitive() const { return aMap.IsCaseSensitive();}

	inline void SetCaseSensitive(BOOL _bCaseSensitive);

public:
	SdbContainer::Count;
	SdbContainer::GetObject;
	SdbContainer::GetPos;

	inline void Clear();
	BOOL InsertAt(const String& _rKey, SvRefBase* _pObj, ULONG _nIndex = SDB_CONT_APPEND);
	BOOL ReplaceAt(const String& _rOldKey,const String& _rKey, SvRefBase* _pObj);
	BOOL ReplaceAt(ULONG _nOldIndex, const String& _rKey, SvRefBase* _pObj);
	BOOL Replace(SvRefBase* _pOldObj, const String& _rKey, SvRefBase* _pObj);

	// werden die Objekte case insensitive abgelegt, ist der Returnwert immer UPPER-CASE
	inline String GetKey(SvRefBase* _pObj) const;
	inline SvRefBase* Lookup(const String& _rKey) const;

	BOOL RemoveAt(const String& _rKey);
	BOOL RemoveAt(ULONG nIndex);
	BOOL Remove(SvRefBase* _pObj);

protected:
	virtual SvRefBase* Find(const String& _rKey) const;
};

#define DECLARE_SDBKEYREFCONT( ClassName, Type )						\
class ClassName : public SdbKeyRefContainer								\
{																		\
public:																	\
	SdbKeyRefContainer::Clear;											\
	SdbKeyRefContainer::Count;											\
	SdbKeyRefContainer::SetCaseSensitive;								\
	SdbKeyRefContainer::IsCaseSensitive;								\
																		\
    ClassName( BOOL _bCaseSensitive = FALSE,							\
			   USHORT _nInitSize = 16,									\
               USHORT _nReSize = 16 ) 									\
		:SdbKeyRefContainer(_bCaseSensitive, _nInitSize, _nReSize ) {}	\
																		\
	ClassName( USHORT _nBlockSize, USHORT _nInitSize,				    \
               USHORT _nReSize, BOOL _bCaseSensitive = FALSE)			\
		:SdbKeyRefContainer(_nBlockSize,								\
						 _nInitSize,_nReSize,_bCaseSensitive) {}		\
                                                                        \
    Type GetObject(ULONG _nIndex) const									\
		{return (Type) SdbKeyRefContainer::GetObject(_nIndex);}			\
																		\
	ULONG GetPos(Type _pObj) const										\
		{return SdbKeyRefContainer::GetPos((SvRefBase*)_pObj);}			\
																		\
	BOOL InsertAt(const String& _rKey,									\
			      Type _pObj,											\
				  ULONG _nIndex = SDB_CONT_APPEND)						\
		{return SdbKeyRefContainer::InsertAt(_rKey, (SvRefBase*) _pObj,	\
												 _nIndex);}				\
	BOOL ReplaceAt(const String& _rOldKey,const String& _rKey,			\
				   Type _pObj)											\
		{return SdbKeyRefContainer::ReplaceAt(_rOldKey,					\
												  _rKey,				\
												  (SvRefBase*) _pObj);}	\
	BOOL ReplaceAt(ULONG _nIndex, const String& _rKey, Type _pObj)\
		{return SdbKeyRefContainer::ReplaceAt(_nIndex,					\
												  _rKey,				\
												  (SvRefBase*) _pObj);}	\
	BOOL Replace(Type _pOldObj, const String& _rKey, Type _pObj)	\
		{return SdbKeyRefContainer::Replace((SvRefBase*) _pOldObj,		\
												_rKey, (SvRefBase*) _pObj);}	\
																		\
	String GetKey(Type _pObj) const										\
		{return SdbKeyRefContainer::GetKey((SvRefBase*)_pObj);}			\
	Type Lookup(const String& _rKey) const								\
		{return (Type) SdbKeyRefContainer::Lookup(_rKey);}				\
																		\
	BOOL RemoveAt(const String& _rKey)									\
		{return SdbKeyRefContainer::RemoveAt(_rKey);}					\
	BOOL RemoveAt(ULONG _nIndex)										\
		{return SdbKeyRefContainer::RemoveAt(_nIndex);}					\
	BOOL Remove(Type _pObj)												\
		{return SdbKeyRefContainer::Remove((SvRefBase*)_pObj);}			\
};

#define DECL_SDBKEYREFCONT_VIRT( ClassName, Type )						\
class ClassName : public SdbKeyRefContainer								\
{																		\
public:																	\
	SdbKeyRefContainer::Clear;											\
	SdbKeyRefContainer::Count;											\
	SdbKeyRefContainer::SetCaseSensitive;								\
	SdbKeyRefContainer::IsCaseSensitive;								\
																		\
    ClassName( BOOL _bCaseSensitive = FALSE,							\
			   USHORT _nInitSize = 16,									\
               USHORT _nReSize = 16 ) 									\
		:SdbKeyRefContainer(_bCaseSensitive, _nInitSize, _nReSize ) {}	\
																		\
	ClassName( USHORT _nBlockSize, USHORT _nInitSize,				    \
               USHORT _nReSize, BOOL _bCaseSensitive = FALSE)			\
		:SdbKeyRefContainer(_nBlockSize,								\
						 _nInitSize,_nReSize,_bCaseSensitive) {}		\
                                                                        \
    Type GetObject(ULONG _nIndex) const									\
		{return (Type) SdbKeyRefContainer::GetObject(_nIndex);}			\
																		\
	ULONG GetPos(Type _pObj) const										\
		{return SdbKeyRefContainer::GetPos((void*)_pObj);}				\
																		\
	virtual BOOL InsertAt(const String& _rKey,							\
			      Type _pObj,											\
				  ULONG _nIndex = SDB_CONT_APPEND);						\
	virtual BOOL ReplaceAt(const String& _rOldKey,const String& _rKey,	\
				   Type _pObj);									\
	virtual BOOL ReplaceAt(ULONG _nIndex, const String& _rKey, Type _pObj);		\
	virtual BOOL Replace(Type _pOldObj, const String& _rKey, Type _pObj);		\
																		\
	String GetKey(Type _pObj) const										\
		{return SdbKeyRefContainer::GetKey((SvRefBase*)_pObj);}			\
	Type Lookup(const String& _rKey) const								\
		{return (Type) SdbKeyRefContainer::Lookup(_rKey);}				\
																		\
	virtual BOOL RemoveAt(const String& _rKey);							\
	virtual BOOL RemoveAt(ULONG _nIndex);								\
	virtual BOOL Remove(Type _pObj);									\
};

#define IMPL_SDBKEYREFCONT_VIRT( ClassName, Type )						\
	BOOL ClassName::InsertAt(const String& _rKey,						\
			      Type _pObj,											\
				  ULONG _nIndex)										\
		{return SdbKeyRefContainer::InsertAt(_rKey, (SvRefBase*) _pObj,	\
												 _nIndex);}				\
	BOOL ClassName::ReplaceAt(const String& _rOldKey,const String& _rKey,\
				   Type _pObj)											\
		{return SdbKeyRefContainer::ReplaceAt(_rOldKey,					\
												  _rKey,				\
												  (SvRefBase*) _pObj);}		\
	BOOL ClassName::ReplaceAt(ULONG _nIndex, const String& _rKey, Type _pObj)		\
		{return SdbKeyRefContainer::ReplaceAt(_nIndex,					\
												  _rKey,				\
												  (SvRefBase*) _pObj);}		\
	BOOL ClassName::Replace(Type _pOldObj, const String& _rKey, Type _pObj)		\
		{return SdbKeyRefContainer::Replace((SvRefBase*) _pOldObj,		\
												_rKey, (SvRefBase*) _pObj);}	\
																		\
	BOOL ClassName::RemoveAt(const String& _rKey)						\
		{return		SdbKeyRefContainer::RemoveAt(_rKey);}				\
	BOOL ClassName::RemoveAt(ULONG _nIndex)								\
		{return		SdbKeyRefContainer::RemoveAt(_nIndex);}				\
	BOOL ClassName::Remove(Type _pObj)									\
		{return		SdbKeyRefContainer::Remove((SvRefBase*)_pObj);}

//==================================================================
// SdbFileContainer:
//	Container, der Zugriff sowohl ueber einen DirEntry von Objecten
//	als auch ueber die Position erlaubt
//==================================================================
#ifdef MAC
class SdbFileContainer : public SdbContainer
#else
class SdbFileContainer : protected SdbContainer
#endif
{
private:
	SdbFileList		aList;

public:
	SdbFileContainer(USHORT _nInitSize = 16, USHORT _nReSize = 16)
		:SdbContainer(1024, _nInitSize, _nReSize){}


	SdbFileContainer(USHORT _nBlockSize, USHORT _nInitSize, USHORT _nReSize)
		:SdbContainer(_nBlockSize, _nInitSize, _nReSize) {}

	virtual ~SdbFileContainer();

public:
	SdbContainer::Count;
	SdbContainer::GetObject;
	SdbContainer::GetPos;

	void Clear();
	BOOL InsertAt(const DirEntry& _rKey, void* _pObj, ULONG _nIndex = SDB_CONT_APPEND);

	BOOL ReplaceAt(const DirEntry& _rOldKey, const DirEntry& _rKey, void* _pObj);
	BOOL ReplaceAt(ULONG _nOldIndex, const DirEntry& _rKey, void* _pObj);
	BOOL Replace(void* _pOldObj, const DirEntry& _rKey, void* _pObj);

	// werden die Objekte case insensitive abgelegt, ist der Returnwert immer UPPER-CASE
	DirEntry GetKey(void* _pObj) const;
	void* Lookup(const DirEntry& _rKey) const;

	BOOL RemoveAt(const DirEntry& _rKey);
	BOOL RemoveAt(ULONG _nIndex);
	BOOL Remove(void* _pObj);
};

#define DECLARE_SDBFILECONT( ClassName, Type )							\
class ClassName : public SdbFileContainer								\
{																		\
public:																	\
	SdbFileContainer::Clear;											\
	SdbFileContainer::Count;											\
																		\
    ClassName( USHORT _nInitSize = 16,									\
               USHORT _nReSize = 16 ) 									\
		:SdbFileContainer(_nInitSize, _nReSize ) {}						\
																		\
	ClassName( USHORT _nBlockSize, USHORT _nInitSize,				    \
               USHORT _nReSize)											\
		:SdbFileContainer(_nBlockSize,									\
						 _nInitSize,_nReSize) {}						\
                                                                        \
    Type GetObject(ULONG _nIndex) const									\
		{return (Type) SdbFileContainer::GetObject(_nIndex);}			\
																		\
	ULONG GetPos(Type _pObj) const										\
		{return SdbFileContainer::GetPos((void*)_pObj);}				\
																		\
    BOOL InsertAt(const DirEntry& _rKey,								\
			      Type _pObj,											\
				  ULONG _nIndex = SDB_CONT_APPEND)						\
		{return SdbFileContainer::InsertAt(_rKey, (void*) _pObj,		\
												 _nIndex);}				\
	BOOL ReplaceAt(const DirEntry& _rOldKey,const DirEntry& _rKey,		\
				   void* _pObj)											\
		{return SdbFileContainer::ReplaceAt(_rOldKey,					\
												  _rKey,				\
												  (void*) _pObj);}		\
	BOOL ReplaceAt(ULONG _nIndex, const DirEntry& _rKey, void* _pObj)	\
		{return SdbFileContainer::ReplaceAt(_nIndex,					\
												  _rKey,				\
												  (void*) _pObj);}		\
	BOOL Replace(void* _pOldObj, const DirEntry& _rKey, void* _pObj)	\
		{return SdbFileContainer::Replace((void*) _pOldObj,				\
												_rKey, (void*) _pObj);}	\
																		\
	DirEntry GetKey(Type _pObj) const									\
		{return SdbFileContainer::GetKey((void*)_pObj);}				\
	Type Lookup(const DirEntry& _rKey) const							\
		{return (Type) SdbFileContainer::Lookup(_rKey);}				\
																		\
	BOOL RemoveAt(const DirEntry& _rKey)								\
		{return SdbFileContainer::RemoveAt(_rKey);}						\
	BOOL RemoveAt(ULONG _nIndex)										\
		{return SdbFileContainer::RemoveAt(_nIndex);}					\
	BOOL Remove(Type _pObj)												\
		{return SdbFileContainer::Remove((void*)_pObj);}				\
};

//------------------------------------------------------------------
inline void SdbKeyContainer::SetCaseSensitive(BOOL _bCaseSensitive)
{
	aMap.SetCaseSensitive(_bCaseSensitive);
}

//------------------------------------------------------------------
inline void SdbKeyContainer::Clear()
{
	SdbContainer::Clear();
	aMap.Clear();
}

//------------------------------------------------------------------
inline String SdbKeyContainer::GetKey(void* _pObj) const
{
	return aMap.GetKey(_pObj);
}

//------------------------------------------------------------------
inline void* SdbKeyContainer::Lookup(const String& _rKey) const
{
	return Find(_rKey);
}


//------------------------------------------------------------------
inline void SdbKeyRefContainer::SetCaseSensitive(BOOL _bCaseSensitive)
{
	DBG_ASSERT(!Count(), "SdbKeyRefContainer::SetCaseSensitive -> Container nicht leer");
#if SUPD<540
	bCaseSensitive = _bCaseSensitive;
#endif // SUPD<540
	aMap.SetCaseSensitive(_bCaseSensitive);
}

//------------------------------------------------------------------
inline void SdbKeyRefContainer::Clear()
{
	SdbContainer::Clear();
	aMap.Clear();
}

//------------------------------------------------------------------
inline String SdbKeyRefContainer::GetKey(SvRefBase* _pObj) const
{
	return aMap.GetKey(_pObj);
}

//------------------------------------------------------------------
inline SvRefBase* SdbKeyRefContainer::Lookup(const String& _rKey) const
{
	return Find(_rKey);
}

#endif
