/*************************************************************************
 *
 *  $RCSfile: sdbcurs.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBCURS_HXX
#define _SDB_SDBCURS_HXX 1

#ifndef _COM_SUN_STAR_LANG_ILLEGALARGUMENTEXCEPTION_HPP_
#include <com/sun/star/lang/IllegalArgumentException.hpp>
#endif

#ifndef _SDB_SDBCOL_HXX
#include "sdbcol.hxx"
#endif

#ifndef _SDB_ROW_HXX
#include "row.hxx"
#endif

#ifndef _VOS_MACROS_HXX
#include <vos/macros.hxx>
#endif

#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _LINK_HXX //autogen
#include <tools/link.hxx>
#endif

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

#include "sdbcancl.hxx"

class SdbSqlParseNode;
class SdbConnection;

namespace vos {
class OMutex;
}

typedef vos::OMutex SdbCursorLock;

// Navigation in der Ergebnismenge
enum SdbCursorPosition
{
	SDB_POS_NEXT,				// Naechster (Offset wird ignoriert)
	SDB_POS_PRIOR,				// Vorheriger (Offset wird ignoriert)
	SDB_POS_FIRST,				// Erster (Offset wird ignoriert)
	SDB_POS_LAST,				// Letzter (Offset wird ignoriert)
	SDB_POS_RELATIVE,			// Offset relativ zum aktuellen
	SDB_POS_ABSOLUTE			// Absolute Row # (beginnt mit 1)
};

enum SdbCursorCapability
{
	SDB_CAP_UPDATE_POSITIONED,		// kann der Treiber selbst positionierte updates fahren?
		// Wie laesst sich der Cursor bewegen?
	SDB_CAP_FETCH,					// Ist ein einzelnes Fetch (mit anderen Aktivitaeten
									// dazwischen, insb. Interaktion) zulaessig?
									// Ansonsten muessen leider alle Datensaetze auf
									// einmal geholt werden (auch mit Fetch, aber ohne
									// Unterbrechung, nur vorwaerts, moeglichst schnell
									// und der Cursor muss dann sofort wieder geschlossen werden)!

	SDB_CAP_FETCH_NEXT,				// Laesst sich der Cursor vorwaerts, rueckwaerts,
	SDB_CAP_FETCH_PRIOR,			// relativ, absolut, an den Anfang, ans
	SDB_CAP_FETCH_FIRST,			// Ende oder auf eine bestimmte Bookmark bewegen?
	SDB_CAP_FETCH_LAST,
	SDB_CAP_FETCH_RELATIVE,
	SDB_CAP_FETCH_ABSOLUTE,
	SDB_CAP_FETCH_BOOKMARK,

	// Bookmarks verfuegbar?
	SDB_CAP_BOOKMARKS,
	SDB_CAP_UPDATE_INSERTED_ROW
};

#define SDB_CURRENT_ROW_UNDEFINED (-2)
#define SDB_CURRENT_ROW_BOF (-1)

DECLARE_LIST(SdbSqlParseNodeList, SdbSqlParseNode*);
class SdbSqlParseTreeIterator;

// Class SdbCursor
//
// Beschreibung:
//	Die Erzeugung eines SdbCursor-Objektes bereitet eine
//	SQL-Anweisung (vor allem SELECT, aber auch INSERT,
//	UPDATE, DELETE, und auch DDL-Anweisungen) fuer die Aus-
//	fuehrung vor (PREPARE). Anschliessend kann das Statement
//	mit Hilfe von 'Execute' beliebig oft ausgefuehrt werden.

//	Jeweils vor der Ausfuehrung eines Statements koennen
//	ggf. Parameter gebunden werden. Parameter sind im
//	Statement-String durch ein Fragezeichen gekennzeichnet.

//	Bei der Ausfuehrung einer Anfrage auf der Datenbank
//	entsteht eine (geordnete) Folge von Datensaetzen
//	(Rows), etwa vergleichbar einer Listenstruktur.
//	SdbStatement stellt eine Art "Fenster" auf diese Folge
//	von Datensaetzen zur Verfuegung; es macht jeweils eine
//	Zeile (Row) aus der Liste zur Zeit sichtbar
//	(Datenbank-CURSOR-Konzept).

//	Gruppen von Methoden:
//	a) Vorbereitung und Ausfuehrung des Statements:
//	Ein SdbCursor bereitet das bei der Erzeugung
//	angegebene Statement automatisch vor. Das unveraenderte
//	Statement kann spaeter erneut ausgefuehrt werden. Die
//	Struktur der Ergebnismenge wird bei der Erzeugung des
//	SdbStatements festgelegt und bleibt danach unveraendert.

//	b) Bindung einer Row fuer die Bereitstellung von
//	Parametern
//	SdbCursor stellt selbst eine Row (ohne
//	Strukturinformation und ohne gebundene Variable) zur
//	Verfuegung, die vom Aufrufer gefuellt werden kann, oder
//	es kann eine vom Aufrufer vorbereitete Row komplett
//	gebunden werden.

//	c) Bindung von Rows fuer die Aufnahme von Ergebnissen
//	der Anfrage
//	Wenn bei einer Fetch-Operation aktuell keine Row
//	gebunden ist, alloziert SdbCursor selbst eine.

//	d) Navigation in der Ergebnismenge:
//	Positionierung an Anfang und Ende der Liste, Vorwaerts-
//	und Rueckwaertsblaettern sowie direkte Positionierung
//	(auf n-te Row).

//	f) Sonstiges
//
class SdbCursor : public SdbObj, public SdbCancelable
{
	friend class SdbConnection;
	friend class OValueSet;
	friend class OKeySet;
	friend class OBookmarkSet;
	friend class ORowCounter;
	friend class ORowCache;
	friend class SdbPredicateCompiler;

public:
	enum SdbCursorMode
	{
		SDB_MOD_NONE,	 // unbekannt
		SDB_MOD_NEW,	 // aktueller Satz ist eingefgt
		SDB_MOD_EDIT,    // aktueller Satz wird editiert
		SDB_MOD_INVALID	 // aktueller Satz ist nicht mehr fr
						 // den Cursor sichtbar
	};

	// Thread-Sicherheit
	class SdbCursorGuard : public NAMESPACE_VOS(OClearableGuard)
	{
	public:
		inline SdbCursorGuard(SdbCursor* pCurs);
	};
	friend class SdbCursor::SdbCursorGuard;

public:
	TYPEINFO();

	// Zeigt an, ob der Cursor bereits am Ende des Resultsets war, un die Zaehlung der
	// der Rows abgeschlossen hat
	BOOL IsRowCountFinal() const {return bRowCountFinal;}

	// Aktuelle Position neu lesen
	BOOL Refresh() {return MoveRel(0);}
	UINT32 RowCount() const {return (nRowCount < 0) ? 0 : nRowCount;}

	// Zugriff auf den Vater
	virtual SdbConnection* GetConnection() const;

	// Caching:
	// CacheSize <= 2 -> kein Caching
	virtual void SetCacheSize(UINT16 nCacheSize);
	inline UINT16 GetCacheSize() const;

	// Preparation:
	// Vorbereitung der SQL-Anfrage fuer die Ausfuehrung, wichtig bei Parametern
	virtual BOOL Prepare(const String& rSQL, BOOL bTable = FALSE);	// kann ein Statement

	// Ist das Statement zur Ausfuehrung vorbereitet?
	BOOL IsPrepared() const {return bPrepared;}

	// Ausfuehrung der SQL-Anfrage (nach erfolgter Bindung
	// aller benoetigten Parameter).
	virtual BOOL Open() { return Open(SDB_CURRENT,SDB_CURRENT_OPT); };
	virtual BOOL Open(SdbCursorType eType,
					  UINT16 eOpt);	// bereits Prepared!
	virtual BOOL Open(const String& rSQL, BOOL bTable = FALSE,
					  SdbCursorType eType = SDB_CURRENT,
					  UINT16 eOpt = SDB_CURRENT_OPT);	// kann ein Statement oder eine Tabelle sein

	// Cursor schliessen. Durch Aufruf von Execute kann das
	// Statement (mit gleichen oder verschiedenen Parametern)
	// erneut ausgefuehrt werden.

	// Falls noch nicht alle Rows der Ergebnismenge abgeholt
	// wurden, werden diese ignoriert.

	// Nach Close ist die Strukturbeschreibung nicht mehr zugreifbar.
	// Gebundene Rows werden durch Close zerstoert. Wenn eine gebundene
	// Row nicht zerstoert werden soll, so muss die Bindung vor dem Close
	// mit Release... aufgehoben werden.
	virtual void Close();

	// Manipulation einer Datenbanktabelle ohne SQL-Statements -- zur Zeit nur fr Files

	/**	Bricht die Ausfuehrung des Statements ab.
		Kann waehrend der Ausfuehrung eines Statements (aus dem
		Busy Callback heraus bzw. aus einem anderen Thread) ausgefuehrt werden!
		Die Default-Implementierung benutzt CancelExecution, allerdings nur, wenn
		gerade ein abbrechbarer Vorgang laeuft (was abgeleitete Klassen durch
		Verwendung von BeginCancelable festlegen). Im Normalfall sollte das waehrend
		Open der Fall sein.
	*/
	virtual void Cancel();

	// Die aktuelle Parameter Row.
	// Kann NULL sein, wenn die Row (noch) nicht gebunden ist
	// (z. B. weil sie gerade "Released" worden ist).

	// Wenn der Aufrufer die Row verwenden moechte, muss
	// er sich bei ihr registrieren (AddReference), sonst
	// ist die Referenz nach ReleaseParameterRow() ggf. ungueltig!
	ODbRowRef GetParameterRow() const {return aParameterRow;}
	SdbColumnsRef GetParameterColumns() const {return aParameterColumns;}

	// Legt eine leere Row (ohne Variable) fuer die Aufnahme
	// von Parametern an.

	// Vor dem Fuellen einer solchen Row muss sichergestellt
	// werden, dass die Strukturinformation zu den Parametern
	// (ParameterColumns) richtig aufgesetzt ist. Wenn gewuenscht (Parmameter
	// bAllocateVariables) werden Variable zu saemtlichen
	// Feldern automatisch alloziert und gebunden (Default: Ja).

	// c) Ergebnismenge:
	// Die aktuelle (gebundene) Row der Ergebnismenge
	// (Cursor-Position).

	// Kann NULL sein, wenn die Row (noch) nicht gebunden ist
	// (z. B. weil sie gerade "Released" worden ist).

	// Wenn bei einer Fetch-Operation aktuell keine Row
	// gebunden ist, alloziert SdbCursor selbst eine.

	// Wenn der Aufrufer die Row verwenden moechte, muss
	// er sich bei ihr registrieren (AddReference), sonst
	// ist die Referenz nach ReleaseResultRow() ggf. ungueltig!
	const ODbRowRef& GetRow() const { return aCurrentRow; }

	// Die Struktur der Ergebnismenge (Columns).
	// ist bekannt nach einem Open
	const SdbColumnsRef& GetColumns() const {return aResultColumns;}

	// Update interface
	// Ein Cursor ist updatebar, wenn der Treiber dazu in der Lage ist,
	// und wenn ReadOnly nicht als Option angegeben wurden
	// Kann der Cursor Updaten?
	BOOL IsUpdatable() const {return nPrivileges > SDB_PR_READ;}
	BOOL IsUpdateAllowed() const;
		// nur ein NEIN als Antwort ist verlaesslich, ein JA bedeutet eigentlich VIELLEICHT ....

	// welche Updatemoeglichkeiten
	BOOL IsAppendOnly() const {return (nOptions & SDB_APPENDONLY) == SDB_APPENDONLY;}
	BOOL CanAppend() const {return (nPrivileges & SDB_PR_INSERT) == SDB_PR_INSERT;}
	BOOL CanUpdate() const {return (nPrivileges & SDB_PR_UPDATE) == SDB_PR_UPDATE;}
	BOOL CanDelete() const {return (nPrivileges & SDB_PR_DELETE) == SDB_PR_DELETE;}

	// Umschalten des Updatemodus, Cursor muss offen sein, darf nicht appendonly
	// oder forwardonly sein !!!
	BOOL SetUpdatable(BOOL bUpdate);

	// Callback fuer Updates und Deletes:
	// fuehrt eine der beiden Aktionen dazu,
	// dass mehrere Datensaetze von einer der beiden Aktionen betroffen ist
	// wird dieser Callback ausgefuehrt
	// Returnwerte beachten:	0: Update/Delete wird abgebrochen
	//							!0: Update/Delete wird durchgefuehrt
	// Parameter des Linkhaendlers UINT32*:
	//							RowCount der betroffenen Rows

	void  SetRowCountHdl(const Link& rLink) {aRowCountHdl = rLink; }
	const Link&	GetRowCountHdl() const { return aRowCountHdl; }

	// Eine neue Row ans Ende hinzufuegen.
	// Die Row ist anschliessend in Bearbeitung.
	// Nach der Bearbeitung muss UpdateRow oder CancelRow aufgerufen werden.
	virtual BOOL AppendRow();

	// Autoinkrementwerte fuer einen Row setzen
	void PrepareAppendRow(const ODbRowRef& rRow);

	// Beginn der Bearbeitung der aktuellen Row:
	// Bewirkt - je nach Concurrency - Lock oder Vorbereitung der Bearbeitung
	// Nach der Bearbeitung muss UpdateRow oder CancelRow aufgerufen werden.
	virtual BOOL EditRow();

	// Abbruch der Bearbeitung der aktuellen Row. Vorgenommene Aenderungen
	// werden nicht in die Datenbank uebernommen, der urspruengliche Zustand
	// der Row wird wiederhergestellt.
	virtual void CancelRow();

	// Ende der Bearbeitung (auch automatisch durch Neupositionierung)
	// Vorgenommene Aenderungen werden in die Datenbank uebernommen.
	virtual BOOL UpdateRow();

	// Lschen des aktuellen Satzes auf der DB
	virtual BOOL DeleteRow();

	// Ist der aktuelle Satz hinzugefuegt
	BOOL IsModeNew()   const {return eMode == SDB_MOD_NEW;}

	// Ist der aktuelle Satz gueltig:
	// Ein Satz ist ungueltig, wenn er beispielsweise von einem anderen Benutzer geloescht worden
	// ist
	BOOL IsValid() const {return eMode != SDB_MOD_INVALID;}
	BOOL IsNotValid() const {return eMode == SDB_MOD_INVALID;}

	// Befinden wir uns im Bearbeitungs-Modus
	BOOL IsModeEdit()  const {return eMode == SDB_MOD_EDIT;}

	// d) Navigation in der Ergebnismenge
	// Steht der Cursor vor dem ersten oder hinter dem
	// letzten Datensatz (EOF/BOF)?

	// In diesem Fall ist der Inhalt der CurrentRow ungueltig.
	virtual BOOL IsBOF() const;			  // Oberhalb der ersten Position
	virtual BOOL IsEOF() const;			  // Unterhalb der letzten Position
	virtual BOOL IsOnFirst() const;
	virtual BOOL IsOnLast() const;
	inline BOOL IsOffRange() const;
	inline BOOL IsInRange() const;

	INT32 GetCurrentPos() const;

	inline BOOL IsCancelable();		// bringt ein Aufruf von Cancel irgend etwas ?
	/**	Zugriffe aus verschiedenen Threads erlaubt (Der SdbCursor selber ist auf jeden Fall Thread-safe, aber vielleicht
		die Implementierung nicht - wie zum Beispiel DAO.)
	*/
	inline BOOL IsThreadsafe() { return bThreadSafe; }

	// liefert die Bookmark der zuletzt modifizierten row
	inline const ::com::sun::star::uno::Any& GetLastModified() const;


	virtual ODbVariant* CreateVariant() const;

	// Navigation
	BOOL First() {return Move(SDB_POS_FIRST,1);}
	BOOL Next()  {return Move(SDB_POS_NEXT,1);}
	BOOL Prev()  {return Move(SDB_POS_PRIOR,1);}
	BOOL Last()  {return Move(SDB_POS_LAST,1);}

	BOOL MoveAbs(UINT32 nPos) {return Move(SDB_POS_ABSOLUTE, nPos);}	// erste Satz hat die Position 1
	BOOL MoveRel(INT32  nOffset) {return Move(SDB_POS_RELATIVE, nOffset);}

	//
	BOOL MoveBookmark(const ::com::sun::star::uno::Any& rBookmark) throw( ::com::sun::star::lang::IllegalArgumentException );


	// Methode zum Festellen des RowCounts, ist dann sinnvoll,
	// wenn der Cursor noch nicht ans Ende bewegt wurde. In diesem Falle kennt
	// der Cursor den RowCount noch nicht abschliessend
	virtual void DetermineRowCount();


	// f) Sonstiges:
	// Das zugrundeliegende SQL-Statement
	// ... als String:
	const String& SQLStatement() const {return aSQLStatement;}

	// ... als Parse Tree (NULL, wenn Statement leer):
	const SdbSqlParseNode* SQLParseNode() const {return pSQLParseNode;}

	const SdbSqlParseTreeIterator*	SQLParseIterator() const {return pParseIterator;}

	// Der Name des Cursors (wenn nicht explizit
	// gesetzt, wird dieser automatisch generiert);
	// Der gesetzte Name wird beim Prepare verwendet und
	// darf danach nicht mehr veraendert werden.
	// Wenn der Name nicht explizit gesetzt wurde, liefert
	// Name() einen leeren String, wenn/solange das Statement
	// noch nicht geoeffnet ist.
	const String& Name() const {return aName;}

	// Legt die Maximale Laenge der zu fetchenden Felder einer Resultrow fest.
	// default Wert ist 0 -> Originallaenge verwenden
	UINT32	GetMaxFieldLen() const { return nMaxFieldLen;}
	void	SetMaxFieldLen(UINT32 nFieldLen) { nMaxFieldLen = nFieldLen;}

	// Legt die Maximale Anzahl der zu fetchenden Zeilen fest.
	// default Wert ist 0 -> Alle Rows uebertragen
	UINT32	GetMaxRows() const { return nMaxRows;}
	void	SetMaxRows(UINT32 nRows) { nMaxRows = nRows;}

protected:
	// Konstruktoren nur aus SdbConnection heraus benutzbar
	// (dort Factory CreateCursor).
	SdbCursor(SdbConnection* pConnection,
			  SdbCursorType eType,
			  UINT16 nOptions);

	// Instanzen eines Cursors koennen nur dynamisch (mit new
	// ueber die Factory in SdbConnection) angelegt werden.
	// Anstelle von delete ist die Methode Delete zu benutzen!
	virtual ~SdbCursor();

	// Cursor gelten als gleich, wenn die zugrundeliegenden SQL-Statements
	// gleich sind.
	BOOL Compare(const SdbObj& _rObj) const;

	virtual BOOL PrepareImpl() = 0;
	virtual BOOL OpenImpl(SdbCursorType eType = SDB_CURRENT,
						  UINT16 eOpt = SDB_CURRENT_OPT)	 = 0;
	virtual BOOL CloseImpl()	 = 0;
	virtual BOOL InsertImpl()	 = 0;
	virtual BOOL UpdateImpl()  = 0;
	virtual BOOL DeleteImpl()  = 0;
	virtual BOOL MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,
						  SdbCursorMode& rMode) = 0;
	virtual BOOL MoveImpl(const ::com::sun::star::uno::Any& rBookmark, SdbCursorMode& rMode, BOOL bRetrieveData)
		throw( ::com::sun::star::lang::IllegalArgumentException )
		{return FALSE;}
	virtual BOOL DetermineRowCountImpl(INT32& nNewCount);
	virtual BOOL SetUpdatableImpl(BOOL bUpdate);
	virtual BOOL OnOpen();					// Initialisierung nach Oeffnen
											// Setzt den Zustand und positioniert auf ersten Satz

	// a) Vorbereitung und Ausfuehrung des Statements:
	// Uebersetzung des SQL-Statements/Parse Tree (Bildung des Parse
	// Tree und Aufruf von DriverTranslate() fuer treiberspezifische
	// Bearbeitung des Parse Tree).
	// Keine Uebersetzung, wenn "native SQL" angegeben wurde.
	void Translate(BOOL bInnerJoin=TRUE);

	BOOL Prepare();	// nur fuer die Interne Verwendung

	// Uebersetzung eines SQL-Statements vom gemeinsamen Sdb-SQL-Dialekt
	// in den treiberspezifischen Dialekt (soweit im Parse Tree darstellbar).
	// Wenn vom Treiber implementiert, muss die Umwandlung des Parse Tree
	// in aNativeSQLStatement mit implementiert werden!
	// Default-Implementation: keine Aenderungen auf dem Parse Tree und
	// Umwandlung in aNativeSQLStatement.
	virtual void DriverTranslate();

	// Beschreibung der Struktur der Parameter
	virtual void DescribeParameter();

	// Beschreibung der Struktur der Ergebnismenge
	virtual void DescribeResult() = 0;

	// Aktuelle bearbeitung abbrechen
	// Dieses sollte von den Treibern implementiert werden, die ein
	// Oeffnen abbrechen koennen, wird ueblicherweise aus einem 2. thread gecalled.
	virtual void CancelExecution();

	// Cursor auf einen Datensatz der Ergebnismenge
	// positionieren: auf den naechsten oder vorherigen
	// den ersten oder letzten, relativ oder absolut
	// (relativ und absolut mit Angabe des Offset).
	virtual BOOL Move(SdbCursorPosition eCursorPosition, INT32 nOffset);

	// Positionieren auf einen Satz
	BOOL DoMove(SdbCursorPosition eCursorPosition, INT32 nOffset,
				BOOL bSkipDeleted,
				BOOL bSetCurrent,
				BOOL bRetrieveData);

	BOOL DoMove(const ::com::sun::star::uno::Any& rBookmark, BOOL bSetCurrent, BOOL bRetrieveData)
		throw( ::com::sun::star::lang::IllegalArgumentException );

	// Parse Tree auf Parameter-Namen untersuchen, Namen im Parse
	// Tree durch '?' ersetzen und Liste der Parameternamen aufbauen
	// (Namen werden an den angegebenen String mit ';' getrennt angehaengt).
	// [Parameter-Namen sind Bezeichner mit ':' vorweg. '?' wird ebenfalls
	// als Parameter-Name erkannt, der zugehoerige Name im String ist dann '?'.]
	virtual void PrepareStatement(SdbConnection* pConnection, SdbSqlParseNode* pParseNode, String& rString);

	virtual BOOL IsCapable(SdbCursorCapability eCapability) const = 0;
protected:

	/**	Gegenstueck zu BeginCancelable, Rueckgabe TRUE, wenn seitem Cancel aufgerufen wurde (aStatus
		wird dann - und nur dann - gleich mit gesetzt)
	*/
	BOOL EndCancelable();
	/**	wurde sei dem letzten BeginCancelable abgebrochen (d.h. Cancel gerufen) ?
		bOnlyIfExecuting==TRUE erzwingt, dass nur dann TRUE geliefert werden kann, wenn man sich gerade
		in einem cancelable Block befindet (also BeginCancelable aufgerufen wurde, aber noch nicht
		EndCancelable)
	*/
	BOOL WasCanceled(BOOL bOnlyIfExecuting = FALSE);

#if __PRIVATE

protected:
	BOOL IsInsertOnly_OldAPI() const {return (nOptions & SDB_APPENDONLY) && !(nOptions & SDB_API_COMPATIBLE);}

	// spezielles Oeffnen um SystemInfo auszulesen
	BOOL Open4Info();

	// methode die waehrend des asynchronen Zugriffs auf aufgerufen werden sollte
	void Idle();

	void InitResultRow();

	// Explizites setzen des RowCounts, geht nur wenn !IsOpen gilt
	// etwa um die anzahl modifizierter Datensaetze nach einem Update anzugeben
	void SetRowCount(UINT32 nCount);

	// flag fuers bysyhandling
	BOOL Open(BOOL bBusyHandling,
			  SdbCursorType eType = SDB_CURRENT,
			  UINT16 eOpt = SDB_CURRENT_OPT);

	// if the cursor doesn't know outer joins then the inner join will be solved as a where condition
	String ResolveInnerJoin(BOOL bCanInnerJoin);

private:
	void SaveStatus();
	void RestoreStatus();
	void SetPositionFlags(BOOL bInRange, SdbCursorPosition ePosition, INT32 nOffset);
	void SetRowCount(BOOL bDataFound, INT32 nOffset);
	BOOL Seek(SdbCursorPosition eCursorPosition, INT32 nOffset, BOOL bRetrieveData);
	BOOL SkipDeleted(SdbCursorPosition eCursorPosition, INT32 nOffset, BOOL bRetrieveData);
	BOOL SeeDeleted() const {return (nOptions & SDB_SEEDELETED) == SDB_SEEDELETED;}
	String BuildBulkSQL(const String& rTableName) const;
	BOOL BulkCopy();
	void GetTableNamesOfSQL(SdbSqlParseNode *pParseNode, String &rTabNames); // rekursiv

	// makes one condition of all inner joins in the from clause
	void ResolveInnerJoin_Recursiv(SdbSqlParseNode *pTableRef,String &rCondition);
public:
	String getParameterNames() const { return aParameterNames; }
#endif
protected:
	// NULL, solange noch keine Strukturbeschreibung angelegt wurde.
	SdbColumnsRef aParameterColumns;

	// Jedesmal bei der Erzeugung eines Cursors (verbunden
	// mit der Ausfuehrung einer Abfrage, Tabellenzugriff o.
	// ae.) wird die Struktur der Ergebnismenge (beschrieben
	// durch ein SdbColumns-Objekt und die zugehoerigen
	// SdbColumn-Objekte) ermittelt. Anschliessend aendert sich
	// diese Struktur nicht mehr.

	// SdbCursor registriert sich bei der "ResultColumns"-
	// Strukturbeschreibung, damit diese erst geloescht wird,
	// wenn keine Referenzen (von geoeffneten Cursor oder von
	// irgendwelchen Rows) mehr bestehen.

	// NULL, solange noch keine Strukturbeschreibung angelegt wurde.
	SdbColumnsRef aResultColumns;

	// Die aktuelle Row mit den Parametern.
	// NULL, nachdem ReleaseParameterRow aufgerufen wurde oder wenn
	// noch keine Row gebunden ist.

	// SdbCursor registriert sich bei der "ParameterRow", damit diese
	// erst geloescht wird, wenn keine Referenzen mehr bestehen.
	ODbRowRef aParameterRow;

	// Die aktuelle Row der Ergebnismenge.
	// NULL, nachdem ReleaseResultRow aufgerufen wurde oder wenn
	// noch keine Row gebunden ist.

	// SdbCursor registriert sich bei der "ResultRow", damit diese
	// erst geloescht wird, wenn keine Referenzen mehr bestehen.
	ODbRowRef aCurrentRow, // Zeile auf die von aussen zugegriffen wird
			  aResultRow,  // Zeile die intern gefuellt wird
			  aOriginalRow;

	// ParseTree, wird erzeugt wenn nicht native SQL verwendet wird
	SdbSqlParseNode*			pSQLParseNode;
	SdbSqlParseTreeIterator*	pParseIterator;
	SdbSqlParseNodeList*		pParseNodeList;
	SdbCursorLock*				pLock;		// Mutex zum Locken des Cursors

private:
	ORowCache*					pCache;

protected:
	// KeySet
	OKeySet*					pKeySet;

	// Der Name des Cursors:
	String aName;

	// Das zugrundeliegende SQL-Statement:
	String aSQLStatement;

	// Jedesmal bei der Erzeugung eines Cursors (verbunden
	// mit der Ausfuehrung einer Abfrage, Tabellenzugriff o.
	// ae.) wird die Struktur der benoetigten Parameter
	// (beschrieben durch ein SdbColumns-Objekt und die zuge-
	// hoerigen SdbColumn-Objekte) ermittelt.

	// SdbCursor registriert sich bei der "ParameterColumns"-
	// Strukturbeschreibung, damit diese erst geloescht wird,
	// wenn keine Referenzen (von geoeffneten Cursor oder von
	// irgendwelchen Rows) mehr bestehen.

	// Liste der Parameter-Namen (mit ';' getrennt, '?' bei
	// Parametern ohne Name)
	// [wird nur intern benoetigt, bis pParameterColumns alloziert
	// worden ist, anschliessend sind die Parameter-Namen auch in
	// pParameterColumns enthalten.]
	String aParameterNames;

	Link	aRowCountHdl;		// Wird im DetermineRowCount gerufen

	::com::sun::star::uno::Any	aLastModified;		// letzte mod. Spalte
	UINT32	nMaxFieldLen;
	UINT32	nMaxRows;

private:
	// Positionierungsattribute, duerfen nur vom Cursor und Cache gesetzt werden
	INT32			nRowCount;
	INT32			nCurrentPos;

protected:
	UINT32	nFree;   			 // free for later use
	void*   pVOID;

private:
	// Satzstatus
	SdbCursorMode	eMode,
					eOriginalMode;

	UINT16			nCacheSize;

protected:
	SdbCursorType eCursorType;  // Type des Cursors

	UINT16	nOptions;			// CursorOptionen
	UINT8	nPrivileges;		// Ist der Cursor schreibbar

	BOOL	bUseCursorSet	: 1;  // wird ein CursorSet verwendet
	BOOL	bPrepared		: 1;  // Ist das Statement "prepared"?
	BOOL	bBulkOptimize	: 1;
	BOOL	bBookmarkable   : 1;
	BOOL	bThreadSafe		: 1;
	BOOL	bIsCancelable	: 1;

private:
	// Befindet sich der Cursor nicht auf einem gueltigen Datensatz
	// (vor dem ersten oder hinter dem letzten)?
	BOOL			bBOF : 1,
					bEOF : 1,
	// Ist das Setende schon einmal erreicht worden, wichtig fuer die Ermittlung des Rowcounts
					bRowCountFinal : 1;
};

SV_DECL_IMPL_REF(SdbCursor);

//==========================================================================
//
#if __PRIVATE
	#define BEGIN_DBACTION()							\
		DBG_ASSERT(pConnection , "kein Vater");			\
		pConnection->BeginHdl().Call(this);
	#define END_DBACTION() 								\
		DBG_ASSERT(pConnection , "kein Vater");			\
		pConnection ->EndHdl().Call(this);
	#define BUSY_DB()									\
		DBG_ASSERT(pConnection, "kein Vater");			\
		pConnection ->BusyHdl().Call(this);
	#define SDB_CHK_CONNECT()							\
		SdbConnection* pConnection = GetConnection();	\
		if (!pConnection)								\
		{												\
			aStatus.SetConnectionLost();				\
			return;										\
		}
	#define SDB_CHK_CONNECT_BOOL()						\
		SdbConnection* pConnection = GetConnection();	\
		if (!pConnection)								\
		{												\
			aStatus.SetConnectionLost();				\
			return FALSE;								\
		}
	#define SDB_CHK_CONNECT_STRING()					\
		SdbConnection* pConnection = GetConnection();	\
		if (!pConnection)								\
		{												\
			aStatus.SetConnectionLost();				\
			return String();								\
		}
	#define SDB_CHK_CONNECT_NULL()						\
		SdbConnection* pConnection = GetConnection();	\
		if (!pConnection)								\
		{												\
			aStatus.SetConnectionLost();				\
			return FALSE;								\
		}

	#define BEGIN_CANCELABLE()		BeginCancelable()
	#define END_CANCELABLE()		if (EndCancelable()) return FALSE

#endif

inline UINT16 SdbCursor::GetCacheSize() const
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	return nCacheSize;
}

inline BOOL SdbCursor::IsOffRange() const
{
	return IsBOF() || IsEOF();
}

inline BOOL SdbCursor::IsInRange() const
{
	return !IsOffRange();
}

inline BOOL SdbCursor::IsCancelable()
{
	return bIsCancelable;
}

inline const ::com::sun::star::uno::Any& SdbCursor::GetLastModified() const
{
	return aLastModified;
}

inline SdbCursor::SdbCursorGuard::SdbCursorGuard(SdbCursor* pCurs)
				 :vos::OClearableGuard(*pCurs->pLock)
{
}

#endif


