/*************************************************************************
 *
 *  $RCSfile: sdbdb.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBDB_HXX
#define _SDB_SDBDB_HXX 1

#define FILELOCATION	 "URL"
#define SDB_DATABASE_EXT "sdb"

#ifndef _SVSTOR_HXX //autogen
#include <so3/svstor.hxx>
#endif

#ifndef _REF_HXX //autogen
#include <tools/ref.hxx>
#endif

#ifndef _RTTI_HXX //autogen
#include <tools/rtti.hxx>
#endif

#ifndef _LINK_HXX //autogen
#include <tools/link.hxx>
#endif

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

class SdbQuery;
class SdbTable;
class SdbDocument;
class SdbDatabaseImpl;
class SdbStorage;
class SvNumberFormatter;
class SvStorageStream;

DECLARE_SDBKEYCONT(SdbQueryList, SdbQuery*);
DECLARE_SDBKEYCONT(SdbDocumentList, SdbDocument*);

// thread safety of a database
// ==================================================================
class SdbDatabase;
class SdbDatabaseGuard
{
	SdbDatabaseImpl*	m_pDBImpl;
		// we need to ref-count the impl as it holds the mutex we're guarding, so
		// we need the impl and not just the mutex
public:
	SdbDatabaseGuard(const SdbDatabase* _pDB) : m_pDBImpl(NULL) { attach(_pDB); }

	// attach/detach
	void attach(const SdbDatabase* _pDB);
	void clear();	// call it "clear" for cmpatibility with the NAMESPACE_VOS(OClearableGuard) ...

	virtual ~SdbDatabaseGuard() { clear(); }

#if __PRIVATE
	// construction from a impl class
	SdbDatabaseGuard(const SdbDatabaseImpl* _pDBImpl) : m_pDBImpl(NULL) { attach(_pDBImpl); }
	void attach(const SdbDatabaseImpl* _pDBImpl);
#endif
};

// ==================================================================
class SdbDatabase : public SdbObj
{
	friend class SdbWorkspace;
	friend class SdbQuery;
	friend class SdbTable;
	friend class SdbView;
	friend class SdbDocument;
	friend class SdbSqlParseNode;
	friend class SdbSqlParseTreeIterator;
	friend class SdbDatabaseGuard;

	SdbDatabaseImpl* pImpl;
	SdbQueryList	 aQueryList;	// Liste zum Cachen von Abfragen
	SdbDocumentList  aFormList,		// Liste zum Cachen von Formularen
					 aReportList;	// Liste zum Cachen von Berichten
	void*  pVOID,			// free for later use
		   *pVOID2;
	UINT32 nFree,
		   nFree2;


protected:
	SdbDatabase(SdbWorkspace* pWorkspace);
	virtual ~SdbDatabase();

public:
	TYPEINFO();

	// Zugriff auf Vater
	virtual SdbWorkspace* GetWorkspace() const;

	// Methoden zur Manipulation der Datenbank (sdb)
	virtual BOOL Drop();
	virtual BOOL Rename(const String& rName);
	virtual BOOL AlterTitle(const String& rTitle);

	// Erzeugen und Zugriff auf Datenbankobjekte

	// util um neue Namen zu finden, bzw zu erzeugen
	virtual String GetUniqueName(DBObject eObj, const String& rName);
	virtual BOOL IsUniqueName(DBObject eObj, const String& rName);

	// Ist das Object bereits in der Datenbank (gilt fuer Dokumente, Abfragen, Tabellen, Views)
	virtual BOOL	HasObject(DBObject _eObj, const String& _rName);
	virtual SdbObj*	CreateObject(DBObject _eObj, const String& _rName);
	virtual SdbObj*	OpenObject(DBObject _eObj, const String& _rName);
	inline  SdbObj* OpenObject(DBObject _eObj, USHORT _nPos);
	virtual String	GetObjectName(DBObject _eObj, USHORT _nPos) const;
	virtual USHORT	GetObjectCount(DBObject _eObj) const;

#if (SUPD>512) || PRIV_DEBUG
	// Verwaltung geloeschter Items
	virtual BOOL	HasObjectDeleted(DBObject _eObj, const String& _rName);
	virtual BOOL	DeleteObject(DBObject _eObj, const String& _rName);
	virtual BOOL	UndeleteObject(DBObject _eObj, String& _rName);
		// wenn erfolgreich, enthaelt _rName den neuen Namen, falls das Objekt umbenannt werden musste
	virtual BOOL	DestroyDeletedObject(DBObject _eObj, const String& _rName);
#endif // (SUPD>512) || PRIV_DEBUG

	// Aktualisieren eines einzelnen Verzeichnisses
	virtual BOOL	Refresh(DBObject _eObj);

	// Gibt einen voll qualifizierten Tabellennamen zurueck
	virtual String GetFullTableName(const String& rName);

	// Erzeugen eines Cursors ueber die Connection
	virtual SdbCursor* CreateCursor(SdbCursorType eType  = SDB_KEYSET,
									UINT16 nOptions  = SDB_READONLY);

	// Oeffnet einen Cursor auf einem bestehendem Object
	virtual SdbCursor* OpenCursor(const String& aObjName,
								  SdbCursorType eType = SDB_KEYSET,
								  UINT16 nOptions = SDB_READONLY);

	virtual String Name() const;
	virtual String ConnectStr() const;
	virtual String Title() const;
	virtual BOOL IsReadOnly() const;

	inline BOOL QualifiedNameComponents(const String& rQualifiedName, String& rCatalog, String& rSchema, String& rName) const;

	// Methoden, die Information ueber die Datenbankverbindung liefern
	// siehe Sdbconnection
	inline BOOL IsCapable(SdbCapability eCapability) const;
	inline ::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > GetMetaData() const;
	inline BOOL IsTypeAvailable(SdbDatabaseType eType) const;
	inline String GetUser() const;
	inline String GetQualifier() const;
	inline const SdbTypeInfoList* GetTypeInfo() const;
	inline SdbSqlParser* GetSqlParser() const;
	inline SdbIdentifierCase GetIdentifierCase() const;
	inline char GetQualifierSeparator() const;
	inline char GetIdentifierQuote() const;
	inline const CommandParser* GetOptionsParser() const;
	inline BOOL StructureIsReadOnly() const;
	inline virtual BOOL IsView(const String &rTableName) const;
	// Konvertierung von Datum/Zeitfeldern in die treiberspezifische Form
	virtual BOOL ExecuteSQL(const String& aStatement);
	virtual SdbColumns*	DescribeResult(const String& aStatement,
									   BOOL bTable = FALSE);

	virtual SdbColumns* CreateColumns();
	virtual SdbColumns* CreateColumns(const SdbColumns& _rColumns);
	virtual SdbColumn*  CreateColumn(const String& _rName,
									 SdbDatabaseType _eType,
									 UINT32 _nLength = 0, UINT16 _nScale = 0,
									 UINT16 _nFlags = 0);
	virtual SdbColumn*  CreateColumn(const SdbColumn& _rColumn);

	virtual String IdentifierQuotation(const String& rString);
	virtual BOOL   IsSQLNameValid(const String& rString);
	virtual String QuoteTableName(const String& rName);
	virtual String CompleteTableName(const String& rName);
	// Erzeugt einen neuen Namen falls noetig
	virtual String ConvertName(const String& rName) const;
	virtual BOOL IsCharOk(char c) const;
	virtual BOOL DoCmd(UINT16 nCmd,const String &rArgs);

	// Zugriff auf den Numberformatter
	SvNumberFormatter* GetFormatter() const;
	BOOL AlterFormatter();	// speichern des Formatters

	// Transaktionen
	BOOL BeginTrans();
	BOOL CommitTrans();
	BOOL RollbackTrans();

protected:
	// Methoden die nur von der Environment Verwendung finden
	virtual BOOL Load(const String& rName);
	virtual BOOL Create(const String& rName, const String& rTitle, const String& rConnectStr);
	// Aendern der Verbindungsoptionen, Datenbank darf nicht geoeffnet sein !!!
	// da ansonsten keine neue Verbindung aufgebaut werden kann.
	virtual BOOL AlterConnection(const String& rConnect);

	// Datenbanken gelten als gleich, wenn die Pfade uebereinstimmen
	// gleich sind.
	virtual BOOL Compare(const SdbObj& _rObj) const;

	// Schnittstelle zu Sba, um UI Informationen in der Datenbank abzulegen
protected:
	SdbStorage* OpenStorage(SdbStorage& rMain, const String& rName, StreamMode = STREAM_STD_READWRITE,
							 StorageMode = 0);
	SdbStorage* OpenStorage(DBObject eObj, BOOL bReadOnly = FALSE);// oeffnen der speziellen Storages Tabellen /Abfragen
	BOOL ReleaseStorage(DBObject eObj, BOOL bCommit = TRUE);

	SvStorageStream* OpenStream(SdbStorage& rMain,						// uebernimmt Fehlerbehandlung
								  const String& rEleName,
								  StreamMode = STREAM_STD_READWRITE);

	SvStorageStream* OpenContent(SdbStorage& rMain,
								   StreamMode = STREAM_STD_READWRITE);

	BOOL RenameStorage(const String& aOldName, const String& aNewName, DBObject eObj);
	BOOL RemoveStorage(const String& rName, DBObject eObj);

	// Relationen auslesen
	BOOL GetRelationLayout(SvStream& rStream);
	BOOL AlterRelationLayout(SvStream& rStream);

protected:
	// Implementierung von Methoden die vom SdbObj erwartet werden
	virtual BOOL Open();
	virtual void Close();

protected:
	virtual void RemoveChildren();
	virtual void RemoveChild(SdbObj* pObj);
	virtual BOOL DoRefresh();
	virtual SdbConnection* GetConnection() const;

private:
	// Statusaenderung der Implementierung
	DECL_LINK(OnStatusChange,SdbStatus*);
#if __PRIVATE
	SdbDatabaseImpl* Impl() const {return pImpl;}
#endif
};

//------------------------------------------------------------------
inline BOOL SdbDatabase::IsCapable(SdbCapability eCapability) const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->IsCapable(eCapability) : FALSE;
}
//------------------------------------------------------------------
inline ::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > SdbDatabase::GetMetaData() const{
	SdbConnection* pCon = GetConnection();
	DBG_ASSERT(pCon," Connection darf an dieser Stelle nicht NULL sein");
	return pCon->GetMetaData();
}

//------------------------------------------------------------------
inline BOOL SdbDatabase::IsTypeAvailable(SdbDatabaseType eType) const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->IsTypeAvailable(eType) : FALSE;
}
//------------------------------------------------------------------
inline String SdbDatabase::GetUser() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->GetUser() : String();
}
//------------------------------------------------------------------
inline String SdbDatabase::GetQualifier() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->GetQualifier() : String();
}
//------------------------------------------------------------------
inline const SdbTypeInfoList* SdbDatabase::GetTypeInfo() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? &pConn->GetTypeInfo() : NULL;
}
//------------------------------------------------------------------
inline SdbSqlParser* SdbDatabase::GetSqlParser() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->GetSqlParser() : NULL;
}
//------------------------------------------------------------------
inline SdbIdentifierCase SdbDatabase::GetIdentifierCase() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->GetIdentifierCase() : SDB_IC_MIXED;
}
//------------------------------------------------------------------
inline char SdbDatabase::GetQualifierSeparator() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->GetQualifierSeparator() : '.';
}
//------------------------------------------------------------------
inline char SdbDatabase::GetIdentifierQuote() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->GetIdentifierQuote() : '\'';
}
//------------------------------------------------------------------
inline const CommandParser* SdbDatabase::GetOptionsParser() const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? &pConn->GetOptionsParser() : NULL;
}

//------------------------------------------------------------------
BOOL SdbDatabase::StructureIsReadOnly() const
{
	return GetConnection()->StructureIsReadOnly();
}

//------------------------------------------------------------------
BOOL SdbDatabase::QualifiedNameComponents(const String& rQualifiedName, String& rCatalog, String& rSchema, String& rName) const
{
	SdbConnection* pConn = GetConnection();
	return pConn ? pConn->QualifiedNameComponents(rQualifiedName, rCatalog, rSchema, rName) : FALSE;
}

//------------------------------------------------------------------
inline SdbObj* SdbDatabase::OpenObject(DBObject _eObj, USHORT _nPos)
{
	return OpenObject(_eObj, GetObjectName(_eObj, _nPos));
}

//------------------------------------------------------------------
inline BOOL SdbDatabase::IsView(const String &rTableName) const
{
	SdbConnection* pConn = GetConnection();
	return (pConn) ? pConn->IsView(rTableName) : FALSE;
}
//------------------------------------------------------------------
SV_DECL_IMPL_REF(SdbDatabase);

#endif

