/*************************************************************************
 *
 *  $RCSfile: sdbdbimp.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBDBIMP_HXX
#define _SDB_SDBDBIMP_HXX 1

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include "sdbtools.hxx"
#endif

#ifndef _SDB_SDBSTOR_HXX
#include "sdbstor.hxx"
#endif

#ifndef _SDB_SDBENV_HXX
#include "sdbenv.hxx"
#endif

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

#ifndef _SDB_SDBHASH_HXX
#include "sdbhash.hxx"
#endif

#ifndef _SDB_SDBCONT_HXX
#include "sdbcont.hxx"
#endif

#ifndef _ZFORLIST_HXX //autogen
#include <svtools/zforlist.hxx>
#endif

class SdbQuery;
class SdbTable;

class SdbDatabaseImpl : public SdbObj
{
	friend class SdbDocument;
	friend class SdbQuery;
	friend class SdbTable;
	friend class SdbView;
	friend class SdbDatabase;
	friend class SdbStorage;
	friend class SdbDatabaseGuard;

	SdbStorageRef		aStorage;
	SdbStorageRef		aTableStor, aQueryStor, aFormStor, aReportStor;
	SdbConnection*		pConnection;
	SvNumberFormatter	aFormatter;

	SdbStorageRef		m_aWasteBasket;

	SdbHashStringList aTableNameList,
					  aQueryNameList,
					  aFormNameList,
					  aReportNameList;
	DirEntry		  aTempFile;


	SvStringsDtor		aMaskedTables;	// die Namen der ausmaskierten Tabellen


	String aName;						// Filename
	String aConnectStr;					// String zum Verbindungsaufbau, mit DirtyFlag
	String aTitle;						// Titel der Datenbank
	UINT32 nTransCount;
	BOOL   bLoaded : 1,					// Datenbank ist Loaded, wenn Titel und ConnectStr eingelesen wurden,
										// und der Filename bekannt ist !!!
										// Dieses Flag ist von der Open/Close Mimik nicht betroffen
										// d.h. wird eine Datenbank geschlossen,
										// dann stehen trotzdem noch Title, Connection zur Verfuegung
										// Eine Verbindung ist dann noch nicht hergestellt
		   bReadOnly : 1,				// Internes Flag
		   bTableFilter : 1,			// werden Tabellen gefiltert, sprich sollen nicht alle
										// Tabellen einer Datenbank angezeigt werden
		   bDeleteOnClose : 1;			// Lschen des Files beim Schliessen

	NAMESPACE_VOS(OMutex)		m_aThreadSafety;

protected:
	SdbDatabaseImpl(SdbWorkspace*);	// wird von der Environment erzeugt
	virtual ~SdbDatabaseImpl();

	SdbEnvironment* GetEnvironment() const;
	SdbWorkspace* GetWorkspace() const;

public:
	// Laedt die Datenbankinformation (Titel, Connection), stellt jedoch keine Datenbankverbindung her
	// Diese Information sind unabhaengig von der Open/Close Mimik
	// Nur ein erfolgreiches Drop auf eine Datenbank setzt den Zustand Loaded wieder zurueck

	BOOL Load(const String& rName);
	BOOL IsLoaded() const {return bLoaded;}

	// Oeffnet die Datenbank und die zugehoerige Verbindung
	virtual BOOL Open();
	virtual void Close();

	// Erzeugt eine Datenbank, oeffnet sie aber nicht!!! Name und ConnectStr werden gesetzt
	BOOL Create(const String& rName, const String& rTitle, const String& rConnectStr);
	BOOL Rename(const String& rName);

	BOOL Drop();

	// Setzt einen neuen String fr den Verbindungsaufbau, fuehrt dazu, da die Verbindung zunaechst
	// geschlossen wird und der neue Verbindungsstring gesetzt wird.
	// Ist die Verbindung vorher geoeffnet, wird sie danach auch wieder geoeffnet
	BOOL AlterConnection(const String& rConnectStr, BOOL bDirty = TRUE);
	BOOL AlterTitle(const String& rTitle);

	void Refresh(DBObject eObj);

	// Is das Object bereits in der Datenbank
	BOOL HasObject(DBObject eObj, const String& rName) const;

	// Verwaltung geloeschter Items
	virtual BOOL	HasObjectDeleted(DBObject _eObj, const String& _rName);
	virtual BOOL	DeleteObject(DBObject _eObj, const String& _rName);
	virtual BOOL	UndeleteObject(DBObject _eObj, String& _rName);
		// wenn erfolgreich, enthaelt _rName den neuen Namen, falls das Objekt umbenannt werden musste
	virtual BOOL	DestroyDeletedObject(DBObject _eObj, const String& _rName);

	// util um neue Namen zu finden, bzw zu erzeugen
	String GetUniqueName(DBObject eObj, const String& rName);
	BOOL IsUniqueName(DBObject eObj, const String& rName);

	BOOL UpdateTables(const String* pTableList = NULL,BOOL bReadOnly=FALSE);							// aktualisiert die Liste der Tabellen
	String GetFullTableName(const String& rName) const;

	const String& Name() const {return aName;}
	const String& ConnectStr() const {return aConnectStr;}
	const String& Title() const {return aTitle;}

	// nur verfuegbar, wenn Datenbank geoeffnet
	SdbConnection* GetConnection() const {return pConnection;}


	// Schnittstelle zu Sba
	SdbStorage* OpenStorage(SdbStorage& rMain, const String& rName, StreamMode = STREAM_STD_READWRITE,
							StorageMode = 0);
	SdbStorage* OpenStorage(DBObject eObj, BOOL bReadOnly);// oeffnen der speziellen Storages Tabellen /Abfragen
	BOOL ReleaseStorage(DBObject eObj, BOOL bCommit = TRUE);

	SvStorageStream* OpenStream(SdbStorage& rMain,						// uebernimmt Fehlerbehandlung
								const String& rEleName,
								StreamMode = STREAM_STD_READWRITE);

	SvStorageStream* OpenContent(SdbStorage& rMain,						// uebernimmt Fehlerbehandlung
								 StreamMode = STREAM_STD_READWRITE);

	SvNumberFormatter&	GetFormatter() {return aFormatter;}
	BOOL				AlterFormatter();	// speichern des Formatters

	// Relationen auslesen
	BOOL GetRelationLayout(SvStream& rStream);
	BOOL AlterRelationLayout(SvStream& rStream);

private:
#if __PRIVATE
	BOOL LoadImpl();
	void AdjustConnectStr(BOOL bInit);
	void AdjustTableList();

protected:
	static ULONG CopyTo(SvStream& rTarget, SvStream& rDest);

	BOOL BeginTrans();
	BOOL EndTrans(BOOL bCommit);

	BOOL RenameStorage(const String& aOldName, const String& aNewName, DBObject eObj);
	BOOL RemoveStorage(const String& rName, DBObject eObj);

	String CategoryFolderName(DBObject eObj) const;

	void			OpenWasteStorage();
	SdbStorageRef	OpenWasteObjectStorage(DBObject eObj, const String& rName);
		// (fuer Bemerkungen zur Struktur des Papierkorbs siehe Implementation von DeleteObject)

	BOOL Read(SvStorageStream&, String&, BOOL bDecrypt = FALSE);
	BOOL Write(SvStorageStream&, const String&, BOOL bEncrypt = FALSE);

	SdbHashString* Find(const ByteString& rName, const SdbHashStringList& rList) const;
	SdbHashString* SdbDatabaseImpl::Find(const String& rName, const SdbHashStringList& rList) const
	{
		return Find(ByteString(rName,gsl_getSystemTextEncoding()),rList);
	}
	BOOL ReadList(DBObject eObj);
	void ReadList(SdbStorage& rStor, SdbHashStringList& rList, BOOL bStor = TRUE,BOOL _bCaseSensitiv=TRUE);

	BOOL WriteDocInfo(SvStorageStream&);
	BOOL ReadDocInfo(SvStorageStream*);

	void Add(const String& rName, DBObject eObj);
	void Drop(const String& rName, DBObject eObj);
#endif
};


#endif


