/*************************************************************************
 *
 *  $RCSfile: sdbenv.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBENV_HXX
#define _SDB_SDBENV_HXX

#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif
#ifndef _SDB_TOOLS_HXX
#include <tools.hxx>
#endif
#ifndef _SDB_SDBOBJ_HXX
#include <sdbobj.hxx>
#endif
#ifndef _SDB_SDBWORK_HXX
#include <sdbwork.hxx>
#endif
#include <vector>
#include <map>

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _UTL_STLTYPES_HXX_
#include <unotools/stl_types.hxx>
#endif
#ifndef _OSL_MODULE_H_
#include <osl/module.h>
#endif

class SdbConnection;
class SdbDatabase;
class SdbSqlParser;
class SvNumberFormatter;
class SimpleResMgr;

#if defined(WNT)
class SdbOTERHandleRef;
#endif

// wird verwendet von Servernames
#define SDB_SEPARATOR	char(11)

namespace com { namespace sun { namespace star { namespace ucb {
	class XProgressHandler;
}}}}

struct SdbLoginInformation
{	// haetten wir gern zu einer lokalen Struktur in SdbEnvironment gemacht, aber dank dem Solaris-Compiler
	// steht das eben hier .... ;(
	USHORT	nVersion;
		// bisher ist fuer nVersion nur 1 zulaessig, dann sehen die Daten so aus wie hier :
		// DB-Name, UserName, Pwd. Wenn irgendwann mal mehr Informationen uebergeben werdem sollen,
		// muss entsprechend eine neue Version eingefuehrt werden
	String	sDbName;	// [in]
	String	sUserName;	// [in]/[out]
	String	sPassword;	// [out]
		// in und out bezieht sich auf den (extended) PasswortHandler, der eine solche Struktur
		// uebergeben bekommt
};

enum KernelState { DEAD,COLD,WARM };

struct AdabasKernelState
{
	String	m_DBName;  // Adabas database name
	String	m_CtrlUid; // Control User
	String	m_CtrlPwd; // Control Password
	KernelState m_KernelState; // state of the kernel reg/yellow/green
};

class SdbEnvironment : public SdbObj
{
	friend class SbaObject;
	friend class SdbWorkspace;
	friend class SdbConnection;
	friend class SdbEnvironmentRef;

	::std::map< ::rtl::OUString, SdbWorkspace*, ::utl::UStringLess >  m_aWspMap;
	::std::vector<SdbWorkspace*>					m_aWspList, m_aTempWspList;
	::std::vector<SdbConnection*>					m_aConnectionList;

	::vos::OMutex									m_aMutex;
	SdbSqlParser*									m_pSqlParser;
	::com::sun::star::uno::Reference< ::com::sun::star::ucb::XProgressHandler >	m_xProgressHandler;
	// Link, der waehrend der laenger andauernden Ausfuehrung
	// von Datenbankoperationen regelmaessig gerufen wird:
	Link m_aBusyHdl;

	// Links, die zu Beginn bzw. am Ende einer laenger andauernden Ausfuehrung
	// von Datenbankoperationen gerufen werden:
	Link m_aBeginHdl;
	Link m_aEndHdl;

	Link m_aPasswordLink;				// Wird fuer eine Verbindung ein Password benoetigt
									// PWD=* im ConnectStr, dann wird dieser Link aufgerufen
									// um das Password zu setzen
	Link m_aDBCreateLink;				// erzeugen von Datenbanken

	Link m_aExtendedPwdHandler;


private:
	SdbEnvironment();

protected:
	virtual ~SdbEnvironment();

public:
	TYPEINFO();

	// Zugriff auf einen SQL-Parser:
	SdbSqlParser* GetSqlParser() const {return m_pSqlParser;}

	// IsOpen wird ueberladen da Parent immer NULL, Ausnahme
	BOOL IsOpen() const {return bOpen;}

	// Link, der *waehrend* einer laenger andauernden Ausfuehrung
	// von Datenbankaktionen in regelmaessigen Abstaenden
	// aufgerufen wird.
	inline void SetBusyHdl(const Link & rNewBusyHdl);
	const Link& BusyHdl() const{return m_aBusyHdl;}

	// Links, die *zu Beginn* bzw. *am Ende* einer laenger andauernden Ausfuehrung
	// von Datenbankoperationen gerufen werden:
	inline void SetBeginHdl(const Link & rNewBeginHdl);
	const Link& BeginHdl() const{return m_aBeginHdl;}
	inline void SetEndHdl(const Link & rNewEndHdl);
	const Link& EndHdl() const {return m_aEndHdl;}

	// Link zum setzen des Passwords beim Verbindungsaufbau
	inline void SetPasswordHdl(const Link& rLink) {m_aPasswordLink = rLink;}
	const Link& GetPasswordHdl() const {return m_aPasswordLink;}

	// Link zum setzen des Passwords beim Verbindungsaufbau, extended version
	inline void SetExtPasswordHdl(const Link& rLink) { m_aExtendedPwdHandler = rLink; }
	const Link& GetExtPasswordHdl() const { return m_aExtendedPwdHandler; }
		// der PasswordHandler bekommt einen Zeiger auf eine SdbLoginInformation-Struktur
	void SetProgressHandler(const ::com::sun::star::uno::Reference< com::sun::star::ucb::XProgressHandler >& _Prog);
	const ::com::sun::star::uno::Reference< com::sun::star::ucb::XProgressHandler >&
		GetProgressHandler() const;

	//
	inline UINT32 GetLoginTimeOut() const;
	inline void SetLoginTimeOut(UINT32 _nTimeOut);

	inline SvNumberFormatter* GetStandardFormatter() const;

	// Erzeugung von SdbConnection [neuerdings hier, nicht mehr
	// in SdbConnection!]:
	// Je nachdem, welcher Server angesprochen werden soll, wird
	// eine andere Ableitung von SdbDatabase erzeugt
	// (Interface/Implementation-Konzept).

	// SdbConnection hat unterschiedliche Implementationen.
	// Welche Implementation fuer eine zu erzeugende Verbindung
	// benutzt werden soll, entscheidet der Typ-Parameter bei
	// Create bzw. die Option "TYP=..." im Optionsstring (zulaessige
	// Typen im Optionsstring sind jeweils der letzte Teil des Enum-
	// Symbols, also z. B. "TYP=ODBC", "TYP=WSQL", ...).

	// Fuer den Benutzer ist die konkrete Ableitung (die
	// Implemetation) nicht relevant, daher wird das Objekt
	// einfach als SdbConnection zurueckgeliefert.

	// Alle erforderlichen Angaben werden jetzt nur noch in EINEM Options-String
	// angegeben.
	// Die Parameter der CreateConnection-Routine werden dann als Optionen
	// (in der Form "TYP=ODBC;DSN=sample;UID=admin;PWD=school") mit in den
	// Optionsstring gepackt. Darueber hinaus kann der Optionsstring
	// treiberspezifische Optionen (im gleichen Format "option=wert;...") enthalten.
	SdbConnection* OpenConnection(const String & rNewOptions);

	// erledigt dieselbe Aufgabe wie OpenConnection, ohne aber die Connection
	// zu oeffnen.
	SdbConnection* CreateConnection(const String& rNewOptions);

	// Eine Kopie einer Connection ist eine neue Verbindung
	// mit den gleichen Verbindungsparametern (Options).
	// Etwaige dort geoeffnete Cursor und sonstige Flags (Isolation
	// Level, ...) werden nicht mit kopiert.
	SdbConnection* OpenConnection(const SdbConnection & rSdbConnection);

	/* workspaces */

	// Informationen ueber die Namen der Tabellen und Views
	String GetWorkspaceName(USHORT _nPos);
	USHORT GetWorkspaceCount();

	// Erzeugt einen Workspace
	SdbWorkspace* CreateWorkspace(const String& _rName);

	// Fuegt den Workspace bei der Engine ein
	BOOL AppendWorkspace(SdbWorkspace* pWsp);
	// herausnehmen aus der Liste
	SdbWorkspace* RemoveWorkspace(SdbWorkspace* pWsp);

	SdbWorkspace* OpenWorkspace(const String& _rName);
	SdbWorkspace* OpenWorkspace(USHORT _nPos) {return OpenWorkspace(GetWorkspaceName(_nPos));}

	/* datenbank handling */
	// Oeffnen einer Datenbank (Verbindungsaufbau)
	SdbDatabase* OpenDatabase(const String& rName);

	// liefert eine schon geoeffnete DB zurueck, NULL, wenn sie nicht offen ist
	SdbDatabase* GetDatabaseNoOpen(const String& rName);

	// Erzeugen einer Datenbankdatei
	// liefert das Object nicht zurck, um nichtunnoetig eine Verbindung zu einer Daten-
	// bank aufzubauen
	BOOL CreateDatabase(const String& rName, const String& rTitle, const String& rConnectStr);

	// Loeschen einer Datenbank (der Datenbandatei (sdb))
	BOOL DropDatabase(const String& rName);

	// Auslesen des Verbindungsstrings
	String GetDatabaseConnectStr(const String& rName);

	// Aendern einer Datenbankverbindung
	BOOL AlterDatabase(const String& rName, const String& rConnectStr);
	BOOL AlterDatabaseName(const String& aOldName, const String& aNewName);
	BOOL AlterDatabaseTitle(const String& aNewName, const String& aTitle);

	// Link zum Erzeugen eigener Datenbankobjecte abgeleitete von SdbDatabase
	// im Prinzip nur fuer Sba gedacht
	inline void SetCreateDBHdl(const Link& rLink) {m_aDBCreateLink = rLink;}

	// Informationen ueber bekannte Server (Typen und Namen zu jedem Typ):

	// Liste der verfuegbaren Server-Typen (als String mit Semikolon getrennt,
	// z. B. "ODBC;TXT;DBF") [Parameter "TYP=..." bei CreateConnection].
	String ServerTypes() const;

	// Is this ServerType a Filebased
	static BOOL IsFileServer(const String& rServerType);

	// Liste der verfuegbaren Server-Namen zu einem Server-Typ (als String mit
	// Semikolon getrennt, z. B. "Sample;StarDemo (blabla);Noch eine"
	// Leerstring, wenn sich keine Liste der verfuegbaren Server-Namen ermitteln
	// laesst, "*" wenn der Server-Name ein Pfadname ist.
	// [Der Server-Name wird bei CreateConnection als Parameter "DSN=..." uebergeben.
	// Der Kommentar in runden Klammern gehoert dabei NICHT zum DSN (muss
	// beim CreateConnection weggelassen werden)!]
	String ServerNames(const String & rTyp);

#if __PRIVATE
	SimpleResMgr* GetResMgr() const {return pResMgr;}
	void AppendLibrary(oslModule);
#endif

protected:
	SdbDatabase* LoadDatabase(const String& rName);

	virtual void RemoveChild(SdbObj* pObj);
	virtual void RemoveChildren();
	virtual BOOL Open();
	String GetFileName(const String& rFile) const;
	void WorkspaceRenamed(SdbWorkspace* pWsp, const String& rOldName);

private:
	// Die folgenden Member sind als Ersatz fuer statische Variable
	// fuer einzelne Komponenten innerhalb Sdb vorgesehen.
	friend class SdbODBCConnection;
	friend class SdbODBC2Connection;
	friend class SdbODBC3Connection;
	friend class SdbDB2Connection;
	friend class SdbAdaConnection;
	friend class SdbOTERConnection;

	// Spezielle Typen hier als void *, damit es keine Schwierigkeiten bei der
	// Uebersetzung fuer andere Architekturen, die diesen Typ nicht unterstutzen,
	// gibt (z. B. Typ HENV (ODBC)).

	SimpleResMgr*	pResMgr;
	void* aEnvironmentHandle;		// ODBC Environment Handle (ODBC Connection)

	SdbWorkspace* pDefaultWorkspace;
	UINT32	nTimeOut;

	SvNumberFormatter*	pStandardFormatter;
#if defined ( WNT )
	SdbOTERHandleRef* pOterroHandle;	// OTERRO Enviroment Handle (OTERRO Connection)
#endif
	void*   aADAHandle;	   				// Adabas
	void* 	aODBC3EnvironmentHandle;	// ODBC 3.0 Environment Handle (ODBC Connection)
	void*	pVOID3;

	SdbDatabase* GetDatabase(const String& rName);
	SdbDatabase* CreateDB();
};

//==================================================================
// SdbEnvironmentRef
// spezielle Klasse, die im Ctor bereits defaultmig ein
// Object anlegt, oder wenn in den Applicationsdaten vorhanden
// ein bestehendes verwendet
//==================================================================
#define SDB_SV_DECL_REF_SIGNATURE( ClassName, Ref )						\
    inline               ClassName##Ref( const ClassName##Ref & rObj );	\
    inline               ClassName##Ref( ClassName * pObjP );			\
    inline void          Clear();										\
    inline               ~ClassName##Ref();								\
    inline ClassName##Ref & operator = ( const ClassName##Ref & rObj );	\
    inline ClassName##Ref & operator = ( ClassName * pObj );			\
    inline BOOL            Is() const { return pObj != NULL; }			\
    inline ClassName *     operator &  () const { return pObj; }		\
    inline ClassName *     operator -> () const { return pObj; }		\
    inline ClassName &     operator *  () const { return *pObj; }		\
    inline operator ClassName * () const { return pObj; }



class SdbEnvironmentRef
{
protected:
	SdbEnvironment*	pObj;

public:
	SDB_SV_DECL_REF_SIGNATURE(SdbEnvironment, Ref);
	SdbEnvironmentRef();
};

SV_IMPL_REF(SdbEnvironment);

// include for vector SdbConnection

#ifndef _SDB_SDBCONN_HXX
#include <sdbconn.hxx>
#endif

inline SvNumberFormatter* SdbEnvironment::GetStandardFormatter() const
{
	return pStandardFormatter;
}

inline UINT32 SdbEnvironment::GetLoginTimeOut() const
{
	return nTimeOut;
}

inline void SdbEnvironment::SetLoginTimeOut(UINT32 _nTimeOut)
{
	nTimeOut = _nTimeOut;
}


inline void SdbEnvironment::SetBusyHdl(const Link & rNewBusyHdl)
{
	aStatus.Clear();
	m_aBusyHdl = rNewBusyHdl;
}

inline void SdbEnvironment::SetBeginHdl(const Link & rNewBeginHdl)
{
	aStatus.Clear();
	m_aBeginHdl = rNewBeginHdl;
}


inline void SdbEnvironment::SetEndHdl(const Link & rNewEndHdl)
{
	aStatus.Clear();
	m_aEndHdl = rNewEndHdl;
}

#endif


