/*************************************************************************
 *
 *  $RCSfile: sdbhash.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBHASH_HXX
#define _SDB_SDBHASH_HXX 1

#ifndef HASH_NO_STRING_CLASS //autogen
#include <svtools/hashtab.hxx>
#endif

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif
#if ( defined OS2 || defined UNX || defined MAC )
#include <ctype.h>	// toupper
#else
#include <stdlib.h>	// toupper
#endif

// Klasse zum schnelle Suche von Strings
// case insensitive
class SdbHashString
{
	ByteString aStr;
	ULONG  nHashID;

public:
	inline SdbHashString(const ByteString& rStr);
	inline SdbHashString(const String& rStr);
	inline SdbHashString(const SdbHashString& rHashStr);
	inline SdbHashString& operator= (const SdbHashString& rHashStr);
	inline SdbHashString& operator= (const ByteString& rStr);

	inline static ULONG Hash(const ByteString& rStr);

	const ByteString& GetString() const {return aStr;}
	ULONG HashID() const {return nHashID;}

	BOOL operator ==( const SdbHashString& rStr ) const
	{ return nHashID == rStr.nHashID; }
	BOOL operator !=( const SdbHashString& rStr ) const
	{ return !SdbHashString::operator == (rStr); }
};

DECLARE_LIST(SdbHashStrList, SdbHashString*);

class SdbHashStringList : public SdbHashStrList
{
	BOOL			m_bCaseSensitive : 1;		// werden die Objekte CaseSensitive abgelegt ?
public:
	SdbHashStringList( USHORT _nInitSize = 16, USHORT _nReSize = 16 )
					 :SdbHashStrList( _nInitSize, _nReSize ),m_bCaseSensitive(TRUE) {}
	SdbHashStringList(USHORT _nBlockSize, USHORT _nInitSize, USHORT _nReSize )
					 :SdbHashStrList( _nBlockSize, _nInitSize, _nReSize ),m_bCaseSensitive(TRUE) {}
	SdbHashStringList(const SdbHashStringList& rClassName)
					 :SdbHashStrList( rClassName ),m_bCaseSensitive(TRUE) {}

	SdbHashStringList& operator =( const SdbHashStringList& rClassName )
	{ 
		SdbHashStrList::operator =( rClassName ); 
		m_bCaseSensitive = rClassName.IsCaseSensitive();
		return *this; 
	}
	BOOL		operator ==( const SdbHashStringList& rList ) const
					{ return SdbHashStrList::operator ==( rList ); }
	BOOL		operator !=( const SdbHashStringList& rList ) const
					{ return SdbHashStrList::operator !=( rList ); }

	SdbHashString* Find(const ByteString& rStr) const;
	SdbHashString* Find(const SdbHashString& rStr) const;

	void			SetCaseSensitive(BOOL bEnable) { m_bCaseSensitive = bEnable; }
	BOOL			IsCaseSensitive() const { return m_bCaseSensitive; }

};

class SdbStringMap : private StringHashTab
{
	BOOL			m_bCaseSensitive : 1;		// werden die Objekte CaseSensitive abgelegt ?

#if !defined( OS2 ) && !defined( UNX ) && !defined( MAC )
protected:
#else
public:
#endif
#ifndef MAC
	StringHashTab::SetDataDelLink;
	StringHashTab::Derefer;
#else
	HashTabBase::SetDataDelLink;
	HashTabBase::Derefer;
#endif

public:
#ifndef MAC
	StringHashTab::GetNearPrime;
#else
	HashTabBase::GetNearPrime;
#endif

	SdbStringMap(HASHTYPE nMaxSlots = 17)
					:StringHashTab(nMaxSlots){}

	void			Clear();
	virtual	void*	SetAt(const String& rKey, void* pValue);
	void*			RemoveAt(const String& rKey);
	void*			Remove(void* pValue);

	String			GetKey(void* pValue) const;
	void*			Lookup(const String& rKey) const;

	ULONG			Count() const;
	void*			First() {return Derefer(StringHashTab::First());}
	void*			Next()	{return Derefer(StringHashTab::Next());}

	void			SetCaseSensitive(BOOL bEnable) { m_bCaseSensitive = bEnable; }
	BOOL			IsCaseSensitive() const { return m_bCaseSensitive; }

	virtual	HASHTYPE	Hash( const HASHNAMEPTR pName ) const;
	virtual int			NameCmp( const HASHNAMEPTR pName1, const HASHNAMEPTR pName2 ) const;

private:
	HASHSYMPTR		Find(void* pValue) const;
};

#ifndef MAC
#define DECL_SDB_STRING_MAP( ClassName, Type ) 					\
class ClassName : private SdbStringMap							\
{																\
public:															\
	SdbStringMap::GetNearPrime;									\
	SdbStringMap::Clear;										\
	SdbStringMap::Count;										\
																\
	ClassName(HASHTYPE nMaxSlots = 17)							\
			:SdbStringMap(nMaxSlots){}							\
																\
	Type*			SetAt(const String& rKey, Type* pValue)		\
		{return (Type*)SdbStringMap::SetAt(rKey, pValue);}		\
	Type*			Remove(Type* pValue)						\
		{return (Type*)SdbStringMap::Remove(pValue);}			\
	String			GetKey(Type* pValue) const 					\
		{return SdbStringMap::GetKey(pValue);}					\
	Type*			Lookup(const String& rKey) const			\
		{return (Type*)SdbStringMap::Lookup(rKey);}				\
	Type*			First()										\
		{return (Type*)SdbStringMap::First();}					\
	Type*			Next()										\
		{return (Type*)SdbStringMap::Next();}					\
};
#else
#define DECL_SDB_STRING_MAP( ClassName, Type ) 					\
class ClassName : private SdbStringMap							\
{																\
public:															\
	HashTabBase::GetNearPrime;									\
	SdbStringMap::Clear;										\
	SdbStringMap::Count;										\
																\
	ClassName(HASHTYPE nMaxSlots = 17)							\
			:SdbStringMap(nMaxSlots){}							\
																\
	Type*			SetAt(const String& rKey, Type* pValue)		\
		{return (Type*)SdbStringMap::SetAt(rKey, pValue);}		\
	Type*			Remove(Type* pValue)						\
		{return (Type*)SdbStringMap::Remove(pValue);}			\
	String			GetKey(Type* pValue) const 					\
		{return SdbStringMap::GetKey(pValue);}					\
	Type*			Lookup(const String& rKey) const			\
		{return (Type*)SdbStringMap::Lookup(rKey);}				\
	Type*			First()										\
		{return (Type*)SdbStringMap::First();}					\
	Type*			Next()										\
		{return (Type*)SdbStringMap::Next();}					\
};
#endif


class SdbStringRefMap : public SdbStringMap
{
	DECL_LINK(OnRemove, HASHSYMPTR);

public:
	SdbStringRefMap(HASHTYPE nMaxSlots = 17);

	virtual	void*  SetAt(const String& rKey, void* pValue);
};

#ifndef MAC
#define DECL_SDB_STRING_REF_MAP( ClassName, Type )				\
class ClassName : private SdbStringRefMap						\
{																\
public:															\
	SdbStringRefMap::GetNearPrime;								\
	SdbStringRefMap::Clear;										\
	SdbStringRefMap::Count;										\
																\
	ClassName(HASHTYPE nMaxSlots = 17)							\
			:SdbStringRefMap(nMaxSlots){}						\
																\
	Type*			SetAt(const String& rKey, Type* pValue)		\
		{return (Type*)SdbStringRefMap::SetAt(rKey, pValue);}	\
	Type*			Remove(Type* pValue)						\
		{return (Type*)SdbStringRefMap::Remove(pValue);}		\
	String			GetKey(Type* pValue) const 					\
		{return SdbStringRefMap::GetKey(pValue);}				\
	Type*			Lookup(const String& rKey) const			\
		{return (Type*)SdbStringRefMap::Lookup(rKey);}			\
	Type*			First()										\
		{return (Type*)SdbStringRefMap::First();}				\
	Type*			Next()										\
		{return (Type*)SdbStringRefMap::Next();}				\
};
#else
#define DECL_SDB_STRING_REF_MAP( ClassName, Type )				\
class ClassName : private SdbStringRefMap						\
{																\
public:															\
	HashTabBase::GetNearPrime;								\
	SdbStringMap::Clear;										\
	SdbStringMap::Count;										\
																\
	ClassName(HASHTYPE nMaxSlots = 17)							\
			:SdbStringRefMap(nMaxSlots){}						\
																\
	Type*			SetAt(const String& rKey, Type* pValue)		\
		{return (Type*)SdbStringRefMap::SetAt(rKey, pValue);}	\
	Type*			Remove(Type* pValue)						\
		{return (Type*)SdbStringRefMap::Remove(pValue);}		\
	String			GetKey(Type* pValue) const 					\
		{return SdbStringRefMap::GetKey(pValue);}				\
	Type*			Lookup(const String& rKey) const			\
		{return (Type*)SdbStringRefMap::Lookup(rKey);}			\
	Type*			First()										\
		{return (Type*)SdbStringRefMap::First();}				\
	Type*			Next()										\
		{return (Type*)SdbStringRefMap::Next();}				\
};
#endif


inline SdbHashString::SdbHashString(const ByteString& rStr)
		   :aStr(rStr)
		   ,nHashID(Hash(rStr)){}

inline SdbHashString::SdbHashString(const String& rStr)
		   :aStr(ByteString(rStr,gsl_getSystemTextEncoding()))
		   ,nHashID(Hash(aStr)){}

inline SdbHashString::SdbHashString(const SdbHashString& rHashStr)
		   :aStr(rHashStr.aStr)
		   ,nHashID(rHashStr.nHashID){}

inline SdbHashString& SdbHashString::operator = (const SdbHashString& rHashStr)
{
	aStr = rHashStr.aStr;
	nHashID = rHashStr.nHashID;
	return *this;
}

inline SdbHashString& SdbHashString::operator = (const ByteString& rStr)
{
	aStr = rStr;
	nHashID = Hash(rStr);
	return *this;
}

inline ULONG SdbHashString::Hash(const ByteString& rStr)
{
	register const char* p = rStr.GetBuffer();
	register ULONG nHash = 0;
	while (*p)
		nHash = (nHash << 1) + toupper(*((unsigned char*)p++));
	return nHash;
}


#endif


