/*************************************************************************
 *
 *  $RCSfile: sdbidx.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBIDX_HXX
#define _SDB_SDBIDX_HXX 1

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBCOL_HXX
#include "sdbcol.hxx"
#endif

class SdbTable;

enum SdbIndexStyle
{
	SDB_IDX_UNIQUE		= 0x0001,
	SDB_IDX_CLUSTERED   = 0x0002
};

class SdbIndex : public SdbObj
{
	friend class SdbTable;

protected:
	String aName,	aQualifierName;
	SdbColumnsRef	aColumns,
					aOriginalColumns;
	UINT16			nStyle;

#if (SUPD >= 375)
	void*  pVOID;			// free for later use
	BOOL			bSystemReadOnly; // gibt an ob der Index von der DB angelegt wurde und somit nicht lschbar ist
	BOOL			b2,b3,b4;
	//UINT32 nFree;
#endif

	SdbIndex(SdbTable* _pTable);
	~SdbIndex();

public:
	TYPEINFO();

	// Zugriff auf den Vater
	virtual SdbTable* GetTable() const;

	// Erzeugen eines Indizes
	virtual BOOL Create(const String & rName);
	virtual BOOL Update();	// Erzeugen des Index in der Datenbank

	// Index loeschen
	virtual BOOL Drop();

	// Setzen von Flags:
	// Einzelne Flags:
	void SetStyle(SdbIndexStyle eNewStyle) {nStyle |= eNewStyle;}
	void ResetStyle(SdbIndexStyle eNewStyle) {nStyle &= ~eNewStyle;}
	void ResetStyles() {nStyle = 0;}

	BOOL IsUnique() const {return (nStyle & SDB_IDX_UNIQUE) == SDB_IDX_UNIQUE;}
	BOOL IsClustered()  const {return (nStyle & SDB_IDX_CLUSTERED) == SDB_IDX_CLUSTERED;}
	BOOL IsSystemReadOnly() const { return bSystemReadOnly;}

	SdbColumnsRef Columns() const {return aColumns;}
	const SdbColumnsRef& OriginalColumns() const {return aOriginalColumns;}

	// Der Name des Index etc.
	const String& Name() const {return aName;}
	const String& Qualifier() const {return aQualifierName;}

	inline String GetFullName() const;

protected:
	virtual BOOL Compare(const SdbObj& ) const;
	virtual void Close();

protected:
	// Oeffnen eines bestehenden Indices
	virtual BOOL Open(const String& rName);
	virtual BOOL Open();

	// Implementierungen, die vom Treiber zu realisieren sind
	virtual BOOL CreateImpl() = 0;
	virtual BOOL DropImpl() = 0;
	virtual BOOL OpenImpl() = 0;

	// aktualisieren
	virtual BOOL DoRefresh();

#if __PRIVATE
	SdbConnection* GetConnection() const;

	#define SDB_CHK_TABLE()								\
		SdbTable* pTable = GetTable();					\
		if (!pTable)									\
		{												\
			aStatus.SetConnectionLost();				\
			return;										\
		}

	#define SDB_CHK_TABLE_BOOL()						\
		SdbTable* pTable = GetTable();					\
		if (!pTable)									\
		{												\
			aStatus.SetConnectionLost();				\
			return FALSE;								\
		}

	#define SDB_CHK_TABLE_STRING()						\
		SdbTable* pTable = GetTable();					\
		if (!pTable)									\
		{												\
			aStatus.SetConnectionLost();				\
			return FALSE;								\
		}

	#define SDB_CHK_TABLE_NULL()						\
		SdbTable* pTable = GetTable();					\
		if (!pTable)									\
		{												\
			aStatus.SetConnectionLost();				\
			return FALSE;								\
		}
#endif
};

SV_DECL_IMPL_REF(SdbIndex);

// inline Implementierung
//------------------------------------------------------------------
inline String SdbIndex::GetFullName() const
{
	String aTemp;
	if(aQualifierName.Len())
	{
		aTemp = aQualifierName;
		aTemp.AppendAscii(".");
	}
	aTemp += aName;
	return aTemp;
}

#endif

