/*************************************************************************
 *
 *  $RCSfile: sdbobj.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBOBJ_HXX
#define _SDB_SDBOBJ_HXX 1

#ifndef _REF_HXX //autogen
#include <tools/ref.hxx>
#endif

#ifndef _RTTI_HXX //autogen
#include <tools/rtti.hxx>
#endif

#ifndef _SDB_SDBSTAT_HXX
#include "sdbstat.hxx"
#endif

//==================================================================
// SdbUiActionDescription
//	gedacht fuer Situationen, in denen ein Eingriff des Benutzers eventuell die Situation retten koennte,
//	oder fuer Warnungen vor Operationen, die vielleicht ungewollte Effekte haben koennten (so etwas laesst sich
//	manchmal wirklich erst tief in der Core feststellen).
//	Hierueber werden keine Fehler gemeldet, die kommen immer ueber den Status zurueck (dito Informationen, die passen in
//	SUCCESS_WITH_INFO) !
//==================================================================
struct SdbUiActionDescription
{
	enum FLAGS
	{
		YES_NO			= 0x0001,
		YES_NO_CANCEL	= 0x0002,
		OK				= 0x0003,
		OK_CANCEL		= 0x0004
	};
	INT16	eType;
	String	sMessage;

	enum RETURN
	{
		RETURN_YES		= 0x0001,
		RETURN_NO		= 0x0002,
		RETURN_CANCEL	= 0x0003,
		RETURN_OK		= 0x0004
	};
};

//==================================================================
// SdbObj
//==================================================================
class SdbObj : public SvRefBase
{
#if SUPD < 561
private:
	SdbObj*		pParent;				// VaterObjekt
	BOOL		bPersistent : 1;		// Objekt persistent
	Link		aCloseLink;				// Link der beim Schlieen eines Objekts
										// aufgerufen wird.
	Link		m_aUiActionLink;		// UI-Operationen

protected:
	static const String aEmptyString;	// Hilfstring

	SdbStatus	aStatus;				// Status des Objekts
	BOOL		bOpen	: 1,			// Objekt geoeffnet
				bNew	: 1;			// wird das Objekt neu angelegt, nur bei persistenten
										// Objekten relevant
	void	SetTemporary(BOOL bTemp=FALSE){	bPersistent = bTemp; }

#else
protected:
	SdbStatus	aStatus;				// Status des Objekts
private:
	Link		aCloseLink;				// Link der beim Schlieen eines Objekts
										// aufgerufen wird.
	Link		m_aUiActionLink;		// UI-Operationen
protected:
	static const String aEmptyString;	// Hilfstring
private:	
	SdbObj*		pParent;				// VaterObjekt
	BOOL		bPersistent : 1;		// Objekt persistent
protected:
	BOOL		bOpen	: 1,			// Objekt geoeffnet
				bNew	: 1,			// wird das Objekt neu angelegt, nur bei persistenten
										// Objekten relevant
				m_bTemporary :1;		// not owned by parent
	void	SetTemporary(BOOL bTemp=TRUE){	m_bTemporary = bTemp; }
#endif // SUPD < 561

public:
	TYPEINFO();

	// Zugriff auf den Vater
	SdbObj*	GetParent() const {return pParent;}

	// Persistenz
	// Objekte sind dann persistent, wenn sie solange leben wie ihr Vater.
	// Dieses ist beispielsweise bei Tabellen der Fall. Diese werden von ihrem
	// Vater auch dann festgehalten, wenn kein RefCount mehr fuer das Objekt
	// besteht. Cursor hingegen sind transient, sie werden nach Freigabe
	// auch vom Parent nicht mehr gehalten
	BOOL IsPersistent() const {return bPersistent;}

	// Statusinformationen, ob ein Objekt geoeffnet oder geschlossen ist.
	// Das Schliessen eines Objekts kann auch implizit erfolgen, in dem z.B.
	// der Parent eines Objects geloescht wird. In diesem Fall wird das Objekt selbst
	// geschlossen aber nicht zerstoert
	BOOL IsOpen() const {return bOpen && GetParent();}

	// Ist das Objekt neu angelegt und noch nicht in der Datenbank abgelegt
	BOOL IsNew() const {return bNew && GetParent();}

	// Aktualisieren eines Objects,
	// dieses kann in der Regel nur fuer persistente Objekte ausgefuerht werden
	// die nicht den Status neu besitzen
	// abgeleitete Klassen muessen ein DoRefresh implementieren
	virtual BOOL Refresh();

	// jedes Object mu vergleichbar sein
	inline BOOL operator==(const SdbObj& _rObj) const;
	BOOL operator!=(const SdbObj& _rObj) const {return !(*this == _rObj);}

	// Callback bei Aenderungen von Open auf Closed
	void SetCloseHdl(const Link& rLink) {aCloseLink = rLink;}

	// Status nach der letzten Datenbankoperation
	const SdbStatus& Status() const {return aStatus;}

	// Callback bei Status-Aenderungen:
	inline void SetStatusChangeHdl(const Link& rNewLink);

	Link SetUiActionLink(const Link& rNewHandler) { Link aOld = m_aUiActionLink; m_aUiActionLink = rNewHandler; return aOld; }
		// der Link bekommt einen Zeiger auf eine SdbUiActionDescription-Struktur uebergeben.
		// zurueckgeben muss er einen der SdbUiActionDescription::RETURN-Werte
	Link GetUiActionLink() const { return m_aUiActionLink; }

protected:
	SdbObj(SdbObj* pPar, BOOL bPersist)
		:pParent(pPar)
		,bPersistent(bPersist)
		,bOpen(FALSE)
#if SUPD < 561
		,bNew(FALSE){};
#else
		,bNew(FALSE)
		,m_bTemporary(FALSE){};
#endif // SUPD < 561

	virtual ~SdbObj();

	// Entscheidung ob ein Objekt tatschlich gelscht wird oder ob
	// es weiterhin im Container seines Parents verwahrt wird
	virtual void QueryDelete();

	// Freigeben von Kinder
	virtual void RemoveChild(SdbObj* pChild);
	virtual void RemoveChildren();

	// Freigeben der Verbindung zum Parent
	// ist noetig,
	// a.) wenn der Parent zerstoert werden soll
	// b.) ein Object garnicht von seinem Parent verwaltet,
	//	   sprich in einer Liste gefuehrt wird
	void RemoveFromParent();

	// Oeffnen eines Objekts kann in der Regel nur ueber
	// seinen Parent uebernommen werden
	virtual BOOL Open()	= 0;
	virtual void Close();

	virtual SdbUiActionDescription::RETURN DoUiAction(const SdbUiActionDescription& rDesc);
		// wenn mein eigener Handler nicht gesetzt ist, wird das DoUiAction des Parents aufgerufen, welches dann wiederum ...

protected:
	//	Vergleichsoperation, default return FALSE;
	virtual BOOL Compare(const SdbObj& _rObj) const;
	virtual BOOL DoRefresh();

	/** This method will be called from within RemoveFromParent, _after_ the object has beed closed (i.e. Close has been called,
		if neccessary) and _before_ the pParent-member will be reset to NULL. After returning from this function you can't make
		any assumptions on your (former) parent, so do any cleanup here which requires a valid parent (e.g. cursors should drop
		their statements here if this requires a valid connection).
	*/
#if SUPD>557
	virtual
#endif
		void RemovingFromParent();

	//  Sollte eigentlich keine Verwendung finden
	void SetPersistent(BOOL bPersist){bPersistent = bPersist;}

private:
	// Kopieren eines SdbObj mit Hilfe eines Copy Constructor
	// oder operator= ist nicht moeglich.
	SdbObj(const SdbObj&){};
	SdbObj& operator=(const SdbObj&){return *this;};

};

// Inline Implementation
//------------------------------------------------------------------
inline BOOL SdbObj::operator==(const SdbObj& _rObj) const
{
	return Compare(_rObj);
}

//------------------------------------------------------------------
inline void SdbObj::SetStatusChangeHdl(const Link& rNewLink)
{
	aStatus.SetStatusChangeHdl(rNewLink);
}

// Hilfsmacros
#if __PRIVATE
	// Makro um zu Pruefen, ob Aktion ueberhaupt erlaubt
	#define SDB_CHK_OPEN()						\
			aStatus.Clear();					\
			if (!IsOpen())						\
			{									\
				aStatus.SetSequenceError();		\
				return;							\
			}

	#define SDB_CHK_OPEN_BOOL()					\
			aStatus.Clear();					\
			if (!IsOpen())						\
			{									\
				aStatus.SetSequenceError();		\
				return FALSE;					\
			}

	#define SDB_CHK_OPEN_NULL()					\
			aStatus.Clear();					\
			if (!IsOpen())						\
			{									\
				aStatus.SetSequenceError();		\
				return NULL;					\
			}

	#define SDB_CHK_OPEN_STRING()				\
			aStatus.Clear();					\
			if (!IsOpen())						\
			{									\
				aStatus.SetSequenceError();		\
				return String();				\
			}

	// Stati
	#define HAS_ERROR()							\
			return aStatus.IsError();

	#define IS_SUCCESSFUL()						\
			return aStatus.IsSuccessful();

	#define IS_EXECUTING()						\
			return aStatus.IsExecuting();

	#define HAS_NO_DATA()						\
			return aStatus.IsNoDataFound();
#endif

#endif
