/*************************************************************************
 *
 *  $RCSfile: sdbrel.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBREL_HXX
#define _SDB_SDBREL_HXX 1

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBCOL_HXX
#include "sdbcol.hxx"
#endif

class SdbConnection;

//==================================================================
// SdbColumnRelation
// Spalte einer Relation, erhaelt zusaetzlich den Spaltennamen der
// referenzierten Tabelle
//==================================================================
class SdbColumnRelation : public SdbColumn
{
	friend class SdbRelation;
	String aForeignName;	// Name der Spalte der referenzierten Tabelle

	SdbColumnRelation(const SdbColumn& _rCol, const String& _rForeignName)
		:SdbColumn(_rCol)
		,aForeignName(_rForeignName)
	{
		if (!aForeignName.Len())
			aForeignName = _rCol.GetName();
	}

	SdbColumnRelation(const SdbColumnRelation& _rCol)
		:SdbColumn(_rCol)
		,aForeignName(_rCol.GetForeignName())
	{
		if (!aForeignName.Len())
			aForeignName = _rCol.GetName();
	}

public:
	TYPEINFO();
	const String& GetForeignName() const {return aForeignName;}
	void SetForeignName(const String& _rName) {aForeignName = _rName;}

protected:
	virtual SdbColumn* Clone() const;
};

typedef ODbRef< SdbColumnRelation > SdbColumnRelationRef;

//==================================================================
// SdbRelation
//==================================================================
class SdbRelation : public SdbObj
{
	friend class SdbConnection;

	SdbColumnsRef aColumns;
	String		  aTableName,			// Name der referenzierenden Tabelle
				  aForeignTableName;	// Name der referenzierten Tabelle
	UINT16		  nUpdateRules;			// ver-oderte Werte wie in den
	UINT16		  nDeleteRules;			// Metadatendefinitionen

protected:
	String		  aName;				// Name des Constraints

protected:
	SdbRelation(SdbConnection* _pConn,
				const String& _rName);
	~SdbRelation();

public:
	TYPEINFO();

	// Zugriff auf den Vater
	virtual SdbConnection* GetConnection() const;
	virtual BOOL Update();	// Erzeugen einer Relation in der Datenbank

	// Relation loeschen
	virtual BOOL Drop();

	virtual BOOL Refresh();

	// Attribute
	UINT16 GetUpdateRules() const {return nUpdateRules;}
	UINT16 GetDeleteRules() const {return nDeleteRules;}
	inline void SetUpdateRules(UINT16 _nFlags);
	inline void SetDeleteRules(UINT16 _nFlags);

	// Name der Relation (Constraints)
	const String& GetName() const {return aName;}

	// Zugriff auf die referenzierende Tabelle
	const String& GetTableName() const {return aTableName;}
	inline void SetTableName(const String& _rName);

	// Zugriff auf die referenzierte Tabelle
	const String& GetForeignTableName() const {return aForeignTableName;}
	inline void SetForeignTableName(const String& _rName);

	SdbColumnsRef GetColumns() const {return aColumns;}

	// ist _rForeignName nicht gesetzt, wird der Name von _rCol uebernommen
	SdbColumnRelation* CreateColumn(const SdbColumn& _rCol, const String& _rForeignName) const;

protected:
	virtual BOOL Compare(const SdbObj& ) const;
	virtual BOOL Open();

#if __PRIVATE
public:
	static String CreateTempName();
	static BOOL IsTempName(const String& rName);
protected:
#endif


	UINT16 GetUpdateRules(UINT16 _nMask) const
		{return nUpdateRules & _nMask;}

	UINT16 GetDeleteRules(UINT16 _nMask) const
		{return nDeleteRules & _nMask;}

	void NameChanged();
};

SV_DECL_IMPL_REF(SdbRelation);

// Inline Implementation
//------------------------------------------------------------------
inline void SdbRelation::SetUpdateRules(UINT16 _nFlags)
{
	DBG_ASSERT(IsNew() || IsOpen(), "Relation not new");
	nUpdateRules = _nFlags;
}

inline void SdbRelation::SetDeleteRules(UINT16 _nFlags)
{
	DBG_ASSERT(IsNew() || IsOpen(), "Relation not new");
	nDeleteRules = _nFlags;
}


inline void SdbRelation::SetTableName(const String& _rName)
{
	DBG_ASSERT(IsNew() || IsOpen(), "Relation not new");
	aTableName = _rName;
}

inline void SdbRelation::SetForeignTableName(const String& _rName)
{
	DBG_ASSERT(IsNew() || IsOpen(), "Relation not new");
	aForeignTableName = _rName;
}

//------------------------------------------------------------------

#endif

