/*************************************************************************
 *
 *  $RCSfile: sdbset.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBSET_HXX
#define _SDB_SDBSET_HXX 1

#ifndef _SDB_ROW_HXX
#include <row.hxx>
#endif

#ifndef _VOS_MACROS_HXX
#include <vos/macros.hxx>
#endif

#ifndef _VOS_THREAD_HXX_ //autogen
#include <vos/thread.hxx>
#endif

#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

#include "sdbcancl.hxx"

class Timer;
class OValueSet;
class SdbTableRef;

using namespace std;

struct OKeySetColumn
{
	BOOL	bUnique;
	UINT16	nId;

	OKeySetColumn():bUnique(FALSE),nId(0){};
	OKeySetColumn(UINT16 Id, BOOL Unique)
		:bUnique(Unique),nId(Id){};
};

class ORowCounter : public vos::OThread
{
	friend class OKeySet;

	OKeySet& m_rSet;

public:
	ORowCounter(OKeySet& _rSet);

protected:
	virtual void SAL_CALL run();
	virtual void SAL_CALL onTerminated();
};

// KeysetImplementierung kann anhand von eindeutigen Werten Positionierung
// und Updates vornehmen
class OKeySet : public SdbCancelable
{
	friend class ORowCounter;

protected:
	SdbColumnsRef		m_xResultColumns;
	String				m_aRefetchTableName;

private:
	Timer*				m_pTimer;
	ORowCounter*		m_pCounter;

protected:
	ODbRowRef			m_xBindRow;		// extra Row fuer Bindings
	SdbCursor&			m_rOrigCursor;
	::com::sun::star::uno::Any
						m_aLastBookmark;
	INT32				m_nCurrentPos;
	
	UINT8				m_nPrivileges;
	BOOL				m_bSnapshot : 1;
	BOOL				m_bBound : 1;

public:
	OKeySet(SdbCursor& rCursor);
	virtual ~OKeySet();

	BOOL seek(SdbCursorPosition eCursorPosition, INT32 nOffset,
			  SdbCursor::SdbCursorMode& rMode, BOOL bRetrieveData = TRUE, BOOL bStopCounter = TRUE );

	BOOL seek(const ::com::sun::star::uno::Any& rBookmark,
			  SdbCursor::SdbCursorMode& rMode, BOOL bRetrieveData = TRUE, BOOL bStopCounter = TRUE ) 
			  throw( ::com::sun::star::lang::IllegalArgumentException );

	BOOL isUpdatable() const;
	UINT8 getPrivileges() const {return m_nPrivileges;}
	INT32 getCurrentPos() const {return m_nCurrentPos;}

	BOOL insertRow();
	BOOL updateRow();
	BOOL deleteRow();

	void startCounting();
	void prepareAppendRow(const ODbRowRef& rRow);
	void prepareDelete();

	BOOL isSnapshot() const { return m_bSnapshot; }
	virtual INT32 size() const = 0;
	virtual BOOL setUpdatable(BOOL bUpdate);
	virtual void determineRowCount();

protected:
	vos::IMutex& getMutex() {return *m_rOrigCursor.pLock;}

	// binden von Ergebnisfeldern
	BOOL	isBound() const {return m_bBound;}
	virtual void bind() = 0;
	virtual void unBind() = 0;

	BOOL getNext(UINT32& i, BOOL bReleaseBindings = FALSE);

private:
	BOOL fillSet(UINT32 nNewPos);
	DECL_LINK(onTimeout, void*);

	// Zaehlen uber IdleHandling
	DECL_LINK(onCount, void*);

protected:
	virtual BOOL doInsertRow() = 0;
	virtual BOOL doUpdateRow() = 0;
	virtual BOOL doDeleteRow() = 0;
	virtual BOOL fetchNext(SdbCursor::SdbCursorMode& rMode) = 0;
	virtual BOOL seekPositioned(BOOL bRetrieveData) 
		throw( ::com::sun::star::lang::IllegalArgumentException ) = 0;
	virtual BOOL getBookmarkPos(INT32& nPos, const ::com::sun::star::uno::Any& rBookmark) const 
		throw( ::com::sun::star::lang::IllegalArgumentException ) = 0;
};

class OValueRow : private vector< ::com::sun::star::uno::Any>
{	
	INT32 m_nBookmark;

public:
	OValueRow():m_nBookmark(0){}
	OValueRow(INT32 nBookmark):m_nBookmark(nBookmark){}
	OValueRow(INT32 nBookmark, const ODbRow& rRow, const OValueSet& rSet);

	INT32 getBookmark() const {return m_nBookmark;}

	void setRow(const ODbRow& rRow, const OValueSet& rSet);
	void getRow(ODbRow& rRow, const OValueSet& rSet) const;

	// vector methods
	vector< ::com::sun::star::uno::Any>::operator[];		
	vector< ::com::sun::star::uno::Any>::begin;
	vector< ::com::sun::star::uno::Any>::end;
	vector< ::com::sun::star::uno::Any>::rbegin;
	vector< ::com::sun::star::uno::Any>::rend;
	vector< ::com::sun::star::uno::Any>::front;
    vector< ::com::sun::star::uno::Any>::back;

	vector< ::com::sun::star::uno::Any>::size;
    vector< ::com::sun::star::uno::Any>::empty;	
};

// Spezielle KeysetImplementierung die Positionierung und Updates uber Bookmarks am
// Originaltreiber vornimmt
class OValueSet : public OKeySet
{
	friend class OValueRow;

	SdbCursorRef			m_xBackgroundCursor;
	vector<OKeySetColumn>	m_aKeyColumns;
	vector<INT16>			m_aRefetchColumns;
	vector<OValueRow>		m_aRowSet;

public:
	OValueSet(SdbCursor& rCursor);
	virtual ~OValueSet();

protected:
	virtual INT32 size() const {return (INT32)m_aRowSet.size();}
	virtual void bind();
	virtual void unBind();
	virtual BOOL doInsertRow();
	virtual BOOL doUpdateRow();
	virtual BOOL doDeleteRow();
	virtual BOOL fetchNext(SdbCursor::SdbCursorMode& rMode);
	virtual BOOL seekPositioned(BOOL bRetrieveData) 
		throw( ::com::sun::star::lang::IllegalArgumentException );
	virtual BOOL getBookmarkPos(INT32& nPos, const ::com::sun::star::uno::Any& rBookmark) const 
		throw( ::com::sun::star::lang::IllegalArgumentException );

private:
	BOOL appendWhere(String& aStr,USHORT nPos,BOOL bCurrency=TRUE) const;
	BOOL prepareCursor(const String& rTableName, ODbRow* pRow = NULL);
	BOOL createCursor(const SdbTableRef& xTable);
	BOOL checkColumns(const SdbTableRef& xTable);
	void setParameterValues(BOOL bCurrency=TRUE);
	void appendParameterColumns(BOOL bCurrency=TRUE);
	void checkConcurrency();
};

// Spezielle KeysetImplementierung die Positionierung und Updates uber Bookmarks am
// Originaltreiber vornimmt
class OBookmarkSet : public OKeySet
{
	// bookmarks are always INT32 and ordered!!!!
	vector<INT32>	m_aRowSet;
	::com::sun::star::uno::Any
					m_aCurBookmark;

public:
	OBookmarkSet(SdbCursor& rCursor);
	~OBookmarkSet();

	// binden von Ergebnisfeldern
	virtual void bind();
	virtual void unBind();

	// Einfuegen von Rows direkt
	// Dieses macht dann Sinn, wenn nach dem Erzeugen des CursorSets bereits alle Rows gelesen worden
	// sind. Wird bisher nur vom FileCursor verwendet!
	void appendRow(INT32 nValue);

protected:
	virtual INT32 size() const {return (INT32)m_aRowSet.size();}
	virtual BOOL doInsertRow();
	virtual BOOL doUpdateRow();
	virtual BOOL doDeleteRow();
	virtual BOOL fetchNext(SdbCursor::SdbCursorMode& rMode);
	virtual BOOL seekPositioned(BOOL bRetrieveData)
		throw( ::com::sun::star::lang::IllegalArgumentException );
	virtual BOOL setUpdatable(BOOL bUpdate);
	virtual void determineRowCount();
	virtual BOOL getBookmarkPos(INT32& nPos, const ::com::sun::star::uno::Any& rBookmark) const 
		throw( ::com::sun::star::lang::IllegalArgumentException );
};

#endif

