/*************************************************************************
 *
 *  $RCSfile: sdbsqlit.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBSQLIT_HXX
#define _SDB_SDBSQLIT_HXX 1

#ifndef _SDB_SDBSTAT_HXX
#include "sdbstat.hxx"
#endif
#ifndef _SVTOOLS_HASHCONT_HXX //autogen
#include <svtools/hashcont.hxx>
#endif

#ifndef _SDB_SDBCOL_HXX //autogen
#include <sdbcol.hxx>
#endif
#ifndef _SDB_SDBDB_HXX //autogen
#include <sdbdb.hxx>
#endif

class SdbTableRef;
DECLARE_SDBKEYCONT(SdbTableRangeMap,SdbTableRef*);

class SdbSqlParseNode;


enum SdbStatementType {
	SDB_STATEMENT_UNKNOWN,
	SDB_STATEMENT_SELECT,
	SDB_STATEMENT_INSERT,
	SDB_STATEMENT_UPDATE,
	SDB_STATEMENT_DELETE,
	SDB_STATEMENT_ODBC_CALL
};

enum SdbPredicateType {
	SDB_PRED_EQUAL = 'a',		// Als sichtbare ASCII-Zeichen, damit man
	SDB_PRED_NOTEQUAL,			// den Predicate Type auch in Strings
	SDB_PRED_LESS,				// speichern kann.
	SDB_PRED_LESSOREQUAL,
	SDB_PRED_GREATER,
	SDB_PRED_GREATEROREQUAL,
	SDB_PRED_LIKE,
	SDB_PRED_ISNULL,
	SDB_PRED_ISNOTNULL,
	SDB_PRED_NOTLIKE
};

//==================================================================
// SbaParseIteratorErrorInfo wird dem Call von aErrorHdl aus SbaParseIterator "ubergeben
// nErrorCode enth"alt eine Zusatzinformation "uber den Fehler
// 1	->		Tabelle nicht gefunden
// 2	->		Spalte nicht gefunden
//==================================================================
struct SdbParseIteratorErrorInfo
{
	SdbStatus	aStatus;
	USHORT		nErrorCode;		// 1 == Tabelle nicht gefunden, 2 == Spalte nicht gefunden
	String		aExpression;	// der Teil-Ausdruck, der das Problem verursacht hat (kann leer sein)
};

#define RET_CONTINUE	1	// Parsevorgang fortsetzen
#define RET_HANDLED		2	// der Fehler wurde schon behandelt, das Parsen soll (mit Status auf Success) abgebrochen werden
#define	RET_BREAK		3	// Abbrechen, Status-Fehlercode setzen

class SdbDatabase;
class SdbConnection;
class SdbSqlParseTreeIterator
{
  private:
	const SdbSqlParseNode	*pParseTree;	// aktueller ParseTree
	SdbStatementType		eStatementType; // Art des Statements
	SdbTableRangeMap		aTableRangeMap; // Alle Tabellen die im ParseTree und bei der Connection gefunden wurden
	SdbColumnsRef			aSelectColumns; // alle Spalten aus dem Select-Clause
	Link					aErrorHdl;		// wird im Fehlerfall gerufen

	// F"ugt eine Tabelle in die Map ein
	void				TraverseOneTableName(const SdbSqlParseNode * pTableName, const String & rTableRange);
	void				TraverseORCriteria(SdbSqlParseNode * pSearchCondition);
	void				TraverseANDCriteria(SdbSqlParseNode * pSearchCondition);
	void				TraverseOnePredicate(
											SdbSqlParseNode * pColumnRef,
											SdbPredicateType ePredicateType,
											String& aValue,
											BOOL bCompareNull,
											SdbSqlParseNode * pParameter);
	SdbSqlParseNode *	TableRef(SdbSqlParseNode *pTableRef,String& aTableRange);
	SdbSqlParseNode *	Qualified_join(SdbSqlParseNode *pTableRef,String& aTableRange);
	void				Select_statement(SdbSqlParseNode *pSelect);
	BOOL				CallError(USHORT nError, const SdbStatus& rStatus, const String& rExpression = String());
	String				GetUniqueColumnName(const String & rColumnName)	const;

  protected:
	SdbStatus		aIteratorStatus;
	SdbConnection*	pConnection;
	void SetSelectColumnName(const String & rColumnName,const String & rColumnAlias, const String & rTableRange,BOOL bFkt=FALSE);
	void AppendColumns(ULONG i,const SdbTableRef* pTableRef);
	// Weitere Member-Variable, die in den "Set"-Funktionen zur
	// Verfuegung stehen sollen, koennen in der abgeleiteten Klasse
	// definiert werden und z. B. in deren Konstruktor initialisiert
	// bzw. nach Benutzung der "Traverse"-Routinen mit Hilfe weiterer
	// Funktionen abgefragt werden.


  public:
	SdbSqlParseTreeIterator();
	SdbSqlParseTreeIterator(const SdbSqlParseTreeIterator & rIter);
	SdbSqlParseTreeIterator(SdbDatabase* rDB,const SdbSqlParseNode* pRoot);
	SdbSqlParseTreeIterator(SdbConnection* rConn,const SdbSqlParseNode* pRoot);
	~SdbSqlParseTreeIterator();

	// Der zu analysierende/zu traversierende Parse Tree:
	// bei "Ubergabe von NULL wird der aktuelle Parsetree gel"oscht und der Fehlerstatus gecleared
	void SetParseTree(const SdbSqlParseNode * pNewParseTree);
	const SdbSqlParseNode * ParseTree() const { return pParseTree; };

	// Teilbaueme bei einem select statement
	const SdbSqlParseNode* WhereTree() const;
	const SdbSqlParseNode* OrderTree() const;

	// Status nach Ausfuerung der "Traverse"-Routinen:
	const SdbStatus & IteratorStatus() const { return aIteratorStatus; }

	// Statement-Typ (wird bereits in SetParseTree gesetzt):
	SdbStatementType StatementType() const { return eStatementType; }

	// Die "Traverse"-Routinen durchlaufen bestimmte Teile des Parse Tree
	// und rufen fuer die erkannten Konstrukte die virtuellen "Set"-Routinen
	// auf und uebergeben diesen die erkannten Informationen als Parameter.
	//
	// Der Parse Tree muss in einer bestimmten Form vorliegen:
	//		SELECT [<tablerange>.]<columnname>, [<tablerange>.]<columnname>, ...
	//		  FROM <tablename> [<tablerange>], <tablename> [<tablerange>], ...
	//		 WHERE (
	//					<predicate>
	//				AND <predicate>
	//				...
	//			   )
	//			OR (
	//				...
	//			   )
	//		    ...
	//		 ORDER BY <columname>, <columnname>, ...
	//
	// (die Klammern sind optional bzw. zusaetzliche Klammern stoeren nicht, aber
	// es duerfen nur mehrere AND-Bedingungen, die ihrerseits mit OR verknuepft sind,
	// auftreten).
	//
	//
	// <predicate> kann sein:
	//			[<tablerange>.]<columnname> <operator> <value>
	//			[<tablerange>.]<columnname> LIKE <value>
	//			[<tablerange>.]<columnname> NOT LIKE <value>
	//			[<tablerange>.]<columnname> IS NULL
	//			[<tablerange>.]<columnname> IS NOT NULL
	//
	// <operator> kann sein:
	//			"="
	//			"<>"
	//			"<"
	//			"<="
	//			">"
	//			">="
	//
	// <value> kann auch ein Parameter sein, in diesem Fall enthaelt
	// das Argument "ParameterName" der "Set"-Funktionen den Namen des Parameters
	// (ohne fuehrenden Doppelpunkt) bzw. "?" bei unbenannten Parametern.
	//
	// <columnname> in der Select-Klausel kann auch "*" oder "COUNT(*)" sein.
	//
	//
	// Wenn das Statement NICHT diese Form hat, oder wenn eine der "Set"-Routinen
	// den IteratorStatus != IsSuccessful() hinterlaesst, wird die weitere Analyse
	// des Parse Tree abgebrochen. Ansonsten liefert "Status().IsSuccessful() == TRUE".

	void TraverseTableNames();
	virtual void SetTableName(const String & rTableName, const String & rDBName, const String& rOwner,
							  const String & rTableRange);
	// [TableName enthaelt immer einen Namen, TableRange ist, falls angegeben, die "Range"-
	// Variable (eine Art Alias-Name fuer den TableName), falls nicht angegeben, identisch
	// zum TableName. SchemaName ist leer, wenn nicht angegeben.]

	void TraverseSelectColumnNames(const SdbSqlParseNode* pSelectNode);
	// [TableRange kann leer sein, wenn nicht angegeben]

	void TraverseOrderByColumnNames(const SdbSqlParseNode* pSelectNode);
	virtual void SetOrderByColumnName(const String & rColumnName, const String & rTableRange, BOOL bAscending);
	// [TableRange kann leer sein, wenn nicht angegeben]

	// Bei Selektionskriterien werden (selbst bei einem einfachen Praedikat)
	// folgende "Set"-Funktionen in der angegebenen Reihenfolge aufgerufen:
	//
	//			SetORCriteriaPre
	//			|
	//			|	SetANDCriteriaPre
	//			|	|	SetPredicate
	//			|	|	[weitere SetPredicate-Aufrufe] ...
	//			|	SetANDCriteriaPost
	//			|
	//			|	... [weitere SetANDCriteriaPre/Post-Paare und darin SetPredicate-Aufrufe]
	//			|
	//			SetORCriteriaPost
	//
	// Die AND-Verknuepfungen sind also implizit ODER-Verknuepft. SetORCriteriaPre und
	// SetORCriteriaPost werden jeweils nur ein einziges Mal aufgerufen (sind also
	// eigentlich ziemlich ueberfluessig, da man diese Aktionen auch vor bzw. nach dem
	// Traverse-Aufruf erledigen kann)!
	//
	void TraverseSelectionCriteria(const SdbSqlParseNode* pSelectNode);
	virtual void SetORCriteriaPre();
	virtual void SetORCriteriaPost();
	virtual void SetANDCriteriaPre();
	virtual void SetANDCriteriaPost();
	virtual void SetPredicate(const String & rColumnName,
							  const String & rTableRange,
							  SdbPredicateType ePredicateType,
							  const String & rValue,
							  const String & rParameterName);


	// Erweiterung auf UPDATE- und INSERT-Statement ... (nyi):
	void TraverseAssignments();
	virtual void SetAssign(const String & rColumnName,
						   const String & rValue, BOOL bSetNull,
						   const String & rParameterName);

	// Alle "Traverse"-Routinen hintereinander aufrufen. Je nach Statement-Typ:
	// Bei UPDATE und INSERT-Statement nur TraverseTableNames und TraverseAssignments,
	// bei DELETE nur TraverseTableNames und bei SELECT-Statement
	// in der Reihenfolge: TraverseTableNames, TraverseSelectColumnNames,
	// TraverseOrderByColumnNames, TraverseSelectionCriteria.
	// Bricht bei irgendwelchen Fehlern sofort ab und liefert entsprechenden
	// Status.
	void TraverseAll();

	virtual void  SetDatabase(SdbDatabase* pDb);
	// Die TableRangeMap enth"alt alle Tabellen unter dem zugeh"origen Rangenamen der zuerst gefunden wird
	const SdbTableRangeMap& GetSdbTableRangeMap() const { return aTableRangeMap;};
	const SdbColumnsRef GetSelectColumns() const { return aSelectColumns;}
	// gibt den Aliasnamen der Column zur"uck, Leer falls nicht vorhanden
	String GetColumnAlias(const SdbSqlParseNode* pDerivedColumn) const;
	// gibt den Columnnamen und die Tablerange (falls vorhanden) zur"uck
	void GetColumnRange(const SdbSqlParseNode* pColumnRef,String &rColumnName,String &rTableRange) const;

	// Ermittelt fuer eine Funktion, Spalten den zugehoeren TableRange,
	// wenn nicht eindeutig, dann leer
	BOOL GetColumnTableRange(const SdbSqlParseNode* pNode, String &rTableRange) const;

	BOOL TablesAreSensitive() const { return pConnection ? (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) != 0 : FALSE; }

	// FehlerHdl setzen
	void	SetErrorHdl(const Link& rHdl){ 	aErrorHdl = rHdl;};
	const Link&	GetErrorHdl(){	return aErrorHdl;};
};

#endif

