/*************************************************************************
 *
 *  $RCSfile: sdbstat.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBSTAT_HXX
#define _SDB_SDBSTAT_HXX 1

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif

#ifndef _LINK_HXX //autogen
#include <tools/link.hxx>
#endif

//==============================================================================
class SvStorage;
class SvFileStream;
//==============================================================================

enum DBObject
{
	dbTable,
	dbQuery,
	dbForm,
	dbReport,
	dbIndex,
	dbView,
	dbRelation,
	dbField,
	dbWorkspace
};

// Statuscodes:
enum SdbStatusCode
{
	SDB_STAT_SUCCESS = 10,			// keine weiteren Informationen verfuegbar
	SDB_STAT_SUCCESS_WITH_INFO,		// erfolgreich, aber mit Info/Warnung
	SDB_STAT_NO_DATA_FOUND,			// alle Rows der Ergebnismenge wurden abgeholt,
									// keine weiteren Daten verfuegbar. Keine weitere Info.
	SDB_STAT_ERROR,					// Funktionsaufruf gescheitert. Weitere Info vorhanden.
	SDB_STAT_STILL_EXECUTING,		// Eine SQL-Anweisung wird gerade (asynchron) ausgefuehrt.
									// Keine weitere Info vorhanden.
	SDB_STAT_CANCELED				// abgebrochen
};

#if __PRIVATE
enum SdbFileType					// von Sdb verwendete Files bzw ihr DB-Repraesentationen
{
	DATABASE,
	TABLE,
	INDEX,
	QUERY,
	REPORT,
	FORM,
	VIEW,
	RELATION,
	MEMO,							// memodatei, dBase spezifisch
	FIELD,
	WORKSPACE
};
#endif

//==============================================================================
// Class SdbStatus
//
// Beschreibung:
//	Status der letzten (bzw. noch laufenden)
//	Datenbankaktion. Waehrend der Ausfuehrung der Anweisung
//	(SDB_STAT_STILL_EXECUTING) und bei bestimmten Status-Codes
//	(SDB_STAT_SUCCESS, SDB_STAT_NO_DATA_FOUND) sind die zusaetzlichen
//	Informationen (aErrorMessage, aSQLState,
//	nNativeErrorCode) ungueltig.
//
//==============================================================================
class SdbStatus
{
private:
	// Leerstring (wird manchmal fuer return benoetigt):
	// static const String aEmptyString;

	// Ergebnis der letzten Datenbankoperation:
	SdbStatusCode eResult;

	// Optimierung: die uebrigen Member werden nur aktualisiert,
	// wenn Informationen vorliegen (abhaengig vom eResult, siehe
	// IsInformationAvailable). Die Zugriffsfunktionen sorgen aber
	// dafuer, dass keine alten Werte zurueckgeliefert werden.

	// SQLSTATE wie in SQL/92 definiert.
	// Falls nicht verfuegbar (aeltere Datenbank) werden
	// mindestens folgende Statuswerte emuliert:
	// 01000 (General Warning), S1000 (General Error), S1008
	// (Operation Cancelled)
	String aSQLState;

	// Text zur Fehlermeldung (spezifisch fuer das Datenbanksystem).
	String aErrorMessage;

	// Fehlercode der Datenbank (spezifisch fuer das Datenbanksystem).
	INT32 nNativeErrorCode;

	// Zusaetzliche Information (Text) zur Fehlermeldung.
	// Kann z. B. das SQL-Kommando enthalten, das zum Fehler fuehrte.
	String aErrorInfo;

	INT32 nRecordsAffected;	// Anzahl Datensaetze die bei einer Datenbank operation berhrt wurden

	// Registrierte Callback-Funktion
	Link aStatusChangeHdl;

public:
	SdbStatus();

	// Callback-Funktion wird NICHT mitkopiert.
	// Beim Kopieren erfolgt KEIN Callback beim Zielobjekt.
	SdbStatus(const SdbStatus &right);

	const SdbStatus & operator=(const SdbStatus &right);

	BOOL operator==(const SdbStatus &right) const;
	BOOL operator!=(const SdbStatus &right) const {return ! (*this == right);}

	virtual ~SdbStatus();

	// Zugriff auf Elemente (Beschreibung siehe oben):
	SdbStatusCode Result() const {return eResult;}

	inline String SQLState() const;

	inline String ErrorMessage() const;

	INT32 NativeErrorCode() const {return IsInformationAvailable() ? nNativeErrorCode : 0;}

	inline String ErrorInfo() const;

	const Link & StatusChangeHdl() const {return aStatusChangeHdl;}


	// Sind zustzliche Informationen zum Resultat vorhanden
	// (nur bei SUCCESS_WITH_INFO oder ERROR)?
	inline BOOL IsInformationAvailable() const;

	// Ist die letzte Datenbankoperation erfolgreich
	// durchgelaufen (SUCCESS oder SUCCESS_WITH_INFO)?
	inline BOOL IsSuccessful() const;

	// Hat die letzte Datenbankoperation einen Fehler
	// ergeben (ERROR)? WARNING, NO_DATA_FOUND und
	// SUCCESS_WITH_INFO zaehlen nicht als Fehler.
	BOOL IsError() const {return eResult == SDB_STAT_ERROR;}

	// Hat die letzte Datenbankoperation den Status SDB_STAT_NO_DATA_FOUND
	// ergeben?
	BOOL IsNoDataFound() const {return eResult == SDB_STAT_NO_DATA_FOUND;}

	// Ist gerade eine Datenbankoperation im Gange?
	BOOL IsExecuting() const {return eResult == SDB_STAT_STILL_EXECUTING;}

	// wurde abgebrochen ?
	BOOL IsCanceled() const { return eResult == SDB_STAT_CANCELED; }


	// Registrierung eines Links, ueber den der Aufrufer
	// benachrichtigt wird, wenn sich der Status aendert.
	void SetStatusChangeHdl(const Link& rNewLink) {aStatusChangeHdl = rNewLink;}

	// Status der letzten Operation setzen (geschieht nur in
	// den abgeleiteten Implementationsklassen).

	// Durch Setzen des Status gilt das Statement als beendet
	// und es wird ein etwaiger registrierter Callback zur
	// Benachrichtigung ueber den Ausgang des Statements
	// ausgefuehrt. NewStatus darf nicht STILL_EXECUTING sein.

	// Die fehlenden Argumente werden mit Defaults aufgefuellt:
	// - bei eNewResult == SUCCESS || eNewResult == NO_DATA_FOUND: "", "", 0
	// - bei eNewResult == SUCCESS_WITH_INFO: "01000", "General Warning", 0
	// - bei eNewResult == ERROR: "S1000", "General Error", 0
	void Set(SdbStatusCode eNewResult);

	// Bei eNewResult == SUCCESS || eNewResult == NO_DATA_FOUND werden die
	// zusaetzlichen Argumente ignoriert.
	void Set(const SdbStatusCode eNewResult, const String &rNewSQLState,
			 const String &rNewErrorMessage, INT32 nNewNativeErrorCode,
			 const String &rNewErrorInfo );

	// ErrorMessage nach ODBC-Konvention konstruieren:
	// 		"[Star Division][Component][DataSource] ErrorMessageText"
	// oder
	// 		"[Star Division][Component] ErrorMessage"
	// bei fehlender DataSource.
	// Die eckigen Klammern duerfen in den Parameter nicht mit angegeben
	// werden.
	const String CreateErrorMessage(const String & pErrorMessageText,
									const String & pDataSource = String());

	// Haeufig auftretende Fehler:
	// fehlerhafte Reihenfolge der Funktionsaufrufe.
	void SetSequenceError();
	// Funktion vom Treiber nicht unterstuetzt:
	void SetDriverNotCapableError();
	// Syntax/Semantikfehler im Statement:
	void SetInvalidStatement();
	// Statement zu komplex:
	void SetStatementTooComplex();
	// Verbindung unterbrochen
	void SetConnectionLost();

	//
	void SetRecordsAffected(INT32 _nRecords) {nRecordsAffected = _nRecords;}
	INT32 GetRecordsAffected() const {return nRecordsAffected;}

	// Name der Komponente, fuer die der Status gilt	 obsolete
	void SetComponent(const String& rComponent){}

	// Beim Beginn einer neuen Datenbankoperation wird
	// Init() aufgerufen, um die vorher gesetzten
	// Fehlermeldungen zu loeschen und den Status auf
	// STILL_EXECUTING zu setzen. Bei Erfolg kann der Status
	// auch mit Clear() auf SDB_SUCCESS gesetzt werden.
	void Clear() {eResult = SDB_STAT_SUCCESS;}	// Status auf SDB_SUCCESS setzen.
	void Init() {eResult = SDB_STAT_STILL_EXECUTING;}	// Status auf SDB_STILL_EXECUTING setzen.
	void SetError(ULONG nErrorCode, DBObject eObj, const String& aName);

#if __PRIVATE
	// Ermittelt aus einer File Operation den FehlerStatus
	BOOL SetError(ULONG nErrorCode, SdbFileType eType, const String& aName);
	BOOL SetError(const SvFileStream& aFileStream, SdbFileType eType);
	BOOL SetError(const SvStorage& aStorage, SdbFileType eType);
	String TypeToString(SdbFileType eType) const;
#endif
};

//==============================================================================

#if __PRIVATE

#ifndef _RESID_HXX //autogen
#include <vcl/resid.hxx>
#endif

//==============================================================================
// ResourceMgr fuer Sdb, hier die geeignete Stelle fuer def, kein eigenes Header
//==============================================================================
class SdbResId
{
	String	m_sValue;
public:
	SdbResId(USHORT nId);
	operator String () const { return m_sValue; }
};

#endif

// Inline-Implementierungen:

inline BOOL SdbStatus::IsInformationAvailable() const
{
	return eResult == SDB_STAT_ERROR || eResult == SDB_STAT_SUCCESS_WITH_INFO ||
		eResult == SDB_STAT_NO_DATA_FOUND;
}

inline BOOL SdbStatus::IsSuccessful() const
{
	return eResult == SDB_STAT_SUCCESS || eResult == SDB_STAT_SUCCESS_WITH_INFO;
}

// Sicherheitshalber liefern die Funktionen keine alten Werte,
// wenn eigentlich keine Informationen vorliegen. Dieses Verhalten
// ist aber nicht garantiert!
inline String SdbStatus::SQLState() const
{
	if(IsInformationAvailable())
		return aSQLState ;
	else
		return String();
}

inline String SdbStatus::ErrorMessage() const
{
	if(IsInformationAvailable())
		return aErrorMessage;
	else
		return String();
}

inline String SdbStatus::ErrorInfo() const
{
	if(IsInformationAvailable())
		return aErrorInfo ;
	else
		return String();
}

#endif

