/*************************************************************************
 *
 *  $RCSfile: sdbstor.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBSTOR_HXX
#define _SDB_SDBSTOR_HXX 1



#ifndef _SDB_SDBHASH_HXX
#include "sdbhash.hxx"
#endif

#ifndef _SDB_SDBSTAT_HXX
#include "sdbstat.hxx"
#endif

#ifndef _SVSTOR_HXX //autogen
#include <so3/svstor.hxx>
#endif

#ifndef _TABLE_HXX //autogen
#include <tools/table.hxx>
#endif

class SdbDatabaseImpl;

DECLARE_TABLE(SdbStringIndex, ByteString*);

/*---------------------------------------------------------------------------
	Internes Verzeichnis fuer Datenbankstorage, ein einzelner Eintrage besteht
	aus Key und Name
/---------------------------------------------------------------------------*/
class SdbStorageDir : private SdbStringIndex
{
	friend class SdbStorage;

	friend  SvStream& operator << (SvStream &rStream, SdbStorageDir&);
	friend  SvStream& operator >> (SvStream &rStream, SdbStorageDir&);

	BOOL bDirty : 1;	//
	SdbStorage& rStorage;
	SvStorageStream* pStream;

public:
	SdbStringIndex::Count;
	SdbStringIndex::IsKeyValid;
	SdbStringIndex::Seek;
	SdbStringIndex::Get;
	SdbStringIndex::First;
	SdbStringIndex::Next;
	SdbStringIndex::Prev;
	SdbStringIndex::Last;

	SdbStorageDir(SdbStorage&);
	~SdbStorageDir();

	BOOL Insert(ULONG nKey, const String& rValue);
	void Remove(ULONG nKey);
	void Clear();

	ULONG GetKey(const String& rName,BOOL _bTable) const;

protected:
	void Init(SvStorageStream* pStream);
	void Flush();
};

class SdbStorInfo
{
	SvStorage*		pImpl;
	BOOL			bReadOnly;

public:
	SdbStorInfo(SvStorage* pStor, StreamMode = STREAM_STD_READWRITE);
	~SdbStorInfo();

	BOOL IsReadOnly() const {return bReadOnly;}
	SvStorage* Storage() const {return pImpl;}
};

/*---------------------------------------------------------------------------
	SdbStorage ist im Prinzip eine Wrapperklasse fuer den SvStorage, da der SvStorage
	nicht in der Lage ist Verzeichnisnamen > 31 zu verwalten, was aber innerhalb
	der Datenbank durchaus vorkommen kann
/---------------------------------------------------------------------------*/
class SdbStorage : public SvRefBase
{
	friend class SdbDatabaseImpl;
	friend class SdbStorageDir;

	SdbDatabaseImpl& rDatabase;
	SdbStorInfo*	 pImpl;

	SdbStorageDir aDir;			// Verzeichniseintraege

protected:
	SdbStorage(SdbDatabaseImpl& rDB,
			   const String & rName,
			   StreamMode = STREAM_STD_READWRITE,
			   StorageMode = 0);
#if __PRIVATE
	SdbStorage(SdbStorage&,
			   SdbFileType,
			   const String& rName,
			   StreamMode = STREAM_STD_READWRITE,
			   StorageMode = 0 );
#endif

	virtual ~SdbStorage();

public:
	void GetStorageNameList(StringList& rList);	// nicht sortiert
	BOOL Remove(const String & rEleName,BOOL _bTable);

#if __PRIVATE
	long GetVersion() const {return pImpl->Storage()->GetVersion();}
	void SetVersion(long lNewVersion);
		// !! mit Vorsicht zu geniessen : wenn schon unter der alten Versionskennung etwas in den Storage geschrieben wurde,
		// was sich darauf verlaesst, dann koennte das beim Einlesen (da ja dann unter der neuen Kennung) Probleme geben.

	BOOL GetUniqueObjectName(String& strName,BOOL _bTable);
		// strName muss anfaenglich eine (nichtleere) Basis enthalten, der - ebenfalls in strName gelieferte - endgueltige Name
		// besteht dann aus dieser Basis und einer angehaengten Zahl
		// wenn Rueckgabe FALSE, dann existieren schon 2^32 Namen, die mit strName beginnen, dann entweder andere Basis suchen
		// oder einfach sein lassen ....

	// da bei Storages eine Namensuebersetzung erfolgt, ein paar Funktionen, die die Arbeit mit den unuebersetzen Namen erlauben
	BOOL HasStorage(const String& strName,BOOL _bTable) const;
	BOOL MoveStorage(const String& strSourceName, SdbStorage& aDest, const String& strDestName,BOOL _bTable);
	BOOL RemoveStorage(const String& strName,BOOL _bTable);
	// ausser dem Dir-Stream noch irgendwelche anderen ?
	BOOL HasStreams() const;
	BOOL HasStorages() const;

private:
	SvStorage* Impl() const {return pImpl->Storage();}
	SvStorageStream* OpenStream(SdbFileType eContext,
								const String & rEleName,
								StreamMode  = STREAM_STD_READWRITE);
	SdbStorage*	OpenStorage(SdbFileType eContext,
							const String & rEleName,
							StreamMode = STREAM_STD_READWRITE,
							StorageMode = STORAGE_TRANSACTED);

	BOOL Commit();
	BOOL Revert() {return Impl()->Revert();}
	BOOL Rename(SdbFileType eContext,
				const String & rEleName,
				const String & rNewName, BOOL bStor);
	BOOL Remove(SdbFileType eContext,
				const String & rEleName, BOOL bStor);

	BOOL GetStorageName(String& rName, BOOL bFirst);
	void GetStorageNameList(SdbHashStringList& rList,BOOL _bCaseSensitiv=TRUE);
	void GetStreamNameList(SdbHashStringList& rList);


	BOOL IsStorage(const String& rName) const {return Impl()->IsStorage(rName);}
	void ResetError() {Impl()->ResetError();}

	BOOL IsReadOnly() const {return pImpl->IsReadOnly();}
	SdbStatus& Status() const;
#endif
};

#if __PRIVATE
#define TITLE TITLE_0
#define TITLE_0 "StarBaseDocument"
#endif


SV_DECL_IMPL_REF(SdbStorage);

#endif


