/*************************************************************************
 *
 *  $RCSfile: sdbtab.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBTAB_HXX
#define _SDB_SDBTAB_HXX 1

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBCOL_HXX
#include "sdbcol.hxx"
#endif

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SVSTDARR_STRINGSDTOR
#define _SVSTDARR_STRINGSDTOR
#include <svtools/svstdarr.hxx>
#endif

class SdbConnection;
class SvStorage;
class SvStorageRef;
struct SdbTypeInfo;

//==============================================================================
// Allgemeine Klasse zum Lesen und Schreiben von Tabellenstrukturen
//	Zugriff auf die Strukturbeschreibung einer Tabelle
//	und deren Bearbeitung.
//==============================================================================
class SdbTable : public SdbObj
{
	friend class SdbIndex;
	friend class SdbView;
	friend class SdbDatabase;
	friend class SdbConnection;

	SvStringsDtor	aIndexNameList; // Liste aller verfuegbaren Indices
public:
	TYPEINFO();

	// Zugriff auf den Vater
	virtual SdbConnection* GetConnection() const;

	// Methoden um Schemaaenderungen durchzufuehren
	// Schreiben von Aenderungen in die Datenbank
	virtual BOOL Update();
	virtual BOOL Rename(const String& aNewName);
	virtual BOOL Drop();

	// Arbeiten mit Indices
	// Liste aller Indices
	virtual String GetIndexName(USHORT _nPos);
	virtual USHORT GetIndexCount();

	// Eine neue Indexstruktur erzeugen
	virtual SdbIndex* CreateIndex(const String& rName);
	virtual SdbIndex* OpenIndex(const String& rName);

	UINT16 GetTablePrivileges() const {return nPrivileges;}

	// ein Tabellenstruktur die eigentlich eine View repraesentiert
	BOOL IsView() const {return bView;}

	// Cursor auf der Tabelle oeffnen
	virtual SdbCursor* OpenCursor(SdbCursorType eType, UINT16 nOptions);

	// Die Struktur der Tabelle (Columns).
	// Ist NULL, wenn die Tabellenstruktur nicht geoeffnet ist.
	const SdbColumnsRef& Columns() {return aColumns;}

	// Die urspruengliche Struktur (beim Open)
	// vorsicht, da SdbColumns RefKlasse, wird kein const SdbColumns* uebergeben
	const SdbColumnsRef& OriginalColumns() const {return aOriginalColumns;}

	// Anlegen von Feldern
	virtual SdbColumn* CreateColumn(const String& _rName,
									 SdbDatabaseType _eType,
									 UINT32 _nLength = 0, UINT16 _nScale = 0,
									 UINT16 _nFlags = 0);
	virtual SdbColumn* CreateColumn(const SdbColumn& _rColumn);

	// Der Name der Tabelle etc.
	const String& Name() const {return aName;}
	const String& OwnerName() const {return aOwnerName;}
	const String& QualifierName() const {return aQualifierName;}
	virtual String GetFullName();

	static String GetFullName(SdbDatabase& rDB, const String& rName, const String& rDBName = String());
	static String GetFullName(SdbConnection& rConnection, const String& rName, const String& rDBName = String());

	// ueberpruefen, ob eine Column eine gueltigen Datentyp besitzt
/*
	const SdbTypeInfo* GetColumnTypeInfo(const SdbColumn& rColumn);
	const SdbTypeInfo* GetColumnTypeInfo(const String& rTypeName);
	const SdbTypeInfo* GetColumnTypeInfo(const SdbDatabaseType eType);
*/
protected:
	// Implementierung von Methoden die vom SdbObj erwartet werden
	virtual BOOL Open();
	virtual void Close();

	virtual void RemoveChildren();
	virtual void RemoveChild(SdbObj* pObj);

	// Tabellenbeschreibungen gelten als gleich, wenn der Tabellenname und
	// die zugrundeliegenden Struktur gleich sind.
	virtual BOOL Compare(const SdbObj&) const;

	// Oeffnen der Tabellenbeschreibung. Nach dem Oeffnen steht die
	// Columns-Beschreibung zum Bearbeiten zur Verfuegung.
	// Liefert SDB_NO_DATA_FOUND, wenn die Tabelle nicht existiert
	// SDB_NO_DATA_FOUND gilt uebrigens weder als IsSuccessful() noch als IsError()!!!
	virtual BOOL Open(const String& rTableName);

	// Neue Tabelle anlegen (zunaechst nur die Tabellenbeschreibung im
	// Hauptspeicher, ohne Namen, erst beim SaveAs wird der Name festgelegt
	// und die betreffende Tabelle angelegt).
	// Nach Create ist die Tabellenbeschreibung geoeffnet.
	virtual BOOL Create(const String& rTableName);

	// Methoden, die vom Treiber zu realisieren sind
	// SdbTablePriviliges mu nach dem Create gesetzt werden
	virtual BOOL CreateImpl() = 0;
	virtual BOOL AlterImpl() = 0;
	virtual BOOL RenameImpl(const String& rNewName) = 0;
	virtual BOOL DropImpl() = 0;
	virtual BOOL OpenImpl() = 0;
	virtual void CloseImpl() = 0;

	// aktualisieren
	virtual BOOL DoRefresh();

	virtual SdbIndex* CreateIndexImpl() = 0;
	virtual void FillIndexNames(SvStringsDtor& _rList) = 0;

	// Ermitteln von Tabellennamen, Qualifier, Owner
	virtual void SetNames(const String& _rName, const String& _rQualifier, SdbConnection* pConnection);

protected:
	// Name der Tabelle:
	String aQualifierName;
	String aOwnerName;
	String aName;
	BOOL   bView	: 1;
	UINT16 nPrivileges;

	// Die eigentliche Tabellenstruktur
	SdbColumnsRef aColumns;

	// Die urspruengliche Tabellenstruktur (beim Open)
	SdbColumnsRef aOriginalColumns;

#if (SUPD >= 375)
	void*  pVOID;			// free for later use
	UINT32 nFree;
#endif

	// Konstruktoren nur aus SdbConnection heraus benutzbar
	SdbTable(SdbConnection* pConnection);
	~SdbTable();

private:
	SdbDatabase* pDatabase;	   // Verbindung zu einer Datenbank, muss nicht gesetzt sein

protected:
	const SdbDatabase* GetDatabase() const { return pDatabase; }

private:
	DECLARE_SDBKEYCONT(SdbIndexList, SdbIndex *)
	SdbIndexList aIndexList;

#if __PRIVATE
	private:
		void Set(SdbDatabase* pDB) {pDatabase = pDB;};	// einklinken einer Datenbank

	public:
	String QualifiedName();  // Voll Qualifiziert mit event. Pfadangaben
#endif
};

SV_DECL_IMPL_REF(SdbTable);

#endif


