/*************************************************************************
 *
 *  $RCSfile: sdbwork.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBWORK_HXX
#define _SDB_SDBWORK_HXX

class SdbDatabase;

#ifndef _SDB_SDBCONT_HXX
#include "sdbcont.hxx"
#endif

DECLARE_SDBFILECONT(SdbDatabaseList, SdbDatabase*);

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#define	DEFAULT_WORKSPACENAME	"DefaultWorkspace"

class SdbWorkspace : public SdbObj
{
	friend class SdbDatabaseImpl;
	friend class SdbDatabase;
	friend class SdbEnvironment;

	SdbDatabaseList	aDBList;	// Liste der geoeffneten Datenbanken
	String			aName;

private:
	SdbWorkspace(SdbEnvironment*, const String& rName);

protected:
	virtual ~SdbWorkspace();

public:
	TYPEINFO();

	// Zugriff auf Vater
	virtual SdbEnvironment* GetEnvironment() const;
	virtual BOOL Rename(const String& rName);

	const String& GetName() const {return aName;}

	/* datenbank handling */
	// Oeffnen einer Datenbank (Verbindungsaufbau)
	SdbDatabase* OpenDatabase(const String& rName);

	// liefert eine schon geoeffnete DB zurueck, NULL, wenn in diesem Workspace
	// keine geoeffnete DB dieses Namens existiert
	SdbDatabase* GetDatabaseNoOpen(const String& rName);

	// Erzeugen einer Datenbankdatei
	// liefert das Object nicht zurck, um nichtunnoetig eine Verbindung zu einer Daten-
	// bank aufzubauen
	BOOL CreateDatabase(const String& rName, const String& rTitle, const String& rConnectStr);

	// Erzeugen einer neuen Datenbank, also z.B. einer konkreten
	// Access-Datenbank
	BOOL CreateNativeDatabase(const String& rOptionString) const;

	// Loeschen einer Datenbank, also z.B. einer konkreten
	// Access-Datenbank
	BOOL DropNativeDatabase(const String& rOptionString) const;

	// Loeschen einer Datenbank (der Datenbandatei (sdb))
	BOOL DropDatabase(const String& rName);

	// Auslesen des Verbindungsstrings
	String GetDatabaseConnectStr(const String& rName);

	// Aendern einer Datenbankverbindung
	BOOL AlterDatabase(const String& rName, const String& rConnectStr);
	BOOL AlterDatabaseName(const String& aOldName, const String& aNewName);
	BOOL AlterDatabaseTitle(const String& aNewName, const String& aTitle);

protected:
	virtual void RemoveChild(SdbObj* pObj);
	virtual void RemoveChildren();
	virtual BOOL Open();

	SdbDatabase* LoadDatabase(const String& rName);

private:
	SdbDatabase* GetDatabase(const String& rName);
	SdbDatabase* CreateDB();
	String GetFileName(const String& rFile) const;
};

SV_DECL_IMPL_REF(SdbWorkspace);

#endif


