/*************************************************************************
 *
 *  $RCSfile: oterconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbmap.hxx"
#include "oterconn.hxx"
#include "otercurs.hxx"
#include "otertab.hxx"
#include "sdbresid.hrc"
#ifndef _SDB_SDBREL_HXX
#include <sdbrel.hxx>
#endif

#include "oter.hrc"

#ifndef _SDB_SDBENV_HXX
#include "sdbenv.hxx"
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _SDB_OTERMD_HXX
#include <otermd.hxx>
#endif // _SDB_OTERMD_HXX

#include <tools/rc.hxx>

#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif

#ifndef _SFXINIPROP_HXX //autogen
#include <svtools/iniprop.hxx>
#endif

#ifndef _TOOLS_SIMPLERESMGR_HXX_
#include <tools/simplerm.hxx>
#endif

DBG_NAME(SdbOTERConnection);

#ifndef _COM_SUN_STAR_DATA_DATABASENKEYRULE_HPP_
#include <com/sun/star/data/DatabaseKeyRule.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;


static char __READONLY_DATA ALL[] = "%";
//-----------------------------------------------------------------------------
DBG_NAME(SdbOTERHandle);
SdbOTERHandle::SdbOTERHandle() : aOTERHandle(SQL_NULL_HENV)
{
	DBG_CTOR(SdbOTERHandle,NULL);
	if (LoadLibrary_OTER(aPath))
	{
		// Environment allozieren
		RETCODE nRetCode = OSQLAllocEnv(&aOTERHandle);
		DBG_ASSERT(nRetCode == 0,"Error in alloac env!");
	}
}
//-----------------------------------------------------------------------------
SdbOTERHandle::~SdbOTERHandle()
{
	DBG_DTOR(SdbOTERHandle,NULL);
	OSQLFreeEnv(aOTERHandle);
	FreeLibrary_OTER();
}
//-----------------------------------------------------------------------------
void SdbOTERHandle::Alloc()
{
}

//-----------------------------------------------------------------------------
HENV SdbOTERConnection::EnvironmentHandle()
{
	String aPath;
	return EnvironmentHandle(GetEnvironment(),aPath);
}
//-----------------------------------------------------------------------------
// OTER Environment (gemeinsam fuer alle Connections):
SdbOTERHandleRef SdbOTERConnection::EnvironmentHandle(SdbEnvironment* pEnv,String &_rPath)
{
	// Ist (fuer diese Instanz) bereits ein Environment erzeugt worden?
	if (pEnv && !pEnv->pOterroHandle->Is())
	{
		// In globaler Struktur merken ...
		(*pEnv->pOterroHandle) = new SdbOTERHandle();
		_rPath = (*pEnv->pOterroHandle)->aPath;

	}

	return pEnv ? (*pEnv->pOterroHandle) : SdbOTERHandleRef();
}
//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::IsMultiUserOn() const { return aOptionsParser.OptionBool(String::CreateFromAscii("multiuser"));};
BOOL SdbOTERConnection::IsTransactOn()  const { return aOptionsParser.OptionBool(String::CreateFromAscii("transact"));};
//-----------------------------------------------------------------------------
void SdbOTERConnection::Construct(SdbEnvironment* pEnv)
{
	// Environment Handle besorgen (einmal beim Anlegen des Objektes -
	// wird aus Effizienzgrunden in jedem OTERConnection-Objekt gespeichert)
	String aPath;
	aEnvironmentHandle = EnvironmentHandle(pEnv,aPath);
	aConnectionHandle  = SQL_NULL_HDBC;
	if (!aEnvironmentHandle.Is())
	{
		String aText(SdbResId(STR_STAT_OTER_NOT_FOUND));
		aText.SearchAndReplace(String::CreateFromAscii("#DLL#"),aPath);
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
		return;
	}
	else if(aEnvironmentHandle->GetHandle() == SQL_NULL_HENV)
	{
		String aText(SdbResId(STR_STAT_OTER_NOT_FOUND));
		if(!aPath.Len())
			aPath = aEnvironmentHandle->aPath;
		aPath.SearchAndReplace(String::CreateFromAscii("OT10_32.DLL"),String::CreateFromAscii("ot101.msg"));
		aText.SearchAndReplace(String::CreateFromAscii("#DLL#"),aPath);
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
		return;
	}

	bReadOnly = TRUE;
	aStatus.SetComponent(String::CreateFromAscii("StarBase Driver"));

	// Wenn kein Default-Schemaname vorhanden, den Benutzernamen verwenden.
	if (aUserName.Len() == 0 && aOptionsParser.OptionString(String::CreateFromAscii("UID")).Len() > 0)
		aUserName = aOptionsParser.OptionString(String::CreateFromAscii("UID"));
}

//-----------------------------------------------------------------------------
SdbOTERConnection::SdbOTERConnection(SdbEnvironment* pEnv, const String& rNewOptions)
				  :SdbConnection(pEnv, rNewOptions)
{
	DBG_CTOR(SdbOTERConnection,NULL);
	Construct(pEnv);
}

//-----------------------------------------------------------------------------
// Copy CTOR nur fuer den internen Gebrauch
SdbOTERConnection::SdbOTERConnection(const SdbOTERConnection& rConn)
				  :SdbConnection(rConn.GetEnvironment(),rConn.Options())
{
	DBG_CTOR(SdbOTERConnection,NULL);
	Construct(rConn.GetEnvironment());
}

//-----------------------------------------------------------------------------
SdbOTERConnection::~SdbOTERConnection()
{

	DBG_DTOR(SdbOTERConnection,NULL);
	DBG_ASSERT(aConnectionHandle == SQL_NULL_HDBC,"SdbOTERConnection: interner Fehler: ConnectionHandle ungueltig");
}
//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::GetOTERInfo(UWORD nInfoType, String & rString) const
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	char aValue[512];
	SWORD nValueLen;
	RETCODE nRetCode(OSQLGetInfo(aConnectionHandle,nInfoType,aValue,sizeof aValue,&nValueLen));

	if (nRetCode == SQL_ERROR)
	{
		DBG_ERROR("GetOTERInfo -> OTERInfo not available");
	}
	else
		rString.AssignAscii(aValue);

	return nRetCode == SQL_SUCCESS;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::GetOTERInfo(UWORD nInfoType, USHORT & rNumber) const
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	UWORD nValue;
	SWORD nValueLen;
	RETCODE nRetCode =
		OSQLGetInfo(aConnectionHandle,nInfoType,&nValue,sizeof nValue,&nValueLen);
	if (nRetCode == SQL_ERROR)
	{
		DBG_ERROR("GetOTERInfo -> OTERInfo not available");
	}
	else
		rNumber = (USHORT)nValue;
	return nRetCode == SQL_SUCCESS;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::GetOTERInfo(UWORD nInfoType, ULONG& rNumber) const
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	ULONG nValue;
	SWORD nValueLen;
	RETCODE nRetCode =
		OSQLGetInfo(aConnectionHandle,nInfoType,&nValue,sizeof nValue,&nValueLen);
	if (nRetCode == SQL_ERROR)
	{
		DBG_ERROR("GetOTERInfo -> OTERInfo not available");
	}
	else
		rNumber = (ULONG)nValue;
	return nRetCode == SQL_SUCCESS;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::GetOTERInfo(UWORD nInfoType, BOOL &rBool) const
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	String aString;
	BOOL bResult = GetOTERInfo(nInfoType,aString);
	if (bResult)
		rBool = (aString == String('Y'));
	return bResult;
}

//-----------------------------------------------------------------------------
void SdbOTERConnection::CheckOTERCapabilities()
{
	// steht die Info nicht zur Verfuegung -> !readonly
	BOOL bTest = FALSE;
	if (GetOTERInfo(SQL_DATA_SOURCE_READ_ONLY,bTest))
		bReadOnly = bTest;
	// IdentifierQuotation
	String aIdentifierQuotation;
	GetOTERInfo(SQL_IDENTIFIER_QUOTE_CHAR,aIdentifierQuotation); // hack
	aIdentifierQuotation.EraseLeadingChars(' ');
	cIdentifierQuotation = aIdentifierQuotation.GetChar(0);
}

//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::OpenImpl()
{
	String aOptions(ConnectStr());

	// Verbindung aufbauen
	RETCODE nRetCode(OpenConnection(aOptionsParser.OptionString(String::CreateFromAscii("DSN")),
		aOptionsParser.OptionString(String::CreateFromAscii("UID")),
		aOptionsParser.OptionString(String::CreateFromAscii("PWD")),GetEnvironment()->GetLoginTimeOut()));
	if (nRetCode == SQL_ERROR || nRetCode == SQL_INVALID_HANDLE)
	{
		if (aStatus.IsSuccessful())
			// das OpenConnection hat nicht schon selber einen Status gesetzt
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("01000"), aStatus.CreateErrorMessage(String(SdbResId(STARBASE_CONN_FAILED))),
					0, String() );
	}
	else if (nRetCode == SQL_NO_DATA_FOUND)
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);

	if (!aStatus.IsSuccessful())
		return FALSE;

	CheckOTERCapabilities();
	if (!aStatus.IsSuccessful())
		return FALSE;

	// name des User in der Datenbank, kann unterschiedlich zum login name sein
	GetOTERInfo(SQL_USER_NAME,aUserName);

	eIdentifierCase = SDB_IC_MIXED;
// den aktuellen Datenbanknamen ermitteln
	char cCurrentQualifier[SQL_MAX_OPTION_STRING_LENGTH + 1] = "";
	GetOTERInfo(SQL_DATABASE_NAME,aQualifierName);

	aStatus.Clear();        //Ergebnis ignorieren
	pTableNames = new SdbStringStlMap(::utl::UStringMixLess(FALSE));
	return TRUE;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::CloseImpl()
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	DBG_ASSERT(IsOpen(),"SdbOTERConnection: interner Fehler: Verbindung nicht geoeffnet");
	DBG_ASSERT(aEnvironmentHandle->GetHandle() != SQL_NULL_HENV,"SdbOTERConnection: interner Fehler: EnvironmentHandle ungueltig");
	DBG_ASSERT(aConnectionHandle != SQL_NULL_HDBC,"SdbOTERConnection: interner Fehler: ConnectionHandle ungueltig");

	SetStatus(OSQLSetConnectOption(aConnectionHandle,SQL_AUTOCOMMIT,SQL_AUTOCOMMIT_ON),aConnectionHandle);
	SetStatus(OSQLDisconnect(aConnectionHandle),aConnectionHandle);
	if (aConnectionHandle != SQL_NULL_HDBC)
		SetStatus(OSQLFreeConnect(aConnectionHandle),aConnectionHandle);
	aConnectionHandle = SQL_NULL_HDBC;

	aEnvironmentHandle = NULL;
	BOOL bIS = GetEnvironment()->pOterroHandle && GetEnvironment()->pOterroHandle->Is();
	if( bIS && (*GetEnvironment()->pOterroHandle)->GetRefCount() == 1)
		(*GetEnvironment()->pOterroHandle) = NULL;

	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
void SdbOTERConnection::SetAutoCommit(BOOL bAuto)
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	// Oterro ignoriert das Umschalten, es ist abhaengig von dem Transactionsmodus
	bAutoCommit = bAuto;
}

//-----------------------------------------------------------------------------
void SdbOTERConnection::CommitImpl()
{
	DBG_ASSERT(aConnectionHandle != SQL_NULL_HDBC,"SdbOTERConnection: interner Fehler: ConnectionHandle ungueltig");
	// alle Cursor Committen
	if (IsTransactOn())
	{
		const SdbCursorList aList = GetCursorList();
		for(USHORT i=0;i<aList.Count();i++)
		{
			SdbCursor* pCursor = aList.GetObject(i);
			pCursor->DetermineRowCount();
		}
		SetStatus(OSQLTransact(SQL_NULL_HENV,aConnectionHandle,SQL_COMMIT), aConnectionHandle);
	}
}

//-----------------------------------------------------------------------------
void SdbOTERConnection::RollbackImpl()
{
	DBG_ASSERT(aConnectionHandle != SQL_NULL_HDBC,"SdbOTERConnection: interner Fehler: ConnectionHandle ungueltig");
	if (IsTransactOn())
	{
		const SdbCursorList aList = GetCursorList();
		for(USHORT i=0;i<aList.Count();i++)
		{
			SdbCursor* pCursor = aList.GetObject(i);
			pCursor->DetermineRowCount();
		}
		SetStatus(OSQLTransact(SQL_NULL_HENV,aConnectionHandle,SQL_ROLLBACK),aConnectionHandle);
	}
}

//-----------------------------------------------------------------------------
SdbCursor* SdbOTERConnection::CreateCursorImpl(SdbCursorType eType,UINT16 nOptions)
{
	DBG_ASSERT(aEnvironmentHandle->GetHandle() != SQL_NULL_HENV,"SdbOTERConnection: interner Fehler: EnvironmentHandle ungueltig");
	DBG_ASSERT(aConnectionHandle != SQL_NULL_HDBC,"SdbOTERConnection: interner Fehler: ConnectionHandle ungueltig");

	SdbOTERCursor* pCursor = new SdbOTERCursor(this, eType, nOptions);
	return pCursor;
}

//-----------------------------------------------------------------------------
SdbTable* SdbOTERConnection::CreateTableImpl()
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	DBG_ASSERT(aEnvironmentHandle->GetHandle() != SQL_NULL_HENV,"SdbOTERConnection: interner Fehler: EnvironmentHandle ungueltig");
	DBG_ASSERT(aConnectionHandle != SQL_NULL_HDBC,"SdbOTERConnection: interner Fehler: ConnectionHandle ungueltig");

	return new SdbOTERTable(this);
}


//-----------------------------------------------------------------------------
String SdbOTERConnection::ServerNames(SdbStatus& rStat, SdbEnvironment* pEnv)
{
	return String();
}
//==================================================================
// Ableitung nur fr das Erzeugen und Refresh, fuer eigenes file
// zu geringfgig
//==================================================================
class SdbOTERRelation : public SdbRelation
{
	friend class SdbOTERConnection;
	BOOL GenerateColumnCommand(String& rStatement, const SdbColumn& rColumn);
protected:
	SdbOTERRelation(SdbConnection* _pConn, const String& _rName)
		:SdbRelation(_pConn, _rName){}

	virtual BOOL DoRefresh();
	void SetName(const String& _rName) {aName = _rName;}
public:
	virtual BOOL Update();	// Erzeugen einer Relation in der Datenbank
	virtual BOOL Drop();
};
//------------------------------------------------------------------
BOOL SdbOTERRelation::Drop()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;

//	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
//	if(((SdbOTERConnection*)pConnection)->IsTransactOn() && ((SdbOTERConnection*)pConnection)->IsMultiUserOn())
//		return FALSE;
//
//	aStatus.Clear();
//	BOOL bResult = FALSE;
//
//	if (!bNew){
//		String aCommand("ALTER TABLE ");
//		aCommand += pConnection->QuoteTableName(GetTableName());
//		aCommand += " DROP CONSTRAINT ";
//		aCommand += aName;
//
//		if (!pConnection->ExecuteSQL(aCommand))
//			aStatus = pConnection->Status();
//
//		bResult = aStatus.IsSuccessful();
//	}else
//		bResult = TRUE;
//
//	if (bResult){	// beim Vater abmelden
//		((SdbOTERConnection*)pConnection)->RemoveChild(this);
//		bNew = bOpen = FALSE;
//	}
//
//	return bResult;
}
//-----------------------------------------------------------------------------
BOOL SdbOTERRelation::GenerateColumnCommand(String& rStatement, const SdbColumn& rColumn)
{
//	SdbConnection* pConnection = GetConnection();
//	// Gibt es zu dieser Column eine Typinformationen
//	const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rColumn);
//
//	if (!pInfo){
//		aStatus.SetDriverNotCapableError();
//		return FALSE;
//	}
//
//	rStatement += pConnection->IdentifierQuotation(rColumn.GetName());
//	rStatement += ' ';
//	rStatement += pInfo->aTypeName;
//
//	switch(pInfo->eType)
//	{
//		case SDB_DBTYPE_CHAR:
//		case SDB_DBTYPE_VARCHAR:
//		case SDB_DBTYPE_BINARY:
//		case SDB_DBTYPE_VARBINARY:
//			// Typen mit Laengenangabe
//			rStatement += '(';
//			rStatement += rColumn.GetLength();
//			rStatement += ')';
//			break;
//		case SDB_DBTYPE_DECIMAL:
//		case SDB_DBTYPE_NUMERIC:
//			// Typen mit Precision und Scale
//			rStatement += '(';
//			rStatement += rColumn.GetPrecision();
//			rStatement += ',';
//			rStatement += rColumn.GetScale();
//			rStatement += ')';
//			break;
//	}
//
//	if (!rColumn.IsNullAllowed())
//		rStatement += " NOT NULL";
//
	return TRUE;
}
//------------------------------------------------------------------
BOOL SdbOTERRelation::Update()
{	// Erzeugen einer Relation in der Datenbank
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;

//	if(((SdbOTERConnection*)pConnection)->IsTransactOn() && ((SdbOTERConnection*)pConnection)->IsMultiUserOn())
//		return FALSE;
//	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
//	aStatus.Clear();
//	BOOL bResult(FALSE);
//
//	if (!bNew){ // nur neu anlegen mglich
//		aStatus.SetSequenceError();
//	}else{
//		String aCommand("ALTER TABLE ");
//		aCommand += pConnection->QuoteTableName(GetTableName());
//
//		SdbColumnsRef rColumns(GetColumns());
//		aCommand += " ADD FOREIGN KEY (";
//		for (USHORT i = 1; i < rColumns->Count(); i++)
//		{
//			aCommand += rColumns->Column(i)->GetName();
//			aCommand += ',';
//		}
//		aCommand[USHORT(aCommand.Len()-1)] = ')';
//		aCommand += " REFERENCES ";
//		aCommand += pConnection->QuoteTableName(GetForeignTableName());
//
//		// update rules
//		UINT16 nAttr(GetUpdateRules());
//		if (nAttr & ::stardata::DatabaseKeyRule::CASCADE)
//			aCommand += " ON UPDATE CASCADE ";
//		else if (nAttr & ::stardata::DatabaseKeyRule::SET_NULL)
//			aCommand += " ON UPDATE SET NULL ";
//		else if (nAttr & ::stardata::DatabaseKeyRule::SET_DEFAULT)
//			aCommand += " ON UPDATE DEFAULT ";
//
//		// delete rules
//		nAttr = GetDeleteRules();
//		if (nAttr & ::stardata::DatabaseKeyRule::CASCADE)
//			aCommand += " ON DELETE CASCADE ";
//		else if (nAttr & ::stardata::DatabaseKeyRule::SET_NULL)
//			aCommand += " ON DELETE SET NULL ";
//		else if (nAttr & ::stardata::DatabaseKeyRule::SET_DEFAULT)
//			aCommand += " ON DELETE DEFAULT ";
//
//		if (!pConnection->ExecuteSQL(aCommand))
//			aStatus = pConnection->Status();
//
//		bResult = aStatus.IsSuccessful();
//		if (bResult)
//		{
//			bNew = FALSE;
//			if (!Refresh())		// temp Name behalten
//				aStatus.Clear();
//		}
//	}
//	return bResult;
}
//------------------------------------------------------------------
BOOL SdbOTERRelation::DoRefresh()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	// den Namen merken, da sich dieser durchaus geaendert haben kann
	if (!((SdbOTERConnection*)pConnection)->FillRelation(*this))
		aStatus = pConnection->Status();

	return aStatus.IsSuccessful();
}


//==================================================================
SdbRelation* SdbOTERConnection::CreateRelationImpl(const String& _rName)
{
	return new SdbOTERRelation(this,_rName);
}

//------------------------------------------------------------------
BOOL SdbOTERConnection::FillRelation(SdbOTERRelation& rRel)
{
	// aktualisieren einer Relation
	SDB_CHK_OPEN_BOOL();

	SdbTableRef xTable(OpenTable(rRel.GetTableName())),
				xForeignTable;
	if (xTable.Is())
		xForeignTable = OpenTable(rRel.GetForeignTableName());
	if (xForeignTable.Is())
	{
		SdbCursor* pCursor = OpenRelationCursor(xTable->Name());
		if (!pCursor)
			aStatus.SetError(ERRCODE_IO_NOTEXISTS,RELATION,rRel.GetName());
		else
		{
			pCursor->AddRef();
			// werden innerhalb der While-Schleife ben"otigt
			SdbTableRef xRefTable;
			String aRefTableName;
			String aConstraintName;
			while (!pCursor->IsOffRange())
			{
				// Auswerten der KeySeq
				ODbRow& rRow = *pCursor->GetRow();
				ODbVariantRef xVar = rRow[9];
				if (xVar->isNull())							// ohne Name geht nichts
					break;

				if (xVar->toINT16() == 1)					// gibt es eine neue Relation
				{
					if (xRefTable.Is())                     // Relation bereits ausgelesen
						break;

					xVar = rRow[12];
					if (xVar->isNull())
						aConstraintName = SdbRelation::CreateTempName();
					else
						aConstraintName = xVar->getString().getStr();

					xVar = rRow[3];
					if (!xVar->isNull())
						aRefTableName = xVar->getString().getStr();

					xRefTable = OpenTable(aRefTableName);
					// Tabelle gefunden
					if (&xRefTable == &xForeignTable)
					{
						rRel.SetName(aConstraintName);
						// jetzt noch die Attributierungen
						UINT16 nAttributes = 0, nOTERAttributes = 0;
						// Update rules
						xVar = rRow[10];
						if (!xVar->isNull())
						{
							nOTERAttributes = xVar->toINT16();
							if (nOTERAttributes & SQL_CASCADE)
								nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
							else if (nOTERAttributes & SQL_RESTRICT)
								nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
							else if (nOTERAttributes & SQL_SET_NULL)
								nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
							else
								nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
							rRel.SetUpdateRules(nAttributes);
						}
						// Delete rules
						xVar = rRow[11];
						if (!xVar->isNull())
						{
							nAttributes = 0;
							nOTERAttributes = xVar->toINT16();
							if (nOTERAttributes & SQL_CASCADE)
								nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
							else if (nOTERAttributes & SQL_RESTRICT)
								nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
							else if (nOTERAttributes & SQL_SET_NULL)
								nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
							else
								nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
							rRel.SetDeleteRules(nAttributes);
						}

					}
					else
						xRefTable = NULL;
				}

				if (xRefTable.Is())
				{
					const SdbColumn* pColumn = NULL;
					xVar = rRow[8];
					if (!xVar->isNull())
						pColumn = xTable->OriginalColumns()->Column(xVar->getString().getStr());

					String aForeignName;
					xVar = rRow[4];
					if (!xVar->isNull())
						aForeignName = xVar->getString().getStr();

					if (pColumn && xRefTable->OriginalColumns()->Column(aForeignName))
					{
						SdbColumnRelation* pRelColumn = rRel.CreateColumn(*pColumn, aForeignName);
						rRel.GetColumns()->AddColumn(pRelColumn);
					}
					else
					{
						// falsche struktur
						aStatus.Set(SDB_STAT_ERROR);
						break;
					}
				}
				pCursor->Next();
			}
			pCursor->ReleaseRef();
		}
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
SdbCursor* SdbOTERConnection::OpenRelationCursor(const String &aTableName)
{
	SdbOTERCursor* pCursor = (SdbOTERCursor *) CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);

	pCursor->OpenForeignKeys(NULL,NULL, NULL,NULL,NULL,aTableName.GetBuffer());
	// Spaltennamen
/*              1       PKTABLE_CAT
		2       PKTABLE_SCHEM
		3       PKTABLE_NAME
		4       PKCOLUMN_NAME
		5       FKTABLE_CAT
		6       FKTABLE_SCHEM
		7       FKTABLE_NAME
		8       FKCOLUMN_NAME
		9       KEY_SEQ
		10      UPDATE_RULE
		11      DELETE_RULE
		12      FK_NAME                 // constraint name
		13  PK_NAME
		14  DEFERRABILITY
*/
	if (pCursor->IsOffRange())
	{
		pCursor->AddRef();
		pCursor->ReleaseRef();
		pCursor = NULL;
	}
	return pCursor;
}
//------------------------------------------------------------------
void SdbOTERConnection::FillRelations(SdbRelationList& _rList, const String& aTableName)
{
	SdbCursor* pCursor = OpenRelationCursor(aTableName);
	if (!pCursor)
		return;

	pCursor->AddRef();

	SdbRelationRef xRelation;
	SdbTableRef    xRefTable,xRefFKTable;
	if(aTableName.Len())
		xRefFKTable = OpenTable(aTableName);
	// werden innerhalb der while-schleife benutzt
	String aConstraintName;
	String aRefTableName;
	String aFKTableName;
	String aForeignName;

	while (!pCursor->IsOffRange())
	{
		// Auswerten der KeySeq
		ODbRow& rRow = *pCursor->GetRow();
		ODbVariantRef xVar = rRow[9];
		if (xVar->isNull())
			break;

		if (xVar->toINT16() == 1)    // gibt es eine neue Relation
		{
			// alte uebernehmen
			if (xRelation.Is())
			{
				((SdbOTERRelation*)&xRelation)->Open(); // nur das open flag setzen
				_rList.InsertAt(xRelation->GetName(), &xRelation);
			}


			xVar = rRow[12];
			if (xVar->isNull())
				aConstraintName = SdbRelation::CreateTempName();
			else
				aConstraintName = xVar->getString().getStr();


			xVar = rRow[3];
			if (!xVar->isNull())
			{
				aRefTableName = xVar->getString().getStr();
				if (!aTableName.Len())
				{
					xVar = rRow[7];
					if (!xVar->isNull())
						aFKTableName = xVar->getString().getStr();
				}
				else
					aFKTableName = aTableName;

			}

			xRelation   = NULL;
			xRefTable   = OpenTable(aRefTableName);
			if(!aTableName.Len())
				xRefFKTable = OpenTable(aFKTableName);

			if (xRefTable.Is() && xRefFKTable.Is())
				xRelation = new SdbOTERRelation(this, aConstraintName);

			if (!xRelation.Is())
			{
				xRefTable = NULL;
				xRefFKTable = NULL;
				xRelation = NULL;
			}
			else
			{
				xRelation->SetTableName(aFKTableName);
				xRelation->SetForeignTableName(aRefTableName);

				// jetzt noch die Attributierungen
				UINT16 nAttributes = 0,nOTERAttributes = 0;

				// Update rules
				xVar = rRow[10];
				if (!xVar->isNull())
				{
					nOTERAttributes = xVar->toINT16();
					if (nOTERAttributes & SQL_CASCADE)
						nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
					else if (nOTERAttributes & SQL_RESTRICT)
						nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
					else if (nOTERAttributes & SQL_SET_NULL)
						nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
					else if (nOTERAttributes & SQL_SET_DEFAULT)
						nAttributes |= ::stardata::DatabaseKeyRule::SET_DEFAULT;
					else
						nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
					xRelation->SetUpdateRules(nAttributes);
				}

				// Delete rules
				xVar = rRow[11];
				if (!xVar->isNull())
				{
					nAttributes = 0;
					nOTERAttributes = xVar->toINT16();
					if (nOTERAttributes & SQL_CASCADE)
						nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
					else if (nOTERAttributes & SQL_RESTRICT)
						nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
					else if (nOTERAttributes & SQL_SET_NULL)
						nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
					else if (nOTERAttributes & SQL_SET_DEFAULT)
						nAttributes |= ::stardata::DatabaseKeyRule::SET_DEFAULT;
					else
						nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
					xRelation->SetDeleteRules(nAttributes);
				}
			}
		}

		if (xRelation.Is())
		{
			const SdbColumn* pColumn = NULL;
			xVar = rRow[8];
			if (!xVar->isNull())
				pColumn = xRefFKTable->OriginalColumns()->Column(xVar->getString().getStr());

			xVar = rRow[4];
			if (!xVar->isNull())
				aForeignName = xVar->getString().getStr();

			if (pColumn && xRefTable->OriginalColumns()->Column(aForeignName))
			{
				SdbColumnRelation* pRelColumn = xRelation->CreateColumn(*pColumn, aForeignName);
				xRelation->GetColumns()->AddColumn(pRelColumn);
			}
			else
			{
				((SdbOTERRelation*)&xRelation)->RemoveFromParent();
				xRelation = NULL;
			}
		}
		pCursor->Next();
	}
	if (xRelation.Is())
	{
		((SdbOTERRelation*)&xRelation)->Open(); // nur das open flag setzen
		_rList.InsertAt(xRelation->GetName(), &xRelation);
	}

	pCursor->ReleaseRef();
}
//------------------------------------------------------------------
void SdbOTERConnection::FillAllRelations(SdbRelationList& _rList)
{
	FillRelations(_rList,String());
}
//-----------------------------------------------------------------------------
void SdbOTERConnection::FillTableNames(SdbStringStlMap* _pList)
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);

	// SdbOTERCursor handelt, daher Type Cast:
	SdbOTERCursor* pCursor = (SdbOTERCursor *) CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();
	// Einen ganz speziellen Cursor oeffnen: anstelle von
	// Prepare und Execute wird Tables ausgefuehrt; die
	// Ergebnismenge enthaelt die gewuenschten Informationen

	// Zugriff auf die Ergebnismenge normalerweise mit Spaltennamen
	// da manche Treiber aber nicht die Standardnamen verwenden, versuchen wir es bei denen
	// mit der Position

	// Auswahl der Tabellenart
	String aTableType(String::CreateFromAscii("TABLE;SYNONYM;VIEW;ALIAS"));
	for (xub_StrLen i = 0, j = 0; i < 4; i++)
	{
		pCursor->OpenTables(NULL,NULL,NULL,aTableType.GetToken(i,';').GetBuffer());
		/*      Reihenfolge
			1       TABLE_QUALIFIER
			2       TABLE_OWNER
			3       TABLE_NAME
		 */
		while (pCursor->IsInRange())
		{
			ODbVariantRef xVar = (*pCursor->GetRow())[3];

			if (!xVar->isNull())
				(*_pList)[xVar->getString().getStr()] = i != 2;

			pCursor->Next();
		}
		pCursor->Close();
	}
	pCursor->ReleaseRef();
}

static const UINT16 TYPE_NAME			= 1;
static const UINT16 DATA_TYPE			= 2;
static const UINT16 PRECISION			= 3;
static const UINT16 LITERAL_PREFIX		= 4;
static const UINT16 LITERAL_SUFFIX		= 5;
static const UINT16 CREATE_PARAMS		= 6;
static const UINT16 NULLABLE			= 7;
static const UINT16 CASE_SENSITIVE		= 8;
static const UINT16 SEARCHABLE			= 9;
static const UINT16 UNSIGNED_ATTRIBUTE	= 10;
static const UINT16 FIXED_PREC_SCALE	= 11;
static const UINT16 AUTO_UNIQUE_VALUE	= 12;
static const UINT16 LOCAL_TYPE_NAME		= 13;
static const UINT16 MINIMUM_SCALE		= 14;
static const UINT16 MAXIMUM_SCALE		= 15;
static const UINT16 SQL_DATA_TYPE		= 16;
static const UINT16 SQL_DATETIME_SUB	= 17;
static const UINT16 NUM_PREC_RADIX		= 18;
static const UINT16 USER_TYPE			= 19;
//-----------------------------------------------------------------------------
void SdbOTERConnection::FillTypeList(SdbTypeInfoList& rList)
{
	// Typ informationen sammeln
	SdbOTERCursor* pCur =
		(SdbOTERCursor *) ((SdbOTERConnection *) this)->CreateCursor(SDB_SNAPSHOT, SDB_READONLY | SDB_FORWARDONLY);
	pCur->AddRef();

	pCur->OpenTypeInfo(SQL_ALL_TYPES);
	aStatus = pCur->Status();

	while (pCur->IsInRange())
	{
		ODbRow& rRow = *pCur->GetRow();
		SdbDatabaseType eType = (SdbDatabaseType)MapType(rRow[DATA_TYPE]->toINT16());
		if (eType == SDB_DBTYPE_UNKNOWN)
		{
			pCur->Next();
			continue;
		}

		SdbTypeInfo* pInfo = new SdbTypeInfo;
		pInfo->eType = eType;
		pInfo->aTypeName            = rRow[TYPE_NAME]->getString().getStr();

		ODbVariantRef xValue		= rRow[PRECISION];
		if (!xValue->isNull() && (eType == SDB_DBTYPE_BINARY || eType == SDB_DBTYPE_VARBINARY))
			pInfo->nPrecision       = xValue->toINT32() / 8;
		else if (!xValue->isNull())
			pInfo->nPrecision       = xValue->toINT32();
		else if (eType == SDB_DBTYPE_TIMESTAMP)
			pInfo->nPrecision       = 17;
		else if (eType == SDB_DBTYPE_TIME || eType == SDB_DBTYPE_DATE)
			pInfo->nPrecision       = 8;
		else
			pInfo->nPrecision       = 0;


		xValue = rRow[MAXIMUM_SCALE];
		if (xValue->isNull())
			pInfo->nScale           = 0;
		else
			pInfo->nScale           = xValue->toINT16();

		xValue = rRow[FIXED_PREC_SCALE];
		pInfo->bCurrency            = xValue.is() && xValue->toBOOL();

		xValue = rRow[CREATE_PARAMS];
		if (pInfo->bCurrency && xValue->isNull())          // wenn keine Parameter zum Kreieren angegeben
			pInfo->eType = SDB_DBTYPE_CURRENCY;                         // dann kann der Datentyp ruhig als Currency angesehen werden

		xValue = rRow[NULLABLE];
		pInfo->bNullable            = (eType != SDB_DBTYPE_LONGVARCHAR) && xValue.is() && xValue->toBOOL();

		xValue = rRow[SEARCHABLE];
		switch (xValue->toINT16())
		{
			case SQL_UNSEARCHABLE:		pInfo->eSearchType = Search_None;			break;
			case SQL_LIKE_ONLY:			pInfo->eSearchType = Search_Like_Only;		break;
			case SQL_ALL_EXCEPT_LIKE:	pInfo->eSearchType = Search_Except_Like;	break;
			default:					pInfo->eSearchType = Search_All;			break;
		}

		switch(eType){
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_REAL:
			case SDB_DBTYPE_DOUBLE:
				{
					// hier wird der counter erfasst. Er ist kein Standardtyp von Oterro, sondern wird ueber ein sql-statement erzeugt
					SdbTypeInfo* pInfoCount		= new SdbTypeInfo(eType,pInfo->aTypeName);
					pInfoCount->bAutoIncrement	= TRUE;
					pInfoCount->bNullable		= FALSE;
					pInfoCount->eSearchType		= pInfo->eSearchType;
					pInfoCount->nPrecision		= pInfo->nPrecision;
					pInfoCount->nScale			= pInfo->nScale;
					pInfoCount->bCurrency		= pInfo->bCurrency;
					rList.Insert(pInfoCount, LIST_APPEND);
				}
		}
		rList.Insert(pInfo, LIST_APPEND);
		pCur->Next();
	}
	pCur->ReleaseRef();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::IsCapableImpl(SdbCapability eCapability) const
{
	BOOL bResult;
	switch(eCapability)
	{
	// Von OTER erfragbare Capabilities:
	// Updatebar, wenn
		case SDB_CAP_UPDATABLE:
			bResult = (bReadOnly) ? FALSE : SdbConnection::IsCapableImpl(eCapability);
			break;

		case SDB_CAP_READ_UNCOMMITTED:
		case SDB_CAP_READ_COMMITTED:
		case SDB_CAP_SERIALIZABLE:
			bResult = FALSE;
			break;

	// Sonstige Capabilities:

		case SDB_CAP_COMMIT_AUTO:
		case SDB_CAP_COMMIT_MANUAL:
			bResult = IsTransactOn();
			break;
		case SDB_CAP_REPEATABLE_READ:
		case SDB_CAP_JOIN:
		case SDB_CAP_DESC_INDEX:
		case SDB_CAP_BULKOPTIMIZE:
		case SDB_CAP_COMPACT:
			bResult = TRUE;
			break;
		case SDB_CAP_CREATE_TABLE:
			//	bResult = !IsMultiUserOn();
			bResult = FALSE;
			break;
		case SDB_CAP_RENAME_TABLE:
		case SDB_CAP_FOREIGNKEYS:
		case SDB_CAP_PRIMARYKEYS:
		case SDB_CAP_ALTER_TABLE:
		case SDB_CAP_CREATE_INDEX:
			//	bResult = !IsMultiUserOn() && !IsTransactOn();
			bResult = FALSE;
			break;
		case SDB_CAP_VERSIONING:
		case SDB_CAP_STATIC_PRIOR:
		case SDB_CAP_KEYSET_PRIOR:
			bResult = TRUE;
			break;
		default:
			bResult = SdbConnection::IsCapableImpl(eCapability);
	}
	return bResult;
}


//-----------------------------------------------------------------------------
void SdbOTERConnection::SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel)
{
	DBG_CHKTHIS(SdbOTERConnection,NULL);
	UDWORD nValue;
	switch (eNewIsolationLevel)
	{
		case SDB_ISO_READ_UNCOMMITTED:  nValue = SQL_TXN_READ_UNCOMMITTED; break;
		case SDB_ISO_READ_COMMITTED:    nValue = SQL_TXN_READ_COMMITTED; break;
		case SDB_ISO_REPEATABLE_READ:   nValue = SQL_TXN_REPEATABLE_READ; break;
		case SDB_ISO_SERIALIZABLE:      nValue = SQL_TXN_SERIALIZABLE; break;
		//case SDB_ISO_VERSIONING:        nValue = SQL_TXN_VERSIONING; break;
		// veraltet
		default:
			aStatus.SetDriverNotCapableError();
			return;
	}

	SetStatus(OSQLSetConnectOption(aConnectionHandle,SQL_TXN_ISOLATION,nValue),aConnectionHandle);
	if (! aStatus.IsSuccessful())
		return;
	eIsolationLevel = eNewIsolationLevel;
}

//-----------------------------------------------------------------------------
SdbDatabaseType SdbOTERConnection::MapType(INT16 nOTERType)
{
	switch(nOTERType)
	{
		case SQL_CHAR:                  return SDB_DBTYPE_CHAR;
		case SQL_VARCHAR:               return SDB_DBTYPE_VARCHAR;
		case SQL_LONGVARCHAR:			return SDB_DBTYPE_LONGVARCHAR;
		case SQL_DECIMAL:               return SDB_DBTYPE_DECIMAL;
		case SQL_NUMERIC:               return SDB_DBTYPE_NUMERIC;
		case SQL_BIT:                   return SDB_DBTYPE_BOOLEAN;
		case SQL_TINYINT:               return SDB_DBTYPE_TINYINT;
		case SQL_SMALLINT:              return SDB_DBTYPE_SMALLINT;
		case SQL_INTEGER:               return SDB_DBTYPE_INTEGER;
		case SQL_BIGINT:                return SDB_DBTYPE_BIGINT;
		case SQL_REAL:                  return SDB_DBTYPE_REAL;
		case SQL_FLOAT:
		case SQL_DOUBLE:                return SDB_DBTYPE_DOUBLE;
		case SQL_BINARY:                return SDB_DBTYPE_BINARY;
		case SQL_VARBINARY:				return SDB_DBTYPE_VARBINARY;
		case SQL_LONGVARBINARY:			return SDB_DBTYPE_LONGVARBINARY;
		case SQL_DATE:                  return SDB_DBTYPE_DATE;
		case SQL_TIME:                  return SDB_DBTYPE_TIME;
		case SQL_TIMESTAMP:				return SDB_DBTYPE_TIMESTAMP;
		case -16:						return SDB_DBTYPE_CURRENCY;
		default:                        return SDB_DBTYPE_UNKNOWN;
	}
}
//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::IsAutoNum(const String& aTabName,const String& aColName) {
	SdbOTERCursor* pCursor = (SdbOTERCursor*)CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();
	String aAutoSql;
	aAutoSql.AssignAscii("select  a.sys_increment,b.sys_column_name,b.sys_column_id  from sys_defaults a,sys_columns b,sys_tables c where c.sys_table_name = '");
	aAutoSql += aTabName;
	aAutoSql += String::CreateFromAscii("' and b.sys_column_name = '");
	aAutoSql += aColName;
	aAutoSql += String::CreateFromAscii("' and a.sys_column_id = b.sys_column_id and c.sys_table_id = b.sys_table_id and a.sys_increment is not null");
	pCursor->Open(aAutoSql);
	BOOL bResult(!pCursor->IsOffRange());
	pCursor->Close();
	pCursor->ReleaseRef();
	return bResult;
}
//-----------------------------------------------------------------------------
class SdbBlobCopy
{
public:
	SdbBlobCopy(SvMemoryStream& _rFStrm, USHORT _nId);
};
//------------------------------------------------------------------------
SdbBlobCopy::SdbBlobCopy(SvMemoryStream& _rFStrm, USHORT _nId)
{
	SimpleResMgr* pResMgr = SdbEnvironmentRef()->GetResMgr();

	void* pBuffer;
	USHORT nSize = pResMgr->ReadBlob(_nId, &pBuffer);
	if (nSize)
	{
		DBG_ASSERT(pBuffer, "SdbBlobCopy::SdbBlobCopy : invalid buffer !");
		_rFStrm.Write(pBuffer, nSize);
	}

	pResMgr->FreeBlob(pBuffer);
}
//-----------------------------------------------------------------------------
BOOL SdbOTERConnection::Create()
{
	DirEntry aFile(aOptionsParser.OptionString(String::CreateFromAscii("DSN")));

	aFile.SetExtension(String::CreateFromAscii("rb1"));
	SvFileStream aNewDbFile(aFile.GetFull(),STREAM_WRITE);

	SvMemoryStream aMemStr;
	SdbBlobCopy(aMemStr,STARBASE0);
	SdbBlobCopy(aMemStr,STARBASE00);
	aMemStr.Seek(STREAM_SEEK_TO_BEGIN);
	aNewDbFile << aMemStr;
	aNewDbFile.Close();
	aMemStr.SwitchBuffer();

	aFile.SetExtension(String::CreateFromAscii("rb2"));
	aNewDbFile.Open(aFile.GetFull(),STREAM_WRITE);
	SdbBlobCopy(aMemStr,STARBASE1);
	SdbBlobCopy(aMemStr,STARBASE10);
	SdbBlobCopy(aMemStr,STARBASE11);
	aMemStr.Seek(STREAM_SEEK_TO_BEGIN);
	aNewDbFile << aMemStr;
	aNewDbFile.Close();
	aMemStr.SwitchBuffer();

	aFile.SetExtension(String::CreateFromAscii("rb3"));
	aNewDbFile.Open(aFile.GetFull(),STREAM_WRITE);
	SdbBlobCopy(aMemStr,STARBASE2);
	SdbBlobCopy(aMemStr,STARBASE20);
	SdbBlobCopy(aMemStr,STARBASE21);
	SdbBlobCopy(aMemStr,STARBASE22);
	SdbBlobCopy(aMemStr,STARBASE23);
	aMemStr.Seek(STREAM_SEEK_TO_BEGIN);
	aNewDbFile << aMemStr;
	aNewDbFile.Close();
	aMemStr.SwitchBuffer();

	aFile.SetExtension(String::CreateFromAscii("rb4"));
	aNewDbFile.Open(aFile.GetFull(),STREAM_WRITE);
	SdbBlobCopy(aMemStr,STARBASE3);
	SdbBlobCopy(aMemStr,STARBASE30);
	aMemStr.Seek(STREAM_SEEK_TO_BEGIN);
	aNewDbFile << aMemStr;
	aNewDbFile.Close();
	aMemStr.SwitchBuffer();

	return TRUE;
}
//-----------------------------------------------------------------------------
// bSilent wird dann verwendet, wenn eine Connection als Copy einer anderen
// verwendet wird
RETCODE SdbOTERConnection::OpenConnection(const String &aDSN,const String &aUID,const String &aPWD,INT32 nTimeOut)
{

	if (aDSN.Len() >= 80)
	{	// riesengrosser mieser Bug in Oterro (64542) : wenn man eine DSN in OSQLConnect steckt, die zwischen 80 und 84 Zeichen
		// hat, schmiert Oterro einfach ab. Bei allen groesseren Strings geht das Connect einfach schief, obwohl die rb?-Dateien
		// alle existieren und auch nicht korrupt sind (in einem kuerzeren Pfad funktionieren sie).
		aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("01000"), aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_OTER_INVALID_DSN_LEN))),
			0, String(SdbResId(STR_STAT_OTER_INVALID_DSN_LEN_DETAILED)) );
		return SQL_ERROR;
	}

	// Connection allozieren
	// (Eine OTER-Connection pro SdbConnection).
	RETCODE nRetCode(OSQLAllocConnect(aEnvironmentHandle->GetHandle(),&aConnectionHandle));
	if (nRetCode == SQL_ERROR || nRetCode == SQL_NO_DATA_FOUND || nRetCode == SQL_INVALID_HANDLE)
		return nRetCode;

	// automatisch upgraden
	OSQLSetConnectOption(aConnectionHandle,
						 1012,	// SQL_MICRORIM_AUTOUPGRADE_MODE
						 1);

	OSQLSetConnectOption(aConnectionHandle,
						 1001,	// SQL_MICRORIM_COMPATIBILITY_MODE
						 1);

	OSQLSetConnectOption(aConnectionHandle,
						 1004,  // SQL_MICRORIM_MULTIUSER_MODE
						 IsMultiUserOn() ? 1 : 0);

	OSQLSetConnectOption(aConnectionHandle,
						 1006, // SQL_MICRORIM_TRANSACTION_MODE
						 IsTransactOn() ? 1 :0 );

	OSQLSetConnectOption(aConnectionHandle,SQL_LOGIN_TIMEOUT,nTimeOut);
	if (!IsTransactOn())
	{
		if (IsMultiUserOn())
		{
			OSQLSetConnectOption(aConnectionHandle,
						 1007, // SQL_MICRORIM_STATICDB_MODE
						 1);

			OSQLSetConnectOption(aConnectionHandle,
						 1008, // SQL_MICRORIM_FASTLOCKS_MODE
						 1);
		}
		OSQLSetConnectOption(aConnectionHandle,SQL_AUTOCOMMIT,SQL_AUTOCOMMIT_ON);
	}
	else
		OSQLSetConnectOption(aConnectionHandle,SQL_AUTOCOMMIT,SQL_AUTOCOMMIT_OFF);

	aDsn = aDSN;
	while(aDsn.SearchAndReplace(String::CreateFromAscii("/"),String::CreateFromAscii("\\")) != STRING_NOTFOUND)
		;

	ByteString aDSN1(aDsn,gsl_getSystemTextEncoding());
	ByteString aUID1(aUID,gsl_getSystemTextEncoding());
	ByteString aPWD1(aPWD,gsl_getSystemTextEncoding());

	SWORD nDsnLen = aDSN1.Len() ? aDSN1.Len() : 1;
	SWORD nUidLen = aUID1.Len() ? aUID1.Len() : 1;
	SWORD nPwdLen = aPWD1.Len() ? aPWD1.Len() : 1;

	SDB_OTER_CHAR *szDSN = new SDB_OTER_CHAR[nDsnLen + 1];
	SDB_OTER_CHAR *szUID = new SDB_OTER_CHAR[nUidLen + 1];
	SDB_OTER_CHAR *szPWD = new SDB_OTER_CHAR[nPwdLen + 1];

	memcpy(szDSN, (SDB_OTER_CHAR*) aDSN1.GetBuffer(), nDsnLen);
	memcpy(szUID, (SDB_OTER_CHAR*) aUID1.GetBuffer(), nUidLen);
	memcpy(szPWD, (SDB_OTER_CHAR*) aPWD1.GetBuffer(), nPwdLen);
	szDSN[nDsnLen] = 0;
	szUID[nUidLen] = 0;
	szPWD[nPwdLen] = 0;

	// nach einem Zugriff mit einem falschen Nutzer hat Otero ein paar Probleme ... soll heissen, der erste Zugriff mit dem neuen
	// (korrekten) Nutzer geht schief, der zweite mit voellig unveraederten Parametern klappt ganz prima ...
	// Also ein Fake : ich mache es einfach ein zweites Mal, wenn es das erste Mal schief geht. Das ist nicht so schlimm, wie es
	// sich anhoert (hoffe ich), da der zweite Versuch wirklich nur bei Misserfolg gestartet wird und ziemllich schnell geht
	// (gegenueber dem ersten erfolglosen - was ein rein subjektiver Eindruck aus einigen Tests ist).
	signed char nCount = 1;
	nRetCode = SQL_SUCCESS;
	do
		nRetCode = OSQLConnect(aConnectionHandle, szDSN, nDsnLen, szUID, nUidLen, szPWD, nPwdLen);
	while ((nRetCode != SQL_SUCCESS) && (nCount-- > 0));


	delete [] szDSN;
	delete [] szUID;
	delete [] szPWD;

	if (nRetCode == SQL_ERROR || nRetCode == SQL_NO_DATA_FOUND || nRetCode == SQL_INVALID_HANDLE)
	{
		aConnectionHandle = SQL_NULL_HDBC;
		return nRetCode;
	}

	BOOL bTest(FALSE);
	if (GetOTERInfo(SQL_DATA_SOURCE_READ_ONLY,bTest))
		bReadOnly = bTest;

	bOpen = TRUE; // muss true gesetzt sein damit die CHECK_OPEN_ASSERT nicht anspricht
	SdbOTERCursor* pCursor = new SdbOTERCursor(this,SDB_SNAPSHOT, SDB_FORWARDONLY);
	pCursor->ExecuteDirect(String::CreateFromAscii("set quotes='"));
	pCursor->ExecuteDirect(String::CreateFromAscii("set idquotes=\""));
	pCursor->ExecuteDirect(String::CreateFromAscii("set currency ' ' suff 2 b"));
	pCursor->ExecuteDirect(String::CreateFromAscii("SET DATE SEQ YYYY-MM-DD"));
	pCursor->ExecuteDirect(String::CreateFromAscii("SET DATE FOR YYYY-MM-DD"));

	if (!IsTransactOn() && IsMultiUserOn())
		pCursor->ExecuteDirect(String::CreateFromAscii("SET QUALCOLS 2"));
/*
	DirEntry aDir;
	DirEntry aTmp2(aDir.TempName());
	aDir = aTmp2.GetPath();
	String aTempPath("set scratch=");
	aTempPath += aDir.GetFull();
	pCursor->ExecuteDirect(aTempPath);
*/
	delete pCursor;
	bOpen = FALSE; // hier wieder den alten Zustand setzen

	SetStatus(nRetCode);
	return nRetCode;
}

//-----------------------------------------------------------------------------
String SdbOTERConnection::ConnectStr() const
{
	String aOptions;
	aOptionsParser.Compose(aOptions,String::CreateFromAscii(pReservedOptions));      // Alles ausser den Sdb-internen Optionen weiterreichen
	aOptions.EraseLeadingChars(';');
	aOptions.EraseTrailingChars(';');
	return aOptions;
}
//-----------------------------------------------------------------------------
::stardata::XDatabaseDescriptor* SdbOTERConnection::GetMetaDataImpl()
{
	SdbOTERMetaData* pMetaData = new SdbOTERMetaData(this);
	return pMetaData;
}
//-----------------------------------------------------------------------------
void SdbOTERConnection::FreeEnv(SdbEnvironment* pEnv)
{
}
//------------------------------------------------------------------
BOOL SdbOTERConnection::DoCmdImpl(UINT16 nLimit,const String &rArgs){
	BOOL bResult(FALSE);
	switch(nLimit){
		case SDB_CAP_COMPACT:
			String aTmp(String::CreateFromAscii("RELOAD "));
			aTmp += rArgs;
			bResult = ExecuteSQL(aTmp);
			break;
	}
	return bResult;
}
//------------------------------------------------------------------
#if SUPD>511 || PRIV_DEBUG
BOOL SdbOTERConnection::StructureIsReadOnly()
{
	//	return IsMultiUserOn() || IsTransactOn();
	return TRUE;
}
#endif // SUPD>511 || PRIV_DEBUG
