/*************************************************************************
 *
 *  $RCSfile: otercurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbresid.hrc"
#include "otercurs.hxx"
#include "oterconn.hxx"

#include <stdlib.h>

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEOUTERJOINSUPPORT_HPP_
#include <com/sun/star/data/DatabaseOuterJoinSupport.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

#ifndef _SDB_TYPECONV_HXX
#include <typeconv.hxx>
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

DBG_NAME(SdbOTERCursor);

// in odbc we have a fraction part for time values, a single unit is 1/100000000
const ULONG OTER_FRACTION_UNITS_PER_HSECOND_READ  = 10000000L;
const ULONG OTER_FRACTION_UNITS_PER_HSECOND_WRITE = 1000000L;	// da Fehler beim chreiben in Oterro
const String aTruncState = String::CreateFromAscii("01004");
const String aValueChangedState = String::CreateFromAscii("01S02");

//------------------------------------------------------------------
static DateTime ToDateTime(const TIMESTAMP_STRUCT& rTS)
{
	return DateTime (Date(rTS.day,rTS.month,rTS.year),
			  Time(rTS.hour,rTS.minute,rTS.second,long (rTS.fraction / OTER_FRACTION_UNITS_PER_HSECOND_READ)));
}

//------------------------------------------------------------------
static Time ToTime(const TIME_STRUCT& rTS)
{
	return Time(rTS.hour,rTS.minute,rTS.second);
}

//------------------------------------------------------------------
static Date ToDate(const DATE_STRUCT& rTS)
{
	return Date(rTS.day,rTS.month,rTS.year);
}

//------------------------------------------------------------------
static void ToDate(DATE_STRUCT& rResult, double fSbxDate)
{
	Date aDate = SdbTools::ToDate(fSbxDate);
	rResult.year	  = aDate.GetYear();
	rResult.month	  = aDate.GetMonth();
	rResult.day		  = aDate.GetDay();
}

//------------------------------------------------------------------
static void ToTime(TIME_STRUCT& rResult, double fSbxDate)
{
	Time aTime = SdbTools::ToTime(fSbxDate);
	// Zeit:
	rResult.hour	  = aTime.GetHour();
	rResult.minute	  = aTime.GetMin();
	rResult.second	  = aTime.GetSec();
}

//------------------------------------------------------------------
static void ToTimeStamp(TIMESTAMP_STRUCT& rResult, double fSbxDate)
{
	DateTime aDateTime = SdbTools::ToDateTime(fSbxDate);
	rResult.year	  = aDateTime.GetYear();
	rResult.month	  = aDateTime.GetMonth();
	rResult.day		  = aDateTime.GetDay();

	// Zeit:
	rResult.hour	  = aDateTime.GetHour();
	rResult.minute	  = aDateTime.GetMin();
	rResult.second	  = aDateTime.GetSec();
	rResult.fraction  = aDateTime.Get100Sec() * OTER_FRACTION_UNITS_PER_HSECOND_WRITE;
}

//-----------------------------------------------------------------------------
SdbOTERCursor::SdbOTERCursor(SdbOTERConnection* pConnection,SdbCursorType eType,UINT16 nOpt)
			  :SdbCursor(pConnection, eType, nOpt)
			  ,aStatementHandle(SQL_NULL_HSTMT)
			  ,nRowVerLen(0)
			  ,pParamBuffer(NULL)
			  ,pParamLength(NULL)
			  ,pRowVer(NULL)
			  ,nCurrentMaxFieldLen(nMaxFieldLen)
			  ,nCurrentMaxRows(nMaxRows)
			  ,bCommitTrans(FALSE)
{
	DBG_CTOR(SdbOTERCursor,NULL);
	aStatus.SetComponent(String::CreateFromAscii("StarBase Driver"));

	bIsCancelable = TRUE;


	RETCODE rRetcode(OSQLAllocStmt(pConnection->ConnectionHandle(),&aStatementHandle));
	SetStatus(rRetcode,aStatementHandle);

	if (aStatus.IsError())
	{
		// es ist nicht moeglich einen weiteree Handle zu bekommen,
		// daher versuchen wir einfach noch eine Verbindung aufzumachen
		aStatus.Clear();
		aStatementHandle = SQL_NULL_HSTMT;
	}
	else if (!(nOpt & SDB_READONLY))
		SetStatus(OSQLSetStmtOption(aStatementHandle,SQL_CONCURRENCY,SQL_CONCUR_ROWVER),aStatementHandle);
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::SetStatus(RETCODE nRetcode, HSTMT hstmt,const String& rErrorInfo)
{
	SdbOTERConnection *pCon = GetOTERConnection();
	SdbOTER_SetStatus(aStatus,nRetcode,pCon->aEnvironmentHandle,pCon->ConnectionHandle(),hstmt,rErrorInfo);
}

//-----------------------------------------------------------------------------
SdbOTERCursor::~SdbOTERCursor()
{
	DBG_DTOR(SdbOTERCursor,NULL);
#if SUPD>557
	// (RemovingFromParent isn't virtual in 557)
	DBG_ASSERT(GetParent() || aStatementHandle == SQL_NULL_HSTMT, "SdbOTERCursor::~SdbOTERCursor : RemovingFromParent should have been called if I have no parent anymore !");
#endif
	if (aStatementHandle != SQL_NULL_HSTMT)
	{
		try
		{
			OSQLFreeStmt(aStatementHandle,SQL_DROP);
		}
		catch ( ... )
		{
		}

		aStatementHandle = SQL_NULL_HSTMT;
	}
	delete [] pRowVer;
	pRowVer = NULL;
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::RemovingFromParent()
{
	DBG_ASSERT(!IsOpen(), "SdbOTERCursor::RemovingFromParent : expected to be called _after_ Close !");

	// drop our statement. We can't do this later as the connection - which is our parent - may be destroyed then.
	if (aStatementHandle != SQL_NULL_HSTMT)
	{
		OSQLFreeStmt(aStatementHandle,SQL_DROP);
		aStatementHandle = SQL_NULL_HSTMT;
	}
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::PrepareImpl()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	// Statement vorbereiten
	Translate(FALSE);
	if (! aStatus.IsSuccessful())
		return FALSE;

	RETCODE nRetcode;
	ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
	nRetcode = OSQLPrepare(aStatementHandle,(SDB_OTER_CHAR *) aSql.GetBuffer(),aSql.Len());

	SetStatus(nRetcode,aStatementHandle,aSQLStatement);
	if (! aStatus.IsSuccessful())
		return FALSE;

	String aCopy(aSQLStatement);
	aCopy.EraseLeadingChars(' ');
	String aTok1(aCopy.GetToken(0,' '));
	if(aTok1.EqualsIgnoreCaseAscii("update") || aTok1.EqualsIgnoreCaseAscii("insert") || aTok1.EqualsIgnoreCaseAscii("delete"))
		bCommitTrans = TRUE;

	// Cursor-Namen abholen (jetzt ist er bestimmt verfuegbar):
	// selbst setzen, da der Name bis jetzt noch nicht wirklich
	// bentigt wird
	if (aName.Len() == 0)
	{
		aName = 'C';
		aName += String::CreateFromInt32((long)this);
	}

	// hack
	// Parameter schon gesetzt?
	if (aParameterColumns->Count() < 2)
		DescribeParameter();
	if (aStatus.IsSuccessful())
		BindParameter();

	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::DescribeResult()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aResultColumns.Is(), "keine Ergebnisbeschreibung");
	// Bookmark-Column ist immer da (schon durch den Konstruktor erzeugt).

	SdbOTERConnection* pConnection = GetOTERConnection();

	if (aResultColumns->Count() > 1 || !pConnection)
		return;

	// Alle ResultColumns ansehen
	SWORD nNumResultCols(0);
	RETCODE nRetcode(0);
	nRetcode = OSQLNumResultCols(aStatementHandle,&nNumResultCols);
	if (nRetcode)
	{
		SetStatus(nRetcode,aStatementHandle);
		return;
	}
	if(pRowVer)
	{
		delete [] pRowVer;
		pRowVer = NULL;
	}
	nRowVerLen = nNumResultCols+1;
	pRowVer = new INT32[nRowVerLen];

	SWORD pcbColName;			// Laenge des Names
	SWORD pfSqlType;			// SQL-Datentyp
	UDWORD pcbColDef;			// Precision (ggf. 0)
	SWORD  pibScale;				// Scale (ggf. 0)
	SWORD  pfNullable;			// NULL erlaubt?
	INT16 nFlags(0);
	SdbDatabaseType eType;
	String aAlias,aColumnName;
	for (SWORD i = 1,nCo=1; i <= nNumResultCols; i++)
	{
		pRowVer[i] = 0;
		nFlags = pibScale = pfNullable = 0;
		// Informationen von SQLDescribeCol:
		// erstmal die L"ange des Spaltennamens
		SDB_OTER_CHAR pTmp[1]; // Oterro spez. gilt nur zum Festhalten
		nRetcode =OSQLDescribeCol(aStatementHandle,
								   i,
								   pTmp,
								   0,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
		if (nRetcode)
		{
			SetStatus(nRetcode,aStatementHandle);
			return;
		}
		DBG_ASSERT(pcbColName>0,"StarBase: pcbColName <= 0. Prepare for GPF! ;-(");
		SDB_OTER_CHAR* pStr = (SDB_OTER_CHAR*)aColumnName.AllocBuffer(pcbColName+1);

		nRetcode =OSQLDescribeCol(aStatementHandle,
								   i,
								   pStr,
								   pcbColName+1,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
		aColumnName.ReleaseBufferAccess();
		if (nRetcode)
		{
			SetStatus(nRetcode,aStatementHandle);
			return;
		}

		xub_StrLen nPoint(aColumnName.Search('.'));

		if(nPoint != STRING_NOTFOUND)
			aColumnName = aColumnName.Copy(nPoint+1);


		if(aColumnName.EqualsIgnoreCaseAscii("SYS_ROWVER"))
		{
			pRowVer[i] = i;//Position merken und Row nicht speichern
			nFlags |= SDB_FLAGS_VERSIONING;
		}

		eType = pConnection->MapType(pfSqlType);

		if (pfNullable != SQL_NO_NULLS)
			nFlags |= SDB_FLAGS_NULLALLOWED;

		if (!(nOptions & SDB_MINIMUM_DESC))
		{
			SDWORD nExtAttrib,nUpd;	// erweitere Attribute
			if (OSQLColAttributes(aStatementHandle,		// ist die Column updatebar?
									 i,
									 SQL_COLUMN_UPDATABLE,
									 NULL,
									 0,
									 NULL,
									 &nUpd
									) == SQL_SUCCESS)
			{
				if (nUpd != SQL_ATTR_READONLY && eType != SDB_DBTYPE_UNKNOWN)
					nFlags |= SDB_FLAGS_UPDATEALLOWED;
			}


			if (OSQLColAttributes(aStatementHandle,	    // handelt es sich um ein MoneyFeld
										 i,
										 SQL_COLUMN_MONEY,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib)
					eType = SDB_DBTYPE_CURRENCY;
			}

			if (OSQLColAttributes(aStatementHandle,		// ist die Column searchable?
										 i,
										 SQL_COLUMN_SEARCHABLE,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				switch (nExtAttrib)
				{
					case SQL_SEARCHABLE:		nFlags |= SDB_FLAGS_SEARCH_ALL; break;
					case SQL_LIKE_ONLY:			nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
					case SQL_ALL_EXCEPT_LIKE:	nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				}
			}

			if((eType == SDB_DBTYPE_BINARY || eType == SDB_DBTYPE_VARBINARY) && pcbColDef > 4)
				pcbColDef -= 4; // Oterro Anhang

			if(nUpd == SQL_ATTR_READWRITE_UNKNOWN){
				SdbOTERCursor* pCursor = (SdbOTERCursor *) pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
				pCursor->AddRef();
				String aAutoSql;
				aAutoSql.AssignAscii("select a.sys_increment from sys_defaults a,sys_columns b where a.sys_column_id = b.sys_column_id and a.sys_increment is not null and b.sys_column_name = '");
				aAutoSql += aColumnName;
				aAutoSql += '\'';
				pCursor->Open(aAutoSql);
				 // einmal reicht, um zu wissen das es sich um ein AutoIncremment handelt
				if(pCursor->IsInRange()){
					nFlags |= SDB_FLAGS_AUTOINCREMENT;
					nFlags &= ~SDB_FLAGS_UPDATEALLOWED;	    // darf nicht updatebar sein
					nFlags &= ~SDB_FLAGS_NULLALLOWED;
					pibScale = 0;
					pcbColDef = 0;
				}
				pCursor->Close();
				pCursor->ReleaseRef();
			}
		}

		aAlias = aColumnName;
		const SdbColumn* pPrevCol = aResultColumns->Column(aColumnName);
				// existiert Name  in aSelectColumns ?
		ULONG nExprCnt(0);
		while (pPrevCol)
		{
			(aAlias = aColumnName) += String::CreateFromInt32(++nExprCnt);
			pPrevCol = aResultColumns->Column(aAlias);
		}
		// Column-Beschreibung hinzufuegen.
		aResultColumns->AddColumn(pConnection->CreateColumn(aAlias,eType,pcbColDef,pibScale, nFlags));
	}
	// Strukturbeschreibung fertig.
	SetStatus(nRetcode,aStatementHandle); // Status aktualisieren
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::DescribeParameter()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	SdbCursor::DescribeParameter();
	return;
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::BindParameter()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	aStatus.Clear();
	AllocParamData();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::BindColumn(const SdbColumn& rCol, UINT16 nPos, const ODbVariantRef& xVar)
{
	RETCODE nRetcode;
	SWORD   fSqlType;
	SWORD   fCType;
	SDWORD  nMaxLen = 0;
	void*   pData = pParamBuffer[nPos];
	SDWORD* pLen  = &pParamLength[nPos];

	switch(rCol.GetType())
	{
		case SDB_DBTYPE_CHAR:			fCType   = SQL_C_CHAR;
										fSqlType = SQL_CHAR; break;
		case SDB_DBTYPE_VARCHAR:		fCType   = SQL_C_CHAR;
										fSqlType = SQL_VARCHAR; break;
		case SDB_DBTYPE_LONGVARCHAR:	fCType = SQL_C_CHAR;
										fSqlType = SQL_LONGVARCHAR; break;
		case SDB_DBTYPE_DECIMAL:		fCType = SQL_C_CHAR;
										fSqlType = SQL_DECIMAL; break;
		case SDB_DBTYPE_CURRENCY:
		case SDB_DBTYPE_NUMERIC:		fCType	 = SQL_C_CHAR;
										fSqlType = SQL_NUMERIC; break;
		case SDB_DBTYPE_BOOLEAN:		fCType   = SQL_C_TINYINT;
										fSqlType = SQL_INTEGER; break;
		case SDB_DBTYPE_TINYINT:		fCType = SQL_C_SHORT;
										fSqlType = SQL_TINYINT; break;
		case SDB_DBTYPE_SMALLINT:		fCType = SQL_C_SHORT;
										fSqlType = SQL_SMALLINT; break;
		case SDB_DBTYPE_INTEGER:		fCType = SQL_C_LONG;
										fSqlType = SQL_INTEGER; break;
		case SDB_DBTYPE_BIGINT:			fCType = SQL_C_CHAR;
										fSqlType = SQL_BIGINT; break;
		case SDB_DBTYPE_REAL:			fCType = SQL_C_FLOAT;
										fSqlType = SQL_REAL; break;
		case SDB_DBTYPE_DOUBLE:			fCType = SQL_C_DOUBLE;
										fSqlType = SQL_DOUBLE; break;
		case SDB_DBTYPE_BINARY:			fCType = SQL_C_BINARY;
										fSqlType = SQL_BINARY; break;
		case SDB_DBTYPE_VARBINARY:		fCType = SQL_C_BINARY;
										fSqlType = SQL_VARBINARY; break;
		case SDB_DBTYPE_LONGVARBINARY: 	fCType = SQL_C_BINARY;
										fSqlType = SQL_LONGVARBINARY; break;
		case SDB_DBTYPE_DATE:			fCType = SQL_C_DATE;
										fSqlType = SQL_DATE; break;
		case SDB_DBTYPE_TIME:			fCType = SQL_C_TIME;
										fSqlType = SQL_TIME; break;
		case SDB_DBTYPE_TIMESTAMP:		fCType = SQL_C_TIMESTAMP;
										fSqlType = SQL_TIMESTAMP; break;
		default:						aStatus.SetDriverNotCapableError();
										return FALSE;
	}

	// memofelder duerfen nicht Null sein
	if ((!xVar.is() || xVar->isNull()) && rCol.GetType() != SDB_DBTYPE_CURRENCY
		&& rCol.GetType() != SDB_DBTYPE_DECIMAL && rCol.GetType() != SDB_DBTYPE_NUMERIC && rCol.GetType() != SDB_DBTYPE_BIGINT)
	{
		*pLen = SQL_NULL_DATA;
		nRetcode = OSQLBindParameter(aStatementHandle,
									nPos,
									SQL_PARAM_INPUT,
									fCType,
									fSqlType,
									0,
									0,
									NULL,
									nMaxLen,
									pLen
									);
	}
	else
	{
		try
		{
			switch (rCol.GetType())
			{
				case SDB_DBTYPE_CHAR:
				case SDB_DBTYPE_VARCHAR:
				{
					ByteString aString(ODbTypeConversion::toString(*xVar).getStr(), GetConnection()->GetCharacterSet());
					nMaxLen = rCol.GetPrecision();

					*pLen = Min((xub_StrLen)nMaxLen, aString.Len());
					*((ByteString*)pData) = aString;

					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}	break;
				case SDB_DBTYPE_CURRENCY:
				case SDB_DBTYPE_BIGINT:
				case SDB_DBTYPE_DECIMAL:
				case SDB_DBTYPE_NUMERIC:
					if(!xVar.is() || xVar->isNull())
					{
						ByteString aString;
						nMaxLen = aString.Len();
						*pLen = SQL_NTS;
						*((ByteString*)pData) = aString;
						// Zeiger auf Char*
						pData = (void*)((ByteString*)pData)->GetBuffer();
					}
					else
					{
						ByteString aString(SdbTools::ToString(ODbTypeConversion::toDouble(*xVar),rCol.GetScale()),gsl_getSystemTextEncoding());
						nMaxLen = aString.Len();
						*pLen = SQL_NTS;
						*((ByteString*)pData) = aString;
						// Zeiger auf Char*
						pData = (void*)((ByteString*)pData)->GetBuffer();
					}
					break;
				case SDB_DBTYPE_BOOLEAN:
					*((INT8*)pData) = ODbTypeConversion::toBOOL(*xVar) ? 1 : 0;
					*pLen = sizeof(INT8);
					break;
				case SDB_DBTYPE_TINYINT:
				case SDB_DBTYPE_SMALLINT:
					*((INT16*)pData) = ODbTypeConversion::toINT16(*xVar);
					*pLen = sizeof(INT16);
					break;
				case SDB_DBTYPE_INTEGER:
					*((INT32*)pData) = ODbTypeConversion::toINT32(*xVar);
					*pLen = sizeof(INT32);
					break;
				case SDB_DBTYPE_REAL:
					*((float*)pData) = ODbTypeConversion::toFloat(*xVar);
					*pLen = sizeof(float);
					break;
				case SDB_DBTYPE_DOUBLE:
					*((double*)pData) = ODbTypeConversion::toDouble(*xVar);
					*pLen = sizeof(double);
					break;
				case SDB_DBTYPE_BINARY:
				case SDB_DBTYPE_VARBINARY:
					if (xVar->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
					{
						const ::staruno::Sequence<sal_Int8>& rData = xVar->getBytes();
						pData = (void*)rData.getConstArray();
						*pLen = rData.getLength();
						nMaxLen = rCol.GetPrecision();
					}	break;
				case SDB_DBTYPE_LONGVARBINARY:
				case SDB_DBTYPE_LONGVARCHAR:
				{
					pData  = (void*)&rCol;
					INT32 nLen = 0;
					if (xVar->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
						nLen = xVar->getBytes().getLength();
					else
						nLen = ODbTypeConversion::toString(*xVar).len();
					*pLen = (SDWORD)SQL_LEN_DATA_AT_EXEC(nLen);
				}	break;
				case SDB_DBTYPE_DATE:
					ToDate(*(DATE_STRUCT*)pData, ODbTypeConversion::toDouble(*xVar));
					*pLen = sizeof(DATE_STRUCT);
					break;
				case SDB_DBTYPE_TIME:
					ToTime(*(TIME_STRUCT*)pData, ODbTypeConversion::toDouble(*xVar));
					*pLen = sizeof(TIME_STRUCT);
					break;
				case SDB_DBTYPE_TIMESTAMP:
					ToTimeStamp(*(TIMESTAMP_STRUCT*)pData, ODbTypeConversion::toDouble(*xVar));
					*pLen = sizeof(TIMESTAMP_STRUCT);
			}

			nRetcode = OSQLBindParameter(aStatementHandle,
							  nPos,
							  SQL_PARAM_INPUT,
							  fCType,
							  fSqlType,
							  rCol.GetPrecision(),
							  rCol.GetScale(),
							  pData,
							  nMaxLen,
							  pLen);
		}
		catch ( ... )
		{
			String strError(SdbResId(STR_DBF_INVALID_FIELD_VALUE));
			strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());
			return FALSE;
		}

	}

	//SetStatus(nRetcode,aStatementHandle);
	return !nRetcode;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::OpenImpl(SdbCursorType eType ,UINT16 eOpt)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

	if (IsPrepared())
	{
		if (IsOpen())
		{
			// Schon einmal Execute ausgefuehrt.
			// Vor dem naechsten Execute muss der Cursor erst geschlossen werden
			// (Option CLOSE).
			SdbConnection *pConnection = GetConnection();
			if (pConnection->GetAutoCommitMode() && !(nOptions & SDB_READONLY) && !bCommitTrans)
				pConnection->CommitTrans();

			OSQLFreeStmt(aStatementHandle,SQL_CLOSE);
		}

		// Parameter uebertragen
		UINT16 nCount(aParameterColumns->Count());
		BOOL bSuccess = TRUE;
		for (UINT16 i=1 ; bSuccess && i < nCount;i++)
			bSuccess = BindColumn(*(*aParameterColumns)[i], i, (*aParameterRow)[i]);

		if (!bSuccess)
			return FALSE;
	}
	else
	{
		Translate(FALSE);
		if (! aStatus.IsSuccessful())
			return FALSE;
		// solange er nicht wirklich gebraucht wird,
		// einfach erst einmal selbst setzen
		if (aName.Len() == 0)
		{
			aName = 'C';
			aName += String::CreateFromInt32((long)this);
		}
	}

	// Anpassen  der Menge der zu bertragenden Daten
	if (nCurrentMaxFieldLen != nMaxFieldLen)
	{
		nRetcode = OSQLSetStmtOption(aStatementHandle, SQL_MAX_LENGTH, nMaxFieldLen);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState()== aValueChangedState)
				OSQLGetStmtOption(aStatementHandle, SQL_MAX_LENGTH, &nMaxFieldLen);

			nCurrentMaxFieldLen = nMaxFieldLen;
		}
		else
			// Wieder zuruecksetzen
			nMaxFieldLen = nCurrentMaxFieldLen;
	}

	if (nCurrentMaxRows != nMaxRows)
	{
		nRetcode = OSQLSetStmtOption(aStatementHandle, SQL_MAX_ROWS, nMaxRows);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState()== aValueChangedState)
				OSQLGetStmtOption(aStatementHandle, SQL_MAX_ROWS, &nMaxRows);

			nCurrentMaxRows = nMaxRows;
		}
		else
			// Wieder zuruecksetzen
			nMaxRows = nCurrentMaxRows;
	}

	if (IsPrepared())
	{
		// execute kann abgebrochen werden
		BEGIN_CANCELABLE();
		nRetcode = OSQLExecute(aStatementHandle);
		END_CANCELABLE();
	}
	else
	{
		BEGIN_CANCELABLE();
		ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
		nRetcode = OSQLExecDirect(aStatementHandle,
									   (SDB_OTER_CHAR *) aSql.GetBuffer(),
										aSql.Len());
		END_CANCELABLE();
	}

	// Parameter at Execution Time erforderlich?
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue = NULL;// irgendetwas unbestimmtes steht hier nach ParamData drin
		nRetcode = OSQLParamData(aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.is(), "Falsches Parameter Handling!");
		INT32 i(1);
		do
		{
			if (nRetcode != SQL_SUCCESS && nRetcode != SQL_SUCCESS_WITH_INFO && nRetcode != SQL_NEED_DATA)
				break;

			if (aParameterColumns->Count() == 1)
			{
				// Keine Strukturbeschreibung fuer die Parameter Row oder keine Parameter Row.
				// Das darf nicht sein - Statement-Ausfuehrung abbrechen!
				nRetcode = OSQLCancel(aStatementHandle);
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1093"),
							aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_PARAMETER_NUMBER))),
							0, String() );
				// Genaugenommen eigentlich: Parameter missing ...
				return FALSE;
			}

			SdbColumn *pCol = (SdbColumn*)prgbValue;
			if (!PutData(*pCol, (*aParameterRow)[pCol->GetId()]))
			{
				// Bei Fehler Statement-Ausfuehrung abbrechen!
				nRetcode = OSQLCancel(aStatementHandle);
				return FALSE;
			}
			nRetcode = OSQLParamData(aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
	}

	// Resultat des SQLExec[Direct] oder des letzten SqlParamData
	// ist das Resultat des gesamten Statements ...
	SetStatus(nRetcode,aStatementHandle);
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::OnOpen()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	SdbStatus aTempStat(aStatus);

	if (!IsOpen())  		// resultset erst nach execute beschreiben!!!
							// und nur wenn noch nicht geffnet
	{
		SdbOTERConnection* pConnection = GetOTERConnection();
		if (!aResultColumns.Is())
			aResultColumns = pConnection->CreateColumns();
		DescribeResult();
	}

	// Bei Statements, die keine Results liefern (insb. UPDATE, DELETE, INSERT)
	// ist die Anzahl der vom Statement betroffenen Datensaetze interessant:
	if (aResultColumns->Count() == 1)
	{
		SDWORD pcrow=-1;
		RETCODE nRetcode;

		nRetcode = OSQLRowCount(aStatementHandle, &pcrow);
		if (pcrow >= 0)
			SetRowCount((UINT32)pcrow);
	}

	// wenn ich hier schon mit einem nicht erfolgreichen Status reingekommen bin, muss ich das fuer die Basisklasse
	// wiederherstellen
	if (!aTempStat.IsSuccessful())
		aStatus = aTempStat;

	return SdbCursor::OnOpen();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenTables(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
							   const sal_Unicode* pTable, const sal_Unicode* pType)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");


	ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pType		? (aCOL = ByteString(pType,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode;
	nRetcode = OSQLTables(aStatementHandle,
						 (SDB_OTER_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_OTER_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_OTER_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_OTER_CHAR *) pCOL, pType ? SQL_NTS : 0);

	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful())
		return;

	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenColumns(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, const sal_Unicode* pColumn)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pColumn		? (aCOL = ByteString(pColumn,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode;
	nRetcode = OSQLColumns(aStatementHandle,
						  (SDB_OTER_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_OTER_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_OTER_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_OTER_CHAR *) pCOL, pColumn ? SQL_NTS : 0);
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenTypeInfo(INT16 nType)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

	nRetcode = OSQLGetTypeInfo(aStatementHandle, nType);
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenPrimaryKeys(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}


	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode;
	nRetcode = OSQLPrimaryKeys(aStatementHandle,
							   (SDB_OTER_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								   (SDB_OTER_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								   (SDB_OTER_CHAR *) pPKN, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenTablePrivileges(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;
	nRetcode = OSQLTablePrivileges(aStatementHandle,
								   0 , 0,
								   (SDB_OTER_CHAR *) pOwner, pOwner ? SQL_NTS : 0,
								   (SDB_OTER_CHAR *) pTable, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenIndices(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,const sal_Unicode* pTable, BOOL bUnique){
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}


	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;
	RETCODE nRetcode;
	nRetcode = OSQLStatistics(aStatementHandle,
							 (SDB_OTER_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							 (SDB_OTER_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							 (SDB_OTER_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							 bUnique ? SQL_INDEX_UNIQUE : SQL_INDEX_ALL,
							 SQL_QUICK);
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenForeignKeys(const sal_Unicode* pPKQualifier, const sal_Unicode* pPKOwner, const sal_Unicode* pPKName,
									const sal_Unicode* pFKQualifier, const sal_Unicode* pFKOwner, const sal_Unicode* pFKName)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN, aFKQ, aFKO, aFKN;

	const char	*pPKQ = pPKQualifier	? (aPKQ = ByteString(pPKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pPKOwner		? (aPKO = ByteString(pPKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pPKName			? (aPKN = ByteString(pPKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL, 
				*pFKQ = pFKQualifier	? (aFKQ = ByteString(pFKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL, 
				*pFKO = pFKOwner		? (aFKO = ByteString(pFKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL, 
				*pFKN = pFKName			? (aFKN = ByteString(pFKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode;
	nRetcode = OSQLForeignKeys(aStatementHandle,
								(SDB_OTER_CHAR *) pPKQ, pPKQualifier ? SQL_NTS : 0,
								(SDB_OTER_CHAR *) pPKO, pPKOwner ? SQL_NTS : 0,
								(SDB_OTER_CHAR *) pPKN, pPKName ? SQL_NTS : 0,
								(SDB_OTER_CHAR *) pFKQ, pFKQualifier ? SQL_NTS : 0,
								(SDB_OTER_CHAR *) pFKO, pFKOwner ? SQL_NTS : 0,
								(SDB_OTER_CHAR *) pFKN, pFKName ? SQL_NTS : 0
		   );
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenProcedures(const String& rQualifier, const String& rSchema, const String& rProcName)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding());
	RETCODE nRetcode;
	nRetcode = OSQLProcedures(aStatementHandle,
								 0 , 0,
								 (SDB_OTER_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_OTER_CHAR *) aP.GetBuffer(), aP.Len()
		   );
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenProcedureColumns(const String& rQualifier, const String& rSchema,
										 const String& rProcName, const String& rProcColumnName)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding()),
		aC(rProcColumnName,gsl_getSystemTextEncoding());

	RETCODE nRetcode;
	nRetcode = OSQLProcedureColumns(aStatementHandle,
								 0,0,
								 (SDB_OTER_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_OTER_CHAR *) aP.GetBuffer(), aP.Len(),
								 (SDB_OTER_CHAR *) aC.GetBuffer(), aC.Len()
		   );
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::OpenSpecialColumns(const String& rTabName)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ByteString aT(rTabName,gsl_getSystemTextEncoding());

	RETCODE nRetcode = OSQLSpecialColumns(aStatementHandle,SQL_ROWVER,
								 NULL, 0,
								 NULL, 0,
								 (SDB_OTER_CHAR *) aT.GetBuffer(), aT.Len(),
								 SQL_SCOPE_TRANSACTION,SQL_NULLABLE
		   );
	SetStatus(nRetcode,aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}
//-----------------------------------------------------------------------------
void SdbOTERCursor::CancelExecution()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	OSQLCancel(aStatementHandle);
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::CloseImpl()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	// Vater nicht mehr vorhanden, dann alles freigeben
	SdbOTERConnection *pConnection = GetOTERConnection();
	if(pConnection->GetAutoCommitMode()	&& !(nOptions & SDB_READONLY) && !bCommitTrans)
		pConnection->CommitTrans();

	if (!GetParent())
	{
		if (aStatementHandle != SQL_NULL_HSTMT)
		{
			OSQLFreeStmt(aStatementHandle,SQL_RESET_PARAMS);
			OSQLFreeStmt(aStatementHandle,SQL_CLOSE);
			OSQLFreeStmt(aStatementHandle,SQL_DROP);
			aStatementHandle = SQL_NULL_HSTMT;
		}
	}
	else
	{
		DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
		// wenn einer der drei nicht klappt, dann den Handle wegschmeissen

		if (OSQLFreeStmt(aStatementHandle,SQL_RESET_PARAMS) || OSQLFreeStmt(aStatementHandle,SQL_CLOSE))
		{
			SetStatus(OSQLFreeStmt(aStatementHandle,SQL_DROP),aStatementHandle);
			if (aStatus.IsSuccessful())
			{
				RETCODE rRetcode(OSQLAllocStmt(pConnection->ConnectionHandle(),&aStatementHandle));
				SetStatus(rRetcode,aStatementHandle);
			}
			if (!aStatus.IsSuccessful())
				aStatementHandle = SQL_NULL_HSTMT;
		}
	}
	FreeParamData();
	return aStatus.IsSuccessful();
}


//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,SdbCursorMode& rMode)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

//	if (nOptions & SDB_FORWARDONLY)
//	{
		DBG_ASSERT(eCursorPosition == SDB_POS_NEXT, "OTERCursor: Falsche MoveRichtung, nur Next zulaessig!");
		if (eCursorPosition != SDB_POS_NEXT)
			aStatus.SetDriverNotCapableError();
		else
		{
			nRetcode = OSQLFetch(aStatementHandle);
			SetStatus(nRetcode,aStatementHandle);
		}
	if (aStatus.IsSuccessful())
	{
		rMode = SdbCursor::SDB_MOD_NONE;
		FixResultRow();
	}
	return aStatus.IsSuccessful();
}


//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::GetData(const SdbColumn& rCol, const ODbVariantRef& xVar, UINT16 nPos)
{
	 RETCODE nRetcode;
	 SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)


	// Zaehlung der OTER-Columns und Zaehlung unserer Columns beginnt bei eins!
		// Column #0 ist die Bookmark!

#define GETDATA(fCType,rgbValue,cbValueMax) \
	{ \
		nRetcode = OSQLGetData(aStatementHandle,(nPos),(fCType),(rgbValue),(cbValueMax),&pcbValue); \
		if (nRetcode != SQL_SUCCESS)	\
			SetStatus(nRetcode,aStatementHandle);	\
		if (aStatus.IsError())	   \
		{						   \
			DBG_ERROR((((ByteString("SQLGetData fehlgeschlagen!: Column ") +=				\
			ByteString::CreateFromInt32(nPos)) += ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding())).GetBuffer()) ;	\
			xVar->setNull();  \
			aStatus.Clear();	   \
			break;				   \
		}						   \
		else if (pcbValue == SQL_NULL_DATA) { \
			xVar->setNull(); \
			break; \
		} \
	}

	// Gebundene Variable. Je nach Datenbanktyp als entsprechenden
	// C-Datentyp (passend zur Variablen) abholen.

	// nyi: festen Variablentyp beruecksichtigen und konvertieren!
	switch (rCol.GetType())
	{
		case SDB_DBTYPE_BOOKMARK:
			OSQLGetStmtOption(aStatementHandle,	 SQL_GET_BOOKMARK, &aBuffer.nINT32);
			xVar->setINT32(aBuffer.nINT32);
			break;
		case SDB_DBTYPE_CURRENCY:
			GETDATA(SQL_C_CHAR,aBuffer.aCharArray,sizeof aBuffer.aCharArray);
			xVar->setDouble(SdbTools::ToDouble(aBuffer.aCharArray));
			break;
		case SDB_DBTYPE_TINYINT:
		case SDB_DBTYPE_SMALLINT:
			GETDATA(SQL_C_SHORT,&aBuffer.nINT16,sizeof(INT16));
			xVar->setINT16(aBuffer.nINT16);
			// nyi: signed/unsigned
			break;
		case SDB_DBTYPE_INTEGER:
			GETDATA(SQL_C_LONG,&aBuffer.nINT32,sizeof(INT32));
			xVar->setINT32(aBuffer.nINT32);
			// nyi: signed/unsigned
			break;
		case SDB_DBTYPE_BOOLEAN:
			GETDATA(SQL_C_LONG,&aBuffer.nINT8,sizeof(INT8));
			xVar->setBOOL(aBuffer.nINT8);
			break;
		case SDB_DBTYPE_REAL:
			GETDATA(SQL_C_FLOAT,&aBuffer.nFloat,sizeof(float));
			xVar->setFloat(aBuffer.nFloat);
			break;
		case SDB_DBTYPE_DOUBLE:
		case SDB_DBTYPE_BIGINT:
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
			GETDATA(SQL_C_DOUBLE,&aBuffer.nDouble,sizeof(double));
			xVar->setDouble(aBuffer.nDouble);
			break;
		case SDB_DBTYPE_CHAR:
		{
			GETDATA(SQL_C_CHAR,aBuffer.aCharArray,sizeof aBuffer.aCharArray);
			ByteString aString(aBuffer.aCharArray);
			aString.EraseTrailingChars(' ');
			xVar->setString(UniString(aString, GetConnection()->GetCharacterSet()));
			aStatus.Clear();
		}	break;
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_LONGVARCHAR:
		{
			// Erstmal versuchen, die Daten mit dem kleinen Puffer
			// abzuholen:
			SDWORD nMaxLen = sizeof aBuffer.aCharArray - 1;
			GETDATA(SQL_C_BINARY,aBuffer.aCharArray,nMaxLen);
			// Bei Fehler bricht der GETDATA-Makro mit return ab,
			// bei NULL mit break!

			// In pcbValue steht jetzt entweder die Gesamtlaenge
			// der Daten in diesem Feld oder aber SQL_NO_TOTAL (wenn
			// der Treiber uns die Laenge nicht liefern kann).
			// [pcbValue enthaelt immer die (Rest-)Laenge VOR
			// dem letzten Aufruf von GetData!]
			if (pcbValue != SQL_NO_TOTAL && pcbValue <= STRING_MAXLEN)
			{
				// Es handelt sich um einen String, der nicht zu lang fuer einen
				// Sv String ist - also als String speichern.
				SDWORD nBytes = pcbValue < nMaxLen ? pcbValue : nMaxLen;
				aBuffer.aCharArray[nBytes] = 0;

				ByteString aString(aBuffer.aCharArray);
				while (pcbValue > nMaxLen)
				{
					// Bei Strings wird der Puffer nie ganz ausgenutzt
					// (das letzte Byte ist immer ein NULL-Byte, das
					// aber bei pcbValue nicht mitgezaehlt wird)
					if ((pcbValue - nMaxLen) < nMaxLen)
						nBytes = pcbValue - nMaxLen;
					else
						nBytes = nMaxLen;

					// Solange eine "truncation"-Warnung vorliegt, weiter Daten abholen
					GETDATA(SQL_C_BINARY,aBuffer.aCharArray, nBytes);

					// Daten zum String hinzufuegen
					// (Text im Puffer ist nach jedem GetData-Aufruf
					// NULL-terminiert):
					aBuffer.aCharArray[nBytes] = 0;
					aString += aBuffer.aCharArray;
				}
				xVar->setString(UniString(aString, GetConnection()->GetCharacterSet()));
				aStatus.Clear();
			}
			else
			{
				SWORD nBytes = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
				::staruno::Sequence<sal_Int8> aData((sal_Int8*)aBuffer.aCharArray, nBytes);

				while ((pcbValue == SQL_NO_TOTAL && aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
						aStatus.SQLState() == aTruncState) || pcbValue > nMaxLen)
				{
					// Bei Strings wird der Puffer nie ganz ausgenutzt
					// (das letzte Byte ist immer ein NULL-Byte, das
					// aber bei pcbValue nicht mitgezaehlt wird)
					if (pcbValue != SQL_NO_TOTAL && (pcbValue - nMaxLen) < nMaxLen)
						nBytes = pcbValue - nMaxLen;
					else
						nBytes = nMaxLen;

					// Solange eine "truncation"-Warnung vorliegt, weiter Daten abholen
					GETDATA(SQL_C_CHAR,aBuffer.aCharArray, nBytes + 1);
					INT32 nLen = aData.getLength();
					aData.realloc(nLen + nBytes);
					memcpy(aData.getArray() + nLen, aBuffer.aCharArray, nBytes);
				}
				xVar->setBytes(aData);
			}
		} break;
		case SDB_DBTYPE_BINARY:
		case SDB_DBTYPE_VARBINARY:
		case SDB_DBTYPE_LONGVARBINARY:
		{
			// Erstmal versuchen, die Daten mit dem kleinen Puffer
			// abzuholen:
			SDWORD nMaxLen = sizeof aBuffer.aCharArray;
			GETDATA(SQL_C_BINARY,aBuffer.aCharArray,nMaxLen);
			// Bei Fehler bricht der GETDATA-Makro mit return ab,
			// bei NULL mit break!

			// In pcbValue steht jetzt entweder die Gesamtlaenge
			// der Daten in diesem Feld oder aber SQL_NO_TOTAL (wenn
			// der Treiber uns die Laenge nicht liefern kann).
			// [pcbValue enthaelt immer die (Rest-)Laenge VOR
			// dem letzten Aufruf von GetData!]

			// Es handelt sich um Binaerdaten, um einen String, der fuer
			// StarView zu lang ist oder der Treiber kann die Laenge der
			// Daten nicht im voraus bestimmen - also als MemoryStream
			// speichern.


			SDWORD nBytes = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
			::staruno::Sequence<sal_Int8> aData((sal_Int8*)aBuffer.aCharArray, nBytes);

			if(pcbValue > 1024*nMaxLen)
				nMaxLen *= 1024;
			else if(pcbValue > 512*nMaxLen)
				nMaxLen *= 512;
			else if(pcbValue > 256*nMaxLen)
				nMaxLen *= 256;
			else if(pcbValue > 128*nMaxLen)
				nMaxLen *= 128;
			else if(pcbValue > 64*nMaxLen)
				nMaxLen *= 64;
			else if(pcbValue > 32*nMaxLen)
				nMaxLen *= 32;

			char* pData = new char[nMaxLen];

			// Invariante der folgenden Schleife : nBytes gibt an, wieviel im letzten Durchlauf schon transferiert
			// wurde (also mittels GETDATA gelesen und in den MemoryStream geschrieben), pcbValue - wenn ungleich
			// SQL_NO_TOTAL - gibt an, wieviel zu diesem Zeitpunkt moch zur Verfuegung stand (_inklusive_ von
			// nBytes !).
			// Das gilt auch vor dem ersten Durchlauf, da diese Bedingungen dann durch das obige GETDATA hergestellt
			// wurden.
			while ((pcbValue == SQL_NO_TOTAL && aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
					aStatus.SQLState() == aTruncState) || pcbValue > nBytes)
			{
				// Bei Strings wird der Puffer nie ganz ausgenutzt
				// (das letzte Byte ist immer ein NULL-Byte, das
				// aber bei pcbValue nicht mitgezaehlt wird)
				if (pcbValue != SQL_NO_TOTAL && (pcbValue - nBytes) < nMaxLen)
					nBytes = pcbValue - nBytes;
				else
					nBytes = nMaxLen;

				GETDATA(SQL_C_BINARY,pData,nBytes);

				INT32 nLen = aData.getLength();
				aData.realloc(nLen + nBytes);
				memcpy(aData.getArray() + nLen, pData, nBytes);
			}
			delete pData;
			xVar->setBytes(aData);
		} break;

		case SDB_DBTYPE_DATE:
		{
			GETDATA(SQL_C_DATE,&aBuffer.aDate,sizeof(aBuffer.aDate));
			xVar->setDate(ToDate(aBuffer.aDate));
		}	break;
		case SDB_DBTYPE_TIME:
		{
			GETDATA(SQL_C_TIME,&aBuffer.aTime,sizeof(aBuffer.aTime));
			xVar->setTime(ToTime(aBuffer.aTime));
		}	break;
		case SDB_DBTYPE_TIMESTAMP:
		{
			GETDATA(SQL_C_TIMESTAMP,&aBuffer.aTimestamp,sizeof(aBuffer.aTimestamp));
			xVar->setDateTime(ToDateTime(aBuffer.aTimestamp));
		} break;
		default:
			DBG_ERROR("SdbOTERCursor: interner Fehler: unbekannter SDB-Datentyp"); break; // Unbekannter Datenbanktyp
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::PutData(const SdbColumn& rCol, const ODbVariantRef& xVar)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	DBG_ASSERT(aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

#define PUTDATA(rgbValue,cbValue) \
	{ \
		nRetcode = OSQLPutData(aStatementHandle,(rgbValue),(cbValue)); \
		if (nRetcode) return FALSE; \
	}

	switch (rCol.GetType())
	{
		case SDB_DBTYPE_LONGVARCHAR:
		case SDB_DBTYPE_LONGVARBINARY:
			if (xVar->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
			{
				const ::staruno::Sequence<sal_Int8>& rData = xVar->getBytes();
				PUTDATA((void *)rData.getConstArray(),rData.getLength());
			}
			else
			{
				ByteString aString;
				if (!xVar->isNull())
					aString = ByteString(ODbTypeConversion::toString(*xVar).getStr(), GetConnection()->GetCharacterSet());
				PUTDATA((void *)aString.GetBuffer(),aString.Len());
			}
			break;
	}
	return TRUE;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::DeleteImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::InsertImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::UpdateImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::FixResultRow()
{
	DBG_ASSERT(aResultRow.is(), "Keine Ergebniszeile vorhanden!!");
	DBG_CHKTHIS(SdbOTERCursor,NULL);

	for (UINT16 i=1; i < nRowVerLen; i++) // Wird BOOKMARK-Spalte verwendet
	{
		const ODbVariantRef& xVar = (*aResultRow)[i];
		if (xVar.is())
			GetData(*(*aResultColumns)[i], xVar, i);
	}
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::AllocParamData()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	UINT16 nCount(aParameterColumns->Count());
	if (nCount <= 1)		// keine columns
		return;

	DBG_ASSERT(pParamBuffer == NULL,"Interner Fehler");
	pParamBuffer = (void**) new char*[nCount];
	pParamLength = new INT32[nCount];
	memset(pParamLength, 0, nCount * sizeof(INT32));
	memset(pParamBuffer, 0, nCount * sizeof(char*));

	for (UINT16 i=1; i < nCount; i++)
	{
		const SdbColumn& rCol	= *(*aParameterColumns)[i];
		INT32& rLength			= pParamLength[i];
		rLength					= rCol.GetLength();

		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				pParamBuffer[i] = new ByteString;
				break;
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
				rLength += 2;
				pParamBuffer[i] = new ByteString;
				break;
			case SDB_DBTYPE_DOUBLE:
				rLength = sizeof(double);
				pParamBuffer[i] = new double;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:	// Daten werden ueber GetData geholt
				rLength = 0;
				pParamBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_DATE:
				rLength = sizeof(DATE_STRUCT);
				pParamBuffer[i] = new DATE_STRUCT;
				break;
			case SDB_DBTYPE_TIME:
				rLength = sizeof(TIME_STRUCT);
				pParamBuffer[i] = new TIME_STRUCT;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				rLength = sizeof(TIMESTAMP_STRUCT);
				pParamBuffer[i] = new TIMESTAMP_STRUCT;
				break;
			case SDB_DBTYPE_BOOLEAN:
				rLength = sizeof(INT8);
				pParamBuffer[i] = new INT8;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				rLength = sizeof(INT16);
				pParamBuffer[i] = new INT16;
				break;
			case SDB_DBTYPE_INTEGER:
				rLength = sizeof(INT32);
				pParamBuffer[i] = new INT32;
				break;
			case SDB_DBTYPE_BOOKMARK:
				pParamBuffer[i] = new INT32;
				break;
			case SDB_DBTYPE_REAL:
				rLength = sizeof(float);
				pParamBuffer[i] = new float;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				pParamBuffer[i] = new BYTE[rLength];
				break;
		}
	}
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::FreeParamData()
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	UINT16 nCount = aParameterColumns.Is() ? aParameterColumns->Count() : 0;
	if (nCount <= 1) return;

	delete pParamLength;
	pParamLength = NULL;

	if (pParamBuffer == NULL)
		return;

	for (UINT16 i = 0; i < nCount; i++)
	{
		const SdbColumn& rCol = *(*aParameterColumns)[i];
		void* pData = pParamBuffer[i];
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				delete (ByteString*)pData;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:
				delete (char*)pData;
				break;
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_BOOKMARK:
				delete (INT32*)pData;
				break;
			case SDB_DBTYPE_BOOLEAN:
				delete (INT8*)pData;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				delete (INT16*)pData;
				break;
			case SDB_DBTYPE_REAL:
				delete (float*)pData;
				break;
			case SDB_DBTYPE_DOUBLE:
				delete (double*)pData;
				break;
			case SDB_DBTYPE_DATE:
				delete (DATE_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIME:
				delete (TIME_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				delete (TIMESTAMP_STRUCT*)pData;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				delete (BYTE *) pData;
				break;
			default:
				DBG_ERROR("Fehlerhafter DatenTyp");
		}
	}

	__DELETE(nCount) pParamBuffer;
	pParamBuffer = NULL;
}


/*
//-----------------------------------------------------------------------------
void SdbOTERCursor::AllocBufferData(void**& pBuffer,INT32*& pLength,SdbColumnsRef aColumns)
{
	DBG_CHKTHIS(SdbOTERCursor,NULL);
	UINT16 nCount(aColumns->Count());
	if (nCount <= 1)		// keine columns
		return;

	DBG_ASSERT(pBuffer == NULL,"Interner Fehler");
	pBuffer = (void**) new char*[nCount];
	pLength = new INT32[nCount];
	memset(pLength, 0, nCount * sizeof(INT32));
	memset(pBuffer, 0, nCount * sizeof(char*));

	for (UINT16 i=1; i < nCount; i++) // Wird BOOKMARK-Spalte verwendet
	{
		const SdbColumn& rCol	= *(*aColumns)[i];
		INT32& rLength			= pLength[i];
		rLength = rCol.GetLength();

		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				pBuffer[i] = new char[rLength + 1];
				break;
			case SDB_DBTYPE_CURRENCY:
				rLength += 2;
				pBuffer[i] = new char[rLength + 1];
				break;
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
				//rLength += 2;
				pBuffer[i] = new double;//char[rLength + 1];
				break;
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:	// Daten werden ueber GetData geholt
				rLength = 0;
				pBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_DATE:
				pBuffer[i] = new DATE_STRUCT;	// ... OTER-Datums-/Zeitstruktur
				break;
			case SDB_DBTYPE_TIME:
				pBuffer[i] = new TIME_STRUCT;	// ... OTER-Datums-/Zeitstruktur
				break;
			case SDB_DBTYPE_TIMESTAMP:
				pBuffer[i] = new TIMESTAMP_STRUCT;	// ... OTER-Datums-/Zeitstruktur
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				pBuffer[i] = new INT16;
				break;
			case SDB_DBTYPE_INTEGER:
				pBuffer[i] = new INT32;
				break;
			case SDB_DBTYPE_BOOLEAN:
				pBuffer[i] = new INT8;
				break;
			case SDB_DBTYPE_BOOKMARK:
				pBuffer[i] = new INT32;
				break;
			case SDB_DBTYPE_REAL:
				pBuffer[i] = new float;
				break;
			case SDB_DBTYPE_FLOAT:
			case SDB_DBTYPE_DOUBLE:
				pBuffer[i] = new double;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				pBuffer[i] = new BYTE[rLength];
				break;
			default:
				pBuffer[i] = new char[2];  // dient nur zum auffinden
		}
	}
}

//-----------------------------------------------------------------------------
void SdbOTERCursor::FreeBufferData(void**& pBuffer,INT32*& pLength,SdbColumnsRef aColumns)
{
	UINT16 nCount(aColumns.Is() ? aColumns->Count() : 0);
	if (nCount <= 1) return;

	delete pLength;
	pLength = NULL;

	if(!pBuffer)
		return;

	for (UINT16 i=0; i < nCount; i++)
	{
		void* pData = pBuffer[i];
		if(i==nRowVer || !pData)
			continue;
		const SdbColumn& rCol = *(*aColumns)[i];

		switch (rCol.GetType())
		{
			default:
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:	// Daten werden ueber GetData geholt
				delete (char*)pData;
				break;
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_BOOKMARK:
				delete (INT32*)pData;
				break;
			case SDB_DBTYPE_BOOLEAN:
				delete (INT8*)pData;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				delete (INT16*)pData;
				break;
			case SDB_DBTYPE_REAL:
				delete (float*)pData;
				break;
			case SDB_DBTYPE_FLOAT:
			case SDB_DBTYPE_DOUBLE:

			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
				delete (double*)pData;
				break;
			case SDB_DBTYPE_DATE:
				delete (DATE_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIME:
				delete (TIME_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				delete (TIMESTAMP_STRUCT*)pData;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				delete (BYTE *) pData;
				break;
		}
	}

	__DELETE(nCount) pBuffer;
	pBuffer = NULL;
}*/

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::DetermineRowCountImpl(INT32&)
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERCursor::ExecuteDirect(const String &rExeSql)
{
	RETCODE nRetcode;

	BEGIN_CANCELABLE();
	ByteString aSql(rExeSql,gsl_getSystemTextEncoding());
	nRetcode = OSQLExecDirect(aStatementHandle,(SDB_OTER_CHAR *) aSql.GetBuffer(),aSql.Len());
	END_CANCELABLE();

	SetStatus(nRetcode, aStatementHandle);
	return aStatus.IsSuccessful();
}
//------------------------------------------------------------------------
BOOL SdbOTERCursor::IsCapable(SdbCursorCapability eCapability) const
{
	BOOL bResult=FALSE;
	switch(eCapability)
	{
		case SDB_CAP_FETCH:
		case SDB_CAP_FETCH_NEXT:
		case SDB_CAP_FETCH_PRIOR:
		case SDB_CAP_FETCH_FIRST:
		case SDB_CAP_FETCH_LAST:
		case SDB_CAP_FETCH_RELATIVE:
		case SDB_CAP_FETCH_ABSOLUTE:
			bResult = TRUE;
	}
	return bResult;
}

//------------------------------------------------------------------------
void SdbOTERCursor::PrepareStatement(SdbConnection* pConnection, SdbSqlParseNode* pParseNode, String & rString)
{
	// Date conversion
	if (SDB_ISRULE(pParseNode,set_fct_spec) && SDB_ISPUNCTUATION(pParseNode->GetChild(0),"{"))
	{
		const SdbSqlParseNode* pODBCNode = pParseNode->GetChild(1);
		const SdbSqlParseNode* pODBCNodeChild = pODBCNode->GetChild(0);

		if (pODBCNodeChild->GetNodeType() == SDB_SQL_KEYWORD && (
			SDB_ISTOKEN(pODBCNodeChild,"D") ||
			SDB_ISTOKEN(pODBCNodeChild,"T") ||
			SDB_ISTOKEN(pODBCNodeChild,"TS") ))
		{
			String aString(pODBCNode->GetChild(1)->TokenValue());
			delete pParseNode->Parent()->Replace(new SdbSqlParseNode(aString,SDB_SQL_STRING), pParseNode);
		}
		return;
	}
	else
		SdbCursor::PrepareStatement(pConnection, pParseNode, rString);
}
//------------------------------------------------------------------------
void SdbOTERCursor::DriverTranslate()
{
	if(SDB_ISRULE(pSQLParseNode,select_statement))
	{
		SdbSqlParseNode* pTableExp = pSQLParseNode->GetChild(3);
		SdbSqlParseNode* pTableRefCommalist = pTableExp->GetChild(0)->GetChild(1);
		for (ULONG i = 0; i < pTableRefCommalist->Count(); i++)
		{
			SdbSqlParseNode* pParseNode = pTableRefCommalist->GetChild(i);
			if(pParseNode->Count() == 4 && SDB_ISPUNCTUATION(pParseNode->GetChild(0),"{") && SDB_ISRULE(pParseNode,table_ref))
			{
				SdbSqlParseNode* pOJ = pParseNode->Remove(2);
				pTableRefCommalist->Replace(pOJ,pParseNode);
				delete pParseNode;
			}
		}
	}
	aSQLStatement.Erase();
	pSQLParseNode->ParseNodeToStr(aSQLStatement, GetOTERConnection()->GetMetaData());
}


