/*************************************************************************
 *
 *  $RCSfile: oteridx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbdbimp.hxx"
#ifndef _SDB_OTERIDX_HXX
#include "oteridx.hxx"
#endif

#ifndef _SDB_OTERTAB_HXX
#include "otertab.hxx"
#endif

#ifndef _SDB_OTERCURS_HXX
#include "otercurs.hxx"
#endif

#ifndef _SDB_OTERCONN_HXX
#include "oterconn.hxx"
#endif

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASETABLEDEFINITION_HPP_
#include <com/sun/star/data/DatabaseTableDefinition.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASESQLCONFORMANCE_HPP_
#include <com/sun/star/data/DatabaseSQLConformance.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

#include "sdbresid.hrc"

DBG_NAME(SdbOTERIndex);

//-----------------------------------------------------------------------------
SdbOTERIndex::SdbOTERIndex(SdbOTERTable* pRefTable)
			 :SdbIndex(pRefTable){}


#define TABLE_QUALIFIER  	1
#define TABLE_OWNER      	2
#define TABLE_NAME       	3
#define NON_UNIQUE       	4
#define INDEX_QUALIFIER  	5
#define INDEX_NAME       	6
#define INDEX_TYPE         	7
#define SEQ_IN_INDEX     	8
#define COLUMN_NAME      	9
#define COLLATION	     	10
//-----------------------------------------------------------------------------
BOOL SdbOTERIndex::OpenImpl()
{
	SdbTable* pTable = GetTable();
	SdbOTERCursor* pCursor =
		(SdbOTERCursor*)pTable->GetConnection()->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY|SDB_READONLY);
	pCursor->AddRef();

	const sal_Unicode* pQualifier = NULL;
	const sal_Unicode* pOwner = NULL;
	if (pTable->OwnerName().Len())
		pOwner = pTable->OwnerName().GetBuffer();

	BOOL bCase = (pTable->GetConnection()->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;

	pCursor->OpenIndices(pQualifier,pOwner,pTable->Name().GetBuffer());
	aStatus = pCursor->Status();
	UniString usQual(aQualifierName);
	UniString usName(aName);
	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();

		ODbVariantRef xQualifier = rRow[INDEX_QUALIFIER];
		ODbVariantRef xName		 = rRow[INDEX_NAME];
		ODbVariantRef xType		 = rRow[INDEX_TYPE];

		if (!xType->isNull() && xType->toINT16() != SQL_TABLE_STAT &&
			(xQualifier->isNull() || ( bCase ? xQualifier->getString() == rtl::OUString(usQual): xQualifier->getString().equalsIgnoreCase(usQual)))
			&& !xName->isNull() && ( bCase ? xName->getString() == rtl::OUString(usName) : xName->getString().equalsIgnoreCase(usName)))
		{
			ODbVariantRef xValue = rRow[NON_UNIQUE];
			if (!xValue->isNull() && xValue->toINT16() == 0)
				SetStyle(SDB_IDX_UNIQUE);

			xValue = rRow[COLUMN_NAME];
			const SdbColumn* pColumn = NULL;
			if (!xValue->isNull())
				pColumn = pTable->OriginalColumns()->Column(xValue->getString().getStr());

			if (pColumn)
			{
				SdbColumn* pNewCol = pTable->CreateColumn(*pColumn);
				aOriginalColumns->AddColumn(pNewCol);
				xValue = rRow[COLLATION];
				if (!xValue->isNull() && !xValue->getString().compareToAscii("D"))
					pNewCol->SetFlag(SDB_FLAGS_DESCENDING);
			}
			else
			{
				String aText(SdbResId(STR_STAT_INDEX_COLUMN_NOT_FOUND));
				aText.SearchAndReplace(String::CreateFromAscii("#"),xValue->getString().getStr());
				aText.SearchAndReplace(String::CreateFromAscii("%"),pTable->GetFullName());
				aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
				break;
			}
		}
		pCursor->Next();
	}

	bSystemReadOnly = aName.GetChar(0) == '#';

	pCursor->ReleaseRef();
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERIndex::DropImpl()
{
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	SdbTable* pTable = GetTable();
//	SdbConnection* pConnection = GetConnection();
//
//	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
//	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;
//
//	String aStatement("DROP INDEX ");
//	if (pTable->OwnerName().Len() && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
//	{
//		aStatement += pConnection->IdentifierQuotation(pTable->OwnerName());
//		aStatement += '.';
//	}
//	aStatement += aName;
//	if (!pConnection->ExecuteSQL(aStatement))
//	{
//		String aText(SdbResId(STR_STAT_OTER_INDEX_NOTDELETED));
//		aText.SearchAndReplace("#",aName);
//		aStatus.Set(SDB_STAT_ERROR,
//					"01000",
//					aStatus.CreateErrorMessage(aText),
//					0, String() );
//	}
//	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERIndex::CreateImpl()
{
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	DBG_ASSERT(aColumns->Count() > 1, String());
//
//	SdbTable* pTable = GetTable();
//	SdbOTERConnection* pConnection = (SdbOTERConnection*)GetConnection();
//
//	if(pConnection->IsTransactOn() && pConnection->IsMultiUserOn())
//		return FALSE;
//
//	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
//	String aStatement("CREATE ");
//	if (IsUnique())
//		aStatement += "UNIQUE ";
//
//	aStatement += "INDEX ";
//
//	aStatement += pConnection->IdentifierQuotation(aName);
//	aStatement += " ON ";
//	aStatement += pConnection->IdentifierQuotation(pTable->QualifiedName());
//
//	aStatement += '(';
//	for (UINT16 i = 1; i < aColumns->Count(); i++)
//	{
//		aStatement += pConnection->IdentifierQuotation((*aColumns)[i]->GetName());
//		if ((*aColumns)[i]->IsDescending())
//			aStatement += " DESC ";
//		aStatement += ',';
//	}
//
//	aStatement.EraseTrailingChars(',');
//	aStatement += ')';
//
//	if (!pConnection->ExecuteSQL(aStatement))
//		aStatus = pConnection->Status();
//
//	return aStatus.IsSuccessful();
}


