/*************************************************************************
 *
 *  $RCSfile: oterstat.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "oterstat.hxx"

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

//-----------------------------------------------------------------------------
void SdbOTER_SetStatus(SdbStatus & rStatus, const RETCODE nRetcode,
				   /* const */ HENV henv, /* const */ HDBC hdbc, /* const */ HSTMT hstmt,
				   const String &rErrorInfo)
{
	SdbStatusCode eResult;
	BOOL bInformationAvailable = FALSE;

	switch(nRetcode)
	{
		case SQL_STILL_EXECUTING:
		case SQL_SUCCESS:			eResult = SDB_STAT_SUCCESS; break;
		case SQL_NO_DATA_FOUND:		eResult = SDB_STAT_NO_DATA_FOUND; break;

		case SQL_SUCCESS_WITH_INFO: eResult = SDB_STAT_SUCCESS_WITH_INFO;
									bInformationAvailable = TRUE;
									break;
		case SQL_ERROR:             eResult = SDB_STAT_ERROR;
									bInformationAvailable = TRUE;
									break;

		case SQL_NEED_DATA:		    return;
		case SQL_INVALID_HANDLE:    DBG_ERROR("SdbOTER_SetStatus: SQL_INVALID_HANDLE"); break;
	}


	if (bInformationAvailable) {
		// Zusaetliche Informationen zum letzten OTER-Funktionsaufruf vorhanden.
		// SQLError liefert diese Informationen.

		SDB_OTER_CHAR szSqlState[5];
		SDWORD pfNativeError;
		SDB_OTER_CHAR szErrorMessage[SQL_MAX_MESSAGE_LENGTH];
		SWORD pcbErrorMsg;

		// Informationen zur letzten Operation:
		// wenn hstmt != SQL_NULL_HSTMT ist (Benutzung von SetStatus in SdbCursor, SdbTable, ...),
		// dann wird der Status des letzten Statements erfragt, sonst der Status des letzten
		// Statements zu dieser Verbindung [was in unserem Fall wahrscheinlich gleichbedeutend ist,
		// aber das Reference Manual drueckt sich da nicht so klar aus ...].
		// Entsprechend bei hdbc.
		RETCODE n = OSQLError(henv,hdbc,hstmt,szSqlState,&pfNativeError,szErrorMessage,sizeof szErrorMessage - 1,&pcbErrorMsg);
		DBG_ASSERT(n != SQL_INVALID_HANDLE,"StarBase_SetStatus: SQLError returned SQL_INVALID_HANDLE");
		DBG_ASSERT(n == SQL_SUCCESS || n == SQL_SUCCESS_WITH_INFO || n == SQL_NO_DATA_FOUND || n == SQL_ERROR,"StarBase_SetStatus: SQLError failed");

		// Zum Return Code von SQLError siehe OTER 2.0 Programmer's Reference Seite 287ff
		if (n == SQL_SUCCESS || n == SQL_SUCCESS_WITH_INFO) {
			String aStrTmp; aStrTmp.AssignAscii((char *)szErrorMessage);
			aStrTmp.SearchAndReplace(String::CreateFromAscii("Microrim"),String::CreateFromAscii("Sun Microsystems"));
			aStrTmp.SearchAndReplace(String::CreateFromAscii("Oterro"),String::CreateFromAscii("StarBase"));
			aStrTmp.SearchAndReplace(String::CreateFromAscii("ODBC Driver"),String());
			rStatus.Set(eResult, String::CreateFromAscii((char *)szSqlState), aStrTmp, pfNativeError, rErrorInfo);
		} else if (n == SQL_NO_DATA_FOUND) {
			rStatus.Set(eResult, String::CreateFromAscii((char *)szSqlState), String(), 0, rErrorInfo);
		} else {
			rStatus.Set(eResult);
		}
	} else {
		// Keine zusaetzlichen Informationen. Einfach eResult setzen.

		rStatus.Set(eResult);
	}

}

