/*************************************************************************
 *
 *  $RCSfile: otertab.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include "otertab.hxx"
#include "otercurs.hxx"
#include "oterconn.hxx"
#include "oteridx.hxx"
#include "sdbresid.hrc"

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _ZFORMAT_HXX //autogen
#include <svtools/zformat.hxx>
#endif
#ifndef _SOLMATH_HXX //autogen wg. SolarMath
#include <tools/solmath.hxx>
#endif

#ifndef _SDB_SDBDB_HXX //autogen
#include "sdbdb.hxx"
#endif

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _SDB_OTERMD_HXX
#include <otermd.hxx>
#endif

#ifndef _COM_SUN_STAR_DATA_DATABASETABLEDEFINITION_HPP_
#include <com/sun/star/data/DatabaseTableDefinition.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASESQLCONFORMANCE_HPP_
#include <com/sun/star/data/DatabaseSQLConformance.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASENKEYRULE_HPP_
#include <com/sun/star/data/DatabaseKeyRule.hpp>
#endif

static char __READONLY_DATA privilegeSelect[] = "SELECT";
static char __READONLY_DATA privilegeInsert[] = "INSERT";
static char __READONLY_DATA privilegeUpdate[] = "UPDATE";
static char __READONLY_DATA privilegeDelete[] = "DELETE";
static char __READONLY_DATA privilegeReferences[] = "REFERENCES";

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

DBG_NAME(SdbOTERTable);

//-----------------------------------------------------------------------------
SdbOTERTable::SdbOTERTable(SdbOTERConnection* pConnection)
			 :SdbTable(pConnection)
{
	DBG_CTOR(SdbOTERTable,NULL);
	aStatus.SetComponent(String::CreateFromAscii("StarBase Driver"));
}
//-----------------------------------------------------------------------------
SdbOTERTable::~SdbOTERTable()
{
	DBG_DTOR(SdbOTERTable,NULL);
}
//-----------------------------------------------------------------------------
BOOL SdbOTERTable::OpenImpl()
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	// Hier ist bekannt, dass es sich eigentlich um einen
	// SdbOTERCursor handelt, daher Type Cast:
	SdbOTERConnection* pConnection = GetOTERConnection();
	SdbOTERCursor* pCursor = (SdbOTERCursor *) pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();
	aStatus = pCursor->Status();

	BOOL bCase = (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;

	if (!aStatus.IsSuccessful())
	{
		pCursor->ReleaseRef();
		return FALSE;
	}

	const sal_Unicode* pOwner = NULL;

	// Handelt es sich um eine Tabelle oder View
	pCursor->OpenTables(0,pOwner,aName.GetBuffer(), String::CreateFromAscii("VIEW").GetBuffer());
	bView = pCursor->IsInRange();
	pCursor->Close();

	// Angaben zu den Columns:
	// Einen ganz speziellen Cursor oeffnen: anstelle von
	// Prepare und Execute wird Columns ausgefuehrt; die
	// Ergebnismenge enthaelt die gewuenschten Informationen
	pCursor->OpenColumns(0,pOwner,aName.GetBuffer(), NULL);
	aStatus = pCursor->Status();
	if (!aStatus.IsSuccessful())
	{
		if (aStatus.IsNoDataFound())	// Tabelle existiert nicht
			aStatus.Clear();
		pCursor->ReleaseRef();
		return FALSE;
	}

	// db2 verwendet immer gleiche Reihenfolge
/*		1 	TABLE_QUALIFIER
		2	TABLE_OWNER
		3	TABLE_NAME
		4	COLUMN_NAME
		5 	DATA_TYPE
		6 	TYPE_NAME
		7 	PRECISION
		8 	LENGTH
		9 	SCALE
		10 	RADIX
		11 	NULLABLE
		12 	REMARKS		*/

	DBG_ASSERT(aOriginalColumns->Count() == 1,"SdbOTERTable::aOriginalColumns nicht >1 sein!");
	INT32 i = 1;
	INT32 nCo=1;
	while (!pCursor->IsOffRange())
	{
		ODbRow& rRow = *pCursor->GetRow();

		// Column-Info aufbauen ...
		String aColumnName = rRow[4]->getString().getStr();
		xub_StrLen nPoint(aColumnName.Search('.'));
		if(nPoint != STRING_NOTFOUND)
			aColumnName = aColumnName.Copy(nPoint+1);

 		SdbDatabaseType eType(SdbOTERConnection::MapType(rRow[5]->toINT16()));
		UINT32 nLength (0);
		UINT16 nScale (0);
		UINT16 nFlags (0);
		// Sonderbehandlung fuer Currency ,Autoincrement
		const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rRow[6]->getString().getStr());
		DBG_ASSERT(pInfo, "SdbOTERTable keine TypeInfo");

		ODbVariantRef xValue = rRow[7];
		if (!xValue->isNull())
			nLength = (UINT32)xValue->toINT32();
		if((eType == SDB_DBTYPE_BINARY || eType == SDB_DBTYPE_VARBINARY) && nLength > 4)
			nLength -= 4; // Oterro Anhang
		xValue = rRow[9];
		if (!xValue->isNull())
			nScale = (UINT16)xValue->toINT16();

		xValue = rRow[11];
		if (!xValue->isNull() && xValue->toINT16() != SQL_NO_NULLS)
			nFlags |= SDB_FLAGS_NULLALLOWED;

		// koennte AutoIncrement sein
		if(pConnection->IsAutoNum(aName,aColumnName))
		{
			switch(eType)
			{
				case SDB_DBTYPE_DECIMAL:
				case SDB_DBTYPE_NUMERIC:
				case SDB_DBTYPE_TINYINT:
				case SDB_DBTYPE_SMALLINT:
				case SDB_DBTYPE_INTEGER:
				case SDB_DBTYPE_BIGINT:
				case SDB_DBTYPE_REAL:
				case SDB_DBTYPE_DOUBLE:
					nFlags |= SDB_FLAGS_AUTOINCREMENT;
					nFlags &= ~SDB_FLAGS_NULLALLOWED;
					break;
				default:
					DBG_ASSERT(0,"Datenbank ist korrupt! Ein Feld wird Autoinkrement angenommen, das keines sein kann!");
			}
		}
		// Zusaetzliche Flags:
		// bei unbekannten Typen ist pInfo NULL, z.B. benutzerdefinierten
		if (pInfo)
		{
			if (pInfo && pInfo->bCurrency)
				eType = SDB_DBTYPE_CURRENCY;
			switch (pInfo->eSearchType)
			{
				case Search_Like_Only  : nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
				case Search_Except_Like: nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				case Search_All:		 nFlags |= SDB_FLAGS_SEARCH_ALL; break;
			}
		}else // keine TypeInfo dann nullable
			nFlags |= SDB_FLAGS_NULLALLOWED;

		// feststellen ob die Column ein foreign key ist
		SdbOTERCursor* pCursorFKey = (SdbOTERCursor *) pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
		pCursorFKey->AddRef();
		String aFKeySql;
		aFKeySql.AssignAscii("SELECT DISTINCT c.sys_foreign_key FROM sys_tables b,sys_indexes c WHERE c.sys_column_name = '");
		aFKeySql += aColumnName;
		aFKeySql.AppendAscii("' AND b.sys_table_name = '");
		aFKeySql += aName;
		aFKeySql += String::CreateFromAscii("' AND NOT ( c.sys_foreign_key = 0)");
		pCursorFKey->Open(aFKeySql);
		 // einmal reicht, um zu wissen das es sich um ein AutoIncremment handelt
		if(pCursorFKey->IsInRange())
			nFlags |= SDB_FLAGS_FOREIGNKEY;
		pCursorFKey->Close();
		pCursorFKey->ReleaseRef();

		if(aOriginalColumns->Column(aColumnName))
		{
			aColumnName = String::CreateFromAscii("Expr");
			aColumnName += String::CreateFromInt32(nCo++);// um einen eindeutigen Namen zu finden
		}
		SdbColumn* pColumn = CreateColumn(aColumnName,eType,nLength,nScale,nFlags);

		xValue = rRow[12];
		if (!xValue->isNull())
			pColumn->SetComment(xValue->getString().getStr());

		if(aColumnName.EqualsIgnoreCaseAscii("SYS_ROWVER"))
			pColumn->SetFlag(SDB_FLAGS_VERSIONING);
		// ... und in Columns-Struktur einfuegen
		aOriginalColumns->AddColumn(pColumn);
		pCursor->Next();
	}
	pCursor->Close();
	// Unique felder finden
	pCursor->OpenIndices(0,pOwner,aName.GetBuffer(),TRUE);
	String aIndexName,aIndexQualifier;

	/*	Reihenfolge */
	#define TABLE_QUALIFIER	1
	#define TABLE_OWNER    	2
	#define TABLE_NAME     	3
	#define NON_UNIQUE     	4
	#define INDEX_QUALIFIER	5
	#define INDEX_NAME     	6
	#define INDEXTYPE      	7
	#define SEQ_IN_INDEX   	8
	#define COLUMN_NAME    	9

	//	BOOL bCase = (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;
	// setzen der uniqueflags
	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		if (rRow[INDEXTYPE]->toINT16() != SQL_TABLE_STAT)
		{
			SdbColumn* pColumn = NULL;
			ODbVariantRef xValue = rRow[INDEX_QUALIFIER];
			String aNewName(rRow[INDEX_NAME]->getString().getStr());

			if ((bCase ? aIndexName == aNewName : aIndexName.EqualsIgnoreCaseAscii(aNewName)) &&
				(xValue->isNull() ||
				(bCase ? xValue->getString().getStr() == aIndexQualifier : xValue->getString().equalsIgnoreCase(aIndexQualifier))))
				// naechste Spalte des aktuellen Index
				pColumn = (SdbColumn*)(*aOriginalColumns)[rRow[COLUMN_NAME]->getString().getStr()];
			else	// neuer Index
			{
				if (rRow[NON_UNIQUE]->toINT16() == 0)
				{
					aIndexName = aNewName;
					if (!xValue->isNull())
						aIndexQualifier = xValue->getString().getStr();
					pColumn = (SdbColumn*)(*aOriginalColumns)[rRow[COLUMN_NAME]->getString().getStr()];
				}
			}
			if (pColumn)
				pColumn->SetFlag(SDB_FLAGS_UNIQUE);
		}
		pCursor->Next();
	}
	pCursor->Close();
	// Primarykeys auslesen (Funktion muss nicht unbedingt verfuegbar sein)
	// daher wird auch der Fehlerstatus nicht bercksichtigt
	String aPkSql;
	aPkSql.AssignAscii("select a.SYS_COLUMN_NAME, d.SYS_INDEX_NAME from SYS_INDEXES d, SYS_COLUMNS a, SYS_TABLES b where b.SYS_TABLE_NAME like '");
	aPkSql += aName;
	aPkSql.AppendAscii("' and b.SYS_TABLE_ID = a.SYS_TABLE_ID and b.SYS_TABLE_ID = d.SYS_TABLE_ID and a.SYS_COLUMN_ID = d.SYS_COLUMN_ID and d.SYS_PRIMARY_KEY = 1");
	pCursor->Open(aPkSql);

	// setzen der uniqueflags
	if (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		// Name des PKeys
		ODbVariantRef xValue = (*pCursor->GetRow())[2];
		if (!xValue->isNull())
			aPKeyName = xValue->getString().getStr();
	}

	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbVariantRef xValue = (*pCursor->GetRow())[1];
		SdbColumn* pColumn = (SdbColumn*)aOriginalColumns->Column(xValue->getString().getStr());
		if (pColumn)
			pColumn->SetFlag(SDB_FLAGS_PRIMARYKEY);
		pCursor->Next();
	}

	pCursor->Close();

	// setzen der Privileges
	nPrivileges = 0;
	if (!pConnection->IsCapable(SDB_CAP_UPDATABLE))
		nPrivileges = SDB_PR_READ;
	else if (!aOwnerName.Len() || (bCase ? aOwnerName == pConnection->GetUser() : aOwnerName.EqualsIgnoreCaseAscii(pConnection->GetUser())))
		nPrivileges |= (SDB_PR_READ	| SDB_PR_INSERT	| SDB_PR_UPDATE | SDB_PR_DELETE | SDB_PR_DROP | SDB_PR_REFERENCES | SDB_PR_ALTER);
	else
	{
		pCursor->OpenTablePrivileges(0,pOwner,aName.GetBuffer());
		#define TABLE_QUALIFIER 1
		#define TABLE_OWNER     2
		#define TABLE_NAME      3
		#define GRANTOR         4
		#define GRANTEE         5
		#define PRIVILEGE       6

		if (!pCursor->IsInRange())
			nPrivileges = SDB_PR_READ;

		while (aStatus.IsSuccessful() && pCursor->IsInRange())
		{
			ODbRow& rRow = *pCursor->GetRow();
			ODbVariantRef xValue = rRow[GRANTEE];
			if (!xValue->isNull())
			{
				String aGrantee = xValue->getString().getStr();
				if (!xValue->isNull() &&
					((bCase) ? aGrantee == aOwnerName :
					 aGrantee.EqualsIgnoreCaseAscii(aOwnerName)))
				{
					xValue = rRow[PRIVILEGE];
					String aPrivilege;
					if (!xValue->isNull())
						aPrivilege = xValue->getString().getStr();

					String aSimplePriv;
					for(xub_StrLen i=0;i<aPrivilege.GetTokenCount(',');i++)
					{
						aSimplePriv = aPrivilege.GetToken(i,',');
						aSimplePriv.EraseLeadingChars(' ');

						if (aSimplePriv.EqualsIgnoreCaseAscii(privilegeSelect))
							nPrivileges |= SDB_PR_READ;
						else if (aSimplePriv.EqualsIgnoreCaseAscii(privilegeInsert))
							nPrivileges |= SDB_PR_INSERT;
						else if (aSimplePriv.EqualsIgnoreCaseAscii(privilegeUpdate))
							nPrivileges |= SDB_PR_UPDATE;
						else if (aSimplePriv.EqualsIgnoreCaseAscii(privilegeDelete))
							nPrivileges |= SDB_PR_DELETE;
						else if (aSimplePriv.EqualsIgnoreCaseAscii(privilegeReferences))
							nPrivileges |= SDB_PR_REFERENCES;
					}

				}
				pCursor->Next();
			}
		}
		pCursor->Close();
	}

	nPrivileges &= ~(SDB_PR_DROP | SDB_PR_REFERENCES | SDB_PR_ALTER);
		// we don't allowe any structure modifications anymore (since 5.2)

	pCursor->ReleaseRef();
	if (aStatus.IsNoDataFound())
		aStatus.Clear();
	return aOriginalColumns->Count() > 1;
}

//-----------------------------------------------------------------------------
BOOL SdbOTERTable::DropImpl()
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	SdbOTERConnection* pConnection = (SdbOTERConnection*)GetConnection();
//
//	if(pConnection->IsTransactOn() && pConnection->IsMultiUserOn())
//	{
//		String aTmp(SdbResId(STR_STAT_OTER_NO_TABLE_CREATE));
//		aStatus.Set(SDB_STAT_ERROR,
//							"S1000",
//							aStatus.CreateErrorMessage(aTmp),
//							0, String() );
//		return FALSE;
//	}
//
//
//
//	// AutoInkrementwerte m"ussen gel"oscht werden bevor die Tabelle gedropt wird(BUG bei Oterro)
//	for (USHORT i = 1; i < aOriginalColumns->Count(); i++)
//		DeleteAutoNum((*aOriginalColumns)[i],pConnection,aName);
//
//	String aCommand("DROP TABLE ");
//
//	aCommand += pConnection->IdentifierQuotation(aName);
//
//	pConnection->ExecuteSQL(aCommand);
//	aStatus = pConnection->Status();
//
//	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
void SdbOTERTable::CloseImpl() {DBG_CHKTHIS(SdbOTERTable,NULL);}
//-----------------------------------------------------------------------------
BOOL SdbOTERTable::CreateImpl()
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	SdbOTERConnection* pConnection = (SdbOTERConnection*)GetConnection();
//
//	if(pConnection->IsTransactOn() && pConnection->IsMultiUserOn()){
//		String aTmp(SdbResId(STR_STAT_OTER_NO_TABLE_CREATE));
//		aStatus.Set(SDB_STAT_ERROR,"S1000",aStatus.CreateErrorMessage(aTmp),0, String() );
//		return FALSE;
//	}
//
//	String aCommand("CREATE TABLE ");
//	aCommand += pConnection->IdentifierQuotation(aName);
//	aCommand += ' ';
//	aCommand += '(';
//
//	if (!GenerateAddCommands(aCommand))
//		return FALSE;
//
//	String aPColumns;
//	for (USHORT i = 1; i < aColumns->Count(); i++)
//	{
//		if (aColumns->Column(i)->IsPrimaryKey())
//		{
//			aPColumns += pConnection->IdentifierQuotation(aColumns->Column(i)->GetName());
//			aPColumns += ',';
//		}
//	}
//	if (aPColumns.Len())
//	{
//		// Ein Komma ist sicherlich noch zuviel ...
//		aPColumns[USHORT(aPColumns.Len()-1)] = ')';
//		aCommand += " PRIMARY KEY (";
//		aCommand += aPColumns;
//		aCommand += ',';
//	}
//	// Ein Komma ist sicherlich noch zuviel ...
//	aCommand[USHORT(aCommand.Len() - 1)] = ')';
//
//	pConnection->ExecuteSQL(aCommand);
//	aStatus = pConnection->Status();
//	if (aStatus.IsSuccessful())
//	{
//		nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_DELETE | SDB_PR_DROP | SDB_PR_ALTER | SDB_PR_REFERENCES;
//		for(USHORT i=1;i<aColumns->Count();i++)
//		{
//			const SdbColumn * pColumn = (*aColumns)[i];
//			if(pColumn->IsAutoIncrement())
//			{
//				String aAutoInc("autonum ");
//				aAutoInc += pConnection->IdentifierQuotation(pColumn->GetName());
//				aAutoInc += " in ";
//				aAutoInc += pConnection->IdentifierQuotation(aName);
//				if(pColumn->GetType() == SDB_DBTYPE_CHAR)
//					aAutoInc += " using 1";
//				else
//					aAutoInc += " using 1";
//				pConnection->ExecuteSQL(aAutoInc);
//			}
//		}
//	}
//
//	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERTable::GenerateAddCommands(String& rStatement)
{
//	DBG_CHKTHIS(SdbOTERTable,NULL);
//	// Welche neuen Columns gibt es jetzt?
//	// (Erkennbar an "fehlender" Identitaet (Id == 0))
//	for (USHORT i = 1; i < aColumns->Count(); i++)
//	{
//		const SdbColumn * pColumn = (*aColumns)[i];
//		if (!GenerateColumnCommand(rStatement, *pColumn))
//			return FALSE;
//
//		rStatement += ',';
//	}
	return TRUE;
}
//-----------------------------------------------------------------------------
BOOL SdbOTERTable::DeletePkeys(SdbOTERConnection* xConnection,const String& aTableName)
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	String aStmt("ALTER TABLE ");
//	aStmt += aTableName;
//	aStmt += " DROP CONSTRAINT ";
//	aStmt += aPKeyName;
//	xConnection->ExecuteSQL(aStmt);
//	aStatus = xConnection->Status();
//	if (aStatus.IsSuccessful())
//		aPKeyName.Erase();
//	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERTable::DeleteAutoNum(const SdbColumn* pRefColumn,SdbOTERConnection* xConnection,const String& aTableName)
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	 // einmal reicht, um zu wissen das es sich um ein AutoIncremment handelt
//	if(xConnection->IsAutoNum(aTableName,pRefColumn->GetName()))
//	{
//		String aAutoDel("AUTONUM ");
//		aAutoDel += xConnection->IdentifierQuotation(pRefColumn->GetName());
//		aAutoDel += " IN ";
//		aAutoDel += aTableName;
//		aAutoDel += " DELETE";
//		xConnection->ExecuteSQL(aAutoDel);
//		aStatus = xConnection->Status();
//	}
//	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
BOOL SdbOTERTable::AlterImpl()
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
	// Alter kann nicht ueber eine Transaktion ausgefuehrt
//	SdbOTERConnection* xConnection = (SdbOTERConnection*)GetConnection();//->GetEnvironment()->OpenConnection(*GetConnection()));
//
//	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(xConnection->GetMetaData());
//	if(xConnection->IsTransactOn()){
//		String aTmp(SdbResId(STR_STAT_OTER_NO_TABLE_ALTER));
//		aStatus.Set(SDB_STAT_ERROR,"S1000",aStatus.CreateErrorMessage(aTmp),0, String() );
//		return FALSE;
//	}
//	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;
//
//	String aStatement;
//	String aTableName;
//
//	if (aOwnerName.Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
//	{
//		aTableName += xConnection->IdentifierQuotation(aOwnerName);
//		aTableName += '.';
//	}
//	aTableName += xConnection->IdentifierQuotation(aName);
//
//	BOOL *bSet = new BOOL[aOriginalColumns->Count()];
//
//	// erst alle identischen und die, wo sich die Struktur gendert hat heraus filtern
//	for (UINT16 i = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && i > 0; i--)
//	{
//		const SdbColumn* pColumn = (*aOriginalColumns)[i];
//		const SdbColumn* pRefColumn = (*aColumns)[pColumn->GetName()];
//		bSet[i] = FALSE;
//		if(pRefColumn && pRefColumn->Compare(*pColumn,SDB_FLAGS_NULLALLOWED|SDB_FLAGS_PRIMARYKEY|SDB_FLAGS_AUTOINCREMENT))
//		{
//			// Falls vorhanden, nichts machen
//			bSet[i] = TRUE;
//		}
//		// hat sich nur das Auto_increment flag geaendert
//		else if(pRefColumn && pRefColumn->Compare(*pColumn,SDB_FLAGS_NULLALLOWED|SDB_FLAGS_PRIMARYKEY))
//		{
//			if(pColumn->IsAutoIncrement())
//			{
//				switch(pColumn->GetType())
//				{
//					case SDB_DBTYPE_DECIMAL:
//					case SDB_DBTYPE_NUMERIC:
//					case SDB_DBTYPE_TINYINT:
//					case SDB_DBTYPE_SMALLINT:
//					case SDB_DBTYPE_INTEGER:
//					case SDB_DBTYPE_BIGINT:
//					case SDB_DBTYPE_REAL:
//					case SDB_DBTYPE_DOUBLE:
//						if(!DeleteAutoNum(pColumn,xConnection,aName))
//							continue;
//						break;
//					default:
//						DBG_ASSERT(0,"Datenbank ist korrupt! Ein Feld wird Autoinkrement angenommen, das keines sein kann!");
//				}
//			}
//			else
//			{
//				String aAutoInc("AUTONUM ");
//				aAutoInc += xConnection->IdentifierQuotation(pRefColumn->GetName());
//				aAutoInc += " IN ";
//				aAutoInc += aTableName;
//				aAutoInc += " USING ";
//				// Die aktuelle Gr"o"se der Spalte feststellen
//				SdbCursor* pCursor = xConnection->CreateCursor(SDB_SNAPSHOT, SDB_FORWARDONLY);
//				pCursor->AddRef();
//				String aMaxStatement("SELECT max(");
//				aMaxStatement += xConnection->IdentifierQuotation(pRefColumn->GetName());
//				aMaxStatement += ") FROM ";
//				aMaxStatement += aTableName;
//
//				ULONG nActMax=0;
//				if (pCursor->Open(aMaxStatement) && pCursor->IsInRange())
//				{
//					nActMax = (*pCursor->GetRow())[1]->toINT32();
//				}
//				pCursor->ReleaseRef();
//
//				aAutoInc += nActMax+1;
//				aAutoInc += " 1"; // increment
//
//				xConnection->ExecuteSQL(aAutoInc);
//			}
//			bSet[i] = TRUE;
//		}
//		else if(pRefColumn  && !pRefColumn->Compare(*pColumn,SDB_FLAGS_NULLALLOWED))
//		{
//			aStatement	= "ALTER TABLE ";
//			aStatement += aTableName;
//			aStatement += " ALTER ";
//			if(GenerateColumnCommand(aStatement, *pRefColumn))
//			{
//				if(!(pColumn->GetAllFlags() & SDB_FLAGS_FOREIGNKEY))
//				{
//					// Pruefen ob autonum an der alten Column gesetzt ist
//					if(!DeleteAutoNum(pColumn,xConnection,aName))
//						continue;
//					// gibt es eine neue Schluesselspalte, dann PKey loeschen
//					if (aPKeyName.Len() && pRefColumn->IsPrimaryKey())
//					{
//						if(!DeletePkeys(xConnection,aTableName))
//							continue;
//					}
//					// sobald die Column ein Fkey ist, kann sie nicht gealtered werden
//					xConnection->ExecuteSQL(aStatement);
//					aStatus = xConnection->Status();
//					if (aStatus.IsSuccessful())
//					{
//						if(pRefColumn->IsAutoIncrement())
//						{
//							String aMaxAuto("select max(");
//							aMaxAuto += xConnection->IdentifierQuotation(pRefColumn->GetName());
//							aMaxAuto += ") from ";
//							aMaxAuto += aTableName;
//							SdbOTERCursor* pCursor = (SdbOTERCursor *) xConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
//							pCursor->AddRef();
//							pCursor->Open(aMaxAuto);
//							INT32 nAutoWert(0);
//							if (pCursor->IsInRange())
//							{
//								nAutoWert = (*pCursor->GetRow())[1]->toINT32();
//							}
//							pCursor->Close();
//							pCursor->ReleaseRef();
//							String aAutoInc("autonum ");
//							aAutoInc += xConnection->IdentifierQuotation(pRefColumn->GetName());
//							aAutoInc += " in ";
//							aAutoInc += aTableName;
//							aAutoInc += " using ";
//							aAutoInc += (nAutoWert+1);
//							aAutoInc += " 1";
//							xConnection->ExecuteSQL(aAutoInc);
//						}
//						aOriginalColumns->ReplaceColumn(CreateColumn(*pRefColumn),i);
//					}
//					bSet[i] = TRUE;
//				}
//				else
//				{
//					String aTmp(SdbResId(STR_FOREIGN_KEY_NOT_ALTER));
//					aStatus.Set(SDB_STAT_ERROR,"S1000",aStatus.CreateErrorMessage(aTmp),0, String(String()) );
//				}
//			}
//		}
//		else if(pRefColumn  && !pRefColumn->Compare(*pColumn,SDB_FLAGS_PRIMARYKEY))
//			bSet[i] = TRUE; // nur PKkey hat sich geaendert
//	}
//
//	// jetzt alle die jenigen, wo sich nur der Namegendert hat
//	for (UINT16 k = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && k > 0; k--)
//	{
//		if(!bSet[k])
//		{
//			const SdbColumn* pColumn = (*aOriginalColumns)[k];
//			for(UINT16 j = aColumns->Count() - 1; aStatus.IsSuccessful() && j > 0; j--)
//			{
//				const SdbColumn* pRefColumn = (*aColumns)[j];
//				if(pRefColumn->CompareWithOutName(*pColumn,SDB_FLAGS_NULLALLOWED|SDB_FLAGS_PRIMARYKEY|SDB_FLAGS_AUTOINCREMENT) && !aOriginalColumns->Column(pRefColumn->GetName()))
//				{
//					// nur Name hat sich gendert
//					aStatement	= "ALTER TABLE ";
//					aStatement += aTableName;
//					aStatement += " ALTER ";
//					aStatement += xConnection->IdentifierQuotation(pColumn->GetName());
//					aStatement += " TO ";
//					if(GenerateColumnCommand(aStatement, *pRefColumn)){
//						// Pruefen ob autonum gesetzt ist
//						if(!DeleteAutoNum(pRefColumn,xConnection,aName))
//							continue;
//						// gibt es eine neue Schluesselspalte, dann PKey loeschen
//						if (aPKeyName.Len() && pRefColumn->IsPrimaryKey()){
//							if(!DeletePkeys(xConnection,aTableName))
//								continue;
//						}
//						xConnection->ExecuteSQL(aStatement);
//						aStatus = xConnection->Status();
//						if (aStatus.IsSuccessful()){
//							aOriginalColumns->ReplaceColumn(CreateColumn(*pRefColumn),k);
//							//aColumns->RemoveColumn(pColumn->GetName());
//							bSet[k] = TRUE;
//						}
//					}
//				}
//			}
//		}
//	}
//	// jetzt die nicht verwendeten lschen
//	for ( i = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && i > 0; i--){
//		if(!bSet[i]){
//			const SdbColumn* pColumn = (*aOriginalColumns)[i];
//			aStatement	= "ALTER TABLE ";
//			aStatement += aTableName;
//			aStatement += " DROP ";
//			aStatement += xConnection->IdentifierQuotation(pColumn->GetName());
//			// Pruefen ob autonum gesetzt ist
//			if(!DeleteAutoNum(pColumn,xConnection,aName))
//				continue;
//			// vorhandene Pkeys loeschen
//			if (aPKeyName.Len() && pColumn->IsPrimaryKey()){
//				if(!DeletePkeys(xConnection,aTableName))
//					continue;
//			}
//			xConnection->ExecuteSQL(aStatement);
//			aStatus = xConnection->Status();
//			if (aStatus.IsSuccessful())
//				aOriginalColumns->RemoveColumn(pColumn->GetName());
//		}
//	}
//	delete [] bSet;
//	// und jetzt die neuen hinzufuegen
//	for (i = 1; aStatus.IsSuccessful() && i < aColumns->Count(); i++)
//	{
//		const SdbColumn* pColumn = (*aColumns)[i];
//		const SdbColumn* pRefColumn = (*aOriginalColumns)[pColumn->GetName()];
//		if (!pRefColumn)
//		{
//			aStatement	= "ALTER TABLE ";
//			aStatement += aTableName;
//			aStatement += " ADD ";
//			if(GenerateColumnCommand(aStatement, *pColumn)){
//				// gibt es eine neue Schluesselspalte, dann PKey loeschen
//				if (aPKeyName.Len() && pColumn->IsPrimaryKey()){
//					if(!DeletePkeys(xConnection,aTableName))
//						continue;
//				}
//				if(pColumn->IsAutoIncrement()){
//					while(aStatement.SearchAndReplace("NOT NULL"," ") != STRING_NOTFOUND)
//						;
//				}
//
//				xConnection->ExecuteSQL(aStatement);
//				aStatus = xConnection->Status();
//				if (aStatus.IsSuccessful()){
//					if(pColumn->IsAutoIncrement()){
//						// neue Werte generieren
//						String aAutoInc("AUTONUM ");
//						aAutoInc += xConnection->IdentifierQuotation(pColumn->GetName());
//						aAutoInc += " IN ";
//						aAutoInc += aTableName;
//						aAutoInc += " USING 1 ORDER BY ";
//						aAutoInc += xConnection->IdentifierQuotation(pColumn->GetName());
//						aAutoInc += " NUM";
//						xConnection->ExecuteSQL(aAutoInc);
//
//						// maximum berechnen
//						String aMaxAuto("select max(");
//						aMaxAuto += xConnection->IdentifierQuotation(pColumn->GetName());
//						aMaxAuto += ") from ";
//						aMaxAuto += aTableName;
//						SdbOTERCursor* pCursor = (SdbOTERCursor *) xConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
//						pCursor->AddRef();
//						pCursor->Open(aMaxAuto);
//						INT32 nAutoWert(0);
//						if (pCursor->IsInRange())
//						{
//							nAutoWert = (*pCursor->GetRow())[1]->toINT32();
//						}
//						pCursor->Close();
//						pCursor->ReleaseRef();
//						// autnum wieder lschen
//						if(DeleteAutoNum(pColumn,xConnection,aName)){
//							// spalte auf "not null" setzen
//							aStatement	= "ALTER TABLE ";
//							aStatement += aTableName;
//							aStatement += " ALTER ";
//							GenerateColumnCommand(aStatement, *pColumn);
//							xConnection->ExecuteSQL(aStatement);
//							// autonum erneut setzen
//							String aAutoInc("autonum ");
//							aAutoInc += xConnection->IdentifierQuotation(pColumn->GetName());
//							aAutoInc += " in ";
//							aAutoInc += aTableName;
//							aAutoInc += " using ";
//							aAutoInc += (nAutoWert+1);
//							aAutoInc += " 1";
//							xConnection->ExecuteSQL(aAutoInc);
//						}
//						aStatus = xConnection->Status();
//					}
//					aOriginalColumns->AddColumn(CreateColumn(*pColumn));
//				}
//			}
//		}
//	}
//
//	// gibt es noch Aenderungen am Schluessel
//	for (i = 1; aPKeyName.Len() && aStatus.IsSuccessful() && i < aColumns->Count(); i++)
//	{
//		const SdbColumn* pColumn = (*aColumns)[i];
//		const SdbColumn* pRefColumn = (*aOriginalColumns)[pColumn->GetName()];
//		if (pRefColumn && pColumn->IsPrimaryKey() != pRefColumn->IsPrimaryKey())
//			DeletePkeys(xConnection,aTableName);
//	}
//
//
//	if (!aPKeyName.Len() && aStatus.IsSuccessful()) // muss jetzt noch ein PKey hinzugefuegt werden ?
//	{
//		String aPColumns;
//		for (UINT16 i = 1; i < aColumns->Count(); i++)
//		{
//			if ((*aColumns)[i]->IsPrimaryKey())
//			{
//				aPColumns += xConnection->IdentifierQuotation((*aColumns)[i]->GetName());
//				aPColumns += ',';
//			}
//		}
//		if (aPColumns.Len())
//		{
//			// Ein Komma ist sicherlich noch zuviel ...
//			aPColumns[USHORT(aPColumns.Len()-1)] = ')';
//
//			String aCommand("ALTER TABLE ");
//			aCommand += aTableName;
//			aCommand += " ADD PRIMARY KEY (";
//			aCommand += aPColumns;
//
//			xConnection->ExecuteSQL(aCommand);
//			aStatus = xConnection->Status();
//		}
//	}
//	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERTable::GenerateColumnCommand(String& rStatement, const SdbColumn& rColumn)
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	SdbConnection* pConnection = GetConnection();
//	// Gibt es zu dieser Column eine Typinformationen
//	const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rColumn);
//
//	if (!pInfo)
//	{
//		pInfo = pConnection->GetColumnTypeInfo(rColumn.GetType());
//		if(!pInfo){
//			DBG_ASSERT(0,"StarBase Table: GetColumnTypeInfo == NULL");
//			aStatus.SetDriverNotCapableError();
//			return FALSE;
//		}
//	}
//
//	rStatement += pConnection->IdentifierQuotation(rColumn.GetName());
//	rStatement += ' ';
//	if(SDB_DBTYPE_VARCHAR == pInfo->eType && rColumn.GetLength() < 4)
//		rStatement += "TEXT";
//	else
//		rStatement += pInfo->aTypeName;
//
//	switch(pInfo->eType)
//	{
//		case SDB_DBTYPE_CHAR:
//		case SDB_DBTYPE_VARCHAR:
//			// Typen mit Laengenangabe
//			rStatement += '(';
//			rStatement += rColumn.GetLength() ? rColumn.GetLength() : 1;
//			rStatement += ')';
//			break;
//		case SDB_DBTYPE_BINARY:
//		case SDB_DBTYPE_VARBINARY:
//			// Typen mit Laengenangabe
//			rStatement += '(';
//			rStatement += rColumn.GetLength()*8 ? rColumn.GetLength() : 1;
//			rStatement += ')';
//			break;
//		case SDB_DBTYPE_DECIMAL:
//		case SDB_DBTYPE_NUMERIC:
//			// Typen mit Precision und Scale
//			rStatement += '(';
//			rStatement += rColumn.GetPrecision();
//			rStatement += ',';
//			rStatement += rColumn.GetScale();
//			rStatement += ')';
//			break;
//	}
//
//	String aDefaultValue(rColumn.GetDefaultValue());
//	if(aDefaultValue.Len() && pInfo->eType != SDB_DBTYPE_BOOLEAN)
//	{
//		rStatement += " DEFAULT ";
//		SvNumberFormatter* pFormatter = GetDatabase() ? GetDatabase()->GetFormatter() : NULL;
//		ULONG nFormatKey = rColumn.GetFormatKey();
//		switch(pInfo->eType)
//		{
//			case SDB_DBTYPE_DECIMAL:
//			case SDB_DBTYPE_NUMERIC:
//			case SDB_DBTYPE_TINYINT:
//			case SDB_DBTYPE_SMALLINT:
//			case SDB_DBTYPE_INTEGER:
//			case SDB_DBTYPE_BIGINT:
//			case SDB_DBTYPE_REAL:
//			case SDB_DBTYPE_DOUBLE:
//			case SDB_DBTYPE_CURRENCY:
//				if (pFormatter)
//				{
//					int nErrno=0;
//					SvNumberformat* pFormat = (SvNumberformat*)pFormatter->GetEntry(nFormatKey);
//						// ein const_cast, da GetFormatPrecision am SvNumberFormat nicht const ist, obwohl es das eigentlich
//						// sein koennte und muesste
//					International aInter(pFormat->GetLanguage());
//					double dValue = SolarMath::StringToDouble(aDefaultValue, aInter, nErrno);
//					aDefaultValue.Erase();
//					SolarMath::DoubleToString(aDefaultValue, dValue, 'F', pFormat->GetFormatPrecision(), '.', TRUE);
//				}
//			break;
//			case SDB_DBTYPE_DATE:
//			case SDB_DBTYPE_TIME:
//			case SDB_DBTYPE_TIMESTAMP:
//				if (pFormatter)
//				{
//					double dValue;
//					BOOL bValid = pFormatter->IsNumberFormat(aDefaultValue, nFormatKey, dValue);
//					DBG_ASSERT(bValid, "SdbOTERTable::GenerateColumnCommand : invalid column default value !");
//					ODbVariantRef xVar = new ODbVariant(SdbTools::ToStandardDate(*pFormatter, dValue));
//					aDefaultValue += "\'";
//					aDefaultValue = SdbTools::SQLString(pInfo->eType, *xVar, FALSE);
//					aDefaultValue += "\'";
//				}
//			break;
//		}
//		rStatement += aDefaultValue;
//	}
//		// FS: 60085: da wir die Defaults selber handeln, braucht die DB sie nicht zu kennen
//
//	if (!rColumn.IsNullAllowed() )
//		rStatement += " NOT NULL";
//
//	return TRUE;
}


//-----------------------------------------------------------------------------
SdbIndex* SdbOTERTable::CreateIndexImpl()
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	return new SdbOTERIndex(this);
}

//-----------------------------------------------------------------------------
void SdbOTERTable::FillIndexNames(SvStringsDtor& _rList)
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	SdbConnection* pConnection = GetConnection();
	BOOL bCase = (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;
		// Einlesen aller Indices
	SdbOTERCursor* pCursor = (SdbOTERCursor *) pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();

	String aIndexName;

	const sal_Unicode* pQualifier = NULL;
	const sal_Unicode* pOwner = NULL;
	if (aOwnerName.Len())
		pOwner = aOwnerName.GetBuffer();

	pCursor->OpenIndices(pQualifier,pOwner,aName.GetBuffer());

	#define TABLE_QUALIFIER  1
	#define TABLE_OWNER      2
	#define TABLE_NAME       3
	#define NON_UNIQUE       4
	#define INDEX_QUALIFIER  5
	#define INDEX_NAME       6
	#define SEQ_IN_INDEX     8
	#define COLUMN_NAME      9
	#define COLLATION        10

	USHORT i (0);
	while (pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		ODbVariantRef xValue = rRow[INDEX_NAME];
		if (rRow[INDEXTYPE]->toINT16() == SQL_TABLE_STAT ||
			xValue->getString().getStr() == aPKeyName)		// Index des Pkeys rausfiltern
		{
			pCursor->Next();
			continue;
		}

		ODbVariantRef xQualifier = rRow[INDEX_QUALIFIER];
		String aNewIndexName;

		if (!xQualifier->isNull())
		{
			aNewIndexName = xQualifier->getString().getStr();
			aNewIndexName += '.';
		}
		aNewIndexName += xValue->getString().getStr();

		BOOL bNewIndex = (bCase) ? aIndexName != aNewIndexName
								 : aIndexName.EqualsIgnoreCaseAscii(aNewIndexName) ;
		if (bNewIndex)
		{
			aIndexName = aNewIndexName;
			_rList.Insert(new String(aIndexName), i++);
		}
		pCursor->Next();
	}
	pCursor->Close();
	pCursor->ReleaseRef();
}

//-----------------------------------------------------------------------------
BOOL SdbOTERTable::RenameImpl(const String& rNewName)
{
	DBG_CHKTHIS(SdbOTERTable,NULL);
	aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("S1000"),String(SdbResId(STR_STAT_OTER_NO_ALTER_ALLOWED)),0,String());
	return FALSE;
//	SdbOTERConnection* pConnection = (SdbOTERConnection*)GetConnection();
//
//	if(pConnection->IsTransactOn() && pConnection->IsMultiUserOn())
//	{
//		String aTmp(SdbResId(STR_STAT_OTER_NO_TABLE_CREATE));
//		aStatus.Set(SDB_STAT_ERROR,
//							"S1000",
//							aStatus.CreateErrorMessage(aTmp),
//							0, String() );
//		return FALSE;
//	}
//
//	String aStatement("RENAME TABLE ");
//	aStatement += pConnection->IdentifierQuotation(aName);
//	aStatement += " TO ";
//	aStatement += pConnection->IdentifierQuotation(rNewName);
//	BOOL bErg(pConnection->ExecuteSQL(aStatement));
//	aStatus = pConnection->Status();
//	return bErg;

}

