/*************************************************************************
 *
 *  $RCSfile: adacurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbdbimp.hxx"
#ifndef _SDB_ADAINC_HXX
#include "adainc.hxx"
#endif
#include "adaconn.hxx"
#include "sdbresid.hrc"
#include "adacurs.hxx"

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _SBXMSTRM_HXX //autogen
#include <svtools/sbxmstrm.hxx>
#endif

#ifndef _DATE_HXX //autogen
#include <tools/date.hxx>
#endif

#ifndef _SDB_TYPECONV_HXX
#include <typeconv.hxx>
#endif

#ifndef _SDB_SDBSQLIT_HXX
#include "sdbsqlit.hxx"
#endif
#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif
#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEOUTERJOINSUPPORT_HPP_
#include <com/sun/star/data/DatabaseOuterJoinSupport.hpp>
#endif
namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

DBG_NAME(SdbAdaCursor);
//========================================================================

// in odbc we have a fraction part for time values, a single unit is 1/100000000
const ULONG ODBC_FRACTION_UNITS_PER_HSECOND = 10000000L;
const String aTruncState=String::CreateFromAscii("01004");
const String aValueChangedState=String::CreateFromAscii("01S02");
const SWORD DESC_COL_NAME_LEN = 19;
//------------------------------------------------------------------
//------------------------------------------------------------------
static DateTime ToDateTime(const TIMESTAMP_STRUCT& rTS)
{
	return DateTime (Date(rTS.day,rTS.month,rTS.year),
			  Time(rTS.hour,rTS.minute,rTS.second,long (rTS.fraction / ODBC_FRACTION_UNITS_PER_HSECOND)));	
}

//------------------------------------------------------------------
static Time ToTime(const TIME_STRUCT& rTS)
{
	return Time(rTS.hour,rTS.minute,rTS.second);
}

//------------------------------------------------------------------
static Date ToDate(const DATE_STRUCT& rTS)
{
	return Date(rTS.day,rTS.month,rTS.year);
}

//------------------------------------------------------------------
static void ToDate(DATE_STRUCT& rResult, double fSbxDate)
{
	Date aDate = SdbTools::ToDate(fSbxDate);
	rResult.year	  = aDate.GetYear();
	rResult.month	  = aDate.GetMonth();
	rResult.day		  = aDate.GetDay();
}

//------------------------------------------------------------------
static void ToTime(TIME_STRUCT& rResult, double fSbxDate)
{
	Time aTime = SdbTools::ToTime(fSbxDate);
	// Zeit:
	rResult.hour	  = aTime.GetHour();
	rResult.minute	  = aTime.GetMin();
	rResult.second	  = aTime.GetSec();
}

//------------------------------------------------------------------
static void ToTimeStamp(TIMESTAMP_STRUCT& rResult, double fSbxDate)
{
	DateTime aDateTime = SdbTools::ToDateTime(fSbxDate);
	rResult.year	  = aDateTime.GetYear();
	rResult.month	  = aDateTime.GetMonth();
	rResult.day		  = aDateTime.GetDay();

	// Zeit:
	rResult.hour	  = aDateTime.GetHour();
	rResult.minute	  = aDateTime.GetMin();
	rResult.second	  = aDateTime.GetSec();
	rResult.fraction  = aDateTime.Get100Sec() * ODBC_FRACTION_UNITS_PER_HSECOND;
}


//-----------------------------------------------------------------------------
SdbAdaCursor::SdbAdaCursor(SdbAdaConnection* pConnection,
							 SdbCursorType eType,
							 UINT16 nOpt)
			  :SdbCursor(pConnection, eType, nOpt)
			  ,m_aStatementHandle(SQL_NULL_HANDLE)
			  ,pParamBuffer(NULL)
			  ,pParamLength(NULL)
			  ,nCurrentMaxFieldLen(nMaxFieldLen)
			  ,nCurrentMaxRows(nMaxRows)
{
	DBG_CTOR(SdbAdaCursor,NULL);

	aStatus.SetComponent(String::CreateFromAscii("Sdb Adabas Driver"));
	bIsCancelable = TRUE;
	
	aName = 'C';
	aName += String::CreateFromInt32((long)this);

	RETCODE rRetcode = NADAAllocHandle(SQL_HANDLE_STMT, GetAdaConnection()->ConnectionHandle(),&m_aStatementHandle);
	if (rRetcode != SQL_SUCCESS)
	{
		SetStatus(rRetcode, SQL_HANDLE_DBC, GetAdaConnection()->ConnectionHandle());
		m_aStatementHandle = SQL_NULL_HANDLE;
	}
}
//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::IsCapable(SdbCursorCapability eCapability) const
{
	BOOL bErg=FALSE;
	switch(eCapability)
	{
		case SDB_CAP_FETCH:
		case SDB_CAP_FETCH_NEXT:
		case SDB_CAP_FETCH_PRIOR:
		case SDB_CAP_FETCH_FIRST:
		case SDB_CAP_FETCH_LAST:    
		case SDB_CAP_FETCH_RELATIVE:
		case SDB_CAP_FETCH_ABSOLUTE:
			bErg = TRUE;
	}
	return bErg;
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::SetStatus(RETCODE nRetcode, SQLSMALLINT HandleType, SQLHANDLE aHandle,
							 const String& rErrorInfo)
{
	SdbAda_SetStatus(aStatus, nRetcode, HandleType, aHandle, rErrorInfo);	
}

//-----------------------------------------------------------------------------
SdbAdaCursor::~SdbAdaCursor()
{
	DBG_DTOR(SdbAdaCursor,NULL);
#if SUPD>557
	// (RemovingFromParent isn't virtual in 557)
	DBG_ASSERT(GetParent() || m_aStatementHandle == SQL_NULL_HANDLE, "SdbAdaCursor::~SdbAdaCursor : RemovingFromParent should have been called if I have no parent anymore !");
#endif
	if (m_aStatementHandle != SQL_NULL_HANDLE)
	{
		NADAFreeHandle(SQL_HANDLE_STMT, m_aStatementHandle);
		m_aStatementHandle = SQL_NULL_HANDLE;
	}
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::RemovingFromParent()
{
	DBG_ASSERT(!IsOpen(), "SdbAdaCursor::RemovingFromParent : expected to be called _after_ Close !");

	// drop our statement. We can't do this later as the connection - which is our parent - may be destroyed then.
	if (m_aStatementHandle != SQL_NULL_HANDLE)
	{
		NADAFreeHandle(SQL_HANDLE_STMT, m_aStatementHandle);
		m_aStatementHandle = SQL_NULL_HANDLE;
	}
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::PrepareImpl()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	// Statement vorbereiten
	if (aParameterColumns->Count() < 2)
		Translate();
	if (! aStatus.IsSuccessful()) return FALSE;

	RETCODE nRetcode;
	{
		SdbCancelableCoreCall aCoreCallFlag(*this);
		ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
		nRetcode = NADAPrepare(m_aStatementHandle, (SDB_ADA_CHAR *) aSql.GetBuffer(), aSql.Len());
	}

	SetStatus(nRetcode,aSQLStatement);
	if (! aStatus.IsSuccessful()) return FALSE;

	// hack
	// Parameter schon gesetzt?
	if (aParameterColumns->Count() < 2)
		DescribeParameter();
	BindParameter();

	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::IsAutoIncrement(const String& _OwnerName,const String& _TableName,const String& _ColName) const
{
	SdbAdaCursor* pCursor = (SdbAdaCursor *) GetConnection()->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();
	String aSql;aSql.AssignAscii("SELECT \"DEFAULT\" FROM DOMAIN.COLUMNS WHERE \"DEFAULT\" = 'DEFAULT STAMP' AND");
	if(_OwnerName.Len())
	{
		aSql.AppendAscii(" OWNER = '");
		aSql += _OwnerName;
		aSql.AppendAscii("' AND");
	}
	
	if(_TableName.Len())
	{
		aSql.AppendAscii(" TABLENAME = '");
		aSql += _TableName;
		aSql.AppendAscii("' AND");
	}
	aSql.AppendAscii(" COLUMNNAME = '");
	aSql += _ColName;
	aSql.AppendAscii("'");
	pCursor->Open(aSql);
	BOOL bAuto = pCursor->IsInRange();
	pCursor->ReleaseRef();
	return bAuto;
}
//-----------------------------------------------------------------------------
void SdbAdaCursor::DescribeResult()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aResultColumns.Is(), "keine Ergebnisbeschreibung");
	// Bookmark-Column ist immer da (schon durch den Konstruktor erzeugt).

	if (aResultColumns->Count() > 1)
		return;

	// Alle ResultColumns ansehen
	SWORD nNumResultCols;
	SetStatus(NADANumResultCols(m_aStatementHandle,&nNumResultCols));
	if (! aStatus.IsSuccessful()) return;

	SWORD  cbColName;           // Laenge des Names
	SWORD  pfSqlType;            // SQL-Datentyp
	UDWORD pcbColDef;           // Precision (ggf. 0)
	SWORD  pibScale;                // Scale (ggf. 0)
	SWORD  pfNullable;          // NULL erlaubt?
	INT16  nFlags;
	RETCODE nRetcode;
	SdbDatabaseType eType;
	String aColumnName,aTableName;
	SQLCHAR cBuffer[DESC_COL_NAME_LEN];
	SdbSqlParseTreeIterator	aIter(GetConnection(),SQLParseNode());
	aIter.TraverseAll();
	const SdbTableRangeMap& aMap = aIter.GetSdbTableRangeMap();
	const SdbColumnsRef aCols = aIter.GetSelectColumns();

	for (SWORD i = 1; i <= nNumResultCols; i++)
	{
		nFlags = pibScale = pfNullable = 0;
		nRetcode =NADADescribeCol(m_aStatementHandle,
								   i,
								   cBuffer,
								   DESC_COL_NAME_LEN,
								   &cbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
		if ( nRetcode == -1)
		{
			SetStatus(nRetcode);
			return;
		}		

		if (cbColName)
		{
			cBuffer[cbColName] = 0;
			aColumnName.AssignAscii((const char*)cBuffer);
		}

		if(aCols.Is())
		{
			const SdbColumn* pCol = aCols->Column(aColumnName);
			if(pCol && pfNullable != SQL_NO_NULLS)
				pfNullable = pCol->IsNullAllowed() ? SQL_NULLABLE : SQL_NO_NULLS;
		}

		String aDefault; // this is only needed for autoincrement

		switch(pfSqlType)
		{
			case SQL_CHAR:          eType = SDB_DBTYPE_CHAR; break;
			case SQL_VARCHAR:       eType = SDB_DBTYPE_VARCHAR; break;
			case SQL_LONGVARCHAR:   eType = SDB_DBTYPE_LONGVARCHAR; break;
			case SQL_DECIMAL:       eType = SDB_DBTYPE_DECIMAL; break;
			case SQL_NUMERIC:       eType = SDB_DBTYPE_NUMERIC; break;
			case SQL_BIT:           eType = SDB_DBTYPE_BOOLEAN; break;
			case SQL_TINYINT:       eType = SDB_DBTYPE_TINYINT; break;
			case SQL_SMALLINT:      eType = SDB_DBTYPE_SMALLINT; break;
			case SQL_INTEGER:       eType = SDB_DBTYPE_INTEGER; break;
			case SQL_BIGINT:        eType = SDB_DBTYPE_BIGINT; break;
			case SQL_REAL:          eType = SDB_DBTYPE_REAL; break;
			case SQL_FLOAT:
			case SQL_DOUBLE:        eType = SDB_DBTYPE_DOUBLE; break;
			case SQL_BINARY:        
				{
					eType = SDB_DBTYPE_BINARY; 
					// check if this column could be a autoincrement
					String aCol = aColumnName;
					if(aCols.Is())
					{
						const SdbColumn* pCol = aCols->Column(aColumnName);
						if(pCol)
							aCol = pCol->GetRealName().Len() ? pCol->GetRealName() : aColumnName;
					}
					String aOwnerName,aTabName;
					if(aMap.Count() == 1)
					{
						SdbTableRef* pTableRef = aMap.GetObject(0);
						if(pTableRef && pTableRef->Is())
						{
							aOwnerName = (*pTableRef)->OwnerName();
							aTabName = (*pTableRef)->Name();
						}
					}
					if(IsAutoIncrement(aOwnerName,aTabName,aCol))
					{
						eType = SDB_DBTYPE_CHAR; 
						nFlags |= SDB_FLAGS_AUTOINCREMENT;
						nFlags &= ~SDB_FLAGS_UPDATEALLOWED;
						aDefault.AssignAscii("DEFAULT STAMP");
					}
				}
				
				break;
			case SQL_VARBINARY:     
				{
					eType = SDB_DBTYPE_VARBINARY;
					// check if this column could be a autoincrement
					String aCol = aColumnName;
					if(aCols.Is())
					{
						const SdbColumn* pCol = aCols->Column(aColumnName);
						if(pCol)
							aCol = pCol->GetRealName().Len() ? pCol->GetRealName() : aColumnName;
					}
					String aOwnerName,aTabName;
					if(aMap.Count() == 1)
					{
						SdbTableRef* pTableRef = aMap.GetObject(0);
						if(pTableRef && pTableRef->Is())
						{
							aOwnerName = (*pTableRef)->OwnerName();
							aTabName = (*pTableRef)->Name();
						}
					}
					if(IsAutoIncrement(aOwnerName,aTabName,aCol))
					{
						eType = SDB_DBTYPE_CHAR; 
						nFlags |= SDB_FLAGS_AUTOINCREMENT;
						nFlags &= ~SDB_FLAGS_UPDATEALLOWED;
						aDefault.AssignAscii("DEFAULT STAMP");
					}
				}
				break;
			case SQL_LONGVARBINARY: eType = SDB_DBTYPE_LONGVARBINARY; break;
			case SQL_DATE:          eType = SDB_DBTYPE_DATE; break;
			case SQL_TIME:          eType = SDB_DBTYPE_TIME; break;
			case SQL_TIMESTAMP:     eType = SDB_DBTYPE_TIMESTAMP; break;
			default:
				eType = SDB_DBTYPE_UNKNOWN; // wird nicht gebunden
		}
		if (pfNullable != SQL_NO_NULLS)
			nFlags |= SDB_FLAGS_NULLALLOWED;

		
		char pTableName[20];
		SQLSMALLINT nTableLen;
		SDWORD nExtAttrib;  // erweitere Attribute
		aTableName.Erase();
		if (NADAColAttribute(m_aStatementHandle,     // ist die Column updatebar?
									 i,
									 SQL_DESC_BASE_TABLE_NAME,
									 pTableName,
									 20,
									 &nTableLen,
									 &nExtAttrib
									) == SQL_SUCCESS)
		{
			if (nTableLen)
				aTableName.AssignAscii(pTableName);
			else
			{
				SdbSqlParseTreeIterator	aIter(GetConnection(),SQLParseNode());
				aIter.TraverseAll();
				const SdbColumnsRef aCols = aIter.GetSelectColumns();
				if(aCols.Is())
				{
					const SdbColumn* pCol = aCols->Column(aColumnName);
					if(pCol)
						aTableName = pCol->GetTableName();
				}
			}
		}

		if (!(nOptions & SDB_MINIMUM_DESC) && eType != SDB_DBTYPE_UNKNOWN)
		{
			
			if (NADAColAttribute(m_aStatementHandle,     // ist die Column updatebar?
									 i,
									 SQL_COLUMN_UPDATABLE,
									 NULL,
									 0,
									 NULL,
									 &nExtAttrib
									) == SQL_SUCCESS)
			{
				if (nExtAttrib != SQL_ATTR_READONLY)
					nFlags |= SDB_FLAGS_UPDATEALLOWED;
			}

			if (NADAColAttribute(m_aStatementHandle,     // handelt es sich um ein MoneyFelt
										 i,
										 SQL_COLUMN_MONEY,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib) eType = SDB_DBTYPE_CURRENCY;
			}

			if (NADAColAttribute(m_aStatementHandle,     // ist die Column updatebar?
										 i,
										 SQL_COLUMN_AUTO_INCREMENT,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib)
				{
					nFlags |= SDB_FLAGS_AUTOINCREMENT;
					nFlags &= ~SDB_FLAGS_UPDATEALLOWED;     // darf nicht updatebar sein
				}
			}

			if (NADAColAttribute(m_aStatementHandle,     // ist die Column updatebar?
										 i,
										 SQL_COLUMN_SEARCHABLE,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				switch (nExtAttrib)
				{
					case SQL_SEARCHABLE:        nFlags |= SDB_FLAGS_SEARCH_ALL; break;
					case SQL_LIKE_ONLY:         nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
					case SQL_ALL_EXCEPT_LIKE:   nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				}
			}
		}

		// Column-Beschreibung hinzufuegen.
		// existiert Name  in aResultColumns ?
		String aAlias(aColumnName);
		const SdbColumn* pPrevCol = aResultColumns->Column(aAlias);

		ULONG nExprCnt(0);
		while(pPrevCol)
		{
			(aAlias = aColumnName) += String::CreateFromInt32(++nExprCnt);
			pPrevCol = aResultColumns->Column(aAlias);
		}
		SdbColumn * pColumn = GetAdaConnection()->CreateColumn(aAlias,eType,pcbColDef,pibScale, nFlags);
		pColumn->SetRealName(aColumnName);
		pColumn->SetDefaultValue(aDefault);
		pColumn->SetTableName(aTableName);

		// Column-Beschreibung hinzufuegen.
		aResultColumns->AddColumn(pColumn);
	}
	// Strukturbeschreibung fertig.
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::DescribeParameter()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aParameterColumns.Is(), "keine Parameterspalten angelegt!");

	
	// Anzahl Parameter ermitteln:
	SWORD nNumParams;
	SdbConnection* pConnection = GetConnection();
	
	SetStatus(NADANumParams(m_aStatementHandle,&nNumParams));
	if (!aStatus.IsSuccessful() || !nNumParams) 
		return;

	String aParameterName;
	SWORD pfSqlType;			// SQL-Datentyp
	UDWORD pcbColDef;			// Precision (ggf. 0)
	SWORD pibScale;				// Scale (ggf. 0)
	SWORD pfNullable;			// NULL erlaubt?
	SQLRETURN nRetcode;
	SdbDatabaseType eType;
	SdbColumn * pColumn = NULL;
	for (UWORD i = 1; i <= (UWORD)nNumParams; i++)
	{
		pfSqlType = pibScale = pfNullable = 0;
		pcbColDef = 0L;
		aParameterName = aParameterNames.GetToken(i-1);
		// Name == "?", falls kein Parametername verfuegbar (z. B. bei Native SQL)
		if (aParameterName.Len() == 0) 
			aParameterName = '?';

		aStatus.Clear();
		nRetcode = NADADescribeParam(m_aStatementHandle,
									 i,
									 &pfSqlType,
									 &pcbColDef,
									 &pibScale,
									 &pfNullable
									 );
		SetStatus(nRetcode);
		if (! aStatus.IsSuccessful()) return;

		String aDefault;

		switch(pfSqlType)
		{
			case SQL_CHAR:			eType = SDB_DBTYPE_CHAR; break;
			case SQL_VARCHAR: 		eType = SDB_DBTYPE_VARCHAR; break;
			case SQL_LONGVARCHAR: 	eType = SDB_DBTYPE_LONGVARCHAR; break;
			case SQL_DECIMAL:		eType = SDB_DBTYPE_DECIMAL; break;
			case SQL_NUMERIC:		eType = SDB_DBTYPE_NUMERIC; break;
			case SQL_BIT:
				eType = SDB_DBTYPE_BOOLEAN;
				if(!pcbColDef)
				{
					pcbColDef = 1; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_TINYINT:
				eType = SDB_DBTYPE_TINYINT;
				if(!pcbColDef)
				{
					pcbColDef = 3; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_SMALLINT:
				eType = SDB_DBTYPE_SMALLINT;
				if(!pcbColDef)
				{
					pcbColDef = 5; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_INTEGER:
				eType = SDB_DBTYPE_INTEGER;
				if(!pcbColDef)
				{
					pcbColDef = 10; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			//case SQL_BIGINT: 		eType = SDB_DBTYPE_BIGINT; break;
			case SQL_REAL:
				eType = SDB_DBTYPE_REAL;
				if(!pcbColDef)
				{
					pcbColDef = 7; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_FLOAT:
			case SQL_DOUBLE:
			case SQL_BIGINT:
				eType = SDB_DBTYPE_DOUBLE;
				if(!pcbColDef)
				{
					pcbColDef = 15; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_BINARY:		
				{
					eType = SDB_DBTYPE_BINARY; 
					// check if this column could be a autoincrement
					SdbSqlParseTreeIterator	aIter(GetConnection(),SQLParseNode());
					aIter.TraverseTableNames();
					const SdbTableRangeMap& aMap = aIter.GetSdbTableRangeMap();
					String aOwnerName,aTabName;
					if(aMap.Count() == 1)
					{
						SdbTableRef* pTableRef = aMap.GetObject(0);
						if(pTableRef && pTableRef->Is())
						{
							aOwnerName = (*pTableRef)->OwnerName();
							aTabName = (*pTableRef)->Name();
						}
					}
					if(IsAutoIncrement(aOwnerName,aTabName,aParameterName))
					{
						eType = SDB_DBTYPE_CHAR; 
						aDefault.AssignAscii("DEFAULT STAMP");
					}
				}
				break;
			case SQL_VARBINARY: 	
				{
					eType = SDB_DBTYPE_VARBINARY;
					// check if this column could be a autoincrement
					SdbSqlParseTreeIterator	aIter(GetConnection(),SQLParseNode());
					aIter.TraverseTableNames();
					const SdbTableRangeMap& aMap = aIter.GetSdbTableRangeMap();
					String aOwnerName,aTabName;
					if(aMap.Count() == 1)
					{
						SdbTableRef* pTableRef = aMap.GetObject(0);
						if(pTableRef && pTableRef->Is())
						{
							aOwnerName = (*pTableRef)->OwnerName();
							aTabName = (*pTableRef)->Name();
						}
					}
					if(IsAutoIncrement(aOwnerName,aTabName,aParameterName))
					{
						eType = SDB_DBTYPE_CHAR; 
						aDefault.AssignAscii("DEFAULT STAMP");
					}
				}
				break;
			case SQL_LONGVARBINARY:	eType = SDB_DBTYPE_LONGVARBINARY; break;
			case SQL_DATE: 			eType = SDB_DBTYPE_DATE; break;
			case SQL_TIME: 			eType = SDB_DBTYPE_TIME; break;
			case SQL_TIMESTAMP: 	eType = SDB_DBTYPE_TIMESTAMP; break;
			default:				eType = SDB_DBTYPE_UNKNOWN; // wird nicht gebunden
		}

		pColumn = pConnection->CreateColumn(aParameterName,eType,pcbColDef,pibScale);
		if (pfNullable == SQL_NULLABLE)
			pColumn->SetFlag(SDB_FLAGS_NULLALLOWED);
		pColumn->SetFlag(SDB_FLAGS_PARAM_IN);
		pColumn->SetDefaultValue(aDefault);

		// Column-Beschreibung hinzufuegen.
		aParameterColumns->AddColumn(pColumn);
	}	
	// Parameter-Strukturbeschreibung fertig.
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::BindParameter()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	aStatus.Clear();
	AllocParamData();
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::BindColumn(SQLHANDLE aStmt, const SdbColumn& rCol, UINT16 nPos, const ODbVariantRef pVariable)
{
	RETCODE nRetcode;
	SWORD   fSqlType;
	SWORD   fCType;
	SDWORD  nMaxLen = 0;
	void*   pData = pParamBuffer[nPos];
	SDWORD* pLen  = &pParamLength[nPos];

	switch(rCol.GetType())
	{
		case SDB_DBTYPE_CHAR:           fCType   = SQL_C_CHAR;
										fSqlType = SQL_CHAR; break;
		case SDB_DBTYPE_VARCHAR:        fCType   = SQL_C_CHAR;
										fSqlType = SQL_VARCHAR; break;
		case SDB_DBTYPE_LONGVARCHAR:    fCType = SQL_C_CHAR;
										fSqlType = SQL_LONGVARCHAR; break;
		case SDB_DBTYPE_DECIMAL:        fCType = SQL_C_CHAR;
										fSqlType = SQL_DECIMAL; break;
		case SDB_DBTYPE_CURRENCY:
		case SDB_DBTYPE_NUMERIC:        fCType   = SQL_C_CHAR;
										fSqlType = SQL_NUMERIC; break;
		case SDB_DBTYPE_TINYINT:        fCType = SQL_C_SHORT;
										fSqlType = SQL_TINYINT; break;
		case SDB_DBTYPE_SMALLINT:       fCType = SQL_C_SHORT;
										fSqlType = SQL_SMALLINT; break;
		case SDB_DBTYPE_BOOLEAN:		fCType = SQL_C_BIT;
										fSqlType = SQL_BIT; break;
		case SDB_DBTYPE_INTEGER:        fCType = SQL_C_LONG;
										fSqlType = SQL_INTEGER; break;
		case SDB_DBTYPE_BIGINT:         fCType = SQL_C_CHAR;
										fSqlType = SQL_BIGINT; break;
		case SDB_DBTYPE_REAL:           fCType = SQL_C_FLOAT;
										fSqlType = SQL_REAL; break;
		case SDB_DBTYPE_DOUBLE:         fCType = SQL_C_DOUBLE;
										fSqlType = SQL_DOUBLE; break;
		case SDB_DBTYPE_BINARY:         fCType = SQL_C_BINARY;
										fSqlType = SQL_BINARY; break;
		case SDB_DBTYPE_VARBINARY:      fCType = SQL_C_BINARY;
										fSqlType = SQL_VARBINARY; break;
		case SDB_DBTYPE_LONGVARBINARY:  fCType = SQL_C_BINARY;
										fSqlType = SQL_LONGVARBINARY; break;
		case SDB_DBTYPE_DATE:           fCType = SQL_C_TIMESTAMP;
										fSqlType = SQL_DATE; break;
		case SDB_DBTYPE_TIME:           fCType = SQL_C_TIMESTAMP;
										fSqlType = SQL_TIME; break;
		case SDB_DBTYPE_TIMESTAMP:      fCType = SQL_C_TIMESTAMP;
										fSqlType = SQL_TIMESTAMP; break;
		default:                        fCType = SQL_C_BINARY;
										fSqlType = SQL_LONGVARBINARY; break;
	}

	SWORD fParamType =	rCol.IsInParameter() ?
							( rCol.IsOutParameter() ? 	SQL_PARAM_INPUT_OUTPUT : SQL_PARAM_INPUT)
											 :	SQL_PARAM_OUTPUT;
	// memofelder duerfen nicht Null sein
	if (!pVariable.is() || pVariable->isNull())
	{
		*pLen = SQL_NULL_DATA;
		nRetcode = NADABindParameter(aStmt,
									nPos,
									fParamType,
									fCType,
									fSqlType,
									0,
									0,
									NULL,
									nMaxLen,
									pLen
									);
	}
	else
	{
		try
		{
			switch (rCol.GetType())
			{
				case SDB_DBTYPE_CHAR:
				case SDB_DBTYPE_VARCHAR:
				{
					ByteString aString(pVariable->toString().getStr(),GetConnection()->GetCharacterSet());
					nMaxLen = rCol.GetPrecision();
					*pLen = SQL_NTS;
					*((ByteString*)pData) = aString;

					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}   break;
				case SDB_DBTYPE_BIGINT:
				case SDB_DBTYPE_DECIMAL:
				case SDB_DBTYPE_CURRENCY:
				case SDB_DBTYPE_NUMERIC:
				{
					ByteString aString(SdbTools::ToString(ODbTypeConversion::toDouble(*pVariable),rCol.GetScale()),GetConnection()->GetCharacterSet());
					nMaxLen = aString.Len();
					*pLen = SQL_NTS;
					*((ByteString*)pData) = aString;
					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}   break;
				case SDB_DBTYPE_BOOLEAN:
					*((BOOL*)pData) = ODbTypeConversion::toBOOL(*pVariable) ? TRUE : FALSE;
					*pLen = sizeof(BOOL);
					break;
				case SDB_DBTYPE_TINYINT:
				case SDB_DBTYPE_SMALLINT:
					*((INT16*)pData) = ODbTypeConversion::toINT16(*pVariable);
					*pLen = sizeof(INT16);
					break;
				case SDB_DBTYPE_INTEGER:
					*((INT32*)pData) = ODbTypeConversion::toINT32(*pVariable);
					*pLen = sizeof(INT32);
					break;
				case SDB_DBTYPE_REAL:
					*((float*)pData) = ODbTypeConversion::toFloat(*pVariable);
					*pLen = sizeof(float);
					break;
				case SDB_DBTYPE_DOUBLE:
					*((double*)pData) = ODbTypeConversion::toDouble(*pVariable);
					*pLen = sizeof(double);
					break;
				case SDB_DBTYPE_BINARY:
				case SDB_DBTYPE_VARBINARY:
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
					{
						const ::staruno::Sequence<sal_Int8>& rData = pVariable->getBytes();
						pData = (void*)rData.getConstArray();
						*pLen = rData.getLength();
						nMaxLen = rCol.GetPrecision();
					}	break;
				case SDB_DBTYPE_UNKNOWN:
				case SDB_DBTYPE_LONGVARBINARY:
				case SDB_DBTYPE_LONGVARCHAR:
				{
					// adabas does not need to have the complete length of data
					// we have to transfer the data in chunks
					pData = (void*)&rCol;					
					*pLen = (SDWORD)SQL_LEN_DATA_AT_EXEC(0);
				}   break;
				case SDB_DBTYPE_DATE:
				case SDB_DBTYPE_TIME:
				case SDB_DBTYPE_TIMESTAMP:
					ToTimeStamp(*(TIMESTAMP_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(TIMESTAMP_STRUCT);
			}
		}
		catch ( ... )
		{
			String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
			strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());			
			return FALSE;
		}
		

		nRetcode = NADABindParameter(aStmt,
						  nPos,
						  fParamType,
						  fCType,
						  fSqlType,
						  rCol.GetPrecision(),
						  rCol.GetScale(),
						  pData,
						  nMaxLen,
						  pLen);
	}

	SetStatus(nRetcode);
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::OpenImpl(SdbCursorType eType ,UINT16 eOpt)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

	if (IsPrepared())
	{
		if (IsOpen())
			// Schon einmal Execute ausgefuehrt.
			// Vor dem naechsten Execute muss der Cursor erst geschlossen werden
			// (Option CLOSE).
			NADAFreeStmt(m_aStatementHandle,SQL_CLOSE);
			// Ergebnis ignorieren (bei Statements ohne Ergebnismenge ggf. Fehler ...)

		// Parameter uebertragen
		UINT16 nCount(aParameterColumns->Count());
		for (UINT16 i=1 ; i < nCount;i++)
			BindColumn(m_aStatementHandle, *(*aParameterColumns)[i], i, (*aParameterRow)[i]);
		// Kann laenger dauern, daher mit Aufruf des Busy Callback
	}
	else
	{
		Translate();
		if (! aStatus.IsSuccessful())
			return FALSE;
	}

	// Anpassen  der Menge der zu bertragenden Daten
	if (nCurrentMaxFieldLen != nMaxFieldLen)
	{
		nRetcode = NADASetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_LENGTH, (SQLPOINTER)nMaxFieldLen,SQL_IS_INTEGER);		
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState() == aValueChangedState)
				NADAGetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_LENGTH, &nMaxFieldLen,0,0);				

			nCurrentMaxFieldLen = nMaxFieldLen;
		}
		else
			// Wieder zuruecksetzen
			nMaxFieldLen = nCurrentMaxFieldLen;
	}

	if (nCurrentMaxRows != nMaxRows)
	{
		nRetcode = NADASetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_ROWS, (SQLPOINTER)nMaxRows,SQL_IS_INTEGER);		
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState() == aValueChangedState)
				NADAGetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_ROWS, &nMaxRows,0,0);				

			nCurrentMaxRows = nMaxRows;
		}
		else
			// Wieder zuruecksetzen
			nMaxRows = nCurrentMaxRows;
	}

	if (IsPrepared())
	{
		// execute kann abgebrochen werden
		BEGIN_CANCELABLE();
		{
			SdbCancelableCoreCall aCoreCallFlag(*this);
			nRetcode = NADAExecute(m_aStatementHandle);
		}
		END_CANCELABLE();
	}
	else
	{
		BEGIN_CANCELABLE();
		{
			SdbCancelableCoreCall aCoreCallFlag(*this);
			ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
			nRetcode = NADAExecDirect(m_aStatementHandle,
									   (SDB_ADA_CHAR *) aSql.GetBuffer(),
										aSql.Len());
		}
		END_CANCELABLE();
	}
	// Parameter at Execution Time erforderlich?
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;

		nRetcode = NADAParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS				&&
				nRetcode != SQL_SUCCESS_WITH_INFO	&&
				nRetcode != SQL_NEED_DATA) 
				break;

			if (aParameterColumns->Count() == 1)
			{
				// Keine Strukturbeschreibung fuer die Parameter Row oder keine Parameter Row.
				// Das darf nicht sein - Statement-Ausfuehrung abbrechen!
				nRetcode = NADACancel(m_aStatementHandle);
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1093"),
							aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_PARAMETER_NUMBER))),
							0, String() );
				// Genaugenommen eigentlich: Parameter missing ...
				return FALSE;
			}

			// Daten fuer diesen Parameter an DB2 uebergeben ...
			// Die Nr. der zugehoerigen Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			//UINT16 nPos = (UINT16)((UINT32)prgbValue);
			PutData(m_aStatementHandle, *pCol, (*aParameterRow)[pCol->GetId()]); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
			{
				// Bei Fehler Statement-Ausfuehrung abbrechen!
				nRetcode = NADACancel(m_aStatementHandle);
				return FALSE;
			}

			nRetcode = NADAParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
	}

	// Resultat des SQLExec[Direct] oder des letzten SqlParamData
	// ist das Resultat des gesamten Statements ...
	SetStatus(nRetcode);
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::OnOpen()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	SdbStatus aTempStat(aStatus);

	if (!IsOpen())          // resultset erst nach execute beschreiben!!!
							// und nur wenn noch nicht geffnet
	{
		if (!aResultColumns.Is())
			aResultColumns = GetConnection()->CreateColumns();
		DescribeResult();
	}

	// Bei Statements, die keine Results liefern (insb. UPDATE, DELETE, INSERT)
	// ist die Anzahl der vom Statement betroffenen Datensaetze interessant:
	if (aResultColumns->Count() == 1)
	{
		SDWORD pcrow=-1;
		NADARowCount(m_aStatementHandle, &pcrow);

		if (pcrow >= 0)
			SetRowCount((UINT32)pcrow);
	}

	// wenn ich hier schon mit einem nicht erfolgreichen Status reingekommen bin, muss ich das fuer die Basisklasse
	// wiederherstellen
	if (!aTempStat.IsSuccessful())
		aStatus = aTempStat;

	return SdbCursor::OnOpen();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenTables(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
							  const sal_Unicode* pTable, const sal_Unicode* pType)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pType		? (aCOL = ByteString(pType,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode = NADATables(m_aStatementHandle,
							(SDB_ADA_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_ADA_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_ADA_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_ADA_CHAR *) pCOL, pType ? SQL_NTS : 0);
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenColumns(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, const sal_Unicode* pColumn)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pColumn		? (aCOL = ByteString(pColumn,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = NADAColumns(m_aStatementHandle,
							(SDB_ADA_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_ADA_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_ADA_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_ADA_CHAR *) pCOL, pColumn ? SQL_NTS : 0);

	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenTypeInfo(INT16 nType)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode = NADAGetTypeInfo(m_aStatementHandle, nType);
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenPrimaryKeys(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = NADAPrimaryKeys(m_aStatementHandle,
								(SDB_ADA_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pPKN, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenTablePrivileges(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = NADATablePrivileges(m_aStatementHandle,
								   (SDB_ADA_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								   (SDB_ADA_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								   (SDB_ADA_CHAR *) pPKN, pTable ? SQL_NTS : 0);
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenIndices(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, BOOL bUnique)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;
	

	SQLRETURN nRetcode = NADAStatistics(m_aStatementHandle,
							 (SDB_ADA_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							 (SDB_ADA_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							 (SDB_ADA_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							 bUnique ? SQL_INDEX_UNIQUE : SQL_INDEX_ALL,
							 SQL_QUICK);

	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenForeignKeys(const sal_Unicode* pPKQualifier, const sal_Unicode* pPKOwner, const sal_Unicode* pPKName,
									const sal_Unicode* pFKQualifier, const sal_Unicode* pFKOwner, const sal_Unicode* pFKName)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN, aFKQ, aFKO, aFKN;

	const char	*pPKQ = pPKQualifier	? (aPKQ = ByteString(pPKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pPKOwner		? (aPKO = ByteString(pPKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pPKName			? (aPKN = ByteString(pPKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL, 
				*pFKQ = pFKQualifier	? (aFKQ = ByteString(pFKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL, 
				*pFKO = pFKOwner		? (aFKO = ByteString(pFKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL, 
				*pFKN = pFKName			? (aFKN = ByteString(pFKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = NADAForeignKeys(m_aStatementHandle,
								(SDB_ADA_CHAR *) pPKQ, pPKQualifier ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pPKO, pPKOwner ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pPKN, pPKName ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pFKQ, pFKQualifier ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pFKO, pFKOwner ? SQL_NTS : 0,
								(SDB_ADA_CHAR *) pFKN, pFKName ? SQL_NTS : 0
		   );
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenProcedures(const String& rQualifier, const String& rSchema, const String& rProcName)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding());

	SQLRETURN nRetcode = NADAProcedures(m_aStatementHandle,
								 (SDB_ADA_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ADA_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ADA_CHAR *) aP.GetBuffer(), aP.Len()
		   );
	
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenProcedureColumns(const String& rQualifier, const String& rSchema,
										 const String& rProcName, const String& rProcColumnName)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding()),
		aC(rProcColumnName,gsl_getSystemTextEncoding());

	SQLRETURN nRetcode = NADAProcedureColumns(m_aStatementHandle,
								 (SDB_ADA_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ADA_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ADA_CHAR *) aP.GetBuffer(), aP.Len(),
								 (SDB_ADA_CHAR *) aC.GetBuffer(), aC.Len()
		   );
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenSpecialColumns(const String& rQualifier, const String& rSchema, const String& rTabName)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aT(rTabName,gsl_getSystemTextEncoding());

	SQLRETURN nRetcode = NADASpecialColumns(m_aStatementHandle,SQL_ROWVER,
								 (SDB_ADA_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ADA_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ADA_CHAR *) aT.GetBuffer(), aT.Len(),
								 SQL_SCOPE_TRANSACTION,SQL_NULLABLE
		   );
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}
//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenDatabaseSize(const String& rQualifier)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aStmt("SELECT SERVERDBSIZE, UNUSEDPAGES FROM ");
	(aStmt += ByteString(rQualifier,gsl_getSystemTextEncoding())) += ".SERVERDBSTATISTICS";

	RETCODE nRetcode = NADAExecDirect(m_aStatementHandle,(SDB_ADA_CHAR *) aStmt.GetBuffer(),aStmt.Len());
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}
//-----------------------------------------------------------------------------
void SdbAdaCursor::OpenDatabaseDevPaths(const String& rQualifier)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aStmt("SELECT DEVSPACENAME FROM ");
	(aStmt += ByteString(rQualifier,gsl_getSystemTextEncoding())) += ".DATADEVSPACES";

	RETCODE nRetcode = NADAExecDirect(m_aStatementHandle,(SDB_ADA_CHAR *) aStmt.GetBuffer(),aStmt.Len());
	SetStatus(nRetcode);
	if (! aStatus.IsSuccessful()) return;
	Open4Info();
}
//-----------------------------------------------------------------------------
void SdbAdaCursor::CancelExecution()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	if (IsCoreCall())
	{
		DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE, "interner Fehler: StatementHandle ungueltig");
		NADACancel(m_aStatementHandle);
	}
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::AllocParamData()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	UINT16 nCount(aParameterColumns->Count());
	if (nCount <= 1)        // keine columns
		return;

	DBG_ASSERT(pParamBuffer == NULL,"Interner Fehler");
	pParamBuffer = (void**) new char*[nCount];
	pParamLength = new INT32[nCount];
	memset(pParamLength, 0, nCount * sizeof(INT32));
	memset(pParamBuffer, 0, nCount * sizeof(char*));

	for (UINT16 i=1; i < nCount; i++)
	{
		const SdbColumn& rCol   = *(*aParameterColumns)[i];
		INT32& rLength          = pParamLength[i];
		rLength                 = rCol.GetLength();

		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				pParamBuffer[i] = new ByteString;
				break;
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
				rLength += 2;
				pParamBuffer[i] = new ByteString;
				break;
			case SDB_DBTYPE_DOUBLE:
				rLength = sizeof(double);
				pParamBuffer[i] = new double;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_UNKNOWN:
				rLength = 0;
				pParamBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_DATE:
			case SDB_DBTYPE_TIME:
			case SDB_DBTYPE_TIMESTAMP:
				rLength = sizeof(TIMESTAMP_STRUCT);
				pParamBuffer[i] = new TIMESTAMP_STRUCT;
				break;			
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				rLength = sizeof(INT16);
				pParamBuffer[i] = new INT16;
				break;
			case SDB_DBTYPE_BOOLEAN:
				rLength = sizeof(BOOL);
				pParamBuffer[i] = new BOOL;
				break;
			case SDB_DBTYPE_INTEGER:
				rLength = sizeof(INT32);
				pParamBuffer[i] = new INT32;
				break;
			case SDB_DBTYPE_BOOKMARK:
				pParamBuffer[i] = new INT32;
				break;
			case SDB_DBTYPE_REAL:
				rLength = sizeof(float);
				pParamBuffer[i] = new float;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				pParamBuffer[i] = new BYTE[rLength];
				break;
		}
	}
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::FreeParamData()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	UINT16 nCount = aParameterColumns.Is() ? aParameterColumns->Count() : 0;
	if (nCount <= 1) return;

	delete pParamLength;
	pParamLength = NULL;

	if (pParamBuffer == NULL)
		return;

	for (UINT16 i = 0; i < nCount; i++)
	{
		const SdbColumn& rCol = *(*aParameterColumns)[i];
		void* pData = pParamBuffer[i];
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				delete (ByteString*)pData;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_UNKNOWN:
				delete (char*)pData;
				break;
			case SDB_DBTYPE_BOOLEAN:
				delete (BOOL*)pData;
				break;
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_BOOKMARK:
				delete (INT32*)pData;
				break;			
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				delete (INT16*)pData;
				break;
			case SDB_DBTYPE_REAL:
				delete (float*)pData;
				break;
			case SDB_DBTYPE_DOUBLE:
				delete (double*)pData;
				break;
			case SDB_DBTYPE_DATE:
			case SDB_DBTYPE_TIME:
			case SDB_DBTYPE_TIMESTAMP:
				delete (TIMESTAMP_STRUCT*)pData;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				delete (BYTE *) pData;
				break;
			default:
				DBG_ERROR("Fehlerhafter DatenTyp");
		}
	}

	__DELETE(nCount) pParamBuffer;
	pParamBuffer = NULL;
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::CloseImpl()
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	// Vater nicht mehr vorhanden, dann alles freigeben
	if (!GetParent())
	{
		if (m_aStatementHandle != SQL_NULL_HANDLE)
		{
			NADAFreeHandle(SQL_HANDLE_STMT, m_aStatementHandle);
			m_aStatementHandle = SQL_NULL_HANDLE;
		}
	}
	else
	{
		DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
		// wenn einer der drei nicht klappt, dann den Handle wegschmeissen
		if (NADAFreeStmt(m_aStatementHandle,SQL_RESET_PARAMS) ||
			NADAFreeStmt(m_aStatementHandle,SQL_UNBIND) ||
			NADAFreeStmt(m_aStatementHandle,SQL_CLOSE))
		{
			SetStatus(NADAFreeHandle(SQL_HANDLE_STMT, m_aStatementHandle));
			if (aStatus.IsSuccessful())
			{
				RETCODE rRetcode = NADAAllocHandle(SQL_HANDLE_STMT, GetAdaConnection()->ConnectionHandle(),&m_aStatementHandle);
				SetStatus(rRetcode, SQL_HANDLE_DBC, GetAdaConnection()->ConnectionHandle());
			}
			if (!aStatus.IsSuccessful())
				m_aStatementHandle = SQL_NULL_HANDLE;
		}
	}
	FreeParamData();
	return TRUE;
}


//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,
							 SdbCursorMode& rMode)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

	DBG_ASSERT(eCursorPosition == SDB_POS_NEXT, "DB2Cursor: Falsche MoveRichtung, nur Next zulaessig!");
	if (eCursorPosition != SDB_POS_NEXT)
	{
		aStatus.SetDriverNotCapableError();
	}
	else
	{
		nRetcode = NADAFetch(m_aStatementHandle);
		SetStatus(nRetcode);
	}
	if (!aStatus.IsSuccessful()) 
		return FALSE;

	FixResultRow();
	if (!aStatus.IsSuccessful()){
		rMode = SdbCursor::SDB_MOD_INVALID;
		aStatus.Clear();
	}
	return aStatus.IsSuccessful();
}


//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::GetData(SQLHANDLE aStmt, const SdbColumn& rCol, ODbVariantRef pVariable, UINT16 nPos)
{
	RETCODE nRetcode;
	SDWORD   pcbValue = 0;          // Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)

	// Zaehlung der DB2-Columns und Zaehlung unserer Columns beginnt bei eins!
		// Column #0 ist die Bookmark!

#define GETDATA(fCType,rgbValue,cbValueMax) \
    { \
        nRetcode = NADAGetData(aStmt,(nPos),(fCType),(rgbValue),(cbValueMax),&pcbValue); \
        if (nRetcode != SQL_SUCCESS)    \
            SetStatus(nRetcode);  \
        if (aStatus.IsError())     \
        {                          \
            DBG_ERROR((((ByteString("SQLGetData fehlgeschlagen!: Column ") +=				\
			ByteString::CreateFromInt32(nPos)) += ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding())).GetBuffer()) ;	\
            pVariable->setNull();  \
            aStatus.Clear();       \
            break;                 \
        }                          \
        else if (pcbValue == SQL_NULL_DATA) { \
            pVariable->setNull(); \
            break; \
        } \
    }

	// Gebundene Variable. Je nach Datenbanktyp als entsprechenden
	// C-Datentyp (passend zur Variablen) abholen.

	// nyi: festen Variablentyp beruecksichtigen und konvertieren!
	switch (rCol.GetType())
	{
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
		case SDB_DBTYPE_BIGINT:
		case SDB_DBTYPE_CURRENCY:
			GETDATA(SQL_C_CHAR,aBuffer.aCharArray,sizeof aBuffer.aCharArray);
			pVariable->setDouble(SdbTools::ToDouble((const char *)aBuffer.aCharArray));			
			break;		
		case SDB_DBTYPE_TINYINT:
		case SDB_DBTYPE_SMALLINT:
			GETDATA(SQL_C_SHORT,&aBuffer.nINT16,sizeof(INT16));
			pVariable->setINT16(aBuffer.nINT16);
			// nyi: signed/unsigned
			break;

		case SDB_DBTYPE_INTEGER:
			GETDATA(SQL_C_LONG,&aBuffer.nINT32,sizeof(INT32));
			pVariable->setINT32(aBuffer.nINT32);
			// nyi: signed/unsigned
			break;

		case SDB_DBTYPE_REAL:
			GETDATA(SQL_C_FLOAT,&aBuffer.nFloat,sizeof(float));
			pVariable->setFloat(aBuffer.nFloat);
			break;

		case SDB_DBTYPE_DOUBLE:
			GETDATA(SQL_C_DOUBLE,&aBuffer.nDouble,sizeof(double));
			pVariable->setDouble(aBuffer.nDouble);
			break;

		case SDB_DBTYPE_BOOLEAN:
			GETDATA(SQL_C_BIT,&aBuffer.nBOOL,sizeof(BOOL));
			pVariable->setBOOL( aBuffer.nBOOL);
			break;

		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_LONGVARCHAR:
		{
			// Erstmal versuchen, die Daten mit dem kleinen Puffer
			// abzuholen:
			SDWORD nMaxLen = sizeof aBuffer.aCharArray - 1;
			GETDATA(SQL_C_CHAR,aBuffer.aCharArray,nMaxLen);

			SWORD nLen = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
			aBuffer.aCharArray[nLen] = 0;												
			UniString aData((const sal_Char*)aBuffer.aCharArray, GetConnection()->GetCharacterSet());
								
			// Es handelt sich um Binaerdaten, um einen String, der fuer
			// StarView zu lang ist oder der Treiber kann die Laenge der
			// Daten nicht im voraus bestimmen - also als MemoryStream
			// speichern.				
			while ((pcbValue == SQL_NO_TOTAL && aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
					aStatus.SQLState() == aTruncState) || pcbValue > nMaxLen)
			{
				// Bei Strings wird der Puffer nie ganz ausgenutzt
				// (das letzte Byte ist immer ein NULL-Byte, das
				// aber bei pcbValue nicht mitgezaehlt wird)
				if (pcbValue != SQL_NO_TOTAL && (pcbValue - nMaxLen) < nMaxLen)
					nLen = pcbValue - nMaxLen;
				else
					nLen = nMaxLen;

				// Solange eine "truncation"-Warnung vorliegt, weiter Daten abholen
				GETDATA(SQL_C_CHAR,aBuffer.aCharArray, nLen + 1);
				aBuffer.aCharArray[nLen] = 0;				

				aData += UniString((const sal_Char*)aBuffer.aCharArray, GetConnection()->GetCharacterSet());
			}
			
			// delete all blanks
			if (SDB_DBTYPE_CHAR == rCol.GetType())
				aData.EraseTrailingChars();

			pVariable->setString(aData);							
		} break;
		case SDB_DBTYPE_BINARY:
		case SDB_DBTYPE_VARBINARY:
		case SDB_DBTYPE_LONGVARBINARY:
		{
			// Erstmal versuchen, die Daten mit dem kleinen Puffer
			// abzuholen:
			SDWORD nMaxLen = sizeof aBuffer.aCharArray;
			GETDATA(SQL_C_BINARY,aBuffer.aCharArray,nMaxLen);
			// Bei Fehler bricht der GETDATA-Makro mit return ab,
			// bei NULL mit break!

			// In pcbValue steht jetzt entweder die Gesamtlaenge
			// der Daten in diesem Feld oder aber SQL_NO_TOTAL (wenn
			// der Treiber uns die Laenge nicht liefern kann).
			// [pcbValue enthaelt immer die (Rest-)Laenge VOR
			// dem letzten Aufruf von GetData!]

			// Es handelt sich um Binaerdaten, um einen String, der fuer
			// StarView zu lang ist oder der Treiber kann die Laenge der
			// Daten nicht im voraus bestimmen - also als MemoryStream
			// speichern.

			SDWORD nBytes = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
			::staruno::Sequence<sal_Int8> aData((sal_Int8*)aBuffer.aCharArray, nBytes);

			while ((pcbValue == SQL_NO_TOTAL && aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
					aStatus.SQLState() == aTruncState) || pcbValue > nMaxLen)
			{
				// Bei Strings wird der Puffer nie ganz ausgenutzt
				// (das letzte Byte ist immer ein NULL-Byte, das
				// aber bei pcbValue nicht mitgezaehlt wird)
				if (pcbValue != SQL_NO_TOTAL && (pcbValue - nMaxLen) < nMaxLen)
					nBytes = pcbValue - nMaxLen;
				else
					nBytes = nMaxLen;

				GETDATA(SQL_C_BINARY,aBuffer.aCharArray,nBytes);

				INT32 nLen = aData.getLength();				
				aData.realloc(nLen + nBytes);
				memcpy(aData.getArray() + nLen, aBuffer.aCharArray, nBytes);
			}
			pVariable->setBytes(aData);
		} break;
		case SDB_DBTYPE_DATE:
		case SDB_DBTYPE_TIME:
		case SDB_DBTYPE_TIMESTAMP:
		{
			DBG_ASSERT(sizeof(aBuffer.aTimestamp) == 16,"SdbAdaCursor::FetchRow: fehlerhaftes Alignment in DB2-Datumsstruktur");
			GETDATA(SQL_C_TIMESTAMP,&aBuffer.aTimestamp,sizeof(aBuffer.aTimestamp));
			pVariable->setDateTime(ToDateTime(aBuffer.aTimestamp));
		} break;
		default:
			DBG_ERROR("SdbAdaCursor: interner Fehler: unbekannter SDB-Datentyp"); break; // Unbekannter Datenbanktyp
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::PutData(SQLHANDLE aStmt, const SdbColumn& rCol, const ODbVariantRef pVariable)
{
	DBG_CHKTHIS(SdbAdaCursor,NULL);
	DBG_ASSERT(aStmt != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	RETCODE nRetcode;

#define PUTDATA(rgbValue,cbValue) \
    { \
        nRetcode = NADAPutData(aStmt,(rgbValue),(cbValue)); \
        SetStatus(nRetcode); \
        if (aStatus.IsError()) return FALSE; \
    }

	switch (rCol.GetType())
	{
		case SDB_DBTYPE_UNKNOWN:
		case SDB_DBTYPE_LONGVARCHAR:
		case SDB_DBTYPE_LONGVARBINARY:
		{					
			// first get the data and the complete length and than send the data in chunks
			String aString;
			INT32 nLen  = 0;
			const BYTE* pData = NULL;

			if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
			{					
				const ::staruno::Sequence<sal_Int8>& rData = pVariable->getBytes();
				nLen  = rData.getLength();
				pData = (const BYTE*)rData.getConstArray();
			}
			else
			{				
				if (!pVariable->isNull())
				{
					try
					{
						aString = pVariable->toString().getStr();
						//	aString.Convert(CHARSET_SYSTEM,GetConnection()->GetCharacterSet());
					}
					catch ( ... )
					{
						String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
						strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
						aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());			
						return FALSE;
					}
					
				}
				nLen = aString.Len();
				pData = (const sal_uInt8*) aString.GetBuffer();				
			}

#define MAX_BUFFER_SIZE 1024L
			// sending the data in 4k blocks
			while ((nRetcode = NADAPutData(aStmt, (void*)pData, min(nLen, MAX_BUFFER_SIZE))) == 0 
					&& nLen > MAX_BUFFER_SIZE)
			{
				pData += MAX_BUFFER_SIZE;
				nLen -= MAX_BUFFER_SIZE;
			}				
			
			SetStatus(nRetcode);
		}
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
void SdbAdaCursor::FixResultRow()
{
	DBG_ASSERT(aResultRow.Is(), "Keine Ergebniszeile vorhanden!!");
	DBG_CHKTHIS(SdbAdaCursor,NULL);

	SdbConnection* pConnection = GetConnection();
	UINT16 nCount(aResultColumns->Count());
	for (UINT16 i=1; i < nCount; i++) // Wird BOOKMARK-Spalte verwendet
	{
		ODbVariantRef &pVal       = (*aResultRow)[i];
		if (!pVal.is())
			continue;   // nicht gebunden

		GetData(m_aStatementHandle, *(*aResultColumns)[i], pVal, i);
	}
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::DeleteImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::InsertImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::UpdateImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbAdaCursor::DetermineRowCountImpl(INT32&)
{
	return FALSE;
}


