/*************************************************************************
 *
 *  $RCSfile: adaidx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif
#ifndef SDB_ADATAB_HXX
#include "adatab.hxx"
#endif
#ifndef SDB_ADACONN_HXX
#include "adaconn.hxx"
#endif
#ifndef SDB_ADACURS_HXX
#include "adacurs.hxx"
#endif
#ifndef _SDB_ADAIDX_HXX
#include "adaidx.hxx"
#endif
#ifndef _SDB_SDBRESID_HRC
#include "sdbresid.hrc"
#endif


DBG_NAME(SdbAdaIndex);

//-----------------------------------------------------------------------------
SdbAdaIndex::SdbAdaIndex(SdbAdaTable* pRefTable)
			 :SdbIndex(pRefTable){}


//-----------------------------------------------------------------------------
BOOL SdbAdaIndex::OpenImpl()
{
	SdbTable* pTable = GetTable();
	SdbConnection* pConnection = GetConnection();
	SdbAdaCursor* pCursor = (SdbAdaCursor*)pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY|SDB_READONLY);

	pCursor->AddRef();

	pCursor->OpenIndices(NULL,pTable->OwnerName().GetBuffer(),pTable->Name().GetBuffer());
	aStatus = pCursor->Status();

	/*	Reihenfolge
		1	TABLE_QUALIFIER
		2	TABLE_OWNER
		3	TABLE_NAME
		4	NON_UNIQUE
		5	INDEX_QUALIFIER
		6	INDEX_NAME
		7	TYPE
		8	SEQ_IN_INDEX
		9	COLUMN_NAME
		10	COLLATION
	*/

	UniString ustrQual(aQualifierName);
	UniString ustrName(aName);

	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();

		ODbVariantRef pQualifier = rRow[5];
		ODbVariantRef pName = rRow[6];
		ODbVariantRef pType = rRow[7];
		if (pType.is() && !pType->isNull() && pType->getINT16() != SQL_TABLE_STAT &&
			(!pQualifier.is() || pQualifier->isNull() || UniString(pQualifier->getString()) == ustrQual) &&
			pName.is() && !pName->isNull() && UniString(pName->getString()) == ustrName )
		{
			ODbVariantRef pValue = rRow[4];
			if (pValue.is() && !pValue->isNull() && pValue->getINT16() == 0)
				SetStyle(SDB_IDX_UNIQUE);

			pValue = rRow[9];
			const SdbColumn* pColumn = NULL;
			if (pValue.is() && !pValue->isNull())
				pColumn = pTable->OriginalColumns()->Column(pValue->getString().getStr());

			if (pColumn)
			{
				SdbColumn* pNewCol = pTable->CreateColumn(*pColumn);
				aOriginalColumns->AddColumn(pNewCol);
				pValue = rRow[10];
				if (pValue.is() && !pValue->isNull() && !pValue->getString().compareToAscii("D"))
					pNewCol->SetFlag(SDB_FLAGS_DESCENDING);
			}
			else
			{
				String aText = String(SdbResId(STR_STAT_INDEX_COLUMN_NOT_FOUND));
				aText.SearchAndReplace(String::CreateFromAscii("#"),pValue->getString().getStr());
				aText.SearchAndReplace(String::CreateFromAscii("%"),pTable->GetFullName());
				aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
				break;
			}
		}
		pCursor->Next();
	}
	pCursor->Close();
	/*		1 	PKTABLE_CAT
		2	PKTABLE_SCHEM
		3	PKTABLE_NAME
		4	PKCOLUMN_NAME
		5 	FKTABLE_CAT
		6 	FKTABLE_SCHEM
		7 	FKTABLE_NAME
		8 	FKCOLUMN_NAME
		9 	KEY_SEQ
		10 	UPDATE_RULE
		11 	DELETE_RULE
		12 	FK_NAME			// constraint name
		13  PK_NAME
		14  DEFERRABILITY
*/

	
	pCursor->OpenForeignKeys(NULL,pTable->OwnerName().GetBuffer(),pTable->Name().GetBuffer(),NULL,NULL,NULL);
	aStatus = pCursor->Status();
	if(aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		ODbVariantRef pFkKeyName = rRow[12];
		if(pFkKeyName.is() && !pFkKeyName->isNull() && (UniString(pFkKeyName->getString()) == ustrName))
			bSystemReadOnly = TRUE;
	}
	pCursor->ReleaseRef();
	
	if(!bSystemReadOnly && Name().Len() && 
		((((SdbAdaTable*)pTable)->aPKeyName == Name()) || String::CreateFromAscii("SYSPRIMARYKEYINDEX") == Name()))
		bSystemReadOnly = TRUE;
	
	if (aStatus.IsNoDataFound())
		aStatus.Clear();
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbAdaIndex::DropImpl()
{
	SdbConnection* pConnection = GetConnection();
	String aStatement;

	// mehrere Versuche, da es bei jeder Datenbank unterschiedlich laeuft
	// nachteil, keine genaue Fehlermeldung

	aStatement.AssignAscii("DROP INDEX ");
	if(aQualifierName.Len())
	{
		aStatement += pConnection->IdentifierQuotation(aQualifierName);
		aStatement += '.';
	}
	aStatement += pConnection->IdentifierQuotation(aName);
	if (!pConnection->ExecuteSQL(aStatement))
		aStatus = pConnection->Status();
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbAdaIndex::CreateImpl()
{
	DBG_ASSERT(aColumns->Count() > 1, "SdbAdaIndex::CreateImpl: no columns");

	SdbConnection* pConnection = GetConnection();
	SdbTable* pTable = GetTable();

	String aStatement;
	aStatement.AssignAscii("CREATE ");
	if (IsUnique())
		aStatement.AppendAscii("UNIQUE ");

	aStatement.AppendAscii("INDEX ");
	// wenn ein Owner verwendet wird in der Indexdefinition
	// dann wird dieser als Index Qualifizierer genommen
	// so funktioniert das bei db2
	if (aQualifierName.Len())
	{
		aStatement += pConnection->IdentifierQuotation(aQualifierName);
		aStatement += '.';
	}

	aStatement += pConnection->IdentifierQuotation(aName);
	aStatement.AppendAscii(" ON ");
	aStatement += pConnection->QuoteTableName(pTable->QualifiedName());

	aStatement.AppendAscii("(");
	for (UINT16 i = 1; i < aColumns->Count(); i++)
	{
		aStatement += pConnection->IdentifierQuotation((*aColumns)[i]->GetName());
		if ((*aColumns)[i]->IsDescending())
			aStatement.AppendAscii(" DESC ");
		aStatement.AppendAscii(",");
	}

	aStatement.EraseTrailingChars(',');
	aStatement.AppendAscii(")");

	if (!pConnection->ExecuteSQL(aStatement))
		aStatus = pConnection->Status();

	return aStatus.IsSuccessful();
}


