/*************************************************************************
 *
 *  $RCSfile: adoconn.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: jmarmion $ $Date: 2000/10/06 13:13:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <tools/solar.h>
#ifndef SDB_ADO_IMPL
#include "adoimp.hxx"
#endif
#ifndef SDB_WRAPADO_HXX
#include "wrapado.hxx"
#endif
#ifndef SDB_ADOCONN_HXX
#include "adoconn.hxx"
#endif
#ifndef _SDB_SDBRESID_HRC
#include "sdbresid.hrc"
#endif
#include <oledb.h>
#ifndef _SDB_SDBSTAT_HXX
#include "sdbstat.hxx"
#endif
#ifndef _SDB_SDBREL_HXX
#include "sdbrel.hxx"
#endif
#ifndef _SDB_SDBMAP_HXX
#include "sdbmap.hxx"
#endif

#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASENKEYRULE_HPP_
#include <com/sun/star/data/DatabaseKeyRule.hpp>
#endif


// statische Member initialisieren
UINT16 SdbADOConnection::ConnectionCount = 0;

DBG_NAME(SdbADOConnection);
DBG_NAME(WpOLEBase);
DBG_NAME(WpOLECollection);


#ifndef _SDB_SDBREL_HXX
#include "sdbrel.hxx"
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

static char __READONLY_DATA CascadeRule[] = "CASCADE";
static char __READONLY_DATA SetNullRule[] = "SET NULL";
static char __READONLY_DATA SetDefaultRule[] = "SET DEFAULT";
static char __READONLY_DATA NoActionRule[] = "NO ACTION";

//==================================================================
// Ableitung nur fr das Erzeugen und Refresh, fuer eigenes file
// zu geringfgig
//==================================================================
class SdbADORelation : public SdbRelation
{
	friend class SdbADOConnection;

protected:
	SdbADORelation(SdbConnection* _pConn, const String& _rName)
		:SdbRelation(_pConn, _rName){}

	virtual BOOL DoRefresh();
	void SetName(const String& _rName) {aName = _rName;}
};

//------------------------------------------------------------------
BOOL SdbADORelation::DoRefresh()
{
	SdbConnection* pConnection = GetConnection();
	// den Namen merken, da sich dieser durchaus geaendert haben kann
	if (!((SdbADOConnection*)pConnection)->FillRelation(*this))
		aStatus = pConnection->Status();
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------
void SdbADOConnection::SetADOError(SdbStatus& anotherStatus)
{
	ADOErrors *pErrors = NULL;
	pADOConnection->GetErrors(&pErrors);
	DBG_ASSERT(pErrors,"Error occured without error objects!");
	pErrors->AddRef( );

	String sError;
	String aSQLState;
	// alle aufgelaufenen Fehler auslesen und ausgeben
	INT32 nLen;
	pErrors->get_Count(&nLen);
	if (nLen)
	{
		for (INT32 i = nLen-1; i>=0; i--)
		{
			ADOError *pError = NULL;
			pErrors->get_Item(OLEVariant(i),&pError);
			WpADOError aErr(pError);
			DBG_ASSERT(pError,"No error in collection found! BAD!");
			if(pError)
			{
				sError += aErr.GetSource();
				sError.AppendAscii(" - error number ");
				sError += aErr.GetNumber();
				sError.AppendAscii(": ");
				sError += aErr.GetDescription();
				sError += '\n';
				aSQLState = aErr.GetSQLState();
				aSQLState += '\n';
			}
		}
		anotherStatus.Set(SDB_STAT_ERROR, aSQLState , sError, 0, String());
	}
	pErrors->Release();
}
//------------------------------------------------------------------------
::stardata::XDatabaseDescriptor* SdbADOConnection::GetMetaDataImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
	return new SdbADOMetaData(this);
}
//------------------------------------------------------------------------
SdbADOConnection::SdbADOConnection(SdbEnvironment* pEnv,const String& rNewOptions) : SdbConnection(pEnv, rNewOptions),
	pADOConnection(NULL)
{
	DBG_CTOR(SdbADOConnection,NULL);
	Construct();
};
//------------------------------------------------------------------------
SdbADOConnection::SdbADOConnection(const SdbADOConnection& rConn):SdbConnection(rConn.GetEnvironment(), rConn.Options()),
	pADOConnection(NULL)
{
	DBG_CTOR(SdbADOConnection,NULL);
	// alle zur Oberklasse hinzugefuegten Member sind entweder statisch
	// oder werden in Construct() gesetzt
	Construct();
};



//------------------------------------------------------------------------
void SdbADOConnection::Construct()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);

	cIdentifierQuotation = '`';
	aStatus.Clear();


	// ggf. Dinge tun, die nur einmal fr alle Connections getan
	// werden muessen...
	if (!ConnectionCount++)
	{
		// COM-Library initialisieren
		if FAILED(CoInitialize(NULL))
		{
			// kann bei intaktem Betriebssystem nicht passieren
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000") ,
				String::CreateFromAscii("Error while initializing Microsoft COM"), 0, String());
			return;
		}
	}

};



//------------------------------------------------------------------------
SdbADOConnection::~SdbADOConnection()
{
	DBG_DTOR(SdbADOConnection,NULL);
	DBG_ASSERT(!IsOpen(), "SdbADOConnection: Die Connection wird zerstoert, "
				"ohne vorher geschlossen worden zu sein")

	if(pADOConnection)
		delete pADOConnection;

	// alle Connections abgeraeumt?
	if (!--ConnectionCount)
	{
		// Freigeben: COM-Library, DBEngine
		CoUninitialize();
	}
}

//------------------------------------------------------------------------
BOOL SdbADOConnection::OpenADOConnection()
{
	BOOL bErg=FALSE;
	if(!pADOConnection)
	{

		//
		IClassFactory2* pIUnknown	= NULL;
		IUnknown        *pOuter		= NULL;
		HRESULT         hr;
		hr = CoGetClassObject( ADOS::CLSID_ADOCONNECTION_21,
							  CLSCTX_INPROC_SERVER,
							  NULL,
							  IID_IClassFactory2,
							  (void**)&pIUnknown );

		if( !FAILED( hr ) )
		{
			pIUnknown->AddRef();

			ADOConnection *pCon			= NULL;
			hr = pIUnknown->CreateInstanceLic(  pOuter,
												NULL,
												ADOS::IID_ADOCONNECTION_21,
												ADOS::GetKeyStr(),
												(void**) &pCon);

			if( !FAILED( hr ) )
			{
				String aConnectionString = aOptionsParser.OptionString(String::CreateFromAscii("PROVIDER"));
				pADOConnection = new WpADOConnection(pCon);
				// Class Factory is no longer needed

				bErg = pADOConnection->Open(aConnectionString,aOptionsParser.OptionString(String::CreateFromAscii("UID")),aOptionsParser.OptionString(String::CreateFromAscii("PWD")),adConnectUnspecified);
				pADOConnection->PutCommandTimeout(GetEnvironment()->GetLoginTimeOut());
				//bErg = TRUE;
			}
			pIUnknown->Release();
		}
	}

	return bErg;
}
//------------------------------------------------------------------------
BOOL SdbADOConnection::OpenImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);

	DBG_ASSERT(!IsOpen(),"SdbADOConnection: Verbindung ist schon geoeffnet!");

	//////////////////////////////////////////////////////////////////
	// wenn noch keine Database vorhanden, eine anlegen
	//
	//
	//
	if (!OpenADOConnection())
		return FALSE;  // Status wurde ggf. schon gesetzt

	// CaseSensitivity
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(GetMetaData());
	sal_Int16 nInfo;
	rMetaData->getInfo(::stardata::DatabaseInfo::IDENT_CASE_QUOTED) >>= nInfo;
	switch (nInfo)
	{
		case 1:
			eIdentifierCase = SDB_IC_UPPER;
			break;
		case 2:
			eIdentifierCase = SDB_IC_LOWER;
			break;
		case 3:
			eIdentifierCase = SDB_IC_ALL;
			break;
		default:
			eIdentifierCase = SDB_IC_MIXED;
	}
	rtl::OUString aQuoteChar;
	rMetaData->getInfo(::stardata::DatabaseInfo::IDENTIFIER_QUOTE_CHAR) >>= aQuoteChar;
	cIdentifierQuotation = String(aQuoteChar.getStr()).GetChar(0);

	aStatus.Clear();        //Ergebnis ignorieren
	pTableNames = new SdbStringStlMap(::utl::UStringMixLess(eIdentifierCase & SDB_IC_OBJECT ? true : false));

	return TRUE;
}

//------------------------------------------------------------------------
SdbRelation* SdbADOConnection::CreateRelationImpl(const String& _rName)
{
  return new SdbADORelation(this,_rName);
}


//------------------------------------------------------------------------
BOOL SdbADOConnection::IsCapableImpl(SdbCapability eCap) const
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
	switch (eCap)
	{
		case SDB_CAP_CREATE_TABLE:
		case SDB_CAP_ALTER_TABLE:
		case SDB_CAP_UPDATABLE:
		case SDB_CAP_RENAME_TABLE:
		case SDB_CAP_COMMIT_AUTO:
		case SDB_CAP_COMMIT_MANUAL:
		case SDB_CAP_PRIMARYKEYS:
		case SDB_CAP_FOREIGNKEYS:
		case SDB_CAP_CREATE_INDEX:
		case SDB_CAP_BULKOPTIMIZE:
		case SDB_CAP_JOIN:
		case SDB_CAP_INSERTTABCOLS:
		case SDB_CAP_STATIC_PRIOR:
		case SDB_CAP_KEYSET_PRIOR:
			return TRUE;
			break;

		case SDB_CAP_MULTITHREADACCESS:
			return TRUE;

		default:
			return FALSE;
	}
}


//------------------------------------------------------------------------
SdbTable* SdbADOConnection::CreateTableImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);

	return new SdbADOTable(this);
};

//------------------------------------------------------------------------
SdbView* SdbADOConnection::CreateViewImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
	return NULL;//new SdbADOView(this);
}
//------------------------------------------------------------------------
void SdbADOConnection::FillTableNames(SdbStringStlMap* _pList)
{
	DBG_CHKTHIS(SdbADOConnection, NULL);


	OLEVariant aTableRes,aTmp;

	aTmp.SetNoArg();
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData = GetMetaData();
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	String aNewName;
	OLEVariant  vtEmpty;
	vtEmpty.SetNoArg();
	char* pArray[] =  {"TABLE","SYNONYM","VIEW","ALIAS"};
	OLEVariant varCriteria[4];
	SAFEARRAYBOUND rgsabound[1];
	rgsabound[0].lLbound   = 0;
	rgsabound[0].cElements = 4;
	for(INT32 i=0;i<4;++i)
	{
		HRESULT hr = S_OK;
		// Create elements used in the array
		varCriteria[3].SetString(String::CreateFromAscii(pArray[i]));
		// Create SafeArray Bounds and initialize the array

		SAFEARRAY *psa         = SafeArrayCreate( VT_VARIANT, 1, rgsabound );

		// Set the values for each element of the array
		for( long j = 0 ; j < 4 && SUCCEEDED( hr );j++)
		  hr  = SafeArrayPutElement(psa, &j,&varCriteria[j]);

		// Initialize and fill the SafeArray
		OLEVariant vsa;
		vsa.SetArray(psa,VT_VARIANT);

		// call OpenSchema
		ADORecordset *pRecordset = NULL;
		pADOConnection->OpenSchema(adSchemaTables,vsa,vtEmpty,&pRecordset);
		//SafeArrayDestroy(psa);
		if(!pRecordset)
		   continue;

		WpADORecordset aRecordset(pRecordset);

		aRecordset.MoveFirst();
		while(!aRecordset.IsAtEOF())
		{
			aNewName.Erase();
			ADOFields  *pFields = aRecordset.GetFields();
			pFields->AddRef();
			ADOField* pField=NULL;
			if (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML)
			{
				OLEVariant aCatalog;
				pFields->get_Item(OLEVariant(0),&pField);
				WpADOField aField(pField);
				aField.get_Value(aCatalog);

				if (!aCatalog.IsNull() && aCatalog.GetString().Len())
				{
					aNewName = aCatalog.GetString();
					aNewName += cQualifierSeparator;
				}
			}
			if (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML)
			{
				OLEVariant aSchema;
				pFields->get_Item(OLEVariant(1),&pField);
				WpADOField aField(pField);
				aField.get_Value(aSchema);

				if (!aSchema.IsNull() && aSchema.GetString().Len())
				{
					aNewName += aSchema.GetString();
					aNewName += '.';
				}
			}
			OLEVariant aTable;
			pFields->get_Item(OLEVariant(2),&pField);
			WpADOField aField(pField);
			aField.get_Value(aTable);
			if (!aTable.IsNull() && aTable.GetString().Len())
				aNewName += aTable.GetString();

			(*_pList)[aNewName] = i!=2;

			pFields->Release();

			aRecordset.MoveNext();
		}
		aRecordset.Close();
	}
}
//----------------------------------------------------------------------
BOOL SdbADOConnection::CloseImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);

	DBG_ASSERT(IsOpen(),"SdbADOConnection: Verbindung soll geschlossen werden, obwohl sie gar nicht geoeffnet ist");
	pADOConnection->Close();
	return TRUE;
}

//------------------------------------------------------------------------
void SdbADOConnection::FillTypeList(SdbTypeInfoList& _rList)
{
	DBG_CHKTHIS(SdbADOConnection,NULL);

		HRESULT hr = S_OK;
	// Create elements used in the array
	OLEVariant varCriteria[2];
	const int nCrit = sizeof varCriteria /
					 sizeof varCriteria[0];
	// Create SafeArray Bounds and initialize the array
	SAFEARRAYBOUND rgsabound[1];
	rgsabound[0].lLbound   = 0;
	rgsabound[0].cElements = nCrit;
	SAFEARRAY *psa         = SafeArrayCreate( VT_VARIANT, 1, rgsabound );

	INT32 nPos = 0;
	SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;
	SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;

	// Initialize and fill the SafeArray
	OLEVariant vsa;
	vsa.SetArray(psa,VT_VARIANT);

	OLEVariant aEmpty;
	aEmpty.SetNoArg();

	ADORecordset *pRec=NULL;
	pADOConnection->OpenSchema(adSchemaProviderTypes,vsa,aEmpty,&pRec);

	//SafeArrayDestroy(psa);
	if(!pRec)
		return;

	WpADORecordset aRecordset(pRec);
	SdbTypeInfo* pInfoTimestamp = NULL;
	INT32 nTimestampTypeNum=0;
	OLEVariant aVar;

#define ADO_GETFIELD(Name)	\
	pFields->get_Item(OLEVariant(Name),&pField);	\
	if(!pField)										\
	{												\
		delete pInfo;								\
		pFields->Release();							\
		aRecordset.Close();							\
		return;										\
	}												\
	aField = WpADOField(pField);					\
	aField.get_Value(aVar);

	aRecordset.MoveFirst();
	while(!aRecordset.IsAtEOF())
	{
		ADOFields  *pFields = aRecordset.GetFields();
		pFields->AddRef();

		SdbTypeInfo* pInfo	= NULL;
		ADOField* pField	= NULL;

		WpADOField  aField;
		ADO_GETFIELD(String::CreateFromAscii("DATA_TYPE"));

		INT32 nTypeNum = (INT32)aVar;
		BOOL bUnsigned = FALSE;

		SdbDatabaseType eType = ADOS::SdbTypeFromADOType((ADO_::DataTypeEnum)nTypeNum,bUnsigned);
		if (eType == SDB_DBTYPE_UNKNOWN)
		{
			aRecordset.MoveNext();
			pFields->Release();
			continue;
		}

		pInfo = new SdbTypeInfo;
		pInfo->eType = eType;

		ADO_GETFIELD(String::CreateFromAscii("TYPE_NAME"));
		pInfo->aTypeName        = aVar.GetString();

		// i need a type for the rowver for SQLSpecialColumns
		if(pInfo->aTypeName.CompareToAscii("timestamp") == COMPARE_EQUAL)
		{
			pInfoTimestamp		= pInfo;
			nTimestampTypeNum	= nTypeNum;
		}

		ADO_GETFIELD(String::CreateFromAscii("COLUMN_SIZE"));

		pInfo->nPrecision		=  aVar.GetInt32();

		ADO_GETFIELD(String::CreateFromAscii("MINIMUM_SCALE"));
		pInfo->nMinimumScale	= aVar.IsNull() ? 0 : aVar.GetByte();

		ADO_GETFIELD(String::CreateFromAscii("MAXIMUM_SCALE"));
		pInfo->nScale			= aVar.IsNull() ? 0 : aVar.GetByte();

		ADO_GETFIELD(String::CreateFromAscii("FIXED_PREC_SCALE"));
		pInfo->bCurrency		= aVar.IsNull() ? FALSE : (BOOL)aVar;

		ADO_GETFIELD(String::CreateFromAscii("LITERAL_PREFIX"));
		if(!aVar.IsNull())
			pInfo->aLiteralPrefix	= (String)aVar;

		ADO_GETFIELD(String::CreateFromAscii("LITERAL_SUFFIX"));
		if(!aVar.IsNull())
			pInfo->aLiteralSuffix	= (String)aVar;

		ADO_GETFIELD(String::CreateFromAscii("CREATE_PARAMS"));
		if(!aVar.IsNull())
			pInfo->aCreateParams	= (String)aVar;

		ADO_GETFIELD(String::CreateFromAscii("AUTO_UNIQUE_VALUE"));
		pInfo->bAutoIncrement	= aVar.IsNull() ? FALSE : (BOOL)aVar;

		ADO_GETFIELD(String::CreateFromAscii("IS_NULLABLE"));
		pInfo->bNullable		= aVar.IsNull() ? FALSE : (BOOL)aVar;

		ADO_GETFIELD(String::CreateFromAscii("SEARCHABLE"));
		pInfo->eSearchType		= Search_All;
		if (!aVar.IsNull())
		{
			switch (aVar.GetInt32())
			{
				case DB_UNSEARCHABLE:		pInfo->eSearchType = Search_None;			break;
				case DB_LIKE_ONLY:			pInfo->eSearchType = Search_Like_Only;		break;
				case DB_ALL_EXCEPT_LIKE:	pInfo->eSearchType = Search_Except_Like;	break;
			}
		}

		_rList.Insert(pInfo, LIST_APPEND);
		pFields->Release();
		aRecordset.MoveNext();
	}
	aRecordset.Close();
	// check if there is already a timestamp type like datetime
	if(pInfoTimestamp)
	{
		if(nTimestampTypeNum == adDBTimeStamp)
		{
			// perhaps I already have a timestamp type with a different name
			for(ULONG i=0;i<_rList.Count();++i)
			{
				SdbTypeInfo* pInfo = _rList.GetObject(i);
				if(pInfo->eType == SDB_DBTYPE_TIMESTAMP && pInfo != pInfoTimestamp)
				{
					pInfoTimestamp->bRowVersion = TRUE;
					break;
				}
			}
		}
		else
			pInfoTimestamp->bRowVersion = TRUE;
	}
}


//------------------------------------------------------------------------
SdbCursor* SdbADOConnection::CreateCursorImpl(SdbCursorType eType, UINT16 nOptions)
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
	DBG_ASSERT(IsOpen(),"SdbADOConnection: Fehler bei Cursorerzeugung - Connection nicht offen");

	return new SdbADOCursor(this, pADOConnection, eType, nOptions);
}

void SdbADOConnection::CommitImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
}


//------------------------------------------------------------------------
INT32 SdbADOConnection::GetTablePrivileges(const String& rTableName) const
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
	INT32 nPermissions = 0;

	return 0;
}

void SdbADOConnection::RollbackImpl()
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
}

//------------------------------------------------------------------------
#define ADO_GETFIELD_VALUE(Name)	\
	pFields->get_Item(OLEVariant(Name),&pField);	\
	aField = WpADOField(pField);					\
	if(pField)										\
		aField.get_Value(aVar);

//------------------------------------------------------------------
BOOL SdbADOConnection::FillRelation(SdbADORelation& rRel)
{
	SdbTableRef xTable = OpenTable(rRel.GetTableName()),
				xForeignTable;
	if (xTable)
		xForeignTable = OpenTable(rRel.GetForeignTableName());
	if (xForeignTable)
	{

		ADORecordset* pCursor = NULL;
		if (!OpenRelationCursor(xTable->QualifierName(), xTable->OwnerName(), xTable->Name(), pCursor))
			aStatus.SetError(ERRCODE_IO_NOTEXISTS,RELATION,rRel.GetName());
		else
		{
			WpADORecordset rxCursor(pCursor);
			::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData = GetMetaData();

			sal_Int16 nCatalogUsage;
			rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
			sal_Int16 nSchemaUsage;
			rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

			SdbTableRef xRefTable;
			rxCursor.MoveFirst();
			while (!rxCursor.IsAtEOF())
			{
				ADOFields  *pFields = rxCursor.GetFields();
				pFields->AddRef();
				ADOField* pField = NULL;
				WpADOField aField;
				OLEVariant aVar;
				ADO_GETFIELD_VALUE(String::CreateFromAscii("ORDINAL"));
				if(aVar.IsEmpty() || aVar.IsNull())
					break;

				if (aVar.GetInt32() == 1)    // gibt es eine neue Relation
				{
					if (xRefTable.Is())                     // Relation bereits ausgelesen
						break;

					String aConstraintName;
					ADO_GETFIELD_VALUE(String::CreateFromAscii("FK_NAME"));
					if (aVar.IsEmpty() || aVar.IsNull())    // ohne Name einen generieren
						aConstraintName = SdbRelation::CreateTempName().GetBuffer();
					else
						aConstraintName = (String)aVar;

					String aRefTableName;
					String aTableName;
					// Zugriff auf referenzierte Tabelle
					if (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML)
					{
						ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_TABLE_CATALOG"));
						if (!(aVar.IsEmpty() || aVar.IsNull()) && aVar.GetString().Len())
						{
							aRefTableName += aVar.GetString();
							aRefTableName += cQualifierSeparator;
							aTableName += xTable->QualifierName().GetBuffer();
							aTableName += cQualifierSeparator;
						}
					}
					if (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML)
					{
						ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_TABLE_SCHEMA"));
						if (!(aVar.IsEmpty() || aVar.IsNull()))
						{
							aRefTableName += aVar.GetString();
							aRefTableName += '.';
							aTableName += xTable->OwnerName().GetBuffer();
							aTableName += '.';
						}
					}
					ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_TABLE_NAME"));
					if (!(aVar.IsEmpty() || aVar.IsNull()))
					{
						aRefTableName += aVar.GetString();
						aTableName += xTable->Name().GetBuffer();
					}

					xRefTable = OpenTable(aRefTableName);
					// Tabelle gefunden
					if (&xRefTable == &xForeignTable)
					{
						rRel.SetName(aConstraintName);
						// jetzt noch die Attributierungen
						UINT16 nAttributes = 0;

						// Update rules
						ADO_GETFIELD_VALUE(String::CreateFromAscii("UPDATE_RULE"));

						if (!(aVar.IsEmpty() || aVar.IsNull()))
						{
							if (aVar.GetString().EqualsAscii(CascadeRule))
								nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
							else if (aVar.GetString().EqualsAscii(SetNullRule))
								nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
							else if (aVar.GetString().EqualsAscii(SetDefaultRule))
								nAttributes |= ::stardata::DatabaseKeyRule::SET_DEFAULT;
							else
								nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
						}
						rRel.SetUpdateRules(nAttributes);
						nAttributes = 0;

						// Delete rules
						ADO_GETFIELD_VALUE(String::CreateFromAscii("DELETE_RULE"));
						if (!(aVar.IsEmpty() || aVar.IsNull()))
						{
							if (aVar.GetString().EqualsAscii(CascadeRule))
								nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
							else if (aVar.GetString().EqualsAscii(SetNullRule))
								nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
							else if (aVar.GetString().EqualsAscii(SetDefaultRule))
								nAttributes |= ::stardata::DatabaseKeyRule::SET_DEFAULT;
							else
								nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
						}
						rRel.SetDeleteRules(nAttributes);
					}
					else
						xRefTable = NULL;
				}

				if (xRefTable.Is())
				{
					const SdbColumn* pColumn = NULL;
					ADO_GETFIELD_VALUE(String::CreateFromAscii("FK_COLUMN_NAME"));
					if (!(aVar.IsEmpty() || aVar.IsNull()))
						pColumn = xTable->OriginalColumns()->Column(aVar);

					String aForeignName;
					ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_COLUMN_NAME"));
					if (!(aVar.IsEmpty() || aVar.IsNull()))
						aForeignName = aVar.GetString();

					if (pColumn && xRefTable->OriginalColumns()->Column(aForeignName))
					{
						SdbColumnRelation* pRelColumn = rRel.CreateColumn(*pColumn, aForeignName);
						rRel.GetColumns()->AddColumn(pRelColumn);
					}
					else
					{
						// falsche struktur
						aStatus.Set(SDB_STAT_ERROR);
						break;
					}
				}
				pFields->Release();
				rxCursor.MoveNext();
			}
		}
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbADOConnection::OpenRelationCursor(const String& rCatalog, const String& rSchema, const String& rName, ADORecordset*& _pCursor)
{
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData = GetMetaData();
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	HRESULT hr = S_OK;
	// Create elements used in the array
	OLEVariant varCriteria[6];
	const int nCrit = sizeof varCriteria /
					 sizeof varCriteria[0];
	// Create SafeArray Bounds and initialize the array
	SAFEARRAYBOUND rgsabound[1];
	rgsabound[0].lLbound   = 0;
	rgsabound[0].cElements = nCrit;
	SAFEARRAY *psa         = SafeArrayCreate( VT_VARIANT, 1, rgsabound );

	INT32 nPos = 0;
	hr = SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;
	hr = SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;
	hr = SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;

	if ((bool)rCatalog.Len() && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML))
		varCriteria[nPos].SetString(rCatalog);
	hr = SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;

	if ((bool)rSchema.Len() && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML))
		varCriteria[nPos].SetString(rSchema);
	hr = SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;

	varCriteria[nPos].SetString(rName);
	hr = SafeArrayPutElement(psa,&nPos,&varCriteria[nPos]);nPos++;

	// Initialize and fill the SafeArray
	OLEVariant vsa;
	vsa.vt = VT_VARIANT | VT_ARRAY;
	V_ARRAY(&vsa) = psa;

	OLEVariant aEmpty;
	aEmpty.SetNoArg();

	pADOConnection->OpenSchema(adSchemaForeignKeys,vsa,aEmpty,&_pCursor);
	//SafeArrayDestroy(psa);
	return _pCursor != NULL;
}
//------------------------------------------------------------------------
void SdbADOConnection::FillRelation(SdbRelationList& _rList, const String& rName)
{
	String sCatalog, sSchema, sName;
	ADORecordset* pCursor = NULL;
	if (!QualifiedNameComponents(rName, sCatalog, sSchema, sName) ||
		!OpenRelationCursor(sCatalog, sSchema, sName, pCursor))
		return;

	SdbRelationRef	xRelation;
	SdbTableRef		xRefTable;
	SdbTableRef		xTable;
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData = GetMetaData();
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	WpADORecordset rxCursor(pCursor);
	rxCursor.MoveFirst();
	while (!rxCursor.IsAtEOF())
	{
		ADOFields  *pFields = rxCursor.GetFields();
		pFields->AddRef();
		ADOField* pField = NULL;
		WpADOField aField;
		OLEVariant aVar;
		ADO_GETFIELD_VALUE(String::CreateFromAscii("ORDINAL"));
		if(aVar.IsEmpty() || aVar.IsNull())
			break;

		if (aVar.GetInt32() == 1)    // gibt es eine neue Relation
		{
			// alte uebernehmen
			if (xRelation.Is())
			{
				((SdbADORelation*)&xRelation)->Open(); // nur das open flag setzen
				_rList.InsertAt(xRelation->GetName(), &xRelation);
				xRelation = NULL;
			}

			String aConstraintName;
			ADO_GETFIELD_VALUE(String::CreateFromAscii("FK_NAME"));
			if (aVar.IsEmpty() || aVar.IsNull())    // ohne Name einen generieren
				aConstraintName = SdbRelation::CreateTempName().GetBuffer();
			else
				aConstraintName = aVar.GetString();

			String aRefTableName;
			String aTableName;
			// Zugriff auf referenzierte Tabelle
			if (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML)
			{
				ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_TABLE_CATALOG"));
				if (!(aVar.IsEmpty() || aVar.IsNull()) && aVar.GetString().Len())
				{
					aRefTableName += aVar.GetString();
					aRefTableName += cQualifierSeparator;
					aTableName += sCatalog;
					aTableName += cQualifierSeparator;
				}
			}
			if (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML)
			{
				ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_TABLE_SCHEMA"));
				if (!(aVar.IsEmpty() || aVar.IsNull()))
				{
					aRefTableName += aVar.GetString();
					aRefTableName += '.';
					aTableName += sSchema;
					aTableName += '.';
				}
			}
			ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_TABLE_NAME"));
			if (!(aVar.IsEmpty() || aVar.IsNull()))
			{
				aRefTableName += aVar.GetString();
				aTableName += sName;
			}

			xRelation = NULL;
			xRefTable = OpenTable(aRefTableName);
			if (xRefTable.Is())
				xRelation = new SdbADORelation(this, aConstraintName);
			if (!xRelation.Is())
			{
				xRefTable = NULL;
				xRelation = NULL;
			}
			else
			{
				xRelation->SetTableName(aTableName);
				xRelation->SetForeignTableName(aRefTableName);

				// jetzt noch die Attributierungen
				UINT16 nAttributes = 0;

				// Update rules
				ADO_GETFIELD_VALUE(String::CreateFromAscii("UPDATE_RULE"));

				if (!(aVar.IsEmpty() || aVar.IsNull()))
				{
					if (aVar.GetString().EqualsAscii(CascadeRule))
						nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
					else if (aVar.GetString().EqualsAscii(SetNullRule))
						nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
					else if (aVar.GetString().EqualsAscii(SetDefaultRule))
						nAttributes |= ::stardata::DatabaseKeyRule::SET_DEFAULT;
					else
						nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
				}
				xRelation->SetUpdateRules(nAttributes);
				nAttributes = 0;

				// Delete rules
				ADO_GETFIELD_VALUE(String::CreateFromAscii("DELETE_RULE"));
				if (!(aVar.IsEmpty() || aVar.IsNull()))
				{
					if (aVar.GetString().EqualsAscii(CascadeRule))
						nAttributes |= ::stardata::DatabaseKeyRule::CASCADE;
					else if (aVar.GetString().EqualsAscii(SetNullRule))
						nAttributes |= ::stardata::DatabaseKeyRule::SET_NULL;
					else if (aVar.GetString().EqualsAscii(SetDefaultRule))
						nAttributes |= ::stardata::DatabaseKeyRule::SET_DEFAULT;
					else
						nAttributes |= ::stardata::DatabaseKeyRule::NO_ACTION;
				}

				xRelation->SetDeleteRules(nAttributes);
			}
		}

		if (xRelation.Is())
		{
			// die Tabelle ist vielleicht noch nicht auf
			if (!xTable.Is())
			{
				xTable = OpenTable(rName);
				if (!xTable.Is())
					return;
			}

			const SdbColumn* pColumn = NULL;
			ADO_GETFIELD_VALUE(String::CreateFromAscii("FK_COLUMN_NAME"));
			if (!(aVar.IsEmpty() || aVar.IsNull()))
				pColumn = xTable->OriginalColumns()->Column(aVar);

			String aForeignName;
			ADO_GETFIELD_VALUE(String::CreateFromAscii("PK_COLUMN_NAME"));
			if (!(aVar.IsEmpty() || aVar.IsNull()))
				aForeignName = (String)aVar;

			if (pColumn && xRefTable->OriginalColumns()->Column(aForeignName))
			{
				SdbColumnRelation* pRelColumn = xRelation->CreateColumn(*pColumn, aForeignName);
				xRelation->GetColumns()->AddColumn(pRelColumn);
			}
			else
			{
				((SdbADORelation*)&xRelation)->RemoveFromParent();
				xRelation = NULL;
			}
		}
		pFields->Release();
		rxCursor.MoveNext();
	}

	if (xRelation.Is())
	{
		((SdbADORelation*)&xRelation)->Open(); // nur das open flag setzen
		_rList.InsertAt(xRelation->GetName(), &xRelation);
	}
}
//------------------------------------------------------------------------
void SdbADOConnection::FillAllRelations(SdbRelationList& _rList)
{
	DBG_CHKTHIS(SdbADOConnection,NULL);
	for (SdbStringStlMap::const_iterator aIter = pTableNames->begin();aIter != pTableNames->end();++aIter)
	{
		if ((*aIter).second)		// es ist keine View
			FillRelation(_rList, (*aIter).first);
	}
}




