/*************************************************************************
 *
 *  $RCSfile: adocurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:41 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define SDB_ADOCUR_CXX
#ifndef SDB_ADOCONN_HXX
#include "adoconn.hxx"
#endif
#ifndef _SDB_SDBRESID_HRC
#include "sdbresid.hrc"
#endif

DBG_NAME(WpADOConnection);
DBG_NAME(WpADOCommand);
DBG_NAME(WpADOField);
DBG_NAME(WpADOParameter);
DBG_NAME(WpADOError);
DBG_NAME(WpADOProperty);
DBG_NAME(WpADORecordset);


DBG_NAME(SdbADOCursor);

//------------------------------------------------------------------------
SdbADOCursor::SdbADOCursor(SdbADOConnection* pConnection, WpADOConnection* pADOConn,SdbCursorType eType,UINT16 nOpt)
			 :SdbCursor(pConnection, eType, nOpt)
			 ,bOnFirstAfterOpen(FALSE)
{
	DBG_CTOR(SdbADOCursor,NULL);


	IClassFactory2* pIUnknown	= NULL;
	IUnknown        *pOuter		= NULL;
	HRESULT         hr;
	hr = CoGetClassObject( ADOS::CLSID_ADOCOMMAND_21,
						  CLSCTX_INPROC_SERVER,
						  NULL,
						  IID_IClassFactory2,
						  (void**)&pIUnknown );
		
	if( !FAILED( hr ) )
	{
		pIUnknown->AddRef();

		ADOCommand *pCommand = NULL;
		hr = pIUnknown->CreateInstanceLic(  pOuter,
											NULL,
											ADOS::IID_ADOCOMMAND_21,
											ADOS::GetKeyStr(),
											(void**) &pCommand);
		
		if( !FAILED( hr ) )
		{
			m_Command = WpADOCommand(pCommand);
			m_Command.putref_ActiveConnection(pADOConn);
		}
		pIUnknown->Release();
	}
	
	if (!m_Command.IsValid())
		aStatus.Set(SDB_STAT_ERROR);
}

//------------------------------------------------------------------------
SdbADOCursor::~SdbADOCursor()
{
	DBG_DTOR(SdbADOCursor,NULL);
	if (IsOpen()) 
		Close();
}

//------------------------------------------------------------------------
BOOL SdbADOCursor::IsSelectStatement()
{
	// do we have a select statement?
	BOOL bRes = FALSE;
	if (aSQLStatement.GetToken(0,' ').EqualsIgnoreCaseAscii("select"))
		bRes = TRUE;
	return bRes;
}

//------------------------------------------------------------------
BOOL SdbADOCursor::SetUpdatableImpl(BOOL bUpdate)
{
	if (bUpdate && (!bBookmarkable || m_Recordset.GetLockType() == adLockReadOnly))
	{
		aStatus.Set(SDB_STAT_ERROR);
		nPrivileges = SDB_PR_READ;
	}
	else if (bUpdate)
	{
		nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_DELETE;
	}
	else
		nPrivileges = SDB_PR_READ;

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------
BOOL SdbADOCursor::OpenImpl(SdbCursorType eType, UINT16 eOpt)
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(m_Command.IsValid(), "No valid statement");

	if (!IsPrepared())
	{		
		Translate();
		if (! aStatus.IsSuccessful())
			return FALSE;
	}
	// Das SQL-Statement wird in eine temporaere (d.h. nicht im .mdb-File zu
	// speichernde) Abfrage "eingepackt",
	// wenn dies nicht schon geschehen ist
	if (!bPrepared && !m_Command.put_CommandText(SQLStatement()))
	{
		SetADOError();
		return FALSE;
	}

	// clear previous need resources
	{
		// cursor should be before the first record, so if we are on the first we stay there when the first next is called
		bOnFirstAfterOpen = FALSE;
		bBookmarkable = FALSE;
		
		if (m_Recordset.IsValid() && m_Recordset.get_State() == adStateOpen)
			m_Recordset.Close();
		m_Recordset.clear();

		m_aBookmarks.clear();
	}

	// wenn es Platzhalter im SQL-Statement gibt, die aktuellen Parameterwerte
	// einsetzen
	if (aParameterColumns->Count() > 1)
		BindParameterValues();

	// do we have a select statement
	if (IsSelectStatement())
	{
		// what kind of recordset is desired?
		// should we use a keyset?
		CursorTypeEnum eTypeEnum = adOpenForwardOnly;
		LockTypeEnum eLockTypeEnum = adLockReadOnly;
		if (eCursorType != SDB_SNAPSHOT)
		{
			eTypeEnum = adOpenKeyset;
			if ((nOptions & SDB_READONLY) == 0)
				eLockTypeEnum = adLockOptimistic;
		}

		if (!m_Recordset.IsValid())
			m_Recordset.Create();

		OLEVariant aCmd;
		aCmd.SetIDispatch(m_Command);
		OLEVariant aCon;
		aCon.SetNoArg();

		if (!m_Recordset.Open(aCmd,aCon,eTypeEnum,eLockTypeEnum,adOpenUnspecified))
		{		
			SetADOError();
			return FALSE;
		}
	}
	else
	{
		m_RecordsAffected.SetNoArg();
		if (m_Parameters.IsEmpty())
			m_Parameters.SetNoArg();

		ADORecordset* pSet=NULL;
		if (!m_Command.Execute(m_RecordsAffected,m_Parameters,adCmdText,&pSet))
		{
			SetADOError();
			return FALSE;
		}
		m_Recordset = pSet;
	}

	if (m_Recordset.IsValid())
	{
		// cursor should be before the first record, so if we are on the first we stay there when the first next is called
		bOnFirstAfterOpen = !m_Recordset.IsAtBOF();
		bBookmarkable = m_Recordset.Supports(adBookmark);
	}
	return aStatus.IsSuccessful();
}
//------------------------------------------------------------------------
BOOL SdbADOCursor::OnOpen()
{
	if (!IsOpen())  		// resultset erst nach execute beschreiben!!!
							// und nur wenn noch nicht geffnet
	{
		SdbConnection* pConnection = GetConnection();
		if (!aResultColumns.Is())
			aResultColumns = pConnection->CreateColumns();
		DescribeResult();
	}

	// Bei Statements, die keine Results liefern (insb. UPDATE, DELETE, INSERT)
	// ist die Anzahl der vom Statement betroffenen Datensaetze interessant:
	if (aResultColumns->Count() == 1 && !m_RecordsAffected.IsNull())
	{
		INT32 pcrow = m_RecordsAffected.GetInt32();
		if (pcrow >= 0)
			SetRowCount((UINT32)pcrow);
	}
	return SdbCursor::OnOpen();
}
//------------------------------------------------------------------------
BOOL SdbADOCursor::PrepareImpl()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(m_Command.IsValid(), "No valid statement");

	// Statement vorbereiten
	Translate();
	if (! aStatus.IsSuccessful()) 
		return FALSE;

	aStatus.Clear();
	
	// say that a command should be prepared
	m_Command.put_Prepared(TRUE);

	// set the command itself
	if (!m_Command.put_CommandText(SQLStatement()))
	{
		SetADOError();
		return FALSE;
	}

	// Falls die Parameter noch nicht beschrieben sind,
	if (aParameterColumns->Count()<2)
		DescribeParameter();

	return TRUE;
}
//------------------------------------------------------------------------
BOOL SdbADOCursor::IsCapable(SdbCursorCapability eCapability) const
{
	switch(eCapability)
	{
		case SDB_CAP_BOOKMARKS:
			return bBookmarkable;
		case SDB_CAP_FETCH:
		case SDB_CAP_FETCH_NEXT:
		case SDB_CAP_FETCH_BOOKMARK:
		case SDB_CAP_UPDATE_POSITIONED:
		case SDB_CAP_FETCH_PRIOR:
			return TRUE;
	}
	return FALSE;
}
// some helpful defines for fields and properties
#define ADO_GETFIELD(Name)	\
	pFields->get_Item(OLEVariant(Name),&pField);	\
	if(!pField)										\
	{												\
		pFields->Release();							\
		return;										\
	}												\
	aField = WpADOField(pField);					

#define ADO_GETFIELD_BOOL(Name)	\
	pFields->get_Item(OLEVariant(Name),&pField);	\
	if(!pField)										\
	{												\
		pFields->Release();							\
		return FALSE;								\
	}												\
	aField = WpADOField(pField);					


// Beschreibung der Struktur der Ergebnismenge
//------------------------------------------------------------------------
void SdbADOCursor::DescribeResult()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	if(!m_Recordset.IsValid())
	{
		SetADOError();
		return;
	}

	ADOFields *pFields = m_Recordset.GetFields();
	if(!pFields)
	{
		SetADOError();
		return;
	}

	pFields->AddRef();

	WpADOField aField;
	INT32 nFieldCount=0;
	pFields->get_Count(&nFieldCount);
	SdbConnection *pConnection = GetConnection();
	INT16  nFlags = 0;
	INT32 nPrec;			// Precision (ggf. 0)
	BYTE nScale;				// Scale (ggf. 0)
	SdbDatabaseType eType;
	String aColumnName;			// Spaltenname

	for(INT32 i=0;i<nFieldCount;++i)
	{
		nFlags = nPrec = nScale = 0;
		ADOField* pField	= NULL;
		ADO_GETFIELD(i);

		eType = aField.GetSdbType();

		INT32 nAttrib = aField.GetAttributes();
		if(nAttrib & adFldUpdatable || adFldUnknownUpdatable & nAttrib)
			nFlags |= SDB_FLAGS_UPDATEALLOWED;
		if(nAttrib & adFldIsNullable)
			nFlags |= SDB_FLAGS_NULLALLOWED;
		if(nAttrib & adFldRowVersion)
			nFlags |= SDB_FLAGS_VERSIONING;
#if 0		
		if(nAttrib & adFldFixed)
			eType = SDB_DBTYPE_CURRENCY;
#endif

		switch(eType)
		{
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:	
			case SDB_DBTYPE_INTEGER:	
			case SDB_DBTYPE_BIGINT:		
			case SDB_DBTYPE_REAL:			
			case SDB_DBTYPE_DOUBLE:			
				nPrec  = aField.GetPrecision();
				break;
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_DECIMAL:
				nPrec  = aField.GetPrecision();
				nScale = aField.GetNumericScale();
				break;
			case SDB_DBTYPE_BOOLEAN:		
			case SDB_DBTYPE_BINARY:			
			case SDB_DBTYPE_VARCHAR:		
			case SDB_DBTYPE_LONGVARCHAR:	
			case SDB_DBTYPE_VARBINARY:		
			case SDB_DBTYPE_LONGVARBINARY:	
			case SDB_DBTYPE_CHAR:	
				nPrec = aField.GetDefinedSize();
				break;
			case SDB_DBTYPE_DATE:			
			case SDB_DBTYPE_TIME:			
			case SDB_DBTYPE_TIMESTAMP:		
			case SDB_DBTYPE_CURRENCY:
				nPrec  = aField.GetPrecision();
				nScale = aField.GetDefinedSize();
				break;
		
		}
		
		// read the dyn property autoincrement, this isn't implemented by all drivers
		{
			ADOProperties* pProps = aField.get_Properties();
			DBG_ASSERT(pProps,"There are no properties at the field! GPF");
			pProps->AddRef();
			
			WpADOProperty aProp;
			OLEVariant aPropVar;		
			
			ADOProperty *pProp = NULL;
			pProps->get_Item(OLEVariant(String::CreateFromAscii("ISAUTOINCREMENT")),&pProp);
			if (pProp)									
			{
				aProp = WpADOProperty(pProp);		
				aPropVar = aProp.GetValue();
			}
			if (!aPropVar.IsNull() && aPropVar.GetBool())
				nFlags |= SDB_FLAGS_AUTOINCREMENT;

			pProps->Release();
		}

		// espacially the ms provider for the jet engine has a bug as it does not know the var char
		const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(eType);
		if (!pInfo && eType == SDB_DBTYPE_VARCHAR)
			pInfo = pConnection->GetColumnTypeInfo(SDB_DBTYPE_CHAR);

		// Column Searchable?
		if (pInfo)
		{
			switch (pInfo->eSearchType)
			{
				case Search_Like_Only  : nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
				case Search_Except_Like: nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				case Search_All: nFlags |= SDB_FLAGS_SEARCH_ALL; break;
			}
		}	

		aColumnName = aField.GetName();
		String aAlias(aColumnName);
		const SdbColumn* pPrevCol = aResultColumns->Column(aAlias);

		ULONG nExprCnt(0);
		while(pPrevCol)
		{
			(aAlias = aColumnName) += String::CreateFromInt32(++nExprCnt);
			pPrevCol = aResultColumns->Column(aAlias);
		}
		SdbColumn * pColumn = pConnection->CreateColumn(aAlias,eType,nPrec,nScale, nFlags);
		pColumn->SetRealName(aColumnName);

		aResultColumns->AddColumn(pColumn);
	}

	pFields->Release();
}


//------------------------------------------------------------------------
void SdbADOCursor::FixResultRow()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(aResultRow.Is(), "Keine Ergebniszeile vorhanden!! DescribeResult()"
				" wurde nicht gerufen!!");

	DBG_ASSERT(m_Recordset.IsValid(), "SdbADOCursor: Kein gueltiges ADO-RecordSet");

	ADOFields *pFields = m_Recordset.GetFields();
	DBG_ASSERT(pFields, "SdbADOCursor: Kein gueltiges ADO-WpADOFields");
	pFields->AddRef();
	INT32 nCount;
	pFields->get_Count(&nCount);
	
	OLEVariant aVariant;
	WpADOField aField;	
	for (INT32 i = 1; i <= nCount; i++)
	{
		const ODbVariantRef& pVal = (*aResultRow)[i];
		if (!pVal.is())
			continue;	// nicht gebunden

		ADOField *pField = NULL;

		ADO_GETFIELD(i-1);
		
		aField.get_Value(aVariant);
		if (aVariant.IsNull()) //oder Nullwert
		{
			pVal->setNull();
			continue;
		}

		switch (aField.GetSdbType())
		{
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_VARCHAR:
				pVal->setString(aVariant.GetUString());
				break;

			case SDB_DBTYPE_CHAR:
				pVal->setString((aVariant.GetUString()));
				break;

			case SDB_DBTYPE_DATE:
			case SDB_DBTYPE_TIME:
			case SDB_DBTYPE_TIMESTAMP:
				pVal->setDouble(aVariant.GetDate());
				break;

			case SDB_DBTYPE_TINYINT:
				pVal->setBYTE(aVariant.GetByte());
				break;

			case SDB_DBTYPE_SMALLINT:
				pVal->setINT16(aVariant.GetInt16());
				break;

			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_BIGINT:
				pVal->setINT32(aVariant.GetInt32());
				break;

			case SDB_DBTYPE_REAL:
				pVal->setFloat(aVariant.GetFloat());
				break;

			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_DOUBLE:
				pVal->setDouble(aVariant.GetDouble());
				break;

			case SDB_DBTYPE_CURRENCY:
				pVal->setDouble(aVariant.GetCurrency());
				//pVal->Convert(SbxDOUBLE);
				break;

			case SDB_DBTYPE_BOOLEAN:
				pVal->setBOOL(aVariant.GetBool());
				break;
			default:						// unbekannter Datentyp
				break;

		}
	}
	pFields->Release();
}
//------------------------------------------------------------------------
BOOL SdbADOCursor::MoveImpl(const ::com::sun::star::uno::Any& rBookmark, SdbCursorMode& rMode, BOOL bData)  throw( ::com::sun::star::lang::IllegalArgumentException )
{
	// do we have a bookmark of valid type
	if (rBookmark.getValueTypeClass() != ::com::sun::star::uno::TypeClass_LONG || !bBookmarkable)
		throw ::com::sun::star::lang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);

	sal_Int32 nBookmarkPos;
	rBookmark >>= nBookmarkPos;
	if (nBookmarkPos > m_aBookmarks.size())
		throw ::com::sun::star::lang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);

	if (!m_Recordset.Move(0,m_aBookmarks[nBookmarkPos - 1]))
	{
		SetADOError();
		return FALSE;
	}
	else if (bData)
		FixResultRow();

	(*aResultRow)[0]->setAny(rBookmark);										
	return aStatus.IsSuccessful();	
}

//------------------------------------------------------------------------
BOOL SdbADOCursor::MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,
							SdbCursorMode& rMode)
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(m_Recordset.IsValid(), "SdbADOCursor: Kein gueltiges ADO-Recordset");

	switch(eCursorPosition)
	{
		case SDB_POS_NEXT:
			if (bOnFirstAfterOpen)	// we are already on the first row
				bOnFirstAfterOpen = FALSE;			
			else if (!m_Recordset.MoveNext())
			{
				SetADOError();
				return FALSE;
			}
			else if (m_Recordset.IsAtEOF())
				aStatus.Set(SDB_STAT_NO_DATA_FOUND);

			if (aStatus.IsSuccessful())
			{				
				if (bBookmarkable)
				{
					OLEVariant aVariant;
					m_Recordset.GetBookmark(aVariant);		
					m_aBookmarks.push_back(aVariant);	

					// set the desired bookmark
					aResultRow->front()->setINT32(m_aBookmarks.size());		
				}
				FixResultRow(); 
			}
			break;
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------
void SdbADOCursor::DescribeParameter()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);

	// Informationen ueber die Parameter von der QueryDef
	// beschaffen

	ADOParameters *pParameters = m_Command.get_Parameters();
	if (!pParameters)
	{
		SetADOError();
		return;
	}
	pParameters->AddRef();
	
	INT32 nParameterCount = 0;
	pParameters->get_Count(&nParameterCount);

	ADOParameter *pParameter;
	WpADOParameter aParameter;
	SdbADOConnection* pCon = GetADOConnection();
	SdbColumn* pCol = NULL;
	aParameterColumns->Reset();

	for (xub_StrLen i = 0; i<(xub_StrLen)nParameterCount;i++)
	{
		pParameter = NULL;
		pParameters->get_Item(OLEVariant(i),&pParameter);
		aParameter = WpADOParameter(pParameter);
		if(aParameter.IsValid())
		{
			INT32 nAttrib = aParameter.GetAttributes();
			INT16 nFlags = 0;
			if(nAttrib & adParamNullable)
				nFlags |= SDB_FLAGS_NULLALLOWED;
			ParameterDirectionEnum eDirection = aParameter.get_Direction();
			switch(eDirection)
			{
				case adParamInput:			nFlags |= SDB_FLAGS_PARAM_IN;	break;
				case adParamOutput:			nFlags |= SDB_FLAGS_PARAM_OUT;	break;
				case adParamInputOutput:	nFlags |= SDB_FLAGS_PARAM_IN | SDB_FLAGS_PARAM_OUT;	break;
			}
			pCol = pCon->CreateColumn(aParameterNames.GetToken(i,';'),aParameter.GetSdbType(),aParameter.GetPrecision(),aParameter.GetNumericScale(),nFlags);
			aParameterColumns->AddColumn(pCol);
		}
	}
	pParameters->Release();
}
//------------------------------------------------------------------------
BOOL SdbADOCursor::CloseImpl()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	if (m_Recordset.IsValid() && m_Recordset.get_State() == adStateOpen)
		m_Recordset.Close();

	m_aBookmarks.clear();
	m_Recordset.clear();

	// do we still need the command?
	if (!GetParent())	
		m_Command.clear();
	
	return TRUE;
}

//--------------------------------------------------------------------------
ADOParameter* SdbADOCursor::AddParameter(const SdbColumn& rCol, ADOParameters* pParameters)
{
	DataTypeEnum eType = ADOS::ADOTypeFromSdbType(rCol.GetType(), FALSE);
	long nSize = rCol.GetLength();
	ParameterDirectionEnum eDirection;
	
	if (rCol.IsInParameter() && rCol.IsOutParameter())
		eDirection = adParamInputOutput;
	else if (rCol.IsOutParameter())
		eDirection = adParamOutput;
	else
		eDirection = adParamInput;

	ADOParameter* pParam = m_Command.CreateParameter(String(),eType,eDirection,nSize,OLEVariant());
	if (pParam)
	{
		pParam->AddRef();
		
		// set the scale for numeric types
		if (rCol.GetScale())
			pParam->put_NumericScale(rCol.GetScale());

		pParameters->Append(pParam);
	}
	return pParam;
}

//--------------------------------------------------------------------------
void SdbADOCursor::BindParameterValues()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);

	UINT16 nCount= aParameterColumns->Count();
	OLEVariant aVariant;

	ADOParameters *pParameters = m_Command.get_Parameters();
	if (!pParameters)
	{
		SetADOError();
		return;
	}
	pParameters->AddRef();
	
	WpADOParameter aParameter;
	for (UINT16 i = 1;  i<nCount; i++)
	{
		ADOParameter *pParameter = NULL;
		pParameters->get_Item(OLEVariant(i-1), &pParameter);
		
		SdbColumnRef xColumn = (SdbColumn*)aParameterColumns->Column(i);
		// some drivers are not able to provide a parameter, so we have to provide them ourself
		if (!pParameter)
			aParameter = AddParameter(*xColumn,pParameters);
		else
			aParameter = WpADOParameter(pParameter);
		
		if (!aParameter.IsValid())
			break;

		ODbVariantRef pVar = (*aParameterRow)[i];
		if(!pVar.is() || pVar->isNull())
			aVariant.SetNull();
		else
		{			
			try
			{
				switch (xColumn->GetType())
				{
					case SDB_DBTYPE_CHAR:
					case SDB_DBTYPE_VARCHAR:
					case SDB_DBTYPE_LONGVARCHAR:
						aVariant.SetUString(pVar->toString());
						break;
					case SDB_DBTYPE_TINYINT:
						aVariant.SetByte(pVar->toBYTE());
						break;
					case SDB_DBTYPE_SMALLINT:
						aVariant.SetInt16(pVar->toINT16());
						break;
					case SDB_DBTYPE_INTEGER:
						aVariant.SetInt32(pVar->toINT32());
						break;
					case SDB_DBTYPE_BIGINT:
						aVariant.SetInt32(pVar->toINT32());
						break;
					case SDB_DBTYPE_REAL:
						aVariant.SetFloat(pVar->toFloat());
						break;
					case SDB_DBTYPE_NUMERIC:
					case SDB_DBTYPE_DOUBLE:
						aVariant.SetDouble(pVar->toDouble());
						break;
					case SDB_DBTYPE_CURRENCY:
						aVariant.SetCurrency(pVar->toDouble());
						break;
					case SDB_DBTYPE_BOOLEAN:
						aVariant.SetFloat(pVar->toBOOL());
						break;
					case SDB_DBTYPE_DATE:
					case SDB_DBTYPE_TIME:
					case SDB_DBTYPE_TIMESTAMP:
						aVariant.SetDate(pVar->toDouble());
						break;
					default:
						aVariant.SetNull();
				}
			}
			catch ( ... )
			{
				String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
				strError.SearchAndReplace(String::CreateFromAscii("$name$"), aParameter.GetName());
				aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());			
				return ;
			}			
		}

		if (!(aParameter.IsValid() && aParameter.PutValue(aVariant)))
		{
			SetADOError();
			pParameters->Release();
			return;
		}
	}
	pParameters->Release();
}

//------------------------------------------------------------------------
BOOL SdbADOCursor::DeleteImpl()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(m_Recordset.IsValid(), "SdbADOCursor: Kein gueltiges ADO-Recordset");

	return m_Recordset.Delete(adAffectCurrent);

}

//------------------------------------------------------------------------
BOOL SdbADOCursor::UpdateImpl()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(m_Recordset.IsValid(), "SdbADOCursor: Kein gueltiges ADO-Recordset");

	// in der ResultRow liegende Werte in die Zeile einfuegen,
	// auf der der das Recordset steht
	if (!UpdateEditableADORecord())
	{
		SetADOError();

		// undo the addition
		m_Recordset.CancelUpdate();
		return FALSE;
	}
	else
		return TRUE;		
}

//------------------------------------------------------------------------
BOOL SdbADOCursor::UpdateEditableADORecord()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	DBG_ASSERT(m_Recordset.IsValid(), "SdbADOCursor: Kein gueltiges ADO-Recordset");

	UINT16 nCount = aResultColumns->Count()-1;
	ADOFields *pFields = m_Recordset.GetFields();
	pFields->AddRef();
	
	WpADOField aField;
	OLEVariant aVariant;

	// Werte aus der ResultRow in den aktuellen Record eintragen
	for (INT32 i = 0; i < nCount; i++)
	{
		const ODbVariantRef& pVal = (*aResultRow)[i+1];		
		if (!pVal.is() || !pVal->isModified())
			continue;	// nicht gebunden

		ADOField *pField = NULL;
		ADO_GETFIELD_BOOL(i);
		if (pVal->isNull())
			aVariant.SetNull();
		else
		{
			try
			{
				switch (aField.GetSdbType())
				{
					case SDB_DBTYPE_LONGVARCHAR:
					case SDB_DBTYPE_VARCHAR:
					case SDB_DBTYPE_CHAR:
						aVariant.SetUString(pVal->toString());
						break;
					case SDB_DBTYPE_DATE:
					case SDB_DBTYPE_TIME:
					case SDB_DBTYPE_TIMESTAMP:
						aVariant.SetDate(pVal->toDouble());
						break;
					case SDB_DBTYPE_TINYINT:
						aVariant.SetByte(pVal->toBYTE());
						break;
					case SDB_DBTYPE_SMALLINT:
						aVariant.SetInt16(pVal->toINT16());;
						break;
					case SDB_DBTYPE_INTEGER:
						aVariant.SetInt32(pVal->toINT32());
						break;
					case SDB_DBTYPE_BIGINT:
						aVariant.SetInt32(pVal->toINT32());
						break;
					case SDB_DBTYPE_REAL:
						aVariant.SetFloat(pVal->toFloat());
						break;
					case SDB_DBTYPE_DOUBLE:
						aVariant.SetDouble(pVal->toDouble());
						break;
					case SDB_DBTYPE_CURRENCY:
						aVariant.SetCurrency(pVal->toDouble());
						break;
					case SDB_DBTYPE_BOOLEAN:
						aVariant.SetBool(pVal->toBOOL());
						break;
					case SDB_DBTYPE_NUMERIC:
						aVariant.SetDouble(pVal->toDouble());
					default:						// unbekannter Datentyp
						break;
				}
			}
			catch ( ... )
			{
				String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
				strError.SearchAndReplace(String::CreateFromAscii("$name$"), aField.GetName());
				aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());			
				return FALSE;
			}			
		}

		if (aField.IsValid())
			aField.PutValue(aVariant);

	}
	pFields->Release();
	OLEVariant aEmpty;
	aEmpty.SetNoArg();
	return m_Recordset.Update(aEmpty,aEmpty);
}

//------------------------------------------------------------------------
BOOL SdbADOCursor::InsertImpl()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);

	DBG_ASSERT(m_Recordset.IsValid(), "SdbADOCursor: Kein gueltiges ADO-Recordset");
	OLEVariant aEmpty;
	aEmpty.SetNoArg();
	if
	(
		!m_Recordset.AddNew(aEmpty,aEmpty) || // neuen "Default"-Datensatz erzeugen

		// neuer Datensatz ist schon editierbar, also mit gewuenschten
		// Eintraegen aktualisieren:
		!UpdateEditableADORecord()
	)
	{
		SetADOError();

		// undo the addition
		m_Recordset.CancelUpdate();
		return FALSE;
	}

	if (bBookmarkable)
	{
		OLEVariant aVariant;
		m_Recordset.GetBookmark(aVariant);		
		m_aBookmarks.push_back(aVariant);	

		// set the desired bookmark
		(*aResultRow)[0]->setINT32(m_aBookmarks.size());		
	}	
	return TRUE;

}
inline void SdbADOCursor::SetADOError()
{
	DBG_CHKTHIS(SdbADOCursor,NULL);
	((SdbADOConnection*) GetConnection())->SetADOError(aStatus);
}



