/*************************************************************************
 *
 *  $RCSfile: row.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <row.hxx>			  

DBG_NAME(ODbRow);

//--------------------------------------------------------------------------
ODbRow::ODbRow(SdbColumns& rColumns, BOOL bBind)
	   :vector<ODbVariantRef>(rColumns.Count())
	   ,m_aColumns(&rColumns)
	   ,m_nState(ROW_CLEAN)
{
	DBG_CTOR(ODbRow,NULL);	
	if (bBind)	// create all variants
	{
		for (ODbRow::iterator i = begin(); i != end(); i++)
			*i = new ODbVariant();
	}	
}	

//--------------------------------------------------------------------------
ODbRow::ODbRow(const ODbRow& rRow)
	   :vector<ODbVariantRef>(rRow.getColumns()->Count())
	   ,m_aColumns(rRow.getColumns())
	   ,m_nState(ROW_CLEAN)
{
	DBG_CTOR(ODbRow,NULL);
	DBG_CHKOBJ(&rRow,ODbRow,NULL);

	*this = rRow;	
}

//--------------------------------------------------------------------------
ODbRow& ODbRow::operator=(const ODbRow &rRow)
{
	DBG_CHKTHIS(ODbRow,NULL);
	DBG_CHKOBJ(&rRow,ODbRow,NULL);
	if(this == &rRow) 
		return *this;

	// Die Struktur muss identisch sein, sonst ist keine Zuweisung moeglich!
	DBG_ASSERT(m_aColumns == rRow.getColumns(),"ODbRow: SdbColumns nicht identisch");
	m_nState = rRow.m_nState;

	// copy the values
	ODbRow::const_iterator j = rRow.begin();
	for (ODbRow::iterator i = begin(); i != end(); i++)
	{
		ODbVariantRef& xTarget = *i;
		const ODbVariantRef& xSource = *j;
		if (!xSource.is())
			xTarget.unbind();
		else
		{
			if (xTarget.is())
				(*xTarget) = *xSource;
			else
				xTarget = new ODbVariant(*xSource);

			xTarget->setModified(xSource->isModified());
		}
		j++;
	}	
	return *this;
}

//--------------------------------------------------------------------------
BOOL ODbRow::operator==(const ODbRow& rRow) const
{
    if (size() == rRow.size())
	{
		for (ODbRow::const_iterator first1 = begin() + 1,
			 last1 = end(), first2 = rRow.begin() + 1;
			 first1 != last1; ++first1, ++first2)
		{
			if ((*first1).is())
			{
				if (!(*first2).is() || **first1 != **first2)
					return FALSE;
			}
			else if ((*first2).is())
				return FALSE;
		}
		return TRUE;
	}
	return FALSE;
}

//--------------------------------------------------------------------------
BOOL ODbRow::isModified() const
{
	for (ODbRow::const_iterator i = (begin() + 1); i != end(); i++)
	{
		if ((*i).is() && (*i)->isModified())
			return TRUE;
	}
	return FALSE;
}

//--------------------------------------------------------------------------
void ODbRow::setModified(BOOL bMod)
{
	for (ODbRow::iterator i = (begin() + 1); i != end(); i++)
	{
		if ((*i).is())
			(*i)->setModified(bMod);
	}	
}	 

//--------------------------------------------------------------------------
void ODbRow::reset()
{
	for (ODbRow::iterator i = begin(); i != end(); i++)
	{
		if ((*i).is())
		{
			(*i)->setNull();
			(*i)->setModified(FALSE);
		}
	}	
}	 



