/*************************************************************************
 *
 *  $RCSfile: rowcache.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <rowcache.hxx>			  
#include <limits.h>

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif 

// Symbolische Konstanten fuer absolute Positionierung
#define ROWCACHE_FIRST (0)
#define ROWCACHE_LAST (ULONG_MAX)

DBG_NAME(ORowCache);

//--------------------------------------------------------------------------
ORowCache::ORowCache(SdbCursor& rNewCursor, UINT16 nCacheSize, BOOL bFromFirst)
			:m_rCursor(rNewCursor)
			,m_nSize(nCacheSize + 2)
			,m_nDefaultOffset(nCacheSize/2)
			,m_nCurrentPos(0)
			,m_nCursorPos(0)		
{
	DBG_CTOR(ORowCache,NULL);
	
	OFFLEFT = new ODbRow();
	OFFLEFT->acquire();
	OFFRIGHT = new ODbRow();
	OFFRIGHT->acquire();	

	m_pRows = new ODbRowRef[m_nSize];

	// Zustand undefiniert
	if (bFromFirst)
		at(0) = OFFLEFT;
	else if (m_rCursor.nCurrentPos == SDB_CURRENT_ROW_UNDEFINED)
		;
	// Cursor noch nicht offen, wird noch positionieren
	else if (bFromFirst || m_rCursor.IsBOF())
		at(0) = OFFLEFT;
	else if (m_rCursor.IsEOF())
	{
		m_nCurrentPos = 0;
		at(0) = OFFRIGHT;
	}
	else
	{
		// steht der cursor auf dem ersten satz
		if (m_rCursor.IsOnFirst())
		{
			at(0) = OFFLEFT;
			at(1) = new ODbRow(*m_rCursor.aResultRow);
			m_nCursorPos  = 1;
			m_nCurrentPos = 1;
			m_rCursor.eMode		= m_rCursor.eMode;
			m_rCursor.aCurrentRow = at(m_nCurrentPos);
			m_rCursor.aCurrentRow->setModified(FALSE);
		}
		else if (m_rCursor.IsOnLast())
		{
			// den aktuellen modus merken
			SdbCursor::SdbCursorMode eMode = m_rCursor.eMode;

			at(0) = new ODbRow(*m_rCursor.aResultRow);
			at(1) = OFFRIGHT;
			m_rCursor.eMode		= eMode;
			m_rCursor.aCurrentRow = at(m_nCurrentPos);
			m_rCursor.aCurrentRow->setModified(FALSE);
		}
		else
		{
			SdbCursor::SdbCursorMode eMode = m_rCursor.eMode;
			at(0) = new ODbRow(*m_rCursor.aResultRow);
			m_rCursor.eMode		= eMode;
			m_rCursor.aCurrentRow = at(m_nCurrentPos);
			m_rCursor.aCurrentRow->setModified(FALSE);
		}
	}
}

//--------------------------------------------------------------------------
ORowCache::~ORowCache()
{
	DBG_DTOR(ORowCache,NULL);
	__DELETE(m_nSize) m_pRows;
	OFFLEFT->release();
	OFFRIGHT->release();
}

// Obsolete Saetze sind solche, die bereits vom Benutzer oder auenstehenden (Multiuser)
// geloescht worden sind
//--------------------------------------------------------------------------
BOOL ORowCache::isObsolete(const ODbRowRef& rRow) const
{
	return rRow.is() && (rRow->isObsolete() || (!m_rCursor.SeeDeleted() && rRow->isDeleted()));
}

//--------------------------------------------------------------------------
void ORowCache::clear(UINT16 eDirection, UINT16 nPos)
{
	if (eDirection == SDB_DIR_LEFT)
		for (UINT16 i = nPos; (INT16)i >= 0; i--)
			at(i).unbind();
	else
		for (UINT16 ii = nPos; ii < m_nSize; ii++)
			at(ii).unbind();
}

//--------------------------------------------------------------------------
BOOL ORowCache::positionRelative(SdbDirection eDirection, UINT32 nOffset)
{
	DBG_CHKTHIS(ORowCache,NULL);

	UINT32 i;
	for (i = 0; i < nOffset && implPositionRelative(eDirection); i++);

	if (i == nOffset && !isOffRange()) // Positionierung hat geklappt und wir stehen auf einer gueltigen
									   // Position
	{
		// Zeilen merken
		m_rCursor.aCurrentRow = at(m_nCurrentPos);
		m_rCursor.aCurrentRow->setModified(FALSE);
		if (m_rCursor.aCurrentRow->isDeleted())
			m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
		else
			m_rCursor.eMode = SdbCursor::SDB_MOD_NONE;

		m_rCursor.aStatus.Clear();
		return TRUE;
	}
	else
	{
		m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
		m_rCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		return FALSE;
	}
}

// Relative Positionierung:
//--------------------------------------------------------------------------
BOOL ORowCache::implPositionRelative(SdbDirection eDirection)
{
	DBG_CHKTHIS(ORowCache,NULL);

	if ((eDirection == SDB_DIR_LEFT && isOffLeft()) ||
		(eDirection == SDB_DIR_RIGHT && isOffRight(FALSE))) return FALSE;

	// Positionieren. Falls am Ende des Cache angekommen,
	// muss dessen Startposition und Inhalt verschoben werden,
	// um Platz zu schaffen.
	if (eDirection == SDB_DIR_LEFT)
	{
		if (m_nCurrentPos == 0) shift(SDB_DIR_LEFT,1);
		DBG_ASSERT(m_nCurrentPos < m_nSize, "ORowCache::ImplPositionRelative : invalid state after Shift !");
		m_nCurrentPos--;
	}
	else if (eDirection == SDB_DIR_RIGHT)
	{
		if (m_nCurrentPos == m_nSize - 1) shift(SDB_DIR_RIGHT,1);
		DBG_ASSERT(m_nCurrentPos >= 0, "ORowCache::ImplPositionRelative : invalid state after Shift !");
		m_nCurrentPos++;
	}

	// Jetzt ist die aktuelle Position auf jeden Fall innerhalb
	// des Cache:
	DBG_ASSERT(m_nCurrentPos >= 0, "ORowCache: Interner Fehler: CurrentOffset ungueltig");
	DBG_ASSERT(m_nCurrentPos < m_nSize,"ORowCache: Interner Fehler: CurrentOffset ungueltig");
	DBG_ASSERT(!isObsolete(), "Datensatz ist geloescht");

	// Aktueller Satz noch nicht gelesen
	if (!at(m_nCurrentPos).is())
	{
		fill(eDirection, m_nDefaultOffset);
		DBG_ASSERT(at(m_nCurrentPos).is(), "Fehler im Cache!");
	}
	return !isOffRange();
}

//--------------------------------------------------------------------------
BOOL ORowCache::positionAbsolute(UINT32 nPos, BOOL bForceFill)
{
	if (nPos == ROWCACHE_FIRST || nPos == 1)
	{
		// Falls der Cache am Anfang steht, nutzen wir das aus:
		BOOL bReFetch = TRUE;
		if (!bForceFill)
		{
			UINT16 i = 0;
			while (i < m_nSize && !isOffLeft(at(i)))
				i++;

			// Es gibt einen Anfang im Cache:
			if (i < m_nSize - 2)
			{
				ODbRowRef& rRow = at(i + 1);
				DBG_ASSERT(!isOffLeft(rRow) && !isObsolete(rRow), "Fehler im Cache");
				if (rRow.is() && !isOffRight(rRow))
				{
					// Es gibt eine gueltige letzte Zeile
					// Ist die Zeile danach auch gueltig?
					ODbRowRef& rRow = at(i + 2);
					if (rRow.is())
					{
						// Es gibt eine gueltige erste Zeile
						m_nCurrentPos = i + 1;
						bReFetch    = FALSE;
					}
				}
			}
		}
		if (bReFetch)
		{
			clear(SDB_DIR_RIGHT, 0);
			// Offleft-Markierung setzen:
			at(0)	= OFFLEFT;

			// An den Anfang positionieren
			m_nCursorPos		= 1;
			m_nCurrentPos		= 1;
			if (fetch(SDB_POS_FIRST))
				fill(SDB_DIR_RIGHT,m_nDefaultOffset);
		}
	}
	else if (nPos == ROWCACHE_LAST)
	{
		// Falls der Cache am Ende steht, nutzen wir das aus:
		BOOL  bReFetch = TRUE;
		if (!bForceFill)
		{
			INT16 i = (INT16) m_nSize - 1;
			while (i >= 0 && !isOffRight(at((UINT16)i)))
				i--;

			// Es gibt einen Anfang im Cache:
			if (i > 1)
			{
				ODbRowRef& rRow = at(i - 1);
				DBG_ASSERT(!isOffRight(rRow) && !isObsolete(rRow), "Fehler im Cache");
				if (rRow.is() && !isOffLeft(rRow))
				{
					// Es gibt eine gueltige letzte Zeile
					// Ist die Zeile davor auch gueltig?
					ODbRowRef& rRow = at(i - 2);
					if (rRow.is())
					{
						m_nCurrentPos = i - 1;
						bReFetch    = FALSE;
					}
				}
			}
		}
		if (bReFetch)
		{
			// remember some members in case the fetch is canceled ...
			INT32 nOldCurrent = m_nCurrentPos;
			INT32 nOldCursor = m_nCursorPos;

			// position on the end of the cache
			m_nCurrentPos	= m_nSize - 2;
			m_nCursorPos	= m_nSize - 2;

			if (fetch(SDB_POS_LAST))
			{
				DBG_ASSERT(m_nCurrentPos == m_nCursorPos, "ORowCache::PositionAbsolute : oops ... the modified code doesn't behave as expected !");
				// clear the cache but save the row we already fetched
				ODbRowRef aJustFetched = new ODbRow(*at(m_nCurrentPos));				
				clear(SDB_DIR_RIGHT, 0);
				at(m_nCurrentPos) = aJustFetched;

				// set the offright mark
				at(m_nSize -1) = OFFRIGHT;

				// refill the cache
				fill(SDB_DIR_LEFT, m_nDefaultOffset);
			}
			else if (m_rCursor.Status().IsCanceled())
			{
				// just restore the two members (further cancel handling has to be done by the caller)
				m_nCurrentPos	= nOldCurrent;
				m_nCursorPos	= nOldCursor;
			}
		}
	}
	else
	{
		clear(SDB_DIR_RIGHT, 0);

		// lesen einen vor der position
		m_nCursorPos	= 0;
		m_nCurrentPos	= 0;

		if (fetch(SDB_POS_ABSOLUTE, nPos))
			fill(SDB_DIR_RIGHT,m_nDefaultOffset);		// Auffuellen mit weiteren Saetzen

	}

	// Cursor Status setzen
	if (!isOffRange())
	{
		m_rCursor.aCurrentRow = at(m_nCurrentPos);
		m_rCursor.aCurrentRow->setModified(FALSE);

		if (m_rCursor.aCurrentRow->isDeleted())
			m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
		else
			m_rCursor.eMode = SdbCursor::SDB_MOD_NONE;

		m_rCursor.aStatus.Clear();
	}
	else // auf keine gueltige Position positioniert
	{
		m_rCursor.aCurrentRow.unbind();
		m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
		m_rCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		m_rCursor.bEOF = TRUE;
		m_rCursor.bBOF = FALSE;
	}
	return !isOffRange();
}

//--------------------------------------------------------------------------
BOOL ORowCache::positionBookmark(const ::com::sun::star::uno::Any& rBookmark) 
		throw( ::com::sun::star::lang::IllegalArgumentException )
{
	// suchen nach der Bookmark
	BOOL bFound = FALSE;
	INT16 i = m_nCurrentPos;
	while (i < m_nSize)
	{
		ODbRowRef xRow = at(i);
		// feststellen auf gueltigkeit
		DBG_ASSERT(!isObsolete(xRow), "Fehler im Cache");
		if (!xRow.is() || isOffLeft(xRow))
			i++;
		else if (isOffRight(xRow))	// ausserhalb des bereichs
			break;
		else if ((*xRow)[0]->equals(rBookmark))
		{
			bFound = TRUE;
			break;
		}
		else
			i++;
	}

	if (!bFound)	// zweiter durchlauf in die andere Richtung
	{
		i = m_nCurrentPos;
		while (i >= 0)
		{
			ODbRowRef xRow = at(i);
			// feststellen auf gueltigkeit
			DBG_ASSERT(!isObsolete(xRow), "Fehler im Cache");
			if (!xRow.is() || isOffRight(xRow))
				i--;
			else if (isOffLeft(xRow))	// ausserhalb des bereichs
				break;
			else if ((*xRow)[0]->equals(rBookmark))
			{
				bFound = TRUE;
				break;
			}
			else
				i--;
		}
	}

	SdbCursor::SdbCursorMode eMode;
	ODbRowRef xRow;

	if (bFound)
	{
		m_nCurrentPos = i;
		eMode = SdbCursor::SDB_MOD_NONE;
		xRow = at(m_nCurrentPos);
	}
	else
	{
		clear(SDB_DIR_RIGHT, 0);

		m_nCursorPos	= 1;
		m_nCurrentPos	= 1;

		ODbRowRef& rRow = at(m_nCursorPos);
		try
		{
			if (!m_rCursor.DoMove(rBookmark, FALSE, TRUE) && !m_rCursor.Status().IsCanceled())
			{
				at(0) = OFFLEFT;
				at(1) = OFFRIGHT;			
				m_rCursor.aStatus.Clear();
			}							  			
			else
			{
				DBG_ASSERT(!isObsolete(rRow), "Fehler im Cache");
				if (!rRow.is() || isOffRange(rRow) )  
					rRow = new ODbRow(*m_rCursor.aResultRow);
				else
					(*rRow) = *m_rCursor.aResultRow;
			}
			if (!m_rCursor.Status().IsCanceled() && !isOffRange())
			{
				eMode = SdbCursor::SDB_MOD_NONE;
				xRow = at(m_nCurrentPos);
			}
		}
		catch (::com::sun::star::lang::IllegalArgumentException e)
		{
			at(0) = OFFLEFT;
			at(1) = OFFRIGHT;						

			m_rCursor.aCurrentRow = xRow;

			m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
			m_rCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
			m_rCursor.nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
			m_rCursor.bEOF = m_rCursor.bBOF = FALSE;

			throw;
		}		
	}

	m_rCursor.aCurrentRow = xRow;

	// Cursor Status setzen
	if (!isOffRange())
	{
		m_rCursor.aStatus.Clear();
		m_rCursor.eMode		= eMode;
		m_rCursor.aCurrentRow->setModified(FALSE);
		if (m_rCursor.aCurrentRow->isDeleted())
			m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
		else
			m_rCursor.eMode = SdbCursor::SDB_MOD_NONE;
	}
	else // auf keine gueltige Position positioniert
	{
		m_rCursor.eMode = SdbCursor::SDB_MOD_INVALID;
		m_rCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		m_rCursor.nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
		m_rCursor.bEOF = m_rCursor.bBOF = FALSE;
	} 
	return !isOffRange();
}

// Platz im Cache schaffen:
// Cache-Anfang (und damit alle Eintraege) verschieben.
// Left: Cache (-Anfangsposition) nach links verschieben.
//       Die vorhandenen Eintraege verschieben sich dadurch nach rechts,
//       links werden dadurch Eintraege frei.
// Right: umgekehrt.
//--------------------------------------------------------------------------
void ORowCache::shift(SdbDirection eDirection, UINT16 nOffset)
{
	DBG_ASSERT(m_nSize > (2 * nOffset),"ORowCache: CacheSize muss mindestens doppelt so gross sein wie die WindowSize, um die verschoben wird");
	if (nOffset == 0)
		nOffset = m_nDefaultOffset;

	UINT16 i;

	if (eDirection == SDB_DIR_LEFT)
	{
		// Falls der Cursor im freizugebenden Bereich steht, muss er da
		// erst weg!
		if (m_nCursorPos >= m_nSize - nOffset)
		{
			DBG_ASSERT(m_nCursorPos > 0,"ORowCache: FetchRow ausserhalb des Cache-Bereiches");
			DBG_ASSERT(!m_rCursor.bBOF, "Cursor darf nicht vor dem Anfang stehen!");
			if (m_rCursor.bEOF)	// steht er hinter dem letzten
			{
				m_nCursorPos--;
				if (!fetch(SDB_POS_LAST))	// Resultset leer
					return;
			}
			while (m_nCursorPos >= m_nSize - nOffset && !m_rCursor.bBOF)
			{
				m_nCursorPos--;
				fetch(SDB_POS_PRIOR, -1);
			}
		}

		DBG_ASSERT(m_nCurrentPos < m_nSize-nOffset,"ORowCache: Aktuelle Position darf nicht im freizugebenden Bereich liegen");
		DBG_ASSERT(m_nCursorPos  < m_nSize-nOffset,"ORowCache: Cursor darf nicht im freizugebenden Bereich stehen");

		// Die Eintraege RECHTS freigeben:
		for (i = m_nSize-nOffset; i < m_nSize; i++)
			at(i).unbind();

		// Alle uebrigen Eintraege nach RECHTS verschieben:
		for (i = m_nSize-1; i >= nOffset; i--)
			at(i) = at(i - nOffset);

		// Anschliessend die LINKEN Eintrage loeschen
		for (i = 0; i < nOffset; i++)
			at(i).unbind();

		// Aktuelle Position und Cursor-Position korrigieren:
		m_nCurrentPos	+= nOffset;
		m_nCursorPos	+= nOffset;
		DBG_ASSERT(m_nCurrentPos >= 0, "ORowCache::Shift : invalid current pos !");
		DBG_ASSERT(m_nCurrentPos < m_nSize, "ORowCache::Shift : invalid current pos !");
	}
	else if (eDirection == SDB_DIR_RIGHT)
	{
		// Falls der Cursor im freizugebenden Bereich steht, muss er da
		// erst weg!
		if (m_nCursorPos < nOffset)
		{
			DBG_ASSERT(m_nCursorPos < m_nSize - 1,"ORowCache: FetchRow ausserhalb des Cache-Bereiches");
			DBG_ASSERT(!m_rCursor.bEOF, "Cursor darf nicht hinter dem Ende stehen!")
			if (m_rCursor.bBOF)
			{
				m_nCursorPos++;
				if (!fetch(SDB_POS_FIRST))
					return;
			}
			while (m_nCursorPos < nOffset && !m_rCursor.bBOF)
			{
				m_nCursorPos++;
				fetch(SDB_POS_NEXT, 1);
			}
		}

		DBG_ASSERT(m_nCurrentPos >= nOffset,"ORowCache: Aktuelle Position darf nicht im freizugebenden Bereich liegen");
		DBG_ASSERT(m_nCursorPos >= nOffset,"ORowCache: Cursor darf nicht im freizugebenden Bereich stehen");

		// Die Eintraege LINKS freigeben:
		for (i = 0; i < nOffset; i++)
			at(i).unbind();

		// Alle Eintraege nach LINKS verschieben:
		for (i=0; i < m_nSize - nOffset; i++)
			at(i) = at(i + nOffset);

		// Anschliessend die RECHTEN Eintrage loeschen
		for (i = m_nSize-nOffset; i < m_nSize; i++)
			at(i).unbind();

		// Aktuelle Position und Cursor-Position korrigieren:
		m_nCurrentPos	-= nOffset;
		m_nCursorPos	-= nOffset;
		DBG_ASSERT(m_nCurrentPos >= 0, "ORowCache::Shift : invalid current pos !");
		DBG_ASSERT(m_nCurrentPos < m_nSize, "ORowCache::Shift : invalid current pos !");
	}
	else
	{
		DBG_ERROR("ORowCache: Interner Fehler: unbekannte Richtung");
	}
}

//--------------------------------------------------------------------------
BOOL ORowCache::fetch(UINT16 ePosition, INT32 nPos)  // SdbCursorPosition kann leider nicht verwendet werden
{
	// Die naechste Row von der Datenbank besorgen:
	// immer von der CursorPos
	ODbRowRef& rRow = at(m_nCursorPos);

	// Fetch ausfuehren:
	if (!m_rCursor.DoMove((SdbCursorPosition)ePosition, nPos, FALSE, FALSE, TRUE) &&
		!m_rCursor.Status().IsCanceled())
	{
		switch ((SdbCursorPosition)ePosition)
		{
			case SDB_POS_ABSOLUTE:
			case SDB_POS_FIRST:
				clear(SDB_DIR_RIGHT, 0);
				// Offright-Markierung setzen:
				at(0) = OFFLEFT;
				at(1) = OFFRIGHT;
				// An das Ende positionieren
				m_nCurrentPos	= m_nCursorPos = 1;
				break;
			case SDB_POS_LAST:
				clear(SDB_DIR_RIGHT, 0);
				// Offright-Markierung setzen:
				at(0) = OFFLEFT;
				at(1) = OFFRIGHT;
				// An das Ende positionieren
				m_nCurrentPos	= m_nCursorPos = 1;
				break;
			case SDB_POS_PRIOR:
				rRow = OFFLEFT;
				if (m_nCursorPos > 0)
					clear(SDB_DIR_LEFT, m_nCursorPos - 1);
				break;
			case SDB_POS_RELATIVE:
				if (nPos < 0)
				{
					rRow = OFFLEFT;
					if (m_nCursorPos > 0)
						clear(SDB_DIR_LEFT, m_nCursorPos - 1);
					break;
				}
				else if (!nPos)
				{
					if (m_rCursor.bBOF)
					{
						rRow = OFFLEFT;
						if (m_nCursorPos > 0)
							clear(SDB_DIR_LEFT, m_nCursorPos - 1);
						break;
					}
				}
			default:
				rRow = OFFRIGHT;
				if (m_nCursorPos < m_nSize - 1)
					clear(SDB_DIR_RIGHT, m_nCursorPos + 1);
		}
		m_rCursor.aStatus.Clear();
		return FALSE;
	}
	else
	{
		DBG_ASSERT(!isObsolete(rRow), "Fehler im Cache");
		if (!rRow.is() || isOffRange(rRow) )  // wenn eine Zeile vorhanden, dann diese verwenden
			rRow = new ODbRow(*m_rCursor.aResultRow);				 // sonst eine neue vorsehen
		else
			(*rRow) = *m_rCursor.aResultRow;						 // direkt die Zeile zuweisen
	}
	return !m_rCursor.Status().IsCanceled();
}

//--------------------------------------------------------------------------
void ORowCache::fill(SdbDirection eDirection, UINT16 nRequestedRowsInCache)
{
	INT16 nDiff = INT16(m_nCurrentPos) - INT16(m_nCursorPos);
	if (nDiff < 0)
	{
		DBG_ASSERT(!(isOffLeft(at(m_nCursorPos))), "Falsche Position!");
		if (m_rCursor.bEOF)					// EOF
		{
			m_nCursorPos--;
			if (!fetch(SDB_POS_LAST))
				return;
			clear(SDB_DIR_RIGHT, m_nCursorPos + 1);
			nDiff++;
		}
		m_nCursorPos = m_nCurrentPos;
		if (nDiff)
			fetch(SDB_POS_RELATIVE, nDiff);
	}
	else if (nDiff > 0)
	{
		DBG_ASSERT(!(isOffRight(at(m_nCursorPos))), "Falsche Position!");
		if (isOffLeft(at(m_nCursorPos)))		// BOF
		{
			m_nCursorPos++;
			if (!fetch(SDB_POS_FIRST, 1))
				return;
			nDiff--;
		}

		m_nCursorPos = m_nCurrentPos;
		if (nDiff)
			fetch(SDB_POS_RELATIVE, nDiff);
	}
	else	// wenn der Cursor am Ende auf EOF oder BOF steht ?
	{
		if (m_rCursor.bBOF && !fetch(SDB_POS_FIRST, 1))
			return;
		else if (m_rCursor.bEOF && !fetch(SDB_POS_LAST, 1))
			return;
	}

	UINT16 n = nRequestedRowsInCache;
	if (n == 0)
		n = m_nSize;			// alle (also mindestens bis zum Ende des Cache)
	else
		n--;				// Eine Row ist ja schon da ...

	if (eDirection == SDB_DIR_LEFT)
	{
		while (n != 0 && m_nCursorPos > 0 && !isOffLeft(at(m_nCursorPos)))
		{
			m_nCursorPos--;
			fetch(SDB_POS_PRIOR);
			n--;
		}
	}
	else if (eDirection == SDB_DIR_RIGHT)
	{
		while (n != 0 && m_nCursorPos < m_nSize - 1 && !isOffRight(at(m_nCursorPos)))
		{
			m_nCursorPos++;
			fetch(SDB_POS_NEXT);
			n--;
		}
	}
	else
	{
		DBG_ERROR("ORowCache: Interner Fehler: unzulaessige Direction");
	}
}

//--------------------------------------------------------------------------
void ORowCache::rowInserted()
{
	// Es ist eine Zeile erfolgreich eingefuegt worden. D.h. fuer den Cache, da er
	// falls schon das SetEnde erreicht wurde (isOffRight()),
	// dieses wieder offen ist, unbestimmt.
	// Damit wird vorausgesetzt, da der neue Satz ans Ende angehaengt wird oder nicht
	// im Set erscheint.
	UINT16 i = 0;
	while (i < m_nSize && !isOffRight(at(i)))
		i++;

	// Position steht auf EOF, dann ersetzen durch neuen Satz
	if (i < m_nSize)
	{
		if ((i + 1) < m_nSize) // EOF um eins verschieben
			at(i+1) = at(i);

		at(i) = new ODbRow(*m_rCursor.aResultRow);
	}
}

//--------------------------------------------------------------------------
void ORowCache::rowDeleted()
{
	DBG_ASSERT(m_nCursorPos == m_nCurrentPos, "Positionsprobleme im Cache");
	// Alle Datensaetze die ab der aktuelle Position um eins verschieben, damit der
	// geloeschte Satz nicht mehr erscheint
	if (m_nCurrentPos > 0)
	{
		for (USHORT i = m_nCurrentPos; i < (m_nSize - 1); i++)
			at(i) = at(i+1);
		// letzen freigeben
		at(m_nSize - 1).unbind();

		m_nCurrentPos--;
		m_nCursorPos--;
	}
	else
	{
		// aktuelle Row mu nachgelesen werden
		at(m_nCurrentPos).unbind();
		fetch(SDB_POS_RELATIVE, 0);
	}
}

//--------------------------------------------------------------------------
INT32 ORowCache::getCurrentPos() const
{
	if (m_rCursor.nCurrentPos == SDB_CURRENT_ROW_UNDEFINED)
		return SDB_CURRENT_ROW_UNDEFINED;
	else
	{
		INT32 nRel = INT32(m_nCurrentPos) - INT32(m_nCursorPos); // Convertierung beachten sonst unterlauf
		return nRel + m_rCursor.nCurrentPos;
	}
}

//--------------------------------------------------------------------------
BOOL ORowCache::synchronize(BOOL bRefreshData)
{
	// Refreshen einer Zeile:
	// Wenn CachePosition = CursorPosition, dann kein Problem da dann der
	// Cursor einfach neu einlesen mu
	// andernfalls muessen Cursor und Cache synchronisiert werden

	// hier gibt es nichts zu tun
	if (isOffLeft() || isOffRight() || isObsolete())
		return TRUE;

	ODbRowRef& rRow = at(m_nCurrentPos);		// aktuelle Zeile im Cache

	// Jetzt die relative Positionierung
	INT32 nRel = INT32(m_nCurrentPos) - INT32(m_nCursorPos); // Convertierung beachten sonst unterlauf

	// the former implementation returned TRUE if nRel == 0. But we have to give the cursor itself the
	// chance to update it's internal state from the current position (e.g. the nFilePos of a SdbFileCursor)
	// 18.06.99 - FS - #67050#
	m_rCursor.DoMove(SDB_POS_RELATIVE, nRel, FALSE, FALSE, bRefreshData);
	if (m_rCursor.Status().IsSuccessful())
	{
		if (bRefreshData)
		{
			if (rRow.is())
				(*rRow) = *m_rCursor.aResultRow;						// Zeile uebernehmen
			else
				rRow = new ODbRow(*m_rCursor.aResultRow);
		}
		m_rCursor.aCurrentRow = rRow;									// Cursor zeigt wieder auf CacheRow
		m_nCursorPos = m_nCurrentPos;									// Der Abgleich zwischen Cursor und Cache hat geklappt
		m_rCursor.aCurrentRow->setModified(FALSE);
		return TRUE;
	}
	else if (m_rCursor.Status().IsNoDataFound())						// Datensatz nicht gefunden
	{
		m_rCursor.aCurrentRow = rRow;									// Cursor zeigt wieder auf CacheRow
		m_nCursorPos = m_nCurrentPos;									// Der Abgleich zwischen Cursor und Cache hat geklappt
		if (m_rCursor.aCurrentRow.is())
		{
			m_rCursor.aCurrentRow->setModified(FALSE);
			m_rCursor.aCurrentRow->setState(ROW_DELETED);
		}
		return FALSE;
	}
	else
		return FALSE;	 											// Cursor Move hat nicht geklappt
}

//--------------------------------------------------------------------------
BOOL ORowCache::refresh()
{
	BOOL bResult;
	if (m_rCursor.nCurrentPos == SDB_CURRENT_ROW_UNDEFINED)
		bResult = FALSE;
	else if (isOnFirst())
		bResult = ORowCache::positionAbsolute(ROWCACHE_FIRST, TRUE);
	else if (isOnLast())
		bResult = ORowCache::positionAbsolute(ROWCACHE_LAST, TRUE);
	else if (isOffLeft()) // vor dem ersten
	{
		bResult = ORowCache::positionAbsolute(ROWCACHE_FIRST, TRUE);
		if (bResult)
			positionRelative(SDB_DIR_LEFT, 1);
	}
	else if (isOffRight()) // vor dem ersten
	{
		bResult = ORowCache::positionAbsolute(ROWCACHE_LAST, TRUE);
		if (bResult)
			positionRelative(SDB_DIR_RIGHT, 1);
	}
	else
		bResult = ORowCache::positionAbsolute(getCurrentPos() + 1, TRUE);
		// Absolut-Positionierung ist noch 1-, das GetCurrentPos dagegen 0-basiert

	return bResult;
}

//--------------------------------------------------------------------------
BOOL ORowCache::isOffRight(BOOL bLookForward) const
{
	// aktuelle Row
	ODbRowRef xRow = at(m_nCurrentPos);

	// selber schon als out of range markiert ? -> raus mit ja
	if (isOffRight(xRow))
		return TRUE;

	// auch nicht geloescht ? -> raus mit nein (es ist eine voll gueltige Row)
	else if (isOffLeft(xRow) || (xRow.is() && !isObsolete(xRow)))
		return FALSE;

	// ich stehe auf geloeschter Zeile ... kennt der Cursor die Gesamtzahl ? wenn ja, Position abgleichen
	else if (m_rCursor.IsRowCountFinal())
		return getCurrentPos() >= (INT32)m_rCursor.RowCount();

	if (m_nCurrentPos < m_nSize - 1)
	{
		const ODbRowRef& rxNext = at(m_nCurrentPos + 1);
		// noch Platz im Cache, mal sehen, ob die naechste Zeile mir was sagen kann
		if (isOffRight(rxNext))
			return TRUE;	// (aktuelle geloescht, naechste ist ausserhalb -> aktuelle auch ausserhalb)

		DBG_ASSERT(!isObsolete(rxNext), "ORowCache::isOffRight : two obsolete rows shouldn't happen !");
			// normalerweise wird eine obsolete Row, wenn die Positionierung von ihr weggeht, entfernt
		if (rxNext.is())
			return FALSE;	// (aktuelle geloescht, aber naechste existiert -> nicht ausserhalb)
	}

	if (bLookForward)	// flag if this method is used within implPositionRelative
	{
		// hierher, wenn ich nicht weiss, ob es eine naechste Zeile gibt -> nachschauen ...
		// (entweder bin ich schon ganz hinten im Cache, oder die naechste Row wurde noch nicht gefetcht)

		ORowCache* pThis = (ORowCache*)this;
		// weiter rechts nachsehen
		pThis->implPositionRelative(SDB_DIR_RIGHT);
		// Zustand restaurieren
		DBG_ASSERT(m_nCurrentPos > 0, "ORowCache::Refresh : invalid current pos !");
		--pThis->m_nCurrentPos;

		pThis->m_rCursor.eMode	   = SdbCursor::SDB_MOD_NONE;
		pThis->m_rCursor.aCurrentRow = xRow;
		pThis->m_rCursor.aStatus.Clear();

		return m_rCursor.IsRowCountFinal() && (getCurrentPos() >= (INT32)m_rCursor.RowCount());
	}
	else
		return FALSE;
}

//--------------------------------------------------------------------------
BOOL ORowCache::isOffLeft() const
{
	// aktuelle Row
	ODbRowRef xRow = at(m_nCurrentPos);
	// selber schon als out of range markiert ? -> raus mit ja
	if (isOffLeft(xRow))
		return TRUE;

	if (getCurrentPos() == SDB_CURRENT_ROW_UNDEFINED)
		return TRUE;

	// auch nicht geloescht ? -> raus mit nein (es ist eine voll gueltige Row)
	if (!isObsolete(xRow))
		return FALSE;

	// ich stehe auf geloeschter Zeile ...
	if (getCurrentPos() == 0)
		return TRUE;	// (aktuelle Zeile ist die erste und geloescht -> ausserhalb)

	if (m_nCurrentPos > 0)
	{
		const ODbRowRef& rxPrev = at(m_nCurrentPos - 1);
		// wir haben links noch was im Cache, wo wir nachsehen koennen
		if (isOffLeft(rxPrev))
			return TRUE;	// (aktuelle geloescht, voherige ausserhalb -> aktuelle auch ausserhalb)

		DBG_ASSERT(!isObsolete(rxPrev), "ORowCache::isOffLeft : two obsolete rows shouldn't happen !");
			// normalerweise wird eine obsolete Row, wenn die Positionierung von ihr weggeht, entfernt
		if (rxPrev.is())
			return FALSE;	// (aktuelle geloescht, aber voherige existiert -> nicht ausserhalb)
	}

	// hierher, wenn die aktuelle Zeile geloescht ist und sich entweder an Position 0 befindet, oder noch
	// keine vorhergehende Zeile gefetcht wurde

	ORowCache* pThis = (ORowCache*)this;
	// weiter rechts nachsehen
	pThis->implPositionRelative(SDB_DIR_LEFT);
	// Zustand restaurieren
	++pThis->m_nCurrentPos;

	pThis->m_rCursor.eMode	   = SdbCursor::SDB_MOD_NONE;
	pThis->m_rCursor.aCurrentRow = xRow;
	pThis->m_rCursor.aStatus.Clear();

	return (getCurrentPos() == 0);
}

//--------------------------------------------------------------------------
BOOL ORowCache::isOnFirst() const
{
	if (isObsolete(at(m_nCurrentPos)))
		// entweder kommt vor dem aktuellen noch ein gueltiger, dann bin ich nicht auf dem ersten, oder es
		// kommt keiner mehr, dann bin ich vor dem ersten
		return FALSE;

	if (!at(m_nCurrentPos).is())
		return FALSE;

	if (m_rCursor.nCurrentPos == SDB_CURRENT_ROW_UNDEFINED || isOffLeft() || isOffRight())
		return FALSE;

	if (m_nCurrentPos == 0)
	{
		// ist der Cache auf der ersten Position und der Cursor ebenfalls
		if (m_nCursorPos == 0)
			return m_rCursor.nCurrentPos == 0 && !m_rCursor.bEOF;
		else
			return FALSE;
	}

	if (m_nCurrentPos == 0 || !at(m_nCurrentPos - 1).is())
	{
		// Zeilen merken
		ODbRowRef xRow = at(m_nCurrentPos);

		ORowCache* pThis = (ORowCache*)this;

		INT32 nInvariant = m_nCurrentPos + m_rCursor.nCurrentPos - m_nCursorPos;
		// weiter links nachsehen
		pThis->implPositionRelative(SDB_DIR_LEFT);
		++pThis->m_nCurrentPos;
		pThis->m_nCursorPos = m_rCursor.nCurrentPos + m_nCurrentPos - nInvariant;

		pThis->m_rCursor.eMode	   = SdbCursor::SDB_MOD_NONE;
		pThis->m_rCursor.aCurrentRow = xRow;
		pThis->m_rCursor.aStatus.Clear();
	}

	return (getCurrentPos() == 0);
}

//--------------------------------------------------------------------------
BOOL ORowCache::isOnLast() const
{
	if (isObsolete(at(m_nCurrentPos)))
		// entweder kommt hinter dem aktuellen noch ein gueltiger, dann bin ich nicht auf dem letzten, oder es
		// kommt keiner mehr, dann bin ich hinter dem letzten
		return FALSE;

	if (!at(m_nCurrentPos).is())
		return FALSE;

	if (m_rCursor.nCurrentPos == SDB_CURRENT_ROW_UNDEFINED || isOffLeft() || isOffRight())
		return FALSE;

/*	if (m_nCurrentPos >= m_nSize - 1 && m_rCursor.IsRowCountFinal())
	{
		// ist der Cache auf der ersten Position und der Cursor ebenfalls
		if (m_nCursorPos == m_nCurrentPos && m_rCursor.IsRowCountFinal())
			return (m_rCursor.nCurrentPos + 1) == (INT32)m_rCursor.RowCount();
		else
			return FALSE;
	}
*/	// Dieser Teil ist irgendwie sinnlos. Nennen wir es mal if (a && b) { if (c) x; else y; }.
	// Dann entspricht x (eigentlich : if (b && c) x;) genau der Behandlung, die gleich unten erfolgt
	// (bei IsRowCountFinal()), ist dabei allerdings voellig unabhaengig von a. Der Fall wird also unten mit erschlagen.
	// Der andere Fall ist sinnlos, denke ich. Warum soll ueberhaupt ein IsOnLast sich anderes verhalten, wenn der
	// Cache-Zeiger am Ende steht ?
	// Da ich mir aber nur 99.999-prozentig sicher bin und der Cache immer sehr ein sensibler Bereich ist, lasse ich das erst
	// mal auskommetiert stehen ;)
	// (FS - 64380)

	if (m_rCursor.IsRowCountFinal())
	{
		// ist der aktuelle Datensatz geloescht?
		// beim Loeschen wird der RowCount runtergezaehlt
		DBG_ASSERT(!(at(m_nCurrentPos).is() && at(m_nCurrentPos)->isObsolete()), "ORowCache::IsOnLast : an obsolete record should have been handled earlier !");
		return (getCurrentPos() + 1) == (INT32)m_rCursor.RowCount();
	}

	DBG_ASSERT((m_nCurrentPos >= m_nSize-1) || !isOffRight(at(m_nCurrentPos + 1)), "ORowCache::IsOnLast : inconsistent state !");
		// OffRight heisst, dass sogar der Cache schon weiss, wo das Ende ist, also erst recht der Cursor, also
		// muesste m_rCursor.IsRowCountFinal() angesprungen sein
	if (m_nCurrentPos == m_nSize - 1 || !at(m_nCurrentPos + 1).is())
	{	// muss leider erst mal eine neue Zeile lesen (lassen)
		// Zeilen merken
		SdbCursor::SdbCursorMode eMode = SdbCursor::SDB_MOD_NONE;
		ODbRowRef xRow = at(m_nCurrentPos);

		ORowCache* pThis = (ORowCache*)this;

		INT32 nInvariant = m_nCurrentPos + m_rCursor.nCurrentPos - m_nCursorPos;
#ifdef _DEBUG
		INT32 nOldCurrent = m_nCurrentPos;
		INT32 nOldCursor = m_nCursorPos;
		INT32 nOldCursorCurrent = m_rCursor.nCurrentPos;
#endif // _DEBUG
		// in die Mitte verschieben
		pThis->implPositionRelative(SDB_DIR_RIGHT);
		DBG_ASSERT(m_nCurrentPos>0, "ORowCache::IsOnLast : something went wrong while positioning !");
		pThis->m_nCurrentPos--;
		pThis->m_nCursorPos = m_rCursor.nCurrentPos + m_nCurrentPos - nInvariant;

		pThis->m_rCursor.eMode	   = eMode;
		pThis->m_rCursor.aCurrentRow = xRow;
		pThis->m_rCursor.aStatus.Clear();
	}

	return m_rCursor.IsRowCountFinal() && (getCurrentPos() + 1) == (INT32)m_rCursor.RowCount();
}



