/*************************************************************************
 *
 *  $RCSfile: sdbcol.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbcol.hxx"

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

TYPEINIT0(SdbColumn);

DBG_NAME(SdbColumn);
DBG_NAME(SdbColumns);

//------------------------------------------------------------------
BOOL HasLength(SdbDatabaseType eType)
{
	switch(eType)
	{// Felder die eine Laengenangabe benoetigen
		case SDB_DBTYPE_TINYINT:
		case SDB_DBTYPE_SMALLINT:
		case SDB_DBTYPE_INTEGER:
		case SDB_DBTYPE_BIGINT:
		case SDB_DBTYPE_REAL:
#ifndef TF_ONE51
		case SDB_DBTYPE_FLOAT:
#endif
		case SDB_DBTYPE_DOUBLE:
		case SDB_DBTYPE_DATE:
		case SDB_DBTYPE_TIME:
		case SDB_DBTYPE_TIMESTAMP:
		case SDB_DBTYPE_CURRENCY:
		case SDB_DBTYPE_BOOLEAN:
		case SDB_DBTYPE_LONGVARCHAR:
		case SDB_DBTYPE_LONGVARBINARY:
			return FALSE;
		default:
			return TRUE;
	}
}

//------------------------------------------------------------------------
// SdbColumn:
//------------------------------------------------------------------
//------------------------------------------------------------------
SdbColumn::SdbColumn(const String& rNewName, const SdbColumn& rSdbColumn)
		:aName(rNewName)
		,eType(rSdbColumn.GetType())
		,nLength(rSdbColumn.GetLength())
		,nScale((UINT8) rSdbColumn.GetScale())
		,nFlags(rSdbColumn.GetAllFlags())
		,nId(rSdbColumn.GetId())
		,nFormatKey(rSdbColumn.GetFormatKey())
		,aRealName(rSdbColumn.aName)
		,aDefaultValue(rSdbColumn.aDefaultValue)
{
	DBG_ASSERT(rSdbColumn.nScale <= rSdbColumn.nLength,"SdbColumn: Scale darf nicht groesser als die Precision sein");
	DBG_CTOR(SdbColumn,NULL);
	pInfo = new SdbColumnInfo(rSdbColumn.GetColumnInfo());

}

SdbColumn::SdbColumn(const String &rNewName, SdbDatabaseType eNewType,
					 UINT32 nNewLength, UINT16 nNewScale,
					 UINT16 nNewFlags, UINT32 _nFormatKey)
		  :aName(rNewName)
		  ,eType(eNewType)
		  ,nLength(nNewLength)
		  ,nScale(nNewScale)
		  ,nId(0)
		  ,nFormatKey(_nFormatKey)
		  ,aRealName(rNewName)
{
	DBG_ASSERT(nNewScale <= nNewLength,"SdbColumn: Scale darf nicht groesser als die Precision sein");
	DBG_CTOR(SdbColumn,NULL);
	pInfo = new SdbColumnInfo(String());
	SetAllFlags(nNewFlags);
}

//------------------------------------------------------------------
SdbColumn::SdbColumn(const SdbColumn &rSdbColumn)
		  :aName(rSdbColumn.aName)
		  ,eType(rSdbColumn.GetType())
		  ,nLength(rSdbColumn.GetLength())
		  ,nScale((UINT8) rSdbColumn.GetScale())
		  ,nFlags(rSdbColumn.GetAllFlags())
		  ,nId(rSdbColumn.GetId())
		  ,nFormatKey(rSdbColumn.GetFormatKey())
		  ,aRealName(rSdbColumn.aRealName)
		  ,aDefaultValue(rSdbColumn.aDefaultValue)
{
	DBG_ASSERT(rSdbColumn.nScale <= rSdbColumn.nLength,"SdbColumn: Scale darf nicht groesser als die Precision sein");
	DBG_CTOR(SdbColumn,NULL);
	pInfo = new SdbColumnInfo(rSdbColumn.GetColumnInfo());
}

//------------------------------------------------------------------
SdbColumn::SdbColumn(const SdbColumn &rSdbColumn, UINT16 nFlagMask)
		  :aName(rSdbColumn.aName)
		  ,eType(rSdbColumn.GetType())
		  ,nLength(rSdbColumn.GetLength())
		  ,nScale((UINT8)rSdbColumn.GetScale())
		  ,nFlags(rSdbColumn.GetAllFlags() & nFlagMask)
		  ,nId(rSdbColumn.GetId())
		  ,nFormatKey(rSdbColumn.GetFormatKey())
		  ,aRealName(rSdbColumn.aName)
		  ,aDefaultValue(rSdbColumn.aDefaultValue)
{
	DBG_ASSERT(rSdbColumn.nScale <= rSdbColumn.nLength,"SdbColumn: Scale darf nicht groesser als die Precision sein");
	DBG_CTOR(SdbColumn,NULL);
	pInfo = new SdbColumnInfo(rSdbColumn.GetColumnInfo());
}

//------------------------------------------------------------------
SdbColumn::~SdbColumn()
{
	DBG_DTOR(SdbColumn,NULL);
	delete pInfo;
}

//------------------------------------------------------------------
BOOL SdbColumn::operator==(const SdbColumn &rSdbColumn) const
{
	DBG_CHKTHIS(SdbColumn,NULL);
	DBG_CHKOBJ(&rSdbColumn,SdbColumn,NULL);

	BOOL bEqual =	eType == rSdbColumn.GetType() &&
					(HasLength(eType) ?
						nLength == rSdbColumn.GetLength() &&
						nScale == rSdbColumn.GetScale() : TRUE) &&
					nFlags == rSdbColumn.GetAllFlags();

	return bEqual ? ::utl::UStringMixEqual(IsCaseSensitive())(aName,rSdbColumn.aName) : FALSE;
}

//------------------------------------------------------------------
BOOL SdbColumn::Compare(const SdbColumn &rSdbColumn, INT16 nFlagMask) const
{
	DBG_CHKTHIS(SdbColumn,NULL);
	DBG_CHKOBJ(&rSdbColumn,SdbColumn,NULL);

	BOOL bEqual = eType == rSdbColumn.GetType() &&
				  (HasLength(eType) ?
						nLength == rSdbColumn.GetLength() &&
						nScale == rSdbColumn.GetScale() : TRUE) &&
				  (nFlags & (UINT16) nFlagMask) == (rSdbColumn.GetAllFlags() & (UINT16)nFlagMask);

	return bEqual ? ::utl::UStringMixEqual(IsCaseSensitive())(aName,rSdbColumn.aName) : FALSE;
}
//------------------------------------------------------------------
BOOL SdbColumn::CompareWithOutName(const SdbColumn &rSdbColumn, INT16 nFlagMask) const
{
	DBG_CHKTHIS(SdbColumn,NULL);
	DBG_CHKOBJ(&rSdbColumn,SdbColumn,NULL);

	return eType == rSdbColumn.GetType() &&
		   (HasLength(eType) ?
				nLength == rSdbColumn.GetLength() &&
				nScale == rSdbColumn.GetScale() : TRUE) &&
			(nFlags & nFlagMask) == (rSdbColumn.GetAllFlags() & nFlagMask);

}
//------------------------------------------------------------------
SdbColumn* SdbColumn::Clone() const
{
	return new SdbColumn(*this);
}

//------------------------------------------------------------------
void SdbColumn::SetAutoIncrement()
{
	// setzen einer Column auf autoinkrement,
	// dieses sollte in Zukunkt nicht mit einem Typ sondern mit einem Flag geschehen

	nFlags |= SDB_FLAGS_AUTOINCREMENT;
	nFlags &= ~SDB_FLAGS_UPDATEALLOWED;
}

//------------------------------------------------------------------------
// SdbColumns:
//------------------------------------------------------------------
SdbColumns::SdbColumns(SdbConnection& _rConn)
		   :m_aMap(::utl::UStringMixLess(_rConn.GetIdentifierCase() == SDB_IC_ALL))
{
	DBG_CTOR(SdbColumns,NULL);

	// Jede Columns-Stukturbeschreibung enthaelt eine Bookmark:
	if (_rConn.IsOpen())
		AddColumn(_rConn.CreateColumn(String::CreateFromAscii("[Bookmark]"),SDB_DBTYPE_BOOKMARK,
							0 , 0, _rConn.GetIdentifierCase() == SDB_IC_ALL ? SDB_FLAGS_CASESENSITIVE : 0));
}

//------------------------------------------------------------------
SdbColumns::SdbColumns(SdbConnection& _rConn, const SdbColumns &rSdbColumns)
		   :m_aMap(::utl::UStringMixLess(_rConn.GetIdentifierCase() == SDB_IC_ALL))
{
	DBG_CTOR(SdbColumns,NULL);
	DBG_CHKOBJ(&rSdbColumns,SdbColumns,NULL);

	// Die Referenzen der kopierten Strukturbeschreibung (SdbColumns)
	// werden natuerlich nicht mitkopiert.
	// (siehe SdbReferenceCount)

	// Alle uebrigen Columns kopieren:
	if (_rConn.IsOpen())
	{
		UINT16 nCount = rSdbColumns.Count();
		for (UINT16 i = 0; i < nCount; i++)
		{
			SdbColumn* pColumn = rSdbColumns[i]->Clone();
			if (_rConn.GetIdentifierCase() == SDB_IC_ALL)
				pColumn->SetFlag(SDB_FLAGS_CASESENSITIVE);
			else
				pColumn->ResetFlag(SDB_FLAGS_CASESENSITIVE);

			AddColumn(pColumn);
		}
	}
}

//------------------------------------------------------------------
SdbColumns::~SdbColumns()
{
	DBG_DTOR(SdbColumns,NULL);	
}

//------------------------------------------------------------------
void SdbColumns::Reset()
// loescht alle Spalten bis auf die BOOKmark Spalte
{
	SdbColumnRef xCol = *begin();		
	vector<SdbColumnRef>::clear();
	m_aMap.clear();				  
	AddColumn(xCol.getBodyPtr());	
}

//------------------------------------------------------------------
BOOL SdbColumns::operator==(const SdbColumns &rColumns) const
{
	DBG_CHKTHIS(SdbColumns,NULL);
	DBG_CHKOBJ(&rColumns,SdbColumns,NULL);

	if (size() != rColumns.size())
		return FALSE;
	else
	{
		vector<SdbColumnRef>::const_iterator i = begin() + 1;
		vector<SdbColumnRef>::const_iterator j = rColumns.begin() + 1;
		for (; i != end(); i++)
		{
			if (**i != **j)
				return FALSE;
			j++;
		}
		return TRUE;
	}
}

//------------------------------------------------------------------
BOOL SdbColumns::Compare(const SdbColumns& rColumns, INT16 nFlagMask) const
{
	DBG_CHKTHIS(SdbColumns,NULL);
	DBG_CHKOBJ(&rColumns,SdbColumns,NULL);

	if (Count() != rColumns.Count())
		return FALSE;
	else
	{
		vector<SdbColumnRef>::const_iterator i = begin() + 1;
		vector<SdbColumnRef>::const_iterator j = rColumns.begin() + 1;
		for (; i != end(); i++)
		{
			if (!(*i)->Compare(*((*j).getBodyPtr()),nFlagMask)) 
				return FALSE;			
			j++;
		}
		return TRUE;
	}
}

//------------------------------------------------------------------
UINT16 SdbColumns::GetColumnNumberByRealName(const String& rRealName) const
{
	DBG_CHKTHIS(SdbColumns,NULL);

	::std::map<String, SdbColumnRef, ::utl::UStringMixLess>::const_iterator i = m_aMap.find(rRealName);
	::utl::UStringMixEqual aEqual(m_aMap.key_comp().isCaseSensitive());
	if (i != m_aMap.end() && aEqual((*i).second->GetRealName(), rRealName))
        return (*i).second->GetId();           
	else
	{
		for (::std::vector<SdbColumnRef>::const_iterator i = (begin() + 1); i != end(); i++)
		{
			if (aEqual((*i)->GetRealName(), rRealName))
				return i - begin();
		}
	}

	return SDB_COLUMN_NOTFOUND;
}

//------------------------------------------------------------------
const SdbColumn* SdbColumns::at(UINT16	_nColNo) const
{
	const SdbColumn *pRet = NULL;
	if(_nColNo < Count())
		pRet = (*this)[_nColNo];
	return pRet;
}
//------------------------------------------------------------------
UINT16 SdbColumns::GetVisibleCount() const
{
	UINT16 nLen=0;
	for (vector<SdbColumnRef>::const_iterator i = begin(); i != end(); i++)
	{
		if (!((*i)->GetAllFlags() & SDB_FLAGS_VERSIONING)) 
			++nLen;
	}	
	return nLen;
}

