/*************************************************************************
 *
 *  $RCSfile: sdbconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBCONN_HXX
#include <sdbconn.hxx>
#endif

#include <ctype.h>		//isdigit

#ifndef _COM_SUN_STAR_LANG_XCOMPONENT_HPP_
#include <com/sun/star/lang/XComponent.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASECAPABILITY_HPP_
#include <com/sun/star/data/DatabaseCapability.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

// Environment:
#ifndef _SDB_SDBRESID_HRC
#include "sdbresid.hrc"
#endif

#ifndef _SDB_SDBVIEW_HXX
#include "sdbview.hxx"
#endif
#ifndef _SDB_SDBREL_HXX
#include <sdbrel.hxx>
#endif
#ifndef _SDB_SDBMAP_HXX
#include <sdbmap.hxx>
#endif

DBG_NAME(SdbConnection);
TYPEINIT1(SdbConnection, SdbObj);

const char * SdbConnection::pReservedOptions = "TYP;OWNER;TABLE;CHARSET;TBL_FILTER;ASKWHENEMPTYPWD";
namespace staruno = com::sun::star::uno;
namespace stardata = com::sun::star::data;

//------------------------------------------------------------------
SdbConnection::SdbConnection(SdbEnvironment* pEnv, const String& rNewOptions)
			  :SdbObj(pEnv, FALSE)
			  ,aOptionsParser(rNewOptions,';','=',TRUE)
			  ,eCharacterSet(RTL_TEXTENCODING_DONTKNOW)
			  ,eIsolationLevel(SDB_ISO_NONE)
			  ,nTransCount(0)
			  ,cIdentifierQuotation('"')
			  ,cQualifierSeparator('.')
			  ,eIdentifierCase(SDB_IC_MIXED)
			  ,bAutoCommit(TRUE)
			  ,pTableNames(NULL)
{
	DBG_CTOR(SdbConnection,NULL);

	aUserName = aOptionsParser.OptionString(String::CreateFromAscii("UID"));
	SetCharacterSet(aOptionsParser.OptionString(String::CreateFromAscii("CHARSET")));
//	pTableNames = new SdbStringStlMap(SdbStringCompare(TRUE));
}


//------------------------------------------------------------------
BOOL SdbConnection::Create(){DBG_CHKTHIS(SdbConnection,NULL);return FALSE;}

//------------------------------------------------------------------
BOOL SdbConnection::Drop(){DBG_CHKTHIS(SdbConnection,NULL);return FALSE;}


//------------------------------------------------------------------
SdbConnection::~SdbConnection()
{
	if(pTableNames)
		delete pTableNames;
	DBG_DTOR(SdbConnection,NULL);
}

//------------------------------------------------------------------
void SdbConnection::RemoveChildren()
// freigeben aller Childs, siehe SdbObj
{
	DBG_CHKTHIS(SdbConnection,NULL);
	ULONG i;
	for ( i=0;i < aTableList.Count();i++)
		SdbObj::RemoveChild(aTableList.GetObject(i));
	aTableList.Clear();

	for ( i=0;i < aCursorList.Count();i++)
		SdbObj::RemoveChild(aCursorList.GetObject(i));
	aCursorList.Clear();

	for ( i=0;i < aRelationList.Count();i++)
		SdbObj::RemoveChild(aRelationList.GetObject(i));
	aRelationList.Clear();
}

//------------------------------------------------------------------
void SdbConnection::RemoveChild(SdbObj* pObj)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	DBG_ASSERT(pObj != NULL, "SdbConnection::RemoveChild : pObj sollte nicht NULL sein !");
	if (pObj->IsA(TYPE(SdbCursor)))
		aCursorList.Remove(PTR_CAST(SdbCursor,pObj));
#if SUPD < 561
	else if (pObj->IsA(TYPE(SdbTable)) && pObj->IsPersistent())
#else
	else if (pObj->IsA(TYPE(SdbTable)))
#endif // SUPD < 561
		aTableList.Remove(PTR_CAST(SdbTable,pObj));
	else if (pObj->IsA(TYPE(SdbRelation)))
		aRelationList.Remove(PTR_CAST(SdbRelation,pObj));

	SdbObj::RemoveChild(pObj);
}

BOOL SdbConnection::HasRelation(const String& rRelName) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return (aRelationList.Lookup(rRelName) != NULL);
};


//------------------------------------------------------------------
void SdbConnection::AddTableName(const String& strNew,BOOL bTable)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	(*pTableNames)[strNew] = bTable;
}

//------------------------------------------------------------------
BOOL SdbConnection::ReplaceTableName(const String& strOld, const String& strNew)
{
	DBG_CHKTHIS(SdbConnection,NULL);

	SdbStringStlMap::iterator aIter = pTableNames->find(strOld);
	if(aIter != pTableNames->end())
	{
		BOOL bTable = (*aIter).second;
		pTableNames->erase(aIter);
		(*pTableNames)[strNew] = bTable;

		return TRUE;
	}
	DBG_ERROR("Name war nicht in Map enthalten");
	return FALSE;
}

//------------------------------------------------------------------
void SdbConnection::Close()
{
	DBG_CHKTHIS(SdbConnection,NULL);

	SDB_CHK_OPEN();

	// Freigeben aller Objekte
	RemoveChildren();

	// Aufrumen der Implementierung
	CloseImpl();

	::staruno::Reference< com::sun::star::lang::XComponent >xComp(xDBMetaData, ::staruno::UNO_QUERY);
	if (xComp.is())
	{
		xComp->dispose();
		xDBMetaData = NULL;
	}

	nTransCount = 0;

	for (ULONG i = 0; i < aTypeList.Count(); i++)
		delete aTypeList.GetObject(i);

	SdbObj::Close();
}

//------------------------------------------------------------------
BOOL SdbConnection::Open()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	// aStatus.Clear();		// nicht den Status zuruecksetzen, da dieser im Constructor gesetzt werden kann
	// Feststellen ob ein Passwort benoetigt wird
	if (aStatus.IsError())
		return FALSE;

	if (!GetParent())
		return FALSE;

	BeginHdl().Call(this);

	String	aPassword = aOptionsParser.OptionString(String::CreateFromAscii("PWD"));
	String	aUID = aOptionsParser.OptionString(String::CreateFromAscii("UID"));
	String  aOptionStr = Options();

	// wenn ein Benutzer, aber kein Pwd, aber ein 'ask when empty password' -> nach Pwd fragen
	if (aUID.Len() && !aPassword.Len() && aOptionsParser.OptionBool(String::CreateFromAscii("ASKWHENEMPTYPWD")))
			// nyi: der String sollte irgendwo zentral hinwandern, damit ich das im SDB und im SBA ordentlich verwenden kann
	{
		aPassword.Erase();
		SdbEnvironment* pEnv = PTR_CAST(SdbEnvironment,GetParent());

		ULONG nPwdDialogResult = 0;

		// gibt es einen extended password handler ?
		if (pEnv->GetExtPasswordHdl().IsSet())
		{	// (dem koennen wir ein paar Informationen mehr mit auf den Weg geben)
			SdbLoginInformation aLogon;
			aLogon.nVersion = 1;
			aLogon.sUserName = aUID;
			nPwdDialogResult = pEnv->GetExtPasswordHdl().Call(&aLogon);
			aPassword = aLogon.sPassword;

			if (!aLogon.sUserName.Equals(aUID))
			{	// der Benutzer-Name wurde geaendert
				aOptionsParser.Parse(aOptionStr);
				aOptionsParser.SetOption(String::CreateFromAscii("UID"), aLogon.sUserName);
				aOptionStr = Options();

				aUserName = aLogon.sUserName;
			}
		}
		else
		{	// nein -> einfachen Handler benutzen (der fragt nur nach dem Passwort)
			nPwdDialogResult = pEnv->GetPasswordHdl().Call(&aPassword);
		}

		if (!nPwdDialogResult)
		{
			aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_CONNECTION))),
					0, String() );
			EndHdl().Call(this);
			return FALSE;
		}

		aOptionStr.EraseLeadingChars(';');
		aOptionStr.EraseTrailingChars(';');
		aOptionStr += String::CreateFromAscii(";PWD=");
		aOptionStr += aPassword;
		aOptionsParser.Clear();
		aOptionsParser.Parse(aOptionStr); // setzen der Optionen
		bOpen = OpenImpl();
	}
	else
		bOpen = OpenImpl();

	if (bOpen)
	{
		SetAutoCommit(TRUE);

		// wie sieht 's mit cassensitive aus
		aTableList.SetCaseSensitive((eIdentifierCase & SDB_IC_OBJECT) == SDB_IC_OBJECT);

		// TypeInfo einlesen
		FillTypeList(aTypeList);
	}

	else if (!bOpen && aStatus.IsSuccessful()) // es konnte kein Status gesetzt werden
	{
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_CONNECTION))),
					0, aEmptyString );
	}

	EndHdl().Call(this);
	return bOpen;
}

//------------------------------------------------------------------
SdbSqlParser* SdbConnection::GetSqlParser() const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SdbEnvironment* pEnv = GetEnvironment();
	return pEnv ? pEnv->GetSqlParser() : NULL;
}

//------------------------------------------------------------------
SdbEnvironment* SdbConnection::GetEnvironment() const
{
	DBG_CHKTHIS(SdbConnection,NULL);
#if DBG_UTIL
	return PTR_CAST(SdbEnvironment,GetParent());
#else
	return (SdbEnvironment*)GetParent();
#endif
}

//------------------------------------------------------------------
String SdbConnection::GetTableName(USHORT _nPos)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if(_nPos >= pTableNames->size())
		return aEmptyString;

	SdbStringStlMap::iterator iter = pTableNames->begin();
	for(USHORT i=0 ; i < _nPos ; i++)
		iter++;

	return (*iter).first;
}

//------------------------------------------------------------------
USHORT SdbConnection::GetTableCount()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if (!IsOpen())
		return 0;
	else
	{
		BeginHdl().Call(this);
		BusyHdl().Call(this);
		pTableNames->clear();
		FillTableNames(pTableNames);
		EndHdl().Call(this);
	}

	return pTableNames->size();
}

//------------------------------------------------------------------
SdbRelation* SdbConnection::CreateRelation(const String& _rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	aStatus.Clear();
	if (aRelationList.Lookup(_rName))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,dbRelation,_rName);
		return NULL;
	}

	SdbRelation* pRelation = CreateRelationImpl(_rName);
	if (pRelation)
		aRelationList.InsertAt(_rName, pRelation);
	return pRelation;
}

//------------------------------------------------------------------
SdbRelation* SdbConnection::CreateRelationImpl(const String& rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	aStatus.SetDriverNotCapableError();
	return NULL;
}

//------------------------------------------------------------------
SdbRelation* SdbConnection::OpenRelation(USHORT _nPos)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return (GetRelationCount()) ? aRelationList.GetObject(_nPos) : NULL;
}

//------------------------------------------------------------------
SdbRelation* SdbConnection::OpenRelation(const String& _rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return (GetRelationCount()) ? aRelationList.Lookup(_rName) : NULL;
}

//------------------------------------------------------------------
String SdbConnection::GetRelationName(USHORT _nPos)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	String aRelationName;
	if (GetRelationCount() && _nPos < aRelationList.Count())
		aRelationName = aRelationList.GetObject(_nPos)->GetName();
	return aRelationName;
}

//------------------------------------------------------------------
USHORT SdbConnection::GetRelationCount()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if (!IsOpen())
		return 0;

	if (!aRelationList.Count() && GetMetaData()->isCapable(::stardata::DatabaseCapability::INTEGRITY))
	{
		BeginHdl().Call(this);
		FillAllRelations(aRelationList);
		EndHdl().Call(this);
	}

	return (USHORT) aRelationList.Count();
}


//------------------------------------------------------------------------
void SdbConnection::FillAllRelations(SdbRelationList& _rList)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	// Default-Implementation. Eine Unterklasse sollte diese
	// Methode ueberschreiben und optimieren.

#ifdef _DEBUG
	USHORT nTabCount = GetTableCount();
	USHORT nCurrent = 0;
#endif
	for(SdbStringStlMap::const_iterator aIter = pTableNames->begin();aIter != pTableNames->end();++aIter)
	{
#ifdef _DEBUG
		++nCurrent;
#endif
		if ((*aIter).second)		// es ist keine View
			FillRelations(_rList, (*aIter).first);
	}
}

//------------------------------------------------------------------
BOOL SdbConnection::DoRefresh()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	pTableNames->clear();
	FillTableNames(pTableNames);
	return TRUE;
}

//------------------------------------------------------------------
void SdbConnection::SetIsolationLevel(SdbIsolationLevel eNewIsolationLevel)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN();
	SetIsolationLevelImpl(eNewIsolationLevel);
}

//------------------------------------------------------------------
void SdbConnection::SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	aStatus.SetDriverNotCapableError();
}

//------------------------------------------------------------------
BOOL SdbConnection::IsCapable(SdbCapability eCap) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return IsOpen() ? IsCapableImpl(eCap) : FALSE;
}
//------------------------------------------------------------------
::staruno::Reference< ::stardata::XDatabaseDescriptor > SdbConnection::GetMetaData()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if (!xDBMetaData.is())
		xDBMetaData = GetMetaDataImpl();
	return xDBMetaData;
}

//------------------------------------------------------------------
::stardata::XDatabaseDescriptor* SdbConnection::GetMetaDataImpl()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return NULL;
}

//------------------------------------------------------------------
BOOL SdbConnection::IsCapableImpl(SdbCapability eCap) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	switch (eCap)
	{
		case SDB_CAP_COUNT:
		case SDB_CAP_UPDATABLE:
		case SDB_CAP_BULKOPTIMIZE: // optimieren von Inserts
			return TRUE;
		case SDB_CAP_MULTITHREADACCESS:
			// assume thread safety as default
			// overwrite IsCapableImpl if your connection isn't thread safe
			return TRUE;
		default:
			return FALSE;
	}
}

//------------------------------------------------------------------
String SdbConnection::Options() const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	String aString;
	aOptionsParser.Compose(aString,aEmptyString);
	return aString;
}

//------------------------------------------------------------------
BOOL SdbConnection::Compare(const SdbObj& _rObj) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SdbConnection* pObj = PTR_CAST(SdbConnection,&_rObj);
	return pObj != NULL && Options() == pObj->Options();
}

//------------------------------------------------------------------
void SdbConnection::SetCharacterSet(const String & rNewCharacterSet)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if      (rNewCharacterSet.EqualsIgnoreCaseAscii("ANSI")) eCharacterSet = RTL_TEXTENCODING_MS_1252;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("MAC")) eCharacterSet = RTL_TEXTENCODING_APPLE_ROMAN;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC")
		 ||	rNewCharacterSet.EqualsIgnoreCaseAscii("DOS")) eCharacterSet = RTL_TEXTENCODING_IBM_850;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC_437")) eCharacterSet = RTL_TEXTENCODING_IBM_437;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC_850")) eCharacterSet = RTL_TEXTENCODING_IBM_850;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC_860")) eCharacterSet = RTL_TEXTENCODING_IBM_860;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC_861")) eCharacterSet = RTL_TEXTENCODING_IBM_861;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC_863")) eCharacterSet = RTL_TEXTENCODING_IBM_863;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("IBMPC_865")) eCharacterSet = RTL_TEXTENCODING_IBM_865;
	else if (rNewCharacterSet.EqualsIgnoreCaseAscii("SYSTEM")) eCharacterSet = gsl_getSystemTextEncoding();
	else
		eCharacterSet = gsl_getSystemTextEncoding();
}

//------------------------------------------------------------------
BOOL SdbConnection::ExecuteSQL(const String& rSQL)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_BOOL();

	SdbCursorRef xCursor(CreateCursor(SDB_DYNAMIC));
	if (!xCursor.Is() || !aStatus.IsSuccessful())
		return FALSE;

	if (!xCursor->Open(rSQL))
		aStatus = xCursor->Status();
	else if (!xCursor->IsValid())
	{
		// Anzahl der betroffenen Datensaetze ermitteln
		aStatus.SetRecordsAffected((INT32)xCursor->RowCount());
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
SdbColumns* SdbConnection::DescribeResult(const String& aStatement, BOOL bTable)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();

	// kann ein ParseTree erzeugt werden, da
	// jetzt wird ermittelt, ob Parameter in dem SQL-Statement verwendet werden
	// wenn ja, dann muessen die elemeniert werden, da ansonsten keine Ergebnisbeschreibung
	// erfolgen kann
	SdbSqlParseNode* pParseNode = NULL;
	String aSQL( aStatement ), aErrorMsg;
	BOOL bIndirect = FALSE;
	if (!bTable && (pParseNode = GetSqlParser()->ParseTree(aErrorMsg, aStatement)) != NULL)
	{
		if ( SDB_ISRULE(pParseNode,select_statement) &&
			(pParseNode->FindRule(SdbSqlParseNode::parameter) ||
			 pParseNode->FindRule(SdbSqlParseNode::parameter_ref)))
		{
			String aNewStatement(String::CreateFromAscii("SELECT "));
			SdbSqlParseNode* pNode = pParseNode->GetChild(2);//(SdbSqlParseNode::scalar_exp_commalist)
			if (pNode)
				pNode->ParseNodeToStr(aNewStatement,GetMetaData());
			else
				aNewStatement += '*';
			aNewStatement += ' ';
			pNode = pParseNode->GetChild(3)->GetChild(0);//FindRule(SdbSqlParseNode::from_clause)
			if (pNode)
			{
				pNode->ParseNodeToStr(aNewStatement,GetMetaData());
				aSQL = aNewStatement;
			}else
				aStatus.SetInvalidStatement();
		}
		delete pParseNode;
		bIndirect = TRUE;
	}

	aStatus.Clear();
	SdbCursorRef xCursor( CreateCursor(SDB_DYNAMIC, bIndirect ? SDB_READONLY | SDB_FORWARDONLY | SDB_INDIRECT : SDB_READONLY | SDB_FORWARDONLY) );

	if (!xCursor.Is() || !aStatus.IsSuccessful())
		return NULL;

	BeginHdl().Call(this);

	if (!xCursor->Prepare(aSQL, bTable))
		aStatus = xCursor->Status();
	else
	{
		// kann Prepare eine Spaltenbeschreibung liefern?
		if (xCursor->GetColumns()->Count() == 1)
			xCursor->DescribeResult();
		if (xCursor->GetColumns()->Count() == 1)
		{
			xCursor->Open();
			aStatus.Clear();
		}
		else
			aStatus = xCursor->Status();
	}

	EndHdl().Call(this);
	return aStatus.IsSuccessful() ? CreateColumns(*xCursor->GetColumns()) : NULL;
}

//------------------------------------------------------------------
BOOL SdbConnection::BeginTrans()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_BOOL();

	DBG_ASSERT(!nTransCount,"BeginTrans wurde zweimal hintereinander aufgerufen: FEHLER!");
	nTransCount++;

	BOOL bTransact = GetMetaData()->isCapable(::stardata::DatabaseCapability::TRANSACTIONS);

	if (bTransact && bAutoCommit)
		SetAutoCommit(FALSE);

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbConnection::CommitTrans()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_BOOL();
	nTransCount--;

	BOOL bTransact = GetMetaData()->isCapable(::stardata::DatabaseCapability::TRANSACTIONS);

	if (bTransact )
		CommitImpl();
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbConnection::RollbackTrans()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_BOOL();
	nTransCount--;


	BOOL bTransact = GetMetaData()->isCapable(::stardata::DatabaseCapability::TRANSACTIONS);

	if (bTransact )
		RollbackImpl();
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
String SdbConnection::IdentifierQuotation(const String& rValue)	const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if (cIdentifierQuotation)
	{
		DBG_ASSERT(cIdentifierQuotation == '\"' ||
				   cIdentifierQuotation == '\'' ||
				   cIdentifierQuotation == '`',
				   "Falsches Zeichen fuer Quoten, ist dem Scanner nicht bekannt!");
		return SdbTools::SetQuotation(rValue,cIdentifierQuotation,cIdentifierQuotation);
	}
	else
		return rValue;
}

//------------------------------------------------------------------
String SdbConnection::StringQuotation(const String& rValue)
{
	return SdbTools::SetQuotation(rValue,String::CreateFromAscii("\'"),String::CreateFromAscii("\'\'"));
}

//------------------------------------------------------------------
BOOL SdbConnection::IsTypeAvailable(SdbDatabaseType eType) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if (eType == SDB_DBTYPE_BOOKMARK)
		return TRUE;

	ULONG nCount = aTypeList.Count(),
		  i = 0;

	while(i < nCount)
	{
		SdbTypeInfo* pInfo = aTypeList.GetObject(i++);
		if (pInfo->eType == eType)
			return TRUE;
		else if (pInfo->bCurrency && eType == SDB_DBTYPE_CURRENCY)
			return TRUE;
	}

	return FALSE;
}

//------------------------------------------------------------------
SdbTable* SdbConnection::CreateTable(const String& _rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();

	if (aTableList.Lookup(SdbTable::GetFullName(*this, _rName)))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,dbTable,SdbTable::GetFullName(*this, _rName));
		return NULL;
	}

	SdbTable* pTable = CreateTableImpl();
	if (!pTable)
		return NULL;
	else if (!pTable->Create(_rName))
	{
		aStatus = pTable->Status();
		pTable->RemoveFromParent();
		pTable = NULL;
	}
	else
		aTableList.InsertAt(pTable->GetFullName(), pTable);

	return pTable;
}

//------------------------------------------------------------------
// funktioniert annaehernd wieCreateTable
SdbView* SdbConnection::CreateView(const String& _rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();

	if (aTableList.Lookup(SdbTable::GetFullName(*this, _rName)))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,dbView,SdbTable::GetFullName(*this, _rName));
		return NULL;
	}

	SdbView* pTable = CreateViewImpl();
	if (!pTable)
		return NULL;
	else if (!pTable->Create(_rName))
	{
		aStatus = pTable->Status();
		pTable->RemoveFromParent();
		pTable = NULL;
	}
	else
		aTableList.InsertAt(pTable->GetFullName(), pTable);

	return pTable;
}
//------------------------------------------------------------------
SdbView* SdbConnection::CreateViewImpl()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return new SdbView(this);
}
//------------------------------------------------------------------
SdbTable* SdbConnection::OpenTable(const String& _rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();

	SdbTable* pTable = aTableList.Lookup(SdbTable::GetFullName(*this, _rName));
	BOOL bInList = pTable != NULL;
	if (!bInList)
		pTable = CreateTableImpl();

	if (!pTable)
		return NULL;
	else if (!bInList && !pTable->Open(_rName))
	{
		aStatus = pTable->Status();
		pTable->RemoveFromParent();
		pTable = NULL;
	}
	else if (!bInList)
	{
		if (pTable->IsView())				// ist das eine view ?
		{
			SdbView* pTemp = CreateViewImpl();
			pTemp->SetWrappedTable(pTable);	// View wird ueber die Tabelle gestuelp
			pTable = pTemp;
		}
		aTableList.InsertAt(pTable->GetFullName(), pTable);
	}
	return pTable;
}

//------------------------------------------------------------------
SdbCursor* SdbConnection::CreateCursor(SdbCursorType _eType,
									   UINT16 _nOptions)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();

	SdbCursor* pCursor = CreateCursorImpl(_eType, _nOptions);
	if (pCursor)
		aCursorList.Insert(pCursor);
	return pCursor;
}

//------------------------------------------------------------------
// erzeugen eines Spalten arrays
SdbColumns* SdbConnection::CreateColumns()
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();
	return new SdbColumns(*this);
}

//------------------------------------------------------------------
// erzeugen eines Spalten arrays
SdbColumns* SdbConnection::CreateColumns(const SdbColumns& _rCols)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();
	return new SdbColumns(*this, _rCols);
}

//------------------------------------------------------------------
SdbColumn* SdbConnection::CreateColumn(const String& _rName, SdbDatabaseType _eType,
									   UINT32 _nLength, UINT16 _nScale, UINT16 _nFlags)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();
	if (eIdentifierCase == SDB_IC_ALL)
		_nFlags |= SDB_FLAGS_CASESENSITIVE;

	// Formatkey anhand des Typs bzw des Scales bestimmen
	UINT32 nKey = SdbTools::DefaultFormat(*GetEnvironment()->GetStandardFormatter(), _eType, _nScale);
	return new SdbColumn(_rName, _eType, _nLength, _nScale, _nFlags, nKey);
}

//------------------------------------------------------------------
SdbColumn* SdbConnection::CreateColumn(const SdbColumn& _rCol)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_NULL();
	return _rCol.Clone();
}

//------------------------------------------------------------------------------
String SdbConnection::QuoteTableName(const String& rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_STRING();

	String aQuotedName;
	String aTableName(rName);
	char cSeparator = cQualifierSeparator;

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(GetMetaData());

	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	if (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML)
	{
		if (aTableName.GetTokenCount(cSeparator) >= 2)
		{
			String aDatabaseName(aTableName.GetToken(0,cSeparator));
			aQuotedName += IdentifierQuotation(aDatabaseName);
			aQuotedName += cSeparator;
			aTableName.Erase(0, aDatabaseName.Len() + 1);
		}
	}
	if (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML)
	{
		if (aTableName.GetTokenCount('.') == 2)
		{
			aQuotedName += IdentifierQuotation(aTableName.GetToken(0,'.'));
			aQuotedName += '.';
			aQuotedName += IdentifierQuotation(aTableName.GetToken(1,'.'));
		}
		else aQuotedName += IdentifierQuotation(aTableName);

	}
	else
		aQuotedName += IdentifierQuotation(aTableName);

	return aQuotedName;
}

//------------------------------------------------------------------------------
String SdbConnection::CompleteTableName(const String& rName)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	SDB_CHK_OPEN_STRING();
	return SdbTable::GetFullName(*this, rName);
}

//------------------------------------------------------------------------------
BOOL SdbConnection::IsSQLNameValid(const String& rName) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	// berprfung auf korrekte Namensgebung im SQL Sinne
	// Dieses ist wichtig fr Tabellennamen beispielsweise
	ByteString aName(rName,gsl_getSystemTextEncoding());
	const char* pStr = aName.GetBuffer();
	if (isdigit(*pStr))
		return FALSE;

	for (; *pStr; ++pStr )
		if(!IsCharOk(*pStr))
			return FALSE;

	if	(	rName.Len()
		&&	(	(rName.GetChar(0) == '_')
			||	(	(rName.GetChar(0) >= '0')
				&&	(rName.GetChar(0) <= '9')
				)
			)
		)
		return FALSE;
	// the SQL-Standard requires the first character to be an alphabetic character, which
	// isn't easy to decide in UniCode ...
	// So we just prohibit the characters which already lead to problems ....
	// 11.04.00 - 74902 - FS

	return TRUE;
}
//------------------------------------------------------------------
BOOL SdbConnection::IsCharOk(char c) const
{

	DBG_CHKTHIS(SdbConnection,NULL);
	if ( ((c >= 97) && (c <= 122)) || ((c >= 65) && (c <=  90)) || ((c >= 48) && (c <=  57)) ||
		  c == '_'	  			   || aSpecialCharStr.Search(c) != STRING_NOTFOUND )
		  return TRUE;
	else
		return FALSE;
}
//------------------------------------------------------------------
// Erzeugt einen neuen Namen falls noetig
String SdbConnection::ConvertName(const String& rName) const
{
	DBG_CHKTHIS(SdbConnection,NULL);
	if(IsSQLNameValid(rName))
		return rName;
	String aNewName(rName);
	const sal_Unicode* pStr = rName.GetBuffer();
	BOOL bValid(!isdigit(*pStr));
	for (; bValid && *pStr; pStr++ )
		if(!IsCharOk(*pStr))
			aNewName.SearchAndReplace(*pStr,'_');

	return aNewName;
}
//------------------------------------------------------------------
const SdbTypeInfo* SdbConnection::GetColumnTypeInfo(const SdbColumn& rColumn)
{
	DBG_CHKTHIS(SdbConnection,NULL);

	ULONG i(0);
	const SdbTypeInfo* pInfo = aTypeList.GetObject(i++);
	const SdbTypeInfo* pPrevInfo = NULL;

	// Suchen ob entsprechende Typinformationen vorhanden sind
	while (pInfo)
	{
		// Ist der Typ vorhanden
		if (pInfo->eType == rColumn.GetType())
		{
			if( (!pInfo->nPrecision || pInfo->nPrecision >= rColumn.GetPrecision())	&&
				(!pInfo->nScale || pInfo->nScale >= rColumn.GetScale()) &&
				(pInfo->bNullable || !rColumn.IsNullAllowed()) &&
				((rColumn.GetAllFlags() & SDB_FLAGS_VERSIONING)==SDB_FLAGS_VERSIONING) == pInfo->bRowVersion)
			{
				if(pInfo->bAutoIncrement == rColumn.IsAutoIncrement())
					break;
				else if(pInfo->bAutoIncrement)
					pPrevInfo = pInfo;
			}
		}
		else if (rColumn.GetType() == SDB_DBTYPE_CURRENCY && pInfo->bCurrency && // Sonderfall fuer Waehrung
			(pInfo->bNullable || !rColumn.IsNullAllowed()))
			break;

		pInfo = aTypeList.GetObject(i++);
	}

	return pInfo ? pInfo : pPrevInfo;
}

//------------------------------------------------------------------
const SdbTypeInfo* SdbConnection::GetColumnTypeInfo(const String& rTypeName)
{
	DBG_CHKTHIS(SdbConnection,NULL);

	ULONG i(0);
	const SdbTypeInfo* pInfo = aTypeList.GetObject(i++);
	BOOL bCase = eIdentifierCase == SDB_IC_ALL;

	// Suchen ob entsprechende Typinformationen vorhanden sind
	while (pInfo)
	{
		// Ist der Typ vorhanden
		if (bCase ? pInfo->aTypeName == rTypeName : pInfo->aTypeName.EqualsIgnoreCaseAscii(rTypeName))
			break;

		pInfo = aTypeList.GetObject(i++);
	}

	return pInfo;
}

//------------------------------------------------------------------
const SdbTypeInfo* SdbConnection::GetColumnTypeInfo(const SdbDatabaseType eType)
{
	DBG_CHKTHIS(SdbConnection,NULL);

	ULONG i(0);
	const SdbTypeInfo* pInfo = aTypeList.GetObject(i++);

	// Suchen ob entsprechende Typinformationen vorhanden sind
	while (pInfo)
	{
		// Ist der Typ vorhanden
		if (pInfo->eType == eType)
			break;

		pInfo = aTypeList.GetObject(i++);
	}

	return pInfo;
}
//------------------------------------------------------------------
const SdbTypeInfo* SdbConnection::GetColumnTypeInfo(const String& rTypeName,INT32 nLength,UINT8 nScale,BOOL bNullAllowed,INT32 nJavaType)
{
	DBG_CHKTHIS(SdbConnection,NULL);

	ULONG i(0);
	const SdbTypeInfo* pInfo = aTypeList.GetObject(i++);
	BOOL bCase = eIdentifierCase == SDB_IC_ALL;

	// Suchen ob entsprechende Typinformationen vorhanden sind
	while (pInfo)
	{
		// Ist der Typ vorhanden

		if( (INT8)nJavaType == (INT8)pInfo->nMinimumScale	&&
			pInfo->nPrecision	>= nLength					&&
			pInfo->nScale >= nScale							&&
			(bNullAllowed ? pInfo->bNullable : TRUE)		&&
			(bCase ? pInfo->aTypeName == rTypeName : pInfo->aTypeName.EqualsIgnoreCaseAscii(rTypeName)))
			break;
		pInfo = aTypeList.GetObject(i++);
	}

	return pInfo;
}
//------------------------------------------------------------------
BOOL SdbConnection::DoCmdImpl(UINT16 nLimit,const String &rArgs)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return FALSE;
}
//------------------------------------------------------------------
BOOL SdbConnection::DoCmd(UINT16 nCmd,const String &rArgs)
{
	DBG_CHKTHIS(SdbConnection,NULL);
	return DoCmdImpl(nCmd,rArgs);
}

//------------------------------------------------------------------
BOOL SdbConnection::StructureIsReadOnly()
{
	return FALSE;
}

//------------------------------------------------------------------
BOOL SdbConnection::QualifiedNameComponents(const String& rQualifiedName, String& rCatalog, String& rSchema, String& rName) const
{
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(((SdbConnection*)this)->GetMetaData());
	char cSeparator = GetQualifierSeparator();

	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	String sName(rQualifiedName);
	// sollte das Ding einen Qualifier haben ?
	if (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML)
	{
		sal_Bool bCatalogAtStart;
		rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_AT_START) >>= bCatalogAtStart;
		if (bCatalogAtStart)
		{
			// Katalogname am Anfang
			xub_StrLen nIndex = sName.Search(cSeparator);
			if (nIndex != STRING_NOTFOUND)
			{
				rCatalog = sName.Copy(0, nIndex);
				sName.Erase(0, nIndex + 1);
			}
		}
		else
		{
			// Katalogname am Ende
			xub_StrLen nIndex = sName.SearchBackward(cSeparator);
			if (nIndex != STRING_NOTFOUND)
			{
				rCatalog = sName.Copy(nIndex + 1);
				sName.Erase(nIndex);
			}
		}
	}

	if ((nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML) && sName.GetTokenCount('.') == 2)
	{
		rSchema = sName.GetToken(0,'.');
		sName = sName.GetToken(1,'.');
	}

	rName = sName;
	return TRUE;
}
//------------------------------------------------------------------
BOOL SdbConnection::IsView(const String& rTableName) const
{
	return !(*pTableNames)[rTableName];
}

