/*************************************************************************
 *
 *  $RCSfile: sdbcont.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbcont.hxx"

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#ifndef _REF_HXX //autogen
#include <tools/ref.hxx>
#endif

//------------------------------------------------------------------
BOOL SdbKeyContainer::InsertAt(const String& _rKey, void* _pObj, ULONG _nIndex)
{
	DBG_ASSERTWARNING(GetPos(_pObj) == SDB_CONT_ENTRY_NOTFOUND && Lookup(_rKey) == NULL,
		"SdbKeyContainer::Insert -> Object already in List");

	SdbContainer::Insert(_pObj, _nIndex);
	aMap.SetAt(_rKey, _pObj);

	return _pObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyContainer::ReplaceAt(const String& _rOldKey,
								 const String& _rKey,
								 void* _pObj)
{
	void* pOldObj = Lookup(_rOldKey);
	DBG_ASSERT(pOldObj, "SdbKeyContainer::Replace -> Objekt not found");

	ULONG nIndex  = SdbContainer::GetPos(pOldObj);
	if (Remove(pOldObj))
		InsertAt(_rKey, _pObj, nIndex);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyContainer::ReplaceAt(ULONG _nIndex, const String& _rKey, void* _pObj)
{
	BOOL bResult = RemoveAt((ULONG)_nIndex);
	if (bResult)
		InsertAt(_rKey, _pObj, _nIndex);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbKeyContainer::Replace(void* _pOldObj, const String& _rKey, void* _pObj)
{
	ULONG nIndex  = SdbContainer::GetPos((void*)_pOldObj);
	BOOL bResult = RemoveAt(nIndex);
	if (bResult)
		InsertAt(_rKey, _pObj, nIndex);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbKeyContainer::RemoveAt(const String& _rKey)
{
	void* pOldObj = aMap.RemoveAt(_rKey);
	DBG_ASSERT(pOldObj, "SdbKeyContainer::RemoveAt -> Objekt not found");
	if (pOldObj)
		SdbContainer::Remove(pOldObj);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyContainer::RemoveAt(ULONG _nIndex)
{
	void* pOldObj = SdbContainer::Remove((ULONG)_nIndex);
	DBG_ASSERT(pOldObj, "SdbKeyContainer::RemoveAt -> Objekt not found");
	if (pOldObj)
		aMap.Remove(pOldObj);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyContainer::Remove(void* _pObj)
{
	void* pOldObj = SdbContainer::Remove((void*)_pObj);
	DBG_ASSERT(pOldObj, "SdbKeyContainer::Remove -> Objekt not found");
	if (pOldObj)
		aMap.Remove(pOldObj);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
void* SdbKeyContainer::Find(const String& _rKey) const
{
	return aMap.Lookup(_rKey);
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::InsertAt(const String& _rKey, SvRefBase* _pObj, ULONG _nIndex)
{
	DBG_ASSERT(GetPos(_pObj) == SDB_CONT_ENTRY_NOTFOUND && Lookup(_rKey) == NULL,
		"SdbKeyRefContainer::Insert -> Object already in List");

	// Speichert immer die Objekte
	SdbContainer::Insert(_pObj, _nIndex);
	aMap.SetAt(_rKey, _pObj);

	return _pObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::ReplaceAt(const String& _rOldKey,
								 const String& _rKey,
								 SvRefBase* _pObj)
{
	SvRefBaseRef xOldObj = Lookup(_rOldKey);
	DBG_ASSERT(xOldObj.Is(), "SdbKeyRefContainer::Replace -> Objekt not found");

	ULONG nIndex  = SdbContainer::GetPos(&xOldObj);

	if (RemoveAt(_rOldKey))
		InsertAt(_rKey, _pObj, nIndex);
	return xOldObj.Is();
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::ReplaceAt(ULONG _nIndex, const String& _rKey, SvRefBase* _pObj)
{
	SvRefBaseRef xOldObj = (SvRefBase*)GetObject(_nIndex);
	BOOL bResult  = RemoveAt((ULONG)_nIndex);

	if (bResult)
		InsertAt(_rKey, _pObj, _nIndex);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::Replace(SvRefBase* _pOldObj, const String& _rKey, SvRefBase* _pObj)
{
	ULONG nIndex  = SdbContainer::GetPos((void*)_pOldObj);
	SvRefBaseRef xOldObj = _pOldObj;	// festhalten
	BOOL bResult  = RemoveAt(nIndex);

	if (bResult)
		InsertAt(_rKey, _pObj, nIndex);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::RemoveAt(const String& _rKey)
{
	// erst aus dem Container entfernen, sonst kann das Objekt sonst zerstrt sein
	SvRefBase* pOldObj = Lookup(_rKey);
	DBG_ASSERT(pOldObj, "SdbKeyRefContainer::RemoveAt -> Objekt not found");
	if (pOldObj)
	{
		SdbContainer::Remove(pOldObj);
		aMap.Remove(pOldObj);
	}
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::RemoveAt(ULONG _nIndex)
{
	SvRefBase* pOldObj = (SvRefBase*)SdbContainer::Remove((ULONG)_nIndex);
	DBG_ASSERT(pOldObj, "SdbKeyRefContainer::RemoveAt -> Objekt not found");
	if (pOldObj)
		aMap.Remove(pOldObj);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbKeyRefContainer::Remove(SvRefBase* _pObj)
{
	SvRefBase* pOldObj = (SvRefBase*)SdbContainer::Remove((void*)_pObj);
	DBG_ASSERT(pOldObj, "SdbKeyRefContainer::Remove -> Objekt not found");
	if (pOldObj)
		aMap.Remove(pOldObj);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
SvRefBase* SdbKeyRefContainer::Find(const String& _rKey) const
{
	return (SvRefBase*)aMap.Lookup(_rKey);
}

//------------------------------------------------------------------
SdbFileContainer::~SdbFileContainer()
{
	ULONG nCount = aList.Count();
	for (ULONG i = 0; i < nCount; i++)
	{
		delete aList.GetObject(i);
	}
	aList.Clear();
}

//------------------------------------------------------------------
void SdbFileContainer::Clear()
{
	SdbContainer::Clear();

	ULONG nCount = aList.Count();
	for (ULONG i = 0; i < nCount; i++)
	{
		delete aList.GetObject(i);
	}
	aList.Clear();
}

//------------------------------------------------------------------
BOOL SdbFileContainer::InsertAt(const DirEntry& _rKey, void* _pObj, ULONG _nIndex)
{
	DBG_ASSERT(GetPos(_pObj) == SDB_CONT_ENTRY_NOTFOUND && Lookup(_rKey) == NULL,
		"SdbFileContainer::Insert -> Object already in List");

	SdbContainer::Insert(_pObj, _nIndex);
	aList.Insert(new DirEntry(_rKey), _nIndex);

	return _pObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbFileContainer::ReplaceAt(const DirEntry& _rOldKey,
								  const DirEntry& _rKey,
								  void* _pObj)
{
	void* pOldObj = Lookup(_rOldKey);
	DBG_ASSERT(pOldObj, "SdbFileContainer::Replace -> Objekt not found");

	ULONG nIndex = SdbContainer::GetPos(pOldObj);
	BOOL bResult = RemoveAt(_rOldKey);

	if (bResult)
		InsertAt(_rKey, _pObj, nIndex);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbFileContainer::ReplaceAt(ULONG _nIndex, const DirEntry& _rKey, void* _pObj)
{
	BOOL bResult = RemoveAt(_nIndex);

	if (bResult)
		InsertAt(_rKey, _pObj, _nIndex);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbFileContainer::Replace(void* _pOldObj, const DirEntry& _rKey, void* _pObj)
{
	ULONG nIndex  = SdbContainer::GetPos((void*)_pOldObj);
	BOOL bResult = RemoveAt(nIndex);

	if (bResult)
		InsertAt(_rKey, _pObj, nIndex);
	return bResult;
}

//------------------------------------------------------------------
DirEntry SdbFileContainer::GetKey(void* _pObj) const
{
	ULONG nIndex = GetPos(_pObj);
	return nIndex != SDB_CONT_ENTRY_NOTFOUND ? *aList.GetObject(nIndex) : DirEntry();
}

//------------------------------------------------------------------
void* SdbFileContainer::Lookup(const DirEntry& _rKey) const
{
	ULONG nCount = aList.Count();
	ULONG i;
	for (i = 0; i < nCount; i++)
	{
		DirEntry* pEntry = aList.GetObject(i);
		if (*pEntry == _rKey)
			break;
	}

	return i < nCount ? GetObject(i) : NULL;
}

//------------------------------------------------------------------
BOOL SdbFileContainer::RemoveAt(const DirEntry& _rKey)
{
	ULONG nCount = aList.Count();
	ULONG i;
	for (i = 0; i < nCount; i++)
	{
		DirEntry* pEntry = aList.GetObject(i);
		if (*pEntry == _rKey)
			break;
	}

	DBG_ASSERT(i < nCount, "SdbFileContainer::RemoveAt -> Objekt not found");
	if (i >= nCount)
		return NULL;

	void* pOldObj = SdbContainer::Remove(i);

	DBG_ASSERT(pOldObj, "SdbFileContainer::RemoveAt -> Objekt not found");

	if (pOldObj)
		delete aList.Remove(i);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbFileContainer::RemoveAt(ULONG _nIndex)
{
	void* pOldObj = SdbContainer::Remove((ULONG)_nIndex);
	DBG_ASSERT(pOldObj, "SdbFileContainer::RemoveAt -> Objekt not found");

	if (pOldObj)
		delete aList.Remove(_nIndex);
	return pOldObj != NULL;
}

//------------------------------------------------------------------
BOOL SdbFileContainer::Remove(void* _pObj)
{
	ULONG nIndex  = SdbContainer::GetPos((void*)_pObj);
	void* pOldObj = SdbContainer::Remove((void*)_pObj);
	DBG_ASSERT(pOldObj, "SdbFileContainer::Remove -> Objekt not found");

	if (pOldObj)
		delete aList.Remove(nIndex);
	return pOldObj != NULL;
}


