/*************************************************************************
 *
 *  $RCSfile: sdbcurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <limits.h>

#ifndef _SDB_SDBCURS_HXX
#include <sdbcurs.hxx>
#endif

#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifndef _SDB_SDBSET_HXX
#include <sdbset.hxx>
#endif

#ifndef _SDB_ROWCACHE_HXX
#include "rowcache.hxx"
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

#ifndef _SDB_SDBRESID_HRC
#include <sdbresid.hrc>
#endif

#ifndef _SDB_SDBTAB_HXX
#include <sdbtab.hxx>
#endif

#ifndef _SDB_SDBSQLIT_HXX //autogen
#include "sdbsqlit.hxx"
#endif

namespace staruno = com::sun::star::uno;
namespace stardata = com::sun::star::data;

DBG_NAME(SdbCursor);

TYPEINIT1(SdbCursor, SdbObj);

//------------------------------------------------------------------
SdbCursor::SdbCursor(SdbConnection* pConnection,
					 SdbCursorType eType,
					 UINT16 nOpt)
		  :SdbObj(pConnection, FALSE)
		  ,pSQLParseNode(NULL)
		  ,nRowCount(0)
		  ,bPrepared(FALSE)
		  ,bBOF(TRUE)
		  ,bEOF(TRUE)
		  ,eMode(SDB_MOD_INVALID)
		  ,eOriginalMode(SDB_MOD_INVALID)
		  ,pCache(NULL)
		  ,nCacheSize(0)
		  ,nOptions(nOpt)
		  ,eCursorType(eType)
		  ,nPrivileges(SDB_PR_READ)
		  ,bRowCountFinal(FALSE)
		  ,nCurrentPos(SDB_CURRENT_ROW_UNDEFINED)		  
		  ,pKeySet(NULL)
		  ,bUseCursorSet(FALSE)
		  ,pParseNodeList(NULL)
		  ,bBookmarkable(TRUE)
		  ,bBulkOptimize(FALSE)
		  ,pParseIterator(NULL)
		  ,bThreadSafe(TRUE)
		  ,nMaxFieldLen(0)
		  ,nMaxRows(0)
		  ,bIsCancelable(FALSE)
{
	DBG_CTOR(SdbCursor,NULL);
	pLock = new NAMESPACE_VOS(OMutex);

	// Feststellen ob der Cursor updatebar
	// Er ist updatebar, wenn
	// a.) nicht ReadOnly,
	// b.) der Treiber updatefaehig
	if ((nOptions & SDB_READONLY) == SDB_READONLY)	// append und readonly widersprechen sich
		nOptions &= ~SDB_APPENDONLY;

	aResultColumns = pConnection->CreateColumns();
	aParameterColumns = pConnection->CreateColumns();
}

//------------------------------------------------------------------
SdbCursor::~SdbCursor()
{
	DBG_DTOR(SdbCursor,NULL);

	delete pLock;
	delete pSQLParseNode;
	delete pCache;
	delete pKeySet;
	delete pParseNodeList;
	delete pParseIterator;

	aResultColumns.unbind();
	aParameterColumns.unbind();
	aCurrentRow.unbind();
	aResultRow = aOriginalRow = aParameterRow = aCurrentRow;
}

//------------------------------------------------------------------
void SdbCursor::Close()
{
	DBG_CHKTHIS(SdbCursor,NULL);

	NAMESPACE_VOS(OGuard) aGuard(pLock);

	if (pKeySet)
		pKeySet->prepareDelete();

	DELETEZ(pKeySet);

	CloseImpl();		// Configurierbar

	DELETEZ(pCache);
	DELETEZ(pSQLParseNode);
	DELETEZ(pParseNodeList);

	aCurrentRow.unbind();
	aResultRow = aOriginalRow = aParameterRow = aCurrentRow;

	// Strukturbeschreibung freigeben
	SdbConnection* pConn = GetConnection();
	aResultColumns = pConn && pConn->IsOpen() ? pConn->CreateColumns() : NULL;
	aParameterColumns = pConn && pConn->IsOpen() ? pConn->CreateColumns() : NULL;

	aSQLStatement  = aEmptyString;

	nRowCount = 0;
	nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
	bRowCountFinal = FALSE;
	bOpen 		= FALSE;
	bPrepared 	= FALSE;
	bEOF = bBOF = TRUE;
	bBulkOptimize = FALSE;
	bBookmarkable = TRUE;
	nPrivileges = SDB_PR_READ;
	eMode		= eOriginalMode = SDB_MOD_INVALID;

	DELETEZ(pParseIterator);
	aStatus.Clear();
}

//------------------------------------------------------------------
void SdbCursor::Cancel()
{
#if SUPD>531 && SUPD<542
	NAMESPACE_VOS(OGuard) aGuard(m_aCancelAccess);
	if (!m_nExecutingCancelable)
		return;

	CancelExecution();

	m_bExecutionCanceled = TRUE;
		// wird von WasCanceled ausgewertet
#else
	SdbCancelable::Cancel();
	if (ExecutingCancelable())
	{
		CancelExecution();
		if (pKeySet && pKeySet->ExecutingCancelable())
			pKeySet->Cancel();
	}
#endif
}

//-----------------------------------------------------------------------------
BOOL SdbCursor::WasCanceled(BOOL bOnlyIfExecuting)
{
	NAMESPACE_VOS(OGuard) aGuard(m_aCancelAccess);
#if SUPD>531 && SUPD<542
	if (bOnlyIfExecuting && !m_nExecutingCancelable)
		return FALSE;
	return m_bExecutionCanceled;
#else
	return SdbCancelable::WasCanceled(bOnlyIfExecuting) || (pKeySet && pKeySet->WasCanceled(bOnlyIfExecuting));
#endif
}

#if SUPD>531 && SUPD<542
//-----------------------------------------------------------------------------
void SdbCursor::BeginCancelable()
{
	NAMESPACE_VOS(OGuard) aGuard(m_aCancelAccess);
	m_bExecutionCanceled = FALSE;
	++m_nExecutingCancelable;
}
#endif

//-----------------------------------------------------------------------------
BOOL SdbCursor::EndCancelable()
{
	NAMESPACE_VOS(OGuard) aGuard(m_aCancelAccess);
#if SUPD>531 && SUPD<542
	--m_nExecutingCancelable;
	if (m_bExecutionCanceled)
#else
	if (SdbCancelable::EndCancelable())
#endif
	{
		aStatus.Set(SDB_STAT_CANCELED);
		return TRUE;
	}

	return FALSE;
}

//------------------------------------------------------------------
void SdbCursor::CancelExecution()
{
}

//------------------------------------------------------------------
void SdbCursor::SetCacheSize(UINT16 nSize)
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);

	if (nSize <= 2 || (IsInsertOnly_OldAPI()))
		nSize = 0;

	// gibt es was zu tun?
	if (nSize == nCacheSize)
		return;

	if (pCache && IsOpen() && nCurrentPos != SDB_CURRENT_ROW_UNDEFINED)
		pCache->synchronize(TRUE);		// Cursor und Cache abgleichen

	DELETEZ(pCache);

	// wenn nicht geoeffnet dann nur die cachesize merken
	if (!IsOpen())
		nCacheSize = nSize;
	else
	{
		pCache = (nSize) ? new ORowCache(*this, nSize, FALSE)
					 : NULL;

		nCacheSize = (pCache) ? nSize : 0;
	}

	if (pCache && pKeySet)
		pKeySet->startCounting();	// Zaehlen anwerfen falls noch nicht getan
}

//------------------------------------------------------------------
BOOL SdbCursor::Prepare(const String& rSQL, BOOL bTable)
{
	DBG_CHKTHIS(SdbCursor,NULL);
	SDB_CHK_CONNECT_BOOL();

	if (bTable)								// wenn eine Tabelle uebergeben wird
	{										// dann wird das SQL-Statement erzeugt
		aSQLStatement.AssignAscii("Select * FROM ");
		aSQLStatement += pConnection->QuoteTableName(rSQL);
		if ((nOptions & SDB_APPENDONLY) == SDB_APPENDONLY)	// Append benoetigt kein ResultSet,
			aSQLStatement.AppendAscii(" WHERE 0 = 1 ");				// nicht sehr schoen
	}
	else
		aSQLStatement = rSQL;

	return Prepare();
}

//------------------------------------------------------------------
BOOL SdbCursor::Prepare()
{
	DBG_CHKTHIS(SdbCursor,NULL);

	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	SDB_CHK_CONNECT_BOOL();
	BEGIN_DBACTION();
	delete pSQLParseNode;
	pSQLParseNode = NULL;

	bPrepared = PrepareImpl();

	// anlegen der ParameterZeile
	if (bPrepared && !aParameterRow.is() && aParameterColumns.Is())
		aParameterRow = new ODbRow(*aParameterColumns);

	END_DBACTION();
	return bPrepared;
}


//------------------------------------------------------------------
BOOL SdbCursor::Open(const String& rSQL, BOOL bTable,SdbCursorType eType,UINT16 eOpt)
{
	DBG_CHKTHIS(SdbCursor,NULL);
	NAMESPACE_VOS(OGuard) aGuard(pLock);

	SDB_CHK_CONNECT_BOOL();

	// Execute nicht moeglich wenn Cursor schon geoeffnet oder
	// wenn Execute als Parameter ein anderes Statement erhaelt als
	// das vorangegangene Prepare.
	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	if(eType != SDB_CURRENT)
		eCursorType = eType;
	if(eOpt != SDB_CURRENT_OPT)
	{
		nOptions = eOpt;
		// Feststellen ob der Cursor updatebar
		// Er ist updatebar, wenn
		// a.) nicht ReadOnly,
		// b.) der Treiber updatefaehig
		if ((nOptions & SDB_READONLY) == SDB_READONLY)	// append und readonly widersprechen sich
			nOptions &= ~SDB_APPENDONLY;
	}

	if (bTable)		// wenn eine Tabelle uebergeben wird
	{				// dann wird das SQL-Statement erzeugt
		aSQLStatement.AssignAscii("Select * FROM ");
		aSQLStatement += pConnection->QuoteTableName(rSQL);		
		if ((nOptions & SDB_APPENDONLY))	// Append benoetigt kein ResultSet,
			aSQLStatement.AppendAscii(" WHERE 0 = 1 ");				// nicht sehr schoen

		// Optimierung fuer Append
		// not possible for the new api
		if (IsInsertOnly_OldAPI() && pConnection->IsCapable(SDB_CAP_BULKOPTIMIZE))
		{
			SdbTableRef xTable = pConnection->OpenTable(rSQL);
			// damit Platzhalter treiberspez. ersetzt werden koennen
			nOptions |= SDB_INDIRECT;
			if (!xTable.Is())
			{
				aStatus = pConnection->Status();
			}
			else if (!(xTable->GetTablePrivileges() & SDB_PR_INSERT))
			{
				// gar nicht weiter versuchen
				aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S1C00"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_CURSOR_NOT_UPDATABLE))),
						0, String() );
			}
			else
			{
				// Oeffnen fuer die Resultbeschreibung
				if (OpenImpl(eType,nOptions) && OnOpen())
				{
					SdbColumnsRef xCols = aResultColumns;
					Close();

					aResultColumns = xCols;
					aSQLStatement	  = BuildBulkSQL(rSQL);
					if (PrepareImpl())
					{
						nPrivileges = SDB_PR_INSERT;
						bBulkOptimize = TRUE;
					}
				}
			}
		}
	}
	else
	{
		aSQLStatement = rSQL;
	}

	BOOL bResult = aStatus.IsSuccessful() && Open(eType,nOptions);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbCursor::Open(SdbCursorType eType, UINT16 eOpt)
{
	if (eType == SDB_CURRENT || eType == eCursorType)
		eType = SDB_CURRENT;
	else
		eCursorType = eType;

	if (eOpt == SDB_CURRENT_OPT || eOpt == nOptions)
		eOpt = SDB_CURRENT_OPT;
	else
	{
		DBG_ASSERT(!bBulkOptimize, "SdbCursor::Open(SdbCursorType eType,UINT16 eOpt), Invalid option");
		nOptions = eOpt;
		// Feststellen ob der Cursor updatebar
		// Er ist updatebar, wenn
		// a.) nicht ReadOnly,
		// b.) der Treiber updatefaehig
		if ((nOptions & SDB_READONLY) == SDB_READONLY)	// append und readonly widersprechen sich
			nOptions &= ~SDB_APPENDONLY;
	}

	if (!bBulkOptimize && nOptions & SDB_APPENDONLY) // Ergebnismenge minimieren fuer Appendonly Cursor
		nMaxRows = 1;

	return Open(TRUE, eType, eOpt);
}

//------------------------------------------------------------------
BOOL SdbCursor::Open(BOOL bBusyHandling,SdbCursorType eType,UINT16 eOpt)
{
	// dient an sich nur dazu, busyhandling an oder abzuschalten

	SDB_CHK_CONNECT_BOOL();
	// Cursor die Parameter verarbeiten, drfen mehrmals ausgefhrt
	// werden, dieses ermglicht einen Parameterwechsel
	if (!IsPrepared() && IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	aStatus.Clear();

	if (bBusyHandling)
	{
		BEGIN_DBACTION();
		BUSY_DB();
	}

	// Werden bereits hier gesetzt, da sie von den Treibern gesetzt werden knnen
	nRowCount	= 0;
	bRowCountFinal = FALSE;

	if (!aResultColumns.Is())
		aResultColumns = pConnection->CreateColumns();

	BOOL bResult;
	// Optimierung fuer Append mit BulkCopy
	if (bBulkOptimize)
	{
		nCurrentPos = 0;
		bBOF		= FALSE;
		bEOF		= TRUE;
		eMode		= SDB_MOD_INVALID;
		bRowCountFinal = TRUE;
		bResult		= bPrepared = bOpen = TRUE;
		nPrivileges = SDB_PR_INSERT;
		InitResultRow();
	}
	else
	{
		nPrivileges = SDB_PR_READ;

		// fuer prepared statements immer ein OnOpen
		// um eventuelle Spaltenbeschreibungen zu erhalten
		// der Status mu allerdings erhalten bleiben
		bResult = OpenImpl(eType,eOpt);
		if (bResult || bPrepared)
		{
			SdbStatus aTempStat(aStatus);

			bOpen = OnOpen();
			if (bOpen)
			{
			   // mit Next auf den ersten Satz, da der Cursor nach dem Open vor dem ersten Satz steht
			   // macht natuerlich nur Sinn, wenn es eine ergebnismenge gibt

			   // Fuer AppendCursor darf normalerweise der Cursor nicht bewegt werden,
			   // jedoch benoetigen wir eine Cursorbewegung, sonst ist der Cursor nicht im gueltigen
			   // Zustand
				if ((nOptions & SDB_API_COMPATIBLE) == SDB_API_COMPATIBLE)
				{
					if (aResultColumns->Count() > 1)
					{
						// don't move to the first row
						// in appendonly discard the first row if there is any
						if (nOptions & SDB_APPENDONLY)
						{
							// eine Positionierung mu vorgenommen werden
							MoveImpl(SDB_POS_NEXT, 1, eMode);							
							nCurrentPos = -1;
							bBOF		= TRUE;
							bEOF		= FALSE;
							eMode		= SDB_MOD_INVALID;
							bRowCountFinal = TRUE;
							nRowCount	= 0;
						}						
					}					
				}
				else
				{
					if (aResultColumns->Count() > 1)
					{	
						// Optimierung fuer Append ohne BulkCopy
						// discard the first row if there is any
						if (nOptions & SDB_APPENDONLY)
						{
							// eine Positionierung mu vorgenommen werden
							MoveImpl(SDB_POS_NEXT, 1, eMode);
							nCurrentPos = 0;
							bBOF		= FALSE;
							bEOF		= TRUE;
							eMode		= SDB_MOD_INVALID;
							bRowCountFinal = TRUE;
						}
						// only if not api go to the first row
						else if ((nOptions & SDB_API_COMPATIBLE) == 0)
							Next();					
					}
					else
					{
						nCurrentPos = 0;
						bBOF		= FALSE;
						bEOF		= TRUE;
						eMode		= SDB_MOD_INVALID;
						bRowCountFinal = TRUE;
					}
				}				
			}
			if (!bResult)  // siehe oben
			{
				bOpen	= FALSE;
				aStatus = aTempStat;
			}
		}
	}
	if (bBusyHandling)
		END_DBACTION();
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbCursor::Open4Info()
{
	SDB_CHK_CONNECT_BOOL();

	// spezielles Open um Informationen aus der Datenbank zu lesen
	nOptions |= SDB_FORWARDONLY|SDB_READONLY|SDB_MINIMUM_DESC;
	// Werden bereits hier gesetzt, da sie von den Treibern gesetzt werden knnen
	nRowCount	= 0;
	bRowCountFinal = FALSE;

	if (!aResultColumns.Is())
		aResultColumns = pConnection->CreateColumns();

	bOpen = OnOpen();
	if (bOpen)
	{
	   // mit Next auf den ersten Satz, da der Cursor nach dem Open vor dem ersten Satz steht
	   // macht natuerlich nur Sinn, wenn es eine ergebnismenge gibt

	   // Fuer AppendCursor darf normalerweise der Cursor nicht bewegt werden,
	   // jedoch benoetigen wir eine Cursorbewegung, sonst ist der Cursor nicht im gueltigen
	   // Zustand
		if (aResultColumns->Count() > 1 && ((nOptions & SDB_API_COMPATIBLE) == 0) )
			Next();
		else
		{
			nCurrentPos = 0;
			bBOF		= FALSE;
			bEOF		= TRUE;
			eMode		= SDB_MOD_INVALID;
			bRowCountFinal = TRUE;
		}
	}
	return bOpen && aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbCursor::OnOpen()
{
	// Stellt den Zustand des Cursors nach dem Oeffnen her:
	// Positionierung auf den ersten Satz
	// Status setzen
	if (aStatus.IsError() || aStatus.IsCanceled())
		return FALSE;

	SDB_CHK_CONNECT_BOOL();

	nCurrentPos = SDB_CURRENT_ROW_BOF;
	bBOF = TRUE;
	bEOF = FALSE;
	eMode		= SDB_MOD_INVALID;
	bOpen		= TRUE;	// First, Next erwarten Status Open

	InitResultRow();

	if (aResultColumns->Count() > 1)
	{
		// cache clearen
		DELETEZ(pCache);
		if (nCacheSize)
			pCache = new ORowCache(*this, nCacheSize, TRUE);

		if(SQLParseNode())	  // nur aufbauen, wenn auch ein ParseNode vorhanden ist
		{
			if(!pParseIterator)
			{
				pParseIterator = new SdbSqlParseTreeIterator(pConnection,SQLParseNode());
				pParseIterator->TraverseAll();
				if(pParseIterator->IteratorStatus().IsSuccessful())
				{
					const SdbColumnsRef rCols = pParseIterator->GetSelectColumns();
					for(ULONG i=1;i<aResultColumns->Count();i++)
					{
						SdbColumn* pResCol	= (SdbColumn*)(*aResultColumns)[i];
						
						SdbColumn* pCol	= (SdbColumn*)rCols->at(i);
						if (pCol)
						{
							pResCol->SetRealName(pCol->GetRealName());

							// Flags anpassen, da die Treiber teilweise dazu nicht in der Lage sind
							if (pCol->IsFunction())
							{
								pResCol->SetFunction(TRUE);
								pResCol->ResetFlag(SDB_FLAGS_UPDATEALLOWED);
							}
							if (pCol->IsNullAllowed())
								pResCol->SetFlag(SDB_FLAGS_NULLALLOWED);
							// this ifnormation as only the column from the table
							if(pCol->GetAllFlags() & SDB_FLAGS_VERSIONING)
								pResCol->SetFlag(SDB_FLAGS_VERSIONING);
							// diese Info hat die ResultColumn vom Treiber-Cursor
							pCol->SetAllFlags(pResCol->GetAllFlags());
						}
					}
				}
				else
					aStatus = pParseIterator->IteratorStatus();
			}

		}
	}

	// brauchen wir ein KeySet ???, wenn
	// a.) es gibt bereits ein Cursorset
	// b.) es wird nicht nur forward positioniert
	// c.) und kein Append_Only
	if (aResultColumns->Count() > 1 &&
		(bUseCursorSet || !(nOptions & SDB_FORWARDONLY)))
	{
		bUseCursorSet = TRUE;
		if (pKeySet)
			pKeySet->prepareDelete();
		DELETEZ(pKeySet);

		// Bookmarks zum Fetchen und Updaten verwenden
		if (IsCapable(SDB_CAP_BOOKMARKS) &&
			(!pConnection->IsCapable(SDB_CAP_UPDATABLE) || IsCapable(SDB_CAP_UPDATE_POSITIONED))
			  && bBookmarkable)
			pKeySet = new OBookmarkSet(*this);
		else
		{
			// hiermit wird erzwungen, da der Treiber auf alle Faelle nur in forwarddirection geht
			// siehe z.B. ODBC
			nOptions  |= SDB_FORWARDONLY;
			pKeySet = new OValueSet(*this);
		}
	}

	// jetzt auf updatable setzen
	if (!(nOptions & SDB_READONLY) &&
		!((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY && !pKeySet && !IsInsertOnly_OldAPI()))
		SetUpdatable(TRUE);

	// Fehlerstati durch nicht updatable ignorieren
	aStatus.Clear();
	return TRUE;
}
//------------------------------------------------------------------
void SdbCursor::Translate(BOOL bInnerJoin)
{
	DBG_CHKTHIS(SdbCursor,NULL);

	SDB_CHK_CONNECT();

	if (aSQLStatement.Len() == 0)
	{
		// Leeres Statement und kein ParseNode
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("42000"),
					aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_SYNTAX_ERROR))),
					0, String() );
		return;
	}

	aStatus.Clear();
	aParameterNames = aEmptyString;

	if ((nOptions & SDB_INDIRECT) == SDB_INDIRECT)	// nicht nativ
	{
		delete pSQLParseNode;
		String aErrorMsg;
		pSQLParseNode = pConnection->GetSqlParser()->ParseTree(aErrorMsg, aSQLStatement);

		if (pSQLParseNode == NULL)
		{
			aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("01000"),aErrorMsg,0,String());			
			return;
		}

		// and now look if we can inner joins

		aSQLStatement = ResolveInnerJoin(bInnerJoin);
		delete pSQLParseNode;
		pSQLParseNode = pConnection->GetSqlParser()->ParseTree(aErrorMsg, aSQLStatement);

		if (pSQLParseNode == NULL)
		{
			aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("01000"),aErrorMsg,0,String());			
			return;
		}
		// Parameter-Namen aus dem Parse Tree herausfinden, durch '?' ersetzen
		// und den Namen merken ...
		PrepareStatement(pConnection, pSQLParseNode,aParameterNames);
		aParameterNames.EraseTrailingChars(';');

		if (aStatus.IsError())
			return;

		// Parse Tree treiberspezifisch bearbeiten und aNativeSQLStatement bilden:
		DriverTranslate();
	}
}

//------------------------------------------------------------------
void SdbCursor::DriverTranslate()
{
	SDB_CHK_CONNECT();

	aSQLStatement.Erase();
	pSQLParseNode->ParseNodeToStr(aSQLStatement, pConnection->GetMetaData());
}
//------------------------------------------------------------------
void SdbCursor::GetTableNamesOfSQL(SdbSqlParseNode *pParseNode, String &rTabNames)
{ // rekursiv
	SDB_CHK_CONNECT();
	if(SDB_ISRULE(pParseNode,table_name))
	{
		pParseNode->GetChild(0)->ParseNodeToStr(rTabNames,pConnection->GetMetaData(),FALSE,FALSE);
		SdbSqlParseNode *pTempNode = pParseNode->Parent();
		if(pTempNode->Count() == 4)
		{
			rTabNames += ' ';
			rTabNames += pTempNode->GetChild(2)->TokenValue();
		}
		rTabNames += ';';
		return;
	}
	// Weiter absteigen im Parse Tree
	for (UINT32 i = 0; i < pParseNode->Count(); i++)
		GetTableNamesOfSQL(pParseNode->GetChild(i),rTabNames);
}
//------------------------------------------------------------------
void SdbCursor::DescribeParameter()
{
	SDB_CHK_CONNECT();
	if(pSQLParseNode && pParseNodeList)
	{
		// Tabellennamen ermitteln
		SdbSqlParseTreeIterator	aIter(pConnection,pSQLParseNode);
		aIter.TraverseTableNames();
		const SdbTableRangeMap &aTableMap = aIter.GetSdbTableRangeMap();

		//if(SDB_ISRULE(pSQLParseNode,select_statement))
		{

			BOOL bCase = pConnection->GetIdentifierCase() == SDB_IC_ALL;
			String aTabName,aTmp,aColName,aParameterName;
			for(USHORT kk=0; kk < pParseNodeList->Count() ; kk++)
			{
				BOOL bNotFound(TRUE);
				SdbSqlParseNode* pParseNode = pParseNodeList->GetObject(kk);
				pParseNode = pParseNode->Parent();
				pParseNode = pParseNode->GetChild(0);

				aIter.GetColumnRange(pParseNode,aColName,aTabName);
				/*
				aTabName.Erase();
				if(pParseNode->Count() == 1)
					aColName = pParseNode->GetChild(0)->TokenValue();
				else
				{
					aColName = pParseNode->GetChild(pParseNode->Count()-1)->GetChild(0)->TokenValue();
					aTabName = pParseNode->GetChild(pParseNode->Count()-3)->TokenValue();
				}
				*/
				if(aTabName.Len())
				{
					SdbTableRef *pTable = aTableMap.Lookup(aTabName);
					if (!pTable)
						return;					

					aParameterName = (aParameterNames.GetToken((USHORT) aParameterColumns->Count() - 1));
					SdbColumn *pClone;
					const SdbColumn *pCol = (*(*pTable)->Columns())[aColName];
					if (aParameterName.Len() && aParameterName.GetChar(0) != '?')
						pClone = GetConnection()->CreateColumn(aParameterName, *pCol);
					else
						pClone = pCol->Clone();

					pClone->SetFlag(SDB_FLAGS_PARAM_IN);
					aParameterColumns->AddColumn(pClone);
				}
				else
				{
					USHORT nTabCount(aTableMap.Count());
					USHORT i;
					for(i=0;i<nTabCount && bNotFound;i++)
					{
						SdbTableRef *pTable = aTableMap.GetObject(i);
						if(pTable)
						{
							SdbColumnsRef xCols = (*pTable)->Columns();
							for(USHORT j=0;j<xCols->Count() && bNotFound;j++)
							{
								const SdbColumn *pCol = (*xCols)[j];
								if(bCase ? aColName == pCol->GetName() : aColName.EqualsIgnoreCaseAscii(pCol->GetName()))
								{
									aParameterName = aParameterNames.GetToken((USHORT) aParameterColumns->Count() - 1);
									SdbColumn *pClone;
									if (aParameterName.Len() && aParameterName.GetChar(0) != '?')
										pClone = GetConnection()->CreateColumn(aParameterName, *pCol);
									else
										pClone = pCol->Clone();

									pClone->SetFlag(SDB_FLAGS_PARAM_IN);
									aParameterColumns->AddColumn(pClone);
									bNotFound = FALSE;
								}
							}
						}
					}
					if(i>=nTabCount && bNotFound) // in diesem Fall standard Column anlegen
					{
						String aAlias(String::CreateFromAscii("a"));
						String aAlias2(String::CreateFromAscii("a"));
						const SdbColumn* pPrevCol = aParameterColumns->Column(aAlias);
						// existiert Name  in aSelectColumns ?
						ULONG nExprCnt(0);
						while(pPrevCol)
						{
							aAlias = aAlias2;
							aAlias += String::CreateFromInt32(++nExprCnt);
							pPrevCol = aParameterColumns->Column(aAlias);
						}

						SdbColumn * pColumn = pConnection->CreateColumn(aAlias,
																SDB_DBTYPE_VARCHAR,
																255,0, SDB_FLAGS_PARAM_IN);
						// Column-Beschreibung hinzufuegen.
						aParameterColumns->AddColumn(pColumn);
					}
				}
			}
		}
	}
}
// Als Seiteneffekt werden auch Tabellennamen, die keinen Schema-Praefix haben,
// mit dem aktuellen Schema-Praefix versehen!
//------------------------------------------------------------------
void SdbCursor::PrepareStatement(SdbConnection* pConnection, SdbSqlParseNode* pParseNode, String & rString)
{
	DBG_CHKTHIS(SdbCursor,NULL);
	DBG_ASSERT(pParseNode != NULL,"SdbCursor: interner Fehler: ungueltiger ParseNode");

	// Parameter Name-Regel gefunden?
	if (SDB_ISRULE(pParseNode,parameter))
	{
		DBG_ASSERT(pParseNode->Count() >= 1,"SdbCursor: Parse Tree fehlerhaft");
		DBG_ASSERT(pParseNode->GetChild(0)->GetNodeType() == SDB_SQL_PUNCTUATION,"SdbCursor: Parse Tree fehlerhaft");
		if(!pParseNodeList)
			pParseNodeList = new SdbSqlParseNodeList();
		pParseNodeList->Insert(pParseNode, LIST_APPEND);

		String aFirstToken(pParseNode->GetChild(0)->TokenValue());
		DBG_ASSERT(aFirstToken == ':' || aFirstToken == '?' || aFirstToken == '[',"SdbCursor: Parse Tree fehlerhaft");

		if (aFirstToken.GetChar(0) == ':')
		{
			// Name
			DBG_ASSERT(pParseNode->Count() == 2,"SdbCursor: Parse Tree fehlerhaft");
			DBG_ASSERT(pParseNode->GetChild(1)->GetNodeType() == SDB_SQL_NAME,"SdbCursor: Parse Tree fehlerhaft");

			rString += pParseNode->GetChild(1)->TokenValue();

			// ":Name" durch Fragezeichen ersetzen
			delete pParseNode->Remove((UINT32)1);
			delete pParseNode->Remove((UINT32)0);

			pParseNode->Append(new SdbSqlParseNode(String::CreateFromAscii("?"),SDB_SQL_PUNCTUATION));
		}
		else if (aFirstToken.GetChar(0) == '[')
		{
			// Name
			DBG_ASSERT(pParseNode->Count() == 3,"SdbCursor: Parse Tree fehlerhaft");
			DBG_ASSERT(pParseNode->GetChild(1)->GetNodeType() == SDB_SQL_NAME,"SdbCursor: Parse Tree fehlerhaft");

			rString += pParseNode->GetChild(1)->TokenValue();

			// "[Name]" durch Fragezeichen ersetzen
			delete pParseNode->Remove((UINT32)2);
			delete pParseNode->Remove((UINT32)1);
			delete pParseNode->Remove((UINT32)0);

			pParseNode->Append(new SdbSqlParseNode(String::CreateFromAscii("?"),SDB_SQL_PUNCTUATION));
		}
		else
		{
			// Fragezeichen
			DBG_ASSERT(pParseNode->Count() == 1,"SdbCursor: Parse Tree fehlerhaft");

			rString += '?';
		}
		rString += ';';

		// Weiterer Abstieg nicht erforderlich
		return;
	}
	else if (SDB_ISRULE(pParseNode,table_name))
	{
		// Ergaenzen des Schema-Namen, wenn dieser nicht bereits
		// angegeben ist.

		DBG_ASSERT(pParseNode->Count() >= 1,"SdbCursor: Parse Tree fehlerhaft");
		DBG_ASSERT(pParseNode->GetChild(0)->GetNodeType() == SDB_SQL_NAME,"SdbCursor: Parse Tree fehlerhaft");

		String aTableName(pParseNode->GetChild(0)->TokenValue());

		::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
		sal_Int16 nCatalogUsage;
		rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

		sal_Int16 nSchemaUsage;
		rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;
		
		if (pParseNode->Count() == 1 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML))
		{
			// Nur Tabellenname angegeben - durch
			// Schemaname, '.' und Tabellenname ersetzen:
			delete pParseNode->Remove((UINT32)0);
			pParseNode->Append(new SdbSqlParseNode(pConnection->GetQualifier(),SDB_SQL_NAME));
			pParseNode->Append(new SdbSqlParseNode(String::CreateFromAscii("."),SDB_SQL_PUNCTUATION));
			if (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML)
			{
				pParseNode->Append(new SdbSqlParseNode(pConnection->GetUser(),SDB_SQL_NAME));
				pParseNode->Append(new SdbSqlParseNode(String::CreateFromAscii("."),SDB_SQL_PUNCTUATION));
			}
			pParseNode->Append(new SdbSqlParseNode(aTableName,SDB_SQL_NAME));
		}
		else if (pParseNode->Count() == 1 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML))
		{
			// Nur Tabellenname angegeben - durch
			// Schemaname, '.' und Tabellenname ersetzen:
			delete pParseNode->Remove((UINT32)0);
			pParseNode->Append(new SdbSqlParseNode(pConnection->GetUser(),SDB_SQL_NAME));
			pParseNode->Append(new SdbSqlParseNode(String::CreateFromAscii("."),SDB_SQL_PUNCTUATION));
			pParseNode->Append(new SdbSqlParseNode(aTableName,SDB_SQL_NAME));
		}
		// Weiterer Abstieg nicht erforderlich
		return;
	}
	else if (SDB_ISRULE(pParseNode,column_ref))
	{
		// nyi: (noch) nicht implementiert
	}

	// Weiter absteigen im Parse Tree
	for (UINT32 i = 0; i < pParseNode->Count(); i++)
		PrepareStatement(pConnection, pParseNode->GetChild(i),rString);
}

// Update und Moveimplemmentation von Cursoren

//------------------------------------------------------------------
void SdbCursor::SaveStatus()
{
	DBG_ASSERT((IsValid() && aCurrentRow.is()) || !IsValid() , "keine aktuelle Zeile");
	if (aCurrentRow.is())
	{
		// Status auf alle Faelle nicht modifiziert!
		aCurrentRow->setModified(FALSE);
		aOriginalRow  = new ODbRow(*aCurrentRow);  // alten Stand merken
		eOriginalMode = eMode;
	}
}

//------------------------------------------------------------------
void SdbCursor::RestoreStatus()
{
	// alten Zustand wiederherstellen
	DBG_ASSERT((aOriginalRow.is() && aCurrentRow.is()) || !aOriginalRow.is(), "keine aktuelle Zeile");
	if (aOriginalRow.is() && aCurrentRow.is())
	{
		(*aCurrentRow) = *aOriginalRow;
		eMode = eOriginalMode;
	}
}

// wichtig:
// Wenn Zeilen hinzugefuegt werden, wird der Cache nicht beruecksichtgt.
// Dieser sieht von der Aktion nichts
//------------------------------------------------------------------
BOOL SdbCursor::AppendRow()
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_OPEN_BOOL();

	if (!IsUpdatable())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();

	// only if not the compatible mode is set
	if (IsInsertOnly_OldAPI())
	{
		if (aResultRow.is())
			aResultRow->reset();
		else
			aResultRow = new ODbRow(*aResultColumns); // und eine frische Row bereitstellen
		aCurrentRow = aResultRow;
	}
	else
	{
		if (eMode != SDB_MOD_NONE)		// erster Aufruf von AppendRow
			CancelRow();				// ansonsten zunchst alten Stand wiederherstellen
		SaveStatus();				// alten Stand merken

		aCurrentRow = new ODbRow(*aResultColumns); // und eine frische Row bereitstellen
	}

	eMode	   = SDB_MOD_NEW;
	return TRUE;
}

//------------------------------------------------------------------
void SdbCursor::PrepareAppendRow(const ODbRowRef& rRow)
{
	if (pKeySet)
		pKeySet->prepareAppendRow(rRow);
}

// wichtig bei Benutzung des Caches
// Wenn Zeilen modifiziert werden sollen, fuehrt der Cache eine Positionierung zum Abgleich der
// aktuellen CacheRow mit der aktuellen Cursor Row. Beide muessen auf das gleiche zeigen, um Positioniert
// updaten zu koennen!!!
//------------------------------------------------------------------
BOOL SdbCursor::EditRow()
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_OPEN_BOOL();

	if (IsOffRange())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	else if (IsNotValid())
	{
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		return FALSE;
	}
	else if (!IsUpdatable() || IsInsertOnly_OldAPI())			
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();
	// Sicherstellen, dass Cacheposition und Cursorposition uebereinstimmen
	if (pCache && !pCache->synchronize(FALSE))   // Nicht moeglich den Cursor auf den aktuellen Satz zu
	{									   	  // positionieren
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		eMode = SDB_MOD_INVALID;
		aCurrentRow->setState(ROW_DELETED);
		return FALSE;
	}

	if (eMode == SDB_MOD_NONE)	// erster Aufruf von EditRow
	{
		SaveStatus();
	}
	else
	{
		RestoreStatus();  // ansonsten zunchst alten Stand wiederherstellen
		SaveStatus();	  // alten Stand merken
	}

	eMode = SDB_MOD_EDIT;
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbCursor::UpdateRow()
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	BOOL bResult = TRUE;
	BEGIN_DBACTION();
	BUSY_DB();

	if (IsModeNew())
	{
		// wird ein Cache verwendet, sind CurrentRow und Result unterschiedlich
		ODbRowRef aSaveRow;
		if (&aResultRow != &aCurrentRow)
		{
			
			aSaveRow = new ODbRow(*aResultRow);
			(*aResultRow) = *aCurrentRow;
		}

		// Es gelten alle Spalte != NULL als modifiziert, ausnahme: AutoInkrementfelder
		ODbVariantRef xVar;
		const SdbColumn* pCol;
		for (UINT16 i = 1; i < aResultColumns->Count(); i++)
		{
			xVar = (*aResultRow)[i];
			if (xVar.is() && !xVar->isModified() && !xVar->isNull())
			{
				pCol = (*aResultColumns)[i];
				if (!pCol->IsAutoIncrement())
					xVar->setModified(TRUE);
			}
		}

		if (pKeySet)
			bResult = pKeySet->insertRow();
		else if (bBulkOptimize)
		{	// damit Platzhalter treiberspez.
			// angepasst werden koennen
			nOptions |= SDB_INDIRECT;
			bResult = BulkCopy();
		}
		else
			bResult = InsertImpl();
		
		if (bResult)
		{
			if (!IsInsertOnly_OldAPI() && bRowCountFinal)		// nur wenn der RowCount abgeschlossen ist,
			{
				nRowCount++;								    // kann dieser inkrementiert werden, sonst nur ueber Navigation moeglich
				bEOF = FALSE;
			}

			if (nCurrentPos != SDB_CURRENT_ROW_UNDEFINED && pCache)						// Kleine Mitteilung an den Cache
				pCache->rowInserted();

			aResultRow->setModified(FALSE);

			if (!IsInsertOnly_OldAPI()) // optimierung fuer appendonly
			{
				aLastModified = aResultRow->front()->getAny();
				RestoreStatus();						// Urspruenglichen Zustand wiederherstellen
			}
		}

		if (aSaveRow.is())
			(*aResultRow) = *aSaveRow;
	}
	else if (IsModeEdit() || (nOptions & SDB_API_COMPATIBLE) == SDB_API_COMPATIBLE)
	{
		if (aCurrentRow->isModified())
		{
			if (&aCurrentRow != &aResultRow)
				*aResultRow = *aCurrentRow;

			if (pKeySet)
				bResult = pKeySet->updateRow();
			else
				bResult = UpdateImpl();
			
			if (bResult)
			{
				// Anzahl Datensaetze die veraendert wurden
				// Cursorset macht das selbst
				if (!pKeySet)
					aStatus.SetRecordsAffected(1);

				// Sicherstellen, Current && Result nicht mod. sind
				aResultRow->setModified(FALSE);
				if (&aCurrentRow != &aResultRow)
					*aCurrentRow = *aResultRow;					

				// Backup loeschen
				aOriginalRow.unbind();
				eMode = SDB_MOD_NONE;
				if (!IsInsertOnly_OldAPI())
					aLastModified = aCurrentRow->front()->getAny();					
			}
			else
			{
				// ResultRow erhaelt wieder den urspruenglichen Zustand
				DBG_ASSERT(aOriginalRow.is(), "kein Backup der ResultRow");
				if (&aCurrentRow != &aResultRow)
					*aResultRow = *aOriginalRow;					
			}
		}
		else
			eMode = SDB_MOD_NONE;
	}
	else
	{
		aStatus.SetSequenceError();
		bResult = FALSE;
	}

	END_DBACTION();
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbCursor::DeleteRow()
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	if (!IsUpdatable() || IsInsertOnly_OldAPI())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	if (IsOffRange())
	{
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		return FALSE;
	}

	switch (eMode)
	{
		case SDB_MOD_EDIT:
			RestoreStatus();		// ursprunglichen Zustand der Zeile wiederherstellen
									// dann kann geloescht werden
		case SDB_MOD_NONE:
			if (pCache)
			{
				if (!pCache->synchronize(FALSE) && !aStatus.IsNoDataFound())
				{
					aStatus.Set(SDB_STAT_NO_DATA_FOUND);	// Positionierungsfehler
					eMode = SDB_MOD_INVALID;
					aCurrentRow->setState(ROW_OBSOLETE);
					break;
				}
			}

			BEGIN_DBACTION();
			BUSY_DB();

			// Wenn der Satz inzwischen schon geloescht sein sollte
			// mu die Implementierung trotzdem funktionieren
			if (pKeySet  ? pKeySet->deleteRow()
							: DeleteImpl())
			{
				if (!aStatus.IsSuccessful())		// hier muss noch eine genaurer Fehlerbehendlung erfolgen
					aStatus.Clear();
				else if (!pKeySet)
					aStatus.SetRecordsAffected(1);

				aCurrentRow->setState(ROW_OBSOLETE);

				if (nRowCount > 0)					// RowCount anpassen
					nRowCount--;
			}

			END_DBACTION();
			break;
		case SDB_MOD_NEW:
			eMode = SDB_MOD_INVALID;
			break;
		default:
			aStatus.SetSequenceError();
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbCursor::CancelRow()
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_OPEN();

	switch (eMode)
	{
		case SDB_MOD_NEW:				// holen der Originalwerte
		case SDB_MOD_EDIT:
			RestoreStatus();
			break;
	}
}			   

//------------------------------------------------------------------
BOOL SdbCursor::DoMove(SdbCursorPosition eCursorPosition,
					   INT32 nOffset,
					   BOOL bSkipDeleted,
					   BOOL bSetCurrent,
					   BOOL bRetrieveData)
{
	SDB_CHK_CONNECT_BOOL();

	// bSkipDeleted ist ein Schalter um das ueberlesen von geloeschten Saetzen einzustellen
	// Wenn diese Flag FALSE, dann werden auf jeden Fall geloeschte mitgelesen !!
	// bSetCurrent ist ein extra Schalter fuer den Cache

	// Positionierung auf invalider Position auer mit First und Last nicht erlaubt
	// Wid verwendet, wenn kein Cache eingesetzt wird

	if (bEOF)
	{
		// eCursorPosition == SDB_POS_RELATIVE && nOffset == 0) ist erlaubt, da der Cursor
		// immer auf dem letzten Satz stehen bleibt

		if ((eCursorPosition == SDB_POS_NEXT) ||
			(eCursorPosition == SDB_POS_RELATIVE && nOffset > 0))
		{
			eMode = SDB_MOD_INVALID;
			return FALSE;
		}
	}
	else if (bBOF)
	{
		if ((eCursorPosition == SDB_POS_PRIOR) ||
			(eCursorPosition == SDB_POS_RELATIVE && nOffset <= 0))
		{
			eMode = SDB_MOD_INVALID;
			return FALSE;
		}
	}

	// if one of the following loops is canceled we have to restore the original position
	::staruno::Any aOriginalBookmark;
	if (IsValid() && aCurrentRow.is())
	{
		// don't simply call GetLong : the value may be void for some cursors, in that case an SbxErrorCode will be
		// set and as we have only one such error (globally) a possibly executed Basic-Statement will fail
		// (with a "data types not compatible" error). Unbelievable but true ;(
		// (To be honest : If no such error would be set nevertheless we had to do this check : GetLong returns 0 if
		// the value is NULL which is wrong in this context ....)
		// 67953 - 30.07.99 - FS		
		if (!aCurrentRow->front()->isNull())
			aOriginalBookmark = aCurrentRow->front()->getAny();		
	}

	switch (eCursorPosition)
	{
		// ------------------
		case SDB_POS_LAST:
			if (!bRowCountFinal)
			{
				BeginCancelable();
					// DetermineRowCount has a cancel handling
				DetermineRowCount();
				if (EndCancelable())
				{
					DoMove(aOriginalBookmark, TRUE, TRUE);
					aStatus.Set(SDB_STAT_CANCELED);
					return FALSE;
				}
			}
			break;
	// Positionierung auf den letzten Satz ist fuer den Cursor ein
	// Problem wenn die Ermittlung des RowCounts noch nicht abgeschlossen ist,
	// da die aktuelle Position und der RowCount nach der Positionierung nicht mehr
	// definiert sind. Um dieses zu vermeiden, wird sequentiell ans Ende gegangen.case SDB_POS_LAST:

		// ------------------
		case SDB_POS_FIRST:
	// Steht der Cursor vor dem ersten Satz, dann immer MoveNext verwenden
			if (nCurrentPos == SDB_CURRENT_ROW_BOF)
				eCursorPosition = SDB_POS_NEXT;

	// fuer Datenbanken die nur in forward direction scrollen knnen
	// wird ein first ingnoriert, wenn bereits auf den ersten positioniert ist (nach OnOpen())
			else if (nCurrentPos == 0 &&
				!pKeySet && !IsCapable(SDB_CAP_FETCH_FIRST))
				return IsValid();
			break;

		// ------------------
		case SDB_POS_RELATIVE:
			if (nOffset > 1 && (((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY) && !pKeySet))
			{
				BeginCancelable();
				while (!WasCanceled(TRUE) && DoMove(SDB_POS_NEXT,1,TRUE,TRUE,TRUE) && (--nOffset > 1)); // in Einzelschritte auflsen
				if (EndCancelable())
				{
					DoMove(aOriginalBookmark, TRUE, TRUE);
					aStatus.Set(SDB_STAT_CANCELED);
					return FALSE;
				}
			}
			else if (nOffset == 1)
				eCursorPosition = SDB_POS_NEXT;	 // Next nehmen, da Next RowCountFinal feststellen kann

			break;

		// ------------------
		case SDB_POS_ABSOLUTE:
			if (nOffset > (nCurrentPos + 1) && (((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY) && !pKeySet))
			{
				nOffset -= nCurrentPos + 1;
				BeginCancelable();
				while (!WasCanceled(TRUE) && DoMove(SDB_POS_NEXT,1,TRUE,TRUE,TRUE) && (--nOffset > 1));	// in Einzelschritte auflsen
				if (EndCancelable())
				{
					DoMove(aOriginalBookmark, TRUE, TRUE);
					aStatus.Set(SDB_STAT_CANCELED);
					return FALSE;
				}
			}
			break;
	}

	BOOL bDataFound;
	if (bSkipDeleted && !SeeDeleted())
		// geloeschte ueberspringen
		bDataFound = SkipDeleted(eCursorPosition, nOffset, bRetrieveData);
	else
		bDataFound = Seek(eCursorPosition, nOffset, bRetrieveData);

	SetPositionFlags(bDataFound, eCursorPosition, nOffset);

	if (bDataFound)
		aResultRow->setModified(FALSE);

	if (bSetCurrent)
	{
		if (!bDataFound)
			eMode = SDB_MOD_INVALID;		
		else
			aCurrentRow = aResultRow;	// Zeigt auf die aktuelle Ergebniszeile
	}
	return bDataFound && IsValid();
}

//------------------------------------------------------------------
BOOL SdbCursor::Seek(SdbCursorPosition eCursorPosition, INT32 nOffset, BOOL bRetrieveData)
{
	BOOL bResult;
	eMode = SDB_MOD_NONE;

	if (pKeySet)
		bResult = pKeySet->seek(eCursorPosition, nOffset, eMode, bRetrieveData);
	else
		bResult = MoveImpl(eCursorPosition, nOffset, eMode);

	if (bResult)
		aResultRow->setState(eMode == SDB_MOD_NONE ? ROW_CLEAN :ROW_DELETED);
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbCursor::SkipDeleted(SdbCursorPosition eCursorPosition, INT32 nOffset, BOOL bRetrieveData)
{
	SdbCursorPosition eDelPosition = eCursorPosition;
	INT32 nDelOffset = abs(nOffset);
	BOOL bDataFound;
	BOOL bDone;

	switch (eCursorPosition)
	{
		case SDB_POS_ABSOLUTE:
		case SDB_POS_FIRST:					// erster Satz nicht gltig, gehe auf nchsten
			eDelPosition = SDB_POS_NEXT;
			nDelOffset = 1;
			break;
		case SDB_POS_LAST:
			eDelPosition = SDB_POS_PRIOR; // letzter Satz nicht gltig, gehe auf vorherigen
			nDelOffset = 1;
			break;
		case SDB_POS_RELATIVE:
			eDelPosition = (nOffset >= 0) ? SDB_POS_NEXT : SDB_POS_PRIOR;
			break;
	}

	// Relatve Bewegung wird in Next oder Prior umgesetzt
	if (eCursorPosition != SDB_POS_RELATIVE)
	{
		bDataFound = Seek(eCursorPosition, nOffset, bRetrieveData);
		bDone = bDataFound && !aResultRow->isDeleted();
	}
	else
	{
		bDataFound = Seek(eDelPosition, 1, bRetrieveData);
		if (bDataFound && !aResultRow->isDeleted())
			bDone = (--nDelOffset) == 0;
		else
			bDone = FALSE;
	}

	while (bDataFound && !bDone)			// solange iterieren bis man auf einem gltigen Satz ist
	{
		bDataFound = Seek(eDelPosition, 1, bRetrieveData);
		if (eCursorPosition != SDB_POS_RELATIVE)
			bDone = bDataFound && !aResultRow->isDeleted();
		else if (bDataFound && !aResultRow->isDeleted())
			bDone = (--nDelOffset) == 0;
		else
			bDone = FALSE;
	}
	return bDataFound;
}

//------------------------------------------------------------------
INT32 SdbCursor::GetCurrentPos() const
{
	if (nCurrentPos == SDB_CURRENT_ROW_UNDEFINED)
		return nCurrentPos;
	else if (IsBOF())	// immer Cache beruecksichtigen
		return SDB_CURRENT_ROW_BOF;
	else if (IsEOF())
		return nRowCount + 1;
	else
	{
		if (pCache)
			return pCache->getCurrentPos() + 1;
		else
			return nCurrentPos + 1;
	}
}

//------------------------------------------------------------------
BOOL SdbCursor::Move(SdbCursorPosition eCursorPosition, INT32 nOffset)
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_CONNECT_BOOL();
	// Wenn der Cursor nicht geoeffnet, oder nur Datensaetze hinzugefuegt werden sollen
	// ist eine Positionierung nicht moeglich

	if (!IsOpen() || IsInsertOnly_OldAPI())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	// relative positionierung auf einer unbekannten Position ist nicht erlaubt
	else if (nCurrentPos == SDB_CURRENT_ROW_UNDEFINED &&
		 (eCursorPosition == SDB_POS_NEXT ||
		  eCursorPosition == SDB_POS_RELATIVE ||
		  eCursorPosition == SDB_POS_PRIOR))
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	BOOL bResult(FALSE);

	CancelRow(); // aktuelle Bearbeitung abbrechen

	BEGIN_DBACTION();

	if (aCurrentRow.is() && aCurrentRow->isObsolete())
	{
		// Fehler, da der Satz geloescht wurde
		if (eCursorPosition == SDB_POS_RELATIVE && nOffset == 0)
		{
			aStatus.SetSequenceError();
			END_DBACTION();
			return FALSE;
		}

		// Reduzieren der aktuelle Position
		nCurrentPos--;
		if (nCurrentPos < 0)	// steht wieder vor dem ersten Datensatz
			bBOF = TRUE;

		// jetzt dem Cache mitteilen, da dieser sich auch noch angleichen mu
		if (pCache)
			pCache->rowDeleted();

		aCurrentRow->setState(ROW_DELETED);

		// jetzt die Art der Positionierung anpassen
		switch (eCursorPosition)
		{
			case SDB_POS_PRIOR:
				// Datensatz mu nur aktualisiert werden
				nOffset	= 0;
			case SDB_POS_RELATIVE:
				if (nOffset < 0)
					nOffset++;
				if (pCache)
				{
					if (nOffset > 0)
						bResult =pCache->positionRelative(SDB_DIR_RIGHT, nOffset);
					else if (nOffset < 0)
						bResult =pCache->positionRelative(SDB_DIR_LEFT, -nOffset);
					else
						bResult =pCache->refresh();
				}
				else
					bResult = DoMove(SDB_POS_RELATIVE, nOffset,TRUE,TRUE,TRUE);

				END_DBACTION();
				return bResult;
		}
	}

	if (pCache)  // Cache bernimmt das Positionieren
	{
		switch (eCursorPosition)
		{
			case SDB_POS_FIRST:
				bResult =pCache->positionAbsolute(ROWCACHE_FIRST);
				break;
			case SDB_POS_LAST:
				BeginCancelable();
				bResult = pCache->positionAbsolute(ROWCACHE_LAST);
				bResult &= !EndCancelable();
				break;
			case SDB_POS_NEXT:
				bResult =pCache->positionRelative(SDB_DIR_RIGHT, 1);
				break;
			case SDB_POS_PRIOR:
				bResult = pCache->positionRelative(SDB_DIR_LEFT, 1);
				break;			
			case SDB_POS_ABSOLUTE:
				BeginCancelable();
				bResult = pCache->positionAbsolute(nOffset);
				bResult &= !EndCancelable();
				break;
			case SDB_POS_RELATIVE:
				if (abs(nOffset) > nCacheSize)
				{
					BeginCancelable();
					bResult = pCache->positionAbsolute(GetCurrentPos() + nOffset);
					bResult &= !EndCancelable();
				}
				else if (nOffset > 0)
					bResult = pCache->positionRelative(SDB_DIR_RIGHT, nOffset);
				else if (nOffset < 0)
					bResult = pCache->positionRelative(SDB_DIR_LEFT, -nOffset);
				else
					bResult = pCache->refresh();
				break;
		}
	}
	else
		bResult = DoMove(eCursorPosition, nOffset,TRUE,TRUE,TRUE);

	DBG_ASSERT((IsValid() && aCurrentRow.is()) || !IsValid(), "aktuelle Zeile Fehlerhaft");

	END_DBACTION();
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbCursor::MoveBookmark(const ::staruno::Any& rBookmark) throw( ::com::sun::star::lang::IllegalArgumentException )
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_CONNECT_BOOL();
	// Wenn der Cursor nicht geoeffnet, oder nur Datensaetze hinzugefuegt werden sollen
	// ist eine Positionierung nicht moeglich

	if (!IsOpen() || IsInsertOnly_OldAPI())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}	

	BOOL bResult(FALSE);

	CancelRow(); // aktuelle Bearbeitung abbrechen	
	
	// is the current bookmark the desired one?
	if (IsValid())
	{
		DBG_ASSERT(aCurrentRow.is(), "aktuelle Zeile Fehlerhaft");
		if (rBookmark.hasValue() && aCurrentRow.is() && aCurrentRow->front()->equals(rBookmark))
			return TRUE;
	}

	BEGIN_DBACTION();

	// do we have an obsolete row
	if (aCurrentRow.is() && aCurrentRow->isObsolete())
	{
		// Reduzieren der aktuelle Position
		nCurrentPos--;
		if (nCurrentPos < 0)	// steht wieder vor dem ersten Datensatz
			bBOF = TRUE;

		// jetzt dem Cache mitteilen, da dieser sich auch noch angleichen mu
		if (pCache)
			pCache->rowDeleted();

		aCurrentRow->setState(ROW_DELETED);		
	}


	if (pCache)  // Cache bernimmt das Positionieren
	{
		BeginCancelable();
		try
		{
			bResult = pCache->positionBookmark(rBookmark);
			bResult &= !EndCancelable();
		}
		catch (::com::sun::star::lang::IllegalArgumentException e)
		{
			bResult &= !EndCancelable();
			throw;
		}		
	}	
	else
		bResult = DoMove(rBookmark, TRUE,TRUE);

	DBG_ASSERT((IsValid() && aCurrentRow.is()) || !IsValid(), "aktuelle Zeile Fehlerhaft");
	END_DBACTION();
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbCursor::DoMove(const ::staruno::Any& rBookmark,
					   BOOL bSetCurrent,
					   BOOL bRetrieveData) throw( ::com::sun::star::lang::IllegalArgumentException )
{
	SDB_CHK_CONNECT_BOOL();

	if (!pKeySet)
	{
		aStatus.SetSequenceError();
		return FALSE;
	}	

	eMode = SDB_MOD_NONE;

	BOOL bDataFound = FALSE;	
	try
	{
		bDataFound = pKeySet->seek(rBookmark, eMode, bRetrieveData);
		if (bDataFound)
		{
			aResultRow->setState(eMode == SDB_MOD_NONE ? ROW_CLEAN :ROW_DELETED);
			nCurrentPos = pKeySet->getCurrentPos();
			bEOF = bBOF = FALSE;
			eMode = SDB_MOD_NONE;
			aResultRow->setModified(FALSE);

			if (bSetCurrent)
				aCurrentRow = aResultRow;	// Zeigt auf die aktuelle Ergebniszeile
		}
		else
		{
			nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
			bEOF = bBOF = FALSE;
			if (bSetCurrent)
				eMode = SDB_MOD_INVALID;		
		}	
	}
	catch (::com::sun::star::lang::IllegalArgumentException e)
	{
		nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
		bEOF = bBOF = FALSE;
		if (bSetCurrent)
			eMode = SDB_MOD_INVALID;		

		throw;
	}	
	return bDataFound && IsValid();
}			

//------------------------------------------------------------------
void SdbCursor::DetermineRowCount()
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	if (bRowCountFinal)					// RowCount bereits festgestellt
		return;

	DBG_ASSERT(aResultRow.is(), "keine Ergebniszeile");

	SDB_CHK_CONNECT();

	if (!IsOpen())
	{
		aStatus.SetSequenceError();
		return;
	}

	aStatus.Clear();

	BEGIN_DBACTION();
	BUSY_DB();

	if (!DetermineRowCountImpl(nRowCount))		// Der Treiber ist nicht in der Lage
										// den RowCount festzustellen
	{									// dann den RowCount sequentiell ermitteln
		ODbRowRef aBufferRow = new ODbRow(*aCurrentRow);
		if (pKeySet)
		{
			UINT32 nSetCount = pKeySet->size();
			BeginCancelable();
			pKeySet->determineRowCount();
			EndCancelable();
		}
		*aCurrentRow = *aBufferRow;
		if (WasCanceled(FALSE))
		{
			// aStatus will be set to FALSE (by EndCancelable)
			bRowCountFinal = FALSE;
			END_DBACTION();
			return;
		}

		if (aStatus.IsNoDataFound())					// wenn ans ende positioniert ist
			aStatus.Clear();
	}
	else
		bRowCountFinal = TRUE;

	END_DBACTION();
}

//------------------------------------------------------------------
void SdbCursor::SetRowCount(UINT32 nCount)
{
	DBG_ASSERT(!bRowCountFinal && !IsOpen(),"interner Fehler: RowCount nach dem Open gesetzt");
	if (!bRowCountFinal && !IsOpen())
	{
		nRowCount = (INT32)nCount;
		aStatus.SetRecordsAffected(nCount);
		bRowCountFinal = TRUE;
	}
}

//------------------------------------------------------------------
BOOL SdbCursor::DetermineRowCountImpl(INT32&)
{
	// vorimplementiert, standardmaessig nicht moeglich,
	// d.h. die Standardimplementierung von DetermineRowCount wird verwendet s.o.
	return FALSE;
}

//------------------------------------------------------------------
void SdbCursor::SetPositionFlags(BOOL bDataFound, SdbCursorPosition ePosition, INT32 nOffset)
{
	INT32 nDirection;
	switch (ePosition)
	{
		case SDB_POS_FIRST:
			nDirection = 1;
			if (!bDataFound)
			{
				// siehe unten
				nCurrentPos = SDB_CURRENT_ROW_BOF;
				nRowCount	= 0;
				bRowCountFinal = TRUE;
			}
			else
			{
				nCurrentPos = 0;
				SetRowCount(bDataFound, 1);
			}
			break;
		case SDB_POS_LAST:
			nDirection = 1;
			if (!bDataFound)
			{
				nCurrentPos    = SDB_CURRENT_ROW_BOF;
				nRowCount	   = 0;
				bRowCountFinal = TRUE;
			}
			else if (bRowCountFinal)
			{
				if (nRowCount)
					nCurrentPos = nRowCount - 1;
				else
					nCurrentPos = 0;
				SetRowCount(bDataFound, 1);
			}
			else
				nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
			break;
		case SDB_POS_ABSOLUTE:
			nDirection = nOffset;
			if (!bDataFound)
				nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
			else if (nOffset > 0)
				nCurrentPos = nOffset - 1;

			if (bDataFound && nCurrentPos >= nRowCount)
				nRowCount = nCurrentPos + 1;
			break;
		case SDB_POS_RELATIVE:
			nDirection = nOffset;
			if (!bDataFound)
			{
				BOOL bPositionUndefined = TRUE;
				if (nOffset<0)
					bPositionUndefined = !((nCurrentPos >= 0) && (-nOffset > nCurrentPos));
					// wenn das FALSE ist, bedeutet das BOF, das wird unten abgehandelt
				if (bPositionUndefined)
					nCurrentPos = SDB_CURRENT_ROW_UNDEFINED;
			}
			else if (nCurrentPos != SDB_CURRENT_ROW_UNDEFINED && bDataFound)
				nCurrentPos += nOffset;
			if (bDataFound && nCurrentPos >= nRowCount)
				nRowCount = nCurrentPos + 1;
			break;
		case SDB_POS_NEXT:
			nDirection = 1;
			if (nCurrentPos != SDB_CURRENT_ROW_UNDEFINED && bDataFound)
				nCurrentPos++;
			SetRowCount(bDataFound, 1);

			// Ist die einzige Mglichkeit, um Festzustellen ob das das Zaehlen von Zeilen fertig ist
			if (!bRowCountFinal && !bDataFound && nRowCount == nCurrentPos + 1)
				bRowCountFinal = TRUE;

			break;
		case SDB_POS_PRIOR:
			nDirection	= -1;
			if (nCurrentPos > SDB_CURRENT_ROW_BOF && bDataFound)
				nCurrentPos--;
			break;
	}

	// Bestimmen von EOF und BOF
	if (!bDataFound)
	{
		// Bei Positionierung auf ersten, letzten oder ueber Bookmark ist der Status
		// des Cursors unbekannt
		if (nCurrentPos == SDB_CURRENT_ROW_UNDEFINED)
		{
			// undefiniert
			bEOF = bBOF = FALSE;
		}
		else
		{
			bEOF = nDirection >= 0 ? TRUE : FALSE;
			bBOF = !bEOF;
		}

		if (nCurrentPos != SDB_CURRENT_ROW_UNDEFINED)
		{
			if (nDirection >= 0)
				nCurrentPos++;		// steht hinter dem letzten
			else
				nCurrentPos--;		// steht vor dem ersten
		}
	}
	else
	{
		bEOF = bBOF = FALSE;
		eMode = SDB_MOD_NONE;
	}
}


//------------------------------------------------------------------
void SdbCursor::SetRowCount(BOOL bDataFound, INT32 nOffset)
{
	if (bDataFound && !bRowCountFinal && nCurrentPos != SDB_CURRENT_ROW_UNDEFINED)
	{
		nRowCount = max(nRowCount, nCurrentPos + nOffset);
	}
}

//------------------------------------------------------------------
void SdbCursor::InitResultRow()
{
	// eine einmal angelegte Ergebniszeile darf bis zum zum Close nicht mehr freigegeben werden!!!
	// da sich andere auf deren Existenz verlassen
	if (!aResultRow.is())
		aCurrentRow = aResultRow = new ODbRow(*aResultColumns);			
	else
	{
		aCurrentRow = aResultRow;
		aResultRow->reset();
	}
}

//------------------------------------------------------------------
BOOL SdbCursor::IsUpdateAllowed() const
{
	if (pKeySet)
		return !pKeySet->isSnapshot();

	// prinzipiell erst mal ja annehmen
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbCursor::SetUpdatable(BOOL bUpdate)
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	SDB_CHK_CONNECT_BOOL();

	if (!IsOpen() ||
		((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY && !pKeySet &&
		!IsInsertOnly_OldAPI()))
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	aStatus.Clear();
	BOOL bResult(FALSE);
	if (pKeySet)
		bResult = pKeySet->setUpdatable(bUpdate);
	else
		bResult = SetUpdatableImpl(bUpdate);

	if (bResult)
	{
		if (!bUpdate)
			nPrivileges = SDB_PR_READ;
		else if (IsInsertOnly_OldAPI())
		{
			// Korrigieren falls mehr mglich ist
			nPrivileges &= ~SDB_PR_UPDATE;
			nPrivileges &= ~SDB_PR_DELETE;

			if (!nPrivileges)
			{
				// gar nicht weiter versuchen
				aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S1C00"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_CURSOR_NOT_UPDATABLE))),
						0, String() );
				nPrivileges = SDB_PR_READ;
			}
		}
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbCursor::SetUpdatableImpl(BOOL bUpdate)
{
	SDB_CHK_CONNECT_BOOL();

	if (IsCapable(SDB_CAP_UPDATE_POSITIONED))
	{
		if (bUpdate)
			nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_DELETE;
		else
			nPrivileges = SDB_PR_READ;
	}
	else
	{
		aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S1C00"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_CURSOR_NOT_UPDATABLE))),
						0, String() );
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
SdbConnection* SdbCursor::GetConnection() const
{
#ifdef DBG_UTIL
	return PTR_CAST(SdbConnection,GetParent());
#else
	return (SdbConnection*)GetParent();
#endif
}

//------------------------------------------------------------------
BOOL SdbCursor::Compare(const SdbObj& _rObj) const
{
	SdbCursor* pObj = PTR_CAST(SdbCursor,&_rObj);
	return pObj != NULL && aSQLStatement == pObj->aSQLStatement;
}

//------------------------------------------------------------------
String SdbCursor::BuildBulkSQL(const String& rTableName) const
{
	SdbConnection* pConnection = GetConnection();
	String aStr(String::CreateFromAscii("INSERT INTO "));
	aStr += pConnection->QuoteTableName(rTableName);
	aStr.AppendAscii(" (");
	String aValues(String::CreateFromAscii(" VALUES ("));

	UINT16 nCount(aResultColumns->Count());
	for (UINT16 i = 1; i < nCount; i++)
	{
		const SdbColumn& rCol = *(*aResultColumns)[i];
		if (rCol.IsUpdateAllowed())
		{
			SdbColumn* pColumn = pConnection->CreateColumn(rCol);
			pColumn->SetFlag(SDB_FLAGS_PARAM_IN);

			aParameterColumns->AddColumn(pColumn);
			aStr += pConnection->IdentifierQuotation(rCol.GetName());  // add to column list
			aStr += ',';
			aValues.AppendAscii("?,");
		}
	}
	// overwrite last ',' with ')' for the column list
	if (aStr.GetChar(aStr.Len()-1) == ',')
		aStr.SetChar(aStr.Len()-1, ')');

	// overwrite last ',' with ')' for the value list
	if (aValues.GetChar(aValues.Len()-1) == ',')
		aValues.SetChar(aValues.Len()-1,')');

	aStr += aValues;
	return aStr;
}

//------------------------------------------------------------------
BOOL SdbCursor::BulkCopy()
{
	// Parameter fuellen
	if (!aParameterRow.is())
	{
		aParameterRow = new ODbRow(*aParameterColumns,FALSE);
		USHORT j=1;
		for (USHORT i=0; i < aResultColumns->Count(); i++)
		{
			if ((*aResultColumns)[i]->IsUpdateAllowed())
				(*aParameterRow)[j++] = (*aResultRow)[i];
		}
		DBG_ASSERT(aParameterRow->size() == aParameterColumns->Count(),"aParameterRow != aParameterColumns");
	}
	return OpenImpl();
}
//------------------------------------------------------------------
String SdbCursor::ResolveInnerJoin(BOOL bCanInnerJoin)
{
	String aNewSql(aSQLStatement);

	SdbConnection* pConnection = GetConnection();

	if(!bCanInnerJoin)
	{
		// this all is only allowed when we have a select statement
		if(SQLParseNode() && SDB_ISRULE(SQLParseNode(),select_statement))
		{
			String aCondition;
			SdbSqlParseNode * pTableRefCommalist = SQLParseNode()->GetChild(3)->GetChild(0)->GetChild(1);
			for (ULONG i = 0; i < pTableRefCommalist->Count(); i++)
			{ // from clause durchlaufen
				if(aCondition.Len())
					aCondition.AppendAscii(" AND ");
				ResolveInnerJoin_Recursiv(pTableRefCommalist->GetChild(i),aCondition);
			}

			if(aCondition.Len()) // if no condition - nothing to change
			{
				SdbSqlParseTreeIterator aIter(pConnection,SQLParseNode());
				aIter.TraverseTableNames();
				const SdbTableRangeMap& aMap = aIter.GetSdbTableRangeMap();
				String aFromClause;
				ULONG i;
				for(i=0;i<aMap.Count();i++)
				{
					SdbTableRef* pTableRef	= aMap.GetObject(i);
					const String& aAlias	= aMap.GetKey(pTableRef);
					(((aFromClause += *pTableRef != NULL ? (*pTableRef)->GetFullName() : String()) += ' ') += aAlias);
					aFromClause.AppendAscii(", ");
				}
				aFromClause.EraseTrailingChars(' ');
				aFromClause.EraseTrailingChars(',');
				String aNew;
				SQLParseNode()->GetChild(0)->ParseNodeToStr(aNew,pConnection->GetMetaData());
				SQLParseNode()->GetChild(1)->ParseNodeToStr(aNew,pConnection->GetMetaData());
				SQLParseNode()->GetChild(2)->ParseNodeToStr(aNew,pConnection->GetMetaData());

				(aNew.AppendAscii(" FROM ")) += aFromClause;

				if(SQLParseNode()->GetChild(3)->GetChild(1)->Count())
				{
					SQLParseNode()->GetChild(3)->GetChild(i)->ParseNodeToStr(aNew,pConnection->GetMetaData());
					aNew.AppendAscii(" AND ");
				}
				else
				{
					aNew.AppendAscii(" WHERE ");
				}
				aNew += aCondition;
				for(i=2;i < SQLParseNode()->GetChild(3)->Count(); ++i)
				{
					SQLParseNode()->GetChild(3)->GetChild(i)->ParseNodeToStr(aNew,pConnection->GetMetaData());
				}
				aNewSql = aNew;
			}
		}
	}
	return aNewSql;
}
//------------------------------------------------------------------
void SdbCursor::ResolveInnerJoin_Recursiv(SdbSqlParseNode *pTableRef,String &rCondition)
{
	// '(' joined_table ')' as range_variable op_column_commalist
	if (SDB_ISRULE(pTableRef,joined_table) || SDB_ISRULE(pTableRef,table_ref) && pTableRef->Count() == 5)
		ResolveInnerJoin_Recursiv(pTableRef->GetChild(1),rCondition);
	else if (SDB_ISRULE(pTableRef,qualified_join) && SDB_ISRULE(pTableRef->GetChild(1),join_type) &&
			 pTableRef->GetChild(1)->Count() == 1 && SDB_ISRULE(pTableRef->GetChild(4),join_condition))
	{	// here I have a inner join, look at the yacc file if you don't belive me
		if(rCondition.Len())
			rCondition.AppendAscii(" AND ");
		pTableRef->GetChild(4)->GetChild(1)->ParseNodeToStr(rCondition, GetConnection()->GetMetaData());

	}
}

//------------------------------------------------------------------
BOOL SdbCursor::IsOnFirst() const
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	return (pCache) ? pCache->isOnFirst() : !bEOF && nCurrentPos == 0;
}

//------------------------------------------------------------------
BOOL SdbCursor::IsBOF() const
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	return (pCache) ? pCache->isOffLeft() :bBOF;
}

//------------------------------------------------------------------
BOOL SdbCursor::IsEOF() const
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	return (pCache) ? pCache->isOffRight() : bEOF;
}

//------------------------------------------------------------------
BOOL SdbCursor::IsOnLast() const
{
	NAMESPACE_VOS(OGuard) aGuard(pLock);
	BOOL bResult = FALSE;
	if (pCache)
		bResult = pCache->isOnLast();
	else if (IsRowCountFinal() && !IsOffRange())
	{
		if (aCurrentRow.is() && aCurrentRow->isObsolete())
			bResult = ((INT32)RowCount()) == nCurrentPos;
		else
			bResult = ((INT32)RowCount()) == nCurrentPos + 1;
	}
	return bResult;
}

//------------------------------------------------------------------
ODbVariant* SdbCursor::CreateVariant() const
{
	return new ODbVariant();
}


