/*************************************************************************
 *
 *  $RCSfile: sdbdb.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


// Environment:
#ifndef _SDB_SDBWORK_HXX
#include "sdbwork.hxx"
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASELIMIT_HPP_
#include <com/sun/star/data/DatabaseLimit.hpp>
#endif

#include "sdbdbimp.hxx"
#include "sdbdb.hxx"
#include "sdbresid.hrc"

#ifndef _SDB_SDBREL_HXX
#include <sdbrel.hxx>
#endif

#ifndef _SDB_SDBQRY_HXX
#include "sdbqry.hxx"
#endif

#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif

#ifndef _SDB_SDBVIEW_HXX
#include "sdbview.hxx"
#endif

#ifndef _SDB_SDBDOC_HXX
#include "sdbdoc.hxx"
#endif

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

DBG_NAME(SdbDatabase);

TYPEINIT1(SdbDatabase,SdbObj);

#define SDB_CHK_DB()								\
	if (!pImpl)										\
	{												\
		aStatus.SetConnectionLost();				\
		return;										\
	}
#define SDB_CHK_DB_BOOL()							\
	if (!pImpl)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}
#define SDB_CHK_DB_STRING()							\
	if (!pImpl)										\
	{												\
		aStatus.SetConnectionLost();				\
		return String();								\
	}
#define SDB_CHK_DB_NULL()							\
	if (!pImpl)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

//==============================================================================
//------------------------------------------------------------------------------
void SdbDatabaseGuard::clear()
{
	if (m_pDBImpl)
	{
		// ! release the mutex _before_ releasing the ref, as the impl object may be deleted within ReleaseRef !
		m_pDBImpl->m_aThreadSafety.release();
		m_pDBImpl->ReleaseRef();
		m_pDBImpl = NULL;
	}
}

//------------------------------------------------------------------------------
void SdbDatabaseGuard::attach(const SdbDatabase* _pDB)
{
	attach(_pDB ? _pDB->Impl() : NULL);
}

//------------------------------------------------------------------------------
void SdbDatabaseGuard::attach(const SdbDatabaseImpl* _pDBImpl)
{
	if (_pDBImpl == m_pDBImpl)
		return;

	clear();
	m_pDBImpl = (SdbDatabaseImpl*)_pDBImpl;
	if (m_pDBImpl)
	{
		m_pDBImpl->m_aThreadSafety.acquire();
		m_pDBImpl->AddRef();
	}
}

//==============================================================================
//------------------------------------------------------------------------------
SdbDatabase::SdbDatabase(SdbWorkspace* pWorkspace)
			:SdbObj(pWorkspace, FALSE)
{
	DBG_CTOR(SdbDatabase,NULL);

	// kein RefCounting sondern RemoveParent zu Zerstoeren
	pImpl = new SdbDatabaseImpl(pWorkspace);
	aQueryList.SetCaseSensitive(TRUE);
	aFormList.SetCaseSensitive(TRUE);
	aReportList.SetCaseSensitive(TRUE);
}

//------------------------------------------------------------------------------
SdbDatabase::~SdbDatabase()
{
	DBG_DTOR(SdbDatabase,NULL);
	if (pImpl)
		pImpl->RemoveFromParent();
}

//------------------------------------------------------------------------------
IMPL_LINK(SdbDatabase, OnStatusChange, SdbStatus*, _pStatus )
{
	aStatus = *_pStatus;
	return 0L;
}

//------------------------------------------------------------------------------
SdbWorkspace* SdbDatabase::GetWorkspace() const
{
	SdbDatabaseGuard aSafeAccess(this);
	return PTR_CAST(SdbWorkspace,GetParent());
}

//------------------------------------------------------------------
BOOL SdbDatabase::Compare(const SdbObj& _rObj) const
{
	SdbDatabaseGuard aSafeAccess(this);
	SdbDatabase* pObj = PTR_CAST(SdbDatabase,&_rObj);
	return pObj != NULL && pImpl != NULL && DirEntry(pImpl->Name()) == DirEntry(pObj->Name());
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::Load(const String& rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->Load(rName);
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::Open()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->Open();
	bOpen =		   pImpl->IsOpen();
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
void SdbDatabase::Close()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN();
	SDB_CHK_DB();

	if (pImpl->aStorage.Is())
	{	// if this check fails we're closing (triggered implicitly) after we were dropped
		if (!IsReadOnly())
		{
			// as our formatter may be altered by objects which don't know anything about this database owning the formatter
			// (e.g. objects accessing via UNO-wrappers) we always write the formatter before closing
			SdbStatus aOldStatus = Status();
			AlterFormatter();
			aStatus = aOldStatus;
		}
	}

	pImpl->Close();
	pImpl->RemoveFromParent();
	pImpl = NULL;

	SdbObj::Close();
}

//------------------------------------------------------------------
void SdbDatabase::RemoveChildren()
// freigeben aller Childs, siehe SdbObj
{
	SdbDatabaseGuard aSafeAccess(this);
	while (aQueryList.Count())
		RemoveChild(aQueryList.GetObject(0));

	while (aFormList.Count())
		RemoveChild(aFormList.GetObject(0));

	while (aReportList.Count())
		RemoveChild(aReportList.GetObject(0));
}

//------------------------------------------------------------------
void SdbDatabase::RemoveChild(SdbObj* pObj)
{
	SdbDatabaseGuard aSafeAccess(this);
	if (pObj->IsA(TYPE(SdbQuery)))
		aQueryList.Remove(PTR_CAST(SdbQuery,pObj));
	else if (pObj->IsA(TYPE(SdbDocument)))
	{
		SdbDocument* pDocument = PTR_CAST(SdbDocument,pObj);
		if (!pDocument)
		{
			DBG_ERROR("SdbDatabase::RemoveChild -> NoDocument to remove!");
			;
		}
		else if (pDocument->GetKind() == dbForm)
			aFormList.Remove(pDocument);
		else
			aReportList.Remove(pDocument);
	}
	else
		return;
	SdbObj::RemoveChild(pObj);
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::Create(const String& rName,
						 const String& rTitle, const String& rConnectStr)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->Create(rName, rTitle, rConnectStr);
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::Drop()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->Drop();
	aStatus = pImpl->Status();
	if (bResult && bOpen) // nur beim Vater abmelden, wenn geoeffnet
		GetWorkspace()->RemoveChild(this);
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::Rename(const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();

	BOOL bResult = pImpl->Rename(_rName);
	aStatus = pImpl->Status();
	if (bResult && bOpen)
		GetWorkspace()->aDBList.Replace(this, DirEntry(_rName), this);
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::AlterTitle(const String& rTitle)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->AlterTitle(rTitle);
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
// wenn die Datenbank noch geffnet ist, dann darf allerdings nur
// die Environment eine Referenz auf die Datenbank halten
// ansonsten kann kein neuer Verbindungsaufbau erfolgen
BOOL SdbDatabase::AlterConnection(const String& _rConnect)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_DB_BOOL();
	if (IsOpen() && GetRefCount() > 1)
	{
		String aText = String(SdbResId(STR_STAT_DB_CHANGE_CONFIG));
		aText.SearchAndReplace(String::CreateFromAscii("#"),Name());
		aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(aText),
						0, String() );
		return FALSE;
	}
	// alle Kinder freigeben
	RemoveChildren();

	BOOL bResult = pImpl->AlterConnection(_rConnect);
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::Refresh(DBObject _eObj)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();

	pImpl->Refresh(_eObj);
	ULONG nCount = 0;
	switch (_eObj)
	{
		case dbTable:
		case dbView:
		{
			SdbConnection* pConnection = GetConnection();
			nCount = pConnection->aTableList.Count();
			// von hinten aufrollen, da auch Objekte aus der Liste entfallen koennen
			while (nCount)
				((SdbObj*)pConnection->aTableList.GetObject(--nCount))->Refresh();
		}	break;
		case dbQuery:
			nCount = aQueryList.Count();
			while (nCount)
				((SdbObj*)aQueryList.GetObject(--nCount))->Refresh();
			break;
		case dbReport:
			nCount = aReportList.Count();
			while (nCount)
				((SdbObj*)aReportList.GetObject(--nCount))->Refresh();
			break;
		case dbForm:
			nCount = aFormList.Count();
			while (nCount)
				((SdbObj*)aFormList.GetObject(--nCount))->Refresh();
			break;
		default:
			DBG_ERROR("Falsches Object verwendet");
	}

	// Status wieder zuruecksetzen
	aStatus.Clear();
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbDatabase::DoRefresh()
{
	SdbDatabaseGuard aSafeAccess(this);
	Refresh(dbTable);
	Refresh(dbQuery);
	Refresh(dbForm);
	Refresh(dbReport);
	Refresh(dbRelation);
	return TRUE;
}

//------------------------------------------------------------------------------
String SdbDatabase::GetObjectName(DBObject _eObj, USHORT _nPos) const
{
	SdbDatabaseGuard aSafeAccess(this);
	String aObjName;
	if (!IsOpen() || !pImpl)
		return aObjName;

	SdbHashString* pString = NULL;
	switch (_eObj)
	{
		case dbTable:
		case dbView:
			pString = pImpl->aTableNameList.GetObject(_nPos);
			break;
		case dbQuery:
			pString = pImpl->aQueryNameList.GetObject(_nPos);
			break;
		case dbForm:
			pString = pImpl->aFormNameList.GetObject(_nPos);
			break;
		case dbReport:
			pString = pImpl->aReportNameList.GetObject(_nPos);
			break;
		case dbRelation:
			aObjName = GetConnection()->GetRelationName(_nPos);
			break;
		default:
			DBG_ERROR("Falsches Object verwendet");
	}
	if (pString)
		aObjName.AppendAscii(pString->GetString().GetBuffer());
	return aObjName;
}

//------------------------------------------------------------------------------
USHORT SdbDatabase::GetObjectCount(DBObject _eObj) const
{
	SdbDatabaseGuard aSafeAccess(this);
	USHORT nRes = 0;
	if (!IsOpen() || !pImpl)
		return nRes;

	switch (_eObj)
	{
		case dbTable:
		case dbView:
			nRes = (USHORT)pImpl->aTableNameList.Count();
			break;
		case dbQuery:
			nRes = (USHORT)pImpl->aQueryNameList.Count();
			break;
		case dbForm:
			nRes = (USHORT)pImpl->aFormNameList.Count();
			break;
		case dbReport:
			nRes = (USHORT)pImpl->aReportNameList.Count();
			break;
		case dbRelation:
			nRes = GetConnection()->GetRelationCount();
			break;
		default:
			DBG_ERROR("Falsches Object verwendet");
	}
	return nRes;
}

#if (SUPD>512) || PRIV_DEBUG
//------------------------------------------------------------------------------
BOOL SdbDatabase::HasObjectDeleted(DBObject _eObj, const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	return Impl()->HasObjectDeleted(_eObj, _rName);
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::DeleteObject(DBObject _eObj, const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	if (!Impl()->DeleteObject(_eObj, _rName))
	{
		aStatus = Impl()->Status();
		return FALSE;
	}

	// wenn ein SdbObject fuer das gerade geloeschte Objekt existiert -> auch wegwerfen
	SdbObj* pObject = NULL;
	switch (_eObj)
	{
		case dbQuery	: pObject = aQueryList.Lookup(_rName);	break;
		case dbForm		: pObject = aFormList.Lookup(_rName);	break;
		case dbReport	: pObject = aReportList.Lookup(_rName);	break;
	}
	if (pObject)
		RemoveChild(pObject);

	aStatus = Impl()->Status();
	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::UndeleteObject(DBObject _eObj, String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	return Impl()->UndeleteObject(_eObj, _rName);
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::DestroyDeletedObject(DBObject _eObj, const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	return Impl()->DestroyDeletedObject(_eObj, _rName);
}
#endif // (SUPD>512) || PRIV_DEBUG

//------------------------------------------------------------------------------
BOOL SdbDatabase::HasObject(DBObject _eObj, const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();

	BOOL bResult = pImpl->HasObject(_eObj, _rName);
	if (!bResult)
	{
		switch (_eObj)
		{
			case dbTable:
			case dbView:
			{
				SdbConnection* pConnection = GetConnection();
				bResult = pConnection->aTableList.Lookup(SdbTable::GetFullName(*pConnection, _rName)) != NULL;
			}	break;
			case dbQuery:
				bResult = aQueryList.Lookup(_rName) != NULL;
				break;
			case dbForm:
				bResult = aFormList.Lookup(_rName) != NULL;
				break;
			case dbReport:
				bResult = aReportList.Lookup(_rName) != NULL;
				break;
			case dbRelation:
			{
				SdbConnection* pConnection = GetConnection();
				bResult = pConnection->aRelationList.Lookup(_rName) != NULL;
			}	break;
			default:
				DBG_ERROR("Falsches Object verwendet");
		}
	}
	return bResult;
}

//------------------------------------------------------------------------------
SdbObj* SdbDatabase::CreateObject(DBObject _eObj, const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();

	if (HasObject(_eObj, _rName))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,_eObj, _rName);
		return NULL;
	}

	SdbObj* pObj = NULL;
	switch (_eObj)
	{
		case dbTable:
			pObj = GetConnection()->CreateTable(_rName);
			if (!pObj)
				aStatus = GetConnection()->Status();
			else
				((SdbTable*) pObj)->Set(this);
			break;
		case dbView:
			pObj = GetConnection()->CreateView(_rName);
			if (!pObj)
				aStatus = GetConnection()->Status();
			else
				((SdbTable*) pObj)->Set(this);
			break;
		case dbQuery:
		{
			SdbQuery* pQuery = new SdbQuery(this);
			if (pQuery->Create(_rName))
			{
				pObj = pQuery;
				aQueryList.InsertAt(_rName, pQuery);
			}
			else
			{
				aStatus = pQuery->Status();
				pQuery->RemoveFromParent();
			}
			break;
		}
		case dbForm:
		case dbReport:
		{
			SdbDocument* pDoc = new SdbDocument(this, _eObj);
			if (pDoc->Create(_rName))
			{
				pObj = pDoc;
				if (_eObj == dbForm)
					aFormList.InsertAt(_rName, pDoc);
				else
					aReportList.InsertAt(_rName, pDoc);
			}
			else
			{
				aStatus = pDoc->Status();
				pDoc->RemoveFromParent();
			}
			break;
		}
		case dbRelation:
			pObj = GetConnection()->CreateRelation(_rName);
			if (!pObj)
				aStatus = GetConnection()->Status();
			break;
		default:
			DBG_ERROR("Falsches Object verwendet");
	}
	return pObj;
}

//------------------------------------------------------------------------------
SdbObj* SdbDatabase::OpenObject(DBObject _eObj, const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();

	if (!HasObject(_eObj, _rName))
	{
		aStatus.SetError(SVSTREAM_FILE_NOT_FOUND, _eObj, _rName);
		return NULL;
	}

	SdbObj* pObj = NULL;
	switch (_eObj)
	{
		case dbTable:
		case dbView:
			pObj = GetConnection()->OpenTable(_rName);
			if (!pObj)
				aStatus = GetConnection()->Status();
			else
				((SdbTable*) pObj)->Set(this);
			break;
		case dbRelation:
			pObj = GetConnection()->OpenRelation(_rName);
			if (!pObj)
				aStatus = GetConnection()->Status();
			break;
		case dbQuery:
		{
			SdbQuery* pQuery = aQueryList.Lookup(_rName);
			BOOL bInList = pQuery != NULL;
			if (!bInList)
				pQuery = new SdbQuery(this);
			if (!bInList && !pQuery->Open(_rName))
			{
				aStatus = pQuery->Status();
				pQuery->RemoveFromParent();
				pQuery = NULL;
			}
			else if (!bInList)
				aQueryList.InsertAt(_rName, pQuery);
			pObj = pQuery;
		}	break;
		case dbReport:
		case dbForm:
		{
			BOOL bForm = _eObj == dbForm;
			SdbDocument* pDocument = bForm ? aFormList.Lookup(_rName)
											:aReportList.Lookup(_rName);

			BOOL bInList = pDocument != NULL;
			if (!bInList)
				pDocument = new SdbDocument(this, _eObj);
			if (!bInList && !pDocument->Open(_rName))
			{
				aStatus = pDocument->Status();
				pDocument->RemoveFromParent();
				pDocument = NULL;
			}
			else if (!bInList && bForm)
				aFormList.InsertAt(_rName, pDocument);
			else if (!bInList)
				aReportList.InsertAt(_rName, pDocument);
			pObj = pDocument;
		}	break;
		default:
			DBG_ERROR("Falsches Object verwendet");
	}
	return pObj;
}

//------------------------------------------------------------------------------
String SdbDatabase::GetFullTableName(const String& _rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_STRING();
	SDB_CHK_DB_STRING();
	return pImpl->GetFullTableName(_rName);
}
//------------------------------------------------------------------------------
BOOL SdbDatabase::IsUniqueName(DBObject _eObj, const String& rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();
	SdbConnection* pConnection = GetConnection();

	BOOL bFound = FALSE;
	String aSearchName = rName;
	switch (_eObj)
	{
		case dbTable:
		case dbView:
			aSearchName = GetFullTableName(aSearchName);
			bFound = HasObject(dbTable, aSearchName);
			// werden Tabellennamen gefiltert, dann muessen alle Tabellennamen
			// bercksichtigt werden
			
			if (!bFound && pImpl->bTableFilter)
			{
				BOOL bCase = (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;
				for (USHORT i=0; i<pImpl->aMaskedTables.Count(); ++i)
				{
					if (bCase ? (*pImpl->aMaskedTables.GetObject(i)) == aSearchName : pImpl->aMaskedTables.GetObject(i)->EqualsIgnoreCaseAscii(aSearchName))
					{
						bFound = TRUE;
						break;
					}
				}

				// dieser alte Teil hier unten macht nichts anderes als obiges HasObject auch, es sieht nur anders aus ...
				// (SdbConnection::GetTableCount() liefert nur die Anzahl tatsaechlich sichtbarer Tabellen)
//				SdbHashStringList aTestList; // Erstellen der neuen Liste aus allen Tabellen einer Datenbank
//				USHORT nCount = pConnection->GetTableCount(),
//						j	  = 0;
//				while (j < nCount)
//					aTestList.Insert(new SdbHashString(pConnection->GetTableName(j++)), LIST_APPEND);

//				bFound = pImpl->Find(aSearchName, aTestList) != NULL;
//				for (UINT32 i=0; i < aTestList.Count(); i++)
//					delete aTestList.GetObject(i);
			}
			break;
		case dbQuery:
		case dbForm:
		case dbReport:
			bFound = HasObject(_eObj, aSearchName);
	}
	return !bFound;
}
//------------------------------------------------------------------------------
String SdbDatabase::GetUniqueName(DBObject _eObj, const String& rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_STRING();
	SDB_CHK_DB_STRING();

	String aSuggestion = rName;
	SdbHashStringList aTestList; // Erstellen der neuen Liste aus allen Tabellen einer Datenbank
	SdbConnection* pConnection = GetConnection();

	aTestList.SetCaseSensitive((pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT);
	::com::sun::star::uno::Reference< ::com::sun::star::data::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());	
	xub_StrLen nNameLen = rName.Len();
	
	// maximale Laenge
	xub_StrLen nMaxLen  = 0;
	if (_eObj == dbTable || _eObj == dbView)
		nMaxLen = rMetaData->getLimit(::com::sun::star::data::DatabaseLimit::TABLE_NAME_LEN);

	ULONG  i = 0;
	String aNewName;	// an den Vorschlag wird einfach eine Zahl angehaengt
	BOOL   bFound;
	do
	{
		i++;
		while (aSuggestion.Len())
		{
			aNewName = aSuggestion;
			aNewName += String::CreateFromInt32(i);
			if (!nMaxLen || aNewName.Len() <= nMaxLen)
				break;
			else
			{
				aSuggestion.Erase(aSuggestion.Len() - 1,1);
				i = 1;
			}
		}

		String aSearchName(aNewName);
		switch (_eObj)
		{
			// fuer Abfrage und Tabellen komplizierter
			// a.) Abfragen und Tabellen muessen vom Namen her eindeutig sein
			// b.) Tabellen koennen gefiltert sein,
			// c.) Tabellen haben eine max. Laenge fuer den Namen
			case dbTable:
			case dbView:
				aSearchName = GetFullTableName(aSearchName);
				if (pImpl->bTableFilter && !aTestList.Count()) //b.)
				{
					USHORT nCount = pConnection->GetTableCount(),
							j	  = 0;
					while (j < nCount)
						aTestList.Insert(new SdbHashString(pConnection->GetTableName(j++)), LIST_APPEND);

				}
				bFound  = HasObject(dbTable, aSearchName) ||
						  pImpl->Find(aSearchName, aTestList) != NULL;

				break;
			case dbQuery:
			case dbForm:
			case dbReport:
				bFound = HasObject(_eObj, aSearchName);
		}
	} while (bFound);

	for (UINT32 j=0; j < aTestList.Count(); j++)
		delete aTestList.GetObject(j);

	return aNewName;

	return pImpl->GetUniqueName(_eObj, rName);
}

//------------------------------------------------------------------------------
SdbCursor* SdbDatabase::CreateCursor(SdbCursorType eType, UINT16 nOptions)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();

	return GetConnection()->CreateCursor(eType, nOptions);
}

//------------------------------------------------------------------------------
// nur fuer interne Verwendung
SdbConnection* SdbDatabase::GetConnection() const
{
	SdbDatabaseGuard aSafeAccess(this);
	return pImpl->GetConnection();
}

//------------------------------------------------------------------------------
SdbCursor* SdbDatabase::OpenCursor(const String& aObjName,
								   SdbCursorType eType,
								   UINT16 nOptions)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();

	SdbCursor* pCursor = NULL;
	if (HasObject(dbTable, aObjName))
	{
		SdbTable* pTable = (SdbTable*) OpenObject(dbTable, aObjName);
		if (pTable)
			return pTable->OpenCursor(eType, nOptions);
	}
	else if (HasObject(dbQuery, aObjName))
	{
		SdbQuery* pQuery = (SdbQuery*) OpenObject(dbQuery, aObjName);
		if (pQuery)
			return pQuery->OpenCursor(eType, nOptions);
	}
	return pCursor;
}

//------------------------------------------------------------------------------
String SdbDatabase::Name() const
{
	SdbDatabaseGuard aSafeAccess(this);
	return pImpl ? pImpl->Name() : String();
}

//------------------------------------------------------------------------------
String SdbDatabase::ConnectStr() const
{
	SdbDatabaseGuard aSafeAccess(this);
	return pImpl ? pImpl->ConnectStr() : String();
}

//------------------------------------------------------------------------------
String SdbDatabase::Title() const
{
	SdbDatabaseGuard aSafeAccess(this);
	return pImpl ? pImpl->Title() : String();
}

//------------------------------------------------------------------------------
SdbStorage* SdbDatabase::OpenStorage(SdbStorage& rMain, const String& rName, StreamMode eMode, StorageMode eStorMode)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();
	SdbStorage* pStor = pImpl->OpenStorage(rMain, rName, eMode, eStorMode);
	aStatus = pImpl->Status();
	return pStor;
}

//------------------------------------------------------------------------------
SdbStorage* SdbDatabase::OpenStorage(DBObject _eObj, BOOL bReadOnly)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();
	SdbStorage* pStor = pImpl->OpenStorage(_eObj, bReadOnly);
	aStatus = pImpl->Status();
	return pStor;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::ReleaseStorage(DBObject _eObj, BOOL bCommit)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->ReleaseStorage(_eObj, bCommit);
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
SvStorageStream* SdbDatabase::OpenStream(SdbStorage& rMain, const String& rEleName, StreamMode eMode)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();
	SvStorageStream* pStream = pImpl->OpenStream(rMain, rEleName, eMode);
	aStatus = pImpl->Status();
	return pStream;
}

//------------------------------------------------------------------------------
SvStorageStream* SdbDatabase::OpenContent(SdbStorage& aMain, StreamMode eMode)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DB_NULL();
	SvStorageStream* pStream = pImpl->OpenContent(aMain, eMode);
	aStatus = pImpl->Status();
	return pStream;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::RenameStorage(const String& aOldName, const String& aNewName, DBObject _eObj)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();

	BOOL bResult = pImpl->RenameStorage(aOldName, aNewName, _eObj);
	aStatus = pImpl->Status();

	// wenn das Objekt eine Tabelle ist und ich Tabelle filtere, muss sich die Aenderung im ConnectString der Tabelle
	// niederschlagen
	if (bResult && (_eObj == dbTable) && pImpl->bTableFilter)
		pImpl->AdjustConnectStr(FALSE);

	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::RemoveStorage(const String& rName, DBObject _eObj)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();

	BOOL bResult = pImpl->RemoveStorage(rName, _eObj);
	aStatus = pImpl->Status();
	return bResult;
}
//------------------------------------------------------------------------------
BOOL SdbDatabase::IsReadOnly() const
{
	return pImpl ? pImpl->bReadOnly : TRUE;
}

//------------------------------------------------------------------------------
SvNumberFormatter* SdbDatabase::GetFormatter() const
{
	return pImpl ? &pImpl->GetFormatter() : NULL;
}
//------------------------------------------------------------------------------
BOOL SdbDatabase::AlterFormatter()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->AlterFormatter();
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabase::GetRelationLayout(SvStream& rStream)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();
	return pImpl ? pImpl->GetRelationLayout(rStream) : FALSE;
}
//------------------------------------------------------------------------------
BOOL SdbDatabase::AlterRelationLayout(SvStream& rStream)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DB_BOOL();
	BOOL bResult = pImpl->AlterRelationLayout(rStream);
	aStatus = pImpl->Status();
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbDatabase::ExecuteSQL(const String& aStatement)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	BOOL bResult = GetConnection()->ExecuteSQL(aStatement);
	if (!bResult)
		aStatus = GetConnection()->Status();
	return bResult;
}

//------------------------------------------------------------------
SdbColumns*	SdbDatabase::DescribeResult(const String& aStatement,
										BOOL bTable)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	SdbColumns* pCols = GetConnection()->DescribeResult(aStatement, bTable);
	if (!pCols)
		aStatus = GetConnection()->Status();
	return pCols;
}

//------------------------------------------------------------------
SdbColumns* SdbDatabase::CreateColumns()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	return GetConnection()->CreateColumns();
}

//------------------------------------------------------------------
SdbColumns* SdbDatabase::CreateColumns(const SdbColumns& _rColumns)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	return GetConnection()->CreateColumns(_rColumns);
}

//------------------------------------------------------------------
SdbColumn*  SdbDatabase::CreateColumn(const String& _rName,
								 SdbDatabaseType _eType,
								 UINT32 _nLength, UINT16 _nScale,
								 UINT16 _nFlags)
{
	SDB_CHK_OPEN_NULL();
	return GetConnection()->CreateColumn(_rName, _eType,
										 _nLength, _nScale, _nFlags);
}

//------------------------------------------------------------------
SdbColumn*  SdbDatabase::CreateColumn(const SdbColumn& _rColumn)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_NULL();
	return GetConnection()->CreateColumn(_rColumn);
}

//------------------------------------------------------------------
String SdbDatabase::IdentifierQuotation(const String& rString)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_STRING();
	return GetConnection()->IdentifierQuotation(rString);
}

//------------------------------------------------------------------
BOOL SdbDatabase::IsSQLNameValid(const String& rString)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	return GetConnection()->IsSQLNameValid(rString);
}

//------------------------------------------------------------------
String SdbDatabase::QuoteTableName(const String& rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_STRING();
	return GetConnection()->QuoteTableName(rName);
}

//------------------------------------------------------------------
String SdbDatabase::CompleteTableName(const String& rName)
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_STRING();
	return GetConnection()->CompleteTableName(rName);
}

//------------------------------------------------------------------
BOOL SdbDatabase::BeginTrans()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	return GetConnection()->BeginTrans();
}

//------------------------------------------------------------------
BOOL SdbDatabase::CommitTrans()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	return GetConnection()->CommitTrans();
}

//------------------------------------------------------------------
BOOL SdbDatabase::RollbackTrans()
{
	SdbDatabaseGuard aSafeAccess(this);
	SDB_CHK_OPEN_BOOL();
	return GetConnection()->RollbackTrans();
}

//------------------------------------------------------------------
// Erzeugt einen neuen Namen falls noetig
String SdbDatabase::ConvertName(const String& rName) const
{
	SdbDatabaseGuard aSafeAccess(this);
	return GetConnection()->ConvertName(rName);
}

//------------------------------------------------------------------
BOOL SdbDatabase::IsCharOk(char c) const
{
	SdbDatabaseGuard aSafeAccess(this);
	return GetConnection()->IsCharOk(c);
}
//------------------------------------------------------------------
BOOL SdbDatabase::DoCmd(UINT16 nCmd,const String &rArgs)
{
	SdbDatabaseGuard aSafeAccess(this);
	return GetConnection()->DoCmd(nCmd,rArgs);
}

