/*************************************************************************
 *
 *  $RCSfile: sdbdbimp.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#define ENABLE_BYTESTRING_STREAM_OPERATORS
#include "sdbdbimp.hxx"
#include "sdbresid.hrc"
#include "sdbmap.hxx"

#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif

#ifndef _SFXINIPROP_HXX //autogen
#include <svtools/iniprop.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _URLOBJ_HXX //autogen wg. INetURLObject
#include <tools/urlobj.hxx>
#endif

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _SDB_SDBQRY_HXX
#include "sdbqry.hxx"
#endif

#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _INTN_HXX //autogen
#include <tools/intn.hxx>
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _SDB_SDBWORK_HXX
#include "sdbwork.hxx"
#endif

DBG_NAME(SdbDatabaseImpl);

static char __READONLY_DATA CONNECTION[] = "Connection";
static char __READONLY_DATA TABLES[] = "Table";
static char __READONLY_DATA QUERIES[] = "Query";
static char __READONLY_DATA CONTENT[] = "Content";
static char __READONLY_DATA FORMS[] = "Form";
static char __READONLY_DATA REPORTS[] = "Report";
static char __READONLY_DATA VERSION[] = "Version";
static char __READONLY_DATA VER[] = "0x0004";
static char __READONLY_DATA FORMATTER[] = "Format";
static char __READONLY_DATA RELATIONS[] = "RelationLayout";
static char __READONLY_DATA WASTE[] = "WasteBasket";

#define TABLE_FILTER  "TBL_FILTER"	// interne Option fuer Tabellenfilter


#define SDB_STD_STREAM_MODE		(STREAM_READ | STREAM_SHARE_DENYNONE | STREAM_WRITE)


const long nTimeOut = 2500;			// 2,5 Sekunden wird probiert, eine DB
									// schreibend zu oeffnen

#define START()		\
	SdbEnvironment* pEnvironment = GetEnvironment();	\
	if (pEnvironment)									\
	{													\
		pEnvironment->BeginHdl().Call(this);			\
		pEnvironment->BusyHdl().Call(this);				\
	}


#define END()		\
	if (pEnvironment)									\
		pEnvironment->EndHdl().Call(this);

#define BEGINREAD()		//			\
	// if (!BeginTrans(FALSE))	\
	//	return FALSE;

#define ENDREAD()			\
	// if (!EndTrans(FALSE))	\
	//	return FALSE;

#define BEGINWRITE()		\
	if (!BeginTrans())	\
		return FALSE;

#define EXECUTE_READ(action)	\
	if (!action)				\
	{							\
		return FALSE;			\
	}

#define EXECUTE_WRITE(action)		\
	DBG_ASSERT(!bReadOnly, "Storage lesend geoeffnet!"); \
	if (!action)				\
	{							\
		EndTrans(FALSE);		\
		return FALSE;			\
	}


//------------------------------------------------------------------------------
SdbDatabaseImpl::SdbDatabaseImpl(SdbWorkspace* pWorkspace)
			:SdbObj(pWorkspace,FALSE)
			,pConnection(NULL)
			,bLoaded(FALSE)
			,bReadOnly(FALSE)
			,aFormatter(GetpApp()->GetAppInternational().GetLanguage())
			,bTableFilter(FALSE)
			,bDeleteOnClose(FALSE)
			,nTransCount(0)
{
	DBG_CTOR(SdbDatabaseImpl,NULL);
}

//------------------------------------------------------------------------------
SdbDatabaseImpl::~SdbDatabaseImpl()
{
	// to prevent a second desctruction if somebody aquires and releases me herein ...
	AddRef();

	DBG_DTOR(SdbDatabaseImpl,NULL);
	// alle Queries zerstoeren:

	UINT32 i;
	for (i=0; i < aTableNameList.Count(); i++)
		delete aTableNameList.GetObject(i);

	for (i=0; i < aQueryNameList.Count(); i++)
		delete aQueryNameList.GetObject(i);

	for (i=0; i < aFormNameList.Count(); i++)
		delete aFormNameList.GetObject(i);

	for (i=0; i < aReportNameList.Count(); i++)
		delete aReportNameList.GetObject(i);

	if (pConnection)
	{
		pConnection->ReleaseRef();
		pConnection = NULL;
	}
	aTableStor = aQueryStor = aFormStor = aReportStor = aStorage = NULL;
#if (SUPD>512) || PRIV_DEBUG
	m_aWasteBasket = NULL;
#endif // (SUPD>512) || PRIV_DEBUG
}

//------------------------------------------------------------------------------
SdbEnvironment* SdbDatabaseImpl::GetEnvironment() const
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	return GetParent() ? PTR_CAST(SdbEnvironment,GetWorkspace()->GetEnvironment()) : NULL;
}

//------------------------------------------------------------------------------
SdbWorkspace* SdbDatabaseImpl::GetWorkspace() const
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	return PTR_CAST(SdbWorkspace,GetParent());
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Load(const String& rName)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	if (IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	else if ((!rName.Len()) || (DirEntry(rName).GetFlag() == FSYS_FLAG_ABSROOT) || (DirEntry(rName).GetFlag() == FSYS_FLAG_RELROOT))
	{
		aStatus.SetError(ERRCODE_IO_NOTEXISTS,DATABASE,rName);
		return FALSE;
	}
	else
	{
		aStatus.Clear();
		DirEntry aEntry(rName);
		if (!aEntry.GetExtension().Len())
			aEntry.SetExtension(String::CreateFromAscii(SDB_DATABASE_EXT));
		aName = aEntry.GetFull();
	}

	// erst den Namen setzen
	LoadImpl();

	return bLoaded;
}

//------------------------------------------------------------------------------
// speichern der Tabellen im ConnectStr
void SdbDatabaseImpl::AdjustConnectStr(BOOL bInit)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	if (!bInit && !bTableFilter)
		return;

	// Existiert ein Filter fuer Tabellen
	CommandParser aOptionsParser(aConnectStr,';','=',TRUE);

	if (bInit)
		bTableFilter = aOptionsParser.Exists(String::CreateFromAscii(TABLE_FILTER));

	if (bTableFilter)
	{
		// dann die Tabellen im ConnectStr vermerken
		ReadList(dbTable);
		String aTableList;

		for (ULONG i = 0; i < aTableNameList.Count(); i++)
		{
			aTableList += String(aTableNameList.GetObject(i)->GetString(),gsl_getSystemTextEncoding());
			aTableList += DEFAULT_LIST_SEPARATOR;
		}
		aTableList.EraseLeadingChars(DEFAULT_LIST_SEPARATOR);

		aConnectStr.Erase();
		aOptionsParser.Compose(aConnectStr,String::CreateFromAscii(TABLE_FILTER));	// Alles ausser den Sdb-internen Optionen weiterreichen
		aConnectStr.EraseLeadingChars(';');
		aConnectStr.EraseTrailingChars(';');
		aConnectStr += ';';
		aConnectStr.AppendAscii("TABLE");
		aConnectStr += '=';
		aConnectStr += aTableList;
	}
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::LoadImpl()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	aStatus.Clear();
	DBG_ASSERT(!IsOpen(), "Load on open database not possible");
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	// Versuch schreiben zu Oeffnen
	aStorage = new SdbStorage(*this, aName, STREAM_READWRITE | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE, STORAGE_TRANSACTED);  // Lesen/Schreibend oeffnen
	if (aStorage->Impl()->GetError() == SVSTREAM_SHARING_VIOLATION ||
		aStatus.SetError(*aStorage->Impl(),DATABASE))
	{
		// dann nur lesend oeffnen
		aStorage = new SdbStorage(*this, aName, STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE, 0);
		if (aStatus.SetError(*aStorage->Impl(),DATABASE))
		{
			// auch lesend konnte nicht geoeffnet werden, dann versuchen eine lokale Kopie anlegen
			// OriginalFile
			aStatus.Clear();
			aTempFile = DirEntry().TempName();
			DirEntry aFile = DirEntry(aName);
			if (aFile.CopyTo(aTempFile, FSYS_ACTION_COPYFILE) != ERRCODE_NONE)
				aStatus.SetError(ERRCODE_IO_ACCESSDENIED, DATABASE, aName);
			if (aStatus.IsSuccessful())
			{
				aStorage = new SdbStorage(*this, aTempFile.GetFull(), STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE, 0);
				if (aStorage->Impl()->GetError() != ERRCODE_NONE)
					aStatus.SetError(ERRCODE_IO_ACCESSDENIED, DATABASE, aName);
			}
			if (aStatus.IsError())
			{
				aStorage.Clear();
				aTempFile.Kill();
				return FALSE;
			}
			else
			{
				bReadOnly = TRUE;
				bDeleteOnClose = TRUE;
			}
		}
		else
			bReadOnly = TRUE;
	}

	BOOL bStreamVersionUnknown = (aStorage->GetVersion() != 1) && (aStorage->GetVersion() != 2) && (aStorage->GetVersion() != 3) && (aStorage->GetVersion() != 5);
	if (bStreamVersionUnknown)
	{
		// das kann nur heissen, dass das Ganze mit einer neueren Version geschrieben wurde
		aStorage.Clear();

		String strError(SdbResId(STR_STAT_UNKNOWN_SDB_VERSION));
		aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String(SdbResId(STR_STAT_UNKNOWN_SDB_VERSION_DETAILED)));
		bLoaded = FALSE;

		return FALSE;
	}


	SvStorageStreamRef aStream = OpenStream(*aStorage,String::CreateFromAscii(CONNECTION), STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE);
	if (aStream.Is())
	{
		EXECUTE_READ(Read(*aStream, aConnectStr, aStorage->GetVersion() == 5));
	}


	if (aStorage->GetVersion()==2)
	{
		CommandParser aOptionsParser(aConnectStr,';','=',TRUE);

		if (aOptionsParser.Exists(String::CreateFromAscii("PWD")))
		{
			ByteString sEncrypted(aOptionsParser.OptionString(String::CreateFromAscii("PWD")),gsl_getSystemTextEncoding());

			// Encryption unter Verwendung eines MemoryStreams...
			SvMemoryStream aMemoryStream(sEncrypted.GetBufferAccess(), sEncrypted.Len(), STREAM_READ);
			aMemoryStream.SetKey(ByteString("a$*'FF|~@dsf"));
			ByteString sPassword;
			aMemoryStream >> sPassword;
			aOptionsParser.SetOption(String::CreateFromAscii("PWD"), String(sPassword,gsl_getSystemTextEncoding()));
			aConnectStr.Erase();
			aOptionsParser.Compose(aConnectStr,String());
		}
	}

	if (aStorage->GetVersion()==3)
	{
		ByteString aTemp(aConnectStr,gsl_getSystemTextEncoding());
		SvMemoryStream aMemoryStream(aTemp.GetBufferAccess(), aTemp.Len(), STREAM_READ);
		aMemoryStream.SetKey(ByteString("a$*'FF|~@dsf"));
		ByteString sTemp;
		aMemoryStream >> sTemp;
		aConnectStr.AssignAscii(sTemp.GetBuffer());
	}

	// bei Version 5 wird das Entschluesseln ueber den Parameter am Read geregelt !


	// Existiert ein Filter fuer Tabellen
	bLoaded = TRUE;	// temp, sonst anpassung nicht moeglich
	AdjustConnectStr(TRUE);
	bLoaded = FALSE;// temp

	if (aStorage->GetVersion())
		aStream = OpenStream(*aStorage,String::CreateFromAscii(TITLE),STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE);
	else
		aStream = OpenStream(*aStorage,String::CreateFromAscii(TITLE_0),STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE);

	EXECUTE_READ(ReadDocInfo(&aStream));

	if (aStatus.IsSuccessful())
	{
		aStream = OpenStream(*aStorage,String::CreateFromAscii(FORMATTER), STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE);
		if (aStream.Is())
			aFormatter.Load(*aStream);
		aStatus.Clear();
	}

	bLoaded = aStatus.IsSuccessful();
	return bLoaded;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Open()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	aStatus.Clear();
	if (IsOpen())
		return TRUE;

	START();

	if (!IsLoaded() && !LoadImpl())
	{
		END();
		return FALSE;
	}

	BeginTrans(); // Transaktion starten
	// Versuch schreibend zu oeffen, fuer UpdateTables

	// feststellen ob die DSN als file URL angegeben wurde, wenn ja dann
	// werden Office weite Platzhalter ersetzt und zweitens
	// relative Pfade zum sdb absolutiert
	String aConnectInfo(aConnectStr);

	CommandParser aOptionsParser(aConnectInfo,';','=',TRUE);
	String aTYPE = aOptionsParser.OptionString(String::CreateFromAscii("TYP"));
	if (SdbEnvironment::IsFileServer(aTYPE))
	{
		String aDSN(aOptionsParser.OptionString(String::CreateFromAscii("DSN")));
		INetURLObject aURL;
		aURL.SetSmartProtocol(INET_PROT_FILE);
		aURL.SetURL(aDSN, INetURLObject::ENCODE_ALL);
		if (aURL.GetProtocol() == INET_PROT_FILE)
		{
			// $Inst mu gesetzt sein
			// sehr umstaendlicher weg, um an den Inimanager zu kommen
			// aber ohne Arme(Sfx) keine Kekse
			SfxAppIniManagerProperty aIniManProp;
			GetpApp()->Property(aIniManProp);
			String aNewDSN = aIniManProp.GetIniManager()->SubstPathVars(aDSN);

			// absolutieren zum Datenbankfile
			INetURLObject aDbURL;
			aDbURL.SetSmartProtocol(INET_PROT_FILE);
			aDbURL.SetURL(aName, INetURLObject::ENCODE_ALL);
			INetURLObject aAbsURL;
			if (aDbURL.GetNewAbsURL(aNewDSN, &aAbsURL) && aAbsURL.GetProtocol() == INET_PROT_FILE)
			{
				aNewDSN = aAbsURL.PathToFileName();
				if (DirEntry(aNewDSN).IsValid())
				{
					aOptionsParser.SetOption(String::CreateFromAscii("DSN"), aNewDSN);
					aConnectInfo.Erase();
					aOptionsParser.Compose(aConnectInfo,String());
				}
			}
		}
	}
	pConnection = GetEnvironment()->OpenConnection(aConnectInfo);

	if (!pConnection)				// allgemeiner Fehler
		aStatus = GetEnvironment()->Status();
	else
		pConnection->AddRef();

	bOpen = pConnection != NULL;	   // TabellenUpdate jedesmal, so gewuenscht von dem Testing

	if (bOpen)
	{
		aTableNameList.SetCaseSensitive((pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT);
		aQueryNameList.SetCaseSensitive(TRUE);
		aFormNameList.SetCaseSensitive(TRUE);
		aReportNameList.SetCaseSensitive(TRUE);
		ReadList(dbQuery);
		ReadList(dbForm);
		ReadList(dbReport);
		if (!UpdateTables(NULL,bReadOnly) || bReadOnly)
		{
			ReadList(dbTable);
			EndTrans(FALSE);
		}
		else
		{
			AdjustTableList();

			// Alte Versionen haben nicht unbedingt alle Verzeichnisse (Berichte und Formulare)
			// beim Erzeugen der Datenbank angelegt
			// Neuere erzeugen diese per default siehe Create
			if (aStorage->GetVersion() == 0)
			{
				SdbStorageRef aStor;
				if (!aStorage->IsStorage(String::CreateFromAscii(QUERIES)))
					aStor = OpenStorage(*aStorage, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
				if (!aStorage->IsStorage(String::CreateFromAscii(FORMS)))
					aStor = OpenStorage(*aStorage, String::CreateFromAscii(FORMS), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
				if (!aStorage->IsStorage(String::CreateFromAscii(REPORTS)))
					aStor = OpenStorage(*aStorage, String::CreateFromAscii(REPORTS), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
			}

			// ausserdem muessen nicht alle Versionen den Papierkorb-Storage schon haben
			if (aStorage->HasStorage(String::CreateFromAscii(WASTE),FALSE))
			{
				SdbStorageRef aStor = OpenStorage(*aStorage, String::CreateFromAscii(WASTE), STREAM_STD_READ, 0);
				// Versionen, die zwar den Waste-Storage, aber nicht die vier untergeordneten haben, gibt es nicht !
				DBG_ASSERT(aStor->HasStorage(String::CreateFromAscii(QUERIES),FALSE), "SdbDatabaseImpl::Open : missing the queries storage in my wastebasket !");
				DBG_ASSERT(aStor->HasStorage(String::CreateFromAscii(FORMS),FALSE), "SdbDatabaseImpl::Open : missing the forms storage in my wastebasket !");
				DBG_ASSERT(aStor->HasStorage(String::CreateFromAscii(REPORTS),FALSE), "SdbDatabaseImpl::Open : missing the reports storage in my wastebasket !");
			}
			else
			{
				SdbStorageRef aStor = OpenStorage(*aStorage, String::CreateFromAscii(WASTE), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
				SdbStorageRef aWasteItems;
				aWasteItems = OpenStorage(*aStor, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
				aWasteItems = OpenStorage(*aStor, String::CreateFromAscii(FORMS), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
				aWasteItems = OpenStorage(*aStor, String::CreateFromAscii(REPORTS), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
			}

			EndTrans(TRUE);
		}

		aStatus.Clear();			   // geaenderten Status ignorieren
	}
	else
		EndTrans(FALSE);
	END();
	return bOpen;
}

//------------------------------------------------------------------------------
void SdbDatabaseImpl::Close()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	aStatus.Clear();

	UINT32 i;
	for (i=0; i < aTableNameList.Count(); i++)
		delete aTableNameList.GetObject(i);
	aTableNameList.Clear();

	for (i=0; i < aQueryNameList.Count(); i++)
		delete aQueryNameList.GetObject(i);
	aQueryNameList.Clear();

	for (i=0; i < aFormNameList.Count(); i++)
		delete aFormNameList.GetObject(i);
	aFormNameList.Clear();

	for (i=0; i < aReportNameList.Count(); i++)
		delete aReportNameList.GetObject(i);
	aReportNameList.Clear();

	DBG_ASSERT(pConnection, "keine Verbindung");
	if (pConnection)
	{
		pConnection->ReleaseRef();
		pConnection = NULL;
	}

	nTransCount = 0;
	aTableStor = aQueryStor = aFormStor = aReportStor = aStorage = NULL;
	m_aWasteBasket = NULL;
	bReadOnly = bOpen = FALSE;
	if (bDeleteOnClose)
		aTempFile.Kill();
	bDeleteOnClose = bReadOnly = bOpen = FALSE;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Drop()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	START();

	// storage loslassen
	aStorage.Clear();
	DirEntry aEntry(aName);
	FSysError nErr = aEntry.Kill();

	END();
	if (nErr != ERRCODE_NONE)
	{
		switch (nErr)
		{
			case ERRCODE_IO_NOTEXISTS:
				aStatus.Clear();
				break;
			case ERRCODE_IO_ACCESSDENIED:
				aStatus.SetError(nErr,DATABASE,aName);
				break;
			default:
				aStatus.Set(SDB_STAT_ERROR);
		}
		// wieder festhalten wenn moeglich
		aStorage = new SdbStorage(*this, aName, STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE, 0);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::AlterConnection(const String& rConnectStr, BOOL)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(IsLoaded(), "Datenbank nicht geladen!");

	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	aStatus.Clear();

	// Speichern des Strings im Storage
	BEGINWRITE();

	CommandParser aOptionsParser(rConnectStr,';','=',TRUE);
	String aTableList = aOptionsParser.OptionString(String::CreateFromAscii("TABLE"));

	String aNewConnectStr;
	// wird ein Filter fuer Tabellen verwendet
	if (aOptionsParser.Exists(String::CreateFromAscii("TABLE")))
	{
		// Liste der Tabellen aktualisieren
		UpdateTables(&aTableList);
		aOptionsParser.Compose(aNewConnectStr,String::CreateFromAscii("TABLE"));	// Alles ausser den Sdb-internen Optionen weiterreichen
		aNewConnectStr.EraseLeadingChars(';');
		aNewConnectStr.EraseTrailingChars(';');
		aNewConnectStr += ';';
		aNewConnectStr.AppendAscii(TABLE_FILTER);

		bTableFilter = TRUE;
	}
	else
	{
		if (bTableFilter)			// gab es einen Filter?
		{
			if (UpdateTables(&aTableList))
				AdjustTableList();
		}

		bTableFilter = FALSE;
		aNewConnectStr = rConnectStr;
	}


	if (aStorage->GetVersion() == 2)
	{	// unter dieser ID werde ich nix mehr schreiben, da das fehlerhaft ist
		aStorage->SetVersion(5);
	}

	if (aStorage->GetVersion()==2)
	{
		CommandParser aOptionsParser2(aNewConnectStr,';','=',TRUE);

		if (aOptionsParser2.Exists(String::CreateFromAscii("PWD")))
		{
			ByteString sPassword(aOptionsParser2.OptionString(String::CreateFromAscii("PWD")),gsl_getSystemTextEncoding());

			// Encryption unter Verwendung eines MemoryStreams...
			SvMemoryStream aMemoryStream;
			aMemoryStream.SetKey(ByteString("a$*'FF|~@dsf"));
			aMemoryStream << sPassword;
			sPassword = *(ByteString*)aMemoryStream.GetData();
			aOptionsParser2.SetOption(String::CreateFromAscii("PWD"), String(sPassword,gsl_getSystemTextEncoding()));
			aNewConnectStr.Erase();
			aOptionsParser2.Compose(aNewConnectStr,String());
		}
	}

	if (aStorage->GetVersion()==3)
	{
		SvMemoryStream aMemoryStream;
		aMemoryStream.SetKey(ByteString("a$*'FF|~@dsf"));
		aMemoryStream << ByteString(aNewConnectStr,gsl_getSystemTextEncoding());
		aNewConnectStr = String(*(ByteString*)aMemoryStream.GetData(),gsl_getSystemTextEncoding());
	}

	// bei Version 5 wird das Verschluesseln ueber den Parameter am Write geregelt !


	// Informationen ueberschreiben
	SvStorageStreamRef aStream = OpenStream(*aStorage,String::CreateFromAscii(CONNECTION), STREAM_STD_READWRITE|STREAM_TRUNC);
	EXECUTE_WRITE(Write(*aStream, aNewConnectStr, aStorage->GetVersion() == 5));

	aConnectStr = rConnectStr;
	return EndTrans(TRUE);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::AlterTitle(const String& rTitle)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(IsLoaded(), "Datenbank nicht geladen!");
	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	aStatus.Clear();

	/*//	Anlegen eines neuen Files, das im Namen dem Titel aehnelt
	DirEntry aTmpEntry(DirEntry(aName).GetPath());
	DirEntry aFile;
	aFile.SetBase(rTitle);
	aFile.SetExtension(SDB_DATABASE_EXT);
	aTmpEntry.MakeShortName(aFile.GetName(),FSYS_KIND_FILE);
	String aNewFile	= aTmpEntry.GetFull();

	if (aTmpEntry.GetError() != ERRCODE_NONE)
	{
		return FALSE;
	}
	aTmpEntry.Kill();		// loeschen des Files

	if (!Rename(aNewFile))
		return FALSE;	*/

	// Speichern des Strings im Storage
	BEGINWRITE();

	// Informationen ueberschreiben
	SvStorageStreamRef aStream;
	if (aStorage->GetVersion())
		aStream = OpenStream(*aStorage,String::CreateFromAscii(TITLE),STREAM_STD_READWRITE|STREAM_TRUNC);
	else
		aStream = OpenStream(*aStorage,String::CreateFromAscii(TITLE_0),STREAM_STD_READWRITE|STREAM_TRUNC);

	String aTemp = aTitle;
	aTitle = rTitle;
	if (!WriteDocInfo(*aStream))
	{
		aTitle = aTemp;
		EndTrans(FALSE);
		return FALSE;
	}
	else
		return EndTrans(TRUE);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::AlterFormatter()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(IsLoaded(), "Datenbank nicht geladen!");
	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();


	BEGINWRITE();
	SvStorageStreamRef aStream = OpenStream(*aStorage,String::CreateFromAscii(FORMATTER),STREAM_STD_READWRITE|STREAM_TRUNC);
	if (!aStream.Is() || !aFormatter.Save(*aStream))
	{
		EndTrans(FALSE);
		return FALSE;
	}
	else
		return EndTrans(TRUE);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::GetRelationLayout(SvStream& rTarget)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(IsLoaded(), "Datenbank nicht geladen!");
	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	else
		aStatus.Clear();

	BEGINREAD();
	BeginTrans();
	SvStorageStreamRef aStream = OpenStream(*aStorage,String::CreateFromAscii(RELATIONS),STREAM_STD_READ);
	if (!aStream.Is())
	{
		EndTrans(FALSE);
		return FALSE;
	}
	else
	{
		ULONG nError = CopyTo(rTarget, *aStream); // Stream uebergeben
		return EndTrans(FALSE) && (nError == ERRCODE_NONE);
	}
}
//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::AlterRelationLayout(SvStream& rToCopy)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(IsLoaded(), "Datenbank nicht geladen!");
	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	else
		aStatus.Clear();

	BEGINWRITE();
	SvStorageStreamRef aStream = OpenStream(*aStorage,String::CreateFromAscii(RELATIONS),STREAM_STD_READWRITE|STREAM_TRUNC);
	if (!aStream.Is())
	{
		EndTrans(FALSE);
		return FALSE;
	}
	else
	{
		ULONG nError = CopyTo(*aStream, rToCopy);// Stream uebergeben
		return EndTrans(TRUE) && nError == ERRCODE_NONE;
	}
}

//------------------------------------------------------------------------------
String SdbDatabaseImpl::GetFullTableName(const String& rName) const
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	if (pConnection)	 // ziemlich aufwendig
		return SdbTable::GetFullName(*pConnection, rName);
	else
		return rName;
}

//------------------------------------------------------------------------------
SdbHashString* SdbDatabaseImpl::Find(const ByteString& rName, const SdbHashStringList& rList) const
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	return rList.Find(rName);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Read(SvStorageStream& rStream, String& rValue, BOOL bDecrypt)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	aStatus.Clear();

	ByteString aTemp;
	// Entschluesseln VOR dem Konvertieren
	if (bDecrypt)
	{
		// den verschluesselten Teil in einen MemoryStream
		USHORT nSize;
		rStream >> nSize;
		SvMemoryStream aMemoryStream(nSize);
		char cBuffer[1024];
		USHORT nDone = 0;
		while (nDone < nSize)
		{
			USHORT nThisRound = (nSize - nDone < sizeof(cBuffer)) ? (nSize - nDone) : sizeof(cBuffer);
			rStream.Read(cBuffer, nThisRound);
			aMemoryStream.Write(cBuffer, nThisRound);
			nDone += nThisRound;
		}
		aMemoryStream.Flush();

		// entschluesseln
		aMemoryStream.SetKey(ByteString("a$*'FF|~@dsf"));
		aMemoryStream.Seek(STREAM_SEEK_TO_BEGIN);
		aMemoryStream >> aTemp;
	}
	else
		rStream >> aTemp;

	aTemp.Convert(RTL_TEXTENCODING_MS_1252, gsl_getSystemTextEncoding());
	rValue.AssignAscii(aTemp.GetBuffer());
	return !aStatus.SetError(rStream.GetErrorCode(), DATABASE, aName);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Write(SvStorageStream& rStream, const String& rValue, BOOL bEncrypt)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	aStatus.Clear();
	ByteString aTmp(rValue,gsl_getSystemTextEncoding());
	aTmp.Convert(gsl_getSystemTextEncoding(),RTL_TEXTENCODING_MS_1252);

	if (bEncrypt)
	{	// Verschluesseln NACH dem Konvertieren
		SvMemoryStream aMemoryStream;
		aMemoryStream.SetKey(ByteString("a$*'FF|~@dsf"));
		aMemoryStream << aTmp;

		// den verschluesselten Buffer duerfen wir jetzt nicht als String behandeln, denn der kann durchaus
		// irgendwo Nullen (also das String-Ende-Zeichen) enthalten _vor_ dem eigentlichen Ende
		// also diese Daten "raw" rausschreiben
		// (der extremste Fall war ein ploetzlich leerer String (nach dem Verschluesseln), nur weil das erste
		// Zeichen zufaellig zu einer 0 geworden war)
		// FS - 18.05.99
		rStream << (USHORT)aMemoryStream.GetSize();
			// wg. Kompatibilitaet zu aelteren Versionen, die das als String geschrieben haben, der cast auf USHORT,
			// denn die Laengenangabe eines Strings ist ein USHORT
		rStream.Write(aMemoryStream.GetData(), (USHORT)aMemoryStream.GetSize());
	}
	else
		rStream << aTmp;

	rStream.Flush();							// sonst werden die Daten nicht geschrieben
	return !aStatus.SetError(rStream.GetErrorCode(), DATABASE, aName);
}


//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::ReadDocInfo(SvStorageStream* pStream)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	aStatus.Clear();

	if (pStream)
	{
		ByteString aValue;
		pStream->ReadLine(aValue);
		aValue.Convert(RTL_TEXTENCODING_MS_1252, gsl_getSystemTextEncoding());

		if (aValue.GetTokenCount(';') == 2)				// Lesen des Titels
		{
			aTitle.AssignAscii(aValue.GetToken(1,';').GetBuffer());
			return TRUE;
		}
	}
	aTitle = DirEntry(aName).GetBase();			   // Wird kein Title gefunden, nehmen wir den Filenamen
	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::WriteDocInfo(SvStorageStream& rStream)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	aStatus.Clear();
	ByteString aTmp;

	aTmp = VERSION;
	aTmp += '=';
	aTmp += VER;
	aTmp += ';';
	aTmp += ByteString(aTitle,gsl_getSystemTextEncoding());
	aTmp.Convert(gsl_getSystemTextEncoding(),RTL_TEXTENCODING_MS_1252);
	rStream.WriteLine(aTmp);
	rStream.Flush();							// sonst werden die Daten nicht geschrieben
	return !aStatus.SetError(rStream.GetErrorCode(),DATABASE,aName);
}


//------------------------------------------------------------------------------
SdbStorage* SdbDatabaseImpl::OpenStorage(SdbStorage& rMain,
										 const String& rName,
										 StreamMode eMode,
										 StorageMode eStorMode)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	aStatus.Clear();
	rMain.ResetError();
	SdbFileType eType;

	// welcher Kontext
	if (&rMain == &aTableStor)
		eType = TABLE;
	else if(&rMain == &aQueryStor)
		eType = QUERY;
	else if (&rMain == &aFormStor)
		eType = FORM;
	else if (&rMain == &aReportStor)
		eType = REPORT;
	else
		eType = DATABASE;

	return rMain.OpenStorage(eType,rName,eMode,eStorMode);
}

//------------------------------------------------------------------------------
SdbStorage* SdbDatabaseImpl::OpenStorage(DBObject eObj, BOOL bReadOnly)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	SdbStorage* pStor = NULL;
	aStatus.Clear();

	switch (eObj)
	{
		case dbTable:
		case dbView:
			pStor = &aTableStor; // leider & ueberladen
			break;
		case dbQuery:
			pStor = &aQueryStor;
			break;
		case dbForm:
			pStor = &aFormStor;
			break;
		case dbReport:
			pStor = &aReportStor;
			break;
		default:
			DBG_ERROR("Falsches Object!");
			return NULL;
	}

	if (pStor)
		pStor->AddRef();
	else if (BeginTrans())
	{
		StorageMode eStorMode = 0;
		StreamMode eMode = STREAM_READ | STREAM_SHARE_DENYNONE;

		if (!bReadOnly)
			eMode |= STREAM_WRITE;

		switch (eObj)
		{
			case dbTable:
			case dbView:
				aTableStor = OpenStorage(*aStorage, String::CreateFromAscii(TABLES), eMode, eStorMode);
				pStor = &aTableStor;
				break;
			case dbQuery:
				aQueryStor = OpenStorage(*aStorage, String::CreateFromAscii(QUERIES), eMode, eStorMode);
				pStor = &aQueryStor;
				break;
			case dbForm:
				aFormStor = OpenStorage(*aStorage, String::CreateFromAscii(FORMS), eMode, eStorMode);
				pStor = &aFormStor;
				break;
			case dbReport:
				aReportStor = OpenStorage(*aStorage, String::CreateFromAscii(REPORTS), eMode, eStorMode);
				pStor = &aReportStor;
				break;
		}
	}
	return pStor;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::ReleaseStorage(DBObject eObj, BOOL bCommit)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	BOOL bResult = TRUE;
	BOOL bEndTrans = FALSE;
	SdbStorage* pStor = NULL;
	aStatus.Clear();

	switch (eObj)
	{
		case dbTable:
		case dbView:
			pStor = &(aTableStor); // leider & ueberladen
			break;
		case dbQuery:
			pStor = &(aQueryStor);
			break;
		case dbForm:
			pStor = &(aFormStor);
			break;
		case dbReport:
			pStor = &(aReportStor);
			break;
		default:
			DBG_ERROR("Falsches Object!");
			return FALSE;
	}

	BOOL bReadOnly = !pStor || pStor->IsReadOnly();
	if (!pStor)
		bResult = TRUE;
	else if (pStor->GetRefCount() > 1)
		pStor->ReleaseRef();
	else
	{
		switch (eObj)
		{
			case dbTable:
			case dbView:
				aTableStor.Clear();
			break;
			case dbQuery:
				aQueryStor.Clear();
			break;
			case dbForm:
				aFormStor.Clear();
			break;
			case dbReport:
				aReportStor.Clear();
			break;
		}
		bEndTrans = TRUE;
	}

	if (bEndTrans)
	{
		if (bReadOnly && nTransCount)
			nTransCount--;
		else
			bResult = EndTrans(bCommit);
	}
	return bResult;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::RenameStorage(const String& aOldName, const String& aNewName, DBObject eObj)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	SdbStorage* pStor = OpenStorage(eObj, FALSE);

	if (!pStor)
		return FALSE;
	else
	{
		BOOL bStor = FALSE;
		SdbFileType eType;
		SdbHashStringList* pList = NULL;
		switch (eObj)										// wenn der Name schon existiert -> Fehler
		{
			case dbTable:
			case dbView:
				eType = TABLE;
				bStor = TRUE;
				pList = &aTableNameList; break;
			case dbQuery:
				eType = QUERY;
				bStor = TRUE;
				pList = &aQueryNameList; break;
			case dbForm:
				eType = FORM;
				pList = &aFormNameList; break;
			default:
				eType = REPORT;
				pList = &aReportNameList; break;
		}

		if (!pStor->Rename(eType, aOldName, aNewName, bStor))
			ReleaseStorage(eObj,FALSE);
		else if (ReleaseStorage(eObj))
		{
			SdbHashString* pString = Find(aOldName, *pList);
			delete pList->Remove(pString);
			pList->Insert(new SdbHashString(ByteString(aNewName,gsl_getSystemTextEncoding())), LIST_APPEND);
		}
		return aStatus.IsSuccessful();
	}
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::RemoveStorage(const String& rName, DBObject eObj)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	SdbStorage* pStor = OpenStorage(eObj, FALSE);

	if (!pStor)
		return FALSE;
	else
	{
		BOOL bStor = FALSE;
		SdbFileType eType;
		SdbHashStringList* pList = NULL;
		switch (eObj)										// wenn der Name schon existiert -> Fehler
		{
			case dbTable:
			case dbView:
				eType = TABLE;
				bStor = TRUE;
				pList = &aTableNameList; break;
			case dbQuery:
				eType = QUERY;
				bStor = TRUE;
				pList = &aQueryNameList; break;
			case dbForm:
				eType = FORM; pList = &aFormNameList; break;
			default:
				eType = REPORT;
				pList = &aReportNameList; break;
		}

		if (!pStor->Remove(eType, rName,bStor))
			ReleaseStorage(eObj,FALSE);
		else if (ReleaseStorage(eObj))
		{
			SdbHashString* pString = Find(rName, *pList);
			pList->Remove(pString);
			delete pString;

			if ((eObj == dbTable || eObj == dbView) && pConnection)
				AdjustTableList();
		}
		return aStatus.IsSuccessful();
	}
}

//------------------------------------------------------------------------------
String SdbDatabaseImpl::CategoryFolderName(DBObject eObj) const
{
	switch (eObj)
	{
		case dbTable	: return String::CreateFromAscii(TABLES);	break;
		case dbQuery	: return String::CreateFromAscii(QUERIES);	break;
		case dbForm		: return String::CreateFromAscii(FORMS);		break;
		case dbReport	: return String::CreateFromAscii(REPORTS);	break;
		default : DBG_ERROR("SdbDatabaseImpl::CategoryFolderName : invalid object type !");
	}
	return String();
}

//------------------------------------------------------------------------------
void  SdbDatabaseImpl::OpenWasteStorage()
{
	if (m_aWasteBasket.Is())
		return;

	StreamMode eMode = STREAM_READ | STREAM_SHARE_DENYNONE | (bReadOnly ? 0 : STREAM_WRITE);
	m_aWasteBasket = OpenStorage(*aStorage, String::CreateFromAscii(WASTE), eMode, 0);
}

//------------------------------------------------------------------------------
SdbStorageRef SdbDatabaseImpl::OpenWasteObjectStorage(DBObject eObj, const String& rName)
{
	// den Papierkorb an sich
	OpenWasteStorage();
	DBG_ASSERT(m_aWasteBasket.Is(), "SdbDatabaseImpl::OpenWasteObjectStorage : have no wastebasket !");
		// sollte nicht passieren, da im Create oder Open angelegt

	// dort den Storage fuer die Objekt-Kategorie
	String strCatName = CategoryFolderName(eObj);
	SdbStorageRef aCategoryStorage = OpenStorage(*m_aWasteBasket, strCatName, SDB_STD_STREAM_MODE, 0);
	DBG_ASSERT(aCategoryStorage.Is(), "SdbDatabaseImpl::OpenWasteObjectStorage : a category storage is missing in the wastebasket !");
		// wenn der Papierkorb selber existiert, dann sollte das auch fuer die Kategorien gelten
	if (!aCategoryStorage.Is())
		return NULL;

	return OpenStorage(*aCategoryStorage, rName, SDB_STD_STREAM_MODE, 0);
}

//------------------------------------------------------------------------------
SvStorageStream* SdbDatabaseImpl::OpenStream(SdbStorage& rMain, const String& rName, StreamMode eMode)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	aStatus.Clear();
	rMain.ResetError();

	return rMain.OpenStream(DATABASE,rName, eMode);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::ReadList(DBObject eObj)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	//DBG_ASSERT(aStorage.Is(),"Datenbankdatei nicht geoeffnet");
	SdbStorage* pStor = OpenStorage(eObj, TRUE);
	if (!pStor)
		return FALSE;

	SdbHashStringList* pList;
	switch (eObj)
	{
		case dbTable:
		case dbView:
			pList = &aTableNameList;
			ReadList(*pStor,*pList,TRUE);
			break;
		case dbQuery:
			pList = &aQueryNameList;
			ReadList(*pStor,*pList,TRUE);
			break;
		case dbForm:
			pList = &aFormNameList;
			ReadList(*pStor,*pList,FALSE);
			break;
		case dbReport:
			pList = &aReportNameList;
			ReadList(*pStor,*pList,FALSE);
			break;
	}
	ReleaseStorage(eObj, FALSE);
	return TRUE;
}

//------------------------------------------------------------------------------
void SdbDatabaseImpl::ReadList(SdbStorage& rStor, SdbHashStringList& rList, BOOL bStorage,BOOL _bCaseSensitiv)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	if (bStorage)
		rStor.GetStorageNameList(rList,_bCaseSensitiv);
	else
		rStor.GetStreamNameList(rList);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Rename(const String& aNewName)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	// To rename the database file, the filename must be known
	// And it is known if the database is Loaded
	DBG_ASSERT(IsLoaded(),"Datenbank nicht geladen");
	DBG_ASSERT(aStorage.Is(), "database file is not open during rename");
	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	START();
	aStatus.Clear();
	aStorage.Clear();

	DirEntry aNewEntry(aNewName);
	aNewEntry.SetExtension(String::CreateFromAscii(SDB_DATABASE_EXT));

	DirEntry aOldEntry(aName);
	if (aStatus.SetError(aOldEntry.MoveTo(aNewEntry),DATABASE,aName))
	{
		END();
		if (bReadOnly)
			aStorage = new SdbStorage(*this, aName, STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE, 0);
		else
			aStorage = new SdbStorage(*this, aName, STREAM_READWRITE | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE, STORAGE_TRANSACTED);  // Lesen/Schreibend oeffnen
		return FALSE;
	}

	// locking der Datei
	aName = aNewName;
	aStorage = new SdbStorage(*this, aName, STREAM_READWRITE | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE, STORAGE_TRANSACTED);  // Lesen/Schreibend oeffnen
	bReadOnly = FALSE;

	END();
	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::Create(const String& rName, const String& rTitle, const String& rConnectStr)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(!IsOpen(),"Datenbank bereits geoeffnet");
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	aStatus.Clear();
	START();

	// extension dranhaengen
	DirEntry aEntry(rName);
	aEntry.SetExtension(String::CreateFromAscii(SDB_DATABASE_EXT));

	if (aEntry.Exists())
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,DATABASE,rName);
		END();
		return FALSE;
	}

	StreamMode eMode = STREAM_READWRITE | STREAM_SHARE_DENYNONE;
	StorageMode eStorMode = STORAGE_TRANSACTED | STORAGE_FAILIFTHERE;

	aStorage = new SdbStorage(*this, aEntry.GetFull(),eMode,eStorMode);
	BOOL bResult = aStatus.IsSuccessful();
	if (bResult)
	{
		// Erzeugen der Storages fuer Tabellen, Reports, Abfrage, Formulare
		SdbStorageRef aStor = OpenStorage(*aStorage, String::CreateFromAscii(TABLES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
		aStor = OpenStorage(*aStorage, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
		aStor = OpenStorage(*aStorage, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
		aStor = OpenStorage(*aStorage, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);

		// dito den Papierkorb (in dem es allerdings keine Tabellen gibt)
		aStor = OpenStorage(*aStorage, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
		SdbStorageRef aWasteItems;
		aWasteItems = OpenStorage(*aStor, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
		aWasteItems = OpenStorage(*aStor, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);
		aWasteItems = OpenStorage(*aStor, String::CreateFromAscii(QUERIES), STREAM_STD_WRITE, STORAGE_FAILIFTHERE);

		//Informationen ueberschreiben
		SvStorageStreamRef aStream = OpenStream(*aStorage,String::CreateFromAscii(CONNECTION),STREAM_STD_READWRITE);
		aStream.Clear();	// Anlegen des Streams

		// build in for sdbapi
		if (rConnectStr.Len())
		{		
			bLoaded = TRUE;		// temp gesetzt
			bResult = AlterConnection(rConnectStr);
			bLoaded = FALSE;
		}

		if (bResult)
		{
			if (aStorage->GetVersion())
				aStream = OpenStream(*aStorage,String::CreateFromAscii(TITLE),STREAM_STD_READWRITE);
			else
				aStream = OpenStream(*aStorage,String::CreateFromAscii(TITLE_0),STREAM_STD_READWRITE);
			aTitle = rTitle;
			bResult = WriteDocInfo(*aStream);
		}
	}

	if (!bResult)
	{
		aStorage = NULL;
		DirEntry(rName).Kill();
	}
	else
	{
		aName = rName;
		aConnectStr = rConnectStr;
		bLoaded = TRUE;
		aStorage->Commit();					// Jetzt erst abspeichern
		aStorage = NULL;
	}
	END();
	return bResult;
}

//------------------------------------------------------------------------------
// Abgleich der Tabellenliste von Connection und Datenbank
// Wichtig zum Einlesen von Relationen
void SdbDatabaseImpl::AdjustTableList()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	if (!pConnection) return;


	// Im Augenblick stehen in rConnTableNameList ALLE Tabellen der Datenbank, inklusive der, die
	// ich ausblenden will. Genau diese muss ich in aMaskedTables packen.
	while (aMaskedTables.Count())
		aMaskedTables.Remove(0);

	DBG_ASSERT(aTableNameList.Count() <= pConnection->pTableNames->size(),
		"SdbDatabaseImpl::AdjustTableList : Tabellenlisten inkonsistent !");
		// eine Verletzung dieser Bedingung wuerde bedeuten, dass mir selber mehr Tabellen bekannt sind als der Connection
#if DEBUG || DBG_UTIL
	USHORT nOriginalOverallCount = pConnection->pTableNames->size();
#endif // DEBUG || DBG_UTIL

	USHORT nOverallCount = pConnection->pTableNames->size();

	SdbStringStlMap::iterator iter = pConnection->pTableNames->begin();

	while(iter != pConnection->pTableNames->end())
	{
		if (!aTableNameList.Find(ByteString((*iter).first,gsl_getSystemTextEncoding())))
		{	// die Connection kennt die Tabelle, aber ich nicht -> sie soll ausgeblendet sein
			String* pNew = new String((*iter).first);
			aMaskedTables.Insert(pNew, aMaskedTables.Count());
				// das 'new String ...' haette in das Insert gekonnt ... aber leider hat Visual 6 dann einen 'internal compiler error'
			pConnection->pTableNames->erase(iter);
			iter = pConnection->pTableNames->begin();
		}
		else
			iter++;
	}

	DBG_ASSERT(aTableNameList.Count() + aMaskedTables.Count() <= nOriginalOverallCount,
		"SdbDatabaseImpl::AdjustTableList : Tabellenlisten inkonsistent !");
		// eine Verletzung dieser Bedingung (das kann nur ein > sein) wuerde bedeuten, dass aTableNameList Tabellen enthielt,
		// die der Connection NICHT bekannt waren
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::UpdateTables(const String* pTableList,BOOL bReadOnly)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(IsLoaded(),"Datenbank nicht geladen");
	if (!IsLoaded())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	START();
	aStatus.Clear();

	// wird eine Liste von Tabellen uebergeben, erfolgt kein Abgleich mit
	// der Connection
	if (!pTableList && !IsOpen() && !Open())
	{
		END();
		return FALSE;
	}

	// Oeffnen des TabellenStorage, vor ReadList!!!
	if (!OpenStorage(dbTable,bReadOnly))
	{
		END();
		return FALSE;
	}

	ReadList(*aTableStor, aTableNameList, TRUE,pConnection ? (pConnection->GetIdentifierCase() & SDB_IC_OBJECT): TRUE); // lesen der aktuellen Liste, nicht ReadList(dbTable) verwenden !!!

	SdbHashStringList aTestList;				 // erstellen der neuen Liste
	BOOL bInsertTables = TRUE;
	aTestList.SetCaseSensitive(aTableNameList.IsCaseSensitive());


	if (pTableList)
	{
		xub_StrLen nIndex = 0;
		for (String aToken = pTableList->GetToken(0,DEFAULT_LIST_SEPARATOR,nIndex);
			 aToken.Len();
			 aToken = pTableList->GetToken(0,DEFAULT_LIST_SEPARATOR,nIndex))
			aTestList.Insert(new SdbHashString(aToken), LIST_APPEND);
	}
	else
	{
		// Wenn ein Filter fuer Tabellen gesetzt ist, dann werden keine neuen Tabellen
		// in die Datenbank uebernommen, da sie ja nicht zu Filter gehoeren
		// Tabellen, die jedoch nicht mehr existieren, wrden gelscht

		bInsertTables = !bTableFilter;
		USHORT nCount = pConnection->GetTableCount(),
			   i	  = 0;
		while (i < nCount)
			aTestList.Insert(new SdbHashString(pConnection->GetTableName(i++)), LIST_APPEND);
	}

	// Sequentielle Zugriffe, sehr langsam
	// 1.Loeschen der Tabellen, die nicht mehr vorhanden sind
	ULONG i;
	for (i = aTableNameList.Count(); i > 0; i--)
	{
		SdbHashString* pName = aTableNameList.GetObject(i-1);
		if (!Find(pName->GetString(), aTestList))
		{
			if(!bReadOnly)
				aTableStor->Remove(String(pName->GetString(),gsl_getSystemTextEncoding()),TRUE);
			aTableNameList.Remove(i-1);
			delete pName;
		}
	}

	if(!bReadOnly)
	{
		// 2. Inserten der neuen Tabellen, allerdings nur wenn noch
		if (bInsertTables)
		{
			for (i = aTestList.Count(); i > 0; i--)
			{
				SdbHashString* pName = aTestList.GetObject(i-1);
				if (!Find(pName->GetString(), aTableNameList))
				{
					SdbStorageRef aStor = OpenStorage(*aTableStor, String::CreateFromAscii(pName->GetString().GetBuffer()), STREAM_STD_READWRITE | STREAM_TRUNC);
				}
				aTestList.Remove(i-1);
				delete pName;
			}
		}
	}
	ReadList(*aTableStor,aTableNameList,TRUE,pConnection ? (pConnection->GetIdentifierCase() & SDB_IC_OBJECT): TRUE); // aktualisieren
	// Storage wieder freigeben
	BOOL bResult = ReleaseStorage(dbTable,TRUE);
	END();
	return bResult;
}

//------------------------------------------------------------------------------
void SdbDatabaseImpl::Add(const String& rName, DBObject eObj)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	// nur Tabellen und Queries werden gemerkt
	switch (eObj)
	{
		case dbTable:
		case dbView:
			aTableNameList.Insert(new SdbHashString(rName),LIST_APPEND);
			AdjustConnectStr(FALSE);
			break;
		case dbQuery:
			aQueryNameList.Insert(new SdbHashString(rName),LIST_APPEND);
			break;
		case dbForm:
			aFormNameList.Insert(new SdbHashString(rName),LIST_APPEND);
			break;
		case dbReport:
			aReportNameList.Insert(new SdbHashString(rName),LIST_APPEND);
			break;
	}
}

//------------------------------------------------------------------------------
void SdbDatabaseImpl::Refresh(DBObject _eObj)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	switch (_eObj)
	{
		case dbTable:
		case dbView:
			UpdateTables(NULL,bReadOnly);
			if (!bReadOnly)
			{
				if (pConnection)
					AdjustTableList();
				break;
			}
		case dbQuery:
		case dbForm:
		case dbReport:
			ReadList(_eObj);
			break;
	}
}

//------------------------------------------------------------------------------
void SdbDatabaseImpl::Drop(const String& rName, DBObject eObj)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	// nur Tabellen und Queries werden gemerkt
	switch (eObj)
	{
		case dbTable:
		case dbView:
			delete aTableNameList.Remove(Find(rName, aTableNameList));
			break;
		case dbQuery:
			delete aQueryNameList.Remove(Find(rName, aQueryNameList));
			break;
		case dbForm:
			delete aFormNameList.Remove(Find(rName, aFormNameList));
			break;
		case dbReport:
			delete aReportNameList.Remove(Find(rName, aReportNameList));
			break;
	}
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::BeginTrans()
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	nTransCount++;
	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::EndTrans(BOOL bCommit)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	DBG_ASSERT(aStorage.Is(), "Fehler in der Transaction");
	DBG_ASSERT(nTransCount > 0, "Fehler beim Setzen von Transactionsklammern");

	if (!bCommit && nTransCount)
	{
		if (aStorage->IsReadOnly())
		{
			--nTransCount;
			return TRUE;
		}
		else // abbruch
		{
			aStorage->Revert();
			nTransCount = 0;
			return !aStatus.SetError(*aStorage->Impl(),DATABASE);
		}
	}
	else if (nTransCount > 0 && --nTransCount == 0)
	{
		aStorage->Commit();
		return !aStatus.SetError(*aStorage->Impl(),DATABASE);
	}
	else
		return TRUE;
}

//------------------------------------------------------------------------------
SvStorageStream* SdbDatabaseImpl::OpenContent(SdbStorage& rMain, StreamMode eMode)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	return OpenStream(rMain, String::CreateFromAscii(CONTENT), eMode);
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::HasObject(DBObject eObj, const String& rName) const
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	BOOL bResult = FALSE;
	switch (eObj)
	{
		case dbTable:
		case dbView:
		{
			String aSearchName(GetFullTableName(rName));
			bResult = Find(aSearchName, aTableNameList) != NULL;
		}	break;
		case dbQuery:
			bResult = Find(rName, aQueryNameList) != NULL;
			break;
		case dbForm:
			bResult = Find(rName, aFormNameList) != NULL;
			break;
		case dbReport:
			bResult = Find(rName, aReportNameList) != NULL;
			break;
	}
	return bResult;
}

#if (SUPD>512) || PRIV_DEBUG
//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::HasObjectDeleted(DBObject _eObj, const String& _rName)
{
	if (_eObj == dbTable)
		return FALSE;
		// da ich fuer Tabellen das reversible Loeschen nicht zulasse ....

	OpenWasteStorage();

	// Source- und Destination-Storage oeffnen
	String strCatStorageName;
	switch (_eObj)
	{
		case dbQuery	: strCatStorageName = String::CreateFromAscii(QUERIES);	break;
		case dbForm		: strCatStorageName = String::CreateFromAscii(QUERIES);		break;
		case dbReport	: strCatStorageName = String::CreateFromAscii(QUERIES);	break;
		default : DBG_ERROR("SdbDatabaseImpl::HasObjectDeleted : invalid object type !"); return FALSE;
	}
	SdbStorageRef aCatStorage = OpenStorage(*m_aWasteBasket, strCatStorageName, STREAM_READ | STREAM_SHARE_DENYNONE, 0);
	if (!aCatStorage.Is())
	{
		DBG_ERROR("SdbDatabaseImpl::HasObjectDeleted : could not open the wastebasket's category storage !");
			// eigentlich sollte das Ding im Create oder Open angelegt worden sein, wenn es nicht schon existierte
		return FALSE;
	}

	SdbStorageRef aObjectStorage = OpenStorage(*aCatStorage, _rName, STREAM_READ | STREAM_SHARE_DENYNONE, 0);
	if (!aObjectStorage.Is())
		// es gibt keinen SubFolder mit dem Namen des Objektes, also auch keine geloeschten solchen Objekte
		return FALSE;

	return (_eObj == dbQuery) ? aObjectStorage->HasStorages() : aObjectStorage->HasStreams();
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::DeleteObject(DBObject _eObj, const String& _rName)
{
	// die Folder-Struktur in meinem Haupt-Storage (soweit sie hier interesant ist, das ist nicht vollstaendig) :
	// (in Klammern steht jeweils der Folder-Name, ohne Beachtung des Renaming, das der SdbStorage intern fuer Folder vornimmt)
	// Datenbank.SDB
	// |
	// +- Tabellen	(TABLES)
	// |
	// +- Formulare (FORMS)
	// |
	// +- Berichte	(REPORTS)
	// |
	// +- Abfragen	(QUERIES)
	// |
	// +- Papierkorb	(WASTE)
	//    |
	//    +- Formulare	(FORMS)
	//    |  |
	//    |  +- Formular_Name1
	//    |  |  |
	//    |  |  +- Objekt_Name1_Instanz1	(1)
	//    |  |  |
	//    |  |  +- Objekt_Name1_Instanz2	(2)
	//    |  |  .
	//    |  |  .
	//    |  |  .
	//    |  |  +- Objekt_Name1_InstanzX1	(X1)
	//    |  |
	//    |  +- FormularName2
	//    |  |	|
	//    |  .	.
	//    |  .	.
	//    |  |  +- Objekt_Name2_InstanzX2	(X2)
	//    |  .
	//    |  .
	//    |  +- FormularNameY
	//    |     |
	//    |     .
	//    |     .
	//    |     +- Objekt_NameY_InstanzXY	(XY)
	//    |
	//    +- Berichte	(REPORTS)
	//    |  |
	//    |  +- ....
	//    |  .
	//    |  .
	//    |
	//    +- Abfragen	(QUERIES)
	//       |
	//       +- ....
	//       .
	//       .
	//
	//
	// Innerhalb eines Dokument-Types gibt es also fuer jeden Dokument-Namen einen Folder, der wiederum die einzelnen
	// geloeschten Objekte enthaelt. Damit wird sichergestellt, dass man auch mehrere Dokumente des selben Typs mit dem selben
	// Namen loeschen kann (also ein Formular 'xyz' loeschen, ein anderes mit eben diesem Namen anlegen, auch loeschen).
	//
	// Ausserdem bedeutet Loeschen damit nur noch das Verschieben eines Objektes in der Storage-Hierarchie (nachdem man ein klein
	// wenig Verwaltungsaufwand getrieben hat ;))
	//
	// So. Das war die Struktur, die ich gern haette. Dummerweise unterstuetzt die UI-Seite im Augenblick das Loeschen von
	// gleichnamigen Objekten nur insofern, als jedesmal die vorherige Instanz weggeworfen wird. Also muss ich das auch so tun.
	//

	aStatus.Clear();

	DBG_ASSERT(!bReadOnly, "SdbDatabaseImpl::DeleteObject : can't delete when in read only mode !");
	DBG_ASSERT(_eObj != dbTable, "SdbDatabaseImpl::DeleteObject : tables can only be (irreversible) dropped, not (reversible) deleted !");
	if (bReadOnly || (_eObj == dbTable))
		return FALSE;
	DBG_ASSERT(HasObject(_eObj, _rName), "SdbDatabaseImpl::DeleteObject : why delete a nonexistent object ?");

	// ---------------------------------------
	// Source- und Destination-Storage oeffnen
	SdbStorageRef aDestinationStorage = OpenWasteObjectStorage(_eObj, _rName);
	if (!aDestinationStorage.Is())
	{
		DBG_ERROR("SdbDatabaseImpl::DeleteObject : something went wrong ... but this shouldn't happen !");
			// eigentlich denke ich, dass das alles haette klappen sollen ...
		return FALSE;
	}

	SdbStorageRef aSourceStorage = OpenStorage(_eObj, FALSE);
	DBG_ASSERT(aSourceStorage.Is(), "SdbDatabaseImpl::DeleteObject : could not open the source storage !");

	BOOL bObjectIsStorage = _eObj == dbQuery;

	// ------------------------------------------
	// da wir i.A. nur das Halten EINER Dokument-Instance mit einem bestimmten Namen unterstuetzen, muessen alle anderen (alten)
	// also wech
	// TODO : das sollte irgendwann mal rausfliegen
	SdbHashStringList aOldInstances;
	if (bObjectIsStorage)
		aDestinationStorage->GetStorageNameList(aOldInstances);
	else
		aDestinationStorage->GetStreamNameList(aOldInstances);
	for (ULONG j=0; j<aOldInstances.Count(); ++j)
	{
		SdbHashString* pCurrent = aOldInstances.GetObject(j);
		if (bObjectIsStorage)
			aDestinationStorage->RemoveStorage(String::CreateFromAscii(pCurrent->GetString().GetBuffer()),_eObj==dbTable);
		else
			aDestinationStorage->Impl()->Remove(String::CreateFromAscii(pCurrent->GetString().GetBuffer()));
		delete pCurrent;
	}
	aOldInstances.Clear();


	// ------------------------------------------
	// ein neuer (eindeutiger) Name fuer das neue Objekt
	String strNewObjectName(String::CreateFromAscii("deleted_"));
	if (!aDestinationStorage->GetUniqueObjectName(strNewObjectName,_eObj==dbTable))
	{
		// ups, kein freier Name mehr ... dann lassen wir es
		aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("01000"), aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_TO_MUCH_FILES_DELETED))),
			0, String() );
		ReleaseStorage(_eObj, FALSE);
		return FALSE;
	}

	// ------------------------------------------
	// das fraglich Objekt verschieben
	SdbHashStringList* pNameList = NULL;
	BOOL bSuccess = FALSE;
	if (bObjectIsStorage)
	{
		DBG_ASSERT(aSourceStorage->HasStorage(_rName,_eObj==dbTable), "SdbDatabaseImpl::DeleteObject : query objects should always be storages !");
			// wenn es kein Storage waere, muesste ich aSourceStorage->Impl()->MoveTo benutzen
		bSuccess = aSourceStorage->MoveStorage(_rName, *aDestinationStorage, strNewObjectName,_eObj==dbTable);
		pNameList = &aQueryNameList;
	}
	else
	{
		DBG_ASSERT(aSourceStorage->Impl()->IsStream(_rName), "SdbDatabaseImpl::DeleteObject : form/report objects should always be streams !");
			// wenn es kein Stream waere, muesste ich aSourceStorage->MoveStorage, nicht das unten stehende MoveTo benutzen
		bSuccess = aSourceStorage->Impl()->MoveTo(_rName, aDestinationStorage->Impl(), strNewObjectName);
		pNameList = (_eObj == dbForm) ? &aFormNameList : &aReportNameList;
	}

	// Storages wegwerfen
	aSourceStorage = NULL;
	aDestinationStorage = NULL;
	ReleaseStorage(_eObj, bSuccess);
		// ReleaseStorage als letztes, damit es meine Member (aFormStor ...) sauber aufraeumen kann !

	// ------------------------------------------
	// Auswertung
	if (bSuccess)
	{
#if DEBUG || DBG_UTIL
		SdbStorage* pSourceStorage = OpenStorage(_eObj, TRUE);
		if (bObjectIsStorage)
		{
			DBG_ASSERT(!pSourceStorage->HasStorage(_rName,_eObj==dbTable), "SdbDatabaseImpl::DeleteObject : MoveStorage said 'succes' but the source is still there ?");
		}
		else
		{
			DBG_ASSERT(!pSourceStorage->Impl()->IsStream(_rName), "SdbDatabaseImpl::DeleteObject : MoveTo said 'succes' but the source is still there ?");
		}
		ReleaseStorage(_eObj);
#endif // DEBUG || DBG_UTIL

		// den Namen aus meinen internen Strukturen tilgen
		if (pNameList != NULL)
		{
			SdbHashString* pString = Find(_rName, *pNameList);
			pNameList->Remove(pString);
			delete pString;
		}
	}
	else
	{
		// und meinen Status
		String strMainMessage = aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_COULDNT_DELETE_DOC)));
		String strDetailed(SdbResId(STR_STAT_COULDNT_DELETE_DOC_DETAILED));
		aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("01000"), strMainMessage, 0, strDetailed);
	}

	return bSuccess;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::UndeleteObject(DBObject _eObj, String& _rName)
{
	DBG_ASSERT(!bReadOnly, "SdbDatabaseImpl::UndeleteObject : can't (un)delete when in read only mode !");
	DBG_ASSERT(_eObj != dbTable, "SdbDatabaseImpl::UndeleteObject : tables can only be (irreversible) dropped, not (reversible) deleted, so they can't be undeleted !");
	if (bReadOnly || (_eObj == dbTable))
		return FALSE;

	BOOL bObjectIsStorage = _eObj == dbQuery;


	// Source- und Destination-Folder
	SdbStorageRef aSourceStorage = OpenWasteObjectStorage(_eObj, _rName);
	if (!aSourceStorage.Is())
	{
		DBG_ERROR("SdbDatabaseImpl::UndeleteObject : something went wrong while opening the source storage ... but this shouldn't happen !");
			// eigentlich denke ich, dass das alles haette klappen sollen ...
		return FALSE;
	}

	SdbStorageRef aDestinationStorage = OpenStorage(_eObj, FALSE);
	DBG_ASSERT(aDestinationStorage.Is(), "SdbDatabaseImpl::UndeleteObject : could not open the destination storage !");

	// ------------------------------------------
	// bestimmen, welches der Objekte, die unter dem gegebenen Namen als geloescht markiert sind, wiederhergestellt werden soll
	String strWhichInstance;
	SdbHashStringList aAllInstances;
	if (bObjectIsStorage)
		aSourceStorage->GetStorageNameList(aAllInstances);
	else
		aSourceStorage->GetStreamNameList(aAllInstances);
	if (aAllInstances.Count() == 0)
	{
		DBG_ERROR("SdbDatabaseImpl::UndeleteObject : there doesn't exist a deleted object with the requested name !");
			// das sollte der Aufrufer vorher mit HasObjectDeleted feststellen !
		return FALSE;
	}
	strWhichInstance = String::CreateFromAscii(aAllInstances.GetObject(0)->GetString().GetBuffer());
		// TODO : eine richtige Indentifizierung, welche Instanz wiederherzustellen ist
	// fix noch die temporaere Liste loeschen
	for (UINT32 i=0; i < aAllInstances.Count(); ++i)
		delete aAllInstances.GetObject(i);

	// ----------------
	// der Name im Ziel
	String strTargetName = _rName;
	BOOL bAlreadyExistent = bObjectIsStorage
		? aDestinationStorage->HasStorage(strTargetName,_eObj==dbTable)
		: aDestinationStorage->Impl()->IsStream(strTargetName);
	if (bAlreadyExistent)
		aDestinationStorage->GetUniqueObjectName(strTargetName,_eObj==dbTable);

	// -------
	// und los
	BOOL bSuccess = FALSE;
	SdbHashStringList* pNameList = NULL;
	if (bObjectIsStorage)
	{
		bSuccess = aSourceStorage->MoveStorage(strWhichInstance, *aDestinationStorage, strTargetName,_eObj==dbTable);
		pNameList = &aQueryNameList;
	}
	else
	{
		bSuccess = aSourceStorage->Impl()->MoveTo(strWhichInstance, aDestinationStorage->Impl(), strTargetName);
		pNameList = (_eObj == dbForm) ? &aFormNameList : &aReportNameList;
	}

	// Storages wegwerfen
	aSourceStorage = NULL;
	aDestinationStorage = NULL;
	ReleaseStorage(_eObj, bSuccess);
		// ReleaseStorage als letztes, damit es meine Member (aFormStor ...) sauber aufraeumen kann !

	// ------------------------------------------
	// Asuwertung
	if (bSuccess)
	{
#if DEBUG || DBG_UTIL
		SdbStorageRef aSourceStorage = OpenWasteObjectStorage(_eObj, _rName);
		if (bObjectIsStorage)
		{
			DBG_ASSERT(!aSourceStorage->HasStorage(strWhichInstance,_eObj==dbTable), "SdbDatabaseImpl::UndeleteObject : MoveStorage said 'succes' but the source is still there ?");
		}
		else
		{
			DBG_ASSERT(!aSourceStorage->Impl()->IsStream(strWhichInstance), "SdbDatabaseImpl::UndeleteObject : MoveTo said 'succes' but the source is still there ?");
		}
		aSourceStorage = NULL;
#endif // DEBUG || DBG_UTIL
		// ich habe ein neues Objekt, das gehoert in die entsprechende Namensliste
		if (pNameList != NULL)
			pNameList->Insert(new SdbHashString(strTargetName), LIST_APPEND);

		_rName = strTargetName;
	}

	return bSuccess;
}

//------------------------------------------------------------------------------
BOOL SdbDatabaseImpl::DestroyDeletedObject(DBObject _eObj, const String& _rName)
{
	DBG_ASSERT(!bReadOnly, "SdbDatabaseImpl::DestroyDeletedObject : can't destroy a deleted object when in read only mode !");
	DBG_ASSERT(_eObj != dbTable, "SdbDatabaseImpl::DestroyDeletedObject : tables can only be (irreversible) dropped, not (reversible) deleted !");
	if (bReadOnly || (_eObj == dbTable))
		return FALSE;

	// den Folder, in dem sich der dem Objekt entsprechende Storage befindet
	OpenWasteStorage();
	DBG_ASSERT(m_aWasteBasket.Is(), "SdbDatabaseImpl::DestroyDeletedObject : have no wastebasket !");
		// sollte nicht passieren, da im Create oder Open angelegt

	// dort den Storage fuer die Objekt-Kategorie
	SdbStorageRef aCategoryStorage = OpenStorage(*m_aWasteBasket, CategoryFolderName(_eObj), SDB_STD_STREAM_MODE, 0);
	DBG_ASSERT(aCategoryStorage.Is(), "SdbDatabaseImpl::DestroyDeletedObject : a category storage is missing in the wastebasket !");
		// wenn der Papierkorb selber existiert, dann sollte das auch fuer die Kategorien gelten
	if (!aCategoryStorage.Is())
		return TRUE;

	if (!aCategoryStorage->HasStorage(_rName,_eObj==dbTable))
		return TRUE;
		// kann man sich vielleicht streiten, ob da ein FALSE hingehoert

	BeginTrans();
	BOOL bSuccess = aCategoryStorage->RemoveStorage(_rName,_eObj==dbTable);
	EndTrans(bSuccess);
	return bSuccess;
}

#endif // (SUPD>512) || PRIV_DEBUG

//------------------------------------------------------------------------------
// ueberprueft, ob der Name eindeutig fuer eine Datenbank ist, wichtig beim
// Anlegen von Tabellen
BOOL SdbDatabaseImpl::IsUniqueName(DBObject eObj, const String& rName)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	return FALSE;
}

//------------------------------------------------------------------------------
// Erzeugt fuer ein Object einen eindeutigen Namen
//
String SdbDatabaseImpl::GetUniqueName(DBObject eObj, const String& rName)
{
	DBG_CHKTHIS(SdbDatabaseImpl,NULL);
	return rName;
}


//-------------------------------------------------------------------------
ULONG SdbDatabaseImpl::CopyTo(SvStream& rTarget, SvStream& rDest)
{
	void * pMem = new BYTE[ 8192 ];
	ULONG nError = ERRCODE_NONE;
	ULONG  nRead;
	rDest.Seek(0);
	while( 0 != (nRead = rDest.Read(pMem, 8192 )))
	{
		if (nRead != rTarget.Write(pMem, nRead))
		{
			nError = SVSTREAM_GENERALERROR;
			break;
		}
	}
	delete pMem;
	if (nError == ERRCODE_NONE)
	{
		rTarget.Flush();
		nError = rTarget.GetError();
	}
	return nError;
}


