/*************************************************************************
 *
 *  $RCSfile: sdbdoc.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbdoc.hxx"

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif

#define SDB_CHK_DATABASE()							\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return;										\
	}

#define SDB_CHK_DATABASE_BOOL()						\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

#define SDB_CHK_DATABASE_STRING()					\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

#define SDB_CHK_DATABASE_NULL()						\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

DBG_NAME(SdbDocument);

static char __READONLY_DATA SQL[] = "SQL";

TYPEINIT1(SdbDocument, SdbObj);

//------------------------------------------------------------------
SdbDocument::SdbDocument(SdbDatabase* _pDb, DBObject _eObj)
			:SdbObj(_pDb, TRUE)
			,eObj(_eObj)
{
	DBG_CTOR(SdbDocument,NULL);
}

//------------------------------------------------------------------
SdbDocument::~SdbDocument()
{
	DBG_DTOR(SdbDocument,NULL);
}

//------------------------------------------------------------------
SdbDatabase* SdbDocument::GetDatabase() const
	{return PTR_CAST(SdbDatabase,GetParent());}

//------------------------------------------------------------------
BOOL SdbDocument::Create(const String& rName)
{
	// Open nicht moeglich wenn Tabellenbeschreibung schon geoeffnet
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();

	SDB_CHK_DATABASE_BOOL();

	aName = rName;
	bOpen = bNew = TRUE;

	return bOpen;
}

//------------------------------------------------------------------
BOOL SdbDocument::Update(SvStream& _rStream)
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();

	SdbDatabaseGuard aGuard(pDB);

	SdbStorage* pStor = pDB->OpenStorage(eObj, FALSE);
	if (!pStor)
	{
		aStatus = pDB->Status();
		return FALSE;
	}
	else
	{
		SvStorageStreamRef aStream;
		if (bNew)
			aStream = pDB->OpenStream(*pStor,aName,STREAM_STD_READWRITE);
		else
		{
			aStream = pDB->OpenStream(*pStor,aName,STREAM_STD_READWRITE | STREAM_TRUNC);
#ifdef _DEBUG
			ULONG nTest = aStream->GetSize();
#endif // _DEBUG
			// da das STREAM_TRUNC nicht funktioniert ....
			aStream->SetSize(0);
#ifdef _DEBUG
			nTest = aStream->GetSize();
			int nDummy = 0;		// to set the brechpunkt ;)
#endif // _DEBUG
		}

		if (!aStream.Is())
			aStatus = pDB->Status();
		else
		{
			ULONG nError = CopyTo(*aStream, _rStream);
			aStream = NULL;
			if (nError != ERRCODE_NONE)
				aStatus.SetError(nError,eObj,aName);
		}
	}
	pDB->ReleaseStorage(eObj, aStatus.IsSuccessful());
	if (aStatus.IsSuccessful())
	{
		if (bNew)
		{
			// Datenbank mitteilen, da ein Storage hinzugefuegt wurde
			pDB->Impl()->Add(aName, eObj);
			bNew = FALSE;
		}
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDocument::Open(const String& rName)
{
	aName = rName;
	return Open();
}

//------------------------------------------------------------------
BOOL SdbDocument::Open()
{
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	// Lesen des Statements
	bOpen = pDB->HasObject(eObj, aName);
	if (!bOpen)
	{
		aStatus.SetError(ERRCODE_IO_NOTEXISTS,eObj,aName);
	}
	return bOpen;
}

//------------------------------------------------------------------
BOOL SdbDocument::Get(SvStream& rStream)
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	SdbStorage* pStor = pDB->OpenStorage(eObj, TRUE);
	if (!pStor)
	{
		aStatus = pDB->Status();
		return FALSE;
	}
	else
	{
		SvStorageStreamRef aStream = pDB->OpenStream(*pStor,aName,STREAM_READ | STREAM_SHARE_DENYNONE);
		if (!aStream.Is())
			aStatus = pDB->Status();
		else
		{
			ULONG nError = CopyTo(rStream, *aStream);
			if (nError != ERRCODE_NONE)
				aStatus.SetError(nError,eObj,aName);
		}
		pDB->ReleaseStorage(eObj, FALSE);
	}
	return bOpen;
}

//------------------------------------------------------------------
BOOL SdbDocument::Get(::com::sun::star::uno::Sequence<sal_Int8>& rData)
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	SdbStorage* pStor = pDB->OpenStorage(eObj, TRUE);
	if (!pStor)
	{
		aStatus = pDB->Status();
		return FALSE;
	}
	else
	{
		SvStorageStreamRef aStream = pDB->OpenStream(*pStor,aName,STREAM_READ | STREAM_SHARE_DENYNONE);
		if (!aStream.Is())
			aStatus = pDB->Status();
		else
		{
			void * pMem = new BYTE[ 8192 ];			
			ULONG  nRead;
			aStream->Seek(0);
			rData.realloc(0);
			while( 0 != (nRead = aStream->Read(pMem, 8192 )))
			{
				rData.realloc(rData.getLength() + nRead);
				memcpy(rData.getArray() + rData.getLength() - nRead, pMem, nRead);				
			}
			delete pMem;			
		}
		pDB->ReleaseStorage(eObj, FALSE);
	}
	return bOpen;
}


//------------------------------------------------------------------
void SdbDocument::Close()
{
	if (!IsOpen())
		return;

	aName.Erase();
	SdbObj::Close();
}

//------------------------------------------------------------------
BOOL SdbDocument::Drop()
{
	DBG_CHKTHIS(SdbDocument,NULL);

	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	// Neu angelegtes Document wird nicht zerstrt
	if (!IsNew())
	{
		pDB->RemoveStorage(aName,eObj);
		aStatus = pDB->Status();
	}
	if (aStatus.IsSuccessful())
	{
		GetDatabase()->RemoveChild(this);
		bNew = bOpen = FALSE;
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDocument::Rename(const String& rNewName)
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	if (!pDB->IsUniqueName(eObj,rNewName))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,eObj, rNewName);
		return FALSE;
	}

	// Neu angelegtes Document fhrt nicht zum Umbennen am Storage
	if (IsNew() || pDB->RenameStorage(aName, rNewName, eObj))
	{
		aName = rNewName;
		if (eObj == dbReport)
			pDB->aReportList.Replace(this, aName, this);
		else
			pDB->aFormList.Replace(this, aName, this);
		return TRUE;
	}
	return FALSE;
}

//------------------------------------------------------------------
BOOL SdbDocument::DoRefresh()
{
	bOpen = FALSE;	//tmp reset
	return Open();
}

//------------------------------------------------------------------
BOOL SdbDocument::Compare(const SdbObj& _rObj) const
{
	SdbDocument* pObj = PTR_CAST(SdbDocument,&_rObj);
	// documents are always casesensitiv
	return pObj != NULL && aName == pObj->aName &&
		   eObj == pObj->eObj;
}

//-------------------------------------------------------------------------
ULONG SdbDocument::CopyTo(SvStream& rTarget, SvStream& rDest)
{
	void * pMem = new BYTE[ 8192 ];
	ULONG nError = ERRCODE_NONE;
	ULONG  nRead;
	rDest.Seek(0);
	while( 0 != (nRead = rDest.Read(pMem, 8192 )))
	{
		if (nRead != rTarget.Write(pMem, nRead))
		{
			nError = SVSTREAM_GENERALERROR;
			break;
		}
	}
	delete pMem;
	if (nError == ERRCODE_NONE)
	{
		rTarget.Flush();
		nError = rTarget.GetError();
	}
	return nError;
}


