/*************************************************************************
 *
 *  $RCSfile: sdbhash.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBHASH_HXX
#include "sdbhash.hxx"
#endif
#ifndef _REF_HXX //autogen
#include <tools/ref.hxx>
#endif

//--------------------------------------------------------------------------
SdbHashString* SdbHashStringList::Find(const ByteString& rStr) const
{
	SdbHashString aHashStr(rStr);
	return Find(aHashStr);
}

//--------------------------------------------------------------------------
SdbHashString* SdbHashStringList::Find(const SdbHashString& rStr) const
{
	SdbHashString* pHashString = NULL;
	ULONG i;
	for (i = 0; i < Count(); i++)
	{
		pHashString = GetObject(i);
		if (pHashString->HashID() == rStr.HashID() &&
			(m_bCaseSensitive 
			? 
			rStr.GetString() == pHashString->GetString() 
			: rStr.GetString().EqualsIgnoreCaseAscii(pHashString->GetString())))
			break;
	}
	return (i < Count()) ? pHashString : NULL;
}

//--------------------------------------------------------------------------
void* SdbStringMap::SetAt(const String& rKey, void* pValue)
{
	return StringHashTab::AddPtrSym((void *const)&rKey, pValue) ? pValue : NULL;
}

//--------------------------------------------------------------------------
void* SdbStringMap::RemoveAt(const String& rKey)
{
	HASHSYMPTR pValue = FindSym((void*const)&rKey);
	if (pValue)
		DelSym(pValue);
	return pValue;
}

//--------------------------------------------------------------------------
void* SdbStringMap::Remove(void* pValue)
{
	HASHSYMPTR pElem = Find(pValue);
	BOOL bElem(FALSE);
	if (pElem){
		DelSym(pElem);
		bElem = TRUE;
	}

	return (bElem) ? pValue : NULL;
}

//--------------------------------------------------------------------------
String SdbStringMap::GetKey(void* pValue) const
{
	HASHSYMPTR pElem = Find(pValue);
	const void* pKey = StringHashTab::GetName(pElem);
	return pKey ? String(*(const String*)pKey) : String();
}

//--------------------------------------------------------------------------
void* SdbStringMap::Lookup(const String& rKey) const
{
	return ((SdbStringMap*)this)->FindPtrSym((void *const)&rKey);
}

//--------------------------------------------------------------------------
HASHSYMPTR SdbStringMap::Find(void* pValue) const
{
	if (!pValue) return NULL;
	SdbStringMap* THIS = (SdbStringMap*) this;	// kleiner const cast

	HASHSYMPTR pElem = THIS->StringHashTab::First();
	while (pElem)
	{
		if (Derefer(pElem) == pValue)
			break;
		else
			pElem =  THIS->StringHashTab::Next();
	}
	return pElem;
}

//--------------------------------------------------------------------------
ULONG SdbStringMap::Count() const
{
#if (SUPD <= 364)
	static HashStats aStats;
	GetStats(&aStats);
	return (ULONG)aStats.nNumSyms;
#else
	return (ULONG) ((SdbStringMap*)this)->StringHashTab::Count();
#endif
}

//--------------------------------------------------------------------------
void SdbStringMap::Clear()
{
	HASHSYMPTR pValue;
	while ((pValue = StringHashTab::First()) != NULL)
		DelSym(pValue);
}

#if SUPD>510 || PRIV_DEBUG
//--------------------------------------------------------------------------
HASHTYPE SdbStringMap::Hash(const HASHNAMEPTR pName) const
{
	if (IsCaseSensitive())
		return StringHashTab::Hash(pName);
	else
	{
		String strNormalized = ((String*)pName)->ToUpperAscii();
		return StringHashTab::Hash((void*)&strNormalized);
	}
}


//--------------------------------------------------------------------------
int SdbStringMap::NameCmp(const HASHNAMEPTR pName1, const HASHNAMEPTR pName2) const
{
	if (IsCaseSensitive())
		return StringHashTab::NameCmp(pName1, pName2);
	else
	{
		switch ( ((const String*)pName1)->CompareIgnoreCaseToAscii( *(const String*)pName2 ) )
		{
			case COMPARE_LESS:
				return -1;
			case COMPARE_GREATER:
				return 1;

			case COMPARE_EQUAL:
			default:
				return 0;
		}
	}
}
#endif // SUPD>510 || PRIV_DEBUG

//--------------------------------------------------------------------------
SdbStringRefMap::SdbStringRefMap(HASHTYPE nMaxSlots)
				:SdbStringMap(nMaxSlots)
{
	SdbStringMap::SetDataDelLink(LINK(this,SdbStringRefMap,OnRemove));
}

//--------------------------------------------------------------------------
void* SdbStringRefMap::SetAt(const String& rKey, void* pValue)
{
	SvRefBase* pNew = (SvRefBase*)pValue;
	if (SdbStringMap::SetAt(rKey, pValue) != NULL)
		pNew->AddRef();
	return pNew;
}

//--------------------------------------------------------------------------
IMPL_LINK(SdbStringRefMap, OnRemove, HASHSYMPTR, pElem)
{
	SvRefBase *pValue = (SvRefBase *)Derefer(pElem);
	if (pValue)
		pValue->ReleaseRef();
	return 0;
}


