/*************************************************************************
 *
 *  $RCSfile: sdbidx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbidx.hxx"

#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

DBG_NAME(SdbIndex);

TYPEINIT1(SdbIndex, SdbObj);

//------------------------------------------------------------------
SdbIndex::SdbIndex(SdbTable* _pTable)
		 :SdbObj(_pTable, TRUE)
		 ,nStyle(0)
		 ,bSystemReadOnly(FALSE)
{
	DBG_CTOR(SdbIndex,NULL);
	// Neue Strukturbeschreibungen allozieren
	SDB_CHK_CONNECT();

	aColumns 			= pConnection->CreateColumns();
	aOriginalColumns 	= pConnection->CreateColumns();
}

//------------------------------------------------------------------
SdbIndex::~SdbIndex()
{
	DBG_DTOR(SdbIndex,NULL);
}

//------------------------------------------------------------------
BOOL SdbIndex::Create(const String& rName)
{
	// Open nicht moeglich wenn Tabellenbeschreibung schon geoeffnet
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();

	if (rName.GetTokenCount('.') == 2)
	{
		aQualifierName = rName.GetToken(0,'.');
		aName = rName.GetToken(1,'.');
	}
	else
		aName = rName;

	switch (GetConnection()->GetIdentifierCase())
	{
		case SDB_IC_LOWER:
			aName.ToLowerAscii();
			break;
		case SDB_IC_UPPER:
			aName.ToUpperAscii();
			break;
	}

	bNew = bOpen = TRUE;
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbIndex::Open(const String& rName)
{
	DBG_CHKTHIS(SdbIndex,NULL);

	if (rName.GetTokenCount('.') == 2)
	{
		aQualifierName = rName.GetToken(0,'.');
		aName = rName.GetToken(1,'.');
	}
	else
		aName = rName;
	return Open();
}

//------------------------------------------------------------------
BOOL SdbIndex::Open()
{
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	SDB_CHK_CONNECT_BOOL();
	aStatus.Clear();
	aOriginalColumns = pConnection->CreateColumns();

	BEGIN_DBACTION();
	BUSY_DB();
	bOpen = OpenImpl();

	if (bOpen)
	{
		// wird ein Index geoeffnet der keine Spalten enthaelt gilt dieser als
		// nicht gefunden #66608#
		if (aOriginalColumns->Count() < 2)
		{
			aColumns->Reset();
			aOriginalColumns->Reset();
			aStatus.SetError(ERRCODE_IO_NOTEXISTS, dbIndex, aName);
			bOpen = FALSE;
		}
		else
			aColumns = pConnection->CreateColumns(*aOriginalColumns);
	}
	else
	{
		// Strukturbeschreibungen freigeben
		aColumns->Reset();
		aOriginalColumns->Reset();
	}
	END_DBACTION();
	return bOpen;
}


//------------------------------------------------------------------
void SdbIndex::Close()
{
	if (!IsOpen())
		return;

	// Neue Strukturbeschreibungen allozieren
	aColumns->Reset();
	aOriginalColumns->Reset();
	aStatus.Clear();

	SdbObj::Close();
}

//------------------------------------------------------------------
BOOL SdbIndex::Update()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_TABLE_BOOL();

	// Hat sich ueberhaupt etwas geaendert?
	if (aColumns->Count() > 1 && aOriginalColumns->Count() <= 1)
	{
		if (CreateImpl())
		{
			aOriginalColumns = GetConnection()->CreateColumns(*aColumns);
			bNew = FALSE;
		}
		else
			return FALSE;
	}
	else if (aOriginalColumns->Count() > 1 && (*aOriginalColumns != *aColumns))
	{
		if (DropImpl() && CreateImpl())
			aOriginalColumns = GetConnection()->CreateColumns(*aColumns);
		else
			return FALSE;
	}
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbIndex::Drop()
{
	DBG_CHKTHIS(SdbIndex,NULL);

	SDB_CHK_OPEN_BOOL();
	SDB_CHK_TABLE_BOOL();

	// Neu angelegte Tabelle, wird nicht zerstoert
	if (!DropImpl())
		return FALSE;
	else
	{
		pTable->RemoveChild(this);
		return TRUE;
	}
}

//------------------------------------------------------------------
BOOL SdbIndex::DoRefresh()
{
	if (bNew)
		return TRUE;
	else
	{
		bOpen = FALSE;
		return Open();
	}
}

//------------------------------------------------------------------
BOOL SdbIndex::Compare(const SdbObj& _rObj) const
{
	SdbIndex* pObj = PTR_CAST(SdbIndex,&_rObj);
	SdbConnection* pConnection = GetConnection();

	return pObj != NULL && pConnection != NULL &&
		   nStyle == pObj->nStyle &&
			((pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT ?
				aName.Equals(pObj->aName) &&
				aQualifierName.Equals(pObj->aQualifierName) :
				aName.EqualsIgnoreCaseAscii(pObj->aName) &&
				aQualifierName.EqualsIgnoreCaseAscii(pObj->aQualifierName)) &&
			aColumns->Compare(*pObj->aColumns,0);
}

//------------------------------------------------------------------
SdbTable* SdbIndex::GetTable() const
	{return PTR_CAST(SdbTable,GetParent());}

//------------------------------------------------------------------
SdbConnection* SdbIndex::GetConnection() const
	{return GetParent() ? GetTable()->GetConnection() : NULL;}


