/*************************************************************************
 *
 *  $RCSfile: sdbmd.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBCONN_HXX
#include <sdbconn.hxx>
#endif

#include <sdbmd.hxx>

#ifndef _COM_SUN_STAR_BEANS_PROPERTYVALUE_HPP_
#include <com/sun/star/beans/PropertyValue.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYATTRIBUTE_HPP_
#include <com/sun/star/beans/PropertyAttribute.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_EVENTOBJECT_HPP_
#include <com/sun/star/lang/EventObject.hpp>
#endif

#include "sdbresid.hrc"

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;
namespace starlang = com::sun::star::lang;
namespace starbeans = com::sun::star::beans;

//------------------------------------------------------------------------------
SdbMetaData::SdbMetaData(SdbConnection *pConnection)
			:m_aDisposeListeners(m_aMutex)
{
	pCon = pConnection;
	// auf 0 zuruecksetzen
	memset(nLimits,-1,sizeof(sal_Int32) * (::stardata::DatabaseLimit::USER_NAME_LEN+1));
	memset(nCapas,-1,sizeof(sal_Int8) * (::stardata::DatabaseCapability::TRANSACTIONS+1));	
}

//==================================================================
//	Component-Interface
//==================================================================
//------------------------------------------------------------------------------
void SdbMetaData::dispose(void) throw(::staruno::RuntimeException)
{
	::starlang::EventObject aEvt;
	aEvt.Source = *this;
	m_aDisposeListeners.disposeAndClear(aEvt);
	if (pCon)
	{
		pCon->xDBMetaData = NULL;
		pCon = NULL;
	}
}

//------------------------------------------------------------------
void SdbMetaData::addEventListener(const ::staruno::Reference< ::starlang::XEventListener > & rListener) 
								throw(::staruno::RuntimeException)
{
	m_aDisposeListeners.addInterface(rListener);
}

//------------------------------------------------------------------
void SdbMetaData::removeEventListener(const ::staruno::Reference< ::starlang::XEventListener > & rListener)
								throw(::staruno::RuntimeException)
{
	m_aDisposeListeners.removeInterface(rListener);
}

//------------------------------------------------------------------
sal_Bool SdbMetaData::canConvert( sal_Int16 nFrom, sal_Int16 nIn ) throw(::stardata::DatabaseException, ::staruno::RuntimeException)
{
	if (!pCon)
	{
		::staruno::Reference< ::staruno::XInterface >  xInt(*this);
		throw ::stardata::DatabaseException(UniString(SdbResId(STR_STAT_INVALID_CONNECTION)), xInt, rtl::OUString(), 0);
	}
	return canConvertImpl(nFrom, nIn);
}

//------------------------------------------------------------------
sal_Bool SdbMetaData::isCapable( sal_Int16 nCapability ) throw(::stardata::DatabaseException, ::staruno::RuntimeException)
{
	DBG_ASSERT(nCapability <= ::stardata::DatabaseCapability::TRANSACTIONS,"SdbMetaData:getInfo nCapability > ::stardata::DatabaseCapability::TRANSACTIONS");
	if (!pCon)
	{
		::staruno::Reference< ::staruno::XInterface >  xInt(*this);			
		throw ::stardata::DatabaseException(UniString(SdbResId(STR_STAT_INVALID_CONNECTION)), xInt, rtl::OUString(), 0);
	}
	else if(nCapas[nCapability] == -1)
		((SdbMetaData*)this)->nCapas[nCapability] = isCapableImpl(nCapability);


	return nCapas[nCapability];
}

//------------------------------------------------------------------
::staruno::Any SdbMetaData::getInfo( sal_Int16 nInfo ) throw(::stardata::DatabaseException, ::staruno::RuntimeException)
{
	DBG_ASSERT(nInfo <= ::stardata::DatabaseInfo::USER_NAME,"SdbMetaData:getInfo nInfo > ::stardata::DatabaseInfo::USER_NAME");
	if (!pCon)
	{
		::staruno::Reference< ::staruno::XInterface >  xInt(*this);
		throw ::stardata::DatabaseException(UniString(SdbResId(STR_STAT_INVALID_CONNECTION)), xInt, rtl::OUString(), 0);
	}

	::staruno::Any aResult;
	switch (nInfo)
	{
		case ::stardata::DatabaseInfo::USER_NAME:
			aResult <<= rtl::OUString(pCon->GetUser());
			break;
		case ::stardata::DatabaseInfo::FIELD_TYPES:
		{
			const SdbTypeInfoList& rTypeList = pCon->GetTypeInfo();
			::staruno::Sequence< ::staruno::Sequence< ::starbeans::PropertyValue > > aSequence(rTypeList.Count());
			::staruno::Sequence< ::starbeans::PropertyValue >* pSeq = aSequence.getArray();
			for (sal_uInt32 i = 0; i < rTypeList.Count(); i++)
			{
				SdbTypeInfo* pInfo = rTypeList.GetObject(i);
				pSeq[i].realloc(14);
				::starbeans::PropertyValue* pProperties = pSeq[i].getArray();
				sal_Int32 j = 0;
				::staruno::Any aValue;
				aValue <<= ::rtl::OUString(pInfo->aTypeName);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("TYPE_NAME"),-1, 
					aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= sal_Int16(pInfo->eType);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("DATA_TYPE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);
				
				aValue <<= sal_Int32(pInfo->nPrecision);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("PRECISION"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);
				
				aValue <<= rtl::OUString(pInfo->aLiteralPrefix);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("LITERAL_PREFIX"),-1,
								aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= rtl::OUString(pInfo->aLiteralSuffix);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("LITERAL_SUFFIX"),-1,
								aValue, ::starbeans::PropertyState_DIRECT_VALUE);								

				aValue <<= rtl::OUString(pInfo->aCreateParams);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("CREATE_PARAMS"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);								 

				aValue <<= sal_Bool(pInfo->bNullable);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("NULLABLE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);								 

				aValue <<= sal_Bool(pInfo->bCaseSensitive);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("CASE_SENSITIVE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= sal_Int16(pInfo->eSearchType);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("SEARCHABLE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= sal_Bool(pInfo->bUnsigned);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("UNSIGNED_ATTRIBUTE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= sal_Bool(pInfo->bAutoIncrement);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("AUTO_INCREMENT"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= rtl::OUString(pInfo->aLocalTypeName);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("LOCAL_TYPE_NAME"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= sal_Int16(pInfo->nMinimumScale);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("MINIMUM_SCALE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);

				aValue <<= sal_Int16(pInfo->nScale);
				pProperties[j++] = ::starbeans::PropertyValue(rtl::OUString::createFromAscii("MAXIMUM_SCALE"),-1,
								 aValue, ::starbeans::PropertyState_DIRECT_VALUE);
			}
			aResult <<= aSequence;
		}	break;
		default:
		{
			if (!aInfos[nInfo].hasValue())
				((SdbMetaData*)(this))->aInfos[nInfo] = getInfoImpl(nInfo);
			return aInfos[nInfo];
		}
	}
	return aResult;
}

//------------------------------------------------------------------
sal_Int32 SdbMetaData::getLimit( sal_Int16 Limit ) throw(::stardata::DatabaseException, ::staruno::RuntimeException)
{
	DBG_ASSERT(Limit <= ::stardata::DatabaseLimit::USER_NAME_LEN,"SdbMetaData:getLimit nLimit > ::stardata::DatabaseLimit::USER_NAME_LEN");
	if (!pCon)
	{
		::staruno::Reference< ::staruno::XInterface >  xInt(*this);
		throw ::stardata::DatabaseException(UniString(SdbResId(STR_STAT_INVALID_CONNECTION)), xInt, rtl::OUString(), 0);
	}

	else if(nLimits[Limit] == -1)
	{
		((SdbMetaData*)(this))->nLimits[Limit] = getLimitImpl(Limit);
	}
	return nLimits[Limit];
}



