/*************************************************************************
 *
 *  $RCSfile: sdbobj.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbobj.hxx"

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

const String SdbObj::aEmptyString;


TYPEINIT0(SdbObj);

//------------------------------------------------------------------
SdbObj::~SdbObj()
{
}

//------------------------------------------------------------------
void SdbObj::QueryDelete()
{
	RestoreNoDelete();
	// existiert kein Parent, dann wird das Objekt auf alle Faelle
	// freigegeben
#if SUPD < 561
	if (!pParent)
#else
	if (!pParent || m_bTemporary)
#endif // SUPD < 561
	{
		RemoveChildren();
		delete this;
	}
	// nicht persistente Objekte werden ebenfalls freigegeben
	// allerdings hierfuer ist der Parent verantwortlich
	// Persistentes Object, das nicht abgespeichert wurde
	else if (!IsPersistent() || bNew)
	{
		pParent->RemoveChild(this);
	}
}

//------------------------------------------------------------------
void SdbObj::RemoveFromParent()
{
	// Handelt es sich um ein persistentes Objekt,
	// welches nicht mehr referenziert wird, mus es geloescht werden
	if (GetRefCount() == SV_NO_DELETE_REFCOUNT)
	{
		// bis jetzt benoetigen nur nicht persistente Objekte (Connection/Cursor) ein Close
		// gilt vielleicht spaeter auch fuer persistente Objekte
		if (pParent && IsOpen() && !IsPersistent())
			Close();

		// give derived classes the possibility to do any parent-dependent cleanup
		RemovingFromParent();

		pParent = NULL;
		// QueryDelete wird direkt gerufen, da es sonst keiner tuen
		// wuerde, s.o.
		QueryDelete();
	}
	else
	{
		RemoveChildren();

		if (pParent && IsOpen())
			Close();

		// give derived classes the possibility to do any parent-dependent cleanup
		RemovingFromParent();

		pParent = NULL;
	}
}

//------------------------------------------------------------------
void SdbObj::RemovingFromParent()
{
}

//------------------------------------------------------------------
void SdbObj::RemoveChildren()
// die Implementierung haengt davon ab, welche Container verwendet werden
// SdbObj selbst besitzt keinen.
{
}

//------------------------------------------------------------------
void SdbObj::RemoveChild(SdbObj* _pChild)
{
	DBG_ASSERT(_pChild, "SdbObj::RemoveChild -> kein Child angegeben");
	DBG_ASSERT(this == _pChild->pParent, "SdbObj::RemoveChild -> Falscher Parent");

	// werden Container verwendet, dann Methode ueberladen, damit Objekt ausgetragen wird
	// wird das Objekt noch verwendet ?
	// dann muss es geschlossen werden, andernfalls wird es eh zerstoert
	// siehe RemoveFromParent();
	_pChild->RemoveFromParent();
}

//------------------------------------------------------------------
void SdbObj::Close()
{
	if (!IsOpen())
		return;
	else
	{
		aStatus.Clear();
		bNew = bOpen = FALSE;
		aCloseLink.Call(this);
	}
}

//------------------------------------------------------------------
BOOL SdbObj::Refresh()
{
	BOOL bResult;
	if (IsNew())
		bResult = TRUE;
	else if (!IsOpen())
		bResult = FALSE;
	else
	{
		aStatus.Clear();
		bResult = DoRefresh();
		// Das Objekt konnte nicht aktualisiert werden
		// also ist es nicht mehr gueltig
		// und wird geschlossen
		if (!bResult)
		{
			GetParent()->RemoveChild(this);
		}
	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbObj::DoRefresh()
{
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbObj::Compare(const SdbObj&) const
{
	return FALSE;
}

//------------------------------------------------------------------
SdbUiActionDescription::RETURN SdbObj::DoUiAction(const SdbUiActionDescription& rDesc)
{
	if (m_aUiActionLink.IsSet())
		return (SdbUiActionDescription::RETURN)m_aUiActionLink.Call((void*)&rDesc);
	else if (pParent)
		return pParent->DoUiAction(rDesc);
	else
		return SdbUiActionDescription::RETURN_OK;
}


