/*************************************************************************
 *
 *  $RCSfile: sdbqry.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbqry.hxx"

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif


#define SDB_CHK_DATABASE()							\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return;										\
	}

#define SDB_CHK_DATABASE_BOOL()						\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

#define SDB_CHK_DATABASE_STRING()					\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

#define SDB_CHK_DATABASE_NULL()						\
	SdbDatabase* pDB = GetDatabase();				\
	if (!pDB)										\
	{												\
		aStatus.SetConnectionLost();				\
		return FALSE;								\
	}

DBG_NAME(SdbQuery);

static char __READONLY_DATA SQL[] = "SQL";

TYPEINIT1(SdbQuery, SdbObj);

//------------------------------------------------------------------
SdbQuery::SdbQuery(SdbDatabase* _pDb)
		 :SdbObj(_pDb, TRUE)
		 ,bNative(FALSE)
{
	DBG_CTOR(SdbQuery,NULL);
}

//------------------------------------------------------------------
SdbQuery::~SdbQuery()
{
	DBG_DTOR(SdbQuery,NULL);
}

//------------------------------------------------------------------
SdbDatabase* SdbQuery::GetDatabase() const
	{return PTR_CAST(SdbDatabase,GetParent());}

//------------------------------------------------------------------
BOOL SdbQuery::Create(const String& rName)
{
	// Open nicht moeglich wenn Tabellenbeschreibung schon geoeffnet
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	aStatus.Clear();

	SDB_CHK_DATABASE_BOOL();

	aName = rName;
	bOpen = bNew = TRUE;

	return bOpen;
}

//------------------------------------------------------------------
SdbCursor* SdbQuery::OpenCursor(SdbCursorType eType,
								   UINT16 nOptions)
{
	SDB_CHK_OPEN_NULL();
	SDB_CHK_DATABASE_NULL();
	SdbDatabaseGuard aGuard(pDB);

	SdbCursor* pCursor = pDB->GetConnection()->
							CreateCursor(eType, nOptions);
	if (!pCursor->Open(aStatement))
	{
		aStatus = pCursor->Status();
		pCursor->AddRef();
		pCursor->ReleaseRef();
		pCursor = NULL;
	}
	return pCursor;
}

//------------------------------------------------------------------
BOOL SdbQuery::Update(const String& rSQLStatement, BOOL bNat)
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	SdbStorage* pStor = pDB->OpenStorage(dbQuery, FALSE);
	if (!pStor)
	{
		aStatus = pDB->Status();
		return FALSE;
	}
	else
	{
		SdbStorageRef aQueryStor = (bNew) ? pDB->OpenStorage(*pStor, aName, STREAM_STD_READWRITE, STORAGE_FAILIFTHERE)
										  : pDB->OpenStorage(*pStor, aName, STREAM_STD_READWRITE);
		SvStorageStreamRef aStream;
		if (!aQueryStor.Is())
			goto End;
		else
		{
			SvStorageStreamRef aStream = pDB->OpenStream(*aQueryStor,String::CreateFromAscii(SQL),STREAM_STD_READWRITE|STREAM_TRUNC);
			if (!aStream.Is())
				goto End;

			(*aStream) << (BYTE)bNat;
			if (!pDB->Impl()->Write(*aStream, rSQLStatement))
				goto End;
			else
				aStream->Flush();
		}
End:
		aStatus = pDB->Status();
		pDB->ReleaseStorage(dbQuery, aStatus.IsSuccessful());

		if (aStatus.IsSuccessful())
		{
			aStatement = rSQLStatement;
			bNative 	= bNat;
			if (bNew)
			{
				pDB->Impl()->Add(aName, dbQuery); // Datenbank mitteilen, da ein Storage hinzugefuegt wurde
				bNew = FALSE;
			}
		}
	}

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbQuery::Open(const String& rName)
{
	aName = rName;
	return Open();
}

//------------------------------------------------------------------
BOOL SdbQuery::Open()
{
	aStatus.Clear();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	// Lesen des Statements
	SdbStorage* pStor = pDB->OpenStorage(dbQuery, TRUE);
	if (!pStor)
	{
		aStatus = pDB->Status();
		return FALSE;
	}
	else
	{
		SvStorageStreamRef aStream;
		SdbStorageRef aQueryStor = pDB->OpenStorage(*pStor, aName, STREAM_READ | STREAM_SHARE_DENYNONE);
		if (!aQueryStor.Is())
			goto End;
		else
		{
			SvStorageStreamRef aStream = pDB->OpenStream(*aQueryStor,String::CreateFromAscii(SQL),STREAM_READ | STREAM_SHARE_DENYNONE);
			if (!aStream.Is())
				goto End;

#if ( defined GCC && defined C272 ) || defined( WTC ) || defined IRIX || defined( ICC ) || defined ( HPUX)
			(*aStream) >> (BYTE &)bNative;
#else
			(*aStream) >> (BYTE)bNative;
#endif
			if (!pDB->Impl()->Read(*aStream, aStatement))
				goto End;
			else
				bOpen = TRUE;
		}
End:
		aStatus = pDB->Status();
		pDB->ReleaseStorage(dbQuery, FALSE);
	}
	return bOpen;
}

//------------------------------------------------------------------
void SdbQuery::Close()
{
	if (!IsOpen())
		return;

	aName.Erase();
	aStatement.Erase();
	SdbObj::Close();
}

//------------------------------------------------------------------
BOOL SdbQuery::Drop()
{
	DBG_CHKTHIS(SdbQuery,NULL);

	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	if (!IsNew())
	{
		pDB->RemoveStorage(aName,dbQuery);
		aStatus = pDB->Status();
	}
	if (aStatus.IsSuccessful())
	{
		bNew = bOpen = FALSE;
		pDB->RemoveChild(this);
		//pDB->aQueryList.Remove(this);
		
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbQuery::Rename(const String& rNewName)
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_DATABASE_BOOL();
	SdbDatabaseGuard aGuard(pDB);

	if (!pDB->IsUniqueName(dbQuery,rNewName))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,QUERY,rNewName);
		return FALSE;
	}

	if (bNew || pDB->RenameStorage(aName, rNewName, dbQuery))
	{
		aName = rNewName;
		pDB->aQueryList.Replace(this, aName, this);
		return TRUE;
	}
	return FALSE;
}

//------------------------------------------------------------------
BOOL SdbQuery::DoRefresh()
{
	bOpen = FALSE;	//tmp reset
	return Open();
}

//------------------------------------------------------------------
BOOL SdbQuery::Compare(const SdbObj& _rObj) const
{
	SdbQuery* pObj = PTR_CAST(SdbQuery,&_rObj);
	BOOL bCase = (GetDatabase()->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;

	return pObj != NULL && (bCase ? aName == pObj->aName : aName.EqualsIgnoreCaseAscii(pObj->aName)) &&
		(bCase ? aStatement == pObj->aStatement : aStatement.EqualsIgnoreCaseAscii(pObj->aStatement)) &&
		   bNative == pObj->bNative;
}


