/*************************************************************************
 *
 *  $RCSfile: sdbrel.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

#ifndef _SDB_SDBREL_HXX
#include <sdbrel.hxx>
#endif

#ifndef _COM_SUN_STAR_DATA_DATABASEKEYRULE_HPP_
#include <com/sun/star/data/DatabaseKeyRule.hpp>
#endif

#ifndef _INTN_HXX //autogen
#include <tools/intn.hxx>
#endif

#ifndef _DATE_HXX //autogen
#include <tools/date.hxx>
#endif

#ifndef _TOOLS_TIME_HXX //autogen
#include <tools/time.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif


TYPEINIT1(SdbColumnRelation, SdbColumn);
TYPEINIT1(SdbRelation, SdbObj);
DBG_NAME(SdbRelation);

namespace stardata = com::sun::star::data;

// geeignet als temp name, da der Name nicht den SQL Regeln entspricht
static char __READONLY_DATA TEMPNAME[] = "1Rel";

//------------------------------------------------------------------
SdbColumn* SdbColumnRelation::Clone() const
{
	return new SdbColumnRelation(*this);
}

//------------------------------------------------------------------
SdbRelation::SdbRelation(SdbConnection* _pConn,
						 const String& _rName)
			:SdbObj(_pConn, TRUE)
			,aName(_rName)
			,nUpdateRules(0)
			,nDeleteRules(0)

{
	DBG_CTOR(SdbRelation,NULL);
	aColumns = _pConn->CreateColumns();
	bNew  = TRUE;	// default neu angelegt
	bOpen = TRUE;

	// wenn keine Name angegeben wird, dann wird ein temp Name vergeben
	// nach einem Update wird der vom System vergebene Name benutzt
	if (!aName.Len())
	{
		// temp name wird erkannt ueber Ziffer am Anfang
		const International& rIntn = Application::GetAppInternational();
		aName = CreateTempName();
	}
}

//------------------------------------------------------------------
SdbRelation::~SdbRelation()
{
	DBG_DTOR(SdbRelation,NULL);
}

//------------------------------------------------------------------
String SdbRelation::CreateTempName()
{
	// temp name wird erkannt ueber Ziffer am Anfang
	const International& rIntn = Application::GetAppInternational();
	String aTemp;
	aTemp.AssignAscii(TEMPNAME);
	aTemp+=rIntn.GetDate(Date());
	aTemp+=rIntn.GetTime(Time());
	return aTemp;
}

//------------------------------------------------------------------
BOOL SdbRelation::IsTempName(const String& _rName)
{
	// temp name wird erkannt ueber Ziffer am Anfang
	return _rName.Copy(0,String::CreateFromAscii(TEMPNAME).Len()).Equals(String::CreateFromAscii(TEMPNAME));
}

//------------------------------------------------------------------
SdbColumnRelation* SdbRelation::CreateColumn(const SdbColumn& _rCol,
											 const String& _rForeignName) const
{
	return new SdbColumnRelation(_rCol, _rForeignName);
}

//------------------------------------------------------------------
BOOL SdbRelation::Open()
{
	aStatus.Clear();
	if (!IsNew())	// schon einmal aufgerufen
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	bNew = FALSE;
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbRelation::Update()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	aStatus.Clear();
	BOOL bResult = FALSE;

	if (!bNew) // nur neu anlegen mglich
	{
		aStatus.SetSequenceError();
	}
	else
	{
		String aCommand(String::CreateFromAscii("ALTER TABLE "));
		aCommand += pConnection->QuoteTableName(aTableName);
		aCommand += String::CreateFromAscii(" ADD ");
		if (aName.Len() && !IsTempName(aName))
		{
			aCommand += String::CreateFromAscii(" CONSTRAINT ");
			aCommand += pConnection->IdentifierQuotation(GetName());
		}
		aCommand += String::CreateFromAscii(" FOREIGN KEY (");
		USHORT i;
		for (i = 1; i < aColumns->Count(); i++)
		{
			aCommand += pConnection->IdentifierQuotation(aColumns->Column(i)->GetName());
			aCommand += String::CreateFromAscii(",");
		}
		aCommand.SetChar(aCommand.Len()-1,')');
		aCommand += String::CreateFromAscii(" REFERENCES ");
		aCommand += pConnection->QuoteTableName(aForeignTableName);
		aCommand += String::CreateFromAscii(" (");
		for (i = 1; i < aColumns->Count(); i++)
		{
			SdbColumnRelationRef xCol = PTR_CAST(SdbColumnRelation, aColumns->Column(i));
			aCommand += pConnection->IdentifierQuotation(xCol.Is() ? xCol->GetForeignName() : String());
			aCommand += String::CreateFromAscii(",");
		}
		aCommand.SetChar(aCommand.Len()-1,')');


		// update rules
		if ((nUpdateRules & ::stardata::DatabaseKeyRule::CASCADE) == ::stardata::DatabaseKeyRule::CASCADE)
			aCommand += String::CreateFromAscii(" ON UPDATE CASCADE ");
		else if ((nUpdateRules & ::stardata::DatabaseKeyRule::SET_NULL) == ::stardata::DatabaseKeyRule::SET_NULL)
			aCommand += String::CreateFromAscii(" ON UPDATE SET NULL ");
		else if ((nUpdateRules & ::stardata::DatabaseKeyRule::SET_DEFAULT) == ::stardata::DatabaseKeyRule::SET_DEFAULT)
			aCommand += String::CreateFromAscii(" ON UPDATE DEFAULT ");

		// delete rules
		if ((nDeleteRules & ::stardata::DatabaseKeyRule::CASCADE) == ::stardata::DatabaseKeyRule::CASCADE)
			aCommand += String::CreateFromAscii(" ON DELETE CASCADE ");
		else if ((nDeleteRules & ::stardata::DatabaseKeyRule::SET_NULL) == ::stardata::DatabaseKeyRule::SET_NULL)
			aCommand += String::CreateFromAscii(" ON DELETE SET NULL ");
		else if ((nDeleteRules & ::stardata::DatabaseKeyRule::SET_DEFAULT) == ::stardata::DatabaseKeyRule::SET_DEFAULT)
			aCommand += String::CreateFromAscii(" ON DELETE DEFAULT ");

		if (!pConnection->ExecuteSQL(aCommand))
			aStatus = pConnection->Status();

		bResult = aStatus.IsSuccessful();
		if (bResult)
		{
			bNew = FALSE;
			if (IsTempName(aName))
			{
				if (!Refresh())		// temp Name behalten
					aStatus.Clear();
			}
		}
	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbRelation::Drop()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	aStatus.Clear();
	BOOL bResult = FALSE;

	if (!bNew)
	{
		String aCommand(String::CreateFromAscii("ALTER TABLE "));
		aCommand += pConnection->QuoteTableName(aTableName);
		aCommand += String::CreateFromAscii(" DROP CONSTRAINT ");
		aCommand += pConnection->IdentifierQuotation(GetName());

		if (!pConnection->ExecuteSQL(aCommand))
			aStatus = pConnection->Status();

		bResult = aStatus.IsSuccessful();
	}
	else
	{
		bResult = TRUE;
	}

	if (bResult)	// beim Vater abmelden
	{
		pConnection->RemoveChild(this);
		bNew = bOpen = FALSE;
	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbRelation::Refresh()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	aColumns->Reset();
	// den Namen merken, da sich dieser durchaus geaendert haben kann
	String aTempName = aName;

	if (SdbObj::Refresh() && aName == aTempName)
	{
		pConnection->aRelationList.Remove(this);
		pConnection->aRelationList.InsertAt(aName,this);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbRelation::Compare(const SdbObj& _rObj) const
{
	SdbRelation* pObj = PTR_CAST(SdbRelation,&_rObj);
	SdbConnection* pConnection = GetConnection();

	return pObj != NULL && pConnection != NULL &&
		   GetUpdateRules() == pObj->GetUpdateRules() &&
		   GetDeleteRules() == pObj->GetDeleteRules() &&
			((pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT ?
				aName == pObj->aName &&
				aTableName == pObj->aTableName &&
				aForeignTableName == pObj->aForeignTableName :
				aName.EqualsIgnoreCaseAscii(pObj->aName) &&
				aTableName.EqualsIgnoreCaseAscii(pObj->aTableName) &&
				aForeignTableName.EqualsIgnoreCaseAscii(pObj->aForeignTableName)) &&
			aColumns->Compare(*pObj->aColumns,0);
}

//------------------------------------------------------------------
SdbConnection* SdbRelation::GetConnection() const
	{return PTR_CAST(SdbConnection,GetParent());}


void SdbRelation::NameChanged()
{
	GetConnection()->aRelationList.Remove(this);
	GetConnection()->aRelationList.InsertAt(aName,this);
}


