/*************************************************************************
 *
 *  $RCSfile: sdbset.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <cstdio>

#include "sdbset.hxx"
#ifndef _VOS_TIMER_HXX_
#include <vos/timer.hxx>
#endif

#ifndef _SV_TIMER_HXX //autogen
#include <vcl/timer.hxx>
#endif

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _SDB_SDBIDX_HXX //autogen
#include "sdbidx.hxx"
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include "sdbtools.hxx"
#endif

#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif

#ifndef _SDB_SDBRESID_HRC
#include "sdbresid.hrc"
#endif

#ifndef _SDB_SDBIDX_HXX //autogen
#include "sdbidx.hxx"
#endif

#ifndef _SDB_SDBSQLIT_HXX //autogen
#include "sdbsqlit.hxx"
#endif

#include <stl_algo.h>

namespace staruno = com::sun::star::uno;
namespace starlang = com::sun::star::lang;

using namespace vos;

DBG_NAME(OValueSetRow);

#define IS_REFETCHABLE(i)	\
	(find(m_aRefetchColumns.begin(), m_aRefetchColumns.end(), i) != m_aRefetchColumns.end())

#define IS_UPDATEABLE(aCol)	\
	!aCol.IsAutoIncrement() && aCol.IsUpdateAllowed() && !aCol.IsFunction()

#define IS_SEARCHABLE(pCol) (pCol->GetAllFlags() & SDB_FLAGS_SEARCH_EXCEPT_LIKE)

//--------------------------------------------------------------------------
class OINT32Less
{	
public:				
	bool operator() (INT32 lhs, INT32 rhs) const {return lhs < rhs;}	
};	

//--------------------------------------------------------------------------
class OValueRowLess
{	
public:				
	bool operator() (const OValueRow& lhs, const OValueRow& rhs) const {return lhs.getBookmark() < rhs.getBookmark();}	
};	

//--------------------------------------------------------------------------
OValueRow::OValueRow(INT32 nBookmark, const ODbRow& rRow, const OValueSet& rSet)
		  :m_nBookmark(nBookmark)
		  ,vector< ::staruno::Any>(rSet.m_aKeyColumns.size())
{
	setRow(rRow, rSet);
}

//--------------------------------------------------------------------------
void OValueRow::setRow(const ODbRow& rRow, const OValueSet& rSet)
{
	vector<OKeySetColumn>::const_iterator j = rSet.m_aKeyColumns.begin();
	for (vector< ::staruno::Any>::iterator i = begin(); i != end(); i++)
		(*i) = rRow[(*j++).nId]->getAny();
}

//--------------------------------------------------------------------------
void OValueRow::getRow(ODbRow& rRow, const OValueSet& rSet) const
{
	vector<OKeySetColumn>::const_iterator j = rSet.m_aKeyColumns.begin();
	for (vector< ::staruno::Any>::const_iterator i = begin(); i != end(); i++)
	{
		UINT16 nId = (*j++).nId;
		if (rRow[nId].is())
			rRow[nId]->setAny(*i);
	}	  	
	// setting the bookmark
	rRow[0]->setINT32(m_nBookmark);
	// clear Modified
	rRow.setModified(sal_False);
}

// Thread zum ermitteln des RowCounts
//--------------------------------------------------------------------------
ORowCounter::ORowCounter(OKeySet& _rSet)
		    :m_rSet(_rSet)
{
}

//--------------------------------------------------------------------------
void ORowCounter::run()
{
	UINT32 i = 0;
	SdbCursor& rCursor = m_rSet.m_rOrigCursor;
	BOOL bNext = !rCursor.IsRowCountFinal();
	BOOL bNotified = TRUE;
	while (schedule() && bNext && !rCursor.IsRowCountFinal())
	{
		if (m_rSet.getMutex().tryToAcquire())
		{
			if ((bNext = m_rSet.getNext(i, TRUE)))
			{
				rCursor.nRowCount++;
				bNotified = sal_False;
			}

			if ((bNext && i == 100) || (!bNext && i))
			{
				m_rSet.getMutex().release();
				if (schedule())
				{
					if (Application::GetSolarMutex().tryToAcquire())
					{
						bNotified = TRUE;
						rCursor.GetRowCountHdl().Call(&rCursor);
						Application::GetSolarMutex().release();
					}
				}
				i = 0;
			}
			else
				m_rSet.getMutex().release();
		}
	}

	// Wenn der RowCount final, dann mu sichergestellt werden,
	// da eine Benachrichtigung vorgenommen wird
	while (schedule() && !bNotified)
	{
		if (Application::GetSolarMutex().tryToAcquire())
		{
			bNotified = TRUE;
			rCursor.GetRowCountHdl().Call(&rCursor);
			Application::GetSolarMutex().release();
		}
	}
}

//--------------------------------------------------------------------------
void ORowCounter::onTerminated()
{
	m_rSet.m_pCounter = NULL;
	delete this;
}

//--------------------------------------------------------------------------
DBG_NAME(OKeySet);

//--------------------------------------------------------------------------
OKeySet::OKeySet(SdbCursor& rCursor)
			 :m_rOrigCursor(rCursor)
			 ,m_xResultColumns(rCursor.GetColumns())
			 ,m_nCurrentPos(-1)
			 ,m_nPrivileges(SDB_PR_READ)			 
			 ,m_bBound(TRUE)
			 ,m_bSnapshot(sal_False)
			 ,m_pCounter(NULL)
			 ,m_pTimer(NULL)
{
	DBG_CTOR(OKeySet,NULL);
	m_xBindRow = new ODbRow(*m_xResultColumns, sal_False);
	startCounting();
}

//--------------------------------------------------------------------------
OKeySet::~OKeySet()
{
	DELETEZ(m_pTimer);
	DBG_DTOR(OKeySet,NULL);
}

//------------------------------------------------------------------------------
void OKeySet::startCounting()
{
	if (!m_rOrigCursor.pCache || m_rOrigCursor.IsRowCountFinal())
		return;

	if (m_rOrigCursor.bThreadSafe)
	{
		if (!m_pCounter && !m_pTimer)
		{
			m_pTimer = new Timer();
			m_pTimer->SetTimeoutHdl(LINK(this,OKeySet,onTimeout));
			m_pTimer->SetTimeout(1500);
			m_pTimer->Start();
		}
	}
	else if (!m_pTimer)
	{
		m_pTimer = new Timer();
		m_pTimer->SetTimeoutHdl(LINK(this,OKeySet,onCount));
		m_pTimer->SetTimeout(100);
		m_pTimer->Start();
	}
}

/*
//--------------------------------------------------------------------------
BOOL OKeySet::isRefetchable(const SdbColumnRef& xCol)
{
	for (USHORT i = 0; i < m_nKeyCount; i++)
		if (m_nKeyCount[i].nId == xCol->GetId())
			return sal_False;
	return TRUE;
}
*/

//------------------------------------------------------------------------------
void OKeySet::prepareDelete()
{
	if (m_pCounter && m_pCounter->isRunning())
	{
		// warten auf das beenden des Threads
		m_pCounter->terminate();
		m_pCounter->join();
	}
	else if (!m_rOrigCursor.bThreadSafe && !m_rOrigCursor.IsRowCountFinal())
	{
		DELETEZ(m_pTimer);
	}
}

//--------------------------------------------------------------------------
BOOL OKeySet::setUpdatable(BOOL bUpdate)
{
	DBG_CHKTHIS(OKeySet,NULL);
	if (!bUpdate)
	{
		m_rOrigCursor.nPrivileges = SDB_PR_READ;
		return TRUE;
	}
	else if (m_bSnapshot && m_nPrivileges > SDB_PR_READ)
	{
		// update nicht moeglich, da zu unsicher
//		m_rOrigCursor.aStatus.Set(SDB_STAT_ERROR,
//						"S1000",
//						String(SdbResId(STR_STAT_CURSOR_UPDATE_ERROR)),
//						0, "");
		m_nPrivileges &= ~(SDB_PR_UPDATE | SDB_PR_DELETE | SDB_PR_INSERT);
		m_rOrigCursor.nPrivileges |= m_nPrivileges;
	}
	else
		m_rOrigCursor.nPrivileges |= m_nPrivileges;

	return m_nPrivileges > SDB_PR_READ;
}

//------------------------------------------------------------------------------
IMPL_LINK(OKeySet, onTimeout, void*, EMPTYTAG)
{
	if (!m_rOrigCursor.IsRowCountFinal())
		m_pCounter = new ORowCounter(*this);

	if (m_pCounter && m_pCounter->createSuspended())
	{
		m_pCounter->setPriority(OThread::TPriority_Lowest);
		m_pCounter->resume();
	}
	DELETEZ(m_pTimer);
	return 1;
}

//------------------------------------------------------------------------------
IMPL_LINK(OKeySet, onCount, void*, EMPTYTAG)
{
	INT32 nLimit = Application::AnyInput() ? 5 : 25;
	UINT32 i = 0;
	OGuard aGuard(getMutex());
	BOOL bNotified = TRUE;
	BOOL bNext = !m_rOrigCursor.IsRowCountFinal();
	unBind();
	
	while (bNext && !m_rOrigCursor.IsRowCountFinal() && i < nLimit)
	{
		if ((bNext = getNext(i, TRUE)))
		{
			m_rOrigCursor.nRowCount++;
			bNotified = sal_False;
		}

		if ((bNext && i == (nLimit - 1)) || (!bNext && i))
		{
			if (Application::GetSolarMutex().tryToAcquire())
			{
				bNotified = TRUE;
				m_rOrigCursor.GetRowCountHdl().Call(&m_rOrigCursor);
				Application::GetSolarMutex().release();
			}
		}		
	}

	bind();

	// Wenn der RowCount final, dann mu sichergestellt werden,
	// da eine Benachrichtigung vorgenommen wird
	while (!bNotified)
	{
		if (Application::GetSolarMutex().tryToAcquire())
		{
			bNotified = TRUE;
			m_rOrigCursor.GetRowCountHdl().Call(&m_rOrigCursor);
			Application::GetSolarMutex().release();
		}
	}

	if (m_rOrigCursor.IsRowCountFinal())
		DELETEZ(m_pTimer);
	else
		m_pTimer->Start();
	return 1;
}

//--------------------------------------------------------------------------
BOOL OKeySet::getNext(UINT32& i, BOOL bReleaseBindings)
{
	DBG_CHKTHIS(OKeySet,NULL);

	BOOL bUnbind = bReleaseBindings && isBound();
	if (bUnbind)
		unBind();

	SdbCursor::SdbCursorMode eMode = SdbCursor::SDB_MOD_NONE;
	BOOL bResult = fetchNext(eMode);
	if (bResult)
	{
		if (++i == 50 && m_rOrigCursor.GetParent())
			m_rOrigCursor.GetConnection()->BusyHdl().Call(&m_rOrigCursor);
	}
	else
		m_rOrigCursor.bRowCountFinal = TRUE;

	if (bUnbind)
		bind();
	return bResult;
}

//--------------------------------------------------------------------------
BOOL OKeySet::isUpdatable() const
{
	DBG_CHKTHIS(OKeySet,NULL);
	return m_nPrivileges > SDB_PR_READ;
}

//--------------------------------------------------------------------------
BOOL OKeySet::fillSet(UINT32 nNewPos)
{
	// Auffuellen des Sets bis zur Position nPos
	DBG_ASSERT(nNewPos >= (INT32)size(), "Falsche Position");

	BOOL bResult = sal_False;
	BOOL bIsBound = isBound();
	if (bIsBound)
		unBind();							 // Bindungen aufheben

	UINT32 i = 0;
	m_nCurrentPos = (INT32)size() - 1;

	BeginCancelable();
	while ((bResult = getNext(i)) && (nNewPos > (INT32)size()) && !WasCanceled())
	{
		m_nCurrentPos++;
		if (i == 100)
		{
			m_rOrigCursor.nRowCount += i;
			m_rOrigCursor.GetRowCountHdl().Call(&m_rOrigCursor);
			i = 0;
		}
	}
	if (i)
	{
		m_rOrigCursor.nRowCount += i;
		m_rOrigCursor.GetRowCountHdl().Call(&m_rOrigCursor);
		i = 0;
	}
	m_nCurrentPos++;
	EndCancelable();
		// the caller need's to check if it was canceled

	// wird kein datensatz gefunden, mu der Cursor hinter dem letzten stehen
	DBG_ASSERT((bResult && m_nCurrentPos < size()) ||
			   (!bResult && m_nCurrentPos == size()), "Falsche Satzposition");

	if (bIsBound)
		bind();

	return bResult;
}

//--------------------------------------------------------------------------
BOOL OKeySet::seek(const ::staruno::Any& rBookmark,
				   SdbCursor::SdbCursorMode& rMode, BOOL bRetrieveData, BOOL bStopCounter) throw( ::starlang::IllegalArgumentException )
{
	DBG_CHKTHIS(OKeySet,NULL);

	INT32 nTempPos(m_nCurrentPos);
	BOOL  bResult = getBookmarkPos(m_nCurrentPos, rBookmark);
	if (!bResult)
	{
		m_rOrigCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		return sal_False;
	}

	DBG_ASSERT(m_nCurrentPos <= (INT32)size(), "Fehler im KeySet");

	// lesen des naechsten Satzes, aber nur wenn
	// der Rowcount nicht final
	if (m_nCurrentPos == (INT32)size() &&
		(!m_rOrigCursor.IsRowCountFinal() || (INT32)size() < m_rOrigCursor.RowCount()))
	{		
		// let the thread wait
		if (m_pCounter && m_pCounter->isRunning())
			m_pCounter->sleep(TTimeValue(2000));
		
		bResult = fetchNext(rMode);
	}
	// aus dem Bereich herausgelaufen?
	else if (m_nCurrentPos < 0 || m_nCurrentPos >= (INT32)size())
	{
		m_rOrigCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		bResult = sal_False;
	}
	else
	{
		// let the thread wait
		if (m_pCounter && m_pCounter->isRunning())
			m_pCounter->sleep(TTimeValue(2000));

		rMode = seekPositioned(bRetrieveData) ? SdbCursor::SDB_MOD_NONE : SdbCursor::SDB_MOD_INVALID;
	}

	if (!bResult)
		m_nCurrentPos = nTempPos;	 // vorherige Position
														  
	return bResult;
}	

//--------------------------------------------------------------------------
BOOL OKeySet::seek(SdbCursorPosition eCursorPosition, INT32 nOffset,
						SdbCursor::SdbCursorMode& rMode, BOOL bRetrieveData, BOOL bStom_pCounter)
{
	DBG_CHKTHIS(OKeySet,NULL);

	INT32 nTempPos(m_nCurrentPos);
	BOOL  bResult(TRUE);
	switch(eCursorPosition)
	{
		case SDB_POS_NEXT:
			m_nCurrentPos++;
			break;
		case SDB_POS_PRIOR:
			if (m_nCurrentPos >= 0)
				m_nCurrentPos--;
			break;
		case SDB_POS_FIRST:
			m_nCurrentPos = 0;
			break;
		case SDB_POS_LAST:
			DBG_ASSERT(m_rOrigCursor.IsRowCountFinal(), "Fehler im Keyset!");
			m_nCurrentPos = size() - 1;
			break;
		case SDB_POS_RELATIVE:
			// Satze bis einen vor der letzten Position lesen
			if ((m_nCurrentPos + nOffset) > (INT32)size())
			{
				if (!(bResult = fillSet(m_nCurrentPos + nOffset)))
					goto End;
				else
					m_nCurrentPos++;
			}
			else
				m_nCurrentPos += nOffset;
			break;
		case SDB_POS_ABSOLUTE:
			// (Absolut-Positionierung ist 1-basiert !)
			// Satze bis einen vor der letzten Position lesen
			if ((nOffset - 1) > (INT32)size())
			{
				if (!(bResult = fillSet(nOffset - 1)))
					goto End;
			}
			m_nCurrentPos = nOffset - 1;
			break;
	}

	DBG_ASSERT(m_nCurrentPos <= (INT32)size(), "Fehler im KeySet");

	// lesen des naechsten Satzes, aber nur wenn
	// der Rowcount nicht final
	if (m_nCurrentPos == (INT32)size() &&
		(!m_rOrigCursor.IsRowCountFinal() || (INT32)size() < m_rOrigCursor.RowCount()))
	{
		// let the thread wait
		if (m_pCounter && m_pCounter->isRunning())
			m_pCounter->sleep(TTimeValue(2000));
		
		bResult = fetchNext(rMode);
	}
	// aus dem Bereich herausgelaufen?
	else if (m_nCurrentPos < 0 || m_nCurrentPos >= (INT32)size())
	{
		m_rOrigCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		bResult = sal_False;
	}
	else
	{
		// let the thread wait
		if (m_pCounter && m_pCounter->isRunning())
			m_pCounter->sleep(TTimeValue(2000));
		
		rMode = seekPositioned(bRetrieveData) ? SdbCursor::SDB_MOD_NONE : SdbCursor::SDB_MOD_INVALID;
	}

	if (!bResult)
	{
		switch(eCursorPosition)
		{
			case SDB_POS_PRIOR:
			case SDB_POS_FIRST:
				m_nCurrentPos = -1;
				break;
			case SDB_POS_LAST:
			case SDB_POS_NEXT:
			case SDB_POS_ABSOLUTE:
			case SDB_POS_RELATIVE:
				if (nOffset > 0)
					m_nCurrentPos = size();
				else if (nOffset < 0)
					m_nCurrentPos = -1;
				break;
/*			case SDB_POS_BOOKMARK:
				m_nCurrentPos = nTempPos;	 // vorherige Position
*/		}
	}

End:
	return bResult;
}

//--------------------------------------------------------------------------
void OKeySet::prepareAppendRow(const ODbRowRef& rRow)
{
	DBG_CHKTHIS(OKeySet,NULL);
	DBG_ASSERT(m_aRefetchTableName.Len() , "OKeySet::prepareAppendRow, no TableName for select!");

	// einfuegezeile mit Autowerten bestuecken
	// diese Werte muessen nicht richtig sein
	// da zwischen append und Insert einer Zeile eine beliebige Zeitspanne liegen kann
	// in der andere Benutzer Zeile einfuegen koennen
	String aColStr;
	SdbConnection& rConnection = *m_rOrigCursor.GetConnection();
	UINT16 i;
	for (i = 1; i < m_xResultColumns->Count(); i++)
	{
		const SdbColumn& rCol	= *(*m_xResultColumns)[i];
		if (rCol.IsAutoIncrement()) // kann daneben gehen
		{
			aColStr.AppendAscii("MAX(");
			if(rCol.IsFunction())
				aColStr += rCol.GetRealName();  // add to column list
			else
				aColStr += rConnection.IdentifierQuotation(rCol.GetRealName());  // add to column list
			aColStr.AppendAscii("),");
		}
	}

	if (aColStr.Len())
	{
		String aStr;
		aStr.AssignAscii("SELECT ");
		aColStr.EraseTrailingChars(',');
		aStr += aColStr;
		aStr.AppendAscii(" FROM ");
		aStr += m_aRefetchTableName;

		SdbCursorRef xCursor(rConnection.CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY));
		if (xCursor->Open(aStr))
		{
			USHORT j = 1;
			ODbRowRef xSrcRow = xCursor->GetRow();
			for (i = 1; i < m_xResultColumns->Count(); i++)
			{
				if ((*m_xResultColumns)[i]->IsAutoIncrement())
				{
					ODbVariantRef xSrcVar = (*xSrcRow)[j++];
					ODbVariantRef xTargetVar = (*rRow)[i];
					if (xSrcVar->isNull())
					{
						// xSrcVar->setINT32(1);		// das kann nicht richtig sein, oder ??
						xTargetVar->setINT32(1);
					}
					else
						xTargetVar->setINT32(xSrcVar->getINT32() + 1);
					xTargetVar->setModified(sal_False);
				}
			}
		}
	}
}

//--------------------------------------------------------------------------
BOOL OKeySet::insertRow()
{
	DBG_CHKTHIS(OKeySet,NULL);

	if (!doInsertRow())
		return sal_False;

	if ((m_nCurrentPos < 0) && (m_rOrigCursor.nCurrentPos == 0) && m_rOrigCursor.IsEOF())
		// Ich befinde mich auf dem einem ungueltigen Datensatz und mein Cursor auf dem nullten und gleichzeitig EOF
		// -> der gerade neu eingefuegte Datensatz ist der erste. In diesem Fall steht der Cursor allein durch das Einfuegen
		// auf dem (neuen) ersten Datensatz (denn seine current pos wird nicht geaendert durch das insert), das muss fuer mich
		// selber jetzt also auch zutreffen.
		m_nCurrentPos = 0;

	return TRUE;
}

//--------------------------------------------------------------------------
BOOL OKeySet::updateRow()
{
	DBG_CHKTHIS(OKeySet,NULL);
	if (doUpdateRow())
	{
		// saave the status and refetch the row to reflect all changes espacially calculated fields
		SdbStatus aStatus(m_rOrigCursor.aStatus);
		// Nachfetchen der Zeile ohne blobs
		if (seekPositioned(TRUE))		
			m_rOrigCursor.aStatus = aStatus;
		return m_rOrigCursor.aStatus.IsSuccessful();
	}
	else
		return sal_False;
}

//--------------------------------------------------------------------------
BOOL OKeySet::deleteRow()
{
	DBG_CHKTHIS(OKeySet,NULL);
	return doDeleteRow();
}

//--------------------------------------------------------------------------
void OKeySet::determineRowCount()
{
	DBG_CHKTHIS(OKeySet,NULL);

	if (m_rOrigCursor.bRowCountFinal)
		return;

	if (m_pCounter && m_pCounter->isRunning())
	{
		// warten auf das beenden des Threads
		m_pCounter->terminate();
	    m_pCounter->join();
	}
    
	BOOL bIsBound = isBound();
	if (bIsBound)
		unBind();

	UINT32 i = 0;
	BeginCancelable();
	while (!m_rOrigCursor.bRowCountFinal && !WasCanceled())
	{
		BOOL bNext = getNext(i);
		if ((bNext && i == 100) || (!bNext && i))
		{
			m_rOrigCursor.nRowCount += i;
			m_rOrigCursor.GetRowCountHdl().Call(&m_rOrigCursor);
			i = 0;
		}
	}
	EndCancelable();
		// the caller need's to check if it was canceled

	if (bIsBound)
		bind();
}

//--------------------------------------------------------------------------
DBG_NAME(SdbQueryAnalyzer);

//------------------------------------------------------------------------------
class SdbQueryAnalyzer
{
	enum JoinType
	{
		Undefined,
		OneToOne,
		ManyToOne,
		OneToMany
	}								m_ePrevJoinType;

	// contains the list of table candidates
	vector<SdbTable*>	 			m_aTableCandidates;
	vector<SdbColumn*>				m_aLeftColumns, m_aRightColumns;
	SdbTableRef						m_xPrevLeftTable, m_xLeftTable, m_xRightTable;

	const SdbSqlParseTreeIterator&	m_rParseIterator;

public:
	SdbQueryAnalyzer(const SdbSqlParseTreeIterator&	rParseIterator)
			:m_rParseIterator(rParseIterator)
			,m_ePrevJoinType(Undefined){DBG_CTOR(SdbQueryAnalyzer,NULL);}
	~SdbQueryAnalyzer(){DBG_DTOR(SdbQueryAnalyzer,NULL);}

	// Looks for tables which could be refetched or updated
	BOOL execute();

	// List of possible table candidates
	const vector<SdbTable*>& getCandidates() const {return m_aTableCandidates;}

private:
	BOOL checkOneTableKeyset();
	BOOL checkMultiTableKeyset();
	BOOL checkJoin(SdbSqlParseNode *pNode);
	BOOL checkJoinCondition(SdbSqlParseNode *pNode);
	void extendResultColumns(const SdbColumnsRef& rxTableCols);
};

//--------------------------------------------------------------------------
BOOL SdbQueryAnalyzer::execute()
{
	DBG_CHKTHIS(SdbQueryAnalyzer,NULL);

	if(SDB_ISRULE(m_rParseIterator.ParseTree(),union_statement)) // union not supported
		return sal_False;
	// kein GroupBy im Statement
	SdbSqlParseNode * pTableExp = m_rParseIterator.ParseTree()->GetChild(3);
	DBG_ASSERT(pTableExp != NULL,"SdbQueryAnalyzer: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp),"SdbQueryAnalyzer: Fehler im Parse Tree");

	SdbSqlParseNode* pGroupByTree = pTableExp->GetChild(2);
	if (pGroupByTree && pGroupByTree->Count() == 3)
		return sal_False;

	// kein UNION: TODO
	if (m_rParseIterator.GetSdbTableRangeMap().Count() == 1)
	{
		// es gibt nur einen Tabellenkandidaten
		SdbTableRef xTable(*m_rParseIterator.GetSdbTableRangeMap().GetObject(0));
		if (!xTable)
			return sal_False;

		m_aTableCandidates.push_back(xTable);
		return TRUE;
	}
	else if (m_rParseIterator.GetSdbTableRangeMap().Count() > 1)
	{
		return checkMultiTableKeyset();
	}
	else // == 0
	{
		DBG_ERROR("SdbQueryAnalyzer::execute: no tables !");
		return sal_False;
	}
}

//------------------------------------------------------------------------------
BOOL SdbQueryAnalyzer::checkMultiTableKeyset()
{
	DBG_CHKTHIS(SdbQueryAnalyzer,NULL);

	// es ist hier nur eine Tabelle vorhanden
	DBG_ASSERT(m_rParseIterator.GetSdbTableRangeMap().Count() > 1,
		"SdbQueryAnalyzer::checkOneTableKeyset(): Mehr als eine Tabelle vorhanden");

	// ein Join ist nicht updatebar/refetchable, wenn:
	// a.) eine Tabelle eigentlich eine View ist
	// b.) Join basierend auf "Select ..." durchgefhrt wird
	// c.) Full Outer Join, Cross Join, Natural Join, UNION
	// d.) Outer Join (Dieses wird spter noch implementiert werden) TODO

	// a.)
	for (INT32 i = 0; i < m_rParseIterator.GetSdbTableRangeMap().Count(); i++)
	{
		SdbTableRef xTable = *m_rParseIterator.GetSdbTableRangeMap().GetObject(i);
		if (!xTable || xTable->IsView())
			return sal_False;
	}

	// b.) Tabellenliste auf joins berprfen
	SdbSqlParseNode * pTableRefCommalist = m_rParseIterator.ParseTree()->GetChild(3)->GetChild(0)->GetChild(1);
	DBG_ASSERT(pTableRefCommalist != NULL,"SdbQueryAnalyzer::checkMultiTableKeyset(): Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pTableRefCommalist,table_ref_commalist),"SdbQueryAnalyzer::checkMultiTableKeyset(): Fehler im Parse Tree");

	if (pTableRefCommalist->Count() == 1 &&
			(	SDB_ISRULE(pTableRefCommalist->GetChild(0),qualified_join)	||
				SDB_ISRULE(pTableRefCommalist->GetChild(0),joined_table)	||
				(	SDB_ISRULE(pTableRefCommalist->GetChild(0),table_ref) && 
					SDB_ISPUNCTUATION(pTableRefCommalist->GetChild(0)->GetChild(0),"{")
				)
			)
		)
		return checkJoin(pTableRefCommalist->GetChild(0));
	else
	// es wird kein qualified join verwendet
		return sal_False;
}

//-----------------------------------------------------------------------------
BOOL SdbQueryAnalyzer::checkJoin(SdbSqlParseNode *pNode)
{
	DBG_CHKTHIS(SdbQueryAnalyzer,NULL);
	DBG_ASSERT(SDB_ISRULE(pNode, qualified_join) || SDB_ISRULE(pNode, joined_table) ||
		(SDB_ISRULE(pNode,table_ref) && SDB_ISPUNCTUATION(pNode->GetChild(0),"{")),
		"SdbQueryAnalyzer::checkJoin(): Fehler im Parse Tree");

	if (SDB_ISRULE(pNode,joined_table))
		return checkJoin(pNode->GetChild(1));
	else if(SDB_ISRULE(pNode,table_ref) && SDB_ISPUNCTUATION(pNode->GetChild(0),"{"))
		return checkJoin(pNode->GetChild(2));


	if (SDB_ISRULE(pNode->GetChild(0),qualified_join))
	{
		if (!checkJoin(pNode->GetChild(0)))
			return sal_False;
	}
	else if (SDB_ISRULE(pNode->GetChild(0), joined_table))
	{
		if (!checkJoin(pNode->GetChild(0)->GetChild(1)))
			return sal_False;
	}
	else if (!SDB_ISRULE(pNode->GetChild(0), table_ref) ||
		!SDB_ISRULE(pNode->GetChild(0)->GetChild(0), table_name))
		return sal_False;

	// ist es ein inner join?
	if (!SDB_ISRULE(pNode->GetChild(1), join_type) ||
		!SDB_ISTOKEN(pNode->GetChild(1)->GetChild(0), "INNER"))
		return sal_False;

	// geschachtelter join?
	if (SDB_ISRULE(pNode->GetChild(3),qualified_join))
	{
		if (!checkJoin(pNode->GetChild(3)))
			return sal_False;
	}
	else if (SDB_ISRULE(pNode->GetChild(3), joined_table))
	{
		if (!checkJoin(pNode->GetChild(3)->GetChild(1)))
			return sal_False;
	}
	// sonst sollte es eine Tabelle sein
	else if (!SDB_ISRULE(pNode->GetChild(3), table_ref) ||
		!SDB_ISRULE(pNode->GetChild(3)->GetChild(0), table_name))
		return sal_False;

	// named column join wird spter vieleicht noch implementiert
	// SDB_ISRULE(pNode->GetChild(4),named_columns_join)
	if (!SDB_ISRULE(pNode->GetChild(4),join_condition) ||
		!checkJoinCondition(pNode->GetChild(4)->GetChild(1)))
		return sal_False;

	// Auswerten der Spalten
	// Zunaechst zaehlen der gefundenen PKeys
	// Links:
	INT32 nLeftTotalKeys = 0, nLeftKeys = 0,
		nRightTotalKeys = 0, nRightKeys = 0;

	SdbColumnsRef xColumns = m_xLeftTable->OriginalColumns();
	USHORT i, nCount;
	for (i = 1, nCount = xColumns->Count(); i < nCount; i++)
	{
		SdbColumnRef xColumn = (SdbColumn*)(*xColumns)[i];
		if (xColumn->IsPrimaryKey())
		{
			nLeftTotalKeys++;
			// jetzt die linken Columns durchsuchen auf die Column
			vector<SdbColumn*>::const_iterator i = m_aLeftColumns.begin();
			while (i != m_aLeftColumns.end())
			{
				if ((*i) == (SdbColumn*)xColumn.getBodyPtr())
				{
					nLeftKeys++;
					break;
				}
				i++;
			}
		}
	}

	xColumns = m_xRightTable->OriginalColumns();
	for (i = 1, nCount = xColumns->Count(); i < nCount; i++)
	{
		SdbColumnRef xColumn = (SdbColumn*)(*xColumns)[i];
		if (xColumn->IsPrimaryKey())
		{
			nRightTotalKeys++;
			// jetzt die linken Columns durchsuchen auf die Column
			vector<SdbColumn*>::const_iterator i = m_aRightColumns.begin();
			while (i != m_aRightColumns.end())
			{
				if ((*i) == (SdbColumn*)xColumn.getBodyPtr())
				{
					nRightKeys++;
					break;
				}
				i++;
			}
		}
	}

	JoinType eJoinType;

	// Es ist kein korrekter Join wenn:
	// a.) beide Seiten keine Keys enthalten
	if (!nLeftKeys && !nRightKeys)
		return sal_False;
	// b.) beide Seiten einen Keys enthalten aber
	else if (nLeftKeys && nRightKeys)
	{
		if (nLeftKeys != nRightKeys ||
			(nLeftKeys < nLeftTotalKeys && nRightKeys < nRightTotalKeys ))
			return sal_False;

		if (nLeftTotalKeys == nRightTotalKeys)
			eJoinType = OneToOne;
		else if (nLeftTotalKeys < nRightTotalKeys)
			eJoinType = OneToMany;
		else
			eJoinType = ManyToOne;
	}
	else if (nLeftKeys)
	{
		if (nLeftKeys != nLeftTotalKeys)
			return sal_False;
		else
			eJoinType = OneToMany;
	}
	else if (nRightKeys == nRightTotalKeys)
		eJoinType = ManyToOne;
	else
		return sal_False;

	if (eJoinType == ManyToOne)
	{
		// Vertauschen das macht das Leben einfacher
		SdbTableRef xTable(m_xLeftTable);
		m_xLeftTable = m_xRightTable;
		m_xRightTable = xTable;

		eJoinType = OneToMany;
	}

	// Jetzt werden die candidates ermittelt
	// dabei gibt es folgende Matrix
	// Zellen enthalten die Kandidaten
	// ----------------------------
	// 	  |a:c|b:c|a:c|b:c|c:a|c:b
	// a:b|1:1|1:1|1:n|1:n|1:n|1:n
	// ----------------------------
	// 1:1|abc|abc|c  |c  |ab |ab
	// 1:n|b  |bc |   |c  |b  |b

	switch (m_ePrevJoinType)
	{
		case Undefined:
		// einfacher Fall: es gab vorher keine Kandidaten
			switch (eJoinType)
			{
				case OneToOne:
					m_aTableCandidates.push_back(m_xLeftTable);
					m_aTableCandidates.push_back(m_xRightTable);
					break;
				case OneToMany:
					m_aTableCandidates.push_back(m_xRightTable);
					break;
				default:
					m_aTableCandidates.push_back(m_xLeftTable);
			}	break;
		case OneToOne:
			switch (eJoinType)
			{
				case OneToOne:
					// Tabellen nur hinzufgen, wenn sie noch nicht Kandidaten sind
					if (find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xLeftTable) == m_aTableCandidates.end())
						m_aTableCandidates.push_back(m_xLeftTable);
					if (find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xRightTable) == m_aTableCandidates.end())
						m_aTableCandidates.push_back(m_xRightTable);
					break;
				case OneToMany:
					if (find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xRightTable) == m_aTableCandidates.end())
					{
						m_aTableCandidates.erase(m_aTableCandidates.begin(), m_aTableCandidates.end());
						m_aTableCandidates.push_back(m_xRightTable);
					}
					// sonst bleibt alles beim alten
					break;
			}	break;
		case OneToMany:
			switch (eJoinType)
			{
				case OneToOne:
					if (find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xLeftTable) != m_aTableCandidates.end())
					{
						if (m_xLeftTable != m_xRightTable)
							m_aTableCandidates.push_back(m_xRightTable);
					}
					else if	(find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xRightTable) != m_aTableCandidates.end())
					{
						if (m_xLeftTable != m_xRightTable)
							m_aTableCandidates.push_back(m_xLeftTable);
					}
					break;
				case OneToMany:
					if (find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xLeftTable) != m_aTableCandidates.end())
					{
						m_aTableCandidates.erase(m_aTableCandidates.begin(), m_aTableCandidates.end());
						m_aTableCandidates.push_back(m_xRightTable);
					}
					else if (find(m_aTableCandidates.begin(), m_aTableCandidates.end(), m_xRightTable) != m_aTableCandidates.end())
					{
						// alles bleibt beim alten
					}
					else
					{
						if (m_xPrevLeftTable == m_xLeftTable)
							return sal_False;
					}
					break;
			}	break;

	}

	// merken des JoinTypes und der JoinTable (wird nur bentigt bei 1:n)
	m_ePrevJoinType = eJoinType;
	m_xPrevLeftTable = m_xLeftTable;

	m_xLeftTable = m_xRightTable = NULL;
	m_aLeftColumns = m_aRightColumns = vector<SdbColumn*>();

	return TRUE;
}

//-----------------------------------------------------------------------------
BOOL SdbQueryAnalyzer::checkJoinCondition(SdbSqlParseNode *pNode)
{
	DBG_CHKTHIS(SdbQueryAnalyzer,NULL);

	if (pNode->Count() == 3 &&	// Ausdruck is geklammert
		SDB_ISPUNCTUATION(pNode->GetChild(0),"(") &&
		SDB_ISPUNCTUATION(pNode->GetChild(2),")"))
	{
		return checkJoinCondition(pNode->GetChild(1));
	}
	else if (SDB_ISRULE(pNode,search_condition) ||
			 SDB_ISRULE(pNode,boolean_term)	&&			// AND/OR-Verknuepfung:
			 pNode->Count() == 3)
	{
		// nur AND Verknpfung zulassen
		if (!SDB_ISTOKEN(pNode->GetChild(1),"AND"))
			return sal_False;
		else
			return checkJoinCondition(pNode->GetChild(0)) &&
				   checkJoinCondition(pNode->GetChild(2));
	}
	else if (SDB_ISRULE(pNode,comparison_predicate))
	{
		// Nur ein Vergleich auf Spalten ist erlaubt
		DBG_ASSERT(pNode->Count() == 3,"SdbQueryAnalyzer::checkJoinCondition(): Fehler im Parse Tree");
		if (!(SDB_ISRULE(pNode->GetChild(0),column_ref) &&
			  SDB_ISRULE(pNode->GetChild(2),column_ref) &&
			   pNode->GetChild(1)->GetNodeType() == SDB_SQL_EQUAL))
			return sal_False;

		// Sammeln der Spalten und Tabellen
		String aColumnName, aTableRange;

		// zuerst links
		m_rParseIterator.GetColumnRange(pNode->GetChild(0), aColumnName, aTableRange);

		SdbTableRef xLeftTable = *m_rParseIterator.GetSdbTableRangeMap().Lookup(aTableRange);
		if (!xLeftTable)
			return sal_False;

		SdbColumnRef xLeftColumn = (SdbColumn*) xLeftTable->OriginalColumns()->Column(aColumnName);
		if (!xLeftColumn.is())
			return sal_False;

		// jetzt rechts
		m_rParseIterator.GetColumnRange(pNode->GetChild(2), aColumnName, aTableRange);
		SdbTableRef xRightTable = *m_rParseIterator.GetSdbTableRangeMap().Lookup(aTableRange);
		if (!xLeftTable)
			return sal_False;

		SdbColumnRef xRightColumn = (SdbColumn*) xRightTable->OriginalColumns()->Column(aColumnName);
		if (!xRightColumn.is())
			return sal_False;

		if (xRightColumn == xLeftColumn) // es sollten nicht die gleichen Spalten sein
			return sal_False;

		if (!m_xLeftTable)	// noch kein Tabellen zugeordnet
		{
			m_xLeftTable = xLeftTable;
			m_xRightTable = xRightTable;
		}
		else
		{
			// sind die Seiten vertauscht?
			if (m_xRightTable == xLeftTable)
			{
				SdbColumnRef xColumn = xLeftColumn;
				xLeftColumn = xRightColumn;
				xRightColumn = xColumn;
			}

			// existieren die Columns schon
			vector<SdbColumn*>::const_iterator i = m_aLeftColumns.begin();
			while (i != m_aLeftColumns.end())
			{
				if ((*i) == (SdbColumn*)xLeftColumn.getBodyPtr())
					return TRUE;
				i++;
			}

			i = m_aRightColumns.begin();
			while (i != m_aRightColumns.end())
			{
				if ((*i) == (SdbColumn*)xRightColumn.getBodyPtr())
					return TRUE;
				i++;
			}
		}

		m_aLeftColumns.push_back(xLeftColumn.getBodyPtr());
		m_aRightColumns.push_back(xRightColumn.getBodyPtr());

		return TRUE;
	}
	else
		return sal_False;
}

//--------------------------------------------------------------------------
DBG_NAME(OValueSet);
//--------------------------------------------------------------------------
OValueSet::OValueSet(SdbCursor& rCursor)
			:OKeySet(rCursor)

{
	DBG_CTOR(OValueSet,NULL);
	m_aRowSet.reserve(16);

	SdbTableRef xTable;

	// Feststellen ob Daten dynamisch nachgelesen werden koennen
	// kein Parsetree -> Snapshot, nicht updatable
	if (!m_rOrigCursor.pParseIterator ||
		m_rOrigCursor.pParseIterator->StatementType() != SDB_STATEMENT_SELECT)
	{
		m_bSnapshot = TRUE;
	}
	else
	{
		SdbQueryAnalyzer aAnalyzer(*m_rOrigCursor.pParseIterator);
		if (!m_rOrigCursor.pParseIterator->GetSdbTableRangeMap().Count())
			// if the parse iterator couldn't analyze the statement we don't have to ask the analyzer for the
			// KeyColumns
			// 73757 - 02.03.00 - FS
			m_bSnapshot = TRUE;
		else if (aAnalyzer.execute())
		{
			vector<SdbTable*>::const_iterator i;
			for (i = aAnalyzer.getCandidates().begin();
				 i != aAnalyzer.getCandidates().end(); i++)
			{
				xTable = *i;
				if (checkColumns(xTable))
					break;
			}

			m_bSnapshot = i == aAnalyzer.getCandidates().end();
		}
	}

	if (m_bSnapshot || !createCursor(xTable))
	{
		// alle Werte werden vom Keyset gehalten -> Snapshot
		m_aKeyColumns = vector<OKeySetColumn>(m_xResultColumns->Count() - 1);
		for (INT32 i = 0; i < m_aKeyColumns.size(); i++)
			m_aKeyColumns[i] = OKeySetColumn(i+1,sal_False);

		m_bSnapshot = TRUE;
	}
}

//--------------------------------------------------------------------------
OValueSet::~OValueSet()
{
	DBG_DTOR(OValueSet,NULL);
}

//--------------------------------------------------------------------------
BOOL OValueSet::createCursor(const SdbTableRef& xTable)
{
	DBG_CHKTHIS(OValueSet,NULL);

	if (!xTable)
		return sal_False;

	const SdbSqlParseTreeIterator& rParseIterator = *m_rOrigCursor.pParseIterator;

	SdbConnection& rConnection = *m_rOrigCursor.GetConnection();
	SdbColumnsRef xTableCols(xTable->OriginalColumns());

	// verwenden eines korrekt quotierten Tabellennamens
	m_aRefetchTableName = rConnection.QuoteTableName(xTable->GetFullName());
	DBG_ASSERT(m_aRefetchTableName.Len() , "SdbQueryAnalyzer::Init(), no TableName for Update!");

	const SdbColumnsRef xSelectCols(rParseIterator.GetSelectColumns());

	BOOL bConcurrency = m_rOrigCursor.nOptions & SDB_CONCURRENCY;
	// welche Spalten muessen im Cache gehalten werden
	for (USHORT i = 1, nCount = m_xResultColumns->Count(); i < nCount; i++)
	{
		SdbColumn* pCol = (SdbColumn*)(*m_xResultColumns)[i];

		// Unique und Element der RefetchTable
		if (pCol->IsUnique() && !(pCol->GetAllFlags() & SDB_FLAGS_VERSIONING))
			m_aKeyColumns.push_back(OKeySetColumn(i,TRUE));
		else if(bConcurrency && (pCol->GetAllFlags() & SDB_FLAGS_VERSIONING))
			m_aRefetchColumns.push_back(i);
		else if (xTable->GetFullName() == (*xSelectCols)[i]->GetTableName())
			m_aRefetchColumns.push_back(i);
		else
		{
			// kein autoinkrement, kein Update
			pCol->ResetFlag(SDB_FLAGS_AUTOINCREMENT);
			pCol->ResetFlag(SDB_FLAGS_UPDATEALLOWED);
			m_aKeyColumns.push_back(OKeySetColumn(i,sal_False));
		}
	}

	if (prepareCursor(m_aRefetchTableName))
	{
		m_nPrivileges |= xTable->GetTablePrivileges();
		m_nPrivileges &= ~SDB_TABLEPRIVILEGES;

		return TRUE;
	}
	else
	{
		m_aRefetchTableName = String();
		m_xBackgroundCursor = NULL;
		m_aRefetchColumns = vector<INT16>();
		m_aKeyColumns = vector<OKeySetColumn>();
		return sal_False;
	}
}

//--------------------------------------------------------------------------
BOOL OValueSet::checkColumns(const SdbTableRef& xTable)
{
	DBG_CHKTHIS(OValueSet,NULL);

	SdbColumnsRef xTableCols(xTable->OriginalColumns());

	// Suchen nach unique columns
	USHORT i;
	for (i = 1; i < xTableCols->Count() && !(*xTableCols)[i]->IsUnique(); i++)
		;

	// Tabelle besitzt keine Unique Identifier -> kein Keyset
	if (i >= xTableCols->Count())
		return sal_False;

	USHORT nLenTableCols(xTableCols->Count());
	USHORT nPKeyCols(0);
	for (i = 1; i < nLenTableCols; i++)
	{
		SdbColumn* pCol = (SdbColumn*)(*xTableCols)[i];
		if (pCol->IsPrimaryKey())
			nPKeyCols++;
	}
	if (nPKeyCols)
		nPKeyCols++;	// Zeigt an, da ein PKey existiert


	// Feststellen, welche Spalten nachgeladen werden koennen
	const SdbSqlParseTreeIterator& rParseIterator = *m_rOrigCursor.pParseIterator;
	USHORT nLenResultCols(m_xResultColumns->Count());
	const SdbColumnsRef xSelectCols(rParseIterator.GetSelectColumns());
	for (i = 1; i < nLenResultCols; i++)
	{
		SdbColumn* pCol = (SdbColumn*)(*m_xResultColumns)[i];
		// Stammt die Spalte aus der Tabelle?
		if (xTable->GetFullName() == (*xSelectCols)[i]->GetTableName())
		{
			if (!pCol->IsFunction())
			{
				const SdbColumn* pTableCol = xTableCols->Column(pCol->GetRealName());
				if (pTableCol->IsUnique())
				{
					pCol->SetFlag(SDB_FLAGS_UNIQUE);
					if (pTableCol->IsAutoIncrement())
						pCol->SetAutoIncrement();

					if (pTableCol->IsPrimaryKey())
					{
						pCol->SetFlag(SDB_FLAGS_PRIMARYKEY);
						nPKeyCols--;
					}
				}
			}
		}
	}

	// Primary Key verwenden?
	if (nPKeyCols == 1)
	{
		for (i = 1; i < nLenResultCols; i++)
		{
			SdbColumn* pCol = (SdbColumn*)(*m_xResultColumns)[i];
			if (!pCol->IsPrimaryKey())
				pCol->ResetFlag(SDB_FLAGS_UNIQUE);
		}
		return TRUE;
	}

	// Welche Spalten identifizieren minimal einen Datensatz
	// Index der die Datensaetze eindeutig identifiziert
	SdbIndexRef xIndex;
	BOOL bColumnsComplett = sal_False;
	USHORT nIndexCount = xTable->GetIndexCount();
	for (i = 0;
		 i < nIndexCount; i++)
	{
		xIndex = xTable->OpenIndex(xTable->GetIndexName(i));
		if (xIndex && xIndex->IsUnique())
		{
			SdbColumnsRef xColumns = xIndex->OriginalColumns();
			USHORT j;
			for (j = 1; j < xColumns->Count(); j++)
			{
				if (m_xResultColumns->ColumnNumber(xColumns->Column(j)->GetRealName()) == SDB_COLUMN_NOTFOUND)
					break;
			}
			// sind alle Spalten vorhanden?
			if (j >= xColumns->Count())
				break;
		}
		xIndex = NULL;
	}

	for (i = 1; i < nLenResultCols; i++)
		((SdbColumn*)(*m_xResultColumns)[i])->ResetFlag(SDB_FLAGS_PRIMARYKEY);

	// Jetzt die UniqueFlags wieder zuruecksetzen
	if (xIndex)
	{
		SdbColumnsRef xColumns = xIndex->OriginalColumns();
		for (USHORT j = 1; j < xColumns->Count(); j++)
			((SdbColumn*)m_xResultColumns->Column(xColumns->Column(j)->GetRealName()))->SetFlag(SDB_FLAGS_UNIQUE);
	}
	return xIndex.Is();
}

//--------------------------------------------------------------------------
void OValueSet::bind()
{
	DBG_CHKTHIS(OValueSet,NULL);
	if (!m_rOrigCursor.IsOpen() || m_bBound || m_bSnapshot)
	{
		m_bBound = TRUE;
		return;
	}

	// entfernt die Werte, nicht zum Refetch benoetigt werden
	ODbRow& rTarget = *m_rOrigCursor.aResultRow;		
	// free values not needed
	for (vector<ODbVariantRef>::iterator i = (m_xBindRow->begin() + 1),
		 j = (rTarget.begin() + 1);
		 i != m_xBindRow->end(); i++)
	{			
		if ((*i).is())
		{
			(*j) = (*i);
			(*i).unbind();
		}
		j++;
	}
	m_bBound = TRUE;
}

//--------------------------------------------------------------------------
void OValueSet::unBind()
{
	DBG_CHKTHIS(OValueSet,NULL);
	if (!m_rOrigCursor.IsOpen() || m_bSnapshot || !m_bBound)
	{
		m_bBound = sal_False;
		return;
	}

	// entfernt die Werte, nicht zum Refetch benoetigt werden
	const SdbColumns& rCols  = *m_xResultColumns;
	ODbRow& rTarget = *m_rOrigCursor.aResultRow;

	for (vector<ODbVariantRef>::iterator i = (m_xBindRow->begin() + 1),
		 j = (rTarget.begin() + 1);
		 i != m_xBindRow->end(); i++)
	{			
		if (IS_REFETCHABLE(j - rTarget.begin()))
		{
			(*i) = (*j); 
			(*j).unbind();
		}
		j++;		
	}
	m_bBound = sal_False;
}

//--------------------------------------------------------------------------
BOOL OValueSet::getBookmarkPos(INT32& nPos, const ::staruno::Any& rBookmark) const throw( ::starlang::IllegalArgumentException )
{
	// do we have a bookmark of valid type
	if (rBookmark.getValueTypeClass() != ::com::sun::star::uno::TypeClass_LONG)
		throw ::starlang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);

	// determine the position of the bookmark
	sal_Int32 nBookmark;
	rBookmark >>= nBookmark;
	OValueRow aSearchValue(nBookmark);	
	vector<OValueRow>::const_iterator i = lower_bound(m_aRowSet.begin(), m_aRowSet.end(), aSearchValue, OValueRowLess());

	if (i != m_aRowSet.end() && !OValueRowLess()(aSearchValue, *i))
		nPos = i - m_aRowSet.begin();
	
	return i != m_aRowSet.end();		
}

//--------------------------------------------------------------------------
BOOL OValueSet::seekPositioned(BOOL bRetrieveData) throw( ::starlang::IllegalArgumentException )
{
	DBG_CHKTHIS(OValueSet,NULL);
	// werden keine Daten benoetigt, sind wir schon fertig
	if (!m_rOrigCursor.IsOpen() || !bRetrieveData)
		return m_rOrigCursor.IsOpen();

	OValueRow& rKeyRow = m_aRowSet[m_nCurrentPos];
	ODbRow& rTarget = *m_rOrigCursor.aResultRow;
	SdbConnection& rConnection = *m_rOrigCursor.GetConnection();
	BOOL bResult(TRUE);

	m_rOrigCursor.aStatus.Clear();

	rKeyRow.getRow(rTarget,*this);
	if (m_xBackgroundCursor.Is())  // muss positioniert werden
	{
		if(!m_xBackgroundCursor->IsPrepared())
			prepareCursor(m_aRefetchTableName);
		// ist ein Parameterwert NULL, dann muss ein neues Selectstatement erstellt
		// wert (statt Feld = ? Feld IS NULL)
		BOOL bParamNull(sal_False);
		ODbRowRef aParameterRow(m_xBackgroundCursor->GetParameterRow());
		USHORT i, j;
		for (i = 0, j = 1; i < m_aKeyColumns.size(); i++)
		{
			if (m_aKeyColumns[i].bUnique && (*aParameterRow)[j].is())
			{
				(*aParameterRow)[j++]->setAny(rKeyRow[i]);
				bParamNull = bParamNull || !rKeyRow[i].hasValue();					
			}
		}

		DBG_ASSERT(m_aRefetchTableName.Len() , "OValueSet::seekPositioned(), no TableName for select!");
		if (bParamNull)
			prepareCursor(m_aRefetchTableName, &(*aParameterRow));

		bResult = m_xBackgroundCursor->Open(sal_False);

		if (bResult)
		{
			const ODbRowRef xDest(m_xBackgroundCursor->GetRow());
			for (i = 0; i < m_aRefetchColumns.size(); i++)
			{
				// nur uebertragen, wenn die Werte gebunden sind
				ODbVariantRef xVar = rTarget[m_aRefetchColumns[i]];
				if (xVar.is())
					*xVar = *(*xDest)[i+1];
			}
		}
		else
			m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();

		if (bParamNull)
			prepareCursor(m_aRefetchTableName, NULL);
	}
	return bResult;
}

//--------------------------------------------------------------------------
BOOL OValueSet::fetchNext(SdbCursor::SdbCursorMode& rMode)
{
	DBG_CHKTHIS(OValueSet,NULL);

	if (m_rOrigCursor.MoveImpl(SDB_POS_NEXT, 1, rMode))
	{
		// inkrement the bookmark
		sal_Int32 nVal= 0;
		if (m_aLastBookmark.hasValue())
			m_aLastBookmark >>= nVal;
		m_aLastBookmark <<= sal_Int32(nVal + 1);

		OValueRow aKeyRow(nVal + 1, *m_rOrigCursor.aResultRow, *this);
		m_aRowSet.push_back(aKeyRow);

		// Setzen der Bookmark wichtig
		m_rOrigCursor.aResultRow->front()->setAny(m_aLastBookmark);
		return TRUE;
	}
	return sal_False;
}

//--------------------------------------------------------------------------
BOOL OValueSet::prepareCursor(const String& rTableName, ODbRow* pRow)
{
	DBG_CHKTHIS(OValueSet,NULL);
	if (!m_rOrigCursor.IsOpen())
		return sal_False;

	String aStatement;
	aStatement.AssignAscii("SELECT ");
	String aColStr;
	const SdbColumns& rCols    = *m_rOrigCursor.GetColumns();
	SdbConnection& rConnection = *m_rOrigCursor.GetConnection();
	USHORT i;
	for (i = 1; i < rCols.Count(); i++)
	{
		if (!IS_REFETCHABLE(i))
			continue;

		const SdbColumn* pCol = rCols[i];

		if (pCol->IsFunction() && pCol->GetRealName() != pCol->GetName())
		{
			aColStr += pCol->GetRealName();
			aColStr.AppendAscii(" AS ");
			aColStr += rConnection.IdentifierQuotation(pCol->GetName());
		}
		else if (pCol->IsFunction())
			aColStr += pCol->GetRealName();
		else
			aColStr += rConnection.IdentifierQuotation(pCol->GetRealName());

		aColStr += ',';
	}

	if (!aColStr.Len())
	{
		aColStr = '*';
		//return TRUE;
	}

	aColStr.EraseTrailingChars(',');
	aStatement += aColStr;
	aStatement.AppendAscii(" FROM ");
	aStatement += rTableName;
	aStatement.AppendAscii(" WHERE ");

	BOOL bFirst = TRUE;
	for (i = 1; i < rCols.Count(); i++)
	{
		const SdbColumn* pCol = rCols[i];
		if (!pCol->IsUnique())
			continue;

		if (!bFirst)
			aStatement.AppendAscii(" AND ");
		else
			bFirst = sal_False;

		if(pCol->IsFunction())
			aStatement += pCol->GetRealName();
		else
			aStatement += rConnection.IdentifierQuotation(pCol->GetRealName());

		if (pRow)
		{
			ODbVariantRef xVar = (*pRow)[i];
			if (!xVar.is() || xVar->isNull())
				aStatement.AppendAscii(" IS NULL");
			else
				aStatement.AppendAscii(" = ?");
		}
		else
			aStatement.AppendAscii(" = ?");
	}

	if (m_xBackgroundCursor)
		m_xBackgroundCursor->Close();
	else
		m_xBackgroundCursor = rConnection.CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY|SDB_READONLY|SDB_INDIRECT);

	// sicherstellen das ein richtiger Type verwendet wird
	SdbColumnsRef aCols = m_xBackgroundCursor->aParameterColumns;
	for (i = 1; i < m_xResultColumns->Count(); i++)
	{
		const SdbColumn* pCol = (*m_xResultColumns)[i];
		if (pCol->IsUnique())
		{
			SdbColumn* pNewCol = rConnection.CreateColumn(*pCol);
			pNewCol->SetFlag(SDB_FLAGS_PARAM_IN);
			aCols->AddColumn(pNewCol);
		}
	}
	return m_xBackgroundCursor->Prepare(aStatement);
}

//------------------------------------------------------------------------------
BOOL OValueSet::appendWhere(String& aStr,USHORT nPos,BOOL bCurrency) const
{
	DBG_CHKTHIS(OValueSet,NULL);
	SdbConnection& rConnection = *m_rOrigCursor.GetConnection();

	BOOL bConcurrency = bCurrency && m_rOrigCursor.nOptions & SDB_CONCURRENCY;
	BOOL bRowVersion = sal_False;

	const SdbColumns& rCols		= *m_xResultColumns;
	const ODbRow& aResultRow	= *m_rOrigCursor.aResultRow;
	if(bConcurrency)
		bRowVersion = rCols.Count() != rCols.GetVisibleCount();

	const OValueRow& rKeyRow = m_aRowSet[nPos];
	String aCondStr;
	UINT16 i;
	for (i = 0; i < m_aKeyColumns.size(); i++)
	{
		if (m_aKeyColumns[i].bUnique)
		{
			if (aCondStr.Len())
				aCondStr.AppendAscii(" AND ");  // for the first key we don't add an "and"

			const SdbColumn* pCol = (*m_xResultColumns)[m_aKeyColumns[i].nId];
			if (pCol->IsFunction())
				aCondStr += pCol->GetRealName();
			else
				aCondStr += rConnection.IdentifierQuotation(pCol->GetRealName());  // add to column list

			if (!rKeyRow[i].hasValue())
				aCondStr.AppendAscii(" IS NULL");
			else
				aCondStr.AppendAscii(" = ?");
		}
	}

	if (bConcurrency)
	{
		if (bRowVersion)
		{
			for(UINT16 j=1;j<rCols.Count();++j)
			{
				SdbColumn* pCol = (SdbColumn*)rCols[j];
				if (pCol->GetAllFlags() & SDB_FLAGS_VERSIONING)
				{
					if (aCondStr.Len())
						aCondStr.AppendAscii(" AND ");  // for the first key we don't add an "and"
					const SdbColumn* pCol = (*m_xResultColumns)[m_aKeyColumns[i].nId];
					if (pCol->IsFunction())
						aCondStr += pCol->GetRealName();
					else
						aCondStr += rConnection.IdentifierQuotation(pCol->GetRealName());  // add to column list

					if (!rKeyRow[i].hasValue())
						aCondStr.AppendAscii(" IS NULL");
					else
						aCondStr.AppendAscii(" = ?");
				}
			}
		}
		else
		{
			for(UINT16 j=1;j<rCols.Count();++j)
			{
				if (!rCols[j]->IsUnique() && IS_SEARCHABLE(rCols[j]) && !aResultRow[j]->isModified())
				{
					if (aCondStr.Len())
						aCondStr.AppendAscii(" AND ");  // for the first key we don't add an "and"
					
					if(rCols[j]->IsFunction())
						aCondStr += rCols[j]->GetRealName();
					else
						aCondStr += rConnection.IdentifierQuotation(rCols[j]->GetRealName());  // add to column list
					
					if (aResultRow[j]->isNull())
						aCondStr.AppendAscii(" IS NULL");
					else
						aCondStr.AppendAscii(" = ?");
				}
			}
		}
	}
	aStr += aCondStr;
	return aCondStr.Len() != 0;
}

//--------------------------------------------------------------------------
BOOL OValueSet::doInsertRow()
{
	DBG_CHKTHIS(OValueSet,NULL);
	if (!m_rOrigCursor.IsOpen())
		return sal_False;

	SdbConnection& rConnection = *m_rOrigCursor.GetConnection();

	sal_Bool bAdabasAutoIncrement = sal_False;

	const SdbColumn* pAutoCol;
	for (sal_Int32 k = 1; k < m_xResultColumns->Count(); k++)
	{
		const SdbColumn& rCol = *(*m_xResultColumns)[k];
		// this can only be a adabas autoinc
		if(rCol.IsAutoIncrement() && rCol.GetDefaultValue().Len())
		{
			pAutoCol = (*m_xResultColumns)[k];
			bAdabasAutoIncrement = TRUE;
			break;
		}
	}

	const ODbRow& rResultRow = *m_rOrigCursor.aResultRow;
	if(bAdabasAutoIncrement)
	{
		m_xBackgroundCursor->Close();
		// ermitteln der autoincrements
		// erst die Select Spalten und die Bedingungen fr die Autoinkrementwerte
		SdbColumnsRef aCols(m_xBackgroundCursor->aResultColumns);
		String aColStr;
		sal_Int16 i;
		for (i = 1; i < m_xResultColumns->Count(); i++)
		{
			const SdbColumn& rCol = *(*m_xResultColumns)[i];
			// this can only be a adabas autoinc
			if (rCol.IsAutoIncrement()) // kann daneben gehen
			{
				aColStr.AppendAscii("MAX(");
				
				if(rCol.IsFunction())
					aColStr += rCol.GetRealName();
				else
					aColStr += rConnection.IdentifierQuotation(rCol.GetRealName());  // add to column list
				
				aColStr.AppendAscii(")");
				aColStr += ',';
				aCols->AddColumn(rConnection.CreateColumn(rCol));
			}
		}

		// jetzt das Statement um die maxima zu ermitteln
		if (aColStr.Len())
		{
			String aStr;
			aStr.AssignAscii("SELECT ");
			aColStr.EraseTrailingChars(',');
			aStr += aColStr;
			aStr.AppendAscii(" FROM ");
			aStr += m_aRefetchTableName;

			if (m_xBackgroundCursor->Open(aStr))
			{
				ByteString aFormat;
				sal_Int16 j = 1;
				const ODbRow& rSrcRow = *m_xBackgroundCursor->aResultRow;				
				for (i = 1; i < m_xResultColumns->Count(); i++)
				{
					if ((*m_xResultColumns)[i]->IsAutoIncrement())
					{
						ODbVariantRef xSrcVar = rSrcRow[j++];
						if (xSrcVar->isNull())
						{
							xSrcVar->setString(String::CreateFromAscii("000000000000000001"));
						}

						ODbVariantRef xTargetVar = rResultRow[i];
						ByteString aVal(xSrcVar->toString().getStr(), RTL_TEXTENCODING_ASCII_US);
						sal_Int32 nLen = aVal.Len();
						sal_Int32 nStamp = strtol(aVal.GetBuffer(),NULL,16);
						++nStamp;
						
						aFormat = '%';
						aFormat += ByteString::CreateFromInt32(nLen);
						aFormat += 'X';

						sal_Char *pData = aVal.GetBufferAccess();
						sprintf(pData,aFormat.GetBuffer(),nStamp);
						aVal.SearchAndReplaceAll(' ','0');
						
						xTargetVar->setString(::rtl::OUString::createFromAscii(aVal.GetBuffer()));
						xTargetVar->setModified(sal_False);
					}
				}
			}
			m_xBackgroundCursor->Close();
		}
	}

	
	// INSERT INTO <tablename> (<colname1>[,<colname2>]) VALUES (?[,?])
	String aStr;
	aStr.AssignAscii("INSERT INTO ");
	aStr += m_aRefetchTableName;
	aStr.AppendAscii(" (");
	String aValues;
	aValues.AssignAscii(" VALUES (");

	
	vector<ODbVariantRef>::const_iterator i;
	for (i = (rResultRow.begin() + 1);	i != rResultRow.end(); ++i)
	{			
		const SdbColumn& rCol = *(*m_xResultColumns)[i - rResultRow.begin()];
		if ((*i)->isModified() || (bAdabasAutoIncrement && rCol.IsAutoIncrement()))
		{
			DBG_ASSERT(!(rCol.IsAutoIncrement() && !rCol.GetDefaultValue().Len()),"AutoInkrement darf nicht modifiziert sein!");
			if (IS_UPDATEABLE(rCol) || rCol.IsAutoIncrement())
			{
				aStr += rConnection.IdentifierQuotation(rCol.GetRealName());  // add to column list
				aStr += ',';

				aValues.AppendAscii("?,");
			}
		}		
	}

	// overwrite last ',' with ')' for the column list
	if (aStr.GetChar(aStr.Len()-1) == ',')
		aStr.SetChar(aStr.Len()-1,')' );
	else
	{
		m_rOrigCursor.aStatus.Set(SDB_STAT_ERROR);
		return sal_False;    // if there was no column appended, we don't execute the statement
	}

	// overwrite last ',' with ')' for the value list
	if (aValues.GetChar(aValues.Len()-1) == ',')
		aValues.SetChar(aValues.Len()-1,')');
	else
	{
		m_rOrigCursor.aStatus.Set(SDB_STAT_ERROR);
		return sal_False;    // if there was no column appended, we don't execute the statement
	}

	aStr += aValues;

	if (m_xBackgroundCursor.Is())
		m_xBackgroundCursor->Close();


	SdbColumnsRef aParamCols(m_xBackgroundCursor->aParameterColumns);
	for (i = (rResultRow.begin() + 1);	i != rResultRow.end(); ++i)
	{			
		const SdbColumn& rCol = *(*m_xResultColumns)[i - rResultRow.begin()];
		if ((*i)->isModified() || (bAdabasAutoIncrement && rCol.IsAutoIncrement()))
		{			
			DBG_ASSERT(!(rCol.IsAutoIncrement() && !rCol.GetDefaultValue().Len()),"AutoInkrement darf nicht modifiziert sein!");
			if (IS_UPDATEABLE(rCol) || rCol.IsAutoIncrement())
			{
				SdbColumn* pCol = rConnection.CreateColumn(rCol);
				pCol->SetFlag(SDB_FLAGS_PARAM_IN);
				aParamCols->AddColumn(pCol);
			}
		}
	}

	//
	BOOL bResult;
	if (bResult = m_xBackgroundCursor->Prepare(aStr))
	{
		// Parameter fuellen
		ODbRowRef aParameterRow = m_xBackgroundCursor->GetParameterRow();
		USHORT j = 1;
		for (vector<ODbVariantRef>::const_iterator i = (rResultRow.begin() + 1);i != rResultRow.end(); ++i)
		{			
			const SdbColumn& rCol = *(*m_xResultColumns)[i - rResultRow.begin()];
			if ((*i)->isModified() || (bAdabasAutoIncrement && rCol.IsAutoIncrement()))
			{			
				DBG_ASSERT(IS_UPDATEABLE(rCol) || rCol.IsAutoIncrement(),"Column nicht updatebar!");
				if (IS_UPDATEABLE(rCol) || rCol.IsAutoIncrement())
					*(*aParameterRow)[j++] = *(*i);
			}
		}

		if (!(bResult = m_xBackgroundCursor->Open()))
			m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();
	}
	else
		m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();

	if (bResult)
	{
		m_xBackgroundCursor->Close();
		if(!bAdabasAutoIncrement)
		{
			// ermitteln der autoincrements
			// erst die Select Spalten und die Bedingungen fr die Autoinkrementwerte
			String aColStr,aWhere;		
			UINT16 i;
			for (i = 1; i < m_xResultColumns->Count(); i++)
			{
				const SdbColumn& rCol = *(*m_xResultColumns)[i];
				if (rCol.IsAutoIncrement()) // kann daneben gehen
				{
					aColStr.AppendAscii("MAX(");
					
					if(rCol.IsFunction())
						aColStr += rCol.GetRealName();
					else
						aColStr += rConnection.IdentifierQuotation(rCol.GetRealName());  // add to column list
					
					aColStr += ')';
					aColStr += ',';
					// Bedingung fuer die Autoinkremente
					if (aWhere.Len())
						aWhere.AppendAscii(" AND ");  // for the first key we don't add an "and"
					
					if(rCol.IsFunction())
						aWhere += rCol.GetRealName();
					else
						aWhere += rConnection.IdentifierQuotation(rCol.GetRealName());  // add to column list
					aWhere.AppendAscii(" = ?");
				}
			}

			// jetzt das Statement um die maxima zu ermitteln
			if (aColStr.Len())
			{
				String aStr;
				aStr.AssignAscii("SELECT ");
				aColStr.EraseTrailingChars(',');
				aStr += aColStr;
				aStr.AppendAscii(" FROM ");
				aStr += m_aRefetchTableName;


				if (m_xBackgroundCursor->Open(aStr))
				{
					USHORT j = 1;
					const ODbRow& rSrcRow = *m_xBackgroundCursor->aResultRow;				
					for (i = 1; i < m_xResultColumns->Count(); i++)
					{
						if ((*m_xResultColumns)[i]->IsAutoIncrement())
						{
							ODbVariantRef xSrcVar = rSrcRow[j++];
							if (xSrcVar->isNull())
								xSrcVar->setINT32(1);

							ODbVariantRef xTargetVar = rResultRow[i];
							*xTargetVar = *xSrcVar;
							xTargetVar->setModified(sal_False);
						}
					}
				}
				m_xBackgroundCursor->Close();
			}
		}

		// Datensatz eintragen
		// inkrement the bookmark
		sal_Int32 nVal= 0;
		if (m_aLastBookmark.hasValue())
			m_aLastBookmark >>= nVal;
		m_aLastBookmark <<= sal_Int32(nVal + 1);
		
		OValueRow aKeyRow(nVal + 1, rResultRow, *this);
		m_aRowSet.push_back(aKeyRow);

		rResultRow.front()->setAny(m_aLastBookmark);				

		INT32 nTempPos = m_nCurrentPos;
		m_nCurrentPos = m_aRowSet.size() -1;

		// Daten nochmals aktualisieren (Functions, Defaults)
		seekPositioned(TRUE);
		m_nCurrentPos = nTempPos;

		m_rOrigCursor.aStatus.Clear();
		m_rOrigCursor.aStatus.SetRecordsAffected(1);
	}
	else if (m_xBackgroundCursor.Is())
		m_xBackgroundCursor->Close();

	return bResult;
}

//--------------------------------------------------------------------------
BOOL OValueSet::doUpdateRow()
{
	DBG_CHKTHIS(OValueSet,NULL);
	DBG_ASSERT(m_nPrivileges > SDB_PR_READ && m_nCurrentPos >= 0 && m_nCurrentPos < (INT32)size(), "Falsche Postion im Set");
	if (m_nCurrentPos < 0 || m_nCurrentPos >= (INT32)size())
		return sal_False;

	if (!m_rOrigCursor.IsOpen())
		return sal_False;

	SdbConnectionRef xConnection(m_rOrigCursor.GetConnection());
	OValueRow& rKeyRow = m_aRowSet[m_nCurrentPos];
	const ODbRow& rResultRow = *m_rOrigCursor.aResultRow;

	String aStr;
	aStr.AssignAscii("UPDATE ");
	aStr += m_aRefetchTableName;
	aStr.AppendAscii(" SET ");

	for (vector<ODbVariantRef>::const_iterator i = (rResultRow.begin() + 1);	
		 i != rResultRow.end(); i++)
	{			
		if ((*i)->isModified())
		{			
			const SdbColumn& rCol = *(*m_xResultColumns)[i - rResultRow.begin()];			
			DBG_ASSERT(IS_UPDATEABLE(rCol),"Column nicht updatebar!");
			if (IS_UPDATEABLE(rCol))
			{
				aStr += xConnection->IdentifierQuotation(rCol.GetRealName());  // add to column list
				aStr.AppendAscii(" = ?,");
			}
		}
	}
	aStr.EraseTrailingChars(',');

	aStr.AppendAscii(" WHERE ");
	if (!appendWhere(aStr,m_nCurrentPos))
		return sal_False;

	if(m_xBackgroundCursor.Is())
		m_xBackgroundCursor->Close();

	appendParameterColumns();

	BOOL bResult;
	if (bResult = m_xBackgroundCursor->Prepare(aStr))
	{
		// Parameter fuellen,
		setParameterValues();

		if (!(bResult = m_xBackgroundCursor->Open()))
			m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();
		else if (!m_xBackgroundCursor->RowCount())	// kein datensatz veraendert
		{
			if(m_xBackgroundCursor.Is())
				m_xBackgroundCursor->Close();

			// check if another one modify my row
			checkConcurrency();			
			bResult = sal_False;
		}
		else
			m_rOrigCursor.aStatus.SetRecordsAffected(m_xBackgroundCursor->RowCount());
	}
	else
		m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();

	if (bResult)
	{
		// Unique werte uebernehmen
		for (UINT16 n = 0; n < m_aKeyColumns.size(); n++)
		{
			if (m_aKeyColumns[n].bUnique && rResultRow[m_aKeyColumns[n].nId]->isModified())
				rKeyRow[n] = rResultRow[m_aKeyColumns[n].nId]->getAny();								
		}
	}

	if (m_xBackgroundCursor)
		m_xBackgroundCursor->Close();

	return bResult;
}
//--------------------------------------------------------------------------
void OValueSet::checkConcurrency()
{
	String aStr;

	((aStr.AssignAscii("SELECT * FROM ")) += m_aRefetchTableName).AppendAscii(" WHERE ");
	if (appendWhere(aStr,m_nCurrentPos))
	{
		appendParameterColumns();
		if (m_xBackgroundCursor->Prepare(aStr))
		{
			setParameterValues();			
			
			if(m_xBackgroundCursor->Open() && !m_xBackgroundCursor->RowCount())
				m_rOrigCursor.aStatus.Set(SDB_STAT_ERROR,String::CreateFromAscii("01000"),String(SdbResId(STR_STAT_RECORD_MODIFIED)),0,String());
			else
				m_rOrigCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		}
		else 
			m_rOrigCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
	}
	else 
		m_rOrigCursor.aStatus.Set(SDB_STAT_NO_DATA_FOUND);
}
//--------------------------------------------------------------------------
void OValueSet::setParameterValues(BOOL bCurrency)
{
	const ODbRow& rResultRow = *m_rOrigCursor.aResultRow;
	ODbRowRef aParameterRow(m_xBackgroundCursor->GetParameterRow());
	OValueRow& rKeyRow = m_aRowSet[m_nCurrentPos];

	USHORT j = 1;
	for (vector<ODbVariantRef>::const_iterator i = (rResultRow.begin() + 1);	
		 i != rResultRow.end(); i++)
	{			
		if ((*i)->isModified())
		{
			const SdbColumn& rCol = *(*m_xResultColumns)[i - rResultRow.begin()];			
			DBG_ASSERT(IS_UPDATEABLE(rCol),"Column nicht updatebar!");
			if (IS_UPDATEABLE(rCol))
				*(*aParameterRow)[j++] = *(*i);							
		}
	}

	for (UINT16 n = 0; n < m_aKeyColumns.size(); n++)
	{
		if (m_aKeyColumns[n].bUnique && rKeyRow[n].hasValue())
			(*aParameterRow)[j++]->setAny(rKeyRow[n]);
	}

	if (bCurrency && m_rOrigCursor.nOptions & SDB_CONCURRENCY)
	{
		// if there is no row version
		if (m_xResultColumns->Count() == m_xResultColumns->GetVisibleCount())
		{
			for(UINT16 n=1;n<m_xResultColumns->Count();++n)
			{
				ODbVariantRef xVal = rResultRow[n];
				if(!(*m_xResultColumns)[n]->IsUnique() && IS_SEARCHABLE((*m_xResultColumns)[n]) && !xVal->isModified() && !xVal->isNull())
					*(*aParameterRow)[j++] = *xVal;
			}
		}
		else
		{   //here I have row version columns
			for (UINT16 n = 1; n < m_xResultColumns->Count(); n++)
			{
				SdbColumn* pCol = (SdbColumn*)(*m_xResultColumns)[n];
				if (pCol->GetAllFlags() & SDB_FLAGS_VERSIONING && !rResultRow[n]->isNull())
					*(*aParameterRow)[j++] = *rResultRow[n];
			}
		}
	}
}
//--------------------------------------------------------------------------
void OValueSet::appendParameterColumns(BOOL bCurrency)
{
	const ODbRow& rResultRow = *m_rOrigCursor.aResultRow;
	SdbColumnsRef aParamCols(m_xBackgroundCursor->aParameterColumns);
	OValueRow& rKeyRow = m_aRowSet[m_nCurrentPos];
	SdbConnectionRef xConnection(m_rOrigCursor.GetConnection());

	for (vector<ODbVariantRef>::const_iterator i = (rResultRow.begin() + 1);	
		 i != rResultRow.end(); i++)
	{			
		if ((*i)->isModified())
		{				
			SdbColumn* pCol = xConnection->CreateColumn((*(*m_xResultColumns)[i - rResultRow.begin()]));
			pCol->SetFlag(SDB_FLAGS_PARAM_IN);
			DBG_ASSERT(IS_UPDATEABLE((*pCol)),"Column nicht updatebar!");
			if (IS_UPDATEABLE((*pCol)))
				aParamCols->AddColumn(pCol);
		}
	}
	for (UINT16 n = 0; n < m_aKeyColumns.size(); n++)
	{
		if (m_aKeyColumns[n].bUnique && rKeyRow[n].hasValue())
		{
			SdbColumn* pCol = xConnection->CreateColumn(*(*m_xResultColumns)[m_aKeyColumns[n].nId]);
			pCol->SetFlag(SDB_FLAGS_PARAM_IN);
			aParamCols->AddColumn(pCol);
		}
	}
	
	if (bCurrency && m_rOrigCursor.nOptions & SDB_CONCURRENCY)
	{
		// if there is no row version
		if (m_xResultColumns->Count() == m_xResultColumns->GetVisibleCount())
		{
			for (UINT16 j=1;j<m_xResultColumns->Count();++j)
			{
				if(!(*m_xResultColumns)[j]->IsUnique() && IS_SEARCHABLE((*m_xResultColumns)[j]) && !rResultRow[j]->isModified() && !rResultRow[j]->isNull())
				{
					SdbColumn* pCol = xConnection->CreateColumn(*(*m_xResultColumns)[j]);
					pCol->SetFlag(SDB_FLAGS_PARAM_IN);
					aParamCols->AddColumn(pCol);				
				}
			}
		}
		else
		{   //here I have row version columns
			for (UINT16 n = 1; n < m_xResultColumns->Count(); n++)
			{
				SdbColumn* pCol = (SdbColumn*)(*m_xResultColumns)[n];
				if (pCol->GetAllFlags() & SDB_FLAGS_VERSIONING && !rResultRow[n]->isNull())
				{
					SdbColumn* pColumn = xConnection->CreateColumn(*pCol);
					pColumn->SetFlag(SDB_FLAGS_PARAM_IN);
					aParamCols->AddColumn(pColumn);
				}
			}
		}
	}
}
//--------------------------------------------------------------------------
BOOL OValueSet::doDeleteRow()
{
	DBG_CHKTHIS(OValueSet,NULL);
	DBG_ASSERT(m_nPrivileges > SDB_PR_READ && m_nCurrentPos >= 0 && m_nCurrentPos < (INT32)size(), "Falsche Postion im Set");
	if (m_nCurrentPos < 0 || m_nCurrentPos >= (INT32)size())
		return sal_False;

	if (!m_rOrigCursor.IsOpen())
		return sal_False;

	SdbConnectionRef xConnection(m_rOrigCursor.GetConnection());

	// aktuelle Zeile schon geloescht
	if (m_rOrigCursor.aResultRow->isDeleted())
	{
		m_aRowSet.erase(m_aRowSet.begin() + m_nCurrentPos);
		m_nCurrentPos--;
		return TRUE;
	}

	String aStr;
	aStr.AssignAscii("DELETE FROM ");
	(aStr += m_aRefetchTableName).AppendAscii(" WHERE ");

	BOOL bCurrency(sal_False);
	if (!appendWhere(aStr,m_nCurrentPos,bCurrency))
		return sal_False;

	if (m_xBackgroundCursor)
		m_xBackgroundCursor->Close();

	appendParameterColumns(bCurrency);
	BOOL bResult;

	if (bResult = m_xBackgroundCursor->Prepare(aStr))
		setParameterValues(bCurrency);
//	else
//		m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();

	bResult = bResult && m_xBackgroundCursor->Open();
	if (!bResult)
		m_rOrigCursor.aStatus = m_xBackgroundCursor->Status();
	else if(!m_xBackgroundCursor->RowCount())	// kein datensatz gelscht
	{
		// I don't find any records, so I can delete what is not there
		bResult = TRUE;
	}
	else
		m_rOrigCursor.aStatus.SetRecordsAffected(m_xBackgroundCursor->RowCount());

	if (bResult)
	{	// Satz aus dem Set loeschen
		m_aRowSet.erase(m_aRowSet.begin() + m_nCurrentPos);
		m_nCurrentPos--;
	}

	if (m_xBackgroundCursor)
		m_xBackgroundCursor->Close();
	return bResult;
}

DBG_NAME(OBookmarkSet);
//--------------------------------------------------------------------------
OBookmarkSet::OBookmarkSet(SdbCursor& rCursor)
			 :OKeySet(rCursor)			 
{
	DBG_CTOR(OBookmarkSet,NULL);	

	// Name der Tabelle feststellen
	if (m_rOrigCursor.pParseIterator)
	{
		const SdbSqlParseTreeIterator& rParseIterator = *m_rOrigCursor.pParseIterator;

		if (rParseIterator.StatementType() == SDB_STATEMENT_SELECT &&
			rParseIterator.GetSdbTableRangeMap().Count() == 1)
		{
			SdbTableRef xTable = *rParseIterator.GetSdbTableRangeMap().GetObject(0); // es ist hier nur eine Tabelle vorhanden
			if (xTable)
			{
				SdbConnection& rConnection = *m_rOrigCursor.GetConnection();
				m_aRefetchTableName = rConnection.QuoteTableName(xTable->GetFullName());
			}
		}
	}
}

//--------------------------------------------------------------------------
OBookmarkSet::~OBookmarkSet()
{
	DBG_DTOR(OBookmarkSet,NULL);
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::setUpdatable(BOOL bUpdate)
{
	BOOL bResult(m_rOrigCursor.SetUpdatableImpl(bUpdate));
	m_nPrivileges  = m_rOrigCursor.nPrivileges;
	return bResult;
}

//--------------------------------------------------------------------------
void OBookmarkSet::bind()
{
	DBG_CHKTHIS(OKeySet,NULL);
	if (!m_rOrigCursor.IsOpen() || m_bBound || m_bSnapshot)
	{
		m_bBound = TRUE;
		return;
	}

	// entfernt die Werte, nicht zum Refetch benoetigt werden
	ODbRow& rTarget = *m_rOrigCursor.aResultRow;		
	// free values not needed
	for (vector<ODbVariantRef>::iterator i = (m_xBindRow->begin() + 1),
		 j = (rTarget.begin() + 1);
		 i != m_xBindRow->end(); i++)
	{			
		if ((*i).is())
		{
			(*j) = (*i);
			(*i).unbind();
		}
		j++;
	}
	m_bBound = TRUE;
}

//--------------------------------------------------------------------------
void OBookmarkSet::unBind()
{
	DBG_CHKTHIS(OKeySet,NULL);
	if (!m_rOrigCursor.IsOpen() || m_bSnapshot || !m_bBound)
	{
		m_bBound = sal_False;
		return;
	}

	// entfernt die Werte, nicht zum Refetch benoetigt werden
	const SdbColumns& rCols  = *m_xResultColumns;
	ODbRow& rTarget = *m_rOrigCursor.aResultRow;

	for (vector<ODbVariantRef>::iterator i = (m_xBindRow->begin() + 1),
		 j = (rTarget.begin() + 1);
		 i != m_xBindRow->end(); i++)
	{			
		(*i) = (*j); 
		(*j).unbind();		
		j++;		
	}
	m_bBound = sal_False;
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::fetchNext(SdbCursor::SdbCursorMode& rMode)
{
	if (!m_rOrigCursor.IsOpen())
		return sal_False;

	if (m_aLastBookmark.hasValue() && !areEqual(m_aLastBookmark,m_aCurBookmark))	// Auf die letzte bookmark positionieren
		m_rOrigCursor.MoveImpl(m_aLastBookmark, rMode, sal_False);			

	BOOL bResult(m_rOrigCursor.MoveImpl(SDB_POS_NEXT, 1, rMode));
	if (bResult)
	{
		m_aCurBookmark = m_rOrigCursor.aResultRow->front()->getAny();
		m_aLastBookmark = m_aCurBookmark;
		sal_Int32 nVal;
		m_aLastBookmark >>= nVal;
		m_aRowSet.push_back(nVal);
		return TRUE;
	}
	return sal_False;
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::getBookmarkPos(INT32& nPos, const ::staruno::Any& rBookmark) const throw( ::starlang::IllegalArgumentException )
{
	if (rBookmark.getValueTypeClass() != ::com::sun::star::uno::TypeClass_LONG)
		throw ::starlang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);
	
	// determine the position of the bookmark
	INT32 nValue;
	rBookmark >>= nValue;
	vector<INT32>::const_iterator i = lower_bound(m_aRowSet.begin(), m_aRowSet.end(), nValue, OINT32Less());

	if (i != m_aRowSet.end() && !(nValue < *i))
		nPos = i - m_aRowSet.begin();
	
	return i != m_aRowSet.end();	
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::seekPositioned(BOOL bRetrieveData) throw( ::starlang::IllegalArgumentException )
{
	// besorgt die Spalte ueber die Bookmark
	// Ist der OriginalCursor noch nicht am letzten Datensatz angekommen
	// wird wie auf den letzen gelesenen Datensatz positioniert
	::staruno::Any aBookmark;
	aBookmark <<= m_aRowSet[m_nCurrentPos];

	SdbCursor::SdbCursorMode eMode = SdbCursor::SDB_MOD_NONE;
	BOOL bResult = sal_False;
	try
	{
		bResult = m_rOrigCursor.MoveImpl(aBookmark, eMode, bRetrieveData);
		if (bResult && eMode == SdbCursor::SDB_MOD_NONE)
			m_aCurBookmark = aBookmark;

		// DBG_ASSERT(bResult ? m_rOrigCursor.aResultRow->front()->equals(m_aCurBookmark) : TRUE, "Falsche Bookmark");
	}
	catch (::starlang::IllegalArgumentException e)
	{	
		throw;
	}	
	return bResult && eMode == SdbCursor::SDB_MOD_NONE;
}

//--------------------------------------------------------------------------
void OBookmarkSet::appendRow(INT32 nValue)
{	
	m_aRowSet.push_back(nValue);
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::doInsertRow()
{
	BOOL bFinal = m_rOrigCursor.bRowCountFinal;
	BOOL bResult(m_rOrigCursor.InsertImpl());
	
	// nur Datensaetze die bei EOF eingefuegt wurden uebernehmen
	if (bResult && bFinal)
	{
		sal_Int32 nVal;
		m_aLastBookmark = m_rOrigCursor.aResultRow->front()->getAny();
		m_aLastBookmark >>= nVal;
		m_aRowSet.push_back(nVal);

		// reread data (functions, defaults)		
		INT32 nTempPos = m_nCurrentPos;
		m_nCurrentPos = m_aRowSet.size() -1;

		seekPositioned(TRUE);
		m_nCurrentPos = nTempPos;

		m_rOrigCursor.aStatus.Clear();
		m_rOrigCursor.aStatus.SetRecordsAffected(1);		
	}
	return bResult;
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::doUpdateRow()
{
	if (!seekPositioned(sal_False))
		return sal_False;

	if (!m_rOrigCursor.UpdateImpl())
		return sal_False;

	m_rOrigCursor.aStatus.SetRecordsAffected(1);
	return TRUE;
}

//--------------------------------------------------------------------------
BOOL OBookmarkSet::doDeleteRow()
{
	if (m_rOrigCursor.aResultRow->isDeleted() || m_rOrigCursor.DeleteImpl())
	{
		// geloeschten Satz entfernen
		// aktuelle Position um eins Verringern
		m_aRowSet.erase(m_aRowSet.begin() + m_nCurrentPos);
		m_nCurrentPos--;

		// steht die letzte Bookmark auf der Current, dann ist diese auch anzupassen
		BOOL bSetLast( areEqual(m_aLastBookmark,m_aCurBookmark));
		if (m_nCurrentPos >= 0)
			m_aCurBookmark <<= m_aRowSet[m_nCurrentPos];
		else
			// steht vor dem ersten Satz
			m_aCurBookmark.clear();

		if (bSetLast)
			m_aLastBookmark = m_aCurBookmark;

		m_rOrigCursor.aStatus.SetRecordsAffected(1);
		return TRUE;
	}
	return sal_False;
}

//--------------------------------------------------------------------------
void OBookmarkSet::determineRowCount()
{
	if (m_rOrigCursor.bRowCountFinal)
		return;

	::staruno::Any aTempBookmark = m_aCurBookmark;
	OKeySet::determineRowCount();

	// reposition if we have a valid bookmark!
	if (aTempBookmark.hasValue() && !areEqual(aTempBookmark,m_aCurBookmark))
	{
		SdbCursor::SdbCursorMode eMode = SdbCursor::SDB_MOD_NONE;
		m_rOrigCursor.MoveImpl(aTempBookmark, eMode, sal_False);
	}
}



