/*************************************************************************
 *
 *  $RCSfile: sdbstat.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbstat.hxx"
#include "sdbresid.hrc"

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#ifndef _SVSTOR_HXX //autogen
#include <so3/svstor.hxx>
#endif

DBG_NAME(SdbStatus);

//------------------------------------------------------------------
SdbStatus::SdbStatus()
		  :eResult(SDB_STAT_SUCCESS)
		  ,nRecordsAffected(0)
{
	DBG_CTOR(SdbStatus,NULL);
	SetComponent(String::CreateFromAscii("Sdb Driver"));
}

//------------------------------------------------------------------
SdbStatus::SdbStatus(const SdbStatus &rSdbStatus)
{
	DBG_CTOR(SdbStatus,NULL);
	DBG_CHKOBJ(&rSdbStatus,SdbStatus,NULL);

	eResult = rSdbStatus.Result();
	aSQLState = rSdbStatus.SQLState();
	aErrorMessage = rSdbStatus.ErrorMessage();
	nNativeErrorCode = rSdbStatus.NativeErrorCode();
	nRecordsAffected = rSdbStatus.GetRecordsAffected();
	aErrorInfo = rSdbStatus.ErrorInfo();
}

//------------------------------------------------------------------
SdbStatus::~SdbStatus()
{
	DBG_DTOR(SdbStatus,NULL);
}

//------------------------------------------------------------------
const SdbStatus & SdbStatus::operator=(const SdbStatus &rSdbStatus)
{
	DBG_CHKTHIS(SdbStatus,NULL);
	DBG_CHKOBJ(&rSdbStatus,SdbStatus,NULL);

	// Absicherung gegen Objekt-Identitaet nicht erforderlich,
	// da StarView String-Selbstzuweisung erlaubt.

	eResult = rSdbStatus.Result();
	aSQLState = rSdbStatus.SQLState();
	aErrorMessage = rSdbStatus.ErrorMessage();
	nNativeErrorCode = rSdbStatus.NativeErrorCode();
	nRecordsAffected = rSdbStatus.GetRecordsAffected();
	aErrorInfo = rSdbStatus.ErrorInfo();

	return *this;
}

//------------------------------------------------------------------
BOOL SdbStatus::operator==(const SdbStatus &rSdbStatus) const
{
	DBG_CHKTHIS(SdbStatus,NULL);
	DBG_CHKOBJ(&rSdbStatus,SdbStatus,NULL);

	if (IsInformationAvailable())
	{
		// keine && wegen CFfront
		if (eResult == rSdbStatus.Result())
		 if(aSQLState == rSdbStatus.SQLState())
		  if(aErrorMessage == rSdbStatus.ErrorMessage())
		   if(nNativeErrorCode == rSdbStatus.NativeErrorCode())
			if(aErrorInfo == rSdbStatus.ErrorInfo())
			   return TRUE;
		return FALSE;
	}
	else
	{
		return eResult == rSdbStatus.Result();
	}
}


//------------------------------------------------------------------
void SdbStatus::Set(const SdbStatusCode eNewResult, const String &rNewSQLState,
					const String &rNewErrorMessage, INT32 nNewNativeErrorCode,
					const String &rNewErrorInfo)
{
	DBG_CHKTHIS(SdbStatus,NULL);

	DBG_ASSERT(eNewResult == SDB_STAT_SUCCESS || eNewResult == SDB_STAT_SUCCESS_WITH_INFO ||
			   eNewResult == SDB_STAT_NO_DATA_FOUND || eNewResult == SDB_STAT_ERROR || eNewResult == SDB_STAT_CANCELED,
			   "SdbStatus: ungueltiger Status");

	BOOL bStatusChange = eResult != eNewResult;

	eResult = eNewResult;
	aSQLState = rNewSQLState;
	aErrorMessage = rNewErrorMessage;
	nNativeErrorCode = nNewNativeErrorCode;
	aErrorInfo = rNewErrorInfo;
	nRecordsAffected = 0;

	if (bStatusChange)
		aStatusChangeHdl.Call(this);
}

//------------------------------------------------------------------
void SdbStatus::Set(SdbStatusCode eNewResult)
{
	DBG_CHKTHIS(SdbStatus,NULL);

	DBG_ASSERT(!(eNewResult == eResult && eNewResult == SDB_STAT_ERROR) ,"Fehler wurde doppelt gesetzt");
	if(eNewResult == eResult && eNewResult == SDB_STAT_ERROR)
		return;
	DBG_ASSERT(eNewResult == SDB_STAT_SUCCESS || eNewResult == SDB_STAT_SUCCESS_WITH_INFO ||
			   eNewResult == SDB_STAT_NO_DATA_FOUND || eNewResult == SDB_STAT_ERROR || eNewResult == SDB_STAT_CANCELED,
			   "SdbStatus: ungueltiger Status");

	BOOL bStatusChange = eResult != eNewResult;
	eResult = eNewResult;

	switch(eNewResult)
	{
	  case SDB_STAT_SUCCESS_WITH_INFO:
			aSQLState = String::CreateFromAscii("01000");
			aErrorMessage = String(SdbResId(STR_STAT_WARNING));
			nNativeErrorCode = 0;
			break;
	  case SDB_STAT_ERROR:
			aSQLState = String::CreateFromAscii("S1000");
			aErrorMessage = String(SdbResId(STR_STAT_ERROR));
			nNativeErrorCode = 0;
			break;
	  case SDB_STAT_NO_DATA_FOUND:
			aSQLState = String::CreateFromAscii("S1000");
			aErrorMessage = String(SdbResId(STR_STAT_NO_DATA_FOUND));
			nNativeErrorCode = 0;
	}

	nRecordsAffected = 0;

	if (bStatusChange)
		aStatusChangeHdl.Call(this);
}

//------------------------------------------------------------------
const String SdbStatus::CreateErrorMessage(const String & pErrorMessageText,
										   const String & pDataSource)
{
	DBG_CHKTHIS(SdbStatus,NULL);

	/*String s = String("[Star Division][");
	s += aComponent;
	s += "]";*/

	String s;
	if (pDataSource.Len()) 
	{
		s += String::CreateFromAscii("[");
		s += pDataSource;
		s += String::CreateFromAscii("] ");
	}
	s += pErrorMessageText;
	return s;
}

//------------------------------------------------------------------
BOOL SdbStatus::SetError(const SvFileStream& aFileStream, SdbFileType eType)
{
	return SetError(aFileStream.GetError(), eType, aFileStream.GetFileName());
}

//------------------------------------------------------------------
BOOL SdbStatus::SetError(const SvStorage& aStorage, SdbFileType eType)
{
	return SetError(aStorage.GetError(), eType, aStorage.GetName());
}

//------------------------------------------------------------------
void SdbStatus::SetError(ULONG nErrorCode, DBObject eObj, const String& aName)
{
	SdbFileType eType;

	switch (eObj)
	{
		case dbTable:
			eType = TABLE; break;
		case dbQuery:
			eType = QUERY; break;
		case dbForm:
			eType = FORM; break;
		case dbReport:
			eType = REPORT; break;
		case dbIndex:
			eType = INDEX; break;
		case dbView:
			eType = VIEW; break;
		case dbRelation:
			eType = RELATION; break;
		case dbField:
			eType = FIELD; break;
		case dbWorkspace:
			eType = WORKSPACE; break;
	};

	SetError(nErrorCode, eType, aName);
}

//------------------------------------------------------------------
BOOL SdbStatus::SetError(ULONG nErrorCode, SdbFileType eType, const String& rName)
{
	SdbStatus* THIS = (SdbStatus*)this;  // const_cast !!!

	switch (nErrorCode)
	{
		case ERRCODE_NONE:
			break;
		case ERRCODE_IO_LOCKVIOLATION:
			{
				String aText = String(SdbResId(STR_STAT_FILE_IN_USE));
				aText.SearchAndReplace(String::CreateFromAscii("%%d"),rName);
				aText.SearchAndReplace(String::CreateFromAscii("%%t"),TypeToString(eType));
				THIS->Set(SDB_STAT_ERROR,
						String::CreateFromAscii("40001"),
						THIS->CreateErrorMessage(aText),
						0, String());
			}break;
		case ERRCODE_IO_NOTEXISTS:
		case ERRCODE_IO_NOTEXISTSPATH:
			{
				String aText = String(SdbResId(STR_STAT_FILE_NOT_FOUND));
				aText.SearchAndReplace(String::CreateFromAscii("%%d"),rName);
				aText.SearchAndReplace(String::CreateFromAscii("%%t"),TypeToString(eType));
				THIS->Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S0002"),
						THIS->CreateErrorMessage(aText),
						0, String());
			}break;
		case ERRCODE_IO_ALREADYEXISTS:
			{
				String aText = String(SdbResId(STR_STAT_FILE_ALREADYEXISTS));
				aText.SearchAndReplace(String::CreateFromAscii("%%d"),rName);
				aText.SearchAndReplace(String::CreateFromAscii("%%t"),TypeToString(eType));

				THIS->Set(SDB_STAT_ERROR,
						String::CreateFromAscii("42001"),
						THIS->CreateErrorMessage(aText),
						0, String());
			}
			break;
		case ERRCODE_IO_INVALIDACCESS:
		case ERRCODE_IO_ACCESSDENIED:
			{
				String aText = String(SdbResId(STR_STAT_FILE_INVALID_ACCESS));
				aText.SearchAndReplace(String::CreateFromAscii("%%d"),rName);
				aText.SearchAndReplace(String::CreateFromAscii("%%t"),TypeToString(eType));

				THIS->Set(SDB_STAT_ERROR,
						String::CreateFromAscii("42001"),
						THIS->CreateErrorMessage(aText),
						0, String());
			}
			break;
		case ERRCODE_IO_CANTREAD:
		case ERRCODE_IO_CANTTELL:
		case ERRCODE_IO_CANTSEEK:
			THIS->Set(SDB_STAT_NO_DATA_FOUND);
			break;
		case ERRCODE_IO_CANTCREATE:
		case ERRCODE_IO_CANTWRITE:
		case ERRCODE_IO_OUTOFSPACE:
			THIS->Set(SDB_STAT_ERROR,
						String::CreateFromAscii("42000"),
						THIS->CreateErrorMessage(String(SdbResId(STR_STAT_ERROR_WRITING))),
						0, String());
			break;
		default:
			THIS->Set(SDB_STAT_ERROR);
			break;
	}

	return IsError();
}

//------------------------------------------------------------------
String SdbStatus::TypeToString(SdbFileType eType) const
{
	switch (eType)
	{
		case DATABASE:
			return String(SdbResId(STR_STAT_DATABASE));
		case TABLE:
			return String(SdbResId(STR_STAT_TABLE));
		case QUERY:
			return String(SdbResId(STR_STAT_QUERY));
		case FORM:
			return String(SdbResId(STR_STAT_FORM));
		case REPORT:
			return String(SdbResId(STR_STAT_REPORT));
		case INDEX:
			return String(SdbResId(STR_STAT_INDEX));
		case MEMO:
			return String(SdbResId(STR_STAT_MEMO));
		case VIEW:
			return String(SdbResId(STR_STAT_VIEW));
		case RELATION:
			return String(SdbResId(STR_STAT_RELATION));
		case FIELD:
			return String(SdbResId(STR_STAT_FIELD));
		case WORKSPACE:
			return String(SdbResId(STR_STAT_WORKSPACE));
		default:
			DBG_ERROR("Unknown type");
			return String();
	}
}

//------------------------------------------------------------------
void SdbStatus::SetSequenceError()
{
	DBG_ASSERT(0,"Fehler in der Funkionsaufruf- Reihenfolge!");
	Set(SDB_STAT_ERROR, String::CreateFromAscii("S1010"), CreateErrorMessage(String(SdbResId(STR_STAT_FUNCTION_SEQUENCE_ERROR))), 0, String() );
}

//------------------------------------------------------------------
void SdbStatus::SetDriverNotCapableError()
{
	Set(SDB_STAT_ERROR, String::CreateFromAscii("S1C00"), CreateErrorMessage(String(SdbResId(STR_STAT_NOT_CAPABLE))), 0, String() );
}

//------------------------------------------------------------------
void SdbStatus::SetInvalidStatement()
{
	Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_STATEMENT))), 0, String() );
}

//------------------------------------------------------------------
void SdbStatus::SetStatementTooComplex()
{
	Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), CreateErrorMessage(String(SdbResId(STR_STAT_STATEMENT_TOO_COMPLEX))), 0, String() );
}

//------------------------------------------------------------------
void SdbStatus::SetConnectionLost()
{
	Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), CreateErrorMessage(String(SdbResId(STR_STAT_CONN_LOST))), 0, String() );
}


#ifndef _SDB_SDBENV_HXX
#include "sdbenv.hxx"
#endif

#ifndef _TOOLS_SIMPLERESMGR_HXX_
#include <tools/simplerm.hxx>
#endif

//------------------------------------------------------------------
SdbResId::SdbResId(USHORT nId)
	:m_sValue(SdbEnvironmentRef()->GetResMgr()->ReadString(nId))
{
}


