/*************************************************************************
 *
 *  $RCSfile: sdbstor.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#define ENABLE_BYTESTRING_STREAM_OPERATORS
#include "sdbstor.hxx"

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _SOT_STORINFO_HXX
#include <sot/storinfo.hxx>
#endif

#include <vector>
#include <stl_algo.h>

#ifndef _SDB_TOOLS_HXX
#include "tools.hxx"
#endif


SvGlobalName aClassName(0x91B54A60, 0xC097, 0x11D0,
						0xAA, 0x75, 0x00, 0xA0, 0x24, 0x8A, 0xA7, 0xBC);

#define	USERTYPENAME		USERTYPENAME_5
#define	USERTYPENAME_1		"Scotty"
#define	USERTYPENAME_2		"Lt. Ohura"
#define	USERTYPENAME_3		"Pille"
#define	USERTYPENAME_4		"Spock"
#define	USERTYPENAME_5		"Lt. Cmd. Data"

#define DIR					"Dir"
#define NAME_OFFSET			100
DBG_NAME(SdbStorInfo);
//------------------------------------------------------------------------------
SdbStorInfo::SdbStorInfo(SvStorage* pStor, StreamMode eMode)
			:pImpl(pStor)
			,bReadOnly((eMode & STREAM_WRITE) == 0)
{
	DBG_CTOR(SdbStorInfo,NULL);
	pImpl->AddRef();
}

//------------------------------------------------------------------------------
SdbStorInfo::~SdbStorInfo()
{
	DBG_DTOR(SdbStorInfo,NULL);
	pImpl->ReleaseRef();
}

//------------------------------------------------------------------------------
SvStream& operator << (SvStream &rStream, SdbStorageDir& rDir)
{
	rStream.Seek(0);
	rStream.SetStreamSize(0);
	rStream.SetBufferSize(1024);
	rStream.SetStreamCharSet(RTL_TEXTENCODING_MS_1252);
	//	rStream.SetTargetCharSet(gsl_getSystemTextEncoding());
	rStream.SetNumberFormatInt(NUMBERFORMAT_INT_LITTLEENDIAN);
	rStream.Seek(0);
	for (ByteString* pString = rDir.First(); pString; pString = rDir.Next())
	{
		rStream << rDir.GetCurKey() << *pString;
#if _DEBUG || DBG_UTIL
		if (!pString->Len())
			DBG_WARNING("operator<<(SvStream&, SdbStorageDir&) : writing an empty String !");
			// leere Verzeichnisnamen sollten nicht vorkommen
#endif // _DEBUG || DBG_UTIL
	}
	rStream.Flush();
	return rStream;
}

//------------------------------------------------------------------------------
SvStream& operator >> (SvStream &rStream, SdbStorageDir& rDir)
{
	ULONG	nKey;
	ByteString	aName;

	rStream.Seek(STREAM_SEEK_TO_END);	// steht etwas im Stream?

	if (rStream.Tell() < 4)
		return rStream;

	rStream.Seek(0);
	rStream.SetNumberFormatInt(NUMBERFORMAT_INT_LITTLEENDIAN);
	rStream.SetStreamCharSet(RTL_TEXTENCODING_MS_1252);
	//	rStream.SetTargetCharSet(gsl_getSystemTextEncoding());
	rStream >> nKey >> aName;
	while (!rStream.IsEof() && rStream.GetError() == SVSTREAM_OK)
	{
		if (aName.Len())
			rDir.Insert(nKey, String(aName,gsl_getSystemTextEncoding()));
		else
			DBG_WARNING("operator>>(SvStream&, SdbStorageDir&) : reading an empty String !");
			// leere Verzeichnisnamen sollten nicht vorkommen
			// (das ist nur eine Warnung : es gab mal einen Bug, der im Storage wirklich leere Namen geschrieben hat, der ist
			// wech, aber die leeren Namen sind noch da ....)
		rStream >> nKey >> aName;
	}
	return rStream;
}
DBG_NAME(SdbStorageDir);
//------------------------------------------------------------------------------
SdbStorageDir::SdbStorageDir(SdbStorage& rStor)
			  :bDirty(FALSE)
			  ,rStorage(rStor)
			  ,pStream(NULL)
{
	DBG_CTOR(SdbStorageDir,NULL);
}

//------------------------------------------------------------------------------
SdbStorageDir::~SdbStorageDir()
{
	// geaendertes Verzeichnis schreiben
	Clear();
	if (pStream)
		pStream->ReleaseRef();
	DBG_DTOR(SdbStorageDir,NULL);
}

//------------------------------------------------------------------------------
void SdbStorageDir::Flush()
{
	if (bDirty && pStream)
	{
		(*pStream) << *this;
		if (!rStorage.Status().SetError(pStream->GetErrorCode(),DATABASE,String::CreateFromAscii(DIR)))
			bDirty = FALSE;
	}
}

//------------------------------------------------------------------------------
BOOL SdbStorageDir::Insert(ULONG nKey, const String& rValue)
{
	DBG_CHKTHIS(SdbStorageDir,NULL);
	DBG_ASSERT(rValue.Len(), "SdbStorageDir::Insert : invalid (empty) name !");
	DBG_ASSERT(!IsKeyValid(nKey), "FalscherKey");

	BOOL bResult = SdbStringIndex::Insert(nKey, new ByteString(rValue,gsl_getSystemTextEncoding()));
	bDirty = (bResult) ? TRUE : bDirty;

	return bResult;
}

//------------------------------------------------------------------------------
ULONG SdbStorageDir::GetKey(const String& rName,BOOL _bTable) const
{
	DBG_CHKTHIS(SdbStorageDir,NULL);

	// wenn ich schon einen Key fuer den String habe, returne ich lieber den, das ist 'korrekter'
	// (Szenario : man legt einen Eintrag <name> an, laesst sich dafuer einen Key ermitteln,
	// das wird aber nicht der Hashwert n, sondern n + 1 (weil n schon von irgend-
	// einem anderen Eintrag benutzt wird). Dann loescht man den Eintrag, der fuer n
	// verantwortlich ist, und fragt erneut nach dem Key von <name>. -> Jetzt wird n
	// geliefert, obwohl der zugehoerige Storage eigentlich n+1 ist.
	// Diese Zeilen hier verhindern das.)
	SdbStorageDir* pThis = (SdbStorageDir*)this;
	String* pLoop = (String*)pThis->First();
	while (pLoop)
	{
		if (rName == *pLoop)
			return GetCurKey();
		pLoop = (String*)pThis->Next();
	}

	BOOL bCaseSensitiv = (_bTable && rStorage.rDatabase.GetConnection()) ? (rStorage.rDatabase.GetConnection()->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT: TRUE;
	ULONG nKey = SdbHashString::Hash(ByteString(rName,gsl_getSystemTextEncoding()));
	ByteString* pString;
	while (pString = Get(nKey))
	{
		if (bCaseSensitiv ? rName.EqualsAscii(pString->GetBuffer()) : rName.EqualsIgnoreCaseAscii(pString->GetBuffer()))
			break;
		else
			nKey++;
	}

	return nKey;
}

//------------------------------------------------------------------------------
void SdbStorageDir::Remove(ULONG nKey)
{
	DBG_CHKTHIS(SdbStorageDir,NULL);
	delete SdbStringIndex::Remove(nKey);
	bDirty = TRUE;
}

//------------------------------------------------------------------------------
void SdbStorageDir::Clear()
{
	DBG_CHKTHIS(SdbStorageDir,NULL);
	if (Count())
	{
		for (INT32 i=0; i < Count(); i++)
			delete GetObject(i);

		SdbStringIndex::Clear();
		bDirty = TRUE;
	}
}

//------------------------------------------------------------------------------
void SdbStorageDir::Init(SvStorageStream* pStr)
{
	DBG_CHKTHIS(SdbStorageDir,NULL);
	pStream = pStr;
	pStream->SetBufferSize(1024);
	pStream->AddRef();
	(*pStream) >> (*this);

	rStorage.Status().SetError(pStream->GetErrorCode(),DATABASE,String::CreateFromAscii(DIR));
	if (rStorage.Status().IsSuccessful())
	{
		// Manchmal kommt es vor, dass das Directory noch Eintraege enthaelt, die gar nicht mehr
		// (als Storages) existieren. (Keine Ahnung, unter welchen Umstaenden, eigentlich
		// sollte das nicht passieren, vielleicht ein Absturz waehrend des DB-Zugriffs ?)
		// Die muss ich in dem Fall noch ausfiltern.
		String* pLoop = (String*)First();
		while (pLoop)
		{
			if (!rStorage.Impl()->IsStorage(String::CreateFromInt32(GetCurKey())))
				Remove(GetCurKey());
			pLoop = (String*)Next();
		}
	}

	bDirty = FALSE;
}

DBG_NAME(SdbStorage);
//------------------------------------------------------------------------------
SdbStorage::SdbStorage(SdbDatabaseImpl& rDB,
					   const String & rName,
					   StreamMode eStreamMode,
					   StorageMode eStorMode)
		   :rDatabase(rDB)
#pragma warning (disable : 4355)
		   ,aDir(*this)
#pragma warning (default : 4355)
		   ,pImpl(NULL)
{
	DBG_CTOR(SdbStorage,NULL);
	SdbDatabaseGuard aSafeAccess(&rDatabase);

	BOOL bExists = DirEntry(rName).Exists();
	pImpl = new SdbStorInfo(new SvStorage(rName,eStreamMode,eStorMode), eStreamMode);

	// Setzen des Status an der Datenbank
	if (rDatabase.aStatus.SetError(Impl()->GetErrorCode(),DATABASE,rName))
		return;

	if (!bExists)
	{
		String aUserName;
		aUserName.AssignAscii(USERTYPENAME);
		Impl()->SetClass(aClassName, 0, aUserName);
		if (aUserName.EqualsAscii(USERTYPENAME_1))
			Impl()->SetVersion(1);
		else if (aUserName.EqualsAscii(USERTYPENAME_2))
			Impl()->SetVersion(2);
		else if (aUserName.EqualsAscii(USERTYPENAME_3))
			Impl()->SetVersion(3);
		else if (aUserName.EqualsAscii(USERTYPENAME_5))
			Impl()->SetVersion(5);
		else
			Impl()->SetVersion(4);
	}
	else
	{
		if (Impl()->GetClassName() == aClassName)
		{
			if (Impl()->GetUserName().EqualsAscii(USERTYPENAME_1))
				Impl()->SetVersion(1);
			else if (Impl()->GetUserName().EqualsAscii(USERTYPENAME_2))
				Impl()->SetVersion(2);
			else if (Impl()->GetUserName().EqualsAscii(USERTYPENAME_3))
				Impl()->SetVersion(3);
			else if (Impl()->GetUserName().EqualsAscii(USERTYPENAME_5))
				Impl()->SetVersion(5);
			else
				Impl()->SetVersion(4);
		}
		// fuer ltere Versionen, die noch keinen Classname verwenden
		else if (Impl()->IsStream(String::CreateFromAscii(TITLE_0)))
			Impl()->SetVersion(0);
		else
			rDatabase.aStatus.Set(SDB_STAT_ERROR);
	}

	if (GetVersion())
	{
		eStreamMode &= ~STREAM_TRUNC;
		SvStorageStream* pStream = Impl()->OpenStream(String::CreateFromAscii(DIR), eStreamMode);
		if (Status().SetError(pStream->GetErrorCode(),DATABASE,String::CreateFromAscii(DIR)))
		{
			pStream->AddRef();
			pStream->ReleaseRef();
		}
		else
			aDir.Init(pStream);
	}
}

//------------------------------------------------------------------------------
SdbStorage::SdbStorage(SdbStorage& rParent,
					   SdbFileType eContext,
					   const String& rName,
					   StreamMode eStreamMode,
					   StorageMode eStorMode)
		   :rDatabase(rParent.rDatabase)
#pragma warning (disable : 4355)
		   ,aDir(*this)
#pragma warning (default : 4355)
		   ,pImpl(NULL)
{
	DBG_CTOR(SdbStorage,NULL);
	SdbDatabaseGuard aSafeAccess(&rDatabase);

	String aRealName;
	ULONG nKey;
	if (rParent.GetVersion())
	{
		nKey = rParent.aDir.GetKey(rName,eContext == TABLE);
		aRealName = String::CreateFromInt32(nKey);
	}
	else
		aRealName = rName;

	BOOL bExists = rParent.IsStorage(aRealName);
	if (bExists && (eStorMode & STORAGE_FAILIFTHERE))
	{
		Status().SetError(ERRCODE_IO_ALREADYEXISTS ,eContext, rName);
		return;
	}

	pImpl = new SdbStorInfo(rParent.Impl()->OpenStorage(aRealName,eStreamMode,eStorMode), eStreamMode);

	// Setzen des Status an der Datenbank
	if (rDatabase.aStatus.SetError(Impl()->GetErrorCode(),eContext, rName))
		return;

	Impl()->SetVersion(rParent.GetVersion());
	if (!GetVersion())
		return;

	// beim Vater registrieren
	if (!rParent.aDir.IsKeyValid(nKey) && GetVersion())
		rParent.aDir.Insert(nKey, rName);

	// eigenes Verzeichnis erzeugen
	eStreamMode &= ~STREAM_TRUNC;
	SvStorageStream* pStream = Impl()->OpenStream(String::CreateFromAscii(DIR), eStreamMode);
	if (Status().SetError(pStream->GetErrorCode(),eContext,String::CreateFromAscii(DIR)))
	{
		pStream->AddRef();
		pStream->ReleaseRef();
		return;
	}
	aDir.Init(pStream);

}

//------------------------------------------------------------------------------
SdbStorage::~SdbStorage()
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	if (pImpl)
	{
		if (GetVersion())
			aDir.Flush();

		delete pImpl;
	}
	DBG_DTOR(SdbStorage,NULL);
}

//------------------------------------------------------------------------------
BOOL SdbStorage::GetUniqueObjectName(String& strName,BOOL _bTable)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_ASSERT(strName.Len(), "SdbStorage::GetUniqueObjectName : the base must be have length > 0 !");

	String strThisRound, strKey;
	BOOL bTestedAll = FALSE;
	for (UINT32 i=0; !bTestedAll; bTestedAll = (0xFFFFFFFF == i), ++i)
	{
		strThisRound = strName;
		strThisRound += String::CreateFromInt32(i);

		if (Impl()->IsContained(strThisRound))
			continue;

		// eine Datei mit diesem Namen gibt es schon mal definitiv nicht (Folder sowieso nicht, die fangen nach meiner Umsetzung
		// nie mit _ an, so wie gerade strThisRound
		if (GetVersion() == 0)
		{	// bei Version 0 erfolgt keine Namens-Umsetzung fuer Storages, also habe ich eine freie Zahl gefunden
			strName = strThisRound;
			return TRUE;
		}

		strKey = String::CreateFromInt32(aDir.GetKey(strThisRound,_bTable));
		if (Impl()->IsStorage(strKey))
			continue;
			// es gibt schon einen Storage mit dem 'uebersetzten' Namen -> weiter suchen

		strName = strThisRound;
		return TRUE;
	}

	return FALSE;
}

//------------------------------------------------------------------------------
BOOL SdbStorage::HasStorage(const String& strName,BOOL _bTable) const
{
	String strRealName = GetVersion() ? String::CreateFromInt32(aDir.GetKey(strName,_bTable)) : strName;
	return Impl()->IsStorage(strRealName);
}

//------------------------------------------------------------------------------
BOOL SdbStorage::MoveStorage(const String& strSourceName, SdbStorage& aDest, const String& strDestName,BOOL _bTable)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	// die 'richtigen' Namen
	ULONG nSrcKey = GetVersion() ? aDir.GetKey(strSourceName,_bTable) : 0;
	String strRealSrcName = GetVersion() ? String::CreateFromInt32(nSrcKey) : strSourceName;

	ULONG nDestKey = aDest.GetVersion() ? aDest.aDir.GetKey(strDestName,_bTable) : 0;
	String strRealDestName = aDest.GetVersion() ? String::CreateFromInt32(nDestKey) : strDestName;

	DBG_ASSERT(Impl()->IsStorage(strRealSrcName), "SdbStorage::MoveStorage : destination doesn't have an object with the given name !");

	if (aDest.GetVersion() && aDest.aDir.IsKeyValid(nDestKey))
	{
		DBG_ERROR("SdbStorage::MoveStorage : destination storage already has an object with the same (shortened) name !");
		return FALSE;
	}

	// das Objekt verschieben
	if (!Impl()->MoveTo(strRealSrcName, aDest.Impl(), strRealDestName))
		return FALSE;

	// im Ziel-Ordner registrieren
	if (GetVersion())
		aDir.Remove(nSrcKey);
	if (aDest.GetVersion())
		aDest.aDir.Insert(nDestKey, strDestName);

	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SdbStorage::RemoveStorage(const String& strName,BOOL _bTable)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	ULONG nKey = GetVersion() ? aDir.GetKey(strName,_bTable) : 0;
	String strRealName = GetVersion() ? String::CreateFromInt32(nKey) : strName;

	if (!Impl()->Remove(strRealName))
		return FALSE;

	if (GetVersion())
		aDir.Remove(nKey);

	return TRUE;
}

//------------------------------------------------------------------------------
BOOL SdbStorage::HasStreams() const
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	SvStorageInfoList aInfoList;
	Impl()->FillInfoList(&aInfoList);
	for (ULONG i = 0; i < aInfoList.Count(); i++)
	{
		SvStorageInfo& rInfo = aInfoList[i];
		if (!rInfo.IsStream())
			continue;
		String aObjName(rInfo.GetName());
		if (aObjName == String::CreateFromAscii(DIR))
			continue;
		return TRUE;
	}
	return FALSE;
}

//------------------------------------------------------------------------------
BOOL SdbStorage::HasStorages() const
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	if (GetVersion())
	{
		return aDir.Count() > 0;
	}
	else
	{
		SvStorageInfoList aInfoList;
		Impl()->FillInfoList(&aInfoList);
		return aInfoList.Count() > 0;
	}
}

//------------------------------------------------------------------------------
void SdbStorage::SetVersion(long lNewVersion)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	char* aVersionIdents[] = { USERTYPENAME_1, USERTYPENAME_2, USERTYPENAME_3, USERTYPENAME_4, USERTYPENAME_5 };
	if ((lNewVersion < 1) || (lNewVersion > 5))
	{
		DBG_ERROR("SdbStorage::SetVersion : ungueltiger Parameter");
		lNewVersion = 5;		// das ist bisher das 'sicherste'
	}

	Impl()->SetClass(aClassName, 0, String::CreateFromAscii(aVersionIdents[lNewVersion - 1]));
	Impl()->SetVersion(lNewVersion);
}

//------------------------------------------------------------------------------
SdbStatus& SdbStorage::Status() const
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	return rDatabase.aStatus;
}

//------------------------------------------------------------------------------
BOOL SdbStorage::Commit()
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	if (GetVersion())
		aDir.Flush();
	return Status().IsSuccessful() && Impl()->Commit();
}

//------------------------------------------------------------------------------
void SdbStorage::GetStorageNameList(SdbHashStringList& rList,BOOL _bCaseSensitiv)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	ULONG i;
	for (i=0; i < rList.Count(); i++)
		delete rList.GetObject(i);
	rList.Clear();

	if (GetVersion())
	{
		vector<String> aSortList;
		String aName;

		for (BOOL bResult = GetStorageName(aName, TRUE);
			bResult;
			bResult = GetStorageName(aName, FALSE))
		{
			DBG_ASSERT(aName.Len(), "SdbStorage::GetStorageNameList : empty storage names shouldn't occure ! (aDir may be corrupt)");
			if (aName.Len())
				aSortList.push_back(aName);//, nIndex);
		}
		
		::std::sort(aSortList.begin(),aSortList.end(),::utl::UStringMixLess(_bCaseSensitiv ? true : false));
		for (vector<String>::const_iterator j=aSortList.begin(); j != aSortList.end(); ++j)
		{
			SdbHashString* pStr = new SdbHashString(ByteString(*j,gsl_getSystemTextEncoding()));
			if(!rList.Find(*pStr))
				rList.Insert(pStr, LIST_APPEND);
			else
				delete pStr;
		}
	}
	else
	{
		SvStorageInfoList aInfoList;
		Impl()->FillInfoList(&aInfoList);
		for (i = 0; i < aInfoList.Count(); i++)
		{
			SvStorageInfo& rInfo = aInfoList[i];
			String aObjName(rInfo.GetName());
			rList.Insert(new SdbHashString(ByteString(aObjName,gsl_getSystemTextEncoding())), LIST_APPEND);
		}
	}
}

//------------------------------------------------------------------------------
void SdbStorage::GetStorageNameList(StringList& rList)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	ULONG i;
	for (i=0; i < rList.Count(); i++)
		delete rList.GetObject(i);
	rList.Clear();

	if (GetVersion())
	{
		String aName;

		for (BOOL bResult = GetStorageName(aName, TRUE);
			bResult;
			bResult = GetStorageName(aName, FALSE))
		{
			DBG_ASSERT(aName.Len(), "SdbStorage::GetStorageNameList : empty storage names shouldn't occure ! (aDir may be corrupt)");
			if (aName.Len())
				rList.Insert(new String(aName), LIST_APPEND);
		}
	}
	else
	{
		SvStorageInfoList aInfoList;
		Impl()->FillInfoList(&aInfoList);
		for (i = 0; i < aInfoList.Count(); i++)
		{
			SvStorageInfo& rInfo = aInfoList[i];
			if (rInfo.IsStream())
				continue;
			String aObjName(rInfo.GetName());
			rList.Insert(new String(aObjName), LIST_APPEND);
		}
	}
}

//------------------------------------------------------------------------------
void SdbStorage::GetStreamNameList(SdbHashStringList& rList)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	ULONG i;
	for (i=0; i < rList.Count(); i++)
		delete rList.GetObject(i);
	rList.Clear();

	SvStorageInfoList aInfoList;
	Impl()->FillInfoList(&aInfoList);
	for (i = 0; i < aInfoList.Count(); i++)
	{
		SvStorageInfo& rInfo = aInfoList[i];
		if (!rInfo.IsStream())
			continue;
		String aObjName(rInfo.GetName());
		if (aObjName == String::CreateFromAscii(DIR))
			continue;
		if (!aObjName.Len())
		{
			DBG_WARNING("SdbStorage::GetStreamNameList : there is a stream with an empty name ! how this ?");
			continue;
		}
		rList.Insert(new SdbHashString(ByteString(aObjName,gsl_getSystemTextEncoding())), LIST_APPEND);
	}
}

//------------------------------------------------------------------------------
BOOL SdbStorage::GetStorageName(String& rName, BOOL bFirst)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	ByteString* pString = (bFirst) ? aDir.First() : aDir.Next();
	if (pString)
		rName.AssignAscii(pString->GetBuffer());
	return pString != NULL;
}

//------------------------------------------------------------------------------
SvStorageStream* SdbStorage::OpenStream(SdbFileType eContext,
										const String& rName,
										StreamMode eMode)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	SvStorageStream* pStream = Impl()->OpenStream(rName, eMode);
	if (Status().SetError(pStream->GetErrorCode(),eContext,rName))
	{
		pStream->AddRef();
		pStream->ReleaseRef();
		pStream = NULL;
	}
	else
		pStream->SetBufferSize(1024);

	return pStream;
}

//------------------------------------------------------------------------------
SdbStorage*	SdbStorage::OpenStorage(SdbFileType eContext,
									const String& rName,
									StreamMode eStreamMode,
									StorageMode eStorageMode)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	SdbStorage* pStorage = new SdbStorage(*this, eContext, rName, eStreamMode, eStorageMode);
	if (Status().IsError())
	{
		pStorage->AddRef();
		pStorage->ReleaseRef();
		pStorage = NULL;
	}
	return pStorage;
}

//------------------------------------------------------------------------------
BOOL SdbStorage::Rename(SdbFileType eContext,
						const String & rName,
						const String & rNewName, BOOL bStor)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);

	if (bStor && GetVersion())
	{
		ULONG nKey = aDir.GetKey(rName,eContext==TABLE),
		nNewKey = aDir.GetKey(rNewName,eContext==TABLE);

		if (Impl()->Rename(String::CreateFromInt32(nKey),String::CreateFromInt32(nNewKey)))
		{
			aDir.Remove(nKey);
			aDir.Insert(nNewKey, rNewName);
		}
		else
			Status().SetError(Impl()->GetErrorCode(),eContext,rName);
	}
	else if (!Impl()->Rename(rName,rNewName))
		Status().SetError(Impl()->GetErrorCode(),eContext,rName);

	return Status().IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SdbStorage::Remove(SdbFileType eContext,
						const String & rName, BOOL bStor)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	if (bStor && GetVersion())
	{
		ULONG nKey = aDir.GetKey(rName,eContext==TABLE);
		if (Impl()->Remove(String::CreateFromInt32(nKey)) || !Impl()->IsStorage(String::CreateFromInt32(nKey)))
			aDir.Remove(nKey);
		else
			Status().SetError(Impl()->GetErrorCode(),eContext,rName);
	}
	else if (!Impl()->Remove(rName))
		Status().SetError(Impl()->GetErrorCode(),eContext,rName);

	return Status().IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SdbStorage::Remove(const String & rName,BOOL _bTable)
{
	SdbDatabaseGuard aSafeAccess(&rDatabase);
	DBG_CHKTHIS(SdbStorage,NULL);
	if (GetVersion())
	{
		ULONG nKey = aDir.GetKey(rName,_bTable);
		if (Impl()->Remove(String::CreateFromInt32(nKey)) || !Impl()->IsStorage(String::CreateFromInt32(nKey)))
			aDir.Remove(nKey);
		else
			Status().SetError(Impl()->GetErrorCode(),DATABASE,rName);
	}
	else if (!Impl()->Remove(rName))
		Status().SetError(Impl()->GetErrorCode(),DATABASE,rName);

	return Status().IsSuccessful();
}



