/*************************************************************************
 *
 *  $RCSfile: sdbtab.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbresid.hrc"

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#include "sdbcont.hxx"
#include "sdbtab.hxx"

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif

#ifndef _SDB_SDBIDX_HXX
#include "sdbidx.hxx"
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASECAPABILITY_HPP_
#include <com/sun/star/data/DatabaseCapability.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEIDENTIFIERCASE_HPP_
#include <com/sun/star/data/DatabaseIdentifierCase.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASELIMIT_HPP_
#include <com/sun/star/data/DatabaseLimit.hpp>
#endif

#include "sdbmap.hxx"

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

DBG_NAME(SdbTable);

TYPEINIT1(SdbTable, SdbObj);

//------------------------------------------------------------------
SdbTable::SdbTable(SdbConnection* _pConnection)
		 :SdbObj(_pConnection,TRUE)
		 ,pDatabase(NULL)
		 ,nPrivileges(0)
		 ,bView(FALSE)
{
	DBG_CTOR(SdbTable,NULL);
	aIndexList.SetCaseSensitive((_pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT);
}

//------------------------------------------------------------------
SdbTable::~SdbTable()
{
	DBG_DTOR(SdbTable,NULL);
}

//------------------------------------------------------------------
BOOL SdbTable::Create(const String& rName)
{
	// Open nicht moeglich wenn Tabellenbeschreibung schon geoeffnet
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	SDB_CHK_CONNECT_BOOL();

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	// den Tabellen-Namen ohne Katalog und Schema
	String strPureName = rName;
	if ((nCatalogUsage & ::stardata::DatabaseQualifier::IN_TABLE) || (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML))
	{
		sal_Bool bQualifierAtStart;
		rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_AT_START) >>= bQualifierAtStart;
		rtl::OUString aSep;
		rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_SEPARATOR) >>= aSep;
		sal_Unicode cChar = aSep[0];
		if (strPureName.GetTokenCount(cChar)>1)
		{
			xub_StrLen nIndex = 0;
			strPureName.GetToken(0, cChar, nIndex);
			if (bQualifierAtStart)
				strPureName = strPureName.Copy(nIndex);
			else
				strPureName.Erase(nIndex--);
		}
	}

	if ((nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE) || (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML))
	{
		if (strPureName.GetTokenCount('.'))
			strPureName = strPureName.GetToken(1,'.');
	}


	// Test der Laenge des Namens
	sal_uInt32 nMaxTableNameLen(rMetaData->getLimit(::stardata::DatabaseLimit::TABLE_NAME_LEN));
	if (nMaxTableNameLen && (sal_uInt32)strPureName.Len() > nMaxTableNameLen)
	{
		String aText(SdbResId(STR_STAT_INVALID_TABLENAMELENGTH));
		aText.SearchAndReplace(String::CreateFromAscii("%"),String::CreateFromInt32((long)nMaxTableNameLen));

		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
		return FALSE;
	}

	aStatus.Clear();
	SetNames(rName,String(), pConnection);

	// Owner wird von der Connection aus gesetzt
	if ((nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE) || (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML))
		aOwnerName = pConnection->GetUser();

	// aktuelle Datenbank
	if (!aQualifierName.Len() && ((nCatalogUsage & ::stardata::DatabaseQualifier::IN_TABLE) || (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML)))
		aQualifierName = pConnection->GetQualifier();
	switch (GetConnection()->GetIdentifierCase())
	{
		// mindestens aName wird schon von SetNames erschlagen...
		case SDB_IC_UPPER:
			aOwnerName.ToUpperAscii();
			aQualifierName.ToUpperAscii();
		break;
		case SDB_IC_LOWER:
			aOwnerName.ToUpperAscii();
			aQualifierName.ToUpperAscii();
		break;
	}

	aColumns = pConnection->CreateColumns();
	aOriginalColumns = pConnection->CreateColumns();

	bNew = bOpen = TRUE;
	return TRUE;
}

//------------------------------------------------------------------
SdbCursor* SdbTable::OpenCursor(SdbCursorType eType,
								UINT16 nOptions)
{
	SDB_CHK_OPEN_NULL();
	SDB_CHK_CONNECT_NULL();

	SdbCursor* pCursor = pConnection->CreateCursor(eType, nOptions);
	if (!pCursor->Open(GetFullName(), TRUE))
	{
		aStatus = pCursor->Status();
		pCursor->AddRef();
		pCursor->ReleaseRef();
		pCursor = NULL;
	}
	return pCursor;
}

//------------------------------------------------------------------
BOOL SdbTable::Open(const String& rName)
{
	SDB_CHK_CONNECT_BOOL();

	SetNames(rName,String(), pConnection);
	return Open();
}

//------------------------------------------------------------------
BOOL SdbTable::Open()
{
	SDB_CHK_CONNECT_BOOL();

	// Open nicht moeglich wenn Tabellenbeschreibung schon geoeffnet
	if (IsOpen())
	{
		aStatus.SetSequenceError();
		return FALSE;
	}
	aStatus.Clear();

	if (pDatabase) // enthaelt die Datenbank die Tabelle
	{
		if (!pDatabase->HasObject(dbTable,QualifiedName()))
		{
			aStatus.SetError(ERRCODE_IO_NOTEXISTS, dbTable, QualifiedName());
			return FALSE;
		}
	}

	aOriginalColumns = pConnection->CreateColumns();

	BEGIN_DBACTION();
	BUSY_DB();
	bOpen = OpenImpl();

	if (bOpen)
	{
		aColumns = pConnection->CreateColumns(*aOriginalColumns);
	}
	else
	{
		if (aStatus.IsSuccessful())	// keine Fehlerbeschreibung gesetzt
		{
			String aText = String(SdbResId(STR_STAT_INVALID_TABLE_DEF));
			aText.SearchAndReplace(String::CreateFromAscii("#"),QualifiedName());
			aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
		}
		// Strukturbeschreibungen freigeben
		if(aColumns.Is())
			aColumns->Reset();
		if(aOriginalColumns.Is())
			aOriginalColumns->Reset();
	}
	END_DBACTION();
	return bOpen;
}

//------------------------------------------------------------------
void SdbTable::Close()
{
	if (!IsOpen())
		return;

	RemoveChildren();

	CloseImpl();
	pDatabase = NULL;

	// Strukturbeschreibungen freigeben
	aColumns->Reset();
	aOriginalColumns->Reset();
	aStatus.Clear();

	SdbObj::Close();
}

//------------------------------------------------------------------
BOOL SdbTable::Update()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	// Hat sich ueberhaupt etwas geaendert?
	BOOL bResult = TRUE;
	BEGIN_DBACTION();
	BUSY_DB();

	if (aOriginalColumns->Count() <= 1 || bNew)
	{
		if (CreateImpl())
		{
			aOriginalColumns = pConnection->CreateColumns(*aColumns);
			if (pDatabase)	// legt eine Tabelle in der SDB-Datenbank an
			{
				SdbStorage* pStor = pDatabase->OpenStorage(dbTable, FALSE);
				if (!pStor)
				{
					aStatus = pDatabase->Status();
					bResult = FALSE;
				}
				else
				{
					String aQualifiedName = QualifiedName();
					SdbStorageRef aTableStor = pDatabase->OpenStorage(*pStor, aQualifiedName, STREAM_STD_READWRITE | STREAM_TRUNC);
					if (!aTableStor.Is())
					{
						aStatus = pDatabase->Status();
						pDatabase->ReleaseStorage(dbTable,FALSE);
						bResult = FALSE;
					}
					else
					{
						aTableStor.Clear();
						pDatabase->ReleaseStorage(dbTable,TRUE);
						pDatabase->Impl()->Add(aQualifiedName, dbTable);
					}
				}
			}

			// die Connection sollte mitbekommen, dass es jetzt einen neuen Namen gibt
			if (bNew && bResult)
				pConnection->AddTableName(GetFullName(),TRUE);

			if (bResult)
			{
				// noch einmal abgleichen
				// bNew kurzzeitig auf FALSE, damit
				// Refresh() auch etwas tut.
				bNew = FALSE;
				aColumns = pConnection->CreateColumns();
				aOriginalColumns = pConnection->CreateColumns();
				bResult = DoRefresh();
				bNew = TRUE;
			}
		}
		else
			bResult = FALSE;
	}
	else if (!aColumns->Compare(*aOriginalColumns,SDB_FLAGS_NULLALLOWED|SDB_FLAGS_PRIMARYKEY|SDB_FLAGS_AUTOINCREMENT))
	{
		if (AlterImpl())
		{
			SdbStatus aTempStatus(aStatus);
			// auf alle Faelle neu lesen
			aColumns = pConnection->CreateColumns();
			aOriginalColumns = pConnection->CreateColumns();
			DoRefresh();

			if (aStatus.IsSuccessful())
				aStatus = aTempStatus;
		}
		else
			bResult = FALSE;
	}
	END_DBACTION();

	if (bResult)
		bNew = FALSE;	// Tabelle auf alle Faelle nicht mehr neu
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbTable::Drop()
{
	DBG_CHKTHIS(SdbTable,NULL);

	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	aStatus.Clear();
	BOOL bResult = TRUE;
	BEGIN_DBACTION();
	BUSY_DB();

	// Neu angelegte Tabelle, wird nicht zerstrt
	if (IsNew())
		;
	else if (DropImpl())
	{
		if (bResult && pDatabase)
		{
			String aQualifiedName(QualifiedName());
			SdbStringStlMap::iterator aIter = pConnection->pTableNames->find(aQualifiedName);
			pConnection->pTableNames->erase(aIter);
			pDatabase->RemoveStorage(aQualifiedName,dbTable);
		}
	}
	else
		bResult = FALSE;

	END_DBACTION();
	if (bResult)	// beim Vater abmelden
	{
		pConnection->RemoveChild(this);
		bNew = bOpen = FALSE;
	}
	return bResult;
}

//------------------------------------------------------------------
String SdbTable::GetIndexName(USHORT _nPos)
{
	String aIndexName;
	if (GetIndexCount() && aIndexNameList.Count() > _nPos)
		aIndexName = *aIndexNameList[_nPos];
	return aIndexName;
}

//------------------------------------------------------------------
USHORT SdbTable::GetIndexCount()
{
	if (!IsOpen())
		return 0;
	else if (!aIndexNameList.Count())
		FillIndexNames(aIndexNameList);
	return aIndexNameList.Count();
}

//------------------------------------------------------------------
SdbIndex* SdbTable::CreateIndex(const String& _rName)
{
	SDB_CHK_OPEN_NULL();

	if (aIndexList.Lookup(_rName))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,dbIndex,_rName);
		return NULL;
	}

	SdbIndex* pIndex = CreateIndexImpl();
	if (!pIndex)
		return NULL;
	else if (!pIndex->Create(_rName))
	{
		aStatus = pIndex->Status();
		pIndex->RemoveFromParent();
		pIndex = NULL;
	}
	else
		aIndexList.InsertAt(_rName, pIndex);

	return pIndex;
}

//------------------------------------------------------------------
SdbIndex* SdbTable::OpenIndex(const String& _rName)
{
	SDB_CHK_OPEN_NULL();

	SdbIndex* pIndex = aIndexList.Lookup(_rName);
	BOOL bInList = pIndex != NULL;
	if (!bInList)
		pIndex = CreateIndexImpl();

	if (!pIndex)
		return NULL;
	else if (!bInList && !pIndex->Open(_rName))
	{
		pIndex->RemoveFromParent();
		pIndex = NULL;
	}
	else if (!bInList)
		aIndexList.InsertAt(_rName, pIndex);
	return pIndex;
}

//------------------------------------------------------------------
BOOL SdbTable::Rename(const String& rNewName)
{
	SDB_CHK_OPEN_NULL();
	SDB_CHK_CONNECT_BOOL();

	// Hat sich der Name geaendert
	if ((pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT)
	{
		if (rNewName == aName)
			return TRUE;
	}
	else if (aName.EqualsIgnoreCaseAscii(rNewName))
		return TRUE;

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
	ULONG nMaxTableNameLen(rMetaData->getLimit(::stardata::DatabaseLimit::TABLE_NAME_LEN));

	String sCatalog, sSchema, sName;
	if (!pConnection->QualifiedNameComponents(rNewName, sCatalog, sSchema, sName))
		sName = rNewName;	// das Ding war noch nicht qualifiziert
	if (nMaxTableNameLen && (sal_uInt32)sName.Len() > nMaxTableNameLen)
	{
		String aText(SdbResId(STR_STAT_INVALID_TABLENAMELENGTH));
		aText.SearchAndReplace(String::CreateFromAscii("%"),String(nMaxTableNameLen));

		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
		return FALSE;
	}

	// Qualifier und Owner interesieren beim Rename eigentlich nicht
	xub_StrLen  nCount = rNewName.GetTokenCount('.');
	String  aOldName = aName;
	String strOldFullName = GetFullName();

	if (pDatabase)		// Name existiert bereits in der Datenbank?
	{
		String aTestName;

		if (nCount > 1) // Ist der Tabellenname bereits qualifiziert?
			aTestName = rNewName;
		else
		{
			aName = rNewName;
			aTestName = QualifiedName();
			aName = aOldName;
		}

		if (!pDatabase->IsUniqueName(dbTable,aTestName))
		{
			aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,TABLE,aTestName);
			return FALSE;
		}
	}

	// Tabelle ist noch nicht gespeichert worden
	if (bNew)
	{
		SetNames(rNewName,String(),pConnection);

		// auch wenn die Tab neu ist : die TableList der Connection kennt sie schon unter dem alten Namen,
		// das muss ich ihr noch ausreden
		pConnection->aTableList.Replace(this, GetFullName(), this);

		return TRUE;
	}

	BEGIN_DBACTION();
	BUSY_DB();

	BOOL bRefresh(TRUE);
	if (nCount == 3)	// Databasename im Tabellenname (Database.Tabelle)
		bRefresh = RenameImpl(rNewName.GetToken(2,'.'));
	else if (nCount == 2)
		bRefresh = RenameImpl(rNewName.GetToken(1,'.'));
	else
		bRefresh = RenameImpl(rNewName);

	if (pDatabase && aStatus.IsSuccessful())
	{
		// jetzt umbennenen und
		// Datenbank benachrichtigen
		String aOldQualifiedName = QualifiedName();
		aName = rNewName;
		String aNewQualifiedName = QualifiedName();
		pDatabase->RenameStorage(aOldQualifiedName, aNewQualifiedName, dbTable);
	}
	if(bRefresh){
		BOOL bTemp = bNew;
		bNew = FALSE; // ist noetig, damit Refresh() wirklich auffrischt
		DoRefresh();
		bNew = bTemp;
	}
	END_DBACTION();

	BOOL bResult = aStatus.IsSuccessful();
	if (bResult)
	{
		pConnection->aTableList.Replace(this, GetFullName(), this);
		pConnection->ReplaceTableName(strOldFullName, GetFullName());
	}

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbTable::DoRefresh()
{
	if (bNew)
		return TRUE;
	else
	{
		bOpen = FALSE;
		while (aIndexNameList.Count())
			aIndexNameList.Remove(0);
		RemoveChildren();
		BOOL bRes = Open();
		if(!bRes)
		{
			GetConnection()->pTableNames->erase(GetFullName());
			if(pDatabase)
				pDatabase->RemoveStorage(QualifiedName(),dbTable);
		}

		return bRes;
	}
}

//-----------------------------------------------------------------------------
SdbColumn* SdbTable::CreateColumn(const String& _rName,
										 SdbDatabaseType _eType,
										 UINT32 _nLength, UINT16 _nScale,
										 UINT16 _nFlags)
{
	return GetConnection() ? GetConnection()->CreateColumn(_rName,
										 _eType,
										 _nLength,
										 _nScale,
										 _nFlags)
					: NULL;
}

//-----------------------------------------------------------------------------
SdbColumn* SdbTable::CreateColumn(const SdbColumn& _rColumn)
{
	return GetConnection() ? GetConnection()->CreateColumn(_rColumn)
					: NULL;
}

//------------------------------------------------------------------
String SdbTable::QualifiedName()
{
	SDB_CHK_CONNECT_STRING();

	String aTableName = aQualifierName;
	if (aTableName.Len())
		aTableName += pConnection->GetQualifierSeparator();
	if (aOwnerName.Len())
	{
		aTableName += aOwnerName;
		aTableName += '.';
	}
	aTableName += aName;
	return aTableName;
}

//------------------------------------------------------------------
String SdbTable::GetFullName()
{
	SDB_CHK_CONNECT_STRING();

	String aTableName(aQualifierName);
	if (aTableName.Len())
		aTableName += pConnection->GetQualifierSeparator();
	if (aOwnerName.Len())
	{
		aTableName += aOwnerName;
		aTableName += '.';
	}
	aTableName += aName;
	return aTableName;
}

//------------------------------------------------------------------
String SdbTable::GetFullName(SdbDatabase& rDatabase, const String& rName, const String& rDBName)
{
	SdbConnection* pConn = rDatabase.GetConnection();
	return (pConn) ? GetFullName(*pConn, rName, rDBName) : String();
}

//------------------------------------------------------------------
String SdbTable::GetFullName(SdbConnection& rConnection, const String& rName, const String& rDB)
{
	// Der Name kann voll qualifiziert angegeben werden
	// also Qualifier.Owner.Table
	//		Owner.Table + Datenbank
	//		Qualifier.Table
	//		Owner.Table
	//		Table + Datenbank
	//		Table

	String aTableName(rName);
	String aQualifierName, aOwnerName, aName;
	
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(rConnection.GetMetaData());	
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	BOOL   bQualifier = nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML;
	BOOL   bOwner	  = nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML;

	char cSeparator = rConnection.GetQualifierSeparator();
	if (bQualifier)
	{
		char cSeparator = rConnection.GetQualifierSeparator();
		if (rDB.Len())
			aQualifierName = rDB;
		else if (cSeparator != '.' && aTableName.GetTokenCount(cSeparator) == 2)
		{
			aQualifierName = aTableName.GetToken(0,cSeparator);
			aTableName = aTableName.GetToken(1,cSeparator);
		}
		else if (!bOwner && aTableName.GetTokenCount(cSeparator) == 2)
		{
			aQualifierName = aTableName.GetToken(0,cSeparator);
			aTableName = aTableName.GetToken(1,cSeparator);
		}
		else if (bOwner && aTableName.GetTokenCount(cSeparator) == 3)
		{
			aQualifierName = aTableName.GetToken(0,cSeparator);
			aTableName.Erase(0, aQualifierName.Len() + 1);
		}
	}

	if (bOwner && aTableName.GetTokenCount('.') == 2)
	{
		aOwnerName = aTableName.GetToken(0,'.');
		aTableName = aTableName.GetToken(1,'.');
	}


	if (bQualifier && !aQualifierName.Len())
	{

		aQualifierName = rConnection.GetQualifier();
		// versuchen, sicherzustellen, dass der Qualifier
		// nicht doch schon im Namen vorkommt...
		{
			xub_StrLen nPos;
			String sToSearch;
			sal_Int16 nInfo;

			rMetaData->getInfo(::com::sun::star::data::DatabaseInfo::IDENT_CASE_QUOTED) >>= nInfo;
			if (nInfo == ::stardata::DatabaseIdentifierCase::SENSITIVE)
			{
				// case sensitive
				sToSearch = aQualifierName.ToUpperAscii();
				sToSearch += cSeparator;
				String sTableName(aTableName.ToUpperAscii());
				nPos = sTableName.Search(sToSearch);
			}
			else
			{
				// case insensitive
				sToSearch = aQualifierName;
				sToSearch += cSeparator;
				nPos = aTableName.Search(sToSearch);
			};


			if (nPos != STRING_NOTFOUND)
			// QualifierName ist enthalten, also erstmal "herausschneiden"
			// QualifierName muss am Beginn stehen...
				aTableName.Erase(0, sToSearch.Len());
		}

	}

	if (bOwner && !aOwnerName.Len())
		aOwnerName = rConnection.GetUser();

	aName = aTableName;

	// jetzt den Tabellennamen wieder zusammensetzen
	aTableName = aQualifierName;
	if (aTableName.Len())
		aTableName += cSeparator;
	if (aOwnerName.Len())
	{
		aTableName += aOwnerName;
		aTableName += '.';
	}
	aTableName += aName;
	return aTableName;
}

//------------------------------------------------------------------
void SdbTable::SetNames(const String& rName, const String& rDB, SdbConnection* pConnection)
{
	// Der Name kann voll qualifiziert angegeben werden
	// also Qualifier.Owner.Table
	//		Owner.Table + Datenbank
	//		Qualifier.Table
	//		Owner.Table
	//		Table + Datenbank
	//		Table
	String aTableName(rName);
	char cSeparator = pConnection->GetQualifierSeparator();

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());	
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	BOOL   bQualifier = nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML;
	BOOL   bOwner	  = nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML;

	if (bQualifier)
	{
		if (rDB.Len())
			aQualifierName = rDB;
		else if (cSeparator != '.' && aTableName.GetTokenCount(cSeparator) == 2)
		{
			aQualifierName = aTableName.GetToken(0,cSeparator);
			aTableName = aTableName.GetToken(1,cSeparator);
		}
		else if (!bOwner && aTableName.GetTokenCount(cSeparator) == 2)
		{
			aQualifierName = aTableName.GetToken(0,cSeparator);
			aTableName = aTableName.GetToken(1,cSeparator);
		}
		else if (bOwner && aTableName.GetTokenCount(cSeparator) == 3)
		{
			aQualifierName = aTableName.GetToken(0,cSeparator);
			aTableName.Erase(0, aQualifierName.Len() + 1);
		}
	}

	if (bOwner && aTableName.GetTokenCount('.') == 2)
	{
		aOwnerName = aTableName.GetToken(0,'.');
		aTableName = aTableName.GetToken(1,'.');
	}


	if (bQualifier && !aQualifierName.Len())
	{
		aQualifierName = GetConnection()->GetQualifier();
		// versuchen, sicherzustellen, dass der Qualifier
		// nicht doch schon im Namen vorkommt...
		{
			xub_StrLen nPos;
			String sToSearch;
			sal_Int16 nInfo;
			rMetaData->getInfo(::com::sun::star::data::DatabaseInfo::IDENT_CASE_QUOTED) >>= nInfo;

			if (nInfo == ::stardata::DatabaseIdentifierCase::SENSITIVE)
			{
				// case sensitive
				sToSearch = aQualifierName.ToUpperAscii();
				sToSearch += cSeparator;
				String sTableName(aTableName.ToUpperAscii());
				nPos = sTableName.Search(sToSearch);
			}
			else
			{
				// case insensitive
				sToSearch = aQualifierName;
				sToSearch += cSeparator;
				nPos = aTableName.Search(sToSearch);
			};


			if (nPos != STRING_NOTFOUND)
			// QualifierName ist enthalten, also erstmal "herausschneiden"
			// QualifierName muss am Beginn stehen...
				aTableName.Erase(0, sToSearch.Len());
		}

	}

	if (bOwner && !aOwnerName.Len())
		aOwnerName = GetConnection()->GetUser();

	aName = aTableName;

	switch (GetConnection()->GetIdentifierCase())
	{
		case SDB_IC_LOWER:
			aName.ToLowerAscii();
			aOwnerName.ToLowerAscii();
			aQualifierName.ToLowerAscii();
			break;
		case SDB_IC_UPPER:
			aName.ToUpperAscii();
			aOwnerName.ToUpperAscii();
			aQualifierName.ToUpperAscii();
			break;
	}
}

//------------------------------------------------------------------
BOOL SdbTable::Compare(const SdbObj& _rObj) const
{
	SdbTable* pObj = PTR_CAST(SdbTable,&_rObj);
	SdbConnection* pConnection = GetConnection();

	return pObj != NULL && pConnection != NULL &&
		   ((pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT ?
				aName.EqualsIgnoreCaseAscii(pObj->aName) &&
				aQualifierName.EqualsIgnoreCaseAscii(pObj->aQualifierName) &&
				aOwnerName.EqualsIgnoreCaseAscii(pObj->aOwnerName)
											:
				aName == pObj->aName &&
				aQualifierName == pObj->aQualifierName &&
				aOwnerName == pObj->aOwnerName)
											&&
			aColumns->Compare(*pObj->aColumns,SDB_FLAGS_NULLALLOWED|SDB_FLAGS_PRIMARYKEY|SDB_FLAGS_AUTOINCREMENT);

}

//------------------------------------------------------------------
SdbConnection* SdbTable::GetConnection() const
{
#ifdef DBG_UTIL
	return PTR_CAST(SdbConnection,GetParent());
#else
	return (SdbConnection*)GetParent();
#endif
}

//------------------------------------------------------------------
void SdbTable::RemoveChildren()
// freigeben aller Childs, siehe SdbObj
{
	while (aIndexList.Count())
		RemoveChild(aIndexList.GetObject(0));
}

//------------------------------------------------------------------
void SdbTable::RemoveChild(SdbObj* pObj)
{
	aIndexList.Remove(PTR_CAST(SdbIndex,pObj));
	SdbObj::RemoveChild(pObj);
}



