/*************************************************************************
 *
 *  $RCSfile: sdbtools.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// sdbtools.cxx

// include math.h before "using namespace std" in tools.hxx
// to prevent ambigous usage of std::exception and exception
#include <math.h>

#ifndef _SDB_VARIANT_HXX
#include <variant.hxx>
#endif

#include "sdbtools.hxx"

#include <stdio.h>			// sprintf
#include <stdlib.h>			// strtod
#include <ctype.h>			// touppe
#include <string.h>

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _INTN_HXX //autogen
#include <tools/intn.hxx>
#endif

#ifndef _ZFORLIST_HXX //autogen
#include <svtools/zforlist.hxx>
#endif

#ifndef _SOLMATH_HXX //autogen wg. SolarMath
#include <tools/solmath.hxx>
#endif

#ifndef _SDB_TYPECONV_HXX
#include <typeconv.hxx>
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif


const double fMilliSecondsPerDay = 86400000.0;
static Date aStandard(1,1,1900);

//------------------------------------------------------------------
String SdbTools::SetQuotation(const String& rValue, const String& rQuot, const String& rQuotToReplace)
{
	String rNewValue = rQuot;
	rNewValue+= rValue;

	USHORT nIndex = (USHORT)-1;	// Quotes durch zweifache Quotes ersetzen, sonst kriegt der Parser Probleme

	if (rQuot.Len())
		rNewValue.SearchAndReplaceAll(rQuot,rQuotToReplace);
	rNewValue += rQuot;
	return rNewValue;
}

//------------------------------------------------------------------
UINT32 SdbTools::DefaultFormat(SvNumberFormatter& rFormatter, SdbDatabaseType eType, UINT16 nScale)
{
	UINT32 nKey = 0;
	switch (eType)
	{
		case SDB_DBTYPE_BOOLEAN:
		case SDB_DBTYPE_TINYINT:
			nKey = rFormatter.GetStandardFormat(NUMBERFORMAT_LOGICAL, Application::GetAppInternational().GetLanguage());
			break;
		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_LONGVARCHAR:
			nKey = rFormatter.GetStandardFormat(NUMBERFORMAT_TEXT, Application::GetAppInternational().GetLanguage());
			break;
		case SDB_DBTYPE_SMALLINT:
		case SDB_DBTYPE_INTEGER:
		case SDB_DBTYPE_BIGINT:
			nKey = rFormatter.GetStandardFormat(NUMBERFORMAT_NUMBER, Application::GetAppInternational().GetLanguage());
			break;
		case SDB_DBTYPE_CURRENCY:
			nKey = rFormatter.GetStandardFormat(NUMBERFORMAT_CURRENCY, Application::GetAppInternational().GetLanguage());
			break;
		case SDB_DBTYPE_REAL:
		case SDB_DBTYPE_DOUBLE:
			if (!nScale)
				nScale = 2;
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
		{
			String	aFormatString;
			xub_StrLen	nCheckPos=0;
			short	nType;
			rFormatter.GenerateFormat(aFormatString, 0L, LANGUAGE_DONTKNOW, FALSE,	FALSE, nScale);
			rFormatter.PutEntry(aFormatString,
								nCheckPos,
								nType,
								nKey);
		}	break;
		case SDB_DBTYPE_TIMESTAMP:
		case SDB_DBTYPE_DATE:
			nKey = rFormatter.GetStandardFormat(NUMBERFORMAT_DATE, Application::GetAppInternational().GetLanguage());
			break;
		case SDB_DBTYPE_TIME:
			nKey = rFormatter.GetStandardFormat(NUMBERFORMAT_TIME, Application::GetAppInternational().GetLanguage());
			break;
	}
	return nKey;
}

//------------------------------------------------------------------
String SdbTools::ToDateString(const Date& rDate)
{
	sal_Char s[11];
	sprintf(s,"%04d-%02d-%02d",
				(int)rDate.GetYear(),
				(int)rDate.GetMonth(),
				(int)rDate.GetDay());
	s[10] = 0;
	return String::CreateFromAscii(s);
}

//------------------------------------------------------------------
String SdbTools::ToTimeString(const Time& rTime)
{
	sal_Char s[9];
	sprintf(s,"%02d:%02d:%02d",
			(int)rTime.GetHour(),
			(int)rTime.GetMin(),
			(int)rTime.GetSec());
	s[8] = 0;
	return String::CreateFromAscii(s);
}

//------------------------------------------------------------------
String SdbTools::ToDateTimeString(const DateTime& rDateTime)
{	
	String aTemp(ToDateString(rDateTime));
	aTemp += String::CreateFromAscii(" ");
	aTemp += ToTimeString(rDateTime);
	return  aTemp;
}

//------------------------------------------------------------------
String SdbTools::SQLString(SdbDatabaseType eType, const ODbVariant& rVal, BOOL bQuote)
{
	String aRet;
	if (rVal.isNull())
		aRet = String::CreateFromAscii(" NULL ");
	else
	{
		try
		{
			switch (eType)
			{
				case SDB_DBTYPE_INTEGER:
				case SDB_DBTYPE_BOOLEAN:
				case SDB_DBTYPE_TINYINT:
				case SDB_DBTYPE_SMALLINT:
					if (rVal.getValueType().getTypeClass() == ::com::sun::star::uno::TypeClass_BOOLEAN)
					{
						if (ODbTypeConversion::toBOOL(rVal))
							aRet = '1';
						else
							aRet = '0';
					}
					else
						aRet = ODbTypeConversion::toString(rVal).getStr();
					break;
				case SDB_DBTYPE_CHAR:
				case SDB_DBTYPE_VARCHAR:
					if (bQuote)
						aRet += '\'';
					aRet += ODbTypeConversion::toString(rVal).getStr();
					if (bQuote)
						aRet += '\'';
					break;
				case SDB_DBTYPE_CURRENCY:
				case SDB_DBTYPE_REAL:
				case SDB_DBTYPE_DOUBLE:
				case SDB_DBTYPE_DECIMAL:
				case SDB_DBTYPE_NUMERIC:
					aRet = ToString(ODbTypeConversion::toDouble(rVal),4);
					break;
				case SDB_DBTYPE_TIMESTAMP:
				{
					DateTime aDateTime = ODbTypeConversion::toDateTime(rVal);
					if (aDateTime.GetMSFromTime() / fMilliSecondsPerDay)
					{
						if (bQuote) 
							aRet += String::CreateFromAscii("{TS '");
						aRet += ToDateString(aDateTime);
						aRet += String::CreateFromAscii(" ");
						aRet += ToTimeString(aDateTime);
						if (bQuote) 
							aRet += String::CreateFromAscii("'}");
						break;
					}
					// else continue
				} 			
				case SDB_DBTYPE_DATE:
				{
					Date aDate = ODbTypeConversion::toDate(rVal);
					if (bQuote) aRet += String::CreateFromAscii("{D '");
					aRet += ToDateString(aDate);;
					if (bQuote) aRet += String::CreateFromAscii("'}");
				}	break;
				case SDB_DBTYPE_TIME:
				{
					Time aTime = ODbTypeConversion::toTime(rVal);
					if (bQuote) aRet += String::CreateFromAscii("{T '");
					aRet += ToTimeString(aTime);
					if (bQuote) aRet += String::CreateFromAscii("'}");
				} break;			
				default:
					aRet = ODbTypeConversion::toString(rVal).getStr();
			}
		}
		catch ( ... )
		{
			DBG_ERROR("TypeConversion Error");
		}		
	}
	return aRet;
}

//------------------------------------------------------------------
// Zahlenumwandlung
/*int GetNumber(char*&Text)
{
	int Result = 0;
	if (isdigit (*Text))
	{
		// Assume at least one digit
		Result = int (::strtol (Text,(char**)&Text,10)); // Eat all leading digits
		if (*Text && !isalpha(*Text)) Text++; // jump over a delimiter
	}
	return Result;
}*/

/*
//------------------------------------------------------------------
BOOL SdbTools::DBDateTime(double& nRes, SdbDatabaseType eDBType, const SdbVariable& rValue)
{
	DBG_TRACE( "DBDateTime" );
	// String kann Fliesskommazahl darstellen !!

	// Pruefen, ob nicht schon Datum
	if (eDBType != SDB_DBTYPE_CHAR &&
		eDBType != SDB_DBTYPE_VARCHAR)
	{
		nRes = rValue.GetDate();
		return FALSE;
	}

	// Pruefen, ob Fliesskommazahl
	if( rValue.IsDouble() )
	{
		nRes = rValue.GetDouble();
		return FALSE;
	}

	// Wenn nicht, Stringobjekt erzeugen
	String aDateTimeString;
	aDateTimeString = rValue.GetString();

	// Pruefen, ob Zeitangabe, wenn nicht, anhaengen -> Standard
	if( aDateTimeString.GetTokenCount(' ') < 2 )
		aDateTimeString.Insert( " 00:00:00" );

	// Splitten nach Date und Time
	String aDateString = aDateTimeString.GetToken( 0, ' ' );
	String aTimeString = aDateTimeString.GetToken( 1, ' ' );

	// Pruefen, ob dieses eine Fliesskommazahl in Bruchdarstellung ist
	if( aDateString.GetTokenCount('/') == 2 )
	{
		nRes = rValue.GetDouble();
		return FALSE;
	}

	// Pruefen, ob Datumsseparation mit internat. Separator
	International aInternational = GetpApp()->GetAppInternational();
	DateFormat eDateFormat = aInternational.GetDateFormat();

	char aSeparator = aInternational.GetDateSep();

	if (aDateString.GetTokenCount( aSeparator ) == 3 )
		;

	// Pruefen, ob Datumsseparation mit '.'
	else if( aDateString.GetTokenCount( '.' ) == 3 )
		aSeparator = '.';

	// Pruefen, ob Datumsseparation mit '/'
	else if( aDateString.GetTokenCount( '/' ) == 3 )
		aSeparator = '/';

	// Pruefen, ob Datumsseparation mit '-'
	else if( aDateString.GetTokenCount( '-' ) == 3 )
		aSeparator = '-';

	// Wenn keiner der Separatoren in DateString, Abbruch
	else
	{
		nRes = 0;
		return FALSE;
	}

	// ... Auswertung des Strings
	int nDay, nMonth, nYear, nHour, nMinute, nSecond;
	String aYear;

	// Wenn Abfrage in default SQL-Datumsformat, dann das gesetzte Format ignorieren
	if (aDateString.Len() > 5 && aDateString[USHORT(4)] == '-')
		eDateFormat = YMD;

	switch (eDateFormat)
	{
		case MDY:
			nMonth	= (int)aDateString.GetToken(0, aSeparator);
			nDay	= (int)aDateString.GetToken(1, aSeparator);
			aYear	= aDateString.GetToken(2, aSeparator);
			break;

		case YMD:
			aYear	= aDateString.GetToken(0, aSeparator);
			nMonth	= (int)aDateString.GetToken(1, aSeparator);
			nDay	= (int)aDateString.GetToken(2, aSeparator);
			break;
		default:
			nDay	= (int)aDateString.GetToken(0, aSeparator);
			nMonth	= (int)aDateString.GetToken(1, aSeparator);
			aYear	= aDateString.GetToken(2, aSeparator);

	}

	// Expansion von zweistelligen Jahreswerten auf vierstellige
	// Bsp.: aus '97' wird '1997'
	if( aYear.Len() == 2 )
	{
		String aTemp = "19";
		aTemp += aYear;
		aYear = aTemp;
	}
	nYear = (int)aYear;

	nHour = (int)aTimeString.GetToken(0, ':');
	nMinute = (int)aTimeString.GetToken(1, ':');
	nSecond = (int)aTimeString.GetToken(2, ':');

	// Datum als long berechnen
	Date aStandard(1,1,1900);
	Date aDate(nDay,nMonth,nYear);

	// Anzahl Tage:
	long nDays;
	if (aStandard > aDate)
	{
		nDays = (long)(aStandard - aDate);
		nDays *= -1;
	}
	else
		nDays = (long)(aDate - aStandard);

	nDays += 2;		// Basic-Anpassung!

	// Sekunden seit Tagesbeginn:
	long nSeconds = nHour * 3600 +
					nMinute * 60 +
					nSecond;

	nRes = (double)nDays + ((double)nSeconds / (double) (24.0*3600.0));
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbTools::DBDate(double& nRes, SdbDatabaseType eDBType, const SdbVariable& rValue)
{
	DBG_TRACE( "DBDate" );
	return DBDateTime( nRes, eDBType, rValue );
}

//------------------------------------------------------------------
BOOL SdbTools::DBTime(double& nRes, SdbDatabaseType eDBType, const SdbVariable& rValue)
{
	// String wird erwartet als HH:MM:SS
	if (eDBType != SDB_DBTYPE_CHAR &&
		eDBType != SDB_DBTYPE_VARCHAR)
	{
		nRes = rValue.GetDate();
		return FALSE;
	}

	static char buffer[11];
	char* pC = buffer;

	strncpy(buffer,rValue.GetString(),10);

	if (buffer[2] != ':')
	{
		nRes = rValue.GetDouble();
		return FALSE;
	}

	int nHour = GetNumber(pC);
	int nMinute = GetNumber(pC);
	int nSecond = GetNumber(pC);

	// Sekunden seit Tagesbeginn:
	long nSeconds = nHour * 3600 +
					nMinute * 60 +
					nSecond;

	nRes = ((double)nSeconds / (double) (24.0*3600.0));
	return FALSE;
}*/

//------------------------------------------------------------------
BOOL SdbTools::ToDateTime(double& fResult, const String& rStr)
{
	fResult = 0;
	BOOL bContainsDate = rStr.GetTokenCount('-') == 3;
	BOOL bContainsTime = rStr.GetTokenCount(':') == 3;
	String aDateStr;
	String aTimeStr;
	if (bContainsDate && bContainsTime)
	{
		if (rStr.GetTokenCount(' ') != 2)
			return FALSE;
		
		aDateStr = rStr.GetToken(0, ' ');
		aTimeStr = rStr.GetToken(1, ' ');
	}
	else if (bContainsDate)
		aDateStr = rStr;
	else if (bContainsTime)
		aTimeStr = rStr;
	else 
		return FALSE;

	fResult = 0;
	if (bContainsDate)
	{
		if (aDateStr.Len() != 10)
			return FALSE;

		String aYear = aDateStr.GetToken(0,'-');
		String aMonth = aDateStr.GetToken(1,'-');
		String aDay = aDateStr.GetToken(2,'-');

		if (aYear.Len() != 4 || aMonth.Len() != 2 || aDay.Len() != 2)
			return FALSE;

		fResult = ToDouble(Date(aDay.ToInt32(),aMonth.ToInt32(),aYear.ToInt32()));
	}

	if (bContainsTime)
	{
		if (aTimeStr.Len() != 8)
			return FALSE;

		String aHour = aTimeStr.GetToken(0,':');
		String aMin = aTimeStr.GetToken(1,':');
		String aSec = aTimeStr.GetToken(2,':');

		if (aHour.Len() != 2 || aMin.Len() != 2 || aSec.Len() != 2)
			return FALSE;

		fResult += ToDouble(Time(aHour.ToInt32(), aMin.ToInt32(), aSec.ToInt32()));
	}
	return TRUE;
}

//------------------------------------------------------------------
String SdbTools::ToString(double fValue,int nScale,char cOp)
{
	String aString;
	SolarMath::DoubleToString(aString,fValue,cOp,nScale,'.');
	return aString;
}

//------------------------------------------------------------------
double SdbTools::ToDouble(const String& rString)
{
	static International aInter(LANGUAGE_ENGLISH);
	static int nErrno=0;
	BOOL bInitialized = FALSE;
	if (!bInitialized)
	{	// ensure that the two members we're interested in are really set
		// (if the system doesn't know the locale en_US aIntl would be initialized with the
		// system language which may be anything - which we don't want ...)
		// 74342 - 21.03.00 - FS
		aInter.SetNumThousandSep(',');
		aInter.SetNumDecimalSep('.');
		bInitialized = TRUE;
	}
	return SolarMath::StringToDouble(rString.GetBuffer(),aInter,nErrno);
}
//------------------------------------------------------------------
double SdbTools::ToDouble(const ByteString& rString)
{
	static International aInter(LANGUAGE_ENGLISH);
	static int nErrno=0;
	BOOL bInitialized = FALSE;
	if (!bInitialized)
	{	// ensure that the two members we're interested in are really set
		// (if the system doesn't know the locale en_US aIntl would be initialized with the
		// system language which may be anything - which we don't want ...)
		// 74342 - 21.03.00 - FS
		aInter.SetNumThousandSep(',');
		aInter.SetNumDecimalSep('.');
		bInitialized = TRUE;
	}
	return SolarMath::StringToDouble(UniString(rString,gsl_getSystemTextEncoding()).GetBuffer(),aInter,nErrno);
}
//------------------------------------------------------------------
double SdbTools::ToStandardDate(SvNumberFormatter& rFormatter, double rVal)
{
	if (*rFormatter.GetNullDate() != aStandard)
		rVal -= (aStandard - *rFormatter.GetNullDate());
	return rVal;
}

//------------------------------------------------------------------
double SdbTools::ToFormatterDate(SvNumberFormatter& rFormatter, double rVal)
{		
	if (*rFormatter.GetNullDate() != aStandard)
		rVal += (aStandard - *rFormatter.GetNullDate());
	return rVal;
}

//------------------------------------------------------------------
double SdbTools::ToDouble(const DateTime& x)
{
	// In Float umrechnen [nach Visual Basic-Konventionen,
	// d. h. Tage seit 1.1.1900 *** + 2.0 ***]
	// Anzahl Tage:
	long nDays = 0;
	if (aStandard > x)
	{
		nDays = (long)(aStandard - x);
		nDays *= -1;
	}
	else
	{
		nDays = (long)(x - aStandard);
	}
//	nDays += 2;		// Basic-Anpassung!

	// Als double (Tage und Bruchteile von Tagen) in die
	// Basic-Variable schreiben:
	double fMSeconds = x.GetMSFromTime();
	return ((double)nDays) + (fMSeconds * (1/fMilliSecondsPerDay));
}

//------------------------------------------------------------------
double SdbTools::ToDouble(const Date& x)
{
	// In Float umrechnen [nach Visual Basic-Konventionen,
	// d. h. Tage seit 1.1.1900 *** + 2.0 ***]
	// Anzahl Tage:
	long nDays = 0;	
	if (aStandard > x)
	{
		nDays = (long)(aStandard - x);
		nDays *= -1;
	}
	else
		nDays = (long)(x - aStandard);

//	nDays += 2;		// Basic-Anpassung!

	return (double)nDays;
}

//------------------------------------------------------------------
double SdbTools::ToDouble(const Time& x)
{
	// Als double (Tage und Bruchteile von Tagen) in die
	// Basic-Variable schreiben:
	return (double)x.GetMSFromTime() / fMilliSecondsPerDay;
}

//------------------------------------------------------------------
DateTime SdbTools::ToDateTime(const double& fSbxDate)
{
	// Basic-Variable enthaelt Tage und Bruchteile von
	// Tagen seit 1.1.1900 *** + 2 ***!
	long nDays     = (long)fSbxDate;
	long nMSeconds = long((fSbxDate - (double)nDays) * fMilliSecondsPerDay + 0.5);

	// Datum:
	DateTime x(aStandard);
	if (nDays >= 0)
		x += (ULONG)nDays;
	else
		x -= (ULONG)(-nDays);

	// Zeit:
	x.MakeTimeFromMS(nMSeconds);
	if(x.GetTime() < 0)
	{	x -= 1;
		x += Time(23,59,59,99);
	}
	return x;
}

//------------------------------------------------------------------
Date SdbTools::ToDate(const double& fSbxDate)
{
	// Basic-Variable enthaelt Tage und Bruchteile von
	// Tagen seit 1.1.1900 *** + 2 ***!
	long nDays     = (long)fSbxDate;

	// Datum:
	Date x(aStandard);
//	x -= 2;				// Korrektur fuer Basic
	if (nDays >= 0)
		x += (ULONG)nDays;
	else
		x -= (ULONG)(-nDays);

	return x;
}

//------------------------------------------------------------------
Time SdbTools::ToTime(const double& fSbxDate)
{
	long nDays     = (long)fSbxDate;
	long nMSeconds = long((fSbxDate - (double)nDays) * fMilliSecondsPerDay + 0.5);

	Time x;
	x.MakeTimeFromMS(nMSeconds);
	if(x.GetTime() < 0)
		x += Time(23,59,59,99);
	return x;
}
//-----------------------------------------------------------------------------
SdbSqlParseNode* SdbTools::MakeANDNode(SdbSqlParseNode *pLeftLeaf,SdbSqlParseNode *pRightLeaf)
{
	SdbSqlParseNode* pNewNode = new SdbSqlParseNode(String(),SDB_SQL_RULE,SdbSqlParser::RuleID(SdbSqlParseNode::boolean_term));
	pNewNode->Append(pLeftLeaf);
	pNewNode->Append(new SdbSqlParseNode(String::CreateFromAscii("AND"),SDB_SQL_KEYWORD,SDB_TOKEN_AND));
	pNewNode->Append(pRightLeaf);
	return pNewNode;
}

