/*************************************************************************
 *
 *  $RCSfile: sdbview.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbview.hxx"

#ifndef _SDB_SDBCONN_HXX
#include <sdbconn.hxx>
#endif

#ifndef _SDB_SDBCURS_HXX
#include <sdbcurs.hxx>
#endif

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASECAPABILITY_HPP_
#include <com/sun/star/data/DatabaseCapability.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEIDENTIFIERCASE_HPP_
#include <com/sun/star/data/DatabaseIdentifierCase.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif


DBG_NAME(SdbView);

TYPEINIT1(SdbView, SdbTable);

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

//------------------------------------------------------------------
SdbView::SdbView(SdbConnection* _pConn)
		:SdbTable(_pConn)
{
	DBG_CTOR(SdbView,NULL);
	bView = TRUE;
}

//------------------------------------------------------------------
void SdbView::SetWrappedTable(SdbTable* pTable)
{
	xTable = pTable;

	aName = xTable->Name();
	aOwnerName = xTable->OwnerName();
	aQualifierName = xTable->QualifierName();

	bOpen = TRUE;
	bNew = FALSE;

	// einlesen der Felder
	aOriginalColumns = xTable->OriginalColumns();
	aColumns = xTable->Columns();
	nPrivileges = xTable->GetTablePrivileges();
	nPrivileges &= ~(SDB_PR_ALTER|SDB_PR_REFERENCES);
}

//------------------------------------------------------------------
SdbView::~SdbView()
{
	DBG_DTOR(SdbView,NULL);

	if (xTable.Is())
	{
		xTable->RemoveFromParent();
	}
}

//------------------------------------------------------------------
void SdbView::CloseImpl()
{
	if (xTable.Is())
	{
		xTable->RemoveFromParent();
		xTable.Clear();
	}
}

//------------------------------------------------------------------
BOOL SdbView::DoRefresh()
{
	DBG_ASSERT(xTable.Is(), "kein Tabelle fuer die View gesetzt");
	if (bOpen)
	{
			// einlesen der Felder
		aOriginalColumns = xTable->OriginalColumns();
		aColumns = xTable->Columns();
		nPrivileges = xTable->GetTablePrivileges();
		nPrivileges &= ~(SDB_PR_ALTER|SDB_PR_REFERENCES);
	}
	else if (!bOpen && xTable.Is())
	{
		aStatus = xTable->Status();
		xTable.Clear();
	}
	return bOpen;
}
//------------------------------------------------------------------
BOOL SdbView::Update()
{
	// dummy implementierung
	return FALSE;
}

//------------------------------------------------------------------
BOOL SdbView::Update(const String& _rSQL, BOOL _bWithCheckOption)
{
	SDB_CHK_OPEN_BOOL();
	if (!bNew)
	{
		aStatus.SetSequenceError();
		return FALSE;
	}

	SDB_CHK_CONNECT_BOOL();
	BOOL bResult = TRUE;
	
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	String aCommand = String::CreateFromAscii("CREATE VIEW ");	

	// wann ein Qualifier genommen wird und wann nicht richtet sich in der Regel an
	// die Qualifierzuweisung bei Indices
	if (aQualifierName.Len() > 0 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_INDEX))
	{
		aCommand += pConnection->IdentifierQuotation(aQualifierName);
		aCommand += pConnection->GetQualifierSeparator();
	}
	if (aOwnerName.Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_INDEX))
	{
		aCommand += pConnection->IdentifierQuotation(aOwnerName);
		aCommand += '.';
	}

	aCommand += pConnection->IdentifierQuotation(aName);
	aCommand += String::CreateFromAscii(" AS ");
	aCommand += _rSQL;

	if (!pConnection->ExecuteSQL(aCommand))
		aStatus = pConnection->Status();

	if (bResult = aStatus.IsSuccessful())
	{
		bNew	= FALSE;

		if (pDatabase)		// in der Datenbank eintragen
		{
			SdbStorage* pStor = pDatabase->OpenStorage(dbTable, FALSE);
			if (!pStor)
			{
				aStatus = pDatabase->Status();
				DropImpl();
				bResult = FALSE;
			}
			else
			{
				String aQualifiedName = QualifiedName();
				SdbStorageRef aTableStor = pDatabase->OpenStorage(*pStor, aQualifiedName, STREAM_STD_READWRITE | STREAM_TRUNC);
				if (!aTableStor.Is())
				{
					aStatus = pDatabase->Status();
					pDatabase->ReleaseStorage(dbTable,FALSE);
					DropImpl();
					bResult = FALSE;
				}
				else
				{
					aTableStor.Clear();
					pDatabase->ReleaseStorage(dbTable,TRUE);
					pDatabase->Impl()->Add(aQualifiedName, dbTable);
				}
			}
		}

		if (bResult)
		{
			xTable	= pConnection->CreateTableImpl();
			if (xTable->Open(GetFullName()))
			{
				// einlesen der Felder
				aOriginalColumns = xTable->OriginalColumns();
				aColumns = xTable->Columns();
				nPrivileges = xTable->GetTablePrivileges();
				nPrivileges &= ~(SDB_PR_ALTER|SDB_PR_REFERENCES);
				// die Connection sollte mitbekommen, dass es jetzt einen neuen Namen gibt
				pConnection->AddTableName(GetFullName(),FALSE);
				bResult = DoRefresh();
			}
			else
			{
				SdbStatus aTempStatus = xTable->Status();
				bResult = FALSE;
				xTable->RemoveFromParent();
				xTable.Clear();
				DropImpl();
				String aQualifiedName = QualifiedName();
				pDatabase->RemoveStorage(aQualifiedName,dbTable);
				aStatus = aTempStatus;
				bOpen = FALSE;
			}
		}
	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbView::DropImpl()
{
	DBG_CHKTHIS(SdbView,NULL);
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;

	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	String aCommand( String::CreateFromAscii("DROP VIEW ") );
	if (aQualifierName.Len() > 0 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_INDEX))
	{
		aCommand += pConnection->IdentifierQuotation(aQualifierName);
		aCommand += pConnection->GetQualifierSeparator();
	}
	if (aOwnerName.Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_INDEX))
	{
		aCommand += pConnection->IdentifierQuotation(aOwnerName);
		aCommand += '.';
	}
	aCommand += pConnection->IdentifierQuotation(aName);

	if (!pConnection->ExecuteSQL(aCommand))
		aStatus = pConnection->Status();

	return aStatus.IsSuccessful();
}
//------------------------------------------------------------------
BOOL SdbView::UpdateStorage()
{
	SDB_CHK_OPEN_BOOL();
	SDB_CHK_CONNECT_BOOL();
	BOOL bResult;
	if (bResult = aStatus.IsSuccessful())
	{
		bNew	= FALSE;

		if (pDatabase)		// in der Datenbank eintragen
		{
			SdbStorage* pStor = pDatabase->OpenStorage(dbTable, FALSE);
			if (!pStor)
			{
				aStatus = pDatabase->Status();
				DropImpl();
				bResult = FALSE;
			}
			else
			{
				String aQualifiedName = QualifiedName();
				SdbStorageRef aTableStor = pDatabase->OpenStorage(*pStor, aQualifiedName, STREAM_STD_READWRITE | STREAM_TRUNC);
				if (!aTableStor.Is())
				{
					aStatus = pDatabase->Status();
					pDatabase->ReleaseStorage(dbTable,FALSE);
					DropImpl();
					bResult = FALSE;
				}
				else
				{
					aTableStor.Clear();
					pDatabase->ReleaseStorage(dbTable,TRUE);
					pDatabase->Impl()->Add(aQualifiedName, dbTable);
				}
			}
		}

		if (bResult)
		{
			xTable	= pConnection->CreateTableImpl();
			if (xTable->Open(GetFullName()))
			{
				// einlesen der Felder
				aOriginalColumns = xTable->OriginalColumns();
				aColumns = xTable->Columns();
				nPrivileges = xTable->GetTablePrivileges();
				nPrivileges &= ~(SDB_PR_ALTER|SDB_PR_REFERENCES);

			}
			else
			{
				aStatus = xTable->Status();
				bResult = FALSE;
				xTable->RemoveFromParent();
				xTable.Clear();
				DropImpl();
				String aQualifiedName = QualifiedName();
				pDatabase->RemoveStorage(aQualifiedName,dbTable);
			}
		}
	}
	return bResult;
}
//------------------------------------------------------------------
// das kann man alles mit einer View nicht machen
//------------------------------------------------------------------
BOOL SdbView::CreateImpl(){return TRUE;}
BOOL SdbView::AlterImpl() {aStatus.SetSequenceError();return FALSE;}
BOOL SdbView::RenameImpl(const String& rNewName) {aStatus.SetSequenceError();return FALSE;}
BOOL SdbView::OpenImpl()  {aStatus.SetSequenceError();return FALSE;}
SdbIndex* SdbView::CreateIndexImpl() {aStatus.SetSequenceError();return NULL;}
void SdbView::FillIndexNames(SvStringsDtor& _rList){}

