/*************************************************************************
 *
 *  $RCSfile: sdbwork.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbwork.hxx"
#include "sdbresid.hrc"

#ifndef _SDB_SDBENV_HXX
#include "sdbenv.hxx"
#endif

#ifndef _SDB_SDBDB_HXX
#include "sdbdb.hxx"
#endif

#ifndef _SDB_SDBDBIMP_HXX
#include "sdbdbimp.hxx"
#endif

#ifndef _URLOBJ_HXX //autogen
#include <tools/urlobj.hxx>
#endif

#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#define SDB_DATABASE_MARK "explorer/database/"

TYPEINIT1(SdbWorkspace, SdbObj);
DBG_NAME(SdbWorkspace);

//------------------------------------------------------------------------------
SdbWorkspace::SdbWorkspace(SdbEnvironment* pEnv, const String& rName)
			 :SdbObj(pEnv,FALSE)
			 ,aName(rName)
{
	bOpen = TRUE;	// benoetigt kein Open
	DBG_CTOR(SdbWorkspace,NULL);
}

//------------------------------------------------------------------------------
SdbWorkspace::~SdbWorkspace()
{
	DBG_DTOR(SdbWorkspace,NULL);
}

//------------------------------------------------------------------
void SdbWorkspace::RemoveChildren()
// freigeben aller Childs, siehe SdbObj
{
	while (aDBList.Count())
		RemoveChild(aDBList.GetObject(0));
}

//------------------------------------------------------------------
void SdbWorkspace::RemoveChild(SdbObj* pObj)
{
	if (pObj->IsA(TYPE(SdbDatabase)))
		aDBList.Remove(PTR_CAST(SdbDatabase,pObj));
	else
		return;

	SdbObj::RemoveChild(pObj);
}

//------------------------------------------------------------------
SdbEnvironment* SdbWorkspace::GetEnvironment() const
	{return PTR_CAST(SdbEnvironment,GetParent());}

//------------------------------------------------------------------
BOOL SdbWorkspace::Rename(const String& rNewName)
{
	SDB_CHK_OPEN_BOOL();

	// Hat sich der Name geaendert
	if (rNewName == aName)
		return TRUE;

	SdbEnvironment* pEnvironment = GetEnvironment();
	if (pEnvironment->OpenWorkspace(rNewName))
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,WORKSPACE,rNewName);
		return FALSE;
	}

	String rOldName = aName;
	aName = rNewName;
	pEnvironment->WorkspaceRenamed(this, aName);	
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
String SdbWorkspace::GetFileName(const String& rFile) const
{
	// Auswerten des erhaltenen filenames fuer eine datenbank
	DirEntry aEntry(rFile);
	if (!aEntry.GetExtension().Len())
		aEntry.SetExtension(String::CreateFromAscii(SDB_DATABASE_EXT));
	if (aEntry.Exists())
		return aEntry.GetFull();
	else 
	{
		INetURLObject aUrl;
		aUrl.SetSmartProtocol(INET_PROT_FILE);
		aUrl.SetURL(rFile);
		if (aUrl.HasError()) // kann nicht ausgewertet werden
			return rFile;
		else
			return aUrl.PathToFileName();		
	}
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::CreateDatabase(const String& rName, const String& rTitle, const String& rConnectStr)
{
	// erzeugen einer Datenbank, diese wird nicht in der Liste aufgenommen
	// da sie nicht geoeffnet wird
	aStatus.Clear();
	SdbDatabase* pDB = CreateDB();

	SdbDatabaseGuard aGuard(pDB);
	BOOL bResult = pDB->Create(GetFileName(rName), rTitle, rConnectStr);
	if (!bResult || (rConnectStr.Len() && !pDB->IsOpen()))
		aStatus = pDB->Status();
	pDB->RemoveFromParent();

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::CreateNativeDatabase(const String& rOptionString) const
{
	SdbConnectionRef xConnection =
		SdbConnectionRef(GetEnvironment()->CreateConnection(rOptionString));
	return xConnection->Create();
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::DropNativeDatabase(const String& rOptionString) const
{
	SdbConnectionRef xConnection =
		SdbConnectionRef(GetEnvironment()->CreateConnection(rOptionString));
	return xConnection->Drop();
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::DropDatabase(const String& _rName)
{
	aStatus.Clear();
	SdbDatabase* pDB = GetDatabase(_rName);

	if (pDB)
	{
		SdbDatabaseGuard aGuard(pDB);
		BOOL bKnowsDB = pDB->IsOpen();
		if (!pDB->Drop())
			aStatus = pDB->Status();

		if (!bKnowsDB)
			pDB->RemoveFromParent();
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
SdbDatabase* SdbWorkspace::LoadDatabase(const String& rName)
{
	// is the database already loaded, but not opened?
	SdbDatabase* pReturn = aDBList.Lookup(DirEntry(GetFileName(rName)));
	if (pReturn)
	{
		// databse exists and is already opened
		if (pReturn->IsOpen())
			pReturn = NULL;		
	}
	else 
	{
		pReturn = GetDatabase(rName);	// die wird automatisch geladen
		if (pReturn)
			aDBList.InsertAt(DirEntry(pReturn->Name()), pReturn);		
	}
	return pReturn;
}

//------------------------------------------------------------------------------
SdbDatabase* SdbWorkspace::OpenDatabase(const String& rName)
{
	// is the database already loaded, but not opened?
	SdbDatabase* pDB = aDBList.Lookup(DirEntry(GetFileName(rName)));
	BOOL bLoadedBefore = pDB && !pDB->IsOpen();
	
	pDB = GetDatabase(rName);
	if (!pDB)
		return pDB;

	SdbDatabaseGuard aGuard(pDB);
	if (!pDB->IsOpen())
	{
		if (!pDB->Open())
		{
			aStatus = pDB->Status();

			if (!bLoadedBefore)
			{
				if (aDBList.GetPos(pDB) != SDB_CONT_ENTRY_NOTFOUND)
					aDBList.Remove(pDB);
				pDB->RemoveFromParent(); // after this call pDB is no more longer valid
			}
			pDB = NULL;
		}
		else
		{
			DirEntry aKey(pDB->Name());
			if (aDBList.Lookup(aKey) == NULL)
				aDBList.InsertAt(aKey, pDB);
			// (maybe somebody did a LoadDatabase and then an OpenDatabase, in this case we don't have to insert it again ....
		}
	}
	return pDB;
}

//------------------------------------------------------------------------------
SdbDatabase* SdbWorkspace::GetDatabaseNoOpen(const String& rName)
{
	String aFileName = GetFileName(rName);
	DirEntry aEntry(aFileName);
	return aDBList.Lookup(aEntry);
}

//------------------------------------------------------------------------------
SdbDatabase* SdbWorkspace::GetDatabase(const String& rName)
{
	aStatus.Clear();
	String aFileName = GetFileName(rName);
	DirEntry aEntry(aFileName);
	SdbDatabase* pDB = aDBList.Lookup(aEntry);
	if (!pDB)
	{
		if (!aEntry.Exists())
		{
			String aText = SdbResId(STR_STAT_FILE_NOT_FOUND);
			aText.SearchAndReplace(String::CreateFromAscii("%%d"),rName);
			aText.SearchAndReplace(String::CreateFromAscii("%%t"),SdbResId(STR_STAT_DATABASE));			
			aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("S0002"),
					aStatus.CreateErrorMessage(aText),
					0, String() );			
			return NULL;
		}
		
		pDB = CreateDB();
		// kann die Datenbankdatei ausgelesen werden
		if (!pDB->Load(aFileName))
		{
			aStatus = pDB->Status();
			pDB->RemoveFromParent();
			pDB = NULL;
		}
	}
	return pDB;
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::AlterDatabase(const String& rName, const String& rConnectStr)
{
	// Aendert eine bestehende Datenbankverbindung
	// Nach der Aenderung muss die Datenbank eine neue Verbindung aufbauen
	aStatus.Clear();
	SdbDatabase* pDB = GetDatabase(rName);
	if (pDB)
	{
		SdbDatabaseGuard aGuard(pDB);
		BOOL bKnowsDB = pDB->IsOpen();
		pDB->AlterConnection(rConnectStr);
		aStatus = pDB->Status();
		if(!bKnowsDB)
			pDB->RemoveFromParent();
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
String SdbWorkspace::GetDatabaseConnectStr(const String& rName)
{
	// Auslesen einer bestehenden Verbindung
	aStatus.Clear();
	String aConnectStr;

	SdbDatabase* pDB = GetDatabase(rName);
	if (pDB)
	{
		SdbDatabaseGuard aGuard(pDB);
		aConnectStr = pDB->ConnectStr();
		BOOL bKnowsDB = pDB->IsOpen();
		if(!bKnowsDB)
			pDB->RemoveFromParent();
	}

	return aConnectStr;
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::AlterDatabaseName(const String& _rOldName, const String& _rNewName)
{
	// Aendert eine bestehende Datenbankverbindung
	// Nach der Aenderung ist die Datenbank obsolet und sollte wieder geoeffnet werden
	// obsolete Datenbank wird aber nicht zerstoert
	aStatus.Clear();
	SdbDatabase* pDB = GetDatabase(_rOldName);
	if (pDB)
	{
		SdbDatabaseGuard aGuard(pDB);
		String aOldFileName = pDB->Name();
		String aNewFileName = GetFileName(_rNewName);
		pDB->Rename(aNewFileName);
		aStatus = pDB->Status();
		// wenn die Datenbank geoeffnet ist, mu sie unter dem neuen Namen
		// in der Liste gefuehrt werden.
		if (aStatus.IsSuccessful() && pDB->IsOpen())
		{
			aDBList.ReplaceAt(DirEntry(aOldFileName), DirEntry(aNewFileName), &pDB);
		}else
			pDB->RemoveFromParent();
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
BOOL SdbWorkspace::AlterDatabaseTitle(const String& rName, const String& rTitle)
{
	aStatus.Clear();
	SdbDatabase* pDB = GetDatabase(rName);
	if (pDB)
	{
		SdbDatabaseGuard aGuard(pDB);
		BOOL bKnowsDB = pDB->IsOpen();
		pDB->AlterTitle(rTitle);
		aStatus = pDB->Status();
		if(!bKnowsDB)
			pDB->RemoveFromParent();
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------------
SdbDatabase* SdbWorkspace::CreateDB()
{
	SdbEnvironment* pParent = GetEnvironment();

	if (pParent)
	{
		SdbDatabase* pDB = (SdbDatabase*)pParent->m_aDBCreateLink.Call(this);
		return pDB ? pDB : new SdbDatabase(this);
	}
	else
		return NULL;
}

//------------------------------------------------------------------
BOOL SdbWorkspace::Open()
{
	bOpen = TRUE;
	return TRUE;
}

