/*************************************************************************
 *
 *  $RCSfile: typeconv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <typeconv.hxx>			  

#ifndef _COM_SUN_STAR_SCRIPT_XTYPECONVERTER_HPP_
#include <com/sun/star/script/XTypeConverter.hpp>
#endif

#ifndef _COM_SUN_STAR_SCRIPT_FAILREASON_HPP_
#include <com/sun/star/script/FailReason.hpp>
#endif


#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

namespace staruno = ::com::sun::star::uno;
namespace starscript = ::com::sun::star::script;

::staruno::Reference< ::starscript::XTypeConverter >  ODbTypeConversion::xTypeConverter;

//--------------------------------------------------------------------------------------------------
sal_Bool ODbTypeConversion::toBOOL(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	sal_Bool bRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:
			bRes = rValue.getBOOL();
			break;		
		case ::staruno::TypeClass_CHAR:
			bRes = sal_Bool(rValue.getChar());
			break;
		case ::staruno::TypeClass_BYTE:
			bRes = sal_Bool(rValue.getBYTE());
			break;
		case ::staruno::TypeClass_SHORT:
			bRes = sal_Bool(rValue.getINT16());
			break;				
		case ::staruno::TypeClass_LONG:
			bRes = sal_Bool(rValue.getINT32());
			break;					
		case ::staruno::TypeClass_FLOAT:
			bRes = sal_Bool(rValue.getFloat() != 0);
			break;
		case ::staruno::TypeClass_DOUBLE:
			bRes = sal_Bool(rValue.getDouble() != 0);
			break;		
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
					 aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_BOOLEAN) >>= bRes;			
			break;		
		default:
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return bRes;
}

//--------------------------------------------------------------------------------------------------
sal_Int8 ODbTypeConversion::toBYTE(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	sal_Int8 nRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	
	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:
			nRes = rValue.getBOOL();
			break;		
		case ::staruno::TypeClass_CHAR:
			nRes = sal_Int8(rValue.getChar());
			break;
		case ::staruno::TypeClass_BYTE:
			nRes = rValue.getBYTE();
			break;
		case ::staruno::TypeClass_SHORT:
			nRes = sal_Int8(rValue.getINT16());
			break;				
		case ::staruno::TypeClass_LONG:
			nRes = sal_Int8(rValue.getINT32());
			break;					
		case ::staruno::TypeClass_FLOAT:
			nRes = sal_Int8(rValue.getFloat());
			break;
		case ::staruno::TypeClass_DOUBLE:
			nRes = sal_Int8(rValue.getDouble());
			break;		
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_BYTE) >>= nRes;
			break;		
		default:
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return nRes;
}

//--------------------------------------------------------------------------------------------------
sal_Int16 ODbTypeConversion::toINT16(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	sal_Int16 nRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:
			nRes = rValue.getBOOL();
			break;		
		case ::staruno::TypeClass_CHAR:
			nRes = rValue.getChar();
			break;
		case ::staruno::TypeClass_BYTE:
			nRes = rValue.getBYTE();
			break;
		case ::staruno::TypeClass_SHORT:
			nRes = rValue.getINT16();
			break;				
		case ::staruno::TypeClass_LONG:
			nRes = sal_Int16(rValue.getINT32());
			break;					
		case ::staruno::TypeClass_FLOAT:
			nRes = sal_Int16(rValue.getFloat());
			break;
		case ::staruno::TypeClass_DOUBLE:
			nRes = sal_Int16(rValue.getDouble());
			break;		
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_SHORT) >>= nRes;
			break;		
		default:
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return nRes;
}

//--------------------------------------------------------------------------------------------------
sal_Int32 ODbTypeConversion::toINT32(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	sal_Int32 nRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:
			nRes = rValue.getBOOL();
			break;		
		case ::staruno::TypeClass_CHAR:
			nRes = rValue.getChar();
			break;
		case ::staruno::TypeClass_BYTE:
			nRes = rValue.getBYTE();
			break;
		case ::staruno::TypeClass_SHORT:
			nRes = rValue.getINT16();
			break;				
		case ::staruno::TypeClass_LONG:
			nRes = rValue.getINT32();
			break;					
		case ::staruno::TypeClass_FLOAT:
			nRes = sal_Int32(rValue.getFloat());
			break;
		case ::staruno::TypeClass_DOUBLE:
			nRes = sal_Int32(rValue.getDouble());
			break;		
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_LONG) >>= nRes;
			break;		
		default:
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return nRes;
}

//--------------------------------------------------------------------------------------------------
sal_Int64 ODbTypeConversion::toINT64(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	sal_Int64 nRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:			
		case ::staruno::TypeClass_CHAR:			
		case ::staruno::TypeClass_BYTE:			
		case ::staruno::TypeClass_SHORT:			
		case ::staruno::TypeClass_LONG:			
		case ::staruno::TypeClass_FLOAT:			
		case ::staruno::TypeClass_DOUBLE:		
		case ::staruno::TypeClass_HYPER:
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_HYPER) >>= nRes;
			break;		
		default:
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return nRes;
}

//--------------------------------------------------------------------------------------------------
float ODbTypeConversion::toFloat(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	float nRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:
			nRes = rValue.getBOOL();
			break;		
		case ::staruno::TypeClass_CHAR:
			nRes = rValue.getChar();
			break;
		case ::staruno::TypeClass_BYTE:
			nRes = rValue.getBYTE();
			break;
		case ::staruno::TypeClass_SHORT:
			nRes = rValue.getINT16();
			break;				
		case ::staruno::TypeClass_LONG:
			nRes = rValue.getINT32();
			break;					
		case ::staruno::TypeClass_FLOAT:
			nRes = rValue.getFloat();
			break;
		case ::staruno::TypeClass_DOUBLE:
			nRes = float(rValue.getDouble());
			break;		
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_FLOAT) >>= nRes;
			break;		
		default:
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return nRes;
}

//--------------------------------------------------------------------------------------------------
double ODbTypeConversion::toDouble(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	double nRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:
			nRes = rValue.getBOOL();
			break;		
		case ::staruno::TypeClass_CHAR:
			nRes = rValue.getChar();
			break;
		case ::staruno::TypeClass_BYTE:
			nRes = rValue.getBYTE();
			break;
		case ::staruno::TypeClass_SHORT:
			nRes = rValue.getINT16();
			break;				
		case ::staruno::TypeClass_LONG:
			nRes = rValue.getINT32();
			break;					
		case ::staruno::TypeClass_FLOAT:
			nRes = rValue.getFloat();
			break;
		case ::staruno::TypeClass_DOUBLE:
			nRes = rValue.getDouble();
			break;		
		case ::staruno::TypeClass_STRING:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			if (!SdbTools::ToDateTime(nRes, rValue.getString()))
				xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_DOUBLE) >>= nRes;
			break;		
		case ::staruno::TypeClass_STRUCT:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Date*)0))
				nRes = SdbTools::ToDouble(rValue.getDate());
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::DateTime*)0))
				nRes = SdbTools::ToDouble(rValue.getDateTime());
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Time*)0))
				nRes = SdbTools::ToDouble(rValue.getTime());
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		default:					   
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return nRes;
}

//--------------------------------------------------------------------------------------------------
rtl::OUString ODbTypeConversion::toString(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	rtl::OUString aRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_BOOLEAN:		
		case ::staruno::TypeClass_CHAR:			
		case ::staruno::TypeClass_BYTE:			
		case ::staruno::TypeClass_SHORT:			
		case ::staruno::TypeClass_LONG:			
		case ::staruno::TypeClass_FLOAT:			
		case ::staruno::TypeClass_DOUBLE:
			if (!xTypeConverter.is())
			{
				 throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("stardiv.script.Converter!"), ::staruno::Reference< ::staruno::XInterface > (),
											    aDestinationClass, ::starscript::FailReason::UNKNOWN, 0 ) );
			}
			xTypeConverter->convertToSimpleType(rValue.getAny(), ::staruno::TypeClass_STRING) >>= aRes;
			break;		
		case ::staruno::TypeClass_STRING:
			aRes = rValue.getString();			
			break;		
		case ::staruno::TypeClass_STRUCT:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Date*)0))
				aRes = SdbTools::ToDateString(rValue.getDate());
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::DateTime*)0))
				aRes = SdbTools::ToDateTimeString(rValue.getDateTime());
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Time*)0))
				aRes = SdbTools::ToTimeString(rValue.getTime());
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		default:					   
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );

	}
	return aRes;
}

//--------------------------------------------------------------------------------------------------
Date ODbTypeConversion::toDate(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	Date aRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_DOUBLE:
			aRes = SdbTools::ToDate(rValue.getDouble());			
			break;				
		case ::staruno::TypeClass_STRING:
		{
			double fResult;
			sal_Bool bRes = SdbTools::ToDateTime(fResult, rValue.getString());
			if (bRes)
			{
				aRes = SdbTools::ToDate(fResult);
			}
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
		}	break;
		case ::staruno::TypeClass_STRUCT:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Date*)0))
				aRes = rValue.getDate();
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::DateTime*)0))
				aRes = rValue.getDateTime();			
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		default:					   
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
	}
	return aRes;
}

//--------------------------------------------------------------------------------------------------
DateTime ODbTypeConversion::toDateTime(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	DateTime aRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_DOUBLE:
			aRes = SdbTools::ToDateTime(rValue.getDouble());			
			break;			
		case ::staruno::TypeClass_STRING:
		{
			double fResult;
			sal_Bool bRes = SdbTools::ToDateTime(fResult, rValue.getString());
			if (bRes)
				aRes = SdbTools::ToDateTime(fResult);			
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
		}	break;
		case ::staruno::TypeClass_STRUCT:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Date*)0))
				aRes = rValue.getDate();
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::DateTime*)0))
				aRes = rValue.getDateTime();			
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Time*)0))
				aRes = rValue.getTime();						
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		default:					   
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
	}
	return aRes;
}

//--------------------------------------------------------------------------------------------------
Time ODbTypeConversion::toTime(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	DateTime aRes;
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_DOUBLE:
			aRes = SdbTools::ToDateTime(rValue.getDouble());			
			break;
		case ::staruno::TypeClass_STRING:
		{
			double fResult;
			sal_Bool bRes = SdbTools::ToDateTime(fResult, rValue.getString());
			if (bRes)
			{
				aRes = SdbTools::ToTime(fResult);
			}
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
		}	break;
		case ::staruno::TypeClass_STRUCT:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::DateTime*)0))
				aRes = rValue.getDateTime();			
			else if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Time*)0))
				aRes = rValue.getTime();						
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		default:					   
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
	}
	return aRes;
}

//--------------------------------------------------------------------------------------------------
::staruno::Sequence< sal_Int8 > ODbTypeConversion::toBytes(const ODbVariant& rValue) throw( ::starscript::CannotConvertException )
{
	::staruno::TypeClass aDestinationClass = rValue.getValueType().getTypeClass();	

	switch (aDestinationClass)
	{
		case ::staruno::TypeClass_VOID:
			return ::staruno::Sequence< sal_Int8 >();
		case ::staruno::TypeClass_BOOLEAN:
		{
			sal_Bool bRes = rValue.getBOOL();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&bRes, sizeof(sal_Bool));
		}
		case ::staruno::TypeClass_CHAR:
		{
			sal_Unicode aRes = rValue.getChar();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&aRes, sizeof(sal_Unicode));
		}
		case ::staruno::TypeClass_BYTE:
		{
			sal_Int8 aRes = rValue.getBYTE();
			return ::staruno::Sequence< sal_Int8 >(&aRes, sizeof(sal_Int8));
		}				
		case ::staruno::TypeClass_SHORT:
		{
			sal_Int16 aRes = rValue.getINT16();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&aRes, sizeof(sal_Int16));
		}						
		case ::staruno::TypeClass_LONG:
		{
			sal_Int32 aRes = rValue.getINT32();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&aRes, sizeof(sal_Int32));
		}								
		case ::staruno::TypeClass_FLOAT:
		{
			float aRes = rValue.getFloat();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&aRes, sizeof(float));
		}	
		case ::staruno::TypeClass_DOUBLE:
		{
			double aRes = rValue.getDouble();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&aRes, sizeof(double));
		}	
		case ::staruno::TypeClass_HYPER:
		{
			sal_Int64 aRes = rValue.getINT64();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)&aRes, sizeof(sal_Int64));
		}	
		case ::staruno::TypeClass_STRING:
		{
			rtl::OUString aRes = rValue.getString();
			return ::staruno::Sequence< sal_Int8 >((sal_Int8*)aRes.getStr(), aRes.getLength() * sizeof(sal_Unicode));
		}	
		case ::staruno::TypeClass_STRUCT:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Date*)0) ||
				rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::DateTime*)0) ||
				rValue.getValueType() == ::getCppuType((const ::com::sun::star::util::Time*)0))
			{
				rtl::OUString aRes = toString(rValue);
				return ::staruno::Sequence< sal_Int8 >((sal_Int8*)aRes.getStr(), aRes.getLength() * sizeof(sal_Unicode));
			}
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		case ::staruno::TypeClass_SEQUENCE:
			if (rValue.getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
				return rValue.getBytes();	
			else
				throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
			break;
		default:					   
			throw( ::starscript::CannotConvertException( ::rtl::OUString::createFromAscii("TYPE is not supported!"), ::staruno::Reference< ::staruno::XInterface > (),
									   aDestinationClass, ::starscript::FailReason::TYPE_NOT_SUPPORTED, 0 ) );
	}
	return ::staruno::Sequence< sal_Int8 >();
}



