/*************************************************************************
 *
 *  $RCSfile: variant.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:17:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <variant.hxx>			  
#include <memory.h>			  

#ifndef _SDB_TYPECONV_HXX
#include <typeconv.hxx>
#endif

namespace staruno = ::com::sun::star::uno;
namespace starutil = ::com::sun::star::util;

//-----------------------------------------------------------------------------
void OVariant::set(const void* pValue, const ::staruno::Type& rType)
{
	m_aData.setValue(pValue, rType);
}

//-----------------------------------------------------------------------------
void ODbVariant::set(const void* pValue, const ::staruno::Type& rType)
{
	OVariant::set(pValue, rType);
	m_bModified = sal_True;
}

//-----------------------------------------------------------------------------
sal_Bool areEqual(const staruno::Any& rLeft, const staruno::Any& rRight)
{
	sal_Bool bRes(sal_False);
	if (rLeft.getValueTypeClass() == rRight.getValueTypeClass())
	{
		switch (rLeft.getValueTypeClass())
		{
			case ::com::sun::star::uno::TypeClass_VOID:
				bRes = sal_True;
				break;
			case ::com::sun::star::uno::TypeClass_BOOLEAN:
				bRes = *(sal_Bool *)rLeft.getValue() == *(sal_Bool *)rRight.getValue(); break;
			case ::com::sun::star::uno::TypeClass_CHAR:
				bRes = *(sal_Unicode *)rLeft.getValue() == *(sal_Unicode *)rRight.getValue(); break;
			case ::com::sun::star::uno::TypeClass_BYTE:
				bRes = *(sal_Int8 *)rLeft.getValue() == *(sal_Int8 *)rRight.getValue(); break;
			case ::com::sun::star::uno::TypeClass_STRING:
				bRes = ((rtl::OUString*)rLeft.getValue())->equals(*(rtl::OUString *)rRight.getValue()); break;
			case ::com::sun::star::uno::TypeClass_FLOAT:
				bRes = *(float *)rLeft.getValue() == *(float *)rRight.getValue(); break;
			case ::com::sun::star::uno::TypeClass_DOUBLE:
				bRes = *(double *)rLeft.getValue() == *(double *)rRight.getValue(); break;		
			case ::com::sun::star::uno::TypeClass_SHORT:
				bRes = *(sal_Int16 *)rLeft.getValue() == *(sal_Int16 *)rRight.getValue(); break;		
			case ::com::sun::star::uno::TypeClass_LONG:
				bRes = *(sal_Int32 *)rLeft.getValue() == *(sal_Int32 *)rRight.getValue(); break;		
			case ::com::sun::star::uno::TypeClass_SEQUENCE:
				if (rLeft.getValueType() == ::getCppuType((const ::staruno::Sequence< sal_Int8 > *)0))
				{
					const ::staruno::Sequence<sal_Int8>& rLeftSeq = * (::staruno::Sequence< sal_Int8 > *)rLeft.getValue();
					const ::staruno::Sequence<sal_Int8>& rRightSeq = * (::staruno::Sequence< sal_Int8 > *)rRight.getValue();
					bRes =  rLeftSeq.getLength() == rRightSeq.getLength() && 
							memcmp(rLeftSeq.getConstArray(), rRightSeq.getConstArray(),rLeftSeq.getLength()) == 0;
				}
				break;						
			default:
				bRes = rLeft == rRight;
		}
	}
	return bRes;
}

// Typeconversion
//-----------------------------------------------------------------------------
sal_Bool	ODbVariant::toBOOL() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toBOOL(*this);}
sal_Int8	ODbVariant::toBYTE() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toBYTE(*this);}
sal_Int16	ODbVariant::toINT16() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toINT16(*this);}
sal_Int32	ODbVariant::toINT32()  const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toINT32(*this);}
sal_Int64	ODbVariant::toINT64() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toINT64(*this);}
float	ODbVariant::toFloat() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toFloat(*this);}
double	ODbVariant::toDouble() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toDouble(*this);}
rtl::OUString ODbVariant::toString() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toString(*this);}
Date	ODbVariant::toDate() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toDate(*this);}
Time	ODbVariant::toTime() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toTime(*this);}
DateTime ODbVariant::toDateTime() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toDateTime(*this);}
::staruno::Sequence < sal_Int8 > ODbVariant::toBytes() const throw( ::com::sun::star::script::CannotConvertException )
	{return ODbTypeConversion::toBytes(*this);}



