/*************************************************************************
 *
 *  $RCSfile: dbfconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_DBFCONN_HXX
#include "dbfconn.hxx"
#endif
#ifndef _SDB_DBFTAB_HXX
#include "dbftab.hxx"
#endif

#ifndef _SDB_DBFMD_HXX
#include <dbfmd.hxx>
#endif // _SDB_ODBCMD_HXX


#ifndef _SDB_SDBRESID_HRC
#include "sdbresid.hrc"
#endif

DBG_NAME(SdbDBFConnection);

static const char * const pFilenameExtension = "dbf";

//------------------------------------------------------------------
SdbDBFConnection::SdbDBFConnection(SdbEnvironment* pEnv, const String & rNewOptions)
				 :SdbFILEConnection(pEnv, rNewOptions, pFilenameExtension)
{
	DBG_CTOR(SdbDBFConnection,NULL);
	aStatus.SetComponent(String::CreateFromAscii("Sdb DBF Driver"));
}
//------------------------------------------------------------------
SdbDBFConnection::~SdbDBFConnection()
{
	DBG_DTOR(SdbDBFConnection,NULL);
}
//------------------------------------------------------------------
BOOL SdbDBFConnection::OpenImpl()
{
	DBG_CHKTHIS(SdbDBFConnection,NULL);

	// DBF-Dateien sind haeufig im OEM-Zeichensatz.
	// Wenn Zeichensatz nicht explizit angegeben, 437 benutzen:
	if (eCharacterSet == RTL_TEXTENCODING_DONTKNOW)
		// CHARSET_DONTKNOW heisst: ist nicht explizit gesetzt worden.
		eCharacterSet = RTL_TEXTENCODING_IBM_437;

	BOOL bTempOpen = SdbFILEConnection::OpenImpl();

	if (bTempOpen)
	{
		BOOL bCompress = aOptionsParser.OptionBool(String::CreateFromAscii("COMPRESS"));
		if (bCompress)
		{
			// open auf true, damit Compress funktioniert
			bOpen = bTempOpen;
			Compress();
			bOpen = FALSE;
		}
	}
	return bTempOpen;
}

//------------------------------------------------------------------
SdbTable* SdbDBFConnection::CreateTableImpl()
{
	DBG_CHKTHIS(SdbDBFConnection,NULL);

	return new SdbDBFTable(this);
}

//------------------------------------------------------------------
void SdbDBFConnection::FillTypeList(SdbTypeInfoList& rList)
{
	SdbTypeInfo* pInfo		= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_CHAR;
	pInfo->aTypeName		= String::CreateFromAscii("CHAR");
	pInfo->nPrecision		= 254;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_VARCHAR;
	pInfo->aTypeName		= String::CreateFromAscii("VARCHAR");
	pInfo->nPrecision		= 254;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_LONGVARCHAR;
	pInfo->aTypeName		= String::CreateFromAscii("LONGVARCHAR");
	pInfo->nPrecision		= 65535;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	pInfo->eSearchType		= Search_Like_Only;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_BOOLEAN;
	pInfo->aTypeName		= String::CreateFromAscii("BOOL");
	pInfo->nPrecision		= 1;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DATE;
	pInfo->aTypeName		= String::CreateFromAscii("DATE");
	pInfo->nPrecision		= 10;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DECIMAL;
	pInfo->aTypeName		= String::CreateFromAscii("DECIMAL");
	pInfo->nPrecision	= 20;
	pInfo->nScale		= 15;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_NUMERIC;
	pInfo->aTypeName		= String::CreateFromAscii("NUMERIC");
	pInfo->nPrecision	= 20;
	pInfo->nScale		= 20;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);
}

//------------------------------------------------------------------
BOOL SdbDBFConnection::IsCapableImpl(SdbCapability eCapability) const
{
	DBG_CHKTHIS(SdbDBFConnection,NULL);
	BOOL bResult;
	switch (eCapability)
	{
		case SDB_CAP_STATIC_PRIOR:
		case SDB_CAP_KEYSET_PRIOR:
		case SDB_CAP_CREATE_TABLE:
		case SDB_CAP_ALTER_TABLE:
		case SDB_CAP_CREATE_INDEX:
		case SDB_CAP_UPDATABLE:
		case SDB_CAP_COMMIT_AUTO:
		case SDB_CAP_COMPRESS:
		case SDB_CAP_READ_UNCOMMITTED:	bResult = TRUE; break;
		case SDB_CAP_BULKOPTIMIZE:
			bResult = FALSE; break;
		case SDB_CAP_INSERTTABCOLS:
			bResult = TRUE; break;
		default:
			bResult = SdbFILEConnection::IsCapableImpl(eCapability);
	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbDBFConnection::DoCmdImpl(UINT16 nLimit,const String &rArgs)
{
	BOOL bResult(FALSE);
	switch(nLimit){
		case SDB_CAP_COMPRESS:
			Compress();
			bResult = aStatus.IsSuccessful();
			break;
	}
	return bResult;
}
//------------------------------------------------------------------
void SdbDBFConnection::Compress()
{
	BeginHdl().Call(this);

	String aWarning;
	USHORT nCount = GetTableCount(),
		   i	  = 0;
	while (i < nCount)
	{
		String aTableName = GetTableName(i++);
		SdbDBFTable* pTable = (SdbDBFTable*) OpenTable(aTableName);
		if (pTable)
			pTable->AddRef();

		if (!pTable || !pTable->Compress())
		{
			aWarning += aTableName;
			aWarning.AppendAscii(";");
		}

		if (pTable)
			pTable->ReleaseRef();
	}
	aWarning.EraseTrailingChars(';');
	if (aWarning.Len())
	{
		String aText = String(SdbResId(STR_STAT_COMPRESS_WARNING));
		aText.SearchAndReplace(String::CreateFromAscii("#"),aWarning);
		aStatus.Set(SDB_STAT_SUCCESS_WITH_INFO,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
	}
	EndHdl().Call(this);
}
//-----------------------------------------------------------------------------
::com::sun::star::data::XDatabaseDescriptor* SdbDBFConnection::GetMetaDataImpl()
{
	SdbDBFMetaData* pMetaData = new SdbDBFMetaData(this);
	return pMetaData;
}

