/*************************************************************************
 *
 *  $RCSfile: dbfidx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _CONFIG_HXX //autogen
#include <vcl/config.hxx>
#endif

#include "sdbresid.hrc"

#include "dbfidx.hxx"
#include "dbftab.hxx"
#include "dbfconn.hxx"

#ifndef _SDB_FILESET_HXX
#include "fileset.hxx"
#endif

DBG_NAME(SdbNDXIndex);
//------------------------------------------------------------------
SdbNDXIndex::SdbNDXIndex(SdbDBFTable* pTable)
			:SdbFILEIndex(pTable)
			,bUseCollector(FALSE)
			,nPageCount(0)
			,nRootPage(0)
			,nCurNode(NODE_NOTFOUND)

{DBG_CTOR(SdbNDXIndex,NULL);}

//------------------------------------------------------------------
SdbNDXIndex::~SdbNDXIndex()
{
	DBG_DTOR(SdbNDXIndex,NULL);
	bUseCollector = FALSE;

	// alle Referenzen freigeben, bevor der FileStream geschlossen wird
	aCurLeaf.Clear();
	aRoot.Clear();

	for (ULONG i = 0; i < aCollector.Count(); i++)
		aCollector.GetObject(i)->QueryDelete();
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::DropImpl()
{
	if (aFileStream.IsOpen())
		aFileStream.Close();

	SdbDBFConnection* pConnection = GetDBFConnection();

	DirEntry aIndexEntry(pConnection->FileEntry(aName));
	aIndexEntry.SetExtension(String::CreateFromAscii("ndx"));

	ULONG nErrorCode = aIndexEntry.Kill();
	if (nErrorCode != SVSTREAM_OK && nErrorCode != SVSTREAM_FILE_NOT_FOUND)
	{
		aStatus.SetError(nErrorCode,INDEX,aName);
		return FALSE;
	}

	// InfDatei abgleichen
	String aNDX;
	DirEntry aEntry( pConnection->FileEntry(GetTable()->Name()));
	aEntry.SetExtension(String::CreateFromAscii("inf"));

	Config aInfFile(aEntry.GetFull());
	aInfFile.SetGroup(dBASE_III_GROUP);
	USHORT nKeyCnt = aInfFile.GetKeyCount();
	ByteString aKeyName;

	DirEntry aEntryToComp = pConnection->FileEntry(aName);
	aEntryToComp.SetExtension(String::CreateFromAscii("ndx"));

	for (USHORT nKey = 0; nKey < nKeyCnt; nKey++)
	{
		// Verweist der Key auf ein Indexfile?...
		aKeyName = aInfFile.GetKeyName( nKey );
		//...wenn ja, Indexliste der Tabelle hinzufuegen
		if (aEntry.IsCaseSensitive() ? aKeyName.Copy(0,3) == "NDX" : aKeyName.Copy(0,3).EqualsIgnoreCaseAscii("NDX"))
		{
			aEntryToComp.SetName(String(aInfFile.ReadKey(aKeyName),gsl_getSystemTextEncoding()));
			aEntryToComp.SetExtension(String::CreateFromAscii("ndx"));
			if (aEntryToComp == aIndexEntry)
			{
				aInfFile.DeleteKey(aKeyName);
				break;
			}
		}
	}
	SdbFILEColumn* pColumn = (SdbFILEColumn*)GetTable()->OriginalColumns()->Column(aColumns->Column(1)->GetName());
	if (pColumn && pColumn->IsIndexed())
	{
		pColumn->SetIndex(NULL);
		pColumn->ResetFlag(SDB_FLAGS_UNIQUE);
	}
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::CreateImpl()
{
	// Anlegen des Index
	SdbDBFConnection* pConnection = GetDBFConnection();
	DirEntry aEntry(pConnection->FileEntry(aName));
	aEntry.SetExtension(String::CreateFromAscii("ndx"));

	if (aEntry.Exists())
	{
		aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,INDEX,aEntry.GetFull());
		return FALSE;
	}

	// Index ist nur einstufig
	if (aColumns->Count() != 2)
	{
		aStatus.SetDriverNotCapableError();
		return FALSE;
	}

	// ist die Spalte schon indiziert ?
	const SdbFILEColumn* pColumn = (const SdbFILEColumn*)GetTable()->OriginalColumns()->Column(aColumns->Column(1)->GetName());
	if (!pColumn)
	{
		String aText = String(SdbResId(STR_STAT_INDEX_COLUMN_NOT_FOUND));
		aText.SearchAndReplace(String::CreateFromAscii("#"),pColumn->GetName());
		aText.SearchAndReplace(String::CreateFromAscii("%"),GetTable()->Name());
		aStatus.Set(SDB_STAT_ERROR,
				String::CreateFromAscii("01000"),
				aStatus.CreateErrorMessage(aText),
				0, String() );
		return FALSE;
	}
	else if (pColumn && pColumn->IsIndexed())
	{
		String aText = String(SdbResId(STR_STAT_INDEX_COLUMN_ALREADY_INDEXED));
		aText.SearchAndReplace(String::CreateFromAscii("#"),pColumn->GetName());
		aStatus.Set(SDB_STAT_ERROR,
				String::CreateFromAscii("01000"),
				aStatus.CreateErrorMessage(aText),
				0, String() );
		return FALSE;
	}

	// Anlegen des Indexfiles
	aFileStream.Open(aEntry.GetFull(), STREAM_READWRITE | STREAM_SHARE_DENYWRITE | STREAM_TRUNC);
	if (aStatus.SetError(aFileStream,INDEX))
		return FALSE;

	aFileStream.SetNumberFormatInt(NUMBERFORMAT_INT_LITTLEENDIAN);
	aFileStream.SetBufferSize(512);

	// Zunchst mu das Ergebnis sortiert sein
	SdbCursor* pCursor = pConnection->CreateCursor(SDB_KEYSET, SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();

	String aStatement;aStatement.AssignAscii("Select ");
	aStatement += pConnection->IdentifierQuotation(pColumn->GetName());
	aStatement.AppendAscii(" FROM ");
	aStatement += pConnection->IdentifierQuotation(GetTable()->Name());
	aStatement.AppendAscii(" ORDER BY ");
	aStatement += pConnection->IdentifierQuotation(pColumn->GetName());

	if (!IsUnique()) // zusaetzlich sortierung mit der bookmarkspalte
	{
		aStatement.AppendAscii(" ,");
		aStatement += pConnection->IdentifierQuotation(aColumns->Column(0)->GetName());
	}

	if (!pCursor->Open(aStatement))
	{
		aStatus = pCursor->Status();
		pCursor->ReleaseRef();
		aFileStream.Close();
		aEntry.Kill();

		return FALSE;
	}

	// Setzen der Headerinfo
	memset(&aHeader,0,sizeof(aHeader));
	aFileStream.SetStreamSize(512);

	aHeader.db_keytype = (pColumn->GetType() == SDB_DBTYPE_VARCHAR ||
						  pColumn->GetType() == SDB_DBTYPE_CHAR) ? 0 : 1;
	aHeader.db_keylen  = (aHeader.db_keytype) ? 8 : (USHORT)pColumn->GetLength();
	aHeader.db_maxkeys = (512 - 8) / (8 + aHeader.db_keylen);
	ByteString aCol(pColumn->GetName(),gsl_getSystemTextEncoding());
	strcpy(aHeader.db_name,aCol.GetBuffer());
	aHeader.db_unique  = SdbIndex::IsUnique() ? 1: 0;
	aHeader.db_keyrec  = aHeader.db_keylen + 8;

	// modifizierung am Header werden ueber Unterschiede zw. HeaderInfo und nRootPage
	// bzw. nPageCout erkannt

	nRootPage = 1;
	nPageCount = 2;

	SdbDatabaseType eType = aHeader.db_keytype == 0 ? SDB_DBTYPE_VARCHAR : SDB_DBTYPE_DOUBLE;
	aCurLeaf = aRoot = CreatePage(nRootPage);
	aRoot->SetModified(TRUE);

	bUseCollector = TRUE;

	ULONG nRowsLeft = pCursor->RowCount();

	// Erzeugen der Indexstruktur
	while (pCursor->IsInRange())
	{
		BUSY_DB();
		ODbRow& rRow = *pCursor->GetRow();
		// ueberpruefen auf doppelten eintrag
		if (IsUnique() && nCurNode != NODE_NOTFOUND)
		{
			SdbNDXKey  aKey(*rRow[1], eType, 0);
			if (aKey == (*aCurLeaf)[nCurNode].GetKey())
			{
				String aText = String(SdbResId(STR_STAT_INDEX_NOT_UNIQUE));
				aText.SearchAndReplace(String::CreateFromAscii("#"),aName);
				aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(aText),
						0, String() );
				break;
			}
		}
		SdbNDXKey  aKey(*rRow[1], eType, static_cast<SdbFILECursor*>(pCursor)->GetFileSet()->GetKey(rRow[0]->getINT32() - 1));
		SdbNDXNode aNewNode(aKey);
		if (!aCurLeaf->Insert(aNewNode, --nRowsLeft))
			break;

#ifdef DEBUG
		//DBG_TRACE1("SDB: %s", (const char*)pCursor->Variable(1)->GetString());
		//		PrintTree();
#endif
		pCursor->Next();
	}

	BOOL bResult = !pCursor->IsInRange();
	if (!bResult)
	{
		aFileStream.Close();
		aEntry.Kill();
		Release(FALSE);
	}
	else
	{

#ifdef DEBUG
		PrintTree();
#endif
		Release();
//		aFileStream.Close();
		// den FielStream NICHT schliessen, da per definitionem ein SdbObject nach dem Kreieren offen ist

		// inf Datei abgleichen
		CreateINFEntry();
	}

	pCursor->ReleaseRef();
	return bResult;
}

//------------------------------------------------------------------
void SdbNDXIndex::CreateINFEntry()
{
	// inf Datei abgleichen
	String aNDX;
	SdbDBFConnection* pConnection = GetDBFConnection();
	DirEntry aEntry(pConnection->FileEntry(aName));
	aEntry.SetExtension(String::CreateFromAscii("ndx"));

	DirEntry aInfEntry(pConnection->FileEntry(GetTable()->Name()));
	aInfEntry.SetExtension(String::CreateFromAscii("inf"));

	Config aInfFile(aInfEntry.GetFull());
	aInfFile.SetGroup(dBASE_III_GROUP);

	USHORT nSuffix = aInfFile.GetKeyCount();
	ByteString aNewEntry,aKeyName;
	BOOL bCase = (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;
	while (!aNewEntry.Len())
	{
		aNewEntry = "NDX";
		aNewEntry += ByteString::CreateFromInt32(++nSuffix);
		for (USHORT i = 0; i < aInfFile.GetKeyCount(); i++)
		{
			aKeyName = aInfFile.GetKeyName(i);
			if (bCase ? aKeyName == aNewEntry : aKeyName.EqualsIgnoreCaseAscii(aNewEntry))
			{
				aNewEntry.Erase();
				break;
			}
		}
	}
	aInfFile.WriteKey(aNewEntry,ByteString(aEntry.GetName(),gsl_getSystemTextEncoding()));
}


//------------------------------------------------------------------
BOOL SdbNDXIndex::Lock(BOOL bReadOnly)
{
	aStatus.Clear();
	SdbDBFConnection* pConnection = GetDBFConnection();
	DirEntry aEntry(pConnection->FileEntry(aName));
	aEntry.SetExtension(String::CreateFromAscii("ndx"));

	if (aFileStream.IsOpen())
		aFileStream.Close();

	Release(FALSE);

	if (!bReadOnly)
		aFileStream.Open(aEntry.GetFull(), STREAM_READWRITE | STREAM_NOCREATE| STREAM_SHARE_DENYALL);
	else
		aFileStream.Open(aEntry.GetFull(), STREAM_READ | STREAM_NOCREATE| STREAM_SHARE_DENYNONE);

	if (aFileStream.IsOpen())
	{
		aFileStream >> *this;
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbNDXIndex::UnLock()
{
	aFileStream.Close();
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::OpenImpl()
{
	// Anlegen des Index
	SdbDBFConnection* pConnection = GetDBFConnection();
	DirEntry aEntry(pConnection->FileEntry(aName));
	aEntry.SetExtension(String::CreateFromAscii("ndx"));

	// Anlegen des Indexfiles
	SdbFILETable* pParentTable = PTR_CAST(SdbFILETable, GetParent());
	if (pParentTable && !pParentTable->IsReadOnly() && (pParentTable->GetTablePrivileges() & (SDB_PR_INSERT | SDB_PR_UPDATE)))
		aFileStream.Open(aEntry.GetFull(), STREAM_READWRITE | STREAM_NOCREATE| STREAM_SHARE_DENYWRITE);
	else
		aFileStream.Open(aEntry.GetFull(), STREAM_READ | STREAM_NOCREATE| STREAM_SHARE_DENYWRITE);

	if (aStatus.SetError(aFileStream,INDEX))
		return FALSE;

	aFileStream.SetNumberFormatInt(NUMBERFORMAT_INT_LITTLEENDIAN);
	aFileStream.SetBufferSize(512);

	aFileStream >> *this;
	const SdbColumn* pColumn = GetTable()->OriginalColumns()->Column(String::CreateFromAscii(aHeader.db_name));

	if (pColumn)
	{
		aOriginalColumns->AddColumn(pConnection->CreateColumn(*pColumn));
		if (aHeader.db_unique)
			SetStyle(SDB_IDX_UNIQUE);
	}
	else
	{
		String aText = String(SdbResId(STR_STAT_INDEX_INVALID));
		aText.SearchAndReplace(String::CreateFromAscii("#"),aName);
		aStatus.Set(SDB_STAT_ERROR,
				String::CreateFromAscii("01000"),
				aStatus.CreateErrorMessage(aText),
				0, String() );
	}
#ifdef DEBUG
	PrintTree();
#endif

	return aStatus.IsSuccessful();
}


//------------------------------------------------------------------
void SdbNDXIndex::Close()
{
	aFileStream.Close();
	SdbFILEIndex::Close();
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::DoRefresh()
{
	if (!bNew)
		aFileStream.Close();
	return SdbFILEIndex::DoRefresh();
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::ConvertToKey(SdbNDXKey& rKey, UINT32 nRec, const ODbVariant& rValue)
{
	// Sucht ein bestimmten Wert im Index
	// Wenn der Index Unique ist, interssiert der Key nicht, sonst ja
	try
	{
		if (aHeader.db_keytype == 0)
		{
			rKey = SdbNDXKey(rValue.toString(), nRec );
		}
		else
		{
			if (rValue.isNull())
				rKey = SdbNDXKey(rValue, SDB_DBTYPE_DOUBLE, nRec );
			else
				rKey = SdbNDXKey(rValue.toDouble(), nRec );
		}
	}
	catch (...)
	{
		return FALSE;
	}
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::Find(UINT32 nRec, const ODbVariant& rValue)
{
	// Sucht ein bestimmten Wert im Index
	// Wenn der Index Unique ist, interssiert der Key nicht, sonst ja
	SdbNDXKey aKey;
	return ConvertToKey(aKey, nRec, rValue) && GetRoot()->Find(aKey);
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::Insert(UINT32 nRec, const ODbVariant& rValue)
{
	SdbNDXKey aKey;

	// Existiert der Wert bereits
	// Find immer verwenden um das aktuelle Blatt zu bestimmen
	if (!ConvertToKey(aKey, nRec, rValue) || (GetRoot()->Find(aKey) && IsUnique()))
		return FALSE;

	SdbNDXNode aNewNode(aKey);

	// einfuegen in das aktuelle Blatt
	if (!aCurLeaf.Is())
		return FALSE;

	BOOL bResult = aCurLeaf->Insert(aNewNode);
	Release(bResult);

#ifdef DEBUG
		PrintTree();
#endif

	return bResult;
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::Update(UINT32 nRec, const ODbVariant& rOldValue,
						 const ODbVariant& rNewValue)
{
	SdbNDXKey aKey;
	if (!ConvertToKey(aKey, nRec, rNewValue) || (IsUnique() && GetRoot()->Find(aKey)))
		return FALSE;
	else
		return Delete(nRec, rOldValue) && Insert(nRec,rNewValue);
}

//------------------------------------------------------------------
BOOL SdbNDXIndex::Delete(UINT32 nRec, const ODbVariant& rValue)
{
	// Existiert der Wert bereits
	// Find immer verwenden um das aktuelle Blatt zu bestimmen
	SdbNDXKey aKey;
	if (!ConvertToKey(aKey, nRec, rValue) || GetRoot()->Find(aKey))
		return FALSE;

	SdbNDXNode aNewNode(aKey);

	// einfuegen in das aktuelle Blatt
	if (!aCurLeaf.Is())
		return FALSE;
#if DEBUG
	aRoot->PrintPage();
#endif

	return aCurLeaf->Delete(nCurNode);
}

//------------------------------------------------------------------
void SdbNDXIndex::Collect(SdbNDXPage* pPage)
{
	if (pPage)
		aCollector.Insert(pPage, LIST_APPEND);
}

//------------------------------------------------------------------
void SdbNDXIndex::Release(BOOL bSave)
{
	// Freigeben der Indexressourcen
	bUseCollector = FALSE;

	if (aCurLeaf.Is())
	{
		aCurLeaf->Release(bSave);
		aCurLeaf.Clear();
	}

	// Wurzel freigeben
	if (aRoot.Is())
	{
		aRoot->Release(bSave);
		aRoot.Clear();
	}

	// alle Referenzen freigeben, bevor der FileStream geschlossen wird
	for (ULONG i = 0; i < aCollector.Count(); i++)
		aCollector.GetObject(i)->QueryDelete();

	aCollector.Clear();

	// Header modifiziert ?
	if (bSave && (aHeader.db_rootpage != nRootPage ||
		aHeader.db_pagecount != nPageCount))
	{
		aHeader.db_rootpage = nRootPage;
		aHeader.db_pagecount = nPageCount;
		aFileStream << *this;
	}
	nRootPage = nPageCount = 0;
	nCurNode = NODE_NOTFOUND;
}

//------------------------------------------------------------------
SdbNDXPage* SdbNDXIndex::CreatePage(UINT32 nPagePos, SdbNDXPage* pParent, BOOL bLoad)
{
	SdbNDXPage* pPage;
	if (aCollector.Count())
	{
		pPage = aCollector.Remove(aCollector.Count() - 1);
		pPage->SetPagePos(nPagePos);
		pPage->SetParent(pParent);
	}
	else
		pPage = new SdbNDXPage(*this, nPagePos, pParent);

	if (bLoad)
		aFileStream >> *pPage;

	return pPage;
}

//------------------------------------------------------------------
SdbFILEIndexIterator* SdbNDXIndex::CreateIterator(SdbBoolOperator* pOp,
												  const SdbOperand* pOperand)
{
	return new SdbNDXIterator(this, pOp, pOperand);
}

//------------------------------------------------------------------
SdbNDXPagePtr SdbNDXIndex::GetRoot()
{
	if (!aRoot.Is())
	{
		nRootPage = aHeader.db_rootpage;
		nPageCount = aHeader.db_pagecount;
		aRoot = CreatePage(nRootPage,NULL,TRUE);
	}
	return aRoot;
}

#ifdef DEBUG
//------------------------------------------------------------------
void SdbNDXIndex::PrintTree()
{
	return;
	DBG_TRACE("\nSDB: ------------------Index START ------------------");
	DBG_TRACE4("SDB: Header: Name: %s  Root: %d  #Page: %d  #Keys: %d ",ByteString(aName,gsl_getSystemTextEncoding()).GetBuffer(),  nRootPage, nPageCount, aHeader.db_maxkeys);
	DBG_TRACE("SDB: ------------------------------------------------");
	if (aRoot.Is())
		aRoot->PrintPage();
	else
	{
		GetRoot()->PrintPage();
		Release(FALSE);
	}
}
#endif

//------------------------------------------------------------------
SvStream& operator >> (SvStream &rStream, SdbNDXIndex& rIndex)
{
	rStream.Seek(0);
	rStream.Read(&rIndex.aHeader,512);

	// Text convertierung
	ByteString aText(rIndex.aHeader.db_name);
	aText.Convert(rIndex.GetDBFConnection()->GetCharacterSet(), gsl_getSystemTextEncoding());
	strcpy(rIndex.aHeader.db_name,aText.GetBuffer());

	rIndex.nRootPage = rIndex.aHeader.db_rootpage;
	rIndex.nPageCount = rIndex.aHeader.db_pagecount;
	return rStream;
}
//------------------------------------------------------------------
SvStream& operator << (SvStream &rStream, SdbNDXIndex& rIndex)
{
	rStream.Seek(0);
	ByteString aText(rIndex.aHeader.db_name);
	aText.Convert(gsl_getSystemTextEncoding(), rIndex.GetDBFConnection()->GetCharacterSet());
	strcpy(rIndex.aHeader.db_name,aText.GetBuffer());
	rStream.Write(&rIndex.aHeader,512);
	return rStream;
}

//==================================================================
// SdbNDXIterator
//==================================================================
DBG_NAME(SdbNDXIterator);
//------------------------------------------------------------------
SdbNDXIterator::SdbNDXIterator(SdbFILEIndex* pInd, SdbBoolOperator* pOp, const SdbOperand* pOper)
			   :SdbFILEIndexIterator(pInd,pOp,pOper)
			   ,nCurNode(NODE_NOTFOUND)
{
	if (!pIndex->Lock(TRUE))
		aStatus.Set(SDB_STAT_ERROR);
	DBG_CTOR(SdbNDXIterator,NULL);
}

//------------------------------------------------------------------
SdbNDXIterator::~SdbNDXIterator()
{
	DBG_DTOR(SdbNDXIterator,NULL);
	pIndex->UnLock();
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::First()
{
	return Find(TRUE);
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::Next()
{
	return Find(FALSE);
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::Find(BOOL bFirst)
{
	SdbNDXIndex* pIndex = GetNDXIndex();

	ULONG nRes = SDB_INDEX_ENTRY_NOTFOUND;
	if (!pIndex->IsOpen())
		return nRes;

	if (bFirst)
	{
		aRoot = pIndex->GetRoot();
		aCurLeaf = NULL;
	}

	if (!pOperator)
	{
		// Vorbereitung , auf kleinstes Element positionieren
		if (bFirst)
		{
			SdbNDXPage* pPage = aRoot;
			while (pPage && !pPage->IsLeaf())
				pPage = pPage->GetChild(pIndex);

			aCurLeaf = pPage;
			nCurNode = NODE_NOTFOUND;
		}
		SdbNDXKey* pKey = GetNextKey();
		nRes = pKey ? pKey->GetRecord() : SDB_INDEX_ENTRY_NOTFOUND;
	}
	else if (pOperator->IsA(TYPE(SdbOp_ISNOTNULL)))
		nRes = GetNotNull(bFirst);
	else if (pOperator->IsA(TYPE(SdbOp_ISNULL)))
		nRes = GetNull(bFirst);
	else if (pOperator->IsA(TYPE(SdbOp_LIKE)))
		nRes = GetLike(bFirst);
	else if (pOperator->IsA(TYPE(SdbOp_COMPARE)))
		nRes = GetCompare(bFirst);

	return nRes;
}

//------------------------------------------------------------------
SdbNDXKey* SdbNDXIterator::GetFirstKey(SdbNDXPage* pPage, const SdbOperand& rKey)
{
	// sucht den vorgegeben key
	// Besonderheit: gelangt der Algorithmus ans Ende
	// wird immer die aktuelle Seite und die Knotenposition vermerkt
	// auf die die Bedingung <= zutrifft
	// dieses findet beim Insert besondere Beachtung
	SdbNDXIndex* pIndex = GetNDXIndex();
	SdbOp_COMPARE aTempOp(SDB_PRED_GREATER);
	USHORT i = 0;

	if (pPage->IsLeaf())
	{
		// im blatt wird die eigentliche Operation ausgefuehrt, sonst die temp. (>)
		while (i < pPage->Count() && !pOperator->Operate(&((*pPage)[i]).GetKey(),&rKey))
			   i++;
	}
	else
		while (i < pPage->Count() && !aTempOp.Operate(&((*pPage)[i]).GetKey(),&rKey))
			   i++;


	SdbNDXKey* pFoundKey = NULL;
	if (!pPage->IsLeaf())
	{
		// weiter absteigen
		SdbNDXPagePtr aPage = (i==0) ? pPage->GetChild(pIndex)
									 : ((*pPage)[i-1]).GetChild(pIndex, pPage);
		pFoundKey = aPage.Is() ? GetFirstKey(aPage, rKey) : NULL;
	}
	else if (i == pPage->Count())
	{
		pFoundKey = NULL;
	}
	else
	{
		pFoundKey = &(*pPage)[i].GetKey();
		if (!pOperator->Operate(pFoundKey,&rKey))
			pFoundKey = NULL;

		aCurLeaf = pPage;
		nCurNode = pFoundKey ? i : i - 1;
	}
	return pFoundKey;
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::GetCompare(BOOL bFirst)
{
	SdbNDXKey* pKey = NULL;
	SdbNDXIndex* pIndex = GetNDXIndex();
	SdbPredicateType ePredicateType = PTR_CAST(SdbOp_COMPARE,pOperator)->GetPredicateType();

	if (bFirst)
	{
		// Vorbereitung , auf kleinstes Element positionieren
		SdbNDXPage* pPage = aRoot;
		switch (ePredicateType)
		{
			case SDB_PRED_NOTEQUAL:
			case SDB_PRED_LESS:
			case SDB_PRED_LESSOREQUAL:
				while (pPage && !pPage->IsLeaf())
					pPage = pPage->GetChild(pIndex);

				aCurLeaf = pPage;
				nCurNode = NODE_NOTFOUND;
		}


		switch (ePredicateType)
		{
			case SDB_PRED_NOTEQUAL:
				while ((pKey = GetNextKey()) && !pOperator->Operate(pKey,pOperand));
				break;
			case SDB_PRED_LESS:
				while ((pKey = GetNextKey()) && pKey->Value()->isNull());
				break;
			case SDB_PRED_LESSOREQUAL:
				while (pKey = GetNextKey());
				break;
			case SDB_PRED_GREATEROREQUAL:
			case SDB_PRED_EQUAL:
				pKey = GetFirstKey(aRoot,*pOperand);
				break;
			case SDB_PRED_GREATER:
				if (!(pKey = GetFirstKey(aRoot,*pOperand)))
					while ((pKey = GetNextKey()) && !pOperator->Operate(pKey,pOperand));
		}
	}
	else
	{
		switch (ePredicateType)
		{
			case SDB_PRED_NOTEQUAL:
				while ((pKey = GetNextKey()) && !pOperator->Operate(pKey,pOperand))
					;
				break;
			case SDB_PRED_LESS:
			case SDB_PRED_LESSOREQUAL:
			case SDB_PRED_EQUAL:
				if (!(pKey = GetNextKey()) || !pOperator->Operate(pKey,pOperand))
				{
					pKey = NULL;
					aCurLeaf = NULL;
				}
				break;
			case SDB_PRED_GREATEROREQUAL:
			case SDB_PRED_GREATER:
				pKey = GetNextKey();
		}
	}

	return pKey ? pKey->GetRecord() : SDB_INDEX_ENTRY_NOTFOUND;
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::GetLike(BOOL bFirst)
{
	SdbNDXIndex* pIndex = GetNDXIndex();
	if (bFirst)
	{
		SdbNDXPage* pPage = aRoot;

		while (pPage && !pPage->IsLeaf())
			pPage = pPage->GetChild(pIndex);

		aCurLeaf = pPage;
		nCurNode = NODE_NOTFOUND;
	}

	SdbNDXKey* pKey;
	while ((pKey = GetNextKey()) && !pOperator->Operate(pKey,pOperand))
		;
	return pKey ? pKey->GetRecord() : SDB_INDEX_ENTRY_NOTFOUND;
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::GetNull(BOOL bFirst)
{
	SdbNDXIndex* pIndex = GetNDXIndex();
	if (bFirst)
	{
		SdbNDXPage* pPage = aRoot;
		while (pPage && !pPage->IsLeaf())
			pPage = pPage->GetChild(pIndex);

		aCurLeaf = pPage;
		nCurNode = NODE_NOTFOUND;
	}

	SdbNDXKey* pKey;
	if (!(pKey = GetNextKey()) || !pKey->Value()->isNull())
	{
		pKey = NULL;
		aCurLeaf = NULL;
	}
	return pKey ? pKey->GetRecord() : SDB_INDEX_ENTRY_NOTFOUND;
}

//------------------------------------------------------------------
ULONG SdbNDXIterator::GetNotNull(BOOL bFirst)
{
	SdbNDXKey* pKey;
	SdbNDXIndex* pIndex = GetNDXIndex();
	if (bFirst)
	{
		// erst alle NULL werte abklappern
		for (ULONG nRec = GetNull(bFirst);
			 nRec != SDB_INDEX_ENTRY_NOTFOUND;
			 nRec = GetNull(FALSE))
				 ;
		pKey = aCurLeaf.Is() ? &(*aCurLeaf)[nCurNode].GetKey() : NULL;
	}
	else
		pKey = GetNextKey();

	return pKey ? pKey->GetRecord() : SDB_INDEX_ENTRY_NOTFOUND;
}

//------------------------------------------------------------------
SdbNDXKey* SdbNDXIterator::GetNextKey()
{
	SdbNDXIndex* pIndex = GetNDXIndex();
	if (aCurLeaf.Is() && ((++nCurNode) >= aCurLeaf->Count()))
	{
		SdbNDXPage* pPage = aCurLeaf;
		// naechste Seite suchen
		while (pPage)
		{
			SdbNDXPage* pParentPage = pPage->GetParent();
			if (pParentPage)
			{
				USHORT nPos = pParentPage->Search(pPage);
				if (nPos != pParentPage->Count() - 1)
				{	// Seite gefunden
					pPage = (*pParentPage)[nPos+1].GetChild(pIndex,pParentPage);
					break;
				}
			}
			pPage = pParentPage;
		}

		// jetzt wieder zum Blatt
		while (pPage && !pPage->IsLeaf())
			pPage = pPage->GetChild(pIndex);

		aCurLeaf = pPage;
		nCurNode = 0;
	}
	return aCurLeaf.Is() ? &(*aCurLeaf)[nCurNode].GetKey() : NULL;
}

//==================================================================
// SdbNDXPagePtr
//==================================================================
//------------------------------------------------------------------
SvStream& operator >> (SvStream &rStream, SdbNDXPagePtr& rPage)
{
	rStream >> rPage.nPagePos;
	return rStream;
}

//------------------------------------------------------------------
SvStream& operator << (SvStream &rStream, const SdbNDXPagePtr& rPage)
{
	rStream << rPage.nPagePos;
	return rStream;
}

SV_IMPL_REF(SdbNDXPage);

//------------------------------------------------------------------
SdbNDXPagePtr::SdbNDXPagePtr(const SdbNDXPagePtr& rRef)
			  :SdbNDXPageRef(rRef)
			  ,nPagePos(rRef.nPagePos)
{
}

//------------------------------------------------------------------
SdbNDXPagePtr::SdbNDXPagePtr(SdbNDXPage* pRefPage)
			  :SdbNDXPageRef(pRefPage)
			  ,nPagePos(0)
{
	if (pRefPage)
		nPagePos = pRefPage->GetPagePos();
}

//------------------------------------------------------------------
SdbNDXPagePtr& SdbNDXPagePtr::operator=(const SdbNDXPagePtr& rRef)
{
	SdbNDXPageRef::operator=(rRef);
	nPagePos = rRef.nPagePos;
	return *this;
}

//------------------------------------------------------------------
SdbNDXPagePtr& SdbNDXPagePtr::operator= (SdbNDXPage* pRef)
{
	SdbNDXPageRef::operator=(pRef);
	nPagePos = (pRef) ? pRef->GetPagePos() : 0;
	return *this;
}

//==================================================================
// Index Seite
//==================================================================
DBG_NAME(SdbNDXPage);
//------------------------------------------------------------------
SdbNDXPage::SdbNDXPage(SdbNDXIndex& rInd, ULONG nPos, SdbNDXPage* pParent)
		   :rIndex(rInd)
		   ,nPagePos(nPos)
		   ,aParent(pParent)
		   ,nCount(0)
		   ,bModified(FALSE)
		   ,ppNodes(NULL)
{
	ppNodes = new SdbNDXNode[rIndex.GetHeader().db_maxkeys];
	DBG_CTOR(SdbNDXPage,NULL);
}

//------------------------------------------------------------------
SdbNDXPage::~SdbNDXPage()
{
	DBG_DTOR(SdbNDXPage,NULL);
	delete[] ppNodes;
}

//------------------------------------------------------------------
void SdbNDXPage::QueryDelete()
{
	// Ablegen im GarbageCollector
	if (IsModified())
		rIndex.aFileStream << *this;

	bModified = FALSE;
	if (rIndex.UseCollector())
	{
		if (aChild.Is())
			aChild->Release(FALSE);

		for (USHORT i = 0; i < rIndex.GetHeader().db_maxkeys;i++)
		{
			if (ppNodes[i].GetChild().Is())
				ppNodes[i].GetChild()->Release(FALSE);

			ppNodes[i] = SdbNDXNode();
		}
		RestoreNoDelete();

		nCount = 0;
		aParent.Clear();
		rIndex.Collect(this);
	}
	else
		SvRefBase::QueryDelete();
}

//------------------------------------------------------------------
SdbNDXPagePtr& SdbNDXPage::GetChild(SdbNDXIndex* pIndex)
{
	if (!aChild.Is() && pIndex)
	{
		aChild = rIndex.CreatePage(aChild.GetPagePos(),this,aChild.HasPage());
	}
	return aChild;
}

//------------------------------------------------------------------
USHORT SdbNDXPage::FindPos(const SdbNDXKey& rKey) const
{
	// sucht nach Platz fuer den vorgegeben key auf einer Seite
	USHORT i = 0;
	while (i < nCount && rKey > ((*this)[i]).GetKey())
		   i++;

	return i;
}

//------------------------------------------------------------------
BOOL SdbNDXPage::Find(const SdbNDXKey& rKey)
{
	// sucht den vorgegeben key
	// Besonderheit: gelangt der Algorithmus ans Ende
	// wird immer die aktuelle Seite und die Knotenposition vermerkt
	// auf die die Bedingung <= zutrifft
	// dieses findet beim Insert besondere Beachtung
	USHORT i = 0;
	while (i < nCount && rKey > ((*this)[i]).GetKey())
		   i++;

	BOOL bResult = FALSE;

	if (!IsLeaf())
	{
		// weiter absteigen
		SdbNDXPagePtr aPage = (i==0) ? GetChild(&rIndex) : ((*this)[i-1]).GetChild(&rIndex, this);
		bResult = aPage.Is() && aPage->Find(rKey);
	}
	else if (i == nCount)
	{
		rIndex.aCurLeaf = this;
		rIndex.nCurNode = i - 1;
		bResult = FALSE;
	}
	else
	{
		bResult = rKey == ((*this)[i]).GetKey();
		rIndex.aCurLeaf = this;
		rIndex.nCurNode = bResult ? i : i - 1;
	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbNDXPage::Insert(SdbNDXNode& rNode, ULONG nRowsLeft)
{
	// beim Erzeugen eines Index koennen auch mehrere Knoten eingefuegt werden
	// diese sin dann aufsteigend sortiert
	BOOL bAppend = nRowsLeft > 0;
	if (IsFull())
	{
		BOOL bResult = TRUE;
		SdbNDXNode aSplitNode;
		if (bAppend)
			aSplitNode = rNode;
		else
		{
			// merken des letzten Knotens
			aSplitNode = (*this)[nCount-1];
			if(rNode.GetKey() <= aSplitNode.GetKey())
			{

				// und damit habe ich im folgenden praktisch eine Node weniger
				if (IsLeaf() && this == &rIndex.aCurLeaf)
				{
					// geht davon aus, dass der Knoten, auf dem die Bedingung (<=)
					// zutrifft, als nCurNode gesetzt ist
					--nCount;	// (sonst bekomme ich u.U. Assertions und GPFs - 60593)
					bResult = Insert(rIndex.nCurNode + 1, rNode);
				}
				else  // Position unbekannt
				{
					USHORT nPos = NODE_NOTFOUND;
					while (++nPos < nCount && rNode.GetKey() > ((*this)[nPos]).GetKey());

					--nCount;	// (sonst bekomme ich u.U. Assertions und GPFs - 60593)
					bResult = Insert(nPos, rNode);
				}

				// konnte der neue Knoten eingefuegt werden
				if (!bResult)
				{
					nCount++;
					aSplitNode = rNode;
				}
			}
			else
				aSplitNode = rNode;
		}

		ULONG nNewPagePos = rIndex.GetPageCount();
		ULONG nNewPageCount = nNewPagePos + 1;

		// Herausgeloesten Knoten beim Vater einfuegen
		if (!HasParent())
		{
			// Kein Vater, dann neue Wurzel
			SdbNDXPagePtr aNewRoot = rIndex.CreatePage(nNewPagePos + 1);
			aNewRoot->SetChild(this);

			rIndex.aRoot = aNewRoot;
			rIndex.SetRootPos(nNewPagePos + 1);
			rIndex.SetPageCount(++nNewPageCount);
		}

		// neues blatt erzeugen und Seite aufteilen
		SdbNDXPagePtr aNewPage = rIndex.CreatePage(nNewPagePos,aParent);
		rIndex.SetPageCount(nNewPageCount);

		// wieviele Knoten weren noch eingefuegt
		// kommen noch ausreichend, dann koennen die Seiten bis zum Rand vollgestopft werden

		SdbNDXNode aInnerNode;
		if (!IsLeaf() || nRowsLeft < (ULONG)(rIndex.GetMaxNodes() / 2))
			aInnerNode = Split(*aNewPage);
		else
		{
			aInnerNode = (*this)[nCount - 1];
			//aInnerNode = aSplitNode;

			// Knoten zeigt auf neue Seite
			aInnerNode.SetChild(aNewPage);

			// innere Knoten haben keine Recordnummer
			if (rIndex.IsUnique())
				aInnerNode.GetKey().ResetRecord();

			// neue Seite zeigt nun auf Seite des herausgelsten Knoten
			if (!IsLeaf())
				aNewPage->SetChild(aInnerNode.GetChild());
		}

		aNewPage->Append(aSplitNode);
		SdbNDXPagePtr aTempParent = aParent;
		if (IsLeaf())
		{
			rIndex.aCurLeaf = aNewPage;
			rIndex.nCurNode = rIndex.aCurLeaf->Count() - 1;

			// Freigeben nicht benoetigter Seiten, danach besteht keine Referenz
			// mehr auf die Seite, danach kann 'this' nicht mehr gueltig sein!!!
			ReleaseFull();
		}

		// Einfuegen des herausgeloesten Knotens
		return aTempParent->Insert(aInnerNode);
	}
	else // Seite einfach weiter auffuellen
	{
		if (bAppend)
		{
			if (IsLeaf())
				rIndex.nCurNode = nCount - 1;
			return Append(rNode);
		}
		else
		{
			USHORT nNodePos = FindPos(rNode.GetKey());
			if (IsLeaf())
				rIndex.nCurNode = nNodePos;

			return Insert(nNodePos, rNode);
		}
	}
}

//------------------------------------------------------------------
BOOL SdbNDXPage::Insert(USHORT nPos, SdbNDXNode& rNode)
{
	USHORT nMaxCount = rIndex.GetHeader().db_maxkeys;
	if (nPos >= nMaxCount)
		return FALSE;

	if (nCount)
	{
		++nCount;
		// nach rechts verschieben
		for (USHORT i = min(nMaxCount-1, nCount-1); nPos < i; i--)
			(*this)[i] = (*this)[i-1];
	}
	else
		if (nCount < nMaxCount)
			nCount++;

	// einfuegen an der Position
	SdbNDXNode& rInsertNode = (*this)[nPos];
	rInsertNode = rNode;
	if (rInsertNode.GetChild().Is())
	{
		rInsertNode.GetChild()->SetParent(this);
		rNode.GetChild()->SetParent(this);
	}

	bModified = TRUE;

	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbNDXPage::Append(SdbNDXNode& rNode)
{
	DBG_ASSERT(!IsFull(), "kein Append moeglich");
	return Insert(nCount, rNode);
}

//------------------------------------------------------------------
void SdbNDXPage::Remove(USHORT nPos)
{
	DBG_ASSERT(nCount > nPos, "falscher Indexzugriff");

	for (USHORT i = nPos; i < (nCount-1); i++)
		(*this)[i] = (*this)[i+1];

	nCount--;
	bModified = TRUE;
}

//------------------------------------------------------------------
void SdbNDXPage::Release(BOOL bSave)
{
	// freigeben der Pages
	if (aChild.Is())
		aChild->Release(bSave);

	// Pointer freigeben
	aChild.Clear();

	for (USHORT i = 0; i < rIndex.GetHeader().db_maxkeys;i++)
	{
		if (ppNodes[i].GetChild())
			ppNodes[i].GetChild()->Release(bSave);

		ppNodes[i].GetChild().Clear();
	}
	aParent = NULL;
}

//------------------------------------------------------------------
void SdbNDXPage::ReleaseFull(BOOL bSave)
{
	SdbNDXPagePtr aTempParent = aParent;
	Release(bSave);

	if (aTempParent.Is())
	{
		// Freigeben nicht benoetigter Seiten, danach besteht keine Referenz
		// mehr auf die Seite, danach kann 'this' nicht mehr gueltig sein!!!
		USHORT nParentPos = aTempParent->Search(this);
		if (nParentPos != NODE_NOTFOUND)
			(*aTempParent)[nParentPos].GetChild().Clear();
		else
			aTempParent->GetChild().Clear();
	}
}


//------------------------------------------------------------------
SdbNDXNode& SdbNDXPage::operator[] (USHORT nPos)
{
	DBG_ASSERT(nCount > nPos, "falscher Indexzugriff");
	return ppNodes[nPos];
}

//------------------------------------------------------------------
const SdbNDXNode& SdbNDXPage::operator[] (USHORT nPos) const
{
	DBG_ASSERT(nCount > nPos, "falscher Indexzugriff");
	return ppNodes[nPos];
}

//------------------------------------------------------------------
// laeuft rekursiv
void SdbNDXPage::SearchAndReplace(const SdbNDXKey& rSearch,
								  SdbNDXKey& rReplace)
{
	if (rSearch == rReplace)
		return;

	USHORT nPos = NODE_NOTFOUND;
	SdbNDXPage* pPage = this;

	while (pPage && (nPos = pPage->Search(rSearch)) == NODE_NOTFOUND)
		pPage = pPage->aParent;

	if (pPage)
	{
		(*pPage)[nPos].GetKey() = rReplace;
		pPage->SetModified(TRUE);
	}
}

//------------------------------------------------------------------
USHORT SdbNDXPage::Search(const SdbNDXKey& rSearch)
{
	// binare Suche spaeter
	USHORT i = 0xFFFF;
	while (++i < Count())
		if ((*this)[i].GetKey() == rSearch)
			break;

	return (i < Count()) ? i : NODE_NOTFOUND;
}

//------------------------------------------------------------------
USHORT SdbNDXPage::Search(const SdbNDXPage* pPage)
{
	USHORT i = 0xFFFF;
	while (++i < Count())
		if (&((*this)[i]).GetChild() == pPage)
			break;

	// wenn nicht gefunden, dann wird davon ausgegangen, dass die Seite selbst
	// auf die Page zeigt
	return (i < Count()) ? i : NODE_NOTFOUND;
}

//------------------------------------------------------------------
BOOL SdbNDXPage::Delete(USHORT nNodePos)
{
	if (IsLeaf())
	{
		// Letztes Element wird geloescht
		if (nNodePos == (nCount - 1))
		{
			SdbNDXNode aNode = (*this)[nNodePos];

			// beim Parent muss nun der KeyValue ausgetauscht werden
			if (HasParent())
				aParent->SearchAndReplace(aNode.GetKey(),
										  (*this)[nNodePos-1].GetKey());
		}
	}

	// Loeschen des Knoten
	Remove(nNodePos);

	// Unterlauf
	if (HasParent() && nCount < (rIndex.GetMaxNodes() / 2))
	{
		// Feststellen, welcher Knoten auf die Seite zeigt
		USHORT nParentNodePos = aParent->Search(this);
		// letzte Element auf Vaterseite
		// -> zusammenlegen mit vorletzter Seite
		if (nParentNodePos == (aParent->Count() - 1))
		{
			if (!nParentNodePos)
			// zusammenlegen mit linken nachbarn
				Merge(nParentNodePos,aParent->GetChild(&rIndex));
			else
				Merge(nParentNodePos,(*aParent)[nParentNodePos-1].GetChild(&rIndex,aParent));
		}
		// sonst Seite mit naechster Seite zusammenlegen
		else
		{
			// zusammenlegen mit rechten nachbarn
			Merge(nParentNodePos + 1,((*aParent)[nParentNodePos + 1].GetChild(&rIndex,aParent)));
			nParentNodePos++;
		}
		if (HasParent() && !(*aParent)[nParentNodePos].HasChild())
			aParent->Delete(nParentNodePos);
/*
		// letzte Element auf Vaterseite
		// -> zusammenlegen mit vorletzter Seite
		if (nParentNodePos == (aParent->Count() - 1))
		{
			if (!nParentNodePos)
			// zusammenlegen mit linken nachbarn
				Merge(nParentNodePos,aParent->GetChild(&rIndex));
			else
				Merge(nParentNodePos,(*aParent)[nParentNodePos-1].GetChild(&rIndex,aParent));
		}
		// sonst Seite mit naechster Seite zusammenlegen
		else if(nParentNodePos != NODE_NOTFOUND)
		{
			// zusammenlegen mit rechten nachbarn
			Merge(nParentNodePos + 1,((*aParent)[nParentNodePos + 1].GetChild(&rIndex,aParent)));
			nParentNodePos++;
		}
		else // Sonderbehandlung
		{
			// Page ist aChild Page vom Parent => erste Page aus ppNodes an aChild anhngen
			Merge(0,(*aParent)[0].GetChild(&rIndex,aParent));
			nParentNodePos = 0;
		}

		if (HasParent() && !(*aParent)[nParentNodePos].HasChild())
			aParent->Delete(nParentNodePos);
*/

	}
	else if (IsRoot())
		// Sicherstellen das die Position der Wurzel festgehalten wird
		rIndex.SetRootPos(nPagePos);
	return TRUE;
}


//------------------------------------------------------------------
SdbNDXNode SdbNDXPage::Split(SdbNDXPage& rPage)
{
	DBG_ASSERT(IsFull(), "Falsches Splitting");
	/*	Aufteilen einer Seite auf zwei
		Blatt:
			Seite 1 behaelt (n - (n/2))
			Seite 2 erhaelt (n/2)
			Knoten n/2 wird dupliziert
		Innerer Knoten:
			Seite 1 behaelt (n+1)/2
			Seite 2 erhaelt (n/2-1)
			Knoten ((n+1)/2 + 1) : wird herausgenommen
	*/
	SdbNDXNode aResultNode;
	if (IsLeaf())
	{
		for (USHORT i = (nCount - (nCount / 2)), j = 0 ; i < nCount; i++)
			rPage.Insert(j++,(*this)[i]);

		// dieser Knoten enthaelt einen Schluessel der noch einmal im Tree vorkommt
		// und ersetzt werden muss
		SdbNDXNode aLastNode = (*this)[nCount - 1];
		nCount = nCount - (nCount / 2);
		aResultNode = (*this)[nCount - 1];

		if (HasParent())
			aParent->SearchAndReplace(aLastNode.GetKey(),
									  aResultNode.GetKey());
	}
	else
	{
		for (USHORT i = (nCount + 1) / 2 + 1, j = 0 ; i < nCount; i++)
			rPage.Insert(j++,(*this)[i]);

		aResultNode = (*this)[(nCount + 1) / 2];
		nCount = (nCount + 1) / 2;

		// neue Seite zeigt nun auf Seite des herausgelsten Knoten
		rPage.SetChild(aResultNode.GetChild());
	}
	// Knoten zeigt auf neue Seite
	aResultNode.SetChild(&rPage);

	// innere Knoten haben keine Recordnummer
	if (rIndex.IsUnique())
		aResultNode.GetKey().ResetRecord();
	bModified = TRUE;
	return aResultNode;
}

//------------------------------------------------------------------
void SdbNDXPage::Merge(USHORT nParentNodePos, SdbNDXPagePtr xPage)
{
	DBG_ASSERT(HasParent(), "kein Vater vorhanden");
	DBG_ASSERT(nParentNodePos != NODE_NOTFOUND, "Falscher Indexaufbau");

	/*	Zusammenlegen zweier Seiten	*/
	SdbNDXNode aResultNode;
	USHORT nMaxNodes = rIndex.GetMaxNodes(),
		   nMaxNodes_2 = nMaxNodes / 2;

	// Feststellen ob Seite rechter oder linker Nachbar
	BOOL	bRight    = ((*xPage)[0].GetKey() > (*this)[0].GetKey()); // TRUE, wenn xPage die rechte Seite ist
	USHORT	nNewCount = (*xPage).Count() + Count();

	if (IsLeaf())
	{
		// Bedingung fuers zusammenlegen
		if (nNewCount < (nMaxNodes_2 * 2))
		{
			USHORT nLastNode = bRight ? Count() - 1 : xPage->Count() - 1;
			if (bRight)
			{
				DBG_ASSERT(&xPage != this,"xPage und THIS drfen nicht gleich sein: Endlosschleife");
				// alle Knoten aus xPage auf den linken Knoten verschieben (anhngen)
				while (xPage->Count())
				{
					Append((*xPage)[0]);
					xPage->Remove(0);
				}
			}
			else
			{
				DBG_ASSERT(&xPage != this,"xPage und THIS drfen nicht gleich sein: Endlosschleife");
				// xPage ist die linke Page und THIS die rechte
				while (xPage->Count())
				{
					Insert(0,(*xPage)[xPage->Count()-1]);
					xPage->Remove(xPage->Count()-1);
				}
				// alte Position von xPage beim Parent mit this ersetzen
				if (nParentNodePos)
					(*aParent)[nParentNodePos-1].SetChild(this,aParent);
				else // oder als rechten Knoten setzen
					aParent->SetChild(this);
				aParent->SetModified(TRUE);

			}

			// Child beziehung beim Vaterknoten aufheben
			(*aParent)[nParentNodePos].SetChild();
			// Austauschen des KnotenWertes, nur wenn geaenderte Page
			// die linke ist, ansonsten werde

			if(aParent->IsRoot() && aParent->Count() == 1)
			{
				(*aParent)[0].SetChild();
				aParent->ReleaseFull();
				aParent = NULL;
				rIndex.SetRootPos(nPagePos);
				rIndex.aRoot = this;
				SetModified(TRUE);
			}
			else
				aParent->SearchAndReplace((*this)[nLastNode].GetKey(),(*this)[nCount-1].GetKey());

			xPage->SetModified(FALSE);
			xPage->ReleaseFull(); // wird nicht mehr benoetigt
		}
		// Ausgleichen der Elemente	  nNewCount >= (nMaxNodes_2 * 2)
		else
		{
			if (bRight)
			{
				// alle Knoten aus xPage auf den linken Knoten verschieben (anhngen)
				SdbNDXNode aReplaceNode = (*this)[nCount - 1];
				while (nCount < nMaxNodes_2)
				{
					Append((*xPage)[0]);
					xPage->Remove(0);
				}
				// Austauschen des KnotenWertes: Setzt alten letzten Wert durch den letzten von xPage
				aParent->SearchAndReplace(aReplaceNode.GetKey(),(*this)[nCount-1].GetKey());
			}
			else
			{
				// alle Knoten aus this vor die xPage Knoten einfgen
				SdbNDXNode aReplaceNode = (*this)[nCount - 1];
				while (xPage->Count() < nMaxNodes_2)
				{
					xPage->Insert(0,(*this)[nCount-1]);
					Remove(nCount-1);
				}
				// Austauschen des KnotenWertes
				aParent->SearchAndReplace(aReplaceNode.GetKey(),(*this)[Count()-1].GetKey());
			}
		}
	}
	else // !IsLeaf()
	{
		// Bedingung fuers zusammenlegen
		if (nNewCount < nMaxNodes_2 * 2)
		{
			if (bRight)
			{
				DBG_ASSERT(&xPage != this,"xPage und THIS drfen nicht gleich sein: Endlosschleife");
				// Vaterknoten wird mit integriert
				// erhaelt zunaechst Child von xPage
				(*aParent)[nParentNodePos].SetChild(xPage->GetChild(),aParent);
				Append((*aParent)[nParentNodePos]);
				for (USHORT i = 0 ; i < xPage->Count(); i++)
					Append((*xPage)[i]);
			}
			else
			{
				DBG_ASSERT(&xPage != this,"xPage und THIS drfen nicht gleich sein: Endlosschleife");
				// Vaterknoten wird mit integriert
				// erhaelt zunaechst Child
				(*aParent)[nParentNodePos].SetChild(GetChild(),aParent); // Parent merkt sich mein Child
				Insert(0,(*aParent)[nParentNodePos]); // Node vom Parent bei mir einfgen
				while (xPage->Count())
				{
					Insert(0,(*xPage)[xPage->Count()-1]);
					xPage->Remove(xPage->Count()-1);
				}
				SetChild(xPage->GetChild());

				if (nParentNodePos)
					(*aParent)[nParentNodePos-1].SetChild(this,aParent);
				else
					aParent->SetChild(this);
			}

			// danach wird der Vaterknoten zurueckgesetzt
			(*aParent)[nParentNodePos].SetChild();
			aParent->SetModified(TRUE);

			if(aParent->IsRoot() && aParent->Count() == 1)
			{
				(*aParent).SetChild();
				aParent->ReleaseFull();
				aParent = NULL;
				rIndex.SetRootPos(nPagePos);
				rIndex.aRoot = this;
				SetModified(TRUE);
			}
			else if(nParentNodePos)
				// Austauschen des KnotenWertes
				// beim Append wird der Bereich erweitert, beim INsert verweist der alte Knoten von xPage auf this
				// deshalb mu der Knoten auch hier aktualisiert werden
				aParent->SearchAndReplace((*aParent)[nParentNodePos-1].GetKey(),(*aParent)[nParentNodePos].GetKey());

			xPage->SetModified(FALSE);
			xPage->ReleaseFull();
		}
		// Ausgleichen der Elemente
		else
		{
			if (bRight)
			{
				while (nCount < nMaxNodes_2)
				{
					(*aParent)[nParentNodePos].SetChild(xPage->GetChild(),aParent);
					Append((*aParent)[nParentNodePos]);
					(*aParent)[nParentNodePos] = (*xPage)[0];
					xPage->Remove(0);
				}
				xPage->SetChild((*aParent)[nParentNodePos].GetChild());
				(*aParent)[nParentNodePos].SetChild(&xPage,aParent);
			}
			else
			{
				while (nCount < nMaxNodes_2)
				{
					(*aParent)[nParentNodePos].SetChild(GetChild(),aParent);
					Insert(0,(*aParent)[nParentNodePos]);
					(*aParent)[nParentNodePos] = (*xPage)[xPage->Count()-1];
					xPage->Remove(xPage->Count()-1);
				}
				SetChild((*aParent)[nParentNodePos].GetChild());
				(*aParent)[nParentNodePos].SetChild(this,aParent);

			}
			aParent->SetModified(TRUE);
		}
	}
}

#ifdef DEBUG
//------------------------------------------------------------------
void SdbNDXPage::PrintPage()
{
	DBG_TRACE4("\nSDB: -----------Page: %d  Parent: %d  Count: %d  Child: %d-----",
		nPagePos, HasParent() ? aParent->GetPagePos() : 0 ,nCount, aChild.GetPagePos());

	for (USHORT i = 0; i < nCount; i++)
	{
		SdbNDXNode rNode = (*this)[i];
		SdbNDXKey&  rKey = rNode.GetKey();
		if (!IsLeaf())
			rNode.GetChild(&rIndex, this);

		if (rKey.Value()->isNull())
		{
			DBG_TRACE2("SDB: [%d,NULL,%d]",rKey.GetRecord(), rNode.GetChild().GetPagePos());
		}
		else if (rIndex.GetHeader().db_keytype)
		{
			DBG_TRACE3("SDB: [%d,%f,%d]",rKey.GetRecord(), rKey.Value()->getDouble(),rNode.GetChild().GetPagePos());
		}
		else
		{
			DBG_TRACE3("SDB: [%d,%s,%d]",rKey.GetRecord(), (const char* )ByteString(rKey.Value()->getString().getStr(), gsl_getSystemTextEncoding()).GetBuffer(),rNode.GetChild().GetPagePos());
		}
	}
	DBG_TRACE("SDB: -----------------------------------------------\n");
	if (!IsLeaf())
	{
		GetChild(&rIndex)->PrintPage();
		for (USHORT i = 0; i < nCount; i++)
		{
			SdbNDXNode rNode = (*this)[i];
			rNode.GetChild(&rIndex,this)->PrintPage();
		}
	}
	DBG_TRACE("SDB: ===============================================\n");
}
#endif


static UINT32 nValue;

//------------------------------------------------------------------
SvStream& operator >> (SvStream &rStream, SdbNDXPage& rPage)
{
	rStream.Seek(rPage.GetPagePos() * 512);
	rStream >> nValue >> rPage.aChild;
	rPage.nCount = USHORT(nValue);

//	DBG_ASSERT(rPage.nCount && rPage.nCount < rPage.GetIndex().GetMaxNodes(), "Falscher Count");
	for (USHORT i = 0; i < rPage.nCount; i++)
		rPage[i].Read(rStream, rPage.GetIndex());
	return rStream;
}

//------------------------------------------------------------------
SvStream& operator << (SvStream &rStream, const SdbNDXPage& rPage)
{
	// Seite existiert noch nicht
	if ((rPage.GetPagePos() + 1) * 512 > rStream.Seek(STREAM_SEEK_TO_END))
		rStream.SetStreamSize((rPage.GetPagePos() + 1) * 512);
	rStream.Seek(rPage.GetPagePos() * 512);

	nValue = rPage.nCount;
	rStream << nValue << rPage.aChild;

	for (USHORT i = 0; i < rPage.nCount; i++)
		rPage[i].Write(rStream, rPage);
	return rStream;
}

//==================================================================
// SdbNDXNode
//==================================================================

//------------------------------------------------------------------
void SdbNDXNode::Read(SvStream &rStream, SdbNDXIndex& rIndex)
{
	rStream >> aKey.nRecord; // schluessel
	if (rIndex.GetHeader().db_keytype)
	{
		double aDbl;
		rStream >> aDbl;
		aKey = SdbNDXKey(aDbl,aKey.nRecord);
	}
	else
	{
		ByteString aBuf;
		USHORT nLen = rIndex.GetHeader().db_keylen;
		char* pStr = aBuf.AllocBuffer(nLen+1);

		rStream.Read(pStr,nLen);
		pStr[nLen] = 0;
		aBuf.ReleaseBufferAccess();
		aBuf.EraseTrailingChars();

		aKey = SdbNDXKey((aBuf,rIndex.GetDBFConnection()->GetCharacterSet()) ,aKey.nRecord);
	}
	rStream >> aChild;
}

union
{
	double aDbl;
	char   aData[128];
} aNodeData;
//------------------------------------------------------------------
void SdbNDXNode::Write(SvStream &rStream, const SdbNDXPage& rPage) const
{
	const SdbNDXIndex& rIndex = rPage.GetIndex();
	if (!rIndex.IsUnique() || rPage.IsLeaf())
		rStream << aKey.nRecord; // schluessel
	else
		rStream << (ULONG)0;	// schluessel

	if (rIndex.GetHeader().db_keytype) // double
	{
		if (aKey.Value()->isNull())
		{
			memset(aNodeData.aData,0,rIndex.GetHeader().db_keylen);
			rStream.Write((BYTE*)aNodeData.aData,rIndex.GetHeader().db_keylen);
		}
		else
			rStream << (double) aKey.Value()->getDouble();
	}
	else
	{
		memset(aNodeData.aData,0x20,rIndex.GetHeader().db_keylen);
		if (!aKey.Value()->isNull())
		{
			SdbDBFConnection *pCon = rIndex.GetDBFConnection();
			if (pCon)
			{
				ByteString aText(aKey.Value()->getString().getStr(), pCon->GetCharacterSet());
				strncpy(aNodeData.aData,aText.GetBuffer(),min(rIndex.GetHeader().db_keylen, aText.Len()));
			}
			else
			{
				DBG_ERROR("No Connection");
				ByteString aText(aKey.Value()->getString().getStr(), gsl_getSystemTextEncoding());
				strncpy(aNodeData.aData,aText.GetBuffer(),min(rIndex.GetHeader().db_keylen, aText.Len()));
			}
		}
		rStream.Write((BYTE*)aNodeData.aData,rIndex.GetHeader().db_keylen);
	}
	rStream << aChild;
}


//------------------------------------------------------------------
SdbNDXPagePtr& SdbNDXNode::GetChild(SdbNDXIndex* pIndex, SdbNDXPage* pParent)
{
	if (!aChild.Is() && pIndex)
	{
		aChild = pIndex->CreatePage(aChild.GetPagePos(),pParent,aChild.HasPage());
	}
	return aChild;
}

//==================================================================
// SdbNDXKey
//==================================================================
//------------------------------------------------------------------
BOOL SdbNDXKey::IsText(SdbDatabaseType eType)
{
	return eType == SDB_DBTYPE_VARCHAR || eType == SDB_DBTYPE_CHAR;
}

//------------------------------------------------------------------
StringCompare SdbNDXKey::Compare(const SdbNDXKey& rKey) const
{
	DBG_ASSERT(Is(), "Falscher Indexzugriff");
	StringCompare eResult;

	if (!Value() || Value()->isNull())
	{
		if (!rKey.Value() || rKey.Value()->isNull() || (rKey.IsText(DBType()) && !rKey.Value()->getString().len()))
			eResult = COMPARE_EQUAL;
		else
			eResult = COMPARE_LESS;
	}
	else if (!rKey.Value() || rKey.Value()->isNull())
	{
		if (!Value() || Value()->isNull() || (IsText(DBType()) && !Value()->getString().len()))
			eResult = COMPARE_EQUAL;
		else
			eResult = COMPARE_GREATER;
	}
	else if (IsText(DBType()))
	{
		INT32 nRes = Value()->getString().compareTo(rKey.Value()->getString());
		eResult = (nRes > 0) ? COMPARE_GREATER : (nRes == 0) ? COMPARE_EQUAL : COMPARE_LESS;
	}
	else
	{
		double m,n;
		m = Value()->getDouble();
		n = rKey.Value()->getDouble();

		eResult = (m > n) ? COMPARE_GREATER : (n == m) ? COMPARE_EQUAL : COMPARE_LESS;
	}

	// Record vergleich, wenn Index !Unique
	if (eResult == COMPARE_EQUAL && nRecord && rKey.nRecord)
		eResult = (nRecord > rKey.nRecord) ? COMPARE_GREATER :
				  (nRecord == rKey.nRecord) ? COMPARE_EQUAL : COMPARE_LESS;

	return eResult;
}

