/*************************************************************************
 *
 *  $RCSfile: dbftab.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>		//sprintf
#include <stdlib.h>
#include <math.h>

#include "sdbresid.hrc"

#ifndef _SDB_DBFTAB_HXX
#include "dbftab.hxx"
#endif

#ifndef _SDB_DBFCONN_HXX
#include "dbfconn.hxx"
#endif

#ifndef _SDB_DBFIDX_HXX
#include "dbfidx.hxx"
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

#ifndef _SDB_FILECOL_HXX
#include "filecol.hxx"
#endif

#ifndef _SBXMSTRM_HXX //autogen
#include <svtools/sbxmstrm.hxx>
#endif

#ifndef _CONFIG_HXX //autogen
#include <vcl/config.hxx>
#endif

#ifndef _SDB_DBFMD_HXX
#include <dbfmd.hxx>
#endif

#ifndef _SDB_SDBDB_HXX //autogen
#include "sdbdb.hxx"
#endif

#ifndef _TOOLS_INTN_HXX //autogen
#include <tools/intn.hxx>
#endif

#include <svtools/converter.hxx>

#ifndef _COM_SUN_STAR_DATA_DATABASELIMIT_HPP_
#include <com/sun/star/data/DatabaseLimit.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

DBG_NAME(SdbDBFTable);



//------------------------------------------------------------------
SdbDBFTable::SdbDBFTable(SdbDBFConnection* pConnection)
			:SdbFILETable(pConnection)
			,pBuffer(NULL)
			,nBufferSize(0)
{
	aStatus.SetComponent(String::CreateFromAscii("Sdb DBF Driver"));
	DBG_CTOR(SdbDBFTable,NULL);
}

//------------------------------------------------------------------
SdbDBFTable::~SdbDBFTable()
{
	DBG_DTOR(SdbDBFTable,NULL);
	delete pBuffer;
}

//------------------------------------------------------------------
SdbIndex* SdbDBFTable::CreateIndexImpl()
{
	return new SdbNDXIndex(this);
}

//------------------------------------------------------------------
void SdbDBFTable::FillIndexNames(SvStringsDtor& _rList)
{
	// Einlesen aller Indices
	///////////////////////////////////////////////////////////////////////////
	// Liste Tabellenindizes fuellen
	String aNDX;
	DirEntry aEntry = GetDBFConnection()->FileEntry(aName);
	aEntry.SetExtension(String::CreateFromAscii("inf"));

	Config aInfFile(aEntry.GetFull());
	aInfFile.SetGroup(dBASE_III_GROUP);
	USHORT nKeyCnt = aInfFile.GetKeyCount();
	ByteString aKeyName;
	ByteString aIndexName;

	for (USHORT nKey = 0,nPos=0; nKey < nKeyCnt; nKey++)
	{
		// Verweist der Key auf ein Indexfile?...
		aKeyName = aInfFile.GetKeyName( nKey );
		//...wenn ja, Indexliste der Tabelle hinzufuegen
		if (aKeyName.Copy(0,3) == ByteString("NDX") )
		{
			aIndexName = aInfFile.ReadKey(aKeyName);
			aEntry.SetName(String(aIndexName,gsl_getSystemTextEncoding()));
			if (aEntry.Exists())
				_rList.Insert(new String(aEntry.GetBase()), nPos++);
		}
	}
}

// Makro zum Filehandling frs Erzeugen von Tabellen
#define TRY_CREATETABLE(action,type)\
	action;\
	if (aStatus.SetError(aFileStream,type))\
	{\
		aFileStream.ResetError();\
		aFileStream.Close();\
		aFile.Kill();\
		return FALSE;\
	}


// erzeugt grundstzlich dBase IV Datei Format
//------------------------------------------------------------------
BOOL SdbDBFTable::CreateImpl()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	DBG_ASSERT(IsOpen(), "SequenceError");

	DirEntry aFile(GetDBFConnection()->FileEntry(aName));
	if (aFile.Exists())
	{
		// Hack fuer Bug #30609 , nur wenn das File existiert und die Laenge > 0 gibt es einen Fehler
		SvFileStream aFileStream;
		aFileStream.Open(aFile.GetFull(),STREAM_STD_READ);

		if (aFileStream.IsOpen() && aFileStream.Seek(STREAM_SEEK_TO_END))
		{
			aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,TABLE,aFile.GetFull());
			return FALSE;
		}
		aFileStream.Close();
	}

	BOOL bMemoFile = FALSE;

	CreateFile(aFile, bMemoFile);

	FileClose();

	if (aStatus.IsError())
	{
		aFile.Kill();
		return FALSE;
	}

	if (bMemoFile)
	{
		String aExt = aFile.GetExtension();
		aFile.SetExtension(String::CreateFromAscii("dbt"));                      // extension for memo file
		if (aFile.Exists())
		{
			aStatus.SetError(ERRCODE_IO_ALREADYEXISTS,MEMO,aFile.GetFull());
			aFile.SetExtension(aExt);      // kill dbf file
			aFile.Kill();
			return FALSE;
		}
		CreateMemoFile(aFile);
		if (aStatus.IsError())
		{
			aFile.SetExtension(aExt);      // kill dbf file
			aFile.Kill();
			return FALSE;
		}
		aFileHeader.db_typ = dBaseIIIMemo;
	}
	else
		aFileHeader.db_typ = dBaseIII;

	if (GetDBFConnection()->GetShowDeleted())
		nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE |
			SDB_PR_ALTER | SDB_PR_DROP;
	else
		nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE |
			SDB_PR_DELETE | SDB_PR_ALTER | SDB_PR_DROP;

	return TRUE;
}

//------------------------------------------------------------------
// erzeugt grundstzlich dBase IV Datei Format
BOOL SdbDBFTable::CreateFile(const DirEntry& aFile, BOOL& bCreateMemo) // kein Return-Wert da intensive Makronutzung
{
	bCreateMemo = FALSE;
	Date aDate;                                     // aktuelles Datum

	aFileStream.Open(aFile.GetFull(), STREAM_READWRITE | STREAM_SHARE_DENYWRITE | STREAM_TRUNC);
	if (aStatus.SetError(aFileStream,TABLE))
		return FALSE;

	char aBuffer[21];               // write buffer
	memset(aBuffer,0,sizeof(aBuffer));

	TRY_CREATETABLE(aFileStream.Seek(0L),TABLE);
	TRY_CREATETABLE(aFileStream << (BYTE) dBaseIII,TABLE);                                                                // dBase format
	TRY_CREATETABLE(aFileStream << (BYTE) (aDate.GetYear() % 100),TABLE);                 // aktuelles Datum


	TRY_CREATETABLE(aFileStream << (BYTE) aDate.GetMonth(),TABLE);
	TRY_CREATETABLE(aFileStream << (BYTE) aDate.GetDay(),TABLE);
	TRY_CREATETABLE(aFileStream << 0L,TABLE);                                                                                                     // Anzahl der Datenstze
	TRY_CREATETABLE(aFileStream << (USHORT)(aColumns->Count() * 32 + 1),TABLE);                // Kopfinformationen,
																		// pColumns erhlt immer eine Spalte mehr
	TRY_CREATETABLE(aFileStream << (USHORT) 0,TABLE);                                                                                     // Satzlnge wird spter bestimmt
	TRY_CREATETABLE(aFileStream.Write(aBuffer, 20),TABLE);

	USHORT nRecLength = 1;                                                                                          // Lnge 1 fr deleted flag
	ULONG  nMaxFieldLength = GetConnection()->GetMetaData()->getLimit(::stardata::DatabaseLimit::COLUMN_NAME_LEN);
	for (int i = 1; i < aColumns->Count(); i++)
	{
		const SdbColumn& rColumn = *(*aColumns)[i];
		char  cTyp;

		if (aStatus.IsError())
			break;

		if (rColumn.GetName().Len() > nMaxFieldLength)
		{
			String aText = String(SdbResId(STR_DBF_INVALIDFIELDNAMELENGTH));
			aText.SearchAndReplace(String::CreateFromAscii("#"),rColumn.GetName());
			aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(aText),
						0, String() );
					break;
		}

		ByteString aCol(rColumn.GetName(),gsl_getSystemTextEncoding());
		TRY_CREATETABLE(aFileStream << aCol.GetBuffer(),TABLE);                                                                   // FeldName
		TRY_CREATETABLE(aFileStream.Write(aBuffer, 11 - aCol.Len()),TABLE);

		switch (rColumn.GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				cTyp = 'C';
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_REAL:
			case SDB_DBTYPE_DOUBLE:
				cTyp = 'N';                             // nur dBase 3 format
				break;
			case SDB_DBTYPE_DATE:
				cTyp = 'D';
				break;
			case SDB_DBTYPE_BOOLEAN:
				cTyp = 'L';
				break;
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_LONGVARCHAR:
				cTyp = 'M';
				break;
			default:
				{
					aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_DBF_INVALIDFORMAT))),
						0, String() );
					break;
				}

		};

		TRY_CREATETABLE(aFileStream << cTyp,TABLE);
		TRY_CREATETABLE(aFileStream.Write(aBuffer, 4),TABLE);

		switch(cTyp)
		{
			case 'C':
				DBG_ASSERT(rColumn.GetLength() < 255, "SdbDBFTable::Create: Column zu lang!");
				if (rColumn.GetLength() > 254)
				{
					String aText = String(SdbResId(STR_DBF_INVALIDFIELDLENGTH));
					aText.SearchAndReplace(String::CreateFromAscii("#"),rColumn.GetName());
					aStatus.Set(SDB_STAT_ERROR,
								String::CreateFromAscii("01000"),
								aStatus.CreateErrorMessage(aText),
								0, String() );
					break;
				}
				TRY_CREATETABLE(aFileStream << (BYTE) min(rColumn.GetLength(), 255UL),TABLE);      //Feldlnge
				nRecLength += (USHORT)min(rColumn.GetLength(), 255UL);
				TRY_CREATETABLE(aFileStream << (BYTE)0,TABLE);                                                                //Nachkommastellen
				break;
			case 'F':
			case 'N':
				DBG_ASSERT(rColumn.GetLength() >=  rColumn.GetScale(),
						   "SdbDBFTable::Create: Feldlnge mu grer Nachkommastellen sein!");
				if (rColumn.GetLength() <  rColumn.GetScale())
				{
					aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_DBF_INVALIDFORMAT))),
						0, String() );
					break;
				}
				if (rColumn.GetType() == SDB_DBTYPE_CURRENCY) // Currency wird gesondert behandelt
				{
					TRY_CREATETABLE(aFileStream << (BYTE)10,TABLE);          // Standard Laenge
					TRY_CREATETABLE(aFileStream << (BYTE)4,TABLE);
					nRecLength += 10;
				}
				else
				{
					UINT16 nPrecision = SvDbaseConverter::ConvertPrecisionToDbase(rColumn.GetLength(),rColumn.GetScale());

					TRY_CREATETABLE(aFileStream << (BYTE)( nPrecision),TABLE);
					TRY_CREATETABLE(aFileStream << (BYTE)rColumn.GetScale(),TABLE);
					nRecLength += (USHORT)nPrecision;
				}
				break;
			case 'L':
				TRY_CREATETABLE(aFileStream << (BYTE)1,TABLE);
				TRY_CREATETABLE(aFileStream << (BYTE)0,TABLE);
				nRecLength++;
				break;
			case 'D':
				TRY_CREATETABLE(aFileStream << (BYTE)8,TABLE);
				TRY_CREATETABLE(aFileStream << (BYTE)0,TABLE);
				nRecLength += 8;
				break;
			case 'M':
				bCreateMemo = TRUE;
				TRY_CREATETABLE(aFileStream << (BYTE)10,TABLE);
				TRY_CREATETABLE(aFileStream << (BYTE)0,TABLE);
				nRecLength += 10;
				break;
			default:
				aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_DBF_INVALIDFORMAT))),
						0, String() );
				break;
		}
		TRY_CREATETABLE(aFileStream.Write(aBuffer, 14),TABLE);
	}

	if (aStatus.IsError())
		return FALSE;

	TRY_CREATETABLE(aFileStream << (BYTE)0x0d,TABLE);                                     // kopf ende
	TRY_CREATETABLE(aFileStream.Seek(10L),TABLE);
	TRY_CREATETABLE(aFileStream << nRecLength,TABLE);                                     // satzlnge nachtrglich eintragen

	if (bCreateMemo)
	{
		TRY_CREATETABLE(aFileStream.Seek(0L),TABLE);
		TRY_CREATETABLE(aFileStream << (BYTE) dBaseIIIMemo,TABLE);
	}
	return TRUE;
}

//------------------------------------------------------------------
// erzeugt grundstzlich dBase III Datei Format
BOOL SdbDBFTable::CreateMemoFile(const DirEntry& aFile)
{
	// Makro zum Filehandling frs Erzeugen von Tabellen
#define TRY_CREATEMEMO(action,type)\
	action;\
	if (aStatus.SetError(aMemoFileStream,type))\
	{\
		aMemoFileStream.ResetError();\
		aMemoFileStream.Close();\
		aFile.Kill();\
		return FALSE;\
	}
	aMemoFileStream.Open(aFile.GetFull(), STREAM_READWRITE | STREAM_SHARE_DENYWRITE);
	if (aStatus.SetError(aMemoFileStream,MEMO))
		return FALSE;

	char aBuffer[512];              // write buffer
	memset(aBuffer,0,sizeof(aBuffer));

#ifdef WIN
	TRY_CREATEMEMO(aMemoFileStream.Seek(0L),MEMO)
	for (UINT16 i = 0; i < 512; i++)
	{
		TRY_CREATEMEMO(aMemoFileStream << BYTE(0),MEMO);
	}
#else
	aMemoFileStream.SetFiller('\0');
	TRY_CREATEMEMO(aMemoFileStream.SetStreamSize(512),MEMO);
#endif

	TRY_CREATEMEMO(aMemoFileStream.Seek(0L),MEMO)
	TRY_CREATEMEMO(aMemoFileStream << long(1),MEMO)                  // Zeiger auf ersten freien Block

	aMemoFileStream.Close();
	return TRUE;
}


//------------------------------------------------------------------
// erzeugt grundstzlich dBase IV Datei Format
BOOL SdbDBFTable::AlterImpl()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);

	NAMESPACE_VOS(OGuard) aGuard(m_pLock);

	if (InUse())
	{
		aStatus.SetError(ERRCODE_IO_LOCKVIOLATION,TABLE,aName);
		return FALSE;
	}

	// merkt sich die Spaltenpositionen aus der Originaltabelle von den Spalten,
	// der Daten uebernommen werden
	USHORT nColPos[128];
	BOOL bIndexed = FALSE;
	memset(nColPos, 0, sizeof(nColPos));

	SdbFILECursor* pCursor = (SdbFILECursor*) GetConnection()->CreateCursor(SDB_KEYSET, SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();
	// geloeschte Saetze auch uebertragen
//	pCursor->ShowDeleted(TRUE);
	if (!pCursor->Open(aName, TRUE))
	{
		aStatus = pCursor->Status();
		pCursor->ReleaseRef();
		return FALSE;
	}

	for (USHORT i = 1; i < aColumns->Count(); i++)
	{
		// geaendert wird nach folgender Strategie
		// a.) Column identisch -> uebernehmen
		// b.) Column mit gleichem Namen existiert -> uebernehmen (wenn die Type vertraeglich)
		// c.) Column identisch bis auf Namen -> uebernehmen (Rename) // dieses ist nur eine Annaeherung!

		// doch zunaechst die Spalten in SdbFILEColumns umwandeln
		SdbFILEColumn* pColumn = (SdbFILEColumn*)(*aColumns)[i];
		// a.)
		const SdbFILEColumn* pRefColumn = (const SdbFILEColumn*)(*aOriginalColumns)[i];
		if (pRefColumn && pColumn->Compare(*pRefColumn, SDB_FLAGS_NULLALLOWED))
		{
			nColPos[i-1] = pRefColumn->GetId();
			// ist die Spalte indexiert, dann einfach uebernehmen
			if (pRefColumn->IsIndexed())
			{
				// sperren der Tabelle, damit der Index erhalten bleibt
				pColumn->SetIndex(pRefColumn->GetIndex());
				pColumn->GetIndex()->Lock(TRUE);
				bIndexed = TRUE;
			}
		}
		else
			pRefColumn = (const SdbFILEColumn*)(*aOriginalColumns)[pColumn->GetName()];

		// b.)
		if (!nColPos[i-1] && pRefColumn)
		{
			//bei gleichen Type kein Problem
			if (pColumn->GetType() == pRefColumn->GetType())
			{
				nColPos[i-1] = pRefColumn->GetId();
			}
			else
			{
				// ueberpruefen ob Typen kompatibel
				switch (pColumn->GetType())
				{
					case SDB_DBTYPE_TINYINT:
					case SDB_DBTYPE_SMALLINT:
					case SDB_DBTYPE_INTEGER:
					case SDB_DBTYPE_CURRENCY:
					case SDB_DBTYPE_BIGINT:
					case SDB_DBTYPE_DECIMAL:
					case SDB_DBTYPE_NUMERIC:
					case SDB_DBTYPE_REAL:
					case SDB_DBTYPE_DOUBLE:
						switch (pRefColumn->GetType())
						{
							case SDB_DBTYPE_CHAR:
							case SDB_DBTYPE_VARCHAR:
							case SDB_DBTYPE_LONGVARBINARY:
							case SDB_DBTYPE_LONGVARCHAR:
							case SDB_DBTYPE_DATE:
								break;
							default:
								nColPos[i-1] = pRefColumn->GetId();
						} break;
					case SDB_DBTYPE_DATE:
						switch (pRefColumn->GetType())
						{
							case SDB_DBTYPE_CHAR:
							case SDB_DBTYPE_VARCHAR:
							case SDB_DBTYPE_LONGVARBINARY:
							case SDB_DBTYPE_LONGVARCHAR:
							case SDB_DBTYPE_TINYINT:
							case SDB_DBTYPE_SMALLINT:
							case SDB_DBTYPE_INTEGER:
							case SDB_DBTYPE_CURRENCY:
							case SDB_DBTYPE_BIGINT:
							case SDB_DBTYPE_DECIMAL:
							case SDB_DBTYPE_NUMERIC:
							case SDB_DBTYPE_REAL:
							case SDB_DBTYPE_DOUBLE:
							case SDB_DBTYPE_BOOLEAN:
								break;
							default:
								nColPos[i-1] = pRefColumn->GetId();
						} break;
					case SDB_DBTYPE_BOOLEAN:
						switch (pRefColumn->GetType())
						{
							case SDB_DBTYPE_CHAR:
							case SDB_DBTYPE_VARCHAR:
							case SDB_DBTYPE_LONGVARBINARY:
							case SDB_DBTYPE_LONGVARCHAR:
							case SDB_DBTYPE_DATE:
								break;
							default:
								nColPos[i-1] = pRefColumn->GetId();
						} break;
					case SDB_DBTYPE_LONGVARCHAR:
						switch (pRefColumn->GetType())
						{
							case SDB_DBTYPE_CHAR:
							case SDB_DBTYPE_VARCHAR:
								nColPos[i-1] = pRefColumn->GetId();
						} break;
					case SDB_DBTYPE_CHAR:
					case SDB_DBTYPE_VARCHAR:
						nColPos[i-1] = pRefColumn->GetId();
				}
			}
		}

		// ist eine Zuweisung erfolgt, mu noch ueberprueft werden
		// ob fuer die zugewiesene Spalte bereits vorher eine zuweisung (c.)
		// stattgefunden hat
		if (nColPos[i-1])
		{
			for (USHORT j = 0; j < (i - 1); j++)
				if (nColPos[j] == nColPos[i-1])
					nColPos[j] = 0;
		}
		else
		{
			// c.)
			pRefColumn = (const SdbFILEColumn*)(*aOriginalColumns)[i];
			if (pRefColumn && pColumn->GetType() == pRefColumn->GetType())
			{
				nColPos[i-1] = pRefColumn->GetId();
			}
		}
	}

	// OriginalFile
	DirEntry aOrig(GetFILEConnection()->FileEntry(aName));
	DirEntry aMask(aOrig.GetPath());
	aMask.SetExtension( String::CreateFromAscii(".dbf") );
	DirEntry aTemp(aMask.TempName());
	aTemp.Kill();

	// neue Tabelle anlegen
	SdbDBFTable* pNewTable = new SdbDBFTable(GetDBFConnection());
	if (!pNewTable->Create(aTemp.GetBase()))
	{
		pCursor->ReleaseRef();
		delete pNewTable;
		return FALSE;
	}

	// Spalten uebertragen
	pNewTable->aColumns = aColumns;

	if (!(pNewTable->CreateImpl() &&
		  pNewTable->FileOpen(FALSE)))
	{
		pCursor->ReleaseRef();
		delete pNewTable;
		return FALSE;
	}
	else
		pNewTable->aOriginalColumns = pNewTable->aColumns; // Originalcolumns wichtig fuer die Datenuebertragung

	ODbRowRef xRow = new ODbRow(*pNewTable->aColumns);
	USHORT nCount = pNewTable->aColumns->Count();
	// jetzt die Daten uebertragen
	while (pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		for (USHORT i = 1; i < nCount; i++)
		{
			if (!nColPos[i - 1])
				continue;
			(*(*xRow)[i]) = *rRow[nColPos[i - 1]];
		}

		// nichts kopiert und gleiche Spaltenanzahl
		// dann bleibt die neue Tabelle leer

		if (!pNewTable->InsertRow(xRow, FALSE))
		{
			aStatus = pNewTable->Status();
			pCursor->ReleaseRef();

			pNewTable->DropImpl();
			delete pNewTable;

			return FALSE;
		}

		// nichts kopiert und gleiche Spaltenanzahl
		// dann bleibt die neue Tabelle leer
		pCursor->Next();
	}
	pCursor->ReleaseRef();

	if (!DropImpl())
	{
		// Temporaere loeschen
		pNewTable->DropImpl();
		delete pNewTable;
		return FALSE;
	}

	// und die neue an die Stelle setzen
	pNewTable->RenameImpl(aName);

	// nun noch ein Inf file erstellen, falls Indices bestehen
	if (bIndexed)
	{
		for (USHORT i = 1; i < aColumns->Count(); i++)
		{
			SdbFILEColumn* pColumn = (SdbFILEColumn*)(*aColumns)[i];
			if (pColumn->IsIndexed() && pColumn->IsIndexed())
			{
				SdbNDXIndex* pIndex = (SdbNDXIndex*)pColumn->GetIndex();
				pIndex->UnLock();
				pIndex->CreateINFEntry();
			}
		}
	}

	delete pNewTable;
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbDBFTable::DropImpl()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);

	NAMESPACE_VOS(OGuard) aGuard(m_pLock);

	if (InUse())
	{
		aStatus.SetError(ERRCODE_IO_LOCKVIOLATION,TABLE,aName);
		return FALSE;
	}

	FileClose();

	DirEntry aFile(GetFILEConnection()->FileEntry(aName));
	if (aStatus.SetError(aFile.Kill(),TABLE,aName))
		return FALSE;

	if (HasMemoFields())
	{
		String aExt = aFile.GetExtension();
		aFile.SetExtension(String::CreateFromAscii("dbt"));                      // extension for memo file
		if (aFile.Exists())
			aFile.Kill();
	}

	// jetzt noch die Indices loeschen
	String aIndexName;
	aFile.SetExtension(String::CreateFromAscii("ndx"));
	USHORT nCount = GetIndexCount(),
		   i	  = 0;
	while (i < nCount)
	{
		SdbIndex* pIndex = OpenIndex(GetIndexName(i++));
		if(pIndex)
			pIndex->Drop();
		/*
		aFile.SetBase(GetIndexName(i++));
		FSysError aError;
		if(aFile.Exists())
			aError = aFile.Kill();
		*/
	}
	aFile.SetBase(aName);
	aFile.SetExtension(String::CreateFromAscii("inf"));
	aFile.Kill();
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbDBFTable::RenameImpl(const String& rNewName)
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	NAMESPACE_VOS(OGuard) aGuard(m_pLock);

	CloseImpl();

	DirEntry aOldFile(GetFILEConnection()->FileEntry(aName));
	DirEntry aMemo = aOldFile;
	DirEntry aNewFile(GetFILEConnection()->FileEntry(rNewName));

	ULONG nResult = aOldFile.MoveTo(aNewFile);
	BOOL bIsDestinationError = (nResult == ERRCODE_IO_ALREADYEXISTS) || (nResult == ERRCODE_IO_CANTCREATE)
		|| (nResult == ERRCODE_IO_CANTWRITE) || (nResult == ERRCODE_IO_OUTOFSPACE);
		// zumindest fuer diese FehlerCodes kann ich definitiv sagen, dass sie das Zielfile betreffen ... es gibt natuerlich noch
		// andere Codes, die sowohl fuer Quelle als auch fuer Ziel zutreffen koennen, die kann ich leider hier nicht unterscheiden ...
	if (aStatus.SetError(nResult, TABLE, bIsDestinationError ? rNewName : aName))
	{
		// da das nicht geklappt hat, muss ich die alte Tabelle wieder oeffnen mittels FileOpen
		// leider geht dabei der Fehlerstatus verloren, also merke ich ihn mir
		SdbStatus statRenameError(aStatus);
		OpenImpl();
		aStatus = statRenameError;
		return FALSE;
	}
	else
	{
		aOldFile.SetExtension(String::CreateFromAscii("inf"));
		aNewFile.SetExtension(String::CreateFromAscii("inf"));
		aOldFile.MoveTo(aNewFile);

		if (HasMemoFields())
		{
			aMemo.SetExtension(String::CreateFromAscii("dbt"));                      // extension for memo file
			aNewFile.SetExtension(String::CreateFromAscii("dbt"));           // extension for memo file
			aMemo.MoveTo(aNewFile);                         // das muss klappen
		}
		return TRUE;
	}
}

//------------------------------------------------------------------
BOOL SdbDBFTable::OpenImpl()
{
	// nehmen wir als Privilegien erst mal das Minimum
	nPrivileges = SDB_PR_READ;

	if (!SdbFILETable::OpenImpl())
	{
		nPrivileges = SDB_PR_NONE;	// nicht dass es was ausmachen wuerde, aber sicher ist sicher
		return FALSE;
	}

	AllocBuffer();
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbDBFTable::FileOpen(BOOL _bReadOnly)
{
	DirEntry aFileEntry = GetFILEConnection()->FileEntry(aName);
	if (!_bReadOnly && (FileStat::GetReadOnlyFlag(aFileEntry)))
		return FALSE;

	StreamMode eMode = (_bReadOnly) ? STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYNONE :
									 STREAM_READWRITE | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE;

	// Sicherheitshalber erst ein Close (ohne Fehlerabfrage) ...
	FileClose();
	aStatus.Clear();

	aFileStream.Open(aFileEntry.GetFull(), eMode);
	if (aStatus.SetError(aFileStream,TABLE))
		return FALSE;

	// Dummerweise kann der Stream, wenn ich versuche, ihn schreibend zu oeffnen, ohne Fehlermeldung einfach
	// implizit auf read-only schalten. In dem Stream selber kann man das leider nicht ohne weiteres aendern,
	// da wohl schon ziemlich viel auf diesem Verhalten aufbaut. Also eben hier abfangen.
	// FS - 14.04.99 - #64752#
	if (!_bReadOnly && !aFileStream.IsWritable())
	{
		aStatus.SetError(ERRCODE_IO_ACCESSDENIED, TABLE, aFileStream.GetFileName());
		return FALSE;
	}

	if (!ReadHeader())
	{
		FileClose();
		return FALSE;
	}

	if (HasMemoFields())
	{
		// Memo-Dateinamen bilden (.DBT):
		// nyi: Unschoen fuer Unix und Mac!

		if (aFileHeader.db_typ == FoxProMemo) // foxpro verwendet andere extension
			aFileEntry.SetExtension(String::CreateFromAscii("fpt"));  // nyi: Gross-/Kleinschreibung bei Unix? Klein ist sicherlich schoener.
		else
			aFileEntry.SetExtension(String::CreateFromAscii("dbt"));  // nyi: Gross-/Kleinschreibung bei Unix? Klein ist sicherlich schoener.

		// Wenn die Memodatei nicht gefunden wird, werden die Daten trotzdem angezeigt
		// allerdings koennen keine Updates durchgefuehrt werden
		// jedoch die Operation wird ausgefuehrt
		aMemoFileStream.Open(aFileEntry.GetFull(), eMode);
		if (!aStatus.SetError(aMemoFileStream,MEMO))
			ReadMemoHeader();

		if (aStatus.IsError())
		{
			String aText = String(SdbResId(STR_STAT_FILE_NOT_FOUND));
			aText.SearchAndReplace(String::CreateFromAscii("%%d"),aFileEntry.GetName());
			aText.SearchAndReplace(String::CreateFromAscii("%%t"),aStatus.TypeToString(MEMO));
			aStatus.Set(SDB_STAT_SUCCESS_WITH_INFO,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(aText),
						0, String() );
			aMemoFileStream.Close();
		}
	}
	if (aStatus.IsError())
		FileClose();

	// einen Puffer abhaengig von der Filegroesse anlegen
	TRY_FILE(aFileStream.Seek(STREAM_SEEK_TO_END),TABLE);
	UINT32 nFileSize = aFileStream.Tell();
	TRY_FILE(aFileStream.Seek(STREAM_SEEK_TO_BEGIN),TABLE);

	// Buffersize abhaengig von der Filegroesse
	aFileStream.SetBufferSize(nFileSize > 1000000 ? 32768 :
							  nFileSize > 100000 ? 16384 :
							  nFileSize > 10000 ? 4096 : 1024);

	if (IsMemoAvailable())
	{
		// Puffer genau auf Laenge eines Satzes stellen
		TRY_FILE(aMemoFileStream.Seek(STREAM_SEEK_TO_END),MEMO);
		nFileSize = aMemoFileStream.Tell();
		TRY_FILE(aMemoFileStream.Seek(STREAM_SEEK_TO_BEGIN),MEMO);

		// Buffersize abhaengig von der Filegroesse
		aMemoFileStream.SetBufferSize(nFileSize > 1000000 ? 32768 :
									  nFileSize > 100000 ? 16384 :
									  nFileSize > 10000 ? 4096 :
									  aMemoHeader.db_size);
	}

	// koennen die Indices geffnet werden
	for (USHORT i = 1; i < aOriginalColumns->Count(); i++)
	{
		SdbFILEColumn* pColumn = (SdbFILEColumn*)aOriginalColumns->Column(i);
		if (pColumn->IsIndexed() && !pColumn->GetIndex()->Lock(bReadOnly))
		{
			aStatus = pColumn->GetIndex()->Status();
			FileClose();
			break;
		}
	}

	nFilePos = 0;

	// da ich jetzt einen FileStream habe, kann ich je nach dessen Zustand meine Privilegien setzen
	if (!aStatus.IsSuccessful() || !aFileStream.IsWritable())
		nPrivileges = SDB_PR_READ;
	else
	{
		// ansonsten darf ich alles
		nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_ALTER | SDB_PR_DROP;
		if (!GetFILEConnection()->GetShowDeleted())
			nPrivileges |= SDB_PR_DELETE;
		// wenn man geloeschte anzeigt, kann man nicht loeschen ....
	}

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbDBFTable::FileClose()
{
	// falls noch nicht alles geschrieben wurde
	if (aMemoFileStream.IsOpen() && aMemoFileStream.IsWritable())
		aMemoFileStream.Flush();

	aMemoFileStream.Close();

	if (aFileStream.IsOpen() && aFileStream.IsWritable())
		aFileStream.Flush();

	aFileStream.Close();

	if (pBuffer != NULL)
	{
		delete pBuffer;
		pBuffer = NULL;
	}
}

//------------------------------------------------------------------
BOOL SdbDBFTable::IsFileOpen() const
{
	return aFileStream.IsOpen();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::ReadHeader()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	DBG_ASSERT(aFileStream.IsOpen(), "DBF: File not open");

	TRY_FILE(aFileStream.RefreshBuffer(),TABLE); // sicherstellen, dass die Kopfinformationen tatsaechlich neu gelesen werden
	TRY_FILE(aFileStream.Seek(STREAM_SEEK_TO_BEGIN),TABLE);

	BYTE aTyp;
	TRY_FILE(aFileStream >> aTyp,TABLE);
	TRY_FILE(aFileStream.Read((char*)aFileHeader.db_aedat, 3*sizeof(BYTE)),TABLE);
	TRY_FILE(aFileStream >> aFileHeader.db_anz,TABLE);
	TRY_FILE(aFileStream >> aFileHeader.db_kopf,TABLE);
	TRY_FILE(aFileStream >> aFileHeader.db_slng,TABLE);
	TRY_FILE(aFileStream.Read((char*)aFileHeader.db_frei, 20*sizeof(BYTE)),TABLE);

	if (aFileHeader.db_anz  < 0 ||
		aFileHeader.db_kopf <= 0 ||
		aFileHeader.db_slng <= 0 ||
		((aFileHeader.db_kopf - 1) / 32 - 1) <= 0) // anzahl felder
	{
		// Dies ist keine DBase Datei
		String aText = String(SdbResId(STR_STAT_FILE_INVALID));
		aText.SearchAndReplace(String::CreateFromAscii("%%d"),aFileStream.GetFileName());
		aText.SearchAndReplace(String::CreateFromAscii("%%t"),aStatus.TypeToString(TABLE));
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
	}
	else
	{
		// Konsistenzpruefung des Header:
		aFileHeader.db_typ = (DBFType)aTyp;
		switch (aFileHeader.db_typ)
		{
			case dBaseIII:
			case dBaseIV:
			case dBaseV:
			case dBaseFS:
			case dBaseFSMemo:
			case dBaseIVMemoSQL:
			case dBaseIIIMemo:
			case dBaseIVMemo:
			case FoxProMemo:
				TRY_FILE(aFileStream.SetNumberFormatInt(NUMBERFORMAT_INT_LITTLEENDIAN),TABLE);
				break;
			default:
			{	// Dies ist keine DBase Datei
				String aText = String(SdbResId(STR_STAT_FILE_INVALID));
				aText.SearchAndReplace(String::CreateFromAscii("%%d"),aFileStream.GetFileName());
				aText.SearchAndReplace(String::CreateFromAscii("%%t"),aStatus.TypeToString(TABLE));
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("01000"),
							aStatus.CreateErrorMessage(aText),
							0, String() );
			}
		}
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::ReadMemoHeader()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);

	TRY_MEMOFILE(aMemoFileStream.SetNumberFormatInt(NUMBERFORMAT_INT_LITTLEENDIAN),MEMO);
	TRY_MEMOFILE(aMemoFileStream.RefreshBuffer(),MEMO);			// sicherstellen das die Kopfinformationen tatsaechlich neu gelesen werden
	TRY_MEMOFILE(aMemoFileStream.Seek(0L),MEMO);

	TRY_MEMOFILE(aMemoFileStream >> aMemoHeader.db_next,MEMO);
	switch (aFileHeader.db_typ)
	{
		case dBaseIIIMemo:  // dBase III: feste Blockgre
		case dBaseIVMemo:
			// manchmal wird aber auch dBase3 dBase4 Memo zugeordnet
			TRY_MEMOFILE(aMemoFileStream.Seek(20L),MEMO);
			TRY_MEMOFILE(aMemoFileStream >> aMemoHeader.db_size,MEMO);
			if (aMemoHeader.db_size > 1 && aMemoHeader.db_size != 512)	// 1 steht auch fuer dBase 3
				aMemoHeader.db_typ  = MemodBaseIV;
			else if (aMemoHeader.db_size > 1 && aMemoHeader.db_size == 512)
			{
				// nun gibt es noch manche Dateien, die verwenden eine Genangabe,
				// sind aber dennoch dBase Dateien
				char sHeader[4];
				aMemoFileStream.Seek(aMemoHeader.db_size);
				aMemoFileStream.Read(sHeader,4);

				if ((aMemoFileStream.GetErrorCode() != ERRCODE_NONE) || ((BYTE)sHeader[0]) != 0xFF || ((BYTE)sHeader[1]) != 0xFF || ((BYTE)sHeader[2]) != 0x08)
					aMemoHeader.db_typ  = MemodBaseIII;
				else
					aMemoHeader.db_typ  = MemodBaseIV;
			}
			else
			{
				aMemoHeader.db_typ  = MemodBaseIII;
				aMemoHeader.db_size = 512;
			}
			break;
		case FoxProMemo:
			aMemoHeader.db_typ	= MemoFoxPro;
			TRY_MEMOFILE(aMemoFileStream.Seek(6L),MEMO);
			TRY_MEMOFILE(aMemoFileStream.SetNumberFormatInt(NUMBERFORMAT_INT_BIGENDIAN),MEMO);
			TRY_MEMOFILE(aMemoFileStream >> aMemoHeader.db_size,MEMO);
	}
	return TRUE;
}

//------------------------------------------------------------------
BOOL SdbDBFTable::DescribeColumns()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);

	// Dateistruktur ermitteln und neue Strukturbeschreibung anlegen:
	DBG_ASSERT(aOriginalColumns.Is(), "keine Filestruktur angelegt");

	// Felder bereits gelesen
	if (aOriginalColumns->Count() > 1)
		return TRUE;

	TRY_FILE(aFileStream.Seek(32L),TABLE);

	SdbConnection* pConnection = GetConnection();

	// Anzahl Felder:
	ULONG nFieldCount = (aFileHeader.db_kopf - 1) / 32 - 1;

	String aStrFieldName;aStrFieldName.AssignAscii("Column");
	USHORT nFieldCnt = 0;

	// DBF-Feldbeschreibungen lesen:
	for (ULONG i = 0; i < nFieldCount; i++)
	{
		SdbDBFColumn aDBFColumn;
		TRY_FILE(aFileStream.Read((char*)&aDBFColumn, sizeof(aDBFColumn)),TABLE);

		// Info auslesen und in SdbColumn packen:
		String aColumnName((const char *)aDBFColumn.db_fnm,pConnection->GetCharacterSet());

		while (aOriginalColumns->ColumnNumber(aColumnName) != SDB_COLUMN_NOTFOUND)
			(aColumnName = aStrFieldName) += String::CreateFromInt32(++nFieldCnt);

		UINT16 nPrecision = aDBFColumn.db_flng;
		UINT16 nScale = aDBFColumn.db_dez;
		INT16  nFlags = SDB_FLAGS_NULLALLOWED;
		BOOL bSet = FALSE;
		SdbDatabaseType eType;
		switch (aDBFColumn.db_typ)
		{
			case 'C':	eType = SDB_DBTYPE_VARCHAR; break;
			case 'F':
			case 'N':
/*				if (nPrecision == 10 && nScale == 4)		// Per Definition Currency
					eType = SDB_DBTYPE_CURRENCY;
				else */
				eType = SDB_DBTYPE_DECIMAL;

				// Bei numerischen Feldern werden zwei Zeichen mehr geschrieben, als die Precision der Spaltenbeschreibung eigentlich
				// angibt, um Platz fuer das eventuelle Vorzeichen und das Komma zu haben. Das muss ich jetzt aber wieder rausrechnen.
#if SUPD > 565
				nPrecision = SvDbaseConverter::ConvertPrecisionToOdbc(nPrecision,nScale);
#else
				if (nPrecision > nScale + 2)
					nPrecision -= 2,bSet = TRUE;
				else if(!nScale)
					nPrecision -= 1,bSet = TRUE;
#endif
					// leider gilt das eben Gesagte nicht fuer aeltere Versionen ....
					;
				break;
			case 'L':	eType = SDB_DBTYPE_BOOLEAN; break;
			case 'D':	eType = SDB_DBTYPE_DATE; break;
			case 'M':	eType = SDB_DBTYPE_LONGVARCHAR;
						nPrecision = 0;
							// for memo fields the precision of aDBFColumn is 10 which marks the space the column
							// data occupies in the DBF file. But the real memo content is written into another file,
							// so the precision 10 isn't correct
							// 74219 - 16.03.00 - FS
						break;
			default:
						eType = SDB_DBTYPE_UNKNOWN;

		}

		switch (aDBFColumn.db_typ)
		{
			case 'C':
			case 'D':
			case 'L':	nFlags |= SDB_FLAGS_SEARCH_ALL; break;
			case 'F':
			case 'N':	nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
			case 'M':	nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
			default:
						eType = SDB_DBTYPE_UNKNOWN;

		}

		if (aDBFColumn.db_typ != 'M' || aMemoFileStream.IsOpen())
			nFlags |= SDB_FLAGS_UPDATEALLOWED;

		SdbColumn* pColumn = new SdbFILEColumn(aColumnName,eType,nPrecision,nScale,nFlags);

		pColumn->SetDbasePrecisionChanged(bSet);
		// Column-Beschreibung hinzufuegen.
		aOriginalColumns->AddColumn(pColumn);
	}
	// Columns-Struktur fertig.
	if (aStatus.IsSuccessful())
		ExtDescribeColumns();

	return aStatus.IsSuccessful() && aOriginalColumns->Count() > 1;
}

//------------------------------------------------------------------
void SdbDBFTable::ExtDescribeColumns()
{
	// OpenIndex erwartet, das ein die Tabelle geoeffnet ist
	// ExtDescribeColumns() wird aber erst beim Oeffnen aufgerufen
	// daher:
	BOOL bTempOpen = bOpen;
	bOpen = TRUE;

	USHORT nCount = GetIndexCount(),
		   i	  = 0;
	String aColumnName;
	while (i < nCount)
	{
		SdbFILEIndexRef xIndex = (SdbFILEIndex*) OpenIndex(GetIndexName(i++));
		if (xIndex.Is() && xIndex->OriginalColumns()->Count() == 2)
		{
			// Auslesen der indicierten Spalte
			aColumnName = xIndex->OriginalColumns()->Column(1)->GetName();
			SdbFILEColumn* pColumn = (SdbFILEColumn*)aOriginalColumns->Column(aColumnName);
			if (pColumn)
			{
				pColumn->SetIndex(&xIndex);
				if (xIndex->IsUnique())
					pColumn->SetFlag(SDB_FLAGS_UNIQUE);
			}
		}
	}
	bOpen = bTempOpen;
}

//------------------------------------------------------------------
BOOL SdbDBFTable::SeekRow(SdbFileCursorPosition eCursorPosition, INT32 nOffset, INT32& nCurPos)
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	// ----------------------------------------------------------
	// Positionierung vorbereiten:

	ULONG  nNumberOfRecords = (ULONG)aFileHeader.db_anz;
	UINT32 nTempPos = nFilePos;
	nFilePos = nCurPos;

	switch(eCursorPosition)
	{
		case FILE_NEXT:
			nFilePos++;
			break;
		case FILE_PRIOR:
			if (nFilePos > 0)
				nFilePos--;
			break;
		case FILE_FIRST:
			nFilePos = 1;
			break;
		case FILE_LAST:
			nFilePos = nNumberOfRecords;
			break;
		case FILE_RELATIVE:
			nFilePos = (((long)nFilePos) + nOffset < 0) ? 0L
							: (ULONG)(((long)nFilePos) + nOffset);
			break;
		case FILE_ABSOLUTE:
		case FILE_BOOKMARK:
			nFilePos = (ULONG)nOffset;
			break;
	}

	if (nFilePos > (INT32)nNumberOfRecords)
		nFilePos = (INT32)nNumberOfRecords + 1;

	if (nFilePos == 0 || nFilePos == (INT32)nNumberOfRecords + 1)
		goto Error;
	else
	{
		USHORT nEntryLen = aFileHeader.db_slng;

		DBG_ASSERT(nFilePos >= 1,"SdbDBFCursor::FileFetchRow: ungueltige Record-Position");
		long nPos = aFileHeader.db_kopf + (long)(nFilePos-1) * nEntryLen;

		aFileStream.Seek(nPos);
		if (aStatus.SetError(aFileStream,TABLE))
			goto Error;

		aFileStream.Read((char*)pBuffer, nEntryLen);
		if (aStatus.SetError(aFileStream,TABLE))
			goto Error;
	}
	goto End;

Error:
	switch(eCursorPosition)
	{
		case FILE_PRIOR:
		case FILE_FIRST:
			nFilePos = 0;
			break;
		case FILE_LAST:
		case FILE_NEXT:
		case FILE_ABSOLUTE:
		case FILE_RELATIVE:
			if (nOffset > 0)
				nFilePos = nNumberOfRecords + 1;
			else if (nOffset < 0)
				nFilePos = 0;
			break;
		case FILE_BOOKMARK:
			nFilePos = nTempPos;	 // vorherige Position
	}
	aStatus.Set(SDB_STAT_NO_DATA_FOUND);

End:
	nCurPos = nFilePos;
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::FetchRow(ODbRow& rRow, SdbColumns& rColumns, BOOL bRetrieveData)
{
	// Einlesen der Daten
	BOOL bIsCurRecordDeleted = ((char)pBuffer[0] == '*') ? TRUE : FALSE;
	SdbConnection* pConnection = GetConnection();

	// only read the bookmark

	// Satz als geloescht markieren
	rRow.setState(bIsCurRecordDeleted ? ROW_DELETED : ROW_CLEAN );
	rRow[0]->setINT32(nFilePos);

	if (!bRetrieveData)
		return TRUE;

	USHORT nByteOffset = 1;
	ODbVariant* pVal;
	// Felder:
	for (USHORT i = 1; i < rColumns.Count(); i++)
	{
		pVal = rRow[i].getBodyPtr();
		const SdbColumn * pColumn = rColumns[i];

		// Laengen je nach Datentyp:
		// nyi: eine zentrale Funktion, die die Laenge liefert!
		USHORT nLen = (USHORT)pColumn->GetLength();
		switch(pColumn->GetType())
		{
			case SDB_DBTYPE_DATE:		nLen = 8; break;
			case SDB_DBTYPE_DECIMAL:
				nLen = SvDbaseConverter::ConvertPrecisionToDbase(nLen,pColumn->GetScale());
				break;	// das Vorzeichen und das Komma
			case SDB_DBTYPE_BOOLEAN:	nLen = 1; break;
			case SDB_DBTYPE_LONGVARCHAR:nLen = 10; break;
			default:					break;
		}

		// Ist die Variable ueberhaupt gebunden?
		if (pVal == NULL || pColumn->GetType() == SDB_DBTYPE_UNKNOWN)
		{
			// Nein - naechstes Feld.
			nByteOffset += nLen;
			continue;
		}

		char *pData = (char *) (pBuffer + nByteOffset);

		if (pColumn->GetType() == SDB_DBTYPE_CHAR ||
			pColumn->GetType() == SDB_DBTYPE_VARCHAR)
		{
			char cLast = pData[nLen];
			pData[nLen] = 0;
			String aStr(pData,pConnection->GetCharacterSet());
			aStr.EraseTrailingChars();

			if (!aStr.Len())				// keine StringLaenge, dann NULL
				pVal->setNull();
			else
			{
				pVal->setString(aStr);
			}
			pData[nLen] = cLast;
		}
		else
		{
			// Falls Nul-Zeichen im String enthalten sind, in Blanks umwandeln!
			for (USHORT k = 0; k < nLen; k++)
			{
				if (pData[k] == '\0')
					pData[k] = ' ';
			}

			String aStr(pData, nLen);		// Spaces am Anfang und am Ende entfernen:
			aStr.EraseLeadingChars();
			aStr.EraseTrailingChars();

			if (!aStr.Len())
			{
				nByteOffset += nLen;
				pVal->setNull();	// keine Werte -> fertig
				continue;
			}

			switch (pColumn->GetType())
			{
				case SDB_DBTYPE_DATE:
				{
					if (aStr.Len() != nLen)
					{
						pVal->setNull();
						break;
					}
					sal_Int32  nYear   = aStr.Copy( 0, 4 ).ToInt32();
					sal_Int32  nMonth  = aStr.Copy( 4, 2 ).ToInt32();
					sal_Int32  nDay    = aStr.Copy( 6, 2 ).ToInt32();

					Date aDate(nDay,nMonth,nYear);
					pVal->setDouble(SdbTools::ToDouble(aDate));
				}
				break;
				case SDB_DBTYPE_DECIMAL:
					pVal->setDouble(SdbTools::ToDouble(aStr));
				break;
				case SDB_DBTYPE_BOOLEAN:
				{
					BOOL b;
					switch (* ((const char *)pData))
					{
						case 'T':
						case 'Y':
						case 'J':	b = TRUE; break;
						default: 	b = FALSE; break;
					}
					pVal->setDouble(b);
				}
				break;
				case SDB_DBTYPE_LONGVARCHAR:
				{
					long nBlockNo = aStr.ToInt32();	// Blocknummer lesen
					if (nBlockNo > 0 && IsMemoAvailable()) // Daten aus Memo-Datei lesen, nur wenn
					{
						if (!ReadMemo(nBlockNo, *pVal))
							break;
					}
					else
						pVal->setNull();
				}	break;
				default:
					DBG_ERROR("Falscher Type");
			}
		}

		if (aStatus.IsError())
			break;
		// Und weiter ...
		nByteOffset += nLen;
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::ReadMemo(ULONG nBlockNo, ODbVariant& aVariable)
{
	BOOL bIsText = TRUE;
	SdbConnection* pConnection = GetConnection();

	TRY_MEMOFILE(aMemoFileStream.Seek(nBlockNo * aMemoHeader.db_size),MEMO);
	switch (aMemoHeader.db_typ)
	{
		case MemodBaseIII: // dBase III-Memofeld, endet mit Ctrl-Z
		{
			const char cEOF = (char) 0x1a;
			ByteString aStr;
			static char aBuf[514];
			aBuf[512] = 0;			// sonst kann der Zufall uebel mitspielen
			BOOL bReady = FALSE;

			do
			{
				aMemoFileStream.Read(&aBuf,512);

				USHORT i = 0;
				while (aBuf[i] != cEOF && ++i < 512)
					;
				bReady = aBuf[i] == cEOF;

				aBuf[i] = 0;
				aStr += aBuf;

			} while (!bReady && !aMemoFileStream.IsEof() && aStr.Len() < STRING_MAXLEN);

			aVariable.setString(UniString(aStr, pConnection->GetCharacterSet()));
		} break;
		case MemoFoxPro:
		case MemodBaseIV: // dBase IV-Memofeld mit Laengenangabe
		{
			char sHeader[4];
			TRY_MEMOFILE(aMemoFileStream.Read(sHeader,4),MEMO);
			// Foxpro stores text and binary data
			if (aMemoHeader.db_typ == MemoFoxPro)
			{
				if (((BYTE)sHeader[0]) != 0 || ((BYTE)sHeader[1]) != 0 || ((BYTE)sHeader[2]) != 0)
				{
					String aText = String(SdbResId(STR_STAT_FILE_INVALID));
					aText.SearchAndReplace(String::CreateFromAscii("%%d"),aMemoFileStream.GetFileName());
					aText.SearchAndReplace(String::CreateFromAscii("%%t"),aStatus.TypeToString(MEMO));
					aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("01000"),
							aStatus.CreateErrorMessage(aText),
							0, String() );
					return FALSE;
				}

				bIsText = sHeader[3] != 0;
			}
			else if (((BYTE)sHeader[0]) != 0xFF || ((BYTE)sHeader[1]) != 0xFF || ((BYTE)sHeader[2]) != 0x08)
			{
				String aText = String(SdbResId(STR_STAT_FILE_INVALID));
				aText.SearchAndReplace(String::CreateFromAscii("%%d"),aMemoFileStream.GetFileName());
				aText.SearchAndReplace(String::CreateFromAscii("%%t"),aStatus.TypeToString(MEMO));
				aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(aText),
						0, String() );
				return FALSE;
			}

			ULONG nLength;
			TRY_MEMOFILE(aMemoFileStream >> nLength,MEMO);

			if (aMemoHeader.db_typ == MemodBaseIV)
				nLength -= 8;

			char cChar;
			if (nLength < STRING_MAXLEN && bIsText)
			{
				ByteString aStr;
				aStr.Expand(USHORT (nLength));
				TRY_MEMOFILE(aMemoFileStream.Read(aStr.AllocBuffer((USHORT)nLength),nLength),MEMO);
				aStr.ReleaseBufferAccess();
				aVariable.setString(UniString(aStr, pConnection->GetCharacterSet()));
			}
			else
			{
				::staruno::Sequence<sal_Int8> aText(nLength);
				sal_Int8* pData = aText.getArray();
				for (ULONG i = 0; i < nLength; i++)
				{
					TRY_MEMOFILE(aMemoFileStream.Read(&cChar,1),MEMO);
					(*pData++) = cChar;
				}
				aVariable.setBytes(aText);
			}
		}
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::InsertRow(ODbRowRef& rRow, BOOL bFlush)
{
	DBG_CHKTHIS(SdbDBFTable,NULL);

	// Buffer mit Leerzeichen fllen
	AllocBuffer();
	memset(pBuffer, ' ', aFileHeader.db_slng);

	// Gesamte neue Row uebernehmen:
	// ... und am Ende als neuen Record hinzufuegen:
	UINT32 nTempPos = nFilePos,
		   nFileSize,
		   nMemoFileSize;

	nFilePos = (ULONG)aFileHeader.db_anz + 1;
	if (!UpdateBuffer(rRow))
	{
		nFilePos = nTempPos;
		return FALSE;
	}

	TRY_FILE(aFileStream.Seek(STREAM_SEEK_TO_END),TABLE);
	nFileSize = aFileStream.Tell();

	if (HasMemoFields() && IsMemoAvailable())
	{
		TRY_MEMOFILE(aMemoFileStream.Seek(STREAM_SEEK_TO_END),MEMO);
		nMemoFileSize = aMemoFileStream.Tell();
	}

	if (!WriteBuffer())
	{
		aFileStream.SetStreamSize(nFileSize);				// alte Gre restaurieren

		if (HasMemoFields() && IsMemoAvailable())
			aMemoFileStream.SetStreamSize(nMemoFileSize);	// alte Gre restaurieren
		nFilePos = nTempPos;								// Fileposition restaurieren
	}
	else
	{
		// Anzahl Datensaetze im Header erhoehen:
		TRY_FILE(aFileStream.Seek( 4L ),TABLE);
		TRY_FILE(aFileStream << (aFileHeader.db_anz + 1),TABLE);

		// beim AppendOnly kein Flush!
		if (bFlush)
			TRY_FILE(aFileStream.Flush(),TABLE);

		// bei Erfolg # erhhen
		aFileHeader.db_anz++;
		(*rRow)[0]->setINT32(nFilePos);							    // BOOKmark setzen
		nFilePos = nTempPos;
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::UpdateRow(const ODbRowRef& rRow, ODbRow* pOrgRow)
{
	DBG_CHKTHIS(SdbDBFTable,NULL);

	// Buffer mit Leerzeichen fllen
	AllocBuffer();

	// Auf gewuenschten Record positionieren:
	long nPos = aFileHeader.db_kopf + (long)(nFilePos-1) * aFileHeader.db_slng;
	TRY_FILE(aFileStream.Seek(nPos),TABLE);
	TRY_FILE(aFileStream.Read((char*)pBuffer, aFileHeader.db_slng),TABLE);

	UINT32 nMemoFileSize;
	if (HasMemoFields() && IsMemoAvailable())
	{
		TRY_MEMOFILE(aMemoFileStream.Seek(STREAM_SEEK_TO_END),MEMO);
		nMemoFileSize = aMemoFileStream.Tell();
	}
	if (!UpdateBuffer(rRow, pOrgRow) || !WriteBuffer())
	{
		if (HasMemoFields() && IsMemoAvailable())
			aMemoFileStream.SetStreamSize(nMemoFileSize);	// alte Gre restaurieren
	}
	else
	{
		TRY_FILE(aFileStream.Flush(),TABLE);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::DeleteRow()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	// Einfach das Loesch-Flag setzen (egal, ob es schon gesetzt war
	// oder nicht):
	// Auf gewuenschten Record positionieren:
	long nPos = aFileHeader.db_kopf + (long)(nFilePos-1) * aFileHeader.db_slng;
	TRY_FILE(aFileStream.Seek(nPos),TABLE);

	ODbRowRef aRow = new ODbRow(*aOriginalColumns);

	if (!FetchRow(*aRow,*aOriginalColumns,TRUE))
		return FALSE;

	for (USHORT i = 1; i < aOriginalColumns->Count(); i++)
	{
		const SdbFILEColumn *pColumn = (const SdbFILEColumn *)(*aOriginalColumns)[i];
		if (pColumn->IsIndexed())
		{
			UINT16 nPos = aRow->getColumns()->GetColumnNumberByRealName(pColumn->GetName());
			if (nPos == SDB_COLUMN_NOTFOUND)
				continue;
			pColumn->GetIndex()->Delete(nFilePos,*(*aRow)[nPos]);
		}
	}

	TRY_FILE(aFileStream.Seek(nPos),TABLE);
	TRY_FILE(aFileStream << (BYTE)'*',TABLE);
	TRY_FILE(aFileStream.Flush(),TABLE);
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::WriteMemo(ODbVariant& aVariable, ULONG& rBlockNr)
{
	// wird die BlockNr 0 vorgegeben, wird der block ans Ende gehaengt
	char cChar = 0;
	BOOL bIsText = TRUE;
	SdbConnection* pConnection = GetConnection();

	ULONG nSize	= 0;
	ULONG nStreamSize;
	BYTE nHeader[4];

	ByteString aStr;
	::staruno::Sequence<sal_Int8>* pData = NULL;
	if (aVariable.getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
	{
		pData = (::staruno::Sequence<sal_Int8>*)aVariable.get();
		nSize = pData->getLength();
	}
	else
	{
		aStr = ByteString(aVariable.toString().getStr(), pConnection->GetCharacterSet());
		nSize = aStr.Len();
	}

	// Anhaengen oder ueberschreiben
	BOOL bAppend = rBlockNr == 0;

	if (!bAppend)
	{
		switch (aMemoHeader.db_typ)
		{
			case MemodBaseIII: // dBase III-Memofeld, endet mit 2 * Ctrl-Z
				bAppend = nSize > (512 - 2);
				break;
			case MemoFoxPro:
			case MemodBaseIV: // dBase IV-Memofeld mit Laengenangabe
			{
				char sHeader[4];
				TRY_MEMOFILE(aMemoFileStream.Seek(rBlockNr * aMemoHeader.db_size),MEMO);
				TRY_MEMOFILE(aMemoFileStream.SeekRel(4L),MEMO);
				TRY_MEMOFILE(aMemoFileStream.Read(sHeader,4),MEMO);

				ULONG nOldSize;
				if (aMemoHeader.db_typ == MemoFoxPro)
					nOldSize = ((((unsigned char)sHeader[0]) * 256 +
								 (unsigned char)sHeader[1]) * 256 +
								 (unsigned char)sHeader[2]) * 256 +
								 (unsigned char)sHeader[3];
				else
					nOldSize = ((((unsigned char)sHeader[3]) * 256 +
								 (unsigned char)sHeader[2]) * 256 +
								 (unsigned char)sHeader[1]) * 256 +
								 (unsigned char)sHeader[0]  - 8;

				// passt die neue Laenge in die belegten Bloecke
				ULONG nUsedBlocks = ((nSize + 8) / aMemoHeader.db_size) + (((nSize + 8) % aMemoHeader.db_size > 0) ? 1 : 0),
					  nOldUsedBlocks = ((nOldSize + 8) / aMemoHeader.db_size) + (((nOldSize + 8) % aMemoHeader.db_size > 0) ? 1 : 0);
				bAppend = nUsedBlocks > nOldUsedBlocks;
			}
		}
	}

	if (bAppend)
	{
		ULONG nStreamSize;
		TRY_MEMOFILE(nStreamSize = aMemoFileStream.Seek(STREAM_SEEK_TO_END),MEMO);
		// letzten block auffuellen
		rBlockNr = (nStreamSize / aMemoHeader.db_size) + ((nStreamSize % aMemoHeader.db_size) > 0 ? 1 : 0);

		TRY_MEMOFILE(aMemoFileStream.SetStreamSize(rBlockNr * aMemoHeader.db_size),MEMO);
		TRY_MEMOFILE(aMemoFileStream.Seek(STREAM_SEEK_TO_END),MEMO);
	}
	else
	{
		TRY_MEMOFILE(aMemoFileStream.Seek(rBlockNr * aMemoHeader.db_size),MEMO);
	}

	switch (aMemoHeader.db_typ)
	{
		case MemodBaseIII: // dBase III-Memofeld, endet mit Ctrl-Z
		{
			const char cEOF = (char) 0x1a;
			nSize++;

			if (pData)
			{
				TRY_MEMOFILE(aMemoFileStream.Write((const char*) pData->getConstArray(), pData->getLength()),MEMO);
			}
			else
			{
				TRY_MEMOFILE(aMemoFileStream.Write(aStr.GetBuffer(), aStr.Len()),MEMO);
			}

			TRY_MEMOFILE(aMemoFileStream << cEOF << cEOF,MEMO);
		} break;
		case MemoFoxPro:
		case MemodBaseIV: // dBase IV-Memofeld mit Laengenangabe
		{
			TRY_MEMOFILE(aMemoFileStream << (BYTE)0xFF
										 << (BYTE)0xFF
										 << (BYTE)0x08,MEMO);

			UINT32 nWriteSize = nSize;
			if (aMemoHeader.db_typ == MemoFoxPro)
			{
				TRY_MEMOFILE(aMemoFileStream << (BYTE) ((pData = NULL) ? 0x01 : 0x00),MEMO);
				for (int i = 4; i > 0; nWriteSize >>= 8)
					nHeader[--i] = (BYTE) (nWriteSize % 256);
			}
			else
			{
				TRY_MEMOFILE(aMemoFileStream << (BYTE) 0x00,MEMO);
				nWriteSize += 8;
				for (int i = 0; i < 4; nWriteSize >>= 8)
					nHeader[i++] = (BYTE) (nWriteSize % 256);
			}

			TRY_MEMOFILE(aMemoFileStream.Write(nHeader,4),MEMO);
			if (pData)
			{
				TRY_MEMOFILE(aMemoFileStream.Write((const char*) pData->getConstArray(), pData->getLength()),MEMO);
			}
			else
			{
				TRY_MEMOFILE(aMemoFileStream.Write(aStr.GetBuffer(), aStr.Len()),MEMO);
			}
			TRY_MEMOFILE(aMemoFileStream.Flush(),MEMO);
		}
	}


	// Schreiben der neuen Blocknummer
	if (bAppend)
	{
		TRY_MEMOFILE(nStreamSize = aMemoFileStream.Seek(STREAM_SEEK_TO_END),MEMO);
		aMemoHeader.db_next = (nStreamSize / aMemoHeader.db_size) + ((nStreamSize % aMemoHeader.db_size) > 0 ? 1 : 0);

		// Schreiben der neuen Blocknummer
		TRY_MEMOFILE(aMemoFileStream.Seek(0L),MEMO);
		TRY_MEMOFILE(aMemoFileStream << aMemoHeader.db_next,MEMO);
		TRY_MEMOFILE(aMemoFileStream.Flush(),MEMO);
	}
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbDBFTable::AllocBuffer()
{
	UINT16 nSize = aFileHeader.db_slng;
	DBG_ASSERT(nSize > 0, "Size too small");

	if (nBufferSize != nSize)
	{
		delete pBuffer;
		pBuffer = NULL;
	}

	// Falls noch kein Puffer vorhanden: allozieren:
	if (pBuffer == NULL && nSize)
	{
		nBufferSize = nSize;
		pBuffer		= new BYTE[nBufferSize+1];
	}
}

//------------------------------------------------------------------
BOOL SdbDBFTable::UpdateBuffer(const ODbRowRef& rRow, ODbRow* pOrgRow)
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	SdbConnection* pConnection = GetConnection();

	USHORT nByteOffset  = 1;

	// Felder aktualisieren:
	USHORT i;
	for (i = 1; i < aOriginalColumns->Count(); i++)
	{
		const SdbFILEColumn *pColumn = (const SdbFILEColumn *)(*aOriginalColumns)[i];
		UINT16 nPos = rRow->getColumns()->GetColumnNumberByRealName(pColumn->GetName());
		if (nPos == SDB_COLUMN_NOTFOUND)
			continue;

		ODbVariant* pVal = (*rRow)[nPos].getBodyPtr();
		if (pColumn->IsUnique())
		{
			// Update !!
			if (pOrgRow && (pVal == NULL ||
							!pVal->isModified() ||
							*pVal == *(*pOrgRow)[nPos]))
				continue;
			else
			{
				ODbVariantRef xVar = (pVal == NULL) ? new ODbVariant() : pVal;
				if (pColumn->GetIndex()->Find(0,*xVar))
				{
					// es existiert kein eindeutiger Wert
					String aText = String(SdbResId(STR_VALUE_NOTUNIQUE));
					aText.SearchAndReplace(String::CreateFromAscii("#"),pColumn->GetName());
					String strDetailed = String(SdbResId(STR_DBF_DUPL_VALUE_INFO));
					strDetailed.SearchAndReplace(String::CreateFromAscii("$col$"),pColumn->GetName());
					aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("01000"),
						aStatus.CreateErrorMessage(aText),
						0, strDetailed );
					return FALSE;
				}
			}
		}
	}

	for (i = 1; i < aOriginalColumns->Count(); i++)
	{
		const SdbFILEColumn *pColumn	= (const SdbFILEColumn *)(*aOriginalColumns)[i];

		// Laengen je nach Datentyp:
		// nyi: eine zentrale Funktion, die die Laenge liefert!
		USHORT nLen = (USHORT)pColumn->GetLength();
		switch (pColumn->GetType())
		{
			case SDB_DBTYPE_DATE:		nLen = 8; break;
			case SDB_DBTYPE_DECIMAL:
#if SUPD > 565
				nLen = SvDbaseConverter::ConvertPrecisionToDbase(nLen,pColumn->GetScale());
#else
				if(pColumn->IsDbasePrecisionChanged())
				{
					if(!pColumn->GetScale())
					{
						if(nLen < 2)
							nLen++;
						else
							nLen += 2;
					}
					else
					{
						if(nLen > pColumn->GetScale())
							nLen += 2;
						else
							nLen++;
					}
				}
#endif
				break;	// das Vorzeichen und das Komma
			case SDB_DBTYPE_BOOLEAN:	nLen = 1; break;
			case SDB_DBTYPE_LONGVARCHAR:nLen = 10; break;
			default:					break;

		}

		UINT16 nPos = rRow->getColumns()->GetColumnNumberByRealName(pColumn->GetName());
		if (nPos == SDB_COLUMN_NOTFOUND)
		{
			nByteOffset += nLen;
			continue;
		}

		ODbVariant* pVal = (*rRow)[nPos].getBodyPtr();
		if (pColumn->IsIndexed())
		{
			// Update !!
			if (pOrgRow && pVal != NULL && pVal->isModified())
				pColumn->GetIndex()->Update(nFilePos,*(*pOrgRow)[nPos],*pVal);
			else
			{
				ODbVariantRef xVar = (pVal == NULL) ? new ODbVariant() : pVal;
				pColumn->GetIndex()->Insert(nFilePos,*xVar);
			}
		}

		// Ist die Variable ueberhaupt gebunden?
		if (pVal == NULL || !pVal->isModified())
		{
			// Nein - naechstes Feld.
			nByteOffset += nLen;
			continue;
		}

		char* pData = (char *)(pBuffer + nByteOffset);
		if (pVal->isNull())
		{
			memset(pData,' ',nLen);	// Zuruecksetzen auf NULL
			nByteOffset += nLen;
			continue;
		}
		try
		{
			switch (pColumn->GetType())
			{
				case SDB_DBTYPE_DATE:
				{
					Date aDate = pVal->toDate();
					char s[9];
					sprintf(s,"%04d%02d%02d",
						(int)aDate.GetYear(),
						(int)aDate.GetMonth(),
						(int)aDate.GetDay());

					// Genau 8 Byte kopieren:
					strncpy(pData,s,sizeof s - 1);
				} break;
				case SDB_DBTYPE_DECIMAL:
				{
					memset(pData,' ',nLen);	// Zuruecksetzen auf NULL

					double n = 0;
					if (!pVal->isNull())
						n = pVal->toDouble();

					int nPrecision		= (int)pColumn->GetPrecision();
					int nScale			= (int)pColumn->GetScale();
					// ein const_cast, da GetFormatPrecision am SvNumberFormat nicht const ist, obwohl es das eigentlich
					// sein koennte und muesste

					ByteString aDefaultValue(SdbTools::ToString(n, nScale),gsl_getSystemTextEncoding());
					BOOL bValidLength	= FALSE;
					if (aDefaultValue.Len() <= nLen)
					{
						strncpy(pData,aDefaultValue.GetBuffer(),nLen);
						// write the resulting double back
						pVal->setDouble(SdbTools::ToDouble(aDefaultValue));
						bValidLength = TRUE;
					}
	#if 0
					INT32 nExponent = nPrecision - (n<0 ? 1 : 0) - (nScale>0 ? 1 : 0) - nScale;
					if (n>pow(10.0, nExponent + 5))
						// das Ding ist zu gross, als dass es mitsamt dem Vorzeichen und den Nachkommastellen und dem eventuellen
						// Dezimal-Punkt in einen Buffer der Laenge nPrecision + 5 passen wuerde
						// (die 5 ist nur eine Sicherheitsreserve ;), eigentlich sollte es auch ohne gehen)
						bValidLength = FALSE;
					else
					{
						char* s = new char[nPrecision + 5];
						sprintf(s,"%*.*f", nPrecision, nScale, n);
						// wieviel Stellen hat die so formatierte Zahl wirklich ?
						int nRealLen = strlen(s);
						if (nScale > 0)
							--nRealLen;     // (das Komma)
						if ((s[0] == '-') || (s[0] == '+'))
							--nRealLen;     // (das Vorzeichen ... klar zusehen, oder ?)

						bValidLength = nRealLen <= nPrecision;

						if (bValidLength)
						{
							strncpy(pData,s,nLen);
							// da bei Formatieren durchaus Nachkommastellen abgeschnitten worden sein koennen, muss ich den
							// Wert in der Variablen entsprechen anpassen
							pVal->PutDouble(strtod(s, NULL));
						}
						delete[] s;
					}
	#endif
					if (!bValidLength)
					{
						String strError(SdbResId(STR_DBF_INVALID_FIELD_VALUE));
						strError.SearchAndReplace(String::CreateFromAscii("$name$"), pColumn->GetName());

						String strDetailedInformation(SdbResId(STR_DBF_INVALID_FIELD_VALUE_DECIMAL));
						strDetailedInformation.SearchAndReplace(String::CreateFromAscii("$name$"), pColumn->GetName());
						strDetailedInformation.SearchAndReplace(String::CreateFromAscii("#length#"), nPrecision);
						strDetailedInformation.SearchAndReplace(String::CreateFromAscii("#scale#"), nScale);
						aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, strDetailedInformation);
					}
				} break;
				case SDB_DBTYPE_BOOLEAN:
					*pData = pVal->toBOOL() ? 'T' : 'F';
					break;
				case SDB_DBTYPE_LONGVARCHAR:
				{
					char cNext = pData[nLen]; // merken und temporaer durch 0 ersetzen
					pData[nLen] = '\0';		  // das geht, da der Puffer immer ein Zeichen groesser ist ...

					ULONG nBlockNo = strtol((const char *)pData,NULL,10);	// Blocknummer lesen

					// Naechstes Anfangszeichen wieder restaurieren:
					pData[nLen] = cNext;
					if (!IsMemoAvailable() || !WriteMemo(*pVal, nBlockNo))
						break;

					ByteString aStr;
					ByteString aBlock(ByteString::CreateFromInt32(nBlockNo));
					aStr.Expand(nLen - aBlock.Len(), '0');
					aStr += aBlock;
					aStr.Convert(gsl_getSystemTextEncoding(),pConnection->GetCharacterSet());
					// Zeichen kopieren:
					memset(pData,' ',nLen);	// Zuruecksetzen auf NULL
					memcpy(pData, aStr.GetBuffer(), nLen);
				}	break;
				default:
				{
					memset(pData,' ',nLen);	// Zuruecksetzen auf NULL
					ByteString aStr(pVal->getString().getStr(),pConnection->GetCharacterSet());
					// Zeichen kopieren:
					memcpy(pData, aStr.GetBuffer(), min(nLen,aStr.Len()));
				}	break;
			}
		}
		catch ( ... )
		{
			String strError(SdbResId(STR_DBF_INVALID_FIELD_VALUE));
			strError.SearchAndReplace(String::CreateFromAscii("$name$"), pColumn->GetName());
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());
		}
		if (aStatus.IsError())
			break;
		// Und weiter ...
		nByteOffset += nLen;
	}
	return aStatus.IsSuccessful();
}


//------------------------------------------------------------------
BOOL SdbDBFTable::WriteBuffer()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	DBG_ASSERT(nFilePos >= 1,"SdbDBFCursor::FileFetchRow: ungueltige Record-Position");

	// Auf gewuenschten Record positionieren:
	long nPos = aFileHeader.db_kopf + (long)(nFilePos-1) * aFileHeader.db_slng;
	TRY_FILE(aFileStream.Seek(nPos),TABLE);
	TRY_FILE(aFileStream.Write((char*) pBuffer, aFileHeader.db_slng),TABLE);

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbDBFTable::Compress()
{
	DBG_CHKTHIS(SdbDBFTable,NULL);
	SDB_CHK_CONNECT_BOOL();

	if (InUse())
	{
		aStatus.SetError(ERRCODE_IO_LOCKVIOLATION,TABLE,aName);
		return FALSE;
	}

	BUSY_DB();
	// merkt sich die Spaltenpositionen aus der Originaltabelle von den Spalten,
	// der Daten uebernommen werden
	SdbFILEConnection* pFILEConnection = GetFILEConnection();
	SdbFILECursor* pCursor = (SdbFILECursor*) pFILEConnection->CreateCursor(SDB_KEYSET, SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();

	// geloeschte Saetze nicht uebertragen
	pCursor->ShowDeleted(FALSE);
	if (!pCursor->Open(aName, TRUE))
	{
		aStatus = pCursor->Status();
		pCursor->ReleaseRef();
		return FALSE;
	}

	// OriginalFile
	DirEntry aOrig(pFILEConnection->FileEntry(aName));
	DirEntry aMask(aOrig.GetPath());
	aMask.SetExtension( String::CreateFromAscii(".dbf") );
	DirEntry aTemp(aMask.TempName());
	aTemp.Kill();

	// neue Tabelle anlegen
	SdbDBFTable* pNewTable = new SdbDBFTable(GetDBFConnection());
	if (!pNewTable->Create(aTemp.GetBase()))
	{
		pCursor->ReleaseRef();
		delete pNewTable;
		return FALSE;
	}

	// Spalten uebertragen
	pNewTable->aColumns = aColumns;

	if (!(pNewTable->CreateImpl() &&
		  pNewTable->FileOpen(FALSE)))
	{
		pCursor->ReleaseRef();
		delete pNewTable;
		return FALSE;
	}
	else
		pNewTable->aOriginalColumns = pNewTable->aColumns; // Originalcolumns wichtig fuer die Datenuebertragung

	ODbRowRef xRow = new ODbRow(*pNewTable->aColumns);
	USHORT nCount = pNewTable->aColumns->Count();
	// jetzt die Daten uebertragen
	while (pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		for (USHORT i = 1; i < nCount; i++)
			*(*xRow)[i] = *rRow[i];

		// nichts kopiert und gleiche Spaltenanzahl
		// dann bleibt die neue Tabelle leer
		if (!pNewTable->InsertRow(xRow, FALSE))
		{
			aStatus = pNewTable->Status();
			pCursor->ReleaseRef();

			pNewTable->DropImpl();
			delete pNewTable;

			return FALSE;
		}

		// nichts kopiert und gleiche Spaltenanzahl
		// dann bleibt die neue Tabelle leer
		pCursor->Next();
	}
	pCursor->ReleaseRef();

	if (!DropImpl())
	{
		// Temporaere loeschen
		pNewTable->DropImpl();
		delete pNewTable;
		return FALSE;
	}

	// und die neue an die Stelle setzen
	pNewTable->RenameImpl(aName);
	pNewTable->FileClose();
	delete pNewTable;

	FileOpen(FALSE);

	// Sind Indices vorhanden, dann diese ebenfalls komprimieren
	for (USHORT i = 1; i < aOriginalColumns->Count(); i++)
	{
		const SdbFILEColumn* pColumn = (SdbFILEColumn*)(*aOriginalColumns)[i];
		if (pColumn->IsIndexed())
		{
			// Sollte klappen
			SdbNDXIndex* pIndex = (SdbNDXIndex*)pColumn->GetIndex();
			pIndex->DropImpl();
			pIndex->CreateImpl();
		}
	}

	return TRUE;
}


