/*************************************************************************
 *
 *  $RCSfile: fileconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbresid.hrc"
#include "fileconn.hxx"
#include "sdbmap.hxx"

#include "fileconn.hxx"
#include "filecurs.hxx"
#include "filetab.hxx"
#include "filecol.hxx"

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

#ifndef _RTL_CHAR_H_
#include <rtl/char.h>
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

#ifndef _SDB_FILEMD_HXX
#include <filemd.hxx>
#endif // _SDB_FILEMD_HXX
#ifndef _URLOBJ_HXX //autogen wg. INetURLObject
#include <tools/urlobj.hxx>
#endif
#ifndef _SFXINIPROP_HXX //autogen wg. SfxAppIniManagerProperty
#include <svtools/iniprop.hxx>
#endif
#ifndef _SFXINIMGR_HXX //autogen wg. SfxIniManager
#include <svtools/iniman.hxx>
#endif


DBG_NAME(SdbFILEConnection);

//------------------------------------------------------------------
SdbFILEConnection::SdbFILEConnection (SdbEnvironment* pEnv, const String& rOptions,
									  const char* pFilenameExtension)
				  :SdbConnection(pEnv, rOptions)
				  ,pDir(NULL)
				  ,bNoFileNameLimit(FALSE)
{
	DBG_CTOR(SdbFILEConnection,NULL);

	aStatus.Clear();

/////////////////////////////////////////////////////////////////////////////
//
// Der angegebene Pfadname wird wie folgt interpretiert:
// - Wenn es sich um ein Wildcard handelt, werden alle Dateien, die
//   diesem Wildcard entsprechend, als Tabellen aufgelistet
// - Wenn es sich um einen Directory-Namen handelt, werden alle
//   Dateien mit der jeweiligen Default-Extension in dem betreffenden
//	 Verzeichnis als Tabellen aufgelistet.
// - Wenn es sich um einen Dateinamen handelt, werden alle Dateien
//   mit der gleichen Extension, die dieser Dateiname hat und die
//	 in dem selben Verzeichnis wie die angegebene Datei stehen,
//	 als Tabellen aufgelistet. Der Basisname der Datei wird zum
//	 Default-Tabellennamen.
//
/////////////////////////////////////////////////////////////////////////////
	String aFileName;// = aOptionsParser.OptionString("DSN");
	String aWildcard;

#if defined (UNX)
	eIdentifierCase = SDB_IC_OBJECT;
#else
	eIdentifierCase = SDB_IC_MIXED;
#endif

	bNoFileNameLimit = aOptionsParser.OptionBool(String::CreateFromAscii("NOFILENAMELIMIT"));
	bShowDeleted = aOptionsParser.OptionBool(String::CreateFromAscii("DELETED"));

	// feststellen ob die DSN als file URL angegeben wurde, wenn ja dann
	// werden Office weite Platzhalter ersetzt und zweitens
	// relative Pfade zum sdb absolutiert

	String aDSN(aOptionsParser.OptionString(String::CreateFromAscii("DSN")));
	INetURLObject aURL;
	aURL.SetSmartProtocol(INET_PROT_FILE);
	aURL.SetURL(ByteString(aDSN,gsl_getSystemTextEncoding()), INetURLObject::ENCODE_ALL);
//	if (aURL.GetProtocol() == INET_PROT_FILE)
//	{
		// $Inst mu gesetzt sein
		// sehr umstaendlicher weg, um an den Inimanager zu kommen
		// aber ohne Arme(Sfx) keine Kekse
		SfxAppIniManagerProperty aIniManProp;
		GetpApp()->Property(aIniManProp);
		aFileName = aIniManProp.GetIniManager()->SubstPathVars(aDSN);
//	}

	// Ist der gegebene Pfadname ein Wildcard, ein Verzeichnis oder ein Dateiname?
	DirEntry aFile(aFileName);
	FileStat aFileStat(aFile); 	// nicht zusammenfassen - Bug im VC++ 2.0!


	if (aFileStat.IsKind(FSYS_KIND_WILD))
	{
		aDirectoryName = (DirEntry(aFileName).GetPath()).GetFull();
		aFilenameExtension = DirEntry(aFileName).GetExtension();

		// nyi: Verschiedene Extensions (aufgrund des angegebenen Wildcard)
		// (bisher keine Wildcard in der Extension moeglich!)
		aWildcard = DirEntry(aFileName).GetName();
	}
	else if (aFileStat.IsKind(FSYS_KIND_DIR))
	{
		aDirectoryName = DirEntry(aFileName).GetFull();

		if (aOptionsParser.Exists(String::CreateFromAscii("EXT")))
			aFilenameExtension = aOptionsParser.OptionString(String::CreateFromAscii("EXT"));	// wenn extension angegeben, dann diese verwenden
		else	// wenn nicht gesetzt, dann default
			aFilenameExtension.AssignAscii(pFilenameExtension);

		if (aFilenameExtension.Len())
		{
			aWildcard.AssignAscii("*.");
			aWildcard += aFilenameExtension;
		}
	}
	else if (aFileStat.IsKind(FSYS_KIND_FILE))
	{
		aDirectoryName = (DirEntry(aFileName).GetPath()).GetFull();
		aFilenameExtension = DirEntry(aFileName).GetExtension();
		aWildcard.AssignAscii("*.");
		aWildcard += aFilenameExtension;
	}
	else
	{
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_CONNECTION))),
					0, String() );
		return;
	}

	if (aFilenameExtension.Search('*') != STRING_NOTFOUND ||
		aFilenameExtension.Search('?') != STRING_NOTFOUND)

	{
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(String(SdbResId(STR_TXT_NOWILDCARDS))),
					0, String() );
		return;
	}

	DirEntry aDirEntry(aDirectoryName);	// in das angegebene Verzeichnis gehen
	if (!aDirEntry.IsValid()) return;

	// Liste aller Dateien (entsprechend der Wildcard) bilden:
	if (aWildcard.Len())
	{
		aDirEntry += aWildcard;
	}
	aDirEntry.ToAbs();
	pDir = new Dir(aDirEntry, FSYS_KIND_FILE, FSYS_SORT_NAME | FSYS_SORT_ASCENDING | FSYS_SORT_END);

	// Datenbankname ist das Verzeichnis, in dem die files / tabellen stehen
	aQualifierName = aDirectoryName;
}

//------------------------------------------------------------------
SdbFILEConnection::~SdbFILEConnection()
{
	delete pDir;
	DBG_DTOR(SdbFILEConnection,NULL);
}

//------------------------------------------------------------------
String SdbFILEConnection::ServerNames(SdbStatus&, SdbEnvironment*)
{
	return String::CreateFromAscii("*");	// Servername ist Pfadname
}

//------------------------------------------------------------------
DirEntry SdbFILEConnection::FileEntry(const String& rTableName) const
{
	DirEntry aFileEntry(rTableName);
	if (aFileEntry.IsAbs()) // wenn Tabellennamen mit Pfad angegeben -> fertig
		return aFileEntry;

	String aFileName(rTableName);
	aFileEntry = DirEntry(aDirectoryName);

	// Es gibt auch Dateisysteme ohne Extensions
	if (aFilenameExtension.Len())
	{
		(aFileName += String('.')) += aFilenameExtension;
	}

	aFileEntry += DirEntry(aFileName);
	/*
	aFileEntry += DirEntry(aFileName);
	if (aFilenameExtension.Len())
	{
		aFileEntry.SetExtension(aFilenameExtension);
	}
	*/
	return aFileEntry;
}

//------------------------------------------------------------------
SdbCursor* SdbFILEConnection::CreateCursorImpl(SdbCursorType eType,
											   UINT16 nOptions)
{
	DBG_CHKTHIS(SdbFILEConnection,NULL);
	return new SdbFILECursor(this, eType, nOptions);
}

//------------------------------------------------------------------
SdbColumns* SdbFILEConnection::CreateColumns()
{
	DBG_CHKTHIS(SdbFILEConnection,NULL);
	return new SdbFILEColumns(*this);
}

//------------------------------------------------------------------
SdbColumns* SdbFILEConnection::CreateColumns(const SdbColumns& _rCols)
{
	DBG_CHKTHIS(SdbFILEConnection,NULL);
	return new SdbFILEColumns(*this, _rCols);
}

//------------------------------------------------------------------
SdbColumn* SdbFILEConnection::CreateColumn(const String& _rName, SdbDatabaseType _eType,
									   UINT32 _nLength, UINT16 _nScale, UINT16 _nFlags)
{
	SDB_CHK_OPEN_NULL();
	if (eIdentifierCase == SDB_IC_ALL)
		_nFlags |= SDB_FLAGS_CASESENSITIVE;

	// Formatkey anhand des Typs bzw des Scales bestimmen
	UINT32 nKey = SdbTools::DefaultFormat(*GetEnvironment()->GetStandardFormatter(), _eType, _nScale);
	return new SdbFILEColumn(_rName, _eType, _nLength, _nScale, _nFlags, nKey);
}

//------------------------------------------------------------------
SdbColumn* SdbFILEConnection::CreateColumn(const SdbColumn& _rCol)
{
	SDB_CHK_OPEN_NULL();
	return new SdbFILEColumn(_rCol);
}

//------------------------------------------------------------------
void SdbFILEConnection::FillRelations(SdbRelationList& _rList, const String& rName)
{
	aStatus.SetDriverNotCapableError();
}

//------------------------------------------------------------------
void SdbFILEConnection::FillTableNames(SdbStringStlMap* _pList)
{
	DBG_CHKTHIS(SdbFILEConnection,NULL);

	if (pDir == NULL) return;

	pDir->SetSort(FSYS_SORT_NAME | FSYS_SORT_ASCENDING | FSYS_SORT_END);
	pDir->Update();

	for (UINT16 nDirPos = 0;	nDirPos < pDir->Count(); nDirPos++)
	{
		if (aFilenameExtension.Len())
		{
			(*_pList)[(*pDir)[nDirPos].GetBase()] = TRUE;
		}
		else // keine extension, dann selbst filtern
		{
			do
			{
				DirEntry aEntry = (*pDir)[nDirPos];
				if (!(*pDir)[nDirPos].GetExtension().Len())
				{
					(*_pList)[(*pDir)[nDirPos].GetBase()] = TRUE;
					break;
				}
				else
					nDirPos++;
			} while ((unsigned int)nDirPos < pDir->Count());
		}
	}
}

//------------------------------------------------------------------
void SdbFILEConnection::SetAutoCommit(BOOL bAuto)
{
}

//------------------------------------------------------------------
void SdbFILEConnection::CommitImpl()
{
	aStatus.Set( SDB_STAT_ERROR );
}

//------------------------------------------------------------------
void SdbFILEConnection::RollbackImpl()
{
	aStatus.Set( SDB_STAT_ERROR );
}

//------------------------------------------------------------------
BOOL SdbFILEConnection::OpenImpl()
{
	pTableNames = new SdbStringStlMap(::utl::UStringMixLess(FALSE));
	return (pDir && pDir->Exists());
}

//------------------------------------------------------------------
BOOL SdbFILEConnection::CloseImpl()
{
	return TRUE;
}

//------------------------------------------------------------------
::com::sun::star::data::XDatabaseDescriptor* SdbFILEConnection::GetMetaDataImpl()
{
	SdbFILEMetaData* pMetaData = new SdbFILEMetaData(this);
	return pMetaData;
}

//------------------------------------------------------------------
INT32 compareIgnoreCase(const rtl::OUString& rStr1, const rtl::OUString& rStr2, const NAMESPACE_RTL(OLocale)& rLocale)
{
	rtl::OUString aString1 = rStr1.toUpperCase(rLocale);
	rtl::OUString aString2 = rStr2.toUpperCase(rLocale);
	return aString1.compareTo(aString2);
}

//------------------------------------------------------------------
const sal_Unicode CHAR_PLACE = L'_';
const sal_Unicode CHAR_WILD  = L'%';

//------------------------------------------------------------------
// Anleihe aus TOOLS: Wildcard Match.
// Hier aber mit anderen Wildcard- und Fluchtzeichen!
BOOL match(const sal_Unicode* pWild, const sal_Unicode* pStr, const sal_Unicode cEscape)
{
	int    pos=0;
	int    flag=0;

	while ( *pWild || flag )
	{
		switch (*pWild)
		{
			case CHAR_PLACE:
				if ( *pStr == 0 )
					return FALSE;
				break;
			default:
				if (*pWild && (*pWild == cEscape) && ((*(pWild+1)== CHAR_PLACE) || (*(pWild+1) == CHAR_WILD)) )
					pWild++;
				if ( rtl_char_toUpperCase(*pWild) != rtl_char_toUpperCase(*pStr) )
					if ( !pos )
						return FALSE;
					else
						pWild += pos;
				else
					break;          // ACHTUNG laeuft unter bestimmten
									// Umstaenden in den nachsten case rein!!
			case CHAR_WILD:
				while ( *pWild == CHAR_WILD )
					pWild++;
				if ( *pWild == 0 )
					return TRUE;
				flag = 1;
				pos  = 0;
				if ( *pStr == 0 )
					return ( *pWild == 0 );
				while ( *pStr && *pStr != *pWild )
				{
					if ( *pWild == CHAR_PLACE ) {
						pWild++;
						while ( *pWild == CHAR_WILD )
							pWild++;
					}
					pStr++;
					if ( *pStr == 0 )
						return ( *pWild == 0 );
				}
				break;
		}
		if ( *pWild != 0 )
			pWild++;
		if ( *pStr != 0 )
			pStr++;
		else
			flag = 0;
		if ( flag )
			pos--;
	}
	return ( *pStr == 0 ) && ( *pWild == 0 );
}


