/*************************************************************************
 *
 *  $RCSfile: filecurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_FILESET_HXX
#include "fileset.hxx"
#endif

#include "sdbresid.hrc"
#include <float.h>			// DBL_MAX

// filecurs
#ifndef _SDB_FILECURS_HXX
#include "filecurs.hxx"
#endif
#ifndef _SDB_FILECONN_HXX
#include "fileconn.hxx"
#endif

#ifndef _SDB_FILETAB_HXX
#include "filetab.hxx"
#endif
#ifndef _SDB_FILECOL_HXX
#include "filecol.hxx"
#endif
#ifndef _SDB_FILEIDX_HXX
#include "fileidx.hxx"
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifndef _SDB_SDBSET_HXX
#include <sdbset.hxx>
#endif

#ifndef _SDB_SDBSQLIT_HXX //autogen
#include <sdbsqlit.hxx>
#endif

#ifndef _ZFORLIST_HXX //autogen
#include <svtools/zforlist.hxx>
#endif

DBG_NAME(SdbFILECursor);

// Vereinfachen der Tabellenzugriffe
#define TABLE_ACTION(pTable,Action)			\
	if (!pTable->aStatus.IsSuccessful())	\
		pTable->aStatus.Clear();			\
	if (!pTable->Action)					\
		aStatus = pTable->Status();

// Maximale Anzahl von Rows, die mit ORDER BY sortiert durchlaufen werden koennen:
#if defined (WIN)
#define MAX_KEYSET_SIZE 0x3ff0	// Etwas weniger als ein Segment, damit
									// noch Platz fuer Memory Debug-Informationen
#else
#define MAX_KEYSET_SIZE 0x40000 // 256K
#endif

//------------------------------------------------------------------
class ODbAssignRow : public ODbRow
{
	vector<UINT32> m_aIndexList;

public:
	ODbAssignRow(SdbColumns& rColumns)
		:ODbRow(rColumns, TRUE)
		,m_aIndexList(rColumns.Count(), SDB_NO_PARAMETER){}

	void setAssignedIndex(INT32 nPos, UINT32 nIndex)
		{m_aIndexList[nPos] = nIndex;}

	UINT32 getAssignedIndex(INT32 nPos) const
		{return m_aIndexList[nPos];}
};

#pragma warning (disable : 4355)
//------------------------------------------------------------------
SdbFILECursor::SdbFILECursor(SdbFILEConnection* pFILEConnection,
							 SdbCursorType eType,
							 UINT16 nOpt)
			  :SdbCursor(pFILEConnection, eType, nOpt)
			  ,aSQLAnalyzer(*this)
			  ,eStatementType(SDB_STATEMENT_UNKNOWN)
			  ,bForceUseIndex(FALSE)
			  ,bShowDeleted(FALSE)
			  ,pFileSet(NULL)
			  ,pEvaluationKeySet(NULL)
			  ,pSortIndex(NULL)
			  ,nRowCountResult(-1)
			  ,nFetchPos(-1)
			  ,nFilePos(0)
			  ,nLastVisitedPos(-1)
			  ,pTable(NULL)
#pragma warning (default : 4355)
{
	DBG_CTOR(SdbFILECursor,NULL);

	bIsCancelable = TRUE;
	// das (ziemlich zentrale) ExecuteRow ist in der Lage, mit Abbruechen umzugehen. Da der Cursor
	// selber thread-sicher ist, steht also einem potentiellen Abbrechen nichts mehr im Weg

	aStatus.Clear();
	DBG_ASSERT(sizeof nOrderbyColumnNumber / sizeof (* nOrderbyColumnNumber) ==
			   sizeof bOrderbyAscending / sizeof (* bOrderbyAscending),
			   "SdbFILECursor: Interner Fehler: bOrderbyAscending[] und nOrderbyColumnNumber[] muessen gleiche Anzahl Elemente haben.");
	for (int jj = 0; jj < sizeof nOrderbyColumnNumber / sizeof (* nOrderbyColumnNumber); jj++)
	{
		nOrderbyColumnNumber[jj] = SDB_COLUMN_NOTFOUND;
		bOrderbyAscending[jj] = TRUE;
	}

	bShowDeleted = pFILEConnection->GetShowDeleted();
	nOptions |= SDB_INDIRECT;	// bei Filebasierten Cursoren wird grundsaetzlich geparst
}

//------------------------------------------------------------------
SdbFILECursor::~SdbFILECursor()
{
	DBG_DTOR(SdbFILECursor,NULL);
	aFileColumns.unbind();
	if (pTable)
	{
		pTable->Release();
		pTable->ReleaseRef();
		pTable = NULL;
	}
	delete pFileSet;
	delete pEvaluationKeySet;
}

//------------------------------------------------------------------
void SdbFILECursor::DriverTranslate()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	// Statement-Typ ermitteln:
	if (SDB_ISRULE(pSQLParseNode,select_statement))
		eStatementType = SDB_STATEMENT_SELECT;
	else if (SDB_ISRULE(pSQLParseNode,insert_statement))
		eStatementType = SDB_STATEMENT_INSERT;
	else if (SDB_ISRULE(pSQLParseNode,update_statement_positioned))
		eStatementType = SDB_STATEMENT_UPDATE;
	else if (SDB_ISRULE(pSQLParseNode,update_statement_searched))
		eStatementType = SDB_STATEMENT_UPDATE;
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_positioned))
		eStatementType = SDB_STATEMENT_DELETE;
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_searched))
		eStatementType = SDB_STATEMENT_DELETE;
	else
	{
		eStatementType = SDB_STATEMENT_UNKNOWN;
		aStatus.SetInvalidStatement();
		return;
	}
}

//------------------------------------------------------------------
void SdbFILECursor::Bind()
{
	if (!aFileRow.Is())
	{
		// Beim Select werden noch keine Variable gebunden, erst wenn
		// die Spalten auch bentigt werden
		aFileRow = new ODbRow(*aFileColumns, FALSE);
		(*aResultRow)[0] = (*aFileRow)[0] = new ODbVariant();
	}

	// Die Resultrow erhaelt nur Verweise auf die FileRow
	for (USHORT i = 1; i < aResultColumns->Count(); i++)
	{
		const String& aColName = ((SdbFILEColumn*)(*aResultColumns)[i])->GetRealName();
		UINT16 nColNo = aFileColumns->ColumnNumber(aColName);
		if (nColNo != SDB_COLUMN_NOTFOUND)
		{
			if ((*aResultRow)[i].is())
			{
				if ((*aFileRow)[nColNo].is())
					(*aResultRow)[i] = (*aFileRow)[nColNo];
				else
					(*aFileRow)[nColNo] = (*aResultRow)[i];
			}
			else
			{
				(*aResultRow)[i] = (*aFileRow)[nColNo] = new ODbVariant();
			}
		}
	}
}

//------------------------------------------------------------------
BOOL SdbFILECursor::PrepareImpl()
{
	// Statement vorbereiten (und Statement-Typ ermitteln):

	// Sicherstellen, dass immer ein ParseTree erzeugen wird #65694#
	nOptions |= SDB_INDIRECT;

	Translate(FALSE);
	if (!aStatus.IsSuccessful())
		return FALSE;

	// Cursor-Name setzen
	if (aName.Len() == 0)
	{
		aName = 'C';
		aName += String::CreateFromInt32((long)this);
	}

	// Tabellennamen aus Parse Tree entnehmen ...
	String aTableName;
	SdbFILEConnection* pConnection = GetFILEConnection();

	// Tabellenname (bei allen Statement-Typen)
	GetTableName(aTableName, aTableRange);
	if (!aStatus.IsSuccessful())
		return FALSE;

	BOOL bForUpdate = (nOptions & SDB_READONLY) == 0;

	// Tabelle oeffnen
	pTable = (SdbFILETable*)pConnection->OpenTable(aTableName);
	if (pTable && (pTable->GetTablePrivileges() <= SDB_PR_READ))
	{	// The table has only read-only privileges. In this case we create an extra instance
		// of it. Normally this is harmless, but in some cases (67489) it helps a lot ...
		SdbFILETable* pSecondTry = (SdbFILETable*)pConnection->CreateTableImpl();
		if (pSecondTry && pSecondTry->Open(aTableName))
		{	// throw away the old table
			pTable->AddRef();
			pTable->Acquire();
			pTable->Release();
			pTable->ReleaseRef();

			// (because the new one fits better ...)
			pTable = pSecondTry;
			pTable->SetTemporary();
		}
		else if (pSecondTry)
		{
			pSecondTry->AddRef();
			pSecondTry->Acquire();
			pSecondTry->Release();
			pSecondTry->ReleaseRef();
		}
	}

	if (!pTable)
	{
		aStatus = pConnection->Status();
		aFileColumns = pConnection->CreateColumns();
		return FALSE;
	}
	else
	{
		// wenn die Connection Indizes unterstuetzt, dann die Index-Flags setzen
		aFileColumns = pTable->OriginalColumns();
		pTable->AddRef();
		pTable->Acquire();
	}

	// ERST die Assign Values, denn die kommen beim Parsen
	// immer VOR den Selektionskriterien (sonst wuerde die
	// Reihenfolge der Parameter-Namen, die beim Translate
	// ermittelt wurde, nicht stimmen!)
	// ParameterColumns werden ueber den Analyzer gesetzt
	aParameterColumns = GetConnection()->CreateColumns();

	GetAssignValues();
	if (!aStatus.IsSuccessful())
		return FALSE;

	// Analyse des SQLSatements
	AnylizeSQL();
	if (!aStatus.IsSuccessful())
		return FALSE;

	DescribeParameter();
	if (!aStatus.IsSuccessful())
		return FALSE;

	BindParameter();
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbFILECursor::DescribeResult()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);
	DBG_ASSERT(aResultColumns.Is(), "keine Ergebnisbeschreibung");

	GetColumnNames();
}

//------------------------------------------------------------------
void SdbFILECursor::DescribeParameter()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	// Parameter-Strukturbeschreibung wird beim Analysieren angelegt.
	// Strukturbeschreibung weiter verfeinern
	if (aParameterColumns->Count() > 1)
		aSQLAnalyzer.DescribeParam(aParameterColumns);
}


//------------------------------------------------------------------
void SdbFILECursor::BindParameter()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);
}

//------------------------------------------------------------------
BOOL SdbFILECursor::OpenImpl(SdbCursorType eType ,UINT16 eOpt)
{
	SDB_CHK_CONNECT_BOOL();
	if (!IsPrepared())
	{
		// Benutzung von Prepare, aber bPrepared gleich wieder zuruecksetzen.
		Prepare();
		if (!aStatus.IsSuccessful()) return FALSE;
		bPrepared = FALSE;
	}

	if (!IsOpen())
		DescribeResult();

	// Expliziter Call von InitResultRow, wird vor dem OnOpen benoetigt
	InitResultRow();
	if (aResultColumns->Count())
		Bind();

	// Parameter substituieren (AssignValues und Kriterien):
	if (aParameterRow.Is())
	{
		// Zunchst AssignValues
		USHORT nParaCount=0; // gibt die aktuelle Anzahl der bisher gesetzen Parameter an

		// Nach zu substituierenden Parametern suchen:
		UINT16 nCount = (aAssignValues.Is()) ? aAssignValues->size() : 1; // 1 ist wichtig fr die Kriterien
		ODbVariantRef xParameterValue;
		for (UINT16 j = 1; j < nCount; j++)
		{
			const ODbVariantRef& xValue = (*aAssignValues)[j];
			if (!xValue.is() || xValue->isNull())
				continue;

			UINT32 nParameter = ((ODbAssignRow*)aAssignValues.getBodyPtr())->getAssignedIndex(j);
			if (nParameter == SDB_NO_PARAMETER)
				continue;	// dieser AssignValue ist kein Parameter

			++nParaCount; // ab hier ist der Parameter gueltig
			// Parameter ersetzen. Wenn Parameter nicht verfuegbar,
			// Value auf NULL setzen.
			*xValue = *(*aParameterRow)[(UINT16)nParameter];
		}

		if (nParaCount < aParameterRow->size())
			aSQLAnalyzer.BindParameterRow(aParameterRow);
	}

	// Neuen Index aufbauen:
	DELETEZ(pFileSet);
	DELETEZ(pEvaluationKeySet);

	if (!aEvaluateRow.Is())
	{
		aEvaluateRow = new ODbRow(*aFileColumns, FALSE);
		(*aEvaluateRow)[0] = new ODbVariant();

		// Row zur Auswertung binden, wenn Preprocessing erfolg, dann bereits ein Keyset
		pEvaluationKeySet = aSQLAnalyzer.BindResultRow(aEvaluateRow);	// Werte im Code des Compilers setzen
													// (Verbindung zur ResultRow herstellen)
	}

	// An den Anfang positionieren
	nFetchPos = -1;
	nFilePos  = 0;
	nRowCountResult = -1;

	// exclusiver zugriff auf die Tabelle
	NAMESPACE_VOS(OGuard)* pGuard = pTable->Lock();
	nLastVisitedPos = pTable->GetCurrentLastPos();

	UINT32 nLoopCount = 0;
	switch(eStatementType)
	{
		case SDB_STATEMENT_SELECT:
		{
			INT32 nMaxRowCount = MAX_KEYSET_SIZE;
			if  (MaxRowCount() > 0)
			{
				// Wenn genauere Abschaetzung verfuegbar, dann diese benutzen (aber trotzdem Maximum einhalten!):
				if (MaxRowCount() <= nMaxRowCount)
					nMaxRowCount = MaxRowCount();
			}
			else
				nMaxRowCount =  ((SdbFILETable*)pTable)->GetCurrentLastPos();

			BOOL bDistinct = FALSE;
			BOOL bWasSorted = FALSE;
			SdbSqlParseNode *pDistinct = pSQLParseNode->GetChild(1);
			if (pDistinct && pDistinct->GetTokenID() == SDB_TOKEN_DISTINCT )
			{
				if(!IsSorted())
				{
					nOrderbyColumnNumber[0] = 1;
					bOrderbyAscending[0] = FALSE;
				}
				else
					bWasSorted = TRUE;
				bDistinct = TRUE;
			}
			// Ohne Restriction und Sortierung RowCount bekannt.
			if (!HasRestriction() && !IsSorted() && MaxRowCount() >= 0 && bShowDeleted)
				SetRowCount(MaxRowCount());

			DBG_ASSERT(sizeof nOrderbyColumnNumber / sizeof (* nOrderbyColumnNumber) == MAXORDER,"Maximale Anzahl der ORDER BY Columns muss derzeit genau 3 sein!");
			SdbKeyType eKeyType[MAXORDER];
			for (int i = 0; i < MAXORDER; i++)
			{
				if (nOrderbyColumnNumber[i] == SDB_COLUMN_NOTFOUND)
					eKeyType[i] = SDB_ORDERBYKEY_NONE;
				else
				{
					switch ((*aFileColumns)[nOrderbyColumnNumber[i]]->GetType())
					{
						case SDB_DBTYPE_CHAR:
						case SDB_DBTYPE_VARCHAR:
							eKeyType[i] = SDB_ORDERBYKEY_STRING;
							break;

						case SDB_DBTYPE_BOOKMARK:
						case SDB_DBTYPE_TINYINT:
						case SDB_DBTYPE_SMALLINT:
						case SDB_DBTYPE_INTEGER:
						case SDB_DBTYPE_DECIMAL:
						case SDB_DBTYPE_NUMERIC:
						case SDB_DBTYPE_REAL:
						case SDB_DBTYPE_DOUBLE:
						case SDB_DBTYPE_DATE:
						case SDB_DBTYPE_TIME:
						case SDB_DBTYPE_TIMESTAMP:
						case SDB_DBTYPE_BOOLEAN:
							eKeyType[i] = SDB_ORDERBYKEY_DOUBLE;
							break;

					// Andere Typen sind nicht implementiert (und damit immer FALSE)
						default:
							eKeyType[i] = SDB_ORDERBYKEY_NONE;
							DBG_ERROR("SdbFILECursor::Execute: Datentyp nicht implementiert");
							break;
					}
					if (!(*aEvaluateRow)[nOrderbyColumnNumber[i]].is())	// RowBinding
						(*aEvaluateRow)[nOrderbyColumnNumber[i]] = new ODbVariant();
				}
			}

			// Nur wenn Sortierung gewuenscht, ueber alle Datensaetze iterieren und
			// dabei den "Key", nach dem sortiert werden soll, in den Index eintragen:
			if (IsSorted())
			{
				if (!HasRestriction() && nOrderbyColumnNumber[1] == SDB_COLUMN_NOTFOUND)
				{
					// Ist nur ein Feld fuer die Sortierung angegeben
					// Und diese Feld ist indiziert, dann den Index ausnutzen
					const SdbFILEColumn* pColumn = (const SdbFILEColumn*) aFileColumns->Column(nOrderbyColumnNumber[0]);
					if (pColumn && pColumn->IsIndexed())
					{
						pFileSet = new SdbFILESet();
						SdbFILEIndexIterator* pIter = pColumn->GetIndex()->CreateIterator(NULL,NULL);

						if (pIter->Status().IsSuccessful())
						{
							ULONG nRec = pIter->First();
							while (nRec != SDB_INDEX_ENTRY_NOTFOUND)
							{
								if (bOrderbyAscending[0])
									pFileSet->Append(nRec);
								else
									pFileSet->Insert(nRec);
								nRec = pIter->Next();
							}
							pFileSet->Freeze();
							if(!bDistinct)
								SetRowCount(pFileSet->Count());
							goto DISTINCT;
						}
						else
							delete pIter;
					}
				}

				pSortIndex = new SdbFILESortIndex(eKeyType,
												  bOrderbyAscending,
												  nMaxRowCount,
												  GetConnection()->GetCharacterSet());
				if (pEvaluationKeySet)
				{
					if (pEvaluationKeySet->Count())
						pEvaluationKeySet->First();
					else
						aStatus.Set(SDB_STAT_NO_DATA_FOUND);
				}
				while (aStatus.IsSuccessful())
				{
					if (nLoopCount++ > 1000)
					{
						// Nach mehr als 1000 Schleifendurchlaeufen muss die Applikation
						// mal eine Chance erhalten ...
						nLoopCount = 0;
						BUSY_DB();
					}
					if (pEvaluationKeySet)
					{
						ExecuteRow(FILE_BOOKMARK,pEvaluationKeySet->GetKey(),FALSE,TRUE);
						if (!pEvaluationKeySet->Next())
							aStatus.Set(SDB_STAT_NO_DATA_FOUND);
					}
					else
						ExecuteRow(FILE_NEXT,1,FALSE,TRUE);
				}

				// Sortiertes Keyset erzeugen
				DELETEZ(pEvaluationKeySet);
				pFileSet = pSortIndex->CreateKeySet();
				if(!bDistinct)
					SetRowCount(pFileSet->Count());
				DELETEZ(pSortIndex);
				// Nun kann ueber den Index sortiert zugegriffen werden.
			}

			if (!pFileSet)
			{
				pFileSet = new SdbFILESet();

				if (!HasRestriction() && IsRowCountFinal())
				// jetzt kann das Keyset schon gefuellt werden!
				// Aber Achtung: es wird davon ausgegangen, das die FilePositionen als Folge 1..n
				// abgelegt werden!
				{
					for (ULONG i = 0; i < RowCount(); i++)
					   pFileSet->Append(i + 1);
				}

			}
			DBG_ASSERT(pFileSet,"Kein KeySet vorhanden! :-(");
DISTINCT:	if(bDistinct && pFileSet)   // sicher ist sicher
			{
				ODbRowRef aSearchRow = new ODbRow(*aFileRow); // nach dieser wird gesucht
				const ODbRow &rSearchRow = *aSearchRow,
					&rFileRow = *aFileRow;
				INT32 nPos;
				UINT16 nMaxRow = pFileSet->Count();
				if (nMaxRow)
				{
#if DEBUG
					INT32 nFound=0;
#endif
					vector<BOOL> nWasAllwaysFound(nMaxRow,FALSE);
					INT32 nPrev_i;
					for(INT32 j= nMaxRow-1;j >= 0;j--)
					{
						nPos = (*pFileSet)[j]; // aktuell zu lschender Key
						if(!nWasAllwaysFound[j] && nPos) // nur falls noch nicht nach dieser Row gesucht wurde
						{
							ExecuteRow(FILE_BOOKMARK,nPos,TRUE,FALSE);
							(*aSearchRow) = rFileRow;
							// jetzt den Rest nach doppelten durchsuchen
							INT32 nKey;
							nPrev_i = j;
							for(INT32 i = j-1; i >= 0 ;i--)
							{
								nKey = (*pFileSet)[i];
								ExecuteRow(FILE_BOOKMARK,nKey ,TRUE,FALSE);
								if(!nWasAllwaysFound[i] && rSearchRow == rFileRow)
								{
									// gefunden
									// Key an der Stelle 0 setzen.
									(*pFileSet)[nPrev_i] = 0;
									// und altes i merken
									nPrev_i = i;
									nPos = nKey; // auf naechste gltige Position setzen
									nWasAllwaysFound[i] = TRUE;

								}
							}
						}
#if DEBUG
						else
							nFound++;
#endif
					}
					vector<INT32>::iterator aIter = pFileSet->end()-1;
					while(aIter != pFileSet->begin())
					{
						if(!(*aIter))
							pFileSet->erase(aIter);
						aIter--;
					}
					if (!bWasSorted)
					{
						nOrderbyColumnNumber[0] = SDB_COLUMN_NOTFOUND;
						sort(pFileSet->begin(),pFileSet->end());
					}
				}
				SetRowCount(pFileSet->Count());
			}
		}	break;

		case SDB_STATEMENT_SELECT_COUNT:
		case SDB_STATEMENT_UPDATE:
		case SDB_STATEMENT_DELETE:
			// waehrend der Bearbeitung die Anzahl der bearbeiteten Rows zaehlen:
			nRowCountResult = 0;
			// Vorlaeufig einfach ueber alle Datensaetze iterieren und
			// dabei die Aktionen bearbeiten (bzw. einfach nur zaehlen):
			if (pEvaluationKeySet)
			{
				if (pEvaluationKeySet->Count())
					pEvaluationKeySet->First();
				else
					aStatus.Set(SDB_STAT_NO_DATA_FOUND);
			}

			while (aStatus.IsSuccessful())
			{
				if (nLoopCount++ > 1000)
				{
					// Nach mehr als 1000 Schleifendurchlaeufen muss die Applikation
					// mal eine Chance erhalten ...
					nLoopCount = 0;
					BUSY_DB();
				}
				if (pEvaluationKeySet)
					ExecuteRow(FILE_BOOKMARK,pEvaluationKeySet->GetKey(),FALSE,TRUE);
				else
					ExecuteRow(FILE_NEXT,1,FALSE,TRUE);

				if (aStatus.IsSuccessful())
				{
					nRowCountResult++;
					if (pEvaluationKeySet && !pEvaluationKeySet->Next())
						aStatus.Set(SDB_STAT_NO_DATA_FOUND);
				}
			}

			// Ergebnis von COUNT(*) in nRowCountResult merken.
			// nRowCount, also die Anzahl der Rows in der Ergebnismenge, ist bei dieser
			// Anfrage = 1!
			DELETEZ(pEvaluationKeySet);
			SetRowCount(1);
			break;
		case SDB_STATEMENT_INSERT:
			nRowCountResult = 0;

			TABLE_ACTION(pTable,InsertRow(aAssignValues, TRUE));
			if (! aStatus.IsSuccessful())
				break;

			nRowCountResult = 1;
			break;
	}

	// NO_DATA_FOUND ist hier ueblich ...
	if (aStatus.Result() == SDB_STAT_NO_DATA_FOUND)
		aStatus.Clear();

	// FilePos zuruecksetzen
	nFilePos  = 0;

	delete pGuard;
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbFILECursor::OnOpen()
{
	if (SdbCursor::OnOpen())
	{
		if (pFileSet && pFileSet->IsFrozen() && pKeySet)
		{
			// KeySet wird nicht mehr gebraucht, da die Basisimplementierung bereits ein KeySet
			// mit Bookmarks implementiert!
			OBookmarkSet* pBookmarkSet = (OBookmarkSet*)pKeySet;
			for (UINT32 i = 0; i < pFileSet->Count(); )
				pBookmarkSet->appendRow(++i);
		}
		return TRUE;
	}
	else
		return FALSE;
}

//------------------------------------------------------------------
void SdbFILECursor::CancelExecution()
{
	// das Cancel der Basisklasse setzt schon ein Flag, und das reicht mir, denn das kann ich
	// in ExecuteRow abfragen
}


//------------------------------------------------------------------
BOOL SdbFILECursor::CloseImpl()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	// ORDER BY-Index freigeben
	DELETEZ(pFileSet);
	DELETEZ(pEvaluationKeySet);

	aSQLAnalyzer.Clean();
	aEvaluateRow.unbind();
	aFileRow.unbind();				// wird im Bind wieder angelegt
	aFileColumns.unbind();

	if (pTable)
	{
		pTable->Release();
		pTable->ReleaseRef();
		pTable = NULL;
	}

	for (int jj = 0; jj < sizeof nOrderbyColumnNumber / sizeof (* nOrderbyColumnNumber); jj++)
	{
		nOrderbyColumnNumber[jj] = SDB_COLUMN_NOTFOUND;
		bOrderbyAscending[jj] = TRUE;
	}

	// Zuweisungswerte freigeben:
	aAssignValues.unbind();
	return TRUE;
}


//------------------------------------------------------------------
void SdbFILECursor::ExecuteRow(SdbFileCursorPosition eFirstCursorPosition,
							   INT32 nFirstOffset,
							   BOOL bRebind,
							   BOOL bEvaluate,
							   BOOL bRetrieveData)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	SDB_CHK_CONNECT();
	aStatus.Clear();

	// Fuer weitere Fetch-Operationen werden diese Angaben ggf. veraendert ...
	SdbFileCursorPosition eCursorPosition = eFirstCursorPosition;
	INT32  nOffset = nFirstOffset;
	UINT32 nLoopCount = 0;

again:
	if (WasCanceled(TRUE))
	{
		// wenn das anspringt, dann befinde ich mich gerade in einem cancelable block
		// (das erzwingt der TRUE-Parameter), also hat aussen irgend jemand ein BeginCancelable
		// gesetzt, damit kann ich hier einfach abbrechen, denn man kann sich dann ja aussen
		// weiter drum kuemmern
		// Trotzdem sicherheitshalber wenigstens noch den Status
		aStatus.Set(SDB_STAT_CANCELED);
		return;
	}
	if (nLoopCount++ > 1000)
	{
		// Nach mehr als 1000 Schleifendurchlaeufen ohne gefundenen Datensatz
		// muss die Applikation mal eine Chance erhalten ...
		nLoopCount = 0;
		BUSY_DB();
	}

	// protect from reading over the end when someboby is inserting while we are reading
	// this method works only for dBase at the moment !!!!
	if (eCursorPosition == FILE_NEXT && nFilePos == nLastVisitedPos)
	{
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);
		return;
	}

	TABLE_ACTION(pTable,SeekRow(eCursorPosition, nOffset, nFilePos));
	if (!aStatus.IsSuccessful())
	{
		return;
	}

	if (!bEvaluate)	// Laeuft keine Auswertung, dann nur Ergebniszeile fuellen
	{
		TABLE_ACTION(pTable,FetchRow(*aFileRow,*aFileColumns, bRetrieveData));

		if (bShowDeleted && aFileRow->isDeleted())
			aFileRow->setState(ROW_DELETED);
	}
	else
	{
		TABLE_ACTION(pTable,FetchRow(*aEvaluateRow, *aFileColumns, TRUE));

		if (!bShowDeleted && aEvaluateRow->isDeleted() ||// keine Anzeige von geloeschten Stzen
			!aSQLAnalyzer.EvaluateRestriction())		 // Auswerten der Bedingungen
		{												 // naechsten Satz auswerten
			// aktuelle Zeile loeschen im Keyset
			DBG_ASSERT(!pFileSet ||
					   !pFileSet->IsFrozen() ||
						eCursorPosition == FILE_NEXT, "Falsche CursorPosition!");

			if (pEvaluationKeySet)
			{
				if (pEvaluationKeySet->Next())
					nOffset = pEvaluationKeySet->GetKey();
				else
				{
					aStatus.Set(SDB_STAT_NO_DATA_FOUND);
					return;
				}
			}
			else if (pFileSet)
			{
				DBG_ASSERT(!pFileSet->IsFrozen() &&
							eCursorPosition == FILE_NEXT, "Falsche CursorPosition!");
				eCursorPosition = FILE_NEXT;
				nOffset = 1;
			}
			else if (eCursorPosition == FILE_FIRST ||
					 eCursorPosition == FILE_NEXT ||
					 eCursorPosition == FILE_ABSOLUTE)
			{
				eCursorPosition = FILE_NEXT;
				nOffset = 1;
			}
			else if (eCursorPosition == FILE_LAST ||
					 eCursorPosition == FILE_PRIOR)
			{
				eCursorPosition = FILE_PRIOR;
				nOffset = 1;
			}
			else if (eCursorPosition == FILE_RELATIVE)
			{
				eCursorPosition = (nOffset >= 0) ? FILE_NEXT : FILE_PRIOR;
			}
			else
			{
				aStatus.Set(SDB_STAT_NO_DATA_FOUND);
				return;
			}
			// Nochmal probieren ...
			goto again;
		}
	}
	// Evaluate darf nur gesetzt sein,
	// wenn der Keyset weiter aufgebaut	werden soll
	if (eStatementType == SDB_STATEMENT_SELECT &&
		(pFileSet || pSortIndex) && bEvaluate)
	{
		if (pSortIndex)
		{
			SdbFILEKeyValue* pKeyValue = GetOrderbyKeyValue(aEvaluateRow);
			if (!pSortIndex->AddKeyValue(pKeyValue))
			{
				// Ueberwachung auf Ueberschreitung der Index-Kapazitaet:
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1C00"),
							aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_ORDER_BY))),
							0, String() );
			}
		}
		else if (pFileSet)
		{
			DBG_ASSERT(	!pFileSet->IsFrozen() , "Falsche CursorPosition!");
			const ODbVariantRef& xBookmarkValue = (*aEvaluateRow)[0];
			DBG_ASSERT(xBookmarkValue.is(),"SdbFILECursor::Execute: Bookmark Column nicht gebunden");
			ULONG nBookmarkValue = Abs(xBookmarkValue->getINT32());
			pFileSet->Append(nBookmarkValue);
		}
	}

	if (eStatementType == SDB_STATEMENT_UPDATE)
	{
		if (bEvaluate)
		{
			// jetzt die eigentliche Ergebniszeile Lesen
			TABLE_ACTION(pTable,FetchRow(*aEvaluateRow, *aFileColumns, TRUE));
		}

		if (aStatus.IsSuccessful())
		{
			// Nur die zu aendernden Werte uebergeben:
			TABLE_ACTION(pTable,UpdateRow(aAssignValues,aEvaluateRow.getBodyPtr()));
			aStatus = pTable->Status();
		}
	}
	else if (eStatementType == SDB_STATEMENT_DELETE)
	{
		if (bEvaluate)
		{
			TABLE_ACTION(pTable,FetchRow(*aEvaluateRow, *aFileColumns, TRUE));
		}
		if (aStatus.IsSuccessful())
		{
			TABLE_ACTION(pTable, DeleteRow());
			aStatus = pTable->Status();
		}
	}
}

//------------------------------------------------------------------
BOOL SdbFILECursor::Move(SdbFileCursorPosition eCursorPosition, INT32 nOffset, SdbCursorMode& rMode, BOOL bRetrieveData)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	INT32 nTempPos = nFetchPos;
	// exclusiver zugriff auf die Tabelle
	NAMESPACE_VOS(OGuard)* pGuard = pTable->Lock();

	if (eStatementType == SDB_STATEMENT_SELECT)
	{
		if (pFileSet == NULL) // kein Index verfuegbar
		{
			// Normales FETCH
			ExecuteRow(eCursorPosition,nOffset,TRUE,FALSE,bRetrieveData);

			// now set the bookmark for outside
			aResultRow->front()->setINT32(nFetchPos + 1);
		}
		else
		{
			switch(eCursorPosition)
			{
				case FILE_NEXT:
					nFetchPos++;
					break;
				case FILE_PRIOR:
					if (nFetchPos >= 0) nFetchPos--;
					break;
				case FILE_FIRST:
					nFetchPos = 0;
					break;
				case FILE_LAST:
					DBG_ASSERT(IsRowCountFinal(), "Fehler im Keyset!");	// mu eingefroren sein, sonst Fehler beim SDBCursor
					nFetchPos = pFileSet->Count() - 1;
					break;
				case FILE_RELATIVE:
					nFetchPos += nOffset;
					break;
				case FILE_ABSOLUTE:
					nFetchPos = nOffset - 1;
					break;
				case FILE_BOOKMARK:
					nFetchPos = nOffset - 1;
					break;
			}

			// OffRange?
			// Der FileCursor ist auerhalb des gueltigen Bereichs, wenn
			// a.) nFetchPos < 1
			// b.) Ein KeySet besteht und nFetchPos > pFileSet->Count()
			if (nFetchPos < 0 || (eCursorPosition != FILE_BOOKMARK && nFetchPos >= (INT32)pFileSet->Count() && pFileSet->IsFrozen()))
			{
				aStatus.Set(SDB_STAT_NO_DATA_FOUND);
				goto Error;
			}
			else
			{
				if (nFetchPos < (INT32)pFileSet->Count())
				{
					// Fetch ueber Index
					ExecuteRow(FILE_BOOKMARK,pFileSet->GetKey(nFetchPos),TRUE,FALSE,bRetrieveData);

					// now set the bookmark for outside
					aResultRow->front()->setINT32(nFetchPos + 1);
				}
				else // Index mu weiter aufgebaut werden
				{
					// Zunaechst auf die letzte bekannte Zeile setzen
					if (pFileSet->Count())
					{
						pFileSet->SeekPos(pFileSet->Count()-1);
						TABLE_ACTION(pTable,SeekRow(FILE_BOOKMARK, pFileSet->GetKey(), nFilePos));
					}
					// Ermitteln der Anzahl weiterer Fetches
					while (aStatus.IsSuccessful() && nFetchPos >= (INT32)pFileSet->Count())
					{
						if (pEvaluationKeySet)
						{
							if (nFetchPos >= (INT32)pEvaluationKeySet->Count())
								aStatus.Set(SDB_STAT_NO_DATA_FOUND);
							else if (nFetchPos == 0)
							{
								pEvaluationKeySet->First();
								ExecuteRow(FILE_BOOKMARK,pEvaluationKeySet->GetKey(),FALSE,TRUE, bRetrieveData);
							}
							else
							{
								pEvaluationKeySet->Next();
								ExecuteRow(FILE_BOOKMARK,pEvaluationKeySet->GetKey(),FALSE,TRUE, bRetrieveData);
							}
						}
						else
							ExecuteRow(FILE_NEXT,1,FALSE,TRUE, bRetrieveData);
					}

					if (aStatus.IsSuccessful())
					{
						// jetzt nochmal die Ergebnisse lesen
						TABLE_ACTION(pTable,FetchRow(*aFileRow, *aFileColumns, bRetrieveData));

						// now set the bookmark for outside
						aResultRow->front()->setINT32(nFetchPos + 1);
					}
					else if (!pFileSet->IsFrozen())					// keinen gueltigen Satz gefunden
					{
						pFileSet->Freeze();

						DELETEZ(pEvaluationKeySet);
						aStatus.Set(SDB_STAT_NO_DATA_FOUND);
						goto Error;
					}
				}
			}
		}
	}
	else if (eStatementType == SDB_STATEMENT_SELECT_COUNT)
	{
		// Fetch des COUNT(*)
		switch (eCursorPosition)
		{
			case FILE_NEXT:
				nFetchPos++;
				break;
			case FILE_PRIOR:
				nFetchPos--;
				break;
			case FILE_FIRST:
				nFetchPos = 0;
				break;
			case FILE_LAST:
				nFetchPos = 0;
				break;
			case FILE_RELATIVE:
				nFetchPos += nOffset;
				break;
			case FILE_BOOKMARK:
			case FILE_ABSOLUTE:
				nFetchPos = nOffset - 1;
				break;
		}

		if (nFetchPos == 0)
		{
			// COUNT(*) in Ergebnisrow packen
			// (muss die erste und einzige Variable in der Row sein)
			if (aResultRow->size() == 2)
			{
				(*aResultRow)[1]->setINT32((long)nRowCountResult);
				(*aResultRow)[0]->setINT32(1L);
			}
			aStatus.Clear();
		}
		else
		{
			aStatus.Set(SDB_STAT_NO_DATA_FOUND);
			goto Error;
		}
	}
	else
		// Fetch nur bei SELECT moeglich!
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);

	goto End;

Error:
	// steht der Cursor vor dem ersten Satz
	// dann wird die position beibehalten
	if (nTempPos == -1)
		nFetchPos = nTempPos;
	else
	{
		switch(eCursorPosition)
		{
			case FILE_PRIOR:
			case FILE_FIRST:
				nFetchPos = -1;
				break;
			case FILE_LAST:
			case FILE_NEXT:
			case FILE_ABSOLUTE:
			case FILE_RELATIVE:
				if (nOffset > 0)
					nFetchPos = (pFileSet) ? pFileSet->Count() : MaxRowCount();
				else if (nOffset < 0)
					nFetchPos = -1;
				break;
			case FILE_BOOKMARK:
				nFetchPos = nTempPos;	 // vorherige Position
		}
	}
End:
	delete pGuard;
	rMode = (!bShowDeleted && aStatus.IsSuccessful() && aFileRow->isDeleted()) ?  // keine Anzeige von gelschten Stzen
				SdbCursor::SDB_MOD_INVALID : SdbCursor::SDB_MOD_NONE;
	return aStatus.IsSuccessful();
}


//------------------------------------------------------------------
void SdbFILECursor::GetTableName(String& rTableName, String& rTableRange)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);
	rTableName.Erase();
	rTableRange.Erase();

	if (pSQLParseNode == NULL)
	{
		aStatus.SetInvalidStatement();
		return;
	}


	SdbSqlParseNode * pTableName = NULL;

	// pTableName auf den (einzigen) Tabellennamen im Statement setzen.
	// Je nach Statement (SELECT, INSERT, UPDATE, DELETE) an anderer Position.
	// Bei Select darf nur ein Tabellenname auftreten, bei den uebrigen Statements
	// ist ohnehin nur einer moeglich.

	if (SDB_ISRULE(pSQLParseNode,select_statement))
	{
		// nyi: mehr Pruefung auf korrekte Struktur!
		SdbSqlParseNode * pTableRefCommalist = pSQLParseNode->GetChild(3)->GetChild(0)->GetChild(1);
		DBG_ASSERT(pTableRefCommalist != NULL,"SdbFILECursor: Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pTableRefCommalist,table_ref_commalist),"SdbFILECursor: Fehler im Parse Tree");

		// Muss genau eine Table enthalten!
		if (pTableRefCommalist->Count() != 1)
		{
			aStatus.SetStatementTooComplex();
			return;
		}

		if (SDB_ISRULE(pTableRefCommalist->GetChild(0),table_ref))
		{
			if (SDB_ISRULE(pTableRefCommalist->GetChild(0)->GetChild(0),table_name)){
				pTableName = pTableRefCommalist->GetChild(0)->GetChild(0);
				if(pTableRefCommalist->GetChild(0)->Count() == 4)
					rTableRange = pTableRefCommalist->GetChild(0)->GetChild(2)->TokenValue();
			}
		}
		else if (SDB_ISRULE(pTableRefCommalist->GetChild(0),table_name))
		{
			pTableName = pTableRefCommalist->GetChild(0);
			// Keine Range Variable
		}
		else
		{
			aStatus.SetInvalidStatement();
			return;
		}
	}
	else if (SDB_ISRULE(pSQLParseNode,insert_statement))
			pTableName = pSQLParseNode->GetChild(2);
	else if (SDB_ISRULE(pSQLParseNode,update_statement_positioned))
			pTableName = pSQLParseNode->GetChild(1);
	else if (SDB_ISRULE(pSQLParseNode,update_statement_searched))
			pTableName = pSQLParseNode->GetChild(1);
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_positioned))
			pTableName = pSQLParseNode->GetChild(2);
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_searched))
			pTableName = pSQLParseNode->GetChild(2);
	else
	{
		aStatus.SetInvalidStatement();
		return;
	}

	DBG_ASSERT(pTableName != NULL,"SdbFILECursor::GetTableName: pTableName == NULL");

	// Tabellenname abholen
	if (pTableName->Count() == 1)
		rTableName = pTableName->GetChild(0)->TokenValue();
	else if (pTableName->Count() == 3)
		// nyi: Schemaname?
		rTableName = pTableName->GetChild(2)->TokenValue();
	else
	{
		aStatus.SetInvalidStatement();
		return;
	}

	// Wenn keine Range Variable angegeben, dann den Tabellennamen verwenden.
	if (rTableRange.Len() == 0)
		rTableRange = rTableName;	// Double Quotes bereits entfernt.

}
//------------------------------------------------------------------
void SdbFILECursor::InsertAllColumns(USHORT &nExprCnt)
{
	String aStrExpr;aStrExpr.AssignAscii("expr");
	for (USHORT j = 1; j < aFileColumns->Count(); j++)
	{
		const SdbColumn * pFileColumn = (SdbColumn *)(*aFileColumns)[j];
		SdbColumn* pPrevCol = (SdbColumn*)aResultColumns->Column(pFileColumn->GetName());
		// Column-Struktur kopieren:
		SdbColumn * pColumn = NULL;
		if (!pPrevCol)
			pColumn = new SdbFILEColumn(*pFileColumn);
		else
		{
			// Name existiert bereits
			String aAlias(aStrExpr);
			aAlias += String::CreateFromInt32(++nExprCnt);
			pPrevCol->ResetFlag(SDB_FLAGS_UPDATEALLOWED);
			pColumn = new SdbFILEColumn(aAlias, *pFileColumn);
			// nicht updatebar
			pColumn->ResetFlag(SDB_FLAGS_UPDATEALLOWED);
		}

		// Column-Beschreibung hinzufuegen.
		aResultColumns->AddColumn(pColumn);
	}
}
//------------------------------------------------------------------
void SdbFILECursor::GetColumnNames()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	if (! pSQLParseNode ||
		! SDB_ISRULE(pSQLParseNode,select_statement))
	{
		return;
	}

	SdbSqlParseTreeIterator* pIter = new SdbSqlParseTreeIterator(GetFILEConnection(),pSQLParseNode);
	pIter->TraverseAll();
	if(pIter->IteratorStatus().IsSuccessful())
	{
		const SdbColumnsRef rColumns = pIter->GetSelectColumns();
		for(USHORT i=1;i<rColumns->Count();i++)
		{
			const SdbColumn* pColumn = (*rColumns)[i];
			if(pColumn->IsFunction())
			{
				String aRealName(pColumn->GetRealName());
				aRealName.EraseAllChars(' ');
				if(aRealName.EqualsIgnoreCaseAscii("COUNT(*)"))
				{
					((SdbColumn*)pColumn)->SetType(SDB_DBTYPE_INTEGER);
					eStatementType = SDB_STATEMENT_SELECT_COUNT;
				}
				else
				{
					aStatus.SetStatementTooComplex();
					continue;
				}

			}
			aResultColumns->AddColumn(pColumn->Clone());
		}

	}
	else
		aStatus = pIter->IteratorStatus();
	delete 	pIter;
}


//------------------------------------------------------------------
void SdbFILECursor::AnylizeSQL()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	if (! pSQLParseNode)
		return;

	// Parse Tree analysieren (je nach Statement-Typ)
	// und Zeiger auf WHERE-Klausel setzen:
	aSQLAnalyzer.setOrigColumns(aFileColumns);
	aSQLAnalyzer.Start(pSQLParseNode);

	SdbSqlParseNode * pOrderbyClause = NULL;

	if (SDB_ISRULE(pSQLParseNode,select_statement))
	{
		DBG_ASSERT(pSQLParseNode->Count() >= 4,"SdbFILECursor: Fehler im Parse Tree");
		SdbSqlParseNode * pTableExp = pSQLParseNode->GetChild(3);
		pOrderbyClause = pTableExp->GetChild(4);
	}

	// ggf. ORDER BY analysieren (sofern vorhanden):
	if (pOrderbyClause)
	{
		DBG_ASSERT(SDB_ISRULE(pOrderbyClause,opt_order_by_clause),"SdbFILECursor: Fehler im Parse Tree");
		if (pOrderbyClause->Count() > 0)
		{
			DBG_ASSERT(pOrderbyClause->Count() == 3,"SdbFILECursor: Fehler im Parse Tree");
			DBG_ASSERT(SDB_ISTOKEN(pOrderbyClause->GetChild(0),"ORDER"),"SdbFILECursor: Fehler im Parse Tree");
			DBG_ASSERT(SDB_ISTOKEN(pOrderbyClause->GetChild(1),"BY"),"SdbFILECursor: Fehler im Parse Tree");

			SdbSqlParseNode * pOrderingSpecCommalist = pOrderbyClause->GetChild(2);
			DBG_ASSERT(SDB_ISRULE(pOrderingSpecCommalist,ordering_spec_commalist),"SdbFILECursor: Fehler im Parse Tree");

			for (ULONG m = 0; m < pOrderingSpecCommalist->Count(); m++)
			{
				SdbSqlParseNode * pOrderingSpec = pOrderingSpecCommalist->GetChild(m);
				DBG_ASSERT(SDB_ISRULE(pOrderingSpec,ordering_spec),"SdbFILECursor: Fehler im Parse Tree");
				DBG_ASSERT(pOrderingSpec->Count() == 2,"SdbFILECursor: Fehler im Parse Tree");

				SdbSqlParseNode * pColumnRef = pOrderingSpec->GetChild(0);
				if(!SDB_ISRULE(pColumnRef,column_ref))
				{
					aStatus.SetStatementTooComplex();
					return;
				}
				SdbSqlParseNode * pAscendingDescending = pOrderingSpec->GetChild(1);
				SetOrderbyColumn((UINT16)m,pColumnRef,pAscendingDescending);
			}
		}
	}
	// Fehler wird ggf. einfach weitergereicht.
}


//------------------------------------------------------------------
void SdbFILECursor::SetOrderbyColumn(UINT16 nOrderbyColumnNo,
									 SdbSqlParseNode* pColumnRef,
									 SdbSqlParseNode* pAscendingDescending)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	if (nOrderbyColumnNo >= (sizeof nOrderbyColumnNumber / sizeof (* nOrderbyColumnNumber)))
	{
		aStatus.SetStatementTooComplex();
		return;
	}

	String aColumnName;
	if (pColumnRef->Count() == 1)
		aColumnName = pColumnRef->GetChild(0)->TokenValue();
	else if (pColumnRef->Count() == 3)
	{
		// Nur die Table Range-Variable darf hier vorkommen:
		if (!(pColumnRef->GetChild(0)->TokenValue() == aTableRange))
		{
			aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S1000"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_RANGE_VAR))),
						0, String() );
			return;
		}
		aColumnName = pColumnRef->GetChild(2)->TokenValue();
	}
	else
	{
		aStatus.SetStatementTooComplex();
		return;
	}

	SdbColumn* pFileColumn = (SdbColumn*)(*aFileColumns)[aColumnName];

	// besonderheit, intern kann auch nach der Bookmarksplate sortiert werden
	if (pFileColumn == NULL)
	{
		if (aColumnName == aFileColumns->Column(0)->GetName())
			pFileColumn = (SdbColumn*)aFileColumns->Column(0);
		else
		{
			// Diese Column gibt es nicht!
			aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S0022"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_COLUMN_NOT_FOUND))),
						0, String() );
			return;
		}
	}

	// Alles geprueft und wir haben den Namen der Column.
	// Die wievielte Column ist das?
	nOrderbyColumnNumber[nOrderbyColumnNo] = pFileColumn->GetId();

	// Ascending or Descending?
	bOrderbyAscending[nOrderbyColumnNo]  = (SDB_ISTOKEN(pAscendingDescending,"DESC")) ?
											FALSE : TRUE;
}


//------------------------------------------------------------------
void SdbFILECursor::GetAssignValues()
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	if (pSQLParseNode == NULL)
	{
		aStatus.SetInvalidStatement();
		return;
	}

	if (SDB_ISRULE(pSQLParseNode,select_statement))
		// Keine zu setzenden Werte bei SELECT
		return;
	else if (SDB_ISRULE(pSQLParseNode,insert_statement))
	{
		// Row fuer die zu setzenden Werte anlegen (Referenz durch new)
		aAssignValues = new ODbAssignRow(*aFileColumns);

		// Liste der Columns-Namen, die in der column_commalist vorkommen (mit ; getrennt):
		String aColumnNameList;

		DBG_ASSERT(pSQLParseNode->Count() >= 4,"SdbFILECursor: Fehler im Parse Tree");

		SdbSqlParseNode * pOptColumnCommalist = pSQLParseNode->GetChild(3);
		DBG_ASSERT(pOptColumnCommalist != NULL,"SdbFILECursor: Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pOptColumnCommalist,opt_column_commalist),"SdbFILECursor: Fehler im Parse Tree");
		if (pOptColumnCommalist->Count() == 0)
		{
			// Liste fehlt. Alle Column-Namen der Datei benutzen:
			// nyi: hoffentlich enthalten die Column-Namen kein Semikolon!
			for (USHORT i = 1; i < aFileColumns->Count(); i++)
			{
				const SdbColumn * pColumn = aFileColumns->Column(i);
				DBG_ASSERT(pColumn != NULL,"SdbFILECursor: pColumn == NULL");
				aColumnNameList += pColumn->GetName();
				aColumnNameList += ';';
			}
		}
		else
		{
			DBG_ASSERT(pOptColumnCommalist->Count() == 3,"SdbFILECursor: Fehler im Parse Tree");

			SdbSqlParseNode * pColumnCommalist = pOptColumnCommalist->GetChild(1);
			DBG_ASSERT(pColumnCommalist != NULL,"SdbFILECursor: Fehler im Parse Tree");
			DBG_ASSERT(SDB_ISRULE(pColumnCommalist,column_commalist),"SdbFILECursor: Fehler im Parse Tree");
			DBG_ASSERT(pColumnCommalist->Count() > 0,"SdbFILECursor: Fehler im Parse Tree");

			// Alle Columns in der column_commalist ...
			for (ULONG i = 0; i < pColumnCommalist->Count(); i++)
			{
				SdbSqlParseNode * pCol = pColumnCommalist->GetChild(i);
				DBG_ASSERT(pCol != NULL,"SdbFILECursor: Fehler im Parse Tree");
				aColumnNameList += pCol->TokenValue();
				aColumnNameList += ';';
			}
		}
		aColumnNameList.EraseTrailingChars(';');
		if(!aColumnNameList.Len())
		{
			aStatus.SetInvalidStatement();
			return;
		}

		// Werte ...
		SdbSqlParseNode * pValuesOrQuerySpec = pSQLParseNode->GetChild(4);
		DBG_ASSERT(pValuesOrQuerySpec != NULL,"SdbFILECursor: pValuesOrQuerySpec darf nicht NULL sein!");
		DBG_ASSERT(SDB_ISRULE(pValuesOrQuerySpec,values_or_query_spec),"SdbFILECursor: ! SDB_ISRULE(pValuesOrQuerySpec,values_or_query_spec)");
		DBG_ASSERT(pValuesOrQuerySpec->Count() > 0,"SdbFILECursor: pValuesOrQuerySpec->Count() <= 0");

		// nur "VALUES" ist erlaubt ...
		if (! SDB_ISTOKEN(pValuesOrQuerySpec->GetChild(0),"VALUES"))
		{
			aStatus.SetStatementTooComplex();
			return;
		}

		DBG_ASSERT(pValuesOrQuerySpec->Count() == 2,"SdbFILECursor: pValuesOrQuerySpec->Count() != 2");

		// Liste von Werten
		SdbSqlParseNode * pInsertAtomCommalist = pValuesOrQuerySpec->GetChild(1);
		DBG_ASSERT(pInsertAtomCommalist != NULL,"SdbFILECursor: pInsertAtomCommalist darf nicht NULL sein!");
		DBG_ASSERT(pInsertAtomCommalist->Count() > 0,"SdbFILECursor: pInsertAtomCommalist <= 0");

		String aColumnName;
		SdbSqlParseNode * pRow_Value_Const;
		xub_StrLen nIndex=0;
		for (ULONG i = 0; i < pInsertAtomCommalist->Count(); i++)
		{
			pRow_Value_Const = pInsertAtomCommalist->GetChild(i); // row_value_constructor
			if(pRow_Value_Const->Count() == 3)	// '(' row_value_const_list ')'
			{
				pRow_Value_Const = pRow_Value_Const->GetChild(1); // row_value_const_list
				DBG_ASSERT(pRow_Value_Const != NULL,"SdbFILECursor: pRow_Value_Const darf nicht NULL sein!");
				if(pRow_Value_Const->Count() == 0)
				{
					if(pRow_Value_Const->Count() == (aColumnNameList.GetTokenCount()-1))
						ParseAssignValues(aColumnNameList,pRow_Value_Const,nIndex++); // kann nur ein Columnname vorhanden sein pro Schleife
					else
					{
						aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S1000"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_SYNTAX_ERROR))),
						0, String() );
					}
				}
				else
				{
					if(pRow_Value_Const->Count() == aColumnNameList.GetTokenCount())
					{
						for (ULONG j = 0; j < pRow_Value_Const->Count() && aStatus.IsSuccessful(); j++)
							ParseAssignValues(aColumnNameList,pRow_Value_Const->GetChild(j),nIndex++);
					}
					else
					{
						aStatus.Set(SDB_STAT_ERROR,
						String::CreateFromAscii("S1000"),
						aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_SYNTAX_ERROR))),
						0, String() );
					}

				}
			}
			else
			{
				aStatus.SetStatementTooComplex();
				return;
			}
		}
	}
	else if (SDB_ISRULE(pSQLParseNode,update_statement_positioned) ||
			   SDB_ISRULE(pSQLParseNode,update_statement_searched))
	{
		aAssignValues = new ODbAssignRow(*aFileColumns);

		DBG_ASSERT(pSQLParseNode->Count() >= 4,"SdbFILECursor: Fehler im Parse Tree");

		SdbSqlParseNode * pAssignmentCommalist = pSQLParseNode->GetChild(3);
		DBG_ASSERT(pAssignmentCommalist != NULL,"SdbFILECursor: pAssignmentCommalist == NULL");
		DBG_ASSERT(SDB_ISRULE(pAssignmentCommalist,assignment_commalist),"SdbFILECursor: Fehler im Parse Tree");
		DBG_ASSERT(pAssignmentCommalist->Count() > 0,"SdbFILECursor: pAssignmentCommalist->Count() <= 0");

		// Alle Zuweisungen (Kommaliste) bearbeiten ...
		for (ULONG i = 0; i < pAssignmentCommalist->Count() && aStatus.IsSuccessful(); i++)
		{
			SdbSqlParseNode * pAssignment = pAssignmentCommalist->GetChild(i);
			DBG_ASSERT(pAssignment != NULL,"SdbFILECursor: pAssignment == NULL");
			DBG_ASSERT(SDB_ISRULE(pAssignment,assignment),"SdbFILECursor: Fehler im Parse Tree");
			DBG_ASSERT(pAssignment->Count() == 3,"SdbFILECursor: pAssignment->Count() != 3")

			SdbSqlParseNode * pCol = pAssignment->GetChild(0);
			DBG_ASSERT(pCol != NULL,"SdbFILECursor: pCol == NULL")

			SdbSqlParseNode * pComp = pAssignment->GetChild(1);
			DBG_ASSERT(pComp != NULL,"SdbFILECursor: pComp == NULL")
			DBG_ASSERT(pComp->GetNodeType() == SDB_SQL_EQUAL,"SdbFILECursor: pComp->GetNodeType() != SDB_SQL_COMPARISON");
			if (pComp->TokenValue().GetChar(0) != '=')
			{
				aStatus.SetInvalidStatement();
				return;
			}

			SdbSqlParseNode * pVal = pAssignment->GetChild(2);
			DBG_ASSERT(pVal != NULL,"SdbFILECursor: pVal == NULL");
			ParseAssignValues(pCol->TokenValue(),pVal,0);
		}
	}
}

void SdbFILECursor::ParseAssignValues(const String& aColumnNameList,SdbSqlParseNode* pRow_Value_Constructor_Elem,xub_StrLen nIndex)
{
	DBG_ASSERT(nIndex <= aColumnNameList.GetTokenCount(),"SdbFileCursor::ParseAssignValues: nIndex > aColumnNameList.GetTokenCount()");
	String aColumnName(aColumnNameList.GetToken(nIndex,';'));
	DBG_ASSERT(aColumnName.Len() > 0,"SdbFILECursor: Column-Name nicht gefunden");
	DBG_ASSERT(pRow_Value_Constructor_Elem != NULL,"SdbFILECursor: pRow_Value_Constructor_Elem darf nicht NULL sein!");

	if (pRow_Value_Constructor_Elem->GetNodeType() == SDB_SQL_STRING ||
		pRow_Value_Constructor_Elem->GetNodeType() == SDB_SQL_INTNUM ||
		pRow_Value_Constructor_Elem->GetNodeType() == SDB_SQL_APPROXNUM)
	{
		// Wert setzen:
		SetAssignValue(aColumnName, pRow_Value_Constructor_Elem->TokenValue());
		if (! aStatus.IsSuccessful()) return;
	}
	else if (SDB_ISTOKEN(pRow_Value_Constructor_Elem,"NULL"))
	{
		// NULL setzen
		SetAssignValue(aColumnName, String(), TRUE);
		if (! aStatus.IsSuccessful()) return;
	}
	else if (SDB_ISRULE(pRow_Value_Constructor_Elem,parameter))
	{
		// Parameter hinzufuegen, Typ ... entsprechend der Column, der der Wert zugewiesen wird
		UINT32 nParameter = AddParameter(pRow_Value_Constructor_Elem,(*aFileColumns)[aColumnName]);
		// Nr. des Parameters in der Variablen merken:
		SetAssignValue(aColumnName, String(), TRUE, nParameter);
	}
	else
	{
		aStatus.SetStatementTooComplex();
		return;
	}
}
//------------------------------------------------------------------
void SdbFILECursor::SetAssignValue(const String& aColumnName,
								   const String& aValue,
								   BOOL bSetNull,
								   UINT32 nParameter)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	// Kommt diese Column ueberhaupt in der Datei vor?
	const SdbColumn* pFileColumn = (*aFileColumns)[aColumnName];

	if (pFileColumn == NULL)
	{
		// Diese Column gibt es nicht!
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("S0022"),
					aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_COLUMN_NOT_FOUND))),
					0, String() );
		return;
	}

	// Value an die Row mit den zuzuweisenden Werten binden:
	const ODbVariantRef& xValue = (*aAssignValues)[pFileColumn->GetId()];

	// Alles geprueft und wir haben den Namen der Column.
	// Jetzt eine Value allozieren, den Wert setzen und die Value an die Row binden.
	if (bSetNull)
		xValue->setNull();
	else
	{
		switch (pFileColumn->GetType())
		{
			// Kriterium je nach Typ als String oder double in die Variable packen ...
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				xValue->setString(aValue);
				// Zeichensatz ist bereits konvertiert, da ja das gesamte Statement konvertiert wurde
				break;

			case SDB_DBTYPE_BOOLEAN:
				{
					// nyi: gemischte Gross-/Kleinschreibung???
					if (aValue.EqualsIgnoreCaseAscii("TRUE")  || aValue.GetChar(0) == '1')
						xValue->setDouble(TRUE);
					else if (aValue.EqualsIgnoreCaseAscii("FALSE") || aValue.GetChar(0) == '0')
						xValue->setDouble(FALSE);
					else
					{
						aStatus.Set(SDB_STAT_ERROR);	// nyi: genauer!
						return;
					}
				}
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_REAL:
			case SDB_DBTYPE_DOUBLE:
			case SDB_DBTYPE_DATE:
			case SDB_DBTYPE_TIME:
			case SDB_DBTYPE_TIMESTAMP:
			{
				xValue->setString(aValue);
				try
				{
					double n = xValue->toDouble();
					xValue->setDouble(n);
				}
				catch ( ... )
				{
					aStatus.SetDriverNotCapableError();
				}
			}	break;
			default:
				aStatus.SetDriverNotCapableError();
				return;
		}
	}

	// Parameter-Nr. merken (als User Data)
	// SDB_NO_PARAMETER = kein Parameter.
	((ODbAssignRow*)aAssignValues.getBodyPtr())->setAssignedIndex(pFileColumn->GetId(), nParameter);
}

//------------------------------------------------------------------
UINT32 SdbFILECursor::AddParameter(SdbSqlParseNode * pParameter, const SdbColumn* pInfoColumn)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	// Nr. des neu hinzuzufuegenden Parameters:
	UINT32 nParameter = aParameterColumns->Count();

	DBG_ASSERT(SDB_ISRULE(pParameter,parameter),"SdbFILECursor::AddParameter: Argument ist kein Parameter");
	DBG_ASSERT(pParameter->Count() > 0,"SdbFILECursor: Fehler im Parse Tree");
	SdbSqlParseNode * pMark = pParameter->GetChild(0);

	String aParameterName;
	if (SDB_ISPUNCTUATION(pMark,"?"))
	{
		// Beim vorherigen Translate des Statements wurden bereits die
		// benannten Parameter in '?' uebersetzt. Die Parameter-Namen
		// sind dann allerdings noch verfuegbar:
		aParameterName = aParameterNames.GetToken((USHORT) nParameter - 1);
		// Name == "?", falls kein Parametername verfuegbar (z. B. bei Native SQL)
		if (aParameterName.Len() == 0) aParameterName = '?';
	}
	else if (SDB_ISPUNCTUATION(pMark,":"))
		aParameterName = pParameter->GetChild(1)->TokenValue();
	else if (SDB_ISPUNCTUATION(pMark,"["))
		aParameterName = pParameter->GetChild(1)->TokenValue();
	else
	{
		DBG_ERROR("SdbFILECursor: Fehler im Parse Tree");
	}

	// Parameter-Column aufsetzen:
	SdbDatabaseType eType = SDB_DBTYPE_VARCHAR;
	UINT32 nPrecision = 255;
	UINT16 nScale = 0;
	BOOL bNullable = TRUE;

	if (pInfoColumn)
	{
		// Typ, Precision, Scale ... der angegebenen Column verwenden,
		// denn dieser Column wird der Wert zugewiesen bzw. mit dieser
		// Column wird der Wert verglichen.
		eType = pInfoColumn->GetType();
		nPrecision = pInfoColumn->GetPrecision();
		nScale = pInfoColumn->GetScale();
		bNullable = pInfoColumn->IsNullAllowed();
	}

	SdbColumn * pColumn = new SdbFILEColumn(aParameterName,eType,nPrecision,nScale);
	if (bNullable) pColumn->SetFlag(SDB_FLAGS_NULLALLOWED);

	// Column-Beschreibung hinzufuegen.
	aParameterColumns->AddColumn(pColumn);
	return nParameter;
}


//------------------------------------------------------------------
SdbFILEKeyValue* SdbFILECursor::GetOrderbyKeyValue(ODbRowRef& rRow)
{
	DBG_CHKTHIS(SdbFILECursor,NULL);

	// Value:
	const ODbVariantRef& xBookmarkValue = rRow->front();
	DBG_ASSERT(!xBookmarkValue->isNull(),"SdbFILECursor::Execute: Bookmark Column nicht gebunden");
	UINT32 nBookmarkValue = Abs(xBookmarkValue->getINT32());

	SdbFILEKeyValue* pKeyValue = new SdbFILEKeyValue((UINT32)nBookmarkValue);
	for (int i = 0; i < sizeof nOrderbyColumnNumber / sizeof (* nOrderbyColumnNumber); i++)
	{
		if (nOrderbyColumnNumber[i] == SDB_COLUMN_NOTFOUND) break;

		const ODbVariantRef& xKey = (*rRow)[nOrderbyColumnNumber[i]];
		switch (rRow->getColumns()->Column(nOrderbyColumnNumber[i])->GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				if (!xKey.is() || xKey->isNull())
					pKeyValue->SetKey(i,rtl::OUString());		// NULL etc. wie Leerstring sortieren
				else
					pKeyValue->SetKey(i,xKey->getString());
				break;
			case SDB_DBTYPE_BOOKMARK:	//Sonderfall siehe Indices
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_REAL:
			case SDB_DBTYPE_DOUBLE:
			case SDB_DBTYPE_DATE:
			case SDB_DBTYPE_TIME:
			case SDB_DBTYPE_TIMESTAMP:
			case SDB_DBTYPE_BOOLEAN:
				if (!xKey.is() || xKey->isNull())
					pKeyValue->SetKey(i,-DBL_MAX);		// NULL etc. wie negativste Zahl sortieren
														// nyi: Ggf. unschoen, wenn diese Zahl vorkommt - ist aber seeeehr unwahrscheinlich.
				else
				{
					try
					{
						pKeyValue->SetKey(i,xKey->toDouble());
					}
					catch (...)
					{
						pKeyValue->SetKey(i,0);
					}
				}
				break;
			default:
				DBG_ERROR("SdbFILECursor::GetOrderbyKeyValue: Datentyp nicht implementiert");
				break;
		}
	}
	return pKeyValue;
}

//------------------------------------------------------------------
BOOL SdbFILECursor::InsertImpl()
{
	// exclusiver zugriff auf die Tabelle
	NAMESPACE_VOS(OGuard)* pGuard = pTable->Lock();

	TABLE_ACTION(pTable, InsertRow(aResultRow, !IsInsertOnly_OldAPI()));

	// Einfuegen ins Keyset
	if (aStatus.IsSuccessful() && pFileSet && pFileSet->IsFrozen())
	{
		pFileSet->Append((ULONG)Abs(aResultRow->front()->getINT32()));
		aResultRow->front()->setINT32(pFileSet->Count());
	}

	if (aStatus.IsSuccessful())
		nLastVisitedPos = pTable->GetCurrentLastPos();

	delete pGuard;
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbFILECursor::UpdateImpl()
{
	// exclusiver zugriff auf die Tabelle
	NAMESPACE_VOS(OGuard)* pGuard = pTable->Lock();

	TABLE_ACTION(pTable,UpdateRow(aResultRow, aOriginalRow.getBodyPtr()));

	delete pGuard;
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
BOOL SdbFILECursor::DeleteImpl()
{
	// exclusiver zugriff auf die Tabelle
	NAMESPACE_VOS(OGuard)* pGuard = pTable->Lock();

	TABLE_ACTION(pTable,DeleteRow());

	if (aStatus.IsSuccessful() && nFetchPos >= 0)
		nFetchPos--;	// weg vom geloeschten

	delete pGuard;
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
INT32 SdbFILECursor::MaxRowCount() const
{
	return -1;
}

//------------------------------------------------------------------
BOOL SdbFILECursor::DetermineRowCountImpl(INT32& nCount)
{
	if (nRowCountResult >= 0)
	{
		nCount = 1;
		return TRUE;
	}
	else if (!HasRestriction() && !IsSorted() && MaxRowCount() >= 0 && bShowDeleted)
	{
		nCount = MaxRowCount();
		return TRUE;
	}
	else if (pFileSet && pFileSet->IsFrozen())
	{
		nCount = pFileSet->Count();
		return TRUE;
	}
	else
		return FALSE;
}

//------------------------------------------------------------------
BOOL SdbFILECursor::SetUpdatableImpl(BOOL bUpdate)
{
	if (bUpdate && pTable->IsReadOnly())
	{
		aStatus.SetError(SVSTREAM_ACCESS_DENIED,
						 dbTable,
						 pTable->Name());
		nPrivileges = SDB_PR_READ;
	}
	else if (bUpdate)
	{
		if (GetFILEConnection()->GetShowDeleted())
			nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE;
		else
			nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_DELETE;
	}
	else
		nPrivileges = SDB_PR_READ;

	return aStatus.IsSuccessful();
}
//------------------------------------------------------------------
BOOL SdbFILECursor::IsCapable(SdbCursorCapability eCapability) const
{
	BOOL bResult = FALSE;
	switch(eCapability)
	{
		case SDB_CAP_UPDATE_POSITIONED:
		case SDB_CAP_FETCH:
		case SDB_CAP_FETCH_NEXT:
		case SDB_CAP_FETCH_PRIOR:
		case SDB_CAP_FETCH_FIRST:
		case SDB_CAP_FETCH_LAST:
		case SDB_CAP_FETCH_RELATIVE:
		case SDB_CAP_FETCH_ABSOLUTE:
		case SDB_CAP_BOOKMARKS:
			bResult = TRUE;

	}
	return bResult;
}

//------------------------------------------------------------------
BOOL SdbFILECursor::MoveImpl(const ::com::sun::star::uno::Any& rBookmark, SdbCursorMode& rMode, BOOL bRetrieveData)  throw( ::com::sun::star::lang::IllegalArgumentException )
{
	// do we have a bookmark of valid type
	if (rBookmark.getValueTypeClass() != ::com::sun::star::uno::TypeClass_LONG)
		throw ::com::sun::star::lang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);

	sal_Int32 nBookmark;
	rBookmark >>= nBookmark;
	return Move(FILE_BOOKMARK, nBookmark, rMode, bRetrieveData);
}

