/*************************************************************************
 *
 *  $RCSfile: fileset.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "fileset.hxx"

#include <stdlib.h>

#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#ifndef _INTN_HXX //autogen
#include <tools/intn.hxx>
#endif

DBG_NAME(SdbFILESortIndex);

SdbFILESortIndex * SdbFILESortIndex::pCurrentIndex;
CharSet SdbFILESortIndex::eCurrentCharSet;


#if defined (WIN)
// Maximale Anzahl von Rows, die mit ORDER BY sortiert durchlaufen werden koennen:
#define MAX_KEYSET_SIZE 0x3ff0	// Etwas weniger, damit noch Platz fuer Memory Debug-Informationen
#endif
DBG_NAME(SdbFILESet);
DBG_NAME(SdbFILEKeyValue);
//------------------------------------------------------------------
SdbFILESet::SdbFILESet(const SdbEvaluateSet& rEvalSet)
		  :vector<INT32>(rEvalSet.Count())
		  ,bFrozen(FALSE)
		  ,nPos(0)
{
	DBG_CTOR(SdbFILESet,NULL);
	reserve(16);
	for (ULONG i = 0; i < rEvalSet.Count(); i++)
		(*this)[i] = rEvalSet.GetKey(i);
}

SdbFILESet::~SdbFILESet()
{
	DBG_DTOR(SdbFILESet,NULL);
}
//------------------------------------------------------------------
SdbFILEKeyValue::~SdbFILEKeyValue()
{
	DBG_DTOR(SdbFILEKeyValue,NULL);
	/* nyi: Strings aufraeumen! - hier nicht moeglich, wegen Union! */
}

//------------------------------------------------------------------
SdbFILESortIndex::SdbFILESortIndex(const SdbKeyType eKeyType2[],	// Genau 3 Eintraege!
						   const BOOL bAscending2[],		// Genau 3 Eintraege!
						   INT32 nMaxNumberOfRows, CharSet eSet)	// Obere Schranke fuer die Anzahl indizierbarer Zeilen
	: nMaxCount(nMaxNumberOfRows),
	  nCount(0),
	  bFrozen(FALSE), eCharSet(eSet)
{
	DBG_CTOR(SdbFILESortIndex,NULL);

	for (int j = 0; j < SDB_ORDERBYKEYS; j++)
	{
		eKeyType[j] = eKeyType2[j];
		bAscending[j] = bAscending2[j];
	}

#if defined MAX_KEYSET_SIZE
	// Zur Sicherheit Maximalgroesse nochmal pruefen:
	if (nMaxCount > MAX_KEYSET_SIZE)
	{
		DBG_WARNING("SdbFILESortIndex::SdbFILESortIndex: nMaxNumberOfRows zur Zeit auf <16K beschraenkt!");
		nMaxCount = MAX_KEYSET_SIZE;
	}
#endif

	ppKeyValueArray = new SdbFILEKeyValuePtr[nMaxCount];

	for (INT32 i = 0; i < nMaxCount; i++)
		ppKeyValueArray[i] = NULL;
}

//------------------------------------------------------------------
SdbFILESortIndex::~SdbFILESortIndex()
{
	DBG_DTOR(SdbFILESortIndex,NULL);

// Nicht mehr noetig - wird durch Freeze() erledigt ...
//	for (INT32 i = 0; i < nCount; i++) {
//		delete ppKeyValueArray[i];
//	}

	__DELETE(nMaxCount) ppKeyValueArray;
}


//------------------------------------------------------------------
BOOL SdbFILESortIndex::AddKeyValue(SdbFILEKeyValue * pKeyValue)
{
	DBG_CHKTHIS(SdbFILESortIndex,NULL);

	if (nCount < nMaxCount)
	{
		if (bFrozen)							// wenn der Index schon eingefroren
												// dann wird der Key einfach ans Ende gehaengt
		{
			DBG_ASSERT(pKeyValue != NULL,"SdbFILESortIndex::Freeze: pKeyValue == NULL");
			INT32 nValue = pKeyValue->GetValue();		// Wert holen ...

			// Strings in KeyValue freigeben!
			for (int j = 0; j < SDB_ORDERBYKEYS; j++)
			{
				if (eKeyType[j] == SDB_ORDERBYKEY_STRING)
					delete pKeyValue->GetKeyString(j);
			}
			delete pKeyValue;
			ppKeyValueArray[nCount++] = (SdbFILEKeyValuePtr) nValue;
		}
		else
			ppKeyValueArray[nCount++] = pKeyValue;
		return TRUE;
	}
	else
		return FALSE;
}


//------------------------------------------------------------------
void SdbFILESortIndex::Freeze()
{
	DBG_CHKTHIS(SdbFILESortIndex,NULL);

	DBG_ASSERT(! bFrozen,"SdbFILESortIndex::Freeze: already frozen!");

	// Kritischer Bereich: Hinterlegung von this in statischer Variable.
	// Zugriff auf diese Variable von der SdbFILECompare-Funktion aus.
	// Da dies NUR waehrend der Ausfuehrung der qsort-Funktion stattfindet,
	// ist dies aber unkritisch: unter Windows 3.x ist diese Ausfuehrung
	// UNUNTERBRECHBAR; unter NT, OS/2, Unix, ... hat jede DLL ihr
	// eigenes Datensegment.
	pCurrentIndex = this;
	eCurrentCharSet = eCharSet;

	// Sortierung:
	if (eKeyType[0] != SDB_ORDERBYKEY_NONE)
		// Sortierung, wenn mindestens nach dem ersten Key sortiert werden soll:
		qsort(ppKeyValueArray,nCount,sizeof(void *),&SdbFILEKeyCompare);


	// Ende des kritischen Bereiches
	pCurrentIndex = NULL;

	// Wert auslesen, KeyValue loeschen und in den void * den Value
	// reinschreiben (uebler Trick mit Typecast!)
	for (INT32 i = 0; i < nCount; i++)
	{
		SdbFILEKeyValuePtr pKeyValue = ppKeyValueArray[i];

		DBG_ASSERT(pKeyValue != NULL,"SdbFILESortIndex::Freeze: pKeyValue == NULL");
		INT32 nValue = pKeyValue->GetValue();		// Wert holen ...

		// Strings in KeyValue freigeben!
		for (int j = 0; j < SDB_ORDERBYKEYS; j++)
		{
			if (eKeyType[j] == SDB_ORDERBYKEY_STRING)
				delete pKeyValue->GetKeyString(j);
		}
		delete pKeyValue;			
		ppKeyValueArray[i] = (SdbFILEKeyValuePtr) nValue;
	}

	bFrozen = TRUE;
}

//------------------------------------------------------------------
INT32 SdbFILESortIndex::GetValue(INT32 nPos) const
{
	DBG_CHKTHIS(SdbFILESortIndex,NULL);

	DBG_ASSERT(nPos > 0,"SdbFILESortIndex::GetValue: nPos == 0");
	DBG_ASSERT(nPos <= nCount,"SdbFILESortIndex::GetValue: Zugriff ausserhalb der Array-Grenzen");

//	DBG_ASSERT(ppKeyValueArray[nPos-1] != NULL,"SdbFILESortIndex::GetValue: interner Fehler: kein KeyValue an dieser Stelle");
//	return ppKeyValueArray[nPos-1]->GetValue();

	if (!bFrozen)
	{
		if (eKeyType[0] == SDB_ORDERBYKEY_NONE)  // wenn keine Sortierung vorliegt
												 // darf auf die Values schon vorher zugegriffen werden
			return ppKeyValueArray[nPos-1]->GetValue();
		else
		{
			DBG_ERROR("SdbFILESortIndex::GetValue: Invalid use of index!");
			return 0;
		}
	}
	else
		return (INT32) ppKeyValueArray[nPos-1];	// Trick: nach Freeze sind hier nur noch Values, keine KeyValue-Strukturen mehr!

}

//------------------------------------------------------------------
SdbFILESet* SdbFILESortIndex::CreateKeySet()
{

	DBG_ASSERT(! bFrozen,"SdbFILESortIndex::Freeze: already frozen!");

	// Kritischer Bereich: Hinterlegung von this in statischer Variable.
	// Zugriff auf diese Variable von der SdbFILECompare-Funktion aus.
	// Da dies NUR waehrend der Ausfuehrung der qsort-Funktion stattfindet,
	// ist dies aber unkritisch: unter Windows 3.x ist diese Ausfuehrung
	// UNUNTERBRECHBAR; unter NT, OS/2, Unix, ... hat jede DLL ihr
	// eigenes Datensegment.
	pCurrentIndex = this;
	eCurrentCharSet = eCharSet;

	// Sortierung:
	if (eKeyType[0] != SDB_ORDERBYKEY_NONE)
		// Sortierung, wenn mindestens nach dem ersten Key sortiert werden soll:
		qsort(ppKeyValueArray,nCount,sizeof(void *),&SdbFILEKeyCompare);


	// Ende des kritischen Bereiches
	pCurrentIndex = NULL;


	SdbFILESet* pKeySet = new SdbFILESet(FALSE,USHORT(nCount));
	for (INT32 i = 0; i < nCount; i++)
	{
		SdbFILEKeyValuePtr pKeyValue = ppKeyValueArray[i];

		DBG_ASSERT(pKeyValue != NULL,"SdbFILESortIndex::Freeze: pKeyValue == NULL");
		INT32 nValue = pKeyValue->GetValue();		// Wert holen ...

		// Strings in KeyValue freigeben!
		for (int j = 0; j < SDB_ORDERBYKEYS; j++)
		{
			if (eKeyType[j] == SDB_ORDERBYKEY_STRING)
				delete pKeyValue->GetKeyString(j);
		}
		delete pKeyValue;

		pKeySet->Append(nValue);		
	}							
	bFrozen = TRUE;
	pKeySet->Freeze();
	return pKeySet;
}

//------------------------------------------------------------------
static int
#if defined(WIN) || defined(WNT)
__cdecl
#endif
#if defined(ICC) && defined(OS2)
_Optlink
#endif
SdbFILEKeyCompare(const void * elem1, const void * elem2)
{
	const SdbFILESortIndex * pIndex = SdbFILESortIndex::pCurrentIndex;
	const SdbFILEKeyValue * pKeyValue1 = (SdbFILEKeyValue *) * (SdbFILEKeyValue **) elem1;
	const SdbFILEKeyValue * pKeyValue2 = (SdbFILEKeyValue *) * (SdbFILEKeyValue **) elem2;

	// Ueber die (max.) drei ORDER BY-Columns iterieren. Abbruch des Vergleiches, wenn Ungleichheit erkannt
	// oder alle Columns gleich.
	for (UINT16 i = 0; i < SDB_ORDERBYKEYS && pIndex->eKeyType[i] != SDB_ORDERBYKEY_NONE; i++)
	{
		const int nGreater = (pIndex->bAscending[i]) ? 1 : -1;
		const int nLess = - nGreater;

		// Vergleich (je nach Datentyp):
		switch (pIndex->eKeyType[i])
		{
			case SDB_ORDERBYKEY_STRING:
			{
				INT32 nRes = pKeyValue1->GetKeyString(i)->compareTo(*pKeyValue2->GetKeyString(i));
				if (nRes < 0)
					return nLess;	
				else if (nRes > 0)
					return nGreater;				
			}
			break;					
			case SDB_ORDERBYKEY_DOUBLE:
			{
				double d1 = pKeyValue1->GetKeyDouble(i);
				double d2 = pKeyValue2->GetKeyDouble(i);

				if (d1 < d2)
					return nLess;
				else if (d1 > d2)
					return nGreater;				
			}
			break;
		}
	}

	// Wenn wir bis hierher gekommen sind, waren alle Werte gleich:
	return 0;
}

