/*************************************************************************
 *
 *  $RCSfile: filetab.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "filetab.hxx"
#include "fileconn.hxx"
#include "filecol.hxx"
#include "sdbresid.hrc"

DBG_NAME(SdbFILETable);

//------------------------------------------------------------------
SdbFILETable::SdbFILETable(SdbFILEConnection* pConnection)
			 :SdbTable(pConnection)
			 ,bReadOnly(TRUE)
			 ,nCursorUse(0)
{
	m_pLock = new NAMESPACE_VOS(OMutex);
	DBG_CTOR(SdbFILETable,NULL);
}

//------------------------------------------------------------------
SdbFILETable::~SdbFILETable()
{
	delete m_pLock;
	DBG_DTOR(SdbFILETable,NULL);
}

//------------------------------------------------------------------
void SdbFILETable::SetNames(const String& rName, const String&)
{
	aName = rName;
}

//------------------------------------------------------------------
NAMESPACE_VOS(OGuard)* SdbFILETable::Lock()
{
	return new NAMESPACE_VOS(OGuard)(m_pLock);
}

//------------------------------------------------------------------
void SdbFILETable::CloseImpl()
{
	FileClose();
	bReadOnly = TRUE;
	// da FileTabellen Indices in der Spaltendefinition verwenden
	// muss sichergestellt sein, da diese auch vor dem Zerstren
	// der Tabellenindizes, siehe SdbTab::Close freigegeben werden
}

//------------------------------------------------------------------
BOOL SdbFILETable::OpenImpl()
{
	DBG_CHKTHIS(SdbFILETable,NULL);

	if (FileOpen(FALSE))
		bReadOnly = FALSE;
	else if (!FileOpen(TRUE))
		return aStatus.IsSuccessful();

	DescribeColumns();
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbFILETable::ExtDescribeColumns()
{
}

//------------------------------------------------------------------
BOOL SdbFILETable::DropImpl()
{
	if (InUse())
	{
		aStatus.SetError(ERRCODE_IO_LOCKVIOLATION,TABLE,aName);
		return FALSE;
	}

	FileClose();   // #73459#

	DirEntry aFile = GetFILEConnection()->FileEntry(aName);
	aStatus.SetError(aFile.Kill(),TABLE,aName);

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
SdbIndex* SdbFILETable::CreateIndexImpl() {return NULL;}
//------------------------------------------------------------------
void SdbFILETable::FillIndexNames(SvStringsDtor& _rList) {}

//------------------------------------------------------------------
BOOL SdbFILETable::RenameImpl(const String& rNewName)
{
	CloseImpl();
	DirEntry aOldFile = GetFILEConnection()->FileEntry(aName);
	DirEntry aNewFile = GetFILEConnection()->FileEntry(rNewName);
	ULONG nErrCode = aOldFile.MoveTo(aNewFile);
	if (nErrCode != ERRCODE_NONE)
	{
		//Alten Zustand wiederherstellen
		aStatus.SetError(nErrCode, TABLE, aOldFile.GetFull());
		OpenImpl();
		return FALSE;
	}
	else
		return TRUE;
}

// FileOperationen die nicht von abgeleiteten Klassen untersttzt werden
// muessen
//------------------------------------------------------------------
BOOL SdbFILETable::InsertRow(ODbRowRef& rRow, BOOL bFlush)
{
	aStatus.SetDriverNotCapableError();
	return FALSE;
}

//------------------------------------------------------------------
BOOL SdbFILETable::UpdateRow(const ODbRowRef& rRow, ODbRow* pOrgRow)
{
	aStatus.SetDriverNotCapableError();
	return FALSE;
}

//------------------------------------------------------------------
BOOL SdbFILETable::DeleteRow()
{
	aStatus.SetDriverNotCapableError();
	return FALSE;
}


