/*************************************************************************
 *
 *  $RCSfile: sdbanaly.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_FILESET_HXX
#include "fileset.hxx"
#endif

#include "sdbanaly.hxx"

#ifndef _SDB_FILECURS_HXX
#include "filecurs.hxx"
#endif

#ifndef _SDB_SDBCODE_HXX
#include "sdbcode.hxx"
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

//------------------------------------------------------------------
SdbSQLAnalyzer::SdbSQLAnalyzer(SdbCursor& rCurs)
			   :aCompiler(this,rCurs)
			   ,aInterpreter(aCompiler)
			   ,m_rCursor(rCurs)
{}
//------------------------------------------------------------------
void SdbSQLAnalyzer::Start(SdbSqlParseNode* pSQLParseNode)
{
	if (!pSQLParseNode) return;

	// Parse Tree analysieren (je nach Statement-Typ)
	// und Zeiger auf WHERE-Klausel setzen:
	SdbSqlParseNode * pWhereClause = NULL;
	SdbSqlParseNode * pOrderbyClause = NULL;

	if (SDB_ISRULE(pSQLParseNode,select_statement))
	{
		DBG_ASSERT(pSQLParseNode->Count() >= 4,"SdbFILECursor: Fehler im Parse Tree");

		SdbSqlParseNode * pTableExp = pSQLParseNode->GetChild(3);
		DBG_ASSERT(pTableExp != NULL,"Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp)," Fehler im Parse Tree");
		DBG_ASSERT(pTableExp->Count() == 5,"Fehler im Parse Tree");

		pWhereClause	= pTableExp->GetChild(1);
		pOrderbyClause	= pTableExp->GetChild(4);
	}
	else if (SDB_ISRULE(pSQLParseNode,update_statement_searched))
	{
		DBG_ASSERT(pSQLParseNode->Count() == 5,"SdbFILECursor: Fehler im Parse Tree");
		pWhereClause = pSQLParseNode->GetChild(4);
	}
	else if (SDB_ISRULE(pSQLParseNode,update_statement_positioned))
	{
		// nyi
		DBG_ERROR("SdbPredicateCompiler: update positioned nyi");
	}
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_searched))
	{
		DBG_ASSERT(pSQLParseNode->Count() == 4,"Fehler im Parse Tree");
		pWhereClause = pSQLParseNode->GetChild(3);
	}
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_positioned))
	{
		// nyi
		DBG_ERROR("SdbPredicateCompiler: positioned nyi");
	}
	else
			// Anderes Statement. Keine Selektionskriterien.
		return;

	if (SDB_ISRULE(pWhereClause,where_clause))
	{
		// Wenn es aber eine where_clause ist, dann darf sie nicht leer sein:
		DBG_ASSERT(pWhereClause->Count() == 2,"SdbFILECursor: Fehler im Parse Tree");

		SdbSqlParseNode * pComparisonPredicate = pWhereClause->GetChild(1);
		DBG_ASSERT(pComparisonPredicate != NULL,"SdbFILECursor: Fehler im Parse Tree");

		aCompiler.Execute(pComparisonPredicate);
	}
	else
	{
		// Die Where Clause ist meistens optional, d. h. es koennte sich auch
		// um "optional_where_clause" handeln.
		DBG_ASSERT(SDB_ISRULE(pWhereClause,opt_where_clause),"SdbFILECursor: Fehler im Parse Tree");
	}
}

//------------------------------------------------------------------
SdbFILESet* SdbSQLAnalyzer::BindResultRow(ODbRowRef& rRow)
{
	SdbCodeList& rCodeList	= aCompiler.aCodeList;
	// Zaehlen, wieviele Kriterien
	// wenn nur ein Kriterium, und das entsprechende Feld ist indiziert
	// dann wird der Index verwendet

	SdbEvaluateSetList aEvaluateSetList;
	SdbEvaluateSet*	   pEvaluateSet = NULL;
	SdbFILESet*		   pKeySet = NULL;

	ULONG i;
	for (i = 0; i < rCodeList.Count(); i++)
	{
		SdbOperandAttr* pAttr = PTR_CAST(SdbOperandAttr,rCodeList.GetObject(i));
		if (pAttr)
		{
			if (pAttr->IsIndexed() && !aCompiler.HasORCondition())
			{
				SdbCode* pCode1 = rCodeList.GetObject(i+1);
				SdbCode* pCode2 = rCodeList.GetObject(i+2);

				if (PTR_CAST(SdbOperand,pCode1))
					pEvaluateSet = pAttr->PreProcess(PTR_CAST(SdbBoolOperator,pCode2), PTR_CAST(SdbOperand,pCode1));
				else
					pEvaluateSet = pAttr->PreProcess(PTR_CAST(SdbBoolOperator,pCode1));
			}

			if (pEvaluateSet)
			{
				aEvaluateSetList.Insert(pEvaluateSet,LIST_APPEND);
				pEvaluateSet = NULL;
			}
			else
				pAttr->BindValue(rRow);
		}
	}

	// Keyset erzeugen mit kleinster Liste
	if (aEvaluateSetList.Count())
	{
		// welche Liste hat den kleinsten Count ?
		pEvaluateSet = aEvaluateSetList.First();
		for (i = 1; i < aEvaluateSetList.Count(); i++)
		{
			SdbEvaluateSet*	pEvaluateSetComp = aEvaluateSetList.GetObject(i);
			for (INT32 j = (INT32)pEvaluateSet->Count() - 1; j >= 0; j--)
			{
				ULONG nKey = pEvaluateSet->GetKey((ULONG)j);
				if (!pEvaluateSetComp->IsKeyValid(nKey))
					pEvaluateSet->Remove(nKey);
			}
		}
		pKeySet = new SdbFILESet(*pEvaluateSet);

		// alle loeschen
		for (i = 0; i < aEvaluateSetList.Count(); i++)
			delete aEvaluateSetList.GetObject(i);
	}

	return pKeySet;
}

//------------------------------------------------------------------
void SdbSQLAnalyzer::BindParameterRow(ODbRowRef& rRow)
{
	SdbCodeList& rCodeList	= aCompiler.aCodeList;
	for (SdbCode* pCode = rCodeList.First(); pCode != NULL; pCode = rCodeList.Next())
	{
		SdbOperandParam* pParam = PTR_CAST(SdbOperandParam,pCode);
		if (pParam)
			pParam->BindValue(rRow);
	}
}


//------------------------------------------------------------------
void SdbSQLAnalyzer::DescribeParam(SdbColumnsRef& rParameterColumns)
{
	SdbCodeList& rCodeList	= aCompiler.aCodeList;
	SdbCodeStack aCodeStack;

	if (!rCodeList.Count()) return;		// kein Prdikat
	if (!rParameterColumns->Count()) return; // keine Parameter

	// Anlegen von Columns, die eine genauere Beschreibung fr die enthalten
	SdbColumnsRef aNewParamColumns(rParameterColumns);

	// Anlegen einer Testzeile, wird bentigt um die Parameter zu beschreiben
	ODbRowRef aTestRow = new ODbRow(*aCompiler.getOrigColumns());
	BindResultRow(aTestRow);					// Binden der Attribute an die Values

	ODbRowRef aParameterRow = new ODbRow(*rParameterColumns);
	BindParameterRow(aParameterRow);

	for (SdbCode* pCode = rCodeList.First(); pCode != NULL; pCode = rCodeList.Next())
	{
		SdbOperand* pOperand = PTR_CAST(SdbOperand,pCode);
		SdbOperator* pOperator = PTR_CAST(SdbOperator,pCode);
		if (pOperand)
			aCodeStack.Push(pOperand);
		else
		{
			if (pOperator->RequestedOperands() == 2)		// bei zwei Operatoren ist es mglich
			{												// einen Parameter weiter zu spezifizieren
				SdbOperandParam *pParam  = PTR_CAST(SdbOperandParam,aCodeStack.Top());
				if (pParam)  // Anpassen des ParameterTyps, wenn der linke Operand ein Attribut ist
				{
					SdbOperandAttr *pLeft	 = PTR_CAST(SdbOperandAttr,aCodeStack.GetObject(aCodeStack.Count() - 2));
					if (pLeft)
					{
						const SdbColumn* pCol = (*aCompiler.getOrigColumns())[pLeft->RowPos()];
						DBG_ASSERT(pCol, "Ungltige Struktur");
						pParam->Describe(*pCol, aNewParamColumns);
					}
				}
			}
			pOperator->Exec(aCodeStack);
		}
	}
	SdbOperand* pOperand = aCodeStack.Pop();
	DBG_ASSERT(aCodeStack.Count() == 0, "StackFehler");
	DBG_ASSERT(pOperand, "StackFehler");
	if (IS_TYPE(SdbOperandResult,pOperand)) delete pOperand;

	rParameterColumns = aNewParamColumns;
}

//------------------------------------------------------------------
void SdbSQLAnalyzer::Clean()
{
	aCompiler.Clean();
}

#pragma SEG_EOFMODULE

