/*************************************************************************
 *
 *  $RCSfile: sdbcode.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// sdbcode.cxx
#include "fileset.hxx"
#include "sdbcode.hxx"
#include "fileconn.hxx"

#ifndef _TOOLS_INTN_HXX
#include <tools/intn.hxx>
#endif

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

#ifndef _ISOLANG_HXX
#include <tools/isolang.hxx>
#endif

#ifndef _SDB_SDBCOMP_HXX
#include "sdbcomp.hxx"
#endif

#ifndef _SDB_FILEIDX_HXX
#include "fileidx.hxx"
#endif

#ifndef _SDB_FILECOL_HXX
#include "filecol.hxx"
#endif

#ifndef _SDB_SDBTOOLS_HXX
#include "sdbtools.hxx"
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

TYPEINIT0(SdbCode);
TYPEINIT1(SdbOperand, SdbCode);
TYPEINIT1(SdbOperandRow, SdbOperand);
TYPEINIT1(SdbOperandAttr, SdbOperandRow);
TYPEINIT1(SdbFILEOperandAttr, SdbOperandAttr);
TYPEINIT1(SdbOperandParam, SdbOperandRow);
TYPEINIT1(SdbOperandValue, SdbOperand);
TYPEINIT1(SdbOperandConst, SdbOperandValue);
TYPEINIT1(SdbOperandResult, SdbOperandValue);

TYPEINIT1(SdbOperator, SdbCode);
TYPEINIT1(SdbBoolOperator,SdbOperator);
TYPEINIT1(SdbOp_AND, SdbBoolOperator);
TYPEINIT1(SdbOp_OR, SdbBoolOperator);
TYPEINIT1(SdbOp_ISNULL, SdbBoolOperator);
TYPEINIT1(SdbOp_ISNOTNULL, SdbOp_ISNULL);
TYPEINIT1(SdbOp_LIKE, SdbBoolOperator);
TYPEINIT1(SdbOp_NOTLIKE, SdbOp_LIKE);
TYPEINIT1(SdbOp_COMPARE, SdbBoolOperator);
TYPEINIT1(SdbNumOperator, SdbOperator);

//------------------------------------------------------------------
SdbCode::~SdbCode(){};

//------------------------------------------------------------------
SdbEvaluateSet* SdbOperand::PreProcess(SdbBoolOperator* pOp, SdbOperand* pRight) {return NULL;}

//------------------------------------------------------------------
void SdbOperandRow::BindValue(const ODbRowRef& rRowToBind)
{
	aRow = rRowToBind;
	if (!(*aRow)[nRowPos].is()) // Wenn noch nicht gebunden, dann Value allozieren und binden:
		(*aRow)[nRowPos] = new ODbVariant();
}

//------------------------------------------------------------------
const ODbVariant* SdbOperandRow::Value() const
{
	return aRow.Is() ? (*aRow)[nRowPos].getBodyPtr() : NULL;
}
//------------------------------------------------------------------
SdbOperandAttr::SdbOperandAttr(const SdbColumn* pCol) : SdbOperandRow(pCol->GetId(),pCol->GetType()),pColumn(pCol)
{
}
//------------------------------------------------------------------
SdbFILEOperandAttr::SdbFILEOperandAttr(const SdbColumn* pCol)
			   :SdbOperandAttr(pCol)
{
}

//------------------------------------------------------------------
SdbEvaluateSet* SdbFILEOperandAttr::PreProcess(SdbBoolOperator* pOp, SdbOperand* pRight)
{
	SdbEvaluateSet* pEvaluateSet = NULL;
	const SdbFILEColumn* pCol = (const SdbFILEColumn*)pColumn;
	if (pCol->IsIndexed())
	{
		SdbFILEIndexIterator* pIter = pCol->GetIndex()->CreateIterator(pOp,pRight);

		if (pIter->Status().IsSuccessful())
		{
			pEvaluateSet = new SdbEvaluateSet();
			ULONG nRec = pIter->First();
			while (nRec != SDB_INDEX_ENTRY_NOTFOUND)
			{
				pEvaluateSet->Insert(nRec);
				nRec = pIter->Next();
			}
		}
		delete pIter;
	}
	return pEvaluateSet;
}

//------------------------------------------------------------------
SdbOperandParam::SdbOperandParam(SdbSqlParseNode* pNode, const String& rParameterNames, SdbColumnsRef& rParamColumns)
				:SdbOperandRow(rParamColumns->Count(), SDB_DBTYPE_VARCHAR)		 // Standard-Typ
{
	DBG_ASSERT(SDB_ISRULE(pNode,parameter),"Argument ist kein Parameter");
	DBG_ASSERT(pNode->Count() > 0,"Fehler im Parse Tree");
	SdbSqlParseNode *pMark = pNode->GetChild(0);

	String aParameterName;
	if (SDB_ISPUNCTUATION(pMark,"?"))
	{
		// Beim vorherigen Translate des Statements wurden bereits die
		// benannten Parameter in '?' uebersetzt. Die Parameter-Namen
		// sind dann allerdings noch verfuegbar:
		aParameterName = rParameterNames.GetToken((USHORT) RowPos() - 1);
		// Name == "?", falls kein Parametername verfuegbar (z. B. bei Native SQL)
		if (aParameterName.Len() == 0) aParameterName = '?';
	}
	else if (SDB_ISPUNCTUATION(pMark,":"))
		aParameterName = pNode->GetChild(1)->TokenValue();
	else
	{
		DBG_ERROR("Fehler im Parse Tree");
	}

	// Parameter-Column aufsetzen mit defult typ, kann zu einem spteren Zeitpunkt ber DescribeParameter
	// genauer spezifiziert werden

	// Identitaet merken (hier eigentlich nicht erforderlich, aber aus
	// Symmetriegruenden ...)
	SdbColumn* pColumn = new SdbFILEColumn(aParameterName,eDBType,255,0,SDB_FLAGS_NULLALLOWED);
	rParamColumns->AddColumn(pColumn);
	// der Wert wird erst kurz vor der Auswertung gesetzt
}


//------------------------------------------------------------------
void SdbOperandParam::Describe(const SdbColumn& rColumn, SdbColumnsRef& rParameterColumns)
{
	// den alten namen beibehalten
	SdbColumn* pCol = new SdbFILEColumn(rParameterColumns->Column(RowPos())->GetName(), rColumn.GetType(),
									rColumn.GetLength(), rColumn.GetScale(), rColumn.GetAllFlags());

	rParameterColumns->ReplaceColumn(pCol, RowPos());
	eDBType = rColumn.GetType();
}

//------------------------------------------------------------------
const ODbVariant* SdbOperandValue::Value() const
{
	return xValue.getBodyPtr();
}

//------------------------------------------------------------------
SdbOperandConst::SdbOperandConst(const SdbSqlParseNode& rColumnRef, const rtl::OUString& aStrValue)
{
	switch (rColumnRef.GetNodeType())
	{
		case SDB_SQL_STRING:
			xValue->setString(aStrValue);
			eDBType = SDB_DBTYPE_VARCHAR;
			return;
		case SDB_SQL_INTNUM:
		case SDB_SQL_APPROXNUM:
		{
			xValue->setString(aStrValue);
			try
			{
				double fVal = xValue->toDouble();
				xValue->setDouble(fVal);
			}
			catch ( ... )
			{

				xValue->setDouble(0);
			}

			eDBType = SDB_DBTYPE_DOUBLE;
			return;
		}
	}

	if (SDB_ISTOKEN(&rColumnRef,"TRUE"))
	{
		xValue->setDouble(1);
		eDBType = SDB_DBTYPE_BOOLEAN;
	}
	else if (SDB_ISTOKEN(&rColumnRef,"FALSE"))
	{
		xValue->setDouble(0);
		eDBType = SDB_DBTYPE_BOOLEAN;
	}
	else
	{
		DBG_ERROR("Parse Error");
	}
}

/////////////////////////////////////////////////////////////////////////////////////////
// Implementation of the operators

//------------------------------------------------------------------
UINT16 SdbOperator::RequestedOperands() const {return 2;}

//------------------------------------------------------------------
BOOL SdbBoolOperator::Operate(const SdbOperand*, const SdbOperand*) const {return FALSE;}


//------------------------------------------------------------------
void SdbBoolOperator::Exec(SdbCodeStack& rCodeStack)
{
	SdbOperand  *pRight	= rCodeStack.Pop(),
				*pLeft	= rCodeStack.Pop();

	rCodeStack.Push(new SdbOperandResultBOOL(Operate(pLeft, pRight)));
	if (IS_TYPE(SdbOperandResult,pLeft)) delete pLeft;
	if (IS_TYPE(SdbOperandResult,pRight)) delete pRight;
}

//------------------------------------------------------------------
BOOL SdbOp_AND::Operate(const SdbOperand* pLeft, const SdbOperand* pRight) const
{
	return pLeft->IsValid() && pRight->IsValid();
}

//------------------------------------------------------------------
BOOL SdbOp_OR::Operate(const SdbOperand* pLeft, const SdbOperand* pRight) const
{
	return pLeft->IsValid() || pRight->IsValid();
}

//------------------------------------------------------------------
UINT16 SdbOp_ISNULL::RequestedOperands() const {return 1;}

//------------------------------------------------------------------
void SdbOp_ISNULL::Exec(SdbCodeStack& rCodeStack)
{
	SdbOperand* pOperand = rCodeStack.Pop();

	rCodeStack.Push(new SdbOperandResultBOOL(Operate(pOperand)));
	if (IS_TYPE(SdbOperandResult,pOperand)) delete pOperand;
}

//------------------------------------------------------------------
BOOL SdbOp_ISNULL::Operate(const SdbOperand* pOperand, const SdbOperand*) const
{
	return !pOperand->Value() || pOperand->Value()->isNull();
}

//------------------------------------------------------------------
BOOL SdbOp_ISNOTNULL::Operate(const SdbOperand* pOperand, const SdbOperand*) const
{
	return !SdbOp_ISNULL::Operate(pOperand);
}

//------------------------------------------------------------------
BOOL SdbOp_LIKE::Operate(const SdbOperand* pLeft, const SdbOperand* pRight) const
{
	BOOL bMatch;
	if (!pLeft->Value() || !pRight->Value())
		bMatch = TRUE;
	else if (pLeft->Value()->isNull())
		bMatch = FALSE;
	else
	{
		bMatch = ::match(pRight->Value()->toString(), pLeft->Value()->toString(), cEscape);
	}
	return bMatch;
}

//------------------------------------------------------------------
BOOL SdbOp_NOTLIKE::Operate(const SdbOperand* pLeft, const SdbOperand* pRight) const
{
	return !SdbOp_LIKE::Operate(pLeft, pRight);
}

//------------------------------------------------------------------
BOOL SdbOp_COMPARE::Operate(const SdbOperand* pLeft, const SdbOperand* pRight) const
{
	if (!pLeft->Value() || !pRight->Value())
		return TRUE;

	if (pLeft->Value()->isNull() || pRight->Value()->isNull())
		return FALSE;

	BOOL bResult = FALSE;
	SdbDatabaseType eDBType = pLeft->DBType();

	// Vergleich (je nach Datentyp):
	switch (eDBType)
	{
		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		{
			static String sLanguage;
			static String sCountry;
			if (!sLanguage.Len())
				ConvertLanguageToIsoNames(Application::GetAppInternational().GetLanguage(), sLanguage, sCountry);

			static rtl::OLocale aLocale = rtl::OLocale::registerLocale(sLanguage, sCountry);

			INT32 nRes = compareIgnoreCase(pLeft->Value()->toString(), pRight->Value()->toString(), aLocale);
			switch(aPredicateType)
			{
				case SDB_PRED_EQUAL:			bResult = (nRes == 0); break;
				case SDB_PRED_NOTEQUAL:			bResult = (nRes != 0); break;
				case SDB_PRED_LESS:				bResult = (nRes <  0); break;
				case SDB_PRED_LESSOREQUAL:		bResult = (nRes <= 0); break;
				case SDB_PRED_GREATER:			bResult = (nRes >  0); break;
				case SDB_PRED_GREATEROREQUAL:	bResult = (nRes >= 0); break;
				default:						bResult = FALSE;
			}
		} break;
		case SDB_DBTYPE_TINYINT:
		case SDB_DBTYPE_SMALLINT:
		case SDB_DBTYPE_INTEGER:
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
		case SDB_DBTYPE_REAL:
		case SDB_DBTYPE_DOUBLE:
		case SDB_DBTYPE_BOOLEAN:
		case SDB_DBTYPE_TIMESTAMP:
		case SDB_DBTYPE_DATE:
		case SDB_DBTYPE_TIME:
		{
			double n = pLeft->Value()->getDouble(),
				   m = pRight->Value()->getDouble();

			switch (aPredicateType)
			{
				case SDB_PRED_EQUAL:			bResult = (n == m); break;
				case SDB_PRED_LIKE:				bResult = (n == m); break;
				case SDB_PRED_NOTEQUAL:			bResult = (n != m); break;
				case SDB_PRED_NOTLIKE:			bResult = (n != m); break;
				case SDB_PRED_LESS:				bResult = (n < m); break;
				case SDB_PRED_LESSOREQUAL:		bResult = (n <= m); break;
				case SDB_PRED_GREATER:			bResult = (n > m); break;
				case SDB_PRED_GREATEROREQUAL:	bResult = (n >= m); break;
				default:						bResult = FALSE;
			}
		} break;
		default:
			DBG_ERROR("SdbFILECursor::ExecuteRow: Vergleich mit diesem Datentyp nicht implementiert");
			bResult = FALSE;
	}
	return bResult;
}

//------------------------------------------------------------------
void SdbNumOperator::Exec(SdbCodeStack& rCodeStack)
{
	SdbOperand  *pRight	= rCodeStack.Pop(),
				*pLeft	= rCodeStack.Pop();

	rCodeStack.Push(new SdbOperandResultNUM(Operate(pLeft->Value()->getDouble(), pRight->Value()->getDouble())));
	if (IS_TYPE(SdbOperandResult,pLeft)) delete pLeft;
	if (IS_TYPE(SdbOperandResult,pRight)) delete pRight;
}
//------------------------------------------------------------------
double SdbOp_ADD::Operate(double fLeft, double fRight) const {return fLeft + fRight;};

//------------------------------------------------------------------
double SdbOp_SUB::Operate(double fLeft, double fRight) const {return fLeft - fRight;};

//------------------------------------------------------------------
double SdbOp_MUL::Operate(double fLeft, double fRight) const {return fLeft * fRight;};

//------------------------------------------------------------------
double SdbOp_DIV::Operate(double fLeft, double fRight) const {return fLeft / fRight;};




