/*************************************************************************
 *
 *  $RCSfile: sdbcomp.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// sdbcomp.cxx

#include <tools.hxx>
#include "sdbcomp.hxx"
#include "sdbresid.hrc"

#ifndef _SDB_SDBTOOLS_HXX
#include <sdbtools.hxx>
#endif

#ifndef _SDB_FILECURS_HXX
#include "filecurs.hxx"
#endif

#ifndef _SDB_FILECONN_HXX
#include "fileconn.hxx"
#endif

#ifndef _SDB_SDBCODE_HXX
#include "sdbcode.hxx"
#endif

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

DBG_NAME(SdbPredicateCompiler);
//------------------------------------------------------------------
SdbPredicateCompiler::SdbPredicateCompiler(SdbSQLAnalyzer* pAnalyzer,SdbCursor& rCurs)
					 :m_rCursor(rCurs)
					 ,m_pAnalyzer(pAnalyzer)
					 ,bORCondition(FALSE)
{
	DBG_CTOR(SdbPredicateCompiler,NULL);
}

//------------------------------------------------------------------
SdbPredicateCompiler::~SdbPredicateCompiler()
{
	Clean();
	DBG_DTOR(SdbPredicateCompiler,NULL);
}

//------------------------------------------------------------------
inline SdbCursor& SdbPredicateCompiler::Cursor() const {return m_rCursor;}
//------------------------------------------------------------------
void SdbPredicateCompiler::Clean()
{
	while (aCodeList.Count())
		delete aCodeList.Remove(ULONG(0));
}

//------------------------------------------------------------------
void SdbPredicateCompiler::Start(SdbSqlParseNode* pSQLParseNode)
{
	if (!pSQLParseNode) return;

	// Parse Tree analysieren (je nach Statement-Typ)
	// und Zeiger auf WHERE-Klausel setzen:
	SdbSqlParseNode * pWhereClause = NULL;
	SdbSqlParseNode * pOrderbyClause = NULL;

	if (SDB_ISRULE(pSQLParseNode,select_statement))
	{
		DBG_ASSERT(pSQLParseNode->Count() >= 4,"SdbFILECursor: Fehler im Parse Tree");

		SdbSqlParseNode * pTableExp = pSQLParseNode->GetChild(3);
		DBG_ASSERT(pTableExp != NULL,"Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp)," Fehler im Parse Tree");
		DBG_ASSERT(pTableExp->Count() == 5,"Fehler im Parse Tree");

		pWhereClause	= pTableExp->GetChild(1);
		pOrderbyClause	= pTableExp->GetChild(4);
	}
	else if (SDB_ISRULE(pSQLParseNode,update_statement_searched))
	{
		DBG_ASSERT(pSQLParseNode->Count() == 5,"SdbFILECursor: Fehler im Parse Tree");
		pWhereClause = pSQLParseNode->GetChild(4);
	}
	else if (SDB_ISRULE(pSQLParseNode,update_statement_positioned))
	{
		// nyi
		DBG_ERROR("SdbPredicateCompiler: update positioned nyi");
	}
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_searched))
	{
		DBG_ASSERT(pSQLParseNode->Count() == 4,"Fehler im Parse Tree");
		pWhereClause = pSQLParseNode->GetChild(3);
	}
	else if (SDB_ISRULE(pSQLParseNode,delete_statement_positioned))
	{
		// nyi
		DBG_ERROR("SdbPredicateCompiler: positioned nyi");
	}
	else
			// Anderes Statement. Keine Selektionskriterien.
		return;

	if (SDB_ISRULE(pWhereClause,where_clause))
	{
		// Wenn es aber eine where_clause ist, dann darf sie nicht leer sein:
		DBG_ASSERT(pWhereClause->Count() == 2,"SdbFILECursor: Fehler im Parse Tree");

		SdbSqlParseNode * pComparisonPredicate = pWhereClause->GetChild(1);
		DBG_ASSERT(pComparisonPredicate != NULL,"SdbFILECursor: Fehler im Parse Tree");

		Execute(pComparisonPredicate );
	}
	else
	{
		// Die Where Clause ist meistens optional, d. h. es koennte sich auch
		// um "optional_where_clause" handeln.
		DBG_ASSERT(SDB_ISRULE(pWhereClause,opt_where_clause),"SdbFILECursor: Fehler im Parse Tree");
	}
}

//------------------------------------------------------------------
SdbOperand* SdbPredicateCompiler::Execute(SdbSqlParseNode* pPredicateNode)
{
	SdbOperand* pOperand = NULL;
	if (m_rCursor.aStatus.IsError())
		;
	else if (pPredicateNode->Count() == 3 &&							// Ausdruck is geklammert
		SDB_ISPUNCTUATION(pPredicateNode->GetChild(0),"(") &&
		SDB_ISPUNCTUATION(pPredicateNode->GetChild(2),")"))
	{
		Execute(pPredicateNode->GetChild(1));
	}
	else if ((SDB_ISRULE(pPredicateNode,search_condition) || (SDB_ISRULE(pPredicateNode,boolean_term)))
							&&			// AND/OR-Verknuepfung:
			 pPredicateNode->Count() == 3)
	{
		Execute(pPredicateNode->GetChild(0));						    // Bearbeiten des linken Zweigs
		Execute(pPredicateNode->GetChild(2));							// Bearbeiten des rechten Zweigs

		if (SDB_ISTOKEN(pPredicateNode->GetChild(1),"OR"))				// OR-Operator
		{
			aCodeList.Insert(new SdbOp_OR(), LIST_APPEND);
			bORCondition = TRUE;
		}
		else if (SDB_ISTOKEN(pPredicateNode->GetChild(1),"AND"))		// AND-Operator
			aCodeList.Insert(new SdbOp_AND(), LIST_APPEND);
		else
		{
			DBG_ERROR("SdbFILECursor: Fehler im Parse Tree");
		}
	}
	else if (SDB_ISRULE(pPredicateNode,comparison_predicate))
	{
		Execute_COMPARE(pPredicateNode);
	}
	else if (SDB_ISRULE(pPredicateNode,like_predicate))
	{
		Execute_LIKE(pPredicateNode);
	}
	else if (SDB_ISRULE(pPredicateNode,test_for_null))
	{
		Execute_ISNULL(pPredicateNode);
	}
	else
		pOperand = Execute_Operand(pPredicateNode);						// jetzt werden nur einfache Operanden verarbeitet

	return pOperand;
}

//------------------------------------------------------------------
SdbOperand* SdbPredicateCompiler::Execute_COMPARE(SdbSqlParseNode* pPredicateNode)
{
	DBG_ASSERT(pPredicateNode->Count() == 3,"SdbFILECursor: Fehler im Parse Tree");

	if (!(SDB_ISRULE(pPredicateNode->GetChild(0),column_ref) ||
		  pPredicateNode->GetChild(2)->GetNodeType() == SDB_SQL_STRING ||
		  pPredicateNode->GetChild(2)->GetNodeType() == SDB_SQL_INTNUM ||
		  pPredicateNode->GetChild(2)->GetNodeType() == SDB_SQL_APPROXNUM ||
		  SDB_ISTOKEN(pPredicateNode->GetChild(2),"TRUE") ||
		  SDB_ISTOKEN(pPredicateNode->GetChild(2),"FALSE") ||
		  SDB_ISRULE(pPredicateNode->GetChild(2),parameter) ||
		  // odbc date
		  (SDB_ISRULE(pPredicateNode->GetChild(2),set_fct_spec) && SDB_ISPUNCTUATION(pPredicateNode->GetChild(2)->GetChild(0),"{"))))
	{
		m_rCursor.aStatus.SetStatementTooComplex();
		return NULL;
	}

	SdbPredicateType ePredicateType;
	SdbSqlParseNode *pPrec = pPredicateNode->GetChild(1);

	if (pPrec->GetNodeType() == SDB_SQL_EQUAL)
		ePredicateType = SDB_PRED_EQUAL;
	else if (pPrec->GetNodeType() == SDB_SQL_NOTEQUAL)
		ePredicateType = SDB_PRED_NOTEQUAL;
	else if (pPrec->GetNodeType() == SDB_SQL_LESS)
		ePredicateType = SDB_PRED_LESS;
	else if (pPrec->GetNodeType() == SDB_SQL_LESSEQ)
		ePredicateType = SDB_PRED_LESSOREQUAL;
	else if (pPrec->GetNodeType() == SDB_SQL_GREATEQ)
		ePredicateType = SDB_PRED_GREATEROREQUAL;
	else if (pPrec->GetNodeType() == SDB_SQL_GREAT)
		ePredicateType = SDB_PRED_GREATER;

	SdbOperand* pOb = Execute(pPredicateNode->GetChild(0));
	SdbOperand* pOperand  = Execute(pPredicateNode->GetChild(2));

	SdbBoolOperator* pOperator = new SdbOp_COMPARE(ePredicateType);
	//pOb->PreProcess(pOperator, pOperand);

	aCodeList.Insert(pOperator, LIST_APPEND);

	// wenn es sich um eine Vergleichsoperation auf datum/Zeit handelt, dann
	// erfolgt jetzt bereits eine Umwandlung fuer die Konstante
	if (pOb)
	{
		switch (pPredicateNode->GetChild(2)->GetNodeType())
		{
			case SDB_SQL_STRING:
			{
				SdbOperandConst* pConst = PTR_CAST(SdbOperandConst,aCodeList.GetObject(aCodeList.Count() - 2));
				switch (pOb->DBType())
				{
					case SDB_DBTYPE_DECIMAL:
					case SDB_DBTYPE_NUMERIC:
					case SDB_DBTYPE_REAL:
					case SDB_DBTYPE_DOUBLE:
					case SDB_DBTYPE_TIMESTAMP:
					case SDB_DBTYPE_DATE:
					case SDB_DBTYPE_TIME:
					{
						try
						{
							((ODbVariant*)pConst->Value())->setDouble(pConst->Value()->toDouble());
						}
						catch( ... )
						{
							m_rCursor.aStatus.Set(SDB_STAT_ERROR,
										String::CreateFromAscii("S1C00"),
										m_rCursor.aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_SQL_DATATYPE_MISMATCH))),
										0, String() );
						}						
					}	break;
					case SDB_DBTYPE_TINYINT:
					case SDB_DBTYPE_SMALLINT:
					case SDB_DBTYPE_INTEGER:
					case SDB_DBTYPE_BOOLEAN:
						m_rCursor.aStatus.Set(SDB_STAT_ERROR,
										String::CreateFromAscii("S1C00"),
										m_rCursor.aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_SQL_DATATYPE_MISMATCH))),
										0, String() );
				}
			}
		}
	}
	return NULL;
}

//------------------------------------------------------------------
SdbOperand* SdbPredicateCompiler::Execute_LIKE(SdbSqlParseNode* pPredicateNode)
{
	DBG_ASSERT(pPredicateNode->Count() >= 4,"SdbFILECursor: Fehler im Parse Tree");

	if (!SDB_ISRULE(pPredicateNode->GetChild(0),column_ref))
	{
		m_rCursor.aStatus.SetInvalidStatement();
		return NULL;
	}

	SdbPredicateType ePredicateType;
	SdbSqlParseNode *pAtom;
	SdbSqlParseNode *pOptEscape;
	sal_Unicode cEscape = L'\0';

	if (SDB_ISTOKEN(pPredicateNode->GetChild(1),"NOT"))
		ePredicateType = SDB_PRED_NOTLIKE;
	else
		ePredicateType = SDB_PRED_LIKE;

	pAtom = pPredicateNode->GetChild(3);
	pOptEscape = pPredicateNode->GetChild(4);

	if (!(pAtom->GetNodeType() == SDB_SQL_STRING || SDB_ISRULE(pAtom,parameter)))
	{
		m_rCursor.aStatus.SetInvalidStatement();
		return NULL;
	}
	if (pOptEscape->Count() != 0)
	{
		if (pOptEscape->Count() != 2)
		{
			m_rCursor.aStatus.SetInvalidStatement();
			return NULL;
		}
		SdbSqlParseNode *pEscNode = pOptEscape->GetChild(1);
		if (pEscNode->GetNodeType() != SDB_SQL_STRING)
		{
			m_rCursor.aStatus.SetInvalidStatement();
			return NULL;
		}
		else
			cEscape = pEscNode->TokenValue().GetChar(0);
	}

	SdbOperand* pOb = Execute(pPredicateNode->GetChild(0));
	SdbOperand* pOperand  = Execute(pAtom);
	SdbBoolOperator* pOperator = (ePredicateType == SDB_PRED_LIKE)
													? new SdbOp_LIKE(cEscape)
													: new SdbOp_NOTLIKE(cEscape);
	aCodeList.Insert(pOperator, LIST_APPEND);
	return NULL;
}

//------------------------------------------------------------------
SdbOperand* SdbPredicateCompiler::Execute_ISNULL(SdbSqlParseNode* pPredicateNode)
{
	if (!SDB_ISRULE(pPredicateNode->GetChild(0),column_ref))
	{
		m_rCursor.aStatus.SetInvalidStatement();
		return NULL;
	}

	DBG_ASSERT(pPredicateNode->Count() >= 3,"SdbFILECursor: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISTOKEN(pPredicateNode->GetChild(1),"IS"),"SdbFILECursor: Fehler im Parse Tree")

	SdbPredicateType ePredicateType;
	if (SDB_ISTOKEN(pPredicateNode->GetChild(2),"NOT"))
		ePredicateType = SDB_PRED_ISNOTNULL;
	else
		ePredicateType = SDB_PRED_ISNULL;

	SdbOperand* pOb = Execute(pPredicateNode->GetChild(0));
	SdbBoolOperator* pOperator = (ePredicateType == SDB_PRED_ISNULL) ?
								new SdbOp_ISNULL() : new SdbOp_ISNOTNULL();

	//pOb->PreProcess(pOperator);


	aCodeList.Insert(pOperator, LIST_APPEND);
	return NULL;
}
//------------------------------------------------------------------
SdbOperand* SdbPredicateCompiler::Execute_Operand(SdbSqlParseNode* pPredicateNode)
{
	SdbOperand* pOperand = NULL;

	if (SDB_ISRULE(pPredicateNode,column_ref))
	{
		String	aColumnName;
		if (pPredicateNode->Count() == 1)
		{
			aColumnName = pPredicateNode->GetChild(0)->TokenValue();
		}
		else if (pPredicateNode->Count() == 3)
		{
			String aTableName = pPredicateNode->GetChild(0)->TokenValue();
			aColumnName = pPredicateNode->GetChild(2)->TokenValue();
		}

		const SdbColumn* pCol = (*m_orgColumns)[aColumnName];
		if (pCol)  // Column existiert nicht im Resultset
			pOperand = m_pAnalyzer->CreateOperandAttr(pCol); //new SdbOperandAttr(pCol);
		else
		{
			m_rCursor.aStatus.SetInvalidStatement();
		}
	}
	else if (SDB_ISRULE(pPredicateNode,parameter))
	{
		pOperand = new SdbOperandParam(pPredicateNode, Cursor().getParameterNames(), Cursor().aParameterColumns);
	}
	else if (pPredicateNode->GetNodeType() == SDB_SQL_STRING ||
			 pPredicateNode->GetNodeType() == SDB_SQL_INTNUM ||
			 pPredicateNode->GetNodeType() == SDB_SQL_APPROXNUM ||
			 pPredicateNode->GetNodeType() == SDB_SQL_NAME ||
			 SDB_ISTOKEN(pPredicateNode,"TRUE") ||
			 SDB_ISTOKEN(pPredicateNode,"FALSE") ||
			 SDB_ISRULE(pPredicateNode,parameter))
	{
		pOperand = new SdbOperandConst(*pPredicateNode, pPredicateNode->TokenValue());
	}
	else if((pPredicateNode->Count() == 2) &&
			(SDB_ISPUNCTUATION(pPredicateNode->GetChild(0),"+") || SDB_ISPUNCTUATION(pPredicateNode->GetChild(0),"-")) &&
			pPredicateNode->GetChild(1)->GetNodeType() == SDB_SQL_INTNUM)
	{ // falls -1 bzw. +1 vorhanden ist
		String aValue(pPredicateNode->GetChild(0)->TokenValue());
		aValue += pPredicateNode->GetChild(1)->TokenValue();
		pOperand = new SdbOperandConst(*pPredicateNode->GetChild(1), aValue);
	}
	else if(SDB_ISRULE(pPredicateNode,set_fct_spec) && SDB_ISPUNCTUATION(pPredicateNode->GetChild(0),"{"))
	{ 
		const SdbSqlParseNode* pODBCNode = pPredicateNode->GetChild(1);
		const SdbSqlParseNode* pODBCNodeChild = pODBCNode->GetChild(0);
		
		// Odbc Date or time
		if (pODBCNodeChild->GetNodeType() == SDB_SQL_KEYWORD && (
			SDB_ISTOKEN(pODBCNodeChild,"D") ||
			SDB_ISTOKEN(pODBCNodeChild,"T") ||
			SDB_ISTOKEN(pODBCNodeChild,"TS") ))
		{
			pOperand = new SdbOperandConst(*pODBCNode->GetChild(1), pODBCNode->GetChild(1)->TokenValue());
			
			// setting the Date
			try
			{
				((ODbVariant*)pOperand->Value())->setDouble(pOperand->Value()->toDouble());
			}
			catch( ... )
			{
				m_rCursor.aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1C00"),
							m_rCursor.aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_SQL_DATATYPE_MISMATCH))),
							0, String() );
			}			
		}
		else
			m_rCursor.aStatus.SetStatementTooComplex();		
	}
	else
	{
		m_rCursor.aStatus.SetStatementTooComplex();
	}
	if (pOperand)
		aCodeList.Insert(pOperand, LIST_APPEND);
	return pOperand;
}

////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------
SdbPredicateInterpreter::~SdbPredicateInterpreter()
{
	while (aStack.Count())
		delete aStack.Pop();
}

//------------------------------------------------------------------
BOOL SdbPredicateInterpreter::Evaluate(SdbCodeList& rCodeList)
{
	static BOOL bResult;

	SdbCode* pCode = rCodeList.First();
	if (pCode == NULL) return TRUE;		// kein Prdikat

	while (pCode != NULL)
	{
		SdbOperand* pOperand = PTR_CAST(SdbOperand,pCode);
		if (pOperand)
			aStack.Push(pOperand);
		else
			((SdbOperator *)pCode)->Exec(aStack);

		pCode = rCodeList.Next();
	}
	SdbOperand* pOperand = aStack.Pop();
	DBG_ASSERT(aStack.Count() == 0, "StackFehler");
	DBG_ASSERT(pOperand, "StackFehler");

	bResult = pOperand->IsValid();
	if (IS_TYPE(SdbOperandResult,pOperand)) delete pOperand;
	return bResult;
}

