/*************************************************************************
 *
 *  $RCSfile: filemake.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/* Includes fuer das Apple-Geraffel  */

#include "vcl/Sysdep.hxx"

#include <mac_start.h>
 #include <Types.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <string.h>
 #include <ErrMgr.h>
 #include <Errors.h>
 #include <Aliases.h>
 #include <AppleEvents.h>
 #include <Memory.h>
 #include <Events.h>
 #include <AEPackObject.h>
 #include <AEObjects.h>
 #include <AERegistry.h>
#include <mac_end.h>

#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif
#include <vcl/wrkwin.hxx>

#include <stdio.h>

#include <fmtab.hxx>
#include <fmconn.hxx>
#include <filecol.hxx>

//-----------------------------------------------------------------------------

#define kDisposeInputs TRUE

//-----------------------------------------------------------------------------

static BOOL GetAEEror(OSErr nErr, AppleEvent& rReply)
{
	if (nErr == noErr)
	{
		long 	nLongErr;
		long  	nLen;
		OSType	theTypeCode;

		nErr = AEGetParamPtr(&rReply, keyErrorNumber, typeLongInteger,
							 &theTypeCode, &nLongErr, sizeof(long), &nLen );

		if (nErr != errAEDescNotFound)	// kein Fehlercode, auch gut
		{
			char Buffer[255];
			nErr = AEGetParamPtr(&rReply, keyErrorString, typeChar,
									&theTypeCode, Buffer, 255, &nLen );
			Buffer[nLen] = '\0';
			DBG_ERROR(Buffer);
			return TRUE;
		}
		else
			return FALSE;
	}
	return TRUE;
}


/* -------------------------------------------------------------------------------------------------

	Opens the database with the name specified in theDocument. The argument must contain the whole
	path to the file.

	------------------------------------------------------------------------------------------------- */
OSErr SdbFmTable::OpenFMDatabase(const String& rFileName)
{
	const OSType 	finderAppl = 'MACS';
	OSErr 			nErr;

	AEAddressDesc 	anAddress;
	FSSpec 			dirSpecFS, docToOpenFS;
	AliasHandle 	dirAlias, docAlias;
	AppleEvent 		theAppleEvent, theReply;
	AEDesc 			docDesc, dirDesc;
	AEDescList 		docList;

	/* All descriptors must be initaliased before use */
	anAddress.dataHandle = theAppleEvent.dataHandle = docList.dataHandle = dirDesc.dataHandle = nil;
	docDesc.dataHandle = theReply.dataHandle = nil;

	/* Other intialisations */
	dirAlias = nil;
	docAlias = nil;
	nErr = noErr;

	/* Create aliases for the document name and the directory. It is
		needed by Finder. The aliases is stored in a AEList. */
	if (!nErr)
		nErr = FSMakeFSSpec(0, 0, rFileName.GetPascalStr(), &docToOpenFS);

	FInfo aFinderInfo;
	if (!nErr)
	{
		nErr =FSpGetFInfo(&docToOpenFS,&aFinderInfo);
		if (!nErr)
			AECreateDesc(typeApplSignature, &aFinderInfo.fdCreator, sizeof(OSType), &aFileMakerAdress);
	}

	// Ist die Datenbank schon geffnet ?

	aFmFileName = String((char*) &docToOpenFS.name[1],docToOpenFS.name[0]);
	nErr = BuildDatabaseDesc(aFmFileName);

	/* Create the Apple Event */
	if (!nErr)
		nErr = AECreateAppleEvent('core','getd',GetFMAdressDesc(),kAutoGenerateReturnID,kAnyTransactionID,&theAppleEvent);

	/* Gib mir die Datenbank selbst */
	if (!nErr)
		nErr = AEPutKeyDesc(&theAppleEvent, keyDirectObject, GetDatabaseSpec());

	if (!nErr)
		nErr = AESend(&theAppleEvent, &theReply, kAEWaitReply | kAENeverInteract, kAENormalPriority, kAEDefaultTimeout,
					 Sysdepen::GetAEIdleProc(), nil);

	if (!GetAEEror(nErr,theReply))
	{
		AEDisposeDesc(&anAddress);
		AEDisposeDesc(&theAppleEvent);
		AEDisposeDesc(&theReply);
		return noErr;			// Datenbank ist schon offen, raus hier
	}

	nErr = noErr;	// Datenbank nicht gefunden -> Finder sag ihm er soll sie aufmachen

	/* Create a descriptor for Finder. It is used to specify where the Apple Events should
	  be sent. Opening a file is made by sending an open event to Finder, which opens the file
	  and send it to the correct application. You could also send an open document directly to
	  FileMaker, but you should not do that according to Inside Mac. */
	if (!nErr)
		nErr = AECreateDesc( typeApplSignature, (MAC_Ptr) &finderAppl,
								  sizeof(finderAppl), &anAddress );

	if (!nErr)
		nErr = AECreateAppleEvent('FNDR','sope',&anAddress,kAutoGenerateReturnID,kAnyTransactionID,&theAppleEvent);

	if (!nErr)
		nErr = FSMakeFSSpec( docToOpenFS.vRefNum, docToOpenFS.parID,
								  nil, &dirSpecFS );

	if (!nErr)
		nErr = NewAlias(nil, &dirSpecFS, &dirAlias);

	if (!nErr)
		nErr = NewAlias(nil, &docToOpenFS, &docAlias);

	if (!nErr)
		nErr = AECreateList(nil, 0, false, &docList);

	if (!nErr)
	{
		HLock((Handle)dirAlias);
		nErr = AECreateDesc(typeAlias, (MAC_Ptr) * dirAlias, GetHandleSize((Handle)dirAlias), &dirDesc);
		HUnlock((Handle)dirAlias);
	}

	if (!nErr)
		nErr = AEPutParamDesc(&theAppleEvent, keyDirectObject, &dirDesc);

	if (!nErr)
	{
		HLock((Handle)docAlias);
		nErr = AECreateDesc(typeAlias, (MAC_Ptr) * docAlias, GetHandleSize((Handle)docAlias), &docDesc);
		HUnlock((Handle)docAlias);
	}

	if (!nErr)
		nErr = AEPutDesc(&docList, 0, &docDesc);

	/* The list which contains the two alisases is ready and will now be put at the end of
	  the Apple Event. */
	if (!nErr)
		nErr = AEPutParamDesc(&theAppleEvent, keySelection, &docList);

	/* Send the Apple Event. */
	if (!nErr)									// kAENoReply |
		nErr = AESend(&theAppleEvent, &theReply, kAEWaitReply | kAECanInteract, kAENormalPriority, kAEDefaultTimeout,
					 Sysdepen::GetAEIdleProc(), nil);

#ifdef DBG_UTIL
	GetAEEror(nErr,theReply);
#endif

	DisposeHandle((Handle)docAlias);
	DisposeHandle((Handle)dirAlias);

	/* Dispose all the descriptors and handles we used. */
	AEDisposeDesc(&anAddress);
	AEDisposeDesc(&theAppleEvent);
	AEDisposeDesc(&docList);
	AEDisposeDesc(&dirDesc);
	AEDisposeDesc(&docDesc);
	AEDisposeDesc(&theReply);

	if (!nErr)
	{
		// Return Name of DataBase, which is the pure filename
		aFmFileName = String((char*) &docToOpenFS.name[1],docToOpenFS.name[0]);

		nErr = BuildDatabaseDesc(aFmFileName);
	}

	/* Return the error code. It is noErr if everything is OK. */
	return nErr;
}


OSErr SdbFmTable::BuildDatabaseDesc(const String& rDatabaseName)
{
	AEDesc 		docDesc = {typeNull, NULL };
	AEDesc		nullDesc = {typeNull, NULL };
	OSErr		nErr;

	/* Create an object for the FileMaker document. The object contains the document name. */
	nErr = AECreateDesc(typeChar, rDatabaseName.GetStr(), rDatabaseName.Len(), &docDesc);

	if (!nErr)
		nErr = CreateObjSpecifier(cDocument, &nullDesc, formName, &docDesc, kDisposeInputs, &aDatabaseSpec);

	// AEDisposeDesc(&docDesc);	// durch kDisposeInputs	schon passiert

	return nErr;
}

/* -------------------------------------------------------------------------------------------------

	Gets the header record, i.e. the names of the fields in the database. The names are returned in
	a list, in a certain order. Please note that FileMaker Pro 2.0 has a bug, where the field order
	is mixed up everytime the field order in the Define Field dialog box in FileMaker is changed.
	The database must the be recovered to work properly again. That bug is fixed in FileMaker Pro 2.1.

	------------------------------------------------------------------------------------------------- */

BOOL SdbFmTable::DescribeColumns()
{
	AEDesc 		desc0, desc1, propObject,
				colObject, tempDesc, layoutObj;
	AEDescList 	fieldList;
	AppleEvent 	theAppleEvent, theReply;
	DescType 	nAEAll = kAEAll;
	DescType 	nPnam = 'pnam';
	long 		layoutNumber = 0;
	OSErr 		nErr;
	long 		i, nItems = 0, fieldLen;
	char		Field[256];
	MAC_Ptr 		pField = Field;
	AEKeyword 	theAEKeyWord;
	DescType 	theTypeCode;

	if (aOriginalColumns->Count() > 1)
		return TRUE;

	/* All descriptors must be initaliased before use */
	theAppleEvent.dataHandle = theReply.dataHandle = propObject.dataHandle = nil;
	desc0.dataHandle = desc1.dataHandle = colObject.dataHandle = tempDesc.dataHandle = nil;
	layoutObj.dataHandle = nil;

	nErr = noErr;

	/* All information you get from FileMaker must be from Layout #0. It is a non-visible, abstract
	  layout which contains all fields in the database. If we not specify Layout #0, we only get
	  the fields on the current layout. */
	if (!nErr)
		nErr = AECreateDesc(typeLongInteger, &layoutNumber, sizeof(long), &desc0);

	if (!nErr)
		nErr = CreateObjSpecifier('ctbl', GetDatabaseSpec(), formAbsolutePosition, &desc0, kDisposeInputs, &layoutObj);

	/* Create an object that specifies that we want all of the fields. */
	if (!nErr)
		nErr = AECreateDesc(typeAbsoluteOrdinal, &nAEAll, sizeof(DescType), &desc1);

	if (!nErr)
		nErr = CreateObjSpecifier('ccol', &layoutObj, formAbsolutePosition, &desc1, kDisposeInputs, &colObject);

	/* Create an object that specifies that we want to know the name property of the fields */
	if (!nErr)
		nErr = AECreateDesc(typeType, &nPnam, sizeof(DescType), &tempDesc);

	if (!nErr)
		nErr = CreateObjSpecifier(typeProperty, &colObject, formPropertyID, &tempDesc, kDisposeInputs, &propObject);

	/* Create the Apple Event */
	if (!nErr)
		nErr = AECreateAppleEvent('core','getd',GetFMAdressDesc(),kAutoGenerateReturnID,kAnyTransactionID,&theAppleEvent);

	/* Put the objects at the end of the Apple Event */
	if (!nErr)
		nErr = AEPutKeyDesc(&theAppleEvent, keyDirectObject, &propObject);
	AEDisposeDesc(&propObject);

	/* And send it. */
	if (!nErr)
		nErr = AESend(&theAppleEvent, &theReply, kAEWaitReply | kAENeverInteract, kAENormalPriority, kAEDefaultTimeout,
					 Sysdepen::GetAEIdleProc(), nil);

#ifdef DBG_UTIL
	GetAEEror(nErr,theReply);
#endif

	/* Get the result. It is in fieldList */
	if (!nErr)
		nErr = AEGetParamDesc(&theReply, keyDirectObject, typeAEList, &fieldList);

	/* Count the items (the fields) in fieldList. */
	if (!nErr)
		nErr = AECountItems(&fieldList, &nItems);

	/* Go through the fieldList and get the field names */
	for (i = 1; i <= nItems; i++) {
		nErr = AEGetNthPtr(&fieldList, i, typeChar, &theAEKeyWord,
								&theTypeCode, pField, 255, &fieldLen);
		if (fieldLen > 255)
		{
			pField = new char[fieldLen];
			nErr = AEGetNthPtr(&fieldList, i, typeChar, &theAEKeyWord,
								&theTypeCode, pField, fieldLen, &fieldLen);
		}

		if (nErr != noErr)
			break;

		String aColumnName(pField,fieldLen);

		SdbDatabaseType eType = SDB_DBTYPE_VARCHAR;
		UINT16 nPrecision = 0;
		UINT16 nScale = 0;

		SdbFILEColumn * pColumn = new SdbFILEColumn(aColumnName,eType,nPrecision,nScale);

		// Identitaet merken (hier eigentlich nicht erforderlich, aber aus
		// Symmetriegruenden ...)

		if (eType == SDB_DBTYPE_VARCHAR) {
			// CHARACTER-Felder koennen leer (NULL) sein.
			pColumn->SetFlag(SDB_FLAGS_NULLALLOWED);
		}

		// Column-Beschreibung hinzufuegen.
		aOriginalColumns->AddColumn(pColumn);

		if (fieldLen > 255)
		{
			delete pField;
			pField = Field;
		}
	}

	/* Dispose all the descriptos we used */
	AEDisposeDesc(&theAppleEvent);
	AEDisposeDesc(&theReply);
	AEDisposeDesc(&desc1);
	AEDisposeDesc(&colObject);
	AEDisposeDesc(&tempDesc);
	AEDisposeDesc(&layoutObj);

	if (nErr != noErr)
		aStatus.Set(SDB_STAT_ERROR);
	return aStatus.IsSuccessful();
}


/* -------------------------------------------------------------------------------------------------

	Gets the number of records in the current selection in the FileMaker database.

	------------------------------------------------------------------------------------------------- */

OSErr SdbFmTable::GetRecordCount()
{
	OSErr 			nErr;
	AppleEvent 		theAppleEvent, theReply;
	DescType 		theTypeCode;
	const DescType 	vCRow = cRow;
	long 			len;
	AEDesc 			tempDesc;

	/* All descriptors must be initaliased before use */
	theAppleEvent.dataHandle = theReply.dataHandle = tempDesc.dataHandle = nil;
	nErr = noErr;

	/* Create the Apple Event. */
	if (!nErr)
		nErr = AECreateAppleEvent( 'core', 'cnte', GetFMAdressDesc(),
										  kAutoGenerateReturnID, kAnyTransactionID,
										  &theAppleEvent );

	/* Put the document object at end of the Apple Event */
	if (!nErr)
		nErr = AEPutKeyDesc(&theAppleEvent, keyDirectObject, GetDatabaseSpec());

	/* Create a descriptor for the class we want to count. In this case it is the number of rows
	  we want. */
	if (!nErr)
		nErr = AECreateDesc(typeType, &vCRow, sizeof(DescType), &tempDesc);

	if (!nErr)
		nErr = AEPutKeyDesc(&theAppleEvent, 'kocl', &tempDesc);

	/* Send the event. */
	if (!nErr)
		nErr = AESend(&theAppleEvent, &theReply, kAEWaitReply | kAENeverInteract,
					 kAENormalPriority, 1200, Sysdepen::GetAEIdleProc(), nil);

	/* The result is returned in the variable nFmMaxPos. */
	if (!nErr)
		nErr = AEGetParamPtr( &theReply, keyDirectObject, typeLongInteger,
									&theTypeCode, &nFmMaxPos, sizeof(long), &len );
#ifdef DBG_UTIL
	GetAEEror(nErr,theReply);
#endif

	/* Dispose the descriptors we used. */
	AEDisposeDesc(&theAppleEvent);
	AEDisposeDesc(&theReply);
	AEDisposeDesc(&tempDesc);

	if (!nErr)
		GetpApp()->GetAppWindow()->ToTop();

	return nErr;

}


/* -------------------------------------------------------------------------------------------------

	Get a certain record from the database specified with the name in the Document and the record
	number in the variable recordNumber. The result is returned in a list, which should be in the
	same order as the name of the fields returned by GetHeaderRecord (see note about bug in
	FileMaker Pro 2.0 above).

	------------------------------------------------------------------------------------------------- */

OSErr SdbFmTable::GetRecord(long nNumber,
							 SdbRow& rRow,
							 SdbColumns& rColumns,
							 BOOL bConvertChar)
{
	OSErr 			nErr;
	AppleEvent 		theAppleEvent, theReply;
	DescType 		theTypeCode;
	long 			fieldLen, i, nItems;
	AEDesc 			desc0, desc1, recNum, recordObj, layoutObj;
	long 			layoutNumber = 0;
	const OSType 	reqType = 'list';
	AEDescList		recordList;
	char			Field[256];
	MAC_Ptr 			pField = Field;

	AEKeyword 		theAEKeyWord;

	/* All descriptors must be initaliased before use */
	theAppleEvent.dataHandle = nil;
	theReply.dataHandle = desc0.dataHandle = desc1.dataHandle = recNum.dataHandle = nil;
	recordObj.dataHandle = recordList.dataHandle = nil;
	nErr = noErr;

	/* All information you get from FileMaker must be from Layout #0. It is a non-visible, abstract
	  layout which contains all fields in the database. If we not specify Layout #0, we only get
	  the contents of the fields on the current layout. */
	if (!nErr)
		nErr = AECreateDesc(typeLongInteger, &layoutNumber, sizeof(layoutNumber), &desc0);

	if (!nErr)
		nErr = CreateObjSpecifier('ctbl', GetDatabaseSpec(), formAbsolutePosition, &desc0, kDisposeInputs, &layoutObj);

	/* We want the data from a specific record. Create a descriptor for that record number.
	  the record number must exist in the database, otherwise FileMaker will return "Object not
	  found". */
	if (!nErr)
		nErr = AECreateDesc(typeLongInteger, &nNumber, sizeof(long), &recNum);

	if (!nErr)
		nErr = CreateObjSpecifier('crow', &layoutObj, formAbsolutePosition, &recNum, kDisposeInputs, &recordObj);

	/* Create the Apple Event. */
	if (!nErr)
		nErr = AECreateAppleEvent( 'core', 'getd', GetFMAdressDesc(),
										  kAutoGenerateReturnID, kAnyTransactionID,
										  &theAppleEvent );

	/* Put the object specifying the record we want at the end of the event. */
	if (!nErr)
		nErr = AEPutKeyDesc(&theAppleEvent, keyDirectObject, &recordObj);
	AEDisposeDesc(&recordObj);

	/*	The result must be 'list'. Make a descriptor and put it at end of the Apple Event. */
	if (!nErr)
		nErr = AECreateDesc(typeChar, &reqType, sizeof(reqType), &desc1);

	if (!nErr)
		nErr = AEPutKeyDesc(&theAppleEvent, keyAERequestedType, &desc1);

	AEDisposeDesc(&desc1);


	/* Send the event */
	if (!nErr)
		nErr = AESend( &theAppleEvent, &theReply, kAEWaitReply | kAENeverInteract,
						  kAENormalPriority, kAEDefaultTimeout, Sysdepen::GetAEIdleProc(), nil );

	/* Get the result. It is a list with all the data. */
	if (!nErr)
		nErr = AEGetParamDesc(&theReply, keyDirectObject, typeAEList, &recordList);

	if ( nErr != noErr )
	{
		AEGetParamPtr(&theReply, keyErrorString, typeChar,
					&theTypeCode, pField, 255, &fieldLen);
		pField[fieldLen]='\0';
		DBG_ERROR(pField);
		// aStatus.CreateErrorMessage(pField,"FileMaker");
	}

	/* Count the number of fields. */
	nItems = 0;
	if (!nErr)
		nErr = AECountItems(&recordList, &nItems);

	DBG_ASSERT(rColumns.Count() == (nItems + 1),"Number of Colums are different ?");

	/* Get the data from each field. */
	if ( nErr == noErr)
	for (i = 1; i <= nItems; i++)
	{
		// Jetzt effizienter Zugreifen:
		SdbVariable* pVal = rRow.Variable(i);
		const SdbColumn * pColumn = rColumns[i];

		// Ist die Variable ueberhaupt gebunden?
		if (pVal == NULL) {
			// Nein - naechstes Feld.
			continue;
		}

		nErr = AEGetNthPtr(&recordList, i, typeChar, &theAEKeyWord, &theTypeCode, pField, 255, &fieldLen);

		if (nErr != noErr)
		{	pVal->PutNull();
			nErr = noErr;
			continue;
		}

		if (fieldLen > 255)
		{
			pField = new char[fieldLen];
			nErr = AEGetNthPtr(&recordList, i, typeChar, &theAEKeyWord, &theTypeCode, pField, fieldLen, &fieldLen);
		}

		if (fieldLen == 0) {
			pVal->PutNull();
		}
		else switch(pColumn->GetType())
		{
/***
			case SDB_DBTYPE_DATE:
			{
				USHORT  nYear   = aStr.Copy( 0, 4 );
				USHORT  nMonth  = aStr.Copy( 4, 2 );
				USHORT  nDay    = aStr.Copy( 6, 2 );

// nyi: Diese Routine nur EINMAL im ganzen Projekt!!!!!!
				// In Float umrechnen [nach Visual Basic-Konventionen,
				// d. h. Tage seit 1.1.1900 *** + 2.0 ***]
				Date aRefDate(1,1,1900);
				Date aDate(nDay,nMonth,nYear);

				// Anzahl Tage:
				long nDays;
				if (aRefDate > aDate)
				{
					nDays = (long)(aRefDate - aDate);
					nDays *= -1;
				} else {
					nDays = (long)(aDate - aRefDate);
				}
				nDays += 2;		// Basic-Anpassung!

				// Wert in Variable der Row uebernehmen
				pVariable->PutDouble((double)nDays);

				// nyi: Direkt als Date schreiben.
				// Vorlaeufig als Double und anschliessende Konversion:
				pVariable->Convert(SbxDATE);
			}
			break;
***/
/***
			case SDB_DBTYPE_DOUBLE:
			{
				char cDecimalDelimiter = ((SdbFmConnection *)pConnection)->DecimalDelimiter();
				char cThousandDelimiter = ((SdbFmConnection *)pConnection)->ThousandDelimiter();
				String aStrConverted;

				// In Standard-Notation (DezimalPUNKT ohne Tausender-Komma) umwandeln:
				for (USHORT i = 0; i < aStr.Len(); i++) {
					if (aStr[i] == cDecimalDelimiter) {
						aStrConverted += '.';
					} else if (aStr[i] == cThousandDelimiter) {
						// weglassen
					} else {
						aStrConverted += aStr[i];
					}
				}

				double n;
				n = strtod((const char *)aStrConverted,NULL);
				pVariable->PutDouble((double)n);
			}
			break;
***/
			default:
			{
				String aStr((char*) pField,fieldLen);

				// Zeichensatzkonversion
				if (bConvertChar)
					aStr.Convert(GetConnection()->GetCharacterSet());	// Brauchen wir (wohl) nicht

				// Wert als String in Variable der Row uebernehmen
				pVal->PutString(aStr);
			}
			break;
		}

		if (fieldLen > 255)
		{
			delete pField;
			pField = Field;
		}
	}

	AEDisposeDesc(&theAppleEvent);
	AEDisposeDesc(&theReply);
	AEDisposeDesc(&recordList);

	return nErr;
}


#pragma SEG_EOFMODULE
