/*************************************************************************
 *
 *  $RCSfile: fmconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "fmconn.hxx"
#include "fmtab.hxx"
#include <svtools/extattr.hxx>

#include "sdbenv.hxx"


DBG_NAME(SdbFmConnection);

static const char * const pFilenameExtension = "";
static const char * const pFileType = "FMP3";

//------------------------------------------------------------------
SdbFmConnection::SdbFmConnection (SdbEnvironment* pEnv, const String& rOptions)
				  :SdbFILEConnection(pEnv, rOptions,
									 pFilenameExtension),
									 cFieldDelimiter(';'),
									 cStringDelimiter('"'),
									 cDecimalDelimiter('.'),
									 cThousandDelimiter(',')

{
	DBG_CTOR(SdbFmConnection,NULL);

	// Verbindungsaufbau-Optionen:
	bFixedLength = aOptionsParser.OptionBool("FIX");
	bHeaderLine = aOptionsParser.OptionBool("HDR");

#ifdef MUSTER
	if (!isdigit(aOptionsParser.OptionString("DEL")[(UINT16)0])) {
		if (aOptionsParser.OptionString("DEL").Len() > 0) {
			cFieldDelimiter = aOptionsParser.OptionString("DEL")[(UINT16)0];
		}
	} else {
		cFieldDelimiter = (char) aOptionsParser.OptionInt("DEL");
	}
#endif

}

//------------------------------------------------------------------
SdbTable * SdbFmConnection::CreateTableImpl()
{
	DBG_CHKTHIS(SdbFmConnection,NULL);

	if (! IsOpen()) {
		aStatus.SetSequenceError();
		return NULL;
	}

	aStatus.Clear();
	return new SdbFmTable(this);
}


void SdbFmConnection::FillTypeList(SdbTypeInfoList& rList)
{
	SdbTypeInfo* pInfo 		= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_VARCHAR;
	pInfo->aTypeName		= "VARCHAR";
	pInfo->nPrecision		= 255;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DOUBLE;
	pInfo->aTypeName		= "DOUBLE";
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DATE;
	pInfo->aTypeName		= "DATE";
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);
}


BOOL SdbFmConnection::IsCapableImpl(SdbCapability eCapability) const
{
	DBG_CHKTHIS(SdbFmConnection,NULL);

	BOOL bResult;
	switch(eCapability)
	{
		case SDB_CAP_BOOKMARKS:
		case SDB_CAP_COMMIT_AUTO:
		case SDB_CAP_FETCH:
		case SDB_CAP_FETCH_NEXT:
		case SDB_CAP_FETCH_PRIOR:
		case SDB_CAP_FETCH_FIRST:
		case SDB_CAP_FETCH_LAST:
		case SDB_CAP_FETCH_RELATIVE:
		case SDB_CAP_FETCH_ABSOLUTE:
		case SDB_CAP_READ_UNCOMMITTED:
			bResult = TRUE;
			break;
		default:
			bResult =  SdbFILEConnection::IsCapableImpl(eCapability);
	}
	return bResult;
}


