/*************************************************************************
 *
 *  $RCSfile: fmcurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>			// sprintf
#include <string.h>			// strncpy
#include <stdlib.h>			// strtod
#include <limits.h>

// Fmcurs
#include "Fmcurs.hxx"
#include "Fmconn.hxx"

DBG_NAME(SdbFmCursor);

SdbFmCursor::SdbFmCursor(SdbFmConnection& rSdbFmConnection,
						   SdbCursorObject eObj,
						   SdbCursorType eType,
						   UINT16 nOptions)
			 :SdbFILECursor(rSdbFmConnection, eObj, eType, nOptions)
			 ,bFileIsOpen(FALSE)
{
	DBG_CTOR(SdbFmCursor,NULL);

	AECreateDesc(typeNull, nil, 0, &aFileMakerAdress);
	AECreateDesc(typeNull, nil, 0, &aDatabaseSpec);

	aStatus.Clear();
	aStatus.SetComponent("Sdb Filemake Driver");
}


void SdbFmCursor::FileOpen(const DirEntry& aFileEntry, BOOL bForUpdate)
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	// Sicherheitshalber erst ein Close (ohne Fehlerabfrage) ...
	FileClose();

	aStatus.Clear();

	OSErr nErr = OpenFMDatabase(aFileEntry.GetFull());

	if (nErr == noErr)
		nErr = GetRecordCount();

	if (nErr == noErr)
	{
		bFileIsOpen 	= TRUE;
		nFilePos 	= 0;

		FileReadColumns();
	}
	else
		aStatus.Set(SDB_STAT_ERROR);	// nyi: genauer!
}

void SdbFmCursor::FileClose()
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	if (bFileIsOpen)
	{
		aFmFileName.Erase();
		AEDisposeDesc(&aFileMakerAdress);
		AEDisposeDesc(&aDatabaseSpec);

		bFileIsOpen = FALSE;
	}
	SdbFILECursor::FileClose();
}


void SdbFmCursor::FileReset()
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	// nichts zu tun ...
	// nyi: oder doch?
}

BOOL SdbFmCursor::OnOpen()
{
	if (!IsOpen())
		nFilePos = 0;
	return SdbCursor::OnOpen();
}


void SdbFmCursor::Seek(SdbCursorPosition eCursorPosition, INT32 nOffset)
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	// ----------------------------------------------------------
	// Positionierung vorbereiten:
	UINT32 nTempPos = nFilePos;

	switch(eCursorPosition)
	{
		case SDB_POS_NEXT:
			nFilePos++;
			break;
		case SDB_POS_PRIOR:
			if (nFilePos > 0)
				nFilePos--;
			break;
		case SDB_POS_FIRST:
			nFilePos = 1;
			break;
		case SDB_POS_LAST:
			nFilePos = nFmMaxPos;
			break;
		case SDB_POS_RELATIVE:
			nFilePos = (((long)nFilePos) + nOffset < 0) ? 0L
							: (ULONG)(((long)nFilePos) + nOffset);
			break;
		case SDB_POS_ABSOLUTE:
		case SDB_POS_BOOKMARK:
			nFilePos = (ULONG)nOffset;
			break;
	}

	if (nFilePos > nFmMaxPos)
		nFilePos = nFmMaxPos + 1;

	if (nFilePos == 0 || nFilePos == nFmMaxPos + 1)
	{
		switch(eCursorPosition)
		{
			case SDB_POS_PRIOR:
			case SDB_POS_FIRST:
				nFilePos = -1;
				break;
			case SDB_POS_LAST:
			case SDB_POS_NEXT:
			case SDB_POS_ABSOLUTE:
			case SDB_POS_RELATIVE:
				if (nOffset > 0)
					nFilePos = nFilePos + 1;
				else if (nOffset < 0)
					nFilePos = -1;
				break;
			case SDB_POS_BOOKMARK:
				nFilePos = nTempPos;	 // vorherige Position
		}
		aStatus.Set(SDB_STAT_NO_DATA_FOUND);
	}
}

void SdbFmCursor::FetchRow(SdbRow& rRow, SdbColumns& rColumns, BOOL)
{
	OSErr  nErr = GetRecord(nFilePos,rRow,rColumns);
	if (nErr != noErr)	// nyi, Fehler besser auswerten !
	{
		aStatus.Set(SDB_STAT_ERROR);
		return;
	}
	else if (rRow.Value(0) != NULL) // Aktuelle Record Number: (ala Bookmark)
		rRow.Value(0)->PutLong(nFilePos);
}


void SdbFmCursor::FileInsertRow(SdbRowRef&)
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	DBG_ERROR("FileInsertRow noch nicht implementiert");
}


void SdbFmCursor::FileUpdateRow(const SdbRowRef&)
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	DBG_ERROR("FileUpdateRow noch nicht implementiert");
}


void SdbFmCursor::FileDeleteRow()
{
	DBG_CHKTHIS(SdbFmCursor,NULL);

	DBG_ERROR("FileDeleteRow noch nicht implementiert");
}


INT32 SdbFmCursor::MaxRowCount() const
{
	return (UINT32)nFmMaxPos;
}

