/*************************************************************************
 *
 *  $RCSfile: flatconn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "flatconn.hxx"
#include "flattab.hxx"
#include <ctype.h>

DBG_NAME(SdbFLATConnection);

static const char * const pFilenameExtension = "txt";

//------------------------------------------------------------------
SdbFLATConnection::SdbFLATConnection (SdbEnvironment* pEnv, const String& rOptions)
				  :SdbFILEConnection(pEnv, rOptions,
									 pFilenameExtension)
				  ,cFieldDelimiter(';')
				  ,cStringDelimiter('"')
				  ,cDecimalDelimiter('.')
				  ,cThousandDelimiter(',')
{
	DBG_CTOR(SdbFLATConnection,NULL);

	aStatus.SetComponent(String::CreateFromAscii("Sdb FLAT Driver"));

	// Verbindungsaufbau-Optionen:
	bFixedLength = aOptionsParser.OptionBool(String::CreateFromAscii("FIX"));
	bHeaderLine  = aOptionsParser.OptionBool(String::CreateFromAscii("HDR"));

	if (!isdigit(aOptionsParser.OptionString(String::CreateFromAscii("DEL")).GetChar(0)))
	{
		if (aOptionsParser.OptionString(String::CreateFromAscii("DEL")).Len() > 0)
			cFieldDelimiter = aOptionsParser.OptionString(String::CreateFromAscii("DEL")).GetChar(0);
	}
	else
		cFieldDelimiter = (char) aOptionsParser.OptionInt(String::CreateFromAscii("DEL"));


	if (!isdigit(aOptionsParser.OptionString(String::CreateFromAscii("STR")).GetChar(0)))
	{
		if (aOptionsParser.OptionString(String::CreateFromAscii("STR")).Len() > 0)
			cStringDelimiter = aOptionsParser.OptionString(String::CreateFromAscii("STR")).GetChar(0);

	}
	else
		cStringDelimiter = (char) aOptionsParser.OptionInt(String::CreateFromAscii("STR"));

	if (!isdigit(aOptionsParser.OptionString(String::CreateFromAscii("DECDEL")).GetChar(0)))
	{
		if (aOptionsParser.OptionString(String::CreateFromAscii("DECDEL")).Len() > 0)
			cDecimalDelimiter = aOptionsParser.OptionString(String::CreateFromAscii("DECDEL")).GetChar(0);
		else
			cDecimalDelimiter = 0;
	}
	else
		cDecimalDelimiter = (char) aOptionsParser.OptionInt(String::CreateFromAscii("DECDEL"));

	if (!isdigit(aOptionsParser.OptionString(String::CreateFromAscii("THODEL")).GetChar(0)))
	{
		if (aOptionsParser.OptionString(String::CreateFromAscii("THODEL")).Len() > 0)
			cThousandDelimiter = aOptionsParser.OptionString(String::CreateFromAscii("THODEL")).GetChar(0);
		else
			cThousandDelimiter = 0;
	}
	else
		cThousandDelimiter = (char) aOptionsParser.OptionInt(String::CreateFromAscii("THODEL"));
}
SdbFLATConnection::~SdbFLATConnection()
{
	DBG_DTOR(SdbFLATConnection,NULL);
}
//------------------------------------------------------------------
SdbTable* SdbFLATConnection::CreateTableImpl()
{
	DBG_CHKTHIS(SdbFLATConnection,NULL);

	return new SdbFLATTable(this);
}

//------------------------------------------------------------------
void SdbFLATConnection::FillTypeList(SdbTypeInfoList& rList)
{
	SdbTypeInfo* pInfo		= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_CHAR;
	pInfo->aTypeName		= String::CreateFromAscii("CHAR");
	pInfo->nPrecision		= 255;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_VARCHAR;
	pInfo->aTypeName		= String::CreateFromAscii("VARCHAR");
	pInfo->nPrecision		= 255;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_LONGVARCHAR;
	pInfo->aTypeName		= String::CreateFromAscii("LONGVARCHAR");
	pInfo->nPrecision		= 65535;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DECIMAL;
	pInfo->aTypeName		= String::CreateFromAscii("DECIMAL");
	pInfo->nPrecision	= 15;
	pInfo->nScale		= 15;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_NUMERIC;
	pInfo->aTypeName		= String::CreateFromAscii("NUMERIC");
	pInfo->nPrecision	= 15;
	pInfo->nScale		= 15;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= FALSE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DOUBLE;
	pInfo->aTypeName		= String::CreateFromAscii("DOUBLE");
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_INTEGER;
	pInfo->aTypeName		= String::CreateFromAscii("INTEGER");
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_DATE;
	pInfo->aTypeName		= String::CreateFromAscii("DATE");
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_TIMESTAMP;
	pInfo->aTypeName		= String::CreateFromAscii("DATETIME");
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

	pInfo					= new SdbTypeInfo;
	pInfo->eType			= SDB_DBTYPE_TIME;
	pInfo->aTypeName		= String::CreateFromAscii("TIME");
	pInfo->nPrecision		= 0;
	pInfo->nScale			= 0;
	pInfo->bCurrency		= FALSE;
	pInfo->bAutoIncrement	= FALSE;
	pInfo->bNullable		= TRUE;
	rList.Insert(pInfo, LIST_APPEND);

}

//------------------------------------------------------------------
BOOL SdbFLATConnection::IsCapableImpl(SdbCapability eCapability) const
{
	DBG_CHKTHIS(SdbFLATConnection,NULL);

	switch (eCapability)
	{
		case SDB_CAP_COMMIT_AUTO:
		case SDB_CAP_READ_UNCOMMITTED:
			return TRUE;
		case SDB_CAP_UPDATABLE:
		case SDB_CAP_BULKOPTIMIZE:
			return FALSE;
		default:
			return SdbFILEConnection::IsCapableImpl(eCapability);
	}
}

