/*************************************************************************
 *
 *  $RCSfile: impgen.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <iostream.h>
#include <ctype.h>

#include "impout.hxx"
#include "cmdparse.hxx"

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

//--------------------------------------------------------------------------

// Mini-Scanner:
static String aEmptyString;

static nLineNumber = 1;
static nColumnNumber = 0;

const String Token(SvStream & rStream)
{
	static char aChar;
	String aResult;

again:
	// Ein Zeichen lesen, EOF behandeln
	while (aChar == 0 || aChar == ' ' || aChar == '\t' || aChar == '\r' || aChar == '\n') {

		// Zeilen zaehlen (ob das wohl auf dem Mac klappt?)
		if (aChar == '\n') nLineNumber++;
		if (aChar == '\n' || aChar == '\r') nColumnNumber = 0;

		rStream >> aChar;
		nColumnNumber++;
		if (rStream.IsEof()) {
//cout << "Token: " << (const char *) aResult << "\n";
			return aResult;	// ggf. Leerstring falls EOF
		}
	}

	// erstes Zeichen != ' ' etc. gefunden:

	// Bezeichner:
	if (isalpha(aChar) || aChar == '_') {
		while (isalnum(aChar) || aChar == '_') {
			aResult += aChar;

			// weiter lesen ...
			rStream >> aChar;
			nColumnNumber++;
			if (rStream.IsEof()) {
				break;
			}
		}
	} else {
		// Token erkannt, der aus Sonderzeichen besteht.
		aResult += aChar;

		// weiter lesen ...
		rStream >> aChar;
		nColumnNumber++;

		// Kommentar?
		if (aResult == "/" && aChar == '/') {
			aResult = "";

			// Zeichen ueberlesen bis Ende der Zeile erreicht, EOF behandeln
			while (aChar != '\r' && aChar != '\n') {

				rStream >> aChar;
				nColumnNumber++;
				if (rStream.IsEof()) {
		//cout << "Token: " << (const char *) aResult << "\n";
					return aResult;	// ggf. Leerstring falls EOF
				}
			}

			// Jetzt am Ende der Zeile. Nochmal von vorne ...
			goto again;
		}
	}

//cout << "Token: " << (const char *) aResult << "\n";
	return aResult;
}

// Mini-Parser:

void Parse(SvStream & rInputStream, const String aModName, const String aFileName, const String aIncFile, const BOOL bVerbose = FALSE, const String &rFuncPrefix = aEmptyString)
{
	SvFileStream aHeaderStream;
	SvFileStream aBodyStream;

	String aFunctionBody2Tmp;	// gesammelte FunctionBody2-String ... (erspart 2. Durchlauf)

	String aToken;


	String aHeaderFileName(aFileName);
	aHeaderFileName += ".hxx";
	String aBodyFileName(aFileName);
	aBodyFileName += ".cxx";

	aHeaderStream.Open(aHeaderFileName,STREAM_WRITE | STREAM_TRUNC);
	if (aHeaderStream.GetError()) {
		cerr << "Cannot create file '" << (const char *) aHeaderFileName << "'\n";
		return;
	}
	aBodyStream.Open(aBodyFileName,STREAM_WRITE | STREAM_TRUNC);
	if (aBodyStream.GetError()) {
		cerr << "Cannot create file '" << (const char *) aBodyFileName << "'\n";
		aHeaderStream.Close();
		return;
	}

	aHeaderStream << (const char *) ImpModule(aModuleHeader1,aModName,aFileName,aIncFile);
	aBodyStream   << (const char *) ImpModule(aModuleBody1,aModName,aFileName,aIncFile);


	while (1) {
		aToken = Token(rInputStream);
		if (aToken.Len() == 0) break;

		if (aHeaderStream.IsOpen() && aBodyStream.IsOpen()) {
			// Funktionsdeklarationen parsen und sammeln ...
			String aRetTyp;
			String aModifier;
			String aFuncName;
			String aFuncParam;
			int    nNumParam = 0;

			String t[10];
			int i;
			for (i=0; i<10 && aToken != "("; i++) {
				t[i] = aToken;
// cout << "t[" << i << "] = " << (const char *) t[i] << "\n";
				aToken = Token(rInputStream);
				if (aToken.Len() == 0) {
					cerr << "Line #" << nLineNumber << ": "
						 << "EOF before start of function ...\n";
					break;
				}
			}
			if (aToken != "(") {
				cerr << "Line #" << nLineNumber << ": "
					 << "Cannot find '(' ...\n";
				break;
			}

			// Der Token an der aktuellen Array-Position ist der Funktionsname!
			if (i > 0) {
				aFuncName = t[i-1];
				if (bVerbose) {
					cout << "Line #" << nLineNumber << ": "
						 << "Function: " << (const char *) aFuncName << "\n";
				}

				// Jetzt Tokens im Array bearbeiten:
				// Compiler-Direktiven ...
				int k;
				for (k = i-2; k >= 0 && (t[k] == "SQL_API" || t[k][(USHORT)0] == '_') ; k--);

				for (int l = 0; l < i-1; l++) {
					if (l <= k) {
						// RetTyp
						aRetTyp += t[l];
						aRetTyp += ' ';
					} else {
						// Modifier
						aModifier += t[l];
						aModifier += ' ';
					}
				}

			}

			aToken = Token(rInputStream);
			if (aToken.Len() == 0) {
				cerr << "Line #" << nLineNumber << ": "
					 << "EOF in function parameter list ...\n";
				break;
			}

			while (aToken != ")") {
				aFuncParam += aToken;
				aFuncParam += ' ';

				aToken = Token(rInputStream);
				if (aToken.Len() == 0) {
					cerr << "Line #" << nLineNumber << ": "
						 << "EOF in function parameter list ...\n";
					break;
				}
			}

			aToken = Token(rInputStream);
			if (aToken.Len() == 0) {
				cerr << "Line #" << nLineNumber << ": "
					 << "EOF - ';' expected ...\n";
				break;
			}
			if (aToken != ";") {
				cerr << "';' expected ...\n";
				break;
			}

			aFuncParam.EraseTrailingChars(' ');
			if (aFuncParam.Len() > 0) {
				for (USHORT j = 0; j < aFuncParam.Len(); j++) if (aFuncParam[j] == ',') nNumParam++;
				nNumParam++;
			}

			aHeaderStream << (const char *) ImpFunction(aFunctionHeader1,aRetTyp,aModifier,aFuncName,aFuncParam,nNumParam,rFuncPrefix);
			aBodyStream   << (const char *) ImpFunction(aFunctionBody1,aRetTyp,aModifier,aFuncName,aFuncParam,nNumParam,rFuncPrefix);
			aFunctionBody2Tmp += ImpFunction(aFunctionBody2,aRetTyp,aModifier,aFuncName,aFuncParam,nNumParam,rFuncPrefix);

		}
	}

	aHeaderStream << (const char *) ImpModule(aModuleHeader2,aModName,aFileName,aIncFile);

	aBodyStream   << (const char *) ImpModule(aModuleBody2,aModName,aFileName,aIncFile);
	aBodyStream   << (const char *) aFunctionBody2Tmp;		// Jetzt erst die gesammelten Funktionszeiger ...

	if (aHeaderStream.IsOpen()) aHeaderStream.Close();
	if (aBodyStream.IsOpen()) aBodyStream.Close();
}

//--------------------------------------------------------------------------

int
#ifdef WNT
__cdecl
#endif
main(int argc, char * argv[])
{
	CommandParser aCommandParser("/MODULE:/BASENAME:/INCLUDE:stdlib.h/PREFIX:/VERBOSE:0",'/',':');
	// Include ist erforderlich, daher ist der Default etwas, was jedes System
	// hat und was nicht weiter stoert.

	aCommandParser.Parse(argc,argv);
	if (aCommandParser.ParameterCount() != 2 ||
		aCommandParser.IsInvalid() ||
		aCommandParser.OptionString("MODULE").Len() == 0) {
		cerr << "IMPGEN 004 [19-Apr-95 rdm]\n";
		cerr << "Create import library that loads DLLs/shared libs at runtime from header file.\n";
		cerr << "Usage: IMPGEN /MODULE:modname [/BASENAME:basename] [/INCLUDE:incfile]\n";
		cerr << "              [/PREFIX:funcprefix] [/VERBOSE] headerfile\n";
		cerr << "'basename.HXX' and 'basename.CXX' will be created (overwritten!)\n";
		cerr << "('modname.HXX' and 'modname.CXX' if /BASENAME option is missing).\n";
		return 1;
	}

	String aInputFileName = aCommandParser.ParameterString(1);
	String aModName = aCommandParser.OptionString("MODULE");
	String aFileName = aCommandParser.OptionString("BASENAME");
	if (aFileName.Len() == 0) aFileName = aModName;
	String aIncFile = aCommandParser.OptionString("INCLUDE");
	BOOL bVerbose = aCommandParser.OptionBool("VERBOSE");
	String aFuncPrefix = aCommandParser.OptionString("PREFIX");


	SvFileStream aInputStream(aInputFileName,STREAM_READ | STREAM_NOCREATE);
	if (aInputStream.GetError()) {
		cerr << "Cannot open '" << (const char *) aInputFileName << "'\n";
		return 1;
	}


	Parse(aInputStream,aModName,aFileName,aIncFile,bVerbose,aFuncPrefix);

	aInputStream.Close();

	return 0;
}

#ifdef WIN
SdbColumn::~SdbColumn(){};
#endif
