/*************************************************************************
 *
 *  $RCSfile: adaconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef SDB_ADACONN_HXX
#define SDB_ADACONN_HXX 1

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

// Class SdbAdaConnection
//
// Beschreibung:
//	Implementation der SdbAdaConnection fuer ODBC
//
class SdbAdaConnection : public SdbConnection
{
	friend class SdbEnvironment;
	friend class SdbAdaCursor;
	friend class SdbAdaTable;
	friend class SdbAdaRelation;

protected:
	// Neue Verbindungen werden nur mit SdbAdaConnection::Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbAdaConnection(SdbEnvironment* pEnv, const String & rOptions);
	SdbAdaConnection(const SdbAdaConnection& _rConn);

	// Verbindungen koennen nur mit der Create-Factory
	// *dynamisch* angelegt werden. Statische oder automatische
	// Instanzen sind nicht moeglich.
	// Zur Zerstoerung eines Connection-Objektes ist Delete zu
	// verwenden (welches selbst 'delete this' ausfuehrt).
	virtual ~SdbAdaConnection();

	//-------------------------------------------------------------
	// von der Implementation zu realisieren:

protected:
	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).
	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType,
										UINT16 nOptions);
	virtual SdbTable * CreateTableImpl();
	virtual SdbRelation* CreateRelationImpl(const String& _rName);

	virtual void SetAutoCommit(BOOL bAuto);

	// COMMIT ausfuehren
	virtual void CommitImpl();

	// ROLLBACK ausfuehren
	virtual void RollbackImpl();

	// Verbindung aufbauen
	virtual BOOL OpenImpl();

	// Verbindung schliessen
	virtual BOOL CloseImpl();

	static SdbDatabaseType MapType(INT16 nDB2Type);
	virtual void FillTypeList(SdbTypeInfoList& rList);
	virtual void FillTableNames(SdbStringStlMap* _pList);
	virtual void FillRelations(SdbRelationList& _rList, const String& rName);

	// Faehigkeiten des Treibers und des SQL-Dialektes:
	virtual BOOL IsCapableImpl(SdbCapability eCapability) const;
	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl();
	virtual void SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel);
	// Anzahl der Relationen, ggf. Relationenliste fuellen - fuer alle Tabellen
	virtual void FillAllRelations(SdbRelationList& _rList);
	virtual BOOL DoCmdImpl(UINT16 nCmd,const String &rArgs);

private:
	//-------------------------------------------------------------
	// Eigene Members der Implementation

	// Connection Handle (Environment siehe auch sdbenv.hxx)
	SQLHANDLE				aEnvironmentHandle;	 // einmal global
	SQLHANDLE				aConnectionHandle;	 // einmal pro Connection
	String					aDsn;		// Name der Datasource

	// Status der letzten Operation abfragen und aStatus entsprechend setzen.
	// Wenn sich die Operation auf eine Connection bezieht, muss deren Handle
	// mit angegeben werden.

	// Die ODBC-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	void SetStatus(RETCODE nRetcode, SQLSMALLINT HandleType, SQLHANDLE aHandle,
								 const String& rErrorInfo = aEmptyString);

	void Construct(SdbEnvironment* pEnv);

	BOOL FillRelation(SdbAdaRelation& rRel);
	SdbCursor* OpenRelationCursor(const String& rSchema, const String& rName);
	BOOL CheckNewDevSpace(const String& _rHostname);
	// return TRUE if DB is on local host
	BOOL GetDbNameFromDsn(const String& _rDsn,String &rDB) const;
	KernelState Actualize(const String& _rHost) const;

public:

	struct ServerInfo
	{
		String	aCtrlUser;
		String	aCtrlPwd;
		BOOL	aShutdown;
	};
	typedef ServerInfo	AdaServerInfo;

	String GetDsn() const {return aDsn;}
	String ConnectStr() const;
	RETCODE OpenConnection(const String& aConnectStr,INT32 nTimeOut);
	BOOL GetAdaInfo(UWORD nInfoType, String & rString) const;
	BOOL GetAdaInfo(UWORD nInfoType, USHORT & rNumber) const;
	BOOL GetAdaInfo(UWORD nInfoType, ULONG & rNumber) const;
	BOOL GetAdaInfo(UWORD nInfoType, BOOL & rBool) const;
	BOOL IsDriverCapable(UWORD nInfoType, ULONG nInfoFlag) const;
	static void FreeEnv(SdbEnvironment* pEnv);
	static map< ::rtl::OUString,AdaServerInfo,::utl::UStringMixLess> st_ServerMap;

	// Konvertierung von Datum/Zeitfeldern in die treiberspezifische Form
public:
	// ODBC Environment und Connection Handles
	// (Zugriff wird von SdbODBCCursor, SdbODBCTable, ... benoetigt)
	static SQLHANDLE EnvironmentHandle(SdbEnvironment* pEnv,String &_rPath);
	SQLHANDLE EnvironmentHandle() const;
	SQLHANDLE ConnectionHandle() const;

	// Namen aller verfuegbaren Server:
	static String ServerNames(SdbStatus& rStat, SdbEnvironment* pEnv);
};


// Inline-Implementationen:
inline SQLHANDLE SdbAdaConnection::ConnectionHandle() const
{
	return aConnectionHandle;
}

#endif


