/*************************************************************************
 *
 *  $RCSfile: adocurs.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef SDB_ADOCUR_HXX
#define SDB_ADOCUR_HXX

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

class SdbADOConnection;

#ifndef SDB_ADO_IMPL
#include "adoimp.hxx"
#endif
#ifndef SDB_WRAPADO_HXX
#include "wrapado.hxx"
#endif

#ifdef SDB_ADOCUR_CXX

#if STLPORT_VERSION < 321
#include <tools/presys.h>
#include <stl/vector.h>
#include <tools/postsys.h>
#else
#include <vector>
using namespace std;
#endif
#endif

//========================================================================
//
// Class SdbADOCursor
//
// Beschreibung:
//	Implementation des SdbCursor fuer Oracle 7.3
//
//========================================================================
class SdbADOCursor : public SdbCursor

{
	friend class SdbADOConnection;
	//friend class SdbADOTable;
private:

	// sorgt dafuer, dass Fehlermeldungen von ADO in den Status
	// geschrieben werden.
	inline void SetADOError();

	// hierin deponiert ADO die Abfrageergebnisse
	WpADORecordset	m_Recordset;

	// Command holds recordset
	WpADOCommand	m_Command;
	OLEVariant		m_RecordsAffected;
	OLEVariant		m_Parameters;
	vector<OLEVariant> m_aBookmarks;
	BOOL			bOnFirstAfterOpen;

	BOOL IsSelectStatement();
	BOOL UpdateEditableADORecord();

protected:
	SdbADOCursor(SdbADOConnection* pConnection, WpADOConnection* pADOConn, SdbCursorType eType, UINT16 nOptions );
	~SdbADOCursor();

	void DescribeResult();

	void DescribeParameter();

	void FixResultRow();		

	// Vorbereitung der SQL-Anfrage fuer die Ausfuehrung
	// Diese Methode muss auf jeden Fall vor Open() gerufen werden,
	// falls es sich um ein parametrisiertes SQL-Statement handelt.
	BOOL PrepareImpl();

	// Ausfuehrung der SQL-Anfrage (nach erfolgter Bindung
	// aller benoetigten Parameter).
	virtual BOOL OpenImpl(SdbCursorType eType = SDB_CURRENT,UINT16 eOpt = SDB_CURRENT_OPT);

	// Bricht die Ausfuehrung des Statements (waehrend Execute
	// oder waehrend Fetch-Operationen) ab.
	// Kann waehrend der Ausfuehrung eines Statements (aus dem
	// Busy Callback heraus) ausgefuehrt werden!
	// NICHT IMPLEMENTIERT.
	void Cancel() {};


	BOOL CloseImpl();


	BOOL MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset, SdbCursorMode& rMode);
	BOOL MoveImpl(const ::com::sun::star::uno::Any& rBookmark, SdbCursorMode& rMode, BOOL) throw( ::com::sun::star::lang::IllegalArgumentException );

	// gibt einen Zeiger auf die SdbADOConnection zurck,
	// zu der der Cursor gehrt.
	inline SdbADOConnection* const GetADOConnection() const
		{return (SdbADOConnection*) GetConnection();}

	void BindParameterValues();

	BOOL DeleteImpl();
	BOOL UpdateImpl();
	BOOL InsertImpl();

	virtual BOOL OnOpen();
	virtual BOOL IsCapable(SdbCursorCapability eCapability) const;
	virtual BOOL SdbADOCursor::SetUpdatableImpl(BOOL bUpdate);

	ADOParameter* SdbADOCursor::AddParameter(const SdbColumn& rCol, ADOParameters* pParameters);
};



#endif //SDB_ADOCUR_HXX


