/*************************************************************************
 *
 *  $RCSfile: dbfidx.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_DBFIDX_HXX
#define _SDB_DBFIDX_HXX 1

#ifndef _SDB_FILEIDX_HXX
#include "fileidx.hxx"
#endif

#ifndef _SDBCODE_HXX
#include "sdbcode.hxx"
#endif

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

//==================================================================
// Forward
//==================================================================
class SdbNDXPage;
class SdbNDXIndex;
class SdbDBFConnection;

#define dBASE_III_GROUP "dBase III"

//==================================================================
// Kopfsatz-Struktur, verbleibt im Speicher
//==================================================================
struct SdbNDXHeader
{
	UINT32  db_rootpage;                    /* Position der Rootpage		*/
	UINT32  db_pagecount;                   /* Anzahl Pages					*/
	BYTE	db_frei[4];						/* reserviert					*/
	UINT16  db_keylen;						/* Laenge des Schluessels	    */
	UINT16  db_maxkeys;						/* Max. # keys pro Seite	    */
	UINT16  db_keytype;                     /* Art des Schluessels
											   (0-Text)
											   (1-Numerisch)				*/
	UINT16  db_keyrec;                      /* Laenge eines IndexSatzes
											   SatzNr + keylen				*/
	BYTE	db_frei1[3];					/* reserviert					*/
	BYTE	db_unique;						/* eindeutig					*/
	char    db_name[488];                   /* index_name	(Feldname)		*/
};

//==================================================================
// Index Key
//==================================================================
class SdbNDXKey : public SdbOperand
{
	friend class SdbNDXNode;
	UINT32			nRecord;				/* Satzzeiger				*/
	ODbVariantRef	xValue;					/* Schluesselwert			*/

public:
	SdbNDXKey():nRecord(0){}
	inline SdbNDXKey(const ODbVariant& rVal, SdbDatabaseType eType, UINT32 nRec);
	inline SdbNDXKey(const rtl::OUString& aStr, UINT32 nRec = 0);
	inline SdbNDXKey(double aVal, UINT32 nRec = 0);
	inline SdbNDXKey(UINT32 nRec);
	inline SdbNDXKey(const SdbNDXKey& rKey);	

	inline SdbNDXKey& operator= (const SdbNDXKey& rKey);
	inline void SetValue(const ODbVariantRef& xVal);

	ODbVariant* Value() {return xValue.getBodyPtr();}
	virtual const ODbVariant* Value() const {return xValue.getBodyPtr();}

	BOOL   Is() const {return xValue.getBodyPtr() != NULL;}
	UINT32 GetRecord() const {return nRecord;}
	void   ResetRecord() {nRecord = 0;}

	BOOL operator == (const SdbNDXKey& rKey) const;
	BOOL operator != (const SdbNDXKey& rKey) const;
	BOOL operator <	 (const SdbNDXKey& rKey) const;
	BOOL operator <= (const SdbNDXKey& rKey) const;
	BOOL operator >  (const SdbNDXKey& rKey) const;
	BOOL operator >= (const SdbNDXKey& rKey) const;

	BOOL Load (SvFileStream& rStream, BOOL bText);
	BOOL Write(SvFileStream& rStream, BOOL bText);

	static BOOL IsText(SdbDatabaseType eType);

private:
	StringCompare Compare(const SdbNDXKey& rKey) const;
};


//==================================================================
// Index Seitenverweis
//==================================================================
SV_DECL_REF(SdbNDXPage); // Basisklasse da weitere Informationen gehalten werden muessen

class SdbNDXPagePtr : public SdbNDXPageRef
{
	friend  SvStream& operator << (SvStream &rStream, const SdbNDXPagePtr&);
	friend  SvStream& operator >> (SvStream &rStream, SdbNDXPagePtr&);

	UINT32	nPagePos;		// Position in der Indexdatei

public:
	SdbNDXPagePtr(UINT32 nPos = 0):nPagePos(nPos){}
	SdbNDXPagePtr(const SdbNDXPagePtr& rRef);
	SdbNDXPagePtr(SdbNDXPage* pRefPage);

	SdbNDXPagePtr& operator=(const SdbNDXPagePtr& rRef);
	SdbNDXPagePtr& operator=(SdbNDXPage* pPageRef);

	UINT32 GetPagePos() const {return nPagePos;}
	BOOL HasPage() const {return nPagePos != 0;}
};

//==================================================================
// Index Knoten
//==================================================================
class SdbNDXNode
{
	friend class SdbNDXPage;
	SdbNDXPagePtr aChild;				/* naechster Seitenverweis	*/
	SdbNDXKey	  aKey;

public:
	SdbNDXNode(){}
	SdbNDXNode(const SdbNDXKey& rKey,
			   SdbNDXPagePtr aPagePtr = SdbNDXPagePtr())
			   :aKey(rKey), aChild(aPagePtr) {}

	// verweist der Knoten auf eine Seite
	BOOL			HasChild() const {return aChild.HasPage();}
	// Ist ein Index angegeben, kann gegebenfalls die Seite nachgeladen werden
	SdbNDXPagePtr&	GetChild(SdbNDXIndex* pIndex = NULL, SdbNDXPage* = NULL);

	const SdbNDXKey& GetKey() const {return aKey;}
	SdbNDXKey&		 GetKey() {return aKey;}

	// Setzen des Childs, ueber Referenz, um die PagePos zu erhalten
	void			SetChild(SdbNDXPagePtr aCh = SdbNDXPagePtr(), SdbNDXPage* = NULL);
	void			SetKey(SdbNDXKey& rKey) {aKey = rKey;}

	void Write(SvStream &rStream, const SdbNDXPage& rPage) const;
	void Read(SvStream &rStream, SdbNDXIndex&);
};

#define NODE_NOTFOUND 0xFFFF

//==================================================================
// Index Seite
//==================================================================
class SdbNDXPage : public SvRefBase
{
	friend class SdbNDXIndex;

	friend  SvStream& operator << (SvStream &rStream, const SdbNDXPage&);
	friend  SvStream& operator >> (SvStream &rStream, SdbNDXPage&);

	UINT32		nPagePos;				// Position in der Indexdatei
	BOOL		bModified : 1;
	USHORT		nCount;

	SdbNDXPagePtr	aParent,			// VaterSeite
					aChild;				// Zeiger auf rechte ChildPage
	SdbNDXIndex& rIndex;
	SdbNDXNode*  ppNodes;				// array von Knoten

public:
	// Knoten Operationen
	USHORT	Count() const {return nCount;}

	BOOL	Insert(SdbNDXNode& rNode, ULONG nRowsLeft = 0);
	BOOL	Insert(USHORT nIndex, SdbNDXNode& rNode);
	BOOL	Append(SdbNDXNode& rNode);
	BOOL	Delete(USHORT);
	void	Remove(USHORT);
	void	Release(BOOL bSave = TRUE);
	void	ReleaseFull(BOOL bSave = TRUE);

	// Aufteilen und Zerlegen
	SdbNDXNode Split(SdbNDXPage& rPage);
	void Merge(USHORT nParentNodePos, SdbNDXPagePtr xPage);

	// Zugriffsoperationen
	SdbNDXNode& operator[] (USHORT nPos);
	const SdbNDXNode& operator[] (USHORT nPos) const;

	BOOL IsRoot() const {return !aParent.Is();}
	BOOL IsLeaf() const {return !aChild.HasPage();}
	BOOL IsModified() const {return bModified;}
	BOOL HasParent() {return aParent.Is();}
	BOOL HasChild() const {return aChild.HasPage();}

	inline BOOL IsFull() const;

	UINT32 GetPagePos() const {return nPagePos;}
	SdbNDXPagePtr& GetChild(SdbNDXIndex* pIndex = 0);

	// Parent braucht nicht nachgeladen zu werden
	SdbNDXPage* GetParent() {return aParent;}
	SdbNDXIndex& GetIndex() {return rIndex;}
	const SdbNDXIndex& GetIndex() const {return rIndex;}

	// Setzen des Childs, ueber Referenz, um die PagePos zu erhalten
	void SetChild(SdbNDXPagePtr aCh = SdbNDXPagePtr());
	void SetParent(SdbNDXPagePtr aPa = SdbNDXPagePtr()) {aParent = aPa;}

	USHORT Search(const SdbNDXKey& rSearch);
	USHORT Search(const SdbNDXPage* pPage);
	void   SearchAndReplace(const SdbNDXKey& rSearch, SdbNDXKey& rReplace);

protected:
	SdbNDXPage(SdbNDXIndex& rIndex, ULONG nPos, SdbNDXPage* = NULL);
	~SdbNDXPage();

	virtual void QueryDelete();

	void SetModified(BOOL bMod) {bModified = bMod;}
	void SetPagePos(UINT32 nPage) {nPagePos = nPage;}

	BOOL Find(const SdbNDXKey&);	// rek. Abstieg
	USHORT FindPos(const SdbNDXKey& rKey) const;

#ifdef DEBUG
	void PrintPage();
#endif
};

DECLARE_LIST(SdbNDXPageList, SdbNDXPage*);
//==================================================================
// IndexStruktur
//==================================================================
class SdbNDXIndex : public SdbFILEIndex
{
	friend SvStream& operator << (SvStream &rStream, SdbNDXIndex&);
	friend SvStream& operator >> (SvStream &rStream, SdbNDXIndex&);

	friend class SdbDBFTable;
	friend class SdbNDXNode;
	friend class SdbNDXPage;
	friend class SdbNDXPagePtr;
	friend class SdbNDXIterator;

	SdbNDXHeader	aHeader;					// Kopfinformationen
	SdbNDXPageList	aCollector;					// Pool von nicht mehr bentigten Seiten
	SdbNDXPagePtr	aRoot,						// Wurzel des b+ Baums
					aCurLeaf;					// aktuelles Blatt
	USHORT			nCurNode;					// Position des aktuellen Knoten

	UINT32			nPageCount,
					nRootPage;

	SvFileStream aFileStream;					// Stream zum Lesen/Schreiben des Index
	BOOL bUseCollector : 1;						// Verwenden des GarbageCollectors

public:
	virtual BOOL Insert(UINT32 nRec, const ODbVariant& rValue);
	virtual BOOL Update(UINT32 nRec, const ODbVariant&, const ODbVariant&);
	virtual BOOL Delete(UINT32 nRec, const ODbVariant& rValue);
	virtual BOOL Find(UINT32 nRec, const ODbVariant& rValue);
	virtual BOOL Lock(BOOL bReadOnly);
	virtual void UnLock();
	virtual SdbFILEIndexIterator* CreateIterator(SdbBoolOperator* pOp,
												 const SdbOperand* pOperand = NULL);

	const SdbNDXHeader& GetHeader() const {return aHeader;}

	void SetRootPos(UINT32 nPos) {nRootPage = nPos;}
	void SetPageCount(UINT32 nCount) {nPageCount = nCount;}

	UINT32 GetRootPos() {return nRootPage;}
	UINT32 GetPageCount() {return nPageCount;}

	BOOL IsText() const {return aHeader.db_keytype == 0;}
	USHORT GetMaxNodes() const {return aHeader.db_maxkeys;}
	// eintrag in der INF Datei
	void CreateINFEntry();

	virtual void Close();

protected:
	SdbNDXIndex(SdbDBFTable* pTable);
	~SdbNDXIndex();

	virtual BOOL CreateImpl();
	virtual BOOL DropImpl();
	virtual BOOL OpenImpl();

	virtual BOOL DoRefresh();

	// Tree operationen
	void Insert(SdbNDXPagePtr aCurPage, SdbNDXNode& rNode);
	void Release(BOOL bSave = TRUE);

	BOOL ConvertToKey(SdbNDXKey& rKey, UINT32 nRec, const ODbVariant& rValue);

	// Garbage Collector
	BOOL UseCollector() const {return bUseCollector;}
	SdbNDXPage* CreatePage(UINT32 nPagePos, SdbNDXPage* pParent = NULL, BOOL bLoad = FALSE);
	void Collect(SdbNDXPage*);

	SdbNDXPagePtr GetRoot();

private:
	SdbDBFTable* GetDBFTable() const {return (SdbDBFTable*) GetTable();}
	SdbDBFConnection* GetDBFConnection() const {return (SdbDBFConnection*) GetConnection();}

#ifdef DEBUG
	void PrintTree();
#endif
};

//==================================================================
// IndexIterator
//==================================================================
class SdbNDXIterator : public SdbFILEIndexIterator
{
	SdbNDXPagePtr	 aRoot,aCurLeaf;
	USHORT			 nCurNode;

public:
	SdbNDXIterator(SdbFILEIndex* pInd, SdbBoolOperator* pOp, const SdbOperand* pOper);

	virtual ~SdbNDXIterator();
	virtual ULONG First();
	virtual ULONG Next();

protected:
	ULONG Find(BOOL bFirst);
	ULONG GetCompare(BOOL bFirst);
	ULONG GetLike(BOOL bFirst);
	ULONG GetNull(BOOL bFirst);
	ULONG GetNotNull(BOOL bFirst);

	SdbNDXKey* GetFirstKey(SdbNDXPage* pPage,
						   const SdbOperand& rKey);
	SdbNDXKey* GetNextKey();
	SdbNDXIndex* GetNDXIndex() const {return (SdbNDXIndex*) pIndex;}
};


//==================================================================
// inline implementation
//==================================================================
inline SdbNDXKey::SdbNDXKey(const ODbVariant& rVal, SdbDatabaseType eType, UINT32 nRec)
				 :SdbOperand(eType)
				 ,nRecord(nRec),xValue(new ODbVariant(rVal))
{		
}


inline SdbNDXKey::SdbNDXKey(const rtl::OUString& aStr, UINT32 nRec)
				 :SdbOperand(SDB_DBTYPE_VARCHAR)
				 ,nRecord(nRec),xValue(new ODbVariant())
{
	if (aStr.len())
		xValue->setString(aStr);	
}

inline SdbNDXKey::SdbNDXKey(double aVal, UINT32 nRec)
				 :SdbOperand(SDB_DBTYPE_DOUBLE)
				 ,nRecord(nRec),xValue(new ODbVariant(aVal))
{
}

inline SdbNDXKey::SdbNDXKey(UINT32 nRec)
				 :nRecord(nRec),xValue(new ODbVariant())
{
}

inline SdbNDXKey::SdbNDXKey(const SdbNDXKey& rKey)
				 :SdbOperand(rKey.DBType())
				 ,nRecord(rKey.nRecord),xValue(rKey.xValue)
{		
}

inline SdbNDXKey& SdbNDXKey::operator=(const SdbNDXKey& rKey)
{
	SetValue(rKey.xValue);
	nRecord = rKey.nRecord;
	eDBType = rKey.DBType();
	return *this;
}

inline void SdbNDXKey::SetValue(const ODbVariantRef& xVal)
{
	xValue = xVal;		
}

inline BOOL SdbNDXKey::operator == (const SdbNDXKey& rKey) const
{
	return Compare(rKey) == COMPARE_EQUAL;
}
inline BOOL SdbNDXKey::operator != (const SdbNDXKey& rKey) const
{
	return !operator== (rKey);
}
inline BOOL SdbNDXKey::operator <  (const SdbNDXKey& rKey) const
{
	return Compare(rKey) == COMPARE_LESS;
}
inline BOOL SdbNDXKey::operator >  (const SdbNDXKey& rKey) const
{
	return Compare(rKey) == COMPARE_GREATER;
}
inline BOOL SdbNDXKey::operator <= (const SdbNDXKey& rKey) const
{
	return !operator > (rKey);
}
inline BOOL SdbNDXKey::operator >= (const SdbNDXKey& rKey) const
{
	return !operator< (rKey);
}

inline BOOL SdbNDXPage::IsFull() const
{
	return Count() == rIndex.GetHeader().db_maxkeys;
}

inline void SdbNDXPage::SetChild(SdbNDXPagePtr aCh)
{
	aChild = aCh;
	if (aChild.Is())
		aChild->SetParent(this);
}

inline void SdbNDXNode::SetChild(SdbNDXPagePtr aCh, SdbNDXPage* pParent)
{
	aChild = aCh;
	if (aChild.Is())
		aChild->SetParent(pParent);
}

#endif
