/*************************************************************************
 *
 *  $RCSfile: dbftab.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_DBFTAB_HXX
#define _SDB_DBFTAB_HXX 1

#ifndef _SDB_FILETAB_HXX
#include "filetab.hxx"
#endif

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

class DirEntry;

// der Typ einer dBase datei wird mit dem ersten Byte bestimmt
enum DBFType  {	dBaseIII = 0x03,
				dBaseIV  = 0x04,
				dBaseV	 = 0x05,
				dBaseFS  = 0x43,
				dBaseFSMemo  = 0xB3,
				dBaseIIIMemo = 0x83,
				dBaseIVMemo = 0x8B,
				dBaseIVMemoSQL = 0x8E,
				FoxProMemo = 0xF5
			  };

// Format von DBF-Dateien (uebernommen aus svbase):
/*--------------------------------------------------------------------
	 Beschreibung:	DFH
 --------------------------------------------------------------------*/

struct SdbDBFHeader {                       /* Kopfsatz-Struktur            */
	DBFType db_typ;		                    /* Dateityp						*/
	BYTE    db_aedat[3];                    /* Datum der letzen Aenderung   */
											/* JJ MM TT                     */
	ULONG   db_anz;                         /* Anzahl der Saetze            */
	USHORT  db_kopf;                        /* laenge Kopfsatz-Struktur     */
	USHORT  db_slng;                        /* laenge der Daten-Saetze      */
	BYTE    db_frei[20];                    /* reserviert                   */
};

/*--------------------------------------------------------------------
	 Beschreibung: Field Description Array (FDA) fuer jedes Feld ein FDA
 --------------------------------------------------------------------*/

struct SdbDBFColumn {                       /* Feldbezeichner               */
	BYTE    db_fnm[11];                     /* Feldname                     */
	BYTE    db_typ;                         /* Feldtyp                      */
	ULONG   db_adr;                         /* Feldadresse                  */
	BYTE    db_flng;                        /* Feldlaenge                   */
	BYTE    db_dez;                         /* Dezimalstellen fuer N        */
	BYTE    db_frei2[14];                   /* reserviert                   */
};


/*--------------------------------------------------------------------
	 Beschreibung:	Memodateityp
 --------------------------------------------------------------------*/
enum DBFMemoType  {	MemodBaseIII = 0,
					MemodBaseIV,
					MemoFoxPro
				  };

/*--------------------------------------------------------------------
	 Beschreibung:	Memodatei-Header
 --------------------------------------------------------------------*/
struct SdbMemoHeader
{
	DBFMemoType db_typ;						/* Dateityp						*/
	UINT32	db_next;						/* nchster freier Block		*/
	USHORT	db_size;						/* Blockgre: dBase 3 fest		*/
};


#define TRY_MEMOFILE(action, Type)\
	action;\
	if (aStatus.SetError(aMemoFileStream,Type))\
	{\
		aMemoFileStream.ResetError();\
		return FALSE;\
	}

//==================================================================
// DBase Tabelle
//==================================================================
class SdbDBFTable : public SdbFILETable
{
	friend class SdbDBFConnection;
	friend class SdbNDXIndex;

	// Header-Record der DBF-Datei:
	SdbDBFHeader aFileHeader;

	// Header-Record einer Memo-Datei, falls vorhanden:
	SdbMemoHeader aMemoHeader;

	// Read/Write Buffer fuer dBase-Records
	BYTE	*pBuffer;
	UINT16  nBufferSize;	// Groesse des ReadBuffer, wenn pBuffer != NULL

	// Stream fuer DBF, DBT (Dateien)
	SvFileStream aFileStream, aMemoFileStream;

public:
	SdbDBFConnection* GetDBFConnection() const {return (SdbDBFConnection*) GetConnection();}

	// Allgemeine Tabellen Funktionalitaet
protected:
	SdbDBFTable(SdbDBFConnection* pConnection);
	~SdbDBFTable();

	virtual BOOL CreateImpl();
	virtual BOOL AlterImpl(); 
	virtual BOOL DropImpl();  
	virtual BOOL OpenImpl();
	virtual BOOL RenameImpl(const String& rNewName);

	virtual SdbIndex* CreateIndexImpl();
	virtual void FillIndexNames(SvStringsDtor& _rList);

	// Filespezifische Funktionalitaet
protected:
	// FileOperationen
	virtual BOOL FileOpen(BOOL bReadOnly = TRUE); 
	virtual void FileClose(); 
	virtual BOOL IsFileOpen() const;

	virtual BOOL DescribeColumns();
	virtual void ExtDescribeColumns();

	virtual BOOL SeekRow(SdbFileCursorPosition eCursorPosition, INT32 nOffset, INT32& nCurPos); 
	virtual BOOL FetchRow(ODbRow& rRow, SdbColumns& rColumns, BOOL bRetrieveData); 
	virtual BOOL InsertRow(ODbRowRef& rRow, BOOL bFlush); 
	virtual BOOL UpdateRow(const ODbRowRef& rRow, ODbRow* pOrgRow); 
	virtual BOOL DeleteRow(); 

	// determine the position of the last row at the moment
	virtual INT32 GetCurrentLastPos() const {return aFileHeader.db_anz;}

	BOOL Compress();

private:
	BOOL ReadHeader(); 

	// Erzeugen der dBase Dateien
	BOOL CreateFile(const DirEntry& aFile, BOOL& bCreateMemo); 

	// Memo operationen
	BOOL CreateMemoFile(const DirEntry& aFile); 
	BOOL ReadMemo(ULONG nBlockNo, ODbVariant& aVariable);
	BOOL WriteMemo(ODbVariant& aValue, ULONG& rBlockNo);
	BOOL ReadMemoHeader(); 

	// Feststellen ob Memofelder vorhanden
	BOOL IsMemoAvailable() const {return aMemoFileStream.IsOpen();}
	BOOL HasMemoFields() const { return aFileHeader.db_typ > dBaseIV;} 

	void AllocBuffer(); 
	// Hilfsroutine (gemeinsam fuer Insert und Update):
	// neue oder geaenderte Daten in den Puffer eintragen
	BOOL UpdateBuffer(const ODbRowRef& rRow, ODbRow* pOrgRow = NULL);

	// Puffer (nach Aenderungen oder beim Einfuegen) in die
	// Datei (zurueck-)schreiben. Bei bAppend == TRUE:
	// Puffer hinten an die Datei anfuegen.
	BOOL WriteBuffer();	
};

// Inline-Implementationen:

#endif



