/*************************************************************************
 *
 *  $RCSfile: fileconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_FILECONN_HXX
#define _SDB_FILECONN_HXX 1

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

#ifndef _FSYS_HXX //autogen
#include <tools/fsys.hxx>
#endif

class SdbFILEConnection : public SdbConnection
{
	friend class SdbFILECursor;
	friend class SdbFILEMetaData;

protected:
	// Zusaetzlich hier: Filename Extension (z. B. "dbf") der
	// jeweiligen Implementation.
	SdbFILEConnection(SdbEnvironment* pEnv, const String& rOptions,
					  const char* pFilenameExtension);

	virtual ~SdbFILEConnection();

protected:
	//-------------------------------------------------------------
	// Eigene Member der Implementation

	// Filename Extension der jeweiligen Implementation (z. B. "dbf"):
	String aFilenameExtension;

	// Verzeichnisname (Server), in dem Dateien (Tabellen) gesucht werden
	String aDirectoryName;

	// Dito als "Dir"-Struktur (fuer Zugriff auf enthaltene FILE-Dateien):
	Dir *pDir;

	// Keine Einschraenkung bei der Lnge des Dateinamen einer Tabelle
	BOOL bNoFileNameLimit,
		 bShowDeleted;		// sollen auch geloeschte Datensaetze angezeigt werden


public:
	// Zugriff auf eigene Member (z. B. vom Cursor aus):
	const String& FilenameExtension() const { return aFilenameExtension; }
	const String& DirectoryName() const { return aDirectoryName; }
	BOOL GetShowDeleted() const {return bShowDeleted;}

	// Erzeugen eines Dateinamen aus einem Tabellennamen
	DirEntry  FileEntry(const String& rTableName) const;

	// ueberladen da niemals CaseSensitive, Tabellennamen hingegen schon
	virtual SdbColumns* CreateColumns();
	virtual SdbColumns* CreateColumns(const SdbColumns& _rColumns);

	virtual SdbColumn*  CreateColumn(const String& _rName,
									 SdbDatabaseType _eType,
									 UINT32 _nLength = 0, UINT16 _nScale = 0,
									 UINT16 _nFlags = 0);
	virtual SdbColumn*  CreateColumn(const SdbColumn& _rColumn);

public:

	// Namen aller verfuegbaren Server:
	static String ServerNames(SdbStatus& rStat, SdbEnvironment* pEnv);

protected:
	// Informationen ueber die Namen der Tabellen und Views, die im Default-
	// Schema (oder optional in einem anderen Schema) vorhanden sind.
	virtual void FillTableNames(SdbStringStlMap* _pList);
	virtual void FillRelations(SdbRelationList& _rList, const String& rName);

	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType = SDB_KEYSET,
										UINT16 nOptions = SDB_READONLY);

		// Verbindung aufbauen
	virtual BOOL OpenImpl();

	// Verbindung schliessen
	virtual BOOL CloseImpl();

	// virtual void Automatisches Commit
	virtual void SetAutoCommit(BOOL bAuto);

	// COMMIT ausfuehren
	virtual void CommitImpl();

	// ROLLBACK ausfuehren
	virtual void RollbackImpl();

	// Metadaten der Datenbank als Interface
	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl();
};

// Pruefung auf Wildcard Match:
// "match"t der Pattern in pWild mit dem String in pStr?
// Vertragen sich die beiden, so wird TRUE zurueckgegeben, sonst FALSE.
//
// ein '%' in pWild bedeutet n beliebige Zeichen, mit n>=0
// ein '_' in pWild bedeutet genau ein beliebiges Zeichen
//
// Ein '%' oder '_' in pWild, das das Fluchtzeichen cEscape (z. B: '\\') vorweg hat,
// wird als normaler Buchstabe und nicht als Pattern interpretiert. Wenn nicht
// angegeben, kein Fluchtzeichen.
BOOL match(const sal_Unicode* pWild, const sal_Unicode* pStr, const sal_Unicode cEscape = L'\0');

// comparision of text
INT32 compareIgnoreCase(const rtl::OUString& rStr1, const rtl::OUString& rStr2, const rtl::OLocale& rLocale);

#endif


