/*************************************************************************
 *
 *  $RCSfile: filecurs.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
			   
#ifndef _SDB_FILECURS_HXX
#define _SDB_FILECURS_HXX 1

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SDB_SDBANALY_HXX
#include "sdbanaly.hxx"
#endif

enum SdbFileCursorPosition
{
	FILE_NEXT		= SDB_POS_NEXT,
	FILE_PRIOR		= SDB_POS_PRIOR,			
	FILE_FIRST		= SDB_POS_FIRST,			
	FILE_LAST		= SDB_POS_LAST,			
	FILE_RELATIVE	= SDB_POS_RELATIVE,		
	FILE_ABSOLUTE	= SDB_POS_ABSOLUTE,
	FILE_BOOKMARK	= SDB_POS_ABSOLUTE + 1
};


#include <limits.h>

class SdbFILESortIndex;
class SdbFILEKeyValue;
class SdbFILESet;

#define SDB_NO_PARAMETER (ULONG_MAX)

// Class SdbFILECursor
//
//
#define MAXORDER	10

class SdbFILECursor : public SdbCursor
{
	friend class SdbFILEConnection;
	friend class SdbFILETable;
	

	// Stream fuer Lesen/Schreiben aus/in FILE-Datei
	SdbFILEAnalyzer aSQLAnalyzer;

	// Maximal drei Column-Nummern, nach denen sortiert wird (ORDER BY-Kriterien).
	// Die Numerierung bezieht sich auf die PHYSISCHE (FILE...) Struktur,
	// NICHT auf die Struktur gemaess Select Statement.
	// 0, wenn keine Sortierung (wenn Column-Nummer 0 ist, muessen alle folgenden
	// Column-Nummern auch 0 sein).
	UINT16 nOrderbyColumnNumber[MAXORDER];

	// Das zugehoerige Flag "Ascending" (TRUE = Ascending, FALSE = Decending)
	// (nur relevant, wenn ColumnNumber != 0):
	BOOL bOrderbyAscending[MAXORDER];

	enum SdbFILEStatementType
	{
		SDB_STATEMENT_UNKNOWN,
		SDB_STATEMENT_SELECT,
		SDB_STATEMENT_SELECT_COUNT,
		SDB_STATEMENT_INSERT,
		SDB_STATEMENT_UPDATE,
		SDB_STATEMENT_DELETE
	};
protected:
	// KeySet fuer direkte Positionierung
	SdbFILESortIndex* pSortIndex;
	SdbFILESet* pFileSet;
	SdbFILESet* pEvaluationKeySet;

	// Maximal drei mit OR verknuepfte Saetze von Selektionskriterien:
	// Die Rows, sofern vorhanden, enthalten die Werte mit denen die
	// FileRow beim FileFetch verglichen wird (die Rows sind von Typ
	// "FileColumns").
	// Art des Vergleiches: zur Zeit immer Gleichheit ("=") mit Sonderbehandlung
	// fuer "LIKE" und "IS NULL".
	//ODbRow * pSelectionCriteria[6];

	// Bei UPDATE oder INSERT-Statement: die neu zuzuweisenden Werte.
	// NULL = kein UPDATE oder INSERT-Statement.
	// Variable nicht gebunden = wird nicht aktualisiert (nur bei UPDATE moeglich),
	// Variable NULL = wird auf NULL gesetzt.
	ODbRowRef aAssignValues;

	// Row die jeweils gefetched wird
	ODbRowRef aEvaluateRow,	// Row zur SQL-Analyse
			  aFileRow;	    // Zeile einer FileTabelle

	// Range-Variable (Alias-Name) fuer den Tabellennamen.
	String aTableRange;

	// Ergebnis einer COUNT(*)-Anfrage: Anzahl der gezaehlten Rows
	// Aber Achtung: die Anzahl der Zeilen in der Ergebnismenge (nRowCount)
	// ist bei dieser Anfrage = 1!
	INT32 nRowCountResult;
	INT32 nFetchPos,		// Cursorposition
		  nFilePos,			// Position im File
		  nLastVisitedPos;	// last known position in the file

	SdbColumnsRef aFileColumns;

	SdbFILETable* pTable;

	// Eigene Member dieser Implementation:

	// Art des Statements (SELECT, SELECT COUNT(*), INSERT, UPDATE, DELETE)
	// (nur gueltig, wenn Statement Prepared)
	SdbFILEStatementType eStatementType;

		// Benutzung des Index erzwingen (z. B. fuer FLAT File, bei denen
	// Direktpositionierung etc. sonst nicht moeglich ist)
	BOOL bForceUseIndex : 1;
	// sollen geloeschte Datensaetze gezeigt werden?
	// nur fuer dBase
	BOOL bShowDeleted : 1;
	
protected:
	SdbFILECursor(SdbFILEConnection* pConnection,
				  SdbCursorType eType,
				  UINT16 nOptions);

	virtual ~SdbFILECursor();

// cursor interface
protected:
	// Uebersetzung des Parse Tree in native SQL
	virtual void DriverTranslate();

	// Parameter binden
	virtual void BindParameter();

	// Beschreibung der Struktur der Parameter
	virtual void DescribeParameter();

	// Beschreibung der Struktur der Ergebnismenge
	virtual void DescribeResult();

	// Cursor auf einen Datensatz der Ergebnismenge
	// positionieren: auf den naechsten oder vorherigen
	// den ersten oder letzten, relativ oder absolut
	// (relativ und absolut mit Angabe des Offset).
	BOOL Move(SdbFileCursorPosition eCursorPosition, INT32 nOffset,
				  SdbCursorMode& rCursorMode, BOOL bRetrieveData);	
	virtual BOOL MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,
						  SdbCursorMode& rCursorMode) 
			{return Move((SdbFileCursorPosition)eCursorPosition, nOffset, rCursorMode, TRUE);}
	
	virtual BOOL MoveImpl(const ::com::sun::star::uno::Any& rBookmark, 
		SdbCursorMode& rMode, BOOL bRetrieveData)  throw( ::com::sun::star::lang::IllegalArgumentException );

	virtual BOOL PrepareImpl();
	virtual BOOL OpenImpl(SdbCursorType eType = SDB_CURRENT,
						  UINT16 eOpt = SDB_CURRENT_OPT);
	virtual BOOL CloseImpl();

	virtual BOOL InsertImpl();
	virtual BOOL UpdateImpl();
	virtual BOOL DeleteImpl();

	// dient nur fuer Treiber wie DBF zur Eingrenzung der erwarteten ResultMenge
	virtual INT32 MaxRowCount() const;
	virtual BOOL DetermineRowCountImpl(INT32&);
	virtual BOOL SetUpdatableImpl(BOOL bUpdate);

	// Nachbereiten des Opens
	virtual BOOL OnOpen();

	virtual BOOL IsCapable(SdbCursorCapability eCapability) const;

public:
	// Bricht die Ausfuehrung des Statements (waehrend Execute
	// oder waehrend Fetch-Operationen) ab.
	// Kann waehrend der Ausfuehrung eines Statements (aus dem
	// Busy Callback heraus) ausgefuehrt werden!
	virtual void CancelExecution();
	void ShowDeleted(BOOL bShow) {bShowDeleted = bShow;}

	const SdbFILESet* GetFileSet() const { return pFileSet; }

// file cursor interface
protected:
	BOOL IsSorted() const {return nOrderbyColumnNumber[0] != SDB_COLUMN_NOTFOUND;}
	BOOL HasRestriction() const {return aSQLAnalyzer.HasRestriction();}
	SdbFILEConnection* GetFILEConnection() const {return (SdbFILEConnection*)GetConnection();}

private:
	ODbRowRef Row(const String& /*aColumnName*/) const {return aEvaluateRow;}
	ODbRowRef Row(const String& /*aTableName*/, const String& /*aColumnName*/) const {return aEvaluateRow;}

	// Ausfuehren eines FETCH und/oder von Aktionen auf einer Row:
	void ExecuteRow(SdbFileCursorPosition eCursorPosition,
					INT32 nOffset = 1,
					BOOL bRebind = TRUE,
					BOOL bEvaluate = TRUE,
					BOOL bRetrieveData = TRUE);

	// Analyse des Parse Tree:
	// Den Tabellennamen aus dem Parse Tree holen:
	void GetTableName(String & rTableName, String & rTableRange);

	// Die gewuenschten Columns ermitteln:
	void GetColumnNames();

	// Selektionskriterien ermitteln
	void AnylizeSQL();

	// Bei UPDATE und INSERT: die neuen Werte (Zuweisungen) ermitteln:
	void GetAssignValues();


	// Setzen der Werte zu einer Column (fuer UPDATE und INSERT)
	void SetAssignValue(const String& aColumnName,
						const String& aValue,
						BOOL bSetNull = FALSE,
						UINT32 nParameter = SDB_NO_PARAMETER);

	// Parameter zur Liste der Parameter (pParameterColumns) hinzufuegen.
	// Liefert Column-Nr. (ab 1) fuer diesen Parameter.
	// pInfoColumn: die Column, der der Wert zugewiesen wird oder mit der
	// der Wert verglichen wird. Aus dieser Column werden Typ, Precision,
	// Scale etc. fuer den Parameter entnommen. Wenn NULL, dann wird einfach
	// VARCHAR(255) benutzt.
	UINT32 AddParameter(SdbSqlParseNode * pParameter,
						const SdbColumn * pInfoColumn = NULL);


	void SetOrderbyColumn(UINT16 nOrderbyColumnNumber,
						  SdbSqlParseNode * pColumnRef,
						  SdbSqlParseNode * pAscendingDescending);

	SdbFILEKeyValue * GetOrderbyKeyValue(ODbRowRef& Row);
	// F"ugt alle FileColumns in die Resultcolumns ein
	// nExprCnt wird f"ur doppelte Spaltennamen ben"otigt
	// nExprCnt muss einen g"ultigen Wert haben. Er kann innerhalb der Methode ver"andert werden.
	void InsertAllColumns(USHORT &nExprCnt);

	void ParseAssignValues(const String& aColumnList,SdbSqlParseNode* pRow_Value_Constructor_Elem,xub_StrLen nIndex);
	void Bind();
};

#endif



