/*************************************************************************
 *
 *  $RCSfile: fileset.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_FILESET_HXX
#define _SDB_FILESET_HXX 1

#include <tools.hxx>

#ifndef _TOOLS_TABLE_HXX
#include <tools/table.hxx>
#endif

#ifndef _SDB_FILECURS_HXX
#include "filecurs.hxx"
#endif

#include <stl_algo.h>

#define SDB_ORDERBYKEYS MAXORDER		// MUSS derzeit genau 10 sein! siehe filecurs.cxx

class SdbEvaluateSet;

typedef union
{
	double	aDouble;
	rtl::OUString* pString;
} SdbFILEKey;


DBG_NAMEEX(SdbFILEKeyValue);
class SdbFILEKeyValue
{
private:
	INT32 nValue;
	SdbFILEKey pKey[SDB_ORDERBYKEYS];

public:
	SdbFILEKeyValue() { DBG_CTOR(SdbFILEKeyValue,NULL);}
	SdbFILEKeyValue(INT32 nVal) : nValue(nVal) {DBG_CTOR(SdbFILEKeyValue,NULL); }
	~SdbFILEKeyValue();

	void SetKey(UINT16 i, double d) { pKey[i].aDouble = d; }
	void SetKey(UINT16 i, const rtl::OUString& rString) { pKey[i].pString = new rtl::OUString(rString); }
	void SetValue(INT32 nVal) { nValue = nVal; }

	rtl::OUString* GetKeyString(UINT16 i) const { return pKey[i].pString; }
	double GetKeyDouble(UINT16 i) const { return pKey[i].aDouble; }

	inline INT32 GetValue() const {return nValue;}
};

typedef SdbFILEKeyValue * SdbFILEKeyValuePtr;

typedef enum
{
	SDB_ORDERBYKEY_NONE,		// Nicht sortieren
	SDB_ORDERBYKEY_DOUBLE,		// Numerischer Key
	SDB_ORDERBYKEY_STRING		// String Key
} SdbKeyType;


typedef void* ROWKEY;
DBG_NAMEEX(SdbFILESet);

class SdbFILESet : public vector<INT32>
{
	BOOL bFrozen;
	INT32 nPos;

	SdbFILESet(const SdbFILESet& rhs) { DBG_CTOR(SdbFILESet,NULL);}
public:
	SdbFILESet(INT32 _nInitSize = 16,INT32 _nReSize = 16) : vector<INT32>()
			  ,bFrozen(FALSE)
			  ,nPos(0)
	{ DBG_CTOR(SdbFILESet,NULL); reserve(_nReSize);}

	SdbFILESet(const SdbEvaluateSet& rEvalSet);
	~SdbFILESet();
	inline void Insert(INT32 nKey)	;
	inline void Append(INT32 nKey)	;
	inline void Remove(INT32 nKey)	;

	inline INT32 Count() const					;
	inline void First()						;
	inline BOOL Next()							;
	inline void SeekPos(INT32 nNewPos)			;

	inline INT32 GetKey(INT32  nPos) const		;
	inline INT32 GetKey() const				;
	inline BOOL  IsKeyValid(INT32 nKey) const	;
	// besagt, da ein Resultset einmal vollstaendig gelesen wurde
	void  Freeze()	{bFrozen = TRUE;}
	BOOL  IsFrozen() const {return bFrozen;}
};

//------------------------------------------------------------------
inline void SdbFILESet::Insert(INT32 nKey)
{
	insert(begin(),nKey);
}
inline void SdbFILESet::Append(INT32 nKey)
{
	push_back(nKey);
}
inline void SdbFILESet::Remove(INT32 nKey)
{
	INT32 *pPo = find(begin(),end(),nKey);
	if (pPo)
		erase(pPo);
}

inline INT32 SdbFILESet::Count() const
{
	return size();
}
inline void SdbFILESet::First()
{
	nPos = 0;
}
inline BOOL SdbFILESet::Next()
{
	nPos++;
	return nPos <= size();
}
inline void SdbFILESet::SeekPos(INT32 nNewPos)
{
	nPos = nNewPos;
}

inline INT32 SdbFILESet::GetKey(INT32  nPo) const
{
	INT32 nP = (*this)[nPo];
	return	nP;
}
inline INT32 SdbFILESet::GetKey() const
{
	INT32 nP = (*this)[nPos];
	return	nP;
}
inline BOOL  SdbFILESet::IsKeyValid(INT32 nKey) const
{
	return find(begin(),end(),nKey) != NULL;
}
//-------------------------------------------------

class SdbEvaluateSet : private Table
{
public:
	Table::Count;
	Table::Remove;
	Table::IsKeyValid;

	SdbEvaluateSet(USHORT _nInitSize = 16,
				   USHORT _nReSize = 16 )
				  :Table( _nInitSize, _nReSize ){}

	void	Insert(INT32 nKey)			{Table::Insert(nKey, (void*)nKey);}
	INT32	GetKey(INT32 nPos) const	{return (INT32) GetObject(nPos);}
};

DECLARE_LIST(SdbEvaluateSetList, SdbEvaluateSet*);

class SdbFILESortIndex
{
private:
	INT32 nMaxCount;			// Maximal moegliche Anzahl Key/Value-Paare im Index (und damit Array-Groesse)
	INT32 nCount;				// Anzahl Key/Value-Paare im Index (und damit naechste freie Position)

	SdbFILEKeyValuePtr * ppKeyValueArray;
								// Zeiger auf Array der Groesse [nMaxCount]

	BOOL bFrozen;
	CharSet eCharSet;

public:	// nur fuer SdbFILECompare:
	static SdbFILESortIndex *pCurrentIndex;					// Waehrend der Ausfuehrung von qsort ist hier der Zeiger
	static CharSet eCurrentCharSet;
															// auf den gerade zur Sortierung verwendeten Index hinterlegt
															// (wird von der Vergleichsfunktion SdbFILEKeyCompare verwendet).
	SdbKeyType eKeyType[SDB_ORDERBYKEYS];
	BOOL bAscending[SDB_ORDERBYKEYS];


public:

	SdbFILESortIndex(const SdbKeyType eKeyType[],			// Art des Schluessels: numerisch/String/nicht sortieren (Genau 3 Eintraege!)
					 const BOOL bAscending[],				// TRUE = Aufsteigend sortieren (Genau 3 Eintraege!)
					 INT32 nMaxNumberOfRows,
					 CharSet eSet);

	~SdbFILESortIndex();


	BOOL AddKeyValue(SdbFILEKeyValue * pKeyValue);
														// TRUE, wenn erfolgreich hinzugefuegt, FALSE bei Ueberschreitung
														// der Index-Kapazitaet.
														// pKeyValue wird beim Zerstoeren des Index automatisch freigegeben.

	void Freeze();										// "Einfrieren" des Index:
														// Vor "Freeze" duerfen Count() und Get() nicht gerufen werden,
														// nach "Freeze" darf dafuer Add() nicht mehr gerufen werden.

	SdbFILESet* CreateKeySet();



	BOOL IsFrozen() { return bFrozen; }			// TRUE nach Aufruf von Freeze()

	INT32 Count() const { return nCount; }		// Anzahl Key/Value-Paare im Index
	INT32 GetValue(INT32 nPos) const;			// Value an Position nPos (1..n) [sortierter Zugriff].
};



// Vergleichsfunktion (fuer qsort):
// Vergleicht zwei SdbFILEKeyValue-Strukturen (liefert -1/0/1 wenn 1 < 2, 1 == 2, 1 > 2)
// Benutzt SdbFILEIndex::pCurrentIndex, um Aufbau des KeyValue (Art/Aufsteigend) zu ermitteln.
static int
#if defined(WIN) || defined(WNT)
__cdecl
#endif
#if defined(ICC) && defined(OS2)
_Optlink
#endif
SdbFILEKeyCompare(const void * elem1, const void * elem2);

#endif





