/*************************************************************************
 *
 *  $RCSfile: filetab.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_FILETAB_HXX
#define _SDB_FILETAB_HXX 1

#ifndef _VOS_MACROS_HXX
#include <vos/macros.hxx>
#endif

#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif

#ifndef _SDB_SDBTAB_HXX
#include "sdbtab.hxx"
#endif

#ifndef _SDB_FILECURS_HXX
#include "filecurs.hxx"
#endif

// Ausfhren von File aktionen
#define TRY_FILE(action,Type)\
	action;\
	if (aStatus.SetError(aFileStream,Type))\
	{\
		aFileStream.ResetError();\
		return FALSE;\
	}


typedef vos::OMutex SdbFILETableLock;

//==================================================================
// DBase Tabelle
//==================================================================
class SdbFILETable : public SdbTable
{
	friend class SdbFILEConnection;
	friend class SdbFILECursor;

protected:
	BOOL  bReadOnly;				// Flag mu im Open gesetzt werden
	INT32 nFilePos;					// aktuelle FilePosition
	SdbFILETableLock *m_pLock;		// Mutex zum Locken
	INT32 nCursorUse;				// Zaehler fuer verwendete Cursor

protected:
	SdbFILETable(SdbFILEConnection* pConnection);
	~SdbFILETable();

public:
	// Stellt fest, ob eine Tabelle beschreibbar ist oder nicht.
	BOOL IsReadOnly() const {return bReadOnly;}

	// Count fuer Cursor
	void Acquire() {nCursorUse++;}
	void Release() {nCursorUse--;}
	BOOL InUse() const {return nCursorUse != 0;}

	// Locking wird nicht innerhalb der Methode vorgenommen,
	// da z.B. Seek und Fetch meist paarig aufgerufen werden
	NAMESPACE_VOS(OGuard)* Lock();

	virtual BOOL  SeekRow(SdbFileCursorPosition eCursorPosition, INT32 nOffset, INT32& nCurPos) = 0;
	virtual BOOL  FetchRow(ODbRow& rRow, SdbColumns& rColumns, BOOL bRetrieveData) = 0;
	virtual BOOL  InsertRow(ODbRowRef& rRow, BOOL bFlush);
	virtual BOOL  UpdateRow(const ODbRowRef& rRow, ODbRow* pOrgRow);
	virtual BOOL  DeleteRow();
	
	// determine the position of the last row at the moment
	virtual INT32 GetCurrentLastPos() const {return -1;}

	SdbFILEConnection* GetFILEConnection() const {return (SdbFILEConnection*)GetConnection();}
protected:
	// ermitteln der Strukturbeschreibung
	virtual BOOL DescribeColumns() = 0;

	// erweiterte Strukturbeschreibung
	virtual void ExtDescribeColumns();

	// Festlegen des Tabellennamens
	virtual void SetNames(const String&, const String&);

	// FileOperationen
	virtual BOOL FileOpen(BOOL bReadOnly) = 0;
	virtual void FileClose() = 0;
	virtual BOOL IsFileOpen() const = 0;

	// Oeffnen einer Tabelle -> Einlesen der Strukturinformationen
	// anschliessend wird (kann) der Filezugriff wieder beendet,
	// damit die Daten nicht gelockt werden (filespezifisch)
	virtual BOOL OpenImpl();
	virtual void CloseImpl();
	virtual BOOL RenameImpl(const String& rNewName);
	virtual BOOL DropImpl();

	virtual SdbIndex* CreateIndexImpl();
	virtual void FillIndexNames(SvStringsDtor& _rList);
};

#endif

