/*************************************************************************
 *
 *  $RCSfile: flatconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_FLATCONN_HXX
#define _SDB_FLATCONN_HXX 1

#ifndef _SDB_FILECONN_HXX
#include "fileconn.hxx"
#endif

// Class SdbFLATConnection
//
// Beschreibung:
//	Implementation der SdbFLATConnection fuer FLAT
//
class SdbFLATConnection : public SdbFILEConnection
{
	friend class SdbEnvironment;
	friend class SdbFLATTable;

protected:
	// Neue Verbindungen werden nur mit SdbFLATConnection::Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbFLATConnection(SdbEnvironment* pEnv, const String& rOptions);
	~SdbFLATConnection();
	//-------------------------------------------------------------
	// von der Implementation zu realisieren:
	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).

	virtual SdbTable * CreateTableImpl();

	// Faehigkeiten des Treibers und des SQL-Dialektes:
	virtual BOOL IsCapableImpl(SdbCapability eCapability) const;
	virtual void FillTypeList(SdbTypeInfoList& rList);

private:
	//-------------------------------------------------------------
	// Eigene Members der Implementation

	// Optionen:
	BOOL bFixedLength;		// Datensaetze fester Laenge
	BOOL bHeaderLine; 		// Feldnamen in der ersten Zeile
	char cFieldDelimiter;	// Trennzeichen zwischen den Feldern
	char cStringDelimiter;	// Trennzeichen, mit dem Strings eingeschlossen sind
	char cDecimalDelimiter;	// Dezimal-Trennzeichen (Dezimalpunkt)
	char cThousandDelimiter;// Tausender-Trennzeichen

public:
	// Member dem Cursor zur Verfuegung stellen:
	BOOL IsFixedLength() { return bFixedLength;}
	BOOL IsHeaderLine() { return bHeaderLine; }
	char FieldDelimiter() { return cFieldDelimiter; };
	char StringDelimiter() { return cStringDelimiter; }
	char DecimalDelimiter() { return cDecimalDelimiter; }
	char ThousandDelimiter() { return cThousandDelimiter; }
};


#endif


