/*************************************************************************
 *
 *  $RCSfile: jdbccurs.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_JDBCCURS_HXX
#define _SDB_JDBCCURS_HXX


#ifndef _LINK_HXX //autogen
#include <tools/link.hxx>
#endif

#ifndef _SDB_SDBOBJ_HXX
#include "sdbobj.hxx"
#endif

#ifndef _SDB_SDBCOL_HXX
#include "sdbcol.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif


// Class SdbJDBCCursor
//
// Beschreibung:
//	Die Erzeugung eines SdbCursor-Objektes bereitet eine
//	SQL-Anweisung (vor allem SELECT, aber auch INSERT,
//	UPDATE, DELETE, und auch DDL-Anweisungen) fuer die Aus-
//	fuehrung vor (PREPARE). Anschliessend kann das Statement
//	mit Hilfe von 'Execute' beliebig oft ausgefuehrt werden.

//	Jeweils vor der Ausfuehrung eines Statements koennen
//	ggf. Parameter gebunden werden. Parameter sind im
//	Statement-String durch ein Fragezeichen gekennzeichnet.

//	Bei der Ausfuehrung einer Anfrage auf der Datenbank
//	entsteht eine (geordnete) Folge von Datensaetzen
//	(Rows), etwa vergleichbar einer Listenstruktur.
//	SdbStatement stellt eine Art "Fenster" auf diese Folge
//	von Datensaetzen zur Verfuegung; es macht jeweils eine
//	Zeile (Row) aus der Liste zur Zeit sichtbar
//	(Datenbank-CURSOR-Konzept).

//	Gruppen von Methoden:
//	a) Vorbereitung und Ausfuehrung des Statements:
//	Ein SdbCursor bereitet das bei der Erzeugung
//	angegebene Statement automatisch vor. Das unveraenderte
//	Statement kann spaeter erneut ausgefuehrt werden. Die
//	Struktur der Ergebnismenge wird bei der Erzeugung des
//	SdbStatements festgelegt und bleibt danach unveraendert.

//	b) Bindung einer Row fuer die Bereitstellung von
//	Parametern
//	SdbCursor stellt selbst eine Row (ohne
//	Strukturinformation und ohne gebundene Variable) zur
//	Verfuegung, die vom Aufrufer gefuellt werden kann, oder
//	es kann eine vom Aufrufer vorbereitete Row komplett
//	gebunden werden.

//	c) Bindung von Rows fuer die Aufnahme von Ergebnissen
//	der Anfrage
//	Wenn bei einer Fetch-Operation aktuell keine Row
//	gebunden ist, alloziert SdbCursor selbst eine.

//	d) Navigation in der Ergebnismenge:
//	Positionierung an Anfang und Ende der Liste, Vorwaerts-
//	und Rueckwaertsblaettern sowie direkte Positionierung
//	(auf n-te Row).

//	f) Sonstiges
//


#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif


class java_sql_ResultSetMetaData;
class java_sql_ResultSet;
class java_sql_Statement;
class java_sql_PreparedStatement;
class java_sql_Connection;
class java_sql_CallableStatement;

class SdbJDBCCursor : public SdbCursor
{
	friend class SdbJDBCConnection;
	friend class SdbJDBCTable;
	friend class SdbJDBCIndex;
	friend SdbDatabaseType javaTyp2SdbTyp(INT32 typ);

	// Manipulation einer Datenbanktabelle ohne SQL-Statements -- zur Zeit nur fr Files

public:
	// Pr"uft ob an der VM eine Exception aufgetreten ist.
	static void SetStatus(SdbStatus& aStatus,java_sql_Connection* _pConn);

protected:

	virtual BOOL PrepareImpl();
	virtual BOOL OpenImpl(SdbCursorType eType = SDB_CURRENT,
						  UINT16 eOpt = SDB_CURRENT_OPT);
	virtual BOOL CloseImpl();
	// JDBC kann kein natives Delete, Update oder Insert
	virtual BOOL DeleteImpl() {return FALSE;}
	virtual BOOL UpdateImpl() {return FALSE;}
	virtual BOOL InsertImpl() {return FALSE;}
	virtual BOOL MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,SdbCursorMode& rMode);
	virtual BOOL DetermineRowCountImpl(INT32& nNewCount);
	//virtual BOOL SetUpdatableImpl(BOOL bUpdate);
	virtual BOOL OnOpen();					// Initialisierung nach Oeffnen
											// Setzt den Zustand und positioniert auf ersten Satz
	// Beschreibung der Struktur der Parameter
	virtual void DescribeParameter();

	// Beschreibung der Struktur der Ergebnismenge
	virtual void DescribeResult();

	// Bricht die Ausfuehrung des Statements (waehrend Execute
	// oder waehrend Fetch-Operationen) ab.
	// Kann waehrend der Ausfuehrung eines Statements (aus dem
	// Busy Callback heraus bzw. aus einem anderen Thread) ausgefuehrt werden!
	// (wird von der public Methode Cancel() gerufen)
	virtual void CancelExecution();

protected:
	virtual ~SdbJDBCCursor();
	// Konstruktoren nur aus SdbConnection heraus benutzbar
	// (dort Factory CreateCursor).
	SdbJDBCCursor(SdbConnection* pConnection,SdbCursorType eType,UINT16 nOptions);

	// Gibt einen Cursor bereit, der auf die Ergebnisliste zeigt
	// Die Liste beinhaltet alle Tabellen auf die die Suchkriterien zutreffen
	// aList kann beinhalten TABLE;VIEW;ALIAS;SYNONYM;etc.
	// Es werden alle nicht bekannten Tabellentypen herausgefiltert
	// pQualifier == NULL => wird aus der Auswahl herausgenommen
	// pOwner == NULL => wird aus der Auswahl herausgenommen
	// pTable == aList == "%" => beinghaltet alle Tabellen
	// naeheres siehe java.sql.* HelpFile
	void OpenTables(const String* pQualifier, const String* pOwner,
					const String* pTable    , const String& aList);
	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Columns in einer Tabelle als Ergebnismenge liefert.
	void OpenColumns(const String* pQualifier, const String* pOwner,
					 const String* pTable, const String* pColumn);	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// eines Primaerschluessels einer Tabelle als Ergebnismenge liefert.
	void OpenPrimaryKeys(const String* pQualifier, const String* pOwner, const String* pTable);
	// Spezielle Methode anstelle von Execute, die Informationen ueber die Rechte
	// des Benutzers fuer eine gegebene Tabelle liefert
	void OpenTablePrivileges(const String* pQualifier, const String* pOwner, const String* pTable);

	void OpenIndices(const String* pQualifier, const String* pOwner, const String* pTable, BOOL bUnique = FALSE);

	void OpenForeignKeys(const String* pPKQualifier, const String* pPKOwner, const String* pPKName,
						 const String* pFKQualifier, const String* pFKOwner, const String* pFKName);

	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Prozeduren als Ergebnismenge liefert.
	void OpenProcedures(const String* rQualifier, const String* rSchema, const String* rProcName);

	void OpenVersionColumns(const String* rQualifier, const String* rSchema, const String* rTabName);

	virtual BOOL IsCapable(SdbCursorCapability eCapability) const;

#if SUPD>557
	virtual
#endif
		void RemovingFromParent();

private:
	// Schreibt die Daten aus pRs (Cursor) in die aResultRow; TRUE == OK
	BOOL FillResultRow();
	// Schreibt die Daten aus Callable Statement (Cursor) in die aParameterRow; TRUE == OK
	BOOL FillParameterRow();
	// Fuellt das Prepared Statement mit den Daten aus der aParameterRow
	void FillPreStatement();
	// Fuellt das Callable Statement mit den Daten aus der aParameterRow
	void FillCallStatement();
	// Beschreibt die Columns mittels der Metadaten pRsM und fuegt sie in cols ein
	void DescribeColumns(SdbColumnsRef &cols,INT32 len,java_sql_ResultSetMetaData *pRsM);
	BOOL IsSelectStatement();

	// member java
	java_sql_ResultSet *pRs;					// entspricht dem Cursor
	java_sql_Statement *pStatem;				// native SQL Statement
	java_sql_PreparedStatement *pPreStatem;		// Prepared SQL Statement
	java_sql_CallableStatement *pCallableStatem;// Callable SQL Statement
	java_sql_Connection *pConnect;
	BOOL bOwnCon;
	INT32 nRows;
	INT32 nMaxRowsOld;							// zu letzt gesetzte MaxRow
	INT32 nMaxFieldLenOld;							// zu letzt gesetzte MaxRow
};

#endif //_SDB_JDBCCURS_HXX




