/*************************************************************************
 *
 *  $RCSfile: odbc2conn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_ODBC2CONN_HXX
#define _SDB_ODBC2CONN_HXX 1

// odbcinc.hxx ganz am Anfang!
//#ifndef _SDB_ODBCINC_HXX
//#include "odbcinc.hxx"
//#endif

#ifndef _SDB_ODBC2STAT_HXX
#include "odbc2stat.hxx"
#endif

#ifndef _SDB_ODBCCONN_HXX
#include "odbcconn.hxx"
#endif

class SdbCursorRef;

// Class SdbODBCConnection
//
// Beschreibung:
//	Implementation der SdbODBCConnection fuer ODBC
//
class SdbODBC2Connection : public SdbODBCConnection
{
	friend class SdbEnvironment;
	friend class SdbODBC2Cursor;

protected:
	// Neue Verbindungen werden nur mit SdbODBCConnection::Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbODBC2Connection(SdbEnvironment* pEnv, const String& rOptions);
	SdbODBC2Connection(const SdbODBC2Connection& _rConn);

	// Verbindungen koennen nur mit der Create-Factory
	// *dynamisch* angelegt werden. Statische oder automatische
	// Instanzen sind nicht moeglich.
	// Zur Zerstoerung eines Connection-Objektes ist Delete zu
	// verwenden (welches selbst 'delete this' ausfuehrt).
	virtual ~SdbODBC2Connection();

	//-------------------------------------------------------------
	// von der Implementation zu realisieren:

protected:
	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).
	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType = SDB_KEYSET,UINT16 nOptions = SDB_READONLY);

	virtual void SetAutoCommit(BOOL bAuto);

	// COMMIT ausfuehren
	virtual void CommitImpl();

	// ROLLBACK ausfuehren
	virtual void RollbackImpl();

	// Verbindung aufbauen
	virtual BOOL OpenImpl();

	// Verbindung schliessen
	virtual BOOL CloseImpl();

	virtual SdbDatabaseType MapType(INT16 nODBCType);

	virtual BOOL IsCapableImpl(SdbCapability eCapability) const;
	virtual void SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel);
	virtual SdbTypeSearch GetSearchType(int nType);
	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl();
	void CheckODBCCapabilities();
	void Construct(SdbEnvironment* pEnv);
	// Die ODBC-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	void SetStatus(INT16 nRetcode, void* hdbc = NULL,const String& rErrorInfo = aEmptyString);

public:
	TYPEINFO();
	// Pruefung der Faehigkeiten (ODBC Level, verfuegbare Funktionen, ...)
	// des jeweiligen ODBC-Treibers. Setzt Status auf Fehler, wenn die
	// Faehigkeiten nicht ausreichen.
	virtual BOOL IsODBCFunctionAvailable(UWORD nFunction) const;
	virtual BOOL GetODBCInfo(UWORD nInfoType, String & rString) const;
	virtual BOOL GetODBCInfo(UWORD nInfoType, USHORT & rNumber) const;
	virtual BOOL GetODBCInfo(UWORD nInfoType, ULONG & rNumber) const;
	virtual BOOL GetODBCInfo(UWORD nInfoType, BOOL & rBool) const;

	static RETCODE OpenConnection(HDBC hDBC, const String& aConnectStr,UINT32 nTimeOut, BOOL bInternal = FALSE);
	static void FreeEnv(SdbEnvironment* pEnv);

	// ODBC Environment und Connection Handles
	// (Zugriff wird von SdbODBCCursor, SdbODBCTable, ... benoetigt)
	static HENV EnvironmentHandle(SdbEnvironment* pEnv,String &_rPath);
	virtual void* EnvironmentHandle();

	// Namen aller verfuegbaren Server:
	static String ServerNames(SdbStatus& rStat, SdbEnvironment* pEnv);
};


// Inline-Implementationen:

inline void SdbODBC2Connection::SetStatus(RETCODE nRetcode, void* hdbc,
										 const String & rErrorInfo)
{
	SdbODBC2_SetStatus(aStatus,nRetcode,m_aEnvironmentHandle,hdbc,SQL_NULL_HSTMT,rErrorInfo);
}

#endif


