/*************************************************************************
 *
 *  $RCSfile: odbc3conn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_ODBC3CONN_HXX
#define _SDB_ODBC3CONN_HXX 1

// odbcinc.hxx ganz am Anfang!
//#ifndef _SDB_ODBCINC_HXX
//#include "odbcinc.hxx"
//#endif

#ifndef _SDB_ODBC3STAT_HXX
#include "odbc3stat.hxx"
#endif

#ifndef _SDB_ODBCCONN_HXX
#include "odbcconn.hxx"
#endif
#if defined(WIN) || defined(WNT)
#include <tools/prewin.h>
#include <odbc/sql.h>
#include <tools/postwin.h>
#else
#include <odbc/sql.h>
#endif

class SdbCursorRef;

// Class SdbODBC3Connection
//
// Beschreibung:
//	Implementation der SdbODBC3Connection fuer ODBC
//
class SdbODBC3Connection : public SdbODBCConnection
{
	friend class SdbEnvironment;
	friend class SdbODBC3Cursor;

protected:
	// new member
	// forwardonly cursor
	SQLUINTEGER m_nForwardOnlyAttr1;
	SQLUINTEGER m_nForwardOnlyAttr2;

	// keysetdrivenr cursor
	SQLUINTEGER m_nKeySetAttr1;
	SQLUINTEGER m_nKeySetAttr2;

	// static cursor
	SQLUINTEGER m_nStaticAttr1;
	SQLUINTEGER m_nStaticAttr2;
	SQLHANDLE	m_pODBC2Env;
	BOOL		m_bUseOldTimeDate;
	BOOL		m_bIsValid;



	// Neue Verbindungen werden nur mit SdbODBC3Connection::Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbODBC3Connection(SdbEnvironment* pEnv, const String& rOptions);
	SdbODBC3Connection(const SdbODBC3Connection& _rConn);

	// Verbindungen koennen nur mit der Create-Factory
	// *dynamisch* angelegt werden. Statische oder automatische
	// Instanzen sind nicht moeglich.
	// Zur Zerstoerung eines Connection-Objektes ist Delete zu
	// verwenden (welches selbst 'delete this' ausfuehrt).
	virtual ~SdbODBC3Connection();

	//-------------------------------------------------------------
	// von der Implementation zu realisieren:

protected:
	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).
	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType = SDB_KEYSET,UINT16 nOptions = SDB_READONLY);

	virtual void SetAutoCommit(BOOL bAuto);

	// COMMIT ausfuehren
	virtual void CommitImpl();

	// ROLLBACK ausfuehren
	virtual void RollbackImpl();

	// Verbindung aufbauen
	virtual BOOL OpenImpl();

	// Verbindung schliessen
	virtual BOOL CloseImpl();

	virtual BOOL IsCapableImpl(SdbCapability eCapability) const;
	virtual void SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel);
	virtual SdbTypeSearch GetSearchType(int nType);

	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl();
	virtual SdbDatabaseType MapType(INT16 nODBCType);

	// Die ODBC-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	void SetStatus(INT16 nSQLRETURN, void* hdbc = NULL,const String& rErrorInfo = aEmptyString);
	void CheckODBCCapabilities();
	void Construct(SdbEnvironment* pEnv);
	BOOL IsValid() const{ return m_bIsValid; }
public:
	TYPEINFO();
	// Pruefung der Faehigkeiten (ODBC Level, verfuegbare Funktionen, ...)
	// des jeweiligen ODBC-Treibers. Setzt Status auf Fehler, wenn die
	// Faehigkeiten nicht ausreichen.
	BOOL IsODBCFunctionAvailable(UWORD nFunction) const;
	BOOL GetODBCInfo(UWORD nInfoType, String & rString) const;
	BOOL GetODBCInfo(UWORD nInfoType, USHORT & rNumber) const;
	BOOL GetODBCInfo(UWORD nInfoType, ULONG & rNumber) const;
	BOOL GetODBCInfo(UWORD nInfoType, BOOL & rBool) const;

	static SQLRETURN OpenConnection(SQLHDBC hDBC, const String& aConnectStr,UINT32 nTimeOut, BOOL bInternal = FALSE);
	static void FreeEnv(SdbEnvironment* pEnv);
	// ODBC Environment und Connection Handles
	// (Zugriff wird von SdbODBC3Cursor, SdbODBC3Table, ... benoetigt)
	static SQLHENV EnvironmentHandle(SdbEnvironment* pEnv,String &_rPath);
	virtual void* EnvironmentHandle();

	// Namen aller verfuegbaren Server:
	static String ServerNames(SdbStatus& rStat, SdbEnvironment* pEnv);
};


// Inline-Implementationen:

inline void SdbODBC3Connection::SetStatus(INT16 nSQLRETURN, void* hdbc,const String & rErrorInfo)
{
	if(hdbc == m_aConnectionHandle)
		SdbODBC3_SetStatus(aStatus,nSQLRETURN,SQL_HANDLE_DBC,hdbc,rErrorInfo);
	else
		SdbODBC3_SetStatus(aStatus,nSQLRETURN,SQL_HANDLE_ENV,hdbc,rErrorInfo);
}

#endif


