/*************************************************************************
 *
 *  $RCSfile: odbcconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_ODBCCONN_HXX
#define _SDB_ODBCCONN_HXX 1

#include "sqldefs.hxx"

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

class SdbCursorRef;

// Kann laenger dauern, daher mit Aufruf des Busy Callback
#define ODBC_ASYNC(aCommand)							\
	while ((nRetcode = aCommand) == SQL_STILL_EXECUTING)   \
		BUSY_DB();

// Class SdbODBCConnection
//
// Beschreibung:
//	Implementation der SdbODBCConnection fuer ODBC
//
class SdbODBCConnection : public SdbConnection
{
	friend class SdbEnvironment;
	friend class SdbODBCCursor;
	friend class SdbODBC2Cursor;
	friend class SdbODBC3Cursor;
	friend class SdbODBCTable;
	friend class SdbODBCRelation;

protected:
	// Neue Verbindungen werden nur mit SdbODBCConnection::Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbODBCConnection(SdbEnvironment* pEnv, const String& rOptions);
	SdbODBCConnection(const SdbODBCConnection& _rConn);

	// Verbindungen koennen nur mit der Create-Factory
	// *dynamisch* angelegt werden. Statische oder automatische
	// Instanzen sind nicht moeglich.
	// Zur Zerstoerung eines Connection-Objektes ist Delete zu
	// verwenden (welches selbst 'delete this' ausfuehrt).
	virtual ~SdbODBCConnection();

	//-------------------------------------------------------------
	// von der Implementation zu realisieren:

protected:
	virtual void RemoveChild(SdbObj* _pChild);
	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).
	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType = SDB_KEYSET,UINT16 nOptions = SDB_READONLY)=0;

	virtual SdbTable * CreateTableImpl();
	virtual SdbRelation* CreateRelationImpl(const String& _rName);

	// Verbindung aufbauen
	virtual BOOL OpenImpl();


	virtual void FillTypeList(SdbTypeInfoList& rList);
	virtual void FillTableNames(SdbStringStlMap* _pList);
	virtual void FillAllRelations(SdbRelationList& _rList);
	virtual void FillRelations(SdbRelationList& _rList, const String& rName) {};

	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl()=0;

	virtual SdbTypeSearch GetSearchType(int nType)=0;
	//-------------------------------------------------------------
	// Eigene Members der Implementation

	// Connection Handle (Environment siehe auch sdbenv.hxx)
	void* m_aEnvironmentHandle;			// einmal global
	void* m_aConnectionHandle;				// einmal pro Connection
	ULONG nDriverConcurrency;			// Welche Art von Concurrency unterstuetzt der Treiber
	ULONG nFetchDirection;				// welche Navigationsmoeglichkeiten?
	ULONG nQualifierUsage;
	ULONG nOwnerUsage;
	ULONG nStaticSensitivity;			//
	ULONG nPosOperations;				//
	USHORT nCommitBehavior;				// wie verhaelt sich ein Cursor nach einem Commit
										// wenn der Treiber nach einem Commit alle
										// Cursor schliesst, kann dies zu Problemen
										// fuehren
	USHORT nMaxStmts,					// Maximale Anzahl Statements, 0 -> keine Grenze
		   nActiveStmts,				// gerade verwendete Stmt
		   nODBCVer;					// Version des DriverManagers

	BOOL   bReadOnly : 1,				// Treiber ReadOnly ?
		   bNoPrompt : 1;				// Prompting beim Open
	BOOL   bNeedLen;
	BOOL   m_bUserWChar;

	// Status der letzten Operation abfragen und aStatus entsprechend setzen.
	// Wenn sich die Operation auf eine Connection bezieht, muss deren Handle
	// mit angegeben werden.

	// Die ODBC-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	virtual void SetStatus(INT16 nRetcode, void* hdbc = NULL,const String& rErrorInfo = aEmptyString)=0;
	virtual void CheckODBCCapabilities() = 0;
	virtual void Construct(SdbEnvironment* pEnv)=0;

public:
	TYPEINFO();
	// Pruefung der Faehigkeiten (ODBC Level, verfuegbare Funktionen, ...)
	// des jeweiligen ODBC-Treibers. Setzt Status auf Fehler, wenn die
	// Faehigkeiten nicht ausreichen.
	BOOL IsDriverCapable(UWORD nInfoType, ULONG nInfoFlag) const;
	virtual BOOL IsODBCFunctionAvailable(UWORD nFunction) const = 0;
	virtual BOOL GetODBCInfo(UWORD nInfoType, String & rString) const = 0;
	virtual BOOL GetODBCInfo(UWORD nInfoType, USHORT & rNumber) const = 0;
	virtual BOOL GetODBCInfo(UWORD nInfoType, ULONG & rNumber) const = 0;
	virtual BOOL GetODBCInfo(UWORD nInfoType, BOOL & rBool) const = 0;

	USHORT	GetCommitBehavior() const {return nCommitBehavior;}
	USHORT	GetODBCVersion()	const {return nODBCVer;}
	ULONG	GetQualifierUsage() const {return nQualifierUsage; }
	ULONG	GetOwnerUsage()		const {return nOwnerUsage; }
	BOOL	HasFreeStatement()	const {return nMaxStmts == 0 || nActiveStmts < nMaxStmts;}
	String	ConnectStr()		const;

	virtual SdbDatabaseType MapType(INT16 nODBCType)=0;
	virtual void* EnvironmentHandle()=0;

	void* ConnectionHandle() const {return m_aConnectionHandle;}
private:

	void FillRelation(SdbRelationList& _rList, const String& rName, SdbCursorRef& rxCursor);
	BOOL FillRelation(SdbODBCRelation& rRel);
	BOOL OpenRelationCursor(const String& rCatalog, const String& rSchema, const String& rName, SdbCursorRef& rxCursor);

public:
	BOOL NeedLongDataLen(){ return bNeedLen;};
};


// Inline-Implementationen:


#endif


