/*************************************************************************
 *
 *  $RCSfile: odbccurs.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_ODBCCURS_HXX
#define _SDB_ODBCCURS_HXX 1
#include "odbc3inc.hxx"
#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

// Class SdbODBCCursor
//
//
class SdbODBCCursor : public SdbCursor

{
	friend class SdbODBCConnection;
	friend class SdbODBC2Connection;
	friend class SdbODBC3Connection;
	friend class SdbODBCTable;
	friend class SdbODBCIndex;
protected:
	union
	{
		sal_Unicode aCharArray[2048];			// Temporaerer Speicher fuer: String und BLOB-Daten
		sal_Unicode waCharArray[2048];			// Temporaerer Speicher fuer: String und BLOB-Daten
		INT8  nINT8;					// ... INT8
		INT16 nINT16;					// ... INT16
		INT32 nINT32;                	// ... INT32
		float nFloat;                	// ... float
		double nDouble;              	// ... double
		DATE_STRUCT	aDate;
		TIME_STRUCT aTime;
		TIMESTAMP_STRUCT aTimestamp;	// ... OTER-Datums-/Zeitstruktur
	}	aBuffer;


	

public:
	BOOL Open(const String& rSQL, BOOL bTable = FALSE, SdbCursorType eType = SDB_CURRENT, UINT16 eOpt = SDB_CURRENT_OPT);

protected:
	SdbODBCCursor(SdbODBCConnection* pConnection,SdbCursorType eType,UINT16 nOptions);

	virtual ~SdbODBCCursor();
	// Beschreibung der Struktur der Parameter
	virtual void DescribeParameter()=0;

	// Beschreibung der Struktur der Ergebnismenge
	virtual void DescribeResult()=0;

	// Vorbereitung der SQL-Anfrage fuer die Ausfuehrung
	// (wird in der Regel automatisch durch den Konstruktor
	// aufgerufen [ausser bei ExecuteDirect])
	virtual BOOL PrepareImpl();

	// Ausfuehrung der SQL-Anfrage (nach erfolgter Bindung
	// aller benoetigten Parameter).
	virtual BOOL OpenImpl(SdbCursorType eType = SDB_CURRENT,UINT16 eOpt = SDB_CURRENT_OPT);
	virtual BOOL OnOpen();

	virtual BOOL InsertImpl()=0;
	virtual BOOL UpdateImpl()=0;
	virtual BOOL DeleteImpl()=0;

	virtual BOOL DetermineRowCountImpl(INT32&);

	// muss aus einem anderen Thread heraus aufgerufen werden, bricht aktuelle Statement-Ausfuehrung ab, wenn eine aktiv
	virtual void CancelExecution();

	// Cursor schliessen. Durch Aufruf von Execute kann das
	// Statement (mit gleichen oder verschiedenen Parametern)
	// erneut ausgefuehrt werden.

	// Falls noch nicht alle Rows der Ergebnismenge abgeholt
	// wurden, werden diese ignoriert.

	// Nach Close ist die Strukturbeschreibung nicht mehr zugreifbar.
	// Gebundene Rows werden durch Close zerstoert. Wenn eine gebundene
	// Row nicht zerstoert werden soll, so muss die Bindung vor dem Close
	// mit Release... aufgehoben werden.
	virtual BOOL CloseImpl();


	// Cursor auf einen Datensatz der Ergebnismenge
	// positionieren: auf den naechsten oder vorherigen
	// den ersten oder letzten, relativ oder absolut
	// (relativ und absolut mit Angabe des Offset).
	virtual BOOL MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset, SdbCursorMode& rMode)=0;	
	virtual void ImplChangeOpenOptions(SdbCursorType eType, UINT16 eOpt)=0;

	void*	ConnectionHandle() const {return m_aConnectionHandle;}
	void*	StmtHandle() const {return m_aStatementHandle;}

	static DateTime ToDateTime(const TIMESTAMP_STRUCT& rTS);
	static Time ToTime(const TIME_STRUCT& rTS);
	static Date ToDate(const DATE_STRUCT& rTS);
	static void ToDate(DATE_STRUCT& rResult, double fSbxDate);
	static void ToTime(TIME_STRUCT& rResult, double fSbxDate);
	static void ToTimeStamp(TIMESTAMP_STRUCT& rResult, double fSbxDate);

private:
	//-------------------------------------------------------------
	// Eigene Member der Implementation

protected:
	void  **pParamBuffer, **pFieldBuffer;// Buffer fr Parameter, Feldbinding
	INT32  *pParamLength, *pFieldLength,
			nCursorType, nConcurrency;
	UINT32	nCurrentMaxFieldLen, nCurrentMaxRows;

	// Environment und Connection Handle
	void*	m_aConnectionHandle;				// in manchen Faellen muss eine extra Conn angelegt werden
	void*	m_aStatementHandle;				// ODBC Statement Handle
	UINT16  nRowStatus;					// aktueller SatzStatus
	BOOL	bBookmarks	  : 1;				// werden Bookmarks verwendet
	// Status der letzten Operation abfragen und aStatus entsprechend setzen.
	// Wenn sich die Operation auf eine Connection bezieht, muss deren Handle
	// mit angegeben werden.

	// Die ODBC-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	virtual void SetStatus(INT16 nRetcode, void* hstmt = NULL,const String& rErrorInfo = aEmptyString)=0;

	// Parameter binden
	void BindParameter();
	virtual void AllocBuffer(void**& pDataBuffer, INT32*& pLenBuffer, const SdbColumns& rCols, BOOL bBindBookmark) = 0; 
	virtual void FreeBuffer(void**& pDataBuffer, INT32*& pLenBuffer, const SdbColumns& rCols, BOOL bBindBookmark) = 0;
	virtual BOOL BindColumn(UINT16 nPos, void** pDataBuffer, INT32* pLenBuffer,
					const SdbColumn& rCol, const ODbVariantRef& pVariable, BOOL bParam = TRUE)=0;

	// DatenHolen - Schreiben
	BOOL GetData(const SdbColumn&, const ODbVariantRef&, UINT16 nPos);
	BOOL PutData(const SdbColumn&, const ODbVariantRef&);

	void FixResultRow();

	virtual void DetermineOptions(SdbODBCConnection* pConnection)=0;
	virtual void SetOptions(SdbODBCConnection* pConnection)=0;

	// Updaten	
	BOOL ExecutePosUpdate(UINT16 nOption);
	void ClearAfterUpdate();

	virtual void PrepareODBC()=0;
	virtual int GetDataImpl(UWORD icol , SWORD fCType , PTR rgbValue , SDWORD cbValueMax , SDWORD FAR * pcbValue)=0;
	virtual int PutDataImpl(PTR rgbValue , SDWORD cbValue)=0;
	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Tabellen, Views, ... als Ergebnismenge liefert.
	virtual void OpenTables(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,const sal_Unicode* pTable, const sal_Unicode* pType)=0;

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Rechte
	// des Benutzers fuer eine gegebene Tabelle liefert
	virtual void OpenTablePrivileges(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)=0;

	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Columns in einer Tabelle als Ergebnismenge liefert.
	virtual void OpenColumns(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,const sal_Unicode* pTable, const sal_Unicode* pColumn)=0;

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// eines Primaerschluessels einer Tabelle als Ergebnismenge liefert.
	virtual void OpenPrimaryKeys(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)=0;

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// einer Fremdschluesselbeziehung zwischen zwei Tabellen als Ergebnismenge liefert.
	virtual void OpenForeignKeys(const sal_Unicode* pPkQualifier, const sal_Unicode* pPkOwner, const sal_Unicode* pPKName,
						 const sal_Unicode* pFkQualifier, const sal_Unicode* pFkOwner, const sal_Unicode* _rFKName)=0;

	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Prozeduren als Ergebnismenge liefert.
	virtual void OpenProcedures(const String& rQualifier, const String& rSchema, const String& rProcName)=0;

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// einer Prozedur als Ergebnismenge liefert.
	virtual void OpenProcedureColumns(const String& rQualifier, const String& rSchema,
							  const String& rProcName, const String& rProcColumnName)=0;

	virtual void OpenIndices(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable, BOOL bUnique = FALSE)=0;

	// Informationen ueber alle verfuegbaren DatenTypen erhalten
	virtual void OpenTypeInfo(INT16 nType)=0;

	virtual void OpenSpecialColumns(const String& rQualifier, const String& rSchema, const String& rTabName)=0;
	virtual void RowCountImpl()=0;	
	virtual BOOL GetDateValues(UINT16 nPos,DATE_STRUCT &rValue) = 0;
	virtual BOOL GetTimeValues(UINT16 nPos,TIME_STRUCT &rValue) = 0;
	virtual BOOL GetTimeDateValues(UINT16 nPos,TIMESTAMP_STRUCT &rValue) = 0;

	SdbODBCConnection* GetODBCConnection() const {return (SdbODBCConnection*) GetConnection();}
};


#define DESC_COL_NAME_LEN 128
extern const String aTruncState;
extern const String aValueChangedState;

#endif								 


