/*************************************************************************
 *
 *  $RCSfile: olevariant.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef SDB_OLEVARIANT_HXX
#define SDB_OLEVARIANT_HXX

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>			  
#endif

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _OSL_TYPES_H_
#include <osl/types.h>
#endif


class OLEVariant	:	public ::tagVARIANT
{
public:
	OLEVariant()					{	VariantInit(this);}
	OLEVariant(const VARIANT& varSrc) 
	{
		::VariantInit(this);
		::VariantCopy(this, const_cast<VARIANT*>(&varSrc));
	}
	OLEVariant(const OLEVariant& varSrc)	{	VariantInit(this); 
			VariantCopy(this, const_cast<VARIANT*>(static_cast<const VARIANT*>(&varSrc)));}
	OLEVariant(sal_Int32 n)				{	VariantInit(this);	vt = VT_I4;		lVal		= n;}
	OLEVariant(const rtl::OUString& us)	{	VariantInit(this);	vt = VT_BSTR;	bstrVal		= SysAllocString(us);}
	~OLEVariant()					{	VariantClear(this); } // clears all the memory that was allocated before
	
	OLEVariant& operator=(const OLEVariant& varSrc)
	{
		VariantCopy(this, const_cast<VARIANT*>(static_cast<const VARIANT*>(&varSrc)));
		return *this;
	}
	// Assign a const VARIANT& (::VariantCopy handles everything)
	//
	inline OLEVariant& operator=(const tagVARIANT& varSrc)
	{
		::VariantCopy(this, const_cast<VARIANT*>(&varSrc));

		return *this;
	}

	// Assign a const VARIANT* (::VariantCopy handles everything)
	//
	inline OLEVariant& operator=(const VARIANT* pSrc) 
	{
		::VariantCopy(this, const_cast<VARIANT*>(pSrc));

		return *this;
	}

	void SetByte(sal_uInt8 n)				{	VariantClear(this); vt = VT_UI1;	bVal		= n;}
	void SetInt16(sal_Int16 n)				{	VariantClear(this); vt = VT_I2;		iVal		= n;}
	void SetInt32(sal_Int32 n)				{	VariantClear(this); vt = VT_I4;		lVal		= n;}
	void SetFloat(float f)					{	VariantClear(this); vt = VT_R4;		fltVal		= f;}
	void SetDouble(double d)				{	VariantClear(this); vt = VT_R8;		dblVal		= d;}
	void SetDate(DATE d)					{	VariantClear(this); vt = VT_DATE;	date		= d;}
	void SetChar(unsigned char a)			{	VariantClear(this); vt = VT_UI1;	bVal		= a;}
	void SetCurrency(double aCur)			{	VariantClear(this); vt = VT_CY;		Set(aCur*10000);}
	void SetBool(sal_Bool b)				{	VariantClear(this); vt = VT_BOOL;	boolVal		= b ? -1:0;}
	void SetString(const String& sStr)		{	VariantClear(this); vt = VT_BSTR;	bstrVal		= SysAllocString(sStr.GetBuffer());}
	void SetUString(const rtl::OUString& us){	VariantClear(this);	vt = VT_BSTR;	bstrVal		= SysAllocString(us);}
	void SetNoArg()							{	VariantClear(this); vt = VT_ERROR;	scode		= DISP_E_PARAMNOTFOUND;}
	
	void SetIDispatch(IDispatch* pDispInterface)
									{	VariantClear(this); vt = VT_DISPATCH; pdispVal = pDispInterface;}

	void SetNull()					{	VariantClear(this); vt = VT_NULL;}
	void SetEmpty()					{	VariantClear(this); vt = VT_EMPTY;}
	
	void SetUI1SAFEARRAYPtr(SAFEARRAY* pSafeAr)
									{	VariantClear(this); vt = VT_ARRAY|VT_UI1; parray = pSafeAr;	}

	inline void SetArray(SAFEARRAY* pSafeArray, VARTYPE vtType)
									{   VariantClear(this);	vt = VT_ARRAY|vtType; parray = pSafeArray; }

	sal_Bool IsNull() const  {	return (vt == VT_NULL);		}
	sal_Bool IsEmpty() const {	return (vt == VT_EMPTY);	}

	String GetString()
	{
		if (vt != VT_BSTR) VariantChangeType(this, this, NULL, VT_BSTR);
		return bstrVal;
	}
	
	operator rtl::OUString()
	{
		if (vt != VT_BSTR) VariantChangeType(this, this, NULL, VT_BSTR);
		return rtl::OUString((sal_Unicode*)bstrVal);
	}

	rtl::OUString GetUString()
	{
		return (rtl::OUString)*this;
	}
	operator String()	{	return GetString();	}
	operator sal_Bool()		{	return GetBool();	}
	operator sal_Int32()	{	return GetInt32();	}
	operator sal_Int16()	{	return GetInt16();	}
	operator float()	{	return GetFloat();	}
	operator double()	{	return GetDouble();	}
	sal_Bool GetBool()
	{			 
		if (vt != VT_BOOL) VariantChangeType(this, this, NULL, VT_BOOL);
		return (sal_Bool) boolVal;
	};

	IUnknown* GetIUnknown()
	{
		if (vt != VT_UNKNOWN) VariantChangeType(this, this, NULL, VT_UNKNOWN);
		return (IUnknown*) punkVal;

	}

	IDispatch* GetIDispatch()
	{
		if (vt != VT_DISPATCH) VariantChangeType(this,this, NULL, VT_DISPATCH);
		return (IDispatch*) pdispVal;

	}


	sal_uInt8 GetByte()
	{
		if (vt != VT_UI1) VariantChangeType(this, this, NULL, VT_UI1);
		return (sal_uInt8) bVal;
	}

	sal_Int16 GetInt16()
	{
		if (vt != VT_I2) VariantChangeType(this, this, NULL, VT_I2);
		return (sal_Int16) iVal;
	}

	sal_Int32 GetInt32()
	{
		if (vt != VT_I4) VariantChangeType(this, this, NULL, VT_I4);
		return (sal_Int32) lVal;
	}

	sal_Int32 GetUInt32()
	{
		if (vt != VT_UI4) VariantChangeType(this, this, NULL, VT_UI4);
		return (sal_uInt32) lVal;
	}

	float GetFloat()
	{
		if (vt != VT_R4) VariantChangeType(this, this, NULL, VT_R4);
		return (float) fltVal;
	}

	double GetDouble()
	{
		if (vt != VT_R8) VariantChangeType(this, this, NULL, VT_R8);
		return (double) dblVal;
	}

	double GetDate()
	{
		if (vt != VT_DATE) VariantChangeType(this, this, NULL, VT_DATE);
		return (double) date;
	}

	double GetCurrency()
	{
		if (vt != VT_CY) VariantChangeType(this, this, NULL, VT_CY);
		double toRet = ((double)cyVal.Hi*(double)4294967296.0 + (double)cyVal.Lo) / 10000;
		return toRet;
	}

	SAFEARRAY* GetUI1SAFEARRAYPtr()
	{
		if (vt != (VT_ARRAY|VT_UI1))
			VariantChangeType(this,this, NULL, VT_ARRAY|VT_UI1);
		return (parray);
	}

	// static methods

	// ACHTUNG! Der zurueckgegebene UnicodeString mu vom
	// Rufer mit delete[] freigegeben werden!!!!
	inline static OLECHAR* UniCodeFromC(const char* pText)
	{

		int nLength = strlen(pText)+1;
		OLECHAR* aReturnString = new OLECHAR[nLength];

		MultiByteToWideChar(CP_ACP, //ANSI Code Page
						MB_PRECOMPOSED,
						pText,
						nLength,
						(OLECHAR*) aReturnString,
						nLength);

		return aReturnString;

	}

	// Der zurueckgegebene BSTR muss noch mit
	// SysFreeString() wieder freigegeben werden!
//	inline static BSTR BSTRFromC(const char* pText)
//	{
//		OLECHAR* pUniString = UniCodeFromC(pText);
//		BSTR aBSTR = SysAllocString(pUniString);
//		delete[] pUniString;
//		return aBSTR;
//	}

	inline static VARIANT_BOOL VariantBool(sal_Bool bEinBoolean)
	{ 
		return (VARIANT_BOOL) (bEinBoolean? -1:0);
	}

//	static String StringFromBSTR(BSTR& aBSTR)
//	{
//		sal_uInt16 nLength = SysStringLen(aBSTR);
//		String sRetString;
//		char* pBuf = sRetString.AllocStrBuf(nLength);
//
//		WideCharToMultiByte(CP_ACP, //ANSI Code Page
//						WC_COMPOSITECHECK,
//						aBSTR,
//						nLength,
//						pBuf,
//						nLength,
//						NULL, NULL);
//
//		return sRetString;
//	}
private:
	void CHS()
	{
		cyVal.Lo  ^= (sal_uInt32)-1;
		cyVal.Hi ^= -1;
		cyVal.Lo++;
		if( !cyVal.Lo )
			cyVal.Hi++;
	}

	void Set(double n)
	{
		if( n >= 0 )
		{
			cyVal.Hi = (sal_Int32)(n / (double)4294967296.0);
			cyVal.Lo  = (sal_uInt32)(n - ((double)cyVal.Hi * (double)4294967296.0));
		}
		else {
			cyVal.Hi = (sal_Int32)(-n / (double)4294967296.0);
			cyVal.Lo  = (sal_uInt32)(-n - ((double)cyVal.Hi * (double)4294967296.0));
			CHS();
		}
	}

};

#endif
