/*************************************************************************
 *
 *  $RCSfile: olewrap.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef SDB_WPDAOTMP_HXX
#define SDB_WPDAOTMP_HXX

#ifndef _TOOLS_DEBUG_HXX //autogen wg. DBG_NAMEEX
#include <tools/debug.hxx>
#endif

//////////////////////////////////////////////////////////////////////////
//
// Template-Klasse WpOLEBase<class T>
// ==================================
//
// Objekte dieser Klasse hlt einen Zeiger auf ein Interface vom Typ T.
// Es gibt Konstruktoren und Zuweisungsoperator die sicherstellen, dass
// AddRef() und Release() entsprechend den COM-Konventionen gerufen werden.
// Ein Objekt kann auch keinen Zeiger halten (Nullzeiger), dann ergibt
// der Aufruf von IsValid() FALSE.
//
// Um effizientes pass-by-value machen zu koennen, ist diese (ebenso wie die
// abgeleiteten Klassen) eine ganz schmale Wrapper-Klasse unter Vermeidung
// virtueller Methoden und mit Inlining.

DBG_NAMEEX(WpOLEBase);
DBG_NAMEEX(WpOLECollection);
//------------------------------------------------------------------------
template<class T> class WpOLEBase
{

protected:
	T* pInterface;

public:

	WpOLEBase();
	WpOLEBase(T* pInt);


	//inline
	WpOLEBase<T>& operator=(const WpOLEBase<T>& rhs)
	{
		if (rhs.pInterface != pInterface)
		{
			if (pInterface)	pInterface->Release();
			pInterface = rhs.pInterface;
			if (pInterface) pInterface->AddRef();
		}
		return *this;
	};

	WpOLEBase<T>& operator=(T* rhs)
	{
		if (pInterface != rhs)
		{
			if (pInterface)	pInterface->Release();
			pInterface = rhs;
			if (pInterface) pInterface->AddRef();
		}
		return *this;
	};

	WpOLEBase(const WpOLEBase<T>& aWrapper)
	{
		DBG_CTOR(WpOLEBase,NULL);
		operator=(aWrapper);
	};

	virtual ~WpOLEBase()
	{
		if (pInterface)
		{
			pInterface->Release();
			pInterface = NULL;
		};
		DBG_DTOR(WpOLEBase,NULL);
	};

	inline void clear()
	{
		if (pInterface)
		{
			pInterface->Release();
			pInterface = NULL;
		};
	};


	inline BOOL IsValid() const {return pInterface != NULL;};
	operator IDispatch*() { return pInterface; }
};


template<class T> WpOLEBase<T>::WpOLEBase<T>()
{
	DBG_CTOR(WpOLEBase,NULL);
	pInterface = NULL;
};


template<class T> WpOLEBase<T>::WpOLEBase<T>(T* pInt)
{
	DBG_CTOR(WpOLEBase,NULL);
	pInterface = pInt;
};
// kein AddRef(), da
// der nach Konvention von der erz. Methode
// schon durchgefhrt wurde.





//////////////////////////////////////////////////////////////////////////
//
// Template-Klasse WpOLECollection<class Ts, class T, class WrapT>
// ===============================================================
//
// Diese Klasse, welche sich von WpOLEBase<Ts> ableitet, abstrahiert die
// den DAO-Collections gemeinsamen Eigenschaften:
//
// Sie werden ber ein Interface Ts (etwa: DAOFields) angesprochen
// und koennen ueber get_Item (hier:GetItem) Items des Typs T (genauer:
// mit Interface T, etwa DAOField) herausgeben.
//
// Diese Wrapperklasse gibt aber nicht ein Interface T heraus,
// sondern ein Objekt der Klasse WrapT. Dieses mu eine Konstruktion
// durch T zulassen, vorzugsweise ist es von WpOLEBase<T> abgeleitet.
//

//------------------------------------------------------------------------
template<class Ts, class T, class WrapT> class WpOLECollection: public WpOLEBase<Ts>
{
public:
	// Konstruktoren, operator=
	// diese rufen nur die Oberklasse
	WpOLECollection() {DBG_CTOR(WpOLECollection,NULL);};
	WpOLECollection(Ts* pInt):WpOLEBase<Ts>(pInt){DBG_CTOR(WpOLECollection,NULL);};
	WpOLECollection(const WpOLECollection& rhs){DBG_CTOR(WpOLECollection,NULL);operator=(rhs);};
	inline WpOLECollection& operator=(const WpOLECollection& rhs)
		{WpOLEBase<Ts>::operator=(rhs); return *this;};

	//////////////////////////////////////////////////////////////////////

	inline void Refresh(){pInterface->Refresh();};

	inline INT16 GetItemCount()	const
	{
		INT16 nCount;
		pInterface->get_Count(&nCount);
		return nCount;
	};

	inline WrapT GetItem(INT16 index) const
	{
		OLEVariant aINT16Variant;
		aINT16Variant.SetInt16(index);
		T* pT;
		if (FAILED(pInterface->get_Item(aINT16Variant, &pT))) return WrapT(NULL);
		return WrapT(pT);
	};

	inline WrapT GetItem(const String& sStr) const
	{
		OLEVariant aStringVariant;
		aStringVariant.SetString(sStr);
		T* pT;
		if (FAILED(pInterface->get_Item(aStringVariant, &pT))) return WrapT(NULL);
		return WrapT(pT);
	}


public:

	virtual ~WpOLECollection(){DBG_DTOR(WpOLECollection,NULL);};

};

DBG_NAMEEX(WpOLEAppendCollection);
template<class Ts, class T, class WrapT> class WpOLEAppendCollection:
		public WpOLECollection<Ts,T,WrapT>
{

public:
	// Konstruktoren, operator=
	// diese rufen nur die Oberklasse
	WpOLEAppendCollection() {DBG_CTOR(WpOLEAppendCollection,NULL);};
	WpOLEAppendCollection(Ts* pInt):WpOLECollection<Ts,T,WrapT>(pInt){DBG_CTOR(WpOLEAppendCollection,NULL);};
	WpOLEAppendCollection(const WpOLEAppendCollection& rhs){DBG_CTOR(WpOLEAppendCollection,NULL);operator=(rhs);};
	inline WpOLEAppendCollection& operator=(const WpOLEAppendCollection& rhs)
		{WpOLEBase<Ts>::operator=(rhs); return *this;};
	~WpOLEAppendCollection() {DBG_DTOR(WpOLEAppendCollection,NULL);};
	//////////////////////////////////////////////////////////////////////

	inline BOOL Append(WrapT aWrapT)
	{
		return SUCCEEDED(pInterface->Append(aWrapT.pInterface));
	};

	inline BOOL Delete(const String& sName)
	{
		return SUCCEEDED(pInterface->Delete(OLEVariant::BSTRFromC(sName)));
	};


};
#endif // SDB_WPDAOTMP_HXX

