/*************************************************************************
 *
 *  $RCSfile: oterconn.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_OTERCONN_HXX
#define _SDB_OTERCONN_HXX 1

// oterinc.hxx ganz am Anfang!
//#ifndef _SDB_OTERINC_HXX
//#include "oterinc.hxx"
//#endif

#ifndef _SDB_OTERSTAT_HXX
#include "oterstat.hxx"
#endif

#ifndef _SDB_SDBCONN_HXX
#include "sdbconn.hxx"
#endif

// Kann laenger dauern, daher mit Aufruf des Busy Callback
#define OTER_ASYNC(aCommand)							\
	while ((nRetcode = aCommand) == SQL_STILL_EXECUTING)   \
		BUSY_DB();

class SdbOTERHandle	: public SvRefBase
{
	HENV aOTERHandle;
public:
	SdbOTERHandle();
	~SdbOTERHandle();
	HENV GetHandle() const { return aOTERHandle; };
	void Alloc();
	String aPath;
};
SV_DECL_IMPL_REF(SdbOTERHandle);
// Class SdbOTERConnection
//
// Beschreibung:
//	Implementation der SdbOTERConnection fuer OTER
//
class SdbOTERConnection : public SdbConnection
{
	friend class SdbEnvironment;
	friend class SdbOTERCursor;
	friend class SdbOTERTable;
	friend class SdbOTERRelation;

protected:
	// Neue Verbindungen werden nur mit SdbOTERConnection::Create (Factory)
	// angelegt, daher ist der Constructor nicht public.
	SdbOTERConnection(SdbEnvironment* pEnv, const String& rOptions);
	SdbOTERConnection(const SdbOTERConnection& _rConn);

	// Verbindungen koennen nur mit der Create-Factory
	// *dynamisch* angelegt werden. Statische oder automatische
	// Instanzen sind nicht moeglich.
	// Zur Zerstoerung eines Connection-Objektes ist Delete zu
	// verwenden (welches selbst 'delete this' ausfuehrt).
	virtual ~SdbOTERConnection();

	//-------------------------------------------------------------
	// von der Implementation zu realisieren:

protected:
	// Factory fuer Cursor, View, ...-Objekte
	// Die von der Factory erzeugten Objekte liegen in der
	// Verantwortung der Connection (und werden daher beim
	// Schliessen der Connection automatisch zerstoert).
	virtual SdbCursor* CreateCursorImpl(SdbCursorType eType = SDB_KEYSET,
										UINT16 nOptions = SDB_READONLY);

	virtual SdbTable * CreateTableImpl();
	virtual SdbRelation* CreateRelationImpl(const String& _rName);
	// ein Komandointerpreter mit Argument(en) : mu von den abgeleiteten Klassen definiert werdem
	virtual BOOL DoCmdImpl(UINT16 nCmd,const String &rArgs);

	virtual void SetAutoCommit(BOOL bAuto);

	// COMMIT ausfuehren
	virtual void CommitImpl();

	// ROLLBACK ausfuehren
	virtual void RollbackImpl();

	// Verbindung aufbauen
	virtual BOOL OpenImpl();

	// Verbindung schliessen
	virtual BOOL CloseImpl();

	static SdbDatabaseType MapType(INT16 nOTERType);

	virtual void FillTypeList(SdbTypeInfoList& rList);
	virtual void FillTableNames(SdbStringStlMap* _pList);
	virtual void FillRelations(SdbRelationList& _rList, const String& rName);
	virtual void FillAllRelations(SdbRelationList& _rList);

	virtual BOOL IsCapableImpl(SdbCapability eCapability) const;
	virtual ::com::sun::star::data::XDatabaseDescriptor* GetMetaDataImpl();
	virtual void SetIsolationLevelImpl(SdbIsolationLevel eNewIsolationLevel);


private:
	//-------------------------------------------------------------
	// Eigene Members der Implementation

	String aDsn;		// Name der Datasource

	// Connection Handle (Environment siehe auch sdbenv.hxx)
	//HENV aEnvironmentHandle;			// einmal global
	SdbOTERHandleRef aEnvironmentHandle;
	HDBC aConnectionHandle;				// einmal pro Connection

	BOOL   bReadOnly : 1;				// Treiber ReadOnly ?

	// Status der letzten Operation abfragen und aStatus entsprechend setzen.
	// Wenn sich die Operation auf eine Connection bezieht, muss deren Handle
	// mit angegeben werden.

	// Die OTER-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	void SetStatus(RETCODE nRetcode, HDBC hdbc = SQL_NULL_HDBC,
				   const String& rErrorInfo = aEmptyString);

public:
	// Pruefung der Faehigkeiten (OTER Level, verfuegbare Funktionen, ...)
	// des jeweiligen OTER-Treibers. Setzt Status auf Fehler, wenn die
	// Faehigkeiten nicht ausreichen.
	//BOOL IsDriverCapable(UWORD nInfoType, ULONG nInfoFlag) const;
	BOOL GetOTERInfo(UWORD nInfoType, String & rString) const;
	BOOL GetOTERInfo(UWORD nInfoType, USHORT & rNumber) const;
	BOOL GetOTERInfo(UWORD nInfoType, ULONG & rNumber) const;
	BOOL GetOTERInfo(UWORD nInfoType, BOOL & rBool) const;

	String ConnectStr() const;
	String GetDsn() const {return aDsn;}

	RETCODE OpenConnection(const String &aDSN,const String &aUID,const String &aPWD,INT32 nTimeOut);
	virtual BOOL Create();
	static void FreeEnv(SdbEnvironment* pEnv);
	BOOL IsMultiUserOn() const;
	BOOL IsTransactOn() const;

#if SUPD>511 || PRIV_DEBUG
	virtual BOOL StructureIsReadOnly();
#endif // SUPD>512 || PRIV_DEBUG

private:
	void CheckOTERCapabilities();
	void Construct(SdbEnvironment* pEnv);
	BOOL FillRelation(SdbOTERRelation& rRel);
	SdbCursor* OpenRelationCursor(const String &aTableName);
	BOOL IsAutoNum(const String& aTabName,const String& aColName);

public:
	// OTER Environment und Connection Handles
	// (Zugriff wird von SdbOTERCursor, SdbOTERTable, ... benoetigt)
	static SdbOTERHandleRef EnvironmentHandle(SdbEnvironment* pEnv,String &_rPath);
	HENV EnvironmentHandle();
	HDBC ConnectionHandle() const;

	// Namen aller verfuegbaren Server:
	static String ServerNames(SdbStatus& rStat, SdbEnvironment* pEnv);
};


// Inline-Implementationen:

inline void SdbOTERConnection::SetStatus(RETCODE nRetcode, HDBC hdbc,
										 const String & rErrorInfo)
{
	SdbOTER_SetStatus(aStatus,nRetcode,aEnvironmentHandle,hdbc,SQL_NULL_HSTMT,rErrorInfo);
}

inline HDBC SdbOTERConnection::ConnectionHandle() const
{
	return aConnectionHandle;
}

#endif


