/*************************************************************************
 *
 *  $RCSfile: otercurs.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:45 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_OTERCURS_HXX
#define _SDB_OTERCURS_HXX 1

// oterinc.hxx ganz am Anfang!
#ifndef _SDB_OTERINC_HXX
#include "oterinc.hxx"
#endif

#ifndef _SDB_SDBCURS_HXX
#include "sdbcurs.hxx"
#endif

#ifndef _SDB_OTERSTAT_HXX
#include "oterstat.hxx"
#endif

// Class SdbOTERCursor
//
//
class SdbOTERCursor : public SdbCursor
{
	union
	{
		char aCharArray[2048];			// Temporaerer Speicher fuer: String und BLOB-Daten
		INT8  nINT8;					// ... INT8
		INT16 nINT16;					// ... INT16
		INT32 nINT32;                	// ... INT32
		float nFloat;                	// ... float
		double nDouble;              	// ... double
		DATE_STRUCT		 aDate;
		TIME_STRUCT		 aTime;			//
		TIMESTAMP_STRUCT aTimestamp;	// ... OTER-Datums-/Zeitstruktur
	}	aBuffer;

	friend class SdbOTERConnection;
	friend class SdbOTERTable;
	friend class SdbOTERIndex;

	SdbOTERCursor(SdbOTERConnection* pConnection,
				  SdbCursorType eType,
				  UINT16 nOptions);

	virtual ~SdbOTERCursor();

protected:
	// Beschreibung der Struktur der Parameter
	virtual void DescribeParameter();

	// Beschreibung der Struktur der Ergebnismenge
	virtual void DescribeResult();

	virtual void PrepareStatement(SdbConnection* pConnection, SdbSqlParseNode* pParseNode, String& rString);

	// Vorbereitung der SQL-Anfrage fuer die Ausfuehrung
	// (wird in der Regel automatisch durch den Konstruktor
	// aufgerufen [ausser bei ExecuteDirect])
	virtual BOOL PrepareImpl();

	// Ausfuehrung der SQL-Anfrage (nach erfolgter Bindung
	// aller benoetigten Parameter).
	virtual BOOL OpenImpl(SdbCursorType eType = SDB_CURRENT,
						  UINT16 eOpt = SDB_CURRENT_OPT);
	virtual BOOL OnOpen();

	virtual BOOL InsertImpl();
	virtual BOOL UpdateImpl();
	virtual BOOL DeleteImpl();

	virtual BOOL DetermineRowCountImpl(INT32&);

	// Bricht die Ausfuehrung des Statements (waehrend Execute
	// oder waehrend Fetch-Operationen) ab.
	// Kann waehrend der Ausfuehrung eines Statements (aus dem
	// Busy Callback heraus) ausgefuehrt werden!
	virtual void CancelExecution();


	// Cursor schliessen. Durch Aufruf von Execute kann das
	// Statement (mit gleichen oder verschiedenen Parametern)
	// erneut ausgefuehrt werden.

	// Falls noch nicht alle Rows der Ergebnismenge abgeholt
	// wurden, werden diese ignoriert.

	// Nach Close ist die Strukturbeschreibung nicht mehr zugreifbar.
	// Gebundene Rows werden durch Close zerstoert. Wenn eine gebundene
	// Row nicht zerstoert werden soll, so muss die Bindung vor dem Close
	// mit Release... aufgehoben werden.
	virtual BOOL CloseImpl();


	// Cursor auf einen Datensatz der Ergebnismenge
	// positionieren: auf den naechsten oder vorherigen
	// den ersten oder letzten, relativ oder absolut
	// (relativ und absolut mit Angabe des Offset).
	virtual BOOL MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset, SdbCursorMode& rMode);
	virtual BOOL IsCapable(SdbCursorCapability eCapability) const;
	virtual void DriverTranslate();

#if SUPD>557
	virtual
#endif
		void RemovingFromParent();

	HSTMT	StmtHandle() const {return aStatementHandle;}

private:
	//-------------------------------------------------------------
	// Eigene Member der Implementation

	// Environment und Connection Handle
	HSTMT aStatementHandle;				// OTER Statement Handle

	void** pParamBuffer;				// Buffer fr rowbinding
	INT32* pParamLength;
	INT32* pRowVer;						// enth"alt alle ROW_VER Verweise von Oterro
	INT32  nRowVerLen;
	UINT32	nCurrentMaxFieldLen, nCurrentMaxRows;
	BOOL   bCommitTrans;

	// Status der letzten Operation abfragen und aStatus entsprechend setzen.
	// Wenn sich die Operation auf eine Connection bezieht, muss deren Handle
	// mit angegeben werden.

	// Die OTER-Status-Werte STILL_EXECUTING und NEED_DATA werden ignoriert,
	// INVALID_HANDLE fuehrt zu einer Assertion. Alle uebrigen Werte werden
	// in aStatus gesetzt (und damit wird ein ggf. registrierter Link in
	// aStatus ausgefuehrt).
	void SetStatus(RETCODE nRetcode, HSTMT hstmt = SQL_NULL_HSTMT,
				   const String& rErrorInfo = aEmptyString);

	// Parameter binden
	void BindParameter();
	BOOL BindColumn(const SdbColumn& rCol, UINT16 nPos, const ODbVariantRef& xVar);

	// DatenHolen - Schreiben
	BOOL GetData(const SdbColumn&, const ODbVariantRef& xVar, UINT16 nPos);
	BOOL PutData(const SdbColumn&, const ODbVariantRef& xVar);

	void FixResultRow();

	void AllocParamData();
	void FreeParamData();

	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Tabellen, Views, ... als Ergebnismenge liefert.
	void OpenTables(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
					const sal_Unicode* pTable, const sal_Unicode* pType);

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Rechte
	// des Benutzers fuer eine gegebene Tabelle liefert
	void OpenTablePrivileges(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable);

	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Columns in einer Tabelle als Ergebnismenge liefert.
	void OpenColumns(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
					 const sal_Unicode* pTable, const sal_Unicode* pColumn);

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// eines Primaerschluessels einer Tabelle als Ergebnismenge liefert.
	void OpenPrimaryKeys(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable);

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// einer Fremdschluesselbeziehung zwischen zwei Tabellen als Ergebnismenge liefert.
	void OpenForeignKeys(const sal_Unicode* pPkQualifier, const sal_Unicode* pPkOwner, const sal_Unicode* pPKName,
						 const sal_Unicode* pFkQualifier, const sal_Unicode* pFkOwner, const sal_Unicode* pFKName);

	// Spezielle Methode anstelle von Execute, die die Namen der bekannten
	// Prozeduren als Ergebnismenge liefert.
	void OpenProcedures(const String& rQualifier, const String& rSchema, const String& rProcName);

	// Spezielle Methode anstelle von Execute, die Informationen ueber die Columns
	// einer Prozedur als Ergebnismenge liefert.
	void OpenProcedureColumns(const String& rQualifier, const String& rSchema,
							  const String& rProcName, const String& rProcColumnName);

	void OpenIndices(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable, BOOL bUnique = FALSE);

	void OpenSpecialColumns(const String& rTabName);
	// Informationen ueber alle verfuegbaren DatenTypen erhalten
	void OpenTypeInfo(INT16 nType);

	BOOL ExecuteDirect(const String &rExeSql);

	SdbOTERConnection* GetOTERConnection() const {return (SdbOTERConnection*) GetConnection();}
};


// Inline-Implementationen:


#endif



