/*************************************************************************
 *
 *  $RCSfile: sdbcode.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:46 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SDB_SDBCODE_HXX
#define _SDB_SDBCODE_HXX

#ifndef _SDB_ROW_HXX
#include "row.hxx"
#endif

#ifndef _RTTI_HXX //autogen
#include <tools/rtti.hxx>
#endif

#ifndef _SDB_FILECOL_HXX
#include "filecol.hxx"
#endif

#include "sdbsqlit.hxx"

class SdbCodeStack;
class SdbSqlParseNode;
class SdbBoolOperator;
class SdbEvaluateSet;


class SdbCode
{
public:
	virtual ~SdbCode();

	TYPEINFO();
};


// Operanden die aus einem ParseTree ermittelt werden
class SdbOperand : public SdbCode
{
protected:
	SdbDatabaseType eDBType;

	SdbOperand(SdbDatabaseType Type) : eDBType(Type){};
	SdbOperand(){};

public:
	virtual const ODbVariant* Value() const = 0;
	virtual SdbDatabaseType DBType() const {return eDBType;}
	virtual SdbEvaluateSet* PreProcess(SdbBoolOperator* pOp, SdbOperand* pRight = 0);
	inline BOOL	IsValid() const;

	TYPEINFO();
};

class SdbOperandRow : public SdbOperand
{
	UINT16	nRowPos;

protected:
	ODbRowRef aRow;

	SdbOperandRow(UINT16 nPos, SdbDatabaseType Type) : SdbOperand(Type)
													  , nRowPos(nPos){}
public:
	UINT16 RowPos() const {return nRowPos;}
	virtual const ODbVariant* Value() const;
	void BindValue(const ODbRowRef& rRow);						// Bindung an den Wert, den der Operand reprsentiert

	TYPEINFO();
};

// Attribute aus einer Ergebniszeile
class SdbOperandAttr : public SdbOperandRow
{
protected:
	const SdbColumn* pColumn;

public:
	SdbOperandAttr(const SdbColumn*);

	virtual BOOL IsIndexed() const {return FALSE;}
	virtual SdbEvaluateSet* PreProcess(SdbBoolOperator* pOp, SdbOperand* pRight = 0) { return NULL;}
	TYPEINFO();
};

// Attribute aus einer Ergebniszeile
class SdbFILEOperandAttr : public SdbOperandAttr
{
public:
	SdbFILEOperandAttr(const SdbColumn*);

	virtual BOOL IsIndexed() const {return ((const SdbFILEColumn*)pColumn)->IsIndexed();}
	virtual SdbEvaluateSet* PreProcess(SdbBoolOperator* pOp, SdbOperand* pRight = 0);
	TYPEINFO();
};


// Parameter fr ein Prdikat
class SdbOperandParam : public SdbOperandRow
{
public:
	SdbOperandParam(SdbSqlParseNode* pNode, const String& rParameterNames, SdbColumnsRef& rParamColumns);
	void Describe(const SdbColumn& rColumn, SdbColumnsRef& rParameterColumns);

	TYPEINFO();
};


// WerteOperanden
class SdbOperandValue : public SdbOperand
{
protected:
	ODbVariantRef xValue;

protected:
	SdbOperandValue():xValue(new ODbVariant()) {}
	SdbOperandValue(const ODbVariant& rVar, SdbDatabaseType eDbType):SdbOperand(eDbType)
																	 ,xValue(new ODbVariant(rVar))
																	 {}
	SdbOperandValue(SdbDatabaseType eDbType) :SdbOperand(eDbType)
											 ,xValue(new ODbVariant()){}
public:
	virtual const ODbVariant* Value() const;	

	TYPEINFO();
};


// Konstanten
class SdbOperandConst : public SdbOperandValue
{
public:
	SdbOperandConst(const SdbSqlParseNode& rColumnRef, const rtl::OUString& aStrValue);

	TYPEINFO();
};


// Ergebnis Operanden
class SdbOperandResult : public SdbOperandValue
{
protected:
	SdbOperandResult(const ODbVariant& rVar, SdbDatabaseType eDbType)
					:SdbOperandValue(rVar, eDbType) {};
	SdbOperandResult(SdbDatabaseType eDbType)
					:SdbOperandValue(eDbType) {};
public:
	TYPEINFO();
};


class SdbOperandResultBOOL : public SdbOperandResult
{
public:
	SdbOperandResultBOOL(BOOL bResult):SdbOperandResult(SDB_DBTYPE_BOOLEAN) {xValue->setDouble(bResult);}
};

class SdbOperandResultNUM : public SdbOperandResult
{
public:
	SdbOperandResultNUM(double fNum):SdbOperandResult(SDB_DBTYPE_DOUBLE) {xValue->setDouble(fNum);}
};


// Operatoren

class SdbOperator : public SdbCode
{
public:
	virtual void Exec(SdbCodeStack&) = 0;
	virtual UINT16 RequestedOperands() const;			// Anzahl bentigter Operanden
														// Standard ist 2
	TYPEINFO();
};


// boolsche Operatoren

class SdbBoolOperator : public SdbOperator
{
public:
	TYPEINFO();
	virtual void Exec(SdbCodeStack&);
	virtual BOOL Operate(const SdbOperand*, const SdbOperand*) const;
};


class SdbOp_AND : public SdbBoolOperator
{
public:
	TYPEINFO();

protected:
	virtual BOOL Operate(const SdbOperand*, const SdbOperand*) const;
};

class SdbOp_OR : public SdbBoolOperator
{
public:
	TYPEINFO();
protected:
	virtual BOOL Operate(const SdbOperand*, const SdbOperand*) const;
};

class SdbOp_ISNULL : public SdbBoolOperator
{
public:
	TYPEINFO();
public:
	virtual void Exec(SdbCodeStack&);
	virtual UINT16 RequestedOperands() const;
	virtual BOOL Operate(const SdbOperand*, const SdbOperand* = NULL) const;
};

class SdbOp_ISNOTNULL : public SdbOp_ISNULL
{
public:
	TYPEINFO();
	virtual BOOL Operate(const SdbOperand*, const SdbOperand* = NULL) const;
};

class SdbOp_LIKE : public SdbBoolOperator
{
public:
	TYPEINFO();
protected:
	const sal_Unicode cEscape;

public:
	SdbOp_LIKE(const sal_Unicode cEsc = L'\0'):cEscape(cEsc){};

	virtual BOOL Operate(const SdbOperand*, const SdbOperand*) const;
};

class SdbOp_NOTLIKE : public SdbOp_LIKE
{
public:
	TYPEINFO();
public:
	SdbOp_NOTLIKE(const char cEsc = '\0'):SdbOp_LIKE(cEsc){};

	virtual BOOL Operate(const SdbOperand*, const SdbOperand*) const;
};

class SdbOp_COMPARE : public SdbBoolOperator
{
	SdbPredicateType aPredicateType;	

public:
	TYPEINFO();
	SdbOp_COMPARE(SdbPredicateType aPType)
				 :aPredicateType(aPType) {}

	SdbPredicateType GetPredicateType() const {return aPredicateType;}	
	virtual BOOL Operate(const SdbOperand*, const SdbOperand*) const;
};

// numerische Operatoren

class SdbNumOperator : public SdbOperator
{
public:
	virtual void Exec(SdbCodeStack&);

	TYPEINFO();

protected:
	virtual double Operate(double fLeft, double fRight) const = 0;
};

class SdbOp_ADD : public SdbNumOperator
{
protected:
	virtual double Operate(double fLeft, double fRight) const;
};

class SdbOp_SUB : public SdbNumOperator
{
protected:
	virtual double Operate(double fLeft, double fRight) const;
};

class SdbOp_MUL : public SdbNumOperator
{
protected:
	virtual double Operate(double fLeft, double fRight) const;
};

class SdbOp_DIV : public SdbNumOperator
{
protected:
	virtual double Operate(double fLeft, double fRight) const;
};

inline BOOL	SdbOperand::IsValid() const 
{	
	const ODbVariant* pValue = Value();
	return pValue && (pValue->getValueTypeClass() == ::com::sun::star::uno::TypeClass_DOUBLE) &&
		   pValue->getDouble() != 0;
}

#endif

