/*************************************************************************
 *
 *  $RCSfile: sdbcomp.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:46 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SDB_SDBCOMP_HXX
#define _SDB_SDBCOMP_HXX

#ifndef _SDB_ROW_HXX
#include "row.hxx"
#endif

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif

#ifndef _STACK_HXX //autogen
#include <tools/stack.hxx>
#endif

class SdbCode;
class SdbOperand;
class SdbSqlParseNode;
class SdbStatus;

DECLARE_LIST(SdbCodeList, SdbCode*);

class SdbPredicateCompiler
{
	friend class SdbCursor;
	friend class SdbPredicateInterpreter;
	friend class SdbSQLAnalyzer;

	SdbCodeList		aCodeList;
	SdbCursor&		m_rCursor;
	SdbColumnsRef	m_orgColumns; // in filecurs this are the filecolumns
	SdbSQLAnalyzer* m_pAnalyzer;
	BOOL			bORCondition;

public:
	SdbPredicateCompiler(SdbSQLAnalyzer* pAnalyzer,SdbCursor& rCurs);

	virtual ~SdbPredicateCompiler();

	void Start(SdbSqlParseNode* pSQLParseNode);
	SdbOperand* Execute(SdbSqlParseNode* pPredicateNode);

	void Clean();
	BOOL IsClean() const {return aCodeList.Count() == 0;}
	BOOL HasCode() const {return !IsClean();}
	BOOL HasORCondition() const {return bORCondition;}
	void setOrigColumns(const SdbColumnsRef& rCols) { m_orgColumns = rCols; }
	const SdbColumnsRef getOrigColumns() const { return m_orgColumns; }

protected:
	SdbOperand* Execute_COMPARE(SdbSqlParseNode* pPredicateNode);
	SdbOperand* Execute_LIKE(SdbSqlParseNode* pPredicateNode);
	SdbOperand* Execute_ISNULL(SdbSqlParseNode* pPredicateNode);
	SdbOperand* Execute_Operand(SdbSqlParseNode* pPredicateNode);

private:
	SdbCursor& Cursor() const;
};


DECLARE_STACK(SdbCodeStack, SdbOperand*);

class SdbPredicateInterpreter
{
	SdbCodeStack aStack;
	SdbPredicateCompiler& rCompiler;

public:
	SdbPredicateInterpreter(SdbPredicateCompiler& rComp):rCompiler(rComp){};
	virtual ~SdbPredicateInterpreter();

	BOOL Start() {return Evaluate(rCompiler.aCodeList);}
	BOOL Evaluate(SdbCodeList& rCodeList);
};

#endif

