/*************************************************************************
 *
 *  $RCSfile: odbccurs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:46 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbresid.hrc"
#include "odbccurs.hxx"
#include "odbcconn.hxx"

#include <stdlib.h>
#include "sdbtools.hxx"

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEOUTERJOINSUPPORT_HPP_
#include <com/sun/star/data/DatabaseOuterJoinSupport.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

// in odbc we have a fraction part for time values, a single unit is 1/100000000
const ULONG ODBC_FRACTION_UNITS_PER_HSECOND = 10000000L;
const String aTruncState = String::CreateFromAscii("01004");
const String aValueChangedState = String::CreateFromAscii("01S02");

//------------------------------------------------------------------
DateTime SdbODBCCursor::ToDateTime(const TIMESTAMP_STRUCT& rTS)
{
	return DateTime (Date(rTS.day,rTS.month,rTS.year),
			  Time(rTS.hour,rTS.minute,rTS.second,long (rTS.fraction / ODBC_FRACTION_UNITS_PER_HSECOND)));
}

//------------------------------------------------------------------
Time SdbODBCCursor::ToTime(const TIME_STRUCT& rTS)
{
	return Time(rTS.hour,rTS.minute,rTS.second);
}

//------------------------------------------------------------------
Date SdbODBCCursor::ToDate(const DATE_STRUCT& rTS)
{
	return Date(rTS.day,rTS.month,rTS.year);
}

//------------------------------------------------------------------
void SdbODBCCursor::ToDate(DATE_STRUCT& rResult, double fSbxDate)
{
	Date aDate = SdbTools::ToDate(fSbxDate);
	rResult.year	  = aDate.GetYear();
	rResult.month	  = aDate.GetMonth();
	rResult.day		  = aDate.GetDay();
}

//------------------------------------------------------------------
void SdbODBCCursor::ToTime(TIME_STRUCT& rResult, double fSbxDate)
{
	Time aTime = SdbTools::ToTime(fSbxDate);
	// Zeit:
	rResult.hour	  = aTime.GetHour();
	rResult.minute	  = aTime.GetMin();
	rResult.second	  = aTime.GetSec();
}

//------------------------------------------------------------------
void SdbODBCCursor::ToTimeStamp(TIMESTAMP_STRUCT& rResult, double fSbxDate)
{
	DateTime aDateTime = SdbTools::ToDateTime(fSbxDate);
	rResult.year	  = aDateTime.GetYear();
	rResult.month	  = aDateTime.GetMonth();
	rResult.day		  = aDateTime.GetDay();

	// Zeit:
	rResult.hour	  = aDateTime.GetHour();
	rResult.minute	  = aDateTime.GetMin();
	rResult.second	  = aDateTime.GetSec();
	rResult.fraction  = aDateTime.Get100Sec() * ODBC_FRACTION_UNITS_PER_HSECOND;
}

DBG_NAME(SdbODBCCursor);

//-----------------------------------------------------------------------------
SdbODBCCursor::SdbODBCCursor(SdbODBCConnection* pConnection,
							 SdbCursorType eType,
							 UINT16 nOpt)
			  :SdbCursor(pConnection, eType, nOpt)
			  ,pParamBuffer(NULL)
			  ,pParamLength(NULL)
			  ,pFieldBuffer(NULL)
			  ,pFieldLength(NULL)
			  ,bBookmarks(FALSE)
			  ,m_aConnectionHandle(NULL)
			  ,m_aStatementHandle(NULL)
{
	DBG_CTOR(SdbODBCCursor,NULL);

	aStatus.SetComponent(String::CreateFromAscii("Sdb ODBC Driver"));
	bIsCancelable = TRUE;

	nCurrentMaxFieldLen = nMaxFieldLen;
	nCurrentMaxRows = nMaxRows;
}

//-----------------------------------------------------------------------------
void SdbODBCCursor::DetermineOptions(SdbODBCConnection* pConnection)
{
	bBookmarks = IsCapable(SDB_CAP_BOOKMARKS);

	// Cursor kann nur in eine Richtung bewegt werden
	if ((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY ||
		!IsCapable(SDB_CAP_FETCH_FIRST) ||
		!IsCapable(SDB_CAP_UPDATE_POSITIONED))
	{
		bBookmarks = FALSE;
		nCursorType = SQL_CURSOR_FORWARD_ONLY;	// default
	}
	else if (eCursorType == SDB_DYNAMIC || eCursorType == SDB_KEYSET)
		nCursorType = SQL_CURSOR_KEYSET_DRIVEN;
	else
		nCursorType = SQL_CURSOR_STATIC;

	// Updatefaehigkeiten und Cursortyp setzen
	nConcurrency = SQL_CONCUR_READ_ONLY;
	if (IsCapable(SDB_CAP_UPDATE_POSITIONED) && nCursorType == SQL_CURSOR_KEYSET_DRIVEN)
	{
		if (pConnection->nDriverConcurrency & SQL_SCCO_OPT_ROWVER)
			nConcurrency = SQL_CONCUR_ROWVER;
		else if (pConnection->nDriverConcurrency & SQL_SCCO_OPT_VALUES)
			nConcurrency = SQL_CONCUR_VALUES;
		else if (pConnection->nDriverConcurrency & SQL_SCCO_LOCK)
			nConcurrency = SQL_CONCUR_LOCK;
	}
}

//-----------------------------------------------------------------------------
SdbODBCCursor::~SdbODBCCursor()
{
	DBG_DTOR(SdbODBCCursor,NULL);
}

//-----------------------------------------------------------------------------
void SdbODBCCursor::CancelExecution()
{
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::PrepareImpl()
{
	DBG_CHKTHIS(SdbODBCCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	// Statement vorbereiten
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(GetConnection()->GetMetaData());
	sal_Int16 nInfo;
	rMetaData->getInfo(::stardata::DatabaseInfo::OUTER_JOIN_SUPPORT) >>= nInfo;
	BOOL bInnerJoin = (nInfo & ::stardata::DatabaseOuterJoinSupport::YES || nInfo & ::stardata::DatabaseOuterJoinSupport::PARTIAL);
	Translate(bInnerJoin);
	if (! aStatus.IsSuccessful())
		return FALSE;

	PrepareODBC(); // set the status inside
	if (! aStatus.IsSuccessful()) return FALSE;

	// Cursor-Namen abholen (jetzt ist er bestimmt verfuegbar):
	// selbst setzen, da der Name bis jetzt noch nicht wirklich
	// bentigt wird
	if (aName.Len() == 0)
	{
		aName = 'C';
		aName += String::CreateFromInt32((long)this);
	}

	// hack
	// Parameter schon gesetzt?
	if (aParameterColumns->Count() < 2)
		DescribeParameter();

	if (aStatus.IsSuccessful())
		BindParameter();

	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
void SdbODBCCursor::BindParameter()
{
	DBG_CHKTHIS(SdbODBCCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	aStatus.Clear();
	AllocBuffer(pParamBuffer, pParamLength, *aParameterColumns,FALSE);
}
//------------------------------------------------------------------
BOOL SdbODBCCursor::Open(const String& rSQL, BOOL bTable, SdbCursorType eType, UINT16 eOpt)
{
	ImplChangeOpenOptions(eType, eOpt);
	return SdbCursor::Open(rSQL, bTable, SDB_CURRENT, SDB_CURRENT_OPT);
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::OpenImpl(SdbCursorType eType ,UINT16 eOpt )
{
	DBG_CHKTHIS(SdbODBCCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ImplChangeOpenOptions(eType, eOpt);
#if 0
	RETCODE nRetcode;

	if (IsPrepared())
	{
		if (IsOpen())
		{
			// Schon einmal Execute ausgefuehrt.
			// Vor dem naechsten Execute muss der Cursor erst geschlossen werden
			// (Option CLOSE).
			NSQLFreeStmt(m_aStatementHandle,SQL_CLOSE);
		}
		UINT16 nCount(aParameterColumns->Count());
		for (UINT16 i=1 ; i < nCount;i++)
			BindColumn( i, pParamBuffer, pParamLength, *(*aParameterColumns)[i], (*aParameterRow)[i]);
	}
	else
	{
		::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(GetConnection()->GetMetaData());
		::staruno::Any aAny(rMetaData->getInfo(::stardata::DatabaseInfo::OUTER_JOIN_SUPPORT));
		BOOL bInnerJoin = (aAny.getINT16() & ::stardata::DatabaseOuterJoinSupport::YES || aAny.getINT16() & ::stardata::DatabaseOuterJoinSupport::PARTIAL);
		Translate(bInnerJoin);
		if (! aStatus.IsSuccessful())
			return FALSE;
	}

	// Anpassen  der Menge der zu bertragenden Daten
	if (nCurrentMaxFieldLen != nMaxFieldLen)
	{
		nRetcode = NSQLSetStmtOption(m_aStatementHandle, SQL_MAX_LENGTH, nMaxFieldLen);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, m_aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState().Compare(aValueChangedState) == COMPARE_EQUAL)
				NSQLGetStmtOption(m_aStatementHandle, SQL_MAX_LENGTH, &nMaxFieldLen);

			nCurrentMaxFieldLen = nMaxFieldLen;
		}
		else
			// Wieder zuruecksetzen
			nMaxFieldLen = nCurrentMaxFieldLen;
	}

	if (nCurrentMaxRows != nMaxRows)
	{
		nRetcode = NSQLSetStmtOption(m_aStatementHandle, SQL_MAX_ROWS, nMaxRows);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, m_aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState().Compare(aValueChangedState) == COMPARE_EQUAL)
				NSQLGetStmtOption(m_aStatementHandle, SQL_MAX_ROWS, &nMaxRows);

			nCurrentMaxRows = nMaxRows;
		}
		else
			// Wieder zuruecksetzen
			nMaxRows = nCurrentMaxRows;
	}

	if (IsPrepared())
	{
		BEGIN_CANCELABLE();
		nRetcode = NSQLExecute(m_aStatementHandle);
		END_CANCELABLE();
	}
	else
	{
		BEGIN_CANCELABLE();
		nRetcode = NSQLExecDirect(m_aStatementHandle, (SDB_ODBC_CHAR*)(const char*)aSQLStatement, aSQLStatement.Len());
		END_CANCELABLE();
	}

	nCurrentMaxFieldLen = nMaxFieldLen;
	nCurrentMaxRows = nCurrentMaxRows;

	// Parameter at Execution Time erforderlich?
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;

		nRetcode = NSQLParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS &&
				nRetcode != SQL_SUCCESS_WITH_INFO &&
				nRetcode != SQL_NEED_DATA) break;

			if (aParameterColumns->Count() == 1)
			{
				// Keine Strukturbeschreibung fuer die Parameter Row oder keine Parameter Row.
				// Das darf nicht sein - Statement-Ausfuehrung abbrechen!
				if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLCANCEL))
					NSQLCancel(m_aStatementHandle);
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1093"),
							aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_PARAMETER_NUMBER))),
							0, String() );
				// Genaugenommen eigentlich: Parameter missing ...
				return FALSE;
			}

			// Daten fuer diesen Parameter an ODBC uebergeben ...
			// Die zugehoerige Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			PutData(*pCol, aParameterRow->Variable(pCol->GetId())); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
			{
				// Bei Fehler Statement-Ausfuehrung abbrechen!
				if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLCANCEL))
					NSQLCancel(m_aStatementHandle);
				return FALSE;
			}

			nRetcode = NSQLParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
	}

	// Resultat des SQLExec[Direct] oder des letzten SqlParamData
	// ist das Resultat des gesamten Statements ...
	SetStatus(nRetcode,m_aStatementHandle);
	if (!aStatus.IsSuccessful()) return FALSE;

	// solange er nicht wirklich gebraucht wird,
	// einfach erst einmal selbst setzen
	if (aName.Len() == 0)
	{
		aName = 'C';
		aName += (long)this;
	}
#endif
	return TRUE;
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::OnOpen()
{
	DBG_CHKTHIS(SdbODBCCursor,NULL);
	SdbStatus aTempStat(aStatus);

	if (!IsOpen())  		// resultset erst nach execute beschreiben!!!
							// und nur wenn noch nicht geffnet
	{
		SdbODBCConnection* pConnection = GetODBCConnection();
		if (!aResultColumns.Is())
			aResultColumns = pConnection->CreateColumns();
		DescribeResult();
	}

	// Bei Statements, die keine Results liefern (insb. UPDATE, DELETE, INSERT)
	// ist die Anzahl der vom Statement betroffenen Datensaetze interessant:
	if (aResultColumns->Count() == 1)
		RowCountImpl();

	// wenn ich hier schon mit einem nicht erfolgreichen Status reingekommen bin, muss ich das fuer die Basisklasse
	// wiederherstellen
	if (!aTempStat.IsSuccessful())
		aStatus = aTempStat;

	return SdbCursor::OnOpen();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::CloseImpl()
{
	DBG_CHKTHIS(SdbODBCCursor,NULL);

	if (aParameterColumns.is())
		FreeBuffer(pParamBuffer, pParamLength, *aParameterColumns,FALSE);
	if (aResultColumns.is())
		FreeBuffer(pFieldBuffer, pFieldLength, *aResultColumns,TRUE);
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::GetData(const SdbColumn& rCol, const ODbVariantRef& pVariable, UINT16 nPos)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)

	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
		// Column #0 ist die Bookmark!

#define GETDATA(fCType,rgbValue,cbValueMax) \
	{ \
		nRetcode = GetDataImpl((nPos),(fCType),(rgbValue),(cbValueMax),&pcbValue); \
	    SetStatus(nRetcode,m_aStatementHandle); \
		if (aStatus.IsError())	   \
		{						   \
			DBG_ERROR((((ByteString("SQLGetData fehlgeschlagen!: Column ") +=				\
			ByteString::CreateFromInt32(nPos)) += ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding())).GetBuffer()) ;	\
			pVariable->setNull();  \
			aStatus.Clear();	   \
			break;				   \
		}						   \
		else if (pcbValue == SQL_NULL_DATA) { \
			pVariable->setNull(); \
			break; \
		} \
	}

	// Gebundene Variable. Je nach Datenbanktyp als entsprechenden
	// C-Datentyp (passend zur Variablen) abholen.

	// nyi: festen Variablentyp beruecksichtigen und konvertieren!
	switch (rCol.GetType())
	{
		case SDB_DBTYPE_BOOKMARK:
			// bookmark ueber getOption abholen
			break;
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
		case SDB_DBTYPE_BIGINT:
		case SDB_DBTYPE_CURRENCY:
			if(GetODBCConnection()->m_bUserWChar)
			{
				GETDATA( SQL_C_WCHAR ,aBuffer.waCharArray,sizeof aBuffer.waCharArray);
				pVariable->setDouble(rtl::OUString(aBuffer.waCharArray).toDouble());
			}
			else
			{
				GETDATA(SQL_C_CHAR,aBuffer.aCharArray,sizeof aBuffer.aCharArray);
				pVariable->setDouble(SdbTools::ToDouble((const char *)aBuffer.aCharArray));
			}
			break;
		case SDB_DBTYPE_TINYINT:
		case SDB_DBTYPE_SMALLINT:
			aBuffer.nINT16 = 0;
			GETDATA(SQL_C_SHORT,&aBuffer.nINT16,sizeof(INT16));
			pVariable->setINT16(aBuffer.nINT16);
			// nyi: signed/unsigned
			break;

		case SDB_DBTYPE_INTEGER:
			aBuffer.nINT32 = 0;
			GETDATA(SQL_C_LONG,&aBuffer.nINT32,sizeof(INT32));
			pVariable->setINT32(aBuffer.nINT32);
			// nyi: signed/unsigned
			break;

		case SDB_DBTYPE_REAL:
			aBuffer.nFloat = (float)0;
			GETDATA(SQL_C_FLOAT,&aBuffer.nFloat,sizeof(float));
			pVariable->setFloat(aBuffer.nFloat);
			break;

		case SDB_DBTYPE_DOUBLE:
			aBuffer.nDouble = 0.0;
			GETDATA(SQL_C_DOUBLE,&aBuffer.nDouble,sizeof(double));
			pVariable->setDouble(aBuffer.nDouble);
			break;

		case SDB_DBTYPE_BOOLEAN:
			aBuffer.nINT16 = 0;
			GETDATA(SQL_C_TINYINT,&aBuffer.nINT16,sizeof(INT16));
			pVariable->setBOOL( ((INT16)aBuffer.nINT16) ? TRUE : FALSE );
			break;

		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_LONGVARCHAR:
		{
			UniString aData;
			if (GetODBCConnection()->m_bUserWChar)
			{
				// read the unicode data
				INT32 nMaxLen = sizeof(aBuffer.waCharArray) - sizeof(sal_Unicode);
				GETDATA(SQL_C_WCHAR, aBuffer.waCharArray, nMaxLen + sizeof(sal_Unicode));
				// Bei Fehler bricht der GETDATA-Makro mit return ab,
				// bei NULL mit break!
				SWORD nLen = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
				aBuffer.waCharArray[nLen] = 0;
				aData = UniString(aBuffer.waCharArray);

				// Es handelt sich um Binaerdaten, um einen String, der fuer
				// StarView zu lang ist oder der Treiber kann die Laenge der
				// Daten nicht im voraus bestimmen - also als MemoryStream
				// speichern.
				while ((pcbValue == SQL_NO_TOTAL && aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
						aStatus.SQLState() == aTruncState) || pcbValue > nMaxLen)
				{
					// Bei Strings wird der Puffer nie ganz ausgenutzt
					// (das letzte Byte ist immer ein NULL-Byte, das
					// aber bei pcbValue nicht mitgezaehlt wird)
					if (pcbValue != SQL_NO_TOTAL && (pcbValue - nMaxLen) < nMaxLen)
						nLen = pcbValue - nMaxLen;
					else
						nLen = nMaxLen;

					// Solange eine "truncation"-Warnung vorliegt, weiter Daten abholen
					GETDATA(SQL_C_CHAR,aBuffer.waCharArray, nLen + 1);
					aBuffer.aCharArray[nLen] = 0;

					aData += UniString(aBuffer.waCharArray);
				}
			}
			else
			{
				// Erstmal versuchen, die Daten mit dem kleinen Puffer
				// abzuholen:
				SDWORD nMaxLen = sizeof aBuffer.aCharArray - 1;
				GETDATA(SQL_C_CHAR,aBuffer.aCharArray,nMaxLen);

				SWORD nLen = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
				aBuffer.aCharArray[nLen] = 0;
				aData = UniString((const sal_Char*)aBuffer.aCharArray, GetConnection()->GetCharacterSet());

				// Es handelt sich um Binaerdaten, um einen String, der fuer
				// StarView zu lang ist oder der Treiber kann die Laenge der
				// Daten nicht im voraus bestimmen - also als MemoryStream
				// speichern.
				while ((pcbValue == SQL_NO_TOTAL && aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
						aStatus.SQLState() == aTruncState) || pcbValue > nMaxLen)
				{
					// Bei Strings wird der Puffer nie ganz ausgenutzt
					// (das letzte Byte ist immer ein NULL-Byte, das
					// aber bei pcbValue nicht mitgezaehlt wird)
					if (pcbValue != SQL_NO_TOTAL && (pcbValue - nMaxLen) < nMaxLen)
						nLen = pcbValue - nMaxLen;
					else
						nLen = nMaxLen;

					// Solange eine "truncation"-Warnung vorliegt, weiter Daten abholen
					GETDATA(SQL_C_CHAR,aBuffer.aCharArray, nLen + 1);
					aBuffer.aCharArray[nLen] = 0;

					aData += UniString((const sal_Char*)aBuffer.aCharArray, GetConnection()->GetCharacterSet());
				}
			}

			// delete all blanks
			if (SDB_DBTYPE_CHAR == rCol.GetType())
				aData.EraseTrailingChars();

			pVariable->setString(aData);
		} break;
		case SDB_DBTYPE_BINARY:
		case SDB_DBTYPE_VARBINARY:
		case SDB_DBTYPE_LONGVARBINARY:
		{
			// Erstmal versuchen, die Daten mit dem kleinen Puffer
			// abzuholen:
			INT32 nMaxLen = sizeof aBuffer.aCharArray;
			GETDATA(SQL_C_BINARY,aBuffer.aCharArray,nMaxLen);
			// Bei Fehler bricht der GETDATA-Makro mit return ab,
			// bei NULL mit break!

			// In pcbValue steht jetzt entweder die Gesamtlaenge
			// der Daten in diesem Feld oder aber SQL_NO_TOTAL (wenn
			// der Treiber uns die Laenge nicht liefern kann).
			// [pcbValue enthaelt immer die (Rest-)Laenge VOR
			// dem letzten Aufruf von GetData!]

			// Es handelt sich um Binaerdaten, um einen String, der fuer
			// StarView zu lang ist oder der Treiber kann die Laenge der
			// Daten nicht im voraus bestimmen - also als MemoryStream
			// speichern.
			SDWORD nBytes = pcbValue != SQL_NO_TOTAL ? Min(pcbValue, nMaxLen) : nMaxLen;
			::staruno::Sequence<sal_Int8> aData((sal_Int8*)aBuffer.aCharArray, nBytes);

			while (aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO &&
				   aStatus.SQLState() == aTruncState)
			{
				// Bei Strings wird der Puffer nie ganz ausgenutzt
				// (das letzte Byte ist immer ein NULL-Byte, das
				// aber bei pcbValue nicht mitgezaehlt wird)
				if (pcbValue != SQL_NO_TOTAL && (pcbValue - nMaxLen) < nMaxLen)
					nBytes = pcbValue - nMaxLen;
				else
					nBytes = nMaxLen;

				// Solange eine "truncation"-Warnung vorliegt, weiter Daten abholen
				GETDATA(SQL_C_BINARY,aBuffer.aCharArray, nBytes);

				INT32 nLen = aData.getLength();
				aData.realloc(nLen + nBytes);
				memcpy(aData.getArray() + nLen, aBuffer.aCharArray, nBytes);
			}
			pVariable->setBytes(aData);
		} break;

		case SDB_DBTYPE_DATE:
		{
			BOOL bNotNull = GetDateValues(nPos,aBuffer.aDate);
			if(aStatus.IsSuccessful())
			{
				if(bNotNull)
					pVariable->setDate(ToDate(aBuffer.aDate));
				else
					pVariable->setNull();
			}
			aStatus.Clear();

		} break;
		case SDB_DBTYPE_TIME:
		{
			BOOL bNotNull = GetTimeValues(nPos,aBuffer.aTime);
			if(aStatus.IsSuccessful())
			{
				if(bNotNull)
					pVariable->setTime(ToTime(aBuffer.aTime));
				else
					pVariable->setNull();
			}
			aStatus.Clear();

		} break;
		case SDB_DBTYPE_TIMESTAMP:
		{
			DBG_ASSERT(sizeof(aBuffer.aTimestamp) == 16,"SdbODBCCursor::FetchRow: fehlerhaftes Alignment in ODBC-Datumsstruktur");
			//GETDATA(SQL_C_TIMESTAMP,&aBuffer.aTimestamp,sizeof(aBuffer.aTimestamp));

			BOOL bNotNull = GetTimeDateValues(nPos,aBuffer.aTimestamp);
			if(aStatus.IsSuccessful())
			{
				if(bNotNull)
					pVariable->setDateTime(ToDateTime(aBuffer.aTimestamp));
				else
					pVariable->setNull();
			}
			aStatus.Clear();

		} break;
		default:
			DBG_ERROR("SdbODBCCursor: interner Fehler: unbekannter SDB-Datentyp"); break; // Unbekannter Datenbanktyp
	}

	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::PutData(const SdbColumn& rCol, const ODbVariantRef& pVariable)
{
	DBG_CHKTHIS(SdbODBCCursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	RETCODE nRetcode;

#define PUTDATA(rgbValue,cbValue) \
	{ \
		nRetcode = PutDataImpl((rgbValue),(cbValue)); \
	    SetStatus(nRetcode,m_aStatementHandle); \
		if (aStatus.IsError()) return FALSE; \
	}

	try
	{
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_UNKNOWN:
			case SDB_DBTYPE_LONGVARCHAR:
			case SDB_DBTYPE_LONGVARBINARY:
				// if the driver does not support long data len we send the data in chunks
				if (!GetODBCConnection()->NeedLongDataLen())
				{
					// but the data in chunks as Db2 can not get the data in one block
					// first get the data and the complete length and than send the data in chunks
					String aString;
					INT32 nLen  = 0;
					const sal_Int8* pData = NULL;
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
					{
						const ::staruno::Sequence<sal_Int8>& rData = pVariable->getBytes();
						nLen  = rData.getLength();
						pData = rData.getConstArray();
					}
					else
					{
						if (!pVariable->isNull())
						{
							try
							{
								aString = pVariable->toString().getStr();
								//	aString.Convert(CHARSET_SYSTEM,GetConnection()->GetCharacterSet());
							}
							catch ( ... )
							{
								String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
								strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
								aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());
								return FALSE;
							}

						}
						nLen = aString.Len();
						pData = (const sal_Int8*) aString.GetBuffer();
					}

#define MAX_BUFFER_SIZE 1024L
					// sending the data in 4k blocks
					while ((nRetcode = PutDataImpl((void*)pData, min(nLen, MAX_BUFFER_SIZE))) == 0
							&& nLen > MAX_BUFFER_SIZE)
					{
						pData += MAX_BUFFER_SIZE;
						nLen -= MAX_BUFFER_SIZE;
					}

					SetStatus(nRetcode);
				}
				else if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
				{
					const ::staruno::Sequence<sal_Int8>& rData = pVariable->getBytes();
					PUTDATA((void *)rData.getConstArray(),rData.getLength());
				}
				else
				{
					if(GetODBCConnection()->m_bUserWChar)
					{
						rtl::OUString aString;
						if (!pVariable->isNull())
							aString = pVariable->toString();
						PUTDATA((void *)aString.getStr(),sizeof(sal_Unicode)*aString.getLength());
					}
					else
					{
						ByteString aString;
						if (!pVariable->isNull())
							aString = ByteString(pVariable->toString().getStr(),gsl_getSystemTextEncoding());
							
						PUTDATA((void *)((const char *)aString.GetBuffer()),aString.Len());
					}
				}
				break;
		}
	}
	catch ( ... )
	{
		String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
		strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
		aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());
		return FALSE;
	}

	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
void SdbODBCCursor::FixResultRow()
{
	DBG_ASSERT(aResultRow.Is(), "Keine Ergebniszeile vorhanden!!");
	DBG_CHKTHIS(SdbODBCCursor,NULL);

	SdbConnection* pConnection = GetConnection();
	UINT16 nCount(aResultColumns->Count());
	for (UINT16 i=1; i < nCount; i++) // Wird BOOKMARK-Spalte verwendet
	{
		const ODbVariantRef& pVal = (*aResultRow)[i];
		if (!pVal.is())
			continue;	// nicht gebunden
		GetData(*(*aResultColumns)[i], pVal, i);
	}
}

//-----------------------------------------------------------------------------
BOOL SdbODBCCursor::DetermineRowCountImpl(INT32&)
{
	return FALSE;
}

