/*************************************************************************
 *
 *  $RCSfile: odbcidx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:46 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbdbimp.hxx"
#ifndef _SDB_ODBCCONN_HXX
#include "odbcconn.hxx"
#endif

#ifndef _SDB_ODBCIDX_HXX
#include "odbcidx.hxx"
#endif

#ifndef _SDB_ODBCTAB_HXX
#include "odbctab.hxx"
#endif

#ifndef _SDB_ODBCCURS_HXX
#include "odbccurs.hxx"
#endif

#include "sdbresid.hrc"

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASETABLEDEFINITION_HPP_
#include <com/sun/star/data/DatabaseTableDefinition.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASESQLCONFORMANCE_HPP_
#include <com/sun/star/data/DatabaseSQLConformance.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

DBG_NAME(SdbODBCIndex);

//-----------------------------------------------------------------------------
SdbODBCIndex::SdbODBCIndex(SdbODBCTable* pRefTable)
			 :SdbIndex(pRefTable){}


//-----------------------------------------------------------------------------
BOOL SdbODBCIndex::OpenImpl()
{
	SdbTable* pTable = GetTable();
	SdbODBCCursor* pCursor =
		(SdbODBCCursor*)pTable->GetConnection()->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY|SDB_READONLY);
	pCursor->AddRef();

	const sal_Unicode* pQualifier = NULL;
	if (pTable->QualifierName().Len())
		pQualifier = pTable->QualifierName().GetBuffer();

	const sal_Unicode* pOwner = NULL;
	if (pTable->OwnerName().Len())
		pOwner = pTable->OwnerName().GetBuffer();

	pCursor->OpenIndices(pQualifier,pOwner,pTable->Name().GetBuffer());
	aStatus = pCursor->Status();
	/*	Reihenfolge
		1	TABLE_QUALIFIER
		2	TABLE_OWNER
		3	TABLE_NAME
		4	NON_UNIQUE
		5	INDEX_QUALIFIER
		6	INDEX_NAME
		7	TYPE
		8	SEQ_IN_INDEX
		9	COLUMN_NAME
		10	COLLATION
	*/
	BOOL bCase = (pTable->GetConnection()->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;
	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		ODbVariantRef pQualifier = rRow[5];
		ODbVariantRef pName = rRow[6];
		ODbVariantRef pType = rRow[7];
		if (pType.is() && !pType->isNull() && pType->getINT16() != SQL_TABLE_STAT &&
			(!pQualifier.is() || pQualifier->isNull() ||
			(bCase ? pQualifier->getString().getStr() == aQualifierName : pQualifier->getString().equalsIgnoreCase(aQualifierName))) &&
			pName.is() && !pName->isNull() &&
			(bCase ? pName->getString().getStr() == aName : pName->getString().equalsIgnoreCase(aName)))
		{
			ODbVariantRef pValue = rRow[4];
			if (pValue.is() && !pValue->isNull() && pValue->getINT16() == 0)
				SetStyle(SDB_IDX_UNIQUE);

			pValue = rRow[9];
			const SdbColumn* pColumn = NULL;
			if (pValue.is() && !pValue->isNull())
				pColumn = pTable->OriginalColumns()->Column(pValue->getString().getStr());

			if (pColumn)
			{
				SdbColumn* pNewCol = pTable->CreateColumn(*pColumn);
				aOriginalColumns->AddColumn(pNewCol);
				pValue = rRow[10];
				if (pValue.is() && !pValue->isNull() && pValue->getString() == ::rtl::OUString::createFromAscii("D"))
					pNewCol->SetFlag(SDB_FLAGS_DESCENDING);
			}
			else
			{
				String aText = String(SdbResId(STR_STAT_INDEX_COLUMN_NOT_FOUND));
				aText.SearchAndReplace(String::CreateFromAscii("#"),pValue->getString().getStr());
				aText.SearchAndReplace(String::CreateFromAscii("%"),pTable->GetFullName());
				aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
				break;
			}
		}
		pCursor->Next();
	}

	pCursor->Close();
/*		1 	PKTABLE_CAT
		2	PKTABLE_SCHEM
		3	PKTABLE_NAME
		4	PKCOLUMN_NAME
		5 	FKTABLE_CAT
		6 	FKTABLE_SCHEM
		7 	FKTABLE_NAME
		8 	FKCOLUMN_NAME
		9 	KEY_SEQ
		10 	UPDATE_RULE
		11 	DELETE_RULE
		12 	FK_NAME			// constraint name
		13  PK_NAME
		14  DEFERRABILITY
*/

	pCursor->OpenForeignKeys(NULL,pTable->OwnerName().GetBuffer(),pTable->Name().GetBuffer(),NULL,NULL,NULL);
	aStatus = pCursor->Status();
	if(aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		ODbVariantRef pFkKeyName = rRow[12];
		if (pFkKeyName.is() && !pFkKeyName->isNull() && pFkKeyName->getString().getStr() == aName)
			bSystemReadOnly = TRUE;
		else
		{

		}
	}
	pCursor->ReleaseRef();
	if(!bSystemReadOnly && Name().Len() && (((SdbODBCTable*)pTable)->aPKeyName == Name()))
		bSystemReadOnly = TRUE;
	if (aStatus.IsNoDataFound())
		aStatus.Clear();
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCIndex::DropImpl()
{
	SdbTable* pTable = GetTable();
	SdbConnection* pConnection = GetConnection();
	String aStatement;

	// mehrere Versuche, da es bei jeder Datenbank unterschiedlich laeuft
	// nachteil, keine genaue Fehlermeldung
	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData = pConnection->GetMetaData();
	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	sal_Int16 nDefInfo;
	rMetaData->getInfo(::stardata::DatabaseInfo::TABLE_DEFINITION) >>= nDefInfo;
	if ((nDefInfo & ::stardata::DatabaseTableDefinition::ALTER_ADD_COLUMN) ||
		(nDefInfo & ::stardata::DatabaseTableDefinition::ALTER_DROP_COLUMN))
	{
		aStatement.AssignAscii("ALTER TABLE ");
		if (pTable->QualifierName().Len() > 0 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_TABLE))
		{
			aStatement += pConnection->IdentifierQuotation(pTable->QualifierName());
			aStatement += pConnection->GetQualifierSeparator();
		}
		if (pTable->OwnerName().Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
		{
			aStatement += pConnection->IdentifierQuotation(pTable->OwnerName());
			aStatement += '.';
		}
		aStatement += pConnection->IdentifierQuotation(pTable->Name());
		aStatement.AppendAscii(" DROP CONSTRAINT ");
		aStatement += pConnection->IdentifierQuotation(aName);

		if (pConnection->ExecuteSQL(aStatement))
			return TRUE;
	}

	if (aQualifierName.Len())
	{
		aStatement.AssignAscii("DROP INDEX ");
		aStatement += pConnection->IdentifierQuotation(aQualifierName);
		aStatement += '.';
		aStatement += pConnection->IdentifierQuotation(aName);
		if (pConnection->ExecuteSQL(aStatement))
			return TRUE;
	}

	aStatement.AssignAscii("DROP INDEX ");
	if (pTable->OwnerName().Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aStatement += pConnection->IdentifierQuotation(pTable->OwnerName());
		aStatement += '.';
	}
	aStatement += pConnection->IdentifierQuotation(pTable->Name());
	aStatement += '.';
	aStatement += pConnection->IdentifierQuotation(aName);
	if (pConnection->ExecuteSQL(aStatement))
		return TRUE;

	aStatement.AssignAscii("DROP INDEX ");
	if (pTable->OwnerName().Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aStatement += pConnection->IdentifierQuotation(pTable->OwnerName());
		aStatement += '.';
	}
	aStatement += pConnection->IdentifierQuotation(aName);
	if (!pConnection->ExecuteSQL(aStatement))
	{

		String aText = String(SdbResId(STR_STAT_ODBC_INDEX_NOTDELETED));
		aText.SearchAndReplace(String::CreateFromAscii("#"),aName);
		aStatus.Set(SDB_STAT_ERROR,
					String::CreateFromAscii("01000"),
					aStatus.CreateErrorMessage(aText),
					0, String() );
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCIndex::CreateImpl()
{
	DBG_ASSERT(aColumns->Count() > 1, "No column for create");

	SdbTable* pTable = GetTable();
	SdbConnection* pConnection = GetConnection();

	String aStatement;
	aStatement.AppendAscii("CREATE ");
	if (IsUnique())
		aStatement.AppendAscii("UNIQUE ");

	aStatement.AppendAscii("INDEX ");
	// wenn ein Owner verwendet wird in der Indexdefinition
	// dann wird dieser als Index Qualifizierer genommen
	// so funktioniert das bei db2
	if (aQualifierName.Len())
	{
		aStatement += pConnection->IdentifierQuotation(aQualifierName);
		aStatement += '.';
	}

	aStatement += pConnection->IdentifierQuotation(aName);
	aStatement.AppendAscii(" ON ");
	aStatement += pConnection->QuoteTableName(pTable->QualifiedName());

	aStatement.AppendAscii("(");
	for (UINT16 i = 1; i < aColumns->Count(); i++)
	{
		aStatement += pConnection->IdentifierQuotation((*aColumns)[i]->GetName());
		if ((*aColumns)[i]->IsDescending())
			aStatement.AppendAscii(" DESC ");
		aStatement.AppendAscii(",");
	}

	aStatement.EraseTrailingChars(',');
	aStatement.AppendAscii(")");

	if (!pConnection->ExecuteSQL(aStatement))
		aStatus = pConnection->Status();

	return aStatus.IsSuccessful();
}


