/*************************************************************************
 *
 *  $RCSfile: odbctab.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#include "odbctab.hxx"
#include "odbccurs.hxx"
#include "odbcconn.hxx"
#include "odbcidx.hxx"

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _SDB_ODBCMD_HXX
#include <odbcmd.hxx>
#endif

#include "sdbresid.hrc"

#ifndef _COM_SUN_STAR_DATA_DATABASETABLEDEFINITION_HPP_
#include <com/sun/star/data/DatabaseTableDefinition.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASESQLCONFORMANCE_HPP_
#include <com/sun/star/data/DatabaseSQLConformance.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEQUALIFIER_HPP_
#include <com/sun/star/data/DatabaseQualifier.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASENKEYRULE_HPP_
#include <com/sun/star/data/DatabaseKeyRule.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

static char __READONLY_DATA privilegeSelect[] = "SELECT";
static char __READONLY_DATA privilegeInsert[] = "INSERT";
static char __READONLY_DATA privilegeUpdate[] = "UPDATE";
static char __READONLY_DATA privilegeDelete[] = "DELETE";
static char __READONLY_DATA privilegeReferences[] = "REFERENCES";


DBG_NAME(SdbODBCTable);

//-----------------------------------------------------------------------------
SdbODBCTable::SdbODBCTable(SdbODBCConnection* pConnection)
			 :SdbTable(pConnection)
{
	DBG_CTOR(SdbODBCTable,NULL);
	aStatus.SetComponent(String::CreateFromAscii("Sdb ODBC Driver"));
}
SdbODBCTable::~SdbODBCTable()
{
	DBG_DTOR(SdbODBCTable,NULL);
};
//-----------------------------------------------------------------------------
// werden in OpenImpl verwendet
static const UINT16 C_COLUMN_NAME				= 4;
static const UINT16 C_DATA_TYPE					= 5;
static const UINT16 C_TYPE_NAME					= 6;
static const UINT16 C_PRECISION					= 7;
static const UINT16 C_LENGTH					= 8;
static const UINT16 C_SCALE						= 9;
static const UINT16 C_RADIX						= 10;
static const UINT16 C_NULLABLE					= 11;
static const UINT16 C_REMARKS					= 12;

static const UINT16 I_NON_UNIQUE				= 4;
static const UINT16 I_INDEX_QUALIFIER			= 5;
static const UINT16 I_INDEX_NAME				= 6;
static const UINT16 I_TYPE						= 7;
static const UINT16 I_COLUMN_NAME				= 9;

static const UINT16 P_COLUMN_NAME				= 4;
static const UINT16 P_PK_NAME					= 6;

static const UINT16 TP_GRANTEE					= 5;
static const UINT16 TP_PRIVILEGE				= 6;

//-----------------------------------------------------------------------------
BOOL SdbODBCTable::OpenImpl()
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	// Hier ist bekannt, dass es sich eigentlich um einen
	// SdbODBCCursor handelt, daher Type Cast:
	SdbODBCConnection* pConnection = GetODBCConnection();
	SdbODBCCursor* pCursor = (SdbODBCCursor *) pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();
	aStatus = pCursor->Status();

	BOOL bCase = pConnection->GetIdentifierCase() == SDB_IC_ALL;

	if (!aStatus.IsSuccessful())
	{
		pCursor->ReleaseRef();
		return FALSE;
	}

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());

	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	const sal_Unicode* pQualifier = NULL;
	if (aQualifierName.Len() && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML))
		pQualifier = aQualifierName.GetBuffer();

	const sal_Unicode* pOwner = NULL;
	if (aOwnerName.Len() && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML))
		pOwner = aOwnerName.GetBuffer();

	// Handelt es sich um eine Tabelle oder View
	pCursor->OpenTables(pQualifier,pOwner,aName.GetBuffer(), String::CreateFromAscii("VIEW").GetBuffer());
	bView = pCursor->IsInRange();
	pCursor->Close();

	// Angaben zu den Columns:
	// Einen ganz speziellen Cursor oeffnen: anstelle von
	// Prepare und Execute wird Columns ausgefuehrt; die
	// Ergebnismenge enthaelt die gewuenschten Informationen
	pCursor->OpenColumns(pQualifier,pOwner,aName.GetBuffer(), NULL);
	aStatus = pCursor->Status();
	if (!aStatus.IsSuccessful())
	{
		if (aStatus.IsNoDataFound())	// Ungueltige Struktur -> Fehler wird in der Basisklasse gehandelt
			aStatus.Clear();
		pCursor->ReleaseRef();
		return FALSE;
	}

	int i = 1;
	String aColumnName;
	SdbDatabaseType eType ;
	while (!pCursor->IsOffRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		// Column-Info aufbauen ...
		aColumnName = rRow[C_COLUMN_NAME]->getString().getStr();
		eType = pConnection->MapType(rRow[C_DATA_TYPE]->getINT16());
		if(eType == SDB_DBTYPE_BIGINT)
			eType = SDB_DBTYPE_DOUBLE;

		UINT32 nLength (0);
		UINT16 nScale (0);
		UINT16 nFlags (0);
		BOOL bNullAllowed = FALSE;
		String aComment;

		ODbVariantRef pValue = rRow[C_PRECISION];
		if (pValue.is() && !pValue->isNull())
			nLength = (UINT32)pValue->getINT32();
		pValue = rRow[C_SCALE];
		if (pValue.is() && !pValue->isNull())
			nScale = (UINT16)pValue->getINT16();
		pValue = rRow[C_NULLABLE];
		if (pValue.is() && !pValue->isNull())
			bNullAllowed = pValue->getINT16() != SQL_NO_NULLS;

		pValue = rRow[C_REMARKS];
		if (pValue.is() && !pValue->isNull())
			aComment = pValue->getString().getStr();

		// Sonderbehandlung fuer Currency ,Autoincrement
		const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rRow[C_TYPE_NAME]->getString().getStr());
		if(!pInfo)
		{
			pInfo = pConnection->GetColumnTypeInfo(eType); // das ist die letzte M"oglichkeit einen Typ zu bestimmen
		}
		DBG_ASSERT(pInfo, "keine TypeInfo");

		// Zusaetzliche Flags:
		// bei unbekannten Typen ist pInfo NULL, z.B. benutzerdefinierten
		if (pInfo)
		{
			if (bNullAllowed && pInfo->bNullable)
				nFlags |= SDB_FLAGS_NULLALLOWED;
			if (pInfo && pInfo->bCurrency)
				eType = SDB_DBTYPE_CURRENCY;
			else if (pInfo->bAutoIncrement)
				nFlags |= SDB_FLAGS_AUTOINCREMENT;
			switch (pInfo->eSearchType)
			{
				case Search_Like_Only  : nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
				case Search_Except_Like: nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				case Search_All: nFlags |= SDB_FLAGS_SEARCH_ALL; break;
			}
		}
		else // keine TypeInfo dann nullable
			nFlags |= SDB_FLAGS_NULLALLOWED;

		SdbColumn* pColumn = CreateColumn(aColumnName,eType,nLength,nScale,nFlags);
		pColumn->SetComment(aComment);

		// ... und in Columns-Struktur einfuegen
		aOriginalColumns->AddColumn(pColumn);
		pCursor->Next();
	}

	// Unique felder finden
	pCursor->Close();
	pCursor->OpenIndices(pQualifier,pOwner,aName.GetBuffer(),TRUE);
	aStatus = pCursor->Status();
	rtl::OUString aIndexName,aIndexQualifier,aNewName;
	// setzen der uniqueflags
	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		if (rRow[I_TYPE]->getINT16() != SQL_TABLE_STAT)
		{
			SdbColumn* pColumn = NULL;
			ODbVariantRef pIndexQualifier = rRow[I_INDEX_QUALIFIER];
			aNewName = rRow[I_INDEX_NAME]->getString();

			if ((bCase ? aIndexName == aNewName : aIndexName.equalsIgnoreCase(aNewName))&&
				(!pIndexQualifier.is() || pIndexQualifier->isNull() ||
				(bCase ? pIndexQualifier->getString() == aIndexQualifier : pIndexQualifier->getString().equalsIgnoreCase(aIndexQualifier))))
				// naechste Spalte des aktuellen Index
				pColumn = (SdbColumn*)(*aOriginalColumns)[rRow[I_COLUMN_NAME]->getString().getStr()];
			else	// neuer Index
			{
				if ((rRow[I_NON_UNIQUE])->getINT16() == 0)
				{
					aIndexName = aNewName;
					if (pIndexQualifier.is() && !pIndexQualifier->isNull())
						aIndexQualifier = pIndexQualifier->getString();
					pColumn = (SdbColumn*)(*aOriginalColumns)[rRow[I_COLUMN_NAME]->getString().getStr()];
				}
			}
			if (pColumn)
				pColumn->SetFlag(SDB_FLAGS_UNIQUE);
		}
		pCursor->Next();
	}

	// Primarykeys auslesen (Funktion muss nicht unbedingt verfuegbar sein)
	// daher wird auch der Fehlerstatus nicht bercksichtigt
	pCursor->Close();
	pCursor->OpenPrimaryKeys(pQualifier,pOwner,aName.GetBuffer());
	aStatus = pCursor->Status();
	// setzen der uniqueflags
	if (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		// Name des PKeys
		ODbVariantRef pPKName = rRow[P_PK_NAME];
		if (pPKName.is() && !pPKName->isNull())
			aPKeyName = pPKName->getString().getStr();
	}

	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		SdbColumn* pColumn = (SdbColumn*)aOriginalColumns->Column(rRow[P_COLUMN_NAME]->getString().getStr());
		if (pColumn)
			pColumn->SetFlag(SDB_FLAGS_PRIMARYKEY);
		pCursor->Next();
	}

	pCursor->Close();

	// setzen der Privileges
	nPrivileges = 0;
	if (!pConnection->IsCapable(SDB_CAP_UPDATABLE))
		nPrivileges = SDB_PR_READ;
	else if (!aOwnerName.Len() || (bCase ? aOwnerName == pConnection->GetUser() : aOwnerName.EqualsIgnoreCaseAscii(pConnection->GetUser())))
	{
		// Tabelle gehoert dem Benutzer
		// der darf alles
		if(!bView)
		{
			nPrivileges |= (SDB_PR_READ	| SDB_PR_INSERT	| SDB_PR_UPDATE
				| SDB_PR_DELETE | SDB_PR_DROP
				| SDB_PR_REFERENCES );

			sal_Int16 nInfo;
			rMetaData->getInfo(::stardata::DatabaseInfo::TABLE_DEFINITION) >>= nInfo;
			if ((nInfo & ::stardata::DatabaseTableDefinition::ALTER_ADD_COLUMN) ||
				(nInfo & ::stardata::DatabaseTableDefinition::ALTER_DROP_COLUMN))
				nPrivileges |= SDB_PR_ALTER;
		}
		else
			nPrivileges |= (SDB_PR_READ	| SDB_PR_DROP);
	}
	else
	{
		pCursor->OpenTablePrivileges(pQualifier,pOwner,aName.GetBuffer());
		aStatus = pCursor->Status();
		if (!pCursor->IsInRange())
		  // gibt es keine, dann lesen default
			nPrivileges = SDB_PR_READ;

		rtl::OUString usUser(pConnection->GetUser());
		String aPrivilege;
		while (aStatus.IsSuccessful() && pCursor->IsInRange())
		{
			ODbRow& rRow = *pCursor->GetRow();
			ODbVariantRef pGrantee = rRow[TP_GRANTEE];
			if (pGrantee.is() && !pGrantee->isNull() &&
				((bCase) ? pGrantee->getString() == usUser : pGrantee->getString().equalsIgnoreCase(usUser)))
			{
				ODbVariantRef pPrivilege = rRow[TP_PRIVILEGE];
				aPrivilege = pPrivilege.is() && !pPrivilege->isNull() ? String(pPrivilege->getString().getStr()) : String();
				if (aPrivilege.EqualsAscii(privilegeSelect))
					nPrivileges |= SDB_PR_READ;
				else if (aPrivilege.EqualsAscii(privilegeInsert))
					nPrivileges |= SDB_PR_INSERT;
				else if (aPrivilege.EqualsAscii(privilegeUpdate))
					nPrivileges |= SDB_PR_UPDATE;
				else if (aPrivilege.EqualsAscii(privilegeDelete))
					nPrivileges |= SDB_PR_DELETE;
				else if (aPrivilege.EqualsAscii(privilegeReferences))
					nPrivileges |= SDB_PR_REFERENCES;

			}
			pCursor->Next();
		}
		pCursor->Close();
	}

	pCursor->OpenSpecialColumns(pQualifier,pOwner,aName);
	aStatus = pCursor->Status();
	//	String aColumnName;
	while (aStatus.IsSuccessful() && pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		SdbColumn* pColumn = (SdbColumn*)(*aOriginalColumns)[rRow[2]->getString().getStr()];
		if(pColumn)
			pColumn->SetFlag(SDB_FLAGS_VERSIONING);
		else
		{
			aColumnName = rRow[2]->getString().getStr();
			SdbDatabaseType eType = pConnection->MapType(rRow[3]->getINT16());
			if(eType == SDB_DBTYPE_BIGINT)
				eType = SDB_DBTYPE_DOUBLE;

			UINT32 nLength (0);
			UINT16 nScale (0);
			UINT16 nFlags (SDB_FLAGS_VERSIONING);
			BOOL bNullAllowed = FALSE;
			String aComment;

			ODbVariantRef pValue = rRow[5];
			if (pValue.is() && !pValue->isNull())
				nLength = (UINT32)pValue->getINT32();
			pValue = rRow[7];
			if (pValue.is() && !pValue->isNull())
				nScale = (UINT16)pValue->getINT32();

			// Sonderbehandlung fuer Currency ,Autoincrement
			const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rRow[4]->getString().getStr());
			if(!pInfo)
			{
				pInfo = pConnection->GetColumnTypeInfo(eType); // das ist die letzte M"oglichkeit einen Typ zu bestimmen
			}
			DBG_ASSERT(pInfo, "keine TypeInfo");

			// Zusaetzliche Flags:
			// bei unbekannten Typen ist pInfo NULL, z.B. benutzerdefinierten
			if (pInfo)
			{
				if (bNullAllowed && pInfo->bNullable)
					nFlags |= SDB_FLAGS_NULLALLOWED;
				if (pInfo && pInfo->bCurrency)
					eType = SDB_DBTYPE_CURRENCY;
				else if (pInfo->bAutoIncrement)
					nFlags |= SDB_FLAGS_AUTOINCREMENT;
				switch (pInfo->eSearchType)
				{
					case Search_Like_Only  : nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
					case Search_Except_Like: nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
					case Search_All: nFlags |= SDB_FLAGS_SEARCH_ALL; break;
				}
			}
			else // keine TypeInfo dann nullable
				nFlags |= SDB_FLAGS_NULLALLOWED;

			SdbColumn* pColumn = CreateColumn(aColumnName,eType,nLength,nScale,nFlags);
			pColumn->SetComment(aComment);

			// ... und in Columns-Struktur einfuegen
			aOriginalColumns->AddColumn(pColumn);
		}
		pCursor->Next();
	}

	pCursor->ReleaseRef();
	if (aStatus.IsNoDataFound())
		aStatus.Clear();
	return aOriginalColumns->Count() > 1;
}

//-----------------------------------------------------------------------------
BOOL SdbODBCTable::DropImpl()
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	SdbConnection* pConnection = GetConnection();

	String aCommand; aCommand.AssignAscii("DROP TABLE ");

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());

	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	if (aQualifierName.Len() > 0 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aCommand += pConnection->IdentifierQuotation(aQualifierName);
		aCommand += pConnection->GetQualifierSeparator();
	}

	if (aOwnerName.Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aCommand += pConnection->IdentifierQuotation(aOwnerName);
		aCommand += '.';
	}
	aCommand += pConnection->IdentifierQuotation(aName);

	pConnection->ExecuteSQL(aCommand);
	aStatus = pConnection->Status();

	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
void SdbODBCTable::CloseImpl() {DBG_CHKTHIS(SdbODBCTable,NULL);}
//-----------------------------------------------------------------------------
BOOL SdbODBCTable::CreateImpl()
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	SdbConnection* pConnection = GetConnection();

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());


	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;
	sal_Bool bNotNullAllowed = rMetaData->isCapable(::stardata::DatabaseCapability::NON_NULLABLE_COLUMNS);

	String aCommand; aCommand.AssignAscii("CREATE TABLE ");

	if (aQualifierName.Len() > 0 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aCommand += pConnection->IdentifierQuotation(aQualifierName);
		aCommand += pConnection->GetQualifierSeparator();
	}

	if (aOwnerName.Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aCommand += pConnection->IdentifierQuotation(aOwnerName);
		aCommand += '.';
	}
	aCommand += pConnection->IdentifierQuotation(aName);
	aCommand += ' ';
	aCommand += '(';

	if (!GenerateAddCommands(aCommand,bNotNullAllowed)) return FALSE;

	sal_Int16 nInfo;
	rMetaData->getInfo(::stardata::DatabaseInfo::SQL_CONFORMANCE) >>= nInfo;

	if (nInfo & ::stardata::DatabaseSQLConformance::CORE)
	{
		String aPColumns;
		for (USHORT i = 1; i < aColumns->Count(); i++)
		{
			if (aColumns->Column(i)->IsPrimaryKey())
			{
				aPColumns += pConnection->IdentifierQuotation(aColumns->Column(i)->GetName());
				aPColumns += ',';
			}
		}
		if (aPColumns.Len())
		{
			// Ein Komma ist sicherlich noch zuviel ...
			aPColumns.SetChar(aPColumns.Len()-1,')');
			aCommand.AppendAscii(" PRIMARY KEY (");
			aCommand += aPColumns;
			aCommand += ',';
		}
	}
	// Ein Komma ist sicherlich noch zuviel ...
	aCommand.SetChar(aCommand.Len() - 1,')');

	pConnection->ExecuteSQL(aCommand);
	aStatus = pConnection->Status();
	if (aStatus.IsSuccessful())
	{
		nPrivileges = SDB_PR_READ | SDB_PR_INSERT | SDB_PR_UPDATE | SDB_PR_DELETE | SDB_PR_DROP;

		sal_Int16 nDefInfo;
		rMetaData->getInfo(::stardata::DatabaseInfo::TABLE_DEFINITION) >>= nDefInfo;
		if ((nDefInfo & ::stardata::DatabaseTableDefinition::ALTER_ADD_COLUMN) ||
			(nDefInfo & ::stardata::DatabaseTableDefinition::ALTER_DROP_COLUMN))
			nPrivileges |= SDB_PR_ALTER | SDB_PR_REFERENCES;
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCTable::GenerateAddCommands(String& rStatement, BOOL bNotNullAllowed)
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	// Welche neuen Columns gibt es jetzt?
	// (Erkennbar an "fehlender" Identitaet (Id == 0))
	for (int i = 1; i < aColumns->Count(); i++)
	{
		const SdbColumn * pColumn = (*aColumns)[i];
		if (!GenerateColumnCommand(rStatement, *pColumn, bNotNullAllowed))
			return FALSE;

		rStatement += ',';
	}
	return TRUE;
}

//-----------------------------------------------------------------------------
BOOL SdbODBCTable::AlterImpl()
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	// Alter wird immer ueber eine Transaktion ausgefuehrt
	SdbConnectionRef xConnection(GetConnection()->GetEnvironment()->OpenConnection(*GetConnection()));
	if (!xConnection.Is())
	{
		aStatus = GetConnection()->GetEnvironment()->Status();
		return FALSE;
	}

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(xConnection->GetMetaData());


	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;
	BOOL bNotNullAllowed = rMetaData->isCapable(::stardata::DatabaseCapability::NON_NULLABLE_COLUMNS);

	xConnection->BeginTrans();
	aStatus = xConnection->Status();
	if (!aStatus.IsSuccessful())
		return FALSE;

	String aStatement;
	String aTableName;


	if (aQualifierName.Len() > 0 && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aTableName += xConnection->IdentifierQuotation(aQualifierName);
		aTableName += xConnection->GetQualifierSeparator();
	}

	if (aOwnerName.Len() > 0 && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_TABLE))
	{
		aTableName += xConnection->IdentifierQuotation(aOwnerName);
		aTableName += '.';
	}
	aTableName += xConnection->IdentifierQuotation(aName);

	// Zunaechst werden die Felder, die nicht mehr benoetigt werden, geloescht.
	UINT16 i;
	for (i = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && i > 0; i--)
	{
		const SdbColumn* pColumn = (*aOriginalColumns)[i];
		const SdbColumn* pRefColumn = (*aColumns)[pColumn->GetName()];
		if (!pRefColumn)
		{
			aStatement.AssignAscii("ALTER TABLE ");
			aStatement += aTableName;
			aStatement.AppendAscii(" DROP ");
			aStatement += xConnection->IdentifierQuotation(pColumn->GetName());

			// wird eine Spalte mit PKeyInfo geloescht ?
			if (aPKeyName.Len() && pColumn->IsPrimaryKey())
			{
				String aStmt; aStmt.AssignAscii("ALTER TABLE ");
				aStmt += aTableName;
				aStmt.AppendAscii(" DROP CONSTRAINT ");
				aStmt += aPKeyName;
				xConnection->ExecuteSQL(aStmt);
				aStatus = xConnection->Status();
				if (!aStatus.IsSuccessful())
					continue;
				else
					aPKeyName.Erase();
			}

			xConnection->ExecuteSQL(aStatement);
			aStatus = xConnection->Status();
			if (aStatus.IsSuccessful())
				aOriginalColumns->RemoveColumn(i);
		}
	}

	// Die Felder, die sich geaendert haben, geloescht
	for (i = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && i > 0; i--)
	{
		const SdbColumn* pColumn = (*aOriginalColumns)[i];
		const SdbColumn* pRefColumn = (*aColumns)[pColumn->GetName()];
		if (pRefColumn && !pColumn->Compare(*pRefColumn, SDB_FLAGS_NULLALLOWED))
		{
			aStatement.AssignAscii("ALTER TABLE ");
			aStatement += aTableName;
			aStatement.AppendAscii(" DROP ");
			aStatement += xConnection->IdentifierQuotation(pColumn->GetName());

			// wird eine Spalte mit PKeyInfo geloescht ?
			if (aPKeyName.Len() && pColumn->IsPrimaryKey())
			{
				String aStmt; aStmt.AssignAscii("ALTER TABLE ");
				aStmt += aTableName;
				aStmt.AppendAscii(" DROP CONSTRAINT ");
				aStmt += aPKeyName;
				xConnection->ExecuteSQL(aStmt);
				aStatus = xConnection->Status();
				if (!aStatus.IsSuccessful())
					continue;
				else
					aPKeyName.Erase();
			}

			xConnection->ExecuteSQL(aStatement);
			aStatus = xConnection->Status();

			if (aStatus.IsSuccessful())
				aOriginalColumns->RemoveColumn(i);
		}
	}

	// und jetzt die neuen bzw. geaenderten hinzufuegen
	for (i = 1; aStatus.IsSuccessful() && i < aColumns->Count(); i++)
	{
		const SdbColumn* pColumn = (*aColumns)[i];
		const SdbColumn* pRefColumn = (*aOriginalColumns)[pColumn->GetName()];
		if (!pRefColumn)
		{
			aStatement.AssignAscii("ALTER TABLE ");
			aStatement += aTableName;
			aStatement.AppendAscii(" ADD ");
			GenerateColumnCommand(aStatement, *pColumn, bNotNullAllowed);

			// gibt es eine neue Schluesselspalte, dann PKey loeschen
			if (aPKeyName.Len() && pColumn->IsPrimaryKey())
			{
				String aStmt; aStmt.AssignAscii("ALTER TABLE ");
				aStmt += aTableName;
				aStmt.AppendAscii(" DROP CONSTRAINT ");
				aStmt += aPKeyName;
				xConnection->ExecuteSQL(aStmt);
				aStatus = xConnection->Status();
				if (!aStatus.IsSuccessful())
					continue;
				else
					aPKeyName.Erase();
			}

			xConnection->ExecuteSQL(aStatement);
			aStatus = xConnection->Status();
			if (aStatus.IsSuccessful())
				aOriginalColumns->AddColumn(CreateColumn(*pColumn));
		}
	}

	// gibt es noch Aenderungen am Schluessel
	for (i = 1; aPKeyName.Len() && aStatus.IsSuccessful() && i < aColumns->Count(); i++)
	{
		const SdbColumn* pColumn = (*aColumns)[i];
		const SdbColumn* pRefColumn = (*aOriginalColumns)[pColumn->GetName()];
		if (pRefColumn && pColumn->IsPrimaryKey() != pRefColumn->IsPrimaryKey())
		{
			String aStmt; aStmt.AssignAscii("ALTER TABLE ");
			aStmt += aTableName;
			aStmt.AppendAscii(" DROP CONSTRAINT ");
			aStmt += aPKeyName;
			xConnection->ExecuteSQL(aStmt);
			aStatus = xConnection->Status();
			if (aStatus.IsSuccessful())
				aPKeyName.Erase();
		}
	}

	if (!aStatus.IsSuccessful())
	{
		xConnection->RollbackTrans();
		goto End;
	}

	if (!aPKeyName.Len()) // muss jetzt noch ein PKey hinzugefuegt werden ?
	{
		String aPColumns;
		for (UINT16 i = 1; i < aColumns->Count(); i++)
		{
			if ((*aColumns)[i]->IsPrimaryKey())
			{
				aPColumns += xConnection->IdentifierQuotation((*aColumns)[i]->GetName());
				aPColumns += ',';
			}
		}
		if (aPColumns.Len())
		{
			// Ein Komma ist sicherlich noch zuviel ...
			aPColumns.SetChar(aPColumns.Len()-1,')');

			String aCommand; aCommand.AssignAscii(" ALTER TABLE ");
			aCommand += aTableName;
			aCommand.AppendAscii(" ADD PRIMARY KEY (");
			aCommand += aPColumns;

			xConnection->ExecuteSQL(aCommand);
			aStatus = xConnection->Status();
		}
	}

	if (!aStatus.IsSuccessful() || !xConnection->CommitTrans())
	{
		aStatus = xConnection->Status();
		xConnection->RollbackTrans();
	}

End: // zum schlu nochmals einlesen
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCTable::GenerateColumnCommand(String& rStatement, const SdbColumn& rColumn, BOOL bNotNullAllowed)
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	SdbConnection* pConnection = GetConnection();
	// Gibt es zu dieser Column eine Typinformationen
	const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rColumn);

	if (!pInfo)
	{
		pInfo = pConnection->GetColumnTypeInfo(rColumn.GetType());
		if(!pInfo)
		{
			DBG_ASSERT(0,"ODBC Table: GetColumnTypeInfo == NULL");
			aStatus.SetDriverNotCapableError();
			return FALSE;
		}
	}

	rStatement += pConnection->IdentifierQuotation(rColumn.GetName());
	rStatement += ' ';
	rStatement += pInfo->aTypeName;

	switch(pInfo->eType)
	{
		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_BINARY:
		case SDB_DBTYPE_VARBINARY:
			// Typen mit Laengenangabe
			if(pInfo->aCreateParams.Len()) // only when the driver accepts create params
			{
				rStatement += '(';
				rStatement += String::CreateFromInt32(rColumn.GetLength());
				rStatement += ')';
			}
			break;
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
			// Typen mit Precision und Scale
			if(pInfo->aCreateParams.Len()) // only when the driver accepts create params
			{
				rStatement += '(';
				rStatement += String::CreateFromInt32(rColumn.GetPrecision());
				rStatement += ',';
				rStatement += String::CreateFromInt32(rColumn.GetScale());
				rStatement += ')';
			}
			break;
	}

	if (!rColumn.IsNullAllowed() && bNotNullAllowed)
		rStatement.AppendAscii(" NOT NULL");

	return TRUE;
}


//-----------------------------------------------------------------------------
SdbIndex* SdbODBCTable::CreateIndexImpl()
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	return new SdbODBCIndex(this);
}

//-----------------------------------------------------------------------------
void SdbODBCTable::FillIndexNames(SvStringsDtor& _rList)
{
	DBG_CHKTHIS(SdbODBCTable,NULL);
	SdbConnection* pConnection = GetConnection();
	BOOL bCase = (pConnection->GetIdentifierCase() & SDB_IC_OBJECT) == SDB_IC_OBJECT;
		// Einlesen aller Indices
	SdbODBCCursor* pCursor = (SdbODBCCursor *) pConnection->CreateCursor(SDB_SNAPSHOT,SDB_FORWARDONLY | SDB_READONLY);
	pCursor->AddRef();

	const sal_Unicode* pQualifier = NULL;
	if (aQualifierName.Len())
		pQualifier = aQualifierName.GetBuffer();

	const sal_Unicode* pOwner = NULL;
	if (aOwnerName.Len())
		pOwner = aOwnerName.GetBuffer();

	pCursor->OpenIndices(pQualifier,pOwner,aName.GetBuffer());
	USHORT i (0);
	rtl::OUString aNewIndexName;
	rtl::OUString aIndexName;
	rtl::OUString ustrPkeyName(aPKeyName);
	while (pCursor->IsInRange())
	{
		ODbRow& rRow = *pCursor->GetRow();
		ODbVariantRef pVal = rRow[I_INDEX_NAME];
		ODbVariantRef pCatalog = rRow[I_INDEX_QUALIFIER];

		if (rRow[I_TYPE]->getINT16() == SQL_TABLE_STAT ||
			pVal->getString() == ustrPkeyName)		// Index des Pkeys rausfiltern
		{
			pCursor->Next();
			continue;
		}
		if (pCatalog.is() && !pCatalog->isNull())
		{
			aNewIndexName = pCatalog->getString();
			aNewIndexName += String::CreateFromAscii(".");
		}
		else
			aNewIndexName = rtl::OUString();
		aNewIndexName += pVal->getString();
		BOOL bNewIndex = (bCase) ? aIndexName == aNewIndexName
								 : aIndexName.equalsIgnoreCase(aNewIndexName);
		if (bNewIndex)
		{
			aIndexName = aNewIndexName;
			_rList.Insert(new String(aIndexName.getStr()), i++);
		}
		pCursor->Next();
	}
	pCursor->ReleaseRef();
	if (aStatus.IsNoDataFound())
		aStatus.Clear();
}

//-----------------------------------------------------------------------------
BOOL SdbODBCTable::RenameImpl(const String& rNewName)
{
	// eine Warnung, dass wir Umbenennen nur durch Kopieren erreichen koennen
	SdbUiActionDescription aAsk;
	aAsk.eType = SdbUiActionDescription::YES_NO;
	aAsk.sMessage = String(SdbResId(RID_STR_WARNING_RENAME));
	SdbUiActionDescription::RETURN eResult = DoUiAction(aAsk);
	if ((eResult != SdbUiActionDescription::RETURN_YES) && (eResult != SdbUiActionDescription::RETURN_OK))
	{
		aStatus.Set(SDB_STAT_CANCELED);
		return FALSE;
	}

	DBG_CHKTHIS(SdbODBCTable,NULL);
	SdbConnection* pConnection = GetConnection();

	String aOldName(aName);
	aName = rNewName;

	// Neue Tabelle unter neuen Namen anlegen
	if (!CreateImpl())
	{
		aName = aOldName;
		return FALSE;
	}

	::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(pConnection->GetMetaData());

	sal_Int16 nCatalogUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::CATALOG_USAGE) >>= nCatalogUsage;
	sal_Int16 nSchemaUsage;
	rMetaData->getInfo(::stardata::DatabaseInfo::SCHEMA_USAGE) >>= nSchemaUsage;

	String aQualified;

	if (aQualifierName.Len() && (nCatalogUsage & ::stardata::DatabaseQualifier::IN_DML))
	{
		aQualified += pConnection->IdentifierQuotation(aQualifierName);
		aQualified += pConnection->GetQualifierSeparator();
	}

	if (aOwnerName.Len() && (nSchemaUsage & ::stardata::DatabaseQualifier::IN_DML))
	{
		aQualified += pConnection->IdentifierQuotation(aOwnerName);
		aQualified += '.';
	}

	// Daten in die neue Tabelle kopieren
	String aStatement; aStatement.AssignAscii("INSERT INTO ");
	aStatement += aQualified;
	aStatement += pConnection->IdentifierQuotation(rNewName);
	aStatement.AppendAscii(" SELECT * FROM ");
	aStatement += aQualified;
	aStatement += pConnection->IdentifierQuotation(aOldName);

	if (!pConnection->ExecuteSQL(aStatement)) // bei Mierfolg alten Zustand wiederherstellen
	{
		SdbStatus aExecuteStatus = pConnection->Status();
		DropImpl();
		aStatus = aExecuteStatus;
		aName = aOldName;
		return FALSE;
	}

	// Alte Tabelle wegwerfen
	aName = aOldName;

	return DropImpl();
}

