/*************************************************************************
 *
 *  $RCSfile: odbc2curs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "sdbresid.hrc"
#include "odbc2curs.hxx"
#include "odbc2conn.hxx"

#include <stdlib.h>

#ifndef _SDB_TYPECONV_HXX
#include <typeconv.hxx>
#endif

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEOUTERJOINSUPPORT_HPP_
#include <com/sun/star/data/DatabaseOuterJoinSupport.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;

#include <sdbtools.hxx>

DBG_NAME(SdbODBC2Cursor);
//-----------------------------------------------------------------------------
SdbODBC2Cursor::SdbODBC2Cursor(SdbODBC2Connection* pConnection,
							 SdbCursorType eType,
							 UINT16 nOpt)
			  :SdbODBCCursor(pConnection, eType, nOpt)
{
	DBG_CTOR(SdbODBC2Cursor,NULL);

	if (pConnection->HasFreeStatement())
	{
		RETCODE rRetcode = NSQLAllocStmt(pConnection->ConnectionHandle(),&m_aStatementHandle);
		SetStatus(rRetcode,m_aStatementHandle);

		if (aStatus.IsError())
		{
			// es ist nicht moeglich einen weiteree Handle zu bekommen,
			// daher versuchen wir einfach noch eine Verbindung aufzumachen
			aStatus.Clear();
			m_aStatementHandle = SQL_NULL_HSTMT;
		}
	}

	// Wenn die Verbindung bereits alle Statements verbraucht hat, dann eine neue extra fr den Cursor
	if (m_aStatementHandle == SQL_NULL_HSTMT)
	{
		HENV m_aEnvironmentHandle = pConnection->EnvironmentHandle();
		DBG_ASSERT(m_aEnvironmentHandle != SQL_NULL_HENV,"SdbODBC2Connection: interner Fehler: EnvironmentHandle ungueltig");

		// Connection allozieren extra fr den Cursor
		// (Eine ODBC-Connection pro SdbConnection).
		SetStatus(NSQLAllocConnect(m_aEnvironmentHandle,&m_aConnectionHandle),m_aConnectionHandle);
		if (aStatus.IsError()) return;

		RETCODE rRetcode = SdbODBC2Connection::OpenConnection(m_aConnectionHandle, pConnection->ConnectStr(), TRUE);
		SetStatus(rRetcode,m_aStatementHandle);
		if (aStatus.IsError()) return;

		// Statement allozieren (ein ODBC-Statement pro SdbCursor).
		rRetcode = NSQLAllocStmt(m_aConnectionHandle,&m_aStatementHandle);
		SetStatus(rRetcode,m_aStatementHandle);
		if (aStatus.IsError()) return;
	}

	DetermineOptions(pConnection);
	SetOptions(pConnection);
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::DetermineOptions(SdbODBCConnection* pConnection)
{
	// Cursor kann nur in eine Richtung bewegt werden
	if ((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY ||
		!IsCapable(SDB_CAP_FETCH_FIRST) ||
		!IsCapable(SDB_CAP_UPDATE_POSITIONED))
	{
		nCursorType = SQL_CURSOR_FORWARD_ONLY;	// default
	}
	else if (eCursorType == SDB_DYNAMIC || eCursorType == SDB_KEYSET)
		nCursorType = SQL_CURSOR_KEYSET_DRIVEN;
	else
		nCursorType = SQL_CURSOR_STATIC;

	// Updatefaehigkeiten und Cursortyp setzen
	nConcurrency = SQL_CONCUR_READ_ONLY;
	if (IsCapable(SDB_CAP_UPDATE_POSITIONED) && nCursorType == SQL_CURSOR_KEYSET_DRIVEN)
	{
		if (pConnection->nDriverConcurrency & SQL_SCCO_OPT_ROWVER)
			nConcurrency = SQL_CONCUR_ROWVER;
		else if (pConnection->nDriverConcurrency & SQL_SCCO_OPT_VALUES)
			nConcurrency = SQL_CONCUR_VALUES;
		else if (pConnection->nDriverConcurrency & SQL_SCCO_LOCK)
			nConcurrency = SQL_CONCUR_LOCK;
	}
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::SetOptions(SdbODBCConnection* pConnection)
{
	if (nCursorType != SQL_CURSOR_FORWARD_ONLY)	// default
	{
		SetStatus(NSQLSetStmtOption(m_aStatementHandle,
									SQL_CURSOR_TYPE,
									nCursorType),m_aStatementHandle);

		if (nConcurrency != SQL_CONCUR_READ_ONLY)
			SetStatus(NSQLSetStmtOption(m_aStatementHandle,
										SQL_CONCURRENCY,
										nConcurrency),m_aStatementHandle);
	}
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::SetStatus(INT16 nRetcode, void* hstmt,const String& rErrorInfo)
{
	SdbODBCConnection *pCon = GetODBCConnection();
	if (m_aConnectionHandle != SQL_NULL_HDBC)
		SdbODBC2_SetStatus(aStatus,nRetcode,pCon->m_aEnvironmentHandle,m_aConnectionHandle,hstmt,rErrorInfo);
	else
		SdbODBC2_SetStatus(aStatus,nRetcode,pCon->m_aEnvironmentHandle,pCon->ConnectionHandle(),hstmt,rErrorInfo);
}

//-----------------------------------------------------------------------------
SdbODBC2Cursor::~SdbODBC2Cursor()
{
	DBG_DTOR(SdbODBC2Cursor,NULL);
#if SUPD>557
	// (RemovingFromParent isn't virtual in 557)
	DBG_ASSERT(GetParent() || m_aStatementHandle == SQL_NULL_HSTMT, "SdbODBC2Cursor::~SdbODBC2Cursor : RemovingFromParent should have been called if I have no parent anymore !");
#endif
	if (m_aStatementHandle != SQL_NULL_HSTMT)
	{
		NSQLFreeStmt(m_aStatementHandle,SQL_DROP);
		m_aStatementHandle = SQL_NULL_HSTMT;

		if (m_aConnectionHandle != SQL_NULL_HDBC)	// extra Verbindung
		{
			NSQLDisconnect(m_aConnectionHandle);
	#ifdef ODBCIMP
			if (pSQLFreeConnect)
				NSQLFreeConnect(m_aConnectionHandle);
	#else
			NSQLFreeConnect(m_aConnectionHandle);
	#endif
		}
	}
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::RemovingFromParent()
{
	DBG_ASSERT(!IsOpen(), "SdbODBC2Cursor::RemovingFromParent : expected to be called _after_ Close !");

	// drop our statement. We can't do this later as the connection - which is our parent - may be destroyed then.
	if (m_aStatementHandle != SQL_NULL_HSTMT)
	{
		NSQLFreeStmt(m_aStatementHandle,SQL_DROP);
		m_aStatementHandle = SQL_NULL_HSTMT;

		if (m_aConnectionHandle != SQL_NULL_HDBC)	// extra Verbindung
		{
			NSQLDisconnect(m_aConnectionHandle);
	#ifdef ODBCIMP
			if (pSQLFreeConnect)
				NSQLFreeConnect(m_aConnectionHandle);
	#else
			NSQLFreeConnect(m_aConnectionHandle);
	#endif
		}
	}
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::AllocBuffer(void**& pDataBuffer, INT32*& pLenBuffer, const SdbColumns& rCols, BOOL bBindBookmark)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);
	UINT16 nCount(rCols.Count());
	if (nCount <= 1)		// keine columns
		return;

	DBG_ASSERT(pDataBuffer == NULL,"Interner Fehler");
	pDataBuffer = (void**) new char*[nCount];
	pLenBuffer = new INT32[nCount];
	memset(pLenBuffer, 0, nCount * sizeof(INT32));
	memset(pDataBuffer, 0, nCount * sizeof(char*));

	for (UINT16 i= bBindBookmark ? 0 : 1; i < nCount; i++)
	{
		const SdbColumn& rCol	= *rCols[i];
		INT32& rLength			= pLenBuffer[i];
		rLength					= rCol.GetLength();

		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					pDataBuffer[i] = new rtl::OUString;
				else
					pDataBuffer[i] = new String;
				break;
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
				rLength += 2;
				if(GetODBCConnection()->m_bUserWChar)
					pDataBuffer[i] = new rtl::OUString;
				else
					pDataBuffer[i] = new String;
				break;
			case SDB_DBTYPE_DOUBLE:
				rLength = sizeof(double);
				pDataBuffer[i] = new double;
				*(double*)pDataBuffer[i] = 0.0;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					pDataBuffer[i] = new sal_Unicode[2];
				else
					pDataBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_UNKNOWN:
				rLength = 0;
				pDataBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_DATE:
				rLength = sizeof(DATE_STRUCT);
				pDataBuffer[i] = new DATE_STRUCT;
				break;
			case SDB_DBTYPE_TIME:
				rLength = sizeof(TIME_STRUCT);
				pDataBuffer[i] = new TIME_STRUCT;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				rLength = sizeof(TIMESTAMP_STRUCT);
				pDataBuffer[i] = new TIMESTAMP_STRUCT;
				break;
			case SDB_DBTYPE_BOOLEAN:
				rLength = sizeof(INT8);
				pDataBuffer[i] = new INT8;
				*(INT8*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				rLength = sizeof(INT16);
				pDataBuffer[i] = new INT16;
				*(INT16*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_INTEGER:
				rLength = sizeof(INT32);
				pDataBuffer[i] = new INT32;
				*(INT32*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_BOOKMARK:
				pDataBuffer[i] = new INT32;
				*(INT32*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_REAL:
				rLength = sizeof(float);
				pDataBuffer[i] = new float;
				*(float*)pDataBuffer[i] = (float)0;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				pDataBuffer[i] = new BYTE[rLength];
				break;
		}
	}
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::FreeBuffer(void**& pDataBuffer, INT32*& pLenBuffer, const SdbColumns& rCols, BOOL bBindBookmark)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);
	UINT16 nCount = rCols.Count();
	if (nCount <= 1) return;

	delete pLenBuffer;
	pLenBuffer = NULL;

	if (pDataBuffer == NULL)
		return;

	for (UINT16 i= bBindBookmark ? 0 : 1; i < nCount; i++)
	{
		const SdbColumn& rCol = *rCols[i];
		void* pData = pDataBuffer[i];
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					delete (rtl::OUString*)pData;
				else
					delete (String*)pData;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					delete (sal_Unicode*)pData;
				else
					delete (char*)pData;
				break;
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_UNKNOWN:
				delete (char*)pData;
				break;
			case SDB_DBTYPE_INTEGER:
			case SDB_DBTYPE_BOOKMARK:
				delete (INT32*)pData;
				break;
			case SDB_DBTYPE_BOOLEAN:
				delete (INT8*)pData;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				delete (INT16*)pData;
				break;
			case SDB_DBTYPE_REAL:
				delete (float*)pData;
				break;
			case SDB_DBTYPE_DOUBLE:
				delete (double*)pData;
				break;
			case SDB_DBTYPE_DATE:
				delete (DATE_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIME:
				delete (TIME_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				delete (TIMESTAMP_STRUCT*)pData;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
				delete (BYTE *) pData;
				break;
			default:
				DBG_ERROR("Fehlerhafter DatenTyp");
		}
	}

	__DELETE(nCount) pDataBuffer;
	pDataBuffer = NULL;
}


//-----------------------------------------------------------------------------
void SdbODBC2Cursor::CancelExecution()
{
	NSQLCancel(m_aStatementHandle);
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::PrepareODBC()
{
	ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
	RETCODE nRetcode = NSQLPrepare(m_aStatementHandle,
						   (SDB_ODBC_CHAR *) aSql.GetBuffer(),
							aSql.Len());

	SetStatus(nRetcode,m_aStatementHandle,aSQLStatement);
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::DescribeResult()
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aResultColumns.Is(), "keine Ergebnisbeschreibung");
	// Bookmark-Column ist immer da (schon durch den Konstruktor erzeugt).

	SdbODBCConnection* pConnection = GetODBCConnection();

	if (aResultColumns->Count() > 1 || !pConnection)
		return;

	// Alle ResultColumns ansehen
	SWORD nNumResultCols=0;
	RETCODE nRetcode = NSQLNumResultCols(m_aStatementHandle,&nNumResultCols);
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	SWORD pcbColName;			// Laenge des Names
	SWORD pfSqlType;			// SQL-Datentyp
	UDWORD pcbColDef;			// Precision (ggf. 0)
	SWORD  pibScale;				// Scale (ggf. 0)
	SWORD  pfNullable;			// NULL erlaubt?
	INT16  nFlags = 0;
	SdbDatabaseType eType;
	ByteString aColumnName;			// Spaltenname
	SDWORD nNumTemp = nNumResultCols; // hack um sicher zustellen das dieser Wert nicht mehr ver"andert werden kann
	SDB_ODBC_CHAR pTmp[DESC_COL_NAME_LEN];
	for (SWORD i = 1; i <= nNumTemp; i++)
	{
		// Informationen von SQLDescribeCol:
		// erstmal die L"ange des Spaltennamens

		nFlags = pibScale = pfNullable = pcbColName = 0;
		pcbColDef = 0l;
		pTmp[0] = ' ';
		pTmp[1] = '\0';
		nRetcode =NSQLDescribeCol(m_aStatementHandle,
								   i,
								   pTmp,
								   DESC_COL_NAME_LEN,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
		SetStatus(nRetcode,m_aStatementHandle);
		if(aStatus.SQLState() == aTruncState)
		{
			// first try mit Lnge == 0
			nRetcode =NSQLDescribeCol(m_aStatementHandle,
								   i,
								   pTmp,
								   0,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
			if(!pcbColName) // second try da pcbColName == 0
			{
				pTmp[0] = ' ';
				pTmp[1] = '\0';
				nRetcode =NSQLDescribeCol(m_aStatementHandle,
								   i,
								   pTmp,
								   1,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
			}
			DBG_ASSERT(pcbColName>0,"ODBC: pcbColName <= 0. Treiber verhlt sich nicht wie blich! ;-(");
			SDB_ODBC_CHAR* pStr = (SDB_ODBC_CHAR*)aColumnName.AllocBuffer(pcbColName+1);
			nRetcode =NSQLDescribeCol(m_aStatementHandle,
									   i,
									   pStr,
									   pcbColName+1,
									   &pcbColName,
									   &pfSqlType,
									   &pcbColDef,
									   &pibScale,
									   &pfNullable
									 );
			aColumnName.ReleaseBufferAccess();  // wieder freigeben
			SetStatus(nRetcode,m_aStatementHandle);
		}
		else if (aStatus.IsSuccessful())
			aColumnName = (char*)pTmp;

		if (!aStatus.IsSuccessful())
			return;



		switch(pfSqlType)
		{
			case SQL_WCHAR:
			case SQL_CHAR:			eType = SDB_DBTYPE_CHAR; break;
			case SQL_WVARCHAR:
			case SQL_VARCHAR: 		eType = SDB_DBTYPE_VARCHAR; break;
			case SQL_WLONGVARCHAR:
			case SQL_LONGVARCHAR: 	eType = SDB_DBTYPE_LONGVARCHAR; break;
			case SQL_DECIMAL:		eType = SDB_DBTYPE_DECIMAL; break;
			case SQL_NUMERIC:		eType = SDB_DBTYPE_NUMERIC; break;
			case SQL_BIT:
				eType = SDB_DBTYPE_BOOLEAN;
				if(!pcbColDef)
				{
					pcbColDef = 1; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_TINYINT:
				eType = SDB_DBTYPE_TINYINT;
				if(!pcbColDef)
				{
					pcbColDef = 3; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_SMALLINT:
				eType = SDB_DBTYPE_SMALLINT;
				if(!pcbColDef)
				{
					pcbColDef = 5; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_INTEGER:
				eType = SDB_DBTYPE_INTEGER;
				if(!pcbColDef)
				{
					pcbColDef = 10; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			//case SQL_BIGINT:
				//eType = SDB_DBTYPE_BIGINT; break;
			case SQL_REAL:
				eType = SDB_DBTYPE_REAL;
				if(!pcbColDef)
				{
					pcbColDef = 7; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_FLOAT:
			case SQL_DOUBLE:
			case SQL_BIGINT:
				eType = SDB_DBTYPE_DOUBLE;
				if(!pcbColDef)
				{
					pcbColDef = 15; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_BINARY:		eType = SDB_DBTYPE_BINARY; break;
			case SQL_VARBINARY: 	eType = SDB_DBTYPE_VARBINARY; break;
			case SQL_LONGVARBINARY:	eType = SDB_DBTYPE_LONGVARBINARY; break;
			case SQL_DATE: 			eType = SDB_DBTYPE_DATE; break;
			case SQL_TIME: 			eType = SDB_DBTYPE_TIME; break;
			case SQL_TIMESTAMP: 	eType = SDB_DBTYPE_TIMESTAMP; break;
			default:
				eType = SDB_DBTYPE_UNKNOWN; // wird nicht gebunden
		}
		if (pfNullable != SQL_NO_NULLS) nFlags |= SDB_FLAGS_NULLALLOWED;

		if (!(nOptions & SDB_MINIMUM_DESC) && eType != SDB_DBTYPE_UNKNOWN)
		{
			SDWORD nExtAttrib;	// erweitere Attribute
			if (NSQLColAttributes(m_aStatementHandle,		// ist die Column updatebar?
									 i,
									 SQL_COLUMN_UPDATABLE,
									 NULL,
									 0,
									 NULL,
									 &nExtAttrib
									) == SQL_SUCCESS)
			{
				if (nExtAttrib != SQL_ATTR_READONLY)
					nFlags |= SDB_FLAGS_UPDATEALLOWED;
			}

			if (NSQLColAttributes(m_aStatementHandle,	    // handelt es sich um ein MoneyFelt
										 i,
										 SQL_COLUMN_MONEY,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib) eType = SDB_DBTYPE_CURRENCY;
			}

			if (NSQLColAttributes(m_aStatementHandle,		// ist die Column AUTO_INCREMENT?
										 i,
										 SQL_COLUMN_AUTO_INCREMENT,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib)
				{
					nFlags |= SDB_FLAGS_AUTOINCREMENT;
					nFlags &= ~SDB_FLAGS_UPDATEALLOWED;	    // darf nicht updatebar sein
				}
			}

			if (NSQLColAttributes(m_aStatementHandle,		// ist die Column SEARCHABLE?
										 i,
										 SQL_COLUMN_SEARCHABLE,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				switch (nExtAttrib)
				{
					case SQL_SEARCHABLE:		nFlags |= SDB_FLAGS_SEARCH_ALL; break;
					case SQL_LIKE_ONLY:			nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
					case SQL_ALL_EXCEPT_LIKE:	nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				}
			}
		}

		// Column-Beschreibung hinzufuegen.
		// existiert Name  in aResultColumns ?
		String aAlias(aColumnName,gsl_getSystemTextEncoding());
		const SdbColumn* pPrevCol = aResultColumns->Column(aAlias);

		ULONG nExprCnt(0);
		while(pPrevCol)
		{
			(aAlias = String(aColumnName,gsl_getSystemTextEncoding())) += String::CreateFromInt32(++nExprCnt);
			pPrevCol = aResultColumns->Column(aAlias);
		}
		SdbColumn * pColumn = GetODBCConnection()->CreateColumn(aAlias,eType,pcbColDef,pibScale, nFlags);
		pColumn->SetRealName(String(aColumnName,gsl_getSystemTextEncoding()));

		aResultColumns->AddColumn(pColumn);
	}
	// Strukturbeschreibung fertig.
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::DescribeParameter()
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aParameterColumns.Is(), "keine Parameterspalten angelegt!");

	// Bookmark-Column ist immer da (schon durch den Konstruktor erzeugt).

	// Anzahl Parameter ermitteln:
	RETCODE nRetcode;
	SWORD nNumParams;
	SdbODBCConnection* pConnection = GetODBCConnection();

	if (!pConnection || !pConnection->IsODBCFunctionAvailable(SQL_API_SQLNUMPARAMS))
		return;

	nRetcode = NSQLNumParams(m_aStatementHandle,
							 &nNumParams
							 );
	SetStatus(nRetcode,m_aStatementHandle);
	if (!aStatus.IsSuccessful() || !nNumParams) return;

	// steht die Funktion zur Verfuegung
	BOOL bDescribe = pConnection->IsODBCFunctionAvailable(SQL_API_SQLDESCRIBEPARAM);
	if (!bDescribe)
		SdbCursor::DescribeParameter();
	else
	{
		for (UWORD i = 1; i <= (UWORD)nNumParams; i++)
		{
			SdbColumn * pColumn = NULL;
			String aParameterName(aParameterNames.GetToken(i-1));
			// Name == "?", falls kein Parametername verfuegbar (z. B. bei Native SQL)
			if (aParameterName.Len() == 0) aParameterName = '?';

			SWORD pfSqlType=0;			// SQL-Datentyp
			UDWORD pcbColDef=0L;			// Precision (ggf. 0)
			SWORD pibScale=0;				// Scale (ggf. 0)
			SWORD pfNullable=0;			// NULL erlaubt?
			SdbDatabaseType eType;


			aStatus.Clear();
			RETCODE nRetcode = NSQLDescribeParam(m_aStatementHandle,
										 i,
										 &pfSqlType,
										 &pcbColDef,
										 &pibScale,
										 &pfNullable
										 );
			SetStatus(nRetcode,m_aStatementHandle);
			if (! aStatus.IsSuccessful()) return;

			switch(pfSqlType)
			{
				case SQL_WCHAR:
				case SQL_CHAR:			eType = SDB_DBTYPE_CHAR; break;
				case SQL_WVARCHAR:
				case SQL_VARCHAR: 		eType = SDB_DBTYPE_VARCHAR; break;
				case SQL_WLONGVARCHAR:
				case SQL_LONGVARCHAR: 	eType = SDB_DBTYPE_LONGVARCHAR; break;
				case SQL_DECIMAL:		eType = SDB_DBTYPE_DECIMAL; break;
				case SQL_NUMERIC:		eType = SDB_DBTYPE_NUMERIC; break;
				case SQL_BIT:
					eType = SDB_DBTYPE_BOOLEAN;
					if(!pcbColDef)
					{
						pcbColDef = 1; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_TINYINT:
					eType = SDB_DBTYPE_TINYINT;
					if(!pcbColDef)
					{
						pcbColDef = 3; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_SMALLINT:
					eType = SDB_DBTYPE_SMALLINT;
					if(!pcbColDef)
					{
						pcbColDef = 5; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_INTEGER:
					eType = SDB_DBTYPE_INTEGER;
					if(!pcbColDef)
					{
						pcbColDef = 10; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				//case SQL_BIGINT: 		eType = SDB_DBTYPE_BIGINT; break;
				case SQL_REAL:
					eType = SDB_DBTYPE_REAL;
					if(!pcbColDef)
					{
						pcbColDef = 7; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_FLOAT:
				case SQL_DOUBLE:
				case SQL_BIGINT:
					eType = SDB_DBTYPE_DOUBLE;
					if(!pcbColDef)
					{
						pcbColDef = 15; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_BINARY:		eType = SDB_DBTYPE_BINARY; break;
				case SQL_VARBINARY: 	eType = SDB_DBTYPE_VARBINARY; break;
				case SQL_LONGVARBINARY:	eType = SDB_DBTYPE_LONGVARBINARY; break;
				case SQL_DATE: 			eType = SDB_DBTYPE_DATE; break;
				case SQL_TIME: 			eType = SDB_DBTYPE_TIME; break;
				case SQL_TIMESTAMP: 	eType = SDB_DBTYPE_TIMESTAMP; break;
				default:				eType = SDB_DBTYPE_UNKNOWN; // wird nicht gebunden
			}

			pColumn = pConnection->CreateColumn(aParameterName,eType,pcbColDef,pibScale);
			if (pfNullable == SQL_NULLABLE)
				pColumn->SetFlag(SDB_FLAGS_NULLALLOWED);
			pColumn->SetFlag(SDB_FLAGS_PARAM_IN);

			// Column-Beschreibung hinzufuegen.
			aParameterColumns->AddColumn(pColumn);
		}
	}
	// Parameter-Strukturbeschreibung fertig.
}

//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::BindColumn(UINT16 nPos, void** pDataBuffer, INT32* pLenBuffer,
							   const SdbColumn& rCol, const ODbVariantRef& pVariable, BOOL bParam)
{
	RETCODE nRetcode;
	SWORD   fSqlType;
	SWORD   fCType;
	SDWORD  nMaxLen = 0;
	void*   pData   = pDataBuffer[nPos];
	SDWORD* pLen    = &pLenBuffer[nPos];

	switch(rCol.GetType())
	{
		case SDB_DBTYPE_CHAR:			if(GetODBCConnection()->m_bUserWChar)
										{
											fCType   = SQL_C_WCHAR;
											fSqlType = SQL_WCHAR;
										}
										else
										{
											fCType   = SQL_C_CHAR;
											fSqlType = SQL_CHAR;
										}
										break;
		case SDB_DBTYPE_VARCHAR:		if(GetODBCConnection()->m_bUserWChar)
										{
											fCType   = SQL_C_WCHAR;
											fSqlType = SQL_WVARCHAR;
										}
										else
										{
											fCType   = SQL_C_CHAR;
											fSqlType = SQL_VARCHAR;
										}
										break;
		case SDB_DBTYPE_LONGVARCHAR:	if(GetODBCConnection()->m_bUserWChar)
										{
											fCType   = SQL_C_WCHAR;
											fSqlType = SQL_WLONGVARCHAR;
										}
										else
										{
											fCType	 = SQL_C_CHAR;
											fSqlType = SQL_LONGVARCHAR;
										}
										break;
		case SDB_DBTYPE_DECIMAL:		fCType   = GetODBCConnection()->m_bUserWChar ? SQL_C_WCHAR : SQL_C_CHAR;
										fSqlType = SQL_DECIMAL; break;
		case SDB_DBTYPE_CURRENCY:
		case SDB_DBTYPE_NUMERIC:		fCType	 = GetODBCConnection()->m_bUserWChar ? SQL_C_WCHAR : SQL_C_CHAR;
										fSqlType = SQL_NUMERIC; break;
		case SDB_DBTYPE_BOOLEAN:		fCType   = SQL_C_TINYINT;
										fSqlType = SQL_INTEGER; break;
		case SDB_DBTYPE_TINYINT:		fCType = SQL_C_SHORT;
										fSqlType = SQL_TINYINT; break;
		case SDB_DBTYPE_SMALLINT:		fCType = SQL_C_SHORT;
										fSqlType = SQL_SMALLINT; break;
		case SDB_DBTYPE_INTEGER:		fCType = SQL_C_LONG;
										fSqlType = SQL_INTEGER; break;
		case SDB_DBTYPE_BIGINT:			fCType =  GetODBCConnection()->m_bUserWChar ? SQL_C_WCHAR : SQL_C_CHAR;
										fSqlType = SQL_BIGINT; break;
		case SDB_DBTYPE_REAL:			fCType = SQL_C_FLOAT;
										fSqlType = SQL_REAL; break;
		case SDB_DBTYPE_DOUBLE:			fCType = SQL_C_DOUBLE;
										fSqlType = SQL_DOUBLE; break;
		case SDB_DBTYPE_BINARY:			fCType = SQL_C_BINARY;
										fSqlType = SQL_BINARY; break;
		case SDB_DBTYPE_VARBINARY:		fCType = SQL_C_BINARY;
										fSqlType = SQL_VARBINARY; break;
		case SDB_DBTYPE_LONGVARBINARY: 	fCType = SQL_C_BINARY;
										fSqlType = SQL_LONGVARBINARY; break;
		case SDB_DBTYPE_DATE:			fCType = SQL_C_DATE;
										fSqlType = SQL_DATE; break;
		case SDB_DBTYPE_TIME:			fCType = SQL_C_TIME;
										fSqlType = SQL_TIME; break;
		case SDB_DBTYPE_TIMESTAMP:		fCType = SQL_C_TIMESTAMP;
										fSqlType = SQL_TIMESTAMP; break;
		default:						fCType = SQL_C_BINARY;
										fSqlType = SQL_LONGVARBINARY; break;
	}

	SWORD fParamType =				rCol.IsInParameter()
											?
				( rCol.IsOutParameter() ? 	SQL_PARAM_INPUT_OUTPUT : SQL_PARAM_INPUT)
											:
									SQL_PARAM_OUTPUT ;

	if (!pVariable.is() || pVariable->isNull())
	{
		*pLen = SQL_NULL_DATA;
		if (bParam)
			nRetcode = NSQLBindParameter(m_aStatementHandle,
									nPos,
									fParamType,
									fCType,
									fSqlType,
									0,
									0,
									NULL,
									nMaxLen,
									pLen
									);
		else
			nRetcode = NSQLBindCol(m_aStatementHandle,
									nPos,
									fCType,
									pData,
									nMaxLen,
									pLen
									);
	}
	else
	{
		try
		{
			switch (rCol.GetType())
			{
				case SDB_DBTYPE_CHAR:
				case SDB_DBTYPE_VARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
				{
					rtl::OUString aString(pVariable->toString());
					//	aString.Convert(CHARSET_SYSTEM,GetConnection()->GetCharacterSet());
					nMaxLen = rCol.GetPrecision();
					*pLen = SQL_NTS;
					*((rtl::OUString*)pData) = aString;

					// Zeiger auf Char*
					pData = (void*)((rtl::OUString*)pData)->getStr();
				}
				else
				{
					ByteString aString(pVariable->toString().getStr(),GetConnection()->GetCharacterSet());
					nMaxLen = rCol.GetPrecision();
					*pLen = SQL_NTS;
					*((ByteString*)pData) = aString;

					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}	break;
				case SDB_DBTYPE_BIGINT:
				case SDB_DBTYPE_DECIMAL:
				case SDB_DBTYPE_CURRENCY:
				case SDB_DBTYPE_NUMERIC:
				if(GetODBCConnection()->m_bUserWChar)
				{
					rtl::OUString aString(SdbTools::ToString(ODbTypeConversion::toDouble(*pVariable),rCol.GetScale()));
					nMaxLen = aString.len();
					*pLen = nMaxLen;
					*((rtl::OUString*)pData) = aString;
					// Zeiger auf Char*
					pData = (void*)((rtl::OUString*)pData)->getStr();
				}
				else
				{
					ByteString aString(SdbTools::ToString(ODbTypeConversion::toDouble(*pVariable),rCol.GetScale()),gsl_getSystemTextEncoding());
					nMaxLen = aString.Len();
					*pLen = SQL_NTS;
					*((ByteString*)pData) = aString;
					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}	break;
				case SDB_DBTYPE_BOOLEAN:
					*((INT8*)pData) = ODbTypeConversion::toBOOL(*pVariable) ? 1 : 0;
					*pLen = sizeof(INT8);
					break;
				case SDB_DBTYPE_TINYINT:
				case SDB_DBTYPE_SMALLINT:
					*((INT16*)pData) = ODbTypeConversion::toINT16(*pVariable);
					*pLen = sizeof(INT16);
					break;
				case SDB_DBTYPE_INTEGER:
					*((INT32*)pData) = ODbTypeConversion::toINT32(*pVariable);
					*pLen = sizeof(INT32);
					break;
				case SDB_DBTYPE_REAL:
					*((float*)pData) = ODbTypeConversion::toFloat(*pVariable);
					*pLen = sizeof(float);
					break;
				case SDB_DBTYPE_DOUBLE:
					*((double*)pData) = ODbTypeConversion::toDouble(*pVariable);
					*pLen = sizeof(double);
					break;
				case SDB_DBTYPE_BINARY:
				case SDB_DBTYPE_VARBINARY:
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
					{
						const ::staruno::Sequence<sal_Int8>& rData = pVariable->getBytes();
						pData = (void*)rData.getConstArray();
						*pLen = rData.getLength();
						nMaxLen = rCol.GetPrecision();
					}	break;
				case SDB_DBTYPE_UNKNOWN:
				case SDB_DBTYPE_LONGVARBINARY:
				case SDB_DBTYPE_LONGVARCHAR:
				{
					pData  = (void*)&rCol;
					INT32 nLen = 0;
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
						nLen = pVariable->getBytes().getLength();
					else
						nLen = ODbTypeConversion::toString(*pVariable).len();
					*pLen = (SDWORD)SQL_LEN_DATA_AT_EXEC(nLen);
				}   break;
				case SDB_DBTYPE_DATE:
					ToDate(*(DATE_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(DATE_STRUCT);
					break;
				case SDB_DBTYPE_TIME:
					ToTime(*(TIME_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(TIME_STRUCT);
					break;
				case SDB_DBTYPE_TIMESTAMP:
					ToTimeStamp(*(TIMESTAMP_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(TIMESTAMP_STRUCT);
			}
		}
		catch ( ... )
		{
			String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
			strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());
			return FALSE;
		}


		if (bParam)
			nRetcode = NSQLBindParameter(m_aStatementHandle,
						  nPos,
						  fParamType,
						  fCType,
						  fSqlType,
						  rCol.GetPrecision(),
						  rCol.GetScale(),
						  pData,
						  nMaxLen,
						  pLen);
		else
			nRetcode = NSQLBindCol(m_aStatementHandle,
									nPos,
									fCType,
									pData,
									nMaxLen,
									pLen
									);
	}

	if(bParam && (fParamType == SQL_PARAM_INPUT_OUTPUT || fParamType == SQL_PARAM_OUTPUT))
	{
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_BOOKMARK:
			case SDB_DBTYPE_INTEGER:
				pVariable->setINT32(*(INT32*)pLen);
				break;
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_CURRENCY:
				pVariable->setDouble(SdbTools::ToDouble((const char *)pLen));
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				pVariable->setINT16(*(INT16*)pLen);
				break;

			case SDB_DBTYPE_REAL:
				pVariable->setFloat(*(float*)pLen);
				break;

			case SDB_DBTYPE_DOUBLE:
				pVariable->setDouble(*(double*)pLen);
				break;

			case SDB_DBTYPE_BOOLEAN:
				pVariable->setBOOL( (*(INT16*)pLen) ? TRUE : FALSE );
				break;

			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
			case SDB_DBTYPE_LONGVARCHAR:
				if (GetODBCConnection()->m_bUserWChar)
					pVariable->setString(UniString((const sal_Unicode *)pLen));
				else
					pVariable->setString(UniString((const char *)pLen ,GetConnection()->GetCharacterSet()));
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
			case SDB_DBTYPE_LONGVARBINARY:
			// Sonst BINAER-Datentyp.
				pVariable->setString(UniString((const char *)pLen ,GetConnection()->GetCharacterSet()));
				break;
			case SDB_DBTYPE_DATE:
				pVariable->setDate(ToDate(*(DATE_STRUCT*)pLen));
				break;
			case SDB_DBTYPE_TIME:
				pVariable->setTime(ToTime(*(TIME_STRUCT*)pLen));
				break;
			case SDB_DBTYPE_TIMESTAMP:
				pVariable->setDateTime(ToDateTime(*(TIMESTAMP_STRUCT*)pLen));
				break;
			default:
				DBG_ERROR("SdbODBC2Cursor: interner Fehler: unbekannter SDB-Datentyp"); break; // Unbekannter Datenbanktyp
		}
	}

	SetStatus(nRetcode,m_aStatementHandle);
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbODBC2Cursor::ImplChangeOpenOptions(SdbCursorType eType, UINT16 eOpt)
{
	BOOL bChange = FALSE;
	if(eType != SDB_CURRENT)
	{
		eCursorType = eType;
		bChange = TRUE;
	}
	if (eOpt != SDB_CURRENT_OPT)
	{
		nOptions = eOpt;
		if ((nOptions & SDB_READONLY) == SDB_READONLY)	// append und readonly widersprechen sich
			nOptions &= ~SDB_APPENDONLY;
		bChange = TRUE;
	}
	if (bChange)
	{
		INT32 nTempCursorType(nCursorType), nTempConcurrency(nConcurrency);

		SdbODBCConnection* pConnection = GetODBCConnection();
		DetermineOptions(pConnection);

		// bei PreparedStatements schliessen, da Optionen sonst nicht umgesetzt werden koennen
		// bitter aber wahr, was macht man nicht alles fuer eine gute Schnittstelle !!!
		BOOL bPrepareAgain;
		if (bPrepareAgain = (IsPrepared() && (nCursorType != nTempCursorType ||
											  nConcurrency != nTempConcurrency)))
		{
			// Handle neu setzen
			SetStatus(NSQLFreeStmt(m_aStatementHandle,SQL_DROP),m_aStatementHandle);
			if (aStatus.IsSuccessful())
			{
				RETCODE nRetcode = (m_aConnectionHandle != SQL_NULL_HDBC)
					? NSQLAllocStmt(m_aConnectionHandle,&m_aStatementHandle)
					: NSQLAllocStmt(GetODBCConnection()->ConnectionHandle(),&m_aStatementHandle);
				SetStatus(nRetcode,m_aStatementHandle);
			}
		}

		if (aStatus.IsSuccessful())
		{
			SetOptions(pConnection);
			if (bPrepareAgain)
			{
				if (pParamBuffer)
				{	// if we already were prepared
					// FS - 70334 - 01.12.99
					FreeBuffer(pParamBuffer, pParamLength, *aParameterColumns, FALSE);
					pParamBuffer = NULL;
				}

				PrepareImpl();
			}
		}
	}
}
//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::OpenImpl(SdbCursorType eType ,UINT16 eOpt )
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	ImplChangeOpenOptions(eType, eOpt);

	RETCODE nRetcode;

	if (IsPrepared())
	{
		if (IsOpen())
		{
			// Schon einmal Execute ausgefuehrt.
			// Vor dem naechsten Execute muss der Cursor erst geschlossen werden
			// (Option CLOSE).
			NSQLFreeStmt(m_aStatementHandle,SQL_CLOSE);
		}
		UINT16 nCount(aParameterColumns->Count());
		for (UINT16 i=1 ; i < nCount;i++)
			BindColumn(i, pParamBuffer, pParamLength, *(*aParameterColumns)[i], (*aParameterRow)[i]);
	}
	else
	{
		::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(GetConnection()->GetMetaData());
		sal_Int16 nInfo;
		rMetaData->getInfo(::stardata::DatabaseInfo::OUTER_JOIN_SUPPORT) >>= nInfo;
		BOOL bInnerJoin = (nInfo & ::stardata::DatabaseOuterJoinSupport::YES || nInfo & ::stardata::DatabaseOuterJoinSupport::PARTIAL);
		Translate(bInnerJoin);
		if (! aStatus.IsSuccessful())
			return FALSE;
	}

	// Anpassen  der Menge der zu bertragenden Daten
	if (nCurrentMaxFieldLen != nMaxFieldLen)
	{
		nRetcode = NSQLSetStmtOption(m_aStatementHandle, SQL_MAX_LENGTH, nMaxFieldLen);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, m_aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState() == aValueChangedState)
				NSQLGetStmtOption(m_aStatementHandle, SQL_MAX_LENGTH, &nMaxFieldLen);

			nCurrentMaxFieldLen = nMaxFieldLen;
		}
		else
			// Wieder zuruecksetzen
			nMaxFieldLen = nCurrentMaxFieldLen;
	}

	if (nCurrentMaxRows != nMaxRows)
	{
		nRetcode = NSQLSetStmtOption(m_aStatementHandle, SQL_MAX_ROWS, nMaxRows);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, m_aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState() == aValueChangedState)
				NSQLGetStmtOption(m_aStatementHandle, SQL_MAX_ROWS, &nMaxRows);

			nCurrentMaxRows = nMaxRows;
		}
		else
			// Wieder zuruecksetzen
			nMaxRows = nCurrentMaxRows;
	}

	if (IsPrepared())
	{
		BEGIN_CANCELABLE();
		nRetcode = NSQLExecute(m_aStatementHandle);
		END_CANCELABLE();
	}
	else
	{
		BEGIN_CANCELABLE();
		ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
		nRetcode = NSQLExecDirect(m_aStatementHandle, (SDB_ODBC_CHAR*)aSql.GetBuffer(), aSql.Len());
		END_CANCELABLE();
	}

	nCurrentMaxFieldLen = nMaxFieldLen;
	nCurrentMaxRows = nCurrentMaxRows;

	// Parameter at Execution Time erforderlich?
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;

		nRetcode = NSQLParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS &&
				nRetcode != SQL_SUCCESS_WITH_INFO &&
				nRetcode != SQL_NEED_DATA) break;

			if (aParameterColumns->Count() == 1)
			{
				// Keine Strukturbeschreibung fuer die Parameter Row oder keine Parameter Row.
				// Das darf nicht sein - Statement-Ausfuehrung abbrechen!
				if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLCANCEL))
					NSQLCancel(m_aStatementHandle);
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1093"),
							aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_PARAMETER_NUMBER))),
							0, String() );
				// Genaugenommen eigentlich: Parameter missing ...
				return FALSE;
			}

			// Daten fuer diesen Parameter an ODBC uebergeben ...
			// Die zugehoerige Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			PutData(*pCol, (*aParameterRow)[pCol->GetId()]); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
			{
				// Bei Fehler Statement-Ausfuehrung abbrechen!
				if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLCANCEL))
					NSQLCancel(m_aStatementHandle);
				return FALSE;
			}

			nRetcode = NSQLParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
	}

	// Resultat des SQLExec[Direct] oder des letzten SqlParamData
	// ist das Resultat des gesamten Statements ...
	SetStatus(nRetcode,m_aStatementHandle);
	if (!aStatus.IsSuccessful()) return FALSE;

	// solange er nicht wirklich gebraucht wird,
	// einfach erst einmal selbst setzen
	if (aName.Len() == 0)
	{
		aName = 'C';
		aName += String::CreateFromInt32((long)this);
	}

	return TRUE;
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenTables(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
							   const sal_Unicode* pTable, const sal_Unicode* pType)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");


	RETCODE nRetcode = NSQLTables(m_aStatementHandle,
						 (SDB_ODBC_CHAR *) pQualifier, pQualifier ? SQL_NTS : 0,
						 (SDB_ODBC_CHAR *) pOwner, pOwner ? SQL_NTS : 0,
						 (SDB_ODBC_CHAR *) pTable, pTable ? SQL_NTS : 0,
						 (SDB_ODBC_CHAR *) pType, pType ? SQL_NTS : 0);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenColumns(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, const sal_Unicode* pColumn)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
		ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pColumn		? (aCOL = ByteString(pColumn,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode = NSQLColumns(m_aStatementHandle,
							(SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pCOL, pColumn ? SQL_NTS : 0);
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenTypeInfo(INT16 nType)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode = NSQLGetTypeInfo(m_aStatementHandle, nType);
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenPrimaryKeys(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLPRIMARYKEYS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode = NSQLPrimaryKeys(m_aStatementHandle,
								(SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenTablePrivileges(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLTABLEPRIVILEGES))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode = NSQLTablePrivileges(m_aStatementHandle,
								   (SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								   (SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								   (SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenIndices(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, BOOL bUnique)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLSTATISTICS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode = NSQLStatistics(m_aStatementHandle,
							 (SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							 (SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							 (SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							 bUnique ? SQL_INDEX_UNIQUE : SQL_INDEX_ALL,
							 SQL_QUICK);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenForeignKeys(const sal_Unicode* pPKQualifier, const sal_Unicode* pPKOwner, const sal_Unicode* pPKName,
									const sal_Unicode* pFKQualifier, const sal_Unicode* pFKOwner, const sal_Unicode* pFKName)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLFOREIGNKEYS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN, aFKQ, aFKO, aFKN;

	const char	*pPKQ = pPKQualifier	? (aPKQ = ByteString(pPKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pPKOwner		? (aPKO = ByteString(pPKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pPKName			? (aPKN = ByteString(pPKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pFKQ = pFKQualifier	? (aFKQ = ByteString(pFKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pFKO = pFKOwner		? (aFKO = ByteString(pFKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pFKN = pFKName			? (aFKN = ByteString(pFKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	RETCODE nRetcode = NSQLForeignKeys(m_aStatementHandle,
								(SDB_ODBC_CHAR *) pPKQ, pPKQualifier ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKO, pPKOwner ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKN, pPKName ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pFKQ, pFKQualifier ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pFKO, pFKOwner ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pFKN, pFKName ? SQL_NTS : 0
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenProcedures(const String& rQualifier, const String& rSchema, const String& rProcName)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLPROCEDURES))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding());

	RETCODE nRetcode = NSQLProcedures(m_aStatementHandle,
								 (SDB_ODBC_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ODBC_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ODBC_CHAR *) aP.GetBuffer(), aP.Len()
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenProcedureColumns(const String& rQualifier, const String& rSchema,
										 const String& rProcName, const String& rProcColumnName)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLPROCEDURECOLUMNS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding()),
		aC(rProcColumnName,gsl_getSystemTextEncoding());

	RETCODE nRetcode = NSQLProcedureColumns(m_aStatementHandle,
								 (SDB_ODBC_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ODBC_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ODBC_CHAR *) aP.GetBuffer(), aP.Len(),
								 (SDB_ODBC_CHAR *) aC.GetBuffer(), aC.Len()
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}
//-----------------------------------------------------------------------------
void SdbODBC2Cursor::OpenSpecialColumns(const String& rQualifier, const String& rSchema, const String& rTabName)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLSPECIALCOLUMNS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aT(rTabName,gsl_getSystemTextEncoding());

	RETCODE nRetcode = NSQLSpecialColumns(m_aStatementHandle,SQL_ROWVER,
								 (SDB_ODBC_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ODBC_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ODBC_CHAR *) aT.GetBuffer(), aT.Len(),
								 SQL_SCOPE_TRANSACTION,SQL_NULLABLE
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	Open4Info();
}
//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::CloseImpl()
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);

	// Vater nicht mehr vorhanden, dann alles freigeben
	if (!GetParent())
	{
		if (m_aStatementHandle != SQL_NULL_HSTMT)
		{
			NSQLFreeStmt(m_aStatementHandle,SQL_DROP);
			m_aStatementHandle = SQL_NULL_HSTMT;

			if (m_aConnectionHandle != SQL_NULL_HDBC)	// extra Verbindung
			{
				NSQLDisconnect(m_aConnectionHandle);
			#ifdef ODBCIMP
				if (pSQLFreeConnect)
					NSQLFreeConnect(m_aConnectionHandle);
			#else
					NSQLFreeConnect(m_aConnectionHandle);
			#endif
			}
		}
	}
	else
	{
		DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
		// wenn einer der drei nicht klappt, dann den Handle wegschmeissen
		if (NSQLFreeStmt(m_aStatementHandle,SQL_RESET_PARAMS) ||
			NSQLFreeStmt(m_aStatementHandle,SQL_UNBIND) ||
			NSQLFreeStmt(m_aStatementHandle,SQL_CLOSE))
		{
			SetStatus(NSQLFreeStmt(m_aStatementHandle,SQL_DROP),m_aStatementHandle);
			if (aStatus.IsSuccessful())
			{
				RETCODE rRetcode = (m_aConnectionHandle != SQL_NULL_HDBC) ?
					NSQLAllocStmt(m_aConnectionHandle,&m_aStatementHandle)
									:
					NSQLAllocStmt(GetODBCConnection()->ConnectionHandle(),&m_aStatementHandle);
				SetStatus(rRetcode,m_aStatementHandle);
			}
			if (!aStatus.IsSuccessful())
			{
				if (m_aConnectionHandle != SQL_NULL_HDBC)	// extra Verbindung
				{
					NSQLDisconnect(m_aConnectionHandle);
				#ifdef ODBCIMP
					if (pSQLFreeConnect)
						NSQLFreeConnect(m_aConnectionHandle);
				#else
						NSQLFreeConnect(m_aConnectionHandle);
				#endif
				}
				m_aStatementHandle = SQL_NULL_HSTMT;
			}
		}
	}
	SdbODBCCursor::CloseImpl();
	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,
							 SdbCursorMode& rMode)
{
	DBG_CHKTHIS(SdbODBC2Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");

	RETCODE nRetcode;

	if (GetODBCConnection()->nFetchDirection == SQL_FD_FETCH_NEXT ||
		(nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY)
	{
		DBG_ASSERT(eCursorPosition == SDB_POS_NEXT, "ODBCCursor: Falsche MoveRichtung, nur Next zulaessig!");
		if (eCursorPosition != SDB_POS_NEXT)
		{
			aStatus.SetDriverNotCapableError();
		}
		else
		{
			nRetcode = NSQLFetch(m_aStatementHandle);
			SetStatus(nRetcode,m_aStatementHandle);
		}
	}
	else
	{
		// Voraussetzung: rowset size == 1 (Default-Einstellung, siehe SQLSetStmtOption)
		UWORD fFetchType;           // type of fetch
		SDWORD irow = nOffset;		// number of the row to fetch
		UDWORD pcrow;				// number of rows actually fetched
		switch(eCursorPosition)
		{
			case SDB_POS_NEXT:			fFetchType = SQL_FETCH_NEXT; break;
			case SDB_POS_PRIOR:			fFetchType = SQL_FETCH_PRIOR; break;
			case SDB_POS_FIRST:			fFetchType = SQL_FETCH_FIRST; break;
			case SDB_POS_LAST:			fFetchType = SQL_FETCH_LAST; break;
			case SDB_POS_RELATIVE:		fFetchType = SQL_FETCH_RELATIVE; break;
			case SDB_POS_ABSOLUTE:		fFetchType = SQL_FETCH_ABSOLUTE; break;
			default: 					DBG_ERROR("SdbODBC2Cursor: unzulaessiger Offset"); break;
		}

		nRetcode = NSQLExtendedFetch(m_aStatementHandle,
											fFetchType,
											irow,
											&pcrow,
											&nRowStatus
											);

		// Status: "off range", d. h. kein weiterer Datensatz (egal bei
		// welcher Positionierung):
		SetStatus(nRetcode,m_aStatementHandle);

		// ist eine gueltige Zeile gefetched worden
		if (nRowStatus == SQL_ROW_DELETED )
			rMode = SdbCursor::SDB_MOD_INVALID;
	}
	if (!aStatus.IsSuccessful()) return FALSE;

	FixResultRow();
	if (!aStatus.IsSuccessful())
	{
		rMode = SdbCursor::SDB_MOD_INVALID;
		aStatus.Clear();
	}
	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
int SdbODBC2Cursor::GetDataImpl(UWORD icol , SWORD fCType , PTR rgbValue , SDWORD cbValueMax , SDWORD FAR * pcbValue)
{
	return NSQLGetData(m_aStatementHandle,icol,fCType,rgbValue,cbValueMax,pcbValue);
}
//-----------------------------------------------------------------------------
int SdbODBC2Cursor::PutDataImpl(PTR rgbValue , SDWORD cbValue)
{
	return NSQLPutData(m_aStatementHandle,rgbValue,cbValue);
}

//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::DeleteImpl()
{
/*
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HSTMT,"interner Fehler: StatementHandle ungueltig");
	return ExecutePosUpdate(SQL_DELETE);
*/
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::InsertImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::UpdateImpl()
{
	return FALSE;
}

//-----------------------------------------------------------------------------
/*BOOL SdbODBC2Cursor::ExecutePosUpdate(UINT16 nOption)
{
#if SDB_ODBC2
	UINT16 nExpectedStatus;

	switch (nOption)
	{
		case SQL_UPDATE:
			nExpectedStatus = SQL_ROW_UPDATED;
			break;
		case SQL_ADD:
			nExpectedStatus = SQL_ROW_ADDED;
			break;
		default:
			nExpectedStatus = SQL_ROW_DELETED;
	}

	RETCODE nRetcode = NSQLSetPos(m_aStatementHandle, 1, nOption, SQL_LOCK_NO_CHANGE);
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;

		nRetcode = NSQLParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS &&
				nRetcode != SQL_SUCCESS_WITH_INFO &&
				nRetcode != SQL_NEED_DATA) break;

			// Daten fuer diesen Parameter an ODBC uebergeben ...
			// Die zugehoerige Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			PutData(*pCol, (*aResultRow)[pCol->GetId()]); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
				return FALSE;

			nRetcode = NSQLParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
	}

	SetStatus(nRetcode,m_aStatementHandle);

	// Update hat nicht geklappt, falscher Status
	if (nRetcode == SQL_SUCCESS_WITH_INFO && nRowStatus != nExpectedStatus)
		aStatus.Set(SDB_STAT_ERROR);


	if (aStatus.IsSuccessful() && (nOption == SQL_UPDATE || nOption == SQL_ADD))
	{
		ClearAfterUpdate();
	}


	return aStatus.IsSuccessful();
#else
	return FALSE;
#endif
}*/

//-----------------------------------------------------------------------------
void SdbODBC2Cursor::RowCountImpl()
{
	if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLROWCOUNT))
	{
		SDWORD pcrow=-1;
		NSQLRowCount(m_aStatementHandle, &pcrow);
		aStatus.Clear();

		if (pcrow >= 0)
			SetRowCount((UINT32)pcrow);
	}
}

//------------------------------------------------------------------------
BOOL SdbODBC2Cursor::IsCapable(SdbCursorCapability eCapability) const
{
	BOOL bResult=FALSE;
	SdbODBCConnection* pCon = GetODBCConnection();
	switch(eCapability)
	{
		case SDB_CAP_FETCH_NEXT:		bResult = (pCon->nFetchDirection & SQL_FD_FETCH_NEXT)		== SQL_FD_FETCH_NEXT; break;
		case SDB_CAP_FETCH_PRIOR:		bResult = (pCon->nFetchDirection & SQL_FD_FETCH_PREV)		== SQL_FD_FETCH_PREV; break;
		case SDB_CAP_FETCH_FIRST:		bResult = (pCon->nFetchDirection & SQL_FD_FETCH_FIRST)		== SQL_FD_FETCH_FIRST; break;
		case SDB_CAP_FETCH_LAST:		bResult = (pCon->nFetchDirection & SQL_FD_FETCH_LAST)		== SQL_FD_FETCH_LAST; break;
		case SDB_CAP_FETCH_RELATIVE:    bResult = (pCon->nFetchDirection & SQL_FD_FETCH_RELATIVE)	== SQL_FD_FETCH_RELATIVE; break;
		case SDB_CAP_FETCH_ABSOLUTE:    bResult = (pCon->nFetchDirection & SQL_FD_FETCH_ABSOLUTE)	== SQL_FD_FETCH_ABSOLUTE; break;
		case SDB_CAP_FETCH:				bResult = TRUE; break;
	}
	return bResult;
}
//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::GetDateValues(UINT16 nPos,DATE_STRUCT &rValue)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)
	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
	// Column #0 ist die Bookmark!

	nRetcode = GetDataImpl(nPos,SQL_C_DATE,&rValue,sizeof(rValue),&pcbValue);
	SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.IsError())
	{
		DBG_ERROR(((((ByteString("SQLGetData fehlgeschlagen!: Column ") += ByteString::CreateFromInt32(nPos))
			+= ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding()))).GetBuffer()) ;
	}
	else if (pcbValue == SQL_NULL_DATA)
		return FALSE;
	return TRUE;
}
//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::GetTimeValues(UINT16 nPos,TIME_STRUCT &rValue)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)
	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
	// Column #0 ist die Bookmark!

	nRetcode = GetDataImpl(nPos,SQL_C_TIME,&rValue,sizeof(rValue),&pcbValue);
	SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.IsError())
	{
		DBG_ERROR(((((ByteString("SQLGetData fehlgeschlagen!: Column ") += ByteString::CreateFromInt32(nPos))
			+= ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding()))).GetBuffer()) ;
	}
	else if (pcbValue == SQL_NULL_DATA)
		return FALSE;
	return TRUE;
}
//-----------------------------------------------------------------------------
BOOL SdbODBC2Cursor::GetTimeDateValues(UINT16 nPos,TIMESTAMP_STRUCT &rValue)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)
	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
	// Column #0 ist die Bookmark!

	nRetcode = GetDataImpl(nPos,SQL_C_TIMESTAMP,&rValue,sizeof(rValue),&pcbValue);
	SetStatus(nRetcode,m_aStatementHandle);
	if (aStatus.IsError())
	{
		DBG_ERROR(((((ByteString("SQLGetData fehlgeschlagen!: Column ") += ByteString::CreateFromInt32(nPos))
			+= ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding()))).GetBuffer()) ;
	}
	else if (pcbValue == SQL_NULL_DATA)
		return FALSE;
	return TRUE;
}

