/*************************************************************************
 *
 *  $RCSfile: odbc3curs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#define _SDB_ODBC3CURS_CXX
#include "sdbresid.hrc"
#include "odbc3curs.hxx"
#include "odbc3conn.hxx"

#include <stdlib.h>

#ifndef _SDB_TYPECONV_HXX
#include <typeconv.hxx>
#endif

#include "sdbtools.hxx"

#ifdef SQL_KEYWORDS
#undef SQL_KEYWORDS
#endif

#ifndef _COM_SUN_STAR_DATA_XDATABASEDESCRIPTOR_HPP_
#include <com/sun/star/data/XDatabaseDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEINFO_HPP_
#include <com/sun/star/data/DatabaseInfo.hpp>
#endif
#ifndef _COM_SUN_STAR_DATA_DATABASEOUTERJOINSUPPORT_HPP_
#include <com/sun/star/data/DatabaseOuterJoinSupport.hpp>
#endif

namespace stardata = com::sun::star::data;
namespace staruno = com::sun::star::uno;



DBG_NAME(SdbODBC3Cursor);
//-----------------------------------------------------------------------------
SdbODBC3Cursor::SdbODBC3Cursor(SdbODBC3Connection* pConnection,SdbCursorType eType,UINT16 nOpt)
			   :SdbODBCCursor(pConnection, eType, nOpt)
			   ,m_nBookmarkLen(0)
{
	DBG_CTOR(SdbODBC3Cursor,NULL);

	if (pConnection->HasFreeStatement())
	{
		SQLRETURN rRetcode = N3SQLAllocHandle(SQL_HANDLE_STMT,pConnection->ConnectionHandle(),&m_aStatementHandle);
		SetStatus(rRetcode,m_aStatementHandle);

		if (aStatus.IsError())
		{
			// es ist nicht moeglich einen weiteree Handle zu bekommen,
			// daher versuchen wir einfach noch eine Verbindung aufzumachen
			aStatus.Clear();
			m_aStatementHandle = SQL_NULL_HANDLE;
		}
	}

	// Wenn die Verbindung bereits alle Statements verbraucht hat, dann eine neue extra fr den Cursor
	if (m_aStatementHandle == SQL_NULL_HANDLE)
	{
		SQLHENV aEnvironmentHandle = pConnection->EnvironmentHandle();
		DBG_ASSERT(aEnvironmentHandle != SQL_NULL_HANDLE,"SdbODBC3Connection: interner Fehler: EnvironmentHandle ungueltig");

		// Connection allozieren extra fr den Cursor
		// (Eine ODBC-Connection pro SdbConnection).
		SetStatus(N3SQLAllocHandle(SQL_HANDLE_DBC,aEnvironmentHandle,&m_aConnectionHandle),aEnvironmentHandle);
		if (aStatus.IsError()) return;

		SQLRETURN rRetcode = SdbODBC3Connection::OpenConnection(m_aConnectionHandle, pConnection->ConnectStr(), TRUE);
		SetStatus(rRetcode,m_aConnectionHandle);
		if (aStatus.IsError()) return;

		// Statement allozieren (ein ODBC-Statement pro SdbCursor).
		rRetcode = N3SQLAllocHandle(SQL_HANDLE_STMT,m_aConnectionHandle,&m_aStatementHandle);
		SetStatus(rRetcode,m_aStatementHandle);
		if (aStatus.IsError()) return;
	}

	DetermineOptions(pConnection);
	SetOptions(pConnection);
}
//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::IsCapable(SdbCursorCapability eCapability) const
{
	BOOL bErg = FALSE;
	SdbODBC3Connection* pConnection = (SdbODBC3Connection*) GetODBCConnection();

	BOOL bSetCon = FALSE;
	SQLUINTEGER nAttr1 = 0;
	SQLUINTEGER nAttr2 = 0;
	// Cursor kann nur in eine Richtung bewegt werden
	if ((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY)
	{
		nAttr1 = pConnection->m_nForwardOnlyAttr1;
		nAttr2 = pConnection->m_nForwardOnlyAttr2;
	}
	else if (eCursorType == SDB_DYNAMIC || eCursorType == SDB_KEYSET)
	{
		nAttr1 = pConnection->m_nKeySetAttr1;
		nAttr2 = pConnection->m_nKeySetAttr2;
	}
	else
	{
		nAttr1 = pConnection->m_nStaticAttr1;
		nAttr2 = pConnection->m_nStaticAttr2;
	}

	switch(eCapability)
	{
		case SDB_CAP_BOOKMARKS:
			bErg = bBookmarks;
			break;
		case SDB_CAP_UPDATE_POSITIONED:
			bErg = !((nOptions & SDB_READONLY) > 0) && (nAttr1 & (SQL_CA1_POS_UPDATE|SQL_CA1_POS_DELETE|SQL_CA1_BULK_ADD))>0;
			break;

		case SDB_CAP_FETCH:					bErg = TRUE; break;
		case SDB_CAP_FETCH_NEXT:			bErg = (SQL_FETCH_NEXT	 & nAttr1) > 0;	break;
		case SDB_CAP_FETCH_RELATIVE:		bErg = (SQL_CA1_RELATIVE & nAttr1) > 0;	break;
		case SDB_CAP_FETCH_PRIOR:			bErg = (SQL_FETCH_PRIOR	 & nAttr1) > 0;	break;
		case SDB_CAP_FETCH_BOOKMARK:		bErg = (SQL_CA1_BOOKMARK & nAttr1) > 0;	break;
		case SDB_CAP_FETCH_FIRST:
		case SDB_CAP_FETCH_LAST:
		case SDB_CAP_FETCH_ABSOLUTE:		bErg = (SQL_CA1_ABSOLUTE & nAttr1) > 0;	break;
		//	case SDB_CAP_UPDATE_INSERTED_ROW:	bErg = (SQL_CA1_ABSOLUTE & nAttr1) > 0;	break;

	}
	return bErg;
}
//-----------------------------------------------------------------------------
void SdbODBC3Cursor::DetermineOptions(SdbODBCConnection* pCon)
{
	SdbODBC3Connection* pConnection = (SdbODBC3Connection*) pCon;

	BOOL bSetCon = FALSE;
	SQLUINTEGER nAttr = 0;
	// Cursor kann nur in eine Richtung bewegt werden
	if ((nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY)
	{
		nCursorType = SQL_CURSOR_FORWARD_ONLY;	// default
		nAttr = pConnection->m_nForwardOnlyAttr2;
		bSetCon = ((pConnection->m_nForwardOnlyAttr1 & SQL_CA1_POS_UPDATE) > 0) && !(nOptions & SDB_READONLY);
		bBookmarks = FALSE;
	}
	else if (eCursorType == SDB_DYNAMIC || eCursorType == SDB_KEYSET)
	{
		nCursorType = SQL_CURSOR_KEYSET_DRIVEN;
		nAttr = pConnection->m_nKeySetAttr2;
		bBookmarks = ((pConnection->m_nKeySetAttr1 & SQL_CA1_BOOKMARK) == SQL_CA1_BOOKMARK);
		bSetCon	   = ((pConnection->m_nKeySetAttr1 & SQL_CA1_POS_UPDATE) > 0) && !(nOptions & SDB_READONLY);
	}
	else
	{
		nCursorType = SQL_CURSOR_STATIC;
		nAttr = pConnection->m_nStaticAttr2;
		bSetCon = ((pConnection->m_nStaticAttr1 & SQL_CA1_POS_UPDATE) > 0) && !((nOptions & SDB_READONLY) > 0);
		bBookmarks = ((pConnection->m_nStaticAttr1 & SQL_CA1_BOOKMARK) == SQL_CA1_BOOKMARK);
	}

	// Updatefaehigkeiten und Cursortyp setzen
	nConcurrency = SQL_CONCUR_READ_ONLY;
	if (bSetCon)
	{
		if (nAttr & SQL_CA2_OPT_ROWVER_CONCURRENCY)
			nConcurrency = SQL_CONCUR_ROWVER;
		else if (nAttr & SQL_CA2_OPT_VALUES_CONCURRENCY)
			nConcurrency = SQL_CONCUR_VALUES;
		else
			nConcurrency = SQL_CONCUR_LOCK;
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::SetOptions(SdbODBCConnection* pConnection)
{
	if (bBookmarks)	//BOOKmarks verwenden
	{
		SetStatus(N3SQLSetStmtAttr(m_aStatementHandle,
									SQL_ATTR_USE_BOOKMARKS,
									(SQLPOINTER)SQL_UB_VARIABLE, SQL_IS_INTEGER),m_aStatementHandle);
		bBookmarks = aStatus.IsSuccessful();
	}

	if (nCursorType != SQL_CURSOR_FORWARD_ONLY)	// default
	{
		SetStatus(N3SQLSetStmtAttr(m_aStatementHandle,
									SQL_ATTR_CURSOR_TYPE,
									(SQLPOINTER)nCursorType,SQL_IS_INTEGER),m_aStatementHandle);
		aStatus.Clear();

		if (nConcurrency != SQL_CONCUR_READ_ONLY)
			SetStatus(N3SQLSetStmtAttr(m_aStatementHandle,
										SQL_ATTR_CONCURRENCY,
										(SQLPOINTER)nConcurrency,SQL_IS_INTEGER),m_aStatementHandle);
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::SetStatus(INT16 nRetcode, void* hstmt,const String& rErrorInfo)
{
	if(hstmt == m_aStatementHandle)
		SdbODBC3_SetStatus(aStatus,nRetcode,SQL_HANDLE_STMT,hstmt,rErrorInfo);
	else
		SdbODBC3_SetStatus(aStatus,nRetcode,SQL_HANDLE_DBC,hstmt,rErrorInfo);
}

//-----------------------------------------------------------------------------
SdbODBC3Cursor::~SdbODBC3Cursor()
{
	DBG_DTOR(SdbODBC3Cursor,NULL);
	// (RemovingFromParent isn't virtual in 557)
	DBG_ASSERT(GetParent() || m_aStatementHandle == SQL_NULL_HANDLE, "SdbODBC2Cursor::~SdbODBC2Cursor : RemovingFromParent should have been called if I have no parent anymore !");
	if (m_aStatementHandle != SQL_NULL_HANDLE)
	{
		N3SQLFreeStmt(m_aStatementHandle,SQL_DROP);
		m_aStatementHandle = SQL_NULL_HANDLE;

		if (m_aConnectionHandle != SQL_NULL_HANDLE)	// extra Verbindung
		{
			N3SQLDisconnect(m_aConnectionHandle);
			N3SQLFreeHandle(SQL_HANDLE_DBC,m_aConnectionHandle);
		}
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::RemovingFromParent()
{
	DBG_ASSERT(!IsOpen(), "SdbODBC3Cursor::RemovingFromParent : expected to be called _after_ Close !");

	// drop our statement. We can't do this later as the connection - which is our parent - may be destroyed then.
	if (m_aStatementHandle != SQL_NULL_HANDLE)
	{
		N3SQLFreeStmt(m_aStatementHandle,SQL_DROP);
		m_aStatementHandle = SQL_NULL_HANDLE;

		if (m_aConnectionHandle != SQL_NULL_HANDLE)	// extra Verbindung
		{
			N3SQLDisconnect(m_aConnectionHandle);
			N3SQLFreeHandle(SQL_HANDLE_DBC,m_aConnectionHandle);
		}
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::CancelExecution()
{
	if (IsCoreCall())
	{
		N3SQLCancel(m_aStatementHandle);
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::PrepareODBC()
{
	SQLRETURN nRetcode;
	{
		SdbCancelableCoreCall aCoreCallFlag(*this);
		ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
		nRetcode = N3SQLPrepare(m_aStatementHandle,(SDB_ODBC_CHAR *) aSql.GetBuffer(),aSql.Len());
	}

	SetStatus(nRetcode,m_aStatementHandle,aSQLStatement);
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::AllocBuffer(void**& pDataBuffer, INT32*& pLenBuffer, const SdbColumns& rCols, BOOL bBindBookmark)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);
	UINT16 nCount(rCols.Count());
	if (nCount <= 1)		// keine columns
		return;

	DBG_ASSERT(pDataBuffer == NULL,"Interner Fehler");
	pDataBuffer = (void**) new char*[nCount];
	pLenBuffer = new INT32[nCount];
	memset(pLenBuffer, 0, nCount * sizeof(INT32));
	memset(pDataBuffer, 0, nCount * sizeof(char*));

	for (UINT16 i= bBindBookmark ? 0 : 1; i < nCount; i++)
	{
		const SdbColumn& rCol	= *rCols[i];
		INT32& rLength			= pLenBuffer[i];
		rLength					= rCol.GetLength();

		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					pDataBuffer[i] = new rtl::OUString;
				else
					pDataBuffer[i] = new ByteString;
				break;
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
				rLength += 2;
				if(GetODBCConnection()->m_bUserWChar)
					pDataBuffer[i] = new rtl::OUString;
				else
					pDataBuffer[i] = new ByteString;
				break;
			case SDB_DBTYPE_DOUBLE:
				rLength = sizeof(double);
				pDataBuffer[i] = new double;
				*(double*)pDataBuffer[i] = 0.0;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					pDataBuffer[i] = new sal_Unicode[2];
				else
					pDataBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_UNKNOWN:
				rLength = 0;
				pDataBuffer[i] = new char[2];  // dient nur zum auffinden
				break;
			case SDB_DBTYPE_DATE:
				rLength = sizeof(DATE_STRUCT);
				pDataBuffer[i] = new DATE_STRUCT;
				break;
			case SDB_DBTYPE_TIME:
				rLength = sizeof(TIME_STRUCT);
				pDataBuffer[i] = new TIME_STRUCT;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				rLength = sizeof(TIMESTAMP_STRUCT);
				pDataBuffer[i] = new TIMESTAMP_STRUCT;
				break;
			case SDB_DBTYPE_BOOLEAN:
				rLength = sizeof(INT8);
				pDataBuffer[i] = new INT8;
				*(INT8*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				rLength = sizeof(INT16);
				pDataBuffer[i] = new INT16;
				*(INT16*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_INTEGER:
				rLength = sizeof(INT32);
				pDataBuffer[i] = new INT32;
				*(INT32*)pDataBuffer[i] = 0;
				break;
			case SDB_DBTYPE_REAL:
				rLength = sizeof(float);
				pDataBuffer[i] = new float;
				*(float*)pDataBuffer[i] = (float)0;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
			case SDB_DBTYPE_BOOKMARK:
				pDataBuffer[i] = new BYTE[rLength];
				break;
		}
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::FreeBuffer(void**& pDataBuffer, INT32*& pLenBuffer, const SdbColumns& rCols, BOOL bBindBookmark)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);
	UINT16 nCount = rCols.Count();
	if (nCount <= 1) return;

	delete pLenBuffer;
	pLenBuffer = NULL;

	if (pDataBuffer == NULL)
		return;

	for (UINT16 i= bBindBookmark ? 0 : 1; i < nCount; i++)
	{
		const SdbColumn& rCol = *rCols[i];
		void* pData = pDataBuffer[i];
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_CURRENCY:
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
				DBG_ASSERT(pData,"is null");
				if(GetODBCConnection()->m_bUserWChar)
					delete (rtl::OUString*)pData;
				else
					delete (ByteString*)pData;
				pData = NULL;
				break;
			case SDB_DBTYPE_LONGVARCHAR:
				DBG_ASSERT(pData,"is null");
				if(GetODBCConnection()->m_bUserWChar)
					delete (sal_Unicode*)pData;
				else
					delete (char*)pData;
				pData = NULL;
				break;
			case SDB_DBTYPE_LONGVARBINARY:
			case SDB_DBTYPE_UNKNOWN:
				delete (char*)pData;
				break;
			case SDB_DBTYPE_INTEGER:
				delete (INT32*)pData;
				break;
			case SDB_DBTYPE_BOOLEAN:
				delete (INT8*)pData;
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				delete (INT16*)pData;
				break;
			case SDB_DBTYPE_REAL:
				delete (float*)pData;
				break;
			case SDB_DBTYPE_DOUBLE:
				delete (double*)pData;
				break;
			case SDB_DBTYPE_DATE:
				delete (DATE_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIME:
				delete (TIME_STRUCT*)pData;
				break;
			case SDB_DBTYPE_TIMESTAMP:
				delete (TIMESTAMP_STRUCT*)pData;
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
			case SDB_DBTYPE_BOOKMARK:
				delete (BYTE *) pData;
				break;
			default:
				DBG_ERROR("Fehlerhafter DatenTyp");
		}
	}
	__DELETE(nCount) pDataBuffer;
	pDataBuffer = NULL;
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::DescribeResult()
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aResultColumns.Is(), "keine Ergebnisbeschreibung");
	// Bookmark-Column ist immer da (schon durch den Konstruktor erzeugt).

	SdbODBCConnection* pConnection = GetODBCConnection();

	if (aResultColumns->Count() > 1 || !pConnection)
		return;

	// Alle ResultColumns ansehen
	SWORD nNumResultCols=0;
	SQLRETURN nRetcode = N3SQLNumResultCols(m_aStatementHandle,&nNumResultCols);
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	SWORD pcbColName;			// Laenge des Names
	SWORD pfSqlType;			// SQL-Datentyp
	UDWORD pcbColDef;			// Precision (ggf. 0)
	SWORD  pibScale;				// Scale (ggf. 0)
	SWORD  pfNullable;			// NULL erlaubt?
	INT16  nFlags = 0;
	SdbDatabaseType eType;
	ByteString aColumnName;			// Spaltenname
	SDWORD nNumTemp = nNumResultCols; // hack um sicher zustellen das dieser Wert nicht mehr ver"andert werden kann
	SDB_ODBC_CHAR pTmp[DESC_COL_NAME_LEN];

	if (bBookmarks)
	{
		// Get the length of the bookmark column
		nRetcode =N3SQLDescribeCol(m_aStatementHandle,
								   0,
								   pTmp,
								   DESC_COL_NAME_LEN,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
		SetStatus(nRetcode,m_aStatementHandle);
		if (!aStatus.IsSuccessful())
			return;

		m_nBookmarkLen = pcbColDef;

		// const cast
		((SdbColumn*)(*aResultColumns)[0])->SetLength(pcbColDef);
	}

	for (SWORD i = 1; i <= nNumTemp; i++)
	{
		// Informationen von SQLDescribeCol:
		// erstmal die L"ange des Spaltennamens

		nFlags = pibScale = pfNullable = pcbColName = 0;
		pcbColDef = 0l;
		pTmp[0] = ' ';
		pTmp[1] = '\0';
		nRetcode =N3SQLDescribeCol(m_aStatementHandle,
								   i,
								   pTmp,
								   DESC_COL_NAME_LEN,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
		SetStatus(nRetcode,m_aStatementHandle);
		if(aStatus.SQLState() == aTruncState)
		{
			// first try mit Lnge == 0
			nRetcode =N3SQLDescribeCol(m_aStatementHandle,
								   i,
								   pTmp,
								   0,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
			if(!pcbColName) // second try da pcbColName == 0
			{
				pTmp[0] = ' ';
				pTmp[1] = '\0';
				nRetcode =N3SQLDescribeCol(m_aStatementHandle,
								   i,
								   pTmp,
								   1,
								   &pcbColName,
								   &pfSqlType,
								   &pcbColDef,
								   &pibScale,
								   &pfNullable
								 );
			}
			DBG_ASSERT(pcbColName>0,"ODBC: pcbColName <= 0. Treiber verhlt sich nicht wie blich! ;-(");
			SDB_ODBC_CHAR* pStr = (SDB_ODBC_CHAR*)aColumnName.AllocBuffer(pcbColName+1);
			nRetcode =N3SQLDescribeCol(m_aStatementHandle,
									   i,
									   pStr,
									   pcbColName+1,
									   &pcbColName,
									   &pfSqlType,
									   &pcbColDef,
									   &pibScale,
									   &pfNullable
									 );
			aColumnName.ReleaseBufferAccess();  // wieder freigeben
			SetStatus(nRetcode,m_aStatementHandle);
		}
		else if (aStatus.IsSuccessful())
			aColumnName = (char*)pTmp;

		if (!aStatus.IsSuccessful())
			return;



		switch(pfSqlType)
		{
			case SQL_WCHAR:
			case SQL_CHAR:			eType = SDB_DBTYPE_CHAR; break;
			case SQL_WVARCHAR:
			case SQL_VARCHAR: 		eType = SDB_DBTYPE_VARCHAR; break;
			case SQL_WLONGVARCHAR:
			case SQL_LONGVARCHAR: 	eType = SDB_DBTYPE_LONGVARCHAR; break;
			case SQL_DECIMAL:		eType = SDB_DBTYPE_DECIMAL; break;
			case SQL_NUMERIC:		eType = SDB_DBTYPE_NUMERIC; break;
			case SQL_BIT:
				eType = SDB_DBTYPE_BOOLEAN;
				if(!pcbColDef)
				{
					pcbColDef = 1; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_TINYINT:
				eType = SDB_DBTYPE_TINYINT;
				if(!pcbColDef)
				{
					pcbColDef = 3; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_SMALLINT:
				eType = SDB_DBTYPE_SMALLINT;
				if(!pcbColDef)
				{
					pcbColDef = 5; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_INTEGER:
				eType = SDB_DBTYPE_INTEGER;
				if(!pcbColDef)
				{
					pcbColDef = 10; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			//case SQL_BIGINT:
				//eType = SDB_DBTYPE_BIGINT; break;
			case SQL_REAL:
				eType = SDB_DBTYPE_REAL;
				if(!pcbColDef)
				{
					pcbColDef = 7; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_FLOAT:
			case SQL_DOUBLE:
			case SQL_BIGINT:
				eType = SDB_DBTYPE_DOUBLE;
				if(!pcbColDef)
				{
					pcbColDef = 15; // ODBC_STANDARD
					DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
				}
				break;
			case SQL_BINARY:		eType = SDB_DBTYPE_BINARY; break;
			case SQL_VARBINARY: 	eType = SDB_DBTYPE_VARBINARY; break;
			case SQL_LONGVARBINARY:	eType = SDB_DBTYPE_LONGVARBINARY; break;
			case SQL_TYPE_DATE:
			case SQL_DATE: 			eType = SDB_DBTYPE_DATE; break;
			case SQL_TYPE_TIME:
			case SQL_TIME: 			eType = SDB_DBTYPE_TIME; break;
			case SQL_TYPE_TIMESTAMP:
			case SQL_TIMESTAMP: 	eType = SDB_DBTYPE_TIMESTAMP; break;
			default:
				eType = SDB_DBTYPE_UNKNOWN; // wird nicht gebunden
		}
		if (pfNullable != SQL_NO_NULLS)
			nFlags |= SDB_FLAGS_NULLALLOWED;

		if (!(nOptions & SDB_MINIMUM_DESC) && eType != SDB_DBTYPE_UNKNOWN)
		{
			SDWORD nExtAttrib;	// erweitere Attribute
			if (N3SQLColAttribute(m_aStatementHandle,		// ist die Column updatebar?
									 i,
									 SQL_DESC_UPDATABLE,
									 NULL,
									 0,
									 NULL,
									 &nExtAttrib
									) == SQL_SUCCESS)
			{
				if (nExtAttrib != SQL_ATTR_READONLY)
					nFlags |= SDB_FLAGS_UPDATEALLOWED;
			}

			if (N3SQLColAttribute(m_aStatementHandle,	    // handelt es sich um ein MoneyFeld
										 i,
										 SQL_DESC_FIXED_PREC_SCALE,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib) eType = SDB_DBTYPE_CURRENCY;
			}

			if (N3SQLColAttribute(m_aStatementHandle,		// ist die Column AUTO_INCREMENT?
										 i,
										 SQL_DESC_AUTO_UNIQUE_VALUE,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				if (nExtAttrib)
				{
					nFlags |= SDB_FLAGS_AUTOINCREMENT;
					nFlags &= ~SDB_FLAGS_UPDATEALLOWED;	    // darf nicht updatebar sein
				}
			}

			if (N3SQLColAttribute(m_aStatementHandle,		// ist die Column SEARCHABLE?
										 i,
										 SQL_DESC_SEARCHABLE,
										 NULL,
										 0,
										 NULL,
										 &nExtAttrib
										) == SQL_SUCCESS)
			{
				switch (nExtAttrib)
				{
					case SQL_PRED_SEARCHABLE:	nFlags |= SDB_FLAGS_SEARCH_ALL; break;
					case SQL_PRED_CHAR:			nFlags |= SDB_FLAGS_SEARCH_LIKE; break;
					case SQL_PRED_BASIC:		nFlags |= SDB_FLAGS_SEARCH_EXCEPT_LIKE; break;
				}
			}
		}

		// Column-Beschreibung hinzufuegen.
		// existiert Name  in aResultColumns ?
		String aAlias(aColumnName,gsl_getSystemTextEncoding());
		const SdbColumn* pPrevCol = aResultColumns->Column(aAlias);

		ULONG nExprCnt(0);
		while(pPrevCol)
		{
			(aAlias = String(aColumnName,gsl_getSystemTextEncoding())) += String::CreateFromInt32(++nExprCnt);
			pPrevCol = aResultColumns->Column(aAlias);
		}
		SdbColumn * pColumn = GetODBCConnection()->CreateColumn(aAlias,eType,pcbColDef,pibScale, nFlags);
		pColumn->SetRealName(String(aColumnName,gsl_getSystemTextEncoding()));

		aResultColumns->AddColumn(pColumn);
	}
	// Strukturbeschreibung fertig.

	// relect row states if neccessary
	if (aResultColumns->Count() > 1)
	{
		N3SQLSetStmtAttr(m_aStatementHandle,
						 SQL_ATTR_ROW_ARRAY_SIZE,
						(SQLPOINTER)1L, SQL_IS_INTEGER);

		N3SQLSetStmtAttr(m_aStatementHandle,
						 SQL_ATTR_ROW_STATUS_PTR,
						 (SQLPOINTER)&nRowStatus, SQL_IS_INTEGER);
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::DescribeParameter()
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	DBG_ASSERT(aParameterColumns.Is(), "keine Parameterspalten angelegt!");

	// Bookmark-Column ist immer da (schon durch den Konstruktor erzeugt).

	// Anzahl Parameter ermitteln:
	SWORD nNumParams;
	SdbODBCConnection* pConnection = GetODBCConnection();

	if (!pConnection || !pConnection->IsODBCFunctionAvailable(SQL_API_SQLNUMPARAMS))
		return;

	SetStatus(N3SQLNumParams(m_aStatementHandle,&nNumParams),m_aStatementHandle);
	if (!aStatus.IsSuccessful() || !nNumParams)
		return;

	// steht die Funktion zur Verfuegung
	if (!pConnection->IsODBCFunctionAvailable(SQL_API_SQLDESCRIBEPARAM))
		SdbCursor::DescribeParameter();
	else
	{
		for (UWORD i = 1; i <= (UWORD)nNumParams; i++)
		{
			SdbColumn * pColumn = NULL;
			String aParameterName(aParameterNames.GetToken(i-1));
			// Name == "?", falls kein Parametername verfuegbar (z. B. bei Native SQL)
			if (aParameterName.Len() == 0)
				aParameterName = '?';

			SWORD pfSqlType=0;			// SQL-Datentyp
			UDWORD pcbColDef=0L;			// Precision (ggf. 0)
			SWORD pibScale=0;				// Scale (ggf. 0)
			SWORD pfNullable=0;			// NULL erlaubt?
			SdbDatabaseType eType;


			aStatus.Clear();
			SQLRETURN nRetcode = N3SQLDescribeParam(m_aStatementHandle,
										 i,
										 &pfSqlType,
										 &pcbColDef,
										 &pibScale,
										 &pfNullable
										 );
			SetStatus(nRetcode,m_aStatementHandle);
			if (! aStatus.IsSuccessful()) return;

			switch(pfSqlType)
			{
				case SQL_WCHAR:
				case SQL_CHAR:			eType = SDB_DBTYPE_CHAR; break;
				case SQL_WVARCHAR:
				case SQL_VARCHAR: 		eType = SDB_DBTYPE_VARCHAR; break;
				case SQL_WLONGVARCHAR:
				case SQL_LONGVARCHAR: 	eType = SDB_DBTYPE_LONGVARCHAR; break;
				case SQL_DECIMAL:		eType = SDB_DBTYPE_DECIMAL; break;
				case SQL_NUMERIC:		eType = SDB_DBTYPE_NUMERIC; break;
				case SQL_BIT:
					eType = SDB_DBTYPE_BOOLEAN;
					if(!pcbColDef)
					{
						pcbColDef = 1; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_TINYINT:
					eType = SDB_DBTYPE_TINYINT;
					if(!pcbColDef)
					{
						pcbColDef = 3; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_SMALLINT:
					eType = SDB_DBTYPE_SMALLINT;
					if(!pcbColDef)
					{
						pcbColDef = 5; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_INTEGER:
					eType = SDB_DBTYPE_INTEGER;
					if(!pcbColDef)
					{
						pcbColDef = 10; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				//case SQL_BIGINT: 		eType = SDB_DBTYPE_BIGINT; break;
				case SQL_REAL:
					eType = SDB_DBTYPE_REAL;
					if(!pcbColDef)
					{
						pcbColDef = 7; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_FLOAT:
				case SQL_DOUBLE:
				case SQL_BIGINT:
					eType = SDB_DBTYPE_DOUBLE;
					if(!pcbColDef)
					{
						pcbColDef = 15; // ODBC_STANDARD
						DBG_WARNING("Der verwendete Treiber gibt hier keine vollstngdigen Daten wider!");
					}
					break;
				case SQL_BINARY:		eType = SDB_DBTYPE_BINARY; break;
				case SQL_VARBINARY: 	eType = SDB_DBTYPE_VARBINARY; break;
				case SQL_LONGVARBINARY:	eType = SDB_DBTYPE_LONGVARBINARY; break;
				case SQL_TYPE_DATE:
				case SQL_DATE: 			eType = SDB_DBTYPE_DATE; break;
				case SQL_TYPE_TIME:
				case SQL_TIME: 			eType = SDB_DBTYPE_TIME; break;
				case SQL_TYPE_TIMESTAMP:
				case SQL_TIMESTAMP: 	eType = SDB_DBTYPE_TIMESTAMP; break;
				default:				eType = SDB_DBTYPE_UNKNOWN; // wird nicht gebunden
			}

			pColumn = pConnection->CreateColumn(aParameterName,eType,pcbColDef,pibScale);
			if (pfNullable == SQL_NULLABLE)
				pColumn->SetFlag(SDB_FLAGS_NULLALLOWED);
			pColumn->SetFlag(SDB_FLAGS_PARAM_IN);

			// Column-Beschreibung hinzufuegen.
			aParameterColumns->AddColumn(pColumn);
		}
	}
	// Parameter-Strukturbeschreibung fertig.
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::BindColumn(UINT16 nPos, void** pDataBuffer, INT32* pLenBuffer,
							    const SdbColumn& rCol, const ODbVariantRef& pVariable, BOOL bParam)
{
	SQLRETURN nRetcode;
	SWORD   fSqlType;
	SWORD   fCType;
	SDWORD  nMaxLen = 0;
	void*   pData   = pDataBuffer[nPos];
	SDWORD* pLen    = &pLenBuffer[nPos];

	switch(rCol.GetType())
	{
		case SDB_DBTYPE_CHAR:			if(GetODBCConnection()->m_bUserWChar)
										{
											fCType   = SQL_C_WCHAR;
											fSqlType = SQL_WCHAR;
										}
										else
										{
											fCType   = SQL_C_CHAR;
											fSqlType = SQL_CHAR;
										}
										break;
		case SDB_DBTYPE_VARCHAR:		if(GetODBCConnection()->m_bUserWChar)
										{
											fCType   = SQL_C_WCHAR;
											fSqlType = SQL_WVARCHAR;
										}
										else
										{
											fCType   = SQL_C_CHAR;
											fSqlType = SQL_VARCHAR;
										}
										break;
		case SDB_DBTYPE_LONGVARCHAR:	if(GetODBCConnection()->m_bUserWChar)
										{
											fCType   = SQL_C_WCHAR;
											fSqlType = SQL_WLONGVARCHAR;
										}
										else
										{
											fCType	 = SQL_C_CHAR;
											fSqlType = SQL_LONGVARCHAR;
										}
										break;
		case SDB_DBTYPE_DECIMAL:		fCType		= GetODBCConnection()->m_bUserWChar ? SQL_C_WCHAR : SQL_C_CHAR;
										fSqlType	= SQL_DECIMAL; break;
		case SDB_DBTYPE_CURRENCY:
		case SDB_DBTYPE_NUMERIC:		fCType		= GetODBCConnection()->m_bUserWChar ? SQL_C_WCHAR : SQL_C_CHAR;
										fSqlType	= SQL_NUMERIC; break;
		case SDB_DBTYPE_BOOLEAN:		fCType		= SQL_C_TINYINT;
										fSqlType	= SQL_INTEGER; break;
		case SDB_DBTYPE_TINYINT:		fCType		= SQL_C_SHORT;
										fSqlType	= SQL_TINYINT; break;
		case SDB_DBTYPE_SMALLINT:		fCType		= SQL_C_SHORT;
										fSqlType	= SQL_SMALLINT; break;
		case SDB_DBTYPE_INTEGER:		fCType		= SQL_C_LONG;
										fSqlType	= SQL_INTEGER; break;
		case SDB_DBTYPE_BIGINT:			fCType		= GetODBCConnection()->m_bUserWChar ? SQL_C_WCHAR : SQL_C_CHAR;
										fSqlType	= SQL_BIGINT; break;
		case SDB_DBTYPE_REAL:			fCType		= SQL_C_FLOAT;
										fSqlType	= SQL_REAL; break;
		case SDB_DBTYPE_DOUBLE:			fCType		= SQL_C_DOUBLE;
										fSqlType	= SQL_DOUBLE; break;
		case SDB_DBTYPE_BINARY:			fCType		= SQL_C_BINARY;
										fSqlType	= SQL_BINARY; break;
		case SDB_DBTYPE_VARBINARY:
		case SDB_DBTYPE_BOOKMARK:		fCType		= SQL_C_BINARY;
										fSqlType	= SQL_VARBINARY; break;
		case SDB_DBTYPE_LONGVARBINARY: 	fCType		= SQL_C_BINARY;
										fSqlType	= SQL_LONGVARBINARY; break;
		case SDB_DBTYPE_DATE:
									if(((SdbODBC3Connection*)GetODBCConnection())->m_bUseOldTimeDate)
									{
										fCType		= SQL_C_DATE;
										fSqlType	= SQL_DATE;
									}
									else
									{
										fCType		= SQL_C_TYPE_DATE;
										fSqlType	= SQL_TYPE_DATE;
									}
									break;
		case SDB_DBTYPE_TIME:
									if(((SdbODBC3Connection*)GetODBCConnection())->m_bUseOldTimeDate)
									{
										fCType		= SQL_C_TIME;
										fSqlType	= SQL_TIME;
									}
									else
									{
										fCType		= SQL_C_TYPE_TIME;
										fSqlType	= SQL_TYPE_TIME;
									}
									break;
		case SDB_DBTYPE_TIMESTAMP:
									if(((SdbODBC3Connection*)GetODBCConnection())->m_bUseOldTimeDate)
									{
										fCType		= SQL_C_TIMESTAMP;
										fSqlType	= SQL_TIMESTAMP;
									}
									else
									{
										fCType		= SQL_C_TYPE_TIMESTAMP;
										fSqlType	= SQL_TYPE_TIMESTAMP;
									}
									break;
		default:						fCType		= SQL_C_BINARY;
										fSqlType	= SQL_LONGVARBINARY; break;
	}

	SWORD fParamType =	rCol.IsInParameter() ?
							( rCol.IsOutParameter() ? 	SQL_PARAM_INPUT_OUTPUT : SQL_PARAM_INPUT)
											 :	SQL_PARAM_OUTPUT;

	if (nPos != 0 && !pVariable->isModified() && !bParam)
	{
		*pLen = SQL_COLUMN_IGNORE;
		nRetcode = N3SQLBindCol(m_aStatementHandle,
									nPos,
									fCType,
									pLen,
									nMaxLen,
									pLen
									);
	}
	else if (nPos != 0 && pVariable->isNull())
	{
		*pLen = SQL_NULL_DATA;
		if (bParam)
			nRetcode = N3SQLBindParameter(m_aStatementHandle,
									nPos,
									fParamType,
									fCType,
									fSqlType,
									rCol.GetPrecision(),
									rCol.GetScale(),
									NULL,
									nMaxLen,
									pLen
									);
		else
			nRetcode = N3SQLBindCol(m_aStatementHandle,
									nPos,
									fCType,
									pData,
									nMaxLen,
									pLen
									);
	}
	else
	{
		try
		{
			switch (rCol.GetType())
			{
				case SDB_DBTYPE_CHAR:
				case SDB_DBTYPE_VARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
				{
					rtl::OUString aString(pVariable->toString());
					//	aString.Convert(CHARSET_SYSTEM,GetConnection()->GetCharacterSet());
					nMaxLen = rCol.GetPrecision();
					*pLen = SQL_NTS;
					*((rtl::OUString*)pData) = aString;

					// Zeiger auf Char*
					pData = (void*)((rtl::OUString*)pData)->getStr();
				}
				else
				{
					ByteString aString(pVariable->toString().getStr(),GetConnection()->GetCharacterSet());
					nMaxLen = rCol.GetPrecision();
					*pLen = SQL_NTS;
					*((ByteString*)pData) = aString;

					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}	break;
				case SDB_DBTYPE_BIGINT:
				case SDB_DBTYPE_DECIMAL:
				case SDB_DBTYPE_CURRENCY:
				case SDB_DBTYPE_NUMERIC:
				if(GetODBCConnection()->m_bUserWChar)
				{
					rtl::OUString aString(SdbTools::ToString(ODbTypeConversion::toDouble(*pVariable),rCol.GetScale()));
					nMaxLen = aString.len();
					*pLen = nMaxLen;
					*((rtl::OUString*)pData) = aString;
					// Zeiger auf Char*
					pData = (void*)((rtl::OUString*)pData)->getStr();
				}
				else
				{
					ByteString aString(SdbTools::ToString(ODbTypeConversion::toDouble(*pVariable),rCol.GetScale()),gsl_getSystemTextEncoding());
					nMaxLen = aString.Len();
					*pLen = nMaxLen;
					*((ByteString*)pData) = aString;
					// Zeiger auf Char*
					pData = (void*)((ByteString*)pData)->GetBuffer();
				}	break;
				case SDB_DBTYPE_BOOLEAN:
					*((INT8*)pData) = ODbTypeConversion::toBOOL(*pVariable) ? 1 : 0;
					*pLen = sizeof(INT8);
					break;
				case SDB_DBTYPE_TINYINT:
				case SDB_DBTYPE_SMALLINT:
					*((INT16*)pData) = ODbTypeConversion::toINT16(*pVariable);
					*pLen = sizeof(INT16);
					break;
				case SDB_DBTYPE_INTEGER:
					*((INT32*)pData) = ODbTypeConversion::toINT32(*pVariable);
					*pLen = sizeof(INT32);
					break;
				case SDB_DBTYPE_REAL:
					*((float*)pData) = ODbTypeConversion::toFloat(*pVariable);
					*pLen = sizeof(float);
					break;
				case SDB_DBTYPE_DOUBLE:
					*((double*)pData) = ODbTypeConversion::toDouble(*pVariable);
					*pLen = sizeof(double);
					break;
				case SDB_DBTYPE_BOOKMARK:
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
					{
						memset(pData,0, *pLen);
						*pLen = rCol.GetPrecision();
						nMaxLen = rCol.GetPrecision();
						break;
					}	// otherwise continue!
				case SDB_DBTYPE_BINARY:
				case SDB_DBTYPE_VARBINARY:
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
					{
						const ::staruno::Sequence<sal_Int8>& rData = pVariable->getBytes();
						pData = (void*)rData.getConstArray();
						*pLen = rData.getLength();
						nMaxLen = rCol.GetPrecision();
					}	break;
				case SDB_DBTYPE_UNKNOWN:
				case SDB_DBTYPE_LONGVARBINARY:
				case SDB_DBTYPE_LONGVARCHAR:
				{
					pData  = (void*)&rCol;
					INT32 nLen = 0;
					if (pVariable->getValueType() == ::getCppuType((const ::com::sun::star::uno::Sequence< sal_Int8 > *)0))
						nLen = pVariable->getBytes().getLength();
					else if(GetODBCConnection()->m_bUserWChar)
						nLen = sizeof(sal_Unicode)*ODbTypeConversion::toString(*pVariable).len();
					else
						nLen = ODbTypeConversion::toString(*pVariable).len();
					*pLen = (SDWORD)SQL_LEN_DATA_AT_EXEC(nLen);
				}   break;
				case SDB_DBTYPE_DATE:
					ToDate(*(DATE_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(DATE_STRUCT);
					break;
				case SDB_DBTYPE_TIME:
					ToTime(*(TIME_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(TIME_STRUCT);
					break;
				case SDB_DBTYPE_TIMESTAMP:
					ToTimeStamp(*(TIMESTAMP_STRUCT*)pData, ODbTypeConversion::toDouble(*pVariable));
					*pLen = sizeof(TIMESTAMP_STRUCT);
			}
		}
		catch ( ... )
		{
			String strError(SdbResId(STR_STAT_VALUE_CAN_NOT_CONVERTED));
			strError.SearchAndReplace(String::CreateFromAscii("$name$"), rCol.GetName());
			aStatus.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aStatus.CreateErrorMessage(strError), 0, String());
			return FALSE;
		}


		if (bParam)
			nRetcode = N3SQLBindParameter(m_aStatementHandle,
						  nPos,
						  fParamType,
						  fCType,
						  fSqlType,
						  rCol.GetPrecision(),
						  rCol.GetScale(),
						  pData,
						  nMaxLen,
						  pLen);
		else
			nRetcode = N3SQLBindCol(m_aStatementHandle,
									nPos,
									fCType,
									pData,
									nMaxLen,
									pLen
									);
	}

	if(bParam && (fParamType == SQL_PARAM_INPUT_OUTPUT || fParamType == SQL_PARAM_OUTPUT))
	{
		switch (rCol.GetType())
		{
			case SDB_DBTYPE_INTEGER:
				pVariable->setINT32(*(INT32*)pLen);
				break;
			case SDB_DBTYPE_DECIMAL:
			case SDB_DBTYPE_NUMERIC:
			case SDB_DBTYPE_BIGINT:
			case SDB_DBTYPE_CURRENCY:
				pVariable->setDouble(SdbTools::ToDouble((const char *)pLen));
				break;
			case SDB_DBTYPE_TINYINT:
			case SDB_DBTYPE_SMALLINT:
				pVariable->setINT16(*(INT16*)pLen);
				break;

			case SDB_DBTYPE_REAL:
				pVariable->setFloat(*(float*)pLen);
				break;

			case SDB_DBTYPE_DOUBLE:
				pVariable->setDouble(*(double*)pLen);
				break;

			case SDB_DBTYPE_BOOLEAN:
				pVariable->setBOOL( (*(INT16*)pLen) ? TRUE : FALSE );
				break;

			case SDB_DBTYPE_CHAR:
			case SDB_DBTYPE_VARCHAR:
			case SDB_DBTYPE_LONGVARCHAR:
				if(GetODBCConnection()->m_bUserWChar)
					pVariable->setString(rtl::OUString((const sal_Unicode *)pLen));
				else
					pVariable->setString(UniString((const char *)pLen,GetConnection()->GetCharacterSet()));
				break;
			case SDB_DBTYPE_BINARY:
			case SDB_DBTYPE_VARBINARY:
			case SDB_DBTYPE_LONGVARBINARY:
			// Sonst BINAER-Datentyp.
				pVariable->setString(UniString((const char *)pLen,GetConnection()->GetCharacterSet()));
				break;
			case SDB_DBTYPE_DATE:
				pVariable->setDate(ToDate(*(DATE_STRUCT*)pLen));
				break;
			case SDB_DBTYPE_TIME:
				pVariable->setTime(ToTime(*(TIME_STRUCT*)pLen));
				break;
			case SDB_DBTYPE_TIMESTAMP:
				pVariable->setDateTime(ToDateTime(*(TIMESTAMP_STRUCT*)pLen));
				break;
			default:
				DBG_ERROR("SdbODBC3Cursor: interner Fehler: unbekannter SDB-Datentyp"); break; // Unbekannter Datenbanktyp
		}
	}

	SetStatus(nRetcode,m_aStatementHandle);
	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------
void SdbODBC3Cursor::ImplChangeOpenOptions(SdbCursorType eType, UINT16 eOpt)
{
	BOOL bChange = FALSE;
	if(eType != SDB_CURRENT)
	{
		eCursorType = eType;
		bChange = TRUE;
	}
	if (eOpt != SDB_CURRENT_OPT)
	{
		nOptions = eOpt;
		if ((nOptions & SDB_READONLY) == SDB_READONLY)	// append und readonly widersprechen sich
			nOptions &= ~SDB_APPENDONLY;
		bChange = TRUE;
	}
	if (bChange)
	{
		BOOL bTempBookmarks(bBookmarks);
		INT32 nTempCursorType(nCursorType), nTempConcurrency(nConcurrency);

		SdbODBCConnection* pConnection = GetODBCConnection();
		DetermineOptions(pConnection);

		// bei PreparedStatements schliessen, da Optionen sonst nicht umgesetzt werden koennen
		// bitter aber wahr, was macht man nicht alles fuer eine gute Schnittstelle !!!
		BOOL bPrepareAgain;
		if (bPrepareAgain = (IsPrepared() && (bBookmarks != bTempBookmarks ||
											  nCursorType != nTempCursorType ||
											  nConcurrency != nTempConcurrency)))
		{
			// Handle neu setzen
			SetStatus(N3SQLFreeStmt(m_aStatementHandle,SQL_DROP),m_aStatementHandle);
			if (aStatus.IsSuccessful())
			{
				SQLRETURN nRetcode = (m_aConnectionHandle != SQL_NULL_HANDLE)
					? N3SQLAllocHandle(SQL_HANDLE_STMT,m_aConnectionHandle,&m_aStatementHandle)
					: N3SQLAllocHandle(SQL_HANDLE_STMT,GetODBCConnection()->ConnectionHandle(),&m_aStatementHandle);
				SetStatus(nRetcode,m_aStatementHandle);
			}
		}

		if (aStatus.IsSuccessful())
		{
			SetOptions(pConnection);
			if (bPrepareAgain)
			{
				if (pParamBuffer)
				{	// if we already were prepared
					// FS - 70334 - 01.12.99
					FreeBuffer(pParamBuffer, pParamLength, *aParameterColumns, FALSE);
					pParamBuffer = NULL;
				}

				PrepareImpl();
			}
		}
	}
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::OpenImpl(SdbCursorType eType ,UINT16 eOpt )
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ImplChangeOpenOptions(eType, eOpt);

	SQLRETURN nRetcode;

	if (IsPrepared())
	{
		if (IsOpen())
		{
			// Schon einmal Execute ausgefuehrt.
			// Vor dem naechsten Execute muss der Cursor erst geschlossen werden
			// (Option CLOSE).
			N3SQLFreeStmt(m_aStatementHandle,SQL_CLOSE);
		}
		UINT16 nCount(aParameterColumns->Count());
		for (UINT16 i=1 ; i < nCount;i++)
			BindColumn(i, pParamBuffer, pParamLength, *(*aParameterColumns)[i], (*aParameterRow)[i]);
	}
	else
	{
		::staruno::Reference< ::stardata::XDatabaseDescriptor > rMetaData(GetConnection()->GetMetaData());
		sal_Int16 nInfo;
		rMetaData->getInfo(::stardata::DatabaseInfo::OUTER_JOIN_SUPPORT) >>= nInfo;
		BOOL bInnerJoin = (nInfo & ::stardata::DatabaseOuterJoinSupport::YES || nInfo & ::stardata::DatabaseOuterJoinSupport::PARTIAL);
		Translate(bInnerJoin);
		if (! aStatus.IsSuccessful())
			return FALSE;
	}

	// Anpassen  der Menge der zu bertragenden Daten
	if (nCurrentMaxFieldLen != nMaxFieldLen)
	{
		nRetcode = N3SQLSetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_LENGTH, (SQLPOINTER)nMaxFieldLen,SQL_IS_INTEGER);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, m_aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState() == aValueChangedState)
				N3SQLGetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_LENGTH, &nMaxFieldLen,0,0);

			nCurrentMaxFieldLen = nMaxFieldLen;
		}
		else
			// Wieder zuruecksetzen
			nMaxFieldLen = nCurrentMaxFieldLen;
	}

	if (nCurrentMaxRows != nMaxRows)
	{
		nRetcode = N3SQLSetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_ROWS, (SQLPOINTER)nMaxRows,SQL_IS_INTEGER);
		if (nRetcode == SQL_SUCCESS)
		{
			SetStatus(nRetcode, m_aStatementHandle);
			// Treiber hat den Wert eigenstaendig veraendert
			if (aStatus.SQLState() == aValueChangedState)
				N3SQLGetStmtAttr(m_aStatementHandle, SQL_ATTR_MAX_ROWS, &nMaxRows,0,0);

			nCurrentMaxRows = nMaxRows;
		}
		else
			// Wieder zuruecksetzen
			nMaxRows = nCurrentMaxRows;
	}

	// clear all previous selected bookmarks
	if (m_nBookmarkLen != 4)
	{
		for (vector<BYTE*>::iterator i = m_aBookmarks.begin(); i != m_aBookmarks.end();++i)
			delete[] (*i);
	}
	m_aBookmarks.clear();

	if (IsPrepared())
	{
		BEGIN_CANCELABLE();
		{
			SdbCancelableCoreCall aCoreCallFlag(*this);
			nRetcode = N3SQLExecute(m_aStatementHandle);
		}
		END_CANCELABLE();
	}
	else
	{
		BEGIN_CANCELABLE();
		{
			SdbCancelableCoreCall aCoreCallFlag(*this);
			ByteString aSql(aSQLStatement,gsl_getSystemTextEncoding());
			nRetcode = N3SQLExecDirect(m_aStatementHandle, (SDB_ODBC_CHAR*)aSql.GetBuffer(), aSql.Len());
		}
		END_CANCELABLE();
	}

	nCurrentMaxFieldLen = nMaxFieldLen;
	nCurrentMaxRows = nCurrentMaxRows;

	// Parameter at Execution Time erforderlich?
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;

		nRetcode = N3SQLParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS &&
				nRetcode != SQL_SUCCESS_WITH_INFO &&
				nRetcode != SQL_NEED_DATA) break;

			if (aParameterColumns->Count() == 1)
			{
				// Keine Strukturbeschreibung fuer die Parameter Row oder keine Parameter Row.
				// Das darf nicht sein - Statement-Ausfuehrung abbrechen!
				if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLCANCEL))
					N3SQLCancel(m_aStatementHandle);
				aStatus.Set(SDB_STAT_ERROR,
							String::CreateFromAscii("S1093"),
							aStatus.CreateErrorMessage(String(SdbResId(STR_STAT_INVALID_PARAMETER_NUMBER))),
							0, String() );
				// Genaugenommen eigentlich: Parameter missing ...
				return FALSE;
			}

			// Daten fuer diesen Parameter an ODBC uebergeben ...
			// Die zugehoerige Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			PutData(*pCol, (*aParameterRow)[pCol->GetId()]); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
			{
				// Bei Fehler Statement-Ausfuehrung abbrechen!
				if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLCANCEL))
					N3SQLCancel(m_aStatementHandle);
				return FALSE;
			}

			nRetcode = N3SQLParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
	}

	// Resultat des SQLExec[Direct] oder des letzten SqlParamData
	// ist das Resultat des gesamten Statements ...
	SetStatus(nRetcode,m_aStatementHandle);
	if (!aStatus.IsSuccessful())
		return FALSE;
	if(aStatus.IsSuccessful() && aStatus.IsInformationAvailable() && aStatus.SQLState().EqualsAscii("01S02"))
	{
		N3SQLGetStmtAttr(m_aStatementHandle, SQL_ATTR_CURSOR_TYPE,&nCursorType,0,0);
		if(nCursorType == SQL_CURSOR_FORWARD_ONLY)
		{
			nOptions = SDB_FORWARDONLY;
			bBookmarks = FALSE;
		}
		else if(nCursorType == SQL_CURSOR_KEYSET_DRIVEN)
			eCursorType = SDB_KEYSET;
		else
			eCursorType = SDB_SNAPSHOT; // cursor type is static

		N3SQLGetStmtAttr(m_aStatementHandle, SQL_ATTR_CONCURRENCY,&nConcurrency,0,0);
		DetermineOptions(GetODBCConnection());
	}

	// solange er nicht wirklich gebraucht wird,
	// einfach erst einmal selbst setzen
	if (!aName.Len())
	{
		aName = 'C';
		aName += String::CreateFromInt32((long)this);
	}

	return TRUE;
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenTables(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
							   const sal_Unicode* pTable, const sal_Unicode* pType)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}
	bBookmarks = FALSE;

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pType		? (aCOL = ByteString(pType,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = N3SQLTables(m_aStatementHandle,
							(SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pCOL, pType ? SQL_NTS : 0);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	bBookmarks = FALSE;
	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenColumns(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, const sal_Unicode* pColumn)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN,aCOL;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pCOL = pColumn		? (aCOL = ByteString(pColumn,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = N3SQLColumns(m_aStatementHandle,
							(SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							(SDB_ODBC_CHAR *) pCOL, pColumn ? SQL_NTS : 0);
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	bBookmarks = FALSE;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenTypeInfo(INT16 nType)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	SQLRETURN nRetcode = N3SQLGetTypeInfo(m_aStatementHandle, nType);
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	bBookmarks = FALSE;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenPrimaryKeys(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLPRIMARYKEYS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = N3SQLPrimaryKeys(m_aStatementHandle,
								(SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;

	bBookmarks = FALSE;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenTablePrivileges(const sal_Unicode* pQualifier, const sal_Unicode* pOwner, const sal_Unicode* pTable)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLTABLEPRIVILEGES))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = N3SQLTablePrivileges(m_aStatementHandle,
								   (SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
								   (SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
								   (SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	bBookmarks = FALSE;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenIndices(const sal_Unicode* pQualifier, const sal_Unicode* pOwner,
								const sal_Unicode* pTable, BOOL bUnique)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLSTATISTICS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	ByteString aPKQ,aPKO,aPKN;

	const char	*pPKQ = pQualifier	? (aPKQ = ByteString(pQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pOwner		? (aPKO = ByteString(pOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pTable		? (aPKN = ByteString(pTable,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;
	

	SQLRETURN nRetcode = N3SQLStatistics(m_aStatementHandle,
							 (SDB_ODBC_CHAR *) pPKQ, pQualifier ? SQL_NTS : 0,
							 (SDB_ODBC_CHAR *) pPKO, pOwner ? SQL_NTS : 0,
							 (SDB_ODBC_CHAR *) pPKN, pTable ? SQL_NTS : 0,
							 bUnique ? SQL_INDEX_UNIQUE : SQL_INDEX_ALL,
							 SQL_QUICK);

	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	bBookmarks = FALSE;
	Open4Info();
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenForeignKeys(const sal_Unicode* pPKQualifier, const sal_Unicode* pPKOwner, const sal_Unicode* pPKName,
									const sal_Unicode* pFKQualifier, const sal_Unicode* pFKOwner, const sal_Unicode* pFKName)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLFOREIGNKEYS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	

	ByteString aPKQ,aPKO,aPKN, aFKQ, aFKO, aFKN;

	const char	*pPKQ = pPKQualifier	? (aPKQ = ByteString(pPKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL,
				*pPKO = pPKOwner		? (aPKO = ByteString(pPKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL,
				*pPKN = pPKName			? (aPKN = ByteString(pPKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL, 
				*pFKQ = pFKQualifier	? (aFKQ = ByteString(pFKQualifier,gsl_getSystemTextEncoding())).GetBuffer()	: NULL, 
				*pFKO = pFKOwner		? (aFKO = ByteString(pFKOwner,gsl_getSystemTextEncoding())).GetBuffer()		: NULL, 
				*pFKN = pFKName			? (aFKN = ByteString(pFKName,gsl_getSystemTextEncoding())).GetBuffer()		: NULL;

	SQLRETURN nRetcode = N3SQLForeignKeys(m_aStatementHandle,
								(SDB_ODBC_CHAR *) pPKQ, pPKQualifier ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKO, pPKOwner ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pPKN, pPKName ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pFKQ, pFKQualifier ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pFKO, pFKOwner ? SQL_NTS : 0,
								(SDB_ODBC_CHAR *) pFKN, pFKName ? SQL_NTS : 0
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	bBookmarks = FALSE;
	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenProcedures(const String& rQualifier, const String& rSchema, const String& rProcName)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLPROCEDURES))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding());

	SQLRETURN nRetcode = N3SQLProcedures(m_aStatementHandle,
								 (SDB_ODBC_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ODBC_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ODBC_CHAR *) aP.GetBuffer(), aP.Len()
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	bBookmarks = FALSE;
	Open4Info();
}


//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenProcedureColumns(const String& rQualifier, const String& rSchema,
										 const String& rProcName, const String& rProcColumnName)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLPROCEDURECOLUMNS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aP(rProcName,gsl_getSystemTextEncoding()),
		aC(rProcColumnName,gsl_getSystemTextEncoding());

	SQLRETURN nRetcode = N3SQLProcedureColumns(m_aStatementHandle,
								 (SDB_ODBC_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ODBC_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ODBC_CHAR *) aP.GetBuffer(), aP.Len(),
								 (SDB_ODBC_CHAR *) aC.GetBuffer(), aC.Len()
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	bBookmarks = FALSE;
	Open4Info();
}
//-----------------------------------------------------------------------------
void SdbODBC3Cursor::OpenSpecialColumns(const String& rQualifier, const String& rSchema, const String& rTabName)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	if (IsOpen() || IsPrepared())
	{
		aStatus.SetSequenceError();
		return;
	}

	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	if (!GetODBCConnection() || !GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLSPECIALCOLUMNS))
	{
		aStatus.SetDriverNotCapableError();
		return;
	}

	ByteString aQ(rQualifier,gsl_getSystemTextEncoding()),
		aS(rSchema,gsl_getSystemTextEncoding()),
		aT(rTabName,gsl_getSystemTextEncoding());

	SQLRETURN nRetcode = N3SQLSpecialColumns(m_aStatementHandle,SQL_ROWVER,
								 (SDB_ODBC_CHAR *) aQ.GetBuffer(), aQ.Len(),
								 (SDB_ODBC_CHAR *) aS.GetBuffer(), aS.Len(),
								 (SDB_ODBC_CHAR *) aT.GetBuffer(), aT.Len(),
								 SQL_SCOPE_TRANSACTION,SQL_NULLABLE
		   );
	SetStatus(nRetcode,m_aStatementHandle);
	if (! aStatus.IsSuccessful()) return;
	bBookmarks = FALSE;
	Open4Info();
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::CloseImpl()
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);

	// clear all previous selected bookmarks
	if (m_nBookmarkLen != 4)
	{
		for (vector<BYTE*>::iterator i = m_aBookmarks.begin(); i != m_aBookmarks.end();++i)
			delete[] (*i);
	}
	m_aBookmarks.clear();

	// Vater nicht mehr vorhanden, dann alles freigeben
	if (!GetParent())
	{
		if (m_aStatementHandle != SQL_NULL_HANDLE)
		{
			N3SQLFreeStmt(m_aStatementHandle,SQL_DROP);
			m_aStatementHandle = SQL_NULL_HANDLE;

			if (m_aConnectionHandle != SQL_NULL_HANDLE)	// extra Verbindung
			{
				N3SQLDisconnect(m_aConnectionHandle);
				N3SQLFreeHandle(SQL_HANDLE_DBC,m_aConnectionHandle);
			}
		}
	}
	else
	{
		DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
		// wenn einer der drei nicht klappt, dann den Handle wegschmeissen
		if (N3SQLFreeStmt(m_aStatementHandle,SQL_RESET_PARAMS) ||
			N3SQLFreeStmt(m_aStatementHandle,SQL_UNBIND) ||
			N3SQLFreeStmt(m_aStatementHandle,SQL_CLOSE))
		{
			SetStatus(N3SQLFreeStmt(m_aStatementHandle,SQL_DROP),m_aStatementHandle);
			if (aStatus.IsSuccessful())
			{
				SQLRETURN rRetcode = (m_aConnectionHandle != SQL_NULL_HANDLE) ?
					N3SQLAllocHandle(SQL_HANDLE_STMT,m_aConnectionHandle,&m_aStatementHandle)
									:
					N3SQLAllocHandle(SQL_HANDLE_STMT,GetODBCConnection()->ConnectionHandle(),&m_aStatementHandle);
				SetStatus(rRetcode,m_aStatementHandle);
			}
			if (!aStatus.IsSuccessful())
			{
				if (m_aConnectionHandle != SQL_NULL_HANDLE)	// extra Verbindung
				{
					N3SQLDisconnect(m_aConnectionHandle);
					N3SQLFreeHandle(SQL_HANDLE_DBC,m_aConnectionHandle);
				}
				m_aStatementHandle = SQL_NULL_HANDLE;
			}
		}
	}
	SdbODBCCursor::CloseImpl();
	return aStatus.IsSuccessful();
}
//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::MoveImpl(const ::staruno::Any& rBookmark, SdbCursorMode& rMode, BOOL bData)  throw( ::com::sun::star::lang::IllegalArgumentException )
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	if (rBookmark.getValueTypeClass() != ::com::sun::star::uno::TypeClass_LONG || !bBookmarkable)
		throw ::com::sun::star::lang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);

	sal_Int32 nBookmarkPos;
	rBookmark >>= nBookmarkPos;
	if (nBookmarkPos > m_aBookmarks.size())
		throw ::com::sun::star::lang::IllegalArgumentException(UniString(SdbResId(STR_STAT_INVALID_BOOKMARK_VALUE)), ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >(), 1);	
	
	if (m_nBookmarkLen == 4)
	{
		INT32 nBookmark = (INT32)m_aBookmarks[nBookmarkPos - 1];
		N3SQLSetStmtAttr(m_aStatementHandle,SQL_ATTR_FETCH_BOOKMARK_PTR,&nBookmark,SQL_LEN_BINARY_ATTR(m_nBookmarkLen));
	}
	else
		N3SQLSetStmtAttr(m_aStatementHandle,SQL_ATTR_FETCH_BOOKMARK_PTR,m_aBookmarks[nBookmarkPos - 1],SQL_LEN_BINARY_ATTR(m_nBookmarkLen));


	SQLRETURN nRetcode = N3SQLFetchScroll(m_aStatementHandle,SQL_FETCH_BOOKMARK,0);
	SetStatus(nRetcode,m_aStatementHandle);

	if (!aStatus.IsSuccessful())
		return FALSE;

	// set the bookmark
	(*aResultRow)[0]->setAny(rBookmark);

	if (nRowStatus == SQL_ROW_DELETED)
		rMode = SdbCursor::SDB_MOD_INVALID;
	else if (bData)
		FixResultRow();

	// Data not transfered correct
	if (!aStatus.IsSuccessful())
	{
		rMode = SdbCursor::SDB_MOD_INVALID;
		aStatus.Clear();
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::MoveImpl(SdbCursorPosition eCursorPosition, INT32 nOffset,
							 SdbCursorMode& rMode)
{
	DBG_CHKTHIS(SdbODBC3Cursor,NULL);
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");

	SQLRETURN nRetcode;
	BOOL bFetchBookmark = TRUE;
	if (GetODBCConnection()->nFetchDirection == SQL_FD_FETCH_NEXT ||
		(nOptions & SDB_FORWARDONLY) == SDB_FORWARDONLY)
	{
		DBG_ASSERT(eCursorPosition == SDB_POS_NEXT, "ODBCCursor: Falsche MoveRichtung, nur Next zulaessig!");
		if (eCursorPosition != SDB_POS_NEXT)
		{
			aStatus.SetDriverNotCapableError();
		}
		else
		{
			nRetcode = N3SQLFetch(m_aStatementHandle);
			SetStatus(nRetcode,m_aStatementHandle);
		}
	}
	else
	{
		// Voraussetzung: rowset size == 1 (Default-Einstellung, siehe SQLSetStmtOption)
		UWORD fFetchType;           // type of fetch
		SDWORD irow = nOffset;		// number of the row to fetch
		switch(eCursorPosition)
		{
			case SDB_POS_NEXT:			fFetchType = SQL_FETCH_NEXT; break;
			default: 					DBG_ERROR("SdbODBC3Cursor: unzulaessiger Offset"); break;
		}

		nRetcode = N3SQLFetchScroll(m_aStatementHandle,fFetchType,irow);

		// Status: "off range", d. h. kein weiterer Datensatz (egal bei
		// welcher Positionierung):
		SetStatus(nRetcode,m_aStatementHandle);

	}
	if (!aStatus.IsSuccessful())
		return FALSE;
	else if (nRowStatus == SQL_ROW_DELETED)
	{
		// Row is deleted
		aResultRow->reset();

		if (bBookmarks)
		{
			AddBookmark();
			(*aResultRow)[0]->setINT32(m_aBookmarks.size());
		}
		rMode = SdbCursor::SDB_MOD_INVALID;
	}
	else
	{
		FixResultRow();
		if (bBookmarks)
		{
			AddBookmark();
			(*aResultRow)[0]->setINT32(m_aBookmarks.size());
		}
	}

	if (!aStatus.IsSuccessful())
	{
		rMode = SdbCursor::SDB_MOD_INVALID;
		aStatus.Clear();
	}
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
int SdbODBC3Cursor::GetDataImpl(UWORD icol , SWORD fCType , PTR rgbValue , SDWORD cbValueMax , SDWORD FAR * pcbValue)
{
	return N3SQLGetData((SQLHSTMT)m_aStatementHandle,(SQLUSMALLINT)icol,(SQLSMALLINT)fCType,(SQLPOINTER)rgbValue,(SQLINTEGER)cbValueMax,(SQLINTEGER *)pcbValue);
}

//-----------------------------------------------------------------------------
int SdbODBC3Cursor::PutDataImpl(PTR rgbValue , SDWORD cbValue)
{
	return N3SQLPutData(m_aStatementHandle,rgbValue,cbValue);
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::InsertImpl()
{
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	if (!PrepareForUpdate())
		return FALSE;

	SQLRETURN nRetcode = N3SQLBulkOperations(m_aStatementHandle,SQL_ADD);

	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;
		nRetcode = N3SQLParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS &&
				nRetcode != SQL_SUCCESS_WITH_INFO &&
				nRetcode != SQL_NEED_DATA) break;

			// Daten fuer diesen Parameter an ODBC uebergeben ...
			// Die zugehoerige Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			PutData(*pCol, (*aResultRow)[pCol->GetId()]); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
				break;

			nRetcode = N3SQLParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);

		if (nRetcode == SQL_ERROR || nRetcode == SQL_NO_DATA_FOUND)
			SetStatus(nRetcode,m_aStatementHandle);
	}
	else
		SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO && nRowStatus != SQL_ROW_ADDED)
		aStatus.Set(SDB_STAT_ERROR, aStatus.SQLState(),aStatus.ErrorMessage(),
					aStatus.NativeErrorCode(),aStatus.ErrorInfo());

	if (aStatus.IsSuccessful() && !IsInsertOnly_OldAPI())
	{
		// retrieve the bookmark
		m_nBookmarkLen  = pFieldLength[0];
		if (m_nBookmarkLen == 4)
			m_aBookmarks.push_back((BYTE*)*(INT32*)pFieldBuffer[0]);
		else
		{
			BYTE* pData = new BYTE[m_nBookmarkLen];
			memcpy(pData, pFieldBuffer[0], m_nBookmarkLen);
			m_aBookmarks.push_back(pData);
		}
		(*aResultRow)[0]->setINT32(m_aBookmarks.size());
	}

	N3SQLFreeStmt(m_aStatementHandle,SQL_UNBIND);
	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::DeleteImpl()
{
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	return ExecutePosUpdate(SQL_DELETE);
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::UpdateImpl()
{
	DBG_ASSERT(m_aStatementHandle != SQL_NULL_HANDLE,"interner Fehler: StatementHandle ungueltig");
	return PrepareForUpdate() && ExecutePosUpdate(SQL_UPDATE);
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::ExecutePosUpdate(UINT16 nOption)
{
	UINT16 nExpectedStatus;
	switch (nOption)
	{
		case SQL_UPDATE:
			nExpectedStatus = SQL_ROW_UPDATED;
			break;
		case SQL_ADD:
			nExpectedStatus = SQL_ROW_ADDED;
			break;
		default:
			nExpectedStatus = SQL_ROW_DELETED;
	}

	SQLRETURN nRetcode = N3SQLSetPos(m_aStatementHandle, 1, nOption, SQL_LOCK_NO_CHANGE);
	if (nRetcode == SQL_NEED_DATA)
	{
		void * prgbValue;
		nRetcode = N3SQLParamData(m_aStatementHandle,&prgbValue);
		DBG_ASSERT(aParameterRow.Is(), "Falsches Parameter Handling!");
		do
		{
			if (nRetcode != SQL_SUCCESS &&
				nRetcode != SQL_SUCCESS_WITH_INFO &&
				nRetcode != SQL_NEED_DATA) break;

			// Daten fuer diesen Parameter an ODBC uebergeben ...
			// Die zugehoerige Parameter Column wurde zuvor beim BindParameter
			// in prgbValue "eingetuetet":
			SdbColumn* pCol = (SdbColumn*)prgbValue;
			PutData(*pCol, (*aResultRow)[pCol->GetId()]); // XXX kann noch zu fehlern fuehren
			if (aStatus.IsError())
				break;

			nRetcode = N3SQLParamData(m_aStatementHandle,&prgbValue);
		} while (nRetcode == SQL_NEED_DATA);
		if (nRetcode == SQL_ERROR || nRetcode == SQL_NO_DATA_FOUND)
			SetStatus(nRetcode,m_aStatementHandle);
	}
	else
		SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.Result() == SDB_STAT_SUCCESS_WITH_INFO && nRowStatus != nExpectedStatus)
		aStatus.Set(SDB_STAT_ERROR, aStatus.SQLState(),aStatus.ErrorMessage(),
					aStatus.NativeErrorCode(),aStatus.ErrorInfo());

	if (nOption == SQL_UPDATE)
		N3SQLFreeStmt(m_aStatementHandle,SQL_UNBIND);

	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::PrepareForUpdate()
{
	UINT16 nCount(aResultColumns->Count());
	if (nCount > 1 && !pFieldBuffer)
		AllocBuffer(pFieldBuffer, pFieldLength, *aResultColumns, TRUE);

	for (UINT16 i=1 ; i < nCount;i++)
	{
		const ODbVariantRef& pVal = (*aResultRow)[i];
		if (pVal.is() && pVal->isModified())
			BindColumn(i, pFieldBuffer, pFieldLength, *(*aResultColumns)[i], pVal, FALSE);
	}

	// reread the bookmark column value to get the correct
	if (IsModeNew() && !IsInsertOnly_OldAPI())
		BindColumn(0, pFieldBuffer, pFieldLength, *(*aResultColumns)[0], (*aResultRow)[0], FALSE);

	return aStatus.IsSuccessful();
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::RowCountImpl()
{
	if (GetODBCConnection()->IsODBCFunctionAvailable(SQL_API_SQLROWCOUNT))
	{
		SDWORD pcrow=-1;
		N3SQLRowCount(m_aStatementHandle, &pcrow);
		aStatus.Clear();

		if (pcrow >= 0)
			SetRowCount((UINT32)pcrow);
	}
}

//-----------------------------------------------------------------------------
void SdbODBC3Cursor::AddBookmark()
{
	DBG_ASSERT(m_nBookmarkLen < (sizeof aBuffer.aCharArray - 1), "Invalid Buffer length");
	SDWORD pcbValue=0;
	SQLRETURN rRetcode=0;
	rRetcode = N3SQLGetData(m_aStatementHandle,(SQLUSMALLINT)0,SQL_C_VARBOOKMARK,(SQLPOINTER)aBuffer.aCharArray,m_nBookmarkLen,(SQLINTEGER *)&pcbValue);
	SetStatus(rRetcode,m_aStatementHandle);
	if (aStatus.IsSuccessful())
	{
		DBG_ASSERT(pcbValue != SQL_NO_TOTAL, "Invalid Bookmark data");
		// store 4 BYTE data as integer
		if (m_nBookmarkLen == 4)
			m_aBookmarks.push_back((BYTE*)*(INT32*)aBuffer.aCharArray);
		else
		{
			BYTE* pData = new BYTE[m_nBookmarkLen];
			memcpy(pData, aBuffer.aCharArray, m_nBookmarkLen);
			m_aBookmarks.push_back(pData);
		}
	}
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::GetDateValues(UINT16 nPos,DATE_STRUCT &rValue)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)
	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
	// Column #0 ist die Bookmark!

	if(((SdbODBC3Connection*)GetODBCConnection())->m_bUseOldTimeDate)
		nRetcode = GetDataImpl(nPos,SQL_C_DATE,&rValue,sizeof(rValue),&pcbValue);
	else
		nRetcode = GetDataImpl(nPos,SQL_C_TYPE_DATE,&rValue,sizeof(rValue),&pcbValue);
	SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.IsError())
	{
		DBG_ERROR((((((ByteString("SQLGetData fehlgeschlagen!: Column ") += ByteString::CreateFromInt32(nPos)) += 
			ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding())))).GetBuffer()) ;
	}
	else if (pcbValue == SQL_NULL_DATA)
		return FALSE;
	return TRUE;
}
//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::GetTimeValues(UINT16 nPos,TIME_STRUCT &rValue)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)
	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
	// Column #0 ist die Bookmark!

	if(((SdbODBC3Connection*)GetODBCConnection())->m_bUseOldTimeDate)
		nRetcode = GetDataImpl(nPos,SQL_C_TIME,&rValue,sizeof(rValue),&pcbValue);
	else
		nRetcode = GetDataImpl(nPos,SQL_C_TYPE_TIME,&rValue,sizeof(rValue),&pcbValue);
	SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.IsError())
	{
		DBG_ERROR((((ByteString("SQLGetData fehlgeschlagen!: Column ") += ByteString::CreateFromInt32(nPos)) += 
			ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding())).GetBuffer());
	}
	else if (pcbValue == SQL_NULL_DATA)
		return FALSE;
	return TRUE;
}

//-----------------------------------------------------------------------------
BOOL SdbODBC3Cursor::GetTimeDateValues(UINT16 nPos,TIMESTAMP_STRUCT &rValue)
{
	RETCODE nRetcode;
	SDWORD pcbValue;				// Tatsaechliche Laenge der Daten
									// (oder SQL_NULL_DATA oder SQL_NO_TOTAL)
	// Zaehlung der ODBC-Columns und Zaehlung unserer Columns beginnt bei eins!
	// Column #0 ist die Bookmark!

	if(((SdbODBC3Connection*)GetODBCConnection())->m_bUseOldTimeDate)
		nRetcode = GetDataImpl(nPos,SQL_C_TIMESTAMP,&rValue,sizeof(rValue),&pcbValue);
	else
		nRetcode = GetDataImpl(nPos,SQL_C_TYPE_TIMESTAMP,&rValue,sizeof(rValue),&pcbValue);
	SetStatus(nRetcode,m_aStatementHandle);

	if (aStatus.IsError())
	{
		DBG_ERROR(((((ByteString("SQLGetData fehlgeschlagen!: Column ") += ByteString::CreateFromInt32(nPos)) 
			+= ByteString(", ")) += ByteString(aStatus.ErrorMessage(),gsl_getSystemTextEncoding()))).GetBuffer()) ;
	}
	else if (pcbValue == SQL_NULL_DATA)
		return FALSE;
	return TRUE;
}

