/*************************************************************************
 *
 *  $RCSfile: sdbsqlit.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbsqlit.hxx"

#ifndef _SVT_SDBPARSE_HXX
#include <svtools/sdbparse.hxx>
#endif

#ifdef SDB_TEST_PARSETREEITERATOR
#include <iostream.h>
#endif
#ifndef _SDB_SDBDB_HXX //autogen
#include <sdbdb.hxx>
#endif
#ifndef _SDB_SDBTAB_HXX //autogen
#include <sdbtab.hxx>
#endif

#include "sdbresid.hrc"

DBG_NAME(SdbSqlParseTreeIterator);
static String aEmptyString;

//-----------------------------------------------------------------------------
SdbSqlParseTreeIterator::SdbSqlParseTreeIterator()
	:pConnection(NULL)
	,pParseTree(NULL)
{
	DBG_CTOR(SdbSqlParseTreeIterator,NULL);
}
//-----------------------------------------------------------------------------
SdbSqlParseTreeIterator::SdbSqlParseTreeIterator(SdbDatabase* rDB,const SdbSqlParseNode* pRoot)
	:pConnection(rDB->GetConnection())
{
	DBG_CTOR(SdbSqlParseTreeIterator,NULL);
	SetParseTree(pRoot);
}
//-----------------------------------------------------------------------------
SdbSqlParseTreeIterator::SdbSqlParseTreeIterator(SdbConnection* rConn,const SdbSqlParseNode* pRoot)
	:pConnection(rConn)
{
	DBG_CTOR(SdbSqlParseTreeIterator,NULL);
	SetParseTree(pRoot);
}
//-----------------------------------------------------------------------------
SdbSqlParseTreeIterator::SdbSqlParseTreeIterator(const SdbSqlParseTreeIterator & rIter)
	:pConnection(NULL)
	,pParseTree(NULL)
{
	DBG_CTOR(SdbSqlParseTreeIterator,NULL);

	DBG_ERROR("SdbSqlParseTreeIterator: Copy-Konstruktor nicht implementiert!");
}

//-----------------------------------------------------------------------------
SdbSqlParseTreeIterator::~SdbSqlParseTreeIterator()
{
	DBG_DTOR(SdbSqlParseTreeIterator,NULL);
	for(ULONG i=0; i< aTableRangeMap.Count();i++)
		delete aTableRangeMap.GetObject(i);
	aTableRangeMap.Clear();
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetDatabase(SdbDatabase* pDb)
	{ pConnection = pDb->GetConnection(); aTableRangeMap.SetCaseSensitive(TablesAreSensitive()); };

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetParseTree(const SdbSqlParseNode * pNewParseTree)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// alle Eingaben zur"uck setzen
	aIteratorStatus.Clear();
	for(ULONG i=0; i< aTableRangeMap.Count();i++)
		delete aTableRangeMap.GetObject(i);
	aTableRangeMap.Clear();

	if(aSelectColumns.Is())
		aSelectColumns->Reset();

	pParseTree = pNewParseTree;
	if (pParseTree == NULL)
	{
		eStatementType = SDB_STATEMENT_UNKNOWN;
		return;
	}

	// falls pParseTree aber keine Connection, dann Fehler
	if(!pConnection)
	{
		//aIteratorStatus.SetInvalidStatement();
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		return;
	}

	aTableRangeMap.SetCaseSensitive(TablesAreSensitive());

	if(!aSelectColumns.Is())
		aSelectColumns = pConnection->CreateColumns();

	// Statement-Typ ermitteln ...
	if (SDB_ISRULE(pParseTree,select_statement) || SDB_ISRULE(pParseTree,union_statement) )
	{
		eStatementType = SDB_STATEMENT_SELECT;
	}
	else if (SDB_ISRULE(pParseTree,insert_statement))
	{
		eStatementType = SDB_STATEMENT_INSERT;
	}
	else if (SDB_ISRULE(pParseTree,update_statement_positioned))
	{
		eStatementType = SDB_STATEMENT_UPDATE;
	}
	else if (SDB_ISRULE(pParseTree,update_statement_searched))
	{
		eStatementType = SDB_STATEMENT_UPDATE;
	}
	else if (SDB_ISRULE(pParseTree,delete_statement_positioned))
	{
		eStatementType = SDB_STATEMENT_DELETE;
	}
	else if (SDB_ISRULE(pParseTree,delete_statement_searched))
	{
		eStatementType = SDB_STATEMENT_DELETE;
	}
	else if (pParseTree->Count() == 3 && SDB_ISRULE(pParseTree->GetChild(1),odbc_call_spec))
	{
		eStatementType = SDB_STATEMENT_ODBC_CALL;
	}
	else
	{
		eStatementType = SDB_STATEMENT_UNKNOWN;
		//aIteratorStatus.SetInvalidStatement();
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		return;
	}
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseOneTableName(const SdbSqlParseNode * pTableName, const String & rTableRange)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	DBG_ASSERT(pTableName != NULL,"SdbSqlParseTreeIterator::TraverseOneTableName: pTableName == NULL");

	String aTableName;
	String aTableRange(rTableRange);

	// Tabellenname abholen
	pTableName->ParseNodeToStr(aTableName,pConnection->GetMetaData(),FALSE,FALSE);
	// Wenn keine Range Variable angegeben, dann den Tabellennamen verwenden.
	if (!aTableRange.Len())
		aTableRange = aTableName;

	if(aTableName.Len())
	{
		SdbTableRef *pTable = new SdbTableRef(pConnection->OpenTable(aTableName));
		if(pTable->Is())
		{
			aTableRangeMap.InsertAt(aTableRange, pTable);
		}
		else
		{
			SdbStatus aSDB;	aSDB.SetInvalidStatement();
			String strError = String(SdbResId(STR_STAT_TABLE_DOESNT_EXIST));
			strError.SearchAndReplace(String::CreateFromAscii("$name$"), aTableName);
			aSDB.Set(SDB_STAT_ERROR, String::CreateFromAscii("S1000"), aSDB.CreateErrorMessage(strError), 0, String() );

			if (CallError(1,aSDB,aTableName))
				aTableRangeMap.InsertAt(aTableRange, pTable);	// ich soll den Fehler ignorieren
			else
				delete pTable;
		}
	}
	else
	{
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB,rTableRange);
	}
}
//-----------------------------------------------------------------------------
SdbSqlParseNode * SdbSqlParseTreeIterator::Qualified_join(SdbSqlParseNode *pTableRef,String& aTableRange)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	aTableRange.Erase();

	SdbSqlParseNode *pNode = TableRef(pTableRef->GetChild(0),aTableRange);
	if(pNode)
		TraverseOneTableName(pNode,aTableRange);
	ULONG nPos = 4;
	if(SDB_ISRULE(pTableRef,cross_union) || pTableRef->GetChild(1)->GetTokenID() != SDB_TOKEN_NATURAL)
		nPos = 3;


	pNode = TableRef(pTableRef->GetChild(nPos),aTableRange);
	if(pNode)
		TraverseOneTableName(pNode,aTableRange);
	return pNode;
}
//-----------------------------------------------------------------------------
SdbSqlParseNode * SdbSqlParseTreeIterator::TableRef(SdbSqlParseNode *pTableRef,String& aTableRange)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	SdbSqlParseNode *pTableName = pTableRef;
	if (pTableName->Count() == 4 )
	{
		if(SDB_ISPUNCTUATION(pTableName->GetChild(0),"{"))
		{ // { OJ joined_table }
			Qualified_join(pTableName->GetChild(2),aTableRange);
			pTableName = NULL;
		}
		else
		{
			// Tabellennamen gefunden
			if(!SDB_ISRULE(pTableName,table_name))
				pTableName = pTableRef->GetChild(0);
			aTableRange.Erase();
			if(pTableRef->Count() == 4)
				aTableRange = pTableRef->GetChild(2)->TokenValue(); // Tabellenrange an Pos 2
		}
	}
	else if(SDB_ISRULE(pTableName,qualified_join) || SDB_ISRULE(pTableName,cross_union))
	{
		Qualified_join(pTableRef,aTableRange);
		pTableName = NULL;
	}
	else if (SDB_ISRULE(pTableName,joined_table))
	{
		// '(' qualified_join ')'
		Qualified_join(pTableName->GetChild(1),aTableRange);
		pTableName = NULL;
	}
	else if(pTableRef->Count() == 6)
	{
		// '(' joined_table ')' as range_variable op_column_commalist
		if(SDB_ISRULE(pTableRef->GetChild(1),qualified_join) || SDB_ISRULE(pTableRef->GetChild(1),cross_union))
			Qualified_join(pTableRef->GetChild(1),aTableRange);
		else if(SDB_ISRULE(pTableRef->GetChild(1),select_statement)) // Unterabfrage
			Select_statement(pTableRef->GetChild(1));
		else if(pTableRef->GetChild(1)->Count() == 4)
			// pTableRef->GetChild(1) ->> non_join_query_exp
			Select_statement(pTableRef->GetChild(0)); // query_exp SDB_TOKEN_UNION all query_term
		else
		{// nyi: tiefere Verschachtelung m"oglch
			SdbStatus aSDB;	aSDB.SetStatementTooComplex();
			CallError(4,aSDB);
		}
	}
	return pTableName;
}
//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::Select_statement(SdbSqlParseNode *pSelect)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	if(SDB_ISRULE(pSelect,union_statement))
	{
		Select_statement(pSelect->GetChild(0));
		//Select_statement(pSelect->GetChild(3));
		return;
	}
	SdbSqlParseNode * pTableRefCommalist = pSelect->GetChild(3)->GetChild(0)->GetChild(1);

	DBG_ASSERT(pTableRefCommalist != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pTableRefCommalist,table_ref_commalist),"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	SdbSqlParseNode * pTableName = NULL;
	String aTableRange;
	for (ULONG i = 0; i < pTableRefCommalist->Count(); i++)
	{ // from clause durchlaufen
		aTableRange.Erase();

		if (SDB_ISRULE(pTableRefCommalist->GetChild(i),table_name))
		{
			pTableName = pTableRefCommalist->GetChild(i);
			TraverseOneTableName(pTableName,aTableRange);// Keine Range Variable
		}
		else if (SDB_ISRULE(pTableRefCommalist->GetChild(i),table_ref))
		{
			// Tabellenreferenz kann aus Tabellennamen, Tabellennamen (+),'('joined_table')'(+) bestehen
			pTableName = pTableRefCommalist->GetChild(i)->GetChild(0);
			if (SDB_ISRULE(pTableName,table_name))
			{	// Tabellennamen gefunden
				if(pTableRefCommalist->GetChild(i)->Count() == 4)	// Tabellenrange an Pos 2
					aTableRange = pTableRefCommalist->GetChild(i)->GetChild(2)->TokenValue();
				TraverseOneTableName(pTableName,aTableRange);
			}
			else if(SDB_ISPUNCTUATION(pTableName,"{"))
				Qualified_join(pTableRefCommalist->GetChild(i)->GetChild(2),aTableRange);
			else // '(' joined_table ')' as range_variable op_column_commalist
				TableRef(pTableRefCommalist->GetChild(i),aTableRange);
		}
		else if (SDB_ISRULE(pTableRefCommalist->GetChild(i),qualified_join) || SDB_ISRULE(pTableRefCommalist->GetChild(i),cross_union)		)
		{
			// qualified_join oder cross_union vorhanden
			Qualified_join(pTableRefCommalist->GetChild(i),aTableRange);
		}
		else if (SDB_ISRULE(pTableRefCommalist->GetChild(i),joined_table))
		{
			// '(' qualified_join ')'
			Qualified_join(pTableRefCommalist->GetChild(i)->GetChild(1),aTableRange);
		}
		else
		{
			SdbStatus aSDB;	aSDB.SetInvalidStatement();
			CallError(3,aSDB,aTableRange);
		}

		if (! aIteratorStatus.IsSuccessful()) break;
	}
}
//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseTableNames()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	aIteratorStatus.Clear();

	if (pParseTree == NULL) {
		//aIteratorStatus.SetInvalidStatement();
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		return;
	}

	SdbSqlParseNode * pTableName = NULL;
	String aTableRange;

	// pTableName auf den (einzigen) Tabellennamen im Statement setzen.
	// Je nach Statement (SELECT, INSERT, UPDATE, DELETE) an anderer Position.
	// Bei Select koennen mehrere Tabellennamen auftreten, bei den uebrigen Statements
	// ist ohnehin immer nur einer moeglich.

	if (eStatementType == SDB_STATEMENT_SELECT)
	{
		SdbSqlParseNode *pTmp = (SdbSqlParseNode *)pParseTree;
		Select_statement(pTmp);
	}
	else if (eStatementType == SDB_STATEMENT_INSERT)
	{
		pTableName = pParseTree->GetChild(2);
		TraverseOneTableName(pTableName,aTableRange);
	}
	else if (eStatementType == SDB_STATEMENT_UPDATE)
	{
		pTableName = pParseTree->GetChild(1);
		TraverseOneTableName(pTableName,aTableRange);
	}
	else if (eStatementType == SDB_STATEMENT_DELETE)
	{
		pTableName = pParseTree->GetChild(2);
		TraverseOneTableName(pTableName,aTableRange);
	} else
	{
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(5,aSDB,aTableRange);
	}
}
//-----------------------------------------------------------------------------
String SdbSqlParseTreeIterator::GetColumnAlias(const SdbSqlParseNode* pDerivedColumn) const
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	String aColumnAlias;
	if(pDerivedColumn->GetChild(1)->Count() == 2)
		aColumnAlias = pDerivedColumn->GetChild(1)->GetChild(1)->TokenValue();
	else if(!pDerivedColumn->GetChild(1)->IsRule())
		aColumnAlias = pDerivedColumn->GetChild(1)->TokenValue();
	return aColumnAlias;
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::GetColumnRange(const SdbSqlParseNode* pColumnRef,String &rColumnName,String &rTableRange) const
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	rColumnName.Erase();
	rTableRange.Erase();
	if(SDB_ISRULE(pColumnRef,column_ref))// ab hier ist es sicher eine Columnref
	{
		if(pColumnRef->Count() > 1)
		{
			for(INT32 i=0;i<((INT32)pColumnRef->Count())-2;i++)	 // mu"s signed sein, falls count == 1
				pColumnRef->GetChild(i)->ParseNodeToStr(rTableRange,pConnection->GetMetaData(),FALSE,FALSE);
			// Spaltenname
			rColumnName = pColumnRef->GetChild(pColumnRef->Count()-1)->GetChild(0)->TokenValue();
		}
		else
			rColumnName = pColumnRef->GetChild(0)->TokenValue();
	}
	else if(SDB_ISRULE(pColumnRef,general_set_fct) || SDB_ISRULE(pColumnRef,set_fct_spec))
	{ // Funktion
		pColumnRef->ParseNodeToStr(rColumnName,pConnection->GetMetaData());
	}
	else  if(pColumnRef->GetNodeType() == SDB_SQL_NAME)
		rColumnName = pColumnRef->TokenValue();
}

//-----------------------------------------------------------------------------
BOOL SdbSqlParseTreeIterator::GetColumnTableRange(const SdbSqlParseNode* pNode, String &rTableRange) const
{
	// Ermitteln ob alle Spalten zu einer Tabelle gehoeren
	if (SDB_ISRULE(pNode,column_ref))
	{
		String aColName, aTableRange;
		GetColumnRange(pNode, aColName, aTableRange);
		if (!aTableRange.Len())	// keinen gefunden
		{
			// dann die Spalte in den Tabellen suchen
			for (UINT32 i = 0, nCount = aTableRangeMap.Count(); i < nCount; i++)
			{
				SdbTableRef *pTableRef = aTableRangeMap.GetObject(i);
				if (pTableRef->Is())
				{
					const SdbColumnsRef rColumns = (*pTableRef)->OriginalColumns();
					const SdbColumn* pColumn = rColumns->Column(aColName);
					if (pColumn)
					{
						aTableRange = aTableRangeMap.GetKey(pTableRef);
						break;
					}
				}
			}
			if (!aTableRange.Len())
				return FALSE;
		}


		if (!rTableRange.Len())
			rTableRange = aTableRange;
		else if (rTableRange != aTableRange)
			return FALSE;
	}
	else
	{
		for (UINT32 i = 0, nCount = pNode->Count(); i < nCount; i++)
		{
			if (!GetColumnTableRange(pNode->GetChild(i), rTableRange))
				return FALSE;
		}
	}
	return TRUE;
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseSelectColumnNames(const SdbSqlParseNode* pSelectNode)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	aIteratorStatus.Clear();

	if (!pSelectNode || eStatementType != SDB_STATEMENT_SELECT || !aTableRangeMap.Count())
	{
		//aIteratorStatus.SetInvalidStatement();
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		return;
	}

	if(SDB_ISRULE(pSelectNode,union_statement))
	{
		TraverseSelectColumnNames(pSelectNode->GetChild(0));
//		TraverseSelectColumnNames(pSelectNode->GetChild(3));
		return;
	}

	// nyi: mehr Pruefung auf korrekte Struktur!
	if (pSelectNode->GetChild(2)->IsRule() && SDB_ISPUNCTUATION(pSelectNode->GetChild(2)->GetChild(0),"*"))
	{
		// SELECT * ...
		SetSelectColumnName(String::CreateFromAscii("*"), aEmptyString,aEmptyString);
	}
	else if (SDB_ISRULE(pSelectNode->GetChild(2),scalar_exp_commalist))
	{
		// SELECT column(,column) oder SELECT COUNT(*) ...
		SdbSqlParseNode * pSelection = pSelectNode->GetChild(2);

		for (ULONG i = 0; i < pSelection->Count(); i++)
		{
			SdbSqlParseNode *pColumnRef = pSelection->GetChild(i);

			if (SDB_ISRULE(pColumnRef,select_sublist))
			{
				DBG_ASSERT(pColumnRef->GetChild(2)->TokenValue().GetChar(0) == '*',"SdbSqlParseTreeIterator::TraverseSelectColumnNames aColumnName sollte * sein")	// select TableRange.*
				// alle Spalten der Tabelle
				String aTableRange;
				pColumnRef->GetChild(0)->ParseNodeToStr(aTableRange,pConnection->GetMetaData(),FALSE,FALSE);
				SetSelectColumnName(String::CreateFromAscii("*"), aEmptyString,aTableRange);
				continue;
			}else if (SDB_ISRULE(pColumnRef,derived_column))
			{
				String aColumnAlias(GetColumnAlias(pColumnRef)); // kann leer sein
				String aColumnName;
				String aTableRange;
				BOOL bFkt(FALSE);
				pColumnRef = pColumnRef->GetChild(0);
				if (SDB_ISRULE(pColumnRef,column_ref))
				{
					GetColumnRange(pColumnRef,aColumnName,aTableRange);
					DBG_ASSERT(aColumnName.Len(),"Columnname darf nicht leer sein");
				}
				else /*if (SDB_ISRULE(pColumnRef,general_set_fct) || SDB_ISRULE(pColumnRef,set_fct_spec)	||
						 SDB_ISRULE(pColumnRef,position_exp)	|| SDB_ISRULE(pColumnRef,extract_exp)	||
						 SDB_ISRULE(pColumnRef,length_exp)		|| SDB_ISRULE(pColumnRef,char_value_fct)||
						 SDB_ISRULE(pColumnRef,num_value_exp)	|| SDB_ISRULE(pColumnRef,term))*/
				{
					/* Funktionsaufruf vorhanden */
					pColumnRef->ParseNodeToStr(aColumnName,pConnection->GetMetaData(),FALSE,TRUE);

					// gehoeren alle beteiligten Spalten der Funktion zu einer Tabelle
					if (aTableRangeMap.Count() == 1)
					{
						SdbTableRef *pTableRef = aTableRangeMap.GetObject(0);
						aTableRange = aTableRangeMap.GetKey(pTableRef);
					}
					else
					{
						GetColumnTableRange(pColumnRef,aTableRange);
					}
					bFkt = TRUE;
				}
				/*
				else
				{
					aIteratorStatus.SetStatementTooComplex();
					return;
				}
				*/
				if(!aColumnAlias.Len())
					aColumnAlias = aColumnName;
				SetSelectColumnName(aColumnName,aColumnAlias,aTableRange,bFkt);
			}
		}

	} else
	{
		//aIteratorStatus.SetInvalidStatement();
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
	}
}


//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseOrderByColumnNames(const SdbSqlParseNode* pSelectNode)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	aIteratorStatus.Clear();

	if (pSelectNode == NULL) 
	{
		//aIteratorStatus.SetInvalidStatement();
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		return;
	}

	if (eStatementType != SDB_STATEMENT_SELECT)
	{
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		//aIteratorStatus.SetInvalidStatement();
		return;
	}

	if(SDB_ISRULE(pSelectNode,union_statement))
	{
		TraverseOrderByColumnNames(pSelectNode->GetChild(0));
		//TraverseOrderByColumnNames(pSelectNode->GetChild(3));
		return;
	}

	DBG_ASSERT(pSelectNode->Count() >= 4,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	SdbSqlParseNode * pTableExp = pSelectNode->GetChild(3);
	DBG_ASSERT(pTableExp != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(pTableExp->Count() == 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	SdbSqlParseNode * pOptOrderByClause = pTableExp->GetChild(4);
	DBG_ASSERT(pOptOrderByClause != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pOptOrderByClause,opt_order_by_clause),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	if (pOptOrderByClause->Count() == 0)
		return;

	DBG_ASSERT(pOptOrderByClause->Count() == 3,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	SdbSqlParseNode * pOrderingSpecCommalist = pOptOrderByClause->GetChild(2);
	DBG_ASSERT(pOrderingSpecCommalist != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pOrderingSpecCommalist,ordering_spec_commalist),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(pOrderingSpecCommalist->Count() > 0,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	String aColumnName,aColumnAlias;
	String aTableRange;
	for (UINT32 i = 0; i < pOrderingSpecCommalist->Count(); i++)
	{
		SdbSqlParseNode * pOrderingSpec = pOrderingSpecCommalist->GetChild(i);
		DBG_ASSERT(pOrderingSpec != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pOrderingSpec,ordering_spec),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(pOrderingSpec->Count() == 2,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		SdbSqlParseNode * pColumnRef = pOrderingSpec->GetChild(0);
		aTableRange.Erase();
		aColumnName.Erase();
		if(SDB_ISRULE(pColumnRef,column_ref))
		{
			// Column-Name (und TableRange):
			if(SDB_ISRULE(pColumnRef,column_ref))
				GetColumnRange(pColumnRef,aColumnName,aTableRange);
			else // eine Expression
				pColumnRef->ParseNodeToStr(aColumnName,pConnection->GetMetaData(),FALSE,FALSE);

			DBG_ASSERT(aColumnName.Len(),"aColumnName darf nicht leer sein");
		}
		else
		{	// here I found a predicate
			pColumnRef->ParseNodeToStr(aColumnName,pConnection->GetMetaData(),FALSE,FALSE);
		}
		DBG_ASSERT(pColumnRef != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		// Ascending/Descending
		SdbSqlParseNode * pOptAscDesc = pOrderingSpec->GetChild(1);
		DBG_ASSERT(pOptAscDesc != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		BOOL bAscending = TRUE;
		if(pOptAscDesc)
		{
			if (SDB_ISTOKEN(pOptAscDesc,"ASC")) 
				bAscending = TRUE;
			else if (SDB_ISTOKEN(pOptAscDesc,"DESC")) 
				bAscending = FALSE;
		}

		SetOrderByColumnName(aColumnName, aTableRange,bAscending);
		if (! aIteratorStatus.IsSuccessful())
			return;
		
	}
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseSelectionCriteria(const SdbSqlParseNode* pSelectNode)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	aIteratorStatus.Clear();

	if (pSelectNode == NULL) 
	{
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		//aIteratorStatus.SetInvalidStatement();
		return;
	}


	// Parse Tree analysieren (je nach Statement-Typ)
	// und Zeiger auf WHERE-Klausel setzen:
	SdbSqlParseNode * pWhereClause = NULL;

	if (eStatementType == SDB_STATEMENT_SELECT) 
	{
		if(SDB_ISRULE(pSelectNode,union_statement))
		{
			TraverseSelectionCriteria(pSelectNode->GetChild(0));
			TraverseSelectionCriteria(pSelectNode->GetChild(3));
			return;
		}
		DBG_ASSERT(pSelectNode->Count() >= 4,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		SdbSqlParseNode * pTableExp = pSelectNode->GetChild(3);
		DBG_ASSERT(pTableExp != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(pTableExp->Count() == 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		pWhereClause = pTableExp->GetChild(1);
	} else if (SDB_ISRULE(pSelectNode,update_statement_searched)) {
		DBG_ASSERT(pSelectNode->Count() == 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		pWhereClause = pSelectNode->GetChild(4);
	} else if (SDB_ISRULE(pSelectNode,update_statement_positioned)) {
		// nyi
		DBG_ERROR("SdbSqlParseTreeIterator::GetSelectionCriteria: positioned nyi");
	} else if (SDB_ISRULE(pSelectNode,delete_statement_searched)) {
		DBG_ASSERT(pSelectNode->Count() == 4,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		pWhereClause = pSelectNode->GetChild(3);
	} else if (SDB_ISRULE(pSelectNode,delete_statement_positioned)) {
		// nyi
		DBG_ERROR("SdbSqlParseTreeIterator::GetSelectionCriteria: positioned nyi");
	} else {
		// Anderes Statement. Keine Selektionskriterien.
		return;
	}

	if (! SDB_ISRULE(pWhereClause,where_clause)) {
		// Die Where Clause ist meistens optional, d. h. es koennte sich auch
		// um "optional_where_clause" handeln.
		DBG_ASSERT(SDB_ISRULE(pWhereClause,opt_where_clause),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		return;
	}

	// Wenn es aber eine where_clause ist, dann darf sie nicht leer sein:
	DBG_ASSERT(pWhereClause->Count() == 2,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	SdbSqlParseNode * pComparisonPredicate = pWhereClause->GetChild(1);
	DBG_ASSERT(pComparisonPredicate != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");


	//
	// Und nun die Vergleichskriterien abarbeiten (rekursiv, alles ist erstmal ein OR-Kriterium):
	//
	SetORCriteriaPre();
	if (! aIteratorStatus.IsSuccessful()) return;

	TraverseORCriteria(pComparisonPredicate);
	if (! aIteratorStatus.IsSuccessful()) return;

	SetORCriteriaPost();

	// Fehler wird ggf. einfach weitergereicht.
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseORCriteria(SdbSqlParseNode * pSearchCondition)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	if (
			pSearchCondition->Count() == 3 &&
			SDB_ISPUNCTUATION(pSearchCondition->GetChild(0),"(") &&
			SDB_ISPUNCTUATION(pSearchCondition->GetChild(2),")")
		)
	{
		// Runde Klammern um den Ausdruck
		TraverseORCriteria(pSearchCondition->GetChild(1));
	} else if (SDB_ISRULE(pSearchCondition,search_condition) &&
		pSearchCondition->Count() == 3 &&
		SDB_ISTOKEN(pSearchCondition->GetChild(1),"OR"))
	{
		// OR-Verknuepfung:

		for (int i = 0; i < 3; i++) {
			if (i == 1) continue;		// Schluesselwort OR ueberspringen

			// Ist das erste Element wieder eine OR-Verknuepfung?
			if (i == 0 &&
				SDB_ISRULE(pSearchCondition->GetChild(0),search_condition) &&
				pSearchCondition->GetChild(0)->Count() == 3 &&
				SDB_ISTOKEN(pSearchCondition->GetChild(0)->GetChild(1),"OR"))
			{
				// Dann rekursiv absteigen ...
				TraverseORCriteria(pSearchCondition->GetChild(0));

			} else {
				// AND-Kriterien ...
				SetANDCriteriaPre();
				if (! aIteratorStatus.IsSuccessful()) break;

				TraverseANDCriteria(pSearchCondition->GetChild(i));
				if (! aIteratorStatus.IsSuccessful()) break;

				SetANDCriteriaPost();
			}

			if (! aIteratorStatus.IsSuccessful()) break;
		}
	} else {
		// Nur *ein* Kriterium oder eine AND-Verknuepfung von Kriterien.
		// Direkt die AND-Kriterien behandeln.
		SetANDCriteriaPre();
		if (! aIteratorStatus.IsSuccessful()) return;

		TraverseANDCriteria(pSearchCondition);
		if (! aIteratorStatus.IsSuccessful()) return;

		SetANDCriteriaPost();
	}

	// Fehler einfach weiterreichen.
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseANDCriteria(SdbSqlParseNode * pSearchCondition)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	if (
			SDB_ISRULE(pSearchCondition,boolean_primary) &&
			pSearchCondition->Count() == 3 &&
			SDB_ISPUNCTUATION(pSearchCondition->GetChild(0),"(") &&
			SDB_ISPUNCTUATION(pSearchCondition->GetChild(2),")")
		)
	{
		// Runde Klammern
		TraverseANDCriteria(pSearchCondition->GetChild(1));
	}
	// Das erste Element ist eine OR-Verknuepfung
	else  if ( SDB_ISRULE(pSearchCondition,search_condition) && pSearchCondition->Count() == 3 )
	{
		// Dann rekursiv absteigen (dieselbe Row benutzen) ...
		TraverseORCriteria(pSearchCondition->GetChild(0));
		if (! aIteratorStatus.IsSuccessful())
			return;

		// Und mit dem rechten Child weitermachen:
		TraverseANDCriteria(pSearchCondition->GetChild(2));
	}
	// Das erste Element ist (wieder) eine AND-Verknuepfung
	else if ( SDB_ISRULE(pSearchCondition,boolean_term) && pSearchCondition->Count() == 3 )
	{
		// Dann rekursiv absteigen (dieselbe Row benutzen) ...
		TraverseANDCriteria(pSearchCondition->GetChild(0));
		if (! aIteratorStatus.IsSuccessful())
			return;

		// Und mit dem rechten Child weitermachen:
		TraverseANDCriteria(pSearchCondition->GetChild(2));
	}
	 // Sonst einzelne Suchkriterien wie =, !=, ..., LIKE, IS NULL usw. behandeln:
	else if (SDB_ISRULE(pSearchCondition,comparison_predicate) )
	{
		SdbPredicateType ePredicateType;
		SdbSqlParseNode *pPrec = pSearchCondition->GetChild(1);
		if (pPrec->GetNodeType() == SDB_SQL_EQUAL)
			ePredicateType = SDB_PRED_EQUAL;
		else if (pPrec->GetNodeType() == SDB_SQL_NOTEQUAL)
			ePredicateType = SDB_PRED_NOTEQUAL;
		else if (pPrec->GetNodeType() == SDB_SQL_LESS)
			ePredicateType = SDB_PRED_LESS;
		else if (pPrec->GetNodeType() == SDB_SQL_LESSEQ)
			ePredicateType = SDB_PRED_LESSOREQUAL;
		else if (pPrec->GetNodeType() == SDB_SQL_GREATEQ)
			ePredicateType = SDB_PRED_GREATEROREQUAL;
		else if (pPrec->GetNodeType() == SDB_SQL_GREAT)
			ePredicateType = SDB_PRED_GREATER;

		String aValue;
		pSearchCondition->GetChild(2)->ParseNodeToStr(aValue,pConnection->GetMetaData(),FALSE,FALSE);
		TraverseOnePredicate(pSearchCondition->GetChild(0),ePredicateType,aValue,FALSE,pSearchCondition->GetChild(2));
		if (! aIteratorStatus.IsSuccessful())
			return;
	}
	else if (SDB_ISRULE(pSearchCondition,like_predicate) /*&& SDB_ISRULE(pSearchCondition->GetChild(0),column_ref)*/)
	{
		SdbPredicateType ePredicateType;

		DBG_ASSERT(pSearchCondition->Count() >= 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		SdbSqlParseNode * pNum_value_exp	= pSearchCondition->GetChild(3);
		SdbSqlParseNode * pOptEscape		= pSearchCondition->GetChild(4);

		if (pSearchCondition->GetChild(1)->GetTokenID() == SDB_TOKEN_NOT)
			ePredicateType = SDB_PRED_NOTLIKE;
		else
			ePredicateType = SDB_PRED_LIKE;

		DBG_ASSERT(pNum_value_exp != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(pOptEscape != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		if (pOptEscape->Count() != 0)
		{
			aIteratorStatus.SetStatementTooComplex();
			return;
		}

		String aValue;
		SdbSqlParseNode * pParam = NULL;
		if (SDB_ISRULE(pNum_value_exp,parameter))
			pParam = pNum_value_exp;
		else if(pNum_value_exp->IsToken())
			// Normaler Wert
			aValue = pNum_value_exp->TokenValue();
		else
		{
			pNum_value_exp->ParseNodeToStr(aValue,pConnection->GetMetaData(),FALSE,FALSE);
			pParam = pNum_value_exp;
		}

		TraverseOnePredicate(pSearchCondition->GetChild(0),ePredicateType,aValue,FALSE,pParam);
		if (! aIteratorStatus.IsSuccessful())
			return;
	}
	else if (SDB_ISRULE(pSearchCondition,test_for_null) /*&& SDB_ISRULE(pSearchCondition->GetChild(0),column_ref)*/)
	{
		SdbPredicateType ePredicateType;

		DBG_ASSERT(pSearchCondition->Count() >= 3,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISTOKEN(pSearchCondition->GetChild(1),"IS"),"SdbSqlParseTreeIterator: Fehler im Parse Tree")

		if (SDB_ISTOKEN(pSearchCondition->GetChild(2),"NOT") )
			ePredicateType = SDB_PRED_ISNOTNULL;
		else
			ePredicateType = SDB_PRED_ISNULL;

		String aString;
		TraverseOnePredicate(pSearchCondition->GetChild(0),ePredicateType,aString,TRUE,NULL);
		if (! aIteratorStatus.IsSuccessful()) return;
	} else {
		// Etwas anderes unterstuetzen wir (noch) nicht. Basta!
		aIteratorStatus.SetStatementTooComplex();
	}
	// Fehler einfach weiterreichen.
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseOnePredicate(
								SdbSqlParseNode * pColumnRef,
								SdbPredicateType ePredicateType,
								String& rValue,
								BOOL bCompareNull,
								SdbSqlParseNode * pParseNode)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	// Column-Name (und TableRange):
	String aColumnName;
	String aTableRange;
	GetColumnRange(pColumnRef,aColumnName,aTableRange);

	String aName;
	if (pParseNode)			//event. Parameter, oder Columnref oder
	{
		if (SDB_ISRULE(pParseNode,parameter))
		{
			DBG_ASSERT(pParseNode->Count() > 0,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
			SdbSqlParseNode * pMark = pParseNode->GetChild(0);

			String aParameterName;
			if (SDB_ISPUNCTUATION(pMark,"?"))
			{
				// Name = "?", da kein Parametername verfuegbar (z. B. bei Native SQL)
				rValue = '?';
				aName  = '?';
			}
			else if (SDB_ISPUNCTUATION(pMark,":"))
			{
				rValue = pParseNode->GetChild(1)->TokenValue();
				aName = ':';
			}
			else if (SDB_ISPUNCTUATION(pMark,"["))
			{
				rValue = pParseNode->GetChild(1)->TokenValue();
				aName = '[';
			}
			else
			{
				DBG_ERROR("SdbSqlParseTreeIterator: Fehler im Parse Tree");
			}
		}
		else if (SDB_ISRULE(pParseNode,column_ref))// Column-Name (und TableRange):
			GetColumnRange(pParseNode,aName,rValue);
	}

	// "Set"-Routine aufrufen ...
	if (bCompareNull) {
		// nyi: was soll das? Vergleich auf NULL geht nur ueber IS NULL oder IS NOT NULL!
		SetPredicate(aColumnName, aTableRange,ePredicateType, String(), String());
	} else {
		SetPredicate(aColumnName, aTableRange, ePredicateType, rValue, aName);
	}

	// Fehler einfach weiterreichen ...
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseAssignments()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	DBG_ERROR("SdbSqlParseTreeIterator::TraverseAssignments: noch nicht implementiert");
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::TraverseAll()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	TraverseTableNames();
	if (! aIteratorStatus.IsSuccessful()) return;

	if (eStatementType == SDB_STATEMENT_SELECT)
	{
		const SdbSqlParseNode* pSelectNode = pParseTree;
		TraverseSelectColumnNames(pSelectNode);
		if (! aIteratorStatus.IsSuccessful())
			return;
		TraverseOrderByColumnNames(pSelectNode);
		if (! aIteratorStatus.IsSuccessful())
			return;
		TraverseSelectionCriteria(pSelectNode);
		if (! aIteratorStatus.IsSuccessful())
			return;
	}
	else if (eStatementType == SDB_STATEMENT_INSERT || eStatementType == SDB_STATEMENT_UPDATE)
	{
		TraverseAssignments();
		if (! aIteratorStatus.IsSuccessful())
			return;
	} else if (eStatementType == SDB_STATEMENT_INSERT) {
		// schon alles erledigt
	} else {
		SdbStatus aSDB;	aSDB.SetInvalidStatement();
		CallError(3,aSDB);
		//aIteratorStatus.SetInvalidStatement();
	}
}

// Dummy-Implementationen:

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetTableName(const String & rTableName, const String & rDBName, const String& rOwner,
							  const String & rTableRange)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetTableName"
		 << (const char *) rTableName << ", "
		 << (const char *) rSchemaName << ", "
		 << (const char *) rTableRange
		 << "\n";
#endif
}
//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::AppendColumns(ULONG i,const SdbTableRef* pTableRef)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	if (!pTableRef->Is())
		return;

	const SdbColumnsRef  rColumns = (*pTableRef)->OriginalColumns();
	for(USHORT k=1;k<rColumns->Count();k++)
	{
		ULONG nExpr(1);
		const SdbColumn* rCol = (*rColumns)[k];
		String aColName(rCol->GetRealName());

		// testen, ob es in meinen schon vorhandenen columns schon eine mit dem
		// selben Namen gibt
		BOOL bUnique;
		do
		{
			bUnique = TRUE;
			for (ULONG j=0; j<aSelectColumns->Count(); ++j)
			{
				const SdbColumn* pCurrent = aSelectColumns->Column(j);
				if (pCurrent && (pCurrent->GetRealName() == aColName))
				{
					aColName += String::CreateFromInt32(nExpr++);
					bUnique = FALSE;
					break;	// aus dem for raus
				}
			}
		} while (!bUnique);
/*
		// FS - 18.05.99 -
		// wir duerfen natuerlich nicht in den Columns aller ueberhaupt beteiligten
		// TableRanges nachsehen, sondern nur in den Columns, die wirklich schon beteiligt
		// sind (also aSelectColumns). Sonst passiert es, dass bei einem "select a.* from a,b"
		// eine Spalte in a, die dem Namen nach auch in b steckt, einen Alias verpasst bekommt.
		// (wenn beide eine Column "x" haben, wuerde obiges Statement also x1 liefern, obwohl x
		// ausreicht)
RETRY:	BOOL bFound = FALSE;
		for(ULONG j=0;j<aTableRangeMap.Count();j++)
		{
			SdbTableRef* pTableRef2=aTableRangeMap.GetObject(j);
			if(pTableRef2 == pTableRef)
				continue;
			if (!pTableRef2->Is())
				continue;
			if((*pTableRef2)->OriginalColumns()->Column(aColName))
			{
				bFound = TRUE;
				continue;
			}
		}
		if(bFound)
		{
			aColName += String(nExpr++);
			goto RETRY;
		}
		else
*/
		{
			SdbColumn *pColumn = pConnection->CreateColumn(GetUniqueColumnName(aColName),*rCol);
			pColumn->SetTableName((*pTableRef)->GetFullName());
			aSelectColumns->AddColumn(pColumn);
		}
	}
}
//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetSelectColumnName(const String & rColumnName,const String & rColumnAlias, const String & rTableRange,BOOL bFkt)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	if(rColumnName.GetChar(0) == '*' && !rTableRange.Len())
	{	// Suche "uber alle vorkommenden Tabellen
		for(ULONG i=0;i<aTableRangeMap.Count();i++)
			AppendColumns(i,aTableRangeMap.GetObject(i));
	}
	else if(rColumnName.GetChar(0) == '*' && rTableRange.Len())	 // alle Columns aus dieser Tabelle
	{
		SdbTableRef* pTableRef = aTableRangeMap.Lookup(rTableRange);
		if(!pTableRef)
		{
			SdbStatus aSDB;	aSDB.SetInvalidStatement();
			String strExpression = rTableRange;
			strExpression += '.';
			strExpression += rColumnName;
			CallError(2, aSDB, strExpression);
		}
		else
			AppendColumns(0, pTableRef);
	}
	else if(!rTableRange.Len())// ein Columnname existiert
	{
		if(!bFkt)
		{
			UINT32 nCount = aSelectColumns->Count();
			for(ULONG i=0;i<aTableRangeMap.Count();i++)
			{
				SdbTableRef *pTableRef = aTableRangeMap.GetObject(i);
				if (pTableRef->Is())
				{
					const SdbColumnsRef rColumns = (*pTableRef)->OriginalColumns();
					const SdbColumn* pColumn = rColumns->Column(rColumnName);
					if(pColumn)
					{
						SdbColumn *pNewColumn = pConnection->CreateColumn(GetUniqueColumnName(rColumnAlias),*pColumn);
						pNewColumn->SetTableName((*pTableRef)->GetFullName());
						aSelectColumns->AddColumn(pNewColumn);
						continue; // diese Column darf nur einmal vorkommen
					}
				}
			}
			if (nCount == aSelectColumns->Count())
			{
				SdbStatus aSDB;	aSDB.SetInvalidStatement();
				String strExpression = rTableRange;
				if (strExpression.Len())
					strExpression += '.';
				strExpression += rColumnName;
				CallError(2, aSDB, strExpression);
			}
		}
		else
		{
			SdbColumn *pNewColumn = pConnection->CreateColumn(GetUniqueColumnName(rColumnAlias),SDB_DBTYPE_VARCHAR);
			pNewColumn->SetFunction(bFkt);
			pNewColumn->SetRealName(rColumnName);
			aSelectColumns->AddColumn(pNewColumn);
		}
	}
	else	// ColumnName und Tablename vorhanden
	{
		SdbTableRef* pTableRef = aTableRangeMap.Lookup(rTableRange);
		BOOL bError = FALSE;
		if (pTableRef && pTableRef->Is())
		{
			const SdbColumn* pColumn = !bFkt ? (*pTableRef)->OriginalColumns()->Column(rColumnName) : NULL;
			if (bFkt)
			{
				SdbColumn *pNewColumn = pConnection->CreateColumn(GetUniqueColumnName(rColumnAlias),SDB_DBTYPE_VARCHAR);
				pNewColumn->SetTableName((*pTableRef)->GetFullName());
				pNewColumn->SetFunction(TRUE);
				pNewColumn->SetRealName(rColumnName);
				aSelectColumns->AddColumn(pNewColumn);
			}
			else if (pColumn)
			{
				SdbColumn *pNewColumn = pConnection->CreateColumn(GetUniqueColumnName(rColumnAlias),*pColumn);
				pNewColumn->SetTableName((*pTableRef)->GetFullName());
				pNewColumn->SetRealName(rColumnName);
				aSelectColumns->AddColumn(pNewColumn);
			}
			else
				bError = TRUE;
		}
		else
			bError = TRUE;

		// Tabelle existiert nicht oder Feld nicht vorhanden
		if (bError)
		{
			SdbStatus aSDB;	aSDB.SetInvalidStatement();
			String strExpression = rTableRange;
			if (strExpression.Len())
				strExpression += '.';
			strExpression += rColumnName;
			if (!CallError(2, aSDB, strExpression))
				return;

			SdbColumn *pNewColumn = pConnection->CreateColumn(GetUniqueColumnName(rColumnAlias),SDB_DBTYPE_VARCHAR);
			pNewColumn->SetFunction(TRUE); // um den Unterschied deutlich zu machen
			aSelectColumns->AddColumn(pNewColumn);
		}
	}
}
//-----------------------------------------------------------------------------
String SdbSqlParseTreeIterator::GetUniqueColumnName(const String & rColumnName)	const
{
	String aAlias(rColumnName);
	const SdbColumn* pPrevCol = aSelectColumns->Column(rColumnName);
	// existiert Name  in aSelectColumns ?
	ULONG nExprCnt(0);
	while(pPrevCol)
	{
		(aAlias = rColumnName) += String::CreateFromInt32(++nExprCnt);
		pPrevCol = aSelectColumns->Column(aAlias);
	}
	return aAlias;
}
//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetOrderByColumnName(const String & rColumnName, const String & rTableRange,BOOL bAscending)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetOrderByColumnName: "
		 << (const char *) rColumnName << ", "
		 << (const char *) rTableRange << ", "
		 << (bAscending ? "TRUE" : "FALSE")
		 << "\n";
#endif
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetORCriteriaPre()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetORCriteriaPre\n";
#endif
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetORCriteriaPost()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetORCriteriaPost\n";
#endif
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetANDCriteriaPre()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetANDCriteriaPre\n";
#endif
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetANDCriteriaPost()
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetANDCriteriaPost\n";
#endif
}

//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetPredicate(const String & rColumnName,
							  const String & rTableRange,
							  SdbPredicateType ePredicateType,
							  const String & rValue,
							  const String & rParameterName)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetPredicate: "
		 << (const char *) rColumnName << ", "
		 << (const char *) rTableRange << ", "
		 << (char) ePredicateType << ", "
		 << (const char *) rValue << ", "
		 << (const char *) rParameterName
		 << "\n";
#endif
}


//-----------------------------------------------------------------------------
void SdbSqlParseTreeIterator::SetAssign(const String & rColumnName,
						   const String & rValue, BOOL bSetNull,
						   const String & rParameterName)
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);
	// nichts zu tun ...

#ifdef SDB_TEST_PARSETREEITERATOR
	cout << "SdbSqlParseTreeIterator::SetAssign: "
		 << (const char *) rColumnName << ", "
		 << (const char *) rValue << ", "
		 << (bSetNull ? "TRUE" : "FALSE") << ", "
		 << (const char *) rParameterName
		 << "\n";
#endif
}

//-----------------------------------------------------------------------------
const SdbSqlParseNode* SdbSqlParseTreeIterator::WhereTree() const
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	if (!pParseTree)
		return NULL;

	// Parse Tree analysieren (je nach Statement-Typ)
	// und Zeiger auf WHERE-Klausel setzen:
	SdbSqlParseNode * pWhereClause = NULL;
	if(StatementType() == SDB_STATEMENT_SELECT)
	{
		DBG_ASSERT(pParseTree->Count() >= 4,"ParseTreeIterator: Fehler im Parse Tree");
		SdbSqlParseNode * pTableExp = pParseTree->GetChild(3);
		DBG_ASSERT(pTableExp != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
		DBG_ASSERT(pTableExp->Count() == 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

		pWhereClause = pTableExp->GetChild(1);
	}
	else if (SDB_ISRULE(pParseTree,update_statement_searched) ||
			 SDB_ISRULE(pParseTree,delete_statement_searched))
	{
		pWhereClause = pParseTree->GetChild(pParseTree->Count()-1);
	}

	return pWhereClause;
}

//-----------------------------------------------------------------------------
const SdbSqlParseNode* SdbSqlParseTreeIterator::OrderTree() const
{
	DBG_CHKTHIS(SdbSqlParseTreeIterator,NULL);

	if (!pParseTree || StatementType() != SDB_STATEMENT_SELECT)
		return NULL;

	// Parse Tree analysieren (je nach Statement-Typ)
	// und Zeiger auf ORDER-Klausel setzen:
	SdbSqlParseNode * pOrderClause = NULL;
	DBG_ASSERT(pParseTree->Count() >= 4,"ParseTreeIterator: Fehler im Parse Tree");
	SdbSqlParseNode * pTableExp = pParseTree->GetChild(3);
	DBG_ASSERT(pTableExp != NULL,"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(SDB_ISRULE(pTableExp,table_exp),"SdbSqlParseTreeIterator: Fehler im Parse Tree");
	DBG_ASSERT(pTableExp->Count() == 5,"SdbSqlParseTreeIterator: Fehler im Parse Tree");

	pOrderClause = pTableExp->GetChild(4);
	// Wenn es aber eine order_by ist, dann darf sie nicht leer sein:
	if(pOrderClause->Count() != 3)
		pOrderClause = NULL;
	return pOrderClause;
}
//-----------------------------------------------------------------------------
BOOL SdbSqlParseTreeIterator::CallError(USHORT nError,const SdbStatus& rStatus,const String& rExpression)
{
	SdbParseIteratorErrorInfo aErrorInfo;
	aErrorInfo.aStatus = rStatus;
	aErrorInfo.nErrorCode = nError;
	aErrorInfo.aExpression = rExpression;

	long nResult = aErrorHdl.Call(&aErrorInfo);
	if (nResult == RET_CONTINUE)
		return TRUE;

	if (nResult == RET_HANDLED)
	{
		aIteratorStatus.Set(SDB_STAT_SUCCESS);
		return FALSE;
	}

	aIteratorStatus = rStatus;
	return FALSE;
}

