/*************************************************************************
 *
 *  $RCSfile: sqltab.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbdbimp.hxx"
#include "sqltab.hxx"

#ifndef SDB_SQLCOL_HXX
#include "sqlcol.hxx"
#endif

//------------------------------------------------------------------------
SdbSqlTable::SdbSqlTable(SdbConnection* pConnection) : SdbTable(pConnection)
{
}



//------------------------------------------------------------------------
BOOL SdbSqlTable::AlterImpl()
{
	// Alter wird immer ueber eine Transaktion ausgefuehrt
	SdbConnectionRef xConnection = GetConnection()->GetEnvironment()->OpenConnection(*GetConnection());
	if (!xConnection.Is())
	{
		aStatus = GetConnection()->GetEnvironment()->Status();
		return FALSE;
	}

	xConnection->BeginTrans();
	aStatus = xConnection->Status();
	if (!aStatus.IsSuccessful())
		return FALSE;

	// Ueberpruefen, ob Drop ausgefuehrt werden soll
	// Zunaechst werden die Felder, die nicht mehr benoetigt werden, geloescht.
	INT16 i;
	for (i = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && i > 0; i--)
	{
		const SdbColumn* pColumn = (*aOriginalColumns)[i];
		const SdbColumn* pRefColumn = (*aColumns)[pColumn->GetName()];

		if (!pRefColumn) // es gibt keine Column dieses Namens mehr
		{
			if (aPKeyName.Len() && pColumn->IsPrimaryKey())
			{
				if (DropPrimaryKey())
					aPKeyName.Erase();
			}

			if (!DropColumn(pColumn))
				break;
		}
	}


	// Herausfinden, welche Spalten geaendert wurden
	for (i = aOriginalColumns->Count() - 1; aStatus.IsSuccessful() && i > 0; i--)
	{
		const SdbSqlColumn* pColumn =  (SdbSqlColumn*) (*aOriginalColumns)[i];
		const SdbSqlColumn* pRefColumn = (SdbSqlColumn*) (*aColumns)[pColumn->GetName()];
		if (pRefColumn && !pColumn->AlterCompare(*pRefColumn, SDB_FLAGS_NULLALLOWED|SDB_FLAGS_AUTOINCREMENT))
			AlterColumn(pColumn, pRefColumn);
	}

	// und jetzt die neuen bzw. geaenderten hinzufuegen
	List AddColList;
	for (i = 1; aStatus.IsSuccessful() && i < aColumns->Count(); i++)
	{
		const SdbColumn* pColumn = (*aColumns)[i];
		const SdbColumn* pRefColumn = (*aOriginalColumns)[pColumn->GetName()];
		if (!pRefColumn)
		{
			// gibt es eine neue Schluesselspalte, dann PKey loeschen
			if (aPKeyName.Len() && pColumn->IsPrimaryKey())
			{
				if (DropPrimaryKey())
					aPKeyName.Erase();
			}
			AddColList.Insert((void*)pColumn);
		}
	}

	// aOriginalColumns aktualisieren
	AddColumns( AddColList);
	if (aStatus.IsSuccessful())
		for (USHORT i=0; i < AddColList.Count(); i++)
			aOriginalColumns->AddColumn(CreateColumn(*((const SdbColumn*)AddColList.GetObject(i))));

	// gibt es noch Aenderungen am Schluessel?
	for (i = 1; aPKeyName.Len() && aStatus.IsSuccessful() && i < aColumns->Count(); i++)
	{
		const SdbColumn* pColumn = (*aColumns)[i];
		const SdbColumn* pRefColumn = (*aOriginalColumns)[pColumn->GetName()];
		if (pRefColumn && pColumn->IsPrimaryKey() != pRefColumn->IsPrimaryKey())
		{
			if (DropPrimaryKey())
				aPKeyName.Erase();
		}
	}

	if (!aStatus.IsSuccessful())
		xConnection->RollbackTrans();
	else
	{
		// muss jetzt noch ein Primary Key hinzugefuegt werden?
		if (!aPKeyName.Len()) CreatePrimaryKey();


		if (!aStatus.IsSuccessful() || !xConnection->CommitTrans())
		{
			aStatus = xConnection->Status();
			xConnection->RollbackTrans();
		}
	};

	xConnection->CommitTrans();

	return aStatus.IsSuccessful();
}

//------------------------------------------------------------------------
BOOL SdbSqlTable::CreatePrimaryKey()
{

	String aPColumns;
	for (UINT16 i = 1; i < aColumns->Count(); i++)
	{
		if ((*aColumns)[i]->IsPrimaryKey())
		{
			aPColumns += GetConnection()->IdentifierQuotation((*aColumns)[i]->GetName());
			aPColumns += ',';
		}
	}
	if (aPColumns.Len())
	{
		// Ein Komma ist sicherlich noch zuviel ...
		aPColumns.SetChar(aPColumns.Len()-1,')');

		String aCommand = String::CreateFromAscii(" ALTER TABLE ");;
		aCommand += GetConnection()->QuoteTableName(QualifiedName());
		aCommand += String::CreateFromAscii(" ADD PRIMARY KEY (");;
		aCommand += aPColumns;
		GetConnection()->ExecuteSQL(aCommand);
		aStatus = GetConnection()->Status();
		return (aStatus.IsSuccessful());
	}
	else return TRUE;
}


//------------------------------------------------------------------------
BOOL SdbSqlTable::DropPrimaryKey()
{
	String aStmt = String::CreateFromAscii("ALTER TABLE ");;
	aStmt += GetConnection()->QuoteTableName(QualifiedName());
	aStmt += String::CreateFromAscii(" DROP CONSTRAINT ");;
	aStmt += aPKeyName;
	GetConnection()->ExecuteSQL(aStmt);
	aStatus = GetConnection()->Status();
	return (aStatus.IsSuccessful());
};

//------------------------------------------------------------------------
BOOL SdbSqlTable::AddColumnTypeString(String& rStatement, const SdbColumn& rColumn)
{
	// Gibt es zu dieser Column eine Typinformationen
	SdbConnection* pConnection = GetConnection();

	const SdbTypeInfo* pInfo = pConnection->GetColumnTypeInfo(rColumn);

	if (!pInfo)
	{
		// in der Column festgelegter Typ wird vom Treiber nicht untersttzt
		pInfo = pConnection->GetColumnTypeInfo(rColumn.GetType());
		if(!pInfo){
			DBG_ASSERT(0,"SdbSql Table: GetColumnTypeInfo == NULL");
			aStatus.SetDriverNotCapableError();
			return FALSE;
		}
	}

	rStatement += pConnection->IdentifierQuotation(rColumn.GetName());
	rStatement += ' ';
	rStatement += pInfo->aTypeName;

	switch(pInfo->eType)
	{
		case SDB_DBTYPE_CHAR:
		case SDB_DBTYPE_VARCHAR:
		case SDB_DBTYPE_BINARY:
		case SDB_DBTYPE_VARBINARY:
			// Typen mit Laengenangabe
			rStatement += '(';
			rStatement += rColumn.GetLength();
			rStatement += ')';
			break;
		case SDB_DBTYPE_DECIMAL:
		case SDB_DBTYPE_NUMERIC:
			// Typen mit Precision und Scale
			rStatement += '(';
			rStatement += rColumn.GetPrecision();
			rStatement += ',';
			rStatement += rColumn.GetScale();
			rStatement += ')';
			break;
	}

	if (!rColumn.IsNullAllowed())
		rStatement += String::CreateFromAscii(" NOT NULL");;

	return TRUE;
}


