/*************************************************************************
 *
 *  $RCSfile: tcol.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <iostream.h>

#include "sdbcol.hxx"

static void show_column(const char * title, const SdbColumn * c)
{
	if (c == NULL)
		cout << title << ": NULL pointer";
	else {
		cout << title << ": " << (const char *) c->Name() << ", " << (int)c->Type() << ", " << c->Length() << ", " << c->Scale() << ", " << c->AllFlags();
		if (c->IsNullAllowed()) cout << " NULL"; else cout << " NOT NULL";
		if (c->IsPrimaryKey()) cout << " PRIMARY KEY";
		if (c->IsUpdateAllowed()) cout << " (Update possible)"; else cout << " (Update impossible)";
	}
	cout << "\n";
}

static void show_columns(const char * title, const SdbColumns * cs)
{
	cout << "Columns:" << title << "\n";
	for (UINT16 i = 0; i < cs->Count(); i++) {
		show_column("    ",cs->Column(i));
	}
}

int main() {
	SdbColumn * c1 = new SdbColumn("Test1", SDB_INTEGER, 10);
	SdbColumn * c2 = new SdbColumn("Test2", SDB_NUMERIC, 22, 1);
	SdbColumn * c3 = new SdbColumn("Test3", SDB_VARCHAR, 22);
	SdbColumn * c4 = new SdbColumn("Test4", SDB_VARCHAR, 10);
	SdbColumn * c5 = new SdbColumn("Test5", SDB_VARCHAR, 2);

	show_column("c1", c1);
	show_column("c2", c2);
	show_column("c3", c3);
	show_column("c4", c4);
	show_column("c5", c5);

	c1->SetFlag(SdbColumn::NullAllowed);
	show_column("c1 NullAllowed", c1);
	c1->SetFlag(SdbColumn::PrimaryKey);
	show_column("c1 PrimaryKey", c1);
	c1->SetFlag(SdbColumn::UpdateAllowed);
	show_column("c1 UpdateAllowed", c1);

	c1->ResetFlag(SdbColumn::NullAllowed);
	show_column("c1 NullAllowed", c1);
	c1->ResetFlag(SdbColumn::PrimaryKey);
	show_column("c1 PrimaryKey", c1);
	c1->ResetFlag(SdbColumn::UpdateAllowed);
	show_column("c1 UpdateAllowed", c1);

	c2->SetFlag(SdbColumn::UpdateAllowed);
	c2->SetAllFlags(SdbColumn::NullAllowed | SdbColumn::PrimaryKey);
	show_column("c2 NullAllowed and Primary Key", c2);

	cout << "-------------------------------------------------------------------\n";

	SdbColumns * cs1 = new SdbColumns();

	cs1->AddColumn(c1);
	cs1->AddColumn(c2);
	cs1->AddColumn(c3);

	show_columns("cs1",cs1);

	cout << "Column #" << cs1->ColumnNumber("Test3") << ": ";
	show_column("Test1",cs1->Column("Test3"));

	cout << "Column #" << cs1->ColumnNumber("Test1") << ": ";
	show_column("Test1",cs1->Column("Test1"));

	cout << "Column #" << cs1->ColumnNumber("Bla") << ": ";
	show_column("Test1",cs1->Column("Bla"));
	if (cs1->ColumnNumber("Bla") == SDB_COLUMN_NOTFOUND) cout << "not found\n";


	cs1->InsertColumn(c4,2);
	cout << "Inserted Column #2 == " << cs1->ColumnNumber("Test4") << ": ";
	show_columns("cs1",cs1);

	cs1->ReplaceColumn(c5,2);
	cout << "Replaced Column #2 == " << cs1->ColumnNumber("Test5") << ": ";
	show_columns("cs1",cs1);

	cs1->RemoveColumn(2);
	cout << "Removed Column #2" << ": ";
	show_columns("cs1",cs1);


	cout << "second reference:\n";
	cs1->AddReference();
	cout << "cs1 reference count: " << cs1->ReferenceCount() << "\n";

//#define WILL_ASSERT
#ifdef WILL_ASSERT
	cs1->AddColumn(c1);	// Hinzufuegen nach AddReference verboten!
#endif

	SdbColumns * cs2 = new SdbColumns(*cs1);
	SdbColumns * cs3 = new SdbColumns();
	*cs3 = *cs1;

	cout << "cs1 wird geaendert, aber nicht die Kopien:\n";
	c2->SetAllFlags(0);
	show_columns("cs1",cs1);

	show_columns("cs2",cs2);
	cout << "cs2 reference count: " << cs2->ReferenceCount() << "\n";
	show_columns("cs3",cs3);
	cout << "cs3 reference count: " << cs3->ReferenceCount() << "\n";
	cs3->AddReference();

	cout << "first release:\n";
	cs1->ReleaseReference();
	cout << "cs1 reference count: " << cs1->ReferenceCount() << "\n";

	cout << "second release:\n";
	cs1->ReleaseReference();
#ifdef MIGHT_CRASH
	cout << "cs1 reference count: " << cs1->ReferenceCount() << "\n";
	show_columns("cs1 void!",cs1);
#endif

	// Columns von cs3 bleiben erhalten, da sie *kopiert* worden sind ...
	show_columns("cs3",cs3);
	cout << "cs3 reference count: " << cs3->ReferenceCount() << "\n";
	cs3->ReleaseReference();

	cout << "Done.\n";
	return 0;
}
