/*************************************************************************
 *
 *  $RCSfile: tdoc.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <iostream.h>
#include <tools.hxx>

//#include "sdb.hxx"

#include "sdbenv.hxx"
#include "sdbconn.hxx"

#include "dbgmsg.cxx"

static void show_status(const char * title, const SdbStatus & s)
{
	cout << "\n";
	cout << title << ":\n";

	cout << "Result:            ";
	switch(s.Result()) {
		case SDB_STAT_SUCCESS:			cout << "SUCCESS\n"; return;
		case SDB_STAT_SUCCESS_WITH_INFO:	cout << "SUCCESS_WITH_INFO\n"; break;;
		case SDB_STAT_ERROR:        		cout << "ERROR\n"; break;
		case SDB_STAT_NO_DATA_FOUND:		cout << "NO_DATA_FOUND\n"; return;
		case SDB_STAT_STILL_EXECUTING:	cout << "STILL_EXECUTING\n"; return;
	}
	cout << "SQLState:          " << (const char *)s.SQLState() << "\n";
	cout << "ErrorMessage:      " << (const char *)s.ErrorMessage() << "\n";
	cout << "NativeErrorCode:	" << s.NativeErrorCode() << "\n";
	cout << "ErrorInfo:         " << (const char *)s.ErrorInfo() << "\n";
}

static void show_column(const char * title, const SdbColumn * c)
{
	if (c == NULL)
		cout << title << ": NULL pointer";
	else {
		cout << title << ": " << (const char *) c->Name() << ", " << (int)c->Type() << ", " << c->Length() << ", " << c->Scale() << ", " << c->AllFlags();
		if (c->IsNullAllowed()) cout << " NULL"; else cout << " NOT NULL";
		if (c->IsPrimaryKey()) cout << " PRIMARY KEY";
		if (c->IsUpdateAllowed()) cout << " (Update possible)"; else cout << " (Update impossible)";
	}
	cout << "\n";
}

static void show_columns(const char * title, const SdbColumns * cs)
{
	cout << "Columns:" << title << "\n";
	for (UINT16 i = 0; i < cs->Count(); i++) {
		show_column("    ",cs->Column(i));
	}
}

static void show_variable(const char * title, const SdbVariable * v)
{
	cout << title << ": ";
	if (v == NULL) {
							cout << "[not bound]";
	} else {
		switch(v->GetType()) {
		case SbxEMPTY:		cout << "EMPTY"; break;
		case SbxNULL:		cout << "NULL"; break;
		case SbxOBJECT:		cout << "OBJECT"; break;
		case SbxINTEGER:	cout << "INTEGER = " << v->GetInteger(); break;
		case SbxLONG:		cout << "LONG = " << v->GetLong(); break;
		case SbxSINGLE:		cout << "SINGLE = " << v->GetSingle(); break;
		case SbxDOUBLE:		cout << "DOUBLE = " << v->GetDouble(); break;
		case SbxBOOL:		cout << "BOOL = " << (v->GetBool() ? "TRUE" : "FALSE"); break;
		case SbxERROR:		cout << "ERROR = " << (int)v->GetError(); break;
		case SbxSTRING:		cout << "STRING = " << (const char *) v->GetString(); break;
		case SbxMEMORYSTREAM:	cout << "MEMORYSTREAM: ";
							{
								SbxMemoryStreamRef pMemoryStream = (SbxMemoryStream *)v->GetObject();
								const char * pData = pMemoryStream->GetData();
								ULONG nSize = pMemoryStream->Seek(STREAM_SEEK_TO_END);
								cout << nSize << " bytes in stream\nBytes: ";
								for (ULONG i = 0; i < nSize; i++) {
									cout << hex << (((int) pData[i]) & 0xff);
									cout << dec << " ";
								}
							}
							break;
		default: 			cout << "[unknown type]";
		}
	}
	cout << "\n";
}

static void show_row(const char * title, const SdbRow * p)
{
	cout << "Row: " << title << "\n";
	for (UINT16 i = 0; i < p->Count(); i++) {
		cout << "	Column(" << i << ") [" << (const char *) p->Column(i)->Name() << "]";
		show_variable("",p->Variable(i));
	}
}

static void show_cursors(const char * title, const SdbConnection * conn1)
{
	cout << "SdbCursorList: " << title << "\n";

	for (unsigned int i = 0; i < conn1->CursorList().Count(); i++) {
		SdbCursor * curs2 = conn1->CursorList().GetObject(i);
		cout << "Cursor " << i << ": '" << (const char *)curs2->SQLStatement() << "'\n";
		cout << "          " << (curs2->IsOpen() ? "Open" : "! Open") << " " << (curs2->IsOffRange() ? "OffRange" : "! OffRange") << "\n";
	}

	cout << "\n";
}

static void fetch_rows(const char * title, SdbCursor * curs1, const UINT32 nShowRows = 0xffffffff)
{
	cout << "Fetching: " << title << "\n";
	UINT32 nRows = 0;
	do {
		curs1->FetchRow();
		if (! curs1->Status().IsSuccessful()) break;

		if (! curs1->IsOffRange()) {
			nRows++;

			SdbRow * pRow = curs1->ResultRow();
			if (nRows <= nShowRows) show_row("curs1",pRow);
		}
	} while (! curs1->IsOffRange());
	show_status("Status of last Fetch",curs1->Status());
	cout << "\nFetched "<< nRows << " rows.\n\n";
}

static void show_document(const char * title, const SdbDocument * d)
{
	cout << title << ": ";
	if (d == NULL) {
							cout << "[NULL]";
	} else {
		cout << "SchemaName:      " << (const char *) d->SchemaName() << "\n";
		cout << "DocumentType:    " << d->DocumentType() << "\n";
		cout << "DocumentName:    " << (const char *) d->DocumentName() << "\n";
		if (d->IsOpen()) {
			cout << "DocumentVersion: " << d->DocumentVersion() << "\n";
			cout << "DocumentStorage: ";

			const char * pData = d->DocumentStorage()->GetData();
			ULONG nSize = d->DocumentStorage()->Seek(STREAM_SEEK_TO_END);
			cout << nSize << " bytes in stream\n                 Bytes: ";
			for (ULONG i = 0; i < nSize; i++) {
				cout << hex << (((int) pData[i]) & 0xff);
				cout << dec << " ";
			}
			cout << "\n";
		} else {
			cout << "[NOT OPEN]\n";
		}
	}
	cout << "\n";
}

#ifdef WNT
__cdecl
#endif
main()
{
	DBG_INIT();

	char srv[100];
	char usr[100];
	char pwd[100];
	cout << "Server:   "; cin.getline(srv,sizeof srv);
	cout << "Username: "; cin.getline(usr,sizeof usr);
	cout << "Password: "; cin.getline(pwd,sizeof pwd);

	SdbEnvironment * pEnv = SdbEnvironment::GetAddress();
	DBG_ASSERT(pEnv != NULL,"Kein SdbEnvironment");

	SdbConnection * conn1 = pEnv->CreateConnection(SDB_SERVER_ODBC,String(srv),String(usr),String(pwd),String());
	if (! conn1->Status().IsSuccessful()) {
		show_status("conn1 CreateConnection",conn1->Status());
		return 1;
	}

	conn1->Connect();
	if (! conn1->Status().IsSuccessful()) {
		show_status("conn1 Connect",conn1->Status());
		return 1;
	}

	cout << "Connected.\n\n";

	char sch[100];
	cout << "Schema: "; cin.getline(sch,sizeof sch);
	conn1->SetSchema(sch);
	if (! conn1->Status().IsSuccessful()) {
		show_status("conn1 SetSchema",conn1->Status());
		return 1;
	}

	SdbDocument * doc1 = conn1->CreateDocument();
	show_status("conn1 CreateDocument",conn1->Status());

	if (! doc1) return 1;

	cout << "\n";
	char aDocumentName[200];
	UINT32 nDocumentType;
	char aMode[100];
	while (TRUE) {
again:
		show_document("doc1",doc1);

		cout << "Mode: (C)ommit/Roll(b)ack/(I)solation and Autocommit/\n";
		cout << "      (G)etDocumentNames/(M)akeContainer/Dro(p)Container/(O)pen/(N)ew/(D)rop/(R)ename/e(X)it: ";
		cin >> aMode;

		if (*aMode == 'c' || *aMode == 'C') {
			conn1->Commit();
			show_status("Commit",conn1->Status());
			goto again;
		} else if (*aMode == 'b' || *aMode == 'B') {
			conn1->Rollback();
			show_status("Rollback",conn1->Status());
			goto again;
		}

		if (*aMode == 'i' || *aMode == 'I') {
			int n;
			cout << "Autocommit: ";
			cin >> n;
			conn1->SetAutoCommit(n);
			show_status("SetAutoCommit",conn1->Status());

			cout << "Isolation: ";
			cin >> n;
			conn1->SetIsolationLevel((SdbIsolationLevel)n);
			show_status("SetIsolationLevel",conn1->Status());

			goto again;

		} else if (*aMode == 'g' || *aMode == 'G') {
			cout << "DocumentType: ";
			cin >> nDocumentType;

			String aDocumentName;

			BOOL bFirst = TRUE;
			BOOL bFound = TRUE;
			do {
				bFound = conn1->GetDocumentName(nDocumentType,aDocumentName,bFirst);
				if (! bFound) break;

				cout << "DocumentName: " << (const char *) aDocumentName << "\n";
				bFirst = FALSE;
			} while (bFound);
			show_status("GetDocumentName",conn1->Status());

			goto again;
		}
		if (*aMode == 'm' || *aMode == 'M') {
			conn1->MakeDocumentContainer();
			show_status("MakeDocumentContainer",conn1->Status());
			goto again;
		}
		if (*aMode == 'p' || *aMode == 'P') {
			conn1->DropDocumentContainer();
			show_status("DropDocumentContainer",conn1->Status());
			goto again;
		}

		if (*aMode == 'o' || *aMode == 'O' ||
			*aMode == 'd' || *aMode == 'D' ||
			*aMode == 'r' || *aMode == 'R') {
			cout << "DocumentType: ";
			cin >> nDocumentType;
			cout << "DocumentName: ";
			cin >> aDocumentName;
		}

		if (*aMode == 'd' || *aMode == 'D') {
			doc1->Drop(nDocumentType,aDocumentName);
			show_status("Drop",doc1->Status());
			goto again;
		} else if (*aMode == 'r' || *aMode == 'R') {
			UINT32 nNewDocumentType;
			char aNewDocumentName[200];
			cout << "NewDocumentType: ";
			cin >> nNewDocumentType;
			cout << "NewDocumentName: ";
			cin >> aNewDocumentName;

			doc1->Rename(nNewDocumentType,aNewDocumentName,nDocumentType,aDocumentName);
			show_status("Rename",doc1->Status());
			goto again;
		} else if (*aMode == 'o' || *aMode == 'O') {
			doc1->Open(nDocumentType,aDocumentName);
			show_status("doc1 Open",doc1->Status());
		} else if (*aMode == 'n' || *aMode == 'N') {
			doc1->OpenNew();
			show_status("doc1 OpenNew",doc1->Status());
		} else if (*aMode == 'x' || *aMode == 'X') {
			break;
		} else goto again;

		cout << "isopen: " << (int) doc1->IsOpen() << "\n";
		if (! doc1->Status().IsSuccessful()) continue;

		cout << "SchemaName: " << (const char *) doc1->SchemaName() << "\n";
		cout << "ContainerName: " << (const char *) doc1->ContainerName() << "\n";


		char aCmd[100];
keepopen:

		while (TRUE) {
			show_document("doc1",doc1);

			// Aenderungen:
			cout << "(U)pdate/(S)ave/Save(A)s/(C)lose/(F)orget: ";
			cin >> aCmd;

			if (*aCmd == 'u' || *aCmd == 'U') {
				// update data
				cout << "DocumentVersion: ";
				UINT32 n;
				cin >> n;
				doc1->SetDocumentVersion(n);
				cout << "DocumentStorage: ";
				char s[2048];
				cin.getline(s,sizeof s);
				cin.getline(s,sizeof s);
				String aString(s);

				doc1->ClearDocumentStorage();
				SvMemoryStream * pMemoryStream = doc1->DocumentStorage();
				pMemoryStream->Seek(STREAM_SEEK_TO_BEGIN);
				*pMemoryStream << aString;
				DBG_ASSERT(! pMemoryStream->GetError(),"MemoryStream konnte nicht geschrieben werden");

				goto keepopen;
			} else if (*aCmd == 'f' || *aCmd == 'F') {
				// Forget changes
				doc1->Close(TRUE);
				show_status("doc1 Close",doc1->Status());
				if (doc1->Status().IsError()) goto keepopen;
				goto again;
			} else if ((*aCmd == 's' || *aCmd == 'S' || *aCmd == 'c' || *aCmd == 'C') && (*aMode == 'n' || *aMode == 'N')
						|| (*aCmd == 'a' || *aCmd == 'A')) {
				// Save As
				cout << "DocumentType: ";
				cin >> nDocumentType;
				cout << "DocumentName: ";
				cin >> aDocumentName;

				// Save oder SaveAs
				doc1->SaveAs(nDocumentType,aDocumentName);
				show_status("doc1 SaveAs",doc1->Status());
				if (doc1->Status().IsError()) goto keepopen;

				*aMode = ' ';	// Naechstes Mal kein Save As noetig.
				if (*aCmd == 'c' || *aCmd == 'C') {
					doc1->Close(TRUE);
					show_status("doc1 Close(TRUE)",doc1->Status());
					if (doc1->Status().IsError()) goto keepopen;
					goto again;
				}
			} else if (*aCmd == 's' || *aCmd == 'S') {
				doc1->Save();
				show_status("doc1 Save",doc1->Status());
				if (doc1->Status().IsError()) goto keepopen;
			} else if (*aCmd == 'c' || *aCmd == 'C') {
				doc1->Close();
				show_status("doc1 Close",doc1->Status());
				if (doc1->Status().IsError()) goto keepopen;
				goto again;
			}
		} // inner WHILE TRUE

	} // while TRUE

	conn1->Disconnect();
	if (! conn1->Status().IsSuccessful()) show_status("conn1 Disconnect",conn1->Status());

	return 0;
}
