/*************************************************************************
 *
 *  $RCSfile: trow.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <iostream.h>
#include <assert.h>

//#include "sdb.hxx"
#include "sdbrow.hxx"

static void show_column(const char * title, const SdbColumn * c)
{
	if (c == NULL)
		cout << title << ": NULL pointer";
	else {
		cout << title << ": " << (const char *) c->Name() << ", " << (int)c->Type() << ", " << c->Length() << ", " << c->Scale() << ", " << c->AllFlags();
		if (c->IsNullAllowed()) cout << " NULL"; else cout << " NOT NULL";
		if (c->IsPrimaryKey()) cout << " PRIMARY KEY";
		if (c->IsUpdateAllowed()) cout << " (Update possible)"; else cout << " (Update impossible)";
	}
	cout << "\n";
}

static void show_columns(const char * title, const SdbColumns * cs)
{
	cout << "Columns:" << title << "\n";
	for (UINT16 i = 0; i < cs->Count(); i++) {
		show_column("    ",cs->Column(i));
	}
}

static void show_variable(const char * title, const SdbVariable * v)
{
	cout << title << ": ";
	if (v == NULL) {
							cout << "[not bound]";
	} else {
		switch(v->GetType()) {
		case SbxEMPTY:		cout << "EMPTY"; break;
		case SbxNULL:		cout << "NULL"; break;
		case SbxINTEGER:	cout << "INTEGER = " << v->GetInteger(); break;
		case SbxLONG:		cout << "LONG = " << v->GetLong(); break;
		case SbxSINGLE:		cout << "SINGLE = " << v->GetSingle(); break;
		case SbxDOUBLE:		cout << "DOUBLE = " << v->GetDouble(); break;
		case SbxBOOL:		cout << "BOOL = " << (v->GetBool() ? "TRUE" : "FALSE"); break;
		case SbxERROR:		cout << "ERROR = " << (long)v->GetError(); break;
		case SbxSTRING:		cout << "STRING = " << (const char *) v->GetString(); break;
		default: 			cout << "[unknown type]";
		}
	}
	cout << "\n";
}

static void show_row(const char * title, const SdbRow * p)
{
	cout << "Row: " << title << "\n";
	for (UINT16 i = 0; i < p->Count(); i++) {
		cout << "	Column(" << i << ") [" << (const char *) p->Column(i)->Name() << "]";
		show_variable("",p->Variable(i));
	}
}

int main() {
	SdbColumn * c1 = new SdbColumn("Test1", SDB_INTEGER, 10);
	SdbColumn * c2 = new SdbColumn("Test2", SDB_NUMERIC, 22, 1);
	SdbColumn * c3 = new SdbColumn("Test3", SDB_VARCHAR, 22);

	show_column("c1", c1);
	show_column("c2", c2);
	show_column("c3", c3);

	c1->SetFlag(SdbColumn::NullAllowed);
	show_column("c1 NullAllowed", c1);
	c1->SetFlag(SdbColumn::PrimaryKey);
	show_column("c1 PrimaryKey", c1);
	c1->SetFlag(SdbColumn::UpdateAllowed);
	show_column("c1 UpdateAllowed", c1);

	c1->ResetFlag(SdbColumn::NullAllowed);
	show_column("c1 NullAllowed", c1);
	c1->ResetFlag(SdbColumn::PrimaryKey);
	show_column("c1 PrimaryKey", c1);
	c1->ResetFlag(SdbColumn::UpdateAllowed);
	show_column("c1 UpdateAllowed", c1);

	c2->SetFlag(SdbColumn::UpdateAllowed);
	c2->SetAllFlags(SdbColumn::NullAllowed | SdbColumn::PrimaryKey);
	show_column("c2 NullAllowed and Primary Key", c2);

	cout << "-------------------------------------------------------------------\n";

	SdbColumns * cs1 = new SdbColumns();

	cs1->AddColumn(c1);
	cs1->AddColumn(c2);
	cs1->AddColumn(c3);

	show_columns("cs1",cs1);


	SdbRow * r1 = new SdbRow(cs1);
	show_row("r1 (not bound)",r1);

	SdbVariable * v0 = new SdbVariable();
	SdbVariable * v1 = new SdbVariable();
	SdbVariable * v2 = new SdbVariable();
	r1->BindVariable(0,v0);
	r1->BindVariable(1,v1);
	r1->BindVariable(2,v2);
	show_row("r1 (bound with empty variables)",r1);

	v0->PutLong(47110815);
	v1->PutInteger(1234);
	v2->PutString("dummy");
	show_row("r1 (bound with filled variables)",r1);

	show_variable("r1->Variable(\"Test3\")",r1->Variable("Test3"));
	show_variable("r1->Variable(\"Test1\")",r1->Variable("Test1"));
	show_variable("r1->Variable(\"Test2\")",r1->Variable("Test2"));
	show_variable("r1->Variable(\"Test4\")",r1->Variable("Test4"));

	show_variable("r1->Variable(2)",r1->Variable(2));
	show_variable("r1->Variable(0)",r1->Variable(0));
	show_variable("r1->Variable(1)",r1->Variable(1));
	show_variable("r1->Variable(3)",r1->Variable(3));

	cout << "-------------------------------------------------------------------\n";
	show_row("r1",r1);

	SdbVariable * v4 = new SdbVariable();
	r1->BindVariable("Test1",v4);
	r1->Variable("Test1")->PutLong(11119999);
	show_variable("r1->Variable(\"Test1\")",r1->Variable("Test1"));
	show_row("r1 (\"Test1\" rebound)",r1);

	r1->ReleaseVariable("Test1");
	show_row("r1 (\"Test1\" unbound)",r1);

	r1->ReleaseVariable(2);
	r1->ReleaseVariable(4);
	show_row("r1 (\"Test3\" unbound)",r1);

	SdbVariable * v5 = new SdbVariable();
	r1->BindVariable("Test3",v5);
	show_row("r1 (\"Test3\" bound)",r1);

	cout << "-------------------------------------------------------------------\n";
	SdbRow * r2 = new SdbRow(*r1);
	show_row("r2 (copied from r1)",r2);

	SdbVariable * vr1 = r1->Variable("Test3");
	assert(vr1);
	vr1->PutLong(1);

	show_row("r1 (\"Test3\": value changed)",r1);
	show_row("r2 (still unchanged)",r2);

	r1->ReleaseAllVariables();
	show_row("r1 (all variables released)",r1);
	show_row("r2 (still unchanged)",r2);

	cout << "-------------------------------------------------------------------\n";
	SdbRow * r3 = new SdbRow(cs1);
	*r3 = *r2;
	show_row("r3 (copied from r2)",r3);

	SdbVariable * vr2 = r2->Variable("Test3");
	assert(vr2);
	vr2->PutLong(5);

	r2->Variable("Test3")->PutLong(5);
	show_row("r2 (\"Test3\": value changed)",r2);
	show_row("r3 (still unchanged)",r3);

	r2->ReleaseAllVariables();
	show_row("r2 (all variables released)",r2);
	show_row("r3 (still unchanged)",r3);

	cout << "-------------------------------------------------------------------\n";
	if (r1) r1->ReleaseReference();
	if (r2) r2->ReleaseReference();
	if (r3) r3->ReleaseReference();
	return 0;
}
