/*************************************************************************
 *
 *  $RCSfile: tsbx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:48 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <iostream.h>

#include <tools.hxx>
#include <sbx.hxx>
//#include "sbxmstrm.hxx"

#include "dbgmsg.cxx"

//#define SHOW_VARIABLE_CONVERSIONS

#ifdef SHOW_VARIABLE_CONVERSIONS
static void show_variable(const char * title, const SbxVariable * v)
{
	cout << title << ":\n";
	cout << "\tType: ";
	switch(v->GetType()) {
		case SbxEMPTY:		cout << "EMPTY"; break;
		case SbxNULL:		cout << "NULL"; break;
		case SbxINTEGER:	cout << "INTEGER"; break;
		case SbxLONG:		cout << "LONG"; break;
		case SbxSINGLE:		cout << "SINGLE"; break;
		case SbxDOUBLE:		cout << "DOUBLE"; break;
		case SbxBOOL:		cout << "BOOL"; break;
		case SbxERROR:		cout << "ERROR"; break;
		case SbxSTRING:		cout << "STRING"; break;
		case SbxOBJECT:		cout << "OBJECT"; break;
		default: 			cout << "[unknown type]";
	}
	cout << "\tEmpty/Null: " << (v->IsEmpty() ? "IsEmpty" : "! IsEmpty") << "   " << (v->IsNull() ? "IsNull" : "! IsNull") << "\n";
	cout << "\tas Integer: " << v->GetInteger() << "\n";
	cout << "\tas Long: " << v->GetLong() << "\n";
	cout << "\tas Single: " << v->GetSingle() << "\n";
	cout << "\tas Double: " << v->GetDouble() << "\n";
	cout << "\tas Bool: " << (v->GetBool() ? "TRUE" : "FALSE") << "\n";
	cout << "\tas Error: " << (int)v->GetError() << "\n";
	cout << "\tas String: " << (const char *) v->GetString() << "\n";
	cout << "\n";
}
#else
static void show_variable(const char * title, const SbxVariable * v)
{
	cout << title << ": ";
	if (v == NULL) {
							cout << "[not bound]";
	} else {
		switch(v->GetType()) {
		case SbxEMPTY:		cout << "EMPTY"; break;
		case SbxNULL:		cout << "NULL"; break;
		case SbxINTEGER:	cout << "INTEGER = " << v->GetInteger(); break;
		case SbxLONG:		cout << "LONG = " << v->GetLong(); break;
		case SbxSINGLE:		cout << "SINGLE = " << v->GetSingle(); break;
		case SbxDOUBLE:		cout << "DOUBLE = " << v->GetDouble(); break;
		case SbxBOOL:		cout << "BOOL = " << (v->GetBool() ? "TRUE" : "FALSE"); break;
		case SbxERROR:		cout << "ERROR = " << (int)v->GetError(); break;
		case SbxSTRING:		cout << "STRING = " << (const char *) v->GetString(); break;
		case SbxOBJECT:		cout << "OBJECT"; break;
		default: 			cout << "[unknown type]";
		}
	}
	cout << "\n";
}
#endif

int
#ifdef WNT
__cdecl
#endif
main() {
	DBG_INIT();

	// Variant-Werte und automatische Konversion:
	SbxVariable * v1 = new SbxVariable;

	show_variable("v1 Not initialized",v1);

	v1->PutString("Test");
	cout << "v1 String \"Test\": " << (const char *) v1->GetString() << "\n";

	v1->PutInteger(4711);
	show_variable("v1 Integer 4711",v1);

	v1->PutLong(47110815);
	show_variable("v1 Long 47110815",v1);

	v1->PutEmpty();
	show_variable("v1 Empty",v1);

	v1->PutSingle(3.1415);
	show_variable("v1 Single 3.1415",v1);

	v1->PutDouble(3.141592653590e200);
	show_variable("v1 Double 3.141592653590e200",v1);

	v1->PutNull();
	show_variable("v1 Null",v1);

	v1->PutBool(TRUE);
	show_variable("v1 Bool TRUE",v1);

	v1->PutBool(FALSE);
	show_variable("v1 Bool FALSE",v1);

	v1->PutString("Hello world");
	show_variable("v1 String \"Hello world\"",v1);

	v1->PutString("42");
	show_variable("v1 String \"42\"",v1);

	v1->PutString("");
	show_variable("v1 String \"\"",v1);

	cout << "---------------------------------------------------------------------------\n";
	v1->PutInteger(123);
	SbxVariable * v2 = new SbxVariable(*v1);

	show_variable("v1 Integer 123",v1);
	show_variable("v2 Integer 123",v2);

	v1->PutLong(6666666);
	show_variable("v1 Long 6666666",v1);
	show_variable("v2 Integer 123",v2);

	v2->PutLong(9999);
	show_variable("v1 Long 6666666",v1);
	show_variable("v2 Long 9999",v2);

	cout << "v2 initial: " << (int)v2->GetRefCount() << "\n";
	v2->AddRef();

	v2->AddNextRef();
	cout << "v2 after AddNextRef: " << (int)v2->GetRefCount() << "\n";
	v2->ReleaseRef();
	cout << "v2 after ReleaseRef: " << (int)v2->GetRefCount() << "\n";
	v2->ReleaseRef();

// #define MAY_CRASH
#ifdef MAY_CRASH
	cout << "v2 finally: " << (int)v2->GetRefCount() << "\n";
#endif

	cout << "---------------------------------------------------------------------------\n";
	v1->PutInteger(456);
	SbxVariable * v3 = new SbxVariable();

	*v3 = *v1;

	show_variable("v1 Integer 456",v1);
	show_variable("v3 Integer 456",v3);

	v1->PutLong(555555);
	show_variable("v1 Long 555555",v1);
	show_variable("v3 Integer 456",v3);

	v3->PutLong(7777);
	show_variable("v1 Integer 555555",v1);
	show_variable("v3 Long 7777",v3);

	cout << "---------------------------------------------------------------------------\n";
	char ca1[257];
	ca1[0] = 'a';
	ca1[1] = 'b';
	ca1[2] = 'c';
	ca1[3] = '\0';
	v1->PutString(String(ca1));
	show_variable("v1 String \"abc\"",v1);

	cout << "---------------------------------------------------------------------------\n";
	SbxMemoryStreamRef pMemoryStream = new SbxMemoryStream();
	SbxVariable * vm = new SbxVariable();
	show_variable("vm EMPTY",vm);

	*pMemoryStream << String("Hello world!");

	vm->PutObject(pMemoryStream);
	if (vm->GetType() == SbxMEMORYSTREAM) {
		cout << "Type: SbxMEMORYSTREAM\n";
	}

	show_variable("vm OBJECT",vm);

	pMemoryStream = (SbxMemoryStream *)vm->GetObject();

	String aString;
	pMemoryStream->Seek(STREAM_SEEK_TO_BEGIN);
	*pMemoryStream >> aString;

	cout << "streamed (from variable): " << (const char *)aString << "\n";

	cout << "GetData: ";
	const char * pData = pMemoryStream->GetData();
	ULONG nSize = pMemoryStream->Seek(STREAM_SEEK_TO_END);
	for (ULONG i = 0; i < nSize; i++) {
		cout << '\'' << pData[i] << '\'' << ", ";
	}
	cout << "\n";


/***
	char s[100];
	cin >> s;
 ***/

	return 0;
}
