/*************************************************************************
 *
 *  $RCSfile: tutil.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "tutil.hxx"

//-----------------------------------------------------------------------------

void __EXPORT DebugPrintMsgBox( const char* pLine )
{
	String aText( pLine );
	aText += "\nAbort Program (Cancel == CoreDump) ?";

	short nRet = (short)MessageBox( 0, (LPSTR)(const char*)aText,
									"Debug Output",
									MB_SYSTEMMODAL | MB_YESNOCANCEL |
									MB_ICONEXCLAMATION | MB_DEFBUTTON2 );
	MessageBeep( MB_ICONHAND );
	switch ( nRet )
	{
		case IDYES:
			//
			exit(1);
			break;
		case IDNO:
			break;
		case IDCANCEL:
			{
				long* pTemp = 0;
				*pTemp = 0xCCCC;
			}
			break;
	}
}

//-----------------------------------------------------------------------------

void dbg_init()
{
	DbgSetPrintMsgBox( DebugPrintMsgBox );
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

void show_status(const char * title, const SdbStatus & s)
{
	cout << "\n";
	cout << title << ":\n";

	cout << "Result:            ";
	switch(s.Result()) {
		case SDB_STAT_SUCCESS:			cout << "SUCCESS\n"; return;
		case SDB_STAT_SUCCESS_WITH_INFO:	cout << "SUCCESS_WITH_INFO\n"; break;;
		case SDB_STAT_ERROR:        		cout << "ERROR\n"; break;
		case SDB_STAT_NO_DATA_FOUND:		cout << "NO_DATA_FOUND\n"; return;
		case SDB_STAT_STILL_EXECUTING:	cout << "STILL_EXECUTING\n"; return;
	}
	cout << "SQLState:          " << (const char *)s.SQLState() << "\n";
	cout << "ErrorMessage:      " << (const char *)s.ErrorMessage() << "\n";
	cout << "NativeErrorCode:	" << s.NativeErrorCode() << "\n";
	cout << "ErrorInfo:         " << (const char *)s.ErrorInfo() << "\n";
}

//-----------------------------------------------------------------------------

void show_column(const char * title, const SdbColumn * c)
{
	if (c == NULL)
		cout << title << ": NULL pointer";
	else {
		cout << title << ": " << (const char *) c->Name() << ", " << (int)c->Type() << ", " << c->Length() << ", " << c->Scale() << ", " << c->AllFlags();
		if (c->IsNullAllowed()) cout << " NULL"; else cout << " NOT NULL";
		if (c->IsPrimaryKey()) cout << " PRIMARY KEY";
		if (c->IsUpdateAllowed()) cout << " (Update possible)"; else cout << " (Update impossible)";
	}
	cout << "\n";
}

//-----------------------------------------------------------------------------

void show_columns(const char * title, const SdbColumns * cs)
{
	cout << "Columns:" << title << "\n";
	for (UINT16 i = 0; i < cs->Count(); i++) {
		show_column("    ",cs->Column(i));
	}
}

//-----------------------------------------------------------------------------

void show_variable(const char * title, const SdbVariable * v)
{
	cout << title << ": ";
	if (v == NULL) {
							cout << "[not bound]";
	} else {
		cout.precision(20);

		switch(v->GetType()) {
		case SbxEMPTY:		cout << "EMPTY"; break;
		case SbxNULL:		cout << "NULL"; break;
		case SbxOBJECT:		cout << "OBJECT"; break;
		case SbxINTEGER:	cout << "INTEGER = " << v->GetInteger(); break;
		case SbxLONG:		cout << "LONG = " << v->GetLong(); break;
		case SbxSINGLE:		cout << "SINGLE = " << v->GetSingle(); break;
		case SbxDOUBLE:		cout << "DOUBLE = " << (double)v->GetDouble(); break;
		case SbxDATE:		cout << "DATE = " << (double)v->GetDouble(); break;
		case SbxBOOL:		cout << "BOOL = " << (v->GetBool() ? "TRUE" : "FALSE"); break;
		case SbxERROR:		cout << "ERROR = " << (int)v->GetError(); break;
		case SbxSTRING:		cout << "STRING = " << (const char *) v->GetString(); break;
		case SbxMEMORYSTREAM:	cout << "MEMORYSTREAM: ";
							{
								SbxMemoryStreamRef pMemoryStream = (SbxMemoryStream *)v->GetObject();
								const char * pData = pMemoryStream->GetData();
								ULONG nSize = pMemoryStream->Seek(STREAM_SEEK_TO_END);
								cout << nSize << " bytes in stream\nBytes: ";
								for (ULONG i = 0; i < nSize; i++) {
									cout << hex << (((int) pData[i]) & 0xff);
									cout << dec << " ";
								}
							}
							break;
		default: 			cout << "[unknown type]";
		}
	}
	cout << "\n";
}

//-----------------------------------------------------------------------------

void show_row(const char * title, const SdbRow * p)
{
	cout << "Row: " << title << "\n";
	for (UINT16 i = 0; i < p->Count(); i++) {
		cout << "	Column(" << i << ") [" << (const char *) p->Column(i)->Name() << "]";
		show_variable("",p->Variable(i));
	}
}

//-----------------------------------------------------------------------------

void show_cursors(const char * title, const SdbConnection * conn1)
{
	cout << "SdbCursorList: " << title << "\n";

	for (unsigned int i = 0; i < conn1->CursorList().Count(); i++) {
		SdbCursor * curs2 = conn1->CursorList().GetObject(i);
		cout << "Cursor " << i << ": '" << (const char *)curs2->SQLStatement() << "'\n";
		cout << "          " << (curs2->IsOpen() ? "Open" : "! Open") << " " << (curs2->IsOffRange() ? "OffRange" : "! OffRange") << "\n";
	}

	cout << "\n";
}

//-----------------------------------------------------------------------------

void fetch_rows(const char * title, SdbCursor * curs1, const UINT32 nShowRows)
{
	cout << "Fetching: " << title << "\n";
	UINT32 nRows = 0;
	do {
		curs1->FetchRow();
		if (! curs1->Status().IsSuccessful()) break;

		if (! curs1->IsOffRange()) {
			nRows++;

			SdbRow * pRow = curs1->ResultRow();
			if (nRows <= nShowRows) show_row("curs1",pRow);
		}
	} while (! curs1->IsOffRange());
	show_status("Status of last Fetch",curs1->Status());
	cout << "\nFetched "<< nRows << " rows.\n\n";
}

//-----------------------------------------------------------------------------

SdbConnection * do_connect()
{
	char opt[512];
	cout << "Options: "; cin.getline(opt,sizeof opt);

	SdbEnvironment * pEnv = SdbEnvironment::GetAddress();
	DBG_ASSERT(pEnv != NULL,"Kein SdbEnvironment");

	SdbConnection * conn1 = pEnv->CreateConnection(opt);
	if (! conn1->Status().IsSuccessful()) {
		show_status("conn1 CreateConnection",conn1->Status());
		return NULL;
	}

	conn1->Connect();
	if (! conn1->Status().IsSuccessful()) {
		show_status("conn1 Connect",conn1->Status());
		return NULL;
	}

	cout << "Connected.\n\n";

	return conn1;
}

//-----------------------------------------------------------------------------

void show_rowcache(const char * title, const SdbRowCache * rc)
{
	cout << "RowCache: " << title << "\n";

	if (rc == NULL) {
		cout << "NULL pointer!\n";
		return;
	}

	if (rc->IsOffRange()) {
		cout << "OffRange: ";
		if (rc->IsOffLeft()) cout << "OffLeft ";
		if (rc->IsOffRight()) cout << "OffRight ";
	} else {
//		show_row("CurrentRow",rc->CurrentRow());
	}
	cout << "\nRowsInCache: " << rc->RowsInCache() << "\n";
	cout << "\nState:\n" << (const char *) rc->State();
}

//-----------------------------------------------------------------------------

void fetch_cached_rows(const char * title, SdbRowCache * rc1, const BOOL bShowRow)
{
	cout << "Fetching: " << title << "\n";

	if (rc1 == NULL) {
		cout << "NULL pointer!\n";
		return;
	}

	UINT32 nRows = 0;
	do {
		rc1->SeekNext();

		if (! rc1->IsOffRange()) {
			nRows++;

			SdbRow * pRow = rc1->CurrentRow();
			if (bShowRow) show_row("curs1",pRow);
		}
	} while (! rc1->IsOffRange());
//	show_status("Status of last Fetch",curs1->Status());
	cout << "\nFetched "<< nRows << " rows.\n\n";
}

