/*************************************************************************
 *
 *  $RCSfile: dbbrows.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef DBBROWS_HXX
#define DBBROWS_HXX 1

#include <browse.hxx>
#include "dbbrows.hrc"
#include <sdb.hxx>

class Browser: public BrowseBox
{
	friend class BrowseWindow;
	BOOL            bInverse;
	SdbCursor&		rCur;

protected:
	void			Fill();
	virtual long    GetRowCount() const;
	virtual BOOL    SeekRow( long nRow );
	virtual void	PaintField(OutputDevice& rDev, const Rectangle& rRect,
							   USHORT nColumnId) const;

public:
	Browser(BrowseWindow* pParent, BrowserMode eMode, SdbCursor&);
	~Browser();

	void            SetInverseSelection( BOOL bInverseSel )
					{ bInverse = bInverseSel; }
};

//------------------------------------------------------------------

class AppWindow: public WorkWindow
{
	friend class BrowseWindow;

public:
	MenuBar				aMenu;
	SdbEnvironmentRef	aEnv;
	static SdbConnection*	pConn;



private:
	DECL_LINK( OnFile, Menu * );
	DECL_LINK( OnTestTable, Menu * );

	void CreateTables();
	void DropTables();
	void FillTables();
	void ModifyRows();
	void DeleteRows();

protected:
	virtual void Activate();
	virtual BOOL Close();

public:
	AppWindow();
	~AppWindow();
};

class BrowseWindow : public MDIWindow
{
	friend class AppWindow;

	Browser         aBrowser;
	MenuBar         aBrowseMenu;
	BrowserMode     eCurMode;
	ULONG           nNewRowNo;

protected:
	DECL_LINK( OnFile, Menu * );
	DECL_LINK( OnTestTable, Menu * );

	void Init(const String& aCommand);
	~BrowseWindow();

public:

	BrowseWindow(SdbCursor& rCur);
	virtual BOOL Close() {delete this; return TRUE;}
	virtual void Activate();
	virtual void Resize();
};

class SQLDialog : public ModalDialog
{
	friend class BrowseWindow;

private:
	MultiLineEdit	aConnectEdit;
	PushButton		aConnectBtn;
	MultiLineEdit	aSQLEdit;
	PushButton		aSQLExecBtn;
	CheckBox		aConnectCB;
	FixedText		aText1,
					aText2;

	DECL_LINK(OnConnect, PushButton *);
	DECL_LINK(OnExec, PushButton *);

public:
	SQLDialog();
};

class CreateTableDialog : public ModalDialog
{
private:
	MultiLineEdit	aConnectEdit;
	PushButton		aConnectBtn;
	Edit			aTableEdit;
	PushButton		aCreateBtn, aDropBtn;
	CancelButton	aCancelBtn;
	FixedText		aText1;
	FixedText		aText2;


	DECL_LINK(OnConnect, PushButton *);
	DECL_LINK(OnDrop, PushButton *);
	DECL_LINK(OnCreate, PushButton *);

public:
	CreateTableDialog();
};

// Verbindungsaufbau
class SQLConnect : public ModalDialog
{
	friend class BrowseWindow;

private:
	MultiLineEdit	aConnectEdit;
	PushButton		aConnectBtn;
	CheckBox		aConnectCB;
	FixedText		aText1;

	DECL_LINK(OnConnect, PushButton *);

public:
	SQLConnect();
};

class SQLDatabase : public ModalDialog
{
	friend class BrowseWindow;

private:
	Edit			aDBNameEdit;
	PushButton		aCreateBtn;
	PushButton		aDropBtn;
	FixedText		aText1;

	DECL_LINK(OnCreate, PushButton *);
	DECL_LINK(OnDrop, PushButton *);

public:
	SQLDatabase(BOOL);
};



#endif
