/*************************************************************************
 *
 *  $RCSfile: dbtest.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "dbbrows.hxx"
//==================================================================

SQLConnect::SQLConnect()
		  :ModalDialog(GetpApp()->GetAppWindow(), ResId(DID_CONNECT))
		  ,aConnectEdit(this, ResId(CID_CONNECT))
		  ,aConnectBtn(this, ResId(CID_CB_CONNECT))
		  ,aConnectCB(this, ResId(CID_CX_CONNECTED))
		  ,aText1(this, ResId(10))
{
	FreeResource();

	Config aConfig("dbbrows.ini");
	aConfig.SetGroup("Last Connect");
	aConnectEdit.SetText(aConfig.ReadKey("ConnectString",""));

	aConnectBtn.SetClickHdl( LINK(this, SQLConnect, OnConnect ) );
}

//------------------------------------------------------------------
IMPL_LINK( SQLConnect, OnConnect, PushButton*, EMPTYARG )
{
	AppWindow::pConn = SdbEnvironmentRef()->CreateConnection(aConnectEdit.GetText());
	DBG_ASSERT(AppWindow::pConn != NULL,"AppWindow::pConn == NULL");
	if (!AppWindow::pConn->Status().IsSuccessful())
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Unable to Connect!").Execute();
		AppWindow::pConn->Delete();
		AppWindow::pConn = NULL;
		return TRUE;
	}
	AppWindow::pConn->Open();
	if (!AppWindow::pConn->Status().IsSuccessful())
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Unable to Connect!").Execute();
		AppWindow::pConn->Delete();
		AppWindow::pConn = NULL;
	}
	else
	{
		Config aConfig("dbbrows.ini");
		aConfig.SetGroup("Last Connect");
		aConfig.WriteKey("ConnectString",aConnectEdit.GetText());

		aConnectCB.Check();
		aConnectBtn.Disable();
	}
	return TRUE;
}

SQLDatabase::SQLDatabase(BOOL bCreate)
		  :ModalDialog(GetpApp()->GetAppWindow(), ResId(DID_CREATEDROPDB))
		  ,aDBNameEdit(this, ResId(CID_DBNAME))
		  ,aCreateBtn(this, ResId(CID_CB_CREATE))
		  ,aDropBtn(this, ResId(CID_CB_DROP))
		  ,aText1(this, ResId(10))
{
	FreeResource();
	aCreateBtn.SetClickHdl( LINK(this, SQLDatabase, OnCreate ) );
	aDropBtn.SetClickHdl( LINK(this, SQLDatabase, OnDrop ) );
	if ( bCreate )
	{
		aCreateBtn.Enable();
		aDropBtn.Disable();
	}
	else
	{
		aCreateBtn.Disable();
		aDropBtn.Enable();
	}
}

//------------------------------------------------------------------
IMPL_LINK( SQLDatabase, OnCreate, PushButton*, EMPTYARG )
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return TRUE;
	}
	String Text = aDBNameEdit.GetText();

	if (!AppWindow::pConn->CreateDatabase(Text,""))
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Unable to Create Database!").Execute();
	else
	{
		AppWindow::pConn->UseDatabase(Text);
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Database created!").Execute();
	}

	return TRUE;
}

IMPL_LINK( SQLDatabase, OnDrop, PushButton*, EMPTYARG )
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return TRUE;
	}
	AppWindow::pConn->UseDatabase("");		// aktuelle keine Datenbank verwenden, sonst klappt drop nicht
	String Text = aDBNameEdit.GetText();
	if (!AppWindow::pConn->DropDatabase(Text))
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Unable to Drop Database!").Execute();
	else
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Database dropped!").Execute();
	return TRUE;
}

//------------------------------------------------------------------
void AppWindow::CreateTables()
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return;
	}

	SdbTable* pTable = AppWindow::pConn->CreateTable();  // Erzeugen der Tabellenstruktur

	if (!pTable)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Erzeugen von Tabellen nicht moeglich!").Execute();
		return;
	}

	pTable->Create("Kunde");					// Anlegen der Tabellenstruktur unter
												// dem Tabellennamen

	SdbColumnsRef aDescription = pTable->Columns();  // Zugriff auf die Spaltenstruktur

	aDescription->AddColumn(new SdbColumn("KUNDE_ID",					// Schluessel
										  SDB_DBTYPE_INTEGER,	// typ
										  0,					// feldlnge (wird bei diesem Typen nicht unterstuetzt)
										  0));					// Nachkommastellen

	aDescription->AddColumn(new SdbColumn("Name",
										  SDB_DBTYPE_VARCHAR,
										  30,
										  0));

	aDescription->AddColumn(new SdbColumn("Vorname",
										  SDB_DBTYPE_VARCHAR,
										  20,
										  0));

	aDescription->AddColumn(new SdbColumn("Lenze",
										  SDB_DBTYPE_SMALLINT,
										  20,
										  0));

	aDescription->AddColumn(new SdbColumn("Vermoegen",
										  SDB_DBTYPE_CURRENCY,
										  15,
										  2));

	aDescription->AddColumn(new SdbColumn("HOBBY_ID",				// fremdschluessel zur Hobby-Tabelle
										  SDB_DBTYPE_SMALLINT,
										  0,
										  0));

	pTable->Update();												// schreiben auf die datenbank

	if (pTable->Status().IsError())
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Fehler beim Erzeugen der Kundentabelle!").Execute();
		pTable->Delete();
		return;
	}

	SdbIndex* pIndex = pTable->CreateIndex(); // Index auf Kunde anlegen
	if (!pIndex)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Fehler beim Erzeugen der Indizes!").Execute();
		pTable->Delete();
		return;
	}

	pIndex->Create("PKey");					// Key anlegen fr Kunde
	pIndex->Columns()->AddColumn(new SdbColumn(*pTable->OriginalColumns()["KUNDE_ID"]));
	pIndex->SetStyle(SDB_IDX_UNIQUE);		// PrimaryKey kann Access nicht, da es nicht NOT NULL kennt
											// Daher nur UNIQUE
	pIndex->Update();					    // Index auf die Datenbank schreiben

	if (pIndex->Status().IsError())
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Fehler beim Erzeugen der Indizes!").Execute();
		pIndex->Delete();
		pTable->Delete();
		return;
	}

	pIndex->Delete();
	pTable->Close();		// Struktur schlieen

	pTable->Create("Hobby");					// Anlegen der Tabellenstruktur unter
												// dem Tabellennamen

	aDescription = pTable->Columns();  // Zugriff auf die Spaltenstruktur

	aDescription->AddColumn(new SdbColumn("HOBBY_ID",					// Schluessel
										  SDB_DBTYPE_SMALLINT,	// typ
										  0,					// feldlnge (wird bei diesem Typen nicht unterstuetzt)
										  0));					// Nachkommastellen

	aDescription->AddColumn(new SdbColumn("Bezeichnung",
										  SDB_DBTYPE_VARCHAR,
										  30,
										  0));
	pTable->Update();												// schreiben auf die datenbank

	if (pTable->Status().IsError())
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Fehler beim Erzeugen der Kundentabelle!").Execute();
		pTable->Delete();
		return;
	}

	pIndex = pTable->CreateIndex(); // Index auf Kunde anlegen
	if (!pIndex)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Fehler beim Erzeugen der Indizes!").Execute();
		pTable->Delete();
		return;
	}

	pIndex->Create("PKey");					// Key anlegen fr Kunde
	pIndex->Columns()->AddColumn(new SdbColumn(*pTable->OriginalColumns()["HOBBY_ID"]));
	pIndex->SetStyle(SDB_IDX_UNIQUE);		// PrimaryKey kann Access nicht, da es nicht NOT NULL kennt
											// Daher nur UNIQUE
	pIndex->Update();					    // Index auf die Datenbank schreiben

	if (pIndex->Status().IsError())
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Fehler beim Erzeugen der Indizes!").Execute();
	}

	pIndex->Delete();
	pTable->Delete();

	InfoBox(this,"Tabellen (Hobby, Kunde) erfolgreich erstellt!").Execute();
}

void AppWindow::DropTables()
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return;
	}

	SdbTable* pTable = AppWindow::pConn->CreateTable();  // Erzeugen der Tabellenstruktur

	if (!pTable)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Loeschen von Tabellen nicht moeglich!").Execute();
		return;
	}

	if (!pTable->Open("Kunde"))					// Anlegen der Tabellenstruktur unter
												// dem Tabellennamen
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde nicht gefunden!").Execute();

	else if (!pTable->Drop())
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht geloescht werden!").Execute();
	else
	{
		pTable->Close();
		InfoBox(this, "Tabelle Kunde ist geloescht worden!").Execute();
	}

	if (!pTable->Open("Hobby"))					// Anlegen der Tabellenstruktur unter
												// dem Tabellennamen
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Hobby nicht gefunden!").Execute();

	else if (!pTable->Drop())
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Hobby kann nicht geloescht werden!").Execute();
	else
		InfoBox(this, "Tabelle Hobby ist geloescht worden!").Execute();

	pTable->Delete();
}

#define MAINSELECT "SELECT Kunde.KUNDE_ID, Kunde.Name, Kunde.Vorname, Kunde.Lenze, Kunde.Vermoegen, \
					Kunde.HOBBY_ID, Hobby.Bezeichnung \
					FROM {oj Kunde Kunde LEFT OUTER JOIN Hobby Hobby \
					ON Kunde.HOBBY_ID = Hobby.HOBBY_ID}"


void AppWindow::FillTables()
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return;
	}

	SdbCursor* pCursor = AppWindow::pConn->CreateCursor(SDB_TABLE,SDB_KEYSET,0);  // Cursor anlegen

	if (!pCursor || !pCursor->Open("Kunde"))
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gelesen werden!").Execute();
		if (pCursor) pCursor->Delete();
		return;
	}

	// Hilfskonstrukt
	SdbCursor& rCur = *pCursor;



	if (!rCur.AppendRow())			// Anlegen einer Zeile
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	// nun fuellen
	rCur["KUNDE_ID"]->PutLong(1);
	rCur["Name"]->PutString("Meyer");
	rCur["Vorname"]->PutString("Andreas");
	rCur["Lenze"]->PutInteger(100);
	rCur["Vermoegen"]->PutInteger(100);


	if (!rCur.UpdateRow())			// Schreiben auf die Datenbank
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}
	else
	{
		DBG_TRACE("Bookmark der zuletzt eingefgten Zeile: ");
		DBG_TRACE(String(rCur.LastModified()));
		DBG_TRACE("\n");

		;
	}

	if (!rCur.AppendRow())			// Anlegen einer Zeile
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	rCur["KUNDE_ID"]->PutLong(2);
	rCur["Name"]->PutString("Pohl");
	rCur["Vorname"]->PutString("Willi");
	rCur["Lenze"]->PutInteger(100);
	rCur["Vermoegen"]->PutInteger(100);

	if (!rCur.UpdateRow())			// Schreiben auf die Datenbank
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	if (!rCur.AppendRow())			// Anlegen einer Zeile
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	rCur["KUNDE_ID"]->PutLong(3);
	rCur["Name"]->PutString("Mustermann");
	rCur["Vorname"]->PutString("Heinz");
	rCur["Lenze"]->PutInteger(50);
	rCur["Vermoegen"]->PutInteger(10000);

	if (!rCur.UpdateRow())			// Schreiben auf die Datenbank
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	rCur.Close();
	rCur.Open("Hobby");

	if (!rCur.AppendRow())			// Anlegen einer Zeile
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Hobby kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	// nun fuellen
	rCur["HOBBY_ID"]->PutInteger(1);
	rCur["Bezeichnung"]->PutString("StarDivision");

	if (!rCur.UpdateRow())			// Schreiben auf die Datenbank
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Hobby kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	if (!rCur.AppendRow())			// Anlegen einer Zeile
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Hobby kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	// nun fuellen
	rCur["HOBBY_ID"]->PutInteger(2);
	rCur["Bezeichnung"]->PutString("Bier trinken");

	if (!rCur.UpdateRow())			// Schreiben auf die Datenbank
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Hobby kann nicht gefuellt werden!").Execute();
		rCur.Delete();
		return;
	}

	rCur.Delete();

	//neuen Cursor fuer die Anzeige, gecached
	pCursor = AppWindow::pConn->CreateCursor();  // Cursor anlegen

	pCursor->SetCacheSize(50);
	if (pCursor->Open(MAINSELECT))
	{
		BrowseWindow* pBrowseWin = new BrowseWindow(*pCursor);
		if (pBrowseWin)
			pBrowseWin->Show();
		else
			pCursor->Delete();
	}
	else
		pCursor->Delete();
}

void AppWindow::ModifyRows()
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return;
	}

	SdbCursor* pCursor = AppWindow::pConn->CreateCursor(SDB_TABLE, SDB_KEYSET, 0);  // Cursor anlegen

	if (!pCursor || !pCursor->Open("Kunde"))
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gelesen werden!").Execute();
		if (pCursor) pCursor->Delete();
		return;
	}

	// Hilfskonstrukt
	SdbCursor& rCur = *pCursor;

	INT16 i = 1;
	while (rCur.IsInRange())
	{
		rCur.EditRow();
		rCur["HOBBY_ID"]->PutLong(i++ % 2 + 1);
		rCur.UpdateRow();
		rCur.Next();
	}

	rCur.Delete();

	//neuen Cursor fuer die Anzeige, gecached
	pCursor = AppWindow::pConn->CreateCursor();  // Cursor anlegen
	pCursor->SetCacheSize(50);

	if (pCursor->Open(MAINSELECT))
	{
		BrowseWindow* pBrowseWin = new BrowseWindow(*pCursor);
		if (pBrowseWin)
			pBrowseWin->Show();
		else
			pCursor->Delete();
	}
	else
		pCursor->Delete();
}

void AppWindow::DeleteRows()
{
	if (!AppWindow::pConn)
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Keine Verbindung aufgebaut").Execute();
		return;
	}

	SdbCursor* pCursor = AppWindow::pConn->CreateCursor(SDB_STATEMENT, SDB_KEYSET, SDB_FORWARDONLY);  // Cursor anlegen

	if (!pCursor || !pCursor->Open("Select * from Kunde"))
	{
		ErrorBox(this, WinBits(WB_OK|RET_OK), "Tabelle Kunde kann nicht gelesen werden!").Execute();
		if (pCursor) pCursor->Delete();
		return;
	}

	// Hilfskonstrukt
	SdbCursor& rCur = *pCursor;

	while (rCur.IsInRange())
	{
		rCur.DeleteRow();
		rCur.Next();
	}

	rCur.Close();
	rCur.Open("Select * from Hobby");
	while (rCur.IsInRange())
	{
		rCur.DeleteRow();
		rCur.Next();
	}
	rCur.Delete();

}
