/*************************************************************************
 *
 *  $RCSfile: sdbtest.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <sv.hxx>
#include "sdbtest.hrc"

#include "test1.hxx"
#include "test2.hxx"

static String	aEmptyString;

// -----------------------------------------------------------------------

class MainWin : public WorkWindow
{
	USHORT nCurrentMenuId;

protected:
	MultiLineEdit	aCmdEdit;
	MultiLineEdit	aResEdit;
	PushButton		aExecButton;
	PushButton		aClearButton;

	SdbTestCommand * pTestCommand;

	DECL_LINK( ButtonHdl, Button * );
	DECL_LINK( MenuSelectHdl, Menu * );



public:
	MainWin( Window * pParent );
	~MainWin();
	virtual void Resize();
	virtual void GetFocus();

	MenuBar			aMenuBar;
};


MainWin::MainWin( Window * pParent )
	:
	nCurrentMenuId(0),
	WorkWindow( pParent, ResId(WIN_MAIN) ),
	aCmdEdit( this, ResId( ED_COMMAND ) ),
	aResEdit( this, ResId( ED_RESULT ) ),
	aExecButton( this, ResId( PB_EXECUTE ) ),
	aClearButton( this, ResId( PB_CLEAR ) ),
	aMenuBar( ResId( MNU_MAIN ) ),
	pTestCommand(NULL)
{
	SetIcon( Icon(ResId(MAIN_ICO)) );

	aCmdEdit.SetText( aEmptyString );
	aResEdit.SetText( aEmptyString );
	aExecButton.SetClickHdl( LINK(this,MainWin,ButtonHdl) );
	aClearButton.SetClickHdl( LINK(this,MainWin,ButtonHdl) );

	aMenuBar.GetPopupMenu( MPOP_FILE )->
		PushSelectHdl( LINK( this, MainWin, MenuSelectHdl ) );

	aMenuBar.GetPopupMenu( MPOP_TEST )->
		PushSelectHdl( LINK( this, MainWin, MenuSelectHdl ) );

	FreeResource();
}

MainWin::~MainWin()
{
	delete pTestCommand;
}

// -----------------------------------------------------------------------

IMPL_LINK( MainWin, MenuSelectHdl, Menu *, pMenu )
{
	USHORT		nId = pMenu->GetCurItemId();

	// Diverse Menueeintraege (ohne "Check")
	switch(nId)
	{
		case MITM_EXIT: 	pApp->Quit();
							return 0;
		case MITM_EXECUTE:	ButtonHdl(&aExecButton);
							return 0;
		case MITM_CLEAR:	//ButtonHdl(&aClearButton);
					aCmdEdit.SetText("");
							return 0;
	}

	// Menueeintraege aus dem "Test"-Menue (mit "Check"):

	if (nCurrentMenuId != 0) {
		pMenu->CheckItem(nCurrentMenuId,FALSE);
	}
	pMenu->CheckItem(nId,TRUE);
	nCurrentMenuId = nId;

	// Altes SdbTestCommand loswerden ...
	delete pTestCommand;

	// Fenster aufraeumen
	//aCmdEdit.SetText(aEmptyString);
	//aResEdit.SetText(aEmptyString);

	// ... und neues erzeugen.
	switch(nId)
	{
		case MITM_PARSE:	{
			pTestCommand = new SdbTest1(aResEdit,aCmdEdit.GetText());
			pTestCommand->Write("Test: Parse\n");
		} break;
		case MITM_SQL:	{
			pTestCommand = new SdbTest2(aResEdit,aCmdEdit.GetText());
			pTestCommand->Write("Test: SQL\n");
		} break;
	}
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( MainWin, ButtonHdl, Button *, pButton )
{
	if( pButton == &aExecButton )
	{
		if (pTestCommand != NULL) {

			// Kommando ins Ausgabefenster uebernehmen:
			pTestCommand->Write("------------------------------------------------------------------------------\n");
			pTestCommand->Write(aCmdEdit.GetText());
			pTestCommand->Write("\n");

			pTestCommand->Execute(aCmdEdit.GetText());
		} else {
			ErrorBox aErrorBox(this,WinBits(WB_OK),"No test selected");
			aErrorBox.Execute();
		}
	} else if( pButton == &aClearButton ) {
		aCmdEdit.SetText("");
		aResEdit.SetText("");
	}
	aCmdEdit.GrabFocus();
	return 0;
}

// -----------------------------------------------------------------------

void MainWin::Resize()
{
	Size	aSize( GetOutputSizePixel() );
	Size	aTmpSize( aSize.Width(),(aSize.Height()-aExecButton.GetSizePixel().Height())/2 );
	Point	aPos;

	aCmdEdit.SetPosSizePixel( aPos, aTmpSize );
	aPos.Y() += aTmpSize.Height();
	aResEdit.SetPosSizePixel( aPos, aTmpSize );

	aPos.Y() += aTmpSize.Height();
	aPos.Y() += 1;
	aExecButton.SetPosPixel( aPos );

	aPos.X() += aExecButton.GetSizePixel().Width();
	aPos.X() += 1;
	aClearButton.SetPosPixel( aPos );
}

// -----------------------------------------------------------------------

void MainWin::GetFocus()
{
	aCmdEdit.GrabFocus();
}

// -----------------------------------------------------------------------

class MyApp : public Application
{
public:
	void Main( int, char* [] );
};

// -----------------------------------------------------------------------

MyApp aMyApp;

// -----------------------------------------------------------------------

void MyApp::Main( int, char* [] )
{
	EnableSVLook();
	MainWin aMainWin( NULL );

	aMainWin.Show();
	aMainWin.Resize();

	pApp->SetAppMenu( &aMainWin.aMenuBar );

	Execute();
}
