/*************************************************************************
 *
 *  $RCSfile: sdbtestc.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "sdbtestc.hxx"

static String aEmptyString;

SdbTestCommand::SdbTestCommand(Edit & rNewOutputWindow, const String & rCommand) :
	rOutputWindow(rNewOutputWindow)
{
	Clear();
}

void SdbTestCommand::Clear()
{
	rOutputWindow.SetText(aEmptyString);
}

void SdbTestCommand::Write(const String & rString)
{
	String aTmpString(rOutputWindow.GetText());
	aTmpString += rString;
	aTmpString.ConvertLineEnd();
	rOutputWindow.SetText(aTmpString);
	rOutputWindow.SetSelection(Selection(aTmpString.Len()));
}

void SdbTestCommand::show_status(const char * title, const SdbStatus & s)
{
	Write("\n"); Write(title); Write(":\n");

	Write("Result:            ");
	switch(s.Result()) {
		case SDB_STAT_SUCCESS:				Write("SUCCESS\n"); return;
		case SDB_STAT_SUCCESS_WITH_INFO:	Write("SUCCESS_WITH_INFO\n"); break;;
		case SDB_STAT_ERROR:        		Write("ERROR\n"); break;
		case SDB_STAT_NO_DATA_FOUND:		Write("NO_DATA_FOUND\n"); return;
		case SDB_STAT_STILL_EXECUTING:		Write("STILL_EXECUTING\n"); return;
	}
	Write("SQLState:          "); Write(s.SQLState()); Write("\n");
	Write("ErrorMessage:      "); Write(s.ErrorMessage()); Write("\n");
	Write("NativeErrorCode:	  "); Write(s.NativeErrorCode()); Write("\n");
	Write("ErrorInfo:         "); Write(s.ErrorInfo()); Write("\n");
}

void SdbTestCommand::show_column(const char * title, const SdbColumn * c)
{
	if (c == NULL) {
		Write(title); Write(": NULL pointer");
	} else {
		Write(title); Write(": "); Write(c->Name()); Write(", "); Write((UINT32)c->Type()); Write(", "); Write((UINT32)c->Length()); Write(", "); Write((UINT32)c->Scale()); Write(", "); Write((UINT32)c->AllFlags());
		if (c->IsNullAllowed()) Write(" NULL"); else Write(" NOT NULL");
		if (c->IsPrimaryKey()) Write(" PRIMARY KEY");
		if (c->IsUpdateAllowed()) Write(" (Update possible)"); else Write(" (Update impossible)");
	}
	Write("\n");
}

void SdbTestCommand::show_columns(const char * title, const SdbColumns * cs)
{
	Write("Columns:"); Write(title); Write("\n");
	for (UINT16 i = 0; i < cs->Count(); i++) {
		show_column("    ",cs->Column(i));
	}
}

void SdbTestCommand::show_variable(const char * title, const SdbVariable * v)
{
	Write(title); Write(": ");
	if (v == NULL) {
							Write("[not bound]");
	} else {
		switch(v->GetType()) {
		case SbxEMPTY:		Write( "EMPTY"); break;
		case SbxNULL:		Write( "NULL"); break;
		case SbxOBJECT:		Write( "OBJECT"); break;
		case SbxINTEGER:	Write( "INTEGER = "); Write((INT32)v->GetInteger()); break;
		case SbxLONG:		Write( "LONG = "); Write((INT32)v->GetLong()); break;
		case SbxSINGLE:		Write( "SINGLE = "); Write("single ..."); break;
		case SbxDOUBLE:		Write( "DOUBLE = "); Write("double ..."); break;
		case SbxBOOL:		Write( "BOOL = "); Write((INT32)(v->GetBool() ? "TRUE" : "FALSE")); break;
		case SbxERROR:		Write( "ERROR = "); Write((INT32)v->GetError()); break;
		case SbxSTRING:		Write( "STRING = "); Write(v->GetString()); break;
		case SbxMEMORYSTREAM:	Write( "MEMORYSTREAM: ");
							{
								SbxMemoryStreamRef pMemoryStream = (SbxMemoryStream *)v->GetObject();
								Write( (INT32)pMemoryStream->Seek(STREAM_SEEK_TO_END)); Write(" bytes in stream");
							}
							break;
		default: 			Write( "[unknown type]");
		}
	}
	Write( "\n");
}

void SdbTestCommand::show_row(const char * title, const SdbRowRef& aRow)
{
	Write( "Row: "); Write(title); Write("\n");
	for (UINT16 i = 0; i < aRow->Count(); i++) {
		Write( "	Column("); Write((UINT32)i); Write(") ["); Write(aRow->Column(i)->Name()); Write("]");
		show_variable("",aRow->Variable(i));
	}
}

void SdbTestCommand::show_cursors(const char * title, const SdbConnection * conn1)
{
	Write( "SdbCursorList: "); Write(title); Write("\n");

	Write( "\n");
}

void SdbTestCommand::fetch_rows(const char * title, SdbCursor * curs1, const UINT32 nShowRows)
{
	Write( "Fetching: "); Write(title); Write("\n");
	UINT32 nRows = 0;
	do {
		curs1->Next();
		if (! curs1->Status().IsSuccessful()) break;

		if (! curs1->IsOffRange()) {
			nRows++;

			SdbRowRef aRow = curs1->ResultRow();
			if (nRows <= nShowRows) show_row("curs1",aRow);
		}
	} while (! curs1->IsOffRange());
	show_status("Status of last Fetch",curs1->Status());
	Write( "\nFetched "); Write(nRows); Write(" rows.\n\n");
}
