/*************************************************************************
 *
 *  $RCSfile: test2.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "test2.hxx"
#include "sdb.hxx"

static SdbConnection * pConn;


IMPL_LINK_INLINE_START( SdbTest2, BeginFunc, void *, EMPTYARG )
{
	GetpApp()->Wait(TRUE);
	bBegin = TRUE;
	return 0;
}
IMPL_LINK_INLINE_END( SdbTest2, BeginFunc, void *, EMPTYARG )

IMPL_LINK_INLINE_START( SdbTest2, EndFunc, void *, EMPTYARG )
{
	GetpApp()->Wait(FALSE);
	return 0;
}
IMPL_LINK_INLINE_END( SdbTest2, EndFunc, void *, EMPTYARG )

IMPL_LINK( SdbTest2, BusyFunc, void *, EMPTYARG )
{
	if (bBegin) {
		// Einmal Bescheid sagen, dass wir BusyFunc gerufen haben
		Write("BusyFunc called ...\n");
		bBegin = FALSE;
	}

	GetpApp()->Reschedule();
	return 0;
}

SdbTest2::SdbTest2(Edit & r, const String & rCommand) : SdbTestCommand(r,rCommand)
{
			pConn = SdbEnvironmentRef()->CreateConnection(rCommand);
//			pConn = pEnv->CreateConnection(SDB_SERVER_ODBC,"sample","admin","school",String());
//			pConn = pEnv->CreateConnection(SDB_SERVER_ODBC,"sample","admin","school");
//			pConn = pEnv->CreateConnection(SDB_SERVER_ODBC,"sample","admin","school",SdbEnvironment::EmptyString());
			DBG_ASSERT(pConn != NULL,"pConn == NULL");
			if (! pConn->Status().IsSuccessful()) {
				show_status("CreateConnection",pConn->Status());
				pConn->Delete();
				pConn = NULL;
				return;
			}

			// Callbacks aufsetzen ...
			pConn->SetBeginHdl(LINK(this,SdbTest2,BeginFunc));
			pConn->SetEndHdl(LINK(this,SdbTest2,EndFunc));
			pConn->SetBusyHdl(LINK(this,SdbTest2,BusyFunc));

			Write("Connecting...\n");
			pConn->Open();
			if (! pConn->Status().IsSuccessful()) {
				show_status("Connect",pConn->Status());
				pConn->Delete();
				pConn = NULL;
				return;
			}
			Write("Connected.\n");


#define SHOW_TABLES
#ifdef SHOW_TABLES
	Write("TableNames: ");
	String aName;
	for (BOOL bFirst = TRUE; pConn->GetTableName(aName, bFirst, SDB_TABTYP_TABLE, String()); bFirst = FALSE) {
		Write("Table: "); Write(aName); Write("\n");
	}
	Write("\n");
#endif

}

SdbTest2::~SdbTest2()
{
			if (pConn != NULL) {
				pConn->Close();
				if (! pConn->Status().IsSuccessful()) {
					show_status("Disconnect",pConn->Status());
					pConn->Delete();
					pConn = NULL;
					return;
				}
				pConn->Delete();
				pConn = NULL;
				Write("Disconnected.\n");
			}
}

void SdbTest2::Execute(const String & rCommand)
{
		SdbCursor * pCurs = pConn->CreateCursor();
		DBG_ASSERT(pCurs != NULL,"pCurs == NULL");
		if (! pConn->Status().IsSuccessful() || ! pCurs->Status().IsSuccessful()) {
			show_status("CreateCursor",pConn->Status());
			return;
		}

		pCurs->Prepare(rCommand);
		if (! pCurs->Status().IsSuccessful()) {
			show_status("Prepare",pCurs->Status());
			pCurs->Delete();
			return;
		}
		Write("Statement prepared.\n");

/*		show_columns("Parameter columns",&pCurs->ParameterColumns());
		if (! pCurs->Status().IsSuccessful()) {
			show_status("show_columns",pCurs->Status());
			pCurs->Delete();
			return;
		}
		show_columns("Result columns",&pCurs->ResultColumns());
		if (! pCurs->Status().IsSuccessful()) {
			show_status("show_columns",pCurs->Status());
			pCurs->Delete();
			return;
		}*/

		pCurs->Open();
		if (! pCurs->Status().IsSuccessful()) {
			show_status("Execute",pCurs->Status());
			pCurs->Delete();
			return;
		}
		Write("Statement executed. RowCount: ");
		Write(String(pCurs->RowCount()));
		Write("\n");

		fetch_rows("Fetch all (and show first 3 rows)",pCurs,3);
		if (! pCurs->Status().IsSuccessful()) {
			show_status("fetch_rows",pCurs->Status());
			pCurs->Delete();
			return;
		}

		pCurs->Delete();
		if (! pCurs->Status().IsSuccessful()) {
			show_status("Delete cursor",pCurs->Status());
			return;
		}
		Write("Cursor closed.\n");

}
