/*************************************************************************
 *
 *  $RCSfile: testbase.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:49:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/*
Erweiterungen des Testprogramms:

1. Alle Operationen sollen Zeitmessungen unterzogen werden koennen.
2. Der Testlauf sollte in irgendeiner Form steuerbar sein.
3. Es gibt fuenf verschiedene feste Tabellenstrukturen:
	a) eine Tabelle mit wenigen Feldern (String, Number, Date, Logical)
	b) eine Tabelle mit Memo-/Binaerfelder(n)
	c) eine Tabelle mit vielen kleinen Feldern
	d) eine Tabelle mit vielen groessen Feldern
	e) eine Tabelle mit wenigen grossen Feldern
4. Es gibt fuer jede Tabelle Indizes:
	a) Index auf Stringfeld (unique)
	b) Index auf Stringfeld (non-unique)
	c) Index auf Numberfeld (unique)
	d) Index auf Numberfeld (non-unique)
	e) Index auf Datumsfeld (unique)
	f) Index auf Datumsfeld (non-unique)
	g) Index auf mehrere Felder (unique)
	h) Index auf mehrere Felder (non-unique)
5. Folgende Operationen sind zu testen:
	a) Datenbank anlegen
	b) Datenbank loeschen
	c) Datenbank oeffnen
	d) Datenbank schliessen
	e) Tabelle anlegen
	f) Tabelle loeschen
	g) Tabelle oeffnen
	h) Tabelle schliessen
	i) Index anlegen
	j*) Index loeschen
	k) Index oeffnen
	l) Index schliessen
	m) Tag auswaehlen
	n) Datensaetze einfuegen ( Anzahl frei waehlbar )
	o) Datensaetze loeschen ( Anzahl und Position frei waehlbar )
	p*) Datensaetze aendern ( Anzahl und Position frei waehlbar )
	q) Datensaetze auslesen ( Tabelle frei waehlbar )

  * ist noch nicht implementiert
*/

#define _TESTBASE_CXX "$Revision: 1.1.1.1 $"

#define TESTBASE_RECSIZE			CNTSTORE_RECORDSIZE_1K

#ifndef _CNTSTOR_HXX
#include <chaos/cntstor.hxx>
#endif

#ifndef _GLOBNAME_HXX
#include <tools/globname.hxx>
#endif

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif

#ifndef _BASESTOR_HXX
#include "..\..\source\inc\basestor.hxx"
#endif

#ifndef _TIME_HXX
#include <tools/time.hxx>
#endif

#include <stdio.h>

/*
 * Globale Variablen
 */

SdbBASEStorage aStorage;		// Die Datenbank
String* pDatabaseName = NULL;	// Das Datenbankfile
String* pTableKey = NULL;		// Der Schluessel der Tabelle
String* pIndexKey = NULL;		// Der Schluessel des Index
String* pTagName = NULL;		// Der Name des Tags
unsigned nNumFields = 5;		// Die Anzahl der Felder einer Tabelle
unsigned nType = 1;				// Der Typ der Tabelle bzw. Index
unsigned nSetCount = 1;			// Anzahl der Stze, die eingefuegt bzw. geloescht werden sollen
unsigned nSetPosition = 1;		// Position ab der Saetze geloescht werden sollen
BOOL bAscending = TRUE;			// Richtung der Iteration beim Lesen einer Tabelle
BOOL bProfiling;				// Flag, ob Zeitmessungen vorgenommen werden sollen

enum ParseCodes
{
	PARSE_NOTHING=0,
	PARSE_CREATE=1,
	PARSE_DROP=2,
	PARSE_OPEN=4,
	PARSE_CLOSE=8,
	PARSE_INSERT=16,
	PARSE_DELETE=32,
	PARSE_SELECT=64,
	PARSE_DATABASE=128,
	PARSE_TABLE=256,
	PARSE_INDEX=512,
	PARSE_FROM=1024,
	PARSE_TAG=2048,
	PARSE_ALL=4096
};

enum ParseErrors
{
	NO_ERROR,
	SYNTAX_ERROR,
	ILLEGAL_ACTION,
	MISPLACED_ACTION,
	ILLEGAL_OBJECT,
	MISPLACED_OBJECT,
	ILLEGAL_TOKEN,
	MISPLACED_TOKEN,
	ILLEGAL_PARAMETER
};

/*
 * Hilfsfunktionen
 */

//------------------------------------------------------------------
void BuildTableDef( FIELD4INFO*& fields, const int nColumns, const int nChoice )
{
	fields = new FIELD4INFO[nColumns+1];

	if ( nChoice==1)
	{
		FIELD4INFO xTemp[] =
			{
				{ "NAME",     r4str, 30, 0, 1 },
				{ "VORNAME",  r4str, 20, 0, 1 },
				{ "GEBDAT",   r4date,  8, 0, 1 },
				{ "GROESSE",  r4num,   4, 1, 1 },
				{ "MANNLICH", r4log,   1, 0, 1 },
				{ 0, 0, 0, 0, 0 }
			};
		memcpy( fields, xTemp, sizeof(FIELD4INFO) * (nColumns+1) );
	}
	else if ( nChoice==2)
	{
		FIELD4INFO xTemp[] =
			{
				{ "NAME",     r4str, 30, 0, 1 },
				{ "VORNAME",  r4str, 20, 0, 1 },
				{ "GEBDAT",   r4date,  8, 0, 1 },
				{ "GROESSE",  r4num,   4, 1, 1 },
				{ "MANNLICH", r4log,   1, 0, 1 },
				{ "BEMERK",   r4memo, 10, 0, 1 },
				{ "RSA_KEY",  r4memoBin,  10, 0, 0 },
				{ 0, 0, 0, 0, 0 }
			};
		memcpy( fields, xTemp, sizeof(FIELD4INFO) * (nColumns+1) );
	}
	else if ( nChoice==3 )
	{
		FIELD4INFO xTemp[] =
			{
				{ "FELD01",  r4str,  5, 0, 1 },
				{ "FELD02",  r4str,  6, 0, 1 },
				{ "FELD03",  r4str,  2, 0, 1 },
				{ "FELD04",  r4str,  5, 0, 0 },
				{ "FELD05",  r4str,  7, 0, 0 },
				{ "NAME",    r4str,  5, 0, 1 },
				{ "FELD07",  r4str,  5, 0, 0 },
				{ "FELD08",  r4str,  5, 0, 0 },
				{ "FELD09",  r4str,  5, 0, 0 },
				{ "VORNAME", r4str,  5, 0, 0 },
				{ "FELD11",  r4float, 5, 1, 0 },
				{ "FELD12",  r4float, 7, 3, 0 },
				{ "FELD13",  r4float, 4, 2, 0 },
				{ "FELD14",  r4num,   2, 0, 0 },
				{ "FELD15",  r4num,   4, 1, 1 },
				{ "FELD16",  r4num,   6, 0, 0 },
				{ "FELD17",  r4num,   5, 0, 0 },
				{ "GROESSE", r4num,   8, 6, 1 },
				{ "FELD19",  r4num,   3, 1, 1 },
				{ "FELD20",  r4num,   2, 0, 0 },
				{ "FELD21",  r4num,   5, 1, 0 },
				{ "FELD22",  r4log,   1, 0, 0 },
				{ "FELD23",  r4log,   1, 0, 0 },
				{ "FELD24",  r4log,   1, 0, 0 },
				{ "FELD25",  r4log,   1, 0, 0 },
				{ "GEBDAT",  r4date,  8, 0, 1 },
				{ 0, 0, 0, 0, 0 }
			};
		memcpy( fields, xTemp, sizeof(FIELD4INFO) * (nColumns+1) );
	}
	else if ( nChoice==4)
	{
		FIELD4INFO xTemp[] =
			{
				{ "FELD01",  r4str, 100,  0, 1 },
				{ "NAME",    r4str,  60,  0, 1 },
				{ "FELD03",  r4str, 253,  0, 1 },
				{ "FELD04",  r4str, 189,  0, 0 },
				{ "FELD05",  r4str, 127,  0, 0 },
				{ "FELD06",  r4str,  85,  0, 1 },
				{ "VORNAME", r4str,  99,  0, 0 },
				{ "FELD08",  r4str, 153,  0, 0 },
				{ "FELD09",  r4str,  77,  0, 0 },
				{ "FELD10",  r4str,  95,  0, 0 },
				{ "FELD11",  r4float,  5,  1, 0 },
				{ "FELD12",  r4float,  7,  3, 0 },
				{ "FELD13",  r4float,  4,  2, 0 },
				{ "FELD14",  r4num,   19,  0, 0 },
				{ "FELD15",  r4num,   12,  8, 1 },
				{ "FELD16",  r4num,    9,  2, 0 },
				{ "FELD17",  r4num,    7,  5, 0 },
				{ "GROESSE", r4num,   13, 10, 1 },
				{ "FELD19",  r4num,   18, 12, 1 },
				{ "FELD20",  r4num,   16,  5, 0 },
				{ "FELD21",  r4num,   15,  9, 0 },
				{ "FELD22",  r4memo,  10,  0, 0 },
				{ "FELD23",  r4memo,  10,  0, 0 },
				{ "FELD24",  r4memo,  10,  0, 0 },
				{ "FELD25",  r4memo,  10,  0, 0 },
				{ "GEBDAT",  r4date,   8,  0, 1 },
				{ 0, 0, 0, 0, 0 }
			};
		memcpy( fields, xTemp, sizeof(FIELD4INFO) * (nColumns+1) );
	}
	else if ( nChoice==5)
	{
		FIELD4INFO xTemp[] =
			{
				{ "FELD01",  r4str, 100,  0, 1 },
				{ "NAME",    r4str, 253,  0, 1 },
				{ "FELD03",  r4str, 189,  0, 0 },
				{ "VORNAME", r4str,  95,  0, 0 },
				{ "FELD05",  r4num,   19,  0, 0 },
				{ "GROESSE", r4num,   13, 10, 1 },
				{ "FELD07",  r4num,   15,  9, 0 },
				{ "FELD08",  r4memo,  10,  0, 0 },
				{ "GEBDAT",  r4date,   8,  0, 1 },
				{ 0, 0, 0, 0, 0 }
			};
		memcpy( fields, xTemp, sizeof(FIELD4INFO) * (nColumns+1) );
	}
}

void BuildIndexDef( TAG4INFO*& tags, const int nChoice )
{
	tags = new TAG4INFO[2];

	if ( nChoice==1)
	{
		TAG4INFO xTemp[] =
		{
			{ "USTRING", "NAME", "", e4unique, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==2)
	{
		TAG4INFO xTemp[] =
		{
			{ "NUSTRING", "NAME", "", 0, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==3)
	{
		TAG4INFO xTemp[] =
		{
			{ "UNUMBER", "GROESSE", "", e4unique, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==4)
	{
		TAG4INFO xTemp[] =
		{
			{ "NUNUMBER", "GROESSE", "", 0, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==5)
	{
		TAG4INFO xTemp[] =
		{
			{ "UDATE", "GEBDAT", "", e4unique, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==6)
	{
		TAG4INFO xTemp[] =
		{
			{ "NUDATE", "GEBDAT", "", 0, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==7)
	{
		TAG4INFO xTemp[] =
		{
			{ "UCINDEX", "NAME+VORNAME+GEBDAT", "", e4unique, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
	else if (nChoice==8)
	{
		TAG4INFO xTemp[] =
		{
			{ "NUCINDEX", "NAME+VORNAME+GEBDAT", "", 0, 0 },
			{ 0, 0, 0, 0, 0 }
		};
		memcpy( tags, xTemp, sizeof(TAG4INFO) * 2 );
	}
}

//------------------------------------------------------------------
//
// Testroutinen
//
//------------------------------------------------------------------

//------------------------------------------------------------------
//	a) Datenbank anlegen
BOOL DatenbankAnlegen( const String& rDatabaseName )
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Datenbank %s anlegen...\n", rDatabaseName.GetStr() );
	if ( !aStorage.CreateDatabase( rDatabaseName, STREAM_STD_READWRITE,
		TESTBASE_RECSIZE ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	b) Datenbank loeschen
BOOL DatenbankLoeschen( const String& rDatabaseName )
{
	BOOL bSuccess = FALSE;

	if ( !aStorage.DropDatabase(rDatabaseName) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	c) Datenbank oeffnen
BOOL DatenbankOeffnen( const String& rDatabaseName )
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Datenbank %s oeffnen...\n", rDatabaseName.GetStr() );
	if ( !aStorage.OpenDatabase( rDatabaseName, STREAM_STD_READWRITE,
		TESTBASE_RECSIZE ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	d) Datenbank schliessen
BOOL DatenbankSchliessen()
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Datenbank schliessen...\n" );
	if ( !aStorage.CloseDatabase() )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	e) Tabelle anlegen
BOOL TabelleAnlegen( const String& rTableKey, const int nNumFields,
					const int nType )
{
	BOOL bSuccess = FALSE;

	FIELD4INFO* xFelder = NULL;
	BuildTableDef( xFelder, nNumFields, nType );

	//fprintf( stderr, "Tabelle %s anlegen...\n", rTableKey.GetStr() );
	if ( !aStorage.CreateTable( rTableKey, xFelder ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	f) Tabelle loeschen
BOOL TabelleLoeschen( const String& rTableKey )
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Tabelle %s loeschen...\n", rTableKey.GetStr() );
	if ( !aStorage.DropTable( rTableKey ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	g) Tabelle oeffnen
BOOL TabelleOeffnen( const String& rTableKey )
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Tabelle %s oeffnen...\n", rTableKey.GetStr() );
	if ( !aStorage.OpenTable( rTableKey ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	h) Tabelle schliessen
BOOL TabelleSchliessen( const String& rTableKey )
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Tabelle %s schliessen...\n", rTableKey.GetStr() );
	if ( !aStorage.CloseTable( rTableKey ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	i) Index anlegen
BOOL IndexAnlegen( const String& rTableKey, const String& rIndexKey, const int nType )
{
	BOOL bSuccess = FALSE;

	TAG4INFO* xTags = NULL;
	BuildIndexDef( xTags, nType );

	//fprintf( stderr, "Index %s fuer Tabelle %s anlegen...\n",
	//	rIndexKey.GetStr(), rTableKey.GetStr() );
	if ( !aStorage.CreateIndex( rTableKey, rIndexKey, xTags ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	j) Index loeschen
BOOL IndexLoeschen(const String& rIndexKey)
{
	BOOL bSuccess = FALSE;

	fprintf( stderr, "Index loeschen ist noch nicht implementiert!\n" );

	return bSuccess;
}

//------------------------------------------------------------------
//	k) Index oeffnen
BOOL IndexOeffnen(const String& rTableKey, const String& rIndexKey)
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Index %s fuer Tabelle %s oeffnen...\n",
	//	rIndexKey.GetStr(), rTableKey.GetStr() );
	if ( !aStorage.OpenIndex( rTableKey, rIndexKey ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	l) Index schliessen
BOOL IndexSchliessen(const String& rTableKey, const String& rIndexKey)
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Index %s schliessen...\n", rIndexKey.GetStr() );
	if ( !aStorage.CloseIndex( rTableKey, rIndexKey ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	m) Tag auswaehlen
BOOL TagAuswaehlen(const String& rTableKey, const String& rTagName)
{
	BOOL bSuccess = FALSE;

	//fprintf( stderr, "Tag %s fuer Tabelle %s auswaehlen...\n",
	//	rTagName.GetStr(), rTableKey.GetStr() );
	if ( !aStorage.SelectTag( rTableKey, rTagName ) )
		bSuccess = TRUE;

	return bSuccess;
}

//------------------------------------------------------------------
//	n) Datensaetze einfuegen ( Anzahl frei waehlbar )
BOOL DatensatzEinfuegen(const String& rTableKey, const ULONG nSetCount)
{
	BOOL bSuccess = TRUE;

	//fprintf( stderr, "%u Datensaetze einfuegen\n", nSetCount );
	for ( ULONG i=0; i<nSetCount && bSuccess; i++ )
	{
		if ( aStorage.InsertSet( *pTableKey )  )
			bSuccess = FALSE;
	}

	return bSuccess;
}

//------------------------------------------------------------------
//	o1) Datensaetze loeschen ( Anzahl und Position frei waehlbar )
BOOL DatensatzLoeschen(const String& rTableKey, const unsigned nSetCount,
					   const unsigned nSetPosition)
{
	BOOL bSuccess = TRUE;

	//fprintf( stderr, "%u Datensaetze ab Satz Nr. %u loeschen...\n", nSetCount,
	//	nSetPosition );
	for ( ULONG i=0; i<nSetCount && bSuccess; i++ )
	{
		if ( aStorage.DeleteSet( *pTableKey, nSetPosition+i ) )
			bSuccess = FALSE;
	}

	return bSuccess;
}

//------------------------------------------------------------------
//	o2) Alle Datensaetze loeschen
BOOL AlleDatensaetzeLoeschen(const String& rTableKey)
{
	BOOL bSuccess = TRUE;

	//fprintf( stderr, "Alle Datensaetze loeschen...\n" );
	for ( ULONG i=0; i<nSetCount && bSuccess; i++ )
	{
		if ( aStorage.DeleteAll( *pTableKey ) )
			bSuccess = FALSE;
	}

	return bSuccess;
}

//------------------------------------------------------------------
//	p) Datensaetze aendern ( Anzahl und Position frei waehlbar )
BOOL DatensatzAendern(const String& rTableKey, const unsigned nSetCount,
					  const unsigned nSetPosition)
{
	BOOL bSuccess = FALSE;

	fprintf( stderr, "Datensatz aendern ist noch nicht implementiert!\n" );

	return bSuccess;
}

//------------------------------------------------------------------
//	q) Datensaetze auslesen ( Tabelle und Richtung frei waehlbar )
BOOL DatensatzAuslesen(const String& rTableKey, const BOOL bFromTop, ErrCode& nCode)
{
	char* sSet = new char[2048];

	//fprintf( stderr, "Datensaetze aus Tabelle %s einlesen...\n",
	//	rTableKey.GetStr() );
	aStorage.PrepareFirstFetch( *pTableKey, bFromTop );
	if ( bFromTop )
		while ((nCode = aStorage.FetchNext( *pTableKey, sSet ))==ERRCODE_NONE);
	else
		while ((nCode = aStorage.FetchPrev( *pTableKey, sSet ))==ERRCODE_NONE);

	delete sSet;

	return (nCode==ERRCODE_NONE ? TRUE : FALSE);
}

//------------------------------------------------------------------
BOOL ExecuteCommand( const int& nCommand, const SvFileStream& pLogFile,
					const BOOL bProfiling)
{
	ErrCode nCode = ERRCODE_NONE;
	ULONG nStartTicks, nTicks;	// Variablen fuer die Zeitmessung
	BOOL bSuccess;				// Flag, ob die Funktion erfolgreich verlief

	if ( bProfiling && (nCommand!=PARSE_NOTHING) )
		nStartTicks = Time::GetSystemTicks();

	// Funktion aufrufen
	switch ( nCommand )
	{
	case PARSE_CREATE|PARSE_DATABASE:
		fprintf( stderr, "Create database ..." );
		bSuccess = DatenbankAnlegen(*pDatabaseName);
		break;
	case PARSE_CREATE|PARSE_TABLE:
		fprintf( stderr, "Create table ..." );
		bSuccess = TabelleAnlegen(*pTableKey,nNumFields,nType);
		break;
	case PARSE_CREATE|PARSE_INDEX:
		fprintf( stderr, "Create index ..." );
		bSuccess = IndexAnlegen(*pTableKey,*pIndexKey,nType);
		break;
	case PARSE_DROP|PARSE_DATABASE:
		fprintf( stderr, "Drop database ..." );
		bSuccess = DatenbankLoeschen(*pDatabaseName);
		break;
	case PARSE_DROP|PARSE_TABLE:
		fprintf( stderr, "Drop table ..." );
		bSuccess = TabelleLoeschen(*pTableKey);
		break;
	case PARSE_OPEN|PARSE_DATABASE:
		fprintf( stderr, "Open database ..." );
		bSuccess = DatenbankOeffnen(*pDatabaseName);
		break;
	case PARSE_OPEN|PARSE_TABLE:
		fprintf( stderr, "Open table ..." );
		bSuccess = TabelleOeffnen(*pTableKey);
		break;
	case PARSE_OPEN|PARSE_INDEX:
		fprintf( stderr, "Open index ..." );
		bSuccess = IndexOeffnen(*pTableKey,*pIndexKey);
		break;
	case PARSE_CLOSE|PARSE_DATABASE:
		fprintf( stderr, "Close database ..." );
		bSuccess = DatenbankSchliessen();
		break;
	case PARSE_CLOSE|PARSE_TABLE:
		fprintf( stderr, "Close table ..." );
		bSuccess = TabelleSchliessen(*pTableKey);
		break;
	case PARSE_CLOSE|PARSE_INDEX:
		fprintf( stderr, "Close index ..." );
		bSuccess = IndexSchliessen(*pTableKey,*pIndexKey);
		break;
	case PARSE_SELECT|PARSE_TAG:
		fprintf( stderr, "Select tag ..." );
		bSuccess = TagAuswaehlen(*pTableKey,*pTagName);
		break;
	case PARSE_SELECT|PARSE_FROM:
		fprintf( stderr, "Select sets ..." );
		bSuccess = DatensatzAuslesen(*pTableKey,bAscending,nCode);
		// EOF fuehrt zwar zum Abbruch der Schleife aber ist kein Fehler
		if ( !bSuccess && nCode==12L )
			bSuccess = TRUE;
		break;
	case PARSE_INSERT:
		fprintf( stderr, "Insert sets ..." );
		bSuccess = DatensatzEinfuegen(*pTableKey,nSetCount);
		break;
	case PARSE_DELETE|PARSE_FROM:
		fprintf( stderr, "Delete sets ..." );
		bSuccess = DatensatzLoeschen(*pTableKey,nSetCount,nSetPosition);
		break;
	case PARSE_DELETE|PARSE_ALL:
		fprintf( stderr, "Delete all sets ..." );
		bSuccess = AlleDatensaetzeLoeschen(*pTableKey);
		break;
	case PARSE_NOTHING:
		bSuccess = TRUE;
		break;
	default:
		fprintf( stderr, "Unrecognized command line. Execution " );
		bSuccess = FALSE;
	}

	if ( nCommand!=PARSE_NOTHING )
	{
		if ( bSuccess )
			fprintf( stderr, "passed!\n" );
		else
			fprintf( stderr, "failed!\n" );
	}

	if ( bProfiling && (nCommand!=PARSE_NOTHING) )
	{
		nTicks = Time::GetSystemTicks();

		fprintf( stderr, "Elapsed time: %d ms\n", nTicks-nStartTicks );
	}

	return ( nCommand==PARSE_NOTHING ? TRUE : bSuccess );
}

//------------------------------------------------------------------
unsigned int ParseCommandLine(const String& rCommandLine, ParseErrors& eError)
{
	BOOL bCommandComplete = TRUE;	// Flag, ob Parsing erfolgreich war
	unsigned int nCommand = PARSE_NOTHING;

	// Anzahl der Tokens einer Kommandozeile bestimmen
	USHORT nTokens = rCommandLine.GetTokenCount( ' ' );

	USHORT nRealToken = 0;
	String* pToken = NULL;
	for ( USHORT i=0; i<nTokens; i++ )
	{
		pToken = new String(rCommandLine.GetToken(i,' ').GetStr());

		// Enthaelt die Zeile ueberhaupt Informationen
		if ( pToken->Len()>0 )
		{
			nRealToken++;

			if ( pToken->Compare(";")==COMPARE_EQUAL )
			{
				// Wenn ein korrektes Kommando gefunden wurde, darf beliebiger
				// Kommentar folgen
				if ( bCommandComplete )
					i = nTokens;
			}
			else if ( pToken->Compare("create")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_CREATE;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("drop")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_DROP;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("open")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_OPEN;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("close")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_CLOSE;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("database")==COMPARE_EQUAL )
			{
				// Das Objekt muss an zweiter Stelle kommen
				if ( nRealToken==2 )
				{
					if ( (nCommand&PARSE_CREATE) ||	(nCommand&PARSE_DROP) ||
						(nCommand&PARSE_OPEN) || (nCommand&PARSE_CLOSE) )
					{
						nCommand |= PARSE_DATABASE;
						if ( nCommand&PARSE_CLOSE )
							bCommandComplete = TRUE;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_OBJECT;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_OBJECT;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("table")==COMPARE_EQUAL )
			{
				// Das Objekt muss an zweiter Stelle kommen
				if ( nRealToken==2 )
				{
					if ( (nCommand&PARSE_CREATE) ||	(nCommand&PARSE_DROP) ||
						(nCommand&PARSE_OPEN) || (nCommand&PARSE_CLOSE) )
						nCommand |= PARSE_TABLE;
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_OBJECT;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_OBJECT;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("index")==COMPARE_EQUAL )
			{
				// Das Objekt muss an zweiter Stelle kommen
				if ( nRealToken==2 )
				{
					if ( (nCommand&PARSE_CREATE) ||
						(nCommand&PARSE_OPEN) || (nCommand&PARSE_CLOSE) )
						nCommand |= PARSE_INDEX;
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_OBJECT;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = ILLEGAL_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("insert")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_INSERT;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("into")==COMPARE_EQUAL )
			{
				// Das Token muss an dritter Stelle kommen
				if ( nRealToken==3 )
				{
					if ( !(nCommand&PARSE_INSERT) )
					{
						// Ein Fehler ist aufgetreten
						i=nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("delete")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_DELETE;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("all")==COMPARE_EQUAL )
			{
				// Das Token muss an zweiter Stelle kommen
				if ( nRealToken==2 )
				{
					if ( nCommand&PARSE_DELETE )
					{
						nCommand |= PARSE_ALL;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("startat")==COMPARE_EQUAL )
			{
				if ( nRealToken==6 )
				{
					if ( !(nCommand&PARSE_DELETE) )
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("select")==COMPARE_EQUAL )
			{
				// Der Befehl muss an erster Stelle kommen
				if ( nRealToken==1 )
				{
					bCommandComplete = FALSE;
					nCommand = PARSE_SELECT;
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_ACTION;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("tag")==COMPARE_EQUAL )
			{
				// Token muss an  zweiter Position stehen
				if ( nRealToken==2 )
				{
					if ( nCommand&PARSE_SELECT )
					{
						nCommand |= PARSE_TAG;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("from")==COMPARE_EQUAL )
			{
				// Token muss an zweiter ...
				if ( nRealToken==2 )
				{
					if ( nCommand&PARSE_SELECT )
					{
						nCommand |= PARSE_FROM;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				// ... oder dritter Position stehen
				else if ( nRealToken==3 )
				{
					if ( !(nCommand&PARSE_DELETE) )
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("ascending")==COMPARE_EQUAL )
			{
				// Das Token muss an vierter Stelle stehen
				if ( nRealToken==4 )
				{
					if ( nCommand==(PARSE_SELECT|PARSE_FROM) )
					{
						// Aufsteigende Reihenfolge beim Iterieren
						bAscending = TRUE;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else if ( pToken->Compare("descending")==COMPARE_EQUAL )
			{
				// Das Token muss an vierter Stelle stehen
				if ( nRealToken==4 )
				{
					if ( nCommand==(PARSE_SELECT|PARSE_FROM) )
					{
						// Aufsteigende Reihenfolge beim Iterieren
						bAscending = TRUE;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_TOKEN;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = MISPLACED_TOKEN;
					bCommandComplete = FALSE;
				}
			}
			else
			{
				if ( nRealToken==2 )
				{
					if ( (nCommand&PARSE_INSERT) || (nCommand&PARSE_DELETE) )
					{
						// Es muss sich um die Anzahl der Saetze handeln
						nSetCount = atoi(pToken->GetStr());
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_PARAMETER;
						bCommandComplete = FALSE;
					}
				}
				else if ( nRealToken==3 )
				{
					if ( (nCommand&PARSE_DATABASE) &&
						(nCommand&(PARSE_CREATE|PARSE_DROP|PARSE_OPEN)) )
					{
						// Es muss ein Datenbank-Dateiname sein
						pDatabaseName = new String(pToken->GetStr());
						bCommandComplete = TRUE;
					}
					else if ( (nCommand&PARSE_TABLE) &&
						(nCommand&(PARSE_CREATE|PARSE_DROP|PARSE_OPEN|PARSE_CLOSE) ) )
					{
						// Es muss ein Tabellen-Key sein
						pTableKey = new String(pToken->GetStr());

						bCommandComplete = TRUE;
					}
					else if ( (nCommand&PARSE_INDEX) &&
						(nCommand&(PARSE_CREATE|PARSE_OPEN|PARSE_CLOSE) ) )
					{
						// Es muss ein Tabellen-Key sein
						pTableKey = new String(pToken->GetStr());
					}
					else if ( nCommand==(PARSE_SELECT|PARSE_TAG) )
					{
						// Es muss ein Tabellen-Key sein
						pTableKey = new String(pToken->GetStr());
					}
					else if ( nCommand==(PARSE_SELECT|PARSE_FROM) )
					{
						// Es muss ein Tabellen-Key sein
						pTableKey = new String(pToken->GetStr());

						bCommandComplete = TRUE;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_PARAMETER;
						bCommandComplete = FALSE;
					}
				}
				else if ( nRealToken==4 )
				{
					if ( (nCommand&PARSE_INDEX) &&
						(nCommand&(PARSE_CREATE|PARSE_OPEN|PARSE_CLOSE)) )
					{
						// Es muss ein Index-Key sein
						pIndexKey = new String(pToken->GetStr());

						bCommandComplete = TRUE;
					}
					else if ( (nCommand==(PARSE_TABLE|PARSE_CREATE)) )
					{
						// Es handelt sich um die Anzahl der Felder
						nNumFields = atoi(pToken->GetStr());
					}
					else if ( (nCommand&PARSE_INSERT) || (nCommand&PARSE_DELETE) )
					{
						// Es muss ein Tabellen-Key sein
						pTableKey = new String(pToken->GetStr());

						bCommandComplete = TRUE;
					}
					else if ( nCommand==(PARSE_SELECT|PARSE_TAG) )
					{
						// Es muss ein Tag-Name sein
						pTagName = new String(pToken->GetStr());

						bCommandComplete = TRUE;
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i=nTokens;
						eError = ILLEGAL_PARAMETER;
						bCommandComplete = FALSE;
					}
				}
				else if ( nRealToken==5 )
				{
					if ( (nCommand&PARSE_CREATE) && (nCommand&(PARSE_TABLE|PARSE_INDEX)) )
					{
						// Es muss sich um den Typ der Tabelle bzw. Index handeln
						nType = atoi(pToken->GetStr());
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_PARAMETER;
						bCommandComplete = FALSE;
					}
				}
				else if ( nRealToken==6 )
				{
					if ( nCommand&PARSE_DELETE )
					{
						// Es muss sich um die Startposition handeln
						nSetPosition = atoi(pToken->GetStr());
					}
					else
					{
						// Ein Fehler ist aufgetreten
						i = nTokens;
						eError = ILLEGAL_PARAMETER;
						bCommandComplete = FALSE;
					}
				}
				else
				{
					// Ein Fehler ist aufgetreten
					i = nTokens;
					eError = SYNTAX_ERROR;
					bCommandComplete = FALSE;
				}
			}
		}

		DELETEZ(pToken);
	}

	return ( bCommandComplete ? nCommand : PARSE_NOTHING );
}

//------------------------------------------------------------------
BOOL ReadCommandLine(SvFileStream& rCommandLine, const SvFileStream& rLogFileStream, const ULONG nLineNumber)
{
	BOOL bSuccess = FALSE;
	String aCommandLine;
	ParseErrors eError = NO_ERROR;

	// Zeile einlesen
	if ( rCommandLine.ReadLine(aCommandLine) )
	{
		// Ueberfluessige Leerzeichen entfernen
		aCommandLine = aCommandLine.EraseLeadingChars();
		aCommandLine = aCommandLine.EraseTrailingChars();

		if ( ExecuteCommand(ParseCommandLine(aCommandLine, eError),
			rLogFileStream,bProfiling) )
			bSuccess = TRUE;
		else
		{
			// Fehlerausgabe
			switch(eError)
			{
			case SYNTAX_ERROR:
				fprintf( stderr, "Syntax error " );
			case ILLEGAL_ACTION:
				fprintf( stderr,
					"Illegal action (only create, drop, open, close, insert, delete, select allowed) " );
				break;
			case MISPLACED_ACTION:
				fprintf( stderr,
					"Action (create, drop, open, close, insert, delete, select) not first token " );
				break;
			case ILLEGAL_OBJECT:
				fprintf( stderr,
					"Illegal object (only database, table, index, tag allowed) " );
				break;
			case MISPLACED_OBJECT:
				fprintf( stderr, "Object (database, table, index, tag) not second token " );
				break;
			case ILLEGAL_TOKEN:
				fprintf( stderr, "Token not allowed with used keyword " );
				break;
			case MISPLACED_TOKEN:
				fprintf( stderr, "Token not allowed at used position " );
				break;
			case ILLEGAL_PARAMETER:
				fprintf( stderr, "Parameter not allowed with action " );
				break;
			}
			fprintf( stderr, "in line %d\n", nLineNumber);
		}
	}

	return bSuccess;
}

//------------------------------------------------------------------

/*
 * main.
 *
 * The program uses command line arguments and is called as follows:
 *
 * testbase <control file> <log file> [noprofile]
 *
 * <control file> is the name of the file containing the commands to be
 *                executed by the test program
 * <log file>     is the name of the file all information about the testing is
 *                written to
 * noprofile      is a flag indicating that no profiling of the commands should
 *                be done
 *
 * Warning!
 * The name of the log file should be chosen carefully, because if the file
 * already exists, it is overwritten commentlessly!!
 *
 */
int main (int argc, char **argv)
{
	String* pControlFileName = NULL;
	String* pLogFileName = NULL;

	// Zufallsgenerator initialisieren
	srand( (unsigned)time( NULL ) );

	// Auswertung der Kommandozeilenparameter
	switch (argc)
	{
	case 3:
		pControlFileName = new String(argv[1]);
		pLogFileName = new String(argv[2]);
		bProfiling = TRUE;
		break;
	case 4:
		pControlFileName = new String(argv[1]);
		pLogFileName = new String(argv[2]);
		bProfiling = FALSE;
		break;
	default:
		printf( "Wrong number of arguments specified!\n" );
		printf( "Usage: testbase <control file> <log file> [noprofile]\n\n" );
		printf( "<control file> is the name of the file containing the commands to be executed\n" );
		printf( "               by the test program\n" );
		printf( "<log file>     is the name of the file all information about the testing is\n" );
		printf( "               written to\n" );
		printf( "noprofile      is a flag indicating that no profiling of the commands should be\n" );
		printf( "               done\n\n" );
		printf( "Warning!\n" );
		printf( "The name of the log file should be chosen carefully, because if the file already\n" );
		printf( "exists, it is overwritten commentlessly!!\n\n" );
		break;
	}

	if (3<=argc<=4)
	{
		// Steuerungsdatei zum Lesen oeffnen
		SvFileStream *pControlStream = new SvFileStream ( *pControlFileName,
			STREAM_STD_READ);

		// Protokolldatei oeffnen
		SvFileStream* pLogFileStream = new SvFileStream( *pLogFileName,
			STREAM_STD_WRITE|STREAM_TRUNC);

		// Einlesen der Steuerungsdatei
		ULONG nLineNumber = 1;
		while ( TRUE )
		{
			// Naechste Zeile einlesen
			if ( !ReadCommandLine(*pControlStream,*pLogFileStream,nLineNumber++) )
				break;
		}

		// Protokollstream wieder schliessen
		pLogFileStream->Close();

		// Steuerungsstream wieder schliessen
		pControlStream->Close();

		// Aufraeumen
		DELETEZ(pLogFileName);		// Name der Protokolldatei
		DELETEZ(pLogFileStream);	// Die Protokolldatei
		DELETEZ(pControlFileName);	// Name der Steuerungsdatei
		DELETEZ(pControlStream);	// Die Steuerungsdatei
		DELETEZ(pDatabaseName);		// Das Datenbankfile
		DELETEZ(pTableKey);			// Der Schluessel der Tabelle
		DELETEZ(pIndexKey);			// Der Schluessel des Index
		DELETEZ(pTagName);			// Der Name des Tags

		// Fertig
		return 0;
	}
	else
		return -1;
}

