/*************************************************************************
 *
 *  $RCSfile: main.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SETUP_MAIN_HXX
#define _SETUP_MAIN_HXX

#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _SV_APP_HXX_
#include <vcl/svapp.hxx>
#endif

#ifdef MAC
#include <vcl/menu.hxx>
#endif

#include "simode.hxx"
#include "event.hxx"

class ResMgr;
class ModelessDialog;
class Button;

class SiModule;
class SiEnvironment;
class SiCompiledScript;
class SiProfileItem;
class SiAgenda;
class SetupWindow;
class SetupAgentDialog;
class ResponseFile;
class RemoteControl;
class ThreadExecuter;
struct InitiateWorkstation;

extern "C" {
extern const char* FindArch(const char* pDefSource, const char* pDefName );
};

struct TopLevelObject;
DECLARE_LIST(TopLevelStack,TopLevelObject*)

class SetupApp : public Application
{
private:
	RemoteControl*		m_pRemoteControl;

	Accelerator*		m_pExitAccel;
	ResMgr*				m_pResMan;
	SetupWindow*		m_pMainWin;
	SetupAgentDialog*	m_pMainDlg;

	SiEnvironment*		m_pEnv;
	SiCompiledScript*	m_pCompiledScript;
	SiAgenda*			m_pAgenda;
	ResponseFile*		m_pResponseFile;

	BOOL				m_bPerformInstall		: 1;
	BOOL				m_bInterrupt			: 1;
	BOOL				m_bParamDoCRCCheck		: 1;
	BOOL 				m_bOfficeMode			: 1;
	BOOL 				m_bIsAppServerInstall	: 1;
	#if defined WIN || defined WNT
	BOOL				m_bShare				: 1;
	#endif

	ThreadExecuter*		m_pExecuter;
	ByteString			m_aOldLicense;

	ByteString			m_aParamDestPath;
	ByteString			m_aParamFollowApp;
	ByteString			m_aParamResponseFile;
	ByteString			m_SetupIniSourcePath;
	int				  	m_nLanguage;
	USHORT 				m_nWorkStationCnt;
	USHORT 				m_nWorkStationIdx;

	// application
	void			SetAccelerators();
	ResMgr*			LoadRessources();
	void			ReadParams();

	// event
	void			OnEventStart();
	void			OnEventEnd();
	void			OnEventEndInstallation();
	void 			GetMainDialogModel();

	void 			RunUpdateOldVersion();
	void			RunInstallation( SiCompiledScript*, SiInstallMode, SiInstallSubMode = ISM_INVALID,
									 BOOL bUpgrade = FALSE, BOOL bNoEvents = FALSE );
	void			InstallWorkstation( InitiateWorkstation* pWorkStation );
	void 			WriteWorkstationDB();
	void			PrepareLanguageContext();
	void 			CloseInstallation();

	BOOL 			ProcessFollowApps( BOOL bTestOnly = FALSE );

	// script
	BOOL			ReadScript(SiDirEntry const&);
	BOOL			ReadScript(SiDirEntry const&, SiCompiledScript*);
	BOOL			FindAndReadScript();
	ByteString  		GetValue(SiProfileItem const*, ByteString const& aDefault) const;
	void 			ShowScriptNotFound();

	// TopLevel
	TopLevelStack*		pTopLevelStack;
	TopLevelObject*		pActTopObject;

	void			AddTopLevelStack(ExecType eType, ByteString aCommand, BOOL bFollow = FALSE);
	void 			CheckDeadTopLevel();
	void			RemoveDeadTopLevel(SiModule* pModule);
	SiModule* 		GetNextTopLevel(SiModule* pModule, USHORT nId, SiModule* pBest);


	// service
	BOOL		AreAllAppsClosed(Window* pParent, USHORT nId);
	void		KillSlideshowFiles(SiCompiledScript*);
	BOOL 		QueryKillEntireDir(ByteString const&);
	void 		KillEntireDir(ByteString const&);
	void 		KillEntireDir(SiDirEntry const& rEntry, BOOL bDelFiles);

	// handler
	#ifdef MAC
					DECL_LINK( ExitMenuHdl, Menu * );
	#endif
					DECL_LINK( ExitHdl, Accelerator* );
protected:
					DECL_LINK( WindowHideHdl, void* );
	virtual void Main();

	virtual void UserEvent(ULONG nEvent, void* pEventData);
	#ifdef MAC
	virtual void AppEvent( const ApplicationEvent& rAppEvent );
	#endif

	// unter WNT wird bShare hart auf TRUE gesetzt, gleicher GUIBASE-Code
	#if defined  WIN || defined WNT
	BOOL		IsShareLoaded() 	 						{ return m_bShare; }
	#endif

public:
				SetupApp();
				~SetupApp();

	void				Exit() { ExitHdl(0); }
	void				PaintExitButton( Window* pWin );
	void 				MainWindowResize();

	BOOL 				IsResponseMode() const { return m_aParamResponseFile.Len()? TRUE : FALSE; }

	SiEnvironment*		GetEnvironment() const { return m_pEnv; }
	SiAgenda*	   		GetAgenda() const { return m_pAgenda; }
	SiCompiledScript* 	GetCScript() const { return m_pCompiledScript; }
	String				GetProgrammName() const;

	const ResMgr*		GetResManager() const { return m_pResMan; }
	int 				GetLanguage() const { return m_nLanguage; }

	const Link			GetExitHdl() const { return LINK(this, SetupApp, ExitHdl); }
};

extern SetupApp *pSetupApp;

#endif

