/*************************************************************************
 *
 *  $RCSfile: action.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:57 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <tools/svwin.h>

#ifdef UNX
#include "sys/stat.h"
#include "unistd.h"
#endif

#ifndef _REGISTRY_REGISTRY_HXX_
#include <registry/registry.hxx>
#endif

#ifndef NOOLDSV //autogen
#include <vcl/system.hxx>
#endif

#ifndef _SV_WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif

#ifndef _LIBCALL_HXX //autogen
#include <svtools/libcall.hxx>
#endif

#ifndef _SV_CONFIG_HXX //autogen
#include <vcl/config.hxx>
#endif

#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif

#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_XCHANGESBATCH_HPP_
#include <com/sun/star/util/XChangesBatch.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XNAMEREPLACE_HPP_
#include <com/sun/star/container/XNameReplace.hpp>
#endif

#include "action.hxx"
#include "environ.hxx"
#include "agenda.hxx"
#include "decltor.hxx"
#include "sicustom.hxx"
#include "script.hxx"
#include "event.hxx"
#include "sihelp.hxx"
#include "fields.hxx"

#include "main.hxx"

#include "zipfile.hxx"
#include "os.hxx"
#include "sistream.hxx"
#include "sibasic.hxx"

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::util;
using namespace ::com::sun::star::container;
using namespace ::com::sun::star::registry;
using namespace ::cppu;

char SEP[] = ", ";

ULONG _Substitute( const ByteString& rFilename, const SiEnvironment& rEnv, SiInstallMode eMode )
{
	SvFileStream aReadStrm( UniString::CreateFromAscii(rFilename.GetBuffer()), STREAM_READ );
	char* pBuf = new char[32000];

	ULONG nRead = aReadStrm.Read( pBuf, 32000 );
	aReadStrm.Close();
	pBuf[nRead] = 0x00;

	ByteString aBuffer(pBuf);
	delete pBuf;

	#if defined(WNT)
	ByteString aDesktopPath( WinOS::SHGetSystemDesktopFolderName() );
	while( aBuffer.SearchAndReplace("<osdesktop>", aDesktopPath) != STRING_NOTFOUND );
	#endif

	ByteString aProgPath( rEnv.GetProgPath(eMode) );
	ByteString aWorkPath( rEnv.GetWorkPath(eMode) );

	while( aBuffer.SearchAndReplace("<progpath>", aProgPath) != STRING_NOTFOUND );
	while( aBuffer.SearchAndReplace("<workpath>", aWorkPath) != STRING_NOTFOUND );

	if( eMode != IM_NETWORK )
	{
		while( aBuffer.SearchAndReplace("<user_progpath>", aProgPath) != STRING_NOTFOUND );
		while( aBuffer.SearchAndReplace("<user_workpath>", aWorkPath) != STRING_NOTFOUND );
	}

#if defined (UNX)
	if ( (eMode == IM_NETWORK) && UnixOS::bSOfficeRemoteApplication() )
	{
		aBuffer.SearchAndReplaceAll( "<exec_host>", UnixOS::rGetExecHostAction() );
  		#if !defined (USE_DTACTION)
		aBuffer.SearchAndReplaceAll( "<display_host>",
									 "-display %DisplayHost%$DISPLAY" );
		#endif
	}
	else
	{
		aBuffer.SearchAndReplaceAll( "<exec_host>", 	"" );
  		#if !defined (USE_DTACTION)
		aBuffer.SearchAndReplaceAll( "<display_host>", 	"" );
		#endif
	}
#endif

	SiDirEntry aToKill( rFilename );
	aToKill.Kill();

	SvFileStream aWriteStrm( UniString::CreateFromAscii(rFilename.GetBuffer()), STREAM_WRITE );

	aWriteStrm.Write( aBuffer.GetBuffer(), aBuffer.Len() );
	aWriteStrm.Close();

	return aBuffer.Len();
}

// SiAction ----------------------------------------------------------------
//
SiAction::SiAction(SiAgenda *pAgenda, SiActionType eType /* = AT_UNKNOWN */ )
{
	m_pAgenda       = pAgenda;
	m_pDependsOn    = NULL;
	m_bSucceeded    = FALSE;
	m_bMayExecute	= TRUE;
	m_eType			= eType;
}

SiAction::~SiAction() // virtual
{}

SiLogStream& SiAction::GetLogfile()
{
	return m_pAgenda->GetLogfile();
}

SiInstallMode SiAction::GetInstallMode() const
{
	return m_pAgenda->GetInstallMode();
}

BOOL SiAction::DoRecoverOnly() const
{
	return m_pAgenda->DoRecoverOnly();
}

BOOL SiAction::DoesTargetExist(SiEnvironment const& Env) const
// Prueft im Falle von Abhaengigkeiten die Targets.
// Sonst wird FALSE zurueckgeben, damit die Aktion bei Recover
// ausgefuehrt wird.
{
	if (m_pDependsOn != NULL)
		return m_pDependsOn->DoesTargetExist(Env);

	return CheckTarget(Env);
}

BOOL SiAction::CheckTarget(SiEnvironment const& Env) const
// Prueft das Ziel nur fuer diese Aktion (ohne Abhaengigkeiten)
{
	return FALSE; // Default: lieber ausfuehren als nicht
}

void SiAction::SetDependence(SiAction *pAction)
// pre:  es gibt noch keine Abhaengigkeit
// post: Abhaengigkeit wurde gesetzt
{
	DBG_ASSERT(m_pDependsOn == NULL, "SiAction::SetDependence() more than one dependence");
	m_pDependsOn = pAction;
}

BOOL SiAction::Succeeded() const
// Abfrage ob Aktion erfolgreich ausgefuehrt wurde
{
	return m_bSucceeded;
}

BOOL SiAction::SetSuccess(BOOL b)
{
	m_bSucceeded = b;
	return b;       // wegen Schreibweise: return SetSuccess(lErr==0);
}

// MakeDirAction ---------------------------------------------------------
//
SiMakeDirAction::SiMakeDirAction(SiAgenda *pAgenda, ByteString const& aDirName, int nRights)
: SiAction(pAgenda, AT_MAKEDIR)
{
	m_aDirName = aDirName;
	m_nUnixRights = nRights;
}

BOOL SiMakeDirAction::CheckTarget(SiEnvironment const& Env) const
{
	SiDirEntry aSiDirEntry  = Env.GetDestPath();
			 aSiDirEntry += m_aDirName;

	return aSiDirEntry.Exists();
}

BOOL SiMakeDirAction::Execute(SiEnvironment& Env)
{
	SiDirEntry aSiDirEntry  = Env.GetDestPath();
			 aSiDirEntry += m_aDirName;

	if( aSiDirEntry.Exists() )
		return TRUE;

	BOOL bSuccess = aSiDirEntry.MakeDir();

	GetLogfile().Success(bSuccess) << "mkdir " << aSiDirEntry.GetFull()
		<< (bSuccess ? " OK" : " ERR") << endl;

	#ifdef UNX
	UnixOS::SetRights( aSiDirEntry.GetFull(), m_nUnixRights );

	GetLogfile().Success(bSuccess) << "chmod " << aSiDirEntry.GetFull()
		<< " " << (long)m_nUnixRights << endl;
	#endif

	#ifdef OS2
	ItemIDPath aID( aSiDirEntry.GetFull(FSYS_STYLE_URL) );
	ItemIDPath aPar, aChild;
	if( aID.Split( aPar, aChild ) )
	{
		Folder aFolder( aPar );
		ItemIDPath aNewID;
		aFolder.RenameItem( aChild, aNewID, aSiDirEntry.GetName() );
		GetLogfile().Success(TRUE) << "write OS2-EA " <<
			aSiDirEntry.GetFull() << " ==> " << aSiDirEntry.GetName() << endl;
	}
	#endif

	return SetSuccess(bSuccess);
}

// TransferAction---------------------------------------------------------
//
SiTransferAction::SiTransferAction
(
	SiAgenda    *pAgenda,
	SiActionType eType,
	SiDataCarrier const* pDataCarrier,
	ByteString const& aSourcePath,
	ByteString const& aSourceFile,
	ByteString const& aDestPath,
	ByteString const& aDestFile,
	Date   const& aDate,
	Time   const& aTime,
	SiFileFlags   Flags,
	int           nUnixRights,
	SiOs2Creator const *pOs2Creator
)
: SiAction(pAgenda, eType)
{
	m_pDataCarrier   = pDataCarrier;
	m_aSourcePath    = aSourcePath;
	m_aSourceFile    = aSourceFile;
	m_aDestPath      = aDestPath;
	m_aDestFile      = aDestFile;
	m_aDate          = aDate;
	m_aTime          = aTime;
	m_Flags          = Flags;
	m_nUnixRights    = nUnixRights;
	m_pOs2Creator	 = pOs2Creator;
}

BOOL SiTransferAction::CheckTarget(SiEnvironment const& Env) const
{
	SiDirEntry aDest  = Env.GetDestPath();
			 aDest += m_aDestPath;
			 aDest += m_aDestFile;

	return aDest.Exists();
}

int SiTransferAction::GetDiskNo() const
{
	return m_pDataCarrier == NULL ? 9999 : m_pDataCarrier->GetDiskNo();
}

void SiTransferAction::RenameFile( ByteString const& aFrom, ByteString const& aTo )
{
	#if defined(WNT)
	rename( aFrom.GetBuffer(), aTo.GetBuffer() );
	#elif defined(UNX)
	if( aFrom != aTo )
		rename( aFrom.GetBuffer(), aTo.GetBuffer() );
	#endif
}

void SiTransferAction::TouchVirtual( ByteString& rName )
{
	char cSysDelim = SiDirEntry::GetAccessDelimiter().GetChar(0);
	if( rName.GetChar(rName.Len()-1) == '/' ||
		rName.GetChar(rName.Len()-1) == cSysDelim )
		return ;

	SiDirEntry aEntry(rName);
	aEntry.ToAbs();
	aEntry.GetPath().MakeDir();

	FILE* pNull = fopen( aEntry.GetFull().GetBuffer(), "w" );
	fclose( pNull );
}

void SiTransferAction::SetDateTime(ByteString const& aFile, BOOL bLog)
{
	if (m_aDate.GetYear() != 0)
	{
		if (bLog)
		{
			International anInt;
			GetLogfile() << SEP << ByteString(anInt.GetDate(m_aDate), osl_getThreadTextEncoding()) << SEP
						 << ByteString(anInt.GetTime(m_aTime), osl_getThreadTextEncoding());
		}
		SiDirEntry 	aEntry( aFile );
		BOOL 		bWasReadOnly = FALSE;

		if( FileStat::HasReadOnlyFlag() &&
			FileStat::GetReadOnlyFlag(aEntry) )
		{
			FileStat::SetReadOnlyFlag( aEntry, FALSE );
			bWasReadOnly = TRUE;
		}
		OS::SetDateTime(aFile, m_aDate, m_aTime);
		if( bWasReadOnly )
			FileStat::SetReadOnlyFlag( aEntry, TRUE );
	}
}

void SiTransferAction::SetUnixRights(ByteString const& aFile, BOOL bLog)
{
	if (m_nUnixRights != 0)
	{
		#ifdef UNX
		UnixOS::SetRights(aFile, m_nUnixRights);
		if (bLog)
			GetLogfile() << SEP << ByteString( m_nUnixRights );
		#else
		//! DBG_ERROR("cannot set rights on this platform");
		#endif
	}
}

void SiTransferAction::SetOs2Creator(ByteString const& aFile,
									 SiEnvironment const& rEnv, BOOL bLog)
{
	if( m_pOs2Creator )
	{
#ifdef OS2
		SiDirEntry anIconDir;

		if (GetInstallMode() == IM_WORKSTATION)
			anIconDir = rEnv.GetProgPath(GetInstallMode());
		else
			anIconDir = rEnv.GetWorkPath(GetInstallMode());

		anIconDir += m_pOs2Creator->GetIconDir()->GetName();
		anIconDir += m_pOs2Creator->GetIcon();

		ByteString anIcon = anIconDir.GetFull();
		BOOL bSuccess = Os2OS::SetFileTypeAndIcon(aFile,
						m_pOs2Creator->GetName(), anIcon);

		if( bLog )
		{
			GetLogfile().Success(bSuccess)
				<< "set os/2 creator: "
				<< aFile       				  << SEP
				<< m_pOs2Creator->GetName()   << SEP
				<< anIcon       			  << endl;
		}
#else
		DBG_ERROR("SiTransferAction::SetOs2Creator(...) wrong platform");
#endif
	}
}

void SiTransferAction::DoMigration( const ByteString& rFilename,
									const SiEnvironment& rEnv )
{
}

void SiTransferAction::Substitute( const ByteString& rFilename,
								   const SiEnvironment& rEnv )
{
	ULONG nLen = _Substitute( rFilename, rEnv, GetInstallMode() );
	GetLogfile().Success(TRUE)
		<< "substitue: "
		<< rFilename
		<< " filesize (limit 32k): "
		<< nLen
		<< endl;
}

void SiTransferAction::MoveSystemFileAfterReboot( const ByteString& rSource, const ByteString& rDest )
{
#if defined(WNT) || defined(WIN)
	if( WinOS::GetOSVersion() == "W95" ||
		WinOS::GetOSVersion() == "W98" )
	{
		SiDirEntry aEntry(OS::GetGUIPath());
		aEntry += ByteString("wininit.ini");

		Config aWinInit( aEntry.GetFullUni() );
		aWinInit.SetGroup( "rename" );
		aWinInit.WriteKey( rDest, rSource );
	}
	else
		MoveFileEx( rSource.GetBuffer(), rDest.GetBuffer(), MOVEFILE_DELAY_UNTIL_REBOOT | MOVEFILE_REPLACE_EXISTING );

	GetLogfile().Success(TRUE)
		<< "move file after boot: "
		<< rSource
		<< " ==> "
		<< rDest
		<< endl;
#endif
}

// CopyAction ------------------------------------------------------------
//
SiCopyAction::SiCopyAction
(
	SiAgenda    * pAgenda,
	SiDataCarrier
		   const* pDataCarrier,
	ByteString const& aSourcePath,
	ByteString const& aSourceFile,
	ByteString const& aDestPath,
	ByteString const& aDestFile,
	Date   const& aDate,
	Time   const& aTime,
	SiFileFlags   Flags,
	int           nUnixRights,
	SiFile* 	  pFile,
	SiOs2Creator const* pOs2Creator
)
: SiTransferAction (pAgenda, AT_COPY, pDataCarrier, aSourcePath, aSourceFile,
	 aDestPath, aDestFile, aDate, aTime, Flags, nUnixRights, pOs2Creator)
{
	m_pFile = pFile;
}

BOOL SiCopyAction::Execute(SiEnvironment& Env)
{
	BOOL bTargetExists = CheckTarget(Env);

	if (m_Flags.bDontOverwrite
	&&  bTargetExists)
		return TRUE;

	if (m_Flags.bOverwriteOnly
	&& !bTargetExists)
		return TRUE;

	SiDirEntry aBigModeTemp;
	if( !m_Flags.bSetupZip && Env.IsBigMode() )
	{
		aBigModeTemp = Env.GetStartPath();
		aBigModeTemp += m_aSourceFile;

		if( !Env.GetArchive() )
			GetAgenda()->GetCallback()->RequestBigFile( m_pDataCarrier->GetName() );

		BOOL bSucc = Env.GetArchive()->GetFile( m_aSourceFile.GetBuffer(), Env.GetStartPath().GetBuffer() );
		if( !bSucc )
			return TRUE;
	}
	else if( !m_Flags.bSetupZip && !GetAgenda()->RequestDisk(m_aSourceFile,
			 m_pDataCarrier->GetDiskNo(), m_pDataCarrier->GetName()))
				return TRUE; // Keine Quelle (weil Abbruch), aber auch kein Fehler beim Kopieren

	SiDirEntry    aSource;
	if( !GetAgenda()->InstallFromNet() && (m_Flags.bSetupZip || Env.IsBigMode()) )
		aSource = Env.GetStartPath();
	else
		aSource = Env.GetSourcePath();

				aSource     += m_aSourcePath;
				aSource     += m_aSourceFile;
	ByteString      aSourceStr   = aSource.GetFull();

	SiDirEntry    aDest        = Env.GetDestPath();
				aDest       += m_aDestPath;
				aDest       += m_aDestFile;
	ByteString      aDestStr     = aDest.GetFull();

	if( m_Flags.bCheckVersion )
	{
		#if defined WIN || defined WNT
			SiDirEntry aDestFile  = aDest;
					 aDestFile += m_aDestFile;

			if( !WinOS::IsSourceFileNewerVersion(aSourceStr,FALSE/*PACKED*/,aDestStr) )
				return TRUE; // nicht ueberschreiben
		#else
			DBG_ERROR("SiCopyAction::Execute() cannot check version");
		#endif
	}

	if (aDest.Exists())
	{
		if( GetAgenda()->GetCallback()->OverwriteFile(aDest.GetFull(), m_pFile) )
		{
			OS::MakeWritable(aDestStr);
			aDest.Kill();
		}
		else
		{
			GetAgenda()->AddCanceledFileSize( m_pFile, FALSE );
			return TRUE;
		}
	}

	FSysError anError = 0;
	if( Env.IsVirtualMode() )
	{
		ByteString aTemp( aDest.GetFull() );
		TouchVirtual( aTemp );
		GetAgenda()->AddCanceledFileSize( m_pFile, FALSE );
	}
	else
	{
		FileCopier aFileCopier( aSource, aDest );
		aFileCopier.SetProgressHdl( LINK(GetAgenda(),SiAgenda,CopyProgressHdl) );
		anError = aFileCopier.Execute();
	}

#ifdef OS2
	if( anError )
	{
		BOOL bErr = Os2OS::ReplaceModule( aDestStr.GetBuffer(), aSourceStr.GetBuffer() );
		anError = bErr ? 1 : 0;

		GetLogfile().Success(anError)
			<< "OS2 DosReplaceModule "
			<< aSourceStr << SEP
			<< aDestStr;
	}
	else
#endif
	{
		GetLogfile().Success(anError==0)
			<< "copy  "
			<< aSourceStr << SEP
			<< aDestStr;
	}

	if( m_Flags.bDoMigration )
		DoMigration( aDestStr, Env );

	if( m_Flags.bSubstitute )
		Substitute( aDestStr, Env );

	// Datei stempeln und unter Unix evtl. Rechte setzen
	SetDateTime(aDestStr);
	SetUnixRights(aDestStr);
	SetOs2Creator(aDestStr, Env);

	// UnoComponent
	if( m_pFile && m_pFile->IsUnoComponent() )
		GetAgenda()->AddUnoComponent( m_pFile );

	if( Env.IsBigMode() )
		aBigModeTemp.Kill();

	GetLogfile() << SEP << "FSysError = " << ByteString::CreateFromInt32(anError) << endl;

	return SetSuccess(anError == 0);
}

// UnzipAction ------------------------------------------------------------
//
SiUnzipAction::SiUnzipAction
(
	SiAgenda    * pAgenda,
	SiDataCarrier
		   const* pDataCarrier,
	ByteString const& aArchiveFile,
	ByteString const& aSourcePath,
	ByteString const& aSourceFile,
	ByteString const& aDestPath,
	ByteString const& aDestFile,
	Date   const& aDate,
	Time   const& aTime,
	SiFileFlags   Flags,
	int           nUnixRights,
	SiFile      * pFile,
	SiOs2Creator const* pOs2Creator
)
: SiTransferAction(pAgenda, AT_UNZIP, pDataCarrier, aSourcePath, aSourceFile,
			aDestPath, aDestFile, aDate, aTime, Flags, nUnixRights, pOs2Creator)
{
	m_aArchiveFile	= aArchiveFile;
	m_pFile			= pFile;
}

BOOL SiUnzipAction::CheckTarget(SiEnvironment const& Env) const
{
	if (m_Flags.bArchive) // viele Zieldateien
	{
		SiDirEntry aDestPath  = Env.GetDestPath();
				 aDestPath += m_aDestPath;

		SiSubfileList const& sfList = m_pFile->GetSubfileList();

		for (USHORT i=0; i<sfList.Count(); i++)
		{
			SiDirEntry aFile  = aDestPath;
					 aFile += sfList.GetObject(i)->aFilename;

			if (!aFile.Exists())
				return FALSE; // min. eine Datei fehlt
		}

		return TRUE; // alles vorhanden
	}
	else // eine Zieldatei
		return SiTransferAction::CheckTarget(Env);
}

BOOL SiUnzipAction::Execute(SiEnvironment& Env)
{
	SiDirEntry aBigModeTemp;
	if( Env.IsBigMode() )
	{
		aBigModeTemp = Env.GetStartPath();
		aBigModeTemp += m_aSourceFile;

		if( !Env.GetArchive() )
			GetAgenda()->GetCallback()->RequestBigFile( m_pDataCarrier->GetName() );

		BOOL bSucc = Env.GetArchive()->GetFile( m_aSourceFile.GetBuffer(), Env.GetStartPath().GetBuffer() );
		if( !bSucc )
			return TRUE;
	}
	else
	{
		if( m_pDataCarrier != NULL &&
			!GetAgenda()->RequestDisk( m_aArchiveFile.Len() != 0 ? m_aArchiveFile : m_aSourceFile,
			m_pDataCarrier->GetDiskNo(), m_pDataCarrier->GetName()) )
		return TRUE;
	}

	SiDirEntry aSource;
	if (m_pDataCarrier != NULL)
	{
		// Entpacken vom Datentraeger
		if( Env.IsBigMode() )
			aSource  = Env.GetStartPath();
		else
			aSource  = Env.GetSourcePath();

		aSource += m_aSourcePath;

		// Eine Datei aus einem Archiv extrahieren
		if (m_aArchiveFile.Len() != 0)
			aSource += m_aArchiveFile;
	}
	else
	{
		// Entpacken im Zielverzeichnis
		aSource  = Env.GetDestPath();
		aSource += m_aDestPath;
	}

	if (m_aArchiveFile.Len() == 0)
		aSource += m_aSourceFile;

	ByteString     aSourceStr = aSource.GetFull();

	SiDirEntry   aDest      = Env.GetDestPath();
			   aDest     += m_aDestPath;
	ByteString     aDestStr   = aDest.GetFull();

	if (m_Flags.bCheckVersion)
	{
		#if defined WIN || defined WNT
			SiDirEntry aDestFile  = aDest;
					 aDestFile += m_aDestFile;

			if (!WinOS::IsSourceFileNewerVersion(aSourceStr,TRUE/*PACKED*/,aDestFile.GetFull()))
			{
				if( Env.IsBigMode() )
					aBigModeTemp.Kill();
				return TRUE; // nicht ueberschreiben
			}
		#else
			DBG_ERROR("SiUnzipAction::Execute() cannot check version");
		#endif
	}

	// ins Zielverzeichnis wechseln
	BOOL bCWD = aDest.SetCWD();

	GetLogfile().Success(bCWD)
		<< "cd    "
		<< aDestStr
		<< endl;

	// wenn es um ein Archive geht muss die SubFileListe aufbaut werden und
	// event. alle Dateien schreibbar machen.
	if( m_Flags.bArchive )
	{
		ByteString aPattern;

		if (m_aArchiveFile.Len() != 0)
			aPattern = ((const SiDirEntry&)(SiDirEntry(m_aSourceFile) + SiDirEntry(OS::GetAllFilesWildCard()))).GetFull();

		SiZipFile aZipFile;
		BOOL bSuccess = aZipFile.AddSubfileListTo( aSourceStr, m_pFile, aPattern.GetBuffer() );
	}

	BOOL bIsSpecialWNTSystemFile = FALSE;
	BOOL bTargetExists = CheckTarget(Env);

	if( m_Flags.bDontOverwrite &&  bTargetExists )
		return TRUE;

	if( m_Flags.bOverwriteOnly && !bTargetExists )
		return TRUE;

	BOOL bDoDontOverwriteUnzip = FALSE;
	BOOL bExplicitArchiveUnzip = FALSE;
	// wenn es Datei schon gibt, diese zuerst schreibbar machen.
	if( !m_Flags.bArchive  )
	{
		SiDirEntry aDestFile  = aDest;
				 aDestFile += m_aDestFile;

		#ifdef WNT
		if( m_aDestFile.CompareIgnoreCaseToAscii("trayhook.dll") == COMPARE_EQUAL ||
			m_aDestFile.CompareIgnoreCaseToAscii("sointgr.exe") == COMPARE_EQUAL )
			bIsSpecialWNTSystemFile = TRUE;
		#endif

		if( aDestFile.Exists() && !bIsSpecialWNTSystemFile )
		{
			if( GetAgenda()->GetCallback()->OverwriteFile(aDestFile.GetFull(), m_pFile) )
			{
				OS::MakeWritable(aDestFile.GetFull());
				aDestFile.Kill();
			}
			else
			{
				GetAgenda()->AddCanceledFileSize( m_pFile );
				return TRUE;
			}
		}
	}
	else
	{
		for( USHORT i=0; i < m_pFile->GetSubfileList().Count(); i++ )
		{
			SiDirEntry aFile( aDest );
			aFile += m_pFile->GetSubfileList().GetObject(i)->aFilename;
			if( aFile.Exists() )
			{
				bExplicitArchiveUnzip = TRUE;
				if( Env.IsUpdateOldVersion() || Env.IsUpgradeOldVersion() )
					bDoDontOverwriteUnzip = TRUE;
				break;
			}
		}
	}

	SiZipFile	aZipFile;
	ByteString	aExtractName;
	int			anError = 0;
	aZipFile.SetProgressHdl(LINK(GetAgenda(),SiAgenda,UnzipProgressHdl));

	if( bExplicitArchiveUnzip )
	{
		for( USHORT i=0; i < m_pFile->GetSubfileList().Count(); i++ )
		{
			SiSubfile* pSubFile = m_pFile->GetSubfileList().GetObject(i);
			if( !pSubFile )
				continue;

			SiDirEntry aFile( aDest );
			aFile += pSubFile->aFilename;

			BOOL bDoUnzip = TRUE;
			FileStat aFileStat(aFile);
			if( aFile.Exists() && aFileStat.GetKind() == FSYS_KIND_FILE )
			{
				if( ! GetAgenda()->GetCallback()->OverwriteFile(aFile.GetFull(), m_pFile) )
				{
					bDoUnzip = FALSE;
					GetAgenda()->AddCanceledFileSize( (ULONG)pSubFile->nSize, TRUE );
					GetLogfile().Success(anError==0) << "unzip (cancel by user) " << aFile.GetFull() << endl;
				}
				else
				{
					OS::MakeWritable(aFile.GetFull());
					aFile.Kill();
				}
			}

			if( bDoUnzip )
			{
				anError = 0;
				if( Env.IsVirtualMode() )
				{
					TouchVirtual( pSubFile->aFilename );
					GetAgenda()->AddCanceledFileSize( (ULONG)pSubFile->nSize, TRUE );
				}
				else
					anError = aZipFile.Unzip( aSourceStr, pSubFile->aFilename, bDoDontOverwriteUnzip );

				GetLogfile().Success(anError==0) << "unzip (explicit) " << aSourceStr << SEP << aFile.GetFull() << endl;
			}
		}
	}
	else if( m_aArchiveFile.Len() != 0 )
	{
		SiDirEntry aTmp( m_aSourceFile );
		aTmp += ByteString( OS::GetAllFilesWildCard() );
		aExtractName = aTmp.GetFull();
	}
	else
	{
		if( m_pFile->IsSetupZip() && Env.GetInstallType() == IT_CHANGE )
			aExtractName = m_aDestFile;
		else
			aExtractName = OS::GetAllFilesWildCard();
	}

	if( Env.IsVirtualMode() )
	{
		if( m_pFile->IsArchive() )
		{
			for( USHORT i = 0; i < m_pFile->GetSubfileList().Count(); i++ )
			{
				SiSubfile* pSubFile = m_pFile->GetSubfileList().GetObject(i);
				TouchVirtual( pSubFile->aFilename );
			}
		}
		else
		{
			ByteString aTemp( m_pFile->GetName() );
			TouchVirtual( aTemp );
		}
		GetAgenda()->AddCanceledFileSize( m_pFile, TRUE );
	}
	else
		anError = aZipFile.Unzip( aSourceStr, aExtractName, bDoDontOverwriteUnzip );

#ifdef OS2
	/*************************
	 *		Fehler OS2
	 *************************/
	if( !m_Flags.bArchive && anError != 0 )
	{
		SiDirEntry aCurDir( "." );
		aCurDir.ToAbs();

		SiDirEntry aFooWPS( SiDirEntry("fo*") );
		SiDirEntry aTmpWPS( aFooWPS.TempName().GetPath() );
		aTmpWPS.ToAbs();
		BOOL bSuccCWD = aTmpWPS.SetCWD();

		if( bSuccCWD )
		{
			anError = aZipFile.Unzip(aSourceStr, OS::GetAllFilesWildCard() );
			aCurDir.SetCWD();

			SiDirEntry aSrcWPS( aTmpWPS.GetFull() );
			aSrcWPS += m_aDestFile;
			aSrcWPS.ToAbs();

			SiDirEntry aDestWPS( aDestStr );
			aDestWPS += m_aDestFile;
			aDestWPS.ToAbs();

			BOOL bErr = Os2OS::ReplaceModule( aDestWPS.GetFull(), aSrcWPS.GetFull() );
			aSrcWPS.Kill();

			GetLogfile().Success(bErr) << "OS2 DosReplaceModule " << aDestWPS.GetFull() << SEP << aSrcWPS.GetFull() << endl;
		}
	}
	else
#elif WNT
	/*************************
	 *		Fehler WNT
	 *************************/
	if( bIsSpecialWNTSystemFile &&
		anError != 0  )
	{
		SiDirEntry aCurDir( "." );
		aCurDir.ToAbs();

		SiDirEntry aFoo( SiDirEntry("fo*") );
		SiDirEntry aTmp( aFoo.TempName().GetPath() );
		aTmp.ToAbs();
		BOOL bSuccCWD = aTmp.SetCWD();

		if( bSuccCWD )
		{
			anError = aZipFile.Unzip( aSourceStr, OS::GetAllFilesWildCard() );
			aCurDir.SetCWD();

			aTmp += m_aDestFile;

			SiDirEntry _Destination( aDestStr );
			_Destination += m_aDestFile;

			SiDirEntry _TempDestination( _Destination );
			_TempDestination.SetExtension( UniString::CreateFromAscii("~") );

			FileCopier aFileCopier( aTmp, _TempDestination );
			aFileCopier.Execute();

			MoveSystemFileAfterReboot( _TempDestination.GetFull(), _Destination.GetFull() );
			SetDateTime( _TempDestination.GetFull() );
			Env.SetReboot(TRUE);

			aTmp.Kill();
		}
		anError = 0;
	}
	else
#endif
	{
		GetLogfile().Success(anError==0) << "unzip " << aSourceStr << SEP << aDestStr;
	}

	// (optional) Datei stempeln und unter Unix evtl. Rechte setzen
	if (!m_Flags.bArchive)
	{
		SiDirEntry aFrom(aDest);
		SiDirEntry aTo(aDest);
		aFrom   += m_aDestFile.ToLowerAscii();
		aTo     += m_aDestFile;
		RenameFile( aFrom.GetFull(), aTo.GetFull() );	// Grosskleinschreibung unter W32 und UNX

		aDestStr = aTo.GetFull();

		if( m_Flags.bDoMigration )
			DoMigration( aDestStr, Env );
		if( m_Flags.bSubstitute )
			Substitute( aDestStr, Env );

		SetDateTime(aDestStr);
		SetUnixRights(aDestStr);
		SetOs2Creator(aDestStr, Env);
	}
	else // dann gibt es mehrere Dateien
	{
		for (USHORT i=0; i<m_pFile->GetSubfileList().Count(); i++)
		{
			SiDirEntry aFile  = aDest;
					 aFile += m_pFile->GetSubfileList().GetObject(i)->aFilename;
			ByteString   aFileStr = aFile.GetFull();

			#ifdef UNX
			// Verzeichnisse duerfenn unter UNX nicht das
			// Recht gesetzt bekommen!
			FileStat aStat( aFile );
			if( aStat.GetKind() != FSYS_KIND_DIR )
				SetUnixRights(aFileStr,i==0);
			#endif

			if( m_Flags.bDoMigration )
				DoMigration( aFileStr, Env );
			if( m_Flags.bSubstitute )
				Substitute( aFileStr, Env );

			SetDateTime(aFileStr,i==0);
			SetOs2Creator(aFileStr, Env,i==0);
		}
	}

	// UnoComponent
	if( m_pFile && m_pFile->IsUnoComponent() )
		GetAgenda()->AddUnoComponent( m_pFile );

	if( Env.IsBigMode() )
		aBigModeTemp.Kill();

	if( anError != 0 )
		GetLogfile() << " Error = " << ByteString::CreateFromInt32(anError);

	GetLogfile() << endl;

	return SetSuccess(anError == 0 && bCWD);
}

// AppendAction ------------------------------------------------
//
SiAppendAction::SiAppendAction
(
	SiAgenda    * pAgenda,
	ByteString const& aSourcePath,
	ByteString const& aSourceFile,
	ByteString const& aDestPath,
	ByteString const& aDestFile,
	SiFile const* pBaseFile,
	int           nPart
)
: SiAction(pAgenda)
{
	m_aSourcePath  = aSourcePath;
	m_aSourceFile  = aSourceFile;
	m_aDestPath    = aDestPath;
	m_aDestFile    = aDestFile;

	m_pBaseFile    = pBaseFile; // zum Sortieren
	m_nPart        = nPart;
}

BOOL SiAppendAction::Execute(SiEnvironment& Env)
{
	SiDirEntry aAbsPath1 = SiDirEntry(Env.GetDestPath()) + SiDirEntry(m_aSourcePath);
	SiDirEntry aAbsPath2 = SiDirEntry(Env.GetDestPath()) + SiDirEntry(m_aDestPath);
	ByteString   aSource   = ((const SiDirEntry&)(aAbsPath1+SiDirEntry(m_aSourceFile))).GetFull();
	ByteString   aDest     = ((const SiDirEntry&)(aAbsPath2+SiDirEntry(m_aDestFile))).GetFull();
	SiDirEntry aDestDir  = aDest;

	// Ausfuehrung wird ueber Abhaengigkeiten geregelt
	//
	// if (DoRecoverOnly()
	// &&  aDestDir.Exists()) // Zielabfrage macht bei Append keinen Sinn
	//     return TRUE;

	SvFileStream  aReadStream;
	SvFileStream  aWriteStream;

	aReadStream.Open( UniString::CreateFromAscii(aSource.GetBuffer()), STREAM_STD_READ);
	aWriteStream.Open( UniString::CreateFromAscii(aDest.GetBuffer()), STREAM_STD_WRITE);

	// Pruefen, ob Dateien offen sind und Zieldatei
	// ueberschreibbar ist
	//
	if (!aWriteStream.IsOpen())
	{
		GetLogfile()
			.Success(FALSE)
			<< "append " << aSource << SEP << aDest
			<< " Error(cannot open for writing)" << endl;
		return SetSuccess(FALSE);
	}

	if (!aWriteStream.IsWritable())
	{
		GetLogfile()
			.Success(FALSE)
			<< "append " << aSource << SEP << aDest
			<< " Error(file is write protected)" << endl;
		return SetSuccess(FALSE);
	}

	if (!aReadStream.IsOpen())
	{
		GetLogfile()
			.Success(FALSE)
			<< "append " << aSource << SEP << aDest
			<< " Error(cannot open for reading)" << endl;
		return SetSuccess(FALSE);
	}

	// Beide Dateien konnten geoeffnet werden.
	// Jetzt Daten anhaengen
	//
	aWriteStream.Seek(STREAM_SEEK_TO_END);              // Append
	ULONG lToDo = aReadStream.Seek(STREAM_SEEK_TO_END); // Laenge ermitteln
	aReadStream.Seek(STREAM_SEEK_TO_BEGIN);

	while (lToDo > 0
	   &&  aReadStream.GetError() == 0
	   &&  aWriteStream.GetError() == 0)
	{
		char Buffer[100];
		ULONG nTransfer = Min((ULONG)sizeof(Buffer),lToDo);
		aReadStream.Read(Buffer,nTransfer);
		aWriteStream.Write(Buffer,nTransfer);
		lToDo -= nTransfer;
	}

	GetLogfile()
		.Success(aReadStream.GetError()==0 && aWriteStream.GetError()==0)
		<< "append " << aSource << SEP << aDest;

	// Ausgabe eines evtl Fehlercodes
	//
	if (aReadStream.GetError() != 0)
	{
		GetLogfile() << " read error " << aReadStream.GetError() << endl;
		return SetSuccess(FALSE);
	}

	if (aWriteStream.GetError() != 0)
	{
		GetLogfile() << " write error " << aWriteStream.GetError() << endl;
		return SetSuccess(FALSE);
	}

	GetLogfile() << endl;
	return SetSuccess(TRUE);
}

// InstallFontAction ------------------------------------------------
//
SiInstallFontAction::SiInstallFontAction
(
	SiAgenda       * pAgenda,
	ByteString  const& aFontName,
	ByteString  const& aFontFile  // ohne Pfad
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFontName = aFontName;
	m_aFontFile = aFontFile;
}

BOOL SiInstallFontAction::CheckTarget(SiEnvironment const&) const
{
	return OS::IsFontInstalled(m_aFontFile,m_aFontName);
}

BOOL SiInstallFontAction::Execute(SiEnvironment&)
// pre/post:  siehe OS::InstallFont()
{
	BOOL bSuccess = OS::InstallFont(m_aFontFile,m_aFontName);

	#ifdef MAC
	GetAgenda()->ForceRestart();
	#endif

	GetLogfile()
		.Success(bSuccess)
		<< "install font " << m_aFontName << SEP
		<< m_aFontFile     << endl;

	return SetSuccess(bSuccess);
}

// ProfileItemAction ------------------------------------------------
//
SiProfileItemAction::SiProfileItemAction
(
	SiAgenda       * pAgenda,
	PIA              ePIA,
	SiProfile const* pProfile,
	ByteString    const& aSection,
	ByteString    const& aKey,
	ByteString    const& aValue,
	ByteString    const& aWorkstationValue,
	int				 nOrder,
	BOOL			 bInstInf,
	SiProfileItem*   pItem
)
: SiAction(pAgenda, ePIA == PIA_CREATE ? AT_REGISTER : AT_UNREGISTER)
{
	m_pProfile			= (SiProfile*)pProfile;
	m_pProfileItem		= pItem;
	m_ePIA				= ePIA;
	m_aSection			= aSection;
	m_aKey				= aKey;
	m_aValue			= aValue;
	m_aWorkstationValue = aWorkstationValue;
	m_nOrder   			= nOrder;
	m_bInstallInfo		= bInstInf;
}

BOOL SiProfileItemAction::Execute(SiEnvironment& rEnv)
{
	SiDirEntry aFile = SiDirEntry(rEnv.GetDestPath())
		   + SiDirEntry(m_pProfile->GetDirectory()->GetName())
		   + SiDirEntry(m_pProfile->GetName());

	// InstallInfoItem nicht bei Netinstallation, AppServer und AppServerClient
	// schreiben.
	if( m_bInstallInfo )
		if( (GetInstallMode() == IM_NETWORK) || rEnv.IsAutoWorkstation() )
			return TRUE;

	Config* pConfig = GetAgenda()->GetOpenProfile( aFile.GetFull() );
	pConfig->SetGroup(m_aSection);

	switch (m_ePIA)
	{
		case PIA_CREATE:
		{
			if (DoRecoverOnly()
			&&  pConfig->ReadKey(m_aKey,"__empty__") != "__empty__")
				return TRUE;

			ByteString aActValue;
			BOOL   bUseWrkValue = FALSE;
			if( GetInstallMode() == IM_WORKSTATION && m_aWorkstationValue.Len() )
			{
				bUseWrkValue = TRUE;
				aActValue = m_aWorkstationValue;
			}
			else
				aActValue = m_aValue;

			// Templatefelder im Value ersetzen
			BOOL	bOpenSubs = FALSE;
			BOOL 	bSubst = FALSE;
			USHORT	nBeginTpl = aActValue.Search('<');

			while( nBeginTpl != STRING_NOTFOUND )
			{
				USHORT nNested = 0;
				USHORT nEndTpl = STRING_NOTFOUND;
				for( USHORT i = nBeginTpl + 1; i <= aActValue.Len(); ++i )
				{
					if( aActValue.GetChar(i) == '<' )
						nNested++;
					if( aActValue.GetChar(i) == '>' )
						if( nNested )
							nNested--;
						else
						{
							nEndTpl = i;
							break;
						}
				}

				if( nEndTpl != STRING_NOTFOUND )
				{
					ByteString aTpl = aActValue.Copy( nBeginTpl, nEndTpl-nBeginTpl+1 );

					if( (rEnv.IsUpdateOldVersion() || rEnv.IsUpgradeOldVersion()) &&
					   (aTpl.CompareIgnoreCaseToAscii("<useraddress>") == COMPARE_EQUAL ||
						aTpl.CompareIgnoreCaseToAscii("<userfirstname>") == COMPARE_EQUAL ||
						aTpl.CompareIgnoreCaseToAscii("<username>") == COMPARE_EQUAL ||
						aTpl.CompareIgnoreCaseToAscii("<userid>") == COMPARE_EQUAL ||
						aTpl.CompareIgnoreCaseToAscii("<customernr>") == COMPARE_EQUAL) )
						return TRUE;

					ByteString aSub = GetAgenda()->QueryTemplateValue(aTpl);
					aActValue.SearchAndReplace( aTpl, aSub );
					bSubst = TRUE;
				}
				nBeginTpl = aActValue.Search( '<', nBeginTpl+1 ); // weitersuchen
			}
			if( bSubst && aActValue.Len() )
				m_pProfileItem->SetProperty( bUseWrkValue? PROPERTY_WORKSTATIONVALUE : PROPERTY_VALUE, aActValue );

			// Eintrag setzen
			if( rEnv.IsUpdateOldVersion() || rEnv.IsUpgradeOldVersion() )
			{
				ByteString aVal = pConfig->ReadKey(m_aKey);
				if( aVal != aActValue )
					pConfig->WriteKey( m_aKey, aActValue );
			}
			else
				pConfig->WriteKey( m_aKey, aActValue );

			#ifdef UNX
			UnixOS::SetRights(aFile.GetFull(), m_pProfile->GetUnixRights());
			#endif

			GetLogfile().Success(TRUE)
				<< "set profile item: " << ByteString(pConfig->GetPathName(), osl_getThreadTextEncoding()) << SEP
				<< m_aSection << SEP << m_aKey  << SEP << aActValue << endl;
			break;
		}

		case PIA_REMOVE:
			if( rEnv.IsUpdateOldVersion() || rEnv.IsUpgradeOldVersion() )
				return TRUE;

			if (aFile.Exists())     // damit keine 0-byte Datei erzeugt wird
				pConfig->DeleteKey(m_aKey);
			GetLogfile()
				.Success(TRUE)
				<< "remove profile item: " << ByteString(pConfig->GetPathName(), osl_getThreadTextEncoding()) << SEP
			<< m_aSection << SEP << m_aKey << endl;
			break;

		default:
			DBG_ERROR("ProfileItemAction::Execute(): unknown action");
			return FALSE;
	}

	return SetSuccess(TRUE);
}

// SiStarRegistryAction ----------------------------------------------

SiStarRegistryAction::SiStarRegistryAction( SiAgenda* pAgenda,
	PIA eActType, const SiStarRegistryItem* pItem ) :
	SiAction( pAgenda, eActType == PIA_CREATE ? AT_REGISTER : AT_UNREGISTER )
{
	m_pItem = pItem;
	m_ePIA = eActType;
}

void SiStarRegistryAction::ReplaceMeta( ByteString& rValue )
{
	USHORT nBeginTpl = rValue.Search('<');
	while( nBeginTpl != STRING_NOTFOUND )
	{
		USHORT nEndTpl = rValue.Search( '>', nBeginTpl+1 );
		if( nEndTpl != STRING_NOTFOUND )
		{
			ByteString aTpl = rValue.Copy( nBeginTpl, nEndTpl-nBeginTpl+1 );
			ByteString aSub = GetAgenda()->QueryTemplateValue( aTpl );
			rValue.SearchAndReplace( aTpl, aSub );
		}
		nBeginTpl = rValue.Search( '<', nBeginTpl+1 );
	}
}

BOOL SiStarRegistryAction::Execute(SiEnvironment& rEnv)
{
	SiDirEntry aRegistryFile( rEnv.GetDestPath() );
	aRegistryFile += m_pItem->GetRegistry()->GetDir()->GetName();
	aRegistryFile += m_pItem->GetRegistry()->GetName();
	aRegistryFile.ToAbs();

	RegistryLoader* static_RegLoader = SiHelp::GetStaticRegistryLoader(rEnv);
	if( !static_RegLoader )
	{
		GetLogfile().Success(FALSE) << "star registry: can't load Registry-DLL" << endl;
		return FALSE;
	}

	BOOL bOpen = FALSE;
	Registry aRegistry( *static_RegLoader );
	if( !aRegistryFile.Exists() &&
		aRegistry.create(UniString::CreateFromAscii(aRegistryFile.GetFull().GetBuffer())) == REG_NO_ERROR )
		bOpen = TRUE;
	else if( aRegistry.open(UniString::CreateFromAscii(aRegistryFile.GetFull().GetBuffer()), REG_READWRITE) == REG_NO_ERROR )
		bOpen = TRUE;

	if( !bOpen )
	{
		GetLogfile().Success(FALSE) << "star registry: can't open/create Registryfile '" <<
			aRegistryFile.GetFull() << "'" << endl;
		return FALSE;
	}

	RegistryKey aRootKey;
	if( aRegistry.openRootKey(aRootKey) != REG_NO_ERROR )
	{
		GetLogfile().Success(FALSE) << "star registry: can't open ROOT-Key for '" <<
			aRegistryFile.GetFull() << "'" << endl;
		return FALSE;
	}

	ByteString aActValue;
	::rtl::OUString aEmpty;
	switch( m_ePIA )
	{
		case PIA_CREATE :
		{
			RegistryKey aKey;
			if( aRootKey.createKey(UniString::CreateFromAscii(m_pItem->GetKey().GetBuffer()),
				aKey) != REG_NO_ERROR )
			{
				GetLogfile().Success(FALSE) << "star registry: can't open Key '" <<
					m_pItem->GetKey() << "' in '" << aRegistryFile.GetFull() << "'" << endl;
				return FALSE;
			}

			if( m_pItem->HasNumValue() || m_pItem->HasStrValue() || m_pItem->HasSeqValue() )
			{
				RegValueType	eType;
				RegValue		pValue = NULL;
				UINT32 			nValueLen = 0;
				long			nVal;
				BOOL 			bErr = FALSE;

				if( m_pItem->HasNumValue() )
				{
					eType = RG_VALUETYPE_LONG;
					nVal = m_pItem->GetNumValue();
					pValue = (RegValue) &nVal;
					nValueLen = sizeof(long);

					bErr = aKey.setValue(aEmpty, eType, pValue, nValueLen) != REG_NO_ERROR;
				}
				else if( m_pItem->HasStrValue() )
				{
					eType = RG_VALUETYPE_STRING;
					aActValue = m_pItem->GetStrValue();

					ReplaceMeta( aActValue );

					pValue = (RegValue) aActValue.GetBuffer();
					nValueLen = aActValue.Len() + 1;

					bErr = aKey.setValue(aEmpty, eType, pValue, nValueLen) != REG_NO_ERROR;
				}
				else if( m_pItem->HasSeqValue() )
				{
					eType = RG_VALUETYPE_STRINGLIST;

					ByteString aSeqValue( m_pItem->GetSeqValue() );
					USHORT nDelimTok = 0;
					USHORT nTokCount = aSeqValue.GetTokenCount( '|' );

					sal_Char** pList = new sal_Char * [nTokCount];

					for( USHORT i = 0; i < nTokCount; ++i )
					{
						ByteString aActValue( aSeqValue.GetToken(0, '|', nDelimTok) );
						ReplaceMeta( aActValue );

						pList[i] = new sal_Char[ aActValue.Len() + 1 ];
						strcpy( pList[i], aActValue.GetBuffer() );
					}

					bErr = aKey.setStringListValue(aEmpty	, pList, nTokCount) != REG_NO_ERROR;
					for( USHORT x = 0; x < nTokCount; ++x )
						delete pList[x];
					delete pList;
				}

				if( bErr )
				{
					GetLogfile().Success(FALSE) << "star register: can't set Key/Value '" <<
						m_pItem->GetKey() << " == " <<
						((eType==RG_VALUETYPE_LONG)? ByteString(m_pItem->GetNumValue()) :
						(eType==RG_VALUETYPE_STRINGLIST)? ByteString(m_pItem->GetSeqValue()) :
						m_pItem->GetStrValue()) << "' in '" << aRegistryFile.GetFull() << "'" << endl;
					return FALSE;
				}
				else
				{
					GetLogfile().Success(TRUE) << "star register: set Key/Value '" <<
						m_pItem->GetKey() << " == " <<
						((eType==RG_VALUETYPE_LONG)? ByteString(m_pItem->GetNumValue()) : m_pItem->GetStrValue()) <<
						"' in '" << aRegistryFile.GetFull() << "'" << endl;
				}
			}
			else
			{	// wenn nur der blanke Key registriert wurde!
				GetLogfile().Success(TRUE) << "star register: set Key '" << m_pItem->GetKey() <<
					"' in '" << aRegistryFile.GetFull() << "'" << endl;
			}
		} break;

		case PIA_REMOVE :
		{
			if( aRootKey.deleteKey(UniString::CreateFromAscii(m_pItem->GetKey().GetBuffer())) != REG_NO_ERROR )
			{
				GetLogfile().Success(FALSE) << "star deregister: can't delete Key '" <<
					m_pItem->GetKey() << "' in '" << aRegistryFile.GetFull() << "'" << endl;
				return FALSE;
			}
			else
			{
				GetLogfile().Success(TRUE) << "star deregister: delete Key '" << m_pItem->GetKey() <<
					"' in '" << aRegistryFile.GetFull() << "'" << endl;
			}
		} break;
	}

	return TRUE;
}

// SiStarRegistryAction ----------------------------------------------
//
SiConfigurationAction::SiConfigurationAction( SiAgenda* pAgenda, PIA eActType, const SiConfigurationItem* pItem ) :
	SiAction( pAgenda, eActType == PIA_CREATE ? AT_REGISTER : AT_UNREGISTER )
{
	m_pItem = pItem;
	m_ePIA = eActType;
}

BOOL SiConfigurationAction::Execute(SiEnvironment& rEnv)
{
	Reference< XInterface > xIFace;
	Sequence< Any > aArgs(1);
	aArgs[0] <<= ::rtl::OUString::createFromAscii( m_pItem->GetPath().GetBuffer() );

	try
	{
		xIFace = Reference< XInterface >( xCfgProvider->createInstanceWithArguments(
			::rtl::OUString::createFromAscii("com.sun.star.configuration.ConfigurationUpdateAccess"),
			aArgs) );
	}
	catch(RuntimeException& e) {
		GetLogfile().Success(FALSE) << ByteString(UniString(e.Message), osl_getThreadTextEncoding());
		return FALSE;
	}
	catch(Exception& e) {
		GetLogfile().Success(FALSE) << ByteString(UniString(e.Message), osl_getThreadTextEncoding());
		return FALSE;
	}

	Any anyValue;
	switch( m_pItem->GetValueType() )
	{
		case SiConfigurationItem::BOOLEAN:
		{
			sal_Bool b = m_pItem->GetValue().CompareIgnoreCaseToAscii("true") == COMPARE_EQUAL ||
						 m_pItem->GetValue().ToInt32() == 1 ? sal_True : sal_False;
			anyValue.setValue( &b, ::getBooleanCppuType() );
		} break;

		case SiConfigurationItem::STRING:
		{
			ByteString aActValue( GetInstallMode() == IM_WORKSTATION && m_pItem->GetWorkstationValue().Len()?
								  m_pItem->GetWorkstationValue() : m_pItem->GetValue() );
			USHORT	nBeginTpl = aActValue.Search('<');
			while( nBeginTpl != STRING_NOTFOUND )
			{
				USHORT nNested = 0;
				USHORT nEndTpl = STRING_NOTFOUND;
				for( USHORT i = nBeginTpl + 1; i <= aActValue.Len(); ++i )
				{
					if( aActValue.GetChar(i) == '<' )
						nNested++;
					if( aActValue.GetChar(i) == '>' )
						if( nNested )
							nNested--;
						else
						{
							nEndTpl = i;
							break;
						}
				}
				if( nEndTpl != STRING_NOTFOUND )
				{
					ByteString aTpl = aActValue.Copy( nBeginTpl, nEndTpl-nBeginTpl+1 );
					ByteString aSub = GetAgenda()->QueryTemplateValue(aTpl);
					aActValue.SearchAndReplace( aTpl, aSub );
				}
				nBeginTpl = aActValue.Search( '<', nBeginTpl+1 ); // weitersuchen
			}
			::rtl::OUString s = UniString(aActValue, osl_getThreadTextEncoding());
			anyValue.setValue( &s, ::getCppuType((const ::rtl::OUString*)0) );
		} break;

		case SiConfigurationItem::NUMERIC:
		{
			sal_Int32 n = m_pItem->GetValue().ToInt32();
			anyValue.setValue( &n, ::getCppuType((const sal_Int32*)0) );
		} break;
	}


	Reference< XNameReplace > xReplaceAccess(xIFace, UNO_QUERY);
	if( !xReplaceAccess.is() )
	{
		return FALSE;
	}
	try
	{
	xReplaceAccess->replaceByName( ::rtl::OUString::createFromAscii(m_pItem->GetKey().GetBuffer()), anyValue );
	}
	catch(Exception& e)
	{
		GetLogfile().Success(FALSE) << ByteString(UniString(e.Message), osl_getThreadTextEncoding());
		return FALSE;
	}

	Reference< XChangesBatch > xChangesAccess(xIFace, UNO_QUERY);
	if( !xChangesAccess.is() )
	{
		return FALSE;
	}

	try
	{
		xChangesAccess->commitChanges();
	}
	catch(RuntimeException& e) {
		GetLogfile().Success(FALSE) << ByteString(UniString(e.Message), osl_getThreadTextEncoding());
		return FALSE;
	}
	catch(Exception& e) {
		GetLogfile().Success(FALSE) << ByteString(UniString(e.Message), osl_getThreadTextEncoding());
		return FALSE;
	}

	return TRUE;
}


// SiCustomAction ----------------------------------------------------
//
static SiCustomAction* static_pCustomAction = NULL;

SiCustomAction::SiCustomAction( SiAgenda* pAgenda, SiModule* pRoot,
	const ByteString& aDllName, const ByteString& aExePath, BOOL bInst, BOOL bPre, SiCustom* pCustom ) :
	SiAction(pAgenda, AT_CUSTOM)
{
	m_pCustom			= pCustom;
	m_aDllName			= aDllName;
	m_aExePath			= aExePath;
	m_bInstallation 	= bInst;
	m_bPre				= bPre;
	m_pCustomDll		= NULL;
	m_bKeepAlive 		= FALSE;

	pModuleInfoList		= NULL;
	pRootModule			= pRoot;
	bInternalError		= FALSE;
}

SiCustomAction::~SiCustomAction()
{
	if(pModuleInfoList)
	{
		for( USHORT i = 0; i < pModuleInfoList->Count(); ++i )
			delete pModuleInfoList->GetObject(i);
		delete pModuleInfoList;
	}

	static_pCustomAction = NULL;
	if( m_pCustomDll )
		delete m_pCustomDll;

	if( m_aTempFile.Len() )
	{
		SiDirEntry aEntry(m_aTempFile);
		aEntry.Kill();
		m_aTempFile = "";
	}
}

void SiCustomAction::AddModuleInfo(SiModule* pModule)
{
	SiCustomModuleInfo* pNew = new SiCustomModuleInfo;

	pNew->SetModuleID(pModule->GetID());
	pNew->SetModuleName(pModule->GetName());
	if(pModule->IsSelected())
		pNew->SelectModule();
	else
		pNew->DeselectModule();

	if(!pModuleInfoList)
		pModuleInfoList = new SiCustomModuleList;
	pModuleInfoList->Insert(pNew, LIST_APPEND);

	const SiModuleList* pModuleList = pModule->GetModuleList();
	for( USHORT x = 0; x < pModuleList->Count(); ++x )
		AddModuleInfo( pModuleList->GetObject(x) );
}

void _SV_CALL ShowSetup()
{
	static_pCustomAction->GetAgenda()->GetWindowHideLink().Call( static_pCustomAction );
}

void _SV_CALL HideSetup()
{
	static_pCustomAction->GetAgenda()->GetWindowHideLink().Call( 0 );
}

ByteString _SV_CALL GetCodeFilename()
{
	ByteString aFilename;
	if( static_pCustomAction )
		aFilename = static_pCustomAction->WriteCodeFile();
	return aFilename;
}

BOOL SiCustomAction::Execute( SiEnvironment& rEnv )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	bInternalError	= FALSE;
	m_aTempFile		= "";

	SiDirEntry aCustom( m_aExePath );
	aCustom += m_aDllName;
	if( !aCustom.Exists() )
	{
		aCustom = rEnv.GetStartPath();
		aCustom += m_aDllName;
		if( !aCustom.Exists() )
		{
#ifndef UNX
			bInternalError = TRUE;
			return TRUE;
#else
			aCustom = rEnv.GetStartPath();
			aCustom += ByteString("../lib/");
			aCustom += m_aDllName;
			if( !aCustom.Exists() )
			{
				bInternalError = TRUE;
				return TRUE;
			}
#endif
		}
	}

	if( !aCustom.Exists() )
	{
		bInternalError = TRUE;
		return TRUE;
	}

	SiDirEntry aOldDir(".");
	SiDirEntry aCwd( aCustom.GetPath() );
	aCwd.SetCWD();

	#ifdef OS2
	aCustom.CutExtension();
	#endif

	if( m_pCustomDll )
		delete m_pCustomDll;

	#ifdef UNX
	m_pCustomDll = new Library( aCustom.GetFullUni() );
	#else
	m_pCustomDll = new Library( UniString::CreateFromAscii(aCustom.GetName().GetBuffer()) );
	#endif

	if( !m_pCustomDll->ModulFound() ) {
		DBG_ASSERT( FALSE, "Konnte Custom DLL nicht laden" );
		aOldDir.SetCWD();
		delete m_pCustomDll;
		m_pCustomDll = NULL;
		bInternalError = TRUE;
		return TRUE;
	}

	ByteString aEntryPoint;
	#if defined(OS2) || defined(WIN)
	aEntryPoint = "_CustomInit";
	#else
	aEntryPoint = "CustomInit";
	#endif

	FncPtrCustomInit pInitFnc = (FncPtrCustomInit)
								m_pCustomDll->GetFunction( UniString::CreateFromAscii(aEntryPoint.GetBuffer()) );
	if( !pInitFnc ) {
		DBG_ASSERT( FALSE, "Konnte CustomInit Func nicht finden" );
		delete m_pCustomDll;
		m_pCustomDll = NULL;
		aOldDir.SetCWD();
		bInternalError = TRUE;
		return TRUE;
	}

	memset( &m_aCustomFnc, 0, sizeof(SiCustomFnc) );
	m_aCustomFnc.nVersion		  	= SICUSTOM_VERSION;

	m_aSetupFnc.nVersion			= SICUSTOM_VERSION;
	m_aSetupFnc.fncHideSetup		= (FncPtrHideSetup)HideSetup;
	m_aSetupFnc.fncShowSetup		= (FncPtrShowSetup)ShowSetup;
	m_aSetupFnc.fncGetCodeFilename	= (FncPtrGetCodeFilename)GetCodeFilename;

	static_pCustomAction = this;

	if( !pInitFnc(&m_aSetupFnc, &m_aCustomFnc) )
	{
		DBG_ASSERT( FALSE, "falsche Version der CustomDLL" );
		delete m_pCustomDll;
		m_pCustomDll = NULL;
		aOldDir.SetCWD();

		bInternalError = TRUE;
		static_pCustomAction = NULL;
		return TRUE;
	}

	m_aCustEnv.SetResponse( rEnv.IsResponsefileMode() );
	m_aCustEnv.SetResponseFileName( rEnv.GetResponseFilename() );
	m_aCustEnv.SetPreFlag( m_bPre );
	m_aCustEnv.SetInstallContext( m_bInstallation );
	m_aCustEnv.SetFirstInstallation( rEnv.IsFirstInstallation() );
	m_aCustEnv.SetStartPath( rEnv.GetStartPath() );
	m_aCustEnv.SetSourcePath( rEnv.GetSourcePath() );
	m_aCustEnv.SetDestPath( rEnv.GetDestPath() );
	m_aCustEnv.SetInstalledPath( rEnv.GetInstalledPath() );
	m_aCustEnv.SetEnvPtr( &rEnv );

	AddModuleInfo(pRootModule);
	BOOL bSucc = m_aCustomFnc.fncMain(&m_aCustEnv, pModuleInfoList);

	if( !m_bKeepAlive )
	{
		delete m_pCustomDll;
		m_pCustomDll = NULL;
		static_pCustomAction = NULL;

		if( m_aTempFile.Len() )
		{
			SiDirEntry aEntry(m_aTempFile);
			aEntry.Kill();
			m_aTempFile = "";
		}
	}

	aOldDir.SetCWD();
	return bSucc;
}

ByteString SiCustomAction::WriteCodeFile()
{
	SiDirEntry		aFoo( SiDirEntry("sic*") );
	SiDirEntry		aTemp( aFoo.TempName() );

	SvFileStream	aWriteStrm( aTemp.GetFullUni(), STREAM_WRITE );
	ByteString		aCode( m_pCustom->GetCode() );

	aTemp.ToAbs();

	aWriteStrm.Write( aCode.GetBuffer(), aCode.Len() );
	aWriteStrm.Close();

	m_aTempFile = aTemp.GetFull();
	return m_aTempFile;
}


// SimpleProcedureAction ---------------------------------------------
SimpleProcedureAction::SimpleProcedureAction( const ByteString& aProcName, const ByteString& aCode ) :
	m_aProcName	( aProcName ),
	m_aCode	( aCode )
{
}

BOOL SimpleProcedureAction::Execute(SiCompiledScript* pCS, SiEnvironment& rEnv)
{
	SiBasic aBasic(*pCS, rEnv);
	return aBasic.Call(m_aProcName,m_aCode);
}

// RunProcedureAction ------------------------------------------------
SiRunProcedureAction::SiRunProcedureAction
(
	SiAgenda       * pAgenda,
	ByteString    const& aProcName,
	ByteString    const& aCode,
	SiCompiledScript *pCS
)
: SiAction(pAgenda, AT_PROCEDURE)
{
	m_aProcName = aProcName;
	m_aCode     = aCode;
	m_pCS		= pCS;
}

BOOL SiRunProcedureAction::Execute(SiEnvironment& rEnv)
{
	BOOL bOk = FALSE;

	SiBasic aBasic(*m_pCS, rEnv);
	bOk = aBasic.Call(m_aProcName,m_aCode);

	return SetSuccess(bOk);
}

// ------------------------------------------------------------------------
// PLATTFORMABHAENGIGE FUNKTIONEN -----------------------------------------
// ------------------------------------------------------------------------

// MakeFolderAction -------------------------------------------------
//
#ifndef OS2

SiMakeFolderAction::SiMakeFolderAction
(
	SiAgenda      * pAgenda,
	ByteString const& aName     // Titel im Folder
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aName = aName;        // Titel im Folder
}

#else

SiMakeFolderAction::SiMakeFolderAction
(
	SiAgenda    * pAgenda,
	ByteString const& aName,        // Titel im Folder
	ByteString const& anOs2ID,      // <OFFICE_FOLDER>
	BOOL          bOs2NetIcon1,
	ByteString const& anOs2IconPath1,
	ByteString const& anOs2Icon1,
	BOOL          bOs2NetIcon2,
	ByteString const& anOs2IconPath2,
	ByteString const& anOs2Icon2
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aName          = aName;        // Titel im Folder
	m_anOs2ID        = anOs2ID;      // <OFFICE_FOLDER>
	m_bOs2NetIcon1   = bOs2NetIcon1;
	m_anOs2IconPath1 = anOs2IconPath1;
	m_anOs2Icon1     = anOs2Icon1;
	m_bOs2NetIcon2   = bOs2NetIcon2;
	m_anOs2IconPath2 = anOs2IconPath2;
	m_anOs2Icon2     = anOs2Icon2;
}

#endif

#if defined (WIN) || defined (WNT)

BOOL SiMakeFolderAction::Execute(SiEnvironment& rEnv)
{
	if( rEnv.IsAutoWorkstation() )
		return TRUE;
	SiDirEntry aEntry( WinOS::SHGetUserProgramFolder() );
	if( !aEntry.Exists() )
	{
		GetLogfile().Success(FALSE) << "SHGetUserProgramFolder not exists" << endl;
		return FALSE;
	}

	aEntry += m_aName;
	BOOL bSuccess = aEntry.MakeDir();

	GetLogfile().Success(bSuccess) << "make folder " << aEntry.GetFull()
		<< (bSuccess ? " OK" : " ERR") << endl;

	return bSuccess;

#ifdef OLD_FUCKING_CODE
/*
 *
	ByteString aCommand;

	aCommand  = "[CreateGroup(\"";
	aCommand += m_aName;
	aCommand += "\",0)]";

	BOOL bSuccess = WinOS::ExecDDECommand("progman", aCommand);
	GetLogfile()
		.Success(bSuccess)
		<< "dde   " << aCommand  << endl;
	return bSuccess;
 *
 */
#endif
}

#elif defined OS2

BOOL SiMakeFolderAction::Execute(SiEnvironment& rEnv)
{
	if( rEnv.IsAutoWorkstation() )
		return TRUE;

	if( DoRecoverOnly() &&  Os2OS::IsFolderInstalled(m_anOs2ID) )
		return TRUE;

	// Bei einer Aenderung der Installation den Folder
	// nur erzeugen, wenn es ihn noch nicht gibt.
	//
	if (!rEnv.IsFirstInstallation()
	&&  Os2OS::IsFolderInstalled(m_anOs2ID))
		return TRUE;

	// Icon-Pfade zusammensetzen
	//
	SiDirEntry anIconPath1;

	if (m_bOs2NetIcon1)
		anIconPath1 = rEnv.GetProgPath(GetInstallMode());
	else
		anIconPath1 = rEnv.GetWorkPath(GetInstallMode());

	anIconPath1 += m_anOs2IconPath1;
	anIconPath1 += m_anOs2Icon1;

	SiDirEntry anIconPath2;

	if (m_bOs2NetIcon2)
		anIconPath2 = rEnv.GetProgPath(GetInstallMode());
	else
		anIconPath2 = rEnv.GetWorkPath(GetInstallMode());

	anIconPath2 += m_anOs2IconPath2;
	anIconPath2 += m_anOs2Icon2;

	ByteString   anIcon1;
	ByteString   anIcon2;

	if (m_anOs2Icon1.Len() != 0)
		anIcon1 = anIconPath1.GetFull();

	if (m_anOs2Icon2.Len() != 0)
		anIcon2 = anIconPath2.GetFull();

	// Folder erzeugen
	//
	BOOL bSuccess
		= Os2OS::CreateFolder
		   (
				m_anOs2ID,  // "<STB_FOLDER>",
				m_aName,    // "Setup Toolbox"
				anIcon1,
				anIcon2
		   );

	GetLogfile()
		.Success(bSuccess)
		<< "make folder: "
		<< m_anOs2ID     << SEP
		<< m_aName       << SEP
		<< anIcon1       << SEP
		<< anIcon2       << endl;

	return SetSuccess(bSuccess);
}

#else

BOOL SiMakeFolderAction::Execute(SiEnvironment&)
{
	DBG_ERROR("MakeFolderAction::Execute() not supported");
	return TRUE; // keine Desktopintegration
}

#endif

// MakeFolderItemAction ---------------------------------------------------
//

#if defined (WIN) || defined (WNT)

SiMakeFolderItemAction::SiMakeFolderItemAction
(
	SiAgenda      * pAgenda,
	ByteString const& aFolderName,
	ByteString const& anItemName,
	BOOL          bNetwork,
	ByteString const& aPath,
	ByteString const& aFileName,
	ByteString const& aParameter
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFolderName  = aFolderName;
	m_anItemName   = anItemName;
	m_bNetwork     = bNetwork;
	m_aPath        = aPath;
	m_aFileName    = aFileName;
	m_aParameter   = aParameter;
}

BOOL SiMakeFolderItemAction::Execute(SiEnvironment& Env)
{
	if( Env.IsAutoWorkstation() )
		return TRUE;

	BOOL bSuccess = TRUE;
	ByteString aLinkFilename(m_anItemName);
	SiDirEntry aLink( WinOS::SHGetUserProgramFolder() );

	aLinkFilename += ".lnk";
	aLink += m_aFolderName;
	aLink += aLinkFilename;

	SiDirEntry aObject;
	if( m_bNetwork )
		aObject = Env.GetProgPath( GetInstallMode() );
	else
		aObject = Env.GetWorkPath( GetInstallMode() );

	aObject += m_aPath;
	aObject += m_aFileName;
	aObject.ToAbs();

	bSuccess = WinOS::CreateShortcut( aObject.GetFull(), ((const SiDirEntry&)aObject.GetPath()).GetFull(),
									  aLink.GetFull(), m_anItemName, m_aParameter );

	GetLogfile().Success(bSuccess) << "make folder item " << aObject.GetFull()
		<< " " << aLink.GetFull() << " " << m_anItemName << " " << m_aParameter << endl;

	return bSuccess;

/*
 *
	ByteString   aCommand;
	BOOL     bSuccess  = TRUE;
	SiDirEntry aWorkPath;

	if (m_bNetwork)
		aWorkPath = Env.GetProgPath(GetInstallMode());
	else
		aWorkPath = Env.GetWorkPath(GetInstallMode());

	aWorkPath += m_aPath;

	aCommand  = "[CreateGroup(\"";
	aCommand += m_aFolderName;
	aCommand += "\")]";

	bSuccess = WinOS::ExecDDECommand("progman", aCommand);
	GetLogfile()
		.Success(bSuccess)
		<< "dde   " << aCommand << endl;

	aCommand  = "[ReplaceItem(\"";
	aCommand += m_anItemName;
	aCommand += "\",0)]";

	bSuccess = bSuccess && WinOS::ExecDDECommand("progman", aCommand);
	GetLogfile()
		.Success(bSuccess)
		<< "dde   " << aCommand << endl;

	aCommand  = "[AddItem(\"";
	aCommand += (aWorkPath+SiDirEntry(m_aFileName)).GetFull();
	if( m_aParameter.Len() )
	{
		aCommand += " ";
		aCommand += m_aParameter;
	}
	aCommand += "\",";
	aCommand += m_anItemName;
	aCommand += ",%ICON%,,,,\"";
	aCommand += aWorkPath.GetFull();
	aCommand += "\",,)]";
	// aCommand += "[ExitProgman(1)]";

	// noch Hack: damit %ICON% nicht versehentlich stehenbleibt
	aCommand.SearchAndReplace("%ICON%","");

	bSuccess = bSuccess && WinOS::ExecDDECommand("progman", aCommand);
	GetLogfile()
		.Success(bSuccess)
		<< "dde   " << aCommand << endl;

	// Versuchen, die Fenster wieder zu schliessen
	//
	#if defined WNT
	if (System::GetGUIVersion() >= 400)
	{
		aCommand  = "[ShowGroup(\"";
		aCommand += m_aFolderName;
		aCommand += "\",0)]"; // Die 0 (nicht anzeigen) ist nicht dokumentiert: klappte nur bei 95!

		bSuccess = WinOS::ExecDDECommand("progman", aCommand);
		GetLogfile()
			.Success(bSuccess)
			<< "dde   " << aCommand << endl;
	}
	else
		Application::GetAppWindow()->ToTop();
	#endif

	#ifdef WIN
	Application::GetAppWindow()->ToTop();
	#endif

	return SetSuccess(bSuccess);
 *
 */
}

#elif defined (OS2)

SiMakeFolderItemAction::SiMakeFolderItemAction
(
	SiAgenda*       pAgenda,
	ByteString const& aFolderName,      // Os2FolderID
	ByteString const& anItemName,
	BOOL          bNetwork,
	ByteString const& aPath,
	ByteString const& aFileName,
	ByteString const& aParameter,
	ByteString const& anOs2ItemID,      // "<STB_ITEM>",
	ByteString const& anOs2Class,       // "WPProgram",
	ByteString const& anOs2Creater,     // "Setup Toolbox",
	ByteString const& anOs2Filters
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFolderName  = aFolderName;
	m_anItemName   = anItemName;
	m_bNetwork     = bNetwork;
	m_aPath        = aPath;
	m_aFileName    = aFileName;
	m_aParameter   = aParameter;
	m_anOs2ID      = anOs2ItemID;       // "<STB_ITEM>",
	m_anOs2Class   = anOs2Class;        // "WPProgram",
	m_anOs2Creater = anOs2Creater;      // "Setup Toolbox",
	m_anOs2Filters = anOs2Filters;
}

SiMakeFolderItemAction::SiMakeFolderItemAction
(
	SiAgenda*       pAgenda,
	ByteString const& aFolderName,      // Os2FolderID
	ByteString const& anItemName,
	ByteString const& anOs2ItemID,      // "<STB_ITEM>",
	ByteString const& anOs2Reference
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFolderName    = aFolderName;
	m_anItemName     = anItemName;
	m_anOs2ID        = anOs2ItemID;
	m_anOs2Reference = anOs2Reference;
	m_bNetwork       = FALSE;
}

BOOL SiMakeFolderItemAction::Execute(SiEnvironment& rEnv)
{
	if( rEnv.IsAutoWorkstation() )
		return TRUE;

	if( DoRecoverOnly() && Os2OS::IsFolderItemInstalled(m_anOs2ID) )
		return TRUE;

	BOOL bSuccess = FALSE;

	if (m_anOs2Reference.Len() == 0)
	{
		SiDirEntry aFilePath;

		if (m_bNetwork)
			aFilePath = rEnv.GetProgPath(GetInstallMode());
		else
			aFilePath = rEnv.GetDestPath();

		aFilePath += SiDirEntry(m_aPath);
		aFilePath += SiDirEntry(m_aFileName);

		bSuccess
			= Os2OS::CreateFolderItem
			   (
					m_aFolderName,            // "<STB_FOLDER>",
					m_anOs2ID,                // "<STB_ITEM>",
					m_anItemName,             // "Setup Toolbox",
					m_anOs2Class,             // "WPProgram",
					m_anOs2Creater,           // "Setup Toolbox",
					aFilePath.GetFull(),
					m_anOs2Filters
			   );

	GetLogfile()
		.Success(bSuccess)
		<< "make folder item: "
		<< m_aFolderName        << SEP
		<< m_anOs2ID            << SEP
		<< m_anItemName         << SEP
		<< m_anOs2Class         << SEP
		<< m_anOs2Creater       << SEP
		<< aFilePath.GetFull()  << SEP
		<< m_anOs2Filters       << endl;
	}
	else
	{
		bSuccess = Os2OS::CreateFolderItem
		   (
			m_aFolderName,            // "<STB_FOLDER>",
			m_anOs2ID,                // "<STB_ITEM>",
			m_anItemName,             // "Setup Toolbox",
			m_anOs2Reference
		   );

		GetLogfile()
			.Success(bSuccess)
			<< "make folder item (ref): "
			<< m_aFolderName        << SEP
			<< m_anOs2ID            << SEP
			<< m_anItemName         << SEP
			<< m_anOs2Reference     << endl;
	}
	return SetSuccess(bSuccess);
}

#else   // Unix/Mac

SiMakeFolderItemAction::SiMakeFolderItemAction
(
	SiAgenda    * pAgenda,
	ByteString const& aFolderName,
	ByteString const& anItemName,
	BOOL          bNetwork,
	ByteString const& aPath,
	ByteString const& aFileName,
	ByteString const& aParameter
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFolderName  = aFolderName;
	m_anItemName   = anItemName;
	m_bNetwork     = bNetwork;
	m_aPath        = aPath;
	m_aFileName    = aFileName;
	m_aParameter   = aParameter;
}

BOOL SiMakeFolderItemAction::Execute(SiEnvironment&)
{
	DBG_ERROR("MakeFolderItemAction::Execute() called on wrong system");
	return TRUE;
}

#endif

// RegistryItemAction -----------------------------------------------
//
SiRegistryItemAction::SiRegistryItemAction
(
	SiAgenda     * pAgenda,
	RIA            eRIA,
	ByteString  const& aKey,
	ByteString  const& aSubkey,
	ByteString  const& aName,
	ByteString  const& aValue,
	BOOL		   bHexValue,
	BOOL		   bDeleteAll
)
: SiAction(pAgenda, eRIA == RIA_CREATE ? AT_REGISTER : AT_UNREGISTER)
{
	m_eRIA     = eRIA;
	m_aKey     = aKey;
	m_aSubkey  = aSubkey;
	m_aName    = aName;
	m_aValue   = aValue;
	m_bHexValue	 = bHexValue;
	m_bDeleteAll = bDeleteAll;
}


#if defined (WIN) || defined (WNT)

BOOL SiRegistryItemAction::Execute(SiEnvironment& rEnv)
{
	switch (m_eRIA)
	{
		case RIA_CREATE:
		{
			if (DoRecoverOnly()
			&&  WinOS::IsRegistered(m_aKey,m_aSubkey,m_aName, m_aValue.Len() != 0))
				return TRUE;

			// Templatefelder im Value ersetzen
			//
			USHORT nBeginTpl = m_aValue.Search('<');

			while (nBeginTpl != STRING_NOTFOUND)
			{
				USHORT nEndTpl = m_aValue.Search('>',nBeginTpl+1);

				if (nEndTpl != STRING_NOTFOUND)
				{
					ByteString aTpl = m_aValue.Copy(nBeginTpl,nEndTpl-nBeginTpl+1);
					m_aValue.SearchAndReplace(aTpl,GetAgenda()->QueryTemplateValue(aTpl));
				}

				nBeginTpl = m_aValue.Search('<', nBeginTpl+1); // weitersuchen
			}

			long lErr = WinOS::Register(m_aKey,m_aSubkey,m_aName,m_aValue,m_bHexValue);

			GetLogfile()
				.Success(lErr==0)
				<< "register: "
				<< m_aKey       << SEP
				<< m_aSubkey;

			if (m_aValue.Len()!=0)
			{
				GetLogfile()
					<< SEP
					<< (m_aName.Len() != 0 ? m_aName : ByteString("(Standard)")) << SEP
					<< m_aValue;
			}

			GetLogfile() << " Error = " << lErr << endl;
			return SetSuccess(lErr==0);
		}

		case RIA_REMOVE:
		{
			long lErr = WinOS::Deregister(m_aKey,m_aSubkey,m_aValue.Len()==0,
								m_aName, m_bDeleteAll);

			GetLogfile()
				.Success(lErr==0)
				<< "unregister: "
				<< m_aKey               << SEP
				<< m_aSubkey;

			if (m_aValue.Len()!=0)
			{
				GetLogfile()
					<< SEP
					<< (m_aName.Len() != 0 ? m_aName : ByteString("(Standard)")) << SEP
					<< m_aValue;
			}

			GetLogfile() << " Error = " << lErr << endl;
			return lErr==0;
		}

		default:
			DBG_ERROR("RegistryItemAction::Execute(): unknown action");
			return FALSE;
	}
}

#else // WIN/WNT

BOOL SiRegistryItemAction::Execute(SiEnvironment&)
{
	DBG_ERROR("RegistryItemAction::Execute()");
	GetLogfile() << "error: RegistryItemAction::Execute() called" << endl;
	return FALSE;
}

#endif // WIN/WNT

// ------------------------------------------------------------------
// OS/2 Aktionen
// ------------------------------------------------------------------

// Os2RegisterClassAction -------------------------------------------
//

SiOs2RegisterClassAction::SiOs2RegisterClassAction
(
	SiAgenda    * pAgenda,
	ByteString const& aName,
	BOOL          bNetwork,
	ByteString const& aDLLPath,
	ByteString const& aDLLName
)
:SiAction(pAgenda, AT_REGISTER)
{
	m_aName     = aName;
	m_bNetwork  = bNetwork;
	m_aDLLPath  = aDLLPath;
	m_aDLLName  = aDLLName;
}

#ifdef OS2

BOOL SiOs2RegisterClassAction::Execute(SiEnvironment& rEnv)
// pre: DLL wurde schon kopiert
{
	if (DoRecoverOnly()
	/* && Os2OS::IsClassRegistered*/)
		return TRUE;

	SiDirEntry aDLLDir;

	if (m_bNetwork)
		aDLLDir = rEnv.GetProgPath(GetInstallMode());
	else
		aDLLDir = rEnv.GetWorkPath(GetInstallMode());

	aDLLDir += m_aDLLPath;
	aDLLDir += m_aDLLName;

	ByteString   aDLL = aDLLDir.GetFull();
	ULONG    lErr = Os2OS::RegisterClass(m_aName, aDLL);

	GetLogfile()
	   .Success(lErr==0)
		<< "register os/2 class: "
		<< m_aName   << SEP
		<< aDLL;

	if (lErr != 0)
		GetLogfile() << " Err = " << lErr;

	GetLogfile()
		<< endl;

	return SetSuccess(lErr==0);
}

#else

BOOL SiOs2RegisterClassAction::Execute(SiEnvironment&)
{
	DBG_ERROR("Os2RegisterClassAction::Execute() wrong platform");
	return TRUE;
}

#endif

// Os2CreateTemplateAction ------------------------------------------
//

SiOs2CreateTemplateAction::SiOs2CreateTemplateAction
(
	SiAgenda    * pAgenda,
	ByteString const& anID,
	BOOL          bNetwork,
	ByteString const& anIconPath,
	ByteString const& anIconName,
	SiOs2Class*   pOs2Class
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_anID          = anID;
	m_bNetwork      = bNetwork;
	m_anIconPath    = anIconPath;
	m_anIconName    = anIconName;
	m_pOs2Class		= pOs2Class;
}

#ifdef OS2

BOOL SiOs2CreateTemplateAction::Execute(SiEnvironment& rEnv)
// Hinweis: Das Template wurde schon durch register class durch
//          die wpswrt3.dll erzeugt (aber ohen Icon).
//
// pre:  DLL wurde schon kopiert
//       Template existiert schon
// post: Template enthaelt Icon
{
	if (DoRecoverOnly())
		return TRUE;

	SiDirEntry anIconDir;

	if (m_bNetwork)
		anIconDir = rEnv.GetProgPath(GetInstallMode());
	else
		anIconDir = rEnv.GetWorkPath(GetInstallMode());

	anIconDir += m_anIconPath;
	anIconDir += m_anIconName;

	ByteString  anIcon = anIconDir.GetFull();
	BOOL	bSuccess = Os2OS::RegisterTemplateIcon(m_pOs2Class->GetName(), m_anID, anIcon);

	GetLogfile().Success(bSuccess)
		<< "create os/2 template: "
		<< m_pOs2Class->GetName() << ", " << m_anID << SEP << anIcon << endl;

	return SetSuccess(bSuccess);
}

#else

BOOL SiOs2CreateTemplateAction::Execute(SiEnvironment&)
{
	DBG_ERROR("Os2CreateTemplateAction::Execute() wrong platform");
	return TRUE;
}

#endif

// Os2CreateTemplateAction ------------------------------------------
//

SiOs2JoinEAsAction::SiOs2JoinEAsAction
(
	SiAgenda    * pAgenda,
	ByteString const& aFileDir,
	ByteString const& aFileName,
	ByteString const& anEAFileDir,
	ByteString const& anEAFileName
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFileDir          = aFileDir;
	m_aFileName         = aFileName;
	m_anEAFileDir       = anEAFileDir;
	m_anEAFileName  = anEAFileName;
}

#ifdef OS2

BOOL SiOs2JoinEAsAction::Execute(SiEnvironment& rEnv)
{
	SiDirEntry aFileDir  = rEnv.GetDestPath();
			 aFileDir += m_aFileDir;
			 aFileDir += m_aFileName;
	ByteString   aFile     = aFileDir.GetFull();

	SiDirEntry anEAFileDir;

	if (GetInstallMode() == IM_WORKSTATION)
		anEAFileDir = rEnv.GetProgPath(GetInstallMode());
	else
		anEAFileDir = rEnv.GetWorkPath(GetInstallMode());

	anEAFileDir += m_anEAFileDir;
	anEAFileDir += m_anEAFileName;

	ByteString anEAFile = anEAFileDir.GetFull();

	BOOL bSuccess = Os2OS::JoinEAs(aFile, anEAFile);

	GetLogfile()
		.Success(bSuccess)
		<< "set os/2 ea: "
		<< aFile     << SEP
		<< anEAFile  << endl;

	return SetSuccess(bSuccess);
}

#else

BOOL SiOs2JoinEAsAction::Execute(SiEnvironment&)
{
	DBG_ERROR("Os2JoinEAsAction::Execute() wrong platform");
	return TRUE;
}

#endif

// Os2SetCreatorAction ------------------------------------------
//
/****
SiOs2SetCreatorAction::SiOs2SetCreatorAction
(
	SiAgenda    * pAgenda,
	ByteString const& aFileDir,
	ByteString const& aFileName,
	ByteString const& anIconDir,
	ByteString const& anIconName,
	ByteString const& aCreator
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aFileDir   = aFileDir;
	m_aFileName  = aFileName;
	m_anIconDir  = anIconDir;
	m_anIconName = anIconName;
	m_aCreator   = aCreator;
}

#ifdef OS2

BOOL SiOs2SetCreatorAction::Execute(SiEnvironment& rEnv)
{
	SiDirEntry aFileDir  = rEnv.GetDestPath();
			 aFileDir += m_aFileDir;
			 aFileDir += m_aFileName;
	ByteString   aFile     = aFileDir.GetFull();

	SiDirEntry anIconDir;

	if (GetInstallMode() == IM_WORKSTATION)
		anIconDir = rEnv.GetProgPath(GetInstallMode());
	else
		anIconDir = rEnv.GetWorkPath(GetInstallMode());

	anIconDir += m_anIconDir;
	anIconDir += m_anIconName;

	ByteString anIcon = anIconDir.GetFull();

	BOOL bSuccess = Os2OS::SetFileTypeAndIcon(aFile, m_aCreator, anIcon);

	GetLogfile()
		.Success(bSuccess)
		<< "set os/2 creator: "
		<< aFile        << SEP
		<< m_aCreator   << SEP
		<< anIcon       << endl;

	return SetSuccess(bSuccess);
}

#else

BOOL SiOs2SetCreatorAction::Execute(SiEnvironment&)
{
	DBG_ERROR("Os2SetCreatorAction::Execute() wrong platform");
	return TRUE;
}

#endif
****/
// ------------------------------------------------------------------
// Unix Aktionen
// ------------------------------------------------------------------

// UnixCreateSoftLinkAction -----------------------------------------
//
SiUnixCreateSoftLinkAction::SiUnixCreateSoftLinkAction
(
	SiAgenda      * pAgenda,
	ByteString const& aSourcePath,
	ByteString const& aName,
	ByteString const& aDestPath
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aSourcePath = aSourcePath;
	m_aName           = aName;
	m_aDestPath       = aDestPath;
}

#ifdef UNX

BOOL SiUnixCreateSoftLinkAction::Execute(SiEnvironment& Env)
{
	SiDirEntry	aSource	    = Env.GetSourcePath();
				aSource    += m_aSourcePath; // Env.GetSourcePath(m_aSourcePath, GetInstallMode());
				aSource    += m_aName;
	ByteString      aSourceStr  = aSource.GetFull();

	SiDirEntry    aDest       = Env.GetDestPath();
				aDest      += m_aDestPath;
				aDest      += m_aName;
	ByteString      aDestStr    = aDest.GetFull();

	if (DoRecoverOnly()
	&&  aDest.Exists())
		return TRUE;

	BOOL bSuccess = UnixOS::CreateSoftLink(aSourceStr,aDestStr);

	GetLogfile()
		.Success(bSuccess)
		<< "slink " << aSourceStr << SEP
		<< aDestStr << endl;

	return SetSuccess(bSuccess);
}

#else

BOOL SiUnixCreateSoftLinkAction::Execute(SiEnvironment&)
{
	DBG_ERROR("UnixCreateSoftLinkAction::Execute() wrong platform");
	return TRUE;
}

#endif

// UnixCreateHardLinkAction -----------------------------------------
//

SiShortcutAction::SiShortcutAction( SiAgenda* pAgenda, ByteString const& aAbsFilename,
	ByteString const& aAbsShortcutName ) :
	SiAction(pAgenda, AT_REGISTER)
{
	m_aAbsFilename      = aAbsFilename;
	m_aAbsShortcutName  = aAbsShortcutName;
}

BOOL SiShortcutAction::Execute(SiEnvironment& rEnv)
{
	BOOL		bSuccess = TRUE;
	SiDirEntry	aSrc( m_aAbsFilename );
	SiDirEntry	aDst( m_aAbsShortcutName );

	#ifdef WNT
	bSuccess = WinOS::CreateShortcut( aSrc.GetFull(), ((const SiDirEntry&)aSrc.GetPath()).GetFull(),
									  aDst.GetFull(), "", "" );
	#endif
	#ifdef UNX
  	bSuccess = UnixOS::CreateSoftLink( aSrc.GetFull(), aDst.GetFull() );
  	#endif

	GetLogfile().Success(bSuccess) << "make shortcut " << aSrc.GetFull() <<
									  " to " << aDst.GetFull() << endl;
	return SetSuccess(bSuccess);
}

// ------------------------------------------------------------------
// Mac Aktionen
// ------------------------------------------------------------------

// MacAddApplicationAction ------------------------------------------
//

SiMacAddApplicationAction::SiMacAddApplicationAction
(
	SiAgenda      * pAgenda,
	ByteString const& aRelDir,
	ByteString const& aFileName,
	ByteString const& aCreator
)
: SiAction(pAgenda, AT_REGISTER)
{
	m_aRelDir    = aRelDir;
	m_aFileName  = aFileName;
	m_aCreator   = aCreator;
}

#ifdef MAC

BOOL SiMacAddApplicationAction::Execute(SiEnvironment& rEnv)
{
	SiDirEntry aFileDir  = rEnv.GetDestPath();
			 aFileDir += m_aRelDir;
			 aFileDir += m_aFileName;
	ByteString   aFile     = aFileDir.GetFull();

	if (DoRecoverOnly()
	&&  MacOS::SearchApplication(m_aCreator).Len() != 0)
		return TRUE;

	BOOL bSuccess = MacOS::AddApplication(aFile);

	GetLogfile()
		.Success(bSuccess)
		<< "mac add app: "
		<< aFile
		<< endl;

	return SetSuccess(bSuccess);
}

#else

BOOL SiMacAddApplicationAction::Execute(SiEnvironment&)
{
	DBG_ERROR("MacAddApplicationAction::Execute() wrong platform");
	return TRUE;
}

#endif

