/*************************************************************************
 *
 *  $RCSfile: environ.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:57 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif
#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

#include "environ.hxx"
#include "decltor.hxx"

SiEnvironment::SiEnvironment()
{
	pArchive = NULL;
	ClearEnvironment();
}

void SiEnvironment::ClearEnvironment()
{
	if( pArchive )
		delete pArchive;

	m_bParamDestPath		= FALSE;
	m_bBigMode				= FALSE;
	pArchive				= NULL;
	m_bFirstInstallation	= TRUE;

	m_bUpdateNoVersion					= FALSE;
	m_bUpdateOldVersion    				= FALSE;
	m_bUpgradeOldVersion				= FALSE;
	m_bUpdateOldVersionNoRights			= FALSE;
	m_bUpdateOldVersionWrongLanguage	= FALSE;

	m_bReboot				= FALSE;
	m_bLogout				= FALSE;
	m_bDefuseRestart		= FALSE;
	m_bPropagateWinEnvChange= FALSE;

	m_bMigrationAvailalbe	= FALSE;
	m_bDoMigration			= FALSE;
	m_nMigrationSize		= 0;

	m_eInstallType 			= IT_INVALID;
	m_bIsLocal				= TRUE;
	m_bInstallFromNet		= FALSE;
	m_bIsResponse			= FALSE;
	m_bIsAutoWorkstation	= FALSE;
	m_bInstallOnRoot		= FALSE;

	ByteString aEmpty;
	m_aProductName		= aEmpty;
	m_aStartPath		= aEmpty;
	m_aSourcePath		= aEmpty;
	m_aDestPath			= aEmpty;
	m_anInstalledPath	= aEmpty;

	m_aUserAddress		= aEmpty;
	m_aUserName			= aEmpty;
	m_aUserFirstName	= aEmpty;
	m_aUserId			= aEmpty;
	m_aUserEmail		= aEmpty;
	m_aCompanyName		= aEmpty;
	m_aStreet			= aEmpty;
	m_aZip				= aEmpty;
	m_aCity				= aEmpty;
	m_aCustomerNr		= aEmpty;
	m_aCountry			= aEmpty;
	m_nCountry			= 0;

	m_bUIDoNiceLangDefault = TRUE;
	m_bHasKHPatched		= FALSE;
	m_bDoOnlineReg		= FALSE;
	m_bNoBasicError		= FALSE;
	m_bUseSystemDir		= FALSE;
	m_bVirtualMode		= FALSE;
	m_bTimeCheck		= TRUE;
	m_bNoRegistration	= FALSE;
	m_bDlgPrecreateDir	= FALSE;

	m_bHackFlag_1		= FALSE;
	m_bHackFlag_2		= FALSE;
	m_bHackFlag_3		= FALSE;
	m_bHackFlag_4		= FALSE;
	m_bHackFlag_5		= FALSE;

	m_bAuseStreamFlush	= FALSE;
}

SiEnvironment::~SiEnvironment()
{
	ULONG i;
	for( i = 0; i < m_aModSetList.Count(); ++i )
		delete m_aModSetList.GetObject(i);
	m_aModSetList.Clear();

	for( i = 0; i < m_aFollowAppList.Count(); ++i )
		delete m_aFollowAppList.GetObject(i);
	m_aFollowAppList.Clear();

	for( i = 0; i < m_aWorkstationList.Count(); ++i )
		delete m_aWorkstationList.GetObject(i);
	m_aWorkstationList.Clear();

	// Languages
	for( i = 0; i < m_aLangContext.Count(); ++i )
		delete m_aLangContext.GetObject(i);
	m_aLangContext.Clear();
	for( i = 0; i < m_aUILangContext.Count(); ++i )
		delete m_aUILangContext.GetObject(i);
	m_aUILangContext.Clear();
	for( i = 0; i < m_aSwitchCtxInstall.Count(); ++i )
		delete m_aSwitchCtxInstall.GetObject(i);
	m_aSwitchCtxInstall.Clear();
	for( i = 0; i < m_aSwitchCtxDelete.Count(); ++i )
		delete m_aSwitchCtxDelete.GetObject(i);
	m_aSwitchCtxDelete.Clear();

	if( pArchive )
		delete pArchive;
}

BOOL SiEnvironment::IsHackFlag(USHORT nFlag) const
{
	switch( nFlag )
	{
		case 1 : return m_bHackFlag_1;
		case 2 : return m_bHackFlag_2;
		case 3 : return m_bHackFlag_3;
		case 4 : return m_bHackFlag_4;
		case 5 : return m_bHackFlag_5;
	}
	return FALSE;
}

void SiEnvironment::SetHackFlag(USHORT nFlag)
{
	switch( nFlag )
	{
		case 1 : m_bHackFlag_1 = TRUE; break;
		case 2 : m_bHackFlag_2 = TRUE; break;
		case 3 : m_bHackFlag_3 = TRUE; break;
		case 4 : m_bHackFlag_4 = TRUE; break;
		case 5 : m_bHackFlag_5 = TRUE; break;
	}
}

void SiEnvironment::ClearHackFlag(USHORT nFlag)
{
	switch( nFlag )
	{
		case 1 : m_bHackFlag_1 = FALSE; break;
		case 2 : m_bHackFlag_2 = FALSE; break;
		case 3 : m_bHackFlag_3 = FALSE; break;
		case 4 : m_bHackFlag_4 = FALSE; break;
		case 5 : m_bHackFlag_5 = FALSE; break;
	}
}

void SiEnvironment::AddSwitchContextInstall(LanguageContext* pToAdd)
{
	for( USHORT i = 0; i < m_aSwitchCtxInstall.Count(); ++i )
	{
		LanguageContext* pCmp = m_aSwitchCtxInstall.GetObject(i);
		if( pCmp->nLanguage == pToAdd->nLanguage	&&
			pCmp->isProg	== pToAdd->isProg		&&
			pCmp->isDoc		== pToAdd->isDoc )
			return;
	}
	m_aSwitchCtxInstall.Insert(pToAdd,LIST_APPEND);
}

void SiEnvironment::AddSwitchContextDelete(LanguageContext* pToAdd)
{
	for( USHORT i = 0; i < m_aSwitchCtxDelete.Count(); ++i )
	{
		LanguageContext* pCmp = m_aSwitchCtxDelete.GetObject(i);
		if( pCmp->nLanguage == pToAdd->nLanguage	&&
			pCmp->isProg	== pToAdd->isProg		&&
			pCmp->isDoc		== pToAdd->isDoc )
			return;
	}
	m_aSwitchCtxDelete.Insert(pToAdd,LIST_APPEND);
}

void SiEnvironment::SetArchive(ArchDirectory* pNew, FncFindArch fncFindArch)
{
	if( pArchive )
		delete pArchive;
	pArchive = pNew;
	pArchive->SetFindArchHdl( fncFindArch );
}

void SiEnvironment::InitStartPath()
{
	SiDirEntry anAppEntry(Application::GetAppFileName());
	anAppEntry.ToAbs();
	m_aStartPath = ((const SiDirEntry&)anAppEntry.GetPath()).GetFull();

	SiDirEntry aKHPatch( m_aStartPath );
	aKHPatch += ByteString(KH_PATCH_DETECTOR);
	if( aKHPatch.Exists() )
		m_bHasKHPatched = TRUE;
};

///////////////////////////////////////////////////////////////////////////////
//
//		Store Environment
//

#define MAIN_KEY	"HKEY_CURRENT_USER"
#define SUB_KEY		"Software\\StarDivision\\setup\\recycle"

ByteString _I2S(const ByteString& rStr) { ByteString aRet(rStr); /* aRet.Convert(CHARSET_IBMPC, CHARSET_SYSTEM); */ return aRet; }
ByteString _S2I(const ByteString& rStr) { ByteString aRet(rStr); /* aRet.Convert(CHARSET_SYSTEM, CHARSET_IBMPC); */ return aRet; }
void SiEnvironment::StoreRecycle()
{
	#ifdef WNT
	WinOS::Register(MAIN_KEY, SUB_KEY, "version",			1, FALSE );

	WinOS::Register(MAIN_KEY, SUB_KEY, "lastname",			m_aUserName, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "firstname",			m_aUserFirstName, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "street",			m_aStreet, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "zip",				m_aZip, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "city",				m_aCity, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "country",			m_aCountry, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "countryid",			m_nCountry, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "address",			m_aUserAddress, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "company",			m_aCompanyName, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "customernr",		m_aCustomerNr, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "id",				m_aUserId, FALSE );
	WinOS::Register(MAIN_KEY, SUB_KEY, "email",				m_aUserEmail, FALSE );
	#else
	#endif
}

void SiEnvironment::InitRecycle()
{
	#ifdef WNT
	USHORT nVersion			= (USHORT) ByteString(WinOS::GetRegValue(MAIN_KEY, SUB_KEY, "version")).ToInt32();

	m_aUserName				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "lastname" );
	m_aUserFirstName		= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "firstname" );
	m_aStreet				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "street" );
	m_aZip					= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "zip" );
	m_aCity					= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "city" );
	m_aCountry				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "country" );
	m_nCountry				= (USHORT) ByteString(WinOS::GetRegValue(MAIN_KEY, SUB_KEY, "countryid")).ToInt32();
	m_aUserAddress			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "address" );
	m_aCompanyName			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "company" );
	m_aCustomerNr			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "customernr" );
	m_aUserId				= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "id" );
	m_aUserEmail 			= WinOS::GetRegValue( MAIN_KEY, SUB_KEY, "email" );

	#else
	#endif
}
