/*************************************************************************
 *
 *  $RCSfile: convert.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifdef MAC
#include "main.hxx"		      // fuer den ResManager!
#endif

#include "convert.hxx"

#include "compiler.hxx"
#include "sistream.hxx"
#include "script.hxx"
#include "fields.hxx"
#include "os.hxx"

#include <tlfsys.hxx>
#include <svdlg.hxx>

#ifdef MAC
FSysPathStyle eFSysPathStyle = FSYS_STYLE_NTFS;
#else
FSysPathStyle eFSysPathStyle = FSYS_STYLE_HOST;
#endif

SiConverter::SiConverter(SvFileStream &rStream, SiCompiler *pCompiler)
: m_aStream(rStream)
, m_Profiles(51,FALSE /* bOwner*/)
, m_SpezialFiles(201)
, m_bRootOnly(FALSE)
{
	m_pCS       = NULL;
	m_pCompiler = pCompiler;
}

ByteString SiConverter::MakeID(ByteString const& rName)
{
	ByteString anID = rName;

	anID.ToUpper();
	anID.EraseAllChars();
	anID.EraseAllChars('-');
	anID.EraseAllChars('.');
	anID.EraseAllChars('~');

	if (anID[(USHORT)0] == '_')
		anID.Erase(0,1);

	return anID;
}

BOOL SiConverter::ConvertTo(SiCompiledScript* pCS)
{
	m_pCS = pCS;

	CreateSystemObjects();

	Pass1();
	Pass2();
	m_pCS->AddUnlinkedObjectsToRoot();

	SiDirEntry aFile = m_aStream.GetFileName();

	aFile.SetName("reg.inf");

	if (aFile.Exists())
	{
		m_aStream.Close();
		m_aStream.Open(aFile.GetFull(), STREAM_STD_READ);
		Pass1();
	}

	m_pCS = NULL;
	return TRUE;
}

BOOL SiConverter::ConvertRegTo(SiCompiledScript* pCS)
{
	m_pCS = pCS;
	m_bRootOnly = TRUE;

	CreateRootModule("recoverreg");
	GetModule("egal")->SetInstalled(TRUE);

	SiInstallation *pInst = new SiInstallation(SiIdentifier("INSTALLATION"), m_pCompiler);
	pInst->SetProperty(PROPERTY_MODE, VALUE_STANDALONE);
	m_pCS->Add(pInst,m_pCompiler);

	CreateSystemObjects();
	Pass1();

	m_bRootOnly = FALSE;
	m_pCS = NULL;
	return TRUE;
}

BOOL SiConverter::ReadLine(ByteString &aLine)
{
	while (m_aStream.ReadLine(m_aCurrentLine)
	   &&  m_aCurrentLine.Len() == 0)
	   ;

	aLine = m_aCurrentLine;
	return m_aCurrentLine.Len() != 0;
}

BOOL SiConverter::Pass1()
{
	BOOL	bRead = TRUE;

	ReadLine(m_aCurrentLine);

	do
	{
		bRead = FALSE;

		if ((char)m_aCurrentLine == '[')
		{
			m_aCurrentLine.ToLower();

				 if (m_aCurrentLine == "[program]")      ConvertProgramSection();
			else if (m_aCurrentLine == "[slides]")       ConvertSlidesSection();
			else if (m_aCurrentLine == "[disks]")        ConvertDiskSection();
			else if (m_aCurrentLine == "[files]")        ConvertFilesSection();
			else if (m_aCurrentLine == "[copy]")         ConvertCopySection();
			else if (m_aCurrentLine == "[ossystem]")     ConvertOsSystemSection();
			else if (m_aCurrentLine == "[system]")       ConvertSystemSection();
			else if (m_aCurrentLine == "[directories]")  ConvertDirectoriesSection();
			else if (m_aCurrentLine == "[registration database]") ConvertRegDBSection();
			else if (m_aCurrentLine == "[clsid-keys]")            ConvertCLSIDKeysSection();
			else if (m_aCurrentLine == "[clsid-section]")         ConvertCLSIDSectionSection();

			else if (m_aCurrentLine.Match("[archieve") == 9) ConvertArchive((USHORT)m_aCurrentLine.Copy(9));
			else if (m_aCurrentLine.Match("[progmod") == 8)	 ConvertProgMod((USHORT)m_aCurrentLine.Copy(8));
			else if (m_aCurrentLine.Match("[module") == 7)  ConvertModule((USHORT)m_aCurrentLine.Copy(7));
			else if (m_aCurrentLine.Match("[option") == 7)  ConvertOption((USHORT)m_aCurrentLine.Copy(7));
			else bRead = TRUE;
		}
		else
			bRead = TRUE;

		if (bRead)
			ReadLine(m_aCurrentLine);
	}
	while(!m_aStream.IsEof());

	return TRUE;
}

BOOL SiConverter::Pass2()
{
	BOOL	bRead = TRUE;

	m_aStream.Seek(0);
	ReadLine(m_aCurrentLine);

	do
	{
		bRead = FALSE;

		if ((char)m_aCurrentLine == '[')
		{
			m_aCurrentLine.ToLower();

				 if (m_aCurrentLine == "[fonts]")    ConvertFontsSection();
			else if (m_aCurrentLine == "[minimal]")  ConvertMinimalDefaultSection(TRUE);
			else if (m_aCurrentLine == "[default]")  ConvertMinimalDefaultSection(FALSE);
			else if (m_aCurrentLine == "[profiles]") ConvertProfilesSection();
			else if (m_aCurrentLine == "[progman]")  ConvertProgmanSection();
			else if (m_aCurrentLine == "[folder]")   ConvertFolderSection();
			else if (m_aCurrentLine == "[wps]")      ConvertWPSSection();
			else if (m_aCurrentLine == "[objects]")  ConvertObjectsSection();
			else if (m_aCurrentLine == "[templates]")ConvertTemplatesSection();
			else if (m_aCurrentLine == "[ea]")       ConvertEASection();
			else if (m_aCurrentLine == "[application]")ConvertApplicationSection();
			else
			{
				ByteString aProfileID = MakeID(m_aCurrentLine.Copy(1,m_aCurrentLine.Len()-2));

				if (m_Profiles.Find(aProfileID) != NULL)
					ConvertProfileItemSection(aProfileID);
				else
					bRead = TRUE;
			}
		}
		else
			bRead = TRUE;

		if (bRead)
			ReadLine(m_aCurrentLine);
	}
	while(!m_aStream.IsEof());

	return TRUE;
}

void SiConverter::CreateSystemObjects()
{
	// Autostart-Folder
	//
	SiFolder *pFolder = new SiFolder(SiIdentifier("PREDEFINED_AUTOSTART"),m_pCompiler,TRUE/*Sys*/);
	#ifdef OS2
		pFolder->SetProperty(PROPERTY_OS2_ID,Os2OS::GetAutostartFolderID());
	#elif defined WIN || defined WNT
		pFolder->SetProperty(PROPERTY_NAME,WinOS::GetAutostartFolderName());
	#endif
	m_pCS->Add(pFolder,m_pCompiler);

	// OsClass Program Class
	//
	SiOs2Class *pOs2Class = new SiOs2Class(
		SiIdentifier("PREDEFINED_OS2_PROGRAMCLASS"),
		m_pCompiler, TRUE /*Sys*/);

	pOs2Class->SetProperty(PROPERTY_NAME,"WPProgram");
	m_pCS->Add(pOs2Class,m_pCompiler);

	// Systemdirectories
	//
	SiDirectory *pDir = NULL;

	pDir = new SiDirectory(
		SiIdentifier("PREDEFINED_OSDIR"),
		m_pCompiler,TRUE /* System-Dir */);
	m_pCS->Add(pDir, m_pCompiler);
	pDir->SetProperty(PROPERTY_HOSTNAME,OS::GetGUIPath());

	pDir = new SiDirectory(
		SiIdentifier("PREDEFINED_OSSYSTEMDIR"),
		m_pCompiler,TRUE /* System-Dir */);
	m_pCS->Add(pDir, m_pCompiler);
	pDir->SetProperty(PROPERTY_HOSTNAME,OS::GetSystemPath());

	pDir = new SiDirectory(
		SiIdentifier("PREDEFINED_OSSYSTEMFONTDIR"),
		m_pCompiler,TRUE /* System-Dir */);
	m_pCS->Add(pDir, m_pCompiler);
	pDir->SetProperty(PROPERTY_HOSTNAME,OS::GetSystemFontPath());

	// RegistryItems
	//
	SiRegistryItem *pItem = NULL;

	pItem = new SiRegistryItem(SiIdentifier("PREDEFINED_HKEY_CLASSES_ROOT"),m_pCompiler,TRUE);
	pItem->SetKey("HKEY_CLASSES_ROOT");
	m_pCS->Add(pItem,m_pCompiler);

	pItem = new SiRegistryItem(SiIdentifier("PREDEFINED_HKEY_CURRENT_USER"),m_pCompiler,TRUE);
	pItem->SetKey("HKEY_CURRENT_USER");
	m_pCS->Add(pItem,m_pCompiler);

	pItem = new SiRegistryItem(SiIdentifier("PREDEFINED_HKEY_LOCAL_MACHINE"),m_pCompiler,TRUE);
	pItem->SetKey("HKEY_LOCAL_MACHINE");
	m_pCS->Add(pItem,m_pCompiler);

	pItem = new SiRegistryItem(SiIdentifier("PREDEFINED_HKEY_USERS"),m_pCompiler,TRUE);
	pItem->SetKey("HKEY_USERS");
	m_pCS->Add(pItem,m_pCompiler);

	pItem = new SiRegistryItem(SiIdentifier("PREDEFINED_HKEY_CURRENT_CONFIG"),m_pCompiler,TRUE);
	pItem->SetKey("HKEY_CURRENT_CONFIG");
	m_pCS->Add(pItem,m_pCompiler);

	pItem = new SiRegistryItem(SiIdentifier("PREDEFINED_HKEY_DYN_DATA"),m_pCompiler,TRUE);
	pItem->SetKey("HKEY_DYN_DATA");
	m_pCS->Add(pItem,m_pCompiler);
}

SiDeclarator* SiConverter::Find(ByteString const& rID)
{
	SiDeclarator *pDec = m_pCS->Find(rID);

#ifdef DBG_UTIL
	if (pDec == NULL)
	{
		ByteString   aMessage  = "Error: not found ";
				 aMessage += rID;
		ErrorBox anErrorBox(NULL, WB_OK, aMessage);
				 anErrorBox.Execute();
	}
#endif

	return pDec;
}

SiDirectory* SiConverter::GetDirectory(ByteString const& rName)
{
	ByteString aNormName = rName;

	if (aNormName[(USHORT)(aNormName.Len()-1)] == '\\')
		aNormName.Erase(aNormName.Len()-1);

	ByteString aName = aNormName;

	if (aName == "."
	||  aName == ""
	||  aName == ":")
	{
		aName 		= "WORKPATH";
		aNormName 	= SiDirEntry().GetFull();
	}

	while (aName.Search('\\') != STRING_NOTFOUND)
		aName.SearchAndReplace('\\','_');

	while (aName.Search('~') != STRING_NOTFOUND)
		aName.SearchAndReplace('~','_');

	while (aName.Search('/') != STRING_NOTFOUND)
		aName.SearchAndReplace('/','_');

	while (aName.Search(':') != STRING_NOTFOUND)
		aName.SearchAndReplace(':','_');

	aName = MakeID(aName);

	SiDirectory *pDir = (SiDirectory *) m_pCS->Find(aName);

	if (pDir == NULL)
	{
		pDir = new SiDirectory(SiIdentifier(aName),m_pCompiler);
		SiDirEntry aDir(aNormName,eFSysPathStyle);

		if (aDir.Level() > 1)
		{
			pDir->SetProperty(PROPERTY_PARENTID,GetDirectory(aDir.GetPath().GetFull(eFSysPathStyle)));
			pDir->SetProperty(PROPERTY_HOSTNAME,aDir.GetName());
		}
		else
		{
			pDir->SetProperty(PROPERTY_HOSTNAME,aNormName);
		}
		m_pCS->Add(pDir,m_pCompiler);
	}

	return pDir;
}

SiModule* SiConverter::GetModule(ByteString const& rValue)
{
	if( m_bRootOnly )
		return (SiModule*) m_pCS->Find("ROOT");

	ByteString aValue = rValue; aValue.EraseAllChars();

	USHORT nProgMod = aValue.GetToken(0,',');
	USHORT nModule  = aValue.GetToken(1,',');
	USHORT nOption  = aValue.GetToken(2,',');
	ByteString anID;

	if (nProgMod == 99
	&&  nModule  == 99
	&&  nOption  == 99)
		anID = "ROOT";

	else if (nProgMod != 0) anID = MakeID(ByteString("PROGMOD") + ByteString(nProgMod));
	else if (nModule  != 0) anID = MakeID(ByteString("MODULE")  + ByteString(nModule));
	else if (nOption  != 0) anID = MakeID(ByteString("OPTION")  + ByteString(nOption));
	else                    anID = "ROOT";

	return (SiModule*) m_pCS->Find(anID);
}

SiDataCarrier* SiConverter::GetArchive(ByteString const& aArchive, SiDataCarrier *pDisk)
{
	ByteString			anID = MakeID(ByteString("DISK_")+aArchive);
	SiDataCarrier  *pDC  = (SiDataCarrier*) m_pCS->Find(anID);

	if (pDC != NULL)
		return pDC;

	pDC = new SiDataCarrier(SiIdentifier(anID), m_pCompiler);
	pDC->SetProperty(PROPERTY_NAME,		pDisk->GetName());
	pDC->SetProperty(PROPERTY_DISKNO,	pDisk->GetDiskNo());
	pDC->SetProperty(PROPERTY_ARCHIVE,	aArchive);
	m_pCS->Add(pDC,m_pCompiler);
	return pDC;
}

SiFolder* SiConverter::GetFolder(ByteString const& rID)
{
	SiFolder *pFolder = (SiFolder*)m_pCS->Find(rID);

	if (pFolder != NULL)
		return pFolder;

	pFolder = new SiFolder(SiIdentifier(rID),m_pCompiler);
	m_pCS->Add(pFolder, m_pCompiler);
	return pFolder;
}

/////////////////////////////////////////////////////////////////////
// Phase 1
/////////////////////////////////////////////////////////////////////

BOOL SiConverter::ConvertProgramSection()
{
	SiInstallation *pInst = new SiInstallation(SiIdentifier("INSTALLATION"), m_pCompiler);
	ByteString          aLine;
	BOOL			bModifyPath = TRUE;

	ReadLine(aLine);

	while(aLine.Search('=') != STRING_NOTFOUND)
	{
		ByteString aName  = aLine.GetToken(0, '=').Lower();
		ByteString aValue = aLine.GetToken(1, '=').EraseLeadingChars().EraseTrailingChars();

		if (aName == "name")
			CreateRootModule(aValue);
		else if (aName == "installpath")
			pInst->SetProperty(PROPERTY_DEFAULTDESTPATH,aValue);
		else if (aName == "updatefor")
		{
			pInst->SetProperty(PROPERTY_UPDATEFOR,aValue);
			pInst->SetProperty(PROPERTY_FLAGS, VALUE_OLD_VERSION_REQUIRED);
		}
		else if (aName == "upgradefor")
			pInst->SetProperty(PROPERTY_UPGRADEFOR,aValue);
		else if (aName == "forceoverwrite")
		{
			if (aValue == "1")
				pInst->SetProperty(PROPERTY_FORCEOVERWRITE,"YES");
		}
		else if (aName == "keepoldversion")
		{
			if (aValue=="1")
				pInst->SetProperty(PROPERTY_FLAGS, VALUE_KEEP_OLD_VERSION);
		}
		else if (aName == "nolibpath")
		{
			if (aValue == "1")
				bModifyPath = FALSE;
		}
		else if (aName == "state")
		{
			if (aValue.CompareIgnoreCaseToAscii("a") == COMPARE_EQUAL)
			{
				#if defined WIN || defined WNT
				pInst->SetProperty(PROPERTY_DELETE_KEY, "SvAppGroups");
				#else if defined( OS2 )
				pInst->SetProperty(PROPERTY_DELETE_KEY, "SvAppGroups:dbtdat");
				#endif
			}
		}

		ReadLine(aLine);
	}

	#ifdef OS2
	if (bModifyPath)
		pInst->SetProperty(PROPERTY_FLAGS,VALUE_OS2_MODIFY_PATH);
	#endif

	m_pCS->Add(pInst,m_pCompiler);
	return TRUE;
}

BOOL SiConverter::ConvertSlidesSection()
{
	ByteString  aLine;
	int     nCount=1;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString   aValue  = aLine.Copy(aLine.Search("=")+1).EraseLeadingChars();
		SiSlide	*pSlide = new SiSlide(SiIdentifier(ByteString("SLIDE")+ByteString(nCount)),m_pCompiler);

		pSlide->SetProperty(PROPERTY_POSX,   (long)aValue.GetToken(0,',').EraseAllChars());
		pSlide->SetProperty(PROPERTY_POSY,   (long)aValue.GetToken(1,',').EraseAllChars());
		pSlide->SetProperty(PROPERTY_WIDTH,  (long)aValue.GetToken(2,',').EraseAllChars());
		pSlide->SetProperty(PROPERTY_HEIGHT, (long)aValue.GetToken(3,',').EraseAllChars());
		pSlide->SetProperty(PROPERTY_ORDER,  (long)aValue.GetToken(4,',').EraseAllChars());
		pSlide->SetProperty(PROPERTY_FILENAME, aValue.GetToken(5,',').EraseAllChars());

		USHORT nTextBegin=0;

		for (USHORT i=0; i<6; i++)
			nTextBegin = aValue.Search(',', nTextBegin+1);

		pSlide->SetProperty(PROPERTY_TEXT, aValue.Copy(nTextBegin+1).EraseLeadingChars());

		m_pCS->Add(pSlide,m_pCompiler);

		nCount++;
		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::CreateRootModule(ByteString const& aName)
{
	SiModule *pRoot = new SiModule(SiIdentifier("ROOT"), m_pCompiler);

	// pRoot->SetProperty(PROPERTY_MINIMAL,"YES"); Nur die Hauptapps sind minimal
	pRoot->SetProperty(PROPERTY_DEFAULT,"YES");
	pRoot->SetProperty(PROPERTY_NAME,aName);
	m_pCS->Add(pRoot,m_pCompiler);
	return TRUE;
}

BOOL SiConverter::ConvertDiskSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while(aLine.Search('=') != STRING_NOTFOUND)
	{
		ByteString aName  = MakeID(aLine.GetToken(0, '='));
		ByteString aValue = aLine.GetToken(1, '=');

		SiDataCarrier *pDC = new SiDataCarrier(SiIdentifier(aName), m_pCompiler);

		pDC->SetProperty(PROPERTY_NAME,aValue);
		pDC->SetProperty(PROPERTY_DISKNO,long(aName.Erase(4)));
		m_pCS->Add(pDC,m_pCompiler);
		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertFilesSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while(aLine.Search('=') != STRING_NOTFOUND)
	{
		ByteString aName  = MakeID(aLine.GetToken(0, '='));
		ByteString aValue = aLine.GetToken(1, '=');
		char   cFlag  = aValue.GetToken(2,',').EraseAllChars().ToLower();

		SiFile *pFile = new SiFile(SiIdentifier(aName), m_pCompiler);

		SiDirEntry aFile(aValue.GetToken(0,',').EraseLeadingChars().EraseTrailingChars(),eFSysPathStyle); // "cache\cache.txt"

		pFile->SetProperty(PROPERTY_DIR,GetDirectory(aFile.GetPath().GetFull(eFSysPathStyle)));
		pFile->SetProperty(PROPERTY_NAME,aFile.GetName());

		SiDirEntry aPackedName(aValue.GetToken(3,',').EraseLeadingChars().EraseTrailingChars(),eFSysPathStyle);
		ByteString	 aArchive	 = aPackedName.GetPath().GetName();
		ByteString	 aDiskId	 = MakeID(ByteString("DISK")+aValue.GetToken(1,',').EraseAllChars());

		pFile->SetProperty(PROPERTY_PACKEDNAME,aPackedName.GetName());

		if (aPackedName.GetPath().GetFlag() == FSYS_FLAG_RELROOT)
			pFile->SetProperty(PROPERTY_CARRIER,Find(aDiskId));
		else
			pFile->SetProperty(PROPERTY_CARRIER,GetArchive(aArchive,(SiDataCarrier*)Find(aDiskId)));


		if (cFlag != 'c')
		{
			pFile->SetProperty(PROPERTY_SIZE,long(aValue.GetToken(4,',').EraseAllChars()));

			ByteString aDate = aValue.GetToken(5,',').EraseAllChars();

			if (aDate.Len() > 0)
			{
				aDate.Insert("19",4);
				pFile->SetProperty(PROPERTY_DATE,aDate);
				pFile->SetProperty(PROPERTY_TIME,aValue.GetToken(6,',').EraseAllChars());
			}
		}
		else
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_DELETE_ONLY);

		if (cFlag == 'p'
		||  cFlag == 'x'
		||  cFlag == 'f'
		||  cFlag == 'a')
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_PACKED);

		if (cFlag == 'x'
		||  cFlag == 'y'
		||  cFlag == 'f'
		||  cFlag == 'g'
		||  cFlag == 'a'
		||  cFlag == 'b')
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_DONT_DELETE);

		if (cFlag == 'a'
		||  cFlag == 'b')
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_DONT_OVERWRITE);

		if (cFlag == 'f'
		||  cFlag == 'g')
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_CHECK_VERSION);

		if (pFile->GetName().Lower().Match("readme") == 6)
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_WORKSTATION);

		#ifdef OS2
		if (pFile->GetName().Lower().Match("wpswrt3") == 7 ||
			pFile->GetName().Lower().Match("wpswrt4") == 7 )
		{
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_WORKSTATION);
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_DONT_OVERWRITE);
		}
		#endif

		#ifdef UNX
		pFile->SetProperty(PROPERTY_UNIX_RIGHTS,long(aValue.GetToken(7,',')));
		pFile->SetProperty(PROPERTY_FLAGS, VALUE_ARCHIVE);
		#endif

		m_pCS->Add(pFile,m_pCompiler);

		// Zum spaeteren Nachschlagen ueber den Dateinamen
		//
		#if defined WIN || defined WNT || defined OS2

		if (aFile.GetExtension().ToLower() == "exe"
		||  aFile.GetExtension().ToLower() == "dll")
			m_SpezialFiles.Insert(aFile.GetName().Upper(),pFile);

		#elif defined UNX

		if (aFile.GetName().ToLower() == "setup")
			m_SpezialFiles.Insert(aFile.GetName().Upper(),pFile);

		#elif defined MAC

		ByteString aSuffix = aFile.GetName();
			   aSuffix = aSuffix.Copy(aSuffix.Len()-3); // "3.1" -> Programme

		if (aSuffix == "3.1")
			m_SpezialFiles.Insert(aFile.GetName().Upper(),pFile);

		#endif

		ReadLine(aLine);
	}

	#if defined WIN || defined WNT || defined OS2
	// Damit auch die Dateien des Setup kopiert werden
	//
	SiFile *pFile = new SiFile(SiIdentifier("SETUP_ZIP"), m_pCompiler);

	pFile->SetProperty(PROPERTY_NAME,       "setup.exe"); // sehr tricky !!!
	pFile->SetProperty(PROPERTY_PACKEDNAME, "f_0000");
	pFile->SetProperty(PROPERTY_CARRIER,    Find("DISK1"));
	pFile->SetProperty(PROPERTY_DIR,        GetDirectory("WORKPATH"));
	pFile->SetProperty(PROPERTY_FLAGS,      VALUE_PACKED);
	pFile->SetProperty(PROPERTY_FLAGS,      VALUE_ARCHIVE);
	pFile->SetProperty(PROPERTY_FLAGS,      VALUE_SETUP);
	pFile->SetProperty(PROPERTY_FLAGS,      VALUE_DONT_DELETE);
	pFile->SetProperty(PROPERTY_SIZE,       1000000L);

	m_pCS->Add(pFile,m_pCompiler);
	m_SpezialFiles.Insert("SETUP.EXE",pFile);
	#endif

	#if defined UNX
	// Unter Unix Install-HardLink auf Setup anlegen
	//
	SiShortcut *pUHL = new SiShortcut(SiIdentifier("INSTALL_HL"), m_pCompiler);

	pUHL->SetProperty(PROPERTY_NAME,   "install");
	pUHL->SetProperty(PROPERTY_FILEID, m_SpezialFiles.Find("SETUP"));

	m_pCS->Add(pUHL,m_pCompiler);
	#endif

	#ifdef MAC
	SiFile *pFile = new SiFile(SiIdentifier("SETUP_EXE"), m_pCompiler);

	pFile->SetProperty(PROPERTY_NAME,       "setup 4.0");
	pFile->SetProperty(PROPERTY_CARRIER,    Find("DISK1"));
	pFile->SetProperty(PROPERTY_DIR,        GetDirectory("WORKPATH"));
	pFile->SetProperty(PROPERTY_FLAGS,      VALUE_SETUP);
	m_pCS->Add(pFile,m_pCompiler);

	pFile = new SiFile(SiIdentifier("SETUP_RES"), m_pCompiler);

	ByteString aResName = pSetupApp->GetResManager()->GetFileName();
	SiDirEntry aEntry( aResName );

/*	ByteString aResname  = "set";
		   aResname += ByteString(SUPD);
		   aResname += "49.rsrc";
*/
	pFile->SetProperty(PROPERTY_NAME,       aEntry.GetName());
	pFile->SetProperty(PROPERTY_CARRIER,    Find("DISK1"));
	pFile->SetProperty(PROPERTY_DIR,        GetDirectory("WORKPATH"));
	m_pCS->Add(pFile,m_pCompiler);
	#endif

	return TRUE;
}

BOOL SiConverter::ConvertCopySection()
{
	ByteString aLine;

	ReadLine(aLine);

	while(aLine.Search('=') != STRING_NOTFOUND)
	{
		aLine.ToLower().EraseAllChars();

		ByteString aValue   = aLine.GetToken(1, '=');
		SiFile *pMVFile = (SiFile*)Find(MakeID(ByteString("File")+aValue.GetToken(0,',')));

		DBG_ASSERT(pMVFile != NULL, "not a file");

		if (pMVFile != NULL)
		{
			USHORT nCount = aValue.GetTokenCount(',');

			for (USHORT i=1; i<nCount; i++)
			{
				SiFile *pPart = (SiFile*)Find(MakeID(ByteString("File")+aValue.GetToken(i,',')));

				DBG_ASSERT(pPart != NULL, "illegal part of a file");
				pPart->SetProperty(PROPERTY_NAME, pPart->GetPackedName());
				pPart->SetProperty(PROPERTY_PACKEDNAME, ByteString());
				pPart->SetProperty(PROPERTY_PARTOF, pMVFile);
				pPart->SetProperty(PROPERTY_PART,	(long)i);
				pPart->AddRefCount(); // siehe script.cxx
				pPart->SetUnpacked(); // damit ein Teil nicht separat enpackt wird
				pMVFile->AddPart(pPart);
			}
		}

		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertOsSystemSection()
{
	ByteString aLine;

	ReadLine(aLine);
	aLine.ToLower().EraseAllChars();

	DBG_ASSERT(aLine.GetToken(0, '=') == "filelist", "error");

	ByteString aValue  = aLine.GetToken(1, '=');
	USHORT nTokens = aValue.GetTokenCount(',');

	for (USHORT i=0; i<nTokens; i++)
	{
		SiFile      *pFile = (SiFile*) Find(MakeID(ByteString("FILE")+aValue.GetToken(i,',')));
		SiDirectory *pDir  = (SiDirectory *)(void*)pFile->GetDirectory();

		pDir->SetProperty(PROPERTY_PARENTID, Find("PREDEFINED_OSDIR"));
		pFile->SetProperty(PROPERTY_NETDIR,  Find("WORKPATH"));
	}
	return TRUE;
}

BOOL SiConverter::ConvertSystemSection()
{
	ByteString aLine;

	ReadLine(aLine);
	aLine.ToLower().EraseAllChars();

	DBG_ASSERT(aLine.GetToken(0, '=') == "filelist", "error");

	ByteString aValue  = aLine.GetToken(1, '=');
	USHORT nTokens = aValue.GetTokenCount(',');

	for (USHORT i=0; i<nTokens; i++)
	{
		SiFile *pFile = (SiFile*)Find(MakeID(ByteString("FILE")+aValue.GetToken(i,',')));

		pFile->SetProperty(PROPERTY_DIR,	Find("PREDEFINED_OSSYSTEMDIR"));
		pFile->SetProperty(PROPERTY_NETDIR,	Find("WORKPATH"));
		pFile->SetProperty(PROPERTY_FLAGS,	VALUE_CHECK_VERSION);
		pFile->SetProperty(PROPERTY_FLAGS,	VALUE_DONT_DELETE);
	}
	return TRUE;
}

BOOL SiConverter::ConvertDirectoriesSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while(aLine.Search('=') != STRING_NOTFOUND)
	{
		aLine.EraseAllChars();

		ByteString       aValue       = aLine.GetToken(1, '=');
		BOOL         bWorkstation = aValue.GetToken(0,',') == "0";
		SiDirEntry     aDir(aValue.GetToken(1,','),eFSysPathStyle);
		SiDirectory *pDir         = GetDirectory(aDir.GetFull(eFSysPathStyle));

		if (bWorkstation)
		{

			if (pDir->GetParent() == NULL
			|| !pDir->GetParent()->IsSystem())
				pDir->SetProperty(PROPERTY_FLAGS,VALUE_WORKSTATION);
		}

		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertArchive(USHORT nNo)
{
	SiFile *pFile = (SiFile*)Find(MakeID(ByteString("FILE")+ByteString(nNo)));

	pFile->SetProperty(PROPERTY_FLAGS,VALUE_ARCHIVE);

	do
	{
		ReadLine(m_aCurrentLine);
	}
	while (m_aCurrentLine.Match("File") == 4);

	return TRUE;
}

BOOL SiConverter::ConvertProgMod(USHORT nNo)
{
	ByteString    aLine;
	SiModule *pModule = new SiModule(SiIdentifier(MakeID(ByteString("PROGMOD")+ByteString(nNo))),m_pCompiler);

	pModule->SetProperty(PROPERTY_PARENTID,Find("ROOT"));

	// name =
	//
	ReadLine(aLine);

	if (aLine.GetToken(0,'=').Match("name") == 4)
	{
		pModule->SetProperty(PROPERTY_NAME,aLine.GetToken(1,'=').EraseLeadingChars());
		ReadLine(aLine);
	}

	// description =
	//

	if (aLine.GetToken(0,'=').Match("description") == 11)
	{
		ByteString aDesc = aLine.GetToken(1,'=').EraseLeadingChars();

		ReadLine(aLine);

		if (aLine.Search('=') == STRING_NOTFOUND)
		{
			// aDesc += '\n';
			aDesc += aLine;
			ReadLine(aLine);
		}

		pModule->SetProperty(PROPERTY_DESCRIPTION,aDesc);
	}

	m_pCS->Add(pModule,m_pCompiler);

	// sublist =
	//
	aLine.ToLower().EraseAllChars();

	if (aLine.GetToken(0,'=') == "sublist")
	{
		ByteString aValue = aLine.GetToken(1,'=');
		USHORT nToken = aValue.GetTokenCount(',');

		for (USHORT i=0; i<nToken; i++)
		{
			SiModule *pSubModule = new SiModule(SiIdentifier(MakeID(ByteString("MODULE")+aValue.GetToken(i,','))),m_pCompiler);

			pSubModule->SetProperty(PROPERTY_PARENTID,pModule);
			m_pCS->Add(pSubModule,m_pCompiler);
		}
		ReadLine(aLine);
	}

	// filelist =
	//
	if (aLine.GetToken(0,'=') == "filelist")
	{
		ByteString aValue = aLine.GetToken(1,'=');
		USHORT nToken = aValue.GetTokenCount(',');

		for (USHORT i=0; i<nToken; i++)
		{
			SiFile *pFile = (SiFile*)Find(MakeID(ByteString("FILE")+aValue.GetToken(i,',')));

			if (pFile != NULL)
				pModule->Add(pFile);
		}
		ReadLine(aLine);
	}
	return TRUE;
}

BOOL SiConverter::ConvertModule(USHORT nNo)
{
	ByteString    aLine;
	SiModule *pModule = (SiModule*)Find(MakeID(ByteString("MODULE")+ByteString(nNo)));

	// name =
	//
	ReadLine(aLine);

	if (aLine.GetToken(0,'=').Match("name") == 4)
	{
		pModule->SetProperty(PROPERTY_NAME,aLine.GetToken(1,'=').EraseLeadingChars());
		ReadLine(aLine);
	}

	// description =
	//
	if (aLine.GetToken(0,'=').Match("description") == 11)
	{
		ByteString aDesc = aLine.GetToken(1,'=').EraseLeadingChars();

		ReadLine(aLine);

		if (aLine.Search('=') == STRING_NOTFOUND)
		{
			// aDesc += '\n';
			aDesc += aLine;
			ReadLine(aLine);
		}

		pModule->SetProperty(PROPERTY_DESCRIPTION,aDesc);
	}

	m_pCS->Add(pModule,m_pCompiler);

	// sublist =
	//
	aLine.ToLower().EraseAllChars();

	if (aLine.GetToken(0,'=') == "sublist")
	{
		ByteString aValue = aLine.GetToken(1,'=');
		USHORT nToken = aValue.GetTokenCount(',');

		for (USHORT i=0; i<nToken; i++)
		{
			SiModule *pSubModule = new SiModule(SiIdentifier(MakeID(ByteString("OPTION")+aValue.GetToken(i,','))),m_pCompiler);

			pSubModule->SetProperty(PROPERTY_PARENTID,pModule);
			m_pCS->Add(pSubModule,m_pCompiler);
		}
		ReadLine(aLine);
	}

	// filelist =
	//
	if (aLine.GetToken(0,'=') == "filelist")
	{
		ByteString aValue = aLine.GetToken(1,'=');
		USHORT nToken = aValue.GetTokenCount(',');

		for (USHORT i=0; i<nToken; i++)
		{
			SiFile *pFile = (SiFile*)Find(MakeID(ByteString("FILE")+aValue.GetToken(i,',')));

			if (pFile != NULL)
				pModule->Add(pFile);
		}
		ReadLine(aLine);
	}
	return TRUE;
}

BOOL SiConverter::ConvertOption(USHORT nNo)
{
	ByteString    aLine;
	SiModule *pModule = (SiModule*)Find(MakeID(ByteString("OPTION")+ByteString(nNo)));

	// name =
	//
	ReadLine(aLine);

	if (aLine.GetToken(0,'=').Match("name") == 4)
	{
		pModule->SetProperty(PROPERTY_NAME,aLine.GetToken(1,'=').EraseLeadingChars());
		ReadLine(aLine);
	}

	// description =
	//
	if (aLine.GetToken(0,'=').Match("description") == 11)
	{
		ByteString aDesc = aLine.GetToken(1,'=').EraseLeadingChars();

		ReadLine(aLine);

		if (aLine.Search('=') == STRING_NOTFOUND)
		{
			// aDesc += '\n';
			aDesc += aLine;
			ReadLine(aLine);
		}

		pModule->SetProperty(PROPERTY_DESCRIPTION,aDesc);
	}

	m_pCS->Add(pModule,m_pCompiler);

	// filelist =
	//
	aLine.ToLower().EraseAllChars();

	if (aLine.GetToken(0,'=') == "filelist")
	{
		ByteString aValue = aLine.GetToken(1,'=');
		USHORT nToken = aValue.GetTokenCount(',');

		for (USHORT i=0; i<nToken; i++)
		{
			SiFile *pFile = (SiFile*)Find(MakeID(ByteString("FILE")+aValue.GetToken(i,',')));

			if (pFile != NULL)
				pModule->Add(pFile);
		}
		ReadLine(aLine);
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////
// Phase 1 reg.inf
/////////////////////////////////////////////////////////////////////

BOOL SiConverter::ConvertRegDBSection()
{
	int    nNo = 1;
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString    aValue     = aLine.GetToken(1,'=');
		SiModule *pModule    = GetModule(aValue);
		BOOL      bDontDelete= aValue.Copy().EraseAllChars().Match("99,99,99") == 8;
		USHORT    nEndSubKey = 0;
		ByteString    aSubKey    = aValue.GetToken(3,',',nEndSubKey).EraseLeadingChars().EraseTrailingChars();
		ByteString    aKeyValue  = aValue.Copy(nEndSubKey).EraseLeadingChars().EraseTrailingChars();
		ByteString    aName;
		USHORT    nNameSep   = aSubKey.Search("##");

		if (nNameSep != STRING_NOTFOUND)
		{
			aName = aSubKey.Copy(nNameSep+2);
			aSubKey.Erase(nNameSep);
		}

		SiRegistryItem *pParent = NULL;

		if (aSubKey.Match("HKEY_CLASSES_ROOT") == 17)
		{
			pParent = (SiRegistryItem*)Find("PREDEFINED_HKEY_CLASSES_ROOT");
			aSubKey.Erase(0,18);
		}
		else
		{
			DBG_ASSERT(aSubKey.Match("HKEY_LOCAL_MACHINE") == 18, "wrong key");
			pParent = (SiRegistryItem*)Find("PREDEFINED_HKEY_LOCAL_MACHINE");
			aSubKey.Erase(0,19);
		}

		SiRegistryItem *pItem = new SiRegistryItem(SiIdentifier(ByteString("RI")+ByteString(nNo)),m_pCompiler);

		pItem->SetProperty(PROPERTY_PARENTID, pParent);
		pItem->SetProperty(PROPERTY_MODULEID, pModule);
		pItem->SetProperty(PROPERTY_SUBKEY,   aSubKey);
		pItem->SetProperty(PROPERTY_NAME,     aName);
		pItem->SetProperty(PROPERTY_VALUE,    aKeyValue);

		if (bDontDelete)
			pItem->SetProperty(PROPERTY_FLAGS, VALUE_DONT_DELETE);
		else
			pItem->SetProperty(PROPERTY_FLAGS, VALUE_DELETE_ALL);

		pModule->Add(pItem);
		m_pCS->Add(pItem,m_pCompiler);

		ReadLine(aLine);
		nNo++;
	}

	return TRUE;
}

BOOL SiConverter::ConvertCLSIDSectionSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		if (m_CLSIDSections.Len() != 0)
			m_CLSIDSections += ',';

		m_CLSIDSections += aLine.GetToken(1,'=');

		ReadLine(aLine);
	}


	return TRUE;
}

BOOL SiConverter::ConvertCLSIDKeysSection()
{
	int       nNo = 1;
	ByteString    aLine;
	SiModule *pModule = GetModule("ROOT");

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString aValue = aLine.GetToken(1,'=');

		for (USHORT i=0; i<m_CLSIDSections.GetTokenCount(','); i++)
		{
			SiRegistryArea *pItem = new SiRegistryArea(SiIdentifier(ByteString("RA")+ByteString(nNo*10+i)),m_pCompiler);

			pItem->SetProperty(PROPERTY_MODULEID, pModule);
			pItem->SetProperty(PROPERTY_SUBKEY,   m_CLSIDSections.GetToken(i,','));
			pItem->SetProperty(PROPERTY_FROMKEY,  aValue.GetToken(0,','));
			pItem->SetProperty(PROPERTY_TOKEY,    aValue.GetToken(1,','));

			pModule->Add(pItem);
			m_pCS->Add(pItem,m_pCompiler);
		}

		ReadLine(aLine);
		nNo++;
	}

	return TRUE;
}

/////////////////////////////////////////////////////////////////////
// Phase 2
/////////////////////////////////////////////////////////////////////

BOOL SiConverter::ConvertFontsSection()
{
	ByteString aLine;

	ReadLine(aLine);
	aLine.EraseAllChars();

	while(aLine.Match("font") == 4)
	{
		ByteString    aValue  = aLine.GetToken(1,'=');
		SiModule *pModule = GetModule(aValue);
		SiFile   *pFile   = (SiFile*)Find(MakeID(ByteString("FILE")+aValue.GetToken(4,',')));
		ByteString    aFileExt = SiDirEntry(pFile->GetName()).GetExtension().ToLower();

		if (aFileExt == "pfb")
		{
			pFile->SetProperty(PROPERTY_NETDIR, (SiFile*)pFile->GetDirectory());
			pFile->SetProperty(PROPERTY_DIR,    Find("PREDEFINED_OSSYSTEMFONTDIR"));
		}
		else
		{
			pFile->SetProperty(PROPERTY_FLAGS, VALUE_FONT);
			pFile->SetProperty(PROPERTY_FONTNAME, aValue.GetToken(3,','));
		}
		pModule->Add(pFile);

		ReadLine(aLine);
		aLine.EraseAllChars();
	}

	return TRUE;
}

BOOL SiConverter::ConvertMinimalDefaultSection(BOOL bMinimal)
{
	BOOL	bContinue = TRUE;
	ByteString	aLine;

	while (bContinue)
	{
		ByteString aName;

		ReadLine(aLine);
		aLine.ToLower().EraseAllChars();
		ByteString aKey = aLine.GetToken(0,'=');

			 if (aKey == "progmodlist") aName = "PROGMOD";
		else if (aKey == "modulelist")  aName = "MODULE";
		else if (aKey == "optionlist")  aName = "OPTION";
		else bContinue = FALSE;

		if (bContinue)
		{
			ByteString aValue = aLine.GetToken(1,'=');
			USHORT nToken = aValue.GetTokenCount(',');

			for (USHORT i=0; i<nToken; i++)
			{
				SiModule *pModule = (SiModule*) Find(MakeID(aName+aValue.GetToken(i,',')));

				if (pModule)
					pModule->SetProperty(bMinimal ? PROPERTY_MINIMAL : PROPERTY_DEFAULT, VALUE_YES);
			}
		}
	}

	return TRUE;
}

BOOL SiConverter::ConvertProfilesSection()
{
	ByteString aLine;

	ReadLine(aLine);
	aLine.ToLower().EraseAllChars();

	while (aLine.Match("profile") == 7)
	{
		if (aLine.GetToken(4,',') != "-")
		{
			ByteString     aValue  = aLine.GetToken(1,'=');
			SiModule  *pModule = GetModule(aValue);
			SiDirEntry   aFile(aValue.GetToken(4,','),eFSysPathStyle);
			ByteString     aPath   = aValue.GetToken(5,',');
			ByteString	   aName   = MakeID(aValue.GetToken(3,','));
			BOOL       bDelete = TRUE;

			// fuer Win16
			if (aName.GetTokenCount(';') > 1)
			{
				if (aName.GetToken(2,';').EraseAllChars().Lower() == "nd")
					bDelete = FALSE;

				aName = aName.GetToken(0,';');
			}

			SiProfile *pProfile= new SiProfile(SiIdentifier(MakeID(aName)), m_pCompiler);

			pProfile->SetProperty(PROPERTY_NAME,aFile.GetName());
			if (bDelete)
				pProfile->SetProperty(PROPERTY_MODULEID, pModule);
			pProfile->SetProperty(PROPERTY_DIR,GetDirectory(aFile.GetPath().GetFull(eFSysPathStyle)));

			if (aPath == "progpath")
				pProfile->SetProperty(PROPERTY_FLAGS,VALUE_NETWORK);

			if (aPath == "ossystem")
			{
				pProfile->SetProperty(PROPERTY_DIR, Find("PREDEFINED_OSDIR"));
				pProfile->SetProperty(PROPERTY_FLAGS,VALUE_DONT_DELETE);
			}

			m_pCS->Add(pProfile,m_pCompiler);
			m_Profiles.Insert(pProfile->GetID(),(void*)1);
		}

		ReadLine(aLine);
		aLine.ToLower().EraseAllChars();
	}
	return TRUE;
}

BOOL SiConverter::ConvertProfileItemSection(ByteString const& aProfileID)
{
	ByteString  aLine;
	int     nCount=1;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString         aValue  = aLine.Copy(aLine.Search("=")+1).EraseLeadingChars();
		SiProfileItem *pItem   = new SiProfileItem(SiIdentifier(aProfileID+ByteString(nCount)),m_pCompiler);
		SiModule	  *pModule = GetModule(aValue);
		SiProfile	  *pProfile= (SiProfile*)Find(aProfileID);

		// Wenn der Eintrag an dem Wurzelmodul haengt, dann
		// wird er an das Modul des Profiles gehaengt.
		//
		if (pModule->GetParent()  == NULL
		&&  pProfile->GetModule() != NULL)
			pModule = (SiModule*)pProfile->GetModule();

		pItem->SetProperty(PROPERTY_MODULEID,   pModule);
		pItem->SetProperty(PROPERTY_PROFILEID,  pProfile);
		pItem->SetProperty(PROPERTY_SECTION,    aValue.GetToken(3,',').EraseLeadingChars().EraseTrailingChars());
	#if SUPD > 336
		pItem->SetProperty(PROPERTY_ORDER,		nCount);
	#endif
		USHORT nKeyEnd = 0;

		// pruefen ob hinten "nd" Flag anhaengt, wenn ja abschneiden
		ByteString aDontDelete = aValue.GetToken(6, ',', nKeyEnd);
		if( aDontDelete.CompareIgnoreCaseToAscii("nd") == COMPARE_EQUAL )
		{
			pItem->SetProperty(PROPERTY_FLAGS, VALUE_DONT_DELETE);
			aValue.Erase(aValue.Len()-3);
		}

		nKeyEnd = 0;
		ByteString aKey = aValue.GetToken(4, ',', nKeyEnd);

		pItem->SetProperty(PROPERTY_KEY,    aKey.EraseLeadingChars().EraseTrailingChars());
		pItem->SetProperty(PROPERTY_VALUE,  aValue.Copy(nKeyEnd).EraseLeadingChars().EraseTrailingChars());

		if (aProfileID.CompareIgnoreCaseToAscii("VERSIONSKONTROLLE") == COMPARE_EQUAL)
//		&&  aValue.Search("<workpath>") != STRING_NOTFOUND)
			pItem->SetProperty(PROPERTY_FLAGS,VALUE_INSTALL_INFO);

		pModule->Add(pItem);
		m_pCS->Add(pItem,m_pCompiler);

		nCount++;
		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertProgmanSection()
{
	ByteString  aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString    aValue  = aLine.GetToken(1,'=');
		SiModule *pModule = GetModule(aValue);
		ByteString    aFolder = aValue.GetToken(3,',');
		ByteString    anItem  = aValue.GetToken(4,',').EraseLeadingChars();
		ByteString    aFile   = SiDirEntry(aValue.GetToken(5,','),eFSysPathStyle).GetName().Upper();

		if (m_SpezialFiles.Find(aFile) != NULL)
		{
			SiFolder *pFolder = NULL;

			if (aFolder.CompareIgnoreCaseToAscii("Autostart") == COMPARE_EQUAL)
			{
				pFolder = GetFolder("PREDEFINED_AUTOSTART");
			}
			else
			{
				pFolder = GetFolder(MakeID(aFolder).EraseAllChars('.'));
				pFolder->SetProperty(PROPERTY_NAME, aFolder);
			}

			SiFolderItem *pItem = new SiFolderItem(SiIdentifier(MakeID(pFolder->GetID()+anItem).EraseAllChars('.')),m_pCompiler);

			pItem->SetProperty(PROPERTY_NAME,     anItem);
			pItem->SetProperty(PROPERTY_FOLDERID, pFolder);
			pItem->SetProperty(PROPERTY_MODULEID, pModule);
			pItem->SetProperty(PROPERTY_FILEID,   m_SpezialFiles.Find(aFile));
			m_pCS->Add(pItem,m_pCompiler);
		}
		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertFolderSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString aValue = aLine.GetToken(1,'=');
		ByteString anID   = aValue.GetToken(4,',');
			   anID   = anID.Copy(1,anID.Len()-2);

		SiFolder *pFolder    = GetFolder(anID);
		SiFile   *pCloseIcon = new SiFile(SiIdentifier(MakeID("FOLDER0_ICO")),m_pCompiler); // steht leider nicht im Script!
		SiFile   *pOpenIcon  = new SiFile(SiIdentifier(MakeID("FOLDER1_ICO")),m_pCompiler); // steht leider nicht im Script!

		pCloseIcon->SetProperty(PROPERTY_NAME,   "folder0.ico");
		pCloseIcon->SetProperty(PROPERTY_CARRIER, Find("DISK1"));
		pCloseIcon->SetProperty(PROPERTY_DIR,    GetDirectory("WORKPATH"));
		pCloseIcon->SetProperty(PROPERTY_FLAGS,  VALUE_DELETE_ONLY); // wurde durch f_0000 erzeugt
		m_pCS->Add(pCloseIcon,m_pCompiler);

		pOpenIcon->SetProperty(PROPERTY_NAME,    "folder1.ico");
		pOpenIcon->SetProperty(PROPERTY_CARRIER, Find("DISK1"));
		pOpenIcon->SetProperty(PROPERTY_DIR,     GetDirectory("WORKPATH"));
		pOpenIcon->SetProperty(PROPERTY_FLAGS,   VALUE_DELETE_ONLY); // wurde durch f_0000 erzeugt
		m_pCS->Add(pOpenIcon,m_pCompiler);

		pFolder->SetProperty(PROPERTY_OS2_ID,anID);
		pFolder->SetProperty(PROPERTY_NAME,aValue.GetToken(5,','));
		pFolder->SetProperty(PROPERTY_OS2_CLOSEICON,pCloseIcon);
		pFolder->SetProperty(PROPERTY_OS2_OPENICON,pOpenIcon);

		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertWPSSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString aValue = aLine.GetToken(1,'=').EraseLeadingChars();

		SiOs2Class *pClass = new SiOs2Class(SiIdentifier(aValue.GetToken(5,',').ToUpper()),m_pCompiler);

		pClass->SetProperty(PROPERTY_MODULEID,GetModule(aValue));
		pClass->SetProperty(PROPERTY_NAME,aValue.GetToken(5,','));
		pClass->SetProperty(PROPERTY_DLL,m_SpezialFiles.Find(aValue.GetToken(4,',').ToUpper()));

		m_pCS->Add(pClass,m_pCompiler);

		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertObjectsSection()
{
	ByteString aLine;
	ByteString aFolder;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString aValue       = aLine.Copy(aLine.Search("=")+1).EraseLeadingChars();
			   aFolder      = aValue.GetToken(3,',').GetToken(0,';');
			   aFolder      = aFolder.Copy(1,aFolder.Len()-2);
		ByteString aFile        = aValue.GetToken(5,',').GetToken(0,';').EraseAllChars();
			   aFile        = SiDirEntry(aFile,eFSysPathStyle).GetName().ToUpper();
		ByteString anID         = aValue.GetToken(4,',').GetToken(0,';').EraseAllChars();
			   anID         = ByteString("FI_") + anID.Copy(1,anID.Len()-2);
		ByteString aClassID     = aValue.GetToken(4,',').GetToken(1,';').ToUpper();
		SiModule *pModule   = GetModule(aValue);

		if (aClassID == "WPPROGRAM")
			aClassID = "PREDEFINED_OS2_PROGRAMCLASS";

		SiFolderItem *pItem = new SiFolderItem(SiIdentifier(anID),m_pCompiler);

		pItem->SetProperty(PROPERTY_MODULEID,       GetModule(aValue));
		pItem->SetProperty(PROPERTY_NAME,           aValue.GetToken(3,',').GetToken(1,';'));
		pItem->SetProperty(PROPERTY_FOLDERID,       GetFolder(aFolder));
		pItem->SetProperty(PROPERTY_FILEID,         m_SpezialFiles.Find(aFile));
		pItem->SetProperty(PROPERTY_OS2_ID,         anID);
		pItem->SetProperty(PROPERTY_OS2_CLASSID,    Find(aClassID));

		if (aValue.Search("ASSOCFILTER=") != STRING_NOTFOUND)
		{
			ByteString aFilterList  = aValue.Copy(aValue.Search("ASSOCFILTER=")+12);
				   aFilterList  = aFilterList.GetToken(0,';');

			pItem->SetProperty(PROPERTY_OS2_FILTERS, aFilterList);
		}

		// Creator erzeugen
		//
		if (aValue.Search("ASSOCTYPE=") != STRING_NOTFOUND)
		{
			ByteString        aCreatorName = aValue.Copy(aValue.Search("ASSOCTYPE=")+10);
						  aCreatorName = aCreatorName.GetToken(0,';');
			SiOs2Creator *pCreator     = new SiOs2Creator(SiIdentifier(MakeID(ByteString("CREATOR_")+ByteString(aCreatorName))),m_pCompiler);
			ByteString        anIconFile   = aValue.Copy(aValue.Search("ICONFILE=")+9);
						  anIconFile   = anIconFile.GetToken(1,'\\');

			pItem->SetProperty(PROPERTY_OS2_CREATORID, pCreator);
			pCreator->SetProperty(PROPERTY_NAME, aCreatorName);

			if (anIconFile.Len() > 0)
			{
				pCreator->SetProperty(PROPERTY_ICON,    anIconFile);
				pCreator->SetProperty(PROPERTY_ICONDIR, GetDirectory("WORKPATH"));
			}

			m_pCS->Add(pCreator,m_pCompiler);
		}

		// Kopie in Autostart
		//
		if (aValue.GetToken(4,',').GetToken(2,';') == "<WP_START>")
		{
			SiFolderItem *pItem2 = new SiFolderItem(SiIdentifier(anID+ByteString("2")),m_pCompiler);

			pItem2->SetProperty(PROPERTY_MODULEID,      pModule);
			pItem2->SetProperty(PROPERTY_NAME,          aValue.GetToken(3,',').GetToken(1,';'));
			pItem2->SetProperty(PROPERTY_FOLDERID,      Find("PREDEFINED_AUTOSTART"));
			pItem2->SetProperty(PROPERTY_OS2_ID,        anID+ByteString("2"));
			pItem2->SetProperty(PROPERTY_OS2_REFERENCEID, pItem);
			pModule->Add(pItem2);
			m_pCS->Add(pItem2,m_pCompiler);
		}

		pModule->Add(pItem);
		m_pCS->Add(pItem,m_pCompiler);

		ReadLine(aLine);
	}

	SiFolderItem *pSetupItem = new SiFolderItem(SiIdentifier("SETUP_FOLDERITEM"),m_pCompiler);

	pSetupItem->SetProperty(PROPERTY_MODULEID,      GetModule("ROOT"));
	pSetupItem->SetProperty(PROPERTY_NAME,          "Setup");
	pSetupItem->SetProperty(PROPERTY_FOLDERID,      GetFolder(aFolder));
	pSetupItem->SetProperty(PROPERTY_FILEID,        Find("SETUP_ZIP"));
	pSetupItem->SetProperty(PROPERTY_OS2_ID,        "Setup2Beta");
	pSetupItem->SetProperty(PROPERTY_OS2_CLASSID,   Find("PREDEFINED_OS2_PROGRAMCLASS"));
	GetModule("ROOT")->Add(pSetupItem);
	m_pCS->Add(pSetupItem,m_pCompiler);

	return TRUE;
}

BOOL SiConverter::ConvertTemplatesSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString         aValue  = aLine.Copy(aLine.Search('=')+1).EraseLeadingChars();
		SiModule      *pModule = GetModule(aValue);
		ByteString         anID    = aValue.GetToken(4,',').GetToken(0,';');
					   anID    = anID.Copy(1,anID.Len()-2);
		ByteString         aClass  = aValue.GetToken(4,',').GetToken(1,';').ToUpper();
		ByteString         anIconName = aValue.GetToken(5,',');

		SiFile        *pFile     = new SiFile(SiIdentifier(MakeID(anIconName)),m_pCompiler); // steht leider nicht im Script!
		SiOs2Template *pTemplate = new SiOs2Template(SiIdentifier(anID),m_pCompiler);

		pFile->SetProperty(PROPERTY_NAME,    anIconName);
		pFile->SetProperty(PROPERTY_CARRIER, Find("DISK1"));
		pFile->SetProperty(PROPERTY_DIR,     GetDirectory("WORKPATH"));
		pFile->SetProperty(PROPERTY_FLAGS,   VALUE_DELETE_ONLY); // wurde durch f_0000 erzeugt

		pTemplate->SetProperty(PROPERTY_OS2_CLASSID, Find(aClass));
		pTemplate->SetProperty(PROPERTY_ID,          anID);
		pTemplate->SetProperty(PROPERTY_ICON,        pFile);

		m_pCS->Add(pTemplate,m_pCompiler);
		m_pCS->Add(pFile,m_pCompiler);
		pModule->Add(pTemplate);

		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertEASection()
{
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("line") == 4)
	{
		ByteString aValue    = aLine.Copy(aLine.Search('=')+1).EraseLeadingChars();
		ByteString aCreator  = MakeID(ByteString("CREATOR_")+aValue.GetToken(0,','));

		for (USHORT i=1; i<aValue.GetTokenCount(','); i++)
		{
			ByteString  aFileID  = "FILE";
					aFileID += aValue.GetToken(i,',').EraseAllChars();
			SiFile *pFile    = (SiFile*)Find(aFileID);

			pFile->SetProperty(PROPERTY_OS2_CREATORID, Find(aCreator));
		}

		ReadLine(aLine);
	}

	return TRUE;
}

BOOL SiConverter::ConvertApplicationSection()
{
	ByteString aLine;

	ReadLine(aLine);

	while (aLine.Match("Line") == 4)
	{
		ByteString aValue = aLine.Copy(aLine.Search('=')+1).EraseLeadingChars();
		SiFile *pFile = (SiFile*)m_SpezialFiles.Find(aValue.GetToken(4,',').ToUpper());

		if (pFile != NULL)
			pFile->SetProperty(PROPERTY_MAC_CREATOR, aValue.GetToken(5,','));

		ReadLine(aLine);
	}

	return TRUE;
}
