/*************************************************************************
 *
 *  $RCSfile: sidacar.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "decltor.hxx"
#include "compiler.hxx"
#include "fields.hxx"
#include "instdb.hxx"

TYPEINIT1(SiDataCarrier,SiDeclarator)

///////////////////////////////////////////////////////////////////////////////
//
//		SiDataCarrier
//

SiDataCarrier::SiDataCarrier(SiIdentifier const& ID, SiCompiler* pCompiler) :
		SiDeclarator(ID,pCompiler)
{
	INIT_MULTI_LANG_MEMBER( aName			);
	INIT_MULTI_LANG_MEMBER( nDiskNo			);
	INIT_MULTI_LANG_MEMBER( aArchive		);
	INIT_MULTI_LANG_MEMBER( Flags			);

	m_nDiskNo	= 0;
	m_Flags		= 0;
}

// ----------------------------------------------------------------------------

void SiDataCarrier::JoinWithParent()
{
	if( !IsLangRef() )
		return ;

	SiDataCarrier* pParent = (SiDataCarrier*) m_pLangParent;

	JOIN_MULTI_LANG_MEMBER( aName			);
	JOIN_MULTI_LANG_MEMBER( nDiskNo			);
	JOIN_MULTI_LANG_MEMBER( aArchive		);
	JOIN_MULTI_LANG_MEMBER( Flags			);
}

// ----------------------------------------------------------------------------

BOOL SiDataCarrier::SetProperty(ByteString const& aProperty, ByteString const& aValue)
{
	if( aProperty == PROPERTY_NAME )
	{
		SET_MULTI_LANG_MEMBER( aName, aValue);
		return TRUE;
	}

	if( aProperty == PROPERTY_ARCHIVE )
	{
		SET_MULTI_LANG_MEMBER( aArchive, aValue);
		return TRUE;
	}

	if( aProperty == PROPERTY_TYPE )
	{
		if( aValue == VALUE_DISKETTE )
		{
			SET_MULTI_LANG_MEMBER( Flags, DCFLAG_DISKETTE );
			return TRUE;
		}
		if( aValue == VALUE_CD )
		{
			SET_MULTI_LANG_MEMBER( Flags, DCFLAG_CD );
			return TRUE;
		}

		ByteString aMsg( "unknown value " );
		aMsg += aValue;
		Error( aMsg );

		return FALSE;
	}

	return SiDeclarator::SetProperty(aProperty,aValue);
}

// ----------------------------------------------------------------------------

BOOL SiDataCarrier::SetProperty(ByteString const& aProperty, long aValue)
{
	if( aProperty == PROPERTY_DISKNO )
	{
		SET_MULTI_LANG_MEMBER( nDiskNo, (int) aValue );
		return TRUE;
	}

	return SiDeclarator::SetProperty(aProperty,aValue);
}

// ----------------------------------------------------------------------------

BOOL SiDataCarrier::Check()
{
	if( !CheckField(m_nDiskNo, PROPERTY_DISKNO) )
		return FALSE;
	else
		return SiDeclarator::Check();
}

// ----------------------------------------------------------------------------

BOOL SiDataCarrier::WriteTo(SiDatabase& aStream) const
{
	if( !IsLangRef() )
		aStream.BeginDeclaration( "DataCarrier", this );

	// PROPERTY_NAME
	WRITE_MULTI_LANG_MEMBER( PROPERTY_NAME, aName );
	// PROPERTY_DISKNO
	WRITE_MULTI_LANG_MEMBER( PROPERTY_DISKNO, nDiskNo );
	// PROPERTY_ARCHIVE
	WRITE_MULTI_LANG_MEMBER( PROPERTY_ARCHIVE, aArchive );

	switch( m_Flags )
	{
		case DCFLAG_DISKETTE:
		{
			BOOL m_Isatmp = TRUE;
			SiIdentifier m_atmp(VALUE_DISKETTE);
			WRITE_MULTI_LANG_MEMBER( PROPERTY_FLAGS, atmp );
		} break;

		case DCFLAG_CD:
		{
			BOOL m_Isatmp = TRUE;
			SiIdentifier m_atmp(VALUE_CD);
			WRITE_MULTI_LANG_MEMBER( PROPERTY_FLAGS, atmp );
		} break;
	}

	for( USHORT nIdx = 0; nIdx < m_aLangRefs.Count(); ++nIdx )
	{
		SiDataCarrier* pDCarrier = (SiDataCarrier*) m_aLangRefs.GetObject(nIdx);
		pDCarrier->WriteTo( aStream );
	}

	if( !IsLangRef() )
		aStream.EndDeclaration();
	return TRUE;
}

ByteString SiDataCarrier::GetNaturalID() const
{
	ByteString aNatID(m_aName);
	if( IsLangRef() ) {
		aNatID += "__";
		aNatID += m_nLanguage;
	}
	return aNatID;
}

