/*************************************************************************
 *
 *  $RCSfile: appserv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:04 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif

#include <vos/process.hxx>
#include <sifsys.hxx>
#include <vos/profile.hxx>
#include <vos/socket.hxx>

#include "environ.hxx"
#include "sicustom.hxx"
#include "appserv.hxx"
#include "aservdlg.hrc"
#include <vcl/svapp.hxx>
#include <vos/thread.hxx>
#ifdef _USE_NAMESPACE
using namespace vos;
#endif

#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

#ifdef _USE_NAMESPACE
using namespace vos;
#endif

//------------------------------------------------------------------------

static SiSetupFnc 	setupFunction;
static BOOL			bWorkAroundWin32SecurityBug = FALSE;

extern "C"
{
	BOOL _SV_CALL CustomInit
	(
		SiSetupFnc*		pSetupFunction,
		SiCustomFnc*	pCustomFunction
	)
	{
		BOOL	bResult = FALSE;

		if( pSetupFunction != 0 && pSetupFunction -> nVersion <= SICUSTOM_VERSION )

		{
			pCustomFunction -> fncMain = ( FncPtrCustomMain ) CustomMain;

			setupFunction.nVersion	   = pSetupFunction -> nVersion;
			setupFunction.fncHideSetup = pSetupFunction -> fncHideSetup;
			setupFunction.fncShowSetup = pSetupFunction -> fncShowSetup;

			bResult = TRUE;
		}

		return bResult;
	}

	BOOL _SV_CALL CustomMain
	(
		SiCustomEnvironment*	pEnvironment,
		SiCustomModuleList*		pListOfModules
	)
	{
		BOOL					bResult = 				FALSE;
		BOOL					bInstallation = 		pEnvironment -> IsInstallation();
		BOOL					bBatchMode = 			pEnvironment -> IsResponseFileMode();
		String					sNameOfBatchFile( 		String::CreateFromAscii( pEnvironment->GetResponseFileName().GetBuffer()) );
		String					sInstallDir(	 		String::CreateFromAscii( pEnvironment->GetDestPath( ).GetBuffer()) );
		String					sSetupDir(				String::CreateFromAscii( pEnvironment->GetStartPath().GetBuffer()) );
		String 					sProductName( 			String::CreateFromAscii( pEnvironment->GetBigEnv()->GetProductName().GetBuffer()) );

		if( sInstallDir.Len( ) > 0 )
		{
			PlugInServerSetUp aPlugInServerSetUp(	sSetupDir,
													sInstallDir,
													sNameOfBatchFile,
													bBatchMode,
													bInstallation);

			if( pEnvironment -> IsPre( ) )
			{
				bResult = aPlugInServerSetUp.mainPreInstallation( );
			}
			else
			{
				bResult = aPlugInServerSetUp.mainPostInstallation( sProductName );
			}
		}

		return bResult;
	}
};

//------------------------------------------------------------------------

PlugInServerSetUp::PlugInServerSetUp
(
	const String	aSetupDir,
	const String	aInstallDir,
	const String	sNameOfBatchFile,
	const BOOL		bBatchMode,
	const BOOL		bInstallation
)
:	m_sSetupDir(aSetupDir),
	m_sInstallDir(aInstallDir),
	m_sBatchFile(sNameOfBatchFile),
	m_bBatchMode(bBatchMode),
	m_bInstallation(bInstallation),
	m_pDialog(NULL),
	m_pInfo(NULL)
{

#if defined UNX

	m_bPlatformIsUnix	= TRUE;
	m_bPlatformIsWin32	= FALSE;

#elif defined WIN32

	m_bPlatformIsUnix	= FALSE;
	m_bPlatformIsWin32	= TRUE;

#endif

	m_pRscMgr				= 0;
	m_bIsDaemonRcOpen		= FALSE;
	m_pDialog				= 0;
}

PlugInServerSetUp::~PlugInServerSetUp( )
{
	if (m_pDialog!=NULL)
	{
		delete m_pDialog;
	}
	if (m_pInfo!=NULL)
	{
		delete m_pInfo;
	}
}

//------------------------------------------------------------------------

BOOL PlugInServerSetUp::mainPreInstallation()
{
	// read ressources for this setup-plugin
	if ( !readResourceFile() && !batchModeEnabled() )
	{
		String sErrorMessage = String::CreateFromAscii( "Could not open resource file!");
		InfoBox( 0, sErrorMessage ).Execute( );
		return FALSE;
	}

//InfoBox( 0, String::CreateFromAscii( "6") ).Execute( );

	// check if the user is Admin or is able to become Admin

	m_bIsAdmin = checkAdmin(TRUE);

	if( !m_bInstallation)
	{
		// find the script that will control the deinstallation

		if ( openDaemonRc() )
		{
			// uninstall the components of the server
			executeSectionOfScript( "stop" );
			executeSectionOfScript( "uninstall" );
		}
	}

	return TRUE;
}

BOOL PlugInServerSetUp::mainPostInstallation(const String sProductName)
{
	String	sPortalPort;
	String	sPortalAdminPort;
	String	sSetupServerPort;
	String	sShutdownPort;
	String	sLBMPort;
	String	sLBShttpPort;
	String	sLBShttpsPort;
	String	sLBSShutdownPort;
	String	sLBSunoPort;
	String	sLBSunoCompPort;
	String	sLBSunoTLSPort;
	String	sLBSunoCompTLSPort;
	String	sSPdaemonPort;
	String	sConfigServerPort;
	BOOL	bInstallDaemons = FALSE;
	BOOL	bStartDaemons	= FALSE;

	if( !m_bInstallation )
	{
		return TRUE;
	}

//InfoBox( 0, String::CreateFromAscii( "2") ).Execute( );

	readResourceFile();
	checkPackages();
	openDaemonRc();
	m_bIsAdmin = checkAdmin(FALSE);

	if( batchModeEnabled( ) )
	{
		char		cPortalPort[31+1]		= "";
		char		cPortalAdminPort[31+1]	= "";
		char		cSetupServerPort[31+1]	= "";
		char		cShutdownPort[31+1]		= "";
		char		cLBMPort[31+1]			= "";
		char		cLBShttpPort[31+1]		= "";
		char		cLBShttpsPort[31+1]		= "";
		char		cLBSShutdownPort[31+1]	= "";
		char		cLBSunoPort[31+1]		= "";
		char		cLBSunoCompPort[31+1]	= "";
		char		cLBSunoTLSPort[31+1]	= "";
		char		cLBSunoCompTLSPort[31+1]= "";
		char		cSPdaemonPort[31+1]		= "";
		char		cConfigServerPort[31+1]	= "";

        /* mfe: Thank you very much indeed Thomas!!!
         *      File Access has to be in UNC!!!
         */
        
        String sTmp = String::CreateFromAscii("//.");
        
        sTmp += m_sBatchFile;
        
		OProfile	aProfile( sTmp );

		bInstallDaemons			= aProfile.readBool( "Portal", "InstallServices", FALSE );
		bStartDaemons			= aProfile.readBool( "Portal", "StartServices", FALSE );

		aProfile.readString( "Portal", "http.port.portal", 					cPortalPort, 		31, "8081" );
		aProfile.readString( "Portal", "http.port.admin", 					cPortalAdminPort,	31, "8082" );
		aProfile.readString( "Portal", "uno.port.setup", 					cSetupServerPort,	31, "8209" );
		aProfile.readString( "Portal", "tcp.port.shutdown", 				cShutdownPort,		31, "8005" );
		aProfile.readString( "Portal", "socket.port.LBM", 					cLBMPort, 			31,	"8200" );
		aProfile.readString( "Portal", "http.port.LBShttp", 				cLBShttpPort, 		31, "8080" );
		aProfile.readString( "Portal", "https.port.LBShttp", 				cLBShttpsPort, 		31, "8443" );
		aProfile.readString( "Portal", "tcp.LBShttp.shutdown",				cLBSShutdownPort,	31, "8006" );
		aProfile.readString( "Portal", "portal.port.LBSuno", 				cLBSunoPort, 		31, "8201" );
		aProfile.readString( "Portal", "portal.port.LBSunoCompressed", 		cLBSunoCompPort, 	31, "8206" );
		aProfile.readString( "Portal", "portal.port.LBSunoTLS", 			cLBSunoTLSPort, 	31, "8207" );
		aProfile.readString( "Portal", "portal.port.LBSunoCompressedTLS", 	cLBSunoCompTLSPort, 31, "8208" );
		aProfile.readString( "Portal", "portal.port.sportald", 				cSPdaemonPort, 		31, "8202" );
		aProfile.readString( "Portal", "portal.port.ConfigServer", 			cConfigServerPort, 	31, "8205" );

		sPortalPort			= String::CreateFromAscii(cPortalPort);
		sPortalAdminPort	= String::CreateFromAscii(cPortalAdminPort);
		sSetupServerPort	= String::CreateFromAscii(cSetupServerPort);
		sShutdownPort		= String::CreateFromAscii(cShutdownPort);
		sLBMPort			= String::CreateFromAscii(cLBMPort);
		sLBShttpPort		= String::CreateFromAscii(cLBShttpPort);
		sLBShttpsPort		= String::CreateFromAscii(cLBShttpsPort);
		sLBSShutdownPort	= String::CreateFromAscii(cLBSShutdownPort);
		sLBSunoPort			= String::CreateFromAscii(cLBSunoPort);	
		sLBSunoCompPort		= String::CreateFromAscii(cLBSunoCompPort);
		sLBSunoTLSPort		= String::CreateFromAscii(cLBSunoTLSPort);
		sLBSunoCompTLSPort	= String::CreateFromAscii(cLBSunoCompTLSPort);
		sSPdaemonPort		= String::CreateFromAscii(cSPdaemonPort);
		sConfigServerPort	= String::CreateFromAscii(cConfigServerPort);
	}
	else
	{
		// read ressources for this setup-plugin
		if ( !haveRessourceManager() )
		{
			String sErrorMessage = String::CreateFromAscii( "Could not open resource file!");
			InfoBox( 0, sErrorMessage ).Execute( );
			return FALSE;
		}

		openDialog( sProductName );
	
		m_pInfo = new ServerSetupInfoWindow(m_pRscMgr);

		// check, if ports are in use
		// the following loop is executed until all ports are free or ignored
		// the ignore state is preserved, so the user mustn't ignore the same busy port over and over again

		BOOL	bPortalPortChange		= m_bPortalInstalled;
		BOOL	bPortalAdminPortChange	= m_bPortalAdminInstalled;
		BOOL	bSetupServerPortChange	= m_bSetupServerInstalled;
		BOOL	bShutdownPortChange		= m_bShutdownInstalled;
		BOOL	bLBMPortChange			= m_bLBMInstalled;
		BOOL	bLBShttpPortChange		= m_bLBShttpInstalled;
		BOOL	bLBShttpsPortChange		= m_bLBShttpInstalled;
		BOOL	bLBSShutdownPortChange	= m_bLBShttpInstalled;
		BOOL	bLBSunoPortChange		= m_bLBSunoInstalled;
		BOOL	bLBSunoCompPortChange	= m_bLBSunoInstalled;
		BOOL	bLBSunoTLSPortChange	= m_bLBSunoInstalled;
		BOOL	bLBSunoCompTLSPortChange= m_bLBSunoInstalled;
		BOOL	bSPdaemonPortChange		= m_bSPdaemonInstalled;
		BOOL	bConfigServerPortChange = m_bConfigServerInstalled;
		BOOL	bChangePort				= TRUE;

		while( bChangePort )
		{
			// display and execute the dialog for starting the services
			int nResultOfExecute = executeDialog( m_bIsDaemonRcOpen );
			
			if (nResultOfExecute != 1)
			{
				return FALSE;
			}

			m_pInfo->execute();
			m_pInfo->changeInfo(InfoTextDescriptor_PORTS);

			// user wants to install the components of the server
			sPortalPort			= m_pDialog->getPortal();
			sPortalAdminPort	= m_pDialog->getPortalAdmin();
			sSetupServerPort	= m_pDialog->getSetupServer();
			sShutdownPort		= m_pDialog->getShutdown();
			sLBMPort			= m_pDialog->getLBM();
			sLBShttpPort		= m_pDialog->getLBShttp();
			sLBShttpsPort		= m_pDialog->getLBShttps();
			sLBSShutdownPort	= m_pDialog->getLBSShutdown();
			sLBSunoPort			= m_pDialog->getLBSuno();
			sLBSunoCompPort		= m_pDialog->getLBSunoComp();
			sLBSunoTLSPort		= m_pDialog->getLBSunoTLS();
			sLBSunoCompTLSPort	= m_pDialog->getLBSunoCompTLS();
			sSPdaemonPort		= m_pDialog->getSPdaemon();
			sConfigServerPort	= m_pDialog->getConfigServer();
//			bInstallDaemons 	= m_pDialog->getInstall();
//			bStartDaemons 		= m_pDialog->getStart();

			if
			(
				(bPortalPortChange 		= checkAndChangePort(sPortalPort.ToInt32(),			bPortalPortChange))		||
				(bPortalAdminPortChange = checkAndChangePort(sPortalAdminPort.ToInt32(),	bPortalAdminPortChange))||
				(bSetupServerPortChange = checkAndChangePort(sSetupServerPort.ToInt32(), 	bSetupServerPortChange))||
				(bShutdownPortChange 	= checkAndChangePort(sShutdownPort.ToInt32(), 		bShutdownPortChange))	||
				(bLBMPortChange 		= checkAndChangePort(sLBMPort.ToInt32(), 			bLBMPortChange))		||
				(bLBShttpPortChange 	= checkAndChangePort(sLBShttpPort.ToInt32(), 		bLBShttpPortChange))	||
				(bLBShttpsPortChange 	= checkAndChangePort(sLBShttpsPort.ToInt32(), 		bLBShttpsPortChange))	||
				(bLBSShutdownPortChange	= checkAndChangePort(sLBSShutdownPort.ToInt32(), 	bLBSShutdownPortChange))||
				(bLBSunoPortChange 		= checkAndChangePort(sLBSunoPort.ToInt32(), 		bLBSunoPortChange))		||
				(bLBSunoCompPortChange 	= checkAndChangePort(sLBSunoCompPort.ToInt32(), 	bLBSunoCompPortChange))	||
				(bLBSunoTLSPortChange 	= checkAndChangePort(sLBSunoTLSPort.ToInt32(), 		bLBSunoTLSPortChange))	||
				(bLBSunoCompTLSPortChange= checkAndChangePort(sLBSunoCompTLSPort.ToInt32(), bLBSunoCompTLSPortChange))||
				(bSPdaemonPortChange 	= checkAndChangePort(sSPdaemonPort.ToInt32(), 		bSPdaemonPortChange))	||
				(bConfigServerPortChange= checkAndChangePort(sConfigServerPort.ToInt32(),	bConfigServerPortChange))
			)
			{
				bChangePort = TRUE;
			}
			else
			{
				bChangePort = FALSE;
			}
		}
		closeDialog( );
	}
		
	
	if (m_pInfo!=NULL)
	{
		m_pInfo->changeInfo(InfoTextDescriptor_WRITEPROPS);
	}

	writeProperties(sPortalPort,
					sPortalAdminPort,
					sSetupServerPort,
					sShutdownPort,
					sLBMPort,
					sLBShttpPort,
					sLBShttpsPort,
					sLBSShutdownPort,
					sLBSunoPort,
					sLBSunoCompPort,
					sLBSunoTLSPort,
					sLBSunoCompTLSPort,
					sSPdaemonPort,
					sConfigServerPort);

	if ( openDaemonRc() )
	{
		if( bInstallDaemons )
		{
			if (m_pInfo!=NULL)
			{
				m_pInfo->changeInfo(InfoTextDescriptor_INSTALLDAEMONS);
			}
			executeSectionOfScript( "install" );
		}
	
		if( bStartDaemons )
		{
			if (m_pInfo!=NULL)
			{
				m_pInfo->changeInfo(InfoTextDescriptor_STARTDAEMONS);
			}
			executeSectionOfScript( "start" );
		}
	}

	if (m_pInfo!=NULL)
	{
		delete m_pInfo;
		m_pInfo = NULL;
	}
	
	return TRUE;
}

//------------------------------------------------------------------------

void PlugInServerSetUp::checkPackages()
{
	SiDirEntry aConfigDir(m_sInstallDir);
	aConfigDir += SiDirEntry("configserver");
	m_bConfigServerInstalled = aConfigDir.Exists();

	SiDirEntry aLoadDir(m_sInstallDir);
	aLoadDir += SiDirEntry("webserver");
	aLoadDir += SiDirEntry("loadbalancer");
	m_bLBShttpInstalled = aLoadDir.Exists();

	SiDirEntry aShutdownDir(m_sInstallDir);
	aShutdownDir += SiDirEntry("webserver");
	aShutdownDir += SiDirEntry("classes");
	aShutdownDir += SiDirEntry("webserver.jar");
	m_bShutdownInstalled = aShutdownDir.Exists();
	
	SiDirEntry aSetupServerDir(m_sInstallDir);
	aSetupServerDir += SiDirEntry("SetupServer");
	m_bSetupServerInstalled = aSetupServerDir.Exists();

	m_bPortalInstalled		= checkWebappsDir(String::CreateFromAscii("portal"));
	m_bPortalAdminInstalled = checkWebappsDir(String::CreateFromAscii("PortalAdministration"));
	m_bPortalSetupInstalled = checkWebappsDir(String::CreateFromAscii("setup"));

	m_bLBMInstalled			= checkEtcInit(String::CreateFromAscii("lbmd"));
	m_bLBSunoInstalled		= checkEtcInit(String::CreateFromAscii("lbsd"));
	m_bSPdaemonInstalled	= checkEtcInit(String::CreateFromAscii("sportald"));
}

BOOL PlugInServerSetUp::checkWebappsDir(String sName)
{
	SiDirEntry aDir(m_sInstallDir);
	aDir += SiDirEntry("webserver");
	aDir += SiDirEntry("webapps");
	aDir += SiDirEntry(sName);
	
	return aDir.Exists();
}

BOOL PlugInServerSetUp::checkEtcInit(String sName)
{
	BOOL	bFound		= FALSE;
	String	sOrigName	= sName;

	SiDirEntry aDir(m_sInstallDir);
	aDir += SiDirEntry("etc");
	
	if( m_bPlatformIsUnix )
	{
		sName.Append(String::CreateFromAscii("rc"));
	}
	else
	{
		sName.Append(String::CreateFromAscii(".ini"));
	}
	
	aDir += SiDirEntry(sName);

	bFound = aDir.Exists();
	
	if (!bFound)
	{
		sOrigName.Append(String::CreateFromAscii(".properties"));

		SiDirEntry aDir(m_sInstallDir);
		aDir += SiDirEntry("etc");
		aDir += SiDirEntry(sOrigName);

		bFound = aDir.Exists();
	}

	return bFound;
}

//------------------------------------------------------------------------
// GUI

BOOL PlugInServerSetUp::readResourceFile()
{
	if( m_pRscMgr == 0 )
	{
		createRessourceManager(Application::GetAppInternational().GetLanguage());

		if (!m_pRscMgr)
		{
			createRessourceManager(LANGUAGE_ENGLISH);
		}
	}
	return m_pRscMgr != 0;
}

BOOL PlugInServerSetUp::haveRessourceManager()
{
	return m_pRscMgr != 0;
}	

void PlugInServerSetUp::createRessourceManager( LanguageType nLangType )
{
	String	sResFileName(String::CreateFromAscii("asv"));
	sResFileName.Append(String::CreateFromAscii(MAKE_NUMSTR( SUPD )));

	SiDirEntry aResource( "." );
	aResource += SiDirEntry( sResFileName );
	m_pRscMgr = ResMgr::CreateResMgr( aResource.GetFull().GetBuffer(), nLangType );

	if ( !m_pRscMgr )
	{
		aResource =  String::CreateFromAscii( "." );
		aResource += String::CreateFromAscii( "resource" );
		aResource += sResFileName;
		m_pRscMgr = ResMgr::CreateResMgr( aResource.GetFull().GetBuffer(), nLangType );
	}
	
	if ( !m_pRscMgr )
	{
		aResource =  m_sSetupDir;
		aResource += sResFileName;
		m_pRscMgr = ResMgr::CreateResMgr( aResource.GetFull().GetBuffer(), nLangType );
	}

	if ( !m_pRscMgr )
	{
		aResource =  m_sSetupDir;
		aResource += String::CreateFromAscii( "resource" );
		aResource += sResFileName;
		m_pRscMgr =  ResMgr::CreateResMgr( aResource.GetFull().GetBuffer(), nLangType );
	}
}

void PlugInServerSetUp::openDialog( String sProductName )
{
	if( !batchModeEnabled( ) )
	{
		if( m_pRscMgr != 0  && !batchModeEnabled( ) )
		{
			m_pDialog = new ServerSetupDialog( 0, m_pRscMgr );

			m_pDialog->SetText( sProductName );

			m_pDialog->setPortal		( String::CreateFromAscii("8081") );
			m_pDialog->setPortalAdmin	( String::CreateFromAscii("8082") );
			m_pDialog->setSetupServer	( String::CreateFromAscii("8209") );
			m_pDialog->setShutdown		( String::CreateFromAscii("8005") );
			m_pDialog->setLBM			( String::CreateFromAscii("8200") );
			m_pDialog->setLBSShutdown	( String::CreateFromAscii("8006") );
			m_pDialog->setLBSuno		( String::CreateFromAscii("8201") );
			m_pDialog->setLBSunoComp	( String::CreateFromAscii("8206") );
			m_pDialog->setLBSunoTLS		( String::CreateFromAscii("8207") );
			m_pDialog->setLBSunoCompTLS	( String::CreateFromAscii("8208") );
			m_pDialog->setSPdaemon		( String::CreateFromAscii("8202") );
			m_pDialog->setConfigServer	( String::CreateFromAscii("8205") );
//			m_pDialog->setInstall		( m_bIsAdmin );
//			m_pDialog->setStart			( m_bIsAdmin );

			if (m_bIsAdmin)
			{
				m_pDialog->setLBShttps		( String::CreateFromAscii("443") );
				m_pDialog->setLBShttp		( String::CreateFromAscii("80") );
			}
			else
			{
				m_pDialog->setLBShttps		( String::CreateFromAscii("8443") );
				m_pDialog->setLBShttp		( String::CreateFromAscii("8080") );
			}

		}
	}
}

short PlugInServerSetUp::executeDialog( BOOL bHaveScript )
{
	short	nResult = 0;

	if( !batchModeEnabled( ) )
	{
		if( m_pRscMgr != 0 && m_pDialog != 0 )
		{
			m_pDialog->Enable();
			m_pDialog->ToTop(); // for Windows 2000
			m_pDialog->GrabFocusToFirstControl();
			m_pDialog->grayPortal		( !m_bPortalInstalled );
			m_pDialog->grayPortalAdmin	( !m_bPortalAdminInstalled );
			m_pDialog->graySetupServer	( !m_bSetupServerInstalled );
			m_pDialog->grayShutdown		( !m_bShutdownInstalled );
			m_pDialog->grayLBM			( !m_bLBMInstalled );
			m_pDialog->grayLBShttp		( !m_bLBShttpInstalled );
			m_pDialog->grayLBShttps		( !m_bLBShttpInstalled );
			m_pDialog->grayLBSShutdown	( !m_bLBShttpInstalled );
			m_pDialog->grayLBSuno		( !m_bLBSunoInstalled );
			m_pDialog->grayLBSunoComp	( !m_bLBSunoInstalled );
			m_pDialog->grayLBSunoTLS	( !m_bLBSunoInstalled );
			m_pDialog->grayLBSunoCompTLS( !m_bLBSunoInstalled );
			m_pDialog->graySPdaemon		( !m_bSPdaemonInstalled );
			m_pDialog->grayConfigServer	( !m_bConfigServerInstalled );
//			m_pDialog->grayInstall		( !bHaveScript || !m_bIsAdmin );
//			m_pDialog->grayStart		( !bHaveScript );

			nResult = m_pDialog->execute( );
		}
	}

	return nResult;
}

void PlugInServerSetUp::closeDialog( )
{
	if( m_pDialog != NULL )
	{
		delete m_pDialog;

		m_pDialog = NULL;
	}
}

//------------------------------------------------------------------------
// scripts, ini-files

BOOL PlugInServerSetUp::openDaemonRc()
{
	if( !m_bIsDaemonRcOpen )
	{
		SiDirEntry	aDaemonRc( m_sInstallDir );

		aDaemonRc += SiDirEntry("etc");

		if( m_bPlatformIsUnix )
		{
			aDaemonRc += SiDirEntry( "daemonsrc" );
		}
		else
		{
			aDaemonRc += SiDirEntry( "daemons.ini" );
		}

		if( aDaemonRc.Exists( ) )
		{
			aDaemonRc.ToAbs( );

			String			sRcDir = aDaemonRc.GetFullUni( FSYS_STYLE_HOST, FALSE );
			rtl::OUString	sRcDirUNC;
	
			osl::FileBase::normalizePath(sRcDir, sRcDirUNC);

			if (m_script.open( sRcDirUNC ))
			{
				m_bIsDaemonRcOpen = TRUE;
			}
			else
			{
				m_bIsDaemonRcOpen = FALSE;
			};
		}
	}
	return m_bIsDaemonRcOpen;
}

void PlugInServerSetUp::writeProperties(const String	sPortalPort,
										const String	sPortalAdminPort,
										const String	sSetupServerPort,
										const String	sShutdownPort,
										const String	sLBMPort,
										const String	sLBShttpPort,
										const String	sLBShttpsPort,
										const String	sLBSShutdownPort,
										const String	sLBSunoPort,
										const String	sLBSunoCompPort,
										const String	sLBSunoTLSPort,
										const String	sLBSunoCompTLSPort,
										const String	sSPdaemonPort,
										const String	sConfigServerPort)
{
	String		sContent;
	SiDirEntry	aDir;

	// --- write webserver.properties

	if (m_bPortalInstalled || m_bLBShttpInstalled || m_bPortalSetupInstalled)
	{
		sContent += String(String::CreateFromAscii("http.port.portal="));
		sContent += sPortalPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bPortalAdminInstalled)
	{
		sContent += String(String::CreateFromAscii("http.port.admin="));
		sContent += sPortalAdminPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bSetupServerInstalled)
	{
		sContent += String(String::CreateFromAscii("uno.port.setup="));
		sContent += sSetupServerPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bShutdownInstalled)
	{
		sContent += String(String::CreateFromAscii("tcp.port.shutdown="));
		sContent += sShutdownPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bLBMInstalled)
	{
		sContent += String(String::CreateFromAscii("socket.port.LBM="));
		sContent += sLBMPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bLBShttpInstalled)
	{
		sContent += String(String::CreateFromAscii("http.port.LBShttp="));
		sContent += sLBShttpPort;
		sContent += String(String::CreateFromAscii("\n"));

		sContent += String(String::CreateFromAscii("https.port.LBShttp="));
		sContent += sLBShttpsPort;
		sContent += String(String::CreateFromAscii("\n"));

		sContent += String(String::CreateFromAscii("tcp.LBShttp.shutdown="));
		sContent += sLBSShutdownPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bLBSunoInstalled)
	{
		sContent += String(String::CreateFromAscii("portal.port.LBSuno="));
		sContent += sLBSunoPort;
		sContent += String(String::CreateFromAscii("\n"));
 
 		sContent += String(String::CreateFromAscii("portal.port.LBSunoCompressed="));
		sContent += sLBSunoCompPort;
		sContent += String(String::CreateFromAscii("\n"));
 
 		sContent += String(String::CreateFromAscii("portal.port.LBSunoTLS="));
		sContent += sLBSunoTLSPort;
		sContent += String(String::CreateFromAscii("\n"));
 
 		sContent += String(String::CreateFromAscii("portal.port.LBSunoCompressedTLS="));
		sContent += sLBSunoCompTLSPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bSPdaemonInstalled)
	{
		sContent += String(String::CreateFromAscii("portal.port.sportald="));
		sContent += sSPdaemonPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	if (m_bConfigServerInstalled)
	{
		sContent += String(String::CreateFromAscii("portal.port.ConfigServer="));
		sContent += sConfigServerPort;
		sContent += String(String::CreateFromAscii("\n"));
	}

	aDir =  m_sInstallDir;
	aDir += SiDirEntry("etc");

	writeConfigFile(aDir, SiDirEntry("webserver.properties"), sContent, FALSE);

	// --- write config server.properties
	
	sContent =  String(String::CreateFromAscii("port="));
	sContent += sConfigServerPort;
	sContent += String(String::CreateFromAscii("\n"));

	aDir =  m_sInstallDir;
	aDir += SiDirEntry("configserver");

	writeConfigFile(aDir, SiDirEntry("configserver.properties"), sContent, FALSE);

	// --- write setup server.properties
	
	sContent =  String(String::CreateFromAscii("[install]"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("cmd=%s/program/setupserver"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("arg=install"));
	sContent += String(String::CreateFromAscii("\n"));

	sContent += String(String::CreateFromAscii("[uninstall]"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("cmd=%s/program/setupserver"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("arg=uninstall"));
	sContent += String(String::CreateFromAscii("\n"));

	sContent += String(String::CreateFromAscii("[start]"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("cmd=%s/program/setupserver"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("arg=start"));
	sContent += String(String::CreateFromAscii("\n"));

	sContent += String(String::CreateFromAscii("[stop]"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("cmd=%s/program/setupserver"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("arg=stop"));
	sContent += String(String::CreateFromAscii("\n"));

	sContent += String(String::CreateFromAscii("[Access]"));
	sContent += String(String::CreateFromAscii("\n"));
	sContent += String(String::CreateFromAscii("port="));
	sContent += sSetupServerPort;
	sContent += String(String::CreateFromAscii("\n"));

	aDir =  m_sInstallDir;
	aDir += SiDirEntry("etc");

	if( m_bPlatformIsUnix )
	{
		writeConfigFile(aDir, SiDirEntry("setupserverrc"), sContent, FALSE);
	}
	else
	{
		writeConfigFile(aDir, SiDirEntry("setupserver.ini"), sContent, FALSE);
	}

	// --- ADD JAVA lbmd properties

	sContent =  String::CreateFromAscii("Connection=socket,host=0,port=");
	sContent += sLBMPort;

	aDir =  m_sInstallDir;
	aDir += SiDirEntry("etc");

	writeConfigFile(aDir, SiDirEntry("lbmd.properties"), sContent, TRUE);

	// --- write rc/ini files

	changeValueInEtcScript(	String::CreateFromAscii("lbmd"), 
							String::CreateFromAscii("Master"),
							String::CreateFromAscii("Connection"),
							String::CreateFromAscii("port="),
							sLBMPort);
	changeValueInEtcScript(	String::CreateFromAscii("lbsd"), 
							String::CreateFromAscii("Master"),
							String::CreateFromAscii("Connection"),
							String::CreateFromAscii("port="),
							sLBMPort);
	changeValueInEtcScript(	String::CreateFromAscii("lbsd"), 
							String::CreateFromAscii("Accept"),
							String::CreateFromAscii("Port"),
							String::CreateFromAscii(""),
							sLBSunoPort);
	changeValueInEtcScript(	String::CreateFromAscii("lbsd"), 
							String::CreateFromAscii("Accept"),
							String::CreateFromAscii("CompressedPort"),
							String::CreateFromAscii(""),
							sLBSunoCompPort);
	changeValueInEtcScript(	String::CreateFromAscii("lbsd"), 
							String::CreateFromAscii("Accept"),
							String::CreateFromAscii("SecurePort"),
							String::CreateFromAscii(""),
							sLBSunoTLSPort);
	changeValueInEtcScript(	String::CreateFromAscii("lbsd"), 
							String::CreateFromAscii("Accept"),
							String::CreateFromAscii("CompressedSecurePort"),
							String::CreateFromAscii(""),
							sLBSunoCompTLSPort);
	changeValueInEtcScript(	String::CreateFromAscii("sportald"), 
							String::CreateFromAscii("Master"),
							String::CreateFromAscii("Connection"),
							String::CreateFromAscii("port="),
							sLBMPort);
	changeValueInEtcScript(	String::CreateFromAscii("sportald"), 
							String::CreateFromAscii("Accept"),
							String::CreateFromAscii("Port"),
							String::CreateFromAscii(""),
							sSPdaemonPort);
	changeValueInEtcScript(	String::CreateFromAscii("sportald"), 
							String::CreateFromAscii("UserServices"),
							String::CreateFromAscii("configuration"),
							String::CreateFromAscii("port="),
							sConfigServerPort);

/*	// not needed now, host is 0 or localhost, not a specialised dotted ip anymore
	// get dotted ip from localhost
	rtl::OUString sLocalHost;

	OSocketAddr::getLocalHostname(sLocalHost);

	OInetSocketAddr aAddress(OSocketAddr::resolveHostname(sLocalHost));

    aAddress.getDottedAddr(sLocalHost);

	changeValueInEtcScript(	String::CreateFromAscii("lbmd"),
							String::CreateFromAscii("Master"),
							String::CreateFromAscii("Connection"),
							String::CreateFromAscii("host="),
							sLocalHost);
	changeValueInEtcScript(	String::CreateFromAscii("lbsd"),
							String::CreateFromAscii("Master"),
							String::CreateFromAscii("Connection"),
							String::CreateFromAscii("host="),
							sLocalHost);
	changeValueInEtcScript(	String::CreateFromAscii("sportald"),
							String::CreateFromAscii("Master"),
							String::CreateFromAscii("Connection"),
							String::CreateFromAscii("host="),
							sLocalHost);
*/
}

void PlugInServerSetUp::writeConfigFile(	const SiDirEntry	aDir,
											const SiDirEntry	aFile,
											const String 		sContent,
											const BOOL			bAppend)
{
	sal_uInt32 nFlags = 0;

	SiDirEntry aFullDir(aDir);

	if (bAppend)
	{
		nFlags = OpenFlag_Write;
	}
	else
	{
		nFlags = (OpenFlag_Create|OpenFlag_Write);
	}

	aFullDir.ToAbs();

	if (aFullDir.Exists())
	{
		aFullDir += aFile;
		
		String			sFile = aFullDir.GetFullUni( FSYS_STYLE_HOST, FALSE );
		rtl::OUString	sFileUNC;

		osl::FileBase::normalizePath(sFile, sFileUNC);

		osl::File	aFile(sFileUNC);
		sal_uInt64	iBytesWritten = 0;

		aFile.open(nFlags);

		if (bAppend)
		{
			aFile.setPos( osl_Pos_End, 0);
		}

		aFile.write(ByteString(sContent, osl_getThreadTextEncoding()).GetBuffer(), sContent.Len(), iBytesWritten);
		aFile.close();
	}
}	

void PlugInServerSetUp::changeValueInEtcScript(	const String sNameOfScript,
												const String sSection,
												const String sEntry,
												const String sSubEntry,
												const String sReplace )
{
	SiDirEntry aInitDir(m_sInstallDir);
	aInitDir += SiDirEntry("etc");

	String sNameOfScriptOS;

	if( m_bPlatformIsUnix )
	{
		sNameOfScriptOS = String(sNameOfScript).Append(String::CreateFromAscii("rc"));
	}
	else
	{
		sNameOfScriptOS = String(sNameOfScript).Append(String::CreateFromAscii(".ini"));
	}
	
	aInitDir += SiDirEntry( sNameOfScriptOS );
	
	if (aInitDir.Exists() && m_bSPdaemonInstalled)
	{
		//	Connection=socket,host=localhost,port=8000
		// -> (e.g.)
		//  Connection=socket,host=110.13.07.05,port=8200

		OProfile		aScript;
		rtl::OUString	sInitDirUNC;
		String			sInitDir = aInitDir.GetFullUni( FSYS_STYLE_HOST, FALSE );

		osl::FileBase::normalizePath(sInitDir, sInitDirUNC);

		if (aScript.open(sInitDirUNC))
		{
			const size_t	SIZE_OF_BUFFER = 8192;
			char* sBuffer = new char[SIZE_OF_BUFFER];
		
			if( aScript.getSectionEntries(	ByteString(sSection, osl_getThreadTextEncoding()).GetBuffer(), 
											sBuffer,
											SIZE_OF_BUFFER ) > 0 )
			{
				char*	sKey = sBuffer;
				while (*sKey)
				{
					if( strncmp( sKey, ByteString(sEntry, osl_getThreadTextEncoding()).GetBuffer(), sEntry.Len() ) == 0 )
					{
						char	sUnoUrl[255+1];
			
						aScript.readString( ByteString(sSection, osl_getThreadTextEncoding()).GetBuffer(),
											ByteString(sEntry, osl_getThreadTextEncoding()).GetBuffer(),
											sUnoUrl,
											127,
											"" );
						
						String sMutator( sUnoUrl , osl_getThreadTextEncoding() );
						String sMutated;
		
						if (sSubEntry.Len()==0)
						{
							sMutated = sReplace;
						}
						else
						{
							xub_StrLen nSubPos = sMutator.Search(sSubEntry);
							
							if (nSubPos!=STRING_NOTFOUND)
							{
								nSubPos += sSubEntry.Len();
								String		sStart	= sMutator.Copy(0, nSubPos );
								xub_StrLen	nEndPos = sMutator.SearchAscii(",", nSubPos);
		
								sMutated	= sStart;
								sMutated.Append(sReplace);
								
								if (nEndPos!=STRING_NOTFOUND)
								{
									String sEnd = sMutator.Copy(nEndPos);
									
									sMutated.Append(sEnd);
								}
							}
						}
		
						aScript.writeString(ByteString(sSection, osl_getThreadTextEncoding()).GetBuffer(),
											ByteString(sEntry, osl_getThreadTextEncoding()).GetBuffer(),
											ByteString(sMutated, osl_getThreadTextEncoding()).GetBuffer() );
		
						aScript.flush();
					}
					sKey+=strlen(sKey)+1;
				}
			}
			aScript.close();
		}
	}
}

void PlugInServerSetUp::executeSectionOfScript
(
	const char*	sNameOfSection
)
{
	const size_t	SIZE_OF_BUFFER = 8192;
	char*			sBuffer = new char[SIZE_OF_BUFFER];
	char*			sDelimiters = ",; ";

	if( m_script.getSectionEntries( sNameOfSection, sBuffer, SIZE_OF_BUFFER ) > 0 )
	{
		char*	sKey = sBuffer;
		char	sString[511+1];

		if( strncmp( sKey, "order", 5 ) == 0 )
		{
			m_script.readString( sNameOfSection, sKey, sString, 511, "" );

			char * sTokenRunner = strtok(sString, sDelimiters);

			while ( sTokenRunner!=NULL)
			{
				SiDirEntry	aConfDir(m_sInstallDir);
				aConfDir += SiDirEntry("etc");

				String sIniFile(String::CreateFromAscii(sTokenRunner));

				if( m_bPlatformIsUnix )
				{
					sIniFile += String(String::CreateFromAscii("rc"));
				}
				else
				{
					sIniFile += String(String::CreateFromAscii(".ini"));
				}

				aConfDir += SiDirEntry(sIniFile);

				aConfDir.ToAbs();

				if (aConfDir.Exists())
				{
					executeCommand(aConfDir.GetFullUni(), sNameOfSection);
				}
				sTokenRunner = strtok(NULL, sDelimiters);
			}
		}
		delete[] sBuffer;
	}
}

void PlugInServerSetUp::executeCommand
(
	const String sFilePath,
	const char*	sNameOfSection
)
{
	OProfile aDaemonScript;

	rtl::OUString	sRcDirUNC;

	osl::FileBase::normalizePath(sFilePath, sRcDirUNC);

	aDaemonScript.open(sRcDirUNC);

	const size_t	SIZE_OF_BUFFER = 8192;
	char* sBuffer = new char[SIZE_OF_BUFFER];

	if( aDaemonScript.getSectionEntries( sNameOfSection, sBuffer, SIZE_OF_BUFFER ) > 0 )
	{
		char*	sKey = sBuffer;
		char	sString[511+1];

		if( strncmp( sKey, "cmd", 3 ) == 0 )
		{
			char	sNameOfExec[255+1];

			aDaemonScript.readString( sNameOfSection, sKey, sString, 127, "" );
			sprintf( sNameOfExec, sString, ByteString(m_sInstallDir, osl_getThreadTextEncoding()).GetBuffer() );

			sKey += strlen( sKey ) + 1;

			if( *sKey && strncmp( sKey, "arg", 3 ) == 0 )
			{
				short			nNoOfArgs = 0;
				char			sBuffer1[4][127+1];
				char			sBuffer2[4][255+1];
				const char*		sArgV[4] = { sBuffer2[0], sBuffer2[1], sBuffer2[2], sBuffer2[3] };

				aDaemonScript.readString( sNameOfSection, sKey, sString, 511, "" );

				nNoOfArgs = sscanf( sString, "%127s%127s%127s%127s", sBuffer1[0], sBuffer1[1], sBuffer1[2], sBuffer1[3] );

				for( short i = 0; i < nNoOfArgs; i++ )
				{
					sprintf( sBuffer2[i], sBuffer1[i], ByteString(m_sInstallDir, osl_getThreadTextEncoding()).GetBuffer() );
				}

				executeProcess( sNameOfExec, nNoOfArgs, sArgV );
			}
		}
	}
	delete[] sBuffer;
}

//------------------------------------------------------------------------
// div. helpers

BOOL PlugInServerSetUp::batchModeEnabled( )
{
	return m_bBatchMode;
}

BOOL PlugInServerSetUp::checkAndChangePort
(
	sal_Int32	nPort,
	BOOL		bChange
)
{
	if ( bChange )
	{
		BOOL			bPortInUse		= FALSE;
		rtl::OUString	sNameOfLocalHost;
	
		if (!(m_pInfo->IsVisible()))
		{
			m_pInfo->execute();
		}
	
		OSocketAddr::getLocalHostname( sNameOfLocalHost );
	
		OInetSocketAddr  localAddress( sNameOfLocalHost, nPort );
	
		if( ( oslSocketAddr ) localAddress != 0 )
		{
			OConnectorSocket			connector;
			ISocketTypes::TResult		eResultOfConnect = ISocketTypes::TResult_Error;
	
			// connect to acceptor socket and set return value
			eResultOfConnect = connector.connect( localAddress );
	
			bPortInUse = ( eResultOfConnect == ISocketTypes::TResult_Ok );
		}
	
		if( bPortInUse )
		{
			// does user want to change used port?
			m_pInfo->Hide();
	
			String sText(ResId( STR_HTTPPORTINUSE, m_pRscMgr ));
			
			sText.SearchAndReplace(String::CreateFromAscii("%d"), String::CreateFromInt32(nPort));
			
			WarningBox	wb( 0, WB_YES_NO, sText );
	
			bChange = ( wb.Execute( ) == BUTTONID_YES );
		}
		else
		{
			bChange = FALSE;
		}
	}
		
	return bChange;
}

BOOL PlugInServerSetUp::checkAdmin(BOOL bIsFirst)
{
	BOOL bIsAdmin = FALSE;
	
	if( !m_security.isAdministrator() )
	{
		if( userIsAbleToBecomeAdministrator( ) )
		{
			bIsAdmin = becomeAdministrator( );
		}
		else
		{
			bIsAdmin = FALSE;
		}

		if( !bIsAdmin && !batchModeEnabled( ) && bIsFirst )
		{
			String sErrorMessage = String( ResId( STR_LOGINASADMIN, m_pRscMgr ) );
			InfoBox( 0, sErrorMessage ).Execute( );
		}
	}
	else
	{
		bIsAdmin = TRUE;
	}
	
	return bIsAdmin;
}

BOOL PlugInServerSetUp::userIsAbleToBecomeAdministrator( )
{
	return m_bPlatformIsWin32 && !batchModeEnabled( );
}

BOOL PlugInServerSetUp::becomeAdministrator( )
{
	BOOL bLogonSuccessful = FALSE;

	if(m_security.isAdministrator())
	{
		return TRUE;
	}

	if(userIsAbleToBecomeAdministrator())
	{
		ServerSetupLoginDialog	dialog( 0, m_pRscMgr );

		while (!bLogonSuccessful)
		{
			short	nResultOfExecute = dialog.Execute( );

			if( nResultOfExecute == 1 )
			{
				BOOL bLogonSuccessful = m_security.logonUser( dialog.getName( ), dialog.getPassword( ) );

				if( !bLogonSuccessful )
				{
					WarningBox	wb( 0, WB_RETRY_CANCEL, String( ResId( STR_LOGINERROR, m_pRscMgr ) ) );

					if( wb.Execute( ) == BUTTONID_CANCEL )
					{
						return FALSE;
					}
					dialog.clearPassword( );
				}
			}
		}
		return TRUE;
	}
	return FALSE;
}

//------------------------------------------------------------------------
// process handling

class ProcessExecutor : public OThread
{

	public:
		ProcessExecutor(const char*	pName,
						int			nArgC,
						const char* sArgV[] );
		~ProcessExecutor();
		BOOL isRunning() const { return !m_bDone; }

	private:
		rtl::OUString	m_sExecutable;
		OArgumentList*	m_sArgumentList;
		BOOL			m_bDone;
		virtual void SAL_CALL	run();
};


ProcessExecutor::ProcessExecutor(const char* pName, int nArgC, const char* sArgV[] )
{
	m_sExecutable = rtl::OUString::createFromAscii(pName);
	m_bDone = FALSE;

	rtl::OUString *aArguments;

	aArguments = new rtl::OUString[nArgC];

	for( int nArgumentRunner = nArgC; nArgumentRunner--; )
	{
		aArguments[nArgumentRunner] = rtl::OUString::createFromAscii( sArgV[nArgumentRunner] );
	}

	m_sArgumentList = new OArgumentList(aArguments, nArgC);
}

ProcessExecutor::~ProcessExecutor()
{
	delete	m_sArgumentList;
}

void ProcessExecutor::run()
{
  	OProcess process(m_sExecutable);

	process.execute((OProcess::TProcessOption)(OProcess::TOption_Hidden | OProcess::TOption_Wait),
					*m_sArgumentList);

	m_bDone = TRUE;
}

void PlugInServerSetUp::executeProcess
(
	const char*		sNameOfExecutable,
	const char*		sArguments
)
{
	BOOL			bWaitForCompletion = TRUE;
	char			sBuffer[4][127+1];
	const char*		sArgs[4] = { sBuffer[0], sBuffer[1], sBuffer[2], sBuffer[3] };
	short			nNoOfArgs = sscanf( sArguments, "%127s%127s%127s%127s", sBuffer[0], sBuffer[1], sBuffer[2], sBuffer[3] );
	executeProcess( sNameOfExecutable, nNoOfArgs, sArgs );
};

void PlugInServerSetUp::executeProcess
(
	const char*		sNameOfExecutable,
	int				nArgC,
	const char*		sArgV[]
)
{
	ProcessExecutor* pProcess = new ProcessExecutor(
		sNameOfExecutable, nArgC, sArgV );

	pProcess->create();

	// Unter OS/2 darf auf einen Prozess, der per "start xxx" aus einer
	// cmd.exe als PM App gestartet wird, nicht aus dem Solarthread
	// gewartet werden, da OS/2 sonst deadlocked.
#ifdef OS2
	while( pProcess->isRunning() ) Application::Yield();
#else
	pProcess->join();
#endif
	delete pProcess;

};

