/*************************************************************************
 *
 *  $RCSfile: appserv.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:04 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _APPSERV_HXX_
#define _APPSERV_HXX_

#include <vcl/resmgr.hxx>
#include <vos/security.hxx>
#include "aservdlg.hxx"

#ifdef _USE_NAMESPACE
using namespace vos;
#endif

class PlugInServerSetUp
{
  public:
						PlugInServerSetUp		(	const String	aSetupDir,
													const String	aInstallDir,
													const String	sNameOfBatchFile,
													const BOOL		bBatchMode,
													const BOOL		bInstallation );
	virtual			   ~PlugInServerSetUp		();

	virtual BOOL		mainPreInstallation		();
	virtual BOOL		mainPostInstallation	(	const String	sProductName );

  protected:
	virtual void 		checkPackages			();

	virtual BOOL		readResourceFile		();
	virtual BOOL 		haveRessourceManager	();
	virtual void 		createRessourceManager	( LanguageType	nLangType );
	virtual void		openDialog				( const String	sProductName );
	virtual short		executeDialog			( const BOOL	bHaveScript );
	virtual void		closeDialog				();

	virtual BOOL		openDaemonRc			();

	virtual void 		writeConfigFile			(	const SiDirEntry	aDir,
													const SiDirEntry	aFile,
													const String 		sContent,
													const BOOL			bAppend);
	virtual void 		writeProperties			(	const String	sPortalPort,
													const String	sPortalAdminPort,
													const String	sSetupServerPort,
													const String	sShutdownPort,
													const String	sLBMPort,
													const String	sLBShttpPort,
													const String	sLBShttpsPort,
													const String	sLBSShutdownPort,
													const String	sLBSunoPort,
													const String	sLBSunoCompPort,
													const String	sLBSunoTLSPort,
													const String	sLBSunoCompTLSPort,
													const String	sSPdaemonPort,
													const String	sConfigServerPort);
	virtual void 		changeValueInEtcScript	(	const String sNameOfScript,
													const String sSection,
													const String sEntry,
													const String sSubEntry,
													const String sReplace );
														
	virtual void		executeSectionOfScript	(	const char*		sNameOfSection 	);
	virtual void 		executeCommand			(	const String	sFilePath,
												  	const char*		sNameOfSection);

	virtual BOOL 		checkWebappsDir			(	String 		sName);
	virtual BOOL 		checkEtcInit			(	String 		sName);

	virtual BOOL		batchModeEnabled				();
	virtual BOOL		checkAndChangePort				(	sal_Int32		nPort,
															BOOL			bChange );
	virtual BOOL		checkAdmin						(	const BOOL 		bIsFirst);
	virtual BOOL		userIsAbleToBecomeAdministrator	();
	virtual BOOL		becomeAdministrator				();


	virtual void		executeProcess			( const char*	sNameOfExecutable,
												  const char*	sArguments );
	virtual void		executeProcess			( const char*	sNameOfExecutable,
												  int			nArgC,
												  const char*	sArgV[] );

  private:

	const String		m_sSetupDir;
	const String		m_sInstallDir;
	const String		m_sBatchFile;
	const BOOL			m_bBatchMode;
	const BOOL			m_bInstallation;

	BOOL				m_bPlatformIsUnix;
	BOOL				m_bPlatformIsWin32;

	BOOL				m_bPortalInstalled;
	BOOL				m_bPortalAdminInstalled;
	BOOL				m_bPortalSetupInstalled;
	BOOL				m_bSetupServerInstalled;
	BOOL				m_bShutdownInstalled;
	BOOL				m_bLBMInstalled;
	BOOL				m_bLBShttpInstalled;
	BOOL				m_bLBSunoInstalled;
	BOOL				m_bSPdaemonInstalled;
	BOOL				m_bConfigServerInstalled;

	ResMgr*				m_pRscMgr;

	OSecurity			m_security;

	BOOL				m_bIsAdmin;
	BOOL				m_bIsDaemonRcOpen;
	OProfile			m_script;

	ServerSetupDialog*		m_pDialog;
	ServerSetupInfoWindow*	m_pInfo;
};

#endif

/************************************************************************

	Source Code Control System - Header

	$Header: /cvs/oo/installation/setup2/source/custom/appserv/appserv.hxx,v 1.1.1.1 2000/09/18 16:51:04 hr Exp $

	Source Code Control System - Update

	$Log: appserv.hxx,v $
	Revision 1.1.1.1  2000/09/18 16:51:04  hr
	initial import
	
	Revision 1.21  2000/09/17 16:18:26  willem.vandorp
	OpenOffice header added.
	
	Revision 1.20  2000/09/08 10:01:45  willem.vandorp
	Header and footer replaced
	
	Revision 1.19  2000/09/01 14:59:27  tpf
	#78417# added http.port.portal to webserver.properties if setup installed
	
	Revision 1.18  2000/08/25 12:58:33  tpf
	#78145# added new ports
	
	Revision 1.17  2000/08/18 11:25:18  tpf
	#77808# new port, some bugfixes
	
	Revision 1.16  2000/08/15 14:37:30  tpf
	#77601# various fixes, start daemons correctly, determine Admin state etc
	
	Revision 1.15  2000/08/11 09:31:19  tpf
	#77293# some new ports etc
	
	Revision 1.14  2000/08/10 14:02:38  tpf
	#77464# #77413# bugs fixed, added some ports
	
	Revision 1.13  2000/08/04 11:33:35  tpf
	checkbox install daemons, new daemons install script syntax/handling

	Revision 1.12  2000/07/27 08:30:48  tpf
	write webserver.properties / added wait message window

	Revision 1.11  2000/07/12 12:11:58  tpf
	#76491# OSecurity::logonUser is handled correctly if logon fails

	Revision 1.10  2000/07/05 10:55:16  tpf
	removed code dealing with schedule server cause schedule server is dead now

	Revision 1.9  2000/06/15 08:21:46  tpf
	DirEntry-Umstellung

	Revision 1.8  2000/05/16 10:17:03  tpf
	Unicode

	Revision 1.7  2000/05/11 15:48:35  ok
	chg: unicode

	Revision 1.6  2000/01/07 17:57:28  tpf
	#71636# title now provied via setup method not via own resource

	Revision 1.5  1999/12/28 15:32:57  ts
	#70422# stomcatd statt shttpd

	Revision 1.4  1999/04/14 14:14:46  KA
	#64686# Beherrscht jetzt auch Batch-Modus


	  Rev 1.3   14 Apr 1999 16:14:46   KA
   #64686# Beherrscht jetzt auch Batch-Modus

	  Rev 1.2   13 Apr 1999 14:20:26   KA
   #64666# Jetzt wieder richtiges Verhalten mit Spaces im Inst-Pfad unter Unix
   #64482# Nun auch Abfrage auf root am Anfang der Installation

	  Rev 1.1   31 Mar 1999 16:26:12   KA
   #64136#

	  Rev 1.0   16 Mar 1999 17:08:10   KA
   Initial revision.

/************************************************************************/
