/*************************************************************************
 *
 *  $RCSfile: moffice.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _SV_CONFIG_HXX
#include <vcl/config.hxx>
#endif

#include "moffice.hxx"
#include "environ.hxx"

#ifndef _CNTVIEWS_HXX
#include "cntviews.hxx"
#endif

#ifndef _PATHS_HXX
#include "paths.hxx"
#endif

#ifdef UNX
#define OFFICE_INI		"sofficerc"
#define ADRBKMAP_INI	"addrconvrc"
#else
#define OFFICE_INI		"soffice.ini"
#define ADRBKMAP_INI	"addrconv.ini"
#endif

/////////////////////////////////////////////////////////////////////////////
//	StarOffice migration plugin base class
// initialise static instance

SetupMigrationPlugin* pPlugin = new OfficeMigrationDaisy;

OfficeMigrationDaisy::OfficeMigrationDaisy()
{
	SetupMigrationTask* pNew = 0;

	pNew = new MigrationTask_CopyFiles( this );
	AddTask( pNew );

	pNew = new MigrationTask_UserData( this );
	AddTask( pNew );

	pNew = new MigrationTask_AddressBook( this );
	AddTask( pNew );

	pNew = new MigrationTask_ChaosViews( this );
	AddTask( pNew );

	pNew = new MigrationTask_Paths( this );
	AddTask( pNew );
}

BOOL OfficeMigrationDaisy::SupportMigrationFor( const ByteString& rKey )
{
	if( rKey.CompareIgnoreCaseToAscii("StarOffice 5.0") == COMPARE_EQUAL ||
		rKey.CompareIgnoreCaseToAscii("StarOffice 5.1") == COMPARE_EQUAL )
		return TRUE;
	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
//	copy file task
MigrationTask_CopyFiles::MigrationTask_CopyFiles( SetupMigrationPlugin* pPlugin ) :
	SetupMigrationTask( pPlugin )
{
}

ULONG MigrationTask_CopyFiles::ExecuteAction( const SiDirEntry& rPath )
{
	ULONG nSize = 0;

	if( m_bDoRecursiveDir )
	{
		Dir aSubDir( rPath, FSYS_KIND_DIR | FSYS_KIND_DIR );
		for( USHORT i = 0; i < aSubDir.Count(); ++i )
		{
			ByteString aFull( ByteString( ((const SiDirEntry&)aSubDir[i]).GetName().GetBuffer(), osl_getThreadTextEncoding()) );
			if( aFull != "." && aFull != ".." )
				nSize += ExecuteAction( aSubDir[i] );
		}
	}

	Dir aDir( rPath, FSYS_KIND_FILE | FSYS_KIND_DIR );
	for( USHORT nDirIdx = 0; nDirIdx < aDir.Count(); ++nDirIdx )
	{
		ByteString aFile( ByteString( ((const SiDirEntry&)aDir[nDirIdx]).GetName().GetBuffer(), osl_getThreadTextEncoding()) );
		FileStat stat( aDir[nDirIdx] );

		if( !rPath.IsCaseSensitive() ) {
			m_aSrcMask.ToLowerAscii();
			aFile.ToLowerAscii();
		}

		SiDirEntry aAbsDestination( m_aDestinationPath );

		ByteString aRelPath( rPath.GetFull() );
		if( aRelPath.Len() > m_aSourcePath.GetFull().Len() )
		{
			aRelPath.Erase( 0,m_aSourcePath.GetFull().Len() + 1 );
			aAbsDestination += aRelPath;
		}

		if( m_bDoRecursiveDir && m_bDoEmptyDir && m_eTyp == EXECUTE )
			aAbsDestination.MakeDir();

		if( aFile.CompareIgnoreCaseToAscii(".") == COMPARE_EQUAL || aFile.CompareIgnoreCaseToAscii("..") == COMPARE_EQUAL ||
			stat.GetKind() == FSYS_KIND_DIR )
			continue;

		WildCard aMatch( UniString::CreateFromAscii(m_aSrcMask.GetBuffer()) );
		if( aMatch == UniString::CreateFromAscii(aFile.GetBuffer()) )
		{
			BOOL bDoAction = TRUE;
			SiDirEntry aAbsSource( rPath );
			aAbsSource += aFile;

			// neagation
			for( USHORT xx = 0; xx < m_aNegList.Count(); ++xx )
			{
				WildCard aNegationMatch( UniString::CreateFromAscii((*(m_aNegList.GetObject(xx))).GetBuffer()) );
				if( aNegationMatch == UniString::CreateFromAscii(aFile.GetBuffer()) )
				{
					bDoAction = FALSE;
					break;
				}
			}

			// timestamp
			for( USHORT xxx = 0; xxx < m_aDateTimeList.Count(); ++xxx )
			{
				DateTime aDt( *(m_aDateTimeList.GetObject(xxx)) );
				FileStat aFileStat( aAbsSource );
				DateTime aFileDt( aFileStat.DateModified(), aFileStat.TimeModified() );

				#ifdef UNX
				aFileDt.ConvertToUTC();
				#endif

				if( aDt.GetDate() == aFileDt.GetDate() )
				{
    				bDoAction = FALSE;
					break;
	  			}
			}

			if( bDoAction )
				switch( m_eTyp )
				{
					case CALCULATE_SIZE : {
						FileStat aFileStat( aAbsSource );
						nSize += aFileStat.GetSize();
						} break;
					case EXECUTE : {
						if( m_bDoRecursiveDir || m_bDestIsPath )
							aAbsDestination += aFile;

						if( FileStat::HasReadOnlyFlag() &&
							FileStat::GetReadOnlyFlag(aAbsDestination) )
							FileStat::SetReadOnlyFlag( aAbsDestination, FALSE );

						if( !aAbsDestination.GetPath().Exists() )
							aAbsDestination.GetPath().MakeDir();
						else
							aAbsDestination.Kill();

						FileCopier aCpy( aAbsSource, aAbsDestination );
						aCpy.Execute();
						} break;
				}
		}
	}

	return nSize;
}

ULONG MigrationTask_CopyFiles::Execute( MigrationExecType eTyp, SiEnvironment* pEnv )
{
	m_eTyp = eTyp;

	ULONG nSize = 0;
	Config aConfig( UniString::CreateFromAscii(m_pPlugin->GetCodeFileName().GetBuffer()) );
	aConfig.SetGroup( "CopyFileTask" );

	for( USHORT i = 0; i < aConfig.GetKeyCount(); ++i )
	{
		m_aSourcePath = pEnv->GetMigrationPath();
		m_aDestinationPath = pEnv->GetDestPath();

		ByteString aDestKey( aConfig.ReadKey(i) );
		aDestKey.EraseLeadingChars( '"' ); aDestKey.EraseTrailingChars( '"' );

		USHORT nTokenCount = aDestKey.GetTokenCount('|');

		ByteString aDestPath, aNegation, aDateTime, aRecursive;

		aDestPath	= aDestKey.GetToken( 0, '|' );
		aNegation	= aDestKey.GetToken( 1, '|' );
		aDateTime	= aDestKey.GetToken( 2, '|' );
		aRecursive	= aDestKey.GetToken( 3, '|' );

		aDestPath.EraseLeadingChars(); aDestPath.EraseTrailingChars();
		aNegation.EraseLeadingChars(); aNegation.EraseTrailingChars();
		aDateTime.EraseLeadingChars(); aDateTime.EraseTrailingChars();
		aRecursive.EraseLeadingChars(); aRecursive.EraseTrailingChars();

		if( aNegation.Len() )
		{
			USHORT nNegTokenCount = aNegation.GetTokenCount(' ');
			for( USHORT x = 0; x < nNegTokenCount; ++x )
			{
				ByteString* pStr = new ByteString( aNegation.GetToken(x, ' ') );
				pStr->EraseLeadingChars(); pStr->EraseTrailingChars();
				m_aNegList.Insert( pStr, LIST_APPEND );
			}
		}
		if( aDateTime.Len() )
		{
			USHORT nDateTimeCount = aDateTime.GetTokenCount(' ');
			for( USHORT x = 0; x < nDateTimeCount; ++x )
			{
				ByteString aTmp( aDateTime.GetToken(x) );
				aTmp.EraseLeadingChars(); aTmp.EraseTrailingChars();

				DateTime* pDateTime = new DateTime;

				pDateTime->SetDay( aTmp.Copy(0,2).ToInt32() );
				pDateTime->SetMonth( aTmp.Copy(2,2).ToInt32() );
				pDateTime->SetYear( aTmp.Copy(4,4).ToInt32() );

				m_aDateTimeList.Insert( pDateTime, LIST_APPEND );
			}
		}

		m_bDoRecursiveDir = FALSE;
		m_bDestIsPath = FALSE;
		m_bDoEmptyDir = ((USHORT)aRecursive.ToInt32() == 0)? FALSE : TRUE;

		ByteString aKey( aConfig.GetKeyName(i) );
		aKey.EraseLeadingChars( '"' ); aKey.EraseTrailingChars( '"' );

		if( aKey.GetChar(aKey.Len()-1) == '/' )
			m_bDoRecursiveDir = TRUE;
		if( aDestPath.GetChar(aDestPath.Len()-1) == '/' )
			m_bDestIsPath = TRUE;

		m_aSourcePath		+= aKey;
		m_aDestinationPath	+= aDestPath;

		m_aSourcePath.ToAbs();
		m_aDestinationPath.ToAbs();

		if( m_bDoRecursiveDir && m_aSourcePath.Exists() ||
			!m_bDoRecursiveDir && m_aSourcePath.GetPath().Exists() )
		{
			if( m_bDoRecursiveDir )
				m_aSrcMask = "*";
			else
				m_aSrcMask = ByteString(m_aSourcePath.CutName(), osl_getThreadTextEncoding());

			nSize += ExecuteAction( m_aSourcePath );
		}

		for( USHORT x = 0; x < m_aDateTimeList.Count(); ++x )
			delete m_aDateTimeList.GetObject(x);
		m_aDateTimeList.Clear();

		for( USHORT xx = 0; xx < m_aNegList.Count(); ++xx )
			delete m_aNegList.GetObject(xx);
		m_aNegList.Clear();
	}
	return nSize;
}

/////////////////////////////////////////////////////////////////////////////
//	userdata task
MigrationTask_UserData::MigrationTask_UserData( SetupMigrationPlugin* pPlugin ) :
	SetupMigrationTask( pPlugin )
{
}

ULONG MigrationTask_UserData::Execute( MigrationExecType eTyp, SiEnvironment* pEnv )
{
	SiDirEntry aSource( pEnv->GetMigrationPath() );
	SiDirEntry aDest	( pEnv->GetDestPath() );

	aSource += ByteString(OFFICE_INI);
	aDest 	+= ByteString("user/");
	aDest	+= ByteString(OFFICE_INI);

	Config aSrcCfg( aSource.GetFullUni() );
	Config aDstCfg( aDest.GetFullUni() );

	aSrcCfg.SetGroup( "User" );
	aDstCfg.SetGroup( "User" );

	if( eTyp == EXECUTE )
	{
		ByteString aValue = aSrcCfg.ReadKey( "User-Id" );
		// aValue.Convert( CHARSET_IBMPC, CHARSET_SYSTEM );
		aDstCfg.WriteKey( "User-Id", aValue );

		aValue = aSrcCfg.ReadKey( "User-Name" );
		// aValue.Convert( CHARSET_IBMPC, CHARSET_SYSTEM );
		aDstCfg.WriteKey( "User-Name", aValue );

		aValue = aSrcCfg.ReadKey( "User-FirstName" );
		// aValue.Convert( CHARSET_IBMPC, CHARSET_SYSTEM );
		aDstCfg.WriteKey( "User-FirstName", aValue );

		aValue = aSrcCfg.ReadKey( "User-Adress" );
		// aValue.Convert( CHARSET_IBMPC, CHARSET_SYSTEM );
		aDstCfg.WriteKey( "User-Adress", aValue );
	}
	return 0L;
}


/////////////////////////////////////////////////////////////////////////////
//	address book task
MigrationTask_AddressBook::MigrationTask_AddressBook( SetupMigrationPlugin* pPlugin ) :
	SetupMigrationTask( pPlugin )
{
}


void MigrationTask_AddressBook::SetSourceAddressBookFilePath(
	SiEnvironment* pEnv )
{
	const ByteString	aDBasePath( "/database/" );
	const ByteString	aSOfficeFilename( OFFICE_INI );
	SiDirEntry		aSource( pEnv->GetMigrationPath() );

	m_aSOfficeIni = aSource + aSOfficeFilename;
	Config aConfig( m_aSOfficeIni.GetFullUni() );

	aConfig.SetGroup( "Database" );
	int	nToken = 1;
	ByteString aAddressBookKey	= aConfig.ReadKey( "Addressbook" );
	ByteString aSdbFilePath		= aAddressBookKey.GetToken( 0, ';' );
	ByteString aDBaseFileName	= aAddressBookKey.GetToken( 1, ';' );

	// substitute a possible macro path definition
	aSdbFilePath.SearchAndReplace( "$(inst)", pEnv->GetMigrationPath() );

	if ( aDBaseFileName.Len() )
	{
		ByteString aDBFileName = aSource.GetFull();
		aDBFileName += aDBasePath;
		aDBFileName += aDBaseFileName;
		aDBFileName += ".dbf";
		m_aDbfSourceFilePath = aDBFileName;

		aDBFileName = aSource.GetFull();
		aDBFileName += aDBasePath;
		aDBFileName += aDBaseFileName;
		aDBFileName += ".dbt";
		m_aDbtSourceFilePath = aDBFileName;
	}

	if ( aSdbFilePath.Len() )
	{
		m_aSdbSourceFilePath = aSdbFilePath;
	}
}

ULONG MigrationTask_AddressBook::GetOldAddressBookFilesSize() const
{
	ULONG	 nSize = 0;
	FileStat aFileStat;
	SiDirEntry aAbsSource;

	aAbsSource = SiDirEntry( m_aDbfSourceFilePath );
	aFileStat = FileStat( aAbsSource );
	nSize += aFileStat.GetSize();

	aAbsSource = SiDirEntry( m_aDbtSourceFilePath );
	aFileStat = FileStat( aAbsSource );
	nSize += aFileStat.GetSize();

	aAbsSource = SiDirEntry( m_aSdbSourceFilePath );
	aFileStat = FileStat( aAbsSource );
	nSize += aFileStat.GetSize();
	return nSize;
}

BOOL MigrationTask_AddressBook::CopyFiles( const SiDirEntry& aDest )
{
	BOOL			bReturn = FALSE;
	const ByteString aSO52DBFFileName( "/user/database/address/address.dbf" );
	const ByteString aSO52DBTFileName( "/user/database/address/address.dbt" );
	const ByteString aSO51SDBFileName( "/user/database/_address.sdb" );

	if ( m_aDbfSourceFilePath.Len() && m_aDbtSourceFilePath.Len() )
	{
		SiDirEntry	aAbsDest1( aDest.GetFull() );
		SiDirEntry	aAbsDest2( aDest.GetFull() );
		SiDirEntry	aAbsSource;
		FileCopier	aCpy;

		aAbsDest1 += aSO52DBFFileName;
		aAbsDest2 += aSO52DBTFileName;

		if( FileStat::HasReadOnlyFlag() )
		{
			if ( FileStat::GetReadOnlyFlag( aAbsDest1 ) )
				 FileStat::SetReadOnlyFlag( aAbsDest1, FALSE );

			if ( FileStat::GetReadOnlyFlag( aAbsDest2 ) )
				 FileStat::SetReadOnlyFlag( aAbsDest2, FALSE );
		}

		aAbsSource	= m_aDbfSourceFilePath;
		aAbsDest1	= SiDirEntry( aDest.GetFull() );
		aAbsDest1 += aSO52DBFFileName;

		if( !aAbsDest1.GetPath().Exists() )
			aAbsDest1.GetPath().MakeDir();
		else
			aAbsDest1.Kill();
		aCpy = FileCopier( aAbsSource, aAbsDest1 );
		aCpy.Execute();

		aAbsSource	= m_aDbtSourceFilePath;
		aAbsDest2	= SiDirEntry( aDest.GetFull() );
		aAbsDest2 += aSO52DBTFileName;

		aAbsDest2.Kill();
		aCpy = FileCopier( aAbsSource, aAbsDest2 );
		aCpy.Execute();
		bReturn = TRUE;
	}

	if ( m_aSdbSourceFilePath.Len() )
	{
		SiDirEntry	aAbsDest( aDest.GetFull() );
		aAbsDest += aSO51SDBFileName;

		SiDirEntry	aAbsSource;
		FileCopier	aCpy;

		if ( FileStat::HasReadOnlyFlag() )
		{
			if ( FileStat::GetReadOnlyFlag( aAbsDest ) )
				 FileStat::SetReadOnlyFlag( aAbsDest, FALSE );
		}

		aAbsSource	= m_aSdbSourceFilePath;
		if ( !aAbsDest.GetPath().Exists() )
			aAbsDest.GetPath().MakeDir();
		else
			aAbsDest.Kill();
		aCpy = FileCopier( aAbsSource, aAbsDest );
		aCpy.Execute();
		bReturn &= TRUE;
	}

	return bReturn;
}

BOOL MigrationTask_AddressBook::CreateAddressBookMapConfigFile( SiEnvironment* pEnv )
{
	ByteString	aConfigFile( ADRBKMAP_INI );
	ByteString	aDestMapConfig( "/user/database/" );
	ByteString	aSOfficeFilename( OFFICE_INI );

	aDestMapConfig += aConfigFile;

	SiDirEntry		aSource( pEnv->GetMigrationPath() );
	SiDirEntry		aDest( pEnv->GetDestPath() );

	m_aSOfficeIni = aSource + aSOfficeFilename;
	Config aConfig( m_aSOfficeIni.GetFullUni() );

	aConfig.SetGroup( "Common" );
	ByteString aLogicToColumnByteString( aConfig.ReadKey( "AddressBookFields" ));

	if ( aLogicToColumnByteString.Len() )
	{
		ByteString aDestFileName( aDest.GetFull() );
		aDestFileName += aDestMapConfig;

		Config aDestConfig( UniString::CreateFromAscii(aDestFileName.GetBuffer()) );

		aDestConfig.SetGroup( "AddressBookConvert" );
		aDestConfig.WriteKey( "LogicColumnToLangColumn", aLogicToColumnByteString );
		return TRUE;
	}
	else
		return FALSE;
}


ULONG MigrationTask_AddressBook::Execute( MigrationExecType eTyp, SiEnvironment* pEnv )
{
	ULONG nSize = 0;

	SetSourceAddressBookFilePath( pEnv );
	switch( eTyp )
	{
		case CALCULATE_SIZE :
			nSize = GetOldAddressBookFilesSize();
			break;
		case EXECUTE :
		{
			if ( CreateAddressBookMapConfigFile( pEnv ))
				CopyFiles( pEnv->GetDestPath() );
			break;
		}
	}
	return nSize;
}


