/*************************************************************************
 *
 *  $RCSfile: installer.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <com/sun/star/uno/Reference.h>
#include <com/sun/star/lang/XComponent.hpp>
#if SUPD > 596
#include <com/sun/star/ucb/XContentProviderConfiguration.hpp>
#include <com/sun/star/ucb/XContentProviderConfigurationManager.hpp>
#else
#include <com/sun/star/ucb/XContentProviderServiceRegistry.hpp>
#endif
#include <cppuhelper/servicefactory.hxx>

#ifndef _COM_SUN_STAR_LANG_XCOMPONENT_HPP_
#include <com/sun/star/lang/XComponent.hpp>
#endif
#ifndef _COM_SUN_STAR_REGISTRY_XSIMPLEREGISTRY_HPP_
#include <com/sun/star/registry/XSimpleRegistry.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_XCHANGESBATCH_HPP_
#include <com/sun/star/util/XChangesBatch.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XNAMEREPLACE_HPP_
#include <com/sun/star/container/XNameReplace.hpp>
#endif

#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

#ifndef _SV_CONFIG_HXX
#include <vcl/config.hxx>
#endif
#include <vcl/svapp.hxx>

#include <svtools/solar.hrc>

#include "decltor.hxx"
#include "action.hxx"
#include "agenda.hxx"
#include "environ.hxx"
#include "fields.hxx"
#include "installer.hxx"
#include "sifsys.hxx"
#include "sihelp.hxx"

#include "progressdlg.hxx"

using namespace osl;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::util;
using namespace ::com::sun::star::container;
using namespace ::com::sun::star::registry;
using namespace ::cppu;

#define DEFINE_CONST_UNICODE(CONSTASCII)        UniString(RTL_CONSTASCII_USTRINGPARAM(CONSTASCII##))

char const INSTALLER_INITFILENAME[]				= "initialize.ini";
char const INSTALLER_SECTION_UNOCOMPONENTS[]	= "UnoComponents";
char const INSTALLER_SECTION_CONFIGURATION[]	= "Configuration";

//////////////////////////////////////////////////////////////////////////////

Installer::Installer()
{
}

Installer::~Installer()
{
}

//////////////////////////////////////////////////////////////////////////////

void Installer::InitializeInstallation( const UniString& rAppFilename )
{
	UniString aAppPath( rAppFilename );
	rtl::OUString aFinishInstallation;
	FileBase::normalizePath( aAppPath, aFinishInstallation );
	aAppPath = UniString( aFinishInstallation );

	xub_StrLen nPos = aAppPath.SearchBackward( '/' );
	aAppPath.Erase( nPos );
	aAppPath += '/';
	aAppPath += DEFINE_CONST_UNICODE( INSTALLER_INITFILENAME );

	DirectoryItem aDI;
	if( DirectoryItem::get(aAppPath, aDI) == osl_File_E_None )
	{
		ByteString aMgrName( "set" );
		aMgrName += ByteString::CreateFromInt32( SOLARUPD );
		ResMgr* pResMgr = ResMgr::CreateResMgr(aMgrName.GetBuffer());
		Resource::SetResManager( pResMgr );

		ProgressDialog* pDlg = new ProgressDialog();
		pDlg->Activate( ProgressDialog::ACTIVATE_REGISTER );
		pDlg->Show();
		pDlg->Update();
		pDlg->Flush();

		/////////////////////////////////////////////////////
		//	INSTALLER_SECTION_UNOCOMPONENTS
		Config aCfg( aAppPath );
		aCfg.SetGroup( ByteString(INSTALLER_SECTION_UNOCOMPONENTS) );

		for( USHORT i = 0; i < aCfg.GetKeyCount(); ++i )
		{
			Application::Reschedule();
			ByteString aFilename( aCfg.GetKeyName(i) );
			BOOL bRegister = aCfg.ReadKey(i) == "1" ? TRUE : FALSE;
			SiHelp::RegisterUnoComponent( aFilename, bRegister );
		}
		SiHelp::DisposeUNOImpl();

		/////////////////////////////////////////////////////
		//	UCB - Services
		SiDirEntry aAbsReg( "." );
		aAbsReg += UniString::CreateFromAscii("applicat.rdb");
		aAbsReg.ToAbs();

		com::sun::star::uno::Reference< com::sun::star::lang::XMultiServiceFactory > xFac;
		xFac = ::cppu::createRegistryServiceFactory( aAbsReg.GetFullUni() );

		::rtl::OUString aStr( UniString("com.sun.star.ucb.Configuration", osl_getThreadTextEncoding()) );
		com::sun::star::uno::Reference< com::sun::star::ucb::XContentProviderConfigurationManager > xCfgMgr(
											xFac->createInstance(aStr), com::sun::star::uno::UNO_QUERY );

		::rtl::OUString aStrCPS( UniString("ContentProviderServices", osl_getThreadTextEncoding()) );
		com::sun::star::uno::Reference< com::sun::star::ucb::XContentProviderConfiguration > xReg(
											xCfgMgr->queryContentProviderConfiguration(aStrCPS) );

		::rtl::OUString aStrLCPS( UniString("LocalContentProviderServices", osl_getThreadTextEncoding()) );
		com::sun::star::uno::Reference< com::sun::star::ucb::XContentProviderConfiguration > xLocalReg(
											xCfgMgr->queryContentProviderConfiguration(aStrLCPS) );
		if( xReg.is() )
		{
			xReg->addContentProviderService( UniString::CreateFromAscii("file"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("ftp"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("http"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("imap"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("news"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.out"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.pop3"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.searchfolder"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.trashcan"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.vim"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("out"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("pop3"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("private"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xReg->addContentProviderService( UniString::CreateFromAscii("vim"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
		}
		if( xLocalReg.is() )
		{
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("file"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("ftp"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("http"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("imap"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("news"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.out"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.pop3"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.searchfolder"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.trashcan"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("vnd.sun.staroffice.vim"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("out"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("pop3"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("private"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
			xLocalReg->addContentProviderService( UniString::CreateFromAscii("vim"),
											 UniString::CreateFromAscii("com.sun.star.ucb.FileContentProvider"),
											 UniString::CreateFromAscii(""),
											 sal_False );
		}

		/////////////////////////////////////////////////////
		//	CONFIGURATION
		pDlg->Activate( ProgressDialog::ACTIVATE_CONFIG );

		SiEnvironment aEnv;
		aCfg.SetGroup( ByteString(INSTALLER_SECTION_CONFIGURATION) );
		if( aCfg.GetKeyCount() )
		{
			SiDirEntry aLogfile(".");
			aLogfile += ByteString("setup.log");
			aLogfile.ToAbs();

			SiAgenda aAgenda;
			aAgenda.OpenLogfile( aLogfile.GetFull() );

			SiDirEntry aApplicat( "." );
			aApplicat += ByteString("applicat.rdb");
			aApplicat.ToAbs();

			if( xFac.is() )
			{
				Reference< XMultiServiceFactory > xCfgProvider;
				xCfgProvider = Reference< XMultiServiceFactory >(
					xFac->createInstance(::rtl::OUString::createFromAscii("com.sun.star.configuration.ConfigurationProvider")),
					UNO_QUERY );

				for( i = 0; i < aCfg.GetKeyCount(); ++i )
				{
					Application::Reschedule();

					ByteString aReadKey( aCfg.GetKeyName(i) );
					ByteString aReadValue( aCfg.ReadKey(i) );

					SiConfigurationItem aCfgItem( SiIdentifier("NULL"), NULL );

					aCfgItem.SetProperty( PROPERTY_PATH, aReadKey.GetToken(0, '|') );
					aCfgItem.SetProperty( PROPERTY_KEY, aReadKey.GetToken(1, '|') );
					aCfgItem.SetProperty( PROPERTY_VALUE, aReadValue );

					ByteString aType( aReadKey.GetToken(2, '|') );
					if( aType == "BOOLEAN" )
						aCfgItem.SetProperty( PROPERTY_FLAGS, VALUE_BOOLEAN );
					else if( aType == "NUMERIC" )
						aCfgItem.SetProperty( PROPERTY_FLAGS, VALUE_NUMERIC );
					else
						aCfgItem.SetProperty( PROPERTY_FLAGS, VALUE_STRING );

					SiConfigurationAction aAction( &aAgenda, PIA_CREATE, &aCfgItem );
					aAction.setUNO( xFac, xCfgProvider );
					aAction.Execute( aEnv );
				}

				Reference< XComponent > xCfgProviderComponent ( xCfgProvider, UNO_QUERY );
				if( xCfgProviderComponent.is() ) xCfgProviderComponent->dispose();
			}
		}

/*		Reference< XComponent > xComponent ( xFac, UNO_QUERY );
		if( xComponent.is() ) xComponent->dispose();
*/
		delete pDlg;
		delete pResMgr;

		SiDirEntry a(aAppPath);
		a.Kill();
	}
}
