/*************************************************************************
 *
 *  $RCSfile: interface.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif

#ifndef _SV_CONFIG_HXX
#include <vcl/config.hxx>
#endif

#include "agenda.hxx"
#include "webaction.hxx"
#include "decltor.hxx"
#include "script.hxx"
#include "environ.hxx"
#include "sistream.hxx"

#include "tools/l2txtenc.hxx"

#include <com/sun/star/setup/OSType.hpp>
#include <com/sun/star/setup/ModuleInfo.hpp>
#include <com/sun/star/setup/MirrorEntry.hpp>

#include "service.hxx"
#include "sihelp.hxx"

//=============================================================================
//	Configuration
//=============================================================================
CachedConfiguration::CachedConfiguration()
{
	pCS = NULL;
}

CachedConfiguration::~CachedConfiguration()
{
	if( pCS ) delete pCS;
	for( USHORT i = 0; i < aMirrorList.Count(); ++i )
		delete aMirrorList.GetObject(i);
	aMirrorList.Clear();
}

void _outDateTime( const DateTime& dt )
{
	fprintf( stdout, "%d:%02d:%02d  %d-%d-%d",
		dt.GetHour(), dt.GetMin(), dt.GetSec(),
		dt.GetDay(), dt.GetMonth(), dt.GetYear() );
}

ByteString _getExtension4OS( OSType eType )
{
	ByteString aExtension;

	if( eType == OSType_WIN )
		aExtension = "WIN";
	else if( eType == OSType_OS2 )
		aExtension = "OS2";
	else if( eType == OSType_MAC )
		aExtension = "MAC";
	else if( eType == OSType_UNIX_SOLS )
		aExtension = "UNIX_SOLS";
	else if( eType == OSType_UNIX_SOLI )
		aExtension = "UNIX_SOLI";
	else if( eType == OSType_UNIX_LINUX )
		aExtension = "UNIX_LINUX";
	else if( eType == OSType_UNIX_HP )
		aExtension = "UNIX_HP";
	else if( eType == OSType_UNIX_SCO )
		aExtension = "UNIX_SCO";
	return aExtension;
}

void SAL_CALL SetupServiceImpl::readOSConfiguration( OSType eType )
{
	ByteString aFilename( "setupserver_" );
	aFilename += _getExtension4OS( eType );
	aFilename += ".ini";

	SiDirEntry aEntry( m_aAppPath );
	aEntry += aFilename;
	aEntry.ToAbs();

	if( !aEntry.Exists() ) return;

	Config aCfg( aEntry.GetFullUni() );
	FileStat aFileStat( aEntry );
	DateTime aDt( aFileStat.DateModified(), aFileStat.TimeModified() );

	fprintf( stdout, "\nread configuration %s: ", _getExtension4OS(eType).GetBuffer() );
	_outDateTime( aDt );

	USHORT xx;
	for( xx = 0; xx < aCfg.GetGroupCount(); ++xx )
	{
		ByteString aGroupname = aCfg.GetGroupName(xx);

		ByteString aProductKey	= aGroupname.GetToken( 0, '-' );
		ByteString aBuildnumber	= aGroupname.GetToken( 1, '-' );
		ByteString aMinor		= aGroupname.GetToken( 2, '-' );
		ByteString aLanguages	= aGroupname.GetToken( 3, '-' );

		CachedConfiguration* pCCfg = new CachedConfiguration();

		pCCfg->aIdentifier.eOSType			= eType;
		pCCfg->aIdentifier.strProductKey	= UniString(aProductKey, osl_getThreadTextEncoding());
 		pCCfg->aIdentifier.nBuildnumber		= aBuildnumber.ToInt32();
		pCCfg->aIdentifier.strInstLanguages	= UniString(aLanguages, osl_getThreadTextEncoding());
		pCCfg->aIdentifier.strInstMinor		= UniString(aMinor, osl_getThreadTextEncoding());
		pCCfg->aLastValidation				= aDt;

		aCfg.SetGroup( aGroupname );

		for( USHORT i = 0; i < aCfg.GetKeyCount(); ++i )
		{
			ByteString aKey = aCfg.GetKeyName(i);
			if( aKey.CompareIgnoreCaseToAscii("script") == COMPARE_EQUAL )
			{
				pCCfg->aScriptLocation = aCfg.ReadKey(i);
			}
			else if( aKey.CompareIgnoreCaseToAscii("mirror") == COMPARE_EQUAL )
			{
				pCCfg->aMirrorList.Insert( new UniString(aCfg.ReadKey(i), osl_getThreadTextEncoding()), LIST_APPEND );
			}
		}
		m_ConfigCache.Insert( pCCfg, LIST_APPEND );
	}

	fprintf( stdout, " %d entrys", xx );
}

void SAL_CALL SetupServiceImpl::readAllOSConfiguration()
{
	readOSConfiguration( OSType_WIN );
	readOSConfiguration( OSType_OS2 );
	readOSConfiguration( OSType_MAC );
	readOSConfiguration( OSType_UNIX_SOLS );
	readOSConfiguration( OSType_UNIX_SOLI );
	readOSConfiguration( OSType_UNIX_LINUX );
	readOSConfiguration( OSType_UNIX_HP );
	readOSConfiguration( OSType_UNIX_SCO );
}

//=============================================================================
//	getCachedScript
//=============================================================================
BOOL _isEqual( const VersionIdentifier& aIdent1, const VersionIdentifier& aIdent2 )
{
	if( aIdent1.eOSType				== aIdent2.eOSType &&
		aIdent1.nBuildnumber		== aIdent2.nBuildnumber &&
		aIdent1.strProductKey		== aIdent2.strProductKey &&
		aIdent1.strInstLanguages	== aIdent2.strInstLanguages &&
		aIdent1.strInstMinor		== aIdent2.strInstMinor )
		return TRUE;
	return FALSE;
}

CachedConfiguration* SAL_CALL SetupServiceImpl::getCachedConfig( const VersionIdentifier& aVerIdentifier )
{
	CachedConfiguration* pReturn = NULL;

	for( USHORT i = 0; i < m_ConfigCache.Count(); ++i )
	{
		CachedConfiguration* pCCfg = m_ConfigCache.GetObject(i);
		if( _isEqual(aVerIdentifier, pCCfg->aIdentifier) )
		{
			pReturn = pCCfg;
			break;
		}
	}

	if( !pReturn &&
		!aVerIdentifier.nBuildnumber &&
		!UniString(aVerIdentifier.strInstLanguages).Len() &&
		!UniString(aVerIdentifier.strInstMinor).Len() )
	{
		USHORT nBestFnd = 0;
		for( USHORT i = 0; i < m_ConfigCache.Count(); ++i )
		{
			CachedConfiguration* pCCfg = m_ConfigCache.GetObject(i);
			if( pCCfg->aIdentifier.strProductKey == aVerIdentifier.strProductKey &&
				pCCfg->aIdentifier.eOSType == aVerIdentifier.eOSType &&
//				pCCfg->aIdentifier.nLanguage == aVerIdentifier.nLanguage &&
				(!nBestFnd || pCCfg->aIdentifier.nBuildnumber > nBestFnd) )
			{
				pReturn = pCCfg;
				nBestFnd = pCCfg->aIdentifier.nBuildnumber;
			}
		}
	}

	if( pReturn )
	{
		ByteString aFilename( "setupserver_" );
		aFilename += _getExtension4OS( pReturn->aIdentifier.eOSType );
		aFilename += ".ini";

		SiDirEntry aEntry( m_aAppPath );
		aEntry += aFilename;
		aEntry.ToAbs();

		FileStat aFileStat( aEntry );
		if( pReturn->aLastValidation != DateTime(aFileStat.DateModified(), aFileStat.TimeModified()) )
		{
			OSType eOS = pReturn->aIdentifier.eOSType;
			for( USHORT i = 0; i < m_ConfigCache.Count(); ++i )
			{
				CachedConfiguration* p = m_ConfigCache.GetObject(i);
				if( p->aIdentifier.eOSType == eOS )
				{
					m_ConfigCache.Remove( p );
					delete p;
					i--;
				}
			}
			readOSConfiguration( eOS );
			pReturn = getCachedConfig( aVerIdentifier );
			return pReturn;
		}

		SiDirEntry aValidate(pReturn->aScriptLocation);
		if( !aValidate.Exists() )
		{
			m_ConfigCache.Remove( pReturn );
			delete pReturn;
			pReturn = NULL;
		}
		else if( !pReturn->pCS )
		{
			SiAnsiFileStream aStream;
			aStream.Open( UniString(pReturn->aScriptLocation, osl_getThreadTextEncoding()),
					  	STREAM_READ );
			if( aStream.IsOpen() )
			{
				SiCompiledScript* cs = new SiCompiledScript;

				SiCompilerRef xCompiler = new SiCompiler( aStream, pReturn->aIdentifier.eOSType, TRUE );
				xCompiler->NoApplication();
				xCompiler->CompileTo( cs );

				pReturn->pCS = cs;
			}
		}
	}
	return pReturn;
}


//=============================================================================
//							INTERFACE IMPLEMENTATION
//=============================================================================

//=============================================================================
//	isVersionSupported
//=============================================================================
sal_Bool SAL_CALL SetupServiceImpl::isVersionSupported( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( pCC ) return sal_True;
	return sal_False;
}

//=============================================================================
//	getAvailableLanguages
//=============================================================================

Sequence< sal_Int16 > SAL_CALL SetupServiceImpl::getAvailableLanguages( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	Sequence< sal_Int16 > seqLanguages;
	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );

	if( !pCC || !pCC->pCS ) return seqLanguages;

	ByteString aInstLanguages = pCC->pCS->GetInstallation()->GetLanguages();

	USHORT nDelimTok = 0;
	USHORT nTokCount = aInstLanguages.GetTokenCount(',');
	seqLanguages.realloc( nTokCount );

	for( USHORT x = 0; x < nTokCount; ++x )
	{
		sal_Int16 n = (sal_Int16) aInstLanguages.GetToken(0, ',', nDelimTok).ToInt32();
		seqLanguages[x] = n;
	}

	return seqLanguages;
}

//=============================================================================
//	getProductname
//	getDefaultDestinationPath
//	getMirrorList
//	getStandardSizeValues
//=============================================================================
OUString SAL_CALL SetupServiceImpl::getProductname( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS ) return OUString::createFromAscii("error: productname not found");

	OUString aProductname = OUString::createFromAscii( pCC->pCS->GetRootModule()->GetName().GetBuffer() );
	return aProductname;
}

OUString SAL_CALL SetupServiceImpl::getDefaultDestinationPath( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS ) return OUString::createFromAscii("error: default destinationpath not found");

	OUString aDefaultDestinationPath;
	if( aVerIdentifier.eOSType != OSType_WIN )
		aDefaultDestinationPath += OUString::createFromAscii( "<homedir>/" );
	aDefaultDestinationPath += OUString::createFromAscii(
		pCC->pCS->GetInstallation()->GetDefaultDestPath().GetBuffer() );

	return aDefaultDestinationPath;
}

Sequence< Any > SAL_CALL SetupServiceImpl::getMirrorList( const VersionIdentifier& aVerIdentifier )
{
	Sequence< Any > seqMirror;
	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );

	if( !pCC || !pCC->pCS ) return seqMirror;

	seqMirror.realloc( pCC->aMirrorList.Count() );
	for( USHORT i = 0; i < pCC->aMirrorList.Count(); ++i )
	{
		MirrorEntry aMirror;
		aMirror.strDisplayname	= pCC->aMirrorList.GetObject(i)->GetToken(0,';');
		aMirror.strPath 		= pCC->aMirrorList.GetObject(i)->GetToken(1,';');
		aMirror.nQuality 		= 0;
		seqMirror[i] <<= aMirror;
	}

	return seqMirror;
}

SizeInfo SAL_CALL SetupServiceImpl::getStandardSizeValues( const VersionIdentifier& aVerIdentifier )
{
	SizeInfo aSzInfo;
	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS ) return aSzInfo;

	LanguageContext* pCtx = new LanguageContext;
	pCtx->nLanguage	= pCC->aIdentifier.nLanguage;
	pCtx->isProg   	= TRUE;
	pCtx->isDoc		= TRUE;

	ULONG lCluster = 16 * 1024;

	SiEnvironment* pEnv = new SiEnvironment;
	pEnv->AddLanguageContext( pCtx );

	pCC->pCS->GetRootModule()->Select( SiModule::ALL_DEFAULT );
	aSzInfo.nNormalSizeKB		= pCC->pCS->GetRootModule()->
								  CalculateSize( *pEnv, SiModule::DEFAULT, lCluster, FALSE, FALSE, FALSE );
	aSzInfo.nNormalTempSizeKB	= aSzInfo.nNormalSizeKB + pCC->pCS->GetRootModule()->
								  CalculateSize( *pEnv, SiModule::TEMP, lCluster, FALSE, FALSE, FALSE );

	pCC->pCS->GetRootModule()->Select( SiModule::ALL_MINIMAL );
	aSzInfo.nMinimalSizeKB		= pCC->pCS->GetRootModule()->
								  CalculateSize( *pEnv, SiModule::MINIMAL, lCluster, FALSE, FALSE, FALSE );
	aSzInfo.nMinimalTempSizeKB	= aSzInfo.nMinimalSizeKB + pCC->pCS->GetRootModule()->
								  CalculateSize( *pEnv, SiModule::TEMP, lCluster, FALSE, FALSE, FALSE );

	ULONG oneMB = 1024L * 1024L;

	aSzInfo.nNormalSizeKB		/= oneMB;
	aSzInfo.nNormalTempSizeKB	/= oneMB;
	aSzInfo.nMinimalSizeKB		/= oneMB;
	aSzInfo.nMinimalTempSizeKB	/= oneMB;

	return aSzInfo;
}

//=============================================================================
//	getReadmeText
//	getLicenseText
//=============================================================================
ByteString _getFilename4OS( OSType eOSType, USHORT nType )
{
	if( nType == 1 )
	{
		switch( eOSType )
		{
			case OSType_WIN:
			case OSType_OS2:
				return ByteString(WIN_README_FILE_NAME);
				break;
			case OSType_MAC:
				return ByteString(MAC_README_FILE_NAME);
				break;
			case OSType_UNIX_SOLS:
			case OSType_UNIX_SOLI:
			case OSType_UNIX_LINUX:
			case OSType_UNIX_HP:
			case OSType_UNIX_SCO:
				return ByteString(UNX_README_FILE_NAME);
				break;
		}
	}
	else
	{
		switch( eOSType )
		{
			case OSType_WIN:
			case OSType_OS2:
				return ByteString(WIN_LICENSE_FILE_NAME);
				break;
			case OSType_MAC:
				return ByteString(MAC_LICENSE_FILE_NAME);
				break;
			case OSType_UNIX_SOLS:
			case OSType_UNIX_SOLI:
			case OSType_UNIX_LINUX:
			case OSType_UNIX_HP:
			case OSType_UNIX_SCO:
				return ByteString(UNX_LICENSE_FILE_NAME);
				break;
		}
	}
	return ByteString();
}

UniString _getReadme( CachedConfiguration* pCC, USHORT nType )
{
	SiDirEntry aOld( "." );
	SiDirEntry aFoo( SiDirEntry("fo*") );
	SiDirEntry aTmp( aFoo.TempName() );

	aTmp.MakeDir();
	aTmp.SetCWD();

	SiDirEntry aPath( pCC->aScriptLocation );

	SiEnvironment aLocalEnv;
	aLocalEnv.SetSourcePath( ((SiDirEntry&)aPath.GetPath()).GetFull() );

	SiHelp::UnzipReadmeZIP( pCC->aIdentifier.nLanguage, pCC->pCS, &aLocalEnv );

	UniString	aReturn;
	SiDirEntry	aFilename( _getFilename4OS(pCC->aIdentifier.eOSType, nType) );

	BOOL bExtracted = TRUE;
	if( !aFilename.Exists() )
	{
		SiDirEntry aOrg( aPath.GetPath().GetFull() );
		aOrg.SetCWD();
		bExtracted = FALSE;
	}

	if( aFilename.Exists() )
	{
		SvFileStream aReadStrm( aFilename.GetFullUni(), STREAM_READ );
		if( aReadStrm.IsOpen() )
		{
			char* pBuf = new char[32000];
			ULONG nRead = aReadStrm.Read( pBuf, 32000 );
			aReadStrm.Close();
			pBuf[nRead] = 0x00;
			aReturn = UniString( pBuf, osl_getThreadTextEncoding() );
			delete pBuf;
		}
	}

	aOld.SetCWD();

	if( bExtracted )
	{
		SiDirEntry aKill( _getFilename4OS(pCC->aIdentifier.eOSType, 1) );
		aKill.Kill();

		aKill = _getFilename4OS(pCC->aIdentifier.eOSType, 2);
		aKill.Kill();
	}
	aTmp.Kill();

	return aReturn;
}

OUString SAL_CALL SetupServiceImpl::getReadmeText( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS ) return OUString::createFromAscii("error: readme file not found");

	return _getReadme( pCC, 1 );
}

OUString SAL_CALL SetupServiceImpl::getLicenseText( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS ) return OUString::createFromAscii("error: license file not found");

	return _getReadme( pCC, 2 );
}

OUString SAL_CALL SetupServiceImpl::getHelpText( const VersionIdentifier& aVerIdentifier,
												 const OUString& strPagename )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS ) return OUString::createFromAscii("error: no help text");

	UniString aThanksToOurBaseTec(strPagename);
	SiHelpText* pText = pCC->pCS->GetHelpTextForId(
							SiHelp::GetUIPageIdByName( ByteString(aThanksToOurBaseTec, osl_getThreadTextEncoding()) ),
							aVerIdentifier.nLanguage );
	if( pText )
	{
		ByteString aHelpText( pText->GetHelpText() );
		while( aHelpText.SearchAndReplace("\\n", "\n") != STRING_NOTFOUND ) {}
		UniString aHelp( aHelpText, osl_getThreadTextEncoding() );
		return aHelp;
	}

	return OUString::createFromAscii("");
}

//=============================================================================
//	getRootModule
//=============================================================================

ModuleInfo _recurseModuleInfo( SiModule* pMod, rtl_TextEncoding eTextEncoding )
{
	SiEnvironment aEmptyEnv;
	ModuleInfo aModule;

	aModule.strID				= UniString( pMod->GetID().GetBuffer(), eTextEncoding );
	aModule.strName				= UniString( pMod->GetName().GetBuffer(), eTextEncoding );
	aModule.strDescription		= UniString( pMod->GetDescription().GetBuffer(), eTextEncoding );
	aModule.strOnSelect			= UniString( pMod->GetOnSelect().GetBuffer(), eTextEncoding );
	aModule.strOnDeselect		= UniString( pMod->GetOnDeselect().GetBuffer(), eTextEncoding );

	aModule.isMinimal			= pMod->IsMinimal();
	aModule.isDefault			= pMod->GetParent() == NULL? TRUE : pMod->IsDefault();
	aModule.isHiddenRoot		= pMod->IsHidden();
	aModule.isDontSelectByUser	= pMod->DontSelectByUser();

	aModule.nSizeKB				= pMod->CalculateSize( aEmptyEnv, SiModule::ALL, 512, FALSE, FALSE) / 1024;

	const SiModuleList* pSubModLst = pMod->GetModuleList();
	aModule.seqSubModules.realloc( pSubModLst->Count() );

	for( USHORT x = 0; x < pSubModLst->Count(); x++ )
		aModule.seqSubModules[x] <<= _recurseModuleInfo( pSubModLst->GetObject(x), eTextEncoding );

	return aModule;
}

ModuleInfo SAL_CALL SetupServiceImpl::getRootModule( const VersionIdentifier& aVerIdentifier )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	if( !pCC || !pCC->pCS )
		return ModuleInfo();

	SiModule* pRoot = pCC->pCS->GetRootModule();
	return _recurseModuleInfo( pRoot, Langcode2TextEncoding(pCC->aIdentifier.nLanguage) );
}

//=============================================================================
//	getActionListForInstall
//=============================================================================
Sequence< Any > SAL_CALL SetupServiceImpl::getActionListForInstall(
		const VersionIdentifier& aVerIdentifier,
		const InstallEnvironment& aEnvironment,
		InstallResponse& aResponse )
{
	MutexGuard aGuard( &m_ScriptCacheMutex );

	SiEnvironment* pEnv = new SiEnvironment;
	pEnv->SetDestPath	( ByteString(UniString(aEnvironment.strDestinationPath), osl_getThreadTextEncoding()) );
	pEnv->SetWebOSType	( aEnvironment.eOSType );
	pEnv->SetUserName	( ByteString(UniString(aEnvironment.strUserName), osl_getThreadTextEncoding()) );
	pEnv->SetUserFirstName( ByteString(UniString(aEnvironment.strUserFirstName), osl_getThreadTextEncoding()) );
	pEnv->SetUserId		( ByteString(UniString(aEnvironment.strUserId), osl_getThreadTextEncoding()) );
	pEnv->SetUserAddress( SiHelp::GetAddressByteString(
						  ByteString(UniString(aEnvironment.strCompanyname), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strStreet), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strCountry), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strZip), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strCity), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strTitle), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strPosition), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strPhonePriv), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strPhoneCompany), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strPhoneFax), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strEmail), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strState), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strFatherName), osl_getThreadTextEncoding()),
						  ByteString(UniString(aEnvironment.strApartmentNr), osl_getThreadTextEncoding()) ));

	sal_Int32 nLangNr = aEnvironment.seqLanguages.getLength();

	for( sal_Int32 n = 0; n < nLangNr; ++n )
	{
		sal_Int16 nLanguage = aEnvironment.seqLanguages[n];

		LanguageContext* pCtx = new LanguageContext;
		pCtx->nLanguage	= nLanguage;
		pCtx->isProg   	= TRUE;
		pCtx->isDoc		= n == 0? TRUE : FALSE;

		pEnv->AddLanguageContext( pCtx );
	}

	CachedConfiguration* pCC = getCachedConfig( aVerIdentifier );
	Sequence< Any > seqActionList;

	if( !pCC || !pCC->pCS ) return seqActionList;

	SiWebAgenda aAgenda;
	aAgenda.SetAppLanguage( aEnvironment.seqLanguages[0] );
	aAgenda.SetEnvironment( pEnv );
	ULONG nSz = aAgenda.CreateForWebInstall( pCC, aEnvironment );
	nSz /= 1024;

	aAgenda.JoinActionLists();
	SiActionList& rLst = aAgenda.GetWebActionList();

	if( !aEnvironment.bFullWebinstall )
		for( USHORT ii = 0; ii < rLst.Count(); ++ii )
			if( ((SiWebAction*)rLst.GetObject(ii))->GetType() != ActionType_DOWNLOAD )
			{
				 SiWebAction* pAction = (SiWebAction*) rLst.Remove( rLst.GetObject(ii) );
				 delete pAction;
				 ii--;
			}

	seqActionList.realloc( rLst.Count() );

	for( USHORT i = 0; i < rLst.Count(); ++i )
	{
		SiWebAction* pAction = (SiWebAction*) rLst.GetObject(i);
		switch( pAction->GetType() )
		{
			// install
			case ActionType_DOWNLOAD :
				seqActionList[i] <<= ((SiWebDownloadAction*)pAction)->GetWebAction();
				break;

			case ActionType_MAKEDIR :
				seqActionList[i] <<= ((SiWebMakeDirAction*)pAction)->GetWebAction();
				break;

			case ActionType_COPYFILE :
				seqActionList[i] <<= ((SiWebCopyFileAction*)pAction)->GetWebAction();
				break;

			case ActionType_UNZIPFILE :
				seqActionList[i] <<= ((SiWebUnzipAction*)pAction)->GetWebAction();
				break;

			case ActionType_PROFILEITEM :
 				seqActionList[i] <<= ((SiWebProfileItemAction*)pAction)->GetWebAction();
				break;

			case ActionType_FONT :
 				seqActionList[i] <<= ((SiWebFontAction*)pAction)->GetWebAction();
				break;

			case ActionType_WINREGISTRY :
 				seqActionList[i] <<= ((SiWebWindowsRegistryAction*)pAction)->GetWebAction();
				break;

			case ActionType_MAKEFOLDER :
 				seqActionList[i] <<= ((SiWebMakeFolderAction*)pAction)->GetWebAction();
				break;

			case ActionType_MAKEFOLDERITEM :
 				seqActionList[i] <<= ((SiWebMakeFolderItemAction*)pAction)->GetWebAction();
				break;

			// deinstall
			case ActionType_DELETEFILE :
 				seqActionList[i] <<= ((SiWebDeleteFileAction*)pAction)->GetWebAction();
				break;

			case ActionType_DELETEDIR :
 				seqActionList[i] <<= ((SiWebDeleteDirAction*)pAction)->GetWebAction();
				break;

			case ActionType_DELETEFOLDERITEM :
 				seqActionList[i] <<= ((SiWebDeleteFolderItemAction*)pAction)->GetWebAction();
				break;

			case ActionType_DELETEFOLDER :
 				seqActionList[i] <<= ((SiWebDeleteFolderAction*)pAction)->GetWebAction();
				break;
		}
		delete pAction;
	}
	rLst.Clear();

	aResponse.nSizeNeeded = nSz;
	if( nSz < aEnvironment.nDriveSize )
	{
		aResponse.bSuccess = TRUE;
		aResponse.eErrorCode = ResponseErrorCode_NOERROR;
	}
	else
	{
		aResponse.bSuccess = FALSE;
		aResponse.eErrorCode = ResponseErrorCode_NOT_ENOUGH_SPACE;
	}

    aResponse.bReboot = FALSE;
    aResponse.bLogout = FALSE;

	if( pEnv->IsReboot() )
		aResponse.bReboot = TRUE;
	else if( pEnv->IsLogout() )
		aResponse.bLogout = TRUE;

	delete pEnv;
	return seqActionList;
}

