/*************************************************************************
 *
 *  $RCSfile: address.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdlib.h>
#include "address.hxx"

#include "dialog.hrc"
#include "address.hrc"


// c-Funktion ------------------------------------------------------------

static ByteString ConvertToStore( const ByteString& rText )
{
	ByteString sRet;
	USHORT i = 0;

	while ( i < rText.Len() )
	{
		if ( (char)rText[i] == '\\' || (char)rText[i] == '#' )
			sRet += '\\';
		sRet += (char)rText[i++];
	}
	return sRet;
}

// class SetupAddressDialog ----------------------------------------------

SetupAddressDialog::SetupAddressDialog( Window *pParent, ByteString const& aProgramName,
	BOOL bRequired )
: ModalDialog( pParent, ResId( RID_ADDRESS_DLG ) )
, m_aCompanyLbl		( this, ResId( FT_COMPANY ) )
, m_aCompanyEdit	( this, ResId( ED_COMPANY ) )
, m_aNameLbl		( this, ResId( FT_NAME ) )
, m_aFirstName		( this, ResId( ED_FIRSTNAME ) )
, m_aName			( this, ResId( ED_NAME ) )
, m_aShortName		( this, ResId( ED_SHORTNAME ) )
, m_aStreetLbl		( this, ResId( FT_STREET ) )
, m_aStreetEdit		( this, ResId( ED_STREET ) )
, m_aCityLbl		( this, ResId( FT_COUNTRY ) )
, m_aCountryEdit	( this, ResId( ED_COUNTRY ) )
, m_aPLZEdit		( this, ResId( ED_PLZ ) )
, m_aCityEdit		( this, ResId( ED_CITY ) )
, m_aUsCityEdit		( this, ResId( ED_US_CITY ) )
, m_aStateLbl		( this, ResId( FT_STATE ) )
, m_aStateEdit		( this, ResId( ED_STATE ) )
, m_aUsStateEdit	( this, ResId( ED_US_STATE ) )
, m_aUsPLZEdit		( this, ResId( ED_US_ZIP ) )
, m_aUsCountryEdit	( this, ResId( ED_US_COUNTRY ) )
, m_aTitlePosLbl	( this, ResId( FT_TITLEPOS ) )
, m_aTitleEdit		( this, ResId( ED_TITLE ) )
, m_aPositionEdit	( this, ResId( ED_POSITION ) )
, m_aTelPrivLbl		( this, ResId( FT_TELPRIVAT ) )
, m_aTelPrivEdit	( this, ResId( ED_TELPRIVAT ) )
, m_aTelCompanyLbl	( this, ResId( FT_TELCOMPANY ) )
, m_aTelCompanyEdit	( this, ResId( ED_TELCOMPANY ) )
, m_aFaxLbl			( this, ResId( FT_FAX ) )
, m_aFaxEdit		( this, ResId( ED_FAX ) )
, m_aEmailLbl		( this, ResId( FT_EMAIL ) )
, m_aEmailEdit		( this, ResId( ED_EMAIL ) )
, m_aAddrFrm		( this, ResId( GB_ADDRESS ) )
, m_aOKBtn			( this, ResId( BTN_ADDRESS_OK 	 ) )
, m_aCancelBtn		( this, ResId( BTN_ADDRESS_CANCEL ) )
, m_aHelpBtn		( this, ResId( BTN_ADDRESS_HELP ) )
, aReqText1	  	 	( this, ResId( FT_REQ_1 ) )
{
	aReqText1.Hide();

	if ( System::GetFormatLanguage() == LANGUAGE_ENGLISH_US )
	{
		// amerikanische Postadresse aufbauen
		m_aCityEdit.Hide();
		m_aCountryEdit.Hide();
		m_aPLZEdit.Hide();
		m_aStateEdit.Hide();
		m_aUsCityEdit.Show();
		m_aUsCountryEdit.Show();
		m_aUsPLZEdit.Show();
		m_aUsStateEdit.Show();
		m_aCityLbl.SetText( ByteString( ResId( STR_US_CITY ) ) );
		m_aStateLbl.SetText( ByteString( ResId( STR_US_STATE ) ) );
	}
	FreeResource();

	m_aCompanyEdit.GrabFocus();

	m_aFirstName.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
	m_aName.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
	m_aHelpBtn.SetClickHdl( LINK( this, SetupAddressDialog, HelpHdl ) );

	ByteString aText(aProgramName);
	aText += ByteString( ResId( STR_DLG_TITLE ) );
	SetText( aText );

	Point aPnt = pParent->GetPosPixel();
	Size aSiz = GetSizePixel(),
		 aWinSiz = pParent->GetSizePixel();

	long nXDelta = ( aWinSiz.Width() - aSiz.Width()  ) / 2;
	long nYDelta = ( aWinSiz.Height() - aSiz.Height()  ) / 2;

	if ( nXDelta > 0 )
		aPnt.X() += nXDelta;
	if ( nYDelta > 0 )
		aPnt.Y() += nYDelta;
	SetPosPixel( aPnt );

	m_bDataRequired = bRequired;
	if( m_bDataRequired )
	{
		Size aSz = GetSizePixel();
		Point aPt;
		aPt.X() = 63L;

		SetSizePixel( Size(aSz.Width(), aSz.Height() + LogicToPixel(aPt, MapMode(MAP_APPFONT)).X()));

		m_aHelpBtn.Hide();
		aReqText1.Show();

		m_aOKBtn.Enable( FALSE );

		m_aFirstName.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
		m_aName.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
		m_aStreetEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );

		if ( System::GetFormatLanguage() == LANGUAGE_ENGLISH_US )
		{
			m_aUsCountryEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
			m_aUsPLZEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
			m_aUsCityEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
		}
		else
		{
			m_aCountryEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
			m_aPLZEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
			m_aCityEdit.SetModifyHdl( LINK( this, SetupAddressDialog, ModifyHdl ) );
		}
	}

	#ifdef UNX
	m_aShortName.SetText( getenv( "LOGNAME" ) );
	#endif
}

// -----------------------------------------------------------------------

ByteString SetupAddressDialog::GetAddress() const
{
	BOOL bUS = ( System::GetFormatLanguage() == LANGUAGE_ENGLISH_US );
	const char cAddrToken = '#';
	ByteString aAddrStr( ::ConvertToStore( m_aCompanyEdit.GetText() ) );
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aStreetEdit.GetText() );

	if ( bUS )
	{
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aUsCountryEdit.GetText() );
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aUsPLZEdit.GetText() );
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aUsCityEdit.GetText() );
	}
	else
	{
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aCountryEdit.GetText() );
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aPLZEdit.GetText() );
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aCityEdit.GetText() );
	}
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aTitleEdit.GetText() );
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aPositionEdit.GetText() );
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aTelPrivEdit.GetText() );
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aTelCompanyEdit.GetText() );
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aFaxEdit.GetText() );
	( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aEmailEdit.GetText() );

	if ( bUS )
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aUsStateEdit.GetText() );
	else
		( aAddrStr += cAddrToken ) += ::ConvertToStore( m_aStateEdit.GetText() );
	return aAddrStr.Convert( CHARSET_SYSTEM, CHARSET_IBMPC );
}

// -----------------------------------------------------------------------

ByteString SetupAddressDialog::GetUserName() const
{
	ByteString aStr( m_aName.GetText() );
	return aStr.Convert( CHARSET_SYSTEM, CHARSET_IBMPC );
}

// -----------------------------------------------------------------------

ByteString SetupAddressDialog::GetUserFirstName() const
{
	ByteString aStr( m_aFirstName.GetText() );
	return aStr.Convert( CHARSET_SYSTEM, CHARSET_IBMPC );
}

// -----------------------------------------------------------------------

ByteString SetupAddressDialog::GetUserId() const
{
	ByteString aStr( m_aShortName.GetText() );
	return aStr.Convert( CHARSET_SYSTEM, CHARSET_IBMPC );
}

// -----------------------------------------------------------------------
// -----------------------------------------------------------------------

IMPL_LINK( SetupAddressDialog, ModifyHdl, Edit *, pEdit )
{
	#ifndef UNX
		ByteString aShortStr( m_aShortName.GetText() );
		switch (aShortStr.Len())
		{
			case 0:	aShortStr = "  "; break;
			case 1: aShortStr += " "; break;
		}
		if ( pEdit == &m_aFirstName )
			aShortStr(0) = (char)m_aFirstName.GetText();
		else if ( pEdit == &m_aName )
			aShortStr(1) = (char)m_aName.GetText();
		m_aShortName.SetText( aShortStr );
	#endif

	if( m_bDataRequired )
	{
		if( m_aFirstName.GetText().Len()	&&
			m_aName.GetText().Len()			&&
			m_aStreetEdit.GetText().Len()	&&
		  ((System::GetFormatLanguage() == LANGUAGE_ENGLISH_US &&
			m_aUsCountryEdit.GetText().Len() 	&&
			m_aUsPLZEdit.GetText().Len()		&&
			m_aUsCityEdit.GetText().Len()) 		||
		  ( m_aCountryEdit.GetText().Len()	 	&&
			m_aPLZEdit.GetText().Len()			&&
			m_aCityEdit.GetText().Len())) )
		{
			m_aOKBtn.Enable( TRUE );
		}
		else
		{
			m_aOKBtn.Enable( FALSE );
		}
	}
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK_INLINE_START( SetupAddressDialog, HelpHdl, PushButton *, EMPTYARG )
{
	InfoBox( this, ResId( DLG_INFOADDRESS ) ).Execute();
	m_aCompanyEdit.GrabFocus();
	return 0;
}
IMPL_LINK_INLINE_END( SetupAddressDialog, HelpHdl, PushButton *, EMPTYARG )



