/*************************************************************************
 *
 *  $RCSfile: instinfo.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <tlfsys.hxx>
#include "decltor.hxx"
#include "instinfo.hxx"

#include "dialog.hrc"
#include "instinfo.hrc"
#include "agenda.hxx"
#include "main.hxx"

// Vobis Hack
static BOOL bMaxPercent = FALSE;

// class SetupInstallInfoDialog ------------------------------------------
SetupInstallInfoDialog::SetupInstallInfoDialog( Window* pParent, ByteString const& aProgramName ) :
	ModelessDialog	( pParent, ResId( RID_INSTINFO_DLG ) ),
	m_aProgress		( this, ResId( WIN_PROGRESS ) ),
	m_aFixedLine	( this, ResId( FL_INST_INFO ) ),
	m_aCancel		( this, ResId( PB_CANCEL_INST ) ),
	m_aSlideWin		( this, ResId( WIN_SILDE ) ),
	m_aDescText( this, ResId( FT_INST_DESC ) ),
	m_aInfoText( this, ResId( FT_INST_INFO ) ),
	m_aEstimateText(this, ResId( FT_ESTIMATE_TEXT ) ),
	m_aEstimate(this, ResId( FT_ESTIMATE) ),
	m_aProgramName (aProgramName),
	m_bStop (FALSE),
	m_lGuess(9999999)
{
	FreeResource();

	m_aDescText.Hide();

	m_aCancel.SetClickHdl( LINK(this, SetupInstallInfoDialog, ExitHdl) );

	ByteString aText(m_aProgramName);
	aText += ByteString( ResId( STR_DLG_TITLE ) );
	SetText( aText );
	m_aStopWatch.Start();
}

#define DLG_DEF_ADD_WIDTH	16
#define DLG_DEF_ADD_HEIGHT	100
#define DLG_DEF_ADD_TOP 	8
#define DEF_DISTANCE		3

void SetupInstallInfoDialog::CalcSizeForSlide(SiSlide* pSiSlide, ByteString const& rPath)
{
	SiDirEntry aFile = rPath;
	aFile += pSiSlide->GetBmpFileName();

	Bitmap aBmp;
	SvFileStream aSvFileStrm( aFile.GetFull(), STREAM_READ );
	aSvFileStrm >> aBmp;

	Size 	aBmpSize( aBmp.GetSizePixel() );
	Point   aCtrlPos( DLG_DEF_ADD_WIDTH/2, aBmpSize.Height() + DLG_DEF_ADD_TOP + DEF_DISTANCE );
	Size	aTextHg( Size( 0, m_aEstimateText.GetSizePixel().Height() +
					m_aInfoText.GetSizePixel().Height()) );

	m_aProgress.SetSizePixel( Size((aBmpSize.Width()/2)-10, 28) );
	Size aSz = m_aProgress.GetOutputSizePixel();
	aSz.Width() -= (aSz.Width() -4)% (((aSz.Height() -4) *2 / 3) + 3);
	m_aProgress.SetOutputSizePixel( aSz );

	// Text hoeher als Progress?
	if( aTextHg.Height() > m_aProgress.GetSizePixel().Height() )
	{
		USHORT nDiff = (aTextHg.Height() - m_aProgress.GetSizePixel().Height()) / 2;
		m_aProgress.SetPosPixel( Point(aCtrlPos.X(), aCtrlPos.Y() + nDiff) );
	}
	else
		m_aProgress.SetPosPixel(aCtrlPos);

	USHORT nXPos = m_aProgress.GetSizePixel().Width() + 5 + aCtrlPos.X();
	m_aInfoText.SetPosSizePixel( Point(nXPos, aCtrlPos.Y()),
		Size(aBmpSize.Width() - 5 - aCtrlPos.X() - nXPos,m_aInfoText.GetSizePixel().Height()) );
	m_aEstimateText.SetPosPixel( Point(m_aProgress.GetSizePixel().Width()+5+aCtrlPos.X(),
		aCtrlPos.Y() + m_aInfoText.GetSizePixel().Height()) );
	m_aEstimate.SetPosPixel( Point(m_aProgress.GetSizePixel().Width()+5+aCtrlPos.X()+
		m_aEstimateText.GetSizePixel().Width(),
		aCtrlPos.Y() + m_aInfoText.GetSizePixel().Height()) );

	Size aESz = m_aEstimate.GetSizePixel();
	if( aESz.Width() + m_aEstimate.GetPosPixel().X() >
		aBmpSize.Width() + DLG_DEF_ADD_WIDTH/2 )
		m_aEstimate.SetSizePixel( Size((aBmpSize.Width()+DLG_DEF_ADD_WIDTH/2)-
			m_aEstimate.GetPosPixel().X(), m_aEstimate.GetSizePixel().Height()) );

	// Text tiefer als Progress?
	if( (m_aEstimateText.GetPosPixel().Y() + m_aEstimateText.GetSizePixel().Height()) >
		(m_aProgress.GetPosPixel().Y() + m_aProgress.GetSizePixel().Height()) )
	{
		m_aFixedLine.SetPosSizePixel( Point(DLG_DEF_ADD_WIDTH/2,
			(m_aEstimateText.GetPosPixel().Y() + m_aEstimateText.GetSizePixel().Height()) + DEF_DISTANCE),
			Size(aBmpSize.Width(), 2) );
	}
	else
	{
		m_aFixedLine.SetPosSizePixel( Point(DLG_DEF_ADD_WIDTH/2,
			m_aProgress.GetPosPixel().Y() + m_aProgress.GetSizePixel().Height() + DEF_DISTANCE),
			Size(aBmpSize.Width(), 2) );
	}

	Size aCSz = m_aCancel.GetSizePixel();
	m_aCancel.SetPosPixel( Point( DLG_DEF_ADD_WIDTH/2 + aBmpSize.Width() -
		aCSz.Width(), m_aFixedLine.GetPosPixel().Y() + DLG_DEF_ADD_TOP +
		m_aFixedLine.GetSizePixel().Height()) );

	SetOutputSizePixel( Size(aBmpSize.Width() + DLG_DEF_ADD_WIDTH,
		 m_aCancel.GetPosPixel().Y() + aCSz.Height() + DLG_DEF_ADD_TOP) );
}

void SetupInstallInfoDialog::CalcSizeForNOSlide()
{
	Size 	aSz( GetOutputSizePixel() );
	Point   aCtrlPos( DLG_DEF_ADD_WIDTH/2, DLG_DEF_ADD_TOP + DEF_DISTANCE );
	Size	aTextHg( Size( 0, m_aEstimateText.GetSizePixel().Height() +
					m_aInfoText.GetSizePixel().Height()) );

	m_aProgress.SetSizePixel( Size(((aSz.Width()-DLG_DEF_ADD_WIDTH)/2)-10, 28) );
	Size aProgSz = m_aProgress.GetOutputSizePixel();
	aProgSz.Width() -= (aProgSz.Width() -4)% (((aProgSz.Height() -4) *2 / 3) + 3);
	m_aProgress.SetOutputSizePixel( aProgSz );
	m_aProgress.SetPosPixel( Point(DLG_DEF_ADD_WIDTH/2, DLG_DEF_ADD_TOP) );

	// Text hoeher als Progress?
	if( aTextHg.Height() > m_aProgress.GetSizePixel().Height() )
	{
		USHORT nDiff = (aTextHg.Height() - m_aProgress.GetSizePixel().Height()) / 2;
		m_aProgress.SetPosPixel( Point(aCtrlPos.X(), aCtrlPos.Y() + nDiff) );
	}
	else
		m_aProgress.SetPosPixel(aCtrlPos);

	USHORT nXPos = m_aProgress.GetSizePixel().Width() + 5 + aCtrlPos.X();
	m_aInfoText.SetPosSizePixel( Point(nXPos,aCtrlPos.Y()),
		Size(aSz.Width()-5-aCtrlPos.X()-nXPos,m_aInfoText.GetSizePixel().Height()));

	m_aEstimateText.SetPosPixel( Point(m_aProgress.GetSizePixel().Width()+5+aCtrlPos.X(),
		aCtrlPos.Y() + m_aInfoText.GetSizePixel().Height()) );

	m_aEstimate.SetPosPixel( Point(m_aProgress.GetSizePixel().Width()+5+aCtrlPos.X()+
		m_aEstimateText.GetSizePixel().Width(),
		aCtrlPos.Y() + m_aInfoText.GetSizePixel().Height()) );

	Size aESz = m_aEstimate.GetSizePixel();
	if( aESz.Width() + m_aEstimate.GetPosPixel().X() >
		aSz.Width() + DLG_DEF_ADD_WIDTH/2 )
		m_aEstimate.SetSizePixel( Size((aSz.Width()+DLG_DEF_ADD_WIDTH/2)-
			m_aEstimate.GetPosPixel().X(), m_aEstimate.GetSizePixel().Height()) );

	// Text tiefer als Progress?
	if( (m_aEstimateText.GetPosPixel().Y() + m_aEstimateText.GetSizePixel().Height()) >
		(m_aProgress.GetPosPixel().Y() + m_aProgress.GetSizePixel().Height()) )
	{
		m_aFixedLine.SetPosSizePixel( Point(DLG_DEF_ADD_WIDTH/2,
			(m_aEstimateText.GetPosPixel().Y() + m_aEstimateText.GetSizePixel().Height()) + DLG_DEF_ADD_TOP),
			Size(aSz.Width(), 2) );
	}
	else
	{
		m_aFixedLine.SetPosSizePixel( Point(DLG_DEF_ADD_WIDTH/2,
			m_aProgress.GetPosPixel().Y() + m_aProgress.GetSizePixel().Height() + DLG_DEF_ADD_TOP),
			Size(aSz.Width(), 2) );
	}

	Size aCSz = m_aCancel.GetSizePixel();
	m_aCancel.SetPosPixel( Point( DLG_DEF_ADD_WIDTH/2 + aSz.Width() -
		aCSz.Width(), m_aFixedLine.GetPosPixel().Y() + DLG_DEF_ADD_TOP +
		m_aFixedLine.GetSizePixel().Height()) );

	SetOutputSizePixel( Size(aSz.Width() + DLG_DEF_ADD_WIDTH,
		 m_aCancel.GetPosPixel().Y() + aCSz.Height() + DLG_DEF_ADD_TOP) );
}

// -----------------------------------------------------------------------

IMPL_LINK( SetupInstallInfoDialog, ExitHdl, Button*, pBtn )
{
	pSetupApp->GetExitHdl().Call(pBtn);
	return 1;
}

// -----------------------------------------------------------------------

void SetupInstallInfoDialog::SetDescText(InstallMode eInstMode)
{
	ByteString aText(m_aProgramName);
	switch (eInstMode)
	{
		case INSTALLMODE_FIRST:
			aText += ByteString( ResId( STR_FIRST_INSTALL ) );
			break;

		case INSTALLMODE_UPDATE:
			aText += ByteString( ResId( STR_UPDATE_INSTALL ) );
			break;

		case INSTALLMODE_RECOVER:
			aText += ByteString( ResId( STR_RECOVER_INSTALL ) );
			break;

		case INSTALLMODE_DEINST:
			aText += ByteString( ResId( STR_DEINST_INSTALL ) );
			break;

		case INSTALLMODE_NETWORK:
			aText += ByteString( ResId( STR_NETWORK_INSTALL ) );
			break;

		case INSTALLMODE_WORKSTAT:
			aText += ByteString( ResId( STR_WORKSTAT_INSTALL ) );
			break;
		default:
			DBG_ERROR( "wrong type" );
	}
	SetText( aText );
}

// -----------------------------------------------------------------------

void SetupInstallInfoDialog::SetInfoText( USHORT nID )
{
	m_aInfoText.SetText( ByteString( ResId( nID ) ) );
}

// -----------------------------------------------------------------------

BOOL SetupInstallInfoDialog::Close()
{
	if( m_aCancel.IsEnabled() )
		return TRUE;
	return FALSE;
}

void SetupInstallInfoDialog::DisableCancel(BOOL bEnable)
{
	m_aCancel.Enable(!bEnable);
}

// -----------------------------------------------------------------------

void SetupInstallInfoDialog::SetInfoText( const ByteString &rStr )
{
	ByteString aStr = rStr;

	// Damit Tilden nicht als Underline dargestellt werden
	aStr.SearchAndReplace("~","~~");
	m_aInfoText.SetText(aStr);
}

// -----------------------------------------------------------------------
void SetupInstallInfoDialog::SetPercent( USHORT nPerc )
{
	USHORT nPercent = Min(nPerc,(USHORT)100);
	m_aProgress.SetValue(nPercent+1);

	International aInter;

	Time   aTime = m_aStopWatch.GetDuration();

	if (nPerc    != 0
	&&  m_lGuess != 0)
	{
		long   lGuess = ((aTime.GetMSFromTime() / 1000) * 100) / nPerc;
		double dDiff  = double(lGuess) / double(m_lGuess);

		if (dDiff > 0.9 && dDiff < 1.0)
		{
			Time aNewTime;

			aNewTime.MakeTimeFromMS(1000*lGuess);
			aNewTime -= aTime;

			if ((aNewTime.GetMSFromTime() / 1000) > 0)
			{
				ByteString sTime( aInter.GetDuration(aNewTime) );
				if (sTime != m_aEstimate.GetText())
				{
					m_aEstimateText.Show();
					m_aEstimate.SetText(sTime);
				}
			}
			else
			{
				m_aEstimateText.Hide();
				m_aEstimate.Hide();
			}
		}

		m_lGuess = lGuess;
	}
}

// -----------------------------------------------------------------------

IMPL_LINK(SetupInstallInfoDialog, ProgressHdl, SiAgenda*, pAgenda)
{
	SetPercent(pAgenda->GetPercentage());
	return 0; // don't care
}

// -----------------------------------------------------------------------

#ifdef MAC
IMPL_LINK_INLINE_START( SetupInstallInfoDialog, StopHdl, Button *, EMPTYARG )
{
	m_bStop = TRUE;
	return 0;
}
IMPL_LINK_INLINE_END( SetupInstallInfoDialog, StopHdl, Button *, EMPTYARG )
#endif

// -----------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////
// Diashow
/////////////////////////////////////////////////////////////////////

SlideWindow::SlideWindow( Window* pParent, const ResId& rResId ) :
	Window( pParent, rResId )
{
	Font aFont = GetFont();
	aFont.SetTransparent(TRUE);
	SetFont(aFont);
	m_aBmp.SetPrefMapMode(MAP_PIXEL);
}

void SlideWindow::SetSlide(ByteString const& rFilename)
{
	// Dia-bmp einlesen
	SvFileStream aSvFileStrm(rFilename.GetBuffer(), STREAM_READ);
	aSvFileStrm >> m_aBmp;
	CalcBmpPos();
	Invalidate();
	Application::Reschedule();
}

void SlideWindow::SetSlide(SiSlide* pSlide, ByteString const& rPath)
{
	// Dia-bmp einlesen
	SiDirEntry aFile  = rPath;
			 aFile += pSlide->GetBmpFileName();

	SvFileStream aSvFileStrm(aFile.GetFull(), STREAM_READ);
	aSvFileStrm >> m_aBmp;
	m_sText = pSlide->GetText();

	// "\n" im Text in Steuerzeichen umwandeln
	while (m_sText.Search("\\n") != STRING_NOTFOUND)
		m_sText.SearchAndReplace("\\n","\n");

	CalcBmpPos();
	m_aTextPos  = Point( 10, 10 );
	m_aTextSize = pSlide->GetTextSize();

	Invalidate();
	Application::Reschedule();
}

void SlideWindow::CalcBmpPos()
{
	SetOutputSizePixel( m_aBmp.GetSizePixel() );

	Size aSz	= GetParent()->GetOutputSizePixel();
	Size aBmpSz	= GetOutputSizePixel();

	m_aBmpPos.X() = aSz.Width()/2 - aBmpSz.Width()/2;
	m_aBmpPos.Y() = DLG_DEF_ADD_TOP;

	SetPosPixel(m_aBmpPos);
}

// -----------------------------------------------------------------------

void SlideWindow::Paint(Rectangle const&)
{
	if (!Application::GetAppWindow()->IsMinimized())
	{
		USHORT nParts = m_sText.GetTokenCount('\n');

		DrawBitmap(Point(0,0), m_aBmp.GetSizePixel(), m_aBmp);

		USHORT nLine   = 0;
		long   nHeight = GetFont().GetSize().Height() + 2;

		// Alle Teilstrings bearbeiten
		for (USHORT i=0; i<nParts; i++)
		{
			ByteString s = m_sText.GetToken(i, '\n'); // Teilstring
			USHORT c = 0;						  // Position im Teil

			while (c < s.Len())
			{
				USHORT j = GetTextBreak(s, m_aTextSize.Width(), c);

				if (j != (USHORT)-1)
				{
					// Teilstring fuer die aktuelle Zeile berechnen
					//
					USHORT nLength = j-c+1;
					USHORT nGoBack = 0;

					// zurueckgehen, wenn Umbruch im Wort
					while (nLength-nGoBack > 0
					   &&  s(j-nGoBack) != ' ')
						nGoBack++;

					// Bis zum Anfang zuruekgegangen
					// -> also nicht zurueckgehen
					if ((nLength - nGoBack) == 0)
						nGoBack = 0;

					// Ausgabe
					DrawText(Point(m_aTextPos.X(),m_aTextPos.Y()+nLine*nHeight), s, c, nLength-nGoBack);
					c = j+1-nGoBack;
					nLine++;
				}
				else // Rest des ByteStrings
				{
					DrawText(Point(m_aTextPos.X(),m_aTextPos.Y()+nLine*nHeight), s, c);
					c = j;
				}
			}

			nLine++;
		}
	}
}

