/*************************************************************************
 *
 *  $RCSfile: instpath.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdlib.h>

#include "instpath.hxx"
#include "event.hxx"

#include "instpath.hrc"
#include "dialog.hrc"
#include "strings.hrc"

#include "environ.hxx"
#include "decltor.hxx"
#include "os.hxx"

#include <tlfsys.hxx>
#include <ctype.h>

#ifdef VCL
#include <svtools/filedlg.hxx>
#endif

// SetupDriveBox ---------------------------------------------------------

void SetupDriveBox::KeyInput( const KeyEvent& rKEvt )
{
	const KeyCode aCode = rKEvt.GetKeyCode();

	if ( aCode.GetModifier() || aCode.GetGroup() != KEYGROUP_ALPHA )
	{
		ListBox::KeyInput( rKEvt );
		return;
	}
	char cKey = rKEvt.GetCharCode();

	for ( USHORT i = 0; i < GetEntryCount(); ++i )
	{
		if ( GetEntry(i).Lower()(1) == cKey )
		{
			SelectEntryPos(i);
			Select();
			break;
		}
	}
}

// class SetupInstallPathDialog ------------------------------------------

SetupInstallPathDialog::SetupInstallPathDialog
(
	Window		  * pParent,
	SiEnvironment * pEnv,
	ByteString const&   aProgramName,
	SiModule const* pModule,
	SiInstallMode	eInstallMode,
	BOOL		    bScanRemoveableDisks
)
: ModalDialog( pParent, ResId( RID_INSTALLPATH_DLG ) )
#if defined(WIN) || defined(OS2) || defined(WNT)
, m_aDriveText	( this, ResId( FT_INST_DRIVE ) )
, m_aDriveBox	( this, ResId( LB_INST_DRIVE ) )
, m_cSelDrive   ('\0')
#endif
, m_aPathText	( this, ResId( FT_INST_PATH ) )
, m_aPathEdit	( this, ResId( ED_INST_PATH ) )
, m_aBrowseBtn	( this, ResId( BTN_INSTPATH_BROWSE ) )
, m_aOKBtn		( this, ResId( BTN_INSTPATH_OK ) )
, m_aCancelBtn	( this, ResId( BTN_INSTPATH_CANCEL ) )
#ifndef VCL
, m_aIcon		( this, ResId( IC_INSTPATH ) )
#endif
, m_aInfo1Text	( this, ResId( FT_INFO1_INSTPATH ) )
, m_aInfo2Text	( this, ResId( FT_INFO2_INSTPATH ) )
, m_aInfo3Text	( this, ResId( FT_INFO3_INSTPATH ) )
, m_aSize1Text	( this, ResId( FT_REQUIRED_MEM ) )
, m_aSize2Text	( this, ResId( FT_REQUIRED_MEMSIZE ) )
, m_aTemp1Text	( this, ResId( FT_TEMP_MEM ) )
, m_aTemp2Text	( this, ResId( FT_TEMP_MEMSIZE ) )
, m_aSystem1Text( this, ResId( FT_SYSTEM_MEM ) )
, m_aSystem2Text( this, ResId( FT_SYSTEM_MEMSIZE ) )
, m_pEnv        (pEnv  )
, m_pModule		(pModule)
, m_lUsedSize	(0)
, m_lTempSize	(0)
, m_lSystemSize (0)
, m_lClusterSize(0)
, m_lClusterSizeSystem(0)
, m_eInstallMode(eInstallMode)
, m_bScanRemoveableDisks (bScanRemoveableDisks)
{
	FreeResource();

	m_aBrowseBtn.SetClickHdl( LINK( this, SetupInstallPathDialog, BrowseHdl ) );
	m_aOKBtn.SetClickHdl( LINK( this, SetupInstallPathDialog, OKHdl ) );
	m_aCancelBtn.SetClickHdl( LINK( this, SetupInstallPathDialog, CancelHdl ) );
#if defined(WIN) || defined(OS2) || defined(WNT)
	m_aDriveBox.SetSelectHdl( LINK( this, SetupInstallPathDialog, SelectHdl ) );
#endif

	SiDirEntry aEntry(OS::GetGUIPath());
	ByteString sSys(m_aSystem1Text.GetText());
	#if defined(MAC)
	sSys.SearchAndReplace( "%:", aEntry.GetVolume() );
	#else
	sSys.SearchAndReplace( "%:", aEntry.GetDevice().GetFull() );
	#endif
	m_aSystem1Text.SetText( sSys );

	// set pathname edit control
	#ifdef UNX
	m_aSystem1Text.Hide();
	m_aSystem2Text.Hide();

	SiDirEntry aUNXDefPath( UnixOS::GetHomeDir() );
	aUNXDefPath += m_pEnv->GetDestPath();
	aUNXDefPath.ToAbs();
	ByteString aText = aUNXDefPath.GetFull();
	m_pEnv->SetDestPath( aText );
	#else
	ByteString aText = m_pEnv->GetDestPath();
	#endif

#if defined(WIN) || defined(OS2) || defined(WNT)

	if (aText.Len() > 0)
		m_cSelDrive = aText[(USHORT)0];

	if (aText.Len() == 3)
		aText = "";
	else
		aText.Erase( 0, 3 );
#endif

	m_aPathEdit.SetText( aText );
	UpdateDrives();
	aText = aProgramName;
	aText += ByteString( ResId( STR_DLG_TITLE ) );
	SetText( aText );

	/** UI - Hack FixedTexte zusammenfassen **/
	m_aInfo2Text.Hide();
	m_aInfo3Text.Hide();
	ByteString aStr = m_aInfo1Text.GetText();
	aStr += " ";
	aStr += m_aInfo2Text.GetText();
	aStr += " ";
	aStr += m_aInfo3Text.GetText();
	m_aInfo1Text.SetText(aStr);
	/** Ende UI - Hack **/

	ULONG lCluster = OS::GetClusterSize(SiDirEntry(OS::GetGUIPath()));
	m_lSystemSize = m_pModule->CalculateSize(SiModule::INSTALL, lCluster, TRUE,  m_eInstallMode == IM_WORKSTATION);
	m_lTempSize   = m_pModule->CalculateSize(SiModule::TEMP,    lCluster, FALSE, m_eInstallMode == IM_WORKSTATION);

	RecalcSize(SiDirEntry(m_pEnv->GetDestPath()));
	UpdateSize();
}

void SetupInstallPathDialog::RecalcSize(SiDirEntry const& rDest)
{
	ULONG lClusterSize = OS::GetClusterSize(rDest);
	if(lClusterSize != m_lClusterSize)
	{
		m_lClusterSize = lClusterSize;
		m_lUsedSize    = m_pModule->CalculateSize(SiModule::INSTALL, lClusterSize, FALSE, m_eInstallMode == IM_WORKSTATION);
	}

	ULONG lClusterSizeSys = OS::GetClusterSize(SiDirEntry(OS::GetGUIPath()));
	if( lClusterSizeSys != m_lClusterSizeSystem )
	{
		m_lClusterSizeSystem = lClusterSizeSys;
		m_lSystemSize = m_pModule->CalculateSize(SiModule::INSTALL, lClusterSizeSys, TRUE, m_eInstallMode == IM_WORKSTATION);
	}
}

// -----------------------------------------------------------------------

IMPL_LINK( SetupInstallPathDialog, BrowseHdl, Button*, pBtn )
{
	SiDirEntry aPath;

	#if defined(WIN) || defined(OS2) || defined(WNT)
	// Laufwerk z.B.: "C:"
	ByteString sDrive(m_aDriveBox.GetSelectEntry()( 1, 2 ) );
	sDrive += '\\'; // damit absolut
	aPath = SiDirEntry( sDrive );

	char* pTmp = getenv( "temp" );
	if( !pTmp ) pTmp = getenv( "tmp" );
	if( pTmp )
	{
		ByteString aTmp( pTmp[0] );
		aTmp += ":\\";
		SiDirEntry aEntry(aTmp);
		aEntry.SetCWD();
	}
	#endif

	// + Pfad
	aPath += SiDirEntry(m_aPathEdit.GetText() );
	aPath.ToAbs();
	if( !aPath.Exists() )
	{
		aPath = aPath.GetDevice();
		aPath.ToAbs();
	}

	PathDialog aDlg( this, WB_SVLOOK );
	aDlg.SetPath( aPath.GetFull() );

	if ( aDlg.Execute() == RET_OK )
	{
		ByteString sPath( aDlg.GetPath());

		#if defined(WIN) || defined(OS2) || defined(WNT)

		char cDrive = sPath.Lower()(0);

		for ( USHORT i = 0; i < m_aDriveBox.GetEntryCount(); i++ )
		{
			if ((char)m_aDriveBox.GetEntry(i).Lower()(1) == cDrive)
				m_aDriveBox.SelectEntryPos(i);
		}
		// Laufwerksbuchstabe abschneiden
		m_aPathEdit.SetText( sPath.Copy( 3 ) );
		#else
		m_aPathEdit.SetText( sPath );
		#endif
	}
	return 1;
}

// -----------------------------------------------------------------------

IMPL_LINK( SetupInstallPathDialog, SelectHdl, SetupDriveBox*, pDriveBox)
{
#if defined(WIN) || defined(OS2) || defined (WNT)
	m_cSelDrive = (char)m_aDriveBox.GetSelectEntry()(1,1);

	ByteString aDest  = m_cSelDrive;
		   aDest += ":\\";

	RecalcSize(SiDirEntry(aDest));
	UpdateSize();
#endif
	return 1;
}

// -----------------------------------------------------------------------

IMPL_LINK( SetupInstallPathDialog, OKHdl, Button*, pBtn )
{
	// Sicherheitsabfrage, dass nicht versehentlich in das
	// Wurzelverzeichnis installiert wird.
	//
	if (m_aPathEdit.GetText().Len() == 0)
	{
		QueryBox aBox(this, WB_YES_NO, ByteString(ResId(STR_INSTALL_ON_ROOT)));

		if (aBox.Execute() != RET_YES)
			return 1;
	}

	SiDirEntry aPath;
	SiDirEntry aRelPath = m_aPathEdit.GetText();

	#if defined(WIN) || defined(OS2) || defined(WNT)
	// Laufwerk z.B.: "C:"
	aPath = SiDirEntry( m_aDriveBox.GetSelectEntry()( 1, 2 ) );
	#endif

	// Pfad
	aPath += aRelPath;

	ByteString aPathStr( aPath.GetFull() );

	#if defined(WIN) || defined(OS2) || defined(WNT)

	if (aPathStr(2) != '\\' )
	{
		aPathStr.Insert( '\\', 2 );
		aPath = SiDirEntry( aPathStr );
	}

	#endif

	if (!aPath.IsValid()
	||  !aRelPath.IsValid()
	||   aPathStr.Search('?') != STRING_NOTFOUND
	||   aPathStr.Search('*') != STRING_NOTFOUND)
	{
		ByteString aMessage = ResId(STR_WRONGPATH);
			   aMessage.SearchAndReplace("%1", m_aPathEdit.GetText());

		ErrorBox(this, WB_OK, aMessage).Execute();
		return 1;
	}

	RecalcSize(aPath);

	// Genug Platz?
	ByteString aDestVol(aPath.GetFull() ); aDestVol.ToLower();
	ByteString aGUIVol (OS::GetGUIPath() ); aGUIVol.ToLower();
	BOOL   bOK = TRUE;

	if (aDestVol(0) != aGUIVol(0) && m_eInstallMode != IM_NETWORK)
	{
		// Ziellaufwerk != Systemlaufwerk
		// deshalb Pruefung aufteilen
		//
		ULONG nUsedSize = (m_lUsedSize + m_lTempSize) - m_lSystemSize;

		if (OS::GetDriveSize(aPath) < (nUsedSize/1024))
		{
			InfoBox(this, ResId(DLG_FULLDRIVEBOX1)).Execute();
			bOK = FALSE;
		}
		else if (OS::GetDriveSize(SiDirEntry(OS::GetSystemPath())) < (m_lSystemSize/1024))
		{
			ByteString aInfo( ResId( STR_FULLSYSTEMDRIVE1 ) );
			aInfo += OS::GetSystemPath().Upper().Copy( 0, 2 );
			aInfo += ByteString( ResId( STR_FULLSYSTEMDRIVE2 ) );
			aInfo += m_lSystemSize / 1024;	// in KB
			aInfo += ByteString( ResId( STR_FULLSYSTEMDRIVE3 ) );
			InfoBox( this, aInfo ).Execute();
			EndDialog(RET_CANCEL);
			bOK = FALSE;
		}
	}
	else if (OS::GetDriveSize(aPath) < ((m_lUsedSize + m_lTempSize) /1024))
	{
		InfoBox( this, ResId( DLG_FULLDRIVEBOX1 ) ).Execute();
		bOK = FALSE;
	}

	// im BigMode muss auf dem TEMP-Laufwerk genug Platz fuer die groesste
	// Datei im Satz sein! m_lTempSize ist die Size der groessten Hilfedatei,
	// da es auch immer die groesste Datei im Satz seinen sollte wird diese
	// Size hier verwendet!
	if( m_pEnv->IsBigMode() )
	{
		SiDirEntry aFoo( SiDirEntry("fo*") );
		SiDirEntry aTmp( aFoo.TempName().GetPath() );
		aTmp.ToAbs();
		if( OS::GetDriveSize(aTmp) < (m_lTempSize /1024) )
		{
			InfoBox aBox(this, ResId(DLG_FULLTEMPDRIVE));
			ByteString aText( aBox.GetMessText() );
			aText.SearchAndReplace( "%1", aTmp.GetFull().Upper().Copy(0, 2) );
			aBox.SetMessText( aText );
			aBox.Execute();
			bOK = FALSE;
		}
	}

	if( bOK )
	{
		m_pEnv->SetDestPath( aPath.GetFull() );
		EndDialog(RET_OK);
	}
	else
	{
		// set pathname edit control
		/* Verhalten nicht mehr gewnscht! (#44397#)

		ByteString aText = m_pEnv->GetDestPath();
		#if defined(WIN) || defined(OS2) || defined(WNT)
		m_cSelDrive = aText[(USHORT)0];

		if ( aText.Len() == 3 )
			aText = "";
		else
			aText.Erase( 0, 3 );
		#endif
		m_aPathEdit.SetText(aText);
		*/
	}

	return 1;
}

// -----------------------------------------------------------------------

IMPL_LINK( SetupInstallPathDialog, CancelHdl, Button*, pBtn )
{
	EndDialog(RET_CANCEL);
	return 1;
}

// -----------------------------------------------------------------------

void SetupInstallPathDialog::Activate()
{
	UpdateDrives();
}

// -----------------------------------------------------------------------

void SetupInstallPathDialog::UpdateSize()
{
	// set used size
	//
	ByteString aKBStr( ResId( STR_KB ) );
	ByteString aText = m_lUsedSize / 1024;

	( aText += ' ' ) += aKBStr;
	m_aSize2Text.SetText(aText);

	aText = (m_lUsedSize + m_lTempSize) / 1024;
	( aText += ' ' ) += aKBStr;
	m_aTemp2Text.SetText(aText);

	aText = m_lSystemSize / 1024;
	( aText += ' ' ) += aKBStr;
	m_aSystem2Text.SetText( aText );
}

void SetupInstallPathDialog::UpdateDrives()
{
#if defined(WIN) || defined(OS2) || defined(WNT)

	// insert drives into listbox
	Application::EnterWait();

	m_aDriveBox.Clear();

	BOOL	bFound = FALSE;
	USHORT	nDriveCnt = 0;
	USHORT	nFirstFreeDrive = 0; // erstes Laufwerk mit ausreichend Platz
	Dir		aDir( SiDirEntry( "*" ), FSYS_KIND_BLOCK);

	for (USHORT i = 0; i < aDir.Count(); i++)
	{
		ByteString sFull = aDir[i].GetFull();
		sFull.ToLower();

		if ((sFull(0) == 'a' || sFull(0) == 'b' )
		&&  !m_bScanRemoveableDisks)
		{
			// kein FileStat erzeugen,
			// damit nicht auf die Floppy zugegriffen wird
			continue;
		}

		FileStat aStat(aDir[i]);

		if (sFull(0) != 'a'
		&&  sFull(0) != 'b'
		&&	( aStat.IsKind (FSYS_KIND_FIXED)  ||
			  aStat.IsKind (FSYS_KIND_REMOTE) ||
			 (aStat.IsKind (FSYS_KIND_REMOVEABLE) && m_bScanRemoveableDisks)))
		{
			char cDrive = (char)aDir[i].GetFull();

			if (!bFound
			||   tolower(m_cSelDrive) == tolower(cDrive))
			{
				RecalcSize(aDir[i]);

				if (OS::GetDriveSize(aDir[i]) > (m_lUsedSize/1024))
				{
					bFound		    = TRUE;
					nFirstFreeDrive = nDriveCnt;
				}
			}
			nDriveCnt++;

			Bitmap aBitmap;

			if ( aStat.IsKind( FSYS_KIND_FIXED ) )
				aBitmap = Bitmap( ResId( BMP_FIXED  ) );
			else if ( aStat.IsKind( FSYS_KIND_REMOTE ) )
				aBitmap = Bitmap( ResId( BMP_REMOTE ) );
			else if ( aStat.IsKind( FSYS_KIND_REMOVEABLE ) )
				aBitmap = Bitmap( ResId( BMP_FLOPPY ) );

			ByteString aDriveStr(aDir[i].GetFull());
			ByteString aEntryStr(" ");

			aEntryStr += aDriveStr.Upper();
			aEntryStr += " ";
			aEntryStr += ByteString(ResId(STR_FREESIZE));
			aEntryStr += " ";
			aEntryStr += ByteString(OS::GetDriveSize( aDir[ i ] ));
			aEntryStr += ' ';
			aEntryStr += ByteString( ResId( STR_KB ) );
			aEntryStr += ' ';
			m_aDriveBox.InsertEntry( aEntryStr, aBitmap );
		}
	}
	Application::LeaveWait();

	if (bFound)
		m_aDriveBox.SelectEntryPos(nFirstFreeDrive);
	else
		RecalcSize(SiDirEntry(m_pEnv->GetDestPath()));

	UpdateSize();
#endif
}


