/*************************************************************************
 *
 *  $RCSfile: options.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "options.hxx"

#include "dialog.hrc"
#include "options.hrc"

#include "modinfo.hxx"
#include "instpath.hxx"
#include "environ.hxx"
#include "decltor.hxx"
#include "modinfo.hxx"
#include "os.hxx"
#include "simode.hxx"

#include <tlfsys.hxx>

// class SetupOptionsDialog ----------------------------------------------

SetupOptionsDialog::SetupOptionsDialog
(
	Window        * pParent,
	ByteString   const& aProgramName,
	SiEnvironment * pEnv,
	SiModule      * pModule,
	SiInstallMode	eInstallMode,
	BOOL            bMin,
	BOOL		    bScanRemoveableDisks,
	BOOL			bDisablePath,
	USHORT			nOkEvent,
	USHORT			nCancelEvent
)
: ModelessDialog(pParent, ResId(RID_OPTIONS_DLG))
#ifndef VCL
, m_aIcon           (this, ResId(1))
#endif
, m_aInfo1Text      (this, ResId(FT_INFO1))
, m_aInfo2Text      (this, ResId(FT_INFO2))
, m_aInfo3Text      (this, ResId(FT_INFO3))
, m_aOptionText     (this, ResId(FT_OPTION))
, m_aModuleView     (this, ResId(LB_OPTION))
, m_aDescText       (this, ResId(FT_DESC))
, m_aDescBox        (this, ResId(GB_DESC))
, m_aDescInfoText   (this, ResId(FT_DESCINFO))
, m_aPathText       (this, ResId(FT_PATH))
, m_aPathBtn        (this, ResId(BTN_PATH))
, m_aWorkPathText   (this, ResId(FT_WORKPATH))
, m_aSysPathText    (this, ResId(FT_SYSTEMPATH))
, m_aRequire1Text   (this, ResId(FT_REQUIRE1))
, m_aRequire2Text   (this, ResId(FT_REQUIRE2))
, m_aRequire3Text   (this, ResId(FT_REQUIRE3))
, m_aTemp1Text      (this, ResId(FT_TEMP1))
, m_aTemp2Text      (this, ResId(FT_TEMP2))
, m_aTemp3Text      (this, ResId(FT_TEMP3))
, m_aExist1Text     (this, ResId(FT_EXIST1))
, m_aExist2Text     (this, ResId(FT_EXIST2))
, m_aExist3Text     (this, ResId(FT_EXIST3))
, m_aMemBox         (this, ResId(GB_MEM))
, m_aInstallBtn     (this, ResId(BTN_OPTIONS_OK))
, m_aCancelBtn      (this, ResId(BTN_OPTIONS_CANCEL))
, m_bMinimal        (bMin)
, m_eInstallMode    (eInstallMode)
, m_lClusterSize	(0)
, m_lSysClusterSize	(0)
, m_aWorkPathStr    (m_aWorkPathText.GetText())
, m_aSysPathStr     (m_aSysPathText.GetText())
, m_aProgramName    (aProgramName)
, m_pModule         (pModule)
, m_pEnv			(pEnv)
, m_bScanRemoveableDisks(bScanRemoveableDisks)
, m_nOkEvent		(nOkEvent)
, m_nCancelEvent	(nCancelEvent)
{
#ifdef MAC

	m_aModuleView.SetWindowBits(
		WB_HASBUTTONS | WB_HASBUTTONSATROOT);

#else

	m_aModuleView.SetWindowBits(
		WB_HASBUTTONS | WB_HASBUTTONSATROOT
	  | WB_HASLINES   | WB_HASLINESATROOT);

#endif

	m_lClusterSize	  = OS::GetClusterSize(SiDirEntry(m_pEnv->GetDestPath()));
	m_lSysClusterSize = OS::GetClusterSize(SiDirEntry(OS::GetGUIPath()));

	#ifdef UNX
	SiDirEntry aHome(UnixOS::GetHomeDir());
	aHome += m_pEnv->GetDestPath();
	aHome.ToAbs();
	if( aHome.GetFull() != m_pEnv->GetDestPath() )
		m_pEnv->SetDestPath(aHome.GetFull());
	#endif

#ifndef MAC

	if (m_pEnv->IsFirstInstallation())
	{
		ULONG nUsedSize = m_pModule->CalculateSize(SiModule::INSTALL, m_lClusterSize, FALSE, m_eInstallMode == IM_WORKSTATION)
						+ m_pModule->CalculateSize(SiModule::TEMP,	  m_lClusterSize, FALSE, m_eInstallMode == IM_WORKSTATION);

		if (OS::GetDriveSize(SiDirEntry(m_pEnv->GetDestPath())) < (nUsedSize / 1024))
		{
			// suche ein Laufwerk, auf dem genug Platz ist
			ByteString aPathStr = m_pEnv->GetDestPath();

			BOOL  bFound    = FALSE;
			Dir   aDir( SiDirEntry( "*" ), FSYS_KIND_BLOCK );

			for ( USHORT i = 0; !bFound && i < aDir.Count(); i++ )
			{
				ByteString sFull = aDir[i].GetFull();
				sFull.ToLower();

				if ((sFull(0) == 'a' || sFull(0) == 'b')
				&& !m_bScanRemoveableDisks)
					// kein FileStat erzeugen,
					// damit nicht auf die Floppy zugegriffen wird
					continue;

				FileStat aStat( aDir[i] );

				if ((sFull(0) != 'a' && sFull(0) != 'b' )
				&&  (	aStat.IsKind( FSYS_KIND_FIXED )
					||  aStat.IsKind( FSYS_KIND_REMOTE)
					||  (  m_bScanRemoveableDisks
						&& aStat.IsKind(FSYS_KIND_REMOVEABLE))
					)
				&& OS::GetDriveSize(aDir[i]) > (nUsedSize / 1024))
				{
					#ifndef UNX
					ByteString aTmp = aDir[i].GetFull();
					// aTmp += "\\";
					aPathStr.Erase( 0, 2 );
					aPathStr.Insert( aTmp, 0 );
					m_pEnv->SetDestPath(aPathStr);
					#endif
					bFound = TRUE;
					m_lClusterSize = OS::GetClusterSize(SiDirEntry(m_pEnv->GetDestPath()));
				}
			}
		}
	}
#else
	// Font der Beschreibung aendern
	Font aNewFont(m_aDescInfoText.GetFont());
	Size aNewSize = aNewFont.GetSize();
	aNewSize.Height() = 12;
	aNewFont.SetSize( aNewSize );
	m_aDescInfoText.SetFont( aNewFont );

#endif

	// Verzeichnis und Speicherbedarf updaten
	UpdatePath();
	UpdateUsedSize();
	UpdateFreeSize();

	// Handler setzen
	m_aInstallBtn.SetClickHdl( LINK( this, SetupOptionsDialog, InstallHdl ) );
	m_aCancelBtn.SetClickHdl( LINK( this, SetupOptionsDialog, CancelHdl ) );
	m_aPathBtn.SetClickHdl( LINK( this, SetupOptionsDialog, PathHdl ) );
	m_aModuleView.SetSelectHdl( LINK( this, SetupOptionsDialog, SelectHdl ) );
	m_aModuleView.SetCheckButtonHdl( LINK( this, SetupOptionsDialog, CheckHdl ) );

	if (!m_pEnv->IsFirstInstallation()
	||  bDisablePath)
		m_aPathBtn.Disable();

	if (m_bMinimal)
	{
		ByteString sInfo1( ByteString( ResId( STR_MIN_INFO1) ) );
		sInfo1.SearchAndReplace( "%s", m_aProgramName);
		m_aInfo1Text.SetText( sInfo1 );
		m_aInfo2Text.SetText( ByteString( ResId( STR_MIN_INFO2) ) );
		m_aInfo3Text.SetText( ByteString() );
	}
	else
	{
		// Diese Sachen muessen eigentlich in den Ressourcen
		// geaendert werden.
		m_aInfo2Text.Hide();
		m_aInfo3Text.Hide();
	}

	// Optionen fuellen
	FillOptionBox();
	// aModuleView.SelectEntryPos( 0 );
	// SelectHdl( &aModuleView );
	FreeResource();

	ByteString aText(m_aProgramName);
	aText += ByteString(ResId(STR_DLG_TITLE));
	SetText(aText);
}

// -----------------------------------------------------------------------

SetupOptionsDialog::~SetupOptionsDialog()
{
}

// -----------------------------------------------------------------------

void SetupOptionsDialog::UpdatePath()
{
#if ! (defined MAC || defined UNX)

	ByteString sDest = m_pEnv->GetDestPath().Upper();
	ByteString sGUI  = OS::GetGUIPath().Upper();
	ByteString sWork = m_aWorkPathStr;

	sWork.SearchAndReplace( '%', sDest(0) );
	m_aWorkPathText.SetText( sWork );

	ByteString sSys = m_aSysPathStr;
	sSys.SearchAndReplace( '%', sGUI(0) );
	m_aSysPathText.SetText( sSys );

	SiDirEntry sDestEntry(m_pEnv->GetDestPath());
	m_aPathText.SetText(sDestEntry.GetFull());

#else

	ByteString sWork = m_aWorkPathStr;
	ByteString sDest = m_pEnv->GetDestPath();

	if (sDest.Len())
	{
		SiDirEntry aTmp( sDest );
		sWork.SearchAndReplace( "%:", aTmp.GetVolume() );
		#ifdef UNX
			sWork.SearchAndReplace( "%!", aTmp.GetDevice().GetFull() );
		#endif
		m_aWorkPathText.SetText( sWork );
		m_aPathText.SetText( aTmp.GetFull(FSYS_STYLE_HOST,FALSE,40) );
	}
	else
	{
		sWork.SearchAndReplace( "(%:)", ByteString() );
		m_aWorkPathText.SetText( sWork );
		m_aPathText.SetText(ByteString());
	}

#endif
}

// -----------------------------------------------------------------------

void SetupOptionsDialog::UpdateUsedSize()
{
	BOOL   bWorkstation		= m_eInstallMode == IM_WORKSTATION;
	ULONG  nSize			= m_pModule->CalculateSize(SiModule::INSTALL,	m_lClusterSize, FALSE, bWorkstation);
	ULONG  nSizeDeinst		= m_pModule->CalculateSize(SiModule::UNINSTALL, m_lClusterSize, FALSE, bWorkstation);
	ULONG  nSysSize			= m_pModule->CalculateSize(SiModule::INSTALL,	m_lSysClusterSize, TRUE /*SYSTEM*/, bWorkstation);
	ULONG  nSysSizeDeinst	= m_pModule->CalculateSize(SiModule::UNINSTALL, m_lSysClusterSize, TRUE /*SYSTEM*/, bWorkstation);
	ULONG  nTmpSize			= m_pModule->CalculateSize(SiModule::TEMP,		m_lClusterSize);

	if (nSize >= nSizeDeinst)
		nSize -= nSizeDeinst;
	else
		nSize = 0;

	if (nSysSize >= nSysSizeDeinst)
		nSysSize -= nSysSizeDeinst;
	else
		nSysSize = 0;

	ByteString aKBStr( ResId( STR_KB ) );
	ByteString aText;
	ULONG  nSizeSubSys		= (nSize >= nSysSize) ? nSize - nSysSize : 0;
	BOOL   bTargetIsSystem	= SiDirEntry(m_pEnv->GetDestPath()).GetDevice()
								== SiDirEntry(OS::GetGUIPath()).GetDevice();

	// benoetigter Speicher: Ziellaufwerk
	aText  = (bTargetIsSystem ? nSize : nSizeSubSys) / 1024;
	( ( aText += ' ' ) += aKBStr ) += ' ';
	m_aRequire2Text.SetText(aText);

	// temporaerer Speicher: Ziellaufwerk
	aText  = (bTargetIsSystem ? (nSize + nTmpSize) : nSizeSubSys + nTmpSize) / 1024;
	( ( aText += ' ' ) += aKBStr ) += ' ';
	m_aTemp2Text.SetText(aText);

	// benoetigter Speicher: Systemlaufwerk
	aText  = (bTargetIsSystem ? nSize : nSysSize) / 1024;
	( ( aText += ' ' ) += aKBStr ) += ' ';
	m_aRequire3Text.SetText(aText);

	// temporaerer Speicher: Systemlaufwerk
	aText  = (bTargetIsSystem ? (nSize + nTmpSize) : nSysSize) / 1024;
	( ( aText += ' ' ) += aKBStr ) += ' ';
	m_aTemp3Text.SetText( aText );
}

// -----------------------------------------------------------------------

void SetupOptionsDialog::UpdateFreeSize()
{
	ByteString aKBStr( ResId( STR_KB ) );
	ByteString aText = OS::GetDriveSize(SiDirEntry(m_pEnv->GetDestPath()));

	( ( aText += ' ' ) += aKBStr ) += ' ';
	m_aExist2Text.SetText( aText );

	aText = OS::GetDriveSize(SiDirEntry(OS::GetGUIPath()));
	( ( aText += ' ' ) += aKBStr ) += ' ';
	m_aExist3Text.SetText( aText );
}

// -----------------------------------------------------------------------

BOOL SetupOptionsDialog::Close()
{
	Application::PostUserEvent(m_nCancelEvent);
	return ModelessDialog::Close();
}

// -----------------------------------------------------------------------

void SetupOptionsDialog::Activate()
{
	UpdateFreeSize();
}

// -----------------------------------------------------------------------

int SetupOptionsDialog::GetMinModuleCount(SiModule const* pModule)
// Gibt die Anzahl der installierten/ zu installierenden
// Minimalmodule zurueck.
{
	SiModuleList const* pML  = pModule->GetModuleList();
	int	nCount = 0;

	for (USHORT i=0; i<pML->Count(); i++)
		nCount += GetMinModuleCount(pML->GetObject(i));

	if (!pModule->IsMinimal())
		return nCount;

	if (pModule->IsInstalled())
		return pModule->IsSelected() ? nCount	  : nCount + 1;
	else
		return pModule->IsSelected() ? nCount + 1 : nCount;
}

ByteString SetupOptionsDialog::GetMinModuleNames(SiModule const* pModule)
{
	SiModuleList const* pML   = pModule->GetModuleList();
	ByteString				aList;

	if (pModule->IsMinimal())
		aList = pModule->GetName();

	for (USHORT i=0; i<pML->Count(); i++)
	{
		ByteString aSubList = GetMinModuleNames(pML->GetObject(i));

		if (aSubList.Len() != 0)
		{
			if (aList.Len() != 0)
				aList += ", ";

			aList += aSubList;
		}
	}

	return aList;
}

BOOL SetupOptionsDialog::CheckMinInstall(SiModule const* pModule)
{
	ByteString aMinList = GetMinModuleNames(pModule);

	if (aMinList.Len() == 0)
		return TRUE;	// Minimalmodule gibt es gar nicht!

	return GetMinModuleCount(pModule) > 0;
}

IMPL_LINK( SetupOptionsDialog, InstallHdl, Button *, EMPTYARG )
{
	// Pruefen, ob mindestens ein Minimal-Modul zur Installation
	// ausgewaehlt wurde.
	//
	if (!CheckMinInstall(m_pModule))
	{
		ByteString aMinList  = ResId(STR_INSTALL_ATLEAST);
			   aMinList += GetMinModuleNames(m_pModule);

		InfoBox(this, aMinList).Execute();
		return 0;
	}

	// Platzbedarf pruefen
	//
	BOOL   bWorkstation		= m_eInstallMode == IM_WORKSTATION;
	ULONG  nSize			= m_pModule->CalculateSize(SiModule::INSTALL,	m_lClusterSize, FALSE, bWorkstation);
	ULONG  nSizeDeinst		= m_pModule->CalculateSize(SiModule::UNINSTALL, m_lClusterSize, FALSE, bWorkstation);
	ULONG  nSysSize			= m_pModule->CalculateSize(SiModule::INSTALL,	m_lClusterSize, TRUE /*SYSTEM*/, bWorkstation);
	ULONG  nSysSizeDeinst	= m_pModule->CalculateSize(SiModule::UNINSTALL, m_lClusterSize, TRUE /*SYSTEM*/, bWorkstation);
	ULONG  nTempSize		= m_pModule->CalculateSize(SiModule::TEMP,		m_lClusterSize);

	if (nSize >= nSizeDeinst)
		nSize -= nSizeDeinst;
	else
		nSize = 0;

	if (nSysSize >= nSysSizeDeinst)
		nSysSize -= nSysSizeDeinst;
	else
		nSysSize = 0;

	ULONG  nSizeSubSys = (nSize >= nSysSize) ? nSize - nSysSize : 0;

#ifdef MAC

	if (m_pEnv->IsFirstInstallation())
	{
		SiDirEntry aPath(SiDirEntry(m_pEnv->GetDestPath()).GetPath());

		if (OS::GetDriveSize(aPath) < ((nSize + nTempSize)/ 1024))
		{
			InfoBox( this, ResId( DLG_FULLDRIVEBOX1 ) ).Execute();
			return 0;
		}
		else
		{
			ByteString sVol  = aPath.GetVolume();
			ByteString sPath = aPath.GetName();

			if (sPath.Len() > 0
			&&  sPath[(USHORT)(sPath.Len()-1)] == ':')	// Volumename
				sPath = m_aProgramName;
			else
			{
				sPath += ':';
				sPath += m_aProgramName;
			}
		}
	}

#else

	// Genug Platz?
	ByteString aDestVol = m_pEnv->GetDestPath();
	ByteString aGUIVol  = OS::GetGUIPath();

	aDestVol.ToLower();
	aGUIVol.ToLower();

	if (aDestVol(0) != aGUIVol(0) && m_eInstallMode != IM_NETWORK)
	{
		// Ziellaufwerk != Systemlaufwerk
		// deshalb Pruefung aufteilen
		//
		ULONG nUsedSize = nSizeSubSys + nTempSize;

		if (OS::GetDriveSize(SiDirEntry(m_pEnv->GetDestPath())) < (nUsedSize / 1024))
		{
			InfoBox( this, ResId( DLG_FULLDRIVEBOX2 )).Execute();
			return 0;
		}

		if (OS::GetDriveSize(SiDirEntry(OS::GetSystemPath())) < (nSysSize / 1024))
		{
			ByteString aInfo( ResId( STR_FULLSYSTEMDRIVE1 ) );
			aInfo += OS::GetSystemPath().Upper().Copy( 0, 2 );
			aInfo += ByteString(ResId( STR_FULLSYSTEMDRIVE2));
			aInfo += nSysSize / 1024;   // in KB
			aInfo += ByteString( ResId( STR_FULLSYSTEMDRIVE3 ) );
			InfoBox( this, aInfo ).Execute();
			return 0;
		}
	}
	else if (OS::GetDriveSize(SiDirEntry(m_pEnv->GetDestPath()))
				< ((nSize + nTempSize) / 1024))
	{
		InfoBox( this, ResId( DLG_FULLDRIVEBOX2 ) ).Execute();
		return 0;
	}

	// im BigMode muss auf dem TEMP-Laufwerk genug Platz fuer die groesste
	// Datei im Satz sein! m_lTempSize ist die Size der groessten Hilfedatei,
	// da es auch immer die groesste Datei im Satz seinen sollte wird diese
	// Size hier verwendet!
	if( m_pEnv->IsBigMode() )
	{
		SiDirEntry aFoo( SiDirEntry("fo*") );
		SiDirEntry aTmp( aFoo.TempName().GetPath() );
		aTmp.ToAbs();
		if( OS::GetDriveSize(aTmp) < (nTempSize /1024) )
		{
			InfoBox aBox(this, ResId(DLG_FULLTEMPDRIVE));
			ByteString aText( aBox.GetMessText() );
			aText.SearchAndReplace( "%1", aTmp.GetFull().Upper().Copy(0, 2) );
			aBox.SetMessText( aText );
			aBox.Execute();
			return 0;
		}
	}

#endif

	Application::PostUserEvent(m_nOkEvent);
	return 0;
}

IMPL_LINK( SetupOptionsDialog, CancelHdl, Button *, EMPTYARG )
{
	Application::PostUserEvent(m_nCancelEvent);
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK_INLINE_START( SetupOptionsDialog, PathHdl, Button *, EMPTYARG )
{
	SetupInstallPathDialog* pDlg
		= new SetupInstallPathDialog
				(
					this,
					m_pEnv,
					m_aProgramName,
					m_pModule,
					m_eInstallMode,
					m_bScanRemoveableDisks
				);

	short nRet = pDlg->Execute();
	delete pDlg;

	if (nRet != RET_CANCEL)
	{
		ULONG lClusterSize = OS::GetClusterSize(SiDirEntry(m_pEnv->GetDestPath()));

		if (m_lClusterSize != lClusterSize)
		{
			m_lClusterSize = lClusterSize;
			UpdateUsedSize();
		}

		UpdateFreeSize();
		UpdatePath();
	}
	return 0;
}

IMPL_LINK_INLINE_END( SetupOptionsDialog, PathHdl, Button *, EMPTYARG )

// -----------------------------------------------------------------------

IMPL_LINK(SetupOptionsDialog, SelectHdl, SiModuleView*, EMPTYARG)
{
	SvLBoxEntry *pEntry = m_aModuleView.GetCurEntry();

	if (pEntry != NULL)
	{
		SiModule* pModule = (SiModule*) pEntry->GetUserData();
		m_aDescInfoText.SetText(pModule->GetDescription());
	}
	return 0;
}


// -----------------------------------------------------------------------

IMPL_LINK( SetupOptionsDialog, CheckHdl, SiModuleView*, EMPTYARG )
{
	SvLBoxEntry *pEntry = m_aModuleView.GetCurEntry();

	if (pEntry == NULL)
		return 0;

	BOOL	  bChecked  = m_aModuleView.GetCheckButtonState(pEntry) == SV_BUTTON_CHECKED;
	SiModule *pModule   = (SiModule*) pEntry->GetUserData();

	if (pModule == NULL)
		return 0;

	/*
	Das macht das ModuleView selbst:

	pModule->Select(
		bChecked
			? SiModule::Selection::THIS_SEL
			: SiModule::Selection::THIS_UNSEL);
	*/

	UpdateUsedSize();
	SelectHdl(&m_aModuleView);

	// Dialog nur anlegen, wenn er auch benutzt wird, sonst flackert es.
	ByteString InfoText = bChecked ? pModule->GetOnSelect() : pModule->GetOnDeselect();

	if (InfoText.Len() > 0)
	{
		SetupModuleInfoDialog* pDlg
			= new SetupModuleInfoDialog(&m_aModuleView, m_aProgramName);
		pDlg->SetInfoText(InfoText);
		pDlg->Execute();
		delete pDlg;
	}

	return 0;
}

// -----------------------------------------------------------------------

void SetupOptionsDialog::FillOptionBox()
{
	m_aModuleView.List(m_pModule, m_lClusterSize, m_eInstallMode == IM_WORKSTATION);
}

// -----------------------------------------------------------------------

void SetupOptionsDialog::KeyInput( const KeyEvent& rKEvt )
{
	const KeyCode& rKeyCode = rKEvt.GetKeyCode();
	USHORT nCode = rKeyCode.GetCode();

	if ( ( KEY_ESCAPE == nCode && rKeyCode.GetModifier() == 0 )
	||   ( KEY_POINT == nCode  && rKeyCode.IsMod1() ) )
		Application::PostUserEvent(m_nCancelEvent);
	else
		ModelessDialog::KeyInput( rKEvt );
}
