/*************************************************************************
 *
 *  $RCSfile: magenda.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif

#ifndef _SV_MSGBOX_HXX
#include <vcl/msgbox.hxx>
#endif

#include "main.hxx"
#include "mainwnd.hxx"
#include "magenda.hxx"
#include "action.hxx"
#include "critical.hxx"

#include "environ.hxx"
#include "script.hxx"

#include "insrtdsk.hxx"
#include "querydlg.hxx"
#include "overdlg.hxx"
#include "strings.hrc"
#include "app.hrc"
#include "error.hrc"

UIAgendaCallback::UIAgendaCallback( SetupWindow* pWin, SiCompiledScript* pCS, SiEnvironment* pEnv )
{
	m_pMainWin			= pWin;
	m_pCS		   		= pCS;
	m_pEnv 				= pEnv;

	m_nOverwriteStatus	= STATUS_YES;

	m_bSlideShow 		= FALSE;
	m_nSlides			= 0;
	m_nCurrentSlide		= 0;

	if( m_pEnv->IsFirstInstallation() )
	{
		SiSlide* pSlide = m_pCS->GetSlideList().GetObject(0);
		if( pSlide )
		{
			m_aSlidePath = m_pEnv->GetStartPath();
			m_nSlides = (USHORT) m_pCS->GetSlideList().Count();

			SiDirEntry aSlidefile( m_aSlidePath );
			aSlidefile += pSlide->GetBmpFileName();

			if( aSlidefile.Exists() )
				m_bSlideShow = TRUE;
		}
	}
	m_pMainWin->GetLeftWin()->StartProgress();
}

UIAgendaCallback::~UIAgendaCallback()
{
	m_pMainWin->GetContWin()->Reset();
	m_pMainWin->GetLeftWin()->ShowInstInfo( FALSE );
	m_pMainWin->GetLeftWin()->StopProgress();
}

void UIAgendaCallback::StartCompileHelp()
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());
	m_pMainWin->GetLeftWin()->SetActionType( AT_HELP );
}

void UIAgendaCallback::EndCompileHelp()
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());
}

void UIAgendaCallback::StartRegisterUNO()
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());
	m_pMainWin->GetLeftWin()->SetActionType( AT_REGUNO );
}

void UIAgendaCallback::EndRegisterUNO()
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());
}

void UIAgendaCallback::ProgressTransfer( USHORT nPercent )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	m_pMainWin->GetLeftWin()->SetPercentage( nPercent );
	if( m_bSlideShow && m_nSlides > 0 )
	{
		int nNextSlide = 1 + nPercent / (100 / m_nSlides);
		if( m_nCurrentSlide != nNextSlide )
		{
			m_nCurrentSlide = nNextSlide;
			if( m_pCS->GetSlideList().Count() != 0 &&
				m_nCurrentSlide <= m_pCS->GetSlideList().Count() )
			{
				SiSlide* pSlide = m_pCS->GetSlideList().GetObject(m_nCurrentSlide-1);
				if( m_pEnv->GetInstallType() == IT_WORKSTATION && m_nCurrentSlide != 1 )
					return;
				m_pMainWin->GetContWin()->AnimSlide( pSlide );
			}
		}
	}
}

void UIAgendaCallback::ProgressAction( SiActionType eType, USHORT nPercent )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	m_pMainWin->GetLeftWin()->SetActionType( eType );
	ProgressTransfer(nPercent);
}

UIAgendaCallback::ChangedDisk UIAgendaCallback::ChangeDisk( const ByteString& rFilename,
															int nDiskNo, const ByteString& rDiskname )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	if( pSetupApp->IsResponseMode() )
		Critical_Error( ERR_INSTFILENOTFOUND );

	m_pMainWin->ToTop( TOTOP_RESTOREWHENMIN );

	SetupInsertDiskDialog *pDlg = new SetupInsertDiskDialog(
			m_pMainWin->GetContWin(),
			Point(50,50),
			ByteString(pSetupApp->GetProgrammName(), osl_getThreadTextEncoding()),
			rDiskname,
			m_pEnv->GetSourcePath() );

	if( rFilename.Len() != 0 )
	{
		pDlg->SetError();
		pDlg->SetFileText(rFilename);
	}

	USHORT nRet = pDlg->Execute();

	if( nRet == RET_OK )
		m_pEnv->SetSourcePath( ByteString(pDlg->GetSourcePath(), osl_getThreadTextEncoding()) );

	delete pDlg;

	if( nRet == RET_CANCEL &&
		QueryBox(m_pMainWin, ResId(DLG_QUERYEXITBOX)).Execute() == RET_NO )
		nRet = RET_OK;

	return nRet == RET_OK ?		 CHANGEDDISK_OK
		: (nRet == RET_CANCEL  ? CHANGEDDISK_CANCEL
							   : CHANGEDDISK_IGNORE );
}

UIAgendaCallback::ExecErrorAction UIAgendaCallback::ErrorRegisterUnoComponent(const ByteString& rFilename)
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	String aErrorText( ResId(ERR_REGISTERUNOCOMPONENT) );
	aErrorText.SearchAndReplace( UniString::CreateFromAscii("%1"),
					UniString::CreateFromAscii(rFilename.GetBuffer()) );

	m_pMainWin->ToTop( TOTOP_RESTOREWHENMIN );

	SetupExecErrorDialog aDlg( m_pMainWin );
	aDlg.SetErrorText( aErrorText );
	switch( aDlg.Execute() )
	{
		case RET_CANCEL:	return EEA_CANCEL;
		case RET_RETRY:		return EEA_RETRY;
		case RET_OK:
		default:			return EEA_IGNORE;
	}
	return EEA_IGNORE;
}

UIAgendaCallback::ExecErrorAction UIAgendaCallback::ExecutionError( SiAction* pAction )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	String	anError;
	BOOL	bExecuteDlg = TRUE;

	switch( pAction->GetType() )
	{
		case AT_COPY:
		{
			if( pSetupApp->IsResponseMode() )
				Critical_Error( ERR_COPY_FILE );

			const SiTransferAction* pTransAction = (const SiTransferAction*) pAction;

			SiDirEntry aDir  = m_pEnv->GetDestPath();
					 aDir += pTransAction->GetDestPath();

			anError = String( ResId(ERR_COPYFILE) );
			anError.SearchAndReplace(UniString::CreateFromAscii("%1"), UniString::CreateFromAscii(pTransAction->GetDestFile().GetBuffer()) );
			anError.SearchAndReplace(UniString::CreateFromAscii("%2"), aDir.GetFullUni());
			break;
		}

		case AT_UNZIP:
		{
			if( pSetupApp->IsResponseMode() )
				Critical_Error( ERR_ZIP_FILE );

			ByteString aFileName( ((SiTransferAction*)pAction)->GetDestFile() );
			SiDirEntry aEntry( aFileName );

			if( aEntry.GetExtension().CompareIgnoreCaseToAscii( "ttf" ) == COMPARE_EQUAL )
				bExecuteDlg = FALSE;
			else if( aEntry.GetName().CompareIgnoreCaseToAscii( "wps", 3 ) == COMPARE_EQUAL )
				anError = String(ResId(ERR_EXPLODE_WPSBIN));
			else
				anError = String(ResId(ERR_EXPLODEFILE));

			anError.SearchAndReplace(UniString::CreateFromAscii("%1"),
							UniString::CreateFromAscii(aFileName.GetBuffer()) );
			break;
		}

		case AT_MAKEDIR:
		{
			if( pSetupApp->IsResponseMode() )
				Critical_Error( ERR_MAKE_DIR );

			anError  = String(ResId(ERR_MAKEDIR));
			SiDirEntry aDir  = m_pEnv->GetDestPath();
					 aDir += ((SiMakeDirAction*)pAction)->GetName();
			anError += aDir.GetFullUni();
			break;
		}

		default:
			return EEA_IGNORE;
	}

	if( bExecuteDlg && !pSetupApp->IsResponseMode() )
	{
		m_pMainWin->ToTop( TOTOP_RESTOREWHENMIN );

		SetupExecErrorDialog aDlg( m_pMainWin );
		aDlg.SetErrorText(anError);
		switch( aDlg.Execute() )
		{
			case RET_CANCEL:	return EEA_CANCEL;
			case RET_RETRY:		return EEA_RETRY;
			case RET_OK:
			default:			return EEA_IGNORE;
		}
	}

	return EEA_IGNORE;
}

BOOL UIAgendaCallback::RequestBigFile( const ByteString& rDiskname )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	if( pSetupApp->IsResponseMode() )
		Critical_Error( ERR_INSTFILENOTFOUND );

	m_pMainWin->ToTop( TOTOP_RESTOREWHENMIN );

	SiDirEntry aEntry( m_pEnv->GetSourcePath() );
	aEntry.ToAbs();
	ByteString aFileName = aEntry.GetName();

	SetupInsertDiskDialog *pDlg = new SetupInsertDiskDialog(
			m_pMainWin->GetContWin(), Point(50,50), ByteString(pSetupApp->GetProgrammName(), osl_getThreadTextEncoding()),
			rDiskname, ((SiDirEntry&)aEntry.GetPath()).GetFull() );

	pDlg->SetError();
	pDlg->SetFileText( aFileName );

	BOOL bFnd = FALSE;
	while( !bFnd )
	{
		USHORT nRet = pDlg->Execute();
		if( nRet == RET_CANCEL && QueryBox(m_pMainWin, ResId(DLG_QUERYEXITBOX)).Execute() == RET_NO ) {}
		if( nRet == RET_OK )
		{
			aEntry = pDlg->GetSourcePath();
			aEntry += aFileName;
			aEntry.ToAbs();
			if( aEntry.Exists() )
			{
				ArchDirectory* pArch = new ArchDirectory;
				pArch->SetArchFile(aEntry.GetFull().GetBuffer());

				m_pEnv->SetArchive( pArch, FindArch );
				m_pEnv->SetSourcePath( aEntry.GetFull() );

				bFnd = TRUE;
			}
		}
	}
	delete pDlg;
	return TRUE;
}

BOOL UIAgendaCallback::OverwriteFile( const ByteString& rFilename, const SiFile* pFile )
{
	NAMESPACE_VOS(OGuard) aGuard(Application::GetSolarMutex());

	if( pSetupApp->IsResponseMode() )
		return TRUE;

	SiDirEntry aEntry(rFilename);

#if SUPD == 506
	ByteString aFilename( aEntry.GetName() );
	aFilename.ToLower();

	if( aFilename.CompareIgnoreCaseToAscii("sshelp.hid")	== COMPARE_EQUAL ||
		aFilename.CompareIgnoreCaseToAscii("oreg.sdw")		== COMPARE_EQUAL ||
		aFilename.CompareIgnoreCaseToAscii("template.sbl")	== COMPARE_EQUAL )
		return TRUE;

	WildCard aMatch( "*.dll" );
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.exe";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.so";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.tlb";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.bin";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.res";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.sdv";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.sdg";
	if( aMatch == aFilename )
		return TRUE;

	aMatch = "*.thm";
	if( aMatch == aFilename )
		return TRUE;

#ifdef WNT
	if( m_pEnv->IsHackFlag(1) )
	{
		aMatch = "*.tlx";
		if( aMatch == aFilename )
			return TRUE;
	}
#endif

	return FALSE;
#else
	if( m_pEnv->IsUpgradeOldVersion() ||
		m_nOverwriteStatus == STATUS_EVER )
		return TRUE;
	else if( m_nOverwriteStatus == STATUS_NEVER )
		return FALSE;

	if( pFile->DontDelete() )
		return TRUE;

	ULONG nSize = 0;
	if( !pFile->IsArchive() )
		nSize = pFile->GetSize();
	else
	{
		const SiSubfileList& rLst = pFile->GetSubfileList();
		for( ULONG i = 0; i < rLst.Count(); i++ )
		{
			SiSubfile* pSubfile = rLst.GetObject(i);
			if( pSubfile->aFilename.Search(aEntry.GetName()) != STRING_NOTFOUND )
				nSize =	pSubfile->nSize;
		}
	}

	FileStat aStat(aEntry);
	ULONG nExistSize = aStat.GetSize();
	DateTime aExistDate = DateTime( aStat.DateCreated(), aStat.TimeCreated() );
	if( nSize == nExistSize &&
		DateTime(pFile->GetDate(), pFile->GetTime()) == aExistDate )
		return TRUE;

	m_pMainWin->ToTop( TOTOP_RESTOREWHENMIN );
	OverwriteFileDialog aDlg(m_pMainWin);

	aDlg.SetExistFile(aEntry);
	aDlg.SetSourceFile( nSize, pFile->GetDate(), pFile->GetTime() );
	aDlg.Execute();

	m_nOverwriteStatus = aDlg.GetStatus();

	if( m_nOverwriteStatus == STATUS_YES ||
		m_nOverwriteStatus == STATUS_EVER )
		return TRUE;

	return FALSE;
#endif
}

