/*************************************************************************
 *
 *  $RCSfile: mainevt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:50:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifdef UNX
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#endif

#include <ctype.h>

#if defined(ICC) && defined(OS2)
#include <tools/svpm.h>
#endif

#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _VOS_PROCESS_HXX //autogen
#include <vos/process.hxx>
#endif

#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif

#ifndef _SV_CONFIG_HXX //autogen
#include <vcl/config.hxx>
#endif

#ifndef _WLDCRD_HXX //autogen
#include <tools/wldcrd.hxx>
#endif

#ifndef _SFXINIDEF_HXX //autogen
#include <svtools/inidef.hxx>
#endif
#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif
#ifndef _PVER_HXX //autogen
#include <svtools/pver.hxx>
#endif

#include "main.hxx"
#include "mainwnd.hxx"
#include "maindlg.hxx"
#include "critical.hxx"
#include "fields.hxx"
#include "texec.hxx"
#include "action.hxx"

#include "zipfile.hxx"
#include "sihelp.hxx"
#include "environ.hxx"
#include "agenda.hxx"
#include "compiler.hxx"
#include "script.hxx"
#include "instdb.hxx"
#include "magenda.hxx"
#include "sibasic.hxx"
#include "respfile.hxx"

#include "arch.hxx"
#include "os.hxx"

#include "infodlg.hxx"
#include "insrtdsk.hxx"

#include "strings.hrc"
#include "app.hrc"
#include "error.hrc"

#define AUTO_UPDATE_SAME_VERSION		1

extern char SETUP_INSTDB_FILE[];

struct TopLevelObject
{

	ExecType        eType;
	ByteString          aCommand;
	ByteString          aParam;

	BOOL			bIsFollow;
};

BOOL 		_PatchLicense( const SiDirEntry& rDllToPatch, const ByteString& rCode,
						   SiCompiledScript* pCScript );
ByteString	_GetLicensePatch( const SiDirEntry& rDllToTest );
BOOL		_GetSvt(SiEnvironment* pEnv, SiDirEntry& rEntry);

// ----------------------------------------------------------------------------

void SetupApp::UserEvent( ULONG nEvent, void* pEventData)
{
	static BOOL bEnd = FALSE;

	if (bEnd)
		return;

	if( m_pMainDlg )
	{
		delete m_pMainDlg;
		m_pMainDlg = NULL;
	}

	switch (nEvent)
	{
		case EVT_START:
			OnEventStart();
			break;

		case EVT_EXIT:
			ExitHdl(0);
			break;

		case EVT_CANCEL_MAINDLG:
			if( m_pEnv->IsDlgPrecreateDir() )
			{
				SiDirEntry aEntry( m_pEnv->GetDestPath() );
				aEntry.Kill();
			}
			PostUserEvent( EVT_END );
			break;

		case EVT_END:
			if( m_pAgenda != NULL )
			{
				if( QueryBox(GetAppWindow(), ResId(DLG_QUERYEXITBOX)).Execute() == RET_YES )
					m_pAgenda->StopRunning();
				break;
			}

			if( pTopLevelStack && pTopLevelStack->Count() )
				PostUserEvent( EVT_TOPLEVEL_LOOP );
			else
			{
				bEnd = TRUE;
				OnEventEnd();
			}
			break;

		case EVT_CLOSE_INSTALLATION:
			CloseInstallation();
			break;

		case EVT_ENDINSTALL:
			OnEventEndInstallation();
			break;

		case EVT_START_TOPLEVEL :
		{
			SiModule* pMod = m_pCompiledScript->GetRootModule();
			SiModule* pFnd = NULL;
			USHORT    nId  = 0;

			pTopLevelStack  = new TopLevelStack;
			pActTopObject   = NULL;

			while( (pFnd = GetNextTopLevel(pMod, nId, NULL)) != NULL )
			{
				if( pFnd->IsSelected() )
				{
					AddTopLevelStack( pFnd->GetExecType(), pFnd->GetExecCommand() );
					if( pFnd->GetFollowExecCommand().Len() )
					{
						AddTopLevelStack( pFnd->GetFollowExecType(),
										  pFnd->GetFollowExecCommand(),
										  TRUE );
					}
				}
				nId = pFnd->GetTopLevelOrder();
			};

			PostUserEvent(EVT_TOPLEVEL_LOOP);
			break;
		}

		case EVT_TOPLEVEL_LOOP :
		{
			if( pActTopObject )
			{
				delete pActTopObject;
				pActTopObject = NULL;
			}

			m_pMainWin->SetProductName(UniString::CreateFromAscii(""));

			if( pTopLevelStack->Count() )
			{
				pActTopObject = pTopLevelStack->Remove( (ULONG) 0 );
				if( pActTopObject->bIsFollow && pActTopObject->aParam.Len() )
				{
					pActTopObject->aParam.SearchAndReplace( "$(INSTPATH)",
								   m_pEnv->GetDestPath() );
				}

				switch( pActTopObject->eType )
				{
					case EXEC_SCRIPT :
					{
						SiDirEntry aCmd( pActTopObject->aCommand );
						m_pEnv->ClearEnvironment();
						m_pEnv->SetStartPath( ((SiDirEntry&)aCmd.GetPath()).GetFull() );
						aCmd.GetPath().SetCWD();

						if( !FindAndReadScript() )
							PostUserEvent( EVT_TOPLEVEL_LOOP );

						m_pMainWin->SetProductName(GetProgrammName());
						m_pEnv->SetProductName( ByteString(GetProgrammName(), osl_getThreadTextEncoding()) );

						PostUserEvent( EVT_START );
					} break;


					case EXEC_SHELL :
					#ifdef UNX
					{
						m_pMainWin->Hide();
						Application::Reschedule();

						SiDirEntry aCmd( pActTopObject->aCommand );
						aCmd.GetPath().SetCWD();

						if( fork() == 0 )
						{
							char* argv[20];
							argv[0] = new char [255];
							argv[1] = new char [255];
							argv[2] = new char [255];

							strcpy( argv[0], "xterm" );
							strcpy( argv[1], "-e" );
							strcpy( argv[2], pActTopObject->aCommand.GetBuffer() );

							USHORT nIdx = 2;
							if( pActTopObject->aParam.Len() )
							{
								USHORT nActTok = 0;
								USHORT nTok = pActTopObject->aParam.GetTokenCount(' ');
								for( USHORT i = 0; i < nTok; ++i )
								{
									if( nIdx < 15 )
									{
										nIdx += 1;
										argv[nIdx] = new char [255];
										strcpy( argv[nIdx], pActTopObject->aParam.GetToken(0, ' ', nActTok).GetBuffer() );
									}
								}
							}
							argv[nIdx+1] = NULL;

							execvp( "xterm", argv );

							for( USHORT i = 0; i <= nIdx; ++i )
								delete argv[i];

							exit( errno == ENOENT? 7 : 8 );
						}

						int status;
						wait( &status );
						if( status != 0)
							switch( status )
							{
								case 7 :
								{
									if( !IsResponseMode() )
									{
										InfoBox aIBox( NULL, UniString::CreateFromAscii("xterm not found!") );
										aIBox.Execute();
									}
									else
										Critical_Error( ERR_XTERMNOTFOUND );

								} break;

								case 8 :
								{
									if( !IsResponseMode() )
									{
										InfoBox aIBox( NULL, UniString::CreateFromAscii("could not start xterm!") );
										aIBox.Execute();
									}
									else
										Critical_Error( ERR_COULDNOTSTARTXTERM );

								} break;
								default:
								{
									if( !IsResponseMode() )
									{
										UniString aMsg( "start xterm return errorcode ", osl_getThreadTextEncoding() );
										aMsg += UniString::CreateFromInt32( status );
										InfoBox aIBox( NULL, aMsg );
										aIBox.Execute();
									}
									else
										Critical_Error( ERR_XTERMERRORCODE );
								} break;
							}

						if( pTopLevelStack->Count() &&
							(pTopLevelStack->GetObject(0)->eType != EXEC_APP &&
							pTopLevelStack->GetObject(0)->eType != EXEC_SHELL) )
							m_pMainWin->Show();

						PostUserEvent( EVT_TOPLEVEL_LOOP );
					} break;
					#endif //UNX

					case EXEC_APP :
					{
						m_pMainWin->Hide();
						Application::Reschedule();

						SiDirEntry aCmd( pActTopObject->aCommand );
						aCmd.GetPath().SetCWD();

						::rtl::OUString aOUCommand( UniString::CreateFromAscii(pActTopObject->aCommand.GetBuffer()) );
						::rtl::OUString aOUEmpty( UniString::CreateFromAscii(pActTopObject->aCommand.GetBuffer()) );
						NAMESPACE_VOS(OProcess) aProcess( aOUCommand, aOUEmpty );
						aProcess.execute( NAMESPACE_VOS(OProcess)::TOption_Detached );
						aProcess.join();

						if( pTopLevelStack->Count() &&
							(pTopLevelStack->GetObject(0)->eType != EXEC_APP &&
							pTopLevelStack->GetObject(0)->eType != EXEC_SHELL) )
							m_pMainWin->Show();

						PostUserEvent( EVT_TOPLEVEL_LOOP );
					} break;
				}
			}
			else
			{
				delete pTopLevelStack;
				pTopLevelStack = NULL;
				PostUserEvent( EVT_END );
			}
			break;
		}

		case EVT_START_UPDATE_OLD_VERSION :
		{
			RunUpdateOldVersion();
		} break;

		case EVT_START_INSTALL :
		case EVT_START_APPSERVER_INSTALL :
		{
/*
  			if( m_pEnv->IsFirstInstallation() && m_pEnv->IsLocal() )
				SiHelp::MakeOffice_AuthFile(m_pCompiledScript, m_pEnv);
*/
			SiInstallMode eInstallMode =
				m_pEnv->GetInstallType() == IT_WORKSTATION ? IM_WORKSTATION :
				m_pEnv->IsLocal() ? IM_STANDALONE : IM_NETWORK;

			RunInstallation( m_pCompiledScript, eInstallMode );
			break;
		}

		case EVT_INITIATE_WORKSTATION :
		{
			if( m_pEnv->GetWorkstationList().Count() )
			{
				if( !m_nWorkStationIdx )
					m_nWorkStationCnt = (USHORT) m_pEnv->GetWorkstationList().Count();
				m_nWorkStationIdx++;

				InitiateWorkstation* pWorkStation = m_pEnv->GetWorkstationList().Remove((ULONG)0);
				InstallWorkstation( pWorkStation );
			}
			else
				PostUserEvent( EVT_ENDINSTALL );
			break;
		}

		case EVT_REINST_CHANGE :
		{
			m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_CHANGE );
			RunInstallation( m_pCompiledScript, IM_INVALID, ISM_CHANGE );
			break;
		}

		case EVT_REINST_UNINSTALL :
		{
			DEL_DELFILE( *m_pEnv );
			m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_DELETE );
			RunInstallation( m_pCompiledScript, IM_INVALID, ISM_DELETE );
			break;
		}

		case EVT_REINST_RECOVER :
		{
			m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_RECOVER );
			RunInstallation( m_pCompiledScript, IM_INVALID, ISM_RECOVER );
			break;
		}

		case EVT_REINST_APPSERVER_USERADD :
		{
/*			SiSubstituteList& rLst = m_pEnv->GetOpenSubstituteList();
			if( rLst.Count() )
			{
				for( ULONG x = 0; x < rLst.Count(); ++x )
				{
					ByteString* pFilename = rLst.GetObject(x);
					_Substitute( *pFilename, *m_pEnv, IM_INVALID );
					delete pFilename;
				}
				rLst.Clear();
				m_pCompiledScript->GetInstallation()->SetOpenSubList( rLst );
			}
			DeInstallAppServerClients();
			InstallAppServerClients();
			if( !IsResponseMode() )
			{
				InfoBox aBox( GetAppWindow(), ResId(DLG_ENDINSTALL_CHANGE) );
				aBox.Execute();
			}
*/			PostUserEvent(EVT_END);
			break;
		}

		case EVT_REINST_APPSERVER_DEINSTALL :
		{
/*			for( USHORT i = 0; i < m_pEnv->GetClientNameList().Count(); ++i ) {
				AppServerClient* pUser = m_pEnv->GetClientNameList().GetObject(i);
				pUser->bToDelete = TRUE;
			}
*/
			m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_DELETE );
			RunInstallation(m_pCompiledScript, IM_INVALID, ISM_DELETE);
/*			if( m_pEnv->DoDeInstallClients() )
				DeInstallAppServerClients();
*/
/*			if( !IsResponseMode() )
			{
				InfoBox aBox( GetAppWindow(), ResId(DLG_ENDINSTALL_DEINSTALL) );
				aBox.Execute();
			}
			PostUserEvent(EVT_END);
*/			break;
		}

		default:
			DBG_ERROR( "SetupApp::UserEvent() invalid event" );
	}
}

// -----------------------------------------------------------------------

void SetupApp::AddTopLevelStack( ExecType eType, ByteString aCommand, BOOL bFollow )
{
	USHORT nParaPos = aCommand.Search( " " );
	ByteString aCmd		= nParaPos == STRING_NOTFOUND? aCommand :
					  aCommand.Copy( 0, nParaPos );
	ByteString aParam	= nParaPos == STRING_NOTFOUND? ByteString("") :
					  aCommand.Copy( nParaPos + 1, aCommand.Len() - nParaPos - 1 );

	SiDirEntry aAbsCmd( m_pEnv->GetStartPath() );
	aAbsCmd += aCmd;
	aAbsCmd.ToAbs();

	TopLevelObject* pNew = new TopLevelObject;

	pNew->eType     = eType;
	pNew->aCommand  = aAbsCmd.GetFull();
	pNew->aParam	= aParam;
	pNew->bIsFollow = bFollow;

	pTopLevelStack->Insert( pNew, LIST_APPEND );
}

// -----------------------------------------------------------------------

void SetupApp::CheckDeadTopLevel()
{
	SiModule* pFnd = NULL;
	USHORT    nId  = 0;

	while( (pFnd = GetNextTopLevel(m_pCompiledScript->GetRootModule(), nId, NULL)) != NULL )
	{
		nId = pFnd->GetTopLevelOrder();
		if( pFnd->GetExecCommand().Len() )
		{
			if( pFnd->IsDefault() )
				pFnd->Select(SiModule::THIS_SEL);

			SiDirEntry aEntry( m_pEnv->GetStartPath() );
			aEntry += pFnd->GetExecCommand();
			if( aEntry.Exists() )
				continue;
		}
		RemoveDeadTopLevel( pFnd );
	}
}

// -----------------------------------------------------------------------

void SetupApp::RemoveDeadTopLevel( SiModule* pModule )
{
	SiModule* pParent = pModule->GetParent();

	m_pCompiledScript->RemoveDeclarator( pModule->GetID() );
	pParent->GetModuleList()->Remove( pModule );
	delete pModule;

	if( !pParent->GetModuleList()->Count() &&
		pParent->GetParent() )
		RemoveDeadTopLevel( pParent );
}

// -----------------------------------------------------------------------

SiModule* SetupApp::GetNextTopLevel(SiModule* pModule, USHORT nId, SiModule* pBest)
{
	const SiModuleList* pModuleList = pModule->GetModuleList();

	if( pModuleList->Count() )
	{
		SiModule* pFnd = pBest;
		for( USHORT i=0; i < pModuleList->Count(); i++ )
		{
			SiModule* pAct = GetNextTopLevel(pModuleList->GetObject(i), nId, pFnd);
			if( pAct )
				pFnd = pAct;
		}
		return pFnd;
	}

	if( pModule->GetTopLevelOrder() <= nId )
		return NULL;

	if( !pBest ||
		(pModule->GetTopLevelOrder() - nId) <
		(pBest->GetTopLevelOrder() - nId) )
		return pModule;

	return NULL;
}

// -----------------------------------------------------------------------

void SetupApp::GetMainDialogModel()
{
	if( !m_pMainDlg )
		return;

	ModelType eModelType = MODEL_NOTSET;
	SiInstallation* pInstObj = m_pCompiledScript->GetInstallation();

	if( m_bParamDoCRCCheck )
	{
		m_pMainDlg->CreateModel_CRCCheck();
		m_pMainDlg->SuccEvent( EVT_END );
		m_pMainDlg->FailEvent( EVT_END );
		return ;
	}
	else if( m_bOfficeMode )
	{
		m_pMainDlg->CreateModel_OfficeRepair();
		m_pMainDlg->SuccEvent( EVT_REINST_RECOVER );
		m_pMainDlg->FailEvent( EVT_END );
		return ;
	}

	if( m_pEnv->IsUpdateOldVersion() || m_pEnv->IsUpgradeOldVersion() )
	{
		m_pMainDlg->CreateModel_UpdateOldVersion();
		m_pMainDlg->FailEvent( EVT_END );
		if( m_pEnv->IsUpdateNoVersion()						||
			m_pEnv->IsUpdateOldVersionNoRights()			||
			m_pEnv->IsUpdateOldVersionWrongLanguage() )
			m_pMainDlg->SuccEvent( EVT_END );
		else
			m_pMainDlg->SuccEvent( EVT_START_UPDATE_OLD_VERSION );
		return;
	}

	ByteString aInstalledPath;
	if( SiHelp::IsVersionAlreadyInstalled(aInstalledPath, m_pCompiledScript, m_pEnv) )
	{
		m_pEnv->SetInstalledPath( aInstalledPath );
		m_pMainDlg->CreateModel_WrongSetup();
		m_pMainDlg->FailEvent( EVT_END );
		m_pMainDlg->SuccEvent( EVT_END );
		return;
	}

	if( pInstObj->GetInstallMode() == IM_TOPLEVEL )
	{
		CheckDeadTopLevel();
		m_pMainDlg->CreateModel_TopLevel();
		m_pMainDlg->SuccEvent( EVT_START_TOPLEVEL );
		m_pMainDlg->FailEvent( EVT_END );

		return;
	}

	if( m_pEnv->IsFirstInstallation() )
	{
		if( pInstObj->GetUiProcName().Len() )
		{
			SiProcedure* pProc = (SiProcedure*)m_pCompiledScript->
								FindProcedureByName( pInstObj->GetUiProcName() );
			BOOL bOk = FALSE;
			if( pProc )
			{
				SiBasic aBasic( *m_pCompiledScript, *m_pEnv );
				aBasic.SetAgentDlg( m_pMainDlg );
				bOk = aBasic.Call( pProc->GetProcName(), pProc->GetCode() );

				m_pMainDlg->FailEvent( EVT_END );
				m_pMainDlg->SuccEvent( EVT_START_INSTALL );

				m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_ADD);
			}
			if( !bOk )
			{
				UniString aMessage( UniString::CreateFromAscii("Error: UiProcedure not found\n\n") );
				aMessage += UniString::CreateFromAscii( pInstObj->GetUiProcName().GetBuffer() );

				if( !IsResponseMode() )
				{
					ErrorBox anErrorBox(NULL, WB_OK, aMessage);
					anErrorBox.Execute();
					PostUserEvent(EVT_END);
				}
				else
					Critical_Error( ERR_UIPROCNOTFOUND );
			}
			return;
		}
		else if( pInstObj->GetInstallMode() == IM_APPSERVER )
		{
			m_pMainDlg->CreateModel_AppServerInstallation();
			m_pMainDlg->SuccEvent( EVT_START_APPSERVER_INSTALL );
		}
		else
		{
			m_pMainDlg->CreateModel_FirstInstallation();
			m_pMainDlg->SuccEvent( EVT_START_INSTALL );
		}

		m_pMainDlg->FailEvent( EVT_CANCEL_MAINDLG );
		m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_ADD);

		return;
	}
	else
	{
		if( pInstObj->GetInstallMode() == IM_APPSERVER )
			m_pMainDlg->CreateModel_AppServerReInstallation();
		else
			m_pMainDlg->CreateModel_ReInstallation();

		m_pMainDlg->FailEvent( EVT_END );
		return;
	}
}

// -----------------------------------------------------------------------

void SetupApp::OnEventStart()
{
	// migration plugin
	if( m_pEnv->IsFirstInstallation() && m_pEnv->IsLocal() )
	{
		SiCustom* pMigPlugin = SiHelp::GetMigrationPlugin( m_pCompiledScript->GetRootModule() );
	 	SiDirEntry aSVersion = m_pCompiledScript->GetInstallInfo()->GetProfile()->GetDirectory()->GetName();
		aSVersion += m_pCompiledScript->GetInstallInfo()->GetProfile()->GetName();

		if( aSVersion.Exists() && pMigPlugin )
		{
			m_pEnv->SetMigrationAvailable( TRUE );
			Config aCfg( aSVersion.GetFullUni() );
			aCfg.SetGroup( m_pCompiledScript->GetInstallInfo()->GetSection() );

			SiCustomAction* pAction = new SiCustomAction( NULL,
							m_pCompiledScript->GetRootModule(),
							pMigPlugin->GetDllName(),
							m_pEnv->GetStartPath(),
							FALSE,
							FALSE,
							pMigPlugin );

			pAction->SetKeepAlive();
			pAction->Execute( *m_pEnv );

			for( USHORT i = 0; i < aCfg.GetKeyCount(); ++i )
			{
				ByteString aKey( aCfg.GetKeyName(i) );
				ByteString aValue( aCfg.ReadKey(i) );
				SiDirEntry aPath( aValue );
				SiDirEntry aSofficeINI( aPath );
				#ifdef UNX
				aSofficeINI += ByteString("sofficerc");
				#else
				aSofficeINI += ByteString("soffice.ini");
				#endif

				if( aPath.Exists() && aSofficeINI.Exists() )
				{
					BOOL bSupportVersion = pAction->GetCustomFnc().fncSupportVersion(aKey);
					if( bSupportVersion )
					{
						m_pEnv->SetMigration( TRUE );
						m_pEnv->SetMigrationPath( aPath.GetFull() );
					}
				}
			}

			if( !pAction->InternalError() ) pAction->GetCustomFnc().fncKill();
			delete pAction;
		}
	}

	if( !m_pEnv->IsUpdateOldVersion() && !m_pEnv->IsUpgradeOldVersion() )
		m_pEnv->InitRecycle();

	m_pMainDlg = new SetupAgentDialog( ((SetupWindow*)GetAppWindow())->GetContWin(), m_pResMan );
	m_pMainDlg->SetLanguage(GetLanguage());
	m_pMainDlg->SetCScript(m_pCompiledScript);
	m_pMainDlg->SetEnv(m_pEnv);
	GetMainDialogModel();

	m_pMainDlg->StartAgentDlg();
	m_pMainDlg->Show();
}

// -----------------------------------------------------------------------

void _FlagAllAsInstalled(SiModule* const pMod)
{
	pMod->SetInstalled();
	SiModuleList const* pModuleList = pMod->GetModuleList();
	if( pModuleList != NULL )
		for (USHORT xx=0; xx < pModuleList->Count(); xx++)
			_FlagAllAsInstalled( pModuleList->GetObject(xx) );
}

// -----------------------------------------------------------------------
extern "C" {
const char* FindArch(const char* pDefSource, const char* pDefName )
{
	SiDirEntry aEntry(pDefName);

	SetupInsertDiskDialog *pDlg = new SetupInsertDiskDialog(
		((SetupWindow*)Application::GetAppWindow)->GetContWin(),
		Point(50,50), ByteString(pSetupApp->GetProgrammName(),osl_getThreadTextEncoding()),
		ByteString(""), ((SiDirEntry&)aEntry.GetPath()).GetFull() );

	pDlg->SetError();
	pDlg->SetFileText(aEntry.GetName());

	BOOL bFnd = FALSE;
	BOOL bEndInst = FALSE;
	while( !bFnd && !bEndInst )
	{
		USHORT nRet = pDlg->Execute();
		if( nRet == RET_CANCEL &&
			QueryBox(NULL, ResId(DLG_QUERYEXITBOX)).Execute() == RET_YES )
		{
			bEndInst = TRUE;
		}
		if( nRet == RET_OK )
		{
			aEntry = pDlg->GetSourcePath();
			pSetupApp->GetEnvironment()->SetSourcePath( aEntry.GetFull() );
			bFnd = TRUE;
		}
	}
	delete pDlg;

	if( bEndInst )
	{
		pSetupApp->GetAgenda()->StopRunning();
		return NULL;
	}
	return pSetupApp->GetEnvironment()->GetSourcePath().GetBuffer();
}
};

// -----------------------------------------------------------------------

void SetupApp::RunUpdateOldVersion()
{
	SiDirEntry aOldInstDB( m_pEnv->GetInstalledPath() );
	aOldInstDB += ByteString( SETUP_INSTDB_FILE );

	ByteString aInstalledPath( ((SiDirEntry&)aOldInstDB.GetPath()).GetFull() );
	ByteString aInstallSourcePath( m_pEnv->GetSourcePath() );

	BOOL bIsNet = FALSE;
	SiCompiledScript aOldScript;
	if( ReadScript(aOldInstDB, &aOldScript) )
	{
		bIsNet = aOldScript.GetInstallation()->GetInstallMode() == IM_NETWORK? TRUE : FALSE;

		m_pEnv->SetInstalledPath	( aInstalledPath );
		m_pEnv->SetDestPath			( aInstalledPath );
		m_pEnv->SetSourcePath		( aInstalledPath );
		m_pEnv->SetFirstInstallation( FALSE );

		if( m_pEnv->IsUpdateOldVersion() )
		{
			// Bei Uninstall werden alle Module selektiert, deshalb muessen vorher die
			// Selections auf das neue Script uebertragen werden.
			m_pCompiledScript->GetRootModule()->Select( SiModule::ALL_UNSEL );
			SiHelp::TransferSelection( aOldScript.GetRootModule(), m_pCompiledScript->GetRootModule() );
			#if SUPD == 506
			SiHelp::SelectAllDontSelectByUser( m_pCompiledScript->GetRootModule() );
			#endif

			// deinstallation wird jetzt sync. gefahren!
			if( m_pAgenda )
				delete m_pAgenda;
			m_pAgenda = new SiAgenda;

			UIAgendaCallback* pUICall = new UIAgendaCallback( m_pMainWin, &aOldScript, m_pEnv );

			m_pAgenda->SetWindowHideLink( LINK(this, SetupApp, WindowHideHdl) );
			m_pAgenda->SetCallback( pUICall );
			m_pAgenda->SetAppLanguage( m_nLanguage );
			m_pAgenda->SetEnvironment( m_pEnv );

			m_pAgenda->CreateForUninstall(aOldScript, ISM_DELETE);

			// und los...
			m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_DELETE );
			m_pAgenda->Run(aOldScript, ISM_DELETE );

			delete m_pAgenda;
			m_pAgenda = NULL;
		}
	}

	m_pEnv->SetFirstInstallation();
	m_pEnv->SetDestPath			( aInstalledPath );
	m_pEnv->SetSourcePath		( aInstallSourcePath );

	m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_ADD, m_pEnv );

	m_pEnv->SetLocal(!bIsNet);
	SiInstallMode eInstallMode = m_pEnv->IsLocal() ? IM_STANDALONE : IM_NETWORK;

	RunInstallation( m_pCompiledScript, eInstallMode );
}

// -----------------------------------------------------------------------

void SetupApp::RunInstallation( SiCompiledScript* pCS, SiInstallMode eInstallMode,
		SiInstallSubMode eInstallSubMode, BOOL bUpgrade, BOOL bNoEvents )
{
	SiModule        *pRootModule    = pCS->GetRootModule();
	SiInstallation  *pInstallation  = pCS->GetInstallation();

	if( !pRootModule )
		return;

	if( m_pEnv->GetUILanguageContext().Count() == 1 )
	{
		LanguageContext* pAct = m_pEnv->GetUILanguageContext().GetObject(0);
		pAct->isProg = TRUE;
		pAct->isDoc	 = TRUE;
	}
	PrepareLanguageContext();

	// Agenda berechnen
	SiCompiledScript aCS;
	if( m_pAgenda )
		delete m_pAgenda;
	m_pAgenda = new SiAgenda;

	UIAgendaCallback* pUICall = new UIAgendaCallback( m_pMainWin, m_pCompiledScript, m_pEnv );
	m_pAgenda->SetCallback( pUICall );

	m_pAgenda->SetWindowHideLink( LINK(this, SetupApp, WindowHideHdl) );
	m_pAgenda->SetAppLanguage( m_nLanguage );
	m_pAgenda->SetEnvironment( m_pEnv );

	if( bUpgrade )
	{
		m_pEnv->SetFirstInstallation();
		eInstallMode = m_pCompiledScript->GetInstallation()->GetInstallMode();
	}
	else if( m_pEnv->IsFirstInstallation() )
	{
		if( pCS->GetRootModule() != NULL )
			pCS->GetRootModule()->Select( SiModule::THIS_SEL );

		if( m_pEnv->GetSourcePath() == "" )
			m_pEnv->SetSourcePath( m_pEnv->GetStartPath() );
	}
	else
	{
		eInstallMode = pInstallation->GetInstallMode();
		m_pEnv->SetSourcePath( pCS->GetInstallation()->GetSourcePath() );
	}

	if( m_pEnv->IsFirstInstallation() )
	{
		m_pAgenda->CreateForInstall( *pCS, eInstallMode,
					pInstallation != NULL && pInstallation->GetInstallMode() == IM_NETWORK );
	}
	else
	{
		switch( eInstallSubMode )
		{
			case ISM_CHANGE:
				m_pAgenda->CreateForChange(*pCS,eInstallSubMode);
				break;

			case ISM_RECOVER:
				m_pAgenda->CreateForRecover(*pCS);
				break;

			case ISM_DELETE:
				m_pAgenda->CreateForUninstall(*pCS, ISM_DELETE);
				break;

			default:
				DBG_ERROR("SetupApp::RunInstallation() wrong submode");
				break;
		}
	}

	m_bIsAppServerInstall = m_pCompiledScript->GetInstallation()->GetInstallMode() == IM_APPSERVER ?
							TRUE : FALSE;

	m_pExecuter = new ThreadExecuter(m_pAgenda);

	m_pExecuter->SetSourcePath		( m_pEnv->GetSourcePath() );
	m_pExecuter->SetInstallMode		( eInstallMode );
	m_pExecuter->SetInstallSubMode	( eInstallSubMode );
	m_pExecuter->SetCScript			( pCS );
	m_pExecuter->SetUpgrade			( bUpgrade );
	m_pExecuter->SetNoEvent			( bNoEvents );

	m_bPerformInstall = TRUE;

	m_pExecuter->create();
	m_pExecuter->setPriority( NAMESPACE_VOS(OThread)::TPriority_AboveNormal );
}

void SetupApp::CloseInstallation()
{
	SiInstallation* pInstallation = m_pExecuter->GetCScript()->GetInstallation();

	// bei AppServer Deinstallation war's das hier, es folgen ja noch
	// die Clients...
/*	if( m_pExecuter->GetInstallMode() == IM_APPSERVER &&
		m_pExecuter->GetInstallSubMode() == ISM_DELETE )
	{
		m_bPerformInstall = FALSE;

		delete m_pExecuter;
		m_pExecuter = NULL;

		delete m_pAgenda;
		m_pAgenda = NULL;

		return;
	}
*/
	if( m_pEnv->IsFirstInstallation() && m_pExecuter->GetInstallMode() == IM_STANDALONE )
		KillSlideshowFiles( m_pExecuter->GetCScript() );

	if( !m_pAgenda->WasRunningStopped() && m_pEnv->IsFirstInstallation() && m_pEnv->IsLocal() )
		m_pEnv->StoreRecycle();

	// Inst-DB
	SiDirEntry aScriptEntry;
	if( !m_bIsAppServerInstall )
		aScriptEntry = m_pEnv->GetInstallType() == IT_WORKSTATION?
					   m_pEnv->GetDestPath() : m_pCompiledScript->GetSetupExePath(*m_pEnv);
	else
		aScriptEntry = m_pCompiledScript->GetSetupExePath(*m_pEnv);
	aScriptEntry += ByteString( SETUP_INSTDB_FILE );
	pInstallation->SetDestPath( m_pEnv->GetDestPath() );

	if( !m_pEnv->IsLocal() && pInstallation->GetLanguages().Len() )
	{
		ByteString aNewLanguages;
		SiLangCtxList& rLst = m_pEnv->GetLanguageContext();
		for( USHORT i = 0; i < rLst.Count(); ++i )
		{
			LanguageContext* pAct = rLst.GetObject(i);
			if( pAct->isProg || pAct->isDoc )
			{
				ByteString add = pAct->nLanguage == LANG_DEFAULT?
								pInstallation->GetDefLanguage() : ByteString::CreateFromInt32( pAct->nLanguage );

				if( add.Len() == 1 ) add.Insert( "0", 0 );
				aNewLanguages += add;

				if( i != rLst.Count()-1 )
					aNewLanguages += ",";
			}
		}
		pInstallation->ClearInstalledLanguages();
		pInstallation->SetLanguages( aNewLanguages );
	}

	if( !m_pAgenda->WasRunningStopped()	&&
		m_pExecuter->GetInstallSubMode() != ISM_RECOVER &&
		m_pExecuter->GetInstallSubMode() != ISM_DELETE  &&
		!m_pExecuter->IsWorkstation() )
	{
		SiAnsiFileStream aStream;
		SiDatabase       aDB(aStream);

		aStream.Open(aScriptEntry.GetFullUni(), STREAM_STD_WRITE | STREAM_TRUNC);

		if (aStream.IsOpen())
		{
			if( m_pExecuter->GetInstallMode() == IM_NETWORK &&
				pInstallation->GetInstallMode() == IM_APPSERVER )
				m_pExecuter->SetInstallMode( IM_APPSERVER );

			if( m_pEnv->IsFirstInstallation() && !m_pExecuter->GetUpgrade() )
			{
				ByteString aSource = m_pExecuter->GetSourcePath();
				if( m_pExecuter->GetInstallMode() == IM_NETWORK && m_pEnv->IsBigMode() )
					aSource = m_pEnv->GetDestPath();

				aDB.Create( m_pExecuter->GetCScript(), m_pExecuter->GetInstallMode(), aSource,
					(pInstallation != NULL && pInstallation->GetInstallMode() == IM_NETWORK) );
			}
			else
			{
			if( m_pExecuter->GetUpgrade() )
			{
				m_pCompiledScript->Join( m_pExecuter->GetCScript() );
				aDB.Create( m_pCompiledScript ); // Inst-DB
			}
			else
				aDB.Create( m_pExecuter->GetCScript() );
			}
		}
		aStream.Flush();
	}
	else if( m_pExecuter->GetInstallSubMode() == ISM_DELETE )
	{
		aScriptEntry.Kill();
	}

	// Installation wurde durchgefuehrt, Hauptfenster ggf. restaurieren
	// damit MessBoxen sichtbar werden koennen.
	if( !IsResponseMode() )
		m_pMainWin->ToTop( TOTOP_RESTOREWHENMIN );

	if( !m_pAgenda->WasRunningStopped() && m_bIsAppServerInstall )
	{
		pInstallation->SetInstallMode( IM_APPSERVER );
		if( m_pEnv->GetWorkstationList().Count() )
		{
			if( m_pExecuter->IsWorkstation() )
			{
				WriteWorkstationDB();

				delete m_pExecuter->GetCScript();
				delete m_pAgenda->GetEnvironment();
			}

			delete m_pExecuter;
			m_pExecuter = NULL;

			delete m_pAgenda;
			m_pAgenda = NULL;

			PostUserEvent( EVT_INITIATE_WORKSTATION );
			return ;
		}
		else
		{
			if( m_pExecuter->IsWorkstation() )
			{
				WriteWorkstationDB();

				delete m_pExecuter->GetCScript();
				delete m_pAgenda->GetEnvironment();
			}
		}
	}

	// Systemdateien anpassen...
	if( !m_pAgenda->WasRunningStopped()  &&
		m_pExecuter->GetInstallMode() != IM_NETWORK &&
		m_pExecuter->GetCScript()->GetInstallation() != NULL )
	{
		if( m_pEnv->IsFirstInstallation() )
		{
			#ifdef WIN
			if( !IsShareLoaded() &&
				m_pExecuter->GetCScript()->GetInstallation()->RequiresShare() )
			{
				WinOS::InstallShare();
				m_pEnv->SetReboot( TRUE );
			}
			#endif
			#ifdef OS2
			if( m_pExecuter->GetCScript()->GetInstallation()->DoModifyPath() )
			{
				BOOL bOK = TRUE;
				Os2OS::AddDestPathToConfigSys( !bOK, m_pEnv->GetProgPath(m_pExecuter->GetInstallMode()) );
				m_pEnv->SetReboot( bOK );
			}
			#endif
		}
		else if (m_pExecuter->GetInstallSubMode() == ISM_DELETE)
		{
			#ifdef OS2
			if( m_pExecuter->GetCScript()->GetInstallation()->DoModifyPath() )
				Os2OS::RemoveDestPathFromConfigSys( m_pEnv->GetDestPath() );
			#endif
		}
	}

	if( !m_pAgenda->WasRunningStopped() )
	{	// have fun ...
		#ifdef OS2
		if( m_pEnv->IsFirstInstallation() &&  m_pExecuter->GetInstallMode() != IM_NETWORK )
			m_pEnv->SetReboot( TRUE );
		#endif

		if( m_aParamFollowApp.Len() != 0 )
		{
			StartApplication( UniString::CreateFromAscii(m_aParamFollowApp.GetBuffer()),
							  UniString(), UniString() );
			if( !m_pExecuter->GetNoEvent() )
				PostUserEvent(EVT_END);
		}
		else if( !m_pExecuter->GetNoEvent() )
			PostUserEvent( EVT_ENDINSTALL );
	}
	else
	{
		if( m_pEnv->IsFirstInstallation() && QueryKillEntireDir(m_pEnv->GetDestPath()) )
			KillEntireDir(m_pEnv->GetDestPath());
		if( !m_pExecuter->GetNoEvent() )
			PostUserEvent(EVT_END);
	}

	m_bPerformInstall = FALSE;

	delete m_pExecuter;
	m_pExecuter = NULL;

	delete m_pAgenda;
	m_pAgenda = NULL;
}

void SetupApp::OnEventEnd()
{
	if( pTopLevelStack && pTopLevelStack->Count() )
		PostUserEvent( EVT_TOPLEVEL_LOOP );
	else
	{
		GetAppWindow()->Hide();
		Quit();
	}
}

void SetupApp::OnEventEndInstallation()
{
	/////////////////////////////////////////////////////////////
	// end dialog or procedure
	BOOL bRetYes 	= FALSE;
	BOOL bShutdown	= m_pEnv->IsReboot() || m_pEnv->IsLogout();

	if( m_pEnv->IsPropagateWinEnvChange() )
	{
		#ifdef WNT
		WinOS::PropagateEnvironmentChange();
		#endif
	}

	if( m_pEnv->GetFollowAppList().Count() && ProcessFollowApps(TRUE) )
		m_pEnv->SetDefuseRestart( TRUE );

	if( !IsResponseMode() )
	{
		m_pMainDlg = new SetupAgentDialog( ((SetupWindow*)GetAppWindow())->GetContWin(), m_pResMan );
		m_pMainDlg->SetLanguage(GetLanguage());
		m_pMainDlg->SetCScript(m_pCompiledScript);
		m_pMainDlg->SetEnv(m_pEnv);

		if( m_pEnv->IsFirstInstallation() )
			m_pMainDlg->CreateModel_EndFirstInstallation( bShutdown );
		else
			m_pMainDlg->CreateModel_EndInstallation( bShutdown );

		m_pMainDlg->StartAgentDlg();
		m_pMainDlg->Show();

		bRetYes = m_pMainDlg->DoModal();

		delete m_pMainDlg;
		m_pMainDlg = NULL;
	}
	else
		m_pResponseFile->ExecEndProcedure();

	/////////////////////////////////////////////////////////////
	// unloader
	BOOL 		bKillDelFile = TRUE;
	ByteString 	aAbsDelFileName;

	GET_DELFILENAME(*m_pEnv, aAbsDelFileName);

	#if defined(WNT) || defined(WIN)
	if( m_pEnv->GetInstallType() == IT_UNINSTALL && (!bShutdown || !bRetYes) )
	{
		SiDirEntry aEntry( m_pEnv->GetInstalledPath() );
		aEntry += ByteString( "instdb.inf" );
		ADD_TO_DELFILE( *m_pEnv, aEntry.GetFull() );

		aEntry = m_pEnv->GetInstalledPath();
		aEntry += ByteString( "setup.log" );
		ADD_TO_DELFILE( *m_pEnv, aEntry.GetFull() );

		aEntry = m_pEnv->GetInstalledPath();
		aEntry += ByteString( "program/setup.log" );
		ADD_TO_DELFILE( *m_pEnv, aEntry.GetFull() );

		SiDirEntry aUnLoader( OS::GetGUIPath() );
		aUnLoader += ByteString( "uinst001.exe" );
		if( aUnLoader.Exists() )
		{
			StartApplication( UniString::CreateFromAscii(aUnLoader.GetFull().GetBuffer()),
							  UniString::CreateFromAscii(aAbsDelFileName.GetBuffer()),
							  UniString() );
			bKillDelFile = FALSE;
		}
	}
	#endif
	if( bKillDelFile ) {
		SiDirEntry aEntry( aAbsDelFileName );
		aEntry.Kill();
	}

	/////////////////////////////////////////////////////////////
	// reboot quit
	if( !m_pEnv->GetFollowAppList().Count() &&
		bShutdown && !m_pEnv->IsDefuseRestart() && (bRetYes || IsResponseMode()) )
	{
		BOOL bLogoutOnly = m_pEnv->IsLogout() && !m_pEnv->IsReboot();
		if( m_pEnv->IsFirstInstallation() && m_pEnv->GetInstallType() != IT_WORKSTATION )
		{
			#if defined(WIN) || defined(WNT) || defined(OS2)
			SiDirEntry aEntry( m_pEnv->GetStartPath() );
			aEntry += ByteString("shutdown.me");
			FILE* pFile = fopen( aEntry.GetFull().GetBuffer(), "w" );
			if( bLogoutOnly )
				fprintf( pFile, "logout" );
			fclose( pFile );
			#endif
		}
		else
		{
			#if defined(WIN) || defined(WNT)
			WinOS::Shutdown(bLogoutOnly);
			#elif defined(OS2)
			Os2OS::Shutdown();
			#endif
		}
	}
	else
		ProcessFollowApps();
	PostUserEvent( EVT_END );
}

// -----------------------------------------------------------------------

BOOL SetupApp::ProcessFollowApps( BOOL bTestOnly )
{
	BOOL bReturn = TRUE;
	SiFollowAppList& rLst = m_pEnv->GetFollowAppList();
	for( USHORT i = 0; i < rLst.Count(); ++i )
	{
		ByteString* pAppName = rLst.GetObject(i);
		SiDirEntry aEntry;
		if( m_pEnv->IsBigMode() ) {
			SiDirEntry aTmp( m_pEnv->GetSourcePath() );
			aEntry = aTmp.GetPath();
		}
		else
			aEntry = m_pEnv->GetSourcePath();
		aEntry += *pAppName;

		if( bTestOnly && !aEntry.Exists() )
			bReturn = FALSE;
		if( !bTestOnly && aEntry.Exists() )
			StartApplication( UniString::CreateFromAscii(aEntry.GetFull().GetBuffer()),
							  UniString(), UniString() );
	}
	return bReturn;
}
// -----------------------------------------------------------------------

BOOL SetupApp::AreAllAppsClosed(Window* pParent, USHORT nId)
{
	return TRUE;
}

void SetupApp::KillSlideshowFiles(SiCompiledScript* pCS)
{
	const SiFileList& rFileList = pCS->GetRootModule()->GetFileList();
	for( USHORT i = 0; i < pCS->GetSlideList().Count(); ++i )
	{
		SiFile* pFile           = NULL;
		ByteString aBmpName         = pCS->GetSlideList().GetObject(i)->GetBmpFileName();
		ByteString aWaveName        = pCS->GetSlideList().GetObject(i)->GetWaveFileName();

		for( USHORT x = 0; x < rFileList.Count(); ++x )
		{
			pFile = rFileList.GetObject(x);
			if( pFile &&
			   (pFile->GetName().CompareIgnoreCaseToAscii(aBmpName) == COMPARE_EQUAL ||
				pFile->GetName().CompareIgnoreCaseToAscii(aWaveName) == COMPARE_EQUAL) )
			{
				pFile->SetDontRecover( TRUE );

				SiDirEntry aFile( m_pEnv->GetDestPath() );
				aFile += pFile->GetDirectory()->GetName();
				aFile += pFile->GetName();
				aFile.ToAbs();
				aFile.Kill();
			}
		}
	}
}

// -----------------------------------------------------------------------
// Zielverzeichnis komplett loeschen; Abfragen und ausfuehren
//

BOOL SetupApp::QueryKillEntireDir(ByteString const& rPath)
{
	QueryBox aBox(GetAppWindow(), ResId(m_pEnv->IsInstallOnRoot() ? QBOX_KILLDIR_ROOT : QBOX_KILLDIR));
	String sMess = aBox.GetMessText();

	sMess.SearchAndReplace(
				UniString::CreateFromAscii("%s"),
				UniString::CreateFromAscii(rPath.GetBuffer()) );
	aBox.SetMessText(sMess);

	return aBox.Execute() == RET_YES;
}

void SetupApp::KillEntireDir(ByteString const& rPath)
{
	SiDirEntry aKillDir(rPath);
	KillEntireDir(aKillDir, TRUE);
	if( aKillDir.Exists() )
		aKillDir.Kill();
}

void SetupApp::KillEntireDir(SiDirEntry const& rEntry, BOOL bDelFiles)
{
	if( !rEntry.Exists() )
		return ;

	SiDirEntry aTmp  = rEntry;
			 aTmp += SiDirEntry( "*" );

	if (bDelFiles)
	{
	Dir aFiles(aTmp , FSYS_KIND_FILE);

	for (USHORT i = 0; i < aFiles.Count(); ++i)
		aFiles[i].Kill();
	}

	Dir aDir(aTmp, FSYS_KIND_DIR);

	for ( USHORT i = 0; i < aDir.Count(); ++i )
	{
//Changed by OBO, parse-error for linux-compiler
		ByteString aDummy( aDir[i].GetName(), osl_getThreadTextEncoding() );
		ByteString aFull( aDummy );
//end of changes. Old line:
//		ByteString aFull( ByteString(aDir[i].GetName(), osl_getThreadTextEncoding()) );
		if (aFull != "." && aFull != "..")
		{
				SiDirEntry aSubDir  = rEntry;
					 	aSubDir += aDir[i];

			KillEntireDir(aSubDir, bDelFiles);
		}
	}

	FSysError nErr = rEntry.Kill();

	if (nErr != 0)
	{
		SiDirEntry aCurrentDir;
		nErr = nErr;
	}
}

void SetupApp::InstallWorkstation(InitiateWorkstation* pWorkStation)
{
	SiDirEntry aServerInstDB( m_pCompiledScript->GetSetupExePath(*m_pEnv) );
	aServerInstDB += ByteString( SETUP_INSTDB_FILE );
	aServerInstDB.ToAbs();

	SiCompiledScript* pWCScript = new SiCompiledScript;
	if( !ReadScript(aServerInstDB, pWCScript) )
	{
		delete pWorkStation;
		delete pWCScript;
		return;
	}

	SiDirEntry aInstPath( pWorkStation->aDirectory );
	aInstPath.ToAbs();
	if( !aInstPath.Exists() )
		aInstPath.MakeDir();

	BOOL bSuccCWD = aInstPath.SetCWD();
	if( !bSuccCWD )
	{
		String aMsg( ResId(STR_APPSERVER_NORIGHTS) );
		aMsg.SearchAndReplace( UniString::CreateFromAscii("%1"), UniString::CreateFromInt32(pWorkStation->nLanguage) );
		aMsg.SearchAndReplace( UniString::CreateFromAscii("%2"), aInstPath.GetFullUni() );
		ErrorBox aBox( GetAppWindow(), WB_OK, aMsg );
		aBox.Execute();

		delete pWorkStation;
		delete pWCScript;
		PostUserEvent( EVT_INITIATE_WORKSTATION );
		return;
	}

	SiEnvironment* pWEnv = new SiEnvironment;

	pWEnv->SetFirstInstallation	( TRUE );
	pWEnv->SetInstalledPath		( m_pEnv->GetDestPath() );
	pWEnv->SetSourcePath		( m_pEnv->GetDestPath() );
	pWEnv->SetDestPath			( aInstPath.GetFull() );
	pWEnv->SetAutoWorkstation	( TRUE );

	if( pWorkStation->nLanguage != LANG_DEFAULT )
	{
		USHORT nDefLang = pWCScript->GetInstallation()->GetDefLanguage().ToInt32();
		LanguageContext* pCtx = new LanguageContext;
		pCtx->nLanguage	= pWorkStation->nLanguage == nDefLang? LANG_DEFAULT : pWorkStation->nLanguage;
		pCtx->isProg   	= TRUE;
		pCtx->isDoc		= TRUE;

		pWEnv->AddLanguageContext( pCtx );
	}

	pWCScript->PrepareForLocalInstallation();

	if( !pWCScript->GetRootModule() )
	{
		delete pWorkStation;
		delete pWCScript;
		delete pWEnv;
		PostUserEvent( EVT_INITIATE_WORKSTATION );
		return;
	}

	ByteString aTitle( GetProgrammName(), osl_getThreadTextEncoding() );
	m_pMainWin->GetLeftWin()->ShowInstInfo( TRUE, ISM_APPSERVER_USER, pWEnv,
											m_nWorkStationIdx, m_nWorkStationCnt );

	pWCScript->GetRootModule()->Select( SiModule::ALL_SEL );

	UIAgendaCallback* pUICall = new UIAgendaCallback( m_pMainWin, pWCScript, pWEnv );

	m_pAgenda = new SiAgenda;

	m_pAgenda->SetWindowHideLink( LINK(this, SetupApp, WindowHideHdl) );
	m_pAgenda->SetCallback		( pUICall );
	m_pAgenda->SetEnvironment	( pWEnv );
	m_pAgenda->CreateForInstall	( *pWCScript, IM_WORKSTATION, TRUE );

	m_pExecuter = new ThreadExecuter(m_pAgenda);

	m_pExecuter->SetWorkstation		( TRUE );
	m_pExecuter->SetSourcePath		( pWEnv->GetSourcePath() );
	m_pExecuter->SetInstallMode		( IM_WORKSTATION );
	m_pExecuter->SetInstallSubMode	( ISM_ADD );
	m_pExecuter->SetCScript			( pWCScript );
	m_pExecuter->SetUpgrade			( FALSE );
	m_pExecuter->SetNoEvent			( FALSE );

	m_bPerformInstall = TRUE;

	m_pExecuter->create();
	m_pExecuter->setPriority( NAMESPACE_VOS(OThread)::TPriority_AboveNormal );

	delete pWorkStation;
}

void SetupApp::WriteWorkstationDB()
{
	if( m_pAgenda->WasRunningStopped() )
		return ;

	SiInstallation* pInst = m_pExecuter->GetCScript()->GetInstallation();

	if( pInst )
		pInst->SetInstalledLanguages( m_pAgenda->GetEnvironment() );

	SiAnsiFileStream aStream;
	SiDatabase		 aDB(aStream);

	SiDirEntry aScriptEntry( m_pAgenda->GetEnvironment()->GetDestPath() );
	aScriptEntry += ByteString( SETUP_INSTDB_FILE );
	aStream.Open( aScriptEntry.GetFullUni(), STREAM_STD_WRITE | STREAM_TRUNC );

	if( aStream.IsOpen() )
	{
		ByteString aSource = m_pAgenda->GetEnvironment()->GetSourcePath();
		aDB.Create( m_pExecuter->GetCScript(), IM_WORKSTATION, aSource, FALSE );
	}
	aStream.Flush();
}

