/*************************************************************************
 *
 *  $RCSfile: pimode.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _INTN_HXX //autogen
#include <tools/intn.hxx>
#endif

#include "maindlg.hxx"
#include "script.hxx"
#include "environ.hxx"
#include "agenda.hxx"
//#include "script.hxx"
#include "sihelp.hxx"
#include "os.hxx"

#include "pimode.hxx"
#include "pimode.hrc"

#define PROP_MAXIMAL		1
#define PROP_MINIMAL		2
#define PROP_USERDEFINED	3
#define PROP_WORKSTATION	4

String GetSizeStr(ULONG lSize)
{
	ULONG lMB = ((1024L * 1024L) / 10L); // Umrechnung: Bytes -> 100 KB
	lSize /= lMB;

	String aSize;
	FormatTable aIntl;

	aSize  = UniString::CreateFromInt32( lSize / 10 );
	aSize += aIntl.GetNumDecimalSep();
	aSize += UniString::CreateFromInt32( lSize % 10 );

	return aSize;
}

PageInstallMode::PageInstallMode( SvAgentDlg* pParent, const ResId& rResId ) :
	SvAgentPage		( pParent, rResId ),
	aFTInfo1		( this, ResId(FT_INFO1, rResId.GetResMgr()) ),
	aRBMaximal		( this, ResId(RB_MAXIMAL, rResId.GetResMgr()) ),
	aRBUserDefined	( this, ResId(RB_USERDEFINED, rResId.GetResMgr()) ),
	aRBMinimal		( this, ResId(RB_MINIMAL, rResId.GetResMgr()) ),
	aRBWorkstation	( this, ResId(RB_WORKSTATION, rResId.GetResMgr()) ),
	aFTMaximal		( this, ResId(FT_MAXIMAL, rResId.GetResMgr()) ),
	aFTUserDefined	( this, ResId(FT_USERDEFINED, rResId.GetResMgr()) ),
	aFTMinimal		( this, ResId(FT_MINIMAL, rResId.GetResMgr()) ),
	aFTWorkstation	( this, ResId(FT_WORKSTATION, rResId.GetResMgr()) ),
	aStr_NetRBTextMax( ResId(STR_RB_TEXT_MAX, rResId.GetResMgr()) ),
	aStr_NetFTTextMax( ResId(STR_FT_TEXT_MAX, rResId.GetResMgr()) )
{
	SiEnvironment* pEnv = GETDLG()->GetEnv();

	String aTitle( ResId(STR_TITLE, rResId.GetResMgr()) );
	aTitle.SearchAndReplace( UniString::CreateFromAscii("%1"),
			UniString::CreateFromAscii(pEnv->GetProductName().GetBuffer()) );
	pParent->SetTitleStr( aTitle );

	FreeResource();

	Font aBold( aRBMaximal.GetFont() );
	aBold.SetWeight(WEIGHT_BOLD);

	aRBMaximal.		SetFont(aBold);
	aRBUserDefined.	SetFont(aBold);
	aRBMinimal.		SetFont(aBold);
	aRBWorkstation.	SetFont(aBold);

	String aProductName( UniString::CreateFromAscii(pEnv->GetProductName().GetBuffer()) );
	String aText;

	aText = aFTMaximal.GetText();
	aText.SearchAndReplace( UniString::CreateFromAscii("%s"), aProductName );
	aFTMaximal.SetText( aText );

	aText = aFTUserDefined.GetText();
	aText.SearchAndReplace( UniString::CreateFromAscii("%s"), aProductName );
	aFTUserDefined.SetText( aText );

	aStr_NetFTTextMax.SearchAndReplace( UniString::CreateFromAscii("%s"), aProductName );

	SiInstallation* pInst = GETDLG()->GetCScript()->GetInstallation();
	if( pInst != NULL && pInst->GetInstallMode() != IM_NETWORK )
	{
		aRBWorkstation.Hide();
		aFTWorkstation.Hide();
	}
	else
	{
		aRBUserDefined.Hide();
		aFTUserDefined.Hide();
		aRBMinimal.Hide();
		aFTMinimal.Hide();

		if( pInst->IsHideStandard() )
		{
			aRBMaximal.Hide();
			aFTMaximal.Hide();
		}
		else
		{
			aRBMaximal.SetText( aStr_NetRBTextMax );
			aFTMaximal.SetText( aStr_NetFTTextMax );

			#define DEF_Y_OFFSET 80

			Point aPt( aRBMaximal.GetPosPixel() );
			aPt.Y() += DEF_Y_OFFSET;
			aRBMaximal.SetPosPixel( aPt );

			aPt = aFTMaximal.GetPosPixel();
			aPt.Y() += DEF_Y_OFFSET;
			aFTMaximal.SetPosPixel( aPt );
			aRBMaximal.Check(TRUE);
		}
	}

	InitSizeStr();
}

PageInstallMode::~PageInstallMode()
{
}

SvAgentPage* PageInstallMode::Create( SvAgentDlg* pParent, const ResId& rResId )
{
	return new PageInstallMode( pParent, rResId );
}

ULONG PageInstallMode::GetProperty()
{
	SiEnvironment* pEnv = GETDLG()->GetEnv();
	SiCompiledScript* pCS = GETDLG()->GetCScript();

	SiModule* pRoot = GETDLG()->GetCScript()->GetRootModule();
	DBG_ASSERT( pRoot, "no rootmodule" );

	SiInstallType eOldType = pEnv->GetInstallType();
	ULONG nPropReturn = PROP_NOTSET;

	// Maximal Installation
	if(	aRBMaximal.IsChecked() )
	{
		if( pRoot )
			pRoot->Select( SiModule::ALL_DEFAULT );
		pEnv->SetInstallType( IT_MAXIMAL );
		nPropReturn = PROP_MAXIMAL;
	}
	// Userdefined Installation
	else if( aRBUserDefined.IsChecked() )
	{
		if( pRoot )
			pRoot->Select( SiModule::ALL_DEFAULT );
		pEnv->SetInstallType( IT_USERDEFINED );
		nPropReturn = PROP_USERDEFINED;
	}
	// Minimal Installation
	else if( aRBMinimal.IsChecked() )
	{
		if( pRoot )
			pRoot->Select( SiModule::ALL_MINIMAL );
		pEnv->SetInstallType( IT_MINIMAL );
		nPropReturn = PROP_MINIMAL;
	}
	// Workstation Installation
	else if( aRBWorkstation.IsChecked() )
	{
		if( pRoot )
			pRoot->Select( SiModule::ALL_SEL );
		pEnv->SetInstallType( IT_WORKSTATION );
		nPropReturn = PROP_WORKSTATION;
	}

	if( eOldType != pEnv->GetInstallType() )
	{	// wenn der Installationstype geaendert wurde mussen die Properties der
		// Pathseite geloescht werden, damit sie eine erneute Korrektur auf das
		// erste Volume mit genuegend Platz faehrt.
		Model* pModel = GetAgentDlg()->GetModel( RESID_PAGE_PAGEINSTPATH );
		if( pModel )
			pModel->SetProperty( PROP_NOTSET );
	}

	return nPropReturn;
}

void PageInstallMode::InitProperty(ULONG nProp)
{
	SiInstallation* pInst = GETDLG()->GetCScript()->GetInstallation();
	BOOL bIsNet = (pInst != NULL && pInst->GetInstallMode() == IM_NETWORK)? TRUE : FALSE;

	if( (nProp == PROP_NOTSET && !bIsNet) ||
		nProp == PROP_MAXIMAL )
		aRBMaximal.Check(TRUE);
	else if( nProp == PROP_MINIMAL )
		aRBMinimal.Check();
	else if( nProp == PROP_USERDEFINED )
		aRBUserDefined.Check();
	else if( (nProp == PROP_NOTSET && bIsNet) ||
			 nProp == PROP_WORKSTATION )
		aRBWorkstation.Check();
}

void PageInstallMode::InitSizeStr()
{
	SiEnvironment* pEnv = GETDLG()->GetEnv();
	ULONG				lCluster = OS::GetClusterSize( SiDirEntry(OS::GetGUIPath()) );
	ULONG				lRealCluster;
	SiCompiledScript*	pCScript = GETDLG()->GetCScript();
	SiModule*			pRoot;
	String				aText;

	BOOL bIsFat = FALSE;
	#ifdef OS2
	bIsFat = Os2OS::IsFATFileSystem( OS::GetGUIPath() );
	#endif

	if( !pCScript ||
		(pRoot = pCScript->GetRootModule()) == NULL )
		return;

	pRoot->Select( SiModule::ALL_DEFAULT );
	ULONG lDefault = pRoot->CalculateSize( *pEnv, SiModule::DEFAULT, lCluster, FALSE, FALSE, bIsFat );

	lRealCluster = SiHelp::GetClusterSizeForFirstFreeVolume( lDefault, bIsFat );
	if( lRealCluster && lRealCluster != lCluster )
		lDefault = pRoot->CalculateSize( *pEnv, SiModule::DEFAULT, lRealCluster, FALSE, FALSE, bIsFat );

	#ifdef OS2
	bIsFat = Os2OS::IsFATFileSystem( OS::GetGUIPath() );
	#endif
	ULONG lTemp = pRoot->CalculateSize( *pEnv, SiModule::TEMP, lCluster, FALSE, FALSE, bIsFat );

	pRoot->Select( SiModule::ALL_MINIMAL );
	ULONG lMinimal = pRoot->CalculateSize( *pEnv, SiModule::MINIMAL, lCluster, FALSE, FALSE, bIsFat );

	lRealCluster = SiHelp::GetClusterSizeForFirstFreeVolume( lMinimal, bIsFat );
	if( lRealCluster && lRealCluster != lCluster )
		lMinimal = pRoot->CalculateSize( *pEnv, SiModule::MINIMAL, lRealCluster, FALSE, FALSE, bIsFat );

	#ifdef OS2
	bIsFat = Os2OS::IsFATFileSystem( OS::GetGUIPath() );
	#endif

	ULONG lTempMini = pRoot->CalculateSize( *pEnv, SiModule::TEMP, lCluster, FALSE, FALSE, bIsFat );
	ULONG lWork = pRoot->CalculateSize( *pEnv, SiModule::DEFAULT, lCluster, FALSE, TRUE, bIsFat );

	lDefault += GETDLG()->GetEnv()->GetMigrationSize();
	lMinimal += GETDLG()->GetEnv()->GetMigrationSize();
	lWork	 += GETDLG()->GetEnv()->GetMigrationSize();

	aText = aFTMaximal.GetText();
	aText.SearchAndReplace( UniString::CreateFromAscii("%1"), GetSizeStr(lDefault) );
	aText.SearchAndReplace( UniString::CreateFromAscii("%2"), GetSizeStr(lDefault+lTemp) );
	aFTMaximal.SetText( aText );

	aText = aFTMinimal.GetText();
	aText.SearchAndReplace( UniString::CreateFromAscii("%1"), GetSizeStr(lMinimal) );
	aText.SearchAndReplace( UniString::CreateFromAscii("%2"), GetSizeStr(lMinimal+lTempMini) );
	aFTMinimal.SetText( aText );

	aText = aFTWorkstation.GetText();
	aText.SearchAndReplace( UniString::CreateFromAscii("%1"), GetSizeStr(lWork) );
	aFTWorkstation.SetText( aText );
}

