/*************************************************************************
 *
 *  $RCSfile: pipath.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:51:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_SOUND_HXX //autogen
#include <vcl/sound.hxx>
#endif

#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif

#ifndef _SISYS_HXX
#include <sifsys.hxx>
#endif

#ifndef _SVT_FILEDLG_HXX //autogen
#include <svtools/filedlg.hxx>
#endif

#include "maindlg.hxx"
#include "os.hxx"
#include "script.hxx"
#include "environ.hxx"
#include "sihelp.hxx"
#include "action.hxx"

#include "maindlg.hrc"
#include "pipath.hxx"
#include "pipath.hrc"

PageInstPath::PageInstPath( SvAgentDlg* pParent, const ResId& rResId ) :
	SvAgentPage		( pParent, rResId ),
	aFTInfo1		( this, ResId(FT_INFO1, rResId.GetResMgr()) ),
	aEDPath			( this, ResId(ED_PATH, rResId.GetResMgr()) ),
	aBtnBrowse		( this, ResId(BTN_INSTPATH_BROWSE, rResId.GetResMgr()) )
{
	String aText( ResId(STR_TITLE, rResId.GetResMgr()) );
	pParent->SetTitleStr( aText );
	FreeResource();

	m_lClusterSize = 0L;
	m_lClusterSizeSystem = 0L;

	pEnv = GETDLG()->GetEnv();

	aText = aFTInfo1.GetText();
	aText.SearchAndReplace( UniString::CreateFromAscii("%1"),
			UniString::CreateFromAscii(pEnv->GetProductName().GetBuffer()) );
	aText.SearchAndReplace( UniString::CreateFromAscii("%2"), aBtnBrowse.GetText() );
	aFTInfo1.SetText( aText );

	aBtnBrowse.SetClickHdl( LINK(this, PageInstPath, BrowseHdl) );
}

PageInstPath::~PageInstPath()
{
}

SvAgentPage* PageInstPath::Create( SvAgentDlg* pParent, const ResId& rResId )
{
	return new PageInstPath( pParent, rResId );
}

BOOL PageInstPath::AllowNext()
{
	String aStr( aEDPath.GetText() );
	if( !aStr.Len() ) {
		ErrorBox( this, WB_OK, String(ResId(STR_INVALIDPATH)) ).Execute();
		return FALSE;
	}

	SiDirEntry aPath( aStr );
	aPath.ToAbs();

	SiDirEntry aSource( pEnv->GetSourcePath() );
	aSource.ToAbs();

	if( aPath == aSource )
	{
		ErrorBox( this, WB_OK, String(ResId(STR_PATHSAMEASSOURCE)) ).Execute();
		return FALSE;
	}

	if( pEnv->DoMigration() )
	{
		SiDirEntry aMigPath( pEnv->GetMigrationPath() );
		aMigPath.ToAbs();
		if( aPath == aMigPath )
		{
			ErrorBox( this, WB_OK, String(ResId(STR_PATHSAMEASMIGRATION)) ).Execute();
			return FALSE;
		}
	}

	/* -----------------12/19/97 3:01PM------------------
	fehlerhafter Path
	 --------------------------------------------------*/
	ByteString aPathStr( aPath.GetFull() );
	if( !aPath.IsValid() ||
		aPathStr.Search('?') != STRING_NOTFOUND ||
		aPathStr.Search('*') != STRING_NOTFOUND )
	{
		String aMessage = ResId( STR_WRONGPATH );
		aMessage.SearchAndReplace( UniString::CreateFromAscii("%1"),
				UniString::CreateFromAscii(aPathStr.GetBuffer()) );

		ErrorBox( this, WB_OK, aMessage ).Execute();
		return FALSE;
	}

	/* -----------------12/19/97 3:01PM------------------
	Installation in die Root ?
	 --------------------------------------------------*/
	if( aPath.Level() == 1 )
	{
		QueryBox aBox( this, WB_YES_NO | WB_DEF_NO, String(ResId(STR_INSTALL_ON_ROOT)) );
		if( aBox.Execute() != RET_YES )
			return FALSE;
		GETDLG()->GetEnv()->SetInstallOnRoot(TRUE);
	}

	/* -----------------12/19/97 3:02PM------------------
	genug Patz? nur wenn != Userdefined
	 --------------------------------------------------*/
	if( pEnv->GetInstallType() != IT_USERDEFINED )
	{
		RecalcSize( aPath );

		SiInstallation*	pInst = GETDLG()->GetCScript()->GetInstallation();
		ByteString			aDestVol( aPath.GetFull() );
		ByteString			aGUIVol( OS::GetGUIPath() );
		aDestVol.ToLowerAscii();
		aGUIVol.ToLowerAscii();

		if( aDestVol.GetChar(0) != aGUIVol.GetChar(0) &&
			pInst->GetInstallMode() != IM_NETWORK )
		{
			// Ziellaufwerk != Systemlaufwerk Pruefung aufteilen
			ULONG nUsedSize = (m_lUsedSize + m_lTempSize) - m_lSystemSize;
			if( OS::GetDriveSize(aPath) < (nUsedSize/1024) )
			{
				ErrorBox( this, WB_OK, String(ResId(STR_FULLDRIVE)) ).Execute();
				return FALSE;
			}
			else if( OS::GetDriveSize(SiDirEntry(OS::GetSystemPath())) < (m_lSystemSize/1024) )
			{
				String aInfo( ResId( STR_FULLSYSTEMDRIVE1 ) );
				aInfo += UniString::CreateFromAscii( OS::GetSystemPath().ToUpperAscii().Copy(0, 2).GetBuffer() );
				aInfo += String( ResId( STR_FULLSYSTEMDRIVE2 ) );
				aInfo += m_lSystemSize / 1024;	// in KB
				aInfo += String( ResId( STR_FULLSYSTEMDRIVE3 ) );

				ErrorBox( this, WB_OK, aInfo ).Execute();
				return FALSE;
			}
		}
		else if( OS::GetDriveSize(aPath) < ((m_lUsedSize + m_lTempSize) /1024) )
		{
			ErrorBox( this, WB_OK, String(ResId(STR_FULLDRIVE)) ).Execute();
			return FALSE;
		}
	}

	SiDirEntry aOldPath( pEnv->GetDestPath() );
	if( aOldPath != aPath && pEnv->IsDlgPrecreateDir() )
		aOldPath.Kill();

	if( !aPath.Exists() )
	{
		String aMessage = ResId( STR_PATH_NOTEXISTS );
		aMessage.SearchAndReplace( UniString::CreateFromAscii("%1"), aPath.GetFullUni() );
		QueryBox aBox( GetAgentDlg(), WB_YES_NO | WB_DEF_YES, aMessage );
		if( aBox.Execute() == RET_NO )
		{
			Sound::Beep();
			return FALSE;
		}
		if( !aPath.MakeDir() )
		{
			String aMessage(ResId(STR_ERROR_MAKEDIR));
			aMessage.SearchAndReplace( UniString::CreateFromAscii("%1"), aPath.GetFullUni() );
			ErrorBox( this, WB_OK, aMessage ).Execute();
			return FALSE;
		}
		else
			pEnv->SetDlgPrecreateDir(TRUE);
	}
	else
		pEnv->SetDlgPrecreateDir(FALSE);

	pEnv->SetDestPath( aPath.GetFull() );
	return TRUE;
}

USHORT PageInstPath::GetReturnVal()
{
	SiCompiledScript* pCS = GETDLG()->GetCScript();

	if( pEnv->GetInstallType() == IT_USERDEFINED ||
		pCS->GetInstallation()->IsOnlyCustom() )
		return RETURN_USERDEFINED;

	return RETURN_NORMAL;
}

ULONG PageInstPath::GetProperty()
{
	return 1;
}

void PageInstPath::InitProperty(ULONG nProp)
{
	if( pEnv->GetProductName() == "UpdateTest" )
	{
		SiDirEntry aEntry( pEnv->GetDestPath() );
		aEntry.ToAbs();
		pEnv->SetDestPath( aEntry.GetFull() );
		RecalcSize( pEnv->GetDestPath() );
	}
	else
	{
		RecalcSize( pEnv->GetDestPath() );
		if( nProp == PROP_NOTSET )
		{
			SiCompiledScript* pCS = GETDLG()->GetCScript();
			if( pCS->GetInstallation() && !GETDLG()->GetEnv()->IsParamDestPath() )
				// ist der DestPath per Parameter gesetzt wird nicht corrected!
				CorrectToFirstFree();
		}
	}

	aEDPath.SetText( UniString::CreateFromAscii(pEnv->GetDestPath().GetBuffer()) );
}

void PageInstPath::CorrectToFirstFree( BOOL bScanRemoveableDisks )
{
	#if defined(WIN) || defined(OS2) || defined(WNT)

	BOOL bFound = FALSE;
	Dir aDir( SiDirEntry( "*" ), FSYS_KIND_BLOCK);
	String sFull;

	for( USHORT i = 0; i < aDir.Count(); i++ )
	{
		FileStat	aStat( aDir[i] );

		sFull = aDir[i].GetFull();
		sFull.ToUpperAscii();

		if( sFull.GetChar(0) != 'A'	&&
			sFull.GetChar(0) != 'B' &&
			(aStat.IsKind(FSYS_KIND_FIXED)  ||
			 aStat.IsKind(FSYS_KIND_REMOTE) ||
			(aStat.IsKind(FSYS_KIND_REMOVEABLE) && bScanRemoveableDisks)) )
		{
			if( OS::GetDriveSize(aDir[i]) > ((m_lUsedSize + m_lTempSize) / 1024) )
			{
				bFound = TRUE;
				break;
			}
		}
	}

	if( bFound )
	{
		ByteString aInstPath( pEnv->GetDestPath() );
		aInstPath.Erase( 0, 2 );
		aInstPath.Insert( ByteString(sFull, osl_getThreadTextEncoding()), 0 );
		pEnv->SetDestPath( aInstPath );
	}

	#endif
}

void PageInstPath::RecalcSize( const SiDirEntry& rDest )
{
	SiModule*		pRoot = GETDLG()->GetCScript()->GetRootModule();
	ULONG			lClusterSize = OS::GetClusterSize( rDest );
	BOOL 			bIsFat = FALSE;
	BOOL 			bIsSysFat = FALSE;

	#ifdef OS2
	bIsFat		= Os2OS::IsFATFileSystem( rDest.GetFull() );
	bIsSysFat	= Os2OS::IsFATFileSystem( OS::GetGUIPath() );
	#endif

	if( lClusterSize != m_lClusterSize )
	{
		m_lClusterSize = lClusterSize;
		m_lUsedSize    = pRoot->CalculateSize( *pEnv, SiModule::INSTALL, lClusterSize,
						 FALSE, pEnv->GetInstallType() == IT_WORKSTATION, bIsFat );
		m_lUsedSize    += pEnv->GetMigrationSize();
	}

	ULONG lClusterSizeSys = OS::GetClusterSize( SiDirEntry(OS::GetGUIPath()) );
	if( lClusterSizeSys != m_lClusterSizeSystem )
	{
		m_lClusterSizeSystem = lClusterSizeSys;
		m_lSystemSize = pRoot->CalculateSize( *pEnv, SiModule::INSTALL, lClusterSizeSys,
						TRUE, pEnv->GetInstallType() == IT_WORKSTATION, bIsSysFat );
	}

	m_lTempSize = pRoot->CalculateSize( *pEnv, SiModule::TEMP, lClusterSize, FALSE,
		pEnv->GetInstallType() == IT_WORKSTATION, bIsFat );
}

IMPL_LINK( PageInstPath, BrowseHdl, Button*, pBtn )
{
	SiDirEntry aPath( aEDPath.GetText() );
	SiDirEntry aMyPath( GETDLG()->GetEnv()->GetStartPath() );

	if( !aPath.Exists() ||
		aPath == aMyPath )
	{
		aPath = aPath.GetDevice();
		aPath.ToAbs();
	}

	SiDirEntry aOldPath;
	aPath.SetCWD();

	PathDialog aDlg( GetAgentDlg() );
	aDlg.SetPath( aPath.GetFullUni() );

	if( aDlg.Execute() == RET_OK )
	{
		pEnv->SetDestPath( ByteString(aDlg.GetPath(), osl_getThreadTextEncoding()) );
		aEDPath.SetText( UniString::CreateFromAscii(pEnv->GetDestPath().GetBuffer()) );
	}

	aOldPath.SetCWD();
	return 1;
}
