/*************************************************************************
 *
 *  $RCSfile: evntconf.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SFX_EVENTCONF_HXX
#define _SFX_EVENTCONF_HXX

#define _SVSTDARR_USHORTS
#include <svtools/svstdarr.hxx>		// SvUShorts
#include "cfgitem.hxx"
#include "event.hxx"

#include "sfxsids.hrc"
#ifdef ITEMID_MACRO
#undef ITEMID_MACRO
#endif
#define ITEMID_MACRO SID_ATTR_MACROITEM
#include <svtools/macitem.hxx>

class SfxMacroInfo;
class SfxMacroTabPage;
class SfxMacroInfoArr_Impl;
class SfxEventConfigItem_Impl;
class SfxEventInfoArr_Impl;
class SfxObjectShell;

#define SFX_NO_EVENT USHRT_MAX

#if __PRIVATE
struct SfxEvent_Impl
{
	String						aEventName;
	USHORT						nEventId;

	SfxEvent_Impl(const String& rName, USHORT nId) :
		aEventName(rName),
		nEventId(nId)
	{}
};

SV_DECL_PTRARR(SfxEventArr_Impl, SfxEvent_Impl*, 5, 5);
#else
typedef SvPtrarr SfxEventArr_Impl;
#endif

class SfxEventConfiguration
{
friend class SfxEventConfigItem_Impl;

	SfxMacroInfoArr_Impl*	pArr;
	SfxEventArr_Impl*		pEventArr;
	SfxEventConfigItem_Impl* pAppEventConfig;
	SfxEventConfigItem_Impl* pDocEventConfig;

public:
							SfxEventConfiguration();
							~SfxEventConfiguration();

	void					RegisterEvent(USHORT nId, const String& rName);
	const String&			GetEventName(USHORT nId) const;
	USHORT					GetEventId(const String& rName) const;
	USHORT					GetEventCount() const
							{ return pEventArr->Count()-1; }
	const SvxMacro*			GetMacroForEventId( USHORT nEventId, SfxObjectShell *pObjSh );
	void					ConfigureEvent(USHORT nId, const SvxMacro&,
								SfxObjectShell* pObjSh);
	void					ConfigureEvent(USHORT nId, const String& rMacro,
								SfxObjectShell* pObjSh);

	const SfxMacroInfo*		GetMacroInfo(USHORT nEventId, SfxObjectShell* pObjSh) const;

	void					ExecuteEvent(
		USHORT nId, SfxObjectShell *pDoc, FASTBOOL bSynchron,
		const String& rArgs );

	void					SetWarningEnabled( BOOL bOn );
	void					SetWarningForced( BOOL bOn );
	BOOL					IsWarningEnabled() const;
	BOOL					IsWarningForced() const;
	void 					AddEvents( SfxMacroTabPage* ) const;
	SvxMacroTableDtor*		GetAppEventTable();
	SvxMacroTableDtor*		GetDocEventTable(SfxObjectShell*);
	void					SetAppEventTable( const SvxMacroTableDtor& );
	void					SetDocEventTable( SfxObjectShell*, const SvxMacroTableDtor& );

#if __PRIVATE
	const SfxEvent_Impl*	GetEvent_Impl(USHORT nPos) const
							{ return (*pEventArr)[nPos+1]; }
	BOOL					Warn_Impl( SfxObjectShell *pDoc, const SvxMacro* pMacro );
#endif
};

#if __PRIVATE
class SfxEventConfigItem_Impl : public SfxConfigItem
{
friend class SfxEventConfiguration;

	SfxEventConfiguration	*pEvConfig;
	BOOL					bWarning;
	BOOL					bAlwaysWarning;
	SvxMacroTableDtor		aMacroTable;

	void					ConstructMacroTable();

public:
							SfxEventConfigItem_Impl( USHORT,
								SfxEventConfiguration* );
							~SfxEventConfigItem_Impl();

	void					Init( SfxConfigManager* );
	virtual int 			Load(SvStream&);
	virtual BOOL			Store(SvStream&);
	virtual BOOL			Reconfigure(SvStream&, BOOL bDefault=FALSE);
	virtual String			GetName() const;
	virtual void			UseDefault();
	void					ConfigureEvent( USHORT nPos, SvxMacro* );
};
#endif

#endif
