/*************************************************************************
 *
 *  $RCSfile: explorer.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _EXPLORER_HXX
#define _EXPLORER_HXX

// includes ******************************************************************

#ifndef _RTTI_HXX //autogen
#include <tools/rtti.hxx>
#endif

#ifndef _ERRCODE_HXX //autogen
#include <tools/errcode.hxx>
#endif

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif

#ifndef _IMAGE_HXX //autogen
#include <vcl/image.hxx>
#endif


//****************************************************************************
// Frames
//****************************************************************************

/*
  [Beschreibung]
  Die Frames geben an, wo eine Instanz der Klasse <SfxExplorerContent>
  geoeffnet werden soll:

  EXPLORER_TOPFRAME     im aktuellen Dokument
  EXPLORER_BLANKFRAME   in einem neuen Dokument
  EXPLORER_DOCUMET      in einem Dokument (EXPLORER_SELFFRAME)
  EXPLORER_EXPLORER     im Explorer-Childwindow
  EXPLORER_BROWSER      im Browser-Childwindow
  EXPLORER_SELFFRAME    im aktuellen Frame
  EXPLORER_EXPLORERROOT im Explorer-Childwindow als Ursprung
  EXPLORER_PRINTER      auf dem Drucker (ausdrucken)
  EXPLORER_SEARCHFRAME  im Suchdialog => wird auf EXPLORER_SELFFRAME umgeleitet
  EXPLORER_FILEDIALOG   im Dialog zum oeffnen von Dateien

  [Querverweise]
  <SfxExplorerContent::Open( const String& )>
  <SfxExplorerContent::GetViewFromContext( const String& )>
  */

// spezielles "_blank f"ur ExplorerContents, da diese beim Open nie einen
// Frame mitschicken und so nicht zwischen "Neue Task" und "Task splitten"
// unterschieden werden kann ( MBA )
#define EXPLORER_NOFRAME       "_null"
#define EXPLORER_TOPFRAME      "_top"
#define EXPLORER_BLANKFRAME    "_blank"
#define EXPLORER_BROWSER       "_browser"
#define EXPLORER_EXPLORER      "_explorer"
#define EXPLORER_DOCUMENT      "_document"
#define EXPLORER_SELFFRAME     "_smartself"
#define EXPLORER_EXPLORERROOT  "_explorerroot"
#define EXPLORER_SEARCHFRAME   "_explorersearch"
#define EXPLORER_PRINTER       "_explorerprinter"
#define EXPLORER_FILEDIALOG    "_filedialog"

//===========================================================================

#define INTERNETSHORTCUT_EXT          "url"
#define INTERNETSHORTCUT_ID_TAG       "InternetShortcut"
#define INTERNETSHORTCUT_URL_TAG      "URL"
#define INTERNETSHORTCUT_PASS_TAG      "Password"
#define INTERNETSHORTCUT_TITLE_TAG    "Title"
#define INTERNETSHORTCUT_TARGET_TAG   "Target"
#define INTERNETSHORTCUT_FOLDER_TAG   "Folder"
#define INTERNETSHORTCUT_FRAME_TAG    "Frame"
#define INTERNETSHORTCUT_BROWSER_TAG  "Browser"
#define INTERNETSHORTCUT_OPENAS_TAG   "OpenAs"
#define INTERNETSHORTCUT_TEMPLATE_TAG "Template"
#define INTERNETSHORTCUT_DEFTEMPL_TAG "DefaultTemplate"
#define INTERNETSHORTCUT_DEFURL_TAG   "DefaultURL"

#define URLFILE_CHARSET               CHARSET_ANSI

enum SfxExplorerModifyImage
{
	EXPLORER_MODIFYIMAGE_LINK = 0,
	EXPLORER_MODIFYIMAGE_INDEX,
	EXPLORER_MODIFYIMAGE_NOT
};

class SfxExplorer
{
private:
	ImageList*					_pSmallImageList;
	ImageList*                  _pFirstBigImageList;
	ImageList*                  _pSecondBigImageList;
	ImageList*                  _pSmallMergeImageList;
	ImageList*                  _pBigMergeImageList;

public:
								SfxExplorer();
								~SfxExplorer();

	static Image 				GetImage( USHORT nId, BOOL bBig );
	static Image				ModifyImage( const Image& rImage,
											 SfxExplorerModifyImage
											 eModifier );

	static ErrCode				ImportBookmarkFile( const String &rFromURL,
													const String &rToURL );

	static String				GetExplorerRoot();
	static void					SetExplorerRoot( const String& rNewRoot );
	static BOOL					HasTempExplorerRoot();
};


#endif

