/*************************************************************************
 *
 *  $RCSfile: odkclie.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef AIX
#include <stdlib.h>
#else
#include <malloc.h>
#endif

#include <stdarg.h>

#include <tools/debug.hxx>
#include <uno/api.h>
#include <uno/uno.h>
#include <vcl/system.hxx>
#include <tools/time.hxx>
#include <odkinter.h>
#include <odkclie.hxx>
#include <vos/process.hxx>

using namespace vos;

#ifndef GCC
UNO_DEFINE_XINTERFACE()
UNO_DEFINE_XFACTORY()
#endif

static UNO_Environment hEnv = NULL;
static UNO_Server hServer = NULL;
ODKCallbackFunc pCallback = NULL;
XODKDispatch xODKServer;
static double nDouble = 0.0;
NAMESPACE_VOS( OMutex )* pMutex = 0;

static NAMESPACE_VOS( OMutex )* GetMutex()
{
	NAMESPACE_VOS( OGuard ) aGuard( NAMESPACE_VOS(OMutex)::getGlobalMutex() );
	if( !pMutex )
		pMutex = new NAMESPACE_VOS( OMutex );
	return pMutex;
}

class MyDispatcherThread : public NAMESPACE_VOS(OThread)
{
	UNO_Environment m_hEnv;
public:
					MyDispatcherThread(UNO_Environment hEnv);
					~MyDispatcherThread();

	virtual void 	run();
	void 			onTerminated();
};

MyDispatcherThread::MyDispatcherThread(UNO_Environment hEnv)
	: m_hEnv(hEnv)
{
}

void MyDispatcherThread::run()
{
	while(schedule() && UNO_waitForPendingRequest(m_hEnv, -1))
		UNO_dispatch(m_hEnv, 0);
}

MyDispatcherThread::~MyDispatcherThread()
{
	// Mein eigenes Environment abr"aumen
	NAMESPACE_VOS( OGuard ) aGuard( GetMutex() );
	if ( pCallback )
		(*pCallback)();
	UNO_destroy(m_hEnv);
}

void MyDispatcherThread::onTerminated()
{
	delete this;
}

BOOL _EXTLIBCALL_ ODKConnect( const char *pAppName, ODKCallbackFunc pFunc )
{
	pCallback = pFunc;
	cStr server = "local://ODK";
	NAMESPACE_VOS(OProcess) aProcess( pAppName, 0 );
	if ( aProcess.execute( OProcess::TOption_Detached ) == OProcess::E_None )
	{
		hEnv = UNO_initialize(0);
		ULONG nTicks = Time::GetSystemTicks();
		ULONG nStartTicks = nTicks;
		while ( 1 )
		{
			ULONG nActualTicks = Time::GetSystemTicks();
			if ( nActualTicks > nStartTicks + 60000L )
			{
				// Maximal eine Minute probieren
				break;
			}
			else if ( nActualTicks > nTicks )
			{
				// Einmal pro Sekunde nachsehen, ob der Server da ist
				nTicks += 1000L;
				hServer	= UNO_registerServer( hEnv, server );
				if ( !hServer )
					continue;
				else
				{
					try
					{
						if ( UNO_createObject( hEnv, hServer, "odk.stardiv.de", UNO_RUIK(XODKDispatch), xODKServer ) )
						{
							(new MyDispatcherThread(hEnv))->create();
							return TRUE;
						}
					}
					catch ( OUnoSystemException& e)
					{
						// Der Server ist abgeflogen
						xODKServer = XODKDispatch();
					}
				}
			}
		}
	}

	return FALSE;
}

void _EXTLIBCALL_ ODKDisconnect()
{
	NAMESPACE_VOS( OGuard ) aGuard( GetMutex() );
	xODKServer = XODKDispatch();
}

static long ODKCall ( unsigned long nFunctionNumber, void *pHandle, const char *pParamString, char cRet, va_list var );
long CDECL ODKCallFunction( unsigned long nFunctionNumber, void *pHandle, const char *pParamString, char cRet, ...)
{
	NAMESPACE_VOS( OGuard ) aGuard( GetMutex() );
	long nRetVal;
	va_list var;
	va_start( var, cRet );
	nRetVal = (long) ODKCall ( nFunctionNumber, pHandle, pParamString, cRet, var );
	va_end( var );
	return nRetVal;
}

static long ODKCall ( unsigned long nFunctionNumber, void *pHandle, const char *pParamString, char cRet, va_list var )
{
	long nRetVal = 0L;
	if ( !UNO_isNull( xODKServer ) )
	{
		const char *pTmp = pParamString;
		USHORT nLen = pTmp ? strlen( pTmp ) : 0;
		OUnoAny *pVar = new OUnoAny[ nLen ];
		OUnoAny *pVarStart = pVar;
		while ( pTmp && *pTmp )
		{
			switch ( *pTmp )
			{
				case 'o' : // Das war ein object handle; das geht nicht mit SBX!
				{
					ObjectHandle aHandle;
					aHandle.nHandle= (long) va_arg(var,long);
					*pVar <<= aHandle;
					break;
				}

				case 'h' : // Das war ein unsigned short
					*pVar <<= (unsigned short) va_arg(var,unsigned short);
				break;

				case 'i' : // Das war ein integer
					*pVar <<= (int) va_arg(var,unsigned short);
				break;

				case 'f' : // Das war ein float
					*pVar <<= (float) va_arg(var,unsigned short);
				break;

				case 'F' : // Das war ein double
					*pVar <<= (double) va_arg(var,unsigned short);
				break;

				case 'b' : // Das war ein bool
					*pVar <<= (BOOL) va_arg(var,unsigned short);
				break;

				case 'l' : // Das war ein unsigned long
					*pVar <<= (unsigned long) va_arg(var,unsigned long);
				break;

				case 'c' : // Das war ein char
					*pVar <<= (char) va_arg(var,unsigned long);
				break;

				case 's' : // Das war ein const char *
				{
					const char *pTmp = (const char *) va_arg(var,const char *);
					if ( pTmp )
						*pVar <<= OUnoString( pTmp );
					else
						*pVar <<= OUnoString( "" );
					break;
				}

				default:
					DBG_ERROR("Unbekannte Parser-Konstante!");
					break;
			}

			pTmp++;
			pVar++;
		}

		OUnoSequence < OUnoAny > aArray( pVarStart, nLen, FALSE );

		try
		{
			OUnoAny aAny = xODKServer.dispatch( nFunctionNumber, (long) pHandle, aArray );

			float fRetVal = (float) 0.0;
			OUnoString aStr;
			ObjectHandle aHandle;
			if ( aAny >>= aStr )
			{
				if ( strlen( aStr ) )
				{
					String aString( aStr );
					if ( cRet != 's' )
					{
						// Numerischer Parameter als String "ubergeben
						switch ( cRet )
						{
							case 'l' :
								nRetVal = (long) aString;
								break;
							case 'h':
								nRetVal = (USHORT) aString;
								break;
							case 'i':
								nRetVal = (int) aString;
								break;
		/*
							case 'f':
								fRetVal = (float) aString;
								nDouble = (double) fRetVal;
								nRetVal = (long) &nDouble;
								break;
							case 'F':
								nDouble = (double) aString;
								nRetVal = (long) &nDouble;
								break;
		*/
						}
					}
					else
					{
						char *pStr = (char*) malloc ((size_t)nLen+1);
						strcpy (pStr, aString.GetStr());
						nRetVal = (long) pStr;
					}
				}
				else
					nRetVal = 0L;
			}
			else if ( aAny >>= nRetVal )
			{
			}
			else if ( aAny >>= fRetVal )
			{
				nDouble = (double) fRetVal;
				nRetVal = (long) &nDouble;
			}
			else if ( aAny >>= nDouble )
			{
				nRetVal = (long) &nDouble;
			}
			else if ( aAny >>= aHandle )
			{
				nRetVal = aHandle.nHandle;
			}
		}
		catch ( OUnoSystemException& e)
		{
			// Der Server ist abgeflogen
			xODKServer = XODKDispatch();
			if ( pCallback )
				(*pCallback)();
		}
		catch ( OUnoException& e)
		{
			// Irgendeine andere Exception; ein Weiterarbeiten k"onnte funktionieren
		}

		delete [] pVarStart;
	}

	return nRetVal;
}

void _EXTLIBCALL_ ODKFreeMemory( void *p )
{
	NAMESPACE_VOS( OGuard ) aGuard( GetMutex() );
	free ( p );
}

void ODKListener::terminated()
{
	if ( pCallback )
		(*pCallback)();
}

ODKListener::ODKListener()
{
}

ODKListener::ODKListener(ORefCounter& rRefCounter)
{
}

ODKListener::~ODKListener()
{
}
