/*************************************************************************
 *
 *  $RCSfile: odkinter.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef WNT
#pragma optimize ( "", off )
#endif

#if defined (WIN) || defined (WNT)
#include <windows.h>
#endif

#ifdef OS2
#define INCL_DOSMODULEMGR
#ifdef BLC
#include <os2.h>
#else	/* CSET */
#include <os2def.h>
#include <bsedos.h>
#endif
#endif

#include <stdio.h>
#include <odkinter.h>
#include <malloc.h>

#if defined UNX
#ifndef TRUE
#define TRUE        1
#endif
#ifndef FALSE
#define FALSE       0
#endif
#endif

static short bInitialized;
static short bConnected;

#if defined (WIN) || defined (WNT)
static HINSTANCE histSvtDLL;
#endif

#ifdef OS2
static HMODULE hmSvtDLL;
#endif

#if defined (WNT) || defined (UNX) || ( defined( OS2 ) && ( defined( CSET3 ) || defined( ICC ) ) )
#define ODKCONNECTSERVER            "ODKConnect"
#define ODKDISCONNECTSERVER         "ODKDisconnect"
#define ODKCALLFUNCTION             "ODKCallFunction"
#define ODKFREEMEMORY               "ODKFreeMemory"
#else
#define ODKCONNECTSERVER 			"_ODKConnect"
#define ODKDISCONNECTSERVER 		"_ODKDisconnect"
#define ODKCALLFUNCTION 			"_ODKCallFunction"
#define ODKFREEMEMORY				"_ODKFreeMemory"
#endif

ODKCONNECTPROC pODKConnect;
ODKDISCONNECTPROC pODKDisconnect;
ODKPROC pODKCallFunction;
ODKFREEMEMPROC pODKFreeMemory;

static void svstrcpy(char *p, const char *q)
{
	while ((*p++ = *q++) != 0)
		;
}

static void svstrcat(char *p, const char *q)
{
	while (*p++)
	;
	p--;
	svstrcpy(p,q);
}

static short svstrcmp(const char *p, const char *q)
{
	short i;
	for ( i=0; p[i]; i++ )
	{
		if ( !p[i] || !q[i] )
			break;
		if ( p[i] != q[i]  )
			return i;
	}

	return 0;
}

static short svstrlen(const char *p)
{
	short i;
	for ( i=0; p[i]; i++ )
		{}
	return i;
}


static void BuildDLLName (char *pDLLName, const char *pDLLVersion)
{
	svstrcpy(pDLLName,"CLI");
	svstrcat(pDLLName,pDLLVersion);
#ifdef WIN
	svstrcat(pDLLName,"MW.DLL");
#endif
#ifdef WNT
	svstrcat(pDLLName,"MI.DLL");
#endif
#ifdef OS2
#ifdef BLC
	svstrcat(pDLLName,"BO");
#endif
#if defined( ICC ) || defined( CSET2 ) || defined( CSET3 )
	svstrcat(pDLLName,"CO");
#endif
#endif
}

static short LoadDLL (const char *pDLLName)
{
	short nRetVal = 0;
#if defined (WIN) || defined (WNT)
#ifdef WNT
	SetLastError(0L);
#endif
	histSvtDLL = LoadLibrary(pDLLName);
#ifdef WIN
	if (histSvtDLL > HINSTANCE_ERROR)
		nRetVal = 1;
#else
	if (histSvtDLL)
		nRetVal = 1;
	else
	{
		DWORD err = GetLastError();
		return (short) err;
	}
#endif
#endif
#ifdef OS2
	char ErrorBuffer[100];
	APIRET rc = DosLoadModule(ErrorBuffer,sizeof(ErrorBuffer),pDLLName,&hmSvtDLL);
	if (!rc)
		nRetVal = 1;
#endif
	return nRetVal;
}

static void UnloadDLL (void)
{
#if defined (WIN) || defined (WNT)
	FreeLibrary(histSvtDLL);
#endif
#ifdef OS2
	DosFreeModule(hmSvtDLL);
#endif
}

static void LoadProcedure (void **ppProcPointer,const char *pProcName)
{
#if defined (WIN) || defined (WNT)
	*ppProcPointer = GetProcAddress(histSvtDLL,pProcName);
#endif
#ifdef OS2
	DosQueryProcAddr(hmSvtDLL,0,pProcName,(PFN *)ppProcPointer);
#endif
}

void CDECL SvODKCallback()
{
	bConnected = FALSE;
}

short SYSCALL SvODKInit(const char *pVersion)
{
	if ( !bInitialized )
	{
		char cDLLName[256];
		BuildDLLName (cDLLName,pVersion);

		if ( LoadDLL(cDLLName) )
		{
			LoadProcedure ((void **)&pODKConnect, ODKCONNECTSERVER);
			LoadProcedure ((void **)&pODKDisconnect, ODKDISCONNECTSERVER);
			LoadProcedure ((void **)&pODKCallFunction, ODKCALLFUNCTION);
			LoadProcedure ((void **)&pODKFreeMemory, ODKFREEMEMORY);

			if ( pODKConnect && pODKFreeMemory && pODKDisconnect && pODKCallFunction )
				bInitialized = 1;
			else
				UnloadDLL();
		}
	}

	return bInitialized;
}


void SYSCALL SvODKDeInit(void)
{
	if ( bInitialized )
	{
		SvODKDisconnect();
		UnloadDLL();
		bInitialized = 0;
	}
}

short SYSCALL SvODKConnect()
{
	ODKCallbackFunc	pFunc = &SvODKCallback;
	if ( bInitialized && !bConnected )
	{
		pODKConnect( "soffice.exe", pFunc );
		bConnected = 1;
	}

	return bConnected;
}

short SYSCALL SvODKIsConnected()
{
	return bConnected;
}

void SYSCALL SvODKDisconnect()
{
	if ( bConnected )
	{
		pODKDisconnect();
		bConnected = 0;
	}
}

void SYSCALL SvODKFreeMemory( void *p )
{
	pODKFreeMemory( p );
}

void SYSCALL SvODKFreeObject( void *p )
{
	pODKCallFunction( 1, p, NULL, 'h' );
}

char* SYSCALL GetObjectName( C_Object ObjHdl )
{
	return (char*) pODKCallFunction( 0, ObjHdl , NULL, 's' );
}
