/*************************************************************************
 *
 *  $RCSfile: odkserv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SBXCLASS_HXX //autogen
#include <svtools/sbx.hxx>
#endif
#ifndef _SFXSTRITEM_HXX //autogen
#include <svtools/stritem.hxx>
#endif
#ifndef _SFXENUMITEM_HXX //autogen
#include <svtools/eitem.hxx>
#endif

#pragma hdrstop

#include <sfx2/app.hxx>
#include <sfx2/shobj.hxx>
#include <odkserv.hxx>

#ifndef GCC
UNO_DEFINE_XINTERFACE()
UNO_DEFINE_XFACTORY()
#endif

UNO_DEFINE_FACTORY( ODKService, odk.stardiv.de )

UNO_BEGIN_INIT_MODULE
UNO_REGISTER_FACTORY( ODKService, odk.stardiv.de )
UNO_END_INIT_MODULE

XODKTerminateListener xListener;

class MyBrokerThread : public NAMESPACE_VOS(OThread)
{
	UNO_Environment m_hEnvInit;
	UNO_Server		m_hServer;
	UNO_Broker		m_hBroker;
public:
					MyBrokerThread();
					~MyBrokerThread();

	virtual void 	run();
	void 			TerminateAndWait();
};

class MyDispatcherThread : public NAMESPACE_VOS(OThread)
{
	UNO_Environment m_hEnv;
	long			m_nEventId;
public:
					MyDispatcherThread(UNO_Environment hEnv);
					~MyDispatcherThread();

	virtual void 	run();
	void 			onTerminated();
	DECL_LINK(		Callback, void* );
};

XFactory				aXFactory;

MyBrokerThread *pBroker = NULL;

MyDispatcherThread::MyDispatcherThread(UNO_Environment hEnv)
	: m_hEnv(hEnv)
	, m_nEventId(0)
{
}

void MyDispatcherThread::run()
{
	// Den UNO-Dispatch an den HauptThread weiterleiten
	while(schedule() && UNO_waitForPendingRequest(m_hEnv, -1))
		m_nEventId = Application::PostUserEvent( LINK( this, MyDispatcherThread, Callback ) );
}

MyDispatcherThread::~MyDispatcherThread()
{
	// Mein eigenes Environment abr"aumen
	UNO_destroy(m_hEnv);
}

void MyDispatcherThread::onTerminated()
{
	delete this;
}

IMPL_LINK( MyDispatcherThread, Callback, void*, pVoid )
{
	// Wird im HauptThread gerufen!
	m_nEventId = 0;
	UNO_dispatch(m_hEnv, 0);
	return 0L;
}

MyBrokerThread::MyBrokerThread()
{
	m_hEnvInit	= UNO_initialize(0);
	m_hServer	= UNO_registerSelfAsServer( m_hEnvInit );

	ODKServicefactory* pFactory = new ODKServicefactory(*(ORefCounter*)m_hServer);
	pFactory->queryInterface(UNO_UIK(XFactory), aXFactory);
	UNO_registerFactory(m_hEnvInit, "odk.stardiv.de", aXFactory, m_hServer, 0);

	m_hBroker	= UNO_createBroker(m_hEnvInit, "local://ODK", UNO_BROKER_FLAG_NONE);
}

MyBrokerThread::~MyBrokerThread()
{
	UNO_destroy(m_hEnvInit);
}

void MyBrokerThread::run()
{
	UNO_Environment hEnvUser;
	do
	{
		hEnvUser = UNO_waitForRequest(m_hEnvInit, m_hBroker);
		if ( hEnvUser != NULL )
		{
			// F"ur jede Verbindung einen Dispatchthread einrichten. Dieser verwaltet
			// sein UNO_Environment selbst
			(new MyDispatcherThread(hEnvUser))->create();
		}
	}
	while ( schedule() && hEnvUser );
}

void MyBrokerThread::TerminateAndWait()
{
	OThread::terminate();
	UNO_destroyBroker(m_hEnvInit, m_hBroker);
	if ( isRunning() )
		join();
}

extern "C" void __LOADONCALLAPI ODKInit()
{
	// Einen Thread aufsetzen, indem der Requestbroker auf Kontakt wartet
	pBroker = new MyBrokerThread();
	pBroker->create();
}

extern "C" void __LOADONCALLAPI ODKExit()
{
	if ( !UNO_isNull( xListener ) )
	{
		try
		{
			xListener.terminated();
		}
		catch ( OUnoSystemException& e)
		{
			// Client ist abgeflogen
		}

		xListener = XODKTerminateListener();
	}

	// Den Brokerthread abr"aumen
	pBroker->TerminateAndWait();
	delete 	pBroker;
	aXFactory = XFactory();
}

ODKService::ODKService()
	: pStarBasic(0)
	, pGlobalSbxObject(0)
	, bEnterBasic(FALSE)
{
}

ODKService::ODKService( ORefCounter& rRefCounter )
	: OUnoSimpleObject ( rRefCounter )
	, pStarBasic(0)
	, pGlobalSbxObject(0)
	, bEnterBasic(FALSE)
{
}

ODKService::~ODKService()
{
	if ( bEnterBasic )
		SFX_APP()->LeaveBasicCall();
}

OUnoAny ODKService::dispatch ( Long nSid, Long nObjectHandle, const OUnoSequence <OUnoAny >& rIn )
{
	if ( !bEnterBasic )
	{
		SfxApplication *pSfxApplication = SFX_APP();
		pSfxApplication->EnterBasicCall();

		// TypeLibrary anziehen
		pGlobalSbxObject = pSfxApplication->GetSbxObject();
		if ( pGlobalSbxObject )
			pGlobalSbxObject -> Find ( "xxxyyy", SbxCLASS_DONTCARE );

		pStarBasic = pSfxApplication->GetBasic();
		bEnterBasic = TRUE;
	}

	OUnoAny aRet;
	const OUnoAny* pIn = rIn.getPointer();

	SbxVariableRef xSbxVariable = NULL;
	if ( nObjectHandle )
	{
		if ( !nSid )
		{
			// Es wird der Name eines Objects gew"unscht; dieses Property ist
			// in jedem SbxObject defaultm"assig vorhanden
			OUnoString aName = ( ((SbxObject *) nObjectHandle)->GetName() );
			aRet <<= aName;
			return aRet;
		}

		if ( nSid == 1 )
		{
			((SbxObject *) nObjectHandle)->ReleaseRef();
			return aRet;
		}

		// Am "ubergebenen Object ausf"uhren
		SbxObject *pObject = (SbxObject *) nObjectHandle;
		xSbxVariable = pObject->FindUserData( nSid );

		//! Hack: solange FindUserData nicht virtuell in SfxShellObject
		// eingebaut werden kann
		if ( !xSbxVariable.Is() && pObject->IsA( TYPE( SfxShellObject ) ) )
			xSbxVariable = ((SfxShellObject*)pObject)->GetShell()->FindSbx( nSid );
	}
	else if ( pGlobalSbxObject && nSid )
	{
		// Mit extended search am Basic ausf"uhren
		xSbxVariable = pGlobalSbxObject->FindUserData (nSid);
	}
	else
	{
		DBG_TRACE("SFX_SERVER : Kein SbxObject");
	}

	if (xSbxVariable.Is())
	{
		DBG_TRACE("SFX_SERVER : Methode gefunden");
		DBG_TRACE(String("SFX_SERVER : Sid = ")+String(nSid));
		DBG_TRACE(String("SFX_SERVER : UserData der Variablen : ") +
				String(xSbxVariable->GetUserData()));

		SbxArrayRef rSbxArray = new SbxArray;
		SbxArray* pSbxArray = rSbxArray;

		// SbxArray aus der AnySequence f"ullen
		for ( ULONG n=0; n<rIn.getLength(); n++)
		{
			SbxVariableRef pVar = new SbxVariable;

			BOOL bBool;
			short nInt;
			long nLong;
			float aFloat;
			OUnoString aString;
			if ( pIn[n] >>= bBool )
				pVar->PutBool( bBool );
			else if ( pIn[n] >>= nInt )
				pVar->PutInteger( nInt );
			else if ( pIn[n] >>= nLong )
				pVar->PutLong( nLong );
			else if ( pIn[n] >>= aFloat )
				pVar->PutSingle( aFloat );
			else if ( pIn[n] >>= aString )
				pVar->PutString( aString );
			else
				continue;

			pSbxArray->Insert( pVar, pSbxArray->Count() );
		}

		BOOL bSetProperty = pSbxArray->Count() && xSbxVariable->GetClass() == SbxCLASS_PROPERTY;
		if ( bSetProperty )
		{
			// Ein Property soll gesetzt werden; der Wert steht im ersten
			// Parameter des SbxArrays
			SbxVariable *pValue = pSbxArray->Get(1U);
			(*xSbxVariable) = (*pValue);
			aRet <<= 1L;
		}
		else
		{
			if ( pSbxArray->Count() )
				xSbxVariable->SetParameters( pSbxArray );
			else
				xSbxVariable->SetParameters( NULL );

			// Je nach Typ der SbxVariablen werden unterschiedliche Ergebnisse
			// erwartet, der Type ist aber noch unbekannt.
			SbxValues aData;
			aData.eType = SbxVARIANT;
			xSbxVariable->Get( aData );

			SbxDataType eSbxDataType = aData.eType;
			switch ( eSbxDataType )
			{
				case SbxOBJECT:
				{
					ObjectHandle aHandle;
					aHandle.nHandle = (long) aData.pObj;
					aRet <<= aHandle;
					break;
				}

				case SbxSTRING :
					if ( !aData.pString )
						aRet <<= OUnoString( L"" );
					else
						aRet <<= OUnoString( *aData.pString );
					break;
				case SbxVOID :
					break;
				case SbxULONG :
					aRet <<= aData.nULong;
					break;
				case SbxSINGLE :
					aRet <<= aData.nSingle;
					break;
				case SbxDOUBLE :
					aRet <<= aData.nDouble;
					break;
				default :
					// Alle anderen Datentypen (char, int etc.) werden als
					// long "ubertragen. Die Konvertierung mit GetLong() darf
					// aber keinen neuen Broadcast ausl"osen, sonst wird der
					// Request ein weiteres mal ausgef"uhrt!
					xSbxVariable->SetFlag(SBX_NO_BROADCAST);
					aRet <<= xSbxVariable->GetLong();
					xSbxVariable->ResetFlag(SBX_NO_BROADCAST);
					break;
			}
		}
	}
	else
	{
		DBG_TRACE(String("SFX_SERVER : Sid = ")+String(nSid));
		DBG_TRACE("SFX_SERVER : Variable nicht gefunden");
	}

	return aRet;
}

void ODKService::addTerminateListener(XODKTerminateListener& aListener)
{
	xListener = aListener;
}

void ODKService::removeTerminateListener(XODKTerminateListener& aListener)
{
	xListener = XODKTerminateListener();
}


