/*************************************************************************
 *
 *  $RCSfile: accmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#pragma hdrstop

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifndef _EVENT_HXX //autogen
#include <vcl/event.hxx>
#endif
#include "accmgr.hxx"
#include "mnumgr.hxx"
#include "app.hxx"
#include "sfxtypes.hxx"
#include "bindings.hxx"
#include "sfxresid.hxx"
#include "macrconf.hxx"
#include "msgpool.hxx"
#include "dispatch.hxx"
#include "accel.hrc"
#include "viewfrm.hxx"
#include "module.hxx"
#include "objface.hxx"
#include "request.hxx"

static const USHORT nVersion = 1;

//====================================================================

SvStream& operator>>( SvStream &rStream, KeyCode& rVal )
{
	USHORT nCode, nMod;
	rStream >> nCode;
	rStream >> nMod;

	if( nCode == USHRT_MAX )
		// virtuelle Taste
		rVal = KeyCode( (KeyFuncType) nMod );
	else
		// normale Taste
		rVal = KeyCode ( nCode, nMod );

	return rStream;
}


//--------------------------------------------------------------------

// stream out a KeyCode

SvStream& operator<<( SvStream &rStream, const KeyCode& rVal )
{
	DBG_MEMTEST();

	if( rVal.IsFunction() )
		// write virtual key code, marked as USHRT_MAX
		rStream << USHORT(USHRT_MAX) << USHORT( rVal.GetFunction() );
	else
		// write normal key code
		rStream << USHORT( rVal.GetCode() ) << USHORT( rVal.GetModifier() );
	return rStream;
}


//====================================================================

// stores the configured accelerators to a stream

BOOL SfxAcceleratorManager::Store( SvStream& rStream )
{
	DBG_MEMTEST();

	USHORT nCount = pAccel->GetItemCount();
	rStream << nVersion;
	rStream << nCount;
	for ( USHORT nPos = 0; nPos < nCount; ++nPos )
	{
		USHORT nId = pAccel->GetItemId( nPos );
		KeyCode aKey = pAccel->GetItemKeyCode( nPos );
		rStream << nId << aKey;

		if ( SfxMacroConfig::IsMacroSlot( nId ) )
			rStream << *(SFX_APP()->GetMacroConfig()->GetMacroInfo(nId));
	}

	return TRUE;
}


//--------------------------------------------------------------------
int SfxAcceleratorManager::Load(SvStream& rStream)
{
	DBG_MEMTEST();

	BOOL bReconfigure = ( pAccel != NULL );
	if ( bReconfigure )
		pAccel->Clear();
	else
		pAccel = new Accelerator;

	USHORT nCount, nFileVersion;
	rStream >> nFileVersion;
	if(nFileVersion != nVersion)
		return SfxConfigItem::WARNING_VERSION;

	rStream >> nCount;
	for ( USHORT nPos = 0; nPos<nCount; ++nPos )
	{
		USHORT nId;
		KeyCode aKey;
		rStream >> nId >> aKey;

		if ( SfxMacroConfig::IsMacroSlot( nId ) )
		{
			USHORT nOldId = nId;
			SfxMacroInfo aInfo;
			rStream >> aInfo;
			nId = aInfo.GetSlotId();
		}

		pAccel->InsertItem( nId, aKey );
	}

	if (bReconfigure)
	{
		SfxMenuManager *pMenuMgr = SFX_APP()->GetMenuBarManager();
		if ( pMenuMgr )
			pMenuMgr->InvalidateKeyCodes();
	}
	else
		pAccel->SetSelectHdl( LINK(this, SfxAcceleratorManager, Select ) );

	return SfxConfigItem::ERR_OK;
}

//--------------------------------------------------------------------
void SfxAcceleratorManager::UseDefault()
{
	DBG_MEMTEST();

	BOOL bReconfigure = ( pAccel != NULL );
	if ( bReconfigure )
		DELETEZ( pAccel );
    ResId aId( GetType() == SFX_ITEMTYPE_ACCEL ? RID_DEFAULTACCEL : GetType(), pResMgr );
    aId.SetRT(RSC_ACCEL);

    if ( Resource::GetResManager()->IsAvailable( aId ) )
        pAccel =  new Accelerator( aId );
    else
        pAccel = new Accelerator;

    if ( bReconfigure )
	{
		SfxMenuManager *pMenuMgr = SFX_APP()->GetMenuBarManager();
		if ( pMenuMgr )
			pMenuMgr->InvalidateKeyCodes();
	}
	else
		pAccel->SetSelectHdl( LINK(this, SfxAcceleratorManager, Select ) );
}

//--------------------------------------------------------------------

SfxAcceleratorManager::SfxAcceleratorManager()
	: SfxConfigItem(SFX_ITEMTYPE_ACCEL),
	nIteratorPos(0),
	pAccel(0),
	pResMgr(0)
{
	DBG_MEMTEST();
}

SfxAcceleratorManager::SfxAcceleratorManager( const ResId& rResId )
	: SfxConfigItem(rResId.GetId() == RID_DEFAULTACCEL ?
				SFX_ITEMTYPE_ACCEL :
				rResId.GetId() ),
	nIteratorPos(0),
	pAccel(0),
	pResMgr(rResId.GetResMgr())
{
	DBG_MEMTEST();
}

//--------------------------------------------------------------------

SfxAcceleratorManager::~SfxAcceleratorManager()
{
	DBG_MEMTEST();

	USHORT nCount = pAccel->GetItemCount();
	for ( USHORT nPos = 0; nPos < nCount; ++nPos )
	{
		USHORT nId = pAccel->GetItemId( nPos );
		if ( SfxMacroConfig::IsMacroSlot( nId ) )
			SFX_APP()->GetMacroConfig()->ReleaseSlotId(nId);
	}

	delete pAccel;
}


//--------------------------------------------------------------------

// resets the item iterator, FALSE if none

BOOL SfxAcceleratorManager::FirstItem()
{
	nIteratorPos = 0;
	return nIteratorPos < pAccel->GetItemCount();
}


//--------------------------------------------------------------------

// skips to the next item, FALSE if no more

BOOL SfxAcceleratorManager::NextItem()
{
	++nIteratorPos;
	return nIteratorPos < pAccel->GetItemCount();
}


//--------------------------------------------------------------------

// id of current binding if IsBinding()

USHORT SfxAcceleratorManager::GetId() const
{
	USHORT nAccelId = pAccel->GetItemId( nIteratorPos );
	return nAccelId;
}

//--------------------------------------------------------------------

USHORT SfxAcceleratorManager::GetId( const KeyCode &rKeyCode ) const
{
	USHORT nAccelId = pAccel->GetItemId( rKeyCode );
	return nAccelId;
}


//--------------------------------------------------------------------

// key code of current group or binding

KeyCode SfxAcceleratorManager::GetKeyCode() const
{
	return pAccel->GetItemKeyCode( nIteratorPos );
}


//--------------------------------------------------------------------

// key code for a func-id

KeyCode SfxAcceleratorManager::GetKeyCode( USHORT nId ) const
{
	return pAccel->GetKeyCode( nId );
}


//--------------------------------------------------------------------
// call this before reconfiguring

void SfxAcceleratorManager::Reset( USHORT nCount )
{
	pAccel->Clear();
}


//--------------------------------------------------------------------

// call this after reconfiguring

void SfxAcceleratorManager::Reconfigure()
{
	SetDefault(FALSE);

	// notify the menu
	SfxMenuManager *pMenuMgr = SFX_APP()->GetMenuBarManager();
	if ( pMenuMgr )
		pMenuMgr->InvalidateKeyCodes();
}


//--------------------------------------------------------------------

// append a new binding

void SfxAcceleratorManager::AppendItem( USHORT nId, const KeyCode& rKeyCode )
{
	DBG_MEMTEST();
	DBG_ASSERT( nId != 0, "null id detected" );
	pAccel->InsertItem( nId, rKeyCode );
}


//--------------------------------------------------------------------

// executes the function for the selected item

IMPL_LINK_INLINE_START( SfxAcceleratorManager, Select, Accelerator *, pAccel )
{
	DBG_MEMTEST();
	DBG_ERROR("old");
	return 0;
}
IMPL_LINK_INLINE_END( SfxAcceleratorManager, Select, Accelerator *, pAccel )


// ------------------------------------------------------------------------

String SfxAcceleratorManager::GetName() const
{
	return String(SfxResId(STR_ACCEL_CFGITEM));
}

BOOL SfxAcceleratorManager::Call(const KeyEvent& rEvnt, SfxBindings& rBind, BOOL bGlobal )
{
	USHORT nId = pAccel->GetItemId( rEvnt.GetKeyCode() );
	if ( !nId )
		return FALSE;

    //(mba/task): Hier muss abgefragt werden, ob denn gerade am aktuellen Document der Bindings Modalitt gegeben ist!
	{
		if ( bGlobal )
		{
			SfxApplication *pApp = SFX_APP();
            SfxDispatcher* pDispatcher = rBind.GetDispatcher();
			const SfxSlot *pSlot=0;
			SfxShell* pShell = pApp->GetActiveModule();
            if ( !( pSlot = pShell->GetInterface()->GetSlot( nId ) ) )
            {
                pShell = pDispatcher->GetFrame();
                if ( !( pSlot = pShell->GetInterface()->GetSlot( nId ) ) )
                {
                    pShell = pApp;
                    if ( !( pSlot = pShell->GetInterface()->GetSlot( nId ) ) )
                        return FALSE;
                }
            }

			SfxItemPool &rPool = pShell->GetPool();
			BOOL bAsynchron = ( pSlot->GetMode() & SFX_SLOT_ASYNCHRON ) || !( pSlot->GetMode() & SFX_SLOT_SYNCHRON );
			SfxRequest aReq( nId, SFX_CALLMODE_SLOT, rPool );
			pShell->ExecuteSlot( aReq, bAsynchron );
			return TRUE;
		}
		else
		{
			// wegen OLE-InPlace und Container-KeyInput (SV liefert TRUE!)
            return rBind.Execute( nId );
		}
	}

	return FALSE;
}


