/*************************************************************************
 *
 *  $RCSfile: basmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif
#ifndef _SVSTOR_HXX //autogen
#include <so3/svstor.hxx>
#endif
#ifndef _URLOBJ_HXX //autogen
#include <tools/urlobj.hxx>
#endif
#ifndef _SFXSMPLHINT_HXX //autogen
#include <svtools/smplhint.hxx>
#endif
#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif
#ifndef _SV_WINDOW_HXX //autogen
#include <vcl/window.hxx>
#endif
#ifndef _SV_WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif
#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _SBXCLASS_HXX //autogen
#include <svtools/sbx.hxx>
#endif
#ifndef _SOT_STORINFO_HXX //autogen
#include <sot/storinfo.hxx>
#endif

#ifndef _SB_SBMOD_HXX
#include <basic/sbmod.hxx>
#endif

#include "basmgr.hxx"
#include "sfxresid.hxx"
#include "app.hrc"
#include "sfxuno.hxx"
#include "helper.hxx"

#define LIB_SEP			0x01
#define LIBINFO_SEP		0x02
#define LIBINFO_ID		0x1491
#define PASSWORD_MARKER	0x31452134


#define CURR_VER		2

// Version 1
//	  ULONG 	nEndPos
//	  USHORT 	nId
//	  USHORT	nVer
//	  BOOL		bDoLoad
//	  String	LibName
//	  String	AbsStorageName
//	  String	RelStorageName
// Version 2
//	+ BOOL		bReference

static const char* szStdLibName = "Standard";
static const char* szBasicStorage = "StarBASIC";
static const char* szOldManagerStream = "BasicManager";
static const char* szManagerStream = "BasicManager2";
static const char* szImbedded = "LIBIMBEDDED";
static const char* szCryptingKey = "CryptedBasic";

TYPEINIT1( SfxBasicManager, BasicManager );
DBG_NAME( SfxBasicManager );

StreamMode eStreamReadMode = STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYALL;
StreamMode eStorageReadMode = STREAM_READ | STREAM_SHARE_DENYWRITE;

DECLARE_LIST( BasErrorLst, BasicError* );

class BasicErrorManager
{
private:
	BasErrorLst	aErrorList;

public:
				~BasicErrorManager();

	void		Reset();
	void		InsertError( const BasicError& rError );

	BOOL		HasErrors()			{ return (BOOL)aErrorList.Count(); }
	BasicError*	GetFirstError()		{ return aErrorList.First(); }
	BasicError*	GetNextError()		{ return aErrorList.Next(); }
};


BasicErrorManager::~BasicErrorManager()
{
	Reset();
}

void BasicErrorManager::Reset()
{
	BasicError* pError = (BasicError*)aErrorList.First();
	while ( pError )
	{
		delete pError;
		pError = (BasicError*)aErrorList.Next();
	}
	aErrorList.Clear();
}

void BasicErrorManager::InsertError( const BasicError& rError )
{
	aErrorList.Insert( new BasicError( rError ), LIST_APPEND );
}


BasicError::BasicError()
{
	nErrorId	= 0;
	nReason 	= 0;
}

BasicError::BasicError( ULONG nId, USHORT nR, const String& rErrStr ) :
	aErrStr( rErrStr )
{
	nErrorId 	= nId;
	nReason 	= nR;
}

BasicError::BasicError( const BasicError& rErr ) :
	aErrStr( rErr.aErrStr )
{
	nErrorId 	= rErr.nErrorId;
	nReason		= rErr.nReason;
}


class BasicLibInfo
{
private:
	StarBASICRef	xLib;
	String			aLibName;
	String			aStorageName;	// String reicht, da zur Laufzeit eindeutig.
	String			aRelStorageName;
	String			aPassword;

	BOOL			bDoLoad;
	BOOL			bReference;
	BOOL			bPasswordVerified;
	BOOL			bFoundInPath;	// Darf dann nicht neu relativiert werden!

public:
	BasicLibInfo();
	BasicLibInfo( const String& rStorageName );

	BOOL			IsReference() const		{ return bReference; }
	BOOL&			IsReference()			{ return bReference; }

	BOOL			IsExtern() const 		{ return ! aStorageName.EqualsAscii(szImbedded); }

	void			SetStorageName( const String& rName )	{ aStorageName = rName; }
	const String&	GetStorageName() const					{ return aStorageName; }

	void			SetRelStorageName( const String& rN )	{ aRelStorageName = rN; }
	const String&	GetRelStorageName()	const				{ return aRelStorageName; }
	void			CalcRelStorageName( const String& rMgrStorageName );

	StarBASICRef	GetLib() const						{ return xLib; }
	StarBASICRef&	GetLibRef()							{ return xLib; }
	void			SetLib( StarBASIC* pBasic )			{ xLib = pBasic; }

	const String&	GetLibName() const					{ return aLibName; }
	void			SetLibName( const String& rName )	{ aLibName = rName; }

	// Nur temporaer fuer Laden/Speichern....
	BOOL			DoLoad()							{ return bDoLoad; }

	BOOL			HasPassword() const 				{ return aPassword.Len() != 0; }
	const String&	GetPassword() const					{ return aPassword; }
	void			SetPassword( const String& rNewPassword )
														{ aPassword = rNewPassword;	}
	BOOL			IsPasswordVerified() const			{ return bPasswordVerified; }
	void			SetPasswordVerified()				{ bPasswordVerified = TRUE; }

	BOOL			IsFoundInPath() const				{ return bFoundInPath; }
	void			SetFoundInPath( BOOL bInPath )		{ bFoundInPath = bInPath; }

	void 					Store( SvStorageStream& rSStream, const SvStorage& rStorageFromStream, const String& rBasMgrStorageName, BOOL bUseOldReloadInfo );
	static BasicLibInfo*	Create( SvStorageStream& rSStream );
};

DECLARE_LIST( BasicLibsBase, BasicLibInfo* );

class BasicLibs : public BasicLibsBase
{
public:
	String	aBasicLibPath;		// soll eigentlich Member vom Manager werden, aber jetzt nicht inkompatibel!
};

BasicLibInfo::BasicLibInfo()
{
	bReference 			= FALSE;
	bPasswordVerified 	= FALSE;
	bDoLoad 			= FALSE;
	bFoundInPath		= FALSE;
	aStorageName 		= String::CreateFromAscii(szImbedded);
	aRelStorageName 	= String::CreateFromAscii(szImbedded);
}

BasicLibInfo::BasicLibInfo( const String& rStorageName )
{
	bReference 			= TRUE;
	bPasswordVerified 	= FALSE;
	bDoLoad 			= FALSE;
	aStorageName 		= rStorageName;
}

void BasicLibInfo::Store( SvStorageStream& rSStream, const SvStorage& rStorageFromStream, const String& rBasMgrStorageName, BOOL bUseOldReloadInfo )
{
	ULONG nStartPos = rSStream.Tell();
	ULONG nEndPos = 0;

	USHORT nId = LIBINFO_ID;
	USHORT nVer = CURR_VER;

	rSStream << nEndPos;
	rSStream << nId;
	rSStream << nVer;

    bool bWasAbsolute = FALSE;
    String aCurStorageName( INetURLObject().smartRel2Abs( rBasMgrStorageName, bWasAbsolute ).GetMainURL() );
    DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );

	// Falls nicht gesetzt, StorageName initialisieren
	if ( aStorageName.Len() == 0 )
        aStorageName = aCurStorageName;

	// Wieder laden?
	BOOL bDoLoad = xLib.Is();
	if ( bUseOldReloadInfo )
		bDoLoad = DoLoad();
	rSStream << bDoLoad;

	// Den Namen der Lib...
	rSStream.WriteByteString(GetLibName());

	// Absoluter Pfad....
	if ( ! GetStorageName().EqualsAscii(szImbedded) )
	{
        bool bWasAbsolute = FALSE;
        rSStream.WriteByteString( INetURLObject().smartRel2Abs( GetStorageName(), bWasAbsolute ).GetMainURL() );
        DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );
	}
	else
		rSStream.WriteByteString( szImbedded );

	// Relativer Pfad...
	if ( ( aStorageName == aCurStorageName ) || ( aStorageName.EqualsAscii(szImbedded) ) )
		rSStream.WriteByteString( szImbedded );
	else
	{
		// Nicht den relativen Pfad ermitteln, wenn die Datei nur im Pfad
		// gefunden wurde: Dann andert sich der relative Pfad und nach einem
		// verschieben der Libs in einen anderen Pfad werden sie nicht gefunden.
		if ( !IsFoundInPath() )
			CalcRelStorageName( aCurStorageName );
		rSStream.WriteByteString(aRelStorageName);
	}

	// ------------------------------
	// Version 2
	// ------------------------------

	// Referenz...
	rSStream << bReference;

	// ------------------------------
	// Schluss
	// ------------------------------

	nEndPos = rSStream.Tell();
	rSStream.Seek( nStartPos );
	rSStream << nEndPos;
	rSStream.Seek( nEndPos );
}

BasicLibInfo* BasicLibInfo::Create( SvStorageStream& rSStream )
{
	BasicLibInfo* pInfo = new BasicLibInfo;

	ULONG nEndPos;
	USHORT nId;
	USHORT nVer;

	rSStream >> nEndPos;
	rSStream >> nId;
	rSStream >> nVer;

	DBG_ASSERT( nId == LIBINFO_ID, "Keine BasicLibInfo !?" );
	if( nId == LIBINFO_ID )
	{
		// Wieder laden?
		BOOL bDoLoad;
		rSStream >> bDoLoad;
		pInfo->bDoLoad = bDoLoad;

		// Den Namen der Lib...
		String aName;
		rSStream.ReadByteString(aName);
		pInfo->SetLibName( aName );

		// Absoluter Pfad....
		String aStorageName;
		rSStream.ReadByteString(aStorageName);
		pInfo->SetStorageName( aStorageName );

		// Relativer Pfad...
		String aRelStorageName;
		rSStream.ReadByteString(aRelStorageName);
		pInfo->SetRelStorageName( aRelStorageName );

		if ( nVer >= 2 )
		{
			BOOL bReferenz;
			rSStream >> bReferenz;
			pInfo->IsReference() = bReferenz;
		}

		rSStream.Seek( nEndPos );
	}
	return pInfo;
}

void BasicLibInfo::CalcRelStorageName( const String& rMgrStorageName )
{
	if ( rMgrStorageName.Len() )
	{
        INetURLObject aPath( rMgrStorageName, INET_PROT_FILE );
        SetRelStorageName( aPath.AbsToRel( GetStorageName() ) );
	}
	else
		SetRelStorageName( String() );
}

SfxBasicManager::SfxBasicManager( SvStorage& rStorage, StarBASIC* pParentFromStdLib, String* pLibPath )
{
	DBG_CTOR( SfxBasicManager, 0 );

	Init();

	if( pLibPath )
		pLibs->aBasicLibPath = *pLibPath;

    String aStorName( rStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

    bool bWasAbsolute = FALSE;
    aStorageName = INetURLObject().smartRel2Abs( aStorName, bWasAbsolute ).GetMainURL();
    DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );

	// Wenn es den Manager-Stream nicht gibt, sind keine weiteren
	// Aktionen noetig.
	if ( rStorage.IsStream( String::CreateFromAscii(szManagerStream) ) )
	{
        LoadBasicManager( rStorage );
		// StdLib erhaelt gewuenschten Parent:
		StarBASIC* pStdLib = GetStdLib();
		DBG_ASSERT( pStdLib, "Standard-Lib nicht geladen?" );
		if ( !pStdLib )
		{
			// Sollte eigentlich nie passieren, aber dann wenigstens nicht abstuerzen...
			pStdLib = new StarBASIC;
			BasicLibInfo* pStdLibInfo = pLibs->GetObject( 0 );
			if ( !pStdLibInfo )
				pStdLibInfo = CreateLibInfo();
			pStdLibInfo->SetLib( pStdLib );
			pStdLibInfo->GetLib()->SetName( String::CreateFromAscii(szStdLibName) );
			pStdLibInfo->SetLibName( String::CreateFromAscii(szStdLibName) );
			pStdLibInfo->GetLib()->SetFlag( SBX_DONTSTORE | SBX_EXTSEARCH );
			pStdLibInfo->GetLib()->SetModified( FALSE );
		}
		else
		{
			pStdLib->SetParent( pParentFromStdLib );
			// Die anderen erhalten die StdLib als Parent:
			for ( USHORT nBasic = 1; nBasic < GetLibCount(); nBasic++ )
			{
				StarBASIC* pBasic = GetLib( nBasic );
				if ( pBasic )
				{
//					pBasic->SetParent( pStdLib );
					pStdLib->Insert( pBasic );
					pBasic->SetFlag( SBX_EXTSEARCH );
				}
			}
			// Durch das Insert modified:
			pStdLib->SetModified( FALSE );
		}
	}
	else
	{
		ImpCreateStdLib( pParentFromStdLib );
		if ( rStorage.IsStream( String::CreateFromAscii(szOldManagerStream) ) )
            LoadOldBasicManager( rStorage );
	}
	bBasMgrModified = FALSE;
}

SfxBasicManager::SfxBasicManager( StarBASIC* pSLib, String* pLibPath )
{
	DBG_CTOR( SfxBasicManager, 0 );
	Init();
	DBG_ASSERT( pSLib, "SfxBasicManager kann nicht mit einem NULL-Pointer erzeugt werden!" );

	if( pLibPath )
		pLibs->aBasicLibPath = *pLibPath;

	BasicLibInfo* pStdLibInfo = CreateLibInfo();
	pStdLibInfo->SetLib( pSLib );
	pStdLibInfo->GetLib()->SetName( String::CreateFromAscii(szStdLibName));
	pStdLibInfo->SetLibName( String::CreateFromAscii(szStdLibName) );
	pSLib->SetFlag( SBX_DONTSTORE | SBX_EXTSEARCH );

	// Speichern lohnt sich nur, wenn sich das Basic aendert.
	pStdLibInfo->GetLib()->SetModified( FALSE );
	bBasMgrModified = FALSE;
}

SfxBasicManager::SfxBasicManager()
{
	DBG_CTOR( SfxBasicManager, 0 );
	// Diese CTOR darf nur verwendet werden um bei 'Speichern unter'
	// die relativen Pfade anzupassen, das gibt kein AppBasic und somit
	// duerfen auch keine Libs geladen werden...
	Init();
}

BOOL SfxBasicManager::HasBasicManager( const SvStorage& rStorage )
{
	return rStorage.IsStream( String::CreateFromAscii(szManagerStream) );
}

void SfxBasicManager::ImpMgrNotLoaded( const String& rStorageName )
{
	// pErrInf wird nur zerstoert, wenn der Fehler von einem ErrorHandler
	// gehandelt wird!
//	String aErrorText( BasicResId( IDS_SBERR_MGROPEN ) );
//	aErrorText.SearchAndReplace( "XX", rStorageName );
	StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_MGROPEN,	rStorageName, ERRCODE_BUTTON_OK );
	pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENMGRSTREAM, rStorageName ) );

	// Eine STD-Lib erzeugen, sonst macht es Peng!
	BasicLibInfo* pStdLibInfo = CreateLibInfo();
	pStdLibInfo->SetLib( new StarBASIC );
	pStdLibInfo->GetLib()->SetName( String::CreateFromAscii(szStdLibName) );
	pStdLibInfo->SetLibName( String::CreateFromAscii(szStdLibName) );
	pStdLibInfo->GetLib()->SetFlag( SBX_DONTSTORE | SBX_EXTSEARCH );
	pStdLibInfo->GetLib()->SetModified( FALSE );
}


void SfxBasicManager::ImpCreateStdLib( StarBASIC* pParentFromStdLib )
{
	BasicLibInfo* pStdLibInfo = CreateLibInfo();
	StarBASIC* pStdLib = new StarBASIC( pParentFromStdLib );
	pStdLibInfo->SetLib( pStdLib );
	pStdLibInfo->GetLib()->SetName( String::CreateFromAscii(szStdLibName) );
	pStdLibInfo->SetLibName( String::CreateFromAscii(szStdLibName) );
	pStdLib->SetFlag( SBX_DONTSTORE | SBX_EXTSEARCH );
}


void SfxBasicManager::LoadBasicManager( SvStorage& rStorage, BOOL bLoadLibs )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

//	StreamMode eStreamMode = STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE;

	SvStorageStreamRef xManagerStream = rStorage.OpenStream
		( String::CreateFromAscii(szManagerStream), eStreamReadMode );

    String aStorName( rStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

	if ( !xManagerStream.Is() || xManagerStream->GetError() || ( xManagerStream->Seek( STREAM_SEEK_TO_END ) == 0 ) )
	{
        ImpMgrNotLoaded( aStorName );
		return;
	}

    bool bWasAbsolute = FALSE;
    aStorageName = INetURLObject().smartRel2Abs( aStorName, bWasAbsolute ).GetMainURL();
    DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );

	String aRealStorageName = aStorageName;  // fuer relative Pfade, kann durch BaseURL umgebogen werden.

	// Wenn aus Vorlagen geladen wird, gilt nur die BaseURL:
	String aBaseURL = INetURLObject::GetBaseURL();
	if ( aBaseURL.Len() )
	{
		INetURLObject aObj( aBaseURL );
		if ( aObj.GetProtocol() == INET_PROT_FILE )
			aRealStorageName = aObj.PathToFileName();
	}

	xManagerStream->SetBufferSize( 1024 );
	xManagerStream->Seek( STREAM_SEEK_TO_BEGIN );

	ULONG nEndPos;
	*xManagerStream >> nEndPos;

	USHORT nLibs;
	*xManagerStream >> nLibs;
	// Plausi!
	if( nLibs & 0xF000 )
	{
		DBG_ASSERT( !this, "SfxBasicManager-Stream defekt!" );
		return;
	}
	for ( USHORT nL = 0; nL < nLibs; nL++ )
	{
		BasicLibInfo* pInfo = BasicLibInfo::Create( *xManagerStream );

		// ggf. absoluten Pfad-Namen korrigieren, wenn rel. existiert
		// Immer erst den relativen versuchen, falls zwei Staende auf der Platte
        if ( pInfo->GetRelStorageName().Len() && ( ! pInfo->GetRelStorageName().EqualsAscii(szImbedded) ) )
		{
            INetURLObject aObj( aRealStorageName, INET_PROT_FILE );
            aObj.removeSegment();
            bool bWasAbsolute = FALSE;
            aObj = aObj.smartRel2Abs( pInfo->GetRelStorageName(), bWasAbsolute );

            if ( SfxContentHelper::Exists( aObj.GetMainURL() ) )
                pInfo->SetStorageName( aObj.GetMainURL() );
			else if ( pLibs->aBasicLibPath.Len() )
			{
				// Lib im Pfad suchen...
                String aResult;
                if ( SfxContentHelper::Find( pLibs->aBasicLibPath, pInfo->GetRelStorageName(), aResult ) )
				{
                    pInfo->SetStorageName( aResult );
					pInfo->SetFoundInPath( TRUE );
				}
			}
		}

		pLibs->Insert( pInfo, LIST_APPEND );
		// Libs aus externen Dateien sollen erst bei Bedarf geladen werden.
		// Aber Referenzen werden gleich geladen, sonst bekommen die Grosskunden
		// vielleicht Probleme...
		if ( bLoadLibs && pInfo->DoLoad() &&
			( ( !pInfo->IsExtern() ) || ( pInfo->IsReference() ) ) )
		{
            ImpLoadLibary( pInfo, &rStorage );
		}
	}

	xManagerStream->Seek( nEndPos );
	xManagerStream->SetBufferSize( 0 );
	xManagerStream.Clear();
}

void SfxBasicManager::LoadOldBasicManager( SvStorage& rStorage )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

//	StreamMode eStreamMode = STREAM_READ | STREAM_NOCREATE | STREAM_SHARE_DENYWRITE;

	SvStorageStreamRef xManagerStream = rStorage.OpenStream
		( String::CreateFromAscii(szOldManagerStream), eStreamReadMode );

    String aStorName( rStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

	if ( !xManagerStream.Is() || xManagerStream->GetError() || ( xManagerStream->Seek( STREAM_SEEK_TO_END ) == 0 ) )
	{
        ImpMgrNotLoaded( aStorName );
		return;
	}

	xManagerStream->SetBufferSize( 1024 );
	xManagerStream->Seek( STREAM_SEEK_TO_BEGIN );
	ULONG nBasicStartOff, nBasicEndOff;
	*xManagerStream >> nBasicStartOff;
	*xManagerStream >> nBasicEndOff;

	DBG_ASSERT( !xManagerStream->GetError(), "Ungueltiger Manager-Stream!" );

	xManagerStream->Seek( nBasicStartOff );
	if( !ImplLoadBasic( *xManagerStream, pLibs->GetObject(0)->GetLibRef() ) )
	{
//		String aErrorText( BasicResId( IDS_SBERR_MGROPEN ) );
//      aErrorText.SearchAndReplace( "XX", aStorName );
        StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_MGROPEN, aStorName, ERRCODE_BUTTON_OK );
        pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENMGRSTREAM, aStorName ) );
		// und es geht weiter...
	}
	xManagerStream->Seek( nBasicEndOff+1 );	// +1: 0x00 als Trenner
	String aLibs;
	xManagerStream->ReadByteString(aLibs);
	xManagerStream->SetBufferSize( 0 );
	xManagerStream.Clear();	// Sream schliessen

	if ( aLibs.Len() )
	{
        String aCurStorageName( aStorName );
        INetURLObject aCurStorage( aCurStorageName, INET_PROT_FILE );
		USHORT nLibs = aLibs.GetTokenCount( LIB_SEP );
		for ( USHORT nLib = 0; nLib < nLibs; nLib++ )
		{
			String aLibInfo( aLibs.GetToken( nLib, LIB_SEP ) );
			// == 2 soll irgendwann weg!
			DBG_ASSERT( ( aLibInfo.GetTokenCount( LIBINFO_SEP ) == 2 ) || ( aLibInfo.GetTokenCount( LIBINFO_SEP ) == 3 ), "Ungueltige Lib-Info!" );
			String aLibName( aLibInfo.GetToken( 0, LIBINFO_SEP ) );
			String aLibAbsStorageName( aLibInfo.GetToken( 1, LIBINFO_SEP ) );
			String aLibRelStorageName( aLibInfo.GetToken( 2, LIBINFO_SEP ) );
            INetURLObject aLibAbsStorage( aLibAbsStorageName, INET_PROT_FILE );

            INetURLObject aLibRelStorage( aStorName );
            aLibRelStorage.removeSegment();
            bool bWasAbsolute = FALSE;
            aLibRelStorage = aLibRelStorage.smartRel2Abs( aLibRelStorageName, bWasAbsolute);
            DBG_ASSERT(!bWasAbsolute, "RelStorageName was absolute!" );

			SvStorageRef xStorageRef;
			if ( ( aLibAbsStorage == aCurStorage ) || ( aLibRelStorageName.EqualsAscii(szImbedded) ) )
				xStorageRef = &rStorage;
			else
			{
                xStorageRef = new SvStorage( aLibAbsStorage.GetMainURL(), eStorageReadMode, TRUE );
                if ( xStorageRef->GetError() != ERRCODE_NONE )
                    xStorageRef = new SvStorage( aLibRelStorage.GetMainURL(), eStorageReadMode, TRUE );
			}
			if ( xStorageRef.Is() )
                AddLib( *xStorageRef, aLibName, FALSE );
			else
			{
//				String aErrorText( BasicResId( IDS_SBERR_LIBLOAD ) );
//				aErrorText.SearchAndReplace( "XX", aLibName );
                StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBLOAD, aStorName, ERRCODE_BUTTON_OK );
                pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_STORAGENOTFOUND, aStorName ) );
			}
		}
	}
}

SfxBasicManager::~SfxBasicManager()
{
	DBG_DTOR( SfxBasicManager, 0 );

	// Listener benachrichtigen, falls noch etwas zu Speichern...
	Broadcast( SfxSimpleHint( SFX_HINT_DYING) );

	// Basic-Infos zerstoeren...
	// In umgekehrter Reihenfolge, weil die StdLib Referenzen haelt, die
	// anderen nur die StdLib als Parent haben.
	BasicLibInfo* pInf = pLibs->Last();
	while ( pInf )
	{
		delete pInf;
		pInf = pLibs->Prev();
	}
	pLibs->Clear();
	delete pLibs;
	delete pErrorMgr;
}

void SfxBasicManager::Init()
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	bBasMgrModified = FALSE;
	pErrorMgr = new BasicErrorManager;
	pLibs = new BasicLibs;
}

BasicLibInfo* SfxBasicManager::CreateLibInfo()
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = new BasicLibInfo;
	pLibs->Insert( pInf, LIST_APPEND );
	return pInf;
}

BOOL SfxBasicManager::CopyBasicData( SvStorage* pStorFrom, const String& rSourceURL, SvStorage* pStorTo )
{
	/*-----------------------------------------------------------------
	 Diese Methode wird vom SXF gerufen bei 'Datei speichern unter',
	 damit die Basic-Storages kopiert werden.
	 Neu: ggf. muessen relative Pfade angepasst werden!
	------------------------------------------------------------------*/
	BOOL bOk = TRUE;

	// bei remote Dokumenten identische Storage
	if ( pStorFrom != pStorTo )
	{
		if( pStorFrom->IsStorage( String::CreateFromAscii(szBasicStorage) ) )
			bOk = pStorFrom->CopyTo( String::CreateFromAscii(szBasicStorage), pStorTo, String::CreateFromAscii(szBasicStorage) );
		if( bOk && pStorFrom->IsStream( String::CreateFromAscii(szManagerStream) ) )
		{
			// bOk = pStorFrom->CopyTo( szManagerStream, pStorTo, szManagerStream );
			SfxBasicManager aBasMgr;
			// Die aktuelle Base-URL ist die vom speichern...
			String aOldURL = INetURLObject::GetBaseURL();
			if ( rSourceURL.Len() )
				INetURLObject::SetBaseURL( rSourceURL );
            String aStorName( pStorFrom->GetName() );
            DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

            aBasMgr.LoadBasicManager( *pStorFrom, FALSE );
			INetURLObject::SetBaseURL( aOldURL );
			aBasMgr.Store( *pStorTo, FALSE );
		}
	}

	return bOk;
}

void SfxBasicManager::Merge( SvStorage& rFromStorage )
{
    String aStorName( rFromStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

    SfxBasicManager aTmpMgr( rFromStorage, NULL, &pLibs->aBasicLibPath );
	USHORT nLibs = aTmpMgr.GetLibCount();
	for ( USHORT nL = 1 /* ohne STANDARD */; nL < nLibs; nL++ )
	{
		BasicLibInfo* pInf = aTmpMgr.pLibs->GetObject( nL );

		// Existierende Lib loeschen...
		USHORT nExistingLib = GetLibId( pInf->GetLibName() );
		BasicLibInfo* pExistingLib = pLibs->GetObject( nExistingLib );
		if ( pExistingLib )
		{
			BOOL bDelFromStorage = TRUE;
			if ( pExistingLib->IsReference() || pExistingLib->IsExtern() )
				bDelFromStorage = FALSE;	// Es werden die vorhandenen externen Libs verwendet
			RemoveLib( nExistingLib, bDelFromStorage );
		}

		// Und die neue einfuegen...
		SvStorageRef xStor = &rFromStorage;
		if ( pInf->IsExtern() || pInf->IsReference() )
			xStor = new SvStorage( pInf->GetStorageName(), eStorageReadMode );
        AddLib( *xStor, pInf->GetLibName(), pInf->IsReference() || pInf->IsExtern() );
		if ( pInf->IsExtern() )
		{
			BasicLibInfo* pNewInf = pLibs->GetObject( GetLibId( pInf->GetLibName() ) );
			if ( pNewInf )
				pNewInf->IsReference() = FALSE;
		}
	}
}

void SfxBasicManager::Store( SvStorage& rStorage )
{
    Store( rStorage, TRUE );
}

void SfxBasicManager::Store( SvStorage& rStorage, BOOL bStoreLibs )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	// Neue Bibliotheken sind nicht unbedingt modified, muessen aber trotzdem
	// gespeichert werden.
	BOOL bStoreAll = FALSE;
	if ( bStoreLibs && !rStorage.IsStorage( String::CreateFromAscii(szBasicStorage) ) )
		bStoreAll = TRUE;

	SvStorageStreamRef xManagerStream = rStorage.OpenStream( String::CreateFromAscii(szManagerStream), STREAM_STD_READWRITE /* | STREAM_TRUNC */ );
	// Assertion, weil es eigentlich nie vorkommen darf.
	DBG_ASSERT( xManagerStream.Is(), "Kein ManagerStream!");

	ClearErrors();
	USHORT nLibs = GetLibCount();

    String aStorName( rStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

	if ( !xManagerStream.Is() || xManagerStream->GetError() )
	{
//		String aErrorText( BasicResId( IDS_SBERR_MGRSAVE ) );
//      aErrorText.SearchAndReplace( "XX", aStorName );
        StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_MGRSAVE, aStorName, ERRCODE_BUTTON_OK );
        ((SfxBasicManager*)this)->pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENMGRSTREAM, aStorName ) );
	}
	else
	{
        aStorageName = aStorName;

		// Funktioniert nicht, auber auch OPEN_TRUNC nicht (UAE in  OLE32)
		// => Erst mal den alten Schrott stehen lassen!
//		xManagerStream->SetSize( 0 );
		xManagerStream->SetBufferSize( 1024 );
		xManagerStream->Seek( 0 );

		ULONG nStartPos = 0;
		ULONG nEndPos = 0;
		*xManagerStream << nEndPos;

		// Fehlerabfrage, falls schreiben nicht moeglich ?
		DBG_ASSERT( !xManagerStream->GetError(), "Unerwarteter Fehler beim Schreiben des Manager-Streams" );


		// Die Namen der Libs und der Storages...
//        DirEntry aCurStorage( aStorName() );
//        aCurStorage.ToAbs();
		String aLibInfo;
		*xManagerStream << nLibs;
		for ( USHORT nL = 0; nL < nLibs; nL++ )
		{
			BasicLibInfo* pInfo = pLibs->GetObject( nL );
			DBG_ASSERT( pInfo, "pInfo?!" );

			String aStrgName = GetStorageName();

			// Falls ins Temp-Verzeichniss gesichert wird, sollte der richtige
			// Storage-Name an der globalen URL haengen...
			String aBaseURL = INetURLObject::GetBaseURL();
			if ( aBaseURL.Len() )
			{
				INetURLObject aObj( aBaseURL );
				if ( aObj.GetProtocol() == INET_PROT_FILE )
					aStrgName = aObj.PathToFileName();
			}

			pInfo->Store( *xManagerStream, rStorage, aStrgName, !bStoreLibs );
		}

		nEndPos = xManagerStream->Tell();
		xManagerStream->Seek( nStartPos );
		*xManagerStream << nEndPos;
		xManagerStream->Seek( nEndPos );
		xManagerStream->SetBufferSize( 0 );
		xManagerStream.Clear();
	}

	// Und dann die Libs in den entsprechenden Storages...
	if ( bStoreLibs )
	{
        String aCurStorage( aStorName );
		for ( USHORT nL = 0; nL < nLibs; nL++ )
		{
			BasicLibInfo* pInfo = pLibs->GetObject( nL );
			DBG_ASSERT( pInfo, "Info?!" );
			StarBASIC* pLib = pInfo->GetLib();
			if ( pLib && ( pLib->IsModified() || bStoreAll ) )	// 0, wenn nicht geladen...
			{
				if ( pInfo->IsReference() )
				{
					// Vielleicht nicht modified, aber bStoreAll.
					// Fehlermeldung nicht, wenn Macro lauft
					// (also speichern durch Macro)
					if ( pLib->IsModified() && !StarBASIC::IsRunning() )
					{
						String sTemp(SfxResId( IDS_SBERR_STOREREF ));
						sTemp += pLib->GetName();
                        WarningBox( NULL, WB_OK, sTemp ).Execute();
						// Einmal meckern reicht.
						pLib->SetModified( FALSE );
					}
				}
				else
				{
					SvStorageRef xStorage;
					if ( ( pInfo->GetStorageName() != aCurStorage ) && ( !pInfo->GetStorageName().EqualsAscii(szImbedded) ) )
						xStorage = new SvStorage( pInfo->GetStorageName() );
					else
						xStorage = &rStorage;
                    ImpStoreLibary( pLib, *xStorage );
				}
			}
		}
	}

	if ( !HasErrors() )
		bBasMgrModified = FALSE;
}

BOOL SfxBasicManager::ImpStoreLibary( StarBASIC* pLib, SvStorage& rStorage ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	DBG_ASSERT( pLib, "pLib = 0 (ImpStorageLibary)" );

	SvStorageRef xBasicStorage = rStorage.OpenStorage
							( String::CreateFromAscii(szBasicStorage), STREAM_STD_READWRITE, FALSE );

    String aStorName( rStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );
	if ( !xBasicStorage.Is() || xBasicStorage->GetError() )
	{
//		String aErrorText( BasicResId( IDS_SBERR_MGRSAVE ) );
//      aErrorText.SearchAndReplace( "XX", aStorName );
        StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_MGRSAVE, aStorName, ERRCODE_BUTTON_OK );
		((SfxBasicManager*)this)->pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENLIBSTORAGE, pLib->GetName() ) );
	}
	else
	{
		// In dem Basic-Storage liegt jede Lib in einem Stream...
		SvStorageStreamRef xBasicStream = xBasicStorage->OpenStream( pLib->GetName(), STREAM_STD_READWRITE );
		if ( !xBasicStream.Is() || xBasicStream->GetError() )
		{
//			String aErrorText( BasicResId( IDS_SBERR_LIBSAVE ) );
//			aErrorText.SearchAndReplace( "XX", pLib->GetName() );
			StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBSAVE,	pLib->GetName(), ERRCODE_BUTTON_OK );
			((SfxBasicManager*)this)->pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENLIBSTREAM, pLib->GetName() ) );
		}
		else
		{
			BasicLibInfo* pLibInfo = FindLibInfo( pLib );
			DBG_ASSERT( pLibInfo, "ImpStoreLibary: LibInfo?!" );

			xBasicStream->SetSize( 0 );
			xBasicStream->SetBufferSize( 1024 );

			// SBX_DONTSTORE, damit Child-Basics nicht gespeichert werden
			SetFlagToAllLibs( SBX_DONTSTORE, TRUE );
			// Aber das hier will ich jetzt speichern:
			pLib->ResetFlag( SBX_DONTSTORE );
			if ( pLibInfo->HasPassword() )
				xBasicStream->SetKey( szCryptingKey );
			BOOL bDone = pLib->Store( *xBasicStream );
			xBasicStream->SetBufferSize( 0 );
			if ( bDone )
			{
				// Diese Informationen immer verschluesseln...
				xBasicStream->SetBufferSize( 1024 );
				xBasicStream->SetKey( szCryptingKey );
				*xBasicStream << (ULONG)PASSWORD_MARKER;
				String aTmpPassword = pLibInfo->GetPassword();
				xBasicStream->WriteByteString( aTmpPassword, RTL_TEXTENCODING_MS_1252 );
				xBasicStream->SetBufferSize( 0 );
			}
			// Vorsichtshalber alle Dont't Store lassen...
			pLib->SetFlag( SBX_DONTSTORE );
//			SetFlagToAllLibs( SBX_DONTSTORE, FALSE );
			pLib->SetModified( FALSE );
			if( !xBasicStorage->Commit() )
				bDone = FALSE;
			xBasicStream->SetKey( ByteString() );
			DBG_ASSERT( bDone, "Warum geht Basic::Store() nicht ?" );
			return bDone;
		}
	}
	return FALSE;
}

/*
BOOL SfxBasicManager::ImpLoadLibary( BasicLibInfo* pLibInfo ) const
{
	return ImpLoadLibary( pLibInfo, 0 );
}
 */

BOOL SfxBasicManager::ImpLoadLibary( BasicLibInfo* pLibInfo, SvStorage* pCurStorage, BOOL bInfosOnly ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	DBG_ASSERT( pLibInfo, "LibInfo!?" );

	String aStorageName( pLibInfo->GetStorageName() );
	if ( !aStorageName.Len() || ( aStorageName.EqualsAscii(szImbedded) ) )
		aStorageName = GetStorageName();

	SvStorageRef xStorage;
	// Der aktuelle darf nicht nochmal geoffnet werden...
	if ( pCurStorage )
	{
        String aStorName( pCurStorage->GetName() );
        DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

        bool bWasAbsolute = FALSE;
        INetURLObject aCurStorageEntry = INetURLObject().smartRel2Abs( aStorName, bWasAbsolute );
        DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );

        INetURLObject aStorageEntry = INetURLObject().smartRel2Abs( aStorageName, bWasAbsolute );
        DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );

		if ( aCurStorageEntry == aStorageEntry )
			xStorage = pCurStorage;
	}

	if ( !xStorage.Is() )
		xStorage = new SvStorage( aStorageName, eStorageReadMode );

	SvStorageRef xBasicStorage = xStorage->OpenStorage
							( String::CreateFromAscii(szBasicStorage), eStorageReadMode, FALSE );

	if ( !xBasicStorage.Is() || xBasicStorage->GetError() )
	{
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_MGROPEN,	xStorage->GetName(), ERRCODE_BUTTON_OK );
		pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENLIBSTORAGE, pLibInfo->GetLibName() ) );
	}
	else
	{
		// In dem Basic-Storage liegt jede Lib in einem Stream...
		SvStorageStreamRef xBasicStream = xBasicStorage->OpenStream( pLibInfo->GetLibName(), eStreamReadMode );
		if ( !xBasicStream.Is() || xBasicStream->GetError() )
		{
			StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBLOAD , pLibInfo->GetLibName(), ERRCODE_BUTTON_OK );
			pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENLIBSTREAM, pLibInfo->GetLibName() ) );
		}
		else
		{
			BOOL bLoaded = FALSE;
			if ( xBasicStream->Seek( STREAM_SEEK_TO_END ) != 0 )
			{
				if ( !bInfosOnly )
				{
					if ( !pLibInfo->GetLib().Is() )
						pLibInfo->SetLib( new StarBASIC( GetStdLib() ) );
					xBasicStream->SetBufferSize( 1024 );
					xBasicStream->Seek( STREAM_SEEK_TO_BEGIN );
					bLoaded = ImplLoadBasic( *xBasicStream, pLibInfo->GetLibRef() );
					xBasicStream->SetBufferSize( 0 );
					pLibInfo->GetLib()->SetName( pLibInfo->GetLibName() );
					pLibInfo->GetLib()->SetModified( FALSE );
					pLibInfo->GetLib()->SetFlag( SBX_DONTSTORE );
				}
				else
				{
					// Das Basic skippen...
					xBasicStream->Seek( STREAM_SEEK_TO_BEGIN );
					ImplEncryptStream( *xBasicStream );
					SbxBase::Skip( *xBasicStream );
					bLoaded = TRUE;
				}
			}
			if ( !bLoaded )
			{
				StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBLOAD,	pLibInfo->GetLibName(), ERRCODE_BUTTON_OK );
				pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_BASICLOADERROR, pLibInfo->GetLibName() ) );
			}
			else
			{
				// Ggf. stehen weitere Informationen im Stream...
				xBasicStream->SetKey( szCryptingKey );
				xBasicStream->RefreshBuffer();
				ULONG nPasswordMarker = 0;
				*xBasicStream >> nPasswordMarker;
				if ( ( nPasswordMarker == PASSWORD_MARKER ) && !xBasicStream->IsEof() )
				{
					String aPassword;
					xBasicStream->ReadByteString(aPassword);
					pLibInfo->SetPassword( aPassword );
				}
				xBasicStream->SetKey( ByteString() );
				CheckModules( pLibInfo->GetLib(), pLibInfo->IsReference() );
			}
//			bBasMgrModified = TRUE;	// Warum?
			return bLoaded;
		}
	}
	return FALSE;
}

BOOL SfxBasicManager::ImplEncryptStream( SvStream& rStrm ) const
{
	ULONG nPos = rStrm.Tell();
	UINT32 nCreator;
	rStrm >> nCreator;
	rStrm.Seek( nPos );
	BOOL bProtected = FALSE;
	if ( nCreator != SBXCR_SBX )
	{
		// sollte nur bei verschluesselten Streams nicht stimmen.
		bProtected = TRUE;
		rStrm.SetKey( szCryptingKey );
		rStrm.RefreshBuffer();
	}
	return bProtected;
}

// Dieser Code ist notwendig, um das BASIC der Beta 1 laden zu koennen
BOOL SfxBasicManager::ImplLoadBasic( SvStream& rStrm, StarBASICRef& rOldBasic ) const
{
	BOOL bProtected = ImplEncryptStream( rStrm );
	SbxBaseRef xNew = SbxBase::Load( rStrm );
	BOOL bLoaded = FALSE;
	if( xNew.Is() )
	{
		if( xNew->IsA( TYPE(StarBASIC) ) )
		{
			StarBASIC* pNew = (StarBASIC*)(SbxBase*) xNew;
			// Den Parent des alten BASICs uebernehmen
			if( rOldBasic.Is() )
			{
				pNew->SetParent( rOldBasic->GetParent() );
				if( pNew->GetParent() )
					pNew->GetParent()->Insert( pNew );
				pNew->SetFlag( SBX_EXTSEARCH );
			}
			rOldBasic = pNew;
/*
			if( rOldBasic->GetParent() )
			{
				rOldBasic->GetParent()->Insert( rOldBasic );
				rOldBasic->SetFlag( SBX_EXTSEARCH );
			}
*/
			pNew->SetModified( FALSE );
			bLoaded = TRUE;
		}
	}
	if ( bProtected )
		rStrm.SetKey( ByteString() );
	return bLoaded;
}

BOOL SfxBasicManager::StoreLib( USHORT nLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	if ( !pLibInfo )
	{
//		String aErrorText( BasicResId( IDS_SBERR_LIBSAVE ) );
//		aErrorText.SearchAndReplace( "XX", "" );
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBSAVE,	String(), ERRCODE_BUTTON_OK );
		((SfxBasicManager*)this)->pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_LIBNOTFOUND, nLib ) );
		return FALSE;
	}
	if ( pLibInfo->GetLib() == 0 ) // Nicht geladen...
		return FALSE;

	String aStorageName( pLibInfo->GetStorageName() );
	if ( !aStorageName.Len() || ( aStorageName.EqualsAscii(szImbedded) ) )
		aStorageName = GetStorageName();

	if ( !aStorageName.Len() )
	{
//		String aErrorText( BasicResId( IDS_SBERR_LIBSAVE ) );
//		aErrorText.SearchAndReplace( "XX", pLibInfo->GetLibName() );
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBSAVE,	pLibInfo->GetLibName(), ERRCODE_BUTTON_OK );
		((SfxBasicManager*)this)->pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_NOSTORAGENAME, nLib ) );
		return FALSE;
	}

	SvStorageRef xStorage = new SvStorage( aStorageName );

	if ( !xStorage.Is() || xStorage->GetError() )
	{
//		String aErrorText( BasicResId( IDS_SBERR_LIBSAVE ) );
//		aErrorText.SearchAndReplace( "XX", pLibInfo->GetLibName() );
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBSAVE,	pLibInfo->GetLibName(), ERRCODE_BUTTON_OK );
		((SfxBasicManager*)this)->pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENSTORAGE, nLib ) );
	}
	else
        return ImpStoreLibary( pLibInfo->GetLib(), *xStorage );

	return FALSE;
}

void SfxBasicManager::CheckModules( StarBASIC* pLib, BOOL bReference ) const
{
	if ( !pLib )
		return;

	BOOL bModified = pLib->IsModified();

	for ( USHORT nMod = 0; nMod < pLib->GetModules()->Count(); nMod++ )
	{
		SbModule* pModule = (SbModule*)pLib->GetModules()->Get( nMod );
		DBG_ASSERT( pModule, "Modul nicht erhalten!" );
		if ( !pModule->IsCompiled() && !StarBASIC::GetErrorCode() )
			pLib->Compile( pModule );
	}

	// #67477, AB 8.12.99 On demand Compilieren bei referenzierten
	// Libraries sollte nicht zu modified fuehren
	if( !bModified && bReference )
	{
		DBG_ERROR( "Per Reference eingebundene Basic-Library ist nicht compiliert!" );
		pLib->SetModified( FALSE );
	}
}

StarBASIC* SfxBasicManager::AddLib( SvStorage& rStorage, const String& rLibName, BOOL bReference )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

    String aStorName( rStorage.GetName() );
    DBG_ASSERT( aStorName.Len(), "No Storage Name!" );

    bool bWasAbsolute = FALSE;
    String aStorageName = INetURLObject().smartRel2Abs( aStorName, bWasAbsolute ).GetMainURL();
    DBG_ASSERT( bWasAbsolute, "Storage name must be absolute!" );

	String aNewLibName( rLibName );
	while ( HasLib( aNewLibName ) )
		aNewLibName += '_';

	BasicLibInfo* pLibInfo = CreateLibInfo();
	// Erstmal mit dem Original-Namen, da sonst ImpLoadLibary nicht geht...
	pLibInfo->SetLibName( rLibName );
	// Funktioniert so aber nicht, wenn Name doppelt
//	USHORT nLibId = GetLibId( rLibName );
	USHORT nLibId = (USHORT) pLibs->GetPos( pLibInfo );

	// Vorm Laden StorageNamen setzen, da er mit pCurStorage verglichen wird.
	pLibInfo->SetStorageName( aStorageName );
    BOOL bLoaded = ImpLoadLibary( pLibInfo, &rStorage );

	if ( bLoaded )
	{
		if ( aNewLibName != rLibName )
			SetLibName( nLibId, aNewLibName );

		if ( bReference )
		{
			pLibInfo->GetLib()->SetModified( FALSE );	// Dann nicht speichern
			pLibInfo->SetRelStorageName( String() );
//			pLibInfo->CalcRelStorageName( GetStorageName() );
			pLibInfo->IsReference() = TRUE;
		}
		else
		{
			pLibInfo->GetLib()->SetModified( TRUE );		// Muss nach Add gespeichert werden!
			pLibInfo->SetStorageName( String::CreateFromAscii(szImbedded) );			// Im BasicManager-Storage speichern
		}
		bBasMgrModified = TRUE;
	}
	else
	{
		RemoveLib( nLibId, FALSE );
		pLibInfo = 0;
	}

	if( pLibInfo )
		return &*pLibInfo->GetLib() ;
	else
		return 0;
}

BOOL SfxBasicManager::IsReference( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
		return pLibInfo->IsReference();

	return FALSE;
}

BOOL SfxBasicManager::IsExtern( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
		return pLibInfo->IsExtern();

	return FALSE;
}

BOOL SfxBasicManager::RemoveLib( USHORT nLib )
{
	// Nur physikalisch loeschen, wenn keine Referenz.
	return RemoveLib( nLib, !IsReference( nLib ) );
}

BOOL SfxBasicManager::RemoveLib( USHORT nLib, BOOL bDelBasicFromStorage )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	DBG_ASSERT( nLib, "Standard-Lib kann nicht entfernt werden!" );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib nicht gefunden!" );

	if ( !pLibInfo || !nLib )
	{
//		String aErrorText( BasicResId( IDS_SBERR_REMOVELIB ) );
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_REMOVELIB, String(), ERRCODE_BUTTON_OK );
		pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_STDLIB, pLibInfo->GetLibName() ) );
		return FALSE;
	}

	// Wenn einer der Streams nicht geoeffnet werden kann, ist es kein
	// Fehler, es gibt halt noch nichts zum loeschen, weil das Basic noch
	// nie geschrieben wurde...
	if ( bDelBasicFromStorage && !pLibInfo->IsReference() &&
			( !pLibInfo->IsExtern() || SvStorage::IsStorageFile( pLibInfo->GetStorageName() ) ) )
	{
		SvStorageRef xStorage;
		if ( !pLibInfo->IsExtern() )
			xStorage = new SvStorage( GetStorageName() );
		else
			xStorage = new SvStorage( pLibInfo->GetStorageName() );

		if ( xStorage->IsStorage( String::CreateFromAscii(szBasicStorage) ) )
		{
			SvStorageRef xBasicStorage = xStorage->OpenStorage
							( String::CreateFromAscii(szBasicStorage), STREAM_STD_READWRITE, FALSE );

			if ( !xBasicStorage.Is() || xBasicStorage->GetError() )
			{
//				String aErrorText( BasicResId( IDS_SBERR_REMOVELIB ) );
				StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_REMOVELIB, String(), ERRCODE_BUTTON_OK );
				pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_OPENLIBSTORAGE, pLibInfo->GetLibName() ) );
			}
			else if ( xBasicStorage->IsStream( pLibInfo->GetLibName() ) )
			{
				xBasicStorage->Remove( pLibInfo->GetLibName() );
				xBasicStorage->Commit();

				// Wenn kein weiterer Stream vorhanden,
				// dann auch den SubStorage loeschen.
				SvStorageInfoList aInfoList( 0, 4 );
				xBasicStorage->FillInfoList( &aInfoList );
				if ( !aInfoList.Count() )
				{
					xBasicStorage.Clear();
					xStorage->Remove( String::CreateFromAscii(szBasicStorage) );
					xStorage->Commit();
					// Wenn kein weiterer Streams oder SubStorages vorhanden,
					// dann auch den Storage loeschen.
					aInfoList.Clear();
					xStorage->FillInfoList( &aInfoList );
					if ( !aInfoList.Count() )
					{
                        String aName( xStorage->GetName() );
						xStorage.Clear();
                        SfxContentHelper::Kill( aName );
					}
				}
			}
		}
	}
	bBasMgrModified = TRUE;
	if ( pLibInfo->GetLib().Is() )
		GetStdLib()->Remove( pLibInfo->GetLib() );
	delete pLibs->Remove( pLibInfo );
	return TRUE;	// Remove hat geklappt, Del unwichtig.
}

USHORT SfxBasicManager::GetLibCount() const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	return (USHORT)pLibs->Count();
}

StarBASIC* SfxBasicManager::GetLib( USHORT nLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	DBG_ASSERT( pInf, "Lib existiert nicht!" );
	if ( pInf )
		return pInf->GetLib();
	return 0;
}

StarBASIC* SfxBasicManager::GetStdLib() const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	StarBASIC* pLib = GetLib( 0 );
	return pLib;
}

StarBASIC* SfxBasicManager::GetLib( const String& rName ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = pLibs->First();
	while ( pInf )
	{
		if ( pInf->GetLibName().CompareIgnoreCaseToAscii( rName ) == COMPARE_EQUAL )		// prueffen, ob vorhanden...
			return pInf->GetLib();

		pInf = pLibs->Next();
	}
	return 0;
}

USHORT SfxBasicManager::GetLibId( const String& rName ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = pLibs->First();
	while ( pInf )
	{
		if ( pInf->GetLibName().CompareIgnoreCaseToAscii( rName ) == COMPARE_EQUAL )
			return (USHORT)pLibs->GetCurPos();

		pInf = pLibs->Next();
	}
	return LIB_NOTFOUND;
}

USHORT SfxBasicManager::GetLibId( StarBASIC* pLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = pLibs->First();
	while ( pInf )
	{
		if ( pInf->GetLib() == pLib )
			return (USHORT)pLibs->GetCurPos();

		pInf = pLibs->Next();
	}
	return LIB_NOTFOUND;
}

BOOL SfxBasicManager::HasLib( const String& rName ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = pLibs->First();
	while ( pInf )
	{
		if ( pInf->GetLibName().CompareIgnoreCaseToAscii( rName ) == COMPARE_EQUAL )
			return TRUE;

		pInf = pLibs->Next();
	}
	return FALSE;
}

BOOL SfxBasicManager::SetLibName( USHORT nLib, const String& rName )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
	{
		pLibInfo->SetLibName( rName );
		if ( pLibInfo->GetLib().Is() )
		{
			pLibInfo->GetLib()->SetName( rName );
			pLibInfo->GetLib()->SetModified( TRUE );
		}
		bBasMgrModified = TRUE;
		return TRUE;
	}
	return FALSE;
}

String SfxBasicManager::GetLibName( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
		return pLibInfo->GetLibName();
	return String();
}

BOOL SfxBasicManager::SetLibStorageName( USHORT nLib, const String& rName )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = nLib ? pLibs->GetObject( nLib ) : 0;
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
	{
		pLibInfo->SetStorageName( rName );
//		pLibInfo->CalcRelStorageName( GetStorageName() );
		pLibInfo->SetRelStorageName( String() );
		bBasMgrModified = TRUE;
		return TRUE;
	}
	return FALSE;
}

String SfxBasicManager::GetLibStorageName( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
		return pLibInfo->GetStorageName();
	return String();
}

String SfxBasicManager::GetRelLibStorageName( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
		return pLibInfo->GetRelStorageName();
	return String();
}

BOOL SfxBasicManager::IsLibLoaded( USHORT nLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
		return pLibInfo->GetLib().Is();
	return FALSE;
}

BOOL SfxBasicManager::LoadLib( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BOOL bDone = FALSE;
	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
	{
        bDone = ImpLoadLibary( pLibInfo, NULL, NULL );
		StarBASIC* pLib = GetLib( nLib );
		if ( pLib )
		{
//			pLib->SetParent( GetStdLib() );
			GetStdLib()->Insert( pLib );
			pLib->SetFlag( SBX_EXTSEARCH );
		}
	}
	else
	{
//		String aErrorText( BasicResId( IDS_SBERR_LIBLOAD ) );
//		aErrorText.SearchAndReplace( "XX", "" );
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_LIBLOAD,	String(), ERRCODE_BUTTON_OK );
		pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_LIBNOTFOUND, nLib ) );
	}
	return bDone;
}

BOOL SfxBasicManager::UnloadLib( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	DBG_ASSERT( nLib, "UnloadLib nicht fuer StandardLib verwerden!" );
	if ( !nLib )
	{
//		String aErrorText( BasicResId( IDS_SBERR_UNLOADLIB ) );
		StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_UNLOADLIB, String(), ERRCODE_BUTTON_OK );
		pErrorMgr->InsertError( BasicError( *pErrInf, BASERR_REASON_STDLIB, nLib ) );
		return FALSE;
	}
	BasicLibInfo* pLibInfo = pLibs->GetObject( nLib );
	DBG_ASSERT( pLibInfo, "Lib?!" );
	if ( pLibInfo )
	{
		StarBASIC* pLib = pLibInfo->GetLib();
		if ( pLib )
			GetStdLib()->Remove( pLib );
		pLibInfo->GetLibRef().Clear();
		return TRUE;
	}
//	String aErrorText( BasicResId( IDS_SBERR_UNLOADLIB ) );
	StringErrorInfo* pErrInf = new StringErrorInfo( ERRCODE_BASMGR_UNLOADLIB ,	String(), ERRCODE_BUTTON_OK );
	pErrorMgr->InsertError( BasicError( BASERR_ID_UNLOADLIB, BASERR_REASON_LIBNOTFOUND, nLib ) );
	return FALSE;
}

StarBASIC* SfxBasicManager::CreateLib( const String& rLibName )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	if ( GetLib( rLibName ) )
		return 0;

	BasicLibInfo* pLibInfo = CreateLibInfo();
	StarBASIC* pNew = new StarBASIC( GetStdLib() );
	GetStdLib()->Insert( pNew );
	pNew->SetFlag( SBX_EXTSEARCH | SBX_DONTSTORE );
	pLibInfo->SetLib( pNew );
	pLibInfo->SetLibName( rLibName );
	pLibInfo->GetLib()->SetName( rLibName );
	return pLibInfo->GetLib();
}

void SfxBasicManager::AddLib( StarBASIC* pLib )
{
	String aLibName( pLib->GetName() );
	while ( HasLib( aLibName ) )
		aLibName += '_';
	if ( aLibName != pLib->GetName() )
		pLib->SetName( aLibName );

	BasicLibInfo* pLibInfo = CreateLibInfo();
	pLibInfo->SetLib( pLib );
	pLibInfo->SetLibName( aLibName );
	GetStdLib()->Insert( pLib );	// Jede Lib hat Standard als Parent
	pLib->SetFlag( SBX_DONTSTORE );
	bBasMgrModified = TRUE;
}

BOOL SfxBasicManager::MoveLib( USHORT nLib, USHORT nNewPos )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
#ifdef DBG_UTIL
	InfoBox( 0, DEFINE_CONST_UNICODE("Sorry, SfxBasicManager::MoveLib not implemented ") ).Execute();
#endif
	return FALSE;
}

BasicLibInfo* SfxBasicManager::FindLibInfo( StarBASIC* pBasic ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = ((SfxBasicManager*)this)->pLibs->First();
	while ( pInf )
	{
		if ( pInf->GetLib() == pBasic )
			return pInf;

		pInf = ((SfxBasicManager*)this)->pLibs->Next();
	}
	return 0;
}


BOOL SfxBasicManager::IsModified() const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	if ( bBasMgrModified )
		return TRUE;
	return IsBasicModified();
}

BOOL SfxBasicManager::IsBasicModified() const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );

	BasicLibInfo* pInf = pLibs->First();
	while ( pInf )
	{
		if ( pInf->GetLib().Is() && pInf->GetLib()->IsModified() )
			return TRUE;

		pInf = pLibs->Next();
	}
	return FALSE;
}

void SfxBasicManager::SetFlagToAllLibs( short nFlag, BOOL bSet ) const
{
	USHORT nLibs = GetLibCount();
	for ( USHORT nL = 0; nL < nLibs; nL++ )
	{
		BasicLibInfo* pInfo = pLibs->GetObject( nL );
		DBG_ASSERT( pInfo, "Info?!" );
		StarBASIC* pLib = pInfo->GetLib();
		if ( pLib )
		{
			if ( bSet )
				pLib->SetFlag( nFlag );
			else
				pLib->ResetFlag( nFlag );
		}
	}
}

BOOL SfxBasicManager::HasErrors()
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	return pErrorMgr->HasErrors();
}

void SfxBasicManager::ClearErrors()
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	pErrorMgr->Reset();
}

BasicError* SfxBasicManager::GetFirstError()
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	return pErrorMgr->GetFirstError();
}

BasicError* SfxBasicManager::GetNextError()
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	return pErrorMgr->GetNextError();
}

void SfxBasicManager::ImplGetPassword( USHORT nLib ) const
{
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	if ( !pInf->HasPassword() && !IsLibLoaded( nLib ) )
        ImpLoadLibary( pInf, NULL, TRUE );
}


BOOL SfxBasicManager::HasPassword( USHORT nLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	if ( pInf )
	{
		ImplGetPassword( nLib );
		return pInf->HasPassword();
	}
	return FALSE;
}

String SfxBasicManager::GetPassword( USHORT nLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	String aPassword;
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	if ( pInf )
	{
		ImplGetPassword( nLib );
		aPassword = pInf->GetPassword();
	}
	return aPassword;
}

void SfxBasicManager::SetPassword( USHORT nLib, const String& rNewPassword )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	DBG_ASSERT( pInf, "Lib existiert nicht!" );
	if ( pInf )
	{
		pInf->SetPassword( rNewPassword );
		StarBASIC* pBasic = pInf->GetLib();
		DBG_ASSERT( pBasic, "SetPassword nur bei geladenem BASIC!" );
		if ( pBasic )
			pBasic->SetModified( TRUE );
	}
}

BOOL SfxBasicManager::IsPasswordVerified(  USHORT nLib ) const
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	DBG_ASSERT( pInf, "Lib existiert nicht!" );
	if ( pInf )
		return pInf->IsPasswordVerified();
	return FALSE;
}

void SfxBasicManager::SetPasswordVerified( USHORT nLib )
{
	DBG_CHKTHIS( SfxBasicManager, 0 );
	BasicLibInfo* pInf = pLibs->GetObject( nLib );
	DBG_ASSERT( pInf, "Lib existiert nicht!" );
	if ( pInf )
		pInf->SetPasswordVerified();
}


