/*************************************************************************
 *
 *  $RCSfile: opipeconnection.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#include "opipeconnection.hxx"

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_FRAME_XTASKSSUPPLIER_HPP_
#include <com/sun/star/frame/XTasksSupplier.hpp>
#endif

#ifndef _COM_SUN_STAR_CONTAINER_XENUMERATIONACCESS_HPP_
#include <com/sun/star/container/XEnumerationAccess.hpp>
#endif

#ifndef _COM_SUN_STAR_CONTAINER_XENUMERATION_HPP_
#include <com/sun/star/container/XEnumeration.hpp>
#endif

#ifndef _COM_SUN_STAR_MOZILLA_XPLUGININSTANCE_HPP_
#include <com/sun/star/mozilla/XPluginInstance.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _OSL_PROCESS_H_
#include <osl/process.h>
#endif

#ifndef _OSL_DIAGNOSE_H_
#include <osl/diagnose.h>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

using namespace ::osl							;
using namespace ::rtl							;
using namespace ::cppu							;
using namespace ::com::sun::star::uno			;
using namespace ::com::sun::star::lang			;
using namespace ::com::sun::star::io			;
using namespace ::com::sun::star::connection	;
using namespace ::com::sun::star::frame			;
using namespace ::com::sun::star::container		;
using namespace ::com::sun::star::mozilla		;

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

#define	SERVICENAME_DESKTOP									OUString(RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.Desktop"				))
//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//	constructor
//*****************************************************************************************************************
OPipeConnection::OPipeConnection( const Reference< XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
		:	OWeakObject	(			)
		// Init member
		,	m_xFactory	( xFactory	)
{
	// Safe impossible cases
	// We need valid incoming parameter for working.
	OSL_ENSHURE( impldbg_checkParameter_OPipeConnectionCtor( xFactory ), "OPipeConnection::OPipeConnection()\nInvalid parameter detected!\n" );

	// Open pipes.
	oslIOResource	aReadConnection	;
	oslIOResource	aWriteConnection;

	m_aReadPipe = NULL;
	m_aWritePipe= NULL;
	if	(
			( osl_getIOResource( &aReadConnection , "READACCESS" )	==	osl_Process_E_None	)	&&
			( osl_getIOResource( &aWriteConnection, "WRITEACCESS")	==	osl_Process_E_None	)
		)
	{
		m_aReadPipe = aReadConnection.Descriptor.File	;
		m_aWritePipe= aWriteConnection.Descriptor.File	;
	}
	// Set state for working.
	m_bReady = (m_aReadPipe!=NULL && m_aWritePipe!=NULL);

	// Safe impossible cases.
	// We cant work without a pipe.
	OSL_ENSHURE( !(m_bReady==sal_False), "OPipeConnection::OPipeConnection()\nCreation of pipes failed. We can't work.\n" );
}

//*****************************************************************************************************************
//	destructor (protected!)
//*****************************************************************************************************************
OPipeConnection::~OPipeConnection()
{
	// We must close any current opend pipe, if programmer forget this.
	close();
}

//*****************************************************************************************************************
//	XInterface
//*****************************************************************************************************************

void SAL_CALL OPipeConnection::acquire() throw( RuntimeException )
{ OWeakObject::acquire(); }

void SAL_CALL OPipeConnection::release() throw( RuntimeException )
{ OWeakObject::release(); }

Any SAL_CALL OPipeConnection::queryInterface( const Type& aType ) throw( RuntimeException )
{
	Any aReturn	( ::cppu::queryInterface( aType, static_cast< XConnection* >( this ) ) );
	if ( aReturn.hasValue() == sal_False )
	{
		aReturn = OWeakObject::queryInterface( aType );
	}
	return aReturn;
}

//*****************************************************************************************************************
//	XConnection
//*****************************************************************************************************************
sal_Int32 SAL_CALL OPipeConnection::read(	Sequence< sal_Int8 >&	seqReadBytes	,
											sal_Int32				nBytesToRead	) throw(	IOException		,
																								RuntimeException)
{
	// Ready for multithreading
	MutexGuard aGuard( m_aReadMutex );
	// Safe impossible cases
	// Method is not defined for all incoming parameter.
	OSL_ENSHURE( impldbg_checkParameter_read( seqReadBytes, nBytesToRead ), "OPipeConnection::read()\nInvalid parameter detected!\n" );
	// We cant work at closed pipe!
	OSL_ENSHURE( !(m_bReady==sal_False), "OPipeConnection::read()\nPipe is closed! You can't read from it.\n" );

	// Set default return value.
	sal_uInt64 nReadedBytes;
	// Work only, if object valid.
	if ( m_bReady==sal_True )
	{
		// Try to read specified count of bytes from pipe.

		// Attention: OSL works with unsigned values for count reading bytes.
		// We can return signed values only! But there can't be a mismatch between these two types ...
		// We call osl function with a signed value - and these is the top of possible counting!
		if ( osl_readFile( m_aReadPipe, seqReadBytes.getArray(), nBytesToRead, &nReadedBytes ) != osl_File_E_None )
		{
			IOException aTmpException;
			throw( aTmpException );
		}
	}

	// Return results.
	return nReadedBytes;
}

//*****************************************************************************************************************
//	XConnection
//*****************************************************************************************************************
void SAL_CALL OPipeConnection::write( const Sequence< sal_Int8 >& seqData ) throw(	IOException		,
																					RuntimeException)
{
	// Ready for multithreading
	MutexGuard aGuard( m_aWriteMutex );
	// Safe impossible cases
	// Method is not defined for all incoming parameter.
	OSL_ENSHURE( impldbg_checkParameter_write( seqData ), "OPipeConnection::write()\nInvalid parameter detected!\n" );
	// We cant work at closed pipe!
	OSL_ENSHURE( !(m_bReady==sal_False), "OPipeConnection::write()\nPipe is closed! You can't write to it.\n" );

	if ( m_bReady==sal_True )
	{
		// Try to write specified bytes to pipe.
		sal_uInt64	nWrittenBytes	= 0						;
		sal_Int32	nDataLength 	= seqData.getLength()	;
		if ( osl_writeFile( m_aWritePipe, seqData.getConstArray(), nDataLength, &nWrittenBytes ) != osl_File_E_None )
		{
			IOException aTmpException;
			throw( aTmpException );
		}
		// Safe impossible cases
		// Some bytes can't be written!
		OSL_ENSHURE( !(nWrittenBytes!=nDataLength), "OPipeConnection::write()\nSome bytes can't be written!\n" );
	}
}

//*****************************************************************************************************************
//	XConnection
//*****************************************************************************************************************
void SAL_CALL OPipeConnection::flush() throw(	IOException		,
												RuntimeException)
{
	//OSL_ENSHURE( sal_False, "OPipeConnection::flush()\nNot implemented yet!\n" )
}

//*****************************************************************************************************************
//	XConnection
//*****************************************************************************************************************
void SAL_CALL OPipeConnection::close() throw(	IOException		,
												RuntimeException)
{
	// Ready for multithreading
	MutexGuard aReadGuard( m_aReadMutex );
	MutexGuard aWriteGuard( m_aWriteMutex );
	// Safe impossible cases
	OSL_ENSHURE( !(m_bReady==sal_False), "OPipeConnection::close()\nAlready called. Don't do it again.\n" );

	// Close pipe for reading.
	if ( m_aReadPipe != NULL )
	{
		osl_closeFile( m_aReadPipe );
		m_aReadPipe = NULL;
	}

	// Close pipe for writing.
	if ( m_aWritePipe != NULL )
	{
		osl_closeFile( m_aWritePipe );
		m_aWritePipe = NULL;
	}

	// Reset state to prevent against calls after closing!
	m_bReady = sal_False;
	// Connection has lost - we must close all plugins in current tree.
	impl_closeAllPlugIns();
}

//*****************************************************************************************************************
//	XConnection
//*****************************************************************************************************************
OUString SAL_CALL OPipeConnection::getDescription() throw( RuntimeException )
{
	return PIPEDESCRIPTION;
}

//*****************************************************************************************************************
//	private method
//*****************************************************************************************************************
void OPipeConnection::impl_closeAllPlugIns()
{
	// We try to close all plugins.
	// Plugins are direct childs of tree root - th desktop.
	// We can instanciate the desktop - it's a OneInstance service and must exist at this mopment!
	Reference< XTasksSupplier > xDesktop( m_xFactory->createInstance( SERVICENAME_DESKTOP ), UNO_QUERY );
	// Safe impossible cases.
	// Desktop must exist and support searched interface at this moment!
	OSL_ENSHURE( !(xDesktop.is()==sal_False), "OPipeConnection::impl_closeAllPlugIns()\nDesktop not exist or don't support XTasksSupplier interface!\n" );
	if ( xDesktop.is()==sal_True )
	{
		// Get all direct childs of desktop as oneway enumeration.
		Reference< XEnumerationAccess > xAccess = xDesktop->getTasks();
		// Safe impossible cases.
		// Desktop live at this moment. He is dead, if there are no direct childs anymore!
		// => Childs must exist at this moment!
		OSL_ENSHURE( !(xAccess.is()==sal_False), "OPipeConnection::impl_closeAllPlugIns()\nDesktop live but has no childs ... that is not allowed!\n" );
		Reference< XEnumeration > xTasks	= xAccess->createEnumeration();
		// Step over all tasks. (PlugIn is a task and frame to!)
		while ( xTasks->hasMoreElements() == sal_True )
		{
			// Get next element from list and cast it to XPluginInstance interface.
			// A PlugIn support this special interface only - a normal task don't do it!
			Reference< XTask > xTask;
			xTasks->nextElement() >>= xTask;
			Reference< XPluginInstance > xCheckPlugIn( xTask, UNO_QUERY );
			// If current element support searched interface ...
			if ( xCheckPlugIn.is() == sal_True )
			{
				// ... we have found a plugin and can destroy it!
				// It will rebuild ouer current frame tree by himself - remove from desktop and ...
				xCheckPlugIn->stop();
				xCheckPlugIn->destroy();
			}
		}
	}
}

//_________________________________________________________________________________________________________________
//	debug methods
//_________________________________________________________________________________________________________________

/*-----------------------------------------------------------------------------------------------------------------
	The follow methods checks the parameter for other functions. If a parameter or his value is non valid,
	we return "sal_False". (else sal_True) This mechanism is used to throw an ASSERT!

	ATTENTION

		If you miss a test for one of this parameters, contact the autor or add it himself !(?)
		But ... look for right testing! See using of this methods!
-----------------------------------------------------------------------------------------------------------------*/

#ifdef _DEBUG

//*****************************************************************************************************************
sal_Bool OPipeConnection::impldbg_checkParameter_OPipeConnectionCtor( const Reference< XMultiServiceFactory >& xFactory )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &xFactory		==	NULL		)	||
			( xFactory.is()	==	sal_False	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OPipeConnection::impldbg_checkParameter_read(	SEQUENCE< sal_Int8 >&	seqReadBytes	,
														sal_Int32				nBytesToRead	)
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &seqReadBytes				==	NULL			)	||
			( nBytesToRead				<	1				)	||
			( seqReadBytes.getLength()	<	nBytesToRead	)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

//*****************************************************************************************************************
sal_Bool OPipeConnection::impldbg_checkParameter_write( const SEQUENCE< sal_Int8 >& seqData )
{
	// Set default return value.
	sal_Bool bOK = sal_True;
	// Check parameter.
	if	(
			( &seqData				==	NULL	)	||
			( seqData.getLength()	<	1		)
		)
	{
		bOK = sal_False ;
	}
	// Return result of check.
	return bOK ;
}

#endif	//	#ifdef ENABLE_ASSERTIONS
