/*************************************************************************
 *
 *  $RCSfile: remote.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "remote.hxx"
#include <usr/smart2uno.hxx>
#include <usr/services.hxx>
#include <usr/factoryhlp.hxx>
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XCONTAINER_HXX_
#include <smart/com/sun/star/container/XContainer.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XINDEXCONTAINER_HXX_
#include <smart/com/sun/star/container/XIndexContainer.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XNAMEREPLACE_HXX_
#include <smart/com/sun/star/container/XNameReplace.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XCONTAINERLISTENER_HXX_
#include <smart/com/sun/star/container/XContainerListener.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XSET_HXX_
#include <smart/com/sun/star/container/XSet.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_CONTAINEREVENT_HXX_
#include <smart/com/sun/star/container/ContainerEvent.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XINDEXREPLACE_HXX_
#include <smart/com/sun/star/container/XIndexReplace.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XNAMECONTAINER_HXX_
#include <smart/com/sun/star/container/XNameContainer.hxx>
#endif
#ifndef _SMART_COM_SUN_STAR_CONTAINER_XCONTENTENUMERATIONACCESS_HXX_
#include <smart/com/sun/star/container/XContentEnumerationAccess.hxx>
#endif
#include <svtools/libcall.hxx>

#include <vos/dynload.hxx>

#ifdef _USE_NAMESPACE
using namespace vos;
using namespace usr;
#endif

RemoteManager_Impl::RemoteManager_Impl(const XMultiServiceFactoryRef & xMgr)
	: m_xMgr(xMgr)
{
}

RemoteManager_Impl::~RemoteManager_Impl()
{
}

// XInterface
BOOL RemoteManager_Impl::queryInterface( Uik aUik, XInterfaceRef & rOut )
{
	if( aUik == XConnectCallback::getSmartUik() )
		rOut = (XConnectCallback *)this;
	else
		return UsrObject::queryInterface( aUik, rOut );
	return TRUE;
}

void RemoteManager_Impl::attemptConnect(const XConnectionServerSideRef& connection)
	THROWS( (SecurityException, UsrSystemException) )
{
	// Fuer Schedule Hotsync muss der Schedule Wrappercode angezogen werden,
	// bis das automatisch durch usr erfolgt.
	static NAMESPACE_VOS(OModule)* pS2uModule = NULL;
	if (pS2uModule == NULL)
	{
		pS2uModule = new NAMESPACE_VOS(OModule)();

		if (!pS2uModule->load( SVLIBRARY( "ssw" )))
		{
			delete pS2uModule;
			pS2uModule = NULL;
		}
	}

	USR_loadS2uModule();
	connection->provideRemoteObject(m_xMgr);
}


//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
PalmPilotRemoteManager_Impl::PalmPilotRemoteManager_Impl(const XMultiServiceFactoryRef & xMgr)
	: RemoteManager_Impl( XMultiServiceFactoryRef() )
	, xOriginalMgr(xMgr)
{
}

void PalmPilotRemoteManager_Impl::attemptConnect(const XConnectionServerSideRef& connection)
	THROWS( (SecurityException, UsrSystemException) )
{
	{
	OGuard aGuard( aMutex );
	if( !m_xMgr.is() )
	{
		XMultiServiceFactoryRef xPalmPilotMgr = createServiceManager();
		XSetRef xSet( xPalmPilotMgr, USR_QUERY );
		sal_Char * szServices[ ] =
		{
			"com.sun.star.schedule.DataView",
			"com.sun.star.schedule.PilotCfg",
			"com.sun.star.schedule.AddressBookFields",
			"com.sun.star.schedule.ScheduleManager",
			"com.sun.star.data.DatabaseEngine",
			"com.sun.star.address.AddressBookSources",
			"com.sun.star.address.AddressBookRecordContainer",
			"com.sun.star.address.AddressBookPilotConduitSettings",
			"com.sun.star.address.AddressBookResultSet"
		};
		for( INT32 i = 0; i < sizeof( szServices ) / sizeof( *szServices );
			 i++ )
		{
			XContentEnumerationAccessRef xEA( xOriginalMgr, USR_QUERY );
			XEnumerationRef xEnum = xEA->createContentEnumeration( ::rtl::OUString::createFromAscii(szServices[i]) );
			if( xEnum.is() && xEnum->hasMoreElements() )
			{
				UsrAny aEle = xEnum->nextElement();
				if( aEle.getReflection()->getTypeClass() == TypeClass_INTERFACE )
				{
					XSingleServiceFactoryRef xOrgFactory( *(XInterfaceRef *)aEle.get(), USR_QUERY );
					XSingleServiceFactoryRef xProxyFact = createFactoryProxy( xPalmPilotMgr, xOrgFactory );
					xSet->insert( UsrAny( &xProxyFact, XSingleServiceFactory_getReflection() ) );
				}
			}
		}
		m_xMgr = xPalmPilotMgr;
	}
	}
	RemoteManager_Impl::attemptConnect(connection);
}


