/*************************************************************************
 *
 *  $RCSfile: downloadcontroller.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_AWT_XVCLCONTAINERLISTENER_HPP_
#include <com/sun/star/awt/XVclContainerListener.hpp>
#endif
#ifndef _COM_SUN_STAR_AWT_XPROGRESSMONITOR_HPP_
#include <com/sun/star/awt/XProgressMonitor.hpp>
#endif
#ifndef _COM_SUN_STAR_AWT_XPROGRESSBAR_HPP_
#include <com/sun/star/awt/XProgressBar.hpp>
#endif
#ifndef _COM_SUN_STAR_AWT_XBUTTON_HPP_
#include <com/sun/star/awt/XButton.hpp>
#endif
#ifndef _COM_SUN_STAR_AWT_XLAYOUTCONSTRAINS_HPP_
#include <com/sun/star/awt/XLayoutConstrains.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_XURLTRANSFORMER_HPP_
#include <com/sun/star/util/XURLTransformer.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_FRAME_XDISPATCHPROVIDER_HPP_
#include <com/sun/star/frame/XDispatchProvider.hpp>
#endif
#ifndef _UNOTOOLS_PROCESSFACTORY_HXX
#include <unotools/processfactory.hxx>
#endif

#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#include <osl/mutex.hxx>
#include <vcl/svapp.hxx>
#include <vcl/filedlg.hxx>
//ASDBG #include <usr/services.hxx>
#include <vcl/button.hxx>
#include <tools/bigint.hxx>

#include "downloadcontroller.hxx"
#include "downimpl.hxx"
#include "sfxresid.hxx"
#include "bastyp.hrc"
#include "inimgr.hxx"
#include "helper.hxx"

//------------------------------------------------------------------------------------------------------------
using namespace ::vos ;
using namespace ::osl ;

//------------------------------------------------------------------------------------------------------------
// start construct/destruct
//------------------------------------------------------------------------------------------------------------
DownloadController::DownloadController( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& xFactory )
	: bInClose( sal_False )
	, bInCancel( sal_False )
	, bDownloading( sal_False )
	, nUserEventID( 0 )
	, nAskEventID( 0 )
	, nTotalSize( 0 )
{
	aDownloadTimer.SetTimeout( 1000 );
	aDownloadTimer.SetTimeoutHdl( LINK(this, DownloadController, Download_Impl ) );
}

DownloadController::~DownloadController ()
{
	if ( nUserEventID )
		Application::RemoveUserEvent ( nUserEventID ) ;

	if ( nAskEventID )
		Application::RemoveUserEvent ( nAskEventID ) ;

	if ( pDownloader )
	{
		// ended, but not cancelled
		SfxBroadcaster *pBrdCst = pDownloader->GetBroadcaster();
		if ( pBrdCst )
			EndListening( *pBrdCst );
		delete pDownloader;
	}
}

//------------------------------------------------------------------------------------------------------------
// end construct/destruct
//------------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
// start interface methods
//------------------------------------------------------------------------------------------------------------
sal_Bool DownloadController::StartDownload ( const String& rSource )
{
	::vos::OClearableGuard aGuard( m_aMutex );

	// save impossible cases
	// Call this method in right order. Start-Cancel-Start andn ot Start-Start-Start.... !!!
	DBG_ASSERT ( (!bDownloading) , "DownloadController::StartDownload()\nDownload already active. Don't call this more then one at time.") ;
	if ( bDownloading )
		return sal_False ;

	_aSourceURL.SetURL( rSource );
	if ( _aSourceURL.HasError() )
	{
		String aURL( _aSourceURL.GetMainURL() );
		aGuard.clear();
		::vos::OGuard aSolarGuard( Application::GetSolarMutex() );
		String aError( SfxResId(ERR_WRONG_URL) );
 		aError += aURL;
		InfoBox( NULL, aError ).Execute();
		return sal_False ;
	}

	aLocalFile.Invalidate();
	bInAsk = sal_False;

	aStat.nStartTime	= Time::GetSystemTicks ()	;
	aStat.nBytesRec		= 0							;
	aStat.bOwnPercent	= sal_False					;
	aStat.nPercent		= 0							;
	aStat.aStatusString	= String()					;

	Download_Impl(0);

	::com::sun::star::uno::Reference< ::com::sun::star::frame::XController >  xTmp( this );
	pDownloader = new Downloader_Impl;
    SfxBroadcaster *pBrdCst = pDownloader->GetBroadcaster();
    if ( pBrdCst )
        StartListening( *pBrdCst );

    bDownloading = sal_True;
	bDownloading = pDownloader->start( _aSourceURL.GetMainURL() );
	if ( bDownloading )
	{
		aDownloadTimer.Start() ;
		nAskEventID = Application::PostUserEvent( LINK(this, DownloadController, AskLocalFile_Impl) );
	}
    else
        EndListening( *pBrdCst );

	return bDownloading;
}

void DownloadController::CancelDownload_Impl()
{
	::vos::OGuard aGuard( m_aMutex );
	if ( pDownloader && !bInCancel && bDownloading )
	{
		bInCancel = sal_True ;
		SfxBroadcaster *pBrdCst = pDownloader->GetBroadcaster();

		// end listening to avoid callbacks
		if ( pBrdCst )
			EndListening( *pBrdCst );
		pDownloader->abort();
		DELETEZ( pDownloader );
	}

	bDownloading = sal_False;
}

/*
#define	DOWNLOADCONTROLLER_HTTP_ROOT_URL						"private:httpcache"
#define	DOWNLOADCONTROLLER_CACHE_URL							".cache:"
sal_Bool DownloadController::CreateNewDownload_HTTP ()
{
	nAskEventID = Application::PostUserEvent( LINK(this, DownloadController, AskLocalFile_Impl) );

	String aURL(DOWNLOADCONTROLLER_CACHE_URL);
	aURL += DOWNLOADCONTROLLER_HTTP_ROOT_URL ;

	String aRequestedURL = _aSourceURL.GetMainURL();
	aURL = DOWNLOADCONTROLLER_HTTP_ROOT_URL;
	aURL += '#';
	aURL += aRequestedURL;
}
*/

void DownloadController::SendClose ()
{
	::vos::OClearableGuard aGuard( m_aMutex );

	// prevent a second call of this method !
	if ( bInClose )
		return ;

	bInClose = sal_True ;
	if ( _xFrame.is() )
	{
		::com::sun::star::util::URL	aURL	;
		aURL.Complete = DEFINE_CONST_UNICODE(".uno:CloseFrame");

		::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > 	xMan	= ::utl::getProcessServiceFactory() ;
        ::com::sun::star::uno::Reference< ::com::sun::star::util::XURLTransformer >       xTrans  ( xMan->createInstance( DEFINE_CONST_UNICODE("com.sun.star.util.URLTransformer") ), ::com::sun::star::uno::UNO_QUERY ) ;
		::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatchProvider > 	xProv	( _xFrame, ::com::sun::star::uno::UNO_QUERY ) ;

		aGuard.clear();

		xTrans->parseStrict( aURL ) ;
		if ( xProv.is() )
		{
			::com::sun::star::uno::Reference< ::com::sun::star::frame::XDispatch >  xDisp = xProv->queryDispatch( aURL, ::rtl::OUString(), 0 ) ;
			if ( xDisp.is() )
				xDisp->dispatch( aURL, ::com::sun::star::uno::Sequence < ::com::sun::star::beans::PropertyValue>() ) ;
		}
	}
}

IMPL_LINK( DownloadController, AskLocalFile_Impl, void*, pVoid )
{
	// use clearable guard because AskLocalFile must not lock m_aMutex!
	::vos::OClearableGuard aGuard( m_aMutex );
	nAskEventID = 0;
	aGuard.clear();

	AskLocalFile();
	return 0;
}

IMPL_LINK( DownloadController, Finished_Impl, void*, pVoid )
{
	::vos::OGuard aGuard( m_aMutex );
	nUserEventID = 0;
	aDownloadTimer.Stop();

	Download_Impl( NULL );
	bDownloading = sal_False;

	if ( !bInAsk )
		WriteToFile();
	return 0;
}

sal_Bool DownloadController::AskLocalFile()
{
	::vos::OClearableGuard aGuard( m_aMutex );
	if ( bInClose )
		return sal_False ;

	bInAsk = sal_True;
	String aExtension( _aSourceURL.GetFileExtension() );
	String aFilename( _aSourceURL.GetLastName() );
	aGuard.clear();

	::vos::OClearableGuard aSolarGuard( Application::GetSolarMutex() );
	VclFileDialog* pFileDialog = GetpApp()->CreateFileDialog( NULL, WB_SAVEAS );
	pFileDialog->SetDefaultExt (aExtension) ;
	SfxIniManager* pIniManager = SFX_INIMANAGER () ;
	String aDefaulDownloadPath = pIniManager->Get( SFX_KEY_DOWNLOAD_DIR );
	INetURLObject aDefDownloadFileObj( aDefaulDownloadPath, INET_PROT_FILE );

	// validate download path
	if ( !aDefDownloadFileObj.HasError() &&
		 SfxContentHelper::MakeFolder( aDefDownloadFileObj.GetMainURL() ) )
	{
		aDefDownloadFileObj.insertName( aFilename );
	}

	pFileDialog->SetPath( aDefDownloadFileObj.GetMainURL() );
	INetURLObject aDefaultDownloadPathObj = aDefDownloadFileObj;
	aDefaultDownloadPathObj.removeSegment();
	aDefaultDownloadPathObj.removeFinalSlash();

	sal_uInt16 nRet = pFileDialog->Execute();
	INetURLObject aSaveFileObj( pFileDialog->GetPath() );
	delete pFileDialog;
	aSolarGuard.clear();

	::vos::OGuard aNewGuard( m_aMutex );
	if( !nRet )
	{
		aLocalFile.Invalidate();
	}
	else
	{
		aLocalFile.SetURL( aSaveFileObj.GetMainURL() );
		aLocalFile.SetValid();
		INetURLObject aSavePathObj = aSaveFileObj;
		aSavePathObj.removeSegment();
		aSavePathObj.removeFinalSlash();
		if ( aDefaultDownloadPathObj != aSavePathObj )
			pIniManager->Set ( aSavePathObj.GetMainURL(), SFX_KEY_DOWNLOAD_DIR );
	}

	bInAsk = sal_False;

	if ( !bDownloading && aLocalFile.IsValid() )
	{
		WriteToFile();
	}
	else if ( !nRet )
	{
		CancelDownload_Impl();
		SendClose();
	}

	return sal_True;
}

void DownloadController::WriteToFile()
{
	::vos::OGuard aGuard( m_aMutex );
	DBG_ASSERT( !bInAsk && !bDownloading, "wrong call to WriteToFile!" );
	if ( bInAsk || bDownloading )
		return;

	if ( !aLocalFile.IsValid() )
	{
		// writing will be done in AskLocalFile
		AskLocalFile();
	}
	else
	{
		while( nTotalSize && aLocalFile.IsValid() && !IsSufficientDiskSpace(nTotalSize, aLocalFile.PathToFileName()) )
		{
			// ask user to continue with another file name
			::vos::OGuard aGuard( Application::GetSolarMutex() );
			String aQueryText( SfxResId( ERR_INSUFFICIENTDISKSPACE ) );
			aQueryText.SearchAndReplaceAscii( "%1",aLocalFile.PathToFileName() );
			QueryBox aBox( NULL, WB_YES_NO | WB_DEF_NO, aQueryText );
			aBox.SetText( String() );
			aLocalFile.Invalidate();
			if ( aBox.Execute() == RET_YES )
				AskLocalFile();
		}

		if ( aLocalFile.IsValid() )
			pDownloader->WriteToFile( aLocalFile.GetMainURL() );
		SendClose();
	}
}

sal_Bool DownloadController::IsSufficientDiskSpace ( sal_uInt32 nFileSize, const String& rDestinationPath )
{
	sal_Int64 nFreeBytes = 0;
	ErrCode nErr = SfxContentHelper::QueryDiskSpace( rDestinationPath, nFreeBytes );
	// Wenn der Aufruf ohne Fehler gelang und der Platz zu klein
	// ist -> False zurueck, ansonsten True
	if ( ( nErr == ERRCODE_NONE ) && ( (sal_Int64)nFileSize > nFreeBytes )  )
		return sal_False;
	else
		return sal_True;
}

void DownloadController::SFX_NOTIFY ( SfxBroadcaster& rBroadCaster, const TypeId& rType, const SfxHint& rHint, const TypeId& rHintType )
{
	if ( rHint.ISA(SfxSimpleHint) )
	{
		switch ( ((SfxSimpleHint&)rHint).GetId() )
		{
/*
			case SFX_HINT_DATAAVAILABLE :
			{
				if ( Application::GetSolarMutex().tryToAcquire() )
				{
					Download_Impl( NULL );
					Application::GetSolarMutex().release();
				}
				break;
			}
 */
			case SFX_HINT_SAVECOMPLETED :
			{
				::com::sun::star::uno::Reference< ::com::sun::star::frame::XController >  xRef( this );
				sal_uInt32 nID = Application::PostUserEvent( LINK(this, DownloadController, Finished_Impl) );

				::vos::OGuard aGuard( m_aMutex );
				if ( bDownloading )
					nUserEventID = nID;
				break;
			}
		}
	}
}

// This method shows some information about current download.
// (::com::sun::star::util::URL of source and destination, bit/s, progress ...)
// assumes that solar mutex has been locked by caller
IMPL_LINK( DownloadController, Download_Impl, void*, pVoid )
{
	::vos::OGuard aGuard( m_aMutex );
	if ( pDownloader && bDownloading && xDownloadMonitor.is () )
	{
		::com::sun::star::uno::Reference< ::com::sun::star::awt::XProgressMonitor > 		xProgressMonitor( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;
		::com::sun::star::uno::Reference< ::com::sun::star::awt::XProgressBar > 			xProgressBar( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;
		::com::sun::star::uno::Reference< ::com::sun::star::awt::XButton > 				xButton( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;
		::com::sun::star::uno::Reference< ::com::sun::star::awt::XLayoutConstrains > 	xLayout			 ;

		DBG_ASSERT ( (
						xProgressMonitor.is	()	&&
						xProgressBar.is		()	&&
						xButton.is			()
					) , "DownloadController::Download()\nSome interfaces of DownloadMonitor are unknown!\n" 	) ;

		// Calculate and set values in window of downloadmonitor

		// Load ressources for ::com::sun::star::util::URL-topics
		::rtl::OUString	sTopicSource		( SfxResId ( STR_DOWNLOAD_SOURCE			) );
		::rtl::OUString	sTopicDestination	( SfxResId ( STR_DOWNLOAD_DESTINATION		) );

		// secure impossible cases !!!
		//.is ressource changed or not valid?
		DBG_ASSERT ( (
						( sTopicSource.len		() != 0	) &&
						( sTopicDestination.len	() != 0	)
					) , "DownloadController::Download()\nLoad of ressources failed." 	) ;

		::rtl::OUString	sTextSource			( DEFINE_CONST_UNICODE(": ")) ;
		::rtl::OUString	sTextDestination	( DEFINE_CONST_UNICODE(": ")) ;
		sTextSource			+= _aSourceURL.GetURLNoPass	();
		sTextDestination	+= aLocalFile.GetMainURL	();

		// update topics for URLs to window
		xProgressMonitor->updateText ( sTopicSource			, sTextSource	  , sal_True	) ;
		xProgressMonitor->updateText ( sTopicDestination	, sTextDestination, sal_True	) ;

		aStat.nBytesRec = pDownloader->getActLen();
		if ( !nTotalSize )
			nTotalSize = pDownloader->getMaxLen();

		// save impossible cases
		// You can't more read, as is in file!
//		DBG_ASSERT ( (nTotalSize>=aStat.nBytesRec) , "DownloadController::Download()\nYou will read much more, as is in source file!\n" ) ;

		::rtl::OUString	sTopicState			( SfxResId ( STR_DOWNLOAD_STATE			) ) ;
		::rtl::OUString	sTopicRemainingTime	( SfxResId ( STR_DOWNLOAD_REMAININGTIME	) ) ;
		String	sTempState					( SfxResId ( STR_DOWNLOAD_STATEFORMAT 	) ) ;

		// save impossible cases
		//.is ressource changed or not valid?
		DBG_ASSERT ( (
						( sTopicState.len			() != 0	) &&
						( sTempState.Len			() != 0	) &&
						( sTopicRemainingTime.len	() != 0	)
					) , "DownloadController::Download()\nLoad of ressources failed." 	) ;

		// Calculate some values to show
		sal_uInt32	nDownloadTime		;	// time for actual download [ms]
		sal_uInt32	nPerSecond			;	// bit per second
		sal_uInt32	nRemainSecond		;	// time to left for download
		String	sReadBytes			;
		String	sTotalBytes			;
		String	sPerSecond			;
		String	sRemainSecond		;
		::rtl::OUString	sTextState			;
		::rtl::OUString	sTextRemainingTime	;

		// Take defaults. The follow instruction do something only, if changes necessary.
		sReadBytes			= String()										;
		sTotalBytes			= String() 										;
		sPerSecond			= String() 										;
		sRemainSecond		= String()										;
		nDownloadTime		= Time::GetSystemTicks () - aStat.nStartTime	;
		sTextState			= DEFINE_CONST_UNICODE(": ")					;
		sTextRemainingTime	= DEFINE_CONST_UNICODE(": ")					;

		// Format output for readed bytes
		if ( aStat.nBytesRec >= 1048576 )
		{
			sReadBytes	 = String::CreateFromInt32( aStat.nBytesRec/1048576 );
			sReadBytes	+= DEFINE_CONST_UNICODE(" MB")	;
		}
		else
		if ( aStat.nBytesRec >= 1024	)
		{
			sReadBytes	 = String::CreateFromInt32( aStat.nBytesRec/1024 );
			sReadBytes	+= DEFINE_CONST_UNICODE(" KB")	;
		}
		else
		{
			sReadBytes	 = String::CreateFromInt32( aStat.nBytesRec );
			sReadBytes	+= DEFINE_CONST_UNICODE(" Byte");
		}

		// Do it only, if time for download well known.
		if ( nDownloadTime > 0 )
		{
			// Calculate speed of download ...
			nPerSecond		= ( ( aStat.nBytesRec * 8 ) / nDownloadTime ) * 1000	;
		}

		// Sometimes nTotalSize is'nt set, but data of file are readed!?
		// Do only the follow, if size of file is known.
		if ( nTotalSize > 0 )
		{
			// Format output for size of file
			if ( nTotalSize >= 1048576 )
			{
				sTotalBytes	 = String::CreateFromInt32( nTotalSize/1048576 );
				sTotalBytes	+= DEFINE_CONST_UNICODE(" MB")					;
			}
			else
			if ( nTotalSize >= 1024	)
			{
				sTotalBytes	 = String::CreateFromInt32( nTotalSize/1024 )	;
				sTotalBytes	+= DEFINE_CONST_UNICODE(" KB")					;
			}
			else
			{
				sTotalBytes	 = String::CreateFromInt32( nTotalSize );
				sTotalBytes	+= DEFINE_CONST_UNICODE(" Byte")		;
			}

			// ... and is it known ...
			if ( nPerSecond != 0 )
			{
				// ..calculate time to left for download
				nRemainSecond	= ( ( nTotalSize - aStat.nBytesRec ) * 8 ) / nPerSecond ;

				Time	aTime ( 0, 0, nRemainSecond ) ;

				// Format ouput like "01 : 24 : 04"
				if ( aTime.GetHour () < 10 )
				{
					sRemainSecond	 =	'0' ;
				}
				sRemainSecond	 +=	String::CreateFromInt32( aTime.GetHour() )	;
				sRemainSecond	 +=	DEFINE_CONST_UNICODE(" : ")					;

				if ( aTime.GetMin () < 10 )
				{
					sRemainSecond	+=	'0'	;
				}
				sRemainSecond	 +=	String::CreateFromInt32( aTime.GetMin() )	;
				sRemainSecond	 +=	DEFINE_CONST_UNICODE(" : ")					;

				if ( aTime.GetSec () < 10 )
				{
					sRemainSecond	+=	'0'	;
				}
				sRemainSecond	 +=	String::CreateFromInt32( aTime.GetSec() )	;
			}

			// Set new value at progressbar
			xProgressBar->setRange ( 0,nTotalSize	 ) ;
			xProgressBar->setValue ( aStat.nBytesRec ) ;
		}
		else
		{
			// nTotalSize is unknown ...
			// Show loades bytes and used time only!
			sRemainSecond	= DEFINE_CONST_UNICODE("00 : 00 : 00")	;
			sTotalBytes		= DEFINE_CONST_UNICODE("???"		 )	;
		}

		// Find out unit for download speed
		if ( nPerSecond >= 1048576 )
		{
			nPerSecond	/= 1048576		;
			sPerSecond	 = String::CreateFromInt32(nPerSecond)	;
			sPerSecond	+= DEFINE_CONST_UNICODE(" Mb/s");
		}
		else
		if ( nPerSecond >= 1024 )
		{
			nPerSecond	/= 1024			;
			sPerSecond	 = String::CreateFromInt32(nPerSecond)	;
			sPerSecond	+= DEFINE_CONST_UNICODE(" Kb/s")	;
		}
		else
		{
			sPerSecond	 = String::CreateFromInt32(nPerSecond)	;
			sPerSecond	+= DEFINE_CONST_UNICODE(" bit/s")		;
		}

		// Replace values at ressource
		sTempState.SearchAndReplaceAscii ( "$(READ)" , sReadBytes	) ;
		sTempState.SearchAndReplaceAscii ( "$(TOTAL)", sTotalBytes	) ;
		sTempState.SearchAndReplaceAscii ( "$(TIME)" , sPerSecond	) ;

		sTextState			+= sTempState		;
		sTextRemainingTime	+= sRemainSecond	;

		// Show collected informations
		xProgressMonitor->updateText ( sTopicState			, sTextState		, sal_False ) ;
		xProgressMonitor->updateText ( sTopicRemainingTime	, sTextRemainingTime, sal_False ) ;
		}

	// Restart timer
	aDownloadTimer.Start () ;
	return 1;
}

//------------------------------------------------------------------------------------------------------------
// end interface methods
//------------------------------------------------------------------------------------------------------------

void SAL_CALL DownloadController::dispose() throw( ::com::sun::star::uno::RuntimeException )
{
	::vos::OGuard aGuard( m_aMutex );
	aDownloadTimer.Stop();
	if ( bDownloading )
		CancelDownload_Impl();

	if ( nUserEventID )
		Application::RemoveUserEvent ( nUserEventID ) ;

	if ( nAskEventID )
		Application::RemoveUserEvent ( nAskEventID ) ;
}

void SAL_CALL  DownloadController::disposing ( const ::com::sun::star::lang::EventObject& Source ) throw( ::com::sun::star::uno::RuntimeException )
{
}

void SAL_CALL  DownloadController::attachFrame ( const ::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame > & xFrame ) throw( ::com::sun::star::uno::RuntimeException )
{
	::vos::OClearableGuard aGuard( m_aMutex );
	_xFrame = xFrame ;

	// secure impossible cases !!!
	DBG_ASSERT ( _xFrame.is(), "DownloadController::attachFrame()\nNo valid frame detected.\n" ) ;
	if ( _xFrame.is() )
	{
		::com::sun::star::uno::Reference< ::com::sun::star::awt::XWindow > 	xWindow( _xFrame->getComponentWindow () );
		xDownloadMonitor = ::com::sun::star::uno::Reference< ::com::sun::star::awt::XControl >  ( xWindow, ::com::sun::star::uno::UNO_QUERY ) ;

		DBG_ASSERT ( (  xWindow.is			() &&
						xDownloadMonitor.is	()
					 ) , "DownloadController::attachFrame()\nOne of searched interface for DownloadMonitor is no valid.\n" 	) ;

		if( xDownloadMonitor.is() )
		{
			::com::sun::star::uno::Reference< ::com::sun::star::awt::XProgressMonitor > 		xProgressMonitor( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;
			::com::sun::star::uno::Reference< ::com::sun::star::awt::XProgressBar > 			xProgressBar( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;
			::com::sun::star::uno::Reference< ::com::sun::star::awt::XButton > 				xButton( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;
			::com::sun::star::uno::Reference< ::com::sun::star::awt::XLayoutConstrains > 	xLayout( xDownloadMonitor, ::com::sun::star::uno::UNO_QUERY ) ;

			aGuard.clear();

			DBG_ASSERT ( (	xProgressMonitor.is	() &&
							xProgressBar.is		() &&
							xButton.is			() &&
							xLayout.is			()
						 ) , "DownloadController::attachFrame()\nOne of searched interface for DownloadMonitor is no valid.\n" 	) ;

			// set some labels and initialize progressbar and layout of window
			::vos::OClearableGuard aSolarGuard( Application::GetSolarMutex() );
			::rtl::OUString	sLabel				( Button::GetStandardText ( BUTTON_CANCEL )) ;
			::rtl::OUString	sTopicSource		( SfxResId ( STR_DOWNLOAD_SOURCE		) ) ;
			::rtl::OUString	sTopicDestination	( SfxResId ( STR_DOWNLOAD_DESTINATION	) ) ;
			::rtl::OUString	sTopicState			( SfxResId ( STR_DOWNLOAD_STATE			) ) ;
			::rtl::OUString	sTopicRemainingTime	( SfxResId ( STR_DOWNLOAD_REMAININGTIME	) ) ;

			DBG_ASSERT ( sLabel.len()!=0, "DownloadController::attachFrame()\nLabel forbutton not valid. Test ressource and converting STRING -> USTRING.") ;
			DBG_ASSERT ( (
							( sTopicSource.len			() != 0 ) &&
							( sTopicDestination.len		() != 0 ) &&
							( sTopicState.len			() != 0 ) &&
							( sTopicRemainingTime.len	() != 0 )
						 ) , "DownloadController::attachFrame()\nLoad \"STR_DOWNLOAD_...\" from ressource failed." 	) ;

			aSolarGuard.clear();

			xButton->setLabel ( sLabel ) ;
			// ':' = 3Ah
			xProgressMonitor->addText ( sTopicSource		, ::rtl::OUString(0x003A), sal_True  ) ;
			xProgressMonitor->addText ( sTopicDestination	, ::rtl::OUString(0x003A), sal_True  ) ;
			xProgressMonitor->addText ( sTopicState			, ::rtl::OUString(0x003A), sal_False ) ;
			xProgressMonitor->addText ( sTopicRemainingTime	, ::rtl::OUString(0x003A), sal_False ) ;

			// init POGRESSBAR
			xProgressBar->setRange	( 0, 10000  ) ;	// take default values - The real filesize ist not known at this point
			xProgressBar->setValue	( 0	  		) ;	// reset progressbar

/* ??
	Noch existiert keine vernnftige Lsung fr das anpassen der Gre des Dialoges!
	Der Hintergrund wird zur Zeit NICHT neu gezeichnet. Daher an dieser Stelle erstmal auf
	dynamisches layouten verzeichten. Die Gre des Dialoges wird von auen (am Frame) gesetzt!
			::com::sun::star::uno::Reference< ::com::sun::star::awt::XWindow > 	xFrameWindow	;
			::com::sun::star::awt::Size		aFrameSize		;
			::com::sun::star::awt::Size		aDownloaderSize	;
			sal_Int32		nX				;
			sal_Int32		nY				;
			sal_Int32		nWidth			;
			sal_Int32		nHeight			;

			xFrameWindow	=	_xFrame->getContainerWindow () ;
			aFrameSize		=	xFrameWindow->getPosSize().GetSize () ;
			aDownloaderSize	=	xLayout->getPreferredSize () ;
			nWidth			=	aDownloaderSize.getWidth  () ;
			nHeight			=	aDownloaderSize.getHeight () ;
			nX				=	( aFrameSize.getWidth  () / 2 ) - ( nWidth  / 2 ) ;
			nY				=	( aFrameSize.getHeight () / 2 ) - ( nHeight / 2 ) ;

			xWindow->setPosSize ( nX, nY, nWidth, nHeight, WINDOW_POSSIZE_ALL ) ;
*/
		}
	}
}

sal_Bool SAL_CALL  DownloadController::attachModel ( const ::com::sun::star::uno::Reference< ::com::sun::star::frame::XModel > & xModel ) throw( ::com::sun::star::uno::RuntimeException )
{
	return sal_False ;
}

//------------------------------------------------------------------------------------
// suspend knows two cases
//    1) bSuspend==sal_True			object must to be suspend
//    2) bSuspend==sal_False		in the most time, the object is suspended, but
//								an other object is not valid => then this must be reactivate!
//------------------------------------------------------------------------------------
sal_Bool SAL_CALL  DownloadController::suspend ( sal_Bool bSuspend ) throw( ::com::sun::star::uno::RuntimeException )
{
	::vos::OGuard aGuard( m_aMutex );
	// suspend downloader
	if ( bSuspend )
	{
		// are downloads already active ?
		if ( bDownloading )
		{
			::vos::OGuard aSolarGuard( Application::GetSolarMutex() );
			String aMessage ( SfxResId(ERR_ACTIVEDOWNLOADS) ) ;
			if ( QueryBox(NULL, WB_YES_NO, aMessage).Execute() == RET_YES )
				CancelDownload_Impl();
			else if ( !aLocalFile.IsValid() )
				nAskEventID = Application::PostUserEvent( LINK(this, DownloadController, AskLocalFile_Impl) );
		}

		bInClose = !bDownloading;
	}
	// reactivate downloader
	else
	{
		StartDownload ( _aSourceURL.GetMainURL () ) ;
	}

	// result to caller
	return !bDownloading ;
}

void SAL_CALL  DownloadController::restoreViewData ( const ::com::sun::star::uno::Any& Data ) throw( ::com::sun::star::uno::RuntimeException )
{
	/*
	erstmal leer implementieren
	*/
}

::com::sun::star::uno::Reference< ::com::sun::star::frame::XFrame >  SAL_CALL  DownloadController::getFrame () throw( ::com::sun::star::uno::RuntimeException )
{
	return _xFrame ;
}

::com::sun::star::uno::Reference< ::com::sun::star::frame::XModel >  SAL_CALL  DownloadController::getModel () throw( ::com::sun::star::uno::RuntimeException )
{
	return ::com::sun::star::uno::Reference< ::com::sun::star::frame::XModel >  () ;
}

::com::sun::star::uno::Any SAL_CALL  DownloadController::getViewData () throw( ::com::sun::star::uno::RuntimeException )
{
	return ::com::sun::star::uno::Any () ;
}

void  SAL_CALL DownloadController::actionPerformed ( const ::com::sun::star::awt::ActionEvent& rEvent ) throw( ::com::sun::star::uno::RuntimeException )
{
	if ( rEvent.ActionCommand.compareToAscii("BreakDownload")==0 )
		SendClose();
}

//------------------------------------------------------------------------------------------------------------
// start listener methods
//------------------------------------------------------------------------------------------------------------
void  SAL_CALL DownloadController::addEventListener ( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener > & aListener ) throw( ::com::sun::star::uno::RuntimeException )
{
}

void  SAL_CALL DownloadController::removeEventListener ( const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener > & aListener ) throw( ::com::sun::star::uno::RuntimeException )
{
}
//------------------------------------------------------------------------------------------------------------
// end listener methods
//------------------------------------------------------------------------------------------------------------

SFX_IMPL_XINTERFACE_4( DownloadController, OWeakObject, ::com::sun::star::lang::XServiceInfo,::com::sun::star::frame::XController,::com::sun::star::lang::XComponent,::com::sun::star::awt::XActionListener )
SFX_IMPL_XTYPEPROVIDER_4( DownloadController, ::com::sun::star::lang::XServiceInfo,::com::sun::star::frame::XController,::com::sun::star::lang::XComponent,::com::sun::star::awt::XActionListener )
SFX_IMPL_XSERVICEINFO( DownloadController, "com.sun.star.frame.Controller", "com.sun.star.comp.sfx2.DownloadController" )
SFX_IMPL_SINGLEFACTORY( DownloadController );


