/*************************************************************************
 *
 *  $RCSfile: streams.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:29 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_IO_XOUTPUTSTREAM_HPP_
#include <com/sun/star/io/XOutputStream.hpp>
#endif
#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HPP_
#include <com/sun/star/io/XInputStream.hpp>
#endif
#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif

#ifndef _TOOLS_DEBUG_HXX //autogen wg. DBG_ASSERT
#include <tools/debug.hxx>
#endif

#include "streams.hxx"

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::io;
using namespace ::cppu;
using rtl::OUString;

// -------------------------------- XInputStream --------------------------------

// -----------------------------------------------------------------------------
sal_Int32 SAL_CALL InputStream_Impl::readBytes( Sequence< sal_Int8 >& aData, sal_Int32 nBytesToRead ) throw( NotConnectedException, BufferSizeExceededException, IOException, RuntimeException )
{
	// POST: Read nBytes
	DBG_ASSERT(nBytesToRead >= 0, "nBytesToRead must be gt -1");
	sal_Int32 nAvailable = available();
	if (nAvailable == 0) return 0;

	sal_Int32 nCount = min(nBytesToRead, nAvailable);
	if ( nCount != aData.getLength() )
		aData.realloc(nCount);

	sal_Int8 *pBuffer = (sal_Int8*) aData.getArray();
	return _aStream.Read(pBuffer, nCount);
}
// -----------------------------------------------------------------------------
sal_Int32 InputStream_Impl::readSomeBytes(Sequence< sal_Int8 >& aData, sal_Int32 nMaxBytesToRead) throw( NotConnectedException, BufferSizeExceededException, IOException, RuntimeException )
{
	// POST: read at least 1 Byte
	return readBytes(aData, nMaxBytesToRead);
}
// -----------------------------------------------------------------------------
void  InputStream_Impl::skipBytes(sal_Int32 nBytesToSkip) throw( NotConnectedException, BufferSizeExceededException, IOException, RuntimeException )
{
	// POST: skip nBytes
	Sequence<sal_Int8> aTmp(nBytesToSkip);
	_aStream.Read(&aTmp, nBytesToSkip);
}
// -----------------------------------------------------------------------------
sal_Int32 InputStream_Impl::available(void) throw( NotConnectedException, IOException, RuntimeException )
{
	// POST: how many bytes can read/skip without blocking
	sal_Int32 nPos = _aStream.Tell();
	_aStream.Seek(STREAM_SEEK_TO_END);
	sal_Int32 nBytesToRead = _aStream.Tell() - nPos;
	_aStream.Seek(nPos);
	return nBytesToRead;
}
// -----------------------------------------------------------------------------
void InputStream_Impl::closeInput(void) throw( NotConnectedException, IOException, RuntimeException )
{
	// POST: close Stream
	//_aStream.Close();
}

Any SAL_CALL InputStream_Impl::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aRet( ::cppu::queryInterface( rType, static_cast< XInputStream* >( this ) ) );
	return (aRet.hasValue() ? aRet : OWeakObject::queryInterface( rType ));
}

void SAL_CALL InputStream_Impl::acquire (void) throw(RuntimeException)
{
	OWeakObject::acquire();
}

void SAL_CALL InputStream_Impl::release (void) throw(RuntimeException)
{
	OWeakObject::release();
}

// ------------------------------- XOutputStream -------------------------------

// -----------------------------------------------------------------------------
void OutputStream_Impl::flush(void) throw( NotConnectedException, BufferSizeExceededException, IOException, RuntimeException )
{
	_pStream->Flush();
}

// -----------------------------------------------------------------------------
void OutputStream_Impl::writeBytes(const Sequence< sal_Int8 >& aData) throw( NotConnectedException, BufferSizeExceededException, IOException, RuntimeException )
{
	Sequence<sal_Int8> *pData = (Sequence<sal_Int8>*)&aData;
	sal_Int8* pBuffer = (sal_Int8 *)pData->getArray();
	sal_Int32 nLen = aData.getLength();
	_pStream->Write(pBuffer, nLen);
}

// -----------------------------------------------------------------------------
void OutputStream_Impl::closeOutput(void) throw( NotConnectedException, BufferSizeExceededException, IOException, RuntimeException )
{
	//_pStream->Close();
}

Any SAL_CALL OutputStream_Impl::queryInterface( const Type & rType ) throw (RuntimeException)
{
	Any aRet( ::cppu::queryInterface( rType, static_cast< XOutputStream* >( this ) ) );
	return (aRet.hasValue() ? aRet : OWeakObject::queryInterface( rType ));
}

void SAL_CALL OutputStream_Impl::acquire (void) throw(RuntimeException)
{
	OWeakObject::acquire();
}

void SAL_CALL OutputStream_Impl::release (void) throw(RuntimeException)
{
	OWeakObject::release();
}

OutputStream_Impl::~OutputStream_Impl()
{
	delete _pStream;
}

