/*************************************************************************
 *
 *  $RCSfile: cfgmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:29 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// include ---------------------------------------------------------------

#ifndef _SVSTOR_HXX //autogen
#include <so3/svstor.hxx>
#include <sot/storinfo.hxx>
#endif
#include <tools/tempfile.hxx>
#include <tools/urlobj.hxx>
#pragma hdrstop

#include "cfgmgr.hxx"
#include "cfgitem.hxx"
#include "inimgr.hxx"
#include "sfxtypes.hxx"
#include "bindings.hxx"
#include "app.hxx"
#include "tbxmgr.hxx"
#include "objsh.hxx"
#include "objface.hxx"
#include "appdata.hxx"
#include "viewfrm.hxx"
#include "frame.hxx"
#include "helper.hxx"

#include "config.hrc"


// STATIC DATA -----------------------------------------------------------

static const char __FAR_DATA pBackupHeader[] = "Star Framework Config Backup";
static const char __FAR_DATA pHeader[] = "Star Framework Config File";

#define CFG_STREAM_BUFFER_SIZE  5000

DBG_NAME(SfxConfigManager);

// ----------------------------------------------------------------------------

void SfxConfigManager::RemovingItem(USHORT nType )
{
	if ( SfxToolBoxManager::IsUserDefToolBox_Impl( nType ) )
	{
		// Sonderfall: benutzerdefinierte Toolboxen (geht leider nicht anders!)
		// ToolBox wird an dem Interface, wo sie registriert ist, abgemeldet.
		SfxInterface::ReleaseUserDefToolBox( nType, this );
	}
}

// ----------------------------------------------------------------------------

void SfxConfigManager::CopyingItem( USHORT& nType, const String& rName )
{
	if ( SfxToolBoxManager::IsUserDefToolBox_Impl( nType ) )
	{
		// Sonderfall: benutzerdefinierte Toolboxen (geht leider nicht anders!)
		// Die ToolBox wird an einem geeigneten Interface registriert
		nType = SfxInterface::RegisterUserDefToolBox( nType, &rName, this );
	}
}

// ----------------------------------------------------------------------------

void SfxConfigManager::StartReconfiguring()
{
    SfxViewFrame* pViewFrame = SfxViewFrame::Current();
    if ( pViewFrame )
        pViewFrame->GetBindings().ENTERREGISTRATIONS();
}

// ----------------------------------------------------------------------------

void SfxConfigManager::FinishedReconfiguring()
{
    SfxViewFrame* pViewFrame = SfxViewFrame::Current();
    if ( pViewFrame )
        pViewFrame->GetBindings().LEAVEREGISTRATIONS();
}

// ----------------------------------------------------------------------------
// "Offnet Konfigurationsdatei und liest das Directory ein.
// ACHTUNG : darf nur aus dem ctor heraus gerufen werden, da das FileItem array
// neu aufgebaut wird!
//

BOOL SfxConfigManager::ReadConfig( SvStorage *pStor )
{
	// Tempfile mit cfg erzeugen
	if ( pStor && !pStor->GetError() )
	{
		SvStorageStreamRef aStr =
			pStor->OpenStream( String::CreateFromAscii(pHeader), STREAM_STD_READ );
		if ( !aStr->GetError() )
		{
			aStr->SetVersion( SOFFICE_FILEFORMAT_40 /*pStor->GetVersion()*/ );
			aStr->SetBufferSize( CFG_STREAM_BUFFER_SIZE );
			return ReadConfiguration( aStr );
		}
	}

	return TRUE;
}

// ------------------------------------------------------------------------
// Speichern der Applikations-Config

BOOL SfxConfigManager::SaveConfig()
{
	if ( !IsModified() )
		return TRUE;

	if ( SFX_APP()->Get_Impl()->bBean )
		return TRUE;

	BOOL bRet = FALSE;
	SvStorageRef aStor = new SvStorage( aCfgFileName );

	// ReOpen mit Trunc nur, wenn der Storage nicht leer ist und nur den
	// CFG-Stream enth"alt

	SvStorageInfoList *pList = new SvStorageInfoList;
	if ( !aStor->GetError() )
		aStor->FillInfoList( pList );
	if ( pList->Count() == 1 )
	{
		if ( (*pList)[0].GetName().EqualsAscii(pHeader) )
		{
			aStor.Clear();
			aStor = new SvStorage( aCfgFileName, STREAM_STD_READWRITE | STREAM_TRUNC );
		}
	}

	DELETEZ( pList );

	if ( !aStor->GetError() )
	{
		SetModified(TRUE);
		bRet = SaveConfig( aStor );
	}
	else
	{
		SetError( ERR_OPEN );
		return FALSE;
	}

	if (bRet)
	{
		SetError( ERR_NO );
#ifdef DBG_UTIL
		SvStorageStreamRef aStr =
			aStor->OpenStream( String::CreateFromAscii(pHeader), STREAM_STD_READ );
		DBG_ASSERT ( !aStr->GetError(), "Fehler beim StreamOpen!" );
		aStr->SetVersion( SOFFICE_FILEFORMAT_40 /*aStor->GetVersion()*/ );
		CheckConfiguration( aStr );
#endif
	}
	else
	{
		DBG_ERROR( "Fehler beim Schreiben der cfg-Datei!" );
		SetError( ERR_WRITE );
	}

	return bRet;
}

// ----------------------------------------------------------------------------
// Schreibt eine kompakte Konfigurationsdatei, indem alle vorhandenen Eintraege
// im ConfigItemArray, die auf einen Eintrag in der aktuellen Konfiguration
// verweisen, enthalten sind.
// ACHTUNG : sollte fuer root configuration nur im dtor gerufen werden, fuer
// Dokumente wird es ueblicherweise beim Speichern aufgerufen.
//

BOOL SfxConfigManager::SaveConfig( SvStorage* pStor )
{
	SvStorageStreamRef aStr =
		pStor->OpenStream(String::CreateFromAscii(pHeader), STREAM_STD_READWRITE | STREAM_TRUNC);
	if ( !aStr->GetError() )
	{
		aStr->SetVersion( SOFFICE_FILEFORMAT_40 /*pStor->GetVersion()*/ );
		aStr->SetBufferSize(CFG_STREAM_BUFFER_SIZE);
		return SaveConfiguration( aStr );
	}
	else
		return FALSE;
}

// ------------------------------------------------------------------------
// Pfad zusammensetzen: Applikationsname mit der Extension .cfg
// wird im Benutzerverzeichnis abgelegt.

String SfxConfigManager::MakeAbs_Impl( const String& rName )
{
	DBG_CHKTHIS(SfxConfigManager, 0);

	String aRet;
	INetURLObject aUserPath( SFX_INIMANAGER()->Get( SFX_KEY_USERCONFIG_PATH ), INET_PROT_FILE );
	aUserPath.setFinalSlash();
	INetURLObject aFile;
	if ( aUserPath.GetNewAbsURL( rName, &aFile ) )
		aRet = aFile.PathToFileName();
	return aRet;
}

// ----------------------------------------------------------------------------

SfxConfigManager::SfxConfigManager( SvStorage* pStor, SfxConfigManager* pMgr ) :

	SvConfigManager( pMgr ),

	pObjShell	( NULL ),
	pBackupStr	( NULL )

{
	DBG_CTOR(SfxConfigManager, 0);
	if ( !pMgr && !pStor )
	{	// default cfg
		aCfgFileName = GetDefaultName();
		SvStorageRef aStor = new SvStorage( aCfgFileName, STREAM_STD_READ );
		ReadConfig( aStor );
	}
	else
		ReadConfig( pStor );	 		// cfg in Dokument/Vorlage
}

// ------------------------------------------------------------------------

SfxConfigManager::SfxConfigManager( const String& rName ) :

	SvConfigManager( NULL ),

	pObjShell	( NULL ),
	pBackupStr	( NULL )

{
	DBG_CTOR(SfxConfigManager, 0);

	aCfgFileName = MakeAbs_Impl( rName );
	SvStorageRef aStor = new SvStorage( aCfgFileName, STREAM_STD_READ );
	ReadConfig( aStor );
}

// ------------------------------------------------------------------------

SfxConfigManager::~SfxConfigManager()
{
	DBG_DTOR(SfxConfigManager, 0);

	if ( SFX_CFGMANAGER() == this )
	{
		if ( GetParentMgr() )
			GetParentMgr()->Activate(this);
	}

	if ( pBackupStr )
	{
		SfxContentHelper::Kill( pBackupStr->GetFileName() );
		delete pBackupStr;
	}
}

// ------------------------------------------------------------------------

void SfxConfigManager::Activate( SvConfigManager *pMgr )
{
	if ( this == pMgr )
		return;
/*
	if (pMgr && pMgr->nActive & CFG_INPLACE)
	{									// wenn altes Dokument InPlace

		pMgr->InPlaceActivate(FALSE);
		pMgr->nActive = CFG_NOT_INPLACE_ACTIVE;

		SfxConfigManager *pNewRootMgr = pParentMgr ? pParentMgr : this;
		SfxConfigManager *pOldRootMgr = pMgr->pParentMgr ? pMgr->pParentMgr : pMgr;

		if ( pOldRootMgr->aCfgFile.GetName() == pNewRootMgr->aCfgFile.GetName() )
		{
			USHORT nItems = pOldRootMgr->pFileItemArr->Count();
			for (USHORT n=0; n<nItems; n++)
			{
				SfxConfigFileItem_Impl *pIt = (*pOldRootMgr->pFileItemArr)[n];
				SfxConfigFileItem_Impl *pItem =
					FindItem_Impl(pNewRootMgr->pFileItemArr, pIt->nType);
				if (!pItem)
				{
					pItem =
						new SfxConfigFileItem_Impl(pIt->nType, pIt->aName);
					pNewRootMgr->pFileItemArr->Insert(pItem, pNewRootMgr->pFileItemArr->Count());
					pNewRootMgr->SetModified(TRUE);
				}

				pItem->lLength = pIt->lLength;
				pItem->lFilePos = pIt->lFilePos;

				if (pIt->pCItem && pIt->pCItem->IsModified())
				{
					pOldRootMgr->StoreConfig(pIt->pCItem);
					pItem->lFilePos = pIt->lFilePos;
					pItem->lLength = pIt->lLength;
					pNewRootMgr->lStart = pOldRootMgr->lStart;
					pNewRootMgr->lDirPos = pOldRootMgr->lDirPos;
					pNewRootMgr->SetModified(TRUE);
					if (pItem && pItem->pCItem)
						pNewRootMgr->ReconfigureItem(pItem->pCItem);
				}
			}
		}

	}
	else if (pMgr)
 */

	SfxViewFrame *pFrame = SfxViewFrame::Current();
	if ( pFrame && GetObjectShell() == pFrame->GetObjectShell() )
        ActivateTask( pFrame );

	SvConfigManager::Activate( pMgr );
	SFX_APP()->SetConfigManager_Impl( this );
}

// ------------------------------------------------------------------------
/*
void SfxConfigManager::InPlaceActivate( BOOL bAct )
{
	if ( !(!bAct || SFX_CFGMANAGER() != this) )
		return;

	if (bAct)							// InPlace-Aktivierung
	{
		SfxConfigManager* pMgr = SFX_CFGMANAGER();
		if (pMgr->nActive == CFG_INPLACE_ACTIVE)
			pMgr->InPlaceActivate(FALSE);

		SfxConfigManager *pOldRootMgr = pMgr->pParentMgr ? pMgr->pParentMgr : pMgr;
		SfxConfigManager *pNewRootMgr = pParentMgr ? pParentMgr : this;

		if ( pOldRootMgr->aCfgFile.GetName() == pNewRootMgr->aCfgFile.GetName() )
		{
			if (pNewRootMgr->pStream != pOldRootMgr->pStream)
			{
				delete pNewRootMgr->pStream;
				pNewRootMgr->aTmp.Kill();
				pNewRootMgr->bOwnStream = FALSE;
				pNewRootMgr->pStream = pOldRootMgr->pStream;
				pNewRootMgr->lDirPos = pOldRootMgr->lDirPos;
			}

			USHORT nItems = pOldRootMgr->pFileItemArr->Count();
			for (USHORT n=0; n<nItems; n++)
			{
				SfxConfigFileItem_Impl *pIt = (*pOldRootMgr->pFileItemArr)[n];
				SfxConfigFileItem_Impl *pItem =
					FindItem_Impl(pNewRootMgr->pFileItemArr, pIt->nType);
				if (!pItem)
				{
					pItem =
						new SfxConfigFileItem_Impl(pIt->nType, pIt->aName);
					pNewRootMgr->pFileItemArr->Insert(pItem, pNewRootMgr->pFileItemArr->Count());
					pNewRootMgr->SetModified(TRUE);
				}

				pItem->lLength = pIt->lLength;
				pItem->lFilePos = pIt->lFilePos;

				if (pIt->pCItem && pIt->pCItem->IsModified())
				{
					pOldRootMgr->StoreConfig(pIt->pCItem);
					pItem->lFilePos = pIt->lFilePos;
					pItem->lLength = pIt->lLength;
					pNewRootMgr->lStart = pOldRootMgr->lStart;
					pNewRootMgr->lDirPos = pOldRootMgr->lDirPos;
					pNewRootMgr->SetModified(TRUE);
					if (pItem && pItem->pCItem)
						pNewRootMgr->ReconfigureItem(pItem->pCItem);
				}
			}

			if (pParentMgr)
			{
				ReconfigureItems(pParentMgr);
			}
		}
		SFX_CFGMANAGER()->nActive = SFX_CFGMANAGER()->nActive == CFG_ACTIVE ?
					CFG_NOT_ACTIVE : CFG_NOT_INPLACE_ACTIVE;

		SFX_APP()->SetConfigManager_Impl(this);
		nActive = CFG_INPLACE_ACTIVE;
	}
}
*/

// ------------------------------------------------------------------------
// Name des Config-Streams im Cfg-Storage.

String SfxConfigManager::GetStreamName()
{
	return String::CreateFromAscii(pHeader);
}

// ----------------------------------------------------------------------------

String SfxConfigManager::GetDefaultName()
{
	String aRet;
	INetURLObject aAppObj( Application::GetAppFileName(), INET_PROT_FILE );
	if ( SFX_APP()->Get_Impl()->bBean )
	{
		String sName( aAppObj.getBase() );
		sName += DEFINE_CONST_UNICODE( "Bean" );
		aAppObj.setName( sName );
	}
	aAppObj.setExtension( DEFINE_CONST_UNICODE( "cfg" ) );
	INetURLObject aUserPath( SFX_INIMANAGER()->Get( SFX_KEY_USERCONFIG_PATH ), INET_PROT_FILE );
	aUserPath.setFinalSlash();
	INetURLObject aFile;
	if ( aUserPath.GetNewAbsURL( aAppObj.getName(), &aFile ) )
		aRet = aFile.PathToFileName();
	return aRet;
}

// ----------------------------------------------------------------------------

void SfxConfigManager::SetModified(BOOL bMod)
{
	SvConfigManager::SetModified( bMod );
	if ( bMod && pObjShell)
		pObjShell->SetModified( TRUE );
}

// ----------------------------------------------------------------------------

void SfxConfigManager::Backup()
{
	if ( pObjShell )
	{
		DBG_ERRORFILE( "Backup nicht fuer DocumentConfig!" );
		return;
	}

	// Zun"achst den tempor"aren Stream komprimieren
	SaveConfig();

	// Einen (tempor"aren) Backupstream anlegen
	if ( pBackupStr )
	{
		// Einen etwa vorhandenen alten l"oschen
		SfxContentHelper::Kill( pBackupStr->GetFileName() );
		DELETEZ( pBackupStr );
	}

	TempFile aBackupFile;
	pBackupStr = new SvFileStream( aBackupFile.GetName(), STREAM_STD_READWRITE );
	if ( pBackupStr->GetError() )
		return;
	pBackupStr->SetVersion( SOFFICE_FILEFORMAT_40 );
	pBackupStr->SetBufferSize( CFG_STREAM_BUFFER_SIZE );

	DoBackup( pBackupStr );
	pBackupStr->Close();
}

// ----------------------------------------------------------------------------

void SfxConfigManager::Restore()
{
	if ( pObjShell )
	{
		DBG_ERROR("Backup nicht fuer DocumentConfig!");
		return;
	}

	if ( !pBackupStr )
		return;

	// Den BackupStream im "offnen
	pBackupStr->ReOpen();
	if ( pBackupStr->GetError() )
		return;

	pBackupStr->SetVersion( SOFFICE_FILEFORMAT_40 );
	DoRestore( pBackupStr );
	SfxContentHelper::Kill( pBackupStr->GetFileName() );
	DELETEZ( pBackupStr );
}

// ----------------------------------------------------------------------------

BOOL SfxConfigManager::CopyConfigItem( USHORT nType )
{
	BOOL bSearchInParent = FALSE;
	return SvConfigManager::CopyItem( nType, SFX_CFGMANAGER(), TRUE );
}


