/*************************************************************************
 *
 *  $RCSfile: resmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:29 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
// INCLUDE ---------------------------------------------------------------

#pragma hdrstop

#include "resmgr.hxx"
#include "msgdescr.hxx"
#include "app.hxx"
#include "sfxtypes.hxx"
#include "segmentc.hxx"
#include "sfx.hrc"

// STATIC DATA -----------------------------------------------------------

SEG_EOFGLOBALS()

DECLARE_TABLE(SfxMessageTable, SfxMessageDescription*)


// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_01)

static BOOL HasColorMon_Impl()
{
    return Application::GetDefaultDevice()->GetColorCount() == 2 || !SFX_APP()->IsSVLookEnabled()?
			FALSE: TRUE;
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_02)

static USHORT FindFirstFree_Impl(SfxResMgrArr &rArr)
{
	const USHORT nCount = rArr.Count();
	for(USHORT i = 0; i < nCount; ++i) {
		if(!rArr[i])
			return i;
	}
	return nCount;
}
// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_03)

static USHORT Register_Impl(SfxResMgrArr &rArr, ResMgr *pRes)
{
	const USHORT nPos = FindFirstFree_Impl(rArr);
	rArr.Insert(nPos, pRes);
	return nPos;
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_04)

void SfxResourceManager::ClearMsgTable()
{
	DBG_MEMTEST();
	DBG_ASSERT( nEnterCount == 0, "ClearMsgTable innerhalb von Enter");

	if(pMessageTable) {
		SfxMessageDescription *pDesc = pMessageTable->First();
		while(pDesc) {
			delete pDesc;
			pDesc = pMessageTable->Next();
		}
		pMessageTable->Clear();
		DELETEZ(pMessageTable);
	}
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_05)

SfxMessageDescription *SfxResourceManager::MakeDesc(USHORT nId)
{
	SfxMessageDescription *pDesc = 0;
	const USHORT nCount = aResMgrArr.Count();
	const ResId aResId(nId);
	aResId.SetRT(RSC_RESOURCE);

	ResMgr *pOld = Resource::GetResManager();
	for(USHORT i = 0; i < nCount; ++i) {
		if(aResMgrArr[i]->IsAvailable(aResId.GetRT(), aResId.GetId(),
											  aResId.GetpResource(), 0)) {
			Resource::SetResManager(aResMgrArr[i]);
			pDesc = new SfxMessageDescription(nId);
			Resource::SetResManager(pOld);
			return pDesc;
		}
	}

	// in der App-Resource?
	Resource::SetResManager(pOld);
	if(pOld->IsAvailable(aResId.GetRT(), aResId.GetId(), aResId.GetpResource(), 0)) {
		pDesc = new SfxMessageDescription(nId);
		return pDesc;
	}

	// nicht gefunden
	return new SfxMessageDescription(SID_UNKNOWN);
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_06)

USHORT	SfxResourceManager::RegisterResource(const	char *pFileName)
{
	DBG_MEMTEST();
	return Register_Impl(aResMgrArr, new ResMgr((char *)pFileName));
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_07)

void	SfxResourceManager::ReleaseResource( USHORT	nRegisterId	)
{
	DBG_MEMTEST();
	DBG_ASSERT( nRegisterId < aResMgrArr.Count(), "Array ueberindiziert");
	if(nRegisterId < aResMgrArr.Count()) {
		delete aResMgrArr[nRegisterId];
		aResMgrArr[nRegisterId] = 0;
	}
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_08)

USHORT	SfxResourceManager::RegisterBitmap(
						const char *pMono, const char *pColor)
{
	DBG_MEMTEST();
	const BOOL bColorMon = HasColorMon_Impl();
	const char *pFileName = bColorMon? pColor: pMono;
	return Register_Impl(aResMgrBmpArr, new ResMgr((char *)pFileName));
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_09)

USHORT	SfxResourceManager::RegisterBitmap(	const char *pSingleFile	)
{
	DBG_MEMTEST();
	return 0;
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_0a)

void	SfxResourceManager::ReleaseBitmap( USHORT nRegisterId )
{
	DBG_MEMTEST();
	DBG_ASSERT( nRegisterId < aResMgrBmpArr.Count(), "BmpArray ueberindiziert");
	if(nRegisterId < aResMgrBmpArr.Count()) {
		delete aResMgrBmpArr[nRegisterId];
		aResMgrBmpArr[nRegisterId] = 0;
	}
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_0b)

void	SfxResourceManager::Enter()
{
	DBG_MEMTEST();
	if(!pMessageTable)
		pMessageTable = new SfxMessageTable;
	++nEnterCount;
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_0c)

void	SfxResourceManager::Leave()
{
	DBG_MEMTEST();
	DBG_ASSERT(nEnterCount >= 1, "mehr Leave als Enter");
	--nEnterCount;
	if(!nEnterCount)
		ClearMsgTable();
}

// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_0d)

SfxMessageDescription* SfxResourceManager::CreateDescription(USHORT nId)
{
	DBG_MEMTEST();
	DBG_ASSERT( pMessageTable, "Zugriff auf MessageDesc ausserhalb von Enter");
	if(!pMessageTable) pMessageTable = new SfxMessageTable;
	SfxMessageDescription *pDesc = pMessageTable->Get(nId);
	if(!pDesc) {
		pDesc = MakeDesc(nId);
		pMessageTable->Insert(nId, pDesc);
	}
	return pDesc;
}

// CTOR / DTOR -----------------------------------------------------------
// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_0e)

SfxResourceManager::SfxResourceManager()
	: nEnterCount(0),
	  pMessageTable(0)
{
	DBG_MEMTEST();
}


// ------------------------------------------------------------------------
#pragma SEG_FUNCDEF(resmgr_0f)

SfxResourceManager::~SfxResourceManager()
{
	DBG_MEMTEST();
	USHORT nCount = aResMgrArr.Count();
	for(USHORT i = 0; i < nCount; ++i)
		delete aResMgrArr[i];
	nCount = aResMgrBmpArr.Count();
	for(i = 0; i < nCount; ++i)
		delete aResMgrBmpArr[i];
	ClearMsgTable();
}
/*------------------------------------------------------------------------

	$Log: resmgr.cxx,v $
	Revision 1.1.1.1  2000/09/18 16:52:29  hr
	initial import
	
	Revision 1.14  2000/09/17 16:47:20  willem.vandorp
	OpenOffice header added.
	
	Revision 1.13  2000/09/06 14:49:48  willem.vandorp
	Header and footer replaced.
	
	Revision 1.12  2000/06/30 11:55:04  mba
	MUST changes for AppWindow, UserEvent
	
	Revision 1.11  2000/04/19 13:05:16  mba
	UNO583 cleanup

	Revision 1.10  1996/09/09 14:06:08  MBA
	static functions _Impl


	  Rev 1.9   09 Sep 1996 16:06:08   MBA
   static functions _Impl

	  Rev 1.8   06 Jul 1995 17:26:44   OG
   segmentierung

	  Rev 1.7   22 Dec 1994 18:04:16   MI
   BUgfix: App-Resource immmer

	  Rev 1.6   25 Oct 1994 11:44:06   MI
   #pragma hdrstop

	  Rev 1.5   12 Oct 1994 20:49:50   MI
   Bugfix: Leerzeilen hinter #pragme

	  Rev 1.4   23 Sep 1994 08:35:10   MI
   fid nach sid

	  Rev 1.3   21 Sep 1994 11:31:14   MI
   pch

	  Rev 1.2   23 Jun 1994 16:31:22   MI
   GetDescription -> Create...

	  Rev 1.1   21 Jun 1994 12:05:54   VB
   warning entfernt

	  Rev 1.0   15 Jun 1994 11:57:02   VB
   Initial revision.

------------------------------------------------------------------------*/

#pragma SEG_EOFMODULE

