/*************************************************************************
 *
 *  $RCSfile: mailwindow.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:31 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// includes --------------------------------------------------------------

#include "mailwindow.hxx"
#include "maillistener.hxx"
#include "mailmodel.hxx"
#include "inimgr.hxx"
#include "sfxtypes.hxx"
#include "sfxresid.hxx"
#include "bindings.hxx"
#include "sfxsids.hrc"
#include "mailwindow.hrc"
#include "dialog.hrc"

#include <vcl/msgbox.hxx>	// InfoBox

// functions -------------------------------------------------------------

BOOL CreateFromAddress_Impl( String& rFrom )

/*	[Beschreibung]

	Diese Funktion versucht mit Hilfe des IniManagers eine From-Adresse
	zu erzeugen. daf"ur werden die Felder 'Vorname', 'Name' und 'EMail'
	aus der Applikations-Ini-Datei ausgelesen. Sollten diese Felder
	nicht gesetzt sein, wird FALSE zur"uckgegeben.

	[R"uckgabewert]

	TRUE:	Adresse konnte erzeugt werden.
	FALSE:	Adresse konnte nicht erzeugt werden.
*/

{
	SfxIniManager* pIni = SFX_INIMANAGER();
	String aName = pIni->Get( SFX_KEY_USER_NAME );
	String aFirstName = pIni->Get( SFX_KEY_USER_FIRSTNAME );

	if ( aFirstName.Len() || aName.Len() )
	{
		if ( aFirstName.Len() )
		{
			rFrom = TRIM( aFirstName );

			if ( aName.Len() )
				rFrom += ' ';
		}
		rFrom += TRIM( aName );
		// unerlaubte Zeichen entfernen
		rFrom.EraseAllChars( '<' );
		rFrom.EraseAllChars( '>' );
		rFrom.EraseAllChars( '@' );
	}
	String aEmailName = pIni->GetAddressToken( ADDRESS_EMAIL );
	// unerlaubte Zeichen entfernen
	aEmailName.EraseAllChars( '<' );
	aEmailName.EraseAllChars( '>' );

	if ( aEmailName.Len() )
	{
		if ( rFrom.Len() )
			rFrom += ' ';
		( ( rFrom += '<' ) += TRIM( aEmailName ) ) += '>';
	}
	else
		rFrom.Erase();
	return ( rFrom.Len() > 0 );
}

// class AddressEdit_Impl ------------------------------------------------

AddressEdit_Impl::AddressEdit_Impl( MailWindow_Impl* pWin, const ResId& rResId ) :

	Edit( pWin, rResId ),

	mpWindow( pWin )

{
}

void AddressEdit_Impl::KeyInput( const KeyEvent& rKEvt )
{
	switch ( rKEvt.GetKeyCode().GetCode() )
	{
		case KEY_RETURN:
			mpWindow->ReceiveAddress();
			break;

		case KEY_ESCAPE:
			mpWindow->ClearAddress();
			break;

		default:
			Edit::KeyInput( rKEvt );
	}
}

// class AddressListBox_Impl ---------------------------------------------

AddressListBox_Impl::AddressListBox_Impl( MailWindow_Impl* pWin, const ResId& rResId ) :

	ListBox( pWin, rResId ),

	mpWindow( pWin )

{
}

long AddressListBox_Impl::Notify( NotifyEvent& rNEvt )
{
	long nRet = 0;

	if ( rNEvt.GetType() == EVENT_KEYINPUT )
	{
		const KeyCode& rKey = rNEvt.GetKeyEvent()->GetKeyCode();

		if ( rKey.GetCode() == KEY_DELETE )
		{
			mpWindow->DeleteAddress();
			nRet = 1;
		}
	}

	return nRet ? nRet : ListBox::Notify( rNEvt );
}

// class MailWindow_Impl -------------------------------------------------

MailWindow_Impl::MailWindow_Impl( Window* pParent, SfxBindings* pBinds, const ResId& rResId ) :

	Window( pParent, rResId ),

	aSendBtn	( this, ResId( PB_MAILWIN_SEND ) ),
	aCloseBtn	( this, ResId( PB_MAILWIN_CLOSE ) ),
	aRoleLB		( this, ResId( LB_MAILWIN_ROLE ) ),
	aRoleEdit	( this, ResId( ED_MAILWIN_ROLE ) ),
	aAddressLB	( this, ResId( LB_MAILWIN_ADDRESS ) ),
	aSubjectFT	( this, ResId( FT_MAILWIN_SUBJECT ) ),
	aSubjectEdit( this, ResId( ED_MAILWIN_SUBJECT ) ),
	aFromFT		( this, ResId( FT_MAILWIN_FROM ) ),
	aFromEdit	( this, ResId( ED_MAILWIN_FROM ) ),
	aPrioFT		( this, ResId( FT_MAILWIN_PRIO ) ),
	aPrioLB		( this, ResId( LB_MAILWIN_PRIO ) ),
	aSendImage	(		ResId( IMG_MAILWIN_SEND ) ),
	aCloseImage	(		ResId( IMG_MAILWIN_CLOSE ) ),

	bSend			( sal_False ),
	pMailListener	( new MailListener_Impl ),
	pBindings		( pBinds )

{
	FreeResource();

	pMailListener->acquire();
	pMailListener->Initialize();
	pMailListener->SetChangeHdl( LINK( this, MailWindow_Impl, ChangeHdl ) );

	aSendBtn.SetClickHdl( LINK( this, MailWindow_Impl, SendHdl ) );
	aCloseBtn.SetClickHdl( LINK( this, MailWindow_Impl, CloseHdl ) );
	Link aLink( LINK( this, MailWindow_Impl, ModifyHdl ) );
	aRoleEdit.SetModifyHdl( aLink );
	aSubjectEdit.SetModifyHdl( aLink );
	aFromEdit.SetModifyHdl( aLink );

/*! (pb) not implemented in VCL
	aSendBtn.SetImage( aSendImage );
	aSendBtn.SetImageAlign( IMAGEALIGN_LEFT );
	aCloseBtn.SetImage( aCloseImage );
	aCloseBtn.SetImageAlign( IMAGEALIGN_LEFT );
*/

	String aFrom;
	if ( CreateFromAddress_Impl( aFrom ) )
		aFromEdit.SetText( aFrom );

	aRoleLB.SelectEntryPos(0);	// To
	aPrioLB.SelectEntryPos(2);	// Normal
	n6Size = LogicToPixel( Size( 6, 6 ), MAP_APPFONT ).Width();
	nMinWidth = ( aSubjectEdit.GetSizePixel().Width() / 2 ) + n6Size +
				aPrioFT.GetSizePixel().Width() + aPrioLB.GetSizePixel().Width();

	ModifyHdl( NULL );

	aCloseTimer.SetTimeout( 200 );
	aCloseTimer.SetTimeoutHdl( LINK( this, MailWindow_Impl, CloseHdl ) );

	aInitTimer.SetTimeout( 500 );
	aInitTimer.SetTimeoutHdl( LINK( this, MailWindow_Impl, InitHdl ) );
	aInitTimer.Start();
}

// -----------------------------------------------------------------------

MailWindow_Impl::~MailWindow_Impl()
{
	pMailListener->Deinitialize();
	pMailListener->release();
}

// -----------------------------------------------------------------------

void MailWindow_Impl::FillModel( SfxMailModel_Impl& rModel )
{
	if ( aRoleEdit.GetText().Len() > 0 )
		ReceiveAddress();
	USHORT i, nCount = aAddressLB.GetEntryCount();
	String aPrefix( DEFINE_CONST_UNICODE(": ") );
	for ( i = 0; i < nCount; ++i )
	{
		String aAddress = aAddressLB.GetEntry(i);
		USHORT nPos = aAddress.Search( aPrefix );
		aAddress.Erase( 0, nPos + 2 );
		SfxMailModel_Impl::AddressRole eRole =
			(SfxMailModel_Impl::AddressRole)(long)aAddressLB.GetEntryData(i);
		rModel.AddAddress( aAddress, eRole );
	}
	rModel.SetSubject( aSubjectEdit.GetText() );
	rModel.SetPriority( (SfxMailModel_Impl::MailPriority)aPrioLB.GetSelectEntryPos() );
	rModel.SetFromAddress( aFromEdit.GetText() );
}

// -----------------------------------------------------------------------

IMPL_LINK( MailWindow_Impl, SendHdl, PushButton*, EMPTYARG )
{
	Send();
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( MailWindow_Impl, CloseHdl, void*, EMPTYARG )
{
    pBindings->Execute( SID_MAIL_CHILDWIN );
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( MailWindow_Impl, ModifyHdl, Edit*, EMPTYARG )
{
	aSendBtn.Enable( CanSend() );
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( MailWindow_Impl, ChangeHdl, MailListener_Impl*, pListener )
{
	aOutboxServerName = pListener->GetOutboxServer();
	if ( !aOutboxServerName.Len() )
	{
		ErrorBox aBox( this, SfxResId( RID_ERRBOX_NO_SMTPSERVER ) );
		aBox.Execute();
	}
	ModifyHdl( NULL );
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( MailWindow_Impl, InitHdl, Timer*, EMPTYARG )
{
	ChangeHdl( pMailListener );
	return 0;
}

// -----------------------------------------------------------------------

void MailWindow_Impl::Resize()
{
	Size aWinSize = GetSizePixel();
	Point aPos = aFromEdit.GetPosPixel();
	Size aOldSize = aFromEdit.GetSizePixel();
	Size aNewSize = aOldSize;
	aNewSize.Width() = aWinSize.Width() - aPos.X() - n6Size;

	if ( nMinWidth < aNewSize.Width() )
	{
		Size aAddressSize = aAddressLB.GetSizePixel();
		aAddressSize.Width() = aNewSize.Width();
		aAddressLB.SetSizePixel( aAddressSize );

		aRoleEdit.SetSizePixel( aNewSize );
		aSubjectEdit.SetSizePixel( aNewSize );

		Size aPrioSize1 = aPrioFT.GetSizePixel();
		Size aPrioSize2 = aPrioLB.GetSizePixel();
		aNewSize.Width() -= ( n6Size + aPrioSize1.Width() + aPrioSize2.Width() );
		aFromEdit.SetSizePixel( aNewSize );

		long nNewXPos = aFromEdit.GetPosPixel().X() + aNewSize.Width() + ( n6Size / 2 );
		Point aNewPos = aPrioFT.GetPosPixel();
		aNewPos.X() = nNewXPos;
		aPrioFT.SetPosPixel( aNewPos );
		aNewPos = aPrioLB.GetPosPixel();
		aNewPos.X() = nNewXPos + aPrioSize1.Width() + ( n6Size / 2 );
		aPrioLB.SetPosPixel( aNewPos );
	}
}

// -----------------------------------------------------------------------

sal_Bool MailWindow_Impl::CanSend() const
{
	return ( aOutboxServerName.Len() > 0 &&
			 aSubjectEdit.GetText().Len() > 0 &&
			 aFromEdit.GetText().Len() > 0 &&
			 ( aRoleEdit.GetText().Len() > 0 || aAddressLB.GetEntryCount() > 0 ) );
}

// -----------------------------------------------------------------------

void MailWindow_Impl::Send()
{
	SfxMailModel_Impl aModel( pBindings );
	EnterWait();
	FillModel( aModel );
	bSend = aModel.Send();
	LeaveWait();
	if ( !bSend )
	{
		ErrorBox aBox( this, SfxResId( RID_ERRBOX_SEND_FAILED ) );
		aBox.Execute();
	}
	else
	{
		InfoBox aBox( this, SfxResId( RID_INFBOX_SEND_SUCCESS ) );
		aBox.Execute();
		aCloseTimer.Start();
	}
}

// -----------------------------------------------------------------------

void MailWindow_Impl::ReceiveAddress()
{
	USHORT nRole = aRoleLB.GetSelectEntryPos();
	String aAddress = aRoleLB.GetEntry( nRole );
	aAddress += DEFINE_CONST_UNICODE(": ");
	aAddress += aRoleEdit.GetText();
	USHORT nPos = aAddressLB.InsertEntry( aAddress );
	aAddressLB.SetEntryData( nPos, (void*)(long)nRole );
	aRoleEdit.SetText( String() );
}

// -----------------------------------------------------------------------

void MailWindow_Impl::ClearAddress()
{
	aRoleEdit.SetText( String() );
}

// -----------------------------------------------------------------------

void MailWindow_Impl::DeleteAddress()
{
	USHORT nPos = aAddressLB.GetSelectEntryPos();
	aAddressLB.RemoveEntry( nPos );
	USHORT nCount = aAddressLB.GetEntryCount();

	if ( nCount )
	{
		if ( nPos >= nCount )
			nPos = ( nCount - 1 );
		aAddressLB.SelectEntryPos( nPos );
	}
}


