/*************************************************************************
 *
 *  $RCSfile: explorer.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:33 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// includes ******************************************************************

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#pragma hdrstop

#include "explorer.hxx"
#include "app.hxx"
#include "docfile.hxx"
#include "sfxresid.hxx"
#include "sfxdir.hxx"
#include "inimgr.hxx"
#include "explorer.hrc"

// defines *******************************************************************

// Registration
#define REGISTRY_INSERTPOS        0

// Settingsspeicherung
#define SETTINGS_LIST_SEPERATOR   ';'

// Typenliste
#define TYPELIST_INSERTPOS        0

// Fileinfo-Registration (Apple)
#ifdef MAC
#define FILEINFOREG_SIZE          5
#define FILEINFOREG_RESIZE        5
#define FILEINFOREG_INSERTPOS     LIST_APPEND
#endif

// Extensionregistration
#define EXTENSIONREG_SIZE         5
#define EXTENSIONREG_RESIZE       5

// Konstanten fuer die Suche nach Verknuepfungen
#define EXPLORER_BMKSEARCH_TIMEOUT     100
#define EXPLORER_BMKSEARCH_STACKSIZE   10
#define EXPLORER_BMKSEARCH_STACKRESIZE 5
#define EXPLORER_BMKSEARCH_SELECTION   FSYS_KIND_FILE | FSYS_KIND_DIR

// Ursprungsliste
#define EXPLORER_ROOTLIST_SIZE      10
#define EXPLORER_ROOTLIST_RESIZE    5
#define EXPLORER_ROOTLIST_INSERTPOS 0UL

// Makros ********************************************************************

#define VALID_SEARCHMODE( SearchMode ) \
    ( ( ( ( SearchMode & SEARCH_ASYNCHRON ) == SEARCH_ASYNCHRON ) && \
		( !( SearchMode & SEARCH_SYNCHRON ) ) ) || \
	  ( ( ( ( SearchMode & SEARCH_SYNCHRON ) == SEARCH_SYNCHRON ) ) && \
		( !( SearchMode & SEARCH_ASYNCHRON ) ) ) )

//****************************************************************************
// SfxExplorer
//****************************************************************************

static SfxExplorer* pExplorer = NULL;

SfxExplorer::SfxExplorer()
{
	_pSmallImageList = NULL;
	_pFirstBigImageList = NULL;
	_pSecondBigImageList = NULL;
	_pSmallMergeImageList = NULL;
	_pBigMergeImageList =  NULL;
}

//****************************************************************************

SfxExplorer::~SfxExplorer()
{
	// Destruktion der Imageliste
	delete _pSmallMergeImageList;
	delete _pBigMergeImageList;
	delete _pFirstBigImageList;
	delete _pSecondBigImageList;
	delete  _pSmallImageList;
}

//****************************************************************************

Image SfxExplorer::GetImage
(
	USHORT nId,    //Id des zu erzeugenden Images
	BOOL   bBig    /* TRUE, grosses Image
					  FALSE, kleines Image */
)
/*
  [Beschreibung]
  Die Methode liefert das Image mit der spezifizierten Id und der
  spezifizierten Groesse.
  */
{
    if ( !pExplorer )
        pExplorer = new SfxExplorer;
	ImageList* pList = NULL;
	if ( bBig )
	{
		if ( nId >= IMG_SFX_EXP_SECONDAREA_START )
		{
			if ( !pExplorer->_pSecondBigImageList )
			{
				DBG_TRACE( "SfxExplorer:2te grosse Imageliste erzeugt" );
				pExplorer->_pSecondBigImageList = new
					ImageList( SfxResId( IMG_SFX_EXPLORER_LC2 ) );
			}
			pList = pExplorer->_pSecondBigImageList;
		}
		else
		{
			if ( !pExplorer->_pFirstBigImageList )
			{
				DBG_TRACE( "SfxExplorer:1te grosse Imageliste erzeugt" );
				pExplorer->_pFirstBigImageList = new
					ImageList( SfxResId( IMG_SFX_EXPLORER_LC1 ) );
			}
			pList = pExplorer->_pFirstBigImageList;
		}
	}
	else
	{
		if ( !pExplorer->_pSmallImageList )
		{
			pExplorer->_pSmallImageList =
				new ImageList( SfxResId( IMG_SFX_EXPLORER_SC ) );
		}
		pList = pExplorer->_pSmallImageList;
	}
	return pList->GetImage( nId );
}

//****************************************************************************

Image SfxExplorer::ModifyImage
(
	const Image& rImage,                /* Image, aus dem das Symbol
										   konstruiert werden soll */
	SfxExplorerModifyImage eModifier    // Art der Aenderung
)
/*
  [Beschreibung]
  Veraendert das spezifizierte Image auf die spezifizierte Art und Weise.

  [Rueckgabewert]
  das neue Image
  */
{
	DBG_ASSERT( eModifier != EXPLORER_MODIFYIMAGE_NOT,
				"SfxExplorer:Ueberfluessiger Aufruf" );

	// Bestimmung der Imageliste
    if ( !pExplorer )
        pExplorer = new SfxExplorer;
	ImageList* pImgList = NULL;
	if ( rImage.GetSizePixel().Height() <=16 )
	{
		if ( !pExplorer->_pSmallMergeImageList )
		{
			pExplorer->_pSmallMergeImageList =
				new ImageList( SfxResId( IMG_SFX_MERGEIMAGELIST_SC ) );
		}
		pImgList = pExplorer->_pSmallMergeImageList;
	}
	else
	{
		if ( !pExplorer->_pBigMergeImageList )
		{
			pExplorer->_pBigMergeImageList =
				new ImageList( SfxResId( IMG_SFX_MERGEIMAGELIST_LC ) );
		}
		pImgList = pExplorer->_pBigMergeImageList;
	}

	// Konstruktion des neuen Images
	USHORT nImgId = IMG_SFX_LINKIMAGE;
	if ( eModifier == EXPLORER_MODIFYIMAGE_INDEX )
	{
		nImgId = IMG_SFX_INDEXEDIMAGE;
	}
	// Neue ImageListe erzeugen, um Problem der Referenz-Counter in der
	// ImageListe zu umgehen (ansonsten wuerde die ImageList immer groesser
	// werden)
	ImageList aTempImgList( 2, 2 );
	aTempImgList.AddImage( 1, rImage );
	aTempImgList.AddImage( 2, pImgList->GetImage( nImgId ) );
	aTempImgList.MergeImage( 1, 2 );
	return aTempImgList.GetImage( 1 );
}

//****************************************************************************

BOOL SfxExplorer::HasTempExplorerRoot()

/*	[Beschreibung]

	Gibt zur"uck, ob ein Explorer-Ursprung gesetzt ist.
*/

{
	return SFX_INIMANAGER()->Get( SFX_KEY_EXPLORERROOT ).Len() != 0;
}


