/*************************************************************************
 *
 *  $RCSfile: fileview.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:33 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "fileview.hxx"
#include "helper.hxx"
#include "infodlgs.hxx"
#include "sfxresid.hxx"
#include "expfile.hxx"
#include "exptypes.hxx"
#include "iodlg.hxx"

#include "explorer.hrc"

#include <tools/urlobj.hxx>
#include <svtools/headbar.hxx>
#include <svtools/svtabbx.hxx>

using namespace ::com::sun::star::uno;

// class ViewTabListBox_Impl ---------------------------------------------

class ViewTabListBox_Impl : public SvHeaderTabListBox
{
private:
	HeaderBar*		mpHeaderBar;
	sal_Bool		mbResizeDisabled;

	DECL_LINK( HeaderSelect_Impl, HeaderBar * );
	DECL_LINK( HeaderEndDrag_Impl, HeaderBar * );

public:
	ViewTabListBox_Impl( Window* pParent );
	~ViewTabListBox_Impl();

	virtual void	Resize();

	void			ClearAll();
};

// -----------------------------------------------------------------------

ViewTabListBox_Impl::ViewTabListBox_Impl( Window* pParent ) :

	SvHeaderTabListBox( pParent, 0 ),

	mpHeaderBar		( NULL ),
	mbResizeDisabled( FALSE )

{
	Size aBoxSize = pParent->GetOutputSizePixel();
	mpHeaderBar = new HeaderBar( pParent, WB_BUTTONSTYLE | WB_BOTTOMBORDER );
	mpHeaderBar->SetPosSizePixel( Point( 0, 0 ), Size( aBoxSize.Width(), 16 ) );

	HeaderBarItemBits nBits = ( HIB_LEFT | HIB_VCENTER );
	mpHeaderBar->InsertItem( 1, String( SfxResId( STR_SFX_IODLG_COLUMN_TITLE ) ), 220, nBits );
	mpHeaderBar->InsertItem( 2, String( SfxResId( STR_SFX_IODLG_COLUMN_SIZE ) ), 80, nBits );
	mpHeaderBar->InsertItem( 3, String( SfxResId( STR_SFX_IODLG_COLUMN_DATE ) ), 500, nBits );

	mpHeaderBar->SetSelectHdl( LINK( this, ViewTabListBox_Impl, HeaderSelect_Impl ) );
	mpHeaderBar->SetEndDragHdl( LINK( this, ViewTabListBox_Impl, HeaderEndDrag_Impl ) );

	Size aHeadSize = mpHeaderBar->GetSizePixel();
	SetPosSizePixel( Point( 0, aHeadSize.Height() ),
					 Size( aBoxSize.Width(), aBoxSize.Height() - aHeadSize.Height() ) );
	InitHeaderBar( mpHeaderBar );
	SetHighlightRange();

	Show();
	mpHeaderBar->Show();
}

// -----------------------------------------------------------------------

ViewTabListBox_Impl::~ViewTabListBox_Impl()
{
	delete mpHeaderBar;
}

// -----------------------------------------------------------------------

IMPL_LINK( ViewTabListBox_Impl, HeaderSelect_Impl, HeaderBar*, pBar )
{
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( ViewTabListBox_Impl, HeaderEndDrag_Impl, HeaderBar*, pBar )
{
	if ( pBar && !pBar->GetCurItemId() )
		return 0;

	if ( !mpHeaderBar->IsItemMode() )
	{
		Size aSize;
		USHORT nTabs = mpHeaderBar->GetItemCount();
		long nTmpSize = 0;

		for ( USHORT i = 1; i <= nTabs; ++i )
		{
			long nWidth = mpHeaderBar->GetItemSize(i);
			aSize.Width() =  nWidth + nTmpSize;
			nTmpSize += nWidth;
			SetTab( i, aSize.Width(), MAP_PIXEL );
		}
	}
	return 0;
}

// -----------------------------------------------------------------------

void ViewTabListBox_Impl::Resize()
{
	SvTabListBox::Resize();
	Size aBoxSize = Control::GetParent()->GetOutputSizePixel();

	if ( mbResizeDisabled || !aBoxSize.Width() )
		return;

	Size aBarSize = mpHeaderBar->GetSizePixel();
	aBarSize.Width() = aBoxSize.Width();
	mpHeaderBar->SetSizePixel( aBarSize );
	mbResizeDisabled = TRUE;
	Point aPos = GetPosPixel();
	SetPosSizePixel( Point( 0, aBarSize.Height() ),
					 Size( aBoxSize.Width(), aBoxSize.Height() - aBarSize.Height() ) );
	mbResizeDisabled = FALSE;
}

// -----------------------------------------------------------------------

void ViewTabListBox_Impl::ClearAll()
{
	for ( USHORT i = 0; i < GetEntryCount(); ++i )
		delete (ContentEntry_Impl*)GetEntry(i)->GetUserData();
	Clear();
}

// class SfxFileView_Impl ------------------------------------------------

void SfxFileView_Impl::OpenFolder()
{
	mpView->ClearAll();
	Sequence< ::rtl::OUString > aFileProperties =
		SfxContentHelper::GetFolderContentProperties( maViewURL, sal_True );
	const ::rtl::OUString* pFileProperties  = aFileProperties.getConstArray();
	UINT32 i, nCount = aFileProperties.getLength();
	sal_Bool bExecFilter = ( maCurrentFilter != maAllFilter );
	for ( i = 0; i < nCount; ++i )
	{
		String aRow( pFileProperties[i] );
		// extract columns
		String aTitle, aSize, aDate, aURL;
		xub_StrLen nIdx = 0;
		aTitle = aRow.GetToken( 0, '\t', nIdx );
		aSize = aRow.GetToken( 0, '\t', nIdx );
		aDate = aRow.GetToken( 0, '\t', nIdx );
		aURL = aRow.GetToken( 0, '\t', nIdx );
		sal_Unicode cFolder = aRow.GetToken( 0, '\t', nIdx ).GetChar(0);
		sal_Bool bIsFolder = ( '1' == cFolder );

		if ( mbOnlyFolder && !bIsFolder )
			continue;

		// build new row
		String aNewRow = aTitle;
		aNewRow += '\t';
		if ( !bIsFolder )
		{
			// folder haven't a size
			sal_Int64 nSize = aSize.ToInt64();
			aNewRow += CreateExactSizeText( nSize );
		}
		aNewRow += '\t';
		aNewRow += aDate;
		// detect image
		sal_Bool bDoInsert = sal_True;
		Image aImage;
		if ( bIsFolder )
			aImage = maFolderImage;
		else
		{
			INetURLObject aObj( aURL );
			aImage = SfxExplorerFile::GetImage( aObj, FALSE );

			if ( bExecFilter )
			{
				bDoInsert = sal_False;
				String aExtension = aObj.getExtension();
				aExtension.ToLowerAscii();
				USHORT nCount = maCurrentFilter.GetTokenCount();

				for ( USHORT i = 0; i < nCount; ++i )
				{
					String aFilter = maCurrentFilter.GetToken(i).Copy(2);
					if ( aFilter == aExtension )
					{
						bDoInsert = sal_True;
						break;
					}
				}
			}
		}

		if ( bDoInsert )
		{
			// insert entry and set user data
			SvLBoxEntry* pEntry = mpView->InsertEntry( aNewRow, aImage, aImage );
			ContentEntry_Impl* pUserData = new ContentEntry_Impl( aURL, bIsFolder );
			pEntry->SetUserData( pUserData );
		}
	}

	mpView->SelectAll( FALSE );
	SvLBoxEntry* pFirst = mpView->First();
	if ( pFirst )
		mpView->SetCursor( pFirst, TRUE );
}

// -----------------------------------------------------------------------

SfxFileView_Impl::SfxFileView_Impl( Window* pParent, const ResId& rResId, sal_Bool bOnlyFolder ) :

	Control( pParent, rResId ),

	mbOnlyFolder	( bOnlyFolder ),
	maFolderImage	( SfxResId( IMG_SFX_FOLDER ) )

{
	mpView = new ViewTabListBox_Impl( this );
	long pTabs[] = { 4, 20, 220, 300, 600 };
	mpView->SetTabs( &pTabs[0], MAP_PIXEL );
	mpView->SetTabJustify( 1, AdjustRight ); // column "Size"
	maAllFilter = String::CreateFromAscii( FILEDIALOG_FILTER_ALL );
}

// -----------------------------------------------------------------------

SfxFileView_Impl::~SfxFileView_Impl()
{
	delete mpView;
}

// -----------------------------------------------------------------------

String SfxFileView_Impl::GetCurrentURL() const
{
	String aURL;
	SvLBoxEntry* pEntry = mpView->FirstSelected();
	if ( pEntry && pEntry->GetUserData() )
		aURL = ( (ContentEntry_Impl*)pEntry->GetUserData() )->maURL;
	return aURL;
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::CreateNewFolder()
{
	INetURLObject aObj( maViewURL );
	QueryFolderNameDialog aDlg( aObj.getName(), String( SfxResId( STR_SFX_FOLDER ) ));

	if ( aDlg.Execute() == RET_OK )
	{
		String aNewFolderName( aDlg.GetName() );
		aObj.insertName( aNewFolderName );
		String aURL = aObj.GetMainURL();
		if ( SfxContentHelper::MakeFolder( aURL ) )
		{
			String aEntry = aObj.getName( INetURLObject::LAST_SEGMENT,
										  true, INetURLObject::DECODE_WITH_CHARSET );
			SvLBoxEntry* pEntry = mpView->InsertEntry( aEntry, maFolderImage, maFolderImage );
			ContentEntry_Impl* pUserData = new ContentEntry_Impl( aURL, TRUE );
			pEntry->SetUserData( pUserData );
			mpView->MakeVisible( pEntry );
		}
	}
}

// -----------------------------------------------------------------------

sal_Bool SfxFileView_Impl::PreviousLevel( String& rNewURL )
{
	sal_Bool bRet = FALSE;
	INetURLObject aObj( maViewURL );
	sal_Int32 nCount = aObj.getSegmentCount();

	if ( nCount > 0 )
	{
		aObj.removeSegment();
		rNewURL = aObj.GetMainURL();
		Initialize( rNewURL, maCurrentFilter );
		bRet = TRUE;
	}
	return bRet;
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetHelpId( sal_uInt32 nHelpId )
{
	mpView->SetHelpId( nHelpId );
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetSizePixel( const Size& rNewSize )
{
	Control::SetSizePixel( rNewSize );
	mpView->SetSizePixel( rNewSize );
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetPosSizePixel( const Point& rNewPos, const Size& rNewSize )
{
	SetPosPixel( rNewPos );
	SetSizePixel( rNewSize );
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::Initialize( const String& rURL, const String& rFilter )
{
	maViewURL = rURL;
	maCurrentFilter = rFilter;
	maCurrentFilter.ToLowerAscii();
	OpenFolder();
	maOpenDoneLink.Call( this );
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::ExecuteFilter( const String& rFilter )
{
	maCurrentFilter = rFilter;
	maCurrentFilter.ToLowerAscii();
	OpenFolder();
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetNoSelection()
{
	mpView->SelectAll( FALSE );
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetFocusInView()
{
	mpView->GrabFocus();
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::ResetCursor()
{
	// deselect
	SvLBoxEntry* pEntry = mpView->FirstSelected();
	if ( pEntry )
		mpView->Select( pEntry, FALSE );
	// set cursor to the first entry
	mpView->SetCursor( mpView->First(), TRUE );
	mpView->Update();
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetSelectHdl( const Link& rHdl )
{
	mpView->SetSelectHdl( rHdl );
}

// -----------------------------------------------------------------------

void SfxFileView_Impl::SetDoubleClickHdl( const Link& rHdl )
{
	mpView->SetDoubleClickHdl( rHdl );
}


