/*************************************************************************
 *
 *  $RCSfile: tbxconf.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:35 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SV_WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif
#include <vos/mutex.hxx>
#pragma hdrstop

#include "tbxconf.hxx"
#include "app.hxx"
#include "toolbox.hrc"
#include "sfxresid.hxx"
#include "dispatch.hxx"
#include "tbxchild.hxx"
#include "tbxmgr.hxx"
#include "appdata.hxx"

static const USHORT nVersion = 6;

// Static member
SfxToolBoxConfig *SfxToolBoxConfig::pTbxConfig = NULL;

SfxToolBoxConfig* SfxToolBoxConfig::GetOrCreate()
{
	NAMESPACE_VOS( OGuard ) aGuard( NAMESPACE_VOS(OMutex)::getGlobalMutex() );
	if ( !pTbxConfig )
	{
		pTbxConfig = new SfxToolBoxConfig;
		pTbxConfig->Initialize();
	}

	return pTbxConfig;
}

void SfxToolBoxConfig::Release()
{
	NAMESPACE_VOS( OGuard ) aGuard( NAMESPACE_VOS(OMutex)::getGlobalMutex() );
	DELETEZ( pTbxConfig );
}

//=========================================================================
SfxToolBoxConfig::SfxToolBoxConfig() :
	SfxConfigItem( SFX_ITEMTYPE_TOOLBOXCONFIG ),
	eSymbolSet( SFX_SYMBOLS_SMALL_COLOR ),
	nOutStyle( TOOLBOX_STYLE_FLAT ),
	bQuiet(FALSE),
	bStatusBarVisible(TRUE)
{
	pPosArr = new SfxToolBoxCfgItemArr_Impl();
}

//-------------------------------------------------------------------------
SfxToolBoxConfig::~SfxToolBoxConfig()
{
	for (USHORT n=0; n<pPosArr->Count(); n++)
		delete (*pPosArr)[n];
	delete pPosArr;
}

//-------------------------------------------------------------------------
int SfxToolBoxConfig::Load(SvStream& rStream)
{
	USHORT nFileVersion;
	rStream >> nFileVersion;
	if ( nFileVersion < 4 )
		return SfxConfigItem::WARNING_VERSION;

	USHORT n;
	for (n=0; n<pPosArr->Count(); n++)
		delete (*pPosArr)[n];
	pPosArr->Remove(0, n);

	USHORT nCount, nPos, nVis, nFloatingLines, nLines, nAlign, nFloating;
	USHORT nButtonType = (USHORT) BUTTON_SYMBOL;
	String aName;
	Point aFloatingPos;

	rStream >> nCount;
	for (n=0; n<nCount; n++)
	{
		rStream >> nPos
				>> nVis
				>> nFloating;
		rStream.ReadByteString(aName,RTL_TEXTENCODING_UTF8);
		rStream	>> nAlign
				>> aFloatingPos
				>> nFloatingLines
				>> nLines;
		if ( nFileVersion > 4 )
			rStream >> nButtonType;

		SfxToolBoxCfgItem_Impl* pItem =
			new SfxToolBoxCfgItem_Impl( nPos, (BOOL) nVis,
			(BOOL) nFloating, aName, (SfxChildAlignment) nAlign);
		pItem->aFloatingPos = aFloatingPos;
		pItem->nFloatingLines = nFloatingLines;
		pItem->nLines = nLines;
		pItem->nWinBits = GetWinBits( nPos );
		pItem->eButtonType = (ButtonType) nButtonType;
		pPosArr->Append( pItem );
	}

	USHORT nSymbolSet;
	rStream >> nSymbolSet;
	if ( nFileVersion < 5 )
		rStream >> nButtonType;
	rStream >> nOutStyle;

	SfxSymbolSet eNewSet = (SfxSymbolSet) nSymbolSet;
	if (eNewSet == SFX_SYMBOLS_SMALL_MONO)
		eNewSet = SFX_SYMBOLS_SMALL_COLOR;
	if (eNewSet == SFX_SYMBOLS_LARGE_MONO)
		eNewSet = SFX_SYMBOLS_LARGE_COLOR;

	eSymbolSet = eNewSet;
	if (!bQuiet)
	{
		SFX_IMAGEMANAGER()->SetSymbolSet(eSymbolSet);
		SFX_IMAGEMANAGER()->SetOutStyle(nOutStyle);
	}

	if ( nFileVersion > 5 )
		rStream >> bStatusBarVisible;

	return SfxConfigItem::ERR_OK;
}

//-------------------------------------------------------------------------
BOOL SfxToolBoxConfig::Store(SvStream& rStream)
{
	rStream << nVersion;
	rStream << (USHORT) pPosArr->Count();
	for (USHORT n=0; n<pPosArr->Count(); n++)
	{
		SfxToolBoxCfgItem_Impl *pItem = (*pPosArr)[n];

		if ( pItem->nPos == SFX_OBJECTBAR_COMMONTASK )
			pItem->nFloatingLines = USHRT_MAX;

		rStream << pItem->nPos
				<< (USHORT) pItem->bVisible
				<< (USHORT) pItem->bFloating;
		rStream.WriteByteString(pItem->aName,RTL_TEXTENCODING_UTF8);
		rStream	<< (USHORT) pItem->eAlign
				<< pItem->aFloatingPos
				<< pItem->nFloatingLines
				<< pItem->nLines
				<< (USHORT) pItem->eButtonType;
	}

	rStream << (USHORT) eSymbolSet << nOutStyle;
	rStream << bStatusBarVisible;
	return TRUE;
}

//-------------------------------------------------------------------------
String SfxToolBoxConfig::GetName() const
{
	return String(SfxResId(STR_TOOLBOX_CFGITEM));
}

//-------------------------------------------------------------------------
void SfxToolBoxConfig::UseDefault()
{
/*
	// Wird nicht mehr "uber Config-Dialog gesteuert, sondern "uber
	// Optionen
	eSymbolSet = SFX_SYMBOLS_SMALL_COLOR;
	nOutStyle = TOOLBOX_STYLE_FLAT;
*/
	if (!bQuiet)
	{
		SFX_IMAGEMANAGER()->SetSymbolSet(eSymbolSet);
		SFX_IMAGEMANAGER()->SetOutStyle(nOutStyle);
	}

	USHORT n;
	for (n=0; n<pPosArr->Count(); n++)
		delete (*pPosArr)[n];
	pPosArr->Remove(0, n);

	bStatusBarVisible = TRUE;

	SfxConfigItem::UseDefault();
}

//-------------------------------------------------------------------------
BOOL SfxToolBoxConfig::Reconfigure(SvStream& rStream, BOOL bDefault)
{
	BOOL bRet = SfxConfigItem::Reconfigure(rStream, bDefault);
	return bRet;
}

//-------------------------------------------------------------------------
BOOL SfxToolBoxConfig::IsToolBoxPositionVisible(USHORT nPos) const
{
	for (USHORT n=0; n<pPosArr->Count(); n++)
		if ((*pPosArr)[n]->nPos == nPos)
			return (*pPosArr)[n]->bVisible;

	return GetObjectbarVisibilityDefault( nPos );
}

//-------------------------------------------------------------------------
void SfxToolBoxConfig::SetToolBoxPositionVisible(USHORT nPos, BOOL bVis)
{
	SfxToolBoxCfgItem_Impl *pItem=0;

	for (USHORT n=0; n<pPosArr->Count(); n++)
	{
		if ((*pPosArr)[n]->nPos == nPos)
		{
			pItem = (*pPosArr)[n];
			break;
		}
	}

	if (!pItem) 					// ggf. neuen Eintrag erzeugen
		pItem = (SfxToolBoxCfgItem_Impl*) GetCfgItem_Impl( nPos );
	pItem->bVisible = bVis;
	SetDefault(FALSE);
}

//-------------------------------------------------------------------------
String SfxToolBoxConfig::GetToolBoxPositionName(USHORT nPos) const
{
	for (USHORT n=0; n<pPosArr->Count(); n++)
		if ((*pPosArr)[n]->nPos == nPos)
			return (*pPosArr)[n]->aName;

	switch ( nPos )
	{
		case SFX_OBJECTBAR_APPLICATION :
			return String(SfxResId(STR_FUNCTIONBAR));
		case SFX_OBJECTBAR_OBJECT :
			return String(SfxResId(STR_OBJECTBAR));
		case SFX_OBJECTBAR_TOOLS :
			return String(SfxResId(STR_TOOLBAR));
		case SFX_OBJECTBAR_MACRO :
			return String(SfxResId(STR_MACROBAR));
		case SFX_OBJECTBAR_OPTIONS :
			return String(SfxResId(STR_OPTIONBAR));
		case SFX_OBJECTBAR_COMMONTASK:
			return String(SfxResId(STR_COMMONTASK));
		case SFX_OBJECTBAR_NAVIGATION:
			return String(SfxResId(STR_NAVIGATION));
	}

	return String();
}

//-------------------------------------------------------------------------
void SfxToolBoxConfig::SetToolBoxPositionName(USHORT nPos, const String *pName)
{
	SfxToolBoxCfgItem_Impl *pItem=0;

	for (USHORT n=0; n<pPosArr->Count(); n++)
	{
		if ((*pPosArr)[n]->nPos == nPos)
		{
			pItem = (*pPosArr)[n];
			break;
		}
	}

	if (!pItem) 					// ggf. neuen Eintrag erzeugen
		pItem = (SfxToolBoxCfgItem_Impl*) GetCfgItem_Impl( nPos );
	if ( pName )
		pItem->aName = *pName;
	else
		pItem->aName = String();
	SetDefault(FALSE);
}

//-------------------------------------------------------------------------
void SfxToolBoxConfig::SetSymbolSet(SfxSymbolSet eSet)
{
	if ( eSet != eSymbolSet )
	{
		if (eSet == SFX_SYMBOLS_SMALL)
			eSet = SFX_SYMBOLS_SMALL_COLOR;
		else if (eSet == SFX_SYMBOLS_LARGE)
			eSet = SFX_SYMBOLS_LARGE_COLOR;

		SetDefault(FALSE);
		SfxSymbolSet eOldSet = eSymbolSet;
		eSymbolSet = eSet;
		if (!bQuiet)
			SFX_IMAGEMANAGER()->SetSymbolSet(eSet);
	}
}

//-------------------------------------------------------------------------
void SfxToolBoxConfig::SetButtonType( USHORT nPos, ButtonType eType )
{
	SfxToolBoxCfgItem_Impl *pItem=0;
	for (USHORT n=0; n<pPosArr->Count(); n++)
	{
		if ((*pPosArr)[n]->nPos == nPos)
		{
			pItem = (*pPosArr)[n];
			break;
		}
	}

	if (!pItem) 					// ggf. neuen Eintrag erzeugen
		pItem = (SfxToolBoxCfgItem_Impl*) GetCfgItem_Impl( nPos );
	pItem->eButtonType = eType;
	SetDefault(FALSE);
}

ButtonType SfxToolBoxConfig::GetButtonType( USHORT nPos ) const
{
	for (USHORT n=0; n<pPosArr->Count(); n++)
		if ((*pPosArr)[n]->nPos == nPos)
			return (*pPosArr)[n]->eButtonType;
	return BUTTON_SYMBOL;
}

//-------------------------------------------------------------------------
void SfxToolBoxConfig::SetOutStyle( USHORT nStyle )
{
	if ( nStyle != nOutStyle )
	{
		SetDefault(FALSE);
		nOutStyle = nStyle;
		if (!bQuiet)
			SFX_IMAGEMANAGER()->SetOutStyle( nStyle );
	}
}

const SfxToolBoxCfgItem_Impl* SfxToolBoxConfig::GetCfgItem_Impl( USHORT nPos )
{
	for (USHORT n=0; n<pPosArr->Count(); n++)
		if ((*pPosArr)[n]->nPos == nPos)
			return (*pPosArr)[n];

	// Noch nicht vorhanden, also ein Item mit Defaultwerten anlegen
	BOOL bFloating = FALSE;
	if ( nPos == SFX_OBJECTBAR_FULLSCREEN || nPos == SFX_OBJECTBAR_RECORDING ||
			nPos == SFX_OBJECTBAR_COMMONTASK )
		bFloating = TRUE;

	BOOL bVis = GetObjectbarVisibilityDefault( nPos );
	SfxChildAlignment eAlign;
	switch ( nPos )
	{
		case SFX_OBJECTBAR_APPLICATION :
			eAlign = SFX_ALIGN_HIGHESTTOP;
			break;

		case SFX_OBJECTBAR_OBJECT :
		case SFX_OBJECTBAR_MACRO :
		case SFX_OBJECTBAR_COMMONTASK:
			eAlign = SFX_ALIGN_TOOLBOXTOP;
			break;

		case SFX_OBJECTBAR_TOOLS :
			eAlign = SFX_ALIGN_TOOLBOXLEFT;
			break;

		case SFX_OBJECTBAR_NAVIGATION:
		case SFX_OBJECTBAR_OPTIONS:
			eAlign = SFX_ALIGN_TOOLBOXBOTTOM;
			break;

		default:
			eAlign = SFX_ALIGN_TOOLBOXTOP;
			break;
	}

	SfxToolBoxCfgItem_Impl* pItem = new SfxToolBoxCfgItem_Impl( nPos, bVis,
			bFloating, GetToolBoxPositionName( nPos ), eAlign );

	// USHORT_MAX bedeutet: soviele FloatingLines wie Eintr"age
	if ( nPos == SFX_OBJECTBAR_COMMONTASK )
	{
		pItem->nFloatingLines = USHRT_MAX;
		pItem->eButtonType = BUTTON_TEXT;
        pItem->aFloatingPos = SFX_APP()->GetTopWindow()->OutputToScreenPixel( Point( 60, 100 ) );
	}

	pPosArr->Append( pItem );
	pItem->nWinBits = GetWinBits( nPos );
	return pItem;
}

void SfxToolBoxConfig::Configure_Impl( SfxToolBoxManager *pMgr )
{
	SfxToolbox& rBox = (SfxToolbox&) pMgr->GetToolBox();
	USHORT nPos = rBox.GetPosition_Impl();

	SfxToolBoxCfgItem_Impl *pItem=0;
	for (USHORT n=0; n<pPosArr->Count(); n++)
	{
		if ((*pPosArr)[n]->nPos == nPos)
		{
			pItem = (*pPosArr)[n];
			break;
		}
	}

	if (!pItem) 					// ggf. neuen Eintrag erzeugen
		pItem = (SfxToolBoxCfgItem_Impl*) GetCfgItem_Impl( nPos );

	pItem->aFloatingPos = rBox.GetFloatingPos();
	if ( pItem->nFloatingLines < rBox.GetItemCount() )
		pItem->nFloatingLines = rBox.GetFloatingLines();
	else
		pItem->nFloatingLines = USHRT_MAX;
	pItem->nLines = rBox.GetLineCount();
	pItem->eAlign = rBox.GetAlignment();
	pItem->bFloating = pMgr->GetToolBox().IsFloatingMode();

	SetDefault(FALSE);
}

WinBits SfxToolBoxConfig::GetWinBits( USHORT nPos )
{
	ULONG nStyles = WB_LINESPACING | WB_BORDER | WB_SCROLL | WB_MOVEABLE | WB_3DLOOK;
	switch ( nPos )
	{
		case SFX_OBJECTBAR_RECORDING :
			break;

		case SFX_OBJECTBAR_FULLSCREEN :
			nStyles |= ( WB_DOCKABLE | WB_SIZEABLE );
			break;

		case SFX_OBJECTBAR_COMMONTASK:
			nStyles |= ( WB_CLOSEABLE | WB_DOCKABLE | WB_SIZEABLE );
			break;

		default:
			nStyles |= ( WB_CLOSEABLE | WB_DOCKABLE | WB_SIZEABLE );
			break;
	}

	return nStyles;
}

BOOL SfxToolBoxConfig::GetObjectbarVisibilityDefault( USHORT nId )
{
/*	if ( SFX_APP()->Get_Impl()->bBean && nId == SFX_OBJECTBAR_APPLICATION )
		return FALSE;
	else */ if ( nId == SFX_OBJECTBAR_OPTIONS )
		return FALSE;
	else
		return TRUE;
}

SfxChildAlignment SfxToolBoxConfig::GetAlignment( USHORT nPos ) const
{
	for (USHORT n=0; n<pPosArr->Count(); n++)
	{
		SfxToolBoxCfgItem_Impl* pItem = (*pPosArr)[n];
		if ( pItem->nPos == nPos )
			return pItem->bFloating ? SFX_ALIGN_NOALIGNMENT : pItem->eAlign;
	}

	SfxChildAlignment eAlign;
	switch ( nPos )
	{
		case SFX_OBJECTBAR_APPLICATION :
			eAlign = SFX_ALIGN_HIGHESTTOP;
			break;

		case SFX_OBJECTBAR_OBJECT :
		case SFX_OBJECTBAR_MACRO :
			eAlign = SFX_ALIGN_TOOLBOXTOP;
			break;

		case SFX_OBJECTBAR_TOOLS :
			eAlign = SFX_ALIGN_TOOLBOXLEFT;
			break;

		case SFX_OBJECTBAR_NAVIGATION:
		case SFX_OBJECTBAR_OPTIONS:
			eAlign = SFX_ALIGN_TOOLBOXBOTTOM;
			break;

		case SFX_OBJECTBAR_COMMONTASK:
		case SFX_OBJECTBAR_FULLSCREEN:
		case SFX_OBJECTBAR_RECORDING:
			eAlign = SFX_ALIGN_NOALIGNMENT;
			break;

		default:
			eAlign = SFX_ALIGN_TOOLBOXTOP;
			break;
	}
	return eAlign;
}


