/*************************************************************************
 *
 *  $RCSfile: tbxmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string> // HACK: prevent conflict between STLPORT and Workshop headers

#include <vcl/system.hxx>
#include <svtools/libcall.hxx>

#pragma hdrstop

#include "tbxmgr.hxx"
#include "dispatch.hxx"
#include "tbxctrl.hxx"
#include "resmgr.hxx"
#include "app.hxx"
#include "request.hxx"
#include "sfxtypes.hxx"
#include "bindings.hxx"
#include "toolbox.hrc"
#include "imgmgr.hxx"
#include "viewfrm.hxx"
#include "sfxresid.hxx"
#include "msgpool.hxx"
#include "macrconf.hxx"
#include "statcach.hxx"
#include "workwin.hxx"
#include "ipfrm.hxx"
#include "ipenv.hxx"
#include "tbxchild.hxx"
#include "tbxconf.hxx"
#include "arrdecl.hxx"

static const USHORT nVersion = 13;

//====================================================================

DECL_PTRARRAY(SfxToolBoxControlArr_Impl, SfxToolBoxControl*, 8, 8 );

//---------------------------------------------------------------------------
// Da SV den Customize-Handler nur an der Source-ToolBox aufruft, SFX aber
// auch der TargetBox etwas ausf"uhren mu\s (z.B. Controllererzeugung), stellt
// diese Klasse den public-Zugriff auf die protected-Methode Customize her.

class TbAccess_Impl : public SfxToolbox
{
public :

	TbAccess_Impl ( SfxToolBoxManager &rMgr, Window *pParent, WinBits nBits )
	  : SfxToolbox(rMgr,pParent,nBits)
	{}

	inline void CustomizeToolBox (const ToolBoxCustomizeEvent& rCEvt)
		{ Customize(rCEvt); }
};

//====================================================================

void SfxToolBoxManager::Construct()
{
	DBG_MEMTEST();

	// set the handlers
	pBox->SetSelectHdl( LINK(this, SfxToolBoxManager, Select) );
	pBox->SetActivateHdl( LINK(this, SfxToolBoxManager, Activate) );
	pBox->SetDeactivateHdl( LINK(this, SfxToolBoxManager, Deactivate) );

	// Nach dem Umkonfigurieren m"ussen die Hilfetexte neu geholt werden
	((SfxToolbox*)pBox)->bHelpInitialized = FALSE;
}

//--------------------------------------------------------------------

void SfxToolBoxManager::Align()
{
	DBG_MEMTEST();

	if ( pBox->IsFloatingMode() )
		return;

	Size aOldSize = pBox->GetSizePixel();
	Size aSize = pBox->CalcWindowSizePixel();

	if (aSize.Width() && aSize.Width() != pBox->GetSizePixel().Width() ||
		aSize.Height() && aSize.Height() != pBox->GetSizePixel().Height())
		pBox->SetPosSizePixel( pBox->GetPosPixel(), aSize );
}


//--------------------------------------------------------------------

// ctor f"ur solche ToolboxManager, die keine Objectbars sind

SfxToolBoxManager::SfxToolBoxManager( Window* _pParent,
									  SfxBindings &rBindings,
									  const ResId& rResId  ) :
	SfxConfigItem(rResId.GetId()),
	pParent(_pParent),
	pBox(0),
	pControls(0),
	eSymbolSet(SFX_SYMBOLS_SMALL),
	nIteratorPos(0),
	pResMgr(rResId.GetResMgr()),
	pBindings(&rBindings),
	pFrame(0),
	pIFace(0)
{
	DBG_MEMTEST();

	// Dieser ctor ist f"ur nicht konfigurierbare Toolboxen; alle Daten
	// werden aus der Resource geladen und k"onnen nicht ver"andert werden
	SetInternal( TRUE );
}


SfxToolBoxManager::SfxToolBoxManager( Window* _pParent,
									  SfxBindings &rBindings,
									  const ResId& rResId,
									  SfxInterface *pInterFace,
									  USHORT nPos, SfxShell* pTask )
/*
	Beschreibung
	ctor f"ur Objectbars; diese merken sich das Modul, aus dem sie erzeugt
	wurden.
*/

	: SfxConfigItem(rResId.GetId(), pTask ),
	pParent(_pParent),
	pBox(0),
	pControls(0),
	eSymbolSet(SFX_SYMBOLS_SMALL),
	nIteratorPos(0),
	pResMgr(rResId.GetResMgr()),
	pBindings(&rBindings),
	pFrame(0),
	pIFace(pInterFace)
{
	DBG_MEMTEST();

	if ( nPos != 0xFFFF )
	{
		const SfxToolBoxCfgItem_Impl *pItem = SfxToolBoxConfig::GetOrCreate()->GetCfgItem_Impl( nPos );

		// Neue leere ToolBox anlegen; wenn pItem->bCustomize == FALSE ist,
		// kann zwar die Config, nicht aber der Inhalt ge"andert werden
		SfxImageManager *pImgMgr = SFX_IMAGEMANAGER();
		pBox = new SfxToolbox( *this, pParent, pItem->nWinBits );
		((SfxToolbox*)pBox)->nPos = pItem->nPos;
		pBox->SetButtonType( pItem->eButtonType );
		pBox->SetOutStyle( pImgMgr->GetOutStyle() );
		pBox->EnableCustomize( pItem->bCustomize );
		SetInternal( !pItem->bCustomize );
		pBox->SetText( pItem->aName );

		SFX_IMAGEMANAGER()->RegisterToolBox(pBox, SFX_TOOLBOX_CHANGEOUTSTYLE );

		// Toolbox aus der Config initialisieren
		pBox->SetFloatingPos( pItem->aFloatingPos );
		pBox->SetLineCount( pItem->nLines );
		if ( pItem->bFloating )
			pBox->SetFloatingMode( TRUE );
		pBox->SetFloatingLines( pItem->nFloatingLines );
		((SfxToolbox*)pBox)->SetToolBoxAlignment_Impl( pItem->eAlign );

		String aStr;
		if ( GetNextToolBox_Impl( &aStr ) )
			pBox->SetNextToolBox( aStr );
		else
			pBox->SetNextToolBox( String() );
	}
	else
	{
		SetInternal( TRUE );
	}

	if ( pBox )
		pBox->SetUniqueId( rResId.GetId() );
}

//-------------------------------------------------------------------------

SfxToolBoxManager::SfxToolBoxManager( const ResId& rResId,
		SfxInterface *pInterFace,
		const String& rName, SfxToolBoxManager *pTbx, SfxShell *pTask)
	: SfxConfigItem(rResId.GetId(), pTask ),
	pParent(pTbx->pParent),
	pBox(pTbx->pBox),
	pControls(pTbx->pControls),
	eSymbolSet(pTbx->eSymbolSet),
	nIteratorPos(0),
	pResMgr(rResId.GetResMgr()),
	pBindings(pTbx->pBindings),
	pFrame(0),
	pIFace(pInterFace)
/*
	Beschreibung
	copy-ctor f"ur Objectbars; diese merken sich das Modul, aus dem sie erzeugt
	wurden.
*/

{
	DBG_MEMTEST();

	pTbx->StoreConfig();

	if (((SfxToolbox*)pBox)->bActivated)
	{
		// Wenn diese ToolBox aktiv ist, stehen wir noch mitten im SV.
		// Daher d"urfen ihre Items nicht angetastet werden.

		pTbx->Hide();
		pBox->Disable();
		pBox = 0;
		pControls = 0;

		// Alte ToolBox zum Deleten vormerken
		SfxToolbox& rTbx = *((SfxToolbox*)pTbx->pBox);
		rTbx.bDeleteToolBox = TRUE;
        GetpApp()->PostUserEvent( LINK(&rTbx, SfxToolbox, Delete), (void*)0 );

		// Neue leere ToolBox anlegen
		SfxImageManager *pImgMgr = SFX_IMAGEMANAGER();

		const SfxToolBoxCfgItem_Impl *pItem = SfxToolBoxConfig::GetOrCreate()->GetCfgItem_Impl( rTbx.nPos );

		pBox = new SfxToolbox( *this, pParent, pItem->nWinBits );
		((SfxToolbox*)pBox)->nPos = rTbx.nPos;
		pBox->SetButtonType( pItem->eButtonType );
		pBox->SetOutStyle( pImgMgr->GetOutStyle() );
		pBox->EnableCustomize( rTbx.IsCustomize() );
		pBox->SetText( rName );

		// Toolbox aus der Config initialisieren
		pBox->SetFloatingLines( rTbx.GetFloatingLines() );
		pBox->SetFloatingPos( rTbx.GetFloatingPos() );
		pBox->SetLineCount( rTbx.GetLineCount() );
		if ( rTbx.IsFloatingMode() )
			pBox->SetFloatingMode( TRUE );
		((SfxToolbox*)pBox)->SetToolBoxAlignment_Impl( rTbx.eAlignment );

		Initialize();
	}
	else
	{
		// Wenn diese ToolBox nicht aktiv ist, k"onnen ihre Items ausgetauscht
		// werden.
		((SfxToolbox*)pBox)->pMgr = this;
		((SfxToolbox*) pBox)->SetConstructed(FALSE);

		pBox->SetText( rName );
		pBox->SetUpdateMode(FALSE);
		Initialize();
		pBox->SetUpdateMode(TRUE);

		pTbx->pBox = 0;
		pTbx->pControls=0;
		delete pTbx;
	}

	String aStr;
	if ( GetNextToolBox_Impl( &aStr ) )
		pBox->SetNextToolBox( aStr );
	else
		pBox->SetNextToolBox( String() );

	pBox->SetUniqueId( rResId.GetId() );
}


//--------------------------------------------------------------------

// stores the configured toolbox to a stream

BOOL SfxToolBoxManager::Store( SvStream& rStream )
{
	DBG_MEMTEST();

	USHORT nCount = pBox->GetItemCount();
	if (((SfxToolbox*)pBox)->IsEmpty())
		nCount = 0;

	rStream << (USHORT) nVersion
			<< nCount;

	USHORT nPos;
	for ( nPos = 0; nPos < nCount; ++nPos )
	{
		const USHORT nId = pBox->GetItemId(nPos);
		BOOL bUserDef	 = SFX_IMAGEMANAGER()->IsUserDef_Impl(nId);

		rStream << (USHORT) pBox->GetItemType(nPos)
				<< nId;
		rStream.WriteByteString( ( ((SfxToolbox*)pBox)->HasItemText_Impl( nId ) ? pBox->GetItemText(nId) : String() ), RTL_TEXTENCODING_UTF8);
		rStream	<< (USHORT) bUserDef
				<< pBox->GetItemBits( nId )
				<< (USHORT) pBox->IsItemVisible( nId );

		if (bUserDef)
			rStream << SFX_IMAGEMANAGER()->GetUserDefBitmap_Impl(nId);

		if ( SfxMacroConfig::IsMacroSlot( nId ) )
			rStream << *(SFX_APP()->GetMacroConfig()->GetMacroInfo(nId));
	}

	for ( nPos = 0; nPos < nCount; ++nPos )
	{
		const USHORT nId = pBox->GetItemId(nPos);

		if (pBox->GetItemWindow(nId))
			rStream << (USHORT) pBox->GetItemWindow(nId)->GetSizePixel().Width();
		else
			rStream << (USHORT) 0;
	}

	LanguageType eLangType = Application::GetAppInternational().GetLanguage();
	if (eLangType == LANGUAGE_SYSTEM)
		eLangType = System::GetLanguage();
	rStream << (USHORT) eLangType;
	return TRUE;
}


//--------------------------------------------------------------------

int SfxToolBoxManager::Load(SvStream& rStream)
{
	DBG_MEMTEST();

	USHORT nFileVersion, nLanguage;
	rStream >> nFileVersion;
	if ( nFileVersion < 9 /*|| nFileVersion > nVersion */)
			return SfxConfigItem::WARNING_VERSION;

	DBG_ASSERT( pBox, "Load bei nicht konfigurierbarer Toolbox ?" );
	SFX_IMAGEMANAGER()->ReleaseToolBox( pBox );

	SfxImageManager *pImgMgr = SFX_IMAGEMANAGER();
	if ( pBox->GetItemCount() )
	{
		// Toolbox soll umkonfiguriert werden
		// Controller, ItemWindows und Items l"oschen
		Clear();
		pBox->Clear();
		((SfxToolbox*)pBox)->ClearItemText_Impl();
	}

	// Alle Items einlesen
	USHORT nCount;
	rStream >> nCount;

	SfxSlotPool& rSlotPool = pIFace && pIFace->GetModule() ? *pIFace->GetModule()->GetSlotPool() : SFX_APP()->GetSlotPool();
	USHORT nPos;
	for ( nPos = 0; nPos < nCount; ++nPos )
	{
		USHORT nItemType, nId, nUserDef, nVisible=1;
		String aItemText;
		Bitmap aBmp;
		ToolBoxItemBits nItemBits;

		rStream >> nItemType >> nId;
		rStream.ReadByteString(aItemText, RTL_TEXTENCODING_UTF8);
		rStream >> nUserDef >> nItemBits;
		if (nFileVersion>12)
			rStream >> nVisible;

		if ( nUserDef && nFileVersion > 9 )
			rStream >> aBmp;

		if ( SfxMacroConfig::IsMacroSlot( nId ) )
		{
			// Die SlotId von Macros ist nicht fest; wir besorgen uns eine
			SfxMacroInfo aInfo;
			rStream >> aInfo;
			nId = aInfo.GetSlotId();
		}
		else
		{
			// Keine Itemtexte aus "alteren Versionen
			if ( nFileVersion<12 )
				aItemText.Erase();
		}

		if ( nUserDef && nFileVersion > 9 && !pImgMgr->IsUserDef_Impl(nId) )
		{
			// Wenn es f"ur die SlotId noch keine Bitmap am Imagemanager
			// gibt, wird die eingelesene Bitmap dort angmeldet.
			// So wird beim Konfigurieren einer Toolbox auch gleichzeitig
			// der ImageManager umkonfiguriert !
			// Diese Toolbox wird am ImageManager kurzfristig abgemeldet,
			// damit ReplaceImage nicht versucht, hier was auszutauschen
			pImgMgr->ReplaceImage(nId, &aBmp);
		}

		// ToolBoxItem einf"ugen
		switch ( (ToolBoxItemType) nItemType )
		{
			case TOOLBOXITEM_BUTTON:
			{
				pBox->InsertItem( nId, aItemText, nItemBits );
				break;
			}

			case TOOLBOXITEM_SEPARATOR:
				pBox->InsertSeparator();
				break;

			case TOOLBOXITEM_SPACE:
				pBox->InsertSpace();
				break;

			case TOOLBOXITEM_BREAK:
				pBox->InsertBreak();
				break;

			default:
				DBG_ERROR( "invalid item-type in toolbox config" );
		}

		if (!nVisible)
			pBox->HideItem( nId );
	}

	SFX_IMAGEMANAGER()->RegisterToolBoxManager( this, 0xFFFF );

	// Alle Images vom ImageManager holen
	pImgMgr->SetImages( *pBox, pIFace ? pIFace->GetModule() : 0 );

	// Handler setzen
	Construct();

	// Controller und ItemWindows anlegen
	CreateFromSVToolBox();

	// ItemWindowbreiten einlesen
	USHORT nWidth=0;
	for ( nPos = 0; nPos<nCount; ++nPos )
	{
		const USHORT nId = pBox->GetItemId(nPos);
		rStream >> nWidth;
		if ( nWidth && pBox->GetItemWindow(nId) )
		{
			// Falls der Controller nur in einem anderen Context ein Itemwindow
			// hat, kann es auch NULL sein !
			Size aSize = pBox->GetItemWindow(nId)->GetSizePixel();
			aSize.Width() = nWidth;
			pBox->GetItemWindow(nId)->SetSizePixel(aSize);
		}
	}

	if ( nFileVersion > 10 )
	{
		rStream >> nLanguage;
		LanguageType eLangType = Application::GetAppInternational().GetLanguage();
		if (eLangType == LANGUAGE_SYSTEM)
			eLangType = System::GetLanguage();

		if ( eLangType != (LanguageType) nLanguage )
		{
			// Wenn die Text f"ur eine andere Sprache sind, Texte aus dem
			// SlotPool verwenden.
			SfxSlotPool& rSlotPool = pIFace && pIFace->GetModule() ? *pIFace->GetModule()->GetSlotPool() : SFX_APP()->GetSlotPool();
			((SfxToolbox*)pBox)->ClearItemText_Impl();
			for ( USHORT n=0; n<pBox->GetItemCount(); n++ )
			{
				USHORT nId = pBox->GetItemId(n);
				if ( pBox->GetItemType(n) == TOOLBOXITEM_BUTTON && !SfxMacroConfig::IsMacroSlot( nId ) )
					pBox->SetItemText( nId, rSlotPool.GetSlotName_Impl(nId) );
			}
		}
	}

	// Gr"o\se berechnen
	Align();
	((SfxToolbox*) pBox)->SetConstructed(TRUE);

	// StatusUpdate anwerfen
	Activate(0);
	return SfxConfigItem::ERR_OK;
}


//--------------------------------------------------------------------
typedef ULONG (__LOADONCALLAPI *HelpIdFunc) ( const String& );

void SfxToolBoxManager::UseDefault()
{
	DBG_MEMTEST();

	SfxImageManager *pImgMgr = SFX_IMAGEMANAGER();
	if ( pBox && pBox->GetItemCount() )
	{
		// Toolbox soll umkonfiguriert werden
		// Controller, ItemWindows und Items l"oschen
		Clear();
		pBox->Clear();
		((SfxToolbox*)pBox)->ClearItemText_Impl();
	}

	// ResId f"ur die ToolBox zusammenbasteln
	// Wenn es keine passende Resource gibt, wird eine leere Toolbox erzeugt
	// Dies ist der Fall bei UserDefined Toolboxen.
	ResId aResId(GetType());
	aResId.SetRT(RSC_TOOLBOX);
	aResId.SetResMgr(pResMgr);

	if ( Resource::GetResManager()->IsAvailable(aResId) )
	{
		BOOL bOwnImageList = FALSE;
		if ( pBox )
		{
			SFX_IMAGEMANAGER()->ReleaseToolBox( pBox );

			// Konfigurierbare Toolboxen sind SFX-Objectbars; sie laden nur
			// ihren Inhalt aus der Resource, der Rest kommt aus der
			// SfxToolboxConfig
			ToolBox *pTb = new ToolBox ( pParent, aResId );
			if ( pTb->GetImageList().GetImageCount() )
				bOwnImageList = TRUE;

			pBox->EnableCustomize( !bOwnImageList );

			pBox->CopyItems( *pTb );
			pBox->SetHelpId( pTb->GetHelpId() );
			pBox->SetHelpText( String() );
			delete pTb;
		}
		else
		{
			// Nicht konfigurierbare Toolboxen werden komplett aus der
			// Resource geladen
			pBox = new SfxToolbox( *this, pParent, aResId );
			pBox->SetUniqueId( aResId.GetId() );
			if ( pBox->GetImageList().GetImageCount() )
				bOwnImageList = TRUE;

			pBox->EnableCustomize( FALSE );
			SfxImageManager *pImgMgr = SFX_IMAGEMANAGER();
			pBox->SetButtonType( BUTTON_SYMBOL );
			pBox->SetOutStyle( pImgMgr->GetOutStyle() );
		}

		if ( !bOwnImageList )
		{
			// Images holen, wenn noch nicht vorhanden
			pImgMgr->SetImages( *pBox, pIFace ? pIFace->GetModule() : 0 );

			if ( Internal() )
			{
				// Bei Popupwindow-Toolboxen k"onnen Symbole fehlen, wenn der
				// Zeitpunkt der Erzeugung ung"unstig ist
                SfxModuleArr_Impl& rArr = SfxModule::GetModules_Impl();
				USHORT nArr = 0;
				USHORT nModCount = rArr.Count();
				USHORT nSearchId = 0;
				do
				{
					nSearchId = 0;
					for ( USHORT nPos=0; nPos<pBox->GetItemCount(); nPos++ )
					{
						USHORT nId = pBox->GetItemId( nPos );
						if ( !pBox->GetItemImage( nId ).GetSizePixel().Width() )
						{
							nSearchId = nId;
							break;
						}
					}

					if ( nSearchId )
						pImgMgr->SetImages( *pBox, rArr[nArr++] );
				}
				while ( nSearchId && nArr < nModCount );
			}

			SFX_IMAGEMANAGER()->RegisterToolBoxManager( this, 0xFFFF );
		}
		else
			SFX_IMAGEMANAGER()->RegisterToolBox( pBox, SFX_TOOLBOX_CHANGEOUTSTYLE );
	}
	else if ( !pBox )
		DBG_ERROR( "Toolbox-Resource fehlt!" );

	// Handler setzen
	Construct();

	// Controller und ItemWindows erzeugen
	CreateFromSVToolBox();

	// Gr"o\se berechnen
	Align();
	((SfxToolbox*) pBox)->SetConstructed(TRUE);

	// StatusUpdate anwerfen
	Activate(0);
}

void SfxToolBoxManager::RefreshImages_Impl()
{
	SFX_IMAGEMANAGER()->SetImages( *pBox, pIFace ? pIFace->GetModule() : 0 );
	USHORT nCount = pBox->GetItemCount();
	for ( USHORT nPos=0; nPos<nCount; nPos++ )
	{
		USHORT nId = pBox->GetItemId( nPos );
		if ( pBox->GetItemType(nPos) == TOOLBOXITEM_BUTTON )
		{
			SfxStateCache *pCache = pBindings->GetStateCache( nId );
			if ( pCache )
				pCache->SetCachedState();
		}
	}
}

//--------------------------------------------------------------------

SfxToolBoxManager::~SfxToolBoxManager()
{
	DBG_MEMTEST();

	// unregister bindings
	Clear();
	if (pBox)
	{
		Hide();
		SFX_IMAGEMANAGER()->ReleaseToolBox(pBox);
		delete pBox;
	}
}


//-------------------------------------------------------------------------

void SfxToolBoxManager::Clear()
{
	if (pControls)
	{
		pBindings->ENTERREGISTRATIONS();
		for ( USHORT n = 0; n < pControls->Count(); ++n )
		{
			SfxToolBoxControl *pControl = pControls->GetObject(n);
			USHORT nId = pControl->GetId();
			Window *pItemWin = pBox->GetItemWindow(nId);
			if ( pItemWin )
			{
				pItemWin->Hide();
				delete pItemWin;
				pBox->SetItemWindow( nId, 0 );
			}
			delete pControl;
		}
		pControls->Clear();
		DELETEZ(pControls);
		pBindings->LEAVEREGISTRATIONS();
	}
}


//--------------------------------------------------------------------

SfxToolBoxControl* SfxToolBoxManager::FindControl_Impl( USHORT nId ) const
{
	for ( USHORT nPos = 0; nPos < pControls->Count(); ++nPos )
		if ( pControls->GetObject(nPos)->GetId() == nId )
			return pControls->GetObject(nPos);
	return 0;
}


//--------------------------------------------------------------------

void SfxToolBoxManager::CreateFromSVToolBox()
{
	DBG_MEMTEST();

	if (pBox->GetItemCount() == 0)
	{
										// SV austricksen, damit die Groesse
										// stimmt
		((SfxToolbox*)pBox)->SetEmpty(TRUE);
		USHORT nId = SID_NEWDOC;
		pBox->InsertItem( nId, String());
		pBox->SetItemImage(nId, SFX_IMAGEMANAGER()->GetImage(nId, pIFace ? pIFace->GetModule() : 0));
		pBox->HideItem(nId);
	}

	// reflect the StarView toolbox in a bindings table
	if (!pControls)
		pControls = new SfxToolBoxControlArr_Impl;
	SfxSlotPool& rSlotPool = pIFace && pIFace->GetModule() ? *pIFace->GetModule()->GetSlotPool() : SFX_APP()->GetSlotPool();
	((SfxToolbox*)pBox)->ClearItemText_Impl();
	pBindings->ENTERREGISTRATIONS();
	for ( USHORT nPos = 0; nPos < pBox->GetItemCount(); ++nPos )
	{
		if ( pBox->GetItemType(nPos) == TOOLBOXITEM_BUTTON )
		{
			// register binding
			USHORT nId = pBox->GetItemId(nPos);

#ifdef DBG_UTIL
			const SfxSlot *pSlot = rSlotPool.GetSlot( nId );
			if ( pSlot && !pSlot->IsMode( SFX_SLOT_TOOLBOXCONFIG ) )
			{
				ByteString aStr( "Slot " );
				aStr += ByteString::CreateFromInt32( nId );
				aStr += ByteString( " : ToolboxConfig fehlt!" );
				DBG_ERROR( aStr.GetBuffer() );
			}
#endif
			pBox->SetHelpId( nId, (ULONG) nId );
			if ( !pBox->GetItemText(nId).Len() )
				pBox->SetItemText( nId, rSlotPool.GetSlotName_Impl( nId ) );
			else
				((SfxToolbox*)pBox)->SetItemText_Impl( nId, TRUE );

			SfxToolBoxControl *pControl;
			if ( pBox->GetItemCommand( nId ).Len() )
			{
				pBox->EnableCustomize( FALSE );
				pControl = SfxToolBoxControl::CreateControl(
					pBox->GetItemCommand( nId ), nId, pBox, *pBindings );
			}
			else
				pControl = SfxToolBoxControl::CreateControl(
					nId, pBox, *pBindings, pIFace ? pIFace->GetModule() : 0 );
			pControls->Append( pControl );
			Window *pItemWin = pControl->CreateItemWindow(pBox);
			if ( pItemWin )
				pBox->SetItemWindow( nId, pItemWin );
		}
	}
	pBindings->LEAVEREGISTRATIONS();
}


//--------------------------------------------------------------------

// updates all state-information

IMPL_LINK( SfxToolBoxManager, Activate, ToolBox *, pSVBox )
{
	DBG_MEMTEST();

	SfxToolbox *pTbx = (SfxToolbox*) pBox;
	SfxBindings *pBindinx = pBindings;

	// Es kann passieren, da\s diese Toolbox im Verlauf des jetzt erzwungenen
	// Updates ausgetauscht werden m"u\ste. Daher merken wir uns, da\s wir auf
	// keinen Fall ausgetauscht oder gel"oscht werden wollen.
	if ( pSVBox )
		pTbx->bActivated = TRUE;

	if ( !pBindinx->GetDispatcher_Impl()->IsLocked() && !pTbx->bReconfigure)
	{
		pBindinx->GetDispatcher_Impl()->Flush();
		if (!pTbx->bDeleteToolBox)			// noch nicht versucht zu deleten
		{
			// update all states-information of each item in this menu
			for ( short nPos = pTbx->GetItemCount(); nPos >= 0; --nPos )
			{	USHORT nId = pTbx->GetItemId(nPos);
				if ( nId != 0 )
					pBindinx->Update( nId );
			}
		}
	}

	// nur bei Call von SV kommt auch ein Deactivate
	if ( pSVBox )
		pBindinx->ENTERREGISTRATIONS();
	return 0;
}

//--------------------------------------------------------------------

// removes the context

IMPL_LINK( SfxToolBoxManager, Deactivate, ToolBox *, pToolBox )
{
	DBG_MEMTEST();
	((SfxToolbox*)pBox)->bActivated = FALSE;
	pBindings->LEAVEREGISTRATIONS();
	return 0;
}


//--------------------------------------------------------------------

// executes the selected function

IMPL_LINK( SfxToolBoxManager, Select, ToolBox *, pBox )
{
	DBG_MEMTEST();

	// Wenn die ToolBox zum Deleten vorgemerkt ist, soll nichts mehr
	// selektiert werden koennen.
	if ( ((SfxToolbox*)pBox)->bDeleteToolBox )
		return 0;

	SfxToolBoxControl *pControl = FindControl_Impl( pBox->GetCurItemId() );
	if ( pControl )
	{
		SfxToolbox *pTbx = (SfxToolbox*) pBox;
		// Auch beim Select darf nicht zwischendurch die ToolBox oder das
		// selektierte Item verschwinden; da Select erst nach Deactivate kommt,
		// mu\s die ToolBox noch einmal "geblockt" werden.
		pTbx->bActivated = 2;
		pControl->Select( pBox->GetModifier() );
		pTbx->bActivated = FALSE;
	}
	return 0;
}


//--------------------------------------------------------------------

// resets the item iterator, FALSE if none

BOOL SfxToolBoxManager::FirstItem()
{
	nIteratorPos = 0;
	return nIteratorPos < pBox->GetItemCount();
}


//--------------------------------------------------------------------

// skips to the next item, FALSE if no more

BOOL SfxToolBoxManager::NextItem()
{
	++nIteratorPos;
	return nIteratorPos < pBox->GetItemCount();
}


//--------------------------------------------------------------------

// TRUE if current item is a binding

BOOL SfxToolBoxManager::IsBinding() const
{
	return pBox->GetItemType(nIteratorPos) == TOOLBOXITEM_BUTTON;
}


//--------------------------------------------------------------------

// TRUE if current item is a separator

BOOL SfxToolBoxManager::IsSeparator() const
{
	return pBox->GetItemType( pBox->GetItemId(nIteratorPos) ) ==
		   TOOLBOXITEM_SEPARATOR;
}


//--------------------------------------------------------------------

// id of current binding if IsBinding()

USHORT SfxToolBoxManager::GetId() const
{
	return pBox->GetItemId( nIteratorPos );
}


//--------------------------------------------------------------------

void SfxToolBoxManager::Customize( const ToolBoxCustomizeEvent& rCEvt )
{
	USHORT nSourcePos = pBox->GetItemPos( rCEvt.GetSourceId() );
	USHORT nTargetPos = rCEvt.GetTargetPos();

	// Ab SV303 : Customize-Handler wird auch beim Resizen der ItemWindows
	// gerufen
	if ( rCEvt.IsResized() )
	{
		// Merken, da\s umkonfiguriert wurde, die ItemWindowBreite wird beim
		// Abspeichern vom ItemWindow direkt abgeholt
		SetDefault(FALSE);
		return;
	}

	if ( rCEvt.GetTargetBox() == NULL )
	{
		// Es wurde ein Item entfernt
		USHORT nId = rCEvt.GetSourceId();
		SfxToolBoxControl *pControl=0;

		USHORT nPos;
		for ( nPos=0; nPos<pControls->Count(); ++nPos )
			if ( pControls->GetObject(nPos)->GetId() == nId )
		{
			pControl = pControls->GetObject(nPos);
			break;
		}

		if (pControl)
		{
			pBindings->ENTERREGISTRATIONS();

			Window *pItemWin = pBox->GetItemWindow(nId);
			if ( pItemWin )
			{
				pItemWin->Hide();
				delete pItemWin;
				pBox->SetItemWindow( nId, 0 );
			}

			delete pControl;
			pControls->Remove(nPos);

			pBindings->LEAVEREGISTRATIONS();
		}

		pBox->RemoveItem( nSourcePos );
	}
	else if ( rCEvt.GetTargetBox() == pBox )
	{
		// Ich bin das Target.
		// Achtung: diesen Aufruf generiert der SFX, nicht SV (s.u.) !!
		if (rCEvt.GetData() != 0)
		{
			if (((SfxToolbox*)pBox)->IsEmpty())
			{
				// Das DummyItem in leeren ToolBoxen entfernen
				pBox->RemoveItem(0);
				((SfxToolbox*)pBox)->SetEmpty(FALSE);
			}

			USHORT nId = rCEvt.GetSourceId();

			// Habe ich das Item noch nicht ?
			if (pBox->GetItemPos(nId) == TOOLBOX_ITEM_NOTFOUND)
			{
				// Der SFX steckt die Source-Toolbox in die User-Daten
				ToolBox *pSource = (ToolBox*) rCEvt.GetData();

				pBox->InsertItem( nId, SFX_IMAGEMANAGER()->GetAndLockImage_Impl( nId,
							pIFace ? pIFace->GetModule() : 0), String(), 0, nTargetPos );

				SetItemText( nId, pSource->GetItemText( nId ) );

				// Macros m"u\sen einen Slot anfordern
				if ( SfxMacroConfig::IsMacroSlot( nId ) )
					SFX_APP()->GetMacroConfig()->RegisterSlotId(nId);

				pBindings->ENTERREGISTRATIONS();

				SfxSlotPool &rSlotPool = SFX_SLOTPOOL();
				pBox->SetHelpText( nId, rSlotPool.GetSlotHelpText_Impl(nId) );

				SfxToolBoxControl *pControl =
					SfxToolBoxControl::CreateControl( nId, pBox, *pBindings, pIFace ? pIFace->GetModule() : 0 );
				pControls->Append( pControl );

				Window *pItemWin = pControl->CreateItemWindow(pBox);
				if ( pItemWin )
					pBox->SetItemWindow( nId, pItemWin );
				pBox->SetHelpId( nId, (ULONG) nId );

				pBindings->LEAVEREGISTRATIONS();
			}
			else if( !pBox->IsItemVisible( nId ) )
			{
				pBox->ShowItem( nId );
				pBox->MoveItem( nId, nTargetPos );
			}
		}
		else if ( nSourcePos+1 == nTargetPos )
		{
			if ( pBox->GetItemType( nSourcePos-1 ) == TOOLBOXITEM_BUTTON )
				pBox->InsertSeparator( nSourcePos );
		}
		else
		{
			if ( (nSourcePos == nTargetPos)  &&
				 (pBox->GetItemType( nSourcePos-1 ) == TOOLBOXITEM_SEPARATOR) )
				pBox->RemoveItem( nSourcePos-1 );
			else
				pBox->MoveItem( rCEvt.GetSourceId(), nTargetPos );
		}
	}
	else
	{
		// Eines meiner Items wird in eine andere ToolBox kopiert; also rufe
		// ich dort den CustomizeHandler auf
		ToolBox* pTarget = rCEvt.GetTargetBox();
		if ( pTarget )
		{
			USHORT nId = rCEvt.GetSourceId();

			// Der Zugriff auf eine protected method ...
			ToolBoxCustomizeEvent aEvnt (pTarget, nId, nTargetPos, pBox);
			((TbAccess_Impl*)pTarget)->CustomizeToolBox(aEvnt);
		}

		return;
	}

	if ( (pBox->GetItemType( nSourcePos ) == TOOLBOXITEM_SEPARATOR) &&
		 ((pBox->GetItemType( nSourcePos-1 ) == TOOLBOXITEM_SEPARATOR) ||
		  (pBox->GetItemType( nSourcePos+1 ) == TOOLBOXITEM_SEPARATOR) ||
		   pBox->GetItemCount() == 1 ) )
		pBox->RemoveItem( nSourcePos );

	if (pBox->GetItemCount() == 0)
	{
		// SV austricksen, damit die Gr"o\se stimmt
		((SfxToolbox*)pBox)->SetEmpty(TRUE);
		USHORT nId = SID_NEWDOC;
		pBox->InsertItem( nId, String());
		pBox->SetItemImage(nId, SFX_IMAGEMANAGER()->GetImage(nId, pIFace ? pIFace->GetModule() : 0));
		pBox->HideItem(nId);
	}

	SetDefault(FALSE);

	// Es k"onnte sein, da\s sich durch das neue Item die Gr"o\se der
	// ToolBox ge"andert hat
	SfxDispatcher* pDisp = GetBindings().GetDispatcher_Impl();
	if ( pDisp )
	{
		SfxViewFrame  *pFrame  = pDisp->GetFrame();
		SfxWorkWindow *pWorkWin = SFX_APP()->GetWorkWindow_Impl(pFrame);
		pWorkWin->ConfigChild_Impl
			( SFX_CHILDWIN_OBJECTBAR, SFX_ALIGNDOCKINGWINDOW, GetType() );
	}
}

//--------------------------------------------------------------------

void SfxToolBoxManager::UpdateControls_Impl()

/*	Interne Methode zum erneuten Updaten aller SfxToolBoxContoller in
	diesem SfxToolBoxManager. Z.B. nach Umschaltung der Button-Art.
*/

{
	for ( USHORT n = 0; n < pControls->Count(); ++n )
	{
		SfxToolBoxControl *pControl = pControls->GetObject(n);
		if ( pControl->IsBound() )
			pBindings->GetStateCache( pControl->GetId() )->SetCachedState();
	}
}

//--------------------------------------------------------------------

void SfxToolBoxManager::Show()
{
//	  if (pBox->IsFloatingMode() == FALSE)
		pBox->Show();
}


//--------------------------------------------------------------------

void SfxToolBoxManager::Hide()
{
//	  if (pBox->IsFloatingMode() == FALSE)
		pBox->Hide();
}


// ------------------------------------------------------------------------

String SfxToolBoxManager::GetName() const
{
	if (pBox)
		return pBox->GetText();

	ResId aResId(GetType());
	aResId.SetRT(RSC_STRING);
	aResId.SetResMgr(pResMgr);
	if (Resource::GetResManager()->IsAvailable(aResId))
		return String(aResId);
	else
		return String();
}

//-------------------------------------------------------------------------

BOOL SfxToolBoxManager::Reconfigure(SvStream& rStream, BOOL bDefault)
{
	SfxToolbox *pTbx = (SfxToolbox*) pBox;
	pTbx->bReconfigure = TRUE;
	BOOL bRet = SfxConfigItem::Reconfigure(rStream, bDefault);
	pTbx->bReconfigure = FALSE;
	return bRet;
}

void SfxToolBoxManager::Reconfigure_Impl( const SfxToolBoxCfgItem_Impl *pItem )
{
	BOOL bChanged = FALSE;
	SfxToolbox* pTbx = (SfxToolbox*) pBox;
	if ( pItem->eButtonType != pTbx->GetButtonType() )
	{
		bChanged = TRUE;
		pTbx->SetButtonType( pItem->eButtonType );
		if ( !pTbx->IsFloatingMode() )
		{
			Size aActSize( pTbx->GetSizePixel() );
			Size aSize( pTbx->CalcWindowSizePixel() );
			if ( pTbx->IsHorizontal() )
				aSize.Width() = aActSize.Width();
			else
				aSize.Height() = aActSize.Height();

			pTbx->SetSizePixel( aSize );
		}
	}

	if ( pTbx->GetFloatingPos() != pItem->aFloatingPos )
	{
		pTbx->SetFloatingPos( pItem->aFloatingPos );
		bChanged = TRUE;
	}
	if ( pTbx->GetLineCount() != pItem->nLines )
	{
		pTbx->SetLineCount( pItem->nLines );
		bChanged = TRUE;
	}
	if ( pTbx->GetFloatingLines() != pItem->nFloatingLines )
	{
		pTbx->SetFloatingLines( pItem->nFloatingLines );
		bChanged = TRUE;
	}
	if ( pTbx->GetAlignment() != pItem->eAlign )
	{
		pTbx->SetToolBoxAlignment_Impl( pItem->eAlign );
		bChanged = TRUE;
	}

	SfxDispatcher* pDisp = GetBindings().GetDispatcher_Impl();
	if ( pDisp )
	{
		SfxViewFrame  *pFrame  =  pDisp->GetFrame();
		SfxWorkWindow *pWorkWin = SFX_APP()->GetWorkWindow_Impl(pFrame);
		if ( pItem->bFloating != pBox->IsFloatingMode() )
		{
			pTbx->SetFloatingMode( pItem->bFloating );
			pWorkWin->ConfigChild_Impl
				( SFX_CHILDWIN_OBJECTBAR, SFX_TOGGLEFLOATMODE, GetType() );
		}
		else if ( bChanged )
		{
			pWorkWin->ConfigChild_Impl
				( SFX_CHILDWIN_OBJECTBAR, SFX_ALIGNDOCKINGWINDOW, GetType() );
		}
	}
}

//---------------------------------------------------------------------------
// ObjectBars werden niemals direkt "uber den dtor, sondern nur "uber diese
// Methode gel"oscht. Falls diese Toolbox gerade aktiv ist, stehen wir mitten
// im SV und m"u\sen daher asynchron l"oschen.


void SfxToolBoxManager::Delete()
{
	if (((SfxToolbox*)pBox)->bActivated)
	{
		((SfxToolbox*)pBox)->bDeleteToolBox = TRUE;
        GetpApp()->PostUserEvent(LINK(pBox, SfxToolbox, Delete), (void*)0);
	}
	else
		delete this;
}

//-------------------------------------------------------------------------
// Diese statische Methode erkennt an der "ubergebenen Id, ob es sich um eine
// benutzerdefinierte Toolbox handelt.


BOOL SfxToolBoxManager::IsUserDefToolBox_Impl(USHORT nId)
{
	if ( nId >= RID_SFX_TOOLBOX_USERSTART && nId <= RID_SFX_TOOLBOX_USEREND )
		return TRUE;
	else
		return FALSE;
}

//-------------------------------------------------------------------------
// Mit dieser statischen Methode kann eine f"ur die Lebensdauer der Toolbox
// g"ultige, eindeutige Id f"ur eine benutzerdefinierte ToolBox angefordert
// werden.


USHORT SfxToolBoxManager::GetUserDefToolBoxId_Impl()
{
	USHORT nRet=0;
	for ( USHORT nId=RID_SFX_TOOLBOX_USERSTART;
			nId <= RID_SFX_TOOLBOX_USEREND; nId++ )
	{
		for (SfxInterface *pIFace = SFX_SLOTPOOL().FirstInterface(); pIFace != 0;
				pIFace = SFX_SLOTPOOL().NextInterface())
		{
			// Nur Interfaces mit Namen d"urfen welche bekommen
			if ( pIFace->HasName())
			{
				if ( pIFace->HasObjectBar(nId) )
					break;
				else
					nRet = nId;
			}
		}

		// Hurra, gefunden !
		if ( nRet )
			break;
	}

	return nRet;
}

void SfxToolBoxManager::ReInitControllers_Impl()
{
	if ( IsDefault() )
		return;
	if (pControls)
	{
		pBindings->ENTERREGISTRATIONS();
		for ( USHORT n = 0; n < pControls->Count(); ++n )
		{
			SfxToolBoxControl *pControl = pControls->GetObject(n);
			if ( !pControl->IsBound() )
				continue;
			SfxToolBoxControl *pNew = pControl->ReInit_Impl();
			if ( pNew != pControl )
			{
				USHORT nId = pNew->GetId();

				Window *pNewItemWin = pNew->CreateItemWindow( pBox );

				Window *pItemWin = pBox->GetItemWindow( nId );
				if ( pItemWin )
				{
					pItemWin->Hide();
					delete pItemWin;
				}

				pBox->SetItemWindow( nId, pNewItemWin );

				pControls->GetObject(n) = pNew;
				delete pControl;
			}
		}
		pBindings->LEAVEREGISTRATIONS();
	}
}

void SfxToolBoxManager::NextToolBox()
{
	SfxDispatcher* pDisp = GetBindings().GetDispatcher_Impl();
	if ( pDisp )
		pDisp->ShowObjectBar( GetNextToolBox_Impl() );
}

USHORT SfxToolBoxManager::GetNextToolBox_Impl( String *pStr )
{
	SfxDispatcher* pDisp = GetBindings().GetDispatcher_Impl();
	if ( pDisp )
		return pDisp->GetNextToolBox_Impl( ((SfxToolbox*)pBox)->nPos, GetType(), pStr );
	else
		return 0;
}

void SfxToolBoxManager::SetItemText( USHORT nId, const String& rText )
{
	if ( SfxMacroConfig::IsMacroSlot( nId ) )
	{
		( (SfxToolbox*)pBox )->SetItemText_Impl( nId, TRUE );
	}
	else
	{
		SfxSlotPool& rSlotPool = pIFace && pIFace->GetModule()
			? *pIFace->GetModule()->GetSlotPool() : SFX_APP()->GetSlotPool();
		if ( rSlotPool.GetSlotName_Impl( nId ) != rText )
			( (SfxToolbox*)pBox )->SetItemText_Impl( nId, TRUE );
		else
			( (SfxToolbox*)pBox )->SetItemText_Impl( nId, FALSE );
	}

	pBox->SetItemText( nId, rText );
}


