/*************************************************************************
 *
 *  $RCSfile: framedlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/urihelper.hxx>

#pragma hdrstop

#include "framedlg.hxx"
#include "sfxresid.hxx"
#include "view.hrc"
#include "frmdescr.hxx"
#include "app.hxx"
#include "iodlg.hxx"
#include "viewsh.hxx"

SfxTabPage *CreateFramePropertiesPage_Impl( Window *pParent, const SfxItemSet& rSet )
{
	return new SfxFramePropertiesPage_Impl( pParent, rSet );
}

SfxTabPage *CreateFrameSizePage_Impl( Window *pParent, const SfxItemSet& rSet )
{
	return new SfxFrameSizePage_Impl( pParent, rSet );
}

SfxTabPage *CreateFrameSpacingPage_Impl( Window *pParent, const SfxItemSet& rSet )
{
	return new SfxFrameSpacingPage_Impl( pParent, rSet );
}

SfxFrameEditDialog_Impl::SfxFrameEditDialog_Impl( Window *pParent, const SfxItemSet *pSet )
	: SfxTabDialog( pParent, SfxResId(RID_FRAMEDLG), pSet )
{
	FreeResource();
	AddTabPage( TP_FRAMEPROPERTIES, CreateFramePropertiesPage_Impl, NULL );
	AddTabPage( TP_FRAMESIZE, CreateFrameSizePage_Impl, NULL );
//	AddTabPage( TP_FRAMESPACING, CreateFrameSpacingPage_Impl, NULL );
}

SfxFramePropertiesPage_Impl::SfxFramePropertiesPage_Impl( Window *pParent, const SfxItemSet& rSet )
	: SfxTabPage( pParent, SfxResId( TP_FRAMEPROPERTIES ), rSet )
	, aFTName ( this, ResId( FT_FRAMENAME ) )
	, aEDName ( this, ResId( ED_FRAMENAME ) )
	, aFTURL ( this, ResId( FT_URL ) )
	, aEDURL ( this, ResId( ED_URL ) )
	, aBTOpen ( this, ResId(BT_FILEOPEN ) )
	, aRBFrameBorderOn ( this, ResId( RB_FRMBORDER_ON ) )
	, aRBFrameBorderOff ( this, ResId( RB_FRMBORDER_OFF ) )
	, aRBFrameBorderInherit ( this, ResId( RB_FRMBORDER_INHERIT ) )
	, aGBFrameBorder( this, ResId( GB_BORDER ) )
	, aFTMarginWidth ( this, ResId( FT_MARGINWIDTH ) )
	, aNMMarginWidth ( this, ResId( NM_MARGINWIDTH ) )
	, aFTMarginHeight ( this, ResId( FT_MARGINHEIGHT ) )
	, aNMMarginHeight ( this, ResId( NM_MARGINHEIGHT ) )
	, aMarginGroupBox ( this, ResId( GB_MARGIN ) )
	, aCBMarginWidthDefault( this, ResId( CB_MARGINWIDTHDEFAULT ) )
	, aCBMarginHeightDefault( this, ResId( CB_MARGINHEIGHTDEFAULT ) )
	, aRBScrollingOn ( this, ResId( RB_SCROLLINGON ) )
	, aRBScrollingOff ( this, ResId( RB_SCROLLINGOFF ) )
	, aRBScrollingAuto ( this, ResId( RB_SCROLLINGAUTO ) )
	, aScrollingGroupBox ( this, ResId( GB_SCROLLING ) )
{
	FreeResource();

	Link aLink( STATIC_LINK( this, SfxFramePropertiesPage_Impl, CheckHdl ) );
//    aRBFrameBorderInherit.SetClickHdl( aLink );
//    aRBFrameBorderOn.SetClickHdl( aLink );
//    aRBFrameBorderOff.SetClickHdl( aLink );
	aCBMarginWidthDefault.SetClickHdl( aLink );
	aCBMarginHeightDefault.SetClickHdl( aLink );
	aBTOpen.SetClickHdl( STATIC_LINK( this, SfxFramePropertiesPage_Impl, OpenHdl ) );
}

void SfxFramePropertiesPage_Impl::Reset( const SfxItemSet& rSet )
{
	const SfxPoolItem* pItem;
	if( SFX_ITEM_SET != rSet.GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();
	SetUpdateMode( FALSE );

	// FrameName und URL
	aEDName.SetText( rProp.aName );
	aEDURL.SetText( rProp.aURL );

	// Margin
	if ( rProp.lMarginWidth == SIZE_NOT_SET )
	{
		aCBMarginWidthDefault.Check( TRUE );
		aNMMarginWidth.SetText( String::CreateFromInt32( DEFAULT_MARGIN_WIDTH )  );
		aFTMarginWidth.Enable( FALSE );
		aNMMarginWidth.Enable( FALSE );
	}
	else
		aNMMarginWidth.SetText( String::CreateFromInt32( rProp.lMarginWidth ) );

	if ( rProp.lMarginHeight == SIZE_NOT_SET )
	{
		aCBMarginHeightDefault.Check( TRUE );
		aNMMarginHeight.SetText( String::CreateFromInt32( DEFAULT_MARGIN_HEIGHT )  );
		aFTMarginHeight.Enable( FALSE );
		aNMMarginHeight.Enable( FALSE );
	}
	else
		aNMMarginHeight.SetText( String::CreateFromInt32( rProp.lMarginHeight ) );

	// ScrollingMode
	BOOL bScrollOn = FALSE;
	BOOL bScrollOff = FALSE;
	BOOL bScrollAuto = FALSE;
	switch( rProp.eScroll )
	{
		case ScrollingYes  :
			bScrollOn = TRUE;
			break;
		case ScrollingNo   :
			bScrollOff = TRUE;
			break;
		case ScrollingAuto :
			bScrollAuto = TRUE;
			break;
	}
	aRBScrollingOn.Check( bScrollOn );
	aRBScrollingOff.Check( bScrollOff );
	aRBScrollingAuto.Check( bScrollAuto );

	// Frame Border
	aRBFrameBorderOn.Check( rProp.bBorderSet && rProp.bHasBorder );
	aRBFrameBorderOff.Check( rProp.bBorderSet && !rProp.bHasBorder );

	// Vererbung
	aRBFrameBorderInherit.Check( !rProp.bBorderSet );

	SetUpdateMode( TRUE );
}

BOOL SfxFramePropertiesPage_Impl::FillItemSet( SfxItemSet& rSet)
{
	// Zuerst auf Veraenderungen testen
	const SfxPoolItem* pItem=NULL;
	const SfxItemSet *pSet = &GetItemSet();
	if ( GetTabDialog() )
		pSet = GetTabDialog()->GetExampleSet();
	if( SFX_ITEM_SET != pSet->GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return FALSE;;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

	BOOL bChange = FALSE;
	String aURL;
	if ( aEDURL.GetText().Len() )
        aURL = URIHelper::SmartRelToAbs( aEDURL.GetText() );
	ScrollingMode eScroll;
	if ( aRBScrollingOn.IsChecked() )
		eScroll = ScrollingYes;
	if ( aRBScrollingOff.IsChecked() )
		eScroll = ScrollingNo;
	if ( aRBScrollingAuto.IsChecked() )
		eScroll = ScrollingAuto;
	long lMarginWidth;
	if ( !aCBMarginWidthDefault.IsChecked() )
		lMarginWidth = (long) aNMMarginWidth.GetText().ToInt32();
	else
		lMarginWidth = SIZE_NOT_SET;

	long lMarginHeight;
	if ( !aCBMarginHeightDefault.IsChecked() )
		lMarginHeight = (long) aNMMarginHeight.GetText().ToInt32();
	else
		lMarginHeight = SIZE_NOT_SET;

	if ( aEDName.GetText() != rProp.aName )
		bChange = TRUE;
	else if ( aURL != rProp.aURL )
		bChange = TRUE;
	else if ( lMarginWidth != rProp.lMarginWidth )
		bChange = TRUE;
	else if ( lMarginHeight != rProp.lMarginHeight )
		bChange = TRUE;
	else if ( rProp.eScroll != eScroll )
		bChange = TRUE;
	else if ( rProp.bHasBorder != aRBFrameBorderOn.IsChecked() )
		bChange = TRUE;
	else if ( rProp.bBorderSet == aRBFrameBorderInherit.IsChecked() )
		bChange = TRUE;

	if ( bChange )
	{
		// Properties der anderen Pages schon vorhanden ?
		SfxFrameProperties aProp;
		aProp = rProp;
		if( SFX_ITEM_SET == rSet.GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
			aProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

		// Properties dieser Page uebernehmen
		aProp.aName = aEDName.GetText();
		aProp.aURL = aURL;
		aProp.lMarginWidth = lMarginWidth;
		aProp.lMarginHeight = lMarginHeight;
		aProp.eScroll = eScroll;
		aProp.bBorderSet = !aRBFrameBorderInherit.IsChecked();
		if ( aProp.bBorderSet )
			aProp.bHasBorder = aRBFrameBorderOn.IsChecked();
		else
			aProp.bHasBorder = aProp.bHasBorderInherited;

		// Item anlegen
		SfxFrameDescriptorItem aItem;
		aItem.SetProperties( aProp );
		rSet.Put( aItem );
	}

	return bChange;
}

IMPL_STATIC_LINK( SfxFramePropertiesPage_Impl, CheckHdl, CheckBox*, pCB )
{
	const SfxPoolItem* pItem;
	if( SFX_ITEM_SET != pThis->GetItemSet().GetItemState( pThis->GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return 0L;;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

/*
	if ( pCB == &pThis->aRBFrameBorderInherit )
	{
		if ( pCB->IsChecked() )
		{
			pThis->aCBFrameBorder.Check( rProp.bHasBorderInherited );
			pThis->aCBFrameBorder.Enable( FALSE );
		}
		else
			pThis->aCBFrameBorder.Enable( TRUE );
	}
*/
	if ( pCB == &pThis->aCBMarginWidthDefault )
	{
		if ( pCB->IsChecked() )
			pThis->aNMMarginWidth.SetText( String::CreateFromInt32( DEFAULT_MARGIN_WIDTH ) );
		pThis->aFTMarginWidth.Enable( !pCB->IsChecked() );
		pThis->aNMMarginWidth.Enable( !pCB->IsChecked() );
	}

	if ( pCB == &pThis->aCBMarginHeightDefault )
	{
		if ( pCB->IsChecked() )
			pThis->aNMMarginHeight.SetText( String::CreateFromInt32( DEFAULT_MARGIN_HEIGHT ) );
		pThis->aFTMarginHeight.Enable( !pCB->IsChecked() );
		pThis->aNMMarginHeight.Enable( !pCB->IsChecked() );
	}

	return 0L;
}

IMPL_STATIC_LINK( SfxFramePropertiesPage_Impl, OpenHdl, PushButton*, pBut )
{
	Application::SetDefDialogParent( pThis );
	SfxFileDialog *pDlg = (SfxFileDialog*)
		SFX_APP()->CreateDocFileDialog( WB_OPEN | WB_STDMODAL | WB_3DLOOK, *(SfxObjectFactory*) NULL );
	pDlg->SetText( SfxResId( STR_FRAME_FILEDLG ) );
	if ( pDlg->Execute() )
		pThis->aEDURL.SetText( pDlg->GetPath() );
	delete pDlg;
	Window *pParent = pThis->GetTabDialog();
	if ( !pParent )
		// SingleTabDialog
		pParent = pThis->GetParent();
	Application::SetDefDialogParent( pParent->GetParent() );
	return 0L;
}

SfxFrameSizePage_Impl::SfxFrameSizePage_Impl( Window *pParent, const SfxItemSet& rSet )
	: SfxTabPage( pParent, SfxResId( TP_FRAMESIZE ), rSet )
	, aGBWidth ( this, ResId( GB_WIDTH ) )
	, aNMWidth ( this, ResId( NM_WIDTH ) )
	, aCBResizeWidth ( this, ResId( CB_WIDTHFIXED ) )
	, aGBHeight ( this, ResId( GB_HEIGHT ) )
	, aNMHeight ( this, ResId( NM_HEIGHT ) )
	, aCBResizeHeight ( this, ResId( CB_HEIGHTFIXED) )
	, aRBPixelWidth( this, ResId( RB_PIXELWIDTH ) )
	, aRBRelativeWidth( this, ResId( RB_RELATIVEWIDTH ) )
	, aRBPercentWidth( this, ResId( RB_PERCENTWIDTH ) )
	, aRBPixelHeight( this, ResId( RB_PIXELHEIGHT ) )
	, aRBRelativeHeight( this, ResId( RB_RELATIVEHEIGHT ) )
	, aRBPercentHeight( this, ResId( RB_PERCENTHEIGHT ) )
	, aWidthInfo( this, ResId( FT_WIDTHCOMMENT ) )
	, aHeightInfo( this, ResId( FT_HEIGHTCOMMENT ) )
{
	FreeResource();
}

void SfxFrameSizePage_Impl::Reset( const SfxItemSet& rSet )
{
	const SfxPoolItem* pItem;
	if( SFX_ITEM_SET != rSet.GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();
	SetUpdateMode( FALSE );

	long lWidth, lHeight;
	BOOL bChangeWidth, bChangeHeight;
	SizeSelector eWidthSelector, eHeightSelector;
	if ( !rProp.bIsInColSet )
	{
		lWidth = rProp.lSetSize;
		lHeight = rProp.lSize;
		bChangeWidth = rProp.bSetResizable;
		bChangeHeight = rProp.bResizable;
		eWidthSelector = rProp.eSetSizeSelector;
		eHeightSelector = rProp.eSizeSelector;
		aHeightInfo.Hide();
		aWidthInfo.Show();
	}
	else
	{
		lWidth = rProp.lSize;
		lHeight = rProp.lSetSize;
		bChangeWidth = rProp.bResizable;
		bChangeHeight = rProp.bSetResizable;
		eWidthSelector = rProp.eSizeSelector;
		eHeightSelector = rProp.eSetSizeSelector;
		aWidthInfo.Hide();
		aHeightInfo.Show();
	}

	// FrameSize
	aNMWidth.SetText( String::CreateFromInt32( lWidth ) );
	aNMHeight.SetText( String::CreateFromInt32( lHeight ) );

	// Controls disablen, falls RootSet
	BOOL bWidthEnable = lWidth != SIZE_NOT_SET;
	BOOL bHeightEnable = lHeight != SIZE_NOT_SET;
	aRBPixelWidth.Enable( bWidthEnable );
	aRBRelativeWidth.Enable( bWidthEnable );
	aRBPercentWidth.Enable( bWidthEnable );
	aNMWidth.Enable( bWidthEnable );
	aCBResizeWidth.Enable( bWidthEnable );
	aRBPixelHeight.Enable( bHeightEnable );
	aRBRelativeHeight.Enable( bHeightEnable );
	aRBPercentHeight.Enable( bHeightEnable );
	aNMHeight.Enable( bHeightEnable );
	aCBResizeHeight.Enable( bHeightEnable );

	// Art der Gr"o\senangabe
	aRBPixelWidth.Check( FALSE );
	aRBPercentWidth.Check( FALSE );
	aRBRelativeWidth.Check( FALSE );
	aRBPixelHeight.Check( FALSE );
	aRBPercentHeight.Check( FALSE );
	aRBRelativeHeight.Check( FALSE );
	switch( eWidthSelector )
	{
		case SIZE_ABS:
			aRBPixelWidth.Check( TRUE );
			break;
		case SIZE_PERCENT:
			aRBPercentWidth.Check( TRUE );
			break;
		case SIZE_REL:
			aRBRelativeWidth.Check( TRUE );
			break;
	}
	switch( eHeightSelector )
	{
		case SIZE_ABS:
			aRBPixelHeight.Check( TRUE );
			break;
		case SIZE_PERCENT:
			aRBPercentHeight.Check( TRUE );
			break;
		case SIZE_REL:
			aRBRelativeHeight.Check( TRUE );
			break;
	}

	// Resizable
	aCBResizeWidth.Check( bChangeWidth );
	aCBResizeHeight.Check( bChangeHeight );

	SetUpdateMode( TRUE );
}

BOOL SfxFrameSizePage_Impl::FillItemSet( SfxItemSet& rSet)
{
	// Zuerst auf Ver"anderungen testen
	const SfxPoolItem* pItem=NULL;
	const SfxItemSet *pSet = &GetItemSet();
	if ( GetTabDialog() )
		pSet = GetTabDialog()->GetExampleSet();
	if( SFX_ITEM_SET != pSet->GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return FALSE;;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

	BOOL bChange = FALSE;
	long lSize, lSetSize;
	BOOL bResizable, bSetResizable;
	SizeSelector eSizeSelector, eSetSizeSelector, eWidthSelector, eHeightSelector;
	if ( aRBPixelWidth.IsChecked() )
		eWidthSelector = SIZE_ABS;
	else if ( aRBPercentWidth.IsChecked() )
		eWidthSelector = SIZE_PERCENT;
	else if ( aRBRelativeWidth.IsChecked() )
		eWidthSelector = SIZE_REL;
	if ( aRBPixelHeight.IsChecked() )
		eHeightSelector = SIZE_ABS;
	else if ( aRBPercentHeight.IsChecked() )
		eHeightSelector = SIZE_PERCENT;
	else if ( aRBRelativeHeight.IsChecked() )
		eHeightSelector = SIZE_REL;

	if ( !rProp.bIsInColSet )
	{
		lSetSize = (long) aNMWidth.GetText().ToInt32();
		lSize = (long) aNMHeight.GetText().ToInt32();
		bSetResizable = aCBResizeWidth.IsChecked();
		bResizable  = aCBResizeHeight.IsChecked();
		eSizeSelector = eHeightSelector;
		eSetSizeSelector = eWidthSelector;
	}
	else
	{
		lSize = (long) aNMWidth.GetText().ToInt32();
		lSetSize = (long) aNMHeight.GetText().ToInt32();
		bResizable = aCBResizeWidth.IsChecked();
		bSetResizable  = aCBResizeHeight.IsChecked();
		eSetSizeSelector = eHeightSelector;
		eSizeSelector = eWidthSelector;
	}

	if ( rProp.lSize != lSize )
		bChange = TRUE;
	else if ( rProp.lSetSize != lSetSize )
		bChange = TRUE;
	else if ( rProp.eSizeSelector != eSizeSelector )
		bChange = TRUE;
	else if ( rProp.eSetSizeSelector != eSetSizeSelector )
		bChange = TRUE;
	else if ( rProp.bResizable != bResizable )
		bChange = TRUE;
	else if ( rProp.bSetResizable != bSetResizable )
		bChange = TRUE;

	if ( bChange )
	{
		// Properties der anderen Pages schon vorhanden ?
		SfxFrameProperties aProp;
		aProp = rProp;
		if( SFX_ITEM_SET == rSet.GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
			aProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

		// Properties dieser Page "ubernehmen
		aProp.lSize = lSize;
		aProp.lSetSize = lSetSize;
		aProp.eSizeSelector = eSizeSelector;
		aProp.eSetSizeSelector = eSetSizeSelector;
		aProp.bResizable = bResizable;
		aProp.bSetResizable = bSetResizable;

		// Item anlegen
		SfxFrameDescriptorItem aItem;
		aItem.SetProperties( aProp );
		rSet.Put( aItem );
	}

	return bChange;
}

SfxFrameSpacingPage_Impl::SfxFrameSpacingPage_Impl( Window *pParent, const SfxItemSet& rSet )
	: SfxTabPage( pParent, SfxResId( TP_FRAMESPACING ), rSet )
	, aFTFrameSpacing( this, ResId( FT_FRAMESPACING ) )
	, aNMFrameSpacing( this, ResId( NM_FRAMESPACING ) )
	, aCBFrameSpacingInherit( this, ResId( CB_FRAMESPACING_INHERIT ) )
	, aFTFrameSpacingColor( this, ResId( FT_FRAMESPACINGCOLOR ) )
	, aEdFrameSpacingColor( this, ResId( ED_FRAMESPACINGCOLOR ) )
	, aCBFrameSpacingColorInherit( this, ResId( CB_FRAMESPACINGCOLOR_INHERIT ) )
{
	FreeResource();
	aCBFrameSpacingInherit.SetClickHdl( STATIC_LINK( this, SfxFrameSpacingPage_Impl, CheckHdl ) );
}

void SfxFrameSpacingPage_Impl::Reset( const SfxItemSet& rSet )
{
	const SfxPoolItem* pItem;
	if( SFX_ITEM_SET != rSet.GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();
	SetUpdateMode( FALSE );

	// Gr"oe\se des FrameSpacings
	if ( rProp.bIsRootSet )
		// Beim RootSet gibt es keine Vererbung, aber Ein - und Ausschalten des Defaults
		aCBFrameSpacingInherit.SetText( SfxResId( STR_FRAME_DEFAULT ) );

	if ( rProp.lFrameSpacing != SPACING_NOT_SET )
	{
		aNMFrameSpacing.SetText( String::CreateFromInt32( rProp.lFrameSpacing ) );
		aCBFrameSpacingInherit.Check( FALSE );
		aNMFrameSpacing.Enable( TRUE );
	}
	else
	{
		aNMFrameSpacing.SetText( String() );
		aCBFrameSpacingInherit.Check( TRUE );
		aNMFrameSpacing.Enable( FALSE );
	}

	SetUpdateMode( TRUE );
}

BOOL SfxFrameSpacingPage_Impl::FillItemSet( SfxItemSet& rSet)
{
	// Zuerst auf Ver"anderungen testen
	const SfxPoolItem* pItem=NULL;
	const SfxItemSet *pSet = &GetItemSet();
	if ( GetTabDialog() )
		pSet = GetTabDialog()->GetExampleSet();
	if( SFX_ITEM_SET != pSet->GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return FALSE;;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

	BOOL bChange = FALSE;
	String aSpacing = aNMFrameSpacing.GetText();
	long lSpacing = aSpacing.Len() ? (long) aSpacing.ToInt32() : SPACING_NOT_SET;
	if ( lSpacing != rProp.lFrameSpacing )
			bChange = TRUE;

	if ( bChange )
	{
		// Properties der anderen Pages schon vorhanden ?
		SfxFrameProperties aProp;
		aProp = rProp;
		if( SFX_ITEM_SET == rSet.GetItemState( GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
			aProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

		// Properties dieser Page "ubernehmen
		aProp.lFrameSpacing = lSpacing;

		// Item anlegen
		SfxFrameDescriptorItem aItem;
		aItem.SetProperties( aProp );
		rSet.Put( aItem );
	}

	return bChange;
}

IMPL_STATIC_LINK( SfxFrameSpacingPage_Impl, CheckHdl, CheckBox*, pCB )
{
	const SfxPoolItem* pItem;
	if( SFX_ITEM_SET != pThis->GetItemSet().GetItemState( pThis->GetWhich( SID_FRAMEDESCRIPTOR ), TRUE, &pItem ) )
	{
		DBG_ERROR( "Kein Item!" );
		return 0L;;
	}

	const SfxFrameProperties& rProp = ((SfxFrameDescriptorItem*)pItem)->GetProperties();

	if ( pCB == &pThis->aCBFrameSpacingInherit )
	{
		pThis->aNMFrameSpacing.Enable( !pCB->IsChecked() );
		if ( rProp.lInheritedFrameSpacing == SPACING_NOT_SET )
			pThis->aNMFrameSpacing.SetText( String() );
		else
			pThis->aNMFrameSpacing.SetText( String::CreateFromInt32( rProp.lInheritedFrameSpacing ) );
	}
	return 0L;
}


