/*************************************************************************
 *
 *  $RCSfile: pinenv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _RCID_H
#include <vcl/rcid.h>
#endif
#ifndef _IPENV_HXX //autogen
#include <so3/ipenv.hxx>
#endif
#ifndef _WRKWIN_HXX //autogen
#include <vcl/wrkwin.hxx>
#endif
#include <tools/urlobj.hxx>
#pragma hdrstop

#include "dispatch.hxx"
#include "viewsh.hxx"
#include "accmgr.hxx"
#include "cfgmgr.hxx"
#include "pinfrm.hxx"
#include "pinenv.hxx"
#include "objsh.hxx"
#include "workwin.hxx"
#include "docfac.hxx"
#include "module.hxx"
#include "helper.hxx"

//=========================================================================

SfxPlugInEnv_Impl::SfxPlugInEnv_Impl
(
	SfxPlugInFrame*	pPIFrame,
	Window *pWindowP
)
	: pFrame( pPIFrame ),
	  pAccMgr( NULL ),
	  pWindow( pWindowP )
{
	// ConfigManager anlegen
	SfxObjectShell *pObjShell = pFrame->GetObjectShell();
	INetURLObject aCfgFile( SFX_INIMANAGER()->Get( SFX_KEY_USERCONFIG_PATH ) );

	// PlugIn soll immer nur eine Default-Konfiguration verwendet werden
	aCfgFile.insertName( DEFINE_CONST_UNICODE( "PLUGIN" ) );
	aCfgFile.setExtension( DEFINE_CONST_UNICODE( "cfg") );

	// Die Default-Konfiguration soll leer sein
	SfxContentHelper::Kill( aCfgFile.GetMainURL() );
	pCfgMgr = new SfxConfigManager( aCfgFile.PathToFileName() );

	// Evtl. Doc-Config ber"ucksichtigen
	if ( pObjShell->GetConfigManager()  )
		pObjShell->GetConfigManager()->SetParent(pCfgMgr);

	// SfxWorkWindow zur Anordnung der Tools anlegen
	pWorkWin = new SfxPIWorkWin_Impl( pWindowP, pFrame->GetBindings(), this);
}


Window* SfxPlugInEnv_Impl::GetModalDialogParent() const
{
	return &pFrame->GetWindow();
}

//-------------------------------------------------------------------------

SfxPlugInEnv_Impl::~SfxPlugInEnv_Impl()
{
	pWorkWin->DeleteControllers_Impl();
	delete pAccMgr;
	delete pCfgMgr;
	delete pWorkWin;
}

//--------------------------------------------------------------------

void SfxPlugInEnv_Impl::TopWinResize()
/*
	[Beschreibung]

	Wird "uber die PlugIn-Schnittstelle aufgerufen, um Gr"o\sen"anderungen
	des PlugIn-Windows mitzuteilen.
*/
{
		ShowUITools( TRUE );
}


//--------------------------------------------------------------------

Rectangle SfxPlugInEnv_Impl::GetTopOuterRectPixel() const
/*
	[Beschreibung]

	Liefert den Platz, den der Container f"ur Tools des PlugIns und die
	ClientArea frei gelassen hat.
*/
{
	Window* pTopWin = pWorkWin->GetWindow();
	Rectangle aOuter = Rectangle( Point(), pTopWin->GetOutputSizePixel() );
	return aOuter;
}

//--------------------------------------------------------------------

void SfxPlugInEnv_Impl::SetTopToolFramePixel( const SvBorder& rBorder )
/*
	[Beschreibung]

	Diese Methode wird aufgerufen, wenn der Container per IPC dem PlugIn
	den vom Container f"ur seine Tools belegten Raum mitteilt. Das PlugIn
	reagiert darauf durch Neuanordnen seiner Tools und Setzen des AppBorders,
	was evtl. ein TopWinResize zur Folge haben wird.
*/
{
	aTopToolFramePixel = rBorder;

	// Es k"onnte sein, da\s das vom Container synchron verschickte Event
	// vor dem asynchron verschickten Activate kommt; dann warten wir auf
	// dieses
	SfxViewFrame *pAct = SfxViewFrame::Current();

	// Der Border vom Container ist auch der Border f"ur ein IPObject, da
	// der PlugServer als OLE-Container keine Tools anzeigt
	SfxInPlaceClient *pCli = pFrame->GetViewShell()->GetUIActiveClient();

	// Wenn der aktuelle Frame ein aktiver interner IPFrame ist oder einen
	// aktiven IPClient enth"alt
	BOOL bInPlaceActive = pCli &&
			( pAct == pFrame || pAct->GetParentViewFrame_Impl() == pFrame );
	if ( bInPlaceActive)
		pCli->GetEnv()->SetTopToolFramePixel( rBorder );
	else if ( pAct == pFrame )
		ShowUITools( TRUE );
}

void SfxPlugInEnv_Impl::SetBorderPixel(const SvBorder& rBorder)
/*
	[Beschreibung]

	Mit dieser Methode wird der f"ur Tools ben"otigte Bereich am Environment
	gesetzt. Je nachdem, ob die Tools im PlugInWindow liegen oder nicht,
	wird der Bereich vom FrameWindow abgezogen oder in den AppBorder des
	Containers einbezogen.
*/
{
	// Gr"o\se des FrameWindows = des PlugIn-Windows
	Size aSize = pWindow->GetOutputSizePixel();

	// Position des FrameWindows relativ zum PlugInWindow
	Point aPoint;
	// ToolSpace abziehen
	aSize.Width() -= ( rBorder.Left() + rBorder.Right() );
	aSize.Height() -= ( rBorder.Top() + rBorder.Bottom() );
	aPoint.X() = rBorder.Left();
	aPoint.Y() = rBorder.Top();

	pFrame->GetWindow().SetPosPixel( aPoint );

	if ( aSize != pFrame->GetWindow().GetSizePixel() )
	{
		pFrame->GetWindow().SetOutputSizePixel( aSize );
		pFrame->DoAdjustPosSizePixel( pFrame->GetViewShell(), Point(), aSize );
	}
}

void SfxPlugInEnv_Impl::ShowUITools( BOOL bShow )
{
	if ( bShow )
	{
		// Update am Dispatcher, falls noch nicht gelaufen
		if ( !pFrame->GetDispatcher()->Update_Impl() )
		{
			// Wenn Update gelaufen, einfach Tools neu anordnen
			pWorkWin->ArrangeChilds_Impl();
			pWorkWin->ShowChilds_Impl();
		}

		pFrame->GetBindings().HidePopups(FALSE);
	}
	else
	{
		pWorkWin->HideChilds_Impl();
	}
}

//--------------------------------------------------------------------

void SfxPlugInEnv_Impl::ActivateConfig()
{
	SfxConfigManager *pMgr = pFrame->GetObjectShell()->GetConfigManager();
	if ( pMgr )
		SFX_APP()->SetConfigManager_Impl( pMgr );
	else
		SFX_APP()->SetConfigManager_Impl( pCfgMgr);

	if ( !pAccMgr )
	{
		// PlugIn soll ein extra definierter, nicht-konfigurierbarer
		// Accelerator verwendet werden
		ResId aResId(RID_PLUGINACCEL, pFrame->GetObjectShell()->
					 GetModule()->GetResMgr());
		aResId.SetRT(RSC_ACCEL);
		if (Resource::GetResManager()->IsAvailable(aResId))
		{
			pAccMgr = new SfxAcceleratorManager( aResId,
												 pFrame->GetBindings() );
		}
		else
			pAccMgr = new SfxAcceleratorManager( pFrame->GetBindings() );
		pAccMgr->Initialize();
	}
}

void SfxPlugInEnv_Impl::SetPresentationMode( BOOL bSet, WorkWindow* pWin )
{
	if ( !bSet && !pWin )
	{
		SfxWorkWindow *pOld = pWorkWin;

		// Tools dicht machen
		pWorkWin->Close_Impl();
		pWorkWin->DeleteControllers_Impl();

		// WorkWindow wieder restaurieren
		pWorkWin = new SfxPIWorkWin_Impl( pWindow, pFrame->GetBindings(), this);

		Application::SetDefDialogParent( pWorkWin->GetWindow() );

		// Auch die Abrei\sfenster closen; da das TopWindow wieder gewechselt
		// hat, werden sie deleted
		pWorkWin->GetBindings().HidePopups( TRUE );

		// Alle children weg, dann kann auch das alte WorkWindow weg
		delete pOld;

        pFrame->GetDispatcher()->Update_Impl(TRUE);
	}
	else if ( bSet && pWindow )
	{
		// Tools dicht machen; Abrei\sfenster bleiben stehen, damit sie
		// gecloned werden k"onnen
		pWorkWin->Close_Impl();
		pWorkWin->DeleteControllers_Impl();
		delete pWorkWin;

		// Neues spezielles WorkWindow erzeugen
		pWorkWin = new SfxPIWorkWin_Impl( pWin, pFrame->GetBindings(), this);

		pWorkWin->SetDockingAllowed( FALSE );
		Application::SetDefDialogParent( pWin );
        pFrame->GetDispatcher()->Update_Impl(TRUE);
		pWorkWin->MakeChildsVisible_Impl( FALSE );
		pWorkWin->HidePopups_Impl( TRUE, 1 );
	}
	else
		DBG_ERROR( "Falscher Aufruf SetPresentationMode!" );
}


