/*************************************************************************
 *
 *  $RCSfile: testdoc.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:38 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <undo.hxx>

#pragma hdrstop

#include <sfxiiter.hxx>
#include "sfxdoc.hxx"
#include "sfxdlg.hxx"
#include "sfx.hrc"
#include "sfx.hxx"
#include "sfxitems.hxx"
#include "testdoc.hxx"
#include "rscsfx.hxx"

#include "sample.hrc"

//==================================================================
static SfxVoidItem aStaticDefault(1);

static SfxPoolItem* aStaticDefaults[1] =
{
	&aStaticDefault
};

class TestAttrib: public SfxSetItem
{
public:
							TestAttrib(SfxItemPool &rPool);
							TestAttrib(SfxItemSet *pSet);

	virtual SfxPoolItem*    Clone( SfxItemPool *pPool ) const;
	virtual SfxPoolItem*    Create(SvStream &, USHORT nVersion) const;
	virtual SvStream&       Store(SvStream &) const;
};

TestAttrib::TestAttrib(SfxItemPool &rPool):
	SfxSetItem( ATR_SET, new SfxItemSet(rPool) )
{
}

TestAttrib::TestAttrib(SfxItemSet *pSet ):
	SfxSetItem( ATR_SET, pSet )
{
}

SfxPoolItem* __EXPORT TestAttrib::Clone( SfxItemPool *pPool ) const
{
	return new TestAttrib( GetItemSet().Clone(TRUE, pPool) );
}

SfxPoolItem* __EXPORT TestAttrib::Create(SvStream &rStream, USHORT nVersion) const
{
	SfxItemSet *pNewSet = new SfxItemSet( *GetItemSet().GetPool() );
	pNewSet->Load( rStream );
	return new TestAttrib( pNewSet );
}

SvStream& __EXPORT TestAttrib::Store(SvStream &rStream) const
{
	GetItemSet().Store(rStream);
	return rStream;
}

static TestAlign    	aDefaultAlign;
static SfxBoolItem  	aDefaultBold( ATR_BOLD, FALSE );
static SfxBoolItem  	aDefaultBool( ATR_BOOL_TEST, FALSE );
static SfxStringItem 	aDefaultString( ATR_STRING_TEST, FALSE );

//==================================================================

USHORT __EXPORT TestAlign::GetValueCount() const
{
	return 4;
}

String __EXPORT TestAlign::GetValueText( USHORT nVal ) const
{
	return String(nVal);
}

SfxPoolItem* __EXPORT TestAlign::Clone( SfxItemPool * ) const
{
	return new TestAlign(*this);
}

SfxPoolItem* __EXPORT TestAlign::Create(SvStream &rStream, USHORT nVersion) const
{
	return new TestAlign(rStream);
}

//==================================================================

SfxBoolItem aDef0( 2000, TRUE );
SfxBoolItem aDef1( 2001, TRUE );
SfxBoolItem aDef2( 2002, TRUE );
SfxBoolItem aDef3( 2003, TRUE );
SfxBoolItem aDef4( 2004, TRUE );
SfxBoolItem aDef5( 2005, TRUE );
SfxBoolItem aDef6( 2006, TRUE );
SfxBoolItem aDef7( 2007, TRUE );
SfxBoolItem aDef8( 2008, TRUE );
SfxBoolItem aDef9( 2009, TRUE );

SfxPoolItem *aDefs[] =
{
	&aDef0,
	&aDef1,
	&aDef2,
	&aDef3,
	&aDef4,
	&aDef5,
	&aDef6,
	&aDef7,
	&aDef8,
	&aDef9
};

static void Testing( SfxItemPool *pDocPool )
{
	SfxItemPool *pPool = pDocPool;
	SfxProgress aProgress(SfxObjectShell::Current(), "Testing", 10000);

	SfxItemPool aSecondary( "SFxTest2", 2000, 2009, aDefs );
	//! pPool->SetSecondaryPool( &aSecondary );
	//! aSecondary.SetSecondaryPool( pDocPool );
	pPool = &aSecondary;

	Time aStartTime;
	for ( USHORT nLoop = 0; nLoop < 2; ++nLoop )
	{
		SfxItemSet aSet(*pPool, 2000, 2009, 1000, 1009, 0);
		for ( USHORT n0 = 0; n0 < 100; ++n0 )
		{
			aProgress.SetState(n0 * nLoop / 10);
			aSet.Put( SfxBoolItem( 2000, TRUE ) );
			aSet.Put( SfxBoolItem( 2002, TRUE ) );
			aSet.Put( SfxBoolItem( 2004, TRUE ) );
			aSet.Put( SfxBoolItem( 2006, TRUE ) );
			aSet.Put( SfxBoolItem( 2008, TRUE ) );
		}

		for ( USHORT n1 = 0; n1 < 10000; ++n1 )
		{
			aProgress.SetState(n1 * nLoop * n0 / 10);
			aSet.Get( 2000 );
			aSet.Get( 2001 );
			aSet.Get( 2002 );
			aSet.Get( 2003 );
			aSet.Get( 2004 );
			aSet.Get( 2005 );
			aSet.Get( 2006 );
			aSet.Get( 2007 );
			aSet.Get( 2008 );
			aSet.Get( 2009 );
		}
	}

	Time aEndTime( Time() - aStartTime );
	if ( SfxViewShell::Current() )
#if defined GCC
	{
		Window* pWin = SfxViewShell::Current()->GetWindow();
		InfoBox( pWin, String( ((((aEndTime.GetHour()*60L)+aEndTime.GetMin())*60L+aEndTime.GetSec())*100L+aEndTime.Get100Sec())*10L ) + String( "ms" ) ).Execute();
	}
#else
		InfoBox( SfxViewShell::Current()->GetWindow(),
				String( ((((aEndTime.GetHour()*60L)+aEndTime.GetMin())*60L+aEndTime.GetSec())*100L+aEndTime.Get100Sec())*10L ) +
				String( "ms" ) ).Execute();
#endif
	//! pPool->SetSecondaryPool(0);
	//! aSecondary.SetSecondaryPool(0);

	{
		SfxItemSet a1( aSecondary );
		SfxItemSet a2( aSecondary, 2000, 2007, 0 );
		a1.Put( SfxBoolItem( 2000, TRUE ) );  a2.Put( SfxBoolItem( 2000, TRUE ) );
		a1.Put( SfxBoolItem( 2001, TRUE ) );  a2.Put( SfxBoolItem( 2001, FALSE ) );
		a1.Put( SfxBoolItem( 2002, TRUE ) );
											  a2.Put( SfxBoolItem( 2003, TRUE ) );
		a1.Put( SfxBoolItem( 2004, FALSE ) );
											  a2.Put( SfxBoolItem( 2005, FALSE ) );
		a1.Put( SfxBoolItem( 2006, FALSE ) ); a2.Put( SfxBoolItem( 2006, TRUE ) );

		a1.Put( SfxBoolItem( 2008, TRUE ) );

		a1.MergeValues( a2, TRUE );

		DBG_ASSERT( a1.GetItemState(2000) == SFX_ITEM_SET,      "Status: s, s, ==" );
		DBG_ASSERT( a1.Get(2000) == SfxBoolItem(2000, TRUE),    "Inhalt: s, s, ==" );

		DBG_ASSERT( a1.GetItemState(2001) == SFX_ITEM_DONTCARE, "Status: s, s, !=" );

		DBG_ASSERT( a1.GetItemState(2002) == SFX_ITEM_SET,      "Status: s, 0, ==" );
		DBG_ASSERT( a1.Get(2002) == SfxBoolItem(2002, TRUE),    "Inhalt: s, 0, ==" );

		DBG_ASSERT( a1.GetItemState(2003) == SFX_ITEM_SET,      "Status: 0, s, ==" );
		DBG_ASSERT( a1.Get(2003) == SfxBoolItem(2003, TRUE),    "Inhalt: 0, s, ==" );

		DBG_ASSERT( a1.GetItemState(2004) == SFX_ITEM_SET,      "Status: s, 0, !=" );
		DBG_ASSERT( a1.Get(2004) == SfxBoolItem(2004, FALSE),   "Inhalt: s, 0, !=" );

		DBG_ASSERT( a1.GetItemState(2005) == SFX_ITEM_SET,      "Status: 0, s, !=" );
		DBG_ASSERT( a1.Get(2005) == SfxBoolItem(2005, FALSE),   "Inhalt: 0, s, !=" );

		DBG_ASSERT( a1.GetItemState(2006) == SFX_ITEM_DONTCARE, "Status: s, s, !=" );

		DBG_ASSERT( a1.GetItemState(2007) == SFX_ITEM_DEFAULT,  "Status: 0, 0" );

		DBG_ASSERT( a1.GetItemState(2008) == SFX_ITEM_SET,      "Status: s, /" );
		DBG_ASSERT( a1.Get(2008) == SfxBoolItem(2008, TRUE),    "Inhalt: s, /" );

		DBG_ASSERT( a1.GetItemState(2009) == SFX_ITEM_DEFAULT,  "Status: 0, /" );

		SfxItemSet a3( a1 );
		a1.MergeValues( a1 );
		DBG_ASSERT( a1 == a3, "a1 mit a1 mergen" );
	}

	SfxAllItemSet aSet( *pDocPool );
	{
		aSet.Put( SfxBoolItem( 30021, TRUE ) );
		aSet.Put( SfxBoolItem( 30022, TRUE ) );
		aSet.Put( SfxBoolItem( 30020, TRUE ) );
		aSet.Put( SfxBoolItem( 30011, TRUE ) );
		aSet.Put( SfxBoolItem( 30012, TRUE ) );
		aSet.Put( SfxBoolItem( 30010, TRUE ) );
		aSet.Put( SfxBoolItem( 30001, TRUE ) );
		aSet.Put( SfxBoolItem( 30002, TRUE ) );
		aSet.Put( SfxBoolItem( 30000, TRUE ) );
		aSet.Put( SfxBoolItem( 30003, TRUE ) );
		aSet.Put( SfxBoolItem( 30004, TRUE ) );
		aSet.Put( SfxBoolItem( 30005, TRUE ) );
		aSet.Put( SfxBoolItem( 30006, TRUE ) );
		aSet.Put( SfxBoolItem( 30007, TRUE ) );
		aSet.Put( SfxBoolItem( 30008, TRUE ) );
		aSet.Put( SfxBoolItem( 30009, TRUE ) );
		aSet.Put( SfxBoolItem( 30011, TRUE ) );
		aSet.Put( SfxBoolItem( 30012, TRUE ) );
		aSet.Put( SfxBoolItem( 30010, TRUE ) );
		aSet.Put( SfxBoolItem( 30031, TRUE ) );
		aSet.Put( SfxBoolItem( 30032, TRUE ) );
		aSet.Put( SfxBoolItem( 30030, TRUE ) );
	}

	{
		aSet.Put( SfxInt16Item( 30021, 30021 ) );
		aSet.Put( SfxInt16Item( 30022, 30022 ) );
		aSet.Put( SfxInt16Item( 30020, 30020 ) );
		aSet.Put( SfxInt16Item( 30011, 30011 ) );
		aSet.Put( SfxInt16Item( 30012, 30012 ) );
		aSet.Put( SfxInt16Item( 30010, 30010 ) );
		aSet.Put( SfxInt16Item( 30001, 30001 ) );
		aSet.Put( SfxInt16Item( 30002, 30002 ) );
		aSet.Put( SfxInt16Item( 30000, 30000 ) );
		aSet.Put( SfxInt16Item( 30003, 30003 ) );
		aSet.Put( SfxInt16Item( 30004, 30004 ) );
		aSet.Put( SfxInt16Item( 30005, 30005 ) );
		aSet.Put( SfxInt16Item( 30006, 30006 ) );
		aSet.Put( SfxInt16Item( 30007, 30007 ) );
		aSet.Put( SfxInt16Item( 30008, 30008 ) );
		aSet.Put( SfxInt16Item( 30009, 30009 ) );
		aSet.Put( SfxInt16Item( 30011, 30011 ) );
		aSet.Put( SfxInt16Item( 30012, 30012 ) );
		aSet.Put( SfxInt16Item( 30010, 30010 ) );
		aSet.Put( SfxInt16Item( 30031, 30031 ) );
		aSet.Put( SfxInt16Item( 30032, 30032 ) );
		aSet.Put( SfxInt16Item( 30030, 30030 ) );
	}

}

//==================================================================

class TestAction: public SfxUndoAction
{
	TestDoc                *pDoc;
	const TestAttrib        *pOldAttrib;
	SfxItemSet              aNewAttrib;

public:
							TYPEINFO();
							TestAction( TestDoc *pTheDoc,
										const TestAttrib *pOldAttr,
										const SfxItemSet &rNewAttr );
	virtual                 ~TestAction();

	virtual void            Undo();
	virtual void            Redo();
	virtual void            Repeat( TestDoc *pDocSh );
};

TYPEINIT1(TestAction, SfxUndoAction);

TestAction::TestAction( TestDoc *pTheDoc,
						const TestAttrib *pOldAttr, const SfxItemSet &rNewAttr ):
		pDoc( pTheDoc ),
		pOldAttrib(pOldAttr),
		aNewAttrib(rNewAttr)
{
	pDoc->GetPool().Put(*pOldAttrib);
}

__EXPORT TestAction::~TestAction()
{
	pDoc->GetPool().Remove(*pOldAttrib);
}

void __EXPORT TestAction::Undo()
{
	pDoc->SetAttr(pOldAttrib);
}

void __EXPORT TestAction::Redo()
{
	pDoc->PutAttr(aNewAttrib);
}

void __EXPORT TestAction::Repeat( TestDoc *pDocSh )
{
	pDocSh->PutAttr(aNewAttrib);
}

//==================================================================

class TestUndoManager: public SfxUndoManager
{
public:
							TestUndoManager() {}

	virtual void            Repeat( SfxUndoAction &rAction );
};

void __EXPORT TestUndoManager::Repeat( SfxUndoAction &rAction )
{
	( (TestAction&) rAction ).Repeat( (TestDoc*) SfxObjectShell::Current() );
}

//==================================================================

class TestItemPool: public SfxWhichMap, public SfxItemPool
{
	static USHORT       aMap[];
	SfxPoolItem 		**ppStaticDefaults;

public:
						TestItemPool();
						~TestItemPool();
};

USHORT TestItemPool::aMap[] =
	{ SID_ALIGNMENT, SID_BOLD, 0, 0, 0 };

TestItemPool::TestItemPool():
	SfxItemPool( "SFxTest", ATR_START, ATR_END, 0, aMap, FALSE )
{
	ppStaticDefaults = new SfxPoolItem*[5];
	ppStaticDefaults[0] = &aDefaultAlign;
	ppStaticDefaults[1] = &aDefaultBold;
	ppStaticDefaults[2] = new TestAttrib( *this );
	ppStaticDefaults[3] = &aDefaultBool;
	ppStaticDefaults[4] = &aDefaultString;
	SetDefaults( ppStaticDefaults );
}

__EXPORT TestItemPool::~TestItemPool()
{
	SetRef( *ppStaticDefaults[2], 0 );
	delete ppStaticDefaults[2];
	delete ppStaticDefaults;
}

//==================================================================

SFX_EXEC_STUB(TestDoc, Execute)
SFX_EXEC_STUB(TestDoc, ExecAttr)
SFX_EXEC_STUB(TestDoc, ExecTempl)
SFX_STATE_STUB(TestDoc, GetState)
SFX_STATE_STUB(TestDoc, GetAttrState)

#define ShellClass TestDoc
SFX_SLOTMAP(TestDoc)
{
	SFX_SLOT( TestDoc, SID_CONTENT, GID_DOC,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxStringItem ),

	SFX_SLOT( TestDoc, SID_TESTING, GID_DOC,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxEnumItem ),
	SFX_SLOT( TestDoc, SID_TEST_TABLE, GID_EDIT,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxEnumItem ),
	SFX_SLOT( TestDoc, SID_TEST_COLOR, GID_EDIT,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxEnumItem ),

	SFX_SLOT_ATTR( SID_ALIGNMENT, GID_DOC, ATR_ALIGNMENT,
				  SFX_STUB_PTR(TestDoc, ExecAttr),
				  SFX_STUB_PTR(TestDoc, GetAttrState),
				  SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE, SfxEnumItem ),
	SFX_SLOT_ENUM( SID_ALIGNLEFT, GID_DOC,
					SID_ALIGNMENT, SW_ALIGNLEFT, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),
	SFX_SLOT_ENUM( SID_ALIGNRIGHT, GID_DOC,
					SID_ALIGNMENT, SW_ALIGNRIGHT, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),
	SFX_SLOT_ENUM( SID_ALIGNCENTER, GID_DOC,
					SID_ALIGNMENT, SW_ALIGNCENTER, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),
	SFX_SLOT_ENUM( SID_ALIGNBLOCK, GID_DOC,
					SID_ALIGNMENT, SW_ALIGNBLOCK, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),

	SFX_SLOT_ATTR( SID_BOLD, GID_DOC, ATR_BOLD,
				  SFX_STUB_PTR(TestDoc, ExecAttr),
				  SFX_STUB_PTR(TestDoc, GetAttrState),
				  SFX_SLOT_CACHABLE | SFX_SLOT_TOGGLE |SFX_SLOT_AUTOUPDATE, SfxEnumItem),

	SFX_SLOT_ATTR( SID_TOOL, GID_DOC, ATR_TOOL,
				  SFX_STUB_PTR(TestDoc, ExecAttr),
				  SFX_STUB_PTR(TestDoc, GetAttrState),
				  SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE, SfxEnumItem ),
	SFX_SLOT_ENUM( SID_TOOL_SELECT, GID_DOC,
					SID_TOOL, TOOL_SELECT, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),
	SFX_SLOT_ENUM( SID_TOOL_RECT, GID_DOC,
					SID_TOOL, TOOL_RECT, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),
	SFX_SLOT_ENUM( SID_TOOL_CIRCLE, GID_DOC,
					SID_TOOL, TOOL_CIRCLE, SFX_SLOT_CACHABLE|SFX_SLOT_AUTOUPDATE ),

	SFX_SLOT( TestDoc, SID_TEST_WRITEREGFILE, GID_DOC,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxVoidItem ),

	SFX_SLOT( TestDoc,SID_TEMPLATE_FAMILY1, 0,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxTemplateItem ),
	SFX_SLOT( TestDoc,SID_TEMPLATE_FAMILY2, 0,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxTemplateItem ),
	SFX_SLOT( TestDoc,SID_TEMPLATE_NEW, 0,
			 SFX_STUB_PTR(TestDoc, ExecTempl),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxStringItem ),
	SFX_SLOT( TestDoc,SID_TEMPLATE_EDIT, 0,
			 SFX_STUB_PTR(TestDoc, ExecTempl),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxStringItem ),
	SFX_SLOT( TestDoc,SID_TEMPLATE_DELETE, 0,
			 SFX_STUB_PTR(TestDoc, ExecTempl),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxStringItem ),

	SFX_SLOT( TestDoc,SID_TEMPLATE_WATERCAN, 0,
			 SFX_STUB_PTR(TestDoc, ExecTempl),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxBoolItem ),

	SFX_SLOT( TestDoc,SID_OBJECTMENU0, 0,
			 SFX_STUB_PTR(TestDoc, Execute),
			 SFX_STUB_PTR(TestDoc, GetState) ,
			 0, SfxStringItem )
};

#undef ShellClass

SFX_IMPL_INTERFACE(TestDoc,SfxObjectShell,STR_TESTDOC)
{
	SFX_OBJECTBAR_REGISTRATION( 1, RID_TEST_OBJECTBAR );
	SFX_POPUPMENU_REGISTRATION( RID_TEST_POPUPMENU );
	SFX_OBJECTMENU_REGISTRATION( SID_OBJECTMENU0, RID_TEST_POPUPMENU );
	SFX_CHILDWINDOW_REGISTRATION(RID_INPUTTOOLBOX);
	SFX_STATUSBAR_REGISTRATION(ResId( RID_TESTSTATUSBAR));
}

SFX_IMPL_OBJECTFACTORY( TestDoc, SfxDem,
	0xBB0D2801L,0x73EE,0x101B, 0x80,0x4C,0xFD,0xFD,0xFD,0xFD,0xFD,0xFD )
{
	const ULONG lStd = Clipboard::RegisterFormatName("Sfx TestDokument");
	const ULONG lDocs = Clipboard::RegisterFormatName("Fremd Dokumente");
	SFX_FILTER_REGISTRATION( "Sfx Test-Document", "*.tst",
							SFX_FILTER_OWN|SFX_FILTER_IMPORT|SFX_FILTER_EXPORT,
							lStd, 0x00000000L, "Sfx Dokument", 0, "" );
	SFX_FILTER_REGISTRATION( "FremdDocs", "*.doc",
							SFX_FILTER_ALIEN|SFX_FILTER_IMPORT|SFX_FILTER_EXPORT,
							lDocs, 0x2L, "Fremd-Dokumente", 0, "");
	Factory().RegisterHelpFile(String("SFD255.HLP"));
}

TestDoc::TestDoc(SfxShellCreateMode eMode) :
	SfxObjectShell(eMode),
	pPool(0),
	pStyleSheetPool(0), // POOL
	pAttr(0),
	pAttrSet(0),
	pUndoManager(0),
	nTool(0)
{
	pPool = new TestItemPool;
	SetPool(pPool);
	SetShell( this ); // unbedingt setzen, fuer InPlace-Editing
	pAttrSet = new SfxItemSet( *pPool );
}

//------------------------------------------------------------------

void TestDoc::ExecAttr( SfxRequest& rReq )
{
	const SfxItemSet *pArgs = rReq.GetArgs();
	DBG_ASSERT(pArgs, "fehlende Argumente");

	GetUndoManager()->AddUndoAction(
		new TestAction( this, pAttr, *rReq.GetArgs()) );
	PutAttr( *rReq.GetArgs() );
	rReq.Done();
}

void TestDoc::GetAttrState(SfxItemSet &rSet)
{
	GetAttr( rSet );
}

//------------------------------------------------------------------

void TestDoc::ExecTool( SfxRequest& rReq )
{
	const SfxItemSet *pArgs = rReq.GetArgs();
	DBG_ASSERT(pArgs, "fehlende Argumente");

	nTool = ( (const SfxEnumItem&) pArgs->Get(SID_TOOL) ).GetValue();
	rReq.Done();
}

void TestDoc::GetToolState(SfxItemSet &rSet)
{
	rSet.Put( SfxAllEnumItem( SID_TOOL, nTool ) );
}

USHORT  TestDoc::DoEdit(const String &rName, USHORT nFamily)
{
	SfxStyleSheetBase *pStyle = 0;
	BOOL bNew = TRUE;
	if(!rName)
		pStyle = &pStyleSheetPool->Make("", (SfxStyleFamily)nFamily, SFXSTYLEBIT_USERDEF);
	else {
		pStyle = pStyleSheetPool->Find(rName, (SfxStyleFamily)nFamily);
		bNew = FALSE;
		DBG_ASSERT(pStyle, "Vorlage nicht gefunden");
	}
	SfxStyleDialog *pDlg = new SfxStyleDialog(SFX_APP()->GetActiveWindow(),
											 ResId(DLG_STYLE),
											 *pStyle);
	USHORT nRet = 0xffff;
	if(RET_OK == pDlg->Execute())
		nRet =  pStyle->GetMask();
	else if(bNew)
		pStyleSheetPool->Erase(pStyle);
	delete pDlg;
	return nRet;
}

USHORT __EXPORT TestDoc::New(USHORT nFamily)
{
	return DoEdit("", nFamily);
}

USHORT TestDoc::Edit(const String &rName, USHORT nFamily)
{
	return DoEdit(rName, nFamily);
}

USHORT TestDoc::Delete(const String &rName, USHORT nFamily)
{
	SfxStyleSheetBase *pStyle = pStyleSheetPool->Find(rName, (SfxStyleFamily)nFamily);
	if(pStyle) {
		pStyleSheetPool->Erase(pStyle);
		return TRUE;
	}
	return FALSE;
}


//------------------------------------------------------------------

void TestDoc::Execute( SfxRequest& rReq )
{
	switch ( rReq.GetSlot() )
	{
		case SID_TEST_WRITEREGFILE:
			break;
		case SID_TESTING:
		{
			Testing( &GetPool() );

			// fuer Test: SfxItemPool:.Load() etc.
			const SfxItemSet *pItems =
					&( (const SfxSetItem&) pAttrSet->Get(ATR_SET) ).GetItemSet();
			DBG_ASSERT( ( (SfxBoolItem&) pItems->Get( ATR_BOLD ) ) ==
						SfxBoolItem( ATR_BOLD, TRUE ),
						"Fehler beim Laden der Attribute 1" );
			DBG_ASSERT( ( (SfxBoolItem&) pItems->Get( ATR_BOOL_TEST ) ) ==
						SfxBoolItem( ATR_BOOL_TEST, FALSE ),
						"Fehler beim Laden der Attribute 2" );
			DBG_ASSERT( ( (SfxStringItem&) pItems->Get( ATR_STRING_TEST ) ) ==
						SfxStringItem( ATR_STRING_TEST, "String-Test1" ),
						"Fehler beim Laden der Attribute 3" );
			DBG_ASSERT( ( (SfxBoolItem&) pAttrSet->Get( ATR_BOOL_TEST ) ) ==
						SfxBoolItem( ATR_BOLD, FALSE ),
						"Fehler beim Laden der Attribute 4" );
			DBG_ASSERT( ( (SfxStringItem&) pAttrSet->Get( ATR_STRING_TEST ) ) ==
						SfxStringItem( ATR_STRING_TEST, "String-Test2" ),
						"Fehler beim Laden der Attribute 5" );
			break;
		}
		case SID_TEST_COLOR:
			Sound::Beep();
			break;
		case SID_TEST_TABLE:
			Sound::Beep();
			break;
	}

	rReq.Done();
}

void TestDoc::ExecTempl( SfxRequest& rReq )
{
	const USHORT nSlot = rReq.GetSlot();
	USHORT nRet = 0xffff;
	switch ( nSlot )
	{
		case SID_TEMPLATE_NEW:
			nRet = New(((const SfxUInt16Item &)rReq.GetArgs()->Get(
				SID_TEMPLATE_FAMILY)).GetValue());
			break;
		case SID_TEMPLATE_EDIT:
		case SID_TEMPLATE_DELETE:
		{
			const String aParam(((
				const SfxStringItem &)rReq.GetArgs()->Get(nSlot)).GetValue());
			USHORT nFamily = ((const SfxUInt16Item &)rReq.GetArgs()->
							  Get(SID_TEMPLATE_FAMILY)).GetValue();
			if(nSlot == SID_TEMPLATE_EDIT)
				nRet = Edit(aParam, nFamily);
			else
				nRet = Delete(aParam, nFamily);
			break;
		}
		case SID_TEMPLATE_WATERCAN:
			if( GetMedium() )
			{
				SfxItemSet *pSet=GetMedium()->GetItemSet();
				if(pSet)
				{
					const SfxPoolItem *pItem;
					if( pSet->GetItemState(
						SID_FILE_FILTEROPTIONS, FALSE, &pItem ) ==
						SFX_ITEM_SET)
					{
						const SfxStringItem *pString=PTR_CAST(SfxStringItem, pItem);
						if(pString)
							DBG_ERROR(pString->GetValue());
					}
					pSet->Put(SfxStringItem(SID_FILE_FILTEROPTIONS, "Test"));
					DBG_ERROR("Test gesetzt");
				}
			}
			break;
	}
	if(nRet != 0xffff)
		rReq.SetReturnValue(SfxUInt16Item(nSlot, nRet));
	rReq.Done();
}

void TestDoc::GetState(SfxItemSet &rSet)
{

	SfxWhichIter aIter(rSet);
	USHORT nWhich = aIter.FirstWhich();
	while(nWhich) {
		switch ( nWhich ) {
			case SID_TEMPLATE_FAMILY1:
				rSet.Put(SfxTemplateItem(nWhich, "Absatzvorlage 1"));
				break;
			case SID_TEMPLATE_FAMILY2:
				rSet.Put(SfxTemplateItem(nWhich, "Zeichenvorlage 1"));
				break;
			case SID_TEMPLATE_WATERCAN:
				rSet.Put(SfxBoolItem(nWhich, FALSE ));
				break;
			case SID_OBJECTMENU0:
				rSet.Put( SfxStringItem( SID_OBJECTMENU0, "TestDoc" ) );
		}
		nWhich = aIter.NextWhich();
	}
}

SfxStyleSheetBasePool* __EXPORT TestDoc::GetStyleSheetPool()    // POOL
{
	if(!pStyleSheetPool) {
		pStyleSheetPool = new SfxStyleSheetPool(GetPool());
		pStyleSheetPool->Make("Absatzvorlage 1", SFX_STYLE_FAMILY_PARA, 0x0001);
		pStyleSheetPool->Make("Absatzvorlage 2", SFX_STYLE_FAMILY_PARA, 0x0001);
		pStyleSheetPool->Make("Absatzvorlage 3", SFX_STYLE_FAMILY_PARA,  0x1000);
		pStyleSheetPool->Make("Absatzvorlage 4", SFX_STYLE_FAMILY_PARA,  0x1000);

		pStyleSheetPool->Make("Zeichenvorlage 1", SFX_STYLE_FAMILY_CHAR, 0x0001);
		pStyleSheetPool->Make("Zeichenvorlage 2", SFX_STYLE_FAMILY_CHAR, 0x0001);
		pStyleSheetPool->Make("Zeichenvorlage 3", SFX_STYLE_FAMILY_CHAR,  0x1000);
		pStyleSheetPool->Make("Zeichenvorlage 4", SFX_STYLE_FAMILY_CHAR,  0x1000);
	}
	return pStyleSheetPool;
}

StatusBar*__EXPORT TestDoc::CreateStatusBar()
{
	StatusBar* pStatusBar = new StatusBar(SFX_APP()->GetAppWindow(), WinBits(WB_BORDER|WB_SVLOOK) );
	pStatusBar->InsertItem( SID_CURRENTTIME, pStatusBar->GetTextSize(
		String("::") + pApp->GetAppInternational().GetTime( Time() ) ).Width() );
	pStatusBar->InsertItem( SID_CURRENTDATE, pStatusBar->GetTextSize(
		String("::") + pApp->GetAppInternational().GetDate( Date() ) ).Width() );
	return pStatusBar;
}

//------------------------------------------------------------------


__EXPORT TestDoc::~TestDoc()
{
	delete pUndoManager;
	delete pAttrSet;

	GetPool().Remove( *pAttr );
	delete pStyleSheetPool; pStyleSheetPool = 0;
	delete pPool; pPool = 0;
}

BOOL __EXPORT TestDoc::ConvertFrom( SfxMedium &rMedium )
{
	return TRUE ;
}

BOOL __EXPORT TestDoc::ConvertTo( SfxMedium &rMedium )
{
	return TRUE ;
}

//------------------------------------------------------------------

ToolBox* __EXPORT TestDoc::CreateObjectBar( Window *pParent ) const
{
	return new ToolBox( pParent, ResId(RID_DEFAULTTOOLBOX) );
}

//------------------------------------------------------------------

void __EXPORT TestDoc::FillClass( SvUniqueName * pClassName,
						 ULONG * pClipFormat,
						 String * pLongUserName,
						 String * pUserName ) const
{
	// niemals convert oder treatas
	*pClassName     = *GetSvFactory();
	*pClipFormat    = Exchange::RegisterFormatName( "Sfx Test Doc" );
	*pLongUserName  = "Sfx Test Document 1.0";
	*pUserName      = "Sfx TestDoc";
}

//------------------------------------------------------------------

SfxUndoManager* __EXPORT TestDoc::GetUndoManager()
{
	if (!pUndoManager)
		pUndoManager = new SfxUndoManager;
	return pUndoManager;
}

//------------------------------------------------------------------
BOOL __EXPORT TestDoc::Load( SvStorage * pStor )
{
	//! pMed->SetToFile();
	if( SfxInPlaceObject::Load( pStor ) )
	{
		aDocStm = pStor->OpenStream( TESTDOC_NAME );
		if ( aDocStm.Is() )
		{
			// den Pool und die Attributierung laden
			aDocStm->SetBufferSize(4096);
			USHORT nTest;
			*aDocStm >> nTest;
			DBG_ASSERT( nTest == 0x1234, "not start of my stream!" );
			pPool->Load( *aDocStm );
			USHORT nWhich = ATR_SET;
			pAttr = (TestAttrib*) pPool->LoadSurrogate( *aDocStm, nWhich, 0 );
			pAttrSet->Load( *aDocStm );
			pPool->LoadCompleted();

			// fuer Test: SfxItemPool:.Load() etc.
			const SfxItemSet *pItems =
					&( (const SfxSetItem&) pAttrSet->Get(ATR_SET) ).GetItemSet();
			DBG_ASSERT( ( (SfxBoolItem&) pItems->Get( ATR_BOLD ) ) == SfxBoolItem( ATR_BOLD, TRUE ),
						"Fehler beim Laden der Attribute 1" );
			DBG_ASSERT( ( (SfxBoolItem&) pItems->Get( ATR_BOOL_TEST ) ) == SfxBoolItem( ATR_BOOL_TEST, FALSE ),
						"Fehler beim Laden der Attribute 2" );
			DBG_ASSERT( ( (SfxStringItem&) pItems->Get( ATR_STRING_TEST ) ) == SfxStringItem( ATR_STRING_TEST, "String-Test1" ),
						"Fehler beim Laden der Attribute 3" );
			DBG_ASSERT( ( (SfxBoolItem&) pAttrSet->Get( ATR_BOOL_TEST ) ) == SfxBoolItem( ATR_BOLD, FALSE ),
						"Fehler beim Laden der Attribute 4" );
			DBG_ASSERT( ( (SfxStringItem&) pAttrSet->Get( ATR_STRING_TEST ) ) == SfxStringItem( ATR_STRING_TEST, "String-Test2" ),
						"Fehler beim Laden der Attribute 5" );

			aDocStm.Clear();
			return TRUE;
		}
	}

	return FALSE;
}

//------------------------------------------------------------------

BOOL __EXPORT TestDoc::Save()
{
	//! GetMedium()->SetToFile();
	if( SfxInPlaceObject::Save() )
	{
		aDocStm = GetStorage()->OpenStream( TESTDOC_NAME );
		if ( aDocStm.Is() )
		{
			aDocStm->SetSize(0);
			aDocStm->SetBufferSize(4096);

			// den Pool und die die Attributierung speichern
			*aDocStm << (USHORT) 0x1234;
			pPool->Store( *aDocStm );
			pPool->StoreSurrogate( *aDocStm, pAttr );
			pAttrSet->Store( *aDocStm );

			aDocStm.Clear();
		}
		GetStorage()->Commit();
		return TRUE;
	}

	return FALSE;
}

//------------------------------------------------------------------

BOOL __EXPORT TestDoc::SaveAs( SvStorage * pNewStor )
{
	//! rNewFile.SetToFile();
	if( SfxInPlaceObject::SaveAs( pNewStor ) )
	{
		SvStorageStreamRef aStm = pNewStor->OpenStream( TESTDOC_NAME );
		if ( aStm.Is() )
		{
			aStm->SetSize(0);
			aStm->SetBufferSize(4096);

			// den Pool und die Attributierung speichern
			*aStm << (USHORT) 0x1234;
			pPool->Store( *aStm );
			pPool->StoreSurrogate( *aStm, pAttr );
			pAttrSet->Store( *aStm );
			aStm.Clear();

			pNewStor->Commit();
			return TRUE;
		}
	}

	return FALSE;
}

//------------------------------------------------------------------

BOOL __EXPORT TestDoc::SaveCompleted( SvStorage * pStor )
{
	if( SfxInPlaceObject::SaveCompleted( pStor ) )
	{
		if( !pStor )
			return TRUE;
		aDocStm = pStor->OpenStream( TESTDOC_NAME );
		return aDocStm.Is();
	}
	return FALSE;
}

//------------------------------------------------------------------

void TestDoc::PutAttr( const SfxItemSet &rSet )
{
	SfxItemPoolCache aCache( &GetPool(), &rSet );
	pAttr = (TestAttrib*) &aCache.ApplyTo( *pAttr );

	SetModified(TRUE);
	Broadcast( SfxSimpleHint( SFX_HINT_DATACHANGED ) );
}

//------------------------------------------------------------------

void TestDoc::SetAttr( const TestAttrib *pAttrib )
{
	GetPool().Remove(*pAttr);
	pAttr = (TestAttrib*) &GetPool().Put(*pAttrib);

	Invalidate( ATR_BOLD );
	Invalidate( ATR_ALIGNMENT );
	SetModified(TRUE);
	Broadcast( SfxSimpleHint( SFX_HINT_DATACHANGED ) );
}

//------------------------------------------------------------------

void TestDoc::GetAttr( SfxItemSet &rSet )
{
	rSet.Set( pAttr->GetItemSet() );
}

//------------------------------------------------------------------

Font TestDoc::GetFont( USHORT nSize ) const
{
	Font aFont;
	aFont.SetSize( Size( 0, nSize ) );
	const SfxBoolItem &rBold = (const SfxBoolItem&) pAttr->GetItemSet().Get( ATR_BOLD );
	aFont.SetWeight( rBold.GetValue() ? WEIGHT_BOLD : WEIGHT_NORMAL );
	return aFont;
}

//------------------------------------------------------------------

USHORT TestDoc::GetAlign() const
{
	const TestAlign &rAlign = (const TestAlign&) pAttr->GetItemSet().Get( ATR_ALIGNMENT );
	return rAlign.GetValue();
}

//------------------------------------------------------------------

BOOL __EXPORT TestDoc::InitNew( SvStorage * pStor )
{
	if( SfxInPlaceObject::InitNew( pStor ) )
	{
		pAttr = (const TestAttrib*) &GetPool().Put( TestAttrib(*pPool) );
		pAttrSet = new SfxItemSet( *pPool );

		// fuer Test: SfxItemPool:.Load() etc.
		SfxItemSet *pItems = new SfxItemSet( *pPool );
		pItems->Put( SfxBoolItem( ATR_BOLD, TRUE ) );
		pItems->Put( SfxBoolItem( ATR_BOOL_TEST, FALSE ) );
		pItems->Put( SfxStringItem( ATR_STRING_TEST, "String-Test1" ) );
		pAttrSet->Put( TestAttrib( pItems ) );
		pAttrSet->Put( SfxBoolItem( ATR_BOLD, FALSE ) );
		pAttrSet->Put( SfxStringItem( ATR_STRING_TEST, "String-Test2" ) );

		// fuer Test: SfxItemPool:.Load() etc.
		{
		const SfxItemSet *pItems =
				&( (const SfxSetItem&) pAttrSet->Get(ATR_SET) ).GetItemSet();
		DBG_ASSERT( ( (SfxBoolItem&) pItems->Get( ATR_BOLD ) ) == SfxBoolItem( ATR_BOLD, TRUE ),
					"Fehler beim Laden der Attribute 1" );
		DBG_ASSERT( ( (SfxBoolItem&) pItems->Get( ATR_BOOL_TEST ) ) == SfxBoolItem( ATR_BOOL_TEST, FALSE ),
					"Fehler beim Laden der Attribute 2" );
		DBG_ASSERT( ( (SfxStringItem&) pItems->Get( ATR_STRING_TEST ) ) == SfxStringItem( ATR_STRING_TEST, "String-Test1" ),
					"Fehler beim Laden der Attribute 3" );
		DBG_ASSERT( ( (SfxBoolItem&) pAttrSet->Get( ATR_BOOL_TEST ) ) == SfxBoolItem( ATR_BOLD, FALSE ),
					"Fehler beim Laden der Attribute 4" );
		DBG_ASSERT( ( (SfxStringItem&) pAttrSet->Get( ATR_STRING_TEST ) ) == SfxStringItem( ATR_STRING_TEST, "String-Test2" ),
					"Fehler beim Laden der Attribute 5" );
		}
		return TRUE;
	}
	return FALSE;
}

//------------------------------------------------------------------

BOOL __EXPORT TestDoc::Insert( SvStorage * pStor )
{
	InfoBox(0, String("Inserting ") + pStor->GetName()).Execute();
	return FALSE;
}

//------------------------------------------------------------------

void __EXPORT TestDoc::Draw( OutputDevice * pDev,
					  const JobSetup & rSetup, USHORT nAspect )
{
	if( nAspect == ASPECT_CONTENT )
	{
		pDev->DrawText(Point(), "Hallo");
		SFX_APP()->SpoilDemoOutput(
			*pDev, Rectangle(Point(0,0),pDev->GetOutputSize()));
		/*
		SfxItemSet aSet( GetPool() );
		GetAttr( aSet );
		USHORT nAlign = ((const TestAlign &)aSet.Get( ATR_ALIGNMENT )).GetValue();

		ULONG nWidth = rBound.GetWidth();
		USHORT nLine = 0;
		ULONG nHeight = 150;
		for ( SfxObjectShell* pDoc = SfxObjectShell::GetFirst();
			  pDoc;
			  pDoc = SfxObjectShell::GetNext(*pDoc) )
		{
			Font aNewFont( FAMILY_SWISS, Size( 0, 320 ) );
			aNewFont.SetTransparent(TRUE);
			Font aFont = pDev->SetFont( aNewFont );
			String aText( pDoc->GetTitle() );
			USHORT nTSize = pDev->GetTextSize( aText ).Width();
			USHORT nTPos  = 0;
			if( nAlign == SW_ALIGNCENTER )
				nTPos = (nWidth - nTSize) / 2;
			else if( nAlign == SW_ALIGNRIGHT )
				nTPos = nWidth - nTSize;
			pDev->DrawText( Point( nTPos, nHeight ), aText );
			nHeight += pDev->GetTextSize( aText ).Height();
			pDev->SetFont( aFont );

			for ( SfxMDIFrame* pFrame = SfxMDIFrame::GetFirst(pDoc);
				  pFrame;
				  pFrame = SfxMDIFrame::GetNext(*pFrame, pDoc) )
			{
				aText = pFrame->GetWindow().GetText();
				nTSize = pDev->GetTextSize( aText ).Width();
				nTPos  = 0;
				if( nAlign == SW_ALIGNCENTER )
					nTPos = (nWidth - nTSize) / 2;
				else if( nAlign == SW_ALIGNRIGHT )
					nTPos = nWidth - nTSize;
				pDev->DrawText( Point( nTSize, nHeight ), aText );
				nHeight += pDev->GetTextSize( aText ).Height();
			}
		}*/
//        if( GetIPClient() )
//            if( pDev == this )
//                GetIPClient()->Paint();
//            else
//                GetIPClient()->Draw( pDev, FALSE );
	}
}

// ------------------------------------------------------------------------

SFX_IMPL_SIMPLE_OBJECTFACTORY(Editor,sedit0)
{
	SFX_FILTER_REGISTRATION( "Sfx Editor", "*.hxx",
							SFX_FILTER_IMPORT|SFX_FILTER_EXPORT,
							0, 0x00000000L, "Sourcen", 0, "" );
}

BOOL __EXPORT Editor::ConvertFrom( SfxMedium &rMedium )
{
	SvStream *pStream = rMedium.GetInStream();
	if(pStream) {
		pStream->SetBufferSize(0);
		String aBuffer;
		while(pStream->ReadLine(aBuffer)) {
			aBuffer += '\n';
			aText += aBuffer;
		}
		aText.ConvertLineEnd();
//      rMedium.CloseInStream();    // File gelockt halten
		return SVSTREAM_OK == rMedium.GetError();
	}
	return FALSE;
}

BOOL __EXPORT Editor::ConvertTo( SfxMedium &rMedium )
{
	SvStream* pStream = rMedium.GetOutStream();
	if(pStream) {
		pStream->SetBufferSize(0);
		aText.ConvertLineEnd(LINEEND_LF);
		const USHORT nCount = aText.GetTokenCount('\n');
		for(USHORT i = 0; i < nCount; ++i) {
			const String aLine(aText.GetToken(i, '\n'));
			pStream->WriteLine(aLine);
		}
		rMedium.CloseOutStream();
		return SVSTREAM_OK == rMedium.GetError();
	}
	return FALSE ;
}

Editor::Editor( SfxShellCreateMode eMode )
:	SfxObjectShell(eMode),
	pPool(0)
{
	pPool = new SfxItemPool("_SFxEditor", 1,1, aStaticDefaults);
	SetPool(pPool);
}

__EXPORT Editor::~Editor()
{
	delete pPool;
}
