/*************************************************************************
 *
 *  $RCSfile: testdoc.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:38 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _TESTDOC_HXX
#define _TESTDOC_HXX

#include <sfx.hxx>
#include <sfxdoc.hxx>
#include <sfxitems.hxx>
#include "sample.hxx"
#include "sample.hrc"

#define SW_ALIGNLEFT    0
#define SW_ALIGNRIGHT   1
#define SW_ALIGNCENTER  2
#define SW_ALIGNBLOCK   3

//==================================================================

class TestAlign: public SfxEnumItem
{
public:
	TestAlign(int nVal = SW_ALIGNLEFT):
		SfxEnumItem(ATR_ALIGNMENT,nVal) {}
	TestAlign( SvStream &rStream ):
		SfxEnumItem(ATR_ALIGNMENT, rStream) {}

	virtual USHORT          GetValueCount() const;
	virtual String          GetValueText( USHORT nVal ) const;

	virtual SfxPoolItem*    Clone( SfxItemPool *pPool ) const;
	virtual SfxPoolItem*    Create(SvStream &, USHORT nVersion) const;
};

//==================================================================

// To show how a document-type can have special functions (which are
// accessible through the Dispatcher) we derive a document class.

class TestAttrib;
class SfxBoolItem;

#define TESTDOC_NAME    "TestDocument"
class TestDoc: public SfxObjectShell, public SfxInPlaceObject
{
	friend class TestAction;

private:
	SfxItemPool*		pPool;
	SfxStyleSheetPool*	pStyleSheetPool;
	SfxUndoManager* 	pUndoManager;
	const TestAttrib*	pAttr;
	SfxItemSet* 		pAttrSet;
	USHORT              nTool;
	SvStorageStreamRef  aDocStm;

	USHORT              DoEdit(const String &rName, USHORT nFamily);
	USHORT              New(USHORT nFamily);
	USHORT              Delete(const String &, USHORT nFamily);
	USHORT              Edit(const String &rName, USHORT nFamily);
protected:
	void                SetAttr( const TestAttrib *pOldAttrib );
	void                PutAttr( const SfxItemSet &rSet );

	virtual ToolBox*    CreateObjectBar( Window *pParent ) const;
	void                FillClass( SvUniqueName * pClassName,
								   ULONG * pClipFormat,
								   String * pLongUserName,
								   String * pUserName ) const;
	virtual BOOL        InitNew( SvStorage * );
	virtual BOOL        Insert( SvStorage * );
	virtual BOOL        Load( SvStorage * );
	virtual BOOL        Save();
	virtual BOOL        SaveAs( SvStorage * pNewStor );
	virtual BOOL        SaveCompleted( SvStorage * pNewStor );

	virtual BOOL        ConvertFrom( SfxMedium &rMedium );
	virtual BOOL        ConvertTo( SfxMedium &rMedium );
						~TestDoc();
public:
	virtual void        Draw( OutputDevice *,
							  const JobSetup & rSetup,
							  USHORT nAspect = ASPECT_CONTENT );
	void                GetAttr( SfxItemSet &rSet );
//    virtual BOOL        InitNew( SvDataObject*, SvStorage * );

						TestDoc(SfxShellCreateMode = SHELL_CREATE_MODE_EMBEDDED);
						SFX_DECL_INTERFACE(SAMPLE_INTERFACE_DOC);
						SFX_DECL_DOCUMENTFACTORY(TestDoc);

	void                Execute( SfxRequest& rReq );
	void                ExecTempl( SfxRequest& rReq );
	void                GetState(SfxItemSet &);

	void                ExecAttr( SfxRequest& rReq );
	void                GetAttrState(SfxItemSet &);

	void                ExecTool( SfxRequest& rReq );
	void                GetToolState(SfxItemSet &);

	Font                GetFont( USHORT nSize ) const;
	USHORT              GetAlign() const;

	virtual SfxUndoManager *GetUndoManager();
	virtual SfxStyleSheetBasePool*  GetStyleSheetPool();

	virtual StatusBar*	CreateStatusBar();
};

class Editor: public SfxObjectShell
{
private:
	SfxItemPool         *pPool;
	String              aText;

protected:
	virtual BOOL        ConvertFrom( SfxMedium &rMedium );
	virtual BOOL        ConvertTo( SfxMedium &rMedium );
public:
						SFX_DECL_SIMPLE_OBJECTFACTORY(Editor);
						Editor( SfxShellCreateMode eMode );
						~Editor();

	void                SetText( const String &rText ) {
						aText = rText;
						SetModified();
	}
	String              GetText() const { return aText; }
};


#endif

