/*************************************************************************
 *
 *  $RCSfile: testview.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:52:38 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _TESTVIEW_HXX
#define _TESTVIEW_HXX

#include <sfx.hxx>
#include <sfxitems.hxx>

#include "testdoc.hxx"

//==================================================================

// We define a special MDIWindow (SfxDocumentView) derivation.

class TestView: public SfxViewShell, public Window
{
	Size            aTotalSize;

private:
	void            Construct();       // common for all constrructors

protected:
	const Size&     GetTotalSize() const { return aTotalSize; }

	virtual void    Paint( const Rectangle& rRect );

	virtual void    OuterResizePixel( const Point &rOfs, const Size &rSize );
	virtual void    InnerResizePixel( const Point &rOfs, const Size &rSize );
	virtual Size    GetOptimalSizePixel() const;

	// to get notifications we overload Listener::Notify()
	virtual void    Notify( SfxBroadcaster& rBC, const TypeId& rBCType,
							const SfxHint& rHint, const TypeId& rHintType );

	virtual void    MouseButtonUp( const MouseEvent& rMEvt );
public:
					TestView( SfxViewFrame *pFrame, const TestView& rWin );
					TestView( SfxViewFrame *pFrame, SfxViewShell *pOld );
					~TestView();
};

//==================================================================

class DefaultView: public TestView
{
	USHORT			nTool;
	SfxShell		*pRect, *pCirc, *pSel, *pTool;

private:
	void			Construct();

protected:
	virtual void    MouseButtonDown( const MouseEvent &rEvt );

public:
					SFX_DECL_INTERFACE(SAMPLE_INTERFACE_DEFAULT_VIEW);
					SFX_DECL_VIEWFACTORY(DefaultView);

					DefaultView( SfxViewFrame *pFrame, const DefaultView& rView );
					DefaultView( SfxViewFrame *pFrame, SfxViewShell *pOld );
					~DefaultView();

	void			ExecTool( SfxRequest& rReq );
	void			GetToolState(SfxItemSet &);
};

//==================================================================

class SpecialView: public TestView
{
	Point               aMousePos;
	SfxPrinter         *pPrinter;

protected:
	virtual void    MouseMove( const MouseEvent &rEvt );
	virtual void    MouseButtonDown( const MouseEvent &rEvt );

	virtual void    Paint( const Rectangle& rRect );

	// Printing Interface
	virtual USHORT              Print();
	virtual SfxPrinter*         GetPrinter();
	virtual USHORT              SetPrinter( SfxPrinter *pNewPriter, USHORT nDiffFlags );
	virtual SfxTabPage*         CreatePrintOptionsPage( Window *pParent,
														const SfxItemSet &rOptions );

public:
					SFX_DECL_VIEWFACTORY(SpecialView);
					SFX_DECL_INTERFACE(SAMPLE_INTERFACE_SPECIAL_VIEW);

					SpecialView( SfxViewFrame *pFrame, const SpecialView& rView );
					SpecialView( SfxViewFrame *pFrame, SfxViewShell *pOld );
					~SpecialView();

	void            Execute( SfxRequest& rReq );
	void            GetState(SfxItemSet &);
};

//------------------------------------------------------------------

class RectShell: public SfxShell
{
	SfxItemPool         *pPool;

public:
						SFX_DECL_INTERFACE(SAMPLE_INTERFACE_RECT);
						RectShell( SfxItemPool &rPool ):
							pPool( &rPool ) { SetPool(pPool); }
};

//------------------------------------------------------------------

class CircShell: public SfxShell
{
	SfxItemPool         *pPool;

public:
						SFX_DECL_INTERFACE(SAMPLE_INTERFACE_CIRCLE);
						CircShell( SfxItemPool &rPool ):
							pPool( &rPool ) { SetPool(pPool); }
};

//==================================================================
class EditorView: public SfxViewShell
{
	MultiLineEdit	aMultiLineEdit;
protected:
	virtual void    OuterResizePixel( const Point &rOfs, const Size &rSize );
	virtual void    InnerResizePixel( const Point &rOfs, const Size &rSize );

public:
					SFX_DECL_VIEWFACTORY(EditorView);

					EditorView( SfxViewFrame *pFrame, const EditorView &);
					EditorView( SfxViewFrame *pFrame, SfxViewShell *pOld);

	DECL_LINK( ModifyHdl, MultiLineEdit * );
};



#endif

