/*************************************************************************
 *
 *  $RCSfile: ibase.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_IBASE_HXX
#define _SIM_IBASE_HXX

#include <vos/macros.hxx>
#include <vcl/bitmapex.hxx>
#include <vcl/salbtype.hxx>

// -----------
// - Defines -
// -----------

#define SIM_MERGE_COLOR( _def_c1, _def_c2, _def_cMerge ) \
	((BYTE)(((long)(_def_c1)*(_def_cMerge)+(_def_c2)*(255L-(_def_cMerge)))/255L))

// ---------------
// - IBase -
// ---------------

class SfxViewFrame;
class SfxProgress;

class IBase
{
private:

	Bitmap				aInitBitmap;
	HPBYTE				pArray1;
	HPBYTE				pArray2;
	HPBYTE				pArray3;
	HPBYTE				pArray4;
	HPBYTE				pOrgArray;
	HPBYTE				pDestArray;
	HPBYTE				pSrcPointer;
	BitmapPalette*		pPal1;
	BitmapPalette*		pPal2;
	BitmapPalette*		pPal3;
	BitmapPalette*		pPal4;
    ULONG				nWidth1;
    ULONG				nWidth2;
    ULONG				nWidth3;
    ULONG				nWidth4;
    ULONG				nHeight1;
    ULONG				nHeight2;
    ULONG				nHeight3;
    ULONG				nHeight4;
    ULONG				nAlignedWidth1;
    ULONG				nAlignedWidth2;
    ULONG				nAlignedWidth3;
    ULONG				nAlignedWidth4;
    ULONG				nWhichOrg;
	ULONG				nLastAccessTime;
	long				nSrcWidth;
	long				nSrcHeight;
	long				nSrcWidthAl;
    USHORT				nOrgDepth;
    USHORT				nUndoDepth;
    USHORT				nRedoDepth;
    BOOL				bIsUndo;
    BOOL				bIsRedo;
    BOOL				bCreateUndo;
    BOOL				bValid;
	BOOL				bInProcessing;

    BOOL				ImplConstruct();
	void				ImplBitmapToArray24();

protected:

	BYTE				mcR, mcG, mcB, mcMerge;

						IBase();
                        IBase( const IBase& rIBase );
    const IBase&		operator=( const IBase& rIBase );

public:

						IBase( const Bitmap& rBitmap );
						~IBase();

    BOOL				IsValid() { return bValid; }

    ULONG				GetOrgWidth() const { return nWhichOrg == 1 ? nWidth1 : nWidth2; }
    ULONG				GetDestWidth() const { return nWhichOrg == 1 ? nWidth2 : nWidth1; }

    ULONG				GetOrgHeight() const { return nWhichOrg == 1 ? nHeight1 : nHeight2; }
    ULONG				GetDestHeight() const { return nWhichOrg == 1 ? nHeight2 : nHeight1; }

    ULONG				GetOrgAlignedWidth() const { return nWhichOrg == 1 ? nAlignedWidth1 : nAlignedWidth2; }
    ULONG				GetDestAlignedWidth() const { return nWhichOrg == 1 ? nAlignedWidth2 : nAlignedWidth1; }

    ULONG				GetOrgAlignedSize() const { return GetOrgAlignedWidth() * GetOrgHeight(); }
    ULONG				GetDestAlignedSize() const { return GetDestAlignedWidth() * GetDestHeight(); }

    // Farbtiefe des Ausgangsbildes ermitteln und setzen
    USHORT				GetDepth() const { return nOrgDepth; }
    void				SetDepth( USHORT nDepth ) { nRedoDepth = nOrgDepth = nDepth; }

    // Vor- und Ruecklauf der Bildverarbeitung
    BOOL				BeginProcessing( BOOL bUndo = TRUE );
    void				EndProcessing();

    BOOL				BeginProcessingExt(ULONG nWidth, ULONG nHeight, BOOL bUndo = TRUE);
    void				EndProcessingExt() { EndProcessing(); }

	BOOL				IsInProcessing() const { return bInProcessing; }

    // Zeiger auf Arrays zur Verfuegung stellen
    HPBYTE				GetOrgPointer() { return pOrgArray; }
    HPBYTE				GetDestPointer() { return pDestArray; }

    // Undo-Verwaltung
    BOOL				DoUndo( SfxProgress* pProgress = NULL );
    BOOL				DoRedo( SfxProgress* pProgress = NULL );

	BitmapPalette*		GetPalette() const { return nWhichOrg == 1 ? pPal1 : pPal2; }
    Bitmap				GetBitmap();

	ULONG				GetLastAccessTime() const { return nLastAccessTime; }

	// Pointer usw. auf zu paintendes Array zur Verfuegung stellen
	HPBYTE				GetSrcPointer() const { return pSrcPointer; }
	long				GetSrcWidth() const { return nSrcWidth; }
	long				GetSrcHeight() const { return nSrcHeight; }
	long				GetSrcWidthAl() const { return nSrcWidthAl; }

	// Falls ::IsInProcessing()==TRUE, direkter Pixelzugriff moeglich
	HPBYTE				GetProcPointer( long nY, long nX )
						{
							return( pSrcPointer + nSrcWidthAl * ( nSrcHeight - nY - 1L ) + 3 * nX );
						}

	void				SetProcColor( const Color& rCol ) { mcR = rCol.GetRed(), mcG = rCol.GetGreen(), mcB = rCol.GetBlue(); }

	void				SetProcPixel( long nY, long nX )
						{
							if( nX > -1L && nX < nSrcWidth && nY > -1L && nY < nSrcHeight )
							{
								HPBYTE pTmp = GetProcPointer( nY, nX );
								*pTmp++ = mcB, *pTmp++ = mcG, *pTmp = mcR;
							}
						}

	void				SetProcPixel( long nY, long nX, const Color& rCol )
						{
							if( nX > -1L && nX < nSrcWidth && nY > -1L && nY < nSrcHeight )
							{
								HPBYTE pTmp = GetProcPointer( nY, nX );
								*pTmp++ = rCol.GetBlue(), *pTmp++ = rCol.GetGreen(), *pTmp = rCol.GetRed();
							}
						}

	void				SetProcPixel( long nY, long nX, const BitmapColor& rCol )
						{
							if( nX > -1L && nX < nSrcWidth && nY > -1L && nY < nSrcHeight )
							{
								HPBYTE pTmp = GetProcPointer( nY, nX );
								*pTmp++ = rCol.GetBlue(), *pTmp++ = rCol.GetGreen(), *pTmp = rCol.GetRed();
							}
						}

	void				MergeProcPixel( long nY, long nX, const Color& rCol )
						{
							if( nX > -1L && nX < nSrcWidth && nY > -1L && nY < nSrcHeight )
							{
								HPBYTE pTmp = GetProcPointer( nY, nX );
								*pTmp++ = SIM_MERGE_COLOR( *pTmp, rCol.GetBlue(), mcMerge );
								*pTmp++ = SIM_MERGE_COLOR( *pTmp, rCol.GetGreen(), mcMerge );
								*pTmp = SIM_MERGE_COLOR( *pTmp, rCol.GetRed(), mcMerge );
							}
						}

	void				MergeProcPixel( long nY, long nX, const BitmapColor& rCol )
						{
							if( nX > -1L && nX < nSrcWidth && nY > -1L && nY < nSrcHeight )
							{
								HPBYTE pTmp = GetProcPointer( nY, nX );
								*pTmp++ = SIM_MERGE_COLOR( *pTmp, rCol.GetBlue(), mcMerge );
								*pTmp++ = SIM_MERGE_COLOR( *pTmp, rCol.GetGreen(), mcMerge );
								*pTmp = SIM_MERGE_COLOR( *pTmp, rCol.GetRed(), mcMerge );
							}
						}

	BOOL				GetProcPixelColor( long nY, long nX, Color& rCol )
						{
							if( nX > -1L && nX < nSrcWidth && nY > -1L && nY < nSrcHeight )
							{
								HPBYTE pTmp = GetProcPointer( nY, nX );
								rCol.SetBlue( *pTmp++ ), rCol.SetGreen( *pTmp++ ), rCol.SetRed( *pTmp );
								return TRUE;
							}
							else
								return FALSE;
						}
};

#endif // _SIM_IBASE_HXX
