/*************************************************************************
 *
 *  $RCSfile: simframe.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_SIMFRAME_HXX
#define _SIM_SIMFRAME_HXX

#include <vcl/animate.hxx>
#include <svtools/itemset.hxx>

// -----------
// - Defines -
// -----------

#define AnimBmp					AnimationBitmap
#define ANIMATION_NO_TIMEOUT	0xffffffffUL

// ------------
// - SimFrame -
// ------------

class SvImage;
class SimModel;

class SimFrame
{
	friend class SimModel;

private:

	SfxItemSet			maPrintAttr;
	AnimBmp				maAnimBmp;
	Size				maOriginalSize;
	SimModel*			mpModel;
	SvImage*			mpImage;
	SvImage*			mpAlpha;
	ULONG				mnAccCount;
	ULONG				mnSyncTime;

	void				ImplIncAccCount();
	void				ImplDecAccCount( BOOL bSynchronize );
	BOOL				ImplIsAccessed() const { return( mnAccCount > 0UL ); }
	void				ImplSynchronize();
	void				ImplUpdatePrefSize() const;

	const SimFrame&		operator=( const SimFrame& rFrame ) { return *this; }
						
public:

						SimFrame( SimModel* pModel, const BitmapEx& rBmpEx, 
								  ULONG nWaitMS = 0UL, Disposal eDisposal = DISPOSE_BACK );
						SimFrame( SimModel* pModel, const BitmapEx& rBmpEx, 
								  const Point& rDisplayPosPix, const Size& rDisplaySizePix,
								  ULONG nWaitMS = 0UL, Disposal eDisposal = DISPOSE_BACK );
						SimFrame( SimModel* pModel, const AnimBmp& rAnimBmp );

	virtual				~SimFrame();

	SimModel*			GetModel() const { return mpModel; }
	SfxItemSet&			GetPrintAttr() { return maPrintAttr; }
	const SfxItemSet&	GetPrintAttr() const { return maPrintAttr; }

	BitmapEx			GetBitmapEx() const;
	AnimBmp				GetAnimBmp() const;

	Size				GetSizePixel() const;
	USHORT				GetColorDepth() const;
	BOOL				IsTransparent() const;

	const Size&			GetPrefSize() const { ImplUpdatePrefSize(); return maAnimBmp.aBmpEx.GetPrefSize(); }
	void				SetPrefSize( const Size& rPrefSize ) { maOriginalSize = GetSizePixel(); maAnimBmp.aBmpEx.SetPrefSize( rPrefSize ); }

	const MapMode&		GetPrefMapMode() const { return maAnimBmp.aBmpEx.GetPrefMapMode(); }
	void				SetPrefMapMode( const MapMode& rPrefMapMode ) { maAnimBmp.aBmpEx.SetPrefMapMode( rPrefMapMode ); }

	const Point&		GetDisplayPosPixel() const { return maAnimBmp.aPosPix; }
	void				SetDisplayPosPixel( const Point& rPosPix ) { maAnimBmp.aPosPix = rPosPix; }
	
	const Size&			GetDisplaySizePixel() const { return maAnimBmp.aSizePix; }
	void				SetDisplaySizePixel( const Size& rSizePix ) { maAnimBmp.aSizePix = rSizePix; }
	
	ULONG				GetWaitMS() const;
	void				SetWaitMS( ULONG nWaitMS );
	
	Disposal			GetDisposal() const { return maAnimBmp.eDisposal; }
	void				SetDisposal( Disposal eDisposal ) { maAnimBmp.eDisposal = eDisposal; }

	BOOL				Acquire();
	void				Release();
	void				ReleaseQuick();

	SvImage*			GetImage() const { return mpImage; }
	SvImage*			GetAlpha() const { return mpAlpha; }
};

// -------------
// - SimFrames -
// -------------

class SimFrames : protected List
{
public:
			
						SimFrames();
	virtual				~SimFrames();

	void				Clear();

	ULONG				GetFrameCount() const;
	SimFrame*			GetFrame( ULONG nFramePos ) const ;
	void				InsertFrame( SimFrame* pFrame, ULONG nFramePos = LIST_APPEND );
	void				RemoveFrame( ULONG nFramePos );
	void				ReplaceFrame( SimFrame* pFrame, ULONG nFramePos );
};

#endif // _SIM_SIMFRAME_HXX
