/*************************************************************************
 *
 *  $RCSfile: simmod.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_SIMMOD_HXX
#define _SIM_SIMMOD_HXX

#ifndef _SIMDLL_HXX
#define _SIM_DLL             // fuer SIM_MOD()
#include "simdll.hxx"        // fuer SimModuleDummy
#endif
#include <vcl/graph.hxx>
#include <sfx2/cfgitem.hxx>
#include "glob.hxx"

#ifndef _COM_SUN_STAR_LANG_XEVENTLISTENER_HPP_
#include <com/sun/star/lang/XEventListener.hpp>
#endif
#ifndef _COM_SUN_STAR_SCANNER_XSCANNERMANAGER_HPP_
#include <com/sun/star/scanner/XScannerManager.hpp>
#endif

// ---------------
// - SimDefaults -
// ---------------

class SimDefaults : public SfxConfigItem
{

			void	Init();
	virtual int 	Load( SvStream& );
	virtual BOOL	Store( SvStream& );
	virtual void	UseDefault();

public:

	float			fRotation;
	BOOL			bRotOrient;
	float			fScaleX;
	float			fScaleY;
	float			fRedAdjust;
	float			fGreenAdjust;
	float			fBlueAdjust;
	float			fBrightness;
	float			fContrast;
	float			fBWThreshold;
	float			fAge;
	float			fSolarThreshold;
	BOOL			bSolarInvert;
	USHORT			nGrayCount;
	USHORT			nMosaicX;
	USHORT			nMosaicY;
	DefinitionType	eMosaicSharpness;
	USHORT			nPosterCount;
	USHORT			nTileX;
	USHORT			nTileY;
	DefinitionType	eSharpness;

					SimDefaults();
	virtual			~SimDefaults() {}
};

/*************************************************************************
|*
|* This subclass of <SfxModule> (which is a subclass of <SfxShell>) is
|* linked to the DLL. One instance of this class exists while the DLL is
|* loaded.
|*
|* SimModule is like to be compared with the <SfxApplication>-subclass.
|*
|* Remember: Don`t export this class! It uses DLL-internal symbols.
|*
\************************************************************************/

class SvFactory;

class SimModule : public SimModuleDummy
{
private:
	Graphic					aNewDocGraphic;
	Timer					aNewDocTimer;
 	SimDefaults*			pDefaults;
	SfxPoolItem**			ppPoolDefaults;

private:
	
	::com::sun::star::uno::Reference< ::com::sun::star::scanner::XScannerManager >	mxScannerManager;
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener >		mxScannerListener;

private:

    void					InitPoolDefaults();
	void					DestroyPoolDefaults();

	virtual SfxFileDialog*  CreateDocFileDialog( ULONG nBits, const SfxObjectFactory& rFact,
												 const SfxItemSet* pSet = NULL );
	virtual SfxItemSet*		CreateItemSet( USHORT nId );
	virtual void			ApplyItemSet( USHORT nId, const SfxItemSet& rSet );
	virtual SfxTabPage*		CreateTabPage( USHORT nId, Window* pParent, const SfxItemSet& rSet );

							DECL_LINK( NewDocHdl, Timer* );


public:
							TYPEINFO();
							SFX_DECL_INTERFACE( SCH_IF_APP );

							SimModule( SvFactory* pObjFact );
    virtual					~SimModule();

	virtual void			FillStatusBar( StatusBar& rBar );

    void					Execute( SfxRequest& rReq );
    void					GetState( SfxItemSet& );

	virtual SfxModule*		Load();
	virtual void			Free();

	void					CreateNewDocWithGraphic( const Graphic& rGraphic );

	void					ScannerEvent( const ::com::sun::star::lang::EventObject& rEventObject );

	SimDefaults*			GetDefaults() const { return pDefaults; }
	SfxPoolItem**			GetPoolDefaults() const { return ppPoolDefaults; }
};


#ifndef SIM_MOD
#define SIM_MOD() ( *(SimModule**) GetAppData( SHL_SIM ) )
#endif

#endif // _SIM_SIMMOD_HXX
