/*************************************************************************
 *
 *  $RCSfile: simmodel.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_SIMMODEL_HXX
#define _SIM_SIMMODEL_HXX

#include <tools/gen.hxx>
#include <vcl/mapmod.hxx>
#include <svtools/brdcst.hxx>
#include <svtools/itempool.hxx>
#include <svx/rectenum.hxx>
#include "glob.hxx"
#include "simframe.hxx"

// -----------
// - Defines -
// -----------

#define FRAME_APPEND				LIST_APPEND
#define FRAME_POS_ALL				0xfffffffeUL
#define FRAME_POS_INVALID			0xffffffffUL

// ----------------
// - Hint-Defines -
// ----------------

#define SIM_HINT_NONE				0x00000000UL
#define SIM_HINT_FLUSHFRAME			0x00000001UL
#define SIM_HINT_FRAMEINSERTED		0x00000002UL
#define SIM_HINT_FRAMEREPLACED		0x00000004UL
#define SIM_HINT_FRAMEREMOVED		0x00000008UL
#define SIM_HINT_REPAINTFRAME		0x00000010UL
#define SIM_HINT_REINITFRAME		0x00000020UL
#define SIM_HINT_MODELCHANGED		0x01000000UL

// -----------
// - SimHint -
// -----------

class SimHint : public SfxHint
{
private:
	
	Rectangle			maRect;
	ULONG				mnType;
	ULONG				mnFramePos;

public:

						TYPEINFO();
		
						SimHint( ULONG nType = SIM_HINT_NONE, ULONG nFramePos = FRAME_POS_INVALID ) : 
							mnType( nType ), mnFramePos( nFramePos ) {}

						SimHint( ULONG nType, const Rectangle& rRect, ULONG nFramePos = FRAME_POS_INVALID ) : 
							maRect( rRect ), mnType( nType ), mnFramePos( nFramePos ) {}

	ULONG				GetType() const { return mnType; }
	ULONG				GetFramePos() const { return mnFramePos; }
	const Rectangle&	GetRect() const { return maRect; }
};

// ------------
// - SimModel -
// ------------

class SfxObjectShell;
class XOutdevItemPool;
class XColorTable;
class SimDefaults;
class Graphic;
class SimFrames;
class SimFrame;

class SimModel : public SfxBroadcaster
{
private:

	SimFrames*				mpFrames;
	SfxObjectShell*			mpObjShell;
	SfxItemPool*			mpItemPool;
	XOutdevItemPool*		mpXOutPool;
	XColorTable*			mpColorTable;
	SfxItemInfo*			mpItemInfo;
	SimDefaults*			mpDefaults;
	long					mnModified;
							
	void					ImplConstruct();
							
public:						
							
							TYPEINFO();
							
							SimModel( SfxObjectShell* pObjSh = NULL );
							SimModel( const Graphic& rGraphic, SfxObjectShell* pObjSh = NULL );
	virtual					~SimModel();
							
	virtual void			SetChanged( BOOL bFlag = TRUE );

	SfxObjectShell*			GetObjectShell() { return mpObjShell; }

	void					SendRepaintBroadcast( ULONG nFramePos, 
												  BOOL bFrameSizeChanged = FALSE );
	void					SendRepaintBroadcast( ULONG nFramePos, const Rectangle& rRect,
												  BOOL bFrameSizeChanged = FALSE );
							
	ULONG					GetFrameCount() const;
	SimFrame*				GetFrame( ULONG nFramePos ) const;
	void					InsertFrame( SimFrame* pFrame, ULONG nFramePos = FRAME_APPEND );
	BOOL					RemoveFrame( ULONG nFramePos );
	BOOL					ReplaceFrame( SimFrame* pFrame, ULONG nFramePos );
							
	void					SetGraphic( const Graphic& rGraphic );
	Graphic					GetGraphic() const;
							
	void					SetDefaults( SimDefaults* pDef ) { mpDefaults = pDef; }
	SimDefaults*			GetDefaults() { return mpDefaults; }
							
	void					IncModificationCount() { mnModified++; }
	void					DecModificationCount() { mnModified--; }
	long					GetModificationCount()	const { return mnModified; }
							
    SfxItemPool*			GetPool() const { return mpItemPool; }
	XOutdevItemPool*		GetXOutPool() const { return mpXOutPool; }
	XColorTable*			GetColorTable() const { return mpColorTable; }
							
	friend SvStream&		operator<<( SvStream& rOut, const SimModel& rModel );
	friend SvStream&		operator>>( SvStream& rIn, SimModel& rModel );
};

#endif // _SIM_SIMMODEL_HXX
