/*************************************************************************
 *
 *  $RCSfile: simframe.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <tools/debug.hxx>	
#include <vcl/svapp.hxx>
#include <vcl/wrkwin.hxx>
#include <svtools/intitem.hxx>
#include <svimage.hxx>
#include "simattr.hxx"
#include "simmodel.hxx"
#include "simframe.hxx"


// ------------
// - SimFrame -
// ------------
						
SimFrame::SimFrame( SimModel* pModel, const BitmapEx& rBmpEx, 
					ULONG nWaitMS, Disposal eDisposal ) :
	maAnimBmp		( rBmpEx, Point(), rBmpEx.GetSizePixel(), nWaitMS / 10, eDisposal ),
	maOriginalSize	( rBmpEx.GetSizePixel() ),
	mpModel			( pModel ),
	maPrintAttr		( *pModel->GetPool(), ATTR_PRINTOPT_START, ATTR_PRINTOPT_END ),
	mpImage			( NULL ),
	mpAlpha			( NULL ),
	mnAccCount		( 0UL ),
	mnSyncTime		( 0UL )
{
}

// -----------------------------------------------------------------------------

SimFrame::SimFrame( SimModel* pModel, const BitmapEx& rBmpEx, 
					const Point& rDisplayPosPix, const Size& rDisplaySizePix,
				    ULONG nWaitMS, Disposal eDisposal ) :
	maAnimBmp		( rBmpEx, rDisplayPosPix, rDisplaySizePix, nWaitMS / 10, eDisposal ),
	maOriginalSize	( rBmpEx.GetSizePixel() ),
	mpModel			( pModel ),
	maPrintAttr		( *pModel->GetPool(), ATTR_PRINTOPT_START, ATTR_PRINTOPT_END ),
	mpImage			( NULL ),
	mpAlpha			( NULL ),
	mnAccCount		( 0UL ),
	mnSyncTime		( 0UL )
{
}

// -----------------------------------------------------------------------------

SimFrame::SimFrame( SimModel* pModel, const AnimBmp& rAnimBmp ) :
	maAnimBmp		( rAnimBmp ),
	maOriginalSize	( rAnimBmp.aBmpEx.GetSizePixel() ),
	mpModel			( pModel ),
	maPrintAttr		( *pModel->GetPool(), ATTR_PRINTOPT_START, ATTR_PRINTOPT_END ),
	mpImage			( NULL ),
	mpAlpha			( NULL ),
	mnAccCount		( 0UL ),
	mnSyncTime		( 0UL )
{
}

// -----------------------------------------------------------------------------

SimFrame::~SimFrame()
{
	DBG_ASSERT( !mnAccCount, "Frame accessed in Dtor?" );

	delete mpImage;
	delete mpAlpha;
}

// -----------------------------------------------------------------------------

void SimFrame::ImplIncAccCount()
{
	if( !mnAccCount++ )
	{
		const BitmapEx& rBmpEx = maAnimBmp.aBmpEx;
		const Bitmap&	rBmp = rBmpEx.GetBitmap();

		if( !!rBmp )
		{
			mpImage = new SvImage( rBmp );
			mnSyncTime = mpImage->GetLastAccessTime();

			if( rBmpEx.IsTransparent() )
			{
				const Bitmap& rAlpha = rBmpEx.GetMask();

				if( !!rAlpha )
					mpAlpha = new SvImage( rAlpha );
			}

			const Size		aPrefSize( GetPrefSize() );
			const MapMode	aPrefMapMode( GetPrefMapMode() );

			maAnimBmp.aBmpEx = BitmapEx();

			SetPrefSize( aPrefSize );
			SetPrefMapMode( aPrefMapMode );
		}
		else
			mnAccCount = 0;
	}
}

// -----------------------------------------------------------------------------

void SimFrame::ImplDecAccCount( BOOL bSynchronize )
{
	DBG_ASSERT( mnAccCount, "Object is not accessed!" );

	if( !--mnAccCount )
	{
		if( bSynchronize )
			ImplSynchronize();

		delete mpImage, mpImage = NULL;
		delete mpAlpha, mpAlpha = NULL;
		mnSyncTime = 0UL;
	}
}

// -----------------------------------------------------------------------------

void SimFrame::ImplSynchronize()
{
	if( mpImage )
	{
		const Size		aPrefSize( GetPrefSize() );
		const MapMode	aPrefMapMode( GetPrefMapMode() );

		if( mpAlpha )
			maAnimBmp.aBmpEx = BitmapEx( mpImage->GetBitmap(), mpAlpha->GetBitmap() );
		else
			maAnimBmp.aBmpEx = mpImage->GetBitmap();

		SetPrefSize( aPrefSize );
		SetPrefMapMode( aPrefMapMode );
		mnSyncTime = mpImage->GetLastAccessTime();
	}
}

// -----------------------------------------------------------------------------

void SimFrame::ImplUpdatePrefSize() const
{
	const Size aNewSizePix( GetSizePixel() );

	if( ( aNewSizePix != maOriginalSize ) && maOriginalSize.Width() && maOriginalSize.Height() )
	{
		Size aPrefSize( maAnimBmp.aBmpEx.GetPrefSize() );

		aPrefSize.Width() = FRound( (double) aPrefSize.Width() * aNewSizePix.Width() / maOriginalSize.Width() );
		aPrefSize.Height() = FRound( (double) aPrefSize.Height() * aNewSizePix.Height() / maOriginalSize.Height() );
		( (SimFrame*) this)->SetPrefSize( aPrefSize );
		( (SimFrame*) this)->maOriginalSize = aNewSizePix;
	}
}

// -----------------------------------------------------------------------------

BitmapEx SimFrame::GetBitmapEx() const
{
	BitmapEx aRet;

	if( mpImage )
	{
		if( mpAlpha )
			aRet = BitmapEx( mpImage->GetBitmap(), mpAlpha->GetBitmap() );
		else
			aRet = mpImage->GetBitmap();

		aRet.SetPrefSize( GetPrefSize() );
		aRet.SetPrefMapMode( GetPrefMapMode() );
	}
	else
		aRet = maAnimBmp.aBmpEx;

	return aRet;
}

// -----------------------------------------------------------------------------

AnimBmp SimFrame::GetAnimBmp() const
{
	return( AnimBmp( GetBitmapEx(), maAnimBmp.aPosPix, maAnimBmp.aSizePix, maAnimBmp.nWait, maAnimBmp.eDisposal ) );
}

// -----------------------------------------------------------------------------

Size SimFrame::GetSizePixel() const
{
	Size aSize;

	if( mpImage )
	{
		aSize.Width() = mpImage->GetOrgWidth(); 
		aSize.Height() = mpImage->GetOrgHeight(); 
	}
	else
		aSize = maAnimBmp.aBmpEx.GetSizePixel();

	return aSize;
}

// -----------------------------------------------------------------------------

ULONG SimFrame::GetWaitMS() const
{
	if( ANIMATION_TIMEOUT_ON_CLICK == maAnimBmp.nWait )
		return ANIMATION_NO_TIMEOUT;
	else
		return( (ULONG) maAnimBmp.nWait * 10UL ); 
}

// -----------------------------------------------------------------------------

void SimFrame::SetWaitMS( ULONG nWaitMS )
{
	if( ANIMATION_NO_TIMEOUT == nWaitMS )
		maAnimBmp.nWait = ANIMATION_TIMEOUT_ON_CLICK;
	else
		maAnimBmp.nWait = (long) ( nWaitMS / 10L );
} 

// -----------------------------------------------------------------------------

USHORT SimFrame::GetColorDepth() const
{
	return( mpImage ? mpImage->GetDepth() : maAnimBmp.aBmpEx.GetBitmap().GetBitCount() );
}

// -----------------------------------------------------------------------------

BOOL SimFrame::IsTransparent() const
{
	return( ( mpImage && mpAlpha ) || maAnimBmp.aBmpEx.IsTransparent() );
}

// -----------------------------------------------------------------------------

BOOL SimFrame::Acquire()
{
	ImplIncAccCount();
	return( mpImage != NULL );
}

// -----------------------------------------------------------------------------

void SimFrame::Release()
{
	ImplDecAccCount( TRUE );
}

// -----------------------------------------------------------------------------

void SimFrame::ReleaseQuick()
{
	ImplDecAccCount( FALSE );
}

// -------------
// - SimFrames -
// -------------

SimFrames::SimFrames()
{
}

// -----------------------------------------------------------------------------

SimFrames::~SimFrames()
{
	Clear();
}

// -----------------------------------------------------------------------------

void SimFrames::Clear()
{
	for( ULONG nFramePos = 0UL, nFrameCount = GetFrameCount(); nFramePos < nFrameCount; nFramePos++ )
		delete (SimFrame*) List::GetObject( nFramePos );

	List::Clear();
}

// -----------------------------------------------------------------------------

ULONG SimFrames::GetFrameCount() const
{
	return List::Count();
}

// -----------------------------------------------------------------------------

SimFrame* SimFrames::GetFrame( ULONG nFramePos ) const
{
	return (SimFrame*) List::GetObject( nFramePos );
}

// -----------------------------------------------------------------------------

void SimFrames::InsertFrame( SimFrame* pFrame, ULONG nFramePos )
{
	List::Insert( pFrame, nFramePos );
}

// -----------------------------------------------------------------------------

void SimFrames::RemoveFrame( ULONG nFramePos )
{
	delete (SimFrame*) List::Remove( (ULONG) nFramePos );
}

// -----------------------------------------------------------------------------

void SimFrames::ReplaceFrame( SimFrame* pFrame, ULONG nFramePos )
{
	delete (SimFrame*) List::Replace( pFrame, nFramePos );
}
