/*************************************************************************
 *
 *  $RCSfile: simmodel.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <vcl/graph.hxx>
#include <vcl/salbtype.hxx>
#include <vcl/wrkwin.hxx>
#include <svtools/iniman.hxx>
#include <svx/xpool.hxx>
#include <svx/xtable.hxx>
#include <sfx2/app.hxx>
#include <sfx2/objsh.hxx>
#include "simattr.hxx"
#include "simmod.hxx"
#include "svimage.hxx"
#include "simmodel.hxx"

// -----------
// - SimHint -
// -----------

TYPEINIT1( SimHint, SfxHint );

// ------------
// - SimModel -
// ------------

TYPEINIT1( SimModel, SfxBroadcaster );

SimModel::SimModel( SfxObjectShell* pShell ) :
    mpObjShell( pShell )
{
	ImplConstruct();
}

// -----------------------------------------------------------------------------

SimModel::SimModel( const Graphic& rGraphic, SfxObjectShell* pShell ) :
    mpObjShell( pShell )
{
	ImplConstruct();
	SetGraphic( rGraphic );
}

// -----------------------------------------------------------------------------

SimModel::~SimModel()
{
	SimHint aHint( SIM_HINT_FLUSHFRAME, FRAME_POS_ALL );
	Broadcast( aHint );

	delete mpFrames;
	delete mpColorTable;

	if(  mpItemPool )
	{
		mpItemPool->SetSecondaryPool( NULL );
        delete mpItemPool;
	}

	delete mpXOutPool;
	delete mpItemInfo;
}

// -----------------------------------------------------------------------------

void SimModel::ImplConstruct()
{
	const USHORT nItemCount = ATTR_END - ATTR_START + 1;

	// init vars
	mnModified = 0;

	// create frame list
	mpFrames = new SimFrames;

    // Defaults vom Modul geben lassen
    mpDefaults = SIM_MOD()->GetDefaults();

	mpItemInfo = new SfxItemInfo[ nItemCount ];
	memset( mpItemInfo, 0, nItemCount * sizeof( SfxItemInfo ) );

	// Pools anlegen
	mpItemPool = new SfxItemPool( String( RTL_CONSTASCII_USTRINGPARAM( "SimItemPool" ) ), ATTR_START, ATTR_END, mpItemInfo );
	mpItemPool->SetDefaults( SIM_MOD()->GetPoolDefaults() );
	mpXOutPool = new XOutdevItemPool( mpItemPool );
	mpItemPool->FreezeIdRanges();

	// ColorTable erzeugen
	mpColorTable = new XColorTable( SFX_APP()->GetAppIniManager()->Get( SFX_KEY_PALETTE_PATH ), mpXOutPool );
}

// -----------------------------------------------------------------------------

void SimModel::SetChanged( BOOL bFlag )
{
    if( mpObjShell && mpObjShell->IsEnableSetModified() )
        mpObjShell->SetModified( bFlag );
}

// -----------------------------------------------------------------------------

void SimModel::SendRepaintBroadcast( ULONG nFramePos, BOOL bFrameSizeChanged )
{
	SimHint aHint( bFrameSizeChanged ? SIM_HINT_REINITFRAME : SIM_HINT_REPAINTFRAME, nFramePos );
	Broadcast( aHint );
}

// -----------------------------------------------------------------------------

void SimModel::SendRepaintBroadcast( ULONG nFramePos, const Rectangle& rRect, BOOL bFrameSizeChanged )
{
	SimHint aHint;

	if( bFrameSizeChanged )
		aHint = SimHint( SIM_HINT_REINITFRAME, nFramePos );
	else
		aHint = SimHint( SIM_HINT_REPAINTFRAME, rRect, nFramePos );

	Broadcast( aHint );
}

// -----------------------------------------------------------------------------

ULONG SimModel::GetFrameCount() const
{
	return mpFrames->GetFrameCount();
}

// -----------------------------------------------------------------------------

SimFrame* SimModel::GetFrame( ULONG nFramePos ) const
{
	DBG_ASSERT( nFramePos < GetFrameCount(), "FramePos out of range!" );
	return mpFrames->GetFrame( nFramePos );
}

// -----------------------------------------------------------------------------

void SimModel::InsertFrame( SimFrame* pFrame, ULONG nFramePos )
{
    const ULONG nNewFramePos = Min( nFramePos, GetFrameCount() );
	SimHint		aHint( SIM_HINT_FRAMEINSERTED, nNewFramePos );
	
	mpFrames->InsertFrame( pFrame, nFramePos );
	Broadcast( aHint );
}

// -----------------------------------------------------------------------------

BOOL SimModel::RemoveFrame( ULONG nFramePos )
{
	BOOL bRet = nFramePos < GetFrameCount();

	DBG_ASSERT( bRet, "FramePos out of range!" );

	if( bRet )
	{
        SimHint aHint1( SIM_HINT_FLUSHFRAME, nFramePos );
        SimHint aHint2( SIM_HINT_FRAMEREMOVED, nFramePos );

        Broadcast( aHint1 );
		mpFrames->RemoveFrame( nFramePos );
        Broadcast( aHint2 );
	}

	return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimModel::ReplaceFrame( SimFrame* pFrame, ULONG nFramePos )
{
	BOOL bRet = nFramePos < GetFrameCount();

	DBG_ASSERT( bRet, "FramePos out of range!" );

	if( bRet )
	{
        SimHint aHint1( SIM_HINT_FLUSHFRAME, nFramePos );
		SimHint aHint2( SIM_HINT_FRAMEREPLACED, nFramePos );

        Broadcast( aHint1 );
		mpFrames->ReplaceFrame( pFrame, nFramePos );
        Broadcast( aHint2 );
	}

	return bRet;
}

// -----------------------------------------------------------------------------

void SimModel::SetGraphic( const Graphic& rGraphic )
{
	SimHint aHint1( SIM_HINT_FLUSHFRAME, FRAME_POS_ALL );
	SimHint aHint2( SIM_HINT_MODELCHANGED, FRAME_POS_ALL );
	BOOL	bChanged = FALSE;

	Broadcast( aHint1 );
	mpFrames->Clear();

	if( rGraphic.GetType() != GRAPHIC_NONE )
	{
		if( rGraphic.IsAnimated() )
		{
			const Animation aAnimation( rGraphic.GetAnimation() );

			for( USHORT i = 0, nCount = aAnimation.Count(); i < nCount; i++ )
				mpFrames->InsertFrame( new SimFrame( this, aAnimation.Get( i ) ) );
		}
		else
			mpFrames->InsertFrame( new SimFrame( this, rGraphic.GetBitmapEx() ) );
	}

	Broadcast( aHint2 );
}

// -----------------------------------------------------------------------------

Graphic SimModel::GetGraphic() const
{
	Graphic		aGraphic;
	const ULONG	nCount = GetFrameCount();

	if( nCount )
	{
		if( 1 == nCount )
			aGraphic = mpFrames->GetFrame( 0 )->GetBitmapEx();
		else
		{
			Animation	aAnimation;
			Rectangle	aFramesRect;

			for( ULONG i = 0UL; i < nCount; i++ )
			{
				const AnimBmp aAnimBmp( mpFrames->GetFrame( i )->GetAnimBmp() );
				aAnimation.Insert( aAnimBmp );
				aFramesRect.Union( Rectangle( aAnimBmp.aPosPix, aAnimBmp.aSizePix ) );
			}

			aAnimation.SetDisplaySizePixel( aFramesRect.GetSize() );
			aGraphic = aAnimation;
		}
	}

	return aGraphic;
}

// -----------------------------------------------------------------------------

SvStream& operator<<( SvStream& rOut, const SimModel& rModel )
{
	Graphic aGraphic( rModel.GetGraphic() );

	if( aGraphic.GetType() != GRAPHIC_NONE )
		rOut << aGraphic;

    return rOut;
}

// -----------------------------------------------------------------------------

SvStream& operator>>( SvStream& rIn, SimModel& rModel )
{
	Graphic	aGraphic;

	rIn >> aGraphic;

	if( aGraphic.GetType() != GRAPHIC_NONE )
		rModel.SetGraphic( aGraphic );

    return rIn;
}
