/*************************************************************************
 *
 *  $RCSfile: svimage.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "svimage.hxx"
#include <sfx2/progress.hxx>
#include <sfx2/viewfrm.hxx>
#include "simresid.hxx"
#include "strings.hrc"

/******************************************************************************/

#define GRAYVAL( nB, nG, nR )	( (BYTE) ( ( (nB)*28L + (nG)*151L + (nR)*77L ) >> 8 ) )

/******************************************************************************/

extern USHORT nDither256[ 256 ];
extern ULONG nLookUp[ 256 ];
extern BYTE cMap2[2];
extern BYTE cMap6[6];
extern BYTE cMap16[16];

/******************************************************************************
|*
|*
|*
\******************************************************************************/

SvImage::SvImage( const Bitmap& rBitmap ) :
		IBase	( rBitmap ),
		bLineSet( TRUE )
{		
}

/******************************************************************************
|*
|*
|*
\******************************************************************************/

SvImage::~SvImage()
{
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::TransformLuminance(BYTE* pTransArray, ULONG nLast, SfxProgress* pProgress)
{
	long	nWidth = GetOrgWidth();
	long	nHeight = GetOrgHeight();
	long	nX;
	long	nY;
	HPBYTE 	pOrgArray = GetOrgPointer();
	HPBYTE 	pDestArray = GetDestPointer();
	BOOL	bCond = (BOOL) ( (nWidth * 3 ) & 3 );
	ULONG	nCond = 4 - ( (nWidth * 3 ) & 3 );
	BOOL	bRet = FALSE;

	if ( pOrgArray && pDestArray )
	{
		INT16*	pModArray = new INT16[ 256 ];
		INT16	nOff;
		INT16	nTmp;

		// Array anpassen, um Luminanz direkt aendern zu  koennen
		for ( INT16 i = 0; i < 256; i++ )
			pModArray[i] = (INT16) pTransArray[i] - i;

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				// RGB-Wert in Y-Wert umrechnen
				nOff = pModArray[ GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] ) ];

				nTmp = nOff + *pOrgArray++;
				*pDestArray++ = (BYTE) VOS_BOUND( nTmp, 0, 255 );

				nTmp = nOff + *pOrgArray++;
				*pDestArray++ = (BYTE) VOS_BOUND( nTmp, 0, 255 );

				nTmp = nOff + *pOrgArray++;
				*pDestArray++ = (BYTE) VOS_BOUND( nTmp, 0, 255 );
			}

 			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( ! ( nY % 4) && pProgress )
				pProgress->SetState( nY+nLast );
		}

		delete[] pModArray;
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::TransformLuminance(BYTE* pTransArray, const Rectangle& rDestRect, ULONG nLast, SfxProgress* pProgress)
{
	long	nWidth = GetOrgWidth();
	long	nHeight = GetOrgHeight();
	long	nX;
	long	nY;
	HPBYTE 	pOrgArray = GetOrgPointer();
	HPBYTE 	pDestArray = GetDestPointer();
	HPBYTE 	pTempOrg;
	HPBYTE 	pTempDest;
	BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
	ULONG	nCond = 4 - ( (nWidth * 3 ) & 3 );
	ULONG	nOff2;
	ULONG	nXOff;
	ULONG	nYOff;
	BOOL	bRet = FALSE;

	if ( pOrgArray && pDestArray )
	{
		Point 		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );
		INT16*		pModArray = new INT16[ 256 ];
		INT16		nOff;
		INT16		nTmp;

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Array anpassen, um Luminanz direkt aendern zu  koennen
		for ( INT16 i = 0; i < 256; i++ )
			pModArray[i] = (INT16) pTransArray[i] - i;

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			nOff2 = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff2;
			pTempDest= pDestArray + nOff2;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				// RGB-Wert in Y-Wert umrechnen
				nOff = pModArray[ GRAYVAL( pTempOrg[ 0 ], pTempOrg[ 1 ], pTempOrg[ 2 ] ) ];

				nTmp = nOff + *pTempOrg++;
				*pTempDest++ = (BYTE) VOS_BOUND( nTmp, 0, 255 );

				nTmp = nOff + *pTempOrg++;
				*pTempDest++ = (BYTE) VOS_BOUND( nTmp, 0, 255 );

				nTmp = nOff + *pTempOrg++;
				*pTempDest++ = (BYTE) VOS_BOUND( nTmp, 0, 255 );
			}

			if ( !( nY % 4 ) && pProgress )
				pProgress->SetState( nY + nLast );
		}

		delete[] pModArray;
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

void SvImage::TransformPaletteLuminance( BYTE* pTransArray, BitmapPalette* pPal )
{
	INT16*	pModArray = new INT16[ 256 ];
	INT16	nTmp;
	USHORT	i, nCount;

	// Array anpassen, um Luminanz direkt aendern zu  koennen
	for ( i = 0; i < 256; i++)
		pModArray[ i ] = (INT16) pTransArray[ i ] - i;

	for ( i = 0, nCount = pPal->GetEntryCount(); i < nCount; i++ )
	{
		BitmapColor&	rColor = (*pPal)[ i ];
		const INT16		nOff = pModArray[ rColor.GetLuminance() ];

		// Farbanteile in der Helligkeit aendern
		nTmp = rColor.GetRed() + nOff;
		rColor.SetRed( (BYTE) VOS_BOUND( nTmp, 0, 255) );

		nTmp = rColor.GetGreen() + nOff;
		rColor.SetGreen( (BYTE) VOS_BOUND( nTmp, 0, 255) );

		nTmp = rColor.GetBlue() + nOff;
		rColor.SetBlue( (BYTE) VOS_BOUND( nTmp, 0, 255) );
	}

	delete[] pModArray;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Convolute3(short* pMask, const short nDivisor, const BYTE cBias,
						 ULONG nLast, SfxProgress* pProgress)
{
	long	nWidth = GetOrgWidth();
	long	nHeight = GetOrgHeight();
	long	nWidth2 = nWidth - 2;
	long	nHeight2 = nHeight - 1 ;
	long	nAlignedWidth = GetOrgAlignedWidth();
	long	nX;
	long	nY;
	HPBYTE 	pOrgArray = GetOrgPointer();
	HPBYTE 	pTempOrg;
	HPBYTE 	pDestArray = GetDestPointer();
	HPBYTE 	pTempDest;
	HPBYTE 	pNear;
	HPBYTE 	p1;
	HPBYTE 	p2;
	HPBYTE 	p3;
	short	nMask11 = pMask[0], nMask12 = pMask[1], nMask13 = pMask[2];
	short	nMask21 = pMask[3], nMask22 = pMask[4], nMask23 = pMask[5];
	short	nMask31 = pMask[6], nMask32 = pMask[7], nMask33 = pMask[8];
	long	nSumR, nSumG, nSumB;
	long	nValue;
	BOOL	bRet = FALSE;

	if ( pOrgArray && pDestArray && ( nWidth > 2 ) && ( nHeight > 2 ) )
	{
		pTempOrg = pOrgArray;

		// Zeilenschleife
		for ( nY = 1; nY < nHeight2; nY++ )
		{
			pTempDest = pDestArray + nAlignedWidth*nY + 3;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth2; nX++ )
			{
				nSumR = nSumG = nSumB = 0;

				p1 = pTempOrg + nX * 3;
				p2 = p1 + nAlignedWidth;
				p3 = p2 + nAlignedWidth;

				// 1. Faltungszeile (je drei Elemente)
				nSumB += nMask11 * *p1++;
				nSumG += nMask11 * *p1++;
				nSumR += nMask11 * *p1++;

				nSumB += nMask12 * *p1++;
				nSumG += nMask12 * *p1++;
				nSumR += nMask12 * *p1++;

				nSumB += nMask13 * *p1++;
				nSumG += nMask13 * *p1++;
				nSumR += nMask13 * *p1;

				// 2. Faltungszeile (je drei Elemente)
				nSumB += nMask21 * *p2++;
				nSumG += nMask21 * *p2++;
				nSumR += nMask21 * *p2++;

				nSumB += nMask22 * *p2++;
				nSumG += nMask22 * *p2++;
				nSumR += nMask22 * *p2++;

				nSumB += nMask23 * *p2++;
				nSumG += nMask23 * *p2++;
				nSumR += nMask23 * *p2;

				// 3. Faltungszeile (je drei Elemente)
				nSumB += nMask31 * *p3++;
				nSumG += nMask31 * *p3++;
				nSumR += nMask31 * *p3++;

				nSumB += nMask32 * *p3++;
				nSumG += nMask32 * *p3++;
				nSumR += nMask32 * *p3++;

				nSumB += nMask33 * *p3++;
				nSumG += nMask33 * *p3++;
				nSumR += nMask33 * *p3;

				nValue = nSumB / nDivisor + cBias;
				*pTempDest++ = (BYTE) VOS_BOUND( nValue, 0L, 255L );

				nValue = nSumG / nDivisor + cBias;
				*pTempDest++ = (BYTE) VOS_BOUND( nValue, 0L, 255L );

				nValue = nSumR / nDivisor + cBias;
				*pTempDest++ = (BYTE) VOS_BOUND( nValue, 0L, 255L );
			}

			pTempOrg += nAlignedWidth;

			if ( ! ( nY % 4 ) && pProgress )
				pProgress->SetState( nY+nLast );
		}

		// Umrandung mit Nachbarpixeln fuellen
		// links
		pTempDest = pDestArray + nAlignedWidth;
		pNear = pTempDest + 3;

		for ( nY = 1; nY < nHeight2; nY++ )
		{
			pTempDest[0] = *pNear++;
			pTempDest[1] = *pNear++;
			pTempDest[2] = *pNear;

			pTempDest += nAlignedWidth;
			pNear = pTempDest + 3;
		}

		// rechts
		pTempDest = pDestArray + nAlignedWidth + 3*(ULONG)(nWidth-1);
		pNear = pTempDest - 3;

		for ( nY = 1; nY < nHeight2; nY++ )
		{
			pTempDest[0] = *pNear++;
			pTempDest[1] = *pNear++;
			pTempDest[2] = *pNear;

			pTempDest += nAlignedWidth;
			pNear = pTempDest - 3;
		}

		// oben
		pTempDest = pDestArray;
		pNear = pTempDest + nAlignedWidth;

		for (nX = 0; nX < nAlignedWidth; nX++)
			*pTempDest++ = *pNear++;

		// unten
		pTempDest = pDestArray + (ULONG)nHeight2*nAlignedWidth;
		pNear = pTempDest	- nAlignedWidth;

		for (nX = 0; nX < nAlignedWidth; nX++)
			*pTempDest++ = *pNear++;

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Convolute3(short* pMask, const short nDivisor, const BYTE cBias,
 					 	 const Rectangle& rDestRect, ULONG nLast, SfxProgress* pProgress)

{
	long	nWidth = GetOrgWidth();
	long	nHeight = GetOrgHeight();
	long	nAlignedWidth = GetOrgAlignedWidth();
	long	nX;
	long	nY;
	long	nValue;
	long	nSumR, nSumG, nSumB;
	HPBYTE 	pOrgArray = GetOrgPointer();
	HPBYTE 	pTempOrg;
	HPBYTE 	pDestArray = GetDestPointer();
	HPBYTE 	pTempDest;
	HPBYTE 	p1;
	HPBYTE 	p2;
	HPBYTE 	p3;
	short	nMask11 = pMask[0], nMask12 = pMask[1], nMask13 = pMask[2];
	short	nMask21 = pMask[3], nMask22 = pMask[4], nMask23 = pMask[5];
	short	nMask31 = pMask[6], nMask32 = pMask[7], nMask33 = pMask[8];
	ULONG	nXOff;
	ULONG	nYOff;
	ULONG	nOff;
	ULONG	nOff2;
	BOOL	bRet = FALSE;

	if ( pOrgArray && pDestArray && ( nWidth > 2 ) && ( nHeight > 2 ) )
	{
		Rectangle	aOrgRect( Point(1, 1), Size(nWidth - 2, nHeight - 2) );
		Rectangle&	rInterRect = aOrgRect.Intersection(rDestRect);

		nXOff = 3 * (rInterRect.Left() - 1);
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight() - 1;
		nOff2 = nAlignedWidth + 3;

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			nOff = (nYOff + nY)*nAlignedWidth + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff + nOff2;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				nSumR = nSumG = nSumB = 0;

				p1 = pTempOrg + nX * 3;
				p2 = p1 + nAlignedWidth;
				p3 = p2 + nAlignedWidth;

				// 1. Faltungszeile (je drei Elemente)
				nSumR += nMask11 * *p1++;
				nSumG += nMask11 * *p1++;
				nSumB += nMask11 * *p1++;

				nSumR += nMask12 * *p1++;
				nSumG += nMask12 * *p1++;
				nSumB += nMask12 * *p1++;

				nSumR += nMask13 * *p1++;
				nSumG += nMask13 * *p1++;
				nSumB += nMask13 * *p1;

				// 2. Faltungszeile (je drei Elemente)
				nSumR += nMask21 * *p2++;
				nSumG += nMask21 * *p2++;
				nSumB += nMask21 * *p2++;

				nSumR += nMask22 * *p2++;
				nSumG += nMask22 * *p2++;
				nSumB += nMask22 * *p2++;

				nSumR += nMask23 * *p2++;
				nSumG += nMask23 * *p2++;
				nSumB += nMask23 * *p2;

				// 3. Faltungszeile (je drei Elemente)
				nSumR += nMask31 * *p3++;
				nSumG += nMask31 * *p3++;
				nSumB += nMask31 * *p3++;

				nSumR += nMask32 * *p3++;
				nSumG += nMask32 * *p3++;
				nSumB += nMask32 * *p3++;

				nSumR += nMask33 * *p3++;
				nSumG += nMask33 * *p3++;
				nSumB += nMask33 * *p3;

				nValue = nSumR / nDivisor + cBias;
				*pTempDest++ = (BYTE) VOS_BOUND( nValue, 0, 255 );

				nValue = nSumG / nDivisor + cBias;
				*pTempDest++ = (BYTE) VOS_BOUND( nValue, 0, 255 );

				nValue = nSumB / nDivisor + cBias;
				*pTempDest++ = (BYTE) VOS_BOUND( nValue, 0, 255 );
			}

			if ( !( nY % 4 ) && pProgress )
				pProgress->SetState( nY + nLast );
		}

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Invert(SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		long	nAlignedWidth = GetOrgAlignedWidth();
		HPBYTE 	pOrgArray = GetOrgPointer();
		HPBYTE 	pDestArray = GetDestPointer();
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId( STR_STAT_INVERT ), nHeight << 1 );

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			// Spaltenschleife
			for ( nX = 0; nX < nAlignedWidth; nX++ )
				*pDestArray++ = ~*pOrgArray++;

			if ( ! ( nY % 4 ) )
				aProgress.SetState( nY+1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette
		BitmapPalette* pPal = GetPalette();

		for( USHORT i = 0, nCount = pPal->GetEntryCount(); i < nCount; i++ )
			(*pPal)[ i ].Invert();
		
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Invert(const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nAlignedWidth = GetOrgAlignedWidth();
		long		nX;
		long		nY;
		HPBYTE 		pOrgArray = GetOrgPointer();
		HPBYTE 		pDestArray = GetDestPointer();
		HPBYTE 		pTempOrg;
		HPBYTE 		pTempDest;
		ULONG		nXOff;
		ULONG		nYOff;
		ULONG 		nOff;
		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_INVERT), nHeight + rInterRect.GetHeight() );

		// Operation auf Rectangle anwenden
		nWidth 	= rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			nOff = (nYOff + nY) * nAlignedWidth + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				*pTempDest++ = ~*pTempOrg++;
				*pTempDest++ = ~*pTempOrg++;
				*pTempDest++ = ~*pTempOrg++;
			}

			if ( ! ( nY % 4 ) )
				aProgress.SetState(nY+1);
		}

		// EIP beenden
		EndProcessing();
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Solarize(float fThres, BOOL bUpper, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE	pOrgArray = GetOrgPointer();
		HPBYTE	pDestArray = GetDestPointer();
		BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
		BYTE	cGrayThres = (BYTE) VOS_BOUND( fThres * 255.F * 0.01F, 0.F, 255.F );
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_SOLAR), nHeight << 1 );

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
			 	if ( GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] ) < cGrayThres )
			 	{
					if ( bUpper )
					{
			 			*pDestArray++ = *pOrgArray++;
						*pDestArray++ =	*pOrgArray++;
						*pDestArray++ = *pOrgArray++;
					}
					else
					{
			 			*pDestArray++ = ~(*pOrgArray++);
						*pDestArray++ =	~(*pOrgArray++);
						*pDestArray++ = ~(*pOrgArray++);
					}
				}
				else
			 	{
					if ( bUpper )
					{
			 			*pDestArray++ = ~(*pOrgArray++);
						*pDestArray++ =	~(*pOrgArray++);
						*pDestArray++ = ~(*pOrgArray++);
					}
					else
					{
			 			*pDestArray++ = *pOrgArray++;
						*pDestArray++ =	*pOrgArray++;
						*pDestArray++ = *pOrgArray++;
					}
				}
			}

			// Long-Alignment der Zeile beachten
			if (bCond)
			{
				pOrgArray 	+= nCond;
				pDestArray 	+= nCond;
			}

			if (!(nY%4))
				aProgress.SetState(nY+1);
		}

		// EIP beenden
		EndProcessing();

		// Palette
		BitmapPalette* pPal = GetPalette();

		for( USHORT i = 0, nCount = pPal->GetEntryCount(); i < nCount; i++ )
		{
			BitmapColor& rCol = (*pPal)[ i ];

			if( rCol.GetLuminance() < cGrayThres )
			{
				if( !bUpper )
					rCol.Invert();
			}
			else if( bUpper )
				rCol.Invert();
		}

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Solarize(float fThres, BOOL bUpper, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE		pOrgArray = GetOrgPointer();
		HPBYTE		pDestArray = GetDestPointer();
		HPBYTE		pTempOrg;
		HPBYTE		pTempDest;
		BYTE		cGrayThres;
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG		nXOff;
		ULONG		nYOff;
		ULONG 		nOff;
		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_SOLAR), nHeight + rInterRect.GetHeight() );

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		cGrayThres = (BYTE) VOS_BOUND( fThres * 255.F * 0.01F, 0.F, 255.F );

		// Zeilenschleife
		for (nY = 0; nY < nHeight; nY++)
		{
			nOff = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
			 	if ( GRAYVAL( pTempOrg[ 0 ], pTempOrg[ 1 ], pTempOrg[ 2 ] ) < cGrayThres )
			 	{
					if ( bUpper )
					{
			 			*pTempDest++ = *pTempOrg++;
						*pTempDest++ = *pTempOrg++;
						*pTempDest++ = *pTempOrg++;
					}
					else
					{
			 			*pTempDest++ = ~(*pTempOrg++);
						*pTempDest++ = ~(*pTempOrg++);
						*pTempDest++ = ~(*pTempOrg++);
					}
				}
				else
			 	{
					if ( bUpper )
					{
			 			*pTempDest++ = ~(*pTempOrg++);
						*pTempDest++ = ~(*pTempOrg++);
						*pTempDest++ = ~(*pTempOrg++);
					}
					else
					{
			 			*pTempDest++ = *pTempOrg++;
						*pTempDest++ = *pTempOrg++;
						*pTempDest++ = *pTempOrg++;
					}
				}
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Grayscale(USHORT nSteps, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() && ( nSteps > 1 ) )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE	pOrgArray = GetOrgPointer();
		HPBYTE	pDestArray = GetDestPointer();
		BYTE	cGray;
		BYTE*	pTable = new BYTE[ 256 ];
		BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
		double*	pMiddle = new double[ nSteps ];
		double	fCurVal = 0.0;
		double	fStep = 255.0 / ( nSteps - 1 );
		long	i, nCurMiddle;
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_GRAYSCALES), nHeight << 1 );

		for( i = 0, fCurVal = fStep * 0.5; i < nSteps; i++, fCurVal += fStep  )
			pMiddle[ i ] = fCurVal;

		for( i = 0, fCurVal = 0.0, nCurMiddle = 0; i < 256; i++ )
		{
			if( i >= pMiddle[ nCurMiddle ] )
				pTable[ i ] = (BYTE) FRound( fCurVal += fStep ), nCurMiddle++;
			else
				pTable[ i ] = (BYTE) FRound( fCurVal );
		}

		delete[] pMiddle;

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				cGray = pTable[ GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] ) ];

				*pDestArray++ = cGray;
				*pDestArray++ = cGray;
				*pDestArray++ = cGray;

				pOrgArray += 3;
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY+1 );
		}

		// EIP beenden
		EndProcessing();
		delete[] pTable;

		// Palette
		BitmapPalette* pPal = GetPalette();

		pPal->SetEntryCount( nSteps );

		for( i = 0, fCurVal = 0.0; i < nSteps; i++, fCurVal += fStep )
		{
			const BYTE cColorVal = (BYTE) FRound( fCurVal );
			(*pPal)[ i ] = BitmapColor( cColorVal, cColorVal, cColorVal );
		}

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Grayscale(USHORT nSteps, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE		pOrgArray = GetOrgPointer();
		HPBYTE		pDestArray = GetDestPointer();
		HPBYTE 		pTempOrg;
		HPBYTE 		pTempDest;
		BYTE		cGray;
		BYTE*		pTable = new BYTE[ 256 ];
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG		nXOff;
		ULONG		nYOff;
		ULONG 		nOff;
		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );
		double*		pMiddle = new double[ nSteps ];
		double		fCurVal = 0.0;
		double		fStep = 255.0 / ( nSteps - 1 );
		long		i, nCurMiddle;
        
		for( i = 0, fCurVal = fStep * 0.5; i < nSteps; i++, fCurVal += fStep  )
			pMiddle[ i ] = fCurVal;

		for( i = 0, fCurVal = 0.0, nCurMiddle = 0; i < 256; i++ )
		{
			if( i >= pMiddle[ nCurMiddle ] )
				pTable[ i ] = (BYTE) FRound( fCurVal += fStep ), nCurMiddle++;
			else
				pTable[ i ] = (BYTE) FRound( fCurVal );
		}

		delete[] pMiddle;

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );
        SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_GRAYSCALES), nHeight + rInterRect.GetHeight());

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			nOff = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				cGray = pTable[ GRAYVAL( pTempOrg[ 0 ], pTempOrg[ 1 ], pTempOrg[ 2 ] ) ];

				*pTempDest++ = cGray;
				*pTempDest++ = cGray;
				*pTempDest++ = cGray;

				pTempOrg += 3;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();
		delete[] pTable;
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Monochrome(float fLimit, BOOL bInvert, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE 		pOrgArray = GetOrgPointer();
		HPBYTE 		pDestArray = GetDestPointer();
		BYTE		cGray;
		BYTE		cMax = (BYTE) (fLimit * 2.55F);
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		BYTE		cLow = bInvert ? 0 : 255;
		BYTE		cHigh = bInvert ? 255 : 0;
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_MONOCHROME), nHeight << 1 );

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			// Spaltenschleife
			for (nX = 0; nX < nWidth; nX++)
			{
				cGray = GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] ) > cMax ? cLow : cHigh;

				*pDestArray++ = cGray;
				*pDestArray++ = cGray;
				*pDestArray++ = cGray;

				pOrgArray += 3;
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray 	+= nCond;
				pDestArray 	+= nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette
		*GetPalette() = Bitmap::GetGreyPalette( 2 );

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Monochrome(float fLimit, BOOL bInvert, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE 		pOrgArray = GetOrgPointer();
		HPBYTE 		pDestArray = GetDestPointer();
		HPBYTE 		pTempOrg;
		HPBYTE 		pTempDest;
		BYTE		cGray;
		BYTE		cMax = (BYTE) ( fLimit * 2.55F );
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG		nOff;
		ULONG		nXOff;
		ULONG		nYOff;
		BYTE		cLow = bInvert ? 0 : 255;
		BYTE		cHigh = bInvert ? 255 : 0;
		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY(pDestArray, pOrgArray, GetOrgAlignedWidth()*(ULONG)nHeight);

        SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_MONOCHROME), nHeight + rInterRect.GetHeight());

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Zeilenschleife
		for (nY = 0; nY < nHeight; nY++)
		{
			nOff = (nYOff + nY)*GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff;

			// Spaltenschleife
			for (nX = 0; nX < nWidth; nX++)
			{
				cGray = GRAYVAL( pTempOrg[ 0 ], pTempOrg[ 1 ], pTempOrg[ 2 ] ) > cMax ? cLow : cHigh;

				*pTempDest++ = cGray;
				*pTempDest++ = cGray;
				*pTempDest++ = cGray;

				pTempOrg += 3;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState(nY+1);
		}

		// EIP beenden
		EndProcessing();
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::AdjustRGB(float fRed, float fGreen, float fBlue, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE 	pOrgArray = GetOrgPointer();
		HPBYTE 	pDestArray = GetDestPointer();
		BYTE*	pTableRed = new BYTE[256];
		BYTE*	pTableGreen	= new BYTE[256];
		BYTE*	pTableBlue = new BYTE[256];
		BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
		USHORT	i, nCount;
        
		SfxProgress	aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_RGB), ((ULONG)nHeight) << 1);

		// Aenderungsbereich auf neue Werte zur
		// Multiplikation umrechnen
		fBlue *= 1.28F;
		fGreen *= 1.28F;
		fRed *= 1.28F;

		// Tabellen mit den neuen RGB-Werten erzeugen, auf
		// die die alten Werte gemappt werden
		for ( i = 0; i < 256; i++ )
		{
			pTableBlue[i] = (BYTE) VOS_BOUND( i + fBlue, 0.F, 255.F );
			pTableGreen[i] = (BYTE) VOS_BOUND( i + fGreen, 0.F, 255.F );
			pTableRed[i] = (BYTE) VOS_BOUND( i + fRed, 0.F, 255.F );
		}

		// Zeilenschleife
		for (nY = 0; nY < nHeight; nY++)
		{
			// Spaltenschleife
			for (nX = 0; nX < nWidth; nX++)
			{
				*pDestArray++ = pTableBlue[ *pOrgArray++ ];
				*pDestArray++ = pTableGreen[ *pOrgArray++ ];
				*pDestArray++ = pTableRed[ *pOrgArray++ ];
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( ! ( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette
		BitmapPalette* pPal = GetPalette();

		for( i = 0, nCount = pPal->GetEntryCount(); i < nCount; i++ )
		{
			BitmapColor& rCol = (*pPal)[ i ];

			rCol.SetRed( pTableRed[ rCol.GetRed() ] );
			rCol.SetGreen( pTableGreen[ rCol.GetGreen() ] );
			rCol.SetBlue( pTableBlue[ rCol.GetBlue() ] );
		}

		delete[] pTableRed;
		delete[] pTableGreen;
		delete[] pTableBlue;

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::AdjustRGB(float fRed, float fGreen, float fBlue, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE		pOrgArray = GetOrgPointer();
		HPBYTE 		pDestArray = GetDestPointer();
		HPBYTE 		pTempOrg;
		HPBYTE 		pTempDest;
		BYTE*		pTableRed = new BYTE[256];
		BYTE*		pTableGreen	= new BYTE[256];
		BYTE*		pTableBlue	= new BYTE[256];
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG		nOff;
		ULONG		nXOff;
		ULONG		nYOff;
		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );

		// Aenderungsbereich auf neue Werte zur
		// Multiplikation umrechnen
		fBlue *= 1.28F;
		fGreen *= 1.28F;
		fRed *= 1.28F;

		// Tabellen mit den neuen RGB-Werten erzeugen, auf
		// die die alten Werte gemappt werden
		for( long i = 0; i < 256; i++ )
		{
			pTableBlue[i] = (BYTE) VOS_BOUND( i + fBlue, 0.F, 255.F );
			pTableGreen[i] = (BYTE) VOS_BOUND( i + fGreen, 0.F, 255.F );
			pTableRed[i] = (BYTE) VOS_BOUND( i + fRed,	0.F, 255.F );
		}

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_RGB), nHeight + rInterRect.GetHeight() );

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			nOff = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				*pTempDest++ = pTableBlue[ *pTempOrg++ ];
				*pTempDest++ = pTableGreen[ *pTempOrg++ ];
				*pTempDest++ = pTableRed[ *pTempOrg++ ];
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();
		delete[] pTableRed;
		delete[] pTableGreen;
		delete[] pTableBlue;
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}

/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Sepia(float fSepia, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE 	pOrgArray = GetOrgPointer();
		HPBYTE 	pDestArray = GetDestPointer();
		BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
		BYTE*	pTableSepia = new BYTE[256];
		USHORT	i, nCount;

        SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_AGE), nHeight << 1 );

		fSepia	= (float) VOS_BOUND( 1.F - fabs(fSepia) / 100.F, +0.F, +1.F );

		// Tabelle mit den zu den Gruen- und Blau-Werten
		// korrespondierenden Werten erzeugen
		for ( i = 0; i < 256; i++ )
			pTableSepia[i] 	= (BYTE) VOS_BOUND( fSepia * i, 0.F, 255.F );

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				// RGB-Wert in Y-Wert umrechnen und Rot-Anteil setzen
				const BYTE cGray = GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] );

				*pDestArray++ = pTableSepia[ cGray ];
				*pDestArray++ = pTableSepia[ cGray ];
				*pDestArray++ = cGray;

				pOrgArray += 3;
			}

			// Long-Alignment der Zeile beachten
			if ( bCond)
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY+1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette
		BitmapPalette* pPal = GetPalette();

		for( i = 0, nCount = pPal->GetEntryCount(); i < nCount; i++ )
		{
			BitmapColor&	rCol = (*pPal)[ i ];
			BYTE			cGray = rCol.GetLuminance();

			rCol.SetRed( cGray );
			rCol.SetGreen( cGray = pTableSepia[ cGray ] );
			rCol.SetBlue( cGray );
		}

		delete[] pTableSepia;
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Sepia(float fSepia, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE 	pOrgArray = GetOrgPointer();
		HPBYTE 	pDestArray = GetDestPointer();
		HPBYTE 	pTempOrg;
		HPBYTE 	pTempDest;
		BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG	nOff;
		ULONG	nXOff;
		ULONG	nYOff;
		BYTE*	pTableSepia = new BYTE[ 256 ];
		BYTE	cGray;

		fSepia = (float) VOS_BOUND(1.F - fabs(fSepia)/100.F, +0.F, +1.F);

		// Tabelle mit den zu den Gruen- und Blau-Werten
		// korrespondierenden Werten erzeugen
		for ( long i = 0; i < 256; i++ )
			pTableSepia[i] 	= (BYTE) VOS_BOUND( fSepia * i,	0.F, 255.F );

		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_AGE), nHeight + rInterRect.GetHeight() );

   		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			nOff = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest= pDestArray + nOff;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				// RGB-Wert in Y-Wert umrechnen und Rot-Anteil setzen
				cGray = GRAYVAL( pTempOrg[ 0 ], pTempOrg[ 1 ], pTempOrg[ 2 ] );

				*pTempDest++ = pTableSepia[ cGray ];
				*pTempDest++ = pTableSepia[ cGray ];
				*pTempDest++ = cGray;

				pTempOrg += 3;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();
		delete[] pTableSepia;
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Mirror(BOOL bHorz, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE 	pOrgArray = GetOrgPointer();
		HPBYTE 	pDestArray = GetDestPointer();
		HPBYTE	pTemp;
		USHORT	nOff;
		BOOL	bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_MIRROR), nHeight << 1 );

		// Horizontal spiegeln
		if ( bHorz )
		{
			nOff = ( nWidth - 1 ) * 3;

			// Zeilenschleife
			for ( nY = 0; nY < nHeight; nY++ )
			{
				pTemp = pDestArray + nY * GetDestAlignedWidth() + nOff;

				// Spaltenschleife
				for ( nX = 0; nX < nWidth; nX++ )
				{
					pTemp[ 0 ] = *pOrgArray++;
					pTemp[ 1 ] = *pOrgArray++;
					pTemp[ 2 ] = *pOrgArray++;

					pTemp -= 3;
				}

				// Long-Alignment der Zeile beachten
				if ( bCond )
					pOrgArray += nCond;

				if ( !( nY % 4 ) )
					aProgress.SetState( nY + 1 );
			}
		}
		// Vertikal spiegeln
		else
		{
			nOff = nHeight - 1;

			// Zeilenschleife
			for ( nY = 0; nY < nHeight; nY++ )
			{
				pTemp = pDestArray + (nOff - nY) * GetDestAlignedWidth();

				// Spaltenschleife
				for ( nX = 0; nX < nWidth; nX++ )
				{
					*pTemp++ = *pOrgArray++;
					*pTemp++ = *pOrgArray++;
					*pTemp++ = *pOrgArray++;
				}

				// Long-Alignment der Zeile beachten
				if ( bCond )
					pOrgArray += nCond;

				if ( !( nY % 4 ) )
					aProgress.SetState( nY + 1 );
			}
		}

		// EIP beenden
		EndProcessing();
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Mirror(BOOL bHorz, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if (BeginProcessing())
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE 		pOrgArray = GetOrgPointer();
		HPBYTE 		pDestArray = GetDestPointer();
		HPBYTE 		pTempOrg;
		HPBYTE 		pTempDest;
		USHORT		nOff;
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG		nOff2;
		ULONG		nXOff;
		ULONG		nYOff;
		Rectangle	aOrgRect(Point(0,0), Size(nWidth, nHeight));
		Rectangle&	rInterRect = aOrgRect.Intersection(rDestRect);

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_MIRROR), nHeight + rInterRect.GetHeight() );

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

		// Horizontal spiegeln
		if ( bHorz )
		{
			nOff = ( nWidth - 1 ) * 3;

			// Zeilenschleife
			for ( nY = 0; nY < nHeight; nY++ )
			{
				nOff2 = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
				pTempOrg = pOrgArray + nOff2;
				pTempDest= pDestArray + nOff2 + nOff;

				// Spaltenschleife
				for ( nX = 0; nX < nWidth; nX++ )
				{
					pTempDest[ 0 ] = *pTempOrg++;
					pTempDest[ 1 ] = *pTempOrg++;
					pTempDest[ 2 ] = *pTempOrg++;

					pTempDest -= 3;
				}

				if ( !( nY % 4 ) )
					aProgress.SetState( nY + 1 );
			}
		}
		// Vertikal spiegeln
		else
		{
			nOff = nHeight - 1;

			// Zeilenschleife
			for ( nY = 0; nY < nHeight; nY++ )
			{
				nOff2 		= (nYOff + nY) * GetOrgAlignedWidth() + nXOff;
				pTempOrg 	= pOrgArray + nOff2;
				pTempDest	= pDestArray + (nYOff + nOff - nY) * GetOrgAlignedWidth() + nXOff;

				// Spaltenschleife
				for ( nX = 0; nX < nWidth; nX++ )
				{
					*pTempDest++ = *pTempOrg++;
					*pTempDest++ = *pTempOrg++;
					*pTempDest++ = *pTempOrg++;
				}

				if ( !( nY % 4 ) )
					aProgress.SetState( nY + 1 );
			}
		}

		// EIP beenden
		EndProcessing();
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Crop(const Point& rOrg, const Size& rExt, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessingExt( rExt.Width(), rExt.Height() ) )
	{
		const long			nWidth = GetDestWidth();
		const long			nHeight = GetDestHeight();
		const long			nWidthAlDst = GetDestAlignedWidth();
		const long			nWidthAlOrg = GetOrgAlignedWidth();
		long				nY;
		HPBYTE 				pOrgArray = GetOrgPointer();
		HPBYTE 				pDestArray = GetDestPointer();
		ULONG				nXOff;
		ULONG				nYOff;
		Point				aTmpPoint;
		Size				aTmpSize( GetOrgWidth(), GetOrgHeight() );
		Rectangle			aOrgRect( aTmpPoint, aTmpSize );
		const Rectangle		aDestRect( rOrg, rExt );
		const Rectangle&	rInterRect = aOrgRect.Intersection( aDestRect );

		nXOff = 3 * rInterRect.Left();
		nYOff = GetOrgHeight() - rInterRect.Top() - rInterRect.GetHeight();

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_CUT), nHeight << 1 );

		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			HMEMCPY( pDestArray, pOrgArray + (nYOff + nY) * nWidthAlOrg + nXOff, nWidth * 3 );
			pDestArray += nWidthAlDst;

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessingExt();
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Pixellize(const Size& rPixelSize, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long			nWidth = GetOrgWidth();
		long			nHeight = GetOrgHeight();
		long			nX;
		long			nY;
		long			nX1;
		long			nY1;
		HPBYTE 			pOrgArray = GetOrgPointer();
		HPBYTE 			pDestArray = GetDestPointer();
		HPBYTE			pTemp;
		long			nPixWidth  = rPixelSize.Width();
		long			nPixHeight = rPixelSize.Height();
		long			nRastRows = nHeight / nPixHeight;
		long			nRestRows = nHeight % nPixHeight;
		long			nRastCols = nWidth / nPixWidth;
		long			nRestCols = nWidth % nPixWidth;
		long			nSumR;
		long			nSumG;
		long			nSumB;
		long			nRasterSize;
		ULONG			nScan;
		ULONG			nOff;
		BOOL			bColFlag;
		BOOL			bRowFlag;
		BOOL			bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG			nCond = 4 - ( ( nWidth * 3 ) & 3 );
        SfxProgress		aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_TILE), nHeight << 1 );

 		for ( nY = 0; nY < nHeight; nY += nPixHeight )
    	{
    		nScan = nY * GetOrgAlignedWidth();

      		// berprfung, ob die letzten Zeilen gesondert behandelt werden mssen
			bRowFlag = ( ( nY / nPixHeight) == nRastRows );

      		for ( nX = 0; nX < nWidth; nX += nPixWidth )
      		{
      			nSumR = nSumG = nSumB = 0;

 	     	  	// berprfung, ob die letzten Spalten gesondert behandelt werden mssen
				bColFlag = ( (nX / nPixWidth) == nRastCols );

      	  		nOff = nScan + nX * 3;

      	  		// Alle Werte des Raster-Rechtecks auslesen und addieren
      	  		for ( nY1 = 0; nY1 < ( bRowFlag ? nRestRows : nPixHeight ); nY1++ )
      	    	{
      	    		pTemp = pOrgArray + nOff + nY1 * GetOrgAlignedWidth();
      	      		for (nX1 = 0; nX1 < (bColFlag ? nRestCols : nPixWidth); nX1++)
      	        	{
      	        		nSumB += *pTemp++;
      	          		nSumG += *pTemp++;
      	          		nSumR += *pTemp++;
      	     		}
      	    	}

      	  		nRasterSize = ( bColFlag ? nRestCols : nPixWidth ) * ( bRowFlag ? nRestRows : nPixHeight );

	      	 	// gemittelten Wert in das Raster-Rechteck schreiben
      	  		for (nY1=0; nY1 < (bRowFlag ? nRestRows : nPixHeight); nY1++)
      	    	{
      	    		pTemp = pDestArray + nOff + nY1 * GetDestAlignedWidth();
      	      		for (nX1=0; nX1 < (bColFlag ? nRestCols : nPixWidth); nX1++)
      	        	{
      	        		*pTemp++ = (BYTE) (nSumB / nRasterSize);
      	          		*pTemp++ = (BYTE) (nSumG / nRasterSize);
      	          		*pTemp++ = (BYTE) (nSumR / nRasterSize);
      	        	}
      	    	}
       		}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Pixellize(const Size& rPixelSize, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		long		nX1;
		long		nY1;
		HPBYTE 		pOrgArray = GetOrgPointer();
		HPBYTE		pDestArray = GetDestPointer();
		HPBYTE		pTemp;
		long		nPixWidth = rPixelSize.Width();
		long		nPixHeight = rPixelSize.Height();
		long		nRastRows = nHeight / nPixHeight;
		long		nRestRows = nHeight % nPixHeight;
		long		nRastCols = nWidth / nPixWidth;
		long		nRestCols = nWidth % nPixWidth;
		long		nSumR;
		long		nSumG;
		long		nSumB;
		long		nRasterSize;
		ULONG		nScan;
		ULONG		nOff;
		BOOL		bColFlag;
		BOOL		bRowFlag;
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		ULONG		nXOff;
		ULONG		nYOff;
		Point		aTmpPoint;
		Size		aTmpSize( nWidth, nHeight );
		Rectangle	aOrgRect( aTmpPoint, aTmpSize );
		Rectangle&	rInterRect = aOrgRect.Intersection( rDestRect );

		nXOff = 3 * rInterRect.Left();
		nYOff = nHeight - rInterRect.Top() - rInterRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, GetOrgAlignedWidth() * nHeight );

        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_TILE), nHeight + rInterRect.GetHeight() );

		// Operation auf Rectangle anwenden
		nWidth = rInterRect.GetWidth();
		nHeight = rInterRect.GetHeight();

 		for ( nY = 0; nY < nHeight; nY += nPixHeight )
    	{
			nScan = (nYOff + nY) * GetOrgAlignedWidth() + nXOff;

      		// berprfung, ob die letzten Zeilen gesondert behandelt werden mssen
			bRowFlag = ( (nY / nPixHeight) == nRastRows );

      		for ( nX = 0; nX < nWidth; nX += nPixWidth )
      		{
      			nSumR = nSumG = nSumB = 0;

 	     	  	// berprfung, ob die letzten Spalten gesondert behandelt werden mssen
				bColFlag = ( (nX / nPixWidth) == nRastCols );

      	  		nOff = nScan + nX * 3;

      	  		// Alle Werte des Raster-Rechtecks auslesen und addieren
      	  		for ( nY1 = 0; nY1 < ( bRowFlag ? nRestRows : nPixHeight ); nY1++ )
      	    	{
      	    		pTemp = pOrgArray + nOff + nY1 * GetOrgAlignedWidth();
      	      		for (nX1 = 0; nX1 < ( bColFlag ? nRestCols : nPixWidth ); nX1++ )
      	        	{
      	        		nSumB += *pTemp++;
      	          		nSumG += *pTemp++;
      	          		nSumR += *pTemp++;
      	     		}
      	    	}

      	  		nRasterSize = (bColFlag ? nRestCols : nPixWidth) * (bRowFlag ? nRestRows : nPixHeight);

	      	 	// gemittelten Wert in das Raster-Rechteck schreiben
      	  		for ( nY1=0; nY1 < (bRowFlag ? nRestRows : nPixHeight); nY1++ )
      	    	{
      	    		pTemp = pDestArray + nOff + nY1 * GetDestAlignedWidth();
      	      		for (nX1=0; nX1 < (bColFlag ? nRestCols : nPixWidth); nX1++)
      	        	{
      	        		*pTemp++ = (BYTE) (nSumB / nRasterSize);
      	          		*pTemp++ = (BYTE) (nSumG / nRasterSize);
      	          		*pTemp++ = (BYTE) (nSumR / nRasterSize);
      	        	}
      	    	}
       		}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}

/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::GammaCorrection(float fGamma, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		BYTE* pArray = new BYTE[ 256 ];
		float fValue;

		if ( fGamma < 0.01F )
			fGamma = 0.01F;

		fGamma = 1.F / fGamma;

		// Array mit den Map-Werten fuellen
		for ( long i = 0; i < 256;  i++ )
		{
			fValue = (float) FRound( pow( ( i / 255.0 ), (double) fGamma ) * 255. );
			pArray[i] = (BYTE) VOS_BOUND( fValue, 0.F, 255.F );
		}

		// Luminanz-Transformation durchfuehren
		if ( rDestRect.IsEmpty() )
		{
            SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_GAMMA), GetOrgHeight() << 1);

			bRet = TransformLuminance(pArray, 0, &aProgress);
			EndProcessing();
			TransformPaletteLuminance( pArray, GetPalette() );
		}

		else
		{
            SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_GAMMA), GetOrgHeight() + rDestRect.GetHeight());

			bRet = TransformLuminance(pArray, rDestRect, 0, &aProgress);
			EndProcessing();
			GetPalette()->SetEntryCount( 0 );
		}

		delete[] pArray;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::DitherRGBToMono(SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long	nWidth = GetOrgWidth();
		long	nHeight = GetOrgHeight();
		long	nX;
		long	nY;
		HPBYTE	pOrgArray = GetOrgPointer();
		HPBYTE	pDestArray = GetDestPointer();
		USHORT	nD;
		BOOL	bCond = (BOOL) ( (nWidth * 3 ) & 3 );
		ULONG	nCond = 4 - ( ( nWidth * 3 ) & 3 );
		BYTE	cModY;
		BYTE	cGray;

        SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_MONOCHROME), nHeight << 1 );

		// Dithering durchfuehren
		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			cModY = (BYTE) ( nY & 0x0F ) << 4;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				nD = nDither256[ cModY + ( nX & 0x0F ) ];

				cGray = GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] );

				// Falls Eintrag nicht in unserer Palette ist
				if ( cGray % 255 )
					cGray = cMap2[ (BYTE) ( ( nLookUp[ cGray ] / 5 + nD ) >> 16 ) ];

				*pDestArray++ = cGray;
				*pDestArray++ =	cGray;
				*pDestArray++ = cGray;

				pOrgArray += 3;
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette
		*GetPalette() = Bitmap::GetGreyPalette( 2 );

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::DitherRGBTo16Gray(SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE		pOrgArray = GetOrgPointer();
		HPBYTE		pDestArray = GetDestPointer();
		USHORT		nD;
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		BYTE		cModY;
		BYTE		cGray;

        SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_GRAY16), nHeight << 1 );

		// Dithering durchfuehren
		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			cModY = (BYTE) ( nY & 0x0F ) << 4;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				nD = nDither256[ cModY + ( nX & 0x0F ) ];

				cGray = GRAYVAL( pOrgArray[ 0 ], pOrgArray[ 1 ], pOrgArray[ 2 ] );
				
				// Falls es kein Eintrag aus unserer Graustufenpalette ist
				if ( cGray % 17 )
					cGray = cMap16[ (BYTE) ( ( nLookUp[ cGray ] * 3 + nD ) >> 16 ) ];

				*pDestArray++ = cGray;
				*pDestArray++ =	cGray;
				*pDestArray++ = cGray;

				pOrgArray += 3;
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette
		*GetPalette() = Bitmap::GetGreyPalette( 16 );

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::DitherRGBTo16(SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if ( BeginProcessing() )
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE		pOrgArray = GetOrgPointer();
		HPBYTE		pDestArray = GetDestPointer();
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		USHORT		nD;
		BYTE		cModY;
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_COL16), nHeight << 1 );

		// Dithering durchfuehren
		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			cModY = (BYTE) ( nY & 0x0F ) << 4;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				nD = nDither256[ cModY + ( nX & 0x0F ) ];

				*pDestArray++ = cMap2[ (BYTE) ( (nLookUp[ *pOrgArray++ ] / 5 + nD ) >> 16 ) ];
				*pDestArray++ =	cMap2[ (BYTE) ( (nLookUp[ *pOrgArray++ ] / 5 + nD ) >> 16 ) ];
				*pDestArray++ = cMap2[ (BYTE) ( (nLookUp[ *pOrgArray++ ] / 5 + nD ) >> 16 ) ];
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette versorgen
		BitmapPalette* pPal = GetPalette();
		
		pPal->SetEntryCount( 8 );
		for( USHORT j = 0; j < 2; j++ )
			for( USHORT k = 0; k < 2; k++ )
				for( USHORT n = 0; n < 2; n++ )
					(*pPal)[ j * 4 + k * 2 + n ] = BitmapColor( cMap2[ j ], cMap2[ k ], cMap2[ n ] );

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::DitherRGBTo256(SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if (BeginProcessing())
	{
		long		nWidth = GetOrgWidth();
		long		nHeight = GetOrgHeight();
		long		nX;
		long		nY;
		HPBYTE		pOrgArray = GetOrgPointer();
		HPBYTE		pDestArray = GetDestPointer();
		BOOL		bCond = (BOOL) ( ( nWidth * 3 ) & 3 );
		ULONG		nCond = 4 - ( ( nWidth * 3 ) & 3 );
		USHORT		nD;
		BYTE		cModY;
        
		SfxProgress	aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_COL256), nHeight << 1 );


		// Dithering durchfuehren
		// Zeilenschleife
		for ( nY = 0; nY < nHeight; nY++ )
		{
			cModY = (BYTE) ( nY & 0x0F ) << 4;

			// Spaltenschleife
			for ( nX = 0; nX < nWidth; nX++ )
			{
				nD = nDither256[ cModY + ( nX & 0x0F ) ];

				*pDestArray++ = cMap6[ (BYTE) ( ( nLookUp[ *pOrgArray++ ] + nD ) >> 16 ) ];
				*pDestArray++ =	cMap6[ (BYTE) ( ( nLookUp[ *pOrgArray++ ] + nD ) >> 16 ) ];
				*pDestArray++ = cMap6[ (BYTE) ( ( nLookUp[ *pOrgArray++ ] + nD ) >> 16 ) ];
			}

			// Long-Alignment der Zeile beachten
			if ( bCond )
			{
				pOrgArray += nCond;
				pDestArray += nCond;
			}

			if ( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

		// EIP beenden
		EndProcessing();

		// Palette versorgen
		BitmapPalette*	pPal = GetPalette();
		BitmapColor		aCol;

		pPal->SetEntryCount( 232 );

		// Grundfarben auf 0;
		for( USHORT i = 0; i < 16; i++ )
			(*pPal)[ i ] = aCol;

		// danach Standardpalette
		for( USHORT j = 0; j < 6; j++ )
			for( USHORT k = 0; k < 6; k++ )
				for( USHORT n = 0; n < 6; n++ )
					(*pPal)[ j*36 + k*6 + n + 16 ] = BitmapColor( cMap6[ j ], cMap6[ k ], cMap6[ n ] );

		bRet = TRUE;
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Smooth(const BYTE	cIntens, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	short	Mask1[9] = { 1, 2, 1, 2, 5, 2, 1, 2, 1 };
	BOOL	bRet = FALSE;

	if ( rDestRect.IsEmpty() )
	{
		// EIP einleiten
		if ( BeginProcessing() )
		{
            SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_SMOOTH), GetOrgHeight() << 1 );

			bRet = Convolute3( Mask1, 17, 0, 0, &aProgress );
			EndProcessing();
			GetPalette()->SetEntryCount( 0 );
		}
	}
	else
	{
		// EIP einleiten
		if (BeginProcessing())
		{
            SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_SMOOTH), GetOrgHeight() + rDestRect.GetHeight() );

			bRet = Convolute3( Mask1, 17, 0, rDestRect, 0, &aProgress );

			// EIP beenden
			EndProcessing();
			GetPalette()->SetEntryCount( 0 );
		}
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Sharpen(const BYTE cIntens, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	short	Mask1[ 9 ] = { -1, -1,  -1, -1,  9, -1, -1, -1,  -1 };
	short	nSum = 0;
	BOOL	bRet = FALSE;

	Mask1[ 4 ] = cIntens;

	for ( long i=0; i < 9; i++ )
		nSum += Mask1[ i ];

	if ( rDestRect.IsEmpty() )
	{
		// EIP einleiten
		if ( BeginProcessing() )
		{
            SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_SHARPEN), GetOrgHeight() << 1 );

			bRet = Convolute3( Mask1, nSum, 0, 0, &aProgress );

			// EIP beenden
			EndProcessing();
			GetPalette()->SetEntryCount( 0 );
		}
	}
	else
	{
		// EIP einleiten
		if ( BeginProcessing() )
		{
            SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_SHARPEN), GetOrgHeight() + rDestRect.GetHeight() );

			bRet = Convolute3( Mask1, nSum, 0, rDestRect, 0, &aProgress );

			// EIP beenden
			EndProcessing();
			GetPalette()->SetEntryCount( 0 );
		}
	}

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Emboss(float fAzimuth,
					 float fElevation,
					 USHORT nWidth45,
					 const Rectangle& rDestRect,
					 SfxViewFrame* pFrame)
{
	BOOL bRet;

	if( rDestRect.IsEmpty() )
		bRet = ImpEmboss( fAzimuth, fElevation, nWidth45, pFrame );
	else 
		bRet = ImpEmboss( fAzimuth, fElevation, nWidth45, rDestRect, pFrame );

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Scale(float fScaleX, float fScaleY, BOOL bFast, SfxViewFrame* pFrame)
{
	return (!bFast ? ImpScaleFast(fScaleX, fScaleY, pFrame) :
					 ImpScale(fScaleX, fScaleY, pFrame));
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Rotate(float fAngle, const Color& rColor, SfxViewFrame* pFrame)
{
	fAngle = (float) fmod(fAngle, (float)360.);

	if (FRound(fAngle) == 90.F || FRound(fAngle) == -270.F)
		return ImpRotate90N(pFrame);

	else
	if (FRound(fAngle) == -90.F || FRound(fAngle) == 270.F)
		return ImpRotate90P(pFrame);

	else
	if (FRound(fAngle) == 180.F || FRound(fAngle) == -180.F)
		return ImpRotate180(pFrame);

	else
		return ImpRotate(fAngle, rColor, pFrame);
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Mosaic(const Size& rPixelSize, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	if (rDestRect.IsEmpty())
		return ImpMosaic(rPixelSize, pFrame);
	else
		return ImpMosaic(rPixelSize, rDestRect, pFrame);
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Charcoal(const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	if (rDestRect.IsEmpty())
		return ImpCharcoal(pFrame);
	else
		return ImpCharcoal(rDestRect, pFrame);
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::RemoveNoise(const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	if (rDestRect.IsEmpty())
		return ImpRemoveNoise(pFrame);
	else
		return ImpRemoveNoise(rDestRect, pFrame);
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Poster(USHORT nColors, const Rectangle& rDestRect, SfxViewFrame* pFrame )
{
	BOOL bRet;
	
	if( rDestRect.IsEmpty() )
		bRet = ImpPoster( nColors, pFrame );
	else 
		bRet = ImpPoster( nColors, rDestRect, pFrame );

	return bRet;
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::Popart(const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	if (rDestRect.IsEmpty())
		return ImpPopart(pFrame);
	else
		return ImpPopart(rDestRect, pFrame);
}


/******************************************************************************
|*
|*
|*
\******************************************************************************/

BOOL SvImage::AdjustLu_Co(float fLum, float fContr, const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	if (rDestRect.IsEmpty())
		return ImpLuCo(fLum, fContr, pFrame);
	else
		return ImpLuCo(fLum, fContr, rDestRect, pFrame);
}
