/*************************************************************************
 *
 *  $RCSfile: svpaint.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <vcl/bmpacc.hxx>
#include <vcl/gdimtf.hxx>
#include <vcl/metaact.hxx>
#include <vcl/virdev.hxx>
#include <vcl/line.hxx>
#include <vcl/region.hxx>
#include "svimage.hxx"

// -----------
// - SvImage -
// -----------

BOOL SvImage::PaintMetaFileInProcessing( const GDIMetaFile& rMtf, BYTE cMerge )
{
	VirtualDevice	aVDev( 1 );
	Point 			aPoint;
	Rectangle		aFull( aPoint, Size( GetSrcWidth(), GetSrcHeight() ) );
	const Color		aWhite( COL_WHITE ), aBlack( COL_BLACK );
	BOOL			bRet = FALSE;

	if( IsInProcessing() && aVDev.SetOutputSizePixel( aFull.GetSize() ) )
	{
		for( ULONG i = 0, nCount = rMtf.GetActionCount(); i < nCount; i++ )
		{
			MetaAction* p = rMtf.GetAction( i );

			switch( p->GetType() )
			{
				case( META_LINE_ACTION ):
				{
					if( aVDev.IsLineColor() )
					{
						Polygon aPoly( 2 );

						aPoly[ 0 ] = ( (MetaLineAction*) p )->GetStartPoint();
						aPoly[ 1 ] = ( (MetaLineAction*) p )->GetEndPoint();
						SetProcColor( aVDev.GetLineColor() );
						ImplPaintPolyPolyLine( aPoly, FALSE, cMerge );
					}
				}
				break;

				case( META_RECT_ACTION ):
				case( META_POLYLINE_ACTION ):
				case( META_POLYGON_ACTION ):
				case( META_POLYPOLYGON_ACTION ):
				{
					if( aVDev.IsFillColor() )
					{
						Rectangle aBound;

						switch( p->GetType() )
						{
							case( META_RECT_ACTION ): 
								aBound = ( (MetaRectAction*) p )->GetRect();
							break;
							
							case( META_POLYLINE_ACTION ):
								aBound = ( (MetaPolyLineAction*) p )->GetPolygon().GetBoundRect();
							break;
							
							case( META_POLYGON_ACTION ):
								aBound = ( (MetaPolygonAction*) p )->GetPolygon().GetBoundRect();
							break;
							
							case( META_POLYPOLYGON_ACTION ):
								aBound = ( (MetaPolyPolygonAction*) p )->GetPolyPolygon().GetBoundRect();
							break;
						}

						SetProcColor( aVDev.GetFillColor() );
						aVDev.Push( PUSH_LINECOLOR | PUSH_FILLCOLOR );
						aVDev.SetLineColor( aWhite );
						aVDev.SetFillColor( aWhite );
						aVDev.DrawRect( aFull );
						aVDev.SetLineColor( aBlack );
						aVDev.SetFillColor( aBlack );
						p->Execute( &aVDev );

						Bitmap aBmp( aVDev.GetBitmap( Point(), aVDev.GetOutputSizePixel() ) );
						ImplPaintMask( aBmp, aBound, cMerge );
						
						aVDev.Pop();
					}
				}
				break;

				default:
					p->Execute( &aVDev );
				break;
			}
		}

		bRet = TRUE;
	}

	return bRet;
}

// -----------------------------------------------------------------------------

BOOL SvImage::PaintInProcessing( long nY, long nX, 
								 const Bitmap& rPaintBmp, const Bitmap& rMaskBmp,
								 Bitmap* pRegionBmp )
{
	BOOL bRet = FALSE;

	if( IsInProcessing() )
	{
		BitmapReadAccess*	pPntAcc = ( (Bitmap&) rPaintBmp ).AcquireReadAccess();
		BitmapReadAccess*	pMskAcc = ( (Bitmap&) rMaskBmp ).AcquireReadAccess();

		if( pPntAcc && pMskAcc )
		{
			BitmapWriteAccess*	pRAcc = pRegionBmp ? ( (Bitmap*) pRegionBmp )->AcquireWriteAccess() : NULL;
			const long			nWidth = pPntAcc->Width(), nHeight = pPntAcc->Height();
			const long			nDstWidth = GetDestWidth(), nDstHeight = GetDestHeight();
			long				nXOrg, nXDst, nYOrg, nYDst;

			if( pRAcc )
			{
				const BitmapColor aSet( pRAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

				if( pMskAcc->GetBitCount() != 8 )
				{
					const BitmapColor aMskBlack( pMskAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
					{
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
						{
							if( nXDst > -1 && nXDst < nDstWidth && 
								nYDst > -1 && nYDst < nDstHeight &&
								pRAcc->GetPixel( nYDst, nXDst ) != aSet &&
								pMskAcc->GetPixel( nYOrg, nXOrg ) == aMskBlack )
							{
								pRAcc->SetPixel( nYDst, nXDst, aSet );
								SetProcPixel( nYDst, nXDst, pPntAcc->GetPixel( nYOrg, nXOrg ) );
							}
	    				}
					}
				}
				else
				{
					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
					{
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
						{
							mcMerge = pMskAcc->GetPixel( nYOrg, nXOrg );

							if( nXDst > -1 && nXDst < nDstWidth && 
								nYDst > -1 && nYDst < nDstHeight &&
								mcMerge < 255 && pRAcc->GetPixel( nYDst, nXDst ) != aSet )
							{
								pRAcc->SetPixel( nYDst, nXDst, aSet );
								MergeProcPixel( nYDst, nXDst, pPntAcc->GetPixel( nYOrg, nXOrg ) );
							}
						}
					}
				}

				pRegionBmp->ReleaseAccess( pRAcc );
			}
			else
			{
				if( pMskAcc->GetBitCount() != 8 )
				{
					const BitmapColor aMskBlack( pMskAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
							if( pMskAcc->GetPixel( nYOrg, nXOrg ) == aMskBlack )
								SetProcPixel( nYDst, nXDst, pPntAcc->GetPixel( nYOrg, nXOrg ) );
				}
				else
				{
					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
					{
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
						{
							mcMerge = pMskAcc->GetPixel( nYOrg, nXOrg );

							if( mcMerge < 255 )
								MergeProcPixel( nYDst, nXDst, pPntAcc->GetPixel( nYOrg, nXOrg ) );
						}
					}
				}
			}
		}

		( (Bitmap&) rPaintBmp ).ReleaseAccess( pPntAcc );
		( (Bitmap&) rMaskBmp ).ReleaseAccess( pMskAcc );

		bRet = TRUE;
	}

	return bRet;
}

// -----------------------------------------------------------------------------

BOOL SvImage::PaintInProcessing( long nY, long nX, 
								 const Color& rPaintCol, const Bitmap& rMaskBmp,
								 Bitmap* pRegionBmp )
{
	BOOL bRet = FALSE;

	if( IsInProcessing() )
	{
		BitmapReadAccess* pMskAcc = ( (Bitmap&) rMaskBmp ).AcquireReadAccess();

		if( pMskAcc )
		{
			BitmapWriteAccess*	pRAcc = pRegionBmp ? ( (Bitmap*) pRegionBmp )->AcquireWriteAccess() : NULL;
			const long			nWidth = pMskAcc->Width(), nHeight = pMskAcc->Height();
			const long			nDstWidth = GetDestWidth(), nDstHeight = GetDestHeight();
			long				nXOrg, nXDst, nYOrg, nYDst;
			BitmapColor			aPntCol( rPaintCol.GetRed(), rPaintCol.GetGreen(), rPaintCol.GetBlue() );

			if( pRAcc )
			{
				const BitmapColor aSet( pRAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

				if( pMskAcc->GetBitCount() != 8 )
				{
					const BitmapColor aMskBlack( pMskAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
					{
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
						{
							if( nXDst > -1 && nXDst < nDstWidth && 
								nYDst > -1 && nYDst < nDstHeight &&
								pRAcc->GetPixel( nYDst, nXDst ) != aSet &&
								pMskAcc->GetPixel( nYOrg, nXOrg ) == aMskBlack )
							{
								pRAcc->SetPixel( nYDst, nXDst, aSet );
								SetProcPixel( nYDst, nXDst, aPntCol );
							}
	    				}
					}
				}
				else
				{
					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
					{
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
						{
							mcMerge = pMskAcc->GetPixel( nYOrg, nXOrg );

							if( nXDst > -1 && nXDst < nDstWidth && 
								nYDst > -1 && nYDst < nDstHeight &&
								mcMerge < 255 && pRAcc->GetPixel( nYDst, nXDst ) != aSet )
							{
								pRAcc->SetPixel( nYDst, nXDst, aSet );
								MergeProcPixel( nYDst, nXDst, aPntCol );
							}
						}
					}
				}

				pRegionBmp->ReleaseAccess( pRAcc );
			}
			else
			{
				if( pMskAcc->GetBitCount() != 8 )
				{
					const BitmapColor aMskBlack( pMskAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
							if( pMskAcc->GetPixel( nYOrg, nXOrg ) == aMskBlack )
								SetProcPixel( nYDst, nXDst, aPntCol );
				}
				else
				{
					for( nYOrg = 0, nYDst = nY; nYOrg < nHeight; nYOrg++, nYDst++ )
					{
						for( nXOrg = 0, nXDst = nX; nXOrg < nWidth; nXOrg++, nXDst++ )
						{
							mcMerge = pMskAcc->GetPixel( nYOrg, nXOrg );

							if( mcMerge < 255 )
								MergeProcPixel( nYDst, nXDst, aPntCol );
						}
					}
				}
			}
		}

		( (Bitmap&) rMaskBmp ).ReleaseAccess( pMskAcc );
		bRet = TRUE;
	}

	return bRet;
}

// -----------------------------------------------------------------------------

void SvImage::ImplPaintRegion( const Region& rRegion, BYTE cMerge )
{
	Region aReg( rRegion );

	aReg.Intersect( Rectangle( Point(), Size( GetSrcWidth(), GetSrcHeight() ) ) );

	if( !aReg.IsEmpty() )
	{
		Rectangle		aRect;
		RegionHandle	aRegHandle( aReg.BeginEnumRects() );
		const long		nHeight = GetDestHeight();

		if( !cMerge )
		{
			while( aReg.GetNextEnumRect( aRegHandle, aRect ) )
			{
				for( long nY = aRect.Top(), nEndY = aRect.Bottom(); nY <= nEndY; nY++ )
				{
					HPBYTE pTempDest = GetProcPointer( nY, aRect.Left() );

					for( long nX = aRect.Left(), nEndX = aRect.Right(); nX <= nEndX; nX++ )
					{
						*pTempDest++ = mcB;
						*pTempDest++ = mcG;
						*pTempDest++ = mcR;
					}
				}
			}
		}
		else
		{
			BitmapColor aCol( mcR, mcG, mcB );

			mcMerge = cMerge;

			while( aReg.GetNextEnumRect( aRegHandle, aRect ) )
				for( long nY = aRect.Top(), nEndY = aRect.Bottom(); nY <= nEndY; nY++ )
					for( long nX = aRect.Left(), nEndX = aRect.Right(); nX <= nEndX; nX++ )
						MergeProcPixel( nY, nX, aCol );
		}

		aReg.EndEnumRects( aRegHandle );
	}
}

// -----------------------------------------------------------------------------

void SvImage::ImplPaintPolyPolyLine( const PolyPolygon& rPolyPoly, BOOL bClose, BYTE cMerge )
{
	if( rPolyPoly.Count() )
	{
		Link aLink;

		if( cMerge )
		{
			mcMerge = cMerge;
			aLink = LINK( this, SvImage, MergeLineEnumHdl );
		}
		else
			aLink = LINK( this, SvImage, LineEnumHdl );

		for( USHORT i = 0, nCount = rPolyPoly.Count(); i < nCount; i++ )
		{
			const Polygon&	rPoly = rPolyPoly[ i ];
			const USHORT	nSize = rPoly.GetSize();

			if( nSize > 1 )
			{
				for( USHORT n = 0, nSize1 = nSize - 1; n < nSize1; n++ )
				{
					Line aLine( rPoly[ n ], rPoly[ n + 1 ] );

					bLineSet = !n && !bClose;
					aLine.Enum( aLink );
				}

				if( bClose && ( rPoly[ nSize - 1 ] != rPoly[ 0 ] ) )
				{
					Line aLine( rPoly[ nSize - 1 ], rPoly[ 0 ] );

					bLineSet = FALSE;
					aLine.Enum( aLink );
				}
			}
		}
	}
}

// -----------------------------------------------------------------------------

void SvImage::ImplPaintMask( Bitmap& rMask, const Rectangle& rBound, BYTE cMerge )
{
	Point		aNullPt;
	Rectangle	aBound( rBound );

	aBound.Justify();
	aBound.Intersection( Rectangle( aNullPt, Size( GetSrcWidth(), GetSrcHeight() ) ) );
	aBound.Intersection( Rectangle( aNullPt, rMask.GetSizePixel() ) );

	if( !aBound.IsEmpty() )
	{
		BitmapReadAccess* pMAcc = rMask.AcquireReadAccess();

		if( pMAcc )
		{
			const BitmapColor aMB( pMAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

			if( !cMerge )
			{
				for( long nY = aBound.Top(), nEndY = aBound.Bottom(); nY <= nEndY; nY++ )
				{
					HPBYTE pTempDest = GetProcPointer( nY, aBound.Left() );

					for( long nX = aBound.Left(), nEndX = aBound.Right(); nX <= nEndX; nX++ )
					{
						if( pMAcc->GetPixel( nY, nX ) == aMB )
						{
							*pTempDest++ = mcB;
							*pTempDest++ = mcG;
							*pTempDest++ = mcR;
						}
						else
							pTempDest += 3;
					}
				}
			}
			else
			{
				BitmapColor aCol( mcR, mcG, mcB );

				mcMerge = cMerge;

				for( long nY = aBound.Top(), nEndY = aBound.Bottom(); nY <= nEndY; nY++ )
					for( long nX = aBound.Left(), nEndX = aBound.Right(); nX <= nEndX; nX++ )
						if( pMAcc->GetPixel( nY, nX ) == aMB )
							MergeProcPixel( nY, nX, aCol );
			}

			rMask.ReleaseAccess( pMAcc );
		}
	}
}

// -----------------------------------------------------------------------------

IMPL_LINK( SvImage, LineEnumHdl, Point*, pPt )
{
	if( bLineSet )
		SetProcPixel( pPt->Y(), pPt->X() );
	else
		bLineSet = TRUE;

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SvImage, MergeLineEnumHdl, Point*, pPt )
{
	if( bLineSet )
		MergeProcPixel( pPt->Y(), pPt->X(), BitmapColor( mcR, mcG, mcB ) );
	else
		bLineSet = TRUE;

	return 0L;
}
