/*************************************************************************
 *
 *  $RCSfile: svpopart.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <time.h>
#include "svimage.hxx"
#include <sfx2/progress.hxx>
#include <sfx2/viewfrm.hxx>
#include "simresid.hxx"
#include "strings.hrc"

/******************************************************************************/

#define BGR9COL(B, G, R) ((((ULONG)(B)>>5)<<6)|(((ULONG)(G)>>5)<<3)|((ULONG)(R)>>5))

/******************************************************************************/

BOOL SvImage::ImpPopart(SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if( BeginProcessing() )
	{
		const ULONG		nColors = 256; 
		BitmapPalette	aPal( nColors );
		ULONG			nWidth = GetOrgWidth(), nHeight = GetOrgHeight(), nAlignedWidth = GetOrgAlignedWidth(), nX, nY;
		ULONG			nBlue, nGreen, nRed;
		HPBYTE 			pOrgArray = GetOrgPointer();
		HPBYTE 			pDestArray = GetDestPointer();
		HPBYTE 			pTempOrg;
		HPBYTE 			pTempDest;
		ULONG*			pColors = new ULONG[ 512 ];
		ULONG			nCond = 4 - ((nWidth * 3) & 3);
		BOOL			bCond = (BOOL)((nWidth*3) & 3);

        SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_POPART), (ULONG)nHeight*3);

		memset( pColors, 0, 512 * sizeof( ULONG ) );
 		pTempOrg = pOrgArray;

		for( nY = 0; nY < nHeight; nY++ )
		{
			for( nX = 0; nX < nWidth; nX++ )
			{
				nBlue = *pTempOrg++;
				nGreen = *pTempOrg++;
				nRed = *pTempOrg++;
				pColors[ BGR9COL( nBlue, nGreen, nRed ) ]++;
			}

			// Long-Alignment der Zeile beachten
			if( bCond )
				pTempOrg += nCond;

			if( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

   		// Popularity-Algorithmus
		USHORT nPalIndex = 0;
   		
		for( nX = 0; nX < nColors; nX++ )
   		{
       		ULONG nCount = 0, nIndex = 0xffffffffUL;

      		for( nY = 0; nY < 512; nY++ )
      		{
       			if( pColors[ nY ] > nCount )
           		{
           			nCount = pColors[ nY ];
           			nIndex = nY;
				}
   			}

			if( nIndex != 0xffffffffUL )
			{
				BitmapColor& rCol = aPal[ nPalIndex++ ];
				
				rCol.SetRed( ( ( nIndex & 7 ) << 5 ) | 63 );
				rCol.SetGreen( ( ( ( nIndex >> 3 ) & 7 ) << 5 ) | 63 );
				rCol.SetBlue( ( ( ( nIndex >> 6 ) & 7 ) << 5 ) | 63 );
	       		pColors[ nIndex ] = 0;
			}
			else
			{
				aPal.SetEntryCount( nPalIndex );
				break;
			}
 		}

		// Indizes erzeugen
		for( nX = 0; nX < 512; nX++ )
		{
			BitmapColor aTestCol( ( nX & 7 ) << 5, ( ( nX >> 3 ) & 7 ) << 5, ( ( nX >> 6 ) & 7 ) << 5 );
			ULONG		nLastErr = aTestCol.GetColorError( aPal[ 0 ] );
 			ULONG		nIndex = 0;

 			for( nY = 1; nY < aPal.GetEntryCount(); nY++ )
 			{
				const ULONG nActErr = aTestCol.GetColorError( aPal[ nY ] );

				if( nActErr < nLastErr )
 					nIndex = nY, nLastErr = nActErr;
 			}

 			pColors[ nX ] = nIndex;
		}

		// swap palette
		BitmapPalette	aSwapPal( aPal.GetEntryCount() );
		long			nLastIndex = aPal.GetEntryCount() - 1;
		
		srand( Time::GetSystemTicks() );

		for( nX = 0; nX < aSwapPal.GetEntryCount(); nX++, nLastIndex-- )
		{
			const ULONG nIndex = FRound( (double) rand() / RAND_MAX * nLastIndex );

			BitmapColor aSwapCol( aPal[ nIndex ] );
			BitmapColor	aLast( aPal[ nLastIndex ] );

			aSwapPal[ nX ] = aPal[ nLastIndex ] = aSwapCol;
			aPal[ nIndex ] = aLast;
		}

		// Farben des Bildes ueber die erzeugten Indizes der Palette zuordnen
		pTempOrg = pOrgArray;
		pTempDest= pDestArray;
		for( nY = 0; nY < nHeight; nY++ )
		{
			for( nX = 0; nX < nWidth; nX++ )
			{
				nBlue = *pTempOrg++;
				nGreen = *pTempOrg++;
				nRed = *pTempOrg++;

				const BitmapColor& rCol = aSwapPal[ pColors[ BGR9COL( nBlue, nGreen, nRed ) ] ];

				*pTempDest++ = rCol.GetBlue();
				*pTempDest++ = rCol.GetGreen();
				*pTempDest++ = rCol.GetRed();
			}

			// Long-Alignment der Zeile beachten
			if( bCond )
			{
				pTempOrg += nCond;
				pTempDest+= nCond;
			}

			if( !( nY % 4 ) )
				aProgress.SetState( nHeight + nY );
		}

		// EIP beenden
		EndProcessing();
		delete[] pColors;
		*GetPalette() = aSwapPal;
		bRet = TRUE;
	}

	return bRet;
}

/******************************************************************************/

BOOL SvImage::ImpPopart(const Rectangle& rDestRect, SfxViewFrame* pFrame)
{
	BOOL bRet = FALSE;

	// EIP einleiten
	if( BeginProcessing() )
	{
		const ULONG		nColors = 256; 
		BitmapPalette	aPal( nColors );
		ULONG			nWidth = GetOrgWidth(), nHeight = GetOrgHeight(), nAlignedWidth = GetOrgAlignedWidth(), nX, nY;
		ULONG			nBlue, nGreen, nRed;
		HPBYTE 			pOrgArray = GetOrgPointer();
		HPBYTE 			pDestArray = GetDestPointer();
		HPBYTE 			pTempOrg;
		HPBYTE 			pTempDest;
		ULONG*			pColors = new ULONG[ 512 ];
		ULONG			nCond = 4 - ((nWidth * 3) & 3);
		BOOL			bCond = (BOOL)((nWidth*3) & 3);
		ULONG			nOff, nXOff, nYOff;
		Point			aPoint;
		Rectangle		aOrgRect( aPoint, Size( nWidth, nHeight ) );

		aOrgRect.Intersection( rDestRect );
		nXOff = 3 * aOrgRect.Left();
		nYOff = nHeight - aOrgRect.Top() - aOrgRect.GetHeight();

		// altes Array komplett kopieren
		HMEMCPY( pDestArray, pOrgArray, nAlignedWidth * nHeight );
        SfxProgress aProgress( pFrame->GetObjectShell(), SimResId(STR_STAT_POPART), nHeight + ( aOrgRect.GetHeight() << 1 ) );

		nWidth 	= aOrgRect.GetWidth();
		nHeight = aOrgRect.GetHeight();

		memset( pColors, 0, 512 * sizeof( ULONG ) );

		for( nY = 0; nY < nHeight; nY++ )
		{
			nOff = ( nYOff + nY ) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;

			for( nX = 0; nX < nWidth; nX++ )
			{
				nBlue = *pTempOrg++;
				nGreen = *pTempOrg++;
				nRed = *pTempOrg++;
				pColors[ BGR9COL( nBlue, nGreen, nRed ) ]++;
			}

			if( !( nY % 4 ) )
				aProgress.SetState( nY + 1 );
		}

   		// Popularity-Algorithmus
		USHORT nPalIndex = 0;
   		
		for( nX = 0; nX < nColors; nX++ )
   		{
       		ULONG nCount = 0, nIndex = 0xffffffffUL;

      		for( nY = 0; nY < 512; nY++ )
      		{
       			if( pColors[ nY ] > nCount )
           		{
           			nCount = pColors[ nY ];
           			nIndex = nY;
				}
   			}

			if( nIndex != 0xffffffffUL )
			{
				BitmapColor& rCol = aPal[ nPalIndex++ ];
				
				rCol.SetRed( ( ( nIndex & 7 ) << 5 ) | 63 );
				rCol.SetGreen( ( ( ( nIndex >> 3 ) & 7 ) << 5 ) | 63 );
				rCol.SetBlue( ( ( ( nIndex >> 6 ) & 7 ) << 5 ) | 63 );
	       		pColors[ nIndex ] = 0;
			}
			else
			{
				aPal.SetEntryCount( nPalIndex );
				break;
			}
 		}

		// Indizes erzeugen
		for( nX = 0; nX < 512; nX++ )
		{
			BitmapColor aTestCol( ( nX & 7 ) << 5, ( ( nX >> 3 ) & 7 ) << 5, ( ( nX >> 6 ) & 7 ) << 5 );
			ULONG		nLastErr = aTestCol.GetColorError( aPal[ 0 ] );
 			ULONG		nIndex = 0;

 			for( nY = 1; nY < aPal.GetEntryCount(); nY++ )
 			{
				const ULONG nActErr = aTestCol.GetColorError( aPal[ nY ] );

				if( nActErr < nLastErr )
 					nIndex = nY, nLastErr = nActErr;
 			}

 			pColors[ nX ] = nIndex;
		}

		// swap palette
		BitmapPalette	aSwapPal( aPal.GetEntryCount() );
		long			nLastIndex = aPal.GetEntryCount() - 1;
		
		srand( Time::GetSystemTicks() );

		for( nX = 0; nX < aSwapPal.GetEntryCount(); nX++, nLastIndex-- )
		{
			const ULONG nIndex = FRound( (double) rand() / RAND_MAX * nLastIndex );

			BitmapColor aSwapCol( aPal[ nIndex ] );
			BitmapColor	aLast( aPal[ nLastIndex ] );

			aSwapPal[ nX ] = aPal[ nLastIndex ] = aSwapCol;
			aPal[ nIndex ] = aLast;
		}

		// Farben des Bildes ueber die erzeugten Indizes der Palette zuordnen
		for( nY = 0; nY < nHeight; nY++ )
		{
			nOff = ( nYOff + nY ) * GetOrgAlignedWidth() + nXOff;
			pTempOrg = pOrgArray + nOff;
			pTempDest = pDestArray + nOff;

			for( nX = 0; nX < nWidth; nX++ )
			{
				nBlue = *pTempOrg++;
				nGreen = *pTempOrg++;
				nRed = *pTempOrg++;

				const BitmapColor& rCol = aSwapPal[ pColors[ BGR9COL( nBlue, nGreen, nRed ) ] ];

				*pTempDest++ = rCol.GetBlue();
				*pTempDest++ = rCol.GetGreen();
				*pTempDest++ = rCol.GetRed();
			}

			if( !( nY % 4 ) )
				aProgress.SetState( nHeight + nY );
		}

		// EIP beenden
		EndProcessing();
		delete[] pColors;
		GetPalette()->SetEntryCount( 0 );
		bRet = TRUE;
	}

	return bRet;
}
