/*************************************************************************
 *
 *  $RCSfile: svrotate.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "svimage.hxx"
#include <vcl/poly.hxx>
#include <sfx2/progress.hxx>
#include <sfx2/viewfrm.hxx>
#include "simresid.hxx"
#include "strings.hrc"

// -----------
// - SvImage -
// -----------

BOOL SvImage::ImpRotate(float fAngle, const Color& rColor, SfxViewFrame* pFrame)
{
	long nWidth, nHeight, nNewWidth, nNewHeight, nAlignedWidth;

	fAngle = -fAngle;

	if( fAngle < 0.0 )
		fAngle = 360.0 + fAngle;

	nWidth = GetOrgWidth(), nHeight = GetOrgHeight(), nAlignedWidth = GetOrgAlignedWidth();

	Point		aTmpPoint;
	Rectangle	aTmpRectangle( aTmpPoint, Size( nWidth, nHeight ) );
	Polygon 	aPoly( aTmpRectangle ); aPoly.Rotate( aTmpPoint, (USHORT) FRound( fAngle * 10.0 ) );
	Rectangle	aNewBound( aPoly.GetBoundRect() );
	const Size	aNewSizePix( aNewBound.GetSize() );

	if( BeginProcessingExt( nNewWidth = aNewBound.GetWidth(), nNewHeight = aNewBound.GetHeight() ) )
	{
        SfxProgress		aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_ROTATE), (ULONG)nHeight << 1);
		HPBYTE 			pOrgArray = GetOrgPointer();
		HPBYTE 			pDestArray = GetDestPointer();
		HPBYTE 			pTempOrg;
		const double	fCosAngle = cos( fAngle * F_PI180 );
		const double	fSinAngle = sin( fAngle * F_PI180 );
		const double	fXMin = aNewBound.Left();
		const double	fYMin = aNewBound.Top();
		long			nX, nY, nRotX, nRotY, nSinY, nCosY;
		long*			pCosX = new long[ nNewWidth ];
		long*			pSinX = new long[ nNewWidth ];
		long*			pCosY = new long[ nNewHeight ];
		long*			pSinY = new long[ nNewHeight ];
		const ULONG		nCond24	= 4 - ((nNewWidth*3)&3);
		const BOOL		bCond24	= (BOOL)((nNewWidth*3)&3);
		const BYTE		cBlue = rColor.GetBlue(), cGreen = rColor.GetGreen(), cRed = rColor.GetRed();

		for( nX = 0; nX < nNewWidth; nX++ )
		{
			const double fTmp = ( fXMin + nX ) * 64.;

			pCosX[ nX ] = FRound( fCosAngle * fTmp );
			pSinX[ nX ] = FRound( fSinAngle * fTmp );
		}

		for( nY = 0; nY < nNewHeight; nY++ )
		{
			const double fTmp = ( fYMin + nY ) * 64.;

			pCosY[ nY ] = FRound( fCosAngle * fTmp );
			pSinY[ nY ] = FRound( fSinAngle * fTmp );
		}

		for( nY = 0L; nY < nNewHeight; nY++ )
		{
			nSinY = pSinY[ nY ];
			nCosY = pCosY[ nY ];

			for( nX = 0L; nX < nNewWidth; nX++ )
			{
				nRotX = ( pCosX[ nX ] - nSinY ) >> 6;
				nRotY = ( pSinX[ nX ] + nCosY ) >> 6;

				if( ( nRotX > -1L ) && ( nRotX < nWidth ) && ( nRotY > -1L ) && ( nRotY < nHeight ) )
				{
					pTempOrg = pOrgArray + nRotY * nAlignedWidth + 3 * nRotX;
					*pDestArray++ = *pTempOrg++;
					*pDestArray++ = *pTempOrg++;
					*pDestArray++ = *pTempOrg;
				}
				else
				{
					*pDestArray++ = cBlue;
					*pDestArray++ = cGreen;
					*pDestArray++ = cRed;
				}
			}

			if( bCond24 )
				pDestArray += nCond24;

			if( !( nY % 4 ) ) 
				aProgress.SetState( nY + 1 );	
		}

		delete[] pSinX;
		delete[] pCosX;
		delete[] pSinY;
		delete[] pCosY;
		EndProcessingExt();
		GetPalette()->SetEntryCount( 0 );
		return TRUE;
	}

	return FALSE;
}


/******************************************************************************/


BOOL SvImage::ImpRotate90P(SfxViewFrame* pFrame)
{
	USHORT	nWidth;
	USHORT	nHeight;
	USHORT	nWidth2;
	USHORT	nHeight2;
	USHORT	nHeight22;
	ULONG	nAlignedWidth;
	USHORT	nX;
	USHORT	nY;
	HPBYTE 	pOrgArray;
	HPBYTE 	pDestArray;
	HPBYTE 	pTempOrg;
	BOOL	bCond24;
	ULONG	nCond24;


	nWidth = (USHORT)GetOrgWidth();
	nHeight= (USHORT)GetOrgHeight();
	nAlignedWidth = GetOrgAlignedWidth();
	nWidth2 = nHeight;
	nHeight2 = nWidth;
	nHeight22 = nHeight2 - 1;

	// EIP einleiten
	if (BeginProcessingExt(nWidth2, nHeight2))
	{
		bCond24	= (BOOL)((nWidth2*3) & 3);
		nCond24	= 4 - ((nWidth2 * 3) & 3);
		pOrgArray = GetOrgPointer();
		pDestArray= GetDestPointer();

        SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_ROTATE), (ULONG)nHeight << 1);

		for (nY = 0; nY < nHeight2; nY++)
		{
			pTempOrg = pOrgArray + 3 * nHeight22--;

			for(nX = 0; nX < nWidth2; nX++)
			{
				*pDestArray++ = *(pTempOrg);
				*pDestArray++ = *(pTempOrg + 1);
				*pDestArray++ = *(pTempOrg + 2);

				pTempOrg += nAlignedWidth;
			}

			if (bCond24)
				pDestArray += nCond24;

			if (!(nY%4)) 
				aProgress.SetState(nY+1);	
		}

		// EIP beenden
		EndProcessingExt();
		return TRUE;
	}

	else return FALSE;
}


/******************************************************************************/


BOOL SvImage::ImpRotate90N(SfxViewFrame* pFrame)
{
	USHORT	nWidth;
	USHORT	nHeight;
	USHORT	nWidth2;
	USHORT	nHeight2;
	ULONG	nAlignedWidth;
	USHORT	nX;
	USHORT	nY;
	HPBYTE 	pOrgArray;
	HPBYTE 	pDestArray;
	HPBYTE 	pTempOrg;
	BOOL	bCond24;
	ULONG	nCond24;


	nWidth = (USHORT)GetOrgWidth();
	nHeight= (USHORT)GetOrgHeight();
	nAlignedWidth = GetOrgAlignedWidth();

	// neue Groesse bestimmen
	nWidth2 	= nHeight;
	nHeight2	= nWidth;


	// EIP einleiten
	if (BeginProcessingExt(nWidth2, nHeight2))
	{
		bCond24	= (BOOL)((nWidth2*3) & 3);
		nCond24	= 4 - ((nWidth2 * 3) & 3);
		pOrgArray = GetOrgPointer() + nAlignedWidth*(ULONG)(nHeight - 1);
		pDestArray= GetDestPointer();

        SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_ROTATE), (ULONG)nHeight << 1);

		for (nY = 0; nY < nHeight2; nY++)
		{
			pTempOrg = pOrgArray + (ULONG)nY*3;

			for(nX = 0; nX < nWidth2; nX++)
			{
				*pDestArray++ = *(pTempOrg);
				*pDestArray++ = *(pTempOrg + 1);
				*pDestArray++ = *(pTempOrg + 2);

				pTempOrg -= nAlignedWidth;
			}

			if (bCond24)
				pDestArray += nCond24;

			if (!(nY%4)) 
				aProgress.SetState(nY+1);	
		}

		// EIP beenden
		EndProcessingExt();
		return TRUE;
	}

	else return FALSE;
}


/******************************************************************************/


BOOL SvImage::ImpRotate180(SfxViewFrame* pFrame)
{
	USHORT	nWidth;
	USHORT	nHeight;
	ULONG	nAlignedWidth;
	USHORT	nX;
	USHORT	nY;
	HPBYTE 	pOrgArray;
	HPBYTE 	pDestArray;
	HPBYTE 	pTempOrg;
	BOOL	bCond24;
	ULONG	nCond24;


	nWidth = (USHORT)GetOrgWidth();
	nHeight= (USHORT)GetOrgHeight();
	nAlignedWidth = GetOrgAlignedWidth();


	// EIP einleiten
	if (BeginProcessing())
	{
		bCond24	= (BOOL)((nWidth*3) & 3);
		nCond24	= 4 - ((nWidth * 3) & 3);

		pOrgArray = GetOrgPointer() +
					nAlignedWidth*(ULONG)(nHeight - 1) +
					(ULONG)(nWidth - 1) * 3;

		pDestArray	= GetDestPointer();

        SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_ROTATE), (ULONG)nHeight << 1);

		for (nY = 0; nY < nHeight; nY++)
		{
			pTempOrg = 	pOrgArray - nAlignedWidth * (ULONG)nY;


			for(nX = 0; nX < nWidth; nX++)
			{
				*pDestArray++ = *(pTempOrg);
				*pDestArray++ = *(pTempOrg + 1);
				*pDestArray++ = *(pTempOrg + 2);

				pTempOrg -= 3;
			}

			if (bCond24)
				pDestArray += nCond24;

			if (!(nY%4)) 
				aProgress.SetState(nY+1);	
		}

		// EIP beenden
		EndProcessingExt();
		return TRUE;
	}

	else return FALSE;
}
