/*************************************************************************
 *
 *  $RCSfile: svscale.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:30:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "svimage.hxx"
#include <sfx2/progress.hxx>
#include <sfx2/viewfrm.hxx>
#include "simresid.hxx"
#include "strings.hrc"

/******************************************************************************/

BOOL SvImage::ImpScale(float fScaleX, float fScaleY, SfxViewFrame* pFrame)
{
	USHORT	nWidth;
	USHORT	nWidth2;
	USHORT	nHeight;
	USHORT	nHeight2;
	ULONG	nAlignedWidth;
	ULONG	nAlignedWidth2;
	USHORT	nX;
	USHORT	nY;
	HPBYTE 	pOrgArray;
	HPBYTE 	pDestArray;
	HPBYTE 	pTempOrg;
	HPBYTE 	pTempDest;
	ULONG	nOffOrg;
    long*	pLutFX;
    long*	pLutFY;
    long*	pLutNX;
    long*	pLutNY;
    long	nTemp;
    long	lXB0, lXG0, lXR0;
    long	lXB1, lXG1, lXR1;
    float	fX;
    long	lX;
    ULONG	nYOff;

	nWidth 	= (USHORT) GetOrgWidth();
	nWidth2	= (USHORT) FRound( fScaleX * 0.01F * nWidth );
	nHeight	= (USHORT) GetOrgHeight();
	nHeight2= (USHORT) FRound( fScaleY * 0.01F * nHeight );

	if( ( nHeight < 2 ) || ( nWidth < 2 ) || ( nHeight2 < 2 ) || ( nWidth2 < 2 ) )
		return ImpScaleFast( fScaleX, fScaleY, pFrame );

	if( ( nHeight2 > 16000 ) || ( nWidth2 > 16000 ) )
		return FALSE;

    SfxProgress aProgress(pFrame->GetObjectShell(), SimResId(STR_STAT_SCALE), (ULONG)nHeight + (ULONG)nWidth2 + (ULONG)nHeight2);

     // Look-Up-Tabelle erzeugen, um schnell die Stellen zu
    // finden, von wo aus die neuen Werte gemappt werden
    pLutFX 	= new long[nWidth2];
    pLutNX 	= new long[nWidth2];

    // Look-Up-Tabellen fuellen
	fScaleX = (float) ( nWidth2 - 1 ) / ( nWidth - 1 );

    for (nX = 0; nX < nWidth2; nX++)
    {
    	// genauen Map-Wert finden
    	fX = (float) nX / fScaleX;

    	// Int-Wert (Pixel 0) bestimmen
    	pLutNX[nX] = VOS_BOUND( (USHORT) fX, 0, nWidth - 2 );

    	// Nachkommaanteil bestimmen fuer Geradenberechnung
    	fX -= (float) pLutNX[nX];

    	// Nachkommaanteil auf long erweitern (<< 10)
    	pLutFX[nX] = (long) (fX * 1024);
    	pLutNX[nX] *= 3;
    }

	// EIP einleiten fuer Skalierung in X-Richtung
	if ( BeginProcessingExt(nWidth2, nHeight, TRUE ) )
	{
		nAlignedWidth 	= GetOrgAlignedWidth();
		nAlignedWidth2	= GetDestAlignedWidth();

		pOrgArray = GetOrgPointer();
		pDestArray= GetDestPointer();

		for (nY = 0; nY < nHeight; nY++)
		{
			pTempOrg  = pOrgArray +  (ULONG)nY * nAlignedWidth;
			pTempDest = pDestArray + (ULONG)nY * nAlignedWidth2;

			for ( nX = 0; nX < nWidth2; nX++ )
			{
				nTemp = pLutNX[nX];
				lX = pLutFX[nX];

				lXB0 = (long)pTempOrg[nTemp++];
				lXG0 = (long)pTempOrg[nTemp++];
				lXR0 = (long)pTempOrg[nTemp++];

				lXB1 = (long)pTempOrg[nTemp++] - lXB0;
				lXG1 = (long)pTempOrg[nTemp++] - lXG0;
				lXR1 = (long)pTempOrg[nTemp]   - lXR0;

				*pTempDest++ = (BYTE) ((lXB1*lX + (lXB0<<10)) >> 10);
				*pTempDest++ = (BYTE) ((lXG1*lX + (lXG0<<10)) >> 10);
				*pTempDest++ = (BYTE) ((lXR1*lX + (lXR0<<10)) >> 10);
			}

			if (!(nY%16))
				aProgress.SetState(nY+1);
		}

		// EIP beenden
		EndProcessingExt();
	}

    // Look-Up-Tabelle erzeugen, um schnell die Stellen zu
    // finden, von wo aus die neuen Werte gemappt werden
	pLutFY 	= new long[nHeight2];
    pLutNY 	= new long[nHeight2];
	fScaleY = (float) ( nHeight2 - 1 ) / ( nHeight - 1 );

    // Look-Up-Tabelle fuellen
    for (nY = 0; nY < nHeight2; nY++)
    {
    	// genauen Map-Wert finden
    	fX = (float) nY / fScaleY;

    	// Int-Wert (Pixel 0) bestimmen
    	pLutNY[nY] = VOS_BOUND( (USHORT)fX, 0, nHeight - 2 );

    	// Nachkommaanteil bestimmen fuer Geradenberechnung
    	fX -= (float) pLutNY[nY];

    	// Nachkommaanteil auf long erweitern (<< 10)
    	pLutFY[nY]	= (long) ( fX * 1024 );
   	}


	// EIP einleiten fuer Skalierung in Y-Richtung
	if (BeginProcessingExt(nWidth2, nHeight2, FALSE))
	{
		pOrgArray = GetOrgPointer();
		pDestArray= GetDestPointer();

		nAlignedWidth 	= GetOrgAlignedWidth();
		nAlignedWidth2	= GetDestAlignedWidth();

		for (nX = 0; nX < nWidth2; nX++)
		{
			nOffOrg = 3*(ULONG)nX;
			pTempOrg  = pOrgArray + nOffOrg;
			pTempDest = pDestArray + nOffOrg;

			for (nY = 0; nY < nHeight2; nY++)
			{
				nTemp   = pLutNY[nY];
				lX		= pLutFY[nY];

				nYOff = (ULONG)nTemp * nAlignedWidth;

				lXB0 = (long) pTempOrg[nYOff];
				lXG0 = (long) pTempOrg[nYOff+1];
				lXR0 = (long) pTempOrg[nYOff+2];

				nYOff += nAlignedWidth;

				lXB1 = (long)pTempOrg[nYOff] 	- lXB0;
				lXG1 = (long)pTempOrg[nYOff+1] 	- lXG0;
				lXR1 = (long)pTempOrg[nYOff+2] 	- lXR0;

				*(pTempDest) 	= (BYTE) ((lXB1*lX + (lXB0<<10)) >> 10);
				*(pTempDest + 1)= (BYTE) ((lXG1*lX + (lXG0<<10)) >> 10);
				*(pTempDest + 2)= (BYTE) ((lXR1*lX + (lXR0<<10)) >> 10);

				pTempDest += nAlignedWidth2;
			}

			if (!(nX%16))
				aProgress.SetState(nHeight+nX);
		}

		// EIP beenden
		EndProcessingExt();
		GetPalette()->SetEntryCount( 0 );
   		delete[] pLutFX;
   		delete[] pLutNX;
   		delete[] pLutFY;
   		delete[] pLutNY;
		return TRUE;
	}
	else
	{
   		delete[] pLutFX;
   		delete[] pLutNX;
   		delete[] pLutFY;
   		delete[] pLutNY;
		DoUndo();
		return FALSE;
	}

}

/***************************************************************************/

BOOL SvImage::ImpScaleFast(float fScaleX, float fScaleY, SfxViewFrame* pFrame)
{
	long			nWidth;
	long			nNewWidth;
	long			nHeight;
	long			nNewHeight;

	nWidth = GetOrgWidth();
	nNewWidth	= FRound( fScaleX * nWidth * 0.01F );
	nHeight	= GetOrgHeight();
	nNewHeight = FRound( fScaleY * nHeight * 0.01F );

	if( !nWidth || !nHeight || !nNewWidth || !nNewHeight )
		return FALSE;

    SfxProgress aProgress( pFrame->GetObjectShell(), SimResId( STR_STAT_SCALE ), nNewHeight << 1 );

	if( BeginProcessingExt( nNewWidth, nNewHeight, TRUE ) )
	{
		HPBYTE 		pOrgArray = GetOrgPointer();
		HPBYTE 		pDestArray = GetDestPointer();
		HPBYTE 		pTmpOrg;
		HPBYTE 		pTmpDest;
		HPBYTE		pOrg;
		HPBYTE		pDest;
		const long	nOrgAlignedWidth = GetOrgAlignedWidth();
		const long	nDestAlignedWidth = GetDestAlignedWidth();
		const long	nWidth_1 = nWidth - 1L;
		const long	nHeight_1 = nHeight - 1L;
		const long	nNewWidth_1 = nNewWidth - 1L;
		const long	nNewHeight_1 = nNewHeight - 1L;
		long*		pLutX = new long[ nNewWidth ];
		long*		pLutY = new long[ nNewHeight ];
		long		nX, nY, nMapY, nActY = 0L;

		if( nNewWidth_1 )
		{
			for( nX = 0L; nX < nNewWidth; nX++ )
				pLutX[ nX ] = FRound( nX * nWidth_1 / (double) nNewWidth_1 );
		}
		else
			pLutX[ 0 ] = 0;

		if( nNewHeight_1 )
		{
			for( nY = 0L; nY < nNewHeight; nY++ )
				pLutY[ nY ] = FRound( nY * nHeight_1 / (double) nNewHeight_1 );
		}
		else
			pLutY[ 0 ] = 0;
				
		while( nActY < nNewHeight )
		{
			nMapY = pLutY[ nActY ];
			pTmpOrg = pOrgArray + nMapY * nOrgAlignedWidth;
			pDest = pTmpDest = pDestArray + nActY * nDestAlignedWidth;

			for( nX = 0L; nX < nNewWidth; nX++ )
			{
				pOrg = pTmpOrg + pLutX[ nX ] * 3;
				*pDest++ = *pOrg++;
				*pDest++ = *pOrg++;
				*pDest++ = *pOrg;
			}

			while( ( nActY < nNewHeight_1 ) && ( pLutY[ nActY + 1 ] == nMapY ) )
			{
				HMEMCPY( pTmpDest + nDestAlignedWidth, pTmpDest, nDestAlignedWidth );
				nActY++;
				pTmpDest += nDestAlignedWidth;

				if ( !( nActY % 16 ) )
					aProgress.SetState( nActY + 1 );
			}

			nActY++;
			if ( !( nActY % 16 ) )
				aProgress.SetState( nActY + 1 );
		}

		delete[] pLutX;
		delete[] pLutY;
		EndProcessingExt();
		return TRUE;
	}

	return FALSE;
}
