/*************************************************************************
 *
 *  $RCSfile: simdll.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <vcl/graph.hxx>
#include <vcl/clip.hxx>
#include <svtools/intitem.hxx>
#include <svx/svxids.hrc>
#include <svx/bmpmask.hxx>
#include <svx/colrctrl.hxx>
#include <svx/modctrl.hxx>
#include <svx/linectrl.hxx>
#include <svx/zoomctrl.hxx>
#include <svx/pszctrl.hxx>
#include "simapp.hrc"
#include "simdll.hxx"
#include "simmod.hxx"
#include "docshell.hxx"
#include "drviewsh.hxx"
#include "simmodel.hxx"
#include "tbxctrl.hxx"
#include "svimage.hxx"
#include "tbxctrl.hxx"
#include "linectrl.hxx"
#include "airctrl.hxx"
#include "colstat.hxx"
#include "alphactl.hxx"

// ----------
// - SimDLL -
// ----------

void SimDLL::Init()
{
	// called directly after loading the DLL
    // do whatever you want, you may use Sd-DLL too
    // the SdModule must be created
    SimModuleDummy** ppShlPtr = (SimModuleDummy**) GetAppData( SHL_SIM );

	if ( (*ppShlPtr)->IsLoaded() )
		return;

#ifndef SO3
    SvFactory* pFact = (*ppShlPtr)->pSimDrawDocShellFactory;
#else
    SvFactory* pFact = PTR_CAST(SvFactory,(*ppShlPtr)->pSimDrawDocShellFactory);
#endif
	SimModule* pMod;

	delete (*ppShlPtr);
	(*ppShlPtr) = pMod = new SimModule( pFact );
	pMod->pSimDrawDocShellFactory = pFact;

	SimDrawDocShell::Factory().RegisterMenuBar( SimResId( RID_SIM_MENU ) );
	SimDrawDocShell::Factory().RegisterAccel( SimResId( RID_SIM_ACCEL ) );

   	// register factories
	SimDrawViewShell::RegisterFactory( 1 );

	// register interfaces
	SimModule::RegisterInterface( pMod );
	SimDrawDocShell::RegisterInterface( pMod );
	SimDrawViewShell::RegisterInterface( pMod );

	// register controls
    SvxPosSizeStatusBarControl::RegisterControl( SID_ATTR_SIZE, pMod );
	SvxZoomStatusBarControl::RegisterControl( SID_ATTR_ZOOM, pMod );
	SvxModifyControl::RegisterControl( SID_DOC_MODIFIED, pMod );
	SimColorStatusBarControl::RegisterControl( SID_COLOR_STATUS, pMod );
	SimLineWidthControl::RegisterControl( SID_LINE_WIDTH, pMod );
	SimLineRoundControl::RegisterControl( SID_LINE_ROUND, pMod );
	SimAirControl::RegisterControl( SID_AIR_INTENS, pMod );
	SvxLineColorToolBoxControl::RegisterControl( SID_COLOR_FORE, pMod );
	SvxLineColorToolBoxControl::RegisterControl( SID_COLOR_BACK, pMod );
	SimAlphaControl::RegisterControl( SID_LINE_TRANSPARENCY, pMod );
	SimAlphaControl::RegisterControl( SID_FILL_TRANSPARENCY, pMod );
	// SimGridControl::RegisterControl( SID_SHOW_GRID, pMod );
	SimTbxControl::RegisterControl( SID_TOOLBOX_ZOOM, pMod );
	SimTbxControl::RegisterControl( SID_TOOLBOX_PICTURE, pMod );
	SimTbxControl::RegisterControl( SID_TOOLBOX_COLOR, pMod );
	SimTbxControl::RegisterControl( SID_TOOLBOX_EFFECTS, pMod );
	SimTbxControl::RegisterControl( SID_TOOLBOX_PIXEL, pMod );

	// register childwindows
	SvxColorChildWindow::RegisterChildWindow( TRUE, pMod, SFX_CHILDWIN_TASK );
	SvxBmpMaskChildWindow::RegisterChildWindow( FALSE, pMod );

	pMod->PutItem( SfxUInt16Item( SID_ATTR_METRIC, (UINT16) FUNIT_NONE ) );
}

// -----------------------------------------------------------------------------

void SimDLL::Exit()
{
	// called directly befor unloading the DLL
    // do whatever you want, Sd-DLL is accessible
    // the SimModule must be destroyed
    SimModuleDummy** ppShlPtr = (SimModuleDummy**) GetAppData( SHL_SIM );

	delete (*ppShlPtr);
	(*ppShlPtr) = NULL;
}

// -----------------------------------------------------------------------------

extern_c void __LOADONCALLAPI SimUpdateClipBrd(SvInPlaceObjectRef aIPObj, OutputDevice* pOut)
{
	if ( Clipboard::HasFormat( FORMAT_BITMAP ) )
		SimUpdateGraphic( aIPObj, Clipboard::PasteBitmap(), pOut );
}

// -----------------------------------------------------------------------------

extern_c void __LOADONCALLAPI SimUpdate(SvInPlaceObjectRef aIPObj, const Bitmap& rBitmap,
									  OutputDevice* pOut)
{
	SimUpdateGraphic( aIPObj, rBitmap, pOut );
}

// -----------------------------------------------------------------------------

extern_c void __LOADONCALLAPI SimGetBitmap(SvInPlaceObjectRef aIPObj, Bitmap* pBitmap)
{
	Graphic aGraphic;

	SimGetGraphic( aIPObj, &aGraphic );
	*pBitmap = aGraphic.GetBitmap();
}

// -----------------------------------------------------------------------------

extern_c void __LOADONCALLAPI SimUpdateGraphic(SvInPlaceObjectRef aIPObj, 
											   const Graphic& rGraphic, 
											   OutputDevice* pOut)
{
    SimDrawDocShellRef aSimDrawDocShellRef = &aIPObj;

    if ( aSimDrawDocShellRef.Is() )
    {
		aSimDrawDocShellRef->GetModel()->SetGraphic( rGraphic );

        SvData aEmpty;
        aIPObj->SendDataChanged(aEmpty);
        aIPObj->SendViewChanged();
    }
}

// -----------------------------------------------------------------------------

extern_c void __LOADONCALLAPI SimGetGraphic( SvInPlaceObjectRef aIPObj, Graphic* pGraphic )
{
    SimDrawDocShellRef  aSimDrawDocShellRef = &aIPObj;

    if ( aSimDrawDocShellRef.Is() )
    {
		const Graphic aGraphic( aSimDrawDocShellRef->GetModel()->GetGraphic() );

        if ( aGraphic.GetType() != GRAPHIC_NONE )
			*pGraphic = aGraphic;

        SvData aEmpty;
        aIPObj->SendDataChanged( aEmpty );
        aIPObj->SendViewChanged();
    }
}

// -----------------------------------------------------------------------------

extern_c void __LOADONCALLAPI SimReplaceGraphic( SimDrawDocShell* pDocShell, Graphic* pGraphic )
{
	DBG_ASSERT( pDocShell, "Wo ist die ObjectShell!?" );
	pDocShell->ReplaceGraphic( pGraphic );
}
