/*************************************************************************
 *
 *  $RCSfile: simmod.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_COLOR	0
#define ITEMID_SEARCH	SID_SEARCH_ITEM
#define _SIM_DLL		

#ifdef UNX
#define SCANNER_NO_SOURCE_RESID  STR_SANE_NO_SOURCE
#elif defined WIN || defined WNT
#define SCANNER_NO_SOURCE_RESID  STR_TWAIN_NO_SOURCE
#endif

#include <unotools/processfactory.hxx>
#include <toolkit/unohlp.hxx>
#include <cppuhelper/implbase1.hxx>
#include <vos/xception.hxx>
#include <tools/string.hxx>
#include <vcl/graph.hxx>
#include <vcl/status.hxx>
#include <vcl/msgbox.hxx>
#include <vcl/config.hxx>
#include <vcl/clip.hxx>
#include <vcl/salbtype.hxx>
#include <svtools/intitem.hxx>
#include <svtools/stritem.hxx>
#include <sfx2/app.hxx>
#include <sfx2/bindings.hxx>
#include <sfx2/dispatch.hxx>
#include <sfx2/iodlg.hxx>
#include <sfx2/request.hxx>
#include <sfx2/objface.hxx>
#include <sfx2/inimgr.hxx>
#include <sfx2/docfile.hxx>
#include <sfx2/fcontnr.hxx>
#include <svx/colritem.hxx>
#include <svx/svxids.hrc>
#include <svx/srchitem.hxx>
#include <svx/pszctrl.hxx>
#include "simmod.hxx"
#include "simresid.hxx"
#include "docshell.hxx"
#include "simapp.hrc"
#include "strings.hrc"
#include "defaults.hxx"
#include "simattr.hxx"

#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_AWT_XBITMAP_HPP_
#include <com/sun/star/awt/XBitmap.hpp>
#endif

#define SimModule
#include "simslots.hxx"

// -----------
// - Defines -
// -----------

#define TEXT_WIDTH( s )   rBar.GetTextSize(( s )).Width()

// ------------------------
// - ScannerEventListener -
// ------------------------

class ScannerEventListener : public ::cppu::WeakImplHelper1< ::com::sun::star::lang::XEventListener >
{
private:

	SimModule*				mpParent;

public:

							ScannerEventListener( SimModule* pParent ) : mpParent( pParent )  {};
							~ScannerEventListener();

	// XEventListener
	virtual void SAL_CALL 	disposing( const ::com::sun::star::lang::EventObject& rEventObject );
	
	void					ParentDestroyed() { mpParent = NULL; }
};

// -----------------------------------------------------------------------------

ScannerEventListener::~ScannerEventListener()
{
}

// -----------------------------------------------------------------------------

void SAL_CALL ScannerEventListener::disposing( const ::com::sun::star::lang::EventObject& rEventObject )
{
	if( mpParent )
		mpParent->ScannerEvent( rEventObject );
}

// -------------
// - SimModule -
// -------------

TYPEINIT1( SimModuleDummy, SfxModule );
TYPEINIT1( SimModule, SimModuleDummy );

// -----------------------------------------------------------------------------

SFX_IMPL_INTERFACE( SimModule, SfxModule, SimResId( STR_APPLICATIONOBJECTBAR ) )
{
    SFX_STATUSBAR_REGISTRATION( ResId( RID_SIM_STATUSBAR ) );
}

// -----------------------------------------------------------------------------

SimModule::SimModule( SvFactory* pObjFact ) :
			SimModuleDummy( SFX_APP()->CreateResManager( "sim" ), FALSE, pObjFact )
{
    SetName( String( RTL_CONSTASCII_USTRINGPARAM( "StarImage" ) ) );

	aNewDocTimer.SetTimeoutHdl( LINK( this, SimModule, NewDocHdl ) );
	aNewDocTimer.SetTimeout( 100 );

	pDefaults = new SimDefaults;
	pDefaults->SetDefault( FALSE );
    pDefaults->Initialize();

	InitPoolDefaults();

	::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > xMgr( ::utl::getProcessServiceFactory() );

	if( xMgr.is() )
	{
		mxScannerManager = ::com::sun::star::uno::Reference< ::com::sun::star::scanner::XScannerManager >( 
						   xMgr->createInstance( ::rtl::OUString::createFromAscii( "com.sun.star.scanner.ScannerManager" ) ),
						   ::com::sun::star::uno::UNO_QUERY );

		if( mxScannerManager.is() )
		{
			mxScannerListener = ::com::sun::star::uno::Reference< ::com::sun::star::lang::XEventListener >(
								static_cast< ::cppu::OWeakObject* >( new ScannerEventListener( this ) ), 
								::com::sun::star::uno::UNO_QUERY );
		}
	}
}

// -----------------------------------------------------------------------------

SimModule::~SimModule()
{
	if( mxScannerListener.is() )
		static_cast< ScannerEventListener* >( mxScannerListener.get() )->ParentDestroyed();

	DestroyPoolDefaults();

	pDefaults->SetDefault( FALSE );
	pDefaults->StoreConfig();
	SFX_APP()->SaveConfiguration();
	delete pDefaults;
}

// -----------------------------------------------------------------------------

SfxFileDialog* SimModule::CreateDocFileDialog( ULONG nBits, const SfxObjectFactory& rFact, const SfxItemSet* pSet )
{
    SfxFileDialog*	pDocDlg;
	SfxItemSet		aSet( GetPool(), 
						  SID_DOC_READONLY, SID_DOC_READONLY,
						  SID_PASSWORD, SID_PASSWORD, 
						  SID_LOAD_LAYOUT, SID_LOAD_LAYOUT, 
						  0 );
	const BOOL		bSaveAs = ( ( nBits & WB_SAVEAS ) == WB_SAVEAS );

	if( pSet )
		aSet.Put( *pSet );

	if( bSaveAs && ( SFXWB_INSERT != ( nBits & SFXWB_INSERT ) ) )
		nBits |= SFXWB_READONLY;

	pDocDlg = new SfxFileDialog( NULL, nBits, rFact, aSet );

	// set default filter
	if( ( nBits & WB_OPEN ) == WB_OPEN )
		pDocDlg->SetCurFilter( SimResId( STR_FILTERNAME_ALL ) );
	else if( bSaveAs )
	{
		SimDrawDocShell* pObjShell = PTR_CAST( SimDrawDocShell, SFX_APP()->GetActiveObjectShell() );

		if( pObjShell )
		{
			pObjShell->CheckFilter();

			SfxMedium* pMed = pObjShell->GetMedium();

			if( pMed && pMed->GetFilter() )
				pDocDlg->SetCurFilter( pMed->GetFilter()->GetName() );
		}	
	}

    return pDocDlg;
}

// -----------------------------------------------------------------------------

void SimModule::FillStatusBar( StatusBar& rBar )
{
	String	aPageStatus( SimResId( STR_FRAMENAME ) );

    rBar.InsertItem( SID_STATUS_PIC_WH, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( " 00000 x 00000 " ) ) ) );
	rBar.InsertItem( SID_STATUS_PIC_DEPTH, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( " 00 Bit " ) ) ) );
	rBar.InsertItem( SID_STATUS_PIC_SIZE, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( " 00000 KB " ) ) ) );
    rBar.InsertItem( SID_STATUS_MOUSEPOS, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( " 00000 / 00000 " ) ) ) );
    rBar.InsertItem( SID_ATTR_SIZE, 200, SIB_IN | SIB_USERDRAW );
    rBar.InsertItem( SID_ATTR_ZOOM, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( " 9999% " ) ) ) );
    rBar.InsertItem( SID_DOC_MODIFIED, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( " * " ) ) ) );
    rBar.InsertItem( SID_COLOR_STATUS, rBar.GetTextWidth( String( RTL_CONSTASCII_USTRINGPARAM( "      " ) ) ), SIB_IN | SIB_USERDRAW );
	rBar.InsertItem( SID_STATUS_FRAME, rBar.GetTextWidth( aPageStatus.Append( String( RTL_CONSTASCII_USTRINGPARAM( " 999 / 999 " ) ) ) ) );
}

// -----------------------------------------------------------------------------

void SimModule::Execute( SfxRequest& rReq )
{
    switch ( rReq.GetSlot() )
    {
        case SID_START_APP:
            SFX_DISPATCHER().Execute(SID_OPENDOC, SFX_CALLMODE_SYNCHRON);
        break;

        case SID_PASTE:
        {
			Graphic aGraphic;

			if ( Graphic::ClipboardHasFormat() && aGraphic.Paste() )
				CreateNewDocWithGraphic( aGraphic );
        }
        break;

#if defined WIN || defined WNT || defined UNX

        case SID_TWAIN_SELECT:
		{
			BOOL bDone = FALSE;

			if( mxScannerManager.is() )
			{
				try
				{
					const ::com::sun::star::uno::Sequence< ::com::sun::star::scanner::ScannerContext > 
						aContexts( mxScannerManager->getAvailableScanners() );
				
					if( aContexts.getLength() )
					{
						::com::sun::star::scanner::ScannerContext aContext( aContexts.getConstArray()[ 0 ] );
						bDone = mxScannerManager->configureScanner( aContext );
					}
				}
				catch(...)
				{
				}
			}

			if( !bDone )
				InfoBox( NULL, String( SimResId( SCANNER_NO_SOURCE_RESID ) ) ).Execute();
		}
		break;

        case SID_TWAIN_TRANSFER:
		{
			BOOL bDone = FALSE;

			if( mxScannerManager.is() )
			{
				try
				{
					const ::com::sun::star::uno::Sequence< ::com::sun::star::scanner::ScannerContext > 
						aContexts( mxScannerManager->getAvailableScanners() );
				
					if( aContexts.getLength() )
					{
						mxScannerManager->startScan( aContexts.getConstArray()[ 0 ], mxScannerListener );
						bDone = TRUE;
					}
				}
				catch(...)
				{
				}
			}

			if( !bDone )
			{
				InfoBox( NULL, String( SimResId( SCANNER_NO_SOURCE_RESID ) ) ).Execute();
			}
			else
			{
				SFX_BINDINGS().Invalidate( SID_TWAIN_SELECT );
				SFX_BINDINGS().Invalidate( SID_TWAIN_TRANSFER );
			}
		}
		break;

#endif

        default:
        break;
    }
}

// -----------------------------------------------------------------------------

void SimModule::GetState( SfxItemSet& rItemSet )
{
	SfxViewFrame* pFrame = SFX_APP()->GetViewFrame();

    if ( !Clipboard::HasFormat( FORMAT_BITMAP ) )
        rItemSet.DisableItem( SID_PASTE );

	rItemSet.DisableItem( SID_CONFIGEVENT );

#if defined WIN || defined WNT || defined UNX

	if( !mxScannerManager.is() || Application::IsRemoteServer() )
	{
		rItemSet.DisableItem( SID_TWAIN_SELECT );
		rItemSet.DisableItem( SID_TWAIN_TRANSFER );
	}

#endif
}

// -----------------------------------------------------------------------------

SfxModule* SimModuleDummy::Load()
{
	return NULL;
}

// -----------------------------------------------------------------------------

SvGlobalName SimModuleDummy::GetID( USHORT nFileFormat )
{
	SvGlobalName aGlobalName;

	if ( nFileFormat <= SOFFICE_FILEFORMAT_31 )
		aGlobalName = SvGlobalName( 0xEA60C941L, 0x2C6C, 0x101C, 0x8E, 0x2C, 0x00, 0x00, 0x1B, 0x4C, 0xC7, 0x11 );
	else
		aGlobalName = SvGlobalName( 0x447BB8A0L, 0x41FB, 0x11D0, 0x89, 0xCA, 0x00, 0x80, 0x29, 0xE4, 0xB0, 0xB1 );

	return aGlobalName;
}

// -----------------------------------------------------------------------------

USHORT SimModuleDummy::HasID( const SvGlobalName& aTestName )
{
	USHORT nRet = 0;

	if ( aTestName == GetID( SOFFICE_FILEFORMAT_40 ) )
		nRet = SOFFICE_FILEFORMAT_40;
	else if ( aTestName == GetID( SOFFICE_FILEFORMAT_31 ) )
		nRet = SOFFICE_FILEFORMAT_31;

	return nRet;
}

// -----------------------------------------------------------------------------

SfxModule* SimModule::Load()
{
	return this;
}

// -----------------------------------------------------------------------------

void SimModule::Free()
{
}

// -----------------------------------------------------------------------------

SfxItemSet*	 SimModule::CreateItemSet( USHORT nId )
{
	return( new SfxItemSet( SFX_APP()->GetPool() ) );
}

// -----------------------------------------------------------------------------

void SimModule::ApplyItemSet( USHORT nId, const SfxItemSet& rSet )
{
	pDefaults->SetDefault( FALSE );
	pDefaults->StoreConfig();
	SFX_APP()->SaveConfiguration();
}

// -----------------------------------------------------------------------------

SfxTabPage*	SimModule::CreateTabPage( USHORT nId, Window* pParent, const SfxItemSet& rSet )
{
	SfxTabPage*	 pRet = 0;
	switch( nId )

	{
		case( SID_SIM_TP_COLOR ):
		{
			pRet = SimStandards1TabPage::Create( pParent, rSet );
			( ( SimStandards1TabPage*) pRet )->SetDefaults( pDefaults );
		}
		break;

		case( SID_SIM_TP_EFFECTS ):
		{
			pRet = SimStandards2TabPage::Create( pParent, rSet );
			( ( SimStandards2TabPage*) pRet )->SetDefaults( pDefaults );
		}
		break;
	}

	DBG_ASSERT( pRet, "Id unbekannt" )

	return pRet;
}

// -----------------------------------------------------------------------------

void SimModule::CreateNewDocWithGraphic( const Graphic& rGraphic )
{
    SfxStringItem       aStringItem( SID_NEWDOCDIRECT, String( RTL_CONSTASCII_USTRINGPARAM( "simage" ) ) );
	const SfxPoolItem*	ppItems[] = { &aStringItem, 0  };

	// Neues ( leeres ) Dokument anlegen
	SFX_DISPATCHER().Execute( SID_NEWDOCDIRECT, SFX_CALLMODE_STANDARD, ppItems );

	aNewDocGraphic = rGraphic;
	aNewDocTimer.Start();
}

// -----------------------------------------------------------------------------

void SimModule::ScannerEvent( const ::com::sun::star::lang::EventObject& rEventObject )
{
	if( mxScannerManager.is() )
	{
		const ::com::sun::star::scanner::ScannerContext	aContext( mxScannerManager->getAvailableScanners().getConstArray()[ 0 ] );
		const ::com::sun::star::scanner::ScanError		eError = mxScannerManager->getError( aContext );

		if( ::com::sun::star::scanner::ScanError_ScanErrorNone == eError )
		{
			const ::com::sun::star::uno::Reference< ::com::sun::star::awt::XBitmap > xBitmap( mxScannerManager->getBitmap( aContext ) );

			if( xBitmap.is() )
			{
				const BitmapEx aScanBmp( VCLUnoHelper::GetBitmap( xBitmap ) );

				if( !!aScanBmp )
					CreateNewDocWithGraphic( aScanBmp );
			}
		}
	}

	SFX_BINDINGS().Invalidate( SID_TWAIN_SELECT );
	SFX_BINDINGS().Invalidate( SID_TWAIN_TRANSFER );
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimModule, NewDocHdl, Timer*, pTimer )
{
	SimDrawDocShell* pDocShell = (SimDrawDocShell*) SFX_APP()->GetActiveObjectShell();

	pTimer->Stop();

	// Bitmap in Dokument reinschiessen
	if ( pDocShell && pDocShell->ISA( SimDrawDocShell ) )
		pDocShell->ReplaceGraphic( &aNewDocGraphic );

	return 0L;
}

// -----------------------------------------------------------------------------

void SimModule::InitPoolDefaults()
{
    USHORT i;

	ppPoolDefaults = new SfxPoolItem*[ ATTR_END - ATTR_START + 1 ];

	for( i = ATTR_START; i < ATTR_PRINTOPT_START; i++ )
		ppPoolDefaults[ i - ATTR_START ] = new SfxVoidItem( i );

	// Printattribute sinnvoll belegen
	ppPoolDefaults[ ATTR_PRINTOPT_POSITION - ATTR_START ] = new SfxUInt16Item( ATTR_PRINTOPT_POSITION, 4 );
	ppPoolDefaults[ ATTR_PRINTOPT_WIDTH - ATTR_START ] = new SfxUInt32Item( ATTR_PRINTOPT_WIDTH, 0 );
	ppPoolDefaults[ ATTR_PRINTOPT_HEIGHT - ATTR_START ] = new SfxUInt32Item( ATTR_PRINTOPT_HEIGHT, 0 );
	ppPoolDefaults[ ATTR_PRINTOPT_DPI- ATTR_START ] = new SfxUInt16Item( ATTR_PRINTOPT_DPI, DPI_ORIGINAL );

	// Zeichenattribute sinnvoll belegt sein
	ppPoolDefaults[ ATTR_PNT_LINE_WIDTH	- ATTR_START ] = new SfxUInt16Item( ATTR_PNT_LINE_WIDTH, 1 );
	ppPoolDefaults[ ATTR_PNT_LINE_ROUND	- ATTR_START ] = new SfxUInt16Item( ATTR_PNT_LINE_ROUND, 0 );
	ppPoolDefaults[ ATTR_PNT_LINE_STYLE	- ATTR_START ] = new SfxVoidItem( ATTR_PNT_LINE_STYLE );
	ppPoolDefaults[ ATTR_PNT_COLOR_FORE	- ATTR_START ] = new SvxColorItem( Color( COL_BLACK ), ATTR_PNT_COLOR_FORE );
	ppPoolDefaults[ ATTR_PNT_COLOR_BACK	- ATTR_START ] = new SvxColorItem( Color( 0, 184 , 255 ), ATTR_PNT_COLOR_BACK );
	ppPoolDefaults[ ATTR_PNT_AIR_INTENS	- ATTR_START ] = new SfxUInt16Item( ATTR_PNT_AIR_INTENS, (UINT16) AIR_MEDIUM );
	ppPoolDefaults[ ATTR_PNT_AIR_WIDTH	- ATTR_START ] = new SfxUInt16Item( ATTR_PNT_AIR_WIDTH, 16 );
	ppPoolDefaults[ ATTR_PNT_PEN_WIDTH	- ATTR_START ] = new SfxUInt16Item( ATTR_PNT_PEN_WIDTH, 1 );
	ppPoolDefaults[ ATTR_PNT_LINE_TRANSPARENCY - ATTR_START ] = new SfxUInt16Item( ATTR_PNT_LINE_TRANSPARENCY, 0 );
	ppPoolDefaults[ ATTR_PNT_FILL_TRANSPARENCY - ATTR_START ] = new SfxUInt16Item( ATTR_PNT_FILL_TRANSPARENCY, 0 );

	for( i = ATTR_PNT_END + 1; i <= ATTR_END; i++ )
		ppPoolDefaults[ i - ATTR_START ] = new SfxVoidItem( i );
}

// -----------------------------------------------------------------------------

void SimModule::DestroyPoolDefaults()
{
	// wurden diese Defaults berhaupt schon einmal benutzt
	if( ppPoolDefaults[ 0 ]->GetRef() == SFX_ITEMS_STATICDEFAULT )
		SfxItemPool::ReleaseDefaults( ppPoolDefaults, ATTR_END - ATTR_START + 1, TRUE );
	else
	{
		// ansonsten selber loeschen
		for( USHORT i = 0, nCount = ATTR_END - ATTR_START + 1; i < nCount; i++ )
			delete ppPoolDefaults[ i ], ppPoolDefaults[ i ] = NULL;

		delete ppPoolDefaults, ppPoolDefaults = NULL;
	}
}

// ---------------
// - SimDefaults -
// ---------------

SimDefaults::SimDefaults() :
	SfxConfigItem( SIMCFG_APP )
{
}

// -----------------------------------------------------------------------------

void SimDefaults::Init()
{
	fRotation = 45.F;
	bRotOrient = FALSE;
	fScaleX = 75.F;
	fScaleY = 75.F;
	fRedAdjust = 10.F;
	fGreenAdjust = 10.F;
	fBlueAdjust = 10.F;
	fBrightness = 10.F;
	fContrast = 10.F;
	fBWThreshold = 50.F;
	fAge = 10.F;
	fSolarThreshold = 10.F;
	bSolarInvert = FALSE;
	nGrayCount = 256;
	nMosaicX = 4;
	nMosaicY = 4;
	eMosaicSharpness = (DefinitionType) 1;
	nPosterCount = 16;
	nTileX = 4;
	nTileY = 4;
	eSharpness = (DefinitionType) 1;
}

// -----------------------------------------------------------------------------

int SimDefaults::Load( SvStream& rIStm )
{
	INT32	nVal32;
	UINT16	nVersion;
	UINT16	nVal16;

	rIStm >> nVersion;

	rIStm >> nVal32; fRotation = (float) nVal32 * 0.01F;
	rIStm >> bRotOrient;
	rIStm >> nVal32; fScaleX = (float) nVal32;
	rIStm >> nVal32; fScaleY = (float) nVal32;
	rIStm >> nVal32; fRedAdjust = (float) nVal32;
	rIStm >> nVal32; fGreenAdjust = (float) nVal32;
	rIStm >> nVal32; fBlueAdjust = (float) nVal32;
	rIStm >> nVal32; fBrightness = (float) nVal32;
	rIStm >> nVal32; fContrast = (float) nVal32;
	rIStm >> nVal32; fBWThreshold = (float) nVal32;
	rIStm >> nVal32; fAge = (float) nVal32;
	rIStm >> nVal32; fSolarThreshold = (float) nVal32;
	rIStm >> bSolarInvert;
	rIStm >> nVal16; nGrayCount = nVal16;
	rIStm >> nVal16; nMosaicX = nVal16;
	rIStm >> nVal16; nMosaicY = nVal16;
	rIStm >> nVal16; eMosaicSharpness = (DefinitionType) nVal16;
	rIStm >> nVal16; nPosterCount = nVal16;
	rIStm >> nVal16; nTileX = nVal16;
	rIStm >> nVal16; nTileY = nVal16;
	rIStm >> nVal16; eSharpness = (DefinitionType) nVal16;;
	rIStm .SeekRel( 4 );

	return SfxConfigItem::ERR_OK;;
}

// -----------------------------------------------------------------------------

BOOL SimDefaults::Store( SvStream& rOStm )
{
	rOStm << (UINT16) 0x0002; // Version

	rOStm << (INT32) FRound( fRotation * 100.F );
	rOStm << bRotOrient;
	rOStm << (INT32) FRound( fScaleX );
	rOStm << (INT32) FRound( fScaleY );
	rOStm << (INT32) FRound( fRedAdjust );
	rOStm << (INT32) FRound( fGreenAdjust );
	rOStm << (INT32) FRound( fBlueAdjust );
	rOStm << (INT32) FRound( fBrightness );
	rOStm << (INT32) FRound( fContrast );
	rOStm << (INT32) FRound( fBWThreshold );
	rOStm << (INT32) FRound( fAge );
	rOStm << (INT32) FRound( fSolarThreshold );
	rOStm << bSolarInvert;
	rOStm << (UINT16) nGrayCount;
	rOStm << (UINT16) nMosaicX;
	rOStm << (UINT16) nMosaicY;
	rOStm << (UINT16) eMosaicSharpness;
	rOStm << (UINT16) nPosterCount;
	rOStm << (UINT16) nTileX;
	rOStm << (UINT16) nTileY;
	rOStm << (UINT16) eSharpness;

	// Dummy Values
	rOStm << (UINT16) 4; // RM_MM
	rOStm << (INT32) 75; // 75DPI

	return SfxConfigItem::ERR_OK;;
}

// -----------------------------------------------------------------------------

void SimDefaults::UseDefault()
{
	Init();
	SfxConfigItem::UseDefault();
}
