/*************************************************************************
 *
 *  $RCSfile: airctrl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string> // HACK: prevent conflict between STLPORT and Workshop headers

#include <vcl/toolbox.hxx>
#include <svtools/intitem.hxx>
#include <sfx2/app.hxx>
#include <sfx2/dispatch.hxx>
#include <svx/dialmgr.hxx>
#include "strings.hrc"
#include "simattr.hxx"
#include "glob.hxx"
#include "simapp.hrc"
#include "airctrl.hxx"
#include "simresid.hxx"

// -----------------
// - SimAirControl -
// -----------------

SFX_IMPL_TOOLBOX_CONTROL( SimAirControl, SfxUInt16Item );

// -----------------------------------------------------------------------------

SimAirControl::SimAirControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBind ) :
			SfxToolBoxControl	( nId, rTbx, rBind )
{
}

// -----------------------------------------------------------------------------

SimAirControl::~SimAirControl()
{
}

// -----------------------------------------------------------------------------

void SimAirControl::StateChanged( USHORT nSID, SfxItemState eState, const SfxPoolItem* pState )
{
	AirListBox* pWin = (AirListBox*) GetToolBox().GetItemWindow( GetId() );

	DBG_ASSERT( pWin, "Window not found" );

	if ( eState == SFX_ITEM_DISABLED )
		pWin->Disable();
	else
	{
		pWin->Enable();

		if ( eState == SFX_ITEM_AVAILABLE )
			pWin->Update( (const SfxUInt16Item*) pState );
		else
			pWin->Update( NULL );
	}
}

// -----------------------------------------------------------------------------

Window* SimAirControl::CreateItemWindow( Window *pParent )
{
	return new AirListBox( pParent );
}

// --------------
// - AirListBox -
// --------------

AirListBox::AirListBox( Window* pParent ) :
			ListBox	( pParent, WB_BORDER | WB_DROPDOWN )
{
	SetSizePixel( Size( 110, 110 ) );
	InsertEntry( String( SimResId( STR_AIR_LOW ) ), 0 );
	InsertEntry( String( SimResId( STR_AIR_MEDIUM ) ), 1 );
	InsertEntry( String( SimResId( STR_AIR_HIGH ) ), 2 );

	SFX_BINDINGS().Invalidate( SID_AIR_INTENS );

	Show();
}

// -----------------------------------------------------------------------------

AirListBox::~AirListBox()
{
}

// -----------------------------------------------------------------------------

void AirListBox::Update( const SfxUInt16Item* pItem )
{
	if ( pItem )
	{
		AirIntensity	eAirIntens = (AirIntensity) pItem->GetValue();
		USHORT			nPos;

		switch ( eAirIntens )
		{
			case ( AIR_LOW ) :
				nPos = 0;
			break;

			case ( AIR_MEDIUM ) :
				nPos = 1;
			break;

			case ( AIR_HIGH ) :
				nPos = 2;
			break;

			default :
				nPos = 1;
			break;
		}

		SelectEntryPos( nPos );
	}
	else
		SetText( String() );
}

// -----------------------------------------------------------------------------

void AirListBox::Select()
{
	AirIntensity	eAirIntens;

	switch ( GetSelectEntryPos() )
	{
		case 0 :
			eAirIntens = AIR_LOW;
		break;

		case 1 :
			eAirIntens = AIR_MEDIUM;
		break;

		case 2 :
			eAirIntens = AIR_HIGH;
		break;

		default:
			eAirIntens = AIR_MEDIUM;
		break;
	}

	SfxUInt16Item aItem( ATTR_PNT_AIR_INTENS, (UINT16) eAirIntens );
	SFX_DISPATCHER().Execute( SID_AIR_INTENS, SFX_CALLMODE_ASYNCHRON, &aItem, 0L );
}
