/*************************************************************************
 *
 *  $RCSfile: linectrl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string> // HACK: prevent conflict between STLPORT and Workshop headers

#include <vcl/toolbox.hxx>
#include <svtools/intitem.hxx>
#include <sfx2/app.hxx>
#include <sfx2/dispatch.hxx>
#include <sfx2/viewfrm.hxx>
#include <svx/dialmgr.hxx>
#include "simapp.hrc"
#include "simattr.hxx"
#include "linectrl.hxx"

// -----------------------
// - SimLineWidthControl -
// -----------------------

SFX_IMPL_TOOLBOX_CONTROL( SimLineWidthControl, SfxUInt16Item );
SFX_IMPL_TOOLBOX_CONTROL( SimLineRoundControl, SfxUInt16Item );

// -----------------------------------------------------------------------------

SimLineWidthControl::SimLineWidthControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBind ) :
			SfxToolBoxControl	( nId, rTbx, rBind )
{
}

// -----------------------------------------------------------------------------

SimLineWidthControl::~SimLineWidthControl()
{
}

// -----------------------------------------------------------------------------

void SimLineWidthControl::StateChanged( USHORT nSID, SfxItemState eState, const SfxPoolItem* pState )
{
	LineMetricField* pWin = (LineMetricField*) GetToolBox().GetItemWindow( GetId() );

	DBG_ASSERT( pWin, "Window not found" );

	if ( eState == SFX_ITEM_DISABLED )
	{
		pWin->Disable();
		pWin->SetText( String() );
	}
	else
	{
		pWin->Enable();

		if ( eState == SFX_ITEM_AVAILABLE )
			pWin->Update( (const SfxUInt16Item*) pState );
		else
			pWin->Update( NULL );
	}
}

// -----------------------------------------------------------------------------

Window* SimLineWidthControl::CreateItemWindow( Window *pParent )
{
	return new LineMetricField( pParent, SID_LINE_WIDTH, ATTR_PNT_LINE_WIDTH );
}

// -----------------------
// - SimLineRoundControl -
// -----------------------

SimLineRoundControl::SimLineRoundControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBind ) :
			SfxToolBoxControl	( nId, rTbx, rBind )
{
}

// -----------------------------------------------------------------------------

SimLineRoundControl::~SimLineRoundControl()
{
}

// -----------------------------------------------------------------------------

void SimLineRoundControl::StateChanged( USHORT nSID, SfxItemState eState, const SfxPoolItem* pState )
{
	LineMetricField* pWin = (LineMetricField*) GetToolBox().GetItemWindow( GetId() );

	DBG_ASSERT( pWin, "Window not found" );

	if ( eState == SFX_ITEM_DISABLED )
	{
		pWin->Disable();
		pWin->SetText( String() );
	}
	else
	{
		pWin->Enable();

		if ( eState == SFX_ITEM_AVAILABLE )
			pWin->Update( (const SfxUInt16Item*) pState );
		else
			pWin->Update( NULL );
	}
}

// -----------------------------------------------------------------------------

Window* SimLineRoundControl::CreateItemWindow( Window *pParent )
{
	return new LineMetricField( pParent, SID_LINE_ROUND, ATTR_PNT_LINE_ROUND );
}

// -------------------
// - LineMetricField -
// -------------------

LineMetricField::LineMetricField( Window* pParent, USHORT nSlotId, USHORT nAttrId ) :
			SvxMetricField	( pParent, SfxViewFrame::Current()->GetBindings() ),
			nSlot			( nSlotId ),
			nId				( nAttrId )
{
	String	aText( String( RTL_CONSTASCII_USTRINGPARAM( "128 Pixel" ) ) );
	Size	aSize( GetTextWidth( aText ), GetTextHeight() );

	aSize.Width() += 20, aSize.Height() += 6;

	SetUnit( FUNIT_CUSTOM );
	SetCustomUnitText( String( RTL_CONSTASCII_USTRINGPARAM( " Pixel" ) ) ); 
	SetSizePixel( aSize );
	SetDecimalDigits( 0 );
	SetMax( 128 );
	SetLast( 128 );
	SetSpinSize( 1 );

	if (  SID_LINE_ROUND == nSlot )
	{
		SetMin( 0 );
		SetFirst( 0 );
	}
	else
	{
		SetMin( 1 );
		SetFirst( 1 );
	}

	Show();
}

// -----------------------------------------------------------------------------

LineMetricField::~LineMetricField()
{
}

// -----------------------------------------------------------------------------

void LineMetricField::Update( const SfxUInt16Item* pItem )
{
	if ( pItem )
		SetValue( pItem->GetValue() );
	else
		SetText( String() );
}

// -----------------------------------------------------------------------------

void LineMetricField::Modify()
{
	SfxUInt16Item aItem( nId, (UINT16) GetValue() );

	SFX_DISPATCHER().Execute( nSlot, SFX_CALLMODE_ASYNCHRON, &aItem, 0L );
}
