/*************************************************************************
 *
 *  $RCSfile: tbxctrl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/eitem.hxx>
#include "simapp.hrc"
#include "tbxctrl.hxx"

// ---------------------
// - SimPopupWindowTbx -
// ---------------------

SFX_IMPL_TOOLBOX_CONTROL( SimTbxControl, SfxEnumItem )

// -----------------------------------------------------------------------------

SimPopupWindowTbx::SimPopupWindowTbx( USHORT nId, WindowAlign eAlign,
									  SimResId aRIdWin, SimResId aRIdTbx,
									  SfxBindings& rBindings ) :
				SfxPopupWindow	( nId, aRIdWin, rBindings ),
				aTbx			( this, GetBindings(), aRIdTbx ),
				aResIdWin		( aRIdWin ),
				aResIdTbx		( aRIdTbx ),
				eTbxAlign		( eAlign )
{
	aTbx.Initialize();
	aSelectLink = aTbx.GetToolBox().GetSelectHdl();
	aTbx.GetToolBox().SetSelectHdl( LINK( this, SimPopupWindowTbx, TbxSelectHdl ) );

	FreeResource();

	if( ( eTbxAlign == WINDOWALIGN_TOP ) || ( eTbxAlign == WINDOWALIGN_BOTTOM ) )
	{
	   aTbx.GetToolBox().SetAlign( WINDOWALIGN_LEFT );
	   SetText( String() );
	}

	const Size aSize( aTbx.CalcWindowSizePixel() );

	aTbx.SetPosSizePixel( Point(), aSize );
	SetOutputSizePixel( aSize );
}

// -----------------------------------------------------------------------------

SimPopupWindowTbx::~SimPopupWindowTbx()
{
}

// -----------------------------------------------------------------------------

void SimPopupWindowTbx::StateChanged(USHORT nSID,
									 SfxItemState eState,
									 SfxPoolItem* pPoolItem)
{
	SfxPopupWindow::StateChanged( nSID, eState, pPoolItem );
}

// -----------------------------------------------------------------------------

void SimPopupWindowTbx::PopupModeEnd()
{
	aTbx.GetToolBox().EndSelection();
	SfxPopupWindow::PopupModeEnd();
}

// -----------------------------------------------------------------------------

SfxPopupWindow* SimPopupWindowTbx::Clone() const
{
	return new SimPopupWindowTbx( GetId(), eTbxAlign, aResIdWin, aResIdTbx,
								  (SfxBindings&) GetBindings() );
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimPopupWindowTbx, TbxSelectHdl, void*, p )
{
	aSelectLink.Call( &aTbx.GetToolBox() );

	if( IsInPopupMode() )
		EndPopupMode();

	return 0L;
}

// -----------------------------------------------------------------------------

SimTbxControl::SimTbxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBind ) :
		SfxToolBoxControl( nId, rTbx, rBind )
{
}

// -----------------------------------------------------------------------------

SfxPopupWindowType SimTbxControl::GetPopupWindowType() const
{
	return( SFX_POPUPWINDOW_ONCLICK );
}

// -----------------------------------------------------------------------------

SfxPopupWindow*	SimTbxControl::CreatePopupWindow()
{
	WindowAlign eAlign = this->GetToolBox().GetAlign();

	switch( GetId() )
	{
		case SID_TOOLBOX_ZOOM:
		{
			SimPopupWindowTbx *pWin =
				new SimPopupWindowTbx( GetId(), eAlign,
									   SimResId( RID_ZOOM_TBXCTRL ),
									   SimResId( RID_ZOOM_TOOLBOX ),
									   GetBindings() );
			pWin->StartPopupMode( &GetToolBox(), TRUE );
			pWin->StartSelection();
			pWin->Show();
			return( pWin );
		}

		case SID_TOOLBOX_PICTURE:
		{
			SimPopupWindowTbx *pWin =
				new SimPopupWindowTbx( GetId(), eAlign,
									   SimResId( RID_PICTURE_TBXCTRL ),
									   SimResId( RID_PICTURE_TOOLBOX ),
									   GetBindings() );
			pWin->StartPopupMode( &GetToolBox(), TRUE );
			pWin->StartSelection();
			pWin->Show();
			return( pWin );
		}

		case SID_TOOLBOX_COLOR:
		{
			SimPopupWindowTbx *pWin =
				new SimPopupWindowTbx( GetId(), eAlign,
									   SimResId( RID_COLOR_TBXCTRL ),
									   SimResId( RID_COLOR_TOOLBOX ),
									   GetBindings() );
			pWin->StartPopupMode( &GetToolBox(), TRUE );
			pWin->StartSelection();
			pWin->Show();
			return( pWin );
		}

		case SID_TOOLBOX_EFFECTS:
		{
			SimPopupWindowTbx *pWin =
				new SimPopupWindowTbx( GetId(), eAlign,
									   SimResId( RID_EFFECT_TBXCTRL ),
									   SimResId( RID_EFFECT_TOOLBOX ),
									   GetBindings() );
			pWin->StartPopupMode( &GetToolBox(), TRUE );
			pWin->StartSelection();
			pWin->Show();
			return( pWin );
		}

		case SID_TOOLBOX_PIXEL:
		{
			SimPopupWindowTbx *pWin =
				new SimPopupWindowTbx( GetId(), eAlign,
									   SimResId( RID_PIXEL_TBXCTRL ),
									   SimResId( RID_PIXEL_TOOLBOX ),
									   GetBindings() );
			pWin->StartPopupMode( &GetToolBox(), TRUE );
			pWin->StartSelection();
			pWin->Show();
			return( pWin );
		}
	}

	return( NULL );
}
