/*************************************************************************
 *
 *  $RCSfile: contrast.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/intitem.hxx>
#include "simattr.hxx"
#include "contrast.hxx"
#include "contrast.hrc"
#include "simresid.hxx"

// ------------------
// - SimContrastDlg -
// ------------------

SimContrastDlg::SimContrastDlg( Window* pWindow, const SfxItemSet& rInAttrs ) :
				ModalDialog		( pWindow, SimResId( DLG_CONTRAST ) ),
				aFtContrast 	( this, ResId( FT_CONTRAST ) ),
				aSbContrast 	( this, ResId( SB_CONTRAST ) ),
				aMtrFldContrast ( this, ResId( MTR_FLD_CONTRAST ) ),
				aFtBrightness   ( this, ResId( FT_BRIGHTNESS ) ),
				aSbBrightness   ( this, ResId( SB_BRIGHTNESS ) ),
				aMtrFldBrightness( this, ResId( MTR_FLD_BRIGHTNESS ) ),
				aGrpContrast	( this, ResId( GRP_CONTRAST ) ),
				aBtnOK			( this, ResId( BTN_OK ) ),
				aBtnCancel		( this, ResId( BTN_CANCEL ) ),
				aBtnHelp		( this, ResId( BTN_HELP ) ),
				rOutAttrs		( rInAttrs )
{
	FreeResource();

	aSbContrast.SetScrollHdl( LINK ( this, SimContrastDlg, ScrollContrastHdl ) );
	aSbBrightness.SetScrollHdl( LINK ( this, SimContrastDlg, ScrollBrightnessHdl ) );

	aMtrFldContrast.SetModifyHdl( LINK ( this, SimContrastDlg, MtrFldContrastHdl ) );
	aMtrFldBrightness.SetModifyHdl( LINK ( this, SimContrastDlg, MtrFldBrightnessHdl ) );

	aSbContrast.SetRange( Range( 0, 200 ) );
	aSbBrightness.SetRange( Range( 0, 200 ) );

	Reset();
}

// -----------------------------------------------------------------------------

SimContrastDlg::~SimContrastDlg()
{
}

// -----------------------------------------------------------------------------

void SimContrastDlg::Reset()
{
	aMtrFldContrast.SetValue( ( ( const SfxInt16Item& ) rOutAttrs.
						Get( ATTR_BRI_CON_CONTRAST ) ).GetValue() );
	aMtrFldBrightness.SetValue( ( ( const SfxInt16Item& ) rOutAttrs.
						Get( ATTR_BRI_CON_BRIGHT ) ).GetValue() );

	MtrFldContrastHdl( this );
	MtrFldBrightnessHdl( this );
}

// -----------------------------------------------------------------------------

void SimContrastDlg::GetAttr( SfxItemSet& rOutAttrs )
{
	rOutAttrs.Put( SfxInt16Item( ATTR_BRI_CON_CONTRAST, (INT16) aMtrFldContrast.GetValue() ) );
	rOutAttrs.Put( SfxInt16Item( ATTR_BRI_CON_BRIGHT, (INT16) aMtrFldBrightness.GetValue() ) );
}

// -----------------------------------------------------------------------------

IMPL_LINK_INLINE_START( SimContrastDlg, ScrollContrastHdl, void *, EMPTYARG )
{
	aMtrFldContrast.SetValue( aSbContrast.GetThumbPos() - 100 );

	return( 0L );
}
IMPL_LINK_INLINE_END( SimContrastDlg, ScrollContrastHdl, void *, EMPTYARG )

// -----------------------------------------------------------------------------

IMPL_LINK_INLINE_START( SimContrastDlg, ScrollBrightnessHdl, void *, EMPTYARG )
{
	aMtrFldBrightness.SetValue( aSbBrightness.GetThumbPos() - 100 );

	return( 0L );
}
IMPL_LINK_INLINE_END( SimContrastDlg, ScrollBrightnessHdl, void *, EMPTYARG )

// -----------------------------------------------------------------------------

IMPL_LINK( SimContrastDlg, MtrFldContrastHdl, void *, EMPTYARG )
{
	aSbContrast.SetThumbPos( aMtrFldContrast.GetValue() + 100 );

	return( 0L );
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimContrastDlg, MtrFldBrightnessHdl, void *, EMPTYARG )
{
	aSbBrightness.SetThumbPos( aMtrFldBrightness.GetValue() + 100 );

	return( 0L );
}
