/*************************************************************************
 *
 *  $RCSfile: defaults.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <vcl/salbtype.hxx>
#include "simmod.hxx"
#include "simattr.hxx"
#include "simresid.hxx"
#include "defaults.hxx"
#include "defaults.hrc"

// ------------------------
// - SimStandards1TabPage -
// ------------------------

SimStandards1TabPage::SimStandards1TabPage( Window* pParent, const SfxItemSet& rInAttrs  ) :
		SfxTabPage			( pParent, SimResId( TP_DEFAULT_COLORS ), rInAttrs ),
		aFtRed 				( this, ResId( FT_RED ) ),
		aMtrFldRed  		( this, ResId( MTR_FLD_RED ) ),
		aFtGreen			( this, ResId( FT_GREEN ) ),
		aMtrFldGreen		( this, ResId( MTR_FLD_GREEN ) ),
		aFtBlue				( this, ResId( FT_BLUE ) ),
		aMtrFldBlue 		( this, ResId( MTR_FLD_BLUE ) ),
		aGrpRgb     		( this, ResId( GRP_RGB ) ),
		aFtBrightness		( this, ResId( FT_BRIGHTNESS ) ),
		aMtrFldBrightness	( this, ResId( MTR_FLD_BRIGHTNESS ) ),
		aFtContrast 		( this, ResId( FT_CONTRAST ) ),
		aMtrFldContrast		( this, ResId( MTR_FLD_CONTRAST ) ),
		aGrpBrightness		( this, ResId( GRP_BRIGHTNESS ) ),
		aFtAngle			( this, ResId( FT_ANGLE ) ),
		aMtrFldAngle		( this, ResId( MTR_FLD_ANGLE ) ),
		aRbtLeft			( this, ResId( RBT_LEFT ) ),	
		aRbtRight			( this, ResId( RBT_RIGHT ) ),	
		aGrpRotation		( this, ResId( GRP_ROTATION ) ),
		aFtNumber   		( this, ResId( FT_NUMBER ) ),
		aLbGrayscales		( this, ResId( LB_GRAYSCALES ) ),
		aGrpGrayscales		( this, ResId( GRP_GRAYSCALES ) ),
		aFtThreshold		( this, ResId( FT_THRESHOLD ) ),
		aMtrFldThreshold	( this, ResId( MTR_FLD_THRESHOLD ) ),
		aGrpBlackWhite		( this, ResId( GRP_BLACK_WHITE ) ),
		aFtPoster	 		( this, ResId( FT_POSTERCOUNT ) ),
		aNumFldPoster		( this, ResId( NUM_FLD_POSTER ) ),
		aGrpPoster  		( this, ResId( GRP_POSTER ) )
{
	FreeResource();
}

// -----------------------------------------------------------------------------

void SimStandards1TabPage::Reset( const SfxItemSet& rOutAttrs )
{
	aMtrFldAngle.SetValue( FRound( pDefaults->fRotation * 100.F ) );
	aMtrFldRed.SetValue( FRound( pDefaults->fRedAdjust ) );
	aMtrFldGreen.SetValue( FRound( pDefaults->fGreenAdjust ) );
	aMtrFldBlue.SetValue( FRound( pDefaults->fBlueAdjust ) );
	aMtrFldBrightness.SetValue( FRound( pDefaults->fBrightness ) );
	aMtrFldContrast.SetValue( FRound( pDefaults->fContrast ) );
	aMtrFldThreshold.SetValue( FRound( pDefaults->fBWThreshold ) );
	aLbGrayscales.SelectEntry( String( pDefaults->nGrayCount ) );
	aNumFldPoster.SetValue( (long) pDefaults->nPosterCount );
	pDefaults->bRotOrient ? aRbtRight.Check() : aRbtLeft.Check();
}

// -----------------------------------------------------------------------------

BOOL SimStandards1TabPage::FillItemSet( SfxItemSet& rOutAttrs )
{
	pDefaults->fRotation = (float) aMtrFldAngle.GetValue() * 0.01F;
	pDefaults->fRedAdjust = (float) aMtrFldRed.GetValue();
	pDefaults->fGreenAdjust = (float) aMtrFldGreen.GetValue();
	pDefaults->fBlueAdjust = (float) aMtrFldBlue.GetValue();
	pDefaults->fBrightness = (float) aMtrFldBrightness.GetValue();
	pDefaults->fContrast = (float) aMtrFldContrast.GetValue();
	pDefaults->fBWThreshold = (float) aMtrFldThreshold.GetValue();
	pDefaults->nPosterCount = (USHORT) aNumFldPoster.GetValue();
	pDefaults->nGrayCount = aLbGrayscales.GetSelectEntry().ToInt32();
	pDefaults->bRotOrient = aRbtRight.IsChecked();

	return TRUE;
}

// -----------------------------------------------------------------------------

SfxTabPage* SimStandards1TabPage::Create( Window* pWindow, const SfxItemSet& rOutAttrs )
{
	return new SimStandards1TabPage( pWindow, rOutAttrs );
}

// -----------------------------------------------------------------------------

SimStandards2TabPage::SimStandards2TabPage( Window* pParent, const SfxItemSet& rInAttrs  ) :
		SfxTabPage			( pParent, SimResId( TP_DEFAULT_EFFECTS ), rInAttrs ),
		aFtMosaicX  	 	( this, ResId( FT_MOSAIC_X ) ),
		aMtrFldMosaicX		( this, ResId( MTR_FLD_MOSAIC_X ) ),
		aFtMosaicY  	 	( this, ResId( FT_MOSAIC_Y ) ),
		aMtrFldMosaicY		( this, ResId( MTR_FLD_MOSAIC_Y ) ),
		aFtMosaicDefinition	( this, ResId( FT_MOSAIC_DEFINITION ) ),
		aLbMosaicDefinition	( this, ResId( LB_MOSAIC_DEFINITION ) ),
		aGrpMosaic  		( this, ResId( GRP_MOSAIC ) ),
		aFtThreshold		( this, ResId( FT_THRESHOLD ) ),
		aMtrFldThreshold	( this, ResId( MTR_FLD_THRESHOLD ) ),
		aCbxInvert  		( this, ResId( CBX_INVERT ) ),
		aGrpSolarization	( this, ResId( GRP_SOLARIZATION ) ),
		aFtTileX		  	( this, ResId( FT_TILE_X ) ),
		aMtrFldTileX		( this, ResId( MTR_FLD_TILE_X ) ),
		aFtTileY	  		( this, ResId( FT_TILE_Y ) ),
		aMtrFldTileY		( this, ResId( MTR_FLD_TILE_Y ) ),
		aGrpTile    		( this, ResId( GRP_TILE ) ),
		aFtDefinition		( this, ResId( FT_DEFINITION ) ),
		aLbDefinition   	( this, ResId( LB_DEFINITION ) ),
		aGrpSharpness   	( this, ResId( GRP_SHARPNESS ) ),
		aFtAge				( this, ResId( FT_AGE ) ),
		aMtrFldAge  		( this, ResId( MTR_FLD_AGE ) ),
		aGrpAge     		( this, ResId( GRP_AGE ) )
{
	FreeResource();
}

// -----------------------------------------------------------------------------

void SimStandards2TabPage::Reset( const SfxItemSet& rOutAttrs )
{
	aMtrFldMosaicX.SetValue( pDefaults->nMosaicX );
	aMtrFldMosaicY.SetValue( pDefaults->nMosaicY );
	aLbMosaicDefinition.SelectEntryPos( (UINT16) pDefaults->eMosaicSharpness );
	aMtrFldAge.SetValue( FRound( pDefaults->fAge ) );
	aMtrFldThreshold.SetValue( FRound( pDefaults->fSolarThreshold ) );
	aCbxInvert.Check( pDefaults->bSolarInvert );
	aMtrFldTileX.SetValue( pDefaults->nTileX );
	aMtrFldTileY.SetValue( pDefaults->nTileY );
	aLbDefinition.SelectEntryPos( (UINT16) pDefaults->eSharpness );
}

// -----------------------------------------------------------------------------

BOOL SimStandards2TabPage::FillItemSet( SfxItemSet& rOutAttrs )
{
	pDefaults->nMosaicX = (USHORT) aMtrFldMosaicX.GetValue();
	pDefaults->nMosaicY = (USHORT) aMtrFldMosaicY.GetValue();
	pDefaults->eMosaicSharpness = (DefinitionType) aLbMosaicDefinition.GetSelectEntryPos();
	pDefaults->fAge = (float) aMtrFldAge.GetValue();
	pDefaults->fSolarThreshold = (float) aMtrFldThreshold.GetValue();
	pDefaults->bSolarInvert = aCbxInvert.IsChecked();
  	pDefaults->nTileX = (USHORT) aMtrFldTileX.GetValue();
  	pDefaults->nTileY = (USHORT) aMtrFldTileY.GetValue();
 	pDefaults->eSharpness = (DefinitionType) aLbDefinition.GetSelectEntryPos();

	return TRUE;
}

// -----------------------------------------------------------------------------

SfxTabPage* SimStandards2TabPage::Create( Window* pWindow, const SfxItemSet& rOutAttrs )
{
	return new SimStandards2TabPage( pWindow, rOutAttrs );
}
