/*************************************************************************
 *
 *  $RCSfile: frmprop.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "simframe.hxx"
#include "simresid.hxx"
#include "frmprop.hxx"
#include "frmprop.hrc"

// -------------------------
// - SimFramePropertiesDlg -
// -------------------------

SimFramePropertiesDlg::SimFramePropertiesDlg( Window* pWindow, const SimFrame& rFrame ) :
				ModalDialog		( pWindow, SimResId( DLG_FRAMEPROPERTIES ) ),
				aGpProperties	( this, ResId( GP_PROPERTIES ) ),
				aFtPosX			( this, ResId( FT_POSX ) ),
				aMfPosX			( this, ResId( MF_POSX ) ),
				aFtPosY			( this, ResId( FT_POSY ) ),
				aMfPosY			( this, ResId( MF_POSY ) ),
				aFtWidth		( this, ResId( FT_WIDTH ) ),
				aMfWidth		( this, ResId( MF_WIDTH ) ),
				aFtHeight		( this, ResId( FT_HEIGHT ) ),
				aMfHeight		( this, ResId( MF_HEIGHT ) ),
				aFtTimeout		( this, ResId( FT_TIMEOUT ) ),
				aTfTimeout		( this, ResId( TF_TIMEOUT ) ),
				aFtReposal		( this, ResId( FT_REPOSAL ) ),
				aLbReposal		( this, ResId( LB_REPOSAL ) ),
				aBtOk			( this, ResId( BT_OK ) ),
				aBtCancel		( this, ResId( BT_CANCEL ) ),
				aBtHelp			( this, ResId( BT_HELP ) )
{
	const Point	aFramePos( rFrame.GetDisplayPosPixel() );
	const Size	aFrameSize( rFrame.GetDisplaySizePixel() );
	const ULONG	nTimeout = rFrame.GetWaitMS();
	USHORT		nSelPos;

	FreeResource();
	aTfTimeout.SetFormat( TIMEF_100TH_SEC );

	switch( rFrame.GetDisposal() )
	{
		case( DISPOSE_NOT ) : nSelPos = 0; break;
		case( DISPOSE_BACK ) : nSelPos = 1; break;
		case( DISPOSE_PREVIOUS ) : nSelPos = 2; break;
		default: nSelPos = 1; break;
	}

	aMfPosX.SetValue( aFramePos.X() );
	aMfPosY.SetValue( aFramePos.Y() );
	aMfWidth.SetValue( aFrameSize.Width() );
	aMfHeight.SetValue( aFrameSize.Height() );

	if( ANIMATION_NO_TIMEOUT == nTimeout )
	{
		aTfTimeout.SetTime( Time( 0, 0, 0, 0 ) );
		aFtTimeout.Disable();
		aTfTimeout.Disable();
		aFtReposal.Disable();
		aLbReposal.Disable();
	}
	else
		aTfTimeout.SetTime( Time( 0, 0, 0, nTimeout / 10 ) );
	
	aLbReposal.SelectEntryPos( nSelPos );
}

// -----------------------------------------------------------------------------

SimFramePropertiesDlg::~SimFramePropertiesDlg()
{
}

// -----------------------------------------------------------------------------

Point SimFramePropertiesDlg::GetDisplayPosPixel() const
{
	return Point( aMfPosX.GetValue(), aMfPosY.GetValue() );
}

// -----------------------------------------------------------------------------

Size SimFramePropertiesDlg::GetDisplaySizePixel() const
{
	return Size( aMfWidth.GetValue(), aMfHeight.GetValue() );
}

// -----------------------------------------------------------------------------

ULONG SimFramePropertiesDlg::GetWaitMS() const
{
	if( aTfTimeout.IsEnabled() )
		return aTfTimeout.GetTime().GetMSFromTime();
	else
		return ANIMATION_NO_TIMEOUT;
}

// -----------------------------------------------------------------------------

Disposal SimFramePropertiesDlg::GetDisposal() const
{
	ULONG		nPos = aLbReposal.GetSelectEntryPos();
	Disposal	eDisposal;

	switch( nPos )
	{
		case( 0 ): eDisposal = DISPOSE_NOT; break;
		case( 1 ): eDisposal = DISPOSE_BACK; break;
		case( 2 ): eDisposal = DISPOSE_PREVIOUS; break;
		default: eDisposal = DISPOSE_BACK; break;
	}

	return eDisposal;
}
