/*************************************************************************
 *
 *  $RCSfile: graydlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/intitem.hxx>
#include <svtools/aeitem.hxx>
#include "glob.hxx"
#include "simattr.hxx"
#include "graydlg.hxx"
#include "graydlg.hrc"
#include "simresid.hxx"

// --------------
// - SimGrayDlg -
// --------------

SimGrayDlg::SimGrayDlg( Window* pWindow, const SfxItemSet& rInAttrs ) :
				ModalDialog		( pWindow, SimResId( DLG_GRAYSCALES ) ),
				aRbt1       	( this, ResId( RBT_1 ) ),
				aRbt2       	( this, ResId( RBT_2 ) ),
				aRbt3       	( this, ResId( RBT_3 ) ),
				aRbt4       	( this, ResId( RBT_4 ) ),
				aRbt5       	( this, ResId( RBT_5 ) ),
				aRbt6       	( this, ResId( RBT_6 ) ),
				aRbt7       	( this, ResId( RBT_7 ) ),
				aRbt8       	( this, ResId( RBT_8 ) ),
				aGrpColorDepth  ( this, ResId( GRP_COLORDEPTH ) ),
				aMtrFldThreshold( this, ResId( MTR_FLD_THRESHOLD ) ),
				aGrpThreshold   ( this, ResId( GRP_THRESHOLD ) ),
				aBtnOK      	( this, ResId( BTN_OK ) ),
				aBtnCancel  	( this, ResId( BTN_CANCEL ) ),
				aBtnHelp    	( this, ResId( BTN_HELP ) ),
				rOutAttrs		( rInAttrs )
{
	FreeResource();

	aRbt1.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
	aRbt2.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
	aRbt3.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
	aRbt4.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
 	aRbt5.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
	aRbt6.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
	aRbt7.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );
	aRbt8.SetClickHdl( LINK ( this, SimGrayDlg, CheckButtonHdl ) );

	Reset();
}

// -----------------------------------------------------------------------------

SimGrayDlg::~SimGrayDlg()
{
}

// -----------------------------------------------------------------------------

void SimGrayDlg::Reset()
{
	GRAYCOUNT eGC;

	eGC = (GRAYCOUNT) ( ( const SfxAllEnumItem& ) rOutAttrs.
				Get( ATTR_GRAYSCALES_COUNT ) ).GetValue();

	switch( eGC )
	{
		case GRAYCOUNT_2:   aRbt1.Check(); break;
		case GRAYCOUNT_4:   aRbt2.Check(); break;
		case GRAYCOUNT_8:   aRbt3.Check(); break;
		case GRAYCOUNT_16:  aRbt4.Check(); break;
		case GRAYCOUNT_32:  aRbt5.Check(); break;
		case GRAYCOUNT_64:  aRbt6.Check(); break;
		case GRAYCOUNT_128: aRbt7.Check(); break;
		case GRAYCOUNT_256: aRbt8.Check(); break;
	}

	aMtrFldThreshold.SetValue( ( ( const SfxInt16Item& ) rOutAttrs.
						Get( ATTR_GRAYSCALES_THRES ) ).GetValue() );

	CheckButtonHdl( this );
}

// -----------------------------------------------------------------------------

void SimGrayDlg::GetAttr( SfxItemSet& rOutAttrs )
{
	rOutAttrs.Put( SfxInt16Item( ATTR_GRAYSCALES_THRES, (INT16) aMtrFldThreshold.GetValue() ) );

	GRAYCOUNT eGC;

	if( aRbt1.IsChecked() ) eGC = GRAYCOUNT_2;
	else if( aRbt2.IsChecked() ) eGC = GRAYCOUNT_4;
	else if( aRbt3.IsChecked() ) eGC = GRAYCOUNT_8;
	else if( aRbt4.IsChecked() ) eGC = GRAYCOUNT_16;
	else if( aRbt5.IsChecked() ) eGC = GRAYCOUNT_32;
	else if( aRbt6.IsChecked() ) eGC = GRAYCOUNT_64;
	else if( aRbt7.IsChecked() ) eGC = GRAYCOUNT_128;
	else if( aRbt8.IsChecked() ) eGC = GRAYCOUNT_256;

	rOutAttrs.Put( SfxAllEnumItem( ATTR_GRAYSCALES_COUNT, eGC ) );
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimGrayDlg, CheckButtonHdl, void *, EMPTYARG )
{
	if( aRbt1.IsChecked() )
	{
		aMtrFldThreshold.Enable();
		aGrpThreshold.Enable();
	}
	else
	{
		aMtrFldThreshold.Disable();
		aGrpThreshold.Disable();
	}

	return( 0L );
}
