/*************************************************************************
 *
 *  $RCSfile: printopt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <math.h>
#include <vcl/wrkwin.hxx>
#include <svtools/intitem.hxx>
#include <svtools/aeitem.hxx>
#include <svx/dlgutil.hxx>
#include <sfx2/module.hxx>
#include <sfx2/saveopt.hxx>
#include <sfx2/app.hxx>
#include "strings.hrc"
#include "simattr.hxx"
#include "printopt.hxx"
#include "printopt.hrc"
#include "simresid.hxx"
#include "simframe.hxx"

// ------------------
// - SimPrintOptDlg -
// ------------------

SimPrintOptDlg::SimPrintOptDlg( Window* pParent, const SfxItemSet& rInAttrs, SimFrame* pFrame, const Size& rPageSize ) :
		SfxSingleTabDialog	( pParent, rInAttrs, TAB_PRINTOPT, FALSE )
{
	SimPrintOptPage* pPage = new SimPrintOptPage( this, rInAttrs, pFrame, rPageSize );

	SetTabPage( pPage );
	SetText( pPage->GetText() );
}

// -----------------------------------------------------------------------------

SimPrintOptDlg::~SimPrintOptDlg()
{
}

// -------------------
// - SimPrintOptPage -
// -------------------

SimPrintOptPage::SimPrintOptPage( Window* pWindow, const SfxItemSet& rInAttrs, SimFrame* _pFrame, const Size& rPageSize ) :
				SvxTabPage		( pWindow, SimResId( TAB_PRINTOPT ), rInAttrs ),
				aFtMeasure  	( this, ResId( FT_MEASURE ) ),
				aLbMeasure  	( this, ResId( LB_MEASURE ) ),
				aFtWidth		( this, ResId( FT_WIDTH ) ),
				aMtrFldWidth	( this, ResId( MTR_FLD_WIDTH ) ),
				aFtHeight		( this, ResId( FT_HEIGHT ) ),
				aMtrFldHeight	( this, ResId( MTR_FLD_HEIGHT ) ),
				aCbxScale		( this, ResId( CBX_SCALE ) ),
				aGrpSize		( this, ResId( GRP_SIZE ) ),
				aCtlPosition	( this, ResId( CTL_POSITION ) ),
				aGrpPosition	( this, ResId( GRP_POSITION ) ),
				rOutAttrs		( rInAttrs ),
				pFrame			( _pFrame ),
				aPageSize		( rPageSize )
{
	FieldUnit	eFUnit = FUNIT_CM;
	long		nMin, nMax;

	FreeResource();

	SetFieldUnit( aMtrFldWidth, eFUnit, TRUE );
	nMin = 10 * (ULONG) pow( 10, aMtrFldWidth.GetDecimalDigits() );
	nMax = 100000 * (ULONG) pow( 10, aMtrFldWidth.GetDecimalDigits() );
	aMtrFldWidth.SetMin( nMin, FUNIT_100TH_MM );
	aMtrFldWidth.SetMax( nMax, FUNIT_100TH_MM );
	aMtrFldWidth.SetFirst( nMin, FUNIT_100TH_MM );
	aMtrFldWidth.SetLast( nMax, FUNIT_100TH_MM );

	SetFieldUnit( aMtrFldHeight, eFUnit, TRUE );
	nMin = 10 * (ULONG) pow( 10, aMtrFldHeight.GetDecimalDigits() );
	nMax = 100000 * (ULONG) pow( 10, aMtrFldHeight.GetDecimalDigits() );
	aMtrFldHeight.SetMin( nMin, FUNIT_100TH_MM );
	aMtrFldHeight.SetMax( nMax, FUNIT_100TH_MM );
	aMtrFldHeight.SetFirst( nMin, FUNIT_100TH_MM );
	aMtrFldHeight.SetLast( nMax, FUNIT_100TH_MM );

	aMtrFldWidth.SetModifyHdl( LINK( this, SimPrintOptPage, ChangeWidthHdl ) );
	aMtrFldHeight.SetModifyHdl( LINK( this, SimPrintOptPage, ChangeHeightHdl ) );
	aCbxScale.SetClickHdl( LINK( this, SimPrintOptPage, ClickScaleHdl ) );
	aLbMeasure.SetSelectHdl( LINK( this, SimPrintOptPage, ChangeMeasureHdl ) );
}

// -----------------------------------------------------------------------------

SimPrintOptPage::~SimPrintOptPage()
{
}

// -----------------------------------------------------------------------------

void SimPrintOptPage::ImplSetFieldValue( MetricField& rField, ULONG nVal )
{
	rField.SetValue( nVal * (ULONG) pow( 10, rField.GetDecimalDigits() ), FUNIT_100TH_MM );
}

// -----------------------------------------------------------------------------

ULONG SimPrintOptPage::ImplGetFieldValue( const MetricField& rField ) const
{
	return( rField.GetValue( FUNIT_100TH_MM ) / (ULONG) pow( 10, rField.GetDecimalDigits() ) );
}

// -----------------------------------------------------------------------------

void SimPrintOptPage::Reset( const SfxItemSet& rOutAttrs )
{
	const Size aSizePixel( pFrame->GetSizePixel() );

	fXY = (double) aSizePixel.Width() / aSizePixel.Height();
	bWidthChanged = TRUE;

	aCtlPosition.Reset();
	aCtlPosition.SetActualRP( (RECT_POINT)((const SfxUInt16Item&) rOutAttrs.Get(ATTR_PRINTOPT_POSITION)).GetValue() );
	nDPI = ((const SfxUInt16Item&) rOutAttrs.Get( ATTR_PRINTOPT_DPI ) ).GetValue();

	// ListBox auf entspr. Eintrag setzen
	switch( nDPI )
	{
		case (75): aLbMeasure.SelectEntryPos( (USHORT) MT_75DPI ); break;
		case (150): aLbMeasure.SelectEntryPos( (USHORT) MT_150DPI ); break;
		case (300): aLbMeasure.SelectEntryPos( (USHORT) MT_300DPI ); break;
		case (600): aLbMeasure.SelectEntryPos( (USHORT) MT_600DPI ); break;
		case (1200): aLbMeasure.SelectEntryPos( (USHORT) MT_1200DPI ); break;
		case (DPI_ORIGINAL): aLbMeasure.SelectEntryPos( (USHORT) MT_ORIGINAL ); break;
		case (DPI_FREEHAND): aLbMeasure.SelectEntryPos( (USHORT) MT_FREEHAND ); break;
		
		case( DPI_PAGESIZE_SCALED ):
		case( DPI_PAGESIZE ): 
			aLbMeasure.SelectEntryPos( (USHORT) MT_PAGESIZE ); 
		break;
	}

	aCbxScale.Check( ( nDPI != DPI_PAGESIZE ) && ( nDPI != DPI_ORIGINAL ) );

	if( nDPI != DPI_FREEHAND )
	{
		aFtWidth.Disable();
		aMtrFldWidth.Disable();
		aFtHeight.Disable();
		aMtrFldHeight.Disable();

		if( DPI_PAGESIZE_SCALED != nDPI && DPI_PAGESIZE != nDPI )
			aCbxScale.Disable();

		if( !aCtlPosition.IsEnabled() )
		{
			aCtlPosition.Enable();
			aCtlPosition.Invalidate();
			aGrpPosition.Enable();
		}

		ChangeMeasureHdl(NULL);
	}
	else
	{
		ImplSetFieldValue( aMtrFldWidth, ((const SfxUInt32Item&) rOutAttrs.Get(ATTR_PRINTOPT_WIDTH)).GetValue() );
		ImplSetFieldValue( aMtrFldHeight, ((const SfxUInt32Item&) rOutAttrs.Get(ATTR_PRINTOPT_HEIGHT)).GetValue() );
	}
}

// -----------------------------------------------------------------------------

BOOL SimPrintOptPage::FillItemSet(SfxItemSet& rOutAttrs )
{
	rOutAttrs.Put(SfxUInt16Item(ATTR_PRINTOPT_POSITION, (UINT16) aCtlPosition.GetActualRP() ) );
	rOutAttrs.Put(SfxUInt32Item(ATTR_PRINTOPT_WIDTH, ImplGetFieldValue( aMtrFldWidth ) ) );
	rOutAttrs.Put(SfxUInt32Item(ATTR_PRINTOPT_HEIGHT, ImplGetFieldValue( aMtrFldHeight ) ) );
	rOutAttrs.Put(SfxUInt16Item(ATTR_PRINTOPT_DPI, nDPI));

	return TRUE;
}

// -----------------------------------------------------------------------------

SfxTabPage* SimPrintOptPage::Create( Window* pWindow, const SfxItemSet& rOutAttrs )
{
	return new SimPrintOptPage( pWindow, rOutAttrs, NULL, Size() );
}

// -----------------------------------------------------------------------------

void SimPrintOptPage::PointChanged( Window* pWindow, RECT_POINT eRP )
{
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimPrintOptPage, ClickScaleHdl, void *, p )
{
	const BOOL bPageSize = (MeasureType) aLbMeasure.GetSelectEntryPos() == MT_PAGESIZE;

	if( aCbxScale.IsChecked() )
	{
		if( !bPageSize )
		{
			if( bWidthChanged )
				aMtrFldHeight.SetValue((long)(aMtrFldWidth.GetValue() / fXY));
			else
				aMtrFldWidth.SetValue((long)(aMtrFldHeight.GetValue() * fXY));
		}
	}

	if( bPageSize )
		ChangeMeasureHdl( NULL );

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimPrintOptPage, ChangeWidthHdl, void *, p )
{
	if( aCbxScale.IsChecked() )
		aMtrFldHeight.SetValue((long)(aMtrFldWidth.GetValue() / fXY));

	bWidthChanged = TRUE;

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimPrintOptPage, ChangeHeightHdl, void *, p )
{
	if( aCbxScale.IsChecked() )
		aMtrFldWidth.SetValue((long)(aMtrFldHeight.GetValue() * fXY));

	bWidthChanged = FALSE;

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimPrintOptPage, ChangeMeasureHdl, void *, p )
{
	const MeasureType eMT = (MeasureType) aLbMeasure.GetSelectEntryPos();

	switch( eMT )
	{
		case MT_75DPI:
		case MT_150DPI:
		case MT_300DPI:
		case MT_600DPI:
		case MT_1200DPI:
		case MT_ORIGINAL:
		{
			long nWidth, nHeight;

			if( MT_ORIGINAL != eMT )
			{
				const Size aSizePix( pFrame->GetSizePixel() );

				
				nDPI = 	eMT == MT_75DPI 	? 	  75 	:
						eMT == MT_150DPI    ?    150	:
						eMT == MT_300DPI    ?    300	:
						eMT == MT_600DPI    ?    600	:
						eMT == MT_1200DPI 	?   1200	:	75;

				nWidth = aSizePix.Width() * 2540 / nDPI;
				nHeight = aSizePix.Height() * 2540 / nDPI;
			}
			else
			{
				Size aPrintSize;

				nDPI = DPI_ORIGINAL;

				if( pFrame->GetPrefMapMode() == MapMode() )
					aPrintSize = GetParent()->PixelToLogic( pFrame->GetSizePixel(), MAP_100TH_MM );
				else
					aPrintSize = OutputDevice::LogicToLogic( pFrame->GetPrefSize(), pFrame->GetPrefMapMode(), MAP_100TH_MM );

				nWidth = aPrintSize.Width();
				nHeight = aPrintSize.Height();
			}
	
			ImplSetFieldValue( aMtrFldWidth, nWidth );
			ImplSetFieldValue( aMtrFldHeight, nHeight );

			aFtWidth.Disable();
			aMtrFldWidth.Disable();
			aFtHeight.Disable();
			aMtrFldHeight.Disable();
			aCbxScale.Disable();
			aCbxScale.Check(TRUE);
			if( !aCtlPosition.IsEnabled() )
			{
				aCtlPosition.Enable();
				aCtlPosition.Invalidate();
				aGrpPosition.Enable();
			}
		}
		break;

		case MT_PAGESIZE:
		{
			if( aCbxScale.IsChecked() )
			{
				double fPageXY = (double) aPageSize.Width() / aPageSize.Height();

				if (fPageXY < fXY)
				{
					ImplSetFieldValue( aMtrFldWidth, aPageSize.Width() );
					ImplSetFieldValue( aMtrFldHeight, (long)(aPageSize.Width()/fXY ) );
				}
				else
				{
					ImplSetFieldValue( aMtrFldWidth, (long)(aPageSize.Height()* fXY) );
					ImplSetFieldValue( aMtrFldHeight, aPageSize.Width() );
				}
			}
			else
			{
				ImplSetFieldValue( aMtrFldWidth, aPageSize.Width() );
				ImplSetFieldValue( aMtrFldHeight, aPageSize.Height() );
			}

			aFtWidth.Disable();
			aMtrFldWidth.Disable();
			aFtHeight.Disable();
			aMtrFldHeight.Disable();
			aCbxScale.Enable();

			aCtlPosition.SetActualRP( RP_MM );
			
			if( aCtlPosition.IsEnabled() )
			{
				aCtlPosition.Disable();
				aCtlPosition.Invalidate();
				aGrpPosition.Disable();
			}

			nDPI = aCbxScale.IsChecked() ? DPI_PAGESIZE_SCALED : DPI_PAGESIZE;
		}
		break;

		case MT_FREEHAND:
		{
			aFtWidth.Enable();
			aMtrFldWidth.Enable();
			aFtHeight.Enable();
			aMtrFldHeight.Enable();
			aCbxScale.Enable();

			if(!aCtlPosition.IsEnabled())
			{
				aCtlPosition.Enable();
				aCtlPosition.Invalidate();
				aGrpPosition.Enable();

			}

			nDPI = DPI_FREEHAND;
		}
		break;
	}

	return( 0L );
}
