/*************************************************************************
 *
 *  $RCSfile: rgbdlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/intitem.hxx>
#include "simattr.hxx"
#include "rgbdlg.hxx"
#include "rgbdlg.hrc"
#include "simresid.hxx"

/*************************************************************************
|*
|* Dialog zum Einstellen der RGB-Werte
|*
\************************************************************************/

SimRgbDlg::SimRgbDlg( Window* pWindow, const SfxItemSet& rInAttrs ) :
				ModalDialog		( pWindow, SimResId( DLG_RGB ) ),
				aFtRed      	( this, ResId( FT_RED ) ),
				aSbRed      	( this, ResId( SB_RED ) ),
				aMtrFldRed		( this, ResId( MTR_FLD_RED ) ),
				aFtGreen    	( this, ResId( FT_GREEN ) ),
				aSbGreen    	( this, ResId( SB_GREEN ) ),
				aMtrFldGreen   	( this, ResId( MTR_FLD_GREEN ) ),
				aFtBlue     	( this, ResId( FT_BLUE ) ),
				aSbBlue     	( this, ResId( SB_BLUE ) ),
				aMtrFldBlue		( this, ResId( MTR_FLD_BLUE ) ),
				aGrpRgb			( this, ResId( GRP_RGB ) ),
				aBtnOK			( this, ResId( BTN_OK ) ),
				aBtnCancel		( this, ResId( BTN_CANCEL ) ),
				aBtnHelp		( this, ResId( BTN_HELP ) ),
				rOutAttrs		( rInAttrs )
{
	FreeResource();

	aSbRed.SetScrollHdl( LINK ( this, SimRgbDlg, ScrollRedHdl ) );
	aSbGreen.SetScrollHdl( LINK ( this, SimRgbDlg, ScrollGreenHdl ) );
	aSbBlue.SetScrollHdl( LINK ( this, SimRgbDlg, ScrollBlueHdl ) );

	aMtrFldRed.SetModifyHdl( LINK ( this, SimRgbDlg, MtrFldRedHdl ) );
	aMtrFldGreen.SetModifyHdl( LINK ( this, SimRgbDlg, MtrFldGreenHdl ) );
	aMtrFldBlue.SetModifyHdl( LINK ( this, SimRgbDlg, MtrFldBlueHdl ) );

	aSbRed.SetRange( Range( 0, 200 ) );
	aSbGreen.SetRange( Range( 0, 200 ) );
	aSbBlue.SetRange( Range( 0, 200 ) );

	Reset();
}

/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/

SimRgbDlg::~SimRgbDlg()
{
}

/*************************************************************************
|*
|* Liest uebergebenen Item-Set
|*
\************************************************************************/

void SimRgbDlg::Reset()
{
	aMtrFldRed.SetValue( ( ( const SfxInt16Item& ) rOutAttrs.
						Get( ATTR_RGB_RED ) ).GetValue() );
	aMtrFldGreen.SetValue( ( ( const SfxInt16Item& ) rOutAttrs.
						Get( ATTR_RGB_GREEN ) ).GetValue() );
	aMtrFldBlue.SetValue( ( ( const SfxInt16Item& ) rOutAttrs.
						Get( ATTR_RGB_BLUE ) ).GetValue() );

	MtrFldRedHdl( this );
	MtrFldGreenHdl( this );
	MtrFldBlueHdl( this );
}

/*************************************************************************
|*
|* Fuellt uebergebenen Item-Set mit Dialogbox-Attributen
|*
\************************************************************************/

void SimRgbDlg::GetAttr( SfxItemSet& rOutAttrs )
{
	rOutAttrs.Put( SfxInt16Item( ATTR_RGB_RED, (INT16) aMtrFldRed.GetValue() ) );
	rOutAttrs.Put( SfxInt16Item( ATTR_RGB_GREEN, (INT16) aMtrFldGreen.GetValue() ) );
	rOutAttrs.Put( SfxInt16Item( ATTR_RGB_BLUE, (INT16) aMtrFldBlue.GetValue() ) );
}

/*************************************************************************
|*
|* ScrollBarHdl fuer Rotwerte
|*
\************************************************************************/

IMPL_LINK_INLINE_START( SimRgbDlg, ScrollRedHdl, void *, EMPTYARG )
{
	aMtrFldRed.SetValue( aSbRed.GetThumbPos() - 100 );

	return( 0L );
}
IMPL_LINK_INLINE_END( SimRgbDlg, ScrollRedHdl, void *, EMPTYARG )

/*************************************************************************
|*
|* ScrollBarHdl fuer Gruenwerte
|*
\************************************************************************/

IMPL_LINK_INLINE_START( SimRgbDlg, ScrollGreenHdl, void *, EMPTYARG )
{
	aMtrFldGreen.SetValue( aSbGreen.GetThumbPos() - 100 );

	return( 0L );
}
IMPL_LINK_INLINE_END( SimRgbDlg, ScrollGreenHdl, void *, EMPTYARG )

/*************************************************************************
|*
|* ScrollBarHdl fuer Blauwerte
|*
\************************************************************************/

IMPL_LINK_INLINE_START( SimRgbDlg, ScrollBlueHdl, void *, EMPTYARG )
{
	aMtrFldBlue.SetValue( aSbBlue.GetThumbPos() - 100 );

	return( 0L );
}
IMPL_LINK_INLINE_END( SimRgbDlg, ScrollBlueHdl, void *, EMPTYARG )

/*************************************************************************
|*
|* MtrFldHdl fuer Rotwerte
|*
\************************************************************************/

IMPL_LINK( SimRgbDlg, MtrFldRedHdl, void *, EMPTYARG )
{
	aSbRed.SetThumbPos( aMtrFldRed.GetValue() + 100 );

	return( 0L );
}

/*************************************************************************
|*
|* MtrFldHdl fuer Gruenwerte
|*
\************************************************************************/

IMPL_LINK( SimRgbDlg, MtrFldGreenHdl, void *, EMPTYARG )
{
	aSbGreen.SetThumbPos( aMtrFldGreen.GetValue() + 100 );

	return( 0L );
}

/*************************************************************************
|*
|* MtrFldHdl fuer Blauwerte
|*
\************************************************************************/

IMPL_LINK( SimRgbDlg, MtrFldBlueHdl, void *, EMPTYARG )
{
	aSbBlue.SetThumbPos( aMtrFldBlue.GetValue() + 100 );

	return( 0L );
}



