/*************************************************************************
 *
 *  $RCSfile: rotate.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/intitem.hxx>
#include <svtools/aeitem.hxx>
#include "glob.hxx"
#include "strings.hrc"
#ifdef ICC
#undef ATTR_ROTATE_ANGLE
#endif
#include "simattr.hxx"
#include "rotate.hxx"
#include "rotate.hrc"
#include "simresid.hxx"

/*************************************************************************
|*
|* Dialog zum Einstellen des Drehwinkels
|*
\************************************************************************/

SimRotateDlg::SimRotateDlg( Window* pParent, const SfxItemSet& rInAttrs  ) :
		SfxSingleTabDialog( pParent, rInAttrs, TAB_ROTATE, FALSE )
{
	SetTabPage( new SimRotatePage( this, rInAttrs ) );

	SetText( SimResId( STR_DLG_ROTATE ) );
}

/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/

SimRotateDlg::~SimRotateDlg()
{
}

/*************************************************************************
|*
|* Dialog (TabPage) zum Einstellen des Drehwinkels
|*
\************************************************************************/

SimRotatePage::SimRotatePage( Window* pWindow, const SfxItemSet& rInAttrs ) :
				SvxTabPage		(pWindow, SimResId( TAB_ROTATE ), rInAttrs ),
				aCtlAngle		(this, ResId(CTL_ANGLE ), RP_RM, 200, 80, CS_ANGLE ),
				aMtrFldAngle	(this, ResId(MTR_FLD_ANGLE)),
				aRbtLeft		(this, ResId(RBT_LEFT)),
				aRbtRight		(this, ResId(RBT_RIGHT)),
				aGrpBox         (this, ResId(GRP_ORIENTATION)),
				rOutAttrs		(rInAttrs)
{
	FreeResource();
	aMtrFldAngle.SetModifyHdl( LINK( this, SimRotatePage, ModifiedHdl ) );
}

/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/

SimRotatePage::~SimRotatePage()
{
}

/*************************************************************************
|*
|* Liest uebergebenen Item-Set
|*
\************************************************************************/

void SimRotatePage::Reset( const SfxItemSet& rOutAttrs )
{
	aMtrFldAngle.SetValue(((const SfxUInt32Item&) rOutAttrs.
						Get(ATTR_ROTATE_ANGLE)).GetValue());
	ModifiedHdl( NULL );

	ORIENTATION eOrientation = (ORIENTATION) ((const SfxAllEnumItem&) rOutAttrs.
								Get(ATTR_ROTATE_ORIENTATION)).GetValue();
	switch( eOrientation )
	{
		case LEFT:	aRbtLeft.Check(); 	break;
		case RIGHT:	aRbtRight.Check(); break;
	}
}

/*************************************************************************
|*
|* Fuellt uebergebenen Item-Set mit Dialogbox-Attributen
|*
\************************************************************************/

BOOL SimRotatePage::FillItemSet( SfxItemSet& rOutAttrs )
{
	rOutAttrs.Put(SfxUInt32Item( ATTR_ROTATE_ANGLE, aMtrFldAngle.GetValue() ) );

	ORIENTATION eOrientation = aRbtLeft.IsChecked() ? LEFT : RIGHT;
	rOutAttrs.Put(SfxAllEnumItem(ATTR_ROTATE_ORIENTATION, eOrientation));

	return( TRUE );
}

/*************************************************************************
|*
|* Erzeugt die Seite
|*
\************************************************************************/

SfxTabPage* SimRotatePage::Create( Window* pWindow,
				const SfxItemSet& rOutAttrs )
{
	return(new SimRotatePage(pWindow, rOutAttrs));
}

/*************************************************************************
|*
|* Setzt das MetricField mit dem Wert im Control
|*
\************************************************************************/

void SimRotatePage::PointChanged( Window* pWindow, RECT_POINT eRP )
{
	if ( &aCtlAngle == pWindow )
	{
		switch( eRP )
		{
			case RP_LT: aMtrFldAngle.SetUserValue( 13500, FUNIT_NONE ); break;
			case RP_MT: aMtrFldAngle.SetUserValue(  9000, FUNIT_NONE ); break;
			case RP_RT: aMtrFldAngle.SetUserValue(  4500, FUNIT_NONE ); break;
			case RP_LM: aMtrFldAngle.SetUserValue( 18000, FUNIT_NONE ); break;
			case RP_RM: aMtrFldAngle.SetUserValue(     0, FUNIT_NONE ); break;
			case RP_LB: aMtrFldAngle.SetUserValue( 22500, FUNIT_NONE ); break;
			case RP_MB: aMtrFldAngle.SetUserValue( 27000, FUNIT_NONE ); break;
			case RP_RB: aMtrFldAngle.SetUserValue( 31500, FUNIT_NONE ); break;
		}
	}
}

/*************************************************************************
|*
|* Setzt das AngleControl mit dem Wert aus dem MetricField
|*
\************************************************************************/

IMPL_LINK( SimRotatePage, ModifiedHdl, void*, p )
{
	// Setzen des Winkels im AngleControl
	switch( aMtrFldAngle.GetValue() )
	{
		case 13500: aCtlAngle.SetActualRP( RP_LT ); break;
		case  9000: aCtlAngle.SetActualRP( RP_MT ); break;
		case  4500: aCtlAngle.SetActualRP( RP_RT ); break;
		case 18000: aCtlAngle.SetActualRP( RP_LM ); break;
		case     0: aCtlAngle.SetActualRP( RP_RM ); break;
		case 22500: aCtlAngle.SetActualRP( RP_LB ); break;
		case 27000: aCtlAngle.SetActualRP( RP_MB ); break;
		case 31500: aCtlAngle.SetActualRP( RP_RB ); break;
		default:    aCtlAngle.SetActualRP( RP_MM ); break;
	}

	return 0L;
}



