/*************************************************************************
 *
 *  $RCSfile: sizedlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svtools/intitem.hxx>
#include <svtools/itemset.hxx>
#include <sfx2/app.hxx>
#include <svx/dlgutil.hxx>
#include "simattr.hxx"
#include "sizedlg.hxx"
#include "sizedlg.hrc"
#include "simresid.hxx"

/*************************************************************************
|*
|* Dialog zum Einstellen der Groesse
|*
\************************************************************************/

SimSizeDlg::SimSizeDlg( Window* pWindow, const SfxItemSet& rInAttrs ) :
				ModalDialog		 ( pWindow, SimResId( DLG_SIZE ) ),
				aFtWidth		 ( this, ResId( FT_WIDTH ) ),
				aMtrFldPixelX    ( this, ResId( MTR_FLD_PIXEL_X ) ),
				aMtrFldMmX	 	 ( this, ResId( MTR_FLD_MM_X ) ),
				aMtrFldPercentX  ( this, ResId( MTR_FLD_PERCENT_X ) ),
				aFtHeight   	 ( this, ResId( FT_HEIGHT ) ),
				aMtrFldPixelY    ( this, ResId( MTR_FLD_PIXEL_Y ) ),
				aMtrFldMmY       ( this, ResId( MTR_FLD_MM_Y ) ),
				aMtrFldPercentY  ( this, ResId( MTR_FLD_PERCENT_Y ) ),
				aCbxScale        ( this, ResId( CBX_SCALE ) ),
				aCbxInterpolation( this, ResId( CBX_INTERPOLATION ) ),
				aGrpSize         ( this, ResId( GRP_SIZE ) ),
				aFtMemorySize    ( this, ResId( FT_MEMORY_SIZE ) ),
				aFiMemorySize	 ( this, ResId( FI_MEMORY_SIZE ) ),
				aGrpInformation  ( this, ResId( GRP_INFORMATION ) ),
				aBtnOK      	 ( this, ResId( BTN_OK ) ),
				aBtnCancel  	 ( this, ResId( BTN_CANCEL ) ),
				aBtnHelp    	 ( this, ResId( BTN_HELP ) ),
				rOutAttrs		 ( rInAttrs )
{
	const FieldUnit eFUnit = SFX_APP()->GetOptions().GetMetric();

	FreeResource();

	aMtrFldMmX.SetUnit( eFUnit );
	aMtrFldMmY.SetUnit( eFUnit );

	aMtrFldPixelX.SetModifyHdl( LINK( this, SimSizeDlg, ChangePixelXHdl ) );
	aMtrFldMmX.SetModifyHdl( LINK( this, SimSizeDlg, ChangeMmXHdl ) );
	aMtrFldPercentX.SetModifyHdl( LINK( this, SimSizeDlg, ChangePercentXHdl ) );
	aMtrFldPixelY.SetModifyHdl( LINK( this, SimSizeDlg, ChangePixelYHdl ) );
	aMtrFldMmY.SetModifyHdl( LINK( this, SimSizeDlg, ChangeMmYHdl ) );
	aMtrFldPercentY.SetModifyHdl( LINK( this, SimSizeDlg, ChangePercentYHdl ) );
	aCbxScale.SetClickHdl( LINK( this, SimSizeDlg, ClickScaleHdl ) );

	Reset();
}

/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/

SimSizeDlg::~SimSizeDlg()
{
}

/*************************************************************************
|*
|* Liest uebergebenen Item-Set
|*
\************************************************************************/

void SimSizeDlg::Reset()
{
	bXChanged = TRUE;

	lOldPixelX = ( ( const SfxUInt32Item& ) rOutAttrs.Get( ATTR_SIZE_PIXEL_X ) ).GetValue();
	aMtrFldPixelX.SetValue( lOldPixelX );

	lOldPixelY = ( ( const SfxUInt32Item& ) rOutAttrs.Get( ATTR_SIZE_PIXEL_Y ) ).GetValue();
	aMtrFldPixelY.SetValue( lOldPixelY );

	lOldMmX = ( ( const SfxUInt32Item& ) rOutAttrs.Get( ATTR_SIZE_MM_X ) ).GetValue();
	SetMetricValue( aMtrFldMmX, lOldMmX, SFX_MAPUNIT_100TH_MM );

	lOldMmY = ( ( const SfxUInt32Item& ) rOutAttrs.Get( ATTR_SIZE_MM_Y ) ).GetValue();
	SetMetricValue( aMtrFldMmY, lOldMmY, SFX_MAPUNIT_100TH_MM );

	lOldPercentX = 10000L;
	aMtrFldPercentX.SetValue( lOldPercentX );

	lOldPercentY = 10000L;
	aMtrFldPercentY.SetValue( lOldPercentY );

	aCbxScale.Check(TRUE);

	nColorDepth = ( ( const SfxUInt16Item& ) rOutAttrs.Get( ATTR_SIZE_DEPTH ) ).GetValue();

	MemorySizeHdl( this );
}

/*************************************************************************
|*
|* Fuellt uebergebenen Item-Set mit Dialogbox-Attributen
|*
\************************************************************************/

void SimSizeDlg::GetAttr( SfxItemSet& rOutAttrs )
{
	rOutAttrs.Put( SfxUInt32Item( ATTR_SIZE_PIXEL_X, (UINT32) aMtrFldPixelX.GetValue() ) );
	rOutAttrs.Put( SfxUInt32Item( ATTR_SIZE_PIXEL_Y, (UINT32) aMtrFldPixelY.GetValue() ) );
	rOutAttrs.Put( SfxUInt32Item( ATTR_SIZE_MM_X, (UINT32) GetCoreValue( aMtrFldMmX, SFX_MAPUNIT_100TH_MM ) ) );
	rOutAttrs.Put( SfxUInt32Item( ATTR_SIZE_MM_Y, (UINT32) GetCoreValue( aMtrFldMmY, SFX_MAPUNIT_100TH_MM ) ) );
	rOutAttrs.Put( SfxBoolItem( ATTR_SIZE_INTERPOL, aCbxInterpolation.IsChecked() ) );
}

/*************************************************************************
|*
|* Memory-Size-Hdl
|*
\************************************************************************/

long __EXPORT SimSizeDlg::MemorySizeHdl ( void *p )
{
	double 	fBytes 	= aMtrFldPixelX.GetValue() * aMtrFldPixelY.GetValue() * nColorDepth * 0.125;
	ULONG 	nTemp 	= (ULONG)fBytes;
	String	aStrg;

	if (nTemp < 1024)
	{
		aStrg.Assign( String::CreateFromInt32( nTemp ) );
		aStrg.Append( String( String( RTL_CONSTASCII_USTRINGPARAM( " Bytes" ) ) ) );
	}
	else
	{
		aStrg.Assign( String::CreateFromInt32( nTemp >> 10 ) );
		aStrg.Append( String( RTL_CONSTASCII_USTRINGPARAM( " KBytes" ) ) );
	}

	aFiMemorySize.SetText( aStrg );

	return 0L;
}

/*************************************************************************
|*
|* Scale-Hdl (Checkbox)
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ClickScaleHdl, void *, p )
{
	if( aCbxScale.IsChecked() )
	{
		double fTemp = 1.0;

		if( bXChanged )
		{
		  	fTemp = (double) aMtrFldPixelX.GetValue() / (double) lOldPixelX;

			aMtrFldPixelY.SetValue( (long) ( (double) lOldPixelY * fTemp ) );
			SetMetricValue( aMtrFldMmY, (long) ( (double) lOldMmY * fTemp ), SFX_MAPUNIT_100TH_MM );
			aMtrFldPercentY.SetValue( (long) ( (double) lOldPercentY * fTemp ) );
		}
		else
		{
			fTemp = (double) aMtrFldPixelY.GetValue() / (double) lOldPixelY;

			aMtrFldPixelX.SetValue( (long) ( (double) lOldPixelX * fTemp ) );
			SetMetricValue( aMtrFldMmX, (long) ( (double) lOldMmX * fTemp ), SFX_MAPUNIT_100TH_MM );
			aMtrFldPercentX.SetValue( (long) ( (double) lOldPercentX * fTemp ) );
		}

		MemorySizeHdl( this );
	}

	return( 0L );
}

/*************************************************************************
|*
|* Pixel-X-Hdl
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ChangePixelXHdl, void *, p )
{
	// Berechnung des Faktors
	double fTemp = (double) aMtrFldPixelX.GetValue() / (double) lOldPixelX;

	SetMetricValue( aMtrFldMmX, (long) ( (double) lOldMmX * fTemp ), SFX_MAPUNIT_100TH_MM );
	aMtrFldPercentX.SetValue( (long) ( (double) lOldPercentX * fTemp ) );

	if( aCbxScale.IsChecked() )
	{
		aMtrFldPixelY.SetValue( (long) ( (double) lOldPixelY * fTemp ) );
		SetMetricValue( aMtrFldMmY, (long) ( (double) lOldMmY * fTemp ), SFX_MAPUNIT_100TH_MM );
		aMtrFldPercentY.SetValue( (long) ( (double) lOldPercentY * fTemp ) );
	}

	bXChanged = TRUE;

	MemorySizeHdl( this );

	return( 0L );
}

/*************************************************************************
|*
|* Millimeter-X-Hdl
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ChangeMmXHdl, void *, p )
{
	// Berechnung des Faktors
	double fTemp = (double) GetCoreValue( aMtrFldMmX, SFX_MAPUNIT_100TH_MM ) / (double) lOldMmX;

	aMtrFldPixelX.SetValue( (long) ( (double) lOldPixelX * fTemp ) );
	aMtrFldPercentX.SetValue( (long) ( (double) lOldPercentX * fTemp ) );

	if( aCbxScale.IsChecked() )
	{
		aMtrFldPixelY.SetValue( (long) ( (double) lOldPixelY * fTemp ) );
		SetMetricValue( aMtrFldMmY, (long) ( (double) lOldMmY * fTemp ), SFX_MAPUNIT_100TH_MM );
		aMtrFldPercentY.SetValue( (long) ( (double) lOldPercentY * fTemp ) );
	}

	bXChanged = TRUE;

	MemorySizeHdl( this );

	return( 0L );
}

/*************************************************************************
|*
|* Prozent-X-Hdl
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ChangePercentXHdl, void *, p )
{
	// Berechnung des Faktors
	double fTemp = (double) aMtrFldPercentX.GetValue() / (double) lOldPercentX;

	aMtrFldPixelX.SetValue( (long) ( (double) lOldPixelX * fTemp ) );
	SetMetricValue( aMtrFldMmX, (long) ( (double) lOldMmX * fTemp ), SFX_MAPUNIT_100TH_MM );

	if( aCbxScale.IsChecked() )
	{
		aMtrFldPixelY.SetValue( (long) ( (double) lOldPixelY * fTemp ) );
		SetMetricValue( aMtrFldMmY, (long) ( (double) lOldMmY * fTemp ), SFX_MAPUNIT_100TH_MM );
		aMtrFldPercentY.SetValue( (long) ( (double) lOldPercentY * fTemp ) );
	}

	bXChanged = TRUE;

	MemorySizeHdl( this );

	return( 0L );
}

/*************************************************************************
|*
|* Pixel-Y-Hdl
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ChangePixelYHdl, void *, p )
{
	// Berechnung des Faktors
	double fTemp = (double) aMtrFldPixelY.GetValue() / (double) lOldPixelY;

	SetMetricValue( aMtrFldMmY, (long) ( (double) lOldMmY * fTemp ), SFX_MAPUNIT_100TH_MM );
	aMtrFldPercentY.SetValue( (long) ( (double) lOldPercentY * fTemp ) );

	if( aCbxScale.IsChecked() )
	{
		aMtrFldPixelX.SetValue( (long) ( (double) lOldPixelX * fTemp ) );
		SetMetricValue( aMtrFldMmX, (long) ( (double) lOldMmX * fTemp ), SFX_MAPUNIT_100TH_MM );
		aMtrFldPercentX.SetValue( (long) ( (double) lOldPercentX * fTemp ) );
	}

	bXChanged = FALSE;

	MemorySizeHdl( this );

	return( 0L );
}

/*************************************************************************
|*
|* Millimeter-Y-Hdl
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ChangeMmYHdl, void *, p )
{
	// Berechnung des Faktors
	double fTemp = (double) GetCoreValue( aMtrFldMmY, SFX_MAPUNIT_100TH_MM ) / (double) lOldMmY;

	aMtrFldPixelY.SetValue( (long) ( (double) lOldPixelY * fTemp ) );
	aMtrFldPercentY.SetValue( (long) ( (double) lOldPercentY * fTemp ) );

	if( aCbxScale.IsChecked() )
	{
		aMtrFldPixelX.SetValue( (long) ( (double) lOldPixelX * fTemp ) );
		SetMetricValue( aMtrFldMmX, (long) ( (double) lOldMmX * fTemp ), SFX_MAPUNIT_100TH_MM );
		aMtrFldPercentX.SetValue( (long) ( (double) lOldPercentX * fTemp ) );
	}

	bXChanged = FALSE;

	MemorySizeHdl( this );

	return( 0L );
}

/*************************************************************************
|*
|* Prozent-Y-Hdl
|*
\************************************************************************/

IMPL_LINK( SimSizeDlg, ChangePercentYHdl, void *, p )
{
	// Berechnung des Faktors
	double fTemp = (double) aMtrFldPercentY.GetValue() / (double) lOldPercentY;

	aMtrFldPixelY.SetValue( (long) ( (double) lOldPixelY * fTemp ) );
	SetMetricValue( aMtrFldMmY, (long) ( (double) lOldMmY * fTemp ), SFX_MAPUNIT_100TH_MM );

	if( aCbxScale.IsChecked() )
	{
		aMtrFldPixelX.SetValue( (long) ( (double) lOldPixelX * fTemp ) );
		SetMetricValue( aMtrFldMmX, (long) ( (double) lOldMmX * fTemp ), SFX_MAPUNIT_100TH_MM );
		aMtrFldPercentX.SetValue( (long) ( (double) lOldPercentX * fTemp ) );
	}

	bXChanged = FALSE;

	MemorySizeHdl( this );

	return( 0L );
}




