/*************************************************************************
 *
 *  $RCSfile: docshell.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _XOUTBMP_STATICS_ONLY 1

#include <string> // HACK: prevent conflict between STLPORT and Workshop headers

#include <tools/urlobj.hxx>
#include <tools/string.hxx>
#include <vcl/graph.hxx>
#include <vcl/status.hxx>
#include <vcl/msgbox.hxx>
#include <so3/clsids.hxx>
#include <so3/svstor.hxx>
#include <svtools/filter.hxx>
#include <svtools/eitem.hxx>
#include <svtools/undo.hxx>
#include <sfx2/bindings.hxx>
#include <sfx2/dispatch.hxx>
#include <sfx2/app.hxx>
#include <sfx2/sfxdir.hxx>
#include <sfx2/docfile.hxx>
#include <sfx2/inimgr.hxx>
#include <sfx2/docfilt.hxx>
#include <sfx2/request.hxx>
#include <sfx2/printer.hxx>
#include <sfx2/fcontnr.hxx>
#include <sfx2/saveopt.hxx>
#include <svx/svxids.hrc>
#include <svx/impgrf.hxx>
#include <svx/xoutbmp.hxx>
#include "simapp.hrc"
#include "strings.hrc"
#include "simresid.hxx"
#include "about.hrc"
#include "viewshel.hxx"
#include "simview.hxx"
#include "colstat.hxx"
#include "progress.hxx"
#include "simmodel.hxx"
#include "svimage.hxx"
#include "docshell.hxx"

#define ITEMID_COLOR_TABLE SID_COLOR_TABLE
#include <svx/drawitem.hxx>

#define SimDrawDocShell
#include "simslots.hxx"

// -----------
// - Defines -
// -----------

#define DOCUMENT_BUFFER_SIZE	((USHORT) 32768)

// -----------
// - Statics -
// -----------

static const char __FAR_DATA pStarImageDoc3[] = "StarImageDocument";
static const char __FAR_DATA pStarImageDoc[] = "StarImageDocument 4.0";

// -------------------
// - SimDrawDocShell -
// -------------------

SFX_IMPL_INTERFACE( SimDrawDocShell, SfxObjectShell, SimResId( 0 ) ) {}
SFX_IMPL_OBJECTFACTORY_DLL( SimDrawDocShell, simage, SvGlobalName(SO3_SIM_CLASSID_50), Sim )

// -----------------------------------------------------------------------------

SimDrawDocShell::SimDrawDocShell(SfxObjectCreateMode eMode) :
			SfxObjectShell		( eMode ),
			pPrinter			( NULL ),
			pMenuBar			( NULL ),
			pMenuMgr			( NULL ),
			pModel				( NULL ),
			bIPActive			( FALSE ),
			pViewShell			( NULL ),
			bInMailExport		( FALSE )
{
    SetShell( this );
    pModel = new SimModel( this );
    SetPool( pModel->GetPool() );
    pUndoManager = new SfxUndoManager( 1 );
	DisableViewScaling( TRUE );
	PutItem( SvxColorTableItem( pModel->GetColorTable() ) );
}

// -----------------------------------------------------------------------------

SimDrawDocShell::SimDrawDocShell( SimModel* pSimModel ) :
			SfxObjectShell		( SFX_CREATE_MODE_STANDARD ),
			pPrinter			( NULL ),
			pMenuBar			( NULL ),
			pMenuMgr			( NULL ),
			pModel				( pSimModel ),
			bIPActive			( FALSE ),
			pViewShell			( NULL ),
			bInMailExport		( FALSE )
{
    SetShell( this );
    SetPool( pModel->GetPool() );
    pUndoManager = new SfxUndoManager( 1 );
	DisableViewScaling( TRUE );
	PutItem( SvxColorTableItem( pModel->GetColorTable() ) );
}

// -----------------------------------------------------------------------------

SimDrawDocShell::~SimDrawDocShell()
{
    delete pPrinter;
    delete pUndoManager;
    delete pModel;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::FillStatusBar( StatusBar& rBar )
{
	SIM_MOD()->FillStatusBar( rBar );
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::Draw( OutputDevice* pOut, const JobSetup& rSetup, USHORT nAspect )
{
    const Graphic aGraphic( pModel->GetGraphic() );

    if( aGraphic.GetType() != GRAPHIC_NONE )
    {
		// Preview
		if( nAspect == ASPECT_THUMBNAIL )
		{
			const Size	aVisSize( GetVisArea( ASPECT_THUMBNAIL ).GetSize() );
			Size		aGrfSize( aGraphic.GetPrefSize() );
			BOOL		bRet = FALSE;

			if ( aGrfSize.Width() && aGrfSize.Height() && aVisSize.Height() )
			{
				const double fGrfWH = (double) aGrfSize.Width() / aGrfSize.Height();

				// Bitmap an Fenstergroesse anpassen
				if ( fGrfWH < ( (double) aVisSize.Width() / aVisSize.Height() ) )
				{
					aGrfSize.Width() = (long) ( aVisSize.Height() * fGrfWH );
					aGrfSize.Height()= aVisSize.Height();
				}
				else
				{
					aGrfSize.Width() = aVisSize.Width();
					aGrfSize.Height()= (long) ( aVisSize.Width() / fGrfWH);
				}

				const Point aGrfPos( ( aVisSize.Width()  - aGrfSize.Width() ) >> 1,
									 ( aVisSize.Height() - aGrfSize.Height() ) >> 1 );

				aGraphic.Draw( pOut, aGrfPos, aGrfSize );
			}
		}
		else
		{
			const MapMode aOldMapMode( pOut->GetMapMode() );

			if ( pOut->GetOutDevType() == OUTDEV_PRINTER )
			{
				MapMode aMapMode( aOldMapMode );
				Point	aOrigin( aMapMode.GetOrigin() );

				aOrigin.X() += 1;
				aOrigin.Y() += 1;
				aMapMode.SetOrigin( aOrigin );
				pOut->SetMapMode( aMapMode );
			}

			const Rectangle aVisArea( GetVisArea() );

			aGraphic.Draw( pOut, aVisArea.TopLeft(), aVisArea.GetSize() );

			if ( pOut->GetOutDevType() == OUTDEV_PRINTER )
				pOut->SetMapMode( aOldMapMode );
		}
    }
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::Execute( SfxRequest& rReq )
{
    switch ( rReq.GetSlot() )
    {
        case( SID_CLOSEDOC ):
			SfxObjectShell::Close();
		break;

		case( SID_SIMFILEINFO ):
			ShowFileInfo();
		break;

		case( SID_MAIL_PREPAREEXPORT ):
		{
			bInMailExport = TRUE;
		}
		break;

		default:
		break;
    }
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::GetState( SfxItemSet &rSet )
{
	if( rSet.GetItemState( SID_SIMFILEINFO ) != SFX_ITEM_UNKNOWN )
	{
		const String&	rPhysName = GetMedium()->GetPhysicalName();
		BOOL			bDisable = TRUE;

		if( rPhysName.Len() )
		{
/* !!!DirEntry
			DirEntry			aEntry( rPhysName ); aEntry.ToAbs();
			SfxExtDirEntryRef	pExtDirEntry = new SfxExtDirEntry( aEntry );

			if ( FSYS_KIND_FILE == pExtDirEntry->GetKind() )
				bDisable = FALSE;
*/
		}
		if ( bDisable )
			rSet.DisableItem( SID_SIMFILEINFO );
	}
}

// -----------------------------------------------------------------------------

SfxUndoManager* SimDrawDocShell::GetUndoManager()
{
    return pUndoManager;
}

// -----------------------------------------------------------------------------

SfxPrinter* SimDrawDocShell::GetPrinter()
{
    if( !pPrinter )
    {
        SfxItemSet* pSet = new SfxItemSet( *pModel->GetPool(), SID_PRINTER_NOTFOUND_WARN, SID_PRINTER_NOTFOUND_WARN );
		pSet->Put( SfxBoolItem( SID_PRINTER_NOTFOUND_WARN, TRUE ) );
        pPrinter = new SfxPrinter( pSet );
    }

    return pPrinter;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::SetPrinter( SfxPrinter *pNewPrinter )
{
    if( pPrinter && ( pPrinter != pNewPrinter ) )
        delete pPrinter;

    pPrinter = pNewPrinter;
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::InitNew( SvStorage * pStor )
{
    BOOL bRet = SfxInPlaceObject::InitNew( pStor );

    if ( bRet )
    {
        SvStorage* pStor2 = GetStorage();

		aDocStm = pStor2->OpenStream( String( pStarImageDoc, RTL_TEXTENCODING_UTF8 ) );
        bRet = ( aDocStm->GetError() == 0 );
		SetVisArea( Rectangle( Point(), Size( 8000, 7000 ) ) );
    }

    return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::Load( SvStorage* pStor )
{
    SimModel*		pOldModel = pModel;
    SfxUndoManager*	pOldUndoManager = pUndoManager;
    BOOL			bRet = SfxInPlaceObject::Load(pStor);

    // altes Dokument "ausklinken", neues erzeugen
    pModel = new SimModel( this );
	PutItem( SvxColorTableItem( pModel->GetColorTable() ) );
	SetPool( pModel->GetPool() );
    pUndoManager = new SfxUndoManager( 1 );

    if( bRet )
    {
        BOOL	bIsEmptyStreams = FALSE;
		String	aStmStr;

		if ( pStor->IsStream( String( pStarImageDoc, RTL_TEXTENCODING_UTF8 ) ) )
			aStmStr = String( pStarImageDoc, RTL_TEXTENCODING_UTF8 );
		else if ( pStor->IsStream( String( pStarImageDoc3, RTL_TEXTENCODING_UTF8 ) ) )
			aStmStr = String( pStarImageDoc3, RTL_TEXTENCODING_UTF8 );

        if ( aStmStr.Len() )
        {
            SvStorageStreamRef aDocStm = pStor->OpenStream( aStmStr );

            if ( !aDocStm->GetError() )
            {
                aDocStm->SetBufferSize(DOCUMENT_BUFFER_SIZE);
                aDocStm->SetKey(pStor->GetKey()); // Passwort setzen

                *aDocStm >> *pModel;

                if (aDocStm->GetError() == SVSTREAM_FILEFORMAT_ERROR)
                {
					ErrorHandler::HandleError( ERRCODE_IO_WRONGFORMAT );
                    bRet = FALSE;
                }

                aDocStm->SetBufferSize(0);
            }
            else
                bRet = FALSE;

			DBG_ASSERT( aDocStm->GetError() == 0, "Fehler beim Laden des Models");

            if ( bRet )
                bRet = ( aDocStm->GetError() == 0 );

            bIsEmptyStreams = FALSE;
        }
        else
        {
            // wenn der Stream gar nicht vorhanden ist, gilt das trotzdem als
            // Erfolg (fuer das Laden ungeaenderter OLE-Objekte)
            bIsEmptyStreams &= TRUE;
            bRet &= TRUE;
        }

        // (noch) keine ungesicherten Aenderungen im Model
        if ( bRet )
            pModel->SetChanged(FALSE);
    }


    // wenn erfolgreich, den Printer und das alte Dokument loeschen,
    // sonst neues Dokument loeschen und altes reaktivieren
    if ( bRet )
    {
        // den Printer loeschen, den der hat ein ItemSet, das den Pool des
        // alten Dokumentes benutzt - und der verschwindet mit dem Dokument!
        delete pPrinter;
        pPrinter = NULL;
        delete pOldModel;
        delete pOldUndoManager;

		FinishedLoading( SFX_LOADED_ALL );
    }
    else
    {
        delete pModel;
        pModel = pOldModel;
	    SetPool( pModel->GetPool() );
        delete pUndoManager;
        pUndoManager = pOldUndoManager;
    }

    return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::ConvertFrom( SfxMedium &rMedium )
{
    Graphic			aGraphic;
	String			aFilterName( rMedium.GetFilter()->GetFilterName() );
    BOOL			bRet = FALSE;
    BOOL			bProg = ( GetCreateMode() != SFX_CREATE_MODE_EMBEDDED );

    rMedium.Close();

    bRet = DoImportExport( rMedium.GetPhysicalName(), aGraphic, IMPEXP_IMPORT, &aFilterName) == 0;

    if ( bRet )
    {
		const Size	aSizePix( aGraphic.GetBitmap().GetSizePixel() );

		bRet = FALSE;

        if ( aSizePix.Width() && aSizePix.Height() )
        {
			pModel->SetGraphic( aGraphic );
			bRet = TRUE;
        }

        if ( !bRet )
			SetError( ERRCODE_IO_OUTOFMEMORY );
		else
			FinishedLoading( SFX_LOADED_ALL );
    }

    return bRet;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::HandsOff()
{
    aDocStm.Clear();
    SfxInPlaceObject::HandsOff();
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::Save()
{
    BOOL bRet = SfxInPlaceObject::Save();

    if( bRet )
    {
        String      aDocStr( pStarImageDoc, RTL_TEXTENCODING_UTF8 );
		SvStorage*	pStor = GetStorage();
		const long	nFileFormat = pStor->GetVersion();

		if ( nFileFormat == SOFFICE_FILEFORMAT_31 )
			aDocStr = String( pStarImageDoc3, RTL_TEXTENCODING_UTF8 );

        SvStorageStreamRef  aStm = pStor->OpenStream( aDocStr, STREAM_READ | STREAM_WRITE | STREAM_TRUNC );

        if ( !aStm->GetError() )
        {
            aStm->SetSize( 0 );
            aStm->SetBufferSize( DOCUMENT_BUFFER_SIZE );
            *aStm << *GetModel();
            aStm->SetBufferSize(0);
        }
        else
            bRet = FALSE;

        if ( bRet )
            bRet = ( aStm->GetError() == 0 );

        DBG_ASSERT( bRet, "Fehler beim Schreiben des Models" );
    }

    return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::SaveAs( SvStorage * pStor )
{
    BOOL bRet = SfxInPlaceObject::SaveAs(pStor);

    if ( bRet )
    {
        String      aDocStr( pStarImageDoc, RTL_TEXTENCODING_UTF8 );
		const long	nFileFormat = pStor->GetVersion();

		if ( nFileFormat == SOFFICE_FILEFORMAT_31 )
			aDocStr = String( pStarImageDoc3, RTL_TEXTENCODING_UTF8 );

        SvStorageStreamRef aStm = pStor->OpenStream( aDocStr, STREAM_READ | STREAM_WRITE | STREAM_TRUNC );

        if ( !aStm->GetError() )
        {
			aStm->SetBufferSize( DOCUMENT_BUFFER_SIZE );
            aStm->SetKey( pStor->GetKey() ); // Passwort setzen
            *aStm << *GetModel();
            aStm->SetBufferSize(0);
        }
        else
            bRet = FALSE;

        if ( bRet )
            bRet = aStm->GetError() == 0;

        DBG_ASSERT(bRet, "Fehler beim Schreiben des Models");
    }

    return bRet;
}

#ifdef WNT
#pragma optimize ( "", off )
#endif

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::ConvertTo(SfxMedium &rMedium)
{
	Graphic	aGraphic( pModel->GetGraphic() );
    String	aFilterName( rMedium.GetFilter()->GetFilterName() );
    USHORT	nReturn = 1;

    rMedium.Close();

	if( aGraphic.GetType() != GRAPHIC_NONE )
		nReturn = DoImportExport( rMedium.GetPhysicalName(), aGraphic, IMPEXP_EXPORT, &aFilterName );

    return( ( nReturn == 0 ) && ( GetError() == 0 ) );
}

#ifdef WNT
#pragma optimize ( "", on )
#endif

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::SaveCompleted( SvStorage * pStor )
{
    return SfxInPlaceObject::SaveCompleted(pStor);
}

// -----------------------------------------------------------------------------

Bitmap SimDrawDocShell::GetStyleFamilyBitmap(SfxStyleFamily eFamily)
{
    return SfxObjectShell::GetStyleFamilyBitmap(eFamily);
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::Insert( SfxObjectShell &rSource, USHORT nSourceIdx1, USHORT nSourceIdx2, USHORT nSourceIdx3,
							  USHORT &nIdx1, USHORT &nIdx2, USHORT &nIdx3, USHORT &nDeleted )
{

    return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::Remove( USHORT nIdx1, USHORT nIdx2, USHORT nIdx3 )
{
    return FALSE;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::SetVisArea( const Rectangle& rRect )
{
    SfxInPlaceObject::SetVisArea( Rectangle( Point(), rRect.GetSize() ) );
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::InPlaceActivate( BOOL bActivate )
{
    bIPActive = bActivate;
    SfxInPlaceObject::InPlaceActivate( bActivate );
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::FillRegInfo( SvEmbeddedRegistryInfo * pInfo )
{
    SfxInPlaceObject::FillRegInfo( pInfo );
    pInfo->aObjName = String( pStarImageDoc3, RTL_TEXTENCODING_UTF8 ); // W.P. 10.12.96: ohne 4.0 !!!
    pInfo->nMajorVers = 5;
    pInfo->nMinorVers = 0;
    pInfo->aHumanShortTypeName = String( SimResId( STR_IMAGE_DOCUMENT ) );
	pInfo->nIconResId = REG_ICON_RESID_IMAGE;
}

// -----------------------------------------------------------------------------

BOOL SimDrawDocShell::IsInformationLost()
{
    return FALSE;
}

// -----------------------------------------------------------------------------

SvDataMemberObjectRef __EXPORT SimDrawDocShell::CreateSnapshot()
{
    return SfxInPlaceObject::CreateSnapshot();
}


#ifdef WNT
#pragma optimize ( "", off )
#endif

// -----------------------------------------------------------------------------

USHORT SimDrawDocShell::DoImportExport( const String& rPath, Graphic& rGraphic, ImpExp eImpExp, String* pFilterName )
{
    GraphicFilter*  pGraphicFilter = GetGrfFilter();
	FilterProgress*	pProgress = NULL;
    USHORT          nFilter = GRFILTER_FORMAT_DONTKNOW;
    USHORT          i = GRFILTER_OPENERROR;
	BOOL			bCont = TRUE;

    if( pFilterName )
    {
        // Filternummer ermitteln
        if (eImpExp == IMPEXP_IMPORT)
            nFilter = pGraphicFilter->GetImportFormatNumber( *pFilterName );
        else
            nFilter = pGraphicFilter->GetExportFormatNumber( *pFilterName );
    }

	// Im-/Export durchfuehren
	if( eImpExp == IMPEXP_IMPORT )
	{
		if ( nFilter != GRFILTER_FORMAT_DONTKNOW && pGraphicFilter->HasImportDialog( nFilter ) )
			bCont = pGraphicFilter->DoImportDialog( NULL, nFilter );

        if( bCont )
		{
			SvFileStream aIStm( rPath, STREAM_READ | STREAM_SHARE_DENYNONE );

			pProgress = new FilterProgress( this, pGraphicFilter );
			i = pGraphicFilter->ImportGraphic( rGraphic, rPath, aIStm, nFilter );
		}
	}
    else
	{
		Graphic aExportGraphic;

		if( !bInMailExport )
		{
			if( rGraphic.IsAnimated() )
			{
				// check if selected filter supports multiframe images
				const String aShortName( pGraphicFilter->GetExportFormatShortName( nFilter ).ToLowerAscii() );

				if( ( aShortName.CompareIgnoreCaseToAscii( "tif" ) != COMPARE_EQUAL ) && 
					( aShortName.CompareIgnoreCaseToAscii( "gif" ) != COMPARE_EQUAL ) )
				{
					QueryBox aQBox( NULL, WB_YES_NO | WB_DEF_YES, 
									String( SimResId( STR_QUERY_MULTI_TO_SINGLE_IMAGE ) ) );

					if( RET_YES == aQBox.Execute() )
					{
						SimView* pView = GetActiveViewShell() ? GetActiveViewShell()->GetView() : NULL;
						aExportGraphic = pView->GetAnimBmp().aBmpEx;
					}
					else
						bCont = FALSE;
				}
			}

			if( bCont && pGraphicFilter->HasExportDialog( nFilter ) )
				bCont = pGraphicFilter->DoExportDialog( NULL, nFilter, SFX_APP()->GetOptions().GetMetric() );
		}

        if( bCont )
		{
			INetURLObject aExportURL; aExportURL.SetSmartURL( rPath );

			if( aExportGraphic.GetType() == GRAPHIC_NONE )
				aExportGraphic = rGraphic;

			pProgress = new FilterProgress( this, pGraphicFilter );
			i = XOutBitmap::ExportGraphic( aExportGraphic, aExportURL, *pGraphicFilter, nFilter, FALSE );
		}
	}

	bInMailExport = FALSE;
	delete pProgress;

	// Wurde der Dialog abgebrochen?
	if( !bCont )
	{
		SetError( ERRCODE_IO_ABORT );
		return 0;
	}

    // Fehlermeldungen
    if( i )
    {
		USHORT nId;

        switch( i )
        {
            case GRFILTER_OPENERROR:
                nId = STR_IMPEXP_GRFILTER_OPENERROR;
                break;

            case GRFILTER_IOERROR:
                nId = STR_IMPEXP_GRFILTER_IOERROR;
                break;

            case GRFILTER_FORMATERROR:
                nId = STR_IMPEXP_GRFILTER_FORMATERROR;
                break;

            case GRFILTER_VERSIONERROR:
                nId = STR_IMPEXP_GRFILTER_VERSIONERROR;
                break;

            case GRFILTER_FILTERERROR:
                nId = STR_IMPEXP_GRFILTER_FILTERERROR;
                break;

            case GRFILTER_TOOBIG:
                nId = STR_IMPEXP_GRFILTER_TOOBIG;
                break;

            default:
                nId = STR_IMPEXP_GRFILTER_FILTERERROR;
                break;
        }

		const ULONG nStmErr = pGraphicFilter->GetLastError().nStreamError;

		if( ERRCODE_NONE != nStmErr )
			ErrorHandler::HandleError( nStmErr );
		else if( STR_IMPEXP_GRFILTER_IOERROR != nId )
		{
			SimResId aTmpSimResId( nId );
			String aTmpString( aTmpSimResId );
	        ErrorBox( NULL, WB_OK, aTmpString ).Execute();
		}
		else
			ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
    }

    return i;
}

#ifdef WNT
#pragma optimize ( "", on )
#endif

// -----------------------------------------------------------------------------

void SimDrawDocShell::Connect( SimViewShell* pViewSh )
{
    pViewShell = pViewSh;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::Disconnect( SimViewShell* pViewSh )
{
    if( pViewShell == pViewSh )
        pViewShell = NULL;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::ReplaceGraphic( Graphic* pGraphic, BOOL bClearUndoMgr, BOOL bStrictAdjust )
{
	DBG_ASSERT( pGraphic, "Wo ist die Graphic?!" );

	const Size aSizePix( pGraphic->GetBitmap().GetSizePixel() );

    if( aSizePix.Width() && aSizePix.Height() )
    {
		BOOL bNewDoc = !pModel->GetFrameCount();

		pModel->SetGraphic( *pGraphic );
        pModel->SetChanged( !bNewDoc );

		if( bClearUndoMgr )
			pUndoManager->Clear();
	}
	else
		ErrorHandler::HandleError( ERRCODE_IO_OUTOFMEMORY );
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::ShowFileInfo()
{
/* !!!
	CntAnchorRef aAnchorRef = new CntAnchor( NULL, GetMedium()->GetURLObject().GetMainURL() );
	CntView::EditProperties( *(&aAnchorRef), TRUE );
*/
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::FillClass(SvGlobalName* pClassName, ULONG* pFormat, String* pAppName,
                                String* pFullTypeName, String* pShortTypeName, long nFileFormat) const
{
    SfxInPlaceObject::FillClass( pClassName, pFormat, pAppName, pFullTypeName, pShortTypeName, nFileFormat );

    if( nFileFormat == SOFFICE_FILEFORMAT_31 )
    {
        *pClassName     = SvGlobalName(SO3_SIM_CLASSID_30);
        *pAppName       = String( RTL_CONSTASCII_USTRINGPARAM( "Simage 3.1" ) );
        *pFullTypeName  = String(SimResId(STR_IMAGE_DOCUMENT_FULLTYPE_31));
        *pShortTypeName = String(SimResId(STR_IMAGE_DOCUMENT));
    }
    else if( nFileFormat == SOFFICE_FILEFORMAT_40 )
    {
        *pClassName     = SvGlobalName(SO3_SIM_CLASSID_40);
        *pFullTypeName  = String(SimResId(STR_IMAGE_DOCUMENT_FULLTYPE_40));;
        *pShortTypeName = String(SimResId(STR_IMAGE_DOCUMENT));
    }
    else if( nFileFormat == SOFFICE_FILEFORMAT_NOW )
    {
        *pFullTypeName  = String(SimResId(STR_IMAGE_DOCUMENT_FULLTYPE_50));
        *pShortTypeName = String(SimResId(STR_IMAGE_DOCUMENT));
    }
}

// -----------------------------------------------------------------------------

ErrCode SimDrawDocShell::Verb( long nVerb, SvEmbeddedClient* pCl, Window* pWin, const Rectangle* pWorkRectPixel )
{
	return SfxInPlaceObject::Verb( nVerb, pCl, pWin, pWorkRectPixel );
}

// -----------------------------------------------------------------------------

SfxFilter* SimDrawDocShell::ImplGetFilter( const String& rShortName )
{
	SfxFilter*		pRet = NULL;
	GraphicFilter*	pGrfFilter = GetGrfFilter();

	for( USHORT i = 0, nFilterCount = pGrfFilter->GetImportFormatCount(); ( i < nFilterCount ) && !pRet; i++ )
		if( pGrfFilter->GetImportFormatShortName( i ) == rShortName )
			pRet = Factory().GetFilterContainer()->GetFilter4FilterName( pGrfFilter->GetImportFormatName( i ) );

	return pRet;
}

// -----------------------------------------------------------------------------

void SimDrawDocShell::CheckFilter()
{
	SfxMedium*	pMed = GetMedium();
	SimModel*	pModel = GetModel();

	if( pModel && pMed && !HasName() )
	{
		String aShortName;

		// choose GIF in case of animation
		if( pModel->GetFrameCount() > 1 )
			aShortName = String( RTL_CONSTASCII_USTRINGPARAM( "GIF" ) );
		else
			aShortName = String( RTL_CONSTASCII_USTRINGPARAM( "BMP" ) );

		// get correct filter, if possible
		SfxFilter* pFilter = ( aShortName.Len() ? ImplGetFilter( aShortName ) : NULL );

		// set filter, if possible
		if( pFilter )
			pMed->SetFilter( pFilter );
	}
}
