/*************************************************************************
 *
 *  $RCSfile: fugeo.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_COLOR 0

#include <math.h>
#include <vcl/poly.hxx>
#include <vcl/virdev.hxx>
#include <svx/colritem.hxx>
#include <svx/xoutx.hxx>
#include <svx/xlineit0.hxx>
#include <svx/xlnasit.hxx>
#include <svx/xlnclit.hxx>
#include <svx/xlnwtit.hxx>
#include <svx/xflasit.hxx>
#include <svx/xfillit0.hxx>
#include <svx/xflclit.hxx>
#include "simapp.hrc"
#include "glob.hxx"
#include "simattr.hxx"
#include "simmodel.hxx"
#include "simview.hxx"
#include "simwin.hxx"
#include "drviewsh.hxx"
#include "svimage.hxx"
#include "simresid.hxx"
#include "strings.hrc"
#include "fugeo.hxx"

// ------------
// - SimFuGeo -
// ------------

SimFuGeo::SimFuGeo( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
				    SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuSelBase( pViewSh, pWin, pView, pSimModel, rReq )
{
}

// -----------------------------------------------------------------------------

SimFuGeo::~SimFuGeo()
{
}

// -----------------------------------------------------------------------------

GDIMetaFile SimFuGeo::ImplCreateFillMetaFile( const PolyPolygon& rSelection )
{
	GDIMetaFile			aMtf;
	VirtualDevice		aVDev;
	Rectangle			aBound( rSelection[ 0 ].GetBoundRect() );
	Color*				pFillCol = IsLeftMouse() ? &aBackCol : &aForeCol;

	aVDev.EnableOutput( FALSE );
	aMtf.Record( &aVDev );
	
	aVDev.SetLineColor( *pFillCol );
	aVDev.SetFillColor( *pFillCol );

	switch( GetSlotID() )
	{
		case SID_PAINTTOOL_RECT:
		case SID_PAINTTOOL_CIRCLE:
			aVDev.DrawPolygon( rSelection[ 0 ] );
		break;

		default:
		break;
	}

	aMtf.Stop();
	aMtf.WindStart();
	aMtf.SetPrefSize( pView->GetSizePixel() );
	aMtf.SetPrefMapMode( MAP_PIXEL );

	return aMtf;
}

// -----------------------------------------------------------------------------

GDIMetaFile SimFuGeo::ImplCreateLineMetaFile( const PolyPolygon& rSelection )
{
	GDIMetaFile			aMtf;
	VirtualDevice		aVDev;
	Rectangle			aBound( rSelection[ 0 ].GetBoundRect() );
	long				nOff = ( nLineWidth > 1 ) ? ( nLineWidth - 1 ): 0;
	const long			nOffH = Min( nOff, ( aBound.GetWidth() - 1L ) >> 1L );
	const long			nOffV = Min( nOff, ( aBound.GetHeight() - 1L ) >> 1L );
	Color*				pLineCol = IsLeftMouse() ? &aForeCol : &aBackCol;

	aVDev.EnableOutput( FALSE );
	aMtf.Record( &aVDev );
	
	aVDev.SetLineColor( *pLineCol );
	aVDev.SetFillColor( *pLineCol );

	switch( GetSlotID() )
	{
		case SID_PAINTTOOL_LINE:
		{
			const Polygon& rLine = rSelection[ 0 ];

			if( nLineWidth > 1 )
			{
				const long		nWidth2 = nLineWidth >> 1;
				Polygon			aPoly( 2 );
				Polygon			aLine( 4 );
				const long		nDX = rLine[ 1 ].X() - rLine[ 0 ].X();
				const long		nDY = rLine[ 1 ].Y() - rLine[ 0 ].Y();
				const double	fAngle = atan2( (double) -nDY, nDX ) / F_PI1800;

				aPoly[ 0 ] = Point( 0, -nWidth2 ); 
				aPoly[ 1 ] = Point( 0, nWidth2 - ( ( nLineWidth & 1 ) ? 0 : 1 ) );
				aPoly.Rotate( Point(), (USHORT) ( ( fAngle < 0.0 ) ? FRound( 3600.0 + fAngle ) : FRound( fAngle ) ) );

				aPoly.Move( rLine[ 0 ].X(), rLine[ 0 ].Y() );
				aLine[ 0 ] = aPoly[ 0 ]; aLine[ 1 ] = aPoly[ 1 ];

				aPoly.Move( nDX, nDY );
				aLine[ 2 ] = aPoly[ 1 ]; aLine[ 3 ] = aPoly[ 0 ];

				aVDev.DrawPolygon( aLine );
			}
			else
				aVDev.DrawLine( rLine[ 0 ], rLine[ 1 ] );
		}
		break;

		case SID_PAINTTOOL_RECT:
		case SID_PAINTTOOL_RECT_NULL:
		{
			if( nOffH || nOffV )
			{
				PolyPolygon aLine;

				aBound.Left() += nOffH; aBound.Top() += nOffV;
				aBound.Right() -= nOffH; aBound.Bottom() -= nOffV;

				aLine.Insert( rSelection[ 0 ] );

				if( nRadius )
					aLine.Insert( Polygon( aBound, nRadius, nRadius ) );
				else
					aLine.Insert( Polygon( aBound ) );

				aVDev.DrawPolyPolygon( aLine );
			}
			else
				aVDev.DrawPolyLine( rSelection[ 0 ] );
		}
		break;

		case SID_PAINTTOOL_CIRCLE:
		case SID_PAINTTOOL_CIRCLE_NULL:
		{
			PolyPolygon aLine;

			if( nOffH || nOffV )
			{
				aBound.Left() += nOffH, aBound.Top() += nOffV;
				aBound.Right() -= nOffH, aBound.Bottom() -= nOffV;

				aLine.Insert( rSelection[ 0 ] ); 
				aLine.Insert( CreateEllipse( aBound ) );
				aVDev.DrawPolyPolygon( aLine );
			}
			else
				aVDev.DrawPolyLine( rSelection[ 0 ] );
		}
		break;

		default:
		break;
	}

	aMtf.Stop();
	aMtf.WindStart();
	aMtf.SetPrefSize( pView->GetSizePixel() );
	aMtf.SetPrefMapMode( MAP_PIXEL );

	return aMtf;
}

// -----------------------------------------------------------------------------

PolyPolygon SimFuGeo::GetSelection() const
{
	PolyPolygon			aPolyPoly;
	const Rectangle&	rRect = GetActImageRect();

	if( !rRect.IsEmpty() )
	{
		switch( GetSlotID() )
		{
			case SID_PAINTTOOL_LINE:
			{
				Polygon	aLinePoly( 2 );

				if( GetStartPos().X() < GetActPos().X() )
				{
					if( GetStartPos().Y() < GetActPos().Y() )
						aLinePoly[ 0 ] = rRect.TopLeft(), aLinePoly[ 1 ] = rRect.BottomRight();
					else
						aLinePoly[ 0 ] = rRect.BottomLeft(), aLinePoly[ 1 ] = rRect.TopRight();
				}
				else
				{
					if( GetStartPos().Y() < GetActPos().Y() )
						aLinePoly[ 0 ] = rRect.TopRight(), aLinePoly[ 1 ] = rRect.BottomLeft();
					else
						aLinePoly[ 0 ] = rRect.BottomRight(), aLinePoly[ 1 ] = rRect.TopLeft();
				}

				aPolyPoly.Insert( aLinePoly );
			}
			break;

			case SID_PAINTTOOL_RECT:
			case SID_PAINTTOOL_RECT_NULL:
			{
				if( nRadius )
					aPolyPoly.Insert( Polygon( rRect, nRadius, nRadius ) );
				else
					aPolyPoly.Insert( Polygon( rRect ) );
			}
			break;

			case SID_PAINTTOOL_CIRCLE:
			case SID_PAINTTOOL_CIRCLE_NULL:
			{
				aPolyPoly.Insert( CreateEllipse( GetActRect() ) );
			}
			break;

			default:
			break;
		}
	}

	return aPolyPoly;
}

// -----------------------------------------------------------------------------

BOOL SimFuGeo::InitSelection( BOOL& rIntercepted )
{
	const SfxItemSet&	rAttrSet = pView->GetAttrSet();
	BOOL				bRet;

	aForeCol = ( (const SvxColorItem&) rAttrSet.Get( ATTR_PNT_COLOR_FORE ) ).GetValue();
	aBackCol = ( (const SvxColorItem&) rAttrSet.Get( ATTR_PNT_COLOR_BACK ) ).GetValue();
	nLineWidth = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_LINE_WIDTH ) ).GetValue();
	nRadius = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_LINE_ROUND ) ).GetValue();
	nLineTransparency = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_LINE_TRANSPARENCY ) ).GetValue();
	nFillTransparency = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_FILL_TRANSPARENCY ) ).GetValue();
	bFill = ( GetSlotID() != SID_PAINTTOOL_LINE ) &&
			( GetSlotID() != SID_PAINTTOOL_RECT_NULL ) && 
			( GetSlotID() != SID_PAINTTOOL_CIRCLE_NULL );

	const USHORT nLineTrans = IsLeftMouse() ? nLineTransparency : nFillTransparency;
	const USHORT nFillTrans = IsLeftMouse() ? nFillTransparency : nLineTransparency;

	if( nLineTrans || ( bFill && nFillTrans ) )
		bRet = ChangeImageDepth( 24, TRUE, rIntercepted );
	else
	{
		BitmapPalette aPal( bFill ? 2 : 1 );

		aPal[ 0 ] = IsLeftMouse() ? aForeCol : aBackCol;

		if( bFill )
			aPal[ 1 ] = IsLeftMouse() ? aBackCol : aForeCol;

		bRet = AddImageColors( aPal, TRUE, rIntercepted );
	}

	return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimFuGeo::EndSelection()
{
	const Rectangle& rRect = GetActImageRect();

	SimFuSelBase::EndSelection();
	
	if( !rRect.IsEmpty() )
	{
	    const PolyPolygon	aSelection( GetSelection() );
		Rectangle			aBound( aSelection.GetBoundRect() );
		SvImage*			pImage = pView->GetImagePointer();
	    SvImage*			pAlpha = pView->GetAlphaPointer();
		GDIMetaFile			aLineMtf( ImplCreateLineMetaFile( aSelection ) );
		GDIMetaFile			aFillMtf( ImplCreateFillMetaFile( aSelection ) );
		const USHORT		nLineTrans = IsLeftMouse() ? nLineTransparency : nFillTransparency;
		const USHORT		nFillTrans = IsLeftMouse() ? nFillTransparency : nLineTransparency;

		if( pImage && pImage->BeginProcessing() )
		{
			HMEMCPY( pImage->GetDestPointer(), pImage->GetOrgPointer(), pImage->GetSrcWidthAl() * pImage->GetSrcHeight() );

			if( bFill )
				pImage->PaintMetaFileInProcessing( aFillMtf, (BYTE) FRound( nFillTrans * 2.55 ) );

			pImage->PaintMetaFileInProcessing( aLineMtf, (BYTE) FRound( nLineTrans * 2.55 ) );
			pImage->EndProcessing();
		}

		if( pAlpha && pAlpha->BeginProcessing() )
		{	
			HMEMCPY( pAlpha->GetDestPointer(), pAlpha->GetOrgPointer(), pAlpha->GetSrcWidthAl() * pAlpha->GetSrcHeight() );

			if( bFill )
				pAlpha->PaintMetaFileInProcessing( aFillMtf.GetMonochromeMtf( Color( COL_BLACK ) ) );

			pAlpha->PaintMetaFileInProcessing( aLineMtf.GetMonochromeMtf( Color( COL_BLACK ) ) );
			pAlpha->EndProcessing();
		}

		aBound.Left()--, aBound.Top()--, aBound.Right()++, aBound.Bottom()++;
		pModel->SendRepaintBroadcast( pView->GetCurFramePos(), aBound, FALSE );
		pModel->SetChanged( TRUE );
		pModel->IncModificationCount();
		( (SimDrawViewShell*) pViewShell )->CreateUndoAction( String( SimResId( STR_PAINTING ) ), SID_SIM_PAINTING );
	}

	return TRUE;
}

// -----------------------------------------------------------------------------

Pointer SimFuGeo::GetPointer() const
{
	USHORT nId;

	switch( GetSlotID() )
	{
		case SID_PAINTTOOL_LINE: 
			nId = POINTER_DRAW_LINE; 
		break;
		
		case SID_PAINTTOOL_RECT:
		case SID_PAINTTOOL_RECT_NULL:
			nId = POINTER_DRAW_RECT; 
		break;
		
		case SID_PAINTTOOL_CIRCLE:
		case SID_PAINTTOOL_CIRCLE_NULL:
			nId = POINTER_DRAW_ELLIPSE; 
		break;

		default:
			nId = POINTER_ARROW;
		break;
	}

	return Pointer( nId );
}
