/*************************************************************************
 *
 *  $RCSfile: fumod.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <vcl/wrkwin.hxx>
#include <sfx2/viewfrm.hxx>	
#include "simattr.hxx"
#include "glob.hxx"
#include "viewshel.hxx"
#include "simview.hxx"
#include "svimage.hxx"
#include "agedlg.hxx"
#include "contrast.hxx"
#include "gamma.hxx"
#include "graydlg.hxx"
#include "mosaic.hxx"
#include "poster.hxx"
#include "relief.hxx"
#include "rgbdlg.hxx"
#include "rotate.hxx"
#include "sharpnss.hxx"
#include "sizedlg.hxx"
#include "solardlg.hxx"
#include "tiledlg.hxx"
#include "simframe.hxx"
#include "simmodel.hxx"
#include "simwin.hxx"
#include "fumod.hxx"

// -----------
// - Defines -
// -----------

#define PARENTWIN() (&(pViewSh->GetViewFrame()->GetWindow()))

// ------------
// - SimFuAge -
// ------------

IMPL_FUMOD( Age )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
        SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_AGE_START, ATTR_AGE_END );
		
		aNewAttr.Put( SfxInt16Item( ATTR_AGE_VALUE, 10 ) );
        
		SimAgeDlg* pDlg = new SimAgeDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }
    
	const INT16			nValue = ( ( const SfxInt16Item& ) pArgs->Get( ATTR_AGE_VALUE ) ).GetValue();
    SvImage*			pImage = pView->GetImagePointer();
    const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
	BOOL				bNoErr;

    pWindow->EnterWait();

	if ( aMarkRect.IsEmpty() )
		bNoErr = pImage->Sepia( nValue, pViewSh->GetViewFrame() );
	else
		bNoErr = pImage->Sepia( nValue, aMarkRect, pViewSh->GetViewFrame() );

    pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// -----------------
// - SimFuContrast -
// -----------------

IMPL_FUMOD( Contrast )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_BRI_CON_START, ATTR_BRI_CON_END );
		
		aNewAttr.Put( SfxInt16Item( ATTR_BRI_CON_BRIGHT, 0 ) );
        aNewAttr.Put( SfxInt16Item( ATTR_BRI_CON_CONTRAST, 0 ) );
        
		SimContrastDlg* pDlg = new SimContrastDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }
    
	INT16 nContrast	= ( (const SfxInt16Item&) pArgs->Get( ATTR_BRI_CON_BRIGHT ) ).GetValue();
	INT16 nBrightness = ( (const SfxInt16Item&) pArgs->Get( ATTR_BRI_CON_CONTRAST ) ).GetValue();

	if( !nContrast && !nBrightness )
		bDlgRet = FALSE;
	else
	{
		BOOL bNoErr;

		pWindow->EnterWait();
		bNoErr = pView->GetImagePointer()->AdjustLu_Co( nBrightness, nContrast, pView->GetSelectionBoundRect(), pViewSh->GetViewFrame() );
	    pWindow->LeaveWait();

		if( !bNoErr )
			pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
	}
	
}

// --------------
// - SimFuGamma -
// --------------

IMPL_FUMOD( Gamma )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
        SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_GAMMA_START, ATTR_GAMMA_END );

        aNewAttr.Put( SfxUInt32Item( ATTR_GAMMA_VALUE, 100 ) );
        
		SimGammaDlg* pDlg = new SimGammaDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

    const UINT32 nValue = ( (const SfxUInt32Item&) pArgs->Get( ATTR_GAMMA_VALUE ) ).GetValue();

	if( nValue == 100 )
		bDlgRet = FALSE;
	else
	{
		BOOL bNoErr;

	    pWindow->EnterWait();
	    bNoErr = pView->GetImagePointer()->GammaCorrection( nValue / 100.F, pView->GetSelectionBoundRect(), pViewSh->GetViewFrame() );
	    pWindow->LeaveWait();

		if( !bNoErr )
			pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
	}
}

// -------------
// - SimFuGray -
// -------------

IMPL_FUMOD( Gray )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_GRAYSCALES_START, ATTR_GRAYSCALES_END );

		aNewAttr.Put( SfxAllEnumItem( ATTR_GRAYSCALES_COUNT, GRAYCOUNT_256 ) );
        aNewAttr.Put( SfxInt16Item( ATTR_GRAYSCALES_THRES, 50 ) );
        
		SimGrayDlg* pDlg = new SimGrayDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }
	
	const GRAYCOUNT 	nCount = (GRAYCOUNT)((const SfxAllEnumItem&)pArgs->Get(ATTR_GRAYSCALES_COUNT)).GetValue();
	const INT16 		nThres = ((const SfxInt16Item&)pArgs->Get(ATTR_GRAYSCALES_THRES)).GetValue();
	SvImage*			pImage = pView->GetImagePointer();
	const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
	BOOL				bNoErr;

	pWindow->EnterWait();

	if( aMarkRect.IsEmpty() )
	{
		if( nCount == GRAYCOUNT_2 )
			bNoErr = pImage->Monochrome( nThres, FALSE, pViewSh->GetViewFrame() );
		else
			bNoErr = pImage->Grayscale(	nCount == GRAYCOUNT_256 ? 256 :
								nCount == GRAYCOUNT_128 ? 128 :
								nCount == GRAYCOUNT_64  ? 64  :
								nCount == GRAYCOUNT_32  ? 32  :
								nCount == GRAYCOUNT_16  ? 16  :
								nCount == GRAYCOUNT_8   ? 8   :
								nCount == GRAYCOUNT_4   ? 4   : 256,
								pViewSh->GetViewFrame() );
	}
	else
	{
		if( nCount == GRAYCOUNT_2 )
			bNoErr = pImage->Monochrome( nThres, FALSE, aMarkRect, pViewSh->GetViewFrame() );
		else
			bNoErr = pImage->Grayscale( nCount == GRAYCOUNT_256 ? 256 :
							   nCount == GRAYCOUNT_128 ? 128 :
							   nCount == GRAYCOUNT_64  ? 64  :
							   nCount == GRAYCOUNT_32  ? 32  :
							   nCount == GRAYCOUNT_16  ? 16  :
							   nCount == GRAYCOUNT_8   ? 8   :
							   nCount == GRAYCOUNT_4   ? 4   : 256,
							   aMarkRect, pViewSh->GetViewFrame() );
	}

    pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// ---------------
// - SimFuMosaic -
// ---------------

IMPL_FUMOD( Mosaic )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_TILE_START, ATTR_TILE_END );

		aNewAttr.Put( SfxUInt16Item( ATTR_TILE_X, 4 ) );
		aNewAttr.Put( SfxUInt16Item( ATTR_TILE_Y, 4 ) );
		aNewAttr.Put( SfxAllEnumItem( ATTR_TILE_SHARPNESS, DEFINITION_MEDIUM ) );
        
		SimMosaicDlg* pDlg = new SimMosaicDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const DefinitionType 	nType = (DefinitionType) ( (const SfxAllEnumItem&) pArgs->Get( ATTR_TILE_SHARPNESS ) ).GetValue();
	const UINT16 			nX = ( (const SfxUInt16Item&) pArgs->Get( ATTR_TILE_X ) ).GetValue();
	const UINT16 			nY = ( (const SfxUInt16Item&) pArgs->Get( ATTR_TILE_Y ) ).GetValue();
	BOOL					bNoErr;


	pWindow->EnterWait();
	bNoErr = pView->GetImagePointer()->Mosaic( Size( nX, nY ), pView->GetSelectionBoundRect(), pViewSh->GetViewFrame() );
    pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// ---------------
// - SimFuPoster -
// ---------------

IMPL_FUMOD( Poster )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
        SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_POSTER_START, ATTR_POSTER_END );

        aNewAttr.Put( SfxUInt16Item( ATTR_POSTER_COLORS, 64 ) );
        
		SimPosterDlg* pDlg = new SimPosterDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

    const UINT16	nColors = ( (const SfxUInt16Item&) pArgs->Get( ATTR_POSTER_COLORS ) ).GetValue();
	BOOL			bNoErr;

	pWindow->EnterWait();
    bNoErr = pView->GetImagePointer()->Poster( nColors, pView->GetSelectionBoundRect(), pViewSh->GetViewFrame() );
	pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// ---------------
// - SimFuRelief -
// ---------------

IMPL_FUMOD( Relief )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_RELIEF_START, ATTR_RELIEF_END );

		aNewAttr.Put( SfxAllEnumItem( ATTR_RELIEF_SOURCE, RP_LT ) );

		SimReliefDlg* pDlg = new SimReliefDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
				rReq.Done( *( pDlg->GetOutputItemSet() ) );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const RECT_POINT	nSource	= (RECT_POINT) ( (const SfxAllEnumItem&) pArgs->Get( ATTR_RELIEF_SOURCE ) ).GetValue();
	float				fAzim = 0.F, fElev = 0.F;
	BOOL				bNoErr;

	switch (nSource)
	{
		case( RP_LT ): fAzim = 45.F, fElev = 45.F; break;
		case( RP_MT ): fAzim = 90.F, fElev = 45.F; break;
		case( RP_RT ): fAzim = 135.F, fElev = 45.F; break;
		case( RP_LM ): fAzim = 0.F, fElev = 45.F; break;
		case( RP_MM ): fAzim = 0.F, fElev = 90.F; break;
		case( RP_RM ): fAzim = 180.F, fElev = 45.F; break;
		case( RP_LB ): fAzim = 315.F, fElev = 45.F; break;
		case( RP_MB ): fAzim = 270.F, fElev = 45.F; break;
		case( RP_RB ): fAzim = 225.F, fElev = 45.F; break;
	}

	pWindow->EnterWait();
	bNoErr = pView->GetImagePointer()->Emboss( fAzim, fElev, 4, pView->GetSelectionBoundRect(), pViewSh->GetViewFrame() );
	pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// ---------------
// - SimFuResize -
// ---------------

IMPL_FUMOD( Resize )
{
    const SfxItemSet*	pArgs = rReq.GetArgs();
	const Size			aSizePix( pView->GetSizePixel() );
	Size				aSizeLog;

	if( pView->GetPrefMapMode().GetMapUnit() == MAP_PIXEL )
		aSizeLog = pWindow->PixelToLogic( aSizePix, MAP_100TH_MM );
	else
		aSizeLog = OutputDevice::LogicToLogic( pView->GetPrefSize(), pView->GetPrefMapMode(), MAP_100TH_MM );

    if( !pArgs )
    {
		SfxItemSet	aNewAttr( *pModel->GetPool(), ATTR_SIZE_START, ATTR_SIZE_END );

		// Setzen der Initialisierungs-Werte fuer Relief
		aNewAttr.Put( SfxUInt32Item( ATTR_SIZE_PIXEL_X, aSizePix.Width() ) );
		aNewAttr.Put( SfxUInt32Item( ATTR_SIZE_PIXEL_Y, aSizePix.Height() ) );
		aNewAttr.Put( SfxUInt32Item( ATTR_SIZE_MM_X, aSizeLog.Width() ) );
		aNewAttr.Put( SfxUInt32Item( ATTR_SIZE_MM_Y, aSizeLog.Height() ) );
		aNewAttr.Put( SfxUInt16Item( ATTR_SIZE_DEPTH, pView->GetColorDepth() ) );
        
		SimSizeDlg* pDlg = new SimSizeDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const float	fNewWidth = (float) ( (const SfxUInt32Item&) pArgs->Get( ATTR_SIZE_PIXEL_X ) ).GetValue();
	const float	fNewHeight = (float) ( (const SfxUInt32Item&) pArgs->Get( ATTR_SIZE_PIXEL_Y ) ).GetValue();
	const BOOL	bInterpol = (BOOL) ( (const SfxBoolItem&) pArgs->Get( ATTR_SIZE_INTERPOL ) ).GetValue();
	const float fScaleX = fNewWidth / (float) aSizePix.Width() * 100.F;
	const float fScaleY = fNewHeight / (float) aSizePix.Height() * 100.F;
	SvImage*	pAlpha = pView->GetAlphaPointer();
	BOOL		bNoErr;

	pWindow->EnterWait();

	bNoErr = pView->GetImagePointer()->Scale( fScaleX, fScaleY, bInterpol, pViewSh->GetViewFrame() );
	
	if( bNoErr && pAlpha )
		pAlpha->Scale( fScaleX, fScaleY, FALSE, pViewSh->GetViewFrame() );

	if( bNoErr && bInterpol )		
		pView->GetImagePointer()->SetDepth( 24 );

    pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// ------------
// - SimFuRgb -
// ------------

IMPL_FUMOD( Rgb )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_RGB_START, ATTR_RGB_END );

		aNewAttr.Put( SfxInt16Item( ATTR_RGB_RED, 0 ) );
        aNewAttr.Put( SfxInt16Item( ATTR_RGB_GREEN, 0 ) );
		aNewAttr.Put( SfxInt16Item( ATTR_RGB_BLUE, 0 ) );
        
		SimRgbDlg* pDlg = new SimRgbDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const INT16 nRed = ( (const SfxInt16Item&) pArgs->Get( ATTR_RGB_RED ) ).GetValue();
	const INT16 nGreen = ( (const SfxInt16Item&) pArgs->Get( ATTR_RGB_GREEN ) ).GetValue();
	const INT16 nBlue = ( (const SfxInt16Item&) pArgs->Get( ATTR_RGB_BLUE ) ).GetValue();

	if( !nRed && !nGreen && !nBlue )
		bDlgRet = FALSE;
	else
	{
		SvImage*			pImage = pView->GetImagePointer();
		const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
		BOOL				bNoErr;

		pWindow->EnterWait();

		if( aMarkRect.IsEmpty() )
			bNoErr = pImage->AdjustRGB( nRed, nGreen, nBlue, pViewSh->GetViewFrame() );
		else
			bNoErr = pImage->AdjustRGB( nRed, nGreen, nBlue, aMarkRect, pViewSh->GetViewFrame() );

	    pWindow->LeaveWait();

		if( !bNoErr )
			pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
	}
}

// ---------------
// - SimFuRotate -
// ---------------

IMPL_FUMOD( Rotate )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_ROTATE_ANGLE, ATTR_ROTATE_ORIENTATION );

		aNewAttr.Put( SfxUInt32Item( ATTR_ROTATE_ANGLE, 0 ) );
		aNewAttr.Put( SfxAllEnumItem( ATTR_ROTATE_ORIENTATION, LEFT ) );
        
		SimRotateDlg* pDlg = new SimRotateDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
				rReq.Done( *( pDlg->GetOutputItemSet() ) );
				pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const UINT32 		nAngle 	= ( (const SfxUInt32Item&) pArgs->Get( ATTR_ROTATE_ANGLE ) ).GetValue();
	const ORIENTATION	eOrient	= (ORIENTATION) ( (const SfxAllEnumItem&) pArgs->Get( ATTR_ROTATE_ORIENTATION ) ).GetValue();

	// muessen wir ueberhaupt etwas machen
	if( !nAngle )
		bDlgRet = FALSE;
	else
	{
		const float	fAngle = ( ( eOrient == RIGHT) ? -0.01F : 0.01F ) * nAngle;
		SvImage*	pAlpha = pView->GetAlphaPointer();
		BOOL		bNoErr;

		pWindow->EnterWait();

		bNoErr = pView->GetImagePointer()->Rotate( fAngle, Color( COL_WHITE ), pViewSh->GetViewFrame() );

		if( bNoErr && pAlpha )
			pAlpha->Rotate( fAngle, Color( COL_WHITE ), pViewSh->GetViewFrame());

	    pWindow->LeaveWait();

		if( !bNoErr )
			pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
	}
}

// --------------
// - SimFuSharp -
// --------------

IMPL_FUMOD( Sharp )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_SHARPNESS_START, ATTR_SHARPNESS_END );

		aNewAttr.Put( SfxAllEnumItem( ATTR_SHARPNESS_TYPE, DEFINITION_MEDIUM ) );

		SimSharpnessDlg* pDlg = new SimSharpnessDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const DefinitionType	eType = (DefinitionType) ( (const SfxAllEnumItem&) pArgs->Get( ATTR_SHARPNESS_TYPE ) ).GetValue();
	const BYTE				cIntens = ( eType == DEFINITION_LOW ) ? 23 : ( eType == DEFINITION_MEDIUM ) ? 13 : 9;
	BOOL					bNoErr;

	pWindow->EnterWait();
	bNoErr = pView->GetImagePointer()->Sharpen( cIntens, pView->GetSelectionBoundRect(), pViewSh->GetViewFrame() );
    pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// --------------
// - SimFuSolar -
// --------------

IMPL_FUMOD( Solar )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_SOLAR_START, ATTR_SOLAR_END );

		aNewAttr.Put( SfxInt16Item( ATTR_SOLAR_THRES, 50 ) );
		aNewAttr.Put( SfxBoolItem( ATTR_SOLAR_INVERT, FALSE ) );

		SimSolarDlg* pDlg = new SimSolarDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const INT16			nThres = ((const SfxInt16Item&)pArgs->Get(ATTR_SOLAR_THRES)).GetValue();
	const BOOL			bInvert	= ((const SfxBoolItem&)pArgs->Get(ATTR_SOLAR_INVERT)).GetValue();
	SvImage*			pImage = pView->GetImagePointer();
	const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
	BOOL				bNoErr;

	pWindow->EnterWait();

	if( aMarkRect.IsEmpty() )
		bNoErr = pImage->Solarize( nThres, !bInvert, pViewSh->GetViewFrame() );
	else
		bNoErr = pImage->Solarize( nThres, !bInvert, aMarkRect, pViewSh->GetViewFrame() );

    pWindow->LeaveWait();

	if( !bNoErr )
		pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
}

// -------------
// - SimFuTile -
// -------------

IMPL_FUMOD( Tile )
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    if( !pArgs )
    {
		SfxItemSet aNewAttr( *pModel->GetPool(), ATTR_TILE_START, ATTR_TILE_END );

		aNewAttr.Put( SfxUInt16Item( ATTR_TILE_X, 4 ) );
		aNewAttr.Put( SfxUInt16Item( ATTR_TILE_Y, 4 ) );

		SimTileDlg* pDlg = new SimTileDlg( PARENTWIN(), aNewAttr );

        switch( pDlg->Execute() )
        {
            case RET_OK:
            {
                pDlg->GetAttr( aNewAttr );
                rReq.Done( aNewAttr );
                pArgs = rReq.GetArgs();
                bDlgRet = TRUE;
                delete pDlg;
            }
            break;

            default:
            {
                bDlgRet = FALSE;
                delete pDlg;
	            return;
            }
        }
    }

	const UINT16 nX = ( (const SfxUInt16Item&) pArgs->Get( ATTR_TILE_X ) ).GetValue();
	const UINT16 nY = ( (const SfxUInt16Item&) pArgs->Get( ATTR_TILE_Y ) ).GetValue();

	if( ( nX == 1 ) && ( nY == 1 ) )
		bDlgRet = FALSE;
	else
	{
		const Size			aTileSize( nX, nY );
		SvImage*			pImage = pView->GetImagePointer();
		const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
		BOOL				bNoErr;

		pWindow->EnterWait();

		if( aMarkRect.IsEmpty() )
			bNoErr = pImage->Pixellize( aTileSize, pViewSh->GetViewFrame() );
		else
			bNoErr = pImage->Pixellize( aTileSize, aMarkRect, pViewSh->GetViewFrame() );

	    pWindow->LeaveWait();

		if( !bNoErr )
			pViewShell->HandleSimError( SIM_ERROR_OUTOFMEMORY );
	}
}
