/*************************************************************************
 *
 *  $RCSfile: fupaint.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_COLOR 0

#include <math.h>
#include <stdlib.h>
#include <tools/time.hxx>
#include <vcl/line.hxx>
#include <vcl/bmpacc.hxx>
#include <vcl/virdev.hxx>
#include <svx/colritem.hxx>
#include "simapp.hrc"
#include "glob.hxx"
#include "simattr.hxx"
#include "simmodel.hxx"
#include "simview.hxx"
#include "simwin.hxx"
#include "drviewsh.hxx"
#include "svimage.hxx"
#include "simresid.hxx"
#include "strings.hrc"
#include "fupaint.hxx"

// ------------------
// - SimFuPaintBase -
// ------------------

SimFuPaintBase::SimFuPaintBase( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
								SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuSelBase	( pViewSh, pWin, pView, pSimModel, rReq ),
		aGrayPal		( 256 ),
		pImage			( NULL ),
		pAlpha			( NULL ),
		pRegionBmp		( NULL ),
		bEnumLineSet	( FALSE ),
		bIsInLineEnum	( FALSE ),
		bTimer			( FALSE )
{
	BitmapColor	aCol;

	for( USHORT i = 0; i < 256; i++ )
	{
		aCol.SetRed( (BYTE) i );
		aCol.SetGreen( (BYTE) i );
		aCol.SetBlue( (BYTE) i );
		aGrayPal[ i ] = aCol;
	} 

	aFillerTimer.SetTimeoutHdl( LINK( this, SimFuPaintBase, FillerTimeoutHdl ) );
	aFillerTimer.SetTimeout( 20 );
}

// -----------------------------------------------------------------------------

SimFuPaintBase::~SimFuPaintBase()
{
	delete pRegionBmp;
}

// -----------------------------------------------------------------------------

void SimFuPaintBase::PaintFiller( const Point& rPosPixel )
{
	if( FillerDiffers() )
	{
		aFiller = GetFiller();

		if( pAlpha )
		{
			Bitmap aWhite( aFiller.GetSizePixel(), 1 );
			aWhite.Erase( COL_WHITE );
			aAlphaFiller = BitmapEx( aWhite, aFiller.GetMask() );
		}
	}

	if( pImage )
	{
		const Size& rFillSize = aFiller.GetSizePixel();
		Point		aFillPos( rPosPixel.X() - ( rFillSize.Width() >> 1 ),
							  rPosPixel.Y() - ( rFillSize.Height() >> 1 ) );
		Rectangle	aFillRect( aFillPos, rFillSize );

		aFillRect.Intersection( pView->GetRectPixel() );

		if( !aFillRect.IsEmpty() )
		{
			if( !!aFiller.GetMask() )
			{
				pImage->PaintInProcessing( aFillPos.Y(), aFillPos.X(), 
										   aFiller.GetBitmap(), aFiller.GetMask(),
										   pRegionBmp );

				if( pAlpha )
				{
					pAlpha->PaintInProcessing( aFillPos.Y(), aFillPos.X(), 
											   aAlphaFiller.GetBitmap(), aAlphaFiller.GetMask(),
											   pRegionBmp );
				}
			}
			else
			{
				pImage->PaintInProcessing( aFillPos.Y(), aFillPos.X(), 
										   aFiller.GetTransparentColor(), aFiller.GetBitmap(),
										   pRegionBmp );

				if( pAlpha )
				{
					pAlpha->PaintInProcessing( aFillPos.Y(), aFillPos.X(), 
											   Color( COL_BLACK ), aAlphaFiller.GetBitmap(),
											   pRegionBmp );
				}
			}
		}
	}
}

// -----------------------------------------------------------------------------

BitmapEx SimFuPaintBase::GetFiller() const
{
	return BitmapEx();
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::FillerDiffers() const
{
	return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::ShouldUseRegion() const
{
	return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::ShouldUseTimer() const
{
	return FALSE;
}

// -----------------------------------------------------------------------------

Rectangle SimFuPaintBase::GetPaintRect() const
{
	Rectangle	aInvRect( GetLastPos(), GetActPos() );
	const long	nExt = ( GetPaintExt() >> 1 ) + 2;

	aInvRect.Justify();
	aInvRect.Left() -= nExt;
	aInvRect.Top() -= nExt;
	aInvRect.Right() += nExt;
	aInvRect.Bottom() += nExt;

	return aInvRect;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::InitSelection( BOOL& rIntercepted )
{
	const SfxItemSet& rAttrSet = pView->GetAttrSet();
	
	aForeCol = ( (const SvxColorItem&) rAttrSet.Get( ATTR_PNT_COLOR_FORE ) ).GetValue();
	aBackCol = ( (const SvxColorItem&) rAttrSet.Get( ATTR_PNT_COLOR_BACK ) ).GetValue();
	nFillWidth = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_PEN_WIDTH ) ).GetValue();
	nLineTransparency = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_LINE_TRANSPARENCY ) ).GetValue();
	nFillTransparency = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_FILL_TRANSPARENCY ) ).GetValue();

	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::BeginSelection()
{
	SimFuSelBase::BeginSelection();

	pImage = pView->GetImagePointer();

	if( pImage && pImage->BeginProcessing() )
	{
		HMEMCPY( pImage->GetDestPointer(), pImage->GetOrgPointer(), pImage->GetSrcWidthAl() * pImage->GetSrcHeight() );
		aFiller = GetFiller();
		pAlpha = pView->GetAlphaPointer();

		if( pAlpha && pAlpha->BeginProcessing() )
		{
			Bitmap aWhite( aFiller.GetSizePixel(), 1 );

			HMEMCPY( pAlpha->GetDestPointer(), pAlpha->GetOrgPointer(), pAlpha->GetSrcWidthAl() * pAlpha->GetSrcHeight() );
			aWhite.Erase( COL_WHITE );
			aAlphaFiller = BitmapEx( aWhite, aFiller.GetMask() );
		}
		else
		{
			pAlpha = NULL;
			aAlphaFiller = BitmapEx();
		}
	}
	else
	{
		pImage = NULL;
		aFiller = BitmapEx();
	}

	if( ShouldUseRegion() )
	{
		pRegionBmp = new Bitmap( pView->GetSizePixel(), 1 );
		pRegionBmp->Erase( Color( COL_WHITE ) );
	}

	if( ShouldUseTimer() )
	{
		bTimer = TRUE;
		aDragTimer.Stop();
		aFillerTimer.Start();
	}

	PaintFiller( GetStartPos() );
	pView->Invalidate( GetPaintRect() );

	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::MoveSelection()
{
	if( IsTimerEnabled() )
		aFillerTimer.Stop();

	SimFuSelBase::MoveSelection();

	if( pImage )
	{
		Line aLine( GetLastPos(), GetActPos() );
		
		bEnumLineSet = FALSE;
		aLastEnumPoint = GetActPos();
		bIsInLineEnum = TRUE;
		aLine.Enum( LINK( this, SimFuPaintBase, LineEnumHdl ) );
		bIsInLineEnum = FALSE;
		pView->Invalidate( GetPaintRect() );
	}

	if( IsTimerEnabled() )
		aFillerTimer.Start();

	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintBase::EndSelection()
{
	if( IsTimerEnabled() )
		aFillerTimer.Stop();

	SimFuSelBase::EndSelection();

	delete pRegionBmp;
	pRegionBmp = NULL;

	if( pImage )
	{
		pImage->EndProcessing();
		pImage = NULL;
	}

	if( pAlpha )
	{
		pAlpha->EndProcessing();
		pAlpha = NULL;
	}

	pModel->SendRepaintBroadcast( pView->GetCurFramePos(), FALSE );
	pModel->SetChanged( TRUE );
	pModel->IncModificationCount();
	( (SimDrawViewShell*) pViewShell )->CreateUndoAction( String( SimResId( STR_PAINTING ) ), SID_SIM_PAINTING );
	
	return TRUE;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimFuPaintBase, LineEnumHdl, Point*, pPt )
{
	if( bEnumLineSet )
		PaintFiller( *pPt );
	else
		bEnumLineSet = TRUE;

	aLastEnumPoint = *pPt;

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimFuPaintBase, FillerTimeoutHdl, Timer*, pTimer )
{
	if( pImage )
	{
		PaintFiller( GetActPos() );
		pView->Invalidate( GetPaintRect() );
		pTimer->Start();
	}

	return 0L;
}

// -------------------
// - SimFuPaintPixel -
// -------------------

SimFuPaintPixel::SimFuPaintPixel( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
								  SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuPaintBase( pViewSh, pWin, pView, pSimModel, rReq )
{
}

// -----------------------------------------------------------------------------

SimFuPaintPixel::~SimFuPaintPixel()
{
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintPixel::InitSelection( BOOL& rIntercepted )
{
	BOOL bRet = SimFuPaintBase::InitSelection( rIntercepted );

	if( bRet  )
	{
		if( GetPaintTransparency() )
			bRet = ChangeImageDepth( 24, TRUE, rIntercepted );
		else
		{
			BitmapPalette aPal( 1 );

			aPal[ 0 ] = GetPaintColor();
			bRet = AddImageColors( aPal, TRUE, rIntercepted );
		}
	}

	return bRet;
}

// -----------------------------------------------------------------------------

BitmapEx SimFuPaintPixel::GetFiller() const
{
	long				nPaintExt = GetPaintExt();
	const Size			aSize( nPaintExt, nPaintExt );
	BitmapEx			aRet;
	BOOL				bTrans = GetPaintTransparency() < 100;
	Bitmap				aAlpha( aSize, bTrans ? 8 : 1, bTrans ? &GetGrayPal() : NULL );
	BitmapWriteAccess*	pWAcc = aAlpha.AcquireWriteAccess();

	if( pWAcc )
	{
		const Color	aWhite( COL_WHITE );
		Color		aAlphaCol;

		if( bTrans )
		{
			const BYTE cVal = (BYTE) FRound( 2.55 * GetPaintTransparency() );
			aAlphaCol = Color( cVal, cVal, cVal );
		}
		else
			aAlphaCol = Color( COL_BLACK );

		pWAcc->SetFillColor( aAlphaCol );
		pWAcc->SetLineColor( aAlphaCol );

		switch( GetSlotID() )
		{
			case SID_PIXEL_ROUND:
			{
				if( nPaintExt == 1 )
					pWAcc->Erase( aAlphaCol );
				else
				{
					VirtualDevice	aVDev( 1 );
					const Color		aBlack( COL_BLACK );

					aVDev.SetOutputSizePixel( aSize );
					aVDev.SetLineColor( aBlack );
					aVDev.SetFillColor( aBlack );
					aVDev.DrawPolygon( CreateEllipse( Rectangle( Point(), aSize ) ) );

					Bitmap				aBmp( aVDev.GetBitmap( Point(), aSize ) );
					BitmapReadAccess*	pRAcc = aBmp.AcquireReadAccess();

					if( pRAcc )
					{
						const BitmapColor aWW( pWAcc->GetBestMatchingColor( aWhite ) );
						const BitmapColor aWP( pWAcc->GetBestMatchingColor( aAlphaCol ) );
						const BitmapColor aRB( pRAcc->GetBestMatchingColor( aBlack ) );

						for( long nY = 0L; nY < nPaintExt; nY++ )
							for( long nX = 0L; nX < nPaintExt; nX++ )
								pWAcc->SetPixel( nY, nX, ( pRAcc->GetPixel( nY, nX ) == aRB ) ? aWP : aWW );

						aBmp.ReleaseAccess( pRAcc );
					}
				}
			}
			break;

			case SID_PIXEL_VERT:
			{
				pWAcc->Erase( aWhite );
				pWAcc->DrawLine( Point( nPaintExt >> 1, 0 ), Point( nPaintExt >> 1, nPaintExt - 1 ) );
			}
			break;

			case SID_PIXEL_HORZ:
			{
				pWAcc->Erase( aWhite );
				pWAcc->DrawLine( Point( 0, nPaintExt >> 1 ), Point( nPaintExt - 1, nPaintExt >> 1 ) );
			}
			break;

			case SID_PIXEL_LEFT:
			{
				pWAcc->Erase( aWhite );
				pWAcc->DrawLine( Point(), Point( nPaintExt - 1, nPaintExt - 1 ) );
			}
			break;

			case SID_PIXEL_RIGHT:
			{
				pWAcc->Erase( aWhite );
				pWAcc->DrawLine( Point( nPaintExt - 1, 0 ), Point( 0, nPaintExt - 1 ) );
			}
			break;

			default:
				pWAcc->Erase( aAlphaCol );
			break;
		}

		aAlpha.ReleaseAccess( pWAcc );
		aRet = BitmapEx( aAlpha );
		aRet.SetTransparentColor( GetPaintColor() );
	}

	return aRet;
}

// -----------------------------------------------------------------------------

void SimFuPaintPixel::PaintFiller( const Point& rPosPixel )
{
	const USHORT nSlotId = GetSlotID();

	SimFuPaintBase::PaintFiller( rPosPixel );

	if( IsInLineEnum() && ( SID_PIXEL_LEFT == nSlotId || SID_PIXEL_RIGHT == nSlotId ) )
	{
		const Point& rLast = GetLastEnumPoint();

		if( SID_PIXEL_LEFT == nSlotId )
		{
			if( ( rPosPixel.X() == rLast.X() - 1 ) && ( rPosPixel.Y() == rLast.Y() + 1 ) )
				SimFuPaintBase::PaintFiller( Point( rPosPixel.X() + 1, rPosPixel.Y() ) );
			else if( ( rPosPixel.X() == rLast.X() + 1 ) && ( rPosPixel.Y() == rLast.Y() - 1 ) )
				SimFuPaintBase::PaintFiller( Point( rPosPixel.X(), rPosPixel.Y() + 1 ) );
		}
		else
		{
			if( ( rPosPixel.X() == rLast.X() + 1 ) && ( rPosPixel.Y() == rLast.Y() + 1 ) )
				SimFuPaintBase::PaintFiller( Point( rPosPixel.X(), rPosPixel.Y() - 1 ) );
			else if( ( rPosPixel.X() == rLast.X() - 1 ) && ( rPosPixel.Y() == rLast.Y() - 1 ) )
				SimFuPaintBase::PaintFiller( Point( rPosPixel.X() + 1, rPosPixel.Y() ) );
		}
	}
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintPixel::ShouldUseRegion() const
{
	return TRUE;
}

// -----------------------------------------------------------------------------

Pointer SimFuPaintPixel::GetPointer() const
{
	return Pointer( POINTER_PEN );
}

// -----------------
// - SimFuPaintAir -
// -----------------

SimFuPaintAir::SimFuPaintAir( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
							  SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuPaintBase	( pViewSh, pWin, pView, pSimModel, rReq ),
		nAirWidth		( 0 ),
		nIntens			( 0 )
{
}

// -----------------------------------------------------------------------------

SimFuPaintAir::~SimFuPaintAir()
{
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintAir::InitSelection( BOOL& rIntercepted )
{
	BOOL bRet = SimFuPaintBase::InitSelection( rIntercepted );

	if( bRet  )
		bRet = ChangeImageDepth( 24, TRUE, rIntercepted );

	return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintAir::BeginSelection()
{
	const SfxItemSet& rAttrSet = pView->GetAttrSet();

	nAirWidth = ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_AIR_WIDTH ) ).GetValue();
	nIntens = nAirWidth * nAirWidth;

	switch( (AirIntensity) ( (const SfxUInt16Item&) rAttrSet.Get( ATTR_PNT_AIR_INTENS ) ).GetValue() )
	{
		case( AIR_LOW ): nIntens >>= 3; break;
		case( AIR_MEDIUM ): nIntens >>= 2; break;
		case( AIR_HIGH ): nIntens >>= 1; break;
		
		default:
		break;
	}

	return SimFuPaintBase::BeginSelection();
}

// -----------------------------------------------------------------------------

BitmapEx SimFuPaintAir::GetFiller() const
{
	BitmapEx aRet;

	if( nAirWidth && nIntens )
	{
		const Size			aSize( nAirWidth, nAirWidth );
		Bitmap				aAlpha( aSize, 8, &GetGrayPal() );
		BitmapWriteAccess*	pWAcc = aAlpha.AcquireWriteAccess();

		if( pWAcc )
		{
			BitmapColor		aVal( (BYTE) 0 );
			const long		nCount1 = nAirWidth * nAirWidth - 1;
			const double	fRadius = nAirWidth * 0.5;
			const double	fCenterX = fRadius, fCenterY = fRadius;

			pWAcc->Erase( Color( COL_WHITE ) );
			srand( Time::GetSystemTicks() + GetActPos().X() * GetActPos().Y() );

			for( long i = 0; i < nIntens; i++ )
			{
				const long		nVal = FRound( nCount1 * ( (double) rand() / RAND_MAX ) );
				const long		nX = nVal % nAirWidth, nY = nVal / nAirWidth;
				const double	fDist = hypot( fCenterX - nX, fCenterY - nY );

				if( fDist <= fRadius )
				{
					aVal.SetIndex( (BYTE) ( 255.0 * fDist / fRadius ) );
					pWAcc->SetPixel( nY, nX, aVal );
				}
			}

			aAlpha.ReleaseAccess( pWAcc );
			aRet = BitmapEx( aAlpha );
			aRet.SetTransparentColor( GetPaintColor() );
		}
	}

	return aRet;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintAir::ShouldUseTimer() const
{
	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPaintAir::FillerDiffers() const
{
	return TRUE;
}

// -----------------------------------------------------------------------------

Rectangle SimFuPaintAir::GetPaintRect() const
{
	Rectangle	aInvRect( GetLastPos(), GetActPos() );
	const long	nExt = ( nAirWidth >> 1 ) + 2;

	aInvRect.Justify();
	aInvRect.Left() -= nExt;
	aInvRect.Top() -= nExt;
	aInvRect.Right() += nExt;
	aInvRect.Bottom() += nExt;

	return aInvRect;
}

// -----------------------------------------------------------------------------

Pointer SimFuPaintAir::GetPointer() const
{
	return Pointer( POINTER_AIRBRUSH );
}
