/*************************************************************************
 *
 *  $RCSfile: fupoor.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <math.h>
#include <vcl/salbtype.hxx>
#include <vcl/seleng.hxx>
#include <vcl/dialog.hxx>
#include "fupoor.hxx"
#include "simwin.hxx"
#include "docshell.hxx"
#include "viewshel.hxx"
#include "simview.hxx"
#include "dragserv.hxx"

// -------------
// - SimFuPoor -
// -------------

SimFuPoor::SimFuPoor( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
					  SimModel* pSimModel, SfxRequest& rReq ) :
			pViewShell		( pViewSh ),
			pWindow			( pWin ),
			pView			( pView ),
			pModel			( pSimModel ),
			aSfxRequest		( rReq ),
			pDialog			( NULL ),
			nDragFramePos	( 0 ),
			bStartDrag		( FALSE ),
			bIsInDDMode		( FALSE )
{
    aScrollTimer.SetTimeoutHdl( LINK( this, SimFuPoor, ScrollHdl ) );
    aScrollTimer.SetTimeout( 50 );

    aDragTimer.SetTimeoutHdl( LINK( this, SimFuPoor, DragHdl ) );
    aDragTimer.SetTimeout( SELENG_DRAGDROP_TIMEOUT );
}

// -----------------------------------------------------------------------------

SimFuPoor::~SimFuPoor()
{
    aDragTimer.Stop();
    aScrollTimer.Stop();

    if ( pDialog )
        delete pDialog;
}

// -----------------------------------------------------------------------------

void SimFuPoor::DragStarted()
{
}

// -----------------------------------------------------------------------------

Polygon	SimFuPoor::CreateEllipse( const Rectangle& rBound ) const
{
	Polygon aPoly;

	if( rBound.GetWidth() && rBound.GetHeight() )
	{
		const Point		aOff( rBound.TopLeft() );
		const USHORT	nPoints = 128, nPoints2 = nPoints >> 1, nPoints4 = nPoints >> 2;
		const double	fCenterX = ( rBound.GetWidth() - 1 ) * 0.5, fCenterY = ( rBound.GetHeight() - 1 ) * 0.5;
		double			fAngle = 0.0, fAngleStep = F_PI2 / ( nPoints4 - 1 );

		aPoly.SetSize( nPoints );

		for( USHORT i = 0; i < nPoints4; i++, fAngle += fAngleStep )
		{
			const long fX = fCenterX * cos( fAngle ), fY = -fCenterY * sin( fAngle );

			( aPoly[ i ] = Point( FRound( fX + fCenterX ), FRound( fY + fCenterY ) ) ) += aOff;
			( aPoly[ nPoints2 - i - 1 ] = Point( FRound( -fX + fCenterX ), FRound( fY + fCenterY ) ) ) += aOff;
			( aPoly[ nPoints2 + i ] = Point( FRound( -fX + fCenterX ), FRound( -fY + fCenterY ) ) ) += aOff;
			( aPoly[ nPoints - i - 1 ] = Point( FRound( fX + fCenterX ), FRound( -fY + fCenterY ) ) ) += aOff;
		}
	}

	return aPoly;
}

// -----------------------------------------------------------------------------

void SimFuPoor::Activate()
{
    if( pDialog )
        pDialog->Show();

	if( pWindow )
	{
		aOldPointer = pWindow->GetPointer();
		pWindow->SetPointer( GetPointer() );
	}
}

// -----------------------------------------------------------------------------

void SimFuPoor::Deactivate()
{
    aDragTimer.Stop();
    aScrollTimer.Stop();

    if( pDialog )
        pDialog->Hide();

	if( pWindow )
	{
		pWindow->ReleaseMouse();
		pWindow->SetPointer( aOldPointer );
	}
}

// -----------------------------------------------------------------------------

void SimFuPoor::SetWindow( SimWindow* pWin )
{
	if( pWindow )
		pWindow->SetPointer( aOldPointer );

	pWindow = pWin;
	aOldPointer = pWindow->GetPointer();
	pWindow->SetPointer( GetPointer() );
}

// -----------------------------------------------------------------------------

void SimFuPoor::ForceScroll( const Point& rPixPos, long nInnerOffsetPix )
{
	if ( !bIsInDDMode )
	{
		const Size	aWinSizePix( pWindow->GetOutputSizePixel() );
		const long	nLeftBorder = nInnerOffsetPix;
		const long	nTopBorder = nInnerOffsetPix;
		const long	nRightBorder = aWinSizePix.Width() - 1 - nInnerOffsetPix;
		const long	nBottomBorder = aWinSizePix.Height() - 1 - nInnerOffsetPix;
		long		nDx = 0;
		long		nDy = 0;
		
		nDx = ( rPixPos.X() < nLeftBorder ) ? -1 : ( rPixPos.X() > nRightBorder ) ? 1 : 0;
		nDy = ( rPixPos.Y() < nTopBorder ) ? -1 : ( rPixPos.Y() > nBottomBorder ) ? 1 : 0;

		aScrollTimer.Stop();

		if( nDx || nDy )
		{
			pViewShell->ScrollLines( nDx, nDy );
			aScrollTimer.Start();
		}
	}
}

// -----------------------------------------------------------------------------

BOOL SimFuPoor::MouseButtonDown( const MouseEvent& rMEvt )
{
	return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPoor::MouseMove( const MouseEvent& rMEvt )
{
	pViewShell->SetMousePosInfo( pView->ScreenToImagePoint( pWindow->GetPointerPosPixel(), pWindow ) );
	return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPoor::MouseButtonUp( const MouseEvent& rMEvt )
{
	return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimFuPoor::KeyInput( const KeyEvent& rKEvt )
{
    BOOL bRet = TRUE;

	if( !bStartDrag && !bIsInDDMode )
	{
		switch( rKEvt.GetKeyCode().GetCode() )
		{
			case KEY_ESCAPE:
			{
				SimDrawDocShell* pDocSh = (SimDrawDocShell*) pViewShell->GetViewFrame()->GetObjectShell();

				if( pDocSh )
					pDocSh->DoInPlaceActivate( FALSE );
			}
			break;

			case KEY_LEFT:
				pViewShell->ScrollLines( -1, 0 );
			break;

			case KEY_RIGHT:
				pViewShell->ScrollLines( 1, 0 );
			break;

			case KEY_UP:
				pViewShell->ScrollLines( 0, -1 );
			break;

			case KEY_DOWN:
				pViewShell->ScrollLines( 0, 1 );
			break;

			case KEY_ADD:
			{
				const long	nOldZoom = pWindow->GetZoom();
				long		nNewZoom = nOldZoom;

				if( nOldZoom >= 100 )
				{
					nNewZoom += 100; 
					nNewZoom -= nNewZoom % 100;
				}
				else
				{
					nNewZoom += 10; 
					nNewZoom -= nNewZoom % 10;
				}

				pViewShell->SetZoom( nNewZoom );
			}
			break;

			case KEY_SUBTRACT:
			{
				const long	nOldZoom = pWindow->GetZoom();
				long		nNewZoom = nOldZoom;

				if( nOldZoom > 100 )
				{
					nNewZoom -= Min( 100L, nOldZoom - 100L );
					nNewZoom -= nNewZoom % 100;
				}
				else
				{
					nNewZoom -= 10;
					nNewZoom -= nNewZoom % 10;
				}

				pViewShell->SetZoom( nNewZoom );
			}
			break;

			default:
				bRet = FALSE;
			break;
		}
	}

	if( bRet )
	{
		pViewShell->Invalidate();

		if( pWindow )
			pWindow->ReleaseMouse();
	}

    return bRet;
}

// -----------------------------------------------------------------------------

Pointer SimFuPoor::GetPointer() const
{
	return pWindow->GetPointer();
}

// -----------------------------------------------------------------------------

IMPL_LINK_INLINE_START( SimFuPoor, ScrollHdl, Timer*, pTimer )
{
	if ( !bIsInDDMode )
		MouseMove( MouseEvent( pWindow->GetPointerPosPixel() ) );

    return 0;
}
IMPL_LINK_INLINE_END( SimFuPoor, ScrollHdl, Timer *, pTimer )

// -----------------------------------------------------------------------------

IMPL_LINK( SimFuPoor, DragHdl, Timer*, pTimer )
{
	SimDragServerRef	xDragServer( new SimDragServer( pView ) );
	Region				aRegion; 

	pWindow->ReleaseMouse();
	nDragFramePos = pView->GetCurFramePos();
	bStartDrag = FALSE;
	bIsInDDMode = TRUE;
	DragStarted();
	xDragServer->ExecuteDrag( pWindow, POINTER_MOVEDATA, POINTER_COPYDATA, POINTER_LINKDATA, DRAG_COPYABLE, &aRegion );
	bIsInDDMode = FALSE;
	nDragFramePos = 0;

    return 0L;
}
