/*************************************************************************
 *
 *  $RCSfile: fusel.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <svx/svxids.hrc>
#include <sfx2/dispatch.hxx>
#include "strings.hrc"
#include "simresid.hxx"
#include "simmodel.hxx"
#include "viewshel.hxx"
#include "simview.hxx"
#include "simwin.hxx"
#include "simapp.hrc"
#include "viewshel.hxx"
#include "svimage.hxx"
#include "fusel.hxx"

// ----------------
// - SimFuSelBase -
// ----------------

SimFuSelBase::SimFuSelBase( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
							SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuPoor			( pViewSh, pWin, pView, pSimModel, rReq ),
		bLeftMouse			( FALSE ),
		bRightMouse			( FALSE ),
		bShift				( FALSE ),
		nMouseClickCounter	( 0UL )
{
}

// -----------------------------------------------------------------------------

SimFuSelBase::~SimFuSelBase()
{
}

// -----------------------------------------------------------------------------

void SimFuSelBase::DragStarted()
{
	if( nMouseClickCounter )
		nMouseClickCounter = 0;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::ActualSelectionDiffersFromLast() const
{
	return( GetActRect() != GetLastRect() );
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::IsDraggingAllowed() const
{
	return FALSE;
}

// -----------------------------------------------------------------------------

PolyPolygon SimFuSelBase::GetSelection() const
{
	return PolyPolygon();
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::InitSelection( BOOL& rIntercepted )
{
	rIntercepted = FALSE;
	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::BeginSelection()
{
	pView->ClearSelection();
	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::MoveSelection()
{
	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::EndSelection()
{
	ClearSelection();
	return TRUE;
}

// -----------------------------------------------------------------------------

void SimFuSelBase::ClearSelection()
{
	pView->PaintSelection( aSelection );
	aSelection.Clear();
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::MouseButtonDown( const MouseEvent& rMEvt )
{
	if( ++nMouseClickCounter == 1 )
	{
		BOOL bIntercepted = FALSE;

		bLeftMouse = rMEvt.IsLeft();
		bRightMouse = rMEvt.IsRight();
		bShift = rMEvt.IsShift();

		if( ( bLeftMouse || bRightMouse ) && ( ( nMouseClickCounter > 1 ) || InitSelection( bIntercepted ) ) )
		{
			if( bIntercepted )
				nMouseClickCounter = 0;
			else
			{
				const Rectangle	aImageRect( pView->GetRectPixel() );

				aStartPos = pView->ScreenToImagePoint( pWindow->GetPointerPosPixel(), pWindow );
				aLastRect.Left() = aActRect.Left() = aStartPos.X();
						aLastRect.Top() = aActRect.Top() = aStartPos.Y();
				aLastPos = aActPos = aStartPos;

				aActRect.SetSize( Size( 1, 1 ) );
				aActImageRect = aActRect;
				aActImageRect.Intersection( aImageRect );

				aLastRect = aActRect;
				aLastImageRect = aActImageRect;

				pWindow->CaptureMouse();
				bStartDrag = TRUE;
				bIsInDDMode = FALSE;

				if( IsDraggingAllowed() && aImageRect.IsInside( aStartPos ) )
					aDragTimer.Start();
				else
					aDragTimer.Stop();

				BeginSelection();
			}
		}
		else
			nMouseClickCounter = 0;
	}

	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::MouseMove(const MouseEvent& rMEvt)
{
	SimFuPoor::MouseMove( rMEvt );

	if( aDragTimer.IsActive() )
		aDragTimer.Stop();

	if( bStartDrag && !bIsInDDMode )
	{
		const Point		aMousePosPix( pWindow->GetPointerPosPixel() );
		const Rectangle	aImageRect( pView->GetRectPixel() );

		aLastPos = aActPos;
		aLastRect = aActRect;
		aLastImageRect = aActImageRect;

		aActPos = pView->ScreenToImagePoint( aMousePosPix, pWindow );

		aActRect = Rectangle( aStartPos, aActPos );
		aActRect.Justify();

		aActImageRect = aActRect;
		aActImageRect.Intersection( aImageRect );

		pViewShell->SetMouseRectInfo( aActRect );

		MoveSelection();

		if( ActualSelectionDiffersFromLast() )
		{
			pView->PaintSelection( aSelection );
			aSelection = GetSelection(); 
			pView->PaintSelection( aSelection );
		}

		ForceScroll( aMousePosPix );
	}

	return bStartDrag;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::MouseButtonUp(const MouseEvent& rMEvt)
{
	if( nMouseClickCounter && !( --nMouseClickCounter ) )
	{
		if( bStartDrag && !bIsInDDMode )
		{
			pViewShell->SetMouseRectInfo();
			pViewShell->Invalidate();
			pWindow->ReleaseMouse();
			bStartDrag = FALSE;
			EndSelection();
		}

		bLeftMouse = bRightMouse = bShift = FALSE;
	}

	return TRUE;
}

// -----------------------------------------------------------------------------

void SimFuSelBase::Paint( const Rectangle& rRect, SimWindow* pWin )
{
	pView->PaintSelection( aSelection, pWin );
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::AddImageColors( const BitmapPalette& rPal, BOOL bShowWarningUI, BOOL& bWarningUIWasShown )
{
	SvImage*	pImage = pView->GetImagePointer();
	BOOL		bInserted;

	bWarningUIWasShown = FALSE;

	if( pImage )
	{
		BitmapPalette* pImagePal = pImage->GetPalette();

		bInserted = TRUE;

		// if image palette has no entries, we've got a true color image
		if( pImagePal->GetEntryCount() )
		{
			for( USHORT i = 0, nNewCount = rPal.GetEntryCount(); ( i < nNewCount ) && bInserted; i++ )
			{
				const BitmapColor& rTestCol = rPal[ i ];
				
				bInserted = FALSE;

				for( USHORT n = 0, nImagePalCount = pImagePal->GetEntryCount(); ( n < nImagePalCount ) && !bInserted; n++ )
				{
					if( rTestCol == (*pImagePal)[ n ] )
						bInserted = TRUE;
				}

				if( !bInserted )
				{
					// try to find free entry
					if( pImagePal->GetEntryCount() < ( 1 << pImage->GetDepth() ) )
					{	
						pImagePal->SetEntryCount( pImagePal->GetEntryCount() + 1 );
						(*pImagePal)[ pImagePal->GetEntryCount() - 1 ] = rTestCol;
						bInserted = TRUE;
					}

					if( !bInserted )
					{
						BOOL bIncrease;

						if( !bShowWarningUI )
							bIncrease = TRUE;
						else
						{
							QueryBox aQuery( pWindow, WB_YES_NO, SimResId( STR_INC_PALETTE ) );
							bIncrease = ( aQuery.Execute() == RET_YES );
							bWarningUIWasShown = TRUE;
						}

						// Image depth needs to be increased
						if( bIncrease )
						{
							pImage->SetDepth( ( pImage->GetDepth() == 1 ) ? 4 : ( ( pImage->GetDepth() == 4 ) ? 8 : 24 ) );
							SFX_BINDINGS().Invalidate( SID_STATUS_PIC_DEPTH );

							if( pImage->GetDepth() == 24 )
								pImagePal->SetEntryCount( 0 );
							else
							{
								pImagePal->SetEntryCount( pImagePal->GetEntryCount() + 1 );
								(*pImagePal)[ pImagePal->GetEntryCount() - 1 ] = rTestCol;
							}

							bInserted = TRUE;
						}
					}
				}
			}
		}
	}
	else
		bInserted = FALSE;

	return bInserted;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelBase::ChangeImageDepth( USHORT nDepth, BOOL bShowWarningUI, BOOL& bWarningUIWasShown )
{
	SvImage*	pImage = pView->GetImagePointer();
	BOOL		bChange;

	bWarningUIWasShown = FALSE;

	if( pImage )
	{
		if( nDepth > pImage->GetDepth() )
		{
			if( !bShowWarningUI )
				bChange = TRUE;
			else
			{
				QueryBox aQuery( pWindow, WB_YES_NO, SimResId( STR_INC_PALETTE ) );
				bChange = ( aQuery.Execute() == RET_YES );
				bWarningUIWasShown = TRUE;
			}
		}
		else
			bChange = TRUE;

		if( bChange )
		{
			pImage->SetDepth( nDepth );
			SFX_BINDINGS().Invalidate( SID_STATUS_PIC_DEPTH );
		}
	}
	else
		bChange = FALSE;

	return bChange;
}

// ---------------
// - SimFuSelect -
// ---------------

SimFuSelect::SimFuSelect( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
						  SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuSelBase( pViewSh, pWin, pView, pSimModel, rReq )
{
}

// -----------------------------------------------------------------------------

SimFuSelect::~SimFuSelect()
{
}

// -----------------------------------------------------------------------------

BOOL SimFuSelect::IsDraggingAllowed() const
{
	return TRUE;
}

// -----------------------------------------------------------------------------

PolyPolygon SimFuSelect::GetSelection() const
{
	PolyPolygon			aPolyPoly;
	const Rectangle&	rRect = GetActImageRect();

	if( !rRect.IsEmpty() )
		aPolyPoly.Insert( Polygon( rRect ) );
	else
		aPolyPoly.Insert( Polygon( GetLastImageRect() ) );

	return aPolyPoly;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelect::EndSelection()
{
	const Rectangle& rRect = GetActImageRect();

	SimFuSelBase::EndSelection();
	
	if( !rRect.IsEmpty() )
	{
		pView->SetSelection( GetSelection(), TRUE );
		pViewShell->SetMouseRectInfo( rRect );
	}
	else
		pViewShell->SetMouseRectInfo();

	return TRUE;
}

// -----------------------------------------------------------------------------

BOOL SimFuSelect::ActualSelectionDiffersFromLast() const
{
	return( GetActImageRect() != GetLastImageRect() );
}

// -----------------------------------------------------------------------------

Pointer SimFuSelect::GetPointer() const
{
	return Pointer( POINTER_ARROW );
}

// -------------
// - SimFuZoom -
// -------------

SimFuZoom::SimFuZoom( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
					  SimModel* pSimModel, SfxRequest& rReq ) :
		SimFuSelect( pViewSh, pWin, pView, pSimModel, rReq )
{
}

// -----------------------------------------------------------------------------

SimFuZoom::~SimFuZoom()
{
}

// -----------------------------------------------------------------------------

BOOL SimFuZoom::IsDraggingAllowed() const
{
	return FALSE;
}

// -----------------------------------------------------------------------------

BOOL SimFuZoom::EndSelection()
{
	SimFuSelBase::EndSelection();

	const Rectangle aActImageRect( GetActImageRect() );

	if( aActImageRect.GetWidth() > 1 || aActImageRect.GetHeight() > 1 )
		pViewShell->SetZoomRect( aActImageRect );
	else
	{
		const long	nOldZoom = pWindow->GetZoom();
		long		nNewZoom = nOldZoom;

		if( nOldZoom >= 100 )
		{
			nNewZoom += 100; 
			nNewZoom -= nNewZoom % 100;
		}
		else
		{
			nNewZoom += 10; 
			nNewZoom -= nNewZoom % 10;
		}

		pViewShell->SetZoomPoint( aActImageRect.Center(), nNewZoom );
	}

	pViewShell->Invalidate( SID_ZOOM_IN );
	pViewShell->Invalidate( SID_ZOOM_OUT );
	pViewShell->Invalidate( SID_ATTR_ZOOM );
	SFX_DISPATCHER().Execute( SID_OBJECT_SELECT, SFX_CALLMODE_ASYNCHRON );
	pViewShell->Cancel();

	return TRUE;
}

// -----------------------------------------------------------------------------

Pointer SimFuZoom::GetPointer() const
{
	return Pointer( POINTER_MAGNIFY );
}
