/*************************************************************************
 *
 *  $RCSfile: fupaint.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:29 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_FUPAINT_HXX
#define _SIM_FUPAINT_HXX

#include <vcl/bitmapex.hxx>
#include <vcl/salbtype.hxx>
#include "fusel.hxx"

// ------------------
// - SimFuPaintBase -
// ------------------

class SvImage;

class SimFuPaintBase : public SimFuSelBase
{
private:

	BitmapPalette		aGrayPal;
	Timer				aFillerTimer;
	BitmapEx			aFiller;
	BitmapEx			aAlphaFiller;
	Color				aForeCol;
	Color				aBackCol;
	long				nFillWidth;
	SvImage*			pImage;
	SvImage*			pAlpha;
	Bitmap*				pRegionBmp;
	Point				aLastEnumPoint;
	USHORT				nLineTransparency;
	USHORT				nFillTransparency;
	BOOL				bEnumLineSet;
	BOOL				bIsInLineEnum;
	BOOL				bTimer;
						
						DECL_LINK( FillerTimeoutHdl, Timer* pTimer );
						DECL_LINK( LineEnumHdl, Point* pPt );
						
protected:				

	virtual	void		PaintFiller( const Point& rPosPixel );
						
	virtual BOOL		InitSelection( BOOL& rIntercepted );
	virtual BOOL		BeginSelection();
	virtual BOOL		MoveSelection();
	virtual BOOL		EndSelection();
						
	virtual BitmapEx	GetFiller() const;
	virtual BOOL		FillerDiffers() const;

	virtual BOOL		ShouldUseRegion() const;
	virtual BOOL		ShouldUseTimer() const;

	virtual Rectangle	GetPaintRect() const;

	const BitmapPalette& GetGrayPal() const { return aGrayPal; }
						
	const Color&		GetPaintColor() const { return( IsLeftMouse() ? aForeCol : aBackCol ); }
	USHORT				GetPaintTransparency() const { return( IsLeftMouse() ? nLineTransparency : nFillTransparency ); }

	long				GetPaintExt() const { return nFillWidth; }

	const Point&		GetLastEnumPoint() const { return aLastEnumPoint; } 
	BOOL				IsInLineEnum() const { return bIsInLineEnum; }
	BOOL				IsTimerEnabled() const { return bTimer; }
						
public:					
						
						SimFuPaintBase( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
										SimModel* pModel, SfxRequest& rReq );
	virtual				~SimFuPaintBase();
};

// -------------------
// - SimFuPaintPixel -
// -------------------

class SimFuPaintPixel : public SimFuPaintBase
{
protected:
						
	virtual BOOL		InitSelection( BOOL& rIntercepted );
	virtual BitmapEx	GetFiller() const;
	virtual	void		PaintFiller( const Point& rPosPixel );
	virtual BOOL		ShouldUseRegion() const;
				
public:					
						
						SimFuPaintPixel( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
										 SimModel* pModel, SfxRequest& rReq );
	virtual				~SimFuPaintPixel();

	virtual Pointer		GetPointer() const;
};

// -----------------
// - SimFuPaintAir -
// -----------------

class SimFuPaintAir : public SimFuPaintBase
{
private:

	long				nAirWidth;
	long				nIntens;

protected:
						
	virtual BOOL		InitSelection( BOOL& rIntercepted );
	virtual BOOL		BeginSelection();
	virtual BitmapEx	GetFiller() const;
	virtual Rectangle	GetPaintRect() const;
	virtual BOOL		FillerDiffers() const;
	virtual BOOL		ShouldUseTimer() const;
				
public:					
						
						SimFuPaintAir( SimViewShell* pViewSh, SimWindow* pWin, SimView* pView,
										 SimModel* pModel, SfxRequest& rReq );
	virtual				~SimFuPaintAir();

	virtual Pointer		GetPointer() const;
};


#endif // _SIM_FUPAINT_HXX
