/*************************************************************************
 *
 *  $RCSfile: simview.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:29 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_SIMVIEW_HXX
#define _SIM_SIMVIEW_HXX

#include <tools/gen.hxx>
#include <vcl/color.hxx>
#include <vcl/animate.hxx>
#include <vcl/poly.hxx>
#include <so3/dtrans.hxx>
#include <svtools/lstner.hxx>
#include <svtools/itemset.hxx>
#include "simframe.hxx"

// -----------
// - Defines -
// -----------

#define SIM_INSERTDATAPOS_REPLACE	0xffffffffUL
#define SIM_INSERTDATAPOS_CURPOS	0xfffffffeUL
#define SIM_INSERTDATAPOS_APPEND	0xfffffffdUL

// -----------
// - SimView -
// -----------

class Bitmap;
class Graphic;
class SimModel;
class SimDrawDocShell;
class SimViewShell;
class SvImage;
class DropEvent;
class SimFrame;
class SimUndoAction;

class SimView : public SfxListener
{
	SfxItemSet				aAttrSet;
	Color					aFgColor;
	Color					aBgColor;
	List					aWinList;
	PolyPolygon				aSelection;
	Rectangle				aOutRect;
	SimDrawDocShell* 		pDocShell;
	SimModel* 				pModel;
	SimViewShell*			pViewShell;
	SimFrame*				pCurFrame;
	long*					pDitherDiff;
	BYTE*					pDitherLow;
	BYTE*					pDitherHigh;
	ULONG					nCurFramePos;
	BOOL					bSelectionVisible;
	BOOL					bAttrChanged;
	BOOL					bIsFrameAcquired;
	BOOL					bDrawBackground;
	BOOL					bDrawGrid;
	BOOL					bDither;

	void					ImplPaintBackground( const Rectangle& rRect, OutputDevice* pOut );
	void					ImplPaintFastBackground( const Rectangle& rRect, OutputDevice* pOut );
	void					ImplPaintTile( const Rectangle& rRect, OutputDevice* pOut );
	void					ImplPaintGrid( const Rectangle& rRect, OutputDevice* pOut );
	void					ImplPaintSelection( PolyPolygon& rSelection, OutputDevice* pOut );

protected:
							
	virtual void			SFX_NOTIFY( SfxBroadcaster& rBC, const TypeId& rBCType,
										const SfxHint& rHint, const TypeId& rHintType );
							
public:						
							
							SimView( SimDrawDocShell* pDShell, SimModel* pModel, SimViewShell* pVShell, OutputDevice* pOut );
							~SimView();
							
	virtual void			Paint( const Rectangle& rRect, OutputDevice* pOut );
	virtual void 			Copy();
	virtual void			Paste();
	virtual BOOL			QueryDrop( DropEvent& rEvt );
	virtual BOOL			Drop( const DropEvent& rEvt );
	virtual void			Undo( SimUndoAction* pUndo );
	virtual void			Redo( SimUndoAction* pUndo );
	virtual void			Repeat( SimUndoAction* pUndo );

	SimModel* 				GetModel() const { return pModel; }

	void					EnableBackground( BOOL bEnable ) { bDrawBackground = bEnable; }
	BOOL					IsBackgroundEnabled() const { return bDrawBackground; }

	void					EnableGrid( BOOL bEnable ) { bDrawGrid = bEnable; }
	BOOL					IsGridEnabled() const { return bDrawGrid; }

	BOOL					IsDitheringEnabled() const { return bDither; }

	void					SetOutputRect( const Rectangle& rRect ) { aOutRect = rRect; }
	const Rectangle&		GetOutputRect() const { return aOutRect; }

	Point					ImageToScreenPoint( const Point& rImgPos, OutputDevice* pOut );
	Rectangle				ImageToScreenRect( const Point& rImgPos, OutputDevice* pOut );

	Point					ScreenToImagePoint( const Point& rScreenPos, OutputDevice* pOut );
	Rectangle				ScreenToImageRect( const Point& rScreenPos, OutputDevice* pOut );

	void					PaintSelection( PolyPolygon& rSelection, OutputDevice* pOut = NULL );
							
	void					Invalidate( OutputDevice* pOut = NULL );
	void					Invalidate( const Rectangle& rRectPixel, OutputDevice* pOut = NULL );
	void 					AddOutputDevice( OutputDevice* pOut );
	void 					DelOutputDevice( OutputDevice* pOut );

	void					SetSelection( const PolyPolygon& rSelection, BOOL bShow );
	const PolyPolygon&		GetSelection() const { return aSelection; }
	Rectangle				GetSelectionBoundRect() const { return aSelection.GetBoundRect(); }

	BOOL					HasSelection() const { return aSelection.Count() > 0; }
	BOOL					IsSelectionVisible() const { return bSelectionVisible; }

	void					ShowSelection( OutputDevice* pOut = NULL );
	void					HideSelection();
	void					ClearSelection();

	const SfxItemSet&		GetAttrSet() const { return aAttrSet; }
	void					AttribsChanged() { bAttrChanged = TRUE; }
	BOOL					HaveAttribsChanged() const { return bAttrChanged; }
	void					AttribsConsumed() { bAttrChanged = FALSE; }
	void					InsertData( SvDataObjectRef pDataObj, ULONG nInsertPos );

	AnimBmp					GetAnimBmp();
							
	BOOL					SwitchToFrame( ULONG nFramePos );
	ULONG					GetFrameCount() const;
	ULONG					GetCurFramePos() const { return nCurFramePos; }
	SimFrame*				GetCurFrame() const { return pCurFrame; }
	void					InsertFrame( SimFrame* pFrame, ULONG nFramePos );
	BOOL					RemoveFrame( ULONG nFramePos );
	BOOL					ReplaceFrame( SimFrame* pFrame, ULONG nFramePos );

	Size					GetSizePixel() const;
	Rectangle				GetRectPixel() const { return Rectangle( Point(), GetSizePixel() ); }
	USHORT					GetColorDepth() const;
	BOOL					IsTransparent() const;

	Size					GetPrefSize() const;
	void					SetPrefSize( const Size& rPrefSize );
							
	MapMode					GetPrefMapMode() const;
	void					SetPrefMapMode( const MapMode& rPrefMapMode );
							
	Point					GetDisplayPosPixel() const;
	void					SetDisplayPosPixel( const Point& rPosPix );
							
	Size					GetDisplaySizePixel() const;
	void					SetDisplaySizePixel( const Size& rSizePix );
							
	ULONG					GetWaitMS() const;
	void					SetWaitMS( ULONG nWaitMS );
							
	Disposal				GetDisposal() const;
	void					SetDisposal( Disposal eDisposal );
								
	SvImage*				GetImagePointer();
	SvImage*				GetAlphaPointer();
};

#endif // _SIM_SIMVIEW_HXX
