/*************************************************************************
 *
 *  $RCSfile: viewshel.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SIM_VIEWSHEL_HXX
#define _SIM_VIEWSHEL_HXX

#include <sfx2/viewsh.hxx>

// -----------
// - Defines -
// -----------

#define MAX_HSPLIT_CNT			2
#define MAX_VSPLIT_CNT			2
#define MIN_SCROLLBAR_SIZE		50

// --------------
// - ErrorCodes -
// --------------

#define SIM_ERROR_NONE			0x00000000UL
#define SIM_ERROR_OUTOFMEMORY	0x00000001UL

// ---------------
// - SimMousInfo -
// ---------------

struct SimMouseInfo
{
	Point		maMousePos;
	Rectangle	maSelRect;
};

// ----------------
// - SimViewShell -
// ----------------

class SimFuPoor;
class SimView;
class SimWindow;
class SvxRuler;
class SimView;
class ScrollBar;
class ScrollBarBox;
class Splitter;

class SimViewShell : public SfxViewShell
{
	void				CreateHSplitElems(long nSplitXPixel);
	void				CreateVSplitElems(long nSplitYPixel);
	void				Construct();

protected:

	Table				aShellTable;
	SimMouseInfo		aMouseInfo;
	Size				aViewSize;
	Size				aScrBarWH;
	Point				aViewPos;
	Splitter*			pHSplit;
	Splitter*			pVSplit;
	ScrollBarBox*		pBox;
	SimView*			pView;
	SimWindow*			pWinArray[ MAX_HSPLIT_CNT ][ MAX_VSPLIT_CNT ];
	ScrollBar*			pHScrlArray[ MAX_HSPLIT_CNT ];
	ScrollBar*			pVScrlArray[ MAX_VSPLIT_CNT ];
	SimWindow*			pWindow;
	SimFuPoor* 			pFuActual;
	SimFuPoor* 			pFuOld;
	USHORT				nCurrentObjectBar;
	BOOL				bObjectBarSwitchEnabled;
	BOOL				bCenterAllowed;
	BOOL				bIsHSplit;
	BOOL				bIsVSplit;
	BOOL				bDragging;
	BOOL				bIPZoomSet;

						DECL_LINK( SplitHdl, Splitter* );
						DECL_LINK( HScrollHdl, ScrollBar* );
						DECL_LINK( VScrollHdl, ScrollBar* );

	virtual void		Activate( BOOL IsMDIActivate );
	virtual void		Deactivate( BOOL IsMDIActivate );
	virtual void		SetZoomFactor( const Fraction &rZoomX, const Fraction &rZoomY );
	virtual long		GetIPZoom( const Size& rLogSize100 );
	virtual long		GetHCtrlWidth();
	virtual void		InnerResizePixel( const Point &rOfs, const Size &rSize );
	virtual void		OuterResizePixel( const Point &rOfs, const Size &rSize );

 public:

						TYPEINFO();

						SimViewShell( SfxViewFrame *pFrame, Window *pParent, BOOL bAllowCenter = TRUE );
						SimViewShell( SfxViewFrame *pFrame, const SimViewShell& rShell );
						~SimViewShell();

	virtual void		AdjustControls( const Point &rNewPos, const Size &rNewSize );
	virtual BOOL		QueryDrop( DropEvent& rEvt, SimWindow* pWin = NULL );
	virtual BOOL		Drop( const DropEvent& rEvt, SimWindow* pWin = NULL );
	virtual void		Paint( const Rectangle& rRect, SimWindow* pWin ) {}
	virtual BOOL		KeyInput( const KeyEvent& rKEvt, SimWindow* pWin );
	virtual void		MouseMove( const MouseEvent& rMEvt, SimWindow* pWin );
	virtual void		MouseButtonUp( const MouseEvent& rMEvt, SimWindow* pWin );
	virtual void		MouseButtonDown( const MouseEvent& rMEvt, SimWindow* pWin );
	virtual void		Draw( OutputDevice &rDev, const Region &rReg ) {}
	virtual BOOL		Command( const CommandEvent& rCEvt, SimWindow* pWin );

	virtual void		AddWindow( SimWindow* pWin ) {}
	virtual void		RemoveWindow( SimWindow* pWin ) {}

	virtual void		Flush();
	virtual void		InitFrameChange();
	virtual void		EndFrameChange();

    virtual PrintDialog* CreatePrintDialog(Window *pParent);

	void				SetMousePosInfo();
	void				SetMousePosInfo( const Point& rMousePos );

	void				SetMouseRectInfo();
	void				SetMouseRectInfo( const Rectangle& rSelRect );

	const SimMouseInfo&	GetMouseInfo() const { return aMouseInfo ; }

	void				Cancel();
	SimView*			GetView() const { return pView; }
	SimWindow*			GetActiveWindow() { return pWindow; }
	void				UpdateScrollBars();
	void				ScrollLines( long nX, long nY );
	void				SetZoom( long nZoom );
	void				SetZoomRect( const Rectangle& rZoomRect );
	void				SetZoomPoint( const Point& rCenter, long nZoom );
	void				InitWindows();
	USHORT				SwitchObjectBar( USHORT nResId );
	USHORT				GetObjectBar() { return nCurrentObjectBar; }
	void				GetState( SfxItemSet &rSet );
	void				ExecReq( SfxRequest &rReq );
	void				EnumSimWindows( const Link& rEnumLink );
	SimFuPoor*			GetFuActual() const { return pFuActual; }
	void				SetFuActual( SimFuPoor* pFu ) { pFuActual = pFu; }
	void				SetDragging( BOOL bDrag ) { bDragging = bDrag; }
	BOOL				IsDragging() const { return bDragging; }

	void				HandleSimError( ULONG nSimErrorCode );
};

#endif // _SIM_VIEWSHEL_HXX
