/*************************************************************************
 *
 *  $RCSfile: drviews2.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_COLOR	0
#define ITEMID_SIZE		0

#include <svtools/whiter.hxx>
#include <svtools/ptitem.hxx>
#include <sfx2/bindings.hxx>
#include <sfx2/dispatch.hxx>
#include <svx/colritem.hxx>
#include <svx/xdef.hxx>
#include <svx/xlnclit.hxx>
#include <svx/bmpmask.hxx>
#include <svx/gallery.hxx>
#include <svx/colrctrl.hxx>
#include <svx/svxids.hrc>
#include <svx/zoomitem.hxx>
#include <svx/sizeitem.hxx>
#include "simapp.hrc"
#include "strings.hrc"
#include "simmodel.hxx"
#include "simattr.hxx"
#include "fusel.hxx"
#include "docshell.hxx"
#include "simview.hxx"
#include "simwin.hxx"
#include "simresid.hxx"
#include "simitems.hxx"
#include "svimage.hxx"
#include "simundo.hxx"
#include "drviewsh.hxx"

// -----------
// - Defines -
// -----------

#define ITEMVALUE(ItemSet,Id,Cast) ((const Cast&)(ItemSet).Get(Id)).GetValue()

// --------------------
// - SimDrawViewShell -
// --------------------

void SimDrawViewShell::CreateUndoAction( const String& rComment, const USHORT nSID )
{
	SfxUndoManager* pUndoMgr = GetViewFrame()->GetObjectShell()->GetUndoManager();
	pUndoMgr->Clear();
	pUndoMgr->AddUndoAction( new SimUndoAction( pView, rComment, nSID ) );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ExecCtrl(SfxRequest &rReq)
{
	const USHORT nSlot = rReq.GetSlot();

	switch( nSlot )
	{
		case( SID_SHOW_GRID ):
		{
			pView->EnableGrid( !pView->IsGridEnabled() );
			pView->Invalidate();
		}
		break;

		default:
		break;
	}

	SFX_BINDINGS().Invalidate( nSlot );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetCtrlState(SfxItemSet &rSet)
{
	rSet.Put( SfxBoolItem( SID_SHOW_GRID, pView->IsGridEnabled() ) );

	if( pWindow->GetZoom() % 100 )
		rSet.DisableItem( SID_SHOW_GRID );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ExecAttr( SfxRequest &rReq )
{
	SfxItemSet&			rAttrSet = (SfxItemSet&) pView->GetAttrSet();
	const SfxItemSet*	pArgs = rReq.GetArgs();
	const USHORT		nSlot = rReq.GetSlot();

	if( pArgs )
	{
		// Linienfarbe nehmen wir als Vorder-/Hintergrundfarbe;
		// dementspr. benutzen wir auch ein eigenes Item
		// fuer Vorder- und Hintergrundfarbe ( ein wenig umstaendlich )
		if ( ( nSlot == SID_COLOR_FORE ) || ( nSlot == SID_COLOR_BACK ) )
		{
			rAttrSet.Put( SvxColorItem( ITEMVALUE( *pArgs, XATTR_LINECOLOR, XLineColorItem ),
										nSlot == SID_COLOR_FORE ? ATTR_PNT_COLOR_FORE : ATTR_PNT_COLOR_BACK ) );

			SFX_BINDINGS().Invalidate( SID_COLOR_STATUS );
		}
		else if ( ( nSlot == SID_COLORBAR_FORE ) || ( nSlot == SID_COLORBAR_BACK ) )
		{
			String aTmpString;
			XLineColorItem	aLineColorItem( aTmpString, ITEMVALUE( *pArgs, nSlot, SvxColorItem ) );
			USHORT			nDispatchSlot;
			const USHORT	nSID = pFuActual ? pFuActual->GetSlotID() : 0;

			switch( nSID )
			{
				case( SID_PAINTTOOL_LINE ):
				case( SID_PAINTTOOL_AIR ):
				case( SID_PIXEL_ROUND ):
				case( SID_PIXEL_SQUARE ):
				case( SID_PIXEL_VERT ):
				case( SID_PIXEL_HORZ ):
				case( SID_PIXEL_LEFT ):
				case( SID_PIXEL_RIGHT ):
					nDispatchSlot = ( nSlot == SID_COLORBAR_FORE ? SID_COLOR_BACK : SID_COLOR_FORE );
				break;

				default:
					nDispatchSlot = ( nSlot == SID_COLORBAR_FORE ? SID_COLOR_FORE : SID_COLOR_BACK );
				break;
			}

			SFX_DISPATCHER().Execute( nDispatchSlot, SFX_CALLMODE_RECORD, &aLineColorItem, 0L );
			SFX_BINDINGS().Invalidate( nDispatchSlot );
		}
		else if( nSlot == SID_LINE_WIDTH )
		{
			// dieser Controller bedient sowohl die
			// Linienbreite als auch die Spruehbreite
			const USHORT	nSID = pFuActual ? pFuActual->GetSlotID() : 0;
			const USHORT	nValue = ITEMVALUE( *pArgs, ATTR_PNT_LINE_WIDTH, SfxUInt16Item );

			if ( SID_PAINTTOOL_AIR == nSID )
				rAttrSet.Put( SfxUInt16Item( ATTR_PNT_AIR_WIDTH, nValue ) );
			else if ( SID_PIXEL_ROUND <= nSID &&  SID_PIXEL_RIGHT >= nSID )
				rAttrSet.Put( SfxUInt16Item( ATTR_PNT_PEN_WIDTH, nValue ) );
			else
				rAttrSet.Put( SfxUInt16Item( ATTR_PNT_LINE_WIDTH, nValue ) );
		}
		else
			rAttrSet.Put( *pArgs );

		pView->AttribsChanged();
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetAttrState( SfxItemSet &rSet )
{
	const USHORT nSID = pFuActual ? pFuActual->GetSlotID() : 0;

	if( ( nSID < SID_PAINTSLOTS_START ) || ( nSID > SID_PAINTSLOTS_END ) )
	{
		rSet.DisableItem( SID_LINE_WIDTH );
		rSet.DisableItem( SID_LINE_ROUND );
		rSet.DisableItem( SID_AIR_INTENS );
		rSet.DisableItem( SID_COLOR_FORE );
		rSet.DisableItem( SID_COLOR_BACK );
		rSet.DisableItem( SID_LINE_TRANSPARENCY );
		rSet.DisableItem( SID_FILL_TRANSPARENCY );
	}
	else
	{
		const SfxItemSet&	rAttrSet = pView->GetAttrSet();
		SfxWhichIter		aIter( rSet );
		USHORT				nWhich = aIter.FirstWhich();

		while( nWhich )
		{
			switch( nWhich )
			{
				case( SID_LINE_WIDTH ):
				{
					const USHORT nAttrId = SID_PAINTTOOL_AIR == nSID ? ATTR_PNT_AIR_WIDTH :
										   ( SID_PIXEL_ROUND <= nSID &&  SID_PIXEL_RIGHT >= nSID ) ?
										   ATTR_PNT_PEN_WIDTH : ATTR_PNT_LINE_WIDTH;

					rSet.Put( SfxUInt16Item( SID_LINE_WIDTH, ITEMVALUE( rAttrSet, nAttrId, SfxUInt16Item ) ) );
				}
				break;

				case( SID_LINE_ROUND ):
				{
					if ( ( nSID != SID_PAINTTOOL_RECT ) && ( nSID != SID_PAINTTOOL_RECT_NULL ) )
						rSet.DisableItem( SID_LINE_ROUND );
					else
						rSet.Put( SfxUInt16Item( SID_LINE_ROUND, ITEMVALUE( rAttrSet, ATTR_PNT_LINE_ROUND, SfxUInt16Item ) ) );
				}
				break;

				case( SID_AIR_INTENS ):
				{
					if ( nSID != SID_PAINTTOOL_AIR )
						rSet.DisableItem( SID_AIR_INTENS );
					else
						rSet.Put( SfxUInt16Item( SID_AIR_INTENS, ITEMVALUE( rAttrSet, ATTR_PNT_AIR_INTENS, SfxUInt16Item ) ) );
				}
				break;

				case( SID_COLOR_FORE ):
				{
					rSet.Put( XLineColorItem( String(),
							  ITEMVALUE( rAttrSet, ATTR_PNT_COLOR_FORE, SvxColorItem ) ),
							  SID_COLOR_FORE );
				}
				break;

				case( SID_COLOR_BACK ):
				{
					rSet.Put( XLineColorItem( String(),
							  ITEMVALUE( rAttrSet, ATTR_PNT_COLOR_BACK, SvxColorItem ) ),
							  SID_COLOR_BACK );
				}
				break;

				case( SID_LINE_TRANSPARENCY ):
				{
					if ( nSID == SID_PAINTTOOL_AIR )
						rSet.DisableItem( SID_LINE_TRANSPARENCY );
					else
						rSet.Put( SfxUInt16Item( SID_LINE_TRANSPARENCY, ITEMVALUE( rAttrSet, ATTR_PNT_LINE_TRANSPARENCY, SfxUInt16Item ) ) );
				}

				case( SID_FILL_TRANSPARENCY ):
				{
					if ( nSID == SID_PAINTTOOL_AIR )
						rSet.DisableItem( SID_FILL_TRANSPARENCY );
					else
						rSet.Put( SfxUInt16Item( SID_FILL_TRANSPARENCY, ITEMVALUE( rAttrSet, ATTR_PNT_FILL_TRANSPARENCY, SfxUInt16Item ) ) );
				}

				default:
				break;
			}

			nWhich = aIter.NextWhich();
		}
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ExecBmpMask( SfxRequest& rReq )
{
	switch ( rReq.GetSlot() )
	{
		case( SID_BMPMASK_PIPETTE ) :
			bPipette = ( (const SfxBoolItem&) ( rReq.GetArgs()->Get( SID_BMPMASK_PIPETTE ) ) ).GetValue();
		break;

		case( SID_BMPMASK_EXEC ) :
		{
			if( pView->GetCurFrame() )
			{
				AnimBmp aAnimBmp( pView->GetCurFrame()->GetAnimBmp() );

				if( !!( aAnimBmp.aBmpEx ) )
				{
					const String	aComment( SimResId( STR_PIPETTE ) );
					SfxUndoManager* pUndoMgr = GetViewFrame()->GetObjectShell()->GetUndoManager();
					SimUndoAction*	pUndoAction = new SimUndoAction( pView, aComment, 0, &aAnimBmp );
					SimModel*		pModel = pView->GetModel();

					aAnimBmp.aBmpEx = SVXBMPMASK()->Mask( aAnimBmp.aBmpEx ).GetBitmapEx();

					pView->ReplaceFrame( new SimFrame( pView->GetModel(), aAnimBmp ), pView->GetCurFramePos() );
					pModel->SetChanged( TRUE );
					pModel->IncModificationCount();
					pUndoMgr->Clear();
					pUndoMgr->AddUndoAction( pUndoAction );
					SFX_BINDINGS().InvalidateAll( TRUE );
				}
			}
		}
		break;

		default:
		break;
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetBmpMaskState( SfxItemSet& rSet )
{
	const USHORT nId = SvxBmpMaskChildWindow::GetChildWindowId();

	if ( GetViewFrame()->HasChildWindow( nId ) )
	{
		SvxBmpMask*	pDlg = (SvxBmpMask*) GetViewFrame()->GetChildWindow( SvxBmpMaskChildWindow::GetChildWindowId() )->GetWindow();

		if ( pDlg->NeedsColorTable() )
			pDlg->SetColorTable( pView->GetModel()->GetColorTable() );
	}

	rSet.Put( SfxBoolItem( SID_BMPMASK_EXEC, TRUE ) );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ExecGallery( SfxRequest& rReq )
{
	const SfxItemSet* pArgs = rReq.GetArgs();

	if ( pArgs )
	{
		GalleryExplorer* pGal = SVX_GALLERY();

		if ( pGal && ( ( (const SfxUInt32Item&) pArgs->Get( SID_GALLERY_FORMATS ) ).GetValue() & SGA_FORMAT_GRAPHIC ) )
		{
			Graphic aGraphic( pGal->GetGraphic() );
			( (SimDrawDocShell*) GetViewFrame()->GetObjectShell() )->ReplaceGraphic( &aGraphic );
		}
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetGalleryState( SfxItemSet& rSet )
{
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetPaintPixelState( SfxItemSet& rSet )
{
	BOOL bPaint = FALSE;

	if ( GetViewFrame()->HasChildWindow( SvxBmpMaskChildWindow::GetChildWindowId() ) )
		rSet.DisableItem( SID_TOOLBOX_PIXEL );
	else
	{
		rSet.Put( SfxBoolItem( SID_PIXEL_ROUND, FALSE ) );
		rSet.Put( SfxBoolItem( SID_PIXEL_SQUARE, FALSE ) );
		rSet.Put( SfxBoolItem( SID_PIXEL_VERT, FALSE ) );
		rSet.Put( SfxBoolItem( SID_PIXEL_HORZ, FALSE ) );
		rSet.Put( SfxBoolItem( SID_PIXEL_LEFT, FALSE ) );
		rSet.Put( SfxBoolItem( SID_PIXEL_RIGHT, FALSE ) );
		
		if( pFuActual )
		{
			const USHORT nSlotId = pFuActual->GetSlotID();

			if( SID_PIXEL_ROUND == nSlotId || SID_PIXEL_SQUARE == nSlotId ||
				SID_PIXEL_VERT == nSlotId || SID_PIXEL_HORZ == nSlotId ||
				SID_PIXEL_LEFT == nSlotId || SID_PIXEL_RIGHT == nSlotId )
			{
				rSet.Put( SfxBoolItem( nSlotId, TRUE ) );
				rSet.Put( SfxBoolItem( SID_TOOLBOX_PIXEL, TRUE ) );
			}
		}
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetMenuState( SfxItemSet &rSet )
{
    if ( pFuActual )
		rSet.Put( SfxBoolItem( pFuActual->GetSlotID(), TRUE ) );

    if( !Graphic::ClipboardHasFormat() )
        rSet.DisableItem( SID_PASTE );

    if( pView )
    {
        SvImage*			pImage = pView->GetImagePointer();
        const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );

        if( pImage )
		{
            const USHORT nActualDepth = pImage->GetDepth();

			if ( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_16_COLORS ) ||
				 SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_16_GRAYSCALES ) ||
				 SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_256_COLORS ) ||
				 SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_256_GRAYSCALES ) )
			{
				BitmapPalette*	pPal = pImage->GetPalette();
				BOOL			bGray = TRUE;

				for( USHORT n = 0, nCount = pPal->GetEntryCount(); n < nCount; n++ )
				{
					const BitmapColor& rColor = (*pPal)[ n ];

					if ( ( rColor.GetRed() != rColor.GetGreen() ) ||
						 ( rColor.GetRed() != rColor.GetBlue() ) )
					{
						bGray = FALSE;
						break;
					}
				}

				if ( nActualDepth == 4 )
				{
					if ( bGray )
						rSet.DisableItem( SID_16_GRAYSCALES );
					else
						rSet.DisableItem( SID_16_COLORS );
				}
				else if ( nActualDepth == 8 )
				{
					if ( bGray )
						rSet.DisableItem( SID_256_GRAYSCALES );
					else
						rSet.DisableItem( SID_256_COLORS );
				}
			}

			if( nActualDepth == 1 )
			{
                rSet.DisableItem(SID_BLACK_WHITE);
                rSet.DisableItem(SID_BLACK_WHITE_DIRECT);
			}
			else if ( nActualDepth == 24 )
				rSet.DisableItem(SID_16M_COLORS);

            // Menue-Punkt fuer Beschneidung disablen, falls keine Selektion vorhanden
            if ( aMarkRect.IsEmpty() )
            {
                rSet.DisableItem( SID_CROP );
                rSet.DisableItem( SID_SIZE_ALL );
            }
            else
            {
                rSet.DisableItem(SID_BLACK_WHITE);
                rSet.DisableItem(SID_BLACK_WHITE_DIRECT);
                rSet.DisableItem(SID_16_COLORS);
                rSet.DisableItem(SID_16_GRAYSCALES);
                rSet.DisableItem(SID_256_COLORS);
                rSet.DisableItem(SID_256_GRAYSCALES);
                rSet.DisableItem(SID_16M_COLORS);

                rSet.DisableItem(SID_ROTATE_RIGHT);
                rSet.DisableItem(SID_ROTATE_LEFT);
                rSet.DisableItem(SID_ROTATE_FREE);
                rSet.DisableItem(SID_USERDEFINED_ROTATE);

                rSet.DisableItem(SID_CHANGE_SIZE);
                rSet.DisableItem(SID_CHANGE_SIZE_FREE);
            }

            if ( pWindow->GetZoom() <= 10 )
                rSet.DisableItem( SID_ZOOM_IN );

            if ( pWindow->GetZoom() >= 3200 )
                rSet.DisableItem( SID_ZOOM_OUT );

			if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_REMOVE_SIMFRAME ) && pView->GetFrameCount() <= 1  )
				rSet.DisableItem( SID_REMOVE_SIMFRAME );

			if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_EDIT_FRAMEPROPERTIES ) && pView->GetFrameCount() < 2 )
				rSet.DisableItem( SID_EDIT_FRAMEPROPERTIES );
		}
        else
        {
            for ( USHORT i = SID_MIRROR_HORZ; i <= SID_16M_COLORS; i++ )
                rSet.DisableItem( i );

            rSet.DisableItem(SID_COPY);
            rSet.DisableItem(SID_GRAYSCALES);
            rSet.DisableItem(SID_ROTATE_FREE);
            rSet.DisableItem(SID_CHANGE_SIZE_FREE);
            rSet.DisableItem(SID_RELIEF);
        }
    }

	const BOOL bBmpMask = GetViewFrame()->HasChildWindow( SvxBmpMaskChildWindow::GetChildWindowId() );

	// Pipette
	if ( rSet.GetItemState( SID_BMPMASK ) == SFX_ITEM_AVAILABLE )
		rSet.Put( SfxBoolItem( SID_BMPMASK, bBmpMask ) );

	if ( bBmpMask )
	{
		rSet.DisableItem( SID_PAINTTOOL_LINE );
		rSet.DisableItem( SID_PAINTTOOL_RECT );
		rSet.DisableItem( SID_PAINTTOOL_RECT_NULL );
		rSet.DisableItem( SID_PAINTTOOL_CIRCLE );
		rSet.DisableItem( SID_PAINTTOOL_CIRCLE_NULL );
		rSet.DisableItem( SID_PAINTTOOL_AIR );
	}

	// Farbleiste
	const USHORT	nId = SvxColorChildWindow::GetChildWindowId();
	const BOOL		bVisible = GetViewFrame()->HasChildWindow( nId );

	// 'richtige' Slots am Controller setzen
	if( bVisible )
	{
		( (SvxColorDockingWindow*) GetViewFrame()->GetChildWindow( nId )->GetWindow() )->
			SetSlotIDs( SID_COLORBAR_BACK, SID_COLORBAR_FORE );
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_COLOR_CONTROL ) )
		rSet.Put( SfxBoolItem( SID_COLOR_CONTROL, bVisible ) );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ExecStatusBar(SfxRequest& rReq)
{
    const SfxItemSet* pArgs = rReq.GetArgs();

    switch ( rReq.GetSlot() )
    {
        case SID_ATTR_ZOOM:
            SFX_DISPATCHER().Execute( SID_ATTR_ZOOM, SFX_CALLMODE_ASYNCHRON );
        break;

        default:
        break;
    }
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::GetStatusBarState( SfxItemSet& rSet )
{
	String				aString;
	SvxZoomItem			aZoom( SVX_ZOOM_PERCENT, (USHORT) pWindow->GetZoom() );
	const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
	const SfxItemSet&	rAttrSet = pView->GetAttrSet();
	const Color&		rForeCol = ITEMVALUE( rAttrSet, ATTR_PNT_COLOR_FORE, SvxColorItem );
	const Color&		rBackCol = ITEMVALUE( rAttrSet, ATTR_PNT_COLOR_BACK, SvxColorItem );
	Size				aSizePix( pView->GetSizePixel() );
	USHORT				nColorDepth = pView->GetColorDepth();

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_STATUS_PIC_WH ) )
	{
		aString.Assign( String::CreateFromInt32( aSizePix.Width() ) );
		aString.Append( String( RTL_CONSTASCII_USTRINGPARAM( " x " ) ) );
		aString.Append( String::CreateFromInt32( aSizePix.Height() ) );
		rSet.Put( SfxStringItem( SID_STATUS_PIC_WH, aString ) );
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_STATUS_PIC_DEPTH ) )
	{
		aString.Assign( String::CreateFromInt32( nColorDepth ) );
		aString.Append( String( RTL_CONSTASCII_USTRINGPARAM( " Bit" ) ) );
		rSet.Put( SfxStringItem( SID_STATUS_PIC_DEPTH, aString ) );
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_STATUS_PIC_SIZE ) )
	{
		aString.Assign( String::CreateFromInt32( ( aSizePix.Width() * aSizePix.Height() * nColorDepth ) >> 13 ) );
		aString.Append( String( RTL_CONSTASCII_USTRINGPARAM( " KB " ) ) );
		rSet.Put( SfxStringItem( SID_STATUS_PIC_SIZE, aString ) );
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_STATUS_MOUSEPOS ) )
	{
		const SimMouseInfo& rMouseInfo = GetMouseInfo();

		if( ( -1 == rMouseInfo.maMousePos.X() ) || ( -1 == rMouseInfo.maMousePos.Y() ) )
			aString.Assign( String( RTL_CONSTASCII_USTRINGPARAM(  "- / -" ) ) );
		else
		{
			aString.Assign( String::CreateFromInt32( rMouseInfo.maMousePos.X() ) );
			aString.Append( String( RTL_CONSTASCII_USTRINGPARAM( " / " ) ) );
			aString.Append( String::CreateFromInt32( rMouseInfo.maMousePos.Y() ) );
		}

		rSet.Put( SfxStringItem( SID_STATUS_MOUSEPOS, aString ) );
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_ATTR_POSITION ) ||
		SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_ATTR_SIZE ) )
	{
		const Rectangle& rSelRect = GetMouseInfo().maSelRect;

		if( !rSelRect.IsEmpty() )
		{
			rSet.Put( SfxPointItem( SID_ATTR_POSITION, rSelRect.TopLeft() ) );
			rSet.Put( SvxSizeItem( SID_ATTR_SIZE, rSelRect.GetSize() ) );
		}
		else
		{
			rSet.Put( SfxPointItem( SID_ATTR_POSITION, Point() ) );
			rSet.Put( SvxSizeItem( SID_ATTR_SIZE, Size() ) );
		}
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_STATUS_FRAME ) )
	{
		aString.Assign( String( SimResId( STR_FRAMENAME ) ) );
		aString.Append( String( RTL_CONSTASCII_USTRINGPARAM( " " ) ) );
		aString.Append( String::CreateFromInt32( pView->GetCurFramePos() + 1 ) );
		aString.Append( String( RTL_CONSTASCII_USTRINGPARAM( " / " ) ) );
		aString.Append( String::CreateFromInt32( pView->GetFrameCount() ) );
		rSet.Put( SfxStringItem( SID_STATUS_FRAME, aString ) );
	}

	if( SFX_ITEM_AVAILABLE == rSet.GetItemState( SID_COLOR_STATUS ) )
	{
		rSet.Put( SimColorItem( rForeCol, rBackCol, SID_COLOR_STATUS ) );
	}

	aZoom.SetValueSet( SVX_ZOOM_ENABLE_50  | SVX_ZOOM_ENABLE_75  | SVX_ZOOM_ENABLE_100 | SVX_ZOOM_ENABLE_150 |
					   SVX_ZOOM_ENABLE_200 | SVX_ZOOM_ENABLE_WHOLEPAGE |
					   ( pView->GetSelectionBoundRect().IsEmpty() ? 0 : SVX_ZOOM_ENABLE_OPTIMAL ) );
	rSet.Put( aZoom );
}
