/*************************************************************************
 *
 *  $RCSfile: drviews3.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <sfx2/bindings.hxx>
#include <sfx2/dispatch.hxx>
#include <sfx2/viewfrm.hxx>
#include <svx/bmpmask.hxx>
#include <svx/colrctrl.hxx>
#include <svx/svxids.hrc>
#include "simmodel.hxx"
#include "simmod.hxx"
#include "simresid.hxx"
#include "simapp.hrc"
#include "strings.hrc"
#include "fuscale.hxx"
#include "fusel.hxx"
#include "fuprint.hxx"
#include "fumod.hxx"
#include "fugeo.hxx"
#include "fupaint.hxx"
#include "fuprint.hxx"
#include "simview.hxx"
#include "simwin.hxx"
#include "svimage.hxx"
#include "drviewsh.hxx"

#define WAIT_ON()	if( GetActiveWindow() ) GetActiveWindow()->EnterWait()
#define WAIT_OFF()	if( GetActiveWindow() ) GetActiveWindow()->LeaveWait()

// --------------------
// - SimDrawViewShell -
// --------------------

void SimDrawViewShell::FuPermanent( SfxRequest& rReq )
{
	BOOL bNew = TRUE;

	if( pFuActual )
	{
		if( pFuActual->GetSlotID() == rReq.GetSlot() )
			bNew = FALSE;
		else
		{
			if( pFuOld == pFuActual )
				pFuOld = NULL;

			pFuActual->Deactivate();
			delete pFuActual; pFuActual = NULL;
		}
	}

	if( bNew )
	{
		SimModel* pModel = pView->GetModel();

		switch( rReq.GetSlot() )
		{
			case SID_OBJECT_SELECT:
				pFuActual = new SimFuSelect( this, pWindow, pView, pModel, rReq );
			break;

			case SID_PAINTTOOL_LINE:
			case SID_PAINTTOOL_RECT:
			case SID_PAINTTOOL_RECT_NULL:
			case SID_PAINTTOOL_CIRCLE:
			case SID_PAINTTOOL_CIRCLE_NULL:
				pFuActual = new SimFuGeo( this, pWindow, pView, pModel, rReq );
			break;

			case SID_PIXEL_ROUND:
			case SID_PIXEL_SQUARE:
			case SID_PIXEL_VERT:
			case SID_PIXEL_HORZ:
			case SID_PIXEL_LEFT:
			case SID_PIXEL_RIGHT:
				pFuActual = new SimFuPaintPixel( this, pWindow, pView, pModel, rReq );
			break;

			case SID_PAINTTOOL_AIR:
				pFuActual = new SimFuPaintAir( this, pWindow, pView, pModel, rReq );
			break;

			default:
			break;
		}

		if( pFuOld )
		{
			pFuOld->Deactivate();
			delete pFuOld; pFuOld = NULL;
		}

		if( pFuActual )
		{
			pFuActual->Activate();
			pFuOld = pFuActual;
		}
	}

	Invalidate();
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::FuSupport(SfxRequest &rReq)
{
	WAIT_ON();

    switch ( rReq.GetSlot() )
    {
        case SID_COPY:
            pView->Copy();
		break;


        case SID_PASTE:
            pView->Paste();
		break;

        default:
        break;
    }

	WAIT_OFF();

	Invalidate();
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::FuAdmin( SfxRequest &rReq )
{
	SimModel*	pModel = pView->GetModel();
	BOOL		bCancel = TRUE;

	if( pFuActual )
	{
		pFuActual->Deactivate();

		if( pFuActual != pFuOld )
			delete pFuActual;

		pFuActual = NULL;
	}

	switch( rReq.GetSlot() )
	{
		case SID_ATTR_ZOOM:
			pFuActual = new SimFuScale( this, pWindow, pView, pModel, rReq );
		break;

		case SID_ZOOM_OUT:
		{
			pFuActual = new SimFuZoom( this, pWindow, pView, pModel, rReq );
			bCancel = FALSE;
		}
		break;

		case SID_ZOOM_IN:
			SetZoom( Max( (long) ( pWindow->GetZoom() >> 1 ), (long) 10 ) );
		break;

		case SID_SIZE_REAL:
			SetZoom( 100 );
		break;

		case SID_SIZE_PAGE:
			SetZoomRect( pView->GetRectPixel() );
		break;

		case SID_SIZE_ALL:
			SetZoomRect( pView->GetSelectionBoundRect() );
		break;

		case SID_SIM_PRINTOPTIONS:
			pFuActual = new SimFuPrint( this, pWindow, pView, pModel, rReq, pView->GetSizePixel() );
		break;

		case SID_BMPMASK:
		{
			GetViewFrame()->ToggleChildWindow( SvxBmpMaskChildWindow::GetChildWindowId() );
			SFX_BINDINGS().Invalidate( SID_BMPMASK );
			SFX_DISPATCHER().Execute( SID_OBJECT_SELECT, SFX_CALLMODE_ASYNCHRON );
		}
		break;

		case SID_COLOR_CONTROL:
		{
			const SfxItemSet*	pArgs = rReq.GetArgs();
			const USHORT		nId = SvxColorChildWindow::GetChildWindowId();

			if ( pArgs )
				GetViewFrame()->SetChildWindow( nId, ( (const SfxBoolItem&) ( pArgs->Get( SID_COLOR_CONTROL ) ) ).GetValue() );
			else
				GetViewFrame()->ToggleChildWindow( nId );

			if ( GetViewFrame()->HasChildWindow( nId ) )
			{
				SvxColorDockingWindow* pWin = ( SvxColorDockingWindow* ) GetViewFrame()->GetChildWindow( nId )->GetWindow();
				pWin->SetSlotIDs( SID_COLORBAR_BACK, SID_COLORBAR_FORE );
			}

			SFX_BINDINGS().Invalidate( SID_COLOR_CONTROL );
		}
		break;

		case SID_INSERT_SIMFRAME:
			InsertFrame();
		break;

		case SID_REMOVE_SIMFRAME:
			RemoveFrame( pView->GetCurFramePos() );
		break;

		case SID_EDIT_FRAMEPROPERTIES:
			EditFrameProperties();
		break;

		default:
		break;	
	}

	if( bCancel )
		Cancel();

	if( pFuActual )
		pFuActual->Activate();

	Invalidate();
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::FuModify( SfxRequest &rReq )
{
	Size				aOldSize;
	Application*		pApp = GetpApp();
	SimModel* 			pModel = pView->GetModel();
	SimDefaults*		pDefaults = pModel->GetDefaults();
	SvImage*			pImage = pView->GetImagePointer();
	SvImage*			pAlpha = pView->GetAlphaPointer();	
	const Rectangle		aMarkRect( pView->GetSelectionBoundRect() );
	SfxViewFrame*		pFrame = GetViewFrame();
	SimDrawDocShell*	pShell = (SimDrawDocShell*) pFrame->GetObjectShell();
	USHORT				nComment;
	BOOL				bMarkEmpty = aMarkRect.IsEmpty();
	BOOL				bNoErr = TRUE;
	BOOL				bRet = FALSE;
	BOOL				bCancel = TRUE;

	if( pFuActual )
	{
		pFuActual->Deactivate();

		if( pFuActual != pFuOld )
			delete pFuActual;

		pFuActual = NULL;
	}

	if( pImage )
	{
		aOldSize = pView->GetSizePixel();
		
		WAIT_ON();

		switch( rReq.GetSlot() )
		{
			case SID_MIRROR_HORZ:
			{
				if( bMarkEmpty )
				{
					bNoErr = bRet = pImage->Mirror( TRUE, pFrame );

					if ( bRet && pAlpha )
						pAlpha->Mirror( TRUE, pFrame );
				}
				else
				{
					bNoErr = bRet = pImage->Mirror( TRUE, aMarkRect, pFrame );

					if( bRet && pAlpha )
						pAlpha->Mirror( TRUE, aMarkRect, pFrame );
				}

				if( bRet )
					nComment = STR_MIRROR_HORZ;
			}
			break;

			case SID_MIRROR_VERT:
			{
				if ( bMarkEmpty )
				{
					bNoErr = bRet = pImage->Mirror( FALSE, pFrame );

					if ( bRet && pAlpha )
						pAlpha->Mirror( FALSE, pFrame );
				}
				else
				{
					bNoErr = bRet = pImage->Mirror( FALSE, aMarkRect, pFrame );

					if( bRet && pAlpha )
						pAlpha->Mirror( FALSE, aMarkRect, pFrame );
				}

				if( bRet )
					nComment = STR_MIRROR_VERT;
			}
			break;

			case SID_ROTATE_RIGHT:
			{
				bNoErr = bRet = pImage->Rotate( -90.F, Color(COL_WHITE), pFrame);

				if( bRet )
				{
					if ( pAlpha )
						pAlpha->Rotate( -90.F, Color(COL_WHITE), pFrame );

					nComment = STR_ROTATE_LEFT;
				}
			}
			break;

			case SID_ROTATE_LEFT:
			{
				bNoErr = bRet = pImage->Rotate( 90.F, Color(COL_WHITE), pFrame);

				if( bRet )
				{
					if( pAlpha )
						pAlpha->Rotate( 90.F, Color(COL_WHITE), pFrame );

					nComment = STR_ROTATE_RIGHT;
				}
			}
			break;

			case SID_USERDEFINED_ROTATE:
			{
				float fAngle = 	pDefaults->fRotation * ( pDefaults->bRotOrient ? -1.F : 1.F );
				
				bNoErr = bRet = pImage->Rotate(fAngle, Color(COL_WHITE), pFrame);

				if( bRet )
				{
					if( pAlpha )
						pAlpha->Rotate( fAngle, Color(COL_WHITE), pFrame );

					nComment = STR_USERDEFINED_ROTATE;
				}
			}
			break;

			case SID_ROTATE_FREE:
			{
				WAIT_OFF();
				pFuActual = new SimFuRotate(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

		        bRet = pFuActual->GetDlgRet();

				if( bRet )
					nComment = STR_USERDEFINED_ROTATE;
			}
			break;

			case SID_MORE_RED:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->AdjustRGB(+(pDefaults->fRedAdjust), 0.F, 0.F, pFrame);
				else
				{
					bNoErr = bRet = pImage->AdjustRGB(+(pDefaults->fRedAdjust), 0.F, 0.F, aMarkRect, pFrame);
					pImage->SetDepth( 24 );
				}

				if( bRet )
					nComment = STR_MORE_RED;
			}
			break;

			case SID_LESS_RED:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->AdjustRGB(-(pDefaults->fRedAdjust), 0.F, 0.F, pFrame);
				else
				{
					bNoErr = bRet = pImage->AdjustRGB(-(pDefaults->fRedAdjust), 0.F, 0.F, aMarkRect, pFrame);
					pImage->SetDepth( 24 );
				}

				if( bRet )
					nComment = STR_LESS_RED;
			}
			break;

			case SID_MORE_BLUE:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->AdjustRGB(0.F, 0.F, +(pDefaults->fBlueAdjust), pFrame);
				else
				{
					bNoErr = bRet = pImage->AdjustRGB(0.F, 0.F, +(pDefaults->fBlueAdjust), aMarkRect, pFrame);
					pImage->SetDepth( 24 );
				}

				if( bRet )
					nComment = STR_MORE_BLUE;
			}
			break;

			case SID_LESS_BLUE:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->AdjustRGB(0.F, 0.F, -(pDefaults->fBlueAdjust), pFrame);
				else
				{
					bNoErr = bRet = pImage->AdjustRGB(0.F, 0.F, -(pDefaults->fBlueAdjust), aMarkRect, pFrame);
					pImage->SetDepth( 24 );
				}

				if( bRet )
					nComment = STR_LESS_BLUE;
			}
			break;

			case SID_MORE_GREEN:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->AdjustRGB(0.F, +(pDefaults->fGreenAdjust), 0.F, pFrame);
				else
				{
					bNoErr = bRet = pImage->AdjustRGB(0.F, +(pDefaults->fGreenAdjust), 0.F, aMarkRect, pFrame);
					pImage->SetDepth( 24 );
				}

				if( bRet )
					nComment = STR_MORE_GREEN;
			}
			break;

			case SID_LESS_GREEN:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->AdjustRGB(0.F, -(pDefaults->fGreenAdjust), 0.F, pFrame);
				else
				{
					bNoErr = bRet = pImage->AdjustRGB(0.F, -(pDefaults->fGreenAdjust), 0.F, aMarkRect, pFrame);
					pImage->SetDepth( 24 );
				}

				if( bRet )
					nComment = STR_LESS_GREEN;
			}
			break;

			case SID_RGB_COLORS:
			{
				WAIT_OFF();
				pFuActual = new SimFuRgb(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_RGB_COLORS;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_MORE_BRIGHTNESS:
			{
				bNoErr = bRet = pImage->AdjustLu_Co( pDefaults->fBrightness, 0.0, aMarkRect, pFrame );

				if( bRet )
				{
					nComment = STR_MORE_BRIGHTNESS;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_LESS_BRIGHTNESS:
			{
				bNoErr = bRet = pImage->AdjustLu_Co( -pDefaults->fBrightness, 0.0, aMarkRect, pFrame );

				if( bRet )
				{
					nComment = STR_LESS_BRIGHTNESS;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_MORE_CONTRAST:
			{
				bNoErr = bRet = pImage->AdjustLu_Co( 0.0, pDefaults->fContrast, aMarkRect, pFrame );

				if( bRet )
				{
					nComment = STR_MORE_CONTRAST;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_LESS_CONTRAST:
			{
				bNoErr = bRet = pImage->AdjustLu_Co( 0.0, -pDefaults->fContrast, aMarkRect, pFrame );

				if( bRet )
				{
					nComment = STR_LESS_CONTRAST;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_BRIGHTNESS:
			{
				WAIT_OFF();
				pFuActual = new SimFuContrast(this, pWindow, pView, pModel, rReq );
				WAIT_ON();

				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_BRIGHTNESS;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_INVERT:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->Invert(pFrame);
				else
					bNoErr = bRet = pImage->Invert( aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_INVERT;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_GRAYSCALES:
			{
				WAIT_OFF();
				pFuActual = new SimFuGray(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

				pImage->SetDepth( 8 );

				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_GRAYSCALES_DIRECT;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_GRAYSCALES_DIRECT:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->Grayscale(pDefaults->nGrayCount, pFrame);
				else
					bNoErr = bRet = pImage->Grayscale(pDefaults->nGrayCount, aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_GRAYSCALES_DIRECT;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( pDefaults->nGrayCount <  3 ? 1 : pDefaults->nGrayCount > 16 ? 8 : 4 );
				}
			}
			break;

			case SID_CHARCOAL_DRAWING:
			{
				bNoErr = bRet = pImage->Charcoal(aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_CHARCOAL_DRAWING;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_MOSAIC:
			{
				WAIT_OFF();
				pFuActual = new SimFuMosaic(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_MOSAIC_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_MOSAIC_DIRECT:
			{
				bNoErr = bRet = pImage->Mosaic(Size(pDefaults->nMosaicX, pDefaults->nMosaicY),
								aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_MOSAIC_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_RELIEF:
			{
				WAIT_OFF();
				pFuActual = new SimFuRelief(this, pWindow, pView, pModel, rReq);
				WAIT_ON();
				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_RELIEF;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_RELIEF_DIRECT:
			{
				bNoErr = bRet = pImage->Emboss( 45.F, 45.F, 4, aMarkRect, pFrame );

				if( bRet )
				{
					nComment = STR_RELIEF;
				
					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_POSTER:
			{
				WAIT_OFF();
				pFuActual = new SimFuPoster( this, pWindow, pView, pModel, rReq );
				WAIT_ON();
				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_POSTER_DIRECT;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_POSTER_DIRECT:
			{
				bNoErr = bRet = pImage->Poster( pDefaults->nPosterCount, aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_POSTER_DIRECT;
				
					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_POPART:
			{
				bNoErr = bRet = pImage->Popart(aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_POPART;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
					else
						pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_AGE:
			{
				WAIT_OFF();
				pFuActual = new SimFuAge(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

		        bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_AGE_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_AGE_DIRECT:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->Sepia(pDefaults->fAge, pFrame);
				else
					bNoErr = bRet = pImage->Sepia(pDefaults->fAge, aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_AGE_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_SOLARIZATION:
			{
				WAIT_OFF();
				pFuActual = new SimFuSolar(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

		        bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_SOLARIZATION_DIRECT;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_SOLARIZATION_DIRECT:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->Solarize(pDefaults->fSolarThreshold, !pDefaults->bSolarInvert, pFrame);
				else
					bNoErr = bRet = pImage->Solarize(pDefaults->fSolarThreshold, !pDefaults->bSolarInvert, aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_SOLARIZATION_DIRECT;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_TILE:
			{
				WAIT_OFF();
				pFuActual = new SimFuTile(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

		        bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_TILE_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_TILE_DIRECT:
			{
				if( bMarkEmpty )
					bNoErr = bRet = pImage->Pixellize(Size(pDefaults->nTileX, pDefaults->nTileY), pFrame);
				else
					bNoErr = bRet = pImage->Pixellize( Size(pDefaults->nTileX, pDefaults->nTileY), aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_TILE_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_GAMMA_CORRECTION:
			{
				WAIT_OFF();
				pFuActual = new SimFuGamma(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

		        bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_GAMMA_CORRECTION;

					if( !bMarkEmpty )
						pImage->SetDepth( 24 );
				}
			}

			break;

			case SID_BLACK_WHITE_DIRECT:
			case SID_BLACK_WHITE:
			{
				bNoErr = bRet = pImage->DitherRGBToMono(pFrame);

				if( bRet )
				{
					nComment = STR_BLACK_WHITE_DIRECT;
					pImage->SetDepth( 1 );
				}
			}
			break;

			case SID_16_GRAYSCALES:
			{
				bNoErr = bRet = pImage->DitherRGBTo16Gray(pFrame);

				if( bRet )
				{
					nComment = STR_16_GRAYSCALES;
					pImage->SetDepth( 4 );
				}
			}
			break;

			case SID_16_COLORS:
			{
				bNoErr = bRet = pImage->DitherRGBTo16(pFrame);

				if( bRet )
				{
					nComment = STR_16_COLORS;
					pImage->SetDepth( 4 );
				}
			}
			break;

			case SID_256_GRAYSCALES:
			{
				bNoErr = bRet = pImage->Grayscale(256, pFrame);

				if( bRet )
				{
					nComment = STR_256_GRAYSCALES;
					pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_256_COLORS:
			{
				bNoErr = bRet = pImage->DitherRGBTo256(pFrame);

				if( bRet )
				{
					nComment = STR_256_COLORS;
					pImage->SetDepth( 8 );
				}
			}
			break;

			case SID_16M_COLORS:
			{
				bRet = TRUE;
				nComment = STR_16M_COLORS;
				pImage->SetDepth( 24 );
			}
			break;

			case SID_SMOOTH:
			case SID_SMOOTH_DIRECT:
			{
				bNoErr = bRet = pImage->Smooth(1, aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_SMOOTH_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_DEFINITION:
			{
				WAIT_OFF();
				pFuActual = new SimFuSharp(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

				bRet = pFuActual->GetDlgRet();

				if( bRet )
				{
					nComment = STR_DEFINITION_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_DEFINITION_DIRECT:
			{
				BYTE cIntens = 	pDefaults->eSharpness == DEFINITION_LOW ? 23 :
					 			pDefaults->eSharpness == DEFINITION_MEDIUM ? 13 : 9;

				bNoErr = bRet = pImage->Sharpen(cIntens, aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_DEFINITION_DIRECT;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_REMOVE_NOISE:
			{
				bNoErr = bRet = pImage->RemoveNoise(aMarkRect, pFrame);

				if( bRet )
				{
					nComment = STR_REMOVE_NOISE;
					pImage->SetDepth( 24 );
				}
			}
			break;

			case SID_CROP:
			{
				bNoErr = bRet = pImage->Crop( aMarkRect.TopLeft(), aMarkRect.GetSize(), pFrame );

				if( bRet && pAlpha )
					pAlpha->Crop( aMarkRect.TopLeft(), aMarkRect.GetSize(), pFrame );

				if( bRet )
					nComment = STR_CROP;
			}
			break;

			case SID_CHANGE_SIZE_FREE:
			case SID_CHANGE_SIZE:
			{
				WAIT_OFF();
				pFuActual = new SimFuResize(this, pWindow, pView, pModel, rReq);
				WAIT_ON();

				bRet = pFuActual->GetDlgRet();

				if( bRet )
					nComment = STR_CHANGE_SIZE;
			}
			break;

			default:
			break;
		}

		WAIT_OFF();

		if( bRet )
		{
			const BOOL bSizeChanged = ( pView->GetSizePixel() != aOldSize );

			if( bSizeChanged )
				pView->ClearSelection();

			pModel->SendRepaintBroadcast( pView->GetCurFramePos(), bSizeChanged );
			pModel->SetChanged( TRUE );
			pModel->IncModificationCount();
			CreateUndoAction( String( SimResId( nComment ) ), rReq.GetSlot() );
		}
	}

	if( bCancel )
		Cancel();

	if( pFuActual )
		pFuActual->Activate();

	if( !bNoErr )
		HandleSimError( SIM_ERROR_OUTOFMEMORY );

	Invalidate();
}
