/*************************************************************************
 *
 *  $RCSfile: drviewsh.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_COLOR 0

#include <tools/multisel.hxx>
#include <vcl/clip.hxx>
#include <vcl/split.hxx>
#include <vcl/wrkwin.hxx>
#include <svtools/printdlg.hxx>
#include <svtools/taskbar.hxx>
#include <svtools/stritem.hxx>
#include <sfx2/bindings.hxx>
#include <sfx2/dispatch.hxx>
#include <sfx2/viewfac.hxx>
#include <sfx2/msg.hxx>
#include <sfx2/printer.hxx>
#include <sfx2/objface.hxx>
#include <sfx2/docfile.hxx>
#include <svx/bmpmask.hxx>
#include <svx/colrctrl.hxx>
#include <svx/svxids.hrc>
#include "simmodel.hxx"
#include "simwin.hxx"
#include "simattr.hxx"
#include "simresid.hxx"
#include "simapp.hrc"
#include "strings.hrc"
#include "svimage.hxx"
#include "newdlg.hxx"
#include "simmod.hxx"
#include "helpids.hrc"
#include "simview.hxx"
#include "docshell.hxx"
#include "tabctrl.hxx"
#include "frmprop.hxx"
#include "drviewsh.hxx"

#define SimDrawViewShell
#include "simslots.hxx"

// -----------
// - Defines -
// -----------

#define PIPETTE_RANGE 0

// -----------
// - Statics -
// -----------

BOOL SimDrawViewShell::bPipette = FALSE;

// -------------
// - Interface -
// -------------

SFX_IMPL_INTERFACE( SimDrawViewShell, SfxViewShell, SimResId( STR_DRAWVIEWSHELL ) )
{
	SFX_OBJECTBAR_REGISTRATION( SFX_OBJECTBAR_OBJECT | SFX_VISIBILITY_STANDARD | SFX_VISIBILITY_SERVER,
								SimResId( RID_OBJ_TOOLBOX ) );

	SFX_OBJECTBAR_REGISTRATION( SFX_OBJECTBAR_TOOLS | SFX_VISIBILITY_STANDARD |
								SFX_VISIBILITY_FULLSCREEN | SFX_VISIBILITY_SERVER,
								SimResId( RID_PAINT_TOOLBOX ) );

	SFX_CHILDWINDOW_REGISTRATION( SvxBmpMaskChildWindow::GetChildWindowId() );
	SFX_CHILDWINDOW_REGISTRATION( SvxColorChildWindow::GetChildWindowId() );
}

// ---------------
// - ViewFactory -
// ---------------

SFX_IMPL_VIEWFACTORY( SimDrawViewShell, SimResId( STR_DEFAULTVIEW ) )
{
    SFX_VIEW_REGISTRATION( SimDrawDocShell );
}

// --------------------
// - SimDrawViewShell -
// --------------------

TYPEINIT1( SimDrawViewShell, SimViewShell );

// -----------------------------------------------------------------------------

SimDrawViewShell::SimDrawViewShell( SfxViewFrame* pFrame, Window *pParent ) :
		    SimViewShell	( pFrame, pParent )
{
    ImplConstruct();
}

// -----------------------------------------------------------------------------

SimDrawViewShell::SimDrawViewShell( SfxViewFrame* pFrame, SfxViewShell *pOldShell ) :
		    SimViewShell	( pFrame, &pFrame->GetWindow() )
{
    ImplConstruct();
}

// -----------------------------------------------------------------------------

SimDrawViewShell::SimDrawViewShell( SfxViewFrame* pFrame, const SimDrawViewShell& rShell) :
			SimViewShell	( pFrame, rShell )
{
    ImplConstruct();
}

// -----------------------------------------------------------------------------

SimDrawViewShell::~SimDrawViewShell()
{
	SimDrawDocShell* pObjShell = GetObjectShell();

	pObjShell->Disconnect( this );
    EndListening( *pObjShell );
    bObjectBarSwitchEnabled = FALSE;

    if( pFuActual )
    {
        if( pFuOld == pFuActual )
            pFuOld = NULL;

        pFuActual->Deactivate();
        delete pFuActual, pFuActual = NULL;
    }

	if( pFuOld )
		delete pFuOld, pFuOld = NULL;

	delete pView, pView = NULL;
    delete pTabControl;
	delete pNewDocTimer;
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ImplConstruct()
{
    const Point			aOrg;
	SimDrawDocShell*	pDocSh = GetObjectShell();
	SfxItemSet*			pSet = pDocSh->GetMedium()->GetItemSet();
    SimModel* 			pModel = pDocSh->GetModel();
	SfxItemPool*		pPool = pModel->GetPool();
    SfxRequest			aReq( SID_OBJECT_SELECT, 0, *pPool );

    StartListening( *pDocSh );
	SetUndoManager( pDocSh->GetUndoManager() );
	SetRepeatTarget( NULL );
    SetPool( pPool );

	// no timer initially
	pNewDocTimer = NULL;

	// init additional controls
	pTabControl = new SimTabControl( &GetViewFrame()->GetWindow(), this );
	pTabControl->SetSplitHdl( LINK( this, SimDrawViewShell, TabControlSplitHdl ) );
	pTabControl->SetSizePixel( Size() );
	pTabControl->Show();

    pView = new SimView( pDocSh, pModel, this, pWindow );

	pDocSh->Connect( this );
	SetHelpId( HID_SIM_VIEWSHELL_DOCUMENT );
	SetName( String( RTL_CONSTASCII_USTRINGPARAM( "SimViewShell" ) ) );

	SFX_BINDINGS().InvalidateAll( TRUE );

    FuPermanent( aReq );

	if( pView->GetFrameCount() )
		pView->SwitchToFrame( 0 );
	else
	{
		// open doc silently?
		if( pSet && ( SFX_ITEM_SET == pSet->GetItemState( SID_SILENT ) ) && 
			( (SfxBoolItem&)( pSet->Get( SID_SILENT ) ) ).GetValue() )
		{
			Bitmap aBmp( Size( 256, 256 ), 24 ); 
			
			aBmp.Erase( Color( COL_WHITE ) );
			Graphic aGraphic( aBmp );
			GetObjectShell()->ReplaceGraphic( &aGraphic );
		}
		else
		{
			pNewDocTimer = new Timer;
			pNewDocTimer->SetTimeoutHdl( LINK( this, SimDrawViewShell, TimerHdl ) );
			pNewDocTimer->SetTimeout( 100 );
			pNewDocTimer->Start();
		}
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::ImplPrintFrame( const SimFrame& rFrame, Printer* pPrinter )
{
	const Graphic aGraphic( rFrame.GetBitmapEx() );

	if( aGraphic.GetType() != GRAPHIC_NONE )
	{
        Size				aPrintSize;
		const Size			aFrameSizePix( rFrame.GetSizePixel() );
		const SfxItemSet&	rPrintAttr = rFrame.GetPrintAttr();
		Point				aPrintOrg( pPrinter->PixelToLogic( Point() ) );
        const Rectangle		aRect( aPrintOrg, pPrinter->GetOutputSize() );
		const USHORT		nPrintDPI = ( (const SfxUInt16Item&) rPrintAttr.Get( ATTR_PRINTOPT_DPI ) ).GetValue();

		// original size (==DPI_ORIGINAL)
		if( rFrame.GetPrefMapMode().GetMapUnit() == MAP_PIXEL )
			aPrintSize = GetViewFrame()->GetWindow().PixelToLogic( aFrameSizePix, MAP_100TH_MM );
		else
			aPrintSize = OutputDevice::LogicToLogic( rFrame.GetPrefSize(), rFrame.GetPrefMapMode(), MAP_100TH_MM );

		switch( nPrintDPI )
		{
			case( DPI_PAGESIZE ):
			{
				aPrintSize.Width() = aRect.GetWidth();
				aPrintSize.Height() = aRect.GetHeight();
			}
			break;

			case( DPI_PAGESIZE_SCALED ):
			{
				double fXY = (double) aPrintSize.Width() / aPrintSize.Height();
				double fPageXY = (double) aRect.GetWidth() / aRect.GetHeight();

				if( fPageXY < fXY )
				{
					aPrintSize.Width() = aRect.GetWidth();
					aPrintSize.Height()= (long) ( aRect.GetWidth() / fXY );
				}
				else
				{
					aPrintSize.Width() = (long)( aRect.GetHeight() * fXY );
					aPrintSize.Height()= aRect.GetHeight();
				}
			}
			break;

			case( DPI_FREEHAND ):
			{
				aPrintSize.Width() = ( (const SfxUInt32Item&) rPrintAttr.Get( ATTR_PRINTOPT_WIDTH ) ).GetValue();
				aPrintSize.Height() = ( (const SfxUInt32Item&) rPrintAttr.Get( ATTR_PRINTOPT_HEIGHT ) ).GetValue();
			}
			break;

			case( DPI_ORIGINAL ):
			break;

			default:
			{
				aPrintSize.Width() = aFrameSizePix.Width() * 2540 / nPrintDPI;
				aPrintSize.Height() = aFrameSizePix.Height() * 2540 / nPrintDPI;
			}
			break;
		}

		const long	nPageWidth = aRect.GetWidth(), nPageHeight = aRect.GetHeight();
		const long	nPrintWidth = aPrintSize.Width(), nPrintHeight = aPrintSize.Height();

        switch( (RECT_POINT) ((const SfxUInt16Item&) rPrintAttr.Get(ATTR_PRINTOPT_POSITION)).GetValue() )
        {
            case( RP_LT ):
			break;

            case( RP_MT ):
                aPrintOrg.X() += ( nPageWidth - nPrintWidth ) >> 1;
            break;

            case( RP_RT ):
                aPrintOrg.X() += nPageWidth - nPrintWidth;
            break;

            case( RP_LM ):
                aPrintOrg.Y() += ( nPageHeight - nPrintHeight ) >> 1;
            break;

            case( RP_MM ):
            {
                aPrintOrg.X() += ( nPageWidth - nPrintWidth ) >> 1;
                aPrintOrg.Y() += ( nPageHeight - nPrintHeight ) >> 1;
            }
            break;

            case( RP_RM):
            {
                aPrintOrg.X() += nPageWidth - nPrintWidth;
                aPrintOrg.Y() += ( nPageHeight - nPrintHeight ) >> 1;
            }
            break;

            case( RP_LB ):
                aPrintOrg.Y() += nPageHeight - nPrintHeight;
            break;

            case( RP_MB ):
            {
                aPrintOrg.X() += ( nPageWidth - nPrintWidth ) >> 1;
                aPrintOrg.Y() += nPageHeight - nPrintHeight;
            }
            break;

            case( RP_RB ):
            {
                aPrintOrg.X() += nPageWidth - nPrintWidth;
                aPrintOrg.Y() += nPageHeight - nPrintHeight;
            }
            break;

            default :
            break;
        }

		aGraphic.Draw( pPrinter, aPrintOrg, aPrintSize );
	}
}

// -----------------------------------------------------------------------------

SimDrawDocShell* SimDrawViewShell::GetObjectShell() const
{
	return( (SimDrawDocShell*) GetViewFrame()->GetObjectShell() );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::AddWindow( SimWindow* pWin )
{
	pView->AddOutputDevice( pWin );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::RemoveWindow( SimWindow* pWin )
{
	pView->DelOutputDevice( pWin );
}

// -----------------------------------------------------------------------------

USHORT SimDrawViewShell::Print(SfxProgress &rProgress, PrintDialog *pPrintDialog)
{
    SfxPrinter*	pPrinter = GetPrinter();

	SfxViewShell::Print( rProgress, pPrintDialog );

    if( pPrinter )
    {
		const MapMode	aOldMap( pPrinter->GetMapMode() );
		const ULONG		nFrameCount = pView->GetFrameCount();
		const Range		aFullRange( 1, nFrameCount );
		MultiSelection	aSelection;

		pPrinter->SetMapMode( MapMode( MAP_100TH_MM ) );
		aSelection.SetTotalRange( aFullRange );

		if( pPrintDialog )
		{
			switch( pPrintDialog->GetCheckedRange() )
			{
				case PRINTDIALOG_RANGE:
				case PRINTDIALOG_FROMTO:
					aSelection = MultiSelection( pPrintDialog->GetRangeText() );
				break;

				default:
					aSelection.Select( aFullRange );
				break;
			}
		}
		else
			aSelection.Select( aFullRange );

		for( long nSel = aSelection.FirstSelected(); nSel != SFX_ENDOFSELECTION; nSel = aSelection.NextSelected() )
		{
			if( nSel > nFrameCount )
				break;
			else if( nSel )
			{
				const SimFrame* pFrame = pView->GetModel()->GetFrame( nSel - 1L );

				if( pFrame )
				{
					pPrinter->StartPage();
					ImplPrintFrame( *pFrame, pPrinter );
					pPrinter->EndPage();
				}
			}
		}

		pPrinter->SetMapMode( aOldMap );
    }

    return 0;
}

// -----------------------------------------------------------------------------

BOOL SimDrawViewShell::KeyInput( const KeyEvent& rKEvt, SimWindow* pWin )
{
    return SimViewShell::KeyInput( rKEvt, pWin );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::MouseButtonDown( const MouseEvent& rMEvt, SimWindow* pWin )
{
    SimViewShell::MouseButtonDown( rMEvt, pWin );

	if( bPipette )
		SVXBMPMASK()->PipetteClicked();
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::MouseMove( const MouseEvent& rMEvt, SimWindow* pWin )
{
    SimViewShell::MouseMove( rMEvt, pWin );

	// Umgebungsfarbe unter MausPointer setzen
	if ( bPipette && SfxViewFrame::Current()->HasChildWindow( SvxBmpMaskChildWindow::GetChildWindowId() ) )
	{
		Point			aLogPos( pWin->PixelToLogic( pWin->GetPointerPosPixel() ) );
		const long		nStartX = aLogPos.X() - PIPETTE_RANGE;
		const long		nEndX = aLogPos.X() + PIPETTE_RANGE;
		const long		nStartY = aLogPos.Y() - PIPETTE_RANGE;
		const long		nEndY = aLogPos.Y() + PIPETTE_RANGE;
		long			nRed = 0;
		long			nGreen = 0;
		long			nBlue = 0;
		const double	fDiv = ( ( PIPETTE_RANGE << 1 ) + 1 ) * ( ( PIPETTE_RANGE << 1 ) + 1 );

		for ( long nY = nStartY; nY <= nEndY; nY++ )
		{
			for( long nX = nStartX; nX <= nEndX; nX++ )
			{
				const Color aCol( pWin->GetPixel( Point( nX, nY ) ) );

				nRed += aCol.GetRed();
				nGreen += aCol.GetGreen();
				nBlue += aCol.GetBlue();
			}
		}

		SVXBMPMASK()->SetColor( Color( (BYTE) ( nRed / fDiv + .5 ),
									   (BYTE) ( nGreen / fDiv + .5 ),
									   (BYTE) ( nBlue / fDiv + .5 ) ) );
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::MouseButtonUp( const MouseEvent& rMEvt, SimWindow* pWin )
{
    SimViewShell::MouseButtonUp( rMEvt, pWin );
}

// -----------------------------------------------------------------------------

SfxPrinter* SimDrawViewShell::GetPrinter( BOOL bCreate )
{
    return GetObjectShell()->GetPrinter();
}

// -----------------------------------------------------------------------------

USHORT SimDrawViewShell::SetPrinter(SfxPrinter* pNewPrinter, USHORT nDiffFlags)
{
    GetObjectShell()->SetPrinter( pNewPrinter );
    return 0;
}

// -----------------------------------------------------------------------------

long SimDrawViewShell::GetIPZoom( const Size& rLogSize100 )
{
	const Size	aLogSize( pWindow->PixelToLogic( pView->GetSizePixel(), MAP_100TH_MM ) );
	long		nRet;

	if( aLogSize.Width() && aLogSize.Height() )
	{
		const double	fFactW = (double) rLogSize100.Width() / aLogSize.Width();
		const double	fFactH = (double) rLogSize100.Height() / aLogSize.Height();

		nRet = (long) ( ( ( fFactW < fFactH ) ? fFactW : fFactH ) * 100. + 0.5 );
	}
	else
		nRet = SimViewShell::GetIPZoom( rLogSize100 );

	return nRet;
}

// -----------------------------------------------------------------------------

long SimDrawViewShell::GetHCtrlWidth()
{
	return pTabControl->GetSizePixel().Width();
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::AdjustControls( const Point& rNewPos, const Size& rNewSize )
{
	Point		aHPos;
	Size		aTabSize( pTabControl->GetSizePixel() );

	if( !aTabSize.Width() || !aTabSize.Height() )
	{
		aTabSize.Width() = Max( ( rNewSize.Width() * 333L ) / 1000L, 300L );
		aTabSize.Height() = aScrBarWH.Height();
		pTabControl->SetSizePixel( aTabSize );
	}

	SimViewShell::AdjustControls( rNewPos, rNewSize );

	aHPos.X() = rNewPos.X(), aHPos.Y() = rNewPos.Y() + rNewSize.Height() - aScrBarWH.Height();
	aTabSize.Width() = Max( pHScrlArray[ 0 ]->GetPosPixel().X() - aHPos.X(), 0L );
	pTabControl->SetPosSizePixel( aHPos, aTabSize );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::SwitchToFrame( ULONG nFramePos )
{
	pView->SwitchToFrame( nFramePos );
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::InsertFrame()
{
	SimNewDlg aNewDlg( &GetViewFrame()->GetWindow() );

	aNewDlg.Execute();

	Bitmap aBmp( aNewDlg.GetSize(), aNewDlg.GetColorDepth() );
	aBmp.Erase( Color( COL_WHITE ) );
	const Size aSizePix( aBmp.GetSizePixel() );

	if( aSizePix.Width() && aSizePix.Height() )
	{
		pView->InsertFrame( new SimFrame( pView->GetModel(), aBmp ), FRAME_APPEND );
		SwitchToFrame( pView->GetFrameCount() - 1UL );
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::RemoveFrame( ULONG nFramePos )
{
	if( pView && ( nFramePos < pView->GetFrameCount() ) )
	{
		QueryBox aQuery( pWindow, WB_YES_NO, String( SimResId( STR_QUERY_REMOVE_FRAME ) ) );

		if( aQuery.Execute() == RET_YES )
			pView->RemoveFrame( nFramePos );
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::EditFrameProperties()
{
	SimFrame* pCurFrame = pView->GetCurFrame();

	if( pCurFrame )
	{
		SimFramePropertiesDlg aDlg( pWindow, *pCurFrame );

		if( aDlg.Execute() == RET_OK )
		{
			pView->SetDisplayPosPixel( aDlg.GetDisplayPosPixel() );
			pView->SetDisplaySizePixel( aDlg.GetDisplaySizePixel() );
			pView->SetWaitMS( aDlg.GetWaitMS() );
			pView->SetDisposal( aDlg.GetDisposal() );
		}
	}
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::InitFrameChange()
{
	SimViewShell::InitFrameChange();
}

// -----------------------------------------------------------------------------

void SimDrawViewShell::EndFrameChange()
{
	const ULONG	nFrameCount = pView->GetFrameCount();
	const ULONG	nActiveFrame = pView->GetCurFramePos();
	String		aFrameBaseName( SimResId( STR_FRAMENAME ) );

	aFrameBaseName += ' ';
	pTabControl->Clear();

	for( ULONG i = 1; i <= nFrameCount; i++ )
	{
		String aFrameName( aFrameBaseName );
		pTabControl->InsertPage( i, aFrameName += String::CreateFromInt32( i ) );
	}
	pTabControl->SetCurPageId( nActiveFrame + 1 );
	SetMouseRectInfo();
	SimViewShell::EndFrameChange();
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimDrawViewShell, TabControlSplitHdl, SimTabControl*, pTab )
{
	const Point	aTabPos( pTabControl->GetPosPixel() );
	Size		aTabSize( pTabControl->GetSizePixel() );
	Point		aScrollPos;
	Size		aScrollSize;
	const long	nSplitPos = pHSplit->GetPosPixel().X();
	const long	nMaxWidth = nSplitPos - aTabPos.X() - MIN_SCROLLBAR_SIZE;

	aTabSize.Width() = Max( Min( pTab->GetSplitSize(), nMaxWidth ), 0L );
	pTabControl->SetSizePixel( aTabSize );

	aScrollPos.X() = aTabPos.X() + aTabSize.Width();
	aScrollPos.Y() = aTabPos.Y();
	aScrollSize.Width() = nSplitPos - ( aTabPos.X() + aTabSize.Width() );
	aScrollSize.Height() = aTabSize.Height();
	pHScrlArray[0]->SetPosSizePixel( aScrollPos, aScrollSize );

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimDrawViewShell, TimerHdl, Timer*, pTimer )
{
	delete pNewDocTimer, pNewDocTimer = NULL;
	Application::PostUserEvent( LINK( this, SimDrawViewShell, NewDocHdl ) );
	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimDrawViewShell, NewDocHdl, void*, pVoid )
{
	if( !pView->GetFrameCount() )
	{
		SimNewDlg aNewDlg( &GetViewFrame()->GetWindow() );

		aNewDlg.Execute();

		Bitmap aBmp( aNewDlg.GetSize(), aNewDlg.GetColorDepth() );
		aBmp.Erase( Color( COL_WHITE ) );
		const Size aSizePix( aBmp.GetSizePixel() );

		if( aSizePix.Width() && aSizePix.Height() )
		{
			Graphic aGraphic( aBmp );
			GetObjectShell()->ReplaceGraphic( &aGraphic );
		}
	}

	return 0L;
}
