/*************************************************************************
 *
 *  $RCSfile: simundo.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "simapp.hrc"
#include "simview.hxx"
#include "simundo.hxx"

// -----------------
// - SimUndoAction -
// -----------------

TYPEINIT1( SimUndoAction, SfxUndoAction );

// -----------------------------------------------------------------------------

SimUndoAction::SimUndoAction( SimView* pView, const String& rComment,
							  const USHORT nSID, const AnimBmp* pAnimBmp ) : 
	mpView		( pView ),
	maComment	( rComment ),
	mbAlpha		( FALSE )
{
	mnSID = FilterSID( nSID );

	if( pAnimBmp )
		maAnimBmp = *pAnimBmp;
}

// -----------------------------------------------------------------------------

SimUndoAction::~SimUndoAction()
{
}

// -----------------------------------------------------------------------------
 
USHORT SimUndoAction::FilterSID( USHORT nSID )
{
	// do we have an alpha action?
	switch( nSID )
	{
		case( SID_CHANGE_SIZE_FREE ):
		case( SID_CHANGE_SIZE ):
		case( SID_ROTATE_FREE ):
		case( SID_USERDEFINED_ROTATE ):
		case( SID_ROTATE_RIGHT ):
		case( SID_ROTATE_LEFT ):
		case( SID_MIRROR_HORZ ):
		case( SID_MIRROR_VERT ):
		case( SID_CROP ):
		case( SID_SIM_PAINTING ):
		{
			mbAlpha = TRUE;
		}
		break;

		default:
			mbAlpha = FALSE;
		break;
	}

	// do we have a repeatable action?
	switch( nSID )
	{
		case( SID_GRAYSCALES_DIRECT ):
		case( SID_POSTER_DIRECT ):
		case( SID_TILE_DIRECT ):
		case( SID_BLACK_WHITE_DIRECT ):
		case( SID_BLACK_WHITE ):
		case( SID_16_GRAYSCALES ):
		case( SID_16_COLORS ):
		case( SID_256_GRAYSCALES ):
		case( SID_256_COLORS ):
		case( SID_16M_COLORS ):
		case( SID_CROP ):
		case( SID_SIM_PAINTING ):
		{
			nSID = 0;
		}
		break;

		default:
		break;
	}

	return nSID;
}

// -----------------------------------------------------------------------------

String SimUndoAction::GetComment() const 
{
	return maComment; 
}

// -----------------------------------------------------------------------------

String SimUndoAction::GetRepeatComment( SfxRepeatTarget& ) const
{
	return maComment; 
}

// -----------------------------------------------------------------------------

void SimUndoAction::Undo()
{
	mpView->Undo( this );
}

// -----------------------------------------------------------------------------

void SimUndoAction::Redo()
{
	mpView->Redo( this );
}

// -----------------------------------------------------------------------------

void SimUndoAction::Repeat( SfxRepeatTarget& )
{
	mpView->Repeat( this ); 
}

// -----------------------------------------------------------------------------

BOOL SimUndoAction::CanRepeat( SfxRepeatTarget& ) const 
{
	return mnSID > 0;
}
