/*************************************************************************
 *
 *  $RCSfile: simview.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_COLOR 0
#define _XOUTBMP_STATICS_ONLY

#include <tools/string.hxx>
#include <vcl/hatch.hxx>
#include <vcl/poly.hxx>
#include <sot/exchange.hxx>
#include <sot/formats.hxx>
#include <vcl/virdev.hxx>
#include <vcl/drag.hxx>
#include <vcl/bmpacc.hxx>
#include <vcl/wrkwin.hxx>
#include <so3/svstor.hxx>
#include <sfx2/bindings.hxx>
#include <sfx2/dispatch.hxx>
#include <svx/xdef.hxx>
#include <svx/fmview.hxx>
#include <svx/fmmodel.hxx>
#include "strings.hrc"
#include "simattr.hxx"
#include "svimage.hxx"
#include "simmodel.hxx"
#include "docshell.hxx"
#include "viewshel.hxx"
#include "simresid.hxx"
#include "simundo.hxx"
#include "simview.hxx"

// -----------
// - Defines -
// -----------

#define SIM_HATCH_MODE					1
#define SIM_USE_DOUBLEBUFFER			1

#define SIM_OBJECTSHELL()				((SimDrawDocShell*)pModel->GetObjectShell())
#define IMG_PTR( _def_pImage, _def_nY )	((BYTE*)(_def_pImage)->GetSrcPointer()+((_def_pImage)->GetSrcHeight()-(_def_nY)-1)*(_def_pImage)->GetSrcWidthAl());

#ifdef SIM_HATCH_MODE
#define SIM_BACK_COL					255, 255, 255
#else // SIM_HATCH_MODE
#define SIM_BACK_COL					234, 234, 234
#endif // SIM_HATCH_MODE

#define DMAP( _def_nVal, _def_nThres )	((pDitherDiff[_def_nVal]>(_def_nThres))?pDitherHigh[_def_nVal]:pDitherLow[_def_nVal])

// -----------
// - statics -
// -----------

static long	aOrdDither8Bit[ 8 ][ 8 ] =
{
	 0, 38,  9, 48,  2, 40, 12, 50, 
	25, 12, 35, 22, 28, 15, 37, 24, 
	 6, 44,  3, 41,  8, 47,  5, 44, 
	32, 19, 28, 16, 34, 21, 31, 18, 
	 1, 40, 11, 49,  0, 39, 10, 48, 
	27, 14, 36, 24, 26, 13, 36, 23, 
	 8, 46,  4, 43,  7, 45,  4, 42, 
	33, 20, 30, 17, 32, 20, 29, 16
};

// -----------
// - SimView -
// -----------

SimView::SimView( SimDrawDocShell* pDShell, SimModel* pSimModel, SimViewShell* pVShell, OutputDevice* pOut ) :
			pDocShell			( pDShell ),
			pModel				( pSimModel ),
			pViewShell			( pVShell ),
			aAttrSet			( *pSimModel->GetPool(), ATTR_PNT_START, ATTR_PNT_END, XATTR_START, XATTR_END, 0 ),
			bAttrChanged		( TRUE ),
			nCurFramePos		( 0 ),
			pCurFrame			( NULL ),
			bIsFrameAcquired	( FALSE ),
			bDrawBackground		( TRUE ),
			bDrawGrid			( FALSE ),
			bSelectionVisible	( FALSE )
{
    StartListening( *pModel );

	if( pOut )
		AddOutputDevice( pOut );

	if( Application::GetDefaultDevice()->GetBitCount() <= 8 )
	{
		long n;

		pDitherDiff = new long[ 256 ];
		pDitherLow = new BYTE[ 256 ];
		pDitherHigh = new BYTE[ 256 ];
		bDither = TRUE;

		for( n = 0; n < 256L; n++ )
			pDitherDiff[ n ] = n % 51L;

		for( n = 0; n < 256L; n++ )
			pDitherLow[ n ] = (BYTE) ( n / 51L );

		for( n = 0; n < 256L; n++ )
			pDitherHigh[ n ] = Min( pDitherLow[ n ] + 1, 5 );
	}
	else
	{
		pDitherDiff = NULL;
		pDitherLow = pDitherHigh = NULL;
		bDither = FALSE;
	}
}

// -----------------------------------------------------------------------------

SimView::~SimView()
{
	if( pCurFrame && bIsFrameAcquired )
		pCurFrame->Release(); 

	EndListening( *pModel );

	delete[] pDitherDiff;
	delete[] pDitherLow;
	delete[] pDitherHigh;
}

// -----------------------------------------------------------------------------

void SimView::ImplPaintBackground( const Rectangle& rRect, OutputDevice* pOut )
{
	const Rectangle		aFrameRect( pOut->LogicToPixel( aOutRect ) );
	const Rectangle		aRect( pOut->LogicToPixel( rRect ) );
	const BOOL			bOldMap = pOut->IsMapModeEnabled();

	pOut->EnableMapMode( FALSE );
	pOut->SetRefPoint( aFrameRect.TopLeft() );
	
	if( aFrameRect.IsEmpty() )
	{
		pOut->SetRefPoint( pOut->LogicToPixel( Point(), pOut->GetMapMode() ) );
		ImplPaintFastBackground( aRect, pOut );
	}
	else if( IsTransparent() )
		ImplPaintFastBackground( aRect, pOut );
	else
	{
		const Size	aWidthPix( pOut->GetOutputSizePixel() );
		Rectangle	aTmpRect;
	
		aTmpRect.Left() = -1;
		aTmpRect.Top() = -1;
		aTmpRect.Right() = aWidthPix.Width(); 
		aTmpRect.Bottom() = aFrameRect.Top() - 1L;
		aTmpRect.Justify();
		aTmpRect.Intersection( aRect );

		if( !aTmpRect.IsEmpty() )
			ImplPaintFastBackground( aTmpRect, pOut );

		aTmpRect.Left() = -1;
		aTmpRect.Top() = aFrameRect.Top();
		aTmpRect.Right() = aFrameRect.Left() - 1L; 
		aTmpRect.Bottom() = aFrameRect.Bottom();
		aTmpRect.Justify();
		aTmpRect.Intersection( aRect );

		if( !aTmpRect.IsEmpty() )
			ImplPaintFastBackground( aTmpRect, pOut );

		aTmpRect.Left() = aFrameRect.Right() + 1L;
		aTmpRect.Top() = aFrameRect.Top();
		aTmpRect.Right() = aWidthPix.Width(); 
		aTmpRect.Bottom() = aFrameRect.Bottom();
		aTmpRect.Justify();
		aTmpRect.Intersection( aRect );

		if( !aTmpRect.IsEmpty() )
			ImplPaintFastBackground( aTmpRect, pOut );

		aTmpRect.Left() = -1;
		aTmpRect.Top() = aFrameRect.Bottom() + 1L;
		aTmpRect.Right() = aWidthPix.Width(); 
		aTmpRect.Bottom() = aWidthPix.Height();
		aTmpRect.Justify();
		aTmpRect.Intersection( aRect );

		if( !aTmpRect.IsEmpty() )
			ImplPaintFastBackground( aTmpRect, pOut );
	}

	pOut->SetRefPoint();
	pOut->EnableMapMode( bOldMap );
}

// -----------------------------------------------------------------------------

void SimView::ImplPaintFastBackground( const Rectangle& rRect, OutputDevice* pOut )
{
#ifdef SIM_HATCH_MODE

	const Hatch	aHatch( HATCH_DOUBLE, COL_LIGHTGRAY, 6, 450 );

#ifdef SIM_USE_DOUBLEBUFFER

	pOut->DrawHatch( Polygon( rRect ), aHatch );

#else // SIM_USE_DOUBLEBUFFER
	
	const Size		aSize( rRect.GetSize() );
	VirtualDevice	aVDev( *pOut );

	aVDev.SetBackground( pOut->GetFillColor() );

	if( aVDev.SetOutputSizePixel( aSize ) )
	{
		Point aNullPt, aRefPt( pOut->GetRefPoint() );

		aVDev.SetRefPoint( aRefPt -= rRect.TopLeft() );
		aVDev.DrawHatch( Polygon( Rectangle( aNullPt, aSize ) ), aHatch );
		pOut->DrawOutDev( rRect.TopLeft(), aSize, aNullPt, aSize, aVDev );
	}
	else
		pOut->DrawHatch( Polygon( rRect ), aHatch );

#endif // SIM_USE_DOUBLEBUFFER

#else // SIM_HATCH_MODE
	
	pOut->DrawRect( rRect );

#endif // SIM_HATCH_MODE
}

// -----------------------------------------------------------------------------

void SimView::ImplPaintTile( const Rectangle& rRect, OutputDevice* pOut )
{
	const Point	aFrameOutPos( pOut->LogicToPixel( aOutRect.TopLeft() ) );
	const Size	aFrameOutSiz( pOut->LogicToPixel( aOutRect.GetSize() ) );
	const Size	aImageSize( GetSizePixel() );
	Rectangle	aWorkRect( pOut->LogicToPixel( rRect ) );

	aWorkRect.Intersection( Rectangle( aFrameOutPos, aFrameOutSiz ) );

	if( !aWorkRect.IsEmpty() )
	{
		Bitmap				aBmp;
		BitmapWriteAccess*	pWAcc = NULL;
		SvImage*			pImage = pCurFrame->GetImage();
		const Point			aImgPos( aWorkRect.Left() - aFrameOutPos.X(), aWorkRect.Top() - aFrameOutPos.Y() );
		const long			nWidth = aWorkRect.GetWidth();
		const long			nHeight = aWorkRect.GetHeight();
		const long			nHeight1 = nHeight - 1L;
		const long			nFrameWidth = aFrameOutSiz.Width();
		const long			nFrameHeight = aFrameOutSiz.Height();
		const long			nImgWidth = aImageSize.Width();
		const long			nImgHeight = aImageSize.Height();
		long				nX, nXPos, nY, nYPos, nMapY, nActY, nScanSize;
		long*				pLutX = new long[ nWidth ];
		long*				pLutY = new long[ nHeight ];
		const BOOL			bOldMap = pOut->IsMapModeEnabled();

		pOut->EnableMapMode( FALSE );

		for( nX = 0L, nXPos = aImgPos.X(); nX < nWidth; )
			pLutX[ nX++ ] = (long) ( nXPos++ * nImgWidth / (double) nFrameWidth );
	
		for( nY = 0L, nYPos = aImgPos.Y(); nY < nHeight; )
			pLutY[ nY++ ] = (long) ( nYPos++ * nImgHeight / (double) nFrameHeight );
		
		if( pImage && pImage->GetSrcPointer() )
		{
			BitmapColor	aCol;
			BYTE*		pSrc;
			BYTE*		pTmpSrc;
			
			if( pCurFrame->GetAlpha() )
			{
				const Rectangle	aLogRect( pOut->PixelToLogic( aWorkRect ) );
				SvImage*		pAlpha = pCurFrame->GetAlpha();
				BYTE*			pMsk;
				BYTE*			pTmpMsk;
				long			nTmpX;
			
				aBmp = pOut->GetBitmap( aLogRect.TopLeft(), aLogRect.GetSize() );

				if( aBmp.GetBitCount() <= 8 )
					aBmp.Convert( BMP_CONVERSION_24BIT );

				pWAcc = aBmp.AcquireWriteAccess();

				if( pAlpha->GetSrcPointer() )
				{
					for( nY = 0L; nY < nHeight; nY++ )
					{
						pSrc = IMG_PTR( pImage, nMapY = pLutY[ nY ] );
						pMsk = IMG_PTR( pAlpha, nMapY );

						for( nX = 0L; nX < nWidth; nX++ )
						{
							pTmpMsk = pMsk + ( nTmpX = pLutX[ nX ] * 3 );

							if( !pTmpMsk[ 0 ] && !pTmpMsk[ 1 ] && !pTmpMsk[ 2 ] )
							{
								aCol.SetBlue( *( pTmpSrc = pSrc + nTmpX )++ );
								aCol.SetGreen( *pTmpSrc++ );
								aCol.SetRed( *pTmpSrc );
								pWAcc->SetPixel( nY, nX, aCol );
							}
						}
					}
				}
				else
				{
					Bitmap				aMask( pCurFrame->GetBitmapEx().GetMask() );
					BitmapReadAccess*	pMAcc = aMask.AcquireReadAccess();
					const BitmapColor	aMB( pMAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );

					for( nY = 0L; nY < nHeight; nY++ )
					{
						pSrc = IMG_PTR( pImage, nMapY = pLutY[ nY ] );

						for( nX = 0L; nX < nWidth; nX++ )
						{
							if( pMAcc->GetPixel( nMapY, pLutX[ nX ] ) == aMB )
							{
								aCol.SetBlue( *( pTmpSrc = pSrc + pLutX[ nX ] * 3 )++ );
								aCol.SetGreen( *pTmpSrc++ );
								aCol.SetRed( *pTmpSrc );
								pWAcc->SetPixel( nY, nX, aCol );
							}
						}
					}

					aMask.ReleaseAccess( pMAcc );
				}

				aBmp.ReleaseAccess( pWAcc );
			}
			else
			{
				aBmp = Bitmap( Size( nWidth, nHeight ), 24 );
				pWAcc = aBmp.AcquireWriteAccess();
				nScanSize = pWAcc->GetScanlineSize();
				nActY = 0L;

				while( nActY < nHeight )
				{
					pSrc = IMG_PTR( pImage, nMapY = pLutY[ nActY ] );

					for( nX = 0L; nX < nWidth; nX++ )
					{
						aCol.SetBlue( *( pTmpSrc = pSrc + pLutX[ nX ] * 3 )++ );
						aCol.SetGreen( *pTmpSrc++ );
						aCol.SetRed( *pTmpSrc );
						pWAcc->SetPixel( nActY, nX, aCol );
					}

					while( ( nActY < nHeight1 ) && ( pLutY[ nActY + 1 ] == nMapY ) )
					{
						HMEMCPY( pWAcc->GetScanline( nActY + 1 ), pWAcc->GetScanline( nActY ), nScanSize );
						nActY++;
					}
					nActY++;
				}

				aBmp.ReleaseAccess( pWAcc );
			}
		}
		else
		{
			Bitmap aSrcBmp, aSrcMask;

			if( pImage )
			{
				// image pointers are available, but not accessed => get bitmaps directly from Image instances
				aSrcBmp = pImage->GetBitmap();

				if( pCurFrame->GetAlpha() )
					aSrcMask = pCurFrame->GetAlpha()->GetBitmap();
			}
			else
			{
				// image pointers are not available => get bitmaps from frame
				const BitmapEx aBmpEx( pCurFrame->GetBitmapEx() );
				
				aSrcBmp = aBmpEx.GetBitmap();

				if( aBmpEx.IsTransparent() )
					aSrcMask = aBmpEx.GetMask();
			}

			BitmapReadAccess* pRAcc = aSrcBmp.AcquireReadAccess();

			if( !!aSrcMask )
			{
				const Rectangle		aLogRect( pOut->PixelToLogic( aWorkRect ) );
				BitmapReadAccess*	pMAcc = aSrcMask.AcquireReadAccess();
				const BitmapColor	aMB( pMAcc->GetBestMatchingColor( Color( COL_BLACK ) ) );
			
				aBmp = pOut->GetBitmap( aLogRect.TopLeft(), aLogRect.GetSize() );

				if( aBmp.GetBitCount() <= 8 )
					aBmp.Convert( BMP_CONVERSION_24BIT );

				pWAcc = aBmp.AcquireWriteAccess();

				if( pRAcc->HasPalette() )
				{
					for( nY = 0L; nY < nHeight; nY++ )
						for( nX = 0L, nMapY = pLutY[ nY ]; nX < nWidth; nX++ )
							if( pMAcc->GetPixel( nMapY, pLutX[ nX ] ) == aMB )
								pWAcc->SetPixel( nY, nX, pRAcc->GetPaletteColor( pRAcc->GetPixel( nMapY, pLutX[ nX ] ) ) );
				}
				else
				{
					for( nY = 0L; nY < nHeight; nY++ )
						for( nX = 0L, nMapY = pLutY[ nY ]; nX < nWidth; nX++ )
							if( pMAcc->GetPixel( nMapY, pLutX[ nX ] ) == aMB )
								pWAcc->SetPixel( nY, nX, pRAcc->GetPixel( nMapY, pLutX[ nX ] ) );
				}

				aSrcMask.ReleaseAccess( pMAcc );
				aBmp.ReleaseAccess( pWAcc );
			}
			else
			{
				aBmp = Bitmap( Size( nWidth, nHeight ), 24 );
				pWAcc = aBmp.AcquireWriteAccess();
				nScanSize = pWAcc->GetScanlineSize();
				nActY = 0L;

				while( nActY < nHeight )
				{
					nMapY = pLutY[ nActY ];

					if( pRAcc->HasPalette() )
					{
						for( nX = 0L; nX < nWidth; nX++ )
							pWAcc->SetPixel( nActY, nX, pRAcc->GetPaletteColor( pRAcc->GetPixel( nMapY, pLutX[ nX ] ) ) );
					}
					else
					{
						for( nX = 0L; nX < nWidth; nX++ )
							pWAcc->SetPixel( nActY, nX, pRAcc->GetPixel( nMapY, pLutX[ nX ] ) );
					}

					while( ( nActY < nHeight1 ) && ( pLutY[ nActY + 1 ] == nMapY ) )
					{
						HMEMCPY( pWAcc->GetScanline( nActY + 1 ), pWAcc->GetScanline( nActY ), nScanSize );
						nActY++;
					}
					nActY++;
				}

				aBmp.ReleaseAccess( pWAcc );
			}

			aSrcBmp.ReleaseAccess( pRAcc );
		}

		delete[] pLutX;
		delete[] pLutY;

		// Dithering if neccessary
		if( bDither )
		{
			Bitmap				aDitherBmp( Size( nWidth, nHeight ), 8 );
			BitmapReadAccess*	pRAcc = aBmp.AcquireReadAccess();
			BitmapWriteAccess*	pWAcc = aDitherBmp.AcquireWriteAccess();
			BitmapColor			aRGB, aIndex( (BYTE) 0 );

			if( pRAcc && pWAcc )
			{
				for( long nY = 0L, nYPos = aImgPos.Y(); nY < nHeight; nY++, nYPos++ )
				{
					const long nModY = nYPos % 8;

					for( long nX = 0L, nXPos = aImgPos.X(); nX < nWidth; nX++, nXPos++ )
					{
						const long nThres = aOrdDither8Bit[ nModY ][ nXPos % 8 ];
						
						aRGB = pRAcc->GetPixel( nY, nX );
						aIndex.SetIndex( DMAP( aRGB.GetRed(), nThres ) + 
										 DMAP( aRGB.GetGreen(), nThres ) * 6 + 
										 DMAP( aRGB.GetBlue(), nThres ) * 36 + 16 );
						pWAcc->SetPixel( nY, nX, aIndex );
					}
				}
			}

			aBmp.ReleaseAccess( pRAcc );
			aDitherBmp.ReleaseAccess( pWAcc );

			if( pRAcc && pWAcc )
				aBmp = aDitherBmp;
		}
		
		pOut->DrawBitmap( aWorkRect.TopLeft(), aBmp.GetSizePixel(), aBmp );
		pOut->EnableMapMode( bOldMap );
	}
}

// -----------------------------------------------------------------------------

void SimView::ImplPaintGrid( const Rectangle& rRect, OutputDevice* pOut )
{
	Rectangle	aFrameRect( pOut->LogicToPixel( aOutRect ) );
	const Size	aSizePix( GetSizePixel() );

	if( !aFrameRect.IsEmpty() && aSizePix.Width() && aSizePix.Height() )
	{
		const long nZoom = aFrameRect.GetWidth() * 100 / aSizePix.Width();

		if( nZoom && !( nZoom % 100 ) )
		{
			const Color	aOldLineColor( pOut->GetLineColor() );
			long		nDist = nZoom / 100;
			const BOOL	bOldMap = pOut->IsMapModeEnabled();

			if( nZoom < 600 )
			{
				switch( nZoom )
				{
					case( 100 ): nDist = 6; break;
					case( 200 ): nDist = 6; break;
					case( 300 ): nDist = 6; break;
					case( 400 ): nDist = 8; break;
					case( 500 ): nDist = 10; break;
				}
			}
			else
				nDist = nZoom / 100;

			pOut->EnableMapMode( FALSE );
			pOut->SetLineColor( COL_LIGHTGRAY );
			
			pOut->DrawGrid( aFrameRect, Size( nDist, nDist ), GRID_LINES );
			pOut->DrawLine( Point( aFrameRect.Left(), aFrameRect.Bottom() + 1 ),
							Point( aFrameRect.Right() + 1, aFrameRect.Bottom() + 1 ) );
			pOut->DrawLine( Point( aFrameRect.Right() + 1, aFrameRect.Top() ),
							Point( aFrameRect.Right() + 1, aFrameRect.Bottom() + 1 ) );
			
			pOut->SetLineColor( aOldLineColor );
			pOut->EnableMapMode( bOldMap );
		}
	}
}

// -----------------------------------------------------------------------------

void SimView::ImplPaintSelection( PolyPolygon& rSelection, OutputDevice* pOut  )
{
	if( rSelection.Count() )
	{
		PolyPolygon		aTmpSelection;
		const Color		aOldLineColor( pOut->GetLineColor() );
		const Color		aOldFillColor( pOut->GetFillColor() );
		const RasterOp	eOldRop = pOut->GetRasterOp();
		const BOOL		bOldMap = pOut->IsMapModeEnabled();
		const Rectangle	aPixelRect( ImageToScreenRect( Point( 1, 1 ), pOut ) );
		const Point		aOffset( aPixelRect.GetWidth() >> 1, aPixelRect.GetHeight() >> 1 );

		pOut->SetLineColor( Color( COL_BLACK ) );
		pOut->SetFillColor();
		pOut->SetRasterOp( ROP_INVERT );

		for( USHORT n = 0, nCount = rSelection.Count(); n < nCount; n++ )
		{
			const Polygon&	rPoly = rSelection[ n ];
			const USHORT	nSize = rPoly.GetSize();
			Polygon			aTmpPoly( nSize );

			for( USHORT i = 0; i < nSize; i++ )
				( aTmpPoly[ i ] = ImageToScreenPoint( rPoly[ i ], pOut ) ) += aOffset;

			aTmpSelection.Insert( aTmpPoly );
		}

		const Polygon& rPoly = aTmpSelection[ 0 ];

		pOut->EnableMapMode( FALSE );

		if( aTmpSelection.Count() == 1 && rPoly.GetSize() == 2 )
			pOut->DrawLine( rPoly[ 0 ], rPoly[ 1 ] );
		else
			pOut->DrawPolyPolygon( aTmpSelection );

		pOut->EnableMapMode( bOldMap );
		pOut->SetRasterOp( eOldRop );
		pOut->SetFillColor( aOldFillColor );
		pOut->SetLineColor( aOldLineColor );
	}
}

// -----------------------------------------------------------------------------

void SimView::Paint( const Rectangle& rRect, OutputDevice* pOut )
{
	VirtualDevice	aVDev( *pOut );
	MapMode			aMap( pOut->GetMapMode() );
	const Point		aOrigin( Point( -rRect.Left(), -rRect.Top() ) );
	const Size		aImageSize( GetSizePixel() );

	aMap.SetOrigin( aOrigin );
	aVDev.SetMapMode( aMap );

	if( aVDev.SetOutputSize( rRect.GetSize() ) )
	{
		if( bDrawBackground )
			ImplPaintBackground( rRect, &aVDev );

		if( aImageSize.Width() && aImageSize.Height() )
		{
			ImplPaintTile( rRect, &aVDev );

			if( bDrawGrid )
				ImplPaintGrid( rRect, &aVDev );

			if( HasSelection() && IsSelectionVisible() )
				ImplPaintSelection( aSelection, &aVDev );
		}

		pOut->DrawOutDev( rRect.TopLeft(), rRect.GetSize(), 
						  rRect.TopLeft(), rRect.GetSize(), aVDev );
	}
	else
	{
		if( bDrawBackground )
			ImplPaintBackground( rRect, pOut );

		if( aImageSize.Width() && aImageSize.Height() )
		{
			ImplPaintTile( rRect, pOut ); 

			if( bDrawGrid )
				ImplPaintGrid( rRect, pOut );

			if( HasSelection() && IsSelectionVisible() )
				ImplPaintSelection( aSelection, pOut );
		}
	}
}

// -----------------------------------------------------------------------------

void SimView::PaintSelection( PolyPolygon& rSelection, OutputDevice* pOut  )
{
	OutputDevice* pCurOut = ( !pOut ? (OutputDevice*) aWinList.First() : pOut );

	while( pCurOut )
	{
		ImplPaintSelection( rSelection, pCurOut );
		pCurOut = ( !pOut ? (OutputDevice*) aWinList.Next() : NULL );
	}
}

// -----------------------------------------------------------------------------

void SimView::SFX_NOTIFY( SfxBroadcaster& rBC, const TypeId& rBCType,
						  const SfxHint& rHint, const TypeId& rHintType )
{
	if( rHint.ISA( SimHint ) )
	{
		SimHint& rSimHint = (SimHint&) rHint;

		switch( rSimHint.GetType() )
		{
			case( SIM_HINT_FLUSHFRAME ):
			{
				pViewShell->Flush();

				if( bIsFrameAcquired && pCurFrame &&
					( rSimHint.GetFramePos() == nCurFramePos || 
					  rSimHint.GetFramePos() == FRAME_POS_ALL ) )
				{
					pCurFrame->Release();
					bIsFrameAcquired = FALSE;
				}
			}
			break;

			case( SIM_HINT_FRAMEREPLACED ):
			{
				if( rSimHint.GetFramePos() == nCurFramePos )
				{
					ClearSelection();
					pCurFrame = pModel->GetFrame( nCurFramePos );
					pViewShell->InitWindows();
				}
			}
			break;

			case( SIM_HINT_FRAMEINSERTED ):
			{
				if( rSimHint.GetFramePos() <= nCurFramePos )
					SwitchToFrame( nCurFramePos + 1 );
			}
			break;

			case( SIM_HINT_FRAMEREMOVED ):
			{
				DBG_ASSERT( GetFrameCount(), "No more frames?" );
				
				if( nCurFramePos && ( rSimHint.GetFramePos() < nCurFramePos ) )
					SwitchToFrame( nCurFramePos - 1 );
				else
					SwitchToFrame( Min( nCurFramePos, GetFrameCount() - 1UL ) );
			}
			break;

			case( SIM_HINT_MODELCHANGED ):
				SwitchToFrame( nCurFramePos = 0 );
			break;

			case( SIM_HINT_REPAINTFRAME ):
			{
				if( ( rSimHint.GetFramePos() == nCurFramePos ) && pCurFrame )
				{
					const Rectangle& rPaintRect = rSimHint.GetRect();

					if( rPaintRect.IsEmpty() )
						Invalidate();
					else
						Invalidate( rPaintRect );
				}
			}
			break;

			case( SIM_HINT_REINITFRAME ):
			{
				if( ( rSimHint.GetFramePos() == nCurFramePos ) && pCurFrame )
					pViewShell->InitWindows();
			}
			break;

			default:
			break;
		}
	}
}

// -----------------------------------------------------------------------------

void SimView::AddOutputDevice( OutputDevice* pOut )
{
	aWinList.Insert( pOut, LIST_APPEND );
}

// -----------------------------------------------------------------------------

void SimView::DelOutputDevice( OutputDevice* pOut )
{
	aWinList.Remove( pOut );
}

// -----------------------------------------------------------------------------

void SimView::Invalidate( OutputDevice* pOut )
{
	if( pOut )
		( (Window*) pOut )->Invalidate();
	else
		for( void* pWin = aWinList.First(); pWin; pWin = aWinList.Next() )
			( (Window*) pWin )->Invalidate();
}

// -----------------------------------------------------------------------------

void SimView::Invalidate( const Rectangle& rImageRect, OutputDevice* pOut )
{
	if( pOut )
	{
		Rectangle aScreenRect( ImageToScreenRect( rImageRect.TopLeft(), pOut ) );

		aScreenRect.Union( ImageToScreenRect( rImageRect.BottomRight(), pOut ) );
		( (Window*) pOut )->Invalidate( pOut->PixelToLogic( aScreenRect ) );
	}
	else
	{
		for( Window* pWin = (Window*) aWinList.First(); pWin; pWin = (Window*) aWinList.Next() )
		{
			Rectangle aScreenRect( ImageToScreenRect( rImageRect.TopLeft(), pWin ) );

			aScreenRect.Union( ImageToScreenRect( rImageRect.BottomRight(), pWin ) );
			pWin->Invalidate( pWin->PixelToLogic( aScreenRect ) );
		}
	}
}

// -----------------------------------------------------------------------------

void SimView::Copy()
{
	Graphic aGraphic( GetAnimBmp().aBmpEx );

    if( aGraphic.GetType() != GRAPHIC_NONE )
    {
        if( HasSelection() )
        {
			BitmapEx aBmpEx( aGraphic.GetBitmapEx() );
			aBmpEx.Crop( GetSelectionBoundRect() );
			aGraphic = aBmpEx;
        }

        if( aGraphic.GetType() != GRAPHIC_NONE )
        {
			Bitmap					aBmp( aGraphic.GetBitmap() );
			SvDataMemberObjectRef	aDataObject = new SvDataMemberObject();
			SvData*					pDataGrf = new SvData( Graphic::RegisterClipboardFormatName() );
			SvData*					pDataBmp = new SvData( FORMAT_BITMAP );

			pDataGrf->SetData( &aGraphic );
			aDataObject->Append( pDataGrf );

			pDataBmp->SetData( &aBmp );
			aDataObject->Append( pDataBmp );

			aDataObject->CopyClipboard();
		}
    }
}

// -----------------------------------------------------------------------------

void SimView::Paste()
{
	InsertData( SvDataObject::PasteClipboard(), SIM_INSERTDATAPOS_REPLACE );
}

// -----------------------------------------------------------------------------

BOOL SimView::QueryDrop( DropEvent& rDEvt )
{
	BOOL bFile = DragServer::HasFormat( 0, FORMAT_FILE );
    BOOL bReturn = DragServer::HasFormat( 0, FORMAT_BITMAP ) ||
				   DragServer::HasFormat( 0, SOT_FORMATSTR_ID_DRAWING ) ||
				   DragServer::HasFormat( 0, FORMAT_GDIMETAFILE) || 
				   bFile;

	if ( rDEvt.IsDefaultAction() )
		rDEvt.SetAction( DROP_COPY );
	else if ( bFile && ( rDEvt.GetAction() == DROP_MOVE ) )
		bReturn = FALSE;

	return bReturn;
}

// -----------------------------------------------------------------------------

BOOL SimView::Drop( const DropEvent& rDEvt )
{
	pViewShell->GetViewFrame()->GetWindow().EnterWait();

	InsertData( SvDataObject::PasteDragServer( rDEvt ), 
				( rDEvt.GetAction() == DROP_LINK ) ? SIM_INSERTDATAPOS_APPEND : SIM_INSERTDATAPOS_REPLACE );

	pViewShell->GetViewFrame()->GetWindow().LeaveWait();
    return TRUE;
}

// -----------------------------------------------------------------------------

void SimView::Undo( SimUndoAction* pAction )
{
	BOOL bDone = FALSE;

	if( pAction->HasAnimBmp() )
	{
		const AnimBmp aAnimBmp( pAction->GetAnimBmp() );
		
		pAction->SetAnimBmp( GetAnimBmp() );
		ReplaceFrame( new SimFrame( GetModel(), aAnimBmp ), GetCurFramePos() );
		bDone = TRUE;
	}
	else
	{
		SvImage* pImage = GetImagePointer();

		if( pImage )
		{
			const Size aOldSize( GetSizePixel() );

			bDone = ( pAction->GetComment() == String( SimResId( STR_16M_COLORS ) ) ) || pImage->Undo();

			if( bDone && pAction->IsAlphaAction() )
			{
				SvImage* pAlpha = GetAlphaPointer();

				if( pAlpha )
					pAlpha->Undo();
			}

			if( bDone )
				pModel->SendRepaintBroadcast( GetCurFramePos(), GetSizePixel() != aOldSize );
 		}
	}

	if( bDone )
	{
		pModel->SetChanged( pModel->GetModificationCount() > 1 );
		pModel->DecModificationCount();
		pViewShell->Invalidate();
		SFX_BINDINGS().InvalidateAll( TRUE );
	}
}

// -----------------------------------------------------------------------------

void SimView::Redo( SimUndoAction* pAction )
{
	BOOL bDone = FALSE;

	if( pAction->HasAnimBmp() )
	{
		const AnimBmp aAnimBmp( pAction->GetAnimBmp() );

		pAction->SetAnimBmp( GetAnimBmp() );
		ReplaceFrame( new SimFrame( GetModel(), aAnimBmp ), GetCurFramePos() );
		bDone = TRUE;
	}
	else
	{
		SvImage* pImage = GetImagePointer();

		if( pImage )
		{
			const Size aOldSize( GetSizePixel() );

			bDone = ( pAction->GetComment() == String( SimResId( STR_16M_COLORS ) ) ) || pImage->Redo();

			if( bDone && pAction->IsAlphaAction() )
			{
				SvImage* pAlpha = GetAlphaPointer();

				if( pAlpha )
					pAlpha->Redo();
			}

			if( bDone )
				pModel->SendRepaintBroadcast( GetCurFramePos(), GetSizePixel() != aOldSize );
 		}
	}

	if( bDone )
	{
		pModel->SetChanged( TRUE );
		pModel->IncModificationCount();
		pViewShell->Invalidate();
		SFX_BINDINGS().InvalidateAll( TRUE );
	}
}

// -----------------------------------------------------------------------------

void SimView::Repeat( SimUndoAction* pAction )
{
	pViewShell->GetViewFrame()->GetDispatcher()->Execute( pAction->GetSID(), SFX_CALLMODE_ASYNCHRON );
}

// -----------------------------------------------------------------------------

void SimView::SetSelection( const PolyPolygon& rSelection, BOOL bShow )
{
	HideSelection();
	aSelection = rSelection;

	if( rSelection.Count() > 1 || rSelection[ 0 ].GetSize() > 2 )
		aSelection.Clip( GetRectPixel() );

	if( bShow )
		ShowSelection();
}

// -----------------------------------------------------------------------------

void SimView::ShowSelection( OutputDevice* pOut )
{
	if( HasSelection() )
	{
		if( !IsSelectionVisible() )
		{
			PaintSelection( aSelection, pOut );
			bSelectionVisible = TRUE;
		}
	}
	else
		bSelectionVisible = FALSE;
}

// -----------------------------------------------------------------------------

void SimView::HideSelection()
{
	if( HasSelection() && IsSelectionVisible() )
		PaintSelection( aSelection );

	bSelectionVisible = FALSE;
}

// -----------------------------------------------------------------------------

void SimView::ClearSelection()
{
	HideSelection();
	aSelection.Clear();
	pViewShell->SetMouseRectInfo();
}

// -----------------------------------------------------------------------------

void SimView::InsertData( SvDataObjectRef pDataObj, ULONG nInsertPos )
{
	Graphic					aGraphic;
	SimDrawDocShell*		pDocSh = SIM_OBJECTSHELL();
	const SvDataTypeList&	rTypeList = pDataObj->GetTypeList();
	const ULONG				nGraphicExchangeFormat = Graphic::RegisterClipboardFormatName();
	ULONG					nCount = rTypeList.Count();
	BOOL					bGraphic = FALSE;

	for ( ULONG nPos = 0; ( nPos < nCount ) && !bGraphic; nPos++ )
	{
		const ULONG	nFormat = rTypeList.GetObject( nPos ).GetFormat();

		if ( nFormat == FORMAT_FILE )
		{
			const String aFile( DragServer::PasteFile( 0 ) );
			bGraphic = ( pDocSh->DoImportExport( aFile, aGraphic, IMPEXP_IMPORT ) == 0 );
		}
	    else if( nFormat == SOT_FORMATSTR_ID_DRAWING )
		{
			SvData aData( SOT_FORMATSTR_ID_DRAWING );

			if ( pDataObj->GetData( &aData ) )
			{
				SvStorageStreamRef xStream;

				if ( aData.GetData( xStream ) )
				{
					FmFormModel aModel;

					xStream->SetVersion( SOFFICE_FILEFORMAT_NOW );
					xStream->Seek( 0UL );
					aModel.SetStreamingSdrModel( TRUE );
					aModel.GetItemPool().Load( *xStream );
					*xStream >> aModel;
					aModel.GetItemPool().FreezeIdRanges();

					if( !xStream->GetError() )
					{
						VirtualDevice	aVDev;
						aVDev.SetMapMode( MapMode( MAP_100TH_MM ) );
						FmFormView aView( &aModel, &aVDev );

						aView.SetMarkHdlHidden( TRUE );
						aView.ShowPagePgNum( 0, Point() );
						aView.MarkAll();
						aGraphic = aView.GetAllMarkedMetaFile();
						bGraphic = TRUE;
					}
				}
			}
		}
		else if ( nFormat == nGraphicExchangeFormat )
		{
			SvData aData( nFormat );

			if ( pDataObj->GetData( &aData ) )
			{
				Graphic* pGraphic = NULL;

				if ( aData.GetData( (SvDataCopyStream**) &pGraphic, Graphic::StaticType(), TRANSFER_MOVE ) )
				{
					aGraphic = *pGraphic;
					bGraphic = TRUE;
				}

				delete pGraphic;
			}
		}
		else if ( nFormat == FORMAT_BITMAP )
		{
			SvData aData( nFormat );

			if ( pDataObj->GetData( &aData ) )
			{
				Bitmap* pBmp = NULL;

				if ( aData.GetData( &pBmp, TRANSFER_MOVE ) )
				{
					aGraphic = *pBmp;
					bGraphic = TRUE;
				}

				delete pBmp;
			}
		}
		else if ( nFormat == FORMAT_GDIMETAFILE )
		{
			SvData aData( nFormat );

			if ( pDataObj->GetData( &aData ) )
			{
				GDIMetaFile* pMtf = NULL;

				if ( aData.GetData( &pMtf, TRANSFER_MOVE ) )
				{
					aGraphic = *pMtf;
					bGraphic = TRUE;
				}

				delete pMtf;
			}
		}

		// Graphik einfuegen
		if( bGraphic )
		{
			const String	aComment( SimResId( STR_INSERT ) );
			SfxUndoManager* pUndoMgr = SIM_OBJECTSHELL()->GetUndoManager();
			SimUndoAction*	pUndoAction = NULL;

			if( SIM_INSERTDATAPOS_REPLACE == nInsertPos )
			{
				AnimBmp	aOldAnimBmp( GetAnimBmp() );

				if( aGraphic.GetType() == GRAPHIC_BITMAP && aGraphic.IsAnimated() )
					pModel->SetGraphic( aGraphic );
				else
					ReplaceFrame( new SimFrame( GetModel(), aGraphic.GetBitmapEx() ), nCurFramePos );

				pUndoAction = new SimUndoAction( this, aComment, 0, &aOldAnimBmp );
			}
			else
			{
				if( SIM_INSERTDATAPOS_CURPOS == nInsertPos )
					nInsertPos = nCurFramePos;
				else if( SIM_INSERTDATAPOS_APPEND == nInsertPos )
					nInsertPos = GetFrameCount();
				else
					nInsertPos = Min( nInsertPos, GetFrameCount() ? ( GetFrameCount() - 1 ) : 0 );

				if( aGraphic.GetType() == GRAPHIC_BITMAP && aGraphic.IsAnimated() )
				{
					const Animation aAnim( aGraphic.GetAnimation() );
					const ULONG		nFirstInsertPos = nInsertPos;

					for( USHORT i = 0, nCount = aAnim.Count(); i < nCount; i++ )
						InsertFrame( new SimFrame( GetModel(), aAnim.Get( i ) ), nInsertPos++ );

					nInsertPos = nFirstInsertPos;
				}
				else
					InsertFrame( new SimFrame( GetModel(), aGraphic.GetBitmapEx() ), nInsertPos );

				SwitchToFrame( nInsertPos );
			}

			GetModel()->SetChanged( TRUE );
			pUndoMgr->Clear();

			if( pUndoAction )
				pUndoMgr->AddUndoAction( pUndoAction );

			SFX_BINDINGS().InvalidateAll( TRUE );
		}
	}
}

// -----------------------------------------------------------------------------

AnimBmp SimView::GetAnimBmp()
{
	AnimBmp	aAnimBmp;

	if( pCurFrame )
		aAnimBmp = pCurFrame->GetAnimBmp();

	return aAnimBmp;
}

// -----------------------------------------------------------------------------

ULONG SimView::GetFrameCount() const
{
	return pModel->GetFrameCount();
}

// -----------------------------------------------------------------------------

BOOL SimView::SwitchToFrame( ULONG nFramePos )
{
	BOOL bRet = FALSE;

	if( nFramePos < pModel->GetFrameCount() )
	{
		if( nCurFramePos != nFramePos )
			ClearSelection();

		pViewShell->InitFrameChange();

		if( bIsFrameAcquired && ( nCurFramePos != nFramePos ) && pCurFrame )
		{
			pCurFrame->Release();
			bIsFrameAcquired = FALSE;
			SIM_OBJECTSHELL()->GetUndoManager()->Clear();
		}

		pCurFrame = pModel->GetFrame( nCurFramePos = nFramePos );
		pViewShell->EndFrameChange();
		bRet = TRUE;
	}

	return bRet;
}

// -----------------------------------------------------------------------------

void SimView::InsertFrame( SimFrame* pFrame, ULONG nFramePos )
{
	if( bIsFrameAcquired && ( nCurFramePos != nFramePos ) && pCurFrame )
	{
		pCurFrame->Release();
		bIsFrameAcquired = FALSE;
		SIM_OBJECTSHELL()->GetUndoManager()->Clear();
	}

	pModel->InsertFrame( pFrame, nFramePos );
}

// -----------------------------------------------------------------------------

BOOL SimView::RemoveFrame( ULONG nFramePos )
{
	return pModel->RemoveFrame( nFramePos );
}

// -----------------------------------------------------------------------------

BOOL SimView::ReplaceFrame( SimFrame* pFrame, ULONG nFramePos )
{
	ClearSelection();
	return pModel->ReplaceFrame( pFrame, nFramePos );
}

// -----------------------------------------------------------------------------

Size SimView::GetSizePixel() const
{
	return( pCurFrame ? pCurFrame->GetSizePixel() : Size() );
}

// -----------------------------------------------------------------------------

USHORT SimView::GetColorDepth() const
{
	return( pCurFrame ? pCurFrame->GetColorDepth() : 0 );
}

// -----------------------------------------------------------------------------

BOOL SimView::IsTransparent() const
{
	return( pCurFrame ? pCurFrame->IsTransparent() : FALSE );
}

// -----------------------------------------------------------------------------

Size SimView::GetPrefSize() const
{
	return( pCurFrame ? pCurFrame->GetPrefSize() : Size() );
}

// -----------------------------------------------------------------------------

void SimView::SetPrefSize( const Size& rPrefSize )
{
	if( pCurFrame )
		pCurFrame->SetPrefSize( rPrefSize );
}

// -----------------------------------------------------------------------------

MapMode SimView::GetPrefMapMode() const
{
	return( pCurFrame ? pCurFrame->GetPrefMapMode() : MapMode() );
}

// -----------------------------------------------------------------------------

void SimView::SetPrefMapMode( const MapMode& rPrefMapMode )
{
	if( pCurFrame )
		pCurFrame->SetPrefMapMode( rPrefMapMode );
}

// -----------------------------------------------------------------------------

Point SimView::GetDisplayPosPixel() const
{
	return( pCurFrame ? pCurFrame->GetDisplayPosPixel() : Point() );
}

// -----------------------------------------------------------------------------

void SimView::SetDisplayPosPixel( const Point& rPosPix )
{
	if( pCurFrame && pCurFrame->GetDisplayPosPixel() != rPosPix )
	{
		pCurFrame->SetDisplayPosPixel( rPosPix );
		pModel->SetChanged( TRUE );
	}
}

// -----------------------------------------------------------------------------

Size SimView::GetDisplaySizePixel() const
{
	return( pCurFrame ? pCurFrame->GetDisplaySizePixel() : Size() );
}

// -----------------------------------------------------------------------------

void SimView::SetDisplaySizePixel( const Size& rSizePix )
{
	if( pCurFrame && pCurFrame->GetDisplaySizePixel() != rSizePix )
	{
		pCurFrame->SetDisplaySizePixel( rSizePix );
		pModel->SetChanged( TRUE );
	}
}

// -----------------------------------------------------------------------------

ULONG SimView::GetWaitMS() const
{
	return( pCurFrame ? pCurFrame->GetWaitMS() : 0UL );
}

// -----------------------------------------------------------------------------

void SimView::SetWaitMS( ULONG nWaitMS )
{
	if( pCurFrame && pCurFrame->GetWaitMS() != nWaitMS )
	{
		pCurFrame->SetWaitMS( nWaitMS );
		pModel->SetChanged( TRUE );
	}
}

// -----------------------------------------------------------------------------

Disposal SimView::GetDisposal() const
{
	return( pCurFrame ? pCurFrame->GetDisposal() : DISPOSE_NOT );
}

// -----------------------------------------------------------------------------

void SimView::SetDisposal( Disposal eDisposal )
{
	if( pCurFrame && pCurFrame->GetDisposal() != eDisposal )
	{
		pCurFrame->SetDisposal( eDisposal );
		pModel->SetChanged( TRUE );
	}
}

// -----------------------------------------------------------------------------

SvImage* SimView::GetImagePointer()
{
	SvImage* pRet;

	if( pCurFrame )
	{
		if( !bIsFrameAcquired )
			bIsFrameAcquired = pCurFrame->Acquire();

		pRet = pCurFrame->GetImage();
	}
	else
		pRet = NULL;

	return pRet;
}

// -----------------------------------------------------------------------------

SvImage* SimView::GetAlphaPointer()
{
	SvImage* pRet;

	if( pCurFrame )
	{
		if( !bIsFrameAcquired )
			bIsFrameAcquired = pCurFrame->Acquire();

		pRet = pCurFrame->GetAlpha();
	}
	else
		pRet = NULL;

	return pRet;
}

// -----------------------------------------------------------------------------

Point SimView::ImageToScreenPoint( const Point& rImgPos, OutputDevice* pOut )
{
	Point		aRetPoint;
	const Point	aFrameOutPos( pOut->LogicToPixel( aOutRect.TopLeft() ) );
	const Size	aFrameOutSize( pOut->LogicToPixel( aOutRect.GetSize() ) );
	const Size	aImageSize( GetSizePixel() );

	if( aFrameOutSize.Width() && aFrameOutSize.Height() && aImageSize.Width() && aImageSize.Height() )
	{
		aRetPoint.X() = (long)( rImgPos.X() * aFrameOutSize.Width() / (double) aImageSize.Width() );
		aRetPoint.Y() = (long)( rImgPos.Y() * aFrameOutSize.Height() / (double) aImageSize.Height() );

		aRetPoint += aFrameOutPos;
	}
	else
		aRetPoint.X() = aRetPoint.Y() = -1L;

	return aRetPoint;
}

// -----------------------------------------------------------------------------

Rectangle SimView::ImageToScreenRect( const Point& rImgPos, OutputDevice* pOut )
{
	Rectangle	aScreenRect;
	Point		aScreenPos( ImageToScreenPoint( rImgPos, pOut ) );

	if( ScreenToImagePoint( aScreenPos, pOut ) == rImgPos )
	{
		Point aSaveScreenPos( aScreenPos );

		aScreenRect.Left() = aScreenRect.Right() = aScreenPos.X();
		aScreenRect.Top() = aScreenRect.Bottom() = aScreenPos.Y();

		aScreenPos.X()--;
		while( ScreenToImagePoint( aScreenPos, pOut ).X() == rImgPos.X() )
			aScreenRect.Left() = aScreenPos.X()--;

		aScreenPos = aSaveScreenPos, aScreenPos.Y()--;
		while( ScreenToImagePoint( aScreenPos, pOut ).Y() == rImgPos.Y() )
			aScreenRect.Top() = aScreenPos.Y()--;

		aScreenPos = aSaveScreenPos, aScreenPos.X()++;
		while( ScreenToImagePoint( aScreenPos, pOut ).X() == rImgPos.X() )
			aScreenRect.Right() = aScreenPos.X()++;

		aScreenPos = aSaveScreenPos, aScreenPos.Y()++;
		while( ScreenToImagePoint( aScreenPos, pOut ).Y() == rImgPos.Y() )
			aScreenRect.Bottom() = aScreenPos.Y()++;
	}
	else
		aScreenRect = Rectangle( aScreenPos, Size( 1, 1 ) );

	return aScreenRect;
}

// -----------------------------------------------------------------------------

Point SimView::ScreenToImagePoint( const Point& rScreenPos, OutputDevice* pOut )
{
	Point		aRetPoint;
	const Point	aFrameOutPos( pOut->LogicToPixel( aOutRect.TopLeft() ) );
	const Size	aFrameOutSize( pOut->LogicToPixel( aOutRect.GetSize() ) );
	const Size	aImageSize( GetSizePixel() );

	if( aFrameOutSize.Width() && aFrameOutSize.Height() && aImageSize.Width() && aImageSize.Height() )
	{
		aRetPoint.X() = (long)( ( rScreenPos.X() - aFrameOutPos.X() ) * aImageSize.Width() / (double) aFrameOutSize.Width() );
		aRetPoint.Y() = (long)( ( rScreenPos.Y() - aFrameOutPos.Y() ) * aImageSize.Height() / (double) aFrameOutSize.Height() );
	}
	else
		aRetPoint.X() = aRetPoint.Y() = -1L;

	return aRetPoint;
}

// -----------------------------------------------------------------------------

Rectangle SimView::ScreenToImageRect( const Point& rScreenPos, OutputDevice* pOut )
{
	return Rectangle();
}
