/*************************************************************************
 *
 *  $RCSfile: simwin.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "helpids.hrc"
#include "svimage.hxx"
#include "viewshel.hxx"
#include "simview.hxx"
#include "drviewsh.hxx"
#include "simframe.hxx"
#include "simwin.hxx"

// -----------
// - Defines -
// -----------

#define MIN_ZOOM			10L
#define MAX_ZOOM			3200L
#define SCROLL_LINE_FACT	0.1        
#define SCROLL_PAGE_FACT	0.7
#define SCROLL_SENSITIVE	20         
#define SIM_BACK_COL		255, 255, 255

// -------------
// - SimWindow -
// -------------

SimWindow::SimWindow( Window* pParent, SimViewShell* pShell ) :
		Window			( pParent, WB_CLIPCHILDREN ),
		pViewShell		( pShell ),
		aViewSize		( 1000, 1000 ),
		bCenterAllowed	( TRUE ),
		nDropScrollCount( 0UL )
{
	const Color aBackCol( SIM_BACK_COL );

	SetBackground();
	SetLineColor( aBackCol );
	SetFillColor( aBackCol );
	SetHelpId( HID_SIM_WIN_DOCUMENT );
	SetUniqueId( HID_SIM_WIN_DOCUMENT );
    EnableDrop();
}

// -----------------------------------------------------------------------------

SimWindow::~SimWindow()
{
}

// -----------------------------------------------------------------------------

void SimWindow::ShareViewArea( const SimWindow* pOtherWin )
{
	const MapMode&	rOtherMap = pOtherWin->GetSimMap();
	const Fraction	aScale( pOtherWin->GetZoom(), 100 );

    aViewOrigin = pOtherWin->aViewOrigin;
    aViewSize = pOtherWin->aViewSize;
    bCenterAllowed = pOtherWin->bCenterAllowed;
	aSimMap = MapMode( rOtherMap.GetMapUnit(), rOtherMap.GetOrigin(), aScale, aScale );
}

// -----------------------------------------------------------------------------

void SimWindow::Paint( const Rectangle& rRect )
{
	SimFuPoor* pFuActual = pViewShell->GetFuActual();

	pViewShell->GetView()->Paint( rRect, this );

	if( pFuActual )
		pFuActual->Paint( rRect, this );
}

// -----------------------------------------------------------------------------

void SimWindow::KeyInput( const KeyEvent& rKEvt )
{
	if( !pViewShell->KeyInput( rKEvt, this ) )
        Window::KeyInput( rKEvt );
}

// -----------------------------------------------------------------------------

void SimWindow::MouseButtonDown( const MouseEvent& rMEvt )
{
	pViewShell->MouseButtonDown( rMEvt, this );
}

// -----------------------------------------------------------------------------

void SimWindow::MouseMove( const MouseEvent& rMEvt )
{
	pViewShell->MouseMove( rMEvt, this );
}

// -----------------------------------------------------------------------------

void SimWindow::MouseButtonUp( const MouseEvent& rMEvt )
{
	pViewShell->MouseButtonUp( rMEvt, this );
}

// -----------------------------------------------------------------------------

void SimWindow::GrabFocus()
{
	nDropScrollCount = 0UL;
	Window::GrabFocus();
}

// -----------------------------------------------------------------------------

void SimWindow::LoseFocus()
{
	nDropScrollCount = 0UL;
	Window::LoseFocus();
}

// -----------------------------------------------------------------------------

void SimWindow::SetWinViewPos( const Point& rPnt )
{
    aWinPos = rPnt;
}

// -----------------------------------------------------------------------------

void SimWindow::SetViewOrigin( const Point& rPnt )
{
    aViewOrigin = rPnt;
}

// -----------------------------------------------------------------------------

void SimWindow::SetViewSize(const Size& rSize)
{
    aViewSize = rSize;
}

// -----------------------------------------------------------------------------

long SimWindow::SetZoomFactor( long nZoom )
{
	const Fraction	aFract( Max( Min( nZoom, MAX_ZOOM ), MIN_ZOOM ), 100 );

	aSimMap.SetScaleX( aFract );
	aSimMap.SetScaleY( aFract );
   	UpdateMapOrigin();

    return nZoom;
}

// -----------------------------------------------------------------------------

void SimWindow::SetZoom( long nZoom )
{
	nZoom = Max( Min( nZoom, MAX_ZOOM ), MIN_ZOOM );

    if( nZoom != GetZoom() )
    {
	    const Size aSize( PixelToLogic( GetOutputSizePixel(), aSimMap ) );
		const long nW = FRound( (double) aSize.Width()  * GetZoom() / nZoom );
		const long nH = FRound( (double) aSize.Height() * GetZoom() / nZoom );

		aWinPos.X() += ( aSize.Width() - nW ) >> 1;
		aWinPos.Y() += ( aSize.Height() - nH ) >> 1;

		if( aWinPos.X() < 0 )
			aWinPos.X() = 0;

		if( aWinPos.Y() < 0 )
			aWinPos.Y() = 0;

		SetZoomFactor( nZoom );
	}
}

// -----------------------------------------------------------------------------

void SimWindow::SetZoomPoint( const Point& rCenter, long nZoom )
{
	nZoom = Max( Min( nZoom, MAX_ZOOM ), MIN_ZOOM );

	const Size		aWinSize( PixelToLogic( GetOutputSizePixel(), aSimMap ) );
	const double	fFact = (double) GetZoom() / nZoom;

	aWinPos.X() = ( rCenter.X() + aViewOrigin.X() ) - FRound( aWinSize.Width() * fFact * 0.5 );
	aWinPos.Y() = ( rCenter.Y() + aViewOrigin.Y() ) - FRound( aWinSize.Height() * fFact  * 0.5 );

	if( aWinPos.X() < 0 )
		aWinPos.X() = 0;

	if( aWinPos.Y() < 0 )
		aWinPos.Y() = 0;

	SetZoomFactor( nZoom );
}

// -----------------------------------------------------------------------------

long SimWindow::SetZoomRect( const Rectangle& rZoomRect )
{
	long nNewZoom;

	if( rZoomRect.GetWidth() && rZoomRect.GetHeight() )
	{
		Size	aWinSize( PixelToLogic( GetOutputSizePixel(), aSimMap ) );
		double	fFactX = (double) aWinSize.Width() / rZoomRect.GetWidth();
		double	fFactY = (double) aWinSize.Height() / rZoomRect.GetHeight();
		long	nOldZoom = GetZoom();
		
		nNewZoom = FRound( GetZoom() * ( ( fFactX < fFactY ) ? fFactX : fFactY ) );
		nNewZoom -= nNewZoom % ( ( nNewZoom >= 100 ) ? 100 : 10 );
		SetZoomPoint( rZoomRect.Center(), nNewZoom );
	}
	else
		nNewZoom = GetZoom();

	return nNewZoom;
}

// -----------------------------------------------------------------------------

void SimWindow::UpdateMapOrigin()
{
	Size		aWinSize( PixelToLogic( GetOutputSizePixel(), aSimMap ) );
	SimView*	pView = pViewShell->GetView();

    if( bCenterAllowed )
    {
        if( aWinPos.X() > aViewSize.Width() - aWinSize.Width() )
            aWinPos.X() = aViewSize.Width() - aWinSize.Width();
    
	    if( aWinPos.Y() > aViewSize.Height() - aWinSize.Height() )
            aWinPos.Y() = aViewSize.Height() - aWinSize.Height();
    
	    if( aWinSize.Width() > aViewSize.Width() || aWinPos.X() < 0 )
            aWinPos.X() = ( aViewSize.Width() - aWinSize.Width() ) >> 1;
    
	    if( aWinSize.Height() > aViewSize.Height() || aWinPos.Y() < 0 )
            aWinPos.Y() = ( aViewSize.Height() - aWinSize.Height()) >> 1;
	}

	aWinPos -= aViewOrigin;
	aSimMap.SetOrigin( Point( -aWinPos.X(), -aWinPos.Y() ) );
	aWinPos += aViewOrigin;
	pView->SetOutputRect( Rectangle( Point(), LogicToPixel( pView->GetSizePixel(), aSimMap ) ) );

	MapMode	aMap;
	aMap.SetOrigin( LogicToPixel( Point(), aSimMap ) );
	SetMapMode( aMap );
}

// -----------------------------------------------------------------------------

double SimWindow::GetVisibleX()
{
	if( aViewSize.Width() )
	    return ( (double) aWinPos.X() / aViewSize.Width() );
	else
		return 1.e-6;
}

// -----------------------------------------------------------------------------

double SimWindow::GetVisibleY()
{
	if( aViewSize.Height() )
	    return ( (double) aWinPos.Y() / aViewSize.Height() );
	else
		return 1.e-6;
}

// -----------------------------------------------------------------------------

void SimWindow::SetVisibleXY( double fX, double fY )
{
    if( fX >= 0 )
        aWinPos.X() = (long) ( fX * aViewSize.Width() );

    if( fY >= 0 )
        aWinPos.Y() = (long) ( fY * aViewSize.Height() );

	const Point	aNullPt;
	const Point aOldOrgPix( LogicToPixel( aNullPt, aSimMap ) );
    UpdateMapOrigin();
	const Point aNewOrgPix( LogicToPixel( aNullPt, aSimMap ) );

    Scroll( aNewOrgPix.X() - aOldOrgPix.X(), aNewOrgPix.Y() - aOldOrgPix.Y() );
}

// -----------------------------------------------------------------------------

double SimWindow::GetVisibleWidth()
{
    Size aWinSize( PixelToLogic( GetOutputSizePixel(), aSimMap ) );

    if( aWinSize.Width() > aViewSize.Width() )
        aWinSize.Width() = aViewSize.Width();

	if ( aViewSize.Width() )
    	return ( (double) aWinSize.Width() / aViewSize.Width() );
	else
		return 1.e-6;
}

// -----------------------------------------------------------------------------

double SimWindow::GetVisibleHeight()
{
    Size aWinSize( PixelToLogic( GetOutputSizePixel(), aSimMap ) );

    if( aWinSize.Height() > aViewSize.Height() )
        aWinSize.Height() = aViewSize.Height();

	if ( aViewSize.Height() )
	    return ( (double) aWinSize.Height() / aViewSize.Height() );
	else
		return 1.e-6;
}

// -----------------------------------------------------------------------------

double SimWindow::GetScrlLineWidth()
{
    return( GetVisibleWidth() * SCROLL_LINE_FACT );
}

// -----------------------------------------------------------------------------

double SimWindow::GetScrlLineHeight()
{
    return( GetVisibleHeight() * SCROLL_LINE_FACT );
}

// -----------------------------------------------------------------------------

double SimWindow::GetScrlPageWidth()
{
    return( GetVisibleWidth() * SCROLL_PAGE_FACT );
}

// -----------------------------------------------------------------------------

double SimWindow::GetScrlPageHeight()
{
    return( GetVisibleHeight() * SCROLL_PAGE_FACT );
}

// -----------------------------------------------------------------------------

BOOL SimWindow::QueryDrop( DropEvent& rDEvt )
{
	SimFuPoor* 	pFuActual = pViewShell->GetFuActual();
    BOOL		bRet = pViewShell->QueryDrop( rDEvt, this );

	if( pFuActual && ( nDropScrollCount++ > SCROLL_SENSITIVE ) )
		pFuActual->ForceScroll( GetPointerPosPixel(), SCROLL_SENSITIVE );

    return bRet;
}

// -----------------------------------------------------------------------------

BOOL SimWindow::Drop(const DropEvent& rEvt)
{
	return pViewShell->Drop( rEvt, this );
}

// -----------------------------------------------------------------------------

void SimWindow::Command( const CommandEvent& rCEvt )
{
	if ( !pViewShell->Command( rCEvt, this ) )
		Window::Command( rCEvt );
}
