/*************************************************************************
 *
 *  $RCSfile: viewshel.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:53:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <vcl/split.hxx>
#include <vcl/scrbar.hxx>
#include <svtools/printdlg.hxx>
#include <sfx2/dispatch.hxx>
#include <svx/svxids.hrc>
#include "simapp.hrc"
#include "simmodel.hxx"
#include "viewshel.hxx"
#include "simwin.hxx"
#include "simview.hxx"
#include "fupoor.hxx"
#include "docshell.hxx"

// -----------
// - Defines -
// -----------

#define SIM_OBJECTSHELL()	((SimDrawDocShell*)GetViewFrame()->GetObjectShell())
#define FRAMEWINDOW()		(&GetViewFrame()->GetWindow())
#define MIN_ZOOM			10
#define MAX_ZOOM			1600
#define DELTA_ZOOM			10
#define FRAME_OFFSET		8

// ----------------
// - SimViewShell -
// ----------------

TYPEINIT1(SimViewShell, SfxViewShell);

SimViewShell::SimViewShell( SfxViewFrame* pFrame, Window *pParent, BOOL bAllowCenter ) :
			SfxViewShell			( pFrame, SFX_VIEW_MAXIMIZE_FIRST | SFX_VIEW_OPTIMIZE_EACH | SFX_VIEW_CAN_PRINT ),
			pView					( NULL ),
			pFuActual				( NULL ),
			pFuOld					( NULL ),
			nCurrentObjectBar		( 0 ),
			bObjectBarSwitchEnabled	( TRUE ),
			bCenterAllowed			( bAllowCenter ),
			bDragging				( FALSE ),
			bIPZoomSet				( FALSE )
{
	Construct();
}

// -----------------------------------------------------------------------------

SimViewShell::SimViewShell( SfxViewFrame* pFrame, const SimViewShell& rShell ) :
			SfxViewShell			( pFrame, SFX_VIEW_MAXIMIZE_FIRST | SFX_VIEW_OPTIMIZE_EACH | SFX_VIEW_CAN_PRINT ),
			pView					( NULL ),
			pFuActual				( NULL ),
			pFuOld					( NULL ),
			nCurrentObjectBar		( 0 ),
			bObjectBarSwitchEnabled	( TRUE ),
			bCenterAllowed			( rShell.bCenterAllowed ),
			bDragging				( FALSE ),
			bIPZoomSet				( FALSE )
{
	Construct();
}

// -----------------------------------------------------------------------------

SimViewShell::~SimViewShell()
{
	USHORT nX, nY;

	delete pHSplit;
	delete pVSplit;
	delete pBox;
	delete pFuActual;

	// ScrollBars zerstoeren
	for( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
		delete pHScrlArray[ nX ];

	for( nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		delete pVScrlArray[ nY ]; 

	for( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
		for( nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
			delete pWinArray[ nX ][ nY ];

	// Shells fuer Object Bars loeschen; vorm Zerstoeren der ObjectBarShells den Dispatcher flushen
	GetViewFrame()->GetDispatcher()->Flush();

	for( void* pObjBarShell = aShellTable.First(); pObjBarShell; pObjBarShell = aShellTable.Next() )
		delete (SfxShell*) pObjBarShell;
}

// -----------------------------------------------------------------------------

void SimViewShell::Construct( void )
{
	Window*		pParent = FRAMEWINDOW();
	const Range aRange( 0, 32000 );

	SetWindow( pWindow = new SimWindow( pParent, this ) );
	pWindow->SetCenterAllowed( bCenterAllowed );

	// alle Zeiger mit NULL initialisieren
	for( USHORT nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		pHScrlArray[nX] = NULL;

		for ( USHORT nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		{
			pVScrlArray[nY] = NULL;
			pWinArray[nX][nY] = NULL;
		}
	}

	// Fenster dem WinArray zuweisen
	pWinArray[0][0] = pWindow;

	// horz. Scrollbar erzeugen und init.
	pHScrlArray[0] = new ScrollBar( pParent, WB_HSCROLL );
	pHScrlArray[0]->SetRange( aRange );
	pHScrlArray[0]->SetScrollHdl( LINK( this, SimViewShell, HScrollHdl ) );
	pHScrlArray[0]->EnableDrag( TRUE );
	pHScrlArray[0]->Show();

	// vert. Scrollbar erzeugen und init.
	pVScrlArray[0] = new ScrollBar( pParent, WB_VSCROLL );
	pVScrlArray[0]->SetRange( aRange );
	pVScrlArray[0]->SetScrollHdl( LINK( this, SimViewShell, VScrollHdl ) );
	pVScrlArray[0]->EnableDrag( TRUE );
	pVScrlArray[0]->Show();

	// horz- und vert. Splitter erzeugen
	pHSplit = new Splitter( pParent, WB_HSCROLL );
	pHSplit->SetSplitHdl( LINK(this, SimViewShell, SplitHdl ) );
	pHSplit->Show();

	pVSplit = new Splitter( pParent, WB_VSCROLL );
	pVSplit->SetSplitHdl( LINK(this, SimViewShell, SplitHdl ) );
	pVSplit->Show();

	// ScrollBarBox anzeigen
	pBox = new ScrollBarBox( pParent, WB_3DLOOK | WB_SIZEABLE );
	pBox->Show();

	aScrBarWH = Size( pVScrlArray[ 0 ]->GetSizePixel().Width(), pHScrlArray[ 0 ]->GetSizePixel().Height() );
	bIsHSplit = bIsVSplit = FALSE;

	SetMousePosInfo();
	SetMouseRectInfo();
}

// -----------------------------------------------------------------------------

void SimViewShell::SetMousePosInfo()
{
	aMouseInfo.maMousePos = Point( -1, - 1 );
	SFX_BINDINGS().Invalidate( SID_STATUS_MOUSEPOS );
}

// -----------------------------------------------------------------------------

void SimViewShell::SetMousePosInfo( const Point& rMousePos )
{
	const Size aImgSizePix( pView->GetSizePixel() );

	if( rMousePos.X() >= 0 && rMousePos.X() < aImgSizePix.Width() &&
		rMousePos.Y() >= 0 && rMousePos.Y() < aImgSizePix.Height() )
	{
		aMouseInfo.maMousePos = rMousePos;
	}
	else
		aMouseInfo.maMousePos = Point( -1, - 1 );

	SFX_BINDINGS().Invalidate( SID_STATUS_MOUSEPOS );
}

// -----------------------------------------------------------------------------

void SimViewShell::SetMouseRectInfo()
{
	aMouseInfo.maSelRect.SetEmpty();
	SFX_BINDINGS().Invalidate( SID_ATTR_POSITION );
	SFX_BINDINGS().Invalidate( SID_ATTR_SIZE );
}

// -----------------------------------------------------------------------------

void SimViewShell::SetMouseRectInfo( const Rectangle& rSelRect )
{
	aMouseInfo.maSelRect = pView->GetRectPixel();
	aMouseInfo.maSelRect.Intersection( rSelRect );
	SFX_BINDINGS().Invalidate( SID_ATTR_POSITION );
	SFX_BINDINGS().Invalidate( SID_ATTR_SIZE );
}

// -----------------------------------------------------------------------------

void SimViewShell::Cancel()
{
	if( pFuActual )
	{
		SimFuPoor* pTemp = pFuActual;
		pFuActual = NULL;
		pTemp->Deactivate();
		delete pTemp;
	}

	if( pFuOld )
	{
		pFuActual = pFuOld;
		pFuActual->Activate();
	}
}

// -----------------------------------------------------------------------------

void SimViewShell::Activate( BOOL bIsMDIActivate )
{
	SfxViewShell::Activate( bIsMDIActivate );

	if( pFuActual && bIsMDIActivate )
		pFuActual->Activate();

	SIM_OBJECTSHELL()->Connect( this );
}

// -----------------------------------------------------------------------------

void SimViewShell::Deactivate( BOOL bIsMDIActivate )
{
	SIM_OBJECTSHELL()->Disconnect( this );

	if( pFuActual && bIsMDIActivate )
		pFuActual->Deactivate();

	SfxViewShell::Deactivate( bIsMDIActivate );
}

// -----------------------------------------------------------------------------

BOOL SimViewShell::KeyInput( const KeyEvent& rKEvt, SimWindow* pWin )
{
	BOOL bRet = FALSE;

	if( pWin )
	{
		pWindow = pWin;

		if ( pFuActual )
			pFuActual->SetWindow( pWin );
	}

	if( pFuActual )
		bRet = pFuActual->KeyInput( rKEvt );

	if( !bRet && pWindow )
	{
		const KeyCode aKeyCode( rKEvt.GetKeyCode() );

		if( aKeyCode.IsMod1() && aKeyCode.IsShift() && ( aKeyCode.GetCode() == KEY_R ) )
		{
			pView->Invalidate();
			bRet = TRUE;
		}
	}

	if( !bRet )
		bRet = SfxViewShell::KeyInput( rKEvt );

	return bRet;
}

// -----------------------------------------------------------------------------

void SimViewShell::MouseButtonDown( const MouseEvent& rMEvt, SimWindow* pWin )
{
	if( pWin )
	{
		if( pWin != pWindow )
		{
			pWindow = pWin;

			if( pFuActual )
				pFuActual->SetWindow( pWin );
		}

		pWin->GrabFocus();
	}

	if( pFuActual )
		pFuActual->MouseButtonDown( rMEvt );
}

// -----------------------------------------------------------------------------

void SimViewShell::MouseMove( const MouseEvent& rMEvt, SimWindow* pWin )
{
	if( pWin && ( pWin != pWindow ) )
	{
		pWindow = pWin;

		if ( pFuActual )
			pFuActual->SetWindow( pWin );
	}

	if( pFuActual )
		pFuActual->MouseMove( rMEvt );
}

// -----------------------------------------------------------------------------

void SimViewShell::MouseButtonUp(const MouseEvent& rMEvt, SimWindow* pWin)
{
	if( pWin && ( pWin != pWindow ) )
	{
		pWindow = pWin;

		if( pFuActual )
			pFuActual->SetWindow( pWin );
	}

	if( pFuActual )
		pFuActual->MouseButtonUp( rMEvt );
}

// -----------------------------------------------------------------------------

void SimViewShell::Flush()
{
	if( pFuActual )
		pFuActual->Flush();
}

// -----------------------------------------------------------------------------

void SimViewShell::InitFrameChange()
{
	if( pFuActual )
		pFuActual->Deactivate();
}

// -----------------------------------------------------------------------------

void SimViewShell::EndFrameChange()
{
	InitWindows();

	if( pFuActual )
		pFuActual->Activate();

	Invalidate();
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimViewShell, SplitHdl, Splitter*, pSplit )
{
	Point		aSplitPos( pSplit->GetPosPixel() );
	const long	nNewSplit = pSplit->GetSplitPosPixel();

	if( pSplit == pHSplit )
	{
		bIsHSplit = TRUE;
		aSplitPos.X() = nNewSplit;
	}
	else if( pSplit == pVSplit )
	{
		bIsVSplit = TRUE;
		aSplitPos.Y() = nNewSplit;
	}

	pSplit->SetPosPixel( aSplitPos );
	OuterResizePixel( aViewPos, aViewSize );

	return 0L;
}

// -----------------------------------------------------------------------------

void SimViewShell::CreateHSplitElems( long nSplitXPixel )
{
	if( bIsHSplit )
	{
		if( !pHScrlArray[1] )
		{
			Point	aPos( pWinArray[0][0]->GetWinViewPos() );
			Window*	pFrameWindow = FRAMEWINDOW();

			pHScrlArray[1] = new ScrollBar( pFrameWindow, WB_HSCROLL );
			pHScrlArray[1]->SetRange( Range( 0, 32000 ) );
			pHScrlArray[1]->SetScrollHdl( LINK( this, SimViewShell, HScrollHdl ) );
			pHScrlArray[1]->EnableDrag( TRUE );
			pHScrlArray[1]->Show();

			pWinArray[1][0] = new SimWindow( pFrameWindow, this );
			pWinArray[1][0]->SetCenterAllowed( bCenterAllowed );

			aPos.X() += pWindow->PixelToLogic( Size( nSplitXPixel, 0 ), pWindow->GetSimMap() ).Width();
			pWinArray[1][0]->ShareViewArea( pWinArray[ 0 ][ 0 ] );
			pWinArray[1][0]->SetWinViewPos( aPos );
			pWinArray[1][0]->Show();
			pView->AddOutputDevice( pWinArray[1][0] );


			if( bIsVSplit )
			{
				pWinArray[1][1] = new SimWindow( pFrameWindow, this );
				pWinArray[1][1]->SetCenterAllowed( bCenterAllowed );

				aPos.Y() = pWinArray[0][1]->GetWinViewPos().Y();
				pWinArray[1][1]->ShareViewArea( pWinArray[1][0] );
				pWinArray[1][1]->SetWinViewPos( aPos );
				pWinArray[1][1]->Show();
				pView->AddOutputDevice( pWinArray[1][1] );
			}
		}
	}
	else
	{
		delete pHScrlArray[1];
		pHScrlArray[1] = NULL;

		if( pWindow == pWinArray[1][0] || pWindow == pWinArray[1][1] )
		{
			pWindow = pWinArray[0][0];

			if( pFuActual )
				pFuActual->SetWindow( pWindow );
		}

		if( pWinArray[1][0] )
		{
			pView->DelOutputDevice( pWinArray[1][0] );
			delete pWinArray[1][0];
			pWinArray[1][0] = NULL;
		}

		if( pWinArray[1][1] )
		{
			pView->DelOutputDevice( pWinArray[1][1] );
			delete pWinArray[1][1];
			pWinArray[1][1] = NULL;
		}
	}
}

// -----------------------------------------------------------------------------

void SimViewShell::CreateVSplitElems( long nSplitYPixel )
{
	if( bIsVSplit )
	{	
		if( !pVScrlArray[1] )
		{
			Point	aPos( pWinArray[0][0]->GetWinViewPos() );
			Window*	pFrameWindow = FRAMEWINDOW();

			pVScrlArray[1] = new ScrollBar( pFrameWindow, WB_VSCROLL );
			pVScrlArray[1]->SetRange( Range( 0, 32000 ) );
			pVScrlArray[1]->SetScrollHdl( LINK( this, SimViewShell, VScrollHdl ) );
			pVScrlArray[1]->EnableDrag( TRUE );
			pVScrlArray[1]->Show();

			pWinArray[0][1] = new SimWindow( pFrameWindow, this );
			pWinArray[0][1]->SetCenterAllowed( bCenterAllowed );

			aPos.Y() += pWindow->PixelToLogic( Size( 0, nSplitYPixel ), pWindow->GetSimMap() ).Height();
			pWinArray[0][1]->ShareViewArea( pWinArray[0][0] );
			pWinArray[0][1]->SetWinViewPos( aPos );
			pWinArray[0][1]->Show();
			pView->AddOutputDevice( pWinArray[0][1] );


			if ( bIsHSplit )
			{
				pWinArray[1][1] = new SimWindow( pFrameWindow, this );
				pWinArray[1][1]->SetCenterAllowed( bCenterAllowed );

				aPos.X() = pWinArray[1][0]->GetWinViewPos().X();
				pWinArray[1][1]->ShareViewArea( pWinArray[0][1] );
				pWinArray[1][1]->SetWinViewPos( aPos );
				pWinArray[1][1]->Show();
				pView->AddOutputDevice( pWinArray[1][1] );
			}
		}
	}
	else
	{
		delete pVScrlArray[1];
		pVScrlArray[1] = NULL;

		if( pWindow == pWinArray[0][1] || pWindow == pWinArray[1][1] )
		{
			pWindow = pWinArray[0][0];

			if( pFuActual )
				pFuActual->SetWindow( pWindow );
		}

		if ( pWinArray[0][1] )
		{
			pView->DelOutputDevice( pWinArray[0][1] );
			delete pWinArray[0][1];
			pWinArray[0][1] = NULL;
		}

		if ( pWinArray[1][1] )
		{
			pView->DelOutputDevice( pWinArray[1][1] );
			delete pWinArray[1][1];
			pWinArray[1][1] = NULL;
		}
	}
}

// -----------------------------------------------------------------------------

void SimViewShell::UpdateScrollBars()
{
	USHORT	i;

	for ( i = 0; ( i < MAX_HSPLIT_CNT ) && pHScrlArray[i]; i++ )
	{
		long nW = (long) ( pWinArray[i][0]->GetVisibleWidth() * 32000 );

		pHScrlArray[i]->SetVisibleSize( nW );
		pHScrlArray[i]->SetThumbPos( (long) ( pWinArray[i][0]->GetVisibleX() * 32000 ) );
		nW = 32000 - nW;

		pHScrlArray[i]->SetLineSize( (long) ( pWinArray[i][0]->GetScrlLineWidth() * nW ) );
		pHScrlArray[i]->SetPageSize( (long) ( pWinArray[i][0]->GetScrlPageWidth() * nW ) );
	}


	for ( i = 0; ( i < MAX_VSPLIT_CNT ) && pVScrlArray[i]; i++ )
	{
		long nH = (long) ( pWinArray[0][i]->GetVisibleHeight() * 32000 );

		pVScrlArray[i]->SetVisibleSize( nH );
		pVScrlArray[i]->SetThumbPos( (long) ( pWinArray[0][i]->GetVisibleY() * 32000) );
		nH = 32000 - nH;

		pVScrlArray[i]->SetLineSize( (long) ( pWinArray[0][i]->GetScrlLineHeight() * nH) );
		pVScrlArray[i]->SetPageSize( (long) ( pWinArray[0][i]->GetScrlPageHeight() * nH) );
	}
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimViewShell, HScrollHdl, ScrollBar*, pHScroll )
{
	const double	fX = (double) pHScroll->GetThumbPos() / pHScroll->GetRange().Len();
	USHORT			nX = 0;
	USHORT			nY = 0;

	// Spalte der zu scrollenden Fenster bestimmen
	while ( ( nX < MAX_HSPLIT_CNT ) && ( pHScroll != pHScrlArray[nX] ) )
		nX++;

	// alle Fenster der Spalte scrollen
	while ( ( nY < MAX_VSPLIT_CNT ) && pWinArray[nX][nY] )
		pWinArray[nX][nY++]->SetVisibleXY( fX, -1 );

	return 0L;
}

// -----------------------------------------------------------------------------

IMPL_LINK( SimViewShell, VScrollHdl, ScrollBar *, pVScroll )
{
	const double	fY = (double) pVScroll->GetThumbPos() / pVScroll->GetRange().Len();
	USHORT			nX = 0;
	USHORT			nY = 0;

	// Zeile der zu scrollenden Fenster bestimmen
	while ( ( nY < MAX_VSPLIT_CNT ) && ( pVScroll != pVScrlArray[nY] ) )
		nY++;

	// alle Fenster der Zeile scrollen
	while ( ( nX < MAX_HSPLIT_CNT ) && pWinArray[nX][nY] )
		pWinArray[nX++][nY]->SetVisibleXY( -1, fY );

	return 0;
}

// -----------------------------------------------------------------------------

void SimViewShell::ScrollLines( long nLinesX, long nLinesY )
{
	long	nThumbPosX, nOldX;
	long	nThumbPosY, nOldY;
	USHORT	nX;
	USHORT	nY;

	for ( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		nY = 0;

		while ( ( nY < MAX_VSPLIT_CNT ) && ( pWindow != pWinArray[nX][nY] ) )
			nY++;

		if ( nY < MAX_VSPLIT_CNT )
			break;
	}

	nOldX = nThumbPosX = pHScrlArray[nX]->GetThumbPos();
	nOldY = nThumbPosY = pVScrlArray[nY]->GetThumbPos();
	
	if ( nLinesX )
		pHScrlArray[nX]->SetThumbPos( nThumbPosX += nLinesX * pHScrlArray[nX]->GetLineSize() );

	if ( nLinesY )
		pVScrlArray[nY]->SetThumbPos( nThumbPosY += nLinesY * pVScrlArray[nY]->GetLineSize() );

	double fX = ( nThumbPosX != nOldX ) ? ( (double) nThumbPosX / pHScrlArray[nX]->GetRange().Len() ) : -1.0;
	double fY = ( nThumbPosY != nOldY ) ? ( (double) nThumbPosY / pVScrlArray[nY]->GetRange().Len() ) : -1.0;

	pWindow->SetVisibleXY( fX, fY );

	// alle Fenster der gleichen Spalte sowie Zeile scrollen
	if ( bIsHSplit || bIsVSplit )
	{
		USHORT	i;

		for ( i = 0; i < MAX_HSPLIT_CNT; i++ )
			if ( ( i != nX ) && pWinArray[i][nY] )
				pWinArray[i][nY]->SetVisibleXY( -1, fY );

		for ( i = 0; i < MAX_VSPLIT_CNT; i++ )
			if ( ( i != nY ) && pWinArray[nX][i] )
				pWinArray[nX][i]->SetVisibleXY( fX, -1 );
	}
}

// -----------------------------------------------------------------------------

void SimViewShell::SetZoom( long nZoom )
{
	for ( USHORT nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
		for ( USHORT nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
			if ( pWinArray[nX][nY] )
				pWinArray[nX][nY]->SetZoom( nZoom );

	UpdateScrollBars();
	pView->Invalidate();
	SFX_BINDINGS().Invalidate( SID_SHOW_GRID );
}

// -----------------------------------------------------------------------------

void SimViewShell::SetZoomFactor( const Fraction& rZoomX, const Fraction& rZoomY )
{
	SetZoom( (long) ( (double) rZoomX / 0.01 ) );
}

// -----------------------------------------------------------------------------

void SimViewShell::SetZoomRect(const Rectangle& rZoomRect)
{
	const long	nZoom = pWindow->SetZoomRect( rZoomRect );
	const Point aPos( pWindow->GetWinViewPos() );
	USHORT		nX, nY, nCol, nRow;

	for ( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		for ( nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		{
			if ( pWinArray[nX][nY] == pWindow )
			{
				nCol = nX;
				nRow = nY;
			}
		}
	}

	for ( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		for ( nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		{
			if ( pWinArray[nX][nY] && ( pWinArray[nX][nY] != pWindow ) )
			{
				Point aNewPos( pWinArray[nX][nY]->GetWinViewPos() );

				if (nX == nCol)
					aNewPos.X() = aPos.X();

				if (nY == nRow)
					aNewPos.Y() = aPos.Y();

				pWinArray[nX][nY]->SetZoom( nZoom );
				pWinArray[nX][nY]->SetWinViewPos( aNewPos );
				pWinArray[nX][nY]->UpdateMapOrigin();
			}
		}
	}

	UpdateScrollBars();
	pView->Invalidate();
	SFX_BINDINGS().Invalidate( SID_SHOW_GRID );
}

// -----------------------------------------------------------------------------

void SimViewShell::SetZoomPoint( const Point& rCenter, long nZoom )
{
	pWindow->SetZoomPoint( rCenter, nZoom );

	const Point aPos( pWindow->GetWinViewPos() );
	USHORT		nX, nY, nCol, nRow;

	for ( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		for ( nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		{
			if ( pWinArray[nX][nY] == pWindow )
			{
				nCol = nX;
				nRow = nY;
			}
		}
	}

	for ( nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		for ( nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		{
			if ( pWinArray[nX][nY] && ( pWinArray[nX][nY] != pWindow ) )
			{
				Point aNewPos( pWinArray[nX][nY]->GetWinViewPos() );

				if (nX == nCol)
					aNewPos.X() = aPos.X();

				if (nY == nRow)
					aNewPos.Y() = aPos.Y();

				pWinArray[nX][nY]->SetZoom( nZoom );
				pWinArray[nX][nY]->SetWinViewPos( aNewPos );
				pWinArray[nX][nY]->UpdateMapOrigin();
			}
		}
	}

	UpdateScrollBars();
	pView->Invalidate();
	SFX_BINDINGS().Invalidate( SID_SHOW_GRID );
}

// -----------------------------------------------------------------------------

long SimViewShell::GetIPZoom( const Size& rLogSize100 )
{
	return pWindow->GetZoom();
}

// -----------------------------------------------------------------------------

void SimViewShell::InitWindows()
{
	if( pView->GetCurFrame() )
	{
		Size		aViewSize( pView->GetSizePixel() );
		Point		aViewOrg( FRAME_OFFSET, FRAME_OFFSET );
		const Point aWinViewPos;

		aViewSize.Width() += FRAME_OFFSET << 1;
		aViewSize.Height() += FRAME_OFFSET << 1;

		for ( USHORT nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
		{
			for ( USHORT nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
			{
				if ( pWinArray[nX][nY] )
				{
					pWinArray[nX][nY]->SetViewOrigin( aViewOrg );
					pWinArray[nX][nY]->SetViewSize( aViewSize );
					pWinArray[nX][nY]->SetWinViewPos( aWinViewPos );
					pWinArray[nX][nY]->UpdateMapOrigin();
				}
			}
		}
	}
	
	UpdateScrollBars();
	pView->Invalidate();
	SFX_BINDINGS().Invalidate( SID_SHOW_GRID );
}

// -----------------------------------------------------------------------------

USHORT SimViewShell::SwitchObjectBar( USHORT nResId )
{
	const USHORT nRet = nCurrentObjectBar;

	if ( ( nCurrentObjectBar != nResId ) && bObjectBarSwitchEnabled )
	{
		if ( nCurrentObjectBar )
			SetSubShell( NULL );

		nCurrentObjectBar = nResId;

		if ( nCurrentObjectBar )
			SetSubShell( (SfxShell*) aShellTable.Get( nCurrentObjectBar ) );
	}

	return nRet;
}

// -----------------------------------------------------------------------------

BOOL SimViewShell::QueryDrop( DropEvent& rEvt, SimWindow* pWin )
{
	return pView->QueryDrop( rEvt );
}

// -----------------------------------------------------------------------------

BOOL SimViewShell::Drop( const DropEvent& rEvt, SimWindow* pWin )
{
	BOOL bRet = FALSE;

	if( !pFuActual || !pFuActual->IsDragging() || ( pFuActual->GetDragFramePos() != pView->GetCurFramePos() ) )
		bRet = pView->Drop( rEvt );

	return bRet;
}

// -----------------------------------------------------------------------------

long SimViewShell::GetHCtrlWidth()
{
	return 0;
}

// -----------------------------------------------------------------------------

void SimViewShell::InnerResizePixel( const Point &rNewPos, const Size &rNewSize )
{
	// given: Point and Size of EditWindow _without_ controls
	// do: arrange controls around this rectangle

	if( rNewSize.Width() && rNewSize.Height() )
	{
		const Size aOuterSize( rNewSize.Width() + aScrBarWH.Width(), 
							   rNewSize.Height() + aScrBarWH.Height() );

		AdjustControls( rNewPos, aOuterSize );

		if( !bIPZoomSet )
		{
			SetZoom( GetIPZoom( pWindow->PixelToLogic( rNewSize, MAP_100TH_MM ) ) );
			bIPZoomSet = TRUE;
		}
		else
		{
			UpdateScrollBars();	
			pView->Invalidate();
		}
	}
}

// -----------------------------------------------------------------------------

void SimViewShell::OuterResizePixel( const Point &rNewPos, const Size &rNewSize )
{
	// given: Point and Size of EditWindow _with_ controls
	// do: arrange controls inside this rectangle
	if( rNewSize.Width() && rNewSize.Height() )
	{
		AdjustControls( rNewPos, rNewSize );
		UpdateScrollBars();
		pView->Invalidate();
	}
}

// -----------------------------------------------------------------------------

void SimViewShell::AdjustControls( const Point &rNewPos, const Size &rNewSize )
{
	const SvBorder	aSvBorder( 0, 0, aScrBarWH.Width(), aScrBarWH.Height() );
	const long		nEditWidth = rNewSize.Width() - aScrBarWH.Width();
	const long		nEditHeight = rNewSize.Height() - aScrBarWH.Height();
	const long		nSplitExt = Max( aScrBarWH.Width() >> 2L, 1L );
	const Rectangle	aDragRect( rNewPos, Size( nEditWidth , nEditHeight ) );
	Point			aHPos0( rNewPos.X() + GetHCtrlWidth(), rNewPos.Y() + rNewSize.Height() - aScrBarWH.Height() );
	Point			aVPos0( rNewPos.X() + rNewSize.Width() - aScrBarWH.Width(), rNewPos.Y() );
	Point			aHSplitPos, aVSplitPos;
	Point			aHPos1, aVPos1;
	Size			aHSiz0, aHSiz1;
	Size			aVSiz0, aVSiz1;
	Size			aHSplitSize, aVSplitSize;
	
	SetBorderPixel( aSvBorder );
	aViewPos = rNewPos, aViewSize = rNewSize;

	// Horizontaler Splitter
	aHSplitPos = Point( pHSplit->GetPosPixel().X(), aHPos0.Y() );
	pHSplit->SetDragRectPixel( aDragRect, FRAMEWINDOW() );

	if( !bIsHSplit || ( ( aHSplitPos.X() < rNewPos.X() + MIN_SCROLLBAR_SIZE ) || 
						( aHSplitPos.X() > rNewPos.X() + nEditWidth - MIN_SCROLLBAR_SIZE ) ) )
	{
		aHSplitPos.X() = rNewPos.X() + nEditWidth - nSplitExt;
		bIsHSplit = FALSE;
	}

	// horz. Splitter-Elemente erzeugen
	CreateHSplitElems( aHSplitPos.X() + nSplitExt );

	// Splitter ggf. auf gesamte Hoehe erweitern
	aHSplitSize.Width() = nSplitExt;

	if( bIsHSplit )
	{
		aHSplitPos.Y() = rNewPos.Y();
		aHSplitSize.Height() = rNewSize.Height();
	}
	else
		aHSplitSize.Height() = aScrBarWH.Height();

	// horz. Scrollbars
	aHSiz0.Width() = aHSplitPos.X() - aHPos0.X();
	aHSiz0.Height() = aScrBarWH.Height();

	if( aHSiz0.Width() < MIN_SCROLLBAR_SIZE )
	{
		aHPos0.X() = aHSplitPos.X() - MIN_SCROLLBAR_SIZE;
		aHSiz0.Width() = MIN_SCROLLBAR_SIZE;
	}

	// Splitter positionieren
	pHSplit->SetPosSizePixel( aHSplitPos, aHSplitSize );
	pHScrlArray[0]->SetPosSizePixel( aHPos0, aHSiz0 );

	// 2.Scrollbar?
	if( pHScrlArray[1] )
	{
		aHPos1.X() = aHPos0.X() + aHSiz0.Width() + nSplitExt;
		aHPos1.Y() = aHPos0.Y();
		aHSiz1 = Size( nEditWidth - aHPos1.X(), aScrBarWH.Height() );
		pHScrlArray[ 1 ]->SetPosSizePixel( aHPos1, aHSiz1 );
	}

	// vertikale Splitter
	aVSplitPos = Point( aVPos0.X(), pVSplit->GetPosPixel().Y() );
	pVSplit->SetDragRectPixel( aDragRect, FRAMEWINDOW() );

	if( !bIsVSplit || ( ( aVSplitPos.Y() < rNewPos.Y() + MIN_SCROLLBAR_SIZE ) || 
						( aVSplitPos.Y() > rNewPos.Y() + nEditHeight - MIN_SCROLLBAR_SIZE ) ) )
	{
		aVSplitPos.Y() = rNewPos.Y() + nEditHeight - nSplitExt;
		bIsVSplit = FALSE;
	}

	// horz. Splitter-Elemente erzeugen
	CreateVSplitElems( aVSplitPos.Y() + nSplitExt );

	// Splitter ggf. auf gesamte Hoehe erweitern
	aVSplitSize.Height() = nSplitExt;

	if( bIsVSplit )
	{
		aVSplitPos.X() = rNewPos.X();
		aVSplitSize.Width() = rNewSize.Width();
	}
	else
		aVSplitSize.Width() = aScrBarWH.Width();

	// horz. Scrollbars
	aVSiz0.Width() = aScrBarWH.Width();
	aVSiz0.Height() = aVSplitPos.Y() - aVPos0.Y();

	if( aVSiz0.Height() < MIN_SCROLLBAR_SIZE )
	{
		aVPos0.Y() = aVSplitPos.Y() - MIN_SCROLLBAR_SIZE;
		aVSiz0.Height() = MIN_SCROLLBAR_SIZE;
	}

	// Splitter positionieren
	pVSplit->SetPosSizePixel( aVSplitPos, aVSplitSize );
	pVScrlArray[0]->SetPosSizePixel( aVPos0, aVSiz0 );

	// 2.Scrollbar?
	if( pVScrlArray[1] )
	{
		aVPos1.X() = aVPos0.X();
		aVPos1.Y() = aVPos0.Y() + aVSiz0.Height() + nSplitExt;
		aVSiz1 = Size( aScrBarWH.Width(), nEditHeight - aVPos1.Y() );
		pVScrlArray[ 1 ]->SetPosSizePixel( aVPos1, aVSiz1 );
	}

	// Setzen der Splitter-Windows
	for( USHORT nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
	{
		for ( USHORT nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
		{
			Point	aPos;
			Size	aSize;

			if( pWinArray[nX][nY] )
			{
				if( !nX )
				{
					aPos.X() = rNewPos.X();

					if( !bIsHSplit )
						aSize.Width() = nEditWidth;
					else
						aSize.Width() = aHPos0.X() + aHSiz0.Width() - rNewPos.X();
				}
				else
				{
					aPos.X() = aHPos1.X();
					aSize.Width() = aHSiz1.Width();
				}

				if( !nY )
				{
					aPos.Y() = rNewPos.Y();
					
					if( !bIsVSplit )
						aSize.Height() = nEditHeight;
					else
						aSize.Height() = aVPos0.Y() + aVSiz0.Height() - rNewPos.Y();
				}
				else
				{
					aPos.Y() = aVPos1.Y();
					aSize.Height() = aVSiz1.Height();
				}

				pWinArray[nX][nY]->SetPosSizePixel( aPos, aSize );
				pWinArray[nX][nY]->UpdateMapOrigin();
			}
		}
	}

	pBox->SetPosPixel( Point( rNewPos.X() + nEditWidth, rNewPos.Y() + nEditHeight ) );
}

// -----------------------------------------------------------------------------

PrintDialog* SimViewShell::CreatePrintDialog(Window* pParent)
{
	PrintDialog*	pDlg =	new PrintDialog( pParent);
	const ULONG		nFrameCount = pView->GetFrameCount();

    pDlg->SetFirstPage( 1 );
    pDlg->SetLastPage( nFrameCount );
	pDlg->EnableRange( PRINTDIALOG_ALL );

	if( nFrameCount > 1 )
	{
		pDlg->EnableRange( PRINTDIALOG_RANGE );
		pDlg->EnableRange( PRINTDIALOG_FROMTO );
		pDlg->EnableCollate();
	}
	else
	{
		pDlg->DisableRange( PRINTDIALOG_RANGE );
		pDlg->DisableRange( PRINTDIALOG_FROMTO );
		pDlg->EnableCollate( FALSE );
	}

	pDlg->SetRangeText( pView->GetCurFramePos() + 1 );

    return pDlg;
}

// -----------------------------------------------------------------------------

void SimViewShell::EnumSimWindows( const Link& rEnumLink )
{
	for ( USHORT nX = 0; nX < MAX_HSPLIT_CNT; nX++ )
		for ( USHORT nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
			if ( pWinArray[nX][nY] )
				rEnumLink.Call( pWinArray[nX][nY] );
}

// ------------------------------------------------------------------------

BOOL SimViewShell::Command( const CommandEvent& rCEvt, SimWindow* pWin )
{
	BOOL bDone = FALSE;

	switch( rCEvt.GetCommand() )
	{
		case( COMMAND_WHEEL ):
		case( COMMAND_STARTAUTOSCROLL ):
		case( COMMAND_AUTOSCROLL ):
		{
			const CommandWheelData* pData = rCEvt.GetWheelData();

			if( pData && pData->IsMod1() )
			{
				const long	nOldZoom = pWindow->GetZoom();
				long		nNewZoom;

				if( pData->GetDelta() < 0L )
					nNewZoom = Max( (long)MIN_ZOOM, (long)(nOldZoom - DELTA_ZOOM) );
				else
					nNewZoom = Min( (long)MAX_ZOOM, (long)(nOldZoom + DELTA_ZOOM) );

				if( nNewZoom != nOldZoom )
				{
					SetZoom( nNewZoom );
					SFX_BINDINGS().Invalidate( SID_ATTR_ZOOM );
				}

				bDone = TRUE;
			}
			else
			{
				ScrollBar*	pWinHScroll = NULL;
				ScrollBar*	pWinVScroll = NULL;

				for( USHORT nX = 0; nX < MAX_HSPLIT_CNT && !bDone; nX++ )
				{
					for( USHORT nY = 0; nY < MAX_VSPLIT_CNT; nY++ )
					{
						if( pWinArray[ nX ][ nY ] == pWin )
						{
							bDone = pWin->HandleScrollCommand( rCEvt,
															   pHScrlArray[ nX ],
															   pVScrlArray[ nY ] );
							break;
						}
					}
				}
			}
		}
		break;

		default:
		break;
	}

	return bDone;
}

// -----------------------------------------------------------------------------

void SimViewShell::HandleSimError( ULONG nSimErrorCode )
{
	switch( nSimErrorCode )
	{
		case( SIM_ERROR_OUTOFMEMORY ):
			ErrorHandler::HandleError( ERRCODE_IO_OUTOFMEMORY );
		break;

		default:
		break;
	}
}
