/*************************************************************************
 *
 *  $RCSfile: advise.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _ADVISE_HXX
#define _ADVISE_HXX

#include <iface.hxx>
#include <dtrans.hxx>
#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

/*************************************************************************
*************************************************************************/
class  ImpViewObject;
class  ImpAdviseSink;
class  ImpOleObject;
class  SvAdvise;
class  SvAdviseSink;
class  SvSoIPCService;
struct IAdviseSink;
struct IAdviseSink2;

SO2_DECL_REF(SvAdvise)

/************** class SvAdviseSink **************************************/
#if __PRIVATE
DECLARE_UNIQUEINDEX( SvAdviseSinkIndex, SvAdviseSink * )
#endif
class SvAdviseSink : virtual public SvObject
{
friend ImpAdviseSink;
friend ImpViewObject; // pAdviseSink wird direkt gesetzt
friend SvAdvise;
friend SvSoIPCService;
	IAdviseSink2 *  pObjI;
	IAdviseSink *   pAdviseSink; // nur extern
	SvAdviseRef     xAdvise;
protected:
	virtual void    Closed();
	virtual void    Saved();
	virtual void    ViewChanged( USHORT nAspects );
	virtual void    DataChanged( SvData & );
					~SvAdviseSink();
public:
					SvAdviseSink();
					SO2_DECL_STANDARD_CLASS_DLL(SvAdviseSink,SOAPP)
	IAdviseSink *   GetAdviseSink() const;
	IAdviseSink2 *  GetAdviseSink2() const;

	SvAdvise *      GetSource() { return &xAdvise; }
	static SvAdviseSink *  Get( ULONG nUniqueId );
	ULONG           GetUniqueId() const;
};
#ifndef SO2_DECL_SVADVISESINK_DEFINED
#define SO2_DECL_SVADVISESINK_DEFINED
SO2_DECL_REF(SvAdviseSink)
#endif
SO2_IMPL_REF(SvAdviseSink)

/************** class SvAdviseSelect *************************************/
#define ADVISE_CLOSED           0x01
#define ADVISE_SAVE             0x02
#define ADVISE_RENAME           0x04
#define ADVISE_LINKSRCCHANGE    0x08
#define ADVISE_CONNECTSTD       0x0F
#define ADVISE_DATA             0x10
#define ADVISE_VIEW             0x20

// Muessen die gleichen Werte wie Ole2 ADVF_* sein
#define ADVISEMODE_NODATA       0x01
#define ADVISEMODE_PRIMEFIRST   0x02
#define ADVISEMODE_ONLYONCE     0x04
#define ADVISEMODE_DATAONSTOP   0x40

class SvAdvSelector
{
	SvAdviseSinkRef aSink;
	USHORT          nAdviseTypes;
public:
	TYPEINFO();
			SvAdvSelector( SvAdviseSink * pSink, USHORT nAdviseTypes );
	virtual ~SvAdvSelector();


	USHORT  GetAdviseTypes() const { return nAdviseTypes; }
	SvAdviseSink * GetSink() const { return aSink; }
};

class SvAdvDataSelector : public SvAdvSelector
{
friend SvAdvise;
	USHORT          nAdviseModes;
	SvDataType      aType;
public:
			TYPEINFO();
			SvAdvDataSelector( SvAdviseSink * pSink, const SvDataType & rType,
							   USHORT nAdviseModes );

	BOOL    bModified;
	ULONG   nConnectNo;

	USHORT  GetAdviseModes() const { return nAdviseModes; }
	const SvDataType & GetType() const { return aType; }
};

class SvAdvViewSelector : public SvAdvSelector
{
friend SvAdvise;
	USHORT          nAdviseModes;
	USHORT          nViewAspects;
public:
			TYPEINFO();
			SvAdvViewSelector( SvAdviseSink * pSink, USHORT nViewAspects,
							   USHORT nAdviseModes );

	USHORT  nModifiedAspects;
	USHORT  GetAdviseModes() const { return nAdviseModes; }
	USHORT  GetViewAspects() const { return nViewAspects; }
};

class SvAdvConnectSelector : public SvAdvSelector
{
public:
	TYPEINFO();
	SvAdvConnectSelector( SvAdviseSink * pSink, USHORT nAdviseType );
	ULONG   nConnectNo;
};

DECLARE_LIST(SvAdvSelectorList,SvAdvSelector*)

/************** class SvAdvise *******************************************/
class ImpViewObject;
class SvAdviseTimer;
class SvAdvise : virtual public SvObject
{
friend ImpViewObject;
friend ImpOleObject;
friend SvAdviseTimer;
	SvAdvSelectorList   *pSelectorList;
	ULONG               nTimeout;
	SvAdviseTimer *     pTimer;

	void            ClearSelectorList();
	void            Remove( SvAdvSelector * );

	void            StartTimer( BOOL bNewTimeOut );
	void            RemoveDataAdvise( ULONG nPos );
protected:
	SvAdvSelectorList & GetSelectorList();
	ULONG           GetSelectorCount() const
					{
						return pSelectorList ? pSelectorList->Count() : 0;
					}
	void            Clear();
	void 			SendDataOnStop();
	void            ClearExternDataAdvises();
	void            ClearExtern();
					~SvAdvise();

	void            SetUpdateTimeout( ULONG nTimeout );
	ULONG           GetUpdateTimeout() const;
public:
					SvAdvise();
					SO2_DECL_STANDARD_CLASS_DLL(SvAdvise,SOAPP)
	static ULONG    Sv2OleAdviseModes( USHORT nSvModes );
	static USHORT   Ole2SvAdviseModes( ULONG nSysModes );

	SvAdvDataSelector *     GetDataSelector( ULONG nConnect ) const;
	SvAdvViewSelector *     GetViewSelector() const;
	SvAdvConnectSelector *  GetConnectSelector( ULONG nConnect ) const;

	BOOL            Closed();
	void            Saved();

	void            SendViewChanged();
	void            ViewChanged( USHORT nAspects );

	void            SendDataChanged( SvData & rSendData );
	void            DataChanged( SvData & );
	void            DataChanged( const SvDataType & rType )
					{
						SvData aData( rType );
						DataChanged( aData );
					}

	BOOL            AddViewAdvise( SvAdviseSink *,
								   USHORT nAspect = ASPECT_CONTENT,
								   USHORT nAdviseModes = 0 );
	void            RemoveViewAdvise( SvAdviseSink *,
									  USHORT nAspects = ASPECT_CONTENT );

	BOOL            AddDataAdvise( SvAdviseSink *, const SvDataType &,
								   USHORT nAdviseModes = 0 );
	void            RemoveDataAdvise( SvAdviseSink *, const SvDataType & );
	void            RemoveDataAdvise( SvAdviseSink * );
	void 			RemoveAllDataAdvise( SvAdviseSink * );

	BOOL            AddConnectAdvise( SvAdviseSink *, USHORT nAdviseTypes );
	void            RemoveConnectAdvise( SvAdviseSink *, USHORT nAdviseTypes );
};
SO2_IMPL_REF(SvAdvise)

#endif // _ADVISE_HXX
