/*************************************************************************
 *
 *  $RCSfile: bindctx.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _BINDCTX_HXX
#define _BINDCTX_HXX

#include <iface.hxx>
#include <binding.hxx>

class  SvStatusCallback;
class  SvData;
class  SvDataTypeList;
class  Impl_BindContext;
struct IBindStatusCallback;
struct IRunningObjectTable;
struct IBindCtx;
struct IBinding;

#ifndef SO2_DECL_SVSTORAGESTREAM_DEFINED
#define SO2_DECL_SVSTORAGESTREAM_DEFINED
SO2_DECL_REF(SvStorageStream)
#endif


//========================================================================
typedef UINT32 SvCallbackOption;
#define	CALLBACK_ONSTARTBINDING		0x1
#define	CALLBACK_GETPRIORITY		0x2
#define	CALLBACK_ONLOWRESOURCE		0x4
#define	CALLBACK_ONPROGRESS			0x8
#define	CALLBACK_ONSTOPBINDING		0x10
#define	CALLBACK_GETBINDINFO		0x20
#define	CALLBACK_ONDATAAVAILABLE	0x40
#define	CALLBACK_ONOBJECTAVAILABLE	0x80
#define	CALLBACK_ALLONIBSC			0xff
#define	CALLBACK_ALLONIBDGSITE		0x1f

class SvBindContext : virtual public SvObject
/*	[Beschreibung]

	Bei einem Verbindungsaufbau zu einem Objekt, werden eventuell
	zus"atzliche Informationen ben"otigt. Diese k"onnen am BindContext
	untergebracht werden.
*/
{
	union
	{
		IBindCtx *  		pObjI;
		Impl_BindContext*	pImp;
	};
	SvDataTypeList *	pTypeList;
protected:
						~SvBindContext();
	void				MakeUnknown();
public:
						SvBindContext();
						SO2_DECL_STANDARD_CLASS_DLL(SvBindContext,SOAPP)

	IBindCtx *			GetBindCtx() const;

							// Properties
	StreamMode				GetStreamMode() const;
	short					GetStorageMode() const;
	void					SetMode( StreamMode, short nStorageMode );
	void					SetTypeList( const SvDataTypeList & );
	const SvDataTypeList &	GetTypeList() const;
	void					SetDeadline( UINT32 nMillSec );
	UINT32  				GetDeadline() const;

	void					Register( SvStatusCallback *,
									  SvCallbackOption = CALLBACK_ALLONIBSC );
	BOOL					Revoke( SvStatusCallback * );

	void					RegisterObjBound( SvObject * );
	void					RevokeObjBound( SvObject * );
};

#ifndef SO2_DECL_SVBINDCONTEXT_DEFINED
#define SO2_DECL_SVBINDCONTEXT_DEFINED
SO2_DECL_REF(SvBindContext)
#endif
SO2_IMPL_REF(SvBindContext)

//========================================================================
// OLE2 Conform
#define	URLINFO_URLENCODESTGMEDDATA	0x1
#define	URLINFO_URLENCODEDEXTRAINFO	0x2

class SoBindInfo
/*	[Beschreibung]

	Diese Klasse liefert dem Moniker weitere Information wie das
	Binden stattzufinden hat.
*/
{
friend class SvStatusCallback;
	String			aExtraInfo;
	SvData *		pData;
	INT32			nUrlInfo;
	SvBindAction	eBindAction;
	String			aCustomAction;
public:
					SoBindInfo();
					SoBindInfo( const String & rExtraName, INT32 eUrlInfo );
					~SoBindInfo();

	const String &	GetExtraInfo() const { return aExtraInfo; }
	const SvData &	GetData() const;
	INT32			GetUrlInfo() const { return nUrlInfo; }
	SvBindAction	GetBindAction() const { return eBindAction; }
	const String &	GetCustomAction() const { return aCustomAction; }
};

//========================================================================
class SoBinding : public SvObject
/*	[Beschreibung]

	Soll eine asynchrone Operation durchgef"uhrt werden, dann wird
	immer ein Objekt dieser Klasse zur Verf"ugung gestellt, um die
	Operation (z.B. Laden einer Datei vom Internet) zu beeinflussen.
*/
{
	friend class ImpBinding;

	union
	{
		IBinding *pObjI;
	};

public:
					SO2_DECL_STANDARD_CLASS_DLL(SoBinding, SOAPP)

					SoBinding();
					~SoBinding();

	IBinding*		GetBinding() const;

	//IBinding
	virtual ErrCode Abort();
	virtual ErrCode Suspend();
	virtual ErrCode Resume();
	virtual BOOL	SetPriority( long nPriority );
	virtual long	GetPriority();
};

#ifndef SV_DECL_SOBINDING_DEFINED
#define SV_DECL_SOBINDING_DEFINED
SV_DECL_REF(SoBinding)
#endif

SV_IMPL_REF(SoBinding)

//========================================================================
// OLE2 Conform
typedef UINT32 SvAsyncData;
#define	ASYNC_FIRSTDATA 1
#define	ASYNC_OTHERDATA 2
#define	ASYNC_LASTDATA	4

// OLE2 Conform
typedef UINT32 SvBindMode;
#define	BINDMODE_ASYNCHRONOUS			0x1
#define	BINDMODE_ASYNCSTORAGE			0x2
#define	BINDMODE_NOPROGRESSIVERENDERING	0x4
#define	BINDMODE_OFFLINEOPERATION		0x8
#define	BINDMODE_GETNEWESTVERSION		0x10
#define	BINDMODE_NOWRITECACHE			0x20
#define	BINDMODE_PULLDATA				0x80

class SvStatusCallback : public SvObject
/*	[Beschreibung]

	Wenn mit einem asynchronen Moniker (URLMoniker) asynchron
	gebunden werden soll, dann muss der Client ein Objekt dieser
	Klasse zu Verf"ugung stellen. Er wird dann "uber den Verlauf
	der Operation mit Hilfe der virtuellen Methoden informiert.

	[Anmerkung]

	Die Set-Methoden d"urfen nur verwendet werden, wenn das Objekt
	selber angelegt wurde.
*/
{
	friend class ImpBindStatusCallback;
	union
	{
		IBindStatusCallback *pObjI;
	};
	SoBindInfo			aBindInfo;
	SvBindMode			nBindMode;
	SoBindingRef    	xBinding;
	SvStorageStreamRef	xCacheStream;
public:
				SO2_DECL_STANDARD_CLASS_DLL(SvStatusCallback, SOAPP)

						SvStatusCallback();
						SvStatusCallback( SvBindContext * );
						~SvStatusCallback();
	IBindStatusCallback* GetBindStatusCallback() const;


	void				SetBindInfo( const SoBindInfo & rInfo )
						{ aBindInfo = rInfo; }
	void				SetBindMode( SvBindMode nMode )
						{ nBindMode = nMode; }

	const SoBindInfo &	GetBindInfo();
	SvBindMode			GetBindMode();
	SoBinding * 		GetBinding() { return xBinding; }

	virtual void		OnStartBinding( SoBinding * );
	virtual void        OnStopBinding( ErrCode nErr, const String & );

	virtual long 		GetPriority() const;
	virtual void 		OnProgress( ULONG nPos, ULONG nMaxPos, SvBindStatus,
									const String& rStatusText );

	virtual void 		OnDataAvailable( SvAsyncData, ULONG nBytes,
										SvData & rData );
	virtual void 		OnObjAvailable( SvObject *pObj );
};

#ifndef SO2_DECL_SVSTATUSCALLBACK_DEFINED
#define SO2_DECL_SVSTATUSCALLBACK_DEFINED
SO2_DECL_REF(SvStatusCallback)
#endif

SO2_IMPL_REF(SvStatusCallback)

#endif // _BINDCTX_HXX
