/*************************************************************************
 *
 *  $RCSfile: linkmgr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:25 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _LINKMGR_HXX
#define _LINKMGR_HXX

#ifndef _TOOLS_STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _SVARRAY_HXX
#include <svtools/svarray.hxx>
#endif
#include <so2ref.hxx>

class SvBaseLink;
class SvBaseLinkRef;
class SvPseudoObject;
class SvPersist;
class SvStream;

#ifndef SO2_DECL_SVPSEUDOOBJECT_DEFINED
#define SO2_DECL_SVPSEUDOOBJECT_DEFINED
SO2_DECL_REF(SvPseudoObject)
#endif

typedef SvBaseLinkRef* SvBaseLinkRefPtr;
SV_DECL_PTRARR( SvBaseLinks, SvBaseLinkRefPtr, 1, 1 )

typedef SvPseudoObject* SvPseudoObjectPtr;
SV_DECL_PTRARR( SvPseudoObjects, SvPseudoObjectPtr, 1, 1 )


class SvLinkManager
{
	SvBaseLinks aLinkTbl;
	SvBaseLinks aErrorLinkTbl;
	SvPseudoObjects aServerTbl;
	SvPersist	*pCacheCont; // LinkMgr muss vor SvPersist freigegeben werden

	BOOL Insert( SvBaseLink& rLink );

protected:
	void   		Load( SvStream & rStm );
	void   		Save( SvStream & rStm );
	BOOL 		InsertLink( SvBaseLink& rLink, USHORT nObjType, USHORT nUpdateType,
							const String* pName = 0 );
public:
				SvLinkManager( SvPersist * pCacheCont );
				~SvLinkManager();

	SvBaseLinks& GetErrorLinkTbl();
	void		ClearErrorLinkTbl();

	SvPersist*	GetCacheContainer() const { return pCacheCont; }
	void        SetCacheContainer( SvPersist * pCont )
				{ pCacheCont = pCont; }

				// den Link mit einem PseudoObject verbinden und in die Liste eintragen
	BOOL 		InsertDDELink( SvBaseLink&,
						const String& rServer,
						const String& rTopic,
						const String& rItem );

				// falls am Link schon alles eingestellt ist !
	BOOL 		InsertDDELink( SvBaseLink& );
	BOOL 		InsertSoLink( SvBaseLink& );

	virtual SvBaseLinkRef LoadLink( USHORT nTyp, SvStream & );
	friend SvStream & operator << ( SvStream & rStm, SvLinkManager & rThis )
				{
					rThis.Save( rStm );
					return rStm;
				}
	friend SvStream & operator >> ( SvStream & rStm, SvLinkManager & rThis )
				{
					rThis.Load( rStm );
					return rStm;
				}

				// einen Link aus der Liste entfernen
	void 		Remove( const SvBaseLink& rLink );
	void 		Remove( USHORT nPos, USHORT nCnt = 1 );

	void 		UpdateAllLinks( BOOL bAskUpdate = TRUE,
								BOOL bCallErrHdl = TRUE,
								BOOL bUpdateGrfLinks = FALSE );

				// Liste aller Links erfragen (z.B. fuer Verknuepfungs-Dialog)
	const 		SvBaseLinks& GetLinks() const { return aLinkTbl; }

				// erfrage die Strings fuer den Dialog
	virtual BOOL GetDisplayNames( const SvBaseLink&,
									String* pType,
									String* pFile = 0,
									String* pLink = 0,
									String* pFilter = 0 ) const;

	virtual SvPseudoObjectRef CreateObj( SvBaseLink & );

	// Fuer den SvxLinkManager: alle FileObjects des LinksManagers oder
	// den spziellen aus dem Cache entfernen. In der Basisimpl. passiert
	// nichts.
	virtual void PrepareReload( SvBaseLink* pLnk = 0 );

	// ----------------- Serverseitige Verwaltung --------------------

				// Liste der zu serviereden Links erfragen
	const SvPseudoObjects& GetServers() const { return aServerTbl; }
				// einen zu servierenden Link eintragen/loeschen
	BOOL 		InsertServer( SvPseudoObject* rObj );
	void 		RemoveServer( SvPseudoObject* rObj );
	void 		RemoveServer( USHORT nPos, USHORT nCnt = 1 )
				{	aServerTbl.Remove( nPos, nCnt ); }

private:
				// diese Methoden gibts nicht!
				SvLinkManager( const SvLinkManager& );
				SvLinkManager& operator=( const SvLinkManager& );
};


// -------------------------- Hilfs-Funktionen ---------------------------

// Trenner im LinkName fuer die DDE-/File-/Grafik- Links
// (nur wer es braucht, um einen SvLinkName zusammenzubasteln)
const sal_Char cTokenSeperator = '\xff';

// erzeuge einen String fuer den SvLinkName. Fuer
// - DDE die ersten 3 Strings, (Server, Topic, Item)
// - File-/Grf-LinkNms die letzen 3 Strings (FileName, Bereich, Filter)
void MakeLnkName( String& rName,
				 const String* pType,		// kann auch 0 sein !!
				 const String& rFile,
				 const String& rLink,
				 const String* pFilter = 0 );


#endif

