/*************************************************************************
 *
 *  $RCSfile: pseudo.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <pseudo.hxx>
#include <svtools/filedlg.hxx>

#ifndef _DATE_HXX //autogen
#include <tools/date.hxx>
#endif

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#include <vcl/svapp.hxx>
#include <vcl/wrkwin.hxx>
#include <vcl/menu.hxx>
#include <embobj.hxx>
#include <lnkbase.hxx>
#include <client.hxx>
#include <svstor.hxx>
#include <bindctx.hxx>
#include <soerr.hxx>
#pragma hdrstop

//=========================================================================
//==================class SvVerb===========================================
//=========================================================================
PRV_SV_IMPL_OWNER_LIST(SvVerbList,SvVerb)

//=========================================================================
SvVerb::SvVerb
(
	long nIdP,				/* Identifer des Verbs	*/
	const String & rNameP,  /* Name des Verbs, sprachabh"angig	*/
	BOOL bConstP,			/* TRUE, dieses Verb ver"andert das Objekt nicht */
	BOOL bOnMenuP			/* TRUE, dieses Verb soll im Menu erscheinen */
)
{
	nId     = nIdP;
	aName   = rNameP;
	aMenuId = GetpApp()->CreateUniqueId();
	bConst  = bConstP;
	bOnMenu = bOnMenuP;
}

//=========================================================================
SvVerb::SvVerb
(
	const SvVerb & rObj
)
{
	nId     = rObj.nId;
	aName   = rObj.aName;
	aMenuId = rObj.aMenuId;
	bConst  = rObj.bConst;
	bOnMenu = rObj.bOnMenu;
}

//=========================================================================
SvVerb & SvVerb::operator =
(
	const SvVerb & rObj
)
{
	if( this != &rObj )
	{
		nId     = rObj.nId;
		aName   = rObj.aName;
		aMenuId = rObj.aMenuId;
		bConst  = rObj.bConst;
		bOnMenu = rObj.bOnMenu;
	}
	return *this;
}

//=========================================================================
SvVerb::~SvVerb()
{
	// UniqueId wird freigegeben
}

//=========================================================================
//==================class SvPseudoObject===================================
//=========================================================================
SV_OLE_FACTORY(SvPseudoObjectFactory)
	{
	}
};
TYPEINIT1(SvPseudoObjectFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS2_DLL(SvPseudoObject,SvPseudoObjectFactory,SvDataObject,SvAdvise,
							 0x2A499E61L, 0x733F, 0x101C,
							 0x8D,0x86,0x4A,0x04,0x12,0x94,0x26,0x0D)


//=========================================================================
SvPseudoObject::SvPseudoObject()
/*	[Beschreibung]

	Konstruktor der Klasse SvPseudoObject. 
*/
	: pVerbs		( NULL )
	, bIsAddAdvise	( FALSE )
	, bDeleteVerbs	( FALSE )
	, nOleROTReg	( 0 )        // nicht in der ROT registriert
	, pROT			( NULL )
{
}

//=========================================================================
SvPseudoObject::~SvPseudoObject()
/*	[Beschreibung]

	Destruktor der Klasse SvPeudoObjekt. Das Objekt tr"agt sich selber
	aus der <Running Object Table> aus. 
*/
{
	DBG_ASSERT( !bIsAddAdvise, "data advise remain connected" )
	if( bDeleteVerbs ) // Selbst erzeugt ?
		delete pVerbs;
}

//=========================================================================
IUnknown * SvPseudoObject::GetMemberInterface( const SvGlobalName & )
/*	[Beschreibung]

	Jede Klasse mu eine Abfrage auf die Ole-Interfaces unterst"utzen.
	Es d"urfen nur die eigenen Ole-Interfaces und nicht die der
	Super-Klassen abgefragt werden. Siehe <So Ole-Unterst"utzung>.
*/
{
	return NULL;
}

//=========================================================================
#ifdef TEST_INVARIANT

void SvPseudoObject::TestMemberObjRef( BOOL /*bFree*/ )
/*	[Beschreibung]

	Siehe <So Debugging>.
*/
{
}

//=========================================================================

void SvPseudoObject::TestMemberInvariant( BOOL bPrint )
/*	[Beschreibung]

	Siehe <So Debugging>.
*/
{
}
#endif

//=========================================================================
ULONG SvPseudoObject::Update()
/*	[Beschreibung]

	Die Methode arbeitet rekursiv und bringt alle Daten (Caches) auf den
	aktuellen Stand. Hierfr ist es notwendig alle Objekte und alle ihre
	Server zu laden bzw. zu starten.

	Die wichtigsten Fehlermeldungen:
	-ERRCODE_SO_CANT_BINDTOSOURCE
	-ERRCODE_SO_NOCACHE_UPDATED

	Die wichtigsten Warnungen:
	-ERRCODE_SO_SOMECACHES_NOTUPDATED
*/
{
	return ERRCODE_NONE;
}

//=========================================================================
ULONG SvPseudoObject::GetUpToDateStatus()
/*	[Beschreibung]

	Die Methode arbeitet rekursiv und stellt fest, ob alle Daten auf dem
	aktuellen Stand sind. Sind sie es, dann muss ERRCODE_NONE zurckgegeben
	werden. Dauert das ermitteln dieses Standes genau so lange wie die
	Daten auf den aktuellen Stand zu bringen, dann muss
	ERRCODE_SO_MK_UNAVAILABLE zurueckgegeben werden.
	Sind die Daten nicht auf dem aktuellen Stand, dann muss
	ERRCODE_SO_FALSE zur"uckgegeben werden.
*/
{
	if( !Owner() )
	{
		return ERRCODE_SO_GENERALERROR;
	}
	return ERRCODE_NONE;
}

//=========================================================================
BOOL SvPseudoObject::IsLink() const
/*	[Beschreibung]

	Ist dieses Objekt ein Link, dann muss TRUE zur"uckgegeben werden.
	Dies ist der Fall, wenn mit den Methoden <SvFactory::CreateAndLoad>
	oder <SvFactory::CreateAndInit> ein Link-Objekt erzeugt wird.
	Ein weiteres Beispiel ist das <DDE-PseudoObject>.
*/
{
	return FALSE;
}

//=========================================================================
void SvPseudoObject::FillClass
(
	SvGlobalName * pClassName,	/* Der Typ der Klasse */
	ULONG * pFormat,			/* Das Dateiformat in dem geschrieben wird */
	String * pAppName,			/* Der Applikationsname */
	String * pFullTypeName,     /* Der genaue Name des Typs	*/
	String * pShortTypeName,	/* Der kurze Name des Typs	*/
	long /*nFileFormat */		/* F"ur dieses Office-Format sollen die
								   Parameter gef"ullt werden */
) const
/*	[Beschreibung]

	Diese Methode liefert Informationen "uber den Typ und das Dateiformat
	des Objektes. Alle Parameter werden von der Methode gefllt.

	[R"uckgabewert]

	*pClassName 	Liefert den Typ-Identifier des Objektes.
	*pFormat		Immer 0, siehe aber <SvEmbeddedObject::FillClass>
	*pAppName		Den sprachabh"angigen Applikationsnamen.
	*pFullTypeName	Den sprachabh"angigen Namen des Typs.
	*pShortTypeName Den kurzen sprachabh"angigen Namen des Typs. Er darf
					nicht l"anger als 15 Zeichen sein.

	[Beispiel]

	MyClass::FillClass( ... )
	{
		*pClassName 	= *SvFactory::GetSvFactory(); // keine emulation
		*pFormat    	= 0;
		*pAppName       = "StarDivison Calc 3.0";
		*pFullTypeName  = "StarDivison Calc 3.0 Tabelle";
		*pShortTypeName = "Tabelle";
	}

	[Querverweise]

	<SvPseudoObject::GetFullTypeName>, <SvPseudoObject::GetShortTypeName>,
	<SvPersist::FillClass()>
*/
{
	*pFormat		= 0;
	*pFullTypeName	= *pShortTypeName = *pAppName = String();
	*pClassName		= SvGlobalName();

	if( Owner() )
	{
		*pClassName = *GetSvFactory();
		*pAppName = Application::GetDisplayName();
	}
}

//=========================================================================
SvGlobalName SvPseudoObject::GetClassName() const
/*	[Beschreibung]

	Ruft <SvPseudoObject::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName	Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return aName;
}

//=========================================================================
String SvPseudoObject::GetAppName() const
/*	[Beschreibung]

	Ruft <SvPseudoObject::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName	Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return aAppName;
}

//=========================================================================
String SvPseudoObject::GetFullTypeName() const
/*	[Beschreibung]

	Ruft <SvPseudoObject::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName	Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return aFullTypeName;
}

//=========================================================================
String SvPseudoObject::GetShortTypeName() const
/*	[Beschreibung]

	Ruft <SvPseudoObject::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName	Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return aShortTypeName;
}

//=========================================================================
ErrCode SvPseudoObject::DoVerb
(
	long nVerb,							/* Der Index in die Verbtabelle	*/
	SvEmbeddedClient * pCallerClient,   /* Die Container-Seite mit dem der
										   Objekt-Server ggf. kommuniziert.
										*/
	Window * pWin,                      /* Das Fenster in dem das Objekt
										   dargestellt wird. */
	const Rectangle * pWorkRectPixel	/* Die Position und Gr"osse des
										   Objektes im Container */
)
/*	[Beschreibung]

	Diese Methode ist eine Wrapper-Funktion um <SvPseudoObject::Verb>.
	Sie teilt dem Container zustzlich mit, dass das Objekt in den
	sichtbaren Bereich geschoben werden soll.
*/
{
	// Default-Verben sind negativ
	return Verb( nVerb, pCallerClient, pWin, pWorkRectPixel );
}

//=========================================================================
ErrCode SvPseudoObject::Verb
(
	long nVerb,							/* Der Index in die Verbtabelle	*/
	SvEmbeddedClient * pCl,   			/* Die Container-Seite mit dem der
										   Objekt-Server ggf. kommuniziert.
										*/
	Window * pWin,                      /* Das Fenster in dem das Objekt
										   dargestellt wird. */
	const Rectangle * pWorkRectPixel	/* Die Position und Gr"osse des
										   Objektes im Container */
)
/*	[Beschreibung]

	Der Objekt-Server wird mit nVerb beauftragt eine bestimmte Aktion
	auszufhren. Dies kann z.B. das Abspielen eines Sound's oder Video's
	sein. Ebenso kann es das ffnen und Anzeigen eines Dokumentes
	bedeuten. Es gibt einige Default-Verben:
	SVVERB_SHOW,      	Anzeigen des Objektes
	SVVERB_OPEN,        Anzeigen des Objektes im eigenen Dokumentfenster
	SVVERB_HIDE,		Das Objekt nicht mehr anzeigen.
	SVVERB_UIACTIVATE,  Das Objekt UI-Aktivieren.
	SVVERB_IPACTIVATE,  Das Objekt IP-Aktivieren.

	[Anmerkung]
	Einige Verben machen nur IP-Aktivierung sinn. Merh ber IP-Aktivierung
	in der Klasse <SvInPLaceObjekt>.

*/
{
	ErrCode nRet = ERRCODE_SO_GENERALERROR;
	return nRet;
}

//=========================================================================
BOOL SvPseudoObject::Close()
/*	[Beschreibung]

	Versucht alle zum Objekt bestehenden Verbindungen abzubrechen.

	[Anmerkung]

	Handelt es sich um einen Link, dann muss die Verbindung zur Quelle
	abgebrochen werden.
*/
{
	if( Owner() )
	{
		// Namen abmelden
		SendDataOnStop();
		//ClearExternDataAdvises();
		SvAdvise::Closed();
		// Links entfernen
		//ClearExtern();
		SvAdvise::Clear();
		// Alle externen Verbindungen zum Objekt l"osen
		ClearExtCon();
	}
	return TRUE;
}

//=========================================================================
void SvPseudoObject::SetVerbList
(
	SvVerbList * pVerbsP	/* die Verbliste wird "ubernommen,
							   sie darf nicht zerst"ort werden, bevor
							   sie nicht mit SetVerbList( NULL ) oder
							   automatisch im Destruktor ausgetragen wurde */
)
/*	[Beschreibung]

	Setzen der Verbliste, diese wird dann in
	<SvPseudoObject::GetVerbList> zur"uckgegeben.
*/
{
	if( bDeleteVerbs )
	{
		delete pVerbs;
		bDeleteVerbs = FALSE;
	}
	pVerbs = pVerbsP;
}


//=========================================================================
const SvVerbList & SvPseudoObject::GetVerbList() const
/*	[Beschreibung]

	Die Verbliste des Objektes wird zur"uckgegeben.
*/
{
	if( pVerbs ) // darf, wegen Unique Id's nicht geaendert werden
		return *pVerbs;

	((SvPseudoObject *)this)->pVerbs = new SvVerbList();
	((SvPseudoObject *)this)->bDeleteVerbs    = TRUE;

	return *pVerbs;
}

//=========================================================================
const SvVerb * SvPseudoObject::GetVerb
(
	USHORT nMenuId	/* Wurden Verben mit der Methode
					   <SvEmbeddedObject::AppendVerbs> an ein Menu
					   angeh"angt, dann ist dies die MenuId, mit der
					   nach dem Verb gesucht wird */
) const
/*	[Beschreibung]

	In der Verbliste wird nach einem Verb mit der Id nMenuId gesucht.

	[Anmerkung]

	Nachdem ein Menu erzeugt wurde darf die Verbliste nicht vera"ndert
	werden, da die MenuIs's in den Verben vom Zeitpunkt des erzeugens und
	nicht vom Inhalt abh"angig sind.

	[R"uckgabewert]

	const SvVerb * 	NULL, das Verb wurde nicht gefunden. Ansonsten wird
					das Verb mit der entsprechenden MenuId geliefert.
*/
{
	const SvVerbList & rVerbs = GetVerbList();
	for( ULONG i = 0; i < rVerbs.Count(); i++ )
		if( rVerbs[ i ].GetMenuId() == nMenuId )
			return &rVerbs[ i ];
	return NULL;
}

//=========================================================================
void SvPseudoObject::AppendVerbs
(
	Menu & rMenu	/* An dieses Menu werden die Verben angeh"angt */
)
/*	[Beschreibung]

	Die Verben des Objektes werden an das Menu rMenu angeh"angt, wenn
	sie das Attribut <SvVerb::IsOnMenu> == TRUE haben.
*/
{
	const SvVerbList & rVerbs = GetVerbList();
	for( ULONG i = 0; i < rVerbs.Count(); i++ )
	{
		const SvVerb & rVerb = rVerbs[ i ];
		if( rVerb.IsOnMenu() )
			rMenu.InsertItem( rVerb.GetMenuId(), rVerb.GetName() );
	}
}

//=========================================================================
BOOL SvPseudoObject::IsRunning() const
/*	[Beschreibung]

	Ist der Server des Objektes gestartet und das Objekt in diesem
	Server aktiv, dann wird TRUE zur"uckgegeben. Nach dem Aufruf von
	<SvPseudoObject::Close> d"urfen externe Objekte nicht mehr im
	Zustand IsRunning() == TRUE sein. F"ur interne Objekte gilt, Close()
	aufrufen und alle Referenzen l"osen.

	[Anmerkung]
	F"ur den Benutzer einer Klasse sollte es m"oglichst transparent
	sein, ob sich das Objekt im <Loaded State> oder im <Running State>
	befindet.
*/
{
	if( Owner() )
		return TRUE;

	return FALSE;
}

//=========================================================================
ULONG SvPseudoObject::Run()
/*	[Beschreibung]

	Das Objekt wird im Server erzeugt. Konnte das Objekt erzeugt werden,
	dann wird ERRCODE_NONE zur"uckgegeben.

*/
{
	if( Owner() )
		return ERRCODE_NONE;

	return ERRCODE_SO_GENERALERROR;
}



//=========================================================================
ULONG SvPseudoObject::GetMiscStatus() const
/*	[Beschreibung]

	Diese Methode liefert weitere Informationen "uber das Objekt.
*/
{
	ULONG nMisc = 0;
	if( !Owner() )
	{
		if( IsLink() )
			nMisc = SVOBJ_MISCSTATUS_LINK;
	}
	return nMisc;
}

/************************************************************************
|*    SvPseudoObject::Connect()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPseudoObject::Connect( SvBaseLink& rLink )
{
	BOOL bRet = Owner();
	if( bRet && !bIsAddAdvise
	  && rLink.GetObjType() != OBJECT_CLIENT_OLE_CACHE )
	{
		bIsAddAdvise = TRUE;
		AddDataAdvise( &rLink, rLink.GetContentType(),
					LINKUPDATE_ONCALL == rLink.GetUpdateMode()
						? ADVISEMODE_PRIMEFIRST | ADVISEMODE_ONLYONCE
						: 0 );
	}
	return FALSE;
}


/************************************************************************
|*    SvPseudoObject::ChangeData()
|*
|*    Beschreibung
*************************************************************************/
// wird noch durch SetData an der SvDataObject-Klasse ersetzt
BOOL SvPseudoObject::ChangeData( SvData& rData )
{
	return SetData( &rData );
}

/************************************************************************
|*    SvPseudoObject::Edit()
|*
|*    Beschreibung
*************************************************************************/
SvLinkName* SvPseudoObject::Edit( Window* pParent, const SvBaseLink& rLink )
{
	if( rLink.GetObjType() == OBJECT_CLIENT_OLE_CACHE )
	{
		DBG_ERROR( "SvPseudoObject::Edit---OBJECT_CLIENT_OLE_CACHE not implemented" )
	}
	return NULL;
}

//=========================================================================
SvConnectedObject::SvConnectedObject
(
	SvEmbeddedObject * pDocP	/* Das Dokument (Objekt), in dem das Objekt
								   enthalten ist. */
)
	: pDoc( pDocP )
/*	[Beschreibung]

	Konstruktor der Klasse SvConnectedObject. pDocP darf nicht NULL sein.
	Das Dokument-Objekt wird sofort gelocked.

	[Querverweise]

	<SvObjectContaioner::Lock>
*/
{
}

//=========================================================================
SvConnectedObject::~SvConnectedObject()
/*	[Beschreibung]

	Destruktor der Klasse SvConnectedObject.
*/
{
}

//=========================================================================
BOOL SvConnectedObject::Close()
/*	[Beschreibung]

	Gibt den Lock auf das Dokument-Objekt wieder frei.

	[Querverweise]

	<SvObjectContaioner::Lock>
*/
{
	// Erst alle externen Verbindungen freigeben, dann den Container rufen
	return SvPseudoObject::Close();
}

//=========================================================================
ErrCode SvConnectedObject::Verb
(
	long nVerbPos,
	SvEmbeddedClient * pCallerClient,
	Window * pWin,
	const Rectangle * pWorkAreaPixel
)
/*	[Beschreibung]

	Ruft an dem Dokument, in dem das Objekt enthalten ist,
	"<SvPseudoObject::DoVerb>( SVVERB_SHOW )" auf.
*/
{
	//return pDoc->DoVerb( SVVERB_SHOW, pCallerClient, pWin, pWorkAreaPixel );
	return pDoc->DoVerb( SVVERB_SHOW );
}

//=========================================================================
USHORT SvConnectedObject::FuzzyLock
(
	BOOL bLock,		/* TRUE, lock. FALSE, unlock. */
	BOOL bIntern,	/* TRUE, es handelt sich um einen internen Lock.
					   FALSE, der Lock kam von aussen (Ole2, Ipc2) */
	BOOL bClose		/* TRUE, Close aufrufen wenn letzte Lock */
)
/*	[Beschreibung]

	"Uberspringen SvPseudoObject ist richtig, da ich selber
	DoClose rufe.

	[Querverweise]

	<SvPseudoObject::FuzzyLock>, <SvObject::FuzzyLock>
*/
{
	SvObjectRef xHoldAlive( this );
	if( !bIntern )
		pDoc->Lock( bLock );
	return SvPseudoObject::FuzzyLock( bLock, bIntern, bClose );
}

//=========================================================================
void SvConnectedObject::FillClass
(
	SvGlobalName * pClassName,
	ULONG * pFormat,
	String * pAppName,
	String * pFullTypeName,
	String * pShortTypeName
) const
/*	[Beschreibung]

	Leitet die Methode 1 zu 1 an das im Konstruktor "ubergeben Objekt
	weiter.
*/
{
	pDoc->FillClass( pClassName, pFormat, pAppName,
					pFullTypeName, pShortTypeName );
}

//=========================================================================
const SvVerbList & SvConnectedObject::GetVerbList() const
/*	[Beschreibung]

	Leitet die Methode 1 zu 1 an das im Konstruktor "ubergeben Objekt
	weiter.
*/
{
	return pDoc->GetVerbList();
}


