/*************************************************************************
 *
 *  $RCSfile: linkname.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>


#include <linkname.hxx>
#include <tools/debug.hxx>
#include <svstor.hxx>
#include <soerr.hxx>
#pragma hdrstop

/************** class SvLinkName ***********************************/
SV_OLE_FACTORY(SvLinkNameFactory)
	{
	}
};
TYPEINIT1(SvLinkNameFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS1_DLL(SvLinkName,SvLinkNameFactory,SvObject,
							 0x251864C0L, 0xB775, 0x101B,
							 0x80, 0x4C, 0xFD, 0xFD, 0xFD, 0xFD, 0xFD, 0xFD )

IUnknown * SvLinkName::GetMemberInterface( const SvGlobalName & )
{
	return NULL;
}

void SvLinkName::TestMemberObjRef( BOOL /*bFree*/ )
{
}

#ifdef TEST_INVARIANT
void SvLinkName::TestMemberInvariant( BOOL /*bPrint*/ )
{
}
#endif


/***********************************************************************/
/***********************************************************************/
struct Impl_Moniker
{
	SvGlobalName	aMonikerType;

	virtual			~Impl_Moniker() {}
	virtual Impl_Moniker * Create() const;

	virtual	String	GetName() const { return String(); }
	virtual	BOOL    ParseName( const String & ) { return FALSE; }

	virtual void	Save( SvStream & rStm ) const;
};

Impl_Moniker * LoadMoniker( SvStream & rStm );
void SaveMoniker( Impl_Moniker * pMon, SvStream & rStm );

/***********************************************************************/
Impl_Moniker * Impl_Moniker::Create() const
{
	return new Impl_Moniker( *this );
}

/***********************************************************************/
void Impl_Moniker::Save( SvStream & /*rStm*/ ) const
{
}

DECLARE_LIST(MonikerList,Impl_Moniker *)

/***********************************************************************/
/***********************************************************************/
struct Impl_FileMoniker : public Impl_Moniker
{
	USHORT			nAntiCount; // 0 == absolut, sonst Anzahl der "..\" oder "../" ...
	String			aPath;		// Pfadname ohne "..\" oder "../" ...
	USHORT			nVolumeOffset; // Zeichen im Path, die das Volume auszeichnen
	SvGlobalName 	aOle1;
	UINT32			nFileKind;

					Impl_FileMoniker()
					: nAntiCount( 0 )
					, nVolumeOffset( 0 )
					, nFileKind( 2 ) // 2 Ole2, 0 Ole1
					{ aMonikerType = SOAPP->aFileMoniker; }
	virtual Impl_Moniker * Create() const;

	virtual	String	GetName() const;
	virtual	BOOL	ParseName( const String & );

	virtual void	Save( SvStream & rStm ) const;
	void			Load( SvStream & rStm );
};

/***********************************************************************/
Impl_Moniker * Impl_FileMoniker::Create() const
{
	return new Impl_FileMoniker( *this );
}

/***********************************************************************/
String Impl_FileMoniker::GetName() const
{
	DBG_ERROR( "Impl_FileMoniker::GetName() not implemented" )
	return String();
}

/***********************************************************************/
BOOL Impl_FileMoniker::ParseName( const String &  )
{
	DBG_ERROR( "Impl_FileMoniker::ParseName() not implemented" )
	return FALSE;
}

/***********************************************************************/
void Impl_FileMoniker::Save( SvStream & rStm ) const
{
	rStm << (UINT16)nAntiCount;
	UINT32 nLen = aPath.Len();
	if( nLen )
	{
		rStm << (UINT32)(nLen +1);
		ByteString aStr( aPath, gsl_getSystemTextEncoding() );
		rStm << aStr.GetBuffer();
		rStm << '\0';
	}
	else
		rStm << nLen;

	// sind verdreht, entgegen der Ole2-Doku
	rStm << nVolumeOffset;
	rStm << 0xDEAD;

	rStm << aOle1;
	rStm << (UINT32)nFileKind;
	rStm << (UINT32)0;
}

/***********************************************************************/
void Impl_FileMoniker::Load( SvStream & rStm )
{
	rStm >> nAntiCount;
	UINT32 nLen;
	rStm >> nLen;
	if( nLen )
	{
		DBG_ASSERT( nLen <= STRING_MAXLEN, "string buffer overflow" )
		sal_Char * pBuf = new sal_Char[ nLen ];
		rStm.Read( pBuf, nLen );
		aPath.Assign( String( pBuf, nLen, gsl_getSystemTextEncoding() ) );
		delete pBuf;
	}

	// sind verdreht, entgegen der Ole2-Doku
	rStm >> nVolumeOffset;
	UINT16 nDEAD;
	rStm >> nDEAD;

	rStm >> aOle1;
	rStm >> nFileKind;
	UINT32 nRemain;
	rStm >> nRemain;
	rStm.SeekRel( nRemain );
}

/***********************************************************************/
/***********************************************************************/
struct Impl_PackagerMoniker : public Impl_FileMoniker
{
					Impl_PackagerMoniker()
					{ aMonikerType = SOAPP->aPackagerMoniker; }
	virtual Impl_Moniker * Create() const;
};

/***********************************************************************/
Impl_Moniker * Impl_PackagerMoniker::Create() const
{
	return new Impl_PackagerMoniker( *this );
}

/***********************************************************************/
struct Impl_ItemMoniker : public Impl_Moniker
{
	String	aDelemitter;
	String	aName;
					Impl_ItemMoniker()
					: aDelemitter( '!' )
					{ aMonikerType = SOAPP->aItemMoniker; }
	virtual Impl_Moniker * Create() const;

	virtual	String	GetName() const;
	virtual	BOOL	ParseName( const String & );

	virtual void	Save( SvStream & rStm ) const;
	void			Load( SvStream & rStm );
};

/***********************************************************************/
Impl_Moniker * Impl_ItemMoniker::Create() const
{
	return new Impl_ItemMoniker( *this );
}

/***********************************************************************/
String Impl_ItemMoniker::GetName() const
{
	String a = aDelemitter;
	a += aName;
	return a;
}

/***********************************************************************/
BOOL Impl_ItemMoniker::ParseName( const String & rName )
{
	sal_Unicode c = rName.GetChar(0);
	if( '\\' == c || '/' == c || '!' == c || '[' == c )
		aName = rName.Copy( 1 );
	else if( aDelemitter == rName )
		aName = rName.Copy( aDelemitter.Len() );
	else
		return FALSE;
	return TRUE;
}

/***********************************************************************/
void Impl_ItemMoniker::Save( SvStream & rStm ) const
{
	UINT32 nLen = aDelemitter.Len();
	if( nLen )
	{
		rStm << (UINT32)(nLen +1);
		ByteString aStr( aDelemitter, gsl_getSystemTextEncoding() );
		rStm << aStr.GetBuffer();
		rStm << '\0';
	}
	else
		rStm << nLen;

	nLen = aName.Len();
	if( nLen )
	{
		rStm << (UINT32)(nLen +1);
		ByteString aStr( aName, gsl_getSystemTextEncoding() );
		rStm << aStr.GetBuffer();
		rStm << '\0';
	}
	else
		rStm << nLen;
}

/***********************************************************************/
void Impl_ItemMoniker::Load( SvStream & rStm )
{
	UINT32 nLen;
	rStm >> nLen;
	if( nLen )
	{
		DBG_ASSERT( nLen <= STRING_MAXLEN, "string buffer overflow" )
		sal_Char * pBuf = new sal_Char[ nLen ];
		rStm.Read( pBuf, nLen );
		aDelemitter.Assign( String( pBuf, nLen, gsl_getSystemTextEncoding() ) );
		delete pBuf;
	}
	nLen = 0;
	rStm >> nLen;
	if( nLen )
	{
		DBG_ASSERT( nLen <= STRING_MAXLEN, "string buffer overflow" )
		sal_Char * pBuf = new sal_Char[ nLen ];
		rStm.Read( pBuf, nLen );
		aName.Assign( String( pBuf, nLen, gsl_getSystemTextEncoding() ) );
		delete pBuf;
	}
}

/***********************************************************************/
/***********************************************************************/
struct Impl_AntiMoniker : public Impl_Moniker
{
	UINT32	nAntiCount;
					Impl_AntiMoniker()
					{ aMonikerType = SOAPP->aItemMoniker; }
	virtual Impl_Moniker * Create() const;

	virtual void	Save( SvStream & rStm ) const
					{
						rStm << nAntiCount;
					}
	void			Load( SvStream & rStm )
					{
						rStm >> nAntiCount;
					}
};

/***********************************************************************/
Impl_Moniker * Impl_AntiMoniker::Create() const
{
	return new Impl_AntiMoniker( *this );
}

/***********************************************************************/
/***********************************************************************/
struct Impl_CompositeMoniker : public Impl_Moniker
{
	MonikerList aList;
					Impl_CompositeMoniker()
					  : aList( 2, 2 )
					{ aMonikerType = SOAPP->aCompositeMoniker; }
					Impl_CompositeMoniker( const Impl_CompositeMoniker & );
					~Impl_CompositeMoniker();

	Impl_CompositeMoniker & operator = ( const Impl_CompositeMoniker & );
	virtual Impl_Moniker * Create() const;
	void			Append( const Impl_CompositeMoniker * );
	void			Append( const Impl_Moniker * );

	void			ClearList();
	virtual	String	GetName() const;
	virtual	BOOL	ParseName( const String & );

	virtual void	Save( SvStream & rStm ) const;
	void			Load( SvStream & rStm );
};

/***********************************************************************/
Impl_CompositeMoniker::Impl_CompositeMoniker( const Impl_CompositeMoniker & rObj )
	: aList( 2, 2 )
{
	Append( &rObj );
}

/***********************************************************************/
Impl_CompositeMoniker & Impl_CompositeMoniker::operator = ( const Impl_CompositeMoniker & rObj )
{
	if( this != &rObj )
	{
		ClearList();
		Append( &rObj );
	}
	return *this;
}

/***********************************************************************/
Impl_CompositeMoniker::~Impl_CompositeMoniker()
{
	ClearList();
}

/***********************************************************************/
Impl_Moniker * Impl_CompositeMoniker::Create() const
{
	return new Impl_CompositeMoniker( *this );
}

/***********************************************************************/
void Impl_CompositeMoniker::Append( const Impl_CompositeMoniker * pMon )
{
	if( pMon )
	{
		ULONG nCount = pMon->aList.Count();
		for( ULONG i = 0; i < nCount; i++ )
			aList.Insert( pMon->aList.GetObject( i )->Create(), LIST_APPEND );
	}
}

/***********************************************************************/
void Impl_CompositeMoniker::Append( const Impl_Moniker * pMon )
{
	if( pMon )
	{
		if( pMon->aMonikerType == SOAPP->aCompositeMoniker )
			Append( (Impl_CompositeMoniker *)pMon );
		else
			aList.Insert( pMon->Create(), LIST_APPEND );
	}
}

/***********************************************************************/
void Impl_CompositeMoniker::ClearList()
{
	Impl_Moniker * pM = aList.First();
	while( pM )
	{
		delete pM;
		pM = aList.Next();
	}
}

/***********************************************************************/
String Impl_CompositeMoniker::GetName() const
{
	String aStr;
	for( ULONG i = 0; i < aList.Count(); i++ )
		aStr += aList.GetObject( i )->GetName();
	return aStr;
}

/***********************************************************************/
BOOL Impl_CompositeMoniker::ParseName( const String & rName )
{
	// andere Delemitter ueberschneiden sich mit Dateitrennern
	USHORT nPos  = rName.Search( '!' );
	USHORT nPos1 = rName.Search( '[' );
	if( nPos1 < nPos )
		nPos = nPos1;

	MonikerList aL( 2, 2 );
	BOOL bRet = TRUE;
	// Ab dem ersten Delemitter abschneiden
	String aStr = rName;
	aStr.Erase( nPos );
	if( aStr.Len() )
	{
		Impl_FileMoniker * pMon = new Impl_FileMoniker;
		aL.Insert( pMon, LIST_APPEND );
		bRet = pMon->ParseName( aStr );
	}

	aStr = rName;
	aStr.Erase( 0, nPos );
	while( aStr.Len() && bRet )
	{
		nPos  = aStr.Search( '!', 1 );
		nPos1 = aStr.Search( '[', 1 );
		if( nPos1 < nPos )
			nPos = nPos1;
		nPos1 = aStr.Search( '\\', 1 );
		if( nPos1 < nPos )
			nPos = nPos1;
		nPos1 = aStr.Search( '/', 1 );
		if( nPos1 < nPos )
			nPos = nPos1;

		// Die naechste Sektion rausschneiden
		String aS = aStr.Copy( 0, nPos );
		aStr.Erase( 0, nPos );

		Impl_ItemMoniker * pMon = new Impl_ItemMoniker;
		aL.Insert( pMon, LIST_APPEND );
		bRet = pMon->ParseName( aS );
	}
	if( bRet )
	{
		ClearList();
		aList = aL;
	}
	return bRet;
}

/***********************************************************************/
void Impl_CompositeMoniker::Save( SvStream & rStm ) const
{
	rStm << (UINT32)aList.Count();
	for( ULONG i = 0; i < aList.Count(); i++ )
		SaveMoniker( aList.GetObject( i ), rStm );
}

/***********************************************************************/
void Impl_CompositeMoniker::Load( SvStream & rStm )
{
	UINT32 nMonCount = 0;
	rStm >> nMonCount;
	for( UINT32 i = 0; i < nMonCount; i++ )
	{
		Impl_Moniker * pM = LoadMoniker( rStm );
		if( pM )
			aList.Insert( pM, LIST_APPEND );
		// leere Moniker nicht beachten
	}
}

/***********************************************************************/
/***********************************************************************/
Impl_Moniker * ParseName( const String & rName )
{
	Impl_Moniker * pRetMon = NULL;
	Impl_CompositeMoniker * pMon = new Impl_CompositeMoniker;
	pMon->ParseName( rName );
	if( pMon->aList.Count() == 1 )
		pRetMon = pMon->aList.GetObject( 0 )->Create();
	else if( pMon->aList.Count() > 1 )
	{
		pRetMon = pMon;
		pMon = NULL;
	}
	delete pMon;
	return pRetMon;
}

/***********************************************************************/
Impl_Moniker * LoadMoniker( SvStream & rStm )
{
	if( rStm.GetError() != SVSTREAM_OK )
		return NULL;

	UINT32 nLen;
	rStm >> nLen;

	// es ist ein leerer Moniker
	if( !nLen )
		return NULL;

	// Miniker Typ lesen
	SvGlobalName aGN;
	rStm >> aGN;

	Impl_Moniker * pMon = NULL;
	if( aGN == SOAPP->aCompositeMoniker )
	{
		pMon = new Impl_CompositeMoniker;
		((Impl_CompositeMoniker *)pMon)->Load( rStm );
	}
	else if( aGN == SOAPP->aFileMoniker )
	{
		pMon = new Impl_FileMoniker;
		((Impl_FileMoniker *)pMon)->Load( rStm );
	}
	else if( aGN == SOAPP->aItemMoniker )
	{
		pMon = new Impl_ItemMoniker;
		((Impl_ItemMoniker *)pMon)->Load( rStm );
	}
	else if( aGN == SOAPP->aAntiMoniker )
	{
		pMon = new Impl_AntiMoniker;
		((Impl_AntiMoniker *)pMon)->Load( rStm );
	}
	else if( aGN == SOAPP->aPackagerMoniker )
	{
		pMon = new Impl_PackagerMoniker;
		((Impl_PackagerMoniker *)pMon)->Load( rStm );
	}
	else
	{
		DBG_ERROR( "LoadMoniker() --- unknown moniker" )
		rStm.SetError( SVSTREAM_GENERALERROR );
	}

	DBG_ASSERT( rStm.GetError() == SVSTREAM_OK, "cannot load Moniker's" )
	if( rStm.GetError() != SVSTREAM_OK )
	{
		delete pMon;
		pMon = NULL;
	}
	return pMon;
}

/***********************************************************************/
void SaveMoniker( Impl_Moniker * pMon, SvStream & rStm )
{
	if( !pMon )
		rStm << (UINT32)0;
	else
	{
		ULONG nOldPos = rStm.Tell();
		rStm.SeekRel( 4 );
		rStm << pMon->aMonikerType;
		pMon->Save( rStm );
		ULONG nNewPos = rStm.Tell();
		rStm.Seek( nOldPos );
		// geammtgroesse schreiben
		rStm << (nNewPos - nOldPos);
		rStm.Seek( nNewPos );
	}
}

/************************************************************************
|*    SvLinkName::SvLinkName()
|*
|*    Beschreibung
*************************************************************************/
#define INIT_CTOR() : pImpl( NULL )

/***********************************************************************/
SvLinkName::SvLinkName()
	INIT_CTOR()
{
}

/***********************************************************************/
SvLinkName::SvLinkName( const String & rName )
	INIT_CTOR()
{
	if( rName.Len() )
	{
		pImpl = new Impl_ItemMoniker;
		((Impl_ItemMoniker *)pImpl)->aName = rName;
	}
}

/***********************************************************************/
SvLinkName::SvLinkName( const String & rName, const String & rDelemitter )
	INIT_CTOR()
{
	if( rName.Len() )
	{
		pImpl = new Impl_ItemMoniker;
		((Impl_ItemMoniker *)pImpl)->aName = rName;
		((Impl_ItemMoniker *)pImpl)->aDelemitter = rDelemitter;
	}
}

/***********************************************************************/
SvLinkName::SvLinkName( SvLinkName * pFirst, SvLinkName * pRest )
	INIT_CTOR()
{
	if( pFirst && pFirst->pImpl )
	{
		if( pRest && pRest->pImpl )
		{
			pImpl = new Impl_CompositeMoniker;
			((Impl_CompositeMoniker *)pImpl)->Append( pFirst->pImpl );
			((Impl_CompositeMoniker *)pImpl)->Append( pRest->pImpl );
		}
		else
			pImpl = pFirst->pImpl->Create();
	}
	if( pRest && pRest->pImpl )
		pImpl = pRest->pImpl->Create();
}

/************************************************************************
|*    SvLinkName::~SvLinkName()
|*
|*    Beschreibung
*************************************************************************/
SvLinkName::~SvLinkName()
{
	delete pImpl;
}

/************************************************************************
|*    SvStorageStream & operator >> ()
|*    SvStorageStream & operator << ()
|*
|*    Beschreibung
*************************************************************************/
SvStream & operator >> ( SvStream & rStm, SvLinkName & rThis )
{
	rThis.pImpl = LoadMoniker( rStm );
	return rStm;
}

SvStream & operator << ( SvStream & rStm, const SvLinkName & rThis )
{
	SaveMoniker( rThis.pImpl, rStm );
	return rStm;
}

/*************************************************************************
|*    SvLinkName::GetDisplayName()
|*    SvLinkName::GetName()
|*
|*    Beschreibung
*************************************************************************/
String SvLinkName::GetDisplayName() const
{
	String aName;

	if( pImpl )
		return pImpl->GetName();
	return aName;
}

String SvLinkName::GetName() const
{
	String aName = GetDisplayName();
	if( pImpl && pImpl->aMonikerType == SOAPP->aFileMoniker )
		// Beim File-Moniker nicht den Delemitter wegnehmen
		return aName;
	if( aName.GetChar( 0 ) == '\\' || aName.GetChar( 0 ) == '/'
	  || aName.GetChar( 0 ) == '!' || aName.GetChar( 0 ) == '[' )
		aName.Erase( 0, 1 );
	return aName;
}

