/*************************************************************************
 *
 *  $RCSfile: persist.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>

#include <persist.hxx>
#include <svstor.hxx>
#include <embobj.hxx>
#include <bindctx.hxx>
#include <pseudo.hxx>   // fuer Hack
#include <soerr.hxx>   // fuer Hack
#include <outplace.hxx>   // im- export

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#pragma hdrstop


/********************** SvPersistAdviseSink *******************************
**************************************************************************/
class SvPersistAdviseSink : public SvAdviseSink
{
	SvPersist * pOwner;
public:
			SvPersistAdviseSink( SvPersist * pOwnerP )
			{ pOwner = pOwnerP; }
	void    DataChanged( SvData & )
			{ pOwner->SetModified( TRUE ); }
};

/************** class SvInfoObject ***************************************/
/*************************************************************************/
SV_IMPL_PERSIST1(SvInfoObject,SvPersistBase)

/************************************************************************
|*    SvInfoObject::SvInfoObject()
|*    SvInfoObject::~SvInfoObject()
|*
|*    Beschreibung
*************************************************************************/
SvInfoObject::SvInfoObject()
	: bDeleted( FALSE )
{
}

SvInfoObject::SvInfoObject( SvPersist * pObj )
	: bDeleted( FALSE )
{
	SetObj( pObj );
	aObjName    = pObj->GetStorage()->GetName();
	aStorName   = aObjName;
}

SvInfoObject::SvInfoObject( SvPersist * pObj, const String & rName )
	: bDeleted( FALSE )
{
	SetObj( pObj );
	aObjName    = rName;
}

SvInfoObject::~SvInfoObject()
{
}

/************************************************************************
|*    SvInfoObject::CreateCopy()
|*    SvInfoObject::MakeCopy()
|*
|*    Beschreibung
*************************************************************************/
SvInfoObjectRef SvInfoObject::CreateCopy() const
{
	SvCreateInstancePersist pFunc = SOAPP->aInfoClassMgr.Get( GetClassId() );
	SvInfoObject* pI;
	SvPersistBase* pB;
	(*pFunc)(&pB);
//    CreateInstance( &pB );

	pI = PTR_CAST(SvInfoObject,pB);
	DBG_ASSERT( pI, "cannot cast" );
	SvInfoObjectRef xI( pI );
	xI->Assign( this );
	return xI;
}

void SvInfoObject::Assign( const SvInfoObject * pObj )
{
	aObjName     = pObj->GetObjName();
	aStorName    = pObj->GetStorageName();
	aSvClassName = pObj->GetClassName();
}

/************************************************************************
|*    SvInfoObject::Load()
|*    SvInfoObject::Save()
|*
|*    Beschreibung
*************************************************************************/
#define INFO_OBJECT_VER_MIN     (BYTE)0
#define INFO_OBJECT_VER_AKT     (BYTE)1

void SvInfoObject::Load( SvPersistStream & rStm )
{
	BYTE nVers = 0;
	rStm >> nVers;
	// wegen Warning unter OS2 auskommentiert
	DBG_ASSERT( /*nVers >= INFO_OBJECT_VER_MIN &&*/
				nVers <= INFO_OBJECT_VER_AKT,
				"SvInfoObject version conflict" )
	if( nVers > INFO_OBJECT_VER_AKT )
		rStm.SetError( SVSTREAM_WRONGVERSION );
	else
	{
		rStm.ReadByteString( aStorName, gsl_getSystemTextEncoding() );
		rStm.ReadByteString( aObjName, gsl_getSystemTextEncoding() );
		if( !aObjName.Len() )
			aObjName = aStorName;
		rStm >> aSvClassName;
		if( aSvClassName == *SvInPlaceObject::ClassFactory()
		  || aSvClassName == *SvEmbeddedObject::ClassFactory() )
		{
			// use SvOutPlaceObject class as ole wrapper
			aSvClassName = *SvOutPlaceObject::ClassFactory();
		}
		if(nVers > 0)
			rStm >> bDeleted;
	}
}

void SvInfoObject::Save( SvPersistStream & rStm )
{
	BYTE    nVer = BYTE(INFO_OBJECT_VER_AKT);
	rStm << nVer;
	String aStgName = GetStorageName();
	DBG_ASSERT( aStgName.Len(), "kein Storage-Name" )
	rStm.WriteByteString( aStgName, gsl_getSystemTextEncoding() );
	String aStr( GetObjName() );
	if( aStgName == aStr )
		aStr = String(); // kleine Optimierung
	rStm.WriteByteString( aStr, gsl_getSystemTextEncoding() );
	SvGlobalName aSvClass = SvFactory::GetSvClass( rStm.GetVersion(), GetClassName() );
	if( rStm.GetVersion() <= SOFFICE_FILEFORMAT_40
	  && aSvClass == *SvOutPlaceObject::ClassFactory() )
	  //old ole-wrapper
	  aSvClass = *SvInPlaceObject::ClassFactory();
	rStm << aSvClass;
	rStm << bDeleted;
}

/************************************************************************
|*    SvInfoObject::SetObj()
|*
|*    Beschreibung
*************************************************************************/
void SvInfoObject::SetObj( SvPersist * pObj )
{
	aObj = pObj;
	if( pObj )
		aSvClassName = *pObj->GetSvFactory();
}

/************************************************************************
|*    SvInfoObject::GetObjName()
|*    SvInfoObject::GetStorageName()
|*
|*    Beschreibung
*************************************************************************/
String SvInfoObject::GetObjName() const
{
	return aObjName;
}

String SvInfoObject::GetStorageName() const
{
	if( GetPersist() && !GetPersist()->bOpHandsOff )
		((SvInfoObject *)this)->aStorName = GetPersist()->GetStorage()->GetName();
	if( aStorName.Len() )
		return aStorName;
	return aObjName;
}

SvGlobalName SvInfoObject::GetClassName() const
{
	if( GetPersist() )
		// es muss die Sv-Klasse zurueckgegeben werden
		((SvInfoObject *)this)->aSvClassName = *GetPersist()->GetSvFactory();
	return aSvClassName;
}

//=========================================================================
//==================class SvObjectContainer================================
//=========================================================================
SV_OLE_FACTORY(SvObjectContainerFactory)
	{
	}
};
TYPEINIT1(SvObjectContainerFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS1_DLL(SvObjectContainer,SvObjectContainerFactory,SvObject,
							 0x96dee2a1, 0x62f6, 0x11cf,
							 0x89, 0xca, 0x0, 0x80, 0x29, 0xe4, 0xb0, 0xb1 )

//=========================================================================
IUnknown * SvObjectContainer::GetMemberInterface( const SvGlobalName & )
{
	return NULL;
}

//=========================================================================
#ifdef TEST_INVARIANT
void SvObjectContainer::TestMemberObjRef( BOOL /*bFree*/ )
{
}

//=========================================================================
void SvObjectContainer::TestMemberInvariant( BOOL /*bPrint*/ )
{
}
#endif

//=========================================================================
SvObjectContainer::SvObjectContainer()
	: pObjI( NULL )
/*  [Beschreibung]

	Konstruktor der Klasse SvObjectContainer.
*/
{
}

//=========================================================================
SvObjectContainer::~SvObjectContainer()
/*  [Beschreibung]

	Destruktor der Klasse SvObjectContainer.
*/
{
	// Parent haelt immer Referenz, deswegen muss der das Objekt
	// schon Removed sein
}


//=========================================================================
ULONG SvObjectContainer::GetObj
(
	SvObjectRef &     xRet,     /* hierin wird das gebundene Objekt
								   oder NULL zurueckgegeben */
	SvBindContext *   pBindCtx, /* Informationen zum Verbinden */
	UINT32 nSpeedNeeded,        /* Die Zeit in ms, die erlaubt ist, um
								   zu dem Objekt zu binden */
	const String & rObjName,    /* Der Name des Objektes, das gebunden
								   werden soll */
	const SvFactory * pFact     /* Das Sv-Interface fuer das zu bindende
								   Objekt */
)
/*  [Beschreibung]

	Das unter rObjName spezifizierte Objekt wird erzeugt.

	[R"uckgabewert]

	ERRCODE_NONE,       Das Objekt wurde gebunden.
	ERRCODE_SO_MK_NO_OBJECT, ERRCODE_SO_MK_EXCEEDED_DEADLINE,
	ERRCODE_SO_GENERALERROR,
	ERRCODE_IO_ACCESSDENIED

	[Anmerkung]

	Der Objekt Name wird ohne den Delemitter angegeben.

	[Querverweise]

	<IMoniker::Bind2Obj>
*/
{
	SvObjectRef xTmp = xRet; // sichern, falls this == xRet
	xRet.Clear();
	ULONG nRet = ERRCODE_SO_MK_NO_OBJECT;

	return nRet;
}

//=========================================================================
ULONG SvObjectContainer::GetObjStorage
(
	SvObjectRef &     xRet,     /* hierin wird das gebundene Speichermedium
								   oder NULL zur"ueckgegeben */
	SvBindContext *   pBindCtx, /* Informationen zum Verbinden */
	const String & rObjName,    /* Der Name des Objektes, von dem der
								   Storage zur"uckgegeben werden soll */
	const SvFactory * pFact     /* Das Sv-Interface fuer das zu bindende
								   Objekt. Muss <SvStorageStream> oder
								   <SvStorage> sein. */
)
/*  [Beschreibung]

	Das unter rObjName spezifizierte Storage-Medium soll zur"uckgegeben
	werden.

	[R"uckgabewert]

	ERRCODE_NONE,       Das Objekt wurde gebunden.
	ERRCODE_SO_MK_NO_OBJECT, ERRCODE_SO_MK_EXCEEDED_DEADLINE,
	ERRCODE_SO_GENERALERROR,
	ERRCODE_IO_ACCESSDENIED

	[Anmerkung]

	Storage bedeutet nicht immer die Klasse SvStorage, sondern eine
	beliebige (SvStorage, SvStorageStream...) StorageMedium.
	Der Objekt Name wird ohne den Delemitter angegeben.


	[Querverweise]

	<IMoniker::Bind2Storage>
*/
{
	SvObjectRef xTmp = xRet; // sichern, falls this == xRet
	xRet.Clear();
	ULONG nRet = ERRCODE_SO_MK_NO_OBJECT;

	return nRet;
}

//=========================================================================
ULONG SvObjectContainer::GetRunningState
(
	const String & rObjName     /* Der Name des Objektes */
)
/*  [Beschreibung]

	Es wird festgestellt, ob das Objekt im Zustand "IsRunning" ist.

	[R"uckgabewert]

	ERRCODE_NONE,           Das Objekt ist im Zustand "IsRunning".
	ERRCODE_SO_FALSE,       Das Objekt ist nicht im Zustand "IsRunning".
	ERRCODE_SO_MK_NO_OBJECT Es gibt kein Objekt unter diesem Namen.

	[Querverweise]

	<SvPseudoObject::IsRunning>
*/
{
	ULONG nRet = ERRCODE_SO_MK_NO_OBJECT;

	return nRet;
}

//=========================================================================
//==============class SvPersist============================================
//=========================================================================
SV_OLE_FACTORY(SvPersistFactory)
	{
	}
};
TYPEINIT1(SvPersistFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS1_DLL(SvPersist,SvPersistFactory,SvObjectContainer,
							 0xC24CC4E0L, 0x73DF, 0x101B,
							 0x80,0x4C,0xFD,0xFD,0xFD,0xFD,0xFD,0xFD )

IUnknown * SvPersist::GetMemberInterface( const SvGlobalName & )
{
	return NULL;
}

//=========================================================================
#ifdef TEST_INVARIANT
void SvPersist::TestMemberObjRef( BOOL /*bFree*/ )
{
	if( pChildList )
	{
		ULONG nCount = pChildList->Count();
		for( ULONG i = 0; i < nCount; i++ )
		{
			SvInfoObject * pEle = pChildList->GetObject( i );
			if( pEle->GetObj() )
			{
				ByteString aTest( "\t\tpChildList[ " );
				aTest += ByteString::CreateFromInt32( i );
				aTest += " ] == ";
				aTest += ByteString::CreateFromInt32( (ULONG)pEle->GetObj() );
				DBG_TRACE( aTest.GetBuffer() );
			}
		}
	}
	if( aStorage.Is() )
	{
		ByteString aTest( "\t\taStorage == " );
		aTest += ByteString::CreateFromInt32( (ULONG)&aStorage );
		DBG_TRACE( aTest.GetBuffer() );
	}
}

//=========================================================================
void SvPersist::TestMemberInvariant( BOOL /*bPrint*/ )
{
#ifdef DBG_UTIL
	DBG_ASSERT( !(bOpSave || bOpSaveAs), "bOpSave && bOpSaveAs" )
	if( !bIsInit )
	{
		if ( !bCreateTempStor )
		{
			DBG_ASSERT( bOpHandsOff && !aStorage.Is(), "failed: bOpHandsOff && !aStorage.Is()" )
			DBG_ASSERT( !bOpHandsOff && aStorage.Is(), "failed: !bOpHandsOff && aStorage.Is()" )
		}
	}
#endif
}
#endif

//=========================================================================
SvPersist::SvPersist()
	: bIsModified     ( FALSE )
	, bIsInit         ( FALSE )
	, bOpSave         ( FALSE )
	, bOpSaveAs       ( FALSE )
	, bSaveExtern     ( FALSE )
	, bOpHandsOff     ( FALSE )
	, bCreateTempStor ( FALSE )
	, bSaveFailed     ( FALSE )
	, pObjI           ( NULL )
	, pPersistFile    ( NULL )
	, pParent         ( NULL )     // kein zusaetzlicher RefCount
	, nModifyCount    ( 0 )        // Anzahl der modifizierten Childs und sich selbst
	, pChildList      ( NULL )
	, bEnableSetModified( TRUE )
{
}

//=========================================================================
SvPersist::~SvPersist()
{
	// Parent haelt immer Referenz, deswegen muss der das Objekt
	// schon Removed sein
	dtorClear();
}


/*************************************************************************
|*    SvPersist::AssertInit()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::AssertInit() const
{
	DBG_ASSERT( bIsInit, "super class SvPersist: call InitNew or Load bevor other calls" );
}

//=========================================================================
void SvPersist::FillClass
(
	SvGlobalName * pClassName,  /* Der Typ der Klasse */
	ULONG * pFormat,            /* Das Dateiformat in dem geschrieben wird */
	String * pAppName,          /* Der Applikationsname */
	String * pFullTypeName,     /* Der genaue Name des Typs */
	String * pShortTypeName,    /* Der kurze Name des Typs  */
	long /*nFileFormat */       /* F"ur dieses Office-Format sollen die
								   Parameter gef"ullt werden */
) const
/*  [Beschreibung]

	Diese Methode liefert Informationen "uber den Typ und das Dateiformat
	des Objektes. Alle Parameter werden von der Methode gefllt.

	[R"uckgabewert]

	*pClassName     Liefert den Typ-Identifier des Objektes.
	*pFormat        Die FormatId des Storages.
	*pAppName       Den sprachabh"angigen Applikationsnamen.
	*pFullTypeName  Den sprachabh"angigen Namen des Typs.
	*pShortTypeName Den kurzen sprachabh"angigen Namen des Typs. Er darf
					nicht l"anger als 15 Zeichen sein.

	[Anmerkung]

	F"uer externe Objekte ist pAppName und pShortName leer.

	[Beispiel]

	MyClass::FillClass( ... )
	{
		*pClassName     = *SvFactory::GetSvFactory(); // keine emulation
		*pFormat        = nMyDocFormat;
		*pAppName       = "StarDivison Calc 3.0";
		*pFullTypeName  = "StarDivison Calc 3.0 Tabelle";
		*pShortTypeName = "Tabelle";
	}

	[Querverweise]

	<SvPersist::GetClassName>, <SvPersist::GetFileFormat>,
	<SvPersist::GetFullTypeName>, <SvPseudoObject::FillClass>
*/
{
	*pFormat        = 0;
	*pFullTypeName  = *pShortTypeName = *pAppName = String();
	*pClassName     = SvGlobalName();

	if( Owner() )
		*pClassName = *GetSvFactory();
}

//=========================================================================
SvGlobalName SvPersist::GetClassName() const
/*  [Beschreibung]

	Ruft <SvPersist::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName    Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return aName;
}

//=========================================================================
ULONG SvPersist::GetFileFormat() const
/*  [Beschreibung]

	Ruft <SvPersist::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName    Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return nFormat;
}

//=========================================================================
String SvPersist::GetFullTypeName() const
/*  [Beschreibung]

	Ruft <SvPersist::FillClass> auf, um den Typ des Objektes zu bekommen.

	[R"uckgabewert]

	SvGlobalName    Der Typ des Objektes.
*/
{
	SvGlobalName    aName;
	String          aAppName, aFullTypeName, aShortTypeName;
	ULONG           nFormat;

	FillClass( &aName, &nFormat, &aAppName, &aFullTypeName,
				&aShortTypeName );
	return aFullTypeName;
}

/*************************************************************************
|*    SvPersist::SetupStorage()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::SetupStorage( SvStorage * pStor ) const
{
	SvUniqueName    aName;
	String          aFullTypeName, aShortTypeName, aAppName;
	ULONG           nClipFormat;
	FillClass( &aName, &nClipFormat, &aAppName,
				&aFullTypeName, &aShortTypeName, pStor->GetVersion() );
	pStor->SetClass( aName, nClipFormat, aShortTypeName );
}

/*************************************************************************
|*    SvPersist::GetInfoList()
|*
|*    Beschreibung
*************************************************************************/
SvInfoObjectMemberList * SvPersist::GetInfoList()
{
	if( !pChildList )
		pChildList = new SvInfoObjectMemberList();
	return pChildList;
}

/*************************************************************************
|*    SvPersist::SetModifiedFormat()
|*
|*    Beschreibung
*************************************************************************/
#ifdef _NO_MODIFY_ADVISE
void SvPersist::SetModifiedFormat( ULONG nFormat )
{
	if( !Owner() )
	{ // sonst wird das durch die Baumstruktur erledigt
		if( aAdvSink.Is() )
		{
			DBG_ASSERT( !nFormat, "SvPersist::SetModifiedFormat: double set" );
			SvAdviseRef aAdv( this );
			aAdv->RemoveAdvise( &aAdvSink, ADVISE_DATA );
			aAdvSink.Clear();
		}
		if( nFormat )
		{
			SvAdviseRef aAdv( this );
			if( aAdv.Is() )
			{ // Es kann ein Link gesetzt werden
				aAdvSink = new SvPersistAdviseSink( this );
				aAdv->AddDataAdvise( &aAdvSink, SvDataType( nFormat ),
									 ADVISEMODE_NODATA );
				return;
			}
			DBG_ASSERT( aAdv.Is(), " cannot install data advise" );
		}
	}
}
#endif

/*************************************************************************
|*    SvPersist::Insert()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::Insert( SvInfoObject * pInfoObj )
{
	ASSERT_INIT()
	SvInfoObjectRef aRef( pInfoObj );

	DBG_ASSERT( pInfoObj->GetObjName().Len(),
				"Try to Insert Object without name" )
	BOOL bRet = TRUE;
	if( !GetInfoList() )
		bRet = FALSE; // es konnte keine Liste erstellt werden

	SvPersist * pChild = pInfoObj->GetPersist();
	DBG_ASSERT( pChild, "missing Object in SvInfoObject" )
	if( !pChild )
		bRet = FALSE;

	if( bRet && this != pChild->pParent )
	{
		// als erstes, wegen Patch in IsModified
		if( pChild->Owner() && pChild->IsModified() )
			// modify-Zaehler fuer Child raufzaehlen
			CountModified( TRUE );

		if( pChild->pParent )
			// Aus altem Parent entfernen
			pChild->pParent->Remove( pChild );

		pChild->pParent   = this;        // kein zusaetzlicher RefCount
		//pChild->SetModifiedFormat( SvEmbeddedObject::GetFormat() );
		pChildList->Append( pInfoObj );
		SetModified( TRUE );
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::Move()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::Move( SvInfoObject * pInfoObj, const String & rStorName )
{
	ASSERT_INIT()
	SvInfoObjectRef aRef( pInfoObj );
	DBG_ASSERT( pInfoObj->GetObjName().Len(),
				"Try to Move Object without name" )

	BOOL bRet = TRUE;
	if( !GetInfoList() )
		bRet = FALSE; // es konnte keine Liste erstellt werden

	SvPersist * pChild = pInfoObj->GetPersist();
	if( !pChild )
		bRet = FALSE;

	if( bRet && this != pChild->pParent )
	{
		bRet = FALSE;
		SvStorageRef aNewStor;
		aNewStor = GetStorage()->OpenStorage( rStorName,
											  STREAM_STD_READWRITE | STREAM_TRUNC,
											  0 );

		if( aNewStor->GetError() == SVSTREAM_OK )
		{
			SvStorageRef aStor = pChild->GetStorage();    // Storage sichern
			pChild->DoHandsOff();                         // Storage loslassen
			if( aStor->CopyTo( aNewStor ) )               // Identische Kopie
			{
				if( pChild->DoSaveCompleted( aNewStor ) ) // neuen zuordnen
					bRet = TRUE;
				else
					pChild->DoSaveCompleted();
			}
		}
		if( bRet )
			bRet = Insert( pInfoObj );
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::Copy()
|*
|*    Beschreibung: Kopiert ein Objekt von der einen in die andere
|*                  Persist Instanz
*************************************************************************/
BOOL SvPersist::Copy( const String & rNewObjName, const String & rNewStorName,
					  SvInfoObject * pSrcI, SvPersist * pSrc )
{
	BOOL bRet = TRUE;
	if( !GetInfoList() )
		bRet = FALSE; // es konnte keine Liste erstellt werden

	SvInfoObjectRef xI( pSrcI->CreateCopy() );
	xI->SetObjName( rNewObjName );
	xI->SetStorageName( rNewStorName );
	if( pSrcI->GetPersist() )
	{
		// Objekt ist geladen
		bRet = FALSE;
		SvStorageRef aNewStor;
		aNewStor = GetStorage()->OpenStorage( rNewStorName,
											  STREAM_STD_READWRITE | STREAM_TRUNC,
											  0 );
		if( aNewStor->GetError() == SVSTREAM_OK )
		{ // Object speichern
			if( pSrcI->GetPersist()->DoSaveAs( aNewStor ) )
			{
				bRet = TRUE;
				pSrcI->GetPersist()->DoSaveCompleted();
			}
		}
	}
	else
	{ // Storage kopieren
		bRet = pSrc->GetStorage()->CopyTo( pSrcI->GetStorageName(),
										   GetStorage(), rNewStorName );
	}
	if( bRet )
	{
		pChildList->Append( xI );
		SetModified( TRUE );
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::unload()
|*
|*    Beschreibung:
*************************************************************************/
BOOL SvPersist::Unload( SvInfoObject *pInfoObj)
{
	if( bOpSaveAs || bOpHandsOff || bOpSave )
		return FALSE;

	DBG_ASSERT(pInfoObj,"Kein InfoObj");
	SvPersistRef xChild = pInfoObj->GetPersist();
	DBG_ASSERT(xChild->pParent == this,"wrong parent");
	if( xChild.Is() )
	{
		xChild->DoClose();

		if( xChild->Owner() )
		{
			DBG_ASSERT(!xChild->IsModified(),"unload modified object");
			if( xChild->IsModified() )
				// modified, do not unload
				return FALSE;
		}
		// update info object before unload
		SvEmbeddedInfoObject * pI = PTR_CAST(SvEmbeddedInfoObject, pInfoObj );
		if( pI )
		{
			pI->GetVisArea();
			pI->IsLink();
		}
		SvPersistRef x;
		pInfoObj->SetObj( x );

		// Ein Hack, da ueber den RefCount keine Logik implementiert werden daerf
		if( xChild->GetRefCount() == 1 )
		{

			xChild->pParent = NULL;
			return TRUE;
		}
		else
			pInfoObj->SetObj( xChild );
	}
	return FALSE;
}

BOOL SvPersist::Unload( SvPersist * pChild )
{
	DBG_ASSERT( pChild, "SvPersist::Unload(): pChild == NULL" )
	if( pChildList )
	{
		SvInfoObjectRef pEle = pChildList->First();
		while( pEle.Is() )
		{
			if( pEle->GetPersist() == pChild )
				return Unload( pEle );
			pEle = pChildList->Next();
		}
	}
	return FALSE;
}

/*************************************************************************
|*    SvPersist::Remove()
|*
|*    Beschreibung:
*************************************************************************/
void SvPersist::Remove( SvInfoObject *pInfoObj)
{
	DBG_ASSERT(pInfoObj,"Kein InfoObj");
	SvPersist * pChild = pInfoObj->GetPersist();
	if( pChild )
	{
		if( pChild->Owner() && pChild->IsModified() )
			// modify-Zaehler fuer Child runterzaehlen
			CountModified( FALSE );
		if( pChild->pParent == this )
			pChild->pParent = NULL;  // kein zusaetzlicher RefCount
	}
	pChildList->Remove( pInfoObj );
	SetModified( TRUE );
}


void SvPersist::Remove( const String & rName )
{
	SvInfoObjectRef pInfo = Find( rName );
	if( pInfo.Is() )
		Remove(pInfo);
}

void SvPersist::Remove( SvPersist * pChild )
{
	DBG_ASSERT( pChild, "SvPersist::Remove(): pChild == NULL" )
	if( pChildList )
	{
		SvInfoObjectRef pEle = pChildList->First();
		while( pEle.Is() )
		{
			if( pEle->GetPersist() == pChild )
			{
				Remove(pEle);
				break;
			}
			pEle = pChildList->Next();
		}
	}
}

void SvPersist::dtorClear()
{
	if( pChildList )
	{
		SvInfoObjectMemberList * pList = pChildList;
		pChildList = NULL; //Schutz vor Remove bei Destruktor
		SvInfoObjectRef pEle = pList->Last();
		pList->Remove();
		while( pEle.Is() )
		{
			if( pEle->GetPersist() )
			{
				//pEle->GetPersist()->SetModifiedFormat( 0 );
				pEle->GetPersist()->pParent = NULL;  // kein zusaetzlicher RefCount
			}
			pEle = pList->Last();
			pList->Remove();
		}
		delete pList;
	}
}

void SvPersist::Clear()
{
	if( pChildList )
	{
		ULONG nCount = pChildList->Count();
		dtorClear();
		if( nCount )
			SetModified( TRUE ); // nicht im dtor
	}
}

SvInfoObject * SvPersist::Find( const String & rName ) const
{
	if( pChildList )
	{ // Befindet sich das Objekt in der aktuellen Liste
		SvInfoObjectRef pEle = pChildList->First();
		while( pEle.Is() )
		{
			if( pEle->GetObjName() == rName )
				return pEle;
			pEle = pChildList->Next();
		}
	}
	return NULL;
}

SvInfoObject * SvPersist::Find( const SvPersist * pObj ) const
{
	if( pChildList )
	{ // Befindet sich das Objekt in der aktuellen Liste
		SvInfoObject* pEle = pChildList->First();
		while( pEle )
		{
			if( pEle->GetPersist() == pObj )
				return pEle;
			pEle = pChildList->Next();
		}
	}
	return NULL;
}

/************************************************************************
|*    SvPersist::GetObj()
|*
|*    Beschreibung
*************************************************************************/
ULONG SvPersist::GetObj
(
	SvObjectRef & xRet/*Out-Parameter */,
	SvBindContext * pBindCtx,
	UINT32 nSpeedNeeded,
	const String & rObjName,
	const SvFactory * pInterface
)
{
	if( Owner() )
	{
		SvObjectRef xTmp = xRet; // sichern, falls this == xRet
		xRet.Clear();
		ULONG nRet = ERRCODE_SO_MK_NO_OBJECT;

		SvInfoObject * pEle = Find( rObjName );

		if( pEle )
		{
			// Objekt holen
			SvPersist * pP = pEle->GetPersist();
			if( !pP )
				// war es noch nicht geladen
				pP = GetObject( pEle->GetObjName() );
			// nur diese koennen Runnable sein
			SvPseudoObjectRef xPO( pP );
			DBG_ASSERT( xPO.Is() && pP || !pP && !xPO.Is(),
						"Sub-Objekte sind keine Pseudo-Objekte" )

			if( xPO.Is() )
			{
				ULONG nMisc = xPO->GetMiscStatus();
				if( !(SVOBJ_MISCSTATUS_CANTLINKINSIDE & nMisc) )
				{// warum? genau in der OleSpec nachlesen
					if( nSpeedNeeded >= 5000 )
					{
						if( xPO->Run() )
							nRet = ERRCODE_NONE;
						else
							nRet = ERRCODE_SO_MK_EXCEEDED_DEADLINE;
					}
					else if( xPO->IsRunning() )
						nRet = ERRCODE_NONE;
					else
						nRet = ERRCODE_SO_MK_EXCEEDED_DEADLINE;
				}
				if( nRet == ERRCODE_NONE )
					if( xPO->GetSvFactory()->Is( pInterface ) )
						xRet = xPO;
					else
						// Objekt geladen, aber nicht das richtige
						// Interface gefunden
						nRet = ERRCODE_SO_NO_INTERFACE;
			}
		}
		return nRet;
	}
	else
		return SvObjectContainer::GetObj( xRet, pBindCtx,
										nSpeedNeeded, rObjName,
										pInterface );
}

/************************************************************************
|*    SvPersist::GetObjStorage()
|*
|*    Beschreibung
*************************************************************************/
ULONG SvPersist::GetObjStorage
(
	SvObjectRef & xRet,
	SvBindContext * pBindCtx,
	const String & rObjName,
	const SvFactory * pInterface
)
{

	if( Owner() )
	{
		SvObjectRef xTmp = xRet; // sichern, falls this == xRet
		xRet.Clear();
		ULONG nRet = ERRCODE_SO_MK_NO_OBJECT;

		// Befindet sich das Objekt in der aktuellen Liste
		SvInfoObject * pEle = Find( rObjName );
		if( pEle )
		{
			SvPersist * pP = pEle->GetPersist();
			if( pP )
			{
				xRet = pP->GetStorage();
			}

			// Es gibt es einen Storage zu diesem Objekt
			xRet = GetStorage()->OpenStorage( pEle->GetStorageName() );
			if( xRet.Is() )
			{
				if( xRet->GetSvFactory()->Is( pInterface ) )
				{
					nRet = ERRCODE_NONE;
				}
				else
				{
					xRet.Clear();
					// Storage gefunden, aber nicht das richtige
					// Interface
					nRet = ERRCODE_SO_NO_INTERFACE;
				}
			}
			else
				nRet = ERRCODE_SO_GENERALERROR;
		}
		return nRet;
	}
	else
		return SvObjectContainer::GetObjStorage( xRet, pBindCtx,
												rObjName,
												pInterface );
}

/************************************************************************
|*    SvPersist::GetRunningState()
|*
|*    Beschreibung: Gibt
*************************************************************************/
ULONG SvPersist::GetRunningState
(
	const String & rObjName
)
{
	if( Owner() )
	{
		ULONG nRet = ERRCODE_SO_MK_NO_OBJECT;

		SvInfoObject * pEle = Find( rObjName );

		if( pEle )
		{
			// Objekt holen
			SvPersist * pP = pEle->GetPersist();
			if( !pP )
				// war es noch nicht geladen
				pP = GetObject( pEle->GetObjName() );
			// nur diese koennen Runnable sein
			SvPseudoObjectRef xPO( pP );
			DBG_ASSERT( xPO.Is() && pP || !pP && !xPO.Is(),
						"Sub-Objekte sind keine Pseudo-Objekte" )

			if( xPO.Is() )
			{
				ULONG nMisc = xPO->GetMiscStatus();
				if( !(SVOBJ_MISCSTATUS_CANTLINKINSIDE & nMisc) )
				{// warum? genau in der OleSpec nachlesen
						if( xPO->IsRunning() )
							nRet = ERRCODE_NONE;
						else
							nRet = ERRCODE_SO_FALSE;
				}
			}
		}
		return nRet;
	}
	else
		return SvObjectContainer::GetRunningState( rObjName );
}

/*************************************************************************
|*    SvPersist::HasObject()
|*    SvPersist::CanRunObject()
|*    SvPersist::GetObject()
|*    SvPersist::GetObjectStorage()
|*    SvPersist::IsRoot()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::HasObject( const String & rObjName )
{
	if( Owner() )
	{
		// Befindet sich das Objekt in der aktuellen Liste
		if( Find( rObjName ) )
			return TRUE;
	}
	else
		// Geht nur so
		return SvPersist::CanRunObject( rObjName, 0xFFFFFFFF );
	return FALSE;
}

BOOL SvPersist::CanRunObject( const String & rObjName,
							  ULONG nSpeedNeeded )
{
	SvBindContextRef xBind = new SvBindContext();
	SvObjectRef xRet;
	SvPersist::GetObj( xRet, xBind, nSpeedNeeded,
						rObjName, (SvFactory*)ClassFactory() );
	return xRet.Is();
}

SvPersistRef SvPersist::GetObject( const String & rObjName )
{
	if( Owner() )
	{
		SvInfoObject * pEle = Find( rObjName );
		if( !pEle )
			return SvPersistRef();
		if( pEle->GetPersist() )
			return pEle->GetPersist();

		SvStorageRef xSt = GetStorage()->OpenStorage( pEle->GetStorageName() );
		if( xSt.Is() )
		{
			// Es gibt einen Storage zum Child
			// entsprechende Sv-Klasse erzeugen
			SvGlobalName aActualClassName =
				SvFactory::GetAutoConvertTo( pEle->GetClassName() );
			const SvFactory * pFact = PTR_CAST( SvFactory, SvFactory::Find( aActualClassName ) );
			SvPersistRef xPer;
			if( pFact )
				xPer = &pFact->CreateAndLoad( xSt );
			DBG_ASSERT( pFact, "ERRCODE_IO_NOFACTORY" );
			//else
				//Fehlercode ERRCODE_IO_NOFACTORY
			if( xPer.Is() )
			{
				// Parent einsetzten
				xPer->pParent = this;

				// ist ein persistentes Objekt
				// muss initialisiert sein
				pEle->SetObj( xPer );
				return xPer;
			}
		}
		else
			GetStorage()->ResetError();
	}
	return SvPersistRef();
}

SvStorageRef SvPersist::GetObjectStorage( const String & rObjName )
{
	SvBindContextRef xBind = new SvBindContext();
	SvObjectRef xRet;
	SvPersist::GetObjStorage( xRet, xBind, rObjName,
								(SvFactory*)SvStorage::ClassFactory() );
	return SvStorageRef( xRet );
}

BOOL SvPersist::IsRoot() const
{
	return bCreateTempStor || aStorage->IsRoot();
}

/*************************************************************************
|*    SvPersist::SetModified()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::SetModified( BOOL bModifiedP )
{
	ASSERT_INIT()
	DBG_ASSERT( bModifiedP || IsEnableSetModified(),
				"SetModified( FALSE ), obwohl IsEnableSetModified() == FALSE" )

	if( !IsEnableSetModified() )
		return;

	if( bIsModified != bModifiedP )
	{ // der Status hat sich geaendert
		// Hilfsflag setzen
		bIsModified = bModifiedP;
		// Hierarchisch ModifyCount setzen
		CountModified( bModifiedP );
	}

	aModifiedTime = Time();
}

/*************************************************************************
|*    SvPersist::CountModified()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::CountModified( BOOL bMod )
{
	DBG_ASSERT( bMod || nModifyCount != 0, "modifiy count underflow" )
	nModifyCount += bMod ? 1 : -1;
	if( pParent )
	{
		/* Den ModifyCount des Parent nur einmal rauf oder runter
		   zaehlen. Bei den Insert und Move Methoden muss dieser
		   Zaehler dann nur einmal erhoeht oder erniedrigt werden
		*/
		if( (bMod && nModifyCount == 1)
		  || (!bMod && nModifyCount == 0) )
			pParent->CountModified( bMod );
	}
	if ( ( nModifyCount == 1 && bMod ) ||
		 ( nModifyCount == 0 ) )
		ModifyChanged();
}

/*************************************************************************
|*    SvPersist::SetModifiedChanged()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::EnableSetModified( BOOL bEnable )
{
	DBG_ASSERT( bEnable != bEnableSetModified,
				"EnableSetModified 2x mit dem gleichen Wert gerufen" )
	bEnableSetModified = bEnable;
}

/*************************************************************************
|*    SvPersist::ModifyChanged()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::ModifyChanged()
{
}

/*************************************************************************
|*    SvPersist::IsModified()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::IsModified()
{
	ASSERT_INIT()
	if( nModifyCount )
		// Wenn Count gesetzt, auf jeden Fall modifiziert
		return TRUE;
	if( Owner() )
	{   
		if( pChildList )
		{
			SvInfoObject * pObj = pChildList->First();
			while( pObj )
			{
				if( pObj->GetPersist() && pObj->GetPersist()->IsModified() )
					return TRUE;
				pObj = pChildList->Next();
			}
		}
	}
	return FALSE;
}

/*************************************************************************
|*    SvPersist::CopyObject()
|*
|*    Beschreibung
*************************************************************************/
SvPersistRef SvPersist::CopyObject( SvStorage * pStor )
{
	// Kopieren durch Speichern und Laden
	if( DoSaveAs( pStor ) )
	{
		DoSaveCompleted();
		const SvFactory * pFact = PTR_CAST( SvFactory, GetSvFactory() );
		SvPersistRef xObj( pFact->CreateAndLoad( pStor ) );
		DBG_ASSERT( xObj.Is(), "not from class SvPersist" )
		return xObj;
	}
	else
		DoSaveCompleted();
	return SvPersistRef();
}

/*************************************************************************
|*    SvPersist::Init()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::InitMembers( SvStorage * pStor )
{
	bIsInit     = TRUE;
	if ( pStor )
		aStorage    = pStor;
	else
		bCreateTempStor = TRUE;
}

/*************************************************************************
|*    SvPersist::DoInitNew()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::DoInitNew( SvStorage * pStor )
{
	EnableSetModified( FALSE );
	BOOL bRet = InitNew( pStor );
	EnableSetModified( TRUE );
	return bRet;
}

/*************************************************************************
|*    SvPersist::DoLoad()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::DoLoad( SvStorage * pStor )
{
	EnableSetModified( FALSE );
	BOOL bRet = Load( pStor );
	EnableSetModified( TRUE );
	return bRet;
}

//=========================================================================
BOOL SvPersist::DoLoad
(
	const String & rFileName,   /* Name der Datei, aus der das Objekt
								   geladen werden soll. */
	StreamMode nStreamMode,     /* Attribute f"ur um die Datei zu "offnen */
	StorageMode nStorageMode    /* Attribute f"ur um die Datei zu "offnen */
)
/*  [Beschreibung]

	Das Objekt soll aus der angegebenen Datei geladen werden.

	[Anmerkung]

	Aufgrund fehlerhafter Ole funktionalitt, wird der Stream auch
	schreibend geffnet, wenn nur READ angegeben ist und er nicht
	schreibgeschtzt ist.

	[Querverweise]

	<SvPseudoObject::DoSave>
*/
{
	SvStorageRef xStg;
	SvGlobalName aGN;
	// Es wird nur die IPersistStorage-Schnittstelle angeboten
	xStg = new SvStorage( rFileName, nStreamMode | STREAM_WRITE, nStorageMode );
	if( !xStg.Is() )
		xStg = new SvStorage( rFileName, nStreamMode, nStorageMode );
	aGN = xStg->GetClassName();
	if( !xStg.Is() && aGN == *GetSvFactory() )
	{
		xStg = new SvStorage( rFileName, nStreamMode | STREAM_WRITE, nStorageMode );
		if( !xStg.Is() )
			xStg = new SvStorage( rFileName, nStreamMode, nStorageMode );
	}
	if( xStg.Is() && SVSTREAM_OK != xStg->GetError() )
		return FALSE;

	SetFileName( rFileName );
	return DoLoad( xStg );
}

BOOL SvPersist::DoSave()
{
	EnableSetModified( FALSE );
	bSaveFailed = !Save();
	EnableSetModified( TRUE );
	return !bSaveFailed;
}

BOOL SvPersist::DoSaveAs( SvStorage * pStor )
{
	EnableSetModified( FALSE );
	BOOL bRet = SaveAs( pStor );
	EnableSetModified( TRUE );
	return bRet;
}

void SvPersist::DoHandsOff()
{
	HandsOff();
}

BOOL SvPersist::DoSaveCompleted( SvStorage * pStor )
{
	return SaveCompleted( pStor );
}


/*************************************************************************
|*    SvPersist::GetStorage() const
|*
|*    Beschreibung
*************************************************************************/
SvStorage * SvPersist::GetStorage() const
{
	ASSERT_INIT()
	DBG_ASSERT( !bOpHandsOff, "in hands off state, no accessible storage" )
	if( bCreateTempStor )
	{
		SvPersist *pThis = (SvPersist*) this;
		DBG_ASSERT( !aStorage.Is(), "bCreateTempStorage && aStorage.Is()" );
		pThis->aStorage = new SvStorage( String() );
		pThis->bCreateTempStor = FALSE;
		SetupStorage( pThis->aStorage );
	}

	return aStorage;
}

/*************************************************************************
|*    SvPersist::InitNew()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::InitNew( SvStorage * pStor )
{
	InitMembers( pStor );
	BOOL bRet = FALSE;

	if( pStor )
		SetupStorage( pStor );

	if( Owner() )
		bRet = TRUE;
	return bRet;
}

/*************************************************************************
|*    SvPersist::Load()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::DoOwnerLoad( SvStorage * pStor )
{
	InitMembers( pStor );

	return DoLoadContent( pStor, TRUE );
}

/*************************************************************************
|*    SvPersist::Load()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::Load( SvStorage * pStor )
{
	dtorClear();
	InitMembers( pStor );

	SvGlobalName aActualClassName =
		SvFactory::GetAutoConvertTo( GetStorage()->GetClassName() );

	if( aActualClassName == *GetSvFactory() )
		return DoLoadContent( pStor, TRUE );

	return TRUE;
	//return DoLoadContent( pStor, FALSE );
}

/*************************************************************************
|*    SvPersist::Save()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::Save()
{
	ASSERT_INIT()

	SvGlobalName aNoName; // wegen MAC
	if( GetStorage()->GetClassName() == aNoName )
		// kein Typ im Storage gesetzt
		SetupStorage( GetStorage() );

	bOpSave = TRUE;
	if( !IsModified() )
		// keine Aendeungen
		return TRUE;

	BOOL bRet = TRUE;
//    GetStorage()->SetClassName( GetClassName() );

	if( Owner() )
		bRet = DoSaveContent( GetStorage(), TRUE );
	return bRet;
}

/*************************************************************************
|*    SvPersist::SaveAs()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::SaveAs( SvStorage * pStor )
{
	ASSERT_INIT()

	bOpSaveAs = TRUE;

	BOOL bRet = TRUE;

	SvGlobalName aNoName; // wegen MAC
	if( pStor->GetClassName() == aNoName )
		// kein Typ im Storage gesetzt
		SetupStorage( pStor );

	if( Owner() )
	{
		if( SOFFICE_FILEFORMAT_31 == pStor->GetVersion()
		  && GetParent() )
		{
			// Es ist ein Objekt
			bRet = DoSaveContent( pStor,
				SvFactory::IsIntern31( pStor->GetClassName() ) );
		}
		else
			bRet = DoSaveContent( pStor, TRUE );
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::HandsOff()
|*
|*    Beschreibung
*************************************************************************/
void SvPersist::HandsOff()
{
	ASSERT_INIT()

	if( pChildList )
	{ // Rekursiv alle Childs retten
		for( ULONG i = 0; i < pChildList->Count(); i++ )
		{
			SvInfoObject * pEle = pChildList->GetObject( i );
			if( pEle->GetPersist() )
			{
				// Im HandsOff darf nicht mehr auf Persist zugegriffen werden
				// deswegen alten Namen sichern
				pEle->aStorName = pEle->GetStorageName();
				pEle->GetPersist()->DoHandsOff();
			}
			pEle = pChildList->Next();
		}
	}
	bOpHandsOff = TRUE;

	aStorage.Clear();
}

/*************************************************************************
|*    SvPersist::SaveCompleted()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::SaveCompleted( SvStorage * pStor )
{
	//DBG_ASSERT( bOpHandsOff | bOpSaveAs | bOpSave, aTest )
	ASSERT_INIT()

	if( !pStor && aStorage.Is() )
		// falls beim Save ein Fehler aufgetreten ist, mu er zurueckgesetzt
		// werden
		aStorage->ResetError();

	BOOL bRet = TRUE;

	if( pStor )
	{
		aStorage  = pStor;
		SvGlobalName aNoName; // wegen MAC
		if( pStor->GetClassName() == aNoName )
			// kein Typ im Storage gesetzt
			SetupStorage( pStor );
		bCreateTempStor=FALSE;
	}

	if( Owner() )
	{
		if( !bSaveFailed  )
		{
			if( bOpSave )
			{
				SetModified( FALSE );
				DBG_ASSERT( !nModifyCount, "IsModified() == TRUE after save" )
				SvAdviseRef aAdv( this );
				if( aAdv.Is() )
					// Es konnte gecastet werden
					aAdv->Saved();
			}
			if( bOpSaveAs )
			{
				if( pStor )
				{   // nur wenn SaveAs und danach der Storage umgesetzt wird,
					// sonst war es z.B. ein SaveAs fuers Clipboard
					SetModified( FALSE );
					DBG_ASSERT( !IsModified(), "SvPersist::SaveCompleted: IsModified() == TRUE" )
					SvAdviseRef aAdv( this );
					if( aAdv.Is() )
						// Es konnte gecastet werden
						aAdv->Saved();
				}
			}
		}
	}
	bOpSaveAs = bOpSave = bOpHandsOff = bSaveFailed = FALSE;
	return bRet;
}

/*************************************************************************
|*    SvPersist::LoadContent()
|*
|*    Beschreibung
*************************************************************************/
#define PERSIST_STREAM "persist elements"
BOOL SvPersist::DoLoadContent( SvStorage * pStor, BOOL bOwner )
{
	SvStorageStreamRef aContStm;
	if( bOwner )
		aContStm = pStor->OpenStream( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( PERSIST_STREAM ) ),
									 STREAM_READ | STREAM_NOCREATE );
	else
	{

		aContStm = pStor->OpenStream( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( SVEXT_PERSIST_STREAM ) ),
									 STREAM_READ | STREAM_NOCREATE );
		if( aContStm->GetError() == SVSTREAM_FILE_NOT_FOUND )
			// Stream nicht vorhanden, Ole10Native probieren
			aContStm = pStor->OpenStream( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "\1Ole10Native" ) ),
										 STREAM_READ | STREAM_NOCREATE );
	}

	// nicht vorhandener Stream ist kein Fehler
	if( aContStm->GetError() == SVSTREAM_FILE_NOT_FOUND )
		return TRUE;

	aContStm->SetVersion( pStor->GetVersion() );
	if( aContStm->GetError() == SVSTREAM_OK )
	{
		aContStm->SetBufferSize( 8192 );
#if defined( SOLARIS ) && defined( C40 )
	 // patch to work around Sparc-Compiler bug
	 SvEmbeddedObjectRef xSvEmbeddedObjectRef = this;
	 if( xSvEmbeddedObjectRef.Is() )
		xSvEmbeddedObjectRef->LoadContent( *aContStm, bOwner );
	 else
#else
		LoadContent( *aContStm, bOwner );
#endif
		aContStm->SetBufferSize( 0 );
		return aContStm->GetError() == SVSTREAM_OK;
	}
	return FALSE;
}

#define CHILD_LIST_VER (BYTE)2
void SvPersist::LoadContent( SvStream & rStm, BOOL bOwner )
{
	if( bOwner )
	{
		BYTE nVers;
		rStm >> nVers;
		DBG_ASSERT( nVers == CHILD_LIST_VER, "version conflict" )

		if( nVers != CHILD_LIST_VER )
			rStm.SetError( SVSTREAM_WRONGVERSION );
		else
		{
			BOOL bList;
			rStm >> bList;
			if( bList )
			{
				SvPersistStream aPStm( SOAPP->aInfoClassMgr, &rStm );
				aPStm >> *GetInfoList();
			}
		}
	}
	// nichts tun bei Fremd-Format
}

/*************************************************************************
|*    SvPersist::SaveContent()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::DoSaveContent( SvStorage * pStor, BOOL bOwner )
{
	// MAC MPW mag's sonst nicht ...
	String aPersistStream( bOwner ? String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( PERSIST_STREAM ) ) 
									: String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( SVEXT_PERSIST_STREAM ) ) );
	SvStorageStreamRef aContStm = pStor->OpenStream( aPersistStream );
	if( aContStm.Is() )
	{
		// Version am Stream setzen
		aContStm->SetVersion( pStor->GetVersion() );
		aContStm->SetBufferSize( 8192 );
#if defined( SOLARIS ) && defined( C40 )
	 // patch to work around Sparc-Compiler bug with virtual multiple inheritance
	 SvEmbeddedObjectRef xSvEmbeddedObjectRef = this;
	 if( xSvEmbeddedObjectRef.Is() )
		 xSvEmbeddedObjectRef->SaveContent( *aContStm, bOwner );
	 else
#else
		SaveContent( *aContStm, bOwner );
#endif
		aContStm->SetBufferSize( 0 );
		return aContStm->GetError() == SVSTREAM_OK;
	}
	return FALSE;
}

void SvPersist::SaveContent( SvStream & rStm, BOOL bOwner )
{
	if( bOwner )
	{
		rStm << (BYTE)CHILD_LIST_VER;
		const SvInfoObjectMemberList * pList = GetObjectList();
		if( pList && pList->Count() )
		{
			rStm << (BOOL)TRUE;
			SvPersistStream aPStm( SOAPP->aInfoClassMgr, &rStm );
			aPStm << *pList;
		}
		else
			rStm << (BOOL)FALSE;
	}
	// nichts tun bei Fremd-Format
}


/*************************************************************************
|*    SvPersist::LoadChilds()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::LoadChilds()
{
	BOOL bRet = TRUE;
	const SvInfoObjectMemberList * pList = GetObjectList();
	if( pList )
	{
		SvPersistRef aPersist;
		for( UINT32 i = 0; i < pList->Count(); i++ )
		{
			// alle Objecte laden
			if( !GetObject( pList->GetObject( i )->GetObjName() ).Is() )
				bRet = FALSE;
		}
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::SaveChilds()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::SaveChilds()
{
	BOOL bRet = TRUE;
	if( pChildList )
	{ // Rekursiv alle Childs retten
		for( ULONG i = 0; i < pChildList->Count(); i++ )
		{
			SvInfoObject * pEle = pChildList->GetObject( i );

			if( pEle->GetPersist() )
			{
				if( pEle->GetPersist()->DoSave() )
				{
					// Immer Commit auf ChildObjecte
					if( !pEle->GetPersist()->GetStorage()->Commit() )
						bRet = FALSE;
				}
				else
					bRet = FALSE;
			}
			pEle = pChildList->Next();
		}
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::SaveAsChilds()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::SaveAsChilds( SvStorage * pStor )
{
	BOOL bRet = TRUE;

	if( pChildList )
	{ // Rekursiv alle Childs retten
		for( ULONG i = 0; i < pChildList->Count(); i++ )
		{
			SvInfoObject * pEle = pChildList->GetObject( i );

			if( pStor->GetVersion() < SOFFICE_FILEFORMAT_NOW )
				// Objekt laden
				GetObject( pEle->GetObjName() );

			if( pEle->GetPersist() )
			{
				SvStorageRef aEleStor = pStor->OpenStorage( pEle->GetStorageName() );
				if( !aEleStor.Is() )
					return FALSE;

				// auf die gew"unschte Version setzen
				aEleStor->SetVersion( pStor->GetVersion() );
				if( pEle->GetPersist()->DoSaveAs( aEleStor ) )
				{
					// Immer Commit auf ChildObjecte
					if( !aEleStor->Commit() )
						bRet = FALSE;
				}
				else
					bRet = FALSE;
				/*Optimierung spaeter
				if( pEle->IsModified() )
				{
				}
				else
				{
					SvStorageRef
					pEle->DoHandsOff(
				}
				*/
			}
			else
			{
				bRet = GetStorage()->CopyTo( pEle->GetStorageName(), pStor,
											 pEle->GetStorageName() );
			}
		}
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::SaveCompletedChilds()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvPersist::SaveCompletedChilds( SvStorage * pStor )
{
	BOOL bRet = TRUE;

	if( pChildList && pChildList->Count() )
	{ // Rekursiv alle Childs retten
		for( ULONG i = 0; i < pChildList->Count(); i++ )
		{
			SvInfoObject * pEle = pChildList->GetObject( i );

			if( pEle->GetPersist() )
			{
				if( pStor )
				{
					SvStorageRef aEleStor;
					aEleStor = pStor->OpenStorage( pEle->aStorName );
					//aEleStor = pStor->OpenStorage( pEle->GetStorageName() );
					// Storage ist schon kopiert
					if( !aEleStor.Is()
					  || !pEle->GetPersist()->DoSaveCompleted( aEleStor ) )
						return FALSE;
				}
				else
				{
					// Storage ist schon kopiert
					if( !pEle->GetPersist()->DoSaveCompleted() )
						return FALSE;
				}
			}
			pEle = pChildList->Next();
		}
	}
	return bRet;
}

/*************************************************************************
|*    SvPersist::CleanUp()
|*
|*    Beschreibung
|*
|*    Loescht Storages, die im SvInfoObject dafuer geflagt wurden
*************************************************************************/

void SvPersist::CleanUp( BOOL bRecurse)
{
//! Bitte demnaechst nicht rekursiv

	if( pChildList && pChildList->Count() )
	{
		for(ULONG i=0;i<pChildList->Count();)
		{
			SvInfoObjectRef pEle = pChildList->GetObject(i);
			if( bRecurse )
			{
				SvPersistRef xPer = pEle->GetPersist();
				if(!xPer.Is())
				{
					SvStorageRef aEleStor;
					aEleStor = GetStorage()->
						OpenStorage( pEle->GetStorageName() );
					if(!aEleStor.Is())
						continue;
					xPer=new SvPersist;
					xPer->DoOwnerLoad( aEleStor);
					pEle->SetObj(xPer);
					xPer->CleanUp();
				}
			}
			if(pEle->IsDeleted())
			{
				DBG_ASSERT(pEle->GetRefCount()==2,
						   "Loeschen von referenziertem Storage");
				String aStorName(pEle->GetStorageName());
				Remove(pEle);
				GetStorage()->Remove(aStorName);
			}
			else
				i++;
		}
	}
}




