/*************************************************************************
 *
 *  $RCSfile: bindctx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#include <bindctx.hxx>
#include <svstor.hxx>
#include <dtrans.hxx>
#include <soerr.hxx>
#pragma hdrstop

//=========================================================================
class Impl_BindContext
{
public:
	SvObjectMemberList  aBoundList;
	UINT32				nTickCountDeadline;
	StreamMode			nStreamMode;
	StorageMode			nStorageMode;
	BOOL				bModesSet;
};

//=========================================================================
//==================class SvBindContext====================================
//=========================================================================
SV_OLE_FACTORY(SvBindContextFactory)
	{
	}
};
TYPEINIT1(SvBindContextFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS1_DLL(SvBindContext,SvBindContextFactory,SvObject,
				0x96ea4480, 0x5c78, 0x11cf,
				0x89, 0xca, 0x0, 0x80, 0x29, 0xe4, 0xb0, 0xb1 );


IUnknown * SvBindContext::GetMemberInterface( const SvGlobalName & )
{
	return NULL;
}

void SvBindContext::TestMemberObjRef( BOOL /*bFree*/ )
{
}

#ifdef TEST_INVARIANT
void SvBindContext::TestMemberInvariant( BOOL /*bPrint*/ )
{
}
#endif



//=========================================================================
SvBindContext::SvBindContext()
/*  [Beschreibung]

	Diese Klasse bildet nur auf das System ab. Deswegen sind alle
	ihre Instanzen unter OLE extern.

	[Querverweise]

	<SvObject::SetExtern>
*/
	: pObjI( NULL )
{
	pImp = new Impl_BindContext();
	pImp->nStreamMode = STREAM_STD_READWRITE;
	pImp->nStorageMode = 0;
	pImp->bModesSet = FALSE;
	pImp->nTickCountDeadline = 0;
}

//=========================================================================
SvBindContext::~SvBindContext()
/*  [Beschreibung]

	Destruktor der Klasse SvBindContext.
*/
{
	delete pImp;
}

//========================================================================
void SvBindContext::MakeUnknown()
/*  [Beschreibung]

	[Querverweise]

	<SvObject::MakeUnknown>
*/
{
}

//======================================================================
StreamMode SvBindContext::GetStreamMode() const
/*  [Beschreibung]

	Der im BindContext gespeicherte StreamMode wird zur"uckgegeben.
	Der Default ist STREAM_STD_READWRITE.

	[R"uckgabewert]

	StreamMode		Der eingetragene StreamMode wird zur"uckgegeben.

	[Querverweise]

	<SvBindContext::SetMode>, <SvStream>
*/
{
	return pImp->nStreamMode;
}

//======================================================================
short SvBindContext::GetStorageMode() const
/*  [Beschreibung]

	Der im BindContext gespeicherte StorageMode wird zur"uckgegeben.
	Der Default ist 0.

	[R"uckgabewert]

	StreamMode		Der eingetragene StorageMode wird zur"uckgegeben.

	[Querverweise]

	<SvBindContext::SetStorageMode>, <SvStorage>
*/
{
	GetStreamMode(); // holt auch den StorageMode
	return pImp->nStorageMode;
}

//======================================================================
void SvBindContext::SetMode
(
	StreamMode nStreamMode,	/* Der StreamMode */
	short nStorMode			/* Der StorageMode */
)
/*  [Beschreibung]

	Im BindContext werden diese Modes zum "offnen von Streams oder
	Storages gesetzt. Bindet der Moniker sp"ater zu einem Storage-Medium,
	dann nimmt er diese Modes.

	[Querverweise]

	<SvBindContext::GetStreamMode>, <SvBindContext::GetStorageMode>,
*/
{
	// setzen des Modes ist nur lokal
	pImp->nStreamMode = nStreamMode;
	pImp->nStorageMode = nStorMode;
	pImp->bModesSet = TRUE;
}

//======================================================================
void SvBindContext::SetTypeList
(
	const SvDataTypeList & rList	/* Die Liste der Typen, die
									   unterst"utzt werden. */
)
/*  [Beschreibung]

	Die Liste der vom Aufrufer unterst"utzen Formate. In der
	<::Bind2Storage> Methode liefert der Server ein
	in rList angegebenes Format. Im Gegensatz zum Clipboard
	wird das Ausw"ahlen des besten Formates vom Server und nicht vom
	Client "ubernommen.
	Ist die Liste leer gibt der Server das Defaultformat zur"uck.

	[Querverweise]

	<SvBindContext::GetTypeList>
*/
{
	if( !pTypeList )
		pTypeList = new SvDataTypeList( rList );
	else
		*pTypeList = rList;
}

//======================================================================
const SvDataTypeList & SvBindContext::GetTypeList() const
/*  [Beschreibung]

	Die im BindContext eingesetzte Liste wird zur"uckgegeben.

	[Querverweise]

	<SvBindContext::SetTypeList>
*/
{
	if( !pTypeList )
		((SvBindContext *)this)->pTypeList = new SvDataTypeList();
	return *pTypeList;
}

//======================================================================
UINT32 SvBindContext::GetDeadline() const
/*  [Beschreibung]

	Die im BindContext gespeicherte Zeit, die zum Binden des Objektes
	erlaubt ist, wird zur"uckgegeben.

	[R"uckgabewert]

	UINT32			Die Zeitgrenze in Milli Sekunden wird zur"uckgegeben.
					0 bedeutet kein Zeitlimit.

	[Querverweise]

	<SvBindContext::SetDeadline>
*/
{
	return pImp->nTickCountDeadline;
}

//======================================================================
void SvBindContext::SetDeadline( UINT32 nMilliSec )
/*  [Beschreibung]

	Die im BindContext gespeicherte Zeit, die zum Binden des Objektes
	erlaubt ist, wird gesetzt.

	[R"uckgabewert]

	UINT32			Die Zeitgrenze in Milli Sekunden wird zur"uckgegeben.
					0 bedeutet kein Zeitlimit.

	[Querverweise]

	<SvBindContext::SetDeadline>
*/
{
	pImp->nTickCountDeadline = nMilliSec;
}

//======================================================================
void SvBindContext::Register
(
	SvStatusCallback * pCB,	/* An diesem Objekt werden die Callbacks
							   gerufen. */
	SvCallbackOption nCBO	/* Gibt an, welche Callbacks gerufen werden
							   sollen. */
)
/*  [Beschreibung]

	Mit dieser Methode wird ein Callback-Objekt angemeldet. W"ahren
	die Verbindung aufgebaut wird, oder wenn Daten "ubertragen werden,
	wird dieses Objekt benachrichtigt.
	Es k"onnen mehrere Objekte an dem BindContext registriert werden.
	Die zuletzt registrierten werden zuerst benachrichtigt.
	Der BindContext h"alt das Objekt, bis zum Revoke oder bis der
	Context zerst"ort wird, fest.

	[Anmerkung]

	CALLBACK_ONSTARTBINDING	und CALLBACK_ONSTOPBINDING m"uessen immer
	paarweise angegeben werden.

	[Querverweise]

	<SvBindContext::Revoke>
*/
{
}

//======================================================================
BOOL SvBindContext::Revoke
(
	SvStatusCallback * pCB	/* Das Objekt, welches vorher mit Register
							   angemeldet wurde. */
)
/*  [Beschreibung]

	Das mit Register eingetragene Objekt wird abgemeldet.

	[Anmerkung]

	Das Objekt kann nicht ausgetragen werden, wenn eine asynchrone
	Operation l"auft.

	[Querverweise]

	<SvBindContext::Register>
*/
{
	return FALSE;
}

//======================================================================
void SvBindContext::RegisterObjBound
(
	SvObject * pObj	/* Dieses Objekt wird im BindContext solange
					   festgehalten, wie der BindContext lebt oder bis
					   es mit <SvBindContext::RevokeObjBound> ausgetragen wird. */
)
/*  [Beschreibung]

	Die Objekte, die w"arend des Bindens nicht freigegeben werden sollen,
	werden hier eingetragen.

	[Querverweise]

	<SvBindContext::RevokeObjBound>
*/
{
	pImp->aBoundList.Append( pObj );
}

//======================================================================
void SvBindContext::RevokeObjBound
(
	SvObject * pObj	/* Dieses Objekt, dass vorher mit
					   <SvBindContext::RevokeObjBound> eingetragen wurde,
					   wird freigegeben. */
)
/*  [Beschreibung]

	Die Objekte pObj wird freigegeben.

	[Querverweise]

	<SvBindContext::RegisterObjBound>
*/
{
	pImp->aBoundList.Remove( pObj );
}

//=========================================================================
//==================class SoBindInfo=======================================
//=========================================================================
SoBindInfo::SoBindInfo()
	: nUrlInfo( 0 )
	, eBindAction( BINDACTION_GET )
	, pData( NULL )
/*	[Beschreibung]

	Konstruktor der Klasse SoBindInfo. nUrlInfo wird auf 0 und
	eBindAction wird auf BINDACTION_GET gesetzt.
*/
{
}

//=========================================================================
SoBindInfo::SoBindInfo( const String & rExtraName, INT32 nUrlInfoP )
	: aExtraInfo( rExtraName )
	, nUrlInfo( nUrlInfoP )
	, eBindAction( BINDACTION_GET )
	, pData( NULL )
/*	[Beschreibung]

	Konstruktor der Klasse SoBindInfo.
	eBindAction wird auf BINDACTION_GET gesetzt.
*/
{
}

//=========================================================================
SoBindInfo::~SoBindInfo()
/*	[Beschreibung]

	Destruktor der Klasse SoBindInfo.
*/
{
	delete pData;
}

//=========================================================================
const SvData & SoBindInfo::GetData() const
/*	[Beschreibung]

	Sollen Daten gesendet werden (BINDACTION_POST, BINDACTION_PUT),
	dann m"ussen sie in die BindInfo eingesetzt werden. Mit dieser
	Methode werden die Daten, von dem der sie "ubertr"agt abgeholt.

	[R"uckgabewert]

	SvData & 	Die Daten die gesendet werden sollen.
*/
{
	if( !pData )
		// keine Daten und kein Medium
		((SoBindInfo *)this)->pData = new SvData( 0, 0 );
	return *pData;
}

//=========================================================================
//==================class SvStatusCallback=================================
//=========================================================================
SV_OLE_FACTORY(SvStatusCallbackFactory)
	{
	}
};
TYPEINIT1(SvStatusCallbackFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS1_DLL(
	SvStatusCallback,SvStatusCallbackFactory,SvObject,
	0x1ef15a21, 0x6634, 0x11cf,
	0x89, 0xcb, 0x0, 0x80, 0x29, 0xe4, 0xb0, 0xb1);


//=========================================================================
SvStatusCallback::SvStatusCallback()
	: pObjI ( NULL )
/*	[Beschreibung]

	Konstruktor der Klasse SvStatusCallback.
*/
{
}

//=========================================================================
SvStatusCallback::SvStatusCallback
(
	SvBindContext * pBindCtx	/* Kontext, unter dem der Callback
								   angemeldet wird. */
)
	: pObjI ( NULL )
/*	[Beschreibung]

	Konstruktor der Klasse SvStatusCallback. Das Objekt wird sofort
	mit CALLBACK_ALLONIBSC an der pBindCtx Instanz angemeldet.
*/
{
	pBindCtx->Register( this );
}

//=========================================================================
SvStatusCallback::~SvStatusCallback()
/*	[Beschreibung]

	Destruktor der Klasse SvStatusCallback.
*/
{
}


//=========================================================================
IUnknown * SvStatusCallback::GetMemberInterface( const SvGlobalName & )
/*	[Beschreibung]

	Jede Klasse mu eine Abfrage auf die Ole-Interfaces unterst"utzen.
	Es d"urfen nur die eigenen Ole-Interfaces und nicht die der
	Super-Klassen abgefragt werden. Siehe <So Ole-Unterst"utzung>.
*/
{
	return NULL;
}

//=========================================================================
#ifdef TEST_INVARIANT
void SvStatusCallback::TestMemberObjRef( BOOL /*bFree*/ )
/*	[Beschreibung]

	Siehe <So Debugging>.
*/
{
}

//=========================================================================
void SvStatusCallback::TestMemberInvariant( BOOL bPrint )
/*	[Beschreibung]

	Siehe <So Debugging>.
*/
{
#ifdef DBG_UTIL
	if( bPrint )
	{
		DBG_TRACE( "SvStatusCallback::Invariant:" )
	}
#endif
}
#endif

//=========================================================================
const SoBindInfo & SvStatusCallback::GetBindInfo()
{
	return aBindInfo;
}

//=========================================================================
SvBindMode SvStatusCallback::GetBindMode()
{
	return nBindMode;
}

//=========================================================================
void SvStatusCallback::OnStartBinding
(
	SoBinding * pBinding	/* Objekt um die Binding-Operation zu
							   beeinflussen */
)
/*	[Beschreibung]

	Diese Methode wird gerufen, wenn ein asynchroner Moniker die
	Binding-Operation startet. Das "Ubergebene Binding kann bis
	zum Aufruf der Methode <SvStatusCallback::OnStopBinding>
	festgehalten werden. Als Default merkt sich die Methode das
	pBinding Objekt. Es kann mit <SvStatusCallback::GetBinding>
	jederzeit abgefragt werden.

	[Anmerkung]

	Die Methode muss von der abgeleiteten Klasse gerufen werden.
*/
{
	xBinding = pBinding;
}

//=========================================================================
void SvStatusCallback::OnStopBinding
(
	ErrCode nErr,        	/* Ursache die zum Beenden der Operation
							   gef"uhrt hat. */
	const String & rStatus  /* Statustext "uber die Ursache. */
)
/*	[Beschreibung]

	Diese Methode wird gerufen, wenn die Binding-Operation beendet wird.
	Wurde <SvStatusCallback::StartBinding> gerufen, dann wird auf
	jedenfall StopBinding gerufen.
	Wurde sich das Binding gemerkt, dann muss es jetzt freigegeben werden.

	[Anmerkung]

	Die Methode muss von der abgeleiteten Klasse gerufen werden.
*/
{
}

//=========================================================================
void SvStatusCallback::OnDataAvailable
(
	SvAsyncData,
	ULONG nBytes,
	SvData & rData
)
{
}

//=========================================================================
void SvStatusCallback::OnObjAvailable
(
	SvObject * /* pObj */
)
{
}

//=========================================================================
void SvStatusCallback::OnProgress
(
	ULONG 			/* nPos */,
	ULONG 			/* nMaxPos */,
	SvBindStatus,
	const String&	/* rStatusText */
)
{
}

//=========================================================================
INT32 SvStatusCallback::GetPriority() const
{
	return 0;
}


//=========================================================================
//==================class SoBinding=================================
//=========================================================================
SV_OLE_FACTORY(SoBindingFactory)
	{
	}
};
TYPEINIT1(SoBindingFactory,SvOleFactory);

SO2_IMPL_STANDARD_CLASS1_DLL(
	SoBinding,SoBindingFactory,SvObject,
	0x29448aa2, 0xa1a4, 0x11cf,
	0x89, 0xca, 0x0, 0x80, 0x29, 0xe4, 0xb0, 0xb1 );


//=========================================================================
SoBinding::SoBinding()
	: pObjI ( NULL )
/*	[Beschreibung]

	Konstruktor der Klasse SoBinding.
*/
{
}

//=========================================================================
SoBinding::~SoBinding()
/*	[Beschreibung]

	Destruktor der Klasse SoBinding.
*/
{
}


//=========================================================================
IUnknown * SoBinding::GetMemberInterface( const SvGlobalName & )
/*	[Beschreibung]

	Jede Klasse mu eine Abfrage auf die Ole-Interfaces unterst"utzen.
	Es d"urfen nur die eigenen Ole-Interfaces und nicht die der
	Super-Klassen abgefragt werden. Siehe <So Ole-Unterst"utzung>.
*/
{
	return NULL;
}

//=========================================================================
#ifdef TEST_INVARIANT
void SoBinding::TestMemberObjRef( BOOL /*bFree*/ )
/*	[Beschreibung]

	Siehe <So Debugging>.
*/
{
}

//=========================================================================
void SoBinding::TestMemberInvariant( BOOL bPrint )
/*	[Beschreibung]

	Siehe <So Debugging>.
*/
{
#ifdef DBG_UTIL
	if( bPrint )
	{
		DBG_TRACE( "SoBinding::Invariant:" )
	}
#endif
}
#endif


//=========================================================================
ErrCode SoBinding::Abort()
/*	[Beschreibung]

	Eine asynchrone Operation wird abgebrochen.

	[R"uckgabewert]

	ERRCODE_NONE, 				Die Operation wurde abgebrochen.
	ERRCODE_SO_FALSE,   		Die Operation ist schon abgebrochen.
	ERRCODE_SO_GENERALERROR,	Die Operation kann nicht abgebrochen
								werden.

	[Querverweise]
*/
{
	return ERRCODE_SO_GENERALERROR;
}

//=========================================================================
ErrCode SoBinding::Suspend()
/*	[Beschreibung]

	Eine asynchrone Operation wird unterbrochen. Sie kann mit Resume
	fortgesetzt werden.

	[R"uckgabewert]

	ERRCODE_NONE, 				Die Operation wurde untergebrochen.
	ERRCODE_SO_FALSE,   		Die Operation ist schon untergebrochen.
	ERRCODE_SO_GENERALERROR,	Die Operation kann nicht untergebrochen
								werden.

	[Querverweise]

	<SoBinding::Resume>
*/
{
	return ERRCODE_SO_GENERALERROR;
}

//=========================================================================
ErrCode SoBinding::Resume()
/*	[Beschreibung]

	Eine asynchrone Operation die unterbrochen wurde, wird
	fortgesetzt.

	[R"uckgabewert]

	ERRCODE_NONE, 				Die Operation wird fortgesetzt.
	ERRCODE_SO_FALSE,   		Die Operation wurde nicht untergebrochen.
	ERRCODE_SO_GENERALERROR,	Die Operation kann nicht fortgesetzt
								werden.

	[Querverweise]

	<SoBinding::Resume>
*/
{
	return ERRCODE_SO_GENERALERROR;
}

//=========================================================================
BOOL SoBinding::SetPriority
(
	long /* nPriority */
)
/*	[Beschreibung]

	Noch nicht festgelegt (Ole2 kompatible).

	[Querverweise]

	<SoBinding::GetPriority>
*/
{
	return FALSE;
}

//=========================================================================
long SoBinding::GetPriority()
/*	[Beschreibung]

	Noch nicht festgelegt (Ole2 kompatible).

	[Querverweise]

	<SoBinding::GetPriority>
*/
{
	return 0;
}

