/*************************************************************************
 *
 *  $RCSfile: linkmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _DATE_HXX //autogen
#include <tools/date.hxx>
#endif


#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#include "linkmgr.hxx"
#include <vcl/msgbox.hxx>
#include <dtrans.hxx>
#include <embobj.hxx>
#include <linkname.hxx>
#include "lnkbase.hxx"
#include "pseudo.hxx"
#include "ddeobj.hxx"
#include "segmentc.hxx"
#include "svuidlg.hrc"

#pragma hdrstop

#define _SVSTDARR_STRINGSDTOR

#include <svtools/svstdarr.hxx>
#include "implbase.hxx"

SEG_EOFGLOBALS()

SV_IMPL_PTRARR( SvBaseLinks, SvBaseLinkRefPtr )

SvLinkManager::SvLinkManager( SvPersist * pCacheContP )
	// RefCount nicht erhoehen
	: pCacheCont( pCacheContP )
{

}


SvLinkManager::~SvLinkManager()
{
	SvBaseLinkRef** ppRef = (SvBaseLinkRef**)aLinkTbl.GetData();
	for( USHORT n = aLinkTbl.Count(); n; --n, ++ppRef )
	{
		if( (*ppRef)->Is() )
		{
			(*(*ppRef))->Disconnect();
			(*(*ppRef))->pLinkMgr = 0;
		}
		delete *ppRef;
	}
	ClearErrorLinkTbl();
}

/************************************************************************
|*    SvLinkManager::LoadLink()
|*
|*    Beschreibung
*************************************************************************/
SvBaseLinks & SvLinkManager::GetErrorLinkTbl()
{
	return aErrorLinkTbl;
}

void SvLinkManager::ClearErrorLinkTbl()
{
	aErrorLinkTbl.DeleteAndDestroy( 0, aErrorLinkTbl.Count() );
}

/************************************************************************
|*    SvLinkManager::LoadLink()
|*
|*    Beschreibung
*************************************************************************/

SvBaseLinkRef SvLinkManager::LoadLink( USHORT nType, SvStream & rStm )
{
	SvBaseLinkRef xBL;
	if( nType == SvBaseLink::PersistId() )
	{
		xBL = new SvBaseLink();
		SvBaseLink & rBL = *xBL;
		rStm >> rBL;
	}
	if( xBL.Is() )
	{
		if( OBJECT_CLIENT_OLE_CACHE == xBL->GetObjType() )
		{// bei diesem Linktyp muss es immer eine Objekt geben
			DBG_ASSERT( pCacheCont, "no cache container (SvPersist) in SvLinkManager" )
			SvEmbeddedObjectRef xObj;
			xObj = &pCacheCont->GetObject( xBL->GetCacheName()->GetName() );
			DBG_ASSERT( xObj.Is(), "cannot load object" )
			if( xObj.Is() )
				xBL->SetCacheObj( xObj );
			else
				xBL.Clear();
		}
	}
	if( !xBL.Is() )
		rStm.SetError( SVSTREAM_GENERALERROR );
	return xBL;
}

/************************************************************************
|*    SvLinkManager::Load()
|*
|*    Beschreibung
*************************************************************************/

#define LNKMGR_VERS	1
void SvLinkManager::Load( SvStream & rStm )
{
	BYTE nVers = 0;
	rStm >> nVers;
	if( nVers == LNKMGR_VERS )
	{
		USHORT nCount = 0;
		rStm >> nCount;
		while( nCount-- )
		{
			USHORT nType;
			rStm >> nType;
			SvBaseLinkRef aBL = LoadLink( nType, rStm );
			if( aBL.Is() )
				Insert( *aBL );
		}
	}
	else
		rStm.SetError( SVSTREAM_WRONGVERSION );
}

/************************************************************************
|*    SvLinkManager::Save()
|*
|*    Beschreibung
*************************************************************************/

void SvLinkManager::Save( SvStream & rStm )
{
	rStm << (BYTE)LNKMGR_VERS;

	USHORT nCount = 0;
	// Count festlegen
	SvBaseLinkRef** ppRef = (SvBaseLinkRef**)aLinkTbl.GetData();
	USHORT n;
	for( n = aLinkTbl.Count(); n; --n, ++ppRef )
	{
		if( (*ppRef)->Is() )
			nCount++;
	}

	rStm << nCount;
	// einzelne Links schreiben
	ppRef = (SvBaseLinkRef**)aLinkTbl.GetData();
	for( n = aLinkTbl.Count(); n; --n, ++ppRef )
	{
		if( (*ppRef)->Is() )
		{
			rStm << (USHORT)(*(*ppRef))->GetPersistId();
			rStm << (***ppRef);
		}
	}
}

/************************************************************************
|*    SvLinkManager::Remove()
|*
|*    Beschreibung
*************************************************************************/

void SvLinkManager::Remove( const SvBaseLink& rLink )
{
	// keine Links doppelt einfuegen
	int bFound = FALSE;
	SvBaseLinkRef** ppRef = (SvBaseLinkRef**)aLinkTbl.GetData();
	for( USHORT n = aLinkTbl.Count(); n; --n, ++ppRef )
	{
		if( &rLink == *(*ppRef) )
		{
			(*(*ppRef))->Disconnect();
			(*(*ppRef))->pLinkMgr = 0;
			(*(*ppRef)).Clear();
			bFound = TRUE;
		}

		// falls noch leere rum stehen sollten, weg damit
		if( !(*ppRef)->Is() )
		{
			delete *ppRef;
			aLinkTbl.Remove( aLinkTbl.Count() - n, 1 );
			if( bFound )
				return ;
			--ppRef;
		}
	}
}


void SvLinkManager::Remove( USHORT nPos, USHORT nCnt )
{
	if( nCnt && nPos < aLinkTbl.Count() )
	{
		if( nPos + nCnt > aLinkTbl.Count() )
			nCnt = aLinkTbl.Count() - nPos;

		SvBaseLinkRef** ppRef = (SvBaseLinkRef**)aLinkTbl.GetData() + nPos;
		for( USHORT n = nCnt; n; --n, ++ppRef )
		{
			if( (*ppRef)->Is() )
			{
				(*(*ppRef))->Disconnect();
				(*(*ppRef))->pLinkMgr = 0;
			}
			delete *ppRef;
		}
		aLinkTbl.Remove( nPos, nCnt );
	}
}


BOOL SvLinkManager::Insert( SvBaseLink& rLink )
{
	// keine Links doppelt einfuegen
	for( USHORT n = 0; n < aLinkTbl.Count(); ++n )
	{
		SvBaseLinkRef* pTmp = aLinkTbl[ n ];
		if( !pTmp->Is() )
			aLinkTbl.DeleteAndDestroy( n-- );

		if( (const SvBaseLink*)&rLink == *pTmp )
			return FALSE;
	}

	SvBaseLinkRef* pTmp = new SvBaseLinkRef( &rLink );
	rLink.pLinkMgr = this;
	aLinkTbl.Insert( pTmp, aLinkTbl.Count() );
	return TRUE;
}


BOOL SvLinkManager::InsertLink( SvBaseLink& rLink,
								USHORT nObjType,
								USHORT nUpdateMode,
								const String* pName )
{
	// unbedingt zuerst
	rLink.SetObjType( nObjType );
	if( pName )
		rLink.SetName( new SvLinkName( *pName ) );
	rLink.SetUpdateMode( nUpdateMode );
	return Insert( rLink );
}


BOOL SvLinkManager::InsertDDELink( SvBaseLink& rLink,
									const String& rServer,
									const String& rTopic,
									const String& rItem )
{
	if( !( OBJECT_CLIENT_SO & rLink.GetObjType() ) )
		return FALSE;

	String sCmd;
	::MakeLnkName( sCmd, &rServer, rTopic, rItem );

	rLink.SetObjType( OBJECT_CLIENT_DDE );
	rLink.SetName( new SvLinkName( sCmd ) );
	return Insert( rLink );
}

/*

BOOL SvLinkManager::InsertSoLink( SvBaseLink& rLink, SvData& rData )
{
	DBG_ASSERT( !this, "Fehlendes Feature");
	return FALSE; 		//Insert( rLink, new SvSoLink( rData ) );
}
*/


BOOL SvLinkManager::InsertDDELink( SvBaseLink& rLink )
{
	DBG_ASSERT( OBJECT_CLIENT_SO & rLink.GetObjType(), "no OBJECT_CLIENT_SO" )
	if( !( OBJECT_CLIENT_SO & rLink.GetObjType() ) )
		return FALSE;

	if( rLink.GetObjType() == OBJECT_CLIENT_SO )
		rLink.SetObjType( OBJECT_CLIENT_DDE );

	return Insert( rLink );
}


BOOL SvLinkManager::InsertSoLink( SvBaseLink& rLink )
{
	DBG_ASSERT( OBJECT_CLIENT_SO & rLink.GetObjType(), "no OBJECT_CLIENT_SO" )
	if( !( OBJECT_CLIENT_SO & rLink.GetObjType() ) )
		return FALSE;

	if( rLink.GetObjType() == OBJECT_CLIENT_SO )
		rLink.SetObjType( OBJECT_CLIENT_OLE_CACHE );

	return Insert( rLink );
}


		// erfrage die Strings fuer den Dialog
BOOL SvLinkManager::GetDisplayNames( const SvBaseLink& rLink,
										String* pType,
										String* pFile,
										String* pLink,
										String* pFilter ) const
{
	BOOL bRet = FALSE;
	const SvLinkName* pLNm = rLink.GetLinkSourceName();
	if( pLNm )
		switch( rLink.GetObjectType() )
		{
		case OBJECT_CLIENT_DDE:
			{
				USHORT nTmp = 0;
				String sCmd( pLNm->GetName() );
				String sServer( sCmd.GetToken( 0, cTokenSeperator, nTmp ) );
				String sTopic( sCmd.GetToken( 0, cTokenSeperator, nTmp ) );

				if( pType )
					*pType = sServer;
				if( pFile )
					*pFile = sTopic;
				if( pLink )
					*pLink = sCmd.Copy( nTmp );
				bRet = TRUE;
			}
			break;
		case OBJECT_CLIENT_OLE_CACHE:
			{
				DBG_ASSERT( rLink.GetCacheObj(), "no object" )
				SvEmbeddedObjectRef aEO = rLink.GetCacheObj();
				DBG_ASSERT( aEO.Is(), "no embedded object" )

				if( pType )
				{
					//SvGlobalName 	aGN = aEO->GetClassName();
					//String			aAppName = aEO->GetLongUserName();
					String			aObjName = aEO->GetShortTypeName();
					*pType = aObjName;
				}
				if( pFile )
					// ??? muss noch aufgeteilt werden
					*pFile = pLNm->GetDisplayName();
				if( pLink )
					// ??? muss noch aufgeteilt werden
					*pLink = pLNm->GetDisplayName();
				bRet = TRUE;
			}
			break;
		case OBJECT_CLIENT_OLE:
			{
				DBG_ERROR( "SvLinkManager::GetDisplayNames: not implemented OBJECT_CLIENT_OLE" )
			}
			break;
		}
	return bRet;
}


static BOOL _SearchStr( const SvStringsDtor& rArr, const String& rSrch )
{
	for( USHORT n = rArr.Count(); n; )
		if( COMPARE_EQUAL == rSrch.CompareTo( *rArr[ --n ] ) )
			return TRUE;
	return FALSE;
}



void SvLinkManager::UpdateAllLinks( BOOL bAskUpdate, BOOL bCallErrHdl,
									BOOL bUpdateGrfLinks )
{
	SvStringsDtor aApps, aTopics, aItems;
	String sApp, sTopic, sItem;

	// erstmal eine Kopie vom Array machen, damit sich updatende Links in
	// Links in ... nicht dazwischen funken!!
	SvPtrarr aTmpArr( 255, 50 );
	USHORT n;
	for( n = 0; n < aLinkTbl.Count(); ++n )
	{
		SvBaseLink* pLink = *aLinkTbl[ n ];
		if( !pLink )
		{
			Remove( n-- );
			continue;
		}
		aTmpArr.Insert( pLink, aTmpArr.Count() );
	}

	for( n = 0; n < aTmpArr.Count(); ++n )
	{
		SvBaseLink* pLink = (SvBaseLink*)aTmpArr[ n ];

		// suche erstmal im Array nach dem Eintrag
		USHORT nFndPos = USHRT_MAX;
		for( USHORT i = 0; i < aLinkTbl.Count(); ++i )
			if( pLink == *aLinkTbl[ i ] )
			{
				nFndPos = i;
				break;
			}

		if( USHRT_MAX == nFndPos )
			continue;					// war noch nicht vorhanden!

		// Graphic-Links noch nicht updaten
		if( !pLink->IsVisible() ||
			( !bUpdateGrfLinks && OBJECT_CLIENT_GRF == pLink->GetObjType() ))
			continue;

		if( bAskUpdate )
		{
			ResId aId( STR_QUERYUPDATELINKS, SOAPP->GetResMgr() );
			int nRet = QueryBox( 0, WB_YES_NO | WB_DEF_YES, String(aId) ).Execute();
			if( RET_YES != nRet )
				return ;		// es soll nichts geupdatet werden

			bAskUpdate = FALSE;		// einmal reicht
		}

		if( !pLink->Update() && bCallErrHdl )
		{
			GetDisplayNames( *pLink, &sApp, &sTopic, &sItem );

			switch( pLink->GetObjType() )
			{
			case OBJECT_CLIENT_GRF:
				break;

			case OBJECT_CLIENT_DDE:
				{
					BOOL bFound = _SearchStr( aApps, sApp );

					if( bFound )
					{
						if( (bFound = _SearchStr( aTopics, sTopic ) ) )
						{
/*
JP 28.02.96: nicht bis in die Item-Ebene runter gehen
							if( (bFound = _SearchStr( aItems, sItem ) ) )
							{
// noch besondere Behandlung ??
							}
// noch besondere Behandlung ??
*/						}
					}

					if( !bFound )		// wurde noch nicht gemeldet
					{
						// also in die Arrays eintragen und melden
						StringPtr pStr = new String( sApp );
						aApps.Insert( pStr, aApps.Count() );
						pStr = new String( sTopic );
						aTopics.Insert( pStr, aTopics.Count() );
						pStr = new String( sItem );
						aItems.Insert( pStr, aItems.Count() );

						aErrorLinkTbl.Insert( new SvBaseLinkRef( pLink ), aErrorLinkTbl.Count() );
						//pLink->ShowError();
					}
				}
				break;
			}
		}
	}
}

/************************************************************************
|*    SvBaseLink::CreateObject()
|*
|*    Beschreibung
*************************************************************************/

SvPseudoObjectRef SvLinkManager::CreateObj( SvBaseLink & rLink )
{
	if( OBJECT_CLIENT_DDE == rLink.GetObjType() )
		return new SvDDEObject();

	DBG_ASSERT( OBJECT_CLIENT_OLE_CACHE == rLink.GetObjType(), "not implemented" )

	SvPseudoObjectRef xObj;
	if( OBJECT_CLIENT_OLE_CACHE == rLink.GetObjType() )
	{
		DBG_ASSERT( pCacheCont, "no cache container" )
		DBG_ASSERT( rLink.GetObj(), "object already exist" )
		String aCacheName = rLink.GetCacheName()->GetName();
		xObj = &pCacheCont->GetObject( aCacheName );
	}

	return xObj;
}


void SvLinkManager::PrepareReload( SvBaseLink* pLnk )
{
}


BOOL SvLinkManager::InsertServer( SvPseudoObject* pObj )
{
	// keine doppelt einfuegen
	if( !pObj || USHRT_MAX != aServerTbl.GetPos( pObj ) )
		return FALSE;

	aServerTbl.Insert( pObj, aServerTbl.Count() );
	return TRUE;
}


void SvLinkManager::RemoveServer( SvPseudoObject* pObj )
{
	USHORT nPos = aServerTbl.GetPos( pObj );
	if( USHRT_MAX != nPos )
		aServerTbl.Remove( nPos, 1 );
}


void MakeLnkName( String& rName, const String* pType, const String& rFile,
					const String& rLink, const String* pFilter )
{
	if( pType )
		(rName = *pType).EraseLeadingChars().EraseTrailingChars() += cTokenSeperator;
	else if( rName.Len() )
		rName.Erase();

	((rName += rFile).EraseLeadingChars().EraseTrailingChars() +=
		cTokenSeperator ).EraseLeadingChars().EraseTrailingChars() += rLink;
	if( pFilter )
		((rName += cTokenSeperator ) += *pFilter).EraseLeadingChars().EraseTrailingChars();
}





