/*************************************************************************
 *
 *  $RCSfile: lnkbase.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:54:27 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <tools/date.hxx>
#include <tools/time.hxx>
#include <tools/rtti.hxx>


#include "lnkbase.hxx"
#include <tools/date.hxx>
#include <vcl/svapp.hxx>
#include <vcl/msgbox.hxx>

#include <embobj.hxx>
#include "ddeobj.hxx"
#include "linkmgr.hxx"
#include "svuidlg.hrc"
#include <soerr.hxx>

#include <tools/bigint.hxx>
#include <tools/debug.hxx>
#pragma hdrstop

#include "implbase.hxx"
#include <svtools/svdde.hxx>


static DdeTopic* FindTopic( const SvLinkName*, USHORT* = 0 );

class ImplDdeItem : public DdeGetPutItem
{
	SvBaseLink* pLink;
	DdeData aData;
	SvData aSvData;
	BOOL bIsValidData : 1;
	BOOL bIsInDTOR : 1;
public:
	ImplDdeItem( SvBaseLink& rLink, const String& rStr )
		: DdeGetPutItem( rStr ), pLink( &rLink ), bIsValidData( FALSE ),
		bIsInDTOR( FALSE )
	{}
	virtual ~ImplDdeItem();

	virtual DdeData* Get( ULONG );
	virtual BOOL Put( const DdeData* );
	virtual void AdviseLoop( BOOL );

	void Notify()
	{
		bIsValidData = FALSE;
		DdeGetPutItem::NotifyClient();
	}

	BOOL IsInDTOR() const { return bIsInDTOR; }
};


TYPEINIT0( SvBaseLink )

/************************************************************************
|*	  SvBaseLink::SvBaseLink()
|*
|*	  Beschreibung
*************************************************************************/

SvBaseLink::SvBaseLink()
{
	nObjType = OBJECT_CLIENT_SO;
	pLinkMgr = 0;
	pImplData = new ImplBaseLinkData;
	bVisible = bSynchron = bUseCache = TRUE;
	memset( pImplData, 0, sizeof(ImplBaseLinkData) );
}

/************************************************************************
|*	  SvBaseLink::SvBaseLink()
|*
|*	  Beschreibung
*************************************************************************/

SvBaseLink::SvBaseLink( USHORT nUpdateMode, ULONG nContentType )
{
	nObjType = OBJECT_CLIENT_SO;
	pLinkMgr = 0;
	pImplData = new ImplBaseLinkData;
	memset( pImplData, 0, sizeof(ImplBaseLinkData) );
	bVisible = bSynchron = bUseCache = TRUE;

	// falls es ein Ole-Link wird,
	pImplData->ClientType.nUpdateMode = nUpdateMode;
	pImplData->ClientType.nCntntType = nContentType;
	pImplData->ClientType.bIntrnlLnk = FALSE;
}

/************************************************************************
|*	  SvBaseLink::Impl_InitCtor()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::Impl_InitCtor( SvLinkName* pNm, USHORT nObjectType,
								SvPseudoObject* pObj )
{
	bVisible = bSynchron = bUseCache = TRUE;
	xLinkName = pNm;
	pImplData = new ImplBaseLinkData;
	memset( pImplData, 0, sizeof(ImplBaseLinkData) );
	nObjType = nObjectType;

	if( !pObj )
	{
		DBG_ASSERT( pObj, "Wo ist mein zu linkendes Object" );
		return;
	}

	if( OBJECT_DDE_EXTERN == nObjType )
	{
		USHORT nItemStt = 0;
		DdeTopic* pTopic = FindTopic( xLinkName, &nItemStt );
		if( pTopic )
		{
			// dann haben wir alles zusammen
			// MM hat gefummelt ???
			String aStr = pNm->GetDisplayName();
			aStr = aStr.Copy( nItemStt );
			pImplData->DDEType.pItem = new ImplDdeItem( *this, aStr );
			pTopic->InsertItem( pImplData->DDEType.pItem );

			// dann koennen wir uns auch das Advise merken
			xObj = pObj;
		}
	}
	else if( OBJECT_CLIENT_OLE == nObjType )
	{
		xObj = pObj;
	}
	else if( OBJECT_CLIENT_OLE_CACHE == nObjType )
	{
		xObj = pObj;
		pImplData->xCacheObj = &xObj;
		if( !xObj->Owner() )
		{
			DBG_ERROR( "SvBaseLink::Impl_InitCtor, alt fuer Ole-Cache Links" )
			// extern So-Links koennen ihre Daten in einem Cache-Speichern
			// Damit dieser immer Up-To-Date mit den Daten im Link
			// uebereinstimmen, wird dieser eingesetzt
			//xObj->SetBaseLink( this );
		}
	}
	else if( pObj->Connect( *this ) )
		xObj = pObj;
}


/************************************************************************
|*	  SvBaseLink::SvBaseLink()
|*
|*	  Beschreibung
*************************************************************************/

SvBaseLink::SvBaseLink( const String& rNm, USHORT nObjectType,
						SvPseudoObject* pObj )
{
	xLinkName = new SvLinkName( rNm );
	Impl_InitCtor( xLinkName, nObjectType, pObj );
}

/************************************************************************
|*	  SvBaseLink::SvBaseLink()
|*
|*	  Beschreibung
*************************************************************************/

SvBaseLink::SvBaseLink( SvLinkName * pNm, USHORT nObjectType,
						SvPseudoObject* pObj )
{
	Impl_InitCtor( pNm, nObjectType, pObj );
}


/************************************************************************
|*	  SvBaseLink::~SvBaseLink()
|*
|*	  Beschreibung
*************************************************************************/

SvBaseLink::~SvBaseLink()
{
	Disconnect();

	switch( nObjType )
	{
	case OBJECT_DDE_EXTERN:
		if( !pImplData->DDEType.pItem->IsInDTOR() )
			delete pImplData->DDEType.pItem;
		break;
	case OBJECT_CLIENT_OLE:
	case OBJECT_CLIENT_OLE_CACHE:
		delete pImplData->ClientType.pLinkInfo;
		break;
//	case OBJECT_CLIENT_DDE:
//	case OBJECT_CLIENT_FILE:
//		break;
	}

	delete pImplData;
}

/************************************************************************
|*	  SvBaseLink::Load()
|*
|*	  Beschreibung
*************************************************************************/

#define SVBASELINK_VERS 1
void SvBaseLink::Load( SvStream & rStm )
{
	BYTE nVers = 0;
	rStm >> nVers;
	if( nVers == SVBASELINK_VERS )
	{
		USHORT nObjT = 0;
		rStm >> nObjT;
		nObjType = nObjT;
		DBG_ASSERT( OBJECT_CLIENT_OLE_CACHE == nObjType, "only load OBJECT_CLIENT_OLE_CACHE" )
		pImplData->ClientType.pLinkInfo = new Impl_LinkInfo;
		pImplData->xCacheName = new SvLinkName();
		SvLinkName & rLN = *pImplData->xCacheName;
		rStm >> rLN;
	}
}

/************************************************************************
|*	  SvBaseLink::Save()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::Save( SvStream & rStm )
{
	rStm << (BYTE)SVBASELINK_VERS;
	rStm << (USHORT)nObjType;
	DBG_ASSERT( OBJECT_CLIENT_OLE_CACHE == nObjType, "only save OBJECT_CLIENT_OLE_CACHE" )
	if( OBJECT_CLIENT_OLE_CACHE == nObjType )
	{
		rStm << *GetCacheName();
	}
}

/************************************************************************
|*	  SvBaseLink::SetObjType()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetObjType( USHORT nObjTypeP )
{
	DBG_ASSERT( nObjType != OBJECT_CLIENT_DDE, "type already set" )
	DBG_ASSERT( nObjType != OBJECT_CLIENT_OLE, "type already set" )
	DBG_ASSERT( nObjType != OBJECT_CLIENT_OLE_CACHE, "type already set" )
	DBG_ASSERT( !xObj.Is(), "object exist" )

	nObjType = nObjTypeP;
	if( nObjType == OBJECT_CLIENT_OLE )
	{
		Impl_LinkInfo * pLI = new Impl_LinkInfo;
		pImplData->ClientType.pLinkInfo = pLI;
		pLI->nUpdateMode = pImplData->ClientType.nUpdateMode;
		pLI->xMoniker = xLinkName;
	}
	else if( nObjType == OBJECT_CLIENT_OLE_CACHE )
	{
		Impl_LinkInfo * pLI = new Impl_LinkInfo;
		pImplData->ClientType.pLinkInfo = pLI;
	}
}

/************************************************************************
|*	  SvBaseLink::SetName()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetName( SvLinkName * pNm )
{
	DBG_ASSERT( nObjType != OBJECT_CLIENT_OLE_CACHE, "SetName not allowed" )

	xLinkName = pNm;
	if( nObjType == OBJECT_CLIENT_OLE )
		pImplData->ClientType.pLinkInfo->xMoniker = pNm;
}

/************************************************************************
|*	  SvBaseLink::GetName()
|*
|*	  Beschreibung
*************************************************************************/

SvLinkName * SvBaseLink::GetName() const
{
	if( nObjType == OBJECT_CLIENT_OLE )
		return pImplData->ClientType.pLinkInfo->xMoniker;
/*
	else if( nObjType == OBJECT_CLIENT_OLE_CACHE && xObj.Is() )
	{
		SvEmbeddedObjectRef aEO = &xObj;
		((SvBaseLink *)this)->xLinkName = aEO->GetName();
	}
*/
	return xLinkName;
}

/************************************************************************
|*	  SvBaseLink::GetCacheName()
|*
|*	  Beschreibung
*************************************************************************/

SvLinkName * SvBaseLink::GetCacheName() const
{
	if( nObjType == OBJECT_CLIENT_OLE_CACHE )
	{
		if( pImplData->xCacheObj.Is() )
		{
			DBG_ASSERT( FALSE, "not implemented" )
		}
		return pImplData->xCacheName;
	}
	return NULL;
}

/************************************************************************
|*	  SvBaseLink::SetObj()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetObj( SvPseudoObject * pObj )
{
	DBG_ASSERT( (nObjType & OBJECT_CLIENT_SO &&
				pImplData->ClientType.bIntrnlLnk) ||
				nObjType == OBJECT_CLIENT_GRF,
				"no intern link" )
	xObj = pObj;
}

/************************************************************************
|*	  SvBaseLink::SetCacheObj()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetCacheObj( SvEmbeddedObject * pObj )
{
	DBG_ASSERT( !xObj.Is(), "object already exist" )
	pImplData->xCacheObj = pObj;
}

/************************************************************************
|*	  SvBaseLink::GetCacheObj()
|*
|*	  Beschreibung
*************************************************************************/

SvEmbeddedObject * SvBaseLink::GetCacheObj() const
{
	return pImplData->xCacheObj;
}

/************************************************************************
|*	  SvBaseLink::SetLinkInfo()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetLinkInfo( Impl_LinkInfo & rInfo )
{
	DBG_ASSERT( nObjType == OBJECT_CLIENT_OLE_CACHE, "no OBJECT_CLIENT_OLE_CACHE" )
	DBG_ASSERT( pImplData->ClientType.pLinkInfo, "no link info" )
	*pImplData->ClientType.pLinkInfo = rInfo;
}

/************************************************************************
|*	  SvBaseLink::GetLinkInfo()
|*
|*	  Beschreibung
*************************************************************************/

Impl_LinkInfo * SvBaseLink::GetLinkInfo() const
{
	DBG_ASSERT( nObjType == OBJECT_CLIENT_OLE ||
				nObjType == OBJECT_CLIENT_OLE_CACHE,
				"no ole link" )
	return pImplData->ClientType.pLinkInfo;
}

/************************************************************************
|*	  SvBaseLink::SetLinkSourceName()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetLinkSourceName( SvLinkName * pLnkNm )
{
	SvLinkNameRef aLnkName( GetLinkSourceName() );
	if( !aLnkName.Is() && !pLnkNm )
		return;

	SvLinkNameRef aTmp( pLnkNm );
	if( aLnkName.Is() && aTmp.Is() && aLnkName->GetName() == pLnkNm->GetName() )
		return;

	if( nObjType == OBJECT_CLIENT_OLE_CACHE
		|| nObjType == OBJECT_CLIENT_OLE )
	{
		// Alte Verbindung weg
		Disconnect();
		// relativen Namen loeschen
		GetLinkInfo()->xRelMoniker = new SvLinkName();
		// Absoluten Namen setzen
		GetLinkInfo()->xMoniker = pLnkNm;
		// Neu verbinden
		_GetRealObject();
	}
	else
	{
		AddNextRef(); // sollte ueberfluessig sein
		// Alte Verbindung weg
		Disconnect();

		xLinkName = pLnkNm;

		// Neu verbinden
		_GetRealObject();
		ReleaseRef(); // sollte ueberfluessig sein
	}
}

/************************************************************************
|*	  SvBaseLink::GetLinkSourceName()
|*
|*	  Beschreibung
*************************************************************************/

const SvLinkName*  SvBaseLink::GetLinkSourceName() const
{
	SvLinkName * pLN = ((SvBaseLink *)this)->GetLinkSourceName();
	return pLN;
}


SvLinkName*  SvBaseLink::GetLinkSourceName()
{
	if( nObjType == OBJECT_CLIENT_OLE ||
		nObjType == OBJECT_CLIENT_OLE_CACHE )
	{
		xLinkName = GetLinkInfo()->xMoniker;
	}
	return xLinkName;
}

/************************************************************************
|*	  SvBaseLink::SetUpdateMode()
|*
|*	  Beschreibung
*************************************************************************/

void SvBaseLink::SetUpdateMode( USHORT nMode )
{
	if( OBJECT_CLIENT_OLE_CACHE == nObjType
		|| OBJECT_CLIENT_OLE == nObjType )
	{
		if( GetLinkInfo()->nUpdateMode != nMode )
		{
			Disconnect();
			GetLinkInfo()->nUpdateMode	= nMode;
			_GetRealObject();
		}
	}
	else if( ( OBJECT_CLIENT_SO & nObjType ) &&
		pImplData->ClientType.nUpdateMode != nMode )
	{
		AddNextRef();
		Disconnect();

		pImplData->ClientType.nUpdateMode = nMode;
		_GetRealObject();
		ReleaseRef();
	}
}


BOOL SvBaseLink::Update()
{
	if( nObjType == OBJECT_CLIENT_OLE_CACHE
		|| nObjType == OBJECT_CLIENT_OLE )
	{
		Disconnect();
		_GetRealObject();

		if( xObj.Is() )
		{
			SvData aData( GetContentType() );
			xObj->GetData( &aData );
			if( aData.HasData() )
			{
				DataChanged( aData );
				return TRUE;
			}
		}
	}
	else if( OBJECT_CLIENT_SO & nObjType )
	{
		AddNextRef();
		Disconnect();

		_GetRealObject();
		ReleaseRef();
		if( xObj.Is() )
		{
			SvData aData( pImplData->ClientType.nCntntType );
			xObj->GetData( &aData );
			if( aData.HasData() )
			{
				DataChanged( aData );
				//JP 13.07.00: Bug 76817 - for manual Updates there is no
				//				need to hold the ServerObject
				if( OBJECT_CLIENT_DDE == nObjType &&
					LINKUPDATE_ONCALL == GetUpdateMode() && xObj.Is() )
					xObj->RemoveAllDataAdvise( this );
				return TRUE;
			}
			if( xObj.Is() )
			{
				// sollten wir asynschron sein?
				if( ERRCODE_SO_PENDING == xObj->GetUpToDateStatus() )
					return TRUE;

				// dann brauchen wir das Object auch nicht mehr
				AddNextRef();
				Disconnect();
				ReleaseRef();
			}
		}
	}
	return FALSE;
}


USHORT SvBaseLink::GetUpdateMode() const
{
	if( nObjType == OBJECT_CLIENT_OLE_CACHE
		|| nObjType == OBJECT_CLIENT_OLE )
	{
		return GetLinkInfo()->nUpdateMode;
	}
	else
		return ( OBJECT_CLIENT_SO & nObjType )
				? pImplData->ClientType.nUpdateMode
				: LINKUPDATE_ONCALL;
}


void SvBaseLink::_GetRealObject( BOOL bConnect )
{
	if( !pLinkMgr )
		return;

	DBG_ASSERT( !xObj.Is(), "object already exist" )

	if( OBJECT_CLIENT_DDE == nObjType )
	{
		String sServer;
		if( pLinkMgr->GetDisplayNames( *this, &sServer ) &&
			sServer == GetpApp()->GetAppName() )		// interner Link !!!
		{
			// damit der Internal - Link erzeugt werden kann !!!
			nObjType = OBJECT_INTERN;
			xObj = pLinkMgr->CreateObj( *this );

			pImplData->ClientType.bIntrnlLnk = TRUE;
			nObjType = OBJECT_CLIENT_DDE;		// damit wir wissen was es mal war !!
		}
		else
		{
			pImplData->ClientType.bIntrnlLnk = FALSE;
			xObj = pLinkMgr->CreateObj( *this );
		}
	}
	else if( OBJECT_CLIENT_OLE_CACHE == nObjType )
	{
		if( pImplData->xCacheObj.Is() )
			xObj = pImplData->xCacheObj;
		else
		{
			xObj = pLinkMgr->CreateObj( *this );
			pImplData->xCacheObj = &xObj;
		}
	}
	else if( OBJECT_CLIENT_OLE == nObjType )
	{
		xObj = pLinkMgr->CreateObj( *this );
	}
	else if( (OBJECT_CLIENT_SO & nObjType) )
		xObj = pLinkMgr->CreateObj( *this );

	if( bConnect && ( !xObj.Is() || !xObj->Connect( *this ) ) )
		Disconnect();
}


ULONG SvBaseLink::GetContentType() const
{
	if( OBJECT_CLIENT_SO & nObjType )
		return pImplData->ClientType.nCntntType;

	return 0;		// alle Formate ?
}


BOOL SvBaseLink::SetContentType( ULONG nType )
{
	if( OBJECT_CLIENT_SO & nObjType )
	{
		pImplData->ClientType.nCntntType = nType;
		return TRUE;
	}
	return FALSE;
}


void SvBaseLink::Disconnect()
{
	if( xObj.Is() )
	{
		xObj->RemoveAllDataAdvise( this );
		xObj->RemoveConnectAdvise( this, ADVISE_CLOSED );
		xObj.Clear();
	}
}


void SvBaseLink::DataChanged( SvData& /*rData*/ )
{
	switch( nObjType )
	{
	case OBJECT_DDE_EXTERN:
		if( pImplData->DDEType.pItem )
			pImplData->DDEType.pItem->Notify();
		break;
	}
}


void SvBaseLink::ShowError( Window *pP ) const
{
	if( !pLinkMgr )
	{
		DBG_ASSERT( pLinkMgr, "ohne LinkManager keine FehlerMeldungen" );
		return ;
	}

	String sApp, sTopic, sItem, sError;
	pLinkMgr->GetDisplayNames( *this, &sApp, &sTopic, &sItem );
	switch( nObjType )
	{
	case OBJECT_CLIENT_DDE:
		{
			sError = String( ResId( STR_ERROR_DDE, SOAPP->GetResMgr()));
			USHORT nFndPos = sError.Search( '%' );
			if( STRING_NOTFOUND != nFndPos )
			{
				sError.Erase( nFndPos, 1 ).Insert( sApp, nFndPos );
				nFndPos += sApp.Len();
			}
			if( STRING_NOTFOUND != ( nFndPos = sError.Search( '%', nFndPos )))
			{
				sError.Erase( nFndPos, 1 ).Insert( sTopic, nFndPos );
				nFndPos += sTopic.Len();
			}
			if( STRING_NOTFOUND != ( nFndPos = sError.Search( '%', nFndPos )))
				sError.Erase( nFndPos, 1 ).Insert( sItem, nFndPos );
		}
		break;

	default:
		return ;
	}

	ErrorBox( pP, WB_OK, sError ).Execute();
}


BOOL SvBaseLink::Edit( Window* pParent )
{
	BOOL bConnect = xObj.Is();
	if( !bConnect )
		_GetRealObject( xObj.Is() );

	SvLinkName* pNewNm = 0;

	if( OBJECT_CLIENT_SO & nObjType &&
		pImplData->ClientType.bIntrnlLnk )
	{
		if( pLinkMgr )
		{
			SvPseudoObjectRef ref = pLinkMgr->CreateObj( *this );
			if( ref.Is() )
				pNewNm = ref->Edit( pParent, *this );
		}
	}
	else
		pNewNm = xObj->Edit( pParent, *this );

	if( pNewNm )
	{
		SetLinkSourceName( pNewNm );
		if( !Update() )
			ShowError( pParent );		// Fehler melden
	}
	else if( !bConnect )
		Disconnect();
	return 0 != pNewNm;
}


void SvBaseLink::Closed()
{
	if( xObj.Is() )
		// beim Advise Abmelden
		xObj->RemoveAllDataAdvise( this );
	SvAdviseSink::Closed();
}


USHORT SvBaseLink::GetPersistId()
{
	return 1;
}


ImplDdeItem::~ImplDdeItem()
{
	bIsInDTOR = TRUE;
	// damit im Disconnect nicht jemand auf die Idee kommt, den Pointer zu
	// loeschen!!
	SvBaseLinkRef aRef( pLink );
	aRef->Disconnect();
}


DdeData* ImplDdeItem::Get( ULONG nFormat )
{
	if( pLink->GetObject() )
	{
		// ist das noch gueltig?
		if( bIsValidData && nFormat == aData.GetFormat() )
			return &aData;

		aSvData = SvData( nFormat );
		if( pLink->GetObject()->GetData( &aSvData ) && aSvData.HasData() )
		{
			void * pGetData;
			long nLen = aSvData.GetMinMemorySize();
			aSvData.GetData( (void**)&pGetData, TRANSFER_REFERENCE );
			aData = DdeData( (const char *)pGetData, nLen, nFormat );
			bIsValidData = TRUE;
			return &aData;
		}
	}
	bIsValidData = FALSE;
	return 0;
}


BOOL ImplDdeItem::Put( const DdeData* pData )
{
	bIsValidData = FALSE;
	if( pLink->GetObject() )
	{
		aSvData = SvData( pData->GetFormat() );
        aSvData.SetData( (void*)(const void*)*pData,
						(long)*pData, TRANSFER_REFERENCE );
		return pLink->GetObject()->ChangeData( aSvData );
	}
	return FALSE;
}


void ImplDdeItem::AdviseLoop( BOOL bOpen )
{
	// Verbindung wird geschlossen, also Link abmelden
	if( pLink->GetObject() )
	{
		if( bOpen )
		{
			// es wird wieder eine Verbindung hergestellt
			if( OBJECT_DDE_EXTERN == pLink->GetObjectType() )
			{
				pLink->GetObject()->AddDataAdvise( pLink,
							SvDataType( FORMAT_STRING ), ADVISEMODE_NODATA );
				pLink->GetObject()->AddConnectAdvise( pLink, ADVISE_CLOSED );
			}
		}
		else
		{
			// damit im Disconnect nicht jemand auf die Idee kommt,
			// den Pointer zu loeschen!!
			SvBaseLinkRef aRef( pLink );
			aRef->Disconnect();
		}
	}
}


static DdeTopic* FindTopic( const SvLinkName* pNm, USHORT* pItemStt )
{
	if( !pNm )
		return 0;

	String sNm( pNm->GetName() );
	USHORT nTokenPos = 0;
	String sService( sNm.GetToken( 0, cTokenSeperator, nTokenPos ) );

	DdeServices& rSvc = DdeService::GetServices();
	for( DdeService* pService = rSvc.First(); pService;
												pService = rSvc.Next() )
		if( pService->GetName() == sService )
		{
			// dann suchen wir uns das Topic
			String sTopic( sNm.GetToken( 0, cTokenSeperator, nTokenPos ) );
			if( pItemStt )
				*pItemStt = nTokenPos+1;

			DdeTopics& rTopics = pService->GetTopics();

			for( int i = 0; i < 2; ++i )
			{
				for( DdeTopic* pTopic = rTopics.First(); pTopic;
												pTopic = rTopics.Next() )
					if( pTopic->GetName() == sTopic )
						return pTopic;

				// Topic nicht gefunden ?
				// dann versuchen wir ihn mal anzulegen
				if( i || !pService->MakeTopic( sTopic ) )
					break;	// hat nicht geklappt, also raus
			}
			break;
		}
	return 0;
}

/************************************************************************
|*	  Impl_LinkTime::Impl_LinkTime()
|*
|*	  Beschreibung
*************************************************************************/
Impl_LinkTime::Impl_LinkTime()
{
	static Time aDefTime;
	Date aDummyData;
	Impl_LinkTime aCurTime( aDummyData, aDefTime );
	nLowTime = aCurTime.nLowTime;
	nHighTime = aCurTime.nHighTime;
}

static long n1601Days =  1601L * 365 + 1601 / 4 - 1601 / 100 + 1601 / 400;
Impl_LinkTime::Impl_LinkTime( const Date & rDate, const Time & rTime )
{
	long nYears = rDate.GetYear();
	long nDays = nYears * 365 + nYears / 4 - nYears / 100 + nYears / 400
				+ rDate.GetDayOfYear() -1;
	nDays -= n1601Days;

	long n10mSec =	rTime.Get100Sec() +
					100L * rTime.GetSec() +
					100L * 60 * rTime.GetMin() +
					100L * 60 * 60 * rTime.GetHour();

	BigInt a100nPer10mSec = 100000L;
	BigInt a100nPerDay = BigInt( 10000000L ) * BigInt( 60L * 60 * 24 );

	BigInt aTime = a100nPerDay * BigInt( nDays )
					+ a100nPer10mSec * BigInt( n10mSec );
/*
	rThis.nLowTime = aTime.GetLong( 0 );
	rThis.nHighTime = aTime.GetLong( 1 );
*/
}

/************************************************************************
|*	  Impl_LinkTime::GetDate()
|*
|*	  Beschreibung
*************************************************************************/
/*
Date Impl_LinkTime::GetDate() const
{
}
*/

/************************************************************************
|*	  Impl_LinkTime::GetTime()
|*
|*	  Beschreibung
*************************************************************************/
/*
Time Impl_LinkTime::GetTime() const
{
}
*/

/************************************************************************
|*	  Impl_LinkTime::operator << >>
|*
|*	  Beschreibung: In MS Format
*************************************************************************/
SvStream &	operator << ( SvStream & rStm, const Impl_LinkTime & rThis )
{
	rStm << rThis.nLowTime;
	rStm << rThis.nHighTime;
	return rStm;
}

SvStream &	operator >> ( SvStream & rStm, Impl_LinkTime & rThis )
{
	rStm >> rThis.nLowTime;
	rStm >> rThis.nHighTime;
	return rStm;
}

/************************************************************************
|*	  Impl_LinkInfo::Impl_LinkInfo()
|*
|*	  Beschreibung
*************************************************************************/
Impl_LinkInfo::Impl_LinkInfo()
	: nUpdateMode( LINKUPDATE_ALWAYS )
	, xLinkMoniker( new SvLinkName() )
	, xRelMoniker( new SvLinkName() )
	, xMoniker( new SvLinkName() )
	, aChangeOfUpdate( 0, 0 )
	, aKnownUpToDate( 0, 0 )
	, aUpdate( 0, 0 )
{
}

/************************************************************************
|*	  Impl_LinkInfo::operator <<
|*
|*	  Beschreibung
*************************************************************************/
SvStream &	operator << ( SvStream & rStm, const Impl_LinkInfo & rThis )
{
	rStm << (UINT32)rThis.nUpdateMode;
	rStm << (UINT32)0;
	rStm << *rThis.xLinkMoniker;
	rStm << *rThis.xRelMoniker;
	rStm << *rThis.xMoniker;
	rStm << (INT32)-1;
	rStm << rThis.aLastClass;
	// entgegen der Doku scheint es ein 2Byte Wert zu sein
	UINT16 nLen = rThis.aLastDisplayStr.Len();
	if( nLen )
	{
		rStm << (UINT16)(nLen +1);
		ByteString aStr( rThis.aLastDisplayStr, gsl_getSystemTextEncoding() );
		rStm << aStr.GetBuffer();
		rStm << '\0';
	}
	else
		rStm << nLen;
	rStm << rThis.aChangeOfUpdate;
	rStm << rThis.aKnownUpToDate;
	rStm << rThis.aUpdate;
	return rStm;
}

/************************************************************************
|*	  Impl_LinkInfo::operator >>
|*
|*	  Beschreibung
*************************************************************************/
SvStream &	operator >> ( SvStream & rStm, Impl_LinkInfo & rThis )
{
	UINT32 n;
	rStm >> n;
	rThis.nUpdateMode = (USHORT)n;
	rStm.SeekRel( 4 );
	rStm >> *rThis.xLinkMoniker;
	rStm >> *rThis.xRelMoniker;
	rStm >> *rThis.xMoniker;
	rStm >> n;
	rStm >> rThis.aLastClass;
	// entgegen der Doku scheint es ein 2Byte Wert zu sein
	UINT16 nLen;
	rStm >> nLen;
	if( nLen )
	{
		DBG_ASSERT( nLen <= STRING_MAXLEN, "string buffer overflow" )
		sal_Char * pBuf = new sal_Char[ nLen ];
		rStm.Read( pBuf, nLen );
		rThis.aLastDisplayStr = String( pBuf, nLen -1, gsl_getSystemTextEncoding() );
		delete pBuf;
	}
	rStm >> rThis.aChangeOfUpdate;
	rStm >> rThis.aKnownUpToDate;
	rStm >> rThis.aUpdate;
	return rStm;
}



