/*************************************************************************
 *
 *  $RCSfile: dtrans.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:56:50 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SOT_DTRANS_HXX
#define _SOT_DTRANS_HXX

#ifndef _SOT_OBJECT_HXX
#include <object.hxx>
#endif
#ifndef _SOT_FACTORY_HXX
#include <factory.hxx>
#endif
#ifndef _TOOLS_OWNLIST_HXX
#include <tools/ownlist.hxx>
#endif

//class JobSetup;
class SvStream;
class AbstractDeviceData;
//========================================================================
// Storage Medium !!! Uebereinstimmung mit MS-Windows
#define MEDIUM_MEMORY   1
#define MEDIUM_FILE     2
#define MEDIUM_STREAM   4
#define MEDIUM_STORAGE  8
#define MEDIUM_GDI      16
#define MEDIUM_MFPICT   32
#define MEDIUM_ALL      (MEDIUM_MEMORY | MEDIUM_FILE | MEDIUM_STREAM |  \
                         MEDIUM_STORAGE | MEDIUM_GDI | MEDIUM_MFPICT)
// Datenart !!! Uebereinstimmung mit MS-Windows
#define ASPECT_CONTENT      1
#define ASPECT_THUMBNAIL    2 /* 120 * 120, 6 Farben DIB in MetaFile */
#define ASPECT_ICON         4
#define ASPECT_DOCPRINT     8
#define ASPECT_ALL      (ASPECT_CONTENT | ASPECT_THUMBNAIL | ASPECT_ICON | ASPECT_DOCPRINT)


class SvDataType
/*	[Beschreibung]

	Diese Klasse beschreibt einen Datentyp, der "uber die <SotDataObject>
	Klasse ausgetausch werden kann. Der Typ wird haubts"achlich durch
	Id nFormat beschrieben. Diese Id muss ein Standardformat sein oder
	mit <Exchange::RegisterFormatName> erzeugt werden.
	Der JobSetup gibt an, f"ur welches Device die Daten formatiert und wie
 	es eingestellt werden soll. In StarView k"onnen zur zeit nur f"ur,
 	Drucker Einstellungen vorgenommen werden.
	Der Aspect gibt an, on man das genaue Aussehen (CONTENT) eine
	Skizze (THUMBNAIL) oder eine repr"asentation als Icon (ICON)
	haben m"ochte. DOCPRINT ist z.Z. ohne Definition (OLE2).
	Das Medium gibt an, in welchen Speicherstrukturen die Daten
	"ubertragen werden k"onnen.
*/
{
friend class SvDataObject;
    ULONG				nFormat;
    AbstractDeviceData* pDevData;
    USHORT				nAspect;
    USHORT				nMedium;
protected:
	void			SetFormat( ULONG nF ) { nFormat = nF; }
    void            SetMedium( USHORT nMediumP ) { nMedium = nMediumP; }
    void            AddMedium( USHORT nMediumP ) { nMedium |= nMediumP; }
public:
    static String   GetAspectName( USHORT nAspectP );

                    SvDataType();
                    SvDataType( ULONG nFormat,
                                USHORT nPrefered = MEDIUM_ALL );
                    SvDataType( const SvDataType & rObj );
    SvDataType &    operator = ( const SvDataType & rObj );
                    ~SvDataType();

    friend SvStream& operator << (SvStream &, const SvDataType &);
    friend SvStream& operator >> (SvStream &, SvDataType &);

    void            SetDeviceData( const AbstractDeviceData * );
    void            SetAspect( USHORT nAspectP ) { nAspect = nAspectP; }

    BOOL            operator == ( const SvDataType & rObj ) const;
    BOOL            operator != ( const SvDataType & rObj ) const
                    { return !(*this == rObj); }
    BOOL            Match( const SvDataType & rObj ) const;

    ULONG           GetFormat() const { return nFormat; }
    USHORT          GetMedium() const { return nMedium; }
    const AbstractDeviceData * GetDeviceData() const { return pDevData; }
    USHORT          GetAspect() const { return nAspect; }

    BOOL            HasFormat() const { return nFormat != 0; }
    BOOL            Is() const { return nFormat != 0; }
};

//========================================================================
class SvDataTypeList
{
/*	[Beschreibung]

	In dieser Liste k"onnen Datentypen abgelegt werden. Mit den Methoden
	<SvDataTypeList::HasEqualType> und <SvDataTypeList::Get> kann in der
	Liste nach passenden Typen gesucht werden.

	[Anmerkung]

	HasEqualType sucht eine Typen der zu 100% passt. Die Methode
	Get nutzt das Matching von SvDataType::Match.
*/
    PRV_SV_DECL_OWNER_LIST(SvDataTypeList, SvDataType)
    BOOL				HasEqualType( const SvDataType & ) const;
    const SvDataType *  Get( const SvDataType & ) const;
    const SvDataType *  Get( ULONG nFormat ) const;
    void                Remove( const SvDataType & );
};

//========================================================================
#ifndef SOT_DECL_SOTSTORAGE_DEFINED
#define SOT_DECL_SOTSTORAGE_DEFINED
SO2_DECL_REF(SotStorage)
#endif
#ifndef SOT_DECL_SOTSTORAGESTREAM_DEFINED
#define SOT_DECL_SOTSTORAGESTREAM_DEFINED
SO2_DECL_REF(SotStorageStream)
#endif

enum TransferOperation { TRANSFER_MOVE, TRANSFER_COPY, TRANSFER_REFERENCE };

class SvDataCopyStream;
class SvStorageStreamRef;
class SvStorageRef;
class GDIMetaFile;
class Bitmap;

class SvData : public SvDataType
/*	[Beschreibung]

	Werden Daten ausgetauscht, dann werden sie immer in ein
	Objekt diesen Typs eingekapselt. Neben dem Speichern in den Formaten
	Memory, Metafile, Bitmap, SotStorage und SotStorageStream, kann die
	Klasse auch noch zwischen den verschiedenen Speichermedien
	konvertieren. Leider kann nicht jedes Speichermedium in eine
	beliebiges anderes umgewandelt werden. Die einzelnen Methoden
	dokumentieren, welche Umwandlungen geleistet werden.
	Um Daten nicht unn"otig zu kopieren, gibt es verschiedene
	<TransferOperation> Typen.
	TRANSFER_MOVE: 		Die Daten werden verschoben. Derjenige, der die
 						Daten bekommt muss sie freigeben.
	TRANSFER_COPY:		Der aufgerufenen bekommt eine Kopie der Daten.
	TRANSFER_REFERENCE:	Es wird nur eine Referenz auf die Daten f"ur den
						kurzfristigen Gebrauch gegeben.
*/
{
friend class SvDataObject;
    // keine union, wegen ctor
    UINT32              nSize;    // nur gesetzt, wenn MEDIUM_MEMORY | MEDIUM_STREAM
    void *              pData;
	SvDataCopyStream *	pSimpleObj;
    SotStorageRef       aStorage;
	UINT32				nStmBeginPos;
    SotStorageStreamRef aStream;
    BOOL                bDataOwner:1,
                        bSimpleObjOwner:1,
                        bStorageOwner:1,
                        bStreamOwner:1;

    void    FreeMemory();
    void    FreeData();
    void    SetMedium( USHORT nMediumP )
            { SvDataType::SetMedium( nMediumP ); }
public:
            SvData( const SvData & );
            SvData( ULONG nFormat = 0, USHORT nPrefered = MEDIUM_ALL );
            SvData( const SvDataType & );
            ~SvData();
    SvData& operator = ( const SvData & );

    friend SvStream& operator << (SvStream &, SvData &);
    friend SvStream& operator >> (SvStream &, SvData &);

    BOOL    HasMedium( USHORT nMedium ) const;
    BOOL    HasData() const;

    void    SetData( SvData * pData, TransferOperation = TRANSFER_COPY );
    void    SetData( void * pData, UINT32 nSize, TransferOperation = TRANSFER_COPY );
    void    SetData( const String & rStr );
    void    SetData( SvDataCopyStream * pObj, TransferOperation = TRANSFER_COPY );
    void    SetData( SotStorageRef &, TransferOperation = TRANSFER_REFERENCE );
    void    SetData( SotStorageStreamRef &, TransferOperation = TRANSFER_REFERENCE );
			// in rLen steht die Laenge der Daten. Der Seek-Pointer steht davor.
    void    SetData( SotStorageStreamRef &, UINT32 nLen, TransferOperation = TRANSFER_REFERENCE );
	void	NewStreamSize();

    UINT32  GetMemorySize() const;
    UINT32  GetMinMemorySize() const;
    BOOL    GetData( void ** ppData, TransferOperation = TRANSFER_COPY );
    BOOL    GetData( String & rStr );
    BOOL    GetData( SvDataCopyStream ** ppObj, TypeId pCreateFunc,
					TransferOperation = TRANSFER_COPY );
    BOOL    GetData( SotStorageRef &, TransferOperation = TRANSFER_REFERENCE );
    BOOL    GetData( SotStorageStreamRef &, TransferOperation = TRANSFER_REFERENCE );
			// in rLen steht die Laenge der Daten, die geschrieben wurden.
			// der Seek-Pointer steht dahinter
    BOOL    GetData( SotStorageStreamRef &, UINT32 & rLen, TransferOperation = TRANSFER_REFERENCE );

	/* werden in so3 implementiert */
    BOOL    GetData( SvStorageRef &, TransferOperation = TRANSFER_REFERENCE );
    BOOL    GetData( SvStorageStreamRef &, TransferOperation = TRANSFER_REFERENCE );
    BOOL    GetData( SvStorageStreamRef &, UINT32 & rLen, TransferOperation = TRANSFER_REFERENCE );
    BOOL    GetData( GDIMetaFile ** ppFile, TransferOperation = TRANSFER_COPY );
    BOOL    GetData( Bitmap ** ppBmp, TransferOperation = TRANSFER_COPY );

    void    SetData( GDIMetaFile * pFile, TransferOperation = TRANSFER_COPY );
    void    SetData( Bitmap * pBmp, TransferOperation = TRANSFER_COPY );
    void    SetData( SvStorageRef &, TransferOperation = TRANSFER_REFERENCE );
    void    SetData( SvStorageStreamRef &, TransferOperation = TRANSFER_REFERENCE );
    void    SetData( SvStorageStreamRef &, UINT32 nLen, TransferOperation = TRANSFER_REFERENCE );
};
DECLARE_LIST( SvDataList, SvData * )

//========================================================================
#ifndef SOT_DECL_SOTDATAOBJECT_DEFINED
#define SOT_DECL_SOTDATAOBJECT_DEFINED
SO2_DECL_REF(SotDataObject)
#endif

class SotDataObject : virtual public SotObject
/*	[Beschreibung]

	Diese Klasse beschreibt, wie Daten von einem Objekt abgeholt oder
	in ein Objekt eingesetzt werden k"onnen. Das wichtigste
 	Designkriterium ist, dass der eigentliche Austausch der Daten vom
	"wie komme ich an die Daten (z.B. D&D, Clipboard)" getrennt ist.

	[Anmerkung]

	Dioe Schnittstellen f"ur D&D und das Clipboard sind nur vorl"aufig.
*/
{
protected:
					~SotDataObject();
public:
					SotDataObject();
                    SO2_DECL_BASIC_CLASS_DLL(SotDataObject,SOTDATA())
					SO2_DECL_INVARIANT()

    virtual const SvDataTypeList & GetTypeList() const;

    virtual BOOL    GetData( SvData * );
    virtual BOOL    GetDataHere( SvData * );
    virtual BOOL    QueryDataType( SvDataType & rInOutType );

	// HACK fuer die Implementation der alten Clipboard-Schnittstelle
	// in VCL, primitives RTTI nur fuer SotData(Member)Object
    virtual BOOL    IsSotDataMemberObject( void );
};
SO2_IMPL_REF(SotDataObject)

/*************** SvDataMemberObject **************************************
*************************************************************************/
class SotDataMemberObject : public SotDataObject
/*	[Beschreibung]

	Diese Klasse ist f"ur einfache Applikationen, die kein
	"Delayed Rendering" f"ur den Datenaustausch nutzen wollen.
	Noch ein Punkt ist das hinzuf"ugen weiterer Formate, unabh"angig
	vom Objekt, welches die Inhaltsdaten liefert.
	Z.B. ein Ole-Container f"ugt noch die Daten f"ur das Linkformat
	auf ein eingebettetes Objekt hinzu.

	[Anmerkung]
*/
{
    SvDataList         aDataList;
    SvDataTypeList      aTypeList;
	SotDataObjectRef	xExtObj;
public:
                    SotDataMemberObject();
                    SotDataMemberObject( SotDataObject * pExtObj );
                    ~SotDataMemberObject();
                    SO2_DECL_BASIC_CLASS_DLL(SotDataMemberObject,SOTDATA())

    virtual const SvDataTypeList & GetTypeList() const;
    virtual BOOL    GetData( SvData * );
    virtual BOOL    GetDataHere( SvData * );

    SvDataList &   GetDataList() { return aDataList; }
    void            Append( SvData * );

	// HACK fuer die Implementation der alten Clipboard-Schnittstelle
	// in VCL, primitives RTTI nur fuer SotData(Member)Object
    virtual BOOL    IsSotDataMemberObject( void );
};

#ifndef SOT_DECL_SOTDATAMEMBEROBJECT_DEFINED
#define SOT_DECL_SOTDATAMEMBEROBJECT_DEFINED
SO2_DECL_REF(SotDataMemberObject)
#endif
SO2_IMPL_REF(SotDataMemberObject)

#endif // _DTRANS_HXX
