/*************************************************************************
 *
 *  $RCSfile: dtrans.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:56:51 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef W31
#include <tools/svwin.h>
#endif

#include <dtrans.hxx>
#include <tools/debug.hxx>
#include <tools/new.hxx>
#include <tools/cachestr.hxx>
#include <storage.hxx>
#include <exchange.hxx>
#include <absdev.hxx>
#pragma hdrstop

/**************************************************************************/
#ifdef DBG_UTIL
void TestZeroString( void * /*pData*/, UINT32 /*nS*/ )
{
	// Geht nicht wegen SotDataObject::GetDataHere
	return;
}

void TestZeroString( SvStream & /*rStm*/ )
{
	// Geht nicht wegen SotDataObject::GetDataHere
	return;
}
#endif

/********************** SvDataType ****************************************
**************************************************************************/
/*************************************************************************
|*    SvDataType::GetAspectName()
|*
|*    Beschreibung
*************************************************************************/
String SvDataType::GetAspectName( USHORT nAspect )
{
    String aAsp;
    if( nAspect == ASPECT_ALL )
        aAsp.AppendAscii( "ALL" );
    else if( nAspect & ASPECT_CONTENT )
        aAsp.AppendAscii( "CONTENT" );
    else if( nAspect & ASPECT_THUMBNAIL )
        aAsp.AppendAscii( "THUMBNAIL" );
    else if( nAspect & ASPECT_ICON )
        aAsp.AppendAscii( "ICON" );
    else if( nAspect & ASPECT_DOCPRINT )
        aAsp.AppendAscii( "DOCPRINT" );
    return aAsp;
}

/*************************************************************************
|*    SvDataType::SvDataType()
|*
|*    Beschreibung
*************************************************************************/
SvDataType::SvDataType() 
	: nFormat( 0 )
    , nMedium( MEDIUM_ALL )
    , nAspect( 0 )
    , pDevData( NULL )
{
}

/*************************************************************************
|*    SvDataType::SvDataType()
|*
|*    Beschreibung
*************************************************************************/
SvDataType::SvDataType( ULONG nFormatP, USHORT nMediumP ) 
	: nFormat( nFormatP )
    , nMedium( nMediumP )
    , nAspect( ASPECT_CONTENT )
    , pDevData( NULL )
{
	if( nFormat == FORMAT_GDIMETAFILE )
	{
#ifdef OLE2
		nMedium = MEDIUM_MFPICT | MEDIUM_MEMORY;
#else
		nMedium &= ~(MEDIUM_STORAGE | MEDIUM_GDI | MEDIUM_MFPICT);
#endif
	}
	else if( nFormat == FORMAT_BITMAP )
	{
#ifdef OLE2
		nMedium = MEDIUM_GDI | MEDIUM_MEMORY;
#else
		nMedium &= ~(MEDIUM_STORAGE | MEDIUM_GDI | MEDIUM_MFPICT);
#endif
	}
	else
	{
		nMedium &= ~(MEDIUM_GDI | MEDIUM_MFPICT);
#ifdef WIN
		if( nMedium & MEDIUM_STREAM )
			nMedium |= MEDIUM_MEMORY;
#endif
	}
}

/*************************************************************************
|*    SvDataType::SvDataType()
|*
|*    Beschreibung
*************************************************************************/
SvDataType::SvDataType( const SvDataType & rObj ) :
    nFormat( rObj.nFormat ),
    nMedium( rObj.nMedium ),
    nAspect( rObj.nAspect )
{
    if( rObj.pDevData )
        pDevData  = rObj.pDevData->Copy();
    else
        pDevData = NULL;
}

/*************************************************************************
|*    SvDataType::operator = ()
|*
|*    Beschreibung
*************************************************************************/
SvDataType & SvDataType::operator = ( const SvDataType & rObj )
{
    if( this != &rObj )
    {
        nFormat         = rObj.nFormat;
        nMedium         = rObj.nMedium;
        delete pDevData;
        if( rObj.pDevData )
            pDevData  = rObj.pDevData->Copy();
        else
            pDevData = NULL;
        nAspect         = rObj.nAspect;
    }
    return *this;
}

/*************************************************************************
|*    SvDataType::~SvDataType()
|*
|*    Beschreibung
*************************************************************************/
SvDataType::~SvDataType()
{
    delete pDevData;
}

/*************************************************************************
|*    SvDataType::operator <<
|*    SvDataType::operator >>
|*
|*    Beschreibung
*************************************************************************/
SvStream& operator << (SvStream & rStm, const SvDataType & rType )
{
    rStm.WriteByteString( SotExchange::GetFormatName( rType.nFormat ), RTL_TEXTENCODING_ASCII_US );
//    rStm << rType.GetDeviceData();
    rStm << rType.nAspect;
    rStm << rType.nMedium;
    return rStm;
}

SvStream& operator >> (SvStream & rStm, SvDataType & rType )
{
    String aFmtName;
    rStm.ReadByteString( aFmtName, RTL_TEXTENCODING_ASCII_US );
    rType.nFormat = SotExchange::RegisterFormatName( aFmtName );
//    if( !rType.pDevData )
//        rType.pDevData = new JobSetup();
//    rStm >> *rType.pDevData;
    rStm >> rType.nAspect;
    rStm >> rType.nMedium;
    return rStm;
}

/*************************************************************************
|*    SvDataType:SetDeviceData()
|*
|*    Beschreibung
*************************************************************************/
void SvDataType::SetDeviceData( const AbstractDeviceData * pDevDataP )
{
	delete pDevData;
	if( pDevDataP )
		pDevData  = pDevDataP->Copy();
	else
		pDevData = NULL;
}

/*************************************************************************
|*    SvDataType::operator == ()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvDataType::operator == ( const SvDataType & rObj ) const
{
    if( nFormat  == rObj.nFormat
      && nMedium == rObj.nMedium
      && nAspect == rObj.nAspect )
    {
        if( pDevData && rObj.pDevData )
            return pDevData->Equals( *rObj.pDevData );
        else
            // TRUE, wenn beide NULL
            return pDevData == rObj.pDevData;
		return TRUE;
    }
    return FALSE;
}

/*************************************************************************
|*    SvDataType:Match()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvDataType::Match( const SvDataType & rObj ) const
{
	if( rObj.nFormat == 0 )
		// 0 Matched alles
		return TRUE;

    if( (nAspect & rObj.nAspect) && nFormat == rObj.nFormat
	  && (nMedium & rObj.nMedium) )
    {
        // Das Medium darf keine Rolle spielen, da es konvertierbar ist !!!
		// Leider spielt das Medium eine Rolle, Ole2-Spec QueryGetData
        if( pDevData && rObj.pDevData )
            return pDevData->Equals( *rObj.pDevData );
        else if( !pDevData && !rObj.pDevData )
			return TRUE;
    }
    return FALSE;
}

/********************** SvData ********************************************
**************************************************************************/
/*************************************************************************
|*    SvData::SvData()
|*
|*    Beschreibung
|*    Letzte Aenderung  MM 14.06.94
*************************************************************************/
#define SVDATA_INIT()                                               \
    , pData           ( NULL   )                                    \
    , nSize           ( 0      )                                    \
    , pSimpleObj      ( NULL   )                                    \
    , bDataOwner      ( FALSE  )                                    \
    , bSimpleObjOwner ( FALSE  )                                    \
	, bStorageOwner   ( FALSE  )                                    \
    , bStreamOwner    ( FALSE  )                                    

SvData::SvData( const SvData & rData )
    : SvDataType( rData )
    SVDATA_INIT()
{
    SetData( (SvData *)&rData );
}
SvData::SvData( ULONG nFormatP, USHORT nPref )
    : SvDataType( nFormatP, nPref )
    SVDATA_INIT()
{
}
SvData::SvData( const SvDataType & rObj )
    : SvDataType( rObj )
    SVDATA_INIT()
{
}
SvData & SvData::operator = ( const SvData & rData )
{
	if( this != &rData )
	{
	    SvDataType::operator = ( rData );
	    SetData( (SvData *)&rData );
	}
    return *this;
}

/*************************************************************************
|*    SvData::~SvData()
|*
|*    Beschreibung
*************************************************************************/
SvData::~SvData()
{
    FreeData();
}

/*************************************************************************
|*    SvData::operator <<
|*    SvData::operator >>
|*
|*    Beschreibung
*************************************************************************/
SvStream& operator << (SvStream & rStm, SvData & r )
{
    rStm << (SvDataType &)r;
    void * pData;
    if( r.GetData( &pData, TRANSFER_REFERENCE ) )
    {
        rStm << (ULONG)r.GetMinMemorySize();
        rStm.Write( pData, r.GetMinMemorySize() );
    }
    else
        rStm.SetError( SVSTREAM_OUTOFMEMORY );
    return rStm;
}

SvStream& operator >> (SvStream & rStm, SvData & r )
{
    r.FreeData();
    rStm >> (SvDataType &)r;
    ULONG nSize;
    rStm >> nSize;
    if( nSize )
    {
        void * pData = SvMemAlloc( nSize );
        if( pData )
        {
            rStm.Read( pData, nSize );
            r.SetData( pData, nSize, TRANSFER_MOVE );
        }
        else
            rStm.SetError( SVSTREAM_OUTOFMEMORY );
    }
    return rStm;
}

/*************************************************************************
|*    SvData::FreeMemory()
|*    SvData::FreeData()
|*
|*    Beschreibung
*************************************************************************/
void SvData::FreeMemory()
{
    if( bDataOwner )
    {
        delete pData;
    }
    pData           = NULL;
	if( !aStream.Is() )
    	nSize           = 0;
}

void SvData::FreeData()
{
    FreeMemory();
    if( bSimpleObjOwner )
	{
        delete pSimpleObj;
	    bSimpleObjOwner  = FALSE;
	}
	pSimpleObj = NULL;
    aStream.Clear();
    aStorage.Clear();
}

/*************************************************************************
|*    SvData::HasMedium()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvData::HasMedium( USHORT nMedium ) const
{
    USHORT nMed = 0;
    if( pData )
        nMed |= MEDIUM_MEMORY;
//  if(  )
//      nMed |= MEDIUM_FILE;
    if( aStream.Is() )
        nMed |= MEDIUM_STREAM;
    if( aStorage.Is() )
        nMed |= MEDIUM_STORAGE;
    return nMedium & nMed ? TRUE : FALSE;
}

/*************************************************************************
|*    SvData::HasData()
|*
|*    Beschreibung
*************************************************************************/
BOOL SvData::HasData() const
{
	return pData != NULL
           || pSimpleObj != NULL || aStorage.Is() || aStream.Is();
}

/*************************************************************************
|*    SvData::SetData()
|*
|*    Beschreibung
*************************************************************************/
void SvData::SetData( SvData * pDataP, TransferOperation nMode )
{
	DBG_ASSERT( this != pDataP, "SetData with this == first parameter not implemented" )

    FreeData();
    if( pDataP->pData )
        SetData( pDataP->pData, pDataP->nSize, nMode );
    if( pDataP->pSimpleObj )
        SetData( pDataP->pSimpleObj, nMode );
    if( pDataP->aStream.Is() )
        SetData( pDataP->aStream, pDataP->nSize, nMode );
    if( pDataP->aStorage.Is() )
        SetData( pDataP->aStorage, nMode );
}

/************************************************************************/
void SvData::SetData( void * pDataP, UINT32 nSizeP, TransferOperation nMode )
{
#ifdef DBG_UTIL
	if( FORMAT_STRING == GetFormat() )
		TestZeroString( pDataP, nSizeP );
#endif

    FreeMemory();
	DBG_ASSERT( !aStream.Is() || nSize == nSizeP, "Stream-Size != MemorySize" )
    nSize       = nSizeP;
    if( TRANSFER_COPY == nMode )
    {
        bDataOwner  = TRUE;
        pData       = SvMemAlloc( nSize );
#ifdef W31
        hmemcpy( pData, pDataP, nSize );
#else
        memcpy( pData , pDataP, nSize );
#endif
    }
    else if( TRANSFER_MOVE == nMode )
    {
        bDataOwner  = TRUE;
        pData = pDataP;
    }
    else
    {
        bDataOwner  = FALSE;
        pData = pDataP;
    }
}

/************************************************************************/
void SvData::SetData( const String & rStr )
{
#ifdef dfngdfgndfgdfngnld
    SotStorageStreamRef aStm = new SotStorageStream( String() );
    *aStm << rStr.GetStr();// nur Tests
	aStm << '\0';
	aStm.Seek( 0 );
    SetData( aStm );
#endif
	ByteString aStr( rStr, gsl_getSystemTextEncoding() );
    // unbedingt mit 0-Byte am Ende
    SetData( (void *)aStr.GetBuffer(), aStr.Len() +1 );
}

//=========================================================================
void SvData::SetData
(
	SvDataCopyStream * pObj,	/* Das Objekt, welches "ubertragen wird. */
	TransferOperation nMode /* Bestimmt den Eigent"umer des Objektes. */
)
/*	[Beschreibung]

	Diese Methode dient zur Steigerung der Performance. Man stellt
	persistente Objekte zum Datenaustausch bereit. Wird genau dieses
	Objekt beim Empf"anger verlangt, dann wird es entweder mit Hilfe
 	der Methode <SvDataCopyStream::Assign> kopiert (*_COPY) oder
	verschoben (*_MOVE) oder nur eine Referenz (*_REFERENCE)
	herausgegeben.
	Findet die Daten"ubertragung nicht innerhalb eines Prozesses statt
	oder wird auf der anderen Seite nicht derselbe Typ verlangt, dann
	k"onnen die Daten mit <SvDataCopyStream::Save> in einen Stream oder
	Speicherbereich gewandelt werden.
*/
{
    if( bSimpleObjOwner )
        delete pSimpleObj;
    if( TRANSFER_COPY == nMode )
    {
		// Objekt erzeugen
        pSimpleObj       = (SvDataCopyStream *)pObj->Type()();
		DBG_ASSERT( pSimpleObj, "subclass of SvDataCopyStream has no Factory (TYPEINIT_FACTORY)" )
		if( pSimpleObj )
		{
			bSimpleObjOwner  = TRUE;
			// Objekt zuweisen
			pSimpleObj->Assign( *pObj );
		}
    }
    else if( TRANSFER_MOVE == nMode )
    {
        bSimpleObjOwner  = TRUE;
        pSimpleObj       = pObj;
    }
    else
    {
        bSimpleObjOwner  = FALSE;
        pSimpleObj       = pObj;
    }
}

//=========================================================================
void SvData::SetData
(
	SotStorageRef & rDataP,
	TransferOperation nMode
)
{
    if( TRANSFER_COPY == nMode )
    {
        aStorage = new SotStorage( String() );
        rDataP->CopyTo( aStorage );
    }
    else
        aStorage = rDataP;
	bStorageOwner = nMode != TRANSFER_REFERENCE;
}
//=========================================================================
void SvData::NewStreamSize()
{
	// Stream Daten merken
	DBG_ASSERT( aStream.Is(), "no stream" )
	SotStorageStreamRef xStm = aStream;
	UINT32 nPos = nStmBeginPos;
	DBG_ASSERT( aStream->Tell() >= nStmBeginPos, "wrong stream position" )
	UINT32 nLen = aStream->Tell() - nStmBeginPos;
	BOOL bOwner = bStreamOwner;
	// alle Daten freigeben
	FreeData();
	xStm->Seek( nPos );
	if( bOwner )
		SetData( xStm, nLen, TRANSFER_MOVE );
	else
		SetData( xStm, nLen, TRANSFER_REFERENCE );
}

//=======================================================================
void SvData::SetData
(
	SotStorageStreamRef & rDataP, /* Der Stream, der die Daten beinhaltet. */
	UINT32 nLen,				 /* Die L"ange der Daten, die "ubertragen
							   		werden sollen. */
	TransferOperation nMode 	 /* Bestimmt den Eigent"umer des Objektes. */
)
{
	nStmBeginPos = rDataP->Tell();
	DBG_ASSERT( !pData || nSize == nLen, "Stream-Size != MemorySize" )
	nSize = nLen;
    if( TRANSFER_COPY == nMode )
    {
        aStream = new SotStorageStream( String() );
        rDataP->CopyTo( aStream );
    }
    else
        aStream = rDataP;
	bStreamOwner = nMode != TRANSFER_REFERENCE;
#ifdef DBG_UTIL
	if( FORMAT_STRING == GetFormat() )
		TestZeroString( *rDataP );
#endif
}

/************************************************************************/
void SvData::SetData( SotStorageStreamRef & rDataP, TransferOperation nMode )
{
	DBG_ASSERT( rDataP->Tell() == 0, "Seek-Pointer in SetData not 0 (vorlauefig bis Umstellung)" )
	UINT32 nPos = rDataP->Tell();
	UINT32 nLen = rDataP->Seek( STREAM_SEEK_TO_END ) - nPos;
	rDataP->Seek( nPos );
	SetData( rDataP, nLen, nMode );
}

/*************************************************************************
|*    SvData::GetSize()
|*
|*    Beschreibung
*************************************************************************/
UINT32 SvData::GetMemorySize() const
{
	return GetMinMemorySize();
}
UINT32 SvData::GetMinMemorySize() const
{
    if( pData || aStream.Is() )
        return nSize;
    if( !aStream.Is() )
	{
		// Groee der Daten durch schreiben in einen Stream ermitteln
		SotStorageStreamRef xStm;
		if( ((SvData *)this)->GetData( xStm, TRANSFER_REFERENCE ) )
			return nSize;
	}
#ifdef DBG_UTIL
    else
        DBG_ERROR( "SvData::GetMinMemorySize: not impl" );
#endif
    return 0;
}


//=========================================================================
BOOL SvData::GetData
(
	void ** pDataP,			/* An diesem Parameter werden die Daten
							   "ubergeben. */
	TransferOperation nMode /* Bestimmt den Eigent"umer des Objektes. */
)
/*	[Beschreibung]

	Mit dieser Methode werden die Daten, die im Objekt stehen, als
	Speicherbereich abgeholt. Stehen sie nicht als Speicherbereich
	zur Ver"ugung, dann werden sie erzeugt.

	[R"uckgabewert]

	BOOL		TRUE, die Daten konnten als Speicherbereich zur Verf"ugung
				gestellt werden.
				FALSE, die Daten konnten nicht als Speicherbereich zur
 				Verf"ugung gestellt werden. Es waren entweder keine Daten
				vorhanden oder sie konnten nicht konvertiert werden.

	[Querverweise]

	<SvData::GetData>, <SotDataObject::GetData>
*/
{
    BOOL bRet = FALSE;
    if( !pData )
    { // noch nicht als Speicher
        SvMemoryStream * pStm = NULL;
		if( pSimpleObj )
        {
            pStm = new SvMemoryStream;
            pSimpleObj->Save( *pStm );
        }
        else if( aStream.Is() )
        {
			aStream->Seek( nStmBeginPos );
            // Konvertieren
            UINT32 nS = GetMinMemorySize();
            void * p = SvMemAlloc( nS );
            if( p )
            {
                aStream->Read( p, nS );
                SetData( p, nS, TRANSFER_MOVE );
            }
        }
        else if( aStorage.Is() )
            pStm = aStorage->CreateMemoryStream();

		if( pStm )
		{
			if( pStm->GetError() == SVSTREAM_OK )
			{
		        ULONG nS = pStm->Seek( STREAM_SEEK_TO_END );
		        SetData( pStm->SwitchBuffer(), nS, TRANSFER_MOVE );
			}
			delete pStm;
		}
    }

    if( pData )
    {
        if( TRANSFER_COPY == nMode )
#ifdef W31
            hmemcpy( *pDataP, pData, GetMinMemorySize() );
#else
            memcpy( *pDataP, pData, (size_t)GetMinMemorySize() );
#endif
        else if( TRANSFER_MOVE == nMode )
        { // Pointer rausgeben
            if( bDataOwner )
            {
                // Eigentuemer und nicht mit GlobalAlloc()
                *pDataP     = pData;
                pData       = NULL;
				if( !aStream.Is() )
                	nSize       = 0;
                bDataOwner  = FALSE;
            }
            else
            {
                // muss kopiert werden, da nicht Eigentuemer
                *pDataP = SvMemAlloc( nSize );
                GetData( pDataP, TRANSFER_COPY );
                FreeMemory();
            }
        }
        else
        {
            *pDataP = pData;
        }
        bRet = TRUE;
    }
    return bRet;
}

//=========================================================================
BOOL SvData::GetData
(
	String & rStr	/* An diesem Parameter werden die Daten "ubergeben. */
)
/*	[Beschreibung]

	Diese Methode holt die Daten im Objekt als <String>. Dies
	hat, ausser dass das Format meistens <FORMAT_STRING> ist, nichts
	mit diesem zu tun.

	[R"uckgabewert]

	BOOL		TRUE, die Daten konnten als String zur Verf"ugung
				gestellt werden.
				FALSE, die Daten konnten nicht als String zur
 				Verf"ugung gestellt werden. Es waren entweder keine Daten
				vorhanden oder sie konnten nicht konvertiert werden.

	[Querverweise]

	<SvData::GetData>, <SotDataObject::GetData>
*/
{
	void * pData;
	if( GetData( &pData, TRANSFER_REFERENCE ) )
	{
		ULONG nMemSize = GetMinMemorySize();
		DBG_ASSERT( nMemSize <= STRING_MAXLEN, "Die Daten sind zu gross" )
		if( nMemSize > STRING_MAXLEN )
			return FALSE;

		sal_Char * pBegin = (sal_Char *)pData;
		sal_Char * pEnd   = (sal_Char *)pData + nMemSize;
		// vorsicht bei Pointern unter WIN, deshalb auf ULONG
		while( (ULONG)pBegin < (ULONG)pEnd && *pBegin != '\0' )
			pBegin++;
		DBG_ASSERT( (ULONG)pBegin < (ULONG)pEnd, "non zero-terminatet String" )
		if( (ULONG)pBegin < (ULONG)pEnd )
		{
			// pBegin - pData kann nicht groesser als STRING_MAXLEN sein
			rStr = String( (sal_Char*)pData,
						USHORT((ULONG)pBegin - (ULONG)(char *)pData),
						gsl_getSystemTextEncoding() );
	    	return TRUE;
		}
	}
    return FALSE;
}

//=========================================================================
BOOL SvData::GetData
(
	SvDataCopyStream ** ppObj,/* An diesem Parameter werden die Daten
							   "ubergeben. */
	TypeId pTypeId,         /* Mit diesem Parameter wird bestimmt, ob
							   es sich um den gleichen Typ handelt.
							   Falls nicht, wird "uber die Funktion
							   das Objekt erzeugt. */
	TransferOperation nMode /* Bestimmt den Eigent"umer des Objektes. */
)
/*	[Beschreibung]

	Diese Methode holt die Daten im Objekt als Objekt vom Typ
	pTypeId. Es muss sich um den gleichen Typ (im c++ Sinne )handeln.

	[R"uckgabewert]

	BOOL		TRUE, die Daten konnten als Objekt zur Verf"ugung
				gestellt werden.
				FALSE, die Daten konnten nicht als Objekt zur
 				Verf"ugung gestellt werden. Es waren entweder keine Daten
				vorhanden oder sie konnten nicht konvertiert werden.

	[Beispiel]

	class A : public SvDataCopyStream
	class B : public class A

	void TestObj( B * pObj )
 	{
		SvData aData( FORMAT_... );
		aData.SetData( pObj, TRANSFER_MOVE );

		B * pB;
		A * pA;

		aData.GetData( (SotDataCopyStream **)&pB, B::StaticType(),
 						TRANSFER_REFERENCE );
		// pB == pObj ist TRUE
		aData.GetData( (SotDataCopyStream **)&pA, A::StaticType(),
						TRANSFER_REFERENCE );
		// pA == pObj ist TRUE
	}

	[Querverweise]

	<SvData::GetData>, <SvDataObject::GetData>
*/
{
    if( !pSimpleObj )
    {
		SotStorageStreamRef xStm;
		UINT32 nLen;
		GetData( xStm, nLen );
		if( xStm.Is() )
		{
			pSimpleObj = (SvDataCopyStream*)pTypeId();
			DBG_ASSERT( pSimpleObj, "subclass of SvDataCopyStream has no Factory (TYPEINIT_FACTORY)" )
			if( pSimpleObj )
			{
				xStm->SeekRel( -(INT32)nLen );
				pSimpleObj->Load( *xStm );
				if( xStm->GetError() != SVSTREAM_OK )
					DELETEZ( pSimpleObj );
			}
		}
    }
    DBG_ASSERT( !pSimpleObj || pSimpleObj->IsA( pTypeId ),
				"Get simple Objects with different types not impl" )
    if( pSimpleObj && pSimpleObj->IsA( pTypeId ) )
	{
	    if( TRANSFER_COPY == nMode )
	        (*ppObj)->Assign( *pSimpleObj );
	    else if( TRANSFER_MOVE == nMode )
	    {
	        if( bSimpleObjOwner )
	            *ppObj = pSimpleObj;
	        else
			{
				*ppObj = (SvDataCopyStream*)pTypeId();
				DBG_ASSERT( *ppObj, "subclass of SvDataCopyStream has no Factory (TYPEINIT_FACTORY)" )
				if( !*ppObj )
					return FALSE;
		        (*ppObj)->Assign( *pSimpleObj );
			}
	        bSimpleObjOwner = FALSE;
	        pSimpleObj = NULL;
	    }
	    else
	        *ppObj = pSimpleObj;
	    return TRUE;
	}
    return FALSE;
}

//=========================================================================
BOOL SvData::GetData
(
	SotStorageRef & rDataP,
	TransferOperation nMode
)
{
    BOOL bRet = FALSE;

    if( !aStorage.Is() )
    {
		SvStream * pStm = NULL;
        if( pData )
        {
			pStm = new SvCacheStream;
			pStm->Write( pData, nSize );
			pStm->Seek( 0L );
        }
        else if( pSimpleObj )
        {
			pStm = new SvCacheStream;
			pSimpleObj->Save( *pStm );
			pStm->Seek( 0L );
        }
        else if( aStream.Is() )
        {
			aStream->Seek( nStmBeginPos );
            SotStorageRef aStg = new SotStorage( *aStream );
            SetData( aStg, TRANSFER_COPY );
        }
		if( pStm )
		{
			if( pStm->GetError() == SVSTREAM_OK )
			{
	            SotStorageRef aStg = new SotStorage( pStm, TRUE );
	            SetData( aStg, TRANSFER_MOVE );
			}
			else
				delete pStm;
		}
    }
    if( aStorage.Is() )
    {
        if( TRANSFER_COPY == nMode )
            aStorage->CopyTo( rDataP );
        else if( TRANSFER_MOVE == nMode )
        {
            if( bStorageOwner )
                rDataP = aStorage;
            else
			{
				if( !rDataP.Is() )
				{
					String aEmpty;
					rDataP = new SotStorage( aEmpty );
				}
				aStorage->CopyTo( rDataP );
			}
            bStorageOwner = FALSE;
			aStorage.Clear();
        }
        else
            rDataP = aStorage;
        bRet = TRUE;
    }
    return bRet;
}

//=========================================================================
BOOL SvData::GetData
(
	SotStorageStreamRef & rDataP,
	UINT32 & rLen,
	TransferOperation nMode
)
{
    BOOL bRet = FALSE;
    if( !aStream.Is() )
    { // eine Stream machen
        SotStorageStreamRef aStm;
        if( pData && nSize )
        {
            aStm = new SotStorageStream( String() );
            aStm->Write( pData, nSize );
        }
        else if( pSimpleObj )
        {
            aStm = new SotStorageStream( String() );
            pSimpleObj->Save( *aStm );
        }
        else if( aStorage.Is() )
        {
            aStm = new SotStorageStream( String() );
			{
            SotStorageRef aStg = new SotStorage( *aStm );
            aStorage->CopyTo( aStg );
			aStg->Commit();
			}
			aStm->Commit();
            aStm->Seek( STREAM_SEEK_TO_END );
        }
		if( aStm.Is() && aStm->GetError() == SVSTREAM_OK )
		{
			UINT32 nLen = aStm->Tell();
            // auf den Anfang setzen
            aStm->Seek( 0L );
            SetData( aStm, nLen, TRANSFER_MOVE );
        }
    }

    if( aStream.Is() )
    {
		rLen = nSize;
        if( TRANSFER_COPY == nMode )
		{
			aStream->Seek( nStmBeginPos );
	        void * pMem = new BYTE[ 8192 ];
	        UINT32 nReadRest = nSize;
	        while( nReadRest )
	        {
				UINT32 nRead = min( nReadRest, 8192 );
				if( nRead != aStream->Read( pMem, nRead ) )
				{
					bRet = FALSE;
	                break;
				}
				nReadRest -= nRead;
	            if( nRead != rDataP->Write( pMem, nRead ) )
	            {
					bRet = FALSE;
	                break;
	            }
	        }
	        delete pMem;
		}
        else if( TRANSFER_MOVE == nMode )
        {
            if( bStreamOwner )
			{
				aStream->Seek( nStmBeginPos + nSize );
                rDataP = aStream;
			}
            else
			{
				if( !rDataP.Is() )
					rDataP = new SotStorageStream( String() );
				GetData( rDataP, rLen, TRANSFER_COPY );
			}
            bStreamOwner = FALSE;
	        aStream.Clear();
		}
        else
		{
			aStream->Seek( nStmBeginPos + nSize );
            rDataP = aStream;
		}
        bRet = TRUE;
    }
	if( !bRet )
		rLen = 0;
    return bRet;
}

//=========================================================================
BOOL SvData::GetData
(
	SotStorageStreamRef & rDataP,
	TransferOperation nMode
)
{
	UINT32 nLen;
	return GetData( rDataP, nLen, nMode );
}

/********************** SvDataTypeList ***************************************
**************************************************************************/

PRV_SV_IMPL_OWNER_LIST(SvDataTypeList,SvDataType)

/*************************************************************************
|*    SvDataTypeList::HasEqualType()
|*
|*    Beschreibung: Sucht 100%ig passenden Typ
*************************************************************************/
BOOL SvDataTypeList::HasEqualType( const SvDataType & rAskType ) const
{
    for( ULONG i = 0; i < Count(); i++ )
        if( GetObject( i ) == rAskType )
			return TRUE;
    return FALSE;
}

/*************************************************************************
|*    SvDataTypeList::Get()
|*
|*    Beschreibung: Sucht einen passenden Typ
*************************************************************************/
const SvDataType *  SvDataTypeList::Get( const SvDataType & rAskType ) const
{
    for( ULONG i = 0; i < Count(); i++ )
    {
        const SvDataType & rType = GetObject( i );
        if( rType.Match( rAskType ) )
            return &rType;
    }
    return NULL;
}

/*************************************************************************
|*    SvDataTypeList::Get()
|*
|*    Beschreibung
*************************************************************************/
const SvDataType * SvDataTypeList::Get( ULONG nFormat ) const
{
    for( ULONG i = 0; i < Count(); i++ )
    {
        const SvDataType & rType = GetObject( i );
        if( rType.GetFormat() == nFormat )
            return &rType;
    }
    return NULL;
}

/*************************************************************************
|*    SvDataTypeList::Get()
|*
|*    Beschreibung
*************************************************************************/
void SvDataTypeList::Remove( const SvDataType & rRemoveType )
{
    SvDataType * pType = (SvDataType *)Get( rRemoveType );
    Remove( pType );
}

/************** class SotDataObject **************************************/
class SotDataObjectFactory : public SotFactory                      			
{                                                                   
public:                                                             
 		TYPEINFO();                                                 
		SotDataObjectFactory( const SvGlobalName & rName,						
					  		const String & rClassName,              
					  		CreateInstanceType pCreateFuncP )       
			: SotFactory( rName, rClassName, pCreateFuncP )
		{}
};
TYPEINIT1(SotDataObjectFactory,SotFactory);

SO2_IMPL_BASIC_CLASS1_DLL(SotDataObject,SotDataObjectFactory,SotObject,
					SvGlobalName( 0xac47f850, 0xf9c3, 0x11d0, 
								0xaa, 0xa1, 0x0, 0xa0, 0x24, 0x9d, 0x55, 0x90 ) )
SO2_IMPL_INVARIANT(SotDataObject)

void SotDataObject::TestMemberObjRef( BOOL /*bFree*/ )
{
}

#ifdef TEST_INVARIANT
void SotDataObject::TestMemberInvariant( BOOL /*bPrint*/ )
{
}
#endif

/************************************************************************
|*    SotDataObject::SotDataObject()
|*
|*    Beschreibung
*************************************************************************/
SotDataObject::SotDataObject()
{
}

SotDataObject::~SotDataObject()
{
}

/*************************************************************************
|*    SotDataObject::GetTypeList()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
const SvDataTypeList & SotDataObject::GetTypeList() const
{
    SotData_Impl * pSotData = SOTDATA();
	if( pSotData->pEmptyList == NULL )
		pSotData->pEmptyList = new SvDataTypeList();
	return *pSotData->pEmptyList;
}

/*************************************************************************
|*    SotDataObject::GetData()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
BOOL SotDataObject::GetData( SvData * )
{
	return FALSE;
}

/*************************************************************************
|*    SotDataObject::GetDataHere()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
BOOL SotDataObject::GetDataHere( SvData * pData )
{
    BOOL bRet = FALSE;

    if( Owner() )
    {
        SvData aData( *(SvDataType *)pData );
        bRet = GetData( &aData );
        if( bRet )
        {
            if( pData->HasMedium( MEDIUM_MEMORY )
              && pData->GetMinMemorySize() >= aData.GetMinMemorySize() )
            {
                void * pTmp;
                // Erst Zeiger holen, dann kopieren
                if( !pData->GetData( &pTmp, TRANSFER_REFERENCE )
                  || !aData.GetData( &pTmp, TRANSFER_COPY ) )
                    bRet = FALSE;
            }
            if( pData->HasMedium( MEDIUM_STREAM ) )
            {
                SotStorageStreamRef aTmp;
                // Erst Zeiger holen, dann kopieren
                if( !pData->GetData( aTmp, TRANSFER_REFERENCE )
                  || !aData.GetData( aTmp, TRANSFER_COPY ) )
                    bRet = FALSE;
            }
            if( pData->HasMedium( MEDIUM_STORAGE ) )
            {
                SotStorageRef aTmp;
                // Erst Zeiger holen, dann kopieren
                if( !pData->GetData( aTmp, TRANSFER_REFERENCE )
                  || !aData.GetData( aTmp, TRANSFER_COPY ) )
                    bRet = FALSE;
            }
        }
    }
    return bRet;
}

/*************************************************************************
|*    SotDataObject::QueryDataType()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
BOOL SotDataObject::QueryDataType( SvDataType & rType )
{
    // Passenden Typ suchen
    const SvDataType * pType = GetTypeList().Get( rType );
    if( pType )
        rType = *pType;
    return pType != NULL;
}

// HACK fuer die Implementation der alten Clipboard-Schnittstelle
// in VCL, primitives RTTI nur fuer SotData(Member)Object
BOOL SotDataObject::IsSotDataMemberObject( void )
{
	return FALSE;
}

/************** class SotDataMemberObject *********************************/
class SotDataMemberObjectFactory : public SotFactory                      			
{                                                                   
public:                                                             
 		TYPEINFO();                                                 
		SotDataMemberObjectFactory( const SvGlobalName & rName,						
					  		const String & rClassName,              
					  		CreateInstanceType pCreateFuncP )       
			: SotFactory( rName, rClassName, pCreateFuncP )
		{}
};
TYPEINIT1(SotDataMemberObjectFactory,SotFactory);

SO2_IMPL_BASIC_CLASS1_DLL(SotDataMemberObject,SotDataMemberObjectFactory,SotDataObject,
				SvGlobalName( 0x2f018340, 0x156a, 0x11d1, 
								0x89, 0xcc, 0x0, 0x80, 0x29, 0xe4, 0xb0, 0xb1 ) )

/************************************************************************
|*    SotDataMemberObject::SotDataMemberObject()
|*    SotDataMemberObject::~SotDataMemberObject()
|*
|*    Beschreibung
*************************************************************************/
SotDataMemberObject::SotDataMemberObject()
{
}

SotDataMemberObject::SotDataMemberObject( SotDataObject * pExtObj )
	: xExtObj( pExtObj )
{
	aTypeList = xExtObj->GetTypeList();
}

SotDataMemberObject::~SotDataMemberObject()
{
    SvData * pData = aDataList.First();
    while( pData )
    {
        delete pData;
        pData = aDataList.Next();
    }
}

// HACK fuer die Implementation der alten Clipboard-Schnittstelle
// in VCL, primitives RTTI nur fuer SotData(Member)Object
BOOL SotDataMemberObject::IsSotDataMemberObject( void )
{
	return TRUE;
}

/************************************************************************
|*    SotDataMemberObject::Append()
|*
|*    Beschreibung
*************************************************************************/
void SotDataMemberObject::Append( SvData * pData )
{
	const SvDataType * pType = aTypeList.Get( *pData );
	if( !pType )
	{
    	aDataList.Insert( pData, LIST_APPEND );
    	// Typenliste auch mitpflegen
    	aTypeList.Append( *pData );
	}
	else
	{
		for( ULONG i = 0; i < aDataList.Count(); i++ )
		{
			if( aDataList.GetObject( i )->Match( *pData ) )
			{
				delete aDataList.Remove( i );
				aDataList.Insert( pData, i );
			}
		}
		aTypeList.Remove( (SvDataType *)pType );
		aTypeList.Insert( *pData );
	}
}

/*************************************************************************
|*    SotDataMemberObject::GetData()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
const SvDataTypeList & SotDataMemberObject::GetTypeList() const
{
    return aTypeList;
}

/*************************************************************************
|*    SotDataMemberObject::GetData()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
BOOL SotDataMemberObject::GetData( SvData * pOutData )
{
    SvData * pData = aDataList.First();
    while( pData )
    {
        if( pData->Match( *pOutData ) )
        {
            pOutData->SetData( pData );
            return TRUE;
        }
        pData = aDataList.Next();
    }
	if( xExtObj )
		return xExtObj->GetData( pOutData );
    return FALSE;
}

/*************************************************************************
|*    SotDataMemberObject::GetDataHere()
|*
|*    Beschreibung      Holt die Daten von einen IDataObject.
*************************************************************************/
BOOL SotDataMemberObject::GetDataHere( SvData * pOutData )
{
    SvData * pData = aDataList.First();
    while( pData )
    {
        if( pData->Match( *pOutData ) )
			return SotDataObject::GetDataHere( pOutData );
        pData = aDataList.Next();
    }
	if( xExtObj.Is() )
		return xExtObj->GetDataHere( pOutData );
    return FALSE;
}


