/*************************************************************************
 *
 *  $RCSfile: jthreadpool.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:29:34 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <string.h>

#include <osl/diagnose.h>

#include <uno/threadpool.h>

#include <com/sun/star/java/XJavaThreadRegister_11.hpp>

#include "jthreadpool.hxx"

using namespace ::com::sun::star::java;
using namespace ::com::sun::star::uno;

//--------------------------------------------------------------------------
// This is a nasty hack to grant the javaloader_callback-routine access to
// the XJavaThreadRegister_11-instance.
// Note : This works only as long as there is only ONE VM PER PROCESS !
// Note : This is only a weak reference
//-------------------------------------------------------------------------
static XJavaThreadRegister_11 *g_pThreadRegister = 0;

struct JavaThreadpoolCallbackHolder
{
	jobject jCallback;
	jobject jThreadSpecificData;
	JavaVM *pVM;
};


//-----------------------
// is called, when a java request shall  be executed
//----------------------
void SAL_CALL javaloader_callback( void *pThreadSpecificData )
{

	JavaThreadpoolCallbackHolder *pHolder = ( JavaThreadpoolCallbackHolder*)pThreadSpecificData;

	// Attach the thread to the java vm
	OSL_ASSERT( g_pThreadRegister );

	JNIEnv *pMyEnv = 0;
	
	pHolder->pVM->AttachCurrentThread(&pMyEnv, NULL);
	if( ! pMyEnv )
	{
		OSL_ASSERT( 0 );
	}
	g_pThreadRegister->registerThread();
	
	
	jclass jClassNativeCallback = pMyEnv->FindClass( "com/sun/star/lib/uno/environments/remote/INativeCallback" );
	if( pMyEnv->ExceptionOccurred() )
	{
		OSL_ASSERT( 0 );
		// There is no possibility to report an error to the caller !
		return;
	}

	jmethodID id = pMyEnv->GetMethodID( jClassNativeCallback ,
											   "doRequest",
											   "(Ljava/lang/Object;)V" );
	if( pMyEnv->ExceptionOccurred() )
	{
		OSL_ASSERT( 0 );
		// There is no possibility to report an error to the caller !
		return;
	}
	OSL_ASSERT( id );

	//--------------------
	//
	// Do the call
	//
	//--------------------	
	pMyEnv->CallVoidMethod( pHolder->jCallback, id , pHolder->jThreadSpecificData );
	if( pMyEnv->ExceptionOccurred() )
	{
		OSL_ASSERT( 0 );
		// There is no possibility to report an error to the caller !
		return;
	}

	// tiding up ....
	pMyEnv->DeleteGlobalRef( pHolder->jCallback );
	if( pMyEnv->ExceptionOccurred() )
	{
		OSL_ASSERT( 0 );
		// There is no possibility to report an error to the caller !
		return;
	}

	pMyEnv->DeleteGlobalRef( pHolder->jThreadSpecificData );
	if( pMyEnv->ExceptionOccurred() )
	{
		OSL_ASSERT( 0 );
		// There is no possibility to report an error to the caller !
		return;
	}

	g_pThreadRegister->revokeThread();
	if( ! g_pThreadRegister->isThreadAttached() )
	{
		pHolder->pVM->DetachCurrentThread();
	}

	delete pHolder;
}

extern "C"
{
	
/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    add
 * Signature: ([B)Z
 */
JNIEXPORT jboolean JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_add
  (JNIEnv *pEnv , jclass theClass, jbyteArray theArray)
{
	jbyte *myJByteArray = pEnv->GetByteArrayElements(  theArray , 0 );
	if( pEnv->ExceptionOccurred() )
		return 0;
		
	sal_Bool b = uno_threadIdent_add( myJByteArray , pEnv->GetArrayLength( theArray ) );
	
	pEnv->ReleaseByteArrayElements( theArray, myJByteArray , JNI_ABORT );
	if( pEnv->ExceptionOccurred() )
		return 0;

	return b;
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    createTicket
 * Signature: ()J
 */
	
JNIEXPORT jlong JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_createTicket
   (JNIEnv *pEnv, jclass theClass, jobject jDisposeId )
{
	return ( jlong ) uno_threadpool_createTicket( (sal_uInt8*) (sal_uInt32) jDisposeId );
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    dispose
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_dispose
  (JNIEnv *pEnv, jclass theClass ,  jobject jDisposeId)
{
	uno_threadpool_disposeThreads( (sal_uInt8 *) ( sal_uInt32 ) jDisposeId );
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    reply
 * Signature: ([BLjava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_reply
  (JNIEnv *pEnv , jclass theClass , jbyteArray jID, jobject jThreadSpecificData )
{
	jobject jGlobalThreadSpecificData = pEnv->NewGlobalRef(  jThreadSpecificData );
	if( pEnv->ExceptionOccurred() )
		return;

	jbyte *myJByteArray = pEnv->GetByteArrayElements( jID , 0 );
	if( pEnv->ExceptionOccurred() )
	{
		return;
	}
	sal_Int32 nLength = pEnv->GetArrayLength( jID );
	if( pEnv->ExceptionOccurred() )
	{
		return;
	}
	

	uno_threadpool_reply( myJByteArray ,
						  nLength ,
						  (void*)jGlobalThreadSpecificData );
	
	pEnv->ReleaseByteArrayElements( jID, myJByteArray , JNI_ABORT );
	if( pEnv->ExceptionOccurred() )
		return;
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    request
 * Signature: ([BLjava/lang/Object;Lcom/sun/star/lib/uno/environments/remote/NativeCallback;Z)V
 */
JNIEXPORT void JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_request
  ( JNIEnv *pEnv,
    jclass theClass ,
	jbyteArray jID,
	jobject jThreadSpecificData,
	jobject callback,
	jboolean isOneway)
{
	JavaThreadpoolCallbackHolder *pHolder = new JavaThreadpoolCallbackHolder;

	pEnv->GetJavaVM( &(pHolder->pVM) );

	pHolder->jThreadSpecificData = pEnv->NewGlobalRef(  jThreadSpecificData );
	if( pEnv->ExceptionOccurred() )
		return;

	pHolder->jCallback = pEnv->NewGlobalRef( callback );
	if( pEnv->ExceptionOccurred() )
		return;	


	jbyte *myJByteArray = pEnv->GetByteArrayElements( jID , 0 );
	if( pEnv->ExceptionOccurred() )
	{
		return;
	}
	sal_Int32 nLength = pEnv->GetArrayLength( jID );
	if( pEnv->ExceptionOccurred() )
	{
		return;
	}
	
	uno_threadpool_request( myJByteArray ,
						  nLength ,
						  (void*)pHolder,
						  javaloader_callback,
						  isOneway);
	
	pEnv->ReleaseByteArrayElements( jID, myJByteArray , JNI_ABORT );
	if( pEnv->ExceptionOccurred() )
		return;
	
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    retrieve
 * Signature: ()[B
 */
JNIEXPORT jbyteArray JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_retrieve
   (JNIEnv *pEnv, jclass theClass)
{
	sal_Int8 *pID = 0;
	sal_Int32 nLength = 0;
	uno_threadIdent_retrieve( &pID , &nLength );

	jbyteArray array = pEnv->NewByteArray( nLength );
	if( pEnv->ExceptionOccurred( ))
	{
		return 0;
	}

	jbyte *pMem = pEnv->GetByteArrayElements( array , 0 );
	if( pEnv->ExceptionOccurred( ))
	{
		return 0;
	}
	memcpy( pMem , pID , nLength );

	pEnv->ReleaseByteArrayElements( array, pMem , 0 );
	if( pEnv->ExceptionOccurred( ))
	{
		return 0;
	}
	
	return array;
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    revoke
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_revoke
  (JNIEnv *, jclass)
{
	uno_threadIdent_revoke();
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    stopDispose
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_stopDispose
  (JNIEnv *pEnv, jclass theClass ,  jobject jDisposeId)
{
	uno_threadpool_disposeThreads( (sal_uInt8 *) ( sal_uInt32 ) jDisposeId );
}

/*
 * Class:     com_sun_star_lib_uno_environments_remote_NativeThreadPool
 * Method:    waitOnTicket
 * Signature: (J)V
 */
JNIEXPORT jobject JNICALL Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_waitOnTicket
  (JNIEnv *pEnv, jclass theClass , jlong jLongValue )
{
	jobject jThreadSpecificData = 0;

	uno_threadpool_waitOnTicket( (uno_threadpool_Ticket *) (sal_uInt32 )jLongValue ,
								 (void**) &jThreadSpecificData );

	return jThreadSpecificData;
}

}


//--------------------------
// Init the java threadpool
//--------------------------
sal_Bool SAL_CALL javaloader_initNativeThreadPool(
	JNIEnv *pEnv ,
	const Reference< XJavaThreadRegister_11 > &rThreadRegister )
{

	// This is an alternative to the System.loadLibrary call
	// 
//  	JNINativeMethod table[] = {
//  		{"add" , "([B)Z" , Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_add},
//  		{"createTicket" , "(Ljava/lang/Object;)J" , Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_createTicket},
//  		{"dispose" , "(Ljava/lang/Object;)V" , Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_dispose},
//  		{"reply" , "([BLjava/lang/Object;)V" , Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_reply},
//  		{"request" , "([BLjava/lang/Object;Lcom/sun/star/lib/uno/environments/remote/INativeCallback;Z)V" ,
//  		             Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_request},
//  		{"retrieve" , "()[B" ,
//  		             Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_retrieve},
//  		{"revoke" , "()V" ,
//  		             Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_revoke},
//  		{"stopDispose" , "(Ljava/lang/Object;)V" ,
//  		 Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_stopDispose},
//  		{"waitOnTicket" , "(J)Ljava/lang/Object;" ,
//  		 Java_com_sun_star_lib_uno_environments_remote_NativeThreadPool_waitOnTicket} };
	
//  	sal_Int32 nCount = sizeof( table );
	
//  	jclass jClassNativeThreadPool = pEnv->FindClass( "com/sun/star/lib/uno/environments/remote/NativeThreadPool" );
//  	if( pEnv->ExceptionOccurred() )
//  	{
//  		pEnv->ExceptionClear();
//  		return sal_False;
//  	}
	
//  	sal_Int32 nResult = pEnv->RegisterNatives( jClassNativeThreadPool , table , 9 );
//  	if( pEnv->ExceptionOccurred() )
//  	{
//  		pEnv->ExceptionClear();
//  		return sal_False;
//  	}

	
	// now instantiate a NativeThreadPoolWrapper
	jclass jClassNativeThreadPoolWrapper = pEnv->FindClass( "com/sun/star/lib/uno/environments/remote/NativeThreadPoolWrapper" );
	if( pEnv->ExceptionOccurred() )
	{
		pEnv->ExceptionClear();
		return sal_False;
	}
	
	jmethodID idCtor = pEnv->GetMethodID( jClassNativeThreadPoolWrapper ,
										  "<init>",
										  "()V" );
	if( pEnv->ExceptionOccurred() )
	{
		pEnv->ExceptionClear();
		return sal_False;
	}

	jobject jThreadpool = pEnv->NewObject( jClassNativeThreadPoolWrapper , idCtor );
	if( pEnv->ExceptionOccurred() )
	{
		pEnv->ExceptionClear();
		return sal_False;
	}

	// get the Threadpool-class   
	jclass jClassTheThreadpool = pEnv->FindClass( "com/sun/star/lib/uno/environments/remote/ThreadPool" );
	if( pEnv->ExceptionOccurred() )
	{
		pEnv->ExceptionClear();
		return sal_False;
	}

  	jmethodID idSetThreadpool = pEnv->GetStaticMethodID(
  		jClassTheThreadpool ,
  		"setThreadPool",
  		"(Lcom/sun/star/lib/uno/environments/remote/IThreadPool;)V" );

	if( pEnv->ExceptionOccurred() )
	{
		pEnv->ExceptionDescribe();
		pEnv->ExceptionClear();
		return sal_False;
	}

	pEnv->CallStaticVoidMethod( jClassTheThreadpool,
						  idSetThreadpool ,
						  jThreadpool );
	if( pEnv->ExceptionOccurred() )
	{
		return sal_False;
	}

	// hold it weak
	g_pThreadRegister = rThreadRegister.get();
	return sal_True;
}
