/*************************************************************************
 *
 *  $RCSfile: w32jre.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:29:34 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
/*
 * Win32 specific JRE support functions
 */

//#include <tools/presys.h>
#include <tools/prewin.h>
#include <windows.h>
#include <jni.h>
//#include <tools/postsys.h>
#include <tools/postwin.h>
//#include <tools/string.hxx>
#include <jre.hxx>
//#include <stdlib.h>

#define JRE_KEY	    "Software\\JavaSoft\\Java Runtime Environment"
#define JDK_KEY	    "Software\\JavaSoft\\Java Development Kit"
#define MS_VM_KEY	"Software\\Mircosoft\\Java VM"

/* Forward Declarations */
jint LoadSettings(JRESettings *set, HKEY key);
jint GetSettings(JRESettings *set, const char *version, const char *keyname);
OString GetStringValue(HKEY key, const char *name);

/*
 * Retrieve settings from registry for current runtime version. Returns
 * 0 if successful otherwise returns -1 if no installed runtime was found
 * or the registry data was invalid.
 */
int JRE_GetCurrentSettings(JRESettings *set)
{
    jint r = -1;
    HKEY hKey;

	if (set->type == JRESettings.MS_VM)
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, MS_VM_KEY, 0, KEY_READ, &hKey) == 0)
		{
			String aJavaLibDir = GetStringValue(hKey, "LibsDirectory");
			char *pChr;
				
			/* remove trailing 'lib'-subdirectory */
			if ((pChr = strrchr(aJavaLibDir, '\\')) != NULL)
				*pChr = '\0';
			
			set->javaHome = OUString::createFromAscii( pChr);
			set->classPath = OStringToOUString( GetStringValue(hKey, "Classpath"),RTL_TEXTENCODING_ASCII_US );
			r = 0;
			RegCloseKey(hKey);
		}
	else
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, JRE_KEY, 0, KEY_READ, &hKey) == 0) 
		{
			String ver = GetStringValue(hKey, "CurrentVersion");
			if (ver.Len() != 0) 
			{
				r = JRE_GetSettings(set, ver);
			}	
			RegCloseKey(hKey);
		}
	
    return r;
}

/*
 * Retrieves settings from registry for specified runtime version.
 * Searches for either installed JRE and JDK runtimes. Returns 0 if
 * successful otherwise returns -1 if requested version of runtime
 * could not be found.
 */
int JRE_GetSettings(JRESettings *set, const char *version)
{
    if (set->type == JRESettings.MS_VM)
		set->type = JRESettings.JRE; //auf default zurcksetzen

	if (set->type == JRESettings.JRE) 
	{
		set->type = JRESettings.JRE;
		if (GetSettings(set, version, JRE_KEY) != 0) 
		{	
			set->type = JRESettings.JDK; // type changed
			return GetSettings(set, version, JDK_KEY);
		}
	} 
	else
	{ // hier kann der Typ nur JDK sein
		if (GetSettings(set, version, JDK_KEY) != 0) 
		{	
			set->type = JRESettings.JRE; // type changed
			return GetSettings(set, version, JRE_KEY);
		}
	} 

    return 0;
}

jint GetSettings(JRESettings *set, const char *version, const char *keyname)
{
    HKEY key;
    int r = -1;

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname, 0, KEY_READ, &key) == 0) 
	{
		char *major, *minor, *micro = 0;
		if (JRE_ParseVersion(version, &major, &minor, &micro) == 0) 
		{
			HKEY subkey;
			char *ver = JRE_MakeVersion(major, minor, 0);
			set->majorVersion = OUString::createFromAscii( major);
			set->minorVersion = OUString::createFromAscii( minor);
			if (RegOpenKeyEx(key, ver, 0, KEY_READ, &subkey) == 0) 
			{
				if ((r = LoadSettings(set, subkey)) == 0) 
				{
					if (micro != 0) 
					{
						if (set->microVersion.getLength() == 0 ||
							strcmp(micro, OUStringToOString( set->microVersion, RTL_TEXTENCODING_ASCII_US)) != 0) 
						{
							r = -1;
						}
					}
				}
				RegCloseKey(subkey);
			}
			free(ver);
			if( major ) free( major );
			if( minor ) free( minor );
			if( micro ) free( micro );
		}
		RegCloseKey(key);
    }
    return r;
}

/*
 * Load runtime settings from specified registry key. Returns 0 if
 * successful otherwise -1 if the registry data was invalid.
 */
static jint LoadSettings(JRESettings *set, HKEY key)
{
    /* Full path name of JRE home directory (required) */
    set->javaHome = OUString::createFromAscii(GetStringValue(key, "JavaHome"));

    if( set->javaHome.getLength() == 0 ) 
		return -1;
    /* Full path name of JRE runtime DLL */
    set->runtimeLib = OUString::createFromAscii(GetStringValue(key, "RuntimeLib"));

    if( set->runtimeLib.getLength() == 0) 
	{
		OString aHome= OUStringToOString( set->javaHome, RTL_TEXTENCODING_ASCII_US);
		set->runtimeLib = OUString::createFromAscii( JRE_GetDefaultRuntimeLib( aHome)); //RTL_TEXTENCODING_ASCII_US
    }

    /* Class path setting to override default */
    set->classPath = OUString::createFromAscii( GetStringValue(key, "ClassPath"));

    if( set->classPath.getLength() == 0) 
	{
		OString aHome= OUStringToOString( set->javaHome, RTL_TEXTENCODING_ASCII_US);
		set->classPath = OUString::createFromAscii(JRE_GetDefaultClassPath( aHome));
    }

    /* Optional JIT compiler library name */
    set->compiler = OUString::createFromAscii( GetStringValue(key, "Compiler"));
    /* Release micro-version */
    set->microVersion = OUString::createFromAscii( GetStringValue(key, "MicroVersion"));
    return 0;
}

/*
 * Returns string data for the specified registry value name, or
 * NULL if not found.
 */
static OString GetStringValue(HKEY key, const char *name)
{
    DWORD type, size;
    OString aValue;

    if (RegQueryValueEx(key, name, 0, &type, 0, &size) == 0 &&
		type == REG_SZ ) 
	{
		unsigned char * value;
		value = (unsigned char *)JRE_Malloc(size);
		if (RegQueryValueEx(key, name, 0, 0, value, &size) == 0) 
			aValue = (char *)value;
		free(value);
    }
    return aValue;
}

