/*************************************************************************
 *
 *  $RCSfile: testclient.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:29:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdio.h>
#include <assert.h>

#include <osl/mutex.hxx>
#include <osl/module.h>
#include <osl/thread.h>
#include <osl/conditn.h>
#include <osl/diagnose.h>

#include <uno/mapping.hxx>

#include <cppuhelper/servicefactory.hxx>

#include <com/sun/star/registry/XImplementationRegistration.hpp>

#include <com/sun/star/connection/XConnector.hpp>

#include <com/sun/star/bridge/XBridgeFactory.hpp>

#include <com/sun/star/lang/XComponent.hpp>

#include <cppuhelper/weak.hxx>

#ifndef _CPPUHELPER_FACTORY_HXX_
#include <cppuhelper/factory.hxx>
#endif


using namespace ::rtl;
using namespace ::cppu;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::bridge;
using namespace ::com::sun::star::registry;
using namespace ::com::sun::star::connection;

Reference <XInterface > createComponent( const ::rtl::OUString &sService ,
										 const ::rtl::OUString &sDllName,
										 const Reference < XMultiServiceFactory > &rSMgr )
{
	Reference< XInterface > rInterface;
	rInterface = rSMgr->createInstance( sService );

	if( ! rInterface.is() )
	{
		// erst registrieren
		Reference < XImplementationRegistration > rReg (
			rSMgr->createInstance(
				OUString::createFromAscii( "com.sun.star.registry.ImplementationRegistration" )),
			UNO_QUERY );
		
		assert( rReg.is() );
#ifdef SAL_W32
		OUString aDllName = sDllName;
#else
		OUString aDllName = L"lib";
		aDllName += sDllName;
		aDllName += L".so";
#endif

		try
		{
			rReg->registerImplementation(
				OUString::createFromAscii( "com.sun.star.loader.SharedLibrary" ),
				aDllName,
				Reference< XSimpleRegistry > () );
			rInterface = rSMgr->createInstance( sService );
		}
		catch( Exception & ) 
		{
			printf( "couldn't register dll %s\n" ,
					OUStringToOString( aDllName, RTL_TEXTENCODING_ASCII_US ).getStr()  );
		}		
	}
	return rInterface;
}


void main( int argc , char *argv[])
{
	sal_Bool bUseNew = ( 3 == argc );
#ifdef SOLARIS
	ChangeGlobalInit(); // Switch on threads !
#endif
	if( argc < 2 )
	{
		printf( "usage : testclient host:port" );
		return;
	}
	
	{

	    Reference< XMultiServiceFactory > rSMgr = createRegistryServiceFactory( "client.rdb" );

		Reference < XConnector > rConnector(
			createComponent( L"com.sun.star.connection.Connector",
							 L"connectr",
							 rSMgr ),
			UNO_QUERY );

		try
		{
			Reference < XConnection > rConnection =
				rConnector->connect( OUString::createFromAscii( argv[1] ) );

			printf( "%s\n" , OUStringToOString( rConnection->getDescription(),
												RTL_TEXTENCODING_ASCII_US ).pData->buffer );
			// just ensure that it is registered
			createComponent( L"com.sun.star.bridge.Bridge.iiop",
							 L"iiopbrdg",
							 rSMgr );

			Reference < XBridgeFactory > rFactory(
				createComponent( L"com.sun.star.bridge.BridgeFactory",
								 L"brdgfctr",
								 rSMgr ),
				UNO_QUERY );

			if( rFactory.is() )
			{

				Reference < XBridge > rBridge = rFactory->createBridge(
					L"bla blub",
					L"iiop",
					rConnection,
					Reference < XInstanceProvider > () );

				{
					printf( "getting initial object\n" );
					Reference < XInterface > rInitialObject	= rBridge->getInstance( L"bla" );
//  					Reference< XComponent > rComp( rBridge , UNO_QUERY );
//  					rComp->dispose();
					printf( "releasing initial object\n" );
				}


				printf( "creating new bridge\n" );
				rBridge  = rFactory->createBridge(
					L"blubber",
					L"iiop",
					rConnection,
					Reference < XInstanceProvider > () );
				
				Reference < XInterface > rInitialObject	= rBridge->getInstance( L"bla" );
				Reference < XMultiServiceFactory > rRemoteSMgr( rInitialObject , UNO_QUERY );
				if( rRemoteSMgr.is() )
				{
					Reference< XInterface > r = rRemoteSMgr->createInstance( L"com.sun.star.registry.ImplementationRegistration" );
					printf( "got the application servicemanager \n" );
					Reference< XImplementationRegistration > rImpl ( r , UNO_QUERY );
					if( rImpl.is() )
					{
						printf( "instantiated remote implementationregistration\n" );
					}
				}
				else
				{
					printf( "couldn't get remote object\n" );
					Reference < XComponent > rComponent( rBridge , UNO_QUERY );
					rComponent->dispose();
				}
			}
		}
		catch( Exception & )
		{
			printf( "Login failed, got an Exception !\n" );
		}		


		printf( "Closing...\n" );
		TimeValue value={2,0};
		osl_waitThread( &value );
		
		Reference < XComponent > rComp( rSMgr , UNO_QUERY );
		rComp->dispose();
	}
	printf( "Closed\n" );

}
