/*************************************************************************
 *
 *  $RCSfile: testdaemon.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:29:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <assert.h>

#include <osl/mutex.hxx>
#include <osl/conditn.h>

#include <vos/thread.hxx>
#include <vos/socket.hxx>

#include <cppuhelper/servicefactory.hxx>
#include <cppuhelper/implbase1.hxx>

#include <com/sun/star/connection/XAcceptor.hpp>
#include <com/sun/star/connection/XConnection.hpp>

#include <com/sun/star/registry/XImplementationRegistration.hpp>
#include <com/sun/star/bridge/XInstanceProvider.hpp>
#include <com/sun/star/bridge/XBridgeFactory.hpp>

#include <com/sun/star/lang/XComponent.hpp>
#include <com/sun/star/lang/XInitialization.hpp>

#include <osl/conditn.h>

#include <vos/channel.hxx>
#include <vos/process.hxx>


#include <cppuhelper/weak.hxx>

using namespace ::rtl;
using namespace ::vos;
using namespace ::cppu;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::bridge;
using namespace ::com::sun::star::registry;
using namespace ::com::sun::star::connection;

#ifdef SAL_W32
#include <conio.h>
#endif

/*********
 *
 ********/


class OInstanceProvider :
	public ::cppu::OWeakObject,
	public XInstanceProvider
{
public:
	OInstanceProvider( const Reference < XMultiServiceFactory  > & rSMgr  ) :
		m_rSMgr( rSMgr )
		{}
public:	
	// XInterface
	Any	        SAL_CALL queryInterface( const Type & aType);
	void 		SAL_CALL acquire() 						 { OWeakObject::acquire(); }
	void 		SAL_CALL release() 						 { OWeakObject::release(); }
	
public:
	// XConnectCallback
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > SAL_CALL
	       getInstance( const ::rtl::OUString& sObjectName )
		         throw( ::com::sun::star::container::NoSuchElementException,
			            ::com::sun::star::uno::RuntimeException);

	Reference < XMultiServiceFactory > m_rSMgr;
};

Any OInstanceProvider::queryInterface( const  Type & aType )
{
	Any a = ::cppu::queryInterface( aType , 
			SAL_STATIC_CAST( XInstanceProvider * , this ) );
	if( a.hasValue() )
	{
	    return a;
	}
	return OWeakObject::queryInterface( aType );
}

::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface > 
	OInstanceProvider::getInstance( const ::rtl::OUString& sObjectName )
		throw(::com::sun::star::container::NoSuchElementException,
			  ::com::sun::star::uno::RuntimeException)
{
	return Reference < XInterface > ( m_rSMgr , UNO_QUERY );
}



Reference <XInterface > createComponent( const ::rtl::OUString &sService,
										 const ::rtl::OUString &sDllName,
										 const Reference < XMultiServiceFactory > &rSMgr )
{
	Reference< XInterface > rInterface;
	rInterface = rSMgr->createInstance( sService );

	if( ! rInterface.is() )
	{
		// erst registrieren
		Reference < XImplementationRegistration > rReg (
			rSMgr->createInstance(
				OUString::createFromAscii( "com.sun.star.registry.ImplementationRegistration" )),
			UNO_QUERY );
		
		assert( rReg.is() );
#ifdef SAL_W32
		OUString aDllName = sDllName;
#else
		OUString aDllName = L"lib";
		aDllName += sDllName;
		aDllName += L".so";
#endif

		try
		{
			rReg->registerImplementation(
				OUString::createFromAscii( "com.sun.star.loader.SharedLibrary" ),
				aDllName,
				Reference< XSimpleRegistry > () );
			rInterface = rSMgr->createInstance( sService );
		}
		catch( Exception & ) 
		{
			printf( "couldn't register dll %s\n" ,
					OUStringToOString( aDllName, RTL_TEXTENCODING_ASCII_US ).getStr()  );
		}		
	}
	return rInterface;
}



class MyThread :
	public OThread,
	public WeakImplHelper1< XEventListener >
{
public:
	MyThread( const Reference< XAcceptor > &r ,
			  const Reference< XBridgeFactory > &rFactory,
			  const Reference< XMultiServiceFactory > & rSMgr,
			  const OUString &sConnectionDescription);
	virtual void run();

public: // XEventListener
	virtual void SAL_CALL disposing( const ::com::sun::star::lang::EventObject& Source )
		throw(::com::sun::star::uno::RuntimeException);

private:
	void launchApplication();
	void provide( const OUString &rString );
	
private:
	Reference < XAcceptor > m_rAcceptor;
	Reference < XBridgeFactory > m_rBridgeFactory;
	Reference < XMultiServiceFactory > m_rSMgr;
	OUString m_sConnectionDescription;
	oslCondition              m_cnd;
};


MyThread::MyThread(const Reference< XAcceptor > &r ,
				   const Reference< XBridgeFactory > &rFactory,
				   const Reference< XMultiServiceFactory > & rSMgr,
				   const OUString &sConnectionDescription)	:
		m_rAcceptor( r ),
		m_rBridgeFactory ( rFactory ),
		m_sConnectionDescription( sConnectionDescription ),
		m_rSMgr( rSMgr )
{
	m_cnd = osl_createCondition();	
	osl_resetCondition( m_cnd );
}


void MyThread::launchApplication()
{
#ifdef SAL_W32
	::vos::OProcess process( "testexe.exe" );
#else
	::vos::OProcess process( "./testexe" );
#endif	
 	process.execute( OProcess::TOption_Detached );
}

void MyThread::provide( const OUString &s )
{
	ORef< OChannelBroker > broker = OChannelBroker::getGlobalChannelBroker();

	OUString str = s.copy( s.lastIndexOf( ':' )+1 );

	OObject *pObject = ( OObject * ) ((sal_Int64) str.toDouble());

	ORef< OChannel > channel = OChannel::createChannel( *pObject );

	broker->provide( channel , OString ( "blubber" ));
	
}

void MyThread::disposing( const ::com::sun::star::lang::EventObject& Source )
	throw(::com::sun::star::uno::RuntimeException)

{
	osl_setCondition( m_cnd );
}

void MyThread::run()
{
	while ( sal_True )
	{
		try
		{
			Reference < XConnection > rConnection =
				m_rAcceptor->accept( m_sConnectionDescription );

			if( ! rConnection.is() )
			{
				break;
			}

			{
				Reference < XBridge > rBridge =
					m_rBridgeFactory->createBridge(
						L"" ,
						L"iiop" ,
						rConnection ,
						(XInstanceProvider * ) new OInstanceProvider( m_rSMgr ) );
				
				// not thread safe !
				// needs synchronization with OInstanceProvider
				Reference< XComponent > rComponent( rBridge , UNO_QUERY );
				rComponent->addEventListener( Reference< XEventListener > ( ( XEventListener * ) this ) );
			}
			
			printf( "daemon waits for the bridge to be disposed\n" );
			osl_waitCondition( m_cnd , 0);
			
			printf( "daemon launches the testexe application\n" );
			launchApplication();

			printf( "daemon provides the connection to the new process\n" );
			provide( rConnection->getDescription() );

		}
		catch ( ... )
		{
			printf( "Exception was thrown by acceptor thread\n" );
			break;
		}
	}
}



#ifdef SOLARIS
extern "C" void ChangeGlobalInit();
#endif

void main( int argc, char *argv[] )
{
	
	sal_Bool bUseNew = sal_False;
	if( argc < 2 )
	{
		printf( "usage : testdaemon host:port\n" );
		exit(1 );
	}

	bUseNew = ( 3 == argc );
	
#ifdef SOLARIS
	ChangeGlobalInit(); // Switch on threads !
#endif	

	osl_startChannelBroker();
	{
		Reference< XMultiServiceFactory > rSMgr = createRegistryServiceFactory( "server.rdb" );

		Reference < XBridgeFactory > rBridgeFactory (
			createComponent( L"com.sun.star.bridge.BridgeFactory",
							 L"brdgfctr",
							 rSMgr ),
			UNO_QUERY );
		
	
		createComponent( L"com.sun.star.bridge.Bridge.iiop",
						 L"iiopbrdg",
						 rSMgr );
		

		Reference < XAcceptor > rAcceptor(
			createComponent( L"com.sun.star.connection.Acceptor",
							 L"acceptor",
							 rSMgr ) ,
			UNO_QUERY );

		MyThread *pThread = new MyThread (
			rAcceptor ,
			rBridgeFactory,
			rSMgr,
			OStringToOUString( argv[1] , RTL_TEXTENCODING_ASCII_US ) );

		pThread->acquire();
		
		pThread->create();

#ifdef SAL_W32	
		_getch();
#elif  SOLARIS
		getchar();
#elif LINUX
		TimeValue value={360,0};
		osl_waitThread( &value );
#endif
		printf( "Closing...\n" );
		
		rAcceptor->stopAccepting();
		pThread->join();
		pThread->release();
		
		printf( "Closed\n" );		

		Reference < XComponent > rComp2( rBridgeFactory , UNO_QUERY );
		rComp2->dispose();
		Reference < XComponent > rComp( rSMgr, UNO_QUERY );
		rComp->dispose();
	}

	osl_stopChannelBroker();
}
