/*************************************************************************
 *
 *  $RCSfile: stordmon.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:32 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _STORE_STORDMON_CXX_ "$Revision: 1.1.1.1 $"

#ifndef __STL_USE_NEWALLOC
#define __STL_USE_NEWALLOC 1
#endif
#ifndef __UTILITY__
#include <utility>
#endif
#ifndef __HASH_SET__
#include <hash_set>
#endif

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _VOS_CONDITN_HXX_
#include <vos/conditn.hxx>
#endif
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
#ifndef _VOS_OBJECT_HXX_
#include <vos/object.hxx>
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif
#ifndef _VOS_THREAD_HXX_
#include <vos/thread.hxx>
#endif
#ifndef _VOS_TIMER_HXX_
#include <vos/timer.hxx>
#endif

#ifndef _STORE_MACROS_HXX_
#include <store/macros.hxx>
#endif

#ifndef _STORE_STORBASE_HXX_
#include <storbase.hxx>
#endif
#ifndef _STORE_STORDMON_HXX_
#include <stordmon.hxx>
#endif

#ifdef _USE_NAMESPACE
using namespace store;
#endif

/*========================================================================
 *
 * OStorePageDaemon... internals.
 *
 *======================================================================*/
typedef OStorePageBIOS* key_type;

struct key_hash :
	public NAMESPACE_STD(unary_function)<key_type, sal_uInt32>
{
	sal_uInt32 operator() (key_type k) const { return (sal_uInt32)k; }
};

struct key_cmp :
	public NAMESPACE_STD(binary_function)<key_type, key_type, bool>
{
	bool operator() (key_type k1, key_type k2) const { return (k1 == k2); }
};

typedef NAMESPACE_STD(hash_set)<key_type, key_hash, key_cmp> set_type;

/*========================================================================
 *
 * OStorePageDaemon_Impl interface.
 *
 *======================================================================*/
#ifdef _USE_NAMESPACE
namespace store {
#endif

class OStorePageDaemon_Impl :
	public NAMESPACE_VOS(OThread)
{
	VOS_DECLARE_CLASSINFO (VOS_NAMESPACE (OStorePageDaemon_Impl, store));

public:
	OStorePageDaemon_Impl (void);
	virtual ~OStorePageDaemon_Impl (void);

	sal_Bool empty (void);

	sal_Bool insert (key_type k);
	sal_Bool remove (key_type k);

	void flush (void);

private:
	/** Representation.
	 */
	NAMESPACE_VOS(OCondition) m_aCond;
	NAMESPACE_VOS(OMutex)     m_aMutex;
	set_type                  m_aSet;

	/** OThread.
	 */
	virtual void SAL_CALL run (void);

	/** Not implemented.
	 */
	OStorePageDaemon_Impl (const OStorePageDaemon_Impl&);
	OStorePageDaemon_Impl& operator= (const OStorePageDaemon_Impl&);
};

#ifdef _USE_NAMESPACE
}
#endif

/*========================================================================
 *
 * OStorePageDaemon implementation.
 *
 *======================================================================*/
VOS_IMPLEMENT_CLASSINFO(
	VOS_CLASSNAME (OStorePageDaemon, store),
	VOS_NAMESPACE (OStorePageDaemon, store),
	VOS_NAMESPACE (OTimer, vos),
	0);

/*
 * The single instance.
 */
NAMESPACE_STORE(OStorePageDaemon)*
NAMESPACE_STORE(OStorePageDaemon)::m_pThis = NULL;

/*
 * OStorePageDaemon.
 */
OStorePageDaemon::OStorePageDaemon (void)
{
	m_pImpl = new OStorePageDaemon_Impl();
	m_pThis = this;
}

/*
 * ~OStorePageDaemon.
 */
OStorePageDaemon::~OStorePageDaemon (void)
{
	NAMESPACE_VOS(IMutex) &rMutex = getGlobalMutex();

	rMutex.acquire();
	m_pThis = NULL;
	rMutex.release();

	OTimer::stop();
	delete m_pImpl;
}

/*
 * getOrCreate.
 */
sal_Bool OStorePageDaemon::getOrCreate (
	NAMESPACE_VOS(ORef)<OStorePageDaemon> &rxThis)
{
	NAMESPACE_VOS(OGuard) aGuard (getGlobalMutex());

	if (!m_pThis)
		new OStorePageDaemon();
	rxThis = m_pThis;

	return rxThis.isValid();
}

/*
 * getGlobalMutex.
 */
NAMESPACE_VOS(IMutex)& OStorePageDaemon::getGlobalMutex (void)
{
	static NAMESPACE_VOS(IMutex) *pMutex = NULL;
	if (!pMutex)
	{
		NAMESPACE_VOS(OGuard) aGuard (NAMESPACE_VOS(OMutex)::getGlobalMutex());
		if (!pMutex)
		{
			static NAMESPACE_VOS(OMutex) aGlobalMutex;
			pMutex = &aGlobalMutex;
		}
	}
	return *pMutex;
}

/*
 * insert.
 */
sal_Bool OStorePageDaemon::insert (key_type k)
{
	sal_Bool result = m_pImpl->insert(k);
	if (result && !m_pImpl->isRunning() && !isTicking())
	{
		// Start timer.
		OTimer::setRemainingTime (NAMESPACE_VOS(TTimeValue)(10000));
		OTimer::start();
	}
	return result;
}

/*
 * remove.
 */
sal_Bool OStorePageDaemon::remove (key_type k)
{
	sal_Bool result = m_pImpl->remove(k);
	if (result && m_pImpl->empty())
	{
		// Stop timer.
		OTimer::stop();
	}
	return result;
}

/*
 * onShot.
 */
void SAL_CALL OStorePageDaemon::onShot (void)
{
	if (!m_pImpl->isRunning() && !m_pImpl->create())
	{
		// Thread creation failure. Need to flush here and now.
		m_pImpl->flush();

		// Restart timer.
		OTimer::setRemainingTime (NAMESPACE_VOS(TTimeValue)(10000));
		OTimer::start();
	}
}

/*========================================================================
 *
 * OStorePageDaemon_Impl implementation.
 *
 *======================================================================*/
VOS_IMPLEMENT_CLASSINFO(
	VOS_CLASSNAME (OStorePageDaemon_Impl, store),
	VOS_NAMESPACE (OStorePageDaemon_Impl, store),
	VOS_NAMESPACE (OThread, vos),
	0);

/*
 * OStorePageDaemon_Impl.
 */
OStorePageDaemon_Impl::OStorePageDaemon_Impl (void)
{
	m_aCond.reset();
}

/*
 * ~OStorePageDaemon_Impl.
 */
OStorePageDaemon_Impl::~OStorePageDaemon_Impl (void)
{
	if (isRunning())
	{
		// Terminate.
		terminate();
		m_aCond.set();

		// Join.
		if (getCurrentIdentifier() != getIdentifier())
			join();
	}
}

/*
 * empty.
 */
sal_Bool OStorePageDaemon_Impl::empty (void)
{
	NAMESPACE_VOS(OGuard) aGuard (m_aMutex);
	return m_aSet.empty();
}

/*
 * insert.
 */
sal_Bool OStorePageDaemon_Impl::insert (key_type k)
{
	NAMESPACE_VOS(OGuard) aGuard (m_aMutex);
	if (k)
	{
		typedef NAMESPACE_STD(pair)<set_type::iterator, bool> result_t;
		result_t result = m_aSet.insert (k);
		if (!result.second)
			k = 0;
	}
	return (!!k);
}

/*
 * remove.
 */
sal_Bool OStorePageDaemon_Impl::remove (key_type k)
{
	NAMESPACE_VOS(OGuard) aGuard (m_aMutex);

	set_type::iterator it = m_aSet.find (k);
	if (it != m_aSet.end())
	{
		m_aSet.erase (it);
		return sal_True;
	}
	return sal_False;
}

/*
 * flush.
 */
void OStorePageDaemon_Impl::flush (void)
{
	m_aMutex.acquire();
	set_type aSet (m_aSet);
	m_aMutex.release();

	set_type::const_iterator first = aSet.begin();
	set_type::const_iterator last  = aSet.end();

	while (schedule() && (first != last))
	{
		key_type k = (key_type)(*first++);
		if (k)
		{
			// Verify and Acquire.
			NAMESPACE_VOS(OGuard) aGuard (m_aMutex);

			set_type::const_iterator it = m_aSet.find(k);
			if (it != m_aSet.end())
				k->acquire();
			else
				k = 0;
		}
		if (k)
		{
			// Flush and Release.
			k->flush();
			k->release();
		}
	}
}

/*
 * run.
 */
void SAL_CALL OStorePageDaemon_Impl::run (void)
{
	setPriority (TPriority_BelowNormal);

	m_aCond.set();
	while (schedule())
	{
		TimeValue tmo;
		tmo.Seconds = 10;
		tmo.Nanosec = 0;

		m_aCond.wait(&tmo);
		flush();
		m_aCond.reset();
	}
}

