/*************************************************************************
 *
 *  $RCSfile: config.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:58:50 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef	_SVCFGMGR_HXX
#define	_SVCFGMGR_HXX

#include <tools/string.hxx>

// forward ---------------------------------------------------------------

struct SvConfigFileItem_Impl;

class SvConfigFileItemArr_Impl;
class SvConfigItem;
class SvStream;
class SvFileStream;
class SfxBroadcaster;

// class SvConfigManager -------------------------------------------------

class SvConfigManager
{
private:
	String						aTempFileName;
	SvConfigFileItemArr_Impl*	pFileItemArr;
	SvFileStream*				pStream;
	SvConfigManager*			pParentMgr;
	ULONG						nErrno;
	long						lStart;
	long						lDirPos;
	BOOL						bActive;
	BOOL						bModified;
	BOOL						bFileIO;
	BOOL						bOwnStream;
#if __PRIVATE
					// Methoden auf FileItem-Basis
	BOOL			RemoveItem(SvConfigFileItem_Impl*);
	BOOL			CopyItem(SvConfigFileItem_Impl*, SvConfigManager *pMgr);
	BOOL			ReconfigureItem(SvConfigFileItem_Impl*);
	BOOL			LoadConfig(SvConfigFileItem_Impl*, SvConfigItem*p=NULL);
	BOOL			StoreConfig(SvConfigFileItem_Impl*);

	void			WriteHeader(SvStream*);
#endif
					// Kopieren verboten
					SvConfigManager(const SvConfigManager &);
					const SvConfigManager &operator=(const SvConfigManager &);

protected:

	virtual void 	RemovingItem( USHORT );
	virtual void 	CopyingItem( USHORT&, const String& );
	virtual void 	StartReconfiguring();
	virtual void 	FinishedReconfiguring();

#ifdef DBG_UTIL
	void			CheckConfiguration(SvStream*);
#endif

public:

	enum Error {
		ERR_NO,
		ERR_READ,
		ERR_WRITE,
		ERR_OPEN,
		ERR_FILETYPE,
		ERR_VERSION
	};
					SvConfigManager( SvConfigManager* pMgr=0 );
					~SvConfigManager();

	void			SetParent( SvConfigManager* pMgr )	{ pParentMgr = pMgr; }
	SvConfigManager* GetParentMgr() const				{ return pParentMgr; }
	ULONG			GetErrorCode() const 				{ return nErrno; }
	void 			SetError( ULONG n )					{ nErrno = n; }
	BOOL			IsActive() const					{ return bActive; }

	BOOL			LoadConfig();
	BOOL			StoreConfig();
	BOOL			ReadConfiguration(SvStream* pStr);
	BOOL			SaveConfiguration(SvStream* pStr);
	void			CallUseDefault();

	USHORT			GetItemCount() const;
	String			GetItem(USHORT nPos) const;
	USHORT  		GetItemId( const String& rName ) const;

	BOOL			Remove(USHORT nType);
	BOOL			RemoveItem(USHORT nPos);
	BOOL			CopyItem(USHORT nPos, USHORT& rPos, SvConfigManager *pMgr);
	BOOL			CopyItem(USHORT nType, SvConfigManager *pMgr, BOOL bSearch = FALSE);
	BOOL			CopyItem(SvConfigItem*);
	BOOL			CopyItems(SvConfigManager *pCfgMgr);

	BOOL			AddConfigItem(SvConfigItem *);
	BOOL			AddConfigItem(SvConfigItem *, SfxBroadcaster* pTask);
	void			InsertConfigItem(USHORT nTyp);
	BOOL			RemoveConfigItem(SvConfigItem *);
	void			ResetConfigItem(USHORT);
	BOOL			ReconfigureItem(SvConfigItem*);
	BOOL			ReconfigureItems(SvConfigManager*);

	BOOL			LoadConfig(SvConfigItem *);
	BOOL			StoreConfig(SvConfigItem *);

	BOOL    		HasConfigItem(USHORT nTyp);
	BOOL			CanDelete(USHORT nPos) const;
	BOOL			IsModified();
	virtual void 	SetModified(BOOL);
	virtual void	Activate(SvConfigManager *pMgr);
	void 			ActivateTask(SfxBroadcaster* pTask);

	void 			DoBackup( SvStream* );
	void 			DoRestore( SvStream* );
};

#endif // #ifndef	_SVCFGMGR_HXX


