/*************************************************************************
 *
 *  $RCSfile: confitem.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:58:50 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef	_SVCFGITEM_HXX
#define	_SVCFGITEM_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

class SvConfigManager;
class SvStream;
class String;
class SfxBroadcaster;

// -----------------------------------------------------------------------

class SvConfigItem

/*	[Beschreibung]

	Basisklasse fr Objekte, die Daten in der Konfigurationsdatei
	ablegen und von dort wieder laden m"ochten. Je ein Objekt der
	abgeleiteten Klassen mu in SfxApplication::Init() angelegt
	werden.

	Wenn die Konfigurationsdatei geladen wird, werden diese Objekte
	benachrichtigt (Load()) und k"onnen ihren Teil laden, wurde
	dieses Objekt in der Konfigurationsdatei nicht gefunden, wird
	das Objekt dar"uber auch benachrichtigt (UseDefault()) und
	erh"alt somit die Mglichkeit, eine Default-Konfiguration z.B.
	aus der Ressource zu laden. Beim Speichern der Konfigurationsdatei
	wird den Objekten mit Store() die M"oglichkeit gegeben, ihren
	Teil der Konfiguration zu schreiben.


	[Anmerkung]

	Im StarFramework selbst, gibt es z.B. eine Klasse
	SfxMenuManager, die von SvConfigItem abgeleitet ist. Darber
	wird die vom Anwender eingestellte Men-Konfiguration geladen
	und gespeichert.

	[Querverweise]
	<SfxApplication::LoadConfig()>
	<SfxApplication::StoreConfig()>
*/

{
private:
	const USHORT		nType;
	BOOL				bDefault;
	BOOL				bModified;
	SvConfigManager*	pCfgMgr;
	BOOL				bInternal : 1;
	BOOL 				bIndividual : 1;

protected:

	// An- / Abmelden beim ConfigManager
						SvConfigItem(USHORT nType, SvConfigManager* pMgr, SfxBroadcaster* pTask);
						SvConfigItem(USHORT nType, SvConfigManager* pMgr);
	virtual 			~SvConfigItem();

public:
	enum {
		ERR_OK,
		WARNING_VERSION,
		ERR_READ
	};
	virtual int 		Load(SvStream&) = 0;

	// File Pointer muss nach dem Schreiben am Ende
	// des geschriebenen Bereiches stehen; dort wird
	// der naechste Eintrag geschrieben.
	virtual BOOL		Store(SvStream&)   = 0;
	virtual BOOL		Reconfigure(SvStream&, BOOL bDefault=FALSE);

	// Defaultimplementierung liefert Leerstring
	virtual String		GetName() const;
			BOOL		Internal() const
						{ return bInternal; }
			BOOL		Individual() const
						{ return bIndividual; }
			void		SetInternal (BOOL bInt)
						{ bInternal = bInt; }
			void 		SetIndividual( BOOL bInd )
						{ bIndividual = bInd; }
	virtual void		UseDefault();
			BOOL		IsDefault() const { return bDefault; }
			void		SetDefault(BOOL bOn = TRUE);
			void		SetDefault_Impl(BOOL bOn = TRUE)
						{ bDefault = bOn; }
			void		SetModified(BOOL bSet = TRUE);
			BOOL		IsModified() { return bModified; }
			BOOL		Initialize();
			BOOL		ReInitialize(SvConfigManager *pMgr);
			BOOL		StoreConfig(BOOL bOn = TRUE);
			void		Connect (SvConfigManager *pMgr)
						{ pCfgMgr = pMgr; }
			void		ReleaseConfigManager()
						{ pCfgMgr = 0; }
	SvConfigManager*	GetConfigManager() const
						{ return pCfgMgr; }


	USHORT	GetType() const { return nType; }
};

#endif

