/*************************************************************************
 *
 *  $RCSfile: useroptions.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:58:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef INCLUDED_SVTOOLS_USEROPTIONS_HXX
#define INCLUDED_SVTOOLS_USEROPTIONS_HXX

#ifndef _UTL_CONFIGITEM_HXX_
#include <unotools/configitem.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif

#define USER_COMPANY			0
#define USER_FIRSTNAME			1
#define USER_LASTNAME			2
#define USER_ID					3
#define USER_STREET				4
#define USER_CITY				5
#define USER_STATE				6
#define USER_ZIP				7
#define USER_COUNTRY			8
#define USER_POSITION			9
#define USER_TITLE				10
#define USER_TELEPHONEHOME		11
#define USER_TELEPHONEWORK		12
#define USER_FAX				13
#define USER_EMAIL				14
#define USER_CUSTOMERNUMBER		15

class SvtUserOptions : public utl::ConfigItem
{
private:
	String			m_aCompany;
	String			m_aFirstName;
	String			m_aLastName;
	String			m_aID;
	String			m_aStreet;
	String			m_aCity;
	String			m_aState;
	String			m_aZip;
	String			m_aCountry;
	String			m_aPosition;
	String			m_aTitle;
	String			m_aTelephoneHome;
	String			m_aTelephoneWork;
	String			m_aFax;
	String			m_aEmail;
	String			m_aCustomerNumber;

	String			m_aEmptyString;

	::osl::Mutex	m_aMutex;

	static SvtUserOptions*	m_pOptions;

	com::sun::star::uno::Sequence< rtl::OUString > GetPropertyNames();

	// not const because of using a mutex
	const String&	GetUserToken( int nIndex );
	void			SetUserToken( int nIndex, const String& rNewToken );

	SvtUserOptions();

public:
	virtual ~SvtUserOptions();

	static SvtUserOptions*	GetUserOptions();
	static void				DestroyUserOptions();

	virtual void 	Notify( const com::sun::star::uno::Sequence< rtl::OUString >& aPropertyNames );
	virtual void	Commit();

	// get the user token, not const because of using a mutex
	const String&	GetCompany() { return GetUserToken( USER_COMPANY ); }
	const String&	GetFirstName() { return GetUserToken( USER_FIRSTNAME ); }
	const String&	GetLastName() { return GetUserToken( USER_LASTNAME ); }
	const String&	GetID() { return GetUserToken( USER_ID ); }
	const String&	GetStreet() { return GetUserToken( USER_STREET ); }
	const String&	GetCity() { return GetUserToken( USER_CITY ); }
	const String&	GetState() { return GetUserToken( USER_STATE ); }
	const String&	GetZip() { return GetUserToken( USER_ZIP ); }
	const String&	GetCountry() { return GetUserToken( USER_COUNTRY ); }
	const String&	GetPosition() { return GetUserToken( USER_POSITION ); }
	const String&	GetTitle() { return GetUserToken( USER_TITLE ); }
	const String&	GetTelephoneHome() { return GetUserToken( USER_TELEPHONEHOME ); }
	const String&	GetTelephoneWork() { return GetUserToken( USER_TELEPHONEWORK ); }
	const String&	GetFax() { return GetUserToken( USER_FAX ); }
	const String&	GetEmail() { return GetUserToken( USER_EMAIL ); }
	const String&	GetCustomerNumber() { return GetUserToken( USER_CUSTOMERNUMBER ); }

	// set the address token
	void			SetCompany( const String& rNewToken ) { SetUserToken( USER_COMPANY, rNewToken ); }
	void			SetFirstName( const String& rNewToken ) { SetUserToken( USER_FIRSTNAME, rNewToken ); }
	void			SetLastName( const String& rNewToken ) { SetUserToken( USER_LASTNAME, rNewToken ); }
	void			SetID( const String& rNewToken ) { SetUserToken( USER_ID, rNewToken ); }
	void			SetStreet( const String& rNewToken ) { SetUserToken( USER_STREET, rNewToken ); }
	void			SetCity( const String& rNewToken ) { SetUserToken( USER_CITY, rNewToken ); }
	void			SetState( const String& rNewToken ) { SetUserToken( USER_STATE, rNewToken ); }
	void			SetZip( const String& rNewToken ) { SetUserToken( USER_ZIP, rNewToken ); }
	void			SetCountry( const String& rNewToken ) { SetUserToken( USER_COUNTRY, rNewToken ); }
	void			SetPosition( const String& rNewToken ) { SetUserToken( USER_POSITION, rNewToken ); }
	void			SetTitle( const String& rNewToken ) { SetUserToken( USER_TITLE, rNewToken ); }
	void			SetTelephoneHome( const String& rNewToken )
						{ SetUserToken( USER_TELEPHONEHOME, rNewToken ); }
	void			SetTelephoneWork( const String& rNewToken )
						{ SetUserToken( USER_TELEPHONEWORK, rNewToken ); }
	void			SetFax( const String& rNewToken ) { SetUserToken( USER_FAX, rNewToken ); }
	void			SetEmail( const String& rNewToken ) { SetUserToken( USER_EMAIL, rNewToken ); }
	void			SetCustomerNumber( const String& rNewToken )
						{ SetUserToken( USER_CUSTOMERNUMBER, rNewToken ); }
};

#define SVT_USEROPTIONS()	SvtUserOptions::GetUserOptions()

#endif // #ifndef INCLUDED_SVTOOLS_USEROPTIONS_HXX

