/*************************************************************************
 *
 *  $RCSfile: libcall.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:58:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#if defined ( WIN ) || defined (WNT)
#include <tools/svwin.h>
#include <stdlib.h>
#endif


#ifdef OS2
#define INCL_DOSMODULEMGR
#include <tools/svpm.h>
#endif


#ifdef MAC
#include <mac_start.h>

#ifndef __FILES__
  #include <Files.h>
#endif
#ifndef __CODEFRAGMENTS__
  #include <CodeFragments.h>
#endif
#ifndef __PROCESSES__
  #include <Processes.h>
#endif
#ifndef __ALIASES__
  #include <Aliases.h>
#endif
#ifndef __RESOURCES__
  #include <Resources.h>
#endif
#ifndef __FOLDERS__
  #include <Folders.h>
#endif
#ifndef __FINDER__
  #include <Finder.h>
#endif
#include <mac_end.h>
#include <tools/debug.hxx>
#endif

#ifndef _SOLAR_HRC
#include <solar.hrc>
#endif

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif

#ifdef MAC
 #ifndef _SV_SVAPP_HXX
  #include <vcl/svapp.hxx>
 #endif
#endif

#ifndef _LIBCALL_HXX
#include <libcall.hxx>
#endif

#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

//=========================================================================
//======================UNIX-Variante======================================
//=========================================================================

#if defined UNX

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#ifdef AIX
#ifdef NOTAIX42
#include <errno.h>
#include <sys/types.h>
#include <sys/ldr.h>
#include <a.out.h>
#include <ldfcn.h>
#define _H_FCNTL
#else
#include <dlfcn.h>
#endif
#define LIBPATH "LIBPATH"
#elif defined HPUX
#include <dl.h>
#include <errno.h>
#define LIBPATH "SHLIB_PATH"
#elif defined S390
#include <errno.h>
#include <dll.h>
#define LIBPATH "LIPPATH"
#elif defined IRIX
#include <dlfcn.h>
#define LIBPATH "LD_LIBRARYN32_PATH"
#elif defined MACOSX
#include <CoreFoundation/CoreFoundation.h>
#define LIBPATH "DYLD_LIBRARY_PATH"
#else
#include <dlfcn.h>
#define LIBPATH "LD_LIBRARY_PATH"
#endif

#if defined AIX && defined NOTAIX42
struct SymbolEntry
{
	ByteString aSymbolName;
	void *aSymbolAdress;
};
DECLARE_LIST(SymbolTable, SymbolEntry*);
#endif

struct LibraryPrivateData
{
	ByteString	aLibname;
	ULONG		nRefCount;

#ifdef HPUX
	shl_t aHandle;
#elif S390
	dllhandle *aHandle;
#else
	void *aHandle;
#endif

#if defined AIX && defined NOTAIX42
	SymbolTable aSymbolTable;
#endif

#if (defined AIX && defined NOTAIX42)
	void (*init)(void);
	void (*exit)(void);
#endif

	LibraryPrivateData( const ByteString & aName )
		: aLibname (aName), nRefCount (1), aHandle (NULL)
#if (defined AIX && defined NOTAIX42)
			, init (NULL), exit (NULL)
#endif
	{}
};

static ByteString aError;

DECLARE_LIST(LibraryList, LibraryPrivateData*);

static LibraryList aLibraryList;

#if (defined AIX && defined NOTAIX42) || defined HPUX
inline void SetError (const ByteString pError, BOOL bErrno = FALSE)
{
	aError = pError;
	if (bErrno)
	{
		aError.Append( " ", 1 );
		aError.Append( strerror(errno) );
	}
	aError += '\n';
#ifdef DBG_UTIL
	DBG_ASSERTWARNING ( TRUE, aError.GetBuffer() );
#endif
}
#endif

inline String FindLibrary( const String &rPathname )
{
	ByteString aPathName( rPathname, RTL_TEXTENCODING_UTF8 );
	if ( aPathName.Search('/') != STRING_NOTFOUND )
		return access( aPathName.GetBuffer(), F_OK | R_OK )
				? String() : rPathname;
	
	const char* pEnv = getenv( LIBPATH );
	ByteString aLibpath( pEnv ? pEnv : "" );
	ByteString aPath;
	USHORT nToken = aLibpath.GetTokenCount(':');
	for (USHORT i = 0; i < nToken; i++)
	{
		aPath = aLibpath.GetToken (i, ':');
		aPath.Append( "/", 1 );
		aPath.Append( aPathName );
		if (! access( aPath.GetBuffer(), F_OK | R_OK ) )
			return String( aPath, RTL_TEXTENCODING_UTF8 );
	}
	return String();
}

#ifdef HPUX

inline void *FindFunction(LibraryPrivateData *pPriv, const String & rSymbol)
{
	void *pReturn;
	ByteString aSymbol( rSymbol, RTL_TEXTENCODING_UTF8 );
	if (shl_findsym( &pPriv->aHandle, aSymbol.GetBuffer(),
										TYPE_PROCEDURE, &pReturn) == -1)
	{
		SetError("FindFunction", TRUE);
		pReturn = NULL;
	}
	return pReturn;
}

inline BOOL Load(LibraryPrivateData *pPriv, const String & rPathname)
{
	ByteString aPathName( rPathName, RTL_TEXTENCODING_UTF8 );
	pPriv->aHandle = shl_load( aPathName.GetBuffer(),
				(BIND_IMMEDIATE | BIND_NONFATAL | DYNAMIC_PATH), 0L);

	//pPriv->aHandle = shl_load(rPathname.GetStr(),
	//			(BIND_DEFERRED | BIND_NONFATAL | DYNAMIC_PATH), 0L);

	BOOL ret = (pPriv->aHandle != NULL);
	if (!ret)
		SetError("Load:", TRUE);
	return (ret);
}

inline BOOL Unload (LibraryPrivateData *pPriv) {
	BOOL ret = TRUE;
	if (pPriv)
	{
		ret = (shl_unload (pPriv->aHandle) != -1);
		if (! ret)
			SetError("Unload:", TRUE);
	}
	return (ret);
}

#elif defined AIX && defined NOTAIX42

inline struct ld_info *GetInfo (void)
{
	for (ULONG size = 4096; TRUE; size += 4096)
	{
		char *pBuffer = new char [size];
		if (! pBuffer) {
			SetError("GetInfo: out of memory");
			return NULL;
		}
		if (loadquery (L_GETINFO, pBuffer, size) != -1)
			return ((struct ld_info *) pBuffer);
		delete[] pBuffer;
		if (errno != ENOMEM)
			return NULL;
	}
}

class MainEntry
{
	void *pPoint;
	MainEntry ()
	{
		struct ld_info *pLdInfo = GetInfo ();
		if (pLdInfo)
		{
			pPoint =  pLdInfo->ldinfo_dataorg;
			delete[] pLdInfo;
		}
		else
		{
			pPoint = NULL;
			SetError("MainEntryPoint: cannot find");
		}
	}
}

static MainEntry aMainEntryPoint;

inline void *FindFunction(LibraryPrivateData *pPriv, const String & rFuncName )
{
	ByteString aFuncName( rFuncName, RTL_TEXTENCODING_UTF8 );
	SymbolEntry *pSymbolEntry;
	for (pSymbolEntry = pPriv->aSymbolTable.First();
			pSymbolEntry && ( pSymbolEntry->aSymbolName.CompareTo( aFuncName ) != COMPARE_EQUAL );
			pSymbolEntry = pPriv->aSymbolTable.Next())
		;
	void *ret = pSymbolEntry ? pSymbolEntry->aSymbolAdress : NULL;
	if (ret)
	{
		ByteString aTmp("FindFunction: symbol ");
		aTmp.Append( aFuncName );
		aTmp.Append( " not found" );
		SetError( aTmp.GetBuffer() );
	}
	return ret;
}

inline BOOL Load(LibraryPrivateData *pPriv, const String & rPathname)
{
	String aPathName( FindLibrary( rPathname ) );
	if ( !aPathName.Len() )
	{
		ByteString aTmp( "Load: " );
		aTmp.Append( ByteStream( rPathName, RTL_TEXTENCODING_UTF8 ) );
		aTmp.Append( " not found" );
		SetError( aTmp.GetBuffer() );
		return FALSE;
	}
	pPriv->aHandle = load ( ByteString(aPathName, RTL_ENCODING_UTF8).GetBuffer(), L_NOAUTODEFER, NULL);
	if (! pPriv->aHandle)
	{
		ByteString aTmp( "LOad: load:" );
		aTmp.Append( ByteString( rPathname, RTL_ENCODING_UTF8 );
		aTmp.Append( ":" );
		SetError ( aTmp.GetBuffer(), TRUE);
		return FALSE;
	}

	if ( loadbind(NULL, aMainEntryPoint.aPoint , pPriv->aHandle) == -1)
	{
		unload (pPriv->aHandle);
		SetError( "Load: loadbind:", TRUE );
		return FALSE;
	}
	LDFILE *pLdFile = ldopen ( ByteString(aPathName, RTL_TEXTENCODING_UTF8).GetBuffer(), NULL);
	if (! pLdFile)
	{
		struct ld_info *pLdInfo;
		if (!(pLdInfo = GetInfo()));
		{
			SetError("Load: cannot find symbol table");
			return FALSE;
		}
		while (pLdInfo && (! pLdFile))
		{
			if (pLdInfo->ldinfo_dataorg == pPriv->aHandle)
				pLdFile = ldopen (pLdInfo->ldinfo_filename,pLdFile);
			else
				pLdInfo = pLdInfo->ldinfo_next ?
						(struct ld_info *)((char *) pLdInfo) +
									pLdInfo->ldinfo_next :
						NULL;
		}
		delete[] pLdInfo;
		if (! pLdFile)
		{
			SetError("Library::Library:");
			return FALSE;
		}
	}
	if (TYPE(pLdFile) != U802TOCMAGIC)
	{
		while (ldclose (pLdFile) == FAILURE)
			;
		SetError("Library::Library: bad magic");
		return FALSE;
	}

	SCNHDR aSectionHeader;
	if (ldnshread (pLdFile, _LOADER, &aSectionHeader) != SUCCESS)
	{
		while (ldclose (pLdFile) == FAILURE)
			;
		SetError("Library::Library: cannot read loader section header");
		return FALSE;
	}
	if (FSEEK(pLdFile, aSectionHeader.s_scnptr, BEGINNING) != OKFSEEK) {
		while (ldclose (pLdFile) == FAILURE)
			;
		SetError("Load: cannot seek to loader section");
		return FALSE;
	}
	char *pBuffer = new char [aSectionHeader.s_size];
	if (FREAD(pBuffer, aSectionHeader.s_size, 1, pLdFile) != 1)
	{
		while (ldclose (pLdFile) == FAILURE)
			;
		SetError("Load: cannot read loader section");
		return FALSE;
	}
	int i = ((LDHDR *)pBuffer)->l_nsyms;
	for (LDSYM *pSymbol = (LDSYM*) (pBuffer + LDHDRSZ); i--; pSymbol++)
	{
		if (LDR_EXPORT (*pSymbol))
		{
			SymbolEntry *pSymbolEntry = new SymbolEntry;

			pSymbolEntry->aSymbolName = ByteString( pSymbol->l_zeroes ?
				pSymbol->l_name :
				pSymbol->l_offset + ((LDHDR *)pBuffer)->l_stoff + pBuffer );

			pSymbolEntry->aSymbolAdress = (void *)((unsigned long)pPriv->aHandle + pSymbol->l_value);
			pPriv->aSymbolTable.Insert( pSymbolEntry );
		}
	}
	delete[] pBuffer;
	while(ldclose(pLdFile) == FAILURE)
		;

	struct dl_info { void (*init)(void); void (*exit)(void); }
			*pInfo = (struct dl_info *) FindFunction ("dl_info");
	if (pInfo)
	{
		pPriv->exit = pInfo->exit;
		pPriv->init = pInfo->init;
		if (pPriv->init)
			pPriv->init();
	}
	return TRUE;
}

inline BOOL Unload (LibraryPrivateData *pPriv)
{
	if (pPriv->exit)
		pPriv->exit();
	if (unload (pPriv->aHandle) == -1)
		SetError("Unload", TRUE);
	for (SymbolEntry *pSymbolEntry = pPriv->aSymbolTable.First();
				pSymbolEntry; pSymbolEntry = pPriv->aSymbolTable.Next())
		delete pSymbolEntry;
}

#elif S390

inline void *FindFunction(LibraryPrivateData *pPriv, const String & rFuncname)
{
	ByteString aFuncName( rFuncName, RTL_TEXTENCODING_UTF8 );
	void *pReturn = (void *)(dllqueryfn(pPriv->aHandle, ByteString(aFuncname, RTL_TEXTENCODING_UTF8).GetBuffer()));
	if (pReturn == NULL)
		aError = strerror(errno);
	return pReturn;
}

inline BOOL Load(LibraryPrivateData *pPriv, const String & rPathname)
{
	ByteString aPathName( rPathName, RTL_TEXTENCODING_UTF8 );
	pPriv->aHandle = dllload(ByteString(aPathName, RTL_TEXTENCODING_UTF8).GetBuffer());
	if (pPriv->aHandle == NULL)
		aError = strerror(errno);
	return (pPriv->aHandle != NULL);
}

inline BOOL Unload (LibraryPrivateData *pPriv)
{
	BOOL bRet = (dllfree(pPriv->aHandle) == 0);
	if (!bRet)
		aError = strerror(errno);
	return bRet;
}

#elif MACOSX

inline void *FindFunction(LibraryPrivateData *pPriv, const String & rFuncname)
{
	CFStringRef pSymbolName;
	void *pReturn;

	// Convert char pointer to CFStringRef
	ByteString aFuncName( rFuncname, RTL_TEXTENCODING_UTF8 );
	pSymbolName = CFStringCreateWithCString( NULL, aFuncName.GetBuffer(),
		kCFStringEncodingUTF8 );

	// Try to get the symbol
	pReturn = CFBundleGetFunctionPointerForName( (CFBundleRef)(pPriv->aHandle),
		pSymbolName );

	// Release CFStringRef
	if ( pSymbolName )
		CFRelease( pSymbolName );

	return pReturn;
}

inline BOOL Load(LibraryPrivateData *pPriv, const String & rPathname)
{
	CFStringRef pPath=0;
	CFURLRef pURL=0;
	CFBundleRef pLib=0;

	// Convert path in rPathname to CFStringRef
	ByteString aPathName( rPathname, RTL_TEXTENCODING_UTF8 );
	pPath = CFStringCreateWithCString( NULL, aPathName.GetBuffer(),
		kCFStringEncodingUTF8 );

	// Get the framework's CFURLRef using its path
	if ( ! (pURL = CFURLCreateWithFileSystemPath( kCFAllocatorDefault,
		pPath, kCFURLPOSIXPathStyle, true) ) ) {
		aError = "Cannot load module ";
		aError += aPathName.GetBuffer();
		aError += " path does not exist";
		CFRelease(pPath);
		return FALSE;
	}

	// Load the framework
	if ( ! ( pLib = CFBundleCreate( NULL, pURL) ) ) {
		aError = "Cannot load module ";
		aError += aPathName.GetBuffer();
		aError += " path is not a bundle";
		CFRelease( pPath );
		CFRelease( pURL );
		return FALSE;
	}

	// Load the library in the framework
	if ( CFBundleLoadExecutable( pLib ) ) {
		CFRelease( pPath );
		CFRelease( pURL );
		pPriv->aHandle = (void *)pLib;
	}
	else {
		aError = "Cannot load module ";
		aError += aPathName.GetBuffer();
		aError += " bundle does not contain a valid library";
		CFRelease( pPath );
		CFRelease( pURL );
		CFRelease( pLib );
		pPriv->aHandle = NULL;
	}

	return (pPriv->aHandle != NULL);
}

inline BOOL Unload (LibraryPrivateData *pPriv)
{
	if ( pPriv->aHandle ) {
		CFRelease((CFBundleRef)(pPriv->aHandle));
		return TRUE;
	}
	else
		return FALSE;
}

#else

inline void *FindFunction(LibraryPrivateData *pPriv, const String & rFuncname)
{
	ByteString aFuncName( rFuncname, RTL_TEXTENCODING_UTF8 );
	void *pReturn = dlsym(pPriv->aHandle, aFuncName.GetBuffer());
	if ( !pReturn ) 
		aError = dlerror();
	return pReturn;
}

inline BOOL Load(LibraryPrivateData *pPriv, const String & rPathname)
{
	ByteString aPathName( rPathname, RTL_TEXTENCODING_UTF8 );
	pPriv->aHandle = dlopen(aPathName.GetBuffer(), RTLD_LAZY);
	if ( !pPriv->aHandle )
		aError = dlerror();
	return (pPriv->aHandle != NULL);
}

inline BOOL Unload (LibraryPrivateData *pPriv)
{
#if defined LINUX
	// gcc 2.9.1 (egcs) registers atexit handlers which obviously cannot
	// be called after dlclose. A compiler "feature". The workaround for now
	// is not to dlclose libraries. Since most of them are closed at shutdown
	// this does not make that much a difference
	return TRUE;
#else
	BOOL ret = (dlclose(pPriv->aHandle) != 0);
	if ( ret ) 
		aError = dlerror();
	return ret;
#endif
}

#endif

BOOL Library::ModulFound( const String &rPathname )
{
	return FindLibrary ( rPathname ).Len();
}

Library::Library(const String & rPathname)
{	
	ByteString aLibname( rPathname, RTL_TEXTENCODING_UTF8 );
	
	char *pLibname = strrchr (const_cast<char*>(aLibname.GetBuffer()), '/');
	if (pLibname)
		aLibname = ByteString( pLibname );

	for ( pPriv = aLibraryList.First();
				pPriv && ( pPriv->aLibname != aLibname );
				pPriv = aLibraryList.Next() )
		;
	if (pPriv)
	{
		pPriv->nRefCount++;
		return;
	}
	pPriv = new LibraryPrivateData(aLibname);
	if (Load(pPriv, rPathname))
		aLibraryList.Insert(pPriv);
	else
	{
		delete pPriv;
		pPriv = NULL;
	}
}

Library::~Library()
{
	if( pPriv && !(--pPriv->nRefCount) )
	{
		Unload (pPriv);
		aLibraryList.Remove(pPriv);

		delete pPriv;
		pPriv = NULL;
	}
}

void* Library::GetFunction(const String & rFuncName) const
{
	return pPriv ? FindFunction(pPriv, rFuncName) : NULL;
}

BOOL Library::ModulFound () const {
	return ((pPriv != NULL) && (pPriv->aHandle != NULL));
}

#endif
//=========================================================================
//======================Windows-Variante===================================
//=========================================================================

#if defined (WIN) || defined (WNT)

struct LibraryPrivateData {
	HINSTANCE hinst;
};

Library::Library( const String& rPathname )
{
	ByteString aPathName( rPathname, RTL_TEXTENCODING_UTF8 );
	const sal_Char*	pPathName = aPathName.GetBuffer();
	pPriv = new LibraryPrivateData;
	pPriv->hinst = LoadLibrary( pPathName );
#ifdef DEBUG
	if( !pPriv->hinst )
	{
		char	aErrMess[250];
		DWORD	nLastError = GetLastError();
		FormatMessage( FORMAT_MESSAGE_ARGUMENT_ARRAY|FORMAT_MESSAGE_FROM_SYSTEM,
			NULL, nLastError, MAKELANGID(LANG_NEUTRAL,SUBLANG_SYS_DEFAULT),
			aErrMess, 250, (char**)&pPathName );
		DBG_ASSERT( pPriv->hinst!=NULL, aErrMess );
	}
#endif
}

Library::~Library()
{
	if (pPriv->hinst > (HINSTANCE) HINSTANCE_ERROR) FreeLibrary(pPriv->hinst);
	delete pPriv;
}

BOOL Library::ModulFound() const
{
	if (pPriv->hinst <= (HINSTANCE) HINSTANCE_ERROR) return FALSE;
	else return TRUE;
}

void * Library::GetFunction(const String & rFuncName) const
{
	if (pPriv->hinst <= (HINSTANCE) HINSTANCE_ERROR)
		return NULL;
	ByteString aFuncName( rFuncName, RTL_TEXTENCODING_UTF8 );	
	return (void*) GetProcAddress(pPriv->hinst, (LPCSTR)aFuncName.GetBuffer() );
}

BOOL Library::ModulFound(const String & rPathname)
{
	ByteString aPathName( rPathname, RTL_TEXTENCODING_UTF8 );
#ifdef WIN
	if( GetModuleHandle( aPathName.GetBuffer() ) ) return TRUE;
	OFSTRUCT aOS;
	OpenFile( aPathname.GetBuffer(), &aOS, OF_EXIST );
	return aOS.nErrCode == 0;
#else
	OFSTRUCT	aOS;
	USHORT		n = aPathName.Len();
	const char*	p = aPathName.GetBuffer();

	while( n && p[n-1] != '\\' && p[n-1] != '.' && p[n-1] != '/' )
		n--;

	if( n && p[n-1] == '.' )
		OpenFile( p, &aOS, OF_EXIST );
	else
	{
		aPathName.Append( ".dll", 4 );
		OpenFile( aPathName.GetBuffer(), &aOS, OF_EXIST );
	}
	return aOS.nErrCode == 0;
#endif
}

#endif

//=========================================================================
//========================OS2-Variante=====================================
//=========================================================================

#ifdef OS2

struct LibraryPrivateData {
	HMODULE 			hmod;
	ULONG				nError;
	String				aPathName;
	ULONG				nCount;
	LibraryPrivateData *pNext;
};

static LibraryPrivateData *pFirst;

Library::Library(const String & rPathName)
{
	pPriv = pFirst;
	ByteString aPathName( rPathName, RTL_TEXTENCODING_UTF8 );
	while( pPriv && pPriv->aPathName != aPathName )
		pPriv = pPriv->pNext;

	if( !pPriv ) {
		pPriv				= new LibraryPrivateData;
		pPriv->nError		= DosLoadModule(NULL, 0,
									(PSZ)aPathName.GetBuffer(),&(pPriv->hmod));
		pPriv->aPathName	= aPathName;
		pPriv->nCount		= 1;
		pPriv->pNext		= pFirst;
		pFirst				= pPriv;
	}
	else {
		if( pPriv->nError )
			pPriv->nError	= DosLoadModule(NULL, 0,
									(PSZ)aPathName.GetBuffer(),&(pPriv->hmod));
		pPriv->nCount++;
	}
}

Library::~Library()
{
	pPriv->nCount--;
	if( !pPriv->nCount ) {
#if 0
		if (!pPriv->nError) DosFreeModule(pPriv->hmod);
		if( pFirst == pPriv )
			pFirst = pPriv->pNext;
		else {
			LibraryPrivateData *pTemp = pFirst;
			while( pTemp && pTemp->pNext != pPriv )
				pTemp = pTemp->pNext;
			if( pTemp )
				pTemp->pNext = pPriv->pNext;
		}
		delete pPriv;
#endif
	}
}

BOOL Library::ModulFound() const
{
	if (pPriv->nError) return FALSE;
	else return TRUE;
}

void * Library::GetFunction(const String & rFuncName) const
{
	PFN pF;
	ByteString( rFuncName, RTL_TEXTENCODING_UTF8 );
	if (pPriv->nError) return NULL;
	else if(DosQueryProcAddr(pPriv->hmod,0,(PSZ)aFuncName.GetBuffer(),&pF)) return NULL;
	else return (void *) pF;
}

BOOL Library::ModulFound(const String & rPathname)
{
	Library aLibrary( rPathname );
	BOOL b = aLibrary.ModulFound();
	return b;
}

#endif


//=========================================================================
//========================MAC-Variante=====================================
//=========================================================================

#ifdef MAC

// ----------------------------------------------------------

#if	!defined(__powerc) &&  !defined (MWPPC)

struct LibraryPrivateData
{
	long	nDummy;
};
// Implementierung fuer 68 K

Library::Library(const String& rPathName)
{
	pPriv = NULL;
}

// ==========================================================

Library::~Library()
{
	delete pPriv;
}

// ==========================================================

BOOL Library::ModulFound() const
{
   return FALSE;
}

// ==========================================================

BOOL Library::ModulFound( const String & )
{
   return FALSE;
}

// ==========================================================

void* Library::GetFunction(const String& rFuncName) const
{
	return NULL;
}

// ----------------------------------------------------------

#else	// PPC

enum {
	kCodeSymbol			= 0,		/*a code symbol*/
	kDataSymbol			= 1,		/*a data symbol*/
	kTVectSymbol		= 2			/*a transition vector symbol*/
};

// ======================= Private Daten ====================

struct LibraryPrivateData
{
	CFragConnectionID	nConnectionID;
};

// ==========================================================

static char* ImpGetPascalString( String& rString )
{
	ByteString aString( rString, RTL_TEXTENCODING_UTF8 );
	sal_Char* pTmp = new sal_Char[ ( rString.Len() & 0xff ) + 1 ];
	pTmp[ 0 ] = aString.Len() & 0xff;
	memcpy( pTmp + 1, aString.GetBuffer(), aString.Len() & 0xff );
	return pTmp;
}

Library::Library( const String& rPathname )
{
	if ( ! rPathname.Len() )
		return;

	FSSpec 		aFileSpec;
	Str255		anErrName;
	MAC_Ptr		pMainEntry;
	OSErr		nErr = noErr + 1;
	BOOL		bNoAbsPath = FALSE;
	BOOL		bFound = FALSE;

	pPriv = new LibraryPrivateData;

	// Wenn wir keinen Pfad sondern nur den Lib Namen haben, dann versuchen
	// wir zuerst die Lib mit GetSharedLibrary() zu laden.

	sal_Char* pPascalStr = ImpGetPascalString( rPathname );

	if ( STRING_NOTFOUND == rPathname.Search(':') )	// kein absoluter Pfad ?
	{
		bNoAbsPath = TRUE;
		nErr = GetSharedLibrary( pPascalStr, kPowerPCCFragArch,
								 kLoadCFrag, &pPriv->nConnectionID,
								 &pMainEntry, anErrName );
		if ( noErr == nErr )
			bFound = TRUE;
	}
	// Wenn die Lib nicht von GetSharedLibrary() gefunden wurde oder wenn wir
	// einen vollstaendigen Pfad auf die Lib haben, dann erzeugen wir uns einen
	// FSSpec auf die Lib und benutzen diesen, um die Lib zu laden.
	if ( noErr != nErr )
	{
		nErr = FSMakeFSSpec( 0, 0, pPascalStr, &aFileSpec );

		if ( bNoAbsPath && ( noErr != nErr ) )
		{
			// Die Lib wurde nicht im aktuellen Verzeichnis gefunden:
			// ==> wir versuchen es noch einmal im App:shared Verzeichnis und
			// anschliessend im App Verzeichnis
			char* pAppName = ImpGetPascalString( Application::GetAppFileName() );
			nErr = FSMakeFSSpec( 0, 0, pAppName, &aFileSpec );
			delete pAppName;
			if ( noErr == nErr )
			{
				String aSharedPath( ":shared:" );
				aSharedPath.Append( rPathName );
				char* pSharedName = ImpGetPascalString( aSharedPath );
				nErr = FSMakeFSSpec( aFileSpec.vRefNum,
									 aFileSpec.parID,
									 pSharedName,
									 &aFileSpec );
				delete pSharedName;
				if ( noErr != nErr )
				{
					nErr = FSMakeFSSpec( aFileSpec.vRefNum,
										 aFileSpec.parID,
										 pPascalStr,
										 &aFileSpec );
				}
			}
		}
	}
	delete pPascalStr;

	if ( !bFound && ( noErr == nErr ) )
	{
		FInfo	aFInfo;
		short	nResId;

		/*	Pruefen, ob wir ein Alias haben
		 */
		nErr = FSpGetFInfo( &aFileSpec, &aFInfo );
		if ( ( noErr == nErr ) && ( aFInfo.fdFlags & kIsAlias ) )
		{
			AliasHandle hAlias;
			BOOL		bChanged;

			SetResLoad( FALSE );
			nResId = FSpOpenResFile( &aFileSpec, fsRdPerm );
			SetResLoad( TRUE );

			if ( nResId == -1 ) return;

			hAlias = (AliasHandle) Get1Resource( 'alis', 0 );
			if ( hAlias )
				nErr = ResolveAlias( nil, hAlias, &aFileSpec, &bChanged );

			CloseResFile( nResId );
		}
		if ( noErr == nErr )
			nErr = GetDiskFragment( &aFileSpec,0,0,aFileSpec.name,
									kNewCFragCopy,&pPriv->nConnectionID,
									&pMainEntry,anErrName );
	}
	if ( noErr != nErr )
	{
		delete pPriv;
		pPriv = NULL;
	}
}

// ==========================================================

Library::~Library()
{
	if (pPriv)
		CloseConnection(&pPriv->nConnectionID);
	delete pPriv;
}

// ==========================================================

BOOL Library::ModulFound() const
{
	if (pPriv) return TRUE; else return FALSE;
}

// ==========================================================

BOOL Library::ModulFound(const String & rPathname)
{
	Library aLibrary( rPathname );
	BOOL b = aLibrary.ModulFound();
	return b;
}

// ==========================================================

void* Library::GetShlbSymbol(const String& rFuncName, int nRequestSymbol)  const
{
	OSErr				nErr;
	MAC_Ptr					pSymbol;
	CFragSymbolClass	nSymClass;

#ifdef DBG_UTIL
	if (!rFuncName.Len() || ( rFuncName.GetChar( 0 ).Compare( '_' ) == COMPARE_EQUAL ) )
	{
		String anError("Library::GetShlbSymbol will fail with >");
		anError.Append( rFuncName );
		anError.Append( "<" );
		DBG_ERROR( ByteString( anError, RTL_TEXTENCODING_UTF8  ) );
	}
#endif

	if (!pPriv)
		return NULL;
	char* pFuncName = ImpGetPascalStr( rFuncName );
	nErr = FindSymbol(pPriv->nConnectionID,(unsigned char*)pFuncName,
						&pSymbol,&nSymClass);
	delete pFuncName;
	if ( (nSymClass != nRequestSymbol)	||	// wrong symbol class
		 (nErr != noErr))
		return NULL;

	return 	pSymbol;
}

void* Library::GetFunction(const String& rFuncName) const
{
	return GetShlbSymbol( rFuncName, kTVectSymbol );
}

void* Library::GetDescriptor( const String& rDescName ) const
{
	return GetShlbSymbol( rDescName, kDataSymbol );
}

#endif

#endif // MAC
