/*************************************************************************
 *
 *  $RCSfile: args.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <sbx.hxx>
#include "args.hxx"

#include "segmentc.hxx"

// STATIC DATA -----------------------------------------------------------

SEG_EOFGLOBALS()

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_01)

SfxArguments::SfxArguments( BOOL bCreateArray ):
	aArrayRef(bCreateArray ? new SbxArray : 0 ),
	nFrom(0), nCount(0)
{
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_02)

SfxArguments::SfxArguments( SbxArray *pArray, USHORT nFromIdx, USHORT nIdxCount ):
	aArrayRef(pArray),
	nFrom(nFromIdx), nCount(nIdxCount)
{
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_03)

SfxArguments::~SfxArguments()
{
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_10)

void SfxArguments::Append( SbxVariable* pArg )
{
	DBG_ASSERT( aArrayRef.Is(), "SfxArguments::Append() not overloaded" );

	if ( pArg->ISA(SbxObject) )
	{
		// Objekte indirekt reinsetzen
		SbxVariableRef xVar = new SbxVariable;
		xVar->PutObject( pArg );
		aArrayRef->Put( xVar, aArrayRef->Count() );
	}
	else
		// alles andere direkt reinsetzen
		aArrayRef->Put( pArg, aArrayRef->Count() );

	++nCount;
}

//------------------------------------------------------------------------

#pragma SEG_FUNCDEF(args_11)

void SfxArguments::AppendEmpty( )
{
	SbxVariableRef pVar = new SbxVariable;
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_12)

void SfxArguments::AppendInteger( INT16 nArg )
{
	SbxVariableRef pVar = new SbxVariable;
	pVar->PutInteger( nArg );
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_13)

void SfxArguments::AppendLong( INT32 nArg )
{
	SbxVariableRef pVar = new SbxVariable;
	pVar->PutLong( nArg );
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_14)

void SfxArguments::AppendSingle( float nArg )
{
	SbxVariableRef pVar = new SbxVariable;
	pVar->PutSingle( nArg );
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_15)

void SfxArguments::AppendDouble( double nArg )
{
	SbxVariableRef pVar = new SbxVariable;
	pVar->PutDouble( nArg );
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_16)

void SfxArguments::AppendBool( BOOL nArg )
{
	SbxVariableRef pVar = new SbxVariable;
	pVar->PutBool( nArg );
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_17)

void SfxArguments::AppendString( const String& rArg )
{
	SbxVariableRef pVar = new SbxVariable;
	pVar->PutString( rArg );
	Append( pVar );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_18)

void SfxArguments::AppendEnum( INT16 nVal )
{
	AppendInteger( nVal );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_20)

USHORT SfxArguments::Count() const
{
	if ( nFrom >= aArrayRef->Count() )
		return 0;
	return Min( nCount, (USHORT)(aArrayRef->Count() - nFrom) );
}

//------------------------------------------------------------------------
#pragma SEG_FUNCDEF(args_21)

const SbxVariable& SfxArguments::Get
(
	USHORT nNo	// Nummer des Parameters beginnend mit 0
) 	const

/*	[Beschreibung]

	Liefert den Parameter an der Position 'nNo', wobei die Numerierung
	mit 0 beginnt. Zugriff auf nicht verf"ugbare Parameter ist nicht
	definiert.
*/

{
	DBG_ASSERT( nNo < Count(), "argument index overflow" );
	return *aArrayRef->Get( nFrom + nNo );
}

/*------------------------------------------------------------------------

	$Log: args.cxx,v $
	Revision 1.1.1.1  2000/09/18 16:59:00  hr
	initial import
	
	Revision 1.23  2000/09/18 14:13:33  willem.vandorp
	OpenOffice header added.
	
	Revision 1.22  2000/08/31 13:41:27  willem.vandorp
	Header and footer replaced
	
	Revision 1.21  2000/04/12 08:01:38  sb
	Adapted to Unicode.
	
	Revision 1.20  1995/11/15 20:18:00  MI
	SbxCalues Ctor raus
	

	  Rev 1.19   15 Nov 1995 21:18:00   MI
   SbxCalues Ctor raus

	  Rev 1.18   15 Nov 1995 16:47:00   MI
   Append mit Args

	  Rev 1.17   26 Sep 1995 15:05:38   MI
   SubArray mit 0 beginnen lassen

	  Rev 1.16   25 Sep 1995 18:00:20   MI
   in Append Objekte sonderbehandeln

	  Rev 1.15   19 Jun 1995 10:13:28   MI
   Append jetzt mit Pointer

	  Rev 1.14   19 Jun 1995 07:39:24   MI
   AppendArguments entfernt

	  Rev 1.13   14 Jun 1995 12:34:44   MI
   Teilbereiche von Parametern richtig

	  Rev 1.12   14 Jun 1995 07:18:18   MI
   falsche Assertion in Get() (argument index overflow)

	  Rev 1.11   12 Jun 1995 18:12:50   ER
   USHORT cast

	  Rev 1.10   12 Jun 1995 18:08:38   MI
   Count() beruecksichtigt jetzt Sbxarray-Count von 0

	  Rev 1.9   01 Jun 1995 19:59:50   MI
   Append mit ++nCount

	  Rev 1.8   08 May 1995 18:00:30   MD
   Umstellung auf SbxArray

	  Rev 1.7   23 Apr 1995 16:06:42   MI
   BUgfix: Basic-Paremeter wurden von MD ungedreht, jetzt also hier nicht mehr

	  Rev 1.6   27 Jan 1995 17:08:04   MM
   Ref Fehler beseitigt

	  Rev 1.5   26 Jan 1995 20:50:18   MH
   jetzt Sbx-Refernzen

	  Rev 1.4   17 Jan 1995 14:21:46   MI
   AppendEnum war n.i.

	  Rev 1.3   16 Jan 1995 11:03:28   MI
   op == Assert via SfxPoolItem
   Construct/Record


------------------------------------------------------------------------*/

#pragma SEG_EOFMODULE

