/*************************************************************************
 *
 *  $RCSfile: cntcmpmn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif

#ifndef _COM_SUN_STAR_CHAOS_MENUDESCRIPTOR_HPP_
#include <com/sun/star/chaos/MenuDescriptor.hpp>
#endif
#ifndef _COM_SUN_STAR_CHAOS_MENUITEMDESCRIPTOR_HPP_
#include <com/sun/star/chaos/MenuItemDescriptor.hpp>
#endif

#include <tools/stream.hxx>
#include <tools/debug.hxx>
#include <tools/string.hxx>

#include "cntcmpmn.hxx"

USHORT GetEntryCount_Impl( const CntCmpMenu* pMenu );
void GetEntries_Impl( const CntCmpMenu* pMenu,
					  com::sun::star::uno::Sequence<
					  	com::sun::star::chaos::MenuItemDescriptor >& rItems,
					  USHORT nParentPos,
					  USHORT & nIndex );

DBG_NAME( CntCmpMenuItem );
TYPEINIT1( CntCmpMenuItem, SfxPoolItem );

// -----------------------------------------------------------------------

class CntCmpMenu_Impl
{
public:
	CntCmpMenu* _pMenu;
	ULONG		_nRefCount;
	USHORT		_nMenuPart;

	CntCmpMenu_Impl() : _pMenu(0),_nRefCount(0) {}
	~CntCmpMenu_Impl() { delete _pMenu; }
};

// -----------------------------------------------------------------------

BOOL CntCmpMenu::HasVerb
( XubString const & rVerb
, XubString& rCommand
, BOOL bDeep ) const
{
	// Erst ueber die eigenen Eintraege...
	USHORT nCount = Count();
	for ( USHORT i = 0; i < nCount; i++ )
	{
		const CntCmpMenuEntry* pEntry = GetObject(i);
		if ( pEntry->GetVerb().Equals(rVerb) )
		{
			rCommand = pEntry->GetCommand();
			return TRUE;
		}
	}

	// dann in die Tiefe
	if ( bDeep )
	{
		for ( USHORT i = 0; i < nCount; i++ )
		{
			const CntCmpMenuEntry* pEntry = GetObject(i);
			const CntCmpMenu* pMenu = pEntry->GetSubMenu();
			if ( pMenu )
			{
				BOOL bHaveIt = pMenu->HasVerb( rVerb, rCommand, bDeep);
				if ( bHaveIt )
					return TRUE;
			}
		}
	}
	return FALSE;
}

// -----------------------------------------------------------------------

USHORT CntCmpMenu::GetVerbs (XubString& aVerbs, xub_Unicode cSep, BOOL bDeep) const
{
	USHORT nVerbCount = 0;
	XubString aVerb;
	// Erst ueber die eigenen Eintraege...
	USHORT nCount = Count();
	for ( USHORT i = 0; i < nCount; i++ )
	{
		const CntCmpMenuEntry* pEntry = GetObject(i);
		aVerb = pEntry->GetVerb();
		if ( aVerb.Len() )
		{
			aVerbs += pEntry->GetVerb();
			aVerbs += cSep;
			nVerbCount++;
		}
	}

	// dann in die Tiefe
	if ( bDeep )
	{
		for ( USHORT i = 0; i < nCount; i++ )
		{
			const CntCmpMenuEntry* pEntry = GetObject(i);
			const CntCmpMenu* pMenu = pEntry->GetSubMenu();
			if ( pMenu )
				nVerbCount += pMenu->GetVerbs( aVerbs, cSep, bDeep);
		}
	}
	return nVerbCount;
}

// -----------------------------------------------------------------------

CntCmpMenu::CntCmpMenu( const CntCmpMenu& rMenu )
{
	USHORT nCount = rMenu.Count();
	for( USHORT nCur = 0; nCur < nCount; nCur++ )
	{
		CntCmpMenuEntry* pEntry = new CntCmpMenuEntry(*rMenu.GetObject(nCur));
		Insert( pEntry, nCur );
	}
}

// -----------------------------------------------------------------------

CntCmpMenu::~CntCmpMenu()
{
	USHORT nCount = Count();
	for( USHORT nCur = 0; nCur < nCount; nCur++ )
	{
		delete (CntCmpMenuEntry*)GetObject( nCur );
	}
}

// -----------------------------------------------------------------------

CntCmpMenuEntry::CntCmpMenuEntry( const CntCmpMenuEntry& rEntry )
	: _aText( rEntry._aText ),
	  _aCommand( rEntry._aCommand ),
	  _aHelpURL( rEntry._aHelpURL ),
	  _nFlags( rEntry._nFlags ),
	  _nMenuPart( rEntry._nMenuPart )
{
	if( rEntry.GetSubMenu() )
		_pSubMenu = new CntCmpMenu( *rEntry.GetSubMenu() );
	else
		_pSubMenu = 0;
}

// -----------------------------------------------------------------------

void CntCmpMenuEntry::SetSubMenu( CntCmpMenu* pMenu )
{
	if ( pMenu != _pSubMenu )
		delete _pSubMenu;
	_pSubMenu=pMenu;
}

// -----------------------------------------------------------------------

CntCmpMenuItem::CntCmpMenuItem( USHORT nWhich ) :
	SfxPoolItem( nWhich ),
	_pMenu(0)
{
	DBG_CTOR( CntCmpMenuItem, 0 );
}

// -----------------------------------------------------------------------

CntCmpMenuItem::CntCmpMenuItem( const CntCmpMenuItem& rItem ) :
	SfxPoolItem( rItem )
{
	DBG_CTOR( CntCmpMenuItem, 0 );
	_pMenu = rItem._pMenu;
	if ( _pMenu )
		_pMenu->_nRefCount++;
}

// -----------------------------------------------------------------------

CntCmpMenuItem::~CntCmpMenuItem()
{
	DBG_DTOR(CntCmpMenuItem, 0);
	ReleaseMenu();
}

CntCmpMenuItem& CntCmpMenuItem::operator=( const CntCmpMenuItem& rItem )
{
	DBG_CHKTHIS( CntCmpMenuItem, 0 );
	ReleaseMenu();
	_pMenu = rItem._pMenu;
	if ( _pMenu )
		_pMenu->_nRefCount++;
	return *this;
}

// -----------------------------------------------------------------------

void CntCmpMenuItem::ReleaseMenu()
{
	DBG_CHKTHIS( CntCmpMenuItem, 0 );
	if( _pMenu )
	{
		_pMenu->_nRefCount--;
		if( !_pMenu->_nRefCount )
			delete _pMenu;
		_pMenu = 0;
	}
}

// -----------------------------------------------------------------------

int CntCmpMenuItem::operator==( const SfxPoolItem& rItem ) const
{
	DBG_CHKTHIS( CntCmpMenuItem, 0 );
	DBG_ASSERT( SfxPoolItem::operator==( rItem ), "unequal type" );

	if(_pMenu == ((const CntCmpMenuItem&)rItem)._pMenu )
		return TRUE;
	else
		return FALSE;

	return 0;
}

// -----------------------------------------------------------------------

SfxPoolItem* CntCmpMenuItem::Create( SvStream& rStream, USHORT ) const
{
	DBG_CHKTHIS(CntCmpMenuItem, 0);
	DBG_ERROR("CntCmpMenuItem::Create() not implemented");
	return new CntCmpMenuItem( Which() );
}

// -----------------------------------------------------------------------

SvStream& CntCmpMenuItem::Store( SvStream& rStream, USHORT ) const
{
	DBG_CHKTHIS( CntCmpMenuItem, 0 );
	DBG_ERROR("CntCmpMenuItem::Store() not implemented");
	return rStream;
}

// -----------------------------------------------------------------------

SfxPoolItem* CntCmpMenuItem::Clone( SfxItemPool* ) const
{
	DBG_CHKTHIS( CntCmpMenuItem, 0 );
	return new CntCmpMenuItem( *this );
}

// -----------------------------------------------------------------------

void CntCmpMenuItem::SetMenu( CntCmpMenu* pMenu, USHORT nMenuPart )
{
	ReleaseMenu();
	if( pMenu )
	{
		_pMenu = new CntCmpMenu_Impl;
		_pMenu->_nRefCount = 1;
		_pMenu->_pMenu = pMenu;
		_pMenu->_nMenuPart = nMenuPart;
	}
}

// -----------------------------------------------------------------------

const CntCmpMenu* CntCmpMenuItem::GetMenu() const
{
	return _pMenu ? _pMenu->_pMenu : 0;
}

// -----------------------------------------------------------------------

USHORT CntCmpMenuItem::GetMenuPart() const
{
	return _pMenu ? _pMenu->_nMenuPart : MENUPART_EDIT_PROPS_BEGIN;
}

// -----------------------------------------------------------------------

BOOL CntCmpMenuItem::QueryValue( com::sun::star::uno::Any& rVal, BYTE nMemberId ) const
{
	USHORT nIndex = 0;
	USHORT nCount = GetEntryCount_Impl( GetMenu() );

	com::sun::star::chaos::MenuDescriptor aPopup;
	com::sun::star::uno::Sequence<
		com::sun::star::chaos::MenuItemDescriptor > aMenu( nCount );

	GetEntries_Impl( GetMenu(), aMenu, 0, nIndex );

	aPopup.Items = aMenu;
	rVal <<= aPopup;

	return TRUE;
}

// -----------------------------------------------------------------------

BOOL CntCmpMenuItem::PutValue( const com::sun::star::uno::Any& rVal, BYTE nMemberId )
{
	BOOL bRet = FALSE;

	com::sun::star::chaos::MenuDescriptor aPopup;

	if( rVal >>= aPopup )
	{
		ULONG i, nPos, nInsertionPos;
		CntCmpMenu *pCntMenu = NULL;
		const com::sun::star::chaos::MenuItemDescriptor *pMenues
			= aPopup.Items.getConstArray();
		List aItemsList;

		nInsertionPos = 0;

		for ( i=0; i < aPopup.Items.getLength(); i++ )
		{
			CntCmpMenuEntry *pEntry = new CntCmpMenuEntry;
			pEntry->SetText( pMenues->ItemText );
			pEntry->SetCommand( pMenues->Command );
			pEntry->SetHelpURL( pMenues->HelpUrl );
			pEntry->SetFlags( pMenues->ItemFlags );
			pEntry->SetMenuPart( pMenues->InsertionPos );
			nPos = pMenues->ParentPos;

			if ( !pCntMenu )
			{
				pCntMenu = new CntCmpMenu;
				nInsertionPos = pMenues->InsertionPos;
			}

			if ( nPos == 0 )
			{
				pCntMenu->Insert( pEntry, pCntMenu->Count() );
			}
			else
			{
				CntCmpMenuEntry *pParent;
				pParent = (CntCmpMenuEntry*) aItemsList.GetObject( (ULONG) (nPos-1) );
				if ( pParent )
				{
					CntCmpMenu *pSubMenu = (CntCmpMenu*) pParent->GetSubMenu();
					if ( !pSubMenu )
						pSubMenu = new CntCmpMenu;
					pSubMenu->Insert( pEntry, pSubMenu->Count() );
					pParent->SetSubMenu( pSubMenu );
				}
				else
				{
					DBG_ERRORFILE( "No parent found for SubMenu!" );
					pCntMenu->Insert( pEntry, pCntMenu->Count() );
				}
			}
			aItemsList.Insert( (void*) pEntry, LIST_APPEND );
			pMenues++;
		}

		SetMenu( pCntMenu, (USHORT) nInsertionPos );

		bRet = TRUE;
	}

	return bRet;
}

USHORT GetEntryCount_Impl( const CntCmpMenu* pMenu )
{
	if ( !pMenu )
		return 0;

	USHORT nCount = pMenu->Count();
	USHORT nTotalCount = nCount;
	for ( USHORT i = 0; i < nCount; i++ )
	{
		CntCmpMenuEntry *pEntry = (CntCmpMenuEntry*) pMenu->GetObject( i );
		const CntCmpMenu* pSubMenu = pEntry->GetSubMenu();
		if ( pSubMenu )
			nTotalCount += GetEntryCount_Impl( pSubMenu );
	}
	return nTotalCount;
}

void GetEntries_Impl( const CntCmpMenu* pMenu,
					  com::sun::star::uno::Sequence<
					  	com::sun::star::chaos::MenuItemDescriptor >& rItems,
					  USHORT nParentPos,
					  USHORT & nIndex )
{
	if ( !pMenu )
		return;

	USHORT i;
	USHORT nCount = pMenu->Count();
	USHORT nStart = nIndex;

	com::sun::star::chaos::MenuItemDescriptor* pItems = rItems.getArray();

	for ( i=0; i<nCount; i++ )
	{
		CntCmpMenuEntry *pEntry = (CntCmpMenuEntry*) pMenu->GetObject( i );
		com::sun::star::chaos::MenuItemDescriptor& rData = pItems[ nIndex++ ];

		rData.ItemText 	   = pEntry->GetText();
		rData.Command  	   = pEntry->GetCommand();
		rData.HelpUrl  	   = pEntry->GetHelpURL();
		rData.ParentPos	   = nParentPos;
		rData.ItemFlags	   = pEntry->GetFlags();
		rData.InsertionPos = pEntry->GetMenuPart();
	}

	for ( i=0; i<nCount; i++ )
	{
		CntCmpMenuEntry *pEntry = (CntCmpMenuEntry*) pMenu->GetObject( i );
		const CntCmpMenu* pSubMenu = pEntry->GetSubMenu();
		if ( pSubMenu )
			GetEntries_Impl( pSubMenu, rItems, nStart + i + 1, nIndex );
	}
}


