/*************************************************************************
 *
 *  $RCSfile: compat.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#pragma hdrstop

#include "compat.hxx"
#include <segmentc.hxx>

#ifndef _SHL_HXX //autogen
#include <tools/shl.hxx>
#endif

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif



SEG_EOFGLOBALS()

#pragma SEG_FUNCDEF(compat_01)

SfxPointerServer::SfxPointerServer()
{
	pCache=0;
	pLast=aPointers+SFX_POINTER_SIZE;
	nObjectCount = 0;
}

#pragma SEG_FUNCDEF(compat_02)

SfxPointerServer::~SfxPointerServer()
{
}


SfxPointerServer* SfxPointerServer::GetServer()
{
	SfxPointerServer** ppSrv=(SfxPointerServer**)GetAppData(SHL_SFC);
	if(!*ppSrv)
	{
		*ppSrv=new SfxPointerServer();
		DBG_WARNING("SfxPointerServer alloziiert");
	}
	return *ppSrv;
}

void SfxPointerServer::ReleaseServer()
{
	SfxPointerServer** ppSrv=(SfxPointerServer**)GetAppData(SHL_SFC);
	*ppSrv=0;
	DBG_WARNING("SfxPointerServer released");
}


#pragma SEG_FUNCDEF(compat_03)


BOOL SfxPointerServer::HasPointer(const void *pObj)
{
	if(pCache==pObj)
		return 1;
	SfxPointerEntry *pEntry=aPointers;
	do
	{
		if(pEntry->pObject==pObj)
			return TRUE;
	}
	while(++pEntry<pLast);
	return FALSE;
}

#pragma SEG_FUNCDEF(compat_04)


void * SfxPointerServer::GetPointer(const void *pObj)

/*  [Beschreibung]

	Holt den dem Object gehoerenden Pointer aus dem Array
*/

{
	if(pCache==pObj) return pCachePtr;
	SfxPointerEntry *pEntry=aPointers+MakeHash(pObj);
	SfxPointerEntry* pHashed = pEntry;
	do
	{
		if( pEntry - aPointers == SFX_POINTER_SIZE )
			pEntry = aPointers;
		if(pEntry->pObject==pObj)
		{
			pCache=pObj;
			return pCachePtr=pEntry->pPointer;
		}
	}
	while( ++pEntry != pHashed );

	DBG_ERROR("SfxPointerSever Overflow");
	return 0;
}

#pragma SEG_FUNCDEF(compat_05)


void **SfxPointerServer::CreatePointer( const void *pObj)

/*  [Beschreibung]

	Gibt einen Pointer auf den fuer den pObj reservierte
	Impl Pointer zurueck.
*/

{
	DBG_ASSERT(this, "Statisches Object benutzt SfxPointerServer");
	nObjectCount++;
	SfxPointerEntry *pEntry=aPointers+MakeHash(pObj);
	SfxPointerEntry* pHashed = pEntry;
	do
	{
		if( pEntry - aPointers == SFX_POINTER_SIZE )
			pEntry = aPointers;
		if(!pEntry->pObject||pEntry->pObject==pObj)
			//Diesen auch nehmen, wenn der this Pointer zum 2. mal kommt.
			//Fuer Klassen ohne Destruktor.
		{
			pEntry->pObject=pObj;
			return &pEntry->pPointer;
		}
	}
	while( ++pEntry != pHashed );

	nObjectCount--;
	DBG_ERROR("SfxPointerSever Object nicht gefunden");
	return 0;
}

#pragma SEG_FUNCDEF(compat_06)


void SfxPointerServer::ReleasePointer(const void *pObj)

/*  [Beschreibung]

	Gibt den fuer pObj reservierten Pointer wieder frei.
*/


{
	SfxPointerEntry *pEntry=aPointers+MakeHash(pObj);
	SfxPointerEntry* pHashed = pEntry;
	do
	{
		if( pEntry - aPointers == SFX_POINTER_SIZE )
			pEntry = aPointers;
		if(pEntry->pObject==pObj)
		{
			pEntry->pObject=0;
			if(pCache==pObj)
				pCache=0;
			nObjectCount--;
			if(!nObjectCount)
			{
				delete this;
				ReleaseServer();
			}
			return;
		}
	}
	while( ++pEntry != pHashed );

	DBG_ERROR("SfxPointerServer Pointer schon released");
}
