/*************************************************************************
 *
 *  $RCSfile: cstitem.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif

#include <tools/stream.hxx>
#include <tools/debug.hxx>
#include <tools/intn.hxx>
#include <tools/datetime.hxx>

#include "svtdata.hxx"
#include "cstitem.hxx"

#include "svtools.hrc"

DBG_NAME( SfxCrawlStatusItem );
TYPEINIT1( SfxCrawlStatusItem, SfxPoolItem );

// -----------------------------------------------------------------------

SfxCrawlStatusItem::SfxCrawlStatusItem( USHORT nWhich ) :
	SfxPoolItem( nWhich )
{
	DBG_CTOR( SfxCrawlStatusItem, 0 );
}

// -----------------------------------------------------------------------

SfxCrawlStatusItem::SfxCrawlStatusItem( USHORT nWhich, CrawlStatus eStat ) :
	SfxPoolItem( nWhich ),
	eStatus( eStat )
{
	DBG_CTOR( SfxCrawlStatusItem, 0 );
}

// -----------------------------------------------------------------------

SfxCrawlStatusItem::SfxCrawlStatusItem( const SfxCrawlStatusItem& rItem ) :
	SfxPoolItem( rItem ),
	eStatus( rItem.eStatus )
{
	DBG_CTOR( SfxCrawlStatusItem, 0 );
}

// -----------------------------------------------------------------------

int SfxCrawlStatusItem::operator==( const SfxPoolItem& rItem ) const
{
	DBG_CHKTHIS( SfxCrawlStatusItem, 0 );
	DBG_ASSERT( SfxPoolItem::operator==( rItem ), "unequal type" );

	return ((SfxCrawlStatusItem&)rItem).eStatus == eStatus;
}

// -----------------------------------------------------------------------

int SfxCrawlStatusItem::Compare( const SfxPoolItem& rItem ) const
{
	DBG_CHKTHIS( SfxCrawlStatusItem, 0 );
	DBG_ASSERT( SfxPoolItem::operator==( rItem ), "unequal type" );

	if( ((const SfxCrawlStatusItem&)rItem).eStatus < eStatus )
		return -1;
	else if( ((const SfxCrawlStatusItem&)rItem).eStatus == eStatus )
		return 0;
	else
		return 1;
}

// -----------------------------------------------------------------------

SfxPoolItem* SfxCrawlStatusItem::Create( SvStream& rStream, USHORT ) const
{
	DBG_CHKTHIS(SfxCrawlStatusItem, 0);

	USHORT _eStatus;
	rStream >> _eStatus;

	return new SfxCrawlStatusItem( Which(), (CrawlStatus)_eStatus );
}

// -----------------------------------------------------------------------

SvStream& SfxCrawlStatusItem::Store( SvStream& rStream, USHORT ) const
{
	DBG_CHKTHIS( SfxCrawlStatusItem, 0 );

	USHORT nStatus = (USHORT)eStatus;
	rStream << (USHORT) nStatus;

	return rStream;
}

// -----------------------------------------------------------------------

SfxPoolItem* SfxCrawlStatusItem::Clone( SfxItemPool* ) const
{
	DBG_CHKTHIS( SfxCrawlStatusItem, 0 );
	return new SfxCrawlStatusItem( *this );
}
// -----------------------------------------------------------------------

SfxItemPresentation SfxCrawlStatusItem::GetPresentation
(
	SfxItemPresentation 	ePresentation,
	SfxMapUnit				eCoreMetric,
	SfxMapUnit				ePresentationMetric,
	XubString& 				rText,
	const International *	pInternational
)	const
{
	DBG_CHKTHIS(SfxCrawlStatusItem, 0);
	DBG_ASSERT(pInternational,
			   "SfxCrawlStatusItem::GetPresentation():"
			    " Using default International");
	switch (eStatus)
	{
		case CSTAT_NEVER_UPD:
			rText = String(SvtResId(STR_CSTAT_NEVER_UPD,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_IN_UPD:
			rText = String(SvtResId(STR_CSTAT_IN_UPD,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_UPD_NEWER:
			rText = String(SvtResId(STR_CSTAT_UPD_NEWER,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_UPD_NOT_NEWER:
			rText = String(SvtResId(STR_CSTAT_UPD_NOT_NEWER,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_UPD_CANCEL:
			rText = String(SvtResId(STR_CSTAT_UPD_CANCEL,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_ERR_GENERAL:
			rText = String(SvtResId(STR_CSTAT_ERR_GENERAL,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_ERR_NOTEXISTS:
			rText = String(SvtResId(STR_CSTAT_ERR_NOTEXISTS,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_ERR_NOTREACHED:
			rText = String(SvtResId(STR_CSTAT_ERR_NOTREACHED,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_UPD_IMMEDIATELY:
			rText = String(SvtResId(STR_CSTAT_UPD_IMMEDIATELY,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		case CSTAT_ERR_OFFLINE:
			rText = String(SvtResId(STR_CSTAT_ERR_OFFLINE,
									pInternational ?
									 pInternational->GetLanguage() :
									 LANGUAGE_DONTKNOW));
			break;

		default:
			rText.Erase();
			break;
	}
	return SFX_ITEM_PRESENTATION_NAMELESS;
}

//----------------------------------------------------------------------------
// virtual
BOOL SfxCrawlStatusItem::PutValue( const com::sun::star::uno::Any& rVal,
							   	   BYTE nMemberId )
{
	sal_Int16 aValue;
	if ( rVal >>= aValue )
	{
		SetStatus( static_cast< CrawlStatus >( aValue ) );
		return TRUE;
	}

	DBG_ERROR( "SfxCrawlStatusItem::PutValue - Wrong type!" );
	return FALSE;
}

//----------------------------------------------------------------------------
// virtual
BOOL SfxCrawlStatusItem::QueryValue( com::sun::star::uno::Any& rVal,
							 	  	 BYTE nMemberId ) const
{
	sal_Int16 aValue = GetStatus();
	rVal <<= aValue;
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//
/*
   $Log: cstitem.cxx,v $
   Revision 1.1.1.1  2000/09/18 16:59:00  hr
   initial import

   Revision 1.13  2000/09/18 14:13:33  willem.vandorp
   OpenOffice header added.

   Revision 1.12  2000/08/31 13:41:27  willem.vandorp
   Header and footer replaced

   Revision 1.11  2000/03/22 11:49:36  kso
   Removed: SmartUno leftovers.

   Revision 1.10  2000/02/21 09:33:42  kso
   #68274# - Added: [Put|QueryValue( ... com::sun::star::uno::Any ... )

   Revision 1.9  2000/02/09 16:22:36  hr
   #70473# changes for unicode ( patched by automated patchtool )

   Revision 1.8  1999/09/15 10:12:38  sb
   #66082# Use SfxCrawlStatusItem::GetPresentation()'s International param.

   Revision 1.7  1999/09/01 06:52:34  sb
   #66082# Added International param to SfxPoolItem::GetPresentation().

   Revision 1.6  1998/09/11 10:40:02  OK
   FIX: #56064#; Fehler, wenn bei Abo Upd Offline


	  Rev 1.5   11 Sep 1998 12:40:02   OK
   FIX: #56064#; Fehler, wenn bei Abo Upd Offline

	  Rev 1.4   11 Sep 1998 09:27:22   OK
   FIX: #55345#, Es wird nur aus dem Cache geladen; #56064# vorbereitet

	  Rev 1.3   07 Jul 1998 16:19:20   KSO
   Removed: Unnecessary SUPD check.

	  Rev 1.2   07 Jul 1998 13:53:30   KSO
   Added: [Put|Query]Value(...) to class CntCrawlStatusItem.

	  Rev 1.1   02 Apr 1998 16:51:36   OK
   NEW: neuer CrawlStaus fuer autoactive ErrorHandling

	  Rev 1.0   09 Mar 1998 17:16:22   OK
   NEW: SfxCrawlStatusItem

*/

