/*************************************************************************
 *
 *  $RCSfile: ctypeitm.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif
#ifndef _TOOLS_INTN_HXX
#include <tools/intn.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _SVTOOLS_CTYPEITM_HXX
#include <ctypeitm.hxx>
#endif

//============================================================================
// The following defines are copied from chaos/source/items/cstritem.cxx:
#define CNTSTRINGITEM_STREAM_MAGIC   ( (UINT32)0xfefefefe )
#define CNTSTRINGITEM_STREAM_SEEKREL (-( (long)( sizeof( UINT32 ) ) ) )

//============================================================================
//
// class CntContentTypeItem Implementation.
//
//============================================================================

TYPEINIT1_AUTOFACTORY( CntContentTypeItem, CntUnencodedStringItem );

#define CONTENT_TYPE_NOT_INIT ( (INetContentType)-1 )

//----------------------------------------------------------------------------
CntContentTypeItem::CntContentTypeItem()
: CntUnencodedStringItem(),
  _eType( CONTENT_TYPE_NOT_INIT )
{
}

//----------------------------------------------------------------------------
CntContentTypeItem::CntContentTypeItem(	USHORT nWhich, const String& rType )
: CntUnencodedStringItem( nWhich, rType ),
  _eType( CONTENT_TYPE_NOT_INIT )
{
}

//----------------------------------------------------------------------------
CntContentTypeItem::CntContentTypeItem(	USHORT nWhich,
										const INetContentType eType )
: CntUnencodedStringItem( nWhich, INetContentTypes::GetContentType( eType ) ),
  _eType( eType )
{
}

//----------------------------------------------------------------------------
CntContentTypeItem::CntContentTypeItem( const CntContentTypeItem& rOrig )
: CntUnencodedStringItem( rOrig ),
  _eType( rOrig._eType ),
  _aPresentation( rOrig._aPresentation )
{
}

//============================================================================
// virtual
USHORT CntContentTypeItem::GetVersion(USHORT) const
{
	return 1; // because it uses SfxPoolItem::read/writeUnicodeString()
}

//----------------------------------------------------------------------------
// virtual
SfxPoolItem* CntContentTypeItem::Create( SvStream& rStream,
										 USHORT nItemVersion ) const
{
	// CntContentTypeItem used to be derived from CntStringItem, so take that
	// into account:
	UniString aValue;
	readUnicodeString(rStream, aValue, nItemVersion >= 1);
	UINT32 nMagic = 0;
	rStream >> nMagic;
	if (nMagic == CNTSTRINGITEM_STREAM_MAGIC)
	{
		BOOL bEncrypted = FALSE;
		rStream >> bEncrypted;
		DBG_ASSERT(!bEncrypted,
				   "CntContentTypeItem::Create() reads encrypted data");
	}
	else
		rStream.SeekRel(CNTSTRINGITEM_STREAM_SEEKREL);

	return new CntContentTypeItem(Which(), aValue);
}

//----------------------------------------------------------------------------
// virtual
SvStream & CntContentTypeItem::Store(SvStream & rStream, USHORT) const
{
	// CntContentTypeItem used to be derived from CntStringItem, so take that
	// into account:
	writeUnicodeString(rStream, GetValue());
	rStream << CNTSTRINGITEM_STREAM_MAGIC << BOOL(FALSE);
	return rStream;
}

//----------------------------------------------------------------------------
// virtual
int CntContentTypeItem::operator==( const SfxPoolItem& rOrig ) const
{
	const CntContentTypeItem& rOther = (const CntContentTypeItem&)rOrig;

	if ( ( _eType != CONTENT_TYPE_NOT_INIT ) &&
		 ( rOther._eType != CONTENT_TYPE_NOT_INIT ) )
		return _eType == rOther._eType;
	else
		return CntUnencodedStringItem::operator==( rOther );
}

//----------------------------------------------------------------------------
// virtual
SfxPoolItem* CntContentTypeItem::Clone( SfxItemPool* /* pPool */ ) const
{
	return new CntContentTypeItem( *this );
}

//----------------------------------------------------------------------------
#ifdef ENABLEUNICODE
void CntContentTypeItem::SetValue( const UniString& rNewVal )
#else
void CntContentTypeItem::SetValue( const String& rNewVal )
#endif
{
	// De-initialize enum type and presentation.
	_eType = CONTENT_TYPE_NOT_INIT;
	_aPresentation.Erase();

	CntUnencodedStringItem::SetValue( rNewVal );
}

//----------------------------------------------------------------------------
#ifdef ENABLEUNICODE
void CntContentTypeItem::SetPresentation( const UniString& rNewVal )
#else
void CntContentTypeItem::SetPresentation( const String& rNewVal )
#endif
{
	_aPresentation = rNewVal;
}

//----------------------------------------------------------------------------
int CntContentTypeItem::Compare( const SfxPoolItem &rWith, const International& rInternational ) const
{
	String aOwnText, aWithText;
	SfxMapUnit eUnit = SFX_MAPUNIT_APPFONT;
	GetPresentation( SFX_ITEM_PRESENTATION_NAMELESS,
					 SFX_MAPUNIT_APPFONT, SFX_MAPUNIT_APPFONT, aOwnText, &rInternational );
	rWith.GetPresentation( SFX_ITEM_PRESENTATION_NAMELESS,
						   SFX_MAPUNIT_APPFONT, SFX_MAPUNIT_APPFONT, aWithText, &rInternational );
	switch ( rInternational.Compare(aOwnText, aWithText,
									INTN_COMPARE_IGNORECASE) )
	{
		case COMPARE_EQUAL:
			return 0;

		case COMPARE_LESS:
			return -1;

		default: // COMPARE_GREATER
			return 1;
	}
}

//----------------------------------------------------------------------------
SfxItemPresentation CntContentTypeItem::GetPresentation(SfxItemPresentation
														 ePres,
														SfxMapUnit
														 eCoreMetric,
														SfxMapUnit
														 ePresMetric,
														XubString & rText,
														const International *
														 pInternational) const
{
	if (_aPresentation.Len() == 0)
	{
		DBG_ASSERT(pInternational,
				   "CntContentTypeItem::GetPresentation(): No International");
		if (pInternational)
			SAL_CONST_CAST(CntContentTypeItem *, this)->_aPresentation
			 = INetContentTypes::GetPresentation(GetEnumValue(),
												 pInternational->
												  GetLanguage());
	}
	if (_aPresentation.Len() > 0)
	{
		rText = _aPresentation;
		return SFX_ITEM_PRESENTATION_COMPLETE;
	}
	else
		return CntUnencodedStringItem::GetPresentation(ePres, eCoreMetric,
													   ePresMetric, rText,
													   pInternational);
}

//----------------------------------------------------------------------------
const INetContentType CntContentTypeItem::GetEnumValue() const
{
	if ( _eType == CONTENT_TYPE_NOT_INIT )
	{
		// Not yet initialized... Get enum value for string content type.

		CntContentTypeItem* pVarThis = SAL_CONST_CAST( CntContentTypeItem*, this );

		pVarThis->_eType = INetContentTypes::GetContentType( GetValue() );
	}

	return _eType;
}

//----------------------------------------------------------------------------
void CntContentTypeItem::SetValue( const INetContentType eType )
{
	SetValue( INetContentTypes::GetContentType( eType ) );

	// Note: SetValue( const String& ....) resets _eType. Set new enum value
	//       after(!) calling it.
	_eType = eType;
}

//----------------------------------------------------------------------------
// virtual
BOOL CntContentTypeItem::QueryValue( com::sun::star::uno::Any& rVal,
							 	  	 BYTE nMemberId ) const
{
	rVal <<= rtl::OUString(GetValue());
	return true;
}

//----------------------------------------------------------------------------
// virtual
BOOL CntContentTypeItem::PutValue( const com::sun::star::uno::Any& rVal,
						   		   BYTE nMemberId )
{
	rtl::OUString aValue;
	if ( rVal >>= aValue )
	{
		// SetValue with an empty string resets the item; so call that
		// function when PutValue is called with an empty string
		if (aValue.getLength() == 0)
			SetValue(aValue);
		else
			SetValue(
				INetContentTypes::RegisterContentType(aValue, UniString()));
		return true;
	}

	DBG_ERROR( "CntContentTypeItem::PutValue - Wrong type!" );
	return false;
}


