/*************************************************************************
 *
 *  $RCSfile: dtritem.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif
#ifndef _COM_SUN_STAR_UTIL_DATETIMERANGE_HPP_
#include <com/sun/star/util/DateTimeRange.hpp>
#endif

#include <tools/stream.hxx>
#include <tools/debug.hxx>
#include <tools/intn.hxx>
#include <tools/datetime.hxx>

#include <vcl/svapp.hxx>

#include "dtritem.hxx"


DBG_NAME( SfxDateTimeRangeItem );
TYPEINIT1( SfxDateTimeRangeItem, SfxPoolItem );

// -----------------------------------------------------------------------

SfxDateTimeRangeItem::SfxDateTimeRangeItem( USHORT nWhich ) :
	SfxPoolItem( nWhich )
{
	DBG_CTOR( SfxDateTimeRangeItem, 0 );
}

// -----------------------------------------------------------------------

SfxDateTimeRangeItem::SfxDateTimeRangeItem( USHORT nWhich, const DateTime& rStartDT,
											const DateTime& rEndDT ) :
	SfxPoolItem( nWhich ),
	aStartDateTime( rStartDT ),
	aEndDateTime( rEndDT )
{
	DBG_CTOR( SfxDateTimeRangeItem, 0 );
}

// -----------------------------------------------------------------------

SfxDateTimeRangeItem::SfxDateTimeRangeItem( const SfxDateTimeRangeItem& rItem ) :
	SfxPoolItem( rItem ),
	aStartDateTime( rItem.aStartDateTime ),
	aEndDateTime( rItem.aEndDateTime )
{
	DBG_CTOR( SfxDateTimeRangeItem, 0 );
}

// -----------------------------------------------------------------------

int SfxDateTimeRangeItem::operator==( const SfxPoolItem& rItem ) const
{
	DBG_CHKTHIS( SfxDateTimeRangeItem, 0 );
	DBG_ASSERT( SfxPoolItem::operator==( rItem ), "unequal type" );

	return  ((SfxDateTimeRangeItem&)rItem ).aStartDateTime == aStartDateTime &&
			((SfxDateTimeRangeItem&)rItem ).aEndDateTime == aEndDateTime;
}

// -----------------------------------------------------------------------

int SfxDateTimeRangeItem::Compare( const SfxPoolItem& rItem ) const
{
	DBG_CHKTHIS( SfxDateTimeRangeItem, 0 );
	DBG_ASSERT( SfxPoolItem::operator==( rItem ), "unequal type" );

	DateTime aThisRange( aEndDateTime - aStartDateTime );
	DateTime aRange(((const SfxDateTimeRangeItem&)rItem).aEndDateTime -
					((const SfxDateTimeRangeItem&)rItem).aStartDateTime );

	if( aRange < aThisRange )
		return -1;
	else if( aRange == aThisRange )
		return 0;
	else
		return 1;
}

// -----------------------------------------------------------------------

SfxPoolItem* SfxDateTimeRangeItem::Create( SvStream& rStream, USHORT ) const
{
	DBG_CHKTHIS(SfxDateTimeRangeItem, 0);

	ULONG	nStartDate, nEndDate;
	long	nStartTime, nEndTime;

	rStream >> nStartDate;
	rStream >> nStartTime;

	rStream >> nEndDate;
	rStream >> nEndTime;

	DateTime aStartDT, aEndDT;

	aStartDT.SetDate( nStartDate );
	aStartDT.SetTime( nStartTime );

	aEndDT.SetDate( nEndDate );
	aEndDT.SetTime( nEndTime );

	return new SfxDateTimeRangeItem( Which(), aStartDT, aEndDT );
}

// -----------------------------------------------------------------------

SvStream& SfxDateTimeRangeItem::Store( SvStream& rStream, USHORT ) const
{
	DBG_CHKTHIS( SfxDateTimeRangeItem, 0 );

	rStream << aStartDateTime.GetDate();
	rStream << aStartDateTime.GetTime();

	rStream << aEndDateTime.GetDate();
	rStream << aEndDateTime.GetTime();

	return rStream;
}

// -----------------------------------------------------------------------

SfxPoolItem* SfxDateTimeRangeItem::Clone( SfxItemPool* ) const
{
	DBG_CHKTHIS( SfxDateTimeRangeItem, 0 );

	return new SfxDateTimeRangeItem( *this );
}
// -----------------------------------------------------------------------

SfxItemPresentation SfxDateTimeRangeItem::GetPresentation
(
	SfxItemPresentation 	ePresentation,
	SfxMapUnit				eCoreMetric,
	SfxMapUnit				ePresentationMetric,
	XubString& 				rText,
	const International *	pInternational
)	const
{
	DBG_CHKTHIS(SfxDateTimeRangeItem, 0);
	DateTime aRange(aEndDateTime - aStartDateTime);
	if (pInternational)
	{
		rText = pInternational->GetDate(aRange);
		rText.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
		rText += pInternational->GetTime(aRange);
	}
	else
	{
		DBG_WARNING("SfxDateTimeRangeItem::GetPresentation():"
					 " Using default International");
		const International aInternational;
		rText = aInternational.GetDate(aRange);
		rText.AppendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
		rText += aInternational.GetTime(aRange);
	}
	return SFX_ITEM_PRESENTATION_NAMELESS;
}

//----------------------------------------------------------------------------
// virtual
BOOL SfxDateTimeRangeItem::PutValue( const com::sun::star::uno::Any& rVal,
						   			 BYTE nMemberId )
{
	com::sun::star::util::DateTimeRange aValue;
	if ( rVal >>= aValue )
	{
		aStartDateTime = DateTime( Date( aValue.StartDay,
			  						     aValue.StartMonth,
			  						     aValue.StartYear ),
							  	   Time( aValue.StartHours,
			  							 aValue.StartMinutes,
			  							 aValue.StartSeconds,
			  							 aValue.StartHundredthSeconds ) );
		aEndDateTime   = DateTime( Date( aValue.EndDay,
			  						     aValue.EndMonth,
			  						     aValue.EndYear ),
							  	   Time( aValue.EndHours,
			  							 aValue.EndMinutes,
			  							 aValue.EndSeconds,
			  							 aValue.EndHundredthSeconds ) );
		return TRUE;
	}

	DBG_ERROR( "SfxDateTimeRangeItem::PutValue - Wrong type!" );
	return FALSE;
}

//----------------------------------------------------------------------------
// virtual
BOOL SfxDateTimeRangeItem::QueryValue( com::sun::star::uno::Any& rVal,
							 	       BYTE nMemberId ) const
{
	com::sun::star::util::DateTimeRange aValue( aStartDateTime.Get100Sec(),
    									   		aStartDateTime.GetSec(),
   										   		aStartDateTime.GetMin(),
    									   		aStartDateTime.GetHour(),
    									   		aStartDateTime.GetDay(),
    									   		aStartDateTime.GetMonth(),
    									   		aStartDateTime.GetYear(),
												aEndDateTime.Get100Sec(),
    									   		aEndDateTime.GetSec(),
   										   		aEndDateTime.GetMin(),
    									   		aEndDateTime.GetHour(),
    									   		aEndDateTime.GetDay(),
    									   		aEndDateTime.GetMonth(),
    									   		aEndDateTime.GetYear() );
	rVal <<= aValue;
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
/*
	$Log: dtritem.cxx,v $
	Revision 1.1.1.1  2000/09/18 16:59:00  hr
	initial import
	
	Revision 1.13  2000/09/18 14:13:34  willem.vandorp
	OpenOffice header added.
	
	Revision 1.12  2000/08/31 13:41:27  willem.vandorp
	Header and footer replaced
	
	Revision 1.11  2000/04/12 08:01:39  sb
	Adapted to Unicode.
	
	Revision 1.10  2000/03/22 11:49:37  kso
	Removed: SmartUno leftovers.
	
	Revision 1.9  2000/02/21 09:33:42  kso
	#68274# - Added: [Put|QueryValue( ... com::sun::star::uno::Any ... )

	Revision 1.8  2000/02/09 16:22:56  hr
	#70473# changes for unicode ( patched by automated patchtool )

	Revision 1.7  2000/01/25 11:43:33  hr
	#65293#: sv -> vcl

	Revision 1.6  1999/11/19 17:12:35  sb
	Module structure transposition.

	Revision 1.5  1999/09/01 13:41:50  sb
	#66082# Changed DBG_ERRORs into DBG_WARNINGS.

	Revision 1.4  1999/09/01 06:52:34  sb
	#66082# Added International param to SfxPoolItem::GetPresentation().

	Revision 1.3  1999/07/19 06:48:14  dv
	chg: Syntax in Kommentar wg. cvs

	Revision 1.2  1998/07/17 08:23:18  KSO
	#52764# ( StarONE-API for CHAOS ) SfxDateTimeRangeItem::[Put|Query]Value(...)

	  Rev 1.1   17 Jul 1998 10:23:18   KSO
   #52764# ( StarONE-API for CHAOS ) SfxDateTimeRangeItem::[Put|Query]Value(...)

	  Rev 1.0   04 Feb 1998 10:19:26   OK
   NEW: SfxDateTimeRangeItem

*/

