/*************************************************************************
 *
 *  $RCSfile: poolitem.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#ifndef _ARGS_HXX
#include <args.hxx>
#endif
#ifndef _SFXPOOLITEM_HXX
#include <poolitem.hxx>
#endif

// STATIC DATA -----------------------------------------------------------

DBG_NAME(SfxPoolItem);
DBG_NAME(SfxVoidItem);
DBG_NAME(SfxInvalidItem);
DBG_NAME(SfxItemHandle);

BYTE nSfxFlag8Val[8] =
{
	1, 2, 4, 8, 16, 32, 64, 128
};

USHORT nSfxFlag16Val[16] =
{
	1, 2, 4, 8, 16, 32, 64, 128, 256, 512,
	1024, 2048, 4096, 8192, 16384, 32768
};

ULONG nSfxFlag32Val[32] =
{
	0x1L, 0x2L, 0x4L, 0x8L,
	0x10L, 0x20L, 0x40L, 0x80L,
	0x100L, 0x200L, 0x400L, 0x800L,
	0x1000L, 0x2000L, 0x40000L, 0x8000L,
	0x10000L, 0x20000L, 0x40000L, 0x80000L,
	0x100000L, 0x200000L, 0x400000L, 0x800000L,
	0x1000000L, 0x2000000L, 0x4000000L, 0x8000000L,
	0x10000000L, 0x20000000L, 0x40000000L, 0x80000000L
};

// RTTI ------------------------------------------------------------------

TYPEINIT0(SfxPoolItem);
TYPEINIT1(SfxVoidItem, SfxPoolItem);
TYPEINIT1(SfxInvalidItem, SfxPoolItem);
TYPEINIT1(SfxSetItem, SfxPoolItem);
TYPEINIT1(SfxItemChangedHint, SfxHint);

// ------------------------------------------------------------------------

IMPL_PTRHINT(SfxPoolItemHint,SfxPoolItem)

// SfxPoolItem -----------------------------------------------------------
SfxPoolItem::SfxPoolItem( USHORT nW )
	: nRef( 0 ),
	  nWhich( nW )
{
	DBG_CTOR(SfxPoolItem, 0);
	DBG_ASSERT(nW <= SHRT_MAX, "Which Bereich ueberschritten");
}

// -----------------------------------------------------------------------
SfxPoolItem::SfxPoolItem( const SfxPoolItem& rCpy )
	: nRef( 0 ),				// wird ja ein neues Object!
	  nWhich( rCpy.Which() )	// Funktion rufen wg. ChkThis()
{
	DBG_CTOR(SfxPoolItem, 0);
}

// ------------------------------------------------------------------------
int SfxPoolItem::Compare( const SfxPoolItem& rWith ) const
{
	return 0;
}

// ------------------------------------------------------------------------
#if SUPD>=565
int SfxPoolItem::Compare( const SfxPoolItem& rWith, const International& ) const
{
	return Compare( rWith );
}
#endif

// ------------------------------------------------------------------------
int SfxPoolItem::operator==( const SfxPoolItem& rCmp ) const
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return rCmp.Type() == Type();
}

// -----------------------------------------------------------------------
#ifndef TF_POOLABLE

int SfxPoolItem::IsPoolable() const
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return TRUE;
}
#endif

// -----------------------------------------------------------------------
SfxPoolItem* SfxPoolItem::Create(SvStream &, USHORT nVersion) const
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return Clone(0);
}

// -----------------------------------------------------------------------
USHORT SfxPoolItem::GetVersion( USHORT nFileFormatVersion ) const
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return 0;
}

// -----------------------------------------------------------------------
SvStream& SfxPoolItem::Store(SvStream &rStream, USHORT nItemVersion ) const
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return rStream;
}

//============================================================================
// static
bool SfxPoolItem::readByteString(SvStream & rStream, UniString & rString)
{
	rStream.ReadByteString(rString);
	return rStream.GetError() == ERRCODE_NONE;
}

//============================================================================
// static
void SfxPoolItem::writeByteString(SvStream & rStream,
								  UniString const & rString)
{
	rStream.WriteByteString(rString);
}

//============================================================================
// static
bool SfxPoolItem::readUnicodeString(SvStream & rStream, UniString & rString,
									bool bUnicode)
{
	rStream.ReadByteString(rString,
						   bUnicode ? RTL_TEXTENCODING_UCS2 :
						              rStream.GetStreamCharSet());
	return rStream.GetError() == ERRCODE_NONE;
}

//============================================================================
// static
void SfxPoolItem::writeUnicodeString(SvStream & rStream,
									 UniString const & rString)
{
	rStream.WriteByteString(rString, RTL_TEXTENCODING_UCS2);
}

// ------------------------------------------------------------------------
SfxItemPresentation SfxPoolItem::GetPresentation
(
	SfxItemPresentation	ePresentation,       // IN:  wie formatiert werden soll
	SfxMapUnit			eCoreMetric,		 // IN:  Ma\seinheit des SfxPoolItems
	SfxMapUnit			ePresentationMetric, // IN:  Wunsch-Ma\einheit der Darstellung
	XubString&			rText,				 // OUT: textuelle Darstellung
	const International *
) 	const

/*	[Beschreibung]

	"Uber diese virtuelle Methode kann von den SfxPoolItem-Subklassen
	eine textuelle Datstellung des Wertes erhalten werden. Sie sollte
	von allen UI-relevanten SfxPoolItem-Subklassen "uberladen werden.

	Da die Ma\seinheit des Wertes im SfxItemPool nur "uber
	<SfxItemPool::GetMetric(USHORT)const> erfragbar ist, und nicht etwa
	in der SfxPoolItem-Instanz oder -Subklasse  verf"ugbar ist, wird die
	eigene Ma\seinheit als 'eCoreMetric' "ubergeben.

	Die darzustellende Ma\seinheit wird als 'ePresentationMetric'
	"ubergeben.


	[R"uckgabewert]

	SfxItemPresentation		SFX_ITEM_PRESENTATION_NONE
							es konnte keine Text-Darstellung erzeugt werden

							SFX_ITEM_PRESENTATION_NAMELESS
							es konnte eine Text-Darstellung (ggf. mit
							Ma\seinheit) erzeugt werden, die jedoch keine
							semantische Bedeutung enth"alt

							SFX_ITEM_PRESENTATION_COMPLETE
							es konnte eine komplette Text-Darstellung mit
							semantischer Bedeutung (und ggf. Ma\seinheit)
							erzeugt werden


	[Beispiele]

	pSvxFontItem->GetPresentation( SFX_PRESENTATION_NAMELESS, ... )
		"12pt" mit return SFX_ITEM_PRESENTATION_NAMELESS

	pSvxColorItem->GetPresentation( SFX_PRESENTATION_COMPLETE, ... )
		"rot" mit return SFX_ITEM_PRESENTATION_NAMELESS
		(das das SvxColorItem nicht wei\s, was f"ur eine Farbe es darstellt,
		kann es keinen Namen angeben, was durch den Returnwert mitgeteilt wird.

	pSvxBorderItem->GetPresentation( SFX_PRESENTATION_COMPLETE, ... )
		"1cm oberer Rand, 2cm linker Rand, 0,2cm unterer Rand, ..."
*/

{
	return SFX_ITEM_PRESENTATION_NONE;
}

// SfxVoidItem ------------------------------------------------------------
SfxVoidItem::SfxVoidItem( USHORT nWhich ):
	SfxPoolItem(nWhich)
{
	DBG_CTOR(SfxVoidItem, 0);
}

// SfxVoidItem ------------------------------------------------------------
SfxVoidItem::SfxVoidItem( const SfxVoidItem& rCopy):
	SfxPoolItem(rCopy)
{
	DBG_CTOR(SfxVoidItem, 0);
}

// ------------------------------------------------------------------------
int SfxVoidItem::operator==( const SfxPoolItem& rCmp) const
{
	DBG_CHKTHIS(SfxVoidItem, 0);
	DBG_ASSERT( SfxPoolItem::operator==( rCmp ), "unequal type" );
	return TRUE;
}

// ------------------------------------------------------------------------
SfxItemPresentation SfxVoidItem::GetPresentation
(
	SfxItemPresentation 	ePresentation,
	SfxMapUnit				eCoreMetric,
	SfxMapUnit				ePresentationMetric,
	XubString& 				rText,
	const International *
)	const
{
	DBG_CHKTHIS(SfxVoidItem, 0);
	rText.AssignAscii(RTL_CONSTASCII_STRINGPARAM("Void"));
	return SFX_ITEM_PRESENTATION_NAMELESS;
}

// ------------------------------------------------------------------------
SfxPoolItem* SfxVoidItem::Clone(SfxItemPool *) const
{
	DBG_CHKTHIS(SfxVoidItem, 0);
	return new SfxVoidItem(*this);
}

// SfxInvalidItem ---------------------------------------------------------
SfxInvalidItem::SfxInvalidItem( USHORT nWhich, const SfxPoolItem &rDefault ):
	SfxPoolItem(nWhich),
	pDefaultItem(&rDefault)
{
	DBG_CTOR(SfxInvalidItem, 0);
}

// ------------------------------------------------------------------------
SfxInvalidItem::SfxInvalidItem( const SfxInvalidItem& rCopy):
	SfxPoolItem(rCopy),
	pDefaultItem(rCopy.pDefaultItem)
{
	DBG_CTOR(SfxInvalidItem, 0);
	//! pDefaultItem->ReleaseRef?
}

// ------------------------------------------------------------------------
SfxInvalidItem::~SfxInvalidItem()
{
	DBG_DTOR(SfxInvalidItem, 0);
}

// ------------------------------------------------------------------------
int SfxInvalidItem::operator==( const SfxPoolItem& rCmp) const
{
	DBG_CHKTHIS(SfxInvalidItem, 0);
	DBG_ASSERT( SfxPoolItem::operator==(rCmp), "unequal type" );
	return *pDefaultItem == *((SfxInvalidItem&)rCmp).pDefaultItem;
}

// ------------------------------------------------------------------------
SfxItemPresentation SfxInvalidItem::GetPresentation
(
	SfxItemPresentation 	ePresentation,
	SfxMapUnit				eCoreMetric,
	SfxMapUnit				ePresentationMetric,
	XubString& 				rText,
	const International *
)	const
{
	DBG_CHKTHIS(SfxInvalidItem, 0);
	rText.AssignAscii(RTL_CONSTASCII_STRINGPARAM("Invalid"));
	return SFX_ITEM_PRESENTATION_NAMELESS;
}

// ------------------------------------------------------------------------
SfxPoolItem* SfxInvalidItem::Clone(SfxItemPool *) const
{
	DBG_CHKTHIS(SfxInvalidItem, 0);
	return new SfxInvalidItem(*this);
}

// ------------------------------------------------------------------------
SfxPoolItem* SfxInvalidItem::Create(SvStream &, USHORT nVersion) const
{
	DBG_CHKTHIS(SfxInvalidItem, 0);
	DBG_ERROR("SfxInvalidItem::Create() ist sinnlos");
	return Clone();
}

// ------------------------------------------------------------------------
SvStream& SfxInvalidItem::Store(SvStream &rStream, USHORT nItemVersion ) const
{
	DBG_CHKTHIS(SfxInvalidItem, 0);
	DBG_ERROR("SfxInvalidItem::Store() ist sinnlos");
	return rStream;
}

// SfxItemHandle ----------------------------------------------------------
SfxItemHandle::SfxItemHandle(SfxPoolItem &rItem):
	pRef(new USHORT(1)),
	pItem(rItem.Clone(0))
{
	DBG_CTOR(SfxItemHandle, 0);
}

// ------------------------------------------------------------------------
SfxItemHandle::SfxItemHandle(const SfxItemHandle &rCopy):
	pRef(rCopy.pRef),
	pItem(rCopy.pItem)
{
	DBG_CTOR(SfxItemHandle, 0);
	++(*pRef);
}

// ------------------------------------------------------------------------
const SfxItemHandle &SfxItemHandle::operator=(const SfxItemHandle &rCopy)
{
	DBG_CHKTHIS(SfxItemHandle, 0);
	if(&rCopy == this || pItem == rCopy.pItem)
		return *this;
	--(*pRef);
	if(!(*pRef))
	{
		delete pItem;
		pItem = 0;
	}
	pRef = rCopy.pRef;
	++(*pRef);
	pItem = rCopy.pItem;
	return *this;
}

// ------------------------------------------------------------------------
SfxItemHandle::~SfxItemHandle()
{
	DBG_DTOR(SfxItemHandle, 0);
	--(*pRef);
	if(!(*pRef)) {
		delete pRef; pRef = 0;
		delete pItem; pItem = 0;
	}
}

// ------------------------------------------------------------------------
#if SUPD<548
SfxPoolItem* SfxPoolItem::DeleteOnIdle()
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return DeleteItemOnIdle( this );
}

// ------------------------------------------------------------------------
void SfxPoolItem::DeleteOnIdleItems()
{
	::DeleteOnIdleItems();
}
#endif

// ------------------------------------------------------------------------
int SfxPoolItem::ScaleMetrics( long /*lMult*/, long /*lDiv*/ )
{
	return 0;
}

// ------------------------------------------------------------------------
int SfxPoolItem::HasMetrics() const
{
	return 0;
}

// -----------------------------------------------------------------------
void SfxPoolItem::GetVersion() const
{
	DBG_ERROR( "dummy called" );
}

// -----------------------------------------------------------------------
void SfxPoolItem::Store(SvStream &rStream) const
{
	DBG_ERROR( "dummy called" );
}

// -----------------------------------------------------------------------

BOOL SfxPoolItem::QueryValue( com::sun::star::uno::Any& rVal, BYTE nMemberId ) const
{
	DBG_ERROR("There is no implementation for QueryValue for this item!");
	return FALSE;
}

// -----------------------------------------------------------------------

BOOL SfxPoolItem::PutValue( const com::sun::star::uno::Any& rVal, BYTE nMemberId )
{
	DBG_ERROR("There is no implementation for PutValue for this item!");
	return FALSE;
}

BOOL SfxPoolItem::importXML( const rtl::OUString& rValue, USHORT nMemberId, const SvXMLUnitConverter& rUnitConverter )
{
	DBG_ERROR("There is no implementation for importXML for this item!");
	return FALSE;
}

BOOL SfxPoolItem::exportXML( rtl::OUString& rValue, USHORT nMemberId, const SvXMLUnitConverter& rUnitConverter ) const
{
	DBG_ERROR("There is no implementation for exportXML for this item!");
	return FALSE;
}

BOOL SfxPoolItem::equalsXML( const SfxPoolItem& rCmp ) const
{
	DBG_CHKTHIS(SfxPoolItem, 0);
	return rCmp == *this;
}


