/*************************************************************************
 *
 *  $RCSfile: whiter.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
// INCLUDE ---------------------------------------------------------------

#pragma hdrstop

#include "whiter.hxx"
#include "itemset.hxx"
#include "segmentc.hxx"

DBG_NAME(SfxWhichIter);

SEG_EOFGLOBALS()

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(whiter_01)

SfxWhichIter::SfxWhichIter( const SfxItemSet& rSet, USHORT nFromWh, USHORT nToWh ):
	pRanges(rSet.GetRanges()),
	pStart(rSet.GetRanges()),
	nOfst(0), nFrom(nFromWh), nTo(nToWh)
{
	DBG_CTOR(SfxWhichIter, 0);
	if ( nFrom > 0 )
		FirstWhich();
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(whiter_02)

SfxWhichIter::~SfxWhichIter()
{
	DBG_DTOR(SfxWhichIter, 0);
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(whiter_03)

USHORT SfxWhichIter::NextWhich()
{
	DBG_CHKTHIS(SfxWhichIter, 0);
	while ( 0 != *pRanges )
	{
		const USHORT nLastWhich = *pRanges + nOfst;
		++nOfst;
		if (*(pRanges+1) == nLastWhich)
		{
			pRanges += 2;
			nOfst = 0;
		}
		USHORT nWhich = *pRanges + nOfst;
		if ( 0 == nWhich || ( nWhich >= nFrom && nWhich <= nTo ) )
			return nWhich;
	}
	return 0;
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(whiter_04)

USHORT  SfxWhichIter::PrevWhich()
{
	DBG_CHKTHIS(SfxWhichIter, 0);
	while ( pRanges != pStart || 0 != nOfst )
	{
		if(nOfst)
			--nOfst;
		else {
			pRanges -= 2;
			nOfst = *(pRanges+1) - (*pRanges);
		}
		USHORT nWhich = *pRanges + nOfst;
		if ( nWhich >= nFrom && nWhich <= nTo )
			return nWhich;
	}
	return 0;
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(whiter_05)

USHORT SfxWhichIter::FirstWhich()
{
	DBG_CHKTHIS(SfxWhichIter, 0);
	pRanges = pStart;
	nOfst = 0;
	if ( *pRanges >= nFrom && *pRanges <= nTo )
		return *pRanges;
	return NextWhich();
}

// -----------------------------------------------------------------------
#pragma SEG_FUNCDEF(whiter_06)

USHORT SfxWhichIter::LastWhich()
{
	DBG_CHKTHIS(SfxWhichIter, 0);
	while(*pRanges)
		++pRanges;
	nOfst = 0;
	USHORT nWhich = *(pRanges-1);
	if ( nWhich >= nFrom && nWhich <= nTo )
		return nWhich;
	return PrevWhich();
}

/*------------------------------------------------------------------------

	$Log: whiter.cxx,v $
	Revision 1.1.1.1  2000/09/18 16:59:01  hr
	initial import
	
	Revision 1.8  2000/09/18 14:13:36  willem.vandorp
	OpenOffice header added.
	
	Revision 1.7  2000/08/31 13:41:29  willem.vandorp
	Header and footer replaced
	
	Revision 1.6  1994/10/25 10:54:50  MI
	#pragma hdrstop
	

	  Rev 1.5   25 Oct 1994 11:54:50   MI
   #pragma hdrstop

	  Rev 1.4   22 Sep 1994 17:37:40   MI
   Undo und ItemSet immer mit Pool

	  Rev 1.3   30 Aug 1994 15:36:34   MI
   mit Bereich

	  Rev 1.2   25 Aug 1994 18:04:20   VB
   GetRanges() -> GetRanges_Impl()
   NextWhich() liefert den Folgewert

	  Rev 1.1   23 Aug 1994 14:01:26   VB
   Code Review, Teil 1

	  Rev 1.0   22 Aug 1994 11:10:42   VB
   Initial revision.

------------------------------------------------------------------------*/

#pragma SEG_EOFMODULE

