/*************************************************************************
 *
 *  $RCSfile: config.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// INCLUDE ---------------------------------------------------------------

#include "config.hxx"

#pragma hdrstop

#ifndef _INTN_HXX //autogen
#include <tools/intn.hxx>
#endif
#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#ifndef _MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _RC_HXX //autogen
#include <vcl/rc.hxx>
#endif
#ifndef _APP_HXX //autogen
#include <vcl/svapp.hxx>
#endif
#ifndef _TIMER_HXX //autogen
#include <vcl/timer.hxx>
#endif
#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif
#ifndef _TOOLS_TEMPFILE_HXX
#include <tools/tempfile.hxx>
#endif

#ifndef _COM_SUN_STAR_UCB_COMMANDABORTEDEXCEPTION_HPP_
#include <com/sun/star/ucb/CommandAbortedException.hpp>
#endif

#include <tools/tempfile.hxx>
#include <tools/urlobj.hxx>
#include <ucbhelper/content.hxx>
using namespace ucb;

#include <vos/mutex.hxx>
#ifndef _VOS_NO_NAMESPACE
	using namespace vos;
#endif
#include <stdio.h>
#include "confitem.hxx"
#include "svtools.hrc"
#include "svarray.hxx"

// STATIC DATA -----------------------------------------------------------

static const char __FAR_DATA pHeader[] = "Star Framework Config File";
static const USHORT nVersion = 26;
static const char cCtrlZ = 26;

#define ITEM_NOT_IN_FILE			-1L
#define CFG_ITEM_NOTFOUND			USHRT_MAX
#define CFG_STREAM_BUFFER_SIZE		5000

DBG_NAME(SvConfigManager);

class SvConfigWarn_Impl
{
private:
	String aWarning;

public:
	SvConfigWarn_Impl( const String& rStr );

	DECL_LINK( TimerHdl, Timer*);
};

SvConfigWarn_Impl::SvConfigWarn_Impl( const String& rStr ) : aWarning( rStr )
{
	Timer* pTimer = new Timer;
	pTimer->SetTimeoutHdl( LINK( this, SvConfigWarn_Impl, TimerHdl ) );
	pTimer->SetTimeout( 50 );
	pTimer->Start();
}

IMPL_LINK( SvConfigWarn_Impl, TimerHdl, Timer*, pTimer )
{
	delete pTimer;
	WarningBox( 0, WinBits(WB_OK), aWarning ).Execute();
	delete this;
	return 0L;
}

//-----------------------------------------------------------------------------
// struct SvConfigItemTask_Impl h"alt f"ur jede angemeldete Task Platz f"ur ein ConfigItem
//
struct SvConfigItemTask_Impl
{
	SvConfigItem*	pCItem;
	SfxBroadcaster*	pTask;

	SvConfigItemTask_Impl( SvConfigItem* pItem, SfxBroadcaster* pTsk=NULL )
		: pCItem( pItem )
		, pTask( pTsk )
	{}
};

typedef SvConfigItemTask_Impl* SvConfigItemTaskPtr;
SV_DECL_PTRARR_DEL(SvConfigItemTaskArr_Impl, SvConfigItemTaskPtr, 2, 2);
SV_IMPL_PTRARR(SvConfigItemTaskArr_Impl, SvConfigItemTaskPtr);

//-----------------------------------------------------------------------------
// struct SvConfigFileItem_Impl h"alt Information fuer das Directory der cfg-Datei
//
struct SvConfigFileItem_Impl
{
	SvConfigItemTaskArr_Impl	aItems; 	// evtl. verbundene ConfigItems
	long						lFilePos;	// Zeiger in cfg-Datei
	long						lLength;	// Laenge in cfg-Datei
	USHORT						nType;		// ConfigId
	String						aName;		// Name des ConfigItems

					SvConfigFileItem_Impl( SvConfigItem *pItem, SfxBroadcaster* pTask );
					SvConfigFileItem_Impl( USHORT, String& );

	BOOL			IsModified();
	void 			Move( SvConfigItemTaskArr_Impl& rArr, SvConfigManager* pMgr );
	void 			Move( SvConfigItemTaskArr_Impl& rArr, SvConfigManager* pMgr, SfxBroadcaster* pTask );
	SvConfigItemTask_Impl*	GetTaskItem( SfxBroadcaster* pTask );
	BOOL 			Remove( SvConfigItem* pCItem );
	void			Store( SvStream& );
	BOOL			IsInternal();
	BOOL			IsDefault()
					{ return (lFilePos == ITEM_NOT_IN_FILE); }
	void			Reset()
					{
						lFilePos = ITEM_NOT_IN_FILE;
						lLength	= 0;
					}
};

typedef SvConfigFileItem_Impl* SvConfigFileItemPtr;
SV_DECL_PTRARR(SvConfigFileItemBaseArr_Impl, SvConfigFileItemPtr, 30, 5);

class SvConfigFileItemArr_Impl : public SvConfigFileItemBaseArr_Impl
{
public:
	SfxBroadcaster*	pActiveTask;

	SvConfigFileItemArr_Impl() : pActiveTask( NULL ) {}
};

// ------------------------------------------------------------------------

SvConfigFileItem_Impl::SvConfigFileItem_Impl( SvConfigItem *pCItem, SfxBroadcaster* pTask ):
	lFilePos(ITEM_NOT_IN_FILE),
	lLength(0),
	nType(0)
{
	nType = pCItem->GetType();
	aItems.Insert( new SvConfigItemTask_Impl( pCItem, pTask ), aItems.Count() );
}

// ------------------------------------------------------------------------

SvConfigFileItem_Impl::SvConfigFileItem_Impl( USHORT nTyp, String& aString ) :
	lFilePos(ITEM_NOT_IN_FILE),
	lLength(0),
	nType(nTyp),
	aName(aString)
{
}

SvConfigItemTask_Impl* SvConfigFileItem_Impl::GetTaskItem( SfxBroadcaster* pTask )
{
	for ( USHORT n=0; n<aItems.Count(); n++ )
		if ( aItems[n]->pTask == pTask )
			return aItems[n];
	return NULL;
}

BOOL SvConfigFileItem_Impl::IsModified()
{
	for ( USHORT n=0; n<aItems.Count(); n++ )
		if ( aItems[n]->pCItem->IsModified() )
			return TRUE;
	return FALSE;
}
BOOL SvConfigFileItem_Impl::Remove( SvConfigItem* pCItem )
{
	for ( USHORT n=0; n<aItems.Count(); n++ )
		if ( aItems[n]->pCItem == pCItem )
		{
			aItems.DeleteAndDestroy( n );
			return TRUE;
		}

	return FALSE;
}

// ------------------------------------------------------------------------

void SvConfigFileItem_Impl::Store(SvStream& rStream)
{
    rStream << nType << lFilePos << lLength;
    rStream.WriteByteString( aName );
}

void SvConfigFileItem_Impl::Move( SvConfigItemTaskArr_Impl& rArr, SvConfigManager* pMgr )
{
	for ( USHORT n=0; n<rArr.Count(); n++ )
	{
		aItems.Insert( rArr[n], aItems.Count() );
		rArr[n]->pCItem->Connect( pMgr );
	}

	rArr.Remove( 0, rArr.Count() );
}

void SvConfigFileItem_Impl::Move( SvConfigItemTaskArr_Impl& rArr, SvConfigManager* pMgr, SfxBroadcaster* pTask )
{
	for ( USHORT n=0; n<rArr.Count(); )
	{
		if ( rArr[n]->pTask == pTask )
		{
			aItems.Insert( rArr[n], aItems.Count() );
			rArr[n]->pCItem->Connect( pMgr );
			n++;
		}
		else
			rArr.Remove( n );
	}
}

// ------------------------------------------------------------------------
// Ermittelt, ob das ConfigItem nur intern verwendet werden soll, also nicht
// im UI (z.B. im Organizer) angezeigt werden soll.

BOOL SvConfigFileItem_Impl::IsInternal()
{
	BOOL bInternal = FALSE;
	SvConfigItem *pCItem = aItems.Count() ? aItems[0]->pCItem : NULL;
	if ( pCItem )
	{
		// ConfigItem fragen
		bInternal = pCItem->Internal();
		if ( !bInternal )
		{
			// Der Name k"onnte sich ge"andert haben
			String aNam = pCItem->GetName();
			if (aNam.Len())
				aName = aNam;
#ifdef DBG_UTIL
			// In Debug-Version Typkennung anzeigen, da Fehler vorliegt
			if (!aName.Len())
				aName = String(nType);
#endif
		}
	}
	else if ( !aName.Len() )
	{
		// Wenn kein ConfigItem da ist, bleibt nur der Name als Kriterium
		bInternal = TRUE;
	}

	// Interne Items werden nicht angezeigt
	return bInternal;
}

// ------------------------------------------------------------------------
SvConfigFileItem_Impl *FindItem_Impl(SvConfigFileItemArr_Impl *pArr, USHORT nType)
{
	if (pArr)
	{
		const USHORT nCount = pArr->Count();
		for(USHORT i = 0; i < nCount; ++i)
		{
			if((*pArr)[i]->nType == nType)
				return (*pArr)[i];
		}
	}
	return 0;
}

void CopyConfigItems_Impl(SvConfigFileItem_Impl *pItem,   		// Dest
								long lPos,    					// SourcePos
								SvStream *pInStream,            // Dest
								SvStream *pOutStream)           // Source
{
	pOutStream->Seek(lPos);
	pInStream->Seek(pItem->lFilePos);

	// Mehr als 64 KB auf einmal is nich ...
	if (pItem->lLength > (long) USHRT_MAX)
	{
		USHORT nLength = USHRT_MAX;
		char *pBuf = new char[nLength];

		USHORT nTimes = (USHORT) ( pItem->lLength / (long) USHRT_MAX ) + 1;
		for (USHORT i=0; i<nTimes; i++)
		{
			if (i == nTimes-1)
				nLength = (USHORT) (pItem->lLength - (long) i * (long) USHRT_MAX);
			pOutStream->Read(pBuf, nLength);
			pInStream->Write(pBuf, nLength);
		}

		delete pBuf;
	}
	else
	{
		USHORT nLength = (USHORT) pItem->lLength;
		char *pBuf = new char[nLength];
		pOutStream->Read(pBuf, nLength);
		pInStream->Write(pBuf, nLength);
		delete pBuf;
	}
}

// ------------------------------------------------------------------------

SvConfigManager::SvConfigManager( SvConfigManager *pMgr ):
	pFileItemArr(0),
	nErrno(ERR_NO),
	bFileIO(FALSE),
	lStart(ITEM_NOT_IN_FILE),
	lDirPos(ITEM_NOT_IN_FILE),
	pParentMgr(pMgr),
	bActive(FALSE),
	bModified(FALSE)
{
	DBG_CTOR( SvConfigManager, 0 );

	pFileItemArr = new SvConfigFileItemArr_Impl;
	if ( pParentMgr ) 					// nur eine HierarchieStufe
	{
		while ( pParentMgr->pParentMgr )
			pParentMgr = pParentMgr->pParentMgr;
	}

	// Zun"achst den Arbeits-Stream anlegen
	nErrno = ERR_NO;
	TempFile aTempFile;
	aTempFileName = aTempFile.GetName();
	pStream = new SvFileStream( aTempFileName, STREAM_STD_READWRITE );
	pStream->SetBufferSize( CFG_STREAM_BUFFER_SIZE );
}

// ------------------------------------------------------------------------

SvConfigManager::~SvConfigManager()
{
	DBG_DTOR(SvConfigManager, 0);

	USHORT nItems = pFileItemArr->Count();
	BOOL bEmpty=TRUE;
	for (USHORT n=0; n<nItems; n++)
	{
		USHORT nCount = (*pFileItemArr)[n]->aItems.Count();
		if ( nCount )
		{
			bEmpty=FALSE;
			for ( USHORT i=0; i<nCount; i++)
			{
				SvConfigItem *pCItem = (*pFileItemArr)[n]->aItems[i]->pCItem;
				pCItem->ReleaseConfigManager();

#ifdef DBG_UTIL
                ByteString aMsg( "SvCfgMgr(" );
                aMsg += ByteString::CreateFromInt32( (long) this );
                aMsg += ',';
                aMsg += ByteString::CreateFromInt32((USHORT)bActive);
				aMsg += ")::Dtor(";
                aMsg += ByteString::CreateFromInt32( (long) pCItem);
                aMsg += ')';
                DBG_TRACE(aMsg.GetBuffer());
#endif
			}
		}
	}

	if ( pParentMgr && !bEmpty )
		DBG_WARNING( "W1:Da sind noch ConfigItems!");

	const USHORT nCount = pFileItemArr ? pFileItemArr->Count() : 0;
	for ( USHORT i = 0; i < nCount; ++i )
		delete (*pFileItemArr)[i];

	delete pFileItemArr;
	delete pStream;

	INetURLObject aObj( aTempFileName, INET_PROT_FILE );
	try
	{
		Content aCnt( aObj.GetMainURL(), ::com::sun::star::uno::Reference< ::com::sun::star::ucb::XCommandEnvironment > () );
		aCnt.executeCommand( rtl::OUString::createFromAscii( "delete" ),
							 	::com::sun::star::uno::makeAny( sal_Bool( sal_True ) ) );
	}
	catch( ContentCreationException& e )
	{
		switch ( e.getReason() )
		{
			case ContentCreationException::NO_CONTENT_BROKER :
				DBG_ERRORFILE( "no content broker" );
				break;

			default:
				DBG_ERRORFILE( "ContentCreationException" );
		}
	}
	catch( ::com::sun::star::ucb::CommandAbortedException& )
	{
		DBG_ERRORFILE( "CommandAbortedException" );
	}
	catch( ... )
	{
		DBG_ERRORFILE( "Any other exception" );
	}
}

// ------------------------------------------------------------------------

void SvConfigManager::Activate( SvConfigManager *pMgr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	if (this == pMgr)
		return;

	if (pMgr)
	{
		ReconfigureItems(pMgr);
		pMgr->bActive = FALSE;
	}

	bActive = TRUE;
}

void SvConfigManager::ActivateTask( SfxBroadcaster* pTask )
{
	pFileItemArr->pActiveTask = pTask;
	if ( pParentMgr )
		pParentMgr->pFileItemArr->pActiveTask = pTask;
}

// ----------------------------------------------------------------------------
// Liefert Anzahl der nach au\sen sichtbaren ConfigItems.

USHORT SvConfigManager::GetItemCount() const
{
	OGuard aGuard( Application::GetSolarMutex() );

	USHORT nCount=0;
	for (USHORT n=0; n<pFileItemArr->Count(); n++)
	{
		SvConfigFileItem_Impl *pItem = (*pFileItemArr)[n];
		// Bei jedem nicht-intern Item Anzahl hochz"ahlen
		if ( !pItem->IsInternal() )
			nCount++;
	}
	return nCount;
}

// ----------------------------------------------------------------------------
// Liefert von den nicht-internen ConfigItems das nPos-te.

String SvConfigManager::GetItem(USHORT nPos) const
{
	OGuard aGuard( Application::GetSolarMutex() );

	USHORT nCount=0;
	SvConfigFileItem_Impl *pItem=0;
	for (USHORT n=0; n<pFileItemArr->Count(); n++)
	{
		pItem = (*pFileItemArr)[n];
		// Bei jedem nicht-intern Item Position hochz"ahlen
		if (!pItem->IsInternal())
			if (++nCount == nPos+1)
				return pItem->aName;
	}

	DBG_ERROR("Item unbekannt!");
	return String();
}

USHORT SvConfigManager::GetItemId( const String& rName ) const
{
	OGuard aGuard( Application::GetSolarMutex() );

	for (USHORT i=0; i<pFileItemArr->Count(); i++)
	{
		if((*pFileItemArr)[i]->aName == rName)
			return (*pFileItemArr)[i]->nType;
	}

	return 0;
}

// ----------------------------------------------------------------------------
// Entfernt ein SvConfigItem aus der Konfigurationsdatei.
// (Identifizierung "uber den Typ).

BOOL SvConfigManager::Remove(USHORT nType)
{
	OGuard aGuard( Application::GetSolarMutex() );

	SvConfigFileItem_Impl *pItem=0;
	const USHORT nCount = pFileItemArr->Count();
	for(USHORT i=0; i<nCount; ++i)
	{
		pItem = (*pFileItemArr)[i];			// FileItem suchen
		if(pItem->nType == nType)
		{
			if (RemoveItem(pItem))			// Eigentliches Remove
			{
				pFileItemArr->Remove(i);	// aus Directory entfernen
				delete pItem;
			}
			else
				return FALSE;
			return TRUE;
		}
	}

	// Wenn bisher nicht gefunden, an parent weiterleiten
	if (pParentMgr)
		return pParentMgr->Remove(nType);
	return FALSE;
}


// ----------------------------------------------------------------------------
// Entfernt ein SvConfigItem aus der Konfigurationsdatei.
// (Identifizierung "uber die Position der sichtbaren Items -> s. Organizer.)

BOOL SvConfigManager::RemoveItem(USHORT nPos)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Abmelden waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	// FileItem mit dieser Position suchen
	USHORT nCount=0;
	SvConfigFileItem_Impl *pFileItem=0;
	USHORT n;
	for (n=0; n<pFileItemArr->Count(); n++)
	{
		pFileItem = (*pFileItemArr)[n];
		// Bei jedem nicht-internen Item Position hochz"ahlen
		if ( !pFileItem->IsInternal() )
			if (++nCount == nPos+1)
				break;
	}

	DBG_ASSERT(n<pFileItemArr->Count(), "Item unbekannt!");
	if (n>=pFileItemArr->Count())
		return FALSE;

	// Wenn gefunden, dann aus Directory entfernen
	if (RemoveItem(pFileItem))
	{
		pFileItemArr->Remove(n);		// Eigentliches Remove
		delete pFileItem;
	}
	else
		return FALSE;
	return TRUE;
}


// ----------------------------------------------------------------------------
// Entfernt ein SvConfigItem aus der Konfigurationsdatei.
// (Eigentliche Arbeitsmethode).

BOOL SvConfigManager::RemoveItem(SvConfigFileItem_Impl* pItem)
{
	OGuard aGuard( Application::GetSolarMutex() );

	BOOL bRet = TRUE;

	// Der SubClass Gelegenheit geben, auf das "oschen zu reagieren
	RemovingItem( pItem->nType );

	if ( pItem->aItems.Count() )
	{
		if ( !pParentMgr )
		{
			// Wenn das FileItem noch an einem ConfigItem h"angt und kein
			// parent manager vorhanden ist, kann das ConfigFileItem nicht
			// entfernt, sondern nur auf default gesetzt werden
			DBG_WARNING("ConfigItem kann nicht entfernt werden!");
			if ( !pItem->IsDefault() )
				SetModified(TRUE);
			pItem->Reset();
			if ( bActive )
				ReconfigureItem( pItem );

			bRet = FALSE;
		}
		else
		{
			// Ansonsten "ubernimmt der parent das ConfigItem
			SvConfigFileItem_Impl *pIt = FindItem_Impl(pParentMgr->pFileItemArr, pItem->nType);
			if (!pIt)
			{
				pIt = new SvConfigFileItem_Impl( pItem->nType, pItem->aName );
				pParentMgr->pFileItemArr->Insert( pIt, pParentMgr->pFileItemArr->Count() );
				pParentMgr->SetModified(TRUE);
			}

			pIt->Move( pItem->aItems, pParentMgr );

			// wenn aktiv, neue Konfiguration wirken lassen
			if ( bActive || pParentMgr->bActive )
				pParentMgr->ReconfigureItem ( pIt );
		}
	}

	SetModified(TRUE);
	return bRet;
}

// ----------------------------------------------------------------------------
// Kopiert die Konfigurationsinformation.
// Falls ein ConfigItem mit dem "ubergebenen Typ noch nicht registriert ist,
// wird dem ConfigItemArray ein neuer Eintrag hinzugef"ugt.
// Es erfolgt kein Weiterleiten an den parent, da diese Methode daf"ur gedacht
// ist, den Bestand eines child managers zu erweitern.
// Identifierung erfolgt "uber die Position.

BOOL SvConfigManager::CopyItem(USHORT nPos, USHORT& rPos, SvConfigManager *pMgr)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Anmelden waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	// In der source das FileItem nPos suchen
	SvConfigFileItem_Impl *pIt=0;
	USHORT nCount=0;
	USHORT n;
	for (n=0; n<pMgr->pFileItemArr->Count(); n++)
	{
		pIt = (*pMgr->pFileItemArr)[n];
		if (!pIt->IsInternal())
			if (++nCount == nPos+1)
				break;
	}

	DBG_ASSERT(n<pMgr->pFileItemArr->Count(), "Item unbekannt!");
	if (n>=pMgr->pFileItemArr->Count())
		return FALSE;

	// Position ermitteln, die das kopierte Item einnehmen wird
	rPos = 0;
	nCount = pFileItemArr->Count();
	for(USHORT i=0; i<nCount; ++i)
	{
		SvConfigFileItem_Impl *pItem = (*pFileItemArr)[i];
		if (!pItem->IsInternal())
			n++;
		if(pItem->nType == pIt->nType)
		{
			rPos = n-1;
			break;
		}
	}

	return CopyItem( pIt, pMgr );
}

// ----------------------------------------------------------------------------
// Kopieren wie oben.
// Identifizierung "uber den Typ.

BOOL SvConfigManager::CopyItem(USHORT nType, SvConfigManager *pMgr, BOOL bSearchInParent )
{
	OGuard aGuard( Application::GetSolarMutex() );

	SvConfigFileItem_Impl *pItem =
			FindItem_Impl(pMgr->pFileItemArr, nType);

	if ( !pItem && bSearchInParent && pMgr->pParentMgr )
	{
		pMgr = pMgr->pParentMgr;
		pItem = FindItem_Impl(pMgr->pFileItemArr, nType);
	}

	if (pItem)
		return CopyItem( pItem, pMgr );
	else
		return FALSE;
}


BOOL SvConfigManager::CopyItem( SvConfigItem *pCItem )
{
	return CopyItem(pCItem->GetType(), pCItem->GetConfigManager());
}

// ----------------------------------------------------------------------------
// Kopiert alle ConfigItems des "ubergebenen ConfigManagers

BOOL SvConfigManager::CopyItems(SvConfigManager *pCfgMgr)
{
	OGuard aGuard( Application::GetSolarMutex() );

	BOOL bRet=TRUE;
	for ( USHORT n=0; n<pCfgMgr->pFileItemArr->Count(); n++ )
	{
		SvConfigFileItem_Impl *pItem = (*pCfgMgr->pFileItemArr)[n];
		CopyItem( pItem, pCfgMgr );
	}

	return bRet;
}


// ----------------------------------------------------------------------------
// Eigentliche Copy-Methode

BOOL SvConfigManager::CopyItem(SvConfigFileItem_Impl *pIt, SvConfigManager *pMgr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	if ( pMgr == this )
		return FALSE;					// nichts einf"ugen

	// Der SubClass Gelegenheit geben, das kopierte Item zu verarbeiten; dabei
	// kann sich der ItemType "andern, falls es keine feste Id, sondern eine
	// on demand vergebene ist.
	USHORT nItemType = pIt->nType;
	CopyingItem( nItemType, pIt->aName );
	if ( !nItemType )
		return FALSE;

	// Gibt es das ConfigItem schon ?
	BOOL bRet=FALSE;                    // Returnwert: Item einfuegen
	SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, nItemType);
	if ( !pItem )
	{
		bRet = TRUE;
		String aName = pIt->aName;
		pItem = new SvConfigFileItem_Impl(nItemType, aName);

		if ( bActive && pParentMgr )
		{
			SvConfigFileItem_Impl *pFItem =
				FindItem_Impl(pParentMgr->pFileItemArr, pItem->nType);
			if ( pFItem && pFItem->aItems.Count() )
			{
				// Wenn das Item bisher nicht von (this) versorgt wurde,
				// obwohl (this) gerade aktiv ist, kann es bisher nur
				// vom parent bedient worden sein.
				// Daher muss dort vor dem Umkonfigurieren noch die
				// Konfiguration gespeichert werden.

				if ( pFItem->IsModified() )
					pParentMgr->StoreConfig( pFItem );
				pItem->Move( pFItem->aItems, this, pFileItemArr->pActiveTask );
			}
		}

		SetModified(TRUE);
		pFileItemArr->Insert( pItem, pFileItemArr->Count() );
	}

	if ( pIt->aItems.Count() )
	{
		// Namen zur Sicherheit noch einmal "ubertragen, falls ge"andert
		SvConfigItem *pCItem = pIt->aItems[0]->pCItem;
		String aName( pCItem->GetName() );
		if ( aName.Len() && !pCItem->Internal() )
			pItem->aName = aName;

		// Falls die Konfiguration am "Spender" noch nicht gespeichert wurde
		if ( pIt->IsModified() )
			pMgr->StoreConfig( pIt );
	}

	if( !pIt->IsDefault() )
	{
		pStream->ReOpen();				// Konfiguration kopieren, wenn vorhanden
		pMgr->pStream->ReOpen();
		bFileIO = TRUE;
		pMgr->bFileIO = TRUE;

		if ( lDirPos == ITEM_NOT_IN_FILE )
			WriteHeader(pStream);

		pItem->lFilePos = lDirPos;		// Position in this
		pItem->lLength = pIt->lLength;
		CopyConfigItems_Impl(pItem, pIt->lFilePos, pStream, pMgr->pStream);

		lDirPos = pStream->Tell();		// Neue Position f"ur Directory
		pStream->Seek( lStart );
		(*pStream) << lDirPos;

		pStream->Close();
		pMgr->pStream->Close();
		DBG_ASSERT( bFileIO != 2, "Bingo!" );
		bFileIO = FALSE;
		DBG_ASSERT( pMgr->bFileIO != 2, "Bingo!" );
		pMgr->bFileIO = FALSE;
		SetModified(TRUE);
	}
	else
	{
		if ( !pItem->IsDefault() )
			SetModified(TRUE);
		pItem->Reset();
	}

	if ( pItem->aItems.Count() )
		ReconfigureItem( pItem );

	return bRet;
}

// ----------------------------------------------------------------------------
// F"ugt dem ConfigItemArray einen neuen Eintrag hinzu oder verbindet einen
// schon vorhandenen Eintrag mit dem "ubergebenen SvConfigItem. Wenn der ItemTyp
// noch unbekannt ist, wird er nur "ubernommen, wenn der ConfigManager keinen
// parent hat (also die Applikationskonfiguration ist).
// ACHTUNG : wird im ctor von SvConfigItem gerufen, kann also keine virtuellen
// Methoden dieser Klasse benutzen !!
//
BOOL SvConfigManager::AddConfigItem(SvConfigItem *pCItem)
{
	return AddConfigItem( pCItem, NULL );
}

BOOL SvConfigManager::AddConfigItem(SvConfigItem *pCItem, SfxBroadcaster* pTask)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Anmelden waehrend Laden/Speichern");
	DBG_ASSERT(pFileItemArr? pFileItemArr->Count() < USHRT_MAX-1: TRUE, "mehr als USHRT_MAX Items");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

#ifdef DBG_UTIL
    ByteString aMsg( "SvCfgMgr(" );
    aMsg += ByteString::CreateFromInt32( (long) this );
    aMsg += ',';
    aMsg += ByteString::CreateFromInt32((USHORT) bActive);
	aMsg += ")::Add(";
    aMsg += ByteString::CreateFromInt32( (long) pCItem);
    aMsg += ',';
    aMsg += ByteString::CreateFromInt32(pCItem->GetType());
    aMsg += ')';
    DBG_TRACE(aMsg.GetBuffer());
#endif

	SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, pCItem->GetType());

	if ( pItem )
	{
		SvConfigItemTask_Impl* p = pItem->GetTaskItem( pTask );
		if( p )
		{
			// Wenn verschiedene ConfigItems den gleichen Typ haben, mu\s das
			// kein Fehler sein (z.B. SfxPopupWindows, die SfxToolBoxManager
			// enthalten). Ein Fehler liegt nur dann vor, wenn die auch konfigurierbar
			// sind, deshalb hier nur eine Warnung.
			if ( p->pCItem != pCItem )
			{
				p->pCItem->ReleaseConfigManager();
				p->pCItem = pCItem;
			}

			DBG_WARNING ("Item gleichen Typs vorhanden!");
		}
		else
			pItem->aItems.Insert( new SvConfigItemTask_Impl( pCItem, pTask ), pItem->aItems.Count() );
		pCItem->Connect(this);
	}
	else if ( pCItem->GetConfigManager() == this )
	{
		// Item wurde schon vorher zu mir connected, also in jedem Fall nehmen
		String aString;
		pItem = new SvConfigFileItem_Impl(pCItem->GetType(), aString);
		pItem->aItems.Insert( new SvConfigItemTask_Impl( pCItem, pTask ), pItem->aItems.Count() );
		pFileItemArr->Insert(pItem, pFileItemArr->Count());
//		SetModified( TRUE );	// Erst jetzt, nicht vorher wg. Modified-Notify
	}
	else
	{
		if (pParentMgr)
		{
			// beim parent versuchen, ob der eine Konfiguration daf"ur hat
			pItem = FindItem_Impl( pParentMgr->pFileItemArr, pCItem->GetType() );
			if ( pItem )
			{
				SvConfigItemTask_Impl* p = pItem->GetTaskItem( pTask );
				if( p )
				{
					// Wenn verschiedene ConfigItems den gleichen Typ haben, mu\s das
					// kein Fehler sein (z.B. SfxPopupWindows, die SfxToolBoxManager
					// enthalten). Ein Fehler liegt nur dann vor, wenn die auch konfigurierbar
					// sind, deshalb hier nur eine Warnung.
					if ( p->pCItem != pCItem )
					{
						p->pCItem->ReleaseConfigManager();
						p->pCItem = pCItem;
					}

					DBG_WARNING ("Item gleichen Typs vorhanden!");
				}
				else
					pItem->aItems.Insert( new SvConfigItemTask_Impl( pCItem, pTask ), pItem->aItems.Count() );
				pCItem->Connect(pParentMgr);
			}
		}

		if (!pItem)                     // Typ bisher unbekannt
		{
			SvConfigFileItemArr_Impl  *pArr = pFileItemArr;
			if (pParentMgr)
			{
				pArr = pParentMgr->pFileItemArr;
				pCItem->Connect(pParentMgr);
			}
			else
			{
				pCItem->Connect(this);
			}

			pItem = new SvConfigFileItem_Impl(pCItem, pTask );
			pArr->Insert(pItem, pArr->Count());

			if ( pParentMgr )
				pParentMgr->SetModified( TRUE );	// Erst jetzt, nicht vorher wg. Modified-Notify
			else
				SetModified(TRUE);
		}
	}

	return (TRUE);
}

// ----------------------------------------------------------------------------
// F"ugt ein leeres ConfigFileItem hinzu.

void SvConfigManager::InsertConfigItem(USHORT nType)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Anmelden waehrend Laden/Speichern");
	DBG_ASSERT(pFileItemArr? pFileItemArr->Count() < USHRT_MAX-1: TRUE, "mehr als USHRT_MAX Items");
	if(bFileIO)
	{
		bFileIO=2;
		return;
	}

	SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, nType);
	if (!pItem)
	{
		SetModified(TRUE);
		String aString;
		pItem = new SvConfigFileItem_Impl(nType, aString);
		pFileItemArr->Insert(pItem, pFileItemArr->Count());
	}
	else
		DBG_WARNING("Insert: dieser Typ ist schon vorhanden!");
}

// ----------------------------------------------------------------------------
// Trennt den zum "ubergebenen SvConfigItem geh"orenden Eintrag im
// ConfigItemArray von ebenjenem ab.
// ACHTUNG : wird im dtor von SvConfigItem gerufen, kann also keine virtuellen
// Methoden dieser Klasse benutzen !!
//
BOOL SvConfigManager::RemoveConfigItem(SvConfigItem *pCItem)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Abmelden waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

#ifdef DBG_UTIL
    ByteString aMsg( "SvCfgMgr(" );
    aMsg += ByteString::CreateFromInt32( (long) this );
    aMsg += ',';
    aMsg += ByteString::CreateFromInt32((USHORT) bActive);
	aMsg += ")::Remove(";
    aMsg += ByteString::CreateFromInt32( (long) pCItem);
    aMsg += ',';
    aMsg += ByteString::CreateFromInt32(pCItem->GetType());
    aMsg += ')';
    DBG_TRACE(aMsg.GetBuffer());
#endif
										// Item suchen und ggf. entfernen
	USHORT nCount = pFileItemArr->Count();
	for(USHORT i=0; i<nCount; i++)
	{
		SvConfigFileItem_Impl *pFileItem = (*pFileItemArr)[i];
		if ( pFileItem->Remove( pCItem ) )
		{
			// Filekompatibles Abspeichern der "Internal"-Information
			if ( pCItem->Internal() )
				pFileItem->aName = String();
			pCItem->ReleaseConfigManager();
			return TRUE;
		}
	}
										// wenn nicht gefunden, muss es beim
										// parent registriert sein
	if (pParentMgr)
		return (pParentMgr->RemoveConfigItem(pCItem));
	return (FALSE);
}

// ----------------------------------------------------------------------------
// Setzt ein ConfigItem auf die DefaultKonfifuration zur"uck

void SvConfigManager::ResetConfigItem(USHORT nType)
{
	OGuard aGuard( Application::GetSolarMutex() );

	SvConfigFileItem_Impl *pItem=0;
	const USHORT nCount = pFileItemArr->Count();
	for(USHORT i=0; i<nCount; ++i)
	{
		pItem = (*pFileItemArr)[i];
		if(pItem->nType == nType)
		{
			pItem->Reset();
			// wenn aktiv, neue Konfiguration wirken lassen
			if ( bActive )
				ReconfigureItem (pItem);
			if ( !pItem->IsDefault() )
				SetModified(TRUE);
			break;
		}
	}

	// Nicht gefunden, mal beim parent versuchen
	if (!pItem && pParentMgr)
	{
		pParentMgr->ResetConfigItem(nType);
		if ( bActive )
		{
			pItem = FindItem_Impl(pParentMgr->pFileItemArr, nType);
			if (pItem)
				pParentMgr->ReconfigureItem(pItem);
		}
	}
}

BOOL SvConfigManager::ReconfigureItems( SvConfigManager *pMgr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT (pMgr, "ConfigManager ist NULL !");
	if (!pMgr)
		return FALSE;

	SvConfigFileItemArr_Impl* pSource = pMgr->pFileItemArr;
	for (USHORT n=0; n<pSource->Count(); n++)
	{
		SvConfigFileItem_Impl* pOld = (*pSource)[n];
		SvConfigItemTask_Impl* p = pOld->GetTaskItem( pFileItemArr->pActiveTask );
		if ( !p )
			p = pOld->GetTaskItem( NULL );

		if ( p )
		{
			// Eventuell wurde das ConfigItem schon vorher umgestellt oder
			// es wird nicht umkonfiguriert
			if ( p->pCItem->GetConfigManager() == this || p->pCItem->Individual() )
				continue;

			// Es gibt ein ConfigItem im alten Dokument, das soll jetzt
			// "ubernommen werden
			USHORT nType = pOld->nType;
			String aName = pOld->aName;

			SvConfigFileItem_Impl* pItem = FindItem_Impl( pFileItemArr, pOld->nType );
            SvConfigManager *pReconfigMgr = this;
			if ( !pItem && pParentMgr )
			{
				if ( pParentMgr == pMgr )
					continue;

				pItem = FindItem_Impl( pParentMgr->pFileItemArr, pOld->nType );
				if ( !pItem )
				{
					pItem = new SvConfigFileItem_Impl( nType, aName );
					pParentMgr->pFileItemArr->Insert( pItem, pParentMgr->pFileItemArr->Count() );
					pParentMgr->SetModified( TRUE );
				}

                pReconfigMgr = pParentMgr;
			}
			else if ( !pItem )
			{
				// Es gibt keines, also mu\s eins angelegt werden
				pItem = new SvConfigFileItem_Impl( nType, aName );
				pFileItemArr->Insert( pItem, pFileItemArr->Count() );
				SetModified( TRUE );
			}

			if ( p->pCItem->IsModified() )
				pMgr->StoreConfig( p->pCItem );
			pItem->aItems.Insert( p, pItem->aItems.Count() );
            pReconfigMgr->ReconfigureItem( pItem );
			pOld->aItems.Remove( pOld->aItems.GetPos( p ) );

		}
	}

	if ( pMgr->pParentMgr )
		ReconfigureItems( pMgr->pParentMgr );

	return TRUE;
}

// ----------------------------------------------------------------------------
// Eigentliche Arbeitsmethode f"ur das Umkonfigurieren

BOOL SvConfigManager::ReconfigureItem(SvConfigFileItem_Impl * pItem)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Umkonfigurieren waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	StartReconfiguring();

	for (USHORT n=0; n<pItem->aItems.Count(); n++ )
	{
		SvConfigItem *pCItem = pItem->aItems[n]->pCItem;
        if ( pCItem->GetConfigManager() == this )
            continue;

		pCItem->Connect(this);
		BOOL bDefault = (pItem->lFilePos == ITEM_NOT_IN_FILE);
		if (!bDefault || !pCItem->IsDefault())
		{
			if (!bDefault)
			{
                ByteString aDbg( "SvConfig: Reconfiguring " );
                aDbg += ByteString::CreateFromInt32( pCItem->GetType() );
				DBG_TRACE( aDbg.GetBuffer() );
				bFileIO = TRUE;
				pStream->ReOpen();
				if (pStream->GetError())
					return FALSE;
				pStream->Seek(pItem->lFilePos);
				USHORT nType;
				(*pStream) >> nType;
				DBG_ASSERT( bFileIO != 2, "Bingo!" );
				bFileIO = FALSE;
                ByteString aDbg2( "SvConfig: Reconfiguring done " );
                aDbg2 += ByteString::CreateFromInt32( pCItem->GetType() );
				DBG_TRACE( aDbg2.GetBuffer() );
			}

			pCItem->Reconfigure(*pStream, bDefault);
			pCItem->SetModified(FALSE);
			pCItem->SetDefault_Impl(bDefault);
		}
	}

	FinishedReconfiguring();

	return TRUE;
}

// ----------------------------------------------------------------------------
// Oeffnet Konfigurationsdatei und liest das Directory ein.
// ACHTUNG : darf nur aus dem ctor heraus gerufen werden, da das FileItem array
// neu aufgebaut wird!
//
BOOL SvConfigManager::ReadConfiguration( SvStream *pStr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!pFileItemArr->Count(),"Konfigurationsverzeichnis schon gelesen!");
	if (pFileItemArr->Count())
		return TRUE;

	if ( pStream->GetError() )
		return TRUE;				// Bei Fehler alle Items default

	// Inhalt kopieren
	pStr->Seek(0);
	pStream->SetVersion( pStr->GetVersion() );
	(*pStream) << (*pStr);

	// Fehler oder leerer Stream
	if ( pStr->GetError() || pStream->Tell() == 0L )
		return TRUE;				// alle Items default

	// Inhalt einlesen
	DBG_TRACE( "SvConfig: Reading" );
	bFileIO = TRUE;
	pStream->Seek(0);

	// Header pr"ufen
	const unsigned nLen = strlen( pHeader );
	char *pBuf = new char[nLen+1];
	pStream->Read( pBuf, nLen );
	pBuf[nLen] = 0;
	if( strcmp( pBuf, pHeader ) )
	{
		delete pBuf;
		nErrno = ERR_FILETYPE;
		DBG_ASSERT( bFileIO != 2, "Bingo!" );
		DBG_TRACE( "SvConfig: Reading done" );
		bFileIO = FALSE;
		pStream->Close();
		return FALSE;
	}

	delete pBuf;

	// Version vergleichen
	char c;
	USHORT nFileVersion;
	(*pStream) >> c;					// ASCII 26 (CTRL-Z) "uberlesen
	(*pStream) >> nFileVersion;
	if( nFileVersion != nVersion )
	{
		nErrno = ERR_VERSION;
		DBG_ASSERT( bFileIO != 2, "Bingo!" );
		DBG_TRACE( "SvConfig: Reading done" );
		bFileIO = FALSE;
		pStream->Close();
		return FALSE;
	}

	// Directory suchen
	lStart = pStream->Tell();
	(*pStream) >> lDirPos;

	pStream->Seek(lDirPos);
	USHORT nCount;
	(*pStream) >> nCount;

	// Directory einlesen
	String aName;
	for(USHORT i = 0; i < nCount; ++i)
	{
		USHORT nType;
		long   lLength;
		long lPos;

        (*pStream) >> nType >> lPos >> lLength;
        pStream->ReadByteString( aName );

		if (pStream->GetError())
		{
			nErrno = ERR_READ;
			return FALSE;
		}

		SvConfigFileItem_Impl *pItem =
			new SvConfigFileItem_Impl(nType, aName);
		pFileItemArr->Insert(pItem, pFileItemArr->Count());

		pItem->lLength = lLength;
		pItem->lFilePos = lPos;
	}

	pStream->Close();
	DBG_ASSERT( bFileIO != 2, "Bingo!" );
	DBG_TRACE( "SvConfig: Reading done" );
	bFileIO = FALSE;
	return TRUE;
}

// ------------------------------------------------------------------------
// Liest Konfiguration fuer alle Items, die sich bis zu diesem Zeitpunkt
// angemeldet haben.
//

BOOL SvConfigManager::LoadConfig()
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Laden waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	nErrno = ERR_NO;
	SetModified(FALSE);

	if (lDirPos == ITEM_NOT_IN_FILE)
	{
		// keine cfg-Datei oder falsche Version
		CallUseDefault();
		return TRUE;
	}

	pStream->ReOpen();
	if (pStream->GetError())
	{
		SetModified( TRUE );
		CallUseDefault();
		return TRUE;
	}

	DBG_TRACE( "SvConfig: Loading" );
	bFileIO = TRUE;

	const USHORT nItems = pFileItemArr->Count();

	StartReconfiguring();

	// Items lesen
	BOOL bRet = TRUE;
	for(USHORT i=0; i<nItems; ++i)
	{
		SvConfigFileItem_Impl *pItem = (*pFileItemArr)[i];
		if ( pItem->aItems.Count() )				// schon angemeldet ??
		{
			bRet = LoadConfig(pItem);
			String aName( pItem->aItems[0]->pCItem->GetName() );
			if ( aName.Len() && !pItem->aItems[0]->pCItem->Internal() )
				pItem->aName = aName;
		}
	}

	FinishedReconfiguring();

	pStream->Close();
	DBG_ASSERT( bFileIO != 2, "Bingo!" );
	DBG_TRACE( "SvConfig: Loading done" );
	bFileIO = FALSE;
	return bRet;
}

// ----------------------------------------------------------------------------
// Holt die Konfiguration fuer das uebergebene SvConfigItem
//
BOOL SvConfigManager::LoadConfig(SvConfigItem *pCItem)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Laden waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	nErrno = ERR_NO;
	SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, pCItem->GetType());
	DBG_ASSERT(pParentMgr || pItem, "LoadConfig fuer unbekanntes Item");

	BOOL bRet = TRUE;
	if (!pItem)
	{
		if (pParentMgr)
			return pParentMgr->LoadConfig(pCItem);
		pCItem->UseDefault();
		nErrno = ERR_READ;
		pCItem->SetDefault(TRUE);               // Setzt auch Modified
		bRet = FALSE;
	}
	else if ( lDirPos == ITEM_NOT_IN_FILE )
	{
		// cfg-Datei (noch) nicht da oder falsche Version
		pCItem->UseDefault();
		pCItem->SetDefault(TRUE);               // Setzt auch Modified
	}
/*
	else if ( pItem->pCItem != pCItem )
	{
		// Eine Konfiguration darf nur vorhanden sein, wenn der Typ
		// eindeutig einem einzigen Objekt zugeordnet ist.
		DBG_ASSERT(pItem->lFilePos == ITEM_NOT_IN_FILE,
							"LoadConfig: Typ stimmt, Item nicht!");
		pCItem->UseDefault();
		pCItem->SetDefault_Impl(TRUE);          // Setzt nicht Modified
		pCItem->ReleaseConfigManager();
		bRet = FALSE;
	}
 */
	else
	{
		pStream->ReOpen();
		if (pStream->GetError())
		{
			pCItem->UseDefault();
			pCItem->SetDefault(TRUE);               // Setzt auch Modified
		}
		else
		{
			StartReconfiguring();
            ByteString aDbg( "SvConfig: LoadConfig " );
            aDbg += ByteString::CreateFromInt32( pCItem->GetType() );
			DBG_TRACE( aDbg.GetBuffer() );
			bFileIO = TRUE;
			bRet = LoadConfig(pItem, pCItem);
			FinishedReconfiguring();
			pStream->Close();
			DBG_ASSERT( bFileIO != 2, "Bingo!" );
            ByteString aDbg2( "SvConfig: LoadConfig done " );
            aDbg2 += ByteString::CreateFromInt32( pCItem->GetType() );
			DBG_TRACE( aDbg2.GetBuffer() );
			bFileIO = FALSE;
		}
	}

/*
	String aName(pItem->pCItem->GetName());
	if ( aName.Len() && !pItem->pCItem->Internal() )
		pItem->aName = aName;
*/
	return bRet;
}

// ----------------------------------------------------------------------------
// Holt die Konfiguration fuer das uebergebene SvConfigItem
// Eigentliches Einlesen.

BOOL SvConfigManager::LoadConfig(SvConfigFileItem_Impl *pItem, SvConfigItem* p)
{
	OGuard aGuard( Application::GetSolarMutex() );

	for (USHORT n=0; n<pItem->aItems.Count(); n++ )
	{
		SvConfigItem *pCItem = pItem->aItems[n]->pCItem;
		if (p && p!= pCItem)
			continue;

		pCItem->SetModified(FALSE);

		// nach Lesefehler wird davon ausgegangen, dass das File fehlerhaft ist
		BOOL bRet = (nErrno == ERR_NO);
		if( !pItem->IsDefault() && nErrno == ERR_NO )
		{
			USHORT nType;
			pStream->Seek(pItem->lFilePos);
			(*pStream) >> nType;

			// Bei einigen ConfigItems darf die Id abweichen, da diese nur "uber den
			// Namen, aber nicht "uber die Id identifiziert werden. Dazu w"are eine
			// virtuelle Methode am ConfigManager n"otig, wg. Inkompatibilit"at solange
			// ein Hack f"ur den SfxConfigManager
			BOOL bOk = (nType == pItem->nType ||
						1294 <= nType && nType <= 1301 &&
						1294 <= pItem->nType && pItem->nType <= 1301 );
			if ( !bOk )
			{
				nErrno = ERR_READ;
				pCItem->UseDefault();
				pCItem->SetDefault(TRUE);       // Setzt auch Modified
				pItem->Reset();
				bRet = FALSE;
				continue;
			}

			switch ( pCItem->Load((*pStream)) )
			{
				case SvConfigItem::WARNING_VERSION:
				{
                    ByteString aMgrName( "svt" );
                    aMgrName += ByteString::CreateFromInt32(SOLARUPD); // aktuelle Versionsnummer
                    ResMgr *pResMgr = ResMgr::CreateResMgr( aMgrName.GetBuffer(),
										Application::GetAppInternational().GetLanguage());

					String aWarning(pCItem->GetName());
					if(aWarning.Len())
                        aWarning += ':';
					aWarning += String( ResId(STR_WARNING_ITEM, pResMgr) );

					// asynchrone Fehlermeldung absetzen
					new SvConfigWarn_Impl( aWarning );
					pCItem->UseDefault();
					pCItem->SetDefault(TRUE);		// Setzt auch Modified
					pItem->Reset();
					delete pResMgr;
					break;
				}
				case SvConfigItem::ERR_OK:
					pCItem->SetDefault_Impl(FALSE); // Setzt nicht Modified
					break;
											// bei Fehler Default-Initialisierung
				case SvConfigItem::ERR_READ:
					nErrno = ERR_READ;
					pCItem->UseDefault();
					pCItem->SetDefault(TRUE);       // Setzt auch Modified
					pItem->Reset();
					bRet = FALSE;
					break;

				default:
					pCItem->UseDefault();
					pCItem->SetDefault(TRUE);       // Setzt auch Modified
					pItem->Reset();
					bRet = FALSE;
					DBG_ERROR("Unbekannter Fehler beim Lesen eines Items");
					break;
			}
		}
		else
		{
			// fuer alle angemeldeten Items, die nicht im file stehen,
			// wird die Default-Initialisierung gerufen
			pCItem->SetDefault_Impl(TRUE);          // Setzt auch Modified
			pCItem->UseDefault();
		}
	}

	return TRUE;
}

// ----------------------------------------------------------------------------
// Schreibt eine kompakte Konfigurationsdatei, indem alle vorhandenen Eintraege
// im ConfigItemArray, die auf einen Eintrag in der aktuellen Konfiguration
// verweisen, enthalten sind.
// ACHTUNG : sollte fuer root configuration nur im dtor gerufen werden, fuer
// Dokumente wird es ueblicherweise beim Speichern aufgerufen.
//
BOOL SvConfigManager::SaveConfiguration( SvStream* pStr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	nErrno = ERR_NO;
	pStream->ReOpen();
	nErrno = pStream->GetError();
	if ( nErrno )
		return FALSE;

	WriteHeader( pStr );  				// Header anlegen
	long lNext = lDirPos;               // da geht's weiter
	bFileIO = TRUE;
	DBG_TRACE( "SvConfig: Saving" );
										// cfg-Datei zusammenschieben
	USHORT nItems = pFileItemArr->Count();
	USHORT i;
	for( i=0; i<nItems; i++ )
	{
		SvConfigFileItem_Impl *pIt = (*pFileItemArr)[i];
		SvConfigItem* pCItem=NULL;
		if ( pIt->aItems.Count() )
		{
			// Eintrag zeigt noch auf Item; Name k"onnte sich ge"andert haben
			SvConfigItemTask_Impl* p = pIt->GetTaskItem( pFileItemArr->pActiveTask );
			if ( !p )
				p = pIt->GetTaskItem( NULL );
			if ( !p )
				p = pIt->aItems[0];
			pCItem = p->pCItem;
			String aName( pCItem->GetName() );
			if ( aName.Len() && !pCItem->Internal() )
				pIt->aName = aName;
			if ( pCItem->IsDefault() )
			{
				pIt->Reset();
				pCItem->SetModified( FALSE );
			}
		}

		if ( pCItem && pCItem->IsModified() )
		{
			pIt->lFilePos = pStr->Tell();
			(*pStr) << pIt->nType;		// Eintrag wurde noch nicht gespeichert
			pCItem->Store( *pStr );
			pCItem->SetModified( FALSE );
			pIt->lLength = pStr->Tell() - pIt->lFilePos;
		}
		else
		{
			if( pIt->IsDefault() )
				continue;

			long lPos = pIt->lFilePos;	// Position in alter Datei
			pIt->lFilePos = pStr->Tell(); // Position in neuer Datei
			CopyConfigItems_Impl( pIt, lPos, pStr, pStream );
		}

		nErrno = (ULONG) (*pStr).GetError();
		if (nErrno)
		{
			pStr->Seek( lDirPos );
			nItems=0;
			break;
		}
	}

	lDirPos = pStr->Tell(); 			// Directory gleich hinten dran
	pStr->Seek( lStart );
	(*pStr) << lDirPos;
	pStr->Seek(lDirPos);
	(*pStr) << nItems;

	for(i=0; i<nItems; i++)             // Directory schreiben
	{
		SvConfigFileItem_Impl *pIt = (*pFileItemArr)[i];
		pIt->Store( (*pStr) );
	}

	if (nErrno == ERR_NO)				// Arbeits-Stream auf den Stand bringen
	{
		pStr->Seek(0);
		pStream->Seek(0);
		(*pStream) << (*pStr);			// kopieren
	}

	pStream->Close();
	DBG_ASSERT( bFileIO != 2, "Bingo!" );
	DBG_TRACE( "SvConfig: Saving done" );
	bFileIO = FALSE;
	SetModified(FALSE);
	return nErrno == ERR_NO ? TRUE : FALSE;
}

// ------------------------------------------------------------------------
// Speichert fuer alle ConfigItems, die noch nicht abgemeldet worden sind,
// die Konfiguration.
//
BOOL SvConfigManager::StoreConfig()
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Speichern waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	nErrno = ERR_NO;

	if (!pStream->IsOpen())
	{
		pStream->ReOpen();
		nErrno = pStream->GetError();
		if (nErrno)
			return FALSE;
	}

	DBG_TRACE( "SvConfig: Storing" );
	bFileIO = TRUE;

	BOOL bRet = TRUE;
	USHORT nCount = pFileItemArr? pFileItemArr->Count(): 0;
	for(USHORT i=0; i<nCount; i++)
	{
		SvConfigFileItem_Impl *pItem = (*pFileItemArr)[i];

		if ( pItem->aItems.Count() )
		{
			// Der Name k"onnte sich ge"andert haben
			SvConfigItem *pCItem = pItem->aItems[0]->pCItem;
			String aName( pCItem->GetName() );
			if ( aName.Len() && !pCItem->Internal() )
				pItem->aName = aName;
			bRet = StoreConfig(pItem);
		}

		if (bRet == FALSE)
			break;
	}

	pStream->Close();
	DBG_ASSERT( bFileIO != 2, "Bingo!" );
	DBG_TRACE( "SvConfig: Storing done" );
	bFileIO = FALSE;
	return bRet;
}

// ----------------------------------------------------------------------------
// Speichert die Konfiguration fuer das uebergebene SvConfigItem, sofern
// dieses nicht im Default-Zustand ist.
//
BOOL SvConfigManager::StoreConfig(SvConfigItem *pCItem)
{
	OGuard aGuard( Application::GetSolarMutex() );

	DBG_ASSERT(!bFileIO, "Kein Speichern waehrend Laden/Speichern");
	if(bFileIO)
	{
		bFileIO=2;
		return FALSE;
	}

	nErrno = ERR_NO;
	SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, pCItem->GetType());
	DBG_ASSERT(pParentMgr || pItem, "StoreConfig fuer unbekanntes Item");
	if (!pItem)
	{
		if (pParentMgr)
			return pParentMgr->StoreConfig(pCItem);
		nErrno = ERR_WRITE;
		return FALSE;
	}

	BOOL bRet=TRUE;

	// Der Name k"onnte sich ge"andert haben
	String aName( pCItem->GetName() );
	if ( aName.Len() && !pCItem->Internal() )
		pItem->aName = aName;

	if ( pCItem->IsModified() && ( !pCItem->IsDefault() || !pItem->IsDefault() ) )
	{
		if (!pStream->IsOpen())
		{
			pStream->ReOpen();
			nErrno = pStream->GetError();
			if (nErrno)
				return FALSE;
		}

        ByteString aDbg( "SvConfig: StoreConfig1 " );
        aDbg += ByteString::CreateFromInt32( pCItem->GetType() );
		DBG_TRACE( aDbg.GetBuffer() );
		bFileIO = TRUE;

		pCItem->SetModified(FALSE);
		SetModified(TRUE);
		long lNext;                         // n"achste Position zum Schreiben

		if (lDirPos == ITEM_NOT_IN_FILE)
			WriteHeader( pStream );
		lNext = lDirPos;

		if (pCItem->IsDefault())
		{
			pItem->Reset();
		}
		else
		{
			pStream->Seek(lNext);
			(*pStream) << pItem->nType;
			pCItem->Store((*pStream));

			nErrno = pStream->GetError();
			if (nErrno == ERR_NO)
			{
				pItem->lFilePos = lNext;
				lDirPos = pStream->Tell();
				pItem->lLength = lDirPos - pItem->lFilePos;
			}
			else
			{
				pItem->Reset();
				bRet = FALSE;
			}
		}

		pStream->Close();
		DBG_ASSERT( bFileIO != 2, "Bingo!" );
        ByteString aDbg2( "SvConfig: StoreConfig1 done " );
        aDbg2 += ByteString::CreateFromInt32( pCItem->GetType() );
		DBG_TRACE( aDbg2.GetBuffer() );
		bFileIO = FALSE;
	}

	return bRet;
}

// ----------------------------------------------------------------------------

BOOL SvConfigManager::StoreConfig(SvConfigFileItem_Impl *pItem)
{
	OGuard aGuard( Application::GetSolarMutex() );

	if ( !pItem->aItems.Count() )
		return FALSE;

	SvConfigItemTask_Impl* p = pItem->GetTaskItem( pFileItemArr->pActiveTask );
	if ( !p )
		p = pItem->GetTaskItem( NULL );
	if ( !p )
		p = pItem->aItems[0];

	SvConfigItem *pCItem = p->pCItem;
	if ( !pCItem->IsModified() )
		return TRUE;

	pCItem->SetModified(FALSE);
	if(pCItem->IsDefault() && pItem->IsDefault())
		return TRUE;                    // Zustand unver"andert default

	SetModified(TRUE);
	long lNext;                         // n"achste Position zum Schreiben

	BOOL bOpened = FALSE;
	if (!pStream->IsOpen())
	{
		bOpened = TRUE;
		pStream->ReOpen();
		nErrno = pStream->GetError();
		if (nErrno)
			return FALSE;
	}

    ByteString aDbg( "SvConfig: StoreConfig2 " );
    aDbg += ByteString::CreateFromInt32( pItem->nType );
	DBG_TRACE( aDbg.GetBuffer() );
	bFileIO = TRUE;

	if (lDirPos == ITEM_NOT_IN_FILE)
		WriteHeader( pStream );
	lNext = lDirPos;

	if (pCItem->IsDefault())
	{
		pItem->Reset();
	}
	else
	{
		pStream->Seek(lNext);
		(*pStream) << pItem->nType;
		pCItem->Store((*pStream));

		nErrno = pStream->GetError();
		if (nErrno == ERR_NO)
		{
			pItem->lFilePos = lNext;
			lDirPos = pStream->Tell();
			pItem->lLength = lDirPos - pItem->lFilePos;
		}
		else
		{
			pItem->Reset();
			if ( bOpened )
			{
				DBG_ASSERT( bFileIO != 2, "Bingo!" );
                ByteString aDbg( "SvConfig: StoreConfig2 done " );
                aDbg += ByteString::CreateFromInt32( pItem->nType );
				DBG_TRACE( aDbg.GetBuffer() );
				bFileIO = FALSE;
			}

			return FALSE;
		}
	}

	if ( bOpened )
	{
		DBG_ASSERT( bFileIO != 2, "Bingo!" );
        ByteString aDbg( "SvConfig: StoreConfig2 done " );
        aDbg += ByteString::CreateFromInt32( pItem->nType );
		DBG_TRACE( aDbg.GetBuffer() );
		bFileIO = FALSE;
	}

	return TRUE;
}

// ------------------------------------------------------------------------
// Setzt alle ConfigItems, die angemeldet sind, auf die Default-Konfiguration.
//

void SvConfigManager::CallUseDefault()
{
	OGuard aGuard( Application::GetSolarMutex() );

	StartReconfiguring();

	const USHORT nCount = pFileItemArr->Count();
	for(USHORT i = 0; i < nCount; ++i)
	{
		SvConfigFileItem_Impl *pItem = (*pFileItemArr)[i];
		if ( pItem->aItems.Count() )
		{
			for ( USHORT n=0; n<pItem->aItems.Count(); n++ )
			{
				SvConfigItem *pCItem = pItem->aItems[n]->pCItem;
				pCItem->SetModified(FALSE);
				pCItem->UseDefault();
				pCItem->SetDefault_Impl(TRUE);
				pItem->Reset();
				String aName( pCItem->GetName() );
				if ( aName.Len() && !pCItem->Internal() )
					pItem->aName = aName;
			}
		}
	}

	FinishedReconfiguring();
}


//-------------------------------------------------------------------------

BOOL SvConfigManager::IsModified()
{
	OGuard aGuard( Application::GetSolarMutex() );

	if (bModified == FALSE)
	{
		for (USHORT n=0; n<pFileItemArr->Count(); n++)
		{
			SvConfigFileItem_Impl *pItem = (*pFileItemArr)[n];
			if ( pItem->IsModified() )
			{
				bModified = TRUE;
				break;
			}
		}
	}

	return bModified;
}

// ----------------------------------------------------------------------------
//

BOOL SvConfigManager::HasConfigItem(USHORT nType)
{
	OGuard aGuard( Application::GetSolarMutex() );

	return ( FindItem_Impl(pFileItemArr, nType) != 0 );
}

#ifdef DBG_UTIL
void SvConfigManager::CheckConfiguration( SvStream *pStr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	pStream->ReOpen();
	pStream->Seek(lStart);

	long ldp;
	(*pStream) >> ldp;
	DBG_ASSERT(ldp == lDirPos,"Inkonsistenz in CfgMgr : lDirPos!");

	pStream->Seek(ldp);
	USHORT nCount;
	(*pStream) >> nCount;
	DBG_ASSERT(nCount == pFileItemArr->Count(),"Inkonsistenz in CfgMgr : nCount!");

	String aName;
	for(USHORT i = 0; i < nCount; ++i)
	{
		USHORT nType;
		long   lLength,lPos;

        (*pStream) >> nType >> lPos >> lLength;
        pStream->ReadByteString( aName );
		if (pStream->GetError())
		{
			DBG_ERROR("ReadError im Konsistenz-Check!");
			break;
		}

		SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, nType);
		DBG_ASSERT(pItem, "Inkonsistenz in CfgMgr : pItem!");
		if (!pItem)
			continue;
		DBG_ASSERT(pItem->lLength == lLength, "Inkonsistenz in CfgMgr : Length!");
		DBG_ASSERT(pItem->lFilePos == lPos, "Inkonsistenz in CfgMgr : Pos!");
	}

	pStream->Close();

	pStr->SetBufferSize(CFG_STREAM_BUFFER_SIZE);
	const unsigned nLen = strlen(pHeader);
	char *pBuf = new char[nLen+1];
	pStr->Read(pBuf, nLen);
	pBuf[nLen] = 0;
	if(strcmp(pBuf, pHeader))
	{
		DBG_ERROR("Falscher Header!");
		delete pBuf;
		return;
	}

	delete pBuf;
	char c;
	USHORT nFileVersion;
	(*pStr) >> c;
	(*pStr) >> nFileVersion;
	if(nFileVersion != nVersion)
	{
		DBG_ERROR("Falsche Versionskennung!");
		return;
	}

	(*pStr) >> ldp;
	DBG_ASSERT(ldp == lDirPos,"Inkonsistenz in CfgMgr : lDirPos!");

	pStr->Seek(ldp);
	(*pStr) >> nCount;
	DBG_ASSERT(nCount == pFileItemArr->Count(),"Inkonsistenz in CfgMgr : nCount!");

	for(i = 0; i < nCount; ++i)
	{
		USHORT nType;
		long   lLength,lPos;

        (*pStr) >> nType >> lPos >> lLength;
        pStr->ReadByteString( aName );
		if (pStr->GetError())
		{
			DBG_ERROR("ReadError im Konsistenz-Check!");
			break;
		}

		SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, nType);
		long lfp = pItem->lFilePos;
		USHORT nt = pItem->nType;
		long ll = pItem->lLength;
		DBG_ASSERT(pItem, "Inkonsistenz in CfgMgr : pItem!");
		DBG_ASSERT(pItem->lLength == lLength, "Inkonsistenz in CfgMgr : Length!");
		DBG_ASSERT(pItem->lFilePos == lPos, "Inkonsistenz in CfgMgr : Pos!");
		if (! nType)
		{
            DBG_ERROR(ByteString::CreateFromInt32(ll).GetBuffer());
            DBG_ERROR(ByteString::CreateFromInt32(lfp).GetBuffer());
            DBG_ERROR(ByteString::CreateFromInt32(nt).GetBuffer());
		}
	}
}
#endif

// ----------------------------------------------------------------------------
//

BOOL SvConfigManager::CanDelete(USHORT nPos) const
{
	OGuard aGuard( Application::GetSolarMutex() );

	USHORT nCount=0;
	SvConfigFileItem_Impl *pFileItem=0;
	USHORT n;
	for (n=0; n<pFileItemArr->Count(); n++)
	{
		pFileItem = (*pFileItemArr)[n];
		// Bei jedem nicht-internen Item Position hochz"ahlen
		if ( !pFileItem->IsInternal() )
			if (++nCount == nPos+1)
				break;
	}

	DBG_ASSERT(n<pFileItemArr->Count(), "Item unbekannt!");
	if (n>=pFileItemArr->Count())
		return FALSE;

	if (pFileItem->aItems.Count() && !pParentMgr)
	{
		// Wenn das FileItem noch an einem ConfigItem h"angt und kein
		// parent manager vorhanden ist, kann das ConfigFileItem nicht
		// entfernt, sondern nur auf default gesetzt werden.
		return FALSE;
	}
	else
		return TRUE;
}

void SvConfigManager::WriteHeader( SvStream *pStream )
{
	OGuard aGuard( Application::GetSolarMutex() );

	pStream->Write(pHeader, strlen(pHeader));
	(*pStream) << cCtrlZ << nVersion;
	lStart = pStream->Tell();	// StartPosition fuer Daten merken
	(*pStream) << lStart;		// file pointer weiterschieben
	lDirPos = pStream->Tell();
}

void SvConfigManager::SetModified(BOOL bMod)
{
	OGuard aGuard( Application::GetSolarMutex() );
	bModified = bMod;
}

void SvConfigManager::DoBackup( SvStream *pStr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	// Den tempor"aren Stream in den BackupStream kopieren
	pStream->ReOpen();
	nErrno = pStream->GetError();
	if (nErrno)
		return;

	pStr->Seek(0);
	pStream->Seek(0);
	(*pStr) << (*pStream);
	pStream->Close();
}

void SvConfigManager::DoRestore( SvStream *pStr )
{
	OGuard aGuard( Application::GetSolarMutex() );

	// Den neuen Stream in den tempor"aren Stream kopieren
	pStream->ReOpen();
	nErrno = pStream->GetError();
	if (nErrno)
		return;
	pStr->Seek(0);
	pStream->Seek(0);
	(*pStream) << (*pStr);

	// Der neue Stream kann nicht einfach r"uberkopiert werden, da er mehr
	// oder weniger oder andere ConfigItems enthalten kann als aktuell
	// vorhanden sind. Das Directory mu\s also auch aktualisiert werden.
	// Zun"achst alle Eintr"age auf default setzen.
	USHORT nItems = pFileItemArr->Count();
	USHORT i;
	for(i=0; i<nItems; i++)
		(*pFileItemArr)[i]->Reset();

	// Nun noch das Directory anpassen
	pStr->Seek(lStart);
	(*pStr) >> lDirPos;
	pStr->Seek(lDirPos);
	(*pStr) >> nItems;
	for(i = 0; i < nItems; ++i)
	{
		String aName;
		USHORT nType;
		long   lLength;
		long lPos;

        (*pStr) >> nType >> lPos >> lLength;
        pStr->ReadByteString( aName );

		if (pStr->GetError())
		{
			nErrno = ERR_READ;
			break;
		}

		SvConfigFileItem_Impl *pItem = FindItem_Impl(pFileItemArr, nType);
		if ( !pItem )
		{
			// Wenn es dieses ConfigItem mittlerweile nicht mehr gibt
			pItem = new SvConfigFileItem_Impl(nType, aName);
			pFileItemArr->Insert(pItem, pFileItemArr->Count());
		}

		pItem->lLength = lLength;
		pItem->lFilePos = lPos;
	}

	// Alle ConfigItems an die neue Konfiguration anpassen
	nItems = pFileItemArr->Count();
	for ( USHORT n=0; n<nItems; n++ )
	{
		SvConfigFileItem_Impl *pItem = (*pFileItemArr)[n];
		if ( pItem->aItems.Count() )
			ReconfigureItem(pItem);
	}

	// Der tempor"are Stream weicht jetzt vom StorageStream ab
	pStream->Close();
	SetModified(TRUE);
}

void SvConfigManager::RemovingItem( USHORT ) {}

void SvConfigManager::CopyingItem( USHORT&, const String& ) {}

void SvConfigManager::StartReconfiguring() {}

void SvConfigManager::FinishedReconfiguring() {}


