/*************************************************************************
 *
 *  $RCSfile: confitem.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:02 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
// INCLUDE ---------------------------------------------------------------
#include "confitem.hxx"

#pragma hdrstop

#include "config.hxx"

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
DBG_NAME(SvConfigItem);

// ------------------------------------------------------------------------
SvConfigItem::SvConfigItem(
				USHORT nTypeId, 	// eindeutiger Identifier f"ur das Objekt
				SvConfigManager* pMgr )

/*  [Beschreibung]

	Konstruktor der Klasse SvConfigItem.

	Die Instanzen der abgeleiteten Klassen m"ussen in
	<SfxApplication::Init()> angelegt werden und d"urfen fr"uhestens
	<in SfxApplication::Exit()> wieder erst"ort werden.


	[Anmerkungen]

	Der Typ-Id Bereich unterhalb SFX_ITEMTYPE_USER_BEGIN ist f"ur das
	StarFramework reserviert. Ab diser Id tr"agt die Applikation die
	Verantwortung f"ur die Eindeutigkeit der Ids.

	F"ur Bibliotheks-Entwickler, die konfigurierbare Objekte allgemein
	bereitstellen wollen, k"onnen bei StarDivision Typ-Ids angefordert werden.
*/

:   nType(nTypeId),
	bDefault(TRUE),
	bModified(FALSE),
	bInternal(FALSE),
	bIndividual(FALSE),
    pCfgMgr( NULL )                     // initialize with NULL, AddConfigItem will find the right one
{
	DBG_CTOR(SvConfigItem, 0);
    DBG_ASSERT(pMgr, "ConfigManager nicht erzeugt");
    if (pMgr)
        pMgr->AddConfigItem(this);
}

SvConfigItem::SvConfigItem(USHORT nTypeId, SvConfigManager* pMgr, SfxBroadcaster* pTask)
:   nType(nTypeId),
	bDefault(TRUE),
	bModified(FALSE),
	bInternal(FALSE),
	bIndividual(FALSE),
    pCfgMgr( NULL )                     // initialize with NULL, AddConfigItem will find the right one
{
	DBG_CTOR(SvConfigItem, 0);
    DBG_ASSERT(pMgr, "ConfigManager nicht erzeugt");

    if (pMgr)
        pMgr->AddConfigItem(this, pTask);
}

// ------------------------------------------------------------------------
SvConfigItem::~SvConfigItem()

/*	[Beschreibung]

	 Destruktor der Klasse SvConfigItem.

	 Die Instanzen der abgeleiteten Klassen d"urfen erst in
	 SfxApplication::Exit() zerst"ort werden, wenn ihre Einstellungen
	 in die Konfigurationsdatei "ubernommen werden sollen.
*/

{
	 DBG_DTOR(SvConfigItem, 0);
	 if (pCfgMgr && !pCfgMgr->RemoveConfigItem(this))
	 {
//	 	  DBG_ERROR ("Remove auf unbekanntes ConfigItem!");
	 }
}


// ------------------------------------------------------------------------
void SvConfigItem::UseDefault()

/*  [Beschreibung]

	Diese Methode wird vom StarFramework gerufen, wenn die
	Konfigurationsdatei gelesen wurde und f"ur dieses Objekt kein
	Eintrag in der Konfigurationsdatei vorhanden war.

	Wenn der Konstruktor bereits eine Default-Konfiguration eintr"agt,
	braucht diese Methode nicht "uberladen zu werden.

	Die Basisklasse mu\s in jedem Fall gerufen werden!


	[Anmerkung]

	In den Konstruktoren der von SvConfigItem abgeleiteten
	Klassen sollte m"oglichst nichts gemacht werden, was beim Laden
	einer Konfiguration wieder "uberschrieben wird. Das w"urde die
	Startup-Zeit der Applikation nur unn"otig verl"angern.

	[Querverweise]
	<SvConfigItem::Load()>
*/

{
	DBG_CHKTHIS(SvConfigItem, 0);
	bDefault = TRUE;
}


// ------------------------------------------------------------------------
String SvConfigItem::GetName() const

/*  [Beschreibung]

	Diese Methode wird vom StarFramework gerufen, um einen textuellen,
	(sprachlich) lokalisierten Namen f"ur dieses konfigurierbare
	Objekt zu erfragen.


	[R"uckgabewert]
	String      Ein (sprachlich) lokalisierter, meschenlesbarer Name f"ur
				dieses Objekt.


	[Anmerkungen]
	I.d.R. wird dieser Name aus der Resource gelesen.


	[Beispiel]

	String ScreenConfigItem::GetName() const
	{
			return String( SfxResId( STR_SCREEN_CONFIG ) );
	}

	...

	String STR_SCREEN_CONFIG
	{
			Text = "Bildschirm-Konfiguration";
			Text [English] = "Screen Settings";
	};
*/

{
	DBG_CHKTHIS(SvConfigItem, 0);
	return String();
}


// ------------------------------------------------------------------------

// int SvConfigItem::Load(
//        SvStream &rConfigStream );// bina"rer Stream, asu dem geladen wird

/*  [Beschreibung]

	Diese Methode wird vom StarFramework gerufen, wenn w"ahrend des
	Lesens der Konfigurationsdatei eine Konfiguration f"ur
	das betreffende Objekt gefunden wurde.

	Die abgeleitete Klasse implementiert hier das Einlesen
	der Konfiguration in der Form, in der sie sie auch selbst
	rausschreibt.

	Diese Methode mu"s von abgeleiteten Klassen "uberladen und
	implementiert werden.


	[R"uckgabewert]

	int                 SvConfigItem::ERR_OK
						Die Konfiguration wurde korrekt gelesen.

						SvConfigItem::WARNING_VERSION
						Die Konfiguration lag in einer nicht supporteten
						Version vor und wurde nicht gelesen.

						SvConfigItem::ERR_READ
						Die Konfiguration konnte aufgrund eines
						nicht n"aher erkannten Fehlers nicht gelesen
						werden.

	[Anmerkungen]

	Applikationsentwickler sollten als eine Versionsnummer schreiben
	und lesen, um auch alte Formate lesen zu k"onnen oder wenigstens
	zu erkennen, da"s es sich um ein Format handelt, was nicht gelesen
	werden kann.

	[Querverweise]
	<SvConfigItem::Store()>
*/

// ----------------------------------------------------------------------

// BOOL SvConfigItemStore(
//        SvStream &rConfigStream ); // bin"arer Stream, in den gespeichert wird

/*  [Beschreibung]

	Diese Methode wird vom StarFramework an jeder Instanz einer von
	SvConfigItem abgeleiteten Klasse gerufen, wenn die
	Konfigurationen geschrieben werden sollen.

	Diese Methode mu"s von abgeleiteten Klassen "uberladen und
	implementiert werden.


	[R"uckgabewert]

	BOOL                TRUE
						Die Konfiguration konnte gespeichert
						werden. Der Eintrag wird daher (vom
						StarFramework) in ein Inhaltsverzeichnis
						der Konfigurationsdatei eingetragen.

						FALSE
						Beim Schreiben der Konfiguration ist ein
						Fehler aufgetreten, der Eintrag wird
						daher (vom StarFramework) nicht in das
						Inhaltsverzeichnis der Konfigurations-
						datei aufgenommen. Beim n"achsten Laden
						der Konfiguration wird daher wieder
						die Default-Konfiguratin f"ur dieses
						Objekt verwendet.


	[Querverweise]
	<SvConfigItem::Load()>
*/

// ----------------------------------------------------------------------
// BOOL SvConfigItem::IsDefault() const;

/*  [Beschreibung]

	Mit dieser Methode kann ermittelt werden, ob das Objekt
	seiner Default-Konfiguration entspricht.

	Nur ver"anderte Objekte werden in der Konfigurationsdatei
	abgespeichert.


	[R"uckgabewert]
	BOOL                TRUE
						Das Objekt wurde gegen"uber seier
						Default-Konfiguration nicht ver"andert.
						Es wird daher beim n"achten Schreiben
						der Konfigurationsdatei nicht gepeichert
						und daher in Zukunft wieder aus der
						Default-Konfiguration erzeugt.

						FALSE
						Das Objekt wurde gegen"uber seiner
						Default-Konfiguration ver"andert.
						Es wird daher beim n"achsten Schreiben
						der Konfigurationsdatei mitgespeichert.


	[Querverweise]
	<SvConfigItem::SetDefault(BOOL)>
*/

// ------------------------------------------------------------------------
void SvConfigItem::SetDefault(
				BOOL bOn )     // TRUE == zur"uck auf Default, FALSE == ver"andert

/*  [Beschreibung]

	Mit dieser Methode wird eingestellt, ob das Objekt beim
	n"achsten Speichern der Konfigurationsdatei dort aufgenommen
	wird oder nicht. Grundprinzip dabei ist, da"s Objekte solange
	wieder aus der Default-Konfiguration erzeugt werden, wie
	sie nicht (im Bezug auf die in der Konfigurationsdatei zu
	speichernden Eingenschaften) ge"andert wurden.


	[Anmerkung]
	Diese Methode l"adt nicht erneut die Default-Konfiguration, sondern
	verwaltet nur das betreffende Flag.


	[Querverweise]
	<SvConfigItem::IsDefault()>
	<SvConfigItem::UseDefault()>
*/
{
	if (!bOn || !bDefault)
		SetModified(TRUE);
	bDefault = bOn;
}


//-------------------------------------------------------------------------
void    SvConfigItem::SetModified (BOOL bMod)
{
	bModified = bMod;
	if ( bMod && pCfgMgr )
		pCfgMgr->SetModified(bMod);
}


//-------------------------------------------------------------------------
BOOL	SvConfigItem::Reconfigure(SvStream& rStream, BOOL bDefault)
{
	BOOL  bRet=TRUE;
	if (bDefault)
	{
		UseDefault();
		bDefault = TRUE;
	}
	else
	{
		bRet = (Load(rStream) == ERR_OK);
		if ( !bRet )
		{
			UseDefault();
			bDefault = TRUE;
		}
		bDefault = FALSE;
	}
	return bRet;
}


// ----------------------------------------------------------------------
BOOL    SvConfigItem::Initialize()

/*  [Beschreibung]

	Mit dieser Methode wird das Objekt konfiguriert, d.h. entweder die
	Default-Koniguration eingestellt oder eine gespeicherte Konfiguration
	geladen.

	[Anmerkung]
	Jedes von SvConfigItem abgeleitete Objekt mu nach(!) seiner Erzeugung
	diese Methode aufrufen, da der Konstruktor keine Initialisierung
	vornimmt.

	[Querverweise]
	<SvConfigItem::StoreConfig()>
	<SvConfigItem::Load()>
*/

{
	BOOL bRet = TRUE;
	if ( !pCfgMgr )
		UseDefault();
	else
		bRet = pCfgMgr->LoadConfig(this);
	bModified = FALSE;
	return bRet;
}


// ----------------------------------------------------------------------
BOOL    SvConfigItem::StoreConfig(BOOL bOn)

/*  [Beschreibung]

	Mit dieser Methode wird im (Falle bOn == TRUE) die Objekt-Konfiguration
	gesichert, sofern es sich nicht in der Default-Koniguration befindet,
	oder (im Falle bOn == FALSE) eine von der Default-Konfiguration
	abweichende Einstellung aus der Konfiguration entfernt.

	[Anmerkung]
	Wenn eine SvConfigItem abgeleitetes Objekt nicht vor seiner
	Zerst"orung diese Methode ruft, bleibt der zuletzt gesicherte Zustand
	erhalten.

	[Querverweise]
	<SvConfigItem::Initialize()>
	<SvConfigItem::Store()>
*/

{
	BOOL bRet = TRUE;
	if (bOn)
	{
		if (pCfgMgr)
			bRet = pCfgMgr->StoreConfig(this);
	}

	bModified = FALSE;
	return bRet;
}



// ----------------------------------------------------------------------
// USHORT SvConfigItem::GetType() const;

/*  [Beschreibung]

	Liefert die Typ-Id des Objekts.

	Die Typ-Id identifiziert Objekte, die Einstellungen in der
	Konfigurationsdatei ablegen eindeutig in der Applikation.


	[R"uckgabewert]

	USHORT              Die im Konstruktor angegebene Typ-Id.
*/


// ----------------------------------------------------------------------
BOOL	SvConfigItem::ReInitialize( SvConfigManager *pMgr )

/*  [Beschreibung]

	Mit dieser Methode wird das Objekt konfiguriert, d.h. entweder die
	Default-Koniguration eingestellt oder eine gespeicherte Konfiguration
	geladen.

	[Anmerkung]

	[Querverweise]
	<SvConfigItem::StoreConfig()>
	<SvConfigItem::Load()>
*/

{
	if (pCfgMgr != pMgr)
	{
		if (pCfgMgr)
		{
			if (bModified)
				pCfgMgr->StoreConfig(this);
			pCfgMgr->RemoveConfigItem(this);
		}

		pCfgMgr = pMgr;
		if ( pCfgMgr )
		{
			if ( !pCfgMgr->HasConfigItem( nType ) )
				pCfgMgr->InsertConfigItem( nType );
			pCfgMgr->AddConfigItem( this );
		}

		bModified = FALSE;
		return Initialize();
	}
	else
		return TRUE;
}


