/*************************************************************************
 *
 *  $RCSfile: demo.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:02 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SVTDEMO_HXX
#define _SVTDEMO_HXX

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif
#ifndef _DATE_HXX //autogen
#include <tools/date.hxx>
#endif
#ifndef _LINK_HXX //autogen
#include <tools/link.hxx>
#endif


typedef USHORT SvDemoKind;

#define	DEMO_LIB_NAME	   		"so5dbt.dll"
#define	PRELOAD_KEY		   		"preload"

#define SFX_DEMOKIND_FULL      1
#define SFX_DEMOKIND_COMPACT   2
#define SFX_DEMOKIND_TRYBUY    3
#define SFX_DEMOKIND_DEMO      4
#define SFX_DEMOKIND_INVALID   5
#define SFX_DEMOKIND_OLDTRYBUY 6
#define SFX_DEMOKIND_RUNTIME   7

#ifdef DEMO_IMPL

#include "svarray.hxx"

struct DemoAppData_Impl
{
	Date    aInvalidateDay;
	ByteString aSystemName;
	UINT32  nLoadSum;
	UINT32  nCalcSum;
	USHORT  nDaysToRun;
	USHORT  nDemoKind;
	BOOL    bSaveSum;
};

typedef DemoAppData_Impl *DemoAppDataPtr_Impl;
SV_DECL_PTRARR_DEL(DemoAppDataArr_Impl, DemoAppDataPtr_Impl,5,5);

class DemoData_Impl
{
	ByteString aPath;
	ByteString aGroup;

	DemoAppDataArr_Impl aDemoAppDataArr;
public:
	DemoData_Impl( BOOL bAuth, const ByteString &rPath, const ByteString& rGroup,
				   const UniString* pApp = 0 );
	BOOL Load(BOOL bCheckReg);
	BOOL Save(BOOL bCreateSum);
	UINT32 CalcSum(const ByteString &rStr) const;
	DemoAppData_Impl *GetDemoData(const ByteString& aSystemName);
};

enum DemoResponse
{
	DEMO_NONE,
	DEMO_INVALID,
	DEMO_OLDTRYBUY,
	DEMO_EXPIREDTRYBUY,
	DEMO_PUFF_INVALID,
	DEMO_PUFF_LIMITED,
	DEMO_PUFF_EXPIRED,
	DEMO_PUFF_LASTLIMITED
};

struct DemoArg_Impl
{
	DemoResponse eResponse;
	String aSystem;
};

class Config;
class Timer;

struct SvDemo_Impl
{
	BOOL			bForceFullVersion;
	BOOL			bPuffVersion;
	String			aGroup;
	String			aConfigDir;
	String			aDemoDataDir;

	static Timer*			pRestartTimer;
	static DemoArg_Impl*	pSaveArg;
	DECL_STATIC_LINK( SvDemo_Impl, RestartHdl, void* );
	DECL_STATIC_LINK( SvDemo_Impl, ShowMessage, DemoArg_Impl* );

	DemoData_Impl*	pDemoData;
	Config*			pConfig;

public:
			SvDemo_Impl() :
					pDemoData( 0 ), bForceFullVersion( FALSE ), pConfig( 0 ), bPuffVersion( FALSE ) {}
			~SvDemo_Impl();

	Config&				GetRegistryConfig();
	DemoData_Impl&		GetDemoData();
	void				SetGroup( const String& rStr ) { aGroup = rStr; }
	const String&		GetGroup() const { return aGroup; }
	void				SetFullVersion() { bForceFullVersion = TRUE; }
	void				SetPuffVersion() { bPuffVersion = TRUE; }
	BOOL				IsFullVersion() { return bForceFullVersion; }
	BOOL				IsPuffVersion() { return bPuffVersion; }
	void				SetConfigDir( const String& rDir ) { aConfigDir = rDir; }
	void				SetDemoDataDir( const String& rDir ) { aDemoDataDir = rDir; }
};

#endif

class DemoData_Impl;
class OutputDevice;
class Rectangle;
struct SvDemo_Impl;

class Config;
class SvDemo
{
	friend long CheckButtonImpl( void* pObj, void* );

private:
	static SvDemo_Impl&  GetDemo();

	static void			InitDynamicKeyEdition(const XubString& rKey);

	static BOOL         CheckPLK();
	static String       CreatePLK();

public:
	static void         Init( const char* pDemoIndicator, const char* pGroup );
	static BOOL         IsInited();

	static void         ShowDemoMessagesAndQuitAppOnExpiredVersion( const String& rSystem );
	static SvDemoKind   GetDemoKind( const String& rSystem );
	static void         SpoilDemoOutput( const String& rSystem, OutputDevice &, const Rectangle &);
	static Date			GetExpireDate(const XubString& rStr);

	static void         SetConfigDir( const String& rDir );
	static void         SetDemoDataDir( const String& rDir );

	static void         SetUnauthorizedState( const String& rSystem, const String& rPLK );
	static void         SetAuthorizedState( const String& rSystem, const String& rPALK );
	static void         SetInvalidState( const String& rSystem );

	static BOOL         IsPuffVersion();

	static String       GetPLK();
	static String       GetPALK();
	static String       GetOldPALK();

//	static KeyError 	DetectPALKError( const String& rPALK, const XubString& rCustomerNr );
	static BOOL         CheckPALK( const String& rPALK, const String& rCustomerNr );
	static Config* 		GetNewAuthConfig(XubString* pPath = NULL);

	enum ErrorType { InvalidKey, InvalidUserdata, InvalidAddressdata, InvalidLicense,
					 InvalidLanguage, InvalidOS, InvalidCustomerNr, InvalidCustomerNrLen };
	static XubString		RequestErrorString( ErrorType eType );
};


#endif

