/*************************************************************************
 *
 *  $RCSfile: wordsel.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:04 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <string.h>
#include <limits.h>

#include "wordsel.hxx"

#ifndef _STRING_HXX //autogen
#include <tools/string.hxx>
#endif

#pragma hdrstop

static const sal_Char* __FAR_DATA pWordDelim = pDefWordDelim;

void WordSelection::SetWordDelimiter(const sal_Char* pDelim)
{
	pWordDelim = pDelim;
}

void WordSelection::ResetWordDelimiter()
{
	pWordDelim = pDefWordDelim;
}

int WordSelection::ChkChar( const sal_Unicode c )
{
	int nC = IsBlank;
	if( ' ' != c )
		nC = 256 > c && 0 != strchr( pWordDelim, c ) ? IsTrenner : IsZeichen;
	return nC;
}


int WordSelection::IsStartWord( const String& rStr, xub_StrLen nPtPos )
{
	const sal_Unicode* pStr = rStr.GetBuffer();
	return IsZeichen == ChkChar( *(pStr + nPtPos )) && ( !nPtPos ||
		IsZeichen != ChkChar( *(pStr + nPtPos - 1 )));
}

int WordSelection::IsEndWord( const String& rStr, xub_StrLen nPtPos )
{
	if( !nPtPos-- )
		return FALSE;
	const sal_Unicode* pStr = rStr.GetBuffer();
	return IsZeichen == ChkChar( *(pStr + nPtPos )) &&
			( nPtPos == rStr.Len() ||
			IsZeichen != ChkChar( *(pStr + nPtPos + 1 )));
}

int WordSelection::IsInWord( const String& rStr, xub_StrLen nPtPos )
{
	return IsZeichen == ChkChar( *(rStr.GetBuffer() + nPtPos ));
}

xub_StrLen WordSelection::GoStartWord( const String& rStr, xub_StrLen nPtPos )
{
	if( !nPtPos )
		return STRING_NOTFOUND;

	int SttChar = ChkChar( rStr.GetChar( --nPtPos ) ), nFnd = SttChar;
	if( SttChar == IsZeichen )
	{
		while( nPtPos && IsZeichen ==
							( nFnd = ChkChar( rStr.GetChar( --nPtPos ) )))
			;
		if( !nPtPos && nFnd == IsZeichen )
			--nPtPos;
	}
	else
	{
		while( nPtPos && IsZeichen !=
							( nFnd = ChkChar( rStr.GetChar( --nPtPos ) )))
			;
		if( !nPtPos && nFnd != IsZeichen )
			--nPtPos;
	}

	return nPtPos+1;
}

xub_StrLen WordSelection::GoEndWord( const String& rStr, xub_StrLen nPtPos )
{
	if( nPtPos == rStr.Len() )
		return nPtPos;

	do // middle check loop
	{
		int SttChar = ChkChar( rStr.GetChar( nPtPos ) );
		if( IsBlank == SttChar )
		{
			while( nPtPos < rStr.Len() && ' ' == rStr.GetChar( ++nPtPos ) )
				;
			if( nPtPos >= rStr.Len() )
				break;	// return FALSE;		// nur Blanks
			SttChar = ChkChar( rStr.GetChar( nPtPos ) );
		}

		while( SttChar == ChkChar( rStr.GetChar( ++nPtPos ) ))
			if( nPtPos == rStr.Len()  )
				break; 		// return FALSE; ??
			// ok, das naechste Wort gefunden
	}
	while( FALSE );

	return nPtPos;
}


xub_StrLen WordSelection::GoNextWord( const String& rStr, xub_StrLen nPtPos )
{
	if( nPtPos+1 >= rStr.Len() )
		return STRING_NOTFOUND;

	int nFnd, SttChar = ChkChar( rStr.GetChar( nPtPos ) );
	while( SttChar == ( nFnd = ChkChar( rStr.GetChar( ++nPtPos ) )))
		if( nPtPos == rStr.Len() )
			return STRING_NOTFOUND;
		// ok, das naechste Wort gefunden
	if( IsBlank == nFnd )
		while( nPtPos < rStr.Len() && ' ' == rStr.GetChar( ++nPtPos ) )
			;

	return nPtPos;
}


xub_StrLen WordSelection::GoPrevWord( const String& rStr, xub_StrLen nPtPos )
{
	if( 2 > nPtPos-- )
		return STRING_NOTFOUND;

	int SttChar = ChkChar( rStr.GetChar( nPtPos ) );
	if( SttChar == IsBlank )
	{
		while( nPtPos && ' ' == rStr.GetChar( --nPtPos ) )
			;
		SttChar = ChkChar( rStr.GetChar( nPtPos ) );
	}
	if( nPtPos )
	{
		while( SttChar == ChkChar( rStr.GetChar( --nPtPos ) ))
			if( !nPtPos )
				return STRING_NOTFOUND;
	}
	else
		--nPtPos;

	return nPtPos+1;
}

/*************************************************************************

	  Source Code Control System - Header

	  $Header: /cvs/oo/util/svtools/source/search/wordsel.cxx,v 1.1.1.1 2000/09/18 16:59:04 hr Exp $

	  Source Code Control System - Update

	  $Log: wordsel.cxx,v $
	  Revision 1.1.1.1  2000/09/18 16:59:04  hr
	  initial import
	
	  Revision 1.8  2000/09/18 14:13:44  willem.vandorp
	  OpenOffice header added.
	
	  Revision 1.7  2000/03/31 10:47:56  jp
	  Changes for unicode
	
	  Revision 1.6  1998/01/19 17:14:30  MH
	  chg: header


	  Rev 1.5   19 Jan 1998 18:14:30   MH
   chg: header

	  Rev 1.4   07 Aug 1997 16:45:42   TJ
   include

	  Rev 1.3   25 Mar 1996 14:52:34   OM
   Neu: Umschaltbare Worttrenner

	  Rev 1.2   09 May 1995 20:40:02   ER
   srch.lib wird DLL (SV248) => __FAR_DATA pDefWordDelim

	  Rev 1.1   08 Feb 1995 11:56:16   JP
   GoStart: nFnd richtig initialisiert

	  Rev 1.0   01 Dec 1994 16:35:28   MT
   Initial revision.

*************************************************************************/
