%{

//--------------------------------------------------------------------------
//
// $Header: /cvs/Archiv/svtools/source/svsql/sdblex.l,v 1.13 2000/07/25 10:07:58 oj Exp $
//
// (c) Copyright 1994,1995 by Star Division GmbH, Hamburg.
//
// Ersterstellung:
//	GH
//
// Letzte Aenderung:
//	$Author: oj $ $Date: 2000/07/25 10:07:58 $ $Revision: 1.13 $
//
//
// Beschreibung:
//	Lexikalische Analyse des SQL-Parsers.
//	[Muss mit MKS Lex uebersetzt werden!]
//
//	Achtung: Token (bis auf die Sonderzeichen) muessen immer mit
//	dem Praefix "SDB_TOKEN_" beginnen und der Name des Tokens muss
//	exakt mit dem SQL-Schluesselwort uebereinstimmen, denn der Token-
//	name wird (nach Entfernung des Praefix) wieder fuer die Erzeugung
//	von SQL-Text benutzt!

// Allerlei Defines ...
#define YY_EXIT 1				// YY_FATAL will not halt the application

#if STLPORT_VERSION>=321
#include <cstdarg>      // std::va_list
#endif

#include <string.h>
#include <tools/solar.h>
#include "sdbnode.hxx"

#include "svtools/sdbbison.hxx"

#ifndef _SVT_SDBSCAN_HXX
#include "sdbscan.hxx"
#endif
//=============================================================================
//
// Erzeugung der Blaetter fuer die Token
// Blaetter werden generell vom Lexer erzeugt

static String aEmptyString;

static int	gatherString(int delim, int nTyp);
static int  gatherName(const char*);
static int	gatherNamePre(const char* );
// has to be set before the parser starts
SdbSqlScanner* xxx_pGLOBAL_SQLSCAN = NULL;

#define NEW_NODE(text, token)	\
		SQLyylval.pParseNode = new SqlInternalNode(text, token);	    

#define SDB_SQL_NEW_KEYWORD(token)						\
		SQLyylval.pParseNode = new SqlInternalNode(aEmptyString, SDB_SQL_KEYWORD, (token));		

#define SDB_SQL_NEW_NAME 		NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_NAME)
#define SDB_SQL_NEW_INTNUM 		NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_INTNUM)
#define SDB_SQL_NEW_APPROXNUM 	NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_APPROXNUM)
#define SDB_SQL_NEW_STRING 		NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_STRING)
#define SDB_SQL_NEW_COMPARISON 	NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_COMPARISON)
#define SDB_SQL_NEW_AMMSC 		NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_AMMSC)
#define SDB_SQL_NEW_DATE 		NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_ACCESS_DATE)

#define YY_INPUT(buf,result,max_size) 				\
{													\
	buf[0] = xxx_pGLOBAL_SQLSCAN->Sdbyygetc();		\
	result = buf[0] != -1;							\
}

#define YY_FATAL_ERROR(msg)							\
{													\
	xxx_pGLOBAL_SQLSCAN->Sdbyyerror(msg);			\
}													

//
//=============================================================================

%}
%e 2500
%n 2900 /* DFA entries */
%p 8500

%s SQL
%s PREDICATE_ENG
%s PREDICATE_GER
%s DATE
%s STRING

%option noyywrap
%option never-interactive
%%

[Aa][Ll][Ll]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ALL); return SDB_TOKEN_ALL; }
[Aa][Ll][Tt][Ee][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ALTER); return SDB_TOKEN_ALTER; }
[Aa][Nn][Dd]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_AND); return SDB_TOKEN_AND; }
[Aa][Nn][Yy]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ANY); return SDB_TOKEN_ANY; }
[Aa][Ss]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_AS); return SDB_TOKEN_AS; }
[Aa][Ss][Cc]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ASC); return SDB_TOKEN_ASC; }
[Aa][Tt]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_AT); return SDB_TOKEN_AT; }
[Aa][Uu][Tt][Hh][Oo][Rr][Ii][Zz][Aa][Tt][Ii][Oo][Nn]   		{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_AUTHORIZATION); return SDB_TOKEN_AUTHORIZATION; }
[Aa][Vv][Gg]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_AVG); return SDB_TOKEN_AVG; }

[Bb][Ee][Tt][Ww][Ee][Ee][Nn]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_BETWEEN); return SDB_TOKEN_BETWEEN; }
[Bb][Ii][Tt]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_BIT); return SDB_TOKEN_BIT; }
[Bb][Ii][Tt]_[Ll][Ee][Nn][Gg][Tt][Hh]           {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_BIT_LENGTH); return SDB_TOKEN_BIT_LENGTH; }
[Bb][Oo][Tt][Hh]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_BOTH); return SDB_TOKEN_BOTH; }
[Bb][Yy]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_BY); return SDB_TOKEN_BY; }

[Cc][Aa][Ll][Ll]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CALL); return SDB_TOKEN_CALL; }
[Cc][Aa][Ss][Tt]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CAST); return SDB_TOKEN_CAST; }
[Cc][Hh][Aa][Rr]([Aa][Cc][Tt][Ee][Rr])?			{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CHARACTER); return SDB_TOKEN_CHARACTER; }
[Cc][Hh][Aa][Rr]([Aa][Cc][Tt][Ee][Rr])?_[Ll][Ee][Nn][Gg][Tt][Hh]  		{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CHAR_LENGTH); return SDB_TOKEN_CHAR_LENGTH; }
[Cc][Hh][Ee][Cc][Kk]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CHECK); return SDB_TOKEN_CHECK; }
[Cc][Oo][Ll][Ll][Aa][Tt][Ee]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_COLLATE); return SDB_TOKEN_COLLATE; }
[Cc][Oo][Mm][Mm][Ii][Tt]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_COMMIT); return SDB_TOKEN_COMMIT; }
[Cc][Oo][Nn][Tt][Ii][Nn][Uu][Ee]				{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CONTINUE); return SDB_TOKEN_CONTINUE; }
[Cc][Oo][Nn][Vv][Ee][Rr][Tt]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CONVERT); return SDB_TOKEN_CONVERT; }
[Cc][Oo][Uu][Nn][Tt]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_COUNT); return SDB_TOKEN_COUNT; }
[Cc][Rr][Ee][Aa][Tt][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CREATE); return SDB_TOKEN_CREATE; }
[Cc][Rr][Oo][Ss][Ss]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CROSS); return SDB_TOKEN_CROSS; }
[Cc][Uu][Rr][Rr][Ee][Nn][Tt]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CURRENT); return SDB_TOKEN_CURRENT; }
[Cc][Uu][Rr][Rr][Ee][Nn][Tt]_[Dd][Aa][Tt][Ee]	{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CURRENT_DATE); return SDB_TOKEN_CURRENT_DATE; }
[Cc][Uu][Rr][Rr][Ee][Nn][Tt]_[Tt][Ii][Mm][Ee]	{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CURRENT_TIME); return SDB_TOKEN_CURRENT_TIME; }
[Cc][Uu][Rr][Rr][Ee][Nn][Tt]_[Tt][Ii][Mm][Ee][Ss][Tt][Aa][Mm][Pp]	{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CURRENT_TIMESTAMP); return SDB_TOKEN_CURRENT_TIMESTAMP; }
[Cc][Uu][Rr][Ss][Oo][Rr]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_CURSOR); return SDB_TOKEN_CURSOR; }
[Dd]											{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_D); return SDB_TOKEN_D; }
[Dd][Aa][Tt][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DATE); return SDB_TOKEN_DATE; }
[Dd][Aa][Yy]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DAY); return SDB_TOKEN_DAY; }
[Dd][Ee][Cc]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DEC); return SDB_TOKEN_DEC; }
[Dd][Ee][Cc][Ii][Mm][Aa][Ll]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DECIMAL); return SDB_TOKEN_DECIMAL; }
[Dd][Ee][Cc][Ll][Aa][Rr][Ee]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DECLARE); return SDB_TOKEN_DECLARE; }
[Dd][Ee][Ff][Aa][Uu][Ll][Tt]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DEFAULT); return SDB_TOKEN_DEFAULT; }
[Dd][Ee][Ll][Ee][Tt][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DELETE); return SDB_TOKEN_DELETE; }
[Dd][Ee][Ss][Cc]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DESC); return SDB_TOKEN_DESC; }
[Dd][Ii][Ss][Tt][Ii][Nn][Cc][Tt]				{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DISTINCT); return SDB_TOKEN_DISTINCT; }
[Dd][Oo][Uu][Bb][Ll][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DOUBLE); return SDB_TOKEN_DOUBLE; }
[Dd][Rr][Oo][Pp]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_DROP); return SDB_TOKEN_DROP; }
[Ee][Ss][Cc][Aa][Pp][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ESCAPE); return SDB_TOKEN_ESCAPE; }
[Ee][Xx][Cc][Ee][Pp][Tt]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_EXCEPT); return SDB_TOKEN_EXCEPT; }
[Ee][Xx][Ii][Ss][Tt][Ss]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_EXISTS); return SDB_TOKEN_EXISTS; }
[Ee][Xx][Tt][Rr][Aa][Cc][Tt]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_EXTRACT); return SDB_TOKEN_EXTRACT; }
[Ff][Aa][Ll][Ss][Ee]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FALSE); return SDB_TOKEN_FALSE; }
[Ff][Ee][Tt][Cc][Hh]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FETCH); return SDB_TOKEN_FETCH; }
[Ff][Ll][Oo][Aa][Tt]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FLOAT); return SDB_TOKEN_FLOAT; }
[Ff][Nn]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FN); return SDB_TOKEN_FN; }
[Ff][Oo][Rr]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FOR); return SDB_TOKEN_FOR; }
[Ff][Oo][Rr][Ee][Ii][Gg][Nn]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FOREIGN); return SDB_TOKEN_FOREIGN; }
[Ff][Oo][Uu][Nn][Dd]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FOUND); return SDB_TOKEN_FOUND; }
[Ff][Rr][Oo][Mm]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FROM); return SDB_TOKEN_FROM; }
[Ff][Uu][Ll][Ll]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_FULL); return SDB_TOKEN_FULL; }
[Gg][Rr][Aa][Nn][Tt]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_GRANT); return SDB_TOKEN_GRANT; }
[Gg][Rr][Oo][Uu][Pp]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_GROUP); return SDB_TOKEN_GROUP; }
[Hh][Aa][Vv][Ii][Nn][Gg]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_HAVING); return SDB_TOKEN_HAVING; }
[Hh][Oo][Uu][Rr]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_HOUR); return SDB_TOKEN_HOUR; }
[Ii][Nn]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_IN); return SDB_TOKEN_IN; }
[Ii][Nn][Nn][Ee][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_INNER); return SDB_TOKEN_INNER; }
[Ii][Nn][Ss][Ee][Rr][Tt]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_INSERT); return SDB_TOKEN_INSERT; }
[Ii][Nn][Tt]([Ee][Gg][Ee][Rr])?					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_INTEGER); return SDB_TOKEN_INTEGER; }
[Ii][Nn][Tt][Ee][Rr][Ss][Ee][Cc][Tt]			{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_INTERSECT); return SDB_TOKEN_INTERSECT; }
[Ii][Nn][Tt][Oo]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_INTO); return SDB_TOKEN_INTO; }
[Ii][Ss]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_IS); return SDB_TOKEN_IS; }
[Jj][Oo][Ii][Nn]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_JOIN); return SDB_TOKEN_JOIN; }
[Kk][Ee][Yy]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_KEY); return SDB_TOKEN_KEY; }
[Ll][Ee][Aa][Dd][Ii][Nn][Gg]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_LEADING); return SDB_TOKEN_LEADING; }
[Ll][Ee][Ff][Tt]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_LEFT); return SDB_TOKEN_LEFT; }
[Ll][Ii][Kk][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_LIKE); return SDB_TOKEN_LIKE; }
[Ll][Oo][Cc][Aa][Ll]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_LOCAL); return SDB_TOKEN_LOCAL; }
[Ll][Oo][Ww][Ee][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_LOWER); return SDB_TOKEN_LOWER; }
[Mm][Aa][Xx]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_MAX); return SDB_TOKEN_MAX; }
[Mm][Ii][Nn]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_MIN); return SDB_TOKEN_MIN; }
[Mm][Ii][Nn][Uu][Tt][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_MINUTE); return SDB_TOKEN_MINUTE; }
[Mm][Oo][Nn][Tt][Hh]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_MONTH); return SDB_TOKEN_MONTH; }
[Nn][Aa][Tt][Uu][Rr][Aa][LL]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_NATURAL); return SDB_TOKEN_NATURAL; }
[Nn][Cc][Hh][Aa][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_NCHAR); return SDB_TOKEN_NCHAR; }
[Nn][Oo][Tt]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_NOT); return SDB_TOKEN_NOT; }
[Nn][Uu][Ll][Ll]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_NULL); return SDB_TOKEN_NULL; }
[Nn][Uu][Mm][Ee][Rr][Ii][Cc]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_NUMERIC); return SDB_TOKEN_NUMERIC; }
[Oo][Cc][Tt][Ee][Tt]_[Ll][Ee][Nn][Gg][Tt][Hh]	{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_OCTECT_LENGTH); return SDB_TOKEN_OCTECT_LENGTH; }
[Oo][Ff]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_OF); return SDB_TOKEN_OF; }
[Oo][Jj]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_OJ); return SDB_TOKEN_OJ; }
[Oo][Nn]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ON); return SDB_TOKEN_ON; }
[Oo][Pp][Tt][Ii][Oo][Nn]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_OPTION); return SDB_TOKEN_OPTION; }
[Oo][Rr]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_OR); return SDB_TOKEN_OR; }
[Oo][Rr][Dd][Ee][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ORDER); return SDB_TOKEN_ORDER; }
[Oo][Uu][Tt][Ee][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_OUTER); return SDB_TOKEN_OUTER; }
[Pp][Oo][Ss][Ii][Tt][Ii][Oo][Nn]	            {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_POSITION); return SDB_TOKEN_POSITION; }
[Pp][Rr][Ee][Cc][Ii][Ss][Ii][Oo][Nn]            {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_PRECISION); return SDB_TOKEN_PRECISION; }
[Pp][Rr][Ii][Mm][Aa][Rr][Yy]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_PRIMARY); return SDB_TOKEN_PRIMARY; }
[Pp][Rr][Ii][Vv][Ii][Ll][Ee][Gg][Ee][Ss]        {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_PRIVILEGES); return SDB_TOKEN_PRIVILEGES; }
[Pp][Rr][Oo][Cc][Ee][Dd][Uu][Rr][Ee]            {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_PROCEDURE); return SDB_TOKEN_PROCEDURE; }
[Pp][Uu][Bb][Ll][Ii][Cc]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_PUBLIC); return SDB_TOKEN_PUBLIC; }
[Rr][Ee][Aa][Ll]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_REAL); return SDB_TOKEN_REAL; }
[Rr][Ee][Ff][Ee][Rr][Ee][Nn][Cc][Ee][Ss]        {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_REFERENCES); return SDB_TOKEN_REFERENCES; }
[Rr][Oo][Ll][Ll][Bb][Aa][Cc][Kk]                {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ROLLBACK); return SDB_TOKEN_ROLLBACK; }
[Rr][Ii][Gg][Hh][Tt]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_RIGHT); return SDB_TOKEN_RIGHT; }
[Ss][Cc][Hh][Ee][Mm][Aa]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SCHEMA); return SDB_TOKEN_SCHEMA; }
[Ss][Ee][Cc][Ee][Oo][Nn][Dd]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SECOND); return SDB_TOKEN_SECOND; }
[Ss][Ee][Ll][Ee][Cc][Tt]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SELECT); return SDB_TOKEN_SELECT; }
[Ss][Ee][Tt]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SET); return SDB_TOKEN_SET; }
[Ss][Ii][Zz][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SIZE); return SDB_TOKEN_SIZE; }
[Ss][Mm][Aa][Ll][Ll][Ii][Nn][Tt]                {SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SMALLINT); return SDB_TOKEN_SMALLINT; }
[Ss][Oo][Mm][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SOME); return SDB_TOKEN_SOME; }
[Ss][Uu][Bb][Ss][Tt][Rr][Ii][Nn][Gg]			{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SUBSTRING); return SDB_TOKEN_SUBSTRING; }
[Ss][Uu][Mm]									{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_SUM); return SDB_TOKEN_SUM; }
[Tt]											{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_T); return SDB_TOKEN_T; }
[Tt][Aa][Bb][Ll][Ee]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TABLE); return SDB_TOKEN_TABLE; }
[Tt][Ii][Mm][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TIME); return SDB_TOKEN_TIME; }
[Tt][Ii][Mm][Ee][Ss][Tt][Aa][Mm][Pp]			{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TIMESTAMP); return SDB_TOKEN_TIMESTAMP; }
[Tt][Ii][Mm][Ee][Zz][Oo][Nn][Ee]_[Hh][Oo][Uu][Rr]	{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TIMEZONE_HOUR); return SDB_TOKEN_TIMEZONE_HOUR; }
[Tt][Ii][Mm][Ee][Zz][Oo][Nn][Ee]_[Mm][Ii][Nn][Uu][Tt][Ee]	{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TIMEZONE_MINUTE); return SDB_TOKEN_TIMEZONE_MINUTE; }
[Tt][Oo]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TO); return SDB_TOKEN_TO; }
[Tt][Rr][Aa][Ii][Ll][Ii][Nn][Gg]				{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TRAILING); return SDB_TOKEN_TRAILING; }
[Tt][Rr][Aa][Nn][Ss][Ll][Aa][Tt][Ee]			{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TRANSLATE); return SDB_TOKEN_TRANSLATE; }
[Tt][Rr][Ii][Mm]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TRIM); return SDB_TOKEN_TRIM; }
[Tt][Rr][Uu][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TRUE); return SDB_TOKEN_TRUE; }
[Tt][Ss]										{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_TS); return SDB_TOKEN_TS; }
[Uu][Nn][Ii][Oo][Nn]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_UNION); return SDB_TOKEN_UNION; }
[Uu][Nn][Ii][Qq][Uu][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_UNIQUE); return SDB_TOKEN_UNIQUE; }
[Uu][Nn][Kk][Nn][Oo][Ww][Nn]					{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_UNKNOWN); return SDB_TOKEN_UNKNOWN; }
[Uu][Pp][Pp][Ee][Rr]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_UPPER); return SDB_TOKEN_UPPER; }
[Uu][Pp][Dd][Aa][Tt][Ee]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_UPDATE); return SDB_TOKEN_UPDATE; }
[Uu][Ss][Aa][Gg][Ee]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_USAGE); return SDB_TOKEN_USAGE; }
[Uu][Ss][Ee][Rr]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_USER); return SDB_TOKEN_USER; }
[Uu][Ss][Ii][Nn][Gg]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_USING); return SDB_TOKEN_USING; }
[Vv][Aa][Ll][Uu][Ee][Ss]						{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_VALUES); return SDB_TOKEN_VALUES; }
[Vv][Ii][Ee][Ww]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_VIEW); return SDB_TOKEN_VIEW; }
[Ww][Hh][Ee][Rr][Ee]							{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_WHERE); return SDB_TOKEN_WHERE; }
[Ww][Ii][Tt][Hh]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_WITH); return SDB_TOKEN_WITH; }
[Ww][Oo][Rr][Kk]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_WORK); return SDB_TOKEN_WORK; }
[Yy][Ee][Aa][Rr]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_YEAR); return SDB_TOKEN_YEAR; }
[Zz][Oo][Nn][Ee]								{SDB_SQL_NEW_KEYWORD(SDB_TOKEN_ZONE); return SDB_TOKEN_ZONE; }

"<"					{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_LESS);return LESS;}
">"					{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_GREAT);return GREAT;}
"="					{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_EQUAL);return EQUAL;}
"<="				{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_LESSEQ);return LESSEQ;}
">="				{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_GREATEQ);return GREATEQ;}
"<>"				{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_NOTEQUAL);return NOTEQUAL;}
"!="				{ NEW_NODE(String::CreateFromAscii(SQLyytext), SDB_SQL_NOTEQUAL);return NOTEQUAL;}

[-+*/:(),.;?{}]    { return SQLyytext[0]; }

<SQL>[A-Za-z][A-Za-z0-9_]*		{return gatherName( SQLyytext);}

<SQL>([0-9]+) | 
<SQL>([0-9]+"."[0-9]*) | 
<SQL>("."[0-9]*)					{SDB_SQL_NEW_INTNUM; return SDB_TOKEN_INTNUM;}

<SQL>[0-9]+[eE][+-]?[0-9]+   |
<SQL>[0-9]+"."[0-9]*[eE][+-]?[0-9]+ |
<SQL>"."[0-9]*[eE][+-]?[0-9]+		{SDB_SQL_NEW_APPROXNUM; return SDB_TOKEN_APPROXNUM; }

<PREDICATE_GER,PREDICATE_ENG,DATE>[A-Za-z][A-Za-z0-9_%.,*?]* {return gatherNamePre(SQLyytext);}

<PREDICATE_GER,PREDICATE_ENG>([0-9]+)		{SDB_SQL_NEW_INTNUM; return SDB_TOKEN_INTNUM;}
<PREDICATE_ENG>([0-9]{1,3}(","[0-9]{3})+) 	{SDB_SQL_NEW_INTNUM; return SDB_TOKEN_INTNUM;}
<PREDICATE_GER>([0-9]{1,3}("."[0-9]{3})+)	{SDB_SQL_NEW_INTNUM; return SDB_TOKEN_INTNUM;}

<PREDICATE_ENG>([0-9]+"."[0-9]+) |
<PREDICATE_ENG>([0-9]{1,3}(","[0-9]{3})+"."[0-9]+) | 
<PREDICATE_ENG>("."[0-9]+)					{SDB_SQL_NEW_APPROXNUM; return SDB_TOKEN_APPROXNUM; }
<PREDICATE_ENG>[0-9]+[eE][+-]?[0-9]+   |
<PREDICATE_ENG>[0-9]+"."[0-9]*[eE][+-]?[0-9]+ |
<PREDICATE_ENG>"."[0-9]*[eE][+-]?[0-9]+		{SDB_SQL_NEW_APPROXNUM; return SDB_TOKEN_APPROXNUM; }

<PREDICATE_GER>([0-9]+","[0-9]+) |
<PREDICATE_GER>([0-9]{1,3}("."[0-9]{3})+","[0-9]+) | 
<PREDICATE_GER>(","[0-9]+)					{SDB_SQL_NEW_APPROXNUM; return SDB_TOKEN_APPROXNUM; }
<PREDICATE_GER>[0-9]+[eE][+-]?[0-9]+   |
<PREDICATE_GER>[0-9]+","[0-9]*[eE][+-]?[0-9]+ |
<PREDICATE_GER>","[0-9]*[eE][+-]?[0-9]+		{SDB_SQL_NEW_APPROXNUM; return SDB_TOKEN_APPROXNUM; }

<PREDICATE_GER,PREDICATE_ENG>[0-9.,][A-Za-z0-9_.,%]* {return gatherNamePre(SQLyytext);}

<SQL>\"	{ return gatherString('\"',0); }
<SQL>`	{ return gatherString('`' ,0); }											 

<PREDICATE_GER,PREDICATE_ENG,DATE>"["		{ return gatherString(']' ,0);}

\'		{ return gatherString('\'',1); }

<PREDICATE_GER,PREDICATE_ENG,DATE>#	{ return gatherString('#' ,2); }	/* Datum fuer Access */

<DATE>[0-9]{1,4}[^ ]*[0-9] |
<DATE>[0-9]{1,4}[^ ]*[0-9][ ][0-9]{1,4}[^ ]*[0-9]		{ SDB_SQL_NEW_DATE; return SDB_TOKEN_ACCESS_DATE;}

<STRING>["-""+""*""/"":""("")"",""."";""?""{""}"]		{ return SQLyytext[0]; } /*  */
<STRING>"["					{ return gatherString(']' ,0); }
<STRING>[^ ':[]*			{ return gatherNamePre(SQLyytext); }

\n              {}

[ \t\r]+        ;       /* white space */

"--".*$         ;       /* comment */

.				{YY_FATAL_ERROR("Invalid symbol"); return SDB_TOKEN_INVALIDSYMBOL;}

%%

/*
 * Read SQL string literal
 * Valid strings:
 *	''	'a string'	'quote '' within string'
 *	""	"a string"	"quote "" within string"
 * nTyp == 0 -> SDB_SQL_NAME
 * nTyp == 1 -> SDB_SQL_STRING
 * nTyp == 2 -> SDB_SQL_ACCESS_DATE
 */
int gatherString( int delim, int nTyp)
{
	int ch;
	static sal_Int32 BUFFERSIZE = 256;
	static char* Buffer = new char[BUFFERSIZE];
	
	char *s = Buffer;
	int nPos = 0;

	while ((ch = yyinput()) != EOF) 
	{
		if (ch == delim) 
		{
			if ((ch = yyinput()) != delim) 
			{
				if (ch != EOF)
					unput(ch);
				
				*s = '\0';		 		
				
				switch(nTyp)
				{
					case 0:
						NEW_NODE(String(Buffer,RTL_TEXTENCODING_UTF8), SDB_SQL_NAME);
						return SDB_TOKEN_NAME;											
					case 1:
						NEW_NODE(String(Buffer,RTL_TEXTENCODING_UTF8), SDB_SQL_STRING);
						return SDB_TOKEN_STRING;
					case 2:
						NEW_NODE(String(Buffer,RTL_TEXTENCODING_UTF8), SDB_SQL_ACCESS_DATE);
						return SDB_TOKEN_ACCESS_DATE;
				}				
			} 
			else
			{
				*s++ = ch;
				if (++nPos == BUFFERSIZE)
				{
					ByteString aBuf(Buffer);
					delete Buffer;
					BUFFERSIZE *=2;
					Buffer = new char[BUFFERSIZE];
					for(xub_StrLen i=0;i<aBuf.Len();++i,++Buffer)
						*Buffer = aBuf.GetChar(i);
					s = &Buffer[nPos];
				}
			}

		} 
		else if (ch == '\r' || ch == '\n')
			break;					
		else
		{
			*s++ = ch;
			if (++nPos == BUFFERSIZE)
			{
				ByteString aBuf(Buffer);
				delete Buffer;
				BUFFERSIZE *=2;
				Buffer = new char[BUFFERSIZE];
				for(xub_StrLen i=0;i<aBuf.Len();++i,++Buffer)
					*Buffer = aBuf.GetChar(i);
				s = &Buffer[nPos];
			}
		}
	}
	*s = '\0';		 		
	YY_FATAL_ERROR("Unterminated name string"); 
	return SDB_TOKEN_INVALIDSYMBOL;		
}

/*
 * Read SQL Name literal
 * Valid Names or internatioanl keywords:
 *	As we have international keywords, we test first on them  
 */
int gatherName(const char* text)
{
	DBG_ASSERT(xxx_pGLOBAL_SQLSCAN,"You forgot to set the scanner!");
	int nToken = xxx_pGLOBAL_SQLSCAN->CheckForIntlKeyword(text);
	switch (nToken)
	{
		case SDB_TOKEN_LIKE:
		case SDB_TOKEN_NOT:
		case SDB_TOKEN_NULL:
		case SDB_TOKEN_TRUE:
		case SDB_TOKEN_FALSE:
		case SDB_TOKEN_IS:
		case SDB_TOKEN_BETWEEN:
		case SDB_TOKEN_OR:
		case SDB_TOKEN_AND:
		case SDB_TOKEN_COUNT:
		case SDB_TOKEN_AVG:
		case SDB_TOKEN_MAX:
		case SDB_TOKEN_MIN:
		case SDB_TOKEN_SUM:
			SDB_SQL_NEW_KEYWORD(nToken); 
			return nToken;
		default:
			NEW_NODE(String::CreateFromAscii(text), SDB_SQL_NAME); 
			return SDB_TOKEN_NAME; 
	}
}
/**
 Read SQL Name literal for predicate check
 Valid Names or internatioanl keywords:
 As we have international keywords, we test first on them  
*/
int gatherNamePre(const char* text)
{ 
	DBG_ASSERT(xxx_pGLOBAL_SQLSCAN,"You forgot to set the scanner!");
	int nToken = xxx_pGLOBAL_SQLSCAN->CheckForIntlKeyword(text);
	switch (nToken)
	{
		case SDB_TOKEN_LIKE:
		case SDB_TOKEN_NOT:
		case SDB_TOKEN_NULL:
		case SDB_TOKEN_TRUE:
		case SDB_TOKEN_FALSE:
		case SDB_TOKEN_IS:
		case SDB_TOKEN_BETWEEN:
		case SDB_TOKEN_OR:
		case SDB_TOKEN_AND:
		case SDB_TOKEN_COUNT:
		case SDB_TOKEN_AVG:
		case SDB_TOKEN_MAX:
		case SDB_TOKEN_MIN:
		case SDB_TOKEN_SUM:
			SDB_SQL_NEW_KEYWORD(nToken); 
			return nToken;
		default:
			// we need a special handling for parameter
			if (yy_hold_char == ':')
			{
				NEW_NODE(String::CreateFromAscii(text), SDB_SQL_NAME); 
				return SDB_TOKEN_NAME; 				
			}
			else
			{
				NEW_NODE(String::CreateFromAscii(text), SDB_SQL_STRING);
				return SDB_TOKEN_STRING;			
			}
	}															   	
}
#include "sdbscan.hxx"
#include "sdbparse.hxx"
#include "svtools.hrc"
#include "svtdata.hxx"

#ifndef _VOS_MUTEX_HXX_ //autogen wg. OMutex
#include <vos/mutex.hxx>
#endif

static sal_uInt32 Intl_TokenID[] =
{
	SDB_TOKEN_LIKE, SDB_TOKEN_NOT, SDB_TOKEN_NULL, SDB_TOKEN_TRUE,
	SDB_TOKEN_FALSE, SDB_TOKEN_IS, SDB_TOKEN_BETWEEN, SDB_TOKEN_OR,
	SDB_TOKEN_AND, SDB_TOKEN_AVG, SDB_TOKEN_COUNT, SDB_TOKEN_MAX,
	SDB_TOKEN_MIN, SDB_TOKEN_SUM
};

static String aSQLInternationals;
static LanguageType eLanguage;

DBG_NAME(SdbSqlScanner);

SdbSqlScanner::SdbSqlScanner(LanguageType eLang)
			  : nCurrentPos(0)
			  ,bInternational(FALSE)
			  , m_nRule(0) // 0 is INITIAL
{
	DBG_CTOR(SdbSqlScanner,NULL);
	
	SetLanguage(eLang);	
}

void SdbSqlScanner::SetLanguage(LanguageType eType)
{
	static NAMESPACE_VOS(OMutex) aMutex;
	NAMESPACE_VOS(OGuard) aGuard( aMutex );

	if (!aSQLInternationals.Len() || eLanguage != eType)
	{
		eLanguage = eType;
		aSQLInternationals = SvtSimpleResId(STR_SVT_SQL_INTERNATIONAL,eLanguage);
	}
}

SdbSqlScanner::~SdbSqlScanner()
{
	DBG_DTOR(SdbSqlScanner,NULL);
}

void SdbSqlScanner::Sdbyyerror(char *fmt, ...)
{
	aErrorMessage = SvtSimpleResId(STR_SVT_SQL_SYNTAX_ERROR, eLanguage);
	if (nCurrentPos < aString.Len())
	{
		aErrorMessage.AppendAscii(": ");
		aErrorMessage += String(SvtSimpleResId(STR_SVT_SQL_SYNTAX_ERROR_BEFORE, eLanguage));
		aErrorMessage.SearchAndReplaceAscii("#",String::CreateFromAscii(SQLyytext));	
	}
}

void SdbSqlScanner::Initialize(const String & rNewString, BOOL bIntl)
{
	DBG_CHKTHIS(SdbSqlScanner,NULL);

	BEGIN(m_nRule);

	aErrorMessage.Erase();	
	aString = rNewString;
	nCurrentPos = 0;
	bInternational = bIntl;
}

int SdbSqlScanner::Sdbyygetc(void)
{
	return (nCurrentPos >= aString.Len()) ? -1 : ByteString::ConvertFromUnicode(aString.GetChar(nCurrentPos++),RTL_TEXTENCODING_ASCII_US);
}

int SdbSqlScanner::CheckForIntlKeyword(const char* sToken) const
{		
	int nRet = 0;
	if (bInternational)
	{
		xub_StrLen nIndex = 0;
		sal_Int32 i = 0;
		String aKey;
		aKey.AssignAscii(sToken);
		for (String aToken = aSQLInternationals.GetToken(0,';',nIndex);
			 aToken.Len();
			 aToken = aSQLInternationals.GetToken(0,';',nIndex))
		{
			if (aToken.EqualsIgnoreCaseAscii(aKey))
				nRet = Intl_TokenID[i];
			else
				i++;
		}
	}
	return nRet;
}

String SdbSqlScanner::GetInternationalKeywordString(sal_uInt32 nTokenID)
{
	sal_uInt32 nCount = sizeof Intl_TokenID / sizeof Intl_TokenID[0];
	for (sal_uInt32 i = 0; i < nCount; i++)
	{
		if (Intl_TokenID[i] == nTokenID)
			return aSQLInternationals.GetToken(i,';');
	}

	return SdbSqlParser::TokenIDToStr(nTokenID);
}

// -------------------------------------------------------------------------
int	SdbSqlScanner::GetCurrentRule() const { return m_nRule; }
int	SdbSqlScanner::GetGERRule() const		{ return PREDICATE_GER; }
int	SdbSqlScanner::GetENGRule() const		{ return PREDICATE_ENG; }
int	SdbSqlScanner::GetSQLRule() const		{ return SQL; }
int	SdbSqlScanner::GetDATERule() const	{ return DATE; }
int	SdbSqlScanner::GetSTRINGRule() const	{ return STRING; }
// -------------------------------------------------------------------------
void SdbSqlScanner::setScanner(sal_Bool _bNull)
{
	xxx_pGLOBAL_SQLSCAN = _bNull ? NULL : this;
}
// -------------------------------------------------------------------------
int SdbSqlScanner::SQLlex()
{
	return SQLyylex();
}

/*--------------------------------------------------------------------------*

Aenderungshistorie:
	$Log: sdblex.l,v $
	Revision 1.13  2000/07/25 10:07:58  oj
	use of bison and flex
	
	Revision 1.12  2000/04/05 09:27:07  oj
	unistring
	
	Revision 1.11  2000/01/12 10:41:52  oj
	#71765# lex modi for string
	
	Revision 1.10  2000/01/04 08:30:53  oj
	#71363# error in lex description
	
	Revision 1.9  1999/12/09 17:23:47  oj
	#70639# #70643# some changes for datetime and strings
	
	Revision 1.8  1999/11/03 14:30:43  hr
	#65293#: STLPORT 3.2.1
	
	Revision 1.7  1999/11/01 16:10:45  oj
	some changes
	
	Revision 1.6  1999/10/29 15:07:17  oj
	some changes on sql syntax
	
	Revision 1.5  1999/09/06 08:43:43  dg
	DateTime Handling
	
	Revision 1.4  1999/09/01 16:44:04  dg
	Enhancements for predicate checking
	
	Revision 1.3  1999/08/30 08:11:43  dg
	#new improvements in the context of PredicateChecking
	
	Revision 1.2  1999/08/25 10:21:53  dg
	Errors of changing the container implementation fixed
	
	Revision 1.1  1999/08/20 14:54:00  dg
	SQLParser moved from SDB to svtools
	
	Revision 1.29  1999/07/19 06:20:33  oj
	#67708# predicate_check and number convention
	
	Revision 1.28  1999/06/25 12:03:38  OJ
	gatherstring has now no more limitaion
	
   
      Rev 1.27   25 Jun 1999 14:03:38   OJ
   gatherstring has now no more limitaion
   
      Rev 1.26   05 Feb 1999 09:47:50   OJ
   Fehler im Keyword
   
      Rev 1.25   14 Jan 1999 14:45:30   OJ
    #60583# neues Keyword fuer Access Datum
   
      Rev 1.24   21 Dec 1998 19:43:42   OJ
   #60050# neue klammern
   
      Rev 1.23   03 Dec 1998 09:19:50   OJ
   #60050# neue Keywords
   
      Rev 1.22   28 Aug 1997 12:51:12   DG
   includes
   
      Rev 1.21   29 Apr 1997 16:35:10   DG
   
      Rev 1.20   24 Jan 1997 14:55:34   DG
   erweitert
   
      Rev 1.19   22 Jan 1997 13:24:42   DG
   
      Rev 1.18   09 Jul 1996 16:36:12   DG
   
      Rev 1.17   02 Jul 1996 12:23:46   DG
   Absturz bei  Bufferueberschreitung behoben
   
      Rev 1.16   24 Jun 1996 17:01:22   DG
   Single Quotes neu
   
      Rev 1.15   18 Apr 1996 18:25:08   DG
   Scanner um TRUE und FALSE erweitert
   
      Rev 1.14   31 Mar 1995 12:13:00   RDM
   Eigenes Include auf sdblex.hxx (um sdb.hxx von dem Scanner freizuhalten)
   
      Rev 1.13   30 Mar 1995 17:59:12   RDM
   Umstellung auf MKS Lex/Yacc
   
      Rev 1.12   23 Feb 1995 16:26:36   RDM
   Umstellung auf SdbEnvironment (noch nicht durch Lex gelaufen!)
   
      Rev 1.11   22 Feb 1995 19:52:04   GH
   Fix der EOF-Behandlung in my_yyinput (gezaubert), sonst wird bei Strings am
   Ende eines Statements der INhalt verschluckt.
   
      Rev 1.10   22 Feb 1995 19:04:30   GH
   INVALIDSYMBOL als Token fuer das Abfangen von Fehlern eingebaut.
   Einzelne nicht erkannte Zeichen werden auch mit INVALIDSYMBOL
   zurueckgegeben.
   
      Rev 1.9   22 Feb 1995 14:14:34   GH
   Folgende Token rausgeworfen: alle Sprachen, goto, open, close
   
      Rev 1.8   17 Feb 1995 18:33:46   GH
   Bei Parameter wird zusaetzlich zu :NAME auch ? akzeptiert. Lexer reicht
   ? einfach durch.
   
      Rev 1.7   14 Feb 1995 14:32:32   GH
   Fehler bei count, min, max, avg, sum beseitgt.
   
      Rev 1.6   14 Feb 1995 10:02:42   GH
   
      Rev 1.5   14 Feb 1995 08:51:00   GH
   Versions-Header und Log-Epilog hinzugefuegt.
   
 *--------------------------------------------------------------------------*/
 
