/*************************************************************************
 *
 *  $RCSfile: sdbparse.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SVT_SDBPARSE_HXX
#define _SVT_SDBPARSE_HXX

#ifndef _SVT_SDBSQL_HXX
#include "svtools/sdbsql.hxx"
#endif

#ifndef _LANG_HXX
#include <tools/lang.hxx>
#endif

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

// forward declarations
namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace beans
			{
				class XPropertySet;
			}
			namespace util
			{
				class XNumberFormatter;
			}
		}
	}
}

class SdbSqlScanner;
class International;

// wichtig: dieses define mu vorhanden sein, auch im sdb.hdb -> sdb.hxx
// sonst hat der Parser eine falsche Size siehe sdbyacc.hxx
#ifndef YYBISON
#ifndef FLEX_SCANNER
#include "svtools/sdbbison.hxx"
#endif
#endif

class SdbSqlParser 
{
	friend class SdbSqlParseNode;

private:
	static sal_uInt32 nRuleIDs[SdbSqlParseNode::rule_count + 1];
	SdbSqlParseNode*	m_pParseTree;	// Ergebnis des Parsens: ein Parse Tree
	SdbSqlScanner*		m_pScanner;
	International*		m_pIntl;
	String				m_aFieldName;
	String				m_aErrorMessage;
	::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet >     m_xField;
	::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter >  m_xFormatter;
	sal_Int32			m_nFormatKey;
	LanguageType		m_eLanguage;

public:
	SdbSqlParser(LanguageType);
	~SdbSqlParser();

	// Parsing an SQLStatement
	SdbSqlParseNode* ParseTree(String& rErrorMessage,
				   const String& rStatement,
				   sal_Bool bInternational = sal_False);
	// Check a Predicate
	SdbSqlParseNode* PredicateTree(String& rErrorMessage, const String& rStatement,
				       const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
				       const International& rIntl,
				       const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & xField);

	void SetLanguage(LanguageType eType);
	LanguageType GetLanguageType() const {return m_eLanguage;}

	static String TokenIDToStr(sal_uInt32 nTokenID, sal_Bool bInternational);

	// TokenIDToStr: Token-Name zu einer Token-Nr.
	static String TokenIDToStr(sal_uInt32 nTokenID);

	// StrToTokenID: Token-Nr. zu einem Token-Namen.
	static sal_uInt32 StrToTokenID(const ByteString & rName);

	// RuleIDToStr gibt den zu einer RuleID gehoerenden String zurueck
	// (Leerstring, falls nicht gefunden)
	static String RuleIDToStr(sal_uInt32 nRuleID);

	// StrToRuleID berechnet zu einem String die RuleID (d.h. ::com::sun::star::sdbcx::Index in yytname)
	// (0, falls nicht gefunden). Die Suche nach der ID aufgrund eines Strings ist
	// extrem ineffizient (sequentielle Suche nach String)!
	static sal_uInt32 StrToRuleID(const ByteString & rValue);

	// RuleId mit enum, wesentlich effizienter
	static sal_uInt32 RuleID(SdbSqlParseNode::Rule eRule);

	static String GetInternationalKeywordString(sal_uInt32 nTokenID);

	void error(char *fmt);
	int SQLlex();
#ifdef YYBISON
	inline void SetParseTree(SdbSqlParseNode * pNewParseTree) {m_pParseTree = pNewParseTree;}

	// Is the parse in a special mode?
	// Predicate chack is used to check a condition for a field
	sal_Bool InPredicateCheck() const {return m_xField.is();}
	const String& GetFieldName() const {return m_aFieldName;}

	void ReduceLiteral(SdbSqlParseNode*& pLiteral, sal_Bool bAppendBlank);
	 // does not change the pLiteral argument
	sal_Int16 BuildNode(SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral,SdbSqlParseNode*& pCompare);
	// makes a string out of a number, pLiteral will be deleted
	sal_Int16 BuildNode_STR_NUM(SdbSqlParseNode*& pAppend,SdbSqlParseNode*& pLiteral,SdbSqlParseNode*& pCompare);
	sal_Int16 BuildNode_Date(const double& fValue, sal_Int16 nType, SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral,SdbSqlParseNode*& pCompare);

	sal_Int16 BuildComparsionRule(SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral);
	// pCompre will be deleted if it is not used
	sal_Int16 BuildComparsionRule(SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral,SdbSqlParseNode*& pCompare);

	sal_Int16 BuildLikeRule(SdbSqlParseNode*& pAppend,SdbSqlParseNode*& pLiteral,const SdbSqlParseNode* pEscape);
	sal_Int16 BuildStringNodes(SdbSqlParseNode*& pLiteral);
#else
#endif
};

#endif

