/*************************************************************************
 *
 *  $RCSfile: urlbmk.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 16:59:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef WNT
#include <tools/prewin.h>
#include <shlobj.h>
#include <tools/postwin.h>
#endif

#include <tools/urlobj.hxx>

#pragma hdrstop

#include <string.h>
#if defined(UNX) || defined(MAC)
#else
#include <io.h>
#ifdef WTC
#include <dos.h>
#endif
#endif
#ifndef _SOT_FORMATS_HXX //autogen
#include <sot/formats.hxx>
#endif

#include "urlbmk.hxx"

#ifdef OS2
#include "extattr.hxx"
#endif

//Sonst internal compiler error mit BLC 1.5
#ifdef BLC
#define STATIC_CONST
#else
#ifdef WNT
// sonst wird in Win95 / WINNT diese Variable NIE initialisiert!
#define STATIC_CONST static
#else
#define STATIC_CONST static const
#endif
#endif

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif
#ifndef _SV_DRAG_HXX //autogen
#include <vcl/drag.hxx>
#endif
#ifndef _SV_CLIP_HXX //autogen
#include <vcl/clip.hxx>
#endif
#include <sot/dtrans.hxx>
#include <sot/exchange.hxx>

//=======================================================================

BOOL INetBookmark::ClipboardHasFormat()

/*	[Beschreibung]

	Liefert TRUE, falls sich im Clipboard das Bookmark-Format befindet,
	sonst FALSE.
*/

{ return Clipboard::HasFormat( SOT_FORMATSTR_ID_SOLK ); }

//-----------------------------------------------------------------------

BOOL INetBookmark::DragServerHasFormat( USHORT nItem )

/*	[Beschreibung]

	Liefert TRUE, falls sich im DragServer an Position 'nItem' das
	Bookmark-Format befindet, sonst FALSE.
*/

{
	return
		DragServer::HasFormat( nItem, SOT_FORMATSTR_ID_SOLK ) ||
		DragServer::HasFormat( nItem, SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK ) ||
		DragServer::HasFormat( nItem, SOT_FORMATSTR_ID_FILENAME );
}

//-----------------------------------------------------------------------

BOOL INetBookmark::CopyClipboard() const

/*	[Beschreibung]

	Diese Methode kopiert diese Instanz in das Clipboard.
*/

{
	static ULONG __READONLY_DATA aFormats[] = {
		SOT_FORMATSTR_ID_SOLK,
		FORMAT_STRING,
		SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK,
		SOT_FORMATSTR_ID_UNIFORMRESOURCELOCATOR,
#ifdef WNT
		SOT_FORMATSTR_ID_FILEGRPDESCRIPTOR,
		SOT_FORMATSTR_ID_FILECONTENT,
#endif
		0
	};

	for( const ULONG* p = aFormats; *p; ++p )
	{
		SvData aData( *p, MEDIUM_ALL );
		if( SetData( aData ) )
		{
			void *pData;
			aData.GetData( &pData, TRANSFER_REFERENCE );
			UINT32 nLen = aData.GetMinMemorySize();

			Clipboard::CopyData( pData, nLen, *p );
		}
	}
	return TRUE;
}

//-----------------------------------------------------------------------

String INetBookmark::CopyExchange() const
{
	 String aString( String::CreateFromInt32( aUrl.Len() ));
	 aString += '@';
	 aString += aUrl;
	 aString += String::CreateFromInt32( aDescr.Len() );
	 aString += '@';
	 aString += aDescr;
	 return aString;
}

//-----------------------------------------------------------------------

BOOL INetBookmark::CopyDragServer() const

/*	[Beschreibung]

	Diese Methode kopiert diese Instanz in den DragServer.
*/

{
	static ULONG __READONLY_DATA aFormats[] = {
		SOT_FORMATSTR_ID_SOLK,
		FORMAT_STRING,
		SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK,
		SOT_FORMATSTR_ID_UNIFORMRESOURCELOCATOR,
#ifdef WNT
		SOT_FORMATSTR_ID_FILEGRPDESCRIPTOR,
		SOT_FORMATSTR_ID_FILECONTENT,
#endif
		0
	};

	for( const ULONG* p = aFormats; *p; ++p )
	{
		SvData aData( *p, MEDIUM_ALL );
		if( SetData( aData ) )
		{
			void *pData;
			aData.GetData( &pData, TRANSFER_REFERENCE );
			UINT32 nLen = aData.GetMinMemorySize();

			DragServer::CopyData( pData, nLen, *p );
		}
	}
	return TRUE;
}

//-----------------------------------------------------------------------

BOOL INetBookmark::PasteClipboard()

/*	[Beschreibung]

	Diese Methode "uberschreibt den Wert dieser Instanz mit dem
	Inhalt des Clipboards. Es wird vorausgesetzt, da\s sich das eigene
	Format im Clipboard befindet.
*/

{
	 DBG_ASSERTWARNING( Clipboard::HasFormat( SOT_FORMATSTR_ID_SOLK ), "Has not" );
	 ULONG nLen = Clipboard::GetDataLen( SOT_FORMATSTR_ID_SOLK );
	 ByteString aClpData;
	 Clipboard::PasteData( aClpData.AllocBuffer( nLen ), nLen,
	 						SOT_FORMATSTR_ID_SOLK );
	 PasteExchange( String( aClpData, gsl_getSystemTextEncoding() ));
	 return TRUE;
}

//-----------------------------------------------------------------------

void INetBookmark::PasteExchange( String aString )
{
	 xub_StrLen nStart	= aString.Search( '@' );
	 xub_StrLen nLen = aString.ToInt32();
	 if( !nLen && aString.GetChar( 0 ) != '0' )
	 {
		  DBG_WARNING( "SOLK: 1. len=0" )
		  return;
	 }
	 if ( nStart == STRING_NOTFOUND || nLen > aString.Len() - nStart - 3 )
	 {
		  DBG_WARNING( "SOLK: 1. illegal start or wrong len" )
		  return;
	 }
	 aUrl = aString.Copy( nStart + 1, nLen );

	 aString.Erase( 0, nStart + 1 + nLen );

	 nStart = aString.Search( '@' );
	 nLen = aString.ToInt32();
	 if ( !nLen && aString.GetChar( 0 ) != '0' )
	 {
		  DBG_WARNING( "SOLK: 2. len=0" )
		  return;
	 }
	 if ( nStart == STRING_NOTFOUND || nLen > aString.Len() - nStart - 1 )
	 {
		  DBG_WARNING( "SOLK: 2. illegal start or wrong len" )
		  return;
	 }
	 aDescr = aString.Copy( nStart+1, nLen );
}

//-----------------------------------------------------------------------

BOOL INetBookmark::PasteDragServer( USHORT nItem )

/*	[Beschreibung]

	Diese Methode "uberschreibt den Wert dieser Instanz mit dem
	Inhalt des DragServers an Position 'nItem'. Es wird vorausgesetzt,
	da\s sich das eigene Format im DragServer befindet.
*/

{
	BOOL bRet = FALSE, bSetUrl = FALSE, bSetDesc = FALSE;
	ULONG nFormat = SOT_FORMATSTR_ID_SOLK;
	rtl_TextEncoding eSysCSet = gsl_getSystemTextEncoding();
	ByteString sUrl, sDescr;
	if( DragServer::HasFormat( nItem, nFormat ) )
	{
		ULONG nLen = DragServer::GetDataLen( nItem, nFormat );
		ByteString aClpData;
		DragServer::PasteData( nItem, aClpData.AllocBuffer( nLen ), nLen,
								nFormat );
	 	PasteExchange( String( aClpData, eSysCSet ));
		bRet = TRUE;
	}
	else if( DragServer::HasFormat( nItem,
					( nFormat = SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK ) ) )
	{
		// JP 05.02.96: zur Zeit definiert als
		//		char[ 1024 ] -> URL
		//		char[ 1024 ] -> Hinweistext
		ULONG nLen = DragServer::GetDataLen( nItem, nFormat );
		if( 2048 == nLen )
		{
			ByteString aClpData;
			DragServer::PasteData( nItem, aClpData.AllocBuffer( nLen ),
									nLen, nFormat );
			sUrl = aClpData.GetBuffer();
			sDescr = aClpData.GetBuffer() + 1024;
			bRet = bSetUrl = bSetDesc = TRUE;
		}
	}
	else
	{
		String aFile( DragServer::PasteFile( nItem ) );
		if( !aFile.Len() &&
			(DragServer::HasFormat( nItem, nFormat = FORMAT_FILE ) ||
			DragServer::HasFormat( nItem, nFormat = SOT_FORMATSTR_ID_FILENAME ) ))
		{
			ULONG nLen = DragServer::GetDataLen( nItem, nFormat );
			ByteString aClpData;
			DragServer::PasteData( nItem, aClpData.AllocBuffer( nLen ),
									nLen, nFormat );
			aFile = String( aClpData.GetBuffer(), eSysCSet );
		}

#ifdef OS2
		// OS/2 URL-Object?
		SvEaMgr aMgr( aFile );
		String aType;
		if( aMgr.GetFileType( aType ) &&
			aType.EqualsAscii( "UniformResourceLocator" ))
		{
			SvFileStream aStream( aFile, STREAM_STD_READ );
			bRet = aStream.ReadLine( sUrl );
			bRet = bRet && aMgr.GetLongName( sDescr );
 			bSetUrl = bSetDesc = TRUE;
		}
		else

#endif
		{
			// ...
			xub_StrLen nPos;
			if( aFile.Len() &&
				STRING_NOTFOUND != (nPos = aFile.SearchAscii( ".URL" )) )
			{
				// Format fuer URL-Shortcuts vom Desktop:
				// FileName:	"Shortcut to" <Name> ".URL"
				// FileInhalt:	"URL=" <URL>

				ByteString sTmpURL;
				SvFileStream aTmp( aFile, STREAM_READ );
				if( !aTmp.GetError() && aTmp.ReadLine( sTmpURL ) &&
					aTmp.ReadLine( sTmpURL ) && 0 == sTmpURL.Search( "URL=" ))
				{
#ifdef WNT
					{
					ByteString aBFile( aFile, eSysCSet );
#if !defined( WTC )
					// wir wollen aber auf jedenfall den LangNamen haben !!
					_finddata_t aData;
					if( _findfirst( (char*)aBFile.GetBuffer(), &aData ) )
						aBFile.Assign( aData.name );
#else
					_find_t aData;
					if( _dos_findfirst( (char*)aBFile.GetBuffer(),
							_A_NORMAL | _A_RDONLY | _A_HIDDEN | _A_SYSTEM |
							_A_VOLID  | _A_SUBDIR | _A_ARCH, &aData) )
						aBFile.Assign( aData.name );
#endif
					aFile = String( aBFile, eSysCSet );
					}
#endif

                    aFile = INetURLObject( aFile, INET_PROT_FILE ).GetMainURL();
					String sLower( aFile );
					aFile.Erase( sLower.ToLowerAscii().SearchAscii( ".url" ) );

                    if( STRING_NOTFOUND != ( nPos = aFile.SearchAscii( "Shortcut to" )) )
						aFile.Erase( 0, nPos + 12 );

					sUrl = sTmpURL.Erase( 0, 4 );		// "URL=" loeschen
					aDescr = aFile.EraseLeadingChars().EraseTrailingChars();
					bSetUrl = bRet = TRUE;
				}
			}
		}
	}

	if( bSetUrl )
	{
		if( sUrl.Len() )
			aUrl = String( sUrl.GetBuffer(), eSysCSet );
		else
			aUrl.Erase();
	}

	if( bSetDesc )
	{
		if( sDescr.Len() )
			aDescr = String( sDescr.GetBuffer(), eSysCSet );
		else
			aDescr.Erase();
	}

	return bRet;
}


ULONG INetBookmark::HasFormat( const SotDataObject& rObj )

/*	[Beschreibung]

	Liefert TRUE, falls in dem SvDataObject ein entsp. Bookmark-Format
	gefunden wurde.
*/

{
	ULONG nFmt;
	const SvDataTypeList& rTypeLst = rObj.GetTypeList();
	if( !rTypeLst.Get( nFmt = SOT_FORMATSTR_ID_SOLK ) &&
		!rTypeLst.Get( nFmt = SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK ) &&
		!( rTypeLst.Get( SOT_FORMATSTR_ID_FILECONTENT ) &&
		   rTypeLst.Get( nFmt = SOT_FORMATSTR_ID_FILEGRPDESCRIPTOR )) &&
		!rTypeLst.Get( nFmt = SOT_FORMATSTR_ID_UNIFORMRESOURCELOCATOR ) &&
		!rTypeLst.Get( nFmt = SOT_FORMATSTR_ID_FILENAME ) )
		nFmt = 0;

	return nFmt;
}

//-----------------------------------------------------------------------

BOOL INetBookmark::SetData( SvData& rData ) const

/*	[Beschreibung]

	Liefert TRUE, falls in dem SvDataObject ein entsp. Bookmark-Format
	gefunden steht, das wir unterstuetzen. Wenn ja, werden die Daten
	ins Data-Object kopiert.
*/

{
	BOOL bOK = TRUE;
	rtl_TextEncoding eSysCSet = gsl_getSystemTextEncoding();
	switch( rData.GetFormat() )
	{
	case SOT_FORMATSTR_ID_SOLK:
		rData.SetData( CopyExchange() );
		break;

	case FORMAT_STRING:
		rData.SetData( GetURL() );
		break;

	case SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK:
		{
			// dann als Netscape Bookmark bereit stellen
			char aNew[ 2048 ];
			memset( aNew, 0, 2048 );

			ByteString sTmp( GetURL(), eSysCSet );
			strcpy( aNew, sTmp.GetBuffer() );
			sTmp = ByteString( GetDescription(), eSysCSet );
			strcpy( aNew + 1024, sTmp.GetBuffer() );
			rData.SetData( (void*)aNew, 2048 );
		}
		break;

	case SOT_FORMATSTR_ID_UNIFORMRESOURCELOCATOR:
		// dann als UniformResourceLocator bereit stellen
		rData.SetData( GetURL() );
		break;

#ifdef WNT
	case SOT_FORMATSTR_ID_FILEGRPDESCRIPTOR:
		{
			// dann als FileGroupDescriptor bereitstellen
			FILEGROUPDESCRIPTOR aFDesc;
			aFDesc.cItems = 1;
			FILEDESCRIPTOR& rFDesc1 = aFDesc.fgd[0];
			memset( &rFDesc1, 0, sizeof( FILEDESCRIPTOR ) );
			rFDesc1.dwFlags = FD_LINKUI;

			ByteString sTmp( GetDescription(), eSysCSet );
			for( USHORT nChar = 0; nChar < sTmp.Len(); ++nChar )
				if( strchr( "\\/:*?\"<>|", sTmp.GetChar( nChar ) ) )
					sTmp.Erase( nChar--, 1 );

			sTmp.Insert( "Shortcut to ", 0 );
			sTmp += ".URL";

			strcpy( rFDesc1.cFileName, sTmp.GetBuffer() );
			rData.SetData( (void*)&aFDesc, sizeof( FILEGROUPDESCRIPTOR ) );
		}
		break;

	case SOT_FORMATSTR_ID_FILECONTENT:
		{
			String sTmp( RTL_CONSTASCII_STRINGPARAM(
							"[InternetShortcut]\x0aURL=" ));
			sTmp += GetURL();
			rData.SetData( sTmp );
		}
		break;
#endif

	default:
		bOK = FALSE;
	}

	return bOK;
}

//-----------------------------------------------------------------------

BOOL INetBookmark::Copy( SotDataObject& rObj ) const

/*	[Beschreibung]

	Diese Methode kopiert diese Instanz in das Clipboard.
*/

{
	SvDataTypeList& rTypeList = (SvDataTypeList&)rObj.GetTypeList();
	rTypeList.Append( SvDataType( FORMAT_STRING, MEDIUM_MEMORY ) );
	rTypeList.Append( SvDataType( SOT_FORMATSTR_ID_SOLK, MEDIUM_MEMORY ) );
	rTypeList.Append( SvDataType( SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK, MEDIUM_MEMORY ) );
	rTypeList.Append( SvDataType( SOT_FORMATSTR_ID_FILECONTENT, MEDIUM_MEMORY ) );
	rTypeList.Append( SvDataType( SOT_FORMATSTR_ID_FILEGRPDESCRIPTOR, MEDIUM_MEMORY ) );
	rTypeList.Append( SvDataType( SOT_FORMATSTR_ID_UNIFORMRESOURCELOCATOR, MEDIUM_MEMORY ) );
	return TRUE;
}

//-----------------------------------------------------------------------

BOOL INetBookmark::Paste( SotDataObject& rObj, ULONG nFormat )

/*	[Beschreibung]

	Diese Methode "uberschreibt den Wert dieser Instanz mit dem
	Inhalt des DragServers an Position 'nItem'. Es wird vorausgesetzt,
	da\s sich das eigene Format im DragServer befindet.
*/

{
	BOOL bRet = FALSE;

	SvData aData( nFormat, MEDIUM_ALL );
	if( rObj.GetData( &aData ) )
	{
		void *pData;
		aData.GetData( &pData, TRANSFER_REFERENCE );
		UINT32 nLen = aData.GetMinMemorySize();
		rtl_TextEncoding eSysCSet = gsl_getSystemTextEncoding();

		switch( nFormat )
		{
		case SOT_FORMATSTR_ID_SOLK:
			PasteExchange( String( (sal_Char*)pData, eSysCSet ));
			bRet = TRUE;
			break;

		case SOT_FORMATSTR_ID_NETSCAPE_BOOKMARK:
			// JP 05.02.96: zur Zeit definiert als
			//		char[ 1024 ] -> URL
			//		char[ 1024 ] -> Hinweistext
			if( 2048 == nLen )
			{
				SetURL( String( (sal_Char*)pData, eSysCSet ) );
				SetDescription( String( (sal_Char*)pData + 1024, eSysCSet ) );
				bRet = TRUE;
			}
			break;

		case SOT_FORMATSTR_ID_UNIFORMRESOURCELOCATOR:
			{
				String sTmp( (sal_Char*)pData, eSysCSet );
				SetURL( sTmp );
				SetDescription( sTmp );
			}
			break;

#ifdef WNT
		case SOT_FORMATSTR_ID_FILEGRPDESCRIPTOR:
			{
				// dann als FileGroupDescriptor bereitstellen
				FILEGROUPDESCRIPTOR* pFDesc = (FILEGROUPDESCRIPTOR*)pData;
				if( pFDesc->cItems )
				{
					FILEDESCRIPTOR& rFDesc1 = pFDesc->fgd[0];
					// ?? rFDesc1.dwFlags = FD_LINKUI;

					// wir koennen nur URL-Dateien lesen
					ByteString sDesc( rFDesc1.cFileName );
					if( sDesc.Len() > 4 && sDesc.Copy( sDesc.Len() - 4 ).
										EqualsIgnoreCaseAscii( ".URL" ) )
					{
						SvStream* pStream = 0;
                        INetURLObject aEntry( String( sDesc, eSysCSet ) );
                        SvData aFCData( SOT_FORMATSTR_ID_FILECONTENT, MEDIUM_ALL );

                        pStream = new SvFileStream( aEntry.GetMainURL(), STREAM_STD_READ );
                        if( pStream->GetError() )
						{
                            DELETEZ( pStream );
							void* pMem;
							aFCData.GetData( &pMem, TRANSFER_REFERENCE );
							pStream = new SvMemoryStream( (sal_Char*)pMem,
												aFCData.GetMinMemorySize(),
												STREAM_STD_READ );
						}

						if( pStream )
						{
							BOOL bSttFnd = FALSE;
							ByteString sLine;
							while( pStream->ReadLine( sLine ) )
							{
								if( sLine.EqualsIgnoreCaseAscii( "[InternetShortcut]" ))
									bSttFnd = TRUE;
								else if( bSttFnd && sLine.Copy( 0, 4 ).
										EqualsIgnoreCaseAscii( "URL=" ))
								{
									SetURL( String( sLine.Erase(0,4), eSysCSet ));
									sDesc.Erase( sDesc.Len() - 4 );
									SetDescription( String( sDesc, eSysCSet ));
									bRet = TRUE;
									break;
								}
							}

							delete pStream;
						}
					}
				}
			}
			break;
#endif
		}
	}
	return bRet;
}



