/*************************************************************************
 *
 *  $RCSfile: charmap.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:00:55 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SVX_CHARMAP_HXX
#define _SVX_CHARMAP_HXX

// include ---------------------------------------------------------------


#ifndef _BUTTON_HXX //autogen
#include <vcl/button.hxx>
#endif
#ifndef _LSTBOX_HXX //autogen
#include <vcl/lstbox.hxx>
#endif
#ifndef _FIXED_HXX //autogen
#include <vcl/fixed.hxx>
#endif
#ifndef _BASEDLGS_HXX //autogen
#include <sfx2/basedlgs.hxx>
#endif
#ifndef _CTRL_HXX //autogen
#include <vcl/ctrl.hxx>
#endif

// define ----------------------------------------------------------------

#ifdef MAC
#define CHARMAP_MAXLEN  28
#define COLUMN_COUNT    28
#define ROW_COUNT        8
#else
#define CHARMAP_MAXLEN  32
#define COLUMN_COUNT    32
#define ROW_COUNT        7
#endif

// class SvxShowCharSet --------------------------------------------------

class SvxShowCharSet : public Control
{
public:
					SvxShowCharSet( Window* pParent, const ResId& rResId );
					~SvxShowCharSet();

	void            SetFont( const Font& rFont );

	void            SelectCharacter( sal_Unicode cNew, BOOL bFocus = FALSE );
	sal_Unicode     GetSelectCharacter() const { return c; }

	Link            GetDoubleClickHdl() const { return aDoubleClkHdl; }
	void			SetDoubleClickHdl( const Link& rLink )
						{ aDoubleClkHdl = rLink; }
	Link            GetSelectHdl() const { return aSelectHdl; }
	void            SetSelectHdl( const Link& rHdl )
						{ aSelectHdl = rHdl; }
	Link            GetHighlightHdl() const { return aHighHdl; }
	void            SetHighlightHdl( const Link& rHdl )
						{ aHighHdl = rHdl; }
	Link            GetPreSelectHdl() const { return aHighHdl; }
	void            SetPreSelectHdl( const Link& rHdl )
						{ aPreSelectHdl = rHdl; }

protected:
	virtual void    Paint( const Rectangle& );
	virtual void    MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    MouseButtonUp( const MouseEvent& rMEvt );
	virtual void    MouseMove( const MouseEvent& rMEvt );
	virtual void    KeyInput( const KeyEvent& rKEvt );
	virtual void    GetFocus();
	virtual void    LoseFocus();
	virtual void	StateChanged( StateChangedType nStateChange );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

private:
	Link            aDoubleClkHdl;
	Link            aSelectHdl;
	Link            aHighHdl;
	Link			aPreSelectHdl;
	long	        nX;
	long			nY;
	sal_Unicode     c;
	BOOL            bDrag;

	void            DrawChars_Impl( USHORT n1, USHORT n2 );
	void			InitSettings( BOOL bForeground, BOOL bBackground );
};

// class SvxShowText -----------------------------------------------------

class SvxShowText : public Control
{
public:
					SvxShowText( Window* pParent,
								 const ResId& rResId,
								 BOOL bCenter = FALSE );
					~SvxShowText();

	void            SetFont( const Font& rFont );
	void            SetText( const String& rText );

protected:
	virtual void    Paint( const Rectangle& );

private:
	long            nY;
	BOOL            bCenter;

};

// class SvxCharacterMap -------------------------------------------------
/*
{k:\svx\prototyp\dialog\charmap.bmp}
[Beschreibung]
Mit diesem Dialog kann ein oder mehrere Sonderzeichen ausgewaehlt werden
[Items]
*/

class SvxCharacterMap : public SfxModalDialog
{
private:
	SvxShowCharSet	aShowSet;
	FixedText		aFontText;
	ListBox			aFontLB;
	FixedText		aSymbolText;
	SvxShowText		aShowText;
	SvxShowText		aShowChar;
	FixedText		aCharCodeText;
	OKButton		aOKBtn;
	CancelButton	aCancelBtn;
	HelpButton		aHelpBtn;
	PushButton		aDeleteBtn;

	Font			aFont;
	BOOL			bOne;
#if __PRIVATE
	DECL_LINK( OKHdl, OKButton* );
	DECL_LINK( FontSelectHdl, ListBox* );
	DECL_LINK( CharDoubleClickHdl, Control* pControl );
	DECL_LINK( CharSelectHdl, Control* pControl );
	DECL_LINK( CharHighlightHdl, Control* pControl );
	DECL_LINK( CharPreSelectHdl, Control* pControl );
	DECL_LINK( DeleteHdl, PushButton* pBtn );
#endif

public:
					SvxCharacterMap( Window* pParent, BOOL bOne = TRUE );
					~SvxCharacterMap();

	void			SetFont( const Font& rFont ); //! irgendwann entfernen

	void			DisableFontSelection();

	const Font&		GetCharFont() const { return aFont; }
	void			SetCharFont( const Font& rFont );

	void			SetChar( sal_Unicode c ) { aShowSet.SelectCharacter( c ); }
	sal_Unicode		GetChar() const { return aShowSet.GetSelectCharacter(); }

	String			GetCharacters() const { return aShowText.GetText(); }
};


#endif

