/*************************************************************************
 *
 *  $RCSfile: dbbrowse.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:00:55 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SVX_DBBROWSE_HXX
#define _SVX_DBBROWSE_HXX

#ifndef _REF_HXX //autogen
#include <tools/ref.hxx>
#endif

#ifndef _RTTI_HXX //autogen
#include <tools/rtti.hxx>
#endif

#ifndef _WINDOW_HXX //autogen
#include <vcl/window.hxx>
#endif

#ifndef _SV_COMBOBOX_HXX //autogen
#include <vcl/combobox.hxx>
#endif

#ifndef _SV_LSTBOX_HXX //autogen
#include <vcl/lstbox.hxx>
#endif

#ifndef _IMAGEBTN_HXX //autogen
#include <vcl/imagebtn.hxx>
#endif

#ifndef _SVX_BRWBOX_HXX //autogen
#include <svtools/brwbox.hxx>
#endif

#ifndef _SV_TIMER_HXX //autogen
#include <vcl/timer.hxx>
#endif

#ifndef _BRWHEAD_HXX //autogen
#include <svtools/brwhead.hxx>
#endif

#ifndef _SV_SVAPP_HXX //autogen
#include <vcl/svapp.hxx>
#endif

//==================================================================
// DataBaseBrowseFlags (DBBF ;)

#define DBBF_NONE					((sal_Int32)0x0000)
#define DBBF_NOROWPICTURE			((sal_Int32)0x0001)
	// wenn dieses Bit _NICHT_ gesetzt ist, wird bei jedem Zeilenwechsel in der BrowseBox die HandleColumn der entsprechenden
	// Zeile invalidiert, damit dort entsprechend dem Status der Zeile (siehe DbBrowseBox::RowStatus) ein Bildchen
	// gemalt werden kann
	// wenn das Bit gesetzt ist, erfolgt die Invalidierung demzufolge nicht, was Flackern verhindert (falls man gar keine
	// Bildchen braucht)

//==================================================================

class DbCellControllerRef;
class Edit;
class ListBoxFrame;
class ButtonCtrl;
class SpinField;

//==================================================================
class DbCellController : public SvRefBase
{
	friend class DbBrowseBox;

protected:
	Window* pWindow;

public:
	TYPEINFO();

	DbCellController(Window* pW);
	virtual ~DbCellController();			// zerstoert das Fenster

	Window& GetWindow() const {return *((DbCellController*) this)->pWindow;}

	virtual void SetModified();
	virtual void ClearModified() = 0;
	virtual sal_Bool IsModified()  const = 0;

protected:
	virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
	virtual void SetModifyHdl(const Link& rLink) = 0;
	virtual sal_Bool WantMouseEvent() const;
};

SV_DECL_IMPL_REF(DbCellController);

//==================================================================
class DbEditCellController : public DbCellController
{
public:
	TYPEINFO();
	DbEditCellController(Edit* pEdit);
	Edit& GetEditWindow() const {return (Edit &)GetWindow();}

	virtual void SetModified();
	virtual sal_Bool IsModified() const;
	virtual void ClearModified();

protected:
	virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
	virtual void SetModifyHdl(const Link& rLink);
};

//==================================================================
class DbSpinCellController : public DbCellController
{
public:
	TYPEINFO();
	DbSpinCellController(SpinField* pSpinField);
	SpinField& GetSpinWindow() const {return (SpinField &)GetWindow();}

	virtual void SetModified();
	virtual sal_Bool IsModified() const;
	virtual void ClearModified();

protected:
	virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
	virtual void SetModifyHdl(const Link& rLink);
};

//==================================================================
// CheckBox fuer den Browser
class DbCheckBoxCtrl : public Control
{
	CheckBox*	pBox;
	Rectangle	aFocusRect;
	Link		m_aClickLink,m_aModifyLink;

public:
	DbCheckBoxCtrl(Window* pParent, WinBits nWinStyle = 0);
	~DbCheckBoxCtrl();

	virtual void GetFocus();
	virtual long PreNotify(NotifyEvent& rEvt);
	virtual void Paint(const Rectangle& rClientRect);
	virtual void Resize();

	void SetClickHdl(const Link& rHdl) {m_aClickLink = rHdl;}
	const Link& GetClickHdl() const {return m_aClickLink;}

	void SetModifyHdl(const Link& rHdl) {m_aModifyLink = rHdl;}
	const Link& GetModifyHdl() const {return m_aModifyLink;}

	CheckBox&	GetBox() {return *pBox;};

private:
	DECL_LINK( OnClick, void* );
};

//==================================================================
class DbCheckBoxCellController : public DbCellController
{
public:
	TYPEINFO();

	DbCheckBoxCellController(DbCheckBoxCtrl* pWin):DbCellController(pWin){}
	CheckBox& GetCheckBox() const;

	virtual sal_Bool IsModified() const;
	virtual void ClearModified();

protected:
	virtual void SetModifyHdl(const Link& rLink);
	virtual sal_Bool WantMouseEvent() const;
};

//==================================================================
class DbComboBoxCtrl : public ComboBox
{
	friend class DbComboBoxCellController;

public:
	DbComboBoxCtrl(Window* pParent, WinBits nWinStyle = 0);

protected:
	virtual long PreNotify( NotifyEvent& rNEvt );
};

//==================================================================
class DbComboBoxCellController : public DbCellController
{
public:
	DbComboBoxCellController(DbComboBoxCtrl* pParent);
	DbComboBoxCtrl& GetComboBox() const {return (DbComboBoxCtrl &)GetWindow();}

	virtual sal_Bool IsModified() const;
	virtual void ClearModified();

protected:
	virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
	virtual void SetModifyHdl(const Link& rLink);
};

//==================================================================
class DbListBoxCtrl : public ListBox
{
	friend class DbListBoxCellController;

public:
	DbListBoxCtrl(Window* pParent, WinBits nWinStyle = 0);

protected:
	virtual long PreNotify( NotifyEvent& rNEvt );
};

//==================================================================
class DbListBoxCellController : public DbCellController
{
public:
	DbListBoxCellController(DbListBoxCtrl* pParent);
	DbListBoxCtrl& GetListBox() const {return (DbListBoxCtrl &)GetWindow();}

	virtual sal_Bool IsModified() const;
	virtual void ClearModified();

protected:
	virtual sal_Bool MoveAllowed(const KeyEvent& rEvt) const;
	virtual void SetModifyHdl(const Link& rLink);
};


//==================================================================
// DbBrowseHeader
//==================================================================
class DbBrowseHeader : public BrowserHeader
{
public:
	DbBrowseHeader( BrowseBox* pParent, WinBits nWinBits = WB_BUTTONSTYLE )
		:BrowserHeader(pParent, nWinBits){}

protected:
	//virtual void ::com::sun::star::ucb::Command( const CommandEvent& rCEvt );
	virtual void DoubleClick();
};

//==================================================================
class DbBrowseBox: public BrowseBox
{
	friend class DbBrowseHeader;

	enum BrowseInfo
	{
		COLSELECT	=	1,
		ROWSELECT	=	2,
		ROWCHANGE	=	4,
		COLCHANGE	=	8
	};

public:
	enum RowStatus
	{
		CLEAN		=	0,
		CURRENT		=	1,
		CURRENTNEW	=	2,
		MODIFIED	=	3,
		NEW			=	4,
		DELETED		=	5,
		PRIMARYKEY	=	6,
		CURRENT_PRIMARYKEY = 7,
		FILTER		=	8
	};

private:
	class BrowserMouseEventPtr // sich merken eines MEvt, mit down,up siehe impl
	{
		BrowserMouseEvent* pEvent;
		sal_Bool bDown;

	public:
		BrowserMouseEventPtr():pEvent(NULL){}
		~BrowserMouseEventPtr(){Clear();}

		sal_Bool Is() const {return pEvent != NULL;}
		sal_Bool IsDown() const {return bDown;}
		const BrowserMouseEvent* operator->() const {return pEvent;}
		const BrowserMouseEvent& operator*() const {return *pEvent;}

		void Clear();
		void Set(const BrowserMouseEvent* pEvt, sal_Bool bIsDown);
	} aMouseEvent;

#if (SUPD < 517)
	ImageList	aStatusList;
#endif

	const BrowserMouseEvent* pMouseEvent;	// ist waehrend eines MouseEvents gesetzt
	DbCellControllerRef		 aController,
							 aOldController;

	sal_uInt32	nStartEvent, nEndEvent, nCellModifiedEvent;		// EventIDs
#if SUPD>511
	Window*	m_pFocusWhileRequest;
#endif // SUPD>511
		// Im ActivateCell hole ich mir asynchron den Fokus, allerdings nicht, wenn zwischen diesem Request und der tatsaechlichen
		// Ausfuehrung der Fokus schon gewandert ist. Dazu muss ich mit vor dem Request das aktuelle Fokus-Window merken.

	long	nPaintRow;	// Row die gerade gepainted werden soll
	long	nEditRow, nOldEditRow;
	sal_uInt16	nEditCol, nOldEditCol;

	sal_Bool	bHasFocus : 1;
	sal_Bool	bPaintStatus : 1;	// Soll ein Status in der Handlecolumn angezeigt werden oder nicht

	DbCheckBoxCtrl* pCheckBoxPaint;

	sal_Int32	m_nBrowserFlags;

protected:
	BrowserHeader*  pHeader;

	sal_Bool isGetCellFocusPending() const { return nStartEvent != 0; }
	void cancelGetCellFocus() { if (nStartEvent) Application::RemoveUserEvent(nStartEvent); nStartEvent = 0; }
	void forceGetCellFocus() { cancelGetCellFocus(); LINK(this, DbBrowseBox, StartEditHdl).Call((void*)NULL); }

	BrowserMouseEventPtr& getMouseEvent() { return aMouseEvent; }

protected:
	BrowserHeader*	GetHeaderBar() const {return pHeader;}

	virtual BrowserHeader* CreateHeaderBar(BrowseBox* pParent);

	// um header zu erzeugen, immer die Impl methode verwenden!!!
	virtual BrowserHeader* imp_CreateHeaderBar(BrowseBox* pParent);
		// eigentlich haetten wir das gern CreateHeaderBar_Impl genannt .. dann wird aber die Methode nicht exportiert,
		// die Klasse aber wird projekt-uebergreifend benoetigt, also ergibt das dann unresolved externals in den benutzenden
		// Modulen
	virtual void ColumnMoved(sal_uInt16 nId);
	virtual void ColumnResized(sal_uInt16 nColId);
	virtual void Resize();
	virtual void ArrangeControls(sal_uInt16& nX, sal_uInt16 nY);
	virtual sal_Bool SeekRow(long nRow);

	virtual void GetFocus();
	virtual void LoseFocus();
	virtual void KeyInput(const KeyEvent& rEvt);
	virtual void MouseButtonDown(const BrowserMouseEvent& rEvt);
	virtual void MouseButtonUp(const BrowserMouseEvent& rEvt);
	virtual void StateChanged( StateChangedType nType );
	virtual void DataChanged( const DataChangedEvent& rDCEvt );

	virtual long PreNotify(NotifyEvent& rNEvt );
	virtual long Notify(NotifyEvent& rNEvt);

	virtual void EndScroll();

	// Zeichnen des Statusfeldes einer Zeile
	// sollte statt GetFieldRectPixel verwendet werden,
	// da diese Methode die Raender miteinbezieht
	Rectangle GetCellRect(long nRow, sal_uInt16 nColId, sal_Bool bRelToBrowser = sal_True) const;
	virtual sal_uInt32 GetTotalCellWidth(long nRow, sal_uInt16 nColId);
	virtual sal_uInt32 GetAutoColumnWidth(sal_uInt16 nColId);

	virtual void PaintStatusCell(OutputDevice& rDev, const Rectangle& rRect) const;
	virtual void PaintCell(OutputDevice& rDev, const Rectangle& rRect, sal_uInt16 nColId) const = 0;

	virtual RowStatus GetRowStatus(long nRow) const;

	// Zeilenwechsel
	// CursorMoving:	Cursor wird bewegt, GetCurRow() ... liefern aber noch die original Werte
	//					hier sollte alles vorgenommen werden, was das Painten noch beeinflut
	virtual sal_Bool CursorMoving(long nNewRow, sal_uInt16 nNewCol);

	// Cursorbewegung ist durchgefuehrt worden
	virtual void CursorMoved();

	virtual void CellModified(); // sobald eine Zelle modifiziert wird, erfolgt der Aufruf
	virtual sal_Bool SaveModified(); // wird aufgerufen vor einem Zellenwechsel
								 // return sal_False, verhindert Zellenwechsel
	virtual sal_Bool SaveRow();		 // speichern der aktuellen Zeile

	virtual sal_Bool IsModified() const {return aController.Is() && aController->IsModified();}

	virtual DbCellController* GetController(long nRow, sal_uInt16 nCol);
	virtual void InitController(DbCellControllerRef& rController, long nRow, sal_uInt16 nCol);
	virtual void ResizeController(DbCellControllerRef& rController, const Rectangle&);
	virtual void ReleaseController(DbCellControllerRef& pController, long nRow, sal_uInt16 nCol);
	virtual void DoubleClick(const BrowserMouseEvent&);

	void ActivateCell() {ActivateCell(GetCurRow(), GetCurColumnId());}
	// Image fuer einen RowStatus erfahren
	Image GetImage(RowStatus) const;

	// einfuegen von spalten
	// bei width = 0 wird default groee verwendet
	virtual void InsertHandleColumn(sal_uInt16 nWidth = 0);
	// ist die spaltebreite nicht gesetzt, wird diese berechnet
	// if the id isn't set the smallest unused will do it ...
	virtual sal_uInt16 AppendColumn(const String& rName, sal_uInt16 nWidth = 0, sal_uInt16 nPos = HEADERBAR_APPEND, sal_uInt16 nId = (sal_uInt16)-1);

	// darf Tab, shifttab verwendet werden um zur nchsten Zelle zu springen
	virtual sal_Bool IsTabAllowed(sal_Bool bForward) const;

	virtual sal_Bool IsCursorMoveAllowed(long nNewRow, sal_uInt16 nNewColId) const;

	void PaintTristate(OutputDevice& rDev, const Rectangle& rRect,const TriState& eState,sal_Bool _bEnabled=sal_True) const;
	
	void AsynchGetFocus();
		// sicheres Starten von StartEditHdl

public:
	DbBrowseBox(Window* pParent, sal_Int32 nBrowserFlags = DBBF_NONE, WinBits nBits = WB_TABSTOP, BrowserMode nMode = 0 );
	DbBrowseBox(Window* pParent, const ResId& rId, sal_Int32 nBrowserFlags = DBBF_NONE, BrowserMode nMode = 0 );
	~DbBrowseBox();

	sal_Bool IsEditing() const {return aController.Is();}
	void InvalidateStatusCell(long nRow) {RowModified(nRow, 0);}
	void InvalidateHandleColumn();

	virtual sal_uInt16 GetDefaultColumnWidth(const String& rName) const;

	// spaetes Construieren
	virtual void Init();
	virtual void RemoveRows();
	virtual void Dispatch(sal_uInt16 nId);

	DbCellControllerRef Controller() const { return aController; }
	sal_Int32	GetBrowserFlags() const { return m_nBrowserFlags; }
	void	SetBrowserFlags(sal_Int32 nFlags);

	virtual void ActivateCell(long nRow, sal_uInt16	nCol, sal_Bool bSetCellFocus = sal_True);
	virtual void DeactivateCell(sal_Bool bUpdate = sal_True);

private:
	virtual void PaintField(OutputDevice& rDev, const Rectangle& rRect,
							sal_uInt16 nColumnId ) const;
	void ImplInitSettings( sal_Bool bFont, sal_Bool bForeground, sal_Bool bBackground );
	void DetermineFocus();
	inline void HideAndDisable(DbCellControllerRef& rController);
	inline void EnableAndShow() const;

	DECL_LINK(ModifyHdl, void* );
	DECL_LINK(StartEditHdl, void* );
	DECL_LINK(EndEditHdl, void* );
	DECL_LINK(CellModifiedHdl, void* );
};


#endif // _SVX_DBBROWSE_HXX

