/*************************************************************************
 *
 *  $RCSfile: scene3d.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:00:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _E3D_SCENE3D_HXX
#define _E3D_SCENE3D_HXX

#ifndef _CAMERA3D_HXX
#include "camera3d.hxx"
#endif

#ifndef _E3D_LIGHT3D_HXX
#include "light3d.hxx"
#endif

#ifndef _E3D_LABEL3D_HXX
#include "label3d.hxx"
#endif

#ifndef _B3D_BASE3D_HXX
#include <goodies/base3d.hxx>
#endif

#ifndef _B3D_B3DTRANS_HXX
#include <goodies/b3dtrans.hxx>
#endif

#ifndef _TOOLS_TIME_HXX
#include <tools/time.hxx>
#endif

//************************************************************
//   Defines
//************************************************************

#define E3D_SORT_NO_SORTING                 0x00000001
#define E3D_SORT_FAST_SORTING               0x00000002
#define E3D_SORT_IN_PARENTS                 0x00000004
#define E3D_SORT_NON_POLYOBJ                0x00000008
#define E3D_SORT_LOOKUP_FIELD               0x00000010
#define E3D_SORT_TEST_LENGTH                0x00000020

/*************************************************************************
|*
|* GeoData relevant fuer Undo-Actions
|*
\************************************************************************/

class E3DSceneGeoData : public E3DObjGeoData
{
public:
	Camera3D					aCamera;
	E3dLabelList				aLabelList;

	E3DSceneGeoData() {}
};

/*************************************************************************
|*
|* Basisklasse fuer 3D-Szenen
|*
\************************************************************************/

class E3dScene : public E3dObject
{
protected:
	// Transformationen
	B3dCamera					aCameraSet;

	// Beleuchtung
	B3dLightGroup				aLightGroup;

	Camera3D					aCamera;
	E3dLabelList				aLabelList;

	long						nSaveStatus;
	long						nRestStatus;

	UINT32						nSortingMode;

	// Richtung der Schattenflaeche
	Vector3D					aShadowPlaneDirection;

	// Darstellungsmodus
	Base3DShadeModel			eShadeModel;

	// Zeit, die der letzte Paint() benoetigte
	Time						aPaintTime;

	// Darstellungsqualitaet, wird in Base3D verwendet
	UINT8						nDisplayQuality;

	// BOOLean Flags
	unsigned					bDoubleBuffered			: 1;
	unsigned					bClipping				: 1;
	unsigned					bFitInSnapRect			: 1;
	unsigned					bDither					: 1;

	// Darstellungsmodus auf Entwurfsmodus (Draft) zwingen
	unsigned					bForceDraftShadeModel	: 1;

	// Merker, ob die Szene urspruenglich selektiert war
	unsigned					bWasSelectedWhenCopy	: 1;

	// Flag to determine if only selected objects should be drawn
	unsigned					bDrawOnlySelected		: 1;

	virtual void NewObjectInserted(const E3dObject* p3DObj);
	virtual void StructureChanged(const E3dObject* p3DObj);

	void RebuildLists();
	virtual void ForceDefaultAttr(SfxItemPool* pPool);

	virtual void SFX_NOTIFY(SfxBroadcaster &rBC,
							const TypeId   &rBCType,
							const SfxHint  &rHint,
							const TypeId   &rHintType);

	BOOL AreThereTransparentParts() const;

protected:
	void SetDefaultAttributes(E3dDefaultAttributes& rDefault);

public:
	TYPEINFO();
	E3dScene();
	E3dScene(E3dDefaultAttributes& rDefault);
	virtual ~E3dScene();

	// set flag to draw only selected
	void SetDrawOnlySelected(BOOL bNew) { bDrawOnlySelected = bNew; }
	BOOL DoDrawOnlySelected() const { return bDrawOnlySelected; }

	virtual Volume3D FitInSnapRect();

	virtual UINT16 GetObjIdentifier() const;

	virtual USHORT	GetHdlCount() const;
	virtual void	AddToHdlList(SdrHdlList& rHdlList) const;
	virtual FASTBOOL HasSpecialDrag() const;

	virtual void	NbcSetSnapRect(const Rectangle& rRect);
	virtual void	NbcMove(const Size& rSize);
	virtual void	NbcResize(const Point& rRef, const Fraction& rXFact,
												 const Fraction& rYFact);
	virtual void	RecalcSnapRect();

	// Paint MUSS ueberladen werden
	virtual FASTBOOL Paint(ExtOutputDevice&, const SdrPaintInfoRec&) const = 0;

	virtual E3dScene* GetScene() const;

	// TransformationSet vorbereiten
	void InitTransformationSet();

	Time GetLastPaintTime() { return aPaintTime; }

	// Darstellungsqualitaet
	void SetDisplayQuality(UINT8 nNew) { nDisplayQuality = nNew; }
	UINT8 GetDisplayQuality() { return nDisplayQuality; }

	void SetCamera(const Camera3D& rNewCamera);
	const Camera3D& GetCamera() const { return aCamera; }

	void SetDoubleBuffered(FASTBOOL bBuff = TRUE);
	FASTBOOL IsDoubleBuffered() const { return bDoubleBuffered; }

	// Clipping auf umschliessendes Rechteck der Szene
	// (Double Buffering ist immer geclipt!)
	void SetClipping(FASTBOOL bClip = TRUE);
	FASTBOOL IsClipping() const { return bClipping; }

	void SetFitInSnapRect(FASTBOOL bFit = TRUE);
	void CorrectSceneDimensions();
	FASTBOOL IsFitInSnapRect() const { return bFitInSnapRect; }

	void FitSnapRectToBoundVol();
	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);

	virtual void operator=(const SdrObject&);

	virtual SdrObjGeoData *NewGeoData() const;
	virtual void          SaveGeoData(SdrObjGeoData& rGeo) const;
	virtual void          RestGeoData(const SdrObjGeoData& rGeo);

	// Attribute setzen/lesen
	virtual void NbcSetAttributes(const SfxItemSet& rAttr, FASTBOOL bReplaceAll);
	virtual void TakeAttributes(SfxItemSet& rAttr, FASTBOOL bMerge, FASTBOOL bOnlyHardAttr) const;
	virtual void NbcSetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);
	virtual SfxStyleSheet* GetStyleSheet() const;

	virtual void NbcRotate(const Point& rRef, long nWink, double sn, double cs);
	void RotateScene(const Point& rRef, long nWink, double sn, double cs);

	// TakeObjName...() ist fuer die Anzeige in der UI, z.B. "3 Rahmen selektiert".
	virtual void TakeObjNameSingul(String& rName) const;
	virtual void TakeObjNamePlural(String& rName) const;

	void   SetSortingMode(UINT32 nMode) {nSortingMode = nMode;}
	UINT32 GetSortingMode() {return nSortingMode;}

	// Projektionsflaeche 3D Schatten holen/setzen
	const Vector3D& GetShadowPlaneDirection() { return aShadowPlaneDirection; }
	void SetShadowPlaneDirection(const Vector3D& rNew);

	// Transformationen rausgeben
	B3dCamera& GetCameraSet() { return aCameraSet; }

	// Beleuchtung rausgeben
	B3dLightGroup& GetLightGroup() { return aLightGroup; }

	// LightGroup aus geladenen PointObj's fuellen
	void FillLightGroup();

	// Licht-Objekte rauswerfen
	void RemoveLightObjects();

	// Licht-Objekte erzeugen, um kompatibel zur 4.0
	// speichern zu koennen
	void CreateLightObjectsFromLightGroup();

	// Lichter zaehlen
	UINT16 CountNumberOfLights();

	// Dithering
	BOOL GetDither() { return bDither; }
	void SetDither(BOOL bNew) { bDither = bNew; }

	// ShadeModel
	Base3DShadeModel GetShadeModel() { return eShadeModel; }
	void SetShadeModel(Base3DShadeModel eNew);
	BOOL GetForceDraftShadeModel() { return bForceDraftShadeModel; }
	void SetForceDraftShadeModel(BOOL bNew);

	// Marker fuer WasSelectedWhenCopy
	BOOL GetWasSelectedWhenCopy() { return bWasSelectedWhenCopy; }
	void SetWasSelectedWhenCopy(BOOL bNew) { bWasSelectedWhenCopy = bNew; }

	// Aufbrechen
	virtual BOOL IsBreakObjPossible();

	// ItemPool fuer dieses Objekt wechseln
	virtual void MigrateItemPool(SfxItemPool* pSrcPool, SfxItemPool* pDestPool);
};



#endif			// _E3D_SCENE3D_HXX
