/*************************************************************************
 *
 *  $RCSfile: svdoattr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:00 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SVDOATTR_HXX
#define _SVDOATTR_HXX

#ifndef SVX_XFILLIT0_HXX //autogen
#include "xfillit0.hxx"
#endif

#ifndef _SVX_XFLASIT_HXX //autogen
#include "xflasit.hxx"
#endif

#ifndef _SVX_XLINEIT0_HXX //autogen
#include "xlineit0.hxx"
#endif

#ifndef _SVX_XLNASIT_HXX //autogen
#include "xlnasit.hxx"
#endif

#ifndef _SVDOBJ_HXX
#include "svdobj.hxx"
#endif

#ifndef _SVDATTR_HXX
#include "svdattr.hxx"
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Folgende Methoden muessen ueberladen werden, wenn ein abgeleitetes Objekt
//    eigene Attribute besitzt
//    (siehe auch Legendenobjekt: SdrCaptObj, SvdoCapt.?XX)
//    - Diese Methoden in jedem Falle:
//        virtual USHORT            GetSetItemCount() const;
//        virtual const SfxSetItem* GetSetItem(USHORT nNum) const;
//        virtual void              SetSetItem(USHORT nNum, const SfxSetItem* pAttr);
//        virtual SfxSetItem*       MakeNewSetItem(USHORT nNum, FASTBOOL bClone) const;
//      - virtual void              ForceDefaultAttr(SfxItemPool* pPool);
//    - Diese Methoden, wenn die Attribute vorlagenfaehig sein sollen:
// na die beiden   x     - virtual void RemoveStyleSheet();
// wohl doch nicht x     - virtual void AddStyleSheet(SfxStyleSheet* pNewStyleSheet);
//    - und diese Methoden sowieso:
//      - virtual void operator=(const SdrObject& rObj);
//      - virtual void WriteData(SvStream& rOut) const;
//      - virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);
//      - virtual void SetModel(SdrModel* pNewModel);
//    Versteht sich von selbst, dass jeweils die Methoden der Basisklasse gerufen
//    werden muessen.
//
////////////////////////////////////////////////////////////////////////////////////////////////////

//************************************************************
//   Vorausdeklarationen
//************************************************************

class SfxPoolItem;
class SfxSetItem;
class SdrOutliner;

//************************************************************
//   SdrAttrObj
//
// DrawObjekt mit Attributen (Farbe,...)
//
//************************************************************

class SdrAttrObj : public SdrObject
{
	friend class				SdrOutliner;

protected:
	Rectangle					aSnapRect;
	const XLineAttrSetItem*		pLineAttr;
	const XFillAttrSetItem*		pFillAttr;
	const XTextAttrSetItem*		pTextAttr;
	const SdrShadowSetItem*		pShadAttr;
	const SdrOutlinerSetItem*	pOutlAttr;
	const SdrMiscSetItem*		pMiscAttr;
	SfxStyleSheet*				pStyleSheet;

protected:
	// Holt den ItemPool von Model. Falls kein Model da, wird versucht ein Pool von
	// einem der Items zu holen.
	SfxItemPool* ImpGetItemPool(const SfxPoolItem* pAlternateItem1, const SfxPoolItem* pAlternateItem2) const;

	// Holt den ItemPool von Model. Falls kein Model da, wird versucht ein Pool von
	// einem der Items des Objs zu holen.
	SfxItemPool* ImpGetItemPool() const;

	// Strichstaerke ermitteln. Keine Linie -> 0.
	INT32 ImpGetLineWdt() const;

	// Feststellen, wieviel wegen der Linienenden auf's BoundRect draufaddiert werden muss.
	INT32 ImpGetLineEndAdd() const;

	// HitTest auf's Linienende
	// ueber nSin/nCos wird der Winkel vorgegeben
	FASTBOOL ImpLineEndHitTest(const Point& rEndPt, double nSin, double nCos, FASTBOOL bStart, const Point& rHit, USHORT nTol) const;

	// Schattenabstand ermitteln. FALSE=Kein Schatten.
	FASTBOOL ImpGetShadowDist(INT32& nXDist, INT32& nYDist) const;

	// ggf. Schattenversatz zum BoundRect draufaddieren
	void ImpAddShadowToBoundRect();

	// Line und Fill Attribute fuer Schatten setzen.
	// Return=FALSE: kein Schatten attributiert.
	FASTBOOL ImpSetShadowAttributes(ExtOutputDevice& rXOut, FASTBOOL bNoFill=FALSE) const;
	const SfxPoolItem* ImpSetNewAttr(const SfxPoolItem* pAkt, const SfxPoolItem* pNew, FASTBOOL bChg=TRUE);

	// Zuhoeren, ob sich ein StyleSheet aendert
	virtual void SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType, const SfxHint& rHint, const TypeId& rHintType);
	virtual void RemoveStyleSheet();
	virtual void AddStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);

	// aus NULL-AttrPointern Pointer auf defaults machen
	virtual void ForceDefaultAttr(SfxItemPool* pPool);
	virtual USHORT GetSetItemCount() const;
	virtual const SfxSetItem* GetSetItem(USHORT nNum) const;
	virtual void SetSetItem(USHORT nNum, const SfxSetItem* pAttr);
	virtual SfxSetItem* MakeNewSetItem(USHORT nNum, FASTBOOL bClone) const;

	SdrAttrObj();
	virtual ~SdrAttrObj();

public:
	TYPEINFO();

	// Feststellen, ob bFilledObj && Fuellung!=FillNone
	inline FASTBOOL HasFill() const;

	// Feststellen, ob Linie!=LineNone
	inline FASTBOOL HasLine() const;

	virtual const Rectangle& GetSnapRect() const;
	virtual void operator=(const SdrObject& rObj);
	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);
	virtual void SetModel(SdrModel* pNewModel);

	virtual void TakeAttributes(SfxItemSet& rAttr, FASTBOOL bMerge, FASTBOOL bOnlyHardAttr) const;
	virtual void NbcSetAttributes(const SfxItemSet& rAttr, FASTBOOL bReplaceAll);
	virtual void SetAttributes(const SfxItemSet& rAttr, FASTBOOL bReplaceAll);
	virtual SfxStyleSheet* GetStyleSheet() const;
	virtual void SetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);
	virtual void NbcSetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);

	virtual void BurnInStyleSheetAttributes();

	// Attribute und StyleSheet einem Objekt zuweisen
	void CopyAttributes(SdrObject* pDestObj) const;

	// ItemPool fuer dieses Objekt wechseln
	virtual void MigrateItemPool(SfxItemPool* pSrcPool, SfxItemPool* pDestPool);

	// temporarly unless the SdrGrafObj and SdrOle2Obj supports the fill and line attributes
	virtual const XLineAttrSetItem* GetLineAttrSetItem() const { return pLineAttr; }
	virtual const XFillAttrSetItem* GetFillAttrSetItem() const { return pFillAttr; }
};

//************************************************************
//   Inlines
//************************************************************

inline FASTBOOL SdrAttrObj::HasFill() const
{
	return (!bClosedObj) ? FALSE
		: (pLineAttr==NULL) ? TRUE // sonst default ist TRUE
		: ((XFillStyleItem&)(pFillAttr->GetItemSet().Get(XATTR_FILLSTYLE))).GetValue()!=XFILL_NONE;
}

inline FASTBOOL SdrAttrObj::HasLine() const
{
	return (pLineAttr==NULL) ? TRUE // Default ist TRUE
		: ((XLineStyleItem&)(pLineAttr->GetItemSet().Get(XATTR_LINESTYLE))).GetValue()!=XLINE_NONE;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif //_SVDOATTR_HXX

