/*************************************************************************
 *
 *  $RCSfile: tp3dattr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <tools/ref.hxx>
#ifndef _SHL_HXX //autogen
#include <tools/shl.hxx>
#endif
#ifndef _SFXENUMITEM_HXX //autogen
#include <svtools/eitem.hxx>
#endif
#ifndef _SFXAPP_HXX
#include <sfx2/app.hxx>
#endif
#ifndef _SFXMODULE_HXX
#include <sfx2/module.hxx>
#endif
#ifndef _SFX_SAVEOPT_HXX //autogen
#include <sfx2/saveopt.hxx>
#endif
#pragma hdrstop

#include "dlgutil.hxx"
#include "dialmgr.hxx"
#include "dialogs.hrc"

#include "tp3dattr.hrc"
#include "tp3dattr.hxx"

// static ----------------------------------------------------------------

static USHORT p3DAttrRanges[] =
{
	SID_ATTR_3D_DEPTH,
	SID_ATTR_3D_INTERN,
	SID_ATTR_3D_DOUBLE_SIDED,
	SID_ATTR_3D_DOUBLE_SIDED,
	SID_ATTR_3D_HORZ_SEGS,
	SID_ATTR_3D_VERT_SEGS,
	0
};

#define MAX_DISTANCE 10
static ULONG aDistanceArray[ MAX_DISTANCE ] =
{
	1000,
	2000,
	5000,
	10000,
	20000,
	50000,
	100000,
	200000,
	500000,
	1000000
};

static USHORT aBmpDistance[ MAX_DISTANCE ] =
{
	RID_SVXBMP_3D_DISTANCE_01,
	RID_SVXBMP_3D_DISTANCE_02,
	RID_SVXBMP_3D_DISTANCE_03,
	RID_SVXBMP_3D_DISTANCE_04,
	RID_SVXBMP_3D_DISTANCE_05,
	RID_SVXBMP_3D_DISTANCE_06,
	RID_SVXBMP_3D_DISTANCE_07,
	RID_SVXBMP_3D_DISTANCE_08,
	RID_SVXBMP_3D_DISTANCE_09,
	RID_SVXBMP_3D_DISTANCE_10
};

#define MAX_FOCAL_LENGTH 10
static ULONG aFocalLengthArray[ MAX_FOCAL_LENGTH ] =
{
	1000,
	2000,
	2400,
	2800,
	3500,
	5000,
	7500,
	10000,
	25000,
	50000
};

static USHORT aBmpFocalLength[ MAX_FOCAL_LENGTH ] =
{
	RID_SVXBMP_3D_FOCAL_LENGTH_01,
	RID_SVXBMP_3D_FOCAL_LENGTH_02,
	RID_SVXBMP_3D_FOCAL_LENGTH_03,
	RID_SVXBMP_3D_FOCAL_LENGTH_04,
	RID_SVXBMP_3D_FOCAL_LENGTH_05,
	RID_SVXBMP_3D_FOCAL_LENGTH_06,
	RID_SVXBMP_3D_FOCAL_LENGTH_07,
	RID_SVXBMP_3D_FOCAL_LENGTH_08,
	RID_SVXBMP_3D_FOCAL_LENGTH_09,
	RID_SVXBMP_3D_FOCAL_LENGTH_10
};

/*************************************************************************
|*
|* 3D-Attribute-TabPage
|*
\************************************************************************/

Svx3DAttrTabPage::Svx3DAttrTabPage( Window* pParent, const SfxItemSet& rInAttrs  ) :
	SfxTabPage		( pParent, ResId( RID_SVXPAGE_3DATTR, DIALOG_MGR() ), rInAttrs ),

	aFtSegments			( this, ResId( FT_SEGMENTS ) ),
	aNumFldHorzSegs		( this, ResId( NUM_FLD_HORZ_SEGS ) ),
	aNumFldVertSegs		( this, ResId( NUM_FLD_VERT_SEGS ) ),
	aFtDepth			( this, ResId( FT_DEPTH ) ),
	aMtrFldDepth		( this, ResId( MTR_FLD_DEPTH ) ),
	aTsbDoubleSided		( this, ResId( TSB_DOUBLE_SIDED ) ),
	aGrpObj				( this, ResId( GRP_OBJ ) ),

	aCbxAutoMode		( this, ResId( CBX_AUTO_MODE ) ),

	aFtDistance			( this, ResId( FT_DISTANCE ) ),
	aMtrFldDistance		( this, ResId( MTR_FLD_DISTANCE ) ),
	aCtlDistance		( this, ResId( CTL_DISTANCE ) ),
	aGrpDistance		( this, ResId( GRP_DISTANCE ) ),

	aFtFocalLength		( this, ResId( FT_FOCAL_LENGTH ) ),
	aMtrFldFocalLength	( this, ResId( MTR_FLD_FOCAL_LENGTH ) ),
	aCtlFocalLength		( this, ResId( CTL_FOCAL_LENGTH ) ),
	aGrpFocalLength 	( this, ResId( GRP_FOCAL_LENGTH ) ),

	rOutAttrs			( rInAttrs ),

	bHorzSegsChanged	( FALSE ),
	bVertSegsChanged	( FALSE )
{
	FreeResource();

	// Metrik einstellen
	FieldUnit eFUnit;
	GET_MODULE_FIELDUNIT( eFUnit );
	//SetFieldUnit( aMtrFldDepth, eFUnit );
	aMtrFldDepth.SetUnit( eFUnit );
	//SetFieldUnit( aMtrFldDistance, eFUnit );
	aMtrFldDistance.SetUnit( eFUnit );
	//SetFieldUnit( aMtrFldFocalLength, eFUnit );
	aMtrFldFocalLength.SetUnit( eFUnit );

	SfxItemPool* pPool = rOutAttrs.GetPool();
	DBG_ASSERT( pPool, "Wo ist der Pool?" );
	ePoolUnit = pPool->GetMetric( SID_ATTR_LINE_WIDTH );

	aMtrFldDistance.SetModifyHdl( LINK( this, Svx3DAttrTabPage,
					ModifyDistanceHdl ) );
	aMtrFldFocalLength.SetModifyHdl( LINK( this, Svx3DAttrTabPage,
					ModifyFocalLengthHdl ) );

	aCtlDistance.SetSelectHdl( LINK( this, Svx3DAttrTabPage,
					SelectDistanceHdl ) );
	aCtlFocalLength.SetSelectHdl( LINK( this, Svx3DAttrTabPage,
					SelectFocalLengthHdl ) );

	// Link fuer Segmentaenderung ...
	aNumFldHorzSegs.SetModifyHdl( LINK( this, Svx3DAttrTabPage,
					SegmentsChangedHdl ) );
	aNumFldVertSegs.SetModifyHdl( LINK( this, Svx3DAttrTabPage,
					SegmentsChangedHdl ) );


	FillValueSet( aCtlDistance );
	FillValueSet( aCtlFocalLength );
}

// -----------------------------------------------------------------------

BOOL Svx3DAttrTabPage::FillItemSet( SfxItemSet& rAttrs )
{
	BOOL	bModified = FALSE;
	UINT32	nValue;

	// Anzahl Segmente (horz.)
	String aStr = aNumFldHorzSegs.GetText();
	if( aStr != aNumFldHorzSegs.GetSavedValue() )
	{
		nValue = aNumFldHorzSegs.GetValue();
		rAttrs.Put( SfxUInt32Item( SID_ATTR_3D_HORZ_SEGS, nValue ) );
		bModified = TRUE;
	}

	// Anzahl Segmente (vert.)
	aStr = aNumFldVertSegs.GetText();
	if( aStr != aNumFldVertSegs.GetSavedValue() )
	{
		nValue = aNumFldVertSegs.GetValue();
		rAttrs.Put( SfxUInt32Item( SID_ATTR_3D_VERT_SEGS, nValue ) );
		bModified = TRUE;
	}

	// Tiefe
	aStr = aMtrFldDepth.GetText();
	if( aStr != aMtrFldDepth.GetSavedValue() )
	{
		nValue = GetCoreValue( aMtrFldDepth, ePoolUnit );
		rAttrs.Put( SfxUInt32Item( SID_ATTR_3D_DEPTH, nValue ) );
		bModified = TRUE;
	}

	// Doppelseitig
	TriState eState = aTsbDoubleSided.GetState();
	if( eState != aTsbDoubleSided.GetSavedValue() )
	{
		BOOL bValue = STATE_CHECK == eState;
		rAttrs.Put( SfxBoolItem( SID_ATTR_3D_DOUBLE_SIDED, bValue ) );
		bModified = TRUE;
	}

	// Kameraabstand
	aStr = aMtrFldDistance.GetText();
	if( aStr != aMtrFldDistance.GetSavedValue() )
	{
		nValue = GetCoreValue( aMtrFldDistance, ePoolUnit );
		rAttrs.Put( SfxUInt32Item( SID_ATTR_3D_DISTANCE, nValue ) );
		bModified = TRUE;
	}

	// Brennweite
	aStr = aMtrFldFocalLength.GetText();
	if( aStr != aMtrFldFocalLength.GetSavedValue() )
	{
		nValue = GetCoreValue( aMtrFldFocalLength, ePoolUnit );
		rAttrs.Put( SfxUInt32Item( SID_ATTR_3D_FOCAL_LENGTH, nValue ) );
		bModified = TRUE;
	}

	return( bModified );
}

// -----------------------------------------------------------------------

void Svx3DAttrTabPage::Reset( const SfxItemSet& rAttrs )
{
	// Anzahl Segmente (horz.)
	const SfxPoolItem* pItem = GetItem( rAttrs, SID_ATTR_3D_HORZ_SEGS );
	if( pItem )
		aNumFldHorzSegs.SetValue( ( ( const SfxUInt32Item* )pItem )->GetValue() );
	else
		aNumFldHorzSegs.SetText( String() );

	// Anzahl Segmente (vert.)
	pItem = GetItem( rAttrs, SID_ATTR_3D_VERT_SEGS );
	if( pItem )
		aNumFldVertSegs.SetValue( ( ( const SfxUInt32Item* )pItem )->GetValue() );
	else
		aNumFldVertSegs.SetText( String() );

	// Tiefe
	pItem = GetItem( rAttrs, SID_ATTR_3D_DEPTH );
	if ( pItem )
	{
		ULONG nValue = ( ( const SfxUInt32Item* )pItem )->GetValue();
		SetMetricValue( aMtrFldDepth, nValue, ePoolUnit );
	}
	else
#ifndef VCL
		aMtrFldDepth.SetEmptyValue();
#else
		aMtrFldDepth.SetEmptyFieldValue();
#endif

	// Doppelwandig/-seitig
	pItem = GetItem( rAttrs, SID_ATTR_3D_DOUBLE_SIDED );
	if( pItem )
	{
		aTsbDoubleSided.SetState( ( ( const SfxBoolItem* )pItem )->GetValue() ?
										STATE_CHECK : STATE_NOCHECK );
		aTsbDoubleSided.EnableTriState( FALSE );
	}
	else
		aTsbDoubleSided.SetState( STATE_DONTKNOW );


	// Kameraabstand
	pItem = GetItem( rAttrs, SID_ATTR_3D_DISTANCE );
	if ( pItem )
	{
		ULONG nValue = ( ( const SfxUInt32Item* )pItem )->GetValue();
		SetMetricValue( aMtrFldDistance, nValue, ePoolUnit );
		aOldDistance = GetCoreValue( aMtrFldDistance, ePoolUnit );
	}
	else
	{
#ifndef VCL
		aMtrFldDistance.SetEmptyValue();
#else
		aMtrFldDistance.SetEmptyFieldValue();
#endif
		aOldDistance = 0L;
	}

	// Brennweite
	pItem = GetItem( rAttrs, SID_ATTR_3D_FOCAL_LENGTH );
	if ( pItem )
	{
		ULONG nValue = ( ( const SfxUInt32Item* )pItem )->GetValue();
		SetMetricValue( aMtrFldFocalLength, nValue, ePoolUnit );
		aOldFocalLength = GetCoreValue( aMtrFldFocalLength, ePoolUnit );
	}
	else
	{
		aMtrFldFocalLength.SetText( String() );
		aOldFocalLength = 0L;
	}

	aNumFldHorzSegs.SaveValue();
	aNumFldVertSegs.SaveValue();
	aMtrFldDepth.SaveValue();
	aTsbDoubleSided.SaveValue();
	aMtrFldDistance.SaveValue();
	aMtrFldFocalLength.SaveValue();

	pItem = GetItem (rAttrs, SID_ATTR_3D_INTERN);
	if (pItem && ((const SfxUInt32Item *) pItem)->GetValue() & 1)
	{
		aFtSegments.Enable( FALSE );
		aNumFldHorzSegs.Enable( FALSE );
		aNumFldVertSegs.Enable( FALSE );
	}

	if (pItem && ((const SfxUInt32Item *) pItem)->GetValue() & 2)
	{
		aFtDepth.Enable( FALSE );
		aMtrFldDepth.Enable( FALSE );
	}

	ModifyDistanceHdl( NULL );
	ModifyFocalLengthHdl( NULL );

	// Jetzt wird der Wert aus der INI-Datei geholt (#42258#)
	String aStr = GetUserData();
	aCbxAutoMode.Check( (BOOL)aStr.ToInt32() );
}

// -----------------------------------------------------------------------

SfxTabPage* Svx3DAttrTabPage::Create( Window* pWindow,
				const SfxItemSet& rOutAttrs )
{
	return( new Svx3DAttrTabPage( pWindow, rOutAttrs ) );
}

//------------------------------------------------------------------------

USHORT*	Svx3DAttrTabPage::GetRanges()
{
	return( p3DAttrRanges );
}

/*************************************************************************
|*
|* Setzt Attribute und fuellt das ValueSet mit Bitmaps
|*
\************************************************************************/

void Svx3DAttrTabPage::FillValueSet( ValueSet& rVs )
{
	rVs.SetStyle( rVs.GetStyle() | WB_ITEMBORDER );
	rVs.SetColCount( 5 );
	rVs.SetLineCount( 2 );
	rVs.SetExtraSpacing( 1 );

	// String sollte noch formatiert werden
	// (ueber MetricField mit aktuell gewaehlter Metric)
	if( &rVs == &aCtlDistance )
	{
		for( USHORT i = 0; i < MAX_DISTANCE; i++ )
		{
			rVs.InsertItem( i + 1,
					Bitmap( ResId( aBmpDistance[ i ], DIALOG_MGR() ) ),
					UniString::CreateFromInt32( aDistanceArray[ i ] ) );
		}
	}
	else if( &rVs == &aCtlFocalLength )
	{
		for( USHORT i = 0; i < MAX_FOCAL_LENGTH; i++ )
		{
			rVs.InsertItem( i + 1,
					Bitmap( ResId( aBmpFocalLength[ i ], DIALOG_MGR() ) ),
					UniString::CreateFromInt32( aFocalLengthArray[ i ] ) );
		}
	}

	rVs.Show();
}

//------------------------------------------------------------------------

IMPL_LINK( Svx3DAttrTabPage, SelectDistanceHdl, void *, EMPTYARG )
{
	short nId = aCtlDistance.GetSelectItemId();

	SetMetricValue( aMtrFldDistance, aDistanceArray[ nId - 1 ], ePoolUnit );

	if( aCbxAutoMode.IsChecked() && aOldDistance != 0 )
	{
		long nValue = GetCoreValue( aMtrFldFocalLength, ePoolUnit ) *
					  GetCoreValue( aMtrFldDistance, ePoolUnit ) /
					  aOldDistance;

		SetMetricValue( aMtrFldFocalLength, nValue, ePoolUnit );
	}
	aOldDistance = GetCoreValue( aMtrFldDistance, ePoolUnit );

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( Svx3DAttrTabPage, SelectFocalLengthHdl, void *, EMPTYARG )
{
	short nId = aCtlFocalLength.GetSelectItemId();

	SetMetricValue( aMtrFldFocalLength, aFocalLengthArray[ nId - 1 ], ePoolUnit );

	if( aCbxAutoMode.IsChecked() && aOldFocalLength != 0 )
	{
		long nValue = GetCoreValue( aMtrFldDistance, ePoolUnit ) *
					  GetCoreValue( aMtrFldFocalLength, ePoolUnit ) /
					  aOldFocalLength;

		SetMetricValue( aMtrFldDistance, nValue, ePoolUnit );
	}
	aOldFocalLength = GetCoreValue( aMtrFldFocalLength, ePoolUnit );

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( Svx3DAttrTabPage, ModifyDistanceHdl, void *, EMPTYARG )
{
	BOOL bFound = FALSE;
	long nValue = GetCoreValue( aMtrFldDistance, ePoolUnit );

	for( USHORT i = 0; i < MAX_DISTANCE && !bFound; i++ )
	{
		if( aDistanceArray[ i ] == (ULONG)nValue )
		{
			bFound = TRUE;
			aCtlDistance.SelectItem( i + 1 );
		}
	}
	if( !bFound )
		aCtlDistance.SetNoSelection();

	if( aCbxAutoMode.IsChecked() && aOldDistance != 0 )
	{
		long nValue = GetCoreValue( aMtrFldFocalLength, ePoolUnit ) *
					  GetCoreValue( aMtrFldDistance, ePoolUnit ) /
					  aOldDistance;

		SetMetricValue( aMtrFldFocalLength, nValue, ePoolUnit );
	}
	aOldDistance = GetCoreValue( aMtrFldDistance, ePoolUnit );

	return( 0L );
}

//------------------------------------------------------------------------

IMPL_LINK( Svx3DAttrTabPage, ModifyFocalLengthHdl, void *, EMPTYARG )
{
	BOOL bFound = FALSE;
	long nValue = GetCoreValue( aMtrFldFocalLength, ePoolUnit );

	for( USHORT i = 0; i < MAX_FOCAL_LENGTH && !bFound; i++ )
	{
		if( aFocalLengthArray[ i ] == (ULONG)nValue )
		{
			bFound = TRUE;
			aCtlFocalLength.SelectItem( i + 1 );
		}
	}
	if( !bFound )
		aCtlFocalLength.SetNoSelection();

	if( aCbxAutoMode.IsChecked() && aOldFocalLength != 0 )
	{
		long nValue = GetCoreValue( aMtrFldDistance, ePoolUnit ) *
					  GetCoreValue( aMtrFldFocalLength, ePoolUnit ) /
					  aOldFocalLength;

		SetMetricValue( aMtrFldDistance, nValue, ePoolUnit );
	}
	aOldFocalLength = GetCoreValue( aMtrFldFocalLength, ePoolUnit );

	return( 0L );
}

// -----------------------------------------------------------------------
IMPL_LINK( Svx3DAttrTabPage, SegmentsChangedHdl, void *, p )
{
	if( p == &aNumFldHorzSegs )
	{
		long nVal = aNumFldHorzSegs.GetValue();
		long nMax = Min( 10000L / nVal, 256L );
		aNumFldVertSegs.SetMax( nMax );
		aNumFldVertSegs.SetLast( nMax );
		//aNumFldVertSegs.Reformat();
	}
	else
	{
		long nVal = aNumFldVertSegs.GetValue();
		long nMax = Min( 10000L / nVal, 256L );
		aNumFldHorzSegs.SetMax( nMax );
		aNumFldHorzSegs.SetLast( nMax );
		//aNumFldHorzSegs.Reformat();
	}

	return( 0L );
}

//------------------------------------------------------------------------

void Svx3DAttrTabPage::FillUserData()
{
	// Abgleich wird in der Ini-Datei festgehalten
	String aStrAutoMode( UniString::CreateFromInt32( (sal_Int32)aCbxAutoMode.IsChecked() ) );
	SetUserData( aStrAutoMode );
}


