/*************************************************************************
 *
 *  $RCSfile: dbbrwcbx.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#pragma hdrstop
#include "dbbrowse.hxx"

//==================================================================
DbComboBoxCtrl::DbComboBoxCtrl(Window* pParent, WinBits nWinStyle)
			   :ComboBox(pParent, nWinStyle|WB_DROPDOWN|WB_NOBORDER)
{
	EnableAutoSize(sal_False);
	EnableAutocomplete(sal_True);
	SetDropDownLineCount(5);
}

//------------------------------------------------------------------------------
long DbComboBoxCtrl::PreNotify( NotifyEvent& rNEvt )
{
	switch (rNEvt.GetType())
	{
		case EVENT_KEYINPUT:
			if (!IsInDropDown())
			{
				const KeyEvent *pEvt = rNEvt.GetKeyEvent();
				const KeyCode rKey = pEvt->GetKeyCode();

				if ((rKey.GetCode() == KEY_UP || rKey.GetCode() == KEY_DOWN) &&
					(!pEvt->GetKeyCode().IsShift() && pEvt->GetKeyCode().IsMod1()))
				{
					// Naechsten bzw vorherigen Eintrag selektieren
					int nPos = GetEntryPos(GetText());
					nPos += rKey.GetCode() == KEY_DOWN ? 1 : -1;
					if (nPos < 0)
						nPos = 0;
					if (nPos >= GetEntryCount())
						nPos = GetEntryCount() - 1;
					SetText(GetEntry(nPos));
					return 1;
				}
			}
			break;
	}
	return ComboBox::PreNotify(rNEvt);
}

//------------------------------------------------------------------------------
DbComboBoxCellController::DbComboBoxCellController(DbComboBoxCtrl* pWin)
						 :DbCellController(pWin)
{
}

//------------------------------------------------------------------------------
sal_Bool DbComboBoxCellController::MoveAllowed(const KeyEvent& rEvt) const
{
	DbComboBoxCtrl& rBox = GetComboBox();
	switch (rEvt.GetKeyCode().GetCode())
	{
		case KEY_END:
		case KEY_RIGHT:
		{
			Selection aSel = rBox.GetSelection();
			return !aSel && aSel.Max() == rBox.GetText().Len();
		}
		case KEY_HOME:
		case KEY_LEFT:
		{
			Selection aSel = rBox.GetSelection();
			return !aSel && aSel.Min() == 0;
		}
		case KEY_UP:
		case KEY_DOWN:
			if (rBox.IsInDropDown())
				return sal_False;
			if (!rEvt.GetKeyCode().IsShift() &&
				 rEvt.GetKeyCode().IsMod1())
				return sal_False;
			// aufklappen der Listbox
			else if (rEvt.GetKeyCode().IsMod2() && rEvt.GetKeyCode().GetCode() == KEY_DOWN)
				return sal_False;
		case KEY_PAGEUP:
		case KEY_PAGEDOWN:
		case KEY_RETURN:
			if (rBox.IsInDropDown())
				return sal_False;
		default:
			return sal_True;
	}
}

//------------------------------------------------------------------------------
sal_Bool DbComboBoxCellController::IsModified() const
{
	return GetComboBox().GetSavedValue() != GetComboBox().GetText();
}

//------------------------------------------------------------------------------
void DbComboBoxCellController::ClearModified()
{
	GetComboBox().SaveValue();
}

//------------------------------------------------------------------------------
void DbComboBoxCellController::SetModifyHdl(const Link& rLink)
{
	GetComboBox().SetModifyHdl(rLink);
}

//==================================================================
DbListBoxCtrl::DbListBoxCtrl(Window* pParent, WinBits nWinStyle)
			  :ListBox(pParent, nWinStyle|WB_DROPDOWN|WB_NOBORDER)
{
	EnableAutoSize(sal_False);
	EnableMultiSelection(sal_False);
	SetDropDownLineCount(5);
}

//------------------------------------------------------------------------------
long DbListBoxCtrl::PreNotify( NotifyEvent& rNEvt )
{
	switch (rNEvt.GetType())
	{
		case EVENT_KEYINPUT:
			if (!IsInDropDown())
			{
				const KeyEvent *pEvt = rNEvt.GetKeyEvent();
				const KeyCode rKey = pEvt->GetKeyCode();

				if ((rKey.GetCode() == KEY_UP || rKey.GetCode() == KEY_DOWN) &&
					(!pEvt->GetKeyCode().IsShift() && pEvt->GetKeyCode().IsMod1()))
				{
					// Naechsten bzw vorherigen Eintrag selektieren
					int nPos = GetSelectEntryPos();
					nPos += rKey.GetCode() == KEY_DOWN ? 1 : -1;
					if (nPos < 0)
						nPos = 0;
					if (nPos >= GetEntryCount())
						nPos = GetEntryCount() - 1;
					SelectEntryPos(nPos);
					Select();	// fuers Modify
					return 1;
				}
				else if (GetParent()->PreNotify(rNEvt))
					return 1;
			}
			break;
	}
	return ListBox::PreNotify(rNEvt);
}

//------------------------------------------------------------------------------
DbListBoxCellController::DbListBoxCellController(DbListBoxCtrl* pWin)
						 :DbCellController(pWin)
{
}

//------------------------------------------------------------------------------
sal_Bool DbListBoxCellController::MoveAllowed(const KeyEvent& rEvt) const
{
	DbListBoxCtrl& rBox = GetListBox();
	switch (rEvt.GetKeyCode().GetCode())
	{
		case KEY_UP:
		case KEY_DOWN:
			if (!rEvt.GetKeyCode().IsShift() &&
				 rEvt.GetKeyCode().IsMod1())
				return sal_False;
			// aufklappen der Listbox
			else
				if (rEvt.GetKeyCode().IsMod2() && rEvt.GetKeyCode().GetCode() == KEY_DOWN)
					return sal_False;
		case KEY_PAGEUP:
		case KEY_PAGEDOWN:
			if (rBox.IsTravelSelect())
				return sal_False;
		default:
			return sal_True;
	}
}

//------------------------------------------------------------------------------
sal_Bool DbListBoxCellController::IsModified() const
{
	return GetListBox().GetSelectEntryPos() != GetListBox().GetSavedValue();
}

//------------------------------------------------------------------------------
void DbListBoxCellController::ClearModified()
{
	GetListBox().SaveValue();
}

//------------------------------------------------------------------------------
void DbListBoxCellController::SetModifyHdl(const Link& rLink)
{
	GetListBox().SetSelectHdl(rLink);
}



