/*************************************************************************
 *
 *  $RCSfile: fmstream.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:17 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#pragma hdrstop

#include "fmstream.hxx"

#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif

#ifndef _SVX_FMTOOLS_HXX
#include "fmtools.hxx"
#endif

#ifdef _USE_NAMESPACE
using namespace vos;
using namespace usr;
#endif

/*************************************************************************/
SMART_UNO_IMPLEMENTATION(FmUnoInputStream,OWeakObject)


DBG_NAME(FmUnoInputStream);
//------------------------------------------------------------------
FmUnoInputStream::FmUnoInputStream( SvStream& _rStream )
				 :m_pSvStream(&_rStream)
				 ,m_bSvStreamOwner(FALSE)
{
	DBG_CTOR(FmUnoInputStream,NULL);

}

//------------------------------------------------------------------
FmUnoInputStream::FmUnoInputStream( SvStream* pStream, BOOL bOwner )
				 :m_pSvStream( pStream )
				 ,m_bSvStreamOwner( bOwner )
{
	DBG_CTOR(FmUnoInputStream,NULL);

}

//------------------------------------------------------------------
FmUnoInputStream::~FmUnoInputStream()
{
	if( m_bSvStreamOwner )
		delete m_pSvStream;

	DBG_DTOR(FmUnoInputStream,NULL);
}

//------------------------------------------------------------------
BOOL FmUnoInputStream::queryInterface( Uik aUik, XInterfaceRef& rOut )
{
	if (!OWeakObject::queryInterface( aUik, rOut ))
	{
		if (aUik == XInputStream::getSmartUik())
			rOut = (XInputStream*) this;
	}
	return rOut.is();
}

//------------------------------------------------------------------------------
Sequence< XIdlClassRef > FmUnoInputStream::getIdlClasses()
{
	static XIdlClassRef xClass =
		createStandardClass
		(L"FmUnoInputStream",
		 OWeakObject::getStaticIdlClass(),
		 1,
		 XInputStream_getReflection()) ;

	Sequence< XIdlClassRef> aIdlClasses(&xClass, 1);
	return aIdlClasses;
}

//------------------------------------------------------------------------------
INT32 FmUnoInputStream::readBytes(Sequence< BYTE >& aData, INT32 nBytesToRead)
				THROWS( (NotConnectedException, BufferSizeExceededException, UsrSystemException) )
{
	if (!m_pSvStream)
		THROW(NotConnectedException(UString(), *this));

	if (nBytesToRead < 0)
		THROW(BufferSizeExceededException(UString(),*this));

	NAMESPACE_VOS(OGuard) aGuard( m_aMutex );

	aData.realloc(nBytesToRead);

	ULONG nRead = m_pSvStream->Read((void*)aData.getArray(), nBytesToRead);
	if (m_pSvStream->SvStream::GetError() != ERRCODE_NONE)
		THROW(BufferSizeExceededException(UString(),*this));

	// Wenn gelesene Zeichen < MaxLength, Sequence anpassen
	if (nRead < (ULONG)nBytesToRead)
		aData.realloc( nRead );

	return nRead;
}

//------------------------------------------------------------------------------
INT32 FmUnoInputStream::readSomeBytes(Sequence< BYTE >& aData, INT32 nMaxBytesToRead) THROWS( (NotConnectedException, BufferSizeExceededException, UsrSystemException) )
{
	if (!m_pSvStream)
		THROW(NotConnectedException(UString(), *this));

	if (nMaxBytesToRead < 0)
		THROW(BufferSizeExceededException(UString(),*this));

	if (m_pSvStream->IsEof())
	{
		aData.realloc(0);
		return 0;
	}
	else
		return readSomeBytes(aData, nMaxBytesToRead);
}

//------------------------------------------------------------------------------
void FmUnoInputStream::skipBytes(INT32 nBytesToSkip) THROWS( (NotConnectedException, BufferSizeExceededException, UsrSystemException) )
{
	NAMESPACE_VOS(OGuard) aGuard( m_aMutex );
	if (!m_pSvStream)
		THROW(NotConnectedException(UString(), *this));

#ifdef DBG_UTIL
	ULONG nCurrentPos = m_pSvStream->Tell();
#endif

	m_pSvStream->SeekRel(nBytesToSkip);
	if (m_pSvStream->SvStream::GetError() != ERRCODE_NONE)
	{
		THROW(NotConnectedException(UString(), *this));
	}

#ifdef DBG_UTIL
	nCurrentPos = m_pSvStream->Tell();
#endif
}

//------------------------------------------------------------------------------
INT32 FmUnoInputStream::available(void) THROWS( (NotConnectedException, UsrSystemException) )
{
	NAMESPACE_VOS(OGuard) aGuard( m_aMutex );
	if (!m_pSvStream)
		THROW(NotConnectedException(UString(), *this));

	ULONG nPos = m_pSvStream->Tell();
	if (m_pSvStream->SvStream::GetError())
		THROW(NotConnectedException(UString(), *this));

	m_pSvStream->Seek(STREAM_SEEK_TO_END);
	if (m_pSvStream->SvStream::GetError())
	   THROW(NotConnectedException(UString(),*this));

	INT32 nAvailable = (INT32)m_pSvStream->Tell() - nPos;
	m_pSvStream->Seek(nPos);
	if (m_pSvStream->SvStream::GetError())
	   THROW(NotConnectedException(UString(),*this));
	return nAvailable;
}

//------------------------------------------------------------------------------
void FmUnoInputStream::closeInput(void) THROWS( (NotConnectedException, UsrSystemException) )
{
	if (!m_pSvStream)
		THROW(NotConnectedException(UString(), *this));

	if (m_bSvStreamOwner)
		delete m_pSvStream;

	m_pSvStream = NULL;
}

/*************************************************************************/
SMART_UNO_IMPLEMENTATION(FmUnoStreamSink,UsrObject)
//------------------------------------------------------------------
BOOL FmUnoStreamSink::queryInterface( Uik aUik, XInterfaceRef& rOut )
{
	if (aUik == XOutputStream::getSmartUik())
		rOut = (XOutputStream*) this;
	else
		UsrObject::queryInterface( aUik, rOut );

	return rOut.is();
}

//------------------------------------------------------------------------------
Sequence< XIdlClassRef > FmUnoStreamSink::getIdlClasses()
{
	static XIdlClassRef xClass =
		createStandardClass
		(L"FmUnoStreamSink",
		 UsrObject::getUsrObjectIdlClass(),
		 1,
		 XOutputStream_getReflection()) ;

	Sequence< XIdlClassRef> aIdlClasses(&xClass, 1);
	return aIdlClasses;
}

// XOutputStream
//------------------------------------------------------------------------------
void FmUnoStreamSink::writeBytes(const Sequence< BYTE >& aData) THROWS( (NotConnectedException, BufferSizeExceededException, UsrSystemException) )
{
	ULONG nWritten = rStream.Write(aData.getConstArray(),aData.getLen());
	ErrCode err = rStream.GetError();
	if (err != ERRCODE_NONE || nWritten != aData.getLen())
	{
		THROW(BufferSizeExceededException(UString(),(UsrObject*)this));
	}
}

//------------------------------------------------------------------
void FmUnoStreamSink::flush(void) THROWS( (NotConnectedException, BufferSizeExceededException, UsrSystemException) )
{
	rStream.Flush();
	if (rStream.GetError() != ERRCODE_NONE)
		THROW(NotConnectedException(UString(),(UsrObject*)this));
}

//------------------------------------------------------------------
void FmUnoStreamSink::closeOutput(void) THROWS( (NotConnectedException, BufferSizeExceededException, UsrSystemException) )
{
}


