/*************************************************************************
 *
 *  $RCSfile: propctr2.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:17 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#ifndef SVX_PROPCTR2_HXX
#include "propctr2.hxx"
#endif

#ifndef _NUMUNO_HXX //autogen wg. SvNumberFormatsSupplierObj
#include <svtools/numuno.hxx>
#endif

#ifndef _ZFORMAT_HXX
#include <svtools/zformat.hxx>
#endif

#ifndef SVX_PROPERTY_HXX
#include "property.hxx"
#endif

#ifndef _SV_SVAPP_HXX //autogen wg. Application
#include <vcl/svapp.hxx>
#endif

#ifndef _TOOLS_SOLMATH_HXX //autogen wg. SolarMath
#include <tools/solmath.hxx>
#endif
#define S(aUniCode)				\
	::rtl::OUStringToOString(aUniCode, RTL_TEXTENCODING_ASCII_US)
//==================================================================
// class SvxPropFormatEdit
//==================================================================

//------------------------------------------------------------------
SvxPropFormatEdit::SvxPropFormatEdit( Window* pParent, const ::rtl::OUString& rStandard, WinBits nWinStyle)
		:SvxPropCommonControl(this, rStandard)
		,FormattedField(pParent, nWinStyle)
{
	SetModifyHdl(LINK( this, SvxPropCommonControl, ModifiedHdl ));
	SetGetFocusHdl(LINK( this, SvxPropCommonControl, GetFocusHdl));
	SetLoseFocusHdl(LINK( this, SvxPropCommonControl, LoseFocusHdl));

	ComboBox aComboBox(this,WB_DROPDOWN);
	aComboBox.SetPosSizePixel(Point(0,0),Size(100,100));
	SetSizePixel(aComboBox.GetSizePixel());
}

//------------------------------------------------------------------
long SvxPropFormatEdit::PreNotify( NotifyEvent& rNEvt )
{
	// want to handle two keys myself : Del/Backspace should empty the window (setting my prop to "standard" this way)
	if (EVENT_KEYINPUT == rNEvt.GetType())
	{
		sal_uInt16 nKey = rNEvt.GetKeyEvent()->GetKeyCode().GetCode();

		if ((KEY_DELETE == nKey) || (KEY_BACKSPACE == nKey))
		{
			SetText(XubString());
			ModifiedHdl(this);
			return sal_True;
		}
	}

	if (SvxPropCommonControl::HandlePreNotify(rNEvt))
		return sal_True;

	// it wasn't a PropCommonControl event => let our window handle it
	return FormattedField::PreNotify(rNEvt);
}

//------------------------------------------------------------------
void SvxPropFormatEdit::SetProperty(const ::rtl::OUString& rString, sal_Bool bIsUnknown)
{
	if (bIsUnknown || (rString == m_sStandardString) || !rString.getLength())
		// unknown or standard -> no text
		SetText(XubString());
	else
	{
		// else set the new format key, the text will be reformatted
		SetValue(1234.56789);
		SetFormatKey(UniString(rString).ToInt32());
	}
}

//------------------------------------------------------------------
::rtl::OUString SvxPropFormatEdit::GetProperty() const
{
	if (!GetText().Len())
		return m_sStandardString;
	else
		return XubString(GetFormatKey());
}

//------------------------------------------------------------------
void SvxPropFormatEdit::SetFormatSupplier(const SvNumberFormatsSupplierObj* pSupplier)
{
	if (pSupplier)
	{
		TreatAsNumber(sal_True);

		SvNumberFormatter* pFormatter = pSupplier->GetNumberFormatter();
		SetFormatter(pFormatter, sal_True);
		SetValue(1234.56789);
	}
	else
	{
		TreatAsNumber(sal_False);
		SetFormatter(NULL, sal_True);
		SetText(XubString());
	}
}

//==================================================================
// class SvxPropDoubleNumericField
//==================================================================


DBG_NAME(SvxPropDoubleNumericField);
//------------------------------------------------------------------
SvxPropDoubleNumericField::SvxPropDoubleNumericField( Window* pParent, const ::rtl::OUString& rStandard, WinBits nWinStyle)
		:SvxPropCommonControl(this, rStandard)
		,FormattedField(pParent, nWinStyle)
{
	DBG_CTOR(SvxPropDoubleNumericField,NULL);

	SetModifyHdl(LINK( this, SvxPropCommonControl, ModifiedHdl ));
	SetGetFocusHdl(LINK( this, SvxPropCommonControl, GetFocusHdl));
	SetLoseFocusHdl(LINK( this, SvxPropCommonControl, LoseFocusHdl));

	ComboBox aComboBox(this,WB_DROPDOWN);
	aComboBox.SetPosSizePixel(Point(0,0),Size(100,100));
	SetSizePixel(aComboBox.GetSizePixel());

	TreatAsNumber(sal_True);

	m_dLastDecimalDigits = GetDecimalDigits();
	const SvNumberformat* pFormat = GetFormatter()->GetEntry(GetFormatKey());
	m_aUsedInternational = pFormat ? International(pFormat->GetLanguage()) : Application::GetAppInternational();
}

//------------------------------------------------------------------
SvxPropDoubleNumericField::~SvxPropDoubleNumericField()
{

	DBG_DTOR(SvxPropDoubleNumericField,NULL);
}

//------------------------------------------------------------------
long SvxPropDoubleNumericField::PreNotify( NotifyEvent& rNEvt )
{
	if (SvxPropCommonControl::HandlePreNotify(rNEvt))
		return sal_True;

	// it wasn't a PropCommonControl event => let our window handle it
	return FormattedField::PreNotify(rNEvt);
}

//------------------------------------------------------------------
void SvxPropDoubleNumericField::SetProperty(const ::rtl::OUString& rString, sal_Bool bIsUnknown)
{
	if (bIsUnknown || (rString == m_sStandardString) || !rString.getLength())
		// unknown or standard -> no text
		SetText(String());
	else
	{
		SetValue(UniString(rString).ToDouble());
	}
}

//------------------------------------------------------------------
::rtl::OUString SvxPropDoubleNumericField::GetProperty() const
{
	if (!GetText().Len())
		return m_sStandardString;
	else
	{
		XubString sReturn;
		SolarMath::DoubleToString(
			sReturn, ((SvxPropDoubleNumericField*)this)->GetValue(), 'F', m_dLastDecimalDigits,
			'.', sal_True);
			// always use a '.' (instead of the decimal separator given by m_aUsedInternational) :
			// when the returned string is interpreted nobody knows about this control or the used
			// international, so by convention we always use '.' for describing floating point numbers
			// 27.07.99 - 67901 - FS
		return sReturn;
	}
}

//------------------------------------------------------------------
void SvxPropDoubleNumericField::SetFormatDescription(const FormatDescription& rDesc)
{
	if (rDesc.pSupplier)
	{
		TreatAsNumber(sal_True);

		SvNumberFormatter* pFormatter = rDesc.pSupplier->GetNumberFormatter();
		if (pFormatter != GetFormatter())
			SetFormatter(pFormatter, sal_True);
		SetFormatKey(rDesc.nKey);

		const SvNumberformat* pFormat = GetFormatter()->GetEntry(GetFormatKey());
		DBG_ASSERT(pFormat, "SvxPropDoubleNumericField::SetFormatDescription : could not retrieve a number format !");
		m_aUsedInternational = pFormat ? International(pFormat->GetLanguage()) : Application::GetAppInternational();
	}
	else
	{
		TreatAsNumber(sal_False);
		SetFormatter(NULL, sal_True);
		SetText(XubString());
	}
}


