/*************************************************************************
 *
 *  $RCSfile: galbind.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "gallery.hxx"
#include <vos/module.hxx>
#include <tools/fsys.hxx>
#include <vcl/graph.hxx>

// -----------
// - Defines -
// -----------

#if defined WIN || defined WNT || defined MAC
#define GAL_LIBNAME ( ( ( aLibName.AssignAscii( "gal" ) ).Append( String::CreateFromInt32( SUPD ) ) ).AppendAscii( __DLLEXTENSION ) ).AppendAscii( ".dll" )
#elif defined OS2
#define GAL_LIBNAME ( ( aLibName.AssignAscii( "gal" ) ).Append( String::CreateFromInt32( SUPD ) ) ).AppendAscii( __DLLEXTENSION )
#elif defined UNX
#define GAL_LIBNAME ( ( aLibName.AssignAscii( "libgal" ) ).Append( String::CreateFromInt32( SUPD ) ) ).AppendAscii( __DLLEXTENSION )
#endif

#define GAL_LIBINIT()							\
if( !pLib )										\
{												\
	String aLibName;							\
	pLib = new ::vos::OModule( GAL_LIBNAME );	\
}

// -----------
// - statics -
// -----------

::vos::OModule* pLib = NULL;

// ------------
// - typedefs -
// ------------

typedef BOOL	(SAL_CALL *FncGetPath)( DirEntry& rPath );
typedef BOOL	(SAL_CALL *FncGetPathAsString)( String& rPath );
typedef BOOL	(SAL_CALL *FncGetFilterName)( String& rFilterName );
typedef BOOL	(SAL_CALL *FncGetGraphic)( Graphic& rGraphic );
typedef BOOL	(SAL_CALL *FncGetVCDrawModel)( FmFormModel& rModel );
typedef BOOL	(SAL_CALL *FncIsLinkage)();
typedef BOOL	(SAL_CALL *FncFillThemeList)( List& rThemeList );
typedef BOOL	(SAL_CALL *FncFillObjList)( const String& rThemeName, List& rObjList );
typedef BOOL	(SAL_CALL *FncInsertURL)( const String& rThemeName, const String& rURL, const ULONG nSgaFormat );
typedef ULONG	(SAL_CALL *FncGetObjCount)( const String& rThemeName );
typedef BOOL	(SAL_CALL *FncGetGraphicObj)( const String& rThemeName, ULONG nPos, Graphic* pGraphic, Bitmap* pThumb, BOOL bProgess );
typedef BOOL	(SAL_CALL *FncInsertGraphicObj)( const String& rThemeName, const Graphic& rGraphic );
typedef ULONG	(SAL_CALL *FncGetSdrObjCount)( const String& rThemeName );
typedef BOOL	(SAL_CALL *FncGetSdrObj)( const String& rThemeName, ULONG nSdrModelPos, FmFormModel* pModel, Bitmap* pThumb );
typedef BOOL	(SAL_CALL *FncInsertSdrObj)( const String& rThemeName, FmFormModel& rModel );
typedef BOOL	(SAL_CALL *FncGetThemeNameFromId)( ULONG nThemeId, String& rThemeName );
typedef BOOL	(SAL_CALL *FncBeginLocking)( const String& rThemeName );
typedef BOOL	(SAL_CALL *FncEndLocking)( const String& rThemeName );

// -------------------
// - GalleryExplorer -
// -------------------

void GalleryExplorer::Register( SfxExplorer* pExplorer )
{
	// nothing to do anymore
}

// ------------------------------------------------------------------------

GalleryExplorer* GalleryExplorer::GetGallery()
{
	static GalleryExplorer* pThis = NULL;

	// only create a dummy object which can be accessed
	if( !pThis )
		pThis = new GalleryExplorer;

	return pThis;
}

// ------------------------------------------------------------------------

const DirEntry& GalleryExplorer::GetPath() const
{
	GAL_LIBINIT();
	FncGetPath		pFnc = (FncGetPath) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetPath" ) ) );
	static DirEntry aDirEntry;

	aDirEntry = DirEntry();

	if( pFnc )
		(*pFnc)( aDirEntry );

	return aDirEntry;
}

// ------------------------------------------------------------------------

INetURLObject GalleryExplorer::GetURL() const
{
	INetURLObject aURL;

	aURL.SetSmartURL( GetPathAsString() );

	return aURL;
}

// ------------------------------------------------------------------------

String GalleryExplorer::GetPathAsString() const
{
	GAL_LIBINIT();
	FncGetPathAsString	pFnc = (FncGetPathAsString) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetPathAsString" ) ) );
	String				aString;

	if( pFnc )
		(*pFnc)( aString );

	return aString;
}

// ------------------------------------------------------------------------

String GalleryExplorer::GetFilterName() const
{
	GAL_LIBINIT();
	FncGetFilterName	pFnc = (FncGetFilterName) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetFilterName" ) ) );
	String				aString;

	if( pFnc )
		(*pFnc)( aString );

	return aString;
}

// ------------------------------------------------------------------------

Graphic GalleryExplorer::GetGraphic() const
{
	GAL_LIBINIT();
	FncGetGraphic	pFnc = (FncGetGraphic) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetGraphic" ) ) );
	Graphic			aGraphic;

	if( pFnc )
		(*pFnc)( aGraphic );

	return aGraphic;
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::GetVCDrawModel( FmFormModel& rModel ) const
{
	GAL_LIBINIT();
	FncGetVCDrawModel pFnc = (FncGetVCDrawModel) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetVCDrawModel" ) ) );
	return( pFnc ? (*pFnc)( rModel ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::IsLinkage() const
{
	GAL_LIBINIT();
	FncIsLinkage pFnc = (FncIsLinkage) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_IsLinkage" ) ) );
	return( pFnc ? (*pFnc)() : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::FillThemeList( List& rThemeList )
{
	GAL_LIBINIT();
	FncFillThemeList pFnc = (FncFillThemeList) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_FillThemeList" ) ) );

	return( pFnc ? (*pFnc)( rThemeList ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::FillObjList( const String& rThemeName, List& rObjList )
{
	GAL_LIBINIT();
	FncFillObjList pFnc = (FncFillObjList) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_FillObjList" ) ) );

	return( pFnc ? (*pFnc)( rThemeName, rObjList ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::FillObjList( ULONG nThemeId, List& rObjList )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? FillObjList( aThemeName, rObjList ) : FALSE;
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertURL( const String& rThemeName, const String& rURL )
{
	return InsertURL( rThemeName, rURL, SGA_FORMAT_ALL );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertURL( ULONG nThemeId, const String& rURL )
{
	return InsertURL( nThemeId, rURL, SGA_FORMAT_ALL );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertURL( const String& rThemeName, const String& rURL, const ULONG nSgaFormat )
{
	GAL_LIBINIT();
	FncInsertURL pFnc = (FncInsertURL) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_InsertURL" ) ) );

	return( pFnc ? (*pFnc)( rThemeName, rURL, nSgaFormat ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertURL( ULONG nThemeId, const String& rURL, const ULONG nSgaFormat )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? InsertURL( aThemeName, rURL, nSgaFormat ) : FALSE;
}

// ------------------------------------------------------------------------

ULONG GalleryExplorer::GetObjCount( const String& rThemeName )
{
	GAL_LIBINIT();
	FncGetObjCount pFnc = (FncGetObjCount) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetObjCount" ) ) );

	return( pFnc ? (*pFnc)( rThemeName ) : 0UL );
}

// ------------------------------------------------------------------------

ULONG GalleryExplorer::GetObjCount( ULONG nThemeId  )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? GetObjCount( aThemeName ) : 0UL;
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::GetGraphicObj( const String& rThemeName, ULONG nPos,
									 Graphic* pGraphic, Bitmap* pThumb,
									 BOOL bProgress )
{
	GAL_LIBINIT();
	FncGetGraphicObj pFnc = (FncGetGraphicObj) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetGraphicObj" ) ) );

	return( pFnc ? (*pFnc)( rThemeName, nPos, pGraphic, pThumb, bProgress ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::GetGraphicObj( ULONG nThemeId, ULONG nPos,
									 Graphic* pGraphic, Bitmap* pThumb,
									 BOOL bProgress )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? GetGraphicObj( aThemeName, nPos, pGraphic, pThumb, bProgress ) : FALSE;
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertGraphicObj( const String& rThemeName, const Graphic& rGraphic )
{
	GAL_LIBINIT();
	FncInsertGraphicObj pFnc = (FncInsertGraphicObj) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_InsertGraphicObj" ) ) );

	return( pFnc ? (*pFnc)( rThemeName, rGraphic ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertGraphicObj( ULONG nThemeId, const Graphic& rGraphic )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? InsertGraphicObj( aThemeName, rGraphic ) : FALSE;
}

// ------------------------------------------------------------------------

ULONG GalleryExplorer::GetSdrObjCount( const String& rThemeName )
{
	GAL_LIBINIT();
	FncGetSdrObjCount pFnc = (FncGetSdrObjCount) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetSdrObjCount" ) ) );

	return( pFnc ? (*pFnc)( rThemeName ) : 0UL );
}

// ------------------------------------------------------------------------

ULONG GalleryExplorer::GetSdrObjCount( ULONG nThemeId  )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? GetSdrObjCount( aThemeName ) : 0UL;
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::GetSdrObj( const String& rThemeName, ULONG nSdrModelPos,
								 FmFormModel* pModel, Bitmap* pThumb )
{
	GAL_LIBINIT();
	FncGetSdrObj pFnc = (FncGetSdrObj) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetSdrObj" ) ) );

	return( pFnc ? (*pFnc)( rThemeName, nSdrModelPos, pModel, pThumb ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::GetSdrObj( ULONG nThemeId, ULONG nSdrModelPos,
								 FmFormModel* pModel, Bitmap* pThumb )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? GetSdrObj( aThemeName, nSdrModelPos, pModel, pThumb ) : FALSE;
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertSdrObj( const String& rThemeName, FmFormModel& rModel )
{
	GAL_LIBINIT();
	FncInsertSdrObj	pFnc = (FncInsertSdrObj) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_InsertSdrObj" ) ) );

	return( pFnc ? (*pFnc)( rThemeName, rModel ) : FALSE );
}

// ------------------------------------------------------------------------

BOOL GalleryExplorer::InsertSdrObj( ULONG nThemeId, FmFormModel& rModel )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? InsertSdrObj( aThemeName, rModel ) : FALSE;
}

// -----------------------------------------------------------------------------

BOOL GalleryExplorer::BeginLocking( const String& rThemeName )
{
	GAL_LIBINIT();
	FncBeginLocking	pFnc = (FncBeginLocking) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_BeginLocking" ) ) );

	return( pFnc ? (*pFnc)( rThemeName ) : FALSE );
}

// -----------------------------------------------------------------------------

BOOL GalleryExplorer::BeginLocking( ULONG nThemeId )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? BeginLocking( aThemeName ) : FALSE;
}

// -----------------------------------------------------------------------------

BOOL GalleryExplorer::EndLocking( const String& rThemeName )
{
	GAL_LIBINIT();
	FncEndLocking	pFnc = (FncEndLocking) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_EndLocking" ) ) );

	return( pFnc ? (*pFnc)( rThemeName ) : FALSE );
}

// -----------------------------------------------------------------------------

BOOL GalleryExplorer::EndLocking( ULONG nThemeId )
{
	GAL_LIBINIT();
	FncGetThemeNameFromId	pFnc = (FncGetThemeNameFromId) pLib->getSymbol( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery_GetThemeNameFromId" ) ) );
	String					aThemeName;

	return ( pFnc && (*pFnc)( nThemeId, aThemeName ) ) ? EndLocking( aThemeName ) : FALSE;
}
