/*************************************************************************
 *
 *  $RCSfile: galcmp1.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "galcmp.hxx"
#include "gallery1.hxx"
#include "gallery2.hxx"
#include "gallery.hrc"
#include "gallery.hrc"

#include <vos/mutex.hxx>
#include <vcl/msgbox.hxx>
#include <svtools/cntcmpmn.hxx>
#include <svtools/iniman.hxx>
#include <sfx2/iodlg.hxx>
#include <sfx2/inimgr.hxx>

// -----------
// - Defines -
// -----------

#define GAL_PROP_ANZ			7
#define GAL_COMMAND_ANZ			4

#define	CMP_HAS_FOLDER			1
#define	CMP_IS_FOLDER			2
#define CMP_OWN_URL				3
#define CMP_MENU				4
#define CMP_SORTING				5
#define CMP_TARGETS				6
#define CMP_TITLE				7

// -----------
// - statics -
// -----------

SgaGallery* pGlobalGal = NULL;
List*		pThemeCmpList;
ULONG		nGalCount = 0UL;

// ------------------------------------------------------------------------

const GalPropertyMap pGalMap_Impl[] =
{
	{ "HasFolder", CMP_HAS_FOLDER, NULL /*!!!UNO3 &BOOL_getReflection*/, 2, 0 },
	{ "IsFolder", CMP_IS_FOLDER, NULL /*!!!UNO3 &BOOL_getReflection*/, 2, 0 },
	{ "OwnURL", CMP_OWN_URL, NULL /*!!!UNO3 &OUString_getReflection*/, 2, 0 },
	{ "PopupMenu", CMP_MENU, NULL /*!!!UNO3 &MenuDescriptor_getReflection*/, 2, 0 },
	{ "Sorting", CMP_SORTING, NULL /*!!!UNO3 &SortingInfo_getReflection*/, 2, 0 },
	{ "TargetFrames", CMP_TARGETS, NULL /*!!!UNO3 &OUString_getReflection*/, 2, 0 },
	{ "Title", CMP_TITLE, NULL /*!!!UNO3 &OUString_getReflection*/, 2, 0 },
	{ 0, 0 }
};

// ------------------------------------------------------------------------

void LoadGallery()
{
	if( !nGalCount++ )
	{
		pGlobalGal = new SgaGallery( NULL );
		pThemeCmpList = new List;
	}
}

// ------------------------------------------------------------------------

void UnloadGallery()
{
	if( !--nGalCount )
	{
		delete pGlobalGal;
		pGlobalGal = NULL;

		delete pThemeCmpList;
		pThemeCmpList = NULL;
	}
}

// ---------------------
// - GalleryProperties -
// ---------------------

GalleryProperties::GalleryProperties( Window* pParent, ExchangeData* _pData, SfxItemSet* pItemSet  ) :
			SfxTabDialog    ( pParent, GAL_RESID( RID_SVXTABDLG_GALLERY ), pItemSet ),
			pData			( _pData )
{
	FreeResource();
	AddTabPage( RID_SVXTABPAGE_GALLERY_GENERAL, TPGalleryGeneral::Create, 0 );
	String aText( GetText() );
	SetText( aText += pData->aGalleryTitle );
}

// ------------------------------------------------------------------------

void GalleryProperties::PageCreated( USHORT nId, SfxTabPage &rPage )
{
	( (TPGalleryGeneral&) rPage ).SetXChgData( pData );
}

// --------------------
// - TPGalleryGeneral -
// --------------------

TPGalleryGeneral::TPGalleryGeneral( Window* pParent, const SfxItemSet& rSet ) :
			SfxTabPage              ( pParent, GAL_RESID( RID_SVXTABPAGE_GALLERY_GENERAL ), rSet ),
			aFiMSImage              ( this, ResId( FI_MS_IMAGE ) ),
			aEdtMSName              ( this, ResId( EDT_MS_NAME ) ),
			aFlMSGeneralFirst       ( this, ResId( FL_MS_GENERAL_FIRST ) ),
			aFtMSType               ( this, ResId( FT_MS_TYPE ) ),
			aFtMSShowType           ( this, ResId( FT_MS_SHOW_TYPE ) ),
			aFtMSPath               ( this, ResId( FT_MS_PATH ) ),
			aFtMSShowPath           ( this, ResId( FT_MS_SHOW_PATH ) ),
			aFtMSContent            ( this, ResId( FT_MS_CONTENT ) ),
			aFtMSShowContent        ( this, ResId( FT_MS_SHOW_CONTENT ) ),
			aFlMSGeneralSecond      ( this, ResId( FL_MS_GENERAL_SECOND ) ),
			aFtMSCreateDate         ( this, ResId( FT_MS_CREATEDATE ) ),
			aFtMSShowCreateDate     ( this, ResId( FT_MS_SHOW_CREATEDATE ) ),
			aFtMSChangeDate         ( this, ResId( FT_MS_CHANGEDATE ) ),
			aFtMSShowChangeDate     ( this, ResId( FT_MS_SHOW_CHANGEDATE ) )
{
	FreeResource();
	aEdtMSName.Disable();
}

// ------------------------------------------------------------------------

void TPGalleryGeneral::SetXChgData( ExchangeData* _pData )
{
	pData = _pData;
	InitControls();
}

// ------------------------------------------------------------------------

BOOL TPGalleryGeneral::FillItemSet( SfxItemSet& rSet )
{
	pData->aGalleryTitle = aEdtMSName.GetText();
	return FALSE;
}

// ------------------------------------------------------------------------

void TPGalleryGeneral::InitControls()
{
	const International	aInternational( Application::GetAppInternational() );
	String              aOutStr( pData->nThemeCount );
	String              aThemeStr( GAL_RESID( RID_SVXSTR_GALLERYPROPS_THEME ) );
	String              aAccess;

	// Titel und Pad
	aEdtMSName.SetText( pData->aGalleryTitle );
	aEdtMSName.SetReadOnly( TRUE );
	aFtMSShowType.SetText( String( RTL_CONSTASCII_USTRINGPARAM( STAR_GALLERY ) ) );
	aFtMSShowPath.SetText( pData->pGal->GetUserPath() );

	// Ein- oder Mehrzahl?
	if ( 1 == pData->nThemeCount )
		aThemeStr = aThemeStr.GetToken( 0 );
	else
		aThemeStr = aThemeStr.GetToken( 1 );

	aOutStr += ' ';
	aOutStr += aThemeStr;

	// Inhalt
	aFtMSShowContent.SetText( aOutStr );

	// CreateDate/Time
	aAccess = aInternational.GetDate( pData->aGalleryCreateDate );
	aAccess += String( RTL_CONSTASCII_USTRINGPARAM( ", " ) );
	aAccess += aInternational.GetTime( pData->aGalleryCreateTime );
	aFtMSShowCreateDate.SetText( aAccess );

	// ChangeDate/Time
	aAccess = aInternational.GetDate( pData->aGalleryChangeDate );
	aAccess += String( RTL_CONSTASCII_USTRINGPARAM( ", " ) );
	aAccess += aInternational.GetTime( pData->aGalleryChangeTime );
	aFtMSShowChangeDate.SetText( aAccess );

	// Image setzen
	aFiMSImage.SetImage( Image( Bitmap( GAL_RESID( RID_SVXBMP_GALLERY ) ), COL_LIGHTMAGENTA ) );
}

// ------------------------------------------------------------------------

SfxTabPage* TPGalleryGeneral::Create( Window* pParent, const SfxItemSet& rSet )
{
	return new TPGalleryGeneral( pParent, rSet );
}

// ----------------------
// - GalPropertySetInfo -
// ----------------------

GalPropertySetInfo::GalPropertySetInfo( const GalPropertyMap* pMap ) :
			mpMap( pMap )
{
}

// ------------------------------------------------------------------------

SEQ( NMSP_BEANS::Property ) SAL_CALL GalPropertySetInfo::getProperties() throw( NMSP_UNO::RuntimeException )
{
	SEQ( NMSP_BEANS::Property )	aPropSeq( GAL_PROP_ANZ );
	USHORT						n = 0;

	for( const GalPropertyMap* pMap = mpMap; pMap->pName; ++pMap, n++ )
	{
		aPropSeq.getArray()[n].Name = OUSTRING::createFromAscii( pMap->pName );
		aPropSeq.getArray()[n].Handle = pMap->nId;
		aPropSeq.getArray()[n].Type = 0; /*!!!UNO3 pMap->pGetReflection()->getIdlClass();*/
		aPropSeq.getArray()[n].Attributes = pMap->nFlags;
	}

	return aPropSeq;
}

// ------------------------------------------------------------------------

NMSP_BEANS::Property SAL_CALL GalPropertySetInfo::getPropertyByName( const OUSTRING& rName ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_UNO::RuntimeException )
{
	NMSP_BEANS::Property aProp;

	for( const GalPropertyMap* pMap = mpMap; pMap->pName; ++pMap )
	{
		if( OUSTRING::createFromAscii( pMap->pName ) == rName )
		{
			aProp.Name = rName;
			aProp.Handle = pMap->nId;
			aProp.Type = 0; /*!!!UNO3 pMap->pGetReflection()->getIdlClass();*/
			aProp.Attributes = pMap->nFlags;
			break;
		}
	}

	return aProp;
}

// ------------------------------------------------------------------------

sal_Bool SAL_CALL GalPropertySetInfo::hasPropertyByName( const OUSTRING& rName ) throw( NMSP_UNO::RuntimeException )
{
	for( const GalPropertyMap* pMap = mpMap; pMap->pName; ++pMap )
		if( OUSTRING::createFromAscii( pMap->pName ) == rName )
			return TRUE;

	return FALSE;
}

// --------------------
// - GalComponent -
// --------------------

GalComponent::GalComponent() :
	maListeners( maMutex )
{
	maGalConfigPath = SFX_INIMANAGER()->Get( SFX_KEY_STORAGE_DIR );
	maGalConfigPath += DirEntry( String( RTL_CONSTASCII_USTRINGPARAM( "gallery.sga" ) ) );
	mpData = new ExchangeData;
	mpData->pGal = NULL;
	SetTitle( mpData->aGalleryTitle = String( RTL_CONSTASCII_USTRINGPARAM( STAR_GALLERY ) ) );
}

// ------------------------------------------------------------------------

GalComponent::~GalComponent()
{
	::UnloadGallery();
	delete mpData;

	if( maListeners.getLength() )
	{
		::com::sun::star::lang::EventObject aEvt;

		aEvt.Source = REF( NMSP_UNO::XInterface )( static_cast< ::cppu::OWeakObject* >( this ) );
		maListeners.disposeAndClear( aEvt );
	}
}

// ------------------------------------------------------------------------

REF( NMSP_UNO::XInterface ) SAL_CALL GalComponent::getParent() throw( NMSP_UNO::RuntimeException )
{
	return REF( NMSP_UNO::XInterface )();
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::setParent( const REF( NMSP_UNO::XInterface )& rxParent ) throw( NMSP_LANG::NoSupportException, NMSP_UNO::RuntimeException )
{
}

// ------------------------------------------------------------------------

OUSTRING SAL_CALL GalComponent::getURL() throw( NMSP_UNO::RuntimeException )
{
	return maUrl;
}

// ------------------------------------------------------------------------

sal_Bool SAL_CALL GalComponent::initialize( const REF( NMSP_CHAOS::XURLContent )& rxParent, const OUSTRING& rURL ) throw( NMSP_UNO::RuntimeException )
{
	const String	aFileName( maGalConfigPath.GetFull() );
	UINT16			nVer;
	ByteString		aTmpStr;
	String			aTitle;

	// if file not exists, write it the first time
	if( FileStat( aFileName ).GetSize() <= 2L )
		Save();

	SvFileStream aIStm( aFileName, STREAM_READ );

	aIStm >> nVer;
	aIStm >> aTmpStr; aTitle = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );
	aIStm >> mpData->aGalleryCreateDate;
	aIStm >> mpData->aGalleryCreateTime;
	aIStm >> mpData->aGalleryChangeDate;
	aIStm >> mpData->aGalleryChangeTime;

	if( !aIStm.GetError() )
		SetTitle( aTitle );

	maUrl = rURL;

	return TRUE;
}

// ------------------------------------------------------------------------

SEQ( OUSTRING ) SAL_CALL GalComponent::queryCommands() throw( NMSP_UNO::RuntimeException )
{
	SEQ( OUSTRING ) aCommands( 4 );
	OUSTRING*		pCmd = aCommands.getArray();

	pCmd[ 0 ] = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "initialize" ) );
	pCmd[ 1 ] = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "open" ) );
	pCmd[ 2 ] = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "close" ) );
	pCmd[ 3 ] = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "showPropertyDialog" ) );

	return aCommands;
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::execute( const OUSTRING& rCommand, const ANY& rArg, const REF( NMSP_CHAOS::XStatusCallback )& rxCallback ) throw( NMSP_CHAOS::UnknownCommandException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException )
{
	NMSP_CHAOS::StatusCallbackEvent aEvt( REF( NMSP_UNO::XInterface )( static_cast< ::cppu::OWeakObject* >( this ) ), REF( NMSP_UNO::XInterface)(), rCommand, rArg );

	Application::GetSolarMutex().acquire();

	if( rCommand.compareToAscii( "open" ) == COMPARE_EQUAL )
		ImplHandleOpenEvt( rxCallback, aEvt );
	else if( rCommand.compareToAscii( "NewTheme" ) == COMPARE_EQUAL )
		ImplHandleNewEvt( rxCallback, aEvt );
	else if( rCommand.compareToAscii( "ImportTheme" ) == COMPARE_EQUAL )
		ImplHandleImportEvt( rxCallback, aEvt );
	else if( rCommand.compareToAscii( "showPropertyDialog" ) == COMPARE_EQUAL )
		EditProperties();

	Application::GetSolarMutex().release();
	rxCallback->done( aEvt );
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::cancel( const REF( NMSP_UNO::XInterface )& rxCommandId ) throw( NMSP_UNO::RuntimeException )
{
}

// ------------------------------------------------------------------------

REF( NMSP_BEANS::XPropertySetInfo ) SAL_CALL GalComponent::getPropertySetInfo() throw( NMSP_UNO::RuntimeException )
{
	const REF( NMSP_UNO::XInterface ) xIf( static_cast< ::cppu::OWeakObject* >( new GalPropertySetInfo( pGalMap_Impl ) ) );
	return REF( NMSP_BEANS::XPropertySetInfo )( xIf, NMSP_UNO::UNO_QUERY );
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::setPropertyValue( const OUSTRING& rPropertyName, const ANY& rValue ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_BEANS::PropertyVetoException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException)
{
}

// ------------------------------------------------------------------------

ANY SAL_CALL GalComponent::getPropertyValue( const OUSTRING& rPropertyName ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_BEANS::PropertyVetoException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException)
{
	Application::GetSolarMutex().acquire();

	ANY aAny;

	for( ULONG i = 0UL; i < GAL_PROP_ANZ; i++ )
	{
		if( rPropertyName == OUSTRING::createFromAscii( pGalMap_Impl[ i ].pName ) )
		{
			if( !mpData->pGal )
				( (GalComponent*) this )->LoadGallery();

			switch( pGalMap_Impl[ i ].nId )
			{
				case CMP_SORTING:
				{
					const NMSP_CHAOS::SortingInfo aInfo( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "Title" ) ), sal_True );
					aAny <<= SEQ( NMSP_CHAOS::SortingInfo )( &aInfo, 1 );
				}
				break;

				case( CMP_HAS_FOLDER ): aAny <<= (sal_Bool)( mpData->pGal->GetThemeCount() > 0UL ); break;
				case( CMP_IS_FOLDER ): aAny <<= (sal_Bool) sal_True; break;
				case( CMP_MENU ): ImplCreatePopupMenu( aAny ); break;
				case( CMP_OWN_URL ): aAny <<= maUrl; break;
				case( CMP_TARGETS ): aAny <<= OUSTRING( RTL_CONSTASCII_USTRINGPARAM( ";_top;_blank" ) ); break;
				case( CMP_TITLE ): aAny <<= maName; break;
			};

			break;
		}
	}

	Application::GetSolarMutex().release();

	return aAny;
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::addPropertyChangeListener( const OUSTRING& rPropertyName, const REF( NMSP_BEANS::XPropertyChangeListener )& rxListener ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_BEANS::PropertyVetoException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException )
{
	maListeners.addInterface( rxListener );
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::removePropertyChangeListener( const OUSTRING& rPropertyName, const REF( NMSP_BEANS::XPropertyChangeListener )& rxListener ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_BEANS::PropertyVetoException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException )
{
	maListeners.removeInterface( rxListener );
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::addVetoableChangeListener( const OUSTRING& rPropertyName, const REF( NMSP_BEANS::XVetoableChangeListener )& rxListener ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_BEANS::PropertyVetoException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException )
{
}

// ------------------------------------------------------------------------

void SAL_CALL GalComponent::removeVetoableChangeListener( const OUSTRING& rPropertyName, const REF( NMSP_BEANS::XVetoableChangeListener )& rxListener ) throw( NMSP_BEANS::UnknownPropertyException, NMSP_BEANS::PropertyVetoException, NMSP_LANG::IllegalArgumentException, NMSP_LANG::WrappedTargetException, NMSP_UNO::RuntimeException )
{
}

// ------------------------------------------------------------------------

void GalComponent::ImplCreatePopupMenu( ANY& rAny ) const
{
	NMSP_CHAOS::MenuDescriptor			aPopup;
	NMSP_CHAOS::MenuItemDescriptor		aMenu[ 4 ];
	const String						aNewStr( GAL_RESID( RID_SVXSTR_GALLERY_CREATETHEME ) );
	const String						aImportStr( GAL_RESID( RID_SVXSTR_GALLERY_IMPORTTHEME ) );

	aMenu[0].ParentPos = 0;
	aMenu[0].InsertionPos = 0;
	aMenu[0].ItemFlags = CNT_COMPMENU_SEPARATOR;

	aMenu[1].ItemText = aNewStr;
	aMenu[1].Command = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "NewTheme" ) );
	aMenu[1].HelpUrl = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "help.NewTheme.url" ) );
	aMenu[1].ParentPos = 0;
	aMenu[1].InsertionPos = 0;
	aMenu[1].ItemFlags = 0;

	aMenu[2].ItemText = aImportStr;
	aMenu[2].Command = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "ImportTheme" ) );
	aMenu[2].HelpUrl = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "help.ImportTheme.url" ) );
	aMenu[2].ParentPos = 0;
	aMenu[2].InsertionPos = 0;
	aMenu[2].ItemFlags = 0;

	aMenu[3].ParentPos = 0;
	aMenu[3].InsertionPos = 0;
	aMenu[3].ItemFlags = CNT_COMPMENU_SEPARATOR;

	aPopup.Items = SEQ( NMSP_CHAOS::MenuItemDescriptor )( aMenu, 4 );
	rAny <<= aPopup;
}

// ------------------------------------------------------------------------

void GalComponent::ImplHandleOpenEvt( const REF( NMSP_CHAOS::XStatusCallback )& rxJob, const NMSP_CHAOS::StatusCallbackEvent& rEvt )
{
	if( !mpData->pGal )
		LoadGallery();

	ThemeList		aThemeList = mpData->pGal->GetThemeList();
	SgaThemeFile*	pThemeFile = aThemeList.First();

	while( pThemeFile )
	{
		// dont't show hidden themes, if not requested in INI
		if( !pThemeFile->IsHidden() ||
			SFX_INIMANAGER()->Get( SFX_GROUP_COMMON, String( RTL_CONSTASCII_USTRINGPARAM( "ShowHiddenGalleryThemes" ) ) ) == String( RTL_CONSTASCII_USTRINGPARAM( "TRUE" ) ) )
		{
			String aChild( pThemeFile->GetThemeName() );
			rxJob->result( rEvt, 1, aChild += String( RTL_CONSTASCII_USTRINGPARAM( ".Theme" ) ) );
		}

		pThemeFile = aThemeList.Next();
	}
}

// ------------------------------------------------------------------------

void GalComponent::ImplHandleNewEvt( const REF( NMSP_CHAOS::XStatusCallback )& rxJob, const NMSP_CHAOS::StatusCallbackEvent& rEvt )
{
	String aNewThemeName( InsertUniqueTheme() );

	if( aNewThemeName.Len() && mpData->pGal )
		rxJob->result( rEvt, 1, aNewThemeName += String( RTL_CONSTASCII_USTRINGPARAM( ".Theme" ) ) );
}

// ------------------------------------------------------------------------

void GalComponent::ImplHandleImportEvt( const REF( NMSP_CHAOS::XStatusCallback )& rxJob, const NMSP_CHAOS::StatusCallbackEvent& rEvt )
{
	SfxFileDialog	aImportDlg( NULL, WB_OPEN | WB_3DLOOK );
	const String	aFilterName( String( RTL_CONSTASCII_USTRINGPARAM( "SGI - Gallery" ) ) );
	const String	aFilterExt( String( RTL_CONSTASCII_USTRINGPARAM( "*.sgi" ) ) );
	String			aThemeName;
	String			aFileName;

	aImportDlg.AddFilter( aFilterName, aFilterExt );
	aImportDlg.SetText( String( RTL_CONSTASCII_USTRINGPARAM( STAR_GALLERY ) ) );

	if( SgaGallery::aLastImportRoot.Len() )
		aImportDlg.SetPath( SgaGallery::aLastImportRoot );

	if( aImportDlg.Execute() == RET_OK )
	{
		String aFileName( INetURLObject( aImportDlg.GetPath() ).PathToFileName() );

		if( InsertImportFile( aFileName, aThemeName ) )
		{
			rxJob->result( rEvt, 1, aThemeName += String( RTL_CONSTASCII_USTRINGPARAM( ".Theme" ) ) );
			SgaGallery::aLastImportRoot = DirEntry( aFileName ).GetPath().GetPath().GetFull();
		}
	}
}

// ------------------------------------------------------------------------

void GalComponent::Save()
{
	if( !mpData->pGal )
		LoadGallery();

	ErrCode		nErrCode = ERRCODE_NONE;
	String		aTitle( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery" ) ) );
	String		aFileName( maGalConfigPath.GetFull() );
	BOOL		bRet = FALSE;
	const BOOL	bNewFile = FileStat( aFileName ).GetSize() <= 2L;

	SvFileStream aFile( aFileName, STREAM_WRITE | STREAM_TRUNC );

	if( !aFile.GetError() && ( nErrCode == ERRCODE_NONE ) )
	{
		const Date		aDate;
		const Time		aTime;
		const UINT16	nVer = 0x0001;

		aFile << nVer;
		aFile << ByteString( GetTitle(), RTL_TEXTENCODING_UTF8 );

		if ( bNewFile )
		{
			mpData->aGalleryCreateDate = aDate;
			mpData->aGalleryCreateTime = aTime;
		}

		mpData->aGalleryChangeDate = aDate;
		mpData->aGalleryChangeTime = aTime;

		// Erstellungsdatum und -zeit
		aFile << mpData->aGalleryCreateDate;
		aFile << mpData->aGalleryCreateTime;

		// Aenderungsdatum und -zeit
		aFile << mpData->aGalleryChangeDate;
		aFile << mpData->aGalleryChangeTime;

		bRet = TRUE;
	}
}

// ------------------------------------------------------------------------

void GalComponent::LoadGallery()
{
	Application::GetSolarMutex().acquire();
	::LoadGallery();
	mpData->pGal = pGlobalGal;
	mpData->aGalleryTitle = GetTitle();
	Application::GetSolarMutex().release();
}

// ------------------------------------------------------------------------

String GalComponent::InsertUniqueTheme()
{
	String			aOldTheme;
	String			aNewTheme( GAL_RESID( RID_SVXSTR_GALLERY_NEWTHEME ) );
	String			aName( aNewTheme );
	ULONG			nCount = 0;

	if( !mpData->pGal )
		LoadGallery();

	SgaTheme* pThm = mpData->pGal->GetActualTheme();

	// save old theme name
	if( pThm )
		aOldTheme = pThm->GetName();

	while( !mpData->pGal->CreateTheme( aName ) && ( nCount++ < 16000 ) )
	{
		aName = aNewTheme;
		aName += ' ';
		aName += nCount;
	}

	pThm = mpData->pGal->GetActualTheme();

	if( pThm )
	{
		ExchangeData*	pData = new ExchangeData;
		SfxItemSet		aSet( SFX_APP()->GetPool() );
		DirEntry		aFile( pThm->GetThmPath() );
		const FileStat	aFStat( aFile );

		pData->pGal = mpData->pGal;
		pData->aThemeTitle = aName;
		pData->nObjCount = pThm->GetObjCount();
		pData->aThemeCreateDate = aFStat.DateCreated();
		pData->aThemeCreateTime = aFStat.TimeCreated();
		pData->aThemeChangeDate = aFStat.DateModified();
		pData->aThemeChangeTime = aFStat.TimeModified();
		pData->bThemeReadOnly = pThm->IsReadOnly();

		GalleryThemeProperties aThemeProps( NULL, pData, &aSet );

		if( RET_OK == aThemeProps.Execute() )
		{
			if( aName != pData->aThemeTitle )
			{
				const String aOldName( aName );

				aName = pData->aThemeTitle;
				nCount = 0;
				while( mpData->pGal->IsInList( aName ) && ( nCount++ < 16000 ) )
				{
					aName = pData->aThemeTitle;
					aName += ' ';
					aName += nCount;
				}

				mpData->pGal->RenameTheme( aOldName, aName );
			}
		}
		else
		{
			mpData->pGal->RemoveTheme( aName );
			aName.Erase();
		}

		delete pData;

		if( aOldTheme.Len() )
			mpData->pGal->GetTheme( aOldTheme );
	}
	else
		aName.Erase();

	return aName;
}

// ------------------------------------------------------------------------

BOOL GalComponent::CheckImportFile( const String& rFileName, String& rThemeName )
{
	const DirEntry	aBaseFile( rFileName );
	BOOL			bRet = FALSE;

	if ( aBaseFile.GetExtension().CompareIgnoreCaseToAscii( "sgi" ) == COMPARE_EQUAL )
	{
		SvFileStream	aIStm( aBaseFile.GetFull(), STREAM_READ );
		ByteString		aTmpStr;
		String			aImportStr;

		if ( aIStm.IsOpen() )
		{
			aIStm >> aTmpStr; aImportStr = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );

			if ( !aIStm.GetError() && ( aImportStr.Len() < 256 ) &&
				 's' == aImportStr.GetChar( 0 )  && 'g' == aImportStr.GetChar( 1 ) )
			{
				DirEntry aFile( aBaseFile.GetPath() );

				aFile += DirEntry( aImportStr );

				if ( aFile.GetExtension().CompareIgnoreCaseToAscii( "thm" ) == COMPARE_EQUAL )
				{
					DirEntry aTestFile( aFile );

					if( aTestFile.Exists() )
						bRet = TRUE;
					else
					{
						aTestFile.SetName( aFile.GetName().ToUpperAscii() );

						if( aTestFile.Exists() )
							bRet = TRUE;
						else
						{
							aTestFile.SetName( aFile.GetName().ToLowerAscii() );

							if( aTestFile.Exists() )
								bRet = TRUE;
						}
					}

					if( bRet )
						rThemeName = aTestFile.GetFull();
				}
			}
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL GalComponent::InsertImportFile( const String& rFileName, String& rThemeName )
{
	String	aNewFileName;
	BOOL	bRet = FALSE;

	if( CheckImportFile( rFileName, aNewFileName ) )
	{
		SgaTheme* pThm;

		if( !mpData->pGal )
			LoadGallery();

		pThm = mpData->pGal->CreateImportTheme( aNewFileName, DirEntry( rFileName ).GetBase() );

		if( pThm )
		{
			rThemeName = pThm->GetName();
			bRet = TRUE;
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL GalComponent::TitleEdited( const String& rNewTitle )
{
	BOOL bRet = FALSE;

	if( rNewTitle.Len() )
	{
		if( !mpData->pGal )
			LoadGallery();

		SetTitle( mpData->aGalleryTitle = rNewTitle );
		bRet = TRUE;
	}

	return bRet;
}

// ------------------------------------------------------------------------

ExchangeData* GalComponent::GetData()
{
	if( !mpData->pGal )
		LoadGallery();

	return mpData;
}

// ------------------------------------------------------------------------

void GalComponent::EditProperties()
{
	if( !mpData->pGal )
		LoadGallery();

	mpData->nThemeCount = mpData->pGal->GetThemeCount();

	SfxItemSet			aSet( SFX_APP()->GetPool() );
	GalleryProperties	aProps( NULL, mpData, &aSet );

	aProps.Execute();

	if( mpData->aGalleryTitle.Len() && ( mpData->aGalleryTitle != GetTitle() ) )
	{
		SetTitle( mpData->aGalleryTitle );
		Save();
	}
}
