/*************************************************************************
 *
 *  $RCSfile: galcmp3.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "galcmp.hxx"
#include "gallery1.hxx"
#include "gallery2.hxx"
#include "gallery.hrc"
#include "gallery.hrc"

#include <cppuhelper/factory.hxx>
#include <uno/mapping.hxx>
#include <toolkit/helper/vclunohelper.hxx>
#include <vos/mutex.hxx>
#include <vcl/msgbox.hxx>
#include <svtools/cntcmpmn.hxx>
#include <sfx2/iodlg.hxx>
#include "fmmodel.hxx"

// -----------
// - statics -
// -----------

String				aLockThemeName;
extern SgaGallery*	pGlobalGal;
extern List*		pThemeCmpList;
extern ULONG		nGalCount;

// ----------------------
// GalleryContentLoader -
// ----------------------

static REF( NMSP_UNO::XInterface ) SAL_CALL GalleryComponent_CreateInstance( const REF( NMSP_LANG::XMultiServiceFactory )& rxMgr ) throw( NMSP_UNO::RuntimeException )
{
	return( *( new GalComponent() ) );
}

//-------------------------------------------------------------------------

static REF( NMSP_UNO::XInterface ) SAL_CALL GalleryThemeComponent_CreateInstance( const REF( NMSP_LANG::XMultiServiceFactory )& rxMgr ) throw( NMSP_UNO::RuntimeException )
{
	return( *( new GalThemeComponent() ) );
}

//-------------------------------------------------------------------------

static REF( NMSP_UNO::XInterface ) SAL_CALL GalleryContentLoader_CreateInstance( const REF( NMSP_LANG::XMultiServiceFactory )& rxMgr ) throw( NMSP_UNO::RuntimeException )
{
	return( *(new GalleryContentLoader() ) );
}

// ------------------------
// - GalleryContentLoader -
// ------------------------

GalleryContentLoader::GalleryContentLoader()
{
}

// -----------------------------------------------------------------------------

GalleryContentLoader::~GalleryContentLoader()
{
}

// -----------------------------------------------------------------------------

OUSTRING GalleryContentLoader::getImplementationName_Static() throw( NMSP_UNO::RuntimeException )
{
	return OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.svx.GalleryLoader.V10" ) );
}

// -----------------------------------------------------------------------------

SEQ( OUSTRING ) GalleryContentLoader::getSupportedServiceNames_Static() throw( NMSP_UNO::RuntimeException )
{
	SEQ( OUSTRING ) aSNS( 1 );
	
	aSNS.getArray()[ 0 ] = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.frame.GalleryLoader" ) );
	
	return aSNS;
}

// -----------------------------------------------------------------------------

OUSTRING SAL_CALL GalleryContentLoader::getImplementationName() throw( NMSP_UNO::RuntimeException )
{
	return getImplementationName_Static();
}

// -----------------------------------------------------------------------------

sal_Bool SAL_CALL GalleryContentLoader::supportsService( const OUSTRING& rServiceName ) throw( NMSP_UNO::RuntimeException )
{
	SEQ( OUSTRING ) aSNL( getSupportedServiceNames() );
	const OUSTRING* pArray = aSNL.getConstArray();
	
	for( INT32 i = 0; i < aSNL.getLength(); i++ )
		if( pArray[ i ] == rServiceName )
			return TRUE;
	
	return FALSE;
}

// -----------------------------------------------------------------------------

SEQ( OUSTRING )	SAL_CALL GalleryContentLoader::getSupportedServiceNames() throw( NMSP_UNO::RuntimeException )
{
	return getSupportedServiceNames_Static();
}

// -----------------------------------------------------------------------------

void SAL_CALL GalleryContentLoader::load( const REF( NMSP_FRAME::XFrame )& rxFrame, const OUSTRING& rURL, 
										  const SEQ( NMSP_BEANS::PropertyValue )& rArgs, 
										  const REF( NMSP_FRAME::XLoadEventListener )& rxListener ) throw( NMSP_UNO::RuntimeException )
{
	maURL = rURL;
	mxFrame = rxFrame;
	mxListener = rxListener;

	Window* pParent = VCLUnoHelper::GetWindow( mxFrame->getContainerWindow() );
	void*	pThmCmp;

	LoadGallery();

	// select theme that matches URL
	for( pThmCmp = pThemeCmpList->First(); pThmCmp; pThmCmp = pThemeCmpList->Next() )
	{
		if( ( (GalThemeComponent*) pThmCmp )->getURL() == maURL )
		{
			pGlobalGal->GetTheme( ( (GalThemeComponent*) pThmCmp )->GetTitle() );

			// create view with selected theme
			mpWin = new SvxGallery( pParent, pGlobalGal );
			mxFrame->setComponent( VCLUnoHelper::GetInterface( mpWin ), REF( NMSP_FRAME::XController )() );
			mxListener->loadFinished( this );
			break;
		}
	}

	if ( !pThmCmp )
		mxListener->loadCancelled( this );

	UnloadGallery();
}

// -----------------------------------------------------------------------

void SAL_CALL GalleryContentLoader::cancel() throw( NMSP_UNO::RuntimeException )
{
	mxListener = REF( NMSP_FRAME::XLoadEventListener )();
}

// -------
// - UNO -
// -------

extern "C" sal_Bool SAL_CALL writeComponentInfo( uno_Interface* pXUnoKey )
{
	if( pXUnoKey )
	{
		NMSP_UNO::Mapping aUno2Current( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( UNO_LB_UNO ) ),
										OUSTRING( RTL_CONSTASCII_USTRINGPARAM( CPPU_CURRENT_LANGUAGE_BINDING_NAME ) ) );
		
		if( aUno2Current.is() )
		{
			NMSP_REGISTRY::XRegistryKey* pKey = (NMSP_REGISTRY::XRegistryKey*) aUno2Current.mapInterface( pXUnoKey, getCppuType((const REF( NMSP_REGISTRY::XRegistryKey )*) 0 ) );
			
			if( pKey )
			{
				OUSTRING							aImpl;
				REF( NMSP_REGISTRY::XRegistryKey )	xKey( pKey ); pKey->release();
				REF( NMSP_REGISTRY::XRegistryKey )	xNewKey;

				// GalComponent
				xNewKey = xKey->createKey( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( 
						  "/com.sun.star.Gallery.GalComponent/UNO/SERVICES/com.sun.star.Gallery.GalComponent" ) ) );

				// GalComponent
				xNewKey = xKey->createKey( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( 
						  "/com.sun.star.Gallery.GalThemeComponent/UNO/SERVICES/com.sun.star.Gallery.GalThemeComponent" ) ) );

				// GalleryContentLoader
				aImpl = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "/" ) );
				aImpl += GalleryContentLoader::getImplementationName_Static();
				aImpl += OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "/UNO/SERVICES" ) );
				xNewKey = xKey->createKey( aImpl );
				xNewKey->createKey( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.frame.FrameLoader" ) ) );
				xNewKey->createKey( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.frame.GalleryFrameLoader" ) ) );
	
				// FrameLoader
				aImpl = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "/" ) );
				aImpl += GalleryContentLoader::getImplementationName_Static();
				aImpl += OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "/UNO/Loader" ) );
				xNewKey = xKey->createKey( aImpl );
				
				aImpl = OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "/" ) );
				aImpl += GalleryContentLoader::getImplementationName_Static();
				aImpl += OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "/Loader" ) );
				xNewKey = xKey->createKey( aImpl );
				xNewKey = xNewKey->createKey( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( "Pattern" ) ) );
				xNewKey->setAsciiValue( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( ".component:Gallery*" ) ) );
			
				return sal_True;
			}
		}
	}

	return sal_False;
}

// -----------------------------------------------------------------------------

extern "C" uno_Interface* SAL_CALL createComponentFactory( const sal_Unicode* pImplementationName,
														   uno_Interface* pXUnoSMgr, uno_Interface* pXUnoKey )
{
	NMSP_UNO::Mapping aUno2Current( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( UNO_LB_UNO ) ),
									OUSTRING( RTL_CONSTASCII_USTRINGPARAM( CPPU_CURRENT_LANGUAGE_BINDING_NAME ) ) );
	NMSP_UNO::Mapping	aCurrent2Uno( OUSTRING( RTL_CONSTASCII_USTRINGPARAM( CPPU_CURRENT_LANGUAGE_BINDING_NAME ) ),
									  OUSTRING( RTL_CONSTASCII_USTRINGPARAM( UNO_LB_UNO ) ) );
	uno_Interface*	pRet = 0;

	if( aCurrent2Uno.is() && aUno2Current.is() )
	{
		REF( NMSP_LANG::XMultiServiceFactory )	xMgr;
		REF( NMSP_LANG::XSingleServiceFactory )	xFact;
		const OUSTRING							aImplementationName( pImplementationName );
		
		if( pXUnoSMgr )
		{
			NMSP_LANG::XMultiServiceFactory* pSF = (NMSP_LANG::XMultiServiceFactory*) aUno2Current.mapInterface( pXUnoSMgr, getCppuType( (const REF( NMSP_LANG::XMultiServiceFactory )*)0 ) );
			
			if( pSF )
			{
				xMgr = pSF;
				pSF->release();
			}
		}
		
		if( aImplementationName == GalleryContentLoader::getImplementationName_Static() )
		{
			xFact = ::cppu::createSingleFactory( xMgr, aImplementationName,
												 GalleryContentLoader_CreateInstance,
												 GalleryContentLoader::getSupportedServiceNames_Static() );
		}
		else if( aImplementationName.compareToAscii( "com.sun.star.Gallery.GalComponent" ) == COMPARE_EQUAL )
		{
			const OUSTRING aServiceName( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.Gallery.GalComponent" ) );
			
			xFact = ::cppu::createSingleFactory( xMgr, aImplementationName, 
												 GalleryComponent_CreateInstance, 
												 SEQ( OUSTRING)( &aServiceName, 1 ) );
		}
		else if( aImplementationName.compareToAscii( "com.sun.star.Gallery.GalThemeComponent" ) == COMPARE_EQUAL )
		{
			const OUSTRING aServiceName( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.Gallery.GalThemeComponent" ) );
			
			xFact = ::cppu::createSingleFactory( xMgr, aImplementationName, 
												 GalleryThemeComponent_CreateInstance, 
												 SEQ( OUSTRING)( &aServiceName, 1 ) );
		}

		if( xFact.is() )
			pRet = (uno_Interface*) aCurrent2Uno.mapInterface( xFact.get(), getCppuType( (const REF( NMSP_LANG::XSingleServiceFactory )*) 0 ) );
	}

	return pRet;
}

// ----------------
// - CreateLoader -
// ----------------

extern "C" NMSP_FRAME::XFrameLoader* SAL_CALL CreateLoader( const String& rURL )
{
	return new GalleryContentLoader;
}

// -----------------
// - GalleryAccess -
// -----------------

extern "C" BOOL SAL_CALL Gallery_GetPath( DirEntry& rPath )
{
	SvxGallery* pView;
	BOOL		bRet = FALSE;

	LoadGallery();

	if( pGlobalGal && ( ( pView = (SvxGallery*) pGlobalGal->GetCurView() ) != NULL ) )
	{
		rPath = pView->GetPath();
		bRet = TRUE;
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetPathAsString( String& rPath )
{
	SvxGallery* pView;
	BOOL		bRet = FALSE;

	LoadGallery();

	if( pGlobalGal && ( ( pView = (SvxGallery*) pGlobalGal->GetCurView() ) != NULL ) )
	{
		rPath = pView->GetPathAsString();
		bRet = TRUE;
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetFilterName( String& rFilterName )
{
	SvxGallery* pView;
	BOOL		bRet = FALSE;

	LoadGallery();

	if( pGlobalGal && ( ( pView = (SvxGallery*) pGlobalGal->GetCurView() ) != NULL ) )
	{
		rFilterName = pView->GetFilterName();
		bRet = TRUE;
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetGraphic( Graphic& rGraphic )
{
	SvxGallery* pView;
	BOOL		bRet = FALSE;

	LoadGallery();

	if( pGlobalGal && ( ( pView = (SvxGallery*) pGlobalGal->GetCurView() ) != NULL ) )
	{
		rGraphic = pView->GetGraphic();
		bRet = TRUE;
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetVCDrawModel( FmFormModel& rModel )
{
	SvxGallery* pView;
	BOOL		bRet = FALSE;

	LoadGallery();

	if( pGlobalGal && ( ( pView = (SvxGallery*) pGlobalGal->GetCurView() ) != NULL ) )
		bRet = pView->GetFmFormModel( rModel );

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_IsLinkage()
{
	SvxGallery* pView;
	BOOL		bRet = FALSE;

	LoadGallery();

	if( pGlobalGal && ( ( pView = (SvxGallery*) pGlobalGal->GetCurView() ) != NULL ) )
		bRet = pView->IsLinkage();

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_FillThemeList( List& rThemeList )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		ThemeList aThemeList( pGlobalGal->GetThemeList() );

		// in Importthemen koennen wir nichts einfuegen, darum ausklammern
		for( SgaThemeFile* pThmFile = aThemeList.First(); pThmFile; pThmFile = aThemeList.Next() )
			if( !pThmFile->IsReadOnly() && !pThmFile->IsHidden() )
				rThemeList.Insert( new String( pThmFile->GetThemeName() ) );

		bRet = ( aThemeList.Count() > 0 );
	}

	LoadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_FillObjList( const String& rThemeName, List& rObjList )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		String		aOldTheme;
		SgaTheme*	pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aOldTheme = pThm->GetName();

		pThm = pGlobalGal->GetTheme( rThemeName );

		if( pThm )
		{
			ObjectList& rObjectList = (ObjectList&) pThm->GetObjList();

			for( SgaObjList* pObj = rObjectList.First(); pObj; pObj = rObjectList.Next() )
				rObjList.Insert( new String( pObj->aPath ) );

			bRet = TRUE;
		}

		if( aOldTheme.Len() )
			pGlobalGal->GetTheme( aOldTheme );
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_InsertURL( const String& rThemeName, const String& rURL,
												   const ULONG nSgaFormat )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		SvxGallery* pView = (SvxGallery*) pGlobalGal->GetCurView();

		bRet = pGlobalGal->InsertURL( rURL, rThemeName, nSgaFormat );

		if( bRet && pView && ( pGlobalGal->GetActualTheme()->GetName() == rThemeName ) )
			pView->UpdateBrowser( OBJ_POS_LAST );
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" ULONG SAL_CALL Gallery_GetObjCount( const String& rThemeName )
{
	ULONG nRet = 0UL;

	LoadGallery();

	if( pGlobalGal )
	{
		String		aOldTheme;
		SgaTheme*	pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aOldTheme = pThm->GetName();

		pThm = pGlobalGal->GetTheme( rThemeName );

		if( pThm )
			nRet = pThm->GetObjCount();

		if( aOldTheme.Len() )
			pGlobalGal->GetTheme( aOldTheme );
	}

	UnloadGallery();
	return nRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetGraphicObj( const String& rThemeName, ULONG nPos,
													   Graphic* pGraphic, Bitmap* pThumb,
													   BOOL bProgress )
{
	BOOL bRet = FALSE;

	if( pGraphic || pThumb )
	{
		LoadGallery();

		if( pGlobalGal )
		{
			String		aOldTheme;
			SgaTheme*	pThm = pGlobalGal->GetActualTheme();

			if( pThm && ( pThm->GetName() != rThemeName ) )
				aOldTheme = pThm->GetName();

			if( ( pThm = pGlobalGal->GetTheme( rThemeName ) ) != NULL )
			{
				if( pGraphic )
					pGlobalGal->GetGraphicFromActualTheme( nPos, *pGraphic, bProgress );

				if( pThumb )
				{
					SgaObject* pObj = pThm->GetObj( nPos );

					if( pObj && pObj->IsThumbBitmap() )
						*pThumb = pObj->GetThumbBmp();
				}

				bRet = TRUE;
			}

			if( aOldTheme.Len() )
				pGlobalGal->GetTheme( aOldTheme );
		}

		UnloadGallery();
	}

	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_InsertGraphicObj( const String& rThemeName, const Graphic& rGraphic )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		String		aOldTheme;
		String		aGraphicPath;
		SgaTheme*	pThm;

		pGlobalGal->CreateTheme( rThemeName );
		pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aOldTheme = pThm->GetName();

		pThm = pGlobalGal->GetTheme( rThemeName );

		if( pThm && ( bRet = pGlobalGal->SaveGraphic( rGraphic, aGraphicPath ) ) == TRUE )
		{
			const SgaObjectBmp aObjBmp( aGraphicPath );
			pThm->InsertObj( aObjBmp );
		}

		if( aOldTheme.Len() )
			pGlobalGal->GetTheme( aOldTheme );
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" ULONG SAL_CALL Gallery_GetSdrObjCount( const String& rThemeName )
{
	ULONG nRet = 0UL;

	LoadGallery();

	if( pGlobalGal )
	{
		String		aOldTheme;
		SgaTheme*	pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aOldTheme = pThm->GetName();

		pThm = pGlobalGal->GetTheme( rThemeName );

		if( pThm )
		{
			for( ULONG i = 0UL, nCount = pThm->GetObjCount(); i < nCount; i++ )
				if( SGA_OBJ_SVDRAW == pGlobalGal->GetObjKindFromActualTheme( i ) )
					nRet++;
		}

		if( aOldTheme.Len() )
			pGlobalGal->GetTheme( aOldTheme );
	}

	UnloadGallery();
	return nRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetSdrObj( const String& rThemeName, ULONG nSdrModelPos,
												   FmFormModel* pModel, Bitmap* pThumb )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		String		aOldTheme;
		SgaTheme*	pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aOldTheme = pThm->GetName();

		pThm = pGlobalGal->GetTheme( rThemeName );

		if( pThm )
		{
			ULONG nActPos = 0UL;

			for( ULONG i = 0UL, nCount = pThm->GetObjCount(); i < nCount; i++ )
			{
				if( SGA_OBJ_SVDRAW == pGlobalGal->GetObjKindFromActualTheme( i ) )
				{
					if( nActPos++ == nSdrModelPos )
					{
						if( pModel )
						{
							ULONG nStmErr;
							pGlobalGal->GetSdrModelFromActualTheme( i, *pModel, nStmErr );
						}

						if( pThumb )
						{
							SgaObject* pObj = pThm->GetObj( i );

							if( pObj && pObj->IsThumbBitmap() )
								*pThumb = pObj->GetThumbBmp();
						}

						bRet = TRUE;

						break;
					}
				}
			}
		}

		if( aOldTheme.Len() )
			pGlobalGal->GetTheme( aOldTheme );
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_InsertSdrObj( const String& rThemeName, FmFormModel& rModel )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		String		aOldTheme;
		SgaTheme*	pThm;

		pGlobalGal->CreateTheme( rThemeName );
		pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aOldTheme = pThm->GetName();

		pThm = pGlobalGal->GetTheme( rThemeName );

		if( pThm )
		{
			const DirEntry	aFile( pGlobalGal->CreateUniqueFileName( SGA_OBJ_SVDRAW ) );
			SgaObjectSvDraw	aNewObj( rModel, aFile.GetFull() );

			if( pThm->InsertObj( aNewObj ) )
			{
				SvxGallery* pView = (SvxGallery*) pGlobalGal->GetCurView();

				if( pView && ( pGlobalGal->GetActualTheme()->GetName() == rThemeName ) )
					pView->UpdateBrowser( OBJ_POS_LAST );

				bRet = TRUE;
			}
		}

		if( aOldTheme.Len() )
			pGlobalGal->GetTheme( aOldTheme );
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_GetThemeNameFromId( ULONG nThemeId, String& rThemeName )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		SgaThemeFile* pEntry = pGlobalGal->GetEntry( nThemeId );

		if( pEntry )
		{
			rThemeName = pEntry->GetThemeName();
			bRet = TRUE;
		}
	}

	UnloadGallery();
	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_BeginLocking( const String& rThemeName )
{
	BOOL bRet = FALSE;

	LoadGallery();

	if( pGlobalGal )
	{
		SgaTheme* pThm = pGlobalGal->GetActualTheme();

		if( pThm && ( pThm->GetName() != rThemeName ) )
			aLockThemeName = pThm->GetName();
		else
			aLockThemeName.Erase();

		bRet = ( pGlobalGal->GetTheme( rThemeName ) != NULL );
	}

	return bRet;
}

// ------------------------------------------------------------------------

extern "C" BOOL SAL_CALL Gallery_EndLocking( const String& rThemeName )
{
	BOOL bRet = FALSE;

	if( pGlobalGal )
	{
		if( aLockThemeName.Len() )
			pGlobalGal->GetTheme( aLockThemeName );

		bRet = TRUE;
	}

	UnloadGallery();
	return bRet;
}
