/*************************************************************************
 *
 *  $RCSfile: gallery1.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#pragma hdrstop

#include "gallery1.hxx"

#include <stdlib.h>
#include <string.h>
#include <tools/urlobj.hxx>
#include <tools/errinf.hxx>
#include <tools/vcompat.hxx>
#include <vcl/virdev.hxx>
#include <vcl/graph.hxx>
#include <so3/svstor.hxx>
#include <vcl/graph.hxx>
#include <svtools/urihelper.hxx>
#include <svtools/itempool.hxx>
#include <sfx2/docfile.hxx>
#include "fmmodel.hxx"
#include "fmview.hxx"
#include "dialmgr.hxx"
#include "impgrf.hxx"
#include "gallery2.hxx"
#include "gallery.hrc"
#include "codec.hxx"

// ----------------------
// - SgaUserDataFactory -
// ----------------------

IMPL_LINK( SgaUserDataFactory, MakeUserData, SdrObjFactory*, pObjFactory )
{
	if ( pObjFactory->nInventor == IV_IMAPINFO && pObjFactory->nIdentifier == ID_IMAPINFO )
		pObjFactory->pNewData = new SgaIMapInfo;

	return 0L;
}

// ------------------------------------------------------------------------

#ifdef WNT
#pragma optimize ( "", off )
#endif

USHORT SGAImport( const String& rFile, Graphic& rGraphic,
				  String& rFilterName, BOOL bShowProgress, const USHORT nDepth )
{
	USHORT nRet = SGA_IMPORT_NONE;

	// Wir duerfen hier maximal 2-mal reinlaufen (1.INET => 2.reelles File );
	// alles andere fuehrt zu einer unendlichen Rekursion => kein gueltiges File
	if ( nDepth < 2 )
	{
		// INet-Download oder normales Laden
		if ( ( INetURLObject( ::URIHelper::SmartRelToAbs( rFile ) ).GetProtocol() != INET_PROT_FILE ) )
		{
			SfxMedium	aMed( rFile, STREAM_READ, TRUE );
			String		aFilterName;

			aMed.SetTransferPriority( SFX_TFPRIO_VISIBLE_HIGHRES_GRAPHIC | SFX_TFPRIO_SYNCHRON );
			aMed.DownLoad();

			// kleine Rekursion
			if ( SGAImport( aMed.GetPhysicalName(), rGraphic, rFilterName, bShowProgress, nDepth + 1 ) == SGA_IMPORT_FILE )
				nRet = SGA_IMPORT_INET;
		}
		else
		{
			GraphicFilter*		pGraphicFilter = GetGrfFilter();
			GalleryProgress*	pProgress = bShowProgress ? new GalleryProgress( pGraphicFilter ) : NULL;
			USHORT				nFormat;

			if( !pGraphicFilter->ImportGraphic( rGraphic, rFile, GRFILTER_FORMAT_DONTKNOW, &nFormat ) )
			{
				rFilterName = pGraphicFilter->GetImportFormatName( nFormat );
				nRet = SGA_IMPORT_FILE;
			}

			delete pProgress;
		}
	}

	return nRet;
}

#ifdef WNT
#pragma optimize ( "", on )
#endif

// ------------------------------------------------------------------------

BOOL SGASvDrawImport( SvStream& rIStm, FmFormModel& rModel )
{
	BOOL bRet;

	if ( RLECodec::IsRLECoded( rIStm ) )
	{
		SvMemoryStream	aMemStm;
		RLECodec		aCodec( rIStm );

		aCodec.Read( aMemStm );
		aMemStm.Seek( 0UL );
		bRet = SGASvDrawImport( aMemStm, rModel );
	}
	else
	{
		SgaUserDataFactory	aFactory;

		rIStm.SetVersion( SOFFICE_FILEFORMAT_NOW );
		rModel.SetStreamingSdrModel( TRUE );
		rModel.GetItemPool().Load( rIStm );
		rIStm >> rModel;
		rModel.SetStreamingSdrModel( FALSE );
		rModel.GetItemPool().LoadCompleted();
		bRet = ( rIStm.GetError() == 0 );
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL SGAIsSoundFile( const String& rFile )
{
	const String	aExt( DirEntry( rFile ).GetExtension().ToLowerAscii() );
	BOOL			bRet = FALSE;

	if( ( aExt == String( RTL_CONSTASCII_USTRINGPARAM( "wav" ) ) ) ||
		( aExt == String( RTL_CONSTASCII_USTRINGPARAM( "aif" ) ) ) ||
		( aExt == String( RTL_CONSTASCII_USTRINGPARAM( "au" ) ) ) )
		bRet = TRUE;
	else
	{
		// File anlesen
		SvFileStream aIStm( rFile, STREAM_READ );

		if ( aIStm.IsOpen() )
		{
			BYTE cVal1, cVal2, cVal3, cVal4;

			aIStm >> cVal1 >> cVal2 >> cVal3 >> cVal4;

			if ( ( cVal1 == 'R' && cVal2 == 'I' && cVal3 == 'F' && cVal4 == 'F' ) ||
				 ( cVal1 == '.' && cVal2 == 's' && cVal3 == 'n' && cVal4 == 'd' ) )
			{
				bRet = TRUE;
			}
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL CreateIMapGraphic( const FmFormModel& rModel, Graphic& rGraphic, ImageMap& rImageMap )
{
	BOOL bRet = FALSE;

	if ( rModel.GetPageCount() )
	{
		const SdrPage*		pPage = rModel.GetPage( 0 );
		const SdrObject*	pObj = pPage->GetObj( 0 );

		if ( pPage->GetObjCount() == 1 && pObj->ISA( SdrGrafObj ) )
		{
			const USHORT nCount = pObj->GetUserDataCount();

			// gibt es in den User-Daten eine IMap-Information?
			for ( USHORT i = 0; i < nCount; i++ )
			{
				const SdrObjUserData* pUserData = pObj->GetUserData( i );

				if ( ( pUserData->GetInventor() == IV_IMAPINFO ) && ( pUserData->GetId() == ID_IMAPINFO ) )
				{
					rGraphic = ( (SdrGrafObj*) pObj )->GetGraphic();
					rImageMap = ( (SgaIMapInfo*) pUserData )->GetImageMap();
					bRet = TRUE;
					break;
				}
			}
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

String ReduceString( const String& rString )
{
	String aStr;

	if ( rString.Len() > 30 )
	{
		const String aName( DirEntry( rString ).GetName() );

		aStr = rString.Copy( 0, 30 - aName.Len() - 4 );
		aStr += String( RTL_CONSTASCII_USTRINGPARAM( "..." ) );
		aStr += DirEntry::GetAccessDelimiter();
		aStr += aName;
	}
	else
		aStr = rString;

	return aStr;
}

// -------------
// - SgaObject -
// -------------

SgaObject::SgaObject() :
		bIsValid    ( FALSE ),
		bIsThumbBmp	( TRUE )
{
}

// ------------------------------------------------------------------------

BOOL SgaObject::CreateThumb( const Bitmap& rBitmap )
{
	const Size	aBmpSize( rBitmap.GetSizePixel() );
	BOOL		bRet = FALSE;

	if( aBmpSize.Width() && aBmpSize.Height() )
	{
		const float	fFactor  = (float) aBmpSize.Width() / aBmpSize.Height();
		const Size	aNewSize( Max( (long) (fFactor < 1. ? S_THUMB * fFactor : S_THUMB), 8L ),
							  Max( (long) (fFactor < 1. ? S_THUMB : S_THUMB / fFactor), 8L ) );

		aThumbBmp = rBitmap;

		if( aThumbBmp.Scale( (double) aNewSize.Width() / aBmpSize.Width(),
							 (double) aNewSize.Height() / aBmpSize.Height(), BMP_SCALE_INTERPOLATE ) )
		{
			aThumbBmp.Dither( BMP_DITHER_FLOYD );
			bRet = TRUE;
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL SgaObject::CreateThumbMetaFile( const Graphic& rGraphic )
{
	const Size	aMtfSize( rGraphic.GetPrefSize() );
	BOOL		bRet = FALSE;

	if( aMtfSize.Width() && aMtfSize.Height() )
	{
		VirtualDevice*	pVDev = new VirtualDevice;
		Size            aVSize( S_THUMB, S_THUMB );
		Point           aVPos;
		Size            aLogSize;
		const double    fFactor  = (double) aMtfSize.Width() / aMtfSize.Height();
		const Size      aNewSize((USHORT)(fFactor < 1. ? S_THUMB * fFactor : S_THUMB),
								 (USHORT)(fFactor < 1. ? S_THUMB : S_THUMB / fFactor));

		pVDev->SetOutputSizePixel( aNewSize );
		rGraphic.Draw( pVDev, aVPos, aNewSize );

		aThumbBmp = pVDev->GetBitmap( aVPos, aNewSize );
		delete pVDev;

		if( !!aThumbBmp )
		{
			aThumbBmp.Dither( BMP_DITHER_FLOYD );
			bRet = TRUE;
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

void SgaObject::WriteData( SvStream& rOut ) const
{
	static const UINT32 nInventor = COMPAT_FORMAT( 'S', 'G', 'A', '3' );

	rOut << nInventor << (UINT16) 0x0004 << GetVersion() << (UINT16) GetObjKind();
	rOut << bIsThumbBmp;

	if( bIsThumbBmp )
	{
		const USHORT	nOldCompressMode = rOut.GetCompressMode();
		const ULONG		nOldVersion = rOut.GetVersion();

		rOut.SetCompressMode( COMPRESSMODE_ZBITMAP );
		rOut.SetVersion( SOFFICE_FILEFORMAT_NOW );

		rOut << aThumbBmp;

		rOut.SetVersion( nOldVersion );
		rOut.SetCompressMode( nOldCompressMode );
	}
	else
		rOut << aThumbMtf;

	rOut << ByteString( aPath, RTL_TEXTENCODING_UTF8 );
}

// ------------------------------------------------------------------------

void SgaObject::ReadData(SvStream& rIn, UINT16& rReadVersion )
{
	ByteString	aTmpStr;
	UINT32		nTmp32;
	UINT16		nTmp16;

	rIn >> nTmp32 >> nTmp16 >> rReadVersion >> nTmp16 >> bIsThumbBmp;

	if( bIsThumbBmp )
		rIn >> aThumbBmp;
	else
		rIn >> aThumbMtf;

	rIn >> aTmpStr; aPath = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );
}

// ------------------------------------------------------------------------

SvStream& operator<<( SvStream& rOut, const SgaObject& rObj )
{
	rObj.WriteData( rOut );
	return rOut;
}

// ------------------------------------------------------------------------

SvStream& operator>>( SvStream& rIn, SgaObject& rObj )
{
	UINT16 nReadVersion;

	rObj.ReadData( rIn, nReadVersion );
	rObj.bIsValid = ( rIn.GetError() == ERRCODE_NONE );

	return rIn;
}

// ------------------------------------------------------------------------

SgaObjectBmp::SgaObjectBmp()
{
}

// ------------------------------------------------------------------------

SgaObjectBmp::SgaObjectBmp( const String& rFile )
{
	Graphic aGraphic;
	String	aStr;

	aGraphic.SetLink( GfxLink() );

	if ( SGA_IMPORT_NONE != SGAImport( rFile, aGraphic, aStr ) )
		Init( aGraphic, rFile );
}

// ------------------------------------------------------------------------

SgaObjectBmp::SgaObjectBmp( const Graphic& rGraphic, const String& rFile, const String& rFormat )
{
	if( DirEntry( rFile ).Exists() )
		Init( rGraphic, rFile );
}

// ------------------------------------------------------------------------

void SgaObjectBmp::Init( const Graphic& rGraphic, const String& rFile )
{
	aPath = rFile;

	if ( rGraphic.GetType() == GRAPHIC_BITMAP )
		bIsValid = CreateThumb( rGraphic.GetBitmap() );
	else if( rGraphic.GetType() != GRAPHIC_NONE )
		bIsValid = CreateThumbMetaFile( rGraphic );
	else
		bIsValid = FALSE;
}

// ------------------------------------------------------------------------

void SgaObjectBmp::WriteData( SvStream& rOut ) const
{
	String	aDummyStr;
	char	aDummy[ 10 ];

	// Version setzen
	SgaObject::WriteData( rOut );
	rOut.Write( aDummy, 10 );
	rOut << ByteString( aDummyStr, RTL_TEXTENCODING_UTF8 ) << ByteString( aTitle, RTL_TEXTENCODING_UTF8 );
}

// ------------------------------------------------------------------------

void SgaObjectBmp::ReadData( SvStream& rIn, UINT16& rReadVersion )
{
	ByteString aTmpStr;

	SgaObject::ReadData( rIn, rReadVersion );
	rIn.SeekRel( 10 ); // 16, 16, 32, 16
	rIn >> aTmpStr; // dummy

	if( rReadVersion >= 5 )
	{
		rIn >> aTmpStr; aTitle = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );
	}
}

// ------------------------------------------------------------------------

SgaObjectSound::SgaObjectSound() :
	eSoundType( SOUND_STANDARD )
{
}

// ------------------------------------------------------------------------

SgaObjectSound::SgaObjectSound( const String& rFile ) :
	eSoundType( SOUND_STANDARD )
{
	if( DirEntry( rFile ).Exists() )
	{
		aPath = rFile;
		aThumbBmp = Bitmap( Size( 1, 1 ), 1 );
		bIsValid = TRUE;
	}
	else
		bIsValid = FALSE;
}

// ------------------------------------------------------------------------

SgaObjectSound::~SgaObjectSound()
{
}

// ------------------------------------------------------------------------

Bitmap SgaObjectSound::GetThumbBmp() const
{
	USHORT nId;

	switch( eSoundType )
	{
		case( SOUND_COMPUTER ): nId = RID_SVXBMP_GALLERY_SOUND_1; break;
		case( SOUND_MISC ): nId = RID_SVXBMP_GALLERY_SOUND_2; break;
		case( SOUND_MUSIC ): nId = RID_SVXBMP_GALLERY_SOUND_3; break;
		case( SOUND_NATURE ): nId = RID_SVXBMP_GALLERY_SOUND_4; break;
		case( SOUND_SPEECH ): nId = RID_SVXBMP_GALLERY_SOUND_5; break;
		case( SOUND_TECHNIC ): nId = RID_SVXBMP_GALLERY_SOUND_6; break;
		case( SOUND_ANIMAL ): nId = RID_SVXBMP_GALLERY_SOUND_7; break;

		// standard
		default:
			nId = RID_SVXBMP_GALLERY_SOUND_0;
		break;
	}

	return Bitmap( GAL_RESID( nId ) );
}

// ------------------------------------------------------------------------

void SgaObjectSound::WriteData( SvStream& rOut ) const
{
	SgaObject::WriteData( rOut );
	rOut << (UINT16) eSoundType << ByteString( aTitle, RTL_TEXTENCODING_UTF8 );
}

// ------------------------------------------------------------------------

void SgaObjectSound::ReadData( SvStream& rIn, UINT16& rReadVersion )
{
	SgaObject::ReadData( rIn, rReadVersion );

	if( rReadVersion >= 5 )
	{
		ByteString	aTmpStr;
		UINT16		nTmp16;

		rIn >> nTmp16; eSoundType = (GalSoundType) nTmp16;

		if( rReadVersion >= 6 )
		{
			rIn >> aTmpStr; aTitle = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );
		}
	}
}

// ------------------------------------------------------------------------

SgaObjectAnim::SgaObjectAnim()
{
}

// ------------------------------------------------------------------------

SgaObjectAnim::SgaObjectAnim( const Graphic& rGraphic, const String& rFile, const String& rFormatName )
{
	aPath = rFile;
	bIsValid = CreateThumb( rGraphic.GetBitmap() );
}

// ------------------------------------------------------------------------

SgaObjectINet::SgaObjectINet()
{
}

// ------------------------------------------------------------------------

SgaObjectINet::SgaObjectINet( const Graphic& rGraphic, const String& rFile, const String& rFormatName ) :
			SgaObjectAnim	( rGraphic, rFile, rFormatName )
{
}

// ------------------------------------------------------------------------

SgaObjectSvDraw::SgaObjectSvDraw()
{
}

// ------------------------------------------------------------------------

SgaObjectSvDraw::SgaObjectSvDraw( FmFormModel& rModel, const String& rFile )
{
	aPath = DirEntry( rFile ).GetBase();
	bIsValid = CreateThumb( rModel );
}

// ------------------------------------------------------------------------

SgaObjectSvDraw::SgaObjectSvDraw( SvStream& rIStm, const String& rFile )
{
	FmFormModel	aModel;

	aModel.GetItemPool().FreezeIdRanges();

	if( SGASvDrawImport( rIStm, aModel ) )
	{
		aPath = DirEntry( rFile ).GetBase();
		bIsValid = CreateThumb( aModel );
	}
}

// ------------------------------------------------------------------------

BOOL SgaObjectSvDraw::CreateThumb( FmFormModel& rModel )
{
	Graphic		aGraphic;
	ImageMap	aImageMap;
	BOOL		bRet = FALSE;

	// Falls das Draw-Objekt nur eine Graphik enthaelt,
	// erzeugen wir mit dieser den Thumb
	if ( CreateIMapGraphic( rModel, aGraphic, aImageMap ) )
		bRet = SgaObject::CreateThumb( aGraphic.GetBitmap() );
	else
	{
		VirtualDevice	aVDev;
		GDIMetaFile		aMtf;

		aVDev.SetMapMode( MapMode( MAP_100TH_MM ) );
		FmFormView aView( &rModel, &aVDev );

		aView.SetMarkHdlHidden( TRUE );
		aView.ShowPagePgNum( 0, Point() );
		aView.MarkAll();
		aThumbMtf = aView.GetAllMarkedMetaFile();

		const Graphic	aGraphic( aThumbMtf );
		const Size		aMtfSize( aGraphic.GetPrefSize() );
		Point			aVPos;
		Size			aLogSize;
		const double	fFactor  = (double) aMtfSize.Width() / aMtfSize.Height();
		Size            aNewSize( (USHORT) (fFactor < 1. ? S_THUMB * fFactor : S_THUMB ),
								  (USHORT) (fFactor < 1. ? S_THUMB : S_THUMB / fFactor ) );

		aThumbMtf = GDIMetaFile();
		aVDev.SetOutputSizePixel( aNewSize );
		aVDev.SetMapMode( MapMode() );
		aGraphic.Draw( &aVDev, aVPos, aNewSize );
		aThumbBmp = aVDev.GetBitmap( aVPos, aNewSize );
		aThumbBmp.Convert( BMP_CONVERSION_8BIT_COLORS );
		bRet = TRUE;
	}

	return bRet;
}

// ------------------------------------------------------------------------

void SgaObjectSvDraw::WriteData( SvStream& rOut ) const
{
	SgaObject::WriteData( rOut );
	rOut << ByteString( aTitle, RTL_TEXTENCODING_UTF8 );
}

// ------------------------------------------------------------------------

void SgaObjectSvDraw::ReadData( SvStream& rIn, UINT16& rReadVersion )
{
	SgaObject::ReadData( rIn, rReadVersion );

	if( rReadVersion >= 5 )
	{
		ByteString aTmpStr;
		rIn >> aTmpStr; aTitle = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );
	}
}

// ------------------------------------------------------------------------

SvStream& operator<<( SvStream& rOut, const SgaObjList& rObj )
{
	String			aNewPath;
	String			aRelParent;
	String			aTemp;
	const USHORT	nLen = rObj.aPath.Len();
	BOOL			bRel;

	aRelParent = SgaGallery::aRelPathParent;
	aTemp = rObj.aPath;
	bRel = ( ( aTemp.Erase( SgaGallery::nRelPathParentLen ) ).CompareIgnoreCaseToAscii( aRelParent ) == COMPARE_EQUAL );

	if( bRel )
	{
		aTemp = rObj.aPath;
		aNewPath = aTemp.Erase( 0, SgaGallery::nRelPathParentLen );
	}
	else
	{
		aRelParent = SgaGallery::aUserPath;
		aTemp = rObj.aPath;
		bRel = ( ( aTemp.Erase( SgaGallery::nUserPathLen ) ).CompareIgnoreCaseToAscii( aRelParent ) == COMPARE_EQUAL );

		if( bRel )
		{
			aTemp = rObj.aPath;
			aNewPath = aTemp.Erase( 0, SgaGallery::nUserPathLen );
		}
		else
			aNewPath = rObj.aPath;
	}

	rOut << bRel;
	rOut << ByteString( aNewPath, RTL_TEXTENCODING_UTF8 );
	rOut << rObj.nOffset;
	rOut << (UINT16) rObj.eObjKind;

	return rOut;
}

// ------------------------------------------------------------------------

SvStream& operator>>(SvStream& rIn, SgaObjList& rObj)
{
	ByteString	aTmpStr;
	String		aTemp;
	String		aTempPath;
	UINT16		nTemp;
	BOOL		bRel;

	rIn >> bRel;
	rIn >> aTmpStr; aTemp = String( aTmpStr.GetBuffer(), SgaGallery::nReadTextEncoding );
	rIn >> rObj.nOffset;
	rIn >> nTemp; rObj.eObjKind = (SgaObjKind) nTemp;

#if defined ( MAC ) || defined ( UNX )

	const String sOwnDelim = DirEntry::GetAccessDelimiter();
	const String sDOSDelim = DirEntry::GetAccessDelimiter( FSYS_STYLE_MSDOS );
	aTemp.SearchAndReplaceAll( sDOSDelim, sOwnDelim );
#endif

	if( bRel )
	{
		// erst einmal mit dem RelPathParent versuchen
		aTempPath = SgaGallery::aRelPathParent;
		aTempPath += aTemp;

		if( DirEntry( aTempPath ).Exists() )
			rObj.aPath = aTempPath;
		else
		{
			// ansonsten noch einmal im UserPath versuchen
			aTempPath = SgaGallery::aUserPath;
			aTempPath += aTemp;
			rObj.aPath = aTempPath;
		}
	}
	else
		rObj.aPath = aTemp;

	return rIn;
}

// ----------------
// - SgaThemeFile -
// ----------------

SgaThemeFile::SgaThemeFile( const DirEntry& rBasePath, const String& rName,
							UINT32 _nFileNumber, BOOL _bReadOnly, BOOL _bImported,
							BOOL _bNewFile, UINT32 _nId, BOOL _bThemeNameFromResource ) :
		nFileNumber				( _nFileNumber ),
		nId						( _nId ),
		bReadOnly				( _bReadOnly || _bImported ),
		bImported				( _bImported ),
		bThemeNameFromResource	( _bThemeNameFromResource )
{
	ImplSetPath( rBasePath );
	SetModified( _bNewFile );

	if( nId && bThemeNameFromResource )
		aName = String( GAL_RESID( RID_GALLERYSTR_THEME_START + (USHORT) nId ) );

	if( !aName.Len() )
		aName = rName;
}

// -----------------------------------------------------------------------------

String SgaThemeFile::ImplGetFileNameIgnoreCase( const DirEntry& rFile ) const
{
	DirEntry	aTestFile( rFile );
	String		aFileName;
	BOOL		bExists = FALSE;

	// check original file name
	if( aTestFile.Exists() )
		bExists = TRUE;
	else
	{
		// check upper case file name
		aTestFile.SetName( rFile.GetName().ToUpperAscii() );

		if( aTestFile.Exists() )
			bExists = TRUE;
		else
		{
			// check lower case file name
			aTestFile.SetName( rFile.GetName().ToLowerAscii() );

			if( aTestFile.Exists() )
				bExists = TRUE;
		}
	}

	return( bExists ? aTestFile.GetFull() : rFile.GetFull() );
}

// -----------------------------------------------------------------------------

void SgaThemeFile::ImplSetPath( const DirEntry& rPath )
{
	DirEntry	aFile( rPath );
	String		aFileName( String( RTL_CONSTASCII_USTRINGPARAM( "sg" ) ) );

	aFile += DirEntry( ( aFileName += String::CreateFromInt32(nFileNumber) ) += String( RTL_CONSTASCII_USTRINGPARAM( ".thm" ) ) );
	aThmPath = ImplGetFileNameIgnoreCase( aFile );

	aFile.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "sdg" ) ) );
	aSdgPath = ImplGetFileNameIgnoreCase( aFile );

	aFile.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "sdv" ) ) );
	aSdvPath = ImplGetFileNameIgnoreCase( aFile );
}

// -----------------------------------------------------------------------------

void SgaThemeFile::SetName( const String& rNewName )
{
	if( aName != rNewName )
	{
		aName = rNewName;
		SetModified( TRUE );
		bThemeNameFromResource = FALSE;
	}
}

// -----------------------------------------------------------------------------

void SgaThemeFile::SetId( UINT32 nNewId, BOOL bResetThemeName )
{
	nId = nNewId;
	SetModified( TRUE );
	bThemeNameFromResource = ( nId && bResetThemeName );
}

// ------------
// - SgaTheme -
// ------------

SgaTheme::SgaTheme( SgaGallery* pGallery, SgaThemeFile* pThmFile ) :
		pParent ( pGallery ),
		pThm	( pThmFile )
{
	if( pThm->IsImported() )
	{
		aSvDrawStorageRef = new SvStorage();
		aImportName = pThm->GetThemeName();
	}
	else
		aSvDrawStorageRef = new SvStorage( GetSdvPath(), pThm->IsReadOnly() ? STREAM_READ : STREAM_STD_READWRITE );
}

// ------------------------------------------------------------------------

SgaTheme::~SgaTheme()
{
	Write();

	for( SgaObjList* pEntry = aObjectList.First(); pEntry; pEntry = aObjectList.Next() )
		delete pEntry;
}

// ------------------------------------------------------------------------

BOOL SgaTheme::InsertObj( const SgaObject& rObj, ULONG nInsertPos )
{
	BOOL bRet = FALSE;

	if( rObj.IsValid() )
	{
		SgaObjList		aNewEntry;
		SgaObjList*		pOldEntry = aObjectList.First();
		USHORT			nPos = 0xffff;

		// Sehen, ob Eintrag in unserer Liste vorhanden ist
		while( pOldEntry )
		{
			if( pOldEntry->aPath.CompareIgnoreCaseToAscii( rObj.GetPath() ) == COMPARE_EQUAL )
			{
				nPos = (USHORT) aObjectList.GetCurPos();
				break;
			}

			pOldEntry = aObjectList.Next();
		}

		// DateiOffset des alten Eintrags wird angepasst
		if( ( 0xffff != nPos ) && WriteSgaObject( rObj, nInsertPos, &aNewEntry ) )
			pOldEntry->nOffset = aNewEntry.nOffset;
		else
			WriteSgaObject( rObj, nInsertPos, NULL );

		pThm->SetModified( TRUE );
		bRet = TRUE;
	}

	return bRet;
}

// ------------------------------------------------------------------------

SgaObject* SgaTheme::GetObj( ULONG nPos )
{
	return ReadSgaObject( aObjectList.GetObject( nPos ) );
}

// ------------------------------------------------------------------------

BOOL SgaTheme::RemoveObj( ULONG nPos )
{
	SgaObjList* pEntry = aObjectList.Remove( nPos );
	BOOL		bRet = FALSE;

	// Wenn keine Eintraege in Liste vorhanden sind,
	// kann das Objekt-File geloescht werden
	if( !aObjectList.Count() )
	{
		const String aSdgPath( GetSdgPath() );
		DirEntry( aSdgPath ).Kill();
	}

	if( pEntry )
	{
		if( pEntry->eObjKind == SGA_OBJ_SVDRAW )
			aSvDrawStorageRef->Remove( pEntry->aPath );

		delete pEntry;
		pThm->SetModified( TRUE );
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL SgaTheme::ChangeObjPos( ULONG nOldPos, ULONG nNewPos )
{
	BOOL bRet = FALSE;

	if( nOldPos != nNewPos )
	{
		SgaObjList* pEntry = aObjectList.GetObject( nOldPos );

		if( pEntry )
		{
			aObjectList.Insert( pEntry, nNewPos );

			if( nNewPos < nOldPos )
				nOldPos++;

			aObjectList.Remove( nOldPos );
			pThm->SetModified( bRet = TRUE );
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

void SgaTheme::Recover()
{
	DirEntry		aTempDir( DirEntry().TempName() );
	SvFileStream	aIStm( GetSdgPath(), STREAM_READ );
	SvFileStream	aOStm( aTempDir.GetFull(), STREAM_WRITE | STREAM_TRUNC );
	SgaObjList*		pEntry = aObjectList.First();

	while( pEntry )
	{
		switch( pEntry->eObjKind )
		{
			case( SGA_OBJ_BMP ) :
			{
				SgaObjectBmp* pObj = new SgaObjectBmp();
				aIStm.Seek( pEntry->nOffset );
				aIStm >> *pObj;
				pEntry->nOffset = aOStm.Tell();
				aOStm << *pObj;
				delete pObj;
			}
			break;

			case (SGA_OBJ_ANIM) :
			{
				SgaObjectAnim* pObj = new SgaObjectAnim();
				aIStm.Seek( pEntry->nOffset );
				aIStm >> *pObj;
				pEntry->nOffset = aOStm.Tell();
				aOStm << *pObj;
				delete pObj;
			}
			break;

			case (SGA_OBJ_INET) :
			{
				SgaObjectINet* pObj = new SgaObjectINet();
				aIStm.Seek( pEntry->nOffset );
				aIStm >> *pObj;
				pEntry->nOffset = aOStm.Tell();
				aOStm << *pObj;
				delete pObj;
			}
			break;

			case( SGA_OBJ_SVDRAW ) :
			{
				SgaObjectSvDraw* pObj = new SgaObjectSvDraw();
				aIStm.Seek( pEntry->nOffset );
				aIStm >> *pObj;
				pEntry->nOffset = aOStm.Tell();
				aOStm << *pObj;
				delete pObj;
			}
			break;

			case (SGA_OBJ_SOUND) :
			{
				SgaObjectSound* pObj = new SgaObjectSound();
				aIStm.Seek( pEntry->nOffset );
				aIStm >> *pObj;
				pEntry->nOffset = aOStm.Tell();
				aOStm << *pObj;
				delete pObj;
			}
			break;

			default:
			break;
		}

		pEntry = aObjectList.Next();
	}

	aIStm.Close();
	aOStm.Close();
	aTempDir.CopyTo( GetSdgPath(), FSYS_ACTION_COPYFILE );

	if( aTempDir.Exists() )
		aTempDir.Kill();

	// storage updaten (kopieren)
	SvStorageRef aTempStorageRef( new SvStorage( aTempDir.GetFull() ) );
	aSvDrawStorageRef->CopyTo( aTempStorageRef );

	if( !aSvDrawStorageRef->GetError() )
	{
		aSvDrawStorageRef.Clear();
		aTempStorageRef.Clear();
		aTempDir.CopyTo( GetSdvPath(), TRUE );
		aSvDrawStorageRef = new SvStorage( GetSdvPath() );
	}

	if( aTempDir.Exists() )
		aTempDir.Kill();

	pThm->SetModified( TRUE );
	Write();
}

// ------------------------------------------------------------------------

BOOL SgaTheme::WriteSgaObject( const SgaObject& rObj, ULONG nPos, SgaObjList* pExistentEntry )
{
	SvFileStream	aOStm;
	BOOL			bRet = FALSE;

	aOStm.Open( GetSdgPath(), STREAM_WRITE );

	if ( aOStm.IsOpen() )
	{
		const UINT32 nOffset = aOStm.Seek( STREAM_SEEK_TO_END );

		aOStm << rObj;

		if( !aOStm.GetError() )
		{
			SgaObjList* pEntry;

			if( !pExistentEntry )
			{
				pEntry = new SgaObjList;
				aObjectList.Insert( pEntry, nPos );
			}
			else
				pEntry = pExistentEntry;

			pEntry->aPath = rObj.GetPath();
			pEntry->nOffset = nOffset;
			pEntry->eObjKind = rObj.GetObjKind();
			bRet = TRUE;
		}

		aOStm.Close();
	}

	return bRet;
}

// ------------------------------------------------------------------------

SgaObject* SgaTheme::ReadSgaObject( SgaObjList* pEntry )
{
	SgaObject* pSgaObj = NULL;

	if( pEntry )
	{
		SvFileStream aIStm( GetSdgPath(), STREAM_READ );

		if( aIStm.IsOpen() )
		{
			UINT32 nInventor;

			// Ueberpruefen, ob das File ein gueltiges SGA-File ist
			aIStm.Seek( pEntry->nOffset );
			aIStm >> nInventor;

			if( nInventor == COMPAT_FORMAT( 'S', 'G', 'A', '3' ) )
			{
				aIStm.Seek( pEntry->nOffset );

				switch( pEntry->eObjKind )
				{
					case ( SGA_OBJ_BMP ) :
					{
						SgaObjectBmp* pObj = new SgaObjectBmp();
						aIStm >> *pObj;
						pSgaObj = (SgaObject*) pObj;
					}
					break;

					case ( SGA_OBJ_ANIM ) :
					{
						SgaObjectAnim* pObj = new SgaObjectAnim();
						aIStm >> *pObj;
						pSgaObj = (SgaObject*) pObj;
					}
					break;

					case ( SGA_OBJ_INET ) :
					{
						SgaObjectINet* pObj = new SgaObjectINet();
						aIStm >> *pObj;
						pSgaObj = (SgaObject*) pObj;
					}
					break;

					case ( SGA_OBJ_SVDRAW ) :
					{
						SgaObjectSvDraw* pObj = new SgaObjectSvDraw();
						aIStm >> *pObj;
						pSgaObj = (SgaObject*) pObj;
					}
					break;

					case ( SGA_OBJ_SOUND ) :
					{
						SgaObjectSound* pObj = new SgaObjectSound();
						aIStm >> *pObj;
						pSgaObj = (SgaObject*) pObj;
					}
					break;

					default:
					break;
				}
			}
		}
	}

	return pSgaObj;
}

// ------------------------------------------------------------------------

void SgaTheme::Actualize( const Link& rActualizeLink, GalleryProgress* pProgress )
{
	if( !IsReadOnly() && !IsImported() )
	{
		Graphic					aGraphic;
		String					aFormat;
		SgaObjList*				pEntry;
		const USHORT			nEntries = (USHORT) aObjectList.Count();
		USHORT					i;

		bAbortActualize = FALSE;

		// LoeschFlag zuruecksetzen
		for ( i = 0; i < nEntries; i++ )
			aObjectList.GetObject( i )->nExtraFlag = 0L;

		for( i = 0; ( i < nEntries ) && !bAbortActualize; i++ )
		{
			if( pProgress )
				pProgress->Update( i, nEntries - 1 );

			pEntry = aObjectList.GetObject( i );
			aActualizeFile = pEntry->aPath;
			rActualizeLink.Call( this );

			// SvDraw-Objekte werden spaeter aktualisiert
			if ( pEntry->eObjKind != SGA_OBJ_SVDRAW )
			{
				// Hier muss noch etwas eingebaut werden,
				// das Files auf den ensprechenden Eintrag matched
				// Grafiken als Grafik-Objekte in die Gallery aufnehmen
#ifndef MAC
				if ( DirEntry( pEntry->aPath ).GetExtension().CompareIgnoreCaseToAscii( "wav" ) != COMPARE_EQUAL )
#else
				if ( TRUE )
#endif
				{
					// vermeiden, dass ein native Link angelegt wird,
					// da dies an dieser Stelle nicht noetig ist
					aGraphic.Clear();
					aGraphic.SetLink( GfxLink() );

					if ( SGAImport( aActualizeFile, aGraphic, aFormat ) )
					{
						SgaObject* pNewObj;

						if ( SGA_OBJ_INET == pEntry->eObjKind )
							pNewObj = (SgaObject*) new SgaObjectINet( aGraphic, aActualizeFile, aFormat );
						else if ( aGraphic.IsAnimated() )
							pNewObj = (SgaObject*) new SgaObjectAnim( aGraphic, aActualizeFile, aFormat );
						else
							pNewObj = (SgaObject*) new SgaObjectBmp( aGraphic, aActualizeFile, aFormat );

						if( !InsertObj( *pNewObj ) )
							pEntry->nExtraFlag = 1L;

						delete pNewObj;
					}
					else
						pEntry->nExtraFlag = 1L; // Loesch-Flag setzen
				}
				// restliche Sachen als Sound-Objekte aufnehmen
				else
				{
					SgaObjectSound aObjSound( aActualizeFile );
					if( !InsertObj( aObjSound ) )
						pEntry->nExtraFlag = 1L;
				}
			}
			else
			{
				const String aStmStr( DirEntry( pEntry->aPath ).GetBase() );

				if ( aSvDrawStorageRef.Is() )
				{
					SvStorageStreamRef pIStm = aSvDrawStorageRef->OpenStream( aStmStr, STREAM_READ );

					if ( pIStm && !pIStm->GetError() )
					{
						SvMemoryStream	aMemStm;

						pIStm->SetBufferSize( 16384 );

						if ( !RLECodec::IsRLECoded( *pIStm ) )
						{
							const ULONG	nSize = pIStm->Seek( STREAM_SEEK_TO_END );
							void*		pBuffer = SvMemAlloc( nSize );

							pIStm->Seek( STREAM_SEEK_TO_BEGIN );

							if ( pBuffer && nSize )
							{
								pIStm->Read( pBuffer, nSize );

								pIStm->SetBufferSize( 0 );
								pIStm.Clear();
								pIStm = aSvDrawStorageRef->OpenStream( aStmStr, STREAM_WRITE | STREAM_TRUNC );
								pIStm->SetBufferSize( 16384 );
								pIStm->Seek( STREAM_SEEK_TO_BEGIN );

								RLECodec aCodec( *pIStm );

								aMemStm.SetBuffer( (char*) pBuffer, nSize, FALSE, nSize );
								aCodec.Write( aMemStm );

								SgaObjectSvDraw aNewObj( aMemStm, aStmStr );
								if( !InsertObj( aNewObj ) )
									pEntry->nExtraFlag = 1L;
							}

							if ( pBuffer )
								SvMemFree( pBuffer );
						}
						else
						{
							SgaObjectSvDraw aNewObj( *pIStm, aStmStr );
							if( !InsertObj( aNewObj ) )
								pEntry->nExtraFlag = 1L;
						}

						pIStm->SetBufferSize( 0L );
					}
				}
			}

			GetpApp()->Reschedule();
		}

		// alle Eintraege mit gesetztem LoeschFlag
		// aus der ObjektListe entfernen
		pEntry = aObjectList.First();
		while ( pEntry )
		{
			if ( pEntry->nExtraFlag )
			{
				delete aObjectList.Remove( pEntry );
				pEntry = aObjectList.GetCurObject();
			}
			else
				pEntry = aObjectList.Next();
		}

		// Thema komplett neu aufbauen
		Recover();
	}
}

// ------------------------------------------------------------------------

void SgaTheme::Read()
{
	SvFileStream aIStm( GetThmPath(), STREAM_READ );

	if( aIStm.IsOpen() )
		aIStm >> *this;
}

// ------------------------------------------------------------------------

void SgaTheme::Write()
{
	if( IsModified() )
	{
		const DirEntry aFileName( GetThmPath() );
		const DirEntry aFilePath( aFileName.GetPath() );

		if( aFilePath.Exists() || aFilePath.MakeDir() )
		{
#ifdef UNX
			SvFileStream aOStm( aFileName.GetFull(), STREAM_WRITE | STREAM_COPY_ON_SYMLINK | STREAM_TRUNC );
#else
			SvFileStream aOStm( aFileName.GetFull(), STREAM_WRITE | STREAM_TRUNC );
#endif

			if( aOStm.IsOpen() )
				aOStm << *this;

			pThm->SetModified( FALSE );
		}
	}
}

// ------------------------------------------------------------------------

SgaThemeFile* SgaTheme::CreateThemeFile( const DirEntry& rFile, BOOL bReadOnly )
{
	SgaThemeFile* pRet = NULL;

	if( rFile.Exists() )
	{
		SvFileStream	aThmStm( rFile.GetFull(), STREAM_READ );
		String			aThemeName;
		UINT32			nThemeId = 0;
		UINT16			nVersion;
		BOOL			bThemeNameFromResource = FALSE;

		aThmStm >> nVersion;

		if( nVersion <= 0x00ff )
		{
			ByteString aTmpStr;

			aThmStm >> aTmpStr; aThemeName = String( aTmpStr.GetBuffer(), RTL_TEXTENCODING_UTF8 );

			// Charakterkonvertierung durchfuehren
			if( nVersion >= 0x0004 )
			{
				UINT32 nCount;
				UINT16 nTemp16;

				aThmStm >> nCount >> nTemp16;
				aThmStm.Seek( STREAM_SEEK_TO_END );

				// pruefen, ob es sich um eine neuere Version handelt;
				// daher um 520Bytes (8Bytes Kennung + 512Bytes Reserverpuffer ) zurueckspringen,
				// falls dies ueberhaupt moeglich ist
				if( aThmStm.Tell() >= 520 )
				{
					ULONG nId1, nId2;

					aThmStm.SeekRel( -520 );
					aThmStm >> nId1 >> nId2;

					if( nId1 == COMPAT_FORMAT( 'G', 'A', 'L', 'R' ) &&
						nId2 == COMPAT_FORMAT( 'E', 'S', 'R', 'V' ) )
					{
						VersionCompat* pCompat = new VersionCompat( aThmStm, STREAM_READ );

						aThmStm >> nThemeId;

						if( pCompat->GetVersion() >= 2 )
						{
							aThmStm >> bThemeNameFromResource;
						}

						delete pCompat;
					}
				}
			}

			pRet = new SgaThemeFile( rFile.GetPath(), aThemeName,
									 rFile.GetBase().Copy( 2, 6 ).ToInt32(),
									 bReadOnly, FALSE, FALSE, nThemeId,
									 bThemeNameFromResource );
		}
	}

	return pRet;
}

// ------------------------------------------------------------------------

SvStream& operator<<( SvStream& rOut, const SgaTheme& rThm )
{
	rOut << (UINT16) 0x0004; // diesen Id _niemals_ aendern
	rOut << ByteString( rThm.GetRealName(), RTL_TEXTENCODING_UTF8 );
	rOut << (UINT32) rThm.aObjectList.Count();
	rOut << (UINT16) gsl_getSystemTextEncoding();

	for( ULONG i = 0, nCount = rThm.aObjectList.Count(); i < nCount; i++ )
		rOut << *rThm.aObjectList.GetObject( i );

	// neuerdings wird ein 512-Byte-Reservepuffer gechrieben;
	// um diesen zu erkennen werden zwei ULONG-Ids geschrieben
	rOut << COMPAT_FORMAT( 'G', 'A', 'L', 'R' ) << COMPAT_FORMAT( 'E', 'S', 'R', 'V' );

	const long		nReservePos = rOut.Tell();
	VersionCompat*	pCompat = new VersionCompat( rOut, STREAM_WRITE, 2 );

	rOut << (UINT32) rThm.GetId();
	rOut << rThm.IsThemeNameFromResource(); // ab Version 2

	delete pCompat;

	// Rest des Puffers auffuellen
	const long	nRest = Max( 512L - ( (long) rOut.Tell() - nReservePos ), 0L );

	if( nRest )
	{
		char* pReserve = new char[ nRest ];
		memset( pReserve, 0, nRest );
		rOut.Write( pReserve, nRest );
		delete[] pReserve;
	}

	return rOut;
}

// ------------------------------------------------------------------------

SvStream& operator>>( SvStream& rIn, SgaTheme& rThm )
{
	UINT32				nCount;
	UINT16				nVersion;
	ByteString			aTmpStr;
	String				aThemeName;
	SgaObjList*			pEntry;
	rtl_TextEncoding	nTextEncoding;

	rIn >> nVersion;
	rIn >> aTmpStr;
	rIn >> nCount;

	if( nVersion >= 0x0004 )
	{
		UINT16 nTmp16;
		rIn >> nTmp16;
		nTextEncoding = (rtl_TextEncoding) nTmp16;
	}
	else
		nTextEncoding = RTL_TEXTENCODING_UTF8;

	aThemeName = String( aTmpStr.GetBuffer(), nTextEncoding );

	// Falls diese Zahl zu gross ist, handelt es sich um keine gueltige Themendatei
	if( nCount <= ( 1L << 14 ) )
	{
		// Objekt-Liste loeschen
		for( pEntry = rThm.aObjectList.First(); pEntry; pEntry = rThm.aObjectList.Next() )
			delete pEntry;

		rThm.aObjectList.Clear();

		// Neue ObjektListe einlesen
		for( UINT32 i = 0; i < nCount; i++ )
		{
			pEntry = new SgaObjList;
			rIn >> *pEntry;
			rThm.aObjectList.Insert( pEntry, LIST_APPEND );
		}

		ULONG nId1, nId2;
		rIn >> nId1 >> nId2;

		// in neueren Versionen befindet sich am Ende ein 512-Byte-Reservepuffer;
		// die Daten befinden sich am Anfang dieses Puffers und
		// sind durch eine VersionCompat geklammert
		if( !rIn.IsEof() &&
			nId1 == COMPAT_FORMAT( 'G', 'A', 'L', 'R' ) &&
			nId2 == COMPAT_FORMAT( 'E', 'S', 'R', 'V' ) )
		{
			VersionCompat*	pCompat = new VersionCompat( rIn, STREAM_READ );
			UINT32			nTemp32;
			BOOL			bThemeNameFromResource = FALSE;

			rIn >> nTemp32;

			if( pCompat->GetVersion() >= 2 )
			{
				rIn >> bThemeNameFromResource;
			}

			rThm.SetId( nTemp32, bThemeNameFromResource );

			delete pCompat;
		}
	}
	else
		rIn.SetError( SVSTREAM_READ_ERROR );

	return rIn;
}
