/*************************************************************************
 *
 *  $RCSfile: gallery2.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#pragma hdrstop

#undef __PRIVATE

#ifdef UNX
#include <unistd.h>
#else
#include <direct.h>
#endif
#include <stdlib.h>

#include "gallery.hxx"
#include "gallery1.hxx"
#include "gallery2.hxx"
#include "gallery.hrc"

#include <tools/urlobj.hxx>
#include <tools/vcompat.hxx>
#include <vcl/graph.hxx>
#include <vcl/system.hxx>
#include <vcl/drag.hxx>
#include <so3/svstor.hxx>
#include <sot/formats.hxx>
#include <svtools/itempool.hxx>
#include <svtools/filter.hxx>
#include <sfx2/app.hxx>
#include <sfx2/inimgr.hxx>
#include <sfx2/explorer.hxx>
#include <sfx2/viewsh.hxx>
#include "svdetc.hxx"
#include "fmmodel.hxx"
#include "fmview.hxx"
#include "svxurlbk.hxx"
#include "xoutbmp.hxx"
#include "dialmgr.hxx"
#include "codec.hxx"

#ifndef _URLBMK_HXX //autogen
#include <svtools/urlbmk.hxx>
#endif

#define MAXPATH			1024

// --------------
// - SgaGallery -
// --------------

BOOL	SgaGallery::bMultiPath = FALSE;
String	SgaGallery::aRelPathParent;
USHORT	SgaGallery::nRelPathParentLen = 0;
String	SgaGallery::aUserPath;
USHORT	SgaGallery::nUserPathLen = 0;
CharSet	SgaGallery::nReadTextEncoding = gsl_getSystemTextEncoding();
String	SgaGallery::aLastImportRoot;

// ------------------------------------------------------------------------

SgaGallery::SgaGallery( String* pStartUpTheme ) :
		pActualTheme    ( NULL ),
		pCurView		( NULL ),
		nLastFileNumber ( 0 ),
		bDragging		( FALSE )
{
	// Timeout setzen
	aDropTimer.SetTimeout( 100 );

	// alle Themen aus den Verzeichnisse lesen
	ImpLoad();

	// ggf. erstes Thema laden
	if( GetThemeCount() && pStartUpTheme )
		GetTheme( *pStartUpTheme );
}

// ------------------------------------------------------------------------

SgaGallery::~SgaGallery()
{
	// aktuelles Thema schreiben
	delete pActualTheme;

	// Themen-Liste loeschen
	for( SgaThemeFile* pThemeFile = aThemeList.First(); pThemeFile; pThemeFile = aThemeList.Next() )
		delete pThemeFile;

	// Import-Liste loeschen
	for( SgaImportEntry* pImportEntry = aImportList.First(); pImportEntry; pImportEntry = aImportList.Next() )
		delete pImportEntry;

	// View bekanntgeben, dass wir nicht mehr da sind
	if( pCurView && ( ( (SvxGallery*) pCurView )->GetGallery() == this ) )
		( (SvxGallery*) pCurView )->SetGallery( NULL );
}

// ------------------------------------------------------------------------

void SgaGallery::ImpLoad()
{
	const String	aGalleryDir( SFX_INIMANAGER()->Get( SFX_KEY_GALLERY_DIR ) );
	const USHORT	nTokenCount = aGalleryDir.GetTokenCount( ';' );

	// eigentlich haben wir jetzt immer einen Multipath
	// (sofern ein GalleryDir existiert)
	bMultiPath = ( nTokenCount > 0 );

	aUserPath = SFX_INIMANAGER()->Get( SFX_KEY_CONFIG_DIR );
	ImpLoadSubDirs( aUserPath, bMultiPath );

	if( bMultiPath )
	{
		aRelPathParent = aGalleryDir.GetToken( 0, ';' );
		aUserPath = aGalleryDir.GetToken( nTokenCount - 1, ';' );

		// dann die Themen aus den Pfaden
		for( USHORT i = 0UL; i < nTokenCount; i++ )
			ImpLoadSubDirs( aGalleryDir.GetToken( i, ';' ), i < ( nTokenCount - 1 ) );
	}
	else
		aRelPathParent = aGalleryDir;

	nRelPathParentLen = aRelPathParent.Len();
	nUserPathLen = aUserPath.Len();

	// Import-Themen lesen
	ImpLoadImports();
}

// ------------------------------------------------------------------------

void SgaGallery::ImpLoadSubDirs( const DirEntry& rBase, BOOL bReadOnly )
{
	DirEntry	aBase( rBase );
	Dir			aDir( aBase += DirEntry( String( RTL_CONSTASCII_USTRINGPARAM( "*.thm" ) ) ), FSYS_KIND_FILE );

	for( USHORT i = 0, nCount = aDir.Count(); i < nCount; i++ )
	{
		const DirEntry&	rThemeFile = aDir[ i ];
		DirEntry		aSDGFile( rThemeFile ); aSDGFile.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "sdg" ) ) );
		DirEntry		aSDVFile( rThemeFile ); aSDVFile.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "sdv" ) ) );

		bReadOnly = ( FileStat::HasReadOnlyFlag() ) &&
					( FileStat::GetReadOnlyFlag( rThemeFile ) ||
					  FileStat::GetReadOnlyFlag( aSDGFile ) ||
					  FileStat::GetReadOnlyFlag( aSDVFile ) );

		SgaThemeFile* pEntry = SgaTheme::CreateThemeFile( rThemeFile, bReadOnly );

		if( pEntry )
		{
			String		aBase( rThemeFile.GetBase() );
			const ULONG nFileNumber = (ULONG) aBase.Erase( 0, 2 ).Erase( 6 ).ToInt32();

			aThemeList.Insert( pEntry, LIST_APPEND );

			if( nFileNumber > nLastFileNumber )
				nLastFileNumber = nFileNumber;
		}
	}
}

// ------------------------------------------------------------------------

void SgaGallery::ImpLoadImports()
{
	DirEntry aPath( GetUserPath() );

	if( FileStat( aPath += DirEntry( String( RTL_CONSTASCII_USTRINGPARAM( "gallery.sdi" ) ) ) ).GetSize() )
	{
		SvFileStream aInStm( aPath.GetFull(), STREAM_READ );

		if( aInStm.IsOpen() )
		{
			// alte Import-Liste loeschen
			SgaImportEntry* pImportEntry;
			for( pImportEntry = aImportList.First(); pImportEntry; pImportEntry = aImportList.Next() )
				delete pImportEntry;

			aImportList.Clear();

			DirEntry		aFile;
			SgaThemeFile* 	pThemeEntry;
			UINT32			nInventor;
			UINT32			nCount;
			UINT16 			nId;
			UINT16			i;
			UINT16 			nTempCharSet;

			aInStm >> nInventor;

			if( nInventor == COMPAT_FORMAT( 'S', 'G', 'A', '3' ) )
			{
				aInStm >> nId;
				aInStm >> nCount;
				aInStm >> nTempCharSet;

				for( i = 0; i < nCount; i++ )
				{
					pImportEntry = new SgaImportEntry;

					aInStm >> *pImportEntry;
					aImportList.Insert( pImportEntry, LIST_APPEND );
					aFile = DirEntry( pImportEntry->aFileName );
					String aNumberStr( aFile.GetBase() );
					pThemeEntry = new SgaThemeFile( aFile.GetPath(),
													pImportEntry->aUIName,
													aNumberStr.Erase( 0, 2 ).Erase( 6 ).ToInt32(),
													TRUE, TRUE, FALSE, 0, FALSE );

					aThemeList.Insert( pThemeEntry, LIST_APPEND );
				}

				if( aInStm.GetError() )
					ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
			}
		}
	}
}

// ------------------------------------------------------------------------

SgaThemeFile* SgaGallery::GetEntry( const String& rThemeName )
{
	SgaThemeFile* pEntry;

	for( pEntry = aThemeList.First(); pEntry; pEntry = aThemeList.Next() )
		if( pEntry->GetThemeName() == rThemeName )
			break;

	return pEntry;
}

// ------------------------------------------------------------------------

SgaThemeFile* SgaGallery::GetEntry( ULONG nThemeId )
{
	SgaThemeFile* pEntry = NULL;

	if( nThemeId )
		for( pEntry = aThemeList.First(); pEntry; pEntry = aThemeList.Next() )
			if( pEntry->GetId() == nThemeId )
				break;

	// try fallback, if no entry was found
	if( !pEntry )
	{
		String		aFallback;
		ByteString	aTmpStr;

		switch( nThemeId )
		{
			case( GALLERY_THEME_3D ): aTmpStr = "3D"; break;
			case( GALLERY_THEME_BULLETS ): aTmpStr = "Bullets"; break;
			case( GALLERY_THEME_HOMEPAGE ): aTmpStr = "Homepage"; break;
			case( GALLERY_THEME_HTMLBUTTONS	): aTmpStr = "private://gallery/hidden/HtmlExportButtons"; break;
			case( GALLERY_THEME_POWERPOINT ): aTmpStr = "private://gallery/hidden/imgppt"; break;
			case( GALLERY_THEME_RULERS ): aTmpStr = "Rulers"; break;
			case( GALLERY_THEME_SOUNDS ): aTmpStr = "Sounds"; break;

			default:
			break;
		}

		aFallback = String::CreateFromAscii( aTmpStr.GetBuffer() );

		if( aFallback.Len() )
			pEntry = GetEntry( aFallback );
	}

	return pEntry;
}

// ------------------------------------------------------------------------

SgaImportEntry* SgaGallery::GetImportEntry(const String& rImportName)
{
	SgaImportEntry* pImportEntry;

	for( pImportEntry = aImportList.First(); pImportEntry; pImportEntry = aImportList.Next() )
		if ( pImportEntry->aUIName == rImportName )
			break;

	return pImportEntry;
}

// ------------------------------------------------------------------------

SgaTheme* SgaGallery::CreateTheme( const String& rThemeName )
{
	SgaTheme* pNewTheme = NULL;

	if( !IsInList( rThemeName ) )
	{
		// Neuer Eintrag fuer Gallery-Liste
		SgaThemeFile* pNewEntry = new SgaThemeFile( GetUserPath(), rThemeName,
													++nLastFileNumber,
													FALSE, FALSE, TRUE, 0, FALSE );

		aThemeList.Insert( pNewEntry, LIST_APPEND );
		pNewTheme = new SgaTheme( this, pNewEntry );
		pNewTheme->Write();
		delete pActualTheme;
		pActualTheme = pNewTheme;
	}

	return pNewTheme;
}

// ------------------------------------------------------------------------

SgaTheme* SgaGallery::CreateImportTheme( const DirEntry& rDirEntry, const String& rImportName )
{
	DirEntry 	aPath( rDirEntry );
	SgaTheme* 	pImportTheme = NULL;

	if( FileStat( aPath ).GetSize() )
	{
		SvFileStream aInStm( aPath.GetFull(), STREAM_READ );

		if( aInStm.IsOpen() )
		{
			ULONG	nStmErr;
			UINT16	nId;

			aInStm >> nId;

			if( nId > 0x0004 )
			{
				aInStm.Close();
				ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
			}
			else
			{
				ByteString		aTmpStr;
				String			aThemeName; aInStm >> aTmpStr; aThemeName = String( aTmpStr, RTL_TEXTENCODING_UTF8 );
				String			aNumberStr( aPath.GetBase() );
				SgaThemeFile*	pThemeFile = new SgaThemeFile( aPath.GetPath(), rImportName,
															   aNumberStr.Erase( 0, 2 ).Erase( 6 ).ToInt32(),
															   TRUE, TRUE, TRUE, 0, FALSE );

				pImportTheme = new SgaTheme( this, pThemeFile );

				aInStm.Seek( STREAM_SEEK_TO_BEGIN );
				aInStm >> *pImportTheme;
				nStmErr = aInStm.GetError();
				aInStm.Close();

				if( nStmErr )
				{
					delete pThemeFile;
					delete pImportTheme;
					pImportTheme = NULL;
					ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
				}
				else
				{
					String	aName( rImportName );
					String	aNewName( aName );
					ULONG	nCount = 0;

					aName += ' ';

					while ( IsInList( aNewName ) && ( nCount++ < 16000 ) )
					{
						aNewName = aName;
						aNewName += nCount;
					}

					pImportTheme->SetImportName( aNewName );
					aThemeList.Insert( pThemeFile, LIST_APPEND );

					// Thema in Import-Liste eintragen und Import-Liste	speichern
					SgaImportEntry* pImportEntry = new SgaImportEntry;
					pImportEntry->aThemeName = pImportEntry->aUIName = aNewName;
					pImportEntry->aFileName	= aPath.GetFull();
					pImportEntry->aImportName = rImportName;
					aImportList.Insert( pImportEntry, LIST_APPEND );

					WriteImportList();
					delete pActualTheme;
					pActualTheme = pImportTheme;
				}
			}
		}
	}

	return pImportTheme;
}

// ------------------------------------------------------------------------

SgaTheme* SgaGallery::GetTheme( const String& rThemeName )
{
	SgaTheme* pThm = NULL;

	// keine Themen vorhanden
	if( GetThemeCount() )
	{
		if( pActualTheme && ( rThemeName == pActualTheme->GetName() ) )
			pThm = pActualTheme;
		else
		{
			if( !rThemeName.Len() )
			{
				delete pActualTheme;
				pActualTheme = NULL;
			}
			else
			{
				SgaThemeFile* pThemeFile = GetEntry( rThemeName );

				if( pThemeFile )
				{
					DirEntry aPath;

					if( !pThemeFile->IsImported() )
						aPath = pThemeFile->GetThmPath();
					else
						aPath = GetImportPath( rThemeName );

					if( FileStat( aPath ).GetSize() )
					{
						SvFileStream aInStm( aPath.GetFull(), STREAM_READ );

						if( aInStm.IsOpen() )
						{
							pThm = new SgaTheme( this, pThemeFile );
							aInStm >> *pThm;

							if( aInStm.GetError() )
							{
								delete pThm;
								pThm = NULL;
								ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
							}
							else
							{
								if( pThemeFile->IsImported() )
									pThm->SetImportName( rThemeName );

								delete pActualTheme;
								pActualTheme = pThm;
							}
						}
					}
				}
			}
		}
	}

	return pThm;
}
// ------------------------------------------------------------------------

SgaTheme* SgaGallery::GetTheme( ULONG nThemeId )
{
	SgaTheme*		pThm = NULL;
	SgaThemeFile*	pEntry = GetEntry( nThemeId );

	if( pEntry )
		pThm = GetTheme( pEntry->GetThemeName() );

	return pThm;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::IsInList( const String& rThemeName )
{
	return( GetEntry( rThemeName ) != NULL );
}

// ------------------------------------------------------------------------

BOOL SgaGallery::IsInList( ULONG nThemeId )
{
	return( GetEntry( nThemeId ) != NULL );
}

// ------------------------------------------------------------------------

BOOL SgaGallery::RenameTheme(const String& rOldName, const String& rNewName)
{
	BOOL bRet = FALSE;

	// Ueberpruefen, ob neuer Themenname schon vorhanden ist
	if( !IsInList( rNewName ) )
	{
		SgaThemeFile* pEntry = GetEntry( rOldName );

		if( pEntry && ( !pEntry->IsReadOnly() || pEntry->IsImported() ) )
		{
			SgaTheme* pThm = GetTheme( rOldName );

			if( pThm )
			{
				// Thema umbenennen
				pEntry->SetName( rNewName );
				pThm->Write();

				// Import-Liste entspr. anpassen
				if( pEntry->IsImported() )
				{
					pThm->SetImportName( rNewName );

					SgaImportEntry* pImportEntry = GetImportEntry( rOldName );

					if( pImportEntry )
					{
						pImportEntry->aUIName = rNewName;
						WriteImportList();
					}
				}

				bRet = TRUE;
			}
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::RemoveTheme(const String& rThemeName)
{
	SgaThemeFile*	pEntry = GetEntry( rThemeName );
	BOOL			bRet = FALSE;

	if( pEntry && ( !pEntry->IsReadOnly() || pEntry->IsImported() ) )
	{
		if( pEntry->IsImported() )
		{
			SgaImportEntry* pImportEntry = GetImportEntry( rThemeName );

			if( pImportEntry )
			{
				delete aImportList.Remove( pImportEntry );
				WriteImportList();
			}

			// falls dieses geloeschte Importthema das
			// aktuelle Thema ist, wird das aktuelle
			// Thema geloescht
			if( pActualTheme && ( pActualTheme->GetImportName() == rThemeName ) )
			{
				delete pActualTheme;
				pActualTheme= NULL;
			}
		}
		else
		{
			SgaTheme* pThm = GetTheme( rThemeName );

			if( pThm )
			{
				delete pActualTheme;
				pActualTheme= NULL;

				DirEntry( pEntry->GetThmPath() ).Kill();
				DirEntry( pEntry->GetSdgPath() ).Kill();
				DirEntry( pEntry->GetSdvPath() ).Kill();
			}
		}

		// Thema aus Themenliste loeschen
		delete aThemeList.Remove( pEntry );

		// erster Eintrag der Themen-Liste ist neuer aktueller Eintrag
		if( ( pEntry = aThemeList.First() ) != NULL )
			GetTheme( pEntry->GetThemeName() );

		bRet = TRUE;
	}

	return bRet;
}

// ------------------------------------------------------------------------

void SgaGallery::WriteImportList()
{
	SvFileStream	aOutStm;
	DirEntry        aPath( GetUserPath() );

	aOutStm.Open( ( aPath += DirEntry( String( "gallery.sdi", RTL_TEXTENCODING_UTF8 ) ) ).GetFull(), STREAM_WRITE | STREAM_TRUNC );

	if( aOutStm.IsOpen() )
	{
		const UINT32 nInventor = (UINT32) COMPAT_FORMAT( 'S', 'G', 'A', '3' );
		const UINT16 nId = 0x0004;

		aOutStm << nInventor << nId << (UINT32) aImportList.Count() << (UINT16) gsl_getSystemTextEncoding();

		for( SgaImportEntry* pImportEntry = aImportList.First(); pImportEntry; pImportEntry = aImportList.Next() )
			aOutStm << *pImportEntry;

		if( aOutStm.GetError() )
			ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
	}
}

// ------------------------------------------------------------------------

BOOL SgaGallery::IsThemeImported(const String& rThemeName)
{
	SgaThemeFile* pEntry = GetEntry( rThemeName );

	return( pEntry ? pEntry->IsImported() : FALSE );
}

// ------------------------------------------------------------------------

BOOL SgaGallery::IsThemeReadOnly(const String& rThemeName)
{
	SgaThemeFile* pEntry = GetEntry( rThemeName );

	return( pEntry ? pEntry->IsReadOnly() : FALSE );
}

// ------------------------------------------------------------------------

BOOL SgaGallery::IsThemeDefault(const String& rThemeName)
{
	SgaThemeFile* pEntry = GetEntry( rThemeName );

	return( pEntry ? pEntry->IsDefault() : FALSE );
}

// ------------------------------------------------------------------------

DirEntry SgaGallery::GetImportPath(const String& rThemeName)
{
	DirEntry		aPath;
	SgaImportEntry* pImportEntry = GetImportEntry( rThemeName );

	if( pImportEntry )
		aPath = DirEntry( pImportEntry->aFileName );

	return aPath;
}

// ------------------------------------------------------------------------

String SgaGallery::GetImportFileName(const String& rThemeName)
{
	String aFileName;

	SgaImportEntry* pImportEntry = GetImportEntry(rThemeName);

	if( pImportEntry )
		aFileName = pImportEntry->aImportName;

	return aFileName;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::IsImportThemeAvailable( const String& rImportTheme )
{
	SgaImportEntry* pImportEntry = GetImportEntry( rImportTheme );
	BOOL			bRet = FALSE;

	if( pImportEntry && FileStat( pImportEntry->aFileName ).GetSize() )
	{
		// Anfangskennung ueberpruefen
		SvFileStream aInStm( pImportEntry->aFileName, STREAM_READ );

		if( aInStm.IsOpen() )
		{
			UINT16 nId;

			aInStm >> nId;

			if( nId <= 0x0004 )
				bRet = TRUE;
		}
	}

	return bRet;
}


// ------------------------------------------------------------------------

BOOL SgaGallery::SetImportPath( const String& rImportTheme, const DirEntry& rNewPath )
{
	SgaImportEntry* pImportEntry = GetImportEntry( rImportTheme );
	BOOL			bRet = FALSE;

	if( pImportEntry )
	{
		SgaThemeFile* pThemeFile = GetEntry( rImportTheme );

		pImportEntry->aFileName	= rNewPath.GetFull();
		WriteImportList();

		if( pThemeFile )
			pThemeFile->SetBasePath( rNewPath.GetPath() );

		bRet = TRUE;
	}

	return bRet;
}


// ------------------------------------------------------------------------

String SgaGallery::GetPathFromActualTheme( const ULONG nObjPos )
{
	String		aPath;
	SgaObjList*	pEntry = GetObjFromActualTheme( nObjPos );

	if ( pEntry )
		aPath = GetCorrectPath( pEntry );

	return aPath;
}


// ------------------------------------------------------------------------

SgaObjKind SgaGallery::GetObjKindFromActualTheme( const ULONG nObjPos )
{
	SgaObjList*	pEntry = GetObjFromActualTheme( nObjPos );

	return ( pEntry ? pEntry->eObjKind : SGA_OBJ_NONE );
}


// ------------------------------------------------------------------------

SgaObjList* SgaGallery::GetObjFromActualTheme( ULONG nObjPos )
{
	return pActualTheme->GetObjList().GetObject( nObjPos );
}


// ------------------------------------------------------------------------

String SgaGallery::GetCorrectPath( SgaObjList* pEntry )
{
	String aFilePath;

	if( pEntry )
	{
		if ( pActualTheme->IsImported() )
		{
			DirEntry aTmp( GetImportPath( pActualTheme->GetName() ).GetPath() );
			aFilePath = ( aTmp += DirEntry( pEntry->aPath ).GetName() ).GetFull();
		}
		else
			aFilePath = pEntry->aPath;
	}

	return aFilePath;
}

#ifdef WNT
#pragma optimize ( "", off )
#endif

// ------------------------------------------------------------------------

DirEntry SgaGallery::CreateUniqueFileName( SgaObjKind eObjKind ) const
{
	DirEntry aTmpPath( FSYS_FLAG_INVALID );

	SvFileStream	aStm;
	DirEntry		aPath( GetUserPath() );
	String			aTmpStr;
	String			aFName( RTL_CONSTASCII_USTRINGPARAM( "dd" ) );
	UINT32			nNumber;

	aTmpPath = aPath;
	aTmpPath += DirEntry( String( RTL_CONSTASCII_USTRINGPARAM( "sdddndx1" ) ) );
	aTmpStr = aTmpPath.GetFull();

	if( aTmpPath.Exists() )
	{
		aStm.Open( aTmpStr, STREAM_READ );
		aStm >> nNumber;
		aStm.Close();
	}
	else
		nNumber = 1999;

	aFName += (ULONG) ( ++nNumber % ( eObjKind == SGA_OBJ_SVDRAW ? 99999999L : 999999L ) );

	aStm.Open( aTmpStr, STREAM_WRITE | STREAM_TRUNC );
	if ( aStm.IsOpen() )
	{
		aStm << nNumber;
		aStm.Close();
	}

	return DirEntry( aFName += String( RTL_CONSTASCII_USTRINGPARAM( ".svm" ) ) );
}

// ------------------------------------------------------------------------

BOOL SgaGallery::InsertFiles( List& rFileList, GalleryProgress* pProgress, ULONG nPos )
{
	SgaTheme*	pThm = pActualTheme;
	BOOL		bRet = FALSE;

	if( pThm )
	{
		ULONG nCounter = 0;

		for( String* pFile = (String*) rFileList.First(); pFile; pFile = (String*) rFileList.Next() )
		{
			Graphic	aGraphic;

			// vermeiden, dass ein native Link angelegt wird,
			// da dies an dieser Stelle nicht noetig ist
			aGraphic.SetLink( GfxLink() );

			String			aFormat;
			const USHORT	nImportRet = SGAImport( *pFile, aGraphic, aFormat );
			SgaObject*		pNewObj = NULL;

			if( pProgress )
				pProgress->Update( ++nCounter, rFileList.Count() );

			if( nImportRet != SGA_IMPORT_NONE )
			{
				if ( SGA_IMPORT_INET == nImportRet )
					pNewObj = (SgaObject*) new SgaObjectINet( aGraphic, *pFile, aFormat );
				else if ( aGraphic.IsAnimated() )
					pNewObj = (SgaObject*) new SgaObjectAnim( aGraphic, *pFile, aFormat );
				else
					pNewObj = (SgaObject*) new SgaObjectBmp( aGraphic, *pFile, aFormat );
			}
			else if( SGAIsSoundFile( *pFile ) )
				pNewObj = (SgaObject*) new SgaObjectSound( *pFile );

			if( pNewObj && pThm->InsertObj( *pNewObj, nPos ) )
				bRet = TRUE;

			delete pNewObj;
		}

		pThm->Write();
	}

	return bRet;
}

// ------------------------------------------------------------------------

#ifdef WNT
#pragma optimize ( "", off )
#endif

BOOL SgaGallery::GetGraphicFromActualTheme( const ULONG nObjPos, Graphic& rGraphic, BOOL bProgress )
{
	DBG_ASSERT( pActualTheme, "Kein aktuelles Thema vorhanden!" );

	SgaObjList*	pEntry = GetObjFromActualTheme( nObjPos );
	SgaObject* 	pObj = pActualTheme->GetObj( nObjPos );
	BOOL		bRet = FALSE;

	if( pObj && pEntry )
	{
		const String aPath( GetCorrectPath( pEntry ) );

		switch( pObj->GetObjKind() )
		{
			case( SGA_OBJ_BMP ):
			case( SGA_OBJ_ANIM ):
			case( SGA_OBJ_INET ):
			{
				String aFilterDummy;

				bRet = SGAImport( aPath, rGraphic, aFilterDummy, bProgress ) != SGA_IMPORT_NONE;

				if( !bRet )
					ErrorHandler::HandleError( ERRCODE_IO_NOTEXISTSPATH );
			}
			break;

			case( SGA_OBJ_SVDRAW ):
			{
				FmFormModel	aModel;
				ULONG		nStmErr;

				aModel.GetItemPool().FreezeIdRanges();

				// zunaechst versuchen, ob wir nicht ein einfaches
				// Grafik-Objekt bekommen koennen
				if( GetSdrModelFromActualTheme( nObjPos, aModel, nStmErr ) )
				{
					ImageMap aIMap;

					if( CreateIMapGraphic( aModel, rGraphic, aIMap ) )
						bRet = TRUE;
					else
					{
						VirtualDevice aVDev;
						aVDev.SetMapMode( MapMode( MAP_100TH_MM ) );
						FmFormView aView( &aModel, &aVDev );

						aView.SetMarkHdlHidden( TRUE );
						aView.ShowPagePgNum( 0, Point() );
						aView.MarkAll();
						rGraphic = aView.GetAllMarkedMetaFile();
						bRet = TRUE;
					}
				}
				else
				{
					ErrorHandler::HandleError( nStmErr );
				}
			}
			break;

			case( SGA_OBJ_SOUND ):
			{
				rGraphic = BitmapEx( pObj->GetThumbBmp(), COL_LIGHTMAGENTA );
				bRet = TRUE;
			}
			break;

			default:
			break;
		}

		delete pObj;
	}
	else
		delete pObj;

	return bRet;
}

#ifdef WNT
#pragma optimize ( "", on )
#endif


// ------------------------------------------------------------------------

BOOL SgaGallery::GetSdrModelFromActualTheme( const ULONG nObjPos, FmFormModel& rModel, ULONG& rStmErr )
{
	DBG_ASSERT( pActualTheme, "Kein aktuelles Thema vorhanden!" );

	SgaObjList*	pEntry = GetObjFromActualTheme( nObjPos );
	SgaObject* 	pObj = pActualTheme->GetObj( nObjPos );
	BOOL		bRet = FALSE;

	if( pObj && pEntry && pObj->GetObjKind() == SGA_OBJ_SVDRAW )
	{
		const DirEntry	aFilePath( GetCorrectPath( pEntry ) );
		SvStorageRef	aStor = pActualTheme->GetSvDrawStorage();

		if( aStor.Is() )
		{
			SvStorageStreamRef pIStm = aStor->OpenStream( aFilePath.GetBase(), STREAM_READ );

			if( pIStm && !pIStm->GetError() )
			{
				pIStm->SetBufferSize( STREAMBUF_SIZE );
				bRet = SGASvDrawImport( *pIStm, rModel );
				pIStm->SetBufferSize( 0L );
			}

			rStmErr = ( pIStm ? pIStm->GetError() : ERRCODE_IO_GENERAL );
		}
		else
			rStmErr = ERRCODE_IO_GENERAL;

		delete pObj;
	}
	else
	{
		delete pObj;
		rStmErr = ERRCODE_IO_GENERAL;
	}

	return bRet;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::SaveGraphic( const Graphic& rGraphic, String& rGraphicPath )
{
	BOOL		bRet = FALSE;
	DirEntry	aGrfName;
	BOOL		bGrfDir = TRUE;

	if( bMultiPath )
		aGrfName = SgaGallery::aUserPath;
	else
		aGrfName = SgaGallery::aRelPathParent;

	aGrfName += DirEntry( String( RTL_CONSTASCII_USTRINGPARAM( "dragdrop" ) ) );

	if ( !aGrfName.Exists() )
		bGrfDir = aGrfName.MakeDir();

	if( bGrfDir )
	{
		aGrfName += CreateUniqueFileName( SGA_OBJ_BMP ).GetBase();

		String aPath;

		// MetaFiles als SVM schreiben
		if ( rGraphic.GetType() != GRAPHIC_BITMAP )
		{
			SvFileStream aOStm;

			aGrfName.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "svm" ) ) );
			aPath = aGrfName.GetFull();
			aOStm.Open( aPath, STREAM_WRITE | STREAM_TRUNC );

			if( aOStm.IsOpen() )
			{
				aOStm.SetVersion( SOFFICE_FILEFORMAT_NOW );
				aOStm << rGraphic.GetGDIMetaFile();
				bRet = ( aOStm.GetError() == 0 );
				aOStm.Close();
			}
		}
		else
		{
			INetURLObject aObj;

			// File-URL fuer WriteGraphic erzeugen
			aObj.SetSmartProtocol( INET_PROT_FILE );
			aPath = aGrfName.GetFull();
			aObj.SetURL( aPath );
			aPath = aObj.GetMainURL();

			if ( XOutBitmap::WriteGraphic( rGraphic, aPath, String( RTL_CONSTASCII_USTRINGPARAM( "bmp" ) ), XOUTBMP_DONT_EXPAND_FILENAME | XOUTBMP_USE_GIF_IF_SENSIBLE ) )
				ErrorHandler::HandleError( ERRCODE_IO_GENERAL );
			else
				bRet = TRUE;
		}

		if( bRet )
			rGraphicPath = aPath;
	}

	return bRet;
}

// -----------------------------------------------------------------------------

SvDataTypeList SgaGallery::GetDragTypeList( const SvDataTypeList& rTypeList, ULONG nObjPos )
{
	SvDataTypeList	aTypeList( rTypeList );
	SgaObjList*		pEntry = GetObjFromActualTheme( nObjPos );

	if ( pEntry )
	{
		if ( pEntry->eObjKind == SGA_OBJ_SVDRAW )
		{
			FmFormModel	aModel;
			Graphic		aGraphic;
			ImageMap	aIMap;
			ULONG		nStmErr;

			aModel.GetItemPool().FreezeIdRanges();

			// Falls Model nur eine Graphic mit IMap enthaelt
			// stellen wir diese ins Clipboard
			if ( GetSdrModelFromActualTheme( nObjPos, aModel, nStmErr ) &&
				 CreateIMapGraphic( aModel, aGraphic, aIMap ) )
			{
				aTypeList.Insert( SvDataType( SOT_FORMATSTR_ID_SVXB, MEDIUM_STREAM | MEDIUM_MEMORY ), LIST_APPEND );
				aTypeList.Insert( SvDataType( SOT_FORMATSTR_ID_SVIM, MEDIUM_STREAM | MEDIUM_MEMORY ), LIST_APPEND );
			}
			else
				aTypeList.Insert( SvDataType( SOT_FORMATSTR_ID_DRAWING ), LIST_APPEND );
		}
		else
		{
			aTypeList.Insert( SvDataType( FORMAT_FILE, MEDIUM_MEMORY ), LIST_APPEND );

			if ( ( pEntry->eObjKind == SGA_OBJ_BMP ) || ( pEntry->eObjKind == SGA_OBJ_ANIM ) )
			{
				aTypeList.Insert( SvDataType( SOT_FORMATSTR_ID_SVXB, MEDIUM_STREAM | MEDIUM_MEMORY ), LIST_APPEND );
				aTypeList.Insert( SvDataType( FORMAT_GDIMETAFILE, MEDIUM_MEMORY ), LIST_APPEND );
				aTypeList.Insert( SvDataType( FORMAT_BITMAP, MEDIUM_MEMORY ), LIST_APPEND );
			}
		}
	}

	return aTypeList;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::GetDragData( SvData* pData, ULONG nFormat, ULONG nObjPos )
{
	BOOL		bRet = FALSE;

	if ( pData )
	{
		const String aOldTheme( GetActualTheme()->GetName() );

		// Thema, aus dem gedragt wurde, laden
		GetTheme( aDragParam.aThemeName );

		if ( nFormat == SOT_FORMATSTR_ID_SVXB )
		{
			Graphic aGraphic;

			GetGraphicFromActualTheme( nObjPos, aGraphic );
			pData->SetData( (SvDataCopyStream*) &aGraphic );
			bRet = TRUE;
		}
		else if ( nFormat == SOT_FORMATSTR_ID_SVIM )
		{
			FmFormModel	aModel;
			Graphic		aGraphic;
			ImageMap	aIMap;
			ULONG		nStmErr;

			aModel.GetItemPool().FreezeIdRanges();

			// Falls Model nur eine Graphic mit IMap enthaelt
			// stellen wir diese ins Clipboard
			if ( GetSdrModelFromActualTheme( nObjPos, aModel, nStmErr ) &&
				 CreateIMapGraphic( aModel, aGraphic, aIMap ) )
			{
				pData->SetData( (SvDataCopyStream*) &aIMap );
				bRet = TRUE;
			}
		}
		else if ( nFormat == SOT_FORMATSTR_ID_DRAWING )
		{
			DirEntry aTmpDirEntry( GetPathFromActualTheme( nObjPos ) );
			const String aStmStr( aTmpDirEntry.GetBase() );
			SvStorageRef aStor = GetActualTheme()->GetSvDrawStorage();

			if ( aStor.Is() )
			{
				SvStorageStreamRef pIStm = aStor->OpenStream( aStmStr, STREAM_READ );

				if ( pIStm && !pIStm->GetError() )
				{
					pIStm->SetBufferSize( 16384 );

					// kodiert oder altes Verhalten?
					if ( RLECodec::IsRLECoded( *pIStm ) )
					{
						SvMemoryStream	aMemStm;
						RLECodec		aCodec( *pIStm );
						ULONG			nSize;

						nSize = aCodec.Read( aMemStm );
						pData->SetData( (void*) aMemStm.GetData(), nSize, TRANSFER_COPY );
						bRet = TRUE;
					}
					else
					{
						const ULONG	nSize = pIStm->Seek( STREAM_SEEK_TO_END );
						void*		pBuffer = SvMemAlloc( nSize );

						if ( pBuffer && nSize )
						{
							pIStm->Seek( STREAM_SEEK_TO_BEGIN );
							pIStm->Read( pBuffer, nSize );
							pData->SetData( pBuffer, nSize, TRANSFER_COPY );
							bRet = TRUE;
						}

						if ( pBuffer )
							SvMemFree( pBuffer );
					}

					pIStm->SetBufferSize( 0L );
				}
			}
		}
		else
		{
			switch (nFormat)
			{
				case (FORMAT_FILE):
				{
					const String aFile( GetPathFromActualTheme( nObjPos ) );

					pData->SetData( aFile );
					bRet = TRUE;
				}
				break;


				case (FORMAT_BITMAP):
				{
					Graphic aGraphic;

					GetGraphicFromActualTheme( nObjPos, aGraphic );
					Bitmap aBmp( aGraphic.GetBitmap() );
					pData->SetData( &aBmp );
					bRet = TRUE;
				}
				break;


				case (FORMAT_GDIMETAFILE):
				{
					Graphic aGraphic;

					GetGraphicFromActualTheme( nObjPos, aGraphic );
					GDIMetaFile aMtf( aGraphic.GetGDIMetaFile() );
					pData->SetData( &aMtf );
					bRet = TRUE;
				}
				break;

				default:
				break;
			}
		}

		// altes Thema wieder herstellen
		GetTheme( aOldTheme );
	}

	return bRet;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaGallery, StartDragHdl, void*, p )
{
	if ( aDragParam.eObjKind == SGA_OBJ_SVDRAW ||
		 aDragParam.eObjKind == SGA_OBJ_INET ||
		 FileStat( aDragParam.aFileName ).GetSize() )
	{
		SgaDataObjectRef	pDataObject = new SgaDataObject( this, aDragParam.nDragObjPos );
		SgaTheme*			pThm = GetActualTheme();
		const String		aOldTheme( pThm->GetName() );

		const DropAction eAction = pDataObject->ExecuteDrag( NULL,
											POINTER_MOVEDATA, POINTER_COPYDATA, POINTER_LINKDATA,
											DRAG_COPYABLE | DRAG_LINKABLE /*| DRAG_MOVEABLE*/,
											&aDragParam.aDragRegion );
		GetTheme( aDragParam.aThemeName );
	}
	else
		ErrorHandler::HandleError( ERRCODE_IO_NOTEXISTSPATH );

	// Focus auf das EditWindow setzen
	SfxViewShell*	pShell = SfxViewShell::Current();
	Window*			pWin = pShell ? pShell->GetWindow() : NULL;

	if( pWin )
		SFX_APP()->GrabFocus( pWin );

	return 0;
}


#ifdef WNT
#pragma optimize ( "", off )
#endif

// ------------------------------------------------------------------------

BOOL SgaGallery::DoQueryDrop( DropEvent& rDEvt )
{
	BOOL bRet = IsDragging();

	if ( !bRet )
	{
		USHORT		nCount = DragServer::GetItemCount();
		USHORT		i = 0;

		while ( !bRet && ( i < nCount ) )
		{
			bRet = DragServer::HasFormat( i, FORMAT_BITMAP ) ||
				   DragServer::HasFormat( i, FORMAT_GDIMETAFILE ) ||
				   DragServer::HasFormat( i, FORMAT_FILE ) ||
				   DragServer::HasFormat( i, SOT_FORMATSTR_ID_SVXB ) ||
				   DragServer::HasFormat( i, SOT_FORMATSTR_ID_DRAWING ) ||
				   INetBookmark::DragServerHasFormat( i );

			i++;
		}

		if ( rDEvt.IsDefaultAction() )
			rDEvt.SetAction( DROP_COPY );
	}
	else if ( rDEvt.IsDefaultAction() )
		rDEvt = DropEvent( rDEvt.GetPosPixel(), rDEvt.GetData(), DROP_MOVE, DROP_MOVE, rDEvt.GetWindowType(), rDEvt.IsDefaultAction() );

	return bRet;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::DoDrop( const DropEvent& rDEvt, const String& rThemeName, List& rFileList, ULONG nInsertPos )
{
	BOOL bRet = FALSE;

	if ( !IsDragging() )
	{
		SgaTheme*		pThm = GetActualTheme();
		const String	aOldTheme( pThm->GetName() );
		BOOL			bToggle = FALSE;

		if ( aOldTheme != rThemeName )
		{
			pThm = GetTheme( rThemeName );
			bToggle = TRUE;
		}

		if ( pThm )
		{
			SvDataObjectRef			aDataObj = SvDataObject::PasteDragServer( rDEvt );
			SvDataType				aIMapType( SOT_FORMATSTR_ID_SVIM, MEDIUM_STREAM | MEDIUM_MEMORY );
			const SvDataTypeList&	rTypeList = aDataObj->GetTypeList();
			const BOOL				bImageMap = rTypeList.HasEqualType( aIMapType );
			BOOL					bURL = FALSE;

			aDropThemeName = rThemeName;
			nDropPos = nInsertPos;

			// ggf. ImageMap besorgen
			if ( bImageMap )
			{
				SvData		aData( SOT_FORMATSTR_ID_SVIM );
				ImageMap*	pIMap = NULL;

				if ( aDataObj->GetData( &aData ) )
				{
					aData.GetData( (SvDataCopyStream**) &pIMap, ImageMap::StaticType(), TRANSFER_MOVE );

					if ( pIMap )
					{
						aDropImageMap = *pIMap;
						delete pIMap;
					}
				}
			}

			for( ULONG nPos = 0, nCount = rTypeList.Count(); ( nPos < nCount ) && !bRet ; nPos++ )
			{
				const ULONG nFormat = rTypeList.GetObject( nPos ).GetFormat();

				if ( nFormat == SOT_FORMATSTR_ID_DRAWING )
				{
					SvData aData( SOT_FORMATSTR_ID_DRAWING );

					// Datenpuffer besorgen
					if ( aDataObj->GetData( &aData ) )
					{
						// Groesse des Datenpuffers
						nDropDataSize = aData.GetMinMemorySize();

						if ( aData.GetData( &pDropDataBytes, TRANSFER_MOVE ) )
						{
							aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropSvDrawHdl ) );
							aDropTimer.Start();
							bRet =TRUE;
						}
					}
				}
				else if( nFormat == FORMAT_FILE )
				{
					SvData aData( nFormat );

					// gedragte Files nur in die uebergebene Liste eintragen;
					// diese werden spater vom Aufrufer eingefuegt
					if( aDataObj->GetData( &aData ) )
					{
						String aFile;

						if( aData.GetData( aFile ) )
						{
							const USHORT nKind = FileStat( aFile ).GetKind();

							if ( ( nKind & FSYS_KIND_DIR ) || ( nKind & FSYS_KIND_DEV ) )
							{
								DirEntry		aDirEntry( aFile );
								Dir				aDir( aDirEntry += DirEntry( String( RTL_CONSTASCII_USTRINGPARAM( "*.*" ) ) ), FSYS_KIND_FILE );
								const USHORT	nCount = aDir.Count();

								for ( USHORT i = 0; i < nCount; i++ )
									rFileList.Insert( new String( aDir[ i ].GetFull() ), LIST_APPEND );
							}
							else
								rFileList.Insert( new String( aFile ), LIST_APPEND );

							bRet = TRUE;
						}
					}
				}
				else if ( nFormat == SOT_FORMATSTR_ID_SVXB )
				{
					SvData aData( nFormat );

					if ( aDataObj->GetData( &aData ) )
					{
						Graphic* pGraphic = NULL;

						if ( aData.GetData( (SvDataCopyStream**) &pGraphic, Graphic::StaticType(), TRANSFER_MOVE ) )
						{
							aDropGraphic = *pGraphic;

							if ( bImageMap )
								aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropIMapGraphicHdl ) );
							else
								aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropGraphicHdl ) );

							aDropTimer.Start();
							bRet = TRUE;
						}

						delete pGraphic;
					}
				}
				else if ( nFormat == FORMAT_GDIMETAFILE )
				{
					SvData aData( nFormat );

					if ( aDataObj->GetData( &aData ) )
					{
						GDIMetaFile* pMtf = NULL;

						if ( aData.GetData( &pMtf, TRANSFER_MOVE ) )
						{
							aDropGraphic = Graphic( *pMtf );

							if ( bImageMap )
								aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropIMapGraphicHdl ) );
							else
								aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropGraphicHdl ) );

							aDropTimer.Start();
							bRet = TRUE;
						}

						delete pMtf;
					}
				}
				else if ( nFormat == FORMAT_BITMAP )
				{
					SvData aData( nFormat );

					if ( aDataObj->GetData(&aData) )
					{
						Bitmap* pBmp = NULL;

						if ( aData.GetData( &pBmp, TRANSFER_MOVE ) )
						{
							aDropGraphic = Graphic( *pBmp );

							if ( bImageMap )
								aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropIMapGraphicHdl ) );
							else
								aDropTimer.SetTimeoutHdl( LINK( this, SgaGallery, DropGraphicHdl ) );

							aDropTimer.Start();
							bRet = TRUE;
						}

						delete pBmp;
					}
				}
			}
		}

		if ( bToggle )
			GetTheme( aOldTheme );
	}
	else if ( ( rThemeName != aDragParam.aThemeName ) && ( aDragParam.eObjKind != SGA_OBJ_SVDRAW ) )
	{
		// D&D von Gallery in Gallery ( aber nur zwischen untersch. Themen )
		SgaTheme*		pThm = GetActualTheme();
		const String	aOldTheme( pThm->GetName() );
		BOOL			bToggle = FALSE;

		if ( aOldTheme != rThemeName )
		{
			pThm = GetTheme( rThemeName );
			bToggle = TRUE;
		}

		if ( pThm )
		{
			List				aFileList;
			GalleryProgress		aProgress;

			aFileList.Insert( new String( aDragParam.aFileName ) );
			bRet = InsertFiles( aFileList, &aProgress, nInsertPos );
			for ( void* pFile = aFileList.First(); pFile; pFile = aFileList.Next() )
				delete (String*) pFile;

			if( pCurView )
				( (SvxGallery*) pCurView )->UpdateBrowser( ( nDropPos == LIST_APPEND ) ? OBJ_POS_LAST : nDropPos );
		}

		if ( bToggle )
			GetTheme( aOldTheme );
	}

	return bRet;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaGallery, DropGraphicHdl, void*, p )
{
	aDropTimer.Stop();

	SgaTheme*		pThm = GetActualTheme();
	const String	aOldTheme( pThm->GetName() );
	BOOL			bToggle = FALSE;

	if ( aOldTheme != aDropThemeName )
	{
		pThm = GetTheme( aDropThemeName );
		bToggle = TRUE;
	}

	if( pThm )
	{
		String aGraphicPath;

		if( SaveGraphic( aDropGraphic, aGraphicPath ) )
		{
			const SgaObjectBmp aObjBmp( aGraphicPath );
			pActualTheme->InsertObj( aObjBmp, nDropPos );
		}
	}

	if( bToggle )
		GetTheme( aOldTheme );
	else if( pCurView )
		( (SvxGallery*) pCurView )->UpdateBrowser( ( nDropPos == LIST_APPEND ) ? OBJ_POS_LAST : nDropPos );

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaGallery, DropIMapGraphicHdl, void*, p )
{
	aDropTimer.Stop();

	SgaTheme*		pThm = GetActualTheme();
	const String	aOldTheme( pThm->GetName() );
	BOOL			bToggle = FALSE;

	if ( aOldTheme != aDropThemeName )
	{
		pThm = GetTheme( aDropThemeName );
		bToggle = TRUE;
	}

	if ( pThm )
	{
		SvStorageRef aStor = pThm->GetSvDrawStorage();

		if ( aStor.Is() )
		{
			const DirEntry		aDirEntry( CreateUniqueFileName( SGA_OBJ_SVDRAW ) );
			SvStorageStreamRef	pOStm = aStor->OpenStream( aDirEntry.GetBase(), STREAM_READ | STREAM_WRITE | STREAM_TRUNC );

			if ( pOStm && !pOStm->GetError() )
			{
				SgaUserDataFactory	aFactory;
				FmFormModel			aNewModel;

				aNewModel.GetItemPool().FreezeIdRanges();

				SdrPage*			pNewPage = aNewModel.AllocPage( FALSE );
				SdrGrafObj*			pGrafObj = new SdrGrafObj( aDropGraphic );

				pGrafObj->InsertUserData( new SgaIMapInfo( aDropImageMap ) );
				pNewPage->InsertObject( pGrafObj );
				aNewModel.SetPageNotValid( TRUE );
				aNewModel.InsertPage( pNewPage );

				pOStm->SetBufferSize( STREAMBUF_SIZE );

				pOStm->SetVersion( SOFFICE_FILEFORMAT_NOW );
				aNewModel.SetStreamingSdrModel( TRUE );
				aNewModel.GetItemPool().Store( *pOStm );
				*pOStm << aNewModel;
				aNewModel.SetStreamingSdrModel( FALSE );
				pOStm->Seek( 0L );

				// und jetzt noch das Gallery-Objekt erzeugen
				if ( !pOStm->GetError() )
				{
					FmFormModel	aModel;

					aModel.GetItemPool().FreezeIdRanges();
					
					SgaObjectSvDraw aObj( *pOStm, aDirEntry.GetFull() );
					pThm->InsertObj( aObj, nDropPos );
				}

				pOStm->SetBufferSize( 0L );
			}
		}
	}

	if ( bToggle )
		GetTheme( aOldTheme );
	else if( pCurView )
		( (SvxGallery*) pCurView )->UpdateBrowser( ( nDropPos == LIST_APPEND ) ? OBJ_POS_LAST : nDropPos );

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaGallery, DropSvDrawHdl, void*, p )
{
	aDropTimer.Stop();

	SgaTheme*		pThm = GetActualTheme();
	const String	aOldTheme( pThm->GetName() );
	BOOL			bToggle = FALSE;

	if ( aOldTheme != aDropThemeName )
	{
		pThm = GetTheme( aDropThemeName );
		bToggle = TRUE;
	}

	if ( pThm )
	{
		SvStorageRef aStor = pThm->GetSvDrawStorage();

		if ( aStor.Is() )
		{
			const DirEntry		aDirEntry( CreateUniqueFileName( SGA_OBJ_SVDRAW ) );
			SvStorageStreamRef	pOStm = aStor->OpenStream( aDirEntry.GetBase(), STREAM_WRITE | STREAM_TRUNC );

			if ( pOStm && !pOStm->GetError() )
			{
				SvMemoryStream	aMemStm;
				RLECodec		aCodec( *pOStm );

				pOStm->SetBufferSize( STREAMBUF_SIZE );
				aMemStm.SetBuffer( (char*) pDropDataBytes, nDropDataSize, FALSE, nDropDataSize );

				// Bytes komprimiert rausschreiben
				aCodec.Write( aMemStm );

				// und jetzt noch das Gallery-Objekt erzeugen
				if ( !pOStm->GetError() )
				{
					// Datenblock als Puffer setzen und in Model streamen
					SgaObjectSvDraw aNewObj( aMemStm, aDirEntry.GetFull() );
					pThm->InsertObj( aNewObj, nDropPos );
				}

				pOStm->SetBufferSize( 0L );
			}
		}
	}

	if ( bToggle )
		GetTheme( aOldTheme );
	else if( pCurView )
		( (SvxGallery*) pCurView )->UpdateBrowser( ( nDropPos == LIST_APPEND ) ? OBJ_POS_LAST : nDropPos );

	// Puffer mit den gedropten Daten wieder freigeben!!!
	delete[] pDropDataBytes;

	return 0L;
}

// ------------------------------------------------------------------------

BOOL SgaGallery::InsertURL( const String& rURL, const String& rThemeName, const ULONG nSgaFormat, 
							GalleryProgress* pProgress, ULONG nInsertPos )
{
	BOOL bRet = ( SGA_FORMAT_ALL == nSgaFormat );

	if( !bRet )
	{
		if( ( SGA_FORMAT_SOUND & nSgaFormat ) && SGAIsSoundFile( rURL ) )
			bRet = TRUE;

		if( !bRet && ( SGA_FORMAT_GRAPHIC & nSgaFormat ) && ( GraphicDescriptor( rURL ).Detect() != GFF_NOT ) )
			bRet = TRUE;
	}

	if( bRet )
	{
		List			aFileList;
		String			aOldTheme;
		SgaTheme*		pThm = GetActualTheme();
		BOOL			bToggle = FALSE;

		// Falls Thema nicht vorhanden, legen wir es an
		if( !IsInList( rThemeName ) )
			CreateTheme( rThemeName );

		if( pThm && ( pThm->GetName() != rThemeName ) )
		{
			aOldTheme = pThm->GetName();
			pThm = GetTheme( rThemeName );
			bToggle = TRUE;
		}
		else if( !pThm )
			pThm = GetTheme( rThemeName );

		aFileList.Insert( new String( rURL ) );
		bRet = InsertFiles( aFileList, pProgress, nInsertPos );
		delete (String*) aFileList.First();

		if( bToggle )
			GetTheme( aOldTheme );
	}

	return bRet;
}

// -----------------
// - SgaDataObject -
// -----------------

SgaDataObject::SgaDataObject( SgaGallery* pGallery, ULONG nActualObjPos ) :
			pGal	( pGallery ),
			nObjPos	( nActualObjPos )
{
	pGal->SetDragging( TRUE );
}

// ------------------------------------------------------------------------

SgaDataObject::~SgaDataObject()
{
	pGal->SetDragging( FALSE );
	aTypeList.Clear();
}

// ------------------------------------------------------------------------

BOOL SgaDataObject::GetData( SvData* pData )
{
	BOOL bRet = FALSE;

	if ( GetTypeList().Get(*pData) && pGal )
		bRet = pGal->GetDragData( pData, pData->GetFormat(), nObjPos );

	return bRet;
}

// ------------------------------------------------------------------------

const SvDataTypeList& SgaDataObject::GetTypeList() const
{
	if ( !aTypeList.Count() && pGal )
		( (SgaDataObject*) this )->aTypeList = pGal->GetDragTypeList( aTypeList, nObjPos );

	return aTypeList;
}

// ------------------
// - SgaImportEntry -
// ------------------

SvStream& operator<<( SvStream& rOut, const SgaImportEntry& rEntry )
{
	rOut << ByteString( rEntry.aThemeName, RTL_TEXTENCODING_UTF8 ) << 
			ByteString( rEntry.aUIName, RTL_TEXTENCODING_UTF8 ) <<
			ByteString( rEntry.aFileName, RTL_TEXTENCODING_UTF8 ) << 
			ByteString( rEntry.aImportName, RTL_TEXTENCODING_UTF8 ) << 
			ByteString( rEntry.aRoot, RTL_TEXTENCODING_UTF8 );

	return rOut;
}

// ------------------------------------------------------------------------

SvStream& operator>>( SvStream& rIn, SgaImportEntry& rEntry )
{
	ByteString aTmpStr;

	rIn >> aTmpStr; rEntry.aThemeName = String( aTmpStr, RTL_TEXTENCODING_UTF8 ); 
	rIn >> aTmpStr; rEntry.aUIName = String( aTmpStr, RTL_TEXTENCODING_UTF8 );
	rIn >> aTmpStr; rEntry.aFileName = String( aTmpStr, RTL_TEXTENCODING_UTF8 ); 
	rIn >> aTmpStr; rEntry.aImportName = String( aTmpStr, RTL_TEXTENCODING_UTF8 ); 
	rIn >> aTmpStr; rEntry.aRoot = String( aTmpStr, RTL_TEXTENCODING_UTF8 );
	
	return rIn;
}
