/*************************************************************************
 *
 *  $RCSfile: gallery3.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_BRUSH 0

#pragma hdrstop

#include <stdlib.h>
#include <unotools/processfactory.hxx>
#include <tools/urlobj.hxx>
#include <tools/errinf.hxx>
#include <vcl/wrkwin.hxx>
#include <svtools/urihelper.hxx>
#include <svtools/filter.hxx>
#include <sfx2/exchobj.hxx>
#include "gallery.hrc"
#include "gallery1.hxx"
#include "gallery2.hxx"

// ------------------------------------------------------------------------

#define GALLERY_PROGRESS_RANGE	10000

// ------------------------------------------------------------------------

String aBrowserPath;
String aLastSearchPath;
String aLastFilterName;

// ------------------------------------------------------------------------

// ----------
// - ResMgr -
// ----------

ResMgr* GetGalleryResMgr()
{
	static ResMgr* pGalleryResMgr = NULL;

	if( !pGalleryResMgr )
	{
		pGalleryResMgr = ResMgr::CreateResMgr( ( ByteString( "gal" ) += ByteString::CreateFromInt32( SOLARUPD ) ).GetBuffer(),
											   ::GetSystemLanguage() );
	}

	return pGalleryResMgr;
}

// -------------------
// - GalleryProgress -
// -------------------

GalleryProgress::GalleryProgress( GraphicFilter* pFilter ) :
	mpFilter( pFilter )
{
	REF( NMSP_LANG::XMultiServiceFactory ) xMgr( ::utl::getProcessServiceFactory() );

	if( xMgr.is() )
	{
		REF( NMSP_AWT::XProgressMonitor )	xMonitor( xMgr->createInstance( 
													  ::rtl::OUString::createFromAscii( "com.sun.star.awt.XProgressMonitor" ) ),
													  NMSP_UNO::UNO_QUERY );

		if ( xMonitor.is() )
		{
			mxProgressBar = REF( NMSP_AWT::XProgressBar )( xMonitor, NMSP_UNO::UNO_QUERY );

			if( mxProgressBar.is() )
			{
				String aProgressText;

				if( mpFilter )
				{
					aProgressText = String( GAL_RESID( RID_SVXSTR_GALLERY_FILTER ) );
					mpFilter->SetUpdatePercentHdl( LINK( this, GalleryProgress, Update ) );
				}
				else
					aProgressText = String( RTL_CONSTASCII_USTRINGPARAM( STAR_GALLERY ) );

				xMonitor->addText( String( RTL_CONSTASCII_USTRINGPARAM( "Gallery" ) ), aProgressText, sal_False ) ;
				mxProgressBar->setRange( 0, GALLERY_PROGRESS_RANGE );
			}
		}
	}
}

// ------------------------------------------------------------------------

GalleryProgress::~GalleryProgress()
{
	if( mpFilter )
		mpFilter->SetUpdatePercentHdl( Link() );
}

// ------------------------------------------------------------------------

void GalleryProgress::Update( ULONG nVal, ULONG nMaxVal )
{
	if( mxProgressBar.is() && nMaxVal )
		mxProgressBar->setValue( Min( (ULONG)( (double) nVal / nMaxVal * GALLERY_PROGRESS_RANGE ), (ULONG) GALLERY_PROGRESS_RANGE ) );
}

// ------------------------------------------------------------------------

IMPL_LINK( GalleryProgress, Update, GraphicFilter*, pFilter )
{
	Update( pFilter->GetPercent(), 100 );
	return 0L;
}

// ------------------------------------------------------------------------

SgaPopup::SgaPopup(  SgaGallery* pGal, USHORT nSelPos, BOOL bPreview ) :
			PopupMenu			( GAL_RESID( RID_SVXMN_GALLERY1 ) ),
			SfxControllerItem	( SID_GALLERY_ENABLE_ADDCOPY, SFX_BINDINGS() ),
			pBGPopup			( NULL )
{
	if( pGal )
	{
		const SgaObjKind	eObjKind = pGal->GetObjKindFromActualTheme( nSelPos );
		PopupMenu*			pAddMenu = GetPopupMenu( MN_ADDMENU );
		SgaTheme*			pThm;
		SfxBindings&		rBindings = SFX_BINDINGS();

		rBindings.ENTERREGISTRATIONS();
		pBGPopup = new SgaBGPopup( pGal->GetPathFromActualTheme( nSelPos ), SGA_OBJ_INET == eObjKind );
		rBindings.LEAVEREGISTRATIONS();

		pAddMenu->EnableItem( MN_ADD_LINK, SGA_OBJ_SVDRAW != eObjKind );
		CheckItem( MN_PREVIEW, bPreview );

		if( pGal && ( pThm = pGal->GetActualTheme() ) && ( pThm->IsReadOnly() || !pThm->GetObjCount() ) )
		{
			EnableItem( MN_DELETE, FALSE );
			EnableItem( MN_TITLE, FALSE );
		}
		else
		{
			EnableItem( MN_DELETE, !bPreview );
			EnableItem( MN_TITLE, TRUE );
		}

		// SvDraw-Objekte koennen nicht gelinkt
		// hinzugefuegt werden, da sie im Storage liegen
		if( !pBGPopup->GetItemCount() || ( eObjKind == SGA_OBJ_SVDRAW ) )
			pAddMenu->EnableItem( MN_BACKGROUND, FALSE );
		else
		{
			pAddMenu->EnableItem( MN_BACKGROUND, TRUE );
			pAddMenu->SetPopupMenu( MN_BACKGROUND, pBGPopup );
		}

		rBindings.Update( SID_GALLERY_ENABLE_ADDCOPY );
		RemoveDisabledEntries();
	}
}


// ------------------------------------------------------------------------

void SgaPopup::StateChanged( USHORT nSID, SfxItemState eState, const SfxPoolItem* pItem )
{
	if ( ( nSID == SID_GALLERY_ENABLE_ADDCOPY ) && pItem && ( eState != SFX_ITEM_DISABLED ) )
	{
		SfxBoolItem* pBoolItem = PTR_CAST( SfxBoolItem, pItem );

		DBG_ASSERT( pBoolItem || pBoolItem == 0, "SfxBoolItem erwartet!");
		GetPopupMenu( MN_ADDMENU )->EnableItem( MN_ADD, pBoolItem->GetValue() );
	}
}


// ------------------------------------------------------------------------

SgaBGPopup::SgaBGPopup( const String& rFileName, BOOL _bINET ) :
			SfxControllerItem	( SID_GALLERY_BG_BRUSH, SFX_BINDINGS() ),
			aFileName			( rFileName ),
			bINET				( _bINET )
{
	SFX_BINDINGS().Update( SID_GALLERY_BG_BRUSH );
	RemoveDisabledEntries();
}


// ------------------------------------------------------------------------

void SgaBGPopup::StateChanged( USHORT nSID, SfxItemState eState, const SfxPoolItem* pItem )
{
	if ( ( nSID == SID_GALLERY_BG_BRUSH ) && pItem && ( eState != SFX_ITEM_DISABLED ) )
	{
		SfxStringListItem*	pStrLstItem;
		SfxStringItem*		pStrItem;

		PopupMenu::Clear();

		if( ( pStrLstItem = PTR_CAST( SfxStringListItem, pItem ) ) != NULL )
		{
			List* pList = pStrLstItem->GetList();

			if( pList )
				for ( ULONG i = 0, nCount = pList->Count(); i < nCount; i++ )
					InsertItem( (USHORT) i + 1, *(String*) pList->GetObject( i ) );
		}
		else if( ( pStrItem = PTR_CAST( SfxStringItem, pItem ) ) != NULL )
			InsertItem( 1, pStrItem->GetValue() );
		else
		{
			DBG_ERROR( "SgaBGPopup::StateChanged(...): Wrong item type!" );
		}
	}
}


// ------------------------------------------------------------------------

void SgaBGPopup::Select()
{
	Menu::Select();

	if ( bINET || DirEntry( aFileName ).Exists() )
	{
		const SvxBrushItem	aBrushItem( aFileName, String(), GPOS_TILED, SID_GALLERY_BG_BRUSH );
		const SfxUInt16Item aPosItem( SID_GALLERY_BG_POS, GetCurItemId() - 1 );
		const SfxStringItem	aPathItem( SID_FILE_NAME, ::URIHelper::SmartRelToAbs( aFileName ) );

		SFX_DISPATCHER().Execute( SID_GALLERY_BG_BRUSH,
								  SFX_CALLMODE_ASYNCHRON | SFX_CALLMODE_RECORD,
								  &aBrushItem, &aPosItem, &aPathItem, 0L );
	}
	else
		ErrorHandler::HandleError( ERRCODE_IO_NOTEXISTSPATH );
}


// ------------------------------------------------------------------------

SvxClipValueSet::SvxClipValueSet( SvxGallery* pParent, WinBits nWinStyle ) :
		ValueSet		( pParent, nWinStyle ),
		pGal			( NULL ),
		pSvxGallery 	( pParent )
{
	EnableDrop( TRUE );
}


// ------------------------------------------------------------------------

void SvxClipValueSet::UserDraw( const UserDrawEvent& rUDEvt )
{
	SgaTheme*		pThm = pSvxGallery->GetTheme();
	const USHORT	nId = rUDEvt.GetItemId();

	if ( nId && pThm )
	{
		const long	nPos = nId - 1;
		SgaObjList*	pEntry = pSvxGallery->GetGallery()->GetObjFromActualTheme( nPos );
		SgaObject*	pObj = pThm->GetObj( nPos );

		if( pEntry && pObj )
		{
			const Rectangle&	rRect = rUDEvt.GetRect();
			OutputDevice*		pDev = rUDEvt.GetDevice();
			Graphic 			aGraphic;

			if ( pObj->IsThumbBitmap() )
			{
				Bitmap aBmp( pObj->GetThumbBmp() );

				if( pObj->GetObjKind() == SGA_OBJ_SOUND )
					aBmp.Replace( COL_LIGHTMAGENTA, COL_WHITE );

				aGraphic = aBmp;
			}
			else
				aGraphic = pObj->GetThumbMtf();

			Size aSize( pDev->LogicToPixel( aGraphic.GetPrefSize(), aGraphic.GetPrefMapMode() ) );

			if ( aSize.Width() && aSize.Height() )
			{
				Point			aNewPos;
				const double	fBmpWH	= (double) aSize.Width() / aSize.Height();
				const double	fThmpWH = (double) rRect.GetWidth() / rRect.GetHeight();

				// Bitmap an Thumbgroesse anpassen
				if ( fBmpWH < fThmpWH )
				{
					aSize.Width() = (long) ( rRect.GetHeight() * fBmpWH );
					aSize.Height()= rRect.GetHeight();
				}
				else
				{
					aSize.Width() = rRect.GetWidth();
					aSize.Height()= (long) ( rRect.GetWidth() / fBmpWH );
				}

				const Point aPos( ( ( rRect.GetWidth() - aSize.Width() ) >> 1 ) + rRect.Left(),
								  ( ( rRect.GetHeight() - aSize.Height() ) >> 1 ) + rRect.Top() );

				aGraphic.Draw( pDev, aPos, aSize );
			}

			UpdateItemText( nId, pThm, pEntry, pObj );
			delete pObj;
		}
	}
}

// ------------------------------------------------------------------------

void SvxClipValueSet::UpdateItemText( USHORT nId, const SgaTheme* pThm,
									  const SgaObjList* pEntry, const SgaObject* pObj )
{
	String		aItemText;
	const BOOL	bImported = pThm->IsImported();
	
	if( bImported )
	{
		DirEntry aPathTmp( pSvxGallery->GetGallery()->GetImportPath( pThm->GetName() ).GetPath() );
		aPathTmp += DirEntry( DirEntry( pEntry->aPath ).GetName() );
#ifndef MAC
		aItemText = aPathTmp.GetFull();
#else
		aItemText = DirEntry( pEntry->aPath ).GetName();
#endif
	}
	else
	{
#ifndef MAC
		aItemText = pEntry->aPath;
#else
		aItemText = DirEntry( pEntry->aPath ).GetName();
#endif
	}

	if( pObj->GetTitle().Len() )
	{
		String aTitleItemText( pObj->GetTitle() );

		if( pObj->GetObjKind() != SGA_OBJ_SVDRAW )
		{
			aTitleItemText += String( RTL_CONSTASCII_USTRINGPARAM( " (" ) );
			aTitleItemText += aItemText;
			aTitleItemText += ')';
		}

		aItemText = aTitleItemText;
	}

	SetItemText( nId, aItemText );
}

// ------------------------------------------------------------------------

void SvxClipValueSet::Command(const CommandEvent& rCEvt )
{
	Region	aRegion;
	USHORT	nSelId = 0;

	if ( StartDrag( rCEvt, aRegion ) && pGal )
	{
		GalDragParams aDragParam;

		aDragParam.aDragRegion = aRegion;
		aDragParam.nDragObjPos = GetItemId( rCEvt.GetMousePosPixel() ) - 1;
		aDragParam.aThemeName = pSvxGallery->GetTheme()->GetName();
		aDragParam.aFileName = pGal->GetPathFromActualTheme( aDragParam.nDragObjPos );
		aDragParam.eObjKind = pGal->GetObjKindFromActualTheme( aDragParam.nDragObjPos );

		pGal->SetDragParams( aDragParam );
		pSvxGallery->aHintPosterRef->Post();
	}
	else if ( ( rCEvt.GetCommand() == COMMAND_CONTEXTMENU ) && rCEvt.IsMouseEvent() )
	{
		pSvxGallery->GetTheme();
		nSelId = GetItemId( rCEvt.GetMousePosPixel() );
		SelectItem( nSelId );

		SfxBindings& rBindings = SFX_BINDINGS();
		rBindings.ENTERREGISTRATIONS();
		SgaPopup aMenu( pGal, nSelId - 1, FALSE );
		rBindings.LEAVEREGISTRATIONS();

		aMenu.SetSelectHdl( LINK( this, SvxClipValueSet, MenuSelectHdl ) );
		aMenu.Execute( this, rCEvt.GetMousePosPixel() );
	}
	else
		ValueSet::Command( rCEvt );
}


// ------------------------------------------------------------------------

IMPL_LINK( SvxClipValueSet, MenuSelectHdl, Menu *, pMenu )
{
	if (pMenu)
	{
		const USHORT nId = pMenu->GetCurItemId();

		switch( nId )
		{
			case( MN_ADD ) :
				pSvxGallery->AddFile( FALSE );
			break;

			case( MN_ADD_LINK ) :
				pSvxGallery->AddFile( TRUE );
			break;

			case( MN_PREVIEW ) :
				pSvxGallery->DoubleClickBmpHdl(NULL);
			break;

			case( MN_DELETE ) :
				pSvxGallery->DeleteSelectedObj();
			break;

			case( MN_TITLE ) :
				pSvxGallery->EditSelectedObjTitle();
			break;

			default :
			break;
		}
	}

	return 0;
}


// ------------------------------------------------------------------------

BOOL SvxClipValueSet::QueryDrop( DropEvent& rDEvt )
{
	SfxExchangeObjectRef	xObj = SfxExchangeObject::PasteDragServer( rDEvt );
	SgaTheme*				pThm = pSvxGallery->GetTheme();
	BOOL					bRet;

	if( pThm && !pThm->IsReadOnly() && !pThm->IsImported() )
		bRet = pGal->DoQueryDrop( rDEvt );
	else
		bRet = FALSE;

	return bRet;
}


// ------------------------------------------------------------------------

BOOL SvxClipValueSet::Drop( const DropEvent& rDEvt )
{
	SgaTheme*	pThm = pSvxGallery->GetTheme();
	BOOL		bRet = FALSE;

	if( pThm )
	{
		const USHORT	nItemId = GetItemId( rDEvt.GetPosPixel() );
		const ULONG		nInsertPos = ( nItemId ? ( nItemId - 1 ) : LIST_APPEND );

		if( pGal->IsDragging() && ( pThm->GetName() == pGal->GetDragParams().aThemeName ) )
		{
			ULONG nUpdatePos;

			pThm->ChangeObjPos( pGal->GetDragParams().nDragObjPos, nInsertPos );

			if( LIST_APPEND == nInsertPos )
				nUpdatePos = OBJ_POS_LAST;
			else if( nInsertPos > pGal->GetDragParams().nDragObjPos )
				nUpdatePos = nInsertPos - 1;
			else
				nUpdatePos = nInsertPos;
				 
			pSvxGallery->UpdateBrowser( nUpdatePos );
		}
		else
		{
			SfxExchangeObjectRef	xObj = SfxExchangeObject::PasteDragServer( rDEvt );
			List					aFileList;

			for( ULONG i = 0UL, nCount = xObj->Count(); i < nCount; i++ )
			{
				bRet |= pGal->DoDrop( DropEvent( Point(), xObj->GetObject( i ),
												 rDEvt.GetAction(), rDEvt.GetSourceOptions(),
												 rDEvt.GetWindowType(), rDEvt.IsDefaultAction() ),
												 pSvxGallery->GetTheme()->GetName(), aFileList, nInsertPos );
			}

			if( aFileList.Count() )
			{
				GalleryProgress aProgress;

				pGal->InsertFiles( aFileList, &aProgress, nInsertPos );
				for( void* pFile = aFileList.First(); pFile; pFile = aFileList.Next() )
					delete (String*) pFile;
			
				pSvxGallery->UpdateBrowser( ( nInsertPos == LIST_APPEND ) ? OBJ_POS_LAST : nInsertPos );
			}
		}
	}

	return bRet;
}

// ------------------------------------------------------------------------

	GalleryWindow::GalleryWindow(SvxGallery* pParent, WinBits nWinStyle) :
		Window		(pParent, nWinStyle),
		pSvxGallery (pParent)
{
	const StyleSettings& rStyleSettings = Application::GetSettings().GetStyleSettings();
	SetBackground( Wallpaper( COL_GRAY ) );
	EnableDrop( TRUE );
	aSound.SetNotifyHdl( LINK( this, GalleryWindow, SoundEndHdl ) );
}

// ------------------------------------------------------------------------

GalleryWindow::~GalleryWindow()
{
}

// ------------------------------------------------------------------------

void GalleryWindow::MouseButtonDown(const MouseEvent& rMEvt)
{
	if( rMEvt.GetClicks() > 1 )
		pSvxGallery->HidePreview();
}

// ------------------------------------------------------------------------

void GalleryWindow::SetGraphic( const Graphic& rGraphic )
{
	aGraphicObj = rGraphic;
}

// ------------------------------------------------------------------------

void GalleryWindow::Paint( const Rectangle& rRect )
{
	if( SvxGallery::GetGraphicCenterRect( aGraphicObj.GetGraphic(), *this, aPreviewRect ) )
	{
		const Point aPos( aPreviewRect.TopLeft() );
		const Size	aSize( aPreviewRect.GetSize() );

		if( aGraphicObj.IsAnimated() )
			aGraphicObj.StartAnimation( this, aPos, aSize );
		else
			aGraphicObj.Draw( this, aPos, aSize );
			
		aGraphicObj.Draw( this, aPos, aSize );
	}
}

// ------------------------------------------------------------------------

void GalleryWindow::Command(const CommandEvent& rCEvt )
{
	// Aktion nur, wenn Mauspointer sich ueber dem
	// Bild befindet
	if ( aPreviewRect.IsInside( rCEvt.GetMousePosPixel() ) )
	{
		if (rCEvt.GetCommand() == COMMAND_STARTDRAG)
		{
			GalDragParams aDragParam;

			aDragParam.aDragRegion = aPreviewRect;
			aDragParam.nDragObjPos = pSvxGallery->pCtlGallery->GetSelectItemId() - 1;
			aDragParam.aThemeName = pSvxGallery->GetTheme()->GetName();
			aDragParam.aFileName = pSvxGallery->GetPathAsString();
			aDragParam.eObjKind = pSvxGallery->pGal->GetObjKindFromActualTheme( aDragParam.nDragObjPos );

			pSvxGallery->pGal->SetDragParams( aDragParam );
			pSvxGallery->aHintPosterRef->Post();
		}
		else if ( ( rCEvt.GetCommand() == COMMAND_CONTEXTMENU ) && rCEvt.IsMouseEvent() )
		{
			pSvxGallery->GetTheme();

			SfxBindings& rBindings = SFX_BINDINGS();
			rBindings.ENTERREGISTRATIONS();
			SgaPopup aMenu( pSvxGallery->pGal, pSvxGallery->pCtlGallery->GetSelectItemId() - 1, TRUE );
			rBindings.LEAVEREGISTRATIONS();

			aMenu.SetSelectHdl( LINK( this, GalleryWindow, MenuSelectHdl ) );
			aMenu.Execute( this, rCEvt.GetMousePosPixel() );
		}
		else
			Window::Command( rCEvt );
	}
	else
		Window::Command( rCEvt );
}

// ------------------------------------------------------------------------

void GalleryWindow::PreviewSound( const String& rPath )
{
	aSound.SetSoundName( rPath );

	if( rPath.Len() )
		aSound.Play();
}

// ------------------------------------------------------------------------

BOOL GalleryWindow::QueryDrop( DropEvent& rDEvt )
{
	SfxExchangeObjectRef	xObj = SfxExchangeObject::PasteDragServer( rDEvt );
	SgaTheme*				pThm = pSvxGallery->GetTheme();
	BOOL					bRet;

	if( pThm && !pThm->IsReadOnly() && !pThm->IsImported() )
		bRet = pSvxGallery->pGal->DoQueryDrop( rDEvt );
	else
		bRet = FALSE;

	return bRet;
}

// ------------------------------------------------------------------------

BOOL GalleryWindow::Drop( const DropEvent& rDEvt )
{
	SfxExchangeObjectRef	xObj = SfxExchangeObject::PasteDragServer( rDEvt );
	List					aFileList;
	BOOL					bRet = FALSE;

	for( ULONG i = 0UL, nCount = xObj->Count(); i < nCount; i++ )
	{
		bRet |= pSvxGallery->pGal->DoDrop( DropEvent( Point(), xObj->GetObject( i ),
													  rDEvt.GetAction(), rDEvt.GetSourceOptions(),
													  rDEvt.GetWindowType(), rDEvt.IsDefaultAction() ),
													  pSvxGallery->GetTheme()->GetName(), aFileList, LIST_APPEND );
	}

	if( aFileList.Count() )
	{
		GalleryProgress aProgress;

		pSvxGallery->pGal->InsertFiles( aFileList, &aProgress );
		for( void* pFile = aFileList.First(); pFile; pFile = aFileList.Next() )
			delete (String*) pFile;
	}

	if( bRet )
		pSvxGallery->UpdateBrowser( OBJ_POS_LAST );

	return bRet;
}

// ------------------------------------------------------------------------

IMPL_LINK( GalleryWindow, MenuSelectHdl, Menu *, pMenu )
{
	if (pMenu)
	{
		const USHORT nId = pMenu->GetCurItemId();

		switch ( nId )
		{
			case( MN_ADD ):
			case( MN_ADD_LINK ):
			case( MN_PREVIEW ):
			{
				if( MN_PREVIEW == nId )
					pSvxGallery->HidePreview();
				else
					pSvxGallery->AddFile( MN_ADD_LINK == nId );
			}
			break;

			case( MN_TITLE ) :
				pSvxGallery->EditSelectedObjTitle();
			break;

			default :
			break;
		}
	}

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( GalleryWindow, SoundEndHdl, Sound*, pSound )
{
	aSound.SetSoundName( String() );

	return 0L;
}


// ------------------------------------------------------------------------

BrowserWindow::BrowserWindow(Window* pParent, const ResId& rId) :
		Window( pParent, rId )
{
	const StyleSettings& rStyleSettings = Application::GetSettings().GetStyleSettings();
	SetBackground( Wallpaper( rStyleSettings.GetWindowColor() ) );
}


// ------------------------------------------------------------------------

void BrowserWindow::Paint( const Rectangle& rRect )
{
	Rectangle aPreviewRect;

	if ( SvxGallery::GetGraphicCenterRect( aGraphic, *this, aPreviewRect ) )
	{
		const Point aPos( aPreviewRect.TopLeft() );
		const Size	aSize( aPreviewRect.GetSize() );

		if ( aGraphic.IsAnimated() )
			aGraphic.StartAnimation( this, aPos, aSize );
		else
			aGraphic.Draw( this, aPos, aSize );
	}
}
