/*************************************************************************
 *
 *  $RCSfile: gallery4.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_BRUSH 0

#pragma hdrstop

#include <stdlib.h>
#include <tools/ref.hxx>
#include <vcl/msgbox.hxx>
#include <vcl/wrkwin.hxx>
#include <sfx2/app.hxx>
#include <sfx2/inimgr.hxx>
#include <sfx2/viewfrm.hxx>
#include <sfx2/viewsh.hxx>
#include "helpid.hrc"
#include "impgrf.hxx"
#include "gallery.hrc"
#include "gallery1.hxx"
#include "gallery2.hxx"
#include "galcmp.hxx"

// ------------------------------------------------------------------------

extern String aBrowserPath;
extern String aLastSearchPath;
extern String aLastFilterName;

// ------------------------------------------------------------------------

SvxGallery::SvxGallery( Window* pParent, SgaGallery* pGallery ) :
		Window					( pParent ),
		pGal					( pGallery ),
		bIsPreview				( FALSE ),
		bLinkage				( FALSE )
{
	SetHelpId( HID_GALLERY_WINDOW );

	LoadGallery();

	pGal->SetCurView( this );
	aThemeName = pGal->GetActualTheme()->GetName();
	pCtlGallery = new SvxClipValueSet( this, WB_BORDER ),
	pCtlGallery->SetHelpId( HID_GALLERY_WINDOW );

	pCtlGallery->SetPosSizePixel( Point(), pParent->GetSizePixel() );
	pCtlGallery->Show();

	pWndPreview = new GalleryWindow( this, WB_BORDER ),
	pWndPreview->SetHelpId( HID_GALLERY_WINDOW );

	aLastSearchPath = SFX_INIMANAGER()->Get( SFX_KEY_GRAPHICS_PATH );
	aLastFilterName = String();

	aPath = DirEntry( SFX_INIMANAGER()->Get( SFX_KEY_CONFIG_DIR ) );
	aPath.ToAbs();
	aPath.MakeDir();
	aBrowserPath = aPath.GetFull();

	pCtlGallery->SetGallery( pGal );
	pCtlGallery->SetSelectHdl( LINK( this, SvxGallery, SelectBmpHdl ) );
	pCtlGallery->SetDoubleClickHdl( LINK( this, SvxGallery, DoubleClickBmpHdl ) );
	pCtlGallery->SetWinStyle(pCtlGallery->GetWinStyle() | WB_ITEMBORDER | WB_DOUBLEBORDER | WB_NAMEFIELD | WB_VSCROLL);
	pCtlGallery->SetColCount( 3 );
	pCtlGallery->SetLineCount( 3 );
	pCtlGallery->SetExtraSpacing( 2 );
	UpdateBrowser( OBJ_POS_FIRST );
	pCtlGallery->Show();

	aHintPosterRef = new SfxHintPoster( LINK( pGal, SgaGallery, StartDragHdl ) );
}


// ------------------------------------------------------------------------

SvxGallery::~SvxGallery()
{
	if( pGal && ( pGal->GetCurView() == this ) )
		pGal->SetCurView( NULL );

	delete pCtlGallery;
	delete pWndPreview;

	UnloadGallery();
}

// ------------------------------------------------------------------------

void SvxGallery::Resize()
{
	Window::Resize();
	AdjustControls( GetOutputSizePixel() );
}


// ------------------------------------------------------------------------

void SvxGallery::KeyInput( const KeyEvent& rKEvt )
{
	if ( ( rKEvt.GetKeyCode().GetCode() == KEY_DELETE ) && !bIsPreview )
		DeleteSelectedObj();
	else
		pCtlGallery->KeyInput( rKEvt );
}


// ------------------------------------------------------------------------

void SvxGallery::AppendToValueSet()
{
	SgaTheme* pThm = GetTheme();

	if ( pThm )
	{
		SgaObjList* pEntry = ( (ObjectList&) pThm->GetObjList() ).Last();

		if ( pEntry )
		{
			USHORT	nId = pCtlGallery->GetItemCount();

			pCtlGallery->InsertItem( ++nId );

#ifndef MAC
			pCtlGallery->SetItemText( nId, pEntry->aPath );
#else
			pCtlGallery->SetItemText( nId, DirEntry( pEntry->aPath ).GetName() );
#endif

			pCtlGallery->SelectItem( nId );
		}
	}
}


// ------------------------------------------------------------------------

void SvxGallery::DeleteSelectedObj()
{
	SgaTheme* pThm = GetTheme();

	if( pThm && !pThm->IsReadOnly() )
	{
		if (QueryBox(this, (WinBits)WB_YES_NO,
					 String(GAL_RESID(RID_SVXSTR_GALLERY_DELETEOBJ))).Execute() == RET_YES )
		{
			const long	nPos = (long) pCtlGallery->GetSelectItemId() - 1L;
			SgaObjList* pEntry = pGal->GetObjFromActualTheme( nPos );
			BOOL		bDD = FALSE;

			// Test, ob es sich um eine D&D-Datei handelt
			if ( pEntry )
			{
				const DirEntry aFilePath( pGal->GetCorrectPath( pEntry ) );

				if ( aFilePath.GetPath() ==
					 ( SgaGallery::bMultiPath ? SgaGallery::aUserPath : SgaGallery::aRelPathParent ) )
				{
					String			aFileName( aFilePath.GetName() );
					String			aExt( aFilePath.GetExtension() );
					sal_Unicode		aNext = aFileName.GetChar( 2 );

					aFileName.Erase( 2 );

					if ( ( aFileName.CompareToAscii( "dd" ) == COMPARE_EQUAL ) &&
						 ( aNext >= '0' ) && ( aNext <= '9' ) &&
						 ( ( aExt.CompareToAscii( "bmp" ) == COMPARE_EQUAL ) || 
						   ( aExt.CompareToAscii( "svm" ) == COMPARE_EQUAL ) ) )
					{
						if ( QueryBox(this, (WinBits)WB_YES_NO,
							 String(GAL_RESID(RID_SVXSTR_GALLERY_DELETEDD))).
							 Execute() == RET_YES )
						{
							aFilePath.Kill();
						}
					}
				}
			}

			pThm->RemoveObj( nPos );
			UpdateBrowser( OBJ_POS_CURRENT );
		}
	}
}

// ------------------------------------------------------------------------

void SvxGallery::EditSelectedObjTitle()
{
	SgaTheme* pThm = GetTheme();

	if( pThm && !pThm->IsReadOnly() )
	{
		const long		nPos = (long) pCtlGallery->GetSelectItemId() - 1L;
		SgaObjList*		pEntry = pGal->GetObjFromActualTheme( nPos );

		if( pEntry )
		{
			SgaObject* pObj = pThm->GetObj( nPos );

			if( pObj )
			{
				TitleDialog	aDlg( this, pObj->GetTitle() );

				if( aDlg.Execute() == RET_OK )
				{
					const String aNewTitle( aDlg.GetTitle() );

					if( aNewTitle != pObj->GetTitle() )
					{
						pObj->SetTitle( aNewTitle );
						pThm->InsertObj( *pObj );
						pCtlGallery->UpdateItemText( nPos + 1, pThm, pEntry, pObj );
					}
				}
			}

			delete pObj;
		}
	}
}

// ------------------------------------------------------------------------

void SvxGallery::AdjustControls( const Size& rWinSize )
{
	USHORT	nSelLine = 0;
	USHORT	nSelectId = 1;
	USHORT	nColCount = pCtlGallery->GetColCount();

	// Valueset und Preview-Fenster in der Groesse anpassen
	if ( bIsPreview )
		pWndPreview->Hide();
	else if ( nColCount )
	{
		nSelectId = pCtlGallery->GetSelectItemId();
		nSelLine = (nSelectId - 1) / nColCount - pCtlGallery->GetFirstLine();
		pCtlGallery->Hide();
	}
	else
		pCtlGallery->Hide();

	pCtlGallery->SetColCount( nColCount = Max( (USHORT) ( rWinSize.Width() / 80 ), (USHORT) 1 ) );
	pCtlGallery->SetLineCount( Max( (USHORT) ( rWinSize.Height() / 70 ), (USHORT) 1 ) );
	pCtlGallery->SetSizePixel( rWinSize );
	pWndPreview->SetSizePixel( rWinSize );

	if ( bIsPreview )
		pWndPreview->Show();
	else
	{
		USHORT nFirstLine = (nSelectId - 1) / nColCount;

		nSelLine = Min( (USHORT) nSelLine, (USHORT) ( pCtlGallery->GetLineCount() - 1 ) );

		if (nFirstLine > nSelLine)
			nFirstLine -= nSelLine;

		pCtlGallery->SetFirstLine( nFirstLine );
		pCtlGallery->Show();
	}
}


// ------------------------------------------------------------------------

void SvxGallery::UpdateBrowser( USHORT nSelectObjPos )
{
	SgaTheme*	pThm = GetTheme();
	USHORT		i = 1;
	USHORT		nCurItemPos = pCtlGallery->GetSelectItemId();
	USHORT		nItem;

	pCtlGallery->Clear();
	HidePreview();

	if ( pThm )
	{
		ObjectList& rObjectList = (ObjectList&) pThm->GetObjList();

		GetParent()->EnterWait();

		for( SgaObjList* pEntry = rObjectList.First(); pEntry; pEntry = rObjectList.Next() )
			pCtlGallery->InsertItem( i++ );

		GetParent()->LeaveWait();
	}

	// Welchen sollen wir selektieren ???
	if ( nSelectObjPos == OBJ_POS_CURRENT )
		nItem = Min( nCurItemPos, pCtlGallery->GetItemCount() );
	else if ( nSelectObjPos == OBJ_POS_LAST )
		nItem = pCtlGallery->GetItemCount();
	else
		nItem = nSelectObjPos + 1;

	pCtlGallery->SelectItem( nItem );
	pCtlGallery->Show();
}


// ------------------------------------------------------------------------

IMPL_LINK( SvxGallery, SelectBmpHdl, void*, p )
{
	GrabFocus();

	return 0L;
}


// ------------------------------------------------------------------------

IMPL_LINK( SvxGallery, DoubleClickBmpHdl, void *, p )
{
	if ( !bIsPreview )
	{
		const Graphic aGraphic( GetGraphic() );

		if ( aGraphic.GetType() != GRAPHIC_NONE )
		{
			const USHORT nId = pCtlGallery->GetSelectItemId() - 1;

			bIsPreview = TRUE;
			pCtlGallery->Hide();
			pWndPreview->SetGraphic( aGraphic );
			pWndPreview->Show();

			// Falls es sich um ein Sound-Objekt handelt,
			// spielen wir waehrend der Preview den Sound ab
			if ( pGal->GetObjKindFromActualTheme( nId ) == SGA_OBJ_SOUND )
				pWndPreview->PreviewSound( pGal->GetPathFromActualTheme( nId ) );
		}
	}
	else
	{
		pWndPreview->SetGraphic( Graphic() );
		pWndPreview->PreviewSound( String() );
	}

	return 0;
}


// ------------------------------------------------------------------------

void SvxGallery::AddFile( BOOL bAsLinkage )
{
	UINT32		nFormat;
	SgaTheme*	pThm = GetTheme();

	if ( pThm )
	{
		SgaObject* pObj = pThm->GetObj( pCtlGallery->GetSelectItemId() - 1 );

		if (pObj)
		{
			switch (pObj->GetObjKind())
			{
				case( SGA_OBJ_BMP ):
				case( SGA_OBJ_ANIM ):
				case( SGA_OBJ_INET ):
					nFormat = SGA_FORMAT_GRAPHIC | SGA_FORMAT_STRING;
				break;

				case ( SGA_OBJ_SOUND ) :
					nFormat = SGA_FORMAT_SOUND | SGA_FORMAT_STRING;
				break;

				case( SGA_OBJ_SVDRAW ):
					nFormat = SGA_FORMAT_GRAPHIC | SGA_FORMAT_SVDRAW | SGA_FORMAT_STRING;
				break;

				default :
				break;
			}

			delete pObj;
		}
	}

	SfxUInt32Item aItem(SID_GALLERY_FORMATS, nFormat);

	bLinkage = bAsLinkage;
	SFX_DISPATCHER().Execute(SID_GALLERY_FORMATS, SFX_CALLMODE_ASYNCHRON | SFX_CALLMODE_RECORD, &aItem, 0L);

	// Focus auf das EditWindow setzen
	SfxViewShell*	pShell = SfxViewShell::Current();
	Window*			pWin = pShell ? pShell->GetWindow() : NULL;

	if( pWin )
		SFX_APP()->GrabFocus( pWin );
}


// ------------------------------------------------------------------------

void SvxGallery::HidePreview()
{
	if (bIsPreview)
	{
		bIsPreview = FALSE;
		pWndPreview->Hide();
		pWndPreview->SetGraphic( Graphic() );
		pWndPreview->PreviewSound( String() );
		pCtlGallery->Show();
	}
}

// ------------------------------------------------------------------------

String SvxGallery::GetFilterName() const
{
	String 		aFilterName;
	SgaTheme* 	pThm = GetTheme();

	if( pThm )
	{
		const USHORT	nId = pCtlGallery->GetSelectItemId() - 1;
		SgaObjList*		pEntry = pGal->GetObjFromActualTheme( nId );
		SgaObject*		pObj = pThm->GetObj( nId );

		if( pObj && pEntry )
		{
			if ( ( pObj->GetObjKind() == SGA_OBJ_BMP ) || ( pObj->GetObjKind() == SGA_OBJ_ANIM ) )
			{
				GraphicFilter*		pFilter = GetGrfFilter();
				const DirEntry		aFilePath( pGal->GetCorrectPath( pEntry ) );
				const String		aExt( aFilePath.GetExtension() );
				Config&				rConfig = pFilter->GetConfig();
				const ByteString	aOldGroup( rConfig.GetGroup() );
				USHORT				nFilter = GRFILTER_FORMAT_DONTKNOW;

				rConfig.SetGroup( IMP_FILTERSECTION );

				for ( USHORT i = 0, nKeyCount = rConfig.GetKeyCount(); i < nKeyCount; i++ )
				{
					const String aStr( rConfig.ReadKey( i ).GetToken(2, ',').GetBuffer(), RTL_TEXTENCODING_UTF8 );

					if ( aStr.CompareIgnoreCaseToAscii( aExt ) == COMPARE_EQUAL )
					{
						nFilter = i;
						break;
					}
				}

				if( GRFILTER_FORMAT_DONTKNOW != nFilter )
					aFilterName = pFilter->GetImportFormatName( nFilter );

				rConfig.SetGroup( aOldGroup );
			}
		}

		delete pObj;
	}

	return aFilterName;
}


// ------------------------------------------------------------------------

Graphic SvxGallery::GetGraphic() const
{
	Graphic	aGraphic;

	GetTheme();
	GetParent()->EnterWait();
	pGal->GetGraphicFromActualTheme( pCtlGallery->GetSelectItemId() - 1, aGraphic );
	GetParent()->LeaveWait();

	return aGraphic;
}


// ------------------------------------------------------------------------

const DirEntry& SvxGallery::GetPath() const
{
	((SvxGallery*)this)->aObjPath = GetPathAsString();

	return aObjPath;
}


// ------------------------------------------------------------------------

String SvxGallery::GetPathAsString() const
{
	GetTheme();
	return pGal->GetPathFromActualTheme( pCtlGallery->GetSelectItemId() - 1 );
}


// ------------------------------------------------------------------------

BOOL SvxGallery::GetFmFormModel( FmFormModel& rModel ) const
{
	ULONG	nStmErr;
	BOOL	bRet;

	GetTheme();
	GetParent()->EnterWait();
	bRet = pGal->GetSdrModelFromActualTheme( pCtlGallery->GetSelectItemId() - 1, rModel, nStmErr );
	GetParent()->LeaveWait();

	return bRet;
}


// ------------------------------------------------------------------------

BOOL SvxGallery::GetGraphicCenterRect( const Graphic& rGraphic, const Window& rWin, Rectangle& rResultRect )
{
	const Size	aWinSize( rWin.GetOutputSizePixel() );
	Size		aNewSize( rWin.LogicToPixel( rGraphic.GetPrefSize(), rGraphic.GetPrefMapMode() ) );
	BOOL		bRet = FALSE;

	if ( aNewSize.Width() && aNewSize.Height() )
	{
		const double fGrfWH = (double) aNewSize.Width() / aNewSize.Height();
		const double fWinWH = (double) aWinSize.Width() / aWinSize.Height();

		// Bitmap an Fenstergroesse anpassen
		if ( fGrfWH < fWinWH )
		{
			aNewSize.Width() = (long) ( aWinSize.Height() * fGrfWH );
			aNewSize.Height()= aWinSize.Height();
		}
		else
		{
			aNewSize.Width() = aWinSize.Width();
			aNewSize.Height()= (long) ( aWinSize.Width() / fGrfWH);
		}

		const Point aNewPos( ( aWinSize.Width()  - aNewSize.Width() ) >> 1,
							 ( aWinSize.Height() - aNewSize.Height() ) >> 1 );

		rResultRect = Rectangle( aNewPos, aNewSize );
		bRet = TRUE;
	}

	return bRet;
}


// ------------------------------------------------------------------------

USHORT SvxGallery::GetSelection() const
{
	return pCtlGallery->GetSelectItemId();
}


// ------------------------------------------------------------------------

void SvxGallery::SetSelection( const USHORT nSelection )
{
	pCtlGallery->SelectItem( nSelection );
}

// ------------------------------------------------------------------------

SgaTheme* SvxGallery::GetTheme() const
{
	SgaTheme* pRet = pGal->GetActualTheme();

	if( pRet && ( pRet->GetName() != aThemeName ) )
		pRet = NULL;

	return pRet;
}
