/*************************************************************************
 *
 *  $RCSfile: gallery5.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define ITEMID_BRUSH 0

#ifdef MAC
#include <mac_start.h>
#include <types.h>
#include <files.h>
#include <mac_end.h>
#endif

#pragma hdrstop

#ifdef OS2
#include <direct.h>
#define getcwd _getcwd
#elif defined UNX || defined (MAC)
#include <unistd.h>
#else
#include <direct.h>
#endif

#include <stdlib.h>
#include <vos/mutex.hxx>
#include <tools/errinf.hxx>
#include <vcl/msgbox.hxx>
#include <vcl/wrkwin.hxx>
#include "impgrf.hxx"
#include "gallery.hrc"
#include "gallery1.hxx"
#include "gallery2.hxx"

#define MAXPATH 1024

// ------------------------------------------------------------------------

extern String   aBrowserPath;
extern String   aLastSearchPath;
extern String   aLastFilterName;
BOOL			bBrowserPathValid;

// ---------------------
// - GalleryPathDialog -
// ---------------------

GalleryPathDlg::GalleryPathDlg( Window* pParent ) :
		SfxFileDialog	( pParent, SFXWB_PATHDIALOG ),
		aCbxSubDirs		( this, GAL_RESID( RID_SVXDLG_GALLERY_CBX ) )
{
	const Size	aSizePix( LogicToPixel( Size( 6, 18 ), MAP_APPFONT ) );
	const Point aPosPix( aSizePix.Width(), GetOutputSizePixel().Height() - aSizePix.Height() );

	aCbxSubDirs.SetPosPixel( aPosPix );
	aCbxSubDirs.Show();
}

// -----------------
// - CommentDialog -
// -----------------

TitleDialog::TitleDialog( Window* pParent, const String& rOldTitle ) :
	ModalDialog	( pParent, GAL_RESID( RID_SVXDLG_GALLERY_TITLE ) ),
	maOk		( this, GAL_RESID( BTN_OK ) ),
	maCancel	( this, GAL_RESID( BTN_CANCEL ) ),
	maHelp		( this, GAL_RESID( BTN_HELP ) ),
	maGrp		( this, GAL_RESID( GRP_TITLE ) ),
	maEdit		( this, GAL_RESID( EDT_TITLE ) ) 
{
	FreeResource();
	maEdit.SetText( rOldTitle );
	maEdit.GrabFocus();
}

// ------------------
// - SgaClipBrowser -
// ------------------

SgaClipBrowser::SgaClipBrowser( Window* pWindow, const SfxItemSet& rSet ) :
		SfxTabPage			( pWindow, GAL_RESID( RID_SVXTABPAGE_GALLERYTHEME_FILES ), rSet ),
		aFtFileType         ( this, ResId(FT_FILETYPE ) ),
		aLbxFound           ( this, ResId(LBX_FOUND ) ),
		aCbbFileType        ( this, ResId(CBB_FILETYPE ) ),
		aBtnSearch          ( this, ResId(BTN_SEARCH ) ),
		aBtnTake            ( this, ResId(BTN_TAKE ) ),
		aBtnTakeAll         ( this, ResId(BTN_TAKEALL ) ),
		aCbxPreview         ( this, ResId(CBX_PREVIEW ) ),
		pWndPreview         ( new BrowserWindow( this, ResId( WND_BRSPRV ) ) ),
		bEntriesFound       (FALSE),
		nCurFilterPos		(0),
		nFirstExtFilterPos	(0),
		bInputAllowed		(TRUE),
		bSearchRecursive	(FALSE)
{
	FreeResource();
	aSound.SetNotifyHdl( LINK( this, SgaClipBrowser, SoundEndHdl ) );
}

// ------------------------------------------------------------------------

SgaClipBrowser::~SgaClipBrowser()
{
	delete pWndPreview;

	for( String* pStr = aFoundList.First(); pStr; pStr = aFoundList.Next() )
		delete pStr;

	for( void* pEntry = aFilterEntryList.First(); pEntry; pEntry = aFilterEntryList.Next() )
		delete (FilterEntry*) pEntry;
}

// ------------------------------------------------------------------------

SfxTabPage* SgaClipBrowser::Create( Window* pParent, const SfxItemSet& rSet )
{
	return new SgaClipBrowser( pParent, rSet );
}

// ------------------------------------------------------------------------

void SgaClipBrowser::SetGalleries( SgaGallery* _pGal, SvxGallery* _pSvxGal )
{
	pGal = _pGal;
	pSvxGallery = _pSvxGal;
	InitControls();
}

// ------------------------------------------------------------------------

void SgaClipBrowser::InitControls()
{
	aBtnSearch.SetClickHdl(LINK(this, SgaClipBrowser, ClickSearchHdl));
	aBtnTake.SetClickHdl(LINK(this, SgaClipBrowser, ClickTakeHdl));
	aBtnTakeAll.SetClickHdl(LINK(this, SgaClipBrowser, ClickTakeAllHdl));
	aCbxPreview.SetClickHdl(LINK(this, SgaClipBrowser, ClickPreviewHdl));
	aCbbFileType.SetSelectHdl(LINK(this, SgaClipBrowser, SelectFileTypeHdl));
	aLbxFound.SetDoubleClickHdl(LINK(this, SgaClipBrowser, DClickFoundHdl));
	aLbxFound.SetSelectHdl(LINK(this, SgaClipBrowser, SelectFoundHdl));
	aLbxFound.InsertEntry(String(GAL_RESID(RID_SVXSTR_GALLERY_NOFILES)));
	aLbxFound.Show();

	FillFilterList(NULL);

	aBtnTake.Enable();
	aBtnTakeAll.Disable();
	aCbxPreview.Disable();

	if( aBrowserPath.Len() && ( DirEntry::GetAccessDelimiter() != aBrowserPath.GetChar( aBrowserPath.Len() - 1 ) ) )
		aBrowserPath += DirEntry::GetAccessDelimiter();

	aPath = aBrowserPath;

	aPreviewTimer.SetTimeoutHdl( LINK( this, SgaClipBrowser, PreviewTimerHdl ) );
	aPreviewTimer.SetTimeout( 500 );
	bBrowserPathValid = FALSE;
}

// ------------------------------------------------------------------------

void SgaClipBrowser::FillFilterList(void* p)
{
	// Ini-Datei auslesen fuer Registrierung
	GraphicFilter*		pFilter = GetGrfFilter();
	Config&				rConfig = pFilter->GetConfig();
	const ByteString	aOldGroup( rConfig.GetGroup() );
	String				aKey;
	String				aExt;
	FilterEntry*		pFilterEntry;
	FilterEntry*		pTestEntry;
	BOOL				bInList;

	rConfig.SetGroup( IMP_FILTERSECTION );

	for( USHORT i = 0, nKeyCount = rConfig.GetKeyCount(); i < nKeyCount; i++ )
	{
		aKey = String( rConfig.ReadKey( i ).GetBuffer(), RTL_TEXTENCODING_UTF8 );

		// System-Filterextension
		aExt = String( RTL_CONSTASCII_USTRINGPARAM( "*." ) );
		aExt += aKey.GetToken( 2, ',' );
		aExt.ToLowerAscii();

		// Testen, ob diese Extension schon
		// in unserer Liste vorhanden ist
		pTestEntry = (FilterEntry*) aFilterEntryList.First();
		bInList = FALSE;
		while( pTestEntry )
		{
			if ( pTestEntry->aFilterName == aExt )
			{
				bInList = TRUE;
				break;
			}

			pTestEntry = (FilterEntry*) aFilterEntryList.Next();
		}

		// Falls Filter noch nicht vorhanden
		// nehmen wir diesen Filter in unsere Liste auf
		if ( !bInList )
		{
			pFilterEntry = new FilterEntry;
			pFilterEntry->aFilterName = aExt;
			aFilterEntryList.Insert( pFilterEntry, aCbbFileType.InsertEntry( aKey.GetToken( 0, ',' ) ) );
		}
	}

	rConfig.SetGroup( aOldGroup );

	// Klaenge als Filter hinzufuegen
	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String(GAL_RESID(RID_SVXSTR_EXTFORMAT1_SYS));
	nFirstExtFilterPos = aCbbFileType.InsertEntry(String(GAL_RESID(RID_SVXSTR_EXTFORMAT1_UI)));
	aFilterEntryList.Insert(pFilterEntry, nFirstExtFilterPos);

#ifdef MAC
	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String(GAL_RESID(RID_SVXSTR_EXTFORMAT2_SYS));
	aFilterEntryList.Insert(pFilterEntry, aCbbFileType.InsertEntry(String(GAL_RESID(RID_SVXSTR_EXTFORMAT2_UI))));
#endif

	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String(GAL_RESID(RID_SVXSTR_EXTFORMAT3_SYS));
	aFilterEntryList.Insert(pFilterEntry, aCbbFileType.InsertEntry(String(GAL_RESID(RID_SVXSTR_EXTFORMAT3_UI))));

	// 'Alle' als Filter-Auswahl
	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String( GAL_RESID( RID_SVXSTR_GALLERY_ALLFILES ) );
	aFilterEntryList.Insert(pFilterEntry, aCbbFileType. InsertEntry( pFilterEntry->aFilterName, 0 ) );

	aCbbFileType.SetText( pFilterEntry->aFilterName );
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, SelectFileTypeHdl, void *, p )
{
	String aText( aCbbFileType.GetText() );

	if( bInputAllowed && ( aLastFilterName != aText ) )
	{
		aLastFilterName = aText;

		if( bBrowserPathValid && QueryBox(this, WB_YES_NO, String( GAL_RESID( RID_SVXSTR_GALLERY_SEARCH ) ) ).
			Execute() == RET_YES )
		{
			SearchFiles();
		}
	}

	return 0L;
}

// ------------------------------------------------------------------------

void SgaClipBrowser::SearchFiles()
{
	char			aBuf[ MAXPATH ]; getcwd( aBuf, MAXPATH );
	DirEntry		aCurDir( String( aBuf, RTL_TEXTENCODING_UTF8 ) );
	SearchProgress*	pProgress = new SearchProgress( this );

	for( String* pStr = aFoundList.First(); pStr; pStr = aFoundList.Next() )
		delete pStr;
	aFoundList.Clear();

	aLbxFound.Clear();

	pProgress->SetFileType( aCbbFileType.GetText() );
	pProgress->SetDirectory( String() );
	pProgress->Update();
	pProgress->Execute();
	delete pProgress;

	if( aFoundList.Count() )
	{
		aLbxFound.SelectEntryPos( 0 );
		aBtnTakeAll.Enable();
		aCbxPreview.Enable();
		bEntriesFound = TRUE;
	}
	else
	{
		aLbxFound.InsertEntry( String( GAL_RESID( RID_SVXSTR_GALLERY_NOFILES ) ) );
		aBtnTakeAll.Disable();
		aCbxPreview.Disable();
		bEntriesFound = FALSE;
	}

	aCurDir.SetCWD();
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, ClickCloseBrowserHdl, void *, p )
{
	if( bInputAllowed )
		aPreviewTimer.Stop();

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, ClickSearchHdl, void *, p )
{
	if( bInputAllowed )
	{
		char			aBuf[ MAXPATH ]; getcwd( aBuf, MAXPATH );
		DirEntry		aCurDir( String( aBuf, RTL_TEXTENCODING_UTF8 ) );
		String 			aDlgPathName( aLastSearchPath );
		GalleryPathDlg*	pBrowser = new GalleryPathDlg( this );

		aPreviewTimer.Stop();

#ifdef MAC
		aDlgPathName += ':';
#endif
		pBrowser->SetPath( aDlgPathName );

		if( pBrowser->Execute() == RET_OK )
		{
			String aDlgPathStr( aLastSearchPath = pBrowser->GetPath() );

#ifdef MAC
			if( aDlgPathStr.Len() && aDlgPathStr[ aDlgPathStr.Len() - 1 ] == ':' )
				aDlgPathStr.Cut(aDlgPathStr.Len() - 1 );
#endif

			if( aDlgPathStr.Len() && ( DirEntry::GetAccessDelimiter() != aDlgPathStr.GetChar( aDlgPathStr.Len() - 1 ) ) )
				aDlgPathStr += DirEntry::GetAccessDelimiter();

			aPath = aBrowserPath = aDlgPathStr;
			bSearchRecursive = pBrowser->IsSubDirs();
			SearchFiles();
			bBrowserPathValid = TRUE;
		}

		nCurFilterPos = aCbbFileType.GetEntryPos( aCbbFileType.GetText() );
		aCurDir.SetCWD();
		delete pBrowser;
	}

	return 0L;
}

// ------------------------------------------------------------------------

void SgaClipBrowser::TakeFiles()
{
	SgaTheme* pThm = pGal->GetActualTheme();

	if( pThm && ( aLbxFound.GetSelectEntryCount() || ( bTakeAll && bEntriesFound ) ) )
	{
		TakeProgress* pTakeProgress = new TakeProgress(this);
		pTakeProgress->Update();
		pTakeProgress->Execute();
		delete pTakeProgress;
		pThm->Write();
	}
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, ClickPreviewHdl, void *, p )
{
	if ( bInputAllowed )
	{
		aPreviewTimer.Stop();
		aPreviewString.Erase();

		// Falls Checkbox ausgeschaltet wird, wird Preview beendet
		if( !aCbxPreview.IsChecked() )
		{
			aSound.SetSoundName( String() );
			pWndPreview->SetGraphic( Graphic() );
			pWndPreview->Invalidate();
		}
		// ansonsten Preview fuer aktuelle Selektion
		else
			DoPreview();
	}

	return 0;
}

// ------------------------------------------------------------------------

#ifdef WNT
#pragma optimize ( "", off )
#endif

void SgaClipBrowser::DoPreview()
{
	String aString( aLbxFound.GetSelectEntry() );

	if( aString != aPreviewString )
	{
		Graphic	aGraphic;

		// vermeiden, dass ein native Link angelegt wird,
		// da dies an dieser Stelle nicht noetig ist
		aGraphic.SetLink( GfxLink() );

		String	aExpanded( *aFoundList.GetObject( aLbxFound.GetEntryPos( aString ) ) );
		String	aFileExt( DirEntry( aExpanded ).GetExtension().ToLowerAscii() );
		BOOL	bNoSound1 = ( aFileExt != DirEntry( String( GAL_RESID( RID_SVXSTR_EXTFORMAT1_SYS ) ) ).GetExtension().ToLowerAscii() );
#ifdef MAC
		BOOL	bNoSound2 = ( aFileExt != DirEntry( String( GAL_RESID( RID_SVXSTR_EXTFORMAT2_SYS ) ) ).GetExtension().ToLowerAscii() );
#else
		BOOL	bNoSound2 = TRUE;
#endif
		BOOL	bNoSound3 = ( aFileExt != DirEntry( String( GAL_RESID( RID_SVXSTR_EXTFORMAT3_SYS ) ) ).GetExtension().ToLowerAscii() );

		GetParent()->EnterWait();
		bInputAllowed = FALSE;

		if( bNoSound1 && bNoSound2 && bNoSound3 )
		{
			GraphicFilter*  pFilter = GetGrfFilter();
			GalleryProgress aProgress( pFilter );

			if( pFilter->ImportGraphic( aGraphic, aExpanded, GRFILTER_FORMAT_DONTKNOW ) )
			{
				GetParent()->LeaveWait();
				ErrorHandler::HandleError( ERRCODE_IO_NOTEXISTSPATH );
				GetParent()->EnterWait();
			}
		}
		else
		{
			Bitmap aBitmap( GAL_RESID( RID_SVXBMP_GALLERY_SOUND_0 ) );
			BitmapEx aBmpEx( aBitmap, COL_LIGHTMAGENTA );
			aGraphic = aBmpEx;
			aSound.SetSoundName( aExpanded );
			aSound.Play();
		}

		pWndPreview->SetGraphic( aGraphic );
		pWndPreview->Invalidate();
		bInputAllowed = TRUE;
		GetParent()->LeaveWait();
		aPreviewString = aString;
	}
}

#ifdef WNT
#pragma optimize ( "", on )
#endif

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, ClickTakeHdl, void*, p )
{
	SgaTheme* pThm;

	if( bInputAllowed && ( ( pThm = pGal->GetActualTheme() ) != NULL ) )
	{
		aPreviewTimer.Stop();

		if( !aLbxFound.GetSelectEntryCount() || !bEntriesFound )
		{
			SvxImportGraphicDialog* pDlg = new SvxImportGraphicDialog( this, String( RTL_CONSTASCII_USTRINGPARAM( STAR_GALLERY ) ) );

			if( pDlg->Execute() == RET_OK )
			{
				Graphic aGraphic;

				// vermeiden, dass ein native Link angelegt wird, da dies an dieser Stelle nicht noetig ist
				aGraphic.SetLink( GfxLink() );

				const DirEntry	aDlgPath( pDlg->GetPath() );
				const String	aFile( aDlgPath.GetFull() );
				String			aFormat;
				const USHORT	nImportRet = SGAImport( aFile, aGraphic, aFormat, TRUE );

				if( nImportRet != SGA_IMPORT_NONE )
				{
					SgaObject* pNewObj;

					GetParent()->EnterWait();
					bInputAllowed = FALSE;

					if( SGA_IMPORT_INET == nImportRet )
						pNewObj = new SgaObjectINet( aGraphic, aFile, aFormat );
					else if ( aGraphic.IsAnimated() )
						pNewObj = new SgaObjectAnim( aGraphic, aFile, aFormat );
					else
						pNewObj = new SgaObjectBmp( aGraphic, aFile, aFormat );

					pThm->InsertObj( *pNewObj );
					delete pNewObj;
					bInputAllowed = TRUE;
					GetParent()->LeaveWait();
				}
			}

			delete pDlg;
		}
		else
		{
			bTakeAll = FALSE;
			TakeFiles();
		}
	}

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, ClickTakeAllHdl, void *, p )
{
	SgaTheme* pThm;

	if( bInputAllowed && ( ( pThm = pGal->GetActualTheme() ) != NULL ) )
	{
		aPreviewTimer.Stop();
		bTakeAll = TRUE;
		TakeFiles();
	}

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, SelectFoundHdl, void *, p )
{
	if( bInputAllowed )
	{
		BOOL bPreviewPossible = FALSE;

		aPreviewTimer.Stop();

		if( bEntriesFound )
		{
			if( aLbxFound.GetSelectEntryCount() == 1 )
			{
				aCbxPreview.Enable();
				bPreviewPossible = TRUE;
			}
			else
				aCbxPreview.Disable();

			if( aFoundList.Count() )
				aBtnTakeAll.Enable();
			else
				aBtnTakeAll.Disable();
		}

		if( bPreviewPossible && aCbxPreview.IsChecked() )
			aPreviewTimer.Start();
	}

	return 0;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, DClickFoundHdl, void *, p )
{
	if( bInputAllowed )
	{
		aPreviewTimer.Stop();

		return (aLbxFound.GetSelectEntryCount() == 1 && bEntriesFound) ?
			ClickTakeHdl(NULL) : 0;
	}
	else
		return 0;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, PreviewTimerHdl, void *, p )
{
	aPreviewTimer.Stop();
	DoPreview();
	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SgaClipBrowser, SoundEndHdl, Sound*, pSound )
{
	aSound.SetSoundName( String() );
	return 0L;
}

// ----------------
// - TakeProgress -
// ----------------

TakeProgress::TakeProgress( Window* pWindow ) :
	ModalDialog     ( pWindow, GAL_RESID( RID_SVXDLG_GALLERY_TAKE_PROGRESS ) ),
	maTakeThread	( this, (SgaClipBrowser*) pWindow, maTakenList ),
	aFtTakeFile     ( this, ResId( FT_TAKE_FILE ) ),
	aGrpTakeProgress( this, ResId( GRP_TAKE_PROGRESS ) ),
	aBtnCancel      ( this, ResId( BTN_CANCEL ) )

{
	FreeResource();
	aBtnCancel.SetClickHdl( LINK( this, TakeProgress, ClickCancelBtn ) );
}

// ------------------------------------------------------------------------

IMPL_LINK( TakeProgress, ClickCancelBtn, void*, p )
{
	maTakeThread.terminate();

	while( maTakeThread.isRunning() )
		Application::Yield();

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( TakeProgress, CleanUpHdl, void*, p )
{
	SgaClipBrowser*	mpBrowser = (SgaClipBrowser*) GetParent();
	ULONG			i, nCount;
	USHORT			nPos;

	// uebernommene Eintraege aus ListBox selektieren
	GetParent()->EnterWait();
	mpBrowser->aLbxFound.SetUpdateMode( FALSE );
	mpBrowser->aLbxFound.SetNoSelection();

	// alle uebernommenen Eintraege in ListBox markieren,
	for( i = 0UL, nCount = maTakenList.Count(); i < nCount; i++ )
		mpBrowser->aLbxFound.SelectEntryPos( (USHORT)(ULONG) maTakenList.GetObject( i ) );
	maTakenList.Clear();

	// uebernommene Eintraege aus Listen entfernen
	nCount = mpBrowser->aLbxFound.GetSelectEntryCount();

	if( mpBrowser->aLbxFound.GetEntryCount() == nCount )
	{
		for( String* pStr = mpBrowser->aFoundList.First(); pStr; pStr = mpBrowser->aFoundList.Next() )
			delete pStr;
		mpBrowser->aFoundList.Clear();
		mpBrowser->aLbxFound.Clear();
	}
	else
	{
		while( nCount-- )
		{
			mpBrowser->aLbxFound.RemoveEntry( nPos = mpBrowser->aLbxFound.GetSelectEntryPos( 0 ) );
			delete mpBrowser->aFoundList.Remove( (ULONG) nPos );
		}
	}

	mpBrowser->aLbxFound.SetUpdateMode( TRUE );
	mpBrowser->SelectFoundHdl( NULL );
	GetParent()->LeaveWait();
	EndDialog( RET_OK );
	return 0L;
}

// ------------------------------------------------------------------------

short TakeProgress::Execute()
{
	maTakeThread.create();
	return ModalDialog::Execute();
}

// --------------
// - TakeThread -
// --------------

TakeThread::TakeThread( TakeProgress* pProgess, SgaClipBrowser* pBrowser, List& rTakenList ) :
		mpProgress	( pProgess ),
		mpBrowser	( pBrowser ),
		mrTakenList	( rTakenList )
{
}

// ------------------------------------------------------------------------

void SAL_CALL TakeThread::run()
{
	FileStat					aFStat;
	DirEntry 					aFileDir;
	String						aName;
	String 						aFile;
	USHORT						nEntries;
	SgaTheme* 					pThm = mpBrowser->pGal->GetActualTheme();
	NAMESPACE_VOS( IMutex )&	rMutex = Application::GetSolarMutex();
	USHORT						nPos;
	GalleryProgress*			pStatusProgress;

	if( !pThm )
		return;

	rMutex.acquire();
	pStatusProgress = new GalleryProgress;
	nEntries = mpBrowser->bTakeAll ? mpBrowser->aLbxFound.GetEntryCount() : mpBrowser->aLbxFound.GetSelectEntryCount();
	rMutex.release();

	for( USHORT i = 0; i < nEntries && schedule(); i++ )
	{
		// kompletten Filenamen aus FoundList holen
		if( mpBrowser->bTakeAll )
			aFile = *mpBrowser->aFoundList.GetObject( nPos = i );
		else
			aFile = *mpBrowser->aFoundList.GetObject( nPos = mpBrowser->aLbxFound.GetSelectEntryPos( i ) );

		// Position in Taken-Liste uebernehmen
		mrTakenList.Insert( (void*) nPos, LIST_APPEND );

		// Dialog mit neuem Filenamen updaten
		aFStat.Update( aFileDir = DirEntry( aFile ) );

		rMutex.acquire();
		mpProgress->SetFile( aFile );
		pStatusProgress->Update( i, nEntries - 1 );

		// SoundFile?
		String	aFileExt( aFileDir.GetExtension().ToLowerAscii() );
		BOOL	bNoSound1 = ( aFileExt != DirEntry( String( GAL_RESID( RID_SVXSTR_EXTFORMAT1_SYS ) ) ).GetExtension().ToLowerAscii() );
#ifdef MAC
		BOOL	bNoSound2 = ( aFileExt != DirEntry( String( GAL_RESID( RID_SVXSTR_EXTFORMAT2_SYS ) ) ).GetExtension().ToLowerAscii() );
#else
		BOOL	bNoSound2 = TRUE;
#endif
		BOOL	bNoSound3 = ( aFileExt != DirEntry( String( GAL_RESID( RID_SVXSTR_EXTFORMAT3_SYS ) ) ).GetExtension().ToLowerAscii() );

		if( bNoSound1 && bNoSound2 && bNoSound3 )
		{
			Graphic aGraphic;

			// vermeiden, dass ein native Link angelegt wird,
			// da dies an dieser Stelle nicht noetig ist
			aGraphic.SetLink( GfxLink() );

			String			aFormat;
			const USHORT	nImportRet = SGAImport( aFile, aGraphic, aFormat );

			if( nImportRet != SGA_IMPORT_NONE )
			{
				SgaObject* pNewObj;

				if( SGA_IMPORT_INET == nImportRet )
					pNewObj = (SgaObject*) new SgaObjectINet( aGraphic, aFile, aFormat );
				else if( aGraphic.IsAnimated() )
					pNewObj = (SgaObject*) new SgaObjectAnim( aGraphic, aFile, aFormat );
				else
					pNewObj = (SgaObject*) new SgaObjectBmp( aGraphic, aFile, aFormat );

				pThm->InsertObj( *pNewObj );
				delete pNewObj;
			}
		}
		else
		{
			// restliche Sachen als Sound-Objekte aufnehmen
			SgaObjectSound aObjSound( aFile );
			pThm->InsertObj( aObjSound );
		}

		rMutex.release();
	}

	rMutex.acquire();
	delete pStatusProgress;
	rMutex.release();

	// Cleanup
	Application::PostUserEvent( LINK( mpProgress, TakeProgress, CleanUpHdl ) );
}

// ------------------
// - SearchProgress -
// ------------------

SearchProgress::SearchProgress( Window* pParent ) :
	ModalDialog     ( pParent, GAL_RESID(RID_SVXDLG_GALLERY_SEARCH_PROGRESS ) ),
	aFtSearchDir    ( this, ResId( FT_SEARCH_DIR ) ),
	aGrpSearchDir   ( this, ResId( GRP_SEARCH_DIR ) ),
	aFtSearchType   ( this, ResId( FT_SEARCH_TYPE ) ),
	aGrpSearchType  ( this, ResId( GRP_SEARCH_TYPE ) ),
	aBtnCancel      ( this, ResId( BTN_CANCEL ) ),
	maSearchThread	( this, (SgaClipBrowser*) pParent )
{
	FreeResource();
	aBtnCancel.SetClickHdl( LINK( this, SearchProgress, ClickCancelBtn ) );
}

// ------------------------------------------------------------------------

IMPL_LINK( SearchProgress, ClickCancelBtn, void*, p )
{
	maSearchThread.terminate();

	while( maSearchThread.isRunning() )
		Application::Yield();

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SearchProgress, CleanUpHdl, void*, p )
{
	EndDialog( RET_OK );
	return 0L;
}

// ------------------------------------------------------------------------

short SearchProgress::Execute()
{
	maSearchThread.create();
	return ModalDialog::Execute();
}

// ----------------
// - SearchThread -
// ----------------

SearchThread::SearchThread( SearchProgress* pProgess, SgaClipBrowser* pBrowser ) :
		mpProgress	( pProgess ),
		mpBrowser	( pBrowser )
{
	if( aBrowserPath.Len() && ( DirEntry::GetAccessDelimiter() != aBrowserPath.GetChar( aBrowserPath.Len() - 1 ) ) )
		aBrowserPath += DirEntry::GetAccessDelimiter();
}

// ------------------------------------------------------------------------

void SAL_CALL SearchThread::run()
{
	if( mpBrowser->bSearchRecursive )
		ImplSearchRecursive( aBrowserPath, 100 );
	else
		ImplSearch( aBrowserPath );

	// Cleanup
	Application::PostUserEvent( LINK( mpProgress, SearchProgress, CleanUpHdl ) );
}

// ------------------------------------------------------------------------

void SearchThread::ImplSearch( const DirEntry& rStartDir )
{
	DirEntry					aEntry( rStartDir );
	DirEntry					aFileEntry;
	String						aFileName;
	String						aFileType;
	NAMESPACE_VOS( IMutex )&	rMutex = Application::GetSolarMutex();

	// Directory im Progress-Fenster setzen
	rMutex.acquire();
	mpProgress->SetDirectory( aEntry.GetName() );
	rMutex.release();

	if( ( aFileType = mpBrowser->aCbbFileType.GetText() ).Len() )
	{
		rMutex.acquire();

		Dir     aDir;
		USHORT  i;
		USHORT	nFileNumber = mpBrowser->aCbbFileType.GetEntryPos( aFileType );
		USHORT  nFilterCount = mpBrowser->aCbbFileType.GetEntryCount();
		USHORT 	nCount;

		// Falls etwas 'per Hand' in der ComboBox eingegeben wurde
		// muss das schon uebereinstimmen mit den vorhandenen
		// Eintraegen; ansonsten wird '<Alle>' genommen
		if ( !nFileNumber || ( nFileNumber >= mpBrowser->aCbbFileType.GetEntryCount() ) )
		{
			String aWild;

			for( i = 1; i < nFilterCount; i++ )
			{
				String aFilterName( ( (FilterEntry*) mpBrowser->aFilterEntryList.GetObject( i ) )->aFilterName );

				aWild += aFilterName;
				aWild += ';';
				aWild += aFilterName.ToUpperAscii();
				aWild += ';';
			}

			Dir	aNewDir( aEntry += aWild.Erase( aWild.Len() - 1, 1 ), FSYS_KIND_FILE );
			aNewDir.Count( TRUE );
			aDir += aNewDir;
		}
		else
		{
			String aFilterName( ( (FilterEntry*) mpBrowser->aFilterEntryList.GetObject( nFileNumber ) )->aFilterName );
			String aWild( aFilterName );

			aWild += ';'; aWild += aFilterName.ToUpperAscii();
			Dir	aNewDir( aEntry += aWild, FSYS_KIND_FILE );
			aNewDir.Count( TRUE );
			aDir += aNewDir;
		}

		rMutex.release();

		// alle gefundenen Files in die ListBox uebernehmen
		for( i = 0, nCount = aDir.Count(); ( i < nCount ) && schedule(); i++ )
		{
			aFileEntry = aDir[i];

			if( aFileEntry.ToAbs() )
			{
				// in die FoundList werden die Strings komplett
				// sortiert eingetragen; die Position des Strings
				// in der ListBox wird der Position in der
				// FoundList angepasst ==> Listen laufen synchron
				String aStr( aFileEntry.GetFull() );

				rMutex.acquire();
				mpBrowser->aFoundList.Insert( new String( aStr ), LIST_APPEND );
				mpBrowser->aLbxFound.InsertEntry( ReduceString( aStr ), mpBrowser->aFoundList.Count() - 1 );
				rMutex.release();
			}
		}
	}
}

// ------------------------------------------------------------------------

void SearchThread::ImplSearchRecursive( const DirEntry& rStartDir, ULONG nDepth )
{
	if( nDepth )
	{
		DirEntry aBeginEntry( rStartDir );

		if( !aBeginEntry.SetCWD() )
			return;

#ifndef MAC
		DirEntry		aCurEntry( FSYS_FLAG_CURRENT ); aCurEntry.ToAbs();
		DirEntry		aParEntry( FSYS_FLAG_PARENT ); aParEntry.ToAbs();
		const USHORT	nLevel = aCurEntry.Level();

#else
		const USHORT	nLevel = rStartDir.Level();
#endif

		Dir aDir( aBeginEntry + DirEntry( String( '*' ) ), FSYS_KIND_DIR );

		for( USHORT i = 0; i < aDir.Count() && schedule(); i++ )
		{
			DirEntry aEntryTmp( aDir[ i ] ); aEntryTmp.ToAbs();

#ifndef MAC
			String aStrTmp( aEntryTmp.GetFull() );

			if( ( aEntryTmp != aCurEntry ) && 
			    ( aEntryTmp != aParEntry ) &&
				( aEntryTmp.Level() > nLevel ) && 
				( aStrTmp.GetChar( aStrTmp.Len() - 1 ) != '.' ) )
			{
			   ImplSearchRecursive( aEntryTmp, ( nDepth - 1 ) );
			}
#else
			if( aEntryTmp.Level() > nLevel )
				ImplSearchRecursive( aEntryTmp, ( nDepth - 1 ) );
#endif
		}

		ImplSearch( aBeginEntry );
	}
}

// ---------------------
// - ActualizeProgress -
// ---------------------

ActualizeProgress::ActualizeProgress( Window* pWindow, SgaTheme* pThm ) :
	ModalDialog				( pWindow, GAL_RESID( RID_SVXDLG_GALLERY_ACTUALIZE_PROGRESS ) ),
	aFtActualizeFile		( this, ResId( FT_ACTUALIZE_FILE ) ),
	aGrpActualizeProgress   ( this, ResId( GRP_ACTUALIZE_PROGRESS ) ),
	aBtnCancel				( this, ResId( BTN_CANCEL ) ),
	pTheme					( pThm )
{
	FreeResource();
	aBtnCancel.SetClickHdl( LINK( this, ActualizeProgress, ClickCancelBtn ) );
}

// ------------------------------------------------------------------------

short ActualizeProgress::Execute()
{
	short nRet;

	pTimer = new Timer;

	if ( pTimer )
	{
		pTimer->SetTimeoutHdl( LINK( this, ActualizeProgress, TimeoutHdl ) );
		pTimer->SetTimeout( 0 );
		pTimer->Start();
	}

	nRet = ModalDialog::Execute();

	return nRet;
}

// ------------------------------------------------------------------------

IMPL_LINK( ActualizeProgress, ClickCancelBtn, void*, p )
{
	pTheme->AbortActualize();
	EndDialog( RET_OK );

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( ActualizeProgress, TimeoutHdl, Timer*, pTimer )
{
	if ( pTimer )
	{
		pTimer->Stop();
		delete pTimer;
	}

	pTheme->Actualize( LINK( this, ActualizeProgress, ActualizeHdl ), &aStatusProgress );
	ClickCancelBtn( NULL );

	return 0;
}

// ------------------------------------------------------------------------

IMPL_LINK( ActualizeProgress, ActualizeHdl, SgaTheme*, pThm )
{
	aFtActualizeFile.SetText( ReduceString( pThm->GetActualizeFile() ) );
	aFtActualizeFile.Flush();

	return 0;
}
