/*************************************************************************
 *
 *  $RCSfile: galdlg.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <ucbhelper/content.hxx>
#include <vos/mutex.hxx>
#include <vcl/svapp.hxx>
#include <vcl/msgbox.hxx>
#include <impgrf.hxx>
#include <sfx2/inimgr.hxx>
#include "gallery1.hxx"
#include "galtheme.hxx" 
#include "galdlg.hxx" 

#ifndef _COM_SUN_STAR_SDBC_XRESULTSET_HPP_
#include <com/sun/star/sdbc/XResultSet.hpp>
#endif
#ifndef _COM_SUN_STAR_UCB_XCONTENTACCESS_HPP_
#include <com/sun/star/ucb/XContentAccess.hpp>
#endif

// --------------
// - Namespaces -
// --------------

using namespace ::ucb;
using namespace ::rtl;
using namespace ::com::sun::star;
using namespace ::com::sun::star::ucb;

// -----------
// - Defines -
// -----------

#define MAXPATH 1024

// ----------------
// - SearchThread -
// ----------------

SearchThread::SearchThread( SearchProgress* pProgess, TPGalleryThemeProperties* pBrowser, const String& rStartDir ) :
		mpProgress	( pProgess ),
		mpBrowser	( pBrowser ),
		maStartURL	( rStartDir, INET_PROT_FILE )
{
}

// ------------------------------------------------------------------------

void SAL_CALL SearchThread::run()
{
	ImplSearch( maStartURL, mpBrowser->bSearchRecursive );
	Application::PostUserEvent( LINK( mpProgress, SearchProgress, CleanUpHdl ) );
}

// ------------------------------------------------------------------------

void SearchThread::ImplSearch( const INetURLObject& rStartURL, BOOL bRecursive )
{
	String			aFileType;
	::vos::IMutex&	rMutex = Application::GetSolarMutex();

	rMutex.acquire();
	mpProgress->SetDirectory( rStartURL.GetName() );
	rMutex.release();

	if( ( aFileType = mpBrowser->aCbbFileType.GetText() ).Len() )
	{
		try
		{
			uno::Reference< XCommandEnvironment >	xEnv;
			Content	   								aCnt( rStartURL.GetMainURL(), xEnv );
			uno::Sequence< OUString >				aProps( 1 );

			aProps.getArray()[ 0 ] == OUString::createFromAscii( "Url" );
			uno::Reference< sdbc::XResultSet > xResultSet( aCnt.createCursor( aProps, INCLUDE_FOLDERS_AND_DOCUMENTS ) );

			if( xResultSet.is() )
			{
				uno::Reference< XContentAccess > xContentAccess( xResultSet, uno::UNO_QUERY );
				
				if( xContentAccess.is() )
				{
					while( xResultSet->next() && schedule() )
					{
						INetURLObject	aFoundURL( xContentAccess->queryContentIdentfierString(), INET_PROT_FILE );
						Content			aFoundCnt( aFoundURL.GetMainURL(), xEnv );

						if( bRecursive && aFoundCnt.isFolder() )
							ImplSearch( aFoundURL, TRUE );
						else if( aFoundCnt.isDocument() )
						{
							const String	aFoundExt( aFoundURL.GetExtension() );
							USHORT			nFileNumber = mpBrowser->aCbbFileType.GetEntryPos( aFileType );
							USHORT			nFilterCount = mpBrowser->aCbbFileType.GetEntryCount();
							USHORT			nBeginFilter, nEndFilter;
							BOOL			bFound = FALSE;

							if( !nFileNumber || ( nFileNumber >= mpBrowser->aCbbFileType.GetEntryCount() ) )
							{
								nBeginFilter = 1;
								nEndFilter = mpBrowser->aCbbFileType.GetEntryCount() - 1;
							}
							else
								nBeginFilter = nEndFilter = nFileNumber;

							for( USHORT i = nBeginFilter; ( i <= nEndFilter ) && !bFound; i++ )
							{
								const String aFilterName( ( (FilterEntry*) mpBrowser->aFilterEntryList.GetObject( i ) )->aFilterName );

								if( aFilterName.CompareIgnoreCaseToAscii( aFoundExt ) == COMPARE_EQUAL )
								{
									String* pFileName = new String( aFoundURL.PathToFileName() );

									rMutex.acquire();
									mpBrowser->aFoundList.Insert( pFileName, LIST_APPEND );
									mpBrowser->aLbxFound.InsertEntry( ReduceString( *pFileName ), 
																	  mpBrowser->aFoundList.Count() - 1 );
									rMutex.release();

									bFound = TRUE;
								}
							}
						}
					}
				}
			}
		}
		catch( ... )
		{
			DBG_ERROR( "SearchThread::ImplSearch: ucb error" );
		}
	}
}

// ------------------
// - SearchProgress -
// ------------------

SearchProgress::SearchProgress( Window* pParent, const String& rStartDir ) :
	ModalDialog     ( pParent, GAL_RESID(RID_SVXDLG_GALLERY_SEARCH_PROGRESS ) ),
	aFtSearchDir    ( this, ResId( FT_SEARCH_DIR ) ),
	aGrpSearchDir   ( this, ResId( GRP_SEARCH_DIR ) ),
	aFtSearchType   ( this, ResId( FT_SEARCH_TYPE ) ),
	aGrpSearchType  ( this, ResId( GRP_SEARCH_TYPE ) ),
	aBtnCancel      ( this, ResId( BTN_CANCEL ) ),
	maSearchThread	( this, (TPGalleryThemeProperties*) pParent, rStartDir )
{
	FreeResource();
	aBtnCancel.SetClickHdl( LINK( this, SearchProgress, ClickCancelBtn ) );
}

// ------------------------------------------------------------------------

IMPL_LINK( SearchProgress, ClickCancelBtn, void*, p )
{
	maSearchThread.terminate();

	while( maSearchThread.isRunning() )
		Application::Yield();

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( SearchProgress, CleanUpHdl, void*, p )
{
	EndDialog( RET_OK );
	return 0L;
}

// ------------------------------------------------------------------------

short SearchProgress::Execute()
{
	maSearchThread.create();
	return ModalDialog::Execute();
}

// --------------
// - TakeThread -
// --------------

TakeThread::TakeThread( TakeProgress* pProgess, TPGalleryThemeProperties* pBrowser, List& rTakenList ) :
		mpProgress	( pProgess ),
		mpBrowser	( pBrowser ),
		mrTakenList	( rTakenList )
{
}

// ------------------------------------------------------------------------

void SAL_CALL TakeThread::run()
{
	String				aName;
	String 				aFile;
	USHORT				nEntries;
	GalleryTheme* 		pThm = mpBrowser->GetXChgData()->pTheme;
	::vos::IMutex&		rMutex = Application::GetSolarMutex();
	USHORT				nPos;
	GalleryProgress*	pStatusProgress;

	rMutex.acquire();
	pStatusProgress = new GalleryProgress;
	nEntries = mpBrowser->bTakeAll ? mpBrowser->aLbxFound.GetEntryCount() : mpBrowser->aLbxFound.GetSelectEntryCount();
	rMutex.release();

	pThm->LockBroadcaster();

	for( USHORT i = 0; i < nEntries && schedule(); i++ )
	{
		// kompletten Filenamen aus FoundList holen
		if( mpBrowser->bTakeAll )
			aFile = *mpBrowser->aFoundList.GetObject( nPos = i );
		else
			aFile = *mpBrowser->aFoundList.GetObject( nPos = mpBrowser->aLbxFound.GetSelectEntryPos( i ) );

		// Position in Taken-Liste uebernehmen
		mrTakenList.Insert( (void*) nPos, LIST_APPEND );

		rMutex.acquire();
		mpProgress->SetFile( aFile );
		pStatusProgress->Update( i, nEntries - 1 );
		pThm->InsertURL( INetURLObject( aFile, INET_PROT_FILE ) );
		rMutex.release();
	}

	pThm->UnlockBroadcaster();
	
	rMutex.acquire();
	delete pStatusProgress;
	rMutex.release();

	// Cleanup
	Application::PostUserEvent( LINK( mpProgress, TakeProgress, CleanUpHdl ) );
}

// ----------------
// - TakeProgress -
// ----------------

TakeProgress::TakeProgress( Window* pWindow ) :
	ModalDialog     ( pWindow, GAL_RESID( RID_SVXDLG_GALLERY_TAKE_PROGRESS ) ),
	maTakeThread	( this, (TPGalleryThemeProperties*) pWindow, maTakenList ),
	aFtTakeFile     ( this, ResId( FT_TAKE_FILE ) ),
	aGrpTakeProgress( this, ResId( GRP_TAKE_PROGRESS ) ),
	aBtnCancel      ( this, ResId( BTN_CANCEL ) )

{
	FreeResource();
	aBtnCancel.SetClickHdl( LINK( this, TakeProgress, ClickCancelBtn ) );
}

// ------------------------------------------------------------------------

IMPL_LINK( TakeProgress, ClickCancelBtn, void*, p )
{
	maTakeThread.terminate();

	while( maTakeThread.isRunning() )
		Application::Yield();

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( TakeProgress, CleanUpHdl, void*, p )
{
	TPGalleryThemeProperties*	mpBrowser = (TPGalleryThemeProperties*) GetParent();
	ULONG						i, nCount;
	USHORT						nPos;

	// uebernommene Eintraege aus ListBox selektieren
	GetParent()->EnterWait();
	mpBrowser->aLbxFound.SetUpdateMode( FALSE );
	mpBrowser->aLbxFound.SetNoSelection();

	// alle uebernommenen Eintraege in ListBox markieren,
	for( i = 0UL, nCount = maTakenList.Count(); i < nCount; i++ )
		mpBrowser->aLbxFound.SelectEntryPos( (USHORT)(ULONG) maTakenList.GetObject( i ) );
	maTakenList.Clear();

	// uebernommene Eintraege aus Listen entfernen
	nCount = mpBrowser->aLbxFound.GetSelectEntryCount();

	if( mpBrowser->aLbxFound.GetEntryCount() == nCount )
	{
		for( String* pStr = mpBrowser->aFoundList.First(); pStr; pStr = mpBrowser->aFoundList.Next() )
			delete pStr;
		mpBrowser->aFoundList.Clear();
		mpBrowser->aLbxFound.Clear();
	}
	else
	{
		while( nCount-- )
		{
			mpBrowser->aLbxFound.RemoveEntry( nPos = mpBrowser->aLbxFound.GetSelectEntryPos( 0 ) );
			delete mpBrowser->aFoundList.Remove( (ULONG) nPos );
		}
	}

	mpBrowser->aLbxFound.SetUpdateMode( TRUE );
	mpBrowser->SelectFoundHdl( NULL );
	GetParent()->LeaveWait();
	EndDialog( RET_OK );
	return 0L;
}

// ------------------------------------------------------------------------

short TakeProgress::Execute()
{
	maTakeThread.create();
	return ModalDialog::Execute();
}

// ---------------------
// - ActualizeProgress -
// ---------------------

ActualizeProgress::ActualizeProgress( Window* pWindow, GalleryTheme* pThm ) :
	ModalDialog				( pWindow, GAL_RESID( RID_SVXDLG_GALLERY_ACTUALIZE_PROGRESS ) ),
	aFtActualizeFile		( this, ResId( FT_ACTUALIZE_FILE ) ),
	aGrpActualizeProgress   ( this, ResId( GRP_ACTUALIZE_PROGRESS ) ),
	aBtnCancel				( this, ResId( BTN_CANCEL ) ),
	pTheme					( pThm )
{
	FreeResource();
	aBtnCancel.SetClickHdl( LINK( this, ActualizeProgress, ClickCancelBtn ) );
}

// ------------------------------------------------------------------------

short ActualizeProgress::Execute()
{
	short nRet;

	pTimer = new Timer;

	if ( pTimer )
	{
		pTimer->SetTimeoutHdl( LINK( this, ActualizeProgress, TimeoutHdl ) );
		pTimer->SetTimeout( 0 );
		pTimer->Start();
	}

	nRet = ModalDialog::Execute();

	return nRet;
}

// ------------------------------------------------------------------------

IMPL_LINK( ActualizeProgress, ClickCancelBtn, void*, p )
{
	pTheme->AbortActualize();
	EndDialog( RET_OK );

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( ActualizeProgress, TimeoutHdl, Timer*, pTimer )
{
	if ( pTimer )
	{
		pTimer->Stop();
		delete pTimer;
	}

	pTheme->Actualize( LINK( this, ActualizeProgress, ActualizeHdl ), &aStatusProgress );
	ClickCancelBtn( NULL );

	return 0;
}

// ------------------------------------------------------------------------

IMPL_LINK( ActualizeProgress, ActualizeHdl, GalleryTheme*, pThm )
{
	Application::Reschedule();
	aFtActualizeFile.SetText( ReduceString( pThm->GetActualizeFile() ) );
	aFtActualizeFile.Flush();

	return 0;
}

// ---------------
// - TitleDialog -
// ---------------

TitleDialog::TitleDialog( Window* pParent, const String& rOldTitle ) :
	ModalDialog	( pParent, GAL_RESID( RID_SVXDLG_GALLERY_TITLE ) ),
	maOk		( this, GAL_RESID( BTN_OK ) ),
	maCancel	( this, GAL_RESID( BTN_CANCEL ) ),
	maHelp		( this, GAL_RESID( BTN_HELP ) ),
	maGrp		( this, GAL_RESID( GRP_TITLE ) ),
	maEdit		( this, GAL_RESID( EDT_TITLE ) ) 
{
	FreeResource();
	maEdit.SetText( rOldTitle );
	maEdit.GrabFocus();
}

// ---------------------
// - GalleryPathDialog -
// ---------------------

GalleryPathDlg::GalleryPathDlg( Window* pParent ) :
		SfxFileDialog	( pParent, SFXWB_PATHDIALOG ),
		aCbxSubDirs		( this, GAL_RESID( RID_SVXDLG_GALLERY_CBX ) )
{
	const Size	aSizePix( LogicToPixel( Size( 6, 18 ), MAP_APPFONT ) );
	const Point aPosPix( aSizePix.Width(), GetOutputSizePixel().Height() - aSizePix.Height() );

	aCbxSubDirs.SetPosPixel( aPosPix );
	aCbxSubDirs.Show();
}

// ------------------
// - GalleryPreview -
// ------------------

GalleryPreview::GalleryPreview( Window* pParent, const ResId& rResId ) :
	Window( pParent, rResId )
{
	ImplConstruct();
}

// ------------------------------------------------------------------------

GalleryPreview::GalleryPreview( Window* pParent, WinBits nWinBits ) :
	Window( pParent, nWinBits )
{
	ImplConstruct();
}

// ------------------------------------------------------------------------

GalleryPreview::~GalleryPreview()
{
}

// ------------------------------------------------------------------------

void GalleryPreview::ImplConstruct()
{
	const StyleSettings& rStyleSettings = Application::GetSettings().GetStyleSettings();

	SetBackground( Wallpaper( COL_GRAY ) );
	EnableDrop( TRUE );
	aSound.SetNotifyHdl( LINK( this, GalleryPreview, SoundEndHdl ) );
}

// ------------------------------------------------------------------------

BOOL GalleryPreview::ImplGetGraphicCenterRect( const Graphic& rGraphic, Rectangle& rResultRect ) const
{
	const Size	aWinSize( GetOutputSizePixel() );
	Size		aNewSize( LogicToPixel( rGraphic.GetPrefSize(), rGraphic.GetPrefMapMode() ) );
	BOOL		bRet = FALSE;

	if( aNewSize.Width() && aNewSize.Height() )
	{
		const double fGrfWH = (double) aNewSize.Width() / aNewSize.Height();
		const double fWinWH = (double) aWinSize.Width() / aWinSize.Height();

		// Bitmap an Fenstergroesse anpassen
		if ( fGrfWH < fWinWH )
		{
			aNewSize.Width() = (long) ( aWinSize.Height() * fGrfWH );
			aNewSize.Height()= aWinSize.Height();
		}
		else
		{
			aNewSize.Width() = aWinSize.Width();
			aNewSize.Height()= (long) ( aWinSize.Width() / fGrfWH);
		}

		const Point aNewPos( ( aWinSize.Width()  - aNewSize.Width() ) >> 1,
							 ( aWinSize.Height() - aNewSize.Height() ) >> 1 );

		rResultRect = Rectangle( aNewPos, aNewSize );
		bRet = TRUE;
	}

	return bRet;
}

// ------------------------------------------------------------------------

void GalleryPreview::MouseButtonDown( const MouseEvent& rMEvt )
{
	Window::MouseButtonDown( rMEvt );
	GetParent()->MouseButtonDown( rMEvt );
}

// ------------------------------------------------------------------------

void GalleryPreview::Paint( const Rectangle& rRect )
{
	Window::Paint( rRect );

	if( ImplGetGraphicCenterRect( aGraphicObj.GetGraphic(), aPreviewRect ) )
	{
		const Point aPos( aPreviewRect.TopLeft() );
		const Size	aSize( aPreviewRect.GetSize() );

		if( aGraphicObj.IsAnimated() )
			aGraphicObj.StartAnimation( this, aPos, aSize );
		else
			aGraphicObj.Draw( this, aPos, aSize );
	}
}

// ------------------------------------------------------------------------

void GalleryPreview::Command(const CommandEvent& rCEvt )
{
	Window::Command( rCEvt );
	GetParent()->Command( rCEvt );
}

// ------------------------------------------------------------------------

void GalleryPreview::PreviewSound( const String& rPath )
{
	aSound.SetSoundName( rPath );

	if( rPath.Len() )
		aSound.Play();
}

// ------------------------------------------------------------------------

BOOL GalleryPreview::QueryDrop( DropEvent& rDEvt )
{
	return(	GetParent()->QueryDrop( rDEvt ) );
}

// ------------------------------------------------------------------------

BOOL GalleryPreview::Drop( const DropEvent& rDEvt )
{
	return(	GetParent()->Drop( rDEvt ) );
}

// ------------------------------------------------------------------------

IMPL_LINK( GalleryPreview, SoundEndHdl, Sound*, pSound )
{
	aSound.SetSoundName( String() );
	return 0L;
}

// --------------------------
// - GalleryThemeProperties -
// --------------------------

GalleryThemeProperties::GalleryThemeProperties( Window* pParent, ExchangeData* _pData, SfxItemSet* pItemSet  ) :
			SfxTabDialog    ( pParent, GAL_RESID( RID_SVXTABDLG_GALLERYTHEME ), pItemSet ),
			pData			( _pData )
{
	FreeResource();
	
	AddTabPage( RID_SVXTABPAGE_GALLERY_GENERAL, TPGalleryThemeGeneral::Create, 0 );
	AddTabPage( RID_SVXTABPAGE_GALLERYTHEME_FILES, TPGalleryThemeProperties::Create, 0 );

	if( pData->pTheme->IsReadOnly() )
		RemoveTabPage( RID_SVXTABPAGE_GALLERYTHEME_FILES );

	String aText( GetText() );

	aText += pData->pTheme->GetName();

	if( pData->pTheme->IsReadOnly() )
		aText += String( GAL_RESID( RID_SVXSTR_GALLERY_READONLY ) );

	SetText( aText );
}

// ------------------------------------------------------------------------

void GalleryThemeProperties::PageCreated( USHORT nId, SfxTabPage &rPage )
{
	if( RID_SVXTABPAGE_GALLERY_GENERAL == nId )
		( (TPGalleryThemeGeneral&) rPage ).SetXChgData( pData );
	else
		( (TPGalleryThemeProperties&) rPage ).SetXChgData( pData );
}

// -------------------------
// - TPGalleryThemeGeneral -
// -------------------------

TPGalleryThemeGeneral::TPGalleryThemeGeneral( Window* pParent, const SfxItemSet& rSet ) :
			SfxTabPage              ( pParent, GAL_RESID( RID_SVXTABPAGE_GALLERY_GENERAL ), rSet ),
			aFiMSImage              ( this, GAL_RESID( FI_MS_IMAGE ) ),
			aEdtMSName              ( this, GAL_RESID( EDT_MS_NAME ) ),
			aFlMSGeneralFirst       ( this, GAL_RESID( FL_MS_GENERAL_FIRST ) ),
			aFtMSType               ( this, GAL_RESID( FT_MS_TYPE ) ),
			aFtMSShowType           ( this, GAL_RESID( FT_MS_SHOW_TYPE ) ),
			aFtMSPath               ( this, GAL_RESID( FT_MS_PATH ) ),
			aFtMSShowPath           ( this, GAL_RESID( FT_MS_SHOW_PATH ) ),
			aFtMSContent            ( this, GAL_RESID( FT_MS_CONTENT ) ),
			aFtMSShowContent        ( this, GAL_RESID( FT_MS_SHOW_CONTENT ) ),
			aFlMSGeneralSecond      ( this, GAL_RESID( FL_MS_GENERAL_SECOND ) ),
			aFtMSCreateDate         ( this, GAL_RESID( FT_MS_CREATEDATE ) ),
			aFtMSShowCreateDate     ( this, GAL_RESID( FT_MS_SHOW_CREATEDATE ) ),
			aFtMSChangeDate         ( this, GAL_RESID( FT_MS_CHANGEDATE ) ),
			aFtMSShowChangeDate     ( this, GAL_RESID( FT_MS_SHOW_CHANGEDATE ) )
{
	FreeResource();
}

// ------------------------------------------------------------------------

void TPGalleryThemeGeneral::SetXChgData( ExchangeData* _pData )
{
	pData = _pData;

	const International aInternational( Application::GetAppInternational() );
	GalleryTheme*		pThm = pData->pTheme;
	String              aOutStr( String::CreateFromInt32( pThm->GetObjectCount() ) );
	String              aObjStr( GAL_RESID( RID_SVXSTR_GALLERYPROPS_OBJECT ) );
	String              aAccess;
	String				aType( GAL_RESID( RID_SVXSTR_GALLERYPROPS_GALTHEME ) );
	BOOL				bReadOnly = pThm->IsReadOnly() && !pThm->IsImported();

	aEdtMSName.SetText( pThm->GetName() );
	aEdtMSName.SetReadOnly( bReadOnly );

	if( bReadOnly )
		aEdtMSName.Disable();
	else
		aEdtMSName.Enable();
		
	if( pThm->IsReadOnly() )
		aType += String( GAL_RESID( RID_SVXSTR_GALLERY_READONLY ) );
	
	aFtMSShowType.SetText( aType );
	aFtMSShowPath.SetText( pThm->GetSdgPath() );

	// Ein- oder Mehrzahl?
	if ( 1 == pThm->GetObjectCount() )
		aObjStr = aObjStr.GetToken( 0 );
	else
		aObjStr = aObjStr.GetToken( 1 );

	aOutStr += ' ';
	aOutStr += aObjStr;

	aFtMSShowContent.SetText( aOutStr );

	// CreateDate/Time
	aAccess = aInternational.GetDate( pData->aThemeCreateDate );
	aAccess += String( RTL_CONSTASCII_USTRINGPARAM( ", " ) );
	aAccess += aInternational.GetTime( pData->aThemeCreateTime );
	aFtMSShowCreateDate.SetText( aAccess );

	// ChangeDate/Time
	aAccess = aInternational.GetDate( pData->aThemeChangeDate );
	aAccess += String( RTL_CONSTASCII_USTRINGPARAM( ", " ) );
	aAccess += aInternational.GetTime( pData->aThemeChangeTime );
	aFtMSShowChangeDate.SetText( aAccess );

	// Image setzen
	USHORT nId;

	if( pThm->IsImported() )
		nId = RID_SVXBMP_THEME_IMPORTED_BIG;
	else if( pThm->IsReadOnly() )
		nId = RID_SVXBMP_THEME_READONLY_BIG;
	else if( pThm->IsDefault() )
		nId = RID_SVXBMP_THEME_DEFAULT_BIG;
	else 
		nId = RID_SVXBMP_THEME_NORMAL_BIG;

	aFiMSImage.SetImage( Image( Bitmap( GAL_RESID( nId ) ), COL_LIGHTMAGENTA ) );
}

// ------------------------------------------------------------------------

BOOL TPGalleryThemeGeneral::FillItemSet( SfxItemSet& rSet )
{
	pData->aEditedTitle = aEdtMSName.GetText();
	return TRUE;
}

// ------------------------------------------------------------------------

SfxTabPage* TPGalleryThemeGeneral::Create( Window* pParent, const SfxItemSet& rSet )
{
	return new TPGalleryThemeGeneral( pParent, rSet );
}

// ----------------------------
// - TPGalleryThemeProperties -
// ----------------------------

TPGalleryThemeProperties::TPGalleryThemeProperties( Window* pWindow, const SfxItemSet& rSet ) :
		SfxTabPage			( pWindow, GAL_RESID( RID_SVXTABPAGE_GALLERYTHEME_FILES ), rSet ),
		aFtFileType         ( this, GAL_RESID(FT_FILETYPE ) ),
		aLbxFound           ( this, GAL_RESID(LBX_FOUND ) ),
		aCbbFileType        ( this, GAL_RESID(CBB_FILETYPE ) ),
		aBtnSearch          ( this, GAL_RESID(BTN_SEARCH ) ),
		aBtnTake            ( this, GAL_RESID(BTN_TAKE ) ),
		aBtnTakeAll         ( this, GAL_RESID(BTN_TAKEALL ) ),
		aCbxPreview         ( this, GAL_RESID(CBX_PREVIEW ) ),
		aWndPreview         ( this, GAL_RESID( WND_BRSPRV ) ),
		bEntriesFound       (FALSE),
		nCurFilterPos		(0),
		nFirstExtFilterPos	(0),
		bInputAllowed		(TRUE),
		bSearchRecursive	(FALSE)
{
	FreeResource();
}

// ------------------------------------------------------------------------

void TPGalleryThemeProperties::SetXChgData( ExchangeData* _pData )
{
	pData = _pData;

	aSound.SetNotifyHdl( LINK( this, TPGalleryThemeProperties, SoundEndHdl ) );
	aPreviewTimer.SetTimeoutHdl( LINK( this, TPGalleryThemeProperties, PreviewTimerHdl ) );
	aPreviewTimer.SetTimeout( 500 );
	aBtnSearch.SetClickHdl(LINK(this, TPGalleryThemeProperties, ClickSearchHdl));
	aBtnTake.SetClickHdl(LINK(this, TPGalleryThemeProperties, ClickTakeHdl));
	aBtnTakeAll.SetClickHdl(LINK(this, TPGalleryThemeProperties, ClickTakeAllHdl));
	aCbxPreview.SetClickHdl(LINK(this, TPGalleryThemeProperties, ClickPreviewHdl));
	aCbbFileType.SetSelectHdl(LINK(this, TPGalleryThemeProperties, SelectFileTypeHdl));
	aLbxFound.SetDoubleClickHdl(LINK(this, TPGalleryThemeProperties, DClickFoundHdl));
	aLbxFound.SetSelectHdl(LINK(this, TPGalleryThemeProperties, SelectFoundHdl));
	aLbxFound.InsertEntry(String(GAL_RESID(RID_SVXSTR_GALLERY_NOFILES)));
	aLbxFound.Show();

	FillFilterList();

	aBtnTake.Enable();
	aBtnTakeAll.Disable();
	aCbxPreview.Disable();
}


// ------------------------------------------------------------------------

TPGalleryThemeProperties::~TPGalleryThemeProperties()
{
	for( String* pStr = aFoundList.First(); pStr; pStr = aFoundList.Next() )
		delete pStr;

	for( void* pEntry = aFilterEntryList.First(); pEntry; pEntry = aFilterEntryList.Next() )
		delete (FilterEntry*) pEntry;
}

// ------------------------------------------------------------------------

SfxTabPage* TPGalleryThemeProperties::Create( Window* pParent, const SfxItemSet& rSet )
{
	return new TPGalleryThemeProperties( pParent, rSet );
}

// ------------------------------------------------------------------------

void TPGalleryThemeProperties::FillFilterList()
{
	GraphicFilter*		pFilter = GetGrfFilter();
	Config&				rConfig = pFilter->GetConfig();
	const ByteString	aOldGroup( rConfig.GetGroup() );
	String				aKey;
	String				aExt;
	FilterEntry*		pFilterEntry;
	FilterEntry*		pTestEntry;
	BOOL				bInList;

	rConfig.SetGroup( IMP_FILTERSECTION );

	for( USHORT i = 0, nKeyCount = rConfig.GetKeyCount(); i < nKeyCount; i++ )
	{
		aKey = String( rConfig.ReadKey( i ).GetBuffer(), RTL_TEXTENCODING_UTF8 );

		aExt = aKey.GetToken( 2, ',' );
		pTestEntry = (FilterEntry*) aFilterEntryList.First();
		bInList = FALSE;
		
		while( pTestEntry )
		{
			if ( pTestEntry->aFilterName == aExt )
			{
				bInList = TRUE;
				break;
			}

			pTestEntry = (FilterEntry*) aFilterEntryList.Next();
		}

		if ( !bInList )
		{
			pFilterEntry = new FilterEntry;
			pFilterEntry->aFilterName = aExt;
			aFilterEntryList.Insert( pFilterEntry, aCbbFileType.InsertEntry( aKey.GetToken( 0, ',' ) ) );
		}
	}

	rConfig.SetGroup( aOldGroup );

	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String(GAL_RESID(RID_SVXSTR_EXTFORMAT1_SYS));
	nFirstExtFilterPos = aCbbFileType.InsertEntry(String(GAL_RESID(RID_SVXSTR_EXTFORMAT1_UI)));
	aFilterEntryList.Insert(pFilterEntry, nFirstExtFilterPos);

#ifdef MAC
	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String(GAL_RESID(RID_SVXSTR_EXTFORMAT2_SYS));
	aFilterEntryList.Insert(pFilterEntry, aCbbFileType.InsertEntry(String(GAL_RESID(RID_SVXSTR_EXTFORMAT2_UI))));
#endif

	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String(GAL_RESID(RID_SVXSTR_EXTFORMAT3_SYS));
	aFilterEntryList.Insert(pFilterEntry, aCbbFileType.InsertEntry(String(GAL_RESID(RID_SVXSTR_EXTFORMAT3_UI))));

	// 'Alle' als Filter-Auswahl
	pFilterEntry = new FilterEntry;
	pFilterEntry->aFilterName = String( GAL_RESID( RID_SVXSTR_GALLERY_ALLFILES ) );
	aFilterEntryList.Insert(pFilterEntry, aCbbFileType. InsertEntry( pFilterEntry->aFilterName, 0 ) );

	aCbbFileType.SetText( pFilterEntry->aFilterName );
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, SelectFileTypeHdl, void *, p )
{
	String aText( aCbbFileType.GetText() );

	if( bInputAllowed && ( aLastFilterName != aText ) )
	{
		aLastFilterName = aText;

		if( QueryBox(this, WB_YES_NO, String( GAL_RESID( RID_SVXSTR_GALLERY_SEARCH ) ) ).Execute() == RET_YES )
			SearchFiles();
	}

	return 0L;
}

// ------------------------------------------------------------------------

void TPGalleryThemeProperties::SearchFiles()
{
	SearchProgress*	pProgress = new SearchProgress( this, aPath );

	for( String* pStr = aFoundList.First(); pStr; pStr = aFoundList.Next() )
		delete pStr;
	
	aFoundList.Clear();
	aLbxFound.Clear();

	pProgress->SetFileType( aCbbFileType.GetText() );
	pProgress->SetDirectory( String() );
	pProgress->Update();
	pProgress->Execute();
	delete pProgress;

	if( aFoundList.Count() )
	{
		aLbxFound.SelectEntryPos( 0 );
		aBtnTakeAll.Enable();
		aCbxPreview.Enable();
		bEntriesFound = TRUE;
	}
	else
	{
		aLbxFound.InsertEntry( String( GAL_RESID( RID_SVXSTR_GALLERY_NOFILES ) ) );
		aBtnTakeAll.Disable();
		aCbxPreview.Disable();
		bEntriesFound = FALSE;
	}
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, ClickCloseBrowserHdl, void *, p )
{
	if( bInputAllowed )
		aPreviewTimer.Stop();

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, ClickSearchHdl, void *, p )
{
	if( bInputAllowed )
	{
		GalleryPathDlg*	pBrowser = new GalleryPathDlg( this );
		String			aDlgPathName( SFX_INIMANAGER()->Get( SFX_KEY_GRAPHICS_PATH ) );

		aPreviewTimer.Stop();
		pBrowser->SetPath( aDlgPathName );

		if( pBrowser->Execute() == RET_OK )
		{
			aPath = pBrowser->GetPath();
			bSearchRecursive = pBrowser->IsSubDirs();
			SearchFiles();
		}

		nCurFilterPos = aCbbFileType.GetEntryPos( aCbbFileType.GetText() );
		delete pBrowser;
	}

	return 0L;
}

// ------------------------------------------------------------------------

void TPGalleryThemeProperties::TakeFiles()
{
	if( aLbxFound.GetSelectEntryCount() || ( bTakeAll && bEntriesFound ) )
	{
		TakeProgress* pTakeProgress = new TakeProgress( this );
		pTakeProgress->Update();
		pTakeProgress->Execute();
		delete pTakeProgress;
	}
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, ClickPreviewHdl, void *, p )
{
	if ( bInputAllowed )
	{
		aPreviewTimer.Stop();
		aPreviewString.Erase();

		if( !aCbxPreview.IsChecked() )
		{
			aSound.SetSoundName( String() );
			aWndPreview.SetGraphic( Graphic() );
			aWndPreview.Invalidate();
		}
		else
			DoPreview();
	}

	return 0;
}

// ------------------------------------------------------------------------

void TPGalleryThemeProperties::DoPreview()
{
	String aString( aLbxFound.GetSelectEntry() );

	if( aString != aPreviewString )
	{
		Graphic	aGraphic;

		// vermeiden, dass ein native Link angelegt wird,
		// da dies an dieser Stelle nicht noetig ist
		aGraphic.SetLink( GfxLink() );

		String	aExpanded( *aFoundList.GetObject( aLbxFound.GetEntryPos( aString ) ) );
		String	aFileExt( INetURLObject( aExpanded, INET_PROT_FILE ).GetExtension().ToLowerAscii() );
		BOOL	bNoSound1 = ( aFileExt != INetURLObject( String( GAL_RESID( RID_SVXSTR_EXTFORMAT1_SYS ) ), INET_PROT_FILE ).GetExtension().ToLowerAscii() );
#ifdef MAC
		BOOL	bNoSound2 = ( aFileExt != INetURLObject( String( GAL_RESID( RID_SVXSTR_EXTFORMAT2_SYS ) ), INET_PROT_FILE ).GetExtension().ToLowerAscii() );
#else
		BOOL	bNoSound2 = TRUE;
#endif
		BOOL	bNoSound3 = ( aFileExt != INetURLObject( String( GAL_RESID( RID_SVXSTR_EXTFORMAT3_SYS ) ), INET_PROT_FILE ).GetExtension().ToLowerAscii() );

		bInputAllowed = FALSE;

		if( bNoSound1 && bNoSound2 && bNoSound3 )
		{
			GraphicFilter*  pFilter = GetGrfFilter();
			GalleryProgress aProgress( pFilter );

			if( pFilter->ImportGraphic( aGraphic, INetURLObject( aExpanded, INET_PROT_FILE ), GRFILTER_FORMAT_DONTKNOW ) )
			{
				GetParent()->LeaveWait();
				ErrorHandler::HandleError( ERRCODE_IO_NOTEXISTSPATH );
				GetParent()->EnterWait();
			}
		}
		else
		{
			Bitmap aBitmap( GAL_RESID( RID_SVXBMP_GALLERY_SOUND_0 ) );
			BitmapEx aBmpEx( aBitmap, COL_LIGHTMAGENTA );
			aGraphic = aBmpEx;
			aSound.SetSoundName( aExpanded );
			aSound.Play();
		}

		aWndPreview.SetGraphic( aGraphic );
		aWndPreview.Invalidate();
		bInputAllowed = TRUE;
		aPreviewString = aString;
	}
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, ClickTakeHdl, void*, p )
{
	if( bInputAllowed )
	{
		aPreviewTimer.Stop();

		if( !aLbxFound.GetSelectEntryCount() || !bEntriesFound )
		{
			SvxImportGraphicDialog* pDlg = new SvxImportGraphicDialog( this, String( RTL_CONSTASCII_USTRINGPARAM( "Gallery" ) ) );

			if( pDlg->Execute() == RET_OK )
				pData->pTheme->InsertURL( INetURLObject( pDlg->GetPath(), INET_PROT_FILE ) );

			delete pDlg;
		}
		else
		{
			bTakeAll = FALSE;
			TakeFiles();
		}
	}

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, ClickTakeAllHdl, void *, p )
{
	if( bInputAllowed )
	{
		aPreviewTimer.Stop();
		bTakeAll = TRUE;
		TakeFiles();
	}

	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, SelectFoundHdl, void *, p )
{
	if( bInputAllowed )
	{
		BOOL bPreviewPossible = FALSE;

		aPreviewTimer.Stop();

		if( bEntriesFound )
		{
			if( aLbxFound.GetSelectEntryCount() == 1 )
			{
				aCbxPreview.Enable();
				bPreviewPossible = TRUE;
			}
			else
				aCbxPreview.Disable();

			if( aFoundList.Count() )
				aBtnTakeAll.Enable();
			else
				aBtnTakeAll.Disable();
		}

		if( bPreviewPossible && aCbxPreview.IsChecked() )
			aPreviewTimer.Start();
	}

	return 0;
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, DClickFoundHdl, void *, p )
{
	if( bInputAllowed )
	{
		aPreviewTimer.Stop();

		return (aLbxFound.GetSelectEntryCount() == 1 && bEntriesFound) ?
			ClickTakeHdl(NULL) : 0;
	}
	else
		return 0;
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, PreviewTimerHdl, void *, p )
{
	aPreviewTimer.Stop();
	DoPreview();
	return 0L;
}

// ------------------------------------------------------------------------

IMPL_LINK( TPGalleryThemeProperties, SoundEndHdl, Sound*, pSound )
{
	aSound.SetSoundName( String() );
	return 0L;
}

