/*************************************************************************
 *
 *  $RCSfile: fmstl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:19 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _SVX_FMSTL_HXX
#define _SVX_FMSTL_HXX

#if !defined(__SGI_STL_VECTOR_H) || !defined(__SGI_STL_MAP_H) || !defined(__SGI_STL_MULTIMAP_H)

// (the <tools/solar.h> doesn't restore the __PRIVATE define in all cases, so we do it ourself)
#ifdef __PRIVATE
#define __PRIVATE_WAS_DEFINED
#else
#undef __PRIVATE_WAS_DEFINED
#endif

#if STLPORT_VERSION < 321
#include <tools/presys.h>
#include <vector.h>
#include <map.h>
#include <multimap.h>
#include <hash_map.h>
#include <stack.h>
#include <tools/postsys.h>
#include <tools/solar.h>
	// Der MS-VC 4.1 Compiler hat ein Problem mit der STL und dem  Operator != fuer Strings
	#ifndef _STRING_HXX //autogen
		#include <tools/string.hxx>
	#endif
	#if defined( WNT ) && defined( __SGI_STL_ITERATOR_H )
		#if defined WANT_NO_4270WARNINGS
			#pragma warning( disable: 4270 )
		#endif
		inline BOOL	operator != (String & rStr1,  const String& rStr2) { return !(operator == (rStr1, rStr2)); }
		inline BOOL	operator != (const String rStr1, String & rStr2 ) { return !(operator == (rStr1, rStr2)); }
	#endif
#else
#include <math.h> // prevent conflict between exception and std::exception
#include <vector>
#include <map>
#include <hash_map>
#include <stack>

	using namespace std;
#endif

#ifdef __PRIVATE_WAS_DEFINED
#undef __PRIVATE
#define __PRIVATE 1
#undef __PRIVATE_WAS_DEFINED
#endif

//========================================================================
// comparisation functions

//------------------------------------------------------------------------
struct FmUStringLess : public binary_function<rtl::OUString,rtl::OUString,bool>
{
	bool operator() (const rtl::OUString& x, const rtl::OUString& y) const { return x < y ? true : false; }		// construct prevents a MSVC6 warning
};

//------------------------------------------------------------------------
struct FmUStringEqual
{
	BOOL operator() (const rtl::OUString& lhs, const rtl::OUString& rhs) const { return lhs.equals( rhs );}
};

//------------------------------------------------------------------------
struct FmUStringIEqual
{
	BOOL operator() (const rtl::OUString& lhs, const rtl::OUString& rhs) const { return lhs.equalsIgnoreCase( rhs );}
};

//------------------------------------------------------------------------
struct FmUStringHash
{
	size_t operator() (const rtl::OUString& rStr) const {return rStr.hashCode();}
};

//------------------------------------------------------------------------
class FmUStringMixEqual
{
	BOOL m_bCaseSensitive;

public:
	FmUStringMixEqual(BOOL bCaseSensitive = TRUE):m_bCaseSensitive(bCaseSensitive){}
	BOOL operator() (const rtl::OUString& lhs, const rtl::OUString& rhs) const
	{
		return m_bCaseSensitive ? lhs.equals( rhs ) : lhs.equalsIgnoreCase( rhs );
	}
	BOOL isCaseSensitive() const {return m_bCaseSensitive;}
};

//------------------------------------------------------------------------
class FmUStringMixHash
{
	BOOL m_bCaseSensitive;

public:
	FmUStringMixHash(BOOL bCaseSensitive = TRUE):m_bCaseSensitive(bCaseSensitive){}
	size_t operator() (const rtl::OUString& rStr) const
	{
		return m_bCaseSensitive ? rStr.hashCode() : rStr.toUpperCase().hashCode();
	}
	BOOL isCaseSensitive() const {return m_bCaseSensitive;}
};

//------------------------------------------------------------------------
struct FmStringLess : public binary_function<String, String, bool>
{
	bool operator() (const String& x, const String& y) const { return x < y ? true : false; }		// construct prevents a MSVC6 warning
};

//==================================================================
// consistently defining stl-types
//==================================================================

#define FM_DECLARE_ITERATORS(classname)							\
	typedef classname::iterator			classname##Iterator;		\
	typedef classname::const_iterator	Const##classname##Iterator;	\

#define FM_DECLARE_MAP(keytype, valuetype, comparefct, classname)			\
	typedef NAMESPACE_STD(map)< keytype, valuetype, comparefct >	classname;	\
	FM_DECLARE_ITERATORS(classname)											\

#define FM_DECLARE_VECTOR(valuetyp, classname)				\
	typedef NAMESPACE_STD(vector)< valuetyp >		classname;	\
	FM_DECLARE_ITERATORS(classname)							\

#define FM_DECLARE_USTRINGACCESS_MAP(valuetype, classname)			\
	FM_DECLARE_MAP(rtl::OUString, valuetype, FmUStringLess, classname)	\

#define FM_DECLARE_STRINGACCESS_MAP(valuetype, classname)			\
	FM_DECLARE_MAP(String, valuetype, FmStringLess, classname)	\

#endif


#endif	// _SVX_FMSTL_HXX

