/*************************************************************************
 *
 *  $RCSfile: optldap.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// include ---------------------------------------------------------------

#ifndef _SHL_HXX
#include <tools/shl.hxx>
#endif
#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif
#ifndef _SFX_INIMGR_HXX
#include <sfx2/inimgr.hxx>
#endif
#pragma hdrstop

#define _SVX_OPTLDAP_CXX

#include "optldap.hxx"
#include "ldapdlg.hxx"
#include "dialmgr.hxx"

#include "dialogs.hrc"
#include "optldap.hrc"

DECLARE_LIST( LDAPServerList, String* )

// class SvxLDAPServerTabPage --------------------------------------------

SvxLDAPServerTabPage::SvxLDAPServerTabPage( Window* pParent,
											const SfxItemSet& rSet ) :

	SfxTabPage( pParent, SVX_RES( RID_SVXPAGE_LDAP ), rSet ),

	aServerLB	( this, ResId( LB_LDAP_SERVER ) ),
	aNewBtn		( this, ResId( BTN_LDAP_SERVER_NEW ) ),
	aEditBtn	( this, ResId( BTN_LDAP_SERVER_EDIT ) ),
	aDeleteBtn	( this, ResId( BTN_LDAP_SERVER_DELETE ) ),
#if SUPD<385
	aHelpBtn	( this, ResId( BTN_LDAP_SERVER_HELP ) ),
#endif
	aServerGB	( this, ResId( GB_LDAP_SERVER ) ),

	pInList		( NULL ),
	pOutList	( NULL )

{
	FreeResource();

	aNewBtn.SetClickHdl( LINK( this, SvxLDAPServerTabPage, NewHdl_Impl ) );
	aEditBtn.SetClickHdl( LINK( this, SvxLDAPServerTabPage, EditHdl_Impl ) );
	aDeleteBtn.SetClickHdl( LINK( this, SvxLDAPServerTabPage, DeleteHdl_Impl ) );

#if SUPD<385
	aHelpBtn.Hide();
#endif
}

// -----------------------------------------------------------------------

SvxLDAPServerTabPage::~SvxLDAPServerTabPage()
{
	USHORT i;

	for ( i = 0; i < pInList->Count(); ++i )
		delete pInList->GetObject(i);
	delete pInList;
	for ( i = 0; i < pOutList->Count(); ++i )
		delete pOutList->GetObject(i);
	delete pOutList;
}

// -----------------------------------------------------------------------

IMPL_LINK( SvxLDAPServerTabPage, NewHdl_Impl, PushButton *, EMPTYARG )
{
	DBG_ASSERT( pOutList, "pOutList == NULL" );
	SvxLDAPServerDlg_Impl aDlg( this );

	if ( aDlg.Execute() == RET_OK )
	{
		String* pStr = new String( aDlg.GetIniString() );
		pOutList->Insert( pStr, LIST_APPEND );
		USHORT nPos =
			aServerLB.InsertEntry( pStr->GetToken( 0, LDAP_TOKEN_DELIM ) );
		aServerLB.SelectEntryPos( nPos );
		aEditBtn.Enable();
		aDeleteBtn.Enable();
	}
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( SvxLDAPServerTabPage, EditHdl_Impl, PushButton *, EMPTYARG )
{
	USHORT nPos = aServerLB.GetSelectEntryPos();
	String aServer = aServerLB.GetEntry( nPos );
	SvxLDAPServerDlg_Impl aDlg( this );
	aDlg.SetText( aServer );
	aDlg.SetIniString( *pOutList->GetObject( nPos ) );

	if ( aDlg.Execute() == RET_OK )
	{
		delete pOutList->Remove( nPos );
		String* pStr = new String( aDlg.GetIniString() );
		pOutList->Insert( pStr, nPos );
		aServerLB.InsertEntry(
			pStr->GetToken( 0, LDAP_TOKEN_DELIM ), nPos + 1 );
		aServerLB.RemoveEntry( nPos );
	}
	aServerLB.SelectEntryPos( nPos );
	return 0;
}

// -----------------------------------------------------------------------

IMPL_LINK( SvxLDAPServerTabPage, DeleteHdl_Impl, PushButton *, EMPTYARG )
{
	USHORT nPos = aServerLB.GetSelectEntryPos();
	aServerLB.RemoveEntry( nPos );
	delete pOutList->Remove( nPos );

	if ( !aServerLB.GetEntryCount() )
	{
		aEditBtn.Disable();
		aDeleteBtn.Disable();
	}
	else
	{
		if ( nPos >= aServerLB.GetEntryCount() )
			nPos = aServerLB.GetEntryCount() - 1;
		aServerLB.SelectEntryPos( nPos );
	}
	return 0;
}

// -----------------------------------------------------------------------

SfxTabPage*	SvxLDAPServerTabPage::Create( Window* pParent,
										  const SfxItemSet& rSet )
{
	return new SvxLDAPServerTabPage( pParent, rSet );
}

// -----------------------------------------------------------------------

BOOL SvxLDAPServerTabPage::FillItemSet( SfxItemSet& rSet )
{
	BOOL bModified = FALSE;
	USHORT i;
	SfxIniManager* pIni = SFX_INIMANAGER();

	for ( i = 0; i < pInList->Count(); ++i )
		pIni->Delete( SFX_KEY_LDAP_SERVER, i );

	for ( i = 0; i < pOutList->Count(); ++i )
		pIni->Set( *pOutList->GetObject(i), SFX_KEY_LDAP_SERVER, i );
	return bModified;
}

// -----------------------------------------------------------------------

void SvxLDAPServerTabPage::Reset( const SfxItemSet& rSet )
{
	USHORT i = 0;

	if ( !pInList )
	{
		// beide Lisetn f"ullen
		DBG_ASSERT( !pOutList, "pInList == NULL, but pOutList != NULL" );
		pInList = new LDAPServerList;
		pOutList = new LDAPServerList;
		SfxIniManager* pIni = SFX_INIMANAGER();
		String aServer = pIni->Get( SFX_KEY_LDAP_SERVER, i++ );

		while ( aServer.Len() )
		{
			String* pInStr = new String( aServer );
			String* pOutStr = new String( aServer );
			pInList->Insert( pInStr, pInList->Count() );
			pOutList->Insert( pOutStr, pOutList->Count() );
			aServer = pIni->Get( SFX_KEY_LDAP_SERVER, i++ );
		}
	}
	else
	{
		// die Out-Liste mit der In-Liste abgleichen
		DBG_ASSERT( pOutList, "pInList != NULL, but pOutList == NULL" );
		delete pOutList;
		pOutList = new LDAPServerList;

		for ( i = 0; i < pInList->Count(); ++i )
		{
			String* pOutStr = new String( *pInList->GetObject(i) );
			pOutList->Insert( pOutStr, LIST_APPEND );
		}
	}

	// die ListBox f"ullen
	for ( i = 0; i < pInList->Count(); ++i )
	{
		String* pServer = pInList->GetObject(i);
		aServerLB.InsertEntry( pServer->GetToken( 0, LDAP_TOKEN_DELIM ) );
	}

	// ersten Eintrag selektieren oder Edit und Delete disablen
	if ( aServerLB.GetEntryCount() )
		aServerLB.SelectEntryPos(0);
	else
	{
		aEditBtn.Disable();
		aDeleteBtn.Disable();
	}
}


