/*************************************************************************
 *
 *  $RCSfile: _ximp.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:01:28 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <vcl/svapp.hxx>
#include <vcl/poly.hxx>
#include <vcl/gradient.hxx>
#include <vcl/virdev.hxx>
#include <vcl/gdimtf.hxx>
#include <vcl/metaact.hxx>
#include <vcl/salbtype.hxx>
#include "xpoly.hxx"
#include "xattr.hxx"
#include "xoutbmp.hxx"
#include "xoutx.hxx"

// -----------
// - Statics -
// -----------

#ifdef WIN
	const BOOL XIMP_bTiledPrintClip = TRUE;
#else
	const BOOL XIMP_bTiledPrintClip = FALSE;
#endif

#ifdef MAC
	const BOOL XIMP_bDrawRasterOps = FALSE;
#else
	const BOOL XIMP_bDrawRasterOps = TRUE;
#endif

// -----------
// - Defines -
// -----------

#define FILL_TILE 200

// ----------------
// - XOuputDevice -
// ----------------

void XOutputDevice::DrawFillPolyPolygon( const PolyPolygon& rPolyPoly, BOOL bRect )
{
	if( ( eFillStyle != XFILL_NONE ) && iRotTransGradient() &&
		( eFillStyle != XFILL_SOLID || iRotTransGradient()->GetStartColor() != iRotTransGradient()->GetEndColor() ) )
	{
		XGradient&		rTransGradient = *iRotTransGradient();
		Gradient		aVCLGradient;
		GDIMetaFile		aMtf;
		VirtualDevice	aVDev;
		OutputDevice*	pOldOut = pOut;
		const Rectangle	aBound( rPolyPoly.GetBoundRect() );
		MapMode			aMap( pOldOut->GetMapMode() );

		pOut = &aVDev;
		aVDev.EnableOutput( FALSE );
		aVDev.SetMapMode( pOldOut->GetMapMode() );
		aMtf.Record( &aVDev );
		aVDev.SetLineColor( pOldOut->GetLineColor() );
		aVDev.SetFillColor( pOldOut->GetFillColor() );
		aVDev.SetFont( pOldOut->GetFont() );
		aVDev.SetDrawMode( pOldOut->GetDrawMode() );
		aVDev.SetRefPoint( pOldOut->GetRefPoint() );
		ImpDrawFillPolyPolygon( rPolyPoly, bRect );
		aMtf.Stop();
		aMtf.WindStart();
		aMap.SetOrigin( aBound.TopLeft() );
		aMtf.SetPrefMapMode( aMap );
		aMtf.SetPrefSize( aBound.GetSize() );
		pOut = pOldOut;

		aVCLGradient.SetStyle((GradientStyle)rTransGradient.GetGradientStyle());
		aVCLGradient.SetStartColor(rTransGradient.GetStartColor());
		aVCLGradient.SetEndColor(rTransGradient.GetEndColor());
		aVCLGradient.SetAngle((USHORT)rTransGradient.GetAngle());
		aVCLGradient.SetBorder(rTransGradient.GetBorder());
		aVCLGradient.SetOfsX(rTransGradient.GetXOffset());
		aVCLGradient.SetOfsY(rTransGradient.GetYOffset());
		aVCLGradient.SetStartIntensity(rTransGradient.GetStartIntens());
		aVCLGradient.SetEndIntensity(rTransGradient.GetEndIntens());
		aVCLGradient.SetSteps(rTransGradient.GetSteps());

		pOut->DrawTransparent( aMtf, aBound.TopLeft(), aBound.GetSize(), aVCLGradient );
	}
	else
		ImpDrawFillPolyPolygon( rPolyPoly, bRect );
}

// ------------------------------------------------------------------------

void XOutputDevice::ImpDrawFillPolyPolygon( const PolyPolygon& rPolyPoly, BOOL bRect )
{
	if( eFillStyle != XFILL_NONE )
	{
		const Color aOldLineColor( pOut->GetLineColor() );
		const ULONG	nDrawMode = pOut->GetDrawMode();

		pOut->SetLineColor();

		if( eFillStyle == XFILL_SOLID )
		{
			if( nFillTransparence )
				pOut->DrawTransparent( rPolyPoly, nFillTransparence );
			else
				pOut->DrawPolyPolygon( rPolyPoly );
		}
		else if( eFillStyle == XFILL_HATCH )
		{
			long		nAngle10 = aHatch.GetAngle() % 3600;
			HatchStyle	eStyle;

			if( nAngle10 < 0 )
				nAngle10 += 3600;

			switch( aHatch.GetHatchStyle() )
			{
				case( XHATCH_TRIPLE ): eStyle = HATCH_TRIPLE; break;
				case( XHATCH_DOUBLE ): eStyle = HATCH_DOUBLE; break;
				
				default: 
					eStyle = HATCH_SINGLE; 
				break;
			}

			if( iSolidHatch() )
				pOut->DrawPolyPolygon( rPolyPoly );

			pOut->DrawHatch( rPolyPoly, Hatch( eStyle, aHatch.GetColor(), aHatch.GetDistance(), (USHORT) nAngle10 ) );
		}
		else if( eFillStyle == XFILL_GRADIENT )
		{
			GDIMetaFile*	pMtf = pOut->GetConnectMetaFile();
			Gradient		aVCLGradient;


			aVCLGradient.SetStyle((GradientStyle)aGradient.GetGradientStyle());
			aVCLGradient.SetStartColor(aGradient.GetStartColor());
			aVCLGradient.SetEndColor(aGradient.GetEndColor());
			aVCLGradient.SetAngle((USHORT)aGradient.GetAngle());
			aVCLGradient.SetBorder(aGradient.GetBorder());
			aVCLGradient.SetOfsX(aGradient.GetXOffset());
			aVCLGradient.SetOfsY(aGradient.GetYOffset());
			aVCLGradient.SetStartIntensity(aGradient.GetStartIntens());
			aVCLGradient.SetEndIntensity(aGradient.GetEndIntens());
			aVCLGradient.SetSteps(aGradient.GetSteps());

			if( pMtf )
			{
				ByteString aName("XGRAD_SEQ_BEGIN");
				pMtf->AddAction( new MetaCommentAction( aName ) );
				pMtf->AddAction( new MetaGradientExAction( rPolyPoly, aVCLGradient ) );
			}

			if( bRect )
				pOut->DrawGradient( rPolyPoly.GetBoundRect(), aVCLGradient );
			else
				pOut->DrawGradient( rPolyPoly, aVCLGradient );

			if( pMtf )
			{
				ByteString aName("XGRAD_SEQ_END");
				pMtf->AddAction( new MetaCommentAction( aName ) );
			}
		}
		else if( eFillStyle == XFILL_BITMAP )
		{
			if( nDrawMode & DRAWMODE_WHITEFILL )
			{
				const Color aOldFillColor( pOut->GetFillColor() );

				pOut->SetFillColor( COL_WHITE );
				pOut->DrawPolyPolygon( rPolyPoly );
				pOut->SetFillColor( aOldFillColor );
			}
			else
			{
				Rectangle       aPolyRect( rPolyPoly.GetBoundRect() );
				Rectangle       aScreenRect( pOut->PixelToLogic( Point() ), pOut->GetOutputSize() );
				GDIMetaFile*    pMtf = pOut->GetConnectMetaFile();
				const BOOL		bPrinter = ( pOut->GetOutDevType() == OUTDEV_PRINTER );

				// calculate output rect
				aScreenRect.Intersection( aPolyRect );

				// clipping set?
				if( pOut->IsClipRegion() )
					aScreenRect.Intersection( pOut->GetClipRegion().GetBoundRect() );

				pOut->Push();
				pOut->SetRasterOp( ROP_OVERPAINT );

				if( pMtf && !bPrinter )
				{
					if( XIMP_bDrawRasterOps )
					{
						pOut->SetRasterOp( ROP_XOR );
						DrawBitmapFill( aPolyRect );

						pOut->Push( PUSH_FILLCOLOR );
						pOut->SetFillColor( COL_BLACK );
						pOut->SetRasterOp( ROP_0 );
						pOut->DrawPolyPolygon( rPolyPoly );
						pOut->Pop();

						pOut->SetRasterOp( ROP_XOR );
						DrawBitmapFill( aPolyRect );
					}
					else
					{
						pOut->IntersectClipRegion( aScreenRect );
						pOut->IntersectClipRegion( rPolyPoly );
						DrawBitmapFill( aPolyRect );
					}
				}
				else if( bRect || ( bPrinter && !XIMP_bTiledPrintClip ) || ( !bPrinter && Application::IsRemoteServer() ) )
				{
					pOut->IntersectClipRegion( aScreenRect );
					pOut->IntersectClipRegion( rPolyPoly );
					DrawBitmapFill( aPolyRect );
				}
				else if( bPrinter && XIMP_bTiledPrintClip )
				{
					PolyPolygon		aClipped;
					Size            aLogTile( pOut->PixelToLogic( Size( FILL_TILE, FILL_TILE ) ) );
					Rectangle       aTile( aPolyRect.TopLeft(), aLogTile );
					long            nOff = pOut->PixelToLogic( Size( 2, 2 ) ).Width();
					long            nCountX = aPolyRect.GetWidth() / ( aLogTile.Width() - nOff ) + 1;
					long            nCountY = aPolyRect.GetHeight() / ( aLogTile.Height() - nOff) + 1;
					BOOL            bDraw;

					for( long i = 0; i < nCountY; i++ )
					{
						aTile.SetPos(Point( aPolyRect.Left(), aPolyRect.Top() + i*(aLogTile.Height() - nOff)));

						for( long j = 0; j < nCountX; j++ )
						{
							bDraw = FALSE;

							aClipped = rPolyPoly;
							aClipped.Clip( aTile );

							if( aClipped.Count() )
							{
								Region aClipRegion( aClipped );

								if( !aClipRegion.IsEmpty() )
								{
									pOut->SetClipRegion( aClipRegion );
									bDraw = TRUE;
								}
							}

							aTile.Move( aLogTile.Width() - nOff, 0 );

							if( bDraw )
								DrawBitmapFill( aPolyRect );
						}
					}
				}
				else
				{
					VirtualDevice   aMemDev;
					PolyPolygon		aPolyPoly( pOut->LogicToPixel( rPolyPoly ) );
					Rectangle		aScreenRectPix( pOut->LogicToPixel( aScreenRect ) );

					// add one pixel to avoid mapping differences
					aScreenRectPix.Right() += 2, aScreenRectPix.Bottom() += 2;

					const Size aVDevSizePix( aScreenRectPix.GetSize() );
					const Size aExtScreenRectSize( pOut->PixelToLogic( aVDevSizePix ) );

					pOut->IntersectClipRegion( aScreenRect );
					aMemDev.SetOutputSizePixel( aVDevSizePix );
					aMemDev.DrawOutDev( Point(), aVDevSizePix, aScreenRect.TopLeft(), aExtScreenRectSize, *pOut );
					DrawBitmapFill( aPolyRect );
					aMemDev.SetRasterOp( ROP_XOR );
					aMemDev.DrawOutDev( Point(), aVDevSizePix, aScreenRect.TopLeft(), aExtScreenRectSize, *pOut );

					aMemDev.Push( PUSH_FILLCOLOR );
					aMemDev.SetFillColor( COL_BLACK );
					aMemDev.SetRasterOp( ROP_0 );
					aPolyPoly.Move( -aScreenRectPix.Left(), -aScreenRectPix.Top() );
					aMemDev.DrawPolyPolygon( aPolyPoly );
					aMemDev.Pop();
					
					pOut->SetRasterOp( ROP_XOR );
					pOut->DrawOutDev( aScreenRect.TopLeft(), aExtScreenRectSize, Point(), aVDevSizePix, aMemDev );
				}

				pOut->Pop();
			}
		}

		pOut->SetLineColor( aOldLineColor );
	}
}

// ------------------------------------------------------------------------

void XOutputDevice::DrawBitmapFill( const Rectangle& rRect )
{
	// Ausgabe-Position und -Groesse bestimmen
	ImpCalcBmpFillStartValues( rRect );

	// Falls notwendig, neu dithern, berechnen usw.
	if ( bRecalc )
		ImpRecalcFillBitmap( rRect );

	const long  nWidth = aFillBitmapSize.Width();
	const long	nHeight = aFillBitmapSize.Height();

	if ( ( bBmpTile || !bBmpStretch ) && ( !nWidth || !nHeight ) )
		return;

	if ( !pOut->GetConnectMetaFile() || ( pOut->GetOutDevType() == OUTDEV_PRINTER ) )
	{
		const MapMode	aOldMap( pOut->GetMapMode() );
		const Rectangle	aRect( pOut->LogicToPixel( rRect ) );
		const Region	aOldReg( pOut->GetClipRegion() );
		const Region	aReg( pOut->LogicToPixel( aOldReg ) );
		const Rectangle aClipRect( aReg.GetBoundRect() );
		const Point		aPointPix( pOut->LogicToPixel( aStartPoint ) );
		BOOL			bClip = pOut->IsClipRegion();

		pOut->SetMapMode( MapMode() );

		if ( bClip )
			pOut->SetClipRegion( aReg );

		if ( bBmpTile )
			ImpDrawTiledBitmap( pOut, aRect, aPointPix, aClipRect );
		else
		{
			if ( bBmpStretch )
				pOut->DrawBitmap( aRect.TopLeft(), aRect.GetSize(), aFillBitmap );
			else
			{
				Point aPos( aRect.TopLeft() );

				aPos.X() += ( aRect.GetWidth() - aFillBitmapSize.Width() ) >> 1;
				aPos.Y() += ( aRect.GetHeight() - aFillBitmapSize.Height() ) >> 1;

				pOut->DrawBitmap( aPos, aFillBitmapSize, aFillBitmap );
			}
		}

		pOut->SetMapMode( aOldMap );

		if( bClip )
			pOut->SetClipRegion(aOldReg);
	}
	// Mtf-Aufzeichnung
	else
	{
		pOut->Push();

		if ( bBmpTile )
		{
			VirtualDevice	aVDev;
			MapMode			aMap( pOut->GetMapMode().GetMapUnit() );
			const Rectangle aRect( pOut->LogicToPixel( rRect, aMap ) );
			const Point		aPointPix( pOut->LogicToPixel( aStartPoint, aMap ) );

			aVDev.SetOutputSizePixel( aRect.GetSize() );
			aMap.SetMapUnit( MAP_PIXEL );
			aMap.SetOrigin( Point( -aRect.Left(), -aRect.Top() ) );
			aVDev.SetMapMode( aMap );

			ImpDrawTiledBitmap( &aVDev, aRect, aPointPix, aRect );
			pOut->DrawBitmap( rRect.TopLeft(), rRect.GetSize(), aVDev.GetBitmap( aRect.TopLeft(), aRect.GetSize() ) );
		}
		else
		{
			if ( bBmpStretch )
				pOut->DrawBitmap( rRect.TopLeft(), rRect.GetSize(), aFillBitmap );
			else
			{
				Point aPos( rRect.TopLeft() );

				aPos.X() += ( rRect.GetWidth() - aFillBitmapSize.Width() ) >> 1;
				aPos.Y() += ( rRect.GetHeight() - aFillBitmapSize.Height() ) >> 1;

				// in MetaFiles machen wir den Hintergrund weiss
				// das Poppen macht das Selektieren der Brush
				// wieder rueckgaengig
				PolyPolygon	aPolyPoly( 2 );
				aPolyPoly.Insert( Polygon( rRect ) );
				aPolyPoly.Insert( Polygon( Rectangle( aPos, aFillBitmapSize ) ) );

				pOut->SetFillColor( Color( COL_WHITE ) );
				pOut->SetLineColor();
				pOut->DrawPolyPolygon( aPolyPoly );
				pOut->DrawBitmap( aPos, aFillBitmapSize, aFillBitmap );
			}
		}

		pOut->Pop();
	}
}

// ------------------------------------------------------------------------

void XOutputDevice::ImpCalcBmpFillStartValues( const Rectangle& rRect )
{
	const MapMode	aCompMap( pOut->GetMapMode().GetMapUnit() );
	BOOL			bOriginalSize = FALSE;
	BOOL			bScaleSize = FALSE;

	// Falls keine Groessen gegeben sind ( z.B. alte Dokumente )
	// berechnen wir uns die Groesse selber aus der Bitmap
	// ==> altes Verhalten;
	// wenn nur eine Groesse gegeben ist, wird die andere
	// Groesse angepasst berechnet
	if ( bBmpLogSize )
	{
		if ( !aBmpSize.Width() && !aBmpSize.Height() )
			bOriginalSize = TRUE;
		else if ( !aBmpSize.Width() || !aBmpSize.Height() )
			bScaleSize = TRUE;
	}
	else
	{
		if ( !nBmpPerCentX && !nBmpPerCentY )
			bOriginalSize = TRUE;
		else if ( !nBmpPerCentX || !nBmpPerCentY )
			bScaleSize = TRUE;
	}

	// entweder Originalgroesse oder angepasste Groesse
	if ( bOriginalSize || bScaleSize )
	{
		Size aPrefSize( aLastAttrBitmap.GetPrefSize() );

		// Falls keine gesetzt ist, nehmen wir Pixel
		if( !aPrefSize.Width() || !aPrefSize.Height() )
			aPrefSize = aLastAttrBitmap.GetSizePixel();

		if( bOriginalSize )
		{
			const MapMode aMapMode( aLastAttrBitmap.GetPrefMapMode() );

			if( aMapMode == MAP_PIXEL )
			{
				const MapMode aMap100( MAP_100TH_MM );
				aTileSize = Application::GetDefaultDevice()->PixelToLogic( aPrefSize, aMap100 );
				aTileSize = pOut->LogicToLogic( aTileSize, aMap100, aCompMap );
			}
			else
				aTileSize = pOut->LogicToLogic( aPrefSize, aMapMode, aCompMap );
		}
		else
		{
			if ( bBmpLogSize )
			{
				if ( !aBmpSize.Width() )
					aBmpSize.Width() = FRound( (double) aBmpSize.Height() * aPrefSize.Width() / aPrefSize.Height() );
				else
					aBmpSize.Height() = FRound( (double) aBmpSize.Width() * aPrefSize.Height() / aPrefSize.Width() );

				aTileSize = aBmpSize;
			}
			else
			{
				if ( !nBmpPerCentX )
				{
					aTileSize.Height() = FRound( (double) rRect.GetHeight() * nBmpPerCentY / 100. );
					aTileSize.Width() = FRound( (double) aTileSize.Height() * aPrefSize.Width() / aPrefSize.Height() );
				}
				else
				{
					aTileSize.Width() = FRound( (double) rRect.GetWidth() * nBmpPerCentX / 100. );
					aTileSize.Height() = FRound( (double) aTileSize.Width() * aPrefSize.Height() / aPrefSize.Width() );
				}
			}
		}
	}
	// ansonsten koennen wir die Groesse leicht selber berechnen
	else
	{
		if ( bBmpLogSize )
			aTileSize = aBmpSize;
		else
		{
			aTileSize.Width() = FRound( (double) rRect.GetWidth() * nBmpPerCentX / 100. );
			aTileSize.Height() = FRound( (double) rRect.GetHeight() * nBmpPerCentY / 100. );
		}
	}

	// schneller Zugriff
	const long nWidth = aTileSize.Width();
	const long nHeight = aTileSize.Height();

	// nur bei Kachelung die anderen Positionen berechnen
	if ( bBmpTile )
	{
		// Grundposition der ersten Kachel berechen;
		// Diese Position wird spaeter zur Berechnung der absoluten
		// Startposition links oberhalb des Objektes benutzt
		switch( eBmpRectPoint )
		{
			case( RP_MT ):
			{
				aStartPoint.X() = rRect.Left() + ( ( rRect.GetWidth() - nWidth ) >> 1 );
				aStartPoint.Y() = rRect.Top();
			}
			break;

			case( RP_RT ):
			{
				aStartPoint.X() = rRect.Right() - nWidth;
				aStartPoint.Y() = rRect.Top();
			}
			break;

			case( RP_LM ):
			{
				aStartPoint.X() = rRect.Left();
				aStartPoint.Y() = rRect.Top() + ( ( rRect.GetHeight() - nHeight ) >> 1  );
			}
			break;

			case( RP_MM ):
			{
				aStartPoint.X() = rRect.Left() + ( ( rRect.GetWidth() - nWidth ) >> 1 );
				aStartPoint.Y() = rRect.Top() + ( ( rRect.GetHeight() - nHeight ) >> 1 );
			}
			break;

			case( RP_RM ):
			{
				aStartPoint.X() = rRect.Right() - nWidth;
				aStartPoint.Y() = rRect.Top() + ( ( rRect.GetHeight() - nHeight ) >> 1 );
			}
			break;

			case( RP_LB ):
			{
				aStartPoint.X() = rRect.Left();
				aStartPoint.Y() = rRect.Bottom() - nHeight;
			}
			break;

			case( RP_MB ):
			{
				aStartPoint.X() = rRect.Left() + ( ( rRect.GetWidth() - nWidth ) >> 1 );
				aStartPoint.Y() = rRect.Bottom() - nHeight;
			}
			break;

			case( RP_RB ):
			{
				aStartPoint.X() = rRect.Right() - nWidth;
				aStartPoint.Y() = rRect.Bottom() - nHeight;
			}
			break;

			// default linke obere Ecke
			default:
				aStartPoint = rRect.TopLeft();
			break;
		}

		// X- oder Y-Positionsoffset beruecksichtigen
		if ( nBmpOffPosX )
			aStartPoint.X() += ( nWidth * nBmpOffPosX / 100 );
		if ( nBmpOffPosY )
			aStartPoint.Y() += ( nHeight * nBmpOffPosY / 100 );

		// echten Startpunkt berechnen ( links oben )
		if ( nWidth && nHeight )
		{
			const long nDiffX = aStartPoint.X() - rRect.Left();
			const long nDiffY = aStartPoint.Y() - rRect.Top();

			if ( nDiffX )
			{
				long nCount = nDiffX / nWidth + 1;

				if ( nBmpOffY && ( nCount & 1L ) )
					nCount++;

				aStartPoint.X() -= ( nCount * nWidth );
			}

			if ( nDiffY )
			{
				long nCount = nDiffY / nHeight + 1;

				if ( nBmpOffX && ( nCount & 1L ) )
					nCount++;

				aStartPoint.Y() -= ( nCount * nHeight );
			}
		}
	}
}

// ------------------------------------------------------------------------

void XOutputDevice::ImpRecalcFillBitmap( const Rectangle& rRect )
{
	Bitmap				aBmp( aLastAttrBitmap );
	const MapMode 		aCompMap( pOut->GetMapMode().GetMapUnit() );
	Size				aSizePix;
	GDIMetaFile*		pMtf = pOut->GetConnectMetaFile();
	const OutDevType	eOutDevType = pOut->GetOutDevType();
	BOOL				bBrushSize 	= ( aSizePix.Width() == 8 ) &&
									  ( aSizePix.Height() == 8 );

	// Kachelung
	if ( bBmpTile )
	{
		VirtualDevice*	pVDev = new VirtualDevice;
		BOOL			bNeedsDither;

		// Pixelgroesse eines Bildes ermitteln
		aSizePix = pOut->LogicToPixel( aTileSize, aCompMap );

		// evtl. Zoomfaktoren beruecksichtigen; nur, wenn kein Mtf aufgezeichnet wird
		if ( !pMtf )
		{
			aSizePix.Width() = (long) ( (double) aLastMapMode.GetScaleX() * aSizePix.Width() );
			aSizePix.Height() = (long) ( (double) aLastMapMode.GetScaleY() * aSizePix.Height() );
		}

		// Groesse soll in jeder Richtung min. 2 Pixel sein
		aSizePix.Width() = Max( aSizePix.Width(), 2L );
		aSizePix.Height() = Max( aSizePix.Height(), 2L );

		// Bei kleinen Bitmaps (entpr. Brushes)
		// werden diese bis min. 200x200 gekachelt
		if( aSizePix.Width() < 70 && aSizePix.Height() < 70 )
		{
			long nStretchLen = 200;

			// ja, wir erzeugen uns ein Intermediate-Bitmap
			bIntermediateBmp = TRUE;

			// evtl. Zoomfaktoren beruecksichtigen nur, wenn kein Mtf aufgezeichnet wird
			if ( !pMtf )
			{
				Fraction aFrac( aLastMapMode.GetScaleX() );

				aFrac += aLastMapMode.GetScaleY();
				aFrac *= Fraction( 1, 2 );
				nStretchLen  = (long) ( (double) aFrac * nStretchLen );
			}

			nStretchLen = Min( nStretchLen, pOut->GetOutputSizePixel().Width() );
			nStretchLen = Max( Max( nStretchLen, aSizePix.Width() ), aSizePix.Height() );

			long nCountX = (USHORT) ( nStretchLen / aSizePix.Width() + 1 );
			long nCountY = (USHORT) ( nStretchLen / aSizePix.Height() + 1 );

			// Wir wollen nur ganzzahlige Kachelanzahl
			if ( nCountX & 1 )
				nCountX++;
			if ( nCountY & 1 )
				nCountY++;

			// Bmp kacheln; ggf. dithern
			if ( eOutDevType != OUTDEV_PRINTER )
				DitherBitmap( aBmp );

			// Intermediate-Bitmap erzeugen
			Point		aOrg;
			const long	nWidth = aSizePix.Width();
			const long	nHeight = aSizePix.Height();
			const Size	aNewSize( nCountX * nWidth, nCountY * nHeight );
			const long	nOffX = nWidth - nWidth * nBmpOffX / 100;
			const long	nOffY = nHeight - nHeight * nBmpOffY / 100;
			long		nXPos = 0;
			long		nYPos = 0;

			// Falls Kacheln untereinander verschoben,
			// muessen wir noch eine zusaetzliche Zeile
			// oder Spalte ausgeben
			if ( nBmpOffX )
				nCountX++;
			else if ( nBmpOffY )
				nCountY++;

			// OutDev setzen
			pVDev->SetOutputSizePixel( aNewSize );

			// Kachelung ausgeben
			for ( long nY = 0; nY < nCountY; nY++ )
			{
				for ( long nX = 0; nX < nCountX; nX++ )
				{
					if ( nBmpOffX && ( nY & 1 ) )
						aOrg = Point( nXPos - nOffX, nYPos );
					else if ( nBmpOffY && ( nX & 1 ) )
						aOrg = Point( nXPos, nYPos - nOffY );
					else
						aOrg = Point( nXPos, nYPos );

					pVDev->DrawBitmap( aOrg, aSizePix, aBmp );

					nXPos += nWidth;
				}

				nXPos = 0;
				nYPos += nHeight;
			}

			// neue Bitmap aus VDev ziehen
			aBmp = pVDev->GetBitmap( Point(), aNewSize );
			aFillBitmapSize = aLastFillBitmapSize = aNewSize;
			bNeedsDither = FALSE;
		}
		else
		{
			bIntermediateBmp = FALSE;
			aFillBitmapSize = aLastFillBitmapSize = aSizePix;
			bNeedsDither = TRUE;
		}

		// VDev zerstoeren
		delete pVDev;

		// Falls Brush-Bitmapfuellung oder geringe
		// Farbtiefe, braucht nicht gedithert zu werden
		if ( bBrushSize )
			aLastFillBitmap = aFillBitmap = aBmp;
		else
		{
			aFillBitmap = aBmp;
			if ( ( eOutDevType != OUTDEV_PRINTER ) && bNeedsDither )
				DitherBitmap( aFillBitmap );
			aLastFillBitmap = aFillBitmap;
		}
	}
	else
	{
		// Falls Brush-Bitmapfuellung oder geringe
		// Farbtiefe, braucht nicht gedithert zu werden
		if ( bBrushSize )
			aLastFillBitmap = aFillBitmap = aBmp;
		else
		{
			aFillBitmap = aBmp;

			if ( eOutDevType != OUTDEV_PRINTER )
				DitherBitmap( aFillBitmap );

			aLastFillBitmap = aFillBitmap;
		}

		// bei Stretchen ermitteln wir direkt die Groesse am Objekt
		if ( !bBmpStretch )
		{
			// evtl. Zoomfaktoren beruecksichtigen;
			// nur, wenn kein Mtf aufgezeichnet
			if ( !pMtf || ( eOutDevType == OUTDEV_PRINTER ) )
			{
				aSizePix = pOut->LogicToPixel( aTileSize, aCompMap );

				// auf dem Bildschirm muessen wir noch Zoom beruecksichtigen
				if( !pMtf )
				{
					aSizePix.Width() = aLastMapMode.GetScaleX() * Fraction( aSizePix.Width() );
					aSizePix.Height() = aLastMapMode.GetScaleY() * Fraction( aSizePix.Height() );
				}
			}
			else
				aSizePix = pOut->LogicToLogic( aTileSize, aCompMap, pOut->GetMapMode() );

			aFillBitmapSize = aLastFillBitmapSize = aSizePix;
		}
	}

	// Wenn wir gerade ein Mtf aufzeichnen, muessen
	// wir auf alle Faelle neu berechnen
	if ( !pMtf )
		bRecalc = FALSE;
}

// ------------------------------------------------------------------------

void XOutputDevice::ImpDrawTiledBitmap( OutputDevice* pOutDev, const Rectangle& rRect,
										const Point& rStartPoint, const Rectangle& rClipRect )
{
	Point  		aOrg;
	long		nXPos = rStartPoint.X();
	long		nYPos = rStartPoint.Y();
	const long	nWidth = aFillBitmapSize.Width();
	const long 	nHeight = aFillBitmapSize.Height();
	const long	nBottom = rRect.Bottom();
	const long	nRight = rRect.Right();
	const long	nLeft = nXPos;

	if ( bIntermediateBmp )
	{
		while( nYPos < nBottom )
		{
			while( nXPos < nRight )
			{
				aOrg = Point( nXPos, nYPos );

				if ( !Rectangle( aOrg, aFillBitmapSize ).Intersection( rClipRect ).IsEmpty() )
					pOutDev->DrawBitmap( aOrg, aFillBitmapSize, aFillBitmap );

				nXPos += nWidth;
			}

			nXPos  = nLeft;
			nYPos += nHeight;
		}
	}
	else
	{
		const long	nOffX = nWidth - nWidth * nBmpOffX / 100;
		const long	nOffY = nHeight - nHeight * nBmpOffY / 100;
		long		nCountX = ( rRect.Right() - rStartPoint.X() ) / nWidth + 1;
		long		nCountY = ( rRect.Bottom() - rStartPoint.Y() ) / nHeight + 1;
		BOOL		bDraw;

		// Falls Kacheln untereinander verschoben,
		// muessen wir noch eine zusaetzliche Zeile
		// oder Spalte ausgeben
		if ( nBmpOffX )
			nCountX++;
		else if ( nBmpOffY )
			nCountY++;

		long nCountX1 = nCountX - 1;
		long nCountY1 = nCountY - 1;

		// Kachelung ausgeben
		for ( long nY = 0; nY < nCountY; nY++ )
		{
			for ( long nX = 0; nX < nCountX; nX++ )
			{
				bDraw = FALSE;

				if ( nBmpOffX && ( nY & 1 ) )
				{
					bDraw = TRUE;
					aOrg = Point( nXPos - nOffX, nYPos );
				}
				else if ( nBmpOffY && ( nX & 1 ) )
				{
					bDraw = TRUE;
					aOrg = Point( nXPos, nYPos - nOffY );
				}
				else
				{
					if ( nBmpOffX )
					{
						// Nicht-Offset-Tiles ausser in der
						// letzten Spalte immer zeichnen
						if ( nX < nCountX1 )
						{
							bDraw = TRUE;
							aOrg = Point( nXPos, nYPos );
						}
					}
					else if ( nBmpOffY )
					{
						// Nicht-Offset-Tiles ausser in der
						// letzten Zeile immer zeichnen
						if ( nY < nCountY1 )
						{
							bDraw = TRUE;
							aOrg = Point( nXPos, nYPos );
						}
					}
					else
					{
						bDraw = TRUE;
						aOrg = Point( nXPos, nYPos );
					}
				}

				// ausgeben, wenn innerhalb des Objektes und innerhalb
				// des gueltigen Clipping-Bereiches
				if ( bDraw && !Rectangle( aOrg, aFillBitmapSize ).Intersection( rClipRect ).IsEmpty() )
					pOutDev->DrawBitmap( aOrg, aFillBitmapSize, aFillBitmap );

				nXPos += nWidth;
			}

			nXPos = nLeft;
			nYPos += nHeight;
		}
	}
}
