/*************************************************************************
 *
 *  $RCSfile: authfld.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:14:24 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _AUTHFLD_HXX
#define _AUTHFLD_HXX

#ifndef _FLDBAS_HXX
#include <fldbas.hxx>
#endif
#ifndef _TOXE_HXX
#include <toxe.hxx>
#endif

class SwAuthDataArr;
/* -----------------21.09.99 13:32-------------------

 --------------------------------------------------*/
class SwAuthEntry
{
	String 		aAuthFields[AUTH_FIELD_END];
	USHORT 		nRefCount;
public:
	SwAuthEntry() : nRefCount(0){}
	SwAuthEntry( const SwAuthEntry& rCopy );
	BOOL 			operator==(const SwAuthEntry& rComp);

	BOOL					GetFirstAuthorField(USHORT& nPos, String& rToFill)const;
	BOOL					GetNextAuthorField(USHORT& nPos, String& rToFill)const;
	inline const String& 	GetAuthorField(ToxAuthorityField ePos)const;
	inline void				SetAuthorField(ToxAuthorityField ePos,
											const String& rField);

	void			AddRef()				{ ++nRefCount; }
	void			RemoveRef()				{ --nRefCount; }
	USHORT			GetRefCount() 			{ return nRefCount; }
};
/* -----------------20.10.99 16:49-------------------

 --------------------------------------------------*/
struct SwTOXSortKey
{
	ToxAuthorityField	eField;
	BOOL				bSortAscending;
	SwTOXSortKey() :
		eField(AUTH_FIELD_END),
		bSortAscending(TRUE){}
};

/* -----------------14.09.99 16:15-------------------

 --------------------------------------------------*/
class SvUShorts;
class SwAuthorityField;
class SvLongs;
class SortKeyArr;
class SwAuthorityFieldType : public SwFieldType
{
	SwDoc*			m_pDoc;
	SwAuthDataArr*	m_pDataArr;
	SvLongs*		m_pSequArr;
	SortKeyArr*		m_pSortKeyArr;
	char			m_cPrefix;
	char			m_cSuffix;
	BOOL			m_bIsSequence :1;
	BOOL			m_bSortByDocument :1;

	const SwAuthorityFieldType& operator=( const SwAuthorityFieldType& );

public:
	SwAuthorityFieldType(SwDoc* pDoc);
	SwAuthorityFieldType( const SwAuthorityFieldType& );
	~SwAuthorityFieldType();

	virtual SwFieldType* Copy()    const;
	virtual void 		Modify( SfxPoolItem *pOld, SfxPoolItem *pNew );

	inline SwDoc*	GetDoc() const						{ return m_pDoc; }
	inline void		SetDoc(SwDoc* pNewDoc)				{ m_pDoc = pNewDoc; }

	void				RemoveField(long nHandle);
	long				AddField(const String& rFieldContents);
	BOOL				AddField(long nHandle);

	const SwAuthEntry*	GetEntryByHandle(long nHandle) const;

	void 				GetAllEntryIdentifiers( SvStringsDtor& rToFill )const;
	const SwAuthEntry* 	GetEntryByIdentifier(const String& rIdentifier)const;

	void				ChangeEntryContent(const SwAuthEntry* pNewEntry);
	// import interface
	USHORT				AppendField(const SwAuthEntry& rInsert);
	void				MergeFieldType(const SwAuthorityFieldType& rNew, SvUShorts& rMap);
	void				RemoveUnusedFields();
	long				GetHandle(USHORT nPos);
	USHORT 				GetPosition(long nHandle);

	USHORT				GetEntryCount() const;
	const SwAuthEntry*	GetEntryByPosition(USHORT nPos) const;

	USHORT				GetSequencePos(long nHandle);

	BOOL				IsSequence() const 		{return m_bIsSequence;}
	void				SetSequence(BOOL bSet) 	{m_bIsSequence = bSet;}

	void				SetPreSuffix(char cPre, char cSuf)
							{
								m_cPrefix = cPre;
								m_cSuffix = cSuf;
							}
	char				GetPrefix() const { return m_cPrefix;}
	char				GetSuffix() const { return m_cSuffix;}

	BOOL				IsSortByDocument() const {return m_bSortByDocument;}
	void				SetSortByDocument(BOOL bSet) {m_bSortByDocument = bSet;}

	USHORT				GetSortKeyCount() const ;
	const SwTOXSortKey*	GetSortKey(USHORT nIdx) const ;
	void				SetSortKeys(USHORT nKeyCount, SwTOXSortKey nKeys[]);

	//initui.cxx
	static const String& 	GetAuthFieldName(ToxAuthorityField eType);
	static const String& 	GetAuthTypeName(ToxAuthorityType eType);

};
/* -----------------14.09.99 16:15-------------------

 --------------------------------------------------*/
class SwAuthorityField : public SwField
{
	long	nHandle;

public:
	SwAuthorityField(SwAuthorityFieldType* pType, const String& rFieldContents);
	SwAuthorityField(SwAuthorityFieldType* pType, long nHandle);
	~SwAuthorityField();

	const String&		GetFieldText(ToxAuthorityField eField) const;

	virtual String		Expand() const;
	virtual SwField* 	Copy() const;
	virtual void		SetPar1(const String& rStr);
	virtual SwFieldType* ChgTyp( SwFieldType* );

	long				GetHandle() const		{ return nHandle; }

	//import interface
	USHORT				GetHandlePosition() const;
};

// --- inlines -----------------------------------------------------------
inline const String& 	SwAuthEntry::GetAuthorField(ToxAuthorityField ePos)const
{
	if(AUTH_FIELD_END > ePos)
		return aAuthFields[ePos];
	else
		return aEmptyStr;
}
inline void	SwAuthEntry::SetAuthorField(ToxAuthorityField ePos, const String& rField)
{
	if(AUTH_FIELD_END > ePos)
		aAuthFields[ePos] = rField;
}

#endif

