/*************************************************************************
 *
 *  $RCSfile: sw3frmio.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:08:23 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PRECOMPILED
#include "core_pch.hxx"
#endif

#pragma hdrstop

#ifndef _SVSTOR_HXX //autogen
#include <so3/svstor.hxx>
#endif

#ifndef _FMTHDFT_HXX //autogen
#include <fmthdft.hxx>
#endif
#ifndef _NODE_HXX //autogen
#include <node.hxx>
#endif

#include "swerror.h"
#include "doc.hxx"
#include "rootfrm.hxx"
#include "pagefrm.hxx"
#include "flyfrm.hxx"
#include "sw3imp.hxx"
#include "sw3frmio.hxx"
#include "notxtfrm.hxx"
#include "hffrm.hxx"
#include "colfrm.hxx"
#include "ftnfrm.hxx"
#include "tabfrm.hxx"
#include "txtfrm.hxx"
#include "flyfrms.hxx"
#include "bodyfrm.hxx"
#include "rowfrm.hxx"
#include "cellfrm.hxx"
#include "sectfrm.hxx"
#include "pagedesc.hxx"


//////////////////////////////////////////////////////////////////////////
//
//	Frame-Support
//
//////////////////////////////////////////////////////////////////////////

// man uebernehme bitte direkt den Contents-Stream aus dem IO-System


Sw3FrameIo::Sw3FrameIo( Sw3IoImp& r )
		  : rIo( r ), rStrm( *rIo.pContents )
{
	bErr 	  = FALSE;
	pDrawPage = NULL;
	pDoc 	  = rIo.pDoc;
	nGblFlags = 0;
	nVersion  = FRMVER_CURRENT;
	nLine 	  =
	nBox  	  =
	nPageCols =
	nFlyCols  = 0;
	pLines 	  = NULL;
	pBoxes    = NULL;
	pPage	  = NULL;
	pDrawPage = NULL;
	pMaster   =
	pFollow	  = NULL;
	pChainMasterFmt = 0;
	bErr	  =
	bRow1	  = FALSE;
	nStartNodeIdx = 0;
}

// Um die Verkettung von Frames speichern zu koennen, werden die
// Ziel-Frames in einem Array gemerkt und deren Index-Position geschrieben.
// Damit sie auch wieder aus dem Array geloescht werden koennen, muss beim
// hinzufuegen ein Ref-Count angegeben werden. Beim Zugriff auf einen
// Frame zum speichern wird der Ref-Count decrementiert. Bei einem Ref-Count
// von 0 werden die Array-Eintrage freigegeben.
//
// Dieser Mechansimus wird an folgenden Stellen benutzt:
// - Verankerung von Rahmen-Frames.
// - Verankerung von Fussnoten-Frames.
// - Folge-Frames von Fussnoten-Frames.
// - Folge-Frames von Content-, Tabellen und Section-Frames.
// - Verkettung von Rahmen-Frames.
//
// In den ersten drei Faellen ist gewaehrleistet, das der Ziel-Frame vor
// dem Ausgangspunkt der Verkettung geschreiben wird.
// Bei den Folge-Frames erfolgt die Verkettung beim Speichern je nach
// Reihenfolge, in der die Frames geschrieben werden in der einen oder
// der anderen Richtung.
// Rahmen-Frames sind doppelt verkettet. Hier wird jeweils die Richtung
// gespeichert, die moeglich ist.


// Index eines Frames ermitteln.

USHORT Sw3FrameIo::_GetIndex( Sw3Frms &rArr, SwFrm* p, BOOL bDecRef )
{
	USHORT nIdx = rArr.GetPos( p );
	if( nIdx != USHRT_MAX )
	{
		if( bDecRef )
			_DecRef( nIdx );	// ist inline!

		return nIdx;
	}

	ASSERT( !this, "Frame nicht gefunden" );
	SetError();

	return 0;
}

USHORT Sw3FrameIo::_GetIndexNoErr( Sw3Frms &rArr, SwFrm* p )
{
	USHORT nIdx = rArr.GetPos( p );
	if( nIdx != USHRT_MAX )
		return nIdx;
	else
		return IDX_NO_VALUE;
}


// Frame per Index ermitteln

SwFrm* Sw3FrameIo::GetFrm( USHORT n )
{
	ASSERT( n < aNewFrms.Count(), "Falsche Frame-ID" );
	if( n < aNewFrms.Count() )
		return aNewFrms[n];

	SetError();
	return 0;
}

// Frame dfinieren


USHORT Sw3FrameIo::AddFrm( SwFrm* p, USHORT nIncCount )
{
	ASSERT( nIncCount, "AddFrm ohne Ref-Count ist nicht vorgesehen" );
	USHORT nFreePos = aNewFrms.Count();
	for( USHORT i=0; i<aNewFrms.Count(); i++ )
	{
		if( aNewFrms[i] == p )
		{
			aRefCounts[i] += nIncCount;
			return i;
		}
		if( aNewFrms.Count()==nFreePos && aNewFrms[i] == 0 )
		{
			nFreePos = i;
		}
	}

	if( nFreePos < aNewFrms.Count() )
	{
		aNewFrms.Replace( p, nFreePos );
		aFollows.Replace( (SwFrm *)0, nFreePos );
		aRefCounts[nFreePos] = nIncCount;
	}
	else
	{
		aNewFrms.Insert( p, nFreePos );
		aFollows.Insert( (SwFrm *)0, nFreePos );
		aRefCounts.Insert( nIncCount, nFreePos );
	}

	return nFreePos;
}

void Sw3FrameIo::DecRef( USHORT nIdx )
{
	ASSERT( nIdx < aNewFrms.Count(), "Falsche Frame-ID" );

	if( nIdx < aNewFrms.Count() )
		_DecRef( nIdx );
	else
		SetError();
}

// Follow definieren


void Sw3FrameIo::SetFollow( USHORT n, SwFrm* p )
{
	ASSERT( n < aFollows.Count(), "Falsche Frame-ID" );

	if( n < aFollows.Count() )
		aFollows.Replace( p, n );
	else
		SetError();
}

// Frame unter einem Index eiontragen


void Sw3FrameIo::RegisterFrm( SwFrm* p, USHORT n )
{
	if( n != IDX_NO_VALUE )
	{
		ASSERT( n <= 0x3F00, "Falsche Frame-ID" );
		if( n > 0x3F00 )
			SetError();
		else
		{
			if( n > aNewFrms.Count() )
			{
				ASSERT( !this, "unerwartete Luecke im Frm-Array" );
				SetError();
				return;
			}
			if( n == aNewFrms.Count() )
			{
				aNewFrms.Insert( p, n );
				aFollows.Insert( (SwFrm*)0, n );
				aRefCounts.Insert( (USHORT)0, n );
			}
			else
			{
				aNewFrms.Replace( p, n );
				aFollows.Replace( (SwFrm *)0, n );
				aRefCounts[n] = 0;
			}
		}
	}
}

// Format einrichten


SwFrmFmt* Sw3FrameIo::MakeFmt( const String& rName )
{
	return rIo.pDoc->MakeFrmFmt( rName, rIo.pDoc->GetDfltFrmFmt() );
}

// Format per Index ermitteln


SwFrmFmt* Sw3FrameIo::FindFmt( BYTE cType, UINT16 nIdx )
{
	SwFrmFmt* p = NULL;
	if( cType == FRMT_COLCONT )
		p = pDoc->GetColumnContFmt();
	else if( cType == FRMT_EMPTY )
		p = pDoc->GetEmptyPageFmt();
	else if( nIdx == IDX_NO_VALUE )
		return p;
	else if( nIdx == IDX_DFLT_VALUE )
		p = pDoc->GetDfltFrmFmt();
	else if( cType != FRMT_STANDARD && FRMT_SECTION != cType )
	{
		const SwFmtHeader* pHdr;
		const SwFmtFooter* pFtr;
		SwPageDesc* pDesc = rIo.FindPageDesc( nIdx );
		if( pDesc )
		  switch( cType )
		{
			case FRMT_PAGE:
				p = &pDesc->GetMaster(); break;
			case FRMT_LEFTPAGE:
				p = &pDesc->GetLeft(); break;
			case FRMT_HEADER:
				p = &pDesc->GetMaster();
				pHdr = &p->GetHeader();
				p = (SwFrmFmt*) pHdr->GetHeaderFmt(); break;
			case FRMT_LEFTHEADER:
				p = &pDesc->GetLeft();
				pHdr = &p->GetHeader();
				p = (SwFrmFmt*) pHdr->GetHeaderFmt(); break;
			case FRMT_FOOTER:
				p = &pDesc->GetMaster();
				pFtr = &p->GetFooter();
				p = (SwFrmFmt*) pFtr->GetFooterFmt(); break;
			case FRMT_LEFTFOOTER:
				p = &pDesc->GetLeft();
				pFtr = &p->GetFooter();
				p = (SwFrmFmt*) pFtr->GetFooterFmt(); break;
		}
		ASSERT( p, "Spezial-FrmFmt nicht gefunden" );
	}
	else
		p = (SwFrmFmt*) rIo.FindFmt( nIdx, FRMT_SECTION == cType ?
									 SWG_SECTFMT : SWG_FRAMEFMT );
	if( !p )
		SetError();
	return p;
}

// Index fuer Format ermitteln


void Sw3FrameIo::FindFmtId( const SwFrmFmt* p, BYTE& cType, UINT16& nIdx )
{
	if( p == pDoc->GetDfltFrmFmt() )
		nIdx = IDX_DFLT_VALUE;
	else if( p == pDoc->GetColumnContFmt() )
		nIdx = IDX_DFLT_VALUE, cType = FRMT_COLCONT;
	else if( p == pDoc->GetEmptyPageFmt() )
		nIdx = IDX_DFLT_VALUE, cType = FRMT_EMPTY;
	else
	{
		if( cType != FRMT_STANDARD && FRMT_SECTION != cType )
		{
			USHORT nArrLen = pDoc->GetPageDescCnt();
			for( USHORT n = 0; n < nArrLen; n++ )
			{
				const SwPageDesc& rPg = pDoc->GetPageDesc( n );
				nIdx = rIo.aStringPool.Find( rPg.GetName(), rPg.GetPoolFmtId() );
				// Master- oder Leftformat?
				if( &rPg.GetMaster() == p )
					return;
				else if( &rPg.GetLeft() == p )
				{
					cType = FRMT_LEFTPAGE; return;
				}
				// Header- oder Footerformat?
				else if( cType == FRMT_HEADER )
				{
					const SwFmtHeader* pAttr = &rPg.GetMaster().GetHeader();
					if( pAttr->GetHeaderFmt() == p )
						return;
					pAttr = &rPg.GetLeft().GetHeader();
					if( pAttr->GetHeaderFmt() == p )
					{
						cType = FRMT_LEFTHEADER; return;
					}
				}
				// Header- oder Footerformat?
				else if( cType == FRMT_FOOTER )
				{
					const SwFmtFooter* pAttr = &rPg.GetMaster().GetFooter();
					if( pAttr->GetFooterFmt() == p )
						return;
					pAttr = &rPg.GetLeft().GetFooter();
					if( pAttr->GetFooterFmt() == p )
					{
						cType = FRMT_LEFTFOOTER; return;
					}
				}
			}
		}
		else
		{
			nIdx = rIo.aStringPool.Find( p->GetName(), p->GetPoolFmtId() );
			if( nIdx != IDX_NO_VALUE )
				return;
		}
		ASSERT( !this, "FrameFmt nicht gefunden" );
	}
}

// PageDesc per Index ermitteln


SwPageDesc* Sw3FrameIo::FindPageDesc( USHORT nIdx )
{
	SwPageDesc* p = rIo.FindPageDesc( nIdx );
	if( !p )
		SetError();
	return p;
}

// Index fuer PageDesc ermitteln


USHORT Sw3FrameIo::FindPageDescId( const SwPageDesc* p )
{
	USHORT nIdx = rIo.aStringPool.Find( p->GetName(), p->GetPoolFmtId() );
	if( nIdx == IDX_NO_VALUE )
		SetError();
	return nIdx;
}


void Sw3FrameIo::CheckRefCnts()
{
	for( USHORT i=0; i<aNewFrms.Count(); i++ )
	{
		if( aNewFrms[i] )
		{
			ASSERT( !this, "Ref-Count an Frame stimmt nicht" );
			SetError();
		}
	}
}


// ID fuer Node ermitteln. Dieses ist der Offset zum vorhergehenden
// Start-Node, wobei Section-Nodes nicht beachtet werden und Tabellen-Nodes
// nicht vorkommen koennen, weil jeder Content-Node innerhalb einer
// Tabelle ja auch noch innerhalb einer Section steht.
// Der aktuelle Start-Node muss beim Laden der Frames jeweils gesetzt werden
// Es ist
// - der Start-Node des Content bereichs bei RootFrms
// - der Start-Node des Fusszeilen-Attributs von FtnFrms
// - der Start-Node des SwFmtCnt-Attributs des Formats von FlyFrms.
// Bei FlyFrms ist zu beachten, dass die Content-Nodes bei Verkettung
// von Rahmen in der Sections des ersten Rahmens liegen. Deshalb muss
// bei allen folgenden Rahmen der String-Pool-Index des Formats gespeichert
// werden.

ULONG Sw3FrameIo::GetNodeId( SwCntntNode* pNd )
{
	ASSERT( pNd->IsCntntNode(), "Kein CntntNode" );

	// Start-Node suchen. Sections sind (noch) nicht im Layout verankert,
	// desshalb werden sie nicht beachtet. Da es immer noch einen Start-Node
	// davor gibt, geht das.
	SwStartNode* pStt = pNd->FindStartNode();
	while( pStt->IsSectionNode() )
		pStt = pStt->FindStartNode();

	// Konsistenz-Check: Passt der gefundene Start-Node zu dem, der
	// beim Einlesen gesetzt wird?
	ASSERT( pStt->GetIndex() == GetStartNodeIndex(),
			"StartNode-Index ist falsch gesetzt" );
	if( pStt->GetIndex() != GetStartNodeIndex() )
		SetError();

	return pNd->GetIndex() - pStt->GetIndex();
}

// Node anhand des Section-Index und Offsets ermitteln


SwCntntNode* Sw3FrameIo::FindNode( ULONG nOff )
{
	SwCntntNode* pNd = 0;
	ULONG nNodeIdx = GetStartNodeIndex() + nOff;
	if( nNodeIdx < pDoc->GetNodes().Count() )
		pNd = pDoc->GetNodes()[ nNodeIdx ]->GetCntntNode();
	ASSERT( pNd, "Kein Cntnt-Node gefunden" );
	if( !pNd )
		SetError();
	return pNd;
}

// Einen Frame einlesen:

SwFrm* Sw3FrameIo::InFrame( SwLayoutFrm* pUpr )
{
	SwFrm* pRes;
	BYTE nId = rIo.Peek();
	// Kleine Plausi:
	if( nId < ( 'A' + 0x80 ) || nId > ( 'z' + 0x80 ) )
	{
		SetError();
		return NULL;
	}
	rIo.OpenRec( nId );
	switch( nId )
	{
		case SWG_FRM_ROOT:		pRes = new SwRootFrm( *this, pUpr ); break;
		case SWG_FRM_PAGE:		pRes = new SwPageFrm( *this, pUpr ); break;
		case SWG_FRM_HEADER:	pRes = new SwHeaderFrm( *this, pUpr ); break;
		case SWG_FRM_FOOTER:	pRes = new SwFooterFrm( *this, pUpr ); break;
		case SWG_FRM_BODY:		pRes = new SwBodyFrm( *this, pUpr ); break;
		case SWG_FRM_COLUMN:	pRes = new SwColumnFrm( *this, pUpr ); break;
		case SWG_FRM_FTN:		pRes = new SwFtnFrm( *this, pUpr ); break;
		case SWG_FRM_TXT:		pRes = new SwTxtFrm( *this, pUpr ); break;
		case SWG_FRM_NOTXT:		pRes = new SwNoTxtFrm( *this, pUpr ); break;
		case SWG_FRM_FLYATCNT:	pRes = new SwFlyAtCntFrm( *this, pUpr ); break;
		case SWG_FRM_FLYINCNT:	pRes = new SwFlyInCntFrm( *this, pUpr ); break;
		case SWG_FRM_FLYLAY:	pRes = new SwFlyLayFrm( *this, pUpr ); break;
		case SWG_FRM_SECTION:	pRes = new SwSectionFrm( *this, pUpr ); break;
		case SWG_FRM_TAB:		pRes = new SwTabFrm( *this, pUpr ); break;
		case SWG_FRM_ROW: 		pRes = new SwRowFrm( *this, pUpr ); break;
		case SWG_FRM_CELL:		pRes = new SwCellFrm( *this, pUpr ); break;
		case SWG_FRM_FTNCONT:
			pRes = new SwFtnContFrm( *this, pUpr );
			ASSERT( pRes->GetLower(), "FtnContFrm hat keine Lowers" );
			if( !pRes->GetLower() )
			{
				// Darf nicht vorkommen!
				delete pRes; pRes = NULL;
			} break;
		default:
			ASSERT( !this, "Unbekannte FrmId" );
			SetError();
			pRes = NULL;
	}
	rIo.CloseRec( nId );
	return pRes;
}


void Sw3FrameIo::OutFrame( const SwFrm* p )
{
	p->Store( *this );
}


void Sw3FrameIo::OpenRec( BYTE cType )
{
	rIo.OpenRec( cType );
}


void Sw3FrameIo::CloseRec( BYTE cType )
{
	rIo.CloseRec( cType );
}


void Sw3FrameIo::SetPercentBar( USHORT nPage, BOOL bWrite )
{
	if( bWrite )
		// Es werden 10 Elemente (Nodes) pro Seite angenommen
		// Auerdem wird angenommen, dass die Funktion nur
		// einmal pro Seite aufgerufen wird
		// siehe auch sw3imp.cxx, wo OpenPercentBar() aufgerufen wird
		rIo.SetPercentBar( rIo.nCurPercent + 10 );
	else
		rIo.SetPercentBar( rStrm.Tell() );
}


void Sw3FrameIo::SetInTableLayout( BOOL bSet )
{
	rIo.bInTableLayout = bSet;
}


BOOL Sw3FrameIo::GetInTableLayout() const
{
	return rIo.bInTableLayout;
}


#ifndef PRODUCT

		// damit beim debuggen ein BreakPoint gesetzt werden kann
void Sw3FrameIo::SetError()
{
	bErr = TRUE;
}

#endif

