/*************************************************************************
 *
 *  $RCSfile: hintlist.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:14:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _HINTLIST_HXX
#define _HINTLIST_HXX

#ifdef XML_CORE_API

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _SVARRAY_HXX
#include <svtools/svarray.hxx>
#endif

class SfxPoolItem;
class SwHintSttEndPosList_Impl;

class SwHintSttEndPos
{
	xub_StrLen nStart;
	xub_StrLen nEnd;
	SfxPoolItem* pItem;

public:

	SwHintSttEndPos( const SfxPoolItem& rItem, xub_StrLen nStt, xub_StrLen nE );
	~SwHintSttEndPos();

	const SfxPoolItem& GetItem() const { return *pItem; }

	xub_StrLen GetStart() const { return nStart; }
	void SetStart( xub_StrLen nStt ) { nStart = nStt; }

	xub_StrLen GetEnd() const { return nEnd; }
	void SetEnd( xub_StrLen nE ) { nEnd = nE; }
};

typedef SwHintSttEndPos*SwHintSttEndPosPtr;
SV_DECL_PTRARR( SwHintSttEndPosList_Impl, SwHintSttEndPosPtr, 5, 5 )

class SwHintEndPosList
{
	SwHintSttEndPosList_Impl *pStartLst;	// List sorted by start positions
	SwHintSttEndPosList_Impl *pEndLst;	// List sorted by end psoitions

protected:

	SwHintEndPosList();
	virtual ~SwHintEndPosList();

	// Find a hints position within the start list
	sal_uInt16 _FindStartPos( const SwHintSttEndPos *pPos ) const;

	// Insert or remove a hint into/from both lists.
	// The parameter is the position in the end list.
	void _InsertHint( SwHintSttEndPos *pPos, sal_uInt16 nEndPos );
	void _RemoveHint( sal_uInt16 nEndPos );

	// Insert a hint into the list. If required, split it.
	void InsertHint( const SfxPoolItem& rItem, xub_StrLen nStart,
					 xub_StrLen nEnd );

	// Get starting/ending hint at a specific position with the list.
	inline SwHintSttEndPos *GetStartingHint( sal_uInt16 nPos ) const;
	inline SwHintSttEndPos *GetEndingHint( sal_uInt16 nPos ) const;

public:

	inline sal_uInt16 GetStartingCount() const;
	inline sal_uInt16 GetEndingCount() const;
};

inline SwHintSttEndPos *SwHintEndPosList::GetStartingHint( sal_uInt16 nPos ) const
{
	return (*pStartLst)[nPos];
}

inline SwHintSttEndPos *SwHintEndPosList::GetEndingHint( sal_uInt16 nPos ) const
{
	return (*pEndLst)[nPos];
}

inline sal_uInt16 SwHintEndPosList::GetStartingCount() const
{ 
	return pStartLst->Count();
}

inline sal_uInt16 SwHintEndPosList::GetEndingCount() const
{ 
	return pEndLst->Count();
}

#endif


#endif // _HINTLIST_HXX
