/*************************************************************************
 *
 *  $RCSfile: finder.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:14:50 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PRECOMPILED
#include "ui_pch.hxx"
#endif

#pragma hdrstop

#ifndef _HINTIDS_HXX
#include <hintids.hxx>
#endif

#ifndef _TOOLS_RESID_HXX //autogen
#include <tools/resid.hxx>
#endif
#ifndef _SFXINIMGR_HXX //autogen
#include <svtools/iniman.hxx>
#endif
#ifndef _SFXAPP_HXX //autogen
#include <sfx2/app.hxx>
#endif
#ifndef _SVX_ADRITEM_HXX //autogen
#include <svx/adritem.hxx>
#endif


#ifndef _SWTYPES_HXX
#include <swtypes.hxx>    // aEmptyStr
#endif
#ifndef _ERRHDL_HXX
#include <errhdl.hxx>
#endif
#ifndef _FINDER_HXX
#include <finder.hxx>
#endif

#ifndef _GLOBALS_HRC
#include <globals.hrc>
#endif
#ifndef _UTLUI_HRC
#include <utlui.hrc>
#endif

#define C2S(cChar) 	UniString::CreateFromAscii(RTL_CONSTASCII_STRINGPARAM(cChar))

/*--------------------------------------------------------------------
	Beschreibung:   Ctor Wrapper um SfxIniManager
 --------------------------------------------------------------------*/

SwPathFinder::SwPathFinder()
{
	pIniManager = SFX_APP()->GetIniManager();
	ASSERT(pIniManager, "Kein Inimanager angelegt !!");
}

/*--------------------------------------------------------------------
	Beschreibung:   Methoden-Mapping auf IniManager
					ACHTUNG nicht inline damit SFX.HXX
					nicht auf der Core-Seite landet
 --------------------------------------------------------------------*/

String SwPathFinder::GetUserName() const
{
	return pIniManager->GetUserFullName();
}

String SwPathFinder::GetShortUserName() const
{
	return pIniManager->Get(SFX_KEY_USER_ID);
}

void SwPathFinder::SetAddress(SvxAddressItem& aAddress)
{
	aAddress.Store( *pIniManager );
}


SvxAddressItem SwPathFinder::GetAddress()
{
	SvxAddressItem aAddress( *pIniManager );
	aAddress.SetWhich(SID_ATTR_ADDRESS);
	return aAddress;
}

/*--------------------------------------------------------------------
	Beschreibung:
 --------------------------------------------------------------------*/

String SwPathFinder::GetIniUserEntry( const String& rKey, USHORT nIdx ) const
{
	return  pIniManager->Get( SFX_GROUP_USER, rKey, nIdx);
}

/*-----------------24.02.97 08.42-------------------

--------------------------------------------------*/

String	SwPathFinder::GetDBInsertMode(const String& rDBName) const
{
	String sGroup(C2S("DataBaseFormatInfo"));
	String sDBName(rDBName);
	sDBName.SearchAndReplace( DB_DELIM, '.');
	String sReturn = pIniManager->Get( sGroup, FALSE, FALSE, sDBName );
	if(!sReturn.Len())
	{
		String sSecName(sDBName);
		sReturn = pIniManager->Get( sGroup, FALSE, FALSE, sSecName );
	}
	if(!sReturn.Len())
	{
		String sSecName(sDBName);
		sReturn = pIniManager->Get( sGroup, FALSE, FALSE, sSecName );
	}
	return  sReturn;
}

/*------------------------------------------------------------------------

	$Log: finder.cxx,v $
	Revision 1.1.1.1  2000/09/18 17:14:50  hr
	initial import
	
	Revision 1.118  2000/09/18 16:06:17  willem.vandorp
	OpenOffice header added.
	
	Revision 1.117  2000/09/07 08:30:54  os
	FaxName now in SwPrintOptions
	
	Revision 1.116  2000/07/20 13:17:35  jp
	change old txtatr-character to the two new characters

	Revision 1.115  2000/04/18 15:14:08  os
	UNICODE

	Revision 1.114  1999/10/21 17:45:03  jp
	have to change - SearchFile with SfxIniManager, dont use SwFinder for this

	Revision 1.113  1999/09/23 14:03:42  os
	#68763# read table autoformats from user/config

	Revision 1.112  1999/08/26 13:45:32  OS
	double methods removed


	  Rev 1.111   26 Aug 1999 15:45:32   OS
   double methods removed

	  Rev 1.110   07 Jul 1999 08:16:32   OS
   extended indexes: Index names in ShellRes

	  Rev 1.109   02 Jul 1998 16:07:12   JP
   ueber den PathFinder multi pathes behandeln

	  Rev 1.108   02 Jul 1998 12:35:48   MA
   unn?tzes Zeug entfernt

	  Rev 1.107   13 Feb 1998 14:18:36   JP
   GetIniFileGPM wird nicht mehr benoetigt

	  Rev 1.106   12 Jan 1998 16:18:24   TJ
   include

	  Rev 1.105   21 Nov 1997 12:10:14   MA
   includes

------------------------------------------------------------------------*/


