/*************************************************************************
 *
 *  $RCSfile: allmake.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:02:58 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "allmake.hxx"


// NOT FULLY DECLARED SERVICES
#include <iostream.h>
#include "fsys.hxx"
#include "updmake.hxx"
#include "stream.hxx"
#include "iparser.hxx"
#include "geninfo.hxx"
#include "prj.hxx"
#include "appdef.hxx"
#include "inimgr.hxx"
#include "sstring.hxx"
#include "make_err.hxx"

// STL einbinden
#if STLPORT_VERSION < 321
#include <tools/presys.h>
#include <map.h>
#include <tools/postsys.h>
#else
#include <map>
#include <vector>
#endif	// STLPORT_VERSION < 321

#include <vos/macros.hxx>



struct ltstr
{
  BOOL operator()(const ByteString & s1, const ByteString & s2) const
  {
	return s1.CompareIgnoreCaseToAscii(s2) == COMPARE_LESS;
  }
};

class DependencyTree;
typedef NAMESPACE_STD(map)<ByteString,DependencyTree*,ltstr>  ProjectNamesMap;
typedef NAMESPACE_STD(vector)<DependencyTree*>            DependencyTreeList;

/**	@ATTENTION
	This class is NOT safe for any other use than this one.
**/
class  DependencyTree
{
  public:
	// LIFECYCLE
						DependencyTree(
							const Star &		i_rStar,
							const ByteString &		i_sProjectName,
							ProjectNamesMap &	io_rMainMap );
						~DependencyTree();

	// OPERATIONS
	void				AddDependent(
						   DependencyTree &	io_rDependent);
	void				RemoveNeeded(
						   DependencyTree &	io_rDependent);

	BOOL				CreateBuildSequence(
							AllMake::ProjectNameList &
											o_rBuildSequence );
	// INQUIRY
	const ByteString &		ProjectName() const { return sMe; }
	BOOL				IsValid() const { return bIsValid; }

  private:
	ByteString				sMe;           		// always lower case.
	DependencyTreeList	aNeededProjects;
	DependencyTreeList	aDependentProjects;
	BOOL				bIsValid;			// FALSE, if this or any of the dependant projects are not known.
	BOOL				bCreatingBuildSequence_ON;	// Used to recognize circlar dependencies during CreateBuildSequence().
};

DependencyTreeList::iterator
					FindProjectInList(
						const ByteString &			i_rProjectName,
						DependencyTreeList &	i_rList );



AllMake::AllMake( const ByteString & i_sDestinationProject,
				  const ByteString & i_sWorkStamp,
				  const ByteString & i_sMakeCommand )
	:	sDestinationProject(i_sDestinationProject),
		sWorkStamp(i_sWorkStamp),
		sMakeCommand(i_sMakeCommand),
		// aBuildSequence,
		nErrorCode(MK_NO_ERROR),
		bGetProjectFromCvs(FALSE)
{
	CreateBuildSequence();
}

AllMake::~AllMake()
{
}

BOOL
AllMake::WriteScript( const ByteString & i_sOutputBatchfile,
					  const ByteString & i_sSwitches,
					  BOOL			 i_bListDependenciesOnly )
{
	if (aBuildSequence.empty())
	{
		nErrorCode = MK_ERROR_NO_PROJECT;
		return FALSE;	// Projektliste konnte nicht erstellt werden.
	}

	// Start work
	SvFileStream aScript( String( i_sOutputBatchfile, gsl_getSystemTextEncoding()), STREAM_STD_WRITE | STREAM_TRUNC );
	if (aScript.GetError())
	{
		nErrorCode = MK_ERROR_OUTPUTFILE;
		return FALSE;
	}

	if (! i_bListDependenciesOnly)
	{
#if !defined( UNX ) && !defined( MAC )
		aScript.WriteLine( "@echo off" );
		aScript.WriteLine( "pushd" );
#else
		aScript.WriteLine( ByteString( "setenv mk_tmp TRUE" ));
#endif

		for ( NAMESPACE_STD(vector)<ByteString>::iterator aIter = aBuildSequence.begin();
			  aIter != aBuildSequence.end();
			  ++aIter )
		{
			UPDMake aUpdMake(*aIter,sWorkStamp,sMakeCommand,"",&sLocalSolarlistPath);
			if (bGetProjectFromCvs)
			{
				aUpdMake.Set_GetProjectFromCvs(TRUE);
			}

			USHORT nUpdError = aUpdMake.GetErrorCode();
			if ( nUpdError != MK_NO_ERROR )
			{
				nErrorCode = nUpdError;
				return FALSE;
			}
			if (aIter != aBuildSequence.begin())
			{   // Deliver all builded projects, but the last one.
				aScript.WriteLine("deliver");
			}
			aUpdMake.WriteScriptInto(aScript,i_sSwitches);
		}	// end for
#if !defined( UNX ) && !defined( MAC )
		aScript.WriteLine( "popd" );
#else
		aScript.WriteLine( ByteString( "unsetenv mk_tmp" ));
#endif

	}
	else
	{
		aScript.WriteLine("List of projects that would be built in this sequence:");
		aScript.WriteLine("");
		for ( NAMESPACE_STD(vector)<ByteString>::iterator aIter = aBuildSequence.begin();
			  aIter != aBuildSequence.end();
			  ++aIter )
		{
			aScript.WriteLine((*aIter).GetBuffer());
		}
	}   // endif (i_bListDependenciesOnly) else

	aScript.Close();
	return TRUE;
}

BOOL
AllMake::CreateBuildSequence()
{
	nErrorCode = UPDMake::GetLocalSolarListPath(sLocalSolarlistPath,sWorkStamp);
	if (nErrorCode != MK_NO_ERROR)
	{
		return FALSE;
	}

	Star aStar( String( sLocalSolarlistPath, gsl_getSystemTextEncoding()));

	// Creates the dependency tree. The objects are gathered in rMainMap:
	ProjectNamesMap rMainMap;
	DependencyTree * pProjects
			= new DependencyTree(aStar,sDestinationProject,rMainMap);
	if (! pProjects->IsValid())
	{
		nErrorCode = MK_ERROR_NO_PROJECT;
		return FALSE;
	}

	BOOL bResult = pProjects->CreateBuildSequence(aBuildSequence);
	if (! bResult)
	{
		nErrorCode = MK_ERROR_PRJDEPEND;
        return FALSE;
	}

	// Clean up:
	for ( ProjectNamesMap::iterator aIter = rMainMap.begin();
		  aIter != rMainMap.end();
		  ++aIter )
	{
		delete (*aIter).second;
	}  // end for

	return TRUE;
}

//***************  DependencyTree ************//

DependencyTree::DependencyTree( const Star &		i_rStar,
								const ByteString &		i_sProjectName,
								ProjectNamesMap &	io_rMainMap )
	:	sMe(i_sProjectName ),
		// aNeededProjects,
		// aDependentProjects,
		bIsValid(FALSE),
		bCreatingBuildSequence_ON(FALSE)
{
//	sMe.ToLowerAscii();

	Prj * pProject = (( Star& )i_rStar).GetPrj( i_sProjectName );
						// Must be casted, because Star is not const-correct declared.
	if (pProject == 0)
		return;	// Project is not known. this remains invalid.

	bIsValid = TRUE;	// So this is a valid project.
	ProjectNamesMap::value_type aThis(sMe,this);
	io_rMainMap.insert(aThis);

	SByteStringList * pNeededProjects = pProject->GetDependencies(FALSE);
	if (pNeededProjects == 0)
		return;	// Project does not have any dependencies.

	// Create instances of DependencyTree for the needed projects. Insert
	//   them into the dependency tree.
	int i_max = pNeededProjects->Count();
	for (int i = 0; i < i_max; i++)
	{
		// Find needed project in io_rMainMap. If it is not contained
		//   already, insert it:
		ByteString sNeededProject = *(*pNeededProjects).GetObject(i);

		// Workaround for bug in solarXXX.lst :
		if (sNeededProject == "null_project")
			continue;	// This is not really a project!

		ProjectNamesMap::iterator pNeeded = io_rMainMap.find(sNeededProject);
		if (pNeeded == io_rMainMap.end())
		{
			ProjectNamesMap::value_type aNewNeeded(
					  sNeededProject,
					   new DependencyTree(i_rStar,sNeededProject,io_rMainMap)
												  );
			if (! aNewNeeded.second->bIsValid)
			{
				bIsValid = FALSE;
				return;
			}
			pNeeded = io_rMainMap.insert(aNewNeeded).first;
		}

		// Establish a dependency-connection between this project and the
		//   needed one:
		aNeededProjects.push_back((*pNeeded).second);
		(*pNeeded).second->AddDependent(*this);
	}  // end for
}

DependencyTree::~DependencyTree()
{
}

void
DependencyTree::AddDependent(DependencyTree & io_rDependent)
{
	if (FindProjectInList(io_rDependent.ProjectName(),aDependentProjects) == aDependentProjects.end())
	{
		aDependentProjects.push_back(&io_rDependent);
	}
}

void
DependencyTree::RemoveNeeded(DependencyTree & io_rDependent)
{
	DependencyTreeList::iterator pRemove
			= FindProjectInList(io_rDependent.ProjectName(),aNeededProjects);
	aNeededProjects.erase(pRemove);
}

BOOL
DependencyTree::CreateBuildSequence(AllMake::ProjectNameList & o_rBuildSequence)
{
	if (bCreatingBuildSequence_ON != FALSE)
	{
    	return FALSE;
	}
	bCreatingBuildSequence_ON = TRUE;

	for ( ; ! aNeededProjects.empty(); )
	{
		BOOL bResult = (*aNeededProjects.begin())->CreateBuildSequence(o_rBuildSequence);
		if (!bResult)
        	return FALSE;
	}

	o_rBuildSequence.push_back(sMe);

	for ( DependencyTreeList::iterator aIter = aDependentProjects.begin();
		  aIter != aDependentProjects.end();
		  ++aIter )
	{
		(*aIter)->RemoveNeeded(*this);
	}
	bCreatingBuildSequence_ON = FALSE;
    return TRUE;
}

DependencyTreeList::iterator
FindProjectInList( const ByteString &		i_rProjectName,
				   DependencyTreeList & i_rList )
{
	ByteString rProjectName = i_rProjectName;
//	rProjectName.ToLowerAscii();
	for ( DependencyTreeList::iterator aIter = i_rList.begin();
		  aIter != i_rList.end();
		  ++aIter )
	{
		if ((*aIter)->ProjectName() == rProjectName)
			return aIter;
	}
    return i_rList.end();
}




