/*************************************************************************
 *
 *  $RCSfile: deliver.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:02:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _DELIVER_HXX
#define _DELIVER_HXX
#include "fsys.hxx"
#ifdef DEBUG
#include <stdio.h>
#endif

#if defined ( OS2 ) || defined ( GCC )
#define aBootstrapEmptyByteString ByteString( "" )
#else
extern ByteString aBootstrapEmptyByteString;
#endif

#ifndef _SSTRING_HXX
#include <sstring.hxx>
#endif

enum DeliverAction { Deliver_Copy, Deliver_DosCommand, Deliver_Touch,
						Deliver_Pro, Deliver_NonPro, Deliver_Hedabu,
						Deliver_Mkdir, Deliver_Ln };

struct FileCopyData
{
	ByteString 		*pSource;
	ByteString 		*pDestination;
	ByteString 		*pCommand;
	DeliverAction	eAction;
};

DECLARE_LIST( FileCopyList, FileCopyData* );

static ByteString theDList( "d.lst" );

//*********************************************************************
// class UPDDeliver:
// Klasse zum delivern von gebuildeten Projekten.
// Ausgewertet werden Dateien im Format der D.LST
//*********************************************************************
enum DeliverResult { NO_ERROR, INVALID_PATH ,NO_FILE, UNKNOWN_ERROR };

// INVALID_PATH		=>	Pfad ungueltig
// NO_FILE			=>	Deliver-File im angegebenen Pfad nicht vorhanden
// UNKNOWN_ERROR	=> 	Unbekannter Fehler
// NO_ERROR			=> 	Kein Fehler aufgetreten

/*
 * ein wenig Statistik als Userentertainment
 *
 */

struct DeliverStat
{
	USHORT			nCopy;
	USHORT			nNotCopy;
	USHORT			nReplace;
	USHORT			nError;
};

DECLARE_LIST( HedabuReplaceList, ByteString * );

class Star;
class UPDDeliver
{
protected:
	ByteString 			sInpath;
	ByteString 			sCommon;
	ByteString 			sPro;
	ByteString			sMinor;
	ByteString			sUpd;
	ByteString			sSolarver;
	ByteString			sSolarversion;
	ByteString			sOffenv;
	ByteString			sWorkStamp;
	ByteString			sGui;
	ByteString			sOutpath;

	DeliverStat		*pDeliverStat;
	HedabuReplaceList *pRplLst;

	BOOL			bHack;
	BOOL			bRel;

	static BOOL		bControlC;

private:
	ByteString		sDeliverPath;
	ByteString		sDeliverFile;
	ByteString		sDestPath;

	Link			aRequestHdl;
	FileCopyList	*pCpyLst;
	DeliverResult	eStatus;

	BOOL			bForce;
	BOOL			bUseMinor;
	BOOL			bNew;		// alles aus prj\inc delivern und hedabuen

	BOOL			Impl_Read( ByteString sFile, Star *pStar );
	ByteString	 	Impl_TransformNative( ByteString &rString );
	ByteString*	 	Impl_Transform( ByteString &rString );
	virtual void	Impl_Init();

	void			InitList( Star *pStar );
	BOOL			ReadData_Impl( ByteString &rBuffer, DeliverAction eReason );

public:
					UPDDeliver( ByteString sInitPath, ByteString sInitFile =
													theDList );
	virtual			~UPDDeliver();

	ByteString			GetPath()				{ return sDeliverPath; }
	ByteString 			GetFile()  				{ return sDeliverFile; }
	void			SetPath( ByteString sPath )	{ sDeliverPath = sPath; }
	void			SetFile( ByteString sFile )	{ sDeliverFile = sFile; }

	DeliverResult	Deliver(BOOL bAll=FALSE,
								ByteString aProjectName = aBootstrapEmptyByteString);
	DeliverResult	Deliver( ByteString sPath, ByteString sFile= theDList )
					{
						sDeliverPath = sPath;
						sDeliverFile = sFile;
						return Deliver();
					}

	void			SetProtectedPublicHack( BOOL bnurfuerwnt = TRUE )
					{ bHack = bnurfuerwnt; }

	void 			SetDestPath( ByteString aDestPath ){sDestPath = aDestPath; }
	DeliverResult	GetStatus(){DeliverResult eTmp = eStatus; eStatus=NO_ERROR;
										return eTmp; }
	ByteString		MKSubDir( const ByteString &rDir, const ByteString &rSub, BOOL bMinor = TRUE );
	void			MkDestDirs();

	virtual void	PutError(FSysError nError);

	virtual char	GetInput( ByteString aSrcFileName, ByteString aDestFileName );
	virtual void	GetDiff( ByteString aSrcFileName, ByteString aDestFileName );
	virtual void	Alarm( ByteString aSrcFileName, ByteString aDestFileName );
	void			SetRequestHdl( const Link& rLink ){aRequestHdl = rLink;}

	void			SetAll( BOOL bFlag ){ bNew = bFlag; }
	void			SetRel( BOOL bFlag ){ bRel = bFlag; }

	void			SetUseMinor( BOOL bFlag ){ bUseMinor = bFlag; }

	DECL_LINK( DeliverErrorHandler, void * );
#ifdef DEBUG
	virtual void	Print( ByteString sMessage ){ printf( sMessage.GetBuffer()); }
#else
	//virtual void	Print( String sMessage ){ printf( sMessage ); }
	virtual void	Print( ByteString sMessage )= 0;
#endif

	static void
#ifdef WNT
		__cdecl
#endif
#ifdef OS2
		_Import
#endif
					Signal( int nSignal );
};

#endif

