/*************************************************************************
 *
 *  $RCSfile: errhdl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:02:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#if SUPD==358
#include <tlgen.hxx>
#endif

#include "errhdl.hxx"
#include "bsecode.hxx"
#include "uierr.hxx"
//#include "app.hxx"

static USHORT aSvWndFunc( Window *pWin, USHORT nFlags, const ByteString &rErr,
					const ByteString &rAction )
{
	return BS_ErrorFunc( pWin, nFlags, rErr, rAction );
}

static USHORT aWndFunc( Window *pWin, USHORT nFlags, const UniString &rErr,
					const UniString &rAction )
{
	USHORT nRet = TRUE;

	ByteString sAction( rAction, gsl_getSystemTextEncoding());
	ByteString sErr( rErr, gsl_getSystemTextEncoding());

	fprintf( stderr, "Action : %s\t", sAction.GetBuffer());
	fprintf( stderr, "Error : %s\n", sErr.GetBuffer());

	return nRet;
}


BOOL BS_ErrorHandler::CreateByteString( const ErrorInfo* pErrInf, ByteString& rStr,
		USHORT &nMask ) const
{
	CreateByteString_Impl( pErrInf-> GetErrorCode(), rStr );

	return TRUE;
}


void BS_ErrorHandler::CreateByteString_Impl( ULONG nId, ByteString &rStr ) const
{

	ULONG nErrorId = nId;
	ULONG nErrorCode = nId & ((1L << ERRCODE_CLASS_SHIFT ) - 1 );
	ULONG nErrorClass = (nId & ERRCODE_CLASS_MASK ) >> ERRCODE_CLASS_SHIFT;
	ULONG nErrorArea = (nId & ERRCODE_ERROR_MASK &
							~((1 << ERRCODE_AREA_SHIFT ) -1 ))
		>> ERRCODE_AREA_SHIFT;
//	if ( &pSVApp == 0L )
	{
		rStr += "Error : "; //rStr += '\7';
	}

	switch ( nErrorArea )
	{
		case ERRCODE_AREA_TOOLS :
			rStr += "Tools: ";
			break;
		case ERRCODE_AREA_SV :
			rStr += "Sv: ";
		default:
			rStr += "unknown Area :";
	}

	if ( nErrorClass == (ERRCODE_IO_GENERAL >> ERRCODE_CLASS_SHIFT ))
			rStr += "General Error";
	else if ( nErrorClass == (ERRCODE_IO_NOTEXISTS >> ERRCODE_CLASS_SHIFT ))
		rStr += "Not exists";
	else if (  nErrorClass == (ERRCODE_IO_ALREADYEXISTS >> ERRCODE_CLASS_SHIFT ))
		rStr += "Already exists";
	else if ( nErrorClass == (ERRCODE_IO_ACCESSDENIED  >> ERRCODE_CLASS_SHIFT))
		rStr += "Access denied";
	else if ( nErrorClass == (ERRCODE_IO_NOTAFILE  >> ERRCODE_CLASS_SHIFT))
		rStr += "Not a File";
	else if ( nErrorClass == (ERRCODE_IO_INVALIDDEVICE  >> ERRCODE_CLASS_SHIFT))
		rStr += "Unknown Device";
	else if ( nErrorClass == (ERRCODE_IO_INVALIDCHAR  >> ERRCODE_CLASS_SHIFT))
		rStr += "Invalid Mode";
	else if ( nErrorClass == (ERRCODE_IO_LOCKVIOLATION  >> ERRCODE_CLASS_SHIFT))
		rStr += "Lock Violation";
	else if ( nErrorClass == (ERRCODE_IO_OUTOFSPACE  >> ERRCODE_CLASS_SHIFT))
		rStr += "Out of Disk Space";
	else if ( nErrorClass == (ERRCODE_IO_CANTREAD >> ERRCODE_CLASS_SHIFT ))
		rStr += "Read Error";
	else if ( nErrorClass == (ERRCODE_IO_CANTWRITE  >> ERRCODE_CLASS_SHIFT))
		rStr += "Write Error";
	else if ( nErrorClass == ( ERRCODE_IO_UNKNOWN >> ERRCODE_CLASS_SHIFT ))
		rStr += "Unknown Error";
	 else
		rStr += "Really Unknown Error";
}

BS_ErrorHandler::BS_ErrorHandler()
{
	RegisterDisplay( &aWndFunc );
}

BS_ErrorContext::BS_ErrorContext( USHORT nCtxIdP, Window *pWin ) :
	ErrorContext( pWin ), nCtxId( nCtxIdP )
{
}

BS_ErrorContext::BS_ErrorContext( USHORT nCtxIdP, const ByteString &aArg1P,
															Window *pWin ) :
	ErrorContext( pWin ), aArg1( aArg1P), nCtxId( nCtxIdP )
{
}

BOOL BS_ErrorContext::GetByteString( ULONG nErrId, ByteString &rStr )
{
	// den nErrId koennte man dazu benutzen, um zwischen Warning und
	// Error zu unterscheiden

	//rStr += "Error Context : ";

	if ( nErrId == ERRCTX_BS_DELIVER )
		rStr += "Deliver : ";

	// wir machen es uns hier einfach !
	rStr += aArg1;

	return TRUE;
}
