/*************************************************************************
 *
 *  $RCSfile: make.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:02:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "fsys.hxx"
#include "updmake.hxx"
#include "allmake.hxx"
#include "make_err.hxx"


#define MAKE_COMLINE_ERROR 0xF000

const char PRJ_DIR_NAME[] = "prj";
const char DLIST_NAME[] = "d.lst";


/**	@descr
	Checks, beginning with the current working directory, if it is a project root.
	If not, the parent directory is checked and so on until 1 level below the root.
	@return
	Full path of the found project root. An empty String, if none was found.
**/
ByteString GetProjectRoot();


// #define USE_OLD_GETPRJ
#ifdef USE_OLD_GETPRJ
ByteString GetPrj( DirEntry aEntry = DirEntry(String(".", gsl_getSystemTextEncoding())) );
#endif



/****************************************************************************/
void PrintResult( USHORT nResult )
/****************************************************************************/
{
	ByteString sMessage;
	switch ( nResult ) {
	case MK_NO_ERROR:
		break;
	case MK_NO_ENVIRONMENT:
		sMessage = ByteString( "\nERROR: No environment set!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - There is not environment set by setsolar.\n" );
		sMessage += ByteString("       - In the configuration file important settings are missing.\n" );
		sMessage += ByteString("       - The configuration file is inconsistent.\n" );
		break;
	case MK_ERROR_READING_VERLST:
		sMessage = ByteString( "\nERROR: No access to configuration file!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - The configuration file cannot be opened (no access rights).\n" );
		sMessage += ByteString("       - The configuration file does not exist.\n" );
		sMessage += ByteString("       - The configuration file is inconsistent.\n" );
		break;
	case MK_ERROR_NO_PROJECT:
		sMessage = ByteString( "\nERROR: Cannot find project to build!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - Make is run in the wrong directory.\n" );
		sMessage += ByteString("       - The project to build is not known by the configuratin file.\n" );
		break;
	case MK_ERROR_NO_DIRS:
		sMessage = ByteString( "\nERROR: Cannot find source code directories!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - The project to build has no source code directories.\n" );
		sMessage += ByteString("       - The configuration file is inconsistent.\n" );
		break;
	case MK_ERROR_DEPEND:
		sMessage = ByteString( "\nERROR: Cannot resolve dependencies within a project!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - There are circular dependencies within the project.\n" );
		sMessage += ByteString("       - The configuration file is inconsistent.\n" );
		break;
	case MK_ERROR_PRJDEPEND:
		sMessage = ByteString( "\nERROR: Cannot resolve dependencies between projects!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - There are circular dependencies between projects.\n" );
		sMessage += ByteString("       - The configuration file is inconsistent.\n" );
		break;
	case MAKE_COMLINE_ERROR:
		sMessage = ByteString( "\nERROR: Error in command line!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - The switch -file is missing.\n" );
		sMessage += ByteString("       - There is the switch -file without a filename.\n" );
		sMessage += ByteString("       - There is the switch -make without a make-tool-name.\n" );
		sMessage += ByteString("       - There is the switch -get without a projectname.\n" );
		break;
	case MK_ERROR_OUTPUTFILE:
		sMessage = ByteString( "\nERROR: The make script file cannot be written!\n" );
		sMessage += ByteString("       Possible reasons are:\n" );
		sMessage += ByteString("       - The switch -file in the command line gave a non existing file path.\n" );
		break;
	}
	fprintf( stderr, "%s\n", sMessage.GetBuffer());
}

/****************************************************************************/
void Print( const char * cStatus )
/****************************************************************************/
{
	fprintf( stderr, "%s\n", cStatus );
}

//
// class CommandLineMake
//

class CommandLineMake : public UPDMake
{
  public:
						CommandLineMake(
							ByteString &rProject,
							ByteString &rWorkStamp,
							ByteString &rMakeCommand,
							ByteString &rFile )
							: UPDMake( rProject, rWorkStamp, rMakeCommand, rFile ) {};
  protected:
	virtual void 		PrintStatus(
							const char * cStatus )
							{ Print( cStatus ); }

};


/****************************************************************************/
#if defined( UNX ) || defined( MAC )
int main( int argc, char *argv[] )
#else
int _cdecl main( int argc, char *argv[] )
#endif
/****************************************************************************/
{
	Print( "" );
	Print( "make Version 1.1 Copyright (c) 1999-2000 Sun Microsystems, Inc" );
	Print( "==============================================================" );
	Print( "" );

	ByteString sCommand( "nmake" );
	BOOL bBuildAll = FALSE;
	BOOL bListDependenciesOnly = FALSE;
	BOOL bGetProjectFromCvs = FALSE;

	ByteString sProject;
	ByteString sSwitches;
	ByteString sFileName;
	for ( int i = 1; i < argc; i++ )
	{
		ByteString sSwitch( argv[ i ] );
		if ( sSwitch.ToUpperAscii() == "-FILE" )
		{
			if ( i < argc - 1 )
			{
				sFileName = ByteString( argv[ i + 1 ] );
				i++;
			}
			else
			{
				PrintResult( MAKE_COMLINE_ERROR );
				return 1;
			}
		}
		else if ( sSwitch.ToUpperAscii() == "-MAKE" )
		{
			if ( i < argc - 1 )
			{
				sCommand = ByteString( argv[ i + 1 ] );
				i++;
			}
			else
			{
				PrintResult( MAKE_COMLINE_ERROR );
				return 1;
			}
		}
		else if ( sSwitch.ToUpperAscii() == "-ALL" )
		{
			bBuildAll = TRUE;
		}
		else if ( sSwitch.ToUpperAscii() == "-LISTALL" )
		{
			bListDependenciesOnly = TRUE;
		}
		else if ( sSwitch.ToUpperAscii() == "-GET" )
		{
			bGetProjectFromCvs = TRUE;
			if ( i < argc - 1 )
			{
				sProject = ByteString( argv[ i + 1 ] );
				i++;
			}
			else
			{
				PrintResult( MAKE_COMLINE_ERROR );
				return 1;
			}
		}
		else
		{
			sSwitches += ByteString( argv[ i ] );
			sSwitches += ByteString( " " );
		}
	}	// end for

	if ( !sFileName.Len())
	{
		PrintResult( MAKE_COMLINE_ERROR );
		return 1;
	}

	if ( !bGetProjectFromCvs )
	{
#ifndef USE_OLD_GETPRJ
		ByteString sFullProjRoot = GetProjectRoot();
#else
		ByteString sFullProjRoot = ::GetPrj();
#endif
		if (sFullProjRoot.Len() == 0)
		{
			PrintResult( MK_ERROR_NO_PROJECT );
			return 1;
		}
		DirEntry aPrjDir( String( sFullProjRoot, gsl_getSystemTextEncoding()));
		aPrjDir.ToAbs();
		aPrjDir += DirEntry(PRJ_DIR_NAME);
		aPrjDir.SetCWD();
		sProject = ByteString( aPrjDir[1].GetName(), gsl_getSystemTextEncoding());
	}
	ByteString sVersion( getenv( "WORK_STAMP" ) );

	Print( "Reading configuration file / project structure ..." );

	USHORT nError = MK_NO_ERROR;
	if (bBuildAll || bListDependenciesOnly)
	{
		AllMake aMake(sProject,sVersion,sCommand);
		if (bGetProjectFromCvs)
		{
			aMake.Set_GetProjectFromCvs(TRUE);
		}

		nError = aMake.GetErrorCode();
		if ( nError == MK_NO_ERROR )
		{
			BOOL bResult = aMake.WriteScript( sFileName, sSwitches, bListDependenciesOnly );
			if (!bResult)
				nError = aMake.GetErrorCode();
		}
	}
	else
	{
		CommandLineMake aMake( sProject, sVersion, sCommand, sFileName );
		if (bGetProjectFromCvs)
		{
			aMake.Set_GetProjectFromCvs(TRUE);
		}

		nError = aMake.GetErrorCode();
		if ( nError == MK_NO_ERROR )
		{
			aMake.WriteScript( sSwitches );
		}
	}	// endif (bBuildAll)
	PrintResult( nError );

	if ( nError != MK_NO_ERROR )
		return 1;

	return 0;
}

/****************************************************************************/
ByteString GetProjectRoot()
/****************************************************************************/
{
	BOOL bFound = FALSE;
	DirEntry aCur;
	aCur.ToAbs();
	for ( ; ! bFound && aCur.Level() > 1; aCur.CutName() )
	{
		DirEntry aTest = aCur + DirEntry(PRJ_DIR_NAME) + DirEntry(DLIST_NAME);
		if ( aTest.Exists() )
		{
			return ByteString( aCur.GetFull(), gsl_getSystemTextEncoding());
		}
	}	// end for
	return "";
}



