/*************************************************************************
 *
 *  $RCSfile: makedepn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:02:59 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <stl/stl_config.h>

// --- Systemabhaengige Definitionen --------------------------------
#define PATH_DELIM              ';'
#define INC_DELIM				'/'

#define UNXEOL                  '\n'

#ifdef UNX

#define DIR_DELIM               '/'
#define DIR_DELIM_ALT           '\\'

#define FILE_OPENREAD           "r"
#define FILE_OPENWRITE          "w"
#define LINE_END                "\n"
#define EOL						'\n'

#else
#ifdef MAC

#define DIR_DELIM               ':'
#define DIR_DELIM_ALT           '\\'

#define FILE_OPENREAD           "r"
#define FILE_OPENWRITE          "w"
#define LINE_END                "\n"
#define EOL						'\n'

#else

#define DIR_DELIM               '\\'
#define DIR_DELIM_ALT           '/'

#define FILE_OPENREAD           "rb"
#define FILE_OPENWRITE          "wb"
#define LINE_END                "\r\n"
#define EOL                     '\r'

#endif
#endif

// --- Maximale Groessenangaben -------------------------------------
#define MAX_PARAMLEN            256

#define MAX_PATH                256

#define MAX_INCPATH             32
#define MAX_EXCFILE             32

#define MAX_INCLDEPN            1000
#define MAX_INCL                2000

#define MAX_FILES               1000

#define MAX_EXT					10		// max length of extension

#define MAX_SOURCETYPE			9

// --- Fehlermeldungen ----------------------------------------------
#define ERR_MAXPARAMLENTOSMALL  "Error: MAX_PARAMLEN to small\n"
#define ERR_MAXINCPATHTOSMALL   "Error: MAX_INCPATH to small\n"
#define ERR_MAXEXCFILETOSMALL   "Error: MAX_EXCFILE to small\n"
#define ERR_MAXINCLTOSMALL      "Error: MAX_INCL to small\n"
#define ERR_MAXINCLDEPNTOSMALL  "Error: MAX_INCLDEPN to small\n"

#define ERR_FILENOTFOUND        "Error: File %s not found\n"
#define ERR_FILENOTREAD         "Error: cant read File %s\n"
#define ERR_OUTOFMEMORY         "Error: out of memory\n"
#define ERR_COMMENTNOTCLOSED    "Error: comment not closed in File %s line %d\n"
#define ERR_STRINGNOTCLOSED     "Error: string not closed in File %s line %d\n"
#define WRN_MULTILINESTRING     "Warning: multi line string in File %s line %d to %d\n"
#define WRN_UNXFORMAT		    "Warning: %s: File has UNIX lineends - possible errors \n"

#define ERR_WRONGINCLUDE        "Error: wrong include syntax in File %s line %d\n"
#define ERR_TOOLONGINCLUDE      "Error: include syntax too long in File %s line %d (MAX_PATH?)\n"
#define ERR_WRONGINCLUDEIS      "Error: the included file is %s\n"
#define ERR_NOFILES             "Error: no Input Files\n"
#define ERR_WRONGPARAM          "Error: wrong Parameter %s\n"
#define ERR_WRONGOUTPUTFILE     "Error: cant create Output File %s\n"
#define ERR_NOOUTPUTFILE        "Error: no Output File\n"
#define ERR_TOMANYOUTPUTFILES   "Error: more than one Output File\n"
#define ERR_RESFILENOTFOUND     "Error: Response File %s not found\n"

// --- Sonstige Defines ---------------------------------------------
#define TRUE                    1
#define FALSE                   0

// --- Struct-Definitionen ------------------------------------------
struct IncPath
{
    char*       pName;
    IncPath*    pNext;

                ~IncPath() { delete pName; }
};

enum	FType
{
	H,
	HXX,
	CXX,
	HDB,
	HRC,
	HBP,
	HBS,
	O,
	OBJ,
	PAR,
	M,
	NOEXT,
	UNKNOWN
};

// --- Variablen ----------------------------------------------------
int         bVerbose = 0;
int         bDebug = 0;
int         bFirstWarn = 0;

int         nIncPath = 1;
char*       aIncPath[MAX_INCPATH] = { "" };

int         nExcFile = 1;
char*       aExcFile[MAX_EXCFILE] = { "" };

int         nIncl = 0;
int         nCurIncl = 0;
IncPath*    aIncl[MAX_INCL];
int         aCurIncl[MAX_INCLDEPN];

char        aFindName[MAX_PATH];
char        aInclName[MAX_PATH];
char        aParam[MAX_PARAMLEN];

int         nFiles = 0;
char*       aFiles[MAX_FILES];

FILE*       pOutput = NULL;

char*		pSourceExt[MAX_SOURCETYPE] = { ".cxx", ".c", ".E", ".e", ".asm", ".s", ".S", ".scp", ".m" };

// --- Find() -------------------------------------------------------

FILE* Find( const char* pFileName, int nStart = 0 )
{
    FILE*   pFile = NULL;

    for ( int i = nStart; i < nIncPath; i++ )
    {
        // Dateiname mit Pfad zusammenbauen
        strcpy( aFindName, aIncPath[i] );
        size_t nLen = strlen( aFindName );
        if ( nLen && (aFindName[nLen-1] != DIR_DELIM) &&
            (aFindName[nLen-1] != DIR_DELIM_ALT) )
        {
            aFindName[nLen]   = DIR_DELIM;
            aFindName[nLen+1] = '\0';
        }
        strcat( aFindName, pFileName );

        pFile = fopen( aFindName, FILE_OPENREAD );
        if ( pFile )
		{
			fclose(pFile);
            break;
		}
    }

    return pFile;
}

// --- NewIncl() ----------------------------------------------------

IncPath* NewIncl( const char* pFileName )
{
    IncPath* pNewIncl = new IncPath;
    size_t nLen = strlen( pFileName );
    pNewIncl->pName = new char[nLen+1];
    strcpy( pNewIncl->pName, pFileName );
    pNewIncl->pNext = NULL;
    return pNewIncl;
}

// --- AppendIncl() -------------------------------------------------

void AppendIncl( IncPath* pIncPath )
{
    for( int i = 0; i < nCurIncl; i++ )
    {
        // Testen, ob schon in der Liste vorhanden
        IncPath* pIncl = aIncl[aCurIncl[i]];
        while ( pIncl )
        {
            // Vergleichen, ob File schon in der Liste vorkommt
            if ( strcmp( pIncl->pName, pIncPath->pName ) == 0 )
                break;

            // An letzen Eintrag anhaengen
            if ( !pIncl->pNext )
            {
                // Es muss nur der Pointer auf den FileName kopiert werden,
                // da die Daten geshared werden koennen
                pIncl->pNext = new IncPath;
                pIncl->pNext->pName = pIncPath->pName;
                pIncl->pNext->pNext = NULL;
                pIncl = NULL;
            }
            else
                pIncl = pIncl->pNext;
        }
    }
}

// --- AppendAllIncl() ----------------------------------------------

void AppendAllIncl( int nIncl )
{
    // Alle Files an die offenen Files anhaengen
    IncPath* pIncl = aIncl[nIncl];
    while ( pIncl )
    {
        AppendIncl( pIncl );
        pIncl = pIncl->pNext;
    }
}

// --- GetFileType() -------------------------------------------------

FType GetFileType(const char* pFileName)
{
	char* pTmpstr = (char *) pFileName;
	char aExt[MAX_EXT];
	FType eBack;
	int i;
	char sh[]="H", shxx[]="HXX", shrc[]="HRC", shdb[]="HDB", shbp[]="HBP", shbs[]="HBS";
	char so[]="O", sobj[]="OBJ", spar[]="PAR", scxx[]="CXX", sm[]="M";

	// da rel.Pfade Punkte enthalten koennen 2 chars testen
/*
     while ( *pTmpstr != '\0' &&
			( *pTmpstr != '.' || ( *pTmpstr == '.'
				&& ( *(pTmpstr+1) == '.' || *(pTmpstr+1) == DIR_DELIM ))))
	{
		pTmpstr++;
	};
*/
	pTmpstr += strlen( pFileName );
	while ( pTmpstr != pFileName && *pTmpstr != '.'
			&& *pTmpstr != DIR_DELIM && *pTmpstr != INC_DELIM  )
	{
		pTmpstr--;
	}

	if ( *pTmpstr == DIR_DELIM || *pTmpstr == INC_DELIM )
	{
		if ( bVerbose )
		{
			printf("Filetype NO EXTENSION\n");
		}
		return NOEXT;
	}

	i=0;
	while ( *pTmpstr != '\0' )
	{
		pTmpstr++;
		aExt[i]=toupper(*pTmpstr);
		i++;
	}
    if ( strcmp( shxx, aExt ) == 0 ) eBack=HXX;
    else if ( strcmp( sh, aExt ) == 0 ) eBack=H;
    else if ( strcmp( shdb, aExt ) == 0 ) eBack=HDB;
    else if ( strcmp( shrc, aExt ) == 0 ) eBack=HRC;
    else if ( strcmp( shbp, aExt ) == 0 ) eBack=HBP;
    else if ( strcmp( shbs, aExt ) == 0 ) eBack=HBS;
    else if ( strcmp( so, aExt ) == 0 ) eBack=O;
    else if ( strcmp( sobj, aExt ) == 0 ) eBack=OBJ;
    else if ( strcmp( spar, aExt ) == 0 ) eBack=PAR;
    else if ( strcmp( scxx, aExt ) == 0 ) eBack=CXX;
    else if ( strcmp( sm, aExt ) == 0 ) eBack=M;
	else eBack=UNKNOWN;

    if ( bVerbose )
	{
		printf("Filetype *.%s\n",aExt);
	}
	return eBack;
};

// --- SubMakedep() -------------------------------------------------

//int SubMakedep( const char* pFileName, FILE* pFile )
int SubMakedep( const char* pFileName, int bInitial = 0 )
{
    int     i;
    int     bFirstChar;
	int		nLine;
	FType	eFileType;

	eFileType=GetFileType(pFileName);
    // Verbose-Mode an, dann geben wir das File aus, welches wir bearbeiten
    if ( bVerbose )
	{
        printf( "%d: %s\n", nCurIncl, pFileName );
		if ( bDebug )
			fflush(stdout);
	}
//	if ( eFileType == UNKNOWN )
//		return 1;

	// zuallererst schauen, ob das File excluded wurde
	if ( nExcFile > 1 )
	{
		const char*	cp = pFileName + strlen( pFileName );
		while ( --cp >= pFileName
				&& *cp != DIR_DELIM
				&& *cp != DIR_DELIM_ALT )
			;
		cp++;
		for ( i=1; i < nExcFile; i++ )
		{
	        if ( strcmp( aExcFile[i], cp ) == 0 )
	        {
			    if ( bVerbose )
					printf( "   %s excluded\n", cp );
//	            fclose( pFile );
	            return 1;
	        }
		}
	}

    // Zuerst suchen wir, ob wir das File schon kennen
	if ( bInitial == 0 )
	{
		for( i = 0; i < nIncl; i++ )
	    {
		    if ( bDebug )
			{
				printf( "   SubMakedep: nIncl: %d, i: %d, aIncl[i]->pName: %s\n", nIncl, i, aIncl[i]->pName );
				fflush(stdout);
			}
        	// Falls das der Fall ist, muessen wir nicht weiter suchen
	        if ( strcmp( aIncl[i]->pName, pFileName ) == 0 )
    	    {
	//            fclose( pFile );
    	        AppendAllIncl( i );
        	    return 1;
	        }
    	}
	}

    // Defines gross genug gesetzt
    if ( nIncl == MAX_INCL )
    {
        printf( ERR_MAXINCLTOSMALL );
        return 0;
    }
    if ( nCurIncl == MAX_INCLDEPN )
    {
        printf( ERR_MAXINCLDEPNTOSMALL );
        return 0;
    }

    // Include-Daten in die Listen mit aufnehmen
    IncPath* pNewIncl = NewIncl( pFileName );
    AppendIncl( pNewIncl );
    aIncl[nIncl] = pNewIncl;
    aCurIncl[nCurIncl] = nIncl;
    nIncl++;
    nCurIncl++;

	if ( eFileType == OBJ || eFileType == O || eFileType == PAR )
	{
		size_t nO_Len = strlen( pFileName );
		strcpy( aInclName, pFileName );
		char* pStrIdx = aInclName + nO_Len;
		for ( i=nO_Len ; i !=0 ; i--, pStrIdx-- )
		{
			if ( aInclName[i] == '.' ) aInclName[i] = '\0';
			if ( aInclName[i] == DIR_DELIM ) break;
		}
		if ( i == 0 ) return 0;

// extensions in fester reihenfolge (prio) suchen
//.cxx .c .E .e .asm .s .S .scp .m

		pStrIdx++;
		char aFullInclName[ MAX_PATH ];

		for ( i = 0; i < MAX_SOURCETYPE ; i++ )
		{
			strcpy( aFullInclName, pStrIdx );
			strcat( aFullInclName, pSourceExt[i] );
			FILE* pIncFile = Find( aFullInclName, 0 );

			if ( pIncFile )
            {
//            	if ( !SubMakedep( aFindName, pIncFile ) )
            	if ( !SubMakedep( aFindName) )
                {
                    return 0;
                }
            }
		}
	}
	else
	{
	    // File oeffen
		FILE* pFile = fopen( pFileName, FILE_OPENREAD );

 	   // Wenn wir es nicht oeffnen koennen, Fehlermeldung ausgeben
    	if ( !pFile )
	    {
    	    printf( ERR_FILENOTFOUND, pFileName );
        	return 0;
	    }
	    // File einlesen
    	fseek( pFile, 0, SEEK_END );
	    long nFileSize = ftell( pFile );
    	if ( !nFileSize )
    	{
	       nCurIncl--;
    	   return 1;
	    }
    	fseek( pFile, 0, SEEK_SET );
	   	char* pFileData = new char[nFileSize+1];
    	if ( !pFileData )
    	{
        	fclose( pFile );
	        printf( ERR_OUTOFMEMORY );
    	    return 0;
	    }
    	if ( !fread( pFileData, nFileSize, 1, pFile ) )
	    {
    	    delete pFileData;
        	fclose( pFile );
	        printf( ERR_FILENOTREAD, pFileName );
    	    return 0;
	    }
    	fclose( pFile );
	    pFileData[nFileSize] = '\0';

		// File scannen
		bFirstChar = TRUE;
	    i = 0;
		nLine = 1;
	    while ( i < nFileSize )
    	{
        	// Kommentar beruecksichtigen
	        if ( (pFileData[i] == '/') && (pFileData[i+1] == '*') )
    	    {
        	    i+=2;
            	while ( (i < nFileSize) && ((pFileData[i] != '*') || (pFileData[i+1] != '/')) )
	            {
    	            if ( pFileData[i] == EOL )
        	            nLine++;
            	    i++;
	            }
    	        if ( i >= nFileSize )
        	    {
            	    delete pFileData;
                	printf( ERR_COMMENTNOTCLOSED, pNewIncl->pName, nLine );
	                return 0;
    	        }
        	    i += 2;
	        }
    	    // Zeilen-Kommentar beruecksichtigen
        	else if ( (pFileData[i] == '/') && (pFileData[i+1] == '/') )
	        {
    	        while ( (i < nFileSize) && (pFileData[i] != '\r') && (pFileData[i] != '\n') )
        	        i++;
	        }
    	    // Strings beruecksichtigen
        	else if ( ( pFileData[i] == '"' || pFileData[i] == '\'' ) &&
				  ( i == 0 || pFileData[i-1] != '\\' ) )
	        {
			  char cStart = pFileData[i];
			  int nStartLine = nLine;
			  i++;
			  while ( (i < nFileSize) &&
					  // allow multiline strings used in __asm__ statements of gcc
					  // (pFileData[i] != EOL ) &&
					  (pFileData[i] != cStart) )
    	      {
				if ( pFileData[i] == '\\' )
				  i++;
				if ( pFileData[i] == EOL )
				  nLine++;
				i++;
			  }

	            if ( nLine != nStartLine )
    	        {
        	        printf( WRN_MULTILINESTRING, pNewIncl->pName, nStartLine, nLine );
            	}
	            if ( i >= nFileSize || pFileData[i] == EOL )
    	        {
        	        delete pFileData;
            	    printf( ERR_STRINGNOTCLOSED, pNewIncl->pName, nStartLine );
                	return 0;
	            }
    	        i++;
        	}
	        // Include-Anweisung
    	    else if ( bFirstChar && (pFileData[i] == '#') )
        	{
            	i++;
	            while ( pFileData[i] <= ' ' )
    	            i++;

				// "import" can be used instead of "include" in
				// Objective-C files
            	if ( strncmp( "include", &(pFileData[i]), 7 ) == 0 || 
            		strncmp( "import", &(pFileData[i]), 6 ) == 0 )
	            {
    	            i += 7;
        	        if ( strncmp( "_next", &(pFileData[i]), 5 ) == 0 )
            	       i += 5;

                	while ( pFileData[i] <= ' ' )
	                    i++;
					
					// stl macro wrappings around filenames

        	        if ( strncmp( "__STL_NATIVE_C_HEADER", &(pFileData[i]), 21 ) == 0 )
            	       i += 21;

        	        if ( strncmp( "__STL_NATIVE_CPP_HEADER", &(pFileData[i]), 23 ) == 0 )
            	       i += 23;

        	        if ( strncmp( "__STL_NATIVE_CPP_C_HEADER", &(pFileData[i]), 25 ) == 0 )
            	       i += 25;

        	        if ( strncmp( "__STL_NATIVE_HEADER", &(pFileData[i]), 19 ) == 0 )
            	       i += 19;

	                int  nStart;
    	            char nEndChar;
        	        if ( pFileData[i] == '<' )
            	    {
                	    nStart = 1;
	                    nEndChar = '>';
    	            }
        	        else if ( pFileData[i] == '"' )
            	    {
                	    nStart = 0;
                    	nEndChar = '"';
	                }
        	        else if ( pFileData[i] == '(' )
            	    {
                	    nStart = 0;
                    	nEndChar = ')';
	                }
    	            else
        	        {
            	        delete pFileData;
                	    printf( ERR_WRONGINCLUDE, pNewIncl->pName, nLine );
	                    return 0;
    	            }

        	        int nNameLen = 0;
            	    i++;
                	while ( pFileData[i] != nEndChar && nNameLen < MAX_PATH-1 )
	                {
    	                aInclName[nNameLen] = pFileData[i];
        	            nNameLen++;
            	        i++;
                	}
	                aInclName[nNameLen] = '\0';
					if ( nNameLen == MAX_PATH-1 )
					{
            	        delete pFileData;
                	    printf( ERR_TOOLONGINCLUDE, pNewIncl->pName, nLine );
	                    printf( ERR_WRONGINCLUDEIS, aInclName );
    	                return 0;
					}
            	    i++;

	                FILE* pIncFile = Find( aInclName, nStart );
    	            if ( pIncFile )
        	        {
//            	        if ( !SubMakedep( aFindName, pIncFile ) )
            	        if ( !SubMakedep( aFindName ) )
                	    {
                    	    delete pFileData;
	                        return 0;
    	                }
        	        }
	            }
    	        else
        	    {
            	    bFirstChar = FALSE;
	                i++;
    	        }
        	}

	        // Include-Anweisung fuer StarMoney-MACRO-Files
    	    else if ( (eFileType == HBP || eFileType == HBS) && (pFileData[i] == 'I') )
        	{
	            //i++;  WE HAVE NO HASHMARK #

    	        while ( pFileData[i] <= ' ' )
        	        i++;

            	if ( strncmp( "INCLUDE", &(pFileData[i]), 7 ) == 0 )
	            {
    	            i += 7;
        	        if ( strncmp( "_next", &(pFileData[i]), 5 ) == 0 )
            	       i += 5;

	                while ( pFileData[i] <= ' ' )
    	                i++;

        	        int  nStart;
            	    char nEndChar;
                	if ( pFileData[i] == '<' )
	                {
    	                nStart = 1;
        	            nEndChar = '>';
            	    }
	                else
    	            {
        	            delete pFileData;
            	        printf( ERR_WRONGINCLUDE, pNewIncl->pName, nLine );
                	    return 0;
	                }

	                int nNameLen = 0;
    	            i++;
        	        while ( pFileData[i] != nEndChar && nNameLen < MAX_PATH-1 )
            	    {
	                    aInclName[nNameLen] = pFileData[i];
    	                nNameLen++;
        	            i++;
            	    }
	                aInclName[nNameLen] = '\0';
					if ( nNameLen == MAX_PATH-1 )
					{
            	        delete pFileData;
                	    printf( ERR_TOOLONGINCLUDE, pNewIncl->pName, nLine );
	                    printf( ERR_WRONGINCLUDEIS, aInclName );
    	                return 0;
					}
            	    i++;

	                FILE* pIncFile = Find( aInclName, nStart );
    	            if ( pIncFile )
        	        {
//            	        if ( !SubMakedep( aFindName, pIncFile ) )
            	        if ( !SubMakedep( aFindName ) )
                	    {
                    	    delete pFileData;
	                        return 0;
    	                }
        	        }
	            }
    	        else
        	    {
            	    bFirstChar = FALSE;
                	i++;
	            }
    	    }
			// file anweisung aus .hdb files
			else if ( eFileType == HDB && (pFileData[i] == 'f') )
    	    {
        	    while ( pFileData[i] <= ' ' )
            	    i++;

	            if ( strncmp( "file", &(pFileData[i]), 4 ) == 0 )
    	        {
        	        i += 4;
            	    if ( strncmp( "_next", &(pFileData[i]), 5 ) == 0 )
                	   i += 5;

	                while ( pFileData[i] <= ' ' )
    	                i++;

					int  nStart=0;
					int nNameLen = 0;
                	while ( pFileData[i] > ' ' && nNameLen < MAX_PATH-1 )
	                {
    	                aInclName[nNameLen] = pFileData[i];
        	            nNameLen++;
            	        i++;
                	}
	                aInclName[nNameLen] = '\0';
					if ( nNameLen == MAX_PATH-1 )
					{
            	        delete pFileData;
                	    printf( ERR_TOOLONGINCLUDE, pNewIncl->pName, nLine );
	                    printf( ERR_WRONGINCLUDEIS, aInclName );
    	                return 0;
					}
            	    i++;
	                FILE* pIncFile = Find( aInclName, nStart );
    	            if ( pIncFile )
        	        {
//            	        if ( !SubMakedep( aFindName, pIncFile ) )
            	        if ( !SubMakedep( aFindName ) )
                	    {
	                        delete pFileData;
    	                    return 0;
        	            }
            	    }
	            }
    	        else
        	    {
            	    bFirstChar = FALSE;
	                i++;
    	        }
        	}
	        // Zeilenumbruch
    	    else if ( pFileData[i] == EOL )
        	{
	            bFirstChar = TRUE;
    	        i++;
				nLine++;
#ifndef UNX
	            // newline am Zeilenende ueberlesen
				if ( (i < nFileSize) && (pFileData[i] == '\n' ) )
				  i++;
#endif
	        }
			else if ( pFileData[i] == UNXEOL )
			{
	            bFirstChar = TRUE;
    	        i++;
				nLine++;
				if ( !bFirstWarn )
				{
        	    	printf( WRN_UNXFORMAT, pNewIncl->pName, nLine );
					bFirstWarn = TRUE;
				}
/*				delete pFileData;
                printf( ERR_UNXFORMAT, pNewIncl->pName, nLine );
    	        return 0;
*/
			}
				
	        // Ein anderes Zeichen als Space etc.
    	    else if ( pFileData[i] > ' ' )
        	{
	            bFirstChar = FALSE;
    	        i++;
        	}
	        else
    	        i++;
	    }
	    delete pFileData;
	}		// mein zeug....
    nCurIncl--;

    return 1;
}

// --- Makedep() ----------------------------------------------------

int Makedep( const char* pFileName )
{
/*
     // File oeffen
	FILE* pFile = fopen( pFileName, FILE_OPENREAD );

    // Wenn wir es nicht oeffnen koennen, Fehlermeldung ausgeben
    if ( !pFile )
    {
        printf( ERR_FILENOTFOUND, pFileName );
        return 0;
    }
*/
    if ( bVerbose )
        printf( "File: %s:\n", pFileName );

    // Depend-Liste ermitteln
    nCurIncl = 0;
    if ( !SubMakedep( pFileName, 1 ) )
        return 0;

    // Depend-Liste rausschreiben
    size_t n = strlen( pFileName ) + 2;
    fprintf( pOutput, "%s: ", pFileName );
    for ( ; n < 14; n++ )
        fprintf( pOutput, " " );
	
	IncPath* pIncl;
	pIncl = aIncl[aCurIncl[0]]->pNext;

	while ( pIncl )
    {
        size_t nLen = strlen( pIncl->pName );
        if ( (n+nLen) >= 75 )
        {
           fprintf( pOutput, "\\%s              ", LINE_END );
           n = 14;
        }
        n += nLen + 2;
        fprintf( pOutput, "%s ", pIncl->pName );
        pIncl = pIncl->pNext;
    }
    fprintf( pOutput, LINE_END );

    return 1;
}

// --- Help() -------------------------------------------------------

void Help()
{
    // Hilfeseite ausgeben
    printf(
        "MakeDepn (c) 1994 STAR DIVISION\n"
        "Create Depend Files\n\n"

        "makedepn -I:incpath [-E:excfile] -O:output [-V] filelist\n"
        "-I:incpath     Include Path[s]\n"
        "-E:excfile     Exclude File[s]\n"
        "-O:output      Output File\n"
        "-V             Verbose Mode\n"
        "-D             Debug Mode\n"
        "filelist       Files\n"
        "@file          Response File" );
}

// --- PassArg() ----------------------------------------------------

int PassArg( const char* pArg )
{
#ifdef UNX
    if ( *pArg == '-' )
#else
    if ( (*pArg == '/') || (*pArg == '-') )
#endif
    {
        if ( ((pArg[1] == 'I') || (pArg[1] == 'i')) && (pArg[2] == ':') )
        {
            int nStart = 3;
            int nLen = 0;

            while ( pArg[nStart+nLen] )
            {
                if ( pArg[nStart+nLen] == PATH_DELIM )
                {
                    if ( nLen )
                    {
                        if ( nIncPath >= MAX_INCPATH )
						{
							printf( ERR_MAXINCPATHTOSMALL );
							return 0;
						}
                        aIncPath[nIncPath] = new char[nLen+1];
                        strncpy( aIncPath[nIncPath], &(pArg[nStart]), nLen );
                        aIncPath[nIncPath][nLen] = '\0';
						nIncPath++;
                        nStart += nLen + 1;
                        nLen = 0;
                    }
					while ( pArg[nStart] == PATH_DELIM )
						nStart++;		// noch'n Delimiter
                }
                else
                    nLen++;
            }

            if ( nLen )
            {
                if ( nIncPath >= MAX_INCPATH )
				{
					printf( ERR_MAXINCPATHTOSMALL );
					return 0;
				}
                aIncPath[nIncPath] = new char[nLen+1];
                strncpy( aIncPath[nIncPath], &(pArg[nStart]), nLen );
                aIncPath[nIncPath][nLen] = '\0';
                nIncPath++;
            }
        }
        else if ( ((pArg[1] == 'E') || (pArg[1] == 'e')) && (pArg[2] == ':') )
        {
            int nStart = 3;
            int nLen = 0;

            while ( pArg[nStart+nLen] )
            {
                if ( pArg[nStart+nLen] == PATH_DELIM )
                {
                    if ( nLen )
                    {
                        if ( nExcFile >= MAX_EXCFILE )
						{
							printf( ERR_MAXEXCFILETOSMALL );
							return 0;
						}
                        aExcFile[nExcFile] = new char[nLen+1];
                        strncpy( aExcFile[nExcFile], &(pArg[nStart]), nLen );
                        aExcFile[nExcFile][nLen] = '\0';
						nExcFile++;
                        nStart += nLen + 1;
                        nLen = 0;
                    }
					while ( pArg[nStart] == PATH_DELIM )
						nStart++;		// noch'n Delimiter
                }
                else
                    nLen++;
            }

            if ( nLen )
            {
                if ( nExcFile >= MAX_EXCFILE )
				{
					printf( ERR_MAXEXCFILETOSMALL );
					return 0;
				}
                aExcFile[nExcFile] = new char[nLen+1];
                strncpy( aExcFile[nExcFile], &(pArg[nStart]), nLen );
                aExcFile[nExcFile][nLen] = '\0';
				nExcFile++;
            }
        }
        else if ( ((pArg[1] == 'O') || (pArg[1] == 'o')) && (pArg[2] == ':') )
        {
            if ( pOutput )
            {
                printf( ERR_TOMANYOUTPUTFILES );
                return 0;
            }

            pOutput = fopen( &(pArg[3]), FILE_OPENWRITE );
            if ( !pOutput )
            {
                printf( ERR_WRONGOUTPUTFILE, &(pArg[3]) );
                return 0;
            }
        }
        else if ( (pArg[1] == 'V') || (pArg[1] == 'v') )
        {
            bVerbose = 1;
        }
        else if ( (pArg[1] == 'D') || (pArg[1] == 'd') )
        {
            bDebug = 1;
        }
        else
        {
            printf( ERR_WRONGPARAM, pArg );
            return 0;
        }
    }
    else
    {
        size_t nLen = strlen( pArg );
        aFiles[nFiles] = new char[nLen+1];
        strcpy( aFiles[nFiles], pArg );
        nFiles++;
    }

    return 1;
}

// --- main() -------------------------------------------------------

int main( int argc, char* argv[] )
{
    int i;

    if ( argc <= 1 )
    {
        Help();
        return 0;
    }

    for ( i = 1; i < argc; i++ )
    {
        char* pArg = argv[i];

        if ( *pArg == '@' )
        {
            FILE* pResponse = fopen( &(pArg[1]), FILE_OPENREAD );
            if ( !pResponse )
            {
                if ( pOutput )
                    fclose( pOutput );
                printf( ERR_RESFILENOTFOUND, &(pArg[1]) );
                return 1;
            }

            unsigned    char c;
            int         bChar = FALSE;
            int         n = 0;
            while ( fread( &c, 1, 1, pResponse ) )
            {
	            if ( n >= MAX_PARAMLEN )
				{
			        if ( pOutput )
			            fclose( pOutput );
			        printf( ERR_MAXPARAMLENTOSMALL );
			        return 1;
		        }
                if ( c <= 32 )
                {
                    if ( bChar )
                    {
                        aParam[n] = '\0';
                        if ( !PassArg( aParam ) )
                        {
                            if ( pOutput )
                                fclose( pOutput );
                            return 1;
                        }

                        n = 0;
                        bChar = FALSE;
                    }
                }
                else
                {
                    bChar = TRUE;
                    aParam[n] = (char)c;
					n++;
                }
            }

            fclose( pResponse );
        }
        else if ( !PassArg( pArg ) )
        {
            if ( pOutput )
                fclose( pOutput );
            return 1;
        }
    }

    if ( !pOutput )
    {
        printf( ERR_NOOUTPUTFILE );
        return 1;
    }

    if ( !nFiles )
    {
        fclose( pOutput );
        printf( ERR_NOFILES );
        return 1;
    }

    if ( bVerbose )
    {
        printf( "IncludePaths:\n" );
        for ( i = 1; i < nIncPath; i++ )
            printf( "%s\n", aIncPath[i] );
        printf( "ExcludeFiles:\n" );
        for ( i = 1; i < nExcFile; i++ )
            printf( "%s\n", aExcFile[i] );
    }

    for ( i = 0; i < nFiles; i++ )
    {
        if ( !Makedep( aFiles[i] ) )
        {
            fclose( pOutput );
            return 1;
        }
    }

    fclose( pOutput );

    return 0;
}
