/*************************************************************************
 *
 *  $RCSfile: solmath.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _TOOLS_SOLMATH_HXX
#define _TOOLS_SOLMATH_HXX


#ifdef MAC
// MAC braucht natuerlich 'ne Extrawurst
#include <mac_start.h>
#include <fp.h>
#include <mac_end.h>
#else
#include <float.h>
#endif

#ifdef SOLARIS
#include <ieeefp.h>
#endif

#include <math.h>

#ifndef _STRING_HXX //autogen wg. xub_Unicode
#include <string.hxx>
#endif
#ifndef _SOLAR_H
#include <solar.h>
#endif


/// 2**-48 bzw. 3.5527136788005e-015 fuer ApproxEqual Skalierung
#define SOMA_MIN_PRECISION	(1.0 / (16777216.0 * 16777216.0))


/// Rounding modes for SolarMath::Round
enum SolarMathRoundingMode
{
	SolarMathRoundCorrected = 0,	/// like HalfUp, but corrects roundoff errors, preferred
	SolarMathRoundDown,		   		/// floor of absolute value, signed return (commercial)
	SolarMathRoundUp,				/// ceil of absolute value, signed return (commercial)
	SolarMathRoundFloor,			/// floor of signed value
	SolarMathRoundCeiling,			/// ceil of signed value
	SolarMathRoundHalfDown,			/// frac <= 0.5 ? floor of abs : ceil of abs, signed return
	SolarMathRoundHalfUp,  			/// frac < 0.5 ? floor of abs : ceil of abs, signed return (mathematical)
	SolarMathRoundHalfEven			/// IEEE rounding mode (statistical)
};


class International;

/**
	Funktionen zur Verarbeitung von IEEE 754 double values.
	Spezielle Approx...() Funktionen zum Ausbuegeln von Rundungsungenauigkeiten.
	Beispiel: 0.1+0.2-0.3 ergibt roh 5.55112e-017 statt 0.0, damit ceil()
	aufgerufen ergibt 1.0 statt erwartetem 0.0.
	ApproxSub( 0.1+0.2, 0.3 ) ergibt 0.0, ApproxCeil( 0.1+0.2-0.3 ) ergibt 0.0.
 */
class SolarMath
{
public:

	/** Gleichheit zweier Werte bei einer Genauigkeit von der
		Groessenordnung der uebergebenen Werte skaliert mit 2^-48
		(4 Bits Ungenauigkeit erlaubt).
		@ATTENTION ApproxEqual( Wert!=0.0, 0.0 ) liefert trotzdem nie TRUE */
	static	inline	BOOL	ApproxEqual( double a, double b );

	/** Addition zweier Werte. Bei entgegengesetzten Vorzeichen und ca.
		Gleichheit der Absolutbetraege im Rahmen von ApproxEqual wird 0.0
		statt eines berechneten Wertes zurueckgegeben.
		Bei mehreren zu addierenden Werten wird zweckmaessigerweise nicht jede
		Addition mit ApproxAdd durchgefuehrt, sondern der erste nicht 0.0 (!)
		Wert wird gemerkt, alle restlichen Werte addiert, und mit dem Ergebnis
		und dem gemerkten Wert ApproxAdd aufgerufen. */
	static	inline	double	ApproxAdd( double a, double b );

	/** Subtraktion zweier Werte (a-b). Bei gleichen Vorzeichen und ca.
	 	Gleichheit der Absolutbetraege im Rahmen von ApproxEqual wird 0.0
		statt eines berechneten Wertes zurueckgegeben. */
	static	inline	double	ApproxSub( double a, double b );

	/** floor() Funktion unter Beruecksichtigung von ApproxEqual Groessen.
		Einsatz bei als Integer erwarteten aber per Double berechneten Werten.
		@ATTENTION Der Schwellenwert betraegt 3.55271e-015 */
	static	inline	double	ApproxFloor( double a );

	/** ceil() Funktion unter Beruecksichtigung von ApproxEqual Groessen.
		@ATTENTION Der Schwellenwert betraegt 3.55271e-015 */
	static	inline	double	ApproxCeil( double a );

	/** Fuer Compiler-Libs die kein finite() haben, z.B. OS2BLCI, WINMSCI.
		Wird evtl. von dem define SOMA_FINITE(double) verwendet, welches
		stattdessen benutzt werden soll und normalerweise auf finite() etc. mappt.
		SOMA_FINITE(d)==0 => Wert ist +INF, -INF oder NAN */
	static	inline	int		Finite( double d );

	/** Ob ein Wert +INF/-INF repraesentiert. Das Vorzeichen (+INF oder -INF)
		kann mit IsSignBitSet(double) ermittelt werden.
		Wenn SOMA_FINITE(d)==0 und IsINF(d)==FALSE dann NAN. */
	static	inline	BOOL	IsINF( double d );

	/** Test on any QNAN or SNAN */
	static	inline	BOOL	IsNAN( double d );

	/** Ob das Vorzeichen-Bit gesetzt ist */
	static	inline	BOOL	IsSignBitSet( double d );

	/** +INF bei bNegative==FALSE oder -INF bei bNegative==TRUE zuweisen. */
	static	inline	void	SetINF( double& rd, BOOL bNegative );

	/** Set a QNAN */
	static	inline	void	SetNAN( double& rd, BOOL bNegative );

	/** Konvertierung analog sprintf mit eigener Rundung.
		Ist bEraseTrailingDecZero==TRUE werden anhaengende Nullen in
		Dezimalstellen sowie evtl. der Dezimalseparator abgeschnitten.
		Ist cType=='G' verhaelt sich die Funktion wie der sprintf Typ %G,
		'F' oder 'E' Format wird erzeugt, je nachdem was kompakter ist.
		Ist hierbei nDec==INT_MAX wird der Default-Wert (z.Zt. 6) gesetzt.
		Ist cType=='A' wird 'F' oder 'E' Format abhaengig von fVal erzeugt.
		Ist hierbei nDec==INT_MAX wird die groesstmoegliche Anzahl
		signifikanter Nachkommastellen erzeugt (gegebenenfalls
		bEraseTrailingDecZero=TRUE uebergeben), ist nDec!=INT_MAX wird die
		angegebene Zahl Nachkommastellen erzeugt.
		+/-HUGE_VAL wird als "1.#INF" bzw. "-1.#INF" umgesetzt (statt '.'
		natuerlich der cDec). */
	static	void			DoubleToString(
								XubString& sStr,
								double fVal,
								xub_Unicode cType,			/// 'F'  'E'  'G'  'A'
								int nDec,					/// Nachkommastellen
								xub_Unicode cDec = '.',		/// Dezimalseparator
								BOOL bEraseTrailingDecZero = FALSE
								);

	/** Konvertiert analog strtod einen dezimalen String in einen double,
		Dezimalseparator und Tausenderseparator werden aus International
		genommen, fuehrende Leerzeichen werden weggeparst.
		Ist ppEnd!=NULL wird *ppEnd hinter das Weggeparste gesetzt.
		Enthaelt pStr nur den zu parsenden String, ist also bei Erfolg
		**ppEnd=='\0' und *ppEnd-pStr==strlen(pStr).
		Bei Ueberlauf wird fVal=+/-HUGE_VAL gesetzt, bei Unterlauf 0,
		nErrno wird in diesen Faellen auf ERANGE gesetzt, sonst 0.
		"+/-1.#INF" werden als +/-HUGE_VAL erkannt. */
	static	double			StringToDouble(
								const xub_Unicode* pStr,
								const International& rIntl,
								int& nErrno,
								const xub_Unicode** ppEnd = NULL
								);

	/** Converts a decimal string to a number, works the same as the previous function */
	static	double			StringToDouble(
								const xub_Unicode* pStr,
								const xub_Unicode cGrpSep,	///	group (AKA thousands) separator
								const xub_Unicode cDecSep,	/// decimal separator
								int& nErrno,
								const xub_Unicode** ppEnd = NULL
								);

	/** Rounds a double value.

		@param fVal
			specifies the value to be rounded.

		@param nDec
			specifies the decimal place where rounding occurs in the range
			-20 to 20. <0 if position is before the decimal point.

		@param eMode
			specifies the rounding mode

	 */
	static	double			Round(
								double fVal,
								short nDec = 0,
								SolarMathRoundingMode eMode = SolarMathRoundCorrected
								);
};



/** @descr
	FP-Errors per Exception und setjmp/longjmp behandeln? doch nicht in C++!
	Mit SOMA_FPCONTROL() ausschalten und munter drauf los rechnen,
	Endergebnis mit SOMA_FINITE(d) pruefen, 0==Fehler (INFinitiy, NaN), sonst ok.
 */
#if 0
#define SOMA_FPSIGNAL_JUMP 1
#else
#define SOMA_FPSIGNAL_JUMP 0
#endif

// signal() ID fuer FP-Exceptions
#ifdef RS6000
#define SOMA_SIGFPE SIGTRAP
#else
#define SOMA_SIGFPE SIGFPE
#endif

// SOMA_FPCONTROL(): FP-Exceptions ein-/ausschalten, je nach SOMA_FPSIGNAL_JUMP
// SOMA_FPRESET(): reinitialize math package
#if defined(WNT) || defined(WIN)
#define SOMA_FPEXCEPTIONS_ON()	_control87( _MCW_EM, 0 )
#define SOMA_FPEXCEPTIONS_OFF()	_control87( _MCW_EM, _MCW_EM )
#define SOMA_FPRESET()			_fpreset()

#elif defined(OS2)
#define SOMA_FPEXCEPTIONS_ON()	_control87( MCW_EM, 0 )
#define SOMA_FPEXCEPTIONS_OFF()	_control87( MCW_EM, MCW_EM )
#define SOMA_FPRESET()			_fpreset()

#elif defined(RS6000)
#define SOMA_FPEXCEPTIONS_ON()	fp_enable_all()
#define SOMA_FPEXCEPTIONS_OFF()	fp_disable_all()
#define SOMA_FPRESET()

#elif defined(LINUX)
#include <fpu_control.h>

extern inline void glibc_setfpucw( fpu_control_t set )
{
	fpu_control_t cw;

	/* Fetch the current control word.  */
	_FPU_GETCW(cw);

	/* Preserve the reserved bits, and set the rest as the user
	   specified (or the default, if the user gave zero).  */
	cw &= _FPU_RESERVED;
	cw |= set & ~_FPU_RESERVED;

	_FPU_SETCW(cw);
}

#define __setfpucw( control_word ) glibc_setfpucw( control_word )

#define SOMA_FPEXCEPTIONS_ON()	__setfpucw( _FPU_DEFAULT & ~0x001F )
#define SOMA_FPEXCEPTIONS_OFF()	__setfpucw( _FPU_IEEE )
#define SOMA_FPRESET()

#else
#define SOMA_FPEXCEPTIONS_ON()
#define SOMA_FPEXCEPTIONS_OFF()
#define SOMA_FPRESET()
#endif

#if SOMA_FPSIGNAL_JUMP
#define SOMA_FPCONTROL() SOMA_FPEXCEPTIONS_ON()
#else
#define SOMA_FPCONTROL() SOMA_FPEXCEPTIONS_OFF()
#endif



/// SOMA_FINITE(d) : double d auf INFINITY, NaN etc. testen
#if defined(WNT)
#define SOMA_FINITE(d) _finite(d)
#elif defined(MAC)
#define SOMA_FINITE(d) isfinite(d)
#elif defined(LINUX)
#define SOMA_FINITE(d) finite(d)
#elif defined(UNX)
#define SOMA_FINITE(d) finite(d)
#else
// fuer Compiler-Libs die kein finite() haben, z.B. OS2BLCI, WINMSCI
#define SOMA_FINITE(d) SolarMath::Finite(d)
#endif



#if defined(__IEEEDOUBLE) && (__SIZEOFDOUBLE == 8)

#if defined(__BIGENDIAN)

/// IEEE 754 double Strukturen fuer BigEndian
typedef union
{
	struct
	{
		UINT32 sign			: 1;
		UINT32 exponent		:11;
		UINT32 fraction_hi	:20;
		UINT32 fraction_lo	:32;
	} inf_parts;
	struct
	{
		UINT32 sign			: 1;
		UINT32 exponent		:11;
		UINT32 qnan_bit		: 1;
		UINT32 bits			:19;
		UINT32 fraction_lo	:32;
	} nan_parts;
	struct
	{
		UINT32 msw			:32;
		UINT32 lsw			:32;
	} w32_parts;
	double value;
} SolarMathDouble;

#elif defined(__LITTLEENDIAN)

/// IEEE 754 double Strukturen fuer LittleEndian
typedef union
{
#ifndef _WIN16
	struct {
		UINT32 fraction_lo	:32;
		UINT32 fraction_hi	:20;
		UINT32 exponent		:11;
		UINT32 sign			: 1;
	} inf_parts;
	struct {
		UINT32 fraction_lo	:32;
		UINT32 bits			:19;
		UINT32 qnan_bit		: 1;
		UINT32 exponent		:11;
		UINT32 sign			: 1;
	} nan_parts;
	struct
	{
		UINT32 lsw			:32;
		UINT32 msw			:32;
	} w32_parts;
#else
	// stupid 16bit compilers don't get 32bit bit fields right
	struct {
        UINT32 fraction_lo;
        UINT32 sign__exponent__fraction_hi;
	} inf_parts;
	struct {
        UINT32 fraction_lo;
        UINT32 sign__exponent__qnan_bit__bits;
	} nan_parts;
	struct
	{
		UINT32 lsw;
		UINT32 msw;
	} w32_parts;
#endif // _WIN16
	double value;
} SolarMathDouble;

#else
#error __FILE__ ": unsupported ENDIAN for SolarMath IEEE 754"
#endif

#else	// !#if defined(__IEEEDOUBLE) && (__SIZEOFDOUBLE == 8)
#error __FILE__ ": don't know how to handle SolarMath IEEE 754"
#endif



inline BOOL SolarMath::ApproxEqual( double a, double b )
{
	if ( a == b )
		return TRUE;
	double x = a - b;
	return (x < 0.0 ? -x : x)  <  ((a < 0.0 ? -a : a) * SOMA_MIN_PRECISION);
}


inline double SolarMath::ApproxAdd( double a, double b )
{
	if ( ((a < 0.0 && b > 0.0) || (b < 0.0 && a > 0.0)) && ApproxEqual( a, -b ) )
		return 0.0;
	return a + b;
}


inline double SolarMath::ApproxSub( double a, double b )
{
	if ( ((a < 0.0 && b < 0.0) || (a > 0.0 && b > 0.0)) && ApproxEqual( a, b ) )
		return 0.0;
	return a - b;
}


inline double SolarMath::ApproxFloor( double a )
{
	double b = floor( a );
	if ( ApproxEqual( a - 1.0, b ) )
		return b + 1.0;
	return b;
}


inline double SolarMath::ApproxCeil( double a )
{
	double b = ceil( a );
	if ( ApproxEqual( a + 1.0, b ) )
		return b - 1.0;
	return b;
}


inline int SolarMath::Finite( double d )
{	// double auf INFINITY, NaN etc. testen
	// Verwendung mit Define SOMA_FINITE(d)
	// Bitmuster im Exponent ist 0x7FF == 2047
	// sign, exponent, qnan_bit:
	// 0x7FF0 ==  INF
	// 0x7FF1 ==  SNAN  bis 0x7FF7
	// 0x7FF8 ==  QNAN  bis 0x7FFF
	// 0xFFF0 == -INF
	// 0xFFF1 == -SNAN  bis 0xFFF7
	// 0xFFF8 == -IND   (M$ extension? should be a QNAN)
    // 0xFFF9 == -QNAN  bis 0xFFFF
#ifndef _WIN16
    return ((SolarMathDouble*)&d)->inf_parts.exponent != 0x7ff;
#else
    return (((SolarMathDouble*)&d)->inf_parts.
        sign__exponent__fraction_hi & 0x7ff00000) != 0x7ff00000;
#endif
}


inline BOOL SolarMath::IsINF( double d )
{	// exponent==0x7ff fraction==0
	return (SOMA_FINITE(d) == 0) &&
#ifndef _WIN16
        (((SolarMathDouble*)&d)->inf_parts.fraction_hi == 0)
#else
        ((((SolarMathDouble*)&d)->inf_parts.sign__exponent__fraction_hi & 0x000fffff) == 0)
#endif
        && (((SolarMathDouble*)&d)->inf_parts.fraction_lo == 0);
}


inline BOOL SolarMath::IsNAN( double d )
{	// exponent==0x7ff fraction!=0
	return (SOMA_FINITE(d) == 0) && (
#ifndef _WIN16
        (((SolarMathDouble*)&d)->inf_parts.fraction_hi != 0)
#else
        ((((SolarMathDouble*)&d)->inf_parts.sign__exponent__fraction_hi & 0x000fffff) != 0)
#endif
        || (((SolarMathDouble*)&d)->inf_parts.fraction_lo != 0) );
}


inline BOOL SolarMath::IsSignBitSet( double d )
{
#ifndef _WIN16
    return ((SolarMathDouble*)&d)->inf_parts.sign != 0;
#else
    return (((SolarMathDouble*)&d)->inf_parts.sign__exponent__fraction_hi & 0x80000000) != 0;
#endif
}


inline void SolarMath::SetINF( double& rd, BOOL bNegative )
{
	((SolarMathDouble&)rd).w32_parts.msw = ( bNegative ? 0xfff00000 : 0x7ff00000 );
	((SolarMathDouble&)rd).w32_parts.lsw = 0;
}


inline void SolarMath::SetNAN( double& rd, BOOL bNegative )
{
	((SolarMathDouble&)rd).w32_parts.msw = ( bNegative ? 0xffffffff : 0x7fffffff );
	((SolarMathDouble&)rd).w32_parts.lsw = 0xffffffff ;
}



#endif	// _TOOLS_SOLMATH_HXX

