/*************************************************************************
 *
 *  $RCSfile: simplecm.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <solar.h>

#ifndef _SIMPLECM_HXX
#include "simplecm.hxx"
#endif


#ifndef MAKEDWORD
#define MAKEDWORD(wl, wh)   ((Dword)((wl) & 0xFFFF) | (((Dword)(wh) & 0xFFFF) << 16))
#endif
#ifndef LOWORD
#define LOWORD(d)           ((Word)((Dword)(d) & 0xFFFF))
#endif
#ifndef HIWORD
#define HIWORD(d)           ((Word)(((Dword)(d) >> 16) & 0xFFFF))
#endif
#ifndef MAKEWORD
#define MAKEWORD(bl, bh)    ((Word)((bl) & 0xFF) | (((Word)(bh) & 0xFF) << 8))
#endif
#ifndef LOBYTE
#define LOBYTE(w)           ((Byte)((Word)(w) & 0xFF))
#endif
#ifndef HIBYTE
#define HIBYTE(w)           ((Byte)(((Word)(w) >> 8) & 0xFF))
#endif
#ifndef MAKEBYTE
#define MAKEBYTE(nl, nh)    ((Byte)(((nl) & 0x0F) | (((nh) & 0x0F) << 4)))
#endif
#ifndef LONIBBLE
#define LONIBBLE(b)         ((Byte)((b) & 0x0F))
#endif
#ifndef HINIBBLE
#define HINIBBLE(b)         ((Byte)(((b) >> 4) & 0x0F))
#endif

#ifndef SWAPWORD
#define SWAPWORD(w)         MAKEWORD(HIBYTE(w),LOBYTE(w))
#endif
#ifndef SWAPDWORD
#define SWAPDWORD(d)        MAKEDWORD(SWAPWORD(HIWORD(d)),SWAPWORD(LOWORD(d)))
#endif

#ifdef __BIGENDIAN
#ifndef NETWORD
#define NETWORD(w)          (Word)(w)
#endif
#ifndef NETDWORD
#define NETDWORD(d)         (Dword)(d)
#endif
#else  // __LITENDIAN
#ifndef NETWORD
#define NETWORD(w)          MAKEWORD(HIBYTE(w),LOBYTE(w))
#endif
#ifndef NETDWORD
#define NETDWORD(d)         MAKEDWORD(NETWORD(HIWORD(d)),NETWORD(LOWORD(d)))
#endif
#endif // __BIGENDIAN

/**
	Es gibt zwei arten von Datenpaketen
	die erste enthlt in den ersten 4 Byte die Lngenangabe und in den Darauffolgenden die Daten
	Die Lngenangabe bezieht sich nur auf die Daten ohne die Lngenangabe selbst.

	Die Zweite Art von Datenpaketen enthlt Header mit weitere Informationen
	wie unten beschrieben.

	Umgeschaltet wird mit dem Boolean bUseMultiChannel im Konstruktor des Managers.
**/
/**
Defines fr Header Typen:

Allgemeiner Header:
		Byte	Lnge		Inhalt
		0..3		4		Lnge des Paketes ohne diese 4 Byte
		4			1		Prfsumme ber die Lnge. Stimmt sie nicht wird die Verbindung geschlossen
		5..6		2		Lnge des Headers ohne diese 2 Byte
		7..8		2		Typ des Headers


CH_SimpleMultiChannel:
		9..10		2		Channel
CH_Handshake				Internal Use ONLY
							Keine Weiteren Daten!

**/
#define CH_SimpleMultiChannel		0x0001
#define CH_Handshake				0x0002



#define CH_REQUEST_HandshakeAlive	((HandshakeType)0x0101)	// Fordert eine Alive Antwort an
#define CH_RESPONSE_HandshakeAlive	((HandshakeType)0x0102)	// Alive Antwort


CommunicationLink::CommunicationLink( CommunicationManager *pMan )
: pMyManager(pMan)
, pServiceData(NULL)
, nServiceProtocol( 0 )
, bIsInsideCallback( FALSE )
{
}

CommunicationLink::~CommunicationLink()
{
	if ( pMyManager )
		pMyManager->DestroyingLink( this );
}

void CommunicationLink::CallInfoMsg( InfoString aMsg )
{
	if ( pMyManager )
		pMyManager->InfoMsg( aMsg );
};

CM_InfoType CommunicationLink::GetInfoType()
{
	if ( pMyManager )
		return pMyManager->GetInfoType();
	else
		return CM_NO_TEXT;
}

IMPL_LINK( CommunicationLink, ConnectionClosed, void*, EMPTYARG )
{
	if ( pMyManager )
		pMyManager->CallConnectionClosed( this );
	return 1;
}

IMPL_LINK( CommunicationLink, DataReceived, void*, EMPTYARG )
{
	if ( pMyManager )
		pMyManager->CallDataReceived( this );
	return 1;
}



SimpleCommunicationLinkViaSocket::SimpleCommunicationLinkViaSocket( CommunicationManager *pMan, NAMESPACE_VOS(OStreamSocket) *pSocket )
: CommunicationLink( pMan )
, pStreamSocket( pSocket )
, pReceiveStream( NULL )
, aCommunicationPartner()
, aMyName()
{
}

SimpleCommunicationLinkViaSocket::~SimpleCommunicationLinkViaSocket()
{
	if ( pMyManager && pMyManager->IsLinkValid( this ) )
		StopCommunication();
	if ( pStreamSocket )
		delete pStreamSocket;
}

BOOL SimpleCommunicationLinkViaSocket::StopCommunication()
{
	if ( pStreamSocket )
		pStreamSocket->shutdown();

	if ( pStreamSocket )
		pStreamSocket->close();
	delete pStreamSocket;
	pStreamSocket = NULL;

	ConnectionClosed();

	return TRUE;
}

BOOL SimpleCommunicationLinkViaSocket::IsCommunicationError()
{
	return !pStreamSocket;
}

ByteString SimpleCommunicationLinkViaSocket::GetCommunicationPartner( CM_NameType eType )
{
	if ( pStreamSocket )
	{
		switch ( eType )
		{
			case CM_DOTTED:
				{
					NAMESPACE_RTL(OUString) aDotted;
					NAMESPACE_VOS(OSocketAddr) *pPeerAdr = new NAMESPACE_VOS(OSocketAddr);
					pStreamSocket->getPeerAddr( *pPeerAdr );
					((NAMESPACE_VOS(OInetSocketAddr*))pPeerAdr)->getDottedAddr( aDotted );
					delete pPeerAdr;
					return ByteString( UniString(aDotted), RTL_TEXTENCODING_UTF8 );
				}
				break;
			case CM_FQDN:
				{
					if ( !aCommunicationPartner.Len() )
					{
						NAMESPACE_RTL(OUString) aFQDN;
						pStreamSocket->getPeerHost( aFQDN );
						aCommunicationPartner = ByteString( UniString(aFQDN), RTL_TEXTENCODING_UTF8 );
					}
					return aCommunicationPartner;
				}
				break;
		}
	}
	return CByteString( "Unknown" );
}

ByteString SimpleCommunicationLinkViaSocket::GetMyName( CM_NameType eType )
{
	if ( pStreamSocket )
	{
		switch ( eType )
		{
			case CM_DOTTED:
				{
					NAMESPACE_RTL(OUString) aDotted;
					NAMESPACE_VOS(OSocketAddr) *pPeerAdr = new NAMESPACE_VOS(OSocketAddr);
					pStreamSocket->getLocalAddr( *pPeerAdr );
					((NAMESPACE_VOS(OInetSocketAddr*))pPeerAdr)->getDottedAddr( aDotted );
					delete pPeerAdr;
					return ByteString( UniString(aDotted), RTL_TEXTENCODING_UTF8 );
				}
				break;
			case CM_FQDN:
				{
					if ( !aMyName.Len() )
					{
						NAMESPACE_RTL(OUString) aFQDN;
						pStreamSocket->getLocalHost( aFQDN );
						aMyName = ByteString( UniString(aFQDN), RTL_TEXTENCODING_UTF8 );
					}
					return aMyName;
				}
				break;
		}
	}
	return CByteString( "Error" );
}

SvStream* SimpleCommunicationLinkViaSocket::GetBestCommunicationStream()
{
	return new SvMemoryStream;
}

unsigned char SimpleCommunicationLinkViaSocket::CalcCheckByte( UINT32 nBytes )
{
	UINT16 nRes = 0;
	nRes += HIBYTE( HIWORD( nBytes ) ) ^ 0xf0;
	nRes += LOBYTE( HIWORD( nBytes ) ) ^ 0x0f;
	nRes += HIBYTE( LOWORD( nBytes ) ) ^ 0xf0;
	nRes += LOBYTE( LOWORD( nBytes ) ) ^ 0x0f;

	nRes ^= HIBYTE( nRes );

	return LOBYTE( nRes );
}


#define READ_SOCKET( pBuffer, nLength )\
	if ( !bWasError )\
		{bWasError |= pStreamSocket->read( pBuffer, nLength ) != nLength;}

#define READ_SOCKET_LEN( pBuffer, nLength, nTotal )\
	READ_SOCKET( pBuffer, nLength );\
	if ( !bWasError )\
		{nTotal += nLength;}

BOOL SimpleCommunicationLinkViaSocket::ReceiveDataStream()
{
	BOOL bWasError = FALSE;
	BOOL bForceMultiChannelThisPacket = FALSE;
	if ( pStreamSocket )
	{
		UINT32 nBytes;
		nReceiveProtocol = CM_PROTOCOL_OLDSTYLE;

		if ( pStreamSocket->read( &nBytes, sizeof(nBytes) ) != sizeof(nBytes) )
			return FALSE;

		if ( 0xFFFFFFFF == nBytes )		// Expliziter Request fr dieses Datenpaket auf MultiChannel umzuschalten
		{
			if ( pStreamSocket->read( &nBytes, sizeof(nBytes) ) != sizeof(nBytes) )
				return FALSE;
			bForceMultiChannelThisPacket = TRUE;
		}

		nBytes = NETDWORD( nBytes );

		char* pBuffer;
		if ( ( pMyManager && pMyManager->IsMultiChannel() ) || bForceMultiChannelThisPacket )
		{
			ULONG nReadSoFar = 0;
			ULONG nHeaderReadSoFar = 0;

			// Prfbyte fr Lngenangabe
			unsigned char nLenCheck;
			READ_SOCKET_LEN( &nLenCheck, 1, nReadSoFar );
			// Stimmt das Prfbyte?
			bWasError |= nLenCheck != CalcCheckByte( nBytes );


			UINT16 nHeaderBytes;
			READ_SOCKET_LEN( &nHeaderBytes, 2, nReadSoFar );
			nHeaderBytes = NETWORD( nHeaderBytes );
			// reicht der Header ber das Ende hinaus?
			bWasError |= !(nBytes >= nReadSoFar + nHeaderBytes);

			READ_SOCKET_LEN( &nReceiveHeaderType, 2, nHeaderReadSoFar );
			nReceiveHeaderType = NETWORD( nReceiveHeaderType );

			switch ( nReceiveHeaderType )
			{
			case CH_SimpleMultiChannel:
				{
					READ_SOCKET_LEN( &nReceiveProtocol, 2, nHeaderReadSoFar );
					nReceiveProtocol = NETWORD( nReceiveProtocol );
				}
				break;
			case CH_Handshake:
				{
				}
				break;
			default:
				{
					DBG_ERROR("Unbekannter Headertyp in der Kommunikation");
					bWasError = TRUE;
				}

			}

			if ( bWasError )
				return FALSE;

			/// Lngen anpassen und ggf restheader berlesen.
			while ( nHeaderBytes > nHeaderReadSoFar )
			{
				unsigned char nDummy;
				READ_SOCKET_LEN( &nDummy, 1, nHeaderReadSoFar );
			}

			nReadSoFar += nHeaderReadSoFar;
			nBytes -= nReadSoFar;

		}

		pBuffer = new char[nBytes];
		if ( bWasError |= pStreamSocket->read( pBuffer, nBytes ) != nBytes )
		{
			__DELETE(nBytes) pBuffer;
			return FALSE;
		}
		pReceiveStream = new SvMemoryStream();
		pReceiveStream->SetBuffer( pBuffer, nBytes, TRUE, nBytes );
	}
	else
		bWasError = TRUE;

	return !bWasError;
}

#define WRITE_SOCKET( pBuffer, nLength )\
	if ( !bWasError )\
		bWasError |= pStreamSocket->write( pBuffer, nLength ) != nLength

BOOL SimpleCommunicationLinkViaSocket::TransferDataStream( SvStream *pDataStream, CMProtocol nProtocol )
{
	INFO_MSG( CByteString("S :").Append( GetCommunicationPartner( CM_FQDN ) ),
		CByteString("Daten Senden:").Append( GetCommunicationPartner( CM_FQDN ) ),
		CM_SEND, this );
	BOOL bWasError = FALSE;

	UINT32 nBuffer = 0;
	nBuffer += pDataStream->SeekRel(0) +1;

	if ( pMyManager && pMyManager->IsMultiChannel() )
		nBuffer += 1+2+2+2;	// fr einen CH_SimpleMultiChannel

	UINT32 nNetworkBuffer = NETDWORD( nBuffer );
	WRITE_SOCKET( &nNetworkBuffer, sizeof(nNetworkBuffer) );


	if ( pMyManager && pMyManager->IsMultiChannel() )
	{
		UINT16 n16;
		unsigned char c;

		c = CalcCheckByte( nBuffer );
		WRITE_SOCKET( &c, 1 );

		n16 = 4;	// Lnge des Headers fr einen CH_SimpleMultiChannel
		n16 = NETWORD( n16 );
		WRITE_SOCKET( &n16, 2 );

		n16 = CH_SimpleMultiChannel;	// Typ des Headers
		n16 = NETWORD( n16 );
		WRITE_SOCKET( &n16, 2 );

		nProtocol = NETWORD( nProtocol );
		WRITE_SOCKET( &nProtocol, 2 );
	}


	nBuffer = pDataStream->SeekRel(0) +1;
	if ( !bWasError )
		bWasError |= pStreamSocket->write( ((SvMemoryStream*)pDataStream)->GetData(), nBuffer ) != nBuffer;
	if ( bWasError )
	{
		INFO_MSG( CByteString("Send Failed:").Append( GetCommunicationPartner( CM_FQDN ) ),
			CByteString( "Socket wird wegen Fehlers beim Senden geschlossen: ").Append( GetCommunicationPartner( CM_FQDN ) ),
			CM_ERROR, this );
		StopCommunication();
	}
	return !bWasError;
}

BOOL SimpleCommunicationLinkViaSocket::SendHandshake( HandshakeType aHandshakeType )
{
	BOOL bWasError = FALSE;

	UINT32 nBuffer = 0;

//	if ( pMyManager->IsMultiChannel() )		Wir senden immer FFFFFFFF vorweg -> immer MultiChannel (Oder GPF bei lteren)
		nBuffer += 1+2+2;	// fr einen CH_Handshake

	nBuffer += 2;	// fr den Typ des Handshakes

	switch ( aHandshakeType )
	{
		case CH_REQUEST_HandshakeAlive:
			nBuffer += 0;	// Keine extra Daten
			break;
		case CH_RESPONSE_HandshakeAlive:
			nBuffer += 0;	// Keine extra Daten
			break;
	}

	UINT32 n32;
	n32 = 0xffffffff;	// Umschalten auf MultiChannel
	n32 = NETDWORD( n32 );
	WRITE_SOCKET( &n32, 4 );

	UINT32 nNetworkBuffer = NETDWORD( nBuffer );
	WRITE_SOCKET( &nNetworkBuffer, sizeof(nNetworkBuffer) );


	UINT16 n16;
	unsigned char c;

	c = CalcCheckByte( nBuffer );
	WRITE_SOCKET( &c, 1 );

	n16 = 2;	// Lnge des Headers fr einen CH_Handshake
	n16 = NETWORD( n16 );
	WRITE_SOCKET( &n16, 2 );

	n16 = CH_Handshake;	// Typ des Headers
	n16 = NETWORD( n16 );
	WRITE_SOCKET( &n16, 2 );

	n16 = aHandshakeType;	// Typ des Handshakes
	n16 = NETWORD( n16 );
	WRITE_SOCKET( &n16, 2 );


	if ( bWasError )
	{
		INFO_MSG( CByteString("Send Failed:").Append( GetCommunicationPartner( CM_FQDN ) ),
			CByteString( "Socket wird wegen Fehlers beim Senden geschlossen: ").Append( GetCommunicationPartner( CM_FQDN ) ),
			CM_ERROR, this );
		StopCommunication();
	}
	return !bWasError;
}

SimpleCommunicationLinkViaSocketWithReceiveCallbacks::SimpleCommunicationLinkViaSocketWithReceiveCallbacks( CommunicationManager *pMan, NAMESPACE_VOS(OStreamSocket) *pSocket )
: SimpleCommunicationLinkViaSocket( pMan, pSocket )
{
}

BOOL SimpleCommunicationLinkViaSocketWithReceiveCallbacks::ReceiveDataStream()
{

	if ( SimpleCommunicationLinkViaSocket::ReceiveDataStream() )
	{
		pServiceData = pReceiveStream;
		nServiceProtocol = nReceiveProtocol;
		nServiceHeaderType = nReceiveHeaderType;
		StartCallback();
		DataReceived();
		return TRUE;
	}
	else
	{
		StartCallback();
		ConnectionClosed();
		return FALSE;
	}
}



CommunicationManager::CommunicationManager( BOOL bUseMultiChannel )
: bIsCommunicationRunning( FALSE )
, nInfoType( CM_NONE )
, bIsMultiChannel( bUseMultiChannel )
{
}

CommunicationManager::~CommunicationManager()
{
	StopCommunication();

	xLastNewLink.Clear();
}

/*BOOL CommunicationManager::IsCommunicationError()
{
	return FALSE;
}*/
BOOL CommunicationManager::StopCommunication()
{
	DBG_WARNING("StopCommunication in Basisklasse gerufen!!!!!!!!!!!");
	return FALSE;
}

ByteString CommunicationManager::GetMyName( CM_NameType eType )
{
	NAMESPACE_RTL(OUString) aHostname;
	NAMESPACE_VOS(OSocketAddr)::getLocalHostname( aHostname );
	return ByteString( UniString(aHostname), RTL_TEXTENCODING_UTF8 );
}

void CommunicationManager::CallConnectionOpened( CommunicationLink* pCL )
{
	pCL->StartCallback();		// Sollte bereits vor dem Aufruf gerufen werden
	bIsCommunicationRunning = TRUE;
	xLastNewLink = pCL;

	INFO_MSG( CByteString("C+:").Append( pCL->GetCommunicationPartner( CM_FQDN ) ),
		CByteString("Verbindung aufgebaut: ").Append( pCL->GetCommunicationPartner( CM_FQDN ) ),
		CM_OPEN, pCL );
	ConnectionOpened( pCL );
	pCL->FinishCallback();
}

void CommunicationManager::CallConnectionClosed( CommunicationLink* pCL )
{
	pCL->StartCallback();		// Sollte bereits vor dem Aufruf gerufen werden
	INFO_MSG( CByteString("C-:").Append( pCL->GetCommunicationPartner( CM_FQDN ) ),
		CByteString("Verbindung abgebrochen: ").Append( pCL->GetCommunicationPartner( CM_FQDN ) ),
		CM_CLOSE, pCL );
	ConnectionClosed( pCL );

	if ( xLastNewLink == pCL )
		xLastNewLink.Clear();

	pCL->FinishCallback();
//	delete pCL;
}

void CommunicationManager::CallDataReceived( CommunicationLink* pCL )
{
	pCL->StartCallback();		// Sollte bereits vor dem Aufruf gerufen werden
	CommunicationLinkRef rHold(pCL);	// Hlt den Zeiger bis zum Ende des calls

	if ( CH_Handshake == pCL->nServiceHeaderType )
	{
		SvStream *pData = pCL->GetServiceData();
		USHORT nType;
		*pData >> nType;
		switch ( nType )
		{
			case CH_REQUEST_HandshakeAlive:
				{
					pCL->SendHandshake( CH_RESPONSE_HandshakeAlive );
				}
				break;
		}
		delete pData;
	}
	else
	{
		INFO_MSG( CByteString("D :").Append( pCL->GetCommunicationPartner( CM_FQDN ) ),
			CByteString("Daten Empfangen:").Append( pCL->GetCommunicationPartner( CM_FQDN ) ),
			CM_RECEIVE, pCL );
		DataReceived( pCL );
	}
	delete pCL->GetServiceData();
	pCL->FinishCallback();
}

void CommunicationManager::CallInfoMsg( InfoString aMsg )
{
	// Hier wird es wohl kein Housekeeping geben
	InfoMsg( aMsg );
}




SingleCommunicationManager::SingleCommunicationManager( BOOL bUseMultiChannel )
: CommunicationManager( bUseMultiChannel )
{
	xActiveLink = NULL;
	pInactiveLink = NULL;
}

SingleCommunicationManager::~SingleCommunicationManager()
{
	StopCommunication();
	if ( pInactiveLink )
		pInactiveLink->InvalidateManager();
}

BOOL SingleCommunicationManager::StopCommunication()
{
	if ( xActiveLink.Is() )
	{
		BOOL bSuccess = xActiveLink->StopCommunication();
		if ( pInactiveLink )
			pInactiveLink->InvalidateManager();
		pInactiveLink = xActiveLink;
		xActiveLink.Clear();
		return bSuccess;
	}
	return TRUE;
}

BOOL SingleCommunicationManager::IsLinkValid( CommunicationLink* pCL )
{
	return &xActiveLink == pCL;
}

USHORT SingleCommunicationManager::GetCommunicationLinkCount()
{
	return IsCommunicationRunning()?1:0;
}

CommunicationLinkRef SingleCommunicationManager::GetCommunicationLink( USHORT nNr )
{
	return xActiveLink;
}

void SingleCommunicationManager::CallConnectionOpened( CommunicationLink* pCL )
{
	DBG_ASSERT( !xActiveLink.Is(), "Es ist bereits ein CommunicationLink aktiv");
	if ( xActiveLink.Is() )
	{
		if ( pInactiveLink )
			pInactiveLink->InvalidateManager();
		pInactiveLink = xActiveLink;
		xActiveLink->StopCommunication();	// Den alten Link brutal abwrgen
	}
	xActiveLink = pCL;
	CommunicationManager::CallConnectionOpened( pCL );
}

void SingleCommunicationManager::CallConnectionClosed( CommunicationLink* pCL )
{
	CommunicationManager::CallConnectionClosed( pCL );

	DBG_ASSERT( pCL == xActiveLink, "SingleCommunicationManager::CallConnectionClosed mit fremdem Link");
	if ( pInactiveLink )
		pInactiveLink->InvalidateManager();
	pInactiveLink = xActiveLink;
	xActiveLink.Clear();
	bIsCommunicationRunning = FALSE;
}

void SingleCommunicationManager::DestroyingLink( CommunicationLink *pCL )
{
	pInactiveLink = NULL;
	pCL->InvalidateManager();
}


SingleCommunicationManagerClientViaSocket::SingleCommunicationManagerClientViaSocket( ByteString aHost, ULONG nPort, BOOL bUseMultiChannel )
: SingleCommunicationManager( bUseMultiChannel )
, aHostToTalk( aHost )
, nPortToTalk( nPort )
{
}


SingleCommunicationManagerClientViaSocket::SingleCommunicationManagerClientViaSocket( BOOL bUseMultiChannel )
: SingleCommunicationManager( bUseMultiChannel )
, aHostToTalk()
, nPortToTalk( 0 )
{
}


BOOL CommonSocketFunctions::DoStartCommunication( CommunicationManager *pCM, ICommunicationManagerClient *pCMC, ByteString aHost, ULONG nPort )
{
	NAMESPACE_VOS(OInetSocketAddr) Addr;
	NAMESPACE_VOS(OConnectorSocket) *pConnSocket;

	Addr.setAddr( NAMESPACE_RTL(OUString)( UniString( aHost, RTL_TEXTENCODING_UTF8 ) ) );
	Addr.setPort( nPort );

	TimeValue aTV;
	aTV.Seconds = 10;		// Warte 10 Sekunden
	aTV.Nanosec = 0;
	do
	{
		pConnSocket = new NAMESPACE_VOS(OConnectorSocket)();
		pConnSocket->setTcpNoDelay( 1 );
		if ( pConnSocket->connect( Addr, &aTV ) == NAMESPACE_VOS(ISocketTypes::TResult_Ok) )
		{
			pConnSocket->setTcpNoDelay( 1 );
			pCM->CallConnectionOpened( CreateCommunicationLink( pCM, pConnSocket ) );
			return TRUE;
		}
		else
			delete pConnSocket;

	} while ( pCMC->RetryConnect() );

	return FALSE;
}

