/*************************************************************************
 *
 *  $RCSfile: intnwnt.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SVWIN_H
#include <svwin.h>
#endif

#ifndef _INTNTAB_HXX
#include <intntab.hxx>
#endif

// System-Sprache und NT-Sprachen
#define MAX_NTLANG		260

static LanguageType eImplSystemLanguage = LANGUAGE_DONTKNOW;
static USHORT		nNTLangCount = 0;
static LanguageType aNTLangTab[MAX_NTLANG];

// =======================================================================

LanguageType GetSVLang( ULONG nWinLangId )
{
	LANGID	nLangId 		= (LANGID)nWinLangId;
	USHORT	nPrimaryLangId	= PRIMARYLANGID( nLangId );
	USHORT	nSubLangId		= SUBLANGID( nLangId );

	// No Translation, we work with the original MS code without the SORT_ID.
	// So we can get never LANG-ID's from MS, which are currently not
	// defined from us
	return (LanguageType)MAKELANGID( nPrimaryLangId, nSubLangId );
}

// =======================================================================

ULONG GetWinLang( LanguageType eType )
{
	if ( (eType == LANGUAGE_DONTKNOW) || (eType == LANGUAGE_SYSTEM) )
		return (ULONG)MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT );
	else
		return (ULONG)(USHORT)eType;
}

// =======================================================================

static USHORT StrToNum( const wchar_t* pStr )
{
	USHORT n = 0;

	// Solange es sich um eine Ziffer handelt, String umwandeln
	while( (*pStr >= 48) && (*pStr <= 57) )
	{
		n *= 10;
		n += ((*pStr) - 48);
		pStr++;
	}

	return n;
}

// =======================================================================

static ULONG HexStrToNum( const char* pStr )
{
	ULONG n = 0;

	while ( *pStr )
	{
		if ( (*pStr >= 48) && (*pStr <= 57) )
			n = n * 16 + (*pStr - '0');
		else if ( (*pStr >= 'a') && (*pStr <= 'f') )
			n = n * 16 + (*pStr - 'a' + 10);
		else if ( (*pStr >= 'A') && (*pStr <= 'F') )
			n = n * 16 + (*pStr - 'A' + 10);
		pStr++;
	}

	return n;
}

// =======================================================================

WIN_BOOL CALLBACK EnumLocalesCallbackProc( LPTSTR pStr )
{
	ULONG nLCID = HexStrToNum( pStr );
	LanguageType eLang = GetSVLang( LANGIDFROMLCID( nLCID ) );

	if ( (eLang != LANGUAGE_DONTKNOW) && (eLang != LANGUAGE_SYSTEM) )
	{
		// keine doppelten dazufuegen
		for ( USHORT i = 0; i < nNTLangCount; i++ )
		{
			if ( aNTLangTab[i] == eLang )
				return TRUE;
		}

		// Sprache dazufuegen
		aNTLangTab[nNTLangCount] = eLang;
		nNTLangCount++;

		if ( nNTLangCount == MAX_NTLANG )
			return FALSE;
	}

	return TRUE;
}

// =======================================================================

static StringCompare ImplSystemLanguageCompare( const UniString& rStr1,
												const UniString& rStr2,
												USHORT nFlags,
												LanguageType eLang )
{
	int 	nRet;
	LANGID	nLangId = (LANGID)GetWinLang( eLang );
	LCID	nLCId	= MAKELCID( nLangId, SORT_DEFAULT );
	DWORD	nCmpFlags  = 0;

	if ( nFlags & INTN_COMPARE_IGNORECASE )
		nCmpFlags |= NORM_IGNORECASE;

	// Falls es sich nicht um den ANSI-CharSet handelt, muessen wir
	// vorher konvertieren
	nRet = CompareStringW( nLCId, nCmpFlags,
						   rStr1.GetBuffer(), rStr1.Len(),
						   rStr2.GetBuffer(), rStr2.Len() );

	if ( nRet == 1 )
		return COMPARE_LESS;
	else if ( nRet == 3 )
		return COMPARE_GREATER;
	else
		return COMPARE_EQUAL;
}

// =======================================================================

USHORT ImplGetSystemLanguageCount()
{
	if ( !nNTLangCount )
		EnumSystemLocales( EnumLocalesCallbackProc, LCID_INSTALLED );
	return nNTLangCount;
}

// -----------------------------------------------------------------------

LanguageType GetSystemLanguage( USHORT nIndex )
{
	// Soll die im System eingestellte Sprache ermittelt werden
	if ( nIndex == INTERNATIONAL_SYSTEM_DEFAULT )
	{
		if ( eImplSystemLanguage == LANGUAGE_DONTKNOW )
		{
			LANGID nLangId;

			nLangId = GetUserDefaultLangID();
			eImplSystemLanguage = GetSVLang( nLangId );

			if ( eImplSystemLanguage == LANGUAGE_DONTKNOW )
			{
				nLangId = GetSystemDefaultLangID();
				eImplSystemLanguage = GetSVLang( nLangId );
			}
		}

		return eImplSystemLanguage;
	}
	else
		return aNTLangTab[nIndex];
}

// -----------------------------------------------------------------------

void ImplUpdateSystemLanguage( LanguageType eLanguage, LanguageTable* pTable )
{
	// Wenn keine Table gesetzt ist, dann Cache-Sprache zuruecksetzen
	if ( !pTable )
	{
		eImplSystemLanguage = LANGUAGE_DONTKNOW;
		return;
	}

	// Window-Funktionen setzen
	pTable->SetCompareProc( ImplSystemLanguageCompare );

	// Im System eingestellte Landesinformationen werden uebernommen
	wchar_t 	aBuf[512];
	UniString	aStr;
	USHORT		i;
	LANGID		nLangID = (LANGID)GetWinLang( eLanguage );
	LCID		nLCID = MAKELCID( nLangID, SORT_DEFAULT );

	// Wochentage und Monatsnamen uebertragen
	for ( i = 0; i < DAYOFWEEK_COUNT; i++ )
	{
		if ( GetLocaleInfoW( nLCID, LOCALE_SDAYNAME1+i, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
		{
			aStr = (const wchar_t*)aBuf;
			pTable->SetDayText( (DayOfWeek)(((USHORT)MONDAY)+i), aStr );
		}
		if ( GetLocaleInfoW( nLCID, LOCALE_SABBREVDAYNAME1+i, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
		{
			aStr = (const wchar_t*)aBuf;
			pTable->SetAbbrevDayText( (DayOfWeek)(((USHORT)MONDAY)+i), aStr );
		}
	}
	for ( i = 0; i < MONTH_COUNT; i++ )
	{
		if ( GetLocaleInfoW( nLCID, LOCALE_SMONTHNAME1+i, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
		{
			aStr = (const wchar_t*)aBuf;
			pTable->SetMonthText( 1+i, aStr );
		}
		if ( GetLocaleInfoW( nLCID, LOCALE_SABBREVMONTHNAME1+i, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
		{
			aStr = (const wchar_t*)aBuf;
			pTable->SetAbbrevMonthText( 1+i, aStr );
		}
	}
}

// -----------------------------------------------------------------------

USHORT ImplGetSystemFormatCount()
{
	return ImplGetSystemLanguageCount();
}

// -----------------------------------------------------------------------

LanguageType GetSystemFormatLanguage( USHORT nIndex )
{
	return GetSystemLanguage( nIndex );
}

// -----------------------------------------------------------------------

void ImplUpdateSystemFormat( LanguageType eLanguage, FormatTable* pTable )
{
	if ( !pTable )
	{
		eImplSystemLanguage = LANGUAGE_DONTKNOW;
		return;
	}

	// Im System eingestellte Landesinformationen werden uebernommen
	xub_StrLen	nIndex;
	xub_StrLen	nIndex2;
	wchar_t 	aBuf[512];
	USHORT		n;
	UniString	aStr;
	LANGID		nLangID = (LANGID)GetWinLang( eLanguage );
	LCID		nLCID = MAKELCID( nLangID, SORT_DEFAULT );

	// Datumsformat uebertragen
	if ( GetLocaleInfoW( nLCID, LOCALE_SDATE, aBuf, 2 ) )
	{
		if ( aBuf[0] )
			pTable->SetDateSep( aBuf[0] );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_SSHORTDATE, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
	{
		aStr = (const wchar_t*)aBuf;
		if ( (aBuf[0] == 'y') || (aBuf[0] == 'Y') )
			pTable->SetDateFormat( YMD );
		else if ( (aBuf[0] == 'm') || (aBuf[0] == 'M') )
			pTable->SetDateFormat( MDY );
		else
			pTable->SetDateFormat( DMY );
		if ( (aStr.SearchAscii( "dd" ) != STRING_NOTFOUND) ||
			 (aStr.SearchAscii( "DD" ) != STRING_NOTFOUND) )
			pTable->SetDateDayLeadingZero( TRUE );
		else
			pTable->SetDateDayLeadingZero( FALSE );
		if ( (aStr.SearchAscii( "MM" ) != STRING_NOTFOUND) ||
			 (aStr.SearchAscii( "mm" ) != STRING_NOTFOUND) )
			pTable->SetDateMonthLeadingZero( TRUE );
		else
			pTable->SetDateMonthLeadingZero( FALSE );
		if ( (aStr.SearchAscii( "YYYY" ) != STRING_NOTFOUND) ||
			 (aStr.SearchAscii( "yyyy" ) != STRING_NOTFOUND) )
			pTable->SetDateCentury( TRUE );
		else
			pTable->SetDateCentury( FALSE );
	}

	// Langes Datumsformat uebertragen
	if ( GetLocaleInfoW( nLCID, LOCALE_SLONGDATE, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
	{
		aStr = (const wchar_t*)aBuf;
		if ( (aStr.SearchAscii( "dddd" ) != STRING_NOTFOUND) ||
			 (aStr.SearchAscii( "DDDD" ) != STRING_NOTFOUND) )
		{
			pTable->SetLongDateDayOfWeekFormat( DAYOFWEEK_LONG );
			aStr.Erase( 0, 4 );
		}
		else if ( (aStr.SearchAscii( "ddd" ) != STRING_NOTFOUND) ||
				  (aStr.SearchAscii( "DDD" ) != STRING_NOTFOUND) )
		{
			pTable->SetLongDateDayOfWeekFormat( DAYOFWEEK_SHORT );
			aStr.Erase( 0, 3 );
		}
		else
			pTable->SetLongDateDayOfWeekFormat( DAYOFWEEK_NONE );

		if ( aStr.GetChar( 0 ) == '\'' )
		{
			aStr.Erase( 0, 1 );
			nIndex2 = aStr.Search( '\'' );
			pTable->SetLongDateDayOfWeekSep( aStr.Copy( 0, nIndex2 ) );
			aStr.Erase( 0, nIndex2+1 );
		}
		else
		{
			UniString aTempStr;
			while ( (aStr.GetChar( 0 ) != 'd') && (aStr.GetChar( 0 ) != 'D') &&
					(aStr.GetChar( 0 ) != 'm') && (aStr.GetChar( 0 ) != 'M') &&
					(aStr.GetChar( 0 ) != 'y') && (aStr.GetChar( 0 ) != 'Y') )
			{
				aTempStr += aStr.GetChar( 0 );
				aStr.Erase( 0, 1 );
			}
			pTable->SetLongDateDayOfWeekSep( aTempStr );
		}

		if ( (aStr.GetChar( 0 ) == 'y') || (aStr.GetChar( 0 ) == 'Y') )
			pTable->SetLongDateFormat( YMD );
		else if ( (aStr.GetChar( 0 ) == 'm') || (aStr.GetChar( 0 ) == 'M') )
			pTable->SetLongDateFormat( MDY );
		else
			pTable->SetLongDateFormat( DMY );

		nIndex = aStr.SearchAscii( "dd" );
		if ( nIndex == STRING_NOTFOUND )
			nIndex = aStr.SearchAscii( "DD" );
		if ( nIndex != STRING_NOTFOUND )
		{
			pTable->SetLongDateDayLeadingZero( TRUE );
			aStr.Erase( nIndex, 2 );
		}
		else
		{
			pTable->SetLongDateDayLeadingZero( FALSE );
			nIndex = aStr.Search( 'd' );
			if ( nIndex == STRING_NOTFOUND )
				nIndex = aStr.Search( 'D' );
			aStr.Erase( nIndex, 1 );
		}
		if ( aStr.GetChar( nIndex ) == '\'' )
		{
			aStr.Erase( nIndex, 1 );
			nIndex2 = aStr.Search( '\'', nIndex );
			pTable->SetLongDateDaySep( aStr.Copy( nIndex, nIndex2-nIndex ) );
			aStr.Erase( nIndex, nIndex2-nIndex+1 );
		}
		else
		{
			UniString aTempStr;
			while ( (nIndex < aStr.Len()) &&
					(aStr.GetChar( nIndex ) != 'm') && (aStr.GetChar( nIndex ) != 'M') &&
					(aStr.GetChar( nIndex ) != 'y') && (aStr.GetChar( nIndex ) != 'Y') )
			{
				aTempStr += aStr.GetChar( nIndex );
				aStr.Erase( nIndex, 1 );
			}
			pTable->SetLongDateDaySep( aTempStr );
		}

		nIndex = aStr.SearchAscii( "mmmm" );
		if ( nIndex == STRING_NOTFOUND )
			nIndex = aStr.SearchAscii( "MMMM" );
		if ( nIndex != STRING_NOTFOUND )
		{
			pTable->SetLongDateMonthFormat( MONTH_LONG );
			aStr.Erase( nIndex, 4 );
		}
		else
		{
			nIndex = aStr.SearchAscii( "mmm" );
			if ( nIndex == STRING_NOTFOUND )
				nIndex = aStr.SearchAscii( "MMM" );
			if ( nIndex != STRING_NOTFOUND )
			{
				pTable->SetLongDateMonthFormat( MONTH_SHORT );
				aStr.Erase( nIndex, 3 );
			}
			else
			{
				nIndex = aStr.SearchAscii( "mm" );
				if ( nIndex == STRING_NOTFOUND )
					nIndex = aStr.SearchAscii( "MM" );
				if ( nIndex != STRING_NOTFOUND )
				{
					pTable->SetLongDateMonthFormat( MONTH_ZERO );
					aStr.Erase( nIndex, 2 );
				}
				else
				{
					nIndex = aStr.Search( 'm' );
					if ( nIndex == STRING_NOTFOUND )
						nIndex = aStr.Search( 'M' );
					pTable->SetLongDateMonthFormat( MONTH_NORMAL );
					aStr.Erase( nIndex, 1 );
				}
			}
		}

		if ( (nIndex != STRING_NOTFOUND) && (aStr.GetChar( nIndex ) == '\'') )
		{
			aStr.Erase( nIndex, 1 );
			nIndex2 = aStr.Search( '\'', nIndex );
			pTable->SetLongDateMonthSep( aStr.Copy( nIndex, nIndex2-nIndex ) );
			aStr.Erase( nIndex, nIndex2-nIndex+1 );
		}
		else
		{
			UniString aTempStr;
			while ( (nIndex < aStr.Len()) &&
					(aStr.GetChar( nIndex ) != 'y') && (aStr.GetChar( nIndex ) != 'Y') )
			{
				aTempStr += aStr.GetChar( nIndex );
				aStr.Erase( nIndex, 1 );
			}
			pTable->SetLongDateMonthSep( aTempStr );
		}

		nIndex = aStr.SearchAscii( "yyyy" );
		if ( nIndex == STRING_NOTFOUND )
			nIndex = aStr.SearchAscii( "YYYY" );
		if ( nIndex != STRING_NOTFOUND )
		{
			pTable->SetLongDateCentury( TRUE );
			aStr.Erase( nIndex, 4 );
		}
		else
		{
			pTable->SetLongDateCentury( FALSE );
			nIndex = aStr.SearchAscii( "yy" );
			if ( nIndex == STRING_NOTFOUND )
				nIndex = aStr.SearchAscii( "YY" );
			aStr.Erase( nIndex, 2 );
		}

		if ( (nIndex != STRING_NOTFOUND) && (aStr.GetChar( nIndex ) == '\'') )
		{
			aStr.Erase( 0, 1 );
			aStr.Erase( aStr.Len(), 1 );
		}
		pTable->SetLongDateYearSep( aStr );
	}

	// Zeitformat uebertragen
	if ( GetLocaleInfoW( nLCID, LOCALE_ITIME, aBuf, 2 ) )
	{
		if ( aBuf[0] == '1' )
			pTable->SetTimeFormat( HOUR_24 );
		else
			pTable->SetTimeFormat( HOUR_12 );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_STIME, aBuf, 2 ) )
	{
		if ( aBuf[0] )
			pTable->SetTimeSep( aBuf[0] );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_S1159, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
	{
		aStr = (const wchar_t*)aBuf;
		if ( aStr.Len() && (aStr.GetChar( 0 ) != ' ') )
			aStr.Insert( ' ', 0 );
		pTable->SetTimeAM( aStr );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_S2359, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
	{
		aStr = (const wchar_t*)aBuf;
		if ( aStr.Len() && (aStr.GetChar( 0 ) != ' ') )
			aStr.Insert( ' ', 0 );
		pTable->SetTimePM( aStr );
	}

	// Anhaengsel wie "Uhr" besorgen
	if ( pTable->GetTimeFormat() == HOUR_24 )
	{
		if ( GetLocaleInfoW( nLCID, LOCALE_STIMEFORMAT, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
		{
			aStr = (const wchar_t*)aBuf;
			if ( aStr.Len() )
			{
				nIndex = aStr.Len()-1;
				if ( aStr.GetChar( nIndex ) == '\'' )
				{
					nIndex--;
					while ( nIndex )
					{
						if ( aStr.GetChar( nIndex ) == '\'' )
							break;
						nIndex--;
					}
					if ( nIndex )
						aStr.Erase( 0, nIndex+1 );
					aStr.Erase( aStr.Len()-1 );
					pTable->SetTimeStr( aStr );
				}
			}
		}
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_ITLZERO, aBuf, 2 ) )
	{
		if ( aBuf[0] == '1' )
			pTable->SetTimeLeadingZero( TRUE );
		else
			pTable->SetTimeLeadingZero( FALSE );
	}

	// Nummerformat uebertragen
	if ( GetLocaleInfoW( nLCID, LOCALE_STHOUSAND, aBuf, 2 ) )
	{
		if ( aBuf[0] )
		{
			pTable->SetNumThousandSep( aBuf[0] );
			pTable->EnableNumThousandSep( TRUE );
		}
		else
			pTable->EnableNumThousandSep( FALSE );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_SDECIMAL, aBuf, 2 ) )
	{
		if ( aBuf[0] )
			pTable->SetNumDecimalSep( aBuf[0] );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_ILZERO, aBuf, 2 ) )
	{
		if ( aBuf[0] == '1' )
			pTable->SetNumLeadingZero( TRUE );
		else
			pTable->SetNumLeadingZero( FALSE );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_IDIGITS, aBuf, 3 ) )
		pTable->SetNumDigits( StrToNum( aBuf ) );

	// Currencyformat uebertragen
	if ( GetLocaleInfoW( nLCID, LOCALE_ICURRENCY, aBuf, 2 ) )
	{
		n = StrToNum( aBuf );
		pTable->SetCurrPositiveFormat( n );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_INEGCURR, aBuf, 3 ) )
	{
		n = StrToNum( aBuf );
		if ( n == 11 )
			n = 12;
		else if ( n == 12 )
			n = 11;
		pTable->SetCurrNegativeFormat( n );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_SCURRENCY, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
	{
		aStr = (const wchar_t*)aBuf;
		pTable->SetCurrSymbol( aStr );
	}
	if ( GetLocaleInfoW( nLCID, LOCALE_SINTLSYMBOL, aBuf, sizeof( aBuf )/sizeof( wchar_t ) ) )
	{
		aStr = (const wchar_t*)aBuf;
		pTable->SetCurrBankSymbol( aStr );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_ICURRDIGITS, aBuf, 3 ) )
		pTable->SetCurrDigits( StrToNum( aBuf ) );

	// ListSep uebertragen
	if ( GetLocaleInfoW( nLCID, LOCALE_SLIST, aBuf, 2 ) )
	{
		if ( aBuf[0] )
			pTable->SetListSep( aBuf[0] );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_IMEASURE, aBuf, 2 ) )
	{
		n = StrToNum( aBuf );
		if ( n == 0 )
			pTable->SetMeasurementSystem( MEASURE_METRIC );
		else if ( n == 1 )
			pTable->SetMeasurementSystem( MEASURE_US );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_IFIRSTWEEKOFYEAR, aBuf, 2 ) )
	{
		n = StrToNum( aBuf );
		if ( n == 0 )
			pTable->SetWeekCountStart( WEEKCOUNT_FIRSTDAY );
		else if ( n == 1 )
			pTable->SetWeekCountStart( WEEKCOUNT_FIRSTFULLWEEK );
		else if ( n == 2 )
			pTable->SetWeekCountStart( WEEKCOUNT_FIRST4DAYWEEK );
	}

	if ( GetLocaleInfoW( nLCID, LOCALE_IFIRSTDAYOFWEEK, aBuf, 2 ) )
	{
		n = StrToNum( aBuf );
		if ( (n >= 0) && (n <= 6) )
			pTable->SetWeekStart( (DayOfWeek)n );
	}
}
