/*************************************************************************
 *
 *  $RCSfile: isolang.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <isolang.hxx>
#include <string.hxx>

// =======================================================================

struct IsoLangEntry
{
	LanguageType		meLang;
	sal_Char			maLangStr[3];
	sal_Char			maCountry[3];
};

struct IsoLangEngEntry
{
	LanguageType		meLang;
	sal_Char			maCountry[3];
};

struct IsoLangNoneStdEntry
{
	LanguageType		meLang;
	sal_Char			maLangStr[3];
	sal_Char			maCountry[9];
};

struct IsoLangOtherEntry
{
	LanguageType		meLang;
	const sal_Char* 	mpLangStr;
};

// -----------------------------------------------------------------------

// Entries for languages are lower case, for countries upper case,
// as recommended by rfc1766.
// ConvertIsoNamesToLanguage is case insensitive

// Sortorder: Most used first
// The default entry for every language string has to be first.

static IsoLangEntry const aImplIsoLangEntries[] =
{
	// Lang (MS-LCID-Codes) 			  ISO639  ISO3166
	{ LANGUAGE_ENGLISH, 					"en", ""   },
	{ LANGUAGE_ENGLISH_US,					"en", "US" },
	{ LANGUAGE_ENGLISH_UK,					"en", "GB" },
	{ LANGUAGE_ENGLISH_AUS, 				"en", "AU" },
	{ LANGUAGE_ENGLISH_CAN, 				"en", "CA" },
	{ LANGUAGE_FRENCH,						"fr", "FR" },
	{ LANGUAGE_GERMAN,						"de", "DE" },
	{ LANGUAGE_ITALIAN, 					"it", "IT" },
	{ LANGUAGE_DUTCH,						"nl", "NL" },
	{ LANGUAGE_SPANISH, 					"es", "ES" },
	{ LANGUAGE_SPANISH_MODERN,				"es", "ES" },
	{ LANGUAGE_PORTUGUESE,					"pt", "PT" },
	{ LANGUAGE_PORTUGUESE_BRAZILIAN,		"pt", "BR" },
	{ LANGUAGE_DANISH,						"da", "DK" },
	{ LANGUAGE_GREEK,						"el", "GR" },
	{ LANGUAGE_CHINESE, 					"zh", ""   },
	{ LANGUAGE_CHINESE_TRADITIONAL, 		"zh", "TW" },
	{ LANGUAGE_CHINESE_SIMPLIFIED,			"zh", "CN" },
	{ LANGUAGE_CHINESE_HONGKONG,			"zh", "HK" },
	{ LANGUAGE_CHINESE_SINGAPORE,			"zh", "SG" },
	{ LANGUAGE_CHINESE_MACAU,				"zh", "MO" },
	{ LANGUAGE_JAPANESE,					"ja", "JP" },
	{ LANGUAGE_KOREAN,						"ko", "KR" },
	{ LANGUAGE_KOREAN_JOHAB,				"ko", "KR" },
	{ LANGUAGE_KOREAN,						"ko", "KP" },   // North Korea
	{ LANGUAGE_SWEDISH, 					"sv", "SE" },
	{ LANGUAGE_SWEDISH_FINLAND, 			"sv", "FI" },
	{ LANGUAGE_FINNISH, 					"fi", "FI" },
	{ LANGUAGE_RUSSIAN, 					"ru", "RU" },
	{ LANGUAGE_ENGLISH_NZ,					"en", "NZ" },
	{ LANGUAGE_ENGLISH_EIRE,				"en", "IE" },
	{ LANGUAGE_ENGLISH_SAFRICA, 			"en", "ZA" },
	{ LANGUAGE_DUTCH_BELGIAN,				"nl", "BE" },
	{ LANGUAGE_FRENCH_BELGIAN,				"fr", "BE" },
	{ LANGUAGE_FRENCH_CANADIAN, 			"fr", "CA" },
	{ LANGUAGE_FRENCH_SWISS,				"fr", "CH" },
	{ LANGUAGE_GERMAN_SWISS,				"de", "CH" },
	{ LANGUAGE_GERMAN_AUSTRIAN, 			"de", "AT" },
	{ LANGUAGE_ITALIAN_SWISS,				"it", "CH" },
	{ LANGUAGE_ARABIC,						"ar", ""   },
	{ LANGUAGE_ARABIC_SAUDI_ARABIA, 		"ar", "SA" },
	{ LANGUAGE_ARABIC_EGYPT,				"ar", "EG" },
	{ LANGUAGE_ARABIC_UAE,					"ar", "AE" },
	{ LANGUAGE_AFRIKAANS,					"af", "ZA" },
	{ LANGUAGE_ALBANIAN,					"sq", "AL" },
	{ LANGUAGE_ARABIC_IRAQ, 				"ar", "IQ" },
	{ LANGUAGE_ARABIC_LIBYA,				"ar", "LY" },
	{ LANGUAGE_ARABIC_ALGERIA,				"ar", "DZ" },
	{ LANGUAGE_ARABIC_MOROCCO,				"ar", "MA" },
	{ LANGUAGE_ARABIC_TUNISIA,				"ar", "TN" },
	{ LANGUAGE_ARABIC_OMAN, 				"ar", "OM" },
	{ LANGUAGE_ARABIC_YEMEN,				"ar", "YE" },
	{ LANGUAGE_ARABIC_SYRIA,				"ar", "SY" },
	{ LANGUAGE_ARABIC_JORDAN,				"ar", "JO" },
	{ LANGUAGE_ARABIC_LEBANON,				"ar", "LB" },
	{ LANGUAGE_ARABIC_KUWAIT,				"ar", "KW" },
	{ LANGUAGE_ARABIC_BAHRAIN,				"ar", "BH" },
	{ LANGUAGE_ARABIC_QATAR,				"ar", "QA" },
	{ LANGUAGE_BASQUE,						"eu", ""   },
	{ LANGUAGE_BULGARIAN,					"bg", "BG" },
	{ LANGUAGE_CATALAN, 					"ca", "AD" },
	{ LANGUAGE_CROATIAN,					"hr", "HR" },
	{ LANGUAGE_CZECH,						"cs", "CZ" },
	{ LANGUAGE_ENGLISH_JAMAICA, 			"en", "JM" },
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"en", "BS" },   // not 100%, because AG is Bahamas
	{ LANGUAGE_ENGLISH_BELIZE,				"en", "BZ" },
	{ LANGUAGE_ENGLISH_TRINIDAD,			"en", "TT" },
	{ LANGUAGE_ENGLISH_ZIMBABWE,			"en", "ZW" },
	{ LANGUAGE_ENGLISH_PHILIPPINES, 		"en", "PH" },
	{ LANGUAGE_ESTONIAN,					"et", "EE" },
	{ LANGUAGE_FAEROESE,					"fo", "DK" },
	{ LANGUAGE_FARSI,						"fa", ""   },
	{ LANGUAGE_FRENCH_LUXEMBOURG,			"fr", "LU" },
	{ LANGUAGE_FRENCH_MONACO,				"fr", "MC" },
	{ LANGUAGE_GERMAN_LUXEMBOURG,			"de", "LU" },
	{ LANGUAGE_GERMAN_LIECHTENSTEIN,		"de", "LI" },
	{ LANGUAGE_HEBREW,						"he", "IL" },   // new: old was "iw"
	{ LANGUAGE_HEBREW,						"iw", "IL" },   // old: new is "he"
	{ LANGUAGE_HUNGARIAN,					"hu", "HU" },
	{ LANGUAGE_ICELANDIC,					"is", "IS" },
	{ LANGUAGE_INDONESIAN,					"id", "ID" },   // new: old was "in"
	{ LANGUAGE_INDONESIAN,					"in", "ID" },   // old: new is "id"
	{ LANGUAGE_NORWEGIAN,					"no", "NO" },
	{ LANGUAGE_POLISH,						"pl", "PL" },
	{ LANGUAGE_RHAETO_ROMAN,				"rm", ""   },
	{ LANGUAGE_ROMANIAN,					"ro", "RO" },
	{ LANGUAGE_SLOVAK,						"sk", "SK" },
	{ LANGUAGE_SLOVENIAN,					"sl", "SI" },
	{ LANGUAGE_SPANISH_MEXICAN, 			"es", "MX" },
	{ LANGUAGE_SPANISH_GUATEMALA,			"es", "GT" },
	{ LANGUAGE_SPANISH_COSTARICA,			"es", "CR" },
	{ LANGUAGE_SPANISH_PANAMA,				"es", "PA" },
	{ LANGUAGE_SPANISH_DOMINICAN_REPUBLIC,	"es", "DO" },
	{ LANGUAGE_SPANISH_VENEZUELA,			"es", "VE" },
	{ LANGUAGE_SPANISH_COLOMBIA,			"es", "CO" },
	{ LANGUAGE_SPANISH_PERU,				"es", "PE" },
	{ LANGUAGE_SPANISH_ARGENTINA,			"es", "AR" },
	{ LANGUAGE_SPANISH_ECUADOR, 			"es", "EC" },
	{ LANGUAGE_SPANISH_CHILE,				"es", "CL" },
	{ LANGUAGE_SPANISH_URUGUAY, 			"es", "UY" },
	{ LANGUAGE_SPANISH_PARAGUAY,			"es", "PY" },
	{ LANGUAGE_SPANISH_BOLIVIA, 			"es", "BO" },
	{ LANGUAGE_SPANISH_EL_SALVADOR, 		"es", "SV" },
	{ LANGUAGE_SPANISH_HONDURAS,			"es", "HN" },
	{ LANGUAGE_SPANISH_NICARAGUA,			"es", "NI" },
	{ LANGUAGE_SPANISH_PUERTO_RICO, 		"es", "PR" },
	{ LANGUAGE_TURKISH, 					"tr", "TR" },
	{ LANGUAGE_UKRAINIAN,					"uk", "UA" },
	{ LANGUAGE_VIETNAMESE,					"vi", "VN" },
	{ LANGUAGE_LATVIAN, 					"lv", "LV" },
	{ LANGUAGE_MACEDONIAN,					"mk", "MK" },
	{ LANGUAGE_MALAY,						"ms", ""   },
	{ LANGUAGE_MALAY_MALAYSIA,				"ms", "MY" },
	{ LANGUAGE_MALAY_BRUNEI_DARUSSALAM, 	"ms", "BN" },
	{ LANGUAGE_BELARUSIAN,					"be", ""   },
	{ LANGUAGE_THAI,						"th", "TH" },
	{ LANGUAGE_LITHUANIAN,					"lt", "LT" },
	{ LANGUAGE_LITHUANIAN_CLASSIC,			"lt", "LT" },
	{ LANGUAGE_SERBIAN, 					"sr", ""   },
	{ LANGUAGE_ARMENIAN,					"hy", "AM" },
	{ LANGUAGE_AZERI,						"az", ""   },
	{ LANGUAGE_BENGALI, 					"bn", "BD" },
	{ LANGUAGE_KAZAK,						"kk", ""   },
	{ LANGUAGE_URDU,						"ur", "IN" },
	{ LANGUAGE_HINDI,						"hi", "IN" },
	{ LANGUAGE_GUJARATI,					"gu", "IN" },
	{ LANGUAGE_KANNADA, 					"kn", "IN" },
	{ LANGUAGE_ASSAMESE,					"as", "IN" },
	{ LANGUAGE_KASHMIRI,					"ks", ""   },
	{ LANGUAGE_KASHMIRI_INDIA,				"as", "IN" },
	{ LANGUAGE_MALAYALAM,					"ml", "IN" },
	{ LANGUAGE_MARATHI, 					"mr", ""   },
	{ LANGUAGE_NEPALI,						"ne", ""   },
	{ LANGUAGE_NEPALI_INDIA,				"mr", "IN" },
	{ LANGUAGE_ORIYA,						"or", "IN" },
	{ LANGUAGE_PUNJABI, 					"pa", "IN" },
	{ LANGUAGE_SANSKRIT,					"sa", ""   },
	{ LANGUAGE_SINDHI,						"sd", "IN" },
//	{ LANGUAGE_KONKANI, 					"",   ""   },   // unknown ISO code
//	{ LANGUAGE_MANIPURI,					"",   ""   },   // unknown ISO code
//	{ LANGUAGE_SORBIAN, 					"",   ""   },   // unknown ISO code and is not used anymore from MS
	{ LANGUAGE_DONTKNOW,					"",   ""   }    // marks end of table
};

// -----------------------------------------------------------------------

// In this table are the countries which should mapped to a specific
// english language
static IsoLangEngEntry const aImplIsoLangEngEntries[] =
{
	{ LANGUAGE_ENGLISH_UK,					"AO" },         // Angola
	{ LANGUAGE_ENGLISH_UK,					"BJ" },         // Benin
	{ LANGUAGE_ENGLISH_UK,					"BW" },         // Botswana
	{ LANGUAGE_ENGLISH_UK,					"BI" },         // Burundi
	{ LANGUAGE_ENGLISH_UK,					"CM" },         // Cameroon
	{ LANGUAGE_ENGLISH_UK,					"GA" },         // Gabon
	{ LANGUAGE_ENGLISH_UK,					"GM" },         // Gambia
	{ LANGUAGE_ENGLISH_UK,					"GH" },         // Ghana
	{ LANGUAGE_ENGLISH_UK,					"GN" },         // Guinea
	{ LANGUAGE_ENGLISH_UK,					"MW" },         // Malawi
	{ LANGUAGE_ENGLISH_UK,					"MT" },         // Malta
	{ LANGUAGE_ENGLISH_UK,					"NG" },         // Nigeria
	{ LANGUAGE_ENGLISH_UK,					"UG" },         // Uganda
	{ LANGUAGE_ENGLISH_UK,					"ZM" },         // Zambia
	{ LANGUAGE_ENGLISH_UK,					"ZW" },         // Zimbabwe
	{ LANGUAGE_ENGLISH_UK,					"SZ" },         // Swaziland
	{ LANGUAGE_ENGLISH_UK,					"NG" },         // Sierra Leone
	{ LANGUAGE_ENGLISH_UK,					"KN" },         // Saint Kitts and Nevis
	{ LANGUAGE_ENGLISH_UK,					"SH" },         // St. Helena
	{ LANGUAGE_ENGLISH_UK,					"IO" },         // British Indian Oceanic Territory
	{ LANGUAGE_ENGLISH_UK,					"FK" },         // Falkland Islands
	{ LANGUAGE_ENGLISH_UK,					"GI" },         // Gibraltar
	{ LANGUAGE_ENGLISH_UK,					"KI" },         // Kiribati
	{ LANGUAGE_ENGLISH_UK,					"VG" },         // Virgin Islands
	{ LANGUAGE_ENGLISH_UK,					"MU" },         // Mauritius
	{ LANGUAGE_ENGLISH_UK,					"FJ" },         // Fiji
	{ LANGUAGE_ENGLISH_US,					"KI" },         // Kiribati
	{ LANGUAGE_ENGLISH_US,					"LS" },         // Lesotho
	{ LANGUAGE_ENGLISH_US,					"LR" },         // Liberia
	{ LANGUAGE_ENGLISH_US,					"GU" },         // Guam
	{ LANGUAGE_ENGLISH_US,					"MH" },         // Marshall Islands
	{ LANGUAGE_ENGLISH_US,					"PW" },         // Palau
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"AI" },         // Anguilla
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"AG" },         // Antigua and Barbuda
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"BS" },         // Bahamas
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"BB" },         // Barbedos
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"BM" },         // Bermuda
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"KY" },         // Cayman Islands
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"GD" },         // Grenada
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"DM" },         // Dominica
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"HT" },         // Haiti
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"MS" },         // Montserrat
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"FM" },         // Micronesia
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"VC" },         // St. Vincent / Grenadines
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"LC" },         // Saint Lucia
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"TC" },         // Turks & Caicos Islands
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"GY" },         // Guyana
	{ LANGUAGE_ENGLISH_CARRIBEAN,			"TT" },         // Trinidad and Tobago
	{ LANGUAGE_ENGLISH_AUS, 				"CX" },         // Christmas Islands
	{ LANGUAGE_ENGLISH_AUS, 				"CC" },         // Cocos (Keeling) Islands
	{ LANGUAGE_ENGLISH_AUS, 				"NF" },         // Norfolk Island
	{ LANGUAGE_ENGLISH_AUS, 				"PG" },         // Papua New Guinea
	{ LANGUAGE_ENGLISH_AUS, 				"SB" },         // Solomon Islands
	{ LANGUAGE_ENGLISH_AUS, 				"TV" },         // Tuvalu
	{ LANGUAGE_ENGLISH_AUS, 				"NR" },         // Nauru
	{ LANGUAGE_ENGLISH_NZ,					"CK" },         // Cook Islands
	{ LANGUAGE_ENGLISH_NZ,					"NU" },         // Niue
	{ LANGUAGE_ENGLISH_NZ,					"TK" },         // Tokelau
	{ LANGUAGE_ENGLISH_NZ,					"TO" },         // Tonga
	{ LANGUAGE_DONTKNOW,					""   }          // marks end of table
};

// -----------------------------------------------------------------------

static IsoLangNoneStdEntry const aImplIsoNoneStdLangEntries[] =
{
	{ LANGUAGE_NORWEGIAN_BOKMAL,			"no", "BOK"      }, // registered subtags for "no" in rfc1766
	{ LANGUAGE_NORWEGIAN_NYNORSK,			"no", "NYN"      }, // registered subtags for "no" in rfc1766
	{ LANGUAGE_SERBIAN_LATIN,				"sr", "latin"    },
	{ LANGUAGE_SERBIAN_CYRILLIC,			"sr", "cyrillic" },
	{ LANGUAGE_AZERI_LATIN, 				"az", "latin"    },
	{ LANGUAGE_AZERI_CYRILLIC,				"az", "cyrillic" },
	{ LANGUAGE_DONTKNOW,					"",   ""         }  // marks end of table
};

// -----------------------------------------------------------------------

// in this table are only names to find the best language
static IsoLangNoneStdEntry const aImplIsoNoneStdLangEntries2[] =
{
	{ LANGUAGE_NORWEGIAN_BOKMAL,			"no", "bokmaal"  },
	{ LANGUAGE_NORWEGIAN_BOKMAL,			"no", "bokmal"   },
	{ LANGUAGE_NORWEGIAN_NYNORSK,			"no", "nynorsk"  },
	{ LANGUAGE_DONTKNOW,					"",   ""         }  // marks end of table
};

// -----------------------------------------------------------------------

// in this table are only names to find the best language
static IsoLangOtherEntry const aImplOtherEntries[] =
{
	{ LANGUAGE_ENGLISH_US,					"c"              },
	{ LANGUAGE_GERMAN,						"german"         },
	{ LANGUAGE_JAPANESE,					"japanese"       },
	{ LANGUAGE_DONTKNOW,					NULL			 }	// marks end of table
};

// =======================================================================

void ConvertLanguageToIsoNames( LanguageType eLang,
								UniString& rLangStr, UniString& rCountry )
{
	// Search for enum (in this table we found only defined ISO combinations)
	const IsoLangEntry* pEntry = aImplIsoLangEntries;
	do
	{
		if ( pEntry->meLang == eLang )
		{
			rLangStr.AssignAscii( pEntry->maLangStr );
			rCountry.AssignAscii( pEntry->maCountry );
			return;
		}
		pEntry++;
	}
	while ( pEntry->meLang != LANGUAGE_DONTKNOW );

	// Search for enum, if we don't found a specific ISO combination.
	// All entries in this tables are allowed for mime specifications,
	// but not defined ISO combinations
	const IsoLangNoneStdEntry* pNoneStdEntry = aImplIsoNoneStdLangEntries;
	do
	{
		if ( pNoneStdEntry->meLang == eLang )
		{
			rLangStr.AssignAscii( pNoneStdEntry->maLangStr );
			rCountry.AssignAscii( pNoneStdEntry->maCountry );
			return;
		}
		pNoneStdEntry++;
	}
	while ( pNoneStdEntry->meLang != LANGUAGE_DONTKNOW );

	// not found
	rLangStr.Erase();
	rCountry.Erase();
}

// -----------------------------------------------------------------------

void ConvertLanguageToIsoNames( LanguageType eLang,
								ByteString& rLangStr, ByteString& rCountry )
{
	// Search for enum (in this table we found only defined ISO combinations)
	const IsoLangEntry* pEntry = aImplIsoLangEntries;
	do
	{
		if ( pEntry->meLang == eLang )
		{
			rLangStr.Assign( pEntry->maLangStr );
			rCountry.Assign( pEntry->maCountry );
			return;
		}
		pEntry++;
	}
	while ( pEntry->meLang != LANGUAGE_DONTKNOW );

	// Search for enum, if we don't found a specific ISO combination.
	// All entries in this tables are allowed for mime specifications,
	// but not defined ISO combinations
	const IsoLangNoneStdEntry* pNoneStdEntry = aImplIsoNoneStdLangEntries;
	do
	{
		if ( pNoneStdEntry->meLang == eLang )
		{
			rLangStr.Assign( pNoneStdEntry->maLangStr );
			rCountry.Assign( pNoneStdEntry->maCountry );
			return;
		}
		pNoneStdEntry++;
	}
	while ( pNoneStdEntry->meLang != LANGUAGE_DONTKNOW );

	// not found
	rLangStr.Erase();
	rCountry.Erase();
}

// -----------------------------------------------------------------------

UniString ConvertLanguageToIsoString( LanguageType eLang, sal_Unicode cSep )
{
	UniString	aLangStr;
	UniString	aCountry;
	ConvertLanguageToIsoNames( eLang, aLangStr, aCountry );
	if ( aCountry.Len() )
	{
		aLangStr.Append( cSep );
		aLangStr.Append( aCountry );
	}
	return aLangStr;
}

// -----------------------------------------------------------------------

ByteString ConvertLanguageToIsoByteString( LanguageType eLang, sal_Char cSep )
{
	ByteString	aLangStr;
	ByteString	aCountry;
	ConvertLanguageToIsoNames( eLang, aLangStr, aCountry );
	if ( aCountry.Len() )
	{
		aLangStr.Append( cSep );
		aLangStr.Append( aCountry );
	}
	return aLangStr;
}

// =======================================================================

LanguageType ConvertIsoNamesToLanguage( const UniString& rLang, const UniString& rCountry )
{
	UniString aLowerLang = rLang;
	UniString aUpperCountry = rCountry;
	// language is lower case in table
	aLowerLang.ToLowerAscii();
	// country is upper case in table
	aUpperCountry.ToUpperAscii();

	//	first look for exact match
	const IsoLangEntry* pFirstLang = NULL;
	const IsoLangEntry* pEntry = aImplIsoLangEntries;
	do
	{
		if ( aLowerLang.EqualsAscii( pEntry->maLangStr ) )
		{
			if ( !aUpperCountry.Len() ||
				 aUpperCountry.EqualsAscii( pEntry->maCountry ) )
				return pEntry->meLang;
			if ( !pFirstLang )
				pFirstLang = pEntry;
		}
		pEntry++;
	}
	while ( pEntry->meLang != LANGUAGE_DONTKNOW );

	// some eng countries should be mapped to a specific english language
	if ( aLowerLang.EqualsAscii( "en" ) )
	{
		const IsoLangEngEntry* pEngEntry = aImplIsoLangEngEntries;
		do
		{
			if ( aUpperCountry.EqualsAscii( pEngEntry->maCountry ) )
				return pEngEntry->meLang;
			pEngEntry++;
		}
		while ( pEngEntry->meLang != LANGUAGE_DONTKNOW );
	}

	// test for specific languages which are not used standard ISO 3166 codes
	const IsoLangNoneStdEntry* pNoneStdEntry = aImplIsoNoneStdLangEntries;
	do
	{
		if ( aLowerLang.EqualsAscii( pNoneStdEntry->maLangStr ) )
		{
			// The countries in this table are not all in upper case
			if ( aUpperCountry.EqualsIgnoreCaseAscii( pNoneStdEntry->maCountry ) )
				return pNoneStdEntry->meLang;
		}
		pNoneStdEntry++;
	}
	while ( pNoneStdEntry->meLang != LANGUAGE_DONTKNOW );
	pNoneStdEntry = aImplIsoNoneStdLangEntries2;
	do
	{
		if ( aLowerLang.EqualsAscii( pNoneStdEntry->maLangStr ) )
		{
			// The countries in this table are not all in upper case
			if ( aUpperCountry.EqualsIgnoreCaseAscii( pNoneStdEntry->maCountry ) )
				return pNoneStdEntry->meLang;
		}
		pNoneStdEntry++;
	}
	while ( pNoneStdEntry->meLang != LANGUAGE_DONTKNOW );

	// If the language is correct, than we return the default language
	if ( pFirstLang )
		return pFirstLang->meLang;

	//	if only the country is set, look for any entry matching the country
	//	(to allow reading country and language in separate steps, in any order)
	if ( rCountry.Len() && !rLang.Len() )
	{
		const IsoLangEntry* pEntry = aImplIsoLangEntries;
		do
		{
			if ( aUpperCountry.EqualsAscii( pEntry->maCountry ) )
				return pEntry->meLang;
			pEntry++;
		}
		while ( pEntry->meLang != LANGUAGE_DONTKNOW );

		aLowerLang = aUpperCountry;
		aLowerLang.ToLowerAscii();
	}

	// Now look for all other definitions, which are not standard
	const IsoLangOtherEntry* pOtherEntry = aImplOtherEntries;
	do
	{
		if ( aLowerLang.EqualsAscii( pOtherEntry->mpLangStr ) )
			return pOtherEntry->meLang;
		pOtherEntry++;
	}
	while ( pOtherEntry->meLang != LANGUAGE_DONTKNOW );

	return LANGUAGE_DONTKNOW;
}

// -----------------------------------------------------------------------

LanguageType ConvertIsoNamesToLanguage( const ByteString& rLang, const ByteString& rCountry )
{
	ByteString aLowerLang = rLang;
	ByteString aUpperCountry = rCountry;
	// language is lower case in table
	aLowerLang.ToLowerAscii();
	// country is upper case in table
	aUpperCountry.ToUpperAscii();

	//	first look for exact match
	const IsoLangEntry* pFirstLang = NULL;
	const IsoLangEntry* pEntry = aImplIsoLangEntries;
	do
	{
		if ( aLowerLang.Equals( pEntry->maLangStr ) )
		{
			if ( !aUpperCountry.Len() ||
				 aUpperCountry.Equals( pEntry->maCountry ) )
				return pEntry->meLang;
			if ( !pFirstLang )
				pFirstLang = pEntry;
		}
		pEntry++;
	}
	while ( pEntry->meLang != LANGUAGE_DONTKNOW );

	// some eng countries should be mapped to a specific english language
	if ( aLowerLang.Equals( "en" ) )
	{
		const IsoLangEngEntry* pEngEntry = aImplIsoLangEngEntries;
		do
		{
			if ( aUpperCountry.Equals( pEngEntry->maCountry ) )
				return pEngEntry->meLang;
			pEngEntry++;
		}
		while ( pEngEntry->meLang != LANGUAGE_DONTKNOW );
	}

	// test for specific languages which are not used standard ISO 3166 codes
	const IsoLangNoneStdEntry* pNoneStdEntry = aImplIsoNoneStdLangEntries;
	do
	{
		if ( aLowerLang.Equals( pNoneStdEntry->maLangStr ) )
		{
			// The countries in this table are not all in upper case
			if ( aUpperCountry.EqualsIgnoreCaseAscii( pNoneStdEntry->maCountry ) )
				return pNoneStdEntry->meLang;
		}
		pNoneStdEntry++;
	}
	while ( pNoneStdEntry->meLang != LANGUAGE_DONTKNOW );
	pNoneStdEntry = aImplIsoNoneStdLangEntries2;
	do
	{
		if ( aLowerLang.Equals( pNoneStdEntry->maLangStr ) )
		{
			// The countries in this table are not all in upper case
			if ( aUpperCountry.EqualsIgnoreCaseAscii( pNoneStdEntry->maCountry ) )
				return pNoneStdEntry->meLang;
		}
		pNoneStdEntry++;
	}
	while ( pNoneStdEntry->meLang != LANGUAGE_DONTKNOW );

	// If the language is correct, than we return the default language
	if ( pFirstLang )
		return pFirstLang->meLang;

	//	if only the country is set, look for any entry matching the country
	//	(to allow reading country and language in separate steps, in any order)
	if ( rCountry.Len() && !rLang.Len() )
	{
		const IsoLangEntry* pEntry = aImplIsoLangEntries;
		do
		{
			if ( aUpperCountry.Equals( pEntry->maCountry ) )
				return pEntry->meLang;
			pEntry++;
		}
		while ( pEntry->meLang != LANGUAGE_DONTKNOW );

		aLowerLang = aUpperCountry;
		aLowerLang.ToLowerAscii();
	}

	// Now look for all other definitions, which are not standard
	const IsoLangOtherEntry* pOtherEntry = aImplOtherEntries;
	do
	{
		if ( aLowerLang.Equals( pOtherEntry->mpLangStr ) )
			return pOtherEntry->meLang;
		pOtherEntry++;
	}
	while ( pOtherEntry->meLang != LANGUAGE_DONTKNOW );

	return LANGUAGE_DONTKNOW;
}

// -----------------------------------------------------------------------

LanguageType ConvertIsoStringToLanguage( const UniString& rString, sal_Unicode cSep )
{
	UniString	aLang;
	UniString	aCountry;
	xub_StrLen	nSepPos = rString.Search( cSep );
	if ( nSepPos != STRING_NOTFOUND )
	{
		aLang = rString.Copy( 0, nSepPos );
		aCountry = rString.Copy( nSepPos+1 );
	}
	else
		aLang = rString;

	return ConvertIsoNamesToLanguage( aLang, aCountry );
}

// -----------------------------------------------------------------------

LanguageType ConvertIsoByteStringToLanguage( const ByteString& rString, sal_Char cSep )
{
	ByteString	aLang;
	ByteString	aCountry;
	xub_StrLen	nSepPos = rString.Search( cSep );
	if ( nSepPos != STRING_NOTFOUND )
	{
		aLang = rString.Copy( 0, nSepPos );
		aCountry = rString.Copy( nSepPos+1 );
	}
	else
		aLang = rString;

	return ConvertIsoNamesToLanguage( aLang, aCountry );
}
