/*************************************************************************
 *
 *  $RCSfile: memmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:08 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#if defined(_MSC_VER) && _MSC_VER >= 1200
#include <new>
#endif

#include <string.h>

#ifdef DBG_UTIL
#include <stdio.h>
#endif

// =======================================================================

#if defined( DOS ) || defined( SVX_LIGHT ) || defined(BOOTSTRAP)

#include <solar.h>

// kein Memory-Manager fuer DOS, da wir dort den vom Compiler benutzen
void* SvMemAlloc( ULONG nBytes, USHORT )
{
	return new BYTE[(size_t)nBytes];
}

void SvMemFree( void* p )
{
	delete p;
}

// =======================================================================

#else

#if defined( WNT )
#include "memwnt.cxx"
#elif defined( WIN )
#include "memwin.cxx"
#elif defined( OS2 )
#include "memos2.cxx"
#elif defined( MAC )
#include "memmac.cxx"
#elif defined( UNX )
#include "memunx.cxx"
#endif

// =======================================================================

#include <solar.h>
#include <new.hxx>

// =======================================================================

void EnterToolsMultiThread( int bEnter )
{
#if defined( WNT ) || defined( OS2 )
	EnterMultiThread( bEnter );
#endif
}

// -----------------------------------------------------------------------

int IsToolsMultiThread()
{
#if defined( WNT ) || defined( OS2 )
	return IsMultiThread();
#else
	return TRUE;
#endif
}

// =======================================================================

#define MEMBLOCK_LAST			0xFFFF
#define MEMBLOCK_SYSBLOCK		0xFFFF
#define MEMBLOCK_NOTFREE		0x8000

// =======================================================================

#if ( __ALIGNMENT8 > 4 )
#define MEMMGR_ALIGNMENT		__ALIGNMENT8
#define MEMBLOCK_OFFSET 		((sizeof( MemBlock )+7) & ~7)
#define MEMBLOCK_MINSIZE		(((sizeof( FreeBlock )+15) & ~15)-MEMBLOCK_OFFSET)
#define MEMBLOCK_MAXSIZEOFF 	64
#else
#define MEMMGR_ALIGNMENT		4
#define MEMBLOCK_OFFSET 		sizeof( MemBlock )
#define MEMBLOCK_MINSIZE		(sizeof( FreeBlock	)-MEMBLOCK_OFFSET)
#define MEMBLOCK_MAXSIZEOFF 	64
#endif

#define MEM_FREELIST_GENERAL	(MEM_FREELIST_COUNT-1)
#define MEM_MAXFREE_SIZE		(((MEM_FREELIST_COUNT-1)*MEMMGR_ALIGNMENT)+8)
#define MEM_MAXFREE_CALCOFF(n)	(((n-1)/MEMMGR_ALIGNMENT)-(8/MEMMGR_ALIGNMENT))

// =======================================================================

#pragma pack( 1 )

// ! MemBlock muss in gleicher Form am Anfang von FreeBlock stehen !
struct MemBlock
{
	USHORT		nSize;		// Groesse des Blocks
	USHORT		nPrev;		// Offset zum vorherigen Block
};

struct FreeBlock
{
	USHORT		nSize;		// Groesse des Blocks
	USHORT		nPrev;		// Offset zum vorherigen Block
	FreeBlock*	pNext;		// Pointer auf naechsten freien Block
	FreeBlock*	pPrev;		// Pointer auf vorherigen freien Block
};

#pragma pack()

// =======================================================================

#include "dbmem.cxx"

// =======================================================================

_NEW_HANDLER SetNewHandler( _NEW_HANDLER pHandler )
{
	MemData* pMemData = GetMemData();

	_NEW_HANDLER pOldHandler = (_NEW_HANDLER)pMemData->pNewHandler;
	pMemData->pNewHandler = (void*)pHandler;
	return pOldHandler;
}

// -----------------------------------------------------------------------

inline void ImpAddFreeBlock( MemData* pMemData, FreeBlock* pBlock, int bSet = FALSE )
{
	DBG_MEM_ADDFREEBLOCK( pBlock->nSize );

	if ( pBlock->nSize > MEM_MAXFREE_SIZE )
	{
		FreeBlock** ppFirstBlock = pMemData->pFirstBlocks+MEM_FREELIST_GENERAL;
		FreeBlock*	pFirstBlock = *ppFirstBlock;
		if ( pFirstBlock )
		{
			pBlock->pNext		 = pFirstBlock;
			pBlock->pPrev		 = pFirstBlock->pPrev;
			pFirstBlock->pPrev	 = pBlock;
			pBlock->pPrev->pNext = pBlock;
			if ( bSet )
				*ppFirstBlock = pBlock;
		}
		else
		{
			pBlock->pNext = pBlock;
			pBlock->pPrev = pBlock;
			*ppFirstBlock = pBlock;
		}
	}
	else
	{
		FreeBlock** ppFirstBlock = pMemData->pFirstBlocks+MEM_MAXFREE_CALCOFF(pBlock->nSize);
		pBlock->pNext = *ppFirstBlock;
		pBlock->pPrev = NULL;
		*ppFirstBlock = pBlock;
		if ( pBlock->pNext )
			pBlock->pNext->pPrev = pBlock;
	}
}

// -----------------------------------------------------------------------

inline void ImpRemoveFreeBlock( MemData* pMemData, FreeBlock* pBlock )
{
	DBG_MEM_REMOVEFREEBLOCK( pBlock->nSize );

	if ( pBlock->nSize > MEM_MAXFREE_SIZE )
	{
		FreeBlock** ppFirstBlock = pMemData->pFirstBlocks+MEM_FREELIST_GENERAL;
		// Sind wir der einzige Block
		if ( pBlock->pNext == pBlock )
			*ppFirstBlock = NULL;
		else
		{
			if ( *ppFirstBlock == pBlock )
				*ppFirstBlock = pBlock->pNext;
			pBlock->pNext->pPrev = pBlock->pPrev;
			pBlock->pPrev->pNext = pBlock->pNext;
			pBlock->pPrev = NULL;
			pBlock->pNext = NULL;
		}
	}
	else
	{
		if ( pBlock->pPrev )
		{
			pBlock->pPrev->pNext = pBlock->pNext;
			if ( pBlock->pNext )
				pBlock->pNext->pPrev = pBlock->pPrev;
		}
		else
		{
			FreeBlock** ppFirstBlock = pMemData->pFirstBlocks+MEM_MAXFREE_CALCOFF(pBlock->nSize);
			*ppFirstBlock = pBlock->pNext;
			if ( *ppFirstBlock )
				(*ppFirstBlock)->pPrev = NULL;
		}
	}
}

// -----------------------------------------------------------------------

static void* MemAlloc( ULONG nAlloc )
{
	// Ueberschreitet Speicheranforderung die Blockgroesse, dann wird
	// ein globaler Block geholt, der nicht in die Speicherverwaltung
	// aufgenommen wird
	if ( nAlloc >= (MEMBLOCK_MAXSIZE-MEMBLOCK_MAXSIZEOFF) )
	{
#ifdef WIN
		// Bei Bloecken kleiner 64 KB geben wir 4 Bytes dazu, da einige
		// Compiler Probleme bei Segementgrenzen haben
		if ( nAlloc < 0xFFFF-4 )
			nAlloc += 4;

		void* p = DBG_MEM_SYSALLOC( nAlloc );
		DBG_MEM_INIT( p, nAlloc, FALSE );
		return p;
#else
		// Speicher anlegen
		void* p = DBG_MEM_SYSALLOC( nAlloc + MEMMGR_ALIGNMENT );

		// Pointer als Systemblock markieren
		if ( p )
		{
			p = (void*)(((BYTE*)p) + MEMMGR_ALIGNMENT);
			void* pTemp = ((BYTE*)p) - sizeof( USHORT );
			*((USHORT*)pTemp) = MEMBLOCK_SYSBLOCK;
			DBG_MEM_INIT( p, nAlloc, FALSE );
		}

		return p;
#endif
	}
	else
	{
		// n muss >= MEMBLOCK_MINSIZE sein und muss auf 4-Byte Grenze
		// gerundet sein
		size_t n = (size_t)nAlloc;
		if ( n <= MEMBLOCK_MINSIZE )
			n = MEMBLOCK_MINSIZE+MEMBLOCK_OFFSET;
		else
		{
			// Groesse des Memory-Blocks dazuzaehlen
			n += MEMBLOCK_OFFSET;
			n = (n + (MEMMGR_ALIGNMENT-1)) & ~(MEMMGR_ALIGNMENT-1);
		}

		// Suche freien Block
		MemData*	pMemData = GetMemData();
		FreeBlock*	p;
		FreeBlock*	pBlock;

		MEM_SEMSTART();

		if ( n > MEM_MAXFREE_SIZE )
			pBlock = NULL;
		else
		{
			// Freien Block in spezieller Liste suchen
			pBlock = *(pMemData->pFirstBlocks+MEM_MAXFREE_CALCOFF(n));
		}

		if ( !pBlock )
		{
			// Freien Block suchen
			FreeBlock* pStartBlock = *(pMemData->pFirstBlocks+MEM_FREELIST_GENERAL);
			FreeBlock* pSearchBlock = pStartBlock;
			if ( pSearchBlock )
			{
				do
				{
					if ( pSearchBlock->nSize >= n )
					{
						pBlock = pSearchBlock;
						break;
					}

					DBG_MEM_SEARCHFREEBLOCK();
					pSearchBlock = pSearchBlock->pNext;
				}
				while ( pSearchBlock != pStartBlock );
			}
		}

		// Passender Block vorhanden
		if ( pBlock )
		{
			p = pBlock;

			// Block aus der Liste entfernen
			ImpRemoveFreeBlock( pMemData, pBlock );

			// Es darf kein Block nachbleiben, der kleiner
			// als sizeof( FreeBlock ) ist
			if ( (pBlock->nSize-n) < sizeof( FreeBlock ) )
				p->nSize |= MEMBLOCK_NOTFREE;
			else
			{
				// Daten vom freien Block anpassen
				pBlock = (FreeBlock*)(((BYTE*)p)+n);
				pBlock->nSize = p->nSize-n;
				pBlock->nPrev = n;
				((MemBlock*)(((BYTE*)pBlock)+pBlock->nSize))->nPrev = pBlock->nSize;

				// Block in neue Liste eintragen
				ImpAddFreeBlock( pMemData, pBlock, TRUE );

				// Daten setzen
				p->nSize = n | MEMBLOCK_NOTFREE;
			}
		}
		else
		{
			// Neuen Block anlegen
			pBlock = (FreeBlock*)DBG_MEM_SYSALLOC( MEMBLOCK_SIZE );

			if ( !pBlock )
			{
				MEM_SEMEND();
				return NULL;
			}

			DBG_MEM_INIT( pBlock, MEMBLOCK_SIZE, TRUE );

			// Daten setzen
			p = pBlock;
			p->nPrev = 0;
			p->nSize = n | MEMBLOCK_NOTFREE;

			// Den letzten Block kennzeichnen
			((MemBlock*)(((BYTE*)pBlock)+MEMBLOCK_SIZE-MEMBLOCK_OFFSET))->nSize = MEMBLOCK_LAST;

			// Den Rest in die Freispeicherliste aufnehmen
			pBlock			= (FreeBlock*)(((BYTE*)pBlock)+n);
			pBlock->nSize	= MEMBLOCK_SIZE-MEMBLOCK_OFFSET-n;
			pBlock->nPrev	= n;
			ImpAddFreeBlock( pMemData, pBlock );
		}

		MEM_SEMEND();

		DBG_MEM_INIT( ((void*)(((BYTE*)p) + MEMBLOCK_OFFSET)), nAlloc, FALSE );

		return ((void*)(((BYTE*)p) + MEMBLOCK_OFFSET));
	}
}

// -----------------------------------------------------------------------

#if defined( WNT ) && defined( MSC )
#if _MSC_VER >= 1200
void* __cdecl operator new( size_t nBytes ) _THROW1(std::bad_alloc)
#else
void* _cdecl operator new( size_t nBytes )
#endif
#else
void* operator new( size_t nBytes )
#endif
{
#ifdef FORCE_SYSALLOC
	return AllocMemory( nBytes );
#endif

#ifdef DBG_UTIL
	DbgData* pData = DbgGetData();
	if ( pData && (pData->nTestFlags & DBG_TEST_MEM_SYSALLOC) )
		return AllocMemory( nBytes );
#endif

#ifdef MAC
	BOOL bOldVal = GetMemData()->bCallNewHdl;
	GetMemData()->bCallNewHdl = FALSE;
#endif

	void* p = DBG_MEM_ALLOC( nBytes );

	if ( !p )
	{
		while ( 1 )
		{
			// kein Speicher, dann Handler rufen
			_NEW_HANDLER pNewHandler = (_NEW_HANDLER)GetMemData()->pNewHandler;
			if ( !pNewHandler )
				break;
			(*pNewHandler)();

			p = DBG_MEM_ALLOC( nBytes );

			if ( p )
				break;
		}
	}

#ifdef MAC
	GetMemData()->bCallNewHdl = bOldVal;
#endif

	return p;
}

// -----------------------------------------------------------------------

#if defined( WIN ) && defined( __BORLANDC__ )
void* operator new( unsigned long nBytes )
{
	return SvMemAlloc( nBytes );
}
#endif

#if defined( MAC ) && defined(MWC13)
void *operator new[](unsigned long nBytes)
{
	return SvMemAlloc( nBytes, 0 );
}
#endif

#ifdef __MWERKS__
void* operator new( size_t n ,void* p )
{
	return p;
}
#endif

// -----------------------------------------------------------------------

void* SvMemAlloc( ULONG nBytes, USHORT nFlags )
{
#ifdef FORCE_SYSALLOC
	return AllocMemory( nBytes );
#endif
#ifdef DBG_UTIL
	DbgData* pData = DbgGetData();
	if ( pData && (pData->nTestFlags & DBG_TEST_MEM_SYSALLOC) )
		return AllocMemory( nBytes );
#endif

#ifdef MAC
	BOOL bOldVal = GetMemData()->bCallNewHdl;
	GetMemData()->bCallNewHdl = FALSE;
#endif

	void* p = DBG_MEM_ALLOC( nBytes );

	if ( !p && !(nFlags & MEM_NOCALLNEWHDL) )
	{
		while ( 1 )
		{
			// kein Speicher, dann Handler rufen
			_NEW_HANDLER pNewHandler = (_NEW_HANDLER)GetMemData()->pNewHandler;
			if ( !pNewHandler )
				break;
			(*pNewHandler)();

			p = DBG_MEM_ALLOC( nBytes );

			if ( p )
				break;
		}
	}

#ifdef MAC
	GetMemData()->bCallNewHdl = bOldVal;
#endif

	return p;
}

// -----------------------------------------------------------------------

#if defined( WNT ) && defined( MSC )
#if _MSC_VER >= 1200
void __cdecl operator delete( void* p ) _THROW0()
#else
void _cdecl operator delete( void* p )
#endif
#else
void operator delete( void* p )
#endif
{
#ifdef MAC
	if ( GetMemData()->bOutOfMain )
		return;
#endif

	if ( !p )
		return;

#ifdef FORCE_SYSALLOC
	FreeMemory( p );
	return;
#endif

#ifdef DBG_UTIL
	DbgData* pData = DbgGetData();
	if ( pData && (pData->nTestFlags & DBG_TEST_MEM_SYSALLOC) )
	{
		FreeMemory( p );
		return;
	}
#endif

	DBG_MEM_FREE( p );

#ifdef WIN
	if ( !OFF( p ) )
	{
		DBG_MEM_SYSFREE( p );
		return;
	}

	// Pointer anpassen
	p = (void*)(((BYTE*)p) - sizeof( USHORT ));
#else
	// Pointer anpassen
	p = (void*)(((BYTE*)p) - sizeof( USHORT ));

	// Wurde Speicher direkt angefordert
	if ( *((USHORT*)p) == MEMBLOCK_SYSBLOCK )
	{
		p = (void*)(((BYTE*)p) - (MEMMGR_ALIGNMENT - sizeof( USHORT )));
		DBG_MEM_SYSFREE( p );
		return;
	}
#endif

	// Block freigeben
	p = (void*)(((BYTE*)p) - MEMBLOCK_OFFSET + sizeof( USHORT ));
	MemData*	pMemData = GetMemData();
	FreeBlock*	pFreeBlock = ((FreeBlock*)p);

	MEM_SEMSTART();

	// Block ist nicht mehr belegt
	pFreeBlock->nSize &= ~MEMBLOCK_NOTFREE;

	// Bloecke zusammenfassen
	if ( pFreeBlock->nPrev )
	{
		FreeBlock* pPrev = (FreeBlock*)(((BYTE*)pFreeBlock)-pFreeBlock->nPrev);
		// Ist Block davor frei
		if ( !(pPrev->nSize & MEMBLOCK_NOTFREE) )
		{
			// Block aus der Liste entfernen
			ImpRemoveFreeBlock( pMemData, pPrev );

			// Block zusammenfassen
			pPrev->nSize += pFreeBlock->nSize;
			pFreeBlock	  = pPrev;
			((MemBlock*)(((BYTE*)pFreeBlock)+pFreeBlock->nSize))->nPrev = pFreeBlock->nSize;
		}
	}

	FreeBlock* pNext = (FreeBlock*)(((BYTE*)pFreeBlock)+pFreeBlock->nSize);
	// Ist Block nicht der letzte und der nachfolgende frei
	if ( (pNext->nSize != MEMBLOCK_LAST) && !(pNext->nSize & MEMBLOCK_NOTFREE) )
	{
		// Block aus der Liste entfernen
		ImpRemoveFreeBlock( pMemData, pNext );

		// Block zusammenfassen
		pFreeBlock->nSize += pNext->nSize;
		((MemBlock*)(((BYTE*)pFreeBlock)+pFreeBlock->nSize))->nPrev = pFreeBlock->nSize;
	}

	if ( pFreeBlock->nSize >= (MEMBLOCK_SIZE-MEMBLOCK_OFFSET) )
	{
		DBG_MEM_INIT( pFreeBlock, pFreeBlock->nSize, TRUE );
		DBG_MEM_SYSFREE( pFreeBlock );
	}
	else
	{
		DBG_MEM_INIT( ((BYTE*)pFreeBlock)+sizeof(FreeBlock),
					  pFreeBlock->nSize-sizeof(FreeBlock), TRUE );
		// Block in neue Liste eintragen
		ImpAddFreeBlock( pMemData, pFreeBlock );
	}

	MEM_SEMEND();
}

// -----------------------------------------------------------------------

#if defined( MAC ) && defined(MWC13)
void operator delete[](void *p)
{
	SvMemFree( p );
}
#endif

// -----------------------------------------------------------------------

void SvMemFree( void* p )
{
	// Wegen Purify in der Debug-Version nicht delete aufrufen, da wir
	// ansonsten zuviele Fehlermeldungen bekommen
#ifdef DBG_UTIL
#ifdef MAC
	if ( GetMemData()->bOutOfMain )
		return;
#endif
	if ( !p )
		return;

	DbgData* pData = DbgGetData();
	if ( pData && (pData->nTestFlags & DBG_TEST_MEM_SYSALLOC) )
	{
		FreeMemory( p );
		return;
	}
#endif

#ifdef FORCE_SYSALLOC
	FreeMemory( p );
	return;
#endif

	delete p;
}

#endif

