/*************************************************************************
 *
 *  $RCSfile: strmwin.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string.h>
#include <tools.h>
#include <svwin.h>
#include <limits.h>
#include <io.h>
#include <sys\locking.h>
#include <errno.h>

#include <dll.hxx>
#include <svwin.h>
#ifndef SEG
#define  SEG(fp)   (*((size_t*)&(fp) + 1))
#endif

#include <debug.hxx>
#include <fsys.hxx>
#include <stream.hxx>

#pragma optimize("",off)

// -----------------------------------------------------------------------

// --------------
// - StreamData -
// --------------

class StreamData
{
public:
    HFILE       hFile;
    OFSTRUCT    ofWinFileBuf;
    BOOL        bIsEof;

		StreamData()
		{
		    hFile = 0;
		    bIsEof = FALSE;
		}
};

// -----------------------------------------------------------------------

// Constants for _llseek
#define WIN_SEEK_BEG 0
#define WIN_SEEK_REL 1
#define WIN_SEEK_END 2

#define OPEN_ERROR_COUNT    13

// Windows gibt nur beim Oeffnen vernuenftige Fehlercodes zurueck

ULONG GetSvOpenError( ULONG nWinError )
{
    static struct { USHORT win; ULONG sv; } errArr[ OPEN_ERROR_COUNT ] =
    {
	{ 0x0002, SVSTREAM_FILE_NOT_FOUND },
	{ 0x0003, SVSTREAM_PATH_NOT_FOUND },
	{ 0x0004, SVSTREAM_TOO_MANY_OPEN_FILES },
	{ 0x0005, SVSTREAM_ACCESS_DENIED },
	{ 0x0006, SVSTREAM_INVALID_HANDLE },
	{ 0x0013, SVSTREAM_ACCESS_DENIED },        // Write protect error
	{ 0x0020, SVSTREAM_SHARING_VIOLATION },
	{ 0x0021, SVSTREAM_LOCKING_VIOLATION },
	{ 0x0024, SVSTREAM_SHARE_BUFF_EXCEEDED },
	{ 0x0041, SVSTREAM_ACCESS_DENIED },        // Network access denied
	{ 0x0052, SVSTREAM_CANNOT_MAKE },
	{ 0x0056, SVSTREAM_ACCESS_DENIED },        // Invalid password
	{ 0x0057, SVSTREAM_INVALID_PARAMETER }
    };

    ULONG nRetVal = SVSTREAM_GENERALERROR;    // Standardfehler
    for( USHORT n=0; n< OPEN_ERROR_COUNT; n++ )
    if ( errArr[n].win == nWinError )
    {
	nRetVal = errArr[n].sv;
	break;
    }
    return nRetVal;
}

/*************************************************************************
|*
|*    SvFileStream::SvFileStream()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 14.06.94
|*    Letzte Aenderung  OV 14.06.94
|*
*************************************************************************/

SvFileStream::SvFileStream( const String& rFileName,StreamMode nOpenMode )
{
    bIsOpen             = FALSE;
    nLockCounter        = 0;
    bIsWritable         = FALSE;
    pInstanceData       = new StreamData;
    SetBufferSize( 1024 );
    Open( rFileName, nOpenMode );
}

/*************************************************************************
|*
|*    SvFileStream::SvFileStream()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 14.06.94
|*    Letzte Aenderung  OV 14.06.94
|*
*************************************************************************/

SvFileStream::SvFileStream()
{
    bIsOpen             = FALSE;
    nLockCounter        = 0;
    bIsWritable         = FALSE;
    pInstanceData       = new StreamData;
    SetBufferSize( 1024 );
}

/*************************************************************************
|*
|*    SvFileStream::~SvFileStream()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 14.06.94
|*    Letzte Aenderung  OV 14.06.94
|*
*************************************************************************/

SvFileStream::~SvFileStream()
{
    Close();
    if ( pInstanceData )
	delete pInstanceData;
}

/*************************************************************************
|*
|*    SvFileStream::GetFileHandle()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 14.06.94
|*    Letzte Aenderung  OV 14.06.94
|*
*************************************************************************/

ULONG SvFileStream::GetFileHandle() const
{
    return (ULONG)pInstanceData->hFile;
}

/*************************************************************************
|*
|*    SvFileStream::IsA()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 14.06.94
|*    Letzte Aenderung  OV 14.06.94
|*
*************************************************************************/

USHORT SvFileStream::IsA() const
{
    return ID_FILESTREAM;
}

/*************************************************************************
|*
|*    SvFileStream::GetData()
|*
|*    Beschreibung      STREAM.SDW, Prueft nicht Eof; IsEof danach rufbar
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

ULONG SvFileStream::GetData( char* pData, ULONG nSize )
{
#ifdef DBG_UTIL
    String aTraceStr( "SvFileStream::GetData(): " );
    aTraceStr += nSize;
    aTraceStr += " Bytes from ";
    aTraceStr += aFilename;
    DBG_TRACE( aTraceStr );
#endif

    ULONG nCount = 0L;
    if ( IsOpen() )
    {
	nCount = _hread( pInstanceData->hFile, (void*)pData, nSize );
	if ( nCount == -1L )
	{
	    SetError( SVSTREAM_READ_ERROR );
	    nCount = 0L;
	}
    }
    return nCount;
}

/*************************************************************************
|*
|*    SvFileStream::PutData()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

ULONG SvFileStream::PutData( const char* pData, ULONG nSize )
{
#ifdef DBG_UTIL
    String aTraceStr( "SvFileStrean::PutData: " );
    aTraceStr += nSize;
    aTraceStr += " Bytes to ";
    aTraceStr += aFilename;
    DBG_TRACE( aTraceStr );
#endif

    ULONG nCount = 0L;
    if ( IsOpen() )
    {
	nCount = _hwrite( pInstanceData->hFile, (void*)pData, nSize );
	if ( nCount == -1L )
	{
	    SetError( SVSTREAM_WRITE_ERROR );
	    nCount = 0L;
	}
	else if( !nCount )
	    SetError( SVSTREAM_DISK_FULL );
	else if( nCount < nSize )
	    SetError( SVSTREAM_WRITE_ERROR );

    }
    return nCount;
}

/*************************************************************************
|*
|*    SvFileStream::SeekPos()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

ULONG SvFileStream::SeekPos( ULONG nPos )
{
    ULONG nRetVal = 0L;

    if ( IsOpen() )
    {
	if ( nPos != STREAM_SEEK_TO_END )
	    nRetVal = _llseek(pInstanceData->hFile, nPos, WIN_SEEK_BEG );
	else
	    nRetVal = _llseek(pInstanceData->hFile, 0L, WIN_SEEK_END );

	if ( nRetVal == HFILE_ERROR )
	    SetError( SVSTREAM_SEEK_ERROR );
    }
    else
	SetError( SVSTREAM_GENERALERROR );

    return nRetVal;
}

/*************************************************************************
|*
|*    SvFileStream::FlushData()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

void SvFileStream::FlushData()
{
    // keine Win-Fkt.
}

/*************************************************************************
|*
|*    SvFileStream::LockRange()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

BOOL SvFileStream::LockRange( ULONG nByteOffset, ULONG nBytes )
{
    BOOL bRetVal = FALSE;
    if ( IsOpen() )
    {
	ULONG nOldPos = Tell();
	Seek( nByteOffset );
	int i = locking( pInstanceData->hFile, LK_LOCK, nBytes );
	if ( i == -1 )
	{
	    switch ( errno )
	    {
		case EACCES:
		case EDEADLOCK:
		    SetError( SVSTREAM_LOCKING_VIOLATION );
		    break;
		case EBADF:
		    SetError( SVSTREAM_INVALID_HANDLE );
		    break;
		case EINVAL: // SHARE.EXE not loaded or invalid param.
		    SetError( SVSTREAM_INVALID_PARAMETER );
		    break;
		default:
		    SetError( SVSTREAM_GENERALERROR );
	    }
	}
	else
	    bRetVal = TRUE;

	Seek( nOldPos );
    }
    return bRetVal;
}

/*************************************************************************
|*
|*    SvFileStream::UnlockRange()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

BOOL SvFileStream::UnlockRange( ULONG nByteOffset, ULONG nBytes )
{
    BOOL bRetVal = FALSE;
    if ( IsOpen() )
    {
	ULONG nOldPos = Tell();
	Seek( nByteOffset );
	int i = locking( pInstanceData->hFile, LK_UNLCK, nBytes );
	if ( i == -1 )
	{
	    switch ( errno )
	    {
		case EACCES:
		case EDEADLOCK:
		    SetError( SVSTREAM_LOCKING_VIOLATION );
		    break;
		case EBADF:
		    SetError( SVSTREAM_INVALID_HANDLE );
		    break;
		case EINVAL: // SHARE.EXE not loaded or invalid param.
		    SetError( SVSTREAM_INVALID_PARAMETER );
		    break;
		default:
		    SetError( SVSTREAM_GENERALERROR );
	    }
	}
	else
	    bRetVal = TRUE;

	Seek( nOldPos );
    }
    return bRetVal;
}


/*************************************************************************
|*
|*    SvFileStream::LockFile()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

BOOL SvFileStream::LockFile()
{
    BOOL bRetVal = FALSE;

    if ( !nLockCounter )
    {
	if ( LockRange( 0L, LONG_MAX ) )
	{
	    nLockCounter = 1;
	    bRetVal = TRUE;
	}
    }
    else
    {
	nLockCounter++;
	bRetVal = TRUE;
    }

    return bRetVal;
}

/*************************************************************************
|*
|*    SvFileStream::UnlockFile()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

BOOL SvFileStream::UnlockFile()
{
    BOOL bRetVal = FALSE;
    if ( nLockCounter > 0)
    {
	if ( nLockCounter == 1)
	{
	    if ( UnlockRange( 0L, LONG_MAX ) )
	    {
		nLockCounter = 0;
		bRetVal = TRUE;
	    }
	}
	else
	{
	    nLockCounter--;
	    bRetVal = TRUE;
	}
    }
    return bRetVal;
}


/*************************************************************************
|*
|*    SvFileStream::Open()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

void SvFileStream::Open( const String& rFilename, StreamMode nOpenMode )
{
	String aParsedFilename;
#ifndef WIN
	if	( Folder::IsAvailable() && (rFilename.Search("{") < 9) )
	{
		String		aVirtualPart;
		String		aRealPart;
		String		aVirtualPath;
		ItemIDPath	aVirtualURL;
		ULONG		nDivider = 0;

		String		aVirtualString(rFilename);

		for (int x=aVirtualString.Len(); x>0; x--)
		{
			if (aVirtualString.Copy(x,1).Compare("}")==COMPARE_EQUAL)
			{
				nDivider = x;
				break;
			}
		}

		aVirtualPart = aVirtualString.Copy(0,nDivider+1);
		aRealPart = aVirtualString.Copy(nDivider+2);

		aVirtualURL  = aVirtualPart;
		aVirtualPath = aVirtualURL.GetHostNotationPath();

		DirEntry aTempDirEntry(aVirtualPath);

		aTempDirEntry += aRealPart;

		aParsedFilename = aTempDirEntry.GetFull();
	}
	else
#endif
    {
		aParsedFilename = rFilename;
	}

    HFILE       hFileTmp = 0;
    OFSTRUCT    aFileBufTmp;
    USHORT      nShareFlag;

    Close();
    SvStream::ClearBuffer();
    eStreamMode = nOpenMode;
    eStreamMode &= ~STREAM_TRUNC; // beim ReOpen nicht cutten

//    !!! NoOp: Ansonsten ToAbs() verwendern
//    !!! DirEntry aDirEntry( rFilename );
//    !!! aFilename = aDirEntry.GetFull();
    aFilename = aParsedFilename;
	FSysRedirector::DoRedirect( aFilename );

#ifdef DBG_UTIL
    String aTraceStr( "SvFileStream::Open(): " );
    aTraceStr += aFilename;
    DBG_TRACE( aTraceStr );
#endif

    // Windows soll bei Sharing-Violations keine MsgBox hochziehen
    UINT nOldErrMode = SetErrorMode(SEM_FAILCRITICALERRORS );

    nOpenMode |= STREAM_SHARE_DENYNONE;  // definierten Zustand

    // Prioritaet : deny nothing (niedrigste),deny read,deny write,deny all
    if ( nOpenMode & STREAM_SHARE_DENYNONE)
	nShareFlag = OF_SHARE_DENY_NONE;

    if ( nOpenMode & STREAM_SHARE_DENYREAD)
	nShareFlag = OF_SHARE_DENY_READ;

    if ( nOpenMode & STREAM_SHARE_DENYWRITE)
	nShareFlag = OF_SHARE_DENY_WRITE;

    if ( nOpenMode & STREAM_SHARE_DENYALL)
	nShareFlag = OF_SHARE_EXCLUSIVE;

    BOOL bCreate = (nOpenMode & STREAM_NOCREATE) == 0;

    // Existiert Datei ?
    BOOL bFileExists = TRUE;
    hFileTmp = OpenFile( (const char*)aFilename,&aFileBufTmp,
			 nShareFlag | OF_READ );
    if ( hFileTmp == HFILE_ERROR || aFileBufTmp.nErrCode != 0 )
    {
	bFileExists = FALSE;
	if (  aFileBufTmp.nErrCode != 0x0002 ) // file not found
	{
	    SetError( ::GetSvOpenError( aFileBufTmp.nErrCode ));
	    SetErrorMode(nOldErrMode);
	    return;
	}
    }
    else
	_lclose( hFileTmp );

    if ( !bCreate && !bFileExists )
    {
	// Datei nicht da und wir duerfen keine neue erzeugen
	SetError( SVSTREAM_FILE_NOT_FOUND );
	SetErrorMode(nOldErrMode);
	return;
    }

    if ( bCreate && !bFileExists )
    {
	// Datei nicht da, aber wir duerfen eine neue erzeugen
	hFileTmp = OpenFile( (const char*)aFilename,&aFileBufTmp,
			     nShareFlag | OF_CREATE );
	if ( hFileTmp == HFILE_ERROR || aFileBufTmp.nErrCode != 0 )
	{
	    SetError( ::GetSvOpenError( aFileBufTmp.nErrCode ));
	    SetErrorMode(nOldErrMode);
	    return;
	}
	else
	    _lclose( hFileTmp );
    }

    if ( nOpenMode & STREAM_TRUNC )
    {
	// Datei abschneiden
	hFileTmp = OpenFile( (const char*)aFilename,&aFileBufTmp,
			     nShareFlag | OF_CREATE );
	if ( hFileTmp == HFILE_ERROR || aFileBufTmp.nErrCode != 0  )
	{
	    SetError( ::GetSvOpenError( aFileBufTmp.nErrCode ));
	    SetErrorMode(nOldErrMode);
	    return;
	}
	else
	    _lclose( hFileTmp );
    }

    bIsWritable = FALSE;
    bIsOpen     = TRUE;

    if ( nOpenMode & STREAM_WRITE )
    {
	pInstanceData->hFile = OpenFile( (const char*)aFilename,
	   &pInstanceData->ofWinFileBuf, nShareFlag | OF_READWRITE );
	if ( pInstanceData->hFile != HFILE_ERROR && !aFileBufTmp.nErrCode )
	{
	    _llseek( pInstanceData->hFile, 0L, WIN_SEEK_BEG ); // !!!
	    bIsWritable = TRUE;
	    SetErrorMode(nOldErrMode);
	    return;
	}
    }

    // read only oeffnen
    pInstanceData->hFile = OpenFile((const char*)aFilename,
	   &pInstanceData->ofWinFileBuf, nShareFlag | OF_READ );
    if ( pInstanceData->hFile != HFILE_ERROR && !aFileBufTmp.nErrCode )
	_llseek( pInstanceData->hFile, 0L, WIN_SEEK_BEG );     // !!!
    else
	SetError( ::GetSvOpenError( aFileBufTmp.nErrCode ));

    SetErrorMode(nOldErrMode);
}

/*************************************************************************
|*
|*    SvFileStream::ReOpen()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

void SvFileStream::ReOpen()
{
    if ( !bIsOpen && aFilename.Len() )
	Open( aFilename, eStreamMode );
}

/*************************************************************************
|*
|*    SvFileStream::Close()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

void SvFileStream::Close()
{
    if ( IsOpen() )
    {
#ifdef DBG_UTIL
	String aTraceStr( "SvFileStream::Close(): " );
	aTraceStr += aFilename;
	DBG_TRACE( aTraceStr );
#endif

	if ( nLockCounter )
	{
	    nLockCounter = 1;
	    UnlockFile();
	}
	Flush();
	_lclose( pInstanceData->hFile );
    }

    pInstanceData->hFile    = 0;
    bIsOpen                 = FALSE;
    nLockCounter            = 0;
    bIsWritable             = FALSE;
    SvStream::ClearBuffer();
    SvStream::ClearError();
}

/*************************************************************************
|*
|*    SvFileStream::SetSize()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

void SvFileStream::SetSize( ULONG nSize )
{
    if ( IsOpen() )
    {
	int nRet = chsize( pInstanceData->hFile, (long)nSize );
	if( nRet == -1 )
	{
	    switch ( errno )
	    {
		case EACCES:
		    SetError( SVSTREAM_LOCKING_VIOLATION );
		    break;
		case EBADF:
		    SetError( SVSTREAM_INVALID_HANDLE );
		    break;
		case ENOSPC:
		    SetError( SVSTREAM_DISK_FULL );
		    break;
		default:
		    SetError( SVSTREAM_GENERALERROR );
	    }
	}
    }
}

/*************************************************************************
|*
|*    SvFileStream::ResetError()
|*
|*    Beschreibung      STREAM.SDW; Setzt Filepointer auf Dateianfang
|*    Ersterstellung    OV 15.06.94
|*    Letzte Aenderung  OV 15.06.94
|*
*************************************************************************/

void SvFileStream::ResetError()
{
    SvStream::ClearError();
}

/*************************************************************************
|*
|*    SvSharedMemoryStream::AllocateMemory()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    CL 05.05.95
|*    Letzte Aenderung  CL 05.05.95
|*
*************************************************************************/

BOOL SvSharedMemoryStream::AllocateMemory( ULONG nNewSize )
{
    HGLOBAL hMem = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, (DWORD)nNewSize );
    pBuf = (char*) GlobalLock( hMem );
    aHandle = (void*)hMem;
    return( hMem != 0 );
}

/*************************************************************************
|*
|*    SvSharedMemoryStream::ReAllocateMemory()
|*
|*    Beschreibung      STREAM.SDW (Bozo-Algorithmus)
|*    Ersterstellung    CL 05.05.95
|*    Letzte Aenderung  CL 05.05.95
|*
*************************************************************************/

BOOL SvSharedMemoryStream::ReAllocateMemory( long nDiff )
{
    BOOL bRetVal    = FALSE;
    ULONG nNewSize  = nSize + nDiff;
	if( nNewSize )
	{
		HGLOBAL hMem = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, (DWORD)nNewSize );
	    char* pNewBuf = (char*) GlobalLock( hMem );
	    if ( pNewBuf )
	    {
		bRetVal = TRUE; // Success!
		if ( nNewSize < nSize )      // Verkleinern ?
		{
		    hmemcpy((void _huge*)pNewBuf,(void _huge*)pBuf,(long)nNewSize );
		    if ( nPos > nNewSize )
			nPos = 0L;
		    if ( nEndOfData >= nNewSize )
			nEndOfData = nNewSize-1L;
		}
		else
		    hmemcpy( (void _huge*)pNewBuf, (void _huge*)pBuf, (long)nSize );

		FreeMemory();

		pBuf  = pNewBuf;
		nSize = nNewSize;
		aHandle = (void*)hMem;
	    }
	}
	else
	{
		FreeMemory();
		bRetVal = TRUE;
		pBuf = 0;
		nSize = 0;
		nEndOfData = 0;
		nPos = 0;
		aHandle = 0;
	}
    return bRetVal;
}


/*************************************************************************
|*
|*    SvSharedMemoryStream::FreeMemory()
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    CL 05.05.95
|*    Letzte Aenderung  CL 05.05.95
|*
*************************************************************************/

void SvSharedMemoryStream::FreeMemory()
{
// aus Kompatibilitaetsgruenden nicht den internen Handle verwenden
#ifdef WIN
    HGLOBAL hMem = (HGLOBAL)SEG(pBuf);
#else
    HGLOBAL hMem = GlobalHandle( pBuf );
#endif
    GlobalUnlock(hMem);
    GlobalFree(hMem);
    aHandle = 0;
}

/*************************************************************************
|*
|*    SvSharedMemoryStream::SetHandle
|*
|*    Beschreibung      STREAM.SDW
|*    Ersterstellung    OV 05.10.95
|*    Letzte Aenderung  OV 05.10.95
|*
*************************************************************************/

void* SvSharedMemoryStream::SetHandle( void* aHandle, ULONG nSize,
				       BOOL bOwnsData, ULONG nEOF )
{
    DBG_ERROR("SvSharedMemoryStream::SetHandle not implemented");
    return 0;
}
