/*************************************************************************
 *
 *  $RCSfile: DEMOSTOR.CXX,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRUNTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRUNTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc..
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _DEMOSTOR_CXX "$Revision: 1.1.1.1 $"

#ifndef _SOLAR_H
#include <solar.h>
#endif

#ifndef _TIME_HXX
#include <time.hxx>
#endif

#ifndef _TOOLS_STORFILE_HXX
#include <storfile.hxx>
#endif

#ifndef _TOOLS_STORLCKB_HXX
#include <storlckb.hxx>
#endif

#include <stdio.h>

void EnterMultiThread (int);

#if (defined(WNT) && defined(PROFILE))

extern "C" {
	void StartCAP (void);
	void StopCAP  (void);
	void DumpCAP  (void);
}

#endif /* PROFILE */

/*========================================================================
 *
 * Demostor internals.
 *
 *======================================================================*/
#define _DEMOSTOR_FILENAME        "demostor.dat"
#define _DEMOSTOR_DUMPNAME        "demostor.dmp"

#define _DEMOSTOR_BUFSIZ          512  /* 4096, 1024, 512 */
#define _DEMOSTOR_LOOPS           1000 /* 1000, 2000 */

#define _DEMOSTOR_ITER            1
#define _DEMOSTOR_READ            0
#define _DEMOSTOR_WRITE           0
#define _DEMOSTOR_TRUNC           0
#define _DEMOSTOR_REMOVE          0
#define _DEMOSTOR_DUMP            1
#define _DEMOSTOR_REBUILD         0

/*========================================================================
 *
 * DirectoryTraveller.
 *
 *======================================================================*/
class DirectoryTraveller : public CntStoreDirectory::traveller
{
	typedef CntStoreDirectory::iterator iter;

	CntStoreDirectoryRef m_xCurDir;
	UINT32               m_nLevel;
	UINT32               m_nCount;

public:
	DirectoryTraveller (CntStoreDirectory *pCurDir = NULL, UINT32 nLevel = 0)
		: m_xCurDir (pCurDir), m_nLevel (nLevel), m_nCount (0)
	{}

	virtual int visit (const iter& it);
};

/*
 * visit.
 */
int DirectoryTraveller::visit (const iter& it)
{
	m_nCount++;
	CntStoreDirectory::entry e = *it;

#if (_DEMOSTOR_DUMP == 1)

	printf ("Visit(%d,%d): %s [0x%08x] %d [Bytes]\n",
			m_nLevel, m_nCount,
			(const char *)(e.m_aName), e.m_nAttrib, e.m_nSize);
#endif /* _DUMP */

	if (m_xCurDir.Is() && (e.m_nAttrib & CNTSTORE_ATTRIB_ISDIR))
	{
		CntStoreDirectoryRef xSubDir;
		m_xCurDir->createDirectory (
			xSubDir, "", e.m_aName, STREAM_STD_READ);
		if (xSubDir.Is())
		{
			DirectoryTraveller aSubTraveller (xSubDir, m_nLevel + 1);
			xSubDir->travel (aSubTraveller);
		}
	}

	return 1;
}

/*========================================================================
 *
 * main.
 *
 *======================================================================*/
int main (int argc, char **argv)
{
#ifdef WNT
	EnterMultiThread (1);
#endif /* WNT */

#if (defined(WNT) && defined(PROFILE))
	StartCAP();
#else
	ULONG nStartTicks = Time::GetSystemTicks();
#endif /* PROFILE */

	String aFilename (_DEMOSTOR_FILENAME);
	if (argc > 1)
		aFilename = argv[1];

	// CntStoreFile.
	CntStoreFileRef xFile = new CntStoreFile();

	// Rebuild.
#if (_DEMOSTOR_REBUILD == 1)

	xFile->rebuild (aFilename, "demostor.bak");

#ifndef PROFILE
	printf ("Total(rebuild): %d[ms]\n", Time::GetSystemTicks() - nStartTicks);
#endif /* PROFILE */

#endif /* _REBUILD */

	StreamMode eMode = STREAM_READ;
#if (_DEMOSTOR_WRITE == 1)
	eMode |= STREAM_WRITE;
#else
	eMode |= STREAM_NOCREATE;
#endif /* _WRITE */

	ErrCode eErrCode = xFile->initialize (aFilename, eMode);
	if (eErrCode == ERRCODE_NONE)
	{
		// Iteration.
#ifndef PROFILE
		ULONG nStartTicks = Time::GetSystemTicks();
#endif /* PROFILE */

#if (_DEMOSTOR_ITER == 1)
		DirectoryTraveller aTraveller (xFile);
		xFile->travel (aTraveller);
#endif /* _ITER */

#ifndef PROFILE
		ULONG nTicks = Time::GetSystemTicks();
		printf ("Total(iter): %d[ms]\n", nTicks - nStartTicks);
#endif /* PROFILE */

#if ((_DEMOSTOR_READ == 1) || (_DEMOSTOR_WRITE == 1) || (_DEMOSTOR_TRUNC == 1))
		// Buffer.
		char pBuffer[_DEMOSTOR_BUFSIZ] = "Hello World";
		pBuffer[_DEMOSTOR_BUFSIZ - 2] = 'B';
		pBuffer[_DEMOSTOR_BUFSIZ - 1] = '\0';

		// Load/Save.
#ifndef PROFILE
		nStartTicks = Time::GetSystemTicks();
#endif /* PROFILE */

		for (ULONG i = 0; i < _DEMOSTOR_LOOPS; i++)
		{
			CntStorePageLockBytesRef xLockBytes;

			String aName ("demostor-");
			aName += i + 1;
			aName += ".dat";

#if (_DEMOSTOR_REMOVE == 1)
			eErrCode = xFile->remove ("", aName);
			if ((eErrCode != ERRCODE_NONE        ) &&
				(eErrCode != ERRCODE_IO_NOTEXISTS)    )
				break;
#endif /* _REMOVE */

			eMode = STREAM_READ;
#if (_DEMOSTOR_WRITE == 1)
			eMode |= STREAM_WRITE;
#if (_DEMOSTOR_TRUNC == 1)
			eMode |= STREAM_TRUNC;
#endif /* _TRUNC */
#else
			eMode |= STREAM_NOCREATE;
#endif /* _WRITE */

			eErrCode = xFile->createLockBytes (
				xLockBytes, "", aName, eMode);
			if (eErrCode != ERRCODE_NONE)
				break;

#if (_DEMOSTOR_TRUNC == 1)
			eErrCode = xLockBytes->SetSize (0);
			if (eErrCode != ERRCODE_NONE)
				break;
#endif /* _TRUNC */

			ULONG nDone = 0;

#if (_DEMOSTOR_WRITE == 1)
			eErrCode = xLockBytes->WriteAt (
				0, pBuffer, sizeof(pBuffer), &nDone);
			if (eErrCode != ERRCODE_NONE)
				break;
#endif /* _WRITE */

#if (_DEMOSTOR_READ == 1)
			ULONG nPos = 0;
			for (;;)
			{
				eErrCode = xLockBytes->ReadAt (
					nPos, pBuffer, sizeof(pBuffer), &nDone);
				if (eErrCode != ERRCODE_NONE)
					break;
				if (nDone == 0)
					break;
				nPos += nDone;
			}
#endif /* _READ */

#ifndef PROFILE
			if (((i + 1) % (_DEMOSTOR_LOOPS/10)) == 0)
			{
				ULONG nTicks = Time::GetSystemTicks();
				printf ("%d: %12.4g\n", i + 1,
						(double)(nTicks-nStartTicks)/(double)(i+1));
			}
#endif /* PROFILE */
		}

#ifndef PROFILE
		nTicks = Time::GetSystemTicks();
		printf ("Total(rd,wr): %d[ms]\n", nTicks - nStartTicks);
#endif /* PROFILE */

#endif /* (_READ || _WRITE || _TRUNC) */
	}

#if 0 /* DISABLED */

	// Link/Rename.
	eErrCode = xFile->symlink (
		"", "demostor-1.lnk",
		"", "demostor-1.dat");
	if (eErrCode == ERRCODE_NONE)
	{
		eErrCode = xFile->rename (
			"", "demostor-1.lnk",
			"", "Shortcut to demostor-1.dat");
		if (eErrCode != ERRCODE_NONE)
		{
		}
	}

	// MakeShortName.
	String aLongName ("abcdefghijklmnopqrstuvwxyz0123456789ABCDEF");
	aLongName += "abcdefghijklmnopqrstuvwxyz0123456789ABCDEF";
	aLongName += "abcdefghijklmnopqrstuvwxyz0123456789ABCDEF";
	aLongName += "abcdefghijklmnopqrstuvwxyz0123456789ABCDEF";
	aLongName += "abcdefghijklmnopqrstuvwxyz0123456789ABCDEF";
	aLongName += "abcdefghijklmnopqrstuvwxyz0123456789ABCDEF";
	aLongName += "abcdefghijklmnopqrstuvwxyz0123456789ABCDEF";
	aLongName += ".dat";

	String aShortName (xFile->makeShortName (aLongName));

#endif /* DISABLED */

	xFile = NULL;

#if (defined(WNT) && defined(PROFILE))
	StopCAP();
	DumpCAP();
#else
	ULONG nTotalTicks = Time::GetSystemTicks();
	printf ("Total: %d[ms]\n", nTotalTicks - nStartTicks);
#endif /* PROFILE */

	return 0;
}

/************************************************************************

      Source Code Control System - Update

      $Log: DEMOSTOR.CXX,v $
      Revision 1.1.1.1  2000/09/18 17:03:11  hr
      initial import

      Revision 1.4  2000/09/18 10:01:29  willem.vandorp
      OpenOffice header added.

      Revision 1.3  1998/07/19 17:59:20  mhu
      Changed default to 'ITER' and 'DUMP' ('dumpstor'). Even more cleanup.

      Revision 1.2  1998/06/30 19:01:39  mhu
      Lot's of cleanup.

      Revision 1.1  1998/05/28 08:41:23  mhu
      class CntStore... Demo/Test Application.

   
      Rev 1.0   05 Dec 1997 12:58:02   MHU
   Initial revision.

************************************************************************/
