/*************************************************************************
 *
 *  $RCSfile: header.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _HEADER_HXX
#define _HEADER_HXX

#include <stdio.h> 
#include <string.h> 
#include <stdlib.h> 
#include <ctype.h> 
#include <tlcontnr.hxx>
#include <tlfsys.hxx>

#include "hashtbl.hxx"

// maximale Zeilenlaenge in den HeaderFiles

#define LINE_LENGTH	16384
#define WHITE_CHAR " \t\n "

#if defined (DOS) || defined (WNT) || defined (OS2)
#define PATH_SEP ";"
#endif
#if defined (UNX) 
#define PATH_SEP ":"
#endif

class ClassInfoSet;

/*
 * Die Struktur 'DefineString' verwaltet Informationen ueber einen '#ifndef/
 * #endif Bereich, unter anderem werden in pScopeClassSet alle Klassen-
 * informationen ueber die in dem aktuelle Scope enthalten Klassen verwaltet,
 * um die Abhaengigkeiten der Klassen innerhalb des Scopes darstellen zu
 * koennen.
 */
struct DefineString
{
	ULONG			nLevel;
	String* 		pStr;
	BOOL			bSubDefine;
	ClassInfoSet 	*pScopeClassSet;
};

struct ClassScope
{
	ULONG	nLevel;
	String*	pClassName;
};

DECLARE_LIST ( StringList, String* )
DECLARE_STACK ( StringStack, DefineString* )
DECLARE_STACK ( ScopeStack, ClassScope* )


#define NOT_THERE 0x0fffffff	// String ist in SStringList nicht enthalten

/*
 * class SStringList :
 * diese Klasse implementiert eine sortierte StringListe
 * 
 */
class SStringList : public StringList
{
	BOOL		Print();
#ifdef DEBUGxxx
	ULONG		nInternalCounter;	// Zaehler fuer Anzahl 
									// der benoetigten Vergleiche
#endif
public:
				SStringList();
				~SStringList();
				// neuen String in Liste einfuegen
	ULONG		PutString(String *);
				// Position des String in Liste, wenn nicht enthalten, dann
				// return = NOT_THERE
	ULONG		IsString(String *);
				// Vorgaenger ermitteln ( auch wenn selbst noch nicht in 
				// Liste enthalten
	ULONG		GetPrevString(String *);
};

/*
 * class SimpleCppParser
 *
 * Durch den SimpleCppParser werden alle Klassen, structs, enums, defines,
 * typedefs eingesammelt und ggfs. in einem LogFile eingesammelt ( class.lst)
 */

class SimpleCppParser
{
public:
	FILE*			fpClassFile;
	StringStack*	pStringStack;
	ULONG			nClass;
	ULONG			nDefine;
	ULONG			nIfdef;
	ULONG			nIfndef;
	BOOL			bClass;
	BOOL			bInCComment;
	ULONG			nIfLevel;

public:
	char			aLineBuffer[LINE_LENGTH];
	char			aBuf[LINE_LENGTH];
	String			aFileName;
	ULONG			nForward;
	FILE* 			fp;
	ULONG			nLines;

	BOOL	Init();
					SimpleCppParser( String aFile );
					SimpleCppParser( String aFile, FILE* fpLogFile );
	virtual 		~SimpleCppParser();

	virtual BOOL	Parse(void);
	virtual BOOL	ProcessLine(void);
};

/*
 * class SecondCppParser
 *
 * der SecondCppParser ermittelt mit Hilfe des vom SimpleCppParser gewonnenen
 * Daten die internern Abhaengigkeiten der Klassen und Strukturen.
 *
 * Beispiel:
 *
 * class classB {
 * 		classA *pA;
 * 	};
 * 
 * Es wird also erkannt, dass die Klasse classB von classA abhaengt.
 *
 */

class SecondCppParser : public SimpleCppParser
{
	char 			pCurrentClassName[255];
	String 			aCurrentMacroName;
	ULONG			nScope;
	StringList*		pStringList;
	ScopeStack*		pScopeStack;
	BOOL			bClassBodyExpected;
	BOOL			bInClassBody;
	BOOL			bDeriviationExpected;
	BOOL			bInMacro;
	BOOL			bReferenceFound;
public:
					SecondCppParser ( String aFile );
					SecondCppParser ( String aFile, FILE* fpLogFile );
	virtual			~SecondCppParser(){/*delete pScopeStack;*/}
	virtual BOOL	Parse(void);
	virtual BOOL	ProcessLine(void);
	BOOL			IsForward( char* pChar );
	BOOL			Put( char* pChar );
};

class ClassesList;
class ClassInfo;
class ClassInfoSet;

/*
 * class ThirdCppParser
 *
 * der ThirdCppParser analysiert die #ifndef/#endif Paarungen in den 
 * HeaderFiles, um die Abhaengigkeiten der Identifier, die innerhalb
 * dieser Klammerung stehen, zu beruecksichtigen.
 *
 * Beispiel:
 *
 * #ifndef _XYZ_HXX
 * 	classA{...};
 * 	struct structB;
 * 	enum Test { ... };
 * 	#define BLOED_DEFINE 1
 * #endif
 *
 * macht classA, structB, Test und BLOE_DEFINE voneinander abhaengig, obwohl
 * keine internen Abhaengigkeiten der Klassen untereinander vorliegen.
 */

class ThirdCppParser : public SimpleCppParser
{
	ULONG			nScope;
	StringStack*	pScopeStack;
	//IntList*		pIntList;
	ClassInfoSet	*pScopeClassSet;
public:
					ThirdCppParser ( String aFile );
					ThirdCppParser ( String aFile, FILE* fpLogFile );
	virtual			~ThirdCppParser (){}
	virtual BOOL	Parse(void);
	virtual BOOL	ProcessLine(void);
	BOOL			PutItem( char *pChar );
	BOOL			AddDeps (void);
	BOOL			Reset();
};

/*
 * class SourceCppParser
 *
 * Der SourceCppParser ermittelt mit Hilfe der vorher gewonnenen Informationen
 * die resultierenden defines, die in das SourceFile eingabut werden duerfen.
 */

class SourceCppParser : public SimpleCppParser
{
	StringList*		pResultList;			// alle Defs, die nicht gesetzt 
											// werden duerfen
	StringList*		pAllDefineList;			// Liste aller Defines aus solar
public:
					SourceCppParser ( String aFile );
					~SourceCppParser();
	virtual BOOL	ProcessLine(void);
	virtual BOOL	Parse(void);
	BOOL			WriteResult();
	BOOL			InitDef();
};

/*
 * ClassOccurence enhaelt die Info ueber das Vorkommen eines Identfierers
 */

struct ClassOccurence
{
	ULONG			nLine;
	String			aFileName;
	String*			pStr;
};

DECLARE_LIST( OccList, ClassOccurence* )

/*
 * class ClassInfo
 *
 * enthaelt alle Informationen ueber Abhaengigkeiten und defines einer
 * Klasse
 */

class ClassInfo
{
	String			aClassName;				// Name der Klasse
	String			aFileName;				// Name des Headerfiles der Klasse
	String			aType;
	BOOL			Init();
public:
	SStringList*	pClassDefineList;		// Liste der ifndef's
	SStringList*	pClassDepList;			// Liste der benoetigten Klassen
	OccList*		pOccList;

					ClassInfo();
					ClassInfo(char* pLine);		
					~ClassInfo();
	SStringList*	GetDefineList(){return pClassDefineList;}
	SStringList*	GetDepList(){return pClassDepList;}
	String			GetClassName(){return aClassName;}
	String			GetClassType(){return aType;}
	BOOL			AddDeps(ClassInfo *pInfo);
};

/*
 * class AllClasses
 *
 * verwaltet alle KlassenInfomationen
 */

DECLARE_HASHTABLE( ClassInfoSet, String, ClassInfo * );

class AllClasses
{
	ClassInfoSet*	pClassSet;
public:
					AllClasses();
	BOOL			Update();
	BOOL			UpdateDefs();
	BOOL			MakeUnique();
	ClassInfoSet*	GetSet(){return pClassSet;}
};

#endif
