%{
/*
 * lexer for parsing xml-property source files (*.xml)
 *
 */


/* enlarge token buffer to tokenize whole strings */
#undef YYLMAX
#define YYLMAX 64000

/* to enable debug output define LEXDEBUG */
#define LEXDEBUG		1
#ifdef LEXDEBUG
#define OUTPUT	fprintf
#else
#define OUTPUT(Par1,Par2);
#endif

/* table of possible token ids */
#include "tokens.h"
#include <stdlib.h>
#include <stdio.h>

/* external functions (C++ code, declared as extren "C" */
extern WorkOnTokenSet( int, char* );
extern Argument( char * );
extern InitXmlExport( char * );
extern EndXmlExport();
extern GetError();
extern SetError();
extern char *GetOutputFile( int argc, char* argv[]);
extern FILE *GetXmlFile();

/* forwards */
void YYWarning();

int bText=0;
%}

%p 24000
%e 1200
%n 500

%%

"<ResourceBundle"[ \t]+"package"[ \t]*=[^\<]+\>	{
	WorkOnTokenSet( XML_RESSOURCEBUNDLE_START, yytext );
}

"</ResourceBundle>"	{
	WorkOnTokenSet( XML_RESSOURCEBUNDLE_END, yytext );
}

"<Context"[ \t]+"name"[ \t]*=[^\<]+\>	{
	WorkOnTokenSet( XML_CONTEXT_START, yytext );
}

"</Context>"	{
	WorkOnTokenSet( XML_CONTEXT_END, yytext );
}

"<Key"[ \t]+"id"[ \t]*=[^\<]+\>	{
	WorkOnTokenSet( XML_KEY_START, yytext );
}

"</Key>"	{
	WorkOnTokenSet( XML_KEY_END, yytext );
}

"<Text"[ \t]+"xml:lang"[ \t]*=[^\<]+\>	{
	bText = 1;
	WorkOnTokenSet( XML_TEXT_START, yytext );
}

"</Text>"	{
	bText = 0;
	WorkOnTokenSet( XML_TEXT_END, yytext );
}

"<Arg"[ \t]+"n"[ \t]*=[^\<]+\>	{
	if ( bText == 1 ) {
		WorkOnTokenSet( XML_TEXTCHAR, yytext );
		Argument( yytext );
	}
	else
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
}

.|\n {
	if ( bText == 1 )
		WorkOnTokenSet( XML_TEXTCHAR, yytext );
	else
		WorkOnTokenSet( UNKNOWNCHAR, yytext );
}


%%

/*****************************************************************************/
int	yywrap(void)
/*****************************************************************************/
{
	return 1;
}

/*****************************************************************************/
void YYWarning( char *s )
/*****************************************************************************/
{
	/* write warning to stderr */
	fprintf( stderr,
		"Warning: \"%s\" in line %d: \"%s\"\n", s, yylineno, yytext  );
}

/*****************************************************************************/
#ifdef GCC
void yyerror ( char *s, ... )
#else
void yyerror ( char *s )
#endif
/*****************************************************************************/
{
	/* write error to stderr */
	fprintf( stderr,
		"Error: \"%s\" in line %d: \"%s\"\n", s, yylineno, yytext  );
	SetError();
}

/*****************************************************************************/
int
#ifdef WNT
_cdecl
#endif
main( int argc, char* argv[])
/*****************************************************************************/
{
	/* error level */
	int nRetValue = 0;
	char *pOutput;
	FILE *pFile;

	fprintf( stdout, "\nXmlEx 0.9 Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.\n" );
	fprintf( stdout, "======================================================================\n" );

	pOutput = GetOutputFile( argc, argv );
	if ( !pOutput ) {
		fprintf( stdout, "Syntax:LNGEX[-p Prj][-r PrjRoot]-i FileIn [-o FileOut][-m DataBase][-e][-b][-u][-d DoneFile][-g]\n" );
		fprintf( stdout, " Prj:      Project\n" );
		fprintf( stdout, " PrjRoot:  Path to project root (..\\.. etc.)\n" );
		fprintf( stdout, " FileIn:   Source files (*.src)\n" );
		fprintf( stdout, " FileOut:  Destination file (*.*)\n" );
		fprintf( stdout, " DataBase: Mergedata (*.sdf)\n" );
		fprintf( stdout, " -e: Disable writing errorlog\n" );
		fprintf( stdout, " -b: Break when Token \"HelpText\" found in source\n" );
		fprintf( stdout, " -u: [english] and [german] are allowed, Id is Taken from DataBase \n" );
		fprintf( stdout, " -g: enables generation of properties - in this case FileOut is the output path\n" );
		fprintf( stdout, " -d: enables generation of *.don if work is done\n" );
		return 1;
	}

	pFile = GetXmlFile();
	if ( !pFile )
		return 1;

   	yyin = pFile;

	/* create global instance of class XmlExport */
	InitXmlExport( pOutput );

	/* start parser */
   	yylex();

	/* get error info. and end export */
	nRetValue = GetError();
	EndXmlExport();

	fprintf( stdout, "\n===================================\n\n" );

	/* return error level */
	return nRetValue;
}
