/*************************************************************************
 *
 *  $RCSfile: xmlmerge.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:26 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdio.h>
#include <tools/string.hxx>
#include <tools/fsys.hxx>

// local includes
#include "export.hxx"
#include "xmlmerge.hxx"
#include "tokens.h"
#include "utf8conv.hxx"

extern "C" { yyerror( char * ); }
extern "C" { YYWarning( char * ); }

// defines to parse command line
#define STATE_NON  		0x0001
#define STATE_INPUT		0x0002
#define STATE_OUTPUT	0x0003
#define STATE_PRJ		0x0004
#define STATE_ROOT		0x0005
#define STATE_MERGESRC	0x0006
#define STATE_ERRORLOG	0x0007
#define STATE_DONEFILE	0x0008
#define STATE_GENERATOR	0x0009

// set of global variables
BOOL bEnableExport;
BOOL bMergeMode;
BOOL bErrorLog;
BOOL bGenerator;
BOOL bDoneFile;
ByteString sPrj;
ByteString sPrjRoot;
ByteString sInputFileName;
ByteString sActFileName;
ByteString sOutputFile;
ByteString sMergeSrc;
ByteString sDoneFile;
XMLResParser *pParser = NULL;

extern "C" {
// the whole interface to lexer is in this extern "C" section

/*****************************************************************************/
extern char *GetOutputFile( int argc, char* argv[])
/*****************************************************************************/
{
	bEnableExport = FALSE;
	bMergeMode = FALSE;
	bErrorLog = TRUE;
	bGenerator = FALSE;
	bDoneFile = FALSE;
	sPrj = "";
	sPrjRoot = "";
	sInputFileName = "";
	sActFileName = "";

	USHORT nState = STATE_NON;
	BOOL bInput = FALSE;

	// parse command line
	for( int i = 1; i < argc; i++ ) {
		if (( ByteString( argv[ i ]) == "-i" ) || ( argv[ i ] == "-I" )) {
			nState = STATE_INPUT; // next token specifies source file
		}
		else if (( ByteString( argv[ i ]) == "-o" ) || ( argv[ i ] == "-O" )) {
			nState = STATE_OUTPUT; // next token specifies the dest file
		}
		else if (( ByteString( argv[ i ]) == "-p" ) || ( argv[ i ] == "-P" )) {
			nState = STATE_PRJ; // next token specifies the cur. project
		}
		else if (( ByteString( argv[ i ]) == "-r" ) || ( argv[ i ] == "-R" )) {
			nState = STATE_ROOT; // next token specifies path to project root
		}
		else if (( ByteString( argv[ i ]) == "-m" ) || ( argv[ i ] == "-M" )) {
			nState = STATE_MERGESRC; // next token specifies the merge database
		}
		else if (( ByteString( argv[ i ]) == "-e" ) || ( argv[ i ] == "-E" )) {
			nState = STATE_ERRORLOG;
			bErrorLog = FALSE;
		}
		else if (( ByteString( argv[ i ]) == "-g" ) || ( argv[ i ] == "-G" )) {
			nState = STATE_GENERATOR;
			bGenerator = TRUE;
		}
		else if (( ByteString( argv[ i ]) == "-d" ) || ( argv[ i ] == "-d" )) {
			nState = STATE_DONEFILE;
			bDoneFile = TRUE;
		}
		else {
			switch ( nState ) {
				case STATE_NON: {
					return NULL;	// no valid command line
				}
				break;
				case STATE_INPUT: {
					sInputFileName = argv[ i ];
					bInput = TRUE; // source file found
				}
				break;
				case STATE_OUTPUT: {
					sOutputFile = argv[ i ]; // the dest. file
				}
				break;
				case STATE_PRJ: {
					sPrj = ByteString( argv[ i ]);
					sPrj.ToLowerAscii(); // the project
				}
				break;
				case STATE_ROOT: {
					sPrjRoot = ByteString( argv[ i ]); // path to project root
				}
				break;
				case STATE_MERGESRC: {
					sMergeSrc = ByteString( argv[ i ]);
					bMergeMode = TRUE; // activate merge mode, cause merge database found
				}
				case STATE_DONEFILE: {
					sDoneFile = ByteString( argv[ i ]);
				}
				break;
			}
		}
	}

	if ( bInput ) {
		// command line is valid
		bEnableExport = TRUE;
		char *pReturn = new char[ sOutputFile.Len() + 1 ];
		strcpy( pReturn, sOutputFile.GetBuffer());
		return pReturn;
	}

	// command line is not valid
	return NULL;
}

/*****************************************************************************/
int InitXmlExport( char *pOutput )
/*****************************************************************************/
{
	// instanciate Export
	ByteString sOutput( pOutput );

	if ( bGenerator ) {
		DirEntry aEntry( String( sInputFileName, RTL_TEXTENCODING_ASCII_US ));
		ByteString sBaseName( aEntry.GetBase(), RTL_TEXTENCODING_ASCII_US );
		pParser = new XMLPropertyGenerator( sOutputFile, sBaseName );
	}
	else if ( bMergeMode )
		pParser = new XMLResMerge( sMergeSrc, sOutputFile, bErrorLog );
  	else if ( sOutputFile.Len()) {
		pParser = new XMLResExport( sOutputFile, sPrj, sActFileName );
	}
	else
		pParser = new XMLResParser();

	return 1;
}

/*****************************************************************************/
int EndXmlExport()
/*****************************************************************************/
{
	if ( bGenerator && bDoneFile && !pParser->GetError()) {
		DirEntry aEntry( String( sDoneFile, RTL_TEXTENCODING_ASCII_US ));
		SvFileStream aStream(
			aEntry.GetFull(), STREAM_STD_WRITE | STREAM_TRUNC );
		if ( !aStream.IsOpen()) {
			ByteString sError( "Unable to open done file: " );
			sError += ByteString(
				aEntry.GetFull(), RTL_TEXTENCODING_ASCII_US );
	 	}
		else
			aStream.Close();
	}

	delete pParser;
	return 1;
}

/*****************************************************************************/
extern FILE *GetXmlFile()
/*****************************************************************************/
{
	// look for valid filename
	if ( sInputFileName.Len()) {

		// able to open file?
		FILE *pFile = fopen( sInputFileName.GetBuffer(), "r" );
		if ( !pFile )
			fprintf( stderr, "Error: Could not open file %s\n",
				sInputFileName.GetBuffer());
		else {
			// this is a valid file which can be opened, so
			// create path to project root
			DirEntry aEntry( String( sInputFileName, RTL_TEXTENCODING_ASCII_US ));
			aEntry.ToAbs();
			ByteString sFullEntry( aEntry.GetFull(), RTL_TEXTENCODING_ASCII_US );
			aEntry += DirEntry( String( "..", RTL_TEXTENCODING_ASCII_US ));
			aEntry += DirEntry( sPrjRoot );
			ByteString sPrjEntry( aEntry.GetFull(), RTL_TEXTENCODING_ASCII_US );

			// create file name, beginnig with project root
			// (e.g.: source\ui\src\menue.src)
			sActFileName = sFullEntry.Copy( sPrjEntry.Len() + 1 );
			sActFileName.ToLowerAscii();

			fprintf( stdout, "\nProcessing File %s ...\n", sInputFileName.GetBuffer());
			return pFile;
		}
	}
	// this means the file could not be opened
	return NULL;
}

/*****************************************************************************/
int WorkOnTokenSet( int nTyp, char *pTokenText )
/*****************************************************************************/
{
	pParser->Execute( nTyp, pTokenText );

	return 1;
}

/*****************************************************************************/
void Argument( char *pArg )
/*****************************************************************************/
{
	ByteString sArg( pArg );
	pParser->Argument( sArg );
}

/*****************************************************************************/
int SetError()
/*****************************************************************************/
{
	pParser->SetError();
	return 1;
}
}

extern "C" {
/*****************************************************************************/
int GetError()
/*****************************************************************************/
{
	return pParser->GetError();
}
}

//
// class XMLResParser
//

// defines for DEA-States
#define XMLRESSTATE_START					0x0000
#define XMLRESSTATE_RESSOURCEBUNDLE         0x0001
#define XMLRESSTATE_CONTEXT                 0x0002
#define XMLRESSTATE_KEY                     0x0004
#define XMLRESSTATE_TEXT                    0x0008

/*****************************************************************************/
XMLResParser::XMLResParser()
/*****************************************************************************/
				: nDEAState( XMLRESSTATE_START ),
				bError( FALSE )
{
}

/*****************************************************************************/
XMLResParser::~XMLResParser()
/*****************************************************************************/
{
}

/*****************************************************************************/
void XMLResParser::Argument( const ByteString &rArg )
/*****************************************************************************/
{
}

/*****************************************************************************/
int XMLResParser::Execute( int nToken, char * pToken )
/*****************************************************************************/
{
	BOOL bError = FALSE;
	ByteString sToken( pToken );

	switch ( nDEAState ) {
		case XMLRESSTATE_START:
			switch ( nToken ) {
				case XML_RESSOURCEBUNDLE_START:
					nDEAState = XMLRESSTATE_RESSOURCEBUNDLE;
					sCurrentRessourceBundle=
						sToken.GetToken( 1, '\"' ).GetToken( 0, '\"' );
					if ( !sCurrentRessourceBundle.Len())
						sCurrentRessourceBundle=
							sToken.GetToken( 1, '\'' ).GetToken( 0, '\'' );
					Output( sToken );
				break;
				case XML_RESSOURCEBUNDLE_END:
				case XML_CONTEXT_START:
				case XML_CONTEXT_END:
				case XML_KEY_START:
				case XML_KEY_END:
				case XML_TEXT_START	:
				case XML_TEXT_END:
					Error( "SyntaxExrror: misplaced Tag" );
					Output( sToken );
				break;
				default:
					Output( sToken );
				break;
			}
		break;
		case XMLRESSTATE_RESSOURCEBUNDLE:
			switch ( nToken ) {
				case XML_RESSOURCEBUNDLE_END:
					nDEAState = XMLRESSTATE_START;
					EndOfRessourceBundle( sCurrentRessourceBundle );
					Output( sToken );
				break;
				case XML_CONTEXT_START:
					nDEAState = XMLRESSTATE_CONTEXT;
					sCurrentContext =
						sToken.GetToken( 1, '\"' ).GetToken( 0, '\"' );
					if ( !sCurrentContext.Len())
						sCurrentContext =
							sToken.GetToken( 1, '\'' ).GetToken( 0, '\'' );
					Output( sToken );
				break;
				case XML_RESSOURCEBUNDLE_START:
				case XML_CONTEXT_END:
				case XML_KEY_START:
				case XML_KEY_END:
				case XML_TEXT_START	:
				case XML_TEXT_END:
					Error( "SyntaxExrror: misplaced Tag" );
					Output( sToken );
				break;
				default:
					Output( sToken );
				break;
			}
		break;
		case XMLRESSTATE_CONTEXT:
			switch ( nToken ) {
				case XML_CONTEXT_END:
					nDEAState = XMLRESSTATE_RESSOURCEBUNDLE;
					Output( sToken );
				break;
				case XML_KEY_START:
					nDEAState = XMLRESSTATE_KEY;
					sCurrentKey =
						sToken.GetToken( 1, '\"' ).GetToken( 0, '\"' );
					if ( !sCurrentKey.Len())
						sCurrentKey =
							sToken.GetToken( 1, '\'' ).GetToken( 0, '\'' );
					Output( sToken );
				break;
				case XML_RESSOURCEBUNDLE_START:
				case XML_RESSOURCEBUNDLE_END:
				case XML_CONTEXT_START:
				case XML_KEY_END:
				case XML_TEXT_START	:
				case XML_TEXT_END:
					Error( "SyntaxExrror: misplaced Tag" );
					Output( sToken );
				break;
				default:
					Output( sToken );
				break;
			}
		break;
		case XMLRESSTATE_KEY:
			switch ( nToken ) {
				case XML_KEY_END:
					nDEAState = XMLRESSTATE_CONTEXT;
					EndOfKey(
						sCurrentRessourceBundle,
						sCurrentContext,
						sCurrentKey
					);
					Output( sToken );
				break;
				case XML_TEXT_START:
					nDEAState = XMLRESSTATE_TEXT;
					nCurrentLang = GetLang( sToken );
					Output( sToken );
				break;
				case XML_RESSOURCEBUNDLE_START:
				case XML_RESSOURCEBUNDLE_END:
				case XML_CONTEXT_START:
				case XML_CONTEXT_END:
				case XML_KEY_START:
				case XML_TEXT_END:
					Error( "SyntaxExrror: misplaced Tag" );
					Output( sToken );
				break;
				default:
					Output( sToken );
				break;
			}
		break;
		case XMLRESSTATE_TEXT:
			switch ( nToken ) {
				case XML_TEXT_END:
					nDEAState = XMLRESSTATE_KEY;

					if ( nCurrentLang )
						WorkOnText(
							sCurrentRessourceBundle,
							sCurrentContext,
							sCurrentKey,
							nCurrentLang,
							sCurrentText
						);
					Output( sCurrentText );
					sCurrentText = "";

					Output( sToken );
				break;
				case XML_TEXTCHAR:
					nDEAState = XMLRESSTATE_TEXT;
					sCurrentText += sToken;
				break;
				case XML_RESSOURCEBUNDLE_START:
				case XML_RESSOURCEBUNDLE_END:
				case XML_CONTEXT_START:
				case XML_CONTEXT_END:
				case XML_KEY_START:
				case XML_KEY_END:
				case XML_TEXT_START	:
					Error( "SyntaxExrror: misplaced Tag" );
					Output( sToken );
				break;
				default:
					Output( sToken );
				break;
			}
		break;
		default:
			Output( sToken );
		break;
	}

	return 1;
};

/*****************************************************************************/
USHORT XMLResParser::GetLang( const ByteString &rStartTextToken )
/*****************************************************************************/
{
	ByteString sLowerLang(
		rStartTextToken.GetToken( 1, '\"' ).GetToken( 0, '\"' ));
	if ( !sLowerLang.Len())
		sLowerLang =
			rStartTextToken.GetToken( 1, '\'' ).GetToken( 0, '\'' );
	ByteString sLang( sLowerLang );
	sLang.ToUpperAscii();

	if ( sLang.ToUpperAscii() == ByteString( COMMENT_ISO ).ToUpperAscii())
		return COMMENT;
	else if ( sLang == ByteString( ENGLISH_US_ISO ).ToUpperAscii())
		return ENGLISH_US;
	else if ( sLang == ByteString( PORTUGUESE_ISO ).ToUpperAscii())
		return PORTUGUESE;
	else if ( sLang == ByteString( RUSSIAN_ISO ).ToUpperAscii())
		return RUSSIAN;
	else if ( sLang == ByteString( GREEK_ISO ).ToUpperAscii())
		return GREEK;
	else if ( sLang == ByteString( DUTCH_ISO ).ToUpperAscii())
		return DUTCH;
	else if ( sLang == ByteString( FRENCH_ISO ).ToUpperAscii())
		return FRENCH;
	else if ( sLang == ByteString( SPANISH_ISO ).ToUpperAscii())
		return SPANISH;
	else if ( sLang == ByteString( FINNISH_ISO ).ToUpperAscii())
		return FINNISH;
	else if ( sLang == ByteString( HUNGARIAN_ISO ).ToUpperAscii())
		return HUNGARIAN;
	else if ( sLang == ByteString( ITALIAN_ISO ).ToUpperAscii())
		return ITALIAN;
	else if ( sLang == ByteString( CZECH_ISO ).ToUpperAscii())
		return CZECH;
	else if ( sLang == ByteString( SLOVAK_ISO ).ToUpperAscii())
		return SLOVAK;
	else if ( sLang == ByteString( ENGLISH_ISO ).ToUpperAscii())
		return ENGLISH;
	else if ( sLang == ByteString( DANISH_ISO ).ToUpperAscii())
		return DANISH;
	else if ( sLang == ByteString( SWEDISH_ISO ).ToUpperAscii())
		return SWEDISH;
	else if ( sLang == ByteString( NORWEGIAN_ISO ).ToUpperAscii())
		return NORWEGIAN;
	else if ( sLang == ByteString( POLISH_ISO ).ToUpperAscii())
		return POLISH;
	else if ( sLang == ByteString( GERMAN_ISO ).ToUpperAscii())
		return GERMAN;
	else if ( sLang == ByteString( PORTUGUESE_BRAZILIAN_ISO ).ToUpperAscii())
		return PORTUGUESE_BRAZILIAN;
	else if ( sLang == ByteString( JAPANESE_ISO ).ToUpperAscii())
		return JAPANESE;
	else if ( sLang == ByteString( KOREAN_ISO ).ToUpperAscii())
		return KOREAN;
	else if ( sLang == ByteString( CHINESE_SIMPLIFIED_ISO ).ToUpperAscii())
		return CHINESE_SIMPLIFIED;
	else if ( sLang == ByteString( CHINESE_TRADITIONAL_ISO ).ToUpperAscii())
		return CHINESE_TRADITIONAL;
	else if ( sLang == ByteString( TURKISH_ISO ).ToUpperAscii())
		return TURKISH;
	else if ( sLang == ByteString( ARABIC_ISO ).ToUpperAscii())
		return ARABIC;
	else if ( sLang == ByteString( HEBREW_ISO ).ToUpperAscii())
		return HEBREW;

	ByteString sError( "Unknown language code: " );
	sError += sLowerLang;
	Error( sError );
	return 0;
}

/*****************************************************************************/
ByteString XMLResParser::GetIsoLangByIndex( USHORT nIndex )
/*****************************************************************************/
{
	switch ( nIndex ) {
		case COMMENT_INDEX: return COMMENT_ISO;
		case ENGLISH_US_INDEX: return ENGLISH_US_ISO;
		case PORTUGUESE_INDEX: return PORTUGUESE_ISO;
		case RUSSIAN_INDEX: return RUSSIAN_ISO;
		case GREEK_INDEX: return GREEK_ISO;
		case DUTCH_INDEX: return DUTCH_ISO;
		case FRENCH_INDEX: return FRENCH_ISO;
		case SPANISH_INDEX: return SPANISH_ISO;
		case FINNISH_INDEX: return FINNISH_ISO;
		case HUNGARIAN_INDEX: return HUNGARIAN_ISO;
		case ITALIAN_INDEX: return ITALIAN_ISO;
		case CZECH_INDEX: return CZECH_ISO;
		case SLOVAK_INDEX: return SLOVAK_ISO;
		case ENGLISH_INDEX: return ENGLISH_ISO;
		case DANISH_INDEX: return DANISH_ISO;
		case SWEDISH_INDEX: return SWEDISH_ISO;
		case NORWEGIAN_INDEX: return NORWEGIAN_ISO;
		case POLISH_INDEX: return POLISH_ISO;
		case GERMAN_INDEX: return GERMAN_ISO;
		case PORTUGUESE_BRAZILIAN_INDEX: return PORTUGUESE_BRAZILIAN_ISO;
		case JAPANESE_INDEX: return JAPANESE_ISO;
		case KOREAN_INDEX: return KOREAN_ISO;
		case CHINESE_SIMPLIFIED_INDEX: return CHINESE_SIMPLIFIED_ISO;
		case CHINESE_TRADITIONAL_INDEX: return CHINESE_TRADITIONAL_ISO;
		case TURKISH_INDEX: return TURKISH_ISO;
		case ARABIC_INDEX: return ARABIC_ISO;
		case HEBREW_INDEX: return HEBREW_ISO;
	}
	return "";
}

/*****************************************************************************/
ByteString XMLResParser::GetPropertyLangByIndex( USHORT nIndex )
/*****************************************************************************/
{
	switch ( nIndex ) {
		case ENGLISH_US_INDEX: return ENGLISH_US_PROPERTY;
		case PORTUGUESE_BRAZILIAN_INDEX: return PORTUGUESE_BRAZILIAN_PROPERTY;
		case CHINESE_SIMPLIFIED_INDEX: return CHINESE_SIMPLIFIED_PROPERTY;
		case CHINESE_TRADITIONAL_INDEX: return CHINESE_TRADITIONAL_PROPERTY;
	}
	return GetIsoLangByIndex( nIndex );
}

/*****************************************************************************/
void XMLResParser::WorkOnText(
	const ByteString &rRessourceBundle,	const ByteString &rContext,
	const ByteString &sKey,	USHORT nLang, ByteString &rText	)
/*****************************************************************************/
{
}

/*****************************************************************************/
void XMLResParser::EndOfRessourceBundle(
	const ByteString &rRessourceBundle )
/*****************************************************************************/
{
}

/*****************************************************************************/
void XMLResParser::Output( const ByteString& rOutput )
/*****************************************************************************/
{
}

/*****************************************************************************/
void XMLResParser::EndOfKey( const ByteString &rRessourceBundle,
	const ByteString &rContext,	const ByteString &sKey )
/*****************************************************************************/
{
}

/*****************************************************************************/
void XMLResParser::Error( const ByteString &rError )
/*****************************************************************************/
{
	yyerror(( char * ) rError.GetBuffer());
}

//
// class XMLResOutputParser
//

/*****************************************************************************/
XMLResOutputParser::XMLResOutputParser ( const ByteString &rOutputFile )
/*****************************************************************************/
{
	pOutputStream =
		new SvFileStream(
			String( rOutputFile, RTL_TEXTENCODING_ASCII_US ),
			STREAM_STD_WRITE | STREAM_TRUNC
		);

	if ( !pOutputStream->IsOpen()) {
		ByteString sError( "Unable to open output file: " );
		sError += rOutputFile;
		Error( sError );
		delete pOutputStream;
		pOutputStream = NULL;
	}
}

/*****************************************************************************/
XMLResOutputParser::~XMLResOutputParser()
/*****************************************************************************/
{
	if ( pOutputStream ) {
		pOutputStream->Close();
		delete pOutputStream;
	}
}


//
// class XMLResExport
//

/*****************************************************************************/
XMLResExport::XMLResExport(
	const ByteString &rOutputFile, const ByteString &rProject,
	const ByteString &rFilePath )
/*****************************************************************************/
				: XMLResOutputParser( rOutputFile ),
				pResData( NULL ),
				sPrj( rProject ),
				sPath( rFilePath )
{
}

/*****************************************************************************/
XMLResExport::~XMLResExport()
/*****************************************************************************/
{
	delete pResData;
}


/*****************************************************************************/
void XMLResExport::WorkOnText(
	const ByteString &rRessourceBundle,	const ByteString &rContext,
		const ByteString &sKey,	USHORT nLang, ByteString &rText )
/*****************************************************************************/
{
	if ( !pResData ) {
		ByteString sGroupId( rRessourceBundle );
		sGroupId += ".";
		sGroupId += rContext;

		ByteString sPlatform( "" );

		pResData = new ResData( sPlatform, sGroupId );
		pResData->sId = sKey;
	}

	pResData->sText[ Export::GetLangIndex( nLang )] =
		UTF8Converter::ConvertFromUTF8( rText, Export::GetCharSet( nLang ));
}

/*****************************************************************************/
void XMLResExport::EndOfKey(
	const ByteString &rRessourceBundle, const ByteString &rContext,
	const ByteString &sKey )
/*****************************************************************************/
{
	if ( pOutputStream ) {
		if ( pResData->sText[ GERMAN_INDEX ].Len() &&
		   	( pResData->sText[ ENGLISH_US_INDEX ].Len() ||
		   		pResData->sText[ ENGLISH_INDEX ].Len()))
		{
			Time aTime;
			ByteString sTimeStamp( ByteString::CreateFromInt64( Date().GetDate()));
			sTimeStamp += " ";
			sTimeStamp += ByteString::CreateFromInt32( aTime.GetHour());
			sTimeStamp += ":";
			sTimeStamp += ByteString::CreateFromInt32( aTime.GetMin());
			sTimeStamp += ":";
			sTimeStamp += ByteString::CreateFromInt32( aTime.GetSec());

			for ( ULONG i = 0; i < LANGUAGES; i++ ) {
				if ( LANGUAGE_ALLOWED( i )) {
					ByteString sAct = pResData->sText[ i ];
					if ( !sAct.Len() && i )
						sAct = pResData->sText[ GERMAN_INDEX ];

					ByteString sOutput( sPrj ); sOutput += "\t";
					sOutput += sPath;
					sOutput += "\t0\t";
					sOutput += "XMLText\t";
					sOutput += pResData->sGId; sOutput += "\t";
					sOutput += pResData->sId; sOutput += "\t\t\t0\t";
					sOutput += ByteString::CreateFromInt64( Export::LangId[ i ] ); sOutput += "\t";
					sOutput += sAct; sOutput += "\t\t\t\t";
					sOutput += sTimeStamp;

					pOutputStream->WriteLine( sOutput );
				}
			}
		}
	}
	delete pResData;
	pResData = NULL;
}

//
// class XMLResMerge
//

/*****************************************************************************/
XMLResMerge::XMLResMerge(
	const ByteString &rMergeSource, const ByteString &rOutputFile,
	BOOL bErrorLog )
/*****************************************************************************/
				: XMLResOutputParser( rOutputFile ),
				pMergeDataFile( NULL ),
				pResData( NULL )
{
	if ( rMergeSource.Len())
		pMergeDataFile = new MergeDataFile(
			rMergeSource, bErrorLog, RTL_TEXTENCODING_MS_1252 );
}

/*****************************************************************************/
XMLResMerge::~XMLResMerge()
/*****************************************************************************/
{
	delete pMergeDataFile;
	delete pResData;
}

/*****************************************************************************/
void XMLResMerge::WorkOnText(
	const ByteString &rRessourceBundle,	const ByteString &rContext,
	const ByteString &sKey,	USHORT nLang, ByteString &rText )
/*****************************************************************************/
{
	if ( pMergeDataFile ) {
		if ( !pResData ) {
			ByteString sGroupId( rRessourceBundle );
			sGroupId += ".";
			sGroupId += rContext;

			ByteString sPlatform( "" );

			pResData = new ResData( sPlatform, sGroupId );
			pResData->sId = sKey;
			pResData->sResTyp = "xmltext";
		}

		PFormEntrys *pEntrys = pMergeDataFile->GetPFormEntrys( pResData );
		if ( pEntrys ) {
			ByteString sContent;
			if (( nLang != GERMAN ) &&
				( nLang != ENGLISH ) &&
				( pEntrys->GetText(
					sContent, STRING_TYP_TEXT, Export::GetLangIndex( nLang ))))
					rText = UTF8Converter::ConvertToUTF8(
						sContent, Export::GetCharSet( nLang )) ;
		}
	}
}

/*****************************************************************************/
void XMLResMerge::Output( const ByteString& rOutput )
/*****************************************************************************/
{
	if ( pOutputStream )
		pOutputStream->Write( rOutput.GetBuffer(), rOutput.Len());
}

/*****************************************************************************/
void XMLResMerge::EndOfKey(
	const ByteString &rRessourceBundle,	const ByteString &rContext,
	const ByteString &sKey )
/*****************************************************************************/
{
	if ( pMergeDataFile ) {
		PFormEntrys *pEntrys = pMergeDataFile->GetPFormEntrys( pResData );
		if ( pEntrys ) {
			for ( ULONG nIndex = 0; nIndex < LANGUAGES; nIndex++ ) {
				ByteString sContent;
				if (( nIndex != GERMAN_INDEX ) &&
					( nIndex != ENGLISH_INDEX ) &&
					( LANGUAGE_ALLOWED( nIndex )) &&
					( pEntrys->GetText(
						sContent, STRING_TYP_TEXT, nIndex, TRUE )) &&
					( sContent != "-" ) && ( sContent.Len()))
				{
					ByteString sText = UTF8Converter::ConvertToUTF8(
						sContent, Export::GetCharSet( Export::LangId[ nIndex ]));
					ByteString sAdditionalLine( "\t<Text xml:lang=\"" );
					sAdditionalLine += GetIsoLangByIndex( nIndex );
					sAdditionalLine += "\">";
					sAdditionalLine += sText;
					sAdditionalLine += "</Text>\n\t";

					Output( sAdditionalLine );
				}
			}
		}
	}
	delete pResData;
	pResData = NULL;
}

//
// class XMLPropertyGenerator
//

/*****************************************************************************/
XMLPropertyGenerator::XMLPropertyGenerator(
	const ByteString &rOutputPath, const ByteString &rBaseName )
/*****************************************************************************/
				: sBaseName( rBaseName ),
				sOutputPath( rOutputPath )
{
	for ( ULONG i = 0; i < LANGUAGES; i++ )
		pOutputStream[ i ] = NULL;
}

/*****************************************************************************/
XMLPropertyGenerator::~XMLPropertyGenerator()
/*****************************************************************************/
{
	for ( ULONG i = 0; i < LANGUAGES; i++ ) {
		if ( pOutputStream[i ] && pOutputStream[ i ]->IsOpen())
			pOutputStream[ i ]->Close();
		delete pOutputStream[ i ];
	}
}

/*****************************************************************************/
void XMLPropertyGenerator::WorkOnText(
	const ByteString &rRessourceBundle,	const ByteString &rContext,
	const ByteString &sKey,	USHORT nLang, ByteString &rText )
/*****************************************************************************/
{
	USHORT nIndex = Export::GetLangIndex( nLang );
	if ( nIndex && nIndex != ENGLISH_INDEX ) {
		if ( !pOutputStream[ nIndex ] ) {
			DirEntry aEntry( String( sOutputPath, RTL_TEXTENCODING_ASCII_US ));
			aEntry.MakeDir();
			for ( ULONG i = 0;
				i < rRessourceBundle.GetTokenCount( '.' ); i++ )
			{
				aEntry += DirEntry(
					String( rRessourceBundle.GetToken( i, '.' ),
						RTL_TEXTENCODING_ASCII_US ));
				aEntry.MakeDir();
			}

			ByteString sOutputName( sBaseName );
			sOutputName += "_";
			sOutputName += GetPropertyLangByIndex( nIndex );
			sOutputName += ".";
			sOutputName += "properties";

			aEntry += DirEntry( String( sOutputName, RTL_TEXTENCODING_ASCII_US ));

			pOutputStream[ nIndex ] =
				new SvFileStream( aEntry.GetFull(),
				 	STREAM_STD_WRITE | STREAM_TRUNC );

			if ( !pOutputStream[ nIndex ]->IsOpen()) {
				ByteString sError( "Error: Could not open file " );
				sError += ByteString( pOutputStream[ nIndex ]->
					GetFileName(), RTL_TEXTENCODING_ASCII_US );
				Error( sError );
			}
		}
		if ( pOutputStream[ nIndex ]->IsOpen()) {
			ByteString sIdentifier( rContext );
			sIdentifier += ".";
			sIdentifier += sKey;

			ByteString sText( rText );
			ReplaceArguments( sText );

			sIdentifier += "=";
			sIdentifier += sText;

			pOutputStream[ nIndex ]->WriteLine( sIdentifier );
		}
	}
}

/*****************************************************************************/
void XMLPropertyGenerator::EndOfRessourceBundle(
	const ByteString &rRessourceBundle )
/*****************************************************************************/
{
	for ( ULONG i = 0; i < LANGUAGES; i++ ) {
		if ( pOutputStream[i ] && pOutputStream[ i ]->IsOpen())
			pOutputStream[ i ]->Close();
		delete pOutputStream[ i ];
		pOutputStream[ i ] = NULL;
	}
}

/*****************************************************************************/
void XMLPropertyGenerator::Argument( const ByteString &rArg )
/*****************************************************************************/
{
	aArgumentList.Insert( new ByteString( rArg ), LIST_APPEND );
}

/*****************************************************************************/
void XMLPropertyGenerator::ReplaceArguments( ByteString &rText )
/*****************************************************************************/
{
	while ( aArgumentList.Count()) {
		ByteString sArgument( *aArgumentList.GetObject(( ULONG ) 0 ));
		delete aArgumentList.Remove(( ULONG ) 0 );

		ByteString sExchange( "{" );
		if ( sArgument.Search( "\"" ) != STRING_NOTFOUND )
			sExchange += sArgument.GetToken( 1, '\"' );
		else
			sExchange += sArgument.GetToken( 1, '\'' );
		sExchange += "}";

		rText.SearchAndReplace( sArgument, sExchange );
	}
}

