/*************************************************************************
 *
 *  $RCSfile: registerucb.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _UCBHELPER_REGISTERUCB_HXX_
#define _UCBHELPER_REGISTERUCB_HXX_

#include <vector>

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_
#include <com/sun/star/uno/Sequence.h>
#endif
#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

namespace com { namespace sun { namespace star {
	namespace lang { class XMultiServiceFactory; }
	namespace ucb {
		struct ContentProviderServiceInfo2;
		class XContentProvider;
		class XContentProviderManager;
	}
	namespace uno { class RuntimeException; }
} } }

//============================================================================
namespace ucb {

/** Information about a registered content provider, passed from
	<method>registerAtUcb</method> to <method>deregisterFromUcb</method>.
 */
struct ContentProviderRegistrationInfo
{
	/** The registered content provider (or null if registration failed).
	 */
	com::sun::star::uno::Reference< com::sun::star::ucb::XContentProvider >
	    m_xProvider;

	/** The arguments the content provider was instantiated with.
	 */
	rtl::OUString m_aArguments;

	/** The URL template the content provider is registered on.
	 */
	rtl::OUString m_aTemplate;

	inline
	ContentProviderRegistrationInfo(
		com::sun::star::uno::Reference<
		        com::sun::star::ucb::XContentProvider > const &
		    rTheProvider,
		rtl::OUString const & rTheArguments,
		rtl::OUString const & rTheTemplate);
};

inline
ContentProviderRegistrationInfo::ContentProviderRegistrationInfo(
	com::sun::star::uno::Reference<
	        com::sun::star::ucb::XContentProvider > const &
	    rTheProvider,
	rtl::OUString const & rTheArguments,
	rtl::OUString const & rTheTemplate):
	m_xProvider(rTheProvider),
	m_aArguments(rTheArguments),
	m_aTemplate(rTheTemplate)
{}

//============================================================================
/** Register content providers at a Universal Content Broker.

	@param rUcb  A Universal Content Broker.  May be null, which is only
	useful if the content providers are
	<type>XParamterizedContentProvider</type>s.

	@param rFactory  A factory through which to obtain the various content
	provider services.

	@param rProviders  A list of content providers to register.

	@param pResults  If not null, an entry is appended to this output
	parameter for every entry in rProviders.
 */
void
registerAtUcb(
    com::sun::star::uno::Reference<
	        com::sun::star::ucb::XContentProviderManager > const &
	    rUcb,
	com::sun::star::uno::Reference<
	        com::sun::star::lang::XMultiServiceFactory > const &
	    rFactory,
	com::sun::star::uno::Sequence<
	        com::sun::star::ucb::ContentProviderServiceInfo2 > const &
	    rProviders,
	std::vector< ContentProviderRegistrationInfo > * pResults)
	throw (com::sun::star::uno::RuntimeException);

//============================================================================
/** Deregister content providers from a Universal Content Broker.

	@param rUcb  A Universal Content Broker.  May be null, which is only
	useful if the content providers are
	<type>XParamterizedContentProvider</type>s.

	@param rProviders  A list of content providers to deregister.
 */
void
deregisterFromUcb(
    com::sun::star::uno::Reference<
	        com::sun::star::ucb::XContentProviderManager > const &
	    rUcb,
	std::vector< ContentProviderRegistrationInfo > const & rProviders)
	throw (com::sun::star::uno::RuntimeException);

}

#endif // _UCBHELPER_REGISTERUCB_HXX_

