/*************************************************************************
 *
 *  $RCSfile: burdclas.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <idl.hxx>

#include <iostream.h>
#include <fstream.h>
#include <sys/types.h>

#include <idlext.hxx>
#include "beurd.hxx"

// Convert an enum Direction value to a char *
//
char *direction_to_string(AST_Argument::Direction d)
{
	switch (d)
	{
		case AST_Argument::dir_IN:
			return "in";
		case AST_Argument::dir_OUT:
			return "out";
		case AST_Argument::dir_INOUT:
			return "inout";
		default:
			return "";
	}
}

// Convert a value from the enum AST_Expression::ExprType to a char *
//
char *exprtype_to_string(AST_Expression::ExprType et)
{
	switch (et)
	{
		case AST_Expression::EV_short:
			return "short";
		case AST_Expression::EV_ushort:
			return "unsigned short";
		case AST_Expression::EV_long:
			return "long";
		case AST_Expression::EV_ulong:
			return "unsigned long";
		case AST_Expression::EV_hyper:
			return "hyper";
		case AST_Expression::EV_uhyper:
			return "unsigned hyper";
		case AST_Expression::EV_float:
			return "float";
		case AST_Expression::EV_double:
			return "double";
		case AST_Expression::EV_char:
			return "char";
		case AST_Expression::EV_octet:
			return "octet";
		case AST_Expression::EV_byte:
			return "byte";
		case AST_Expression::EV_bool:
			return "boolean";
		case AST_Expression::EV_string:
			return "string";
		case AST_Expression::EV_any:
			return "UNO::Any";
		case AST_Expression::EV_type:
			return "Type";
		case AST_Expression::EV_void:
			return "void";
		case AST_Expression::EV_none:
			return "none";
		default:
			internal("exprtype_to_string: cannot convert ExprType %d", et);
			return "";
	}
}

void exprvalue_to_string(AST_Expression::AST_ExprValue *ev, char* value)
{
	switch (ev->et) 
	{
		case AST_Expression::EV_short:
			
			sprintf(value, "%d", ev->u.sval);
			break;
		case AST_Expression::EV_ushort:
			sprintf(value, "%u", ev->u.usval);
			break;
		case AST_Expression::EV_long:
			sprintf(value, "%d", ev->u.lval);
			break;
		case AST_Expression::EV_ulong:
			sprintf(value, "%u", ev->u.ulval);
			break;
		case AST_Expression::EV_hyper:
			sprintf(value, "%d", ev->u.lval);
			break;
		case AST_Expression::EV_uhyper:
			sprintf(value, "%u", ev->u.ulval);
			break;
		case AST_Expression::EV_float:
			sprintf(value, "%g", ev->u.fval);
			break;
		case AST_Expression::EV_double:
			sprintf(value, "%g", ev->u.dval);
			break;
		case AST_Expression::EV_char:
			sprintf(value, "%c", ev->u.cval);
			break;
		case AST_Expression::EV_octet:
			sprintf(value, "%o", ev->u.oval);
			break;
		case AST_Expression::EV_byte:
			sprintf(value, "%o", ev->u.oval);
			break;
		case AST_Expression::EV_bool:
			if (ev->u.bval == sal_True)
				sprintf(value, "%s", "sal_True");
			else
				sprintf(value, "%s", "sal_False");
			break;
		case AST_Expression::EV_string:
			if (ev->u.strval != NULL)
				sprintf(value, "%s", ev->u.strval->getStr());
		case AST_Expression::EV_any:
		case AST_Expression::EV_type:
		case AST_Expression::EV_none:
		case AST_Expression::EV_void:
			break;
	}
}

AST_Type *td2t(AST_Type *td)
{
	AST_Type *ret = td;
	if(td->node_type() != AST_Decl::NT_typedef) return td;

	while(ret->node_type() == AST_Decl::NT_typedef)
		ret = AST_Typedef::narrow_from_decl(ret)->base_type();

	return ret;
}

AST_Type *sq2t(AST_Type *td)
{
	AST_Type *ret = td;
	if(td->node_type() != AST_Decl::NT_sequence) return td;

	while(ret->node_type() == AST_Decl::NT_sequence)
		ret = AST_Sequence::narrow_from_decl(ret)->base_type();

	return ret;
}

sal_Bool isMarshalType(AST_Type* &type)
{
	AST_Type *t = type;

	while (t->node_type() == AST_Decl::NT_typedef ||
			t->node_type() == AST_Decl::NT_sequence)
	{
		if (t->node_type() == AST_Decl::NT_typedef)
			t = td2t(t);
		
		if (t->node_type() == AST_Decl::NT_sequence)
			t = sq2t(t);
	}

	type = t;
	if (t->node_type() == AST_Decl::NT_struct ||
		t->node_type() == AST_Decl::NT_interface)
		return sal_True;
	else
		return sal_False;
}

// Dump the supplied AST_ExprValue to the ostream o
//
void dump_expr_val(ostream &o, AST_Expression::AST_ExprValue *ev)
{
	switch (ev->et)
	{
		case AST_Expression::EV_short:
			o << ev->u.sval;
			break;
		case AST_Expression::EV_ushort:
			o << ev->u.usval;
			break;
		case AST_Expression::EV_long:
			o << ev->u.lval;
			break;
		case AST_Expression::EV_ulong:
			o << ev->u.ulval;
			break;
		case AST_Expression::EV_hyper:
			o << ev->u.lval;
			break;
		case AST_Expression::EV_uhyper:
			o << ev->u.ulval;
			break;
		case AST_Expression::EV_float:
			o << ev->u.fval;
			break;
		case AST_Expression::EV_double:
			o << ev->u.dval;
			break;
		case AST_Expression::EV_char:
			o << ev->u.cval;
			break;
		case AST_Expression::EV_octet:
			o << ev->u.oval;
			break;
		case AST_Expression::EV_byte:
			o << ev->u.oval;
			break;
		case AST_Expression::EV_bool:
			o << (ev->u.bval == sal_True) ? "sal_True" : "sal_False";
			break;
		case AST_Expression::EV_string:
			o << ev->u.strval;
		case AST_Expression::EV_any:
		case AST_Expression::EV_type:
		case AST_Expression::EV_none:
		case AST_Expression::EV_void:
			break;
	}
}

// Dump this binary AST_Expression node to the ostream o
//
void dump_binary_expr(ostream &o, char *s, AST_Expression *n1, AST_Expression *n2)
{
	if (n1 != NULL) n1->dump(o);
	o << " " << s << " ";
	if (n2 != NULL) n2->dump(o);
}

// Dump this unary AST_Expression node to the ostream o
//
void dump_unary_expr(ostream &o, char *s, AST_Expression *e)
{
	o << s;
	e->dump(o);
}

// BE_PredefinedType
//
BE_PredefinedType::BE_PredefinedType()
{
}

BE_PredefinedType::BE_PredefinedType(AST_PredefinedType::PredefinedType t,
									 UTL_ScopedName *n, UTL_StrList *p)
	:	AST_PredefinedType(t, n, p),
		AST_Decl(AST_Decl::NT_pre_defined, n, p)
{
}

const char* BE_PredefinedType::getRelName()
{
	return(local_name()->getStr());	
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_PredefinedType, AST_PredefinedType, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_PredefinedType);

// BE_Object
//
BE_Object::BE_Object()
{
}

BE_Object::BE_Object(UTL_ScopedName *n, UTL_StrList *p, char* name)
	:	AST_Object(n, p, name),
		AST_Decl(AST_Decl::NT_object, n, p),
		UTL_Scope(AST_Decl::NT_object)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Object, AST_Object, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Object);
IMPL_NARROW_FROM_SCOPE(BE_Object);

// BE_Serivce
//
BE_Service::BE_Service()
{
}

BE_Service::BE_Service(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Service(n, p),
		AST_Decl(AST_Decl::NT_service, n, p),
		UTL_Scope(AST_Decl::NT_service)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Service, AST_Service, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Service);
IMPL_NARROW_FROM_SCOPE(BE_Service);

// BE_InterfaceMember
//
BE_InterfaceMember::BE_InterfaceMember()
{
}

BE_InterfaceMember::BE_InterfaceMember(const sal_uInt16 fOptional, UTL_ScopedName *n, UTL_StrList *p)
	:	AST_InterfaceMember(fOptional, n, p),
		AST_Decl(AST_Decl::NT_interface_member, n, p),
		UTL_Scope(AST_Decl::NT_interface_member)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_InterfaceMember, AST_InterfaceMember, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_InterfaceMember);

// BE_ServiceMember
//
BE_ServiceMember::BE_ServiceMember()
{
}

BE_ServiceMember::BE_ServiceMember(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_service_member, n, p),
		UTL_Scope(AST_Decl::NT_service_member)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_ServiceMember, AST_ServiceMember, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_ServiceMember);

// BE_Observes
//
BE_Observes::BE_Observes()
{
}

BE_Observes::BE_Observes(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_observes, n, p),
		UTL_Scope(AST_Decl::NT_observes)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Observes, AST_Observes, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Observes);

// BE_Needs
//
BE_Needs::BE_Needs()
{
}

BE_Needs::BE_Needs(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_needs, n, p),
		UTL_Scope(AST_Decl::NT_needs)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Needs, AST_Needs, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Needs);

// BE_Module
//
BE_Module::BE_Module()
{
}

BE_Module::BE_Module(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_module, n, p),
		UTL_Scope(AST_Decl::NT_module)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Module, AST_Module, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Module);
IMPL_NARROW_FROM_SCOPE(BE_Module);

// BE_Constants
//
BE_Constants::BE_Constants()
{
}

BE_Constants::BE_Constants(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_constants, n, p),
		UTL_Scope(AST_Decl::NT_constants)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Constants, AST_Constants, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Constants);
IMPL_NARROW_FROM_SCOPE(BE_Constants);

// BE_Mapping
//
BE_Mapping::BE_Mapping()
{
}

BE_Mapping::BE_Mapping(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_mapping, n, p),
		UTL_Scope(AST_Decl::NT_mapping)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Mapping, AST_Mapping, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Mapping);
IMPL_NARROW_FROM_SCOPE(BE_Mapping);

// BE_Interface
//
BE_Interface::BE_Interface()
{
	m_forwardDeclExists	= sal_False;
	m_unicast			= sal_True;
	m_upcalls			= sal_False;
	m_proxy_defined		= sal_False;
	m_pSmartProxy	   	= 0;
}

BE_Interface::BE_Interface(UTL_ScopedName *n, AST_Interface **ih, long nih,
						UTL_StrList *p, char* uik, double ver, sal_Bool vrt, char* ident)
	:	AST_Interface(n, ih, nih, p, uik, ver, vrt, ident),
		AST_Decl(AST_Decl::NT_interface, n, p),
		UTL_Scope(AST_Decl::NT_interface)
{
	m_forwardDeclExists	= sal_False;
	m_unicast			= sal_True;
	m_upcalls			= sal_False;
	m_proxy_defined		= sal_False;
	m_pSmartProxy		= 0;
}

BE_Interface::~BE_Interface()
{
	if(m_pSmartProxy != NULL) delete(m_pSmartProxy);
}

void BE_Interface::setProxyDefined(sal_Bool proxy)
{
	m_proxy_defined = proxy;
}

sal_Bool BE_Interface::isProxyDefined()
{
	return m_proxy_defined;
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Interface, AST_Interface, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Interface);
IMPL_NARROW_FROM_SCOPE(BE_Interface);

/*
 * BE_InterfaceFwd
 */
BE_InterfaceFwd::BE_InterfaceFwd()
{

}

BE_InterfaceFwd::BE_InterfaceFwd(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_InterfaceFwd(n, p),
		AST_Decl(AST_Decl::NT_interface_fwd, n, p)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_InterfaceFwd, AST_InterfaceFwd, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_InterfaceFwd);

// BE_Exception
//
BE_Exception::BE_Exception()
{
}

BE_Exception::BE_Exception(UTL_ScopedName *n, AST_Exception **ih, 
							long nih, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_except, n, p),
		AST_Exception(n, ih, nih, p),
		UTL_Scope(AST_Decl::NT_except)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Exception, AST_Exception, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Exception);
IMPL_NARROW_FROM_SCOPE(BE_Exception);

// BE_Structure
//
BE_Structure::BE_Structure()
	:	m_checked(sal_False)
{
}

BE_Structure::BE_Structure(UTL_ScopedName *n, AST_Structure **ih, 
							long nih, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_struct, n, p),
		AST_Structure(n, ih, nih, p),
		UTL_Scope(AST_Decl::NT_struct),
		m_checked(sal_False)
{
}

void BE_Structure::setChecked(sal_Bool ch)
{
	m_checked = ch;
}	

sal_Bool BE_Structure::getChecked()
{
	return(m_checked); 
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_Structure, AST_Structure, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Structure);
IMPL_NARROW_FROM_SCOPE(BE_Structure);

// BE_Enum
//
BE_Enum::BE_Enum()
{
}

BE_Enum::BE_Enum(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Enum(n, p),
		AST_Decl(AST_Decl::NT_enum, n, p),
		UTL_Scope(AST_Decl::NT_enum)
{
}

void BE_Enum::setChecked(sal_Bool ch)
{
	m_checked = ch;
}	

sal_Bool BE_Enum::getChecked()
{
	return(m_checked); 
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_Enum, AST_Enum, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Enum);
IMPL_NARROW_FROM_SCOPE(BE_Enum);

// BE_Operation
//
BE_Operation::BE_Operation()
{
}

BE_Operation::BE_Operation(AST_Type *rt, AST_Operation::Flags fl,
							IFlag ifl, char* local,	sal_Bool cons,
							UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Operation(rt, fl, ifl, local, cons, n, p),
		AST_Decl(AST_Decl::NT_op, n, p),
		UTL_Scope(AST_Decl::NT_op)
{
	m_hasArgs         = sal_False;
	m_hasInInoutArgs  = sal_False;
	m_hasOutInoutArgs = sal_False;
	m_hasInoutArgs    = sal_False;
	m_isGetAttribute  = sal_False;
	m_isSetAttribute  = sal_False;

	// return type != void ?
	//
	m_returnVal = return_type()->node_type() != NT_pre_defined ||
					BE_PredefinedType::narrow_from_decl(return_type())->pt()
					!= AST_PredefinedType::PT_void;
}

// BE_Operation::add_argument() keeps track of whether an operation
// has in, out, or inout args:
//
AST_Argument* BE_Operation::add_argument(AST_Argument *a)
{
	m_hasArgs = sal_True;

	if (a->direction() == AST_Argument::dir_IN ||
		a->direction() == AST_Argument::dir_INOUT)
	{
		m_hasInInoutArgs = sal_True;
	}

	if (a->direction() == AST_Argument::dir_OUT ||
		a->direction() == AST_Argument::dir_INOUT)
	{
		m_hasOutInoutArgs = sal_True;
	}

	if (a->direction() == AST_Argument::dir_INOUT)
	{
		m_hasInoutArgs = sal_True;
	}

	return AST_Operation::add_argument(a);
}

// non-void operation?
//
sal_Bool BE_Operation::hasReturnVal()
{
	return m_returnVal;
}

// not an empty signature?
//
sal_Bool BE_Operation::hasArgs()
{
	return m_hasArgs;
}

// has in or inout args?
//
sal_Bool BE_Operation::hasInInoutArgs()
{
	return m_hasInInoutArgs;
}

// has out or inout args?
//
sal_Bool BE_Operation::hasOutInoutArgs()
{
	return m_hasOutInoutArgs;
}

// has inout args?
//
sal_Bool BE_Operation::hasInoutArgs()
{
	return m_hasInoutArgs;
}

// get attribute operation?
//
sal_Bool BE_Operation::isGetAttribute()
{
	return m_isGetAttribute;
}

// set attribute operation?
//
sal_Bool BE_Operation::isSetAttribute()
{
	return m_isSetAttribute;
}

void BE_Operation::isGetAttribute(sal_Bool v)
{
	m_isGetAttribute = v;
}

void BE_Operation::isSetAttribute(sal_Bool v)
{
	m_isSetAttribute = v;
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Operation, AST_Operation, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Operation);
IMPL_NARROW_FROM_SCOPE(BE_Operation);


// BE_Field
//
BE_Field::BE_Field()
{
}

BE_Field::BE_Field(AST_Type *ft, UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Field(ft, n, p),
		AST_Decl(AST_Decl::NT_field, n, p)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Field, AST_Field, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Field);

// BE_Argument
//
BE_Argument::BE_Argument()
{
}

BE_Argument::BE_Argument(AST_Argument::Direction d, AST_Type *ft,
						 UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Argument(d, ft, n, p),
		AST_Field(AST_Decl::NT_argument, ft, n, p),
		AST_Decl(AST_Decl::NT_argument, n, p)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Argument, AST_Argument, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Argument);

// BE_Attribute
//
BE_Attribute::BE_Attribute()
{
	m_get = 0;
	m_set = 0;
}
BE_Attribute::BE_Attribute(sal_uInt16 attrFlags, IFlag ifl,
							AST_Type *ft, UTL_ScopedName *n,
							UTL_StrList *p)
	:	AST_Attribute(attrFlags, ifl, ft, n, p),
		AST_Field(AST_Decl::NT_attr, ft, n, p),
		AST_Decl(AST_Decl::NT_attr, n, p)
{
	AST_Type *v = new BE_PredefinedType(AST_PredefinedType::PT_void, new UTL_ScopedName("void", NULL), 0);
	m_get = new BE_Operation(field_type(), AST_Operation::OP_noflags,
							ifl, NULL, sal_True, new UTL_ScopedName("", NULL), 0);
	m_set = new BE_Operation(v, AST_Operation::OP_noflags,
							ifl, NULL, sal_False, new UTL_ScopedName("", NULL), 0);
	m_get->isGetAttribute(sal_True);
	m_set->isSetAttribute(sal_True);
	m_get->set_name(new UTL_ScopedName(Identifier(*local_name()), NULL));
	m_set->set_name(new UTL_ScopedName(Identifier(*local_name()), NULL));
	BE_Argument *arg = new BE_Argument(AST_Argument::dir_IN, field_type(), new UTL_ScopedName("", NULL), 0);
	arg->set_name(new UTL_ScopedName(Identifier(*local_name() + "_"), NULL));
	m_set->add_argument(arg);
	m_set->add_to_scope(arg);
	m_set->add_to_referenced(arg, sal_False);
}

BE_Attribute::~BE_Attribute()
{
	delete m_get;
	delete m_set;
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Attribute, AST_Attribute, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Attribute);

// BE_Union
//
BE_Union::BE_Union()
{
}

BE_Union::BE_Union(AST_ConcreteType *dt, UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Union(dt, n, p),
		AST_Decl(AST_Decl::NT_union, n, p),
		UTL_Scope(AST_Decl::NT_union)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Union, AST_Union, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Union);
IMPL_NARROW_FROM_SCOPE(BE_Union);

// BE_UnionBranch
//
BE_UnionBranch::BE_UnionBranch()
{
}

BE_UnionBranch::BE_UnionBranch(AST_UnionLabel *lab, AST_Type *ft,
								UTL_ScopedName *n, UTL_StrList *p)
	:	AST_UnionBranch(lab, ft, n, p),
		AST_Field(AST_Decl::NT_union_branch, ft, n, p),
		AST_Decl(AST_Decl::NT_union_branch, n, p)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_UnionBranch, AST_UnionBranch, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_UnionBranch);

// BE_UnionLabel
//
BE_UnionLabel::BE_UnionLabel()
{
}

BE_UnionLabel::BE_UnionLabel(AST_UnionLabel::UnionLabel ul,
							 AST_Expression *v)
	:	AST_UnionLabel(ul, v)
{
}

// Narrowing
/*
	IMPL_NARROW_METHODS2(BE_UnionLabel, AST_UnionLabel, BE_Decl);
	IMPL_NARROW_FROM_DECL(BE_UnionLabel);
  */

// BE_Constant
//
BE_Constant::BE_Constant()
	:	m_hasTypeMapping(sal_False),
		m_typeMapping(NULL)

{
}

BE_Constant::BE_Constant(AST_Expression::ExprType et,
						 AST_Expression *v,
						 UTL_ScopedName *n,
						 UTL_StrList *p)
	:	AST_Constant(et, v, n, p),
		AST_Decl(AST_Decl::NT_const, n, p),
		m_hasTypeMapping(sal_False),
		m_typeMapping(NULL)
{
}

const char* BE_Constant::getRelName()
{
	return(local_name()->getStr());	
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_Constant, AST_Constant, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Constant);

// BE_expression
//
BE_ExprVal::BE_ExprVal(UTL_ScopedName *n) : AST_Expression(n) {}

BE_ExprVal::BE_ExprVal(AST_Expression *b, AST_Expression::ExprType t)
	:	AST_Expression(b, t)
{
}

BE_ExprVal::BE_ExprVal(AST_Expression::ExprComb c,
						AST_Expression *v1,
						AST_Expression *v2)
	:	AST_Expression(c, v1, v2)
{
}

BE_ExprVal::BE_ExprVal(long l) : AST_Expression(l) {}

BE_ExprVal::BE_ExprVal(long l, AST_Expression::ExprType t)
	:	AST_Expression(l, t)
{
}

BE_ExprVal::BE_ExprVal(unsigned long l) : AST_Expression(l) {}

BE_ExprVal::BE_ExprVal(String *s) : AST_Expression(s) {}

BE_ExprVal::BE_ExprVal(char c) : AST_Expression(c) {}

BE_ExprVal::BE_ExprVal(double d) : AST_Expression(d) {}

// Narrowing
/*
  IMPL_NARROW_METHODS2(BE_ExprVal, AST_Expression, BE_Decl);
  IMPL_NARROW_FROM_DECL(BE_ExprVal);
  */

BE_EnumVal::BE_EnumVal()
	:	m_checked(sal_False)
{
}

BE_EnumVal::BE_EnumVal(unsigned long v, UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Constant(AST_Expression::EV_ulong,
					 AST_Decl::NT_enum_val,
					 new AST_Expression(v),
					 n,
					 p),
		AST_Decl(AST_Decl::NT_enum_val, n, p),
		m_checked(sal_False)
{
}

void BE_EnumVal::setChecked(sal_Bool ch)
{
	m_checked = ch;
}	

sal_Bool BE_EnumVal::getChecked()
{
	return(m_checked); 
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_EnumVal, AST_EnumVal, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_EnumVal);

// BE_Array
//
BE_Array::BE_Array()
{
}

BE_Array::BE_Array(UTL_ScopedName *n, unsigned long ndims, UTL_ExprList *dims)
	:	AST_Array(n, ndims, dims),
		AST_Decl(AST_Decl::NT_array, n, NULL)
{
}

int BE_Array::num_elems()
{
	int numElems = 1;

	for(unsigned long i = 0; i < n_dims(); i++)
	{
		if(dims()[i]->ev()->u.ulval == 0)
			warning("array with zero dimensions declared");

		numElems *= dims()[i]->ev()->u.ulval;
	}
	return numElems;
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Array, AST_Array, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Array);

// BE_Sequence
//
BE_Sequence::BE_Sequence()
{
	m_relName = setRelName("");
}

BE_Sequence::BE_Sequence(AST_Expression *v, AST_Type *t)
	:	AST_Sequence(v, t),
		AST_Decl(AST_Decl::NT_sequence,
				new UTL_ScopedName(Identifier("sequence"), NULL),
				NULL)
{
	m_relName = setRelName("");
}

String BE_Sequence::getFullName(String name)
{
	String seq;
	
	if (name.getLength() > 0)
	{
		seq = name;
		seq += "[]";		
	} else
	{
		seq += "[]";
	}
	
	AST_Type *t = td2t(base_type());
	switch(t->node_type())
	{
		case NT_pre_defined :
			seq += *(t->local_name());
			break;
		case NT_struct :
			seq += *(t->local_name());
			break;
		case NT_string :
			seq += *(t->local_name());
			break;
		case NT_sequence :
            seq = BE_Sequence::narrow_from_decl(t)->getFullName(seq);
			break;
		case NT_enum :
			seq += *(t->local_name());
			break;
		case NT_interface :
			seq += *(t->local_name());
			break;
		default:			
			break;
	}

	return(seq);
}		

String BE_Sequence::getTypeName()
{
	return(getFullName(""));	
}	

String BE_Sequence::setRelName(String name)
{
	String seq;
	
	if (name.getLength() > 0)
	{
		seq = name;
		seq += "[]";		
	} else
	{
		seq += "[]";
	}
	
	AST_Type *t = td2t(base_type());
	switch(t->node_type())
	{
		case NT_pre_defined :
			seq += dd(t)->getRelName();
			break;
		case NT_struct :
			seq += dd(t)->getRelName();
			break;
		case NT_string :
			seq += dd(t)->getRelName();
			break;
		case NT_sequence :
            seq = BE_Sequence::narrow_from_decl(t)->setRelName(seq);
			break;
		case NT_enum :
			seq += dd(t)->getRelName();
			break;
		case NT_interface :
			seq += dd(t)->getRelName();
			break;
		default:			
			break;
	}

	return(seq);	
}	

const char* BE_Sequence::getRelName()
{
	return(m_relName.getStr());	
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_Sequence, AST_Sequence, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Sequence);

// BE_String
//
BE_String::BE_String()
{
}

BE_String::BE_String(AST_Expression *v)
	:	AST_String(v),
		AST_Decl(AST_Decl::NT_string,
				new UTL_ScopedName(Identifier("string"), NULL),
				NULL)
{
}

const char* BE_String::getRelName()
{
	return(local_name()->getStr());	
}	

// Narrowing
IMPL_NARROW_METHODS2(BE_String, AST_String, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_String);

BE_Typedef::BE_Typedef()
{
}

BE_Typedef::BE_Typedef(AST_Type *bt, UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Typedef(bt, n, p),
		AST_Decl(AST_Decl::NT_typedef, n, p)
{
}

// Narrowing
IMPL_NARROW_METHODS2(BE_Typedef, AST_Typedef, BE_Decl);
IMPL_NARROW_FROM_DECL(BE_Typedef);

