/*************************************************************************
 *
 *  $RCSfile: burddecl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <idl.hxx>
#include <idlext.hxx>
#include "beurd.hxx"

using namespace rtl;
//
// this file contains methods which walk trough an IDL specification
// and produce C++ code, as well as methods to produce marshaling code,
// operation signatures etc.
//
void BE_Decl::dumpReflection(RegistryKey& rKey)
{
	UTL_ScopeActiveIterator	*i = NULL;
	BE_Decl					*d = NULL;

	if (node_type() == AST_Decl::NT_module && local_name() == NULL)
	{
		dumpConstantPool(rKey);		
	}

	// Used to iterate through the AST:
	i = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_both);
	while (!(i->is_done()))
	{
		d = narrow_from_decl(i->item());

		if (d->in_main_file())
		{
			if (d->node_type() == AST_Decl::NT_module
				|| d->node_type() == AST_Decl::NT_constants
				|| d->node_type() == AST_Decl::NT_mapping
				|| d->node_type() == AST_Decl::NT_object
				|| d->node_type() == AST_Decl::NT_op
				|| d->node_type() == AST_Decl::NT_attr
				|| d->node_type() == AST_Decl::NT_argument			
				|| d->node_type() == AST_Decl::NT_service
				|| d->node_type() == AST_Decl::NT_interface_member
				|| d->node_type() == AST_Decl::NT_service_member
				|| d->node_type() == AST_Decl::NT_observes
				|| d->node_type() == AST_Decl::NT_needs
				|| d->node_type() == AST_Decl::NT_interface
				|| d->node_type() == AST_Decl::NT_interface_fwd
				|| d->node_type() == AST_Decl::NT_typedef
				|| d->node_type() == AST_Decl::NT_field
				|| d->node_type() == AST_Decl::NT_except
				|| d->node_type() == AST_Decl::NT_struct
				|| d->node_type() == AST_Decl::NT_union
				|| d->node_type() == AST_Decl::NT_enum
				|| d->node_type() == AST_Decl::NT_pre_defined)
			{
				// dump the associated reflection code:
				//
				d->dumpReflection(rKey);
			}
		}
		i->next();
	}
	
	delete i;	
}

void BE_Decl::dumpConstantPool(RegistryKey& rKey)
{
	UTL_ScopeActiveIterator	*iter = NULL;
	AST_Decl				*d = NULL;

	sal_uInt16	nConst= getNodeCount(AST_Decl::NT_const);

	if (nConst)
	{
		RegistryTypeWriter aBlop(pWriterLoader->getApi(), 
								 RT_TYPE_MODULE, 
								 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8), 
								 OUString(), nConst, 0, 0);

		sal_uInt16 	index=0;
		iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
		while(!iter->is_done()) 
		{
			d = iter->item();
			
			if (d->node_type() == NT_const)
			{
				BE_Constant::narrow_from_decl(d)->dumpBlop(aBlop, index);

				index++;		
			}
			iter->next();
		}
		delete iter;

		const sal_uInt8* 	pBlop = aBlop.getBlop();
		sal_uInt32			aBlopSize = aBlop.getBlopSize();

		if (rKey.setValue(OUString(), 
						  RG_VALUETYPE_BINARY, 
						  (RegValue)pBlop, aBlopSize))
		{
			fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
				   getFullName(), rKey.getRegistryName());
		}
	}
}

unsigned short BE_Decl::getNodeCount(AST_Decl::NodeType nt)
{
	UTL_ScopeActiveIterator	*i = NULL;
	BE_Decl					*d = NULL;
	unsigned short			count = 0;
	
	i = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_both);
	while (!(i->is_done()))
	{
		d = narrow_from_decl(i->item());

		if (d->in_main_file() &&
			d->node_type() == nt)
		{
			count++;
		}
		i->next();
	}
	
	delete i;	
	return(count);	
}

// print indentation white space:
//
UTL_Indenter *BE_Decl::indent()
{
	return IDL_Global::idl_global()->indent();
}

// increment the indenter:
//
void BE_Decl::inc()
{
	IDL_Global::idl_global()->indent()->increase();
}

// increment the indenter:
//
void BE_Decl::dec()
{
	IDL_Global::idl_global()->indent()->decrease();
}

// return the indenter:
//
void BE_Decl::indent(ostream& o)
{
	IDL_Global::idl_global()->indent()->skip_to(o);
}


//---- Utilities ---------------------------------------------------------------
//

BE_Decl::BE_Decl()
{
	// set the scoped name (module::interface::...) of this node:
	setNamePref();
//	setScope(getScopeName(file_name()));
	setFullName();
	checkDoku();
}

BE_Decl::BE_Decl(AST_Decl::NodeType nt,
				 UTL_ScopedName *n,
				 UTL_StrList *p)
	:	AST_Decl(nt, n, p)
{
   // set the scoped name (module::interface::...) of this node:
	setNamePref();
//	setScope(getScopeName(file_name()));
	setFullName();
	checkDoku();
}

BE_Decl::~BE_Decl()
{
	//if (scopedName_ != NULL) delete(scopedName_);
	if(namePref_ != NULL) delete(namePref_);
	if(m_fullName != NULL) free((void*)m_fullName);
}

BE_Decl::tKind BE_Decl::dir2dir(AST_Argument::Direction d)
{
	return d == AST_Argument::dir_IN ?
		eIn : d == AST_Argument::dir_OUT ?
			eOut : eInOut;
}

BE_Decl::tKind BE_Decl::dir2dir(AST_Decl *decl)
{
	AST_Argument::Direction d = AST_Argument::narrow_from_decl(decl)->direction();

	return d == AST_Argument::dir_IN ?
		eIn : d == AST_Argument::dir_OUT ?
			eOut : eInOut;
}

// convenience:
//
BE_Decl *BE_Decl::dd(AST_Decl *a)
{
	return BE_Decl::narrow_from_decl(a);
}

// convenience:
//
BE_Decl *BE_Decl::ds(UTL_Scope *s)
{
	return BE_Decl::narrow_from_scope(s);
}

// sets the extended name of a node, e.g. module1::class1::method4:
//
void BE_Decl::setNamePref()
{
	UTL_IdListActiveIterator *iter;
	int                       len =0;
	int                       count;

	// handle global symbols:
	if(global())
	{
		namePref_ = new char [strlen("") +1];
		strcpy(namePref_, "");
		return;
	}

	// calculate needed buffer space:
	//
	iter = new UTL_IdListActiveIterator(name());
	count = 0;

	Identifier id;
	while(!iter->is_done())
	{
		id = iter->item();
		len += iter->item().getLength();
		len += iter->item().getLength() + strlen("::");

		if(++count == name()->length() -1)
			break;

		iter->next();
	}
	delete iter;

	namePref_ = new char [len +1];
	*namePref_ = '\0';

	// generate the scoped name:
	//
	iter = new UTL_IdListActiveIterator(name());
	count = 0;

	while(!iter->is_done())
	{
		strcat(namePref_, iter->item().getStr());
		strcat(namePref_, "::");

		if(++count == name()->length() -1)
			break;

		iter->next();
	}
	delete iter;
}

// returns the scope of an object, relative to the scope where it is
// being accessed:
//
char* BE_Decl::scope(BE_Decl *access)
{
	static char sc            [1024];             // the scope to be returned.
	UTL_IdListActiveIterator *soIter =0, *saIter =0;
	int                       count  =0;
	sal_Bool                   sameScope = (!access || access->global());

	// generate the scoped name:
	//
	*sc = 0;
	soIter = new UTL_IdListActiveIterator(name());
	if(access && name()->length() > 0 && access->name()->length() > 0)
	{
		saIter = new UTL_IdListActiveIterator(access->name());
		if(soIter->item() != saIter->item())
			sameScope = sal_True;
		while(count < name()->length() -1&& count < access->name()->length() -1)
		{
			if(soIter->item() != saIter->item())
			{
				::strcat(sc, soIter->item().getStr());
				::strcat(sc, "::");
			}
			++count;
			soIter->next(); saIter->next();
		}
	}

	if(count < name()->length() -1 && sameScope)
	{
		while(!soIter->is_done())
		{
			::strcat(sc, soIter->item().getStr());
			::strcat(sc, "::");
			if(++count == name()->length() -1) break;
			soIter->next();
		}
	}
	delete soIter; delete saIter;
	return sc;
}

// get the scoped name prefix:
//
char *BE_Decl::getNamePref()
{
	return namePref_;
}

// global scope?
//
sal_Bool BE_Decl::global()
{
	return name() && name()->length() == 1;
}

// check whether the accesstype is equal "dir". decl points to an AST_Argument:
//
sal_Bool BE_Decl::isAccessType(AST_Argument::Direction dir , AST_Decl *decl)
{
	return AST_Argument::narrow_from_decl(decl)->direction() == dir;
}

// check whether an AST_Decl is of predefined type. Used mainly to
// determine whether an argument can be marshaled using memcpy
// or if dump() must be issued on it:
//
sal_Bool BE_Decl::isPredefinedType(AST_Decl *decl)
{
	// AST_Typedef *td;

	// handle arguments:
	//
	if(decl->node_type() == NT_argument)
		decl = BE_Argument::narrow_from_decl(decl)->field_type();


	// handle predefined types:
	//
	if(decl->node_type() == NT_pre_defined) return sal_True;

	// handle typedefs:
	//
	if(decl->node_type() == NT_typedef)
		return td2t(AST_Type::narrow_from_decl(decl))->node_type() == NT_pre_defined;

	return sal_False;
}

// iter points at a certain position in an argument list.
// lastAccess checks whether there will be another access of type "dir"
// at a later point in the list:
//
sal_Bool BE_Decl::lastAccess(AST_Argument::Direction dir,
							UTL_ScopeActiveIterator *iter)
{
	AST_Decl *d = NULL;

	if(iter->is_done()) return sal_True;

	UTL_ScopeActiveIterator *tmpIter =
		new UTL_ScopeActiveIterator(0, UTL_Scope::IK_both);

	*tmpIter = *iter;
	tmpIter->next();

	while(!tmpIter->is_done())
	{
		d = tmpIter->item();

		if(d->node_type() == NT_argument)
		{
			if(AST_Argument::narrow_from_decl(d)->direction() == dir)
				return sal_False;

		}

		tmpIter->next();
	}

	delete tmpIter;
	return sal_True;
}

// given an IDL path name, create the according stub file path name.
// e.g.: "test.idl" -> "test"suffix.
// Called by ccClientFile() etc.
//
char *BE_Decl::buildPath(const char *path, char *suffix, sal_Bool pth)
{
	char *s = NULL;
	static char retPath [1024];

	if(!pth)
	{
#ifdef UNX
		if(s = (char*)::strrchr(path, '/')) path = s + 1;
#else
		if(s = ::strrchr(path, '\\')) path = s + 1;
#endif
	}

	strcpy(retPath, path);

	s = strrchr(retPath, '.');
	strcpy(s, suffix);
	return retPath;
}

sal_Bool BE_Decl::isAny(AST_Type *t)
{
	t = td2t(t);
	return(t->node_type() == AST_Decl::NT_pre_defined
			&& AST_PredefinedType::narrow_from_decl(t)->pt() == AST_PredefinedType::PT_any);
}

sal_Bool BE_Decl::isPseudo(AST_Type *t)
{
	t = td2t(t);
	return t->node_type() == AST_Decl::NT_pre_defined
			&& AST_PredefinedType::narrow_from_decl(t)->pt() == AST_PredefinedType::PT_pseudo;
}

String BE_Decl::getTypeName()
{
	return(*local_name());
}	

void BE_Decl::setFullName()
{
	UTL_IdListActiveIterator *iter;
	iter = new UTL_IdListActiveIterator(name());

	String 	name("/");
	while(!iter->is_done())
	{
		name += iter->item();
		iter->next();
		
		if (!iter->is_done())
			name += "/";
	}

	m_fullName = strdup(name.getStr());
	
	delete iter;
}	

const char* BE_Decl::getFullName() const
{
	return(m_fullName);	
}	

const char* BE_Decl::getRelName()
{
	return(m_fullName+1);	
}	

// ******************************************************************
// If we're looking at whitespace, followed by a star and a space,
// then skip past it.
// ******************************************************************
static char* skipFillStar(char* t)
{
	sal_Char* tOrig = t;
	
	while (*t != '\n' && (isspace(*t) || *t == '\t' || *t == '*'))
		t++;
	return t;
}

// ******************************************************************
// Examine each line.  Remove (whitespace, '*', ' ') from the front of
// each line, if present.
// ******************************************************************
static void stripStars(char* s)
{
	sal_Char* t = s;
	while (*t)
	{
		if ((*s++ = *t++) == '\n')
			t = skipFillStar(t);
	}
	*s = 0;
}

void BE_Decl::checkDoku()
{
	if (node_type() != AST_Decl::NT_string && !coreOnly && IDL_Global::idl_global()->idldoc_valid())
	{
		// don't let anyone else have this comment
		IDL_Global::idl_global()->set_idldoc_valid(0);

//		m_doku = strdup(IDL_Global::idl_global()->idldoc_text());
//		stripStars((sal_Char*)m_doku);

		sal_Char* tmp = strdup(IDL_Global::idl_global()->idldoc_text());
		stripStars(tmp);
		
		sal_Char* pTmp = tmp;
		if (*pTmp == '\n') pTmp++;
		if (pTmp[strlen(pTmp)-1] == '\n') pTmp[strlen(pTmp)-1] = '\0';

		m_doku = ::rtl::OStringToOUString(pTmp, RTL_TEXTENCODING_UTF8);
		free(tmp);
	}
}	

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS2(BE_Decl, AST_Decl, UTL_Scope)
IMPL_NARROW_FROM_DECL(BE_Decl)
IMPL_NARROW_FROM_SCOPE(BE_Decl)
