/*************************************************************************
 *
 *  $RCSfile: burdhead.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:09 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <idl.hxx>

#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <sys/types.h>

#include <idlext.hxx>
#include "beurd.hxx"

#include <rtl/ustring.hxx>

using namespace rtl;

//---- BE_PredefinedType ------------------------------------------------------
//
// produces the C++ definition of a predefined type:
//
void BE_PredefinedType::dumpReflection(RegistryKey& rKey)
{
}	

//---- BE_Object ---------------------------------------------------------------
//
// produces a object definition
//
void BE_Object::dumpReflection(RegistryKey& rKey)
{
}	

//---- BE_Service ---------------------------------------------------------------
//
// produces a service definition
//
void BE_Service::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	AST_Decl				*d;
	UTL_ScopeActiveIterator *iter;
	
	if (coreOnly)
		return;

	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	sal_uInt16	nAttr	 	= getNodeCount(AST_Decl::NT_attr);
	sal_uInt16	nSupports	= getNodeCount(AST_Decl::NT_interface_member);
	sal_uInt16	nExports	= getNodeCount(AST_Decl::NT_service_member);
	sal_uInt16	nObserves	= getNodeCount(AST_Decl::NT_observes);
	sal_uInt16	nNeeds		= getNodeCount(AST_Decl::NT_needs);
	sal_uInt16 	nRefs = nSupports + nExports + nObserves + nNeeds;

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_SERVICE,
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8),
							 OUString(), nAttr, 0, nRefs);

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));

	if (nAttr || nRefs)
	{
		sal_uInt16 	propIndex=0;
		sal_uInt16 	refIndex=0;
		iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
		while(!iter->is_done()) 
		{
			d = iter->item();
			
			switch (d->node_type())
			{
				case AST_Decl::NT_attr:
					BE_Attribute::narrow_from_decl(d)->dumpBlop(aBlop, propIndex);
					propIndex++;		
					break;
				case AST_Decl::NT_interface_member:
					BE_InterfaceMember::narrow_from_decl(d)->dumpBlop(aBlop, refIndex);
					refIndex++;		
					break;
				case AST_Decl::NT_service_member:
					BE_ServiceMember::narrow_from_decl(d)->dumpBlop(aBlop, refIndex);
					refIndex++;		
					break;
				case AST_Decl::NT_observes:
					BE_Observes::narrow_from_decl(d)->dumpBlop(aBlop, refIndex);
					refIndex++;		
					break;
				case AST_Decl::NT_needs:
					BE_Needs::narrow_from_decl(d)->dumpBlop(aBlop, refIndex);
					refIndex++;		
					break;
			}
			
			iter->next();
		}
		delete iter;
	}

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}
}	
	

//---- BE_Implements ---------------------------------------------------------------
//
// produces an interface_member definition
//
void BE_InterfaceMember::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	sal_uInt16 access = (getOptional() == AF_OPTIONAL ? RT_ACCESS_OPTIONAL : RT_ACCESS_INVALID);
	aBlop.setReferenceData(index, 
						   OStringToOUString( dd(getInterface())->getRelName(), RTL_TEXTENCODING_UTF8),
						   RT_REF_SUPPORTS,
						   getDoku(), access);
}	

//---- BE_ServiceMember ---------------------------------------------------------------
//
// produces a service_member definition
//
void BE_ServiceMember::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	aBlop.setReferenceData(index, 
						   OStringToOUString( dd(getService())->getRelName(), RTL_TEXTENCODING_UTF8),
						   RT_REF_EXPORTS,
						   getDoku());
}	

//---- BE_Observes ---------------------------------------------------------------
//
// produces an observes definition
//
void BE_Observes::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	aBlop.setReferenceData(index, 
						   OStringToOUString( dd(getInterface())->getRelName(), RTL_TEXTENCODING_UTF8),
						   RT_REF_OBSERVES,
						   getDoku());
}	

//---- BE_Needs ---------------------------------------------------------------
//
// produces a needs definition
//
void BE_Needs::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	aBlop.setReferenceData(index, 
						   OStringToOUString( dd(getService())->getRelName(), RTL_TEXTENCODING_UTF8),
						   RT_REF_NEEDS,
						   getDoku());
}	

//---- BE_Mapping ---------------------------------------------------------------
//
void BE_Mapping::dumpReflection(RegistryKey& rKey)
{
	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Module ---------------------------------------------------------------
//
// produces a module definition, i.e., a C++ namespace or class definition
//
void BE_Module::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	AST_Decl				*d;
	UTL_ScopeActiveIterator *iter;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	sal_uInt16	nConst= getNodeCount(AST_Decl::NT_const);

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_MODULE,
							 OStringToOUString( getRelName(), RTL_TEXTENCODING_UTF8),
							 OUString(), nConst, 0, 0);

	aBlop.setDoku(getDoku());
	aBlop.setFileName( OStringToOUString( *IDL_Global::idl_global()->main_filename(), 
										  RTL_TEXTENCODING_UTF8) );

	if (nConst)
	{
		sal_uInt16 	index=0;
		iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
		while(!iter->is_done()) 
		{
			d = iter->item();
			
			if (d->node_type() == NT_const)
			{
				BE_Constant::narrow_from_decl(d)->dumpBlop(aBlop, index);

				index++;		
			}
			iter->next();
		}
		delete iter;
	}

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Constants ---------------------------------------------------------------
//
// produces a constants definition
//
void BE_Constants::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	AST_Decl				*d;
	UTL_ScopeActiveIterator *iter;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	sal_uInt16	nConst= getNodeCount(AST_Decl::NT_const);

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_CONSTANTS,
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8),
							 OUString(), nConst, 0, 0);

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));

	if (nConst)
	{
		sal_uInt16 	index=0;
		iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
		while(!iter->is_done()) 
		{
			d = iter->item();
			
			if (d->node_type() == NT_const)
			{
				BE_Constant::narrow_from_decl(d)->dumpBlop(aBlop, index);

				index++;		
			}
			iter->next();
		}
		delete iter;
	}

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}
}	


//---- BE_Interface ------------------------------------------------------------
//
// produce the complete C++ definition of a client object:
//
void BE_Interface::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	AST_Decl				*d;
	UTL_ScopeActiveIterator *iter;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	sal_uInt16	nAttr = getNodeCount(AST_Decl::NT_attr);
	sal_uInt16	nOp = getNodeCount(AST_Decl::NT_op);

	const char* inherit = NULL;
	
	if (n_inherits() != 0)
	{
		inherit = dd(inherits()[0])->getRelName();
 	}

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_INTERFACE, 
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8), 
							 OStringToOUString(inherit, RTL_TEXTENCODING_UTF8), nAttr, nOp, 0);

	RTUik aUik;
	if (!isvirtual())
	{
		aUik.m_Data1 = strtoul(uik1(), NULL, 16);
		aUik.m_Data2 = strtoul(uik2(), NULL, 16);
		aUik.m_Data3 = strtoul(uik3(), NULL, 16);
		aUik.m_Data4 = strtoul(uik4(), NULL, 16);
		aUik.m_Data5 = strtoul(uik5(), NULL, 16);
	} else
	{
		aUik.m_Data1 = 0;
		aUik.m_Data2 = 0;
		aUik.m_Data3 = 0;
		aUik.m_Data4 = 0;
		aUik.m_Data5 = 0;
	}
	aBlop.setUik(aUik);	

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));
	
	sal_uInt16 	attrIndex=0;
	sal_uInt16 	opIndex=0;
	iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
	while(!iter->is_done()) 
	{
		d = iter->item();
		
		if (d->node_type() == NT_op)
		{
			BE_Operation::narrow_from_decl(d)->dumpBlop(aBlop, opIndex);
			opIndex++;		
		} else
		if (d->node_type() == NT_attr)
		{
			BE_Attribute::narrow_from_decl(d)->dumpBlop(aBlop, attrIndex);
			attrIndex++;		
		}

		iter->next();
	}
	delete iter;

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	
void BE_InterfaceFwd::dumpReflection(RegistryKey& rKey)
{
//	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Exception ------------------------------------------------------------
//
void BE_Exception::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	BE_Field              	*f;
	UTL_ScopeActiveIterator *iter;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	const char* inherit = NULL;
	if (n_inherits() != 0)
	{
		inherit = dd(inherits()[0])->getRelName();
 	}

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_EXCEPTION, 
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8), 
							 OStringToOUString(inherit, RTL_TEXTENCODING_UTF8), nmembers(), 0, 0);

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));

	sal_uInt16 index=0;
	const sal_Char* pDoku=NULL;
	iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
	while(!iter->is_done()) 
	{
		f = BE_Field::narrow_from_decl(iter->item());

		aBlop.setFieldData(index, OStringToOUString(*f->local_name(), RTL_TEXTENCODING_UTF8),
						   OStringToOUString(dd(f->field_type())->getRelName(), RTL_TEXTENCODING_UTF8),
						   f->getDoku(), OUString(), RT_ACCESS_READWRITE);
		index++;		
		iter->next();
	}
	delete iter;

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Structure ------------------------------------------------------------
//
void BE_Structure::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	BE_Field              	*f;
	UTL_ScopeActiveIterator *iter;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	const char* inherit = NULL;
	if (n_inherits() != 0)
	{
		inherit = dd(inherits()[0])->getRelName();
 	}

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_STRUCT,
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8),
							 OStringToOUString(inherit, RTL_TEXTENCODING_UTF8), nmembers(), 0, 0);

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));

	sal_uInt16 index=0;
	iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
	while(!iter->is_done()) 
	{
		f = BE_Field::narrow_from_decl(iter->item());

		aBlop.setFieldData(index, OStringToOUString(*f->local_name(), RTL_TEXTENCODING_UTF8),
						   OStringToOUString(dd(f->field_type())->getRelName(), RTL_TEXTENCODING_UTF8), 
						   f->getDoku(), OUString(), RT_ACCESS_READWRITE);
		index++;		
		iter->next();
	}
	delete iter;

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Enum -----------------------------------------------------------------
//
void BE_Enum::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 			localKey;
	UTL_ScopeActiveIterator *iter;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_ENUM,
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8), 
							 OUString(), nmembers(), 0, 0);

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));

	sal_uInt16 index=0;
	iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
	while(!iter->is_done()) 
	{
		BE_EnumVal::narrow_from_decl(iter->item())->dumpBlop(aBlop, index);

		index++;		
		iter->next();
	}
	delete iter;

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
		fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}


	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Operation ------------------------------------------------------------
//
// produces a C++ operation-signature. Both for unicast and for multicast
// operations.
//
void BE_Operation::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	RTMethodMode 			methodMode;
	AST_Decl				*d;
	UTL_ScopeActiveIterator *iter;
	
	sal_uInt16	nParam = getNodeCount(AST_Decl::NT_argument);
	sal_uInt16	nExcep = 0;
	if (exceptions())
	{
		nExcep = exceptions()->length();
	}

	if (!isConst() && flags() != OP_oneway)
	{
		methodMode = RT_MODE_TWOWAY;
	} else
	if (isConst() && flags() != OP_oneway)
	{
		methodMode = RT_MODE_TWOWAY_CONST;
	} else
	if (isConst() && flags() == OP_oneway)
	{
		methodMode = RT_MODE_ONEWAY_CONST;
	} else
	if (!isConst() && flags() == OP_oneway)
	{
		methodMode = RT_MODE_ONEWAY;
	}
	
	aBlop.setMethodData(index, OStringToOUString( *local_name(), RTL_TEXTENCODING_UTF8),
						OStringToOUString(dd(return_type())->getRelName(), RTL_TEXTENCODING_UTF8),
						methodMode, nParam, nExcep, getDoku());

	if (nParam)
	{
		sal_uInt16 argIndex = 0;
		iter = new UTL_ScopeActiveIterator(this, UTL_Scope::IK_decls);
		while(!iter->is_done()) 
		{
			d = iter->item();
			if (d->node_type() == AST_Decl::NT_argument)
			{
				BE_Argument::narrow_from_decl(d)->dumpBlop(aBlop, index, argIndex);
			}

			argIndex++;		
			iter->next();
		}
		delete iter;
	}
	
	if (nExcep)
	{
		UTL_ExceptList	*el	= exceptions();
		if (el != NULL)
		{
			UTL_ExceptlistActiveIterator	*iter;
			BE_Exception					*e;
			sal_uInt16							exceptIndex = 0;				

			iter = new UTL_ExceptlistActiveIterator(el);
			while (!(iter->is_done()))
			{
				e = BE_Exception::narrow_from_decl(iter->item());
				
				aBlop.setExcData(index, exceptIndex, OStringToOUString(e->getRelName(), RTL_TEXTENCODING_UTF8) );

				exceptIndex++;
				iter->next();
			}
			delete iter;
		}
	}
}	

//---- BE_Field -------------------------------------------------------------
//
void BE_Field::dumpReflection(RegistryKey& rKey)
{
}	

//---- BE_Argument -------------------------------------------------------------
//
void BE_Argument::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index, sal_uInt16& argIndex)
{
	RTParamMode paramMode;

	switch (direction())
	{
		case AST_Argument::dir_IN :
			paramMode = RT_PARAM_IN;
			break;
		case AST_Argument::dir_OUT :
			paramMode = RT_PARAM_OUT;
			break;
		case AST_Argument::dir_INOUT :
			paramMode = RT_PARAM_INOUT;
			break;
	}			
	
	aBlop.setParamData(index, argIndex,
					   OStringToOUString(dd(field_type())->getRelName(), RTL_TEXTENCODING_UTF8),	
					   OStringToOUString(*local_name(), RTL_TEXTENCODING_UTF8),
					   paramMode);
}	

//---- BE_Attribute ------------------------------------------------------
//
//
void BE_Attribute::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	RTFieldAccess accessMode = RT_ACCESS_INVALID;


	if (isReadonly())
	{
		accessMode |= RT_ACCESS_READONLY;		
	} else
	{
		accessMode |= RT_ACCESS_READWRITE;
	}
	if (isOptional())
	{
		accessMode |= RT_ACCESS_OPTIONAL;
	}
	if (isBound())
	{
		accessMode |= RT_ACCESS_BOUND;
	}
	if (isMayBeVoid())
	{
		accessMode |= RT_ACCESS_MAYBEVOID;
	}
	if (isConstrained())
	{
		accessMode |= RT_ACCESS_CONSTRAINED;
	}
	if (isTransient())
	{
		accessMode |= RT_ACCESS_TRANSIENT;
	}
	if (isMayBeAmbiguous())
	{
		accessMode |= RT_ACCESS_MAYBEAMBIGUOUS;
	}
	if (isMayBeDefault())
	{
		accessMode |= RT_ACCESS_MAYBEDEFAULT;
	}
	if (isRemoveable())
	{
		accessMode |= RT_ACCESS_REMOVEABLE;
	}
	
	aBlop.setFieldData(index, OStringToOUString( *local_name(), RTL_TEXTENCODING_UTF8),
					   OStringToOUString(dd(field_type())->getRelName(), RTL_TEXTENCODING_UTF8),
					   getDoku(), OUString(), accessMode);
}	

//---- BE_Union ----------------------------------------------------------------
//  NOTE: NOT YET IMPLEMENTED
//
void BE_Union::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 	localKey;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	

void BE_UnionBranch::dumpReflection(RegistryKey& rKey)
{
}	

void BE_UnionLabel::dumpReflection(RegistryKey& rKey)
{
}	

//---- BE_Constant -------------------------------------------------------------
//
//
void BE_Constant::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	RTConstValue 	aConst;
	sal_Unicode* 	str=NULL;
	AST_Expression::AST_ExprValue *exprVal = constant_value()->ev();
	switch (et())
	{
		case AST_Expression::EV_short:
			aConst.m_type = RT_TYPE_INT16; 
			aConst.m_value.aShort = exprVal->u.sval; 
			break;
		case AST_Expression::EV_ushort:
			aConst.m_type = RT_TYPE_UINT16; 
			aConst.m_value.aUShort = exprVal->u.usval;
			break;
		case AST_Expression::EV_long:
			aConst.m_type = RT_TYPE_INT32; 
			aConst.m_value.aLong = exprVal->u.lval;
			break;
		case AST_Expression::EV_ulong:
			aConst.m_type = RT_TYPE_UINT32; 
			aConst.m_value.aULong = exprVal->u.ulval;
			break;
		case AST_Expression::EV_hyper:
			aConst.m_type = RT_TYPE_INT32; 
			aConst.m_value.aLong = exprVal->u.lval;
			break;
		case AST_Expression::EV_uhyper:
			aConst.m_type = RT_TYPE_UINT32; 
			aConst.m_value.aULong = exprVal->u.ulval;
			break;
		case AST_Expression::EV_float:
			aConst.m_type = RT_TYPE_FLOAT; 
			aConst.m_value.aFloat = exprVal->u.fval; 
			break;	
		case AST_Expression::EV_double:
			aConst.m_type = RT_TYPE_DOUBLE; 
			aConst.m_value.aDouble = exprVal->u.dval;
			break;
		case AST_Expression::EV_char:
			break;	
		case AST_Expression::EV_octet:
			break;
		case AST_Expression::EV_byte:
			aConst.m_type = RT_TYPE_BYTE; 
			aConst.m_value.aByte = exprVal->u.oval;
			break;
		case AST_Expression::EV_bool:
			aConst.m_type = RT_TYPE_BOOL; 
			aConst.m_value.aBool = exprVal->u.bval;
			break;
		case AST_Expression::EV_string:
			{
				aConst.m_type = RT_TYPE_STRING; 
				::rtl::OUString aTempStr( ::rtl::OUString::createFromAscii(exprVal->u.strval->getStr()));
				str = new sal_Unicode[aTempStr.getLength()+1];
				rtl_copyMemory(str, aTempStr.getStr(), (aTempStr.getLength()+1) * sizeof(sal_Unicode));
				aConst.m_value.aString = str;
			}
			break;
		default:
			internal("exprtype to const type: cannot convert ExprType %d", 
					 exprtype_to_string(et()));
	}

	String idlSource(IDL_Global::idl_global()->main_filename()->getStr());

	aBlop.setFieldData(index, OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8),
					   OStringToOUString(exprtype_to_string(et()), RTL_TEXTENCODING_UTF8), 
					   getDoku(), OStringToOUString(idlSource, RTL_TEXTENCODING_UTF8), 
					   RT_ACCESS_CONST, aConst);

	if (str)
		delete[] str;
}	

//---- BE_EnumVal -------------------------------------------------------------
//
//
void BE_EnumVal::dumpBlop(RegistryTypeWriter& aBlop, sal_uInt16& index)
{
	RTConstValue aConst;
	AST_Expression::AST_ExprValue *exprVal = constant_value()->ev();

	aConst.m_type = RT_TYPE_INT32; 
	aConst.m_value.aLong = (sal_Int32)(exprVal->u.ulval);

	aBlop.setFieldData(index, OStringToOUString(*local_name(), RTL_TEXTENCODING_UTF8), 
					   OUString(), getDoku(), OUString(), RT_ACCESS_CONST, aConst);
}	

//---- BE_ExprVal -------------------------------------------------------------
//     NOTE: NOT YET IMPLEMENTED
//

//---- BE_Sequence -------------------------------------------------------------
//
// dump declaration of a Sequence:
//
void BE_Sequence::dumpReflection(RegistryKey& rKey)
{
}	

//---- BE_String ---------------------------------------------------------------
//
// dump declaration of a String:
//
void BE_String::dumpReflection(RegistryKey& rKey)
{
}	

//---- BE_Array ----------------------------------------------------------------
//
// dump declaration of a Array:
//
void BE_Array::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 	localKey;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	

//---- BE_Typedef --------------------------------------------------------------
//
// dump a typedef:
//
void BE_Typedef::dumpReflection(RegistryKey& rKey)
{
	RegistryKey 	localKey;
	
	if (rKey.createKey( OStringToOUString(getFullName(), RTL_TEXTENCODING_UTF8 ), localKey))
	{
		fprintf(stderr, "unoidl: warning, could	not create key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	RegistryTypeWriter aBlop(pWriterLoader->getApi(), RT_TYPE_TYPEDEF, 
							 OStringToOUString(getRelName(), RTL_TEXTENCODING_UTF8), 
							 OStringToOUString(dd(base_type())->getRelName(), RTL_TEXTENCODING_UTF8), 
							 0, 0, 0);

	aBlop.setDoku(getDoku());
	aBlop.setFileName(OStringToOUString(*IDL_Global::idl_global()->main_filename(), RTL_TEXTENCODING_UTF8));

	const sal_uInt8* 	pBlop = aBlop.getBlop();
	sal_uInt32			aBlopSize = aBlop.getBlopSize();

	if (localKey.setValue(OUString(), RG_VALUETYPE_BINARY, (RegValue)pBlop, aBlopSize))
	{
 	fprintf(stderr, "unoidl: warning, could	not set value of key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	if (localKey.closeKey())
	{
		fprintf(stderr, "unoidl: warning, could	not close key \"%s\" in %s\n",
			   getFullName(), rKey.getRegistryName());
	}

	BE_Decl::dumpReflection(rKey);
}	
