/*************************************************************************
 *
 *  $RCSfile: burdroot.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <idl.hxx>
#include <idlext.hxx>
#include "beurd.hxx"

#ifndef	_VOS_MODULE_HXX
#include <vos/module.hxx>
#endif

#if defined(SAL_W32) || defined(SAL_OS2)
#include <io.h>
#include <direct.h>
#include <errno.h>
#endif

#ifdef SAL_UNX
#include <sys/stat.h>
#include <errno.h>
#endif

using namespace rtl;

void changeSlash(String& s, String chg)
{
	const char *ch = s.getStr();
	char buffer[2];
	String st;

	buffer[0] = '\0';
	buffer[1] = '\0';
		
	while (*ch != '\0')
	{
		if (*ch == '\\' || *ch == '/')
		{
			st += chg;	
		} else
		{
			buffer[0] = *ch;
			st = st + buffer;
		}
		
		ch++;
	}
	
	if (s.getLength() > 0)
	{
		s = st;
	}
}

static void computeDestinationPath(char* dest)
{
	if (dest != NULL)
	{
		char *destination = strdup(dest);
		char seperators[] = "\\/";
		char *token = NULL;
		String outpath("");

#ifdef SAL_UNX
		if (*destination == '/')
			outpath += "/";
#endif

        token = strtok(destination, seperators);
		while (token != NULL)
		{
			outpath += token;

			if (strcmp(token, "..") == 0
                || *(token+1) == ':'
                || strcmp(token, ".") == 0)
			{				
#ifdef SAL_UNX
				outpath += "/";
#else
				outpath += "\\";
#endif
                token = strtok(NULL, seperators);
                continue;
			}
#ifdef SAL_UNX
			if (mkdir((char*)outpath.getStr(), 0777) == -1)
#else
			if (mkdir((char*)outpath.getStr()) == -1)
#endif
			{
				if (errno == ENOENT)
					programmer("cannot create directory %s", outpath);
            }

#ifdef SAL_UNX
			outpath += "/";
#else
			outpath += "\\";
#endif
            token = strtok(NULL, seperators);
		}
		free(destination);
	}	
}

static char* buildDestPath(char* path, char* dest, char* suffix, char* dir)
{
	char 		*s = NULL;
	char 		retPath [1024];
	static char absolute[1024];

	computeDestinationPath(dest);
		
#ifdef SAL_UNX
	if(s = ::strrchr(path, '/'))
	{
		strcpy(retPath, s+1);
		
		if (dest != NULL)
		{
			strcpy(absolute, dest);
		} else
		{
			strncpy(absolute, path, strlen(path)-strlen(s)+1);
			absolute[strlen(path)-strlen(s)+1] = '\0';
		}

		if(dir != NULL && absolute[strlen(absolute)-1] != '/')
		{
			strcat(absolute, "/");
			strcat(absolute, dir);
		}
	}
#else
	if(s = ::strrchr(path, '\\'))
	{
		strcpy(retPath, s+1);

		if(dest != NULL)
		{
			strcpy(absolute, dest);
		} else 
		{
			strncpy(absolute, path, strlen(path)-strlen(s));
			absolute[strlen(path)-strlen(s)] = '\0';
		}

		if(dir != NULL && absolute[strlen(absolute)-1] != '\\')
		{
			strcat(absolute, "\\");
			strcat(absolute, dir);
		}
	}
#endif
	else
	{
		strcpy(retPath, path);
		if(dest != NULL)
		{
			strcpy(absolute, dest);
		} else
			strcpy(absolute, ".");
#ifdef SAL_UNX
		if(dir != NULL && absolute[strlen(absolute)-1] != '/')
		{
			strcat(absolute, "/");
			strcat(absolute, dir);
		}
#else
		if(dir != NULL && absolute[strlen(absolute)-1] != '\\')
		{
			strcat(absolute, "\\");
			strcat(absolute, dir);
		}
#endif
	}

#ifdef SAL_UNX
	if (mkdir(absolute, 0777) == -1) 
#else
	if (mkdir(absolute) == -1) 
#endif
	{
		if (errno == ENOENT)
			programmer("cannot create directory %s", absolute);
	}

	s = strrchr(retPath, '.');
	strcpy(s, suffix);
#ifdef SAL_UNX
	strcat(absolute, "/");
#else
	strcat(absolute, "\\");
#endif
	strcat(absolute, retPath);
	return absolute;
}

RegistryTypeWriterLoader* pWriterLoader = NULL;

BE_Root::BE_Root(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Root(n, p)
{
	m_mapForwards.insert("Reflection");
}

// BE_Root::dump is invoked to produce all of the stub files.
//
void BE_Root::dump(ostream &)
{
	if (IDL_Global::idl_global()->getMakeDepend())
	{
		makeDependencies();
		return;	
	}

	char *urd_file = buildDestPath((char*)IDL_Global::idl_global()->main_filename()->getStr(),
							   IDL_Global::idl_global()->getHeaderDestination(), URD_SUFFIX, NULL);

	RegistryLoader* pLoader = new RegistryLoader();
	
	pWriterLoader = new RegistryTypeWriterLoader();
	
	if (!pLoader->isLoaded() || !pWriterLoader->isLoaded())
	{
		delete(pLoader);
		delete(pWriterLoader);
		fprintf(stderr, "unoidl: could not load registry dll.\n");
		exit(1);	
	}

 	Registry *pRegistry = new Registry(*pLoader);
	delete(pLoader);
	
	RegistryKey rootKey;
	
	if ( pRegistry->create(OStringToOUString(urd_file, RTL_TEXTENCODING_UTF8)) )
	{
		fprintf(stderr, "unoidl: could not create registry %s\n", urd_file);
		exit(1);
	}

	if (pRegistry->openRootKey(rootKey))
	{
		fprintf(stderr, "unoidl: could not open root of registry %s\n", urd_file);
		exit(1);
	}

	dumpReflection(rootKey);

	if (rootKey.closeKey())
	{
		fprintf(stderr, "unoidl: could not close root of %s\n", urd_file);
		exit(1);
	}

	delete(pWriterLoader);
	delete(pRegistry);	
}

void BE_Root::insertMapInclude(String inc)
{
	m_mapIncludes.insert(inc);
}	

MapIncludeSet& BE_Root::getMapIncludes()
{
	return(m_mapIncludes);
}

void BE_Root::insertMapForward(String inc)
{
	m_mapForwards.insert(inc);
}	

MapForwardSet& BE_Root::getMapForwards()
{
	return(m_mapForwards);
}

void BE_Root::addObjectInclude(char *incl)
{
	m_objectIncludes.append(new ObjectIncludeElem(incl));
}

// produce #include directives for the #pragma include statements.
//
void BE_Root::dumpObjectIncludes(ostream& o)
{
	UTL_NListIter iter(m_objectIncludes);
	ObjectIncludeElem *elem;

	while(elem = (ObjectIncludeElem*)iter.next())
	{
		o << "#include " << elem->str_ << "\n";
	}
}

void BE_Root::dontModifyBlurb(ostream& o)
{
	o << "// Don't modify this file as it will be overwritten.\n"
	  << "//\n";
}

void BE_Root::modifyBlurb(ostream& o)
{
	o << "// You may modify this file as it will not be overwritten.\n"
	  << "//\n";
}

void BE_Root::makeDependencies()
{
	IncludeSet::reverse_iterator incIter;
	String inc;

	String urdfile(buildDestPath((char*)IDL_Global::idl_global()->main_filename()->getStr(), 
										IDL_Global::idl_global()->getHeaderDestination(),
										URD_SUFFIX, NULL));

	changeSlash(urdfile, "$/");

	incIter = IDL_Global::idl_global()->getDepIncFiles()->rbegin();

	if (incIter != IDL_Global::idl_global()->getDepIncFiles()->rend())
	{
		fprintf(stdout, "%s : ", urdfile.getStr());

		String match = IDL_Global::idl_global()->getNegativeMatch();
		sal_Bool hasMatch = (match.getLength() > 0);
		sal_Bool first = sal_True;

		while (incIter != IDL_Global::idl_global()->getDepIncFiles()->rend())
		{
			inc = *incIter;

			if (!(hasMatch && inc.indexOf(match) == 0))
			{
				if (first) { fprintf(stdout, "\\\n"); first = sal_False; }

				changeSlash(inc, "$/");		
				
				fprintf(stdout, "\t%s", inc.getStr());
			}				

			incIter++;

			if (incIter != IDL_Global::idl_global()->getDepIncFiles()->rend())
			{
				fprintf(stdout, " \\\n");
			} else
			{
				fprintf(stdout, "\n\n");
			}
		}
		
		if (first) fprintf(stdout, "\n\n");
	}
		
	fprintf(stdout, "IDL_URD_FILES += %s\n\n\n", urdfile.getStr());
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS1(BE_Root, AST_Root)
IMPL_NARROW_FROM_DECL(BE_Root)
IMPL_NARROW_FROM_SCOPE(BE_Root)

