/*************************************************************************
 *
 *  $RCSfile: astdecl.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _ASTDECL_HXX_
#define _ASTDECL_HXX_



#include	<iostream.h>

#ifndef _IDLFWD_HXX_
#include	<idlfwd.hxx>
#endif
#ifndef _IDLNRW_HXX_
#include	<idlnrw.hxx>
#endif
#ifndef _UTLSCPNA_HXX_
#include	<utlscpna.hxx>
#endif
#ifndef _IDLDEFS_HXX_
#include	<idldefs.hxx>
#endif

class AST_Decl
{
public:
	// Enum defining the different kinds of AST nodes
	enum NodeType 
	{
		NT_object,				// Denotes an object
		NT_service,				// Denotes an servcie
		NT_interface_member,	// Denotes an interface which is exported from object
		NT_service_member,		// Denotes an service which is exported from object
		NT_observes,			// Denotes an observed interface 
		NT_needs,				// Denotes an needed service
		NT_module,				// Denotes a module
		NT_mapping,				// Denotes a mapping section
		NT_root,				// Denotes the root of AST
		NT_interface,			// Denotes an interface
		NT_interface_fwd,		// Fwd declared interface
		NT_constants,		 	// Denotes a constant group
		NT_const,				// Denotes a constant
		NT_except,				// Denotes an exception
		NT_attr,				// Denotes an attribute
		NT_op,					// Denotes an operation
		NT_argument,			// Denotes an op. argument
		NT_union,				// Denotes a union
		NT_union_branch,		// Denotes a union branch
		NT_struct,				// Denotes a structure
		NT_field,				// Denotes a field in structure
		NT_enum,				// Denotes an enumeration
		NT_enum_val,			// Denotes an enum. value
		NT_string,				// Denotes an IDL string
		NT_array,				// Denotes an IDL array
		NT_sequence,			// Denotes an IDL sequence
		NT_typedef,				// Denotes a typedef
		NT_pre_defined,			// Denotes a predefined type
		NT_language
	};

	// Operations

	// Constructor(s)
	AST_Decl();
	AST_Decl(NodeType type, UTL_ScopedName *n, UTL_StrList *pragmas);
	virtual ~AST_Decl(); // {}

	// Data Accessors
	sal_Bool imported();
	void set_imported(sal_Bool is_it);
	sal_Bool in_main_file();
	void set_in_main_file(sal_Bool is_it);
	UTL_Scope *defined_in();
	void set_defined_in(UTL_Scope *);
	NodeType node_type();
	long line();
	void set_line(long l);
	String *file_name();
	void set_file_name(String *s);
	UTL_ScopedName *name();
	void set_name(UTL_ScopedName *n);
	Identifier *local_name();
	void add_pragmas(UTL_StrList *p);
	UTL_StrList *pragmas();
	sal_Bool added();
	void set_added(sal_Bool is_it);

	// Narrowing
	DEF_NARROW_METHODS0(AST_Decl);
	DEF_NARROW_FROM_DECL(AST_Decl);

	// AST Dumping
	virtual void	dump(ostream &o);
	virtual void	dump();

	// Other operations

	// Return TRUE if "this" has "s" as an ancestor
	sal_Bool	has_ancestor(AST_Decl *s);

private:
	// Data
	sal_Bool				pd_imported;		// Imported?
	sal_Bool				pd_in_main_file;	// Defined in main file?
	UTL_Scope			*pd_defined_in;		// Scope
	NodeType			pd_node_type;		// What kind of node
	long				pd_line;			// Line defined in
	String				*pd_file_name;		// What file defined in
	UTL_ScopedName		*pd_name;			// As given
	Identifier			*pd_local_name;		// Name in scope
	UTL_StrList			*pd_pragmas;		// Pragmas
	sal_Bool				pd_added;			// already added

	// Operations

	// Compute the full name of an AST node
	void	compute_full_name(UTL_ScopedName *n);
};

#endif	// _ASTDECL_HXX_

