/*************************************************************************
 *
 *  $RCSfile: astexpr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _ASTEXPR_HXX_
#define _ASTEXPR_HXX_



// An expression (and maybe its value, if computed)
class AST_Expression
{
public:
	// Enum to define all the different operators to combine expressions
	enum ExprComb
	{
		EC_add,		// '+'
		EC_minus,	// '-'
		EC_mul,		// '*'
		EC_div,		// '/'
		EC_mod,		// '%'
		EC_or,		// '|'
		EC_xor,		// '^'
		EC_and,		// '&'
		EC_left,	// '<<'
		EC_right,	// '>>'
		EC_u_plus,	// unary '+'
		EC_u_minus,	// unary '-'
		EC_bit_neg,	// '~'
		EC_none,	// No operator (missing)
		EC_symbol	// a symbol (function or constant name)
	};
  
	// Enum to define the different kinds of evaluation possible
	// Extend this for more kinds of evaluation as required
	enum EvalKind 
	{
		EK_const,			// Must evaluate to constant
		EK_positive_int		// Must evaluate to positive integer
	};

	// Enum to define expression type
	enum ExprType 
	{
		EV_short,		// Expression value is short
		EV_ushort,		// Expression value is unsigned short
		EV_long,		// Expression value is long
		EV_ulong,		// Expression value is unsigned long
		EV_hyper,		// Expression value is hyper (64 bit)
		EV_uhyper,		// Expression value is unsigned hyper
		EV_longlong,	// Expression value is long long
		EV_ulonglong,	// Expression value is unsigned long long
		EV_float,		// Expression value is 32-bit float
		EV_double,		// Expression value is 64-bit float
		EV_longdouble,	// Expression value is 128-bit float
		EV_char,		// Expression value is char
		EV_wchar,		// Expression value is wide char	
		EV_octet,		// Expression value is unsigned char
		EV_byte,		// Expression value is byte
		EV_bool,		// Expression value is boolean
		EV_string,		// Expression value is char *
		EV_wstring,		// Expression value is wchar_t *
		EV_any,			// Expression value is any of above
		EV_void,		// Expression value is void (absent)
		EV_implementation, 
		EV_type, 
		EV_none		// Expression value is missing
	};

	// Structure to descrive value of constant expression and its type
	struct AST_ExprValue 
	{
		union 
		{
			short			sval;		// Contains short expression value
			unsigned short	usval;		// Contains unsigned short expr value
			long			lval;		// Contains long expression value
			unsigned long	ulval;		// Contains unsigned long expr value
			unsigned long	bval;		// Contains boolean expression value
			float			fval;		// Contains 32-bit float expr value
			double			dval;		// Contains 64-bit float expr value
			char			cval;		// Contains char expression value
			unsigned char	oval;		// Contains unsigned char expr value
			String	        *strval;	// Contains String * expr value
			unsigned long	eval;		// Contains enumeration value
		} u;
		ExprType et;
	};

	// Operations

	// Constructor(s)
	AST_Expression(AST_Expression *v, ExprType t);

	AST_Expression(ExprComb c, AST_Expression *v1, AST_Expression *v2);

	AST_Expression(short		s);
	AST_Expression(unsigned short	us);
	AST_Expression(long		l);
	AST_Expression(long		l, ExprType t);
	AST_Expression(unsigned long	ul);
	AST_Expression(float		f);
	AST_Expression(double		d);
	AST_Expression(char		c);
	AST_Expression(unsigned char	uc);
	AST_Expression(String		*s);
	AST_Expression(UTL_ScopedName	*n);
	virtual ~AST_Expression(); // {}

	// Data Accessors
	UTL_Scope *defined_in();
	void set_defined_in(UTL_Scope *d);
	long line();
	void set_line(long l);
	String *file_name();
	void set_file_name(String *f);
	ExprComb ec();
	void set_ec(ExprComb new_ec);
	AST_ExprValue	*ev();
	void set_ev(AST_ExprValue *new_ev);
	AST_Expression *v1();
	void set_v1(AST_Expression *e);
	AST_Expression *v2();
	void set_v2(AST_Expression *e);
	UTL_ScopedName *n();
	void set_n(UTL_ScopedName *new_n);

	// AST Dumping
	virtual void	dump(ostream &o);
	virtual void	dump();

	// Other operations

	// Evaluation and value coercion
	virtual AST_ExprValue		*eval(EvalKind ek);
	virtual AST_ExprValue		*coerce(ExprType t);

	// Evaluate then store value inside this AST_Expression
	virtual void	evaluate(EvalKind ek);

	// Compare to AST_Expressions
	virtual long	operator==(AST_Expression *vc);
	virtual long	compare(AST_Expression *vc);

protected:
	// Evaluate different sets of operators
	virtual AST_ExprValue	*eval_bin_op(EvalKind ek);
	virtual AST_ExprValue	*eval_bit_op(EvalKind ek);
	virtual AST_ExprValue	*eval_un_op(EvalKind ek);
	virtual AST_ExprValue	*eval_symbol(EvalKind ek);

private:
	// Data
	UTL_Scope			*pd_defined_in;	// Scope
	long				pd_line;		// Line defined in
	String				*pd_file_name;	// What file defined in

	ExprComb			pd_ec;		// What combinator
	AST_ExprValue		*pd_ev;		// computed value
	AST_Expression		*pd_v1;		// 1st sub-expression
	AST_Expression		*pd_v2;		// 2nd sub-expression
	UTL_ScopedName		*pd_n;		// symbolic name (if any)

	// Operations

	// Fill out the lineno, filename and definition scope details
	void	fill_definition_details();

	// Internal evaluation
	virtual AST_ExprValue *eval_internal(EvalKind ek);
};

#endif	// _ASTEXPR_HXX_


