/*************************************************************************
 *
 *  $RCSfile: astgen.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _ASTGEN_HXX_
#define _ASTGEN_HXX_



class AST_Generator
{
public:
	// Operations

	// Constructor
	AST_Generator() {}
	virtual ~AST_Generator() {}

	// Create a node representing a predefined type
	virtual AST_PredefinedType *create_predefined_type(AST_PredefinedType::PredefinedType t,
					UTL_ScopedName *n,
					UTL_StrList *p);

    // Create a node representing an object
	virtual AST_Object *create_object(UTL_ScopedName *n,
					UTL_StrList *p,
					char* name);

    // Create a node representing an service
	virtual AST_Service *create_service(UTL_ScopedName *n,
					UTL_StrList *p);

    // Create a node representing an implements
	virtual AST_InterfaceMember *create_interface_member(const sal_uInt16 fOptional,
					UTL_ScopedName *n,
					UTL_StrList *p);

    // Create a node representing a service_member
	virtual AST_ServiceMember *create_service_member(UTL_ScopedName *n,
					UTL_StrList *p);

    // Create a node representing an observes
	virtual AST_Observes *create_observes(UTL_ScopedName *n,
					UTL_StrList *p);

    // Create a node representing an needs
	virtual AST_Needs *create_needs(UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a module
	virtual AST_Module *create_module(UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a const group
	virtual AST_Constants *create_constants(UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a module
	virtual AST_Mapping *create_mapping(UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a module which is the root of an AST
	virtual AST_Root *create_root(UTL_ScopedName *n,
									UTL_StrList *p);

	// Create a node representing an interface
	virtual AST_Interface *create_interface(UTL_ScopedName *n, AST_Interface **ih,
					long nih, UTL_StrList *p, char* uik, 
					double version, sal_Bool vrt, char* ident);

	// Create a node representing a forward declaration of an interface
	virtual AST_InterfaceFwd *create_interface_fwd(UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing an exception
	virtual AST_Exception *create_exception(UTL_ScopedName *n,
					AST_Exception **ih,	long nih, UTL_StrList *p);

	// Create a node representing a structure
	virtual AST_Structure *create_structure(UTL_ScopedName *n,
					AST_Structure **ih,	long nih, UTL_StrList *p);

	// Create a node representing an enum
	virtual AST_Enum *create_enum(UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing an operation on an interface
	virtual AST_Operation *create_operation(AST_Type *rt,
					AST_Operation::Flags fl,
					IFlag ifl,
					char* local,
					sal_Bool cons,
					UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a field in a structure, exception or
	// union
	virtual AST_Field *create_field(AST_Type *ft,
					UTL_ScopedName *n,
				    UTL_StrList *p);

	// Create a node representing an argument to an operation
	virtual AST_Argument *create_argument(AST_Argument::Direction d,
					AST_Type *ft,
					UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing an attribute
	virtual AST_Attribute *create_attribute(sal_uInt16 attrFlags,
					IFlag ifl,
					AST_Type *ft,
					UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a union
	virtual AST_Union *create_union(AST_ConcreteType *dt,
				    UTL_ScopedName *n,
				    UTL_StrList *p);

	// Create a node representing one branch in a union
	virtual AST_UnionBranch *create_union_branch(AST_UnionLabel *lab,
					AST_Type *ft,
					UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing a label on a union branch
	virtual AST_UnionLabel *create_union_label(AST_UnionLabel::UnionLabel ul,
					AST_Expression *lv);

	// Create a node representing a constant
	virtual AST_Constant *create_constant(AST_Expression::ExprType et,
					AST_Expression *ev,
					UTL_ScopedName *n,
					UTL_StrList *p);

	// Create various kinds of nodes representing expressions
	virtual AST_Expression *create_expr(UTL_ScopedName *n);
	virtual AST_Expression *create_expr(AST_Expression *v,
					AST_Expression::ExprType t);
	virtual AST_Expression *create_expr(AST_Expression::ExprComb c,
					AST_Expression *v1,
					AST_Expression *v2);
	virtual AST_Expression *create_expr(long v);
	virtual AST_Expression *create_expr(long v,
					AST_Expression::ExprType t);
	virtual AST_Expression *create_expr(unsigned long v);
	virtual AST_Expression *create_expr(String *s);
	virtual AST_Expression *create_expr(char c);
	virtual AST_Expression *create_expr(double d);

	// Create a node representing an enumerator
	virtual AST_EnumVal *create_enum_val(unsigned long v,
					UTL_ScopedName *n,
					UTL_StrList *p);

	// Create a node representing an array type
	virtual AST_Array *create_array(UTL_ScopedName *n,
				    unsigned long ndims,
				    UTL_ExprList *dims);

	// Create a node representing a sequence type
	virtual AST_Sequence *create_sequence(AST_Expression *v,
					AST_Type *bt);

	// Create a node representing a string type
	virtual AST_String *create_string(AST_Expression *v);

	// Create a node representing a wide string type
	virtual AST_String *create_wstring(AST_Expression *v);

	// Create a node representing a type renaming (typedef)
	virtual AST_Typedef *create_typedef(AST_Type *bt,
					UTL_ScopedName *n,
					UTL_StrList *p);
};

#endif	// _ASTGEN_HXX_

