/*************************************************************************
 *
 *  $RCSfile: astinter.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _ASTINTER_HXX_
#define _ASTINTER_HXX_


#ifndef _IDLFWD_HXX_
#include	<idlfwd.hxx>
#endif
#ifndef _IDLNRW_HXX_
#include	<idlnrw.hxx>
#endif
#ifndef _ASTTYPE_HXX_
#include	<asttype.hxx>
#endif
#ifndef _UTLSCOPE_HXX_
#include	<utlscope.hxx>
#endif
#ifndef _ASTDECL_HXX_
#include	<astdecl.hxx>
#endif
#ifndef _UTLNLIST_HXX_
#include	<utlnlist.hxx>
#endif

class AST_Interface
	:	public virtual AST_Type, 
		public virtual UTL_Scope
{
public:
	// Operations

	// Constructor(s)
	AST_Interface();
	AST_Interface(UTL_ScopedName *n, AST_Interface **ih,
				  long nih, UTL_StrList *p,
				  char* uik, double version, sal_Bool vrt, char* ident);
	virtual ~AST_Interface(); // {}

	// Data Accessors
	AST_Interface	**inherits();
	void set_inherits(AST_Interface **i);
	long n_inherits();
	void set_n_inherits(long i);
  
	char* uik1();
	char* uik2();
	char* uik3();
	char* uik4();
	char* uik5();
	void set_uik(char* uik);
	
	char* ident();
	void set_ident(char* ident);

	sal_Bool isvirtual();
	void set_virtual(sal_Bool bVirtual)
			{ pd_virtual = bVirtual; }
	
	unsigned short version1();
	unsigned short version2();
	void set_version(double version);

	// Is this interface defined? This predicate returns FALSE when an
	// interface was forward declared but not defined yet, and TRUE in
	// all other cases.
	sal_Bool is_defined()
	{
		return (pd_n_inherits < 0) ? sal_False : sal_True;
	}

	// Narrowing
	DEF_NARROW_METHODS2(AST_Interface, AST_Type, UTL_Scope);
	DEF_NARROW_FROM_DECL(AST_Interface);
	DEF_NARROW_FROM_SCOPE(AST_Interface);

	// AST Dumping
	virtual void	dump(ostream &o);
 
	virtual sal_Bool checkInheritance();

	void set_forwarded(sal_Bool fwd);
	sal_Bool is_forwarded();

	void setForwardInSameFile(sal_Bool fwd) { pd_forwardInSameFile = fwd; };
	sal_Bool isForwardInSameFile() { return(pd_forwardInSameFile); };

	virtual void setIdldocText(char* doctext) {}
	virtual char * getIdldocText() { return NULL; }

	virtual void setIdldocTags(char* doctags) {}
	virtual char * getIdldocTags() { return NULL; }

private:
	// Data
	AST_Interface	**pd_inherits;	// Inherited interfaces
									// This is an array of pointers
									// to the inherited interfaces
	long		pd_n_inherits;	// How many of them?
	char*		pd_uik1;		// pd_uiidx contains the transformed uiid
	char*		pd_uik2;
	char*		pd_uik3;
	char*		pd_uik4;
	char*		pd_uik5;
	char*		pd_ident;		// present a public name for the interface
	unsigned short	pd_version1;	// contains the version number
	unsigned short	pd_version2;
	sal_Bool	pd_seen;
	sal_Bool	pd_virtual;				// flag for an virtual interface
	sal_Bool	pd_forwarded;			//
	sal_Bool pd_forwardInSameFile;

	// Scope Management Protocol
	friend int yyparse();

	virtual AST_Interface* checkAllInheritance();

	void transformUIK(char* uik);
	void transformVersion(double version);
	void resetSeen();
	void getAllSuperClasses(UTL_NList&);	// List<be_interface*>
	void getAllSuperClassesSub(UTL_NList&);	// List<be_interface*>

	virtual AST_Constant	*fe_add_constant(AST_Constant	*c);
	virtual AST_Exception	*fe_add_exception(AST_Exception	*e);
	virtual AST_Attribute	*fe_add_attribute(AST_Attribute	*a);
	virtual AST_Operation	*fe_add_operation(AST_Operation	*o);
	virtual AST_Union		*fe_add_union(AST_Union		*u);
	virtual AST_Structure	*fe_add_structure(AST_Structure	*s);
	virtual AST_Enum		*fe_add_enum(AST_Enum		*e);
	virtual AST_EnumVal		*fe_add_enum_val(AST_EnumVal	*v);
	virtual AST_Typedef		*fe_add_typedef(AST_Typedef	*t);
};


class AST_InterfaceElem
	:	public UTL_NListElem 
{
public:
	AST_InterfaceElem(AST_Interface *bi): bi_(bi){};
	int compare(const class UTL_NListElem *){ abort(); return 0;};
	class UTL_NListElem *clone(){ return (UTL_NListElem*)new AST_InterfaceElem(*this); };
	AST_Interface *bi_;
};


#endif	// _ASTINTER_HXX_

