/*************************************************************************
 *
 *  $RCSfile: feheads.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _FEHEADS_HXX_
#define _FEHEADS_HXX_


#ifndef _ASTOPER_HXX_
#include	<astoper.hxx>
#endif

#define AF_INVALID			0x0000
#define AF_READONLY			0x0001
#define AF_OPTIONAL			0x0002
#define AF_MAYBEVOID		0x0004
#define AF_BOUND			0x0008
#define AF_CONSTRAINED		0x0010
#define AF_TRANSIENT		0x0020
#define AF_MAYBEAMBIGUOUS	0x0040
#define AF_MAYBEDEFAULT		0x0080
#define AF_REMOVEABLE		0x0100
#define AF_ATTRIBUTE		0x0200
#define AF_PROPERTY			0x0400


class FE_InterfaceHead;

class FE_InterfaceHeader
{
public:
	// Operations

	// Constructor(s)
	FE_InterfaceHeader(UTL_ScopedName *n, UTL_NameList *l, FE_InterfaceHead* h);
	virtual ~FE_InterfaceHeader(); // {}

	// Data Accessors
	UTL_ScopedName *interface_name() { return m_pInterfaceName; }
	AST_Interface **inherits() { return m_pInherits; }
	long n_inherits() { return m_nInherits; }
	FE_InterfaceHead* interface_head() { return m_pInterface_head; }

protected:
	// Data
	UTL_ScopedName		*m_pInterfaceName;	// Interface name
	AST_Interface		**m_pInherits;		// Inherited interfaces
	long				m_nInherits;		// How many
	FE_InterfaceHead	*m_pInterface_head;

	// Operations

	// Compile the flattened unique list of interfaces which this
	// interface inherits from
	void compile_one_inheritance(AST_Interface *i);
	void compile_inheritance(UTL_NameList *l);
};

class FE_ExceptionHeader
{
public:
	// Operations

	// Constructor(s)
	FE_ExceptionHeader(UTL_ScopedName *n, UTL_NameList *l);
	virtual ~FE_ExceptionHeader(); // {}

	// Data Accessors
	UTL_ScopedName *exception_name() { return m_pExceptionName; }
	AST_Exception **inherits() { return m_pInherits; }
	long n_inherits() { return m_nInherits; }

protected:
	// Data
	UTL_ScopedName		*m_pExceptionName;	// Exception name
	AST_Exception		**m_pInherits;		// Inherited exceptions
	long				m_nInherits;		// How many

	// Operations

	// Compile the flattened unique list of exceptions which this
	// exception inherits from
	void compile_one_inheritance(AST_Exception *i);
	void compile_inheritance(UTL_NameList *l);
};

class FE_StructHeader
{
public:
	// Operations

	// Constructor(s)
	FE_StructHeader(UTL_ScopedName *n, UTL_NameList *l);
	virtual ~FE_StructHeader(); // {}

	// Data Accessors
	UTL_ScopedName *struct_name() { return m_pStructName; }
	AST_Structure **inherits() { return m_pInherits; }
	long n_inherits() { return m_nInherits; }

protected:
	// Data
	UTL_ScopedName		*m_pStructName;		// Structure name
	AST_Structure		**m_pInherits;		// Inherited exceptions
	long				m_nInherits;		// How many

	// Operations

	// Compile the flattened unique list of structures which this
	// structure inherits from
	void compile_one_inheritance(AST_Structure *i);
	void compile_inheritance(UTL_NameList *l);
};

class FE_InterfaceHead
{
public:
	// Operations

	// Constructor
	FE_InterfaceHead(sal_Bool vrt, char *uik, char* ident, double version);
	virtual ~FE_InterfaceHead();

	// Data Accessors
	sal_Bool isvirtual() { return m_virtual; }
	char* uik() { return m_pUik; }
	char* ident() { return m_pIdent; }
	double version() { return m_version; }
	
protected:
	// Data
	sal_Bool m_virtual;
	char	*m_pUik;
	char	*m_pIdent; 
	double	m_version;
};

class FE_Ident
{
public:
	// Operations

	// Constructor
	FE_Ident(char* ident, double version);
	virtual ~FE_Ident();

	// Data Accessors
	char* ident() { return m_pIdent; }
	double version() { return m_version; }
	
protected:
	// Data
	char	*m_pIdent; 
	double 	m_version;
};


class FE_OperationHead
{
public:
	// Constructor
	FE_OperationHead();
	FE_OperationHead(AST_Operation::Flags attr_fl, char* local, sal_Bool co);
	virtual ~FE_OperationHead() {}

	// Data Accessors
	AST_Operation::Flags attribute() { return m_attribute; }
	void setAttribute(AST_Operation::Flags attr) { m_attribute = attr; }

	sal_Bool isLocal() { return m_isLocal; }
	String& local() { return m_local; }
	void setLocal(sal_Bool isl, String local)
		{ m_isLocal =isl; m_local = local; }

	sal_Bool isConst() { return m_isConst; }
	void setConst(sal_Bool co) { m_isConst = co; }

	IFlag implementation() { return m_implementation; }

protected:
	// Data
	AST_Operation::Flags	m_attribute;
	sal_Bool				m_isLocal;
	String					m_local;
	IFlag					m_implementation;
	sal_Bool				m_isConst;	
};


class FE_AttributeHead
{
public:
	// Constructor
	FE_AttributeHead();
	FE_AttributeHead(sal_Bool ro, char* local);
	virtual ~FE_AttributeHead() {}

	// Data Accessors
	sal_Bool readonly() { return m_readonly; }
	void setReadOnly(sal_Bool rdonly) { m_readonly = rdonly; }

	sal_Bool isOptional() { return m_optional; }
	void setOptional(sal_Bool opt) { m_optional = opt; }

	void setFlag(sal_uInt32 flag) { m_flags |= flag; }
	sal_Bool hasFlag(sal_uInt32 flag) { return ((m_flags & flag) == flag) ? sal_True : sal_False; }
	sal_uInt32 getFlags() { return m_flags; }
//	sal_Bool isLocal() { return m_isLocal; }
//	String& local() { return m_local; }
//	void setLocal(sal_Bool isl, String local) 
//		{ m_isLocal =isl; m_local = local; }
	
	IFlag implementation() { return m_implementation; }

protected:
	// Data
	sal_Bool		m_readonly;
	sal_Bool		m_optional;
	sal_Bool		m_isLocal;	
	String			m_local;
	IFlag			m_implementation;
	sal_uInt32 		m_flags;
};


#endif	// _FEHEADS_HXX_

