/*************************************************************************
 *
 *  $RCSfile: idlglob.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _IDLGLOB_HXX_
#define _IDLGLOB_HXX_

#include <hash_map>
#include <set>
#include <utility>

struct LessString
{
	bool operator()(const String& str1, const String& str2) const
	{
		return (str1 < str2);
	}
};
typedef NAMESPACE_STD(set) <String, LessString> IncludeSet;

class IDL_Global
{
public:
	// Types

	// The ParseState type encodes the state of the Yacc parser so that
	// better error messages can be produced, and for use by BEs in situations
	// where context sensitive behavior is required
	enum ParseState 
	{
		PS_NoState,				// No state
		PS_TypeDeclSeen,		// Seen complete typedef declaration
		PS_ConstDeclSeen,		// Seen complete const declaration
		PS_ExceptDeclSeen,		// Seen complete exception declaration
		PS_InterfaceDeclSeen,	// Seen complete interface declaration
		PS_ObjectDeclSeen,		// Seen complete object declaration
		PS_ServiceDeclSeen,		// Seen complete service declaration
		PS_ModuleDeclSeen,		// Seen complete module declaration
		PS_AttrHeadSeen,		// Seen the attribute head
		PS_AttrDeclSeen,		// Seen complete attribute declaration
		PS_PropertyHeadSeen,	// Seen the property head
		PS_PropertyDeclSeen,	// Seen complete property declaration
		PS_OpDeclSeen,			// Seen complete operation declaration	

		PS_ObjectSeen,			// Seen a OBJECT keyword
		PS_ObjectIDSeen,		// Seen the object ID
		PS_ObjectSqSeen,		// '{' seen for object
		PS_ObjectQsSeen,		// '}' seen for object
		PS_ObjectBodySeen,		// Seen complete object body
		PS_ObjectMemberSeen,	// Seen a object member
		PS_ObjectHeadSeen,		// Seen the object head

		PS_ServiceSeen,			// Seen a SERVICE keyword
		PS_ServiceIDSeen,		// Seen the service ID
		PS_ServiceSqSeen,		// '{' seen for service
		PS_ServiceQsSeen,		// '}' seen for service
		PS_ServiceBodySeen, 	// Seen complete service body
		PS_ServiceMemberSeen,	// Seen a service member
		PS_ServiceHeadSeen,		// Seen the service head

		PS_ModuleSeen,			// Seen a MODULE keyword
		PS_ModuleIDSeen,		// Seen the module ID
		PS_ModuleSqSeen,		// '{' seen for module
		PS_ModuleQsSeen,		// '}' seen for module
		PS_ModuleBodySeen,		// Seen complete module body
		
		PS_MappingSeen,			// Seen a MAPPING keyword
		PS_MappingDeclSeen,		// Seen complete mapping declaration  
		PS_MappingSqSeen,		// '{' seen for mapping
		PS_MappingQsSeen,		// '}' seen for mapping
		PS_MappingBodySeen,		// Seen complete mapping body

		PS_LanguageSeen,
		PS_LanguageIdSeen,
		PS_LanguageDeclSeen,
		PS_LanguageSqSeen,
		PS_LanguageQsSeen,
		PS_LanguageBodySeen,

		PS_MapSeen,
		PS_MapIdSeen,
		PS_MapDeclSeen,
		PS_MapSqSeen,
		PS_MapQsSeen,
		PS_MapBodySeen,

		PS_MapNameSeen,
		PS_MapNameIdSeen,
		PS_MapSuperSeen,
		PS_MapSuperIdSeen,
		PS_MapIncSeen,
		PS_MapIncIdSeen,
		PS_GetReflSeen,
		PS_GetReflIdSeen,
		PS_IncReflSeen,
		PS_IncReflIdSeen,
		PS_MapIdentSeen,
		PS_MapIdentIdSeen,
		PS_CreateAllListenerSeen,
		PS_CreateAllListenerIdSeen,

		PS_InterfaceSeen,		// Seen an INTERFACE keyword
		PS_InterfaceIDSeen,		// Seen the interface ID
		PS_InterfaceHeadSeen,	// Seen the interface head
		PS_InterfaceHeadSqSeen,	// '[' seen for interface head
		PS_InterfaceHeadQsSeen,	// ']' seen for interface head
		PS_IHeadUikSeen,		// Seen uik in interface head
		PS_IHeadVirtualSeen,	// Seen virtual in interface head
		PS_IHeadIdentSeen,		// Seen ident in interface head
		PS_InheritSpecSeen,		// Seen a complete inheritance spec
		PS_ForwardDeclSeen,		// Forward interface decl seen
		PS_InterfaceSqSeen,		// '{' seen for interface
		PS_InterfaceQsSeen,		// '}' seen for interface
		PS_InterfaceBodySeen,	// Seen an interface body
		PS_InheritColonSeen,	// Seen ':' in inheritance list
		PS_SNListCommaSeen,		// Seen ',' in list of scoped names
		PS_ScopedNameSeen,		// Seen a complete scoped name
		PS_SN_IDSeen,			// Seen an identifier as part of a scoped name
		PS_ScopeDelimSeen,		// Seen a scope delim as party of a scoped name
		PS_ConstSeen,			// Seen a CONST keyword
		PS_ConstTypeSeen,		// Parsed the type of a constant
		PS_ConstIDSeen,			// Seen the constant ID
		PS_ConstAssignSeen,		// Seen the '='
		PS_ConstExprSeen,		// Seen the constant value expression
		PS_TypedefSeen,			// Seen a TYPEDEF keyword
		PS_TypeSpecSeen,		// Seen a complete type specification
		PS_DeclaratorsSeen,		// Seen a complete list of declarators
		PS_StructSeen,			// Seen a STRUCT keyword
		PS_StructIDSeen,		// Seen the struct ID
		PS_StructSqSeen,		// '{' seen for struct
		PS_StructQsSeen,		// '}' seen for struct
		PS_StructBodySeen,		// Seen complete body of struct decl
		PS_MemberTypeSeen,		// Seen type of struct or except member
		PS_MemberDeclsSeen,		// Seen decls of struct or except members
		PS_MemberDeclsCompleted,// Completed one struct or except member to ';'
		PS_MemberLengthSeen,	// Seen member index_length
		PS_MemberSizeSeen,		// Seen memebr index_size
		PS_UnionSeen,			// Seen a UNION keyword
		PS_UnionIDSeen,			// Seen the union ID
		PS_SwitchSeen,			// Seen the SWITCH keyword
		PS_SwitchOpenParSeen,	// Seen the switch open par.
		PS_SwitchTypeSeen,		// Seen the switch type spec
		PS_SwitchCloseParSeen,	// Seen the switch close par.
		PS_UnionSqSeen,			// '{' seen for union
		PS_UnionQsSeen,			// '}' seen for union
		PS_DefaultSeen,			// Seen DEFAULT keyword
		PS_UnionLabelSeen,		// Seen label of union element
		PS_LabelColonSeen,		// Seen ':' of union branch label
		PS_LabelExprSeen,		// Seen expression of union branch label
		PS_UnionElemSeen,		// Seen a union element
		PS_UnionElemCompleted,	// Completed one union member up to ';'
		PS_CaseSeen,			// Seen a CASE keyword
		PS_UnionElemTypeSeen,	// Seen type spec for union element
		PS_UnionElemDeclSeen,	// Seen declarator for union element
		PS_UnionBodySeen,		// Seen completed union body
		PS_EnumSeen,			// Seen an ENUM keyword
		PS_EnumIDSeen,			// Seen the enum ID
		PS_EnumSqSeen,			// Seen '{' for enum
		PS_EnumQsSeen,			// Seen '}' for enum
		PS_EnumBodySeen,		// Seen complete enum body
		PS_EnumCommaSeen,		// Seen ',' in list of enumerators
		PS_SequenceSeen,		// Seen a SEQUENCE keyword
		PS_SequenceSqSeen,		// Seen '<' for sequence
		PS_SequenceQsSeen,		// Seen '>' for sequence
		PS_SequenceTypeSeen,	// Seen type decl for sequence
		PS_SequenceCommaSeen,	// Seen comma for sequence
		PS_SequenceExprSeen,	// Seen size expression for sequence
		PS_StringSeen,			// Seen a STRING keyword
		PS_StringSqSeen,		// Seen '<' for string
		PS_StringQsSeen,		// Seen '>' for string
		PS_StringExprSeen,		// Seen size expression for string
		PS_StringCompleted,		// Seen a string decl without size spec
		PS_ArrayIDSeen,			// Seen array ID
		PS_ArrayCompleted,		// Seen completed array declaration
		PS_DimSqSeen,			// Seen '[' for array dimension
		PS_DimQsSeen,			// Seen ']' for array dimension
		PS_DimExprSeen,			// Seen size expression for array dimension
		PS_AttrROSeen,			// Seen READONLY keyword
		PS_AttrOptSeen,			// Seen OPTIONAL keyword
		PS_ImplAttrSeen,		// Seen optional implementation attribute
		PS_AttrSeen,			// Seen ATTRIBUTE keyword
		PS_AttrTypeSeen,		// Seen type decl for attribute
		PS_AttrDeclsSeen,		// Seen declarators for attribute
		PS_AttrCompleted,		// Seen complete attribute declaration
		PS_PropertySeen,		// Seen PROPERTY keyword
		PS_PropertyTypeSeen,	// Seen type decl for property
		PS_PropertyDeclsSeen,	// Seen declarators for property
		PS_PropertyCompleted,	// Seen complete property declaration
		PS_ExceptSeen,			// Seen EXCEPTION keyword
		PS_ExceptIDSeen,		// Seen exception identifier
		PS_ExceptSqSeen,		// Seen '{' for exception
		PS_ExceptQsSeen,		// Seen '}' for exception
		PS_ExceptBodySeen,		// Seen complete exception body
		PS_OpHeadSeen,			// Seen the operation head
		PS_OpAttrSeen,			// Seen operation attribute
		PS_OpTypeSeen,			// Seen operation return type
		PS_OpIDSeen,			// Seen operation ID
		PS_OpParsCompleted,		// Completed operation param list
		PS_OpRaiseCompleted,	// Completed operation except list
		PS_OpContextCompleted,	// Completed operation context spec
		PS_OpCompleted,			// Completed operation statement
		PS_OpSqSeen,			// Seen '(' for operation
		PS_OpQsSeen,			// Seen ')' for operation
		PS_OpParCommaSeen,		// Seen ',' in list of op params
		PS_OpParDirSeen,		// Seen parameter direction
		PS_OpParTypeSeen,		// Seen parameter type
		PS_OpParDeclSeen,		// Seen parameter declaration
		PS_OpRaiseSeen,			// Seen RAISES keyword
		PS_OpRaiseSqSeen,		// Seen '(' for RAISES
		PS_OpRaiseQsSeen,		// Seen ')' for RAISES
		PS_OpContextSeen,		// Seen CONTEXT keyword
		PS_OpContextSqSeen,		// Seen '(' for CONTEXT
		PS_OpContextQsSeen,		// Seen ')' for CONTEXT
		PS_OpContextCommaSeen,	// Seen ',' for CONTEXT
		PS_DeclsCommaSeen,		// Seen ',' in declarators list
		PS_DeclsDeclSeen		// Seen complete decl in decls list
	};
	
	// Constructor
	IDL_Global();
	virtual ~IDL_Global();

	static IDL_Global * idl_global();

	// Operations
	virtual UTL_ScopeStack	*scopes();						// Scopes stack
	virtual void			set_scopes(UTL_ScopeStack *);	// Set it

	virtual AST_Root		*root();						// Root of AST
	virtual void			set_root(AST_Root *);			// Set it

	virtual AST_Generator		*gen();						// Generator
	virtual void			set_gen(AST_Generator *);		// Set it

	virtual UTL_Error		*err();							// Error reporter
	virtual void			set_err(UTL_Error *);			// Set it

	virtual long			err_count();					// How many errors?
	virtual void			set_err_count(long);			// Set it

	virtual long			lineno();						// Where in file?
	virtual void			set_lineno(long);				// Set it

	virtual String			*filename();					// What file?
	virtual void			set_filename(String* );			// Set it

	virtual String			*main_filename();				// What's the main
															// file name?
	virtual void			set_main_filename(String *);	// Set it

	virtual String			*real_filename();				// What's the real
															// file name?
	virtual void			set_real_filename(String *);	// Set it
  
	virtual String			*stripped_filename();			// Stripped filename
	virtual void			set_stripped_filename(String *);// Set it

	virtual sal_Bool		imported();						// Are we imported?
	virtual sal_Bool		import();						// Is import on?
	virtual void			set_import(sal_Bool);			// Set it

	virtual sal_Bool		in_main_file();					// Are we?
	virtual void			set_in_main_file(sal_Bool);		// Set it

	virtual char			*prog_name();					// Invoked as..
	virtual void			set_prog_name(const char *);			// Set it

	virtual char			*cpp_location();				// Where's CPP?
	virtual void			set_cpp_location(char *);		// Set it

	virtual long			compile_flags();				// What flags are on?
	virtual void			set_compile_flags(long);		// Turn some on or off

	virtual String&			be();							// Get BE to use
	virtual void			set_be(String);					// Set it

	virtual char			*getHeaderDestination();		// Get destination to use
	virtual void			setHeaderDestination(char *);	// Set it

	virtual char			*getImplDestination();			// Get destination to use
	virtual void			setImplDestination(char *);		// Set it

	virtual char			*be_args();						// Get BE to use
	virtual void			set_be_args(char *);				// Set it

	virtual char			*local_escapes();				// Get local escapes
	virtual void			set_local_escapes(char *);		// Set it

	virtual char			*options();						// Get commandline options
	virtual void			set_options(char *);			// Set it

	virtual UTL_Indenter	*indent();					// Get indenter
	virtual void			set_indent(UTL_Indenter *);		// Set it

	virtual UTL_StrList		*pragmas();						// Get pragmas
	virtual void			set_pragmas(UTL_StrList *);		// Set it

	virtual sal_Bool		read_from_stdin();				// Reading from stdin?
	virtual void			set_read_from_stdin(sal_Bool);	// Set it

	virtual ParseState		parse_state();					// What state we're in
	virtual void			set_parse_state(ParseState s);	// Set it

	// Convert from a predefined type to an expression type
	virtual AST_Expression::ExprType 
				PredefinedTypeToExprType(AST_PredefinedType::PredefinedType);

	virtual void			set_idldoc_text(char*);
	virtual char			*idldoc_text();

	virtual void			set_idldoc_valid(int);
	virtual int				idldoc_valid();

	void			setFilePackages(UTL_PkgList *packages);
	String			*getFilePackagePrefix(String *file);
	void			addFilePackagePrefix(String* file, String* package);

	void			resetSettings();

	void			setMapping(const String langauge, const String name, UTL_Map* mapping);
	UTL_Map* 		getMapping(const String& language, const String& name);

	void			setMappingLanguage(String mapLanguage) { m_mappingLanguage = mapLanguage; }
	String&			getMappingLanguage() { return(m_mappingLanguage); }

	void 			setIncludeFile(String include);
	IncludeSet*		getIncludeFiles();

	void 			setImportOfFile(const String filename, short import);
	short			getImportOfFile(const String& filename);

	void			setIncludePrefix(String);
	String&			getIncludePrefix();

	void			setProjectName(String);
	String&			getProjectName();

	void			setFilePrefix(String fileprefix) { m_filePrefix = fileprefix; }
	String&			getFilePrefix() { return m_filePrefix; }

	void			setAppendCore(sal_Bool append) { m_appendCore = append; }
	sal_Bool			getAppendCore() { return(m_appendCore); }

	void			setMakeDepend(sal_Bool depend) { m_makeDepend = depend; }
	sal_Bool			getMakeDepend() { return(m_makeDepend); }

	void 			setDepIncFile(String depInc) { m_depIncFiles->insert(depInc); }
	IncludeSet*		getDepIncFiles() { return(m_depIncFiles); }

	void			setNegativeMatch(String match) { m_negativeMatch = match; }
	String&			getNegativeMatch() { return m_negativeMatch; }

private:
	
	// Data
	UTL_ScopeStack		*m_scopes;				// Store scopes stack
	AST_Root			*m_root;				// Store AST root
	AST_Generator		*m_generator;	   		// Store generator
	UTL_Error			*m_error;				// Error object
	long				m_err_count;			// Count of errors
	long				m_lineno;				// What line #
  	String				*m_filename;			// What file
	String				*m_main_filename;		// What main filename
	String				*m_real_filename;		// What real filename
	String				*m_stripped_filename;	// Stripped filename
	sal_Bool				m_import;				// Is import on?
	sal_Bool				m_in_main_file;		// Are we in it?
	char				*m_prog_name;			// Argv[0]
	char				*m_cpp_location;		// Where to find CPP
	long				m_compile_flags;		// Compile flags
	String				m_backend; 				// BE name to use
	char				*m_headerdestination;	// destination directory for header output
	char				*m_impldestination;		// destination directory for implementation output
	char				*m_be_args;			// args for the be 
	char				*m_local_escapes;  	// Trapdoor argument
	char				*m_options;			// Trapdoor argument
	UTL_Indenter		*m_indent;		   	// Indent object
	UTL_StrList			*m_pragmas;			// List of pragmas
												// as its being built
	sal_Bool				m_read_from_stdin; 	// Reading from stdin?
	
	ParseState			m_parse_state;	   	// Parse state we're in

	char				*m_idldoc_text;		// buffer for documentation text
	int					m_idldoc_valid;		// indicator if doc text is valid

	UTL_PkgList			*m_file_packages;

	UTL_MapTable		*m_mapTable;
	String				m_mappingLanguage;
	IncludeSet			*m_include_files;
	
	String				m_includePrefix;
	String				m_projectName;
	String				m_filePrefix;

	sal_Bool			m_appendCore;

	sal_Bool			m_makeDepend;		// generate dependencies
	String				m_negativeMatch;	
	IncludeSet			*m_depIncFiles;		// includes (full path) for dependencies
			
	static IDL_Global	*m_global;

	//	
	struct EqualString
	{
		bool operator()(const String& str1, const String& str2) const
		{
			return (str1 == str2);
		}
	};

	struct HashString
	{
		static NAMESPACE_STD(hash)<const char*> h;
		size_t operator()(String str) const
		{
			return h(str.getStr());
		}
	};

	typedef NAMESPACE_STD(hash_map) <const String, short, HashString, EqualString> ImportMap; 			

	ImportMap 		*m_importmap;
};

#endif	//_IDLGLOB_HXX_

