/*************************************************************************
 *
 *  $RCSfile: utlerror.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _UTLERROR_HXX_
#define _UTLERROR_HXX_



class UTL_Error 
{
public:
	enum ErrorCode 
	{
		EIDL_SYNTAX_ERROR,		// Syntax error in IDL input
								// More details will be gleaned from examining
								// idl_global->parse_state()
		EIDL_REDEF,				// Redefinition
		EIDL_REDEF_SCOPE,		// Redefinition inside defining scope
		EIDL_DEF_USE,			// Definition after use
		EIDL_MULTIPLE_BRANCH,	// More than one union branch with this
								// label
		EIDL_COERCION_FAILURE,	// Coercion failure
		EIDL_SCOPE_CONFLICT,	// Between fwd declare and full declare
		EIDL_ONEWAY_CONFLICT,	// Between op decl and argument direction
		EIDL_DISC_TYPE,			// Illegal discriminator type in union
		EIDL_LABEL_TYPE,		// Mismatch with discriminator type in union
		EIDL_ILLEGAL_ADD,		// Illegal add action
		EIDL_ILLEGAL_USE,		// Illegal type used in expression
		EIDL_ILLEGAL_RAISES,	// Error in "raises" clause
		EIDL_ILLEGAL_CONTEXT,	// Error in "context" clause
		EIDL_CANT_INHERIT,		// Cannot inherit from non-interface
		EIDL_LOOKUP_ERROR,		// Identifier not found
		EIDL_INHERIT_FWD_ERROR,	// Cannot inherit from fwd decl interface
		EIDL_CONSTANT_EXPECTED,	// We got something else..
		EIDL_NAME_CASE_ERROR,	// Spelling differences found
		EIDL_ENUM_VAL_EXPECTED,	// Expected an enumerator
		EIDL_ENUM_VAL_NOT_FOUND,	// Didnt find an enumerator with that name
		EIDL_EVAL_ERROR,		// Error in evaluating expression
		EIDL_AMBIGUOUS,			// Ambiguous name definition
		EIDL_DECL_NOT_DEFINED,	// Forward declared but never defined
		EIDL_FWD_DECL_LOOKUP,	// Tried to lookup in fwd declared intf
		EIDL_RECURSIVE_TYPE,	// Illegal recursive use of type
		EIDL_NONVOID_ONEWAY,	// Non-void return type in oneway operation
		EIDL_NOT_A_TYPE,		// Not a type
		EIDL_TYPE_NOT_VALID,	// Type is not valid is this context
		EIDL_BACK_END,			// Back end error
		EIDL_BADPARA_LENGTHIS,  // index in length_is is not a valid argument
		EIDL_BADPARA_SIZEIS,    // index in size_is is not a valid argument
		EIDL_TYPEDEF_ISPOINTER, // pointer declarator in typedef not allowed
		EIDL_INTERPARAM_ISPOINTER,	// interface* in parameter declaration not allowed
		EIDL_OPRETURN_ISPOINTER,	// operation pointer return type is not allowed
		EIDL_INTERFACE_LOOKUP,		// interface is not defined or a fwd declaration not exists
		EIDL_INTERFACEMEMBER_EXISTS,	// interface can not be inseret in an object twice
		EIDL_SERVICEMEMBER_EXISTS,
		EIDL_SERVICE_EXISTS,
		EIDL_OBSERVES_EXISTS,
		EIDL_NEEDS_EXISTS,
		EIDL_MULTIBLE_INHERITANCE,	// multible inheritance is not allowed
		EIDL_MAPPING_MISSED,		// language sections can only defined in a mapping section
		EIDL_MAPPING_EXIST,			// mapping exist, remapping is not allowed
		EIDL_INTERFACE_EXP,			// not all interfaces from the exported service are implemented
		EIDL_ATTRIBUTE_EXP,			// not all attributes from the exported service are implemented
		EIDL_TYPE_IDENT_CONFLICT,	// type and identifier has equal names
		EIDL_ONEWAY_RAISE_CONFLICT,	// oneway function raised excpetion conflict
		EIDL_EXCLUSIVATTRIBUTEFLAG,
		EIDL_WRONGATTRIBUTEFLAG,
		EIDL_WRONGATTRIBUTEKEYWORD,
		EIDL_MISSINGATTRIBUTEKEYWORD,	
		EIDL_OK						// No error
	};

	enum WarningCode 
	{
		WIDL_EXPID_CONFLICT,		// exception id conflict
		WIDL_REQID_CONFLICT,		// request id conflict
		WIDL_INHERIT_IDCONFLICT,		// request id conflict inheritance tree
		WIDL_TYPE_IDENT_CONFLICT	// type and identifier has equal names
	};

	// Operations

	// Report errors with varying numbers of arguments
	void	error0(ErrorCode e);
	void	error1(ErrorCode e, AST_Decl *t);
	void	error2(ErrorCode e, AST_Decl *t1, AST_Decl *t2);
	void	error3(ErrorCode e, AST_Decl *t1, AST_Decl *t2, AST_Decl *t3);

	// Warning
	void	warning1(WarningCode e, AST_Decl *t);
	void	warning2(WarningCode e, AST_Decl *t1, AST_Decl *t2);

	void 	attributeflag_error(ErrorCode e, sal_uInt32 flag);

	// Report a syntax error in IDL input
	void	syntax_error(IDL_Global::ParseState ps, char* errmsg);

	// Report a name being used with different spellings
	void	name_case_error(char *n, char *t);

	// Report an unsuccesful coercion attempt
	void	coercion_error(AST_Expression *v, AST_Expression::ExprType t);

	// Report a failed name lookup attempt
	void	lookup_error(UTL_ScopedName *n);

	// Report an attempt to use a forward declared interface which
	// hasn't been defined yet in an inheritance spec
	void	inheritance_fwd_error(UTL_ScopedName *n, AST_Interface *f);

	// Report an attempt to inherit from something other than an interface
	void	inheritance_error(UTL_ScopedName *n, AST_Decl *d);

	// Report an error while evaluating an expression (division by zero, etc.)
	void	eval_error(AST_Expression *d);

	// Report a situation where a constant was expected but we got
	// something else instead. This most likely is a case where a union
	// label did not evaluate to a constant
	void	constant_expected(UTL_ScopedName *n, AST_Decl *d);

	// Report a situation where an enumerator was expected but we got
	// something else instead. This occurs when a union with an enum
	// discriminator is being parsed and one of the branch labels is
	// not an enumerator in that enum
	void	enum_val_expected(AST_Union *t, AST_UnionLabel *l);

	// Report a failed enumerator lookup in an enum
	void	enum_val_lookup_failure(AST_Union *t, AST_Enum *e, UTL_ScopedName *n);

	// Report an ambiguous name definition
	void	ambiguous(UTL_Scope *s, AST_Decl *l, AST_Decl *d);

	// Report a forward declared interface which was never defined
	void	fwd_decl_not_defined(AST_Interface *d);

	// Report attempt to lookup in forward declared interface
	void	fwd_decl_lookup(AST_Interface *d, UTL_ScopedName *n);

	// Report a redefinition in scope
	void	redefinition_in_scope(AST_Decl *t, AST_Decl *s);

	// Report not a type error
	void	not_a_type(AST_Decl *d);

	// Report mapping error
	void  mapping_error(long lineno, String *s);

	// Report back-end error
	void  back_end(long lineno, String *s);
};

#endif	// _UTLERROR_HXX_

