/*************************************************************************
 *
 *  $RCSfile: utlmap.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _UTLMAP_HXX_
#define _UTLMAP_HXX_

#ifndef _IDLDEFS_HXX_
#include <idldefs.hxx>
#endif

#include <hash_map>
#include <list>
#include <utility>


class UTL_Map
{
public:
	// Operations

	// Constructor(s)
	UTL_Map();
	UTL_Map(const String& name);
	UTL_Map(const String& name, const String& super, const String& inc, 
			const String& getrefl, const String& increfl, 
			const String& ident, sal_Bool createAll);
	UTL_Map(const UTL_Map& rMap);

	~UTL_Map() {}

	void setMapName(const String& name) { m_MapName = name; }
	String& getMapName() { return m_MapName; }	

	void setMapSuper(const String& cl);
	const String& getMapSuper() { return m_MapSuper; }

	void setMapInc(const String& inc) { m_MapInc = inc; }
	const String& getMapInc() { return m_MapInc; }

	void setGetReflection(const String& getrefl) { m_GetReflection = getrefl; }
	const String& getReflection() { return m_GetReflection; }

	void setIncReflection(const String& increfl) { m_IncReflection = increfl; }
	const String& getIncReflection() { return m_IncReflection; }

	void setMapIdent(const String& ident) { m_MapIdent = ident; }
	const String& getMapIdent() { return m_MapIdent; }

	void setCreateAllListener(sal_Bool createAll) { m_CreateAllListener = createAll; }
	sal_Bool getCreateAllListener() { return m_CreateAllListener; }

	sal_Bool hasMoreSuperInterfaces() { return !m_SuperInterfaces.empty(); }
private:
	
	void checkNewlines(String& str);

	// Data
	String		m_MapName;
	String		m_MapSuper;
	String		m_MapInc;
	String		m_GetReflection;
	String 		m_IncReflection;
	String		m_MapIdent;
	sal_Bool	m_CreateAllListener;

	typedef NAMESPACE_STD(list) <String> SuperInterfaceList;
	SuperInterfaceList m_SuperInterfaces;
};


class UTL_MapTable
{
public:
	/// Constructor.
	UTL_MapTable();
	/// Destructor.
	~UTL_MapTable() {}

	void		setMappingOf(const String langauge, const String name, UTL_Map* mapping);
	UTL_Map* 	getMappingOf(const String& language, const String& name);
	
protected:	

	struct EqualString
	{
		bool operator()(const String& str1, const String& str2) const
		{
			return (str1 == str2);
		}
	};

	struct HashString
	{
		static NAMESPACE_STD(hash)<const char*> h;
		size_t operator()(const String& str) const
		{
            return h(str.getStr());
		}
	};

	typedef NAMESPACE_STD(hash_map) <const String, UTL_Map*, HashString, EqualString> MappingSection;
	typedef NAMESPACE_STD(hash_map) <const String, MappingSection, HashString, EqualString> MappingTable;

	MappingTable		m_mappingTable;
};	

/*
 *
 *
 */
class UTL_MapSection
{
public:
	// Enum defining the different kinds of mapping items
	enum SectionType 
	{
		ST_super,
		ST_include,
		ST_getrefl,
		ST_increfl,
		ST_ident,
		ST_allListener
	};

	// Constructor(s)
	UTL_MapSection(SectionType type, const String name)
		: m_sectionType(type), m_str(name), m_Flag(sal_False) {}
	
	UTL_MapSection(SectionType type, sal_Bool flag)
		: m_sectionType(type), m_str(""), m_Flag(flag) {}
	
	UTL_MapSection(const UTL_MapSection& rMapSection)
		: m_sectionType(rMapSection.m_sectionType)
		, m_str(rMapSection.m_str)
		, m_Flag(rMapSection.m_Flag) 
		{}

	~UTL_MapSection() {}

	SectionType 	section_type() { return m_sectionType; }
	const String 	getString() { return m_str; }	
	sal_Bool 		getFlag() { return m_Flag; }
		
protected:
	
	SectionType		m_sectionType;
	const String	m_str;
	sal_Bool		m_Flag;
};

typedef NAMESPACE_STD(list) <UTL_MapSection*> SectionList;


#endif	// _UTLMAP_HXX_
