/*************************************************************************
 *
 *  $RCSfile: utlnlist.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _UTLNLIST_HXX_
#define _UTLNLIST_HXX_


class UTL_NList;							   
class SortedUTL_NList;
class UTL_NListElem;

#define NCAST(x) ((UTL_NListElem*)x)

class UTL_NListIter
{
public:
	UTL_NListIter(UTL_NList& l );
	UTL_NListIter(UTL_NList& l, UTL_NListElem *act);

	UTL_NListElem *goBegin();
	UTL_NListElem *goEnd();
	UTL_NListElem *next();
	UTL_NListElem *prev();
	UTL_NListElem *act();
  
	unsigned char atEnd();
	unsigned char atBegin();
	void remove();

private:
	UTL_NList         *list_;
	UTL_NListElem     *act_;
	unsigned char  atBegin_, atEnd_;
};

class UTL_NListElem
{
	friend class UTL_NList;
	friend class SortedUTL_NList;
	friend class UTL_NListIter;
public:
	UTL_NListElem();
	virtual ~UTL_NListElem();

	virtual int compare(const UTL_NListElem *) =0;
	virtual UTL_NListElem *clone()             =0;

private:
	UTL_NListElem *next_;
	UTL_NListElem *prev_;
};

inline	UTL_NListElem::UTL_NListElem() : next_(0), prev_(0) {};

inline	UTL_NListElem::~UTL_NListElem() {};

class UTL_NList
{
	friend class UTL_NListIter;
public:
	UTL_NList();
	UTL_NList(const UTL_NList&);
	~UTL_NList();
	UTL_NList& operator=(const UTL_NList&);

	UTL_NListElem *append(void*);
	UTL_NListElem *insertOnes(void*);
	void       deleteAll();
	void       remove(UTL_NListElem *);
	unsigned int getNumElems() const;

	UTL_NListElem *getFirstElem();
	UTL_NListElem *getLastElem();

	unsigned char empty();

protected:
	void clone(const UTL_NList&);
  
	UTL_NListElem *first_;
	UTL_NListElem *last_;
	unsigned int numElems_;
};

class SortedUTL_NList: public UTL_NList 
{
	friend class UTL_NListIter;
public:
	UTL_NListElem *append(void *t){ return insert(t); };
	UTL_NListElem *insert(void *);
	UTL_NListElem *search(const void *);
};

#endif 	//_UTLNLIST_HXX_
