/*************************************************************************
 *
 *  $RCSfile: utlscope.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _UTLSCOPE_HXX_
#define _UTLSCOPE_HXX_


#ifndef _IDLFWD_HXX_
#include	<idlfwd.hxx>
#endif
#ifndef _IDLNRW_HXX_
#include	<idlnrw.hxx>
#endif
#ifndef _ASTDECL_HXX_
#include	<astdecl.hxx>
#endif
#ifndef _ASTEXPR_HXX_
#include	<astexpr.hxx>
#endif
#ifndef _UTLSCPNA_HXX_
#include	<utlscpna.hxx>
#endif

// Forward declaration of active iterator for UTL_Scope
class UTL_ScopeActiveIterator;

class UTL_Scope
{
public:
	// Enum to denote the kind of iteration desired
	enum ScopeIterationKind 
	{
		IK_both,		// Iterate through both decls and local types
		IK_decls,		// Iterate only through decls
		IK_localtypes	// Iterate only through local types
	};

	// Operations

	// Constructor(s)
	UTL_Scope();
	UTL_Scope(AST_Decl::NodeType nt);
	virtual ~UTL_Scope(); // {}

	// Scope Management Protocol
	virtual AST_PredefinedType	*add_predefined_type(AST_PredefinedType	*t);
	virtual AST_Object			*add_object(AST_Object	*ob);
	virtual AST_Service			*add_service(AST_Service	*ct);
	virtual AST_InterfaceMember	*add_interface_member(AST_InterfaceMember	*clm);
	virtual AST_ServiceMember	*add_service_member(AST_ServiceMember	*sm);
	virtual AST_Observes		*add_observes(AST_Observes	*obm);
	virtual AST_Needs			*add_needs(AST_Needs	*nm);
	virtual AST_Module			*add_module(AST_Module	*m);
	virtual AST_Constants		*add_constants(AST_Constants	*c);
	virtual AST_Mapping			*add_mapping(AST_Mapping	*m);
	virtual AST_Interface		*add_interface(AST_Interface	*i);
	virtual AST_InterfaceFwd	*add_interface_fwd(AST_InterfaceFwd	*i);
	virtual AST_Constant		*add_constant(AST_Constant	*c);
	virtual AST_Exception		*add_exception(AST_Exception	*e);
	virtual AST_Attribute		*add_attribute(AST_Attribute	*a);
	virtual AST_Operation		*add_operation(AST_Operation	*o);
	virtual AST_Argument		*add_argument(AST_Argument	*a);
	virtual AST_Union			*add_union(AST_Union	*u);
	virtual AST_UnionBranch		*add_union_branch(AST_UnionBranch	*b);
	virtual AST_Structure		*add_structure(AST_Structure	*s);
	virtual AST_Field			*add_field(AST_Field	*f);
	virtual AST_Enum			*add_enum(AST_Enum	*e);
	virtual AST_EnumVal			*add_enum_val(AST_EnumVal	*v);
	virtual AST_Typedef			*add_typedef(AST_Typedef	*t);
	virtual UTL_StrList			*add_context(UTL_StrList	*c);
	virtual UTL_NameList		*add_exceptions(UTL_NameList	*e);
	virtual AST_Sequence		*add_sequence(AST_Sequence	*s);
	virtual AST_String			*add_string(AST_String	*s);
	virtual AST_Array			*add_array(AST_Array	*a);

	// Call back end add_XXX functions for this scope
	virtual AST_Decl 		*call_add();

	// Data Accessors
	const	AST_Decl::NodeType scope_node_type()
	{
		return pd_scope_node_type;
	}

	// Narrowing
	DEF_NARROW_METHODS0(UTL_Scope);
	DEF_NARROW_FROM_SCOPE(UTL_Scope);

	// AST Dumping
	virtual void	dump(ostream &o);

	// Other Operations

	// Name Lookup Mechanism
	virtual AST_Decl	*lookup_by_name(UTL_ScopedName *, sal_Bool treat_as_ref);

	// special Name Lookup Mechanism to find defined interfaces
	virtual AST_Decl	*check_by_name(UTL_ScopedName *, sal_Bool treat_as_ref);

	virtual AST_Decl	*lookup_in_forwarded(UTL_ScopedName *e, sal_Bool);

	// Look up the Identifier * specified only in the local scope
	virtual AST_Decl	*lookup_by_name_local(Identifier *, sal_Bool treat_as_ref);

	// Look up a predefined type by its ExprType
	virtual AST_Decl 	*lookup_primitive_type(AST_Expression::ExprType);

	// How many entries are used?
	virtual unsigned long		nmembers();

public: // be_attribute::be_attribute eends to access add_to_scope.
	// UTL_Scope protected addition protocol. This protocol is
	// used from sub-classes of UTL_Scope to insert nodes directly
	// into the specified node table (either decls or local_types)
	// Additionally, UTL_Scope also maintains a record of what nodes
	// defined elsewhere were referenced in this scope

	// Add to decls. Node represents a local declaration
	void		add_to_scope(AST_Decl *e);

	// Add to local types. Node represents a local manifest type
	void		add_to_local_types(AST_Decl *e);

	// Add to referenced nodes set.
	void		add_to_referenced(AST_Decl *e, sal_Bool recursive);

	// Has this node been referenced here already?
	sal_Bool	referenced(AST_Decl *e);

	// Look up a scoped name in the inherited interfaces of an
	// interface
	virtual AST_Decl	*look_in_inherited(UTL_ScopedName *, sal_Bool treat_as_ref);
protected:
	// Operations

	// Lookup based on the local name
	virtual AST_Decl	*lookup_for_add(AST_Decl *d, sal_Bool treat_as_ref);

private:
	// Data

	// What type is this node? This is a cache of the information
	// in AST_Decl
	const	AST_Decl::NodeType	pd_scope_node_type;

	// Storage for declarations in this scope:
	AST_Decl	**pd_decls;					// Store declarations
	long		pd_decls_allocated;			// How many allocated?
	long		pd_decls_used;				// How many used?

	// Storage for local manifest types in this scope:
	AST_Decl	**pd_local_types;			// Store types
	long		pd_locals_allocated;		// How many allocated?
	long		pd_locals_used;				// How many used?

	// Storage for referenced nodes in this scope:
	AST_Decl	**pd_referenced;			// Store references
	long		pd_referenced_allocated;	// How many allocated?
	long		pd_referenced_used;			// How many used?

	// Friend class UTL_ScopeActiveIterator defines active iterator for
	// UTL_Scope. Definition follows below.
	friend class			UTL_ScopeActiveIterator;

private:
	friend int yyparse();
	friend class AST_Enum;

	// Scope Management Protocol
	virtual AST_PredefinedType	*fe_add_predefined_type(AST_PredefinedType *t);
	virtual AST_Object			*fe_add_object(AST_Object	*ob);
	virtual AST_Service			*fe_add_service(AST_Service	*ct);
	virtual AST_InterfaceMember	*fe_add_interface_member(AST_InterfaceMember	*clm);
	virtual AST_ServiceMember	*fe_add_service_member(AST_ServiceMember	*sm);
	virtual AST_Observes		*fe_add_observes(AST_Observes	*obm);
	virtual AST_Needs			*fe_add_needs(AST_Needs	*nm);
	virtual AST_Module			*fe_add_module(AST_Module	*m);
	virtual AST_Constants		*fe_add_constants(AST_Constants	*c);
	virtual AST_Mapping			*fe_add_mapping(AST_Mapping	*m);
	virtual AST_Interface		*fe_add_interface(AST_Interface	*i);
	virtual AST_InterfaceFwd	*fe_add_interface_fwd(AST_InterfaceFwd	*i);
	virtual AST_Constant		*fe_add_constant(AST_Constant	*c);
	virtual AST_Exception		*fe_add_exception(AST_Exception	*e);
	virtual AST_Attribute		*fe_add_attribute(AST_Attribute	*a);
	virtual AST_Operation		*fe_add_operation(AST_Operation	*o);
	virtual AST_Argument		*fe_add_argument(AST_Argument	*a);
	virtual AST_Union			*fe_add_union(AST_Union	*u);
	virtual AST_UnionBranch		*fe_add_union_branch(AST_UnionBranch	*b);
	virtual AST_Structure		*fe_add_structure(AST_Structure	*s);
	virtual AST_Field			*fe_add_field(AST_Field	*f);
	virtual AST_Enum			*fe_add_enum(AST_Enum	*e);
	virtual AST_EnumVal			*fe_add_enum_val(AST_EnumVal	*v);
	virtual AST_Typedef			*fe_add_typedef(AST_Typedef	*t);
	virtual UTL_StrList			*fe_add_context(UTL_StrList	*c);
	virtual UTL_NameList		*fe_add_exceptions(UTL_NameList	*e);
	virtual AST_Sequence		*fe_add_sequence(AST_Sequence	*s);
	virtual AST_String			*fe_add_string(AST_String	*s);
	virtual AST_Array			*fe_add_array(AST_Array	*a);
};

// Active iterator for a UTL_Scope node
class UTL_ScopeActiveIterator
{
public:
	// Operations

	// Constructor(s)
	UTL_ScopeActiveIterator(UTL_Scope *s, UTL_Scope::ScopeIterationKind ik);

	// Advance to next item
	virtual void			next();

	// Get current item
	virtual AST_Decl		*item();

	// Have we iterated over entire scope?
	virtual sal_Bool		is_done();

	// What kind of iterator is this?
	virtual UTL_Scope::ScopeIterationKind	iteration_kind();

	// What stage are we in with this iterator?
	virtual UTL_Scope::ScopeIterationKind	iteration_stage();

private:
	// Data
	UTL_Scope						*iter_source;	// Scope to iterate over
	UTL_Scope::ScopeIterationKind	ik;				// What kind of iteration
	UTL_Scope::ScopeIterationKind	stage;			// What stage
	long							il;				// What location in stage
};

#endif	// _UTLSCOPE_HXX_

