/*************************************************************************
 *
 *  $RCSfile: drvargs.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif
#ifndef _DRVPRIV_HXX_
#include <drvpriv.hxx>
#endif
#ifndef _DRVLINK_HXX_
#include <drvlink.hxx>
#endif

#include <stdio.h>

/*
 * Push a file into the list of files to be processed
 */
static void DRV_push_file(char *s)
{

	String filename(s);
	String extension;
	
	filename = filename.toUpperCase();
	sal_Int32 pos = filename.lastIndexOf('.');
	if (pos > 0)
	{
		extension = filename.copy(pos + 1);		
	} else
	{
		fprintf(stderr, "%s\nERROR: please check input filename, suffix must be .idl | .smr\n"
			   "wrong input file: %s\n",
			   IDL_Global::idl_global()->prog_name(), s);
		exit(99);
	}

	if ( extension != "IDL" != 0 && extension != "SMR")
	{
		fprintf(stderr, "%s\nERROR: please check input filename, suffix must be .idl | .smr\n"
			   "wrong input file: %s\n",
			   IDL_Global::idl_global()->prog_name(), s);

		exit(99);
	} else
	{
		if (extension == "SMR" && IDL_Global::idl_global()->be() != "s2u")
		{
			IDL_Global::idl_global()->set_be("smart");						
		}
	}
	
	DRV_files[DRV_nfiles++] = strdup(s);
}

/*
 * Prepare a CPP argument
 */
static void DRV_prep_cpp_arg(char *s)
{
	char *newarg = new char[2048];
	char *farg;

	newarg[0] = '\0';
	for (farg = strtok(s, ","); farg != NULL; farg = strtok(NULL, ","))
		strcat(newarg,farg);
	DRV_cpp_putarg(newarg);
}

/*
 * Prepare include option
 */
static void DRV_prep_includes(char* inc)
{
	if (inc != NULL)
	{
		char *ioptions = strdup(inc);
#ifdef UNX
		char seperators[] = ":";
#else
		char seperators[] = ";";
#endif
		char *token = NULL;
		char *option = NULL;
			
		token = strtok(ioptions, seperators);
		while (token != NULL)
		{
			option = new char[strlen(token)+3];
			strcpy(option, "-I");
			strcat(option, token);
			DRV_cpp_putarg(option);
			delete(option);
	
			token = strtok(NULL, seperators);
		}
		free(ioptions);
	}
}	
 
/*
 * Print a usage message and exit
 */
void DRV_usage()
{
	fprintf(stderr, "%s: usage: %s [flag | file]*\nLegal flags:\n"
		   " -E\t\t  truns preprocessor only, prints on stdout\n"
		   " -D<name>\t  defines macro for the preprocessor\n"
		   " -OH<dir>\t  produce header files (or complete output) in dir (pathname)\n"
		   " -OI<dir>\t  produce implemetation files in dir (pathname)\n"
		   "         \t  This option is only used in the 'smart' and 'cpp' backend\n"
		   " -I<dir>\t  includes dir in search path for preprocessor\n"
		   " -P<prefix>\t  use prefix as include prefix for generated header file\n"
		   " -N<name>\t  name=ProjectName, used only in smart to check map includes\n"
		   " -V\t\t  prints version info then exits\n"
		   " -Wb,arg1,argn\t  passes args to BE\n"
		   " -B<backend> \t  use backend for code generation\n"
		   "           \t  supported backends are 'urd' for Uno Reflection Data.\n"
/*
		   "           \t                       | 'cpp'   for Remote Uno C++\n"
		   "           \t                       | 's2u'   for SmartToUno wrapper\n"
		   "           \t                       | 'java'  for Remote Uno Java\n"
		   "           \t                       | 'smart' for Smart Uno\n"
		   "           \t                       | 'doc'   for documentation\n"
		   "           \t                       | 'xml'   for xml documentation\n"
*/
		   "           \t  Default: 'urd' backend is used\n"
		   " -Mdepend[=match] generate dependecies for all generated files to stdout,\n"
		   "                  files beginning with 'match' are not used for dependencies.\n"
//		   "                  No dependencies for the doc backend.\n"
		   " -h\t\t  prints this usage message and exits\n"
		   " -v\t\t  traces compilation stages\n"
		   " -w\t\t  suppresses IDL compiler warning messages\n\n%s"
		   "\nUNO-IDL Compiler Version 1.0, Sun Microsystems, Inc."
		   "\nCopyright (C) 1999-2000. All rights reserved.\n",
		   IDL_Global::idl_global()->prog_name(),
		   IDL_Global::idl_global()->prog_name(),
		   IDL_Global::idl_global()->prog_name());
}

/*
 * Parse arguments on command line
 */
void DRV_parse_args(long ac, char **av, sal_Bool cmd)
{
	FILE	*commandfile;
	char	filename[2048];
	String 	options, space(" ");
	char 	*buffer;
	char  	*s;
	long  	i;

	DRV_cpp_init();
	if (!cmd)
	{
		cmd = sal_True;
		
		IDL_Global::idl_global()->set_prog_name(DRV_stripped_name(av[0]));
	
		if (ac < 2)
		{
			DRV_usage();
			exit(99);
		}
		
		i = 1;
	} else
	{
		i = 0;
	}

	for (i; i < ac; i++)
	{
		if (av[i][0] == '-')
		{
			switch (av[i][1])
			{
				case 0:
					DRV_push_file("standard input");
					break;
				case 'A':
 					options += space + av[i];
					if (av[i][2] == '\0')
					{
						if (i < ac - 1)
						{
							i++;
							s = av[i];
						} else
							exit(99);
					} else
						s = av[i] + 2;
					strcat(IDL_Global::idl_global()->local_escapes(), s);
					strcat(IDL_Global::idl_global()->local_escapes(), " ");
					break;
				case 'D':
				case 'U':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
						if (i < ac - 1)
						{
							buffer = new char[strlen(av[i]) + strlen(av[i + 1]) + 2];
							sprintf(buffer, "%s%s", av[i], av[i+1]);
							DRV_cpp_putarg(buffer);
							delete(buffer);
							i++;
						} else
						{
							fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
								   IDL_Global::idl_global()->prog_name(), av[i]);
							exit(99);
						}
					} else
						DRV_cpp_putarg(av[i]);
					break;
				case 'I':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
							fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
								   IDL_Global::idl_global()->prog_name(), av[i]);
						exit(99);
					} else
				   //		DRV_cpp_putarg(av[i]);
				   		DRV_prep_includes(av[i]+2);
					break;
				case 'E':
					options += space + av[i];
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
                                            IDL_CF_ONLY_PREPROC);
					break;
				case 'V':
					options += space + av[i];
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
											IDL_CF_VERSION);
					break;
				case 'W':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
						if (i < ac - 1)
						{
							i++;
							s = av[i];
						} else
						{
							fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
								   IDL_Global::idl_global()->prog_name(), av[i]);
							exit(99);
						}
					} else
						s = av[i] + 2;
					switch (*s)
					{
						default:
							fprintf(stderr, "%s\nERROR: -W must be followed by 'p' or 'b'\n",
								   IDL_Global::idl_global()->prog_name());
							exit(99);
						case 'p':
							if (*(s + 1) == ',')
							{
								DRV_prep_cpp_arg(s + 2);
							} else
							{
								fprintf(stderr, "%s\nERROR: -Wp must be followed by ',arg1,argn' \n",
									   IDL_Global::idl_global()->prog_name());
								exit(99);
							}
							break;
						case 'b':
							if (*(s + 1) == ',')
							{
								//(*DRV_BE_prep_arg)(s + 2, sal_True);
								IDL_Global::idl_global()->set_be_args(s+2);
							} else 
							{
								fprintf(stderr, "%s\nERROR: -Wb must be followed by ',arg1,argn' \n",
									   IDL_Global::idl_global()->prog_name());
								exit(99);
							}
							break;
					}
					break;
				case 'Y':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
						if (i < ac - 1)
						{
							i++;
							s = av[i];
						} else
						{
							fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
								   IDL_Global::idl_global()->prog_name(), av[i]);
							exit(99);
						}
					} else
						 s = av[i] + 2;
					switch (*s)
					{
						case 'p':
							if (*(s + 1) == ',')
							{
								IDL_Global::idl_global()->set_cpp_location(s + 2);
								DRV_cpp_new_location(s + 2);
							}
							break;
						default:
							break;
					}
					break;
				case 'O':
					options += space + av[i];
					s = av[i] + 2;
					switch (*s)
					{
						default:
							fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
								   IDL_Global::idl_global()->prog_name(), av[i]);
							exit(99);
						case 'H':
							IDL_Global::idl_global()->setHeaderDestination(s+1);
							break;
						case 'I':
							IDL_Global::idl_global()->setImplDestination(s+1);
							break;
					}
		           	break;
			    case 'B':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
						fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
							   IDL_Global::idl_global()->prog_name(), av[i]);
						exit(99);
					} else
					{
						s = av[i] + 2;
						if ( strcmp(s, "urd") != 0 )
						{
							av[i][2] = '\0';
							fprintf(stderr, "%s\nERROR: wrong argument after '%s"
								   "' flag, supported backend is 'urd'\n",
								   IDL_Global::idl_global()->prog_name(), av[i]);
							exit(99);
						}
					}

			       	IDL_Global::idl_global()->set_be(s);
				   	break;
			    case 'P':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
						fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
							   IDL_Global::idl_global()->prog_name(), av[i]);
						exit(99);
					} else
					{
						s = av[i] + 2;
					}

			       	IDL_Global::idl_global()->setIncludePrefix(s);
				   	break;
			    case 'F':
					options += space + av[i];
					if (av[i][2] == '\0'
						|| av[i][2] != 'P')
					{
						fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
							   IDL_Global::idl_global()->prog_name(), av[i]);
						exit(99);
					} else
					{
						s = av[i] + 3;
					}

			       	IDL_Global::idl_global()->setFilePrefix(s);
				   	break;
			    case 'N':
					options += space + av[i];
					if (av[i][2] == '\0')
					{
						fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
							   IDL_Global::idl_global()->prog_name(), av[i]);
						exit(99);
					} else
					{
						s = av[i] + 2;
					}

			       IDL_Global::idl_global()->setProjectName(s);
				   break;
			    case 'M':
					options += space + av[i];
					if (strncmp(av[i]+2, "depend", 6) != 0)
					{
						fprintf(stderr, "%s\nERROR: missing argument after '%s' flag\n",
							   IDL_Global::idl_global()->prog_name(), av[i]);
						exit(99);
					}

					if (strlen(av[i]) > 9 && *(av[i]+8) == '=')
					{
						String negativeMatch(av[i]+9);
						IDL_Global::idl_global()->setNegativeMatch(negativeMatch);	
					}

			       IDL_Global::idl_global()->setMakeDepend(sal_True);
				   break;
				case 'd':
					options += space + av[i];
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
											IDL_CF_DUMP_AST);
					break;
				case 'h':
					options += space + av[i];
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
											IDL_CF_ONLY_USAGE);
					break;
				case 'v':
					options += space + av[i];
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
											IDL_CF_INFORMATIVE);
				break;
				case 'w':
					options += space + av[i];
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
											IDL_CF_NOWARNINGS);
					break;
				default:
					fprintf(stderr, "%s: Illegal option '%s'\n", IDL_Global::idl_global()->prog_name(), av[i]);
					IDL_Global::idl_global()
						->set_compile_flags(IDL_Global::idl_global()->compile_flags() |
											IDL_CF_ONLY_USAGE);
					break;
			}
		} else
		{
			if (av[i][0] == '@')
			{
				commandfile = fopen(av[i]+1, "r");
		  		if( commandfile == NULL )
      			{
					fprintf(stderr, "%s\nERROR: commandfile %s could not be open.\n",
						   IDL_Global::idl_global()->prog_name(), av[i]);
					exit(99);
				} else
				{
					long rargc=0;
					char* rargv[2048];
					
					while ( fscanf(commandfile, "%s", filename) != EOF )
					{
						rargv[rargc]= strdup(filename);
						rargc++;
						//DRV_push_file(filename);
					}
					fclose(commandfile);
					
					DRV_parse_args(rargc, rargv, cmd);
					
					for (long i=0; i < rargc; i++) 
					{
						free(rargv[i]);
					}
				}		
			} else
			{
				DRV_push_file(av[i]);
			}
		}
	} // end for

	IDL_Global::idl_global()->set_options((char*)options.getStr());
}
