/*************************************************************************
 *
 *  $RCSfile: drvmain.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif
#ifndef _DRVPRIV_HXX_
#include <drvpriv.hxx>
#endif
#ifndef _DRVLINK_HXX_
#include <drvlink.hxx>
#endif


// Storage for file names
#define	NFILES	2048

char    *DRV_files[NFILES];
long    DRV_nfiles = 0;
long    DRV_file_index = -1;

static void DRV_version()
{
	fprintf(stderr, "%s %s %s",  IDL_Global::idl_global()->prog_name(),
		    "\nUNO-IDL Compiler Version 1.0, Sun Microsystems, Inc",
		    "\nCopyright (C) 1999-2000. All rights reserved\n");
	//(*DRV_BE_version)();
}

/*
** Drive the compilation
**
** LOGIC:
**
** 1. Initialize the CFE, stage 1. This builds the scope stack
** 2. Initialize the BE. This builds an instance of the generator
** 3. Initialize the CFE, stage 2. This builds the global scope
**    and populates it with the predefined types
** 4. Invoke FE_yyparse
** 5. Check for errors from FE_yyparse. If any, exit now
** 6. Check for undefined forward declared interfaces. If any, exit now
** 7. Check if asked to dump AST. If so, do.
** 8. Invoke BE.
*/
void DRV_drive(char *s)
{
	/*
	 * Pass through CPP
	 */
	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_INFORMATIVE)
	  fprintf(stderr, "%s%s%s\n", IDL_Global::idl_global()->prog_name(),
			 GTDEVEL(": preprocessing "),
			 s);

	DRV_pre_proc(s);
	/*
	 * Initialize FE stage 1
	 */
	(*DRV_FE_init_stage1)();
	/*
	 * Initialize BE
	 */
	IDL_Global::idl_global()->set_gen((*DRV_BE_init)());
	/*
	 * Initialize FE stage 2
	 */
	(*DRV_FE_init_stage2)();
	/*
	 * Parse
	 */
	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_INFORMATIVE)
	  fprintf(stderr, "%s%s%s\n", IDL_Global::idl_global()->prog_name(),
			 GTDEVEL(": parsing "),
			 s);

	(*DRV_FE_yyparse)();
	/*
	 * If there were any errors, stop
	 */
	if (IDL_Global::idl_global()->err_count() > 0)
	{
	  fprintf(stderr, "%s: %s: found %d error%s\n",
	  		 IDL_Global::idl_global()->prog_name(),
			 s,
			 IDL_Global::idl_global()->err_count(),
			 (IDL_Global::idl_global()->err_count() > 1 ? "s" : ""));
			 
		/*
		 * Call BE_abort to allow a BE to clean up after itself
		 */
		(*DRV_BE_abort)();

		int number_of_errors = IDL_Global::idl_global()->err_count();
		DRV_BE_close();
		exit(number_of_errors);
		//exit((int) IDL_Global::idl_global()->err_count());
	}
	/*
	 * Dump the code
	 */
	if ((IDL_Global::idl_global()->compile_flags() & IDL_CF_INFORMATIVE)
		&& (IDL_Global::idl_global()->compile_flags() & IDL_CF_DUMP_AST))
		fprintf(stderr, "%s: dump %s\n", IDL_Global::idl_global()->prog_name(), s);

	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_DUMP_AST)
	{
//		fprintf(stderr, "Dump of AST:\n");
//		IDL_Global::idl_global()->root()->dump(cerr);
	}
	/*
	 * Call the main entry point for the BE
	 */
	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_INFORMATIVE)
		fprintf(stderr, "%s: BE processing on %s\n", IDL_Global::idl_global()->prog_name(), s);

	(*DRV_BE_produce)();

	if (IDL_Global::idl_global()->be() != "cpp" 
		&& IDL_Global::idl_global()->be() != "s2u")
	{
		fprintf(stderr, "%s:   compile file > %s, ok\n", IDL_Global::idl_global()->prog_name(), s);
	}
}

// Funtkion zum aendern von GlobalInit in osl
extern "C" void ChangeGlobalInit();

/*
 * IDL compiler main program. Logic as explained in comment at head
 * of file.
 */
int UNOIDL_CALLTYPE main(int argc, char **argv)
{
#ifdef SOLARIS
	// HACK!!! don't call thread_self before main 
	ChangeGlobalInit();
#endif
	/*
	 * Open front-end library
	 */
	DRV_FE_open();
	/*
	 * Initialize driver and global variables
	 */
	DRV_init();
	/*
	 * Parse arguments
	 */
	DRV_parse_args(argc, argv, sal_False);
	/*
	 * Open back-end library
	 */
	DRV_BE_open();

	/*
	 * If args for the backend, put args 
	 */
	if(IDL_Global::idl_global()->be_args() != NULL)
	{
		(*DRV_BE_prep_arg)(IDL_Global::idl_global()->be_args(), sal_True);
	} 
	
	/*
	 * If a version message is requested, print it and exit
	 */
	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_VERSION)
	{
		DRV_version();
		exit(0);
	}
	/*
	 * If a usage message is requested, give it and exit
	 */
	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_ONLY_USAGE)
	{
		DRV_usage();
		exit(0);
	}
	/*
	 * Fork off a process for each file to process. Fork only if
	 * there is more than one file to process
	 */
	if (DRV_nfiles > 1)
	{
		DRV_fork(argv[0]);
	} else
	{	
		/*
		 * Do the one file we have to parse
		 *
		 * Check if stdin and handle file name appropriately
		 */
		if (DRV_nfiles == 0)
		{
			fprintf(stderr, "%s\nERROR: no input file, please specify an *.idl | *.smr input file\n",
				   IDL_Global::idl_global()->prog_name());
			exit(99);
		}
		DRV_file_index = 0;
		DRV_drive(DRV_files[DRV_file_index]);
	}

	fprintf(stderr, "%s\nStarDivision (R) UNO-IDL Compiler Version 1.0"
		   "\nCopyright (C) StarDivision GmbH 1997. All rights reserved"
		   "\nIDL-Compiler returned successful\n",
		   IDL_Global::idl_global()->prog_name());

	DRV_BE_close();

	for (DRV_file_index = 0; DRV_file_index < DRV_nfiles; DRV_file_index++) 
	{
		free(DRV_files[DRV_file_index]);
	}

	return(0);
}
