/*************************************************************************
 *
 *  $RCSfile: drvprepr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _VOS_PROCESS_HXX_
#include <vos/process.hxx>
#endif

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif
#ifndef _DRVPRIV_HXX_
#include <drvpriv.hxx>
#endif
#ifndef _DRVLINK_HXX_
#include <drvlink.hxx>
#endif

#include <fstream.h>

#ifdef	SAL_W32
#include <io.h>
#endif

#ifdef	SAL_OS2
#include <io.h>
#endif

#ifdef	SAL_UNX
#include <unistd.h>           	// POSIX standard types
#ifdef MACOSX
#include <sys/wait.h>             	// POSIX definition of wait()
#else
#include <wait.h>             	// POSIX definition of wait()
#endif
#endif

#ifdef 	apollo
#include <sysent.h>
#endif

#if defined(hpux) || defined(__hpux) || defined(SUNOS4)
#include <unistd.h>            	// POSIX definitions
#include <sys/wait.h>          	// POSIX definition of wait()
#endif // defined(hpux) || defined(__hpux) || defined(SUNOS4)

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#undef   MAX_ARGLIST
#define  MAX_ARGLIST	512

using namespace rtl;
using namespace vos;

static   char  *arglist[MAX_ARGLIST];
static   int  	argcount = 0;

/*
 * Push the new CPP location
 */
void DRV_cpp_new_location(char *new_loc)
{
	arglist[0] = new_loc;
}

/*
 * Push an argument into the arglist
 */
void DRV_cpp_putarg(char *str)
{
	if (str != NULL)
	{
		for (int i=0; i < argcount; i++)
		{
			if (arglist[i] != NULL)
				if (strcmp(arglist[i], str) == 0)
					return;
		}

		if (argcount >= MAX_ARGLIST)
		{
			fprintf(stderr, "%s: More than %d arguments to preprocessor\n",
				   IDL_Global::idl_global()->prog_name(), MAX_ARGLIST);
			exit (99);
		}
		arglist[argcount++] = strdup(str);
	}
}

/*
 * Initialize the cpp argument list
 */
void DRV_cpp_init()
{
	for (int i=0; i < argcount; i++)
	{
		if (arglist[i] != NULL)
			free(arglist[i]);
	}

	argcount = 0;

	//DRV_cpp_putarg(IDL_Global::idl_global()->cpp_location());
	//DRV_cpp_putarg("-EP");
	DRV_cpp_putarg("-DIDL");
	DRV_cpp_putarg("-Xi");
	DRV_cpp_putarg("-Xc");
	DRV_cpp_putarg("-+");
	DRV_cpp_putarg("-I.");
}

/*
 * lines can be 1024 chars long
 */
#define  LINEBUF_SIZE	1024
static   char  drv_line[LINEBUF_SIZE + 1];

/*
 * Get a line from stdin
 */
static long DRV_get_line(FILE *f)
{
	char *l = fgets(drv_line, LINEBUF_SIZE, f);
	char  *s = l;
	long  i;


	if (l == NULL)
		return sal_False;
	if (*l == '\0' && feof(f))
		return sal_False;

	while (*s == ' ' || *s == '\t')
		s++;
	if (*l == '\0')
		return sal_True;
	i = strlen(l) - 1;
	if (l[i] == '\n')
		l[i] = '\0';
	return sal_True;
}

/*
 * Copy from stdin to a file
 */
static void DRV_copy_input(FILE *fin, char *fn)
{
	FILE  *f = fopen(fn, "w");

	if (f == NULL)
	{
		fprintf(stderr, "%s: cannot open temp file %s for writing\n",
			   IDL_Global::idl_global()->prog_name(), fn);
		exit(99);
	}

	while (DRV_get_line(fin))
		fprintf(f, "%s\n", drv_line);
	fclose(f);
}

/*
 * Strip down a name to the last component, i.e. everything after the last
 * '/' character
 */
const char *DRV_stripped_name(char *fn)
{
	char *n = fn;
	long l;

	if (n == NULL)
		return NULL;
	l = strlen(n);

#if !defined(WIN32) && !defined(__OS2__) && !defined(WNT)
	for (n += l; l > 0 && *n != '/'; l--, n--);
	if (*n == '/') n++;
#else
	for (n += l; l > 0 && *n != '\\'; l--, n--);
	if (*n == '\\') n++;
#endif
	return n;
}

/*
 * File names
 */
static char tmp_file[512];
static char tmp_ifile[512];
static char tmp_cmds[512];

/*
 * Pass input through preprocessor
 */
void DRV_pre_proc(char *myfile)
{
	long  readfromstdin = sal_False;
	OUString uTmpPath;
	OString  tmpPath;
	OStartupInfo StartupInfo;

	if (IDL_Global::idl_global()->be() == "urd")
	{
		DRV_cpp_putarg("-DTYPE_XIDLCLASS=type");
	} else
	{
		DRV_cpp_putarg("-DTYPE_XIDLCLASS=com::sun::star::reflection::XIdlClass");
	}

	if (StartupInfo.getEnvironment(OUString(RTL_CONSTASCII_USTRINGPARAM("TMP")), uTmpPath) != OStartupInfo::E_None)
	{
		if (StartupInfo.getEnvironment(OUString(RTL_CONSTASCII_USTRINGPARAM("TEMP")), uTmpPath) != OStartupInfo::E_None)
		{
#if defined(WIN32) || defined(WNT) || defined(OS2)
			tmpPath = OString("c:\\temp");
#else
			tmpPath = OString("/tmp");
#endif		
		}
	}

	if (uTmpPath.getLength())
	{
		tmpPath = OUStringToOString(uTmpPath, RTL_TEXTENCODING_UTF8);
	}

#if defined(WIN32) || defined(WNT)	
	strcpy(tmp_file, tmpPath);
	strcat(tmp_file, "\\");
	strcat(tmp_file, "idlf_XXXXXX");
	strcpy(tmp_ifile, tmpPath);
	strcat(tmp_ifile, "\\");
	strcat(tmp_ifile, "idli_XXXXXX");
	strcpy(tmp_cmds, tmpPath);
	strcat(tmp_cmds, "\\");
	strcat(tmp_cmds, "idlc_XXXXXX");
	
	(void) mktemp(tmp_file); strcat(tmp_file, ".idl");
	(void) mktemp(tmp_ifile); strcat(tmp_ifile, ".idl");
	(void) mktemp(tmp_cmds);
#endif

#ifdef __OS2__
	strcpy(tmp_file, tempnam(NULL, "idlf"));
	strcpy(tmp_ifile, tempnam(NULL, "idli"));
	strcpy(tmp_cmds, tempnam(NULL, "idlc"));
//	strcat(tmp_file, ".idl");
//	strcat(tmp_ifile, ".idl");
#endif

#ifdef UNX
	strcpy(tmp_file, tmpPath);
	strcat(tmp_file, "/");
	strcat(tmp_file, "idlf_XXXXXX");
	strcpy(tmp_ifile, tmpPath);
	strcat(tmp_ifile, "/");
	strcat(tmp_ifile, "idli_XXXXXX");
	strcpy(tmp_cmds, tmpPath);
	strcat(tmp_cmds, "/");
	strcat(tmp_cmds, "idlc_XXXXXX");

	(void) mktemp(tmp_file); strcat(tmp_file, ".idl");
	(void) mktemp(tmp_ifile); strcat(tmp_ifile, ".idl");
	(void) mktemp(tmp_cmds);
#endif

	if (strcmp(myfile, "standard input") == 0)
	{
		IDL_Global::idl_global()->set_filename(new String(tmp_ifile));
		IDL_Global::idl_global()->set_main_filename(new String(tmp_ifile));
		IDL_Global::idl_global()->
			set_stripped_filename(new String(DRV_stripped_name(tmp_ifile)));
		IDL_Global::idl_global()->set_real_filename(new String(tmp_ifile));
		DRV_copy_input(stdin, tmp_ifile);
		IDL_Global::idl_global()->set_read_from_stdin(sal_True);
	} else
	{
		FILE *fd = fopen(myfile, "r");
		if(!fd) {
		  fprintf(stderr, "%s: couldn't open file: %s\n", IDL_Global::idl_global()->prog_name(), myfile);
		  exit(99);
		}

		DRV_copy_input(fd, tmp_ifile);
		fclose(fd);
		IDL_Global::idl_global()->set_read_from_stdin(sal_False);
		IDL_Global::idl_global()->set_filename(new String(myfile));
		IDL_Global::idl_global()->set_main_filename(new String(myfile));
		IDL_Global::idl_global()->
			set_stripped_filename(new String(DRV_stripped_name(myfile)));
		IDL_Global::idl_global()->set_real_filename(new String(tmp_ifile));
	}

	char *s = NULL;
	char incPath [2048] = "";

#if defined(WIN32) || defined(__OS2__) || defined(WNT)
	if(s = ::strrchr(myfile, '\\'))
#else
	if(s = ::strrchr(myfile, '/'))
#endif
	{
		strcpy(incPath, "-I");
		strncpy(incPath+2, myfile, strlen(myfile)-strlen(s));
		incPath[strlen(myfile)-strlen(s)+2] = '\0';
	}

	NAMESPACE_VOS(OProcess) 				*pProcess;
	NAMESPACE_VOS(OProcess)::TProcessInfo 	Info;

	OUString  cpp;
	VOS_VERIFY(StartupInfo.getExecutableFile(cpp) == OStartupInfo::E_None);

#if defined(WIN32) || defined(__OS2__) || defined(WNT)
	cpp = cpp.copy(0, cpp.getLength() - 10);
	cpp += OUString( RTL_CONSTASCII_USTRINGPARAM("unocpp.exe"));
#else
	cpp = cpp.copy(0, cpp.getLength() - 6);
	cpp += OUString( RTL_CONSTASCII_USTRINGPARAM("unocpp"));
#endif

	FILE  *cmdFile = fopen(tmp_cmds, "w");

	if (cmdFile == NULL)
	{
		fprintf(stderr, "%s: cannot open temp file %s for writing\n",
			   IDL_Global::idl_global()->prog_name(), tmp_cmds);
		exit(99);
	}

	for (int i=0; i < argcount ; i++)
	{
		fprintf(cmdFile, "%s ", arglist[i]);
	}

	fprintf(cmdFile, "%s ", incPath);
	fprintf(cmdFile, "%s ", tmp_ifile);
	fprintf(cmdFile, "%s ", tmp_file);

	fclose(cmdFile);

	String cmds = "@";
	cmds += tmp_cmds;

	const OUString	cmdArg(cmds.getStr(), cmds.getLength(), RTL_TEXTENCODING_UTF8 );
	OArgumentList	cmdArgs(1, &cmdArg);
	
	pProcess = new OProcess(cpp);
	OProcess::TProcessError exitcode;

	if (pProcess == NULL ||
		(exitcode = pProcess->execute(OProcess::TOption_Wait, cmdArgs)) != OProcess::E_None)
	{
		fprintf(stderr, "%s: Start Preprocessor failed\n",
			   IDL_Global::idl_global()->prog_name());
	
		unlink(tmp_ifile);
		unlink(tmp_file);
		unlink(tmp_cmds);
		delete(pProcess);
		exit(exitcode);
	}
	
	VOS_VERIFY(pProcess->getInfo(NAMESPACE_VOS(OProcess)::TData_ExitCode, &Info) == NAMESPACE_VOS(OProcess)::E_None);
	
	if ((int)(Info.Code) != 0)
	{
		fprintf(stderr, "%s: Preprocessor returned with errors\n",
			   IDL_Global::idl_global()->prog_name());

		unlink(tmp_ifile);
		unlink(tmp_file);
		unlink(tmp_cmds);
		delete(pProcess);
		exit((int)(Info.Code));	
	}
	
	delete(pProcess);
	
	FILE * yyin = fopen(tmp_file, "r");
	if (yyin == NULL)
	{	
		fprintf(stderr, "%s: Could not open cpp output file %s\n",
			   IDL_Global::idl_global()->prog_name(), tmp_file);
		exit(99);
	}

	(*DRV_FE_set_yyin)((File *) yyin, tmp_file);
	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_ONLY_PREPROC)
	{
		sal_Char Buffer[2048];
		sprintf(Buffer, "type %s", tmp_file);
		system(Buffer);
	}

	if (unlink(tmp_ifile) != 0)
	{
		fprintf(stderr, "%s: Could not remove cpp input file %s\n",
			   IDL_Global::idl_global()->prog_name(), tmp_ifile);
		exit(99);
	}

	if (unlink(tmp_cmds) != 0)
	{
		fprintf(stderr, "%s: Could not remove unocpp command file %s\n",
			   IDL_Global::idl_global()->prog_name(), tmp_cmds);	
		
		exit(99);
	}

	if (IDL_Global::idl_global()->compile_flags() & IDL_CF_ONLY_PREPROC)
	{
		fclose(yyin);
		if (unlink(tmp_file) != 0)
		{
			fprintf(stderr, "%s: Could not remove parser input file %s\n",
				   IDL_Global::idl_global()->prog_name(), tmp_file);
			exit(99);
		}

		exit(0);
	}
}
