%{
/*
 * idl.ll - Lexical scanner for UNOIDL 1.0
 */
#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLNEW_HXX_
#include <idlnew.hxx>
#endif

#include <string.h>

class FE_Declarator;
class FE_InterfaceHead;
class FE_ExceptionHeader;
class FE_StructHeader;
class FE_OperationHead;
class FE_AttributeHead;
class FE_Ident;

#include "idl_tab.hxx"


#if defined(SAL_W32) || defined(SAL_OS2)
#define YY_NEVER_INTERACTIVE 1
#endif

#if defined(SAL_32)
extern int yyinput(void);
#endif

extern int yylex(void);

// needed for generating documentation
static char idldoc_text[30000];

static char		*stripped_name(String *fn);
static char		idl_escape_reader(char *);
static double	idl_atof(char *);
static long		idl_atoi(char *, long);
static void		idl_parse_line_and_file(char *);
static void		idl_store_pragma(char *);

// HPUX has yytext typed to unsigned char *. We make sure here that
// we'll always use char *
// static char *__yytext = (char *)yytext;

// __yytext nicht mehr benutzt, Probleme da Pointer immmer NULL
// konsistente Umbenennung von __yytext in yytext
// Achtung bei Portierung !!!

%}

%s Name
%s MapInclude
%s MapSuper
%x MappedName

%%

<INITIAL>object			return IDL_OBJECT;
<INITIAL>service	   	return IDL_SERVICE;
<INITIAL>observes	   	return IDL_OBSERVES;
<INITIAL>needs	   		return IDL_NEEDS;
<INITIAL>property  		return IDL_PROPERTY;
<INITIAL>module			return IDL_MODULE;
<INITIAL>raises			return IDL_RAISES;
<INITIAL>readonly		return IDL_READONLY;
<INITIAL>optional		return IDL_OPTIONAL;
<INITIAL>maybevoid		return IDL_MAYBEVOID;
<INITIAL>bound			return IDL_BOUND;
<INITIAL>constrained	return IDL_CONSTRAINED;
<INITIAL>transient		return IDL_TRANSIENT;
<INITIAL>maybeambiguous	return IDL_MAYBEAMBIGUOUS;
<INITIAL>maybedefault	return IDL_MAYBEDEFAULT;
<INITIAL>removable		return IDL_REMOVEABLE;
<INITIAL>attribute		return IDL_ATTRIBUTE;
<INITIAL>exception		return IDL_EXCEPTION;
<INITIAL>context		return IDL_CONTEXT;
<INITIAL>interface		return IDL_INTERFACE;
<INITIAL>const			return IDL_CONST;
<INITIAL>constants		return IDL_CONSTANTS;
<INITIAL>typedef		return IDL_TYPEDEF;
<INITIAL>struct			return IDL_STRUCT;
<INITIAL>enum			return IDL_ENUM;
<INITIAL>string			return IDL_STRING;
<INITIAL>wstring_t		return IDL_WSTRING;
<INITIAL>sequence		return IDL_SEQUENCE;
<INITIAL>union			return IDL_UNION;
<INITIAL>switch			return IDL_SWITCH;
<INITIAL>case			return IDL_CASE;
<INITIAL>default		return IDL_DEFAULT;
<INITIAL>float			return IDL_FLOAT;
<INITIAL>double			return IDL_DOUBLE;
<INITIAL>byte			return IDL_BYTE;
<INITIAL>hyper			return IDL_HYPER;
<INITIAL>long			return IDL_LONG;
<INITIAL>short			return IDL_SHORT;
<INITIAL>unsigned		return IDL_UNSIGNED;
<INITIAL>char			return IDL_CHAR;
<INITIAL>wchar_t		return IDL_WCHAR;
<INITIAL>boolean		return IDL_BOOLEAN;
<INITIAL>octet			return IDL_OCTET;
<INITIAL>void			return IDL_VOID;
<INITIAL>type			return IDL_TYPE;
<INITIAL>implementation return IDL_IMPLEMENTATION;

<INITIAL>TRUE			return TRUETOK;
<INITIAL>FALSE			return FALSETOK;
<INITIAL>True			return TRUETOK;
<INITIAL>False			return FALSETOK;

<INITIAL>inout			return IDL_INOUT;
<INITIAL>in				return IDL_IN;
<INITIAL>out			return IDL_OUT;
<INITIAL>oneway			return IDL_ONEWAY;

<INITIAL>local		   	return IDL_LOCAL;

<INITIAL>name	{
	BEGIN(Name);
	return NAME;
}

<INITIAL>ident	{
	BEGIN(Name);
	return IDENT;
}

<INITIAL>virtual		return VIRTUAL;
<INITIAL>uik			return UIK;
<INITIAL>version		return VERSION;

<INITIAL>language		return LANGUAGE;
<INITIAL>mapping		return MAPPING;

<INITIAL>superinterface	{
	BEGIN(MapSuper);
	return MAPSUPER;
}

<INITIAL>getreflection	{
	BEGIN(Name);
	return GETREFLECTION;
}

<INITIAL>includereflection {
	BEGIN(MapInclude);
	return INCREFLECTION;
}

<INITIAL>createAllListener {
	BEGIN(INITIAL);
	return CREATEALLISTENER;
}

<INITIAL>include	{
	BEGIN(MapInclude);
	return MAPINC;
}

<INITIAL>map	{
	BEGIN(MappedName);
	return MAP;
}
 
<INITIAL>identifier		return MAPIDENT;

<Name>[_a-zA-Z][a-zA-Z0-9_]* {
	//char *z = (char *) malloc(strlen(yytext) + 1);
	char *z = new char[strlen(yytext) + 1];
	strcpy(z, yytext);
	yylval.strval = z;
	BEGIN(INITIAL);
	return IDENTIFIER;
}

<Name>\"[a-z_A-Z][a-zA-Z0-9_.:()]*\" {
	//char* name = (char*) malloc(strlen(yytext) - 1);
	char* name = new char[strlen(yytext) - 1];
	yytext[strlen(yytext)-1]='\0';
	strcpy(name, yytext+1);
	yylval.strval = name;
	BEGIN(INITIAL);
	return NAMEBODY;
}

<MapInclude>\"[^\"]*\" {
	//char* name = (char*) malloc(strlen(yytext) - 1);
	char* name = new char[strlen(yytext) - 1];
	yytext[strlen(yytext)-1]='\0';
	strcpy(name, yytext+1);
	yylval.strval = name;
	BEGIN(INITIAL);
	return MAPINCLUDE;
}

<MapSuper>[a-zA-Z{][a-zA-Z0-9}; ]* {
	//char *z = (char*) malloc(strlen(yytext) + 1);
	char *z = new char[strlen(yytext) + 1];
	strcpy(z, yytext);
	yylval.strval = z;
	BEGIN(INITIAL);
	return MAPSUPERNAME;
}

<MappedName>[a-zA-Z][a-zA-Z0-9<>,_ ]* {
	//char *z = (char*) malloc(strlen(yytext) + 1);
	char *z = new char[strlen(yytext) + 1];
	strcpy(z, yytext);
	yylval.strval = z;
	BEGIN(INITIAL);
	return MAPNAME;
}

[0-9a-fA-F]{8}"-"[0-9a-fA-F]{4}"-"[0-9a-fA-F]{4}"-"[0-9a-fA-F]{8}"-"[0-9a-fA-F]{8} {
	//char* uik = (char*) malloc(strlen(yytext) + 1);
	char* uik = new char[strlen(yytext) + 1];
	strcpy(uik, yytext);
	yylval.strval = uik;
	return UIK_BODY;
}

\<\<   return LEFT_SHIFT;
\>\>   return RIGHT_SHIFT;

\:\: {
	yylval.strval = "::";
	return SCOPE_DELIMITOR;
}

[_a-zA-Z][a-zA-Z0-9_]* {
	//char *z = (char *) malloc(strlen(yytext) + 1);
	char *z = new char[strlen(yytext) + 1];
	strcpy(z, yytext);
	yylval.strval = z;
	return IDENTIFIER;
}

-?[0-9]+"."[0-9]*([eE][+-]?[0-9]+)?[lLfF]? {
	yylval.dval = idl_atof(yytext);
	return FLOATING_PT_LITERAL;
}

-?[0-9]+[eE][+-]?[0-9]+[lLfF]? {
	yylval.dval = idl_atof(yytext);
	return FLOATING_PT_LITERAL;
}

-?[1-9][0-9]* {
	yylval.ival = idl_atoi(yytext, 10);
	return INTEGER_LITERAL;
}

-?[0-9]*[lLuU] {
	yylval.sval = new String(yytext);
	return INTEGER_L_LITERAL;
}

-?0[xX][a-fA-F0-9]+ {
	yylval.ival = idl_atoi(yytext, 16);
	return INTEGER_LITERAL;
}

-?0[0-7]* {
	yylval.ival = idl_atoi(yytext, 8);
	return INTEGER_LITERAL;
}

"\""[^\"]*"\"" {
	yytext[strlen(yytext)-1] = '\0';
	yylval.sval = new String(yytext + 1);
	return STRING_LITERAL;
}

"'"."'" {
	yylval.cval = yytext[1];
	return CHARACTER_LITERAL;
}

"'"\\([0-7]{1,3})"'" {
	// octal character constant
	yylval.cval = idl_escape_reader(yytext + 1);
	return CHARACTER_LITERAL;
}

"'"\\."'" {
	yylval.cval = idl_escape_reader(yytext + 1);
	return CHARACTER_LITERAL;
}

^#[ \t]*pragma[ \t].*\n        {       /* remember pragma */
	IDL_Global::idl_global()->set_lineno(IDL_Global::idl_global()->lineno() + 1);
	idl_store_pragma(yytext);
}

^#[ \t]*line[ \t]*[0-9]*" ""\""[^\"]*"\""\n    {
	idl_parse_line_and_file(yytext);
}

^#[ \t]*[0-9]*" ""\""[^\"]*"\""" "[0-9]*\n {
	idl_parse_line_and_file(yytext);
}

^#[ \t]*[0-9]*" ""\""[^\"]*"\""\n {
	idl_parse_line_and_file(yytext);
}

^#[ \t]*[0-9]*\n {
	idl_parse_line_and_file(yytext);
}

^#[ \t]*ident.*\n {
	/* ignore cpp ident */
	IDL_Global::idl_global()->set_lineno(IDL_Global::idl_global()->lineno() + 1);
}

<*>"//" {
	IDL_Global::idl_global()->set_lineno(IDL_Global::idl_global()->lineno() + 1);
	sal_Bool validDoc = sal_False;
	
	if (yyinput() == '/')
	{
 		validDoc = sal_True;
		IDL_Global::idl_global()->set_idldoc_valid(1);
	}
	
	for(int p=0;p<1000;p++) 
	{
		char c = yyinput();
		if (validDoc)
			idldoc_text[p] = c;

		if (c == '\n') 
		{
			if (validDoc)
				idldoc_text[p] = '\0';
			break;
		}
	}

	if (validDoc)
		IDL_Global::idl_global()->set_idldoc_text(idldoc_text);
}

<*>"/**"	{
	/* magic 'idldoc' comments */
	/* whoever copies this text out must reset idldoc_valid to 0 */
	IDL_Global::idl_global()->set_idldoc_valid(1);
	for(int p=0;p<30000;p++) 
	{
		char c = idldoc_text[p] = yyinput();
		if (c == '*') 
		{
			char next = yyinput();
			if (next == '/') 
			{
				idldoc_text[p] = '\0';
				break;
       		} else
#if defined(SAL_W32) || defined(SAL_OS2) || defined(SAL_UNX)
				unput(next);
#else
				yyunput(next);
#endif
		}
		if (c == '\n') 
			IDL_Global::idl_global()
				->set_lineno(IDL_Global::idl_global()->lineno() + 1);	
	}
	IDL_Global::idl_global()->set_idldoc_text(idldoc_text);
}

<*>"/*" {
	for(;;)
    {
		char c = yyinput();
		if (c == '*')
		{
			char next = yyinput();
			if (next == '/')
				break;
			else
#if defined(SAL_W32) || defined(SAL_OS2) || defined(SAL_UNX)
				unput(next);
#else
				yyunput(next);
#endif
		}

		if (c == '\n')
			IDL_Global::idl_global()->
				set_lineno(IDL_Global::idl_global()->lineno() + 1);
	}
}

<*>[ \t]*   ;

<*>\n {
	IDL_Global::idl_global()->set_lineno(IDL_Global::idl_global()->lineno() + 1);
}

.	return yytext[0];

%%
       /* subroutines */

/*
 * Strip down a name to the last component, i.e. everything after the last
 * '/' character
 */
static char *stripped_name(String *fn)
{
    char       *n = (char*)fn->getStr();
    long       l;

    if (n == NULL)
               return NULL;
    l = strlen(n);
#if !defined(SAL_W32) && !defined(SAL_OS2)
    for (n += l; l > 0 && *n != '/'; l--, n--);
    if (*n == '/') n++;
#else
    for (n += l; l > 0 && !((*n == '\\') || (*n == '/')); l--, n--)
	{
		
	}
    if ((*n == '\\') || (*n == '/')) n++;
#endif
    return n;
}

/*
 * Parse a #line statement generated by the C preprocessor
 */
static void idl_parse_line_and_file(char *buf)
{
	char            *r = buf;
	char            *h;
	sal_Bool		inmain = sal_False;
	short 			import;

	/* Skip initial '#' */
	if (*r != '#')
		return;

	/* Find line number */
	for (r++; *r == ' ' || *r == '\t' || isalpha(*r); r++);
	h = r;
	for (; *r != '\0' && *r != ' ' && *r != '\t'; r++);
	*r++ = 0;
	IDL_Global::idl_global()->set_lineno(idl_atoi(h, 10));

	/* Find file name, if present */
	for (; *r != '"'; r++)
	{
		if (*r == '\n' || *r == '\0')
			return;
	}
	h = ++r;
	for (; *r != '"'; r++);
	*r = 0;
	if (*h == '\0')
		IDL_Global::idl_global()->set_filename(new String("standard input"));
	else
		IDL_Global::idl_global()->set_filename(new String(h));

	inmain = (*(IDL_Global::idl_global()->filename()) ==
			  *(IDL_Global::idl_global()->real_filename())) ? sal_True : sal_False;

	if (!inmain) 
	{
		import = IDL_Global::idl_global()->getImportOfFile(*(IDL_Global::idl_global()->filename()));
		inmain = import ? sal_True : sal_False;
	}
	
	IDL_Global::idl_global()->set_in_main_file(inmain);	
}

/*
 * Store a #pragma line into the list of pragmas
 */
static void idl_store_pragma(char *buf)
{
	String *file = NULL;
	
	char *cp = buf + 1;
	while(*cp != 'p')
		cp++;
	while(*cp != ' ' && *cp != '\t')
		cp++;
	while(*cp == ' ' || *cp == '\t')
		cp++;
	char *pragma = (char*)malloc(strlen(buf)+1);
	char *pp = pragma;
	char *inc, *incf;
	while(*cp != '\n')
		*pp++ = *cp++;

	*pp = 0;
	if (strncmp(pragma, "package", 7) == 0)
	{	
		pp = pragma + 7;
		while(*pp == ' ' || *pp == '\t') pp++;
		if (*pp == '=')
		{
			pp++;
		} else
		{
			// Fehler
		}
		while(*pp == ' ' || *pp == '\t') pp++;

		cp = pp;

		if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
		{
			file = IDL_Global::idl_global()->main_filename();
		} else
		{
			file = IDL_Global::idl_global()->filename();
		}
		
		IDL_Global::idl_global()
			->addFilePackagePrefix(new String(*file), new String(cp));

		free(pragma);
		return;
	}
	if (strncmp(pragma, "import", 6) == 0)
	{
		inc = pp = pragma + 7;
		
		while(*pp != ',') pp++;
		*pp = '\0';
		
		pp += 2;
		incf = pp;
		while(*pp != ')') pp++;
		*(--pp) = '\0';
		
		IDL_Global::idl_global()->setImportOfFile(incf, 1);

		IDL_Global::idl_global()->set_import(sal_True);
		
		free(pragma);
		return;
	}
	if (strncmp(pragma, "include", 7) == 0)
	{
		inc = pp = pragma + 8;
		
		if (*inc == '"') *inc = '\"';

		while(*pp != ',') pp++;
		*pp = '\0';
		if (*(pp-1) == '"') *(pp-1) = '\"';

		pp += 2;
		incf = pp;
		while(*pp != ')') pp++;
		*(--pp) = '\0';
		
		IDL_Global::idl_global()->setIncludeFile(inc);
		IDL_Global::idl_global()->setImportOfFile(incf, 0);
		IDL_Global::idl_global()->setDepIncFile(incf);					

		IDL_Global::idl_global()->set_import(sal_False);
		
		free(pragma);
		return;
	}

	UTL_StrList *p = IDL_Global::idl_global()->pragmas();
	if (p == NULL)
		IDL_Global::idl_global()->set_pragmas(new UTL_StrList(new String(buf), NULL));
	else
	{
		p->nconc(new UTL_StrList(new String(buf), NULL));
		IDL_Global::idl_global()->set_pragmas(p);
	}
	free(pragma);
}

/*
 * idl_atoi - Convert a string of digits into an integer according to base b
 */
static long idl_atoi(char *s, long b)
{
	long    r = 0;
	long    negative = 0;

       if (*s == '-')
       {
               negative = 1;
               s++;
       }
       if (b == 8 && *s == '0')
               s++;
       else if (b == 16 && *s == '0' && (*(s + 1) == 'x' || *(s + 1) == 'X'))
               s += 2;

       for (; *s; s++)
               if (*s <= '9' && *s >= '0')
                       r = (r * b) + (*s - '0');
               else if (b > 10 && *s <= 'f' && *s >= 'a')
                       r = (r * b) + (*s - 'a' + 10);
               else if (b > 10 && *s <= 'F' && *s >= 'A')
                       r = (r * b) + (*s - 'A' + 10);
               else
                       break;

       if (negative)
               r *= -1;

       return r;
}

/*
 * Convert a string to a float; atof doesn't seem to work, always.
 */
static double idl_atof(char *s)
{
       char    *h = s;
       double  d = 0.0;
       double  f = 0.0;
       double  e, k;
       long    neg = 0, negexp = 0;

       if (*s == '-')
       {
               neg = 1;
               s++;
       }
       while (*s >= '0' && *s <= '9')
       {
               d = (d * 10) + *s - '0';
               s++;
       }
       if (*s == '.')
       {
               s++;
               e = 10;
               while (*s >= '0' && *s <= '9')
               {
                       d += (*s - '0') / (e * 1.0);
                       e *= 10;
                       s++;
               }
       }
       if (*s == 'e' || *s == 'E')
       {
               s++;
               if (*s == '-')
               {
                       negexp = 1;
                       s++;
               } else
                       if (*s == '+')
                               s++;
               e = 0;
               while (*s >= '0' && *s <= '9')
               {
                       e = (e * 10) + *s - '0';
                       s++;
               }
               if (e > 0)
               {
                       for (k = 1; e > 0; k *= 10, e--);
                       if (negexp)
                               d /= k;
                       else
                               d *= k;
               }
       }

       if (neg) d *= -1.0;

       return d;
}

/*
 * Convert (some) escaped characters into their ascii values
 */
static char idl_escape_reader(char *str)
{
    if (str[0] != '\\')
               return str[0];

    switch (str[1])
       {
               case 'n':
                       return '\n';
               case 't':
                       return '\t';
               case 'v':
                       return '\v';
               case 'b':
                       return '\b';
               case 'r':
                       return '\r';
               case 'f':
                       return '\f';
               case 'a':
                       return '\a';
               case '\\':
                       return '\\';
               case '\?':
                       return '?';
               case '\'':
                       return '\'';
               case '"':
                       return '"';
               case 'x':
                       {
                       // hex value
                       int i;
                       for (i = 2; str[i] != '\0' && isxdigit(str[i]); i++)
                       {
                               continue;
                       }
                       char save = str[i];
                       str[i] = '\0';
                       char out = (char)idl_atoi(&str[2], 16);
                       str[i] = save;
                       return out;
                       }
                       break;
               default:
                       // check for octal value
                       int i;
                       if (str[1] >= '0' && str[1] <= '7')
                       {
                               for (i = 1; str[i] >= '0' && str[i] <= '7'; i++)
                               {
                                       continue;
                               }
                               char save = str[i];
                               str[i] = '\0';
                               char out = (char)idl_atoi(&str[1], 8);
                               str[i] = save;
                               return out;
                       } else
                       {
                               return str[1] - 'a';
                       }
                       break;
       }
}

/*
 *
 */
#if defined(SAL_W32) || defined(SAL_OS2) || defined(SAL_UNX)
int yywrap()
{
	return 1;
}
#endif
