/*
 * idl.yy - YACC grammar for UNOIDL 1.0
 */

/* Declarations */

%{
#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif
#ifndef _FEDECL_HXX_
#include <fedecl.hxx>
#endif
#ifndef _FEHEADS_HXX_
#include <feheads.hxx>
#endif

#if defined (UNX)
#include <stdlib.h>
#if defined (SOLARIS)
#include <alloca.h>
#endif
#endif

#include <stdio.h>

#ifdef SAL_OS2
#include <malloc.h>
#endif

// global head pointer fuer frontend head definitionen
// die nicht auf dem Stack abgelegt werden
void *globalHead = NULL;

#define YYDEBUG 1
#define YYERROR_VERBOSE 1

#if defined(SAL_W32) || defined(SAL_OS2)
extern int yylex(void);
extern void yyerror(char *);
extern int yywrap(void);
#else

#ifdef SAL_UNX
extern int yylex(void);
extern void yyerror(char *);
#endif

#if (defined(apollo) || defined(hpux)) && defined(__cplusplus)
extern "C" int yylex();
extern "C" int yywrap();
void yyerror(char *);
#endif // (defined(apollo) || defined(hpux)) && defined(__cplusplus)

#endif // (SAL_W32)

%}

/*
 * Declare the type of values in the grammar
 */

%union {
	AST_Decl					*dcval;         /* Decl value           */
	UTL_StrList					*slval;         /* String list          */
	UTL_NameList				*nlval;         /* Name list            */
	UTL_ExprList				*elval;         /* Expression list      */
	UTL_LabelList				*llval;         /* Label list           */
	UTL_DeclList				*dlval;         /* Declaration list     */
	FE_InterfaceHeader			*ihval;         /* Interface header     */
	FE_InterfaceHead			*ihdval;        /* Interface head       */
	FE_ExceptionHeader			*ehval;         /* Exception header     */
	FE_StructHeader				*shval;         /* Structure header     */
	FE_Ident					*idtval;		/* Interface head       */
	FE_OperationHead			*ohdval;        /* Operation head       */
	FE_AttributeHead			*ahdval;        /* Attribute head       */
	AST_Expression				*exval;         /* Expression value     */
	AST_UnionLabel				*ulval;         /* Union label          */
	AST_Field					*ffval;         /* Field value          */
	AST_Expression::ExprType	etval;  		/* Expression type      */
	AST_Argument::Direction		dival;  		/* Argument direction   */
	AST_Operation::Flags		ofval;  		/* Operation flags              */
	IFlag						oifval; 		/* Optional implementation flag */
	FE_Declarator				*deval; 		/* Declarator value            */
	sal_Bool	            bval;           		/* Boolean value        */
	long                ival;           		/* Long value           */
	double              dval;           		/* Double value         */
	float               fval;           		/* Float value          */
	char                cval;           		/* Char value           */

	String              *sval;          		/* String* value         */
	char                *strval;        		/* char * value         */
	Identifier          *idval;         		/* Identifier           */
	UTL_IdList			*idlist;        		/* Identifier list      */
	UTL_Map				*mapval;				/* mapping section		*/
	UTL_MapSection		*mapsec;
	SectionList			*seclist;
}

/*
 * Token types: These are returned by the lexer
 */

%token <strval>        IDENTIFIER

%token         IDL_OBJECT
%token		   IDL_IMPLEMENTS
%token		   IDL_PROPERTY
%token         IDL_SERVICE
%token		   IDL_OBSERVES
%token		   IDL_NEEDS
%token         IDL_CONST
%token         IDL_CONSTANTS
%token         IDL_MODULE
%token         IDL_INTERFACE
%token         IDL_TYPEDEF
%token         IDL_LONG
%token         IDL_HYPER
%token         IDL_BYTE
%token         IDL_SHORT
%token         IDL_UNSIGNED
%token         IDL_DOUBLE
%token         IDL_FLOAT
%token         IDL_CHAR
%token         IDL_WCHAR
%token         IDL_OCTET
%token         IDL_BOOLEAN
%token         IDL_ANY
%token         IDL_STRUCT
%token         IDL_UNION
%token         IDL_SWITCH
%token         IDL_ENUM
%token         IDL_SEQUENCE
%token         IDL_STRING
%token         IDL_WSTRING
%token         IDL_EXCEPTION
%token         IDL_CASE
%token         IDL_DEFAULT
%token         IDL_READONLY
%token         IDL_OPTIONAL
%token         IDL_MAYBEVOID
%token         IDL_BOUND
%token         IDL_CONSTRAINED
%token         IDL_TRANSIENT
%token         IDL_MAYBEAMBIGUOUS
%token         IDL_MAYBEDEFAULT
%token         IDL_REMOVEABLE
%token         IDL_ATTRIBUTE
%token         IDL_ONEWAY
%token         IDL_IDEMPOTENT
%token         IDL_VOID
%token         IDL_TYPE
%token         IDL_IN
%token         IDL_OUT
%token         IDL_INOUT
%token         IDL_RAISES
%token         IDL_CONTEXT
%token         IDL_IMPLEMENTATION

%token <ival>	INTEGER_LITERAL
%token <sval>	INTEGER_L_LITERAL
%token <sval>	STRING_LITERAL
%token <cval>	CHARACTER_LITERAL
%token <dval>	FLOATING_PT_LITERAL
%token			TRUETOK
%token			FALSETOK

%token <strval>	SCOPE_DELIMITOR
%token			LEFT_SHIFT
%token			RIGHT_SHIFT

%token			IDL_LOCAL

%token			NAME
%token <strval> NAMEBODY

%token			VIRTUAL
%token			UIK
%token			IDENT
%token			VERSION
%token <strval>	UIK_BODY

%token			LANGUAGE
%token			MAPPING
%token			MAP
%token <strval>	MAPNAME
%token			MAPSUPER
%token			MAPINC
%token			GETREFLECTION
%token			INCREFLECTION
%token			MAPIDENT
%token			CREATEALLISTENER
%token <strval>	MAPINCLUDE
%token <strval>	MAPSUPERNAME

/*
 * These are production names:
 */

%type <dcval>	type_spec simple_type_spec constructed_type_spec
%type <dcval>	template_type_spec sequence_type_spec string_type_spec

%type <dcval>	type_dcl struct_type enum_type switch_type_spec union_type
%type <dcval>	array_declarator op_type_spec seq_head wstring_type_spec

%type <idlist>	scoped_name
%type <slval>	opt_context at_least_one_string_literal
%type <slval>	string_literals

%type <nlval>	at_least_one_scoped_name scoped_names inheritance_spec
%type <nlval>	opt_raises

%type <elval>	at_least_one_array_dim array_dims

%type <llval>	at_least_one_case_label case_labels

%type <dlval>	at_least_one_declarator declarators

%type <ihval>	interface_header
%type <ehval>	exception_header
%type <shval>	structure_header

%type <exval>	expression const_expr or_expr xor_expr and_expr shift_expr
%type <exval>	add_expr mult_expr unary_expr primary_expr literal
%type <exval>	positive_int_expr array_dim

%type <ulval>	case_label

%type <ffval>	element_spec

%type <etval>	const_type integer_type char_type boolean_type
%type <etval>	floating_pt_type any_type signed_int implementation_type type_type
%type <etval>	unsigned_int base_type_spec octet_type byte_type

%type <dival>	direction

%type <ohdval>	operation_head
%type <ofval>	opt_op_attribute
%type <bval>	opt_const


%type <deval>	declarator simple_declarator complex_declarator

%type <ahdval>	attribute_head
%type <ahdval>	property_head

%type <idval>	interface_decl id

%type <ihdval>	interface_head
%type <strval>	uik
%type <idtval>	opt_ident

%type <strval>	object_head 

%type <mapval>	maps
%type <mapsec>  mapping_entry
%type <seclist> mapping_entries
%type <bval>	boolean_init

%type <ival>	service_interface_header

%%

/*
 * Production starts here
 */
start : definitions;

definitions :
	definition definitions
	| /* empty */
	;

definition :
	type_dcl
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_TypeDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| const_dcl
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| exception
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ExceptDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| interface_def
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| object
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| service
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| module
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleDeclSeen);
	}
	';'
	| constants
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| mapping
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MappingDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| language
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LanguageDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| error
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

module :
	IDL_MODULE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleSeen);
	}
	IDENTIFIER
	{
		UTL_ScopedName	*n = new UTL_ScopedName($3, NULL);
		AST_Module	*m = NULL;
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_StrList	*p = IDL_Global::idl_global()->pragmas();
		AST_Decl	*exists;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleIDSeen);
		/*
		 * Make a new module and add it to the enclosing scope
		 */
		if (s != NULL) 
		{
			m = IDL_Global::idl_global()->gen()->create_module(n, p);
			
			if ((exists = s->lookup_for_add(m, sal_False)) != NULL)
			{
				exists->set_in_main_file(IDL_Global::idl_global()->in_main_file());
				delete(m);
				m = AST_Module::narrow_from_decl(exists);
			} else
				(void) s->fe_add_module(m);
		}
		/*
		 * Push it on the stack
		 */
		IDL_Global::idl_global()->scopes()->push(m);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleSqSeen);
	}
	definitions
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleQsSeen);
		/*
		 * Finished with this module - pop it from the scope stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
	}
	;

constants :
	IDL_CONSTANTS
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleSeen);
	}
	IDENTIFIER
	{
		UTL_ScopedName	*n = new UTL_ScopedName($3, NULL);
		AST_Constants	*c = NULL;
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_StrList	*p = IDL_Global::idl_global()->pragmas();
		AST_Decl	*exists;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleIDSeen);
		/*
		 * Make a new module and add it to the enclosing scope
		 */
		if (s != NULL) 
		{
			c = IDL_Global::idl_global()->gen()->create_constants(n, p);
			
			if ((exists = s->lookup_for_add(c, sal_False)) != NULL)
			{
				exists->set_in_main_file(IDL_Global::idl_global()->in_main_file());
				delete(c);
				c = AST_Constants::narrow_from_decl(exists);
			} else
				(void) s->fe_add_constants(c);
		}
		/*
		 * Push it on the stack
		 */
		IDL_Global::idl_global()->scopes()->push(c);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleSqSeen);
	}
	constants_exports
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleQsSeen);
		/*
		 * Finished with this module - pop it from the scope stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
	}
	;

object :
	object_head
	IDL_OBJECT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectSeen);
	}
	IDENTIFIER
	{
		UTL_ScopedName	*n = new UTL_ScopedName($4, NULL);
		AST_Object	*cl = NULL;
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_StrList	*p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectIDSeen);
		/*
		 * Make a new object and add it to the enclosing scope
		 */
		if (s != NULL) 
		{
			cl = IDL_Global::idl_global()->gen()->create_object(n, p, $1);
			(void) s->fe_add_object(cl);
		}
		/*
		 * Push it on the stack
		 */
		IDL_Global::idl_global()->scopes()->push(cl);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectSqSeen);
	}
	object_exports
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectQsSeen);
		/*
		 * Finished with this class - pop it from the scope stack
		 */
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Object::narrow_from_decl(ScopeAsDecl(s))->checkExport();

		IDL_Global::idl_global()->scopes()->pop();
	}
	;

object_head :
	'['
	NAME
	'('
	NAMEBODY
	')'
	']'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectHeadSeen);
		$$ = $4;
	}
	;

object_exports :
	object_exports object_export
	| /* EMPTY */
	;

object_export :
	IDL_INTERFACE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_InterfaceMember *m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;

				m = IDL_Global::idl_global()->gen()->create_interface_member(AF_INVALID, n, p);
				
				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_interface) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setInterface(AST_Interface::narrow_from_decl(cbn));
						(void) s->fe_add_interface_member(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| IDL_SERVICE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_ServiceMember   *m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_service_member(n, p);

				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_service) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setService(AST_Service::narrow_from_decl(cbn));
						(void) s->fe_add_service_member(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| IDL_OBSERVES
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Observes	    *m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_observes(n, p);
				
				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_interface) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setInterface(AST_Interface::narrow_from_decl(cbn));
						(void) s->fe_add_observes(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| IDL_NEEDS
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Needs			*m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_needs(n, p);

				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_service) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setService(AST_Service::narrow_from_decl(cbn));
						(void) s->fe_add_needs(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| property
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_PropertyDeclSeen);
	}
	';'
	| error
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

property :
	property_head
	simple_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_PropertyTypeSeen);
	}
	at_least_one_declarator
	{
		UTL_Scope               *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_DecllistActiveIterator *l = NULL;
		AST_Attribute           *a = NULL;
		FE_Declarator           *d = NULL;
		UTL_StrList             *p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrCompleted);
		/*
		 * Create nodes representing attributes and add them to the
		 * enclosing scope
		 */
		if (s != NULL && $2 != NULL && $4 != NULL)
		{
			l = new UTL_DecllistActiveIterator($4);
			for (;!(l->is_done()); l->next()) {
				d = l->item();
				if (d == NULL)
					continue;
				AST_Type *tp = d->compose($2);
				if (tp->is_impl_type())
				IDL_Global::idl_global()->err()
					->error0(UTL_Error::EIDL_TYPE_NOT_VALID);

				if (tp == NULL)
					continue;
				a = IDL_Global::idl_global()->gen()
						->create_attribute($1->getFlags(),
										   $1->implementation(), tp, d->name(), p);
				/*
				 * Add one attribute to the enclosing scope
				 */
				(void) s->fe_add_attribute(a);
			}
			delete(l);
		}
		delete($1);
		globalHead = NULL;
	}
	;

property_head :
	opt_attrflags
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_PropertyHeadSeen);

		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		if ( !fAHead->hasFlag(AF_PROPERTY) )
			IDL_Global::idl_global()->err()
				->attributeflag_error(UTL_Error::EIDL_MISSINGATTRIBUTEKEYWORD, AF_PROPERTY);
		if ( fAHead->hasFlag(AF_ATTRIBUTE) )
			IDL_Global::idl_global()->err()
				->attributeflag_error(UTL_Error::EIDL_WRONGATTRIBUTEKEYWORD, AF_ATTRIBUTE);

		$$ = (FE_AttributeHead*)globalHead;
	}
	;

service :
	IDL_SERVICE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceSeen);
	}
	IDENTIFIER
	{
		UTL_ScopedName	*n = new UTL_ScopedName($3, NULL);
		AST_Service	*cl = NULL;
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_StrList		*p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceIDSeen);
		/*
		 * Make a new object and add it to the enclosing scope
		 */
		if (s != NULL) 
		{
			cl = IDL_Global::idl_global()->gen()->create_service(n, p);
			(void) s->fe_add_service(cl);
		}
		/*
		 * Push it on the stack
		 */
		IDL_Global::idl_global()->scopes()->push(cl);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceSqSeen);
	}
	service_exports
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceQsSeen);
		/*
		 * Finished with this class - pop it from the scope stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
	}
	;

service_exports :
	service_exports service_export
	| /* EMPTY */
	;

service_export :
	service_interface_header
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_InterfaceMember *m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_interface_member(($1 & AF_OPTIONAL), n, p);

				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_interface) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setInterface(AST_Interface::narrow_from_decl(cbn));
						(void) s->fe_add_interface_member(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| IDL_SERVICE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_ServiceMember   *m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ObjectMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_service_member(n, p);

				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_service) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setService(AST_Service::narrow_from_decl(cbn));
						(void) s->fe_add_service_member(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| IDL_OBSERVES
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Observes	    *m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_observes(n, p);

				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_interface) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setInterface(AST_Interface::narrow_from_decl(cbn));
						(void) s->fe_add_observes(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| IDL_NEEDS
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceMemberSeen);
	}
	at_least_one_scoped_name
	';'
	{
		UTL_Scope           *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Needs			*m = NULL;
		UTL_StrList         *p = IDL_Global::idl_global()->pragmas();
		UTL_ScopedName 		*n = NULL;
		AST_Decl			*cbn = NULL;
		UTL_NamelistActiveIterator *l;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ServiceMemberSeen);
		/*
		 * Create a node representing a class member.
		 * Store it in the enclosing scope
         */
		if (s != NULL && $3 != NULL) 
		{
			l = new UTL_NamelistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				n = l->item();
				if (n == NULL)
					continue;
				
				m = IDL_Global::idl_global()->gen()->create_needs(n, p);

				cbn = s->lookup_by_name(n, sal_True);
				
				if (m != NULL && cbn != NULL)
				{
					if (cbn->node_type() != AST_Decl::NT_service) 
					{
						IDL_Global::idl_global()->err()
							->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
						delete(m);
					} else 
					{
						m->setService(AST_Service::narrow_from_decl(cbn));
						(void) s->fe_add_needs(m);
					}
				} else 
				{
					IDL_Global::idl_global()->err()
						->error2(UTL_Error::EIDL_INTERFACE_LOOKUP, m, ScopeAsDecl(s));
					delete(m);
				}
			}
			
			delete(l);
		}
	}
	| property
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_PropertyDeclSeen);
	}
	';'
	| error
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

service_interface_header :
	IDL_INTERFACE
	{
		$$ = AF_INVALID;
	}
	| opt_attrflags
	IDL_INTERFACE
	{
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		if ( fAHead->getFlags() == AF_OPTIONAL )
		{
			$$ = AF_OPTIONAL;
		} else
		{
			sal_uInt32 flag = 0;
			if ( fAHead->hasFlag(AF_ATTRIBUTE) )
				IDL_Global::idl_global()->err()
					->attributeflag_error(UTL_Error::EIDL_WRONGATTRIBUTEKEYWORD, AF_ATTRIBUTE);
			else if ( fAHead->hasFlag(AF_PROPERTY) )
				IDL_Global::idl_global()->err()
					->attributeflag_error(UTL_Error::EIDL_WRONGATTRIBUTEKEYWORD, AF_PROPERTY);
			else if ( fAHead->hasFlag(AF_READONLY) )
				flag = AF_READONLY;
			else if ( fAHead->hasFlag(AF_MAYBEVOID) )							  
				flag = AF_MAYBEVOID;										  
			else if ( fAHead->hasFlag(AF_BOUND) )
				flag = AF_BOUND;										  
			else if ( fAHead->hasFlag(AF_CONSTRAINED) )							  
				flag = AF_CONSTRAINED;										  
			else if ( fAHead->hasFlag(AF_MAYBEAMBIGUOUS) )						  
				flag = AF_MAYBEAMBIGUOUS;									  
			else if ( fAHead->hasFlag(AF_MAYBEDEFAULT) )							  
				flag = AF_MAYBEDEFAULT;										  
			else if ( fAHead->hasFlag(AF_REMOVEABLE) )							  
				flag = AF_REMOVEABLE;

			IDL_Global::idl_global()->err()
				->attributeflag_error(UTL_Error::EIDL_EXCLUSIVATTRIBUTEFLAG, flag);
			$$ = AF_INVALID;
		}
		
		delete( globalHead );
		globalHead = NULL;
	}
	;

mapping :
	MAPPING 
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ModuleSeen);
	
		UTL_ScopedName *n = new UTL_ScopedName("", NULL);
		AST_Mapping	*m = NULL;
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_StrList	*p = IDL_Global::idl_global()->pragmas();
		AST_Decl	*exists;

		
		/*
		 * Make a new mapping and add it to the enclosing scope
		 */
		if (s != NULL) 
		{
			m = IDL_Global::idl_global()->gen()->create_mapping(n, p);

			if ((exists = s->lookup_for_add(m, sal_False)) != NULL)
			{
				//exists->set_in_main_file(IDL_Global::idl_global()->in_main_file());
				delete(m);
				m = AST_Mapping::narrow_from_decl(exists);
			} else
				(void) s->fe_add_mapping(m);
		}
		/*
		 * Push it on the stack
		 */
		IDL_Global::idl_global()->scopes()->push(m);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MappingSqSeen);
	}
	definitions
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MappingBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MappingQsSeen);
		/*
		 * Finished with this module - pop it from the scope stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
	}
	;

language :
	LANGUAGE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LanguageSeen);
	}
	IDENTIFIER
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LanguageIdSeen);
		
		UTL_ScopedName *n = new UTL_ScopedName($3, NULL);
		AST_Language   *l = new AST_Language(n, NULL);
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();

		if ((s != NULL) && (s->scope_node_type() == AST_Decl::NT_mapping))
		{
			l = new AST_Language(n, NULL);

			/*
			 * Push it on the stack
			 */
			IDL_Global::idl_global()->scopes()->push(l);
		} else
		{
			if (s != NULL)
			{
				IDL_Global::idl_global()->err()
					->error2(UTL_Error::EIDL_MAPPING_MISSED, l, ScopeAsDecl(s));
			} else
			{
				IDL_Global::idl_global()->err()
					->error1(UTL_Error::EIDL_MAPPING_MISSED, l);
			}

			delete(l);
		}		
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LanguageSqSeen);
	}
	mapping_sections
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LanguageBodySeen);
		/*
		 * Finished with this module - pop it from the scope stack
		 */
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
  		IDL_Global::idl_global()->scopes()->pop();

		if ((s != NULL) && (s->scope_node_type() == AST_Decl::NT_language)) 
		{
			delete(ScopeAsDecl(s));
		}
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LanguageQsSeen);
	}
	| error
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

mapping_sections :
	mapping_section mapping_sections
	| /* empty */
	;

mapping_section :
	MAP
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapSeen);
	}
	MAPNAME
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapIdSeen);

		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		if ((s != NULL) && (s->scope_node_type() == AST_Decl::NT_language)) 
		{
			AST_Language *l = AST_Language::narrow_from_decl(ScopeAsDecl(s));
			if (IDL_Global::idl_global()->getMapping(*(l->local_name()), String($3)))
			{
				// Fehlerbehandlung
				String *err = new String($3);
				IDL_Global::idl_global()->err()->mapping_error(IDL_Global::idl_global()->lineno(), err);
				delete(err);
			}
		}
	}
	'['
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapSqSeen);
	}
	maps
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapBodySeen);
		
		UTL_Scope	*s = IDL_Global::idl_global()->scopes()->top_non_null();
		if ((s != NULL) && (s->scope_node_type() == AST_Decl::NT_language)) 
		{
			AST_Language *l = AST_Language::narrow_from_decl(ScopeAsDecl(s));

			IDL_Global::idl_global()->setMapping(*(l->local_name()), String($3), $7);
		}
	}
	']'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapQsSeen);
	}
	| error
	']'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;


maps :
	NAME
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapNameSeen);
	}
	'='
	IDENTIFIER
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapNameIdSeen);
	}
	mapping_entries
	{
		UTL_Map *mp = new UTL_Map($4);
		
		if ($6)
		{
			SectionList::const_iterator iter($6->begin());
			
			while (iter != $6->end())
			{
				switch ((*iter)->section_type())
				{
				case UTL_MapSection::ST_super :
					mp->setMapSuper((*iter)->getString());
	  				break;
				case UTL_MapSection::ST_include :
					mp->setMapInc((*iter)->getString());
	  				break;
				case UTL_MapSection::ST_getrefl :
					mp->setGetReflection((*iter)->getString());
					break;
				case UTL_MapSection::ST_increfl :
					mp->setIncReflection((*iter)->getString());
					break;
				case UTL_MapSection::ST_ident :
					mp->setMapIdent((*iter)->getString());
					break;
				case UTL_MapSection::ST_allListener :
					mp->setCreateAllListener((*iter)->getFlag());
					break;
				}
				
				iter++;
			}

//			delete($6);
		}

		$$ = mp; 
	}
	;

mapping_entries :
	mapping_entries
	','
	mapping_entry
	{
		if ($1 == NULL)
		{
			$$ = new SectionList();
			$$->insert($$->begin(), $3);
		} else 
		{
			$1->insert($1->begin(), $3);
			$$ = $1;
		}
	}
	| /* EMPTY */
	{
		$$ = NULL
	}
	;

mapping_entry :
	 	MAPSUPER
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapSuperSeen);
		}
		'='
		MAPSUPERNAME
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapSuperIdSeen);
		   	$$ = new UTL_MapSection(UTL_MapSection::ST_super ,$4);
		}
	 |	MAPINC
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapIncSeen);
		}
		'='
		MAPINCLUDE
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapIncIdSeen);
		   	$$ = new UTL_MapSection(UTL_MapSection::ST_include ,$4);
		}	
	| 	GETREFLECTION
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_GetReflSeen);
		}
		'='
		NAMEBODY
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_GetReflIdSeen);
		   	$$ = new UTL_MapSection(UTL_MapSection::ST_getrefl ,$4);
		}
	| 	INCREFLECTION
		{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_IncReflSeen);
		}
		'='
		MAPINCLUDE
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_IncReflIdSeen);
		   	$$ = new UTL_MapSection(UTL_MapSection::ST_increfl ,$4);
		}
	| 	MAPIDENT
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapIdentSeen);
		}
		'='
		IDENTIFIER
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MapIdentIdSeen);
		   	$$ = new UTL_MapSection(UTL_MapSection::ST_ident ,$4);
		}
	| 	CREATEALLISTENER
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_CreateAllListenerSeen);
		}
		'='
		boolean_init
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_CreateAllListenerIdSeen);
		   	$$ = new UTL_MapSection(UTL_MapSection::ST_allListener, $4);
		}
	| error
		','
		{
			IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
			yyerrok;
		}
		;

boolean_init :
	  TRUETOK
	  {
	  	$$ = sal_True;
	  }
	| FALSETOK
	  {
		$$ = sal_False;
	  }
	;

interface_def :
	interface
	| forward
	;

interface :
	interface_header
	{
		UTL_Scope     *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Interface *i = NULL;
		AST_Decl      *v = NULL;
		UTL_StrList   *p = IDL_Global::idl_global()->pragmas();
		AST_Decl      *d = NULL;
		AST_Interface *fd = NULL;

        /*
		 * Make a new interface node and add it to its enclosing scope
		 */
		if (s != NULL && $1 != NULL) 
		{
			i = IDL_Global::idl_global()->gen()->create_interface($1->interface_name(),
															$1->inherits(),
															$1->n_inherits(),
															p, $1->interface_head()->uik(),
															$1->interface_head()->version(),
															$1->interface_head()->isvirtual(),
															$1->interface_head()->ident());
			if (i != NULL &&
				(d = s->lookup_by_name(i->name(), sal_False)) != NULL) 
			{
				/*
				 * See if we're defining a forward declared interface.
				 */
				if (d->node_type() == AST_Decl::NT_interface) 
				{
					/*
					 * Narrow to an interface
					 */
					fd = AST_Interface::narrow_from_decl(d);
					/*
					 * Successful?
					 */
					if (fd == NULL) 
					{
						/*
						 * Should we give an error here?
						 */
                    }
					/*
					 * If it is a forward declared interface..
					 */
					else 
						if (!fd->is_defined()) 
						{
							/*
							 * Check if redefining in same scope
							 */
							if (fd->defined_in() != s) 
							{
								if (*(fd->name()) != *(i->name()))
								{
									IDL_Global::idl_global()->err()
										->error3(UTL_Error::EIDL_SCOPE_CONFLICT,
												 i, fd, ScopeAsDecl(s));
								}
							}
							/*
							 * All OK, do the redefinition
							 */
							else 
							{
								if (fd->is_forwarded() &&
									*(fd->file_name()) == *(IDL_Global::idl_global()->filename()))
								{
									fd->setForwardInSameFile(sal_True);
								}

								fd->set_inherits($1->inherits());
								fd->set_n_inherits($1->n_inherits());
								fd->set_uik($1->interface_head()->uik());
								fd->set_ident($1->interface_head()->ident());
								fd->set_version($1->interface_head()->version());
								
								fd->set_virtual(i->isvirtual());
								/*
								 * Update place of definition
								 */
								
								fd->set_imported(i->imported());
								fd->set_in_main_file(i->in_main_file());
								fd->set_line(i->line());
								fd->set_file_name(new String(*(i->file_name())));
								fd->add_pragmas(p);

								/*
								 * Update documentation
								 */
								fd->setIdldocText(i->getIdldocText());
								fd->setIdldocTags(i->getIdldocTags());

								/*
								 * Use full definition node
								 */
								delete(i);
								i = fd;
							}
						}
				}
			}

			/* check multible inheritance */
			//i->checkInheritance();

			/*
			 * Add the interface to its definition scope
			 */
			(void) s->fe_add_interface(i);
		}
		/*
		 * Push it on the scope stack
		 */
		IDL_Global::idl_global()->scopes()->push(i);
		delete($1);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceSqSeen);
	}
	exports
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceQsSeen);
		/*
		 * Done with this interface - pop it off the scopes stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
	}
	;

interface_decl:
	IDL_INTERFACE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceSeen);
	}
	id
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceIDSeen);
		$$ = $3;
	}
	;

interface_head :
	'['
	{
		IDL_Global::idl_global()
			->set_parse_state(IDL_Global::PS_InterfaceHeadSqSeen);
	}
	uik
	{
		IDL_Global::idl_global()
			->set_parse_state(IDL_Global::PS_IHeadUikSeen);
	}
	opt_ident
	']'
	{
		double	version;
		char*	ident;
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceHeadSeen);
		if ($5 != NULL)
		{
			version = $5->version();
			ident = $5->ident();
		} else
		{
			version = 0;
			ident = NULL;
		}

		$$ = new FE_InterfaceHead(sal_False, $3, ident, version);
/*
	}
	| '['
	{
		IDL_Global::idl_global()
			->set_parse_state(IDL_Global::PS_InterfaceHeadSqSeen);
	}
	VIRTUAL
	{
		IDL_Global::idl_global()
			->set_parse_state(IDL_Global::PS_IHeadVirtualSeen);
	}
	opt_ident
	']'
	{
		double	version;
		char*	ident;
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InterfaceHeadSeen);
		if ($5 != NULL)
		{
			version = $5->version();
			ident = $5->ident();
		} else
		{
			version = 0;
			ident = NULL;
		}

		$$ = new FE_InterfaceHead(sal_True, NULL, ident, version);
*/
	}
	;

uik :
	UIK
	'('
	UIK_BODY
	')'
	{
		$$ = $3;
	}
	| error
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

opt_ident :
	','
	IDENT
	'('
	NAMEBODY
	','
	FLOATING_PT_LITERAL
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_IHeadIdentSeen);
		$$ = new FE_Ident($4, $6);
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	| error
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

interface_header :
	interface_head
	interface_decl inheritance_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InheritSpecSeen);
		/*
		 * Create an AST representation of the information in the header
		 * part of an interface - this representation contains a computed
		 * list of all interfaces which this interface inherits from,
		 * recursively
		 */
		$$ = new FE_InterfaceHeader(new UTL_ScopedName(*$2, NULL), $3, $1);
	}
	;

inheritance_spec :
	':'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InheritColonSeen);
	}
	at_least_one_scoped_name
	{
		$$ = $3;
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;

exports :
	exports export
	| /* EMPTY */
	;

export :
	type_dcl
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_TypeDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| const_dcl
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| exception
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ExceptDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| attribute
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
	| operation
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpDeclSeen);
	}
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
	}
    ;

at_least_one_scoped_name :
	scoped_name scoped_names
	{
		$$ = new UTL_NameList($1, $2);
	}
	;

scoped_names :
	scoped_names
	','
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SNListCommaSeen);
	}
	scoped_name
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ScopedNameSeen);

		if ($1 == NULL)
			$$ = new UTL_NameList($4, NULL);
		else 
		{
			$1->nconc(new UTL_NameList($4, NULL));
			$$ = $1;
		}
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;

scoped_name :
	id
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SN_IDSeen);

		$$ = new UTL_IdList(*$1, NULL);
	}
	| SCOPE_DELIMITOR
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ScopeDelimSeen);
	}
		id
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SN_IDSeen);

		$$ = new UTL_IdList($1, new UTL_IdList(*$3, NULL));
	}
	| scoped_name
	  SCOPE_DELIMITOR
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ScopeDelimSeen);
	}
	id
    {
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SN_IDSeen);

		$1->nconc(new UTL_IdList(*$4, NULL));
		$$ = $1;
	}
	;

id : IDENTIFIER
	{
		$$ = new Identifier($1);
	}
	;

forward :
	interface_decl
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_ScopedName        *n = new UTL_ScopedName(*$1, NULL);
		AST_InterfaceFwd      *f = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ForwardDeclSeen);
		/*
		 * Create a node representing a forward declaration of an
		 * interface. Store it in the enclosing scope
		 */
		if (s != NULL) 
		{
			f = IDL_Global::idl_global()->gen()->create_interface_fwd(n, p);
			(void) s->fe_add_interface_fwd(f);
		}
	}
	;

constants_exports :
	constants_export constants_exports
	| /* empty */
	;

constants_export :
	const_dcl
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstDeclSeen);
	}
	';'
	

const_dcl :
	IDL_CONST
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstSeen);
	}
	const_type
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstTypeSeen);
	}
	id
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstIDSeen);
	}
	'='
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstAssignSeen);
	}
	expression
	{
		UTL_ScopedName        *n = new UTL_ScopedName(*$5, NULL);
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Constant          *c = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();
		AST_Decl              *v = NULL;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ConstExprSeen);
		/*
		 * Create a node representing a constant declaration. Store
		 * it in the enclosing scope
		 */
		if ($9 != NULL && s != NULL) 
		{
			if ($9->coerce($3) == NULL)
				IDL_Global::idl_global()->err()->coercion_error($9, $3);
			else 
			{
				c = IDL_Global::idl_global()->gen()->create_constant($3, $9, n, p);
				(void) s->fe_add_constant(c);
			}
		}
	}
	;

const_type :
	integer_type
	| char_type
	| octet_type
	| byte_type
	| boolean_type
	| floating_pt_type
	| string_type_spec
	{
		$$ = AST_Expression::EV_string;
	}
	| wstring_type_spec
	{
		$$ = AST_Expression::EV_wstring;
	}
	| scoped_name
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Decl              *d = NULL;
		AST_PredefinedType    *c = NULL;
		AST_Typedef           *t = NULL;

		/*
		 * If the constant's type is a scoped name, it must resolve
		 * to a scalar constant type
		 */
		if (s != NULL && (d = s->lookup_by_name($1, sal_True)) != NULL) 
		{
			/*
			 * Look through typedefs
			 */
			while (d->node_type() == AST_Decl::NT_typedef) 
			{
				t = AST_Typedef::narrow_from_decl(d);
				if (t == NULL)
					break;
				d = t->base_type();
			}
			if (d == NULL)
				$$ = AST_Expression::EV_any;
			else 
				if (d->node_type() == AST_Decl::NT_pre_defined) 
				{
					c = AST_PredefinedType::narrow_from_decl(d);
					if (c != NULL) 
					{
						$$ = IDL_Global::idl_global()->PredefinedTypeToExprType(c->pt());
					} else 
					{
						$$ = AST_Expression::EV_any;
					}
				} else
					$$ = AST_Expression::EV_any;
		} else
			$$ = AST_Expression::EV_any;
	}
	;

expression : const_expr ;

const_expr : or_expr ;

or_expr : 
	xor_expr
	| or_expr '|' xor_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_or, $1, $3);
	}
	;

xor_expr :
	and_expr
	| xor_expr '^' and_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_xor, $1, $3);
	}
	;

and_expr :
	shift_expr
	| and_expr '&' shift_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_and, $1, $3);
	}
	;

shift_expr : 
	add_expr
	| shift_expr LEFT_SHIFT add_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_left,$1,$3);
	}
	| shift_expr RIGHT_SHIFT add_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_right,$1,$3);
	}
	;

add_expr : 
	mult_expr
	| add_expr '+' mult_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_add, $1, $3);
	}
	| add_expr '-' mult_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_minus,$1,$3);
	}
	;

mult_expr : 
	unary_expr
	| mult_expr '*' unary_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_mul, $1, $3);
	}
	| mult_expr '/' unary_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_div, $1, $3);
	}
	| mult_expr '%' unary_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_mod, $1, $3);
	}
	;

unary_expr : 
	primary_expr
	| '+' primary_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_u_plus,
																$2,
																NULL);
	}
	| '-' primary_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_u_minus,
																$2,
																NULL);
	}
	| '~' primary_expr
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr(AST_Expression::EC_bit_neg,
																$2,
																NULL);
	}
	;

primary_expr : 
	scoped_name
	{
		/*
		 * An expression which is a scoped name is not resolved now,
		 * but only when it is evaluated (such as when it is assigned
		 * as a constant value)
		 */
		$$ = IDL_Global::idl_global()->gen()->create_expr($1);
	}
	| literal
	| '(' const_expr ')'
	{
		$$ = $2;
	}
	;

literal : 
	INTEGER_LITERAL
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr($1);
	}
	| STRING_LITERAL
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr($1);
	}
	| CHARACTER_LITERAL
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr($1);
	}
	| FLOATING_PT_LITERAL
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr($1);
	}
	| TRUETOK
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr((long) sal_True,
																AST_Expression::EV_bool);
	}
	| FALSETOK
	{
		$$ = IDL_Global::idl_global()->gen()->create_expr((long) sal_False,
																AST_Expression::EV_bool);
	}
	;

positive_int_expr :
	const_expr
	{
		$1->evaluate(AST_Expression::EK_const);
		$$ = IDL_Global::idl_global()->gen()->create_expr($1, AST_Expression::EV_ulong);
	}
	;

type_dcl : 
	IDL_TYPEDEF
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_TypedefSeen);
	}
	type_declarator
	| struct_type
	| union_type
	| enum_type
	;

type_declarator :
	type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_TypeSpecSeen);
	}
	at_least_one_declarator
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_DecllistActiveIterator *l;
		FE_Declarator         *d = NULL;
		AST_Typedef           *t = NULL;
		AST_Decl              *v = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DeclaratorsSeen);
		/*
		 * Create a list of type renamings. Add them to the
		 * enclosing scope
		 */
		if (s != NULL && $1 != NULL && $3 != NULL) 
		{
			l = new UTL_DecllistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				d = l->item();
				if (d == NULL)
					continue;
				AST_Type * tp = d->compose($1);
				if (tp == NULL)
					continue;

				t = IDL_Global::idl_global()->gen()
						->create_typedef(tp, d->name(), p);
				(void) s->fe_add_typedef(t);
			}
			delete(l);
		}
	}
	;

type_spec : 
	simple_type_spec
	| constructed_type_spec
	;

simple_type_spec : 
	base_type_spec
	{
		$$ = IDL_Global::idl_global()->scopes()->bottom()->lookup_primitive_type($1);
	}
	| template_type_spec
	| scoped_name
	{
		UTL_Scope     *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Decl      *d = NULL;

		if (s != NULL)
			d = s->lookup_by_name($1, sal_True);
		if (d == NULL)
			IDL_Global::idl_global()->err()->lookup_error($1);
		$$ = d;
	}
	;

base_type_spec : 
	integer_type
	| floating_pt_type
	| char_type
	| boolean_type
	| octet_type
	| byte_type
	| any_type
	| type_type
	| implementation_type
	;

template_type_spec : 
	sequence_type_spec
	| string_type_spec
	| wstring_type_spec
	;

constructed_type_spec : 
	struct_type
	| union_type
	| enum_type
	;

at_least_one_declarator :
	declarator declarators
	{
		$$ = new UTL_DeclList($1, $2);
	}
	;

declarators : 
	declarators
	','
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DeclsCommaSeen);
	} 
	declarator
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DeclsDeclSeen);

		if ($1 == NULL)
			$$ = new UTL_DeclList($4, NULL);
		else 
		{
			$1->nconc(new UTL_DeclList($4, NULL));
			$$ = $1;
		}
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;

declarator : 
	simple_declarator
	| complex_declarator
	;

simple_declarator :
	id
	{
		$$ = new FE_Declarator(new UTL_ScopedName(*$1, NULL),
								FE_Declarator::FD_simple, NULL);
	}
	;

complex_declarator :
	array_declarator
	{
		$$ = new FE_Declarator(new UTL_ScopedName(*($1->local_name()), NULL),
								FE_Declarator::FD_complex, $1);
	}
	;

integer_type : 
	signed_int
	| unsigned_int
	;

signed_int : 
	IDL_LONG
	{
		$$ = AST_Expression::EV_long;
	}
	| IDL_HYPER
	{
		$$ = AST_Expression::EV_hyper;
	}
	| IDL_LONG IDL_LONG
	{
		$$ = AST_Expression::EV_longlong;
	}
	| IDL_SHORT
	{
		$$ = AST_Expression::EV_short;
	}
	;

unsigned_int : 
	IDL_UNSIGNED IDL_LONG
	{
		$$ = AST_Expression::EV_ulong;
	}
	| IDL_UNSIGNED IDL_HYPER
	{
		$$ = AST_Expression::EV_uhyper;
	}
	| IDL_UNSIGNED IDL_LONG IDL_LONG
	{
		$$ = AST_Expression::EV_ulonglong;
	}
	| IDL_UNSIGNED IDL_SHORT
	{
		$$ = AST_Expression::EV_ushort;
	}
	;

floating_pt_type : 
	IDL_DOUBLE
	{
		$$ = AST_Expression::EV_double;
	}
	| IDL_FLOAT
	{
		$$ = AST_Expression::EV_float;
	}
	| IDL_LONG IDL_DOUBLE
	{
		$$ = AST_Expression::EV_longdouble;
	}
	;

char_type : 
	IDL_CHAR
	{
		$$ = AST_Expression::EV_char;
	}
	| IDL_WCHAR
	{
		$$ = AST_Expression::EV_wchar;
	}
	;

octet_type :
	IDL_OCTET
	{
		$$ = AST_Expression::EV_octet;
	}
	;

byte_type : 
	IDL_BYTE
	{
		$$ = AST_Expression::EV_byte;
	}
	;

boolean_type : 
	IDL_BOOLEAN
	{
		$$ = AST_Expression::EV_bool;
	}
	;

any_type : 
	IDL_ANY
	{
		$$ = AST_Expression::EV_any;
	}
	;

type_type :
	IDL_TYPE
	{
		$$ = AST_Expression::EV_type;
	}
	;

implementation_type :
	IDL_IMPLEMENTATION
	{
		$$ = AST_Expression::EV_implementation;
	}
	;

structure_header :
	IDL_STRUCT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StructSeen);
	}
	id
	inheritance_spec	
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InheritSpecSeen);
		/*
		 * Create an AST representation of the information in the header
		 * part of an structure - this representation contains a computed
		 * list of all structures which this structure inherits from,
		 * recursively
		 */
		$$ = new FE_StructHeader(new UTL_ScopedName(*$3, NULL), $4);
	}
	;

struct_type :
	structure_header
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Structure         *d = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();
		AST_Decl              *v = NULL;	

		/*
		 * Create a node representing a struct declaration. Add it
		 * to the enclosing scope
		 */
		if (s != NULL) 
		{
			d = IDL_Global::idl_global()->gen()
				->create_structure($1->struct_name(), $1->inherits(), $1->n_inherits(), p);
	
			if ($1->n_inherits() > 1)
			{
				IDL_Global::idl_global()->err()
					->error1(UTL_Error::EIDL_MULTIBLE_INHERITANCE, d);
			}

			(void) s->fe_add_structure(d);
		}
		/*
		 * Push the scope of the struct on the scopes stack
		 */
		IDL_Global::idl_global()->scopes()->push(d);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StructSqSeen);
	}
	at_least_one_member
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Structure	*st = NULL;

		if (s != NULL && s->scope_node_type() == AST_Decl::NT_struct) 
		{
			st = AST_Structure::narrow_from_scope(s);
		}

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StructBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StructQsSeen);
		/*
		 * Done with this struct. Pop its scope off the scopes stack
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL)
			$$ = NULL;
		else 
		{
			$$ = AST_Structure::narrow_from_scope(IDL_Global::idl_global()
													->scopes()->top_non_null());
			IDL_Global::idl_global()->scopes()->pop();
		}
	}
	;

at_least_one_member : member members ;

members : 
	members member
	| /* EMPTY */
	;

member :
	type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MemberTypeSeen);
	}
	at_least_one_declarator
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MemberDeclsSeen);
	}
	';'
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_DecllistActiveIterator *l = NULL;
		FE_Declarator         *d = NULL;
		AST_Field             *f = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();
		
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_MemberDeclsCompleted);
		/*
		 * Check for illegal recursive use of type
		 */
		if ($1 != NULL && AST_illegal_recursive_type($1))
			IDL_Global::idl_global()->err()->error1(UTL_Error::EIDL_RECURSIVE_TYPE, $1);
		/*
		 * Create a node representing a struct or exception member
		 * Add it to the enclosing scope
         */
		else if (s != NULL && $1 != NULL && $3 != NULL) 
		{
			l = new UTL_DecllistActiveIterator($3);
			for (;!(l->is_done()); l->next()) 
			{
				d = l->item();
				if (d == NULL)
					continue;
				AST_Type *tp = d->compose($1);
				
				if (tp == NULL)
					continue;

				if (tp->is_impl_type())
					IDL_Global::idl_global()->err()
						->error0(UTL_Error::EIDL_TYPE_NOT_VALID);

				f = IDL_Global::idl_global()->gen()
					->create_field(tp, d->name(), p);
				
				if (!(d->checkType($1)))
					IDL_Global::idl_global()->err()
						->warning2(UTL_Error::WIDL_TYPE_IDENT_CONFLICT, f, tp);
				
				(void) s->fe_add_field(f);
			}
			delete(l);
		}
	}
	| error
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

union_type :
	IDL_UNION
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionSeen);
	}
	id
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionIDSeen);
	}
	IDL_SWITCH
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SwitchSeen);
	}
	'('
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SwitchOpenParSeen);
	}
	switch_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SwitchTypeSeen);
	}
	')'
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_ScopedName        *n = new UTL_ScopedName(*$3, NULL);
		AST_Union             *u = NULL;
		AST_Decl              *v = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SwitchCloseParSeen);
		/*
		 * Create a node representing a union. Add it to its enclosing
		 * scope
		 */
		if ($9 != NULL && s != NULL) 
		{
			AST_ConcreteType    *tp = AST_ConcreteType::narrow_from_decl($9);
			if (tp == NULL) 
			{
				IDL_Global::idl_global()->err()->not_a_type($9);
			} else 
			{
				u = IDL_Global::idl_global()->gen()->create_union(tp, n, p);
				(void) s->fe_add_union(u);
			}
		}
		/*
		 * Push the scope of the union on the scopes stack
		 */
		IDL_Global::idl_global()->scopes()->push(u);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionSqSeen);
	}
		at_least_one_case_branch
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionQsSeen);
		/*
		 * Done with this union. Pop its scope from the scopes stack
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL)
			$$ = NULL;
		else 
		{
			$$ = AST_Union::narrow_from_scope(IDL_Global::idl_global()
													->scopes()->top_non_null());
			 IDL_Global::idl_global()->scopes()->pop();
		}
	}
	;

switch_type_spec :
	integer_type
	{	
		$$ = IDL_Global::idl_global()->scopes()->bottom()->lookup_primitive_type($1);
	}
	| char_type
	{
		$$ = IDL_Global::idl_global()->scopes()->bottom()->lookup_primitive_type($1);
	}
	| octet_type
	{
		$$ = IDL_Global::idl_global()->scopes()->bottom()->lookup_primitive_type($1);
	}
	| boolean_type
	{
		$$ = IDL_Global::idl_global()->scopes()->bottom()->lookup_primitive_type($1);
	}
	| enum_type
	| scoped_name
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Decl              *d = NULL;
		AST_PredefinedType    *p = NULL;
		AST_Typedef           *t = NULL;
		long                  found = sal_False;

		/*
		 * The discriminator is a scoped name. Try to resolve to
		 * one of the scalar types or to an enum. Thread through
		 * typedef's to arrive at the base type at the end of the
		 * chain
		 */
		if (s != NULL && (d = s->lookup_by_name($1, sal_True)) != NULL) 
		{
			while (!found) 
			{
				switch (d->node_type()) 
				{
					case AST_Decl::NT_enum:
						$$ = d;
						found = sal_True;
						break;
					case AST_Decl::NT_pre_defined:
						p = AST_PredefinedType::narrow_from_decl(d);
						if (p != NULL) 
						{
							switch (p->pt()) 
							{
								case AST_PredefinedType::PT_long:
								case AST_PredefinedType::PT_ulong:
								case AST_PredefinedType::PT_longlong:
								case AST_PredefinedType::PT_ulonglong:
								case AST_PredefinedType::PT_short:
								case AST_PredefinedType::PT_ushort:
								case AST_PredefinedType::PT_hyper:
								case AST_PredefinedType::PT_uhyper:
								case AST_PredefinedType::PT_char:
								case AST_PredefinedType::PT_wchar:
								case AST_PredefinedType::PT_octet:
								case AST_PredefinedType::PT_byte:
								case AST_PredefinedType::PT_boolean:
									$$ = p;
									found = sal_True;
									break;
								default:
									$$ = NULL;
									found = sal_True;
									break;
							}
						}
						break;
					case AST_Decl::NT_typedef:
						t = AST_Typedef::narrow_from_decl(d);
						if (t != NULL) d = t->base_type();
						break;
					default:
						$$ = NULL;
						found = sal_True;
		               break;		
				}
			}
		} else
			$$ = NULL;

		if ($$ == NULL)
			IDL_Global::idl_global()->err()->lookup_error($1);
	}
	;

at_least_one_case_branch : case_branch case_branches ;

case_branches : 
	case_branches case_branch
	| /* empty */
	;

case_branch :
	at_least_one_case_label
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionLabelSeen);
	}
	element_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionElemSeen);
	}
	';'
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_LabellistActiveIterator *l = NULL;
		AST_UnionLabel        *d = NULL;
		AST_UnionBranch       *b = NULL;
		AST_Field             *f = $3;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionElemCompleted);
		/*
		 * Create several nodes representing branches of a union.
		 * Add them to the enclosing scope (the union scope)
		 */
		if (s != NULL && $1 != NULL && $3 != NULL) 
		{
			l = new UTL_LabellistActiveIterator($1);
			for (;!(l->is_done()); l->next()) 
			{
				d = l->item();
				if (d == NULL)
					continue;
				b = IDL_Global::idl_global()->gen()->create_union_branch(d,
                                                     f->field_type(),
                                                     f->name(),
                                                     f->pragmas());
				(void) s->fe_add_union_branch(b);
			}
			delete(l);
		}
	}
	| error
	';'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

at_least_one_case_label :
	case_label case_labels
	{
		$$ = new UTL_LabelList($1, $2);
	}
	;

case_labels : 
	case_labels case_label
	{
		if ($1 == NULL)
			$$ = new UTL_LabelList($2, NULL);
		else 
		{
			$1->nconc(new UTL_LabelList($2, NULL));
			$$ = $1;
		}
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;

case_label : 
	IDL_DEFAULT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DefaultSeen);
	}
	':'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LabelColonSeen);

		$$ = IDL_Global::idl_global()->gen()->
                   create_union_label(AST_UnionLabel::UL_default, NULL);
	}
	| IDL_CASE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_CaseSeen);
	}
	const_expr
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LabelExprSeen);
	}
	':'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_LabelColonSeen);

		$$ = IDL_Global::idl_global()->gen()->
					create_union_label(AST_UnionLabel::UL_label, $3);
	}
	;

element_spec :
	type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionElemTypeSeen);
	}
	declarator
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_UnionElemDeclSeen);
		/*
		 * Check for illegal recursive use of type
		 */
		if ($1 != NULL && AST_illegal_recursive_type($1))
			IDL_Global::idl_global()->err()->error1(UTL_Error::EIDL_RECURSIVE_TYPE, $1);
		/*
		 * Create a field in a union branch
		 */
		else 
			if ($1 == NULL || $3 == NULL)
				$$ = NULL;
			else 
			{
				AST_Type *tp = $3->compose($1);
				if (tp == NULL)
					$$ = NULL;
				else
					$$ = IDL_Global::idl_global()->gen()
							->create_field(tp, $3->name(),
											IDL_Global::idl_global()->pragmas());
			}
	}
	;

enum_type :
	IDL_ENUM
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_EnumSeen);
	}
	id
	{
		UTL_Scope             *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_ScopedName        *n = new UTL_ScopedName(*$3, NULL);
		AST_Enum              *e = NULL;
		AST_Decl              *v = NULL;
		UTL_StrList           *p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_EnumIDSeen);
		/*
		 * Create a node representing an enum and add it to its
		 * enclosing scope
		 */
		if (s != NULL) 
		{
			e = IDL_Global::idl_global()->gen()->create_enum(n, p);
			/*
			 * Add it to its defining scope
			 */
			(void) s->fe_add_enum(e);
		}
		/*
		 * Push the enum scope on the scopes stack
		 */
		IDL_Global::idl_global()->scopes()->push(e);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_EnumSqSeen);
	}
	at_least_one_enumerator
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_EnumBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_EnumQsSeen);
		/*
		 * Done with this enum. Pop its scope from the scopes stack
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL)
			$$ = NULL;
		else 
		{
			$$ = AST_Enum::narrow_from_scope(IDL_Global::idl_global()
												->scopes()->top_non_null());
			IDL_Global::idl_global()->scopes()->pop();
		}
	}
	;

at_least_one_enumerator : enumerator enumerators ;

enumerators : 
	enumerators
	','
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_EnumCommaSeen);
	}
	enumerator
	| /* EMPTY */
	;

enumerator :
	IDENTIFIER
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_ScopedName	*n = new UTL_ScopedName($1, NULL);
		AST_EnumVal		*e = NULL;
		AST_Enum		*c = NULL;
		UTL_StrList		*p = IDL_Global::idl_global()->pragmas();

		/*
		 * Create a node representing one enumerator in an enum
		 * Add it to the enclosing scope (the enum scope)
		 */
		if (s != NULL && s->scope_node_type() == AST_Decl::NT_enum) 
		{
			c = AST_Enum::narrow_from_scope(s);
			if (c != NULL)
				e = IDL_Global::idl_global()->gen()->
						create_enum_val(c->next_enum_val(), n, p);
			if (c->check_value(e->constant_value()) != NULL)
				IDL_Global::idl_global()->err()
					->error1(UTL_Error::EIDL_EVAL_ERROR, AST_Decl::narrow_from_decl(e));				
		
			(void) s->fe_add_enum_val(e);
		}
	}
	| IDENTIFIER
	'='
	INTEGER_LITERAL
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_ScopedName	*n = new UTL_ScopedName($1, NULL);
		AST_EnumVal		*e = NULL;
		AST_Enum		*c = NULL;
		UTL_StrList		*p = IDL_Global::idl_global()->pragmas();

		/*
		 * Create a node representing one enumerator in an enum
		 * Add it to the enclosing scope (the enum scope)
		 */
		if (s != NULL && s->scope_node_type() == AST_Decl::NT_enum) 
		{
			c = AST_Enum::narrow_from_scope(s);
			if (c != NULL)
				e = IDL_Global::idl_global()->gen()->
						create_enum_val((unsigned long) $3, n, p);
			if (c->check_value(e->constant_value()) != NULL)
				IDL_Global::idl_global()->err()
					->error1(UTL_Error::EIDL_EVAL_ERROR, AST_Decl::narrow_from_decl(e));				
			
			c->setEnumVal(e->constant_value()->ev()->u.ulval + 1);
			(void) s->fe_add_enum_val(e);
		}
	}
	;

sequence_type_spec : 
	seq_head
	','
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceCommaSeen);
	}
	positive_int_expr
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceExprSeen);
	}
	'>'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceQsSeen);
		/*
		 * Remove sequence marker from scopes stack
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL)
			IDL_Global::idl_global()->scopes()->pop();
		/*
		 * Create a node representing a sequence
		 */
		if ($4 == NULL || $4->coerce(AST_Expression::EV_ulong) == NULL) 
		{
			IDL_Global::idl_global()->err()->coercion_error($4, AST_Expression::EV_ulong);
			$$ = NULL;
		} else 
			if ($1 == NULL) 
			{
				$$ = NULL;
			} else 
			{
				AST_Type *tp = AST_Type::narrow_from_decl($1);
				if (tp == NULL)
					$$ = NULL;
				else 
				{
					if (tp->is_impl_type())
						IDL_Global::idl_global()->err()
							->error0(UTL_Error::EIDL_TYPE_NOT_VALID);

					$$ = IDL_Global::idl_global()->gen()->create_sequence($4, tp);
					/*
					 * Add this AST_Sequence to the types defined in the global scope
					 */
					(void) IDL_Global::idl_global()->root()
							->fe_add_sequence(AST_Sequence::narrow_from_decl($$));
				}
			}
	}
	| seq_head
	'>'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceQsSeen);
		/*
		 * Remove sequence marker from scopes stack
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL)
			IDL_Global::idl_global()->scopes()->pop();
		/*
		 * Create a node representing a sequence
		 */
		if ($1 == NULL)
			$$ = NULL;
		else 
		{
			AST_Type *tp = AST_Type::narrow_from_decl($1);
			if (tp == NULL)
				$$ = NULL;
			else 
			{
				if (tp->is_impl_type())
					IDL_Global::idl_global()->err()
						->error0(UTL_Error::EIDL_TYPE_NOT_VALID);

				$$ = IDL_Global::idl_global()->gen()
						->create_sequence(IDL_Global::idl_global()->gen()
											->create_expr((unsigned long) 0), tp);
				/*
				 * Add this AST_Sequence to the types defined in the global scope
				 */
				(void) IDL_Global::idl_global()->root()
						->fe_add_sequence(AST_Sequence::narrow_from_decl($$));
			}
        }
	}
	;

seq_head :
	IDL_SEQUENCE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceSeen);
		/*
		 * Push a sequence marker on scopes stack
		 */
		IDL_Global::idl_global()->scopes()->push(NULL);
	}
	'<'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceSqSeen);
	}
	simple_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_SequenceTypeSeen);
		$$ = $5;
	}
	;

string_type_spec : 
	string_head
	'<'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringSqSeen);
	}
	positive_int_expr
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringExprSeen);
	}
	'>'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringQsSeen);
		/*
		 * Create a node representing a string
		 */
		if ($4 == NULL || $4->coerce(AST_Expression::EV_ulong) == NULL) 
		{
			IDL_Global::idl_global()->err()->coercion_error($4, AST_Expression::EV_ulong);
			$$ = NULL;
		} else 
		{
			$$ = IDL_Global::idl_global()->gen()->create_string($4);
			/*
			 * Add this AST_String to the types defined in the global scope
			 */
			(void) IDL_Global::idl_global()->root()
                     ->fe_add_string(AST_String::narrow_from_decl($$));
        }
	}
	| string_head
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringCompleted);
		/*
		 * Create a node representing a string
		 */
		$$ = IDL_Global::idl_global()->gen()->create_string(
                        IDL_Global::idl_global()->gen()->create_expr((unsigned long) 0));
		/*
		 * Add this AST_String to the types defined in the global scope
		 */
		(void) IDL_Global::idl_global()->root()
                    ->fe_add_string(AST_String::narrow_from_decl($$));
	}
	;

string_head :
	IDL_STRING
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringSeen);
	}
	;

wstring_type_spec : 
	wstring_head
	'<'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringSqSeen);
	}
	positive_int_expr
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringExprSeen);
	}
	'>'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringQsSeen);
		/*
		 * Create a node representing a string
		 */
		if ($4 == NULL || $4->coerce(AST_Expression::EV_ulong) == NULL) 
		{
			IDL_Global::idl_global()->err()->coercion_error($4, AST_Expression::EV_ulong);
			$$ = NULL;
		} else 
		{
			$$ = IDL_Global::idl_global()->gen()->create_wstring($4);
			/*
			 * Add this AST_String to the types defined in the global scope
			 */
			(void) IDL_Global::idl_global()->root()
                     ->fe_add_string(AST_String::narrow_from_decl($$));
		}
	}
	| wstring_head
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringCompleted);
		/*
		 * Create a node representing a string
         */
		$$ = IDL_Global::idl_global()->gen()->create_wstring(
                        IDL_Global::idl_global()->gen()->create_expr((unsigned long) 0));
        /*
		 * Add this AST_String to the types defined in the global scope
		 */
		(void) IDL_Global::idl_global()->root()
                    ->fe_add_string(AST_String::narrow_from_decl($$));
	}
	;

wstring_head :
	IDL_WSTRING
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_StringSeen);
	}
	;

array_declarator :
	id
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ArrayIDSeen);
	}
	at_least_one_array_dim
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ArrayCompleted);
		/*
		 * Create a node representing an array
		 */
		if ($3 != NULL) 
		{
			$$ = IDL_Global::idl_global()->gen()
					->create_array(new UTL_ScopedName(*$1, NULL), $3->length(), $3);
		}
	}
	;

at_least_one_array_dim :
	array_dim array_dims
	{
		$$ = new UTL_ExprList($1, $2);
	}
	;

array_dims : 
	array_dims array_dim
	{
		if ($1 == NULL)
			$$ = new UTL_ExprList($2, NULL);
		else 
		{
			$1->nconc(new UTL_ExprList($2, NULL));
			$$ = $1;
        }
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
    ;

array_dim :
	'['
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DimSqSeen);
	}
	positive_int_expr
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DimExprSeen);
	}
	']'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_DimQsSeen);
		/*
		 * Array dimensions are expressions which must be coerced to
		 * positive integers
		 */
		if ($3 == NULL || $3->coerce(AST_Expression::EV_ulong) == NULL) 
		{
			IDL_Global::idl_global()->err()->coercion_error($3, AST_Expression::EV_ulong);
			$$ = NULL;
		} else
			$$ = $3;
	}
	;

attribute :
	attribute_head
	simple_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrTypeSeen);
	}
	at_least_one_declarator
	{
		UTL_Scope               *s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_DecllistActiveIterator *l = NULL;
		AST_Attribute           *a = NULL;
		FE_Declarator           *d = NULL;
		UTL_StrList             *p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrCompleted);
		/*
		 * Create nodes representing attributes and add them to the
		 * enclosing scope
		 */
		if (s != NULL && $2 != NULL && $4 != NULL)
		{
			l = new UTL_DecllistActiveIterator($4);
			for (;!(l->is_done()); l->next()) {
				d = l->item();
				if (d == NULL)
					continue;
				AST_Type *tp = d->compose($2);
				if (tp->is_impl_type())
				IDL_Global::idl_global()->err()
					->error0(UTL_Error::EIDL_TYPE_NOT_VALID);

				if (tp == NULL)
					continue;
				a = IDL_Global::idl_global()->gen()
						->create_attribute($1->getFlags(),
											$1->implementation(), tp, d->name(), p);
				/*
				 * Add one attribute to the enclosing scope
				 */
				(void) s->fe_add_attribute(a);
			}
			delete(l);
		}
		delete($1);
		globalHead = NULL;
	}
	;

attribute_head :
	opt_attrflags
	{
		// IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrSeen);
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrHeadSeen);

		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		if ( !fAHead->hasFlag(AF_ATTRIBUTE) )
			IDL_Global::idl_global()->err()
				->attributeflag_error(UTL_Error::EIDL_MISSINGATTRIBUTEKEYWORD, AF_ATTRIBUTE);

		sal_uInt32 flag=0;
		sal_Bool errFlag = sal_False;
		if ( fAHead->hasFlag(AF_OPTIONAL) )								  
		{
			flag = AF_OPTIONAL;
			errFlag = sal_True;
		} else if ( fAHead->hasFlag(AF_MAYBEVOID) )							  
		{
			flag = AF_MAYBEVOID;
			errFlag = sal_True;										  
		} else if ( fAHead->hasFlag(AF_CONSTRAINED) )							  
		{
			flag = AF_CONSTRAINED;
			errFlag = sal_True;
		} else if ( fAHead->hasFlag(AF_MAYBEAMBIGUOUS) )						  
		{
			flag = AF_MAYBEAMBIGUOUS;
			errFlag = sal_True;
		} else if ( fAHead->hasFlag(AF_MAYBEDEFAULT) )							  
		{
			flag = AF_MAYBEDEFAULT;
			errFlag = sal_True;										  
		} else if ( fAHead->hasFlag(AF_REMOVEABLE) )							  
		{
			flag = AF_REMOVEABLE;
			errFlag = sal_True;
		} else if ( fAHead->hasFlag(AF_PROPERTY) )
		{
			flag = AF_PROPERTY;
			errFlag = sal_True;
		} 

		if ( errFlag )
			IDL_Global::idl_global()->err()
				->attributeflag_error(UTL_Error::EIDL_WRONGATTRIBUTEFLAG, flag);

		if (globalHead)
			$$ = (FE_AttributeHead*)globalHead;
		else
			$$ = new FE_AttributeHead();
	}
	;

opt_attrflags :
	'[' opt_attrflags2 ']'

opt_attrflags2 :
	opt_attrflags2 ',' opt_attrflag 
	| opt_attrflag
	;

opt_attrflag :
	IDL_ATTRIBUTE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_ATTRIBUTE);
	}
	| IDL_PROPERTY
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_PROPERTY);
	}
	| IDL_READONLY
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrROSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setReadOnly(sal_True);
		fAHead->setFlag(AF_READONLY);
	}
	| IDL_OPTIONAL
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setOptional(sal_True);
		fAHead->setFlag(AF_OPTIONAL);
	}
	| IDL_MAYBEVOID
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_MAYBEVOID);
	}
	| IDL_BOUND
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_BOUND);
	}
	| IDL_CONSTRAINED
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_CONSTRAINED);
	}
	| IDL_TRANSIENT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_TRANSIENT);
	}
	| IDL_MAYBEAMBIGUOUS
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_MAYBEAMBIGUOUS);
	}
	| IDL_MAYBEDEFAULT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_MAYBEDEFAULT);
	}
	| IDL_REMOVEABLE
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_AttrOptSeen);

		if (globalHead == NULL) globalHead = new FE_AttributeHead(); 
		FE_AttributeHead *fAHead = (FE_AttributeHead*)globalHead;
		fAHead->setFlag(AF_REMOVEABLE);
	}
	;

exception_header :
	IDL_EXCEPTION
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ExceptSeen);
	}
	id
	inheritance_spec	
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_InheritSpecSeen);
		/*
		 * Create an AST representation of the information in the header
		 * part of an exception - this representation contains a computed
		 * list of all exceptions which this exceptions inherits from,
		 * recursively
		 */
		$$ = new FE_ExceptionHeader(new UTL_ScopedName(*$3, NULL), $4);
	}
	;

exception :
	exception_header
	{
		UTL_Scope               *s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Exception           *e = NULL;
		UTL_StrList             *p = IDL_Global::idl_global()->pragmas();

		/*
		 * Create a node representing an exception and add it to
		 * the enclosing scope
		 */
		if (s != NULL)
		{
			e = IDL_Global::idl_global()->gen()
				->create_exception($1->exception_name(), $1->inherits(), $1->n_inherits(), p);

			if ($1->n_inherits() > 1)
			{
				IDL_Global::idl_global()->err()
					->error1(UTL_Error::EIDL_MULTIBLE_INHERITANCE, e);
			}

			(void) s->fe_add_exception(e);
		}
		/*
		 * Push the exception scope on the scope stack
		 */
		IDL_Global::idl_global()->scopes()->push(e);
	}
	'{'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ExceptSqSeen);
	}
	members
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ExceptBodySeen);
	}
	'}'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_ExceptQsSeen);
		/*
		 * Done with this exception. Pop its scope from the scope stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
    }
    ;

operation :
	operation_head
	op_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpTypeSeen);
	}
	IDENTIFIER
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		UTL_ScopedName	*n = new UTL_ScopedName($4, NULL);
		AST_Operation	*o = NULL;
		UTL_StrList		*p = IDL_Global::idl_global()->pragmas();

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpIDSeen);
		/*
		 * Create a node representing an operation on an interface
		 * and add it to its enclosing scope
		 */
		if (s != NULL && $2 != NULL)
		{
			AST_Type *tp = AST_Type::narrow_from_decl($2);
			if (tp == NULL)
			{
				IDL_Global::idl_global()->err()->not_a_type($2);
			} else 
				if (tp->node_type() == AST_Decl::NT_except)
				{
					IDL_Global::idl_global()->err()->not_a_type($2);
				} else
				{
					o = IDL_Global::idl_global()->gen()
								->create_operation(tp, $1->attribute(),
													$1->implementation(), (char*)$1->local().getStr(), 
													$1->isConst(), n, p);
					                                                                                                               
					(void) s->fe_add_operation(o);
				}
		}
		/*
		 * Push the operation scope onto the scopes stack
		 */
		IDL_Global::idl_global()->scopes()->push(o);

		delete($1);
		globalHead = NULL;
	}
	parameter_list
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Operation	*o = NULL;	
		
		if (s != NULL && s->scope_node_type() == AST_Decl::NT_op)
		{
			o = AST_Operation::narrow_from_scope(s);
		}

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpParsCompleted);
	}
	opt_raises
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpRaiseCompleted);
	}
	opt_context
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Operation   *o = NULL;

		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpCompleted);
		/*
		 * Add exceptions and context to the operation
		 */
		if (s != NULL && s->scope_node_type() == AST_Decl::NT_op)
		{
			o = AST_Operation::narrow_from_scope(s);

			if ($8 != NULL && o != NULL)
				(void) o->fe_add_exceptions($8);
			if ($10 != NULL)
				(void) o->fe_add_context($10);
		}
		/*
		 * Done with this operation. Pop its scope from the scopes stack
		 */
		IDL_Global::idl_global()->scopes()->pop();
	}
	;

operation_head :
	'['
	opt_opflags
	']'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpHeadSeen);
		
		if (globalHead == NULL) globalHead = new FE_OperationHead(); 
		
		$$ = (FE_OperationHead*)globalHead;
	}
	| /* EMPTY */
	{
		if (globalHead == NULL) globalHead = new FE_OperationHead(); 
		FE_OperationHead *fOpHead = (FE_OperationHead*)globalHead;
		
		$$ = (FE_OperationHead*)globalHead;
	}
	;

opt_opflags :
	opt_opflag opt_opflags
	| /* Empty */
	;

opt_opflag :
	opt_op_attribute
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);

		if (globalHead == NULL) globalHead = new FE_OperationHead(); 
		FE_OperationHead *fOpHead = (FE_OperationHead*)globalHead;
		fOpHead->setAttribute($1);
	}
	| opt_const
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);

		if (globalHead == NULL) globalHead = new FE_OperationHead(); 
		FE_OperationHead *fOpHead = (FE_OperationHead*)globalHead;
		fOpHead->setConst($1);
	}
	;

opt_op_attribute :
	','
	IDL_ONEWAY
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);
		$$ = AST_Operation::OP_oneway;
	}
	| IDL_ONEWAY
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);
		$$ = AST_Operation::OP_oneway;
	}
	| ','
	IDL_IDEMPOTENT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);
		$$ = AST_Operation::OP_idempotent;
	}
	| IDL_IDEMPOTENT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);
		$$ = AST_Operation::OP_idempotent;
	}
	;

opt_const :
	','
	IDL_CONST
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);
		$$ = sal_True;
	}
	| IDL_CONST
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpAttrSeen);
		$$ = sal_True;
	}
	;

op_type_spec :
	simple_type_spec
	| IDL_VOID
	{
		$$ = IDL_Global::idl_global()->scopes()->bottom()
				->lookup_primitive_type(AST_Expression::EV_void);
	}
	;

parameter_list :
	'('
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpSqSeen);
	}
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpQsSeen);
	}
	| '('
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpSqSeen);
	}
	at_least_one_parameter
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpQsSeen);
	}
	;

at_least_one_parameter : parameter parameters ;

parameters :
	parameters
	','
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpParCommaSeen);
	}
	parameter
	| /* EMPTY */
	;

parameter :
	'['
	direction
	']'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpParDirSeen);
	}
	simple_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpParTypeSeen);
	}
	declarator
	{
		UTL_Scope		*s = IDL_Global::idl_global()->scopes()->top_non_null();
		AST_Argument    *a = NULL;
		UTL_StrList     *p = IDL_Global::idl_global()->pragmas();
		
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpParDeclSeen);
		/*
		 * Create a node representing an argument to an operation
		 * Add it to the enclosing scope (the operation scope)
		 */
		if ($5 != NULL && $7 != NULL && s != NULL)
		{
			AST_Type *tp = $7->compose($5);
			if (tp != NULL)
			{
				a = IDL_Global::idl_global()->gen()->
						create_argument($2, tp, $7->name(), p);
								
				if (!$7->checkType($5))
					IDL_Global::idl_global()->err()
						->warning2(UTL_Error::WIDL_TYPE_IDENT_CONFLICT, a, tp);
				
				(void) s->fe_add_argument(a);
			}
		}
	}
	| error
	simple_type_spec
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_NoState);
		yyerrok;
	}
	;

direction :
	IDL_IN
	{
		$$ = AST_Argument::dir_IN;
	}
	| IDL_OUT
	{
		$$ = AST_Argument::dir_OUT;
	}
	| IDL_INOUT
	{
		$$ = AST_Argument::dir_INOUT;
	}
	;

opt_raises : 
	IDL_RAISES
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpRaiseSeen);
	}
	'('
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpRaiseSqSeen);
	}
	at_least_one_scoped_name
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpRaiseQsSeen);
		$$ = $5;
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;

opt_context : 
	IDL_CONTEXT
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpContextSeen);
	}
	'('
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpContextSqSeen);
	}
	at_least_one_string_literal
	')'
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpContextQsSeen);
		$$ = $5;
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;

at_least_one_string_literal :
	STRING_LITERAL string_literals
	{
		$$ = new UTL_StrList($1, $2);
	}
	;

string_literals : 
	string_literals
	','
	{
		IDL_Global::idl_global()->set_parse_state(IDL_Global::PS_OpContextCommaSeen);
	}
	STRING_LITERAL
	{
		if ($1 == NULL)
			$$ = new UTL_StrList($4, NULL);
		else
		{
			$1->nconc(new UTL_StrList($4, NULL));
			$$ = $1;
		}
	}
	| /* EMPTY */
	{
		$$ = NULL;
	}
	;


%%
/* programs */

/*
 * Report an error situation discovered in a production
 *
 * This does not do anything since we report all error situations through
 * IDL_Global::idl_global()->err() operations
 */
void yyerror(char *errmsg)
{
	IDL_Global::idl_global()->err()
		->syntax_error(IDL_Global::idl_global()->parse_state(), errmsg);
}
