/*************************************************************************
 *
 *  $RCSfile: astarray.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Array::AST_Array()
	:	pd_n_dims(0),
		pd_base_type(NULL),
		pd_dims(NULL)
{
}

AST_Array::AST_Array(UTL_ScopedName *n, unsigned long nd, UTL_ExprList *ds)
	:	pd_n_dims(nd), pd_base_type(NULL),
		AST_Decl(AST_Decl::NT_array, n, NULL)
{
	pd_dims = compute_dims(ds, nd);
}

AST_Array::~AST_Array()
{
/*
	if (pd_base_type != NULL) delete(pd_base_type);
	if (pd_dims != NULL) delete[] pd_dims;
*/
}

/*
 * Private operations
 */

/*
 * Compute how many dimensions there are and collect their expressions
 * into an array
 */
AST_Expression **AST_Array::compute_dims(UTL_ExprList *ds, unsigned long nds)
{
	AST_Expression **result;
	UTL_ExprlistActiveIterator *l;
	unsigned long      i;

	if (ds == NULL)
		return NULL;

	result = new AST_Expression *[nds];
	l      = new UTL_ExprlistActiveIterator(ds);

	for (i = 0; !(l->is_done()) && i < nds; l->next(), i++)
		result[i] = l->item();
	delete l;
	return result;
}

/*
 * Public operations
 */


/*
 * Redefinition of inherited virtual operations
 */

/*
 * Dump this AST_Array node to the ostream o
 */
void AST_Array::dump(ostream &o)
{
	unsigned long  i;

	pd_base_type->dump(o);
	o << " ";
	o << local_name();
	for (i = 0; i < pd_n_dims; i++)
	{
		o << "[";
		pd_dims[i]->dump(o);
		o << "]";
	}
}

/*
 * Data accessors
 */
const unsigned long AST_Array::n_dims()
{
	return pd_n_dims;
}

AST_Expression **AST_Array::dims()
{
	return pd_dims;
}

AST_Type *AST_Array::base_type()
{
	return pd_base_type;
}

void AST_Array::set_base_type(AST_Type *nbt)
{
	pd_base_type = nbt;
}

void AST_Array::set_dims(AST_Expression **ds, unsigned long nds)
{
	pd_dims = ds;
	pd_n_dims = nds;
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS1(AST_Array, AST_ConcreteType)
IMPL_NARROW_FROM_DECL(AST_Array)
