/*************************************************************************
 *
 *  $RCSfile: astconst.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Static functions
 */

/*
 * Convert a value from the enum AST_Expression::ExprType to a char *
 */
static char *exprtype_to_string(AST_Expression::ExprType et)
{
	switch (et) 
	{
		case AST_Expression::EV_short:
			return "short";
		case AST_Expression::EV_ushort:
			return "unsigned short";
		case AST_Expression::EV_long:
			return "long";
		case AST_Expression::EV_ulong:
			return "unsigned long";
		case AST_Expression::EV_float:
			return "float";
		case AST_Expression::EV_double:
			return "double";
		case AST_Expression::EV_char:
			return "char";
		case AST_Expression::EV_byte:
			return "byte";
		case AST_Expression::EV_octet:
			return "octet";
		case AST_Expression::EV_bool:
			return "boolean";
		case AST_Expression::EV_string:
			return "string";
		case AST_Expression::EV_any:
			return "any";
		case AST_Expression::EV_void:
			return "void";
		case AST_Expression::EV_none:
			return "none";
	 }

	  return ("unknown");
}

/*
 * Constructor(s) and destructor
 */

/*
 * Default constructor
 */
AST_Constant::AST_Constant()
	:	pd_constant_value(NULL), pd_et(AST_Expression::EV_none)
{
}

/*
 * Used in constructing AST_EnumVal nodes
 */
AST_Constant::AST_Constant(AST_Expression::ExprType t,
			   AST_Decl::NodeType nt,
			   AST_Expression *v,
			   UTL_ScopedName *n,
			   UTL_StrList *p)
	:	AST_Decl(nt, n, p),
	    pd_constant_value(IDL_Global::idl_global()->gen()->create_expr(v, t)),
	    pd_et(t)
{
}

/*
 * Used when constructing AST_Constant nodes
 */
AST_Constant::AST_Constant(AST_Expression::ExprType t,
			   AST_Expression *v,
			   UTL_ScopedName *n,
			   UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_const, n, p),
	    pd_constant_value(IDL_Global::idl_global()->gen()->create_expr(v, t)),
	    pd_et(t)
{
}

AST_Constant::~AST_Constant()
{
	//if (pd_constant_value != NULL) delete(pd_constant_value);
}


/*
 * Private operations
 */

/*
 * Public operations
 */


/*
 * Redefinition of inherited virtual operations
 */

/*
 * Dump this AST_Constant node to the ostream o
 */
void AST_Constant::dump(ostream &o)
{
	o << "const " << exprtype_to_string(pd_et) << " "
	  << local_name() << " = ";
	pd_constant_value->dump(o);
}

/*
 * Data accessors
 */

AST_Expression *AST_Constant::constant_value()
{
	return pd_constant_value;
}

AST_Expression::ExprType AST_Constant::et()
{
	return pd_et;
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS1(AST_Constant, AST_Decl)
IMPL_NARROW_FROM_DECL(AST_Constant)



/*
 * Constructor(s) and destructor
 */
AST_Constants::AST_Constants()
{
}

AST_Constants::AST_Constants(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_constants, n, p),
		UTL_Scope(AST_Decl::NT_constants)
{
}


/*
 * Add this AST_Constant node (a constant declaration) to this scope
 */
AST_Constant *AST_Constants::fe_add_constant(AST_Constant *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}


/*
 * Dump this AST_Module node to the ostream o
 */
void AST_Constants::dump(ostream &o)
{
	o << "constants " << local_name() << " {\n";
	UTL_Scope::dump(o);
	IDL_Global::idl_global()->indent()->skip_to(o);
	o << "}";
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS2(AST_Constants, AST_Decl, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Constants)
IMPL_NARROW_FROM_SCOPE(AST_Constants)
