/*************************************************************************
 *
 *  $RCSfile: astdecl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */

AST_Decl::AST_Decl()
	:	pd_node_type(NT_module),
		pd_line(-1),
		pd_local_name(NULL),
		pd_file_name(NULL),
		pd_pragmas(NULL),
		pd_defined_in(NULL),
		pd_imported(sal_False),
		pd_in_main_file(sal_False),
		pd_name(NULL),
		pd_added(sal_False)
{
}

AST_Decl::AST_Decl(NodeType nt, UTL_ScopedName *n, UTL_StrList *p)
	:	pd_node_type(nt),
		pd_line(IDL_Global::idl_global()->lineno()),
		pd_local_name(n == NULL ? NULL : &(n->last_component())),
		pd_pragmas(p),
		pd_defined_in(IDL_Global::idl_global()->scopes()->depth() > 0
						? IDL_Global::idl_global()->scopes()->top()
						: NULL),
		pd_imported(IDL_Global::idl_global()->imported()),
		pd_in_main_file(IDL_Global::idl_global()->in_main_file()),
		pd_added(sal_False)
{
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		pd_file_name = new String(*(IDL_Global::idl_global()->main_filename()));
	} else
	{
		pd_file_name = new String(*(IDL_Global::idl_global()->filename()));
	}

	compute_full_name(n);
}

AST_Decl::~AST_Decl()
{
	//if (pd_defined_in != NULL) delete(pd_defined_in);
	//if (pd_file_name != NULL) delete(pd_file_name);

// Hack bis gesamte Struktur auf VOS Strings angepasst ist.
// Alle Listen und aehnliches !!!
#ifndef UNX
//	if (pd_name != NULL) delete(pd_name);
#endif
	//if (pd_local_name != NULL) delete(pd_local_name);
	if (pd_pragmas != NULL) delete(pd_pragmas);
}

/*
 * Private operations
 */

/*
 * Compute the full scoped name of an AST node
 */
void AST_Decl::compute_full_name(UTL_ScopedName *n)
{
	UTL_ScopedName	*cn = NULL;
	AST_Decl		*d  = NULL;

	/*
	 * Initialize this name to NULL
	 */
	pd_name = NULL;
	/*
	 * Global scope?
	 */
	if (defined_in() == NULL 
		|| ScopeAsDecl(defined_in())->local_name() == NULL
		|| defined_in()->scope_node_type() == NT_mapping)
	{
		pd_name = n;
		return;
	}
	/*
	 * OK, not global. So copy name of containing scope, then
	 * smash last cdr of copy with new component
	 */
	d = ScopeAsDecl(defined_in());
/*	
	if (d->local_name()->length() == 0 && d->node_type() == NT_module)
	{
		pd_name = n;
		return;
	}
*/
	if (d != NULL)
		cn = d->name();
	if (cn != NULL)
		pd_name = (UTL_ScopedName *) cn->copy();
	if (pd_name == NULL)
		pd_name = new UTL_ScopedName(*local_name(), NULL);
	else
		pd_name->nconc(new UTL_ScopedName(*local_name(), NULL));
}

/*
 * Public operations
 */

/*
 * Return TRUE if one of my ancestor scopes is "s"
 * and FALSE otherwise
 */
sal_Bool AST_Decl::has_ancestor(AST_Decl *s)
{
	if (this == s)
		return sal_True;
	if (pd_defined_in == NULL)
		return sal_False;
	return ScopeAsDecl(pd_defined_in)->has_ancestor(s);
}    

/*
 * Dump this AST_Decl to the ostream o
 */
void AST_Decl::dump(ostream &o)
{
	o << pd_local_name;
}

void AST_Decl::dump()
{
	fprintf(stderr, "%s", pd_local_name);
}

/*
 * Redefinition of inherited virtual operations
 */

/*
 * Data accessors
 */

sal_Bool AST_Decl::imported()
{
	return pd_imported;
}

void AST_Decl::set_imported(sal_Bool is_it)
{
	pd_imported = is_it;
}

sal_Bool AST_Decl::in_main_file()
{
	return pd_in_main_file;
}

void AST_Decl::set_in_main_file(sal_Bool is_it)
{
	pd_in_main_file = is_it;
}

sal_Bool AST_Decl::added()
{
	return pd_added;
}

void AST_Decl::set_added(sal_Bool is_it)
{
	pd_added = is_it;
}

UTL_Scope *AST_Decl::defined_in()
{
	return pd_defined_in;
}

void AST_Decl::set_defined_in(UTL_Scope *s)
{
	pd_defined_in = s;
}

AST_Decl::NodeType AST_Decl::node_type()
{
	return pd_node_type;
}

long AST_Decl::line()
{
	return pd_line;
}

void AST_Decl::set_line(long l)
{
	pd_line = l;
}

String *AST_Decl::file_name()
{
	return pd_file_name;
}

void AST_Decl::set_file_name(String *s)
{
	pd_file_name = s;
}

UTL_ScopedName *AST_Decl::name()
{
	return pd_name;
}

void AST_Decl::set_name(UTL_ScopedName *n)
{
	pd_name = n;
	if (n != NULL)
		pd_local_name = &(n->last_component());
}

Identifier *AST_Decl::local_name()
{
	return pd_local_name;
}

void AST_Decl::add_pragmas(UTL_StrList *p)
{
	if (p != NULL) 
	{
		if (pd_pragmas != NULL)
			pd_pragmas->nconc(p);
		else
			pd_pragmas = p;
	}
}

UTL_StrList *AST_Decl::pragmas()
{
	return pd_pragmas;
}

/*
 * Narrowing methods for AST_Decl
 */
IMPL_NARROW_METHODS0(AST_Decl)
IMPL_NARROW_FROM_DECL(AST_Decl)

