/*************************************************************************
 *
 *  $RCSfile: astenum.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Enum::AST_Enum()
	:	pd_enum_counter(0)
{
}

AST_Enum::AST_Enum(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_enum, n, p),
		UTL_Scope(AST_Decl::NT_enum),
		pd_enum_counter(0)
{
}

/*
 * Private operations
 */

/*
 * Public operations
 */

/*
 * Look up an enumerator by the value of the supplied expression
 */
AST_EnumVal *AST_Enum::lookup_by_value(const AST_Expression *v)
{
	UTL_ScopeActiveIterator *iter;
	AST_EnumVal		    *item;
	AST_Decl		  *i;

	iter = new UTL_ScopeActiveIterator(this, IK_decls);

	while (!iter->is_done()) 
	{
		i = iter->item();
		item = AST_EnumVal::narrow_from_decl(i);
		if (item->constant_value() == v) 
		{
			delete iter;
			return item;
		}
		iter->next();
	}
	delete iter;
	return NULL;
}


/*
 * Check up an enumerator by the value of the supplied expression
 */
AST_EnumVal *AST_Enum::check_value(AST_Expression *v)
{
	UTL_ScopeActiveIterator *iter;
	AST_EnumVal		    *item;
	AST_Decl		  *i;

	iter = new UTL_ScopeActiveIterator(this, IK_decls);

	while (!iter->is_done()) 
	{
		i = iter->item();
		item = AST_EnumVal::narrow_from_decl(i);
		if (item->constant_value()->compare(v)) 
		{
			delete iter;
			return item;
		}
		iter->next();
	}
	delete iter;
	return NULL;
}


/*
 * Static helper functions
 */

/*
 * Modify scoped name of an enumval so that it is scoped inside the scope
 * in which the enum is defined and not inside the enum itself
 */
static UTL_ScopedName *munge_name_for_enumval(UTL_ScopedName *n, Identifier *last_component)
{
	long		  l	= n->length();
	UTL_ScopedName *hold	= n;

	/*
	 * last three components are:
	 * - scope in which enum is defined
	 * - scope for enum
	 * - name of enumval
	 *
	 * We want to stop cdr'ing down the list when the head of the
	 * list is at the name for the scope in which the enum is defined
	 */
	while (l > 3) 
	{
		l--;
		n = (UTL_ScopedName *) n->tail();
	}

	n->set_tail(new UTL_IdList(*last_component, NULL));

	return hold;
}
  
/*
 * Redefinition of inherited virtual operations
 */

/*
 * Add an AST_EnumVal node to this scope
 */
AST_EnumVal *AST_Enum::fe_add_enum_val(AST_EnumVal *t)
{
	AST_Decl *d;

	if (t != NULL) 
		t->set_name(munge_name_for_enumval(t->name(), t->local_name()));

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);
	/*
	 * Add it to enclosing scope
	 */
//	IDL_Global::idl_global()->scopes()->next_to_top()->fe_add_enum_val(t);

	return t;
}

/*
 * Dump this AST_Enum to the ostream o
 */
void AST_Enum::dump(ostream &o)
{
	UTL_ScopeActiveIterator *i = new UTL_ScopeActiveIterator(this, IK_decls);
	AST_Decl		    *d;

	o << "enum " << local_name() << " {\n";

	while (!(i->is_done())) {
		d = i->item();
		o << d->local_name();
		i->next();
		if (!(i->is_done()))
		o << ", ";
	}
	delete i;

	IDL_Global::idl_global()->indent()->skip_to(o);
	o << "}";
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS2(AST_Enum, AST_ConcreteType, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Enum)
IMPL_NARROW_FROM_SCOPE(AST_Enum)
