/*************************************************************************
 *
 *  $RCSfile: astinter.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:14 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Interface::AST_Interface()
	:	pd_n_inherits(0),
		pd_inherits(NULL),
   		pd_uik1(NULL),
		pd_uik2(NULL),
		pd_uik3(NULL),
		pd_uik4(NULL),
		pd_uik5(NULL),
		pd_ident(NULL),
		pd_version1(0),
		pd_version2(0),
		pd_seen(sal_False),
		pd_virtual(sal_False),
		pd_forwarded(sal_False),
		pd_forwardInSameFile(sal_False)
{
}

AST_Interface::AST_Interface(UTL_ScopedName *n, AST_Interface **ih,
							 long nih, UTL_StrList *p, char* uik,
							 double version, sal_Bool vrt, char* ident)
    :	AST_Decl(AST_Decl::NT_interface, n, p),
		UTL_Scope(AST_Decl::NT_interface),
		pd_n_inherits(nih),
		pd_inherits(ih),
		pd_seen(sal_False),
		pd_virtual(vrt),
		pd_forwarded(sal_False),
		pd_forwardInSameFile(sal_False)
{
	if (ident != NULL) 
	{
		pd_ident = new char[strlen(ident)+1];
		strcpy(pd_ident, ident);
	} else
		pd_ident = NULL;

	transformUIK(uik);
	transformVersion(version);
}

AST_Interface::~AST_Interface()
{
/*
	for(int i=0; i < pd_n_inherits; i++)
		if(pd_inherits[i] != NULL) delete(pd_inherits[i]);
	if(pd_uik1 != NULL) delete(pd_uik1);
	if(pd_uik2 != NULL) delete(pd_uik2);
	if(pd_uik3 != NULL) delete(pd_uik3);
	if(pd_uik4 != NULL) delete(pd_uik4);
	if(pd_uik5 != NULL) delete(pd_uik5);
	if(pd_ident != NULL) delete(pd_ident);
*/
}


/*
 * Private operations
 */

/*
 * Public operations
 */


/*
 * Redefinition of inherited virtual operations
 */

/*
 * Add an AST_Constant node (a constant declaration) to this scope
 */
AST_Constant *AST_Interface::fe_add_constant(AST_Constant *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Exception node (an exception declaration) to this scope
 */
AST_Exception *AST_Interface::fe_add_exception(AST_Exception *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Attribute node (an attribute declaration) to this scope
 */
AST_Attribute *AST_Interface::fe_add_attribute(AST_Attribute *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
		return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Operation node (an operation declaration) to this scope
 */
AST_Operation *AST_Interface::fe_add_operation(AST_Operation *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
		return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Structure (a struct declaration) to this scope
 */
AST_Structure *AST_Interface::fe_add_structure(AST_Structure *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Enum node (an enum declaration) to this scope
 */
AST_Enum *AST_Interface::fe_add_enum(AST_Enum *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Union (a union declaration) to this scope
 */
AST_Union *AST_Interface::fe_add_union(AST_Union *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_EnumVal node (an enumerator) to this scope.
 * This is done to conform to the C++ scoping rules which declare
 * enumerators in the enclosing scope (in addition to declaring them
 * in the enum itself)
 */
AST_EnumVal *AST_Interface::fe_add_enum_val(AST_EnumVal *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Typedef (a typedef) to the current scope
 */
AST_Typedef *AST_Interface::fe_add_typedef(AST_Typedef *t)
{
	AST_Decl *d;

	/*
	 * Can't add to interface which was not yet defined
	 */
	if (!is_defined()) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_DECL_NOT_DEFINED, this, t);
		return NULL;
	}
	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) {
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Dump this AST_Interface node to the ostream o
 */
void
AST_Interface::dump(ostream &o)
{
	long	i;

	o << "interface " << local_name() << " ";
	if (pd_n_inherits > 0) 
	{
		o << ": ";
		for (i = 0; i < pd_n_inherits; i++) 
		{
			o << pd_inherits[i]->local_name();
			if (i < pd_n_inherits - 1)
				o << ", ";
		}
	}
	o << " {\n";
	UTL_Scope::dump(o);
	IDL_Global::idl_global()->indent()->skip_to(o);
	o << "}";
}

/*
 * Data accessors
 */

AST_Interface	**
AST_Interface::inherits()
{
	return pd_inherits;
}

void
AST_Interface::set_inherits(AST_Interface **i)
{
	pd_inherits = i;
}

long
AST_Interface::n_inherits()
{
	return pd_n_inherits;
}

void
AST_Interface::set_n_inherits(long i)
{
	pd_n_inherits = i;
}

char* AST_Interface::uik1() 
{
	return pd_uik1;
}

char* AST_Interface::uik2()
{
	return pd_uik2;
}

char* AST_Interface::uik3() 
{
	return pd_uik3;
}

char* AST_Interface::uik4() 
{
	return pd_uik4;
}

char* AST_Interface::uik5() 
{
	return pd_uik5;
}

char* AST_Interface::ident() 
{
	return pd_ident;
}

void AST_Interface::set_ident(char* ident)
{
	if (pd_ident != NULL) delete(pd_ident);

	if (ident != NULL) 
	{
		pd_ident = new char[strlen(ident)+1];
		strcpy(pd_ident, ident);
	} else
		pd_ident = NULL;
}

sal_Bool AST_Interface::isvirtual()
{
	return pd_virtual;
}

void AST_Interface::set_uik(char* uik)
{
	transformUIK(uik);
}

unsigned short AST_Interface::version1()
{
	return pd_version1;
}

unsigned short AST_Interface::version2()
{
	return pd_version2;
}

void AST_Interface::set_version(double version)
{
	transformVersion(version);
}

// ckeck for multible inheritance
sal_Bool AST_Interface::checkInheritance() 
{
	AST_Interface *bi;

	if(n_inherits() == 0) return sal_True;

	resetSeen();
	pd_seen = sal_True;
	
	if((bi = checkAllInheritance()) != NULL) 
	{
		IDL_Global::idl_global()->err()->
			error2(UTL_Error::EIDL_MULTIBLE_INHERITANCE, this, bi);	
		return sal_False;
	} else
		return sal_True;
}

// check for multible inheritance recursive
AST_Interface* AST_Interface::checkAllInheritance() {
	AST_Interface *ri = NULL; 
	AST_Interface *bi;
  
	for(int i = 0; i < n_inherits(); i++)
	{
		bi = AST_Interface::narrow_from_decl(inherits()[i]);
		if(!bi->pd_seen)
		{
			bi->pd_seen = sal_True;
			if((ri = bi->checkAllInheritance()) != NULL)
				return ri;
		} else
			ri = bi;
	}

	return ri;
}


// transform the uiid in five secions 
void AST_Interface::transformUIK(char* uik) 
{
	char* uid = uik;

	if (uik != NULL && !isdigit(*uik) 
		&& *uik!='a' && *uik!='A'
		&& *uik!='b' && *uik!='B'
		&& *uik!='c' && *uik!='C'
		&& *uik!='d' && *uik!='D'
		&& *uik!='e' && *uik!='E'
		&& *uik!='f' && *uik!='F')
	{
		fprintf(stderr, "Unoidl Error: uik error\n");
		exit(1);
	}

	if (uik != NULL) 
	{
		pd_uik1 = new char[9];
		pd_uik2 = new char[5];
		pd_uik3 = new char[5];
		pd_uik4 = new char[9];
		pd_uik5 = new char[9];
		strncpy(pd_uik1, uid, 8);pd_uik1[8]='\0';
		strncpy(pd_uik2, uid+=9, 4);pd_uik2[4]='\0';
		strncpy(pd_uik3, uid+=5, 4);pd_uik3[4]='\0';
		strncpy(pd_uik4, uid+=5, 8);pd_uik4[8]='\0';
		strncpy(pd_uik5, uid+=9, 8);pd_uik5[8]='\0';
	} else 
	{
		pd_uik1 = NULL;
		pd_uik2 = NULL;
		pd_uik3 = NULL;
		pd_uik4 = NULL;
		pd_uik5 = NULL;
	}
}

// transform the version 
void AST_Interface::transformVersion(double version) 
{
	char buffer[32];
	char *buffer2;
	short len;

	sprintf(buffer, "%f", version);
	strtok(buffer, ".");
	buffer2 = strtok(NULL, ".");

	len = strlen(buffer2)-1;
	while(buffer2[len] == '0')
		len--;
	
	if(len < 0) len++;
	buffer2[len+1]='\0';

	pd_version1 = atoi(buffer);
	pd_version2 = atoi(buffer2);
}

// append all superclasses of this interface to List l:
//
void AST_Interface::getAllSuperClasses(UTL_NList& l) 
{
	resetSeen();
	pd_seen = sal_True;
	getAllSuperClassesSub(l);
}

// needed by getAllSuperClasses:
//
void AST_Interface::getAllSuperClassesSub(UTL_NList& l) 
{
	AST_Interface *bi;
  
	for(int i = 0; i < n_inherits(); i++)
	{
		bi = AST_Interface::narrow_from_decl(inherits()[i]);
		if(!bi->pd_seen)
		{
			bi->pd_seen = sal_True;
			bi->getAllSuperClassesSub(l);
	  
			l.append(new AST_InterfaceElem(bi));
		}
	}
}

// needed by getAllSuperClasses:
//
void AST_Interface::resetSeen() 
{
	pd_seen = sal_False;
  
	for(int i = 0; i < n_inherits(); i++)
	{
		narrow_from_decl(inherits()[i])->resetSeen();
	}
}

void AST_Interface::set_forwarded(sal_Bool fwd) 
{
	pd_forwarded = fwd;
}

sal_Bool AST_Interface::is_forwarded() 
{
	return pd_forwarded;
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS2(AST_Interface, AST_Type, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Interface)
IMPL_NARROW_FROM_SCOPE(AST_Interface)
