/*************************************************************************
 *
 *  $RCSfile: astmodul.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Module::AST_Module()
{
}

AST_Module::AST_Module(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_module, n, p),
		UTL_Scope(AST_Decl::NT_module)
{
}

/*
 * Private operations
 */

/*
 * Public operations
 */

/*
 * Redefinition of inherited virtual operations
 */

/*
 * Add this AST_PredefinedType node (a predefined type declaration) to
 * this scope
 */
AST_PredefinedType *AST_Module::fe_add_predefined_type(AST_PredefinedType *t)
{
	AST_Decl *d = NULL;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Object node (a class declaration) to this scope
 */
AST_Object *AST_Module::fe_add_object(AST_Object *cl)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(cl, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, cl, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, cl, this, d);
			return NULL;
		}
		if (cl->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(cl, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(cl);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(cl, sal_False);

	return cl;
}

/*
 * Add this AST_Service node (a class declaration) to this scope
 */
AST_Service *AST_Module::fe_add_service(AST_Service *cl)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(cl, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, cl, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, cl, this, d);
			return NULL;
		}
		if (cl->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(cl, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(cl);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(cl, sal_False);

	return cl;
}

/*
 * Add this AST_Module node (a module declaration) to this scope
 */
AST_Module *AST_Module::fe_add_module(AST_Module *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	 }
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}


/*
 * Add this AST_Constants node (a const group declaration) to this scope
 */
AST_Constants *AST_Module::fe_add_constants(AST_Constants *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	 }
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Module node (a module declaration) to this scope
 */
AST_Mapping *AST_Module::fe_add_mapping(AST_Mapping *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	 }
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}


/*
 * Add this AST_Interface node (an interface declaration) to this scope
 */
AST_Interface *AST_Module::fe_add_interface(AST_Interface *t)
{
	AST_Decl		*predef;
	AST_Interface		*fwd;

	/*
	 * Already defined?
	 */
	if ((predef = lookup_for_add(t, sal_False)) != NULL) 
	{
		/*
		 * Treat fwd declared interfaces specially
		 */
		if (predef->node_type() == AST_Decl::NT_interface) 
		{
			fwd = AST_Interface::narrow_from_decl(predef);

			if (fwd == NULL)
				return NULL;
      
			if (!fwd->is_defined()) 
			{	/* Forward declared and not defined yet */
				if (fwd->defined_in() != this) 
				{
					IDL_Global::idl_global()->err()
						->error3(UTL_Error::EIDL_SCOPE_CONFLICT, fwd, t, this);
					return NULL;
				}
			}
			/*
			 * OK, not illegal redef of forward declaration. Now check whether
			 * it has been referenced already
			 */
			else 
				if (referenced(predef)) 
				{
					IDL_Global::idl_global()->err()
						->error3(UTL_Error::EIDL_DEF_USE, t, this, predef);
					return NULL;
				}
		} else if (!can_be_redefined(predef)) 
		{
			IDL_Global::idl_global()->err()
				->error3(UTL_Error::EIDL_REDEF, t, this, predef);
			return NULL;
		} else 
			if (referenced(predef)) 
			{
				IDL_Global::idl_global()->err()
					->error3(UTL_Error::EIDL_DEF_USE, t, this, predef);
				return NULL;
			} else 
				if (t->has_ancestor(predef)) 
				{
					IDL_Global::idl_global()->err()->redefinition_in_scope(t, predef);
					return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_InterfaceFwd node (a forward declaration of an IDL
 * interface) to this scope
 */
AST_InterfaceFwd *AST_Module::fe_add_interface_fwd(AST_InterfaceFwd *i)
{
	AST_Decl	*d;
	AST_Interface	*itf;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(i, sal_False)) != NULL) 
	{
		if (d->node_type() == AST_Decl::NT_interface &&
			d->defined_in() == this) 
		{
			itf = AST_Interface::narrow_from_decl(d);
			if (itf == NULL)
			return NULL;
      
			i->set_full_definition(itf);
			return i;
		}
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, i, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, i, this, d);
			return NULL;
		}
		if (i->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(i, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(i);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(i, sal_False);

	return i;
}

/*
 * Add this AST_Constant node (a constant declaration) to this scope
 */
AST_Constant *AST_Module::fe_add_constant(AST_Constant *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Exception node (an exception declaration) to this scope
 */
AST_Exception *AST_Module::fe_add_exception(AST_Exception *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Union node (a union declaration) to this scope
 */
AST_Union *AST_Module::fe_add_union(AST_Union *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}	
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Structure node (a struct declaration) to this scope
 */
AST_Structure *AST_Module::fe_add_structure(AST_Structure *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Enum node (an enum declaration) to this scope
 */
AST_Enum *AST_Module::fe_add_enum(AST_Enum *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_EnumVal node (an enumerator declaration) to this scope
 * This is done to conform to the C++ scoping rules which declare
 * enumerators in the enclosing scope (in addition to declaring them
 * in the enum itself)
 */
AST_EnumVal *AST_Module::fe_add_enum_val(AST_EnumVal *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Typedef node (a typedef) to this scope
 */
AST_Typedef *AST_Module::fe_add_typedef(AST_Typedef *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Dump this AST_Module node to the ostream o
 */
void AST_Module::dump(ostream &o)
{
	o << "module " << local_name() << " {\n";
	UTL_Scope::dump(o);
	IDL_Global::idl_global()->indent()->skip_to(o);
	o << "}";
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS2(AST_Module, AST_Decl, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Module)
IMPL_NARROW_FROM_SCOPE(AST_Module)
