/*************************************************************************
 *
 *  $RCSfile: astoper.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Operation::AST_Operation()
	:	pd_flags(OP_noflags),
		pd_iflag(IFLAG_noflag),
		pd_local(NULL),
		pd_const(sal_False),
	    pd_return_type(NULL),
	    pd_context(NULL),
	    pd_exceptions(NULL)
{
}

AST_Operation::AST_Operation(AST_Type *rt, Flags fl, IFlag ifl, 
				char* local, sal_Bool cons, UTL_ScopedName *n, 
				UTL_StrList *p)
	 :	pd_return_type(rt),
		pd_flags(fl),
		pd_iflag(ifl),
		pd_local(local),
		pd_const(cons),
		pd_context(NULL),
		pd_exceptions(NULL),
		AST_Decl(AST_Decl::NT_op, n, p),
		UTL_Scope(AST_Decl::NT_op)
{
	AST_PredefinedType *pdt;
	/*
	 * Check that if the operation is oneway, the return type must be void
	 */
	if (rt != NULL && pd_flags == OP_oneway) 
	{
		if (rt->node_type() != AST_Decl::NT_pre_defined)
			IDL_Global::idl_global()->err()->error1(UTL_Error::EIDL_NONVOID_ONEWAY, this);
		else 
		{
			pdt = AST_PredefinedType::narrow_from_decl(rt);
			if (pdt == NULL || pdt->pt() != AST_PredefinedType::PT_void)
				IDL_Global::idl_global()->err()
					->error1(UTL_Error::EIDL_NONVOID_ONEWAY, this);
		}
	}

	if (local != NULL) 
	{
		pd_local = new char[strlen(local)+1];
		strcpy(pd_local, local);
	} else
		pd_local = NULL;
}

AST_Operation::~AST_Operation()
{
	//if (pd_return_type != NULL) delete(pd_return_type);
	//if (pd_context != NULL) delete(pd_context);
	//if (pd_exceptions != NULL) delete(pd_exceptions);
	//if (pd_local != NULL) delete(pd_local);
}


/*
 * Private operations
 */

/*
 * Public operations
 */

/*
 * Redefinition of inherited virtual operations
 */

/*
 * Add this context (a UTL_StrList) to this scope
 */
UTL_StrList *AST_Operation::fe_add_context(UTL_StrList *t)
{
	pd_context = t;

	return t;
}

/*
 * Add these exceptions (identified by name) to this scope.
 * This looks up each name to resolve it to the name of a known
 * exception, and then adds the referenced exception to the list
 * of exceptions that this operation can raise.
 *
 * NOTE: No attempt is made to ensure that exceptions are mentioned
 *       only once..
 */
UTL_NameList *AST_Operation::fe_add_exceptions(UTL_NameList *t)
{
	UTL_NamelistActiveIterator *nl_i;
	UTL_ScopedName	     *nl_n;
	UTL_Scope		     *fs = IDL_Global::idl_global()->scopes()->top();
	AST_Exception		     *fe;
	AST_Decl		     *d;

	/*
	 * Cannot raise exceptions in oneway operation
	 */
	if (pd_flags == OP_oneway) 
	{
		IDL_Global::idl_global()->err()->error1(UTL_Error::EIDL_ONEWAY_RAISE_CONFLICT, this);
		return NULL;
	}

	pd_exceptions = NULL;
	nl_i = new UTL_NamelistActiveIterator(t);
	while (!(nl_i->is_done())) 
	{
		nl_n = nl_i->item();
		d = lookup_by_name(nl_n, sal_True);
		if (d == NULL || d->node_type() != AST_Decl::NT_except) 
		{
			IDL_Global::idl_global()->err()->lookup_error(nl_n);
			delete nl_i;
			return NULL;
		}
		fe = AST_Exception::narrow_from_decl(d);
		if (fe == NULL) 
		{
			IDL_Global::idl_global()->err()->error1(UTL_Error::EIDL_ILLEGAL_RAISES, this);
			return NULL;
		}
		if (pd_exceptions == NULL)
			pd_exceptions = new UTL_ExceptList(fe, NULL);
		else
			pd_exceptions->nconc(new UTL_ExceptList(fe, NULL));
		nl_i->next();
	}
	delete nl_i;

	return t;
}

/*
 * Add this AST_Argument node (an operation argument declaration)
 * to this scope
 */
AST_Argument *AST_Operation::fe_add_argument(AST_Argument *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_by_name_local(t->local_name(), sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Cannot add OUT or INOUT argument to oneway operation
	 */
	if ((t->direction() == AST_Argument::dir_OUT ||
		t->direction() == AST_Argument::dir_INOUT) &&
		pd_flags == OP_oneway) 
	{
		IDL_Global::idl_global()->err()->error2(UTL_Error::EIDL_ONEWAY_CONFLICT, t, this);
		return NULL;
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Dump this AST_Operation node (an operation) to the ostream o
 */
void AST_Operation::dump(ostream &o)
{
	UTL_ScopeActiveIterator   		*i;
	UTL_StrlistActiveIterator		*si;
	UTL_ExceptlistActiveIterator 	*ei;
	AST_Decl		    			*d;
	AST_Exception		    		*e;
	String		    				*s;

	if (pd_flags == OP_oneway)
		o << "oneway ";
	else if (pd_flags == OP_idempotent)
		o << "idempotent ";

	i = new UTL_ScopeActiveIterator(this, IK_decls);
	o << pd_return_type->name() << " " << local_name() << "(";
	while (!(i->is_done())) 
	{
		d = i->item();
		d->dump(o);
		i->next();
		if (!(i->is_done()))
			o << ", ";
	}
	delete i;
	o << ")";

	if (pd_exceptions != NULL) 
	{
		o << " raises(";
		ei = new UTL_ExceptlistActiveIterator(pd_exceptions);
		while (!(ei->is_done())) 
		{
			e = ei->item();
			ei->next();
			o << e->local_name();
			if (!(ei->is_done()))
				o << ", ";
		}
		delete ei;
		o << ")";
	}
	if (pd_context != NULL) 
	{
		o << " context(";
		si = new UTL_StrlistActiveIterator(pd_context);
		while (!(si->is_done())) 
		{
			s = si->item();
			si->next();
			o << s;
			if (!(si->is_done()))
			o << ", ";
		}
		delete si;
		o << ")";
	}    
}

/*
 * Data accessors
 */

AST_Type *AST_Operation::return_type()
{
	return pd_return_type;
}

AST_Operation::Flags AST_Operation::flags()
{
	return pd_flags;
}

IFlag AST_Operation::iflag()
{
	return pd_iflag;
}

char* AST_Operation::local()
{
	return pd_local;
}

sal_Bool AST_Operation::isConst()
{
	return pd_const;
}

UTL_StrList *AST_Operation::context()
{
	return pd_context;
}

UTL_ExceptList *AST_Operation::exceptions()
{
	return pd_exceptions;
}

// Narrowing
IMPL_NARROW_METHODS2(AST_Operation, AST_Decl, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Operation)
IMPL_NARROW_FROM_SCOPE(AST_Operation)
