/*************************************************************************
 *
 *  $RCSfile: astrecur.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * FE_illegal_recursive_type() - Implement the algorithm described above
 */
sal_Bool AST_illegal_recursive_type(AST_Decl *t)
{
	UTL_ScopeStackActiveIterator *i;
	UTL_Scope			*s;
	AST_Structure		*st1, *st2;
	AST_Union			*un1, *un2;
	sal_Bool			check_for_struct = sal_False,
						check_for_union = sal_False;

	if (t == NULL) return sal_False;
  
	/*
	 * We only care about structs and unions
	 */
	if (t->node_type() != AST_Decl::NT_struct &&
		t->node_type() != AST_Decl::NT_union)
		return sal_False;	// NOT ILLEGAL
	/*
	 * Narrow the type appropriately so comparison will work
	 */
	if (t->node_type() == AST_Decl::NT_struct) 
	{
		check_for_struct = sal_True;
		st1 = AST_Structure::narrow_from_decl(t);
		if (st1 == NULL)
			return sal_False;	// NOT ILLEGAL
	} else 
		if (t->node_type() == AST_Decl::NT_union) 
		{
			check_for_union = sal_True;
			un1 = AST_Union::narrow_from_decl(t);
			if (un1 == NULL)
				return sal_False;	// NOT ILLEGAL
		}
	/*
	 * OK, iterate up the stack
	 */
	i = new UTL_ScopeStackActiveIterator(IDL_Global::idl_global()->scopes());
	while (!(i->is_done())) 
	{
		s = i->item();
		/*
		 * If we hit a NULL we're done since it means that we're nested inside
		 * a sequence, where recursive types may be used
		 */
		if (s == NULL) 
		{
			delete i;
			return sal_False;	// NOT ILLEGAL
		}
		/*
		 * OK, must check this scope
		 */
		if (s->scope_node_type() == AST_Decl::NT_struct && check_for_struct) 
		{
			st2 = AST_Structure::narrow_from_scope(s);
			if (st2 != NULL && st2 == st1) 
			{
				delete i;
				return sal_True;	// ILLEGAL RECURSIVE TYPE USE
			}
		} else 
			if (s->scope_node_type() == AST_Decl::NT_union &&
				check_for_union) 
			{
				un2 = AST_Union::narrow_from_scope(s);
				if (un2 != NULL && un2 == un1) 
				{
					delete i;
					return sal_True;	// ILLEGAL RECURSIVE TYPE USE
				}
			}
		/*
		 * This one is fine, get next item
		 */
		i->next();
	}
	/*
	 * No more scopes to check. This type was used legally
	 */
	delete i;
	return sal_False;		// NOT ILLEGAL
}

