/*************************************************************************
 *
 *  $RCSfile: astserv.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Service::AST_Service()
{
}

AST_Service::AST_Service(UTL_ScopedName *n, UTL_StrList *p)
	:	AST_Decl(AST_Decl::NT_service, n, p),
		UTL_Scope(AST_Decl::NT_service)
{
}

AST_Service::~AST_Service()
{
}

/*
 * Private operations
 */

/*
 * Public operations
 */

/*
 * Redefinition of inherited virtual operations
 */

/*
 * Add this AST_InterfaceMember node to this scope
 */
AST_InterfaceMember *AST_Service::fe_add_interface_member(AST_InterfaceMember *om)
{
	AST_Decl	  *d;
	/*
	 * Already defined?
	 */
	if ((d = check_for_add(om)) != NULL) 
	{
		if (d->node_type() != NT_interface_member)
		{
			add_to_scope(om);
			return om;
		} else
		{
			AST_InterfaceMember *sM = AST_InterfaceMember::narrow_from_decl(d);

			if (sM && sM->getInterface() != om->getInterface())
			{
				add_to_scope(om);
				return(om); 
			}
		}
		
		IDL_Global::idl_global()->err()
			->error2(UTL_Error::EIDL_INTERFACEMEMBER_EXISTS, om, this);
		return NULL;
	}

	add_to_scope(om);

	return om;
}

/*
 * Add this AST_ServiceMember node to this scope
 */
AST_ServiceMember *AST_Service::fe_add_service_member(AST_ServiceMember *om)
{
	AST_Decl	*d;

	/*
	 * Already defined?
	 */
	if ((d = check_for_add(om)) != NULL)
	{
		if (d->node_type() != NT_service_member)
		{
			add_to_scope(om);
			return(om); 
		} else
		{
			AST_ServiceMember *sM = AST_ServiceMember::narrow_from_decl(d);

			if (sM && sM->getService() != om->getService())
			{
				add_to_scope(om);
				return(om); 
			}
		}

		IDL_Global::idl_global()->err()
			->error2(UTL_Error::EIDL_SERVICEMEMBER_EXISTS, om, this);
		return NULL;
	}

	add_to_scope(om);

	return(om);
}

/*
 * Add this AST_Observes node to this scope
 */
AST_Observes *AST_Service::fe_add_observes(AST_Observes *om)
{
	AST_Decl	*d;

	/*
	 * Already defined?
	 */
	if ((d = check_for_add(om)) != NULL)
	{
		if (d->node_type() != NT_observes)
		{
			add_to_scope(om);
			return(om); 
		} else
		{
			AST_Observes *sM = AST_Observes::narrow_from_decl(d);

			if (sM && sM->getInterface() != om->getInterface())
			{
				add_to_scope(om);
				return(om); 
			}
		}
	
		IDL_Global::idl_global()->err()
			->error2(UTL_Error::EIDL_OBSERVES_EXISTS, om, this);
		return NULL;
	}

	add_to_scope(om);

	return(om);
}

/*
 * Add this AST_Needs node to this scope
 */
AST_Needs *AST_Service::fe_add_needs(AST_Needs *om)
{
	AST_Decl	*d;

	/*
	 * Already defined?
	 */
	if ((d = check_for_add(om)) != NULL)
	{
		if (d->node_type() != NT_needs)
		{
			add_to_scope(om);
			return(om); 
		} else
		{
			AST_Needs *sM = AST_Needs::narrow_from_decl(d);

			if (sM && sM->getService() != om->getService())
			{
				add_to_scope(om);
				return(om); 
			}
		}
	
		IDL_Global::idl_global()->err()
			->error2(UTL_Error::EIDL_NEEDS_EXISTS, om, this);
		return NULL;
	}

	add_to_scope(om);

	return(om);
}

/*
 * Add an AST_Attribute node (an attribute declaration) to this scope
 */
AST_Attribute *AST_Service::fe_add_attribute(AST_Attribute *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * check  
 */
AST_Decl* AST_Service::check_for_add(AST_Decl *dm)
{
	UTL_ScopeActiveIterator *i = new UTL_ScopeActiveIterator(DeclAsScope(this),
																UTL_Scope::IK_both);
	AST_Decl  *ret=NULL;
	AST_Decl  *d=NULL;
	
	/*
	 * Iterate over this scope
	 */
	while (!(i->is_done())) 
	{
		d = i->item();

		if (d->local_name() != NULL && 
			*(d->local_name()) == *(dm->local_name())) 
		{
			delete i;
			
			return(d);

		}

		i->next();
	}

	/*
	 * OK, not found
	 */
	delete i;				
	return ret;
}

/*
 * Dump this AST_Service node to the ostream o
 */
void AST_Service::dump(ostream &o)
{
	o << "class " << local_name() << " {\n";
	UTL_Scope::dump(o);
	IDL_Global::idl_global()->indent()->skip_to(o);
	o << "}";
}

/*
 * Narrowing methods
 */
IMPL_NARROW_METHODS2(AST_Service, AST_Decl, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Service)
IMPL_NARROW_FROM_SCOPE(AST_Service)
