/*************************************************************************
 *
 *  $RCSfile: aststruc.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s) and destructor
 */
AST_Structure::AST_Structure()
	:	m_n_inherits(0),
		m_inherits(NULL)
{
}

AST_Structure::AST_Structure(UTL_ScopedName *n, AST_Structure **ih,
							 long nih, UTL_StrList *p)
	:	m_n_inherits(nih),
		m_inherits(ih),
		AST_Decl(AST_Decl::NT_struct, n, p),
		UTL_Scope(AST_Decl::NT_struct)
{
}

AST_Structure::AST_Structure(AST_Decl::NodeType nt,
			     UTL_ScopedName *n,	AST_Structure **ih,
				 long nih, UTL_StrList *p)
	:	m_n_inherits(nih),
		m_inherits(ih),
		AST_Decl(nt, n, p),
		UTL_Scope(nt)
{
}

/*
 * Private operations
 */

/*
 * Public operations
 */

/*
 * Redefinition of inherited virtual operations
 */

/*
 * Add this AST_Field node (a field declaration) to this scope
 */
AST_Field *AST_Structure::fe_add_field(AST_Field *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Structure node (a manifest struct type) to this scope
 */
AST_Structure *AST_Structure::fe_add_structure(AST_Structure *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to local types
	 */
	add_to_local_types(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add an AST_Union node (a manifest union type) to this scope
 */
AST_Union *AST_Structure::fe_add_union(AST_Union *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to local types
	 */
	add_to_local_types(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_Enum node (a manifest enum declaration) to this scope
 */
AST_Enum *AST_Structure::fe_add_enum(AST_Enum *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to local types
	 */
	add_to_local_types(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Add this AST_EnumVal node (an enumerator declaration) to this scope.
 * This is done to conform to the C++ scoping rules which declare
 * enumerators in the enclosing scope (in addition to declaring them
 * in the enum itself)
 */
AST_EnumVal *AST_Structure::fe_add_enum_val(AST_EnumVal *t)
{
	AST_Decl *d;

	/*
	 * Already defined and cannot be redefined? Or already used?
	 */
	if ((d = lookup_for_add(t, sal_False)) != NULL) 
	{
		if (!can_be_redefined(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_REDEF, t, this, d);
			return NULL;
		}
		if (referenced(d)) 
		{
			IDL_Global::idl_global()->err()->error3(UTL_Error::EIDL_DEF_USE, t, this, d);
			return NULL;
		}
		if (t->has_ancestor(d)) 
		{
			IDL_Global::idl_global()->err()->redefinition_in_scope(t, d);
			return NULL;
		}
	}
	/*
	 * Add it to scope
	 */
	add_to_scope(t);
	/*
	 * Add it to set of locally referenced symbols
	 */
	add_to_referenced(t, sal_False);

	return t;
}

/*
 * Dump this AST_Structure node to the ostream o
 */
void AST_Structure::dump(ostream &o)
{
	o << "struct ";
	AST_Decl::dump(o);
	o << " {\n";
	UTL_Scope::dump(o);
	IDL_Global::idl_global()->indent()->skip_to(o);
	o << "}";
}

// Narrowing
IMPL_NARROW_METHODS2(AST_Structure, AST_ConcreteType, UTL_Scope)
IMPL_NARROW_FROM_DECL(AST_Structure)
IMPL_NARROW_FROM_SCOPE(AST_Structure)
