/*************************************************************************
 *
 *  $RCSfile: feheads.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif
#ifndef _FEHEADS_HXX_
#include <feheads.hxx>
#endif

//*******************************************************************
//  FE_InterfaceHead 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */
FE_InterfaceHeader::FE_InterfaceHeader(UTL_ScopedName *n, UTL_NameList *nl,
									   FE_InterfaceHead* h)
	:	m_pInterfaceName(n),
		m_pInterface_head(h),
		m_pInherits(NULL),
		m_nInherits(0)
{
	compile_inheritance(nl);
}

FE_InterfaceHeader::~FE_InterfaceHeader()
{
	if(m_pInterface_head != NULL) delete(m_pInterface_head);
}

// Compute flattened, non-repeating list of inherited interfaces

#undef	INCREMENT
#define	INCREMENT	512

/*
 * Private storage used to store interfaces seen already in the
 * computation of the unique, flattened inheritance list
 */
static AST_Interface **iseen = NULL;
static long	    iallocated = 0;
static long	    iused = 0;

/*
 * Add an interface to an inheritance spec
 */
static void add_inheritance(AST_Interface *i)
{
	long			oiallocated;
	long			k;
	AST_Interface	      **oiseen;

	/*
	 * Make sure there's space for one more
	 */
	if (iallocated == iused) 
	{
		if (iallocated == 0) 
		{
			iallocated = INCREMENT;
			iseen = new AST_Interface *[iallocated];
		} else 
		{
			oiallocated = iallocated;
			oiseen = iseen;
			iallocated += INCREMENT;
			iseen = new AST_Interface *[iallocated];
			for (k = 0; k < iused; k++)
			iseen[k] = oiseen[k];
			delete oiseen;
		}
	}
	/*
	 * OK, now insert it
	 */
	iseen[iused] = i;
	iused++;
}

/*
 * Have we already seen this interface?
 */
static long already_seen(AST_Interface *ip)
{
	long	i;

	for (i = 0; i < iused; i++) 
	{
		if (iseen[i] == ip)
			return sal_True;
	}
	return sal_False;
}

/*
 * Add this interface to the list of inherited if not already there
 */
void FE_InterfaceHeader::compile_one_inheritance(AST_Interface *i)
{
	/*
	 * Check for badly formed interface
	 */
	if (i == NULL)
		return;
	/*
	 * If we've seen it already then don't expand again
	 */
	if (already_seen(i))
		return;
	/*
	 * OK, add i to the list of inherited interfaces
	 */
	add_inheritance(i);
}

/*
 * Compute the list of top-level interfaces this one inherits from
 */
void FE_InterfaceHeader::compile_inheritance(UTL_NameList *nl)
{
	UTL_NamelistActiveIterator *l = new UTL_NamelistActiveIterator(nl);
	AST_Decl		       *d;
	AST_Interface		       *i;
	long			        k;

	iused = 0;
	/*
	 * Compute expanded flattened non-repeating list of interfaces
	 * which this one inherits from
	 */
	while (!(l->is_done())) 
	{
		/*
		 * Check that scope stack is valid
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL) 
		{
			IDL_Global::idl_global()->err()->lookup_error(l->item());
			return;
		}
		/*
		 * Look it up
		 */
		d = IDL_Global::idl_global()->scopes()->top()->lookup_by_name(l->item(), sal_True);
		/*
		 * Not found?
		 */
		if (d == NULL) 
		{
			IDL_Global::idl_global()->err()->lookup_error(l->item());
			return;
		}
		/*
		 * Not an interface?
		 */
		while(d->node_type() == AST_Decl::NT_typedef) 
			d = AST_Typedef::narrow_from_decl(d)->base_type();
		
		if (d->node_type() != AST_Decl::NT_interface) 
		{
			if (d->node_type() == AST_Decl::NT_interface_fwd) 
			{
				AST_InterfaceFwd *fwd = AST_InterfaceFwd::narrow_from_decl(d);
				IDL_Global::idl_global()->err()->inheritance_fwd_error(m_pInterfaceName, fwd->full_definition());
				return;
			} else
			{
				IDL_Global::idl_global()->err()->inheritance_error(m_pInterfaceName, d);
				return;
			}
		}

		/*
		 * OK, cast to an interface
		 */
		i = AST_Interface::narrow_from_decl(d);
		if (i == NULL)
			IDL_Global::idl_global()->err()->inheritance_error(m_pInterfaceName, d);
		/*
		 * Forward declared interface?
		 */
		if (!i->is_defined()) 
		{
			IDL_Global::idl_global()->err()->inheritance_fwd_error(m_pInterfaceName, i);
			return;
		}
		/*
		 * OK, see if we have to add this to the list of interfaces
		 * inherited from
		 */
		compile_one_inheritance(i);

		/*
		 * Next element in header list
		 */
		l->next();
	}
	delete l;
	/*
	 * OK, install in interface header
	 */
    if (iused > 0)
    {
        m_pInherits = new AST_Interface *[iused];
        for (k = 0; k < iused; k++)
            m_pInherits[k] = iseen[k];
        m_nInherits = iused;
    } else
    {
        m_pInherits = NULL;
        m_nInherits = iused;
    }
}


//*******************************************************************
//  FE_ExceptionHeader 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */
FE_ExceptionHeader::FE_ExceptionHeader(UTL_ScopedName *n, UTL_NameList *nl)
	:	m_pExceptionName(n),
		m_pInherits(NULL),
		m_nInherits(0)
{
	compile_inheritance(nl);
}

FE_ExceptionHeader::~FE_ExceptionHeader()
{
}

/*
 * Private storage used to store exception seen already in the
 * computation of the unique, flattened inheritance list
 */
static AST_Exception **eseen = NULL;
static long	    eallocated = 0;
static long	    eused = 0;

/*
 * Add an exception to an inheritance spec
 */
static void add_exception(AST_Exception *e)
{
	long			oeallocated;
	long			k;
	AST_Exception	**oeseen;

	/*
	 * Make sure there's space for one more
	 */
	if (eallocated == eused) 
	{
		if (eallocated == 0) 
		{
			eallocated = INCREMENT;
			eseen = new AST_Exception *[eallocated];
		} else 
		{
			oeallocated = eallocated;
			oeseen = eseen;
			eallocated += INCREMENT;
			eseen = new AST_Exception *[eallocated];
			for (k = 0; k < eused; k++)
			eseen[k] = oeseen[k];
			delete oeseen;
		}
	}
	/*
	 * OK, now insert it
	 */
	eseen[eused] = e;
	eused++;
}

/*
 * Have we already seen this exception?
 */
static long already_seen_exception(AST_Exception *ep)
{
	long	i;

	for (i = 0; i < eused; i++) 
	{
		if (eseen[i] == ep)
			return sal_True;
	}
	return sal_False;
}

/*
 * Add this exception to the list of inherited if not already there
 */
void FE_ExceptionHeader::compile_one_inheritance(AST_Exception *e)
{
	/*
	 * Check for badly formed exception
	 */
	if (e == NULL)
		return;
	/*
	 * If we've seen it already then don't expand again
	 */
	if (already_seen_exception(e))
		return;
	/*
	 * OK, add i to the list of inherited exception
	 */
	add_exception(e);
}

/*
 * Compute the list of top-level exceptions this one inherits from
 */
void FE_ExceptionHeader::compile_inheritance(UTL_NameList *nl)
{
	UTL_NamelistActiveIterator *l = new UTL_NamelistActiveIterator(nl);
	AST_Decl		       *d;
	AST_Exception	       *e;
	long			        k;

	eused = 0;
	/*
	 * Compute expanded flattened non-repeating list of exceptions
	 * which this one inherits from
	 */
	while (!(l->is_done())) 
	{
		/*
		 * Check that scope stack is valid
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL) 
		{
			IDL_Global::idl_global()->err()->lookup_error(l->item());
			return;
		}
		/*
		 * Look it up
		 */
		d = IDL_Global::idl_global()->scopes()->top()->lookup_by_name(l->item(), sal_True);
		/*
		 * Not found?
		 */
		if (d == NULL) 
		{
			IDL_Global::idl_global()->err()->lookup_error(l->item());
			return;
		}
		/*
		 * Not an exception?
		 */
		while(d->node_type() == AST_Decl::NT_typedef) 
			d = AST_Typedef::narrow_from_decl(d)->base_type();
		
		if (d->node_type() != AST_Decl::NT_except) 
		{
			IDL_Global::idl_global()->err()->inheritance_error(m_pExceptionName, d);
			return;
		}
		/*
		 * OK, cast to an exception
		 */
		e = AST_Exception::narrow_from_decl(d);
		if (e == NULL)
			IDL_Global::idl_global()->err()->inheritance_error(m_pExceptionName, d);

		/*
		 * OK, see if we have to add this to the list of exceptions
		 * inherited from
		 */
		compile_one_inheritance(e);

		/*
		 * Next element in header list
		 */
		l->next();
	}
	delete l;
	/*
	 * OK, install in exception header
	 */
    if (eused > 0)
    {
        m_pInherits = new AST_Exception *[eused];
        for (k = 0; k < eused; k++)
            m_pInherits[k] = eseen[k];
        m_nInherits = eused;
    } else
    {
        m_pInherits = NULL;
        m_nInherits = eused;
    }
}

//*******************************************************************
//  FE_StructHeader 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */
FE_StructHeader::FE_StructHeader(UTL_ScopedName *n, UTL_NameList *nl)
	:	m_pStructName(n),
		m_pInherits(NULL),
		m_nInherits(0)
{
	compile_inheritance(nl);
}

FE_StructHeader::~FE_StructHeader()
{
}

/*
 * Private storage used to store structure seen already in the
 * computation of the unique, flattened inheritance list
 */
static AST_Structure **stseen = NULL;
static long	    stallocated = 0;
static long	    stused = 0;

/*
 * Add an structure to an inheritance spec
 */
static void add_structure(AST_Structure *st)
{
	long			ostallocated;
	long			k;
	AST_Structure	**ostseen;

	/*
	 * Make sure there's space for one more
	 */
	if (stallocated == stused) 
	{
		if (stallocated == 0) 
		{
			stallocated = INCREMENT;
			stseen = new AST_Structure *[stallocated];
		} else 
		{
			ostallocated = stallocated;
			ostseen = stseen;
			stallocated += INCREMENT;
			stseen = new AST_Structure *[stallocated];
			for (k = 0; k < stused; k++)
			stseen[k] = ostseen[k];
			delete ostseen;
		}
	}
	/*
	 * OK, now insert it
	 */
	stseen[stused] = st;
	stused++;
}

/*
 * Have we already seen this structure?
 */
static long already_seen_structure(AST_Structure *st)
{
	long	i;

	for (i = 0; i < stused; i++) 
	{
		if (stseen[i] == st)
			return sal_True;
	}
	return sal_False;
}

/*
 * Add this structure to the list of inherited if not already there
 */
void FE_StructHeader::compile_one_inheritance(AST_Structure *st)
{
	/*
	 * Check for badly formed structure
	 */
	if (st == NULL)
		return;
	/*
	 * If we've seen it already then don't expand again
	 */
	if (already_seen_structure(st))
		return;
	/*
	 * OK, add i to the list of inherited structure
	 */
	add_structure(st);
}

/*
 * Compute the list of top-level exceptions this one inherits from
 */
void FE_StructHeader::compile_inheritance(UTL_NameList *nl)
{
	UTL_NamelistActiveIterator *l = new UTL_NamelistActiveIterator(nl);
	AST_Decl		       *d;
	AST_Structure	       *st;
	long			        k;

	stused = 0;
	/*
	 * Compute expanded flattened non-repeating list of structures
	 * which this one inherits from
	 */
	while (!(l->is_done())) 
	{
		/*
		 * Check that scope stack is valid
		 */
		if (IDL_Global::idl_global()->scopes()->top() == NULL) 
		{
			IDL_Global::idl_global()->err()->lookup_error(l->item());
			return;
		}
		/*
		 * Look it up
		 */
		d = IDL_Global::idl_global()->scopes()->top()->lookup_by_name(l->item(), sal_True);
		/*
		 * Not found?
		 */
		if (d == NULL) 
		{
			IDL_Global::idl_global()->err()->lookup_error(l->item());
			return;
		}
		/*
		 * Not an interface?
		 */
		while(d->node_type() == AST_Decl::NT_typedef) 
			d = AST_Typedef::narrow_from_decl(d)->base_type();
		
		if (d->node_type() != AST_Decl::NT_struct) 
		{
			IDL_Global::idl_global()->err()->inheritance_error(m_pStructName, d);
			return;
		}
		/*
		 * OK, cast to an structure
		 */
		st = AST_Structure::narrow_from_decl(d);
		if (st == NULL)
			IDL_Global::idl_global()->err()->inheritance_error(m_pStructName, d);

		/*
		 * OK, see if we have to add this to the list of strucures
		 * inherited from
		 */
		compile_one_inheritance(st);

		/*
		 * Next element in header list
		 */
		l->next();
	}
	delete l;
	/*
	 * OK, install in structure header
	 */
    if (stused > 0)
    {
        m_pInherits = new AST_Structure *[stused];
        for (k = 0; k < stused; k++)
            m_pInherits[k] = stseen[k];
        m_nInherits = stused;
    } else
    {
        m_pInherits = NULL;
        m_nInherits = stused;
    }
}

//*******************************************************************
//  FE_InterfaceHead 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */

FE_InterfaceHead::FE_InterfaceHead(sal_Bool vrt, char* uik, char* ident, double version)
	:	m_virtual(vrt),
		m_pUik(uik),
		m_pIdent(ident),
		m_version(version)

{
}

FE_InterfaceHead::~FE_InterfaceHead()
{
	if(m_pUik != NULL) delete(m_pUik);
	if(m_pIdent != NULL) delete(m_pIdent);
}


//*******************************************************************
//  FE_Ident 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */

FE_Ident::FE_Ident(char* ident, double version)
	:	m_pIdent(ident),
		m_version(version)

{
}

FE_Ident::~FE_Ident()
{
	if(m_pIdent != NULL) delete(m_pIdent);
}


//*******************************************************************
//  FE_OperationHead 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */
FE_OperationHead::FE_OperationHead()
	:	m_attribute(AST_Operation::OP_noflags),
		m_isConst(sal_False),
		m_isLocal(sal_False),
		m_local(""),
		m_implementation(IFLAG_noflag)
{	
}	

FE_OperationHead::FE_OperationHead(AST_Operation::Flags fl, char* local, sal_Bool co)
	:	m_attribute(fl),
		m_isLocal(sal_False),
		m_isConst(co)
{
	if(local != NULL)
	{
		m_local = String(local);
		m_implementation = IFLAG_local;
	} else
	{
		m_local = String("");
		m_implementation = IFLAG_noflag;
	}
}


//*******************************************************************
//  FE_AttributeHead 
//*******************************************************************

/*
 * Constructor(s) and destructor
 */
FE_AttributeHead::FE_AttributeHead()
	:	m_readonly(sal_False),
		m_optional(sal_False),
		m_isLocal(sal_False),
		m_local(""),
		m_implementation(IFLAG_noflag),
		m_flags(0)
{	
}	

FE_AttributeHead::FE_AttributeHead(sal_Bool ro, char* local)
	:	m_readonly(ro),
		m_optional(sal_False),
		m_isLocal(sal_False),
		m_flags(0)
{
	if(local != NULL)
	{
		m_local = String(local);
		m_implementation = IFLAG_local;
	} else
	{
		m_local = String("");
		m_implementation = IFLAG_noflag;
	}		
}
