/*************************************************************************
 *
 *  $RCSfile: utlerror.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

#ifndef _FEHEADS_HXX_
#include <feheads.hxx>
#endif

/*
 * Helper functions to do:
 *
 * - Convert an error code to an error string
 * - Print out an error message header
 * - Convert an ExprType to a string
 */

// Convert an error code into a char *
static char *error_string(UTL_Error::ErrorCode c)
{
	switch (c) 
	{
		case UTL_Error::EIDL_OK:
			return GTDEVEL("all is fine ");
		case UTL_Error::EIDL_SYNTAX_ERROR:
			return "";
		case UTL_Error::EIDL_REDEF:
			return GTDEVEL("illegal redefinition ");
		case UTL_Error::EIDL_DEF_USE:
			return GTDEVEL("redefinition after use, ");
		case UTL_Error::EIDL_MULTIPLE_BRANCH:
			return GTDEVEL("union with duplicate branch label ");
		case UTL_Error::EIDL_COERCION_FAILURE:
			return GTDEVEL("coercion failure ");
		case UTL_Error::EIDL_SCOPE_CONFLICT:
			return GTDEVEL("definition scope is different than fwd declare scope, ");
		case UTL_Error::EIDL_ONEWAY_CONFLICT:
			return GTDEVEL("oneway operation with OUT or INOUT parameters or raises exceptions, ");
		case UTL_Error::EIDL_DISC_TYPE:
			return GTDEVEL("union with illegal discriminator type, ");
		case UTL_Error::EIDL_LABEL_TYPE:
			return GTDEVEL("label type incompatible with union discriminator type, ");
		case UTL_Error::EIDL_ILLEGAL_ADD:
			return GTDEVEL("illegal add operation, ");
		case UTL_Error::EIDL_ILLEGAL_USE:
			return GTDEVEL("illegal type used in expression, ");
		case UTL_Error::EIDL_ILLEGAL_RAISES:
			return GTDEVEL("error in raises(..) clause, ");
		case UTL_Error::EIDL_ILLEGAL_CONTEXT:
			return GTDEVEL("error in context(..) clause, ");
		case UTL_Error::EIDL_CANT_INHERIT:
			return GTDEVEL("cannot inherit from ");
		case UTL_Error::EIDL_LOOKUP_ERROR:
			return GTDEVEL("error in lookup of symbol: ");
		case UTL_Error::EIDL_INHERIT_FWD_ERROR:
			/* More intelligible message printed by error routine */
			return "";
		case UTL_Error::EIDL_CONSTANT_EXPECTED:
			return GTDEVEL("constant expected: ");
		case UTL_Error::EIDL_EVAL_ERROR:
			return GTDEVEL("expression evaluation error: ");
		case UTL_Error::EIDL_NAME_CASE_ERROR:
			return GTDEVEL("identifier used with two differing spellings: ");
		case UTL_Error::EIDL_ENUM_VAL_EXPECTED:
			return GTDEVEL("enumerator expected: ");
		case UTL_Error::EIDL_ENUM_VAL_NOT_FOUND:
			return GTDEVEL("enumerator by this name not defined: ");
		case UTL_Error::EIDL_AMBIGUOUS:
			return GTDEVEL("ambiguous definition: ");
		case UTL_Error::EIDL_DECL_NOT_DEFINED:
			return GTDEVEL("forward declared but never defined: ");
		case UTL_Error::EIDL_FWD_DECL_LOOKUP:
			/* More intelligible message printed by error routine */
			return "";
		case UTL_Error::EIDL_RECURSIVE_TYPE:
			return GTDEVEL("illegal recursive use of type: ");
		case UTL_Error::EIDL_NONVOID_ONEWAY:
			return GTDEVEL("non-void return type in oneway operation: ");
		case UTL_Error::EIDL_REDEF_SCOPE:
			return GTDEVEL("redefinition inside defining scope: ");
		case UTL_Error::EIDL_NOT_A_TYPE:
			return GTDEVEL("specified symbol is not a type: ");
		case UTL_Error::EIDL_TYPE_NOT_VALID:
			return GTDEVEL("specified type is not valid in this context: implementation");
		case UTL_Error::EIDL_BACK_END:
			return GTDEVEL("back end: ");
		case UTL_Error::EIDL_BADPARA_SIZEIS:
			return GTDEVEL("bad argument in index specifier size_is(): ");
		case UTL_Error::EIDL_BADPARA_LENGTHIS:
			return GTDEVEL("bad argument in index specifier length_is(): ");
		case UTL_Error::EIDL_TYPEDEF_ISPOINTER:
			return GTDEVEL("pointer declarator in typedef not allowed: ");
		case UTL_Error::EIDL_INTERPARAM_ISPOINTER:
			return GTDEVEL("interface* in parameter declaration not allowed: ");
		case UTL_Error::EIDL_OPRETURN_ISPOINTER:
			return GTDEVEL("operation pointer return type not allowed: ");
		case UTL_Error::EIDL_INTERFACE_LOOKUP:
			return GTDEVEL("interface is not defined and no forward exists: ");
		case UTL_Error::EIDL_SERVICE_EXISTS:
			return GTDEVEL("interface can not be insert in a class twice: ");
		case UTL_Error::EIDL_MULTIBLE_INHERITANCE:
			return GTDEVEL("multible inheritance is not allowed, in inheritance tree: ");
		case UTL_Error::EIDL_MAPPING_MISSED:
			return GTDEVEL("language sections can only be defined in a mapping section: ");
		case UTL_Error::EIDL_MAPPING_EXIST:
			return GTDEVEL("defined mapping already exist, remapping not allowed: ");
		case UTL_Error::EIDL_INTERFACE_EXP:
			return GTDEVEL("specified interface from the exported service are not implemented in: ");
		case UTL_Error::EIDL_ATTRIBUTE_EXP:
			return GTDEVEL("specified attribute from the exported service are not implemented in: ");
		case UTL_Error::EIDL_TYPE_IDENT_CONFLICT:
			return GTDEVEL("type and parameter/member name are equal, it's bad: ");
		case UTL_Error::EIDL_ONEWAY_RAISE_CONFLICT:
			return GTDEVEL("oneway operation cannot raises exceptions: ");
		case UTL_Error::EIDL_EXCLUSIVATTRIBUTEFLAG:
			return GTDEVEL("invalid flag (only 'optional' is valid): ");
		case UTL_Error::EIDL_WRONGATTRIBUTEFLAG:
			return GTDEVEL("the used flag is not valid in this context: ");
		case UTL_Error::EIDL_MISSINGATTRIBUTEKEYWORD:
			return GTDEVEL("missing keyword: ");
		case UTL_Error::EIDL_WRONGATTRIBUTEKEYWORD:
			return GTDEVEL("keyword not allowed: ");
	}

	return ("unkown error");
}

static char *warning_string(UTL_Error::WarningCode c)
{
	switch (c) 
	{
		case UTL_Error::WIDL_EXPID_CONFLICT:
			return GTDEVEL("exception id conflict: ");
		case UTL_Error::WIDL_REQID_CONFLICT:
			return GTDEVEL("request id conflict: ");
		case UTL_Error::WIDL_INHERIT_IDCONFLICT:
			return GTDEVEL("request id conflict in inheritance tree: ");
		case UTL_Error::WIDL_TYPE_IDENT_CONFLICT:
			return GTDEVEL("type and parameter/member name are equal, it's bad: ");
	}

	return ("unknown warning");
}


// Print out an error message header on cerr
static void idl_error_header(UTL_Error::ErrorCode c, long lineno, String *s)
{
	fprintf(stderr, "%s(%d) : %s", 
		   (IDL_Global::idl_global()->read_from_stdin() ? "standard input" : (const char*)*s),
		   ((lineno == -1) ? IDL_Global::idl_global()->lineno() : lineno),
		   error_string(c));
}

// Print out an error message header on cerr
static void idl_warning_header(UTL_Error::WarningCode c, long lineno, String *s)
{
	fprintf(stderr, "%s(%d) : WARNING, %s",
		   (IDL_Global::idl_global()->read_from_stdin() ? "standard input" : (const char*)*s),
		   ((lineno == -1) ? IDL_Global::idl_global()->lineno() : lineno),
		   warning_string(c));
}

// Convert the type of an AST_Expression to a char *
static char *exprtype_to_string(AST_Expression::ExprType t)
{
	switch (t) 
	{
		case AST_Expression::EV_short:
			return "short";
		case AST_Expression::EV_ushort:
			return "unsigned short";
		case AST_Expression::EV_long:
			return "long";
		case AST_Expression::EV_ulong:
			return "unsigned long";
		case AST_Expression::EV_hyper:
			return "long";
		case AST_Expression::EV_uhyper:
			return "unsigned long";
		case AST_Expression::EV_float:
			return "float";
		case AST_Expression::EV_double:
			return "double";
		case AST_Expression::EV_char:
			return "char";
		case AST_Expression::EV_octet:
			return "octet";
		case AST_Expression::EV_byte:
			return "byte";
		case AST_Expression::EV_bool:
			return "boolean";
		case AST_Expression::EV_string:
			return "string";
		case AST_Expression::EV_any:
			return "any";
		case AST_Expression::EV_void:
			return "void";
		case AST_Expression::EV_none:
			return "none";
	}

	return ("unkown");
}

/*
 * Convert a parse state into a possible error message
 */
static char *parse_state_to_error_message(IDL_Global::ParseState ps)
{
	switch (ps) 
	{
		case IDL_Global::PS_NoState:
			return GTDEVEL("Statement can not be parsed");
		
		case IDL_Global::PS_TypeDeclSeen:
			return GTDEVEL("Malformed typedef declaration");
		case IDL_Global::PS_ConstDeclSeen:
			return GTDEVEL("Malformed const declaration");
		case IDL_Global::PS_ExceptDeclSeen:
			return GTDEVEL("Malformed exception declaration");
		case IDL_Global::PS_InterfaceDeclSeen:
			return GTDEVEL("Malformed interface declaration");
		case IDL_Global::PS_ModuleDeclSeen:
			return GTDEVEL("Malformed module declaration");
		case IDL_Global::PS_AttrDeclSeen:
			return GTDEVEL("Malformed attribute declaration");
		case IDL_Global::PS_OpDeclSeen:
			return GTDEVEL("Malformed operation declaration");
		
		// Mapping
		//
		case IDL_Global::PS_MappingSeen:
			return GTDEVEL("Missing '{' or illegal syntax following MAPPING keyword");
			break;
		case IDL_Global::PS_MappingDeclSeen:
			return GTDEVEL("Malformed mapping declaration");
			break;
		case IDL_Global::PS_MappingSqSeen:
			return GTDEVEL("Illegal syntax following mapping '{' opener");
			break;
		case IDL_Global::PS_MappingQsSeen:
			return GTDEVEL("Illegal syntax following mapping '}' closer");
			break;
		case IDL_Global::PS_MappingBodySeen:
			return GTDEVEL("Illegal syntax following mapping body statement(s)");
			break;
			
		// Language
		//
		case IDL_Global::PS_LanguageSeen:
			return GTDEVEL("Missing language identifier following LANGUAGE keyword");
			break;
		case IDL_Global::PS_LanguageIdSeen:
			return GTDEVEL("Missing '{' or illegal syntax following language identifier");
			break;
		case IDL_Global::PS_LanguageDeclSeen:
			return GTDEVEL("Malformed language declaration");
			break;
		case IDL_Global::PS_LanguageSqSeen:
			return GTDEVEL("Illegal syntax following language '{' opener");
			break;
		case IDL_Global::PS_LanguageQsSeen:
			return GTDEVEL("Illegal syntax following language '}' closer");
			break;
		case IDL_Global::PS_LanguageBodySeen:
			return GTDEVEL("Illegal syntax following language body statement(s)");
			break;
		
		// Map
		//
		case IDL_Global::PS_MapSeen:
			return GTDEVEL("Missing map identifier following MAP keyword");
			break;
		case IDL_Global::PS_MapIdSeen:
			return GTDEVEL("Missing '[' or illegal syntax following map identifier");
			break;
		case IDL_Global::PS_MapDeclSeen:
			return GTDEVEL("Malformed map declaration");
			break;
		case IDL_Global::PS_MapSqSeen:
			return GTDEVEL("Illegal syntax following map '[' opener");
			break;
		case IDL_Global::PS_MapQsSeen:
			return GTDEVEL("Illegal syntax following map ']' closer");
			break;
		case IDL_Global::PS_MapBodySeen:
			return GTDEVEL("Illegal syntax following map body statement(s)");
			break;
			
		case IDL_Global::PS_MapNameSeen:
			return GTDEVEL("Missing identifier following NAME keyword");
			break;
		case IDL_Global::PS_MapNameIdSeen:
			return GTDEVEL("Illegal syntax following name statement");
			break;
		case IDL_Global::PS_MapSuperSeen:
			return GTDEVEL("Missing identifier following CLASS keyword");
			break;
		case IDL_Global::PS_MapSuperIdSeen:
			return GTDEVEL("Illegal syntax following class statement");
			break;
		case IDL_Global::PS_MapIncSeen:
			return GTDEVEL("Missing identifier following INCLUDE keyword");
			break;
		case IDL_Global::PS_MapIncIdSeen:
			return GTDEVEL("Illegal syntax following include statement");
			break;
		case IDL_Global::PS_GetReflSeen:
			return GTDEVEL("Missing identifier following GETREFLECTION keyword");
			break;
		case IDL_Global::PS_GetReflIdSeen:
			return GTDEVEL("Illegal syntax following getreflection statement");
			break;
		case IDL_Global::PS_IncReflSeen:
			return GTDEVEL("Missing identifier following INCLUDEREFLECTION keyword");
			break;
		case IDL_Global::PS_IncReflIdSeen:
			return GTDEVEL("Illegal syntax following includereflection statement");
			break;
		case IDL_Global::PS_MapIdentSeen:
			return GTDEVEL("Missing identifier following IDENTIFIER keyword");
			break;
		case IDL_Global::PS_MapIdentIdSeen:
			return GTDEVEL("Illegal syntax following identifier statement");
			break;
		case IDL_Global::PS_CreateAllListenerSeen:
			return GTDEVEL("Missing identifier following CREATEALLLISTENER keyword");
			break;
		case IDL_Global::PS_CreateAllListenerIdSeen:
			return GTDEVEL("Illegal syntax following createAllListener statement");
			break;
		
		// Module
		case IDL_Global::PS_ModuleSeen:
			return GTDEVEL("Missing module identifier following MODULE keyword");
		case IDL_Global::PS_ModuleIDSeen:
			return GTDEVEL("Missing '{' or illegal syntax following module identifier");
		case IDL_Global::PS_ModuleSqSeen:
			return GTDEVEL("Illegal syntax following module '{' opener");
		case IDL_Global::PS_ModuleQsSeen:
			return GTDEVEL("Illegal syntax following module '}' closer");
		case IDL_Global::PS_ModuleBodySeen:
			return GTDEVEL("Illegal syntax following module body statement(s)");

		// Object
		case IDL_Global::PS_ObjectSeen:
			return GTDEVEL("Missing module identifier following OBJECT keyword");
		case IDL_Global::PS_ObjectIDSeen:
			return GTDEVEL("Missing '{' or illegal syntax following object identifier");
		case IDL_Global::PS_ObjectSqSeen:
			return GTDEVEL("Illegal syntax following object '{' opener");
		case IDL_Global::PS_ObjectQsSeen:
			return GTDEVEL("Illegal syntax following object '}' closer");
		case IDL_Global::PS_ObjectBodySeen:
			return GTDEVEL("Illegal syntax following object body statement(s)");
		case IDL_Global::PS_ObjectMemberSeen:
			return GTDEVEL("Illegal syntax following object member declaration");
		case IDL_Global::PS_ObjectHeadSeen:
			return GTDEVEL("Illegal syntax following object head");

		// Service
		case IDL_Global::PS_ServiceSeen:
			return GTDEVEL("Missing module identifier following SERVICE keyword");
		case IDL_Global::PS_ServiceIDSeen:
			return GTDEVEL("Missing '{' or illegal syntax following service identifier");
		case IDL_Global::PS_ServiceSqSeen:
			return GTDEVEL("Illegal syntax following service '{' opener");
		case IDL_Global::PS_ServiceQsSeen:
			return GTDEVEL("Illegal syntax following service '}' closer");
		case IDL_Global::PS_ServiceBodySeen:
			return GTDEVEL("Illegal syntax following service body statement(s)");
		case IDL_Global::PS_ServiceMemberSeen:
			return GTDEVEL("Illegal syntax following service member declaration");
		
		// Interface
		//
		case IDL_Global::PS_InterfaceSeen:
			return GTDEVEL("Missing interface identifier following INTERFACE keyword");
		case IDL_Global::PS_InterfaceIDSeen:
			return GTDEVEL("Illegal syntax following interface identifier");
		case IDL_Global::PS_InterfaceHeadSeen:
			return GTDEVEL("Illegal syntax following interface head");
		case IDL_Global::PS_InterfaceHeadSqSeen:
			return GTDEVEL("Illegal syntax following interface head '[' opener");
		case IDL_Global::PS_InterfaceHeadQsSeen:
			return GTDEVEL("Illegal syntax following interface head ']' closer");
		case IDL_Global::PS_IHeadUikSeen:
			return GTDEVEL("Illegal syntax following uik in interface head");
		case IDL_Global::PS_IHeadVirtualSeen:
			return GTDEVEL("Illegal syntax following virtual in interface head");
		case IDL_Global::PS_IHeadIdentSeen:
			return GTDEVEL("Illegal syntax following ident in interface head");
		case IDL_Global::PS_InheritSpecSeen:
			return GTDEVEL("Missing '{' or illegal syntax following inheritance spec");
		case IDL_Global::PS_ForwardDeclSeen:
			return GTDEVEL("Missing ';' following forward interface declaration");
		case IDL_Global::PS_InterfaceSqSeen:
			return GTDEVEL("Illegal syntax following interface '{' opener");
		case IDL_Global::PS_InterfaceQsSeen:
			return GTDEVEL("Illegal syntax following interface '}' closer");
		case IDL_Global::PS_InterfaceBodySeen:
			return GTDEVEL("Illegal syntax following interface body statement(s)");
		case IDL_Global::PS_InheritColonSeen:
			return GTDEVEL("Illegal syntax following ':' starting inheritance list");
		
		// Something
		//
		case IDL_Global::PS_SNListCommaSeen:
			return GTDEVEL("Found illegal scoped name in scoped name list");
		case IDL_Global::PS_ScopedNameSeen:
			return GTDEVEL("Missing ',' following scoped name in scoped name list");
		case IDL_Global::PS_SN_IDSeen:
			return GTDEVEL("Illegal component in scoped name");
		case IDL_Global::PS_ScopeDelimSeen:
			return GTDEVEL("Illegal component in scoped name following '::'");
		case IDL_Global::PS_ConstSeen:
			return GTDEVEL("Missing type or illegal syntax following CONST keyword");
		case IDL_Global::PS_ConstTypeSeen:
			return GTDEVEL("Missing identifier or illegal syntax following const type");
		case IDL_Global::PS_ConstIDSeen:
			return GTDEVEL("Missing '=' or illegal syntax after const identifier");
		case IDL_Global::PS_ConstAssignSeen:
			return GTDEVEL("Missing value expr or illegal syntax following '='");
		case IDL_Global::PS_ConstExprSeen:
			return GTDEVEL("Missing ';' or illegal syntax following value expr in const");
		case IDL_Global::PS_TypedefSeen:
			return GTDEVEL("Missing type or illegal syntax following TYPEDEF keyword");
		case IDL_Global::PS_TypeSpecSeen:
			return GTDEVEL("Missing declarators or illegal syntax following type spec");
		case IDL_Global::PS_DeclaratorsSeen:
			return GTDEVEL("Illegal syntax following declarators in TYPEDEF declaration");
		
		// Struct
		//
		case IDL_Global::PS_StructSeen:
			return GTDEVEL("Missing struct identifier following STRUCT keyword");
		case IDL_Global::PS_StructIDSeen:
			return GTDEVEL("Missing '{' or illegal syntax following struct identifier");
		case IDL_Global::PS_StructSqSeen:
			return GTDEVEL("Illegal syntax following struct '{' opener");
		case IDL_Global::PS_StructQsSeen:
			return GTDEVEL("Illegal syntax following struct '}' closer");
		case IDL_Global::PS_StructBodySeen:
			return GTDEVEL("Illegal syntax following struct body statement(s)");
		
		// Member
		//
		case IDL_Global::PS_MemberTypeSeen:
			return GTDEVEL("Illegal syntax of missing identifier following member type");
		case IDL_Global::PS_MemberDeclsSeen:
			return GTDEVEL("Illegal syntax following member declarator(s)");
		case IDL_Global::PS_MemberDeclsCompleted:
			return GTDEVEL("Missing ',' between member decls of same type(?)");
		case IDL_Global::PS_MemberSizeSeen:
			return GTDEVEL("Illegal syntax of size_is() in member declaration");
		case IDL_Global::PS_MemberLengthSeen:
			return GTDEVEL("Illegal syntax of length_is() in member declaration");
		
		// Union
		//
		case IDL_Global::PS_UnionSeen:
			return GTDEVEL("Missing identifier following UNION keyword");
		case IDL_Global::PS_UnionIDSeen:
			return GTDEVEL("Illegal syntax following union identifier");
		case IDL_Global::PS_SwitchSeen:
			return GTDEVEL("Illegal syntax following SWITCH keyword");
		case IDL_Global::PS_SwitchOpenParSeen:
			return GTDEVEL("Illegal syntax following '(' in switch in union");
		case IDL_Global::PS_SwitchTypeSeen:
			return GTDEVEL("Illegal syntax following type decl in switch in union");
		case IDL_Global::PS_SwitchCloseParSeen:
			return GTDEVEL("Missing union '{' opener");
		case IDL_Global::PS_UnionSqSeen:
			return GTDEVEL("Illegal syntax following union '{' opener");
		case IDL_Global::PS_UnionQsSeen:
			return GTDEVEL("Illegal syntax following union '}' closer");
		case IDL_Global::PS_DefaultSeen:
			return GTDEVEL("Illegal syntax or missing ':' following DEFAULT keyword");
		case IDL_Global::PS_UnionLabelSeen:
			return GTDEVEL("Illegal syntax following branch label in union");
		case IDL_Global::PS_LabelColonSeen:
			return GTDEVEL("Illegal syntax following ':' in branch label in union");
		case IDL_Global::PS_LabelExprSeen:
			return GTDEVEL("Illegal syntax following label expression in union");
		case IDL_Global::PS_UnionElemSeen:
		case IDL_Global::PS_UnionElemCompleted:
			return GTDEVEL("Illegal syntax following union element");
		case IDL_Global::PS_CaseSeen:
			return GTDEVEL("Illegal syntax following CASE keyword in union");
		case IDL_Global::PS_UnionElemTypeSeen:
			return GTDEVEL("Illegal syntax following type decl in union element");
		case IDL_Global::PS_UnionElemDeclSeen:
			return GTDEVEL("Illegal syntax following declarator in union element");
		case IDL_Global::PS_UnionBodySeen:
			return GTDEVEL("Illegal syntax following union body statement(s)");
		case IDL_Global::PS_EnumSeen:
			return GTDEVEL("Illegal syntax or missing identifier following ENUM keyword");
		case IDL_Global::PS_EnumIDSeen:
			return GTDEVEL("Illegal syntax or missing '{' following enum identifier");
		case IDL_Global::PS_EnumSqSeen:
			return GTDEVEL("Illegal syntax following enum '{' opener");
		case IDL_Global::PS_EnumQsSeen:
			return GTDEVEL("Illegal syntax following enum '}' closer");
		case IDL_Global::PS_EnumBodySeen:
			return GTDEVEL("Illegal syntax following enum body statement(s)");
		case IDL_Global::PS_EnumCommaSeen:
			return GTDEVEL("Illegal syntax or missing identifier following ',' in enum");
		
		// Sequence
		//
		case IDL_Global::PS_SequenceSeen:
			return GTDEVEL("Illegal syntax or missing '<' following SEQUENCE keyword");
		case IDL_Global::PS_SequenceSqSeen:
			return GTDEVEL("Illegal syntax or missing type following '<' in sequence");
		case IDL_Global::PS_SequenceQsSeen:
			return GTDEVEL("Illegal syntax following '>' in sequence");
		case IDL_Global::PS_SequenceTypeSeen:
			return GTDEVEL("Illegal syntax following sequence type declaration");
		case IDL_Global::PS_SequenceCommaSeen:
			return GTDEVEL("Illegal syntax of missing size expr following ',' in sequence");
		case IDL_Global::PS_SequenceExprSeen:
			return GTDEVEL("Illegal syntax or missing '>' following size expr in sequence");
		case IDL_Global::PS_StringSeen:
			return GTDEVEL("Illegal syntax or missing '<' following STRING keyword");
		case IDL_Global::PS_StringSqSeen:
			return GTDEVEL("Illegal syntax or missing size expr following '<' in string");
		case IDL_Global::PS_StringQsSeen:
			return GTDEVEL("Illegal syntax following '>' in string");
		case IDL_Global::PS_StringExprSeen:
			return GTDEVEL("Illegal syntax or missing '>' after size expr in string");
		case IDL_Global::PS_StringCompleted:
			return GTDEVEL("Illegal syntax after string declaration");

		// Array
		//
		case IDL_Global::PS_ArrayIDSeen:
			return GTDEVEL("Illegal syntax or missing dimensions after array identifier");
		case IDL_Global::PS_ArrayCompleted:
			return GTDEVEL("Illegal syntax after array declaration");
		case IDL_Global::PS_DimSqSeen:
			return GTDEVEL("Illegal syntax or missing size expr after '[' in array declaration");
		case IDL_Global::PS_DimQsSeen:
			return GTDEVEL("Illegal syntax after ']' in array declaration");
		case IDL_Global::PS_DimExprSeen:
			return GTDEVEL("Illegal syntax or missing ']' after size expr in array declaration");
		
		// Attribut
		//
		case IDL_Global::PS_AttrROSeen:
			return GTDEVEL("Illegal syntax after READONLY keyword");
		case IDL_Global::PS_ImplAttrSeen:
			return GTDEVEL("Illegal syntax or missing type after optional implementation attribute");
		case IDL_Global::PS_AttrSeen:
			return GTDEVEL("Illegal syntax after ATTRIBUTE keyword");
		case IDL_Global::PS_AttrTypeSeen:
			return GTDEVEL("Illegal syntax after type in attribute declaration");
		case IDL_Global::PS_AttrDeclsSeen:
			return GTDEVEL("Illegal syntax after attribute declarators");
		case IDL_Global::PS_AttrCompleted:
			return GTDEVEL("Illegal syntax after attribute declaration");

		// Property
		//
		case IDL_Global::PS_PropertySeen:
			return GTDEVEL("Illegal syntax after PROPERTY keyword");
		case IDL_Global::PS_PropertyTypeSeen:
			return GTDEVEL("Illegal syntax after type in property declaration");
		case IDL_Global::PS_PropertyDeclsSeen:
			return GTDEVEL("Illegal syntax after property declarators");
		case IDL_Global::PS_PropertyCompleted:
			return GTDEVEL("Illegal syntax after property declaration");
		
		// Exception
		//
		case IDL_Global::PS_ExceptSeen:
			return GTDEVEL("Illegal syntax or missing identifier after EXCEPTION keyword");
		case IDL_Global::PS_ExceptIDSeen:
			return GTDEVEL("Illegal syntax or missing '{' after exception identifier");
		case IDL_Global::PS_ExceptSqSeen:
			return GTDEVEL("Illegal syntax after exception '{' opener");
		case IDL_Global::PS_ExceptQsSeen:
			return GTDEVEL("Illegal syntax after exception '}' closer");
		case IDL_Global::PS_ExceptBodySeen:
			return GTDEVEL("Illegal syntax after exception body statement(s)");
		
		// Operation
		//
		case IDL_Global::PS_OpAttrSeen:
			return GTDEVEL("Illegal syntax or missing type after operation attribute");
		case IDL_Global::PS_OpTypeSeen:
			return GTDEVEL("Illegal syntax or missing identifier after operation type");
		case IDL_Global::PS_OpIDSeen:
			return GTDEVEL("Illegal syntax or missing '(' after operation identifier");
		case IDL_Global::PS_OpParsCompleted:
			return GTDEVEL("Illegal syntax after operation parameter list");
		case IDL_Global::PS_OpRaiseCompleted:
			return GTDEVEL("Illegal syntax after optional RAISES in operation declaration");
		case IDL_Global::PS_OpContextCompleted:
			return GTDEVEL("Illegal syntax after optional CONTEXT in operation declaration");
		case IDL_Global::PS_OpCompleted:
			return GTDEVEL("Illegal syntax after operation declaration");
		case IDL_Global::PS_OpSqSeen:
			return GTDEVEL("Illegal syntax after operation parameter list '(' opener");
		case IDL_Global::PS_OpQsSeen:
			return GTDEVEL("Illegal syntax after operation parameter list ')' closer");
		case IDL_Global::PS_OpParCommaSeen:
			return GTDEVEL("Illegal syntax or missing direction in parameter declaration");
		case IDL_Global::PS_OpParDirSeen:
			return GTDEVEL("Illegal syntax or missing type in parameter declaration");
		case IDL_Global::PS_OpParTypeSeen:
			return GTDEVEL("Illegal syntax or missing declarator in parameter declaration");
		case IDL_Global::PS_OpParDeclSeen:
			return GTDEVEL("Illegal syntax following parameter declarator");
		case IDL_Global::PS_OpRaiseSeen:
			return GTDEVEL("Illegal syntax or missing '(' after RAISES keyword");
		case IDL_Global::PS_OpRaiseSqSeen:
			return GTDEVEL("Illegal syntax after RAISES '(' opener");
		case IDL_Global::PS_OpRaiseQsSeen:
			return GTDEVEL("Illegal syntax after RAISES ')' closer");
		case IDL_Global::PS_OpContextSeen:
			return GTDEVEL("Illegal syntax or missing '(' after CONTEXT keyword");
		case IDL_Global::PS_OpContextSqSeen:
			return GTDEVEL("Illegal syntax after CONTEXT '(' opener");
		case IDL_Global::PS_OpContextQsSeen:
			return GTDEVEL("Illegal syntax after CONTEXT ')' closer");
		case IDL_Global::PS_OpContextCommaSeen:
			return GTDEVEL("Illegal syntax after ',' in CONTEXT in operation declaration");

		case IDL_Global::PS_DeclsCommaSeen:
			return GTDEVEL("Illegal syntax after ',' in declarators list");
		case IDL_Global::PS_DeclsDeclSeen:
			return GTDEVEL("Illegal syntax after declarator in declarators list");

		default:
			return GTDEVEL("Some syntax error");
	}
}
/*
 * Public methods
 */

// Report error messages with various numbers of arguments

void UTL_Error::syntax_error(IDL_Global::ParseState ps, char* errmsg)
{
	String *file = NULL;
	
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		file = IDL_Global::idl_global()->main_filename();
	} else
	{
		file = IDL_Global::idl_global()->filename();
	}
	
	idl_error_header(EIDL_SYNTAX_ERROR,
						IDL_Global::idl_global()->lineno(),
						file);
	fprintf(stderr, "%s%s\n", parse_state_to_error_message(ps), errmsg+11); 
	IDL_Global::idl_global()->
		set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

void dumpFlag(sal_uInt32 flag)
{
	switch (flag)
	{
		case AF_READONLY:
			fprintf(stderr, "'readonly'");
			break;			
		case AF_OPTIONAL:
			fprintf(stderr, "'optional'");
			break;			
		case AF_MAYBEVOID:
			fprintf(stderr, "'maybevoid'");
			break;			
		case AF_BOUND:
			fprintf(stderr, "'bound'");
			break;			
		case AF_CONSTRAINED:
			fprintf(stderr, "'constrained'");
			break;			
		case AF_TRANSIENT:
			fprintf(stderr, "'transient'");
			break;			
		case AF_MAYBEAMBIGUOUS:
			fprintf(stderr, "'maybeambiguous'");
			break;			
		case AF_MAYBEDEFAULT:
			fprintf(stderr, "'maybedefault'");
			break;			
		case AF_REMOVEABLE:		
			fprintf(stderr, "'removeable'");
			break;			
		case AF_ATTRIBUTE:		
			fprintf(stderr, "'attribute'");
			break;			
		case AF_PROPERTY:		
			fprintf(stderr, "'property'");
			break;			
	}	
}	

void UTL_Error::attributeflag_error(UTL_Error::ErrorCode c, sal_uInt32 flag)
{
	String *file = NULL;
	
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		file = IDL_Global::idl_global()->main_filename();
	} else
	{
		file = IDL_Global::idl_global()->filename();
	}
	
	idl_error_header(c,	IDL_Global::idl_global()->lineno(), file);
	dumpFlag(flag);

	fprintf(stderr, "\n");
	IDL_Global::idl_global()->
		set_err_count(IDL_Global::idl_global()->err_count() + 1);
}	

void UTL_Error::error0(UTL_Error::ErrorCode c)
{
	String *file = NULL;
	
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		file = IDL_Global::idl_global()->main_filename();
	} else
	{
		file = IDL_Global::idl_global()->filename();
	}
	
	idl_error_header(c,	IDL_Global::idl_global()->lineno(), file);

	fprintf(stderr, "\n");
	IDL_Global::idl_global()->
		set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

void UTL_Error::error1(UTL_Error::ErrorCode c, AST_Decl *d)
{
	idl_error_header(c, d->line(), d->file_name());
	if(d->name() != NULL) 
	{
		d->name()->dump();
		fprintf(stderr, "\n");
	} else 
		fprintf(stderr, "global\n");

	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

void UTL_Error::error2(UTL_Error::ErrorCode c, AST_Decl *d1, AST_Decl *d2)
{
	idl_error_header(c, d1->line(), d1->file_name());
	if(d1->name() != NULL) 
	{
		d1->name()->dump();
		fprintf(stderr, ", ");
	} else 
		fprintf(stderr, "global, ");
	if(d2->name() != NULL) 
	{
		d2->name()->dump();
		fprintf(stderr, "\n");
	} else 
		fprintf(stderr, "global\n");

	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

void UTL_Error::error3(UTL_Error::ErrorCode c,
						AST_Decl *d1,
						AST_Decl *d2,
						AST_Decl *d3)
{
	idl_error_header(c, d1->line(), d1->file_name());
	if(d1->name() != NULL) 
	{
		d1->name()->dump();
		fprintf(stderr, ", ");
	} else 
		fprintf(stderr, "global, ");
	if(d2->name() != NULL) 
	{
		d2->name()->dump();
		fprintf(stderr, ", ");
	} else
		fprintf(stderr, "global, ");
	if(d3->name() != NULL) 
	{
		d3->name()->dump();
		fprintf(stderr, "\n");
	} else 
		fprintf(stderr, "global\n");

	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report warning messages with various numbers of arguments
void UTL_Error::warning1(UTL_Error::WarningCode c, AST_Decl *d)
{
	idl_warning_header(c, d->line(), d->file_name());
	if(d->name() != NULL) 
	{
		d->name()->dump();
		fprintf(stderr, "\n");
	} else 
		fprintf(stderr, "global\n");
}

void UTL_Error::warning2(UTL_Error::WarningCode c, AST_Decl *d1, AST_Decl *d2)
{
	idl_warning_header(c, d1->line(), d1->file_name());
	if(d1->name() != NULL) 
	{
		d1->name()->dump();
		fprintf(stderr, ", ");
	} else 
		fprintf(stderr, "global, ");
	if(d2->name() != NULL) 
	{
		d2->name()->dump();
		fprintf(stderr, "\n");
	} else 
		fprintf(stderr, "global\n");
}

// Report a failed coercion attempt
void UTL_Error::coercion_error(AST_Expression *v, AST_Expression::ExprType t)
{
	idl_error_header(EIDL_COERCION_FAILURE, v->line(), v->file_name());
	v->dump();
	fprintf(stderr, " to %s\n", exprtype_to_string(t));
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report a failed name lookup attempt
void UTL_Error::lookup_error(UTL_ScopedName *n)
{
	String *file = NULL;
	
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		file = IDL_Global::idl_global()->main_filename();
	} else
	{
		file = IDL_Global::idl_global()->filename();
	}
	
	idl_error_header(EIDL_LOOKUP_ERROR,	IDL_Global::idl_global()->lineno(), file);

	n->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report an attempt to inherit from an interface which was only
// declared forward but not yet defined
void UTL_Error::inheritance_fwd_error(UTL_ScopedName *n, AST_Interface *f)
{
	idl_error_header(EIDL_INHERIT_FWD_ERROR, f->line(), f->file_name());
	fprintf(stderr, "interface ");
	n->dump();
	fprintf(stderr, " cannot inherit from forward declared interface ");
	f->name()->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report an attempt to inherit from something other than an interface
void UTL_Error::inheritance_error(UTL_ScopedName *n, AST_Decl *d)
{
	idl_error_header(EIDL_CANT_INHERIT, d->line(), d->file_name());
	fprintf(stderr, " interface ");
	n->dump();
	fprintf(stderr, " attempts to inherit from ");
	d->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report an error while evaluating an expression
void UTL_Error::eval_error(AST_Expression *v)
{
	idl_error_header(EIDL_EVAL_ERROR, v->line(), v->file_name());
	v->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report a situation where a constant was expected but we
// got something else. This may occur in a union when a label
// evaluates to a non-constant
void UTL_Error::constant_expected(UTL_ScopedName *n, AST_Decl *d)
{
	idl_error_header(EIDL_CONSTANT_EXPECTED, d->line(), d->file_name());
	n->dump();
	fprintf(stderr, " bound to ");
	d->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report a situation where an enumerator was expected but we
// got something else. This may occur in a union with an enum
// discriminator where a label did not evaluate to a constant
// enumerator
void UTL_Error::enum_val_expected(AST_Union *u, AST_UnionLabel *l)
{
	idl_error_header(EIDL_ENUM_VAL_EXPECTED, u->line(), u->file_name());
	fprintf(stderr, " union %s, ", u->local_name());
	l->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report a situation where an enumerator was received but we
// could not find it (by name) in an enum. This occurs when
// a union with an enum discriminator is being parsed, and a
// label evaluates to an enumerator which is not a member of
// the enum discriminator type
void UTL_Error::enum_val_lookup_failure(AST_Union *u,
				AST_Enum *e,
				UTL_ScopedName *n)
{
	idl_error_header(EIDL_ENUM_VAL_NOT_FOUND, u->line(), u->file_name());
	fprintf(stderr, " union %s, ", u->local_name());
	fprintf(stderr, " enum %s, ", e->local_name());
	fprintf(stderr, " enumerator ");
	n->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report two or more spellings for an identifier
void UTL_Error::name_case_error(char *b, char *n)
{
	String *file = NULL;
	
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		file = IDL_Global::idl_global()->main_filename();
	} else
	{
		file = IDL_Global::idl_global()->filename();
	}
	
	idl_error_header(EIDL_NAME_CASE_ERROR,	IDL_Global::idl_global()->lineno(), file);

	fprintf(stderr, "\"%s\" and \"%s\"\n", b, n);
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report an ambiguous definition of a name
void UTL_Error::ambiguous(UTL_Scope *s, AST_Decl *l, AST_Decl *d)
{
	idl_error_header(EIDL_AMBIGUOUS, d->line(), d->file_name());
	fprintf(stderr, " scope: %s, collision: ", (ScopeAsDecl(s))->local_name());
	d->name()->dump();
	fprintf(stderr, " vs. ");
	l->name()->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report a forward declared interface which was never defined
void UTL_Error::fwd_decl_not_defined(AST_Interface *i)
{
	AST_Decl *d = AST_Decl::narrow_from_decl(i);
	idl_error_header(EIDL_DECL_NOT_DEFINED, d->line(), d->file_name());
	fprintf(stderr, "interface %s\n", d->local_name());
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report lookup in undefined forward declared but undefined interface
void UTL_Error::fwd_decl_lookup(AST_Interface *i, UTL_ScopedName *n)
{
	AST_Decl *d = AST_Decl::narrow_from_decl(i);
	idl_error_header(EIDL_FWD_DECL_LOOKUP, d->line(), d->file_name());
	fprintf(stderr, "trying to look up ");
	n->dump();
	fprintf(stderr, " in undefined forward declared interface %s\n",
		   d->local_name());
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report a redefinition inside its own scope
void UTL_Error::redefinition_in_scope(AST_Decl *d, AST_Decl *s)
{
	idl_error_header(EIDL_REDEF_SCOPE, d->line(), d->file_name());
	d->name()->dump();
	fprintf(stderr, ", ");
	s->name()->dump();
	fprintf(stderr, "\n");
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

// Report not a type error
void UTL_Error::not_a_type(AST_Decl *d)
{
	idl_error_header(EIDL_NOT_A_TYPE, d->line(), d->file_name());
	if (d == NULL || d->name() == NULL)
		fprintf(stderr, "unknown symbol");
	else
		d->name()->dump();
	fprintf(stderr, "\n"); 
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
}

void  UTL_Error::mapping_error(long lineno, String *s)
{
	String *file = NULL;
	
	if (*(IDL_Global::idl_global()->filename()) == *(IDL_Global::idl_global()->real_filename()))
	{
		file = IDL_Global::idl_global()->main_filename();
	} else
	{
		file = IDL_Global::idl_global()->filename();
	}

	idl_error_header(EIDL_MAPPING_EXIST, lineno, file);

	if (s)
	{
		fprintf(stderr, "%s\n", s);
	}

	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);

}

void UTL_Error::back_end(long lineno, String *s)
{
	idl_error_header(EIDL_BACK_END, lineno, s);
	IDL_Global::idl_global()
		->set_err_count(IDL_Global::idl_global()->err_count() + 1);
} 
