/*************************************************************************
 *
 *  $RCSfile: utllist.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

// NOTE: This list class only works correctly because we use single public
//       inheritance, as opposed to multiple inheritance or public virtual.
//	 It relies on a type-unsafe cast from UTL_List to subclasses, which
//	 will cease to operate correctly if you use either multiple or
//	 public virtual inheritance.
//
//	 For portability reasons we have decided to provide both this and
//	 an implementation of the list classes in terms of templates. If
//	 your compiler supports templates, please use the files in the
//	 include/utl_tmpl and util/utl_tmpl directories instead of the
//	 files by the same names in the include and util directories.

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

/*
 * Constructor(s)
 */

UTL_List::UTL_List(UTL_List *c)
	:	pd_cdr_data(c) 
{
}

UTL_List::~UTL_List()
{
	if (pd_cdr_data != NULL) 
	{
		delete(pd_cdr_data);
		pd_cdr_data = NULL;
	}
}

/*
 * Private operations
 */

// Compute list length
long UTL_List::list_length(long n)
{
	if (pd_cdr_data == NULL)
		return n;
	else
		return pd_cdr_data->list_length(n+1);
}

/*
 * Public operations
 */

// Smash last cdr with l
void UTL_List::nconc(UTL_List *l)
{
	if (pd_cdr_data == NULL)
		pd_cdr_data = l;
	else
		pd_cdr_data->nconc(l);
}

/*
 * Override this operation to copy lists of other types
 */
// Copy a list
UTL_List *UTL_List::copy()
{
	if (pd_cdr_data == NULL)
		return new UTL_List(NULL);
	return new UTL_List(pd_cdr_data->copy());
}

// Get next list
UTL_List *UTL_List::tail()
{
	return pd_cdr_data;
}

// Set next list
void UTL_List::set_tail(UTL_List *l)
{
	pd_cdr_data = l;
}

// Compute list length
long UTL_List::length()
{
	return list_length(1);
}

/*
 * Redefinition of inherited virtual operations
 */

// UTL_List active iterator

/*
 * Constructor
 */

UTL_ListActiveIterator::UTL_ListActiveIterator(UTL_List *s)
	:	source(s)
{
}

/*
 * Private operations
 */

/*
 * Public operations
 */

// Is iterator done?
long UTL_ListActiveIterator::is_done()
{
	return (source == NULL) ? sal_True : sal_False;
}

// Advance to next item
void UTL_ListActiveIterator::next()
{
	if (source != NULL)
		source = source->tail();
}


