/*************************************************************************
 *
 *  $RCSfile: utlmap.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <idl.hxx>

#include <iostream.h>

#ifndef _IDLDEFS_HXX_
#include <idldefs.hxx>
#endif
#ifndef UTLMAP_HXX_
#include <utlmap.hxx>
#endif

#include <algorithm>

//*******************************************************************
//  Class UTL_Map
//
//*******************************************************************
UTL_Map::UTL_Map()
	:	m_MapName(""),
		m_MapSuper(""),
		m_MapInc(""),  
		m_GetReflection(""), 
 		m_MapIdent(""),
		m_CreateAllListener(sal_False)
{
	
}	

UTL_Map::UTL_Map(const String& n)
	:	m_MapName(n),
		m_MapSuper(""),
		m_MapInc(""),  
		m_GetReflection(""), 
 		m_MapIdent(""),
		m_CreateAllListener(sal_False)
{
}	
 
UTL_Map::UTL_Map(const String& n, const String& sp, const String& inc, 
				 const String& getrefl, const String& increfl, 
				 const String& ident, sal_Bool createAll)
	: 	m_MapName(n), 
		m_MapSuper(sp),
		m_MapInc(inc),  
		m_GetReflection(getrefl), 
 		m_MapIdent(ident),
		m_CreateAllListener(createAll)		
{
	if (increfl.getLength() == 0)
	{
		m_IncReflection = inc;
	} else
	{
		m_IncReflection = increfl;		
	}
}

UTL_Map::UTL_Map(const UTL_Map& rMap)
	: 	m_MapName(rMap.m_MapName), 
		m_MapSuper(rMap.m_MapSuper),
		m_MapInc(rMap.m_MapInc),  
		m_GetReflection(rMap.m_GetReflection), 
 		m_MapIdent(rMap.m_MapIdent),
		m_CreateAllListener(rMap.m_CreateAllListener)		
{	
}

void UTL_Map::setMapSuper(const String& sp)
{
	char *super = (char*)sp.getStr();
	char *superName = NULL;
	
	if (super[0] != '{')
	{
		m_MapSuper = sp;
		return;
	}
	
	super++;
	superName = strtok(super, ";");
	m_SuperInterfaces.push_back(String(superName));
	while (superName != NULL)
	{
		superName = strtok(NULL, " ;}");		
		if (superName != NULL)
		{
			m_SuperInterfaces.push_back(String(superName));
		}
	}	

	NAMESPACE_STD(list)<String>::iterator iter;

	iter = m_SuperInterfaces.begin(); 	
	while (iter != m_SuperInterfaces.end())
	{
		m_MapSuper += (*iter);
		iter++;
		if (iter != m_SuperInterfaces.end())
		{
			m_MapSuper += ", ";
		}
	}	
}	

void UTL_Map::checkNewlines(String& str)
{
	sal_Int32 pos = 0;
	sal_Int32 offset = 0;
	String s1 = str;
	String s2;
	
	while ((pos = s1.indexOf("\\n")) >= 0)
	{
		s2 = s1.replaceAt(offset + pos, 2, "\n");
		offset++;
		s1 = s2;
	}	

	str = s1;
}	

//*******************************************************************
//  Class UTL_MapTable
//
//*******************************************************************
NAMESPACE_STD(hash)<const char*> UTL_MapTable::HashString::h;

UTL_MapTable::UTL_MapTable()
{
//	m_mappingTable[0]=0;
}	

void UTL_MapTable::setMappingOf(const String language, const String name, UTL_Map* mapping)
{
	if (m_mappingTable.count(language) > 0)
	{
		m_mappingTable[language][name] = mapping;		
	} else
	{
		MappingSection section;
		section[name] = mapping;
		
		m_mappingTable[language] = section;
	}
}	
	
UTL_Map* UTL_MapTable::getMappingOf(const String& language, const String& name)
{
	if (m_mappingTable.count(language) > 0)
	{
		if (m_mappingTable[language].count(name) > 0)
		{
			return(m_mappingTable[language][name]);	
		} else
		{
			return(NULL);
		}
	} else
	{
		return(NULL);
	}	
}	
