/*************************************************************************
 *
 *  $RCSfile: utlnlist.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <sys/types.h>

#ifndef _UTLNLIST_HXX_
#include <utlnlist.hxx>
#endif

/*
 * UTL_NListIter
 */

UTL_NListIter::UTL_NListIter(UTL_NList& l)
	:	list_(&l), act_(0), atEnd_(0), atBegin_(0) 
{
	atBegin_ = 0;
	atEnd_ = 0;
	goBegin();
}

UTL_NListIter::UTL_NListIter(UTL_NList& l, UTL_NListElem *act)
	:	list_(&l), act_(act), atEnd_(0), atBegin_(0) 
{
	atBegin_ = (act == l.first_ ? 1:0);
	atEnd_   = (act == l.last_  ? 1:0);
	goBegin();
}

UTL_NListElem *UTL_NListIter::goBegin() 
{
	atBegin_ = 1;
	atEnd_   = 0;
	if(!list_ || !list_->first_) return 0;
	return list_->first_;
}

UTL_NListElem *UTL_NListIter::goEnd() 
{
	atEnd_   = 1;
	atBegin_ = 0;
	if(!list_ || !list_->last_) return 0;
	return list_->last_;
}

UTL_NListElem *UTL_NListIter::next() 
{
	if(list_->numElems_ == 0) return 0;

	if(atBegin_) 
	{
		atBegin_ = 0;
		act_ = list_->first_;
		return act_;
	}

	if(!atEnd_ && act_ != list_->last_) 
	{
		act_ = act_->next_;
	} else 
	{
		atEnd_ = 1;
		return 0;
	}

	return act_;
}

UTL_NListElem *UTL_NListIter::prev() 
{
	if(list_->numElems_ == 0) return 0;

	if(atEnd_) 
	{
		atEnd_ = 0;
		act_ = list_->last_;
		return act_;
	}

	if(!atBegin_ && act_ != list_->first_) 
	{
		act_ = act_->prev_;
	} else 
	{
		atBegin_ = 1;
		return 0;
	}

	return act_;
}

UTL_NListElem *UTL_NListIter::act() 
{
	return act_;
}

unsigned char UTL_NListIter::atEnd() 
{
	return list_->numElems_ == 0 || act_ == list_->last_;
}

unsigned char UTL_NListIter::atBegin() 
{
	return list_->numElems_ == 0 || act_ == list_->first_;
}

void UTL_NListIter::remove() 
{
	UTL_NListElem *rm;

	if(list_->numElems_ == 0 || !act_) return;
  
	if(act_ == list_->first_) 
	{
		rm               = list_->first_;
		list_->first_    = rm->next_;
		act_             = list_->first_;

		if(rm->next_) 
			rm->next_->prev_ = 0;
		else 
			list_->first_    = list_->last_ = 0;
    
		atBegin_         = 1;
		list_->numElems_--;
		delete rm;

		return;
	}

	if(act_ == list_->last_) 
	{
		rm               = list_->last_;
		list_->last_     = rm->prev_;
		act_             = list_->last_;

		rm->prev_->next_ = 0;
		atEnd_           = 1;
		list_->numElems_--;
		delete rm;

		return;
	}

	rm = act_;
	act_ = act_->prev_;
	rm->prev_->next_ = rm->next_;
	rm->next_->prev_ = rm->prev_;
	list_->numElems_--;

	delete rm;
}

/*
 * UTL_NList
 */
    
UTL_NList::UTL_NList()
	:	first_(0), last_(0), numElems_(0)
{
}

UTL_NList::UTL_NList(const UTL_NList& he) 
{
	first_ = last_ = 0;
	numElems_ = 0;
	clone(he);
}

UTL_NList& UTL_NList::operator=(const UTL_NList& he) 
{
	clone(he);
	return *this;
}

UTL_NList::~UTL_NList() 
{
	deleteAll();
}

unsigned int UTL_NList::getNumElems() const 
{
	return numElems_;
}

UTL_NListElem *UTL_NList::append(void *v) 
{
	UTL_NListElem *ne = NCAST(v);
	numElems_++;
  
	if(!last_) 
	{
		first_       = ne;
		last_        = ne;
	} else 
	{
		last_->next_ = ne;
		ne->prev_    = last_;
		last_        = ne;
	}

	return ne;
}

UTL_NListElem *UTL_NList::insertOnes(void *v) 
{
	UTL_NListElem *it;
	UTL_NListElem *ne = NCAST(v);

	it = first_;
	while(it) 
	{
		if(ne->compare(it) == 0) 
		{
			return ne;
		}
		it = it->next_;
	}
  
	return UTL_NList::append(ne);
}

void UTL_NList::remove(UTL_NListElem *rm) 
{
	if(!rm) return;
  
	if(rm == first_) 
	{
		first_ = rm->next_;

		if(rm->next_) 
			rm->next_->prev_ = 0;
		else 
			first_ = last_ = 0;
    
		numElems_--;
		delete rm;
		return;
	}

	if(rm == last_) 
	{
		last_     = rm->prev_;
		rm->prev_->next_ = 0;
		numElems_--;
		delete rm;
		return;
	}

	rm->prev_->next_ = rm->next_;
	rm->next_->prev_ = rm->prev_;
	numElems_--;
	delete rm;
}

UTL_NListElem *SortedUTL_NList::insert(void *v) 
{
	UTL_NListElem *it;
	UTL_NListElem *ne = NCAST(v);
	numElems_++;
  
	if(!last_) 
	{
		first_       = ne;
		last_        = ne;
		return ne;
	} 

	it = first_;
	while(it) 
	{
		if(ne->compare(it) < 0) 
		{
			if(it != first_) 
				it->prev_->next_ = ne;
			else 
				first_ = ne;
      
			ne->prev_ = it->prev_;
			ne->next_ = it;
			it->prev_ = ne;

			return ne;
		}
		it = it->next_;
	}
  
	numElems_--;					// to be incremented by append().
	return UTL_NList::append(ne);
}

UTL_NListElem *SortedUTL_NList::search(const void *v) 
{
	UTL_NListElem *it = first_;
	const UTL_NListElem *elem = NCAST(v);

	while(it && (it->compare(elem) < 0)) 
	{
		it = it->next_;
	}
	return (it && it->compare(elem) == 0) ? it : 0;
}
  
void UTL_NList::deleteAll() 
{
	UTL_NListElem *it, *nextElem;

	it = first_;
	while(it) 
	{
		nextElem = it->next_;
		delete it;
		it = nextElem;
	}

	first_    = 0;
	last_     = 0;
	numElems_ = 0;
} 

unsigned char UTL_NList::empty() 
{
	return numElems_ == 0;
}

UTL_NListElem *UTL_NList::getLastElem() 
{
	return numElems_ ? last_  : 0;
}

UTL_NListElem *UTL_NList::getFirstElem() 
{
	return numElems_ ? first_ : 0;
}

void UTL_NList::clone(const UTL_NList& he) 
{
	if (&he == this) return;
	deleteAll();

	UTL_NListElem *elem = he.first_;

	while(elem) 
	{
		append(elem->clone());
		elem = elem->next_;
	}
}
