/*************************************************************************
 *
 *  $RCSfile: utlstack.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:19:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _IDL_HXX_
#include <idl.hxx>
#endif
#ifndef _IDLEXT_HXX_
#include <idlext.hxx>
#endif

// Class UTL_ScopeStack

#undef	INCREMENT
#define	INCREMENT	64

/*
 * Constructor(s) and destructor
 */

UTL_ScopeStack::UTL_ScopeStack()
	:	pd_stack_data_nalloced(INCREMENT),
		pd_stack_data(new UTL_Scope *[INCREMENT]),
		pd_stack_top(0)
{
	for(unsigned long i=0; i < pd_stack_data_nalloced; i++)
		pd_stack_data[i] = NULL;
}

UTL_ScopeStack::~UTL_ScopeStack()
{
	for(unsigned long i=0; i < pd_stack_top; i++) 
		if (pd_stack_data[i] != NULL) delete(pd_stack_data[i]);
	
	delete[] pd_stack_data;
}

/*
 * Private operations
 */

/*
 * Public operations
 */

// Push an element on the stack
UTL_ScopeStack *UTL_ScopeStack::push(UTL_Scope *el)
{
	UTL_Scope	**tmp;
	AST_Decl	*d = ScopeAsDecl(el);
	unsigned long	ostack_data_nalloced;
	unsigned long	i;

	// Make sure there's space for one more
	if (pd_stack_data_nalloced == pd_stack_top) 
	{
		ostack_data_nalloced = pd_stack_data_nalloced;
		pd_stack_data_nalloced += INCREMENT;
		tmp	= new UTL_Scope *[pd_stack_data_nalloced];
	
		for(i=0; i < pd_stack_data_nalloced; i++)
			pd_stack_data[i] = NULL;

		for (i=0; i < ostack_data_nalloced; i++)
			tmp[i] = pd_stack_data[i];

		delete pd_stack_data;
		pd_stack_data = tmp;
	}

	// Insert new scope
	pd_stack_data[pd_stack_top++] = el;

	return this;
}

// Pop an element from the stack
void UTL_ScopeStack::pop()
{
	UTL_Scope	*s;

	if (pd_stack_top <= 0)
		return;
	s = pd_stack_data[--pd_stack_top];
}

// Return top element on stack
UTL_Scope *UTL_ScopeStack::top()
{
	if (pd_stack_top <= 0)
		return NULL;
	return pd_stack_data[pd_stack_top - 1];
}

// Return bottom element on stack
UTL_Scope *UTL_ScopeStack::bottom()
{
	if (pd_stack_top == 0)
		return NULL;
	return pd_stack_data[0];
}

// Clear entire stack
void UTL_ScopeStack::clear()
{
	pd_stack_top = 0;
}

// How deep is the stack?
unsigned long UTL_ScopeStack::depth()
{
	return pd_stack_top;
}

// Return (top - 1) element on stack
UTL_Scope *UTL_ScopeStack::next_to_top()
{
	UTL_Scope	*tmp, *retval;

	if (depth() < 2)
		return NULL;

	tmp = top();		// Save top
	(void) pop();		// Pop it
	retval = top();	// Get next one down
	(void) push(tmp);	// Push top back
	return retval;	// Return next one down
}

// Return topmost non-NULL element
UTL_Scope *UTL_ScopeStack::top_non_null()
{
	long		i;

	for (i = pd_stack_top - 1; i >= 0; i--)
		if (pd_stack_data[i] != NULL) return pd_stack_data[i];
  
	return NULL;
}

/*
 * Redefinition of inherited virtual operations
 */

// Class UTL_ScopeStackActiveIterator

/*
 * Constructor(s)
 */

UTL_ScopeStackActiveIterator::UTL_ScopeStackActiveIterator(UTL_ScopeStack *s)
	:	source(s),
		il(s->pd_stack_top - 1)
{
}

/*
 * Private operations
 */

/*
 * Public operations
 */

// Advance to next item
void UTL_ScopeStackActiveIterator::next()
{
	il--;
}

// Get current item
UTL_Scope *UTL_ScopeStackActiveIterator::item()
{
	if (il >= 0)
		return source->pd_stack_data[il];
	return NULL;
}

// Is this iteration done?
long UTL_ScopeStackActiveIterator::is_done()
{
	if (il >= 0)
		return sal_False;
	return sal_True;
}

