/*************************************************************************
 *
 *  $RCSfile: dbconversion.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:03:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _UTL_DB_CONVERSION_HXX_
#include <unotools/dbconversion.hxx>
#endif
#ifndef _UTL_NUMBERS_HXX_
#include <unotools/numbers.hxx>
#endif
#ifndef _UNOTOOLS_DATETIME_HXX_
#include <unotools/datetime.hxx>
#endif
#ifndef _UTL_UNO3_DB_TOOLS_HXX_
#include <unotools/dbtools.hxx>
#endif
#ifndef _OSL_DIAGNOSE_H_ 
#include <osl/diagnose.h>
#endif

#include <com/sun/star/util/NumberFormat.hpp>
#include <com/sun/star/util/XNumberFormatTypes.hpp>
#include <com/sun/star/sdbc/DataType.hpp>

//.........................................................................
namespace utl
{
//.........................................................................

//------------------------------------------------------------------------------
void DBTypeConversion::setValue(const staruno::Reference<starsdb::XColumnUpdate>& xVariant,
								const staruno::Reference<starutil::XNumberFormatter>& xFormatter,
								const Date& rNullDate,
								const ::rtl::OUString& rString,
								sal_Int32 nKey,
								sal_Int16 nFieldType,
								sal_Int16 nKeyType) throw(starlang::IllegalArgumentException)
{
	double fValue = 0;
	if (rString.len())
	{
			// Muss der String formatiert werden?
		sal_Int16 nTypeClass = nKeyType & ~starutil::NumberFormat::DEFINED;
		sal_Bool bTextFormat = nTypeClass == starutil::NumberFormat::TEXT;
		sal_Int32 nKeyToUse  = bTextFormat ? 0 : nKey;
		sal_Int16 nRealUsedTypeClass = nTypeClass;
			// bei einem Text-Format muessen wir dem Formatter etwas mehr Freiheiten einraeumen, sonst
			// wirft convertStringToNumber eine NotNumericException
		try
		{
			fValue = xFormatter->convertStringToNumber(nKeyToUse, rString);
			sal_Int32 nRealUsedKey = xFormatter->detectNumberFormat(0, rString);
			if (nRealUsedKey != nKeyToUse)
				nRealUsedTypeClass = getNumberFormatType(xFormatter, nRealUsedKey) & ~starutil::NumberFormat::DEFINED;

			// und noch eine Sonderbehandlung, diesmal fuer Prozent-Formate
			if ((starutil::NumberFormat::NUMBER == nRealUsedTypeClass) && (starutil::NumberFormat::PERCENT == nTypeClass))
			{	// die Formatierung soll eigentlich als Prozent erfolgen, aber der String stellt nur eine
				// einfache Nummer dar -> anpassen
				::rtl::OUString sExpanded(rString);
				static ::rtl::OUString s_sPercentSymbol = ::rtl::OUString::createFromAscii("%");
					// need a method to add a sal_Unicode to a string, 'til then we use a static string ...
				sExpanded += s_sPercentSymbol;
				fValue = xFormatter->convertStringToNumber(nKeyToUse, sExpanded);
			}

			switch (nRealUsedTypeClass)
			{
				case starutil::NumberFormat::DATE:
				case starutil::NumberFormat::DATETIME:
					xVariant->updateDouble(ToStandardDbDate(rNullDate, fValue));
					break;
				case starutil::NumberFormat::TIME:
				case starutil::NumberFormat::CURRENCY:
				case starutil::NumberFormat::NUMBER:
				case starutil::NumberFormat::SCIENTIFIC:
				case starutil::NumberFormat::FRACTION:
				case starutil::NumberFormat::PERCENT:
					xVariant->updateDouble(fValue);
					break;
				default:
					xVariant->updateString(rString);
			}
		}
		catch(...)
		{
			xVariant->updateString(rString);
		}
	}
	else
	{
		switch (nFieldType)
		{
			case starsdbc::DataType::CHAR:
			case starsdbc::DataType::VARCHAR:
			case starsdbc::DataType::LONGVARCHAR:
				xVariant->updateString(rString);
				break;
			default:
				xVariant->updateNull();
		}
	}
}

//------------------------------------------------------------------------------
void DBTypeConversion::setValue(const staruno::Reference<starsdb::XColumnUpdate>& xVariant,
								const Date& rNullDate,
								const double& rValue,
								sal_Int16 nKeyType) throw(starlang::IllegalArgumentException)
{
	switch (nKeyType & ~starutil::NumberFormat::DEFINED)
	{
		case starutil::NumberFormat::DATE:
		case starutil::NumberFormat::DATETIME:
		//	case starutil::NumberFormat::TIME:
			xVariant->updateDouble(ToStandardDbDate(rNullDate, rValue));
			break;
		default:
			xVariant->updateDouble(rValue);
	}
}

//------------------------------------------------------------------------------
double DBTypeConversion::getValue(const staruno::Reference<starsdb::XColumn>& xVariant,
								  const Date& rNullDate,
								  sal_Int16 nKeyType)
{
	try
	{
		switch (nKeyType & ~starutil::NumberFormat::DEFINED)
		{
			case starutil::NumberFormat::DATE:
			case starutil::NumberFormat::DATETIME:
			//	case starutil::NumberFormat::TIME:
				return ToNullDate(rNullDate, xVariant->getDouble());
			default:
				return xVariant->getDouble();
		}
	}
	catch(...)
	{
		return 0.0;
	}
}
//------------------------------------------------------------------------------
::rtl::OUString DBTypeConversion::getValue(const staruno::Reference<starbeans::XPropertySet>& _xColumn,
										   const staruno::Reference<starutil::XNumberFormatter>& _xFormatter,
										   const starlang::Locale& _rLocale,
										   const Date& _rNullDate)
{
	sal_Int32 nKey;
	sal_Int16 nKeyType;
	
	OSL_ENSHURE(_xColumn.is() && _xFormatter.is(), "DBTypeConversion::getValue: invalid arg !");	
	if (!_xColumn.is() || !_xFormatter.is())
		return ::rtl::OUString();

	try
	{
		_xColumn->getPropertyValue(::rtl::OUString::createFromAscii("FormatKey")) >>= nKey;
	}
	catch (...)
	{
	}

	if (!nKey)
	{
		staruno::Reference<starutil::XNumberFormats> xFormats( _xFormatter->getNumberFormatsSupplier()->getNumberFormats() );
		staruno::Reference<starutil::XNumberFormatTypes> xTypeList(_xFormatter->getNumberFormatsSupplier()->getNumberFormats(), staruno::UNO_QUERY);

		nKey = utl::getDefaultNumberFormat(_xColumn,
										   staruno::Reference< starutil::XNumberFormatTypes > (xFormats, staruno::UNO_QUERY),
										   _rLocale);

		nKeyType = getNumberFormatType(_xFormatter, nKey) & ~starutil::NumberFormat::DEFINED;
	}
	
	return DBTypeConversion::getValue(staruno::Reference< starsdb::XColumn > (_xColumn, staruno::UNO_QUERY), _xFormatter, _rNullDate, nKey, nKeyType);
}

//------------------------------------------------------------------------------
::rtl::OUString DBTypeConversion::getValue(const staruno::Reference<starsdb::XColumn>& xVariant,
								   const staruno::Reference<starutil::XNumberFormatter>& xFormatter,
								   const Date& rNullDate,
								   sal_Int32 nKey,
								   sal_Int16 nKeyType)
{
	::rtl::OUString aString;
	if (xVariant.is())
	{
		try
		{
			switch (nKeyType & ~starutil::NumberFormat::DEFINED)
			{
				case starutil::NumberFormat::DATE:
				case starutil::NumberFormat::DATETIME:
				{
					double fValue = xVariant->getDouble();
					if (!xVariant->wasNull())
						xFormatter->convertNumberToString(nKey, ToNullDate(rNullDate, fValue));
				}	break;
				case starutil::NumberFormat::TIME:
				case starutil::NumberFormat::NUMBER:
				case starutil::NumberFormat::SCIENTIFIC:
				case starutil::NumberFormat::FRACTION:
				case starutil::NumberFormat::PERCENT:
				{
					double fValue = xVariant->getDouble();
					if (!xVariant->wasNull())
						aString = xFormatter->convertNumberToString(nKey, fValue);
				}	break;
				case starutil::NumberFormat::CURRENCY:
				{
					double fValue = xVariant->getDouble();
					if (!xVariant->wasNull())
						aString = xFormatter->getInputString(nKey, fValue);
				}	break;
				case starutil::NumberFormat::TEXT:
					aString = xFormatter->formatString(nKey, xVariant->getString());
					break;
				default:
					aString = xVariant->getString();
			}
		}
		catch(...)
		{
			aString = xVariant->getString();
		}
	}
	return aString;
}


const double fMilliSecondsPerDay = 86400000.0;
//------------------------------------------------------------------------------
Date DBTypeConversion::toDate(double dVal)
{
	long nDays = (long)dVal;

	Date x(STANDARD_DB_DATE);
	if (nDays >= 0)
		x += (ULONG)nDays;
	else
		x -= (ULONG)(-nDays);

	return x;
}

//------------------------------------------------------------------------------
Time DBTypeConversion::toTime(double dVal)
{
	long nDays     = (long)dVal;
	long nMSeconds = long((dVal - (double)nDays) * fMilliSecondsPerDay + 0.5);

	Time x;
	x.MakeTimeFromMS(nMSeconds);
	if(x.GetTime() < 0)
		x += Time(23,59,59,99);
	return x;
}

//------------------------------------------------------------------------------
DateTime DBTypeConversion::toDateTime(double dVal)
{
	long nDays     = (long)dVal;
	long nMSeconds = long((dVal - (double)nDays) * fMilliSecondsPerDay + 0.5);

	// Datum:
	DateTime x(STANDARD_DB_DATE);
	if (nDays >= 0)
		x += (ULONG)nDays;
	else
		x -= (ULONG)(-nDays);

	// Zeit:
	x.MakeTimeFromMS(nMSeconds);
	if(x.GetTime() < 0)
	{
		x -= (ULONG)1;
		x += Time(23,59,59,99);
	}
	return x;
}

//------------------------------------------------------------------------------
sal_Int32 DBTypeConversion::toINT32(const Date& rVal)
{
	return rVal.GetDate();
}

//------------------------------------------------------------------------------
sal_Int32 DBTypeConversion::toINT32(const Time& rVal)
{
	return rVal.GetTime();
}

//------------------------------------------------------------------------------
sal_Int32 DBTypeConversion::toINT32(const starutil::Date& rVal)
{
	Date aToolsValue;
	typeConvert(rVal, aToolsValue);
	return aToolsValue.GetDate();
}

//------------------------------------------------------------------------------
sal_Int32 DBTypeConversion::toINT32(const starutil::Time& rVal)
{
	Time aToolsValue;
	typeConvert(rVal, aToolsValue);
	return aToolsValue.GetTime();
}

//------------------------------------------------------------------------------
double DBTypeConversion::toDouble(const starutil::Date& rVal)
{
	Date aToolsValue;
	typeConvert(rVal, aToolsValue);
	return toDouble(aToolsValue);
}

//------------------------------------------------------------------------------
double DBTypeConversion::toDouble(const starutil::Time& rVal)
{
	Time aToolsValue;
	typeConvert(rVal, aToolsValue);
	return toDouble(aToolsValue);
}

//------------------------------------------------------------------------------
double DBTypeConversion::toDouble(const starutil::DateTime& rVal)
{
	DateTime aToolsValue;
	typeConvert(rVal, aToolsValue);
	return toDouble(aToolsValue);
}

//------------------------------------------------------------------------------
double DBTypeConversion::toDouble(const Date& rVal)
{
	long nDays = 0;
	if (STANDARD_DB_DATE > rVal)
	{
		nDays = (long)(STANDARD_DB_DATE - rVal);
		nDays *= -1;
	}
	else
		nDays = (long)(rVal - STANDARD_DB_DATE);
	return (double)nDays;
}

//------------------------------------------------------------------------------
double DBTypeConversion::toDouble(const Time& rVal)
{
	return (double)rVal.GetMSFromTime() / fMilliSecondsPerDay;
}

//------------------------------------------------------------------------------
double DBTypeConversion::toDouble(const DateTime& rVal)
{
	long nDays = 0;
	if (STANDARD_DB_DATE > rVal)
	{
		nDays = (long)(STANDARD_DB_DATE - rVal);
		nDays *= -1;
	}
	else
		nDays = (long)(rVal - STANDARD_DB_DATE);

	double fMSeconds = rVal.GetMSFromTime();
	return ((double)nDays) + (fMSeconds * (1/fMilliSecondsPerDay));
}

//.........................................................................
}	// namespace utl
//.........................................................................

