/*************************************************************************
 *
 *  $RCSfile: atom.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _VCL_ATOM_HXX
#define _VCL_ATOM_HXX

#ifndef _COM_SUN_STAR_PORTAL_CLIENT_XATOMSERVER_HPP_
#include <com/sun/star/portal/client/XAtomServer.hpp>
#endif
#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif
#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif
#ifndef _CPPUHELPER_IMPLBASE1_HXX_
#include <cppuhelper/implbase1.hxx>
#endif

#ifndef __SGI_STL_HASH_MAP
#include <hash_map>
#endif

struct std::hash< rtl::OUString >
{
	size_t operator()(const rtl::OUString& rString) const
		{ return (size_t)rString.hashCode(); }
};

#define INVALID_ATOM 0

namespace vcl {

	class AtomProvider
	{
		int										m_nAtoms;
		::std::hash_map< int, ::rtl::OUString, ::std::hash< int > > m_aStringMap;
		::std::hash_map< ::rtl::OUString, int, ::std::hash< ::rtl::OUString > >	m_aAtomMap;
	public:
		AtomProvider();
		~AtomProvider();
		
		int getAtom( const ::rtl::OUString&, sal_Bool bCreate = sal_False );
		const ::rtl::OUString& getString( int ) const;

		::com::sun::star::uno::Sequence< ::com::sun::star::portal::client::AtomDescription > getAll();

		void overrideAtom( int atom, const ::rtl::OUString& description );

		sal_Bool hasAtom( int atom ) const;
	};

	/*	a container for atom providers
	 *	this can be useful if you need to have
	 *	different classes of atoms or different atom lists
	 *	(e.g. for different connections)
	 */
	class MultiAtomProvider
	{
		::std::hash_map< int, AtomProvider*, ::std::hash< int > > m_aAtomLists;
	public:
		MultiAtomProvider();
		~MultiAtomProvider();

		sal_Bool insertAtomClass( int atomClass );
		// returns false, if insertion failed (which means that this class
		// is already inserted )

		int getAtom( int atomClass, const ::rtl::OUString& rString, sal_Bool bCreate = sal_False );
		// if bCreate is true, the class is created if necessary

		const ::rtl::OUString& getString( int atomClass, int atom ) const;
		::com::sun::star::uno::Sequence< ::com::sun::star::portal::client::AtomDescription > getClass( int atomClass ) const;

		void overrideAtom( int atomClass, int atom, const ::rtl::OUString& description );
		void overrideAtom( int atomClass, const ::com::sun::star::portal::client::AtomDescription& newDescription )
			{ overrideAtom( atomClass, newDescription.atom, newDescription.description ); }
		sal_Bool hasAtom( int atomClass, int atom ) const;
	};

	class AtomServer : public ::cppu::WeakAggImplHelper1< ::com::sun::star::portal::client::XAtomServer >
	{
	private:
		MultiAtomProvider	m_aProvider;
		::osl::Mutex		m_aMutex;
	public:
		AtomServer();
		virtual ~AtomServer();

		const ::rtl::OUString& getString( int atomClass, int atom ) const
			{ return m_aProvider.getString( atomClass, atom ); }

		// XAtomServer
		virtual ::com::sun::star::uno::Sequence< ::com::sun::star::portal::client::AtomDescription > SAL_CALL getClass( sal_Int32 atomClass );
		virtual ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Sequence< ::com::sun::star::portal::client::AtomDescription > > SAL_CALL getClasses( const ::com::sun::star::uno::Sequence< sal_Int32 >& atomClasses );
		virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getAtomDescriptions( const ::com::sun::star::uno::Sequence< ::com::sun::star::portal::client::AtomClassRequest >& atoms );
		virtual sal_Int32 SAL_CALL getAtom( sal_Int32 atomClass, const ::rtl::OUString& description, sal_Bool create );
	};

	class AtomClient
	{
	private:
		::com::sun::star::uno::Reference< ::com::sun::star::portal::client::XAtomServer >		m_xServer;
		MultiAtomProvider m_aProvider;
	public:
		AtomClient( const ::com::sun::star::uno::Reference< ::com::sun::star::portal::client::XAtomServer >& );
		~AtomClient();

		void updateAtomClasses( const ::com::sun::star::uno::Sequence< sal_Int32 >& atomClasses );
		int getAtom( int atomClass, const ::rtl::OUString& description, sal_Bool bCreate );
		const ::rtl::OUString& getString( int atomClass, int atom );
	};

}

#endif // _VCL_ATOM_HXX
