/*************************************************************************
 *
 *  $RCSfile: cmdevt.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _VCL_CMDEVT_HXX
#define _VCL_CMDEVT_HXX

#ifndef _GEN_HXX
#include <tools/gen.hxx>
#endif
#ifndef _SV_H
#include <sv.h>
#endif
#ifndef _VCL_KEYCOD_HXX
#include <keycod.hxx>
#endif
#ifndef _VCL_FONT_HXX
#include <font.hxx>
#endif

// ---------------------------
// - CommandExtTextInputData -
// ---------------------------

#define EXTTEXTINPUT_ATTR_INPUT 				((USHORT)0x0001)
#define EXTTEXTINPUT_ATTR_CONVERTED 			((USHORT)0x0002)
#define EXTTEXTINPUT_ATTR_TARGETCONVERTED		((USHORT)0x0004)
#define EXTTEXTINPUT_ATTR_TARGETNOTCONVERTED	((USHORT)0x0008)
#define EXTTEXTINPUT_ATTR_INPUTERROR			((USHORT)0x0010)

#define EXTTEXTINPUT_ATTR_GRAYWAVELINE			((USHORT)0x0100)
#define EXTTEXTINPUT_ATTR_UNDERLINE 			((USHORT)0x0200)
#define EXTTEXTINPUT_ATTR_BOLDUNDERLINE 		((USHORT)0x0400)
#define EXTTEXTINPUT_ATTR_DOTTEDUNDERLINE		((USHORT)0x0800)
#define EXTTEXTINPUT_ATTR_DASHDOTUNDERLINE		((USHORT)0x1000)
#define EXTTEXTINPUT_ATTR_HIGHLIGHT 			((USHORT)0x2000)
#define EXTTEXTINPUT_ATTR_REDTEXT				((USHORT)0x4000)

class CommandExtTextInputData
{
private:
	XubString			maText;
	const USHORT*		mpTextAttr;
	USHORT				mnCursorPos;
	USHORT				mnDeltaStart;
	USHORT				mnOldTextLen;
	BOOL				mbCursorVisible;
	BOOL				mbOnlyCursor;

public:
						CommandExtTextInputData();
						CommandExtTextInputData( const XubString& rText,
												 const USHORT* pTextAttr,
												 USHORT nCursorPos,
												 BOOL bCursorVisible,
												 USHORT nDeltaStart,
												 USHORT nOldTextLen,
												 BOOL bOnlyCursor );

	const XubString&	GetText() const { return maText; }
	const USHORT*		GetTextAttr() const { return mpTextAttr; }
	USHORT				GetCharTextAttr( USHORT nIndex ) const;
	USHORT				GetCursorPos() const { return mnCursorPos; }
	BOOL				IsCursorVisible() const { return mbCursorVisible; }
	USHORT				GetDeltaStart() const { return mnDeltaStart; }
	BOOL				IsOnlyCursorChanged() const { return mbOnlyCursor; }
};

inline CommandExtTextInputData::CommandExtTextInputData()
{
	mpTextAttr		= NULL;
	mnCursorPos 	= 0;
	mnDeltaStart	= 0;
	mnOldTextLen	= 0;
	mbCursorVisible = FALSE;
	mbOnlyCursor	= FALSE;
}

inline CommandExtTextInputData::CommandExtTextInputData( const XubString& rText,
														 const USHORT* pTextAttr,
														 USHORT nCursorPos,
														 BOOL bCursorVisible,
														 USHORT nDeltaStart,
														 USHORT nOldTextLen,
														 BOOL bOnlyCursor ) :
	maText( rText )
{
	mpTextAttr		= pTextAttr;
	mnCursorPos 	= nCursorPos;
	mnDeltaStart	= nDeltaStart;
	mnOldTextLen	= nOldTextLen;
	mbCursorVisible = bCursorVisible;
	mbOnlyCursor	= bOnlyCursor;
}

inline USHORT CommandExtTextInputData::GetCharTextAttr( USHORT nIndex ) const
{
	if ( mpTextAttr && (nIndex < maText.Len()) )
		return mpTextAttr[nIndex];
	else
		return 0;
}

// ------------------------------
// - CommandExtTextInputPosData -
// ------------------------------

#define EXTTEXTINPUTPOS_NOSHOWPOS				((USHORT)0xFFFF)

class CommandExtTextInputPosData
{
private:
	USHORT			mnFirst;
	USHORT			mnChars;
	USHORT			mnShowPos;

public:
					CommandExtTextInputPosData();
					CommandExtTextInputPosData( USHORT nFirstPos, USHORT nChars,
												USHORT nShowPos = 0xFFFF );

	USHORT			GetFirstPos() const { return mnFirst; }
	USHORT			GetChars() const { return mnChars; }
	USHORT			GetShowPos() const { return mnShowPos; }
	BOOL			IsShowPos() const { return mnShowPos != EXTTEXTINPUTPOS_NOSHOWPOS; }
};

inline CommandExtTextInputPosData::CommandExtTextInputPosData()
{
	mnFirst 		= 0;
	mnChars 		= 0;
	mnShowPos		= 0xFFFF;
}

inline CommandExtTextInputPosData::CommandExtTextInputPosData( USHORT nFirstPos,
															   USHORT nChars,
															   USHORT nShowPos )
{
	mnFirst 		= nFirstPos;
	mnChars 		= nChars;
	mnShowPos		= nShowPos;
}

// ---------------------------
// - CommandInputContextData -
// ---------------------------

class CommandInputContextData
{
private:
	Font			maFont;

public:
					CommandInputContextData();
					CommandInputContextData( const Font& rFont );

	const Font& 	GetFont() const { return maFont; }
};

inline CommandInputContextData::CommandInputContextData()
{
}

inline CommandInputContextData::CommandInputContextData( const Font& rFont ) :
	maFont( rFont )
{
}

// --------------------
// - CommandWheelData -
// --------------------

#define COMMAND_WHEEL_SCROLL			((USHORT)0x0001)
#define COMMAND_WHEEL_ZOOM				((USHORT)0x0002)
#define COMMAND_WHEEL_DATAZOOM			((USHORT)0x0004)

#define COMMAND_WHEEL_PAGESCROLL		((ULONG)0xFFFFFFFF)

class CommandWheelData
{
private:
	long			mnDelta;
	long			mnNotchDelta;
	ULONG			mnLines;
	USHORT			mnMode;
	USHORT			mnCode;
	BOOL			mbHorz;

public:
					CommandWheelData();
					CommandWheelData( long nWheelDelta, long nWheelNotchDelta,
									  ULONG nScrollLines,
									  USHORT nWheelMode, USHORT nKeyModifier,
									  BOOL mbHorz = FALSE );

	long			GetDelta() const { return mnDelta; }
	long			GetNotchDelta() const { return mnNotchDelta; }
	ULONG			GetScrollLines() const { return mnLines; }
	BOOL			IsHorz() const { return mbHorz; }

	USHORT			GetMode() const { return mnMode; }

	USHORT			GetModifier() const
						{ return (mnCode & (KEY_SHIFT | KEY_MOD1 | KEY_MOD2)); }
	BOOL			IsShift() const
						{ return ((mnCode & KEY_SHIFT) != 0); }
	BOOL			IsMod1() const
						{ return ((mnCode & KEY_MOD1) != 0); }
	BOOL			IsMod2() const
						{ return ((mnCode & KEY_MOD2) != 0); }
};

inline CommandWheelData::CommandWheelData()
{
	mnDelta 		= 0;
	mnNotchDelta	= 0;
	mnLines 		= 0;
	mnMode			= 0;
	mnCode			= 0;
	mbHorz			= FALSE;
}

inline CommandWheelData::CommandWheelData( long nWheelDelta, long nWheelNotchDelta,
										   ULONG nScrollLines,
										   USHORT nWheelMode, USHORT nKeyModifier,
										   BOOL bHorz )
{
	mnDelta 		= nWheelDelta;
	mnNotchDelta	= nWheelNotchDelta;
	mnLines 		= nScrollLines;
	mnMode			= nWheelMode;
	mnCode			= nKeyModifier;
	mbHorz			= bHorz;
}

// ---------------------
// - CommandScrollData -
// ---------------------

class CommandScrollData
{
private:
	long			mnDeltaX;
	long			mnDeltaY;

public:
					CommandScrollData();
					CommandScrollData( long nDeltaX, long nDeltaY );

	long			GetDeltaX() const { return mnDeltaX; }
	long			GetDeltaY() const { return mnDeltaY; }
};

inline CommandScrollData::CommandScrollData()
{
	mnDeltaX	= 0;
	mnDeltaY	= 0;
}

inline CommandScrollData::CommandScrollData( long nDeltaX, long nDeltaY )
{
	mnDeltaX	= nDeltaX;
	mnDeltaY	= nDeltaY;
}

// --------------------
// - CommandVoiceData -
// --------------------

enum DictationCommandType
{
	VOICECOMMANDTYPE_DICTATION,
	VOICECOMMANDTYPE_CONTROL
};

class CommandVoiceData
{
private:
	XubString				maText;
	USHORT					mnCommand;
	DictationCommandType	meType;

public:
							CommandVoiceData();
							CommandVoiceData( const XubString& rStr,
											  DictationCommandType eType,
											  USHORT nCommand );

	const XubString&		GetText() const { return maText; }
	USHORT					GetCommand() const { return mnCommand; }
	DictationCommandType	GetType() const { return meType; }
};

inline CommandVoiceData::CommandVoiceData()
{
	meType = VOICECOMMANDTYPE_DICTATION;
	mnCommand = 0;
}

inline CommandVoiceData::CommandVoiceData( const XubString& rStr,
										   DictationCommandType eType,
										   USHORT nCommand ) :
	maText( rStr ),
	meType( eType ),
	mnCommand( nCommand )
{
}

// ----------------
// - CommandEvent -
// ----------------

#define COMMAND_CONTEXTMENU 			((USHORT)1)
#define COMMAND_STARTDRAG				((USHORT)2)
#define COMMAND_WHEEL					((USHORT)3)
#define COMMAND_STARTAUTOSCROLL 		((USHORT)4)
#define COMMAND_AUTOSCROLL				((USHORT)5)
#define COMMAND_VOICE					((USHORT)6)
#define COMMAND_STARTEXTTEXTINPUT		((USHORT)7)
#define COMMAND_EXTTEXTINPUT			((USHORT)8)
#define COMMAND_ENDEXTTEXTINPUT 		((USHORT)9)
#define COMMAND_EXTTEXTINPUTPOS 		((USHORT)10)
#define COMMAND_INPUTCONTEXTCHANGE		((USHORT)11)
#define COMMAND_CURSORPOS				((USHORT)12)

#define COMMAND_USER					((USHORT)4096)

class CommandEvent
{
private:
	Point								maPos;
	void*								mpData;
	USHORT								mnCommand;
	BOOL								mbMouseEvent;

public:
										CommandEvent();
										CommandEvent( const Point& rMousePos, USHORT nCmd,
													  BOOL bMEvt = FALSE, const void* pCmdData = NULL );

	USHORT								GetCommand() const { return mnCommand; }
	const Point&						GetMousePosPixel() const { return maPos; }
	BOOL								IsMouseEvent() const { return mbMouseEvent; }
	void*								GetData() const { return mpData; }

	const CommandExtTextInputData*		GetExtTextInputData() const;
	const CommandExtTextInputPosData*	GetExtTextInputPosData() const;
	const CommandInputContextData*		GetInputContextChangeData() const;
	const CommandWheelData* 			GetWheelData() const;
	const CommandScrollData*			GetAutoScrollData() const;
	const CommandVoiceData* 			GetVoiceData() const;
};

inline CommandEvent::CommandEvent()
{
	mpData			= NULL;
	mnCommand		= 0;
	mbMouseEvent	= FALSE;
}

inline CommandEvent::CommandEvent( const Point& rMousePos,
								   USHORT nCmd, BOOL bMEvt, const void* pCmdData ) :
			maPos( rMousePos )
{
	mpData			= (void*)pCmdData;
	mnCommand		= nCmd;
	mbMouseEvent	= bMEvt;
}

inline const CommandExtTextInputData* CommandEvent::GetExtTextInputData() const
{
	if ( mnCommand == COMMAND_EXTTEXTINPUT )
		return (const CommandExtTextInputData*)mpData;
	else
		return NULL;
}

inline const CommandExtTextInputPosData* CommandEvent::GetExtTextInputPosData() const
{
	if ( mnCommand == COMMAND_EXTTEXTINPUTPOS )
		return (const CommandExtTextInputPosData*)mpData;
	else
		return NULL;
}

inline const CommandInputContextData* CommandEvent::GetInputContextChangeData() const
{
	if ( mnCommand == COMMAND_INPUTCONTEXTCHANGE )
		return (const CommandInputContextData*)mpData;
	else
		return NULL;
}

inline const CommandWheelData* CommandEvent::GetWheelData() const
{
	if ( mnCommand == COMMAND_WHEEL )
		return (const CommandWheelData*)mpData;
	else
		return NULL;
}

inline const CommandScrollData* CommandEvent::GetAutoScrollData() const
{
	if ( mnCommand == COMMAND_AUTOSCROLL )
		return (const CommandScrollData*)mpData;
	else
		return NULL;
}

inline const CommandVoiceData* CommandEvent::GetVoiceData() const
{
	if ( mnCommand == COMMAND_VOICE )
		return (const CommandVoiceData*)mpData;
	else
		return NULL;
}

#endif // _VCL_CMDEVT_HXX
