/*************************************************************************
 *
 *  $RCSfile: drag.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:03 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_DRAG_HXX
#define _SV_DRAG_HXX

#ifndef _SV_SV_H
#include <sv.h>
#endif
#ifndef _SV_EXCHANGE_HXX
#include <exchange.hxx>
#endif

#ifndef _SV_EVENT_HXX
#include <event.hxx>
#endif
#ifndef _SV_POINTR_HXX
#include <pointr.hxx>
#endif
#ifndef _SOT_DTRANS_HXX
#include <sot/dtrans.hxx>
#endif


// Vorlaeufig immer zulassen
//#ifndef NOOLDSV
#define OLDDRAGDROP
//#endif

//************************************************************
// TODO: Drag&Drop-Defines aus window.hxx hierher verschieben
// DRAG_DEF_COPYABLE raus
//************************************************************

#ifndef SOT_DECL_MULTIITEMOBJECT_DEFINED
#define SOT_DECL_MULTIITEMOBJECT_DEFINED
SO2_DECL_REF(MultiItemObject)
#endif


class SalFrame;

// Multi-Item-DataObject, um die alte Drag-Server-Schnittstelle zu unterstuetzen
class MultiItemObject : public SotDataMemberObject
/*	[Beschreibung]

	Diese Klasse bildet die Grundlage, um die alte Multi-Item-Schnittstelle
	der Klasse DragServer auf der Basis von SotDataObject nachzubilden.
	Die normale Schnittstelle wird weiterhin von der Basisklasse behandelt.

	Zusaetzlich werden Methoden mit einem Index-Parameter angeboten. Das Objekt
	in seiner Eigenschaft als SotDataMemberObject ist immer identisch mit Item 0.

	[Anmerkung]
*/
{
	friend class DragManager;
	friend class ImplDragDropListener;

	// Weitere Items als Liste
	SotDataObjectRef*	pDataList;
	USHORT				mnListSize;
	USHORT				mnItemCount;

#ifdef OLDDRAGDROP
	SotDataObjectRef	xExtObj;

						// Private Ctor fuer DragManager, um ExecuteDrag am
						// DataObject auf die alte DragServer-Schnittstelle
						// abbilden zu koennen.
						MultiItemObject( SotDataObject* pDataObject_ );
#endif // OLDDRAGDROP

public:
						MultiItemObject();
						~MultiItemObject();

	// Item-Handling
	SotDataObject*		GetItem( USHORT nItem );
	USHORT				GetItemCount( void )	{ return mnItemCount; }
	void				InsertItem( SotDataObject* pDataObject );
	void				NewItem( void );

	virtual const		SvDataTypeList* GetTypeList( USHORT nItem );
	virtual BOOL		GetData( USHORT nItem, SvData * );
	// Fuer alte Schnittstelle nicht noetig, oder?
	// virtual BOOL    GetDataHere( USHORT nItem, SotData * );

	/*
	virtual const SvDataTypeList & GetTypeList() const;
	virtual BOOL	GetData( SotData * );
	virtual BOOL	GetDataHere( SotData * );
	*/
};

SO2_IMPL_REF(MultiItemObject)

// DropSource-Interface
class DropSource
{
public:
	// Source-Reaktion auf Drop-Vorgang, wird waehrend des Drag&Drop
	// regelmaessig aufgerufen. Parameter:
	// - eAction:	Aktuelle Drop-Action, die sich als Ergebnis von
	//				DropEnter, QueryDrop usw. ergibt.
	// - pPos:		Aktuelle Position des Mauszeigers, sofern sie bekannt ist,
	//				NULL sonst
	// - pTarget:	Gibt das aktuelle Ziel der Drag&Drop-Operation an, d.h. das
	//				Window, an dem DragEnter oder DragOver gerufen wird.
	//				NULL, wenn das Target unbekannt ist
	// return		TRUE, wenn GiveFeedback() seine Aufgabe erfllt hat
	//				FALSE, wenn das Default-Verhalten erfolgen soll (StandardCursor)
	virtual BOOL DrawDragPointer( DropAction eAction, const Point& rPos, Window* pTarget ) = 0;

	// Vor dem nchsten Aufruf von DrawPointer(), wird jeweils ClrPointer gerufen,
	// damit die Ausgaben von DrawPointer wieder zurueckgenommen werden knnen.
	// Wenn DrawDragPointer() FALSE liefert, wird der Aufruf von ClrDragPointer()
	// nicht garantiert und kann leer implementiert werden.
	virtual void ClrDragPointer( const Point& rPos ) = 0;

	// Destruktor wird vom DropManager gerufen, wenn D&D beendet wird
	// (egal, ob durch Drop oder Abbruch)
	//virtual ~DropSource() = 0;

};


// Klasse, um Drag&Drop aufzuhaengen
class DragManager
{
	friend class DragServer;
	friend class ImplDragDropListener;

	// Damit wir an das bOwnDragDrop heran kommen
	friend long ImplHandleMouseEvent( Window*, USHORT, BOOL, long, long, ULONG,USHORT, USHORT );

	// Daten von Execute Drag merken
	SotDataObjectRef	xData;
	USHORT				nOptions;
	DropSource* 		pDropSource;

	// Zustand waehrend des Draggens
	BOOL				bDrawPointerResult;
	Pointer 			aDragPointer;
	BOOL				bModifyPointer;
	MouseEvent			aLastMEvt;
	// KLAEREN: Bekomme ich von TH auch ein Leave oder nur Moves?
	Window* 			pLastTargetWindow;	// nur noetig, wenn keine Leaves von TH
	DropAction			eDropResult;
	BOOL				bOwnDragDrop;		// TRUE: In VCL gestartet, d.h. nicht vom System

	// AB: HACK, um den Kram schnell durchzubekommen
public:
	// Data in DragServer
	MultiItemObjectRef	mxDragServerData;

	// DragManager-Instanz holen/anlegen
	static DragManager* ImplGetDragManager( void );

private:

	// Ctor ist private, da nur in ExecuteDrag verwendet
	DragManager( void );

	// DragManager fuer Drag-Vorgang initialisieren
	void InitDrag( SotDataObject* pData_, USHORT nOptions_, DropSource* pDropSource_ );

public:
	// Drag&Drop beginnen
	static DropAction ExecuteDrag( SotDataObject* pData_, USHORT nOptions_, DropSource* pDropSource_=NULL );

	// Wenn ein Drag&Drop-Vorgang im Gange ist, liefert folgender Aufruf != NULL
	static DragManager* GetDragManager( void );

	const MouseEvent& GetLastMouseEvent() const { return aLastMEvt; }

	// EnableDragDrop an das System-Fenster durchreichen
	// TODO: Von Window::EnableDrop() aufrufen, Problem: Inkompatibel, da inline!
	static void SystemEnableDrop( SalFrame* pFrame, BOOL bEnable );

	// Liefert den Standard-Pointer fuer die DropAction
	static Pointer GetDefaultPointer( DropAction eAction );

	// Service: Standard-Pointer setzen, kann von DrawDragPointer gerufen werden
	void SetDragPointer( Pointer aDragPointer_ )
		{ aDragPointer = aDragPointer_; bModifyPointer = TRUE; }

	// Schnittstelle fuer System-Events
	void MouseMove( const MouseEvent& rMEvt, Window* pTargetWindow );
	void ButtonUp( const MouseEvent& rMEvt, Window* pTargetWindow );
	void Escape( Window* pTargetWindow );
	void KeyModChanged( USHORT nKeyCode );

	// SYSTEM_EXCHANGE: Aufzurufen, wenn beim Drag&Drop das App-Fenster verlassen wird.
	void AppWindowLeaved( void );

	BOOL isModifyPointer( void )
		{ return bModifyPointer; }
	Pointer GetDragPointer( void )
		{ return aDragPointer; }
};


#ifdef OLDDRAGDROP
// DragServer als Dummy fuer SourceCode-Kompatibilitaet

class Bitmap;
class GDIMetaFile;


//******************
//*** DragServer ***
//******************

class DragServer : public Exchange
{
public:
						DragServer();
						~DragServer();

	static void 		Clear();

	static void 		NewItem();
	static BOOL 		CopyString( const XubString& rStr );
	static BOOL 		CopyBitmap( const Bitmap& rBitmap );
	static BOOL 		CopyGDIMetaFile( const GDIMetaFile& rGDIMetaFile );
	static BOOL 		CopyPrivateData( void* pData );
	static BOOL 		CopyFile( const XubString& rStr );
	static BOOL 		CopyData( const void* pData, ULONG nLen,
								  ULONG nFormat, USHORT nTypeLen = 8 );
	static BOOL 		CopyRequest( ULONG nFormat );

	static USHORT		GetItemCount();
	static XubString	PasteString( USHORT nItem );
	static Bitmap		PasteBitmap( USHORT nItem );
	static BOOL 		PasteGDIMetaFile( USHORT nItem, GDIMetaFile& rGDIMetaFile );
	static void*		PastePrivateData( USHORT nItem );
	static XubString	PasteFile( USHORT nItem );
	static BOOL 		PasteData( USHORT nItem, void* pBuf,
								   ULONG nLen, ULONG nFormat );
	static ULONG		GetDataLen( USHORT nItem, ULONG nFormat );

	static USHORT		GetFormatCount( USHORT nItem );
	static ULONG		GetFormat( USHORT nItem, USHORT nPos );
	static BOOL 		HasFormat( USHORT nItem, ULONG nFormat );

	static USHORT		GetRequestItem();
	static ULONG		GetRequestFormat();
	static BOOL 		IsRequestName();

	static XubString	GetTargetPrinterName();
};

#endif // OLDDRAGDROP

// Hilfsmethode, um aktuelles Drag-Item zu erhalten
SotDataObject* ImplGetActiveItem();
SotDataObject* ImplGetItem( USHORT nItem );

#endif // _DRAG_HXX
