/*************************************************************************
 *
 *  $RCSfile: filedlg.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:04 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef _VCL_FILEDLG_HXX
#define _VCL_FILEDLG_HXX

#include "dialog.hxx"

//***************************************************************************

struct ControlChain_Impl;
class Control;

//***************************************************************************
// VclFileDialog
//***************************************************************************

class VclFileDialog : public ModalDialog
{
private:
	ControlChain_Impl*			_pUserControls;

protected:

	Link						_aOKHdl;
	Link						_aFileSelectHdl;
	Link						_aFilterSelectHdl;

	String						_aPath;
	String						_aDefExt;

	void						ReleaseOwnerShip( Window* pUserControl );

public:
	VclFileDialog( Window* pParent, const ResId& rResId );
	~VclFileDialog();

	// pure virtual
	virtual void				FileSelect() = 0;
	virtual void				FilterSelect() = 0;
	virtual void				AddFilter( const String& rFilter,
										   const String& rType,ULONG nFlags = 0 ) = 0;
	virtual void				AddFilter( const String& rFilter, const String& rType,
										   const String& rSysType, ULONG nFlags = 0 ) = 0;
	virtual void				RemoveFilter( const String& rFilter ) = 0;
	virtual void				RemoveAllFilter() = 0;
	virtual void				SetCurFilter( const String& rFilter ) = 0;
	virtual String				GetCurFilter() const = 0;
	virtual USHORT				GetFilterCount() const = 0;
	virtual const String&		GetFilterName( USHORT nPos ) const = 0;
	virtual const String&		GetFilterType( USHORT nPos ) const = 0;
	virtual ULONG				GetFilterFlags( USHORT nPos ) const = 0;

	// virtual
#if (SUPD < 540)
	virtual void
#else
	virtual BOOL
#endif
								AddControl( Window* pControl, BOOL bNewLine = FALSE );

	// inline
	inline void					SetPath( const String& rNewURL );
	inline const String&		GetPath() const;
	inline void					SetDefaultExt( const String& rExt );
	inline const String&		GetDefaultExt() const;
	inline void					SetOKHdl( const Link& rLink );
	inline const Link&			GetOKHdl() const;
	inline void					SetFileSelectHdl( const Link& rLink );
	inline const Link&			GetFileSelectHdl() const;
	inline void					SetFilterSelectHdl( const Link& rLink );
	inline const Link&			GetFilterSelectHdl() const;
};

//***************************************************************************

inline void VclFileDialog::SetPath( const String& rNewURL )
{
	_aPath = rNewURL;
}

//***************************************************************************

inline const String& VclFileDialog::GetPath() const
{
	return _aPath;
}

//***************************************************************************

inline void VclFileDialog::SetDefaultExt( const String& rExt )
{
	_aDefExt = rExt;
}

//***************************************************************************

inline const String& VclFileDialog::GetDefaultExt() const
{
	return _aDefExt;
}

//*****************************************************************************

inline void VclFileDialog::SetOKHdl
(
	const Link& rLink
)
{
	_aOKHdl = rLink;
}

//*****************************************************************************

inline const Link& VclFileDialog::GetOKHdl() const
{
	return _aOKHdl;
}

//*****************************************************************************

inline void VclFileDialog::SetFileSelectHdl
(
	const Link& rLink
)
{
	_aFileSelectHdl = rLink;
}

//*****************************************************************************

inline const Link& VclFileDialog::GetFileSelectHdl() const
{
	return _aFileSelectHdl;
}

//*****************************************************************************

inline void VclFileDialog::SetFilterSelectHdl
(
	const Link& rLink
)
{
	_aFilterSelectHdl = rLink;
}

//*****************************************************************************

inline const Link& VclFileDialog::GetFilterSelectHdl() const
{
	return _aFilterSelectHdl;
}


#endif

