/*************************************************************************
 *
 *  $RCSfile: fixed.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:04 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_FIXED_HXX
#define _SV_FIXED_HXX

#ifndef _SV_SV_H
#include <sv.h>
#endif
#ifndef _SV_BITMAP_HXX
#include <bitmap.hxx>
#endif
#ifndef _SV_IMAGE_HXX
#include <image.hxx>
#endif
#ifndef _SV_CTRL_HXX
#include <ctrl.hxx>
#endif

// -------------
// - FixedText -
// -------------

class FixedText : public Control
{
#if __PRIVATE
#ifdef _SV_FIXED_CXX
private:
	void			ImplInit( Window* pParent, WinBits nStyle );
	WinBits 		ImplInitStyle( WinBits nStyle );
	void			ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
	void			ImplDraw( OutputDevice* pDev, ULONG nDrawFlags,
							  const Point& rPos, const Size& rSize );
#endif
public:
	static USHORT	ImplGetTextStyle( WinBits nWinBits );
#endif

public:
					FixedText( Window* pParent, WinBits nStyle = 0 );
					FixedText( Window* pParent, const ResId& rResId );

	virtual void	Paint( const Rectangle& rRect );
	virtual void	Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
	virtual void	Resize();
	virtual void	StateChanged( StateChangedType nType );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

	Size			CalcMinimumSize( long nMaxWidth = 0 ) const;
};

// -------------
// - FixedLine -
// -------------

class FixedLine : public Control
{
#if __PRIVATE
#ifdef _SV_FIXED_CXX
private:
	void			ImplInit( Window* pParent, WinBits nStyle );
	WinBits 		ImplInitStyle( WinBits nStyle );
	void			ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
#endif
#endif

public:
					FixedLine( Window* pParent, WinBits nStyle = WB_HORZ );
					FixedLine( Window* pParent, const ResId& rResId );

	virtual void	Paint( const Rectangle& rRect );
	virtual void	Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
	virtual void	Resize();
	virtual void	StateChanged( StateChangedType nType );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );
};

// ---------------
// - FixedBitmap -
// ---------------

class FixedBitmap : public Control
{
private:
	Bitmap			maBitmap;

#if __PRIVATE
#ifdef _SV_FIXED_CXX
private:
	void			ImplInit( Window* pParent, WinBits nStyle );
	WinBits 		ImplInitStyle( WinBits nStyle );
	void			ImplInitSettings();
	void			ImplDraw( OutputDevice* pDev, ULONG nDrawFlags,
							  const Point& rPos, const Size& rSize );
#endif

protected:
	void			ImplLoadRes( const ResId& rResId );
#endif

public:
					FixedBitmap( Window* pParent, WinBits nStyle = 0 );
					FixedBitmap( Window* pParent, const ResId& rResId );
					~FixedBitmap();

	virtual void	Paint( const Rectangle& rRect );
	virtual void	Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
	virtual void	Resize();
	virtual void	StateChanged( StateChangedType nType );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );

	void			SetBitmap( const Bitmap& rBitmap );
	const Bitmap&	GetBitmap() const { return maBitmap; }
};

// --------------
// - FixedImage -
// --------------

class FixedImage : public Control
{
private:
	Image			maImage;
	BOOL			mbInUserDraw;

#if __PRIVATE
#ifdef _SV_FIXED_CXX
private:
	void			ImplInit( Window* pParent, WinBits nStyle );
	WinBits 		ImplInitStyle( WinBits nStyle );
	void			ImplInitSettings();
	void			ImplDraw( OutputDevice* pDev, ULONG nDrawFlags,
							  const Point& rPos, const Size& rSize );
#endif

protected:
	void			ImplLoadRes( const ResId& rResId );
#endif

public:
					FixedImage( Window* pParent, WinBits nStyle = 0 );
					FixedImage( Window* pParent, const ResId& rResId );
					~FixedImage();

	virtual void	Paint( const Rectangle& rRect );
	virtual void	Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
	virtual void	Resize();
	virtual void	StateChanged( StateChangedType nType );
	virtual void	DataChanged( const DataChangedEvent& rDCEvt );
	virtual void	UserDraw( const UserDrawEvent& rUDEvt );

	void			SetImage( const Image& rImage );
	const Image&	GetImage() const { return maImage; }

	Point			CalcImagePos( const Point& rPos,
								  const Size& rObjSize, const Size& rWinSize );
};

#endif	// _SV_FIXED_HXX
