/*************************************************************************
 *
 *  $RCSfile: outdev.h,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_OUTDEV_H
#define _SV_OUTDEV_H

#ifndef _SV_SV_H
#include <sv.h>
#endif
#ifndef _SV_OUTFONT_HXX
#include <outfont.hxx>
#endif

class Size;
class Font;
class VirtualDevice;
class ImplServerGraphics;

// -----------------------
// - ImplDevFontListData -
// -----------------------

struct ImplDevFontListData
{
	XubString			maName; 					// Fontname (FamilyName)
	XubString			maMatchName;				// Fontname in Lower-Chars
	XubString			maMatchName2;				// Fontname in Lower-Chars without symbols
	ImplFontData*		mpFirst;					// First FontData
#if 0
	BOOL				mbScalable; 				// TRUE: Scalable Ausfuerhung vorhanden
	// This data is only set, after InitMatchData() is called
	FontFamily			meMatchFamily;				// Family vom Font
	FontWeight			meMatchWeight;				// Weight vom Font
	FontWidth			meMatchWidth;				// Width vom Font
	ULONG				mnMatchType;				// Type vom Font
#else
	FontFamily			meMatchFamily;				// Family vom Font
	FontPitch			meMatchPitch;				// Pitch vom Font
	USHORT				mnMatch;					// Match-Gewichtung
	BOOL				mbSymbol;					// TRUE: Font is a symbol font
#endif
};


// --------------------
// - ImplKernInfoData -
// --------------------

struct ImplKernInfoData
{
	USHORT				maFirstAry[256];
	USHORT				maLastAry[256];
};


// ---------------------
// - ImplWidthInfoData -
// ---------------------

struct ImplWidthInfoData
{
	USHORT				mnChar;
	long				mnWidth;
};


// -----------------
// - ImplFontEntry -
// ------------------

#define CHAR_NOKERNING		((ULONG)0xFFFFFFFF)
#define CHARCACHE_STD		128
#define WIDTHARY_INIT		256
#define WIDTHARY_RESIZE 	256

struct ImplFontEntry
{
	ImplFontEntry*		mpNext; 					// Naechster FontEntry
	ImplFontSelectData	maFontSelData;				// FontSelectionData
	ImplFontMetricData	maMetric;					// Font-Metric
	long				mnLineHeight;				// FontHoehe
	long				maWidthAry[CHARCACHE_STD];	// Breiten-Tabelle fuer Standard-Zeichen
	long				mnWidthFactor;				// Breiten-Faktor fuer Zeichenbreiten
	ImplWidthInfoData*	mpWidthAry; 				// Array mit Fontbreiten von Zeichen > MAXCHAR_CACHE
	ImplKernPairData*	mpKernPairs;				// Kerning-Paare
	ImplKernInfoData*	mpKernInfo; 				// Tabelle fuer schnelles Kerning
	ULONG				mnKernPairs;				// Anzahl Kerning-Paare
	USHORT				mnWidthAryCount;			// Anzahl Fontbreiten im Array
	USHORT				mnWidthArySize; 			// Groesse von Fontbreiten-Array
	USHORT				mnRefCount; 				// Anzahl-Referenzen
	USHORT				mnSetFontFlags; 			// Flags von SetFont()
	short				mnOwnOrientation;			// Drehung, falls System nicht dreht
	short				mnOrientation;				// Drehung vom Text
	BOOL				mbInit; 					// Wurden Daten initalisiert
	BOOL				mbInitKernPairs;			// Wurden Kerning-Daten initialisiert
	BOOL				mbFixedFont;				// Ist der Font ein FixedFont

						~ImplFontEntry();
};

// ----------------------
// - ImplGetDevFontList -
// ----------------------

class ImplGetDevFontList : private List
{
public:
							ImplGetDevFontList() :
								List( 4096, 512, 64 )
							{}

	void					Add( ImplFontData* pData )
								{ List::Insert( (void*)pData, LIST_APPEND ); }
	ImplFontData*			Get( ULONG i ) const
								{ return (ImplFontData*)List::GetObject( i ); }
	ULONG					Count() const { return List::Count(); }
};

// ----------------------
// - ImplGetDevSizeList -
// ----------------------

class ImplGetDevSizeList : public List
{
private:
	XubString			maFontName; 	// Name des Fonts, zu dem die Groessen gehoeren

public:
						ImplGetDevSizeList( const XubString& rName ) :
							List( 1024, 16, 16 ),
							maFontName( rName )
						{}

	void				Add( long nNewHeight );
	long				Get( ULONG nIndex ) const
							{ return (long)List::GetObject( nIndex ); }

	void				SetFontName( const XubString& rName ) { maFontName = rName; }
	const XubString&	GetFontName() const { return maFontName; }
};

// -----------------
// - ImplFontCache -
// -----------------

#define MAXFONT_CACHE	48

class ImplFontCache
{
private:
	ImplFontEntry*		mpFirstEntry;
	USHORT				mnRef0Count;
	BOOL				mbPrinter;
	BOOL				mbDummy;

public:
						ImplFontCache( BOOL bPrinter );
						~ImplFontCache();

	ImplFontEntry*		Get( ImplDevFontList* pFontList,
							 const Font& rFont, const Size& rSize );
	void				Release( ImplFontEntry* pEntry );
	void				Clear();
};

// ------------------
// - ImplOutDevData -
// ------------------

struct ImplOutDevData
{
	VirtualDevice*		mpRotateDev;
};

// -----------------
// - Hilfsmethoden -
// -----------------

inline void* ImplGetStackBuffer( ULONG nSize, void* pStackBuffer, ULONG nBufSize )
{
	if ( nSize <= nBufSize )
		return pStackBuffer;
	else
		return new BYTE[nSize];
}

inline void ImplReleaseStackBuffer( void* pBuffer, void* pStackBuffer )
{
	if ( pBuffer != pStackBuffer )
		delete pBuffer;
}

void ImplFreeOutDevFontData();

#endif // _SV_OUTDEV_H
