/*************************************************************************
 *
 *  $RCSfile: print.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:05 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_PRINT_HXX
#define _SV_PRINT_HXX

#ifndef _ERRCODE_HXX
#include <tools/errcode.hxx>
#endif

#ifndef _SV_SV_H
#include <sv.h>
#endif
#ifndef _SV_OUTDEV_HXX
#include <outdev.hxx>
#endif
#ifndef _SV_PRNTYPES_HXX
#include <prntypes.hxx>
#endif
#ifndef _SV_JOBSET_HXX
#include <jobset.hxx>
#endif
#ifndef _SV_GDIMTF_HXX
#include <gdimtf.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

struct SalPrinterInfoQueue;
class SalInfoPrinter;
class SalPrinter;
class VirtualDevice;
class Window;
class ImplQPrinter;
class RmPrinter;

namespace com { namespace sun { namespace star { namespace uno {
	class Any;
} } } }

// -----------------
// - Printer-Types -
// -----------------

#define PAGEQUEUE_ALLPAGES	 0xFFFF

enum PrinterSupport { SUPPORT_SET_ORIENTATION, SUPPORT_SET_PAPERBIN,
					  SUPPORT_SET_PAPERSIZE, SUPPORT_SET_PAPER,
					  SUPPORT_COPY, SUPPORT_COLLATECOPY,
					  SUPPORT_SETUPDIALOG };

// ---------------
// - Seq-Defines -
// ---------------

#ifdef _RMPRINTER_EXT

#define PRNSEQ_GET( _def_Seq, _def_Obj )															\
{																									\
	SvMemoryStream* _def_pStm = new SvMemoryStream( (char*)_def_Seq.getArray(), _def_Seq.getLength(), STREAM_READ );	\
	_def_pStm->SetCompressMode( COMPRESSMODE_FULL );												\
	*_def_pStm >> _def_Obj; 																		\
	delete _def_pStm;																				\
}

// ------------------------------------------------------------------------

#define PRNSEQ_SET( _def_Seq, _def_Obj )										\
{																				\
	SvMemoryStream* _def_pStm = new SvMemoryStream( 8192, 8192 );				\
	_def_pStm->SetCompressMode( COMPRESSMODE_FULL );							\
	*_def_pStm << _def_Obj; 													\
	_def_Seq = ::com::sun::star::uno::Sequence< sal_Int8 >( (sal_Int8*) _def_pStm->GetData(), _def_pStm->Tell() );	\
	delete _def_pStm;															\
}

// ---------------
// - PrinterPage -
// ---------------

class PrinterPage
{
	GDIMetaFile*	mpMtf;
	JobSetup		maJobSetup;
	UINT16			mbNewJobSetup;

public:

					PrinterPage() : mpMtf( new GDIMetaFile() ) {}
					PrinterPage( GDIMetaFile* pMtf, BOOL bNewJobSetup, const JobSetup& rSetup ) :
						mpMtf( pMtf ), mbNewJobSetup( bNewJobSetup ), maJobSetup( rSetup ) {}
					~PrinterPage() { delete mpMtf; }

	GDIMetaFile*	GetGDIMetaFile() const { return mpMtf; }
	const JobSetup& GetJobSetup() const { return maJobSetup; }
	BOOL			IsNewJobSetup() const { return (mbNewJobSetup != 0); }

	friend SvStream& operator<<( SvStream& rOStm, const PrinterPage& rPage )
						{ rOStm << rPage.mbNewJobSetup; rOStm << rPage.maJobSetup; rPage.mpMtf->Write( rOStm ); return rOStm; }
	friend SvStream& operator>>( SvStream& rIStm, PrinterPage& rPage )
						{ return rIStm >> rPage.mbNewJobSetup >> rPage.maJobSetup >> *rPage.mpMtf; }
};

#endif // _RMPRINTER_EXT

// -------------
// - QueueInfo -
// -------------

class QueueInfo
{
	friend class Printer;

private:
	XubString					maPrinterName;
	XubString					maDriver;
	XubString					maLocation;
	XubString					maComment;
	ULONG						mnStatus;
	ULONG						mnJobs;

public:
								QueueInfo();
								QueueInfo( const QueueInfo& rInfo );
								~QueueInfo();

	const XubString&			GetPrinterName() const { return maPrinterName; }
	const XubString&			GetDriver() const { return maDriver; }
	const XubString&			GetLocation() const { return maLocation; }
	const XubString&			GetComment() const { return maComment; }
	ULONG						GetStatus() const { return mnStatus; }
	ULONG						GetJobs() const { return mnJobs; }

	const QueueInfo&			operator==( const QueueInfo& rInfo );

	friend SvStream&			operator>>( SvStream& rIStream, QueueInfo& rInfo );
	friend SvStream&			operator<<( SvStream& rOStream, const QueueInfo& rInfo );
};


// -----------
// - Printer -
// -----------

struct PrintSpooler;

class Printer : public OutputDevice
{
	friend class OutputDevice;
	friend class ImplQPrinter;

private:
#ifndef REMOTE_APPSERVER
	SalInfoPrinter* 			mpInfoPrinter;
	SalPrinter* 				mpPrinter;
	Printer*					mpJobPrinter;
	SalGraphics*				mpJobGraphics;
#else
	RmPrinter*					mpInfoPrinter;
	PrintSpooler*				mpPrinter;
	void*						mpJobPrinter;
	void*						mpJobGraphics;
#endif
	Printer*					mpPrev;
	Printer*					mpNext;
	VirtualDevice*				mpDisplayDev;
	ImplQPrinter*				mpQPrinter;
	GDIMetaFile*				mpQMtf;
	void*						mpDummy1;
	void*						mpDummy2;
	void*						mpDummy3;
	XubString					maPrinterName;
	XubString					maDriver;
	XubString					maPrintFile;
	XubString					maJobName;
	JobSetup					maJobSetup;
	Point						maPageOffset;
	Size						maPaperSize;
	ULONG						mnError;
	USHORT						mnCurPage;
	USHORT						mnCurPrintPage;
	USHORT						mnPageQueueSize;
	USHORT						mnCopyCount;
	BOOL						mbDefPrinter;
	BOOL						mbPrinting;
	BOOL						mbJobActive;
	BOOL						mbCollateCopy;
	BOOL						mbPrintFile;
	BOOL						mbInPrintPage;
	BOOL						mbNewJobSetup;
	BOOL						mbIsQueuePrinter;
	BOOL						mbUserSetupCompleted;
	BOOL						mbUserSetupResult;
	BOOL						mbDummy3;
	Link						maErrorHdl;
	Link						maStartPrintHdl;
	Link						maEndPrintHdl;
	Link						maPrintPageHdl;

#ifdef REMOTE_APPSERVER
	DECL_LINK( UserSetupCompleted, ::com::sun::star::uno::Any* );
#endif

#if __PRIVATE
#ifdef _SV_PRINT_CXX
	void						ImplInitData();
	void						ImplInit( SalPrinterQueueInfo* pInfo );
	void						ImplInitDisplay( const Window* pWindow );
	static SalPrinterQueueInfo* ImplGetQueueInfo( const XubString& rPrinterName,
												  const XubString* pDriver );
	void						ImplUpdatePageData();
	void						ImplUpdateFontList();
#endif // _SV_PRINT_CXX
#ifndef REMOTE_APPSERVER

								DECL_LINK( ImplDestroyPrinterAsync, void* );
public:

	void						ImplEndPrint();

#endif // REMOTE_APPSERVER
#endif // __PRIVATE

#ifdef _SPOOLPRINTER_EXT
public:
	void						GetPreparedMetaFile( const GDIMetaFile& rInMtf, GDIMetaFile& rOutMtf, ULONG nFlags = 0 );
#endif // _SPOOLPRINTER_EXT

protected:

	void						SetSelfAsQueuePrinter( BOOL bQueuePrinter ) { mbIsQueuePrinter = bQueuePrinter; }
	BOOL						IsQueuePrinter() const { return mbIsQueuePrinter; }

public:
								Printer();
								Printer( const Window* pWindow );
								Printer( const JobSetup& rJobSetup );
								Printer( const QueueInfo& rQueueInfo );
								Printer( const XubString& rPrinterName );
	virtual 					~Printer();

	static USHORT				GetQueueCount();
	static const QueueInfo& 	GetQueueInfo( USHORT nQueue );
	static const QueueInfo& 	GetQueueInfo( USHORT nQueue, BOOL bStatus );
	static XubString			GetDefaultPrinterName();

	virtual void				Error();
	virtual void				StartPrint();
	virtual void				EndPrint();
	virtual void				PrintPage();

	const XubString&			GetName() const 			{ return maPrinterName; }
	const XubString&			GetDriverName() const		{ return maDriver; }
	BOOL						IsDefPrinter() const		{ return mbDefPrinter; }
	BOOL						IsDisplayPrinter() const	{ return mpDisplayDev != NULL; }
	BOOL						IsValid() const 			{ return !IsDisplayPrinter(); }

	ULONG						GetCapabilities( USHORT nType ) const;
	BOOL						HasSupport( PrinterSupport eFeature,
											BOOL bInJob = FALSE ) const;

	BOOL						SetJobSetup( const JobSetup& rSetup );
	const JobSetup& 			GetJobSetup() const { return maJobSetup; }
	BOOL						Setup( Window* pWindow = NULL );
	BOOL						SetPrinterProps( const Printer* pPrinter );

	BOOL						SetOrientation( Orientation eOrient );
	Orientation 				GetOrientation() const;
	BOOL						SetPaperBin( USHORT nPaperBin );
	USHORT						GetPaperBin() const;
	BOOL						SetPaper( Paper ePaper );
	BOOL						SetPaperSizeUser( const Size& rSize );
	Paper						GetPaper() const;

	USHORT						GetPaperBinCount() const;
	XubString					GetPaperBinName( USHORT nPaperBin ) const;

	const Size& 				GetPaperSizePixel() const { return maPaperSize; }
	Size						GetPaperSize() const { return PixelToLogic( maPaperSize ); }
	const Point&				GetPageOffsetPixel() const { return maPageOffset; }
	Point						GetPageOffset() const { return PixelToLogic( maPageOffset ); }

	BOOL						SetCopyCount( USHORT nCopy, BOOL bCollate = FALSE );
	USHORT						GetCopyCount() const { return mnCopyCount; }
	BOOL						IsCollateCopy() const { return mbCollateCopy; }

	USHORT						GetCurPrintPage() const { return mnCurPrintPage; }
	BOOL						IsPrinting() const { return mbPrinting; }

	void						SetPrintFile( const XubString& rFileName ) { maPrintFile = rFileName; }
	const XubString&			GetPrintFile() const { return maPrintFile; }
	void						EnablePrintFile( BOOL bEnable ) { mbPrintFile = bEnable; }
	BOOL						IsPrintFileEnabled() const { return mbPrintFile; }
	BOOL						StartJob( const XubString& rJobName );
	BOOL						EndJob();
	BOOL						AbortJob();
	const XubString&			GetCurJobName() const { return maJobName; }
	USHORT						GetCurPage() const { return mnCurPage; }
	BOOL						IsJobActive() const { return mbJobActive; }
	BOOL						StartPage();
	BOOL						EndPage();

	void						SetPageQueueSize( USHORT nPages ) { mnPageQueueSize = nPages; }
	USHORT						GetPageQueueSize() const { return mnPageQueueSize; }

	ULONG						GetError() const { return ERRCODE_TOERROR(mnError); }
	ULONG						GetErrorCode() const { return mnError; }

	void						SetErrorHdl( const Link& rLink ) { maErrorHdl = rLink; }
	const Link& 				GetErrorHdl() const { return maErrorHdl; }
	void						SetStartPrintHdl( const Link& rLink ) { maStartPrintHdl = rLink; }
	const Link& 				GetStartPrintHdl() const { return maStartPrintHdl; }
	void						SetEndPrintHdl( const Link& rLink ) { maEndPrintHdl = rLink; }
	const Link& 				GetEndPrintHdl() const	 { return maEndPrintHdl;   }
	void						SetPrintPageHdl( const Link& rLink ) { maPrintPageHdl = rLink; }
	const Link& 				GetPrintPageHdl() const  { return maPrintPageHdl;  }
};

#endif	// _SV_PRINT_HXX
