/*************************************************************************
 *
 *  $RCSfile: rmprint.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_RMPRINT_HXX
#define _SV_RMPRINT_HXX

#include "rvp.hxx"

#define RVP_CLIENT_SERVER_NAME "[Client]"

BOOL ImplAddRemotePrinter( const UniString& rPrinterName, const UniString& rServerName, BOOL bSetDefault, BOOL bIsLocal  );

// -------------------------
// - RemotePrintServerInfo -
// -------------------------

// forward declaration (see atom.hxx)

namespace vcl {
	class AtomClient;
}

struct RemotePrintServerInfo
{
	UniString													aServerName;
	REF( ::com::sun::star::lang::XMultiServiceFactory )			xFactory;
	REF( NMSP_CLIENT::XRmPrinterEnvironment )					xRmPrinterEnv;
	::vcl::AtomClient*											pAtoms;


	RemotePrintServerInfo( const UniString& rServerName ) :
		aServerName( rServerName ), pAtoms( NULL ) {}

	~RemotePrintServerInfo()
		{
			// ensure correct sequence of object destruction
			xRmPrinterEnv = REF( NMSP_CLIENT::XRmPrinterEnvironment )();
			xFactory = REF( ::com::sun::star::lang::XMultiServiceFactory )();
		}
};

// ---------------------
// - RemotePrinterInfo -
// ---------------------

struct RemotePrinterInfo
{
	UniString						aPrinterName;
	UniString						aServerName;
	BOOL							bDefaultPrinter;
	BOOL							bLocalPrinter;

	RemotePrinterInfo( const String& rPName, const String& rSName, BOOL bDef, BOOL bLocal ) :
		aPrinterName( rPName ), aServerName( rSName ), bDefaultPrinter( bDef ), bLocalPrinter( bLocal ) {}

	UniString GetFullName() const
	{
		UniString aFullName( aPrinterName );
		
		if( !bLocalPrinter )
		{
			aFullName += UniString( OUSTRING::createFromAscii( "@" ) );
			aFullName += aServerName;
		}

		return aFullName;
	}
};

DECLARE_LIST( RemotePrintServerInfoList, RemotePrintServerInfo* );
DECLARE_LIST( RemotePrinterInfoList, RemotePrinterInfo* );

// ---------------------
// - RemotePrinterList -
// ---------------------

class UnoDispatcher;

class RemotePrinterList
{
private:

	REF( NMSP_CLIENT::XRmPrinterObserver )				mxPrinterObserver;
	REF( ::com::sun::star::lang::XMultiServiceFactory ) mxServiceFactory;
	RemotePrintServerInfoList							maRemotePrintServers;
	RemotePrinterInfoList								maRemotePrinters;
	UnoDispatcher*										mpDispatcher;

protected:
	
	RemotePrintServerInfo*						ImplFindPrintServer( const UniString& rServer ) const;
	void										ImplInitPrintServer( RemotePrintServerInfo* pInf );
												
public:											
												RemotePrinterList( const REF( ::com::sun::star::lang::XMultiServiceFactory )& xServiceFactory );
												~RemotePrinterList();
												
	BOOL										AddPrinter( const UniString& rPrinterName, const UniString& rServerName, 
															BOOL bSetDefault, BOOL bIsLocal );
												
	ULONG										GetPrinterCount() const { return maRemotePrinters.Count(); }
	RemotePrinterInfo*							GetPrinter( ULONG n ) const { return maRemotePrinters.GetObject(n); }
	RemotePrinterInfo*							GetPrinter( const String& rPrinterName, const String& rServerName );
	RemotePrinterInfo*							GetDefaultPrinter() const;
	UniString									FindLocalPrintServer( const UniString& rPrinterName );

	REF( ::com::sun::star::lang::XMultiServiceFactory ) GetServerFactory( const String& rServerName );
	::vcl::AtomClient* GetServerAtoms( const String& rServerName );
	REF( NMSP_CLIENT::XRmPrinterEnvironment )	GetPrinterEnv( const UniString& rServerName );
	void CreateNewPrinterConnection( const String& rServerName, REF( ::com::sun::star::lang::XMultiServiceFactory )& xFactory, REF( ::com::sun::star::connection::XConnection )& xConnection, REF( ::com::sun::star::bridge::XBridgeFactory )& xBridgeFactory );
	void CreateNewPrinterConnection( const String& rServerName, REF( ::com::sun::star::lang::XMultiServiceFactory )& xFactory )
		{
			REF( ::com::sun::star::connection::XConnection ) xConnection;
			REF( ::com::sun::star::bridge::XBridgeFactory ) xBridgeFactory;
			CreateNewPrinterConnection( rServerName, xFactory, xConnection, xBridgeFactory );
		}
};

// ----------------------
// - ORmPrinterObserver -
// ----------------------

class ORmPrinterObserver : public ::cppu::WeakImplHelper1< NMSP_CLIENT::XRmPrinterObserver >
{
public:

				
							ORmPrinterObserver() {};
							~ORmPrinterObserver() {};
	
	virtual void SAL_CALL	PrinterSettingsChanged();
};

// ----------------
// - PrintSpooler -
// ----------------

struct PrintSpooler
{
	REF( NMSP_CLIENT::XRmPrintSpooler )		mxPrintSpooler;
};

// -------------
// - RmPrinter -
// -------------

class Printer;

class RmPrinter
{
private:

	REF( NMSP_CLIENT::XRmPrinter )			mxRemotePrinter;
	Printer*								mpPrinter;
									
											RmPrinter( const RmPrinter& ) {}
	RmPrinter&								operator=( const RmPrinter& ) { return *this; }
										
public:									
										
											RmPrinter() : mpPrinter( NULL ) {};
										
	void									Create( Printer* pPrinter );
	BOOL									IsValid() const { return mxRemotePrinter.is(); }

	const REF( NMSP_CLIENT::XRmPrinter )&	GetInterface() const { return mxRemotePrinter; }
	void									SetInterface( const REF( NMSP_CLIENT::XRmPrinter )& rxPrinter ) { mxRemotePrinter = rxPrinter; }

public:

    void									Create( NMSP_CLIENT::RmQueueInfo& aRmQueueInfo, NMSP_CLIENT::RmJobSetup& rRmJobSetup );
    BOOL									SetJobSetup( NMSP_CLIENT::RmJobSetup& rRmJobSetup );
    BOOL									SetOrientation( USHORT nOrientation, NMSP_CLIENT::RmJobSetup& rRmJobSetup );
    BOOL									SetPaperBin( USHORT nPaperBin, NMSP_CLIENT::RmJobSetup& rRmJobSetup );
    BOOL									SetPaper( USHORT nPaper, NMSP_CLIENT::RmJobSetup& rRmJobSetup );
    BOOL									SetPaperSizeUser( long nWidth, long nHeight, NMSP_CLIENT::RmJobSetup& rRmJobSetup );
    void									GetPageInfo( long& rOutWidth, long& rOutHeight, long& rPageOffX, long& rPageOffY, 
														 long& rPageWidth, long& rPageHeight );
    USHORT									GetPaperBinCount();
    OUSTRING								GetPaperBinName( USHORT nPaperBin );

	unsigned long							GetCapabilities( USHORT nType );
	void									UserSetup( const REF( NMSP_CLIENT::XRmFrameWindow )& xParent, sal_uInt32 EventID );
};

#endif // _SV_RMPRINT_HXX
