/*************************************************************************
 *
 *  $RCSfile: sysexchg.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 10:23:07 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _SV_SYSEXCHG_HXX
#define _SV_SYSEXCHG_HXX

// Fuer enum DropAction
#ifndef _SV_EVENT_HXX //autogen
#include <event.hxx>
#endif

// Fuer Point
#ifndef _GEN_HXX //autogen
#include <tools/gen.hxx>
#endif

class SalFrame;
class String;


//*** Allgemeine Interfaces, d.h. sowohl fuer Clipboard als auch fuer D&D

/** 
 * Interface zur Format-Registrierung. <BR>
 * es wird sowohl der Format-Name als auch die Id uebergeben, unter der
 * das Format registriert werden muss. Auf diese Weise kann die VCL-
 * interne Registrierung unabhaengig von der System-Registrierung bleiben.
 */
class ISystemExchange
{
public:
    virtual void registerFormatName( const String& rName, UINT32 nFormatId ) = 0;
};


/** 
 * Interface zum Daten-Austausch. <BR>
 * Orientiert sich an der Datenschnittstelle des SotDataObject.
 * Zur Vereinfachung werden nur die grundlegendsten Methoden
 * unterstuetzt.
 */
class IDataTrans
{
public:
	// RefCounting !
	virtual void aquire( void ) = 0;
	virtual void release( void ) = 0;

    // interfaces must have a virtual destructor
	virtual ~IDataTrans() {};

	/**
	 * Liefert eine Liste der von der Interface-Instanz lieferbaren Typen. <BR> 
	 * Kann NULL liefern, wenn das Interface ungueltig oder leer ist, d.h. ueber
	 * keine Typen verfuegt. Der gelieferte Pointer muss vom Caller freigegeben
	 * werden (Eigentuemer-Uebergang!)
	 * Der erste Eintrag im Array ist das 'wichtigste' Format
	 */
    virtual UINT32*				getTypeList( /*Out*/UINT32& rSize ) const = 0;

	/**
	 * Liefert ...
	 * Kann NULL liefern, wenn ...
	 * Der gelieferte Pointer muss vom Caller freigegeben werden (Eigentuemer-Uebergang!)
	 */
    virtual	BYTE*				getData( UINT32 nFormat, /*Out*/UINT32& rSize ) = 0;

				// Ueberfluessig:
				//virtual BOOL    GetDataHere( XDataItem& );
				//virtual BOOL    QueryDataType( SvDataType & rInOutType );
};



//*** Drag&Drop-Interfaces

// Solange es keine UNO-Interfaces sind, kann das VCL-Enum verwendet werden
#define SysDropAction DropAction

// Die Werte entsprechen denen in vcl\event.hxx
// Wird erst fuer UNO gebraucht
/*
enum SysDropAction
{  
	DROP_NONE, DROP_CANCEL = DROP_NONE,
	DROP_MOVE=0x1, DROP_COPY=0x2, DROP_LINK=0x4,
	DROP_PRINT=0x8, DROP_DISCARD=0x10 
};
*/

/** 
 * Interface zur Entgegennahme von Drag&Drop-System-Events. <BR>
 */
class IDragDropListener
{
public:
	// Problem: pSysWindow abbilden, wer kann das in ein Window umwandeln?

	// Start des Drag&Drop, queryDrop() muss immer folgen!
	virtual void beginDragDrop( IDataTrans* pData, USHORT nSrcOptions ) = 0;

	// queryDrop() wird waehrend des System-Drag&Drop laufend gerufen
	// Wenn der Bereich der VCL-Applikation verlassen wird, d.h. das Target
	// kein Frame der VCL-Applikation mehr ist, muss diese Methode einmal
	// mit pSalFrame==NULL aufgerufen werden, damit die Implementation das
	// Verlassen des VCL-Bereichs bemerkt.
	virtual SysDropAction queryDrop( Point aMousePos, SalFrame* pSalFrame,
		USHORT nModKeys /* oder SysDropAction */ ) = 0;

	// drop() wird gerufen, wenn das D&D-Item auf der VCL-Applikation
	// fallengelassen wird.
	virtual SysDropAction drop( Point aMousePos, SalFrame* pSalFrame,
		USHORT nModKeys /* oder SysDropAction */ ) = 0;

	virtual void endDragDrop( void) = 0;

				// Laeuft ueber queryDrop: void KeyModChanged( USHORT nKeyCode );
};

/** 
 * Interface zur System-Drag&Drop-Schnittstelle. <BR>
 * Die XSystemDragManager-Implementation muss den RefCount der bei
 * bei executeDrag() uebergebenen IDataTrans-Instanz mit aquire() 
 * erhoehen und mit release() wieder runterzaehlen, sobald die
 * Drag&Drop-Operation beendet ist, d.h. vor der Ruckkehr aus
 * executeDrag().
 */
class ISystemDragManager
{
public:
	// Drag&Drop beginnen
	virtual SysDropAction executeDrag( IDataTrans* pData, USHORT nOptions ) = 0;

	// Listener fuer Drag&Drop-Events vom System anmelden
	virtual void registerListener( IDragDropListener& xListener ) = 0;

	// Drag&Drop fuer ein Fenster anmelden (von VCL durchreichen)
	virtual void enableDrop( SalFrame* pSalFrame, BOOL bEnable ) = 0;
};


//*** Clipboard-Interfaces

/** 
 * Interface zur Entgegennahme von Clipboard-System-Events. <BR>
 */
class IClipboardListener
{
public:
	virtual void cleared() = 0;
	virtual void changed() = 0;
};

/** 
 * Interface zum System-Clipboard. <BR>
 * Die XSystemClipboard-Implementation muss den RefCount der bei
 * bei copy() uebergebenen IDataTrans-Instanz mit aquire() erhoehen
 * und mit release() wieder runterzaehlen, sobald diese Daten
 * ungueltig werden. Dies ist der Fall, wenn das Clipboard
 * geloescht wird oder andere Daten in das Clipboard gestellt
 * werden. Dabei spielt es keine Rolle, ob die Aenderung bzw. das
 * Loeschen ueber das XSystemClipboard-Interface oder direkt vom
 * System aus erfolgt.
 */
class ISystemClipboard
{
public:
	virtual void copy( IDataTrans* pData ) = 0;
	virtual IDataTrans* paste() = 0;
	virtual void clear() = 0;

	// Listener fuer Drag&Drop-Events vom System anmelden
	virtual void registerListener( IClipboardListener& xListener ) = 0;
};


/**
 * Daten-Struktur fuer File-Listen (FORMAT_FILE_LIST)
 * Es wird die OLE/Windows-Struktur DROPFILES nachdefiniert
 * Typen entsprechend der Groesse der Original-Typen gewaehlt
 * In den Kommentaren sind jeweils die Original-Typen angegeben.
 * Hier werden immer die 32-Bit-Varianten der Typen gewaehlt, um
 * eine einheitliche Struktur fuer alle Plattformen zu erhalten.
 * Diese Struktor ist zwar nicht Win16-kompatibel, dort kann aber
 * die Struktur auch nicht direkt per Drag&Drop uebertragen werden.
 */
struct SysexchgFileList
{
    /*DWORD = unsigned long*/	UINT32 pFiles;	// offset of file list
    /*POINT = { long, long }*/	INT32 pt_x;     // drop point
								INT32 pt_y;
    /*BOOL = int*/				INT32 fNC;
    /*BOOL = int*/				INT32 fWide;	// wide character-flag

	// Konstruktor
	SysexchgFileList( void )
	{
		pFiles	= 20;	// 5 x 4 Bytes, sizeof gefaehrlich wegen alignment
		pt_x	= 0L;
		pt_y	= 0L;
		fNC		= FALSE;
		fWide	= FALSE;
	}

};

// Funktion die eine Instanz des System-Exchange-Interfaces anlegt, falls noch
// keine existiert.

void CreateSystemExchange();

#endif

