/*************************************************************************
 *
 *  $RCSfile: resmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:35 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <tools/resmgr.hxx>
#include <tools/urlobj.hxx>
#include <tools/intn.hxx>
#include <tools/simplerm.hxx>

#include <svapp.hxx>
#include <svdata.hxx>
#include <settings.hxx>

static XubString *pSharedPath = 0;

// =======================================================================

static XubString* ImplGetSharedFolderPath()
{
	INetURLObject aSharedDir( Application::GetAppFileName(), INET_PROT_FILE, INetURLObject::ENCODE_ALL );
	aSharedDir.CutLastName();	
	aSharedDir.Append( String( RTL_CONSTASCII_USTRINGPARAM( "shared" ) ) );
	aSharedDir.setFinalSlash();

	XubString aSharedPath = aSharedDir.getFSysPath( INetURLObject::FSYS_DETECT );
	return new XubString( aSharedPath );
}

// -----------------------------------------------------------------------

#ifdef UNX
#define SEARCH_DELIMITER ':'
#else
#define SEARCH_DELIMITER ';'
#endif

static void ImplGetResMgrPaths( XubString& rAppPath, XubString& rResPath )
{
	if ( !pSharedPath )
		pSharedPath = ImplGetSharedFolderPath();

	rAppPath = Application::GetAppFileName();
	rResPath = *pSharedPath;
	XubString aResPath2 = Application::GetResourcePath();
	if ( aResPath2.Len() )
	{
		rResPath += SEARCH_DELIMITER;
		rResPath += aResPath2;
	}
}

// =======================================================================

ResMgr::ResMgr( const XubString* pFileName )
{
	XubString aFileName;
	if ( pFileName )
		aFileName = *pFileName;

	XubString aAppName;
	XubString aResPath;
	ImplGetResMgrPaths( aAppName, aResPath );

	pImpRes = InternalResMgr::GetInternalResMgr( aFileName, &aAppName, &aResPath );
	Init( pImpRes->aFileName );
}

// -----------------------------------------------------------------------

const char* ResMgr::GetLang( USHORT nPrio )
{
	LanguageType nType = Application::GetSettings().GetInternational().GetLanguage();
	return GetLang( nType, nPrio );
}

// -----------------------------------------------------------------------

ResMgr* ResMgr::CreateResMgr( const sal_Char* pPrefixName, LanguageType eLanguage )
{
	if ( eLanguage == LANGUAGE_DONTKNOW )
		eLanguage = Application::GetSettings().GetInternational().GetLanguage();
	if ( eLanguage == LANGUAGE_SYSTEM )
		eLanguage = ::GetSystemLanguage();

	XubString aAppName;
	XubString aResPath;
	ImplGetResMgrPaths( aAppName, aResPath );

	return CreateResMgr( pPrefixName, eLanguage, &aAppName, &aResPath );
}

// =======================================================================

SimpleResMgr* SimpleResMgr::Create( const sal_Char* pPrefixName, LanguageType eLanguage )
{
	if ( eLanguage == LANGUAGE_DONTKNOW )
		eLanguage = Application::GetSettings().GetInternational().GetLanguage();
	if ( eLanguage == LANGUAGE_SYSTEM )
		eLanguage = ::GetSystemLanguage();

	XubString aAppName;
	XubString aResPath;
	ImplGetResMgrPaths( aAppName, aResPath );

	return new SimpleResMgr( pPrefixName, eLanguage, &aAppName, &aResPath );
}
