/*************************************************************************
 *
 *  $RCSfile: clip.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: pl $ $Date: 2000/10/04 14:28:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef W31
#include <tools/svwin.h>
#endif
#ifndef _SV_GETSYS_HXX
#include <getsys.hxx>
#endif
#ifndef _SV_EXCHANGE_H
#include <exchange.h>
#endif
#ifndef _SV_CLIP_HXX
#include <clip.hxx>
#endif
#ifndef _SV_BITMAP_HXX
#include <bitmap.hxx>
#endif
#ifndef _SV_SVDATA_HXX
#include <svdata.hxx>
#endif

#ifdef REMOTE_APPSERVER
#include "unoex.hxx"
#ifdef MAC
#include "macex.hxx"
#endif
#endif

// -----------
// - statics -
// -----------

#ifdef WNT
static sal_uInt32 g_nOwnObjects = 0;
#else
static BOOL g_OwnCopy = FALSE;
#endif

static void EmptySystemClipboard();
static void ImplUpdateSystemClipboard();

//********************
//*** Impl-Klassen ***
//********************

// ---------------
// - ImplClipTab -
// ---------------

class ImplClipTab : public ImpPtrTab
{
public:
                    ImplClipTab() : ImpPtrTab() {}

    void            Insert( VclClipboard* pClipboard ) { ImpPtrTab::Insert( (void*)pClipboard ); }
    void            Remove( VclClipboard* pClipboard ) { ImpPtrTab::Remove( (void*)pClipboard ); }
    void            Request();
    void            Cleared();
};

// ---------------
// - ClipManager -
// ---------------

class ClipManager
{
private:

    // Data in Clipboard
    SotDataObjectRef    mxClipData;

    // Wurden die Daten aus der eigenen Applikation bereitgestellt?
    BOOL                bOwnData;

public:
	
						ClipManager() : bOwnData( FALSE ) {}
	
    SotDataObject*      GetClipData( void ) { return mxClipData; }
    void                SetClipData( SotDataObject* pData, BOOL bOwnData_=TRUE ) { mxClipData = pData; bOwnData = bOwnData_; }
    BOOL                IsOwnData( void ) { return bOwnData; }

    // DragManager-Instanz holen/anlegen
    static ClipManager* ImplGetClipManager( void );
};

#ifdef REMOTE_APPSERVER
//*******************************************
//*** Schnittstelle fuer System-Anbindung ***
//*******************************************

// -------------------------------
// - UnoClipboardListenerAdaptor -
// -------------------------------

class UnoClipboardListenerAdaptor : public ::cppu::WeakImplHelper1< NMSP_SYSTEM::XClipboardListener >
{
protected:

        IClipboardListener&		mrListener;

public:

								UnoClipboardListenerAdaptor( IClipboardListener& rListener );
								~UnoClipboardListenerAdaptor();

    virtual void SAL_CALL		cleared() throw( NMSP_UNO::RuntimeException );
    virtual void SAL_CALL		changed() throw( NMSP_UNO::RuntimeException );
};

// -----------------------------------------------------------------------------

UnoClipboardListenerAdaptor::UnoClipboardListenerAdaptor( IClipboardListener& rListener ) :
	mrListener( rListener )
{
}

// -----------------------------------------------------------------------------

UnoClipboardListenerAdaptor::~UnoClipboardListenerAdaptor()
{
}

// -----------------------------------------------------------------------------

void SAL_CALL UnoClipboardListenerAdaptor::cleared() throw( NMSP_UNO::RuntimeException )
{
	mrListener.cleared();
}

// -----------------------------------------------------------------------------

void SAL_CALL UnoClipboardListenerAdaptor::changed() throw( NMSP_UNO::RuntimeException )
{
	mrListener.changed();
}

// SYSTEM_EXCHANGE

// ----------------------
// - UnoSystemClipboard -
// ----------------------

class UnoSystemClipboard : public ISystemClipboard
{
protected:

    REF( NMSP_SYSTEM::XSystemClipboard ) mxSystemClipboard;

public:
    
							UnoSystemClipboard( UnoSystemExchange* pSystemExchange );

    virtual void			copy(IDataTrans* pData) ;
    virtual IDataTrans*		paste();
    virtual void			clear();
    virtual void			registerListener( IClipboardListener& rListener );

    BOOL					isAvailable() { return mxSystemClipboard.is(); }
};

// -----------------------------------------------------------------------------

UnoSystemClipboard::UnoSystemClipboard( UnoSystemExchange* pSystemExchange )
{
    if( pSystemExchange && pSystemExchange->isAvailable() )
		mxSystemClipboard = REF( NMSP_SYSTEM::XSystemClipboard )( pSystemExchange->getInterface(), NMSP_UNO::UNO_QUERY );
}

// -----------------------------------------------------------------------------

void UnoSystemClipboard::copy( IDataTrans* pData )
{
	const REF( NMSP_UNO::XInterface ) xIf( static_cast< ::cppu::OWeakObject* >( new UnoDataTransAdaptor( pData ) ) );

	mxSystemClipboard->copy( REF( NMSP_SYSTEM::XDataTransfer )( xIf, NMSP_UNO::UNO_QUERY ) );
}

// -----------------------------------------------------------------------------

IDataTrans* UnoSystemClipboard::paste()
{
	return( isAvailable() ? new UnoDataTrans( mxSystemClipboard->paste() ) : NULL );
}

// -----------------------------------------------------------------------------

void UnoSystemClipboard::clear()
{
    if( isAvailable() )
        mxSystemClipboard->clear();
}

// -----------------------------------------------------------------------------

void UnoSystemClipboard::registerListener( IClipboardListener& rListener )
{
	const REF( NMSP_UNO::XInterface ) xIf( static_cast< ::cppu::OWeakObject* >( new UnoClipboardListenerAdaptor( rListener ) ) );

    mxSystemClipboard->registerListener( REF( NMSP_SYSTEM::XClipboardListener )( xIf, NMSP_UNO::UNO_QUERY ) );
}

#endif

//*******************************************************************************************

ISystemClipboard* GetSystemClipboard( void )
{
#ifdef REMOTE_APPSERVER
#ifdef MAC
	return GetMacClipboard();
#else
    static UnoSystemClipboard *pSystemClipboard = NULL;

    if (pSystemClipboard == NULL)
    {
        UnoSystemExchange *pExchange = UnoSystemExchange::getSystemExchange();

        if (pExchange->isAvailable())
            pSystemClipboard = new UnoSystemClipboard(pExchange);
        else
            return NULL;
    }

    return pSystemClipboard->isAvailable() ? pSystemClipboard : NULL;
#endif
#else
	return NULL;
#endif
}

//*******************************************************************************************


void InvalidateSystemClipboard()
{
    ClipManager *pClipManager = ClipManager::ImplGetClipManager();
	
    // Daten loslassen wenn wir sie nicht selber reingestellt haben, 
	// damit ggf. neue vom System gefordert werden
	if (pClipManager && !pClipManager->IsOwnData())
		pClipManager->SetClipData( NULL );
}

/**
 * Interface zur Entgegennahme von Clipboard-System-Events. <BR>
 */
class ImplClipboardListener : public IClipboardListener
{
public:
    virtual void cleared();
    virtual void changed();
};

void ImplClipboardListener::cleared()
{
    // Clear-Handler aufrufen
    ImplSVData* pSVData = ImplGetSVData();
    if( pSVData->maExchangeData.mpClipTab )
        pSVData->maExchangeData.mpClipTab->Cleared();

    // Daten loslassen
    ClipManager::ImplGetClipManager()->SetClipData( NULL );

    // ACHTUNG: VclClipboard::Clear() darf NICHT gerufen
    // werden, da es sonst eine Rekursion gibt.
}

void ImplClipboardListener::changed()
{
    // Daten loslassen, damit ggf. neue vom System gefordert werden
#ifdef WNT
	if ( --g_nOwnObjects == 0 )
#else
	if ( !g_OwnCopy )
#endif
		ClipManager::ImplGetClipManager()->SetClipData( NULL );
}

// System-Clipboard holen
ISystemClipboard* ImplGetSystemClipboard( void )
{
    static BOOL bRegisteredListener = FALSE;

    ISystemClipboard* pRet = GetSystemClipboard();

    // Formate registrieren, wenn es ein System gibt
    if( pRet )
    {
        Exchange::ImplRegisterSystemFormats();

        // Listener schon angemeldet? Sonst jetzt machen
        if( !bRegisteredListener )
        {
            pRet->registerListener( *(new ImplClipboardListener()) );
            bRegisteredListener = TRUE;
        }
    }

    return pRet;
}

// Eigene DTrans-Klasse, um VCL-Clipboard beim release zu loeschen
class ImplClipDTrans : public ImplDTrans
{
protected:
    virtual         ~ImplClipDTrans();
public:
    ImplClipDTrans( SotDataObject* pDataObj )
        : ImplDTrans( pDataObj ) {}
};

// In dieser Basisklasse gibt es nichts zu tun
ImplClipDTrans::~ImplClipDTrans()
{
    // Altes Konzept, jetzt ueber ClipboardListener
    // VclClipboard::Clear();
}


//***********************************
//*** Implementation VclClipBoard ***
//***********************************

VclClipboard::VclClipboard()
{
    ImplSVData* pSVData = ImplGetSVData();

    // Ist Clipboard-Tabelle noch nicht angelegt, dann erzeugen
    if ( !pSVData->maExchangeData.mpClipTab )
        pSVData->maExchangeData.mpClipTab = new ImplClipTab;

    // Clipboard in die Verwaltungstabelle einfuegen
    pSVData->maExchangeData.mpClipTab->Insert( this );
}

VclClipboard::~VclClipboard()
{
    // Clipboard in der Verwaltungstabelle entfernen
    ImplGetSVData()->maExchangeData.mpClipTab->Remove( this );
}

void VclClipboard::Cleared()
{
    aClearedHdl.Call( this );

    /* Private Data ???
    SVDATA* pSVData = GetSVData();
    if ( pSVData->aClipD.bPrivateData )
    {
        pSVData->aClipD.bPrivateData = FALSE;
        pSVData->aClipD.pPrivateData = NULL;
    }
    */
}

void VclClipboard::Clear()
{
    // Clear-Handler aufrufen
    ImplSVData* pSVData = ImplGetSVData();
    if( pSVData->maExchangeData.mpClipTab )
        pSVData->maExchangeData.mpClipTab->Cleared();

    // Daten loslassen
    ClipManager::ImplGetClipManager()->SetClipData( NULL );

#ifdef WNT
	// Don`t call TSystemClipboard::clear here: #75995#
	EmptySystemClipboard();
#endif
}

void VclClipboard::Copy( SotDataObject* pData_ )
{
    ClipManager::ImplGetClipManager()->SetClipData( pData_ );

#ifdef WNT
	ImplUpdateSystemClipboard();
#else

	// Bei jedem Copy das Systemclipboard loeschen, um evtl. noch vom System
	// festgehaltene Instanzen loszulassen, damit diese nicht mehr denken,
	// sie waeren im Clipboard. GRRRR !!!!
	EmptySystemClipboard();
#endif
}

SotDataObject* VclClipboard::Paste( void )
{
    // Haben wir Daten?
    SotDataObjectRef xData = ClipManager::ImplGetClipManager()->GetClipData();
    if( !xData.Is() )
    {
        // Sonst noch mal beim System nachfragen
        ISystemClipboard* pSysClipboard = ImplGetSystemClipboard();
        if( pSysClipboard )
        {
            xData = new DTransSotDataObject( pSysClipboard->paste() );

            // Datenobjekt in den VCL-Daten eintragen, kommt von aussen!

            ClipManager::ImplGetClipManager()->SetClipData( xData, FALSE );
        }
    }
    return xData;
}

/********************************************************************************/

void VclClipboard::ReleaseClipboard( BOOL bFlushToSystem )
{
    // Haben wir Daten?
    ClipManager* pClipManager = ClipManager::ImplGetClipManager();
    SotDataObjectRef xData = pClipManager->GetClipData();
    if( xData.Is() )
    {
        // Sollen die Daten in das System-Clipboard geschrieben werden?
        if( pClipManager->IsOwnData() ) {
			ISystemClipboard* pSysClipboard = ImplGetSystemClipboard();

			if ( bFlushToSystem )
				// System-clipboard aktualisieren
	            UpdateSystemClipboard();
			else
				EmptySystemClipboard();
	
			// System-Clipboard rendern

			if( pSysClipboard )
				pSysClipboard->clear();
		}

        // Daten loslassen
        pClipManager->SetClipData( NULL );
    }
}

BOOL VclClipboard::ExistsSystemClipboard( void )
{
    return ( NULL != GetSystemClipboard() );
}


USHORT VclClipboard::GetFormatCount()
{
    SotDataObjectRef xData = Paste();
    if( xData.Is() )
        return (USHORT)xData->GetTypeList().Count();
    else
        return 0;
}

ULONG VclClipboard::GetFormat( USHORT nPos )
{
    SotDataObjectRef xData = Paste();
    if( xData.Is() )
        return xData->GetTypeList().GetObject( nPos ).GetFormat();
    else
        return 0;
}

BOOL VclClipboard::HasFormat( ULONG nFormat )
{
    {
    SotDataObjectRef xData = Paste();
    if( xData.Is() )
        return NULL != xData->GetTypeList().Get( nFormat );
    else
        return FALSE;
    }
}

//**************************
//*** Alte Schnittstelle ***
//**************************

// Hilfs-Methode fuer alte Schnittstelle
SotDataMemberObject* VclClipboard::ImplGetMemberObject( void )
{
    // Haben wir schon ein Daten-Objekt
    SotDataObjectRef xData = ClipManager::ImplGetClipManager()->GetClipData();
    SotDataMemberObjectRef xMemberObject;
    if( xData.Is() )
    {
        // RTTI-Hack benutzen
        if( xData->IsSotDataMemberObject() )
            xMemberObject = (SotDataMemberObject*)( (SotDataObject*)xData );

        // ALT
        // SotDataObject* pDataObject = xData;
        // xMemberObject = PTR_CAST(SotDataMemberObject,pDataObject);
    }
    if( !xMemberObject.Is() )
    {
        xMemberObject = new SotDataMemberObject();
        ClipManager::ImplGetClipManager()->SetClipData( xMemberObject );

        // SYSTEM_EXCHANGE
        // ImplCopyToSystemClipboard( xMemberObject );
    }
    return xMemberObject;
}


/*
HINWEIS:
Da die alte Clipboard-Schnittstelle kein Data-Objekt benoetigt,
muss dafuer gesorgt werden, dass hier eins zur Verf?gung steht.

Dabei wird wie folgt vorgegangen:
Wenn im Clipboard noch kein SotDataMemberObject vorliegt wird ein
SotDataMemberObject erzeugt und ins Clipboard gestellt. An diesem
Objekt werden alle nachfolgenden CopyX-Operationen ausgefuehrt.

Wenn bereits ein SotDataObject vorhanden ist, wird dieses dabei
ueberschrieben. Die neue Schnittstelle sollte also nicht gemischt
mit der alten benutzt werden.
*/

BOOL VclClipboard::CopyString( const XubString& rStr )
{
    SotDataMemberObjectRef xData = ImplGetMemberObject();
    SvData* pData = new SvData( FORMAT_STRING );
    pData->SetData( rStr );
    xData->Append( pData );

#ifdef WNT
	ImplUpdateSystemClipboard();
#else
	// Systemclipboard loeschen 
	EmptySystemClipboard();
#endif

    return TRUE;
}

BOOL VclClipboard::CopyBitmap( const Bitmap& rBitmap )
{
    return FALSE;
}

BOOL VclClipboard::CopyGDIMetaFile( const GDIMetaFile& rGDIMetaFile )
{
    return FALSE;
}

// Schein niemand zu benutzen, nur eine (unwichtige) Anwendung im Calc fuer OS/2-Bug:
BOOL VclClipboard::CopyPrivateData( void* pData )
{
    return FALSE;
}

BOOL VclClipboard::CopyData
( const void* pData, ULONG nLen, ULONG nFormat, USHORT /*nTypeLen (auch in SV ungenutzt)*/ )
{
#ifdef DEBUG
	String aFormatName = SotExchange::GetFormatName( nFormat );
#endif
    SotDataMemberObjectRef xData = ImplGetMemberObject();
    SvData* pSvData = new SvData( nFormat );
    pSvData->SetData( (void*)pData, nLen, TRANSFER_COPY );  // COPY ist das sicherste
    xData->Append( pSvData );

#ifdef WNT
	ImplUpdateSystemClipboard();
#else
	// Systemclipboard loeschen 
	EmptySystemClipboard();
#endif

    return TRUE;
}

BOOL VclClipboard::CopyRequest( ULONG nFormat )
{
    // Zunaechst mal ganz simpel vorgehen und die Daten gleich anfordern
    // Optimierungs-Moeglichkeit durch SvDataRequest : public SvData
    ImplSVData* pSVData = ImplGetSVData();

    // Format eintragen und Request-Handler rufen
    pSVData->maExchangeData.mnClipReqFormat = nFormat;
    pSVData->maExchangeData.mpClipTab->Request();
    pSVData->maExchangeData.mnClipReqFormat = 0L;
    return TRUE;
}

String VclClipboard::PasteString()
{
    if( HasFormat( FORMAT_STRING ) )
    {
        SotDataObjectRef xData = Paste();
        if( xData.Is() )
        {
            // SvData basteln
            SvData aData( FORMAT_STRING );
            if( xData->GetData( &aData ) )
            {
                // String holen
                String aStr;
                aData.GetData( aStr );
                return aStr;
            }
        }
    }
    return String();
}

Bitmap VclClipboard::PasteBitmap()
{
    return Bitmap();
}

BOOL VclClipboard::PasteGDIMetaFile( GDIMetaFile& rGDIMetaFile )
{
    return FALSE;
}

// Schein niemand zu benutzen
void* VclClipboard::PastePrivateData()
{
    return NULL;
}

BOOL VclClipboard::PasteData( void* pBuf, ULONG nLen, ULONG nFormat )
{
    if( HasFormat( nFormat ) )
    {
        SotDataObjectRef xData = Paste();
        if( xData.Is() )
        {
            // SvData basteln
            SvData aData( nFormat );
            if( xData->GetData( &aData ) )
            {
                // Daten als Referenz holen, muss sowieso kopiert werden
                void* pData;
                if( aData.GetData( &pData, TRANSFER_REFERENCE ) )
                {
					ULONG nDataSize = GetDataLen( nFormat );
                    ULONG nSize = min( nLen, nDataSize );
#ifdef W31
                    hmemcpy( pBuf, pData, (size_t)nSize );
#else
                    memcpy( pBuf, pData, (size_t)nSize );
#endif
                    return TRUE;
                }
            }
        }
    }
    return FALSE;
}

ULONG VclClipboard::GetDataLen( ULONG nFormat )
{
    if( HasFormat( nFormat ) )
    {
        SotDataObjectRef xData = Paste();
        if( xData.Is() )
        {
            // SvData basteln
            SvData aData( nFormat );
            if( xData->GetData( &aData ) )
                return aData.GetMinMemorySize();
        }
    }
    return 0L;
}

ULONG VclClipboard::GetRequestFormat()
{
    ImplSVData* pSVData = ImplGetSVData();
    return pSVData->maExchangeData.mnClipReqFormat;
}


//**********************************
//*** Implementation ImplClipTab ***
//**********************************

void ImplClipTab::Request()
{
    // Wird nur noch fuer die alte Schnittstelle gebraucht,
    // da der Datenaustausch ansonsten ueber SvData geregelt wird.

    ImpPtrTabData* pTemp = pLast;
    while ( pTemp )
    {
        if ( pTemp->pPtr )
            ((Clipboard*)pTemp->pPtr)->Request();
        pTemp = pTemp->pPrev;
    }

    // Um evt. geloeschte Instanzen aus der Liste zu entfernen
    Free();
}

void ImplClipTab::Cleared()
{
    ImpPtrTabData* pTemp = pLast;
    while ( pTemp )
    {
        if ( pTemp->pPtr )
            ((Clipboard*)pTemp->pPtr)->Cleared();
        pTemp = pTemp->pPrev;
    }

    // Um evt. geloeschte Instanzen aus der Liste zu entfernen
    Free();
}


//**********************************
//*** Implementation ImplClipTab ***
//**********************************

// ClipManager-Instanz holen/anlegen
ClipManager* ClipManager::ImplGetClipManager( void )
{
    ImplSVData* pSVData = ImplGetSVData();
    ClipManager*& rpClipManager = pSVData->maExchangeData.mpClipManager;
    if( !rpClipManager )
        rpClipManager = new ClipManager();
    return rpClipManager;
}

//**************************************
//*** Dummy-Implementation ClipBoard ***
//**************************************



/* Jetzt direkt auf VclClipboard gemappt
Clipboard::Clipboard()
{
}

Clipboard::~Clipboard()
{
}

void Clipboard::Cleared()
{
}

void Clipboard::Clear()
{
}

BOOL Clipboard::CopyString( const XubString& rStr )
{
    return FALSE;
}

BOOL Clipboard::CopyBitmap( const Bitmap& rBitmap )
{
    return FALSE;
}

BOOL Clipboard::CopyGDIMetaFile( const GDIMetaFile& rGDIMetaFile )
{
    return FALSE;
}

BOOL Clipboard::CopyPrivateData( void* pData )
{
    return FALSE;
}

BOOL Clipboard::CopyData( const void* pData, ULONG nLen, ULONG nFormat, USHORT)
{
    return FALSE;
}

BOOL Clipboard::CopyRequest( ULONG nFormat )
{
    return FALSE;
}

XubString Clipboard::PasteString()
{
    return XubString();
}

Bitmap Clipboard::PasteBitmap()
{
    return Bitmap();
}

BOOL Clipboard::PasteGDIMetaFile( GDIMetaFile& rGDIMetaFile )
{
    return FALSE;
}

void* Clipboard::PastePrivateData()
{
    return NULL;
}

BOOL Clipboard::PasteData( void* pBuf, ULONG nLen, ULONG nFormat )
{
    return FALSE;
}

ULONG Clipboard::GetDataLen( ULONG nFormat )
{
    return 0L;
}

USHORT Clipboard::GetFormatCount()
{
    return 0;
}

ULONG Clipboard::GetFormat( USHORT nPos )
{
    return 0L;
}

BOOL Clipboard::HasFormat( ULONG nFormat )
{
    return FALSE;
}

ULONG Clipboard::GetRequestFormat()
{
    return 0L;
}
*/

void UpdateSystemClipboard()
{
#ifndef WNT
    ImplUpdateSystemClipboard();
#endif
}

void ImplUpdateSystemClipboard()
{
    // Wenn es ein System-Clipboard gibt, hier copy am Interface rufen
    ISystemClipboard* pSysClipboard = ImplGetSystemClipboard();
	ClipManager *pClipManager = ClipManager::ImplGetClipManager();

    if( pSysClipboard && pClipManager && pClipManager->IsOwnData())
    {
		SotDataObjectRef xData = ClipManager::ImplGetClipManager()->GetClipData();
		if( xData.Is() )
		{
			SotDataObject* pData = xData;

			// Daten-Objekt wrappen
			ImplClipDTrans* pClipDTrans = new ImplClipDTrans( pData );

			// Ab ins System-Clipboard
#ifdef WNT
			g_nOwnObjects++;
			pSysClipboard->copy( pClipDTrans );
#else
			g_OwnCopy = TRUE;
			pSysClipboard->copy( pClipDTrans );
			g_OwnCopy = FALSE;
#endif
		}
    }
}

static void EmptySystemClipboard()
{
    // Wenn es ein System-Clipboard gibt, Systemclipboard mit
	// leerem Objekt loeschen
    ISystemClipboard* pSysClipboard = ImplGetSystemClipboard();

    if( pSysClipboard )
    {
		SotDataObject* pData = new SotDataObject();
		ImplClipDTrans* pClipDTrans = new ImplClipDTrans( pData );

#ifdef WNT
		g_nOwnObjects++;
		pSysClipboard->copy( pClipDTrans );
#else
		g_OwnCopy = TRUE;
		pSysClipboard->copy( pClipDTrans );
		g_OwnCopy = FALSE;
#endif
    }
}

