/*************************************************************************
 *
 *  $RCSfile: drag.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: pl $ $Date: 2000/10/04 14:28:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <string.h>

// Fuer DragServer alte Implementation zulassen
#include <sv.h>
#undef NOOLDSV

#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#ifdef W31
#include <tools/svwin.h>
#endif

#ifndef _SV_GETSYS_HXX
#include <getsys.hxx>
#endif
#ifndef _SV_DRAG_HXX
#include <drag.hxx>
#endif
#ifndef _SV_BITMAP
#include <bitmap.hxx>
#endif
#ifndef _SV_SVAPP_HXX
#include <svapp.hxx>
#endif
#ifndef _SV_WINDOW_HXX
#include <window.hxx>
#endif
#ifndef _SV_SVDATA_HXX
#include <svdata.hxx>
#endif
#ifndef _SV_EXCHANGE_H
#include <exchange.h>
#endif

#ifdef REMOTE_APPSERVER

#include "unoex.hxx"
#include "rvp.hxx"

#ifdef MAC
#include "macex.hxx"
#endif

#if SUPD > 364
#define __UNO_Long_ Long
#else
#define __UNO_Long_ Int
#endif

//*******************************************
//*** Schnittstelle fuer System-Anbindung ***
//*******************************************

class UnoDragDropListenerAdaptor : public ::cppu::WeakImplHelper1< NMSP_SYSTEM::XDragDropListener >
{
protected:

		IDragDropListener&			m_Listener;

public:

								UnoDragDropListenerAdaptor( IDragDropListener& rListener );
								~UnoDragDropListenerAdaptor();

    virtual void SAL_CALL		beginDragDrop( const REF( NMSP_SYSTEM::XDataTransfer )& Data, sal_Int16 SrcOptions ) throw( NMSP_UNO::RuntimeException );
    virtual sal_Int16 SAL_CALL	queryDrop( sal_Int32 x, sal_Int32 y, const ANY& Frame, sal_uInt16 nModKeys ) throw( NMSP_UNO::RuntimeException );
    virtual sal_Int16 SAL_CALL	drop( sal_Int32 x, sal_Int32 y, const ANY& Frame, sal_uInt16 nModKeys ) throw( NMSP_UNO::RuntimeException );
    virtual void SAL_CALL		endDragDrop(  ) throw( NMSP_UNO::RuntimeException);
};

// -----------------------------------------------------------------------------

UnoDragDropListenerAdaptor::UnoDragDropListenerAdaptor( IDragDropListener& rListener ) :
	m_Listener( rListener )
{
}

// -----------------------------------------------------------------------------
	
UnoDragDropListenerAdaptor::~UnoDragDropListenerAdaptor()
{
}

// -----------------------------------------------------------------------------

void SAL_CALL UnoDragDropListenerAdaptor::beginDragDrop( const REF( NMSP_SYSTEM::XDataTransfer )& rData, sal_Int16 SrcOptions ) throw( NMSP_UNO::RuntimeException )
{
	m_Listener.beginDragDrop( new UnoDataTrans( rData ), SrcOptions );
}

// -----------------------------------------------------------------------------

sal_Int16 SAL_CALL UnoDragDropListenerAdaptor::queryDrop( sal_Int32 x, sal_Int32 y, const ANY& Frame, sal_uInt16 nModKeys ) throw( NMSP_UNO::RuntimeException )
{
	ULONG pFrame; Frame >>= pFrame;
	return m_Listener.queryDrop(Point(x, y), (SalFrame *)pFrame, nModKeys);
}

// -----------------------------------------------------------------------------

sal_Int16 SAL_CALL UnoDragDropListenerAdaptor::drop( sal_Int32 x, sal_Int32 y, const ANY& Frame, sal_uInt16 nModKeys ) throw( NMSP_UNO::RuntimeException )
{
	ULONG pFrame; Frame >>= pFrame;
	return m_Listener.drop(Point(x, y), (SalFrame *)pFrame, nModKeys);
}

// -----------------------------------------------------------------------------

void SAL_CALL UnoDragDropListenerAdaptor::endDragDrop(  ) throw( NMSP_UNO::RuntimeException )
{
	m_Listener.endDragDrop();
}

// ------------------------
// - UnoSystemDragManager -
// ------------------------

class UnoSystemDragManager : public ISystemDragManager
{
	REF( NMSP_SYSTEM::XSystemDragManager ) mxSystemDragManager;

public:

							UnoSystemDragManager( UnoSystemExchange* pSystemExchange );

	// Drag&Drop beginnen
	virtual SysDropAction	executeDrag( IDataTrans* pData, USHORT nOptions );

	// Listener fuer Drag&Drop-Events vom System anmelden
	virtual void			registerListener( IDragDropListener& rListener );

	// Drag&Drop fuer ein Fenster anmelden (von VCL durchreichen)
	virtual void			enableDrop( SalFrame* pSalFrame, BOOL bEnable );

	BOOL					isAvailable() const { return mxSystemDragManager.is(); }
};

// -----------------------------------------------------------------------------

UnoSystemDragManager::UnoSystemDragManager(UnoSystemExchange* pSystemExchange)
{
	if( pSystemExchange && pSystemExchange->isAvailable() )
		mxSystemDragManager = REF( NMSP_SYSTEM::XSystemDragManager )( pSystemExchange->getInterface(), NMSP_UNO::UNO_QUERY );
}

// -----------------------------------------------------------------------------

SysDropAction UnoSystemDragManager::executeDrag( IDataTrans* pData, USHORT nOptions )
{
	const REF( NMSP_UNO::XInterface ) xIf( static_cast< ::cppu::OWeakObject* >( new UnoDataTransAdaptor( pData ) ) );

	return( (SysDropAction) mxSystemDragManager->executeDrag( REF( NMSP_SYSTEM::XDataTransfer )( xIf, NMSP_UNO::UNO_QUERY ), nOptions ) );
}

// -----------------------------------------------------------------------------

void UnoSystemDragManager::registerListener( IDragDropListener& rListener )
{
	const REF( NMSP_UNO::XInterface ) xIf( static_cast< ::cppu::OWeakObject* >( new UnoDragDropListenerAdaptor( rListener ) ) );

	mxSystemDragManager->registerListener( REF( NMSP_SYSTEM::XDragDropListener )( xIf, NMSP_UNO::UNO_QUERY ) );
}

// -----------------------------------------------------------------------------

void UnoSystemDragManager::enableDrop(SalFrame* pSalFrame, BOOL bEnable)
{
	ANY Frame; Frame <<= (sal_uInt32) pSalFrame;

	try
	{
		mxSystemDragManager->enableDrop(Frame, bEnable);
	}
	catch(...)
	{
	}
}

#endif

//*******************************************************************************************

ISystemDragManager* GetSystemDragManager( void )
{
#ifdef REMOTE_APPSERVER
#ifdef MAC
	return GetMacDragManager();
#else
	static UnoSystemDragManager *pSystemDragManager = NULL;

	if (pSystemDragManager == NULL)
	{
		UnoSystemExchange *pExchange = UnoSystemExchange::getSystemExchange();

		if (pExchange->isAvailable())
			pSystemDragManager = new UnoSystemDragManager(pExchange);
		else
			return NULL;
	}

	return pSystemDragManager->isAvailable() ? pSystemDragManager : NULL;
#endif
#else
	return NULL;
#endif
}

//*******************************************************************************************

// System-DragManager holen
ISystemDragManager* ImplGetSystemDragManager( void )
{
#ifndef REMOTE_APPSERVER
	ISystemDragManager* pRet = GetSystemDragManager();

	// Formate registrieren, wenn es ein System gibt
	if( pRet )
		Exchange::ImplRegisterSystemFormats();

	return pRet;
#else
	return NULL;
#endif
}

#ifndef REMOTE_APPSERVER

/**
 * Interface zur Entgegennahme von Drag&Drop-System-Events. <BR>
 */
class ImplDragDropListener : public IDragDropListener
{
	Window* mpLastWindow;

public:
	ImplDragDropListener( void )
		: mpLastWindow( NULL ) {}

	// Start des Drag&Drop, queryDrop() muss immer folgen!
	virtual void beginDragDrop( IDataTrans* pData, USHORT nSrcOptions_ );

	virtual SysDropAction queryDrop( Point aMousePos, SalFrame* pSalFrame, USHORT nModKeys );

	virtual SysDropAction drop( Point aMousePos, SalFrame* pSalFrame, USHORT nModKeys );

	virtual void endDragDrop( void);
				// Laeuft ueber queryDrop: void KeyModChanged( USHORT nKeyCode );
};

void ImplDragDropListener::beginDragDrop( IDataTrans* pData, USHORT nSrcOptions_ )
{
	// Wenn schon ein ActiveDragManager vorliegt, ist es unsere
	// eigene Drag&Drop-Operation. Dann behalten wir unsere Daten,
	// statt IDataTrans zu wrappen.
	ImplSVData* pSVData = ImplGetSVData();
	if( pSVData->maExchangeData.mpActiveDragManager == NULL )
	{
		// Kommt von aussen, wir muessen also wrappen
		SotDataObjectRef xData = new DTransSotDataObject( pData );

		// Sicherstellen, dass ein DragManager-Instanz existiert
		DragManager* pDragManager = DragManager::ImplGetDragManager();

		// Irgendwelche Options sollten schon unterstuetzt werden
		/*
		if( (nSrcOptions_ & DRAG_MOVEABLE) == 0 &&
			(nSrcOptions_ & DRAG_COPYABLE) == 0 &&
			(nSrcOptions_ & DRAG_LINKABLE) == 0 )
			return DROP_NONE;
			*/

		// DragManager initialisieren
		pDragManager->InitDrag( xData, nSrcOptions_, NULL );

		// Wir sind's NICHT selber, sondern System
		pDragManager->bOwnDragDrop = FALSE;

		// Drag&Drop ist jetzt aktiv
		pSVData->maExchangeData.mpActiveDragManager = pDragManager;

#ifdef OLDDRAGDROP
		// Daten auch im DragManager zur Verfuegung stellen
		DragManager::ImplGetDragManager()->mxDragServerData = new MultiItemObject( xData );
#endif // OLDDRAGDROP
	}
}

SysDropAction ImplDragDropListener::queryDrop
	( Point aMousePos, SalFrame* pSalFrame, USHORT nModKeys )
{
	ImplSVData* pSVData = ImplGetSVData();
	DragManager* pActiveDragManager = pSVData->maExchangeData.mpActiveDragManager;

	DBG_ASSERT(pActiveDragManager,"ImplDragDropListener::queryDrop() ohne aktiven DragManager")

	// Gibt es ein Frame?
	if( pSalFrame )
	{
		// VCL-Window ausgraben und Point umrechnen lassen
		Window* pVclWindow = ImplFindWindow( pSalFrame, aMousePos );

		// Es kann passieren, dass kein VCL-Window gefunden wird
		if( !pVclWindow )
			return DROP_NONE;

		// Wenn es ein anderes Fenster als vorher ist, muss noch ein Leave erfolgen
		if( mpLastWindow && mpLastWindow != pVclWindow )
		{
			// Leave-MouseEvent basteln
			MouseEvent aMEvt( Point( -1, -1 ), 0, MOUSE_LEAVEWINDOW, 0, nModKeys );

			// VCL-queryDrop-Schnittstelle rufen
			pActiveDragManager->MouseMove( aMEvt, mpLastWindow );

			// neues Fenster uebernehmen
			mpLastWindow = pVclWindow;
		}
		// #55942 Irgendwann muss mpLastWindow mal initialisiert werden
		else if( !mpLastWindow )
		{
			mpLastWindow = pVclWindow;
		}

		// MouseEvent basteln
		MouseEvent aMEvt( aMousePos, 0, 0, 0, nModKeys );

		// VCL-queryDrop-Schnittstelle rufen
		pActiveDragManager->MouseMove( aMEvt, pVclWindow );
	}
	// Sonst bedeutet der Aufruf ein Leave aus dem Bereich der VCL-Applikation
	else if( mpLastWindow )
	{
		// MouseEvent basteln
		MouseEvent aMEvt( Point( -1, -1 ), 0, MOUSE_LEAVEWINDOW, 0, 0 );

		// VCL-queryDrop-Schnittstelle rufen
		pActiveDragManager->MouseMove( aMEvt, mpLastWindow );
		mpLastWindow = NULL;
	}
	return pActiveDragManager->eDropResult;
}

SysDropAction ImplDragDropListener::drop
	( Point aMousePos, SalFrame* pSalFrame, USHORT nModKeys )
{
	ImplSVData* pSVData = ImplGetSVData();
	DragManager* pActiveDragManager = pSVData->maExchangeData.mpActiveDragManager;

	DBG_ASSERT(pActiveDragManager,"ImplDragDropListener::queryDrop() ohne aktiven DragManager")

	// VCL-Window ausgraben und Point umrechnen lassen
	Window* pVclWindow = ImplFindWindow( pSalFrame, aMousePos );
	DBG_ASSERT(pVclWindow,"ImplDragDropListener::queryDrop() Keine VCL-Window gefunden")

	// MouseEvent basteln
	MouseEvent aMEvt( aMousePos, 0, 0, 0, nModKeys );

	// VCL-queryDrop-Schnittstelle rufen
	pActiveDragManager->ButtonUp( aMEvt, pVclWindow );

	// #57371 Im Drop auf jeden Fall mpLastWindow initialisieren
	mpLastWindow = NULL;

	return pActiveDragManager->eDropResult;
}

void ImplDragDropListener::endDragDrop( void )
{
	ImplSVData* pSVData = ImplGetSVData();
	DragManager* pActiveDragManager = pSVData->maExchangeData.mpActiveDragManager;

	DBG_ASSERT(pActiveDragManager,"ImplDragDropListener::queryDrop() ohne aktiven DragManager")

	// Entsprechende VCL-Schnittstelle rufen, wenn nicht eigenes Drag&Drop
	if( !pActiveDragManager->bOwnDragDrop )
	{
		pActiveDragManager->Escape( NULL );

#ifdef OLDDRAGDROP
		// Daten auch im DragManager loeschen
		DragManager::ImplGetDragManager()->mxDragServerData = NULL;
#endif // OLDDRAGDROP
	}
}

#endif		// #ifndef REMOTE_APPSERVER


static long ImplCallDragPreNotify( Window* pWindow, USHORT nType,
								   const void* pData, long* pRet )
{
	//EVENT_DROP / EVENT_QUERYDROP
	NotifyEvent aNEvt( nType, pWindow, pData );
	aNEvt.SetReturnValue( *pRet );
	long nRet = ImplCallPreNotify( aNEvt );
	*pRet = aNEvt.GetReturnValue();
	return nRet;
}

// Aufzurufen, wenn beim Drag&Drop das App-Fenster verlassen wird.
void DragManager::AppWindowLeaved( void )
{
#ifndef REMOTE_APPSERVER
	// HINWEIS: Wir befinden uns hier im Reschedule()
	ISystemDragManager* pSystemDragManager = ImplGetSystemDragManager();
	if( pSystemDragManager )
	{
		// #55942 MouseLeave bei vorigem Fenster ausloesen
		if( pLastTargetWindow )
		{
			DropEvent aDEvt( Point( -1, -1 ), xData, DROP_MOVE, nOptions, DROPEVENT_LEAVEWINDOW, TRUE );
			long nRet;
			if ( !ImplCallDragPreNotify( pLastTargetWindow, EVENT_QUERYDROP, &aDEvt, &nRet ) )
				nRet = pLastTargetWindow->QueryDrop( aDEvt );
			pLastTargetWindow = NULL;
		}

		// DTrans-Instanz beschaffen
		ImplDTrans* pDTrans = new ImplDTrans( xData );
		pDTrans->aquire();

		// System-Schnittstelle aufrufen
		eDropResult = pSystemDragManager->executeDrag( pDTrans, nOptions );

		// VCL-Schnittstelle fuer das Ende von Drag&Drop rufen
		Escape( NULL );

		// DTrans-Objekt loslassen
		pDTrans->release();
	}
#endif		// #ifndef REMOTE_APPSERVER
}


//**********************************
//*** Implementation DragManager ***
//**********************************

// Ctor
DragManager::DragManager( void )
{
}

// DragManager fuer Drag-Vorgang initialisieren
void DragManager::InitDrag( SotDataObject* pData_, USHORT nOptions_, DropSource* pDropSource_ )
{
	// Daten uebernehmen
	xData = pData_;
	nOptions = nOptions_;
	pDropSource = pDropSource_;

	bDrawPointerResult = FALSE;
	pLastTargetWindow = NULL;
	eDropResult = DROP_NONE;
}

// DragManager-Instanz holen/anlegen
DragManager* DragManager::ImplGetDragManager( void )
{
	ImplSVData* pSVData = ImplGetSVData();
	DragManager*& rpDragManager = pSVData->maExchangeData.mpDragManager;
	if( !rpDragManager )
		rpDragManager = new DragManager();
	return rpDragManager;
}

// Drag&Drop beginnen
DropAction DragManager::ExecuteDrag
( SotDataObject* pData_, USHORT nOptions_, DropSource* pDropSource_ )
{
	// HACK: Hier wird hart SystemEnableDrop() ohne Window aufgerufen,
	// damit sich ein DragDropListener im System registrieren kann.
	// Dies soll eigentlich von Window::EnableDrop() erfolgen, aber
	// die Methode ist inline -> inkompatibles Update noetig
	//SystemEnableDrop( NULL );


	ImplSVData* pSVData = ImplGetSVData();

	// Tracking beenden
	if ( pSVData->maWinData.mpTrackWin )
		pSVData->maWinData.mpTrackWin->EndTracking( ENDTRACK_CANCEL );

	// Capturing beenden
	DBG_ASSERT( pSVData->maWinData.mpCaptureWin == NULL,
				"Mouse has to be released before ExecuteDrag" );
	if ( pSVData->maWinData.mpCaptureWin )
		pSVData->maWinData.mpCaptureWin->ReleaseMouse();

#ifdef OLDDRAGDROP
	// Daten auch im DragManager zur Verfuegung stellen
	if( !DragManager::ImplGetDragManager()->mxDragServerData.Is() )
		DragManager::ImplGetDragManager()->mxDragServerData = new MultiItemObject( pData_ );
#endif // OLDDRAGDROP

	// Sicherstellen, dass ein DragManager-Instanz existiert
	DragManager* pDragManager = ImplGetDragManager();

	DragManager*& rpActiveDragManager = pSVData->maExchangeData.mpActiveDragManager;

	// Mehrfach-D&D verhindern
	DBG_ASSERT( !rpActiveDragManager, "Mehrfacher Aufruf von ExecuteDrag" )
	if( rpActiveDragManager )
		return DROP_NONE;

	// Irgendwelche Options sollten schon unterstuetzt werden
	if( (nOptions_ & DRAG_MOVEABLE) == 0 &&
		(nOptions_ & DRAG_COPYABLE) == 0 &&
		(nOptions_ & DRAG_LINKABLE) == 0 )
		return DROP_NONE;

	// DragManager initialisieren
	pDragManager->InitDrag( pData_, nOptions_, pDropSource_ );

	// Wir sind's selber
	pDragManager->bOwnDragDrop = TRUE;

	// Drag&Drop ist aktiv
	rpActiveDragManager = pDragManager;

	// Warten, bis Drag&Drop beendet ist
	while( rpActiveDragManager )
		Application::Reschedule();

#ifdef OLDDRAGDROP
	// Daten im DragManager wieder loeschen
	DragServer::Clear();
#endif // OLDDRAGDROP

	return pDragManager->eDropResult;
}


// Wenn ein Drag&Drop-Vorgang im Gange ist, liefert folgender Aufruf != NULL
DragManager* DragManager::GetDragManager( void )
{
	ImplSVData* pSVData = ImplGetSVData();
	return pSVData->maExchangeData.mpActiveDragManager;
	//return pActiveDragManager;
}

// EnableDragDrop an das System-Fenster durchreichen
void DragManager::SystemEnableDrop( SalFrame* pFrame, BOOL bEnable )
{
#ifndef REMOTE_APPSERVER
	static BOOL bRegisteredListener = FALSE;

	ISystemDragManager* pSysManager = ImplGetSystemDragManager();
	if( NULL != pSysManager )
	{
		// Listener schon angemeldet? Sonst jetzt machen
		if( !bRegisteredListener )
		{
			pSysManager->registerListener( *(new ImplDragDropListener()) );
			bRegisteredListener = TRUE;
		}

		// System-Schnittstelle fuer enableDrop() aufrufen
		if( pFrame )
			pSysManager->enableDrop( pFrame, bEnable );
	}
#endif
}

// Service: Standard-Pointer setzen, kann von DrawDragPointer gerufen werden
Pointer DragManager::GetDefaultPointer( DropAction eAction )
{
	switch( eAction )
	{
		case DROP_MOVE: return Pointer( POINTER_MOVEDATA ); break;
		case DROP_COPY: return Pointer( POINTER_COPYDATA ); break;
		case DROP_LINK: return Pointer( POINTER_LINKDATA ); break;
		default:		return Pointer( POINTER_NOTALLOWED );
	}
}


// Schnittstelle fuer System-Events
void DragManager::MouseMove( const MouseEvent& rMEvt, Window* pTargetWindow )
{
	// Drop-Action ermitteln
	// TEST: Erstmal ganz grob
	DropAction eAction = DROP_MOVE;
	BOOL bDefault = TRUE;

	// Ctrl == Mod1?
	if( rMEvt.IsMod1() )
	{
		if( rMEvt.IsShift() )
			eAction = DROP_LINK;
		else
			eAction = DROP_COPY;

		bDefault = FALSE;
	}
	else if( rMEvt.IsShift() )
	{
		bDefault = FALSE;
	}

	// Ist die Default-Action ueberhaupt zulaessig?
	if( bDefault )
	{
		if( eAction == DROP_MOVE && (nOptions & DRAG_MOVEABLE) == 0 )
			eAction = DROP_COPY;
		if( eAction == DROP_COPY && (nOptions & DRAG_COPYABLE) == 0 )
		{
			if( (nOptions & DRAG_MOVEABLE) != 0 )
				eAction = DROP_MOVE;
			else
				eAction = DROP_LINK;
		}
	}

	// Event-Art bestimmen
	BYTE nWindowType = 0;
	//if( rMEvt.IsEnterWindow() )
	if( pLastTargetWindow != pTargetWindow )
	{
		//aTargetWindowPointer = pTargetWindow->GetPointer();
		pLastTargetWindow = pTargetWindow;
		nWindowType = DROPEVENT_ENTERWINDOW;
	}
	else if( rMEvt.IsLeaveWindow() )
	{
		nWindowType = DROPEVENT_LEAVEWINDOW;
	}

	// Source Gelegenheit zum Aufraeumen geben
	if( pDropSource && bDrawPointerResult )
		pDropSource->ClrDragPointer( aLastMEvt.GetPosPixel() );

	// Maus-Event merken
	aLastMEvt = rMEvt;

	// Annahme: Keine Action ist zulaessig
	eDropResult = DROP_NONE;

	// QueryDrop ist nur noetig, wenn die Source-Options die Action zulassen
	if( ( eAction & nOptions ) != 0 )
	{
		// DropEvent erzeugen
		DropEvent aDEvt( aLastMEvt.GetPosPixel(), xData, eAction, nOptions, nWindowType, bDefault );
		long nRet = 0;
		if ( !ImplCallDragPreNotify( pTargetWindow, EVENT_QUERYDROP, &aDEvt, &nRet ) )
			nRet = pTargetWindow->QueryDrop( aDEvt );
		if( nRet )
			eDropResult = aDEvt.GetAction();
	}

	// Bei Leave immer Pointer zuruecksetzen
	if( nWindowType == DROPEVENT_LEAVEWINDOW )
	{
		//pTargetWindow->SetPointer( aTargetWindowPointer );
		bModifyPointer = FALSE;
	}
	// Sonst Feedback erfragen
	else
	{
		if( pDropSource )
			bDrawPointerResult = pDropSource->DrawDragPointer( eDropResult, aLastMEvt.GetPosPixel(), pTargetWindow );

		// Default-Verhalten?
		if( !bDrawPointerResult )
			SetDragPointer( GetDefaultPointer( eDropResult ) );
	}
}

void DragManager::ButtonUp( const MouseEvent& rMEvt, Window* pTargetWindow )
{
	// Auf jeden Fall noch ein QueryDrop() vorweg
	MouseMove( rMEvt, pTargetWindow );

	// Source Gelegenheit zum Aufraeumen geben
	if( pDropSource && bDrawPointerResult )
		pDropSource->ClrDragPointer( aLastMEvt.GetPosPixel() );

	// Drag&Drop schon hier deaktivieren, da sonst z.B. modale Dialoge
	// im Drop() Probleme machen (D&D geht dann faelschlich noch weiter)
	ImplSVData* pSVData = ImplGetSVData();
	pSVData->maExchangeData.mpActiveDragManager = NULL;

	// Pointer zuruecksetzen
	//pTargetWindow->SetPointer( aTargetWindowPointer );
	bModifyPointer = FALSE;

	// Nur droppen, wenn das Target einverstanden ist
	if( eDropResult != DROP_NONE )
	{
		// DropEvent erzeugen
		BOOL bDefault = !( rMEvt.IsMod1() || rMEvt.IsShift() );
		DropEvent aDEvt( rMEvt.GetPosPixel(), xData, eDropResult, nOptions, /*WindowType*/0, bDefault );
		long nRet = 0;
		if ( !ImplCallDragPreNotify( pTargetWindow, EVENT_DROP, &aDEvt, &nRet ) )
			nRet = pTargetWindow->Drop( aDEvt );
		if( !nRet )
			eDropResult = DROP_NONE;
	}
	// Sonst MouseLeave ausloesen
	else
	{
		DropEvent aDEvt( Point( -1, -1 ), xData, DROP_MOVE, nOptions, DROPEVENT_LEAVEWINDOW, TRUE );
		long nRet;
		if ( !ImplCallDragPreNotify( pTargetWindow, EVENT_QUERYDROP, &aDEvt, &nRet ) )
			nRet = pTargetWindow->QueryDrop( aDEvt );
	}

	// Drag&Drop ist vorbei
	xData = NULL;
}

void DragManager::Escape( Window* pTargetWindow )
{
	// Source Gelegenheit zum Aufraeumen geben
	if( pDropSource && bDrawPointerResult )
		pDropSource->ClrDragPointer( aLastMEvt.GetPosPixel() );

	// MouseLeave ausloesen
	if( pLastTargetWindow )
	{
		DropEvent aDEvt( Point( -1, -1 ), xData, DROP_MOVE, nOptions, DROPEVENT_LEAVEWINDOW, TRUE );
		long nRet;
		if ( !ImplCallDragPreNotify( pLastTargetWindow, EVENT_QUERYDROP, &aDEvt, &nRet ) )
			nRet = pLastTargetWindow->QueryDrop( aDEvt );
	}

	// Pointer zuruecksetzen
	//pTargetWindow->SetPointer( aTargetWindowPointer );
	bModifyPointer = FALSE;

	// Drag&Drop wurde abgebrochen
	xData = NULL;
	ImplSVData* pSVData = ImplGetSVData();
	pSVData->maExchangeData.mpActiveDragManager = NULL;
	eDropResult = DROP_NONE;
}

void DragManager::KeyModChanged( USHORT nKeyCode )
{
	MouseEvent aMEvt( aLastMEvt.GetPosPixel(), 0, 0, 0, nKeyCode );
	MouseMove( aMEvt, pLastTargetWindow );
}


#ifdef OLDDRAGDROP

//**********************************
//*** Implementation ImplDragTab ***
//**********************************

// DragTab von alter Implementation uebernehmen
class ImplDragTab : public ImpPtrTab
{
public:
					ImplDragTab() : ImpPtrTab() {}

	void			Insert( DragServer* pDragServer )
						{ ImpPtrTab::Insert( (void*)pDragServer ); }
	void			Remove( DragServer* pDragServer )
						{ ImpPtrTab::Remove( (void*)pDragServer ); }

	void			Request();
};

void ImplDragTab::Request()
{
	// Wird nur noch fuer die alte Schnittstelle gebraucht,
	// da der Datenaustausch ansonsten ueber SotData geregelt wird.

	ImpPtrTabData* pTemp = pLast;
	while ( pTemp )
	{
		if ( pTemp->pPtr )
			((DragServer*)pTemp->pPtr)->Request();
		pTemp = pTemp->pPrev;
	}

	// Um evt. geloeschte Instanzen aus der Liste zu entfernen
	Free();
}


#ifdef OLDDRAGDROP
MultiItemObject::MultiItemObject( SotDataObject* pDataObject_ )
{
	xExtObj = pDataObject_;
	pDataList = NULL;
	mnListSize = 0;
	mnItemCount = 1;
}
#endif // OLDDRAGDROP

MultiItemObject::MultiItemObject()
{
	pDataList = NULL;
	mnListSize = 0;
	mnItemCount = 1;
}

MultiItemObject::~MultiItemObject()
{
	// ueberfluessig wegen delete[]
	// for( USHORT i = 0 ; i < mnItemCount ; i++ )
	//	 pDataList[i] = NULL;

	delete[] pDataList;
}

SotDataObject* MultiItemObject::GetItem( USHORT nItem )
{
	SotDataObject* pRet = NULL;
	if( nItem == 0 )
	{
#ifdef OLDDRAGDROP
		if( xExtObj.Is() )
			pRet = xExtObj;
		else
#endif // OLDDRAGDROP
		pRet = this;
	}
	else if( nItem < mnItemCount )
		pRet = pDataList[ nItem - 1 ];
	return pRet;
}

void MultiItemObject::NewItem( void )
{
	InsertItem( new SotDataMemberObject() );
}

void MultiItemObject::InsertItem( SotDataObject* pDataObject )
{
	// Haben wir schon eine Liste?
	if( !pDataList )
	{
		// Liste anlegen
		mnListSize = 5;
		pDataList = new SotDataObjectRef[mnListSize];
	}
	else if( mnItemCount+1 > mnListSize )
	{
		// Liste vergroessern und umkopieren
		USHORT nNewListSize = mnListSize + 5;
		SotDataObjectRef* pNewDataList = new SotDataObjectRef[nNewListSize];
		for( USHORT i = 0 ; i < mnListSize ; i++ )
			pNewDataList[i] = pDataList[i];
		delete[] pDataList;
		pDataList = pNewDataList;
		mnListSize = nNewListSize;
	}

	// Neues Item anlegen und eintragen
	// ACHTUNG: 0. Item ist this, pDataList[0] enthaelt das 1. Item
	pDataList[ mnItemCount - 1 ] = pDataObject;
	mnItemCount++;
}

// Pointer, damit bei nItem >= nItemCount NULL geliefert werden kann
const SvDataTypeList* MultiItemObject::GetTypeList( USHORT nItem )
{
	SotDataObjectRef xItem = GetItem( nItem );
	const SvDataTypeList* pRet = NULL;
	if( xItem.Is() )
		pRet = &xItem->GetTypeList();

	return pRet;
}

BOOL MultiItemObject::GetData( USHORT nItem, SvData* pData )
{
	SotDataObjectRef xItem = GetItem( nItem );
	BOOL bRet = FALSE;
	if( xItem.Is() )
		return xItem->GetData( pData );
	return bRet;
}


//***************************************
//*** Dummy-Implementation DragServer ***
//***************************************

DragServer::DragServer()
{
	ImplSVData* pSVData = ImplGetSVData();

	// Ist DragServer-Tabelle noch nicht angelegt, dann erzeugen
	if ( !pSVData->maExchangeData.mpDragTab )
		pSVData->maExchangeData.mpDragTab = new ImplDragTab;

	// DragServer in die Verwaltungstabelle einfuegen
	pSVData->maExchangeData.mpDragTab->Insert( this );
}

DragServer::~DragServer()
{
	// DragServer aus der Verwaltungstabelle holen
	ImplSVData* pSVData = ImplGetSVData();
	pSVData->maExchangeData.mpDragTab->Remove( this );
}

void DragServer::Clear()
{
	// Daten-Objekt loslassen
	ImplSVData* pSVData = ImplGetSVData();
	DragManager::ImplGetDragManager()->mxDragServerData = NULL;
	pSVData->maExchangeData.mnDragItem = 0;
}

void DragServer::NewItem()
{
	ImplSVData* pSVData = ImplGetSVData();
	MultiItemObjectRef xDataObject = DragManager::ImplGetDragManager()->mxDragServerData;
	if( !xDataObject.Is() )
	{
		// NewItem ohne CopyXXX vorher -> 0. Item aktiv lassen
		xDataObject = DragManager::ImplGetDragManager()->mxDragServerData = new MultiItemObject();
	}
	else
	{
		xDataObject->NewItem();
		pSVData->maExchangeData.mnDragItem++;
	}
}

BOOL DragServer::CopyString( const XubString& rStr )
{
	SotDataMemberObjectRef xItem = (SotDataMemberObject*)ImplGetActiveItem();
	if( xItem.Is() )
	{
		SvData* pData = new SvData( FORMAT_STRING );
		pData->SetData( rStr );
		xItem->Append( pData );
		return TRUE;
	}
	return FALSE;
}

BOOL DragServer::CopyBitmap( const Bitmap& rBitmap )
{
	return FALSE;
}

BOOL DragServer::CopyGDIMetaFile( const GDIMetaFile& rGDIMetaFile )
{
	return FALSE;
}

BOOL DragServer::CopyPrivateData( void* pData )
{
	return FALSE;
}

BOOL DragServer::CopyFile( const String& rStr )
{
	SotDataMemberObjectRef xItem = (SotDataMemberObject*)ImplGetActiveItem();
	if( xItem.Is() )
	{
		SvData* pData = new SvData( FORMAT_FILE );
		pData->SetData( rStr );
		xItem->Append( pData );
		return TRUE;
	}
	return FALSE;
}

BOOL DragServer::CopyData( const void* pData, ULONG nLen, ULONG nFormat,
						   USHORT /* nTypeLen */ )
{
	SotDataMemberObjectRef xItem = (SotDataMemberObject*)ImplGetActiveItem();
	SvData* pSvData = new SvData( nFormat );
	pSvData->SetData( (void*)pData, nLen, TRANSFER_COPY );	// COPY ist das sicherste
	xItem->Append( pSvData );
	return TRUE;
}

BOOL DragServer::CopyRequest( ULONG nFormat )
{
	// Zunaechst mal ganz simpel vorgehen und die Daten gleich anfordern
	// Optimierungs-Moeglichkeit durch SvDataRequest : public SvData
	ImplSVData* pSVData = ImplGetSVData();

	// Format eintragen
	pSVData->maExchangeData.mnDragReqFormat = nFormat;
	// Request-Handler rufen
	if ( pSVData->maExchangeData.mpDragTab )
		pSVData->maExchangeData.mpDragTab->Request();
	// Kein Request-Handler? Dummy-Daten eintragen,
	// damit das Format da ist (wird als Trick in Draw/Calc verwendet)
	else
		CopyData( "", 1, nFormat );
	pSVData->maExchangeData.mnDragReqFormat = 0L;
	return TRUE;
}

USHORT DragServer::GetItemCount()
{
	USHORT nRet = 0;
	if( DragManager::ImplGetDragManager()->mxDragServerData.Is() )
		nRet = DragManager::ImplGetDragManager()->mxDragServerData->GetItemCount();
	return nRet;
}

String DragServer::PasteString( USHORT nItem )
{
	if( HasFormat( nItem, FORMAT_STRING ) )
	{
		SotDataObjectRef xItem = ImplGetItem( nItem );
		if( xItem.Is() )
		{
			// SvData basteln
			SvData aData( FORMAT_STRING );
			if( xItem->GetData( &aData ) )
			{
				// String holen
				String aStr;
				aData.GetData( aStr );
				return aStr;
			}
		}
	}
	return ImplGetSVEmptyStr();
}

Bitmap DragServer::PasteBitmap( USHORT nItem )
{
	return Bitmap();
}

BOOL DragServer::PasteGDIMetaFile( USHORT nItem, GDIMetaFile& rGDIMetaFile )
{
	return FALSE;
}

void* DragServer::PastePrivateData( USHORT nItem )
{
	return NULL;
}

String DragServer::PasteFile( USHORT nItem )
{
	if( HasFormat( nItem, FORMAT_FILE ) )
	{
		SotDataObjectRef xItem = ImplGetItem( nItem );
		if( xItem.Is() )
		{
			// SvData basteln
			SvData aData( FORMAT_FILE );
			if( xItem->GetData( &aData ) )
			{
				// String holen
				String aStr;
				aData.GetData( aStr );
				return aStr;
			}
		}
	}
	return ImplGetSVEmptyStr();
}

BOOL DragServer::PasteData( USHORT nItem, void* pBuf, ULONG nLen, ULONG nFormat )
{
	if( HasFormat( nItem, nFormat ) )
	{
		SotDataObjectRef xItem = ImplGetItem( nItem );
		if( xItem.Is() )
		{
			// SvData basteln
			SvData aData( nFormat );
			if( xItem->GetData( &aData ) )
			{
				// Daten als Referenz holen, muss sowieso kopiert werden
				void* pData;
				if( aData.GetData( &pData, TRANSFER_REFERENCE ) )
				{
					ULONG nSize = min( nLen, GetDataLen( nItem, nFormat ) );
#ifdef W31
					hmemcpy( pBuf, pData, (size_t)nSize );
#else
					memcpy( pBuf, pData, (size_t)nSize );
#endif
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

ULONG DragServer::GetDataLen( USHORT nItem, ULONG nFormat )
{
	if( HasFormat( nItem, nFormat ) )
	{
		SotDataObjectRef xItem = ImplGetItem( nItem );
		if( xItem.Is() )
		{
			// SvData basteln
			SvData aData( nFormat );
			if( xItem->GetData( &aData ) )
				return aData.GetMinMemorySize();
		}
	}
	return 0L;
}

USHORT DragServer::GetFormatCount( USHORT nItem )
{
	SotDataObjectRef xItem = ImplGetItem( nItem );
	if( xItem.Is() )
		return (USHORT)xItem->GetTypeList().Count();
	return 0;
}

ULONG DragServer::GetFormat( USHORT nItem, USHORT nPos )
{
	SotDataObjectRef xItem = ImplGetItem( nItem );
	if( xItem.Is() )
		return xItem->GetTypeList().GetObject( nPos ).GetFormat();
	return 0L;
}

BOOL DragServer::HasFormat( USHORT nItem, ULONG nFormat )
{
	SotDataObjectRef xItem = ImplGetItem( nItem );
	if( xItem.Is() )
		return ( NULL != xItem->GetTypeList().Get( nFormat ) );
	return FALSE;
}

USHORT DragServer::GetRequestItem()
{
	// Vorlaeufig nicht notwendig, da Request zunaechst ausgeloest wird.
	/* SV:
	SVDATA* pSVData = GetSVData();
	if ( !pSVData->aClipD.pDragItemList )
		return 0;

	return (USHORT)pSVData->aClipD.pDragItemList->
					   GetPos( pSVData->aClipD.pDragReqItem );
	*/
	return 0;
}

ULONG DragServer::GetRequestFormat()
{
	ImplSVData* pSVData = ImplGetSVData();
	return pSVData->maExchangeData.mnDragReqFormat;
}

BOOL DragServer::IsRequestName()
{
	return FALSE;
}

String DragServer::GetTargetPrinterName()
{
	return ImplGetSVEmptyStr();
}

// Hilfsmethode, um aktuelles Drag-Item zu erhalten
SotDataObject* ImplGetActiveItem()
{
	ImplSVData* pSVData = ImplGetSVData();
	MultiItemObjectRef xData = DragManager::ImplGetDragManager()->mxDragServerData;
	if( !xData.Is() )
		xData = DragManager::ImplGetDragManager()->mxDragServerData = new MultiItemObject();

	return xData->GetItem( pSVData->maExchangeData.mnDragItem );
}

SotDataObject* ImplGetItem( USHORT nItem )
{
	MultiItemObjectRef xData = DragManager::ImplGetDragManager()->mxDragServerData;
	if( xData.Is() )
		return xData->GetItem( nItem );
	return &xData;
}

#endif // OLDDRAGDROP


//************************************************************
//*** Alte Drag&Drop-Schnittstelle am Window unterstuetzen ***
//************************************************************

// DropSource-Interface, um die Pointer zu setzen,
// die in ExecuteDrag uebergeben werden
class WindowDropSource : public DropSource
{
	const Pointer& rMovePtr;
	const Pointer& rCopyPtr;
	const Pointer& rLinkPtr;

public:
	WindowDropSource(	const Pointer& rMovePtr_,
						const Pointer& rCopyPtr_,
						const Pointer& rLinkPtr_ )
					  : rMovePtr( rMovePtr_ ),
						rCopyPtr( rCopyPtr_ ),
						rLinkPtr( rLinkPtr_ )
	{}

	virtual BOOL DrawDragPointer( DropAction eAction, const Point& rPos, Window* pTarget );
	virtual void ClrDragPointer( const Point& rPos );
};

BOOL WindowDropSource::DrawDragPointer
( DropAction eAction, const Point& rPos, Window* pTarget )
{
	// Drag&Drop muss aktiv sein, sonst werden wir nicht gerufen
	DragManager* pDragManager = DragManager::GetDragManager();
	if( pDragManager )
	{
		switch( eAction )
		{
			case DROP_MOVE:
				pDragManager->SetDragPointer( rMovePtr );
				break;
			case DROP_COPY:
				pDragManager->SetDragPointer( rCopyPtr );
				break;
			case DROP_LINK:
				pDragManager->SetDragPointer( rLinkPtr );
				break;
			default:
				pDragManager->SetDragPointer( Pointer( POINTER_NOTALLOWED ) );
		}
		return TRUE;
	}
	return FALSE;
}

void WindowDropSource::ClrDragPointer( const Point& rPos )
{
	// Die Pointer muessen hier nicht zurueckgesetzt werden
	// Das macht am Schluss die D&D-Implementation
}


DropAction Window::ExecuteDrag( const Pointer& rMovePtr,
								const Pointer& rCopyPtr,
								const Pointer& rLinkPtr,
								USHORT nOptions,
								const Region* )
{
	DropAction eDropResult = DROP_NONE;

	// Haben wir denn ein DataObject?
	MultiItemObjectRef xData = DragManager::ImplGetDragManager()->mxDragServerData;
	if( xData.Is() )
	{
		// DropSource anlegen, um die Pointer setzen zu koennen
		WindowDropSource aDropSource( rMovePtr, rCopyPtr, rLinkPtr );

		// Neue Schnittstelle aufrufen
		eDropResult = DragManager::ExecuteDrag
			( &xData, nOptions, &aDropSource );
	}
	return eDropResult;
}

DropAction Window::ExecuteDrag( IDataObject * pIDataObj,
								const Pointer& rMovePtr,
								const Pointer& rCopyPtr,
								const Pointer& rLinkPtr,
								USHORT nOptions )
{
	// TODO: Wie sieht dieses Daten-Object ueberhaupt aus?
	// Wer ruft die Schnittstelle? Nur SV-intern oder nur OS/2?
	DropAction eDropResult = DROP_NONE;

	return eDropResult;
}

DropAction Window::ExecuteDrag( const Pointer& rMovePtr,
								const Pointer& rCopyPtr,
								USHORT nOptions,
								const Region * pDropRegion )
{
	return ExecuteDrag( rMovePtr, rCopyPtr, rCopyPtr, nOptions, pDropRegion );
}

DropAction Window::ExecuteDrag( IDataObject * pIDataObj,
								const Pointer& rMovePtr,
								const Pointer& rCopyPtr,
								USHORT nOptions )
{
	return ExecuteDrag( pIDataObj, rMovePtr, rCopyPtr, rCopyPtr, nOptions );
}

