/*************************************************************************
 *
 *  $RCSfile: exchange.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_GETSYS_HXX
#include <getsys.hxx>
#endif
#ifndef _SV_EXCHANGE_H
#include <exchange.h>
#endif
#ifndef _SV_EXCHANGE_HXX
#include <exchange.hxx>
#endif
#ifndef _DEBUG_HXX //autogen
#include <tools/debug.hxx>
#endif

#ifdef MAC
#include "macex.hxx"
#endif

//********************************
//*** Implementation ImpPtrTab ***
//********************************

ImpPtrTab::~ImpPtrTab()
{
    ImpPtrTabData* pTemp;
    ImpPtrTabData* pTempLast = pLast;
    while ( pTempLast )
    {
        pTemp = pTempLast->pPrev;
        delete pTempLast;
        pTempLast = pTemp;
    }
}

void ImpPtrTab::Insert( void* pPtr )
{
    ImpPtrTabData* pTemp = new ImpPtrTabData;
    pTemp->pPtr = pPtr;
    pTemp->pPrev = pLast;
    pLast = pTemp;
}

void ImpPtrTab::Remove( void* pPtr )
{
    // Hier nicht loeschen, da wir ansonsten beim Handler hintenrum
    // wegloeschen koennten
    ImpPtrTabData* pTemp = pLast;
    while ( pTemp )
    {
        if ( pTemp->pPtr == pPtr )
        {
            pTemp->pPtr = NULL;
            break;
        }
        pTemp = pTemp->pPrev;
    }
}

void ImpPtrTab::Free()
{
    // Hier nicht loeschen, da wir ansonsten beim Handler hintenrum
    // wegloeschen koennten
    ImpPtrTabData* pDelTemp;
    ImpPtrTabData* pNext = NULL;
    ImpPtrTabData* pTemp = pLast;
    while ( pTemp )
    {
        if ( !pTemp->pPtr )
        {
            pDelTemp = pTemp;
            pTemp = pTemp->pPrev;
            if ( pNext )
                pNext->pPrev = pTemp;
            else
                pLast = pTemp;
            delete pDelTemp;
        }
        else
        {
            pNext = pTemp;
            pTemp = pTemp->pPrev;
        }
    }
}


//*******************************
//*** Implementation Exchange ***
//*******************************

Exchange::Exchange()
{
}

Exchange::~Exchange()
{
}

void Exchange::Request()
{
    aRequestHdl.Call( this );
}


//********************************************
//*** Dummy-Implementation Exchange-Klasse ***
//********************************************

/*
Exchange::Exchange()
{
}

Exchange::~Exchange()
{
}

ULONG Exchange::RegisterFormatName( const String& rName )
{
    return 0L;
}

String Exchange::GetFormatName( ULONG nFormat )
{
    return String();
}

void Exchange::Request()
{
}
*/


// SYSTEM_EXCHANGE

class _UnoSystemExchange : public ISystemExchange
{
    // Da nur noch die Strings als Formatkennzeichnung verwendet werden,
    // ist es nicht mehr ntig diese zu registrieren
    virtual void registerFormatName( const String& rName, UINT32 nFormatId );
};

void _UnoSystemExchange::registerFormatName( const String& rName, UINT32 nFormatId )
{
}

ISystemExchange* GetSystemExchange( void )
{
#ifdef MAC
	return GetMacExchange();
#else
    static _UnoSystemExchange Exchange;

    return &Exchange;
#endif
}


// Schnittstelle zum Registrieren der Formate am System
void Exchange::ImplRegisterSystemFormats( void )
{
	// HRO: Der ganze Quatsch hier ist ja wohl ueberfluessig
#if 0
    ISystemExchange* pXExchange = GetSystemExchange();
    DBG_ASSERT(pXExchange,"ImplRegisterSystemFormats(): No SystemExchange")

    // Prinzip: Diese Methode wird immer dann aufgerufen, wenn
    // per ImplGetSystemClipboard() oder ImplGetSystemDragManager()
    // erfolgreich ein System-Interface angefordert wurde. Hier
    // werden dann die Formate nachregistriert, die bislang noch
    // nicht am System registriert wurden.
    static ULONG iMaxRegistered = 0L;

    ULONG iMaxFormat = SotExchange::GetMaxFormat();
    if( iMaxRegistered < iMaxFormat )
    {
        for( ULONG i = iMaxRegistered ; i < iMaxFormat ; i++ )
        {
            String aStr = SotExchange::GetFormatName( i );

            // Es gibt auch 'leere' Formate (siehe SotExchange)
            if( aStr.Len() )
                pXExchange->registerFormatName( aStr, i );
        }
        iMaxRegistered = iMaxFormat;
    }
#endif
}


//**************************************************
//*** Daten-Schnittstellen fuer System-Anbindung ***
//**************************************************


ImplDTrans::ImplDTrans( SotDataObject* pDataObj )
{
    xDataObj = pDataObj;
    nRefCount = 0;
}

// In dieser Basisklasse gibt es nichts zu tun
ImplDTrans::~ImplDTrans()
{
}

// Ref-Count rauf
void ImplDTrans::aquire( void )
{
    nRefCount++;
}

// Ref-Count runter
void ImplDTrans::release( void )
{
    nRefCount--;
    if( nRefCount == 0 )
        delete this;
}

UINT32* ImplDTrans::getTypeList( /*Out*/UINT32& rSize ) const
{
    UINT32* pRet = NULL;
    if( xDataObj.Is() )
    {
        const SvDataTypeList& rTypeList = xDataObj->GetTypeList();
        rSize = rTypeList.Count();
        if( rSize > 0 )
        {
            pRet = new UINT32[rSize];
            for( UINT32 i = 0 ; i < rSize ; i++ )
                pRet[i] = rTypeList.GetObject( i ).GetFormat();
        }
    }
    return pRet;
}

BYTE* ImplDTrans::getData( UINT32 nFormat, /*Out*/UINT32& rSize )
{
    BYTE* pRet = NULL;
    if( xDataObj.Is() )
    {
        // SvData basteln
        SvData aData( nFormat );
        if( xDataObj->GetData( &aData ) )
        {
            rSize = aData.GetMinMemorySize();
            pRet = new BYTE[rSize];

            // Daten direkt in das Array kopieren lassen (TRANSFER_COPY == Default)
            aData.GetData( (void**)&pRet );
        }
    }
    return pRet;
}


// DataObject zum Wrappen einer DTrans-Interface-Instanz
DTransSotDataObject::DTransSotDataObject( IDataTrans* pDTrans_ )
{
    DBG_ASSERT(pDTrans_!=NULL, "DTransSotDataObject-Ctor, XDTrans==NULL !" )
    pDTrans = pDTrans_;
    pDTrans->aquire();
    pTypeList = NULL;
}

DTransSotDataObject::~DTransSotDataObject()
{
    pDTrans->release();
    delete pTypeList;
}

const SvDataTypeList & DTransSotDataObject::GetTypeList() const
{
    // Liste anlegen, wenn noch nicht vorhanden
    if( pTypeList == NULL )
    {
        ((DTransSotDataObject*)this)->pTypeList = new SvDataTypeList();
        UINT32 iSize = 0;
        UINT32* pList = pDTrans->getTypeList( iSize );

        // Liste aufbauen
        for( UINT32 i = 0 ; i < iSize ; i++ )
        {
            SvDataType aType( pList[i], MEDIUM_MEMORY );
            pTypeList->Append( aType );
        }

        delete pList;
    }
    return *pTypeList;
}

BOOL DTransSotDataObject::GetData( SvData* pData )
{
    UINT32 nSize;
    BYTE* pSrcData = pDTrans->getData( pData->GetFormat(), nSize );
    if( !pSrcData )
        return FALSE;
    pData->SetData( pSrcData, nSize, TRANSFER_MOVE );
    return TRUE;
}

