/*************************************************************************
 *
 *  $RCSfile: salselection.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:36 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <string.h>

#include <salselection.hxx>

#define MAX_TRY_CONVERTSELECTION 40

XLIB_Window   aSelectionWindow     = None;
Atom          nClipboardAtom       = 0;
Display*      pWexPrivateDisplay             = 0;

// static members
InterfaceHolder* InterfaceHolder::pGlobalHolder =
   new CopyFromSystemInterfaceHolder();

void InterfaceHolder::SetGlobalHolder( InterfaceHolder* pNewHolder )
{
	if( pGlobalHolder )
		delete pGlobalHolder;
	pGlobalHolder = pNewHolder;
}

// selection handler to set in libvcl
extern void (*SelectionRequestHandler)( XEvent & );

void LaunchSalSelection( Display *pOnDisplay )
{
	if( pOnDisplay )
	{
		pWexPrivateDisplay   = pOnDisplay;
		SelectionRequestHandler = HandleSelectionRequest;
	}
	
	if( aSelectionWindow == None )
	{
		aSelectionWindow = XCreateSimpleWindow( pWexPrivateDisplay,
												DefaultRootWindow( pWexPrivateDisplay ),
												10,10,10,10,0,0,1 );
	// will never be destroyed until client termination
	// have to do this becaus GetSalData()->GetDefDisp()->GetDrawable() doesnt seem to live long ...
	// why ???
	}
	if( ! nClipboardAtom )
		nClipboardAtom = XInternAtom( pWexPrivateDisplay, "CLIPBOARD", False );
}

void ShutdownSalSelection()
{
	if( aSelectionWindow != None )
	{
		XDestroyWindow( pWexPrivateDisplay, aSelectionWindow );
		aSelectionWindow = None;
	}
	nClipboardAtom = 0;
	pWexPrivateDisplay = 0;
}

BOOL CheckUnxClipboardChanged()
{
	if( ! pWexPrivateDisplay )
		return FALSE;
	
	XLIB_Window aPrimWindow = XGetSelectionOwner( pWexPrivateDisplay, XA_PRIMARY );
	XLIB_Window aClipWindow = nClipboardAtom ? XGetSelectionOwner( pWexPrivateDisplay, nClipboardAtom ) : 0;
	
	if( aPrimWindow != aSelectionWindow  && aPrimWindow != None )
		return TRUE;
	if( aClipWindow != aSelectionWindow && aClipWindow != None )
		return TRUE;
	return FALSE;
}

void HandleSelectionRequest( XEvent& rXEvent )
{
	if( ! pWexPrivateDisplay )
		return;
	
	if( rXEvent.type == SelectionClear )
	{
		if( InterfaceHolder::GetGlobalHolder()->mpListener )
			InterfaceHolder::GetGlobalHolder()->mpListener->changed();
		
		InterfaceHolder::SetGlobalHolder( new CopyFromSystemInterfaceHolder() );
	}
	else if( rXEvent.type == SelectionRequest )
	{
		BYTE   *pSelectionBuffer;
		UINT32 nSelectionBufferSize;
		
		XSelectionRequestEvent &rSelEvent = rXEvent.xselectionrequest;
		XEvent aSendEvent;
		BOOL bConvertable = FALSE;

		if( rSelEvent.target == XA_STRING )
		{
			SequenceOfString Formats =
				InterfaceHolder::GetGlobalHolder()->getTypeList();
			for( int i=0; i < Formats.getLength(); i++ )
			{
				if( ! strcasecmp( Formats[i], FORMAT_TXT ) )
				{
					SequenceOfByte Bytes =
						InterfaceHolder::GetGlobalHolder()->getData();
					pSelectionBuffer = (BYTE*)Bytes.getPointer();
					nSelectionBufferSize = Bytes.getLength();
					XChangeProperty( pWexPrivateDisplay, rSelEvent.requestor,
									 rSelEvent.property, XA_STRING,
									 8, PropModeReplace, pSelectionBuffer,
									 nSelectionBufferSize );
					bConvertable = TRUE;
					break;
				}
			}
		}
#if defined DEBUG || defined DBG_UTIL
		fprintf( stderr, "Received %s SelectionRequest %sfor target %d:%s on selection %s\n",
				 bConvertable ? "convertable" : "inconvertable",
				 rSelEvent.property == None ?
				 "from an obsolete client" : "",
				 rSelEvent.target,
				 XGetAtomName( pWexPrivateDisplay, rSelEvent.target ),
				 XGetAtomName( pWexPrivateDisplay, rSelEvent.selection )
				 );
#endif

		aSendEvent.type                   = SelectionNotify;
		aSendEvent.xselection.display     = rSelEvent.display;
		aSendEvent.xselection.send_event  = True;
		aSendEvent.xselection.requestor   = rSelEvent.requestor;
		aSendEvent.xselection.selection   = rSelEvent.selection;
		aSendEvent.xselection.target      = XA_STRING;
		aSendEvent.xselection.property    =
			bConvertable ? rSelEvent.property : None;
		aSendEvent.xselection.time        = rSelEvent.time;
		// xterm seems to be the only one to care about time = CurrentTime
		
		if( ! XSendEvent( pWexPrivateDisplay, rSelEvent.requestor,
						  False, 0, &aSendEvent ) )
			fprintf( stderr, "HandleSelectionRequest: XSendEvent failed\n" );
	}
}

SequenceOfByte CopyFromSystemInterfaceHolder::getData()
{
	SequenceOfByte Return;
	UINT32         nSize = 0;

	if( ! pWexPrivateDisplay )
		return Return;
	
	// check if there IS something to copy
	XLIB_Window aOwner = XGetSelectionOwner( pWexPrivateDisplay, XA_PRIMARY );
	if( aOwner == None || aOwner == aSelectionWindow )
		return Return;

	// get the proper Atoms
	Atom aPropertyAtom = XInternAtom( pWexPrivateDisplay, "VCLExchangeProperty", False );
	Atom aTypeAtom   = XA_STRING;

	// now initiate communication with selection owner
	XConvertSelection( pWexPrivateDisplay, XA_PRIMARY, aTypeAtom, aPropertyAtom,
					   aSelectionWindow, CurrentTime );
	// wait for selection notify
	XEvent aXEvent;
	int nTries;
	for( nTries = 0;
		 ! XCheckTypedEvent( pWexPrivateDisplay, SelectionNotify, &aXEvent ) &&
			 nTries <= MAX_TRY_CONVERTSELECTION ;
		 nTries++ )
		usleep( 50000 );

	// check if request could be converted
	if( nTries >= MAX_TRY_CONVERTSELECTION ||
		aXEvent.xselection.property != aPropertyAtom && nClipboardAtom )
	{
		// let us try CLIPBOARD selection
		nTries = 0;
		XConvertSelection( pWexPrivateDisplay, nClipboardAtom, aTypeAtom, aPropertyAtom,
						   aSelectionWindow, CurrentTime );
		for( nTries = 0;
			 ! XCheckTypedEvent( pWexPrivateDisplay, SelectionNotify, &aXEvent ) &&
				 nTries < MAX_TRY_CONVERTSELECTION ;
			 nTries++ )
			usleep( 50000 );
	}

	// check if request could be converted
	if( nTries >= MAX_TRY_CONVERTSELECTION ||
		aXEvent.xselection.property != aPropertyAtom )
		return Return;

	Atom aActualType;
	int  nActualFormat;
	unsigned long nItems;
	unsigned long nBytesAfter;
	unsigned char *pData=0;
	XGetWindowProperty( pWexPrivateDisplay, aSelectionWindow, aPropertyAtom,
						0,          // long_offset
						16384,      // long_length, a maximum of 64k will e retrieved
						True,       // delete property after reading
						XA_STRING,  // the type we want
						&aActualType, &nActualFormat,
						&nItems, &nBytesAfter, &pData );
	
	if( pData )
	{
		if( nBytesAfter )
			fprintf( stderr, "%d bytes remain in property after copying selection\n", nBytesAfter );
		nSize = (UINT32) nItems * (nActualFormat / 8);
		Return.setLength( nSize+1 );
		memcpy( Return.getPointer(), pData, nSize );
		Return[ nSize ] = 0;
		XFree( pData );
	}
	return Return;
}

SequenceOfString CopyFromSystemInterfaceHolder::getTypeList()
{
	SequenceOfString Strings;
	
	if( CheckUnxClipboardChanged() )
	{
		Strings.setLength( 1 );
		Strings[ 0 ] = FORMAT_TXT;
	}
	return Strings;
}

SequenceOfByte CopyToSystemInterfaceHolder::getData()
{
	return mxData.getData( FORMAT_TXT );
}

SequenceOfString CopyToSystemInterfaceHolder::getTypeList()
{
	return mxData.getTypeList();
}
