/*************************************************************************
 *
 *  $RCSfile: unxex.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdlib.h>
#include <string.h>

#include <uno/uno.h>
#include <uno/object.hxx>
#include <vos/diagnose.hxx>
#include <sysex.hxx>
#include <salselection.hxx>
#include <unxex.hxx>
#include <sysdata.hxx>

#ifndef _UNO_NO_NAMESPACE
using namespace uno;
#endif

class OSystemExchange : public OUnoObject
{
// Construction and destruction
public:
	OSystemExchange( ORefCounter &rRefCounter );
	virtual ~OSystemExchange();

// Attributes
protected:
	XSystemEnvironment	m_Environment;
	XClipboardListener  m_ClipboardListener;
	XDragDropListener   m_DragDropListener;
	OUnoAny				m_DragDropFrame;
	XDataTransfer		m_DataTransfer;

// Implementaion
protected:
    UNO_BEGIN_INTERFACE_PART(TSystemExchange, XSystemExchange)
	    void setSystemEnvironment(XSystemEnvironment& Environment);
    UNO_END_INTERFACE_PART(TSystemExchange)

    UNO_BEGIN_INTERFACE_PART(TSystemClipboard, XSystemClipboard)
	    void copy(XDataTransfer& Data);
	    XDataTransfer paste(void);
	    void clear(void);
	    void registerListener(XClipboardListener& xListener);
    UNO_END_INTERFACE_PART(TSystemClipboard)

    UNO_BEGIN_INTERFACE_PART(TSystemDragManager, XSystemDragManager)
    	Short executeDrag(XDataTransfer& Data, Short nOptions);
        void registerListener(XDragDropListener& Listener);
        void enableDrop(const OUnoAny& Frame, Boolean Enable);
    UNO_END_INTERFACE_PART(TSystemDragManager)

	UNO_DECLARE_INTERFACE_TABLE()
};

UNO_BEGIN_INTERFACE_TABLE(OSystemExchange, OUnoObject)
	UNO_INTERFACE_PART(OSystemExchange, TSystemExchange, XSystemExchange)	
	UNO_INTERFACE_PART(OSystemExchange, TSystemClipboard, XSystemClipboard)	
	UNO_INTERFACE_PART(OSystemExchange, TSystemDragManager, XSystemDragManager)	
UNO_END_INTERFACE_TABLE()


// Implementation of XInterface interface
UNO_IMPLEMENT_XINTERFACE(OSystemExchange, TSystemExchange)
UNO_IMPLEMENT_XINTERFACE(OSystemExchange, TSystemClipboard)
UNO_IMPLEMENT_XINTERFACE(OSystemExchange, TSystemDragManager)

OSystemExchange::OSystemExchange( ORefCounter &rRefCounter )
	: OUnoObject( rRefCounter )
{
}

OSystemExchange::~OSystemExchange()
{
	ShutdownSalSelection();
}

/*****************************************************************************/

void OSystemExchange::TSystemExchange::setSystemEnvironment(XSystemEnvironment& Environment)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemExchange);

	This->m_Environment = Environment;
}

/*****************************************************************************/

SequenceOfString WexUnoDataTransferAdaptor::getTypeList(void)
{
	if( InterfaceHolder::GetGlobalHolder() )
		return InterfaceHolder::GetGlobalHolder()->getTypeList();
	else
		return SequenceOfString();
}

/*****************************************************************************/

SequenceOfByte WexUnoDataTransferAdaptor::getData(const OUnoString& Format)
{
	if( ! strcasecmp( FORMAT_TXT, Format ) )
	{
		if( InterfaceHolder::GetGlobalHolder()  )
			return InterfaceHolder::GetGlobalHolder()->getData();
	}
	else
	{
		SequenceOfByte	Bytes;
		return Bytes;
	}
}

/*****************************************************************************/

void OSystemExchange::TSystemClipboard::copy(XDataTransfer& xData)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	InterfaceHolder::SetGlobalHolder(new CopyToSystemInterfaceHolder( xData, &( This->m_ClipboardListener ) ) );

	if( pWexPrivateDisplay )
	{
		XSetSelectionOwner( pWexPrivateDisplay, XA_PRIMARY, aSelectionWindow,
							CurrentTime );
		if( nClipboardAtom )
			XSetSelectionOwner( pWexPrivateDisplay, nClipboardAtom,
								aSelectionWindow, CurrentTime );

#if defined DBG_UTIL || defined DEBUG
		if(  XGetSelectionOwner( pWexPrivateDisplay, XA_PRIMARY ) != aSelectionWindow )
			fprintf( stderr, "Could not acquire ownership of PRIMARY\n" );
		if(  nClipboardAtom &&
			 XGetSelectionOwner( pWexPrivateDisplay, nClipboardAtom ) !=
			 aSelectionWindow )
			fprintf( stderr, "Could not acquire ownership of CLIPBOARD\n" );
#endif
	}
	
	This->m_ClipboardListener.changed();
}

XDataTransfer OSystemExchange::TSystemClipboard::paste(void)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	XDataTransfer DataTransfer;

	WexUnoDataTransferAdaptor *pAdaptor =
		new WexUnoDataTransferAdaptor();
	pAdaptor->queryInterface(UNO_UIK(XDataTransfer), DataTransfer);

	return DataTransfer;
}

void OSystemExchange::TSystemClipboard::clear(void)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);
	
	This->m_ClipboardListener.cleared();
}

void OSystemExchange::TSystemClipboard::registerListener(XClipboardListener& xListener)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	This->m_ClipboardListener = xListener;
}

/*****************************************************************************/

Short OSystemExchange::TSystemDragManager::executeDrag(XDataTransfer& Data, Short nOptions)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemDragManager);

	return SYSEX_DROP_NONE;
}

void OSystemExchange::TSystemDragManager::registerListener(XDragDropListener& Listener)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemDragManager);

	This->m_DragDropListener = Listener;
}

void OSystemExchange::TSystemDragManager::enableDrop(const OUnoAny& Frame, Boolean Enable)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemDragManager);

	if( Enable )
	{
		This->m_DragDropFrame = Frame;
		
		OUnoAny Window = This->m_Environment.getSystemRepresentation(Frame);
		ULONG Data;
		Window >>= Data;
		LaunchSalSelection( ((SystemChildData*)Data)->pDisplay );
	}
}

/*****************************************************************************/

////////////////////////////////////////////////////////////////////////////

UNO_DEFINE_FACTORY( OSystemExchange, SystemExchange.stardiv.de )

UNO_BEGIN_INIT_MODULE 
UNO_REGISTER_FACTORY( OSystemExchange, SystemExchange.stardiv.de )
UNO_END_INIT_MODULE 

WexUnoDataTransferAdaptor::~WexUnoDataTransferAdaptor()
{
//	if( mpTransfer )
//		mpTransfer->release();
}

