/*************************************************************************
 *
 *  $RCSfile: windrag.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <uno/uno.h>
#include <uno/object.hxx>
#include <vos/diagnose.hxx>
#include <sysex.hxx>
#include "winole.hxx"
#include "winex.hxx"


#ifndef _NO_UNO_NAMESPACE
using namespace uno;
#endif



ISVDropTarget::ISVDropTarget(HWND hWnd, const OUnoAny& Frame, XDragDropListener& xListener)
	: nRef( 0 )
{
	m_Frame = Frame;
	m_Listener = xListener;
	m_hWnd = hWnd;
}

ISVDropTarget::~ISVDropTarget()
{
	nRef = nRef;
}

STDMETHODIMP_(ULONG) ISVDropTarget::AddRef()
{
	return ++nRef;
}

STDMETHODIMP_(ULONG) ISVDropTarget::Release()								  
{																		  
    if( 0 == --nRef )
    {
        delete this;
        return 0;
    }
    else
        return nRef;
}

STDMETHODIMP ISVDropTarget::QueryInterface( REFIID nId, LPVOID FAR * ppObj )  
{																		  
	if( nId == IID_IDropTarget || nId == IID_IUnknown )
	{
		nRef++;
		*ppObj = this;
		return NOERROR;
	}
	return ResultFromScode( E_NOINTERFACE );
}																		  

STDMETHODIMP ISVDropTarget::DragEnter(IDataObject *pDataObject, DWORD grfKeyState, POINTL pt, DWORD *pdwEffect)
{

	XDataTransfer DataTransfer;

	UnoDataTransferAdaptor *pAdaptor = new UnoDataTransferAdaptor(pDataObject);
		
	pAdaptor->queryInterface(UNO_UIK(XDataTransfer), DataTransfer);

	Short	nSrcOptions = SYSEX_DROP_NONE;

	if (*pdwEffect & DROPEFFECT_COPY)
		nSrcOptions |= SYSEX_DROP_COPY;
	if (*pdwEffect & DROPEFFECT_MOVE)
		nSrcOptions |= SYSEX_DROP_MOVE;
	if (*pdwEffect & DROPEFFECT_LINK)
		nSrcOptions |= SYSEX_DROP_LINK;

	m_Listener.beginDragDrop(DataTransfer, nSrcOptions);


	return DragOver(grfKeyState, pt, pdwEffect);
}


STDMETHODIMP ISVDropTarget::DragOver(DWORD grfKeyState, POINTL ptl, DWORD *pdwEffect)
{
	USHORT nModKeys = 0;

	if (grfKeyState & MK_SHIFT)
		nModKeys |= SYSEX_KEY_SHIFT;
	if (grfKeyState & MK_CONTROL)
		nModKeys |= SYSEX_KEY_MOD1;

	POINT pt;

	pt.x = ptl.x; pt.y = ptl.y;

	ScreenToClient(m_hWnd, &pt);

    Short nDropAction = m_Listener.queryDrop(pt.x, pt.y, m_Frame, nModKeys);

	if (nDropAction & SYSEX_DROP_MOVE)
		*pdwEffect |= DROPEFFECT_MOVE;
	else
		*pdwEffect &= ~DROPEFFECT_MOVE;

	if (nDropAction & SYSEX_DROP_COPY)
		*pdwEffect |= DROPEFFECT_COPY;
	else
		*pdwEffect &= ~DROPEFFECT_COPY;

	if (nDropAction & SYSEX_DROP_LINK)
		*pdwEffect |= DROPEFFECT_LINK;
	else
		*pdwEffect &= ~DROPEFFECT_LINK;

	return NOERROR;
}


STDMETHODIMP ISVDropTarget::DragLeave()
{
	m_Listener.endDragDrop();

	return S_OK;
}

STDMETHODIMP ISVDropTarget::Drop(IDataObject *, DWORD grfKeyState, POINTL ptl, DWORD *pdwEffect)
{
	USHORT nModKeys = 0;

	if (grfKeyState & MK_SHIFT)
		nModKeys |= SYSEX_KEY_SHIFT;
	if (grfKeyState & MK_CONTROL)
		nModKeys |= SYSEX_KEY_MOD1;

	POINT pt;

	pt.x = ptl.x; pt.y = ptl.y;

	ScreenToClient(m_hWnd, &pt);

    Short	nDropAction = m_Listener.drop(pt.x, pt.y, m_Frame, nModKeys);

	if (nDropAction & SYSEX_DROP_MOVE)
		*pdwEffect |= DROPEFFECT_MOVE;
	else
		*pdwEffect &= ~DROPEFFECT_MOVE;

	if (nDropAction & SYSEX_DROP_COPY)
		*pdwEffect |= DROPEFFECT_COPY;
	else
		*pdwEffect &= ~DROPEFFECT_COPY;

	if (nDropAction & SYSEX_DROP_LINK)
		*pdwEffect |= DROPEFFECT_LINK;
	else
		*pdwEffect &= ~DROPEFFECT_LINK;

	return ResultFromScode(S_OK);
}


//#########################################################

ISVDropSource::ISVDropSource()
	: nRef(0)
{
}

STDMETHODIMP_(ULONG) ISVDropSource::AddRef()
{
	return ++nRef;
}

STDMETHODIMP_(ULONG) ISVDropSource::Release()								  
{																		  
    if( 0 == --nRef )
    {
        delete this;
        return 0;
    }
    else
        return nRef;
}

STDMETHODIMP ISVDropSource::QueryInterface( REFIID nId, LPVOID FAR * ppObj )  
{																		  
	if( nId == IID_IDropSource || nId == IID_IUnknown )
	{
		nRef++;
		*ppObj = this;
		return NOERROR;
	}
	return ResultFromScode( E_NOINTERFACE );
}																		  

STDMETHODIMP ISVDropSource::GiveFeedback(DWORD dwEffect)
{
	return ResultFromScode(DRAGDROP_S_USEDEFAULTCURSORS);
}

STDMETHODIMP ISVDropSource::QueryContinueDrag(BOOL fEscapePressed, DWORD grfKeyState)
{
	if (fEscapePressed)
		return ResultFromScode(DRAGDROP_S_CANCEL);

	if (!(grfKeyState & MK_LBUTTON))
		return ResultFromScode(DRAGDROP_S_DROP);

	return NOERROR;
}

