/*************************************************************************
 *
 *  $RCSfile: winex.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ole2.h>
#include <uno/uno.h>
#include <uno/object.hxx>
#include <vos/diagnose.hxx>
#include <sysex.hxx>
#include "winole.hxx"
#include "winex.hxx"


#ifndef _NO_UNO_NAMESPACE
using namespace uno;
#endif


// -------------------------------------------------------------------------------------------

#define FORMAT_TXT		"Text"
#define FORMAT_MTF		"GDIMetaFile"
#define FORMAT_DIB		"Bitmap"
#define FORMAT_DDB		"Bitmap"
#define FORMAT_FILES	"FileList"

// -------------------------------------------------------------------------------------------

#pragma pack(2)
struct METAFILEHEADER
{
    DWORD       key;
    short       hmf;
    SMALL_RECT  bbox;
    WORD        inch;
    DWORD       reserved;
    WORD        checksum;
};
#pragma pack()

// -------------------------------------------------------------------------------------------

struct SDropTarget
{
	HWND		hWnd;
	IDropTarget	*pIDropTarget;
};

class OSystemExchange : public OUnoObject
{
// Construction and destruction
public:
	OSystemExchange( ORefCounter &rRefCounter );
	virtual ~OSystemExchange();

// Attributes
protected:
	XSystemEnvironment	m_Environment;
	XClipboardListener  m_ClipboardListener;
	XDragDropListener   m_DragDropListener;
	SDropTarget     	m_Targets[50];
	unsigned long		numTargets;

// Implementaion
protected:
    UNO_BEGIN_INTERFACE_PART(TSystemExchange, XSystemExchange)
	    void setSystemEnvironment(XSystemEnvironment& Environment);
    UNO_END_INTERFACE_PART(TSystemExchange)

    UNO_BEGIN_INTERFACE_PART(TSystemClipboard, XSystemClipboard)
	    void copy(XDataTransfer& Data);
	    XDataTransfer paste(void);
	    void clear(void);
	    void registerListener(XClipboardListener& xListener);
    UNO_END_INTERFACE_PART(TSystemClipboard)

    UNO_BEGIN_INTERFACE_PART(TSystemDragManager, XSystemDragManager)
    	Short executeDrag(XDataTransfer& Data, Short nOptions);
        void registerListener(XDragDropListener& Listener);
        void enableDrop(const OUnoAny& Frame, Boolean Enable);
    UNO_END_INTERFACE_PART(TSystemDragManager)

	UNO_DECLARE_INTERFACE_TABLE()
};

UNO_BEGIN_INTERFACE_TABLE(OSystemExchange, OUnoObject)
	UNO_INTERFACE_PART(OSystemExchange, TSystemExchange, XSystemExchange)	
	UNO_INTERFACE_PART(OSystemExchange, TSystemClipboard, XSystemClipboard)	
	UNO_INTERFACE_PART(OSystemExchange, TSystemDragManager, XSystemDragManager)	
UNO_END_INTERFACE_TABLE()


// Implementation of XInterface interface
UNO_IMPLEMENT_XINTERFACE(OSystemExchange, TSystemExchange)
UNO_IMPLEMENT_XINTERFACE(OSystemExchange, TSystemClipboard)
UNO_IMPLEMENT_XINTERFACE(OSystemExchange, TSystemDragManager)

OSystemExchange::OSystemExchange( ORefCounter &rRefCounter )
	: OUnoObject( rRefCounter )
	, numTargets(0)
{
	VOS_VERIFY(OleInitialize(NULL) == S_OK);
}

OSystemExchange::~OSystemExchange()
{
	OleUninitialize;
}

/*****************************************************************************/

void OSystemExchange::TSystemExchange::setSystemEnvironment(XSystemEnvironment& Environment)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemExchange);

	This->m_Environment = Environment;
}

/*****************************************************************************/

SequenceOfString UnoDataTransferAdaptor::getTypeList(void)
{
	IDataObject			*pDataObj = m_Object;
	SequenceOfString	Strings;
	int					nFormats = 0;

//	pDataObj->AddRef();

// Wenn wir ein Interface haben, holen wir uns das Interface zum Abfragen der Typen

	IEnumFORMATETC	*pEnumFormats;

	if (SUCCEEDED(pDataObj->EnumFormatEtc(DATADIR_GET, &pEnumFormats))) 
	{
		FORMATETC fe;

		while (S_OK == pEnumFormats->Next(1, &fe, NULL)) 
		{
			if (fe.dwAspect != DVASPECT_CONTENT || fe.lindex != -1 )
				continue;

			switch( fe.cfFormat) 
			{
				case CF_TEXT:
					Strings.setLength(nFormats+1);
					Strings[nFormats++] = FORMAT_TXT;
				break;

				case CF_HDROP:
					Strings.setLength(nFormats+1);
					Strings[nFormats++] = FORMAT_FILES;
					break;

				case CF_METAFILEPICT:
					Strings.setLength(nFormats+1);
					Strings[nFormats++] = FORMAT_MTF;
				break;
		
				case CF_DIB:
					Strings.setLength(nFormats+1);
					Strings[nFormats++] = FORMAT_DIB;
				break;

				case CF_BITMAP:
/* no DDB's yet
					Strings.setLength(nFormats+1);
					Strings[nFormats++] = FORMAT_DDB;
*/
				break;
				
				default: 
				{
					TCHAR	szFormatName[256];
					if (GetClipboardFormatName(fe.cfFormat, szFormatName, sizeof(szFormatName) / sizeof(TCHAR)) > 0) 
					{
						Strings.setLength(nFormats+1);
						Strings[nFormats++] = szFormatName;
					}
				}
				break;
			}
		}

// IEnumFORMATETC-Interface freigeben

		pEnumFormats->Release();
	}

// Zum Schluss noch das IDataObject-Interface wiedert freigeben

//	pDataObj->Release();

	return Strings;

}

/*****************************************************************************/

SequenceOfByte UnoDataTransferAdaptor::getData(const OUnoString& Format)
{
	const char		*pszFormatName = Format;
	SequenceOfByte	Bytes;
	IDataObject		*pDataObj = m_Object;
	FORMATETC		fe;
	STGMEDIUM		stm;

// Zum Schluss noch das IDataObject-Interface wiedert freigeben

	if( stricmp( pszFormatName, FORMAT_TXT ) == 0 )
		fe.cfFormat = CF_TEXT;
	else if( stricmp( pszFormatName, FORMAT_FILES ) == 0)
	{
		fe.cfFormat = CF_HDROP;
		fe.ptd = NULL;
		fe.dwAspect = DVASPECT_CONTENT;
		fe.lindex = -1;
		fe.tymed = TYMED_HGLOBAL;

		HRESULT	result = pDataObj->GetData(&fe, &stm);

		if (S_OK == result && NULL != stm.hGlobal) 
		{
			LPVOID	lpGlobal = GlobalLock(stm.hGlobal);
			DWORD	dwSize = GlobalSize(stm.hGlobal);
			Bytes.setSeq((Byte *)lpGlobal, dwSize);
			GlobalUnlock(stm.hGlobal);
			ReleaseStgMedium(&stm);
		}
	}
	else if( stricmp( pszFormatName, "GDIMetafile" ) == 0 )
	{
		fe.cfFormat = CF_METAFILEPICT;
		fe.ptd = NULL;
		fe.dwAspect = DVASPECT_CONTENT;
		fe.lindex = -1;
		fe.tymed = TYMED_MFPICT;

		HRESULT	result = pDataObj->GetData( &fe, &stm );

		if( S_OK == result && stm.hMetaFilePict ) 
		{
			METAFILEPICT*	pPict = (METAFILEPICT*) GlobalLock( stm.hMetaFilePict );
			HMETAFILE		hMtf = pPict->hMF;
			const ULONG		nCount = GetMetaFileBitsEx( hMtf, 0, NULL );
			BYTE*			pBuf = new BYTE[ nCount + sizeof( METAFILEHEADER ) ];
			METAFILEHEADER*	pHeader = (METAFILEHEADER*) pBuf;
			SMALL_RECT      aRect = { 0, 0, (short) pPict->xExt, (short) pPict->yExt };
			USHORT			nInch;

			switch( pPict->mm )
			{
				case( MM_TEXT ):
					nInch = 72;
				break;

				case( MM_LOMETRIC ):
					nInch = 100;
				break;

				case( MM_HIMETRIC ):
					nInch = 1000;
				break;

				case( MM_LOENGLISH ):
					nInch = 254;
				break;

				case( MM_HIENGLISH ):
				case( MM_ISOTROPIC ):
				case( MM_ANISOTROPIC ):
					nInch = 2540;
				break;

				case( MM_TWIPS ):
					nInch = 1440;
				break;

				default:
					nInch = 576;
				break;
			}

			pHeader->key = 0x9AC6CDD7L;
			pHeader->hmf = 0;
			pHeader->bbox = aRect;
			pHeader->inch = nInch;
			pHeader->reserved = 0;
			pHeader->checksum = 0;

			if( nCount == GetMetaFileBitsEx( hMtf, nCount, pBuf + sizeof( METAFILEHEADER ) ) )
				Bytes.setSeq( pBuf, nCount + sizeof( METAFILEHEADER ) );

			delete[] pBuf;
			GlobalUnlock( stm.hMetaFilePict );
			ReleaseStgMedium( &stm );
			fe.cfFormat = 0;
		}		  
	}
	else if( stricmp( pszFormatName, FORMAT_DIB ) == 0)
	{
		fe.cfFormat = CF_DIB;
		fe.ptd = NULL;
		fe.dwAspect = DVASPECT_CONTENT;
		fe.lindex = -1;
		fe.tymed = TYMED_HGLOBAL;

		HRESULT	result = pDataObj->GetData( &fe, &stm );

		if( S_OK == result && stm.hMetaFilePict ) 
		{
			const ULONG			nSize = GlobalSize( stm.hGlobal );
			BYTE*				pDIB = (BYTE*) GlobalLock( stm.hGlobal );
			BYTE*				pExtDIB = new BYTE[ nSize + sizeof( BITMAPFILEHEADER ) ];
			DWORD				nOffset = sizeof( BITMAPFILEHEADER ) + sizeof( BITMAPINFOHEADER );
			BITMAPINFOHEADER*	pInfo = (BITMAPINFOHEADER*) pDIB;

			memset( pExtDIB, 0, sizeof( BITMAPFILEHEADER ) );

			if( pInfo->biBitCount <= 8 )
				nOffset += ( pInfo->biClrUsed ? pInfo->biClrUsed : ( 1 << pInfo->biBitCount ) ) << 2;

			*(WORD*) pExtDIB = 0x4D42;
			*(DWORD*)( pExtDIB + 10 ) = nOffset;

			memcpy( pExtDIB + sizeof( BITMAPFILEHEADER ), pDIB, nSize );
			Bytes.setSeq( pExtDIB, nSize + sizeof( BITMAPFILEHEADER ) );
			
			delete[] pExtDIB;
			GlobalUnlock( stm.hGlobal );
			ReleaseStgMedium( &stm );
			fe.cfFormat = 0;
		}
	}
	else if( stricmp( pszFormatName, FORMAT_DDB ) == 0 )
		fe.cfFormat = CF_BITMAP;
	else
		fe.cfFormat = RegisterClipboardFormat(pszFormatName);

	if( fe.cfFormat > 0 )
	{
		fe.ptd = NULL;
		fe.dwAspect = DVASPECT_CONTENT;
		fe.lindex = -1;
		fe.tymed = TYMED_HGLOBAL;

		HRESULT	result = pDataObj->GetData(&fe, &stm);

		if (S_OK == result && NULL != stm.hGlobal) 
		{
			LPVOID	lpGlobal = GlobalLock(stm.hGlobal);
			DWORD	dwSize = GlobalSize(stm.hGlobal);
			Bytes.setSeq((Byte *)lpGlobal, dwSize);
			GlobalUnlock(stm.hGlobal);
			ReleaseStgMedium(&stm);
		}
	}

	return Bytes;

}

/*****************************************************************************/

void OSystemExchange::TSystemClipboard::copy(XDataTransfer& xData)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	OleSetClipboard(new ISVClipDataObject(xData, This->m_ClipboardListener));
}

XDataTransfer OSystemExchange::TSystemClipboard::paste(void)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	IDataObject*  pDataObj;
	XDataTransfer DataTransfer;

	if (SUCCEEDED(OleGetClipboard(&pDataObj)))
	{
		UnoDataTransferAdaptor *pAdaptor = new UnoDataTransferAdaptor(pDataObj);
		
		pAdaptor->queryInterface(UNO_UIK(XDataTransfer), DataTransfer);
	}

	return DataTransfer;
}

void OSystemExchange::TSystemClipboard::clear(void)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	OleFlushClipboard();

	This->m_ClipboardListener.cleared();
}

void OSystemExchange::TSystemClipboard::registerListener(XClipboardListener& xListener)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemClipboard);

	This->m_ClipboardListener = xListener;
}

/*****************************************************************************/

Short OSystemExchange::TSystemDragManager::executeDrag(XDataTransfer& Data, Short nOptions)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemDragManager);

	IDropSource	*pIDropSource = new ISVDropSource();
	IDataObject	*pIDataObject = new ISVDataObject(Data);

	pIDropSource->AddRef();

	DWORD	dwOKEffect = DROPEFFECT_NONE;

	if (nOptions & SYSEX_DROP_COPY)
		dwOKEffect |= DROPEFFECT_COPY;

	if (nOptions & SYSEX_DROP_MOVE)
		dwOKEffect |= DROPEFFECT_MOVE;

	if (nOptions & SYSEX_DROP_LINK)
		dwOKEffect |= DROPEFFECT_LINK;

	DWORD	dwEffect;

	SCODE	sCode = GetScode(DoDragDrop(pIDataObject, pIDropSource, dwOKEffect, &dwEffect));

	pIDataObject->Release();

	Short	nSysDropAction = SYSEX_DROP_NONE;

	switch (sCode) {
		case DRAGDROP_S_DROP:
			if (dwEffect & DROPEFFECT_COPY) {
				nSysDropAction = SYSEX_DROP_COPY;
				break;
			}
			if (dwEffect & DROPEFFECT_LINK) {
				nSysDropAction = SYSEX_DROP_LINK;
				break;
			}
			if (dwEffect & DROPEFFECT_MOVE) {
				nSysDropAction = SYSEX_DROP_MOVE;
				break;
			}
			nSysDropAction = SYSEX_DROP_NONE;
			break;
		case DRAGDROP_S_CANCEL:
			nSysDropAction = SYSEX_DROP_CANCEL;
			break;
		default:
			nSysDropAction = SYSEX_DROP_NONE;
			break;
	}

	pIDropSource->Release();

	return nSysDropAction;
}

void OSystemExchange::TSystemDragManager::registerListener(XDragDropListener& Listener)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemDragManager);

	This->m_DragDropListener = Listener;
}

void OSystemExchange::TSystemDragManager::enableDrop(const OUnoAny& Frame, Boolean Enable)
{
	UNO_METHOD_PROLOGUE(OSystemExchange, TSystemDragManager);
	IDropTarget *pIDropTarget = NULL;

	OUnoAny Window = This->m_Environment.getSystemRepresentation(Frame);

	ULONG	nhWnd;

	Window >>= nhWnd;

	HWND	hWnd = (HWND)nhWnd;

	if (IsWindow(hWnd))
	{
		// register OLE drag & drop
		if (Enable) {
			pIDropTarget = new ISVDropTarget(hWnd, Frame, This->m_DragDropListener);
			pIDropTarget->AddRef();
			CoLockObjectExternal(pIDropTarget, TRUE, FALSE);
			RegisterDragDrop(hWnd, pIDropTarget);
			This->m_Targets[This->numTargets].hWnd = hWnd;
			This->m_Targets[This->numTargets].pIDropTarget = pIDropTarget;
			This->numTargets++;
		} else {
			ULONG	nCurTarget;

			for (nCurTarget = 0; nCurTarget < This->numTargets; nCurTarget++)
			{
				if (This->m_Targets[nCurTarget].hWnd == hWnd) {
					pIDropTarget = This->m_Targets[nCurTarget].pIDropTarget;
					break;
				}
			}
			
			if (pIDropTarget) {
				memmove(&This->m_Targets[nCurTarget], &This->m_Targets[nCurTarget], 
						(--This->numTargets - nCurTarget) * sizeof(SDropTarget));
				RevokeDragDrop(hWnd);
				CoLockObjectExternal(pIDropTarget, FALSE, TRUE);
				pIDropTarget->Release();
			}
		}

	}
}

/*****************************************************************************/

////////////////////////////////////////////////////////////////////////////

UNO_DEFINE_FACTORY( OSystemExchange, SystemExchange.stardiv.de )

UNO_BEGIN_INIT_MODULE 
UNO_REGISTER_FACTORY( OSystemExchange, SystemExchange.stardiv.de )
UNO_END_INIT_MODULE 

