/*************************************************************************
 *
 *  $RCSfile: winole.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <shlobj.h>
#include "winole.hxx"

// -------------------------------------------------------------------------------------------

#define FORMAT_TXT		"Text"
#define FORMAT_MTF		"GDIMetaFile"
#define FORMAT_DIB		"Bitmap"
#define FORMAT_DDB		"Bitmap"
#define FORMAT_FILES	"FileList"

// -------------------------------------------------------------------------------------------

ISVClipDataObject::ISVClipDataObject(XDataTransfer& Data, XClipboardListener& Listener)
	: ISVDataObject(Data)
{
	xListener = Listener;
}																		  

ISVClipDataObject::~ISVClipDataObject()
{
	xListener.changed();
}

// -------------------------------------------------------------------------------------------

ISVDataObject::ISVDataObject(XDataTransfer& Data)
	: nRef( 0 )
{
	xData = Data;
}																		  

STDMETHODIMP_(ULONG) ISVDataObject::AddRef()
{
	return ++nRef;
}

STDMETHODIMP_(ULONG) ISVDataObject::Release()								  
{																		  
    if( 0 == --nRef )
    {
        delete this;
        return 0;
    }
    else
        return nRef;
}

STDMETHODIMP ISVDataObject::QueryInterface( REFIID nId, LPVOID FAR * ppObj )  
{																		  
	if( nId == IID_IDataObject || nId == IID_IUnknown )
	{
		nRef++;
		*ppObj = this;
		return NOERROR;
	}
	return ResultFromScode( E_NOINTERFACE );
}																		  

/*************************************************************************
|*	  ISVDataObject::GetData()
|*
|*	  Beschreibung
*************************************************************************/
STDMETHODIMP ISVDataObject::GetData
(
	LPFORMATETC pFormat,
	LPSTGMEDIUM pMed 
)
{
	if( pFormat->lindex != -1 )
		return ResultFromScode( DV_E_LINDEX );
	if( pFormat->dwAspect != DVASPECT_CONTENT )
		return ResultFromScode( DV_E_DVASPECT );
	if( !(pFormat->tymed & (TYMED_HGLOBAL | TYMED_MFPICT | TYMED_GDI)) )
		return ResultFromScode( DV_E_TYMED );

	TCHAR szClipName[256];

	switch( pFormat->cfFormat ) 
	{
		case CF_TEXT:
			strcpy( szClipName, FORMAT_TXT );
		break;

		case CF_BITMAP:
		case CF_DIB:
			strcpy( szClipName, FORMAT_DIB );
		break;
		
		case CF_METAFILEPICT:
			strcpy( szClipName, FORMAT_MTF );
		break;
		
		case CF_HDROP:
			strcpy( szClipName, FORMAT_FILES );
		break;

		default: 
			if (GetClipboardFormatName(pFormat->cfFormat, szClipName, sizeof(szClipName) / sizeof(TCHAR)) <= 0)
				return ResultFromScode( S_FALSE );
		break;
	}

	SequenceOfByte Bytes = xData.getData( szClipName );

	if (NULL == Bytes.getPointer() || 0 == Bytes.getLength())
		return S_FALSE;

	pMed->tymed = TYMED_NULL;
	pMed->pUnkForRelease = NULL;

	if( pFormat->cfFormat == CF_METAFILEPICT )
	{
		HGLOBAL			hPict = GlobalAlloc( GHND, sizeof( METAFILEPICT ) );
		METAFILEPICT*	pPict = (METAFILEPICT*) GlobalLock( hPict );
		const ULONG		nNewSize = Bytes.getLength();
		BYTE*			pData = (BYTE*) Bytes.getPointer();
		HMETAFILE		hMtf = SetMetaFileBitsEx( nNewSize - 22, pData + 22 );

		if( hMtf )
		{
			pPict->mm = 8;
			pPict->xExt = 0;
			pPict->yExt = 0;
			pPict->hMF = hMtf;
			
			GlobalUnlock( hPict );
			pMed->hGlobal = hPict;
			pMed->tymed = TYMED_HGLOBAL;
			return NOERROR;
		}
		else
			return ResultFromScode( E_OUTOFMEMORY );
	}
	else
	{
		HGLOBAL hMem = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, Bytes.getLength() );
		if( hMem )
		{
			void *pMem = (LPSTR)GlobalLock( hMem );
			memcpy(pMem, Bytes.getPointer(), Bytes.getLength() );
			GlobalUnlock( hMem );
			pMed->hGlobal = hMem;
			pMed->tymed = TYMED_HGLOBAL;
			return NOERROR;
		}
		else
			return ResultFromScode( E_OUTOFMEMORY );
	}
}

/*************************************************************************
|*	  ISVDataObject::GetDataHere()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::GetDataHere
(
	LPFORMATETC pFormat,
	LPSTGMEDIUM pMedium 
)
{
	return ResultFromScode( E_NOTIMPL );
}

/*************************************************************************
|*	  ISVDataObject::QueryGetData()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::QueryGetData( LPFORMATETC pFormat )
{
	if( pFormat->lindex != -1 )
		return ResultFromScode( DV_E_LINDEX );
	if( !(pFormat->dwAspect & DVASPECT_CONTENT) )
		return ResultFromScode( DV_E_DVASPECT );
	if( !(pFormat->tymed & (TYMED_HGLOBAL | TYMED_MFPICT | TYMED_GDI)) )
		return ResultFromScode( DV_E_TYMED );

	TCHAR szClipName[256];

	switch( pFormat->cfFormat ) 
	{
		case CF_TEXT:
			strcpy( szClipName, FORMAT_TXT );
		break;
		
		case CF_BITMAP:
		case CF_DIB:
			strcpy( szClipName, FORMAT_DIB );
		break;
		
		case CF_METAFILEPICT:
			strcpy( szClipName, FORMAT_MTF );
		break;

		case CF_HDROP:
			strcpy( szClipName, FORMAT_FILES );
		break;

		default: 
			if (GetClipboardFormatName(pFormat->cfFormat, szClipName, sizeof(szClipName) / sizeof(TCHAR)) <= 0)
				return ResultFromScode( S_FALSE );
		break;
	}

	SequenceOfString	Strings = xData.getTypeList();
	int					nFormats = Strings.getLength();

	while (nFormats > 0) {
		if (stricmp(Strings[--nFormats], szClipName) == 0)
			return S_OK;
	}

	return S_FALSE;
}

/*************************************************************************
|*	  ISVDataObject::GetDataCanonicalFormatEtc()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::GetCanonicalFormatEtc( LPFORMATETC pInFormat,LPFORMATETC pOutFormat )
{
	if( NULL == pOutFormat )
		return ResultFromScode( E_INVALIDARG );
	pOutFormat->ptd = NULL;
	return ResultFromScode( DATA_S_SAMEFORMATETC );
}

/*************************************************************************
|*	  ISVDataObject::SetData()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::SetData( LPFORMATETC pFormat,
									 LPSTGMEDIUM pMedium,
									 BOOL fRelease )
{
	return ResultFromScode( DATA_E_FORMATETC );
}

/*************************************************************************
|*	  ISVDataObject::EnumFormatEtc()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::EnumFormatEtc( DWORD dwDirection, LPENUMFORMATETC * ppForm )
{
	*ppForm = NULL;
	if( dwDirection == DATADIR_GET )
	{
		*ppForm = new ISVEnumFormatEtc(xData);
		(*ppForm)->AddRef();
		return NOERROR;
	}
	return ResultFromScode( E_FAIL );
}

/*************************************************************************
|*	  ISVDataObject::DAdvise()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::DAdvise( LPFORMATETC pFormat,
									 DWORD advf,
									 LPADVISESINK pAdvSink,
									 DWORD * pConnect )
{
	return ResultFromScode( OLE_E_ADVISENOTSUPPORTED );
}

/*************************************************************************
|*	  ISVDataObject::DUnadvise()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::DUnadvise( DWORD dwConn )
{
	return ResultFromScode( OLE_E_ADVISENOTSUPPORTED );
}

/*************************************************************************
|*	  ISVDataObject::EnumDAdvise()
|*
|*	  Beschreibung		Interface IDataObject
*************************************************************************/
STDMETHODIMP ISVDataObject::EnumDAdvise( LPENUMSTATDATA FAR * /*pAdvise*/ )
{
	return ResultFromScode( OLE_E_ADVISENOTSUPPORTED );
}








ISVEnumFormatEtc::ISVEnumFormatEtc( XDataTransfer &rxData )
	: nRef( 0 )
	, nCur( 0 )
	, nCount( 0 )
	, pFormats( NULL )
{
	SequenceOfString	Strings = rxData.getTypeList();

	nCount = Strings.getLength();
    pFormats = new FORMATETC[ nCount ];
	const char *pszTypeName = "";

    for( ULONG i = 0; i < nCount; i++ )
    {
		pszTypeName = Strings[(int)i];

		if( stricmp(pszTypeName, FORMAT_TXT ) == 0 ) {
			pFormats[i].cfFormat = CF_TEXT;
			pFormats[i].tymed    = TYMED_HGLOBAL;
		} else if( stricmp(pszTypeName, FORMAT_FILES ) == 0 ) {
			pFormats[i].cfFormat = CF_HDROP;
			pFormats[i].tymed    = TYMED_HGLOBAL;
		} else if( stricmp(pszTypeName, FORMAT_DIB ) == 0 ) {
			pFormats[i].cfFormat = CF_DIB;
			pFormats[i].tymed    = TYMED_HGLOBAL;
		} else if( stricmp(pszTypeName, FORMAT_MTF ) == 0 ) {
			pFormats[i].cfFormat = CF_METAFILEPICT;
			pFormats[i].tymed    = TYMED_MFPICT;
		} else {
			pFormats[i].cfFormat = RegisterClipboardFormat(pszTypeName);
			pFormats[i].tymed    = TYMED_HGLOBAL;
		}

		pFormats[i].ptd      = NULL;
		pFormats[i].lindex   = -1;
		pFormats[i].dwAspect = DVASPECT_CONTENT;
    }
}

ISVEnumFormatEtc::ISVEnumFormatEtc( const ISVEnumFormatEtc & rObj )
{
    nRef     = 0;
    nCur     = rObj.nCur;
    nCount   = rObj.nCount;
    pFormats = new FORMATETC[ nCount ];
    memcpy( pFormats, rObj.pFormats, (USHORT)(nCount * sizeof( FORMATETC )) );
    for( ULONG i = 0; i < nCount; i++ )
    {
        if( pFormats[ i ].ptd )
        {
            // Devicedaten kopieren
            DWORD nSize = pFormats[ i ].ptd->tdSize;
            DVTARGETDEVICE * pDev = (DVTARGETDEVICE *)new BYTE[ nSize ];
            memcpy( pDev, pFormats[ i ].ptd, (USHORT)nSize );
            pFormats[ i ].ptd = pDev;
        }
    }
}


ISVEnumFormatEtc::~ISVEnumFormatEtc()
{
    delete [] pFormats;
}

STDMETHODIMP ISVEnumFormatEtc::QueryInterface( REFIID riid, LPVOID FAR *ppv )
{
    *ppv = NULL;

    if( IsEqualIID( riid, IID_IUnknown )
      || IsEqualIID( riid, IID_IEnumFORMATETC ) )
    {
        *ppv = this;
        ++nRef;
        return NOERROR;
    }
    return ResultFromScode(E_NOINTERFACE);
}


STDMETHODIMP_(ULONG) ISVEnumFormatEtc::AddRef()
{
    ++nRef;
    return nRef;
}

STDMETHODIMP_(ULONG) ISVEnumFormatEtc::Release()
{
    if( 0 == --nRef )
    {
        delete this;
        return 0;
    }
    else
        return nRef;
}

STDMETHODIMP ISVEnumFormatEtc::Next( ULONG nMaxEle, LPFORMATETC pFE,
                                     ULONG * pnEle )
{
    ULONG               nRet = 0L;

    while( nCur < nCount && nMaxEle > 0 )
    {
        *pFE++ = pFormats[ nCur++ ];
        nRet++;
        nMaxEle--;
    }

    if( pnEle )
        *pnEle = nRet;

    if( nMaxEle )
        return ResultFromScode( S_FALSE );
    return NOERROR;
}

STDMETHODIMP ISVEnumFormatEtc::Skip( ULONG cSkip )
{
    if( nCur + cSkip >= nCount )
    {
        nCur = nCount;
        return ResultFromScode( S_FALSE );
    }

    nCur += cSkip;
    return NOERROR;
}

STDMETHODIMP ISVEnumFormatEtc::Reset(void)
{
    nCur = 0;
    return NOERROR;
}

STDMETHODIMP ISVEnumFormatEtc::Clone( LPENUMFORMATETC FAR *ppEnum )
{
    //Create the clone
    *ppEnum = new ISVEnumFormatEtc( *this );

    (*ppEnum)->AddRef();
    return NOERROR;
}


