/*************************************************************************
 *
 *  $RCSfile: rmoutdev.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef REMOTE_APPSERVER

#define _SV_RMOUTDEV_CXX

#include <tools/ref.hxx>
#include <rmoutdev.hxx>
#include <svdata.hxx>
#include <virdev.hxx>
#include <tools/stream.hxx>
#include <tools/debug.hxx>
#include <outfont.hxx>
#include <gradient.hxx>
#include <hatch.hxx>
#include "rmvirdev.hxx"
#include <atom.hxx>

#include <com/sun/star/portal/client/FontAtomType.hpp>

using namespace ::com::sun::star::uno;
using namespace vcl;

/*
 *	two atomproviders for the shared font atoms of virtual and window
 *	devices. printers need their own atom lists
 */

// =======================================================================

void ImplServerGraphics::ImplCopyFontMetric( const NMSP_CLIENT::IDLFontMetricData& rFrom, ImplFontMetricData& rTo )
{
	rTo.mnWidth = rFrom.mnWidth;
	rTo.mnAscent = rFrom.mnAscent;
	rTo.mnDescent = rFrom.mnDescent;
	rTo.mnLeading = rFrom.mnLeading;
	rTo.mnSlant = rFrom.mnSlant;
	rTo.mnFirstChar = rFrom.mnFirstChar;
	rTo.mnLastChar = rFrom.mnLastChar;

	rTo.maName = mpAtoms->getString( NMSP_CLIENT::FontAtomType::NAME, rFrom.mnName );
	rTo.maStyleName = mpAtoms->getString( NMSP_CLIENT::FontAtomType::STYLE, rFrom.mnStyleName );
	rTo.mnOrientation = rFrom.mnOrientation;
	rTo.meFamily = (FontFamily)rFrom.meFamily;
	rTo.meCharSet = (CharSet)rFrom.meCharSet;
	rTo.meWeight = (FontWeight)rFrom.meWeight;
	rTo.meItalic = (FontItalic)rFrom.meItalic;
	rTo.mePitch = (FontPitch)rFrom.mePitch;
	rTo.meType = (FontType)rFrom.meType;
	rTo.mbDevice = rFrom.mbDevice;
}

// -----------------------------------------------------------------------

void ImplServerGraphics::ImplCopyFontData( const NMSP_CLIENT::IDLFontData& rFrom, ImplFontData& rTo )
{
	rTo.mpNext = NULL;
	rTo.mpSysData = NULL;

	rTo.maName = mpAtoms->getString( NMSP_CLIENT::FontAtomType::NAME, rFrom.mnName );
	rTo.maStyleName = mpAtoms->getString( NMSP_CLIENT::FontAtomType::STYLE, rFrom.mnStyleName );
	rTo.mnWidth = rFrom.mnWidth;
	rTo.mnHeight = rFrom.mnHeight;
	rTo.meFamily = (FontFamily)rFrom.meFamily;
	rTo.meCharSet = (CharSet)rFrom.meCharSet;
	rTo.mePitch = (FontPitch)rFrom.mePitch;
	rTo.meWidthType = (FontWidth)rFrom.meWidthType;
	rTo.meWeight = (FontWeight)rFrom.meWeight;
	rTo.meItalic = (FontItalic)rFrom.meItalic;
	rTo.meType = (FontType)rFrom.meType;
	rTo.mbOrientation = rFrom.mnBools & 1 ? TRUE : FALSE;
	rTo.mbDevice = rFrom.mnBools & 2 ? TRUE : FALSE;
}

// =======================================================================

BOOL RemoteFont::operator ==( const RemoteFont& rFont ) const
{
	if ( (mnWidth			== rFont.mnWidth)		&&
		 (mnHeight			== rFont.mnHeight)		&&
		 (meWeight			== rFont.meWeight)		&&
		 (meItalic			== rFont.meItalic)		&&
		 (meFamily			== rFont.meFamily)		&&
		 (maName			== rFont.maName)		&&
		 (maStyleName		== rFont.maStyleName)	&&
		 (meCharSet 		== rFont.meCharSet) 	&&
		 (meWidthType		== rFont.meWidthType)	&&
		 (mePitch			== rFont.mePitch)		&&
		 (mnOrientation 	== rFont.mnOrientation) )
		return TRUE;
	else
		return FALSE;
}

// =======================================================================

ImplServerGraphics::ImplServerGraphics( AtomClient* pAtoms ) :
	maLineColor( COL_BLACK ),
	maFillColor( COL_WHITE ),
	maTextColor( COL_BLACK ),
	maClipRegion(),
	mpAtoms( pAtoms ),
	mpLastOutDev( NULL )
{
	maFont.mnWidth			= 0;
	maFont.mnHeight 		= 0;
	maFont.meFamily 		= FAMILY_DONTKNOW;
	maFont.meCharSet		= RTL_TEXTENCODING_DONTKNOW;
	maFont.meWidthType		= WIDTH_DONTKNOW;
	maFont.meWeight 		= WEIGHT_DONTKNOW;
	maFont.meItalic 		= ITALIC_NONE;
	maFont.mePitch			= PITCH_DONTKNOW;
	maFont.mnOrientation	= 0;

	mnDPIX					= 96;
	mnDPIY					= 96;
	meRasterOp				= ROP_OVERPAINT;
	mnBitCount				= 24;
	mbClipRegion			= FALSE;
	mbSendClipRegion		= FALSE;
	mbSendFont				= FALSE;
	mbGetResolution 		= TRUE;

	if( mpAtoms == NULL )
	{
		ImplSVData* pSVData = ImplGetSVData();
		mpAtoms = pSVData->mpAtoms;
	}
}

// -----------------------------------------------------------------------

ImplServerGraphics::~ImplServerGraphics()
{
}

// -----------------------------------------------------------------------

BOOL ImplServerGraphics::Init()
{
	if ( mbSendClipRegion )
	{
		if ( !mbClipRegion )
			mxRemoteDevice->ResetClipRegion();
		else
		{
			SvMemoryStream aMem;
			aMem.SetCompressMode( COMPRESSMODE_FULL );
			aMem << maClipRegion;

			const NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
			mxRemoteDevice->SetClipRegion( aSeq );
		}
		mbSendClipRegion = FALSE;
	}

	return TRUE;
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetClipRegion()
{
	if ( mbClipRegion )
	{
		mbClipRegion		= FALSE;
		mbSendClipRegion	= TRUE;
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetClipRegion( const Region& rRegion )
{
	mbClipRegion		= TRUE;
	mbSendClipRegion	= TRUE;
	maClipRegion		= rRegion;
}

// -----------------------------------------------------------------------

void ImplServerGraphics::GetResolution( long& rDPIX, long& rDPIY )
{
	if ( mbGetResolution )
	{
		mxRemoteDevice->GetResolution( mnDPIX, mnDPIY, mnBitCount );
		mbGetResolution = FALSE;

	}
	rDPIX = mnDPIX;
	rDPIY = mnDPIY;
}

// -----------------------------------------------------------------------

USHORT ImplServerGraphics::GetBitCount()
{
	if ( mbGetResolution )
	{
		mxRemoteDevice->GetResolution( mnDPIX, mnDPIY, mnBitCount );
		mbGetResolution = FALSE;

	}
	return mnBitCount;
}

// -----------------------------------------------------------------------

Color ImplServerGraphics::GetPixel( const Point& rPos )
{
	Color			aColor;
	unsigned long	nColor;

	Init();
	mxRemoteDevice->GetPixel( rPos.X(), rPos.Y(), nColor );
	aColor.SetColor( nColor );

	return aColor;
}

// -----------------------------------------------------------------------

Color* ImplServerGraphics::GetPixel( const Polygon& rPoints )
{
	NMSP_CLIENT::ULongSequence	aColSeq;
	SvMemoryStream				aMem;
	Color*						pRet = NULL;

	aMem.SetCompressMode( COMPRESSMODE_FULL );
	aMem << rPoints;

	mxRemoteDevice->GetPixelArray( NMSP_CLIENT::ByteSequence( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) ), aColSeq );
	const long nLen = aColSeq.getLength();

	if( nLen && ( rPoints.GetSize() == nLen ) )
	{
		const ULONG* pBuffer = aColSeq.getConstArray();

		pRet = new Color[ nLen ];

		for( long i = 0; i < nLen; i++ )
			pRet[ i ].SetColor( pBuffer[ i ] );
	}

	return pRet;
}

// -----------------------------------------------------------------------

void ImplServerGraphics::GetCharWidth( USHORT nChar1, USHORT nChar2, long* pWidthAry )
{
	Sequence< sal_Int32 >		aSeq( mxRemoteDevice->GetCharWidth( nChar1, nChar2 ) );
	for( int i = 0; i <= nChar2-nChar1; i++ )
		pWidthAry[i] = aSeq.getConstArray()[i];
}

// -----------------------------------------------------------------------

void ImplServerGraphics::GetFontMetric( ImplFontMetricData& rMetric, long& rFactor, unsigned short firstChar, unsigned short lastChar, long* pWidths, BOOL bGetKernPairs, ImplKernPairData** ppKernPairs, long& rKernPairs	)
{
	// FontMetric cachen ?!
	NMSP_CLIENT::IDLFontMetricData aMetric;

	mxRemoteDevice->GetFontMetric( aMetric, firstChar, lastChar, bGetKernPairs );
	rFactor 	= aMetric.mnFactor;

	if( firstChar != lastChar )
	{
		// relies on sizeof( sal_Int32 ) == sizeof( long )
		memcpy( pWidths, aMetric.maCharWidths.getConstArray(), aMetric.maCharWidths.getLength() * sizeof( sal_Int32 ) );
	}
	if( bGetKernPairs )
	{
		rKernPairs = aMetric.maKerningPairs.getLength();
		if( rKernPairs )
		{
			*ppKernPairs = new ImplKernPairData[ rKernPairs ];
			const NMSP_CLIENT::KernPair* pIDLPairs = aMetric.maKerningPairs.getConstArray();
			for( ULONG n = 0; n < rKernPairs; n++ )
			{
				(*ppKernPairs)[ n ].mnChar1 = pIDLPairs[ n ].Char1;
				(*ppKernPairs)[ n ].mnChar2 = pIDLPairs[ n ].Char2;
				(*ppKernPairs)[ n ].mnKern	= pIDLPairs[ n ].Kerning;
			}
		}
		else
			ppKernPairs = NULL;
	}
	ImplCopyFontMetric( aMetric, rMetric );
}

// -----------------------------------------------------------------------

ULONG ImplServerGraphics::GetKernPairs( ImplKernPairData** ppKernPairs )
{
	Sequence< NMSP_CLIENT::KernPair > aKernPairs(
		mxRemoteDevice->GetKernPairs()
		);

	ULONG nPairs = aKernPairs.getLength();
	if ( nPairs )
	{
		*ppKernPairs = new ImplKernPairData[ nPairs ];
		const NMSP_CLIENT::KernPair* pIDLPairs = aKernPairs.getConstArray();
		for ( ULONG n = 0; n < nPairs; n++ )
		{
			(*ppKernPairs)[ n ].mnChar1 = pIDLPairs[ n ].Char1;
			(*ppKernPairs)[ n ].mnChar2 = pIDLPairs[ n ].Char2;
			(*ppKernPairs)[ n ].mnKern	= pIDLPairs[ n ].Kerning;
		}
	}
	else
		*ppKernPairs = NULL;

	return nPairs;
}

// -----------------------------------------------------------------------

BOOL ImplServerGraphics::GetGlyphBoundRect( xub_Unicode cChar, Rectangle& rRect, BOOL bOptimize )
{
	long nLeft, nTop, nWidth, nHeight;
	BOOL bRet = mxRemoteDevice->GetGlyphBoundRect( cChar, nLeft, nTop, nWidth, nHeight, bOptimize );
	if ( bRet )
		rRect = Rectangle( Point( nLeft, nTop ), Size( nWidth, nHeight ) );

	return bRet;
}

// -----------------------------------------------------------------------

BOOL ImplServerGraphics::GetGlyphOutline( xub_Unicode cChar, PolyPolygon& rPolyPoly, BOOL bOptimize )
{
	NMSP_CLIENT::ByteSequence	aSeq;
	BOOL						bRet = mxRemoteDevice->GetGlyphOutline( cChar, aSeq, bOptimize );

	if( bRet )
	{
		SvMemoryStream aMem( (char*) aSeq.getConstArray(), aSeq.getLength(), STREAM_READ );
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem >> rPolyPoly;
	}

	return bRet;
}

// -----------------------------------------------------------------------

void ImplServerGraphics::GetDevFontList( ImplDevFontList* pList )
{
	NMSP_CLIENT::IDLFontDataSequence aSeqFontData = mxRemoteDevice->GetDevFontList();
	Sequence< sal_Int32 > aSeq( 2 );
	aSeq.getArray()[0] = NMSP_CLIENT::FontAtomType::NAME;
	aSeq.getArray()[1] = NMSP_CLIENT::FontAtomType::STYLE;
	mpAtoms->updateAtomClasses( aSeq );

	for( ULONG n = 0, nFonts = aSeqFontData.getLength(); n < nFonts; n++ )
	{
		ImplFontData* pData = new ImplFontData;
		ImplCopyFontData( aSeqFontData[(int)n], *pData );
		pList->Add( pData );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetFont( const RemoteFont& rFont )
{
	if ( maFont != rFont )
	{
		maFont = rFont;

		NMSP_CLIENT::IDLFont aIDLFont;

		aIDLFont.mnName 		= mpAtoms->getAtom( NMSP_CLIENT::FontAtomType::NAME, rFont.maName, sal_True );
		aIDLFont.mnStyleName	= mpAtoms->getAtom( NMSP_CLIENT::FontAtomType::STYLE, rFont.maStyleName, sal_True );
		aIDLFont.mnWidth		= rFont.mnWidth;
		aIDLFont.mnHeight		= rFont.mnHeight;
		aIDLFont.meFamily		= (UINT16)rFont.meFamily;
		aIDLFont.meCharSet		= (UINT16)rFont.meCharSet;
		aIDLFont.meWidthType	= (UINT16)rFont.meWidthType;
		aIDLFont.meWeight		= (UINT16)rFont.meWeight;
		aIDLFont.meItalic		= (UINT16)rFont.meItalic;
		aIDLFont.mePitch		= (UINT16)rFont.mePitch;
		aIDLFont.mnOrientation	= rFont.mnOrientation;

		mxRemoteDevice->SetFont( aIDLFont );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetTextColor( const Color& rColor )
{
	if( rColor != maTextColor )
	{
		maTextColor = rColor;
		mxRemoteDevice->SetTextColor( rColor.GetColor() );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetLineColor( const Color& rColor )
{
	if( rColor != maLineColor )
	{
		maLineColor = rColor;
		mxRemoteDevice->SetLineColor( rColor.GetColor() );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetFillColor( const Color& rColor )
{
	if( rColor != maFillColor )
	{
		maFillColor = rColor;
		mxRemoteDevice->SetFillColor( rColor.GetColor() );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::SetRasterOp( RasterOp eROP )
{
	if ( meRasterOp != eROP )
	{
		meRasterOp = eROP;
		mxRemoteDevice->SetRasterOp( (USHORT)eROP );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::CopyArea( long nDestX, long nDestY,
								   long nSrcX, long nSrcY,
								   long nSrcWidth, long nSrcHeight,
								   USHORT nFlags )
{
	if ( Init() )
	{
		mxRemoteDevice->CopyArea( nSrcX, nSrcY, nSrcWidth, nSrcHeight, nDestX, nDestY, nFlags );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::CopyBits( const RemoteTwoRect* pPosAry, ImplServerGraphics* pSrcGraphics )
{
	if ( Init() )
	{
		REF( NMSP_CLIENT::XRmOutputDevice ) xOutDev;

		if( pSrcGraphics )
			xOutDev = pSrcGraphics->GetInterface();

		mxRemoteDevice->CopyBits( pPosAry->mnSrcX, pPosAry->mnSrcY, pPosAry->mnSrcWidth, pPosAry->mnSrcHeight,
								  pPosAry->mnDestX, pPosAry->mnDestY, pPosAry->mnDestWidth, pPosAry->mnDestHeight, xOutDev );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::Invert( const Rectangle& rRect, USHORT nFlags )
{
	if ( Init() )
		mxRemoteDevice->Invert( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), nFlags );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::Invert( const Polygon& rPoly, USHORT nFlags )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rPoly;

		NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->InvertPolygon( aSeq, nFlags );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::InvertTracking( const Rectangle& rRect, USHORT nFlags )
{
	if ( Init() )
		mxRemoteDevice->InvertTracking( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), nFlags );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::InvertTracking( const Polygon& rPoly, USHORT nFlags )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rPoly;

		NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->InvertTrackingPolygon( aSeq, nFlags );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPixel( const Point& rPos )
{
	if ( Init() )
		mxRemoteDevice->DrawPixel( rPos.X(), rPos.Y() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPixel( const Point& rPos, const Color& rColor )
{
	if ( Init() )
		mxRemoteDevice->DrawColoredPixel( rPos.X(), rPos.Y(), rColor.GetColor() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPixel( const Polygon& rPoints, const Color* pColors )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rPoints;

		const USHORT				nCount = rPoints.GetSize();
		NMSP_CLIENT::ByteSequence	aPointSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		NMSP_CLIENT::ULongSequence	aColorSeq( nCount );

		for( USHORT i = 0; i < nCount; i++ )
			aColorSeq[ (int) i ] = pColors[ i ].GetColor();

		mxRemoteDevice->DrawPixelArray( aPointSeq, aColorSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawLine( const Point& rStartPos, const Point& rEndPos )
{
	if ( Init() )
		mxRemoteDevice->DrawLine( rStartPos.X(), rStartPos.Y(), rEndPos.X(), rEndPos.Y() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawRect( const Rectangle& rRect )
{
	if ( Init() )
		mxRemoteDevice->DrawRect( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPolyLine( const Polygon& rPoly )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rPoly;

		NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->DrawPolyLine( aSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPolygon( const Polygon& rPoly )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rPoly;
		NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->DrawPolygon( aSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPolyPolygon( const PolyPolygon& rPoly )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rPoly;

		NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->DrawPolyPolygon( aSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawRect( const Rectangle& rRect, ULONG nHorzRound, ULONG nVertRound )
{
	if ( Init() )
		mxRemoteDevice->DrawRoundedRect( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), nHorzRound, nVertRound );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawEllipse( const Rectangle& rRect )
{
	if ( Init() )
		mxRemoteDevice->DrawEllipse( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawArc( const Rectangle& rRect, const Point& rStartPt, const Point& rEndPt )
{
	if ( Init() )
		mxRemoteDevice->DrawArc( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), rStartPt.X(), rStartPt.Y(), rEndPt.X(), rEndPt.Y() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawPie( const Rectangle& rRect, const Point& rStartPt, const Point& rEndPt )
{
	if ( Init() )
		mxRemoteDevice->DrawPie( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), rStartPt.X(), rStartPt.Y(), rEndPt.X(), rEndPt.Y() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawChord( const Rectangle& rRect, const Point& rStartPt, const Point& rEndPt )
{
	if ( Init() )
		mxRemoteDevice->DrawChord( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), rStartPt.X(), rStartPt.Y(), rEndPt.X(), rEndPt.Y() );
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawGradient( const Rectangle& rRect, const Gradient& rGradient )
{
	if ( Init() )
	{
		SvMemoryStream aMem;
		aMem.SetCompressMode( COMPRESSMODE_FULL );
		aMem << rGradient;

		NMSP_CLIENT::ByteSequence aSeq( (sal_Int8*) aMem.GetData(), aMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->DrawGradient( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(), aSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawGradient( const PolyPolygon& rPolyPoly, const Gradient& rGradient )
{
	if ( Init() )
	{
		SvMemoryStream aGradMem;
		SvMemoryStream aPolyPolyMem;

		aPolyPolyMem.SetCompressMode( COMPRESSMODE_FULL );
		aPolyPolyMem << rPolyPoly;

		aGradMem.SetCompressMode( COMPRESSMODE_FULL );
		aGradMem << rGradient;

		NMSP_CLIENT::ByteSequence aPolyPolySeq( (sal_Int8*) aPolyPolyMem.GetData(), aPolyPolyMem.Seek( STREAM_SEEK_TO_END ) );
		NMSP_CLIENT::ByteSequence aGradSeq( (sal_Int8*) aGradMem.GetData(), aGradMem.Seek( STREAM_SEEK_TO_END ) );

		mxRemoteDevice->DrawPolyPolyGradient( aPolyPolySeq, aGradSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawHatch( const PolyPolygon& rPolyPoly, const Hatch& rHatch )
{
	if ( Init() )
	{
		SvMemoryStream aHatchMem;
		SvMemoryStream aPolyPolyMem;

		aPolyPolyMem.SetCompressMode( COMPRESSMODE_FULL );
		aPolyPolyMem << rPolyPoly;

		aHatchMem.SetCompressMode( COMPRESSMODE_FULL );
		aHatchMem << rHatch;

		NMSP_CLIENT::ByteSequence aPolyPolySeq( (sal_Int8*) aPolyPolyMem.GetData(), aPolyPolyMem.Seek( STREAM_SEEK_TO_END ) );
		NMSP_CLIENT::ByteSequence aHatchSeq( (sal_Int8*) aHatchMem.GetData(), aHatchMem.Seek( STREAM_SEEK_TO_END ) );

		mxRemoteDevice->DrawPolyPolyHatch( aPolyPolySeq, aHatchSeq );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawText( long nX, long nY, const xub_Unicode* pStr, USHORT nLen )
{
	if ( Init() )
	{
		const OUSTRING aStr( pStr, nLen );
		mxRemoteDevice->DrawText( nX, nY, aStr );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawTextArray( long nX, long nY, const xub_Unicode* pStr, USHORT nLen, const long* pDXAry )
{
	if ( Init() )
	{
		const OUSTRING aStr( pStr, nLen );
		mxRemoteDevice->DrawTextArray( nX, nY, aStr, NMSP_CLIENT::LongSequence( pDXAry, nLen ) );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawWaveLine( const Point& rStartPos, const Point& rEndPos, USHORT nStyle )
{
	if ( Init() )
	{
		mxRemoteDevice->DrawWaveLine( rStartPos.X(), rStartPos.Y(), rEndPos.X(), rEndPos.Y(), nStyle );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawGrid(	long nMinX, long nMaxX, const SEQ( long )& rHorzValues,
									long nMinY, long nMaxY, const SEQ( long )& rVertValues,
									USHORT nFlags )
{
	if ( Init() )
	{
		mxRemoteDevice->DrawGrid( nMinX, nMaxX, rHorzValues, nMinY, nMaxY, rVertValues, nFlags );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::DrawTransparent( const PolyPolygon& rPolyPoly, USHORT nTransparencyPercent )
{
	if ( Init() )
	{
		SvMemoryStream aPolyPolyMem;

		aPolyPolyMem.SetCompressMode( COMPRESSMODE_FULL );
		aPolyPolyMem << rPolyPoly;

		NMSP_CLIENT::ByteSequence aPolyPolySeq( (sal_Int8*) aPolyPolyMem.GetData(), aPolyPolyMem.Seek( STREAM_SEEK_TO_END ) );
		mxRemoteDevice->DrawPolyPolyTransparent( aPolyPolySeq, nTransparencyPercent );
	}
}

// -----------------------------------------------------------------------

void ImplServerGraphics::Draw2ColorFrame( const Rectangle& rRect,
										  const Color& rLeftTopColor,
										  const Color& rRightBottomColor )
{
	if ( Init() )
	{
/*
		mxRemoteDevice->SetFillColor( rLeftTopColor.GetColor() );
		mxRemoteDevice->DrawRect( rRect.Left(), rRect.Top(), 1, rRect.Bottom()-rRect.Top() );
		mxRemoteDevice->DrawRect( rRect.Left(), rRect.Top(), rRect.Right()-rRect.Left(), 1 );
		mxRemoteDevice->SetFillColor( rRightBottomColor.GetColor() );
		mxRemoteDevice->DrawRect( rRect.Left(), rRect.Bottom(), rRect.Right()-rRect.Left()+1, 1 );
		mxRemoteDevice->DrawRect( rRect.Right(), rRect.Top(), 1, rRect.Bottom()-rRect.Top()+1 );
*/

		mxRemoteDevice->Draw2ColorFrame( rRect.Left(), rRect.Top(), rRect.GetWidth(), rRect.GetHeight(),
										 rLeftTopColor.GetColor(),
										 rRightBottomColor.GetColor() );

		maFillColor = rRightBottomColor;
	}
}

// =======================================================================

ImplServerGraphics* OutputDevice::ImplGetServerGraphics( BOOL bGlobal )
{
	if (mpGraphics && (mpGraphics->GetOutputDevice() != this))
	{
		mpGraphics->SetOutputDevice( this );

		if ( !mbInitLineColor && (maLineColor != mpGraphics->maLineColor) )
			mbInitLineColor = TRUE;
		if ( !mbInitFillColor && (maFillColor != mpGraphics->maFillColor) )
			mbInitFillColor = TRUE;
		if ( !mbInitTextColor && (GetTextColor() != mpGraphics->maTextColor) )
			mbInitTextColor = TRUE;
		// Font wird bei SetFont verglichen
		mbInitFont = TRUE;
		if ( !bGlobal )
		{
			mbInitClipRegion = TRUE;
			ImplInitClipRegion();
		}
	}
	else
	{
		if ( !bGlobal )
		{
			if ( mbInitClipRegion )
				ImplInitClipRegion();
		}
		else
			mbInitClipRegion = TRUE;
	}

	if (mpGraphics)
	{
		if ( bGlobal )
		{
			mpGraphics->SetClipRegion();
			mpGraphics->SetRasterOp( ROP_OVERPAINT );
			return mpGraphics;
		}
		else
		{
			if ( mbOutputClipped )
				return NULL;
			else
			{
				if ( meRasterOp != mpGraphics->meRasterOp )
					mpGraphics->SetRasterOp( meRasterOp );
				return mpGraphics;
			}
		}
	}
	else
		return NULL;
}

// -----------------------------------------------------------------------

void OutputDevice::ImplReleaseServerGraphics()
{
	if ( mpGraphics && (mpGraphics->GetOutputDevice() == this))
		mpGraphics->SetOutputDevice( NULL );
}

#endif
