/*************************************************************************
 *
 *  $RCSfile: rmprint.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <tools/debug.hxx>
#include <svdata.hxx>
#include <svapp.hxx>
#include <event.hxx>
#include <print.h>
#include <vos/thread.hxx>
#include "rmprint.hxx"
#include <atom.hxx>

using namespace ::com::sun::star::uno;

// -----------------------------------------------------------------------------

RemotePrinterList::RemotePrinterList( const REF( ::com::sun::star::lang::XMultiServiceFactory )& xServiceFactory )
{
	mxServiceFactory = xServiceFactory;
	REF( NMSP_UNO::XInterface ) xIf( static_cast< ::cppu::OWeakObject* >( new ORmPrinterObserver() ) );
	mxPrinterObserver = REF( NMSP_CLIENT::XRmPrinterObserver )( xIf, NMSP_UNO::UNO_QUERY );
}

// -----------------------------------------------------------------------

RemotePrinterList::~RemotePrinterList()
{
	for ( USHORT nPrnServer = 0; nPrnServer < maRemotePrintServers.Count(); nPrnServer++ )
	{
		RemotePrintServerInfo* pInf = maRemotePrintServers.GetObject( nPrnServer );
		
		if( pInf->xRmPrinterEnv.is() )
			pInf->xRmPrinterEnv = REF( NMSP_CLIENT::XRmPrinterEnvironment )();
		if( pInf->pAtoms )
			delete pInf->pAtoms;

		delete pInf;
	}
	maRemotePrintServers.Clear();

	for ( USHORT nPrinter = 0; nPrinter < maRemotePrinters.Count(); nPrinter++ )
	{
		RemotePrinterInfo* pInf = maRemotePrinters.GetObject( nPrinter );
		delete pInf;
	}
	
	maRemotePrinters.Clear();
	mxPrinterObserver = REF( NMSP_CLIENT::XRmPrinterObserver )();

}

// -----------------------------------------------------------------------

RemotePrintServerInfo* RemotePrinterList::ImplFindPrintServer( const UniString& rServerName ) const
{
	for ( ULONG n = maRemotePrintServers.Count(); n; )
	{
		RemotePrintServerInfo* pInf = maRemotePrintServers.GetObject( --n );

		if( pInf->aServerName == rServerName )
			return pInf;
	}
	return NULL;
}

// -----------------------------------------------------------------------

void RemotePrinterList::CreateNewPrinterConnection(
	const String& rServerName,
	Reference< ::com::sun::star::lang::XMultiServiceFactory >& xFactory,
	Reference< ::com::sun::star::connection::XConnection >& xConnection,
	Reference< ::com::sun::star::bridge::XBridgeFactory >& xBridgeFactory
	)
{
	ImplSVData* pSVData = ImplGetSVData();
	if( rServerName.CompareToAscii( RVP_CLIENT_SERVER_NAME ) == COMPARE_EQUAL )
	{
		xFactory = pSVData->mxClientFactory;
	}
	else
	{
		try {
			Reference< ::com::sun::star::connection::XConnector > xConnector( mxServiceFactory->createInstance( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.connection.Connector" ) ) ), UNO_QUERY );
			::rtl::OUString aCon( RTL_CONSTASCII_USTRINGPARAM( "socket,host=" ) );
			String aServerName( rServerName );
			if( aServerName.SearchAscii( "uno://" ) == 0 )
				aServerName.Erase( 0, 6 );
			if( aServerName.GetTokenCount( ':' ) < 2 )
			{
				aCon += aServerName;
				aCon += ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( ",port=8125" ) );
			}
			else
			{
				aCon += aServerName.GetToken( 0, ':' );
				aCon += ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( ",port=" ) );
				aCon += aServerName.GetToken( 1, ':' );
			}
			aCon += ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( ",ignoreFirstClose=1" ) );
			xConnection = xConnector->connect( aCon );
			xBridgeFactory = Reference< ::com::sun::star::bridge::XBridgeFactory >( mxServiceFactory->createInstance( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "com.sun.star.bridge.BridgeFactory" ) ) ), UNO_QUERY );
			if( xBridgeFactory.is() && xConnection.is() )
			{
				Reference< ::com::sun::star::bridge::XBridge > xBridge( xBridgeFactory->createBridge( ::rtl::OUString(), ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "iiop" ) ), xConnection, Reference< ::com::sun::star::bridge::XInstanceProvider >() ) );
				xFactory = Reference< ::com::sun::star::lang::XMultiServiceFactory >( xBridge->getInstance( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "PrintDaemonMultiServiceFactory.sprintd.daemons.stardiv.de" ) ) ), UNO_QUERY );
			}
		}
		catch( ... )
		{
			xFactory.clear();
			xConnection.clear();
			xBridgeFactory.clear();
		}
	}
}

// -----------------------------------------------------------------------

void RemotePrinterList::ImplInitPrintServer( RemotePrintServerInfo* pInf )
{
	if ( pInf && !pInf->xRmPrinterEnv.is() )
	{
		CreateNewPrinterConnection( pInf->aServerName, pInf->xFactory );
		if( pInf->xFactory.is() )
		{
			pInf->xRmPrinterEnv = REF( NMSP_CLIENT::XRmPrinterEnvironment )( pInf->xFactory->createInstance( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "OfficePrinterEnvironment.stardiv.de" ) ) ), NMSP_UNO::UNO_QUERY );
			if( pInf->xRmPrinterEnv.is() )
			{
				pInf->xRmPrinterEnv->SetObserver( mxPrinterObserver );
			}
		}
	}
}

// -----------------------------------------------------------------------

BOOL ImplAddRemotePrinter( const UniString& rPrinterName, const UniString& rServerName, BOOL bSetDefault, BOOL bIsLocal  )
{
	return ImplGetSVData()->mpRemotePrinterList->AddPrinter( rPrinterName, rServerName, bSetDefault, bIsLocal );
}

// -----------------------------------------------------------------------

BOOL RemotePrinterList::AddPrinter( const UniString& rPrinterName, const UniString& rServerName, BOOL bSetDefault, BOOL bIsLocal )
{
	// Keinen Drucker doppelt einfuegen und sicherstellen dass
	// nur ein Standard-Drucker...
	for ( ULONG n = maRemotePrinters.Count(); n; )
	{
		RemotePrinterInfo* pInf = maRemotePrinters.GetObject( --n );
		
		if( ( pInf->aPrinterName == rPrinterName ) && ( pInf->aServerName == rServerName ) )
		{
			return FALSE;
		}
		
		if( bSetDefault && pInf->bDefaultPrinter )
		{
			// Ein lokaler Default-Printer hat vorrang...
//			if ( pInf->bLocalPrinter && !bIsLocal )
//				bSetDefault = FALSE;
//			else
//				pInf->bDefaultPrinter = FALSE;

			// Anders: Der erste gewinnt, dann kann der User die Admin-Einstellung
			// nur geziehlt durch User@Printer@1 uebersteuern.
			bSetDefault = FALSE;
		}
	}

	// Server schon eingetragen?
	if( !ImplFindPrintServer( rServerName ) )
	{
		RemotePrintServerInfo* pServerInf = new RemotePrintServerInfo( rServerName );
		maRemotePrintServers.Insert( pServerInf, LIST_APPEND );
	}

	// Printer eintragen...
	if( !rPrinterName.Len() )
	{
		// Dann alle Drucker vom Server abfragen...
		REF( NMSP_CLIENT::XRmPrinterEnvironment ) xEnv( GetPrinterEnv( rServerName ) );
		
		if( xEnv.is() )
		{
			NMSP_CLIENT::StringSequence aNames; xEnv->GetPrinterNames( aNames );
			
			for( ULONG n = 0, nPrinters = aNames.getLength(); n < nPrinters; n++ )
			{
				const UniString& rName = aNames[ (int) n ];
				
				if( rName.Len() )
				{
					AddPrinter( rName, rServerName, bSetDefault, bIsLocal );
					bSetDefault = FALSE;
				}
			}
		}
	}
	else
	{
		RemotePrinterInfo* pInf = new RemotePrinterInfo( rPrinterName, rServerName, bSetDefault, bIsLocal );
		maRemotePrinters.Insert( pInf, LIST_APPEND );
		return TRUE;
	}
	return FALSE;
}

// -----------------------------------------------------------------------

RemotePrinterInfo* RemotePrinterList::GetPrinter( const String& rPrinterName, const String& rServerName )
{
	for ( ULONG n = maRemotePrinters.Count(); n; )
	{
		RemotePrinterInfo* pInf = maRemotePrinters.GetObject( --n );
		
		if( ( pInf->aPrinterName == rPrinterName ) && ( pInf->aServerName == rServerName ) )
			return pInf;
	}
	return NULL;
}

// -----------------------------------------------------------------------

RemotePrinterInfo* RemotePrinterList::GetDefaultPrinter() const
{
	RemotePrinterInfo* pInf = NULL;
	for ( ULONG n = maRemotePrinters.Count(); n; )
	{
		pInf = maRemotePrinters.GetObject( --n );
		if ( pInf->bDefaultPrinter )
			return pInf;
	}
	return pInf;	// Dann ggf. der erste
}

// -----------------------------------------------------------------------

Reference< ::com::sun::star::lang::XMultiServiceFactory > RemotePrinterList::GetServerFactory( const String& rServerName )
{
 	RemotePrintServerInfo* pServer = ImplFindPrintServer( rServerName );
 	if ( pServer && !pServer->xFactory.is() )
 		ImplInitPrintServer( pServer );
 	return pServer ? pServer->xFactory : Reference< ::com::sun::star::lang::XMultiServiceFactory >();
}

// -----------------------------------------------------------------------

::vcl::AtomClient* RemotePrinterList::GetServerAtoms( const String& rServerName )
{
 	RemotePrintServerInfo* pServer = ImplFindPrintServer( rServerName );
 	if ( pServer && !pServer->xFactory.is() )
 		ImplInitPrintServer( pServer );
	if( pServer && pServer->xFactory.is() )
	{
		if( pServer->pAtoms == NULL )
			pServer->pAtoms = new ::vcl::AtomClient(
				Reference< ::com::sun::star::portal::client::XAtomServer >(
					pServer->xFactory->createInstance( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( "ClientAtomServer.stardiv.de" ) ) ), UNO_QUERY ) );
	}
 	return pServer ? pServer->pAtoms : NULL;
}

// -----------------------------------------------------------------------

REF( NMSP_CLIENT::XRmPrinterEnvironment ) RemotePrinterList::GetPrinterEnv( const UniString& rServerName )
{
	RemotePrintServerInfo* pServer = ImplFindPrintServer( rServerName );
	
	if( pServer && !pServer->xRmPrinterEnv.is() )
		ImplInitPrintServer( pServer );

	return( pServer ? pServer->xRmPrinterEnv : REF( NMSP_CLIENT::XRmPrinterEnvironment )() );
}

// -----------------------------------------------------------------------

UniString RemotePrinterList::FindLocalPrintServer( const UniString& rPrinterName )
{
	UniString aPrintServer;
	
	for( ULONG n = maRemotePrinters.Count(); n; )
	{
		RemotePrinterInfo* pInf = maRemotePrinters.GetObject( --n );
		
		if( pInf->bLocalPrinter && ( pInf->aPrinterName == rPrinterName ) )
		{
			aPrintServer = pInf->aServerName;
			break;
		}
	}
	return aPrintServer;
}

// -----------------------------------------------------------------------

class ImplRmPrinterChangedClass
{
  public:

	DECL_STATIC_LINK( ImplRmPrinterChangedClass, CallEvent, void* pEvent );
};

// -----------------------------------------------------------------------

IMPL_STATIC_LINK( ImplRmPrinterChangedClass, CallEvent, void*, pEvent )
{
	::vos::OGuard guard( Application::GetSolarMutex() );

	Application* pApp = GetpApp();
	ImplDeletePrnQueueList();
	DataChangedEvent aDCEvt( DATACHANGED_PRINTER );
	pApp->DataChanged( aDCEvt );
	pApp->NotifyAllWindows( aDCEvt );
	delete (ApplicationEvent*)pEvent;

	return 0;
}

// -----------------------------------------------------------------------

void ORmPrinterObserver::PrinterSettingsChanged()
{
	ApplicationEvent*	pEvent = 0;
	String				sEmptyString;

	pEvent = new ApplicationEvent( sEmptyString, sEmptyString, "PRINTERCHANGED", sEmptyString );
	Application::PostUserEvent( STATIC_LINK( NULL, ImplRmPrinterChangedClass, CallEvent ), pEvent );
}

// -------------
// - RmPrinter -
// -------------

void RmPrinter::Create( NMSP_CLIENT::RmQueueInfo& rRmQueueInfo, NMSP_CLIENT::RmJobSetup& rRmJobSetup )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );

	if( mxRemotePrinter.is() )
		mxRemotePrinter->Create( rRmQueueInfo, rRmJobSetup );
}

// -----------------------------------------------------------------------------

BOOL RmPrinter::SetJobSetup( NMSP_CLIENT::RmJobSetup& rRmJobSetup )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->SetJobSetup( rRmJobSetup ) : FALSE );
}

// -----------------------------------------------------------------------------

BOOL RmPrinter::SetOrientation( USHORT nOrientation, NMSP_CLIENT::RmJobSetup& rRmJobSetup )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->SetOrientation( nOrientation, rRmJobSetup ) : FALSE );
}

// -----------------------------------------------------------------------------

BOOL RmPrinter::SetPaperBin( USHORT nPaperBin, NMSP_CLIENT::RmJobSetup& rRmJobSetup )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->SetPaperBin( nPaperBin, rRmJobSetup ) : FALSE );
}

// -----------------------------------------------------------------------------

BOOL RmPrinter::SetPaper( USHORT nPaper, NMSP_CLIENT::RmJobSetup& rRmJobSetup )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->SetPaper( nPaper, rRmJobSetup ) : FALSE );
}

// -----------------------------------------------------------------------------

BOOL RmPrinter::SetPaperSizeUser( long nWidth, long nHeight, NMSP_CLIENT::RmJobSetup& rRmJobSetup )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->SetPaperSizeUser( nWidth, nHeight, rRmJobSetup ) : FALSE );
}

// -----------------------------------------------------------------------------

void RmPrinter::GetPageInfo( long& rOutWidth, long& rOutHeight, long& rPageOffX, long& rPageOffY, 
							 long& rPageWidth, long& rPageHeight )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );

	if( mxRemotePrinter.is() )
		mxRemotePrinter->GetPageInfo( rOutWidth, rOutHeight, rPageOffX, rPageOffY, rPageWidth, rPageHeight );
}

// -----------------------------------------------------------------------------

USHORT RmPrinter::GetPaperBinCount()
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->GetPaperBinCount() : 0 );
}

// -----------------------------------------------------------------------------

OUSTRING RmPrinter::GetPaperBinName( USHORT nPaperBin )
{
	DBG_ASSERT( mxRemotePrinter.is(), "XRmPrinter object is _not_ valid!" );
	return( mxRemotePrinter.is() ? mxRemotePrinter->GetPaperBinName( nPaperBin ) : OUSTRING() );
}

// -----------------------------------------------------------------------------

unsigned long RmPrinter::GetCapabilities( USHORT nType )
{
	DBG_ASSERT( mxRemotePrinter.is() , "XRmPrinter is not valid" );
	return mxRemotePrinter.is() ? mxRemotePrinter->GetCapabilities( nType ) : 0;
}

// -----------------------------------------------------------------------------

void RmPrinter::UserSetup( const REF( NMSP_CLIENT::XRmFrameWindow )& xParent, sal_uInt32 EventID )
{
	DBG_ASSERT( mxRemotePrinter.is() , "XRmPrinter is not valid" );
	mxRemotePrinter->UserSetup( xParent, EventID );
}
