/*************************************************************************
 *
 *  $RCSfile: rmwindow.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:39 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <rmwindow.hxx>
#include <rmevents.hxx>
#include <xevthdl.hxx>
#include <svdata.hxx>
#include <rmcache.hxx>

using namespace ::com::sun::star::uno;

static ::vcl::InterfacePairCache< ::com::sun::star::portal::client::XRmFrameWindow, ::com::sun::star::portal::client::XRmOutputDevice >* pRemoteWindowCache = NULL;
typedef  ::std::pair< ::com::sun::star::uno::Reference< ::com::sun::star::portal::client::XRmFrameWindow >, ::com::sun::star::uno::Reference< ::com::sun::star::portal::client::XRmOutputDevice > > windowInterfacePair;

void eraseRemoteWindowCache()
{
	if( pRemoteWindowCache )
	{
		delete pRemoteWindowCache;
		pRemoteWindowCache = NULL;
	}
}

// -----------------
// - RmFrameWindow -
// -----------------

RmFrameWindow::RmFrameWindow( Window* pServerWindow ) :
	mpEvtHdl( 0 ),
	mePtrStyle( ~0 )
{
	if( ! pRemoteWindowCache )
	{
		ImplSVData* pSVData = ImplGetSVData();
		pRemoteWindowCache = new ::vcl::InterfacePairCache< ::com::sun::star::portal::client::XRmFrameWindow, ::com::sun::star::portal::client::XRmOutputDevice >(
			pSVData->mxMultiFactory,
			::rtl::OUString::createFromAscii( "OfficeWindow.stardiv.de" ),
			5, 5, 20 );
	}

	windowInterfacePair aPair = pRemoteWindowCache->takeInterface();
	mxFrameWindow = aPair.first;
	mxOutputDevice = aPair.second;
	
	mpEvtHdl = (XRmsEventHdl*) new XRmsEventHdl( pServerWindow );
	mxEventHdl = Reference< ::com::sun::star::portal::client::XEventHdl >( mpEvtHdl );
}

// -----------------------------------------------------------------------------

RmFrameWindow::~RmFrameWindow()
{
	// jbu  : tell the event handler, that the window is no longer valid.
        Window* pServerWindow = mpEvtHdl->getWindow();
	mpEvtHdl->setWindow(0);
	mxFrameWindow->ReleaseWindow();
	ImplSVData* pSVData = ImplGetSVData();
	pSVData->mpRmEventQueue->RemoveWindowEvents( pServerWindow );
	if( pRemoteWindowCache )
	{
		windowInterfacePair aPair( mxFrameWindow, mxOutputDevice );
		pRemoteWindowCache->putInterface( aPair );
	}
#ifdef DEBUG
	mpEvtHdl = NULL;
	mxFrameWindow.clear();
	mxOutputDevice.clear();
	mxEventHdl.clear();
#endif
}

// -----------------------------------------------------------------------------

ULONG RmFrameWindow::InsertUserEventLink( const Link& rLink )
{
	return maEvents.Insert( new Link( rLink ) );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::Create( ULONG nWinBits, const REF( NMSP_CLIENT::XEventHdl )& rxEventInterface, const ::com::sun::star::uno::Any& aSystemWorkWindowToken, const REF( NMSP_CLIENT::XRmFrameWindow )& xParent )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->Create( nWinBits, rxEventInterface, aSystemWorkWindowToken, xParent );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetTitle( const OUSTRING& rTitle )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetTitle( rTitle );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::Show( BOOL bVisible )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->Show( bVisible );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetClientSize( sal_Int16 nWidth, sal_Int16 nHeight )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetClientSize( nWidth, nHeight );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::GetClientSize( sal_Int32& rWidth, sal_Int32& rHeight )
{
	sal_Int16 nWidth = 0, nHeight = 0;
	if( mxFrameWindow.is() )
		mxFrameWindow->GetClientSize( nWidth, nHeight );
	rWidth = nWidth;
	rHeight = nHeight;
}

// -----------------------------------------------------------------------------

String RmFrameWindow::GetWindowState()
{
	String aRet;

	if( mxFrameWindow.is() )
		aRet = mxFrameWindow->GetWindowState();

	return aRet;
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetWindowState( const String& rState )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetWindowState( rState );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::GetFontResolution( long& rDPIX, long& rDPIY )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->GetFontResolution( rDPIX, rDPIY );
}

// -----------------------------------------------------------------------------

::com::sun::star::portal::client::RmFrameResolutions RmFrameWindow::GetFrameResolutions()
{
	return mxFrameWindow.is() ? mxFrameWindow->GetFrameResolutions() : ::com::sun::star::portal::client::RmFrameResolutions();
}

// -----------------------------------------------------------------------------

void RmFrameWindow::ToTop( sal_uInt16 nFlags )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->ToTop( nFlags );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::StartPresentation( BOOL bStart, sal_uInt16 nStartFlags )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->StartPresentation( bStart, nStartFlags );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetAlwaysOnTop( BOOL bOnTop )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetAlwaysOnTop( bOnTop );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::ShowFullScreen( BOOL bFullScreen )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->ShowFullScreen( bFullScreen );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::CaptureMouse( BOOL bMouse )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->CaptureMouse( bMouse );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetPointer( PointerStyle ePointerStyle )
{
	if( mxFrameWindow.is() && ePointerStyle != mePtrStyle )
	{
		mePtrStyle = ePointerStyle;
		mxFrameWindow->SetPointer( ePointerStyle );
	}
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetPointerPos( sal_Int16 nX, sal_Int16 nY )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetPointerPos( nX, nY );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::Beep( SoundType eSoundType )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->Beep( eSoundType );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::GetKeyNames( NMSP_CLIENT::KeyNameSequence& rKeyNames )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->GetKeyNames( rKeyNames );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::Enable( sal_Bool bEnable )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->Enable( bEnable );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetIcon( sal_Int16 IconID )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetIcon( IconID );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::SetMinClientSize( sal_Int16 Width, sal_Int16 Height )
{
	if( mxFrameWindow.is() )
		mxFrameWindow->SetMinClientSize( Width, Height );
}

// -----------------------------------------------------------------------------

void RmFrameWindow::MouseMoveProcessed()
{
	if( mxFrameWindow.is() )
		mxFrameWindow->MouseMoveProcessed();
}
// -----------------------------------------------------------------------------

void RmFrameWindow::KeyInputProcessed()
{
	if( mxFrameWindow.is() )
		mxFrameWindow->KeyInputProcessed();
}
