/*************************************************************************
 *
 *  $RCSfile: salmain.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#define _SV_SALMAIN_CXX

// -=-= #includes =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#include <sys/time.h>
#include <sys/resource.h>
#include <salunx.h>

#ifndef _SV_SALDATA_HXX
#include <saldata.hxx>
#endif
#ifndef _SV_SALINST_HXX
#include <salinst.hxx>
#endif
 
#if defined(SOLARIS)
extern int GlobalInit; // in sal/osl/unx/mutex.c dreckiger solaris2.5.1 bug
#endif

#include <svapp.hxx>

// -=-= SingleOffice =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#include <unistd.h>
#include <sys/stat.h>

#include <vos/thread.hxx>
#ifdef SCO
#undef raise
#endif
#include <vos/signal.hxx>
#include <vos/pipe.hxx>

#if defined DEBUG || defined DBG_UTIL
#include <stdio.h>
#endif

#ifndef _VOS_NO_NAMESPACE
using namespace vos;
#endif


class ImplForeignAppEventClass
{
public:
	DECL_STATIC_LINK( ImplForeignAppEventClass, CallEvent, void* pEvent );
};

IMPL_STATIC_LINK( ImplForeignAppEventClass, CallEvent, void*, pEvent )
{
	GetpApp()->AppEvent( *((ApplicationEvent*)pEvent) );
	delete (ApplicationEvent*)pEvent;
	return 0;
}

void ImplPostForeignAppEvent( ApplicationEvent* pEvent )
{
	Application::PostUserEvent( STATIC_LINK( NULL, ImplForeignAppEventClass, CallEvent ), pEvent );
}

class OfficeIPCThread : public NAMESPACE_VOS(OThread)
{
  private:
	static OfficeIPCThread* pGlobalOfficeIPCThread;

	NAMESPACE_VOS(OPipe)			maPipe;
	NAMESPACE_VOS(OStreamPipe)		maStreamPipe;
	static NAMESPACE_VOS(OSecurity)		maSecurity;
	String							maPipeIdent;
	
	OfficeIPCThread();

  protected:  
	/// Working method which should be overridden 
	virtual void run();
  public:
	virtual ~OfficeIPCThread();
	
	static BOOL EnableOfficeIPCThread( int, char** );
	// return FALSE if second office
	static void DisableOfficeIPCThread();
};

#define TERMINATION_SEQUENCE "InternalIPC::TerminateThread"
#define TERMINATION_LENGTH 28

class SalMainPipeExchangeSignalHandler : public NAMESPACE_VOS(OSignalHandler)
{
	virtual TSignalAction signal(TSignalInfo *pInfo);
}; 

OSignalHandler::TSignalAction SalMainPipeExchangeSignalHandler::signal(TSignalInfo *pInfo)
{
	BOOL bTerminate = FALSE;
	switch( pInfo->Signal )
	{
		case osl_Signal_Terminate: bTerminate = TRUE;break;
			
		case osl_Signal_IntegerDivideByZero:
		case osl_Signal_FloatDivideByZero:
		default:
			bTerminate = FALSE;
			break;
	}
	if( bTerminate )
		OfficeIPCThread::DisableOfficeIPCThread();
	return (TAction_CallNextHandler);
}

OfficeIPCThread* OfficeIPCThread::pGlobalOfficeIPCThread = 0;
NAMESPACE_VOS(OSecurity) OfficeIPCThread::maSecurity;

BOOL OfficeIPCThread::EnableOfficeIPCThread( int nArgc, char** ppArgv )
{
	if( pGlobalOfficeIPCThread )
		return TRUE;


	OfficeIPCThread* pThread = new OfficeIPCThread;
    // look if we are the first Office: try to open the pipe
	pThread->maPipeIdent = String::CreateFromAscii(  RTL_CONSTASCII_STRINGPARAM( "SingleOfficeIPC" ) );
	//	String aOfficePath = Application::GetAppFileName();
	// does not seem to work so early; use argv[0] instead
	ByteString aOfficePath = ppArgv[0];
	struct stat aStat;
	if( ! stat( aOfficePath.GetBuffer(), &aStat ) )
	{
		pThread->maPipeIdent += '_';
		pThread->maPipeIdent += String::CreateFromInt32( aStat.st_ino );
	}
	
	pThread->maPipe = NAMESPACE_VOS(OPipe)( pThread->maPipeIdent.GetBuffer(), 
							NAMESPACE_VOS(OPipe)::TOption_Open, maSecurity );
	if( pThread->maPipe.isValid() )
	{
		pThread->maStreamPipe = pThread->maPipe;
		
		// seems another office is running. pipe arguments to it
		// and self terminate
		ByteString aArguments;
		for( int n = 0; n < nArgc; n++ )
		{
			aArguments += ppArgv[ n ];
			aArguments += '|';
		}
		pThread->maStreamPipe.write( aArguments.GetBuffer(), aArguments.Len() );
		delete pThread;
		return FALSE;
	}
	// seems we are the one and only, so start listening thread
	pThread->maPipe = NAMESPACE_VOS(OPipe)( pThread->maPipeIdent, NAMESPACE_VOS(OPipe)::TOption_Create, maSecurity );
	if( pThread->maPipe.isValid() )
	{
		pGlobalOfficeIPCThread = pThread;
		pThread->create(); // starts thread
	}

	return TRUE;
}

void OfficeIPCThread::DisableOfficeIPCThread()
{
	if( pGlobalOfficeIPCThread )
	{
		// send thread a termination message
		// this is done so the subsequent join will not hang
		// because the thread hangs in accept of pipe
		NAMESPACE_VOS(OPipe) Pipe( pGlobalOfficeIPCThread->maPipeIdent, NAMESPACE_VOS(OPipe)::TOption_Open, maSecurity );
		Pipe.send( TERMINATION_SEQUENCE, TERMINATION_LENGTH );
		Pipe.close();
		// close the pipe so that the streampipe on the other
		// side produces EOF

		// exit gracefully
		pGlobalOfficeIPCThread->join();
		delete pGlobalOfficeIPCThread;
	}
}

OfficeIPCThread::OfficeIPCThread()
{
}

OfficeIPCThread::~OfficeIPCThread()
{
	maPipe.close();
	maStreamPipe.close();
	pGlobalOfficeIPCThread = 0;
}

void OfficeIPCThread::run()
{
	while (schedule())
	{
		NAMESPACE_VOS(OPipe)::TPipeError
			nError = maPipe.accept( maStreamPipe );
		if( nError == NAMESPACE_VOS(OStreamPipe)::E_None )
		{
			ByteString aArguments;
			char pBuf[ 2049 ];
			int nBytes;
			do
			{
				nBytes = maStreamPipe.read( pBuf, 2048 );
				pBuf[ nBytes ] = 0;
				aArguments += pBuf;
			} while( ! maStreamPipe.isEof() || nBytes > 0 );
			maStreamPipe.close();
			
			// is this a termination message ? if so, terminate
			if( aArguments.CompareTo( TERMINATION_SEQUENCE,
									  TERMINATION_LENGTH ) == COMPARE_EQUAL )
				return;

			// dispatch app events
			String aAppList;
			int nToken = aArguments.GetTokenCount( '|' );
			// nummer 0 ist der aufruf des office
			for( int n = 1; n < nToken; n++ )
			{
				String aFile( aArguments.GetToken( n, '|' ), gsl_getSystemTextEncoding() );
				if( aFile.Len() )
				{
					if( aAppList.Len() )
						aAppList += APPEVENT_PARAM_DELIMITER;
					aAppList += aFile;
				}
			}
			String aEmpty;
			if( aAppList.Len() )
			{
				ApplicationEvent* pAppEvent =
					new ApplicationEvent( aEmpty, aEmpty,
										  APPEVENT_OPEN_STRING, aAppList );
				ImplPostForeignAppEvent( pAppEvent );
			}
			else
			{
				// no document was send, just bring Office to front
				ApplicationEvent* pAppEvent =
					new ApplicationEvent( aEmpty, aEmpty,
										  "APPEAR", aEmpty );
				ImplPostForeignAppEvent( pAppEvent );
			}
		}
		else
		{
#if defined DEBUG || defined DBG_UTIL
			fprintf( stderr, "Error on accept: %d\n", (int)nError );
#endif
			TimeValue tval;
			tval.Seconds = 1;
			tval.Nanosec = 0;
			sleep( tval );
		}
	}
};


// -=-= main() -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
BEGIN_C

#ifdef S390
final int sv_main( int argc, char *argv[] )
#else
final int main( int argc, char *argv[] )
#endif
{
	struct rlimit limit;

#if defined(SOLARIS)
	GlobalInit = 1; // in sal/osl/unx/mutex.c dreckiger solaris2.5.1 bug
#endif
    
	getrlimit( RLIMIT_NOFILE, &limit );
    limit.rlim_cur = limit.rlim_max;
    setrlimit( RLIMIT_NOFILE, &limit );

#	if defined ( LINUX ) && ( GLIBC < 2 ) 
    __setfpucw( 0x1372 );   // enable fpu exceptions
#	endif

	// check whether the vcl application is a fat-office or something else
	// only the fat-office needs to be prevented from starting twice

	Bool bIsFatOffice;
	{
		const int soffice_len = 11; // = strlen ( "soffice.bin" );
		int 	  image_len   = strlen ( argv[0] );
		int 	  offset	  = image_len - soffice_len;

		if ( offset <  0 )
			bIsFatOffice = False;
		else
		if ( offset == 0 && 
			 0 == strncmp (argv[0] + offset, "soffice.bin", soffice_len) )
			bIsFatOffice = True;
		else // offset >= 1
		if ( 0 == strncmp (argv[0] + offset - 1, "/soffice.bin", 1 + soffice_len) )
			bIsFatOffice = True;
		else
			bIsFatOffice = False;
	}
		
	SalMainPipeExchangeSignalHandler* pSignalHandler=0;
	if ( bIsFatOffice && !Application::IsRemoteServer() )
	{
		if( ! OfficeIPCThread::EnableOfficeIPCThread( argc, argv ) )
			exit( 0 );
		pSignalHandler = new SalMainPipeExchangeSignalHandler;
	}

	if (GetSalData())
		GetSalData()->Init( &argc, argv );

    SVMain();

	if( bIsFatOffice && !Application::IsRemoteServer() )
	{
		OfficeIPCThread::DisableOfficeIPCThread();
		if( pSignalHandler )
			delete pSignalHandler;
	}

	// #75628# avoid calling atexit and fini since this core-dumps if calling pgp-java
	// component
	 _exit(0);
	 
    return 0;
}

END_C

