/*************************************************************************
 *
 *  $RCSfile: kdeint.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <salunx.h>
#include <X11/Xatom.h>

#include <saldisp.hxx>
#include <salframe.hxx>
#include <kdeint.hxx>
#include <config.hxx>

#include <osl/file.hxx>

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#include <tools/fsys.hxx>

#include <tools/urlobj.hxx>

#include <strhelper.hxx>

#define NO_DEFAULT_ICON

using namespace rtl;
using namespace osl;

int   KDEIntegrator::nInstances = 0;

String KDEIntegrator::CreateKDEPath( char* pType )
{
	String aSearchPath;

	String aType( pType, strlen(pType), RTL_TEXTENCODING_ISO_8859_1 );
	if( maLocalKDEDir.Len() )
	{
		aSearchPath += maLocalKDEDir;
		aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "/share/" ) );
		aSearchPath += aType;
	}
	if( maGlobalKDEDir.Len() )
	{
		aSearchPath += ':';
		aSearchPath += maGlobalKDEDir;
		aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "/share/" ) );
		aSearchPath += aType;
	}
	aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( ":/opt/kde/share/" ) );
	aSearchPath += aType;
	aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( ":/usr/local/kde/share/" ) );
	aSearchPath += aType;

	return aSearchPath;
}

KDEIntegrator::KDEIntegrator( SalFrame* pFrame ) :
		DtIntegrator( pFrame )
{
	nInstances++;
	meType = DtKDE;
	rtl_TextEncoding aEncoding = gsl_getSystemTextEncoding();
	const char* pEnv = getenv( "KDEDIR" );
	if( pEnv )
		maGlobalKDEDir = String( pEnv, aEncoding );
	pEnv = getenv( "HOME" );
	if( pEnv )
	{
		maLocalKDEDir = String( pEnv, aEncoding );
		maLocalKDEDir.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "/.kde" ) );
	}

	InitMimeTypes();

	ByteString aBinDir( maGlobalKDEDir, aEncoding );
	aBinDir += "/bin";
	ByteString aPath( "PATH=" );
	pEnv = getenv( "PATH" );
	if( pEnv )
		aPath += pEnv;
	if( aPath.Search( aBinDir ) == STRING_NOTFOUND )
	{
		aPath += ':';
		aPath += aBinDir;
		putenv( const_cast<char*>(aPath.GetBuffer()) );
	}
	mnDndProtocolAtom		= XInternAtom( mpDisplay, "DndProtocol", False );
	mnDndSelectionAtom		= XInternAtom( mpDisplay, "DndSelection", False );
	mnCurrentDesktopAtom	= 0;
	mnCurrentDesktop		= 0;

	// get locale
	pEnv = getenv( "HOME" );
	ByteString aKDErc;
	if( pEnv )
	{
		aKDErc = pEnv;
		aKDErc += "/.kderc";
	}
	SvFileStream aStream( String( aKDErc, aEncoding ), STREAM_READ );
	if( aStream.IsOpen() )
	{
		while( ! aStream.IsEof() &&
			   aKDErc.CompareIgnoreCaseToAscii( "language=", 9 ) != COMPARE_EQUAL )
			aStream.ReadLine( aKDErc );
		if( aKDErc.CompareIgnoreCaseToAscii( "language=", 9 ) == COMPARE_EQUAL )
		{
			aKDErc.Erase( 0, 9 );
			maKDELanguage = aKDErc.GetToken( 0, ':' );
		}
	}
	if( maKDELanguage == "C" )
		maKDELanguage.Erase();
}

KDEIntegrator::~KDEIntegrator()
{
	nInstances--;
	
	// empty the mime type list
	while( maMimeTypes.Count() )
		delete maMimeTypes.Remove( (ULONG)0 );
	
}

KDEMimeType* KDEIntegrator::FindTypeForFile( const String& rFile )
{
	if( maMimeTypes.Count() )
	{
		String aExtension;
		String aFile( rFile );
		int nToken = aFile.GetTokenCount( '.' );
		if( nToken > 1 )
		{
			aExtension = aFile.GetToken( nToken-1, '.' );
			for( int i = 0; i < maMimeTypes.Count(); i++ )
			{
				KDEMimeType* pType = maMimeTypes.GetObject( i );
				nToken = pType->maExtensions.GetTokenCount( ':' );
				for( int n=0; n < nToken; n++ )
				{
					if( pType->maExtensions.GetToken( n, ':' ) == aExtension )
						return pType;
				}
			}
		}
	}
	return 0;
}

void KDEIntegrator::IntegrateMimePath( const String& rDir )
{
	rtl_TextEncoding aEncoding = gsl_getSystemTextEncoding();

	OUString aDirPath;
	::osl::FileBase::normalizePath( rDir, aDirPath );

	Directory aDir( aDirPath );
	aDir.open();
	DirectoryItem aItem;

	while( aDir.getNextItem( aItem ) == FileBase::E_None )
	{
		FileStatus aStatus( FileStatusMask_FileName			|
							FileStatusMask_FilePath			|
							FileStatusMask_NativePath		|
							FileStatusMask_Type
							);
		if( aItem.getFileStatus( aStatus ) == FileBase::E_None )
		{
			String aFileName = aStatus.getFileName();
			if( aFileName.EqualsAscii( "." ) || aFileName.EqualsAscii( ".." ) )
				continue;
			
			OUString aSysPath;
			::osl::FileBase::getSystemPathFromNormalizedPath( aStatus.getFilePath(), aSysPath );

			if( aStatus.getFileType() == FileStatus::Directory )
			{
				IntegrateMimePath( aSysPath );
				continue;
			}
			
			if( ! aFileName.Copy( aFileName.Len()-7 ).EqualsAscii( ".kdelnk" ) )
				continue;
			
			SvFileStream aStream( aSysPath, STREAM_READ );
			if( ! aStream.IsOpen() )
				continue;
			
			ByteString aLine, aKey, aValue;
			KDEMimeType* pType = new KDEMimeType;
			BOOL bInsert = FALSE;
			while( ! aStream.IsEof() )
			{
				aStream.ReadLine( aLine );
				aKey = WhitespaceToSpace( aLine.GetToken( 0, '=' ) );
				aValue = WhitespaceToSpace( aLine.Copy( aKey.Len()+1 ) );
				if( aKey.CompareIgnoreCaseToAscii( "mimetype" ) == COMPARE_EQUAL )
					pType->maName = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "icon" ) == COMPARE_EQUAL )
					pType->maIcon = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "miniicon" ) == COMPARE_EQUAL )
					pType->maMiniIcon = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "type" ) == COMPARE_EQUAL &&
						 aValue.CompareIgnoreCaseToAscii( "mimetype" ) == COMPARE_EQUAL )
					bInsert = TRUE;
				else if( aKey.CompareIgnoreCaseToAscii( "defaultapp" ) == COMPARE_EQUAL )
					pType->maApplication = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "patterns" ) == COMPARE_EQUAL && aValue.Len() )
				{
					aValue.SearchAndReplaceAll( ';', ':' );
					aValue.SearchAndReplaceAll( "*.", "" );
					
					if( aValue.Len() && aValue.GetChar( aValue.Len()-1 ) == ':' )
						aValue.Erase( aValue.Len()-1 );
					
					pType->maExtensions = String( aValue, aEncoding );
				}
			}
			if( bInsert && pType-> maExtensions.Len() )
				maMimeTypes.Insert( pType, LIST_APPEND );
			else
				delete pType;
		}
	}
}

void KDEIntegrator::InitMimeTypes()
{
	int i, nPaths;

	String aSearchPath = CreateKDEPath( "mimelnk" );
	nPaths = aSearchPath.GetTokenCount( ':' );
	for( i = 0; i < nPaths; i++ )
	{
		String aPath = aSearchPath.GetToken( i, ':' );
		IntegrateMimePath( aPath );
	}
}

BOOL KDEIntegrator::StartProcess( String& rFileName, String& rParam
, const String& rDir )
{
	String aFiles( rFileName );
	if( rParam.Len() )
	{
		aFiles += ' ';
		aFiles += rParam;
	}
	
	KDEMimeType* pType = FindTypeForFile( rFileName );
	if( pType )
	{
		aFiles.Insert( ' ', 0 );
		aFiles.Insert( pType->maApplication, 0 );
	}
	
	return LaunchProcess( aFiles, rDir );
}

void KDEIntegrator::ImplHandleXEvent( XEvent* pEvent )
{
	int i;
	
	if( pEvent->type == ClientMessage &&
		pEvent->xclient.message_type == mnDndProtocolAtom )
	{
		XLIB_Window aRoot = DefaultRootWindow( mpDisplay );
		XLIB_Window aWindow = pEvent->xclient.window;
		// find foremost window
		
		int nXRoot = pEvent->xclient.data.l[3];
		int nYRoot = pEvent->xclient.data.l[4];
		int nX, nY;
		
		SalFrame* pTargetFrame;

		if( aWindow == None )
			// strange, but can happen
		{
			pTargetFrame = maDropzones.GetObject( maDropzones.Count() - 1 );
			aWindow = pTargetFrame->maFrameData.GetWindow();
		}
		XLIB_Window aChild = None;
		do
		{
			XTranslateCoordinates( mpDisplay, aRoot, aWindow,
								   nXRoot, nYRoot, &nX, &nY, &aChild );
			if( aChild != None )
				aWindow = aChild;
		} while( aChild != None );
		
		for( i = 0; i < maDropzones.Count(); i++ )
		{
			pTargetFrame = maDropzones.GetObject( i );
			if( pTargetFrame->maFrameData.GetWindow() == aWindow )
				break;
		}
		if( i >= maDropzones.Count() )
			pTargetFrame = 0;
		// dont know which frame ? Oops
		// but we have one visible at the moment anyway
		
		Atom aActualType;
		int  nActualFormat;
		unsigned long nItems;
		unsigned long nBytesAfter;
		unsigned char *pData=0;
		XGetWindowProperty( mpDisplay,
							DefaultRootWindow( mpDisplay ),
							mnDndSelectionAtom,
							0,          // long_offset
							16384,      // long_length, a maximum of 64k will e retrieved
							True,       // delete property after reading
							XA_STRING,  // the type we want
							&aActualType, &nActualFormat,
							&nItems, &nBytesAfter, &pData );
	
		if( pData )
		{
			DtDataType eType = DtTypeText;
			switch( pEvent->xclient.data.l[0] )
			{
				case 128: eType = DtTypeUrl; break;
				case 4:
				default: eType = DtTypeText;
			}
			
			DtData* pDtData = new DtData;
			pDtData->meType = eType;
			pDtData->mnX = nX;
			pDtData->mnY = nY;
			pDtData->mpFrame = pTargetFrame;
			pDtData->mnBytes = nItems*(nActualFormat/8 );
			pDtData->mpBytes = new unsigned char[ pDtData->mnBytes + 2 ];
			memcpy( pDtData->mpBytes, pData, pDtData->mnBytes );
			pDtData->mpBytes[ pDtData->mnBytes     ] = 0;
			pDtData->mpBytes[ pDtData->mnBytes + 1 ] = 0;
			pDtData->mnBytes += 2;

			XFree( pData );
			
			DtDropQuery aQuery;
			aQuery.m_nX			= nX;
			aQuery.m_nY			= nY;
			aQuery.m_eAction	= DtDropCopy;
			aQuery.m_pFrame		= pTargetFrame;

			maBeginDropHdl.Call( pDtData );
			maQueryDropHdl.Call( &aQuery );
			maDropFinishHdl.Call( &aQuery );
		}
	}
	else if( meDragState != DtDragNone && pEvent->type == ButtonRelease )
	{
		XLIB_Window aRoot = DefaultRootWindow( mpDisplay );
		
		int nXRoot, nYRoot, nX, nY;
		XLIB_Window aWindow;
		XTranslateCoordinates( mpDisplay, aRoot, aRoot,
							   pEvent->xbutton.x_root, pEvent->xbutton.y_root,
							   &nXRoot, &nYRoot, &aWindow );
		if( aWindow != None )
		{
			// get the topmost window
			XLIB_Window aChild;
			do
			{
				XTranslateCoordinates( mpDisplay, aRoot, aWindow,
									   nXRoot, nYRoot, &nX, &nY, &aChild );
				if( aChild != None )
					aWindow = aChild;
			} while( aChild != None );

			// do not drop onto ourself; this case is better handled
			// by XDND because this transports actions
			for( i = 0; i < maDropzones.Count(); i++ )
			{
				if( maDropzones.GetObject( i )->maFrameData.GetWindow() == aWindow )
					break;
			}
			if( i >= maDropzones.Count() )
			{
#if defined DEBUG
				fprintf( stderr, "KDEIntegrator does KDE drop\n" );
#endif
				DtData aData;
				maQueryDragDataHdl.Call( &aData );
				// the handle fills in some suitable type since
				// aData.mpType is not set
				if( aData.mnBytes && aData.mpBytes )
				{
					XChangeProperty( mpDisplay, aRoot, mnDndSelectionAtom,
									 XA_STRING, 8, PropModeReplace,
									 aData.mpBytes, aData.mnBytes );

					XEvent aEvent;
					aEvent.xclient.type				= ClientMessage;
					aEvent.xclient.display			= mpDisplay;
					aEvent.xclient.message_type		= mnDndProtocolAtom;
					aEvent.xclient.format			= 32;
					aEvent.xclient.window			= aWindow;
					
					switch( aData.meType )
					{
						case DtTypeUrl:	aEvent.xclient.data.l[0] = 128;break;
						case DtTypeText:
						default:
							aEvent.xclient.data.l[0] = 4; break;
					}
					aEvent.xclient.data.l[1]		= 0;
					aEvent.xclient.data.l[2]		= 0;
					aEvent.xclient.data.l[3]		= nXRoot;
					aEvent.xclient.data.l[4]		= nYRoot;
					
					XSendEvent( mpDisplay, aWindow, True, NoEventMask, &aEvent );
				}
			}
		}
	}
	DtIntegrator::ImplHandleXEvent( pEvent );
}


BOOL KDEIntegrator::GetSystemLook( SystemLookInfo& rInfo )
{
	BOOL bRet = FALSE;

	String aFile( aHomeDir );
	aFile.AppendAscii("/.kderc");
	Config aConfig( aFile );

	ByteString aLine;
	if( aConfig.HasGroup( "WM" ) )
	{
		aConfig.SetGroup( "WM" );
		bRet = TRUE;

		aLine = aConfig.ReadKey( "activeBackground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowActiveStart =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
				   	   aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "activeBlend" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowActiveEnd =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
				   	   aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "inactiveBackground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowInactiveStart =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
					   aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "inactiveBlend" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowInactiveEnd =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
					   aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "inactiveForeground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.inactiveForeground =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
					   aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "activeForeground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.activeForeground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "titleFont" );
		rInfo.windowFont = String( aLine, gsl_getSystemTextEncoding() );
	}

	if( aConfig.HasGroup( "General" ) )
	{
		aConfig.SetGroup( "General" );
		bRet = TRUE;

		aLine = aConfig.ReadKey( "foreground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.foreground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "background" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.background = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "selectForeground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.selectForeground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "selectBackground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.selectBackground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "font" );
		rInfo.controlFont = String( aLine, gsl_getSystemTextEncoding() );
	}

	return bRet;
}
