/*************************************************************************
 *
 *  $RCSfile: salpimpl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <assert.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#ifdef AIX
#include <strings.h>
#endif

#define __salpmacr_h
#include <prex.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <xprinter/xp.h>
// not exported xprinter function:
extern "C" int XpSetBitmapCompression( Display*, int );
#include <postx.h>

#include <salpdecl.h>
#include <xsalprn.h>
#include <salconfig.hxx>
#include <ppdparser.hxx>

#ifdef AIX
#define NO_EXTENDED_REQUEST_SIZE
#endif

using namespace vcl_sal;

static const char* CVS_Id = "$Header: /cvs/oo/gsl/vcl/unx/source/gdi/salpimpl.cxx,v 1.1.1.1 2000/09/18 17:05:43 hr Exp $";

/* some private Xprinter defines of version Xp3.10 */
#define XpPRINTERMAGIC 9898
#define XpENV "XPPATH"
/*
#define XpDocENV "XPDOCNAME"
#define XpDefaultPCLFile "xprinter.pcl"
#define XpDefaultPSFile "xprinter.ps"
#define XpDefaultEPSFile "xprinter.eps"
*/


#define ELEMENTS(array)	(sizeof(array)/sizeof(array[0]))

/* ===================================================================== */

extern int GetLevelFromDriver( char* pDriver );
extern int GetColordeviceFromDriver( char* pDriver );

/* ===================================================================== */

#ifndef PRINTER_DUMMY
#define PRINTER_IMPLEMENTATION
int XSalIsDummyPrinter = False;
#else
int XSalIsDummyPrinter = True;
#endif

static Display* dummyPrinter = (Display*)(void*)&dummyPrinter;

const Display * pLastActiveSalPrinter = NULL;

static inline int IsDisplay( const Display * display )
{
	assert( display );

	if ( display != dummyPrinter && ConnectionNumber( display ) != XpPRINTERMAGIC )
		return 1;
	else
	{
		pLastActiveSalPrinter = display;
		return 0;
	}
}

int XSalIsDisplay( const Display * display )
{
	return IsDisplay( display );
}

int XSalIsPrinter( const Display * display )
{
	return !IsDisplay( display );
}


/* ===================================================================== */


static XVisualInfo printerVInfo[] =
{
	/* Visual*				  	id  screen	depth	class			redMask   greenMask blueMask  colors bits */
	{ (Visual*)&printerVInfo[0],	0,	0,		 8,		PseudoColor,	0,		  0,		0,		  265,	 8	},
	{ (Visual*)&printerVInfo[1],	1,	0,		24,		TrueColor,		0x0000FF, 0x00FF00, 0xFF0000, 0,	32	}
};

Visual* XSalDefaultVisual( Display * display, int screen )
{
	if ( IsDisplay( display ) )
		return DefaultVisual( display, screen );
	else
	{
#ifdef PRINTER_IMPLEMENTATION
		int depth = XpDefaultDepth( display, screen );
		int i;
		for ( i = 0; i < ELEMENTS(printerVInfo); i++ )
		{
			XVisualInfo* my_vinfo = &printerVInfo[i];
			if ( my_vinfo->depth == depth )
				return my_vinfo->visual;
		}
		assert( 0 );
#endif
		return printerVInfo[0].visual;
	}
}

long unsigned int XSalVisualIDFromVisual( Visual * visual )
{
	if ( visual >= (Visual*)&printerVInfo[0] &&
		 visual <= (Visual*)&printerVInfo[ELEMENTS(printerVInfo)-1] )
		return ((XVisualInfo*)visual)->visualid;
	else
		return XVisualIDFromVisual( visual );
}

XVisualInfo* XSalGetVisualInfo( Display*		display,
								long int		vinfo_mask,
								XVisualInfo*	vinfo_template,
								int*			nitems )
{
	if ( IsDisplay( display ) )
		return XGetVisualInfo( display, vinfo_mask, vinfo_template, nitems );
	else
	{
		XVisualInfo* vinfo = (XVisualInfo*)malloc( sizeof( XVisualInfo ) * ELEMENTS(printerVInfo) );
		int i;
		*nitems = 0;
		for ( i = 0; i < ELEMENTS(printerVInfo); i++ )
		{
			XVisualInfo* my_vinfo = &printerVInfo[i];
			int bInsert = True;
			if ( vinfo_mask & VisualIDMask
				 && vinfo_template->visualid		!= my_vinfo->visualid )
				bInsert	= False;
			if ( vinfo_mask & VisualScreenMask
				 && vinfo_template->screen		!= my_vinfo->screen	)
				bInsert	= False;
			if ( vinfo_mask & VisualDepthMask
				 && vinfo_template->depth			!= my_vinfo->depth )
				bInsert	= False;
			if ( vinfo_mask & VisualClassMask
				 && vinfo_template->c_class			!= my_vinfo->c_class )
				bInsert	= False;
			if ( vinfo_mask & VisualRedMaskMask
				 && vinfo_template->red_mask		!= my_vinfo->red_mask )
				bInsert	= False;
			if ( vinfo_mask & VisualGreenMaskMask
				 && vinfo_template->green_mask	!= my_vinfo->green_mask )
				bInsert	= False;
			if ( vinfo_mask & VisualBlueMaskMask
				 && vinfo_template->blue_mask		!= my_vinfo->blue_mask )
				bInsert	= False;
			if ( vinfo_mask & VisualColormapSizeMask
				 && vinfo_template->colormap_size	!= my_vinfo->colormap_size )
				bInsert	= False;
			if ( vinfo_mask & VisualBitsPerRGBMask
				 && vinfo_template->bits_per_rgb	!= my_vinfo->bits_per_rgb )
				bInsert	= False;
			if ( bInsert )
				vinfo[(*nitems)++] = *my_vinfo;
		}
		return vinfo;
	}
}

int XSalMatchVisualInfo(
	Display*		display,
	int			screen,
	int			depth,
	int			c_class,
	XVisualInfo*	vinfo )
{
	if ( IsDisplay( display ) )
		return XMatchVisualInfo( display, screen, depth, c_class, vinfo );
	else
	{
		XVisualInfo*	vinfos;
		int				nitems;
		XVisualInfo		vinfo_template;
		vinfo_template.screen	= screen;
		vinfo_template.depth	= depth;
		vinfo_template.c_class	= c_class;
		vinfo = XSalGetVisualInfo( display,
								   VisualScreenMask | VisualDepthMask | VisualClassMask,
								   &vinfo_template, &nitems );
		if ( !vinfo || !nitems )
			return BadMatch;

		*vinfo = vinfos[0];
		XFree( vinfos );

		return 0;
	}
}


/* ===================================================================== */


long int XSalExtendedMaxRequestSize( Display * display )
{
#ifndef NO_EXTENDED_REQUEST_SIZE
	if ( IsDisplay( display ) )
		return XExtendedMaxRequestSize( display );
	else
#endif
		return 0;
}

long int XSalMaxRequestSize( Display * display )
{
	if ( IsDisplay( display ) )
		return XMaxRequestSize( display );
	else
		return 4 * 4096;
}


/* ===================================================================== */


int	XSalCanDrawRotString( Display * display, GC gc )
{
	return !IsDisplay( display );
}

int	XSalDrawRotString( Display*	display,
					   Drawable	drawable,
					   GC		gc,
					   int x, int y,
					   const char* string, int length,
					   int angle )
{
	if ( XSalCanDrawRotString( display, gc ) )
#ifdef PRINTER_IMPLEMENTATION
		return XpDrawRotString( display, drawable, gc, x, y,
								(char*)string, length, angle );
#else
	{
		if ( IsDisplay( display ) )
			return XDrawString( display, drawable, gc, x, y,
								(char*)string, length );
		else
			return 0;
	}
#endif
	else
		return 0;
}

int	XSalDrawRotImageString( Display*	display,
							Drawable	drawable,
							GC		gc,
							int x, int y,
							const char* string, int length,
							int angle )
{
	if ( XSalCanDrawRotString( display, gc ) )
#ifdef PRINTER_IMPLEMENTATION
		return XpDrawRotImageString( display, drawable, gc, x, y,
									 (char*)string, length, angle );
#else
	{
		if ( IsDisplay( display ) )
			return XDrawImageString( display, drawable, gc, x, y,
									 (char*)string, length );
		else
			return 0;
	}
#endif
	else
		return 0;
}

int	XSalDrawRotString16( Display*	display,
						 Drawable	drawable,
						 GC		gc,
						 int x, int y,
						 const unsigned short* string, int length,
						 int angle )
{
	if ( XSalCanDrawRotString( display, gc ) )
#ifdef PRINTER_IMPLEMENTATION
#if 0
		return XpDrawRotString16( display, drawable, gc, x, y,
								  (char*)string, length, angle );
#else
	return 0;
#endif
#else
	{
		if ( IsDisplay( display ) )
			return XDrawString16( display, drawable, gc, x, y,
								  (const XChar2b*)string, length );
		else
			return 0;
	}
#endif
	else
		return 0;
}

int	XSalDrawRotImageString16( Display*	display,
							  Drawable	drawable,
							  GC		gc,
							  int x, int y,
							  const unsigned short* string, int length,
							  int angle )
{
	if ( XSalCanDrawRotString( display, gc ) )
#ifdef PRINTER_IMPLEMENTATION
#if 0
		return XpDrawRotImageString16( display, drawable, gc, x, y,
									   (char*)string, length, angle );
#else
	return 0;
#endif
#else
	{
		if ( IsDisplay( display ) )
			return XDrawImageString16( display, drawable, gc, x, y,
									   (const XChar2b*)string, length );
		else
			return 0;
	}
#endif
	else
		return 0;
}


/* ===================================================================== */


#define MAX_XERROR_LEN 1024
static int PrinterErrorDefaultHandler( XErrorEvent* pEvent )
{
#ifdef DBG_UTIL
	char message[MAX_XERROR_LEN];

	XSalGetErrorText( pEvent->display, pEvent->error_code, message, MAX_XERROR_LEN );

#ifdef NLS
	if( pEvent->serial == XP_WARNING )
		fprintf( stderr, catgets(catd, 1, 1, "Warning") );
	else
		fprintf( stderr, catgets(catd, 1, 2, "Error") );
#else
	if( pEvent->serial == XP_WARNING )
		fprintf( stderr, "Warning" );
	else
		fprintf( stderr, "Error" );
#endif
	fprintf( stderr, ": %s\n", message );
	/* fflush(stderr); */

	/*
	  if( pEvent->serial == XP_ERROR )
	  abort();
	*/
#endif
	return 0;
}

static XSalPrinterErrorHandler pXSalPrinterErrorHandler = PrinterErrorDefaultHandler;

XSalPrinterErrorHandler XSalSetPrinterErrorHandler( XSalPrinterErrorHandler handler )
{
	XSalPrinterErrorHandler pOldHandler = pXSalPrinterErrorHandler;
	if ( handler )
		pXSalPrinterErrorHandler	= handler;
	else
		pXSalPrinterErrorHandler	= PrinterErrorDefaultHandler;
	return pOldHandler;
}

extern int XSalGetErrorText( Display * display, int code, char * buffer, int length )
{
	if ( IsDisplay( display ) )
		return XGetErrorText( display, code, buffer, length );
	else
	{
#ifdef PRINTER_IMPLEMENTATION
		XpGetErrorText( buffer, length );
#else
		strncpy( buffer, "no printer lib installed", length );
#endif
		return 0;
	}
}


/* ===================================================================== */


#include "salpimp1.cxx"


/* ===================================================================== */


/* printer implementation */

static int PrinterErrorWrapper( XpErrorEvent *err )
{
	int res;

	XErrorEvent aEvent;
	aEvent.type			= 0;
	aEvent.display		= (Display*)pLastActiveSalPrinter;
	aEvent.resourceid		= 0;
	aEvent.serial			= err->severity;
	aEvent.error_code		= err->error_code;
	aEvent.request_code	= 0;
	aEvent.minor_code		= 0;

	if ( !pLastActiveSalPrinter )
		aEvent.display = dummyPrinter;

	res = pXSalPrinterErrorHandler( &aEvent );

	/* if( err->severity == XP_WARNING )	*/
	/* if( err->severity == XP_ERROR )	*/

	return res;
}

/* ===================================================================== */

static struct
{
	int			width;
	int			height;
	const char*	name;
	int			namelength;
	Paper		paper;
} aPaperTab[] =
{
	{ 29700, 42000,	"A3",			2,	PAPER_A3		},
		{ 21000, 29700,	"A4",			2,	PAPER_A4		},
			{ 14800, 21000,	"A5",			2,	PAPER_A5		},
				{ 25000, 35300,	"B4",			2,	PAPER_B4		},
					{ 17600, 25000,	"B5",			2,	PAPER_B5		},
						{ 21600, 27900,	"Letter",		6,	PAPER_LETTER	},
							{ 21600, 35600,	"Legal",		5,	PAPER_LEGAL		},
								{ 27900, 43100,	"Tabloid",	7,	PAPER_TABLOID	},
									{ 0, 0,	  		"USER",		4,	PAPER_USER		}
};

#ifdef DBG_UTIL
static void CheckPaperTable()
{
	int entries = ELEMENTS(aPaperTab);

	Paper paper;

	for ( paper = 0; paper < entries; paper++ )
	{
		if ( aPaperTab[paper].paper != paper )
		{
			fprintf( stderr, "wrong paper table in " __FILE__ "\n" );
			abort();
		}
	}
}
#endif

#define PAPER_DEFAULT	PAPER_A4
#define PAPER_ILLEGAL	((Paper)-1)

const char* XSalGetPaperName( XSalEnum ePaper )
{
	if ( ePaper >= ELEMENTS(aPaperTab) )
		return aPaperTab[PAPER_DEFAULT].name;

	return aPaperTab[ePaper].name;
}

XSalEnum XSalGetPaper( const char* sPaperName )
{
	unsigned short nPaper;
	
	for ( nPaper = 0;	nPaper < PAPER_USER; nPaper++ )
	{
		if ( !strcasecmp( sPaperName, aPaperTab[nPaper].name ) )
			return aPaperTab[nPaper].paper;
	}
	return PAPER_ILLEGAL;
}


/* ===================================================================== */

XSalEnum XSalGetOrientation( const char* string )
{
	if ( 0 == strcasecmp( string, "LANDSCAPE" ) )
		return ORIENTATION_LANDSCAPE;
	else
		return ORIENTATION_PORTRAIT;
}

static inline XpOrientation_t GetXpOrientation( Orientation e )
{
	if ( e == ORIENTATION_LANDSCAPE )
		return XpLANDSCAPE;
	else
		return XpPORTRAIT;
}

const char* XSalGetOrientationName( XSalEnum eOrientation )
{
	if ( eOrientation == ORIENTATION_LANDSCAPE )
		return "LANDSCAPE";
	else
		return "PORTRAIT";
}

/* ===================================================================== */

#define DRV_PostScript	0
#define DRV_PCL4		1
#define DRV_PCL5		2
#define DRV_PCL			3

static const char *_DriverID[]	= { "PostScript", "PCL4", "PCL5", "PCL", NULL };

unsigned short XSalGetPrinterDriverId( const char* pDriverType )
{
	unsigned short i;
	for( i = 0;
		 _DriverID[i] && strcasecmp( _DriverID[i], pDriverType );
		 i++ )
		;
	return ( _DriverID[i] ) ? i : 0;	// XSalDefaultDid
}

const char* XSalGetPrinterDriverName( XSalEnum eDriver )
{
	if ( eDriver > ELEMENTS(_DriverID) )
		return NULL;
	return _DriverID[eDriver];
}


/* ===================================================================== */

static const char aImplWindows[]	= "windows";	/* section declaring default printer	*/
static const char aImplDevice[]		= "device";		/* default printer entry				*/
static const char aImplPorts[]		= "ports";		/* section declaring printer commands	*/
static const char aImplDevices[]	= "devices";	/* section declaring printers			*/
static const char aImplAuto[]		= "auto";		/* automaic entry value					*/

/* ===================================================================== */

static const char aImplDefaultPort[]  = "/usr/bin/lpr -n $XSalCopies";

/* ===================================================================== */

void XSalPrinterInit( const String& installPath )
{
	static bInited = False;
	if ( !bInited )
	{
		bInited = True;
#ifdef PRINTER_IMPLEMENTATION
		if ( !getenv(XpENV) )
		{
			ByteString ByteInstallPath( installPath, gsl_getSystemTextEncoding() );

			char* env = (char*)malloc( ByteInstallPath.Len() + sizeof(XpENV) + 6 );
			strcpy( env, XpENV );
			strcat( env, "=" );
			strcat( env, ByteInstallPath.GetBuffer() );
			strcat( env, "/xp3" );
#ifdef DBG_UTIL
			fprintf(stderr, "\"%s\"\n", env );
#endif
			putenv( env );
		}
		XpUnlock( "aihjdw0s" );
		XpSetErrorHandler( PrinterErrorWrapper );
#endif
#ifdef DBG_UTIL
		CheckPaperTable();
#endif
	}
}

void XSalInitPrinterSetup( XSalPrinterSetup* pSetup )
{
	pSetup->nMagic	= XSAL_PRINTER_SETUP_MAGIC;
	pSetup->nVersion	= 0x0003;

	/* set the following as defaults */
	pSetup->nFlags			= XSALPRINTERSETUP_FLAG_DEFAULT ;
	pSetup->eDriver			= 0;	/* Postscript */
	pSetup->nCopies			= 1;
	pSetup->nPaperBin			= 0;
	pSetup->ePaper			= PAPER_USER;
	pSetup->nPaperWidth		= 21000;
	pSetup->nPaperWidth		= 29700;
	pSetup->eOrientation		= ORIENTATION_PORTRAIT;
	pSetup->nScale			= 100;
	pSetup->nResolutionX		= 300;
	pSetup->nResolutionY		= 300;
	pSetup->nTrailingBytes	= 0;

	memset( pSetup->sCompatCommand, 0, sizeof( pSetup->sCompatCommand ) );
	memset( pSetup->sPort, 0, sizeof( pSetup->sPort ) );
	memset( pSetup->cDriverName, 0, sizeof( pSetup->cDriverName ) );
}

static inline void Swap16( unsigned short* p )
{
	*p =
		((*p & 0x00ff) << 8) |
		((*p & 0xff00) >> 8);
}
static inline void Swap32( unsigned int* p )
{
	*p =
		((*p & 0x000000ff) << 24) |
		((*p & 0x0000ff00) << 8)  |
		((*p & 0x00ff0000) >> 8)  |
		((*p & 0xff000000) >> 24);
}

static inline void CorrectEndianess( XSalPrinterSetup* pSetup )
{
	if ( pSetup->nMagic != XSAL_PRINTER_SETUP_MAGIC )
		XSalCorrectEndianess( pSetup );
}

extern void XSalCorrectEndianess( XSalPrinterSetup* pSetup )
{
	if ( pSetup->nMagic == XSAL_PRINTER_SETUP_MAGIC )
	{
		if( pSetup->nVersion < 3 )
			pSetup->nTrailingBytes = 0;
		return;
	}
	else if ( pSetup->nMagic == WRONG_ENDIANESS_MAGIC )
	{
		Swap32( &pSetup->nMagic );
		Swap16( &pSetup->nVersion );
		Swap16( &pSetup->nFlags );
		Swap16( &pSetup->eDriver );
		Swap16( &pSetup->nCopies );
		Swap16( &pSetup->nPaperBin );
		Swap16( &pSetup->ePaper );
		Swap32( &pSetup->nPaperWidth );
		Swap32( &pSetup->nPaperHeight );
		Swap16( (unsigned short*)&pSetup->eOrientation );
		Swap32( &pSetup->nScale );
		Swap16( &pSetup->nResolutionX );
		Swap16( &pSetup->nResolutionY );
		if( pSetup->nVersion >= 3 )
			Swap32( &pSetup->nTrailingBytes );
		else
			pSetup->nTrailingBytes = 0;
	}
	else
	{
	}
}

/* valid entries in printer default section */
/*
  Filename			filename	; name of file to print to
  Scale				0			; 0 = 1.0
  Copies				0			; number of copies
  BitmapCompression	-1			; use bitmap compression
  Orientation			[LandScape,Portrait]
  PageSize			A4			; paper size to use
  PaperTray			Cassette	; paper bin to use
  DPI					300			; resolution
*/
/* following entries inserted by eg */
/*
  Level				2			; postscript level
  BitmapColor			-1			; use color when printing bitmaps
*/

typedef struct
{
	int left;
	int top;
	int right;
	int bottom;
} Margins;

static const char sEmptyString[] = "";

#ifdef PRINTER_IMPLEMENTATION
#define LEFT_MARGIN   1
#define TOP_MARGIN    2
#define RIGHT_MARGIN  4
#define BOTTOM_MARGIN 8

extern void ReadPrinterMargin( char* pDriver, char* pPaper,
							   int* pLeft, int* pRight,
							   int* pUpper, int* pLower );

extern const char* GetDefaultPrinterPaper( char* );

static int ReadMargin( Margins* pMargins, const char* section )
{
	int automatic = 0;
	int manual	= 0;

	if( XpDefaults::getXpDefaults().HasGroup( section ) )
	{
		XpDefaults::getXpDefaults().SetGroup( section );

		/* determine left margin */
		ByteString aLine = XpDefaults::getXpDefaults().ReadKey( "MarginLeft" );
		if( aLine.Len() )
		{
			if ( aLine.CompareIgnoreCaseToAscii( aImplAuto ) == COMPARE_EQUAL )
				automatic |= LEFT_MARGIN;
			else
			{
				manual |= LEFT_MARGIN;
				pMargins->left	= aLine.ToInt32();
			}
		}
		else
			automatic |= LEFT_MARGIN;

		/* determine top margin */
		aLine = XpDefaults::getXpDefaults().ReadKey( "MarginTop" );
		if ( aLine.Len() )
		{
			if ( aLine.CompareIgnoreCaseToAscii( aImplAuto ) == COMPARE_EQUAL )
				automatic |= TOP_MARGIN;
			else
			{
				manual |= TOP_MARGIN;
				pMargins->top	= aLine.ToInt32();
			}
		}
		else
			automatic |= TOP_MARGIN;

		/* determine right margin */
		aLine = XpDefaults::getXpDefaults().ReadKey( "MarginRight" );
		if ( aLine.Len() )
		{
			if ( aLine.CompareIgnoreCaseToAscii( aImplAuto ) == COMPARE_EQUAL )
				automatic |= RIGHT_MARGIN;
			else
			{
				manual |= RIGHT_MARGIN;
				pMargins->right	= aLine.ToInt32();
			}
		}
		else
			automatic |= RIGHT_MARGIN;

		/* determine bottom margin */
		aLine = XpDefaults::getXpDefaults().ReadKey( "MarginBottom" );
		if ( aLine.Len() )
		{
			if ( aLine.CompareIgnoreCaseToAscii( aImplAuto ) == COMPARE_EQUAL )
				automatic |= BOTTOM_MARGIN;
			else
			{
				manual |= BOTTOM_MARGIN;
				pMargins->bottom = aLine.ToInt32();
			}
		}
		else
			automatic |= BOTTOM_MARGIN;
	}

	return automatic | (manual << 4);
}
#endif

unsigned short XSalGetLanguageLevel( const XSalPrinterSetup* pSetup )
{
	return pSetup->nFlags & XSALPRINTERSETUP_FLAG_LEVEL;
}

unsigned short XSalGetDepth( const XSalPrinterSetup* pSetup )
{
	switch ( (pSetup->nFlags & XSALPRINTERSETUP_FLAG_DEPTH ) >> XSALPRINTERSETUP_FLAG_DEPTH_SHIFT )
	{
		case 0: return 1;
		case 1: return 2;
		case 2: return 4;
		case 3: return 8;
		case 4: return 16;
		case 5: return 32;
		case 6: return 24;
		case XSALPRINTERSETUP_FLAG_DEPTH_DEFAULT >> XSALPRINTERSETUP_FLAG_DEPTH_SHIFT:
			return 24;
	}
	return 0;
}

void XSalSetDepth( XSalPrinterSetup* pSetup, unsigned short depth )
{
	switch( depth )
	{
		case 24: depth=6;break;
		case 32: depth=5;break;
		case 16: depth=4;break;
		case  8: depth=3;break;
		case  4: depth=2;break;
		case  2: depth=1;break;
		default: depth=XSALPRINTERSETUP_FLAG_DEPTH_DEFAULT >> XSALPRINTERSETUP_FLAG_DEPTH_SHIFT;break;
	};

	pSetup->nFlags &= ~XSALPRINTERSETUP_FLAG_DEPTH;
	pSetup->nFlags |= depth << XSALPRINTERSETUP_FLAG_DEPTH_SHIFT;

}

void XSalGetLanguageLevels( const XSalPrinterSetup* pSetup,
							unsigned short* minLevel,
							unsigned short* maxLevel )
{
	switch ( pSetup->eDriver )
	{
		case DRV_PostScript:
			if ( minLevel ) *minLevel = 1;
			if ( maxLevel ) *maxLevel = 2;
			break;
		case DRV_PCL4:
			if ( minLevel ) *minLevel = 4;
			if ( maxLevel ) *maxLevel = 4;
			break;
		case DRV_PCL5:
			if ( minLevel ) *minLevel = 5;
			if ( maxLevel ) *maxLevel = 5;
			break;
		case DRV_PCL:
			if ( minLevel ) *minLevel = 4;
			if ( maxLevel ) *maxLevel = 5;
			break;
		default: 
			if ( minLevel ) *minLevel = 0;
			if ( maxLevel ) *maxLevel = 0;
	}
}

void XSalSetLanguageLevel( XSalPrinterSetup* pSetup, unsigned short level )
{
	unsigned short nmin, nmax;
	XSalGetLanguageLevels( pSetup, &nmin, &nmax );

	if ( level < nmin )
		level = nmin;
	else if ( level > nmax )
		level = nmin;  

	pSetup->nFlags	&= ~XSALPRINTERSETUP_FLAG_LEVEL;
	pSetup->nFlags	|= XSALPRINTERSETUP_FLAG_LEVEL & level;
}

XSalEnum XSalGetColorSpace( const XSalPrinterSetup* pSetup )
{
	if ( pSetup->nFlags & XSALPRINTERSETUP_FLAG_COLOR )
		return COLOR_SPACE_COLOR;
	else
		return COLOR_SPACE_GRAY;
}

void XSalSetColorSpace( XSalPrinterSetup* pSetup, XSalEnum space )
{
	if ( COLOR_SPACE_COLOR == space || COLOR_SPACE_GRAY == space )
	{
		pSetup->nFlags	&= ~( XSALPRINTERSETUP_FLAG_COLOR | XSALPRINTERSETUP_FLAG_COLOR_DEFAULT );
		if( space == COLOR_SPACE_COLOR )
			pSetup->nFlags	|= XSALPRINTERSETUP_FLAG_COLOR;
	}
}

XSalBool XSalGetBmpCompression( const XSalPrinterSetup* pSetup )
{
	XSalBool bVal;

	if ( pSetup->nFlags & XSALPRINTERSETUP_FLAG_COMPRESS_DEFAULT ) 
		bVal = 1; /* same default as the xprinter */
	else
		bVal = (0 != ( pSetup->nFlags & XSALPRINTERSETUP_FLAG_COMPRESS ) );

	return bVal;
}

void XSalSetBmpCompression( XSalPrinterSetup* pSetup, XSalBool compress )
{
	if ( compress )
		pSetup->nFlags	|= XSALPRINTERSETUP_FLAG_COMPRESS;
	else
		pSetup->nFlags	&= ~XSALPRINTERSETUP_FLAG_COMPRESS;

	pSetup->nFlags &= ~XSALPRINTERSETUP_FLAG_COMPRESS_DEFAULT ;

}

void XSalSetupPrinter( XSalPrinter * printer, const XSalPrinterSetup* pSetup )
{
#ifdef PRINTER_IMPLEMENTATION
	assert( printer && !IsDisplay( printer ) );
	assert( pSetup );
	
	CorrectEndianess( (XSalPrinterSetup*)pSetup );
	
	PPDParser* pParser = PPDParser::getParser( String( pSetup->cDriverName, gsl_getSystemTextEncoding() ) );

	/* set page size */
	{
		String aPaper;
		if( pParser )
		{
			int w, h, i;
			if( pSetup->ePaper != PAPER_USER )
			{
				aPaper = String::CreateFromAscii( XSalGetPaperName( pSetup->ePaper ) );
				if( ! pParser->getPaperDimension( aPaper, w, h ) )
				{
					for( i = 0; i < sizeof(aPaperTab)/sizeof(aPaperTab[0]); i++ )
					{
						if( aPaperTab[i].paper == pSetup->ePaper )
						{
							aPaper = pParser->matchPaper(
								(int)((double)aPaperTab[i].width / 35.277778 ),
								(int)((double)aPaperTab[i].height / 35.277778 ) );
							break;
						}
					}
				}
			}
			else
				aPaper = pParser->matchPaper(
					(int)( (double)pSetup->nPaperWidth  / 35.277778 ),
					(int)( (double)pSetup->nPaperHeight / 35.277778 )
					);

			if( ! aPaper.Len() )
				aPaper = pParser->getDefaultPaperDimension();

		}
		XpSetPageSize( printer, aPaper.Len() ?
					   ByteString( aPaper, RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer() :
					   "A4" );
	}
	
	/* set orientation */
	{
		int res;
		res = XpSetOrientation( printer, GetXpOrientation( (Orientation)pSetup->eOrientation ) );
	}

	/* set scale */
	{
		int res;
		res = XpSetScale( printer, (double)pSetup->nScale / 100.0 );
	}

	/* set copies */
	{
		int res;
		res = XpSetCopies( printer, pSetup->nCopies );
	}

	/* set postscript data */
	if ( pSetup->eDriver == DRV_PostScript )
	{
		int res;
		
		if ( (pSetup->nFlags & XSALPRINTERSETUP_FLAG_LEVEL) != XSALPRINTERSETUP_FLAG_LEVEL_DEFAULT )
		{
			res = XpSetPSLevel( printer, pSetup->nFlags & XSALPRINTERSETUP_FLAG_LEVEL );
		}
		
		if ( !(pSetup->nFlags & XSALPRINTERSETUP_FLAG_COLOR_DEFAULT) )
		{	
			res = XpSetPSColor( printer, pSetup->nFlags & XSALPRINTERSETUP_FLAG_COLOR );
		}
	}

	/* set bitmap compression */
	if ( !(pSetup->nFlags & XSALPRINTERSETUP_FLAG_COMPRESS_DEFAULT) )
	{
		int res;
		res = XpSetBitmapCompression( printer, pSetup->nFlags & XSALPRINTERSETUP_FLAG_COMPRESS );
	}

	/* set correct paper tray */
	{
		String aPaperBin = pParser->getSlot( pSetup->nPaperBin );
		if( aPaperBin.Len() )
			XpSetPaperTray( (Display*)printer, ByteString( aPaperBin, gsl_getSystemTextEncoding() ).GetBuffer() );
	}
#endif
}

XSalPrinter* XSalOpenPrinter( const XSalPrinterSetup* pSetup, const String& rPrinterName, const String& rPrintFile )
{
#ifdef PRINTER_IMPLEMENTATION
	Display*		printer;


	ByteString aPrinterName( rPrinterName, RTL_TEXTENCODING_UTF8 );
	ByteString aCommand;
	ByteString aPort;
	ByteString aDriver;
	ByteString aType;

	int i,n;

	if( ! aPrinterName.Len() )
	{
		XpDefaults::getXpDefaults().SetGroup( "windows" );
		aPrinterName = XpDefaults::getXpDefaults().ReadKey( "device" );
		aPrinterName = aPrinterName.GetToken( 0, ',' );
	}
	XpDefaults::getXpDefaults().SetGroup( "devices" );
	ByteString aLine = XpDefaults::getXpDefaults().ReadKey( aPrinterName );
	aDriver = aLine.GetToken( 0, ' ' );
	aType = aLine.GetToken( 0, ',' ).GetToken( 1, ' ' );
	aPort = aLine.GetToken( 1, ',' );

	// #75105# check for invalid PPD name, if invalid fall back to SGENPRT
	if( ! PPDParser::getParser( String( aDriver, gsl_getSystemTextEncoding() ) ) )
		aDriver = "SGENPRT";

	if( aPort.Len() )
	{
		XpDefaults::getXpDefaults().SetGroup( "ports" );
		aCommand = XpDefaults::getXpDefaults().ReadKey( aPort );
	}
	if ( ! aCommand.Len() )
	{
		fprintf( stderr, "\ncould not open the printer %s\n\t\"%s\"\n\t\"%s\"\n",
				 aPrinterName.GetBuffer(), aPort.GetBuffer(), aType.GetBuffer() );
		fprintf( stderr, "Your printer installation might be wrong.\n" );
		fprintf( stderr, "Please check environment variable XPPATH.\n\n" );
		return None;
	}

	if( rPrintFile.Len() )
	{
		aCommand = "cat - > \"";
		aCommand += ByteString( rPrintFile, gsl_getSystemTextEncoding() );
		aCommand += '\"';
	}

	CorrectEndianess( (XSalPrinterSetup*)pSetup );

	printer = XpVaOpenPrinter( aDriver.GetBuffer(), aCommand.GetBuffer(), aType.GetBuffer(), NULL );

	if ( !printer )
	{
		fprintf( stderr, "\ncould not open the printer %s\n\t\"%s\"\n\t\"%s\"\n",
				 aPrinterName.GetBuffer(), aPort.GetBuffer(), aType.GetBuffer() );
		fprintf( stderr, "Your printer installation might be wrong.\n" );
		fprintf( stderr, "Please check environment variable XPPATH.\n\n" );
		return None;
	}

	strncpy( const_cast<char*>(pSetup->cDriverName), aDriver.GetBuffer(), sizeof( pSetup->cDriverName ) );
	strncpy( const_cast<char*>(pSetup->sPort), aPort.GetBuffer(), sizeof( pSetup->sPort ) );
	strncpy( const_cast<char*>(pSetup->sCompatCommand), aCommand.GetBuffer(), sizeof( pSetup->sCompatCommand ) );

	XpSetColorDepth( printer, 24 );
	XSalSetupPrinter( printer, pSetup );

	return (XSalPrinter*)printer;
#else
	return dummyPrinter;
#endif
}

void XSalGetPageInfo(
	const XSalPrinter* printer,
	const XSalPrinterSetup*	pSetup,
	XSalPageInfo* pPageInfo )
{
#ifdef PRINTER_IMPLEMENTATION
	char		section[256];  
	Margins	aMargin;
	int		type = 0, nAutoType, nManualType;

	assert( printer && !IsDisplay( printer ) );
	assert( pSetup );

	CorrectEndianess( (XSalPrinterSetup*)pSetup );

	/* initialize aMargin */
	/* our standard defaults */
	aMargin.left   = 750;
	aMargin.right  = 750;
	aMargin.top    = 760;
	aMargin.bottom = 760;

	PPDParser* pParser = PPDParser::getParser( String( pSetup->cDriverName, gsl_getSystemTextEncoding() ) );

	if( pParser )
		pParser->getMargins(
			pSetup->ePaper == PAPER_USER ?
			pParser->matchPaper(
				(int)((double)pSetup->nPaperWidth / 35.277778 ),
				(int)((double)pSetup->nPaperHeight / 35.277778 ) ) :
			String::CreateFromAscii( XSalGetPaperName( pSetup->ePaper ) ),
			aMargin.left, aMargin.right,
			aMargin.top, aMargin.bottom );

	// pt -> mm/100
	aMargin.left	= (int)(((double)aMargin.left)*35.277778);
	aMargin.right	= (int)(((double)aMargin.right)*35.277778);
	aMargin.top		= (int)(((double)aMargin.top)*35.277778);
	aMargin.bottom	= (int)(((double)aMargin.bottom)*35.277778);


	/* read section [Xprinter,Postscript/PCL] */
	sprintf( section, "%s,%s", "Xprinter", _DriverID[pSetup->eDriver] );
	type |= ReadMargin( &aMargin, section );

	/* read section [PrinterName,Postscript/PCL] */
	sprintf( section, "%s,%s", pSetup->cDriverName, _DriverID[pSetup->eDriver] );
	type |= ReadMargin( &aMargin, section );

	/* read section [PrinterName,PostScript/PCL,Port] */
	sprintf( section, "%s,%s,%s",
			 pSetup->cDriverName,
			 _DriverID[pSetup->eDriver],
			 pSetup->sPort );
	type |= ReadMargin( &aMargin, section );

	nAutoType   = type & 0xf0;
	nManualType = type & 0x0f;
  
	if ( nManualType ) /* manual */
	{
		double dpmmX; /* [dots per mm] resolution X	*/
		double dpmmY; /* [dots per mm] resolution Y	*/
	
		pPageInfo->nWidth	= XpDisplayWidth ( (Display*)printer, 0 );
		pPageInfo->nHeight	= XpDisplayHeight( (Display*)printer, 0 );
	
		dpmmX =  (double)pPageInfo->nWidth  / XpDisplayWidthMM ( (Display*)printer, 0 );
		dpmmY =  (double)pPageInfo->nHeight / XpDisplayHeightMM( (Display*)printer, 0 );
	
		pPageInfo->nResolutionX		= dpmmX * 254 / 10;
		pPageInfo->nResolutionY		= dpmmY * 254 / 10;
	
		if ( pSetup->eOrientation == ORIENTATION_PORTRAIT )
		{
			if( nManualType & LEFT_MARGIN )
				pPageInfo->nMarginLeft		= aMargin.left	* dpmmX / 100 + 0.5;
			if( nManualType & TOP_MARGIN )
				pPageInfo->nMarginTop		= aMargin.top	* dpmmY / 100 + 0.5;
			if( nManualType & RIGHT_MARGIN )
				pPageInfo->nMarginRight		= aMargin.right	* dpmmX / 100 + 0.5;
			if( nManualType & BOTTOM_MARGIN )
				pPageInfo->nMarginBottom	= aMargin.bottom* dpmmY / 100 + 0.5;
		}
		else
		{
			if( nManualType & LEFT_MARGIN )
				pPageInfo->nMarginLeft		= aMargin.top	* dpmmX / 100 + 0.5;
			if( nManualType & TOP_MARGIN )
				pPageInfo->nMarginTop		= aMargin.right	* dpmmY / 100 + 0.5;
			if( nManualType & RIGHT_MARGIN )
				pPageInfo->nMarginRight		= aMargin.bottom* dpmmX / 100 + 0.5;
			if( nManualType & BOTTOM_MARGIN )
				pPageInfo->nMarginBottom	= aMargin.left	* dpmmY / 100 + 0.5;
		}
		pPageInfo->nWidth		+= pPageInfo->nMarginLeft + pPageInfo->nMarginRight;
		pPageInfo->nHeight	+= pPageInfo->nMarginTop + pPageInfo->nMarginBottom;
	}

	if ( nAutoType && pSetup->eDriver == 0 )
	{
		if( pSetup->ePaper == PAPER_USER )
		{
			pPageInfo->nWidth = pSetup->nPaperWidth;
			pPageInfo->nHeight = pSetup->nPaperHeight;
		}
		else if( pParser )
		{
			pParser->getPaperDimension(
				String::CreateFromAscii( XSalGetPaperName( pSetup->ePaper ) ),
				pPageInfo->nWidth,
				pPageInfo->nHeight );
			// pt -> mm/100
			pPageInfo->nWidth	= (int)(((double)pPageInfo->nWidth)*35.277778);
			pPageInfo->nHeight	= (int)(((double)pPageInfo->nHeight)*35.277778);
		}

		if ( pSetup->eOrientation == ORIENTATION_PORTRAIT )
		{
			if( nAutoType & LEFT_MARGIN )
				pPageInfo->nMarginLeft	= aMargin.left;
			if( nAutoType & TOP_MARGIN )
				pPageInfo->nMarginTop	= aMargin.top;
			if( nAutoType & RIGHT_MARGIN )
				pPageInfo->nMarginRight	= aMargin.right;
			if( nAutoType & BOTTOM_MARGIN )
				pPageInfo->nMarginBottom= aMargin.bottom;
		}
		else
		{
			if( nAutoType & LEFT_MARGIN )
				pPageInfo->nMarginLeft	= aMargin.top;
			if( nAutoType & TOP_MARGIN )
				pPageInfo->nMarginTop	= aMargin.left;
			if( nAutoType & RIGHT_MARGIN )
				pPageInfo->nMarginRight	= aMargin.bottom;
			if( nAutoType & BOTTOM_MARGIN )
				pPageInfo->nMarginBottom= aMargin.right;
		}
		pPageInfo->nResolutionX	= pSetup->nResolutionX;
		pPageInfo->nResolutionY	= pSetup->nResolutionY;
	}
	pPageInfo->nMarginLeft = pPageInfo->nMarginLeft ? pPageInfo->nMarginLeft : 1;
	pPageInfo->nMarginRight = pPageInfo->nMarginRight ? pPageInfo->nMarginRight : 1;
	pPageInfo->nMarginTop = pPageInfo->nMarginTop ? pPageInfo->nMarginTop : 1;
	pPageInfo->nMarginBottom = pPageInfo->nMarginBottom ? pPageInfo->nMarginBottom : 1;
#endif
}

void XSalClosePrinter( XSalPrinter * printer )
{
#ifdef PRINTER_IMPLEMENTATION
	assert( printer && !IsDisplay( printer ) );

	XpClosePrinter( printer );
#endif
}

int XSalStartDoc( XSalPrinter * printer, const String& jobName )
{
#ifdef PRINTER_IMPLEMENTATION
	ByteString aJob( jobName, RTL_TEXTENCODING_ISO_8859_1 );
	assert( printer && !IsDisplay( printer ) );

	return XpStartDoc( printer, aJob.GetBuffer() );
#else
	return 1;
#endif
}

int XSalStartPage( XSalPrinter * printer)
{
#ifdef PRINTER_IMPLEMENTATION
	assert( printer && !IsDisplay( printer ) );

	return XpStartPage( printer );
#else
	return 1;
#endif
}

int XSalEndPage( XSalPrinter * printer)
{
#ifdef PRINTER_IMPLEMENTATION
	assert( printer && !IsDisplay( printer ) );

	return XpEndPage( printer );
#else
	return 1;
#endif
}

int XSalEndDoc( XSalPrinter * printer )
{
#ifdef PRINTER_IMPLEMENTATION
	assert( printer && !IsDisplay( printer ) );

	XpEndDoc( printer );
	return 0;
#else
	return 0;
#endif
}

int XSalAbortDoc( XSalPrinter * printer )
{
#ifdef PRINTER_IMPLEMENTATION
	assert( printer && !IsDisplay( printer ) );

	return XpAbortDoc( printer );
#else
	return 1;
#endif
}
