/*************************************************************************
 *
 *  $RCSfile: salprn.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 17:05:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/**
  this file implements the sal printer interface ( SalPrinter, SalInfoPrinter
  and some printer relevant methods of SalInstance and SalGraphicsData )

  as aunderlying library the printer features of XSal are used.

  the class MethodInvocation is used just for debugging purpose.

  the class ImplPrinterSetup is an XSalPrinterSetup and holds the system
  dependent information of the underlying printer (XSalPrinter).

  the class ImplSalPrinterData abstracts the XSalPrinter and holds it's
  XSalPrinterSetup. Changes are made by getting the XSalPrinterSetup,
  modifying it and then updating the XSalPrinter.
  An instance of ImplSalPrinterData is an element of SalPrinterData and
  of SalInfoPrinterData.
  
  The query methods of a SalInfoPrinter are implemented by extracting the
  information of its SalInfoPrinterData.ImplSalPrinterData or calling XSal
  query functions of SalInfoPrinterData.ImplSalPrinterData.XSalPrinter.

  The job methods of a SalPrinter are implemented by calling XSal
  printer job functions of SalPrinterData.ImplSalPrinterData.XSalPrinter.
 */

#define _SV_SALPRN_CXX

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>

#include <prex.h>
#include <postx.h>

#include <xsalprn.h>

#include <tools/urlobj.hxx>

#include <salunx.h>

#ifndef _SV_SALDATA_HXX
#include <saldata.hxx>
#endif
#ifndef _SV_SALDISP_HXX
#include <saldisp.hxx>
#endif
#ifndef _SV_SALINST_HXX
#include <salinst.hxx>
#endif
#ifndef _SV_SALGDI_HXX
#include <salgdi.hxx>
#endif
#ifndef _SV_SALPTYPE_HXX
#include <salptype.hxx>
#endif

#ifndef _SV_SALFRAME_HXX
#include <salframe.hxx>
#endif

#ifndef _SV_SALPRN_HXX
#define _INCL_SAL_SALPRN_IMP
#include <salprn.hxx>
#endif

#ifndef _SV_SALPRN_H
#include <salprn.h>
#endif

#ifndef _SV_PRINT_H
#include <print.h>
#endif
#ifndef _SV_JOBSET_H
#include <jobset.h>
#endif

#ifndef _VOS_TIMER_HXX
#include <vos/timer.hxx>
#endif

#ifndef _VOS_MUTEX_HXX
#include <vos/mutex.hxx>
#endif

#ifndef _VOS_PROFILE_HXX
#include <vos/profile.hxx>
#endif

#ifndef _SV_SVAPP_HXX
#include <svapp.hxx>
#endif

#ifndef _SV_PPDPARSER_HXX
#include <ppdparser.hxx>
#endif

#ifndef PRINTER_DUMMY
#define Font XLIB_Font
#define Region XLIB_Region
#include <xprinter/xp.h>
#undef Font
#undef Region
#endif

#include <salconfig.hxx>

#ifdef _USE_NAMESPACE
using namespace vos;
using namespace vcl_sal;
#endif


static const char* CVS_Id = "$Header: /cvs/oo/gsl/vcl/unx/source/gdi/salprn.cxx,v 1.1.1.1 2000/09/18 17:05:43 hr Exp $";

// =======================================================================

// the following string compare wrappers were found in the old 312 printer
// implementation check for need if compiled on AIX, SINIX, SCO

#if defined AIX
// tools.hxx: short svstrnicmp( const char* pStr1, const char* pStr2, USHORT nLen );
static inline strcasecmp( const char* pStr1, const char* pStr2 )
{ return svstrnicmp(pStr1,pStr2,0xffff); }
#elif defined SINIX || defined SCO
// tools.hxx: short svstrnicmp( const char* pStr1, const char* pStr2, USHORT nLen );
// wird leider auch in printer lib of libXSal.so benoetigt
extern "C" strcasecmp( const char* pStr1, const char* pStr2 )
{ return svstrnicmp(pStr1,pStr2,0xffff); }
extern "C" strncasecmp( const char* pStr1, const char* pStr2, int n )
{ return svstrnicmp(pStr1,pStr2,n); }
#endif

// =======================================================================

static const char aImplWindows[]	= "windows";	// section declaring default printer
static const char aImplDevice[]		= "device";		// default printer entry
static const char aImplPorts[]		= "ports";		// section declaring printer commands
static const char aImplDevices[]	= "devices";	// section declaring printers

// =======================================================================

static ByteString getPrinterPort( const String& rPrinterName )
{
	ByteString aRet;
	// determine printer from XSalDefaults
	int			nDevices = 0;

	Config& rXpDefaults = XpDefaults::getXpDefaults();
	rXpDefaults.SetGroup( aImplDevices );
	nDevices = rXpDefaults.GetKeyCount();
	// extract printer names and create info list  
	for( int nDevice = 0; nDevice < nDevices; nDevice++ )
	{		
		if( String( rXpDefaults.GetKeyName( nDevice ), RTL_TEXTENCODING_UTF8 ) == rPrinterName )
		{
			aRet = rXpDefaults.ReadKey( nDevice ).GetToken( 1, ',' );
			break;
		}
	}

	return aRet;
}

static ByteString getPrinterPPD( const String & rPrinterName )
{
	XpDefaults::getXpDefaults().SetGroup( aImplDevices );
	return XpDefaults::getXpDefaults().ReadKey( ByteString( rPrinterName, RTL_TEXTENCODING_UTF8 ) ).GetToken( 0, ' ' );
}

// =======================================================================

/**
  struct ImplPrinterSetup is a C++ XSalPrinterSetup abstraction.

  the Match method returns which kind of update is need to change
  from setup to the other.
  MATCH_ALL		means setups are equal. Nothing is to update.
  MATCH_DISPLAY means XSalPrinter and SalDisplay have to be updated.
  MATCH_DRIVER	means the XSalPrinter has to be updated, changes in
  				SalPrinter can't be updated. So a new instance of
				SalDisplay has to be constructed.
  MATCH_NONE	a new instance of the XSalPrinter and SalDisplay has
  				to be constructed.
 */

#define MATCH_ALL		0	/* nothing to be done					*/
#define MATCH_DISPLAY	1	/* update XSalPrinter update SalDisplay	*/
#define MATCH_DRIVER	2	/* update XSalPrinter renew SalDisplay	*/
#define MATCH_NONE		3	/* renew XSalPrinter and SalDisplay		*/

struct ImplPrinterSetup : public XSalPrinterSetup
{
	// ImplPrinterSetup should not have a constructor
	// because it must be (binary) streamable an is of variable length
	// because of appended bytes

	void			GetData( ImplJobSetup* pJobSetup, ULONG nGetDataFlags = 0 );
	
	inline int		Match( const ImplPrinterSetup* p ) const;
	BOOL			CheckDuplexCommand() const;

	ImplPrinterSetup* copy() const;

	String getPaperName( PPDParser* ) const;

private:
	ImplPrinterSetup( const ImplPrinterSetup& rSetup ) {}
};

// actually creates a ImplPrinterSetup
#ifdef _USE_NAMESPACE
namespace sal_PrinterExport {
#endif
void* createSetupFromPrinterName( const String& );
ImplPrinterSetup* createSetupFromJobsetup( const ImplJobSetup* );
static void correctPaperInSetup( ImplPrinterSetup* pSetup, const String& rPrinterName );
#ifdef _USE_NAMESPACE
}
using namespace sal_PrinterExport;
#endif

String ImplPrinterSetup::getPaperName( PPDParser* pParser ) const
{
	String aPaper;
	if( ePaper != PAPER_USER )
		aPaper = String( XSalGetPaperName( ePaper ), RTL_TEXTENCODING_ASCII_US );
	else
	{
		if( eOrientation == ORIENTATION_LANDSCAPE )
			aPaper = pParser->matchPaper(
				(int)((double)nPaperHeight / 35.277778 ),
				(int)((double)nPaperWidth  / 35.277778 ) );
		else
			aPaper = pParser->matchPaper(
				(int)((double)nPaperWidth  / 35.277778 ),
				(int)((double)nPaperHeight / 35.277778 ) );
	}
	return aPaper;
}

ImplPrinterSetup* ImplPrinterSetup::copy() const
{
	void* pRet = new char[ sizeof( XSalPrinterSetup ) + nTrailingBytes ];
	memcpy( pRet, this, sizeof( XSalPrinterSetup ) + nTrailingBytes );
	return (ImplPrinterSetup*)pRet;
}

void ImplPrinterSetup::GetData( ImplJobSetup* pJobSetup, ULONG nGetDataFlags )
{
	pJobSetup->mnSystem	= JOBSETUP_SYSTEM_UNIX;
	
	pJobSetup->maDriver	= String( cDriverName, gsl_getSystemTextEncoding() );
	pJobSetup->maDriver	+= ' ';
	pJobSetup->maDriver	+= String( XSalGetPrinterDriverName(eDriver), RTL_TEXTENCODING_ISO_8859_1 );
  
	if ( nGetDataFlags & SAL_JOBSET_PAPERSIZE )
	{
		pJobSetup->mePaperFormat	= ePaper;
		pJobSetup->mnPaperWidth		= nPaperWidth;
		pJobSetup->mnPaperHeight	= nPaperHeight;
	}
	if ( nGetDataFlags & SAL_JOBSET_PAPERBIN )
	{
		pJobSetup->mnPaperBin		= nPaperBin;
	}
	if ( nGetDataFlags & SAL_JOBSET_ORIENTATION )
	{
		pJobSetup->meOrientation	= (Orientation)eOrientation;
	}
	
	if( pJobSetup->mpDriverData != (unsigned char*)this )
	{
		if ( pJobSetup->mpDriverData )
			delete pJobSetup->mpDriverData;
		
		pJobSetup->mnDriverDataLen= sizeof( *this ) + nTrailingBytes;
		pJobSetup->mpDriverData	= (unsigned char *)copy();
	}
}

inline int ImplPrinterSetup::Match( const ImplPrinterSetup* p ) const
{
	if ( !p || p == this )
		return MATCH_ALL;
	
	if ( eDriver		!= p->eDriver		||
	   0 != strcmp( cDriverName, p->cDriverName ) )
		return MATCH_NONE;
	
	if ( nScale			== p->nScale )
	{
		if ( nCopies		== p->nCopies						&&
			 nPaperBin		== p->nPaperBin						&&
			 ePaper			== p->ePaper						&&
			 nPaperWidth	== p->nPaperWidth					&&
			 nPaperHeight	== p->nPaperHeight					&&
			 eOrientation	== p->eOrientation					&&
			 nFlags			== p->nFlags						&&
			 0 == strcmp( sPort, p->sPort ) )
	  return MATCH_ALL;
	else
	  return MATCH_DISPLAY;
  }

  return MATCH_DRIVER;
}

BOOL ImplPrinterSetup::CheckDuplexCommand() const
{
	return TRUE;
/*
	if( ! *sDuplexCommand )
		return TRUE;

	PPDParser* pParser = PPDParser::getParser( cDriverName );
	if( pParser )
	{
		for( int i = 0; i < pParser->getDuplexTypes(); i++ )
		{
			if( pParser->getDuplex( i ).Compare( sDuplexCommand ) == COMPARE_EQUAL )
				return TRUE;
		}
		if( pParser->getDefaultDuplexType().Compare( sDuplexCommand ) == COMPARE_EQUAL )
			return TRUE;
	}
	return FALSE;
*/
}

// =======================================================================


#define GetPrinterSetup(p)	((ImplPrinterSetup*)(p->mpDriverData))
#define _GetXDisplay()		(maPrinterData.mpImplData->GetXDisplay())



// =======================================================================


#define GLOBAL_SETUP_PAPERSIZE		1
#define GLOBAL_SETUP_PAPERTRAY		2
#define GLOBAL_SETUP_RESOLUTION		4

class ImplSalPrinterData
{
	friend class SalPrinter;
	
	String				maPrinterName;
	
	ImplPrinterSetup*	mpSetup;			// implementation relevant data
	XSalPrinter*		mpPrinter;			// result of OpenPrinter
	
	XSalPageInfo		maPageInfo;			// page metric info
	
	// recreate the following elemets if mpPrinter is recreated
	SalDisplay*		mpDisplay;
	SalGraphics*		mpGraphics;
	
	int				mnNewGlobalSetup;
	PPDContext		maPPDContext;

	inline void	Init()
		{
			mpSetup		= NULL;
			mpPrinter	= NULL;
			
			mpDisplay	= NULL;
			mpGraphics	= NULL;

			mnNewGlobalSetup = ~0;
		}
	
	BOOL			SetupXPrinter( const ImplPrinterSetup*	pSetup,
								   const String&			rPrinterName,
								   const String&			rPrintFile
								   );
	void			ResetXPrinter();
	
public:
	
	inline		ImplSalPrinterData()
		{ Init(); }
	inline		ImplSalPrinterData( const ImplSalPrinterData& rData )
		{
			Init();
			maPrinterName	=  rData.maPrinterName;
			mpSetup			=  rData.mpSetup->copy();
		}
	
	inline		~ImplSalPrinterData();
	
	inline void	Init( const SalPrinterQueueInfo*	pQueueInfo,
					  ImplJobSetup*					pJobSetup );
	void			SetupDisplay( const ImplPrinterSetup*	pSetup,
								  const String&				pPrinterName,
								  const String&				pPrintFile
								  );
	void			ResetDisplay();
	
	/**
	   Get data from info printer and store it in pJobSetup.
	   Use SAL_JOBSET_* of salptype.hxx as nGetDataFlags to mar which
	   have to be changed
	*/
	void			GetData(ImplJobSetup* pJobSetup, ULONG nGetDataFlags = ~0) const;
	void			SetData(ImplJobSetup* pJobSetup );
	
	XSalPrinter*	GetPrinter() const { return mpPrinter; }
	SalDisplay*	GetDisplay() const { return mpDisplay; }
	
	SalGraphics*	GetGraphics();
	void			ReleaseGraphics( SalGraphics* pGraphics = NULL );
	
	inline	int					GetPaperBinCount()			const
		{
			PPDParser* pParser =
				PPDParser::getParser( String( mpSetup->cDriverName, gsl_getSystemTextEncoding() ) );
			return pParser->getInputSlots();
		}
	inline	const String&			GetPaperBinName( int n )	const
		{
			PPDParser* pParser =
				PPDParser::getParser( String( mpSetup->cDriverName, gsl_getSystemTextEncoding() ) );
			return pParser->getSlot( n );
		}
	inline	const XSalPageInfo&	GetPageInfo()				const { return maPageInfo; }
	
	inline	Display*			GetXDisplay()	const { return GetDisplay()->GetDisplay(); }
	inline	XLIB_Window			GetDrawable()	const { return GetDisplay()->GetRootWindow(); }
	inline	SalColormap&		GetColormap()	const { return GetDisplay()->GetColormap(); }
};

// =======================================================================

inline ImplSalPrinterData::~ImplSalPrinterData()
{
  if ( mpPrinter )
	ResetXPrinter();

  if ( mpDisplay )
  {
	ResetDisplay();
  }

  delete mpGraphics;
  delete mpDisplay;
  delete mpSetup;

#ifdef DBG_UTIL
  mpGraphics	= (SalGraphics*)ILLEGAL_POINTER;
  mpDisplay		= (SalDisplay*)ILLEGAL_POINTER;
  mpSetup		= (ImplPrinterSetup*)ILLEGAL_POINTER;
#endif
}

inline void ImplSalPrinterData::Init(
	const SalPrinterQueueInfo*	pQueueInfo,
	ImplJobSetup* pJobSetup )
{
  // will be inserted into pJobSetup->mpDriverData
  ImplPrinterSetup *pSetup	= (ImplPrinterSetup*)
	  createSetupFromPrinterName( pQueueInfo->maPrinterName );

  maPrinterName	= pQueueInfo->maPrinterName;
  mpSetup = pSetup;

  if ( pJobSetup )
  {
	pJobSetup->mnSystem			= JOBSETUP_SYSTEM_UNIX;
	pJobSetup->maPrinterName	= pQueueInfo->maPrinterName;
	pJobSetup->maDriver			= pQueueInfo->maDriver;
	pJobSetup->meOrientation	= (Orientation)pSetup->eOrientation;
	pJobSetup->mnPaperBin		= pSetup->nPaperBin;
	pJobSetup->mePaperFormat	= pSetup->ePaper;
	if( pSetup->ePaper == PAPER_USER )
	{
		pJobSetup->mnPaperWidth		= pSetup->nPaperWidth;
		pJobSetup->mnPaperHeight	= pSetup->nPaperHeight;
	}
	else
	{
		pJobSetup->mnPaperWidth		= 0;
		pJobSetup->mnPaperHeight	= 0;
	}
	pJobSetup->mnDriverDataLen	= sizeof( ImplPrinterSetup ) + pSetup->nTrailingBytes;
	pJobSetup->mpDriverData		= (unsigned char*)(void*)pSetup->copy();
  }
}

BOOL ImplSalPrinterData::SetupXPrinter(
	const ImplPrinterSetup*	pSetup,
	const String& rPrinterName,
	const String& rPrintFile
	)
{
	int match = mpSetup->Match( pSetup );

	// change of port => MATCH_NONE
	if ( match != MATCH_NONE &&
		 pSetup && 0 != strcmp( pSetup->sPort, mpSetup->sPort ) )
		match = MATCH_NONE;

	// remember the setup
	// no setup means old setup
	if ( pSetup && pSetup != mpSetup )
	{
		delete mpSetup;
		mpSetup = pSetup->copy();
	}
	// remember new port
	// no command means old port
	if ( pSetup )
		strncpy( mpSetup->sPort, pSetup->sPort, sizeof( mpSetup->sPort ) );

	// check for no change / reset old printer
	if ( mpPrinter )
	{
		switch ( match )
		{
			case MATCH_NONE:
				ResetXPrinter();
				break;
			case MATCH_DRIVER:
			case MATCH_DISPLAY:
				break;
			case MATCH_ALL:
				return FALSE;
		}
	}
	
	BOOL ret;
	
	if ( mpPrinter )
	{
		XSalSetupPrinter( mpPrinter, mpSetup );
		ret = FALSE;
	}
	else
	{
		// init XSalPrinter
		static BOOL bInited = FALSE;
		if ( !bInited )
		{
			bInited = TRUE;
			// determine installation path
			INetURLObject aObject( GetSalData()->GetFileName(), INetURLObject::FSYS_DETECT );
			// /opt/Office40/bin/soffice.bin
			aObject.removeSegment();
			aObject.removeSegment();
			// /opt/Office40
			XSalPrinterInit( aObject.PathToFileName() );
		}

		mpPrinter = XSalOpenPrinter( mpSetup, rPrinterName, rPrintFile );
		
		ret = TRUE;		
	}
	
	XSalGetPageInfo( mpPrinter, mpSetup, &maPageInfo );
	

	return ret;
}

void ImplSalPrinterData::ResetXPrinter()
{
	ResetDisplay();

	if ( mpPrinter )
	{
		XSalClosePrinter( mpPrinter );
		mpPrinter	= NULL;
	} 
}

void ImplSalPrinterData::SetupDisplay(
	const ImplPrinterSetup*	pSetup,
	const String& rPrinterName,
	const String& rPrintFile
	)
{
	if( maPPDContext.getParser() )
	{
		if( mpSetup->getPaperName( maPPDContext.getParser() )
			!= pSetup->getPaperName( maPPDContext.getParser() )
			)
		{
			mnNewGlobalSetup |= GLOBAL_SETUP_PAPERSIZE;
		}
		if( mpSetup->nPaperHeight != pSetup->nPaperBin )
			mnNewGlobalSetup |= GLOBAL_SETUP_PAPERTRAY;
		if( mpSetup->nResolutionX != pSetup->nResolutionX ||
			mpSetup->nResolutionY != pSetup->nResolutionY )
			mnNewGlobalSetup |= GLOBAL_SETUP_RESOLUTION;
	}
	
	int match = mpSetup->Match( pSetup );
	
	// change of cmd => MATCH_NONE
	if ( match != MATCH_NONE &&
		 pSetup && 0 != strcmp( pSetup->sPort, mpSetup->sPort ) )
		match = MATCH_NONE;
	
	// check for nothing to be done
	if ( mpDisplay && match == MATCH_ALL )
		return;
	
	// we hope there is nothing to change
	BOOL bChanged = SetupXPrinter( pSetup, rPrinterName, rPrintFile );
	
	if ( !bChanged && mpDisplay && match <= MATCH_DISPLAY )
	{
		/* change data of SalDisplay */
	}
	else
	{
		ResetDisplay();
		mpDisplay	= new SalDisplay( GetPrinter(), NULL );
		mpGraphics	= NULL;
	}
}

void ImplSalPrinterData::ResetDisplay()
{
	if ( mpDisplay )
	{
		if ( mpGraphics )
		{
			mpGraphics->maGraphicsData.DeInit(); // ???
			delete mpGraphics;
			mpGraphics = NULL;
		}
		
		delete mpDisplay;
		mpDisplay = NULL;
	}
}

SalGraphics* ImplSalPrinterData::GetGraphics()
{
	if ( mpGraphics )
		return NULL;

	if ( !mpDisplay )
		SetupDisplay( NULL, maPrinterName, String() );
	
	mpGraphics = new SalGraphics;
	mpGraphics->maGraphicsData.Init( this );
	
	return mpGraphics;
}

void ImplSalPrinterData::ReleaseGraphics( SalGraphics* pGraphics )
{
	if ( mpGraphics )
	{
		if ( pGraphics && pGraphics != mpGraphics )
		{
			fprintf( stderr, "ImplSalPrinterData::ReleaseGraphics: wrong graphics\n" );
			abort();
		}
		delete mpGraphics;
		mpGraphics = NULL;
	}
}

void ImplSalPrinterData::GetData( ImplJobSetup* pJobSetup, ULONG nGetDataFlags ) const
{
	pJobSetup->maPrinterName	= maPrinterName;

	mpSetup->GetData( pJobSetup, nGetDataFlags );

	if ( nGetDataFlags & SAL_JOBSET_PAPERSIZE )
	{
		if ( pJobSetup->mePaperFormat == PAPER_USER )
		{
			pJobSetup->mnPaperWidth		= mpSetup ? mpSetup->nPaperWidth : 0;
			pJobSetup->mnPaperHeight	= mpSetup ? mpSetup->nPaperHeight : 0;
		}
		else
		{
			pJobSetup->mnPaperWidth	= 0;
			pJobSetup->mnPaperHeight	= 0;
		}
	}
}

static BOOL CheckJobSetupConsistency( ImplJobSetup* pJobSetup )
{
	BOOL bConsistent = FALSE;
	
	// determine printer from XSalDefaults
	int			nDevices;
	Config& rXpDefaults = XpDefaults::getXpDefaults();
	rXpDefaults.SetGroup( aImplDevices );
	nDevices = rXpDefaults.GetKeyCount();

	// extract printer names and create info list
	int nDevice;
	for( nDevice = 0; nDevice < nDevices; nDevice++ )
	{
		// get printer name
		String aName( rXpDefaults.GetKeyName( nDevice ), RTL_TEXTENCODING_UTF8 );
		if( aName == pJobSetup->maPrinterName )
		{
			ByteString aPort = getPrinterPort( pJobSetup->maPrinterName );
			
			if( aPort.CompareTo( ((ImplPrinterSetup*)pJobSetup->mpDriverData)->sPort ) == COMPARE_EQUAL )
			{
				bConsistent = TRUE;
				break;
			}
		}
	}
	
	return bConsistent;
}

void ImplSalPrinterData::SetData( ImplJobSetup* pJobSetup )
{
	if( pJobSetup->mpDriverData == (unsigned char*)mpSetup )
		return;

	BOOL bCopyOwnSetup = FALSE;

	if ( pJobSetup->mnSystem != JOBSETUP_SYSTEM_UNIX )
	{
#ifdef DEBUG
		fprintf( stderr, "ImplSalPrinterData::SetData not a JOBSETUP_SYSTEM_UNIX\n" );
#endif
		bCopyOwnSetup = TRUE;
	}
	else if ( pJobSetup->mpDriverData )
	{
		XSalCorrectEndianess( (XSalPrinterSetup*)pJobSetup->mpDriverData );
		if( ! CheckJobSetupConsistency( pJobSetup ) )
		{
#ifdef DEBUG
			fprintf( stderr, "ImplSalPrinterData::SetData inconsistent JobSetup\n" );
#endif
			bCopyOwnSetup = TRUE;
		}
	}

	if( bCopyOwnSetup )
	{
		if( pJobSetup->mpDriverData )
			delete pJobSetup->mpDriverData;

		pJobSetup->mnDriverDataLen	= sizeof( ImplPrinterSetup ) + mpSetup->nTrailingBytes;
		pJobSetup->mpDriverData		= (unsigned char*)mpSetup->copy();
	}

	ImplPrinterSetup *pSetup = GetPrinterSetup( pJobSetup );
	
	maPrinterName			= pJobSetup->maPrinterName;
	//  maDriver				= pJobSetup->maDriver;

	pSetup->nPaperBin		= pJobSetup->mnPaperBin;
	pSetup->eOrientation	= pJobSetup->meOrientation;
	pSetup->ePaper			= pJobSetup->mePaperFormat;
	pSetup->nPaperWidth		= pJobSetup->mnPaperWidth;
	pSetup->nPaperHeight	= pJobSetup->mnPaperHeight;

	correctPaperInSetup( pSetup, pJobSetup->maPrinterName );

	// set new pSetup 
	if ( mpSetup->Match( pSetup ) != MATCH_ALL )
	{
		if ( mpPrinter )
			SetupDisplay( pSetup, maPrinterName, String() );
	}
	
	if( mpSetup )
		delete mpSetup;
	mpSetup = pSetup->copy();
}


// =======================================================================


inline SalInfoPrinterData::SalInfoPrinterData()
{
	mpImplData	= NULL;
}

inline SalInfoPrinterData::~SalInfoPrinterData()
{
	delete mpImplData;

#ifdef DBG_UTIL
	mpImplData	= (ImplSalPrinterData*)ILLEGAL_POINTER;
#endif
}

inline void SalInfoPrinterData::Init(
	SalPrinterQueueInfo*	pQueueInfo,
	ImplJobSetup*			pJobSetup )
{
	mpImplData	= new ImplSalPrinterData();
	mpImplData->Init( pQueueInfo, pJobSetup );
}

// =======================================================================

inline SalPrinterData::SalPrinterData()
{
	mpImplData	= NULL;
	
	mbAbort		= FALSE;
	
	SalData* pSalData = GetSalData();
}

inline SalPrinterData::~SalPrinterData()
{
	delete mpImplData;

#ifdef DBG_UTIL
	mpImplData	= (ImplSalPrinterData*)ILLEGAL_POINTER;
#endif
}

inline void SalPrinterData::Init( SalInfoPrinter*	pInfoPrinter )
{
	mpImplData	= new ImplSalPrinterData( *pInfoPrinter->maPrinterData.mpImplData );
}


// =======================================================================


extern int XSalIsDummyPrinter;

SalInfoPrinter::SalInfoPrinter()
{
}

// -----------------------------------------------------------------------

SalInfoPrinter::~SalInfoPrinter()
{
}

// -----------------------------------------------------------------------

SalGraphics* SalInfoPrinter::GetGraphics()
{
	if ( XSalIsDummyPrinter )
		return NULL;

	return maPrinterData.mpImplData->GetGraphics();
}

// -----------------------------------------------------------------------

void SalInfoPrinter::ReleaseGraphics( SalGraphics* pGraphics )
{
	if ( XSalIsDummyPrinter )
		return;
	maPrinterData.mpImplData->ReleaseGraphics( pGraphics );
}

// -----------------------------------------------------------------------
// NETBSD has no RTLD_GLOBAL
#ifndef RTLD_GLOBAL
#define DLOPEN_MODE (RTLD_LAZY)
#else
#define DLOPEN_MODE (RTLD_GLOBAL | RTLD_LAZY)
#endif
#include <dlfcn.h>
#include <rtsname.hxx>
BOOL SalInfoPrinter::Setup( SalFrame* pFrame, ImplJobSetup* pJobSetup )
{
	if ( XSalIsDummyPrinter )
		return FALSE;
	
	if ( ! pJobSetup->mpDriverData )
	{
		pJobSetup->mpDriverData		=
			(unsigned char*)(void*)createSetupFromJobsetup( pJobSetup );
		pJobSetup->mnDriverDataLen	=
			sizeof( ImplPrinterSetup ) + ((ImplPrinterSetup*)pJobSetup->mpDriverData)->nTrailingBytes;
	}

	ImplPrinterSetup* pSetup	= GetPrinterSetup( pJobSetup );

	static void*		driverLib = NULL; 
	static void*(*pFunction)(void*) = NULL;
	const char* pErr = NULL;
	
	if( ! driverLib )
	{
		driverLib	= dlopen( _XSALSET_LIBNAME, DLOPEN_MODE );
		pErr		= dlerror();
		if ( !driverLib )
		{
			fprintf( stderr, "%s: when opening %s\n", pErr, _XSALSET_LIBNAME );
			return FALSE;
		}
	
		pFunction	= (void*(*)(void*))dlsym( driverLib, "Sal_SetupPrinterDriver" );
		pErr		= dlerror();
		if ( !pFunction )
		{
			fprintf( stderr, "%s: when getting Sal_SetupPrinterDriver\n", pErr );
			return FALSE;
		}
	}

	if( ! pFunction )
		return FALSE;
	
	pSetup = (ImplPrinterSetup*)pFunction( pSetup );
	
	if( pSetup )
	{
		delete pJobSetup->mpDriverData;
		pJobSetup->mpDriverData = (BYTE*)pSetup;
		pJobSetup->mnDriverDataLen = sizeof( XSalPrinterSetup ) + pSetup->nTrailingBytes;
		// copy everything to job setup
		pSetup->GetData( pJobSetup, ~0 );
	}
	
	return TRUE;
}
// -----------------------------------------------------------------------

// This function gets the driver data and puts it into pJobSetup
BOOL SalInfoPrinter::SetPrinterData( ImplJobSetup* pJobSetup )
{
	if ( XSalIsDummyPrinter )
		return FALSE;

	maPrinterData.mpImplData->SetData( pJobSetup );

	return TRUE;
}

// -----------------------------------------------------------------------

// This function merges the independ driver data
// and sets the new independ data in pJobSetup
// Only the data must be changed, where the bit
// in nGetDataFlags is set
BOOL SalInfoPrinter::SetData(
	ULONG nSetDataFlags,
	ImplJobSetup* pJobSetup )
{
	if ( XSalIsDummyPrinter )
		return FALSE;

	maPrinterData.mpImplData->GetData( pJobSetup, ~nSetDataFlags );
	maPrinterData.mpImplData->SetData( pJobSetup );

	return TRUE;
}

// -----------------------------------------------------------------------

void SalInfoPrinter::GetPageInfo(
	const ImplJobSetup* pImplJobSetup,
	long& rOutWidth, long& rOutHeight,
	long& rPageOffX, long& rPageOffY,
	long& rPageWidth, long& rPageHeight )
{
	if ( XSalIsDummyPrinter )
		return;

	/* Setup Display to get size data */
	maPrinterData.mpImplData->SetupDisplay( NULL, pImplJobSetup->maPrinterName, String() );
	
	XSalPageInfo aPageInfo = maPrinterData.mpImplData->GetPageInfo();
	
	rPageWidth	= aPageInfo.nWidth;
	rPageHeight	= aPageInfo.nHeight;
	rPageOffX	= aPageInfo.nMarginLeft;
	rPageOffY	= aPageInfo.nMarginTop;
	rOutWidth	= rPageWidth - rPageOffX - aPageInfo.nMarginRight;
	rOutHeight	= rPageHeight- rPageOffY - aPageInfo.nMarginBottom;
}

ULONG SalInfoPrinter::GetPaperBinCount( const ImplJobSetup* pJobSetup )
{
	if ( XSalIsDummyPrinter )
		return 0;

	// Setup Display to get paper bin data */
	maPrinterData.mpImplData->SetupDisplay( NULL, pJobSetup->maPrinterName, String() );

	return maPrinterData.mpImplData->GetPaperBinCount();
}

String SalInfoPrinter::GetPaperBinName( const ImplJobSetup* pJobSetup, ULONG nPaperBin )
{
	if ( XSalIsDummyPrinter )
		return String();

	// Setup Display to get paper bin data */
	maPrinterData.mpImplData->SetupDisplay( NULL, pJobSetup->maPrinterName, String() );
	
	if( nPaperBin == 0xffff )
		return String();

	if ( nPaperBin < maPrinterData.mpImplData->GetPaperBinCount() )
		return maPrinterData.mpImplData->GetPaperBinName( nPaperBin );
	else
		return String();
}

ULONG SalInfoPrinter::GetCapabilities( const ImplJobSetup* pSetupData, USHORT nType )
{
	switch( nType )
	{
		case PRINTER_CAPABILITIES_SUPPORTDIALOG:
			return 1;
		case PRINTER_CAPABILITIES_COPIES:
			return 0xffff;
		case PRINTER_CAPABILITIES_COLLATECOPIES:
			return 0;
		case PRINTER_CAPABILITIES_SETORIENTATION:
			return 1;
		case PRINTER_CAPABILITIES_SETPAPERBIN:
			return 1;
		case PRINTER_CAPABILITIES_SETPAPERSIZE:
			return 1;
		case PRINTER_CAPABILITIES_SETPAPER:
			return 0;
		default: break;
	};
	return 0;
}


// =======================================================================


SalPrinter::SalPrinter()
{
}

// -----------------------------------------------------------------------

SalPrinter::~SalPrinter()
{
}

// -----------------------------------------------------------------------


BOOL SalPrinter::StartJob(
	const XubString* pFileName,
	const XubString& rJobName,
	const XubString& rAppName,
	ULONG nCopies, BOOL bCollate,
	ImplJobSetup* pJobSetup )
{
	if ( XSalIsDummyPrinter )
		return FALSE;
	
	ImplPrinterSetup* pSetup = pJobSetup ? GetPrinterSetup( pJobSetup ) : NULL;
	
	if ( ! pSetup )
	{
		if( pJobSetup->mpDriverData )
			delete pJobSetup->mpDriverData;
		pSetup = createSetupFromJobsetup( pJobSetup );
		pJobSetup->mpDriverData = (unsigned char*)pSetup;
		pJobSetup->mnDriverDataLen = sizeof( ImplPrinterSetup ) + pSetup->nTrailingBytes;
	}
	
	pSetup->nCopies = nCopies;

	// init PPDContext from setup

	// always set parser
	maPrinterData.mpImplData->maPPDContext.setParser( PPDParser::getParser( String( pSetup->cDriverName, gsl_getSystemTextEncoding() ) ) );
	if( pSetup->nTrailingBytes && pSetup->nVersion >= 3 )
	{
		maPrinterData.mpImplData->maPPDContext.rebuildFromStreamBuffer( ((char*)pSetup) + sizeof( XSalPrinterSetup ), pSetup->nTrailingBytes );
	}

	// set new printer
	maPrinterData.mpImplData->SetupDisplay( pSetup, pJobSetup->maPrinterName, pFileName ? *pFileName : String() );


	XpSetPrologCallback( _GetXDisplay(), (prolog_callback)xprinterPrologHandler, this );
	XpSetGlobalSetupCallback( _GetXDisplay(), (globalsetup_callback)xprinterGlobalSetupHandler, this );
	XpSetPageSetupCallback( _GetXDisplay(), (pagesetup_callback)xprinterPageSetupHandler, this );
	XSalStartDoc( _GetXDisplay(), rJobName ); 

	maPrinterData.mbJobStarted		= TRUE;
	maPrinterData.mnError			= 0;
	maPrinterData.mbAbort			= FALSE;
	maPrinterData.mbFirstPage		= TRUE;

	return TRUE;
}

// -----------------------------------------------------------------------

BOOL SalPrinter::EndJob()
{
  if ( XSalIsDummyPrinter )
	return FALSE;

  if ( !maPrinterData.mbJobStarted )
	return FALSE;

  if( !maPrinterData.mbAbort )
	  XSalEndDoc( _GetXDisplay() );

  maPrinterData.mbJobStarted	= FALSE;

  return TRUE;
}

// -----------------------------------------------------------------------

BOOL SalPrinter::AbortJob()
{
  if ( XSalIsDummyPrinter )
	return FALSE;

  if ( !maPrinterData.mbJobStarted )
	return FALSE;

  XSalAbortDoc( _GetXDisplay() ); 

  maPrinterData.mbAbort = TRUE;

  EndJob();

  return TRUE;
}

// -----------------------------------------------------------------------
SalGraphics* SalPrinter::StartPage( ImplJobSetup* pJobSetup, BOOL bNewJobData )
{
  if ( XSalIsDummyPrinter )
	return NULL;

  if ( pJobSetup )
  {
	ImplPrinterSetup* pSetup =  GetPrinterSetup( pJobSetup );

	BOOL bTempSetup = FALSE;

	if ( ! pSetup )
	{
	  pSetup = createSetupFromJobsetup( pJobSetup );
	  bTempSetup = TRUE;
	}
	
	// neuen Printer setzen
	maPrinterData.mpImplData->SetupDisplay( pSetup, pJobSetup->maPrinterName, String() );
	
	if ( bTempSetup )
	  delete pSetup;
  }

  XSalStartPage( _GetXDisplay() );

  maPrinterData.mbPageStarted = TRUE;
  maPrinterData.mbFirstPage = FALSE;

  return maPrinterData.mpImplData->GetGraphics();
}

// -----------------------------------------------------------------------

BOOL SalPrinter::EndPage()
{
  if ( XSalIsDummyPrinter )
	return FALSE;

  XSalEndPage( _GetXDisplay() );

  maPrinterData.mbPageStarted = FALSE;

  maPrinterData.mpImplData->ReleaseGraphics();

  return TRUE;
}

// -----------------------------------------------------------------------

ULONG SalPrinter::GetErrorCode()
{
  if ( XSalIsDummyPrinter )
	return 0;

  return maPrinterData.mnError;
}


// =======================================================================

BOOL isXprinterInstalled()
{
	static BOOL bInstalled = FALSE;
	static BOOL bChecked = FALSE;

	if( ! bChecked )
	{
		// check for XPPATH, SGENPRT and psstd.fonts here
		bChecked = TRUE;
		const char* pEnv = getenv( "XPPATH" );
		if( pEnv )
		{
			ByteString aFile( pEnv );
			aFile += "/psstd.fonts";
			if( ! access( aFile.GetBuffer(), F_OK ) )
			{
				aFile = pEnv;
				aFile += "/ppds/SGENPRT.PS";
				if( ! access( aFile.GetBuffer(), F_OK ) )
				{
					Display* pPrinter = XpVaOpenPrinter( "SGENPRT", "lpr", "PostScript", NULL );
					if( pPrinter )
					{
						XpClosePrinter( pPrinter );
						bInstalled = TRUE;
					}
				}
			}
		}
	}

	return bInstalled;
}

// -----------------------------------------------------------------------

SalInfoPrinter* SalInstance::CreateInfoPrinter( SalPrinterQueueInfo*	pQueueInfo,
													  ImplJobSetup*			pSetup )
{
	if( ! isXprinterInstalled() )
		return NULL;

	// create and initialize SalInfoPrinter
	SalInfoPrinter* pPrinter = new SalInfoPrinter;
	pPrinter->maPrinterData.Init( pQueueInfo, pSetup );
	
	return pPrinter;
}

void SalInstance::DestroyInfoPrinter( SalInfoPrinter* pPrinter )
{
	delete pPrinter;
}

// -----------------------------------------------------------------------

SalPrinter* SalInstance::CreatePrinter( SalInfoPrinter* pInfoPrinter )
{
	if( ! isXprinterInstalled() )
		return NULL;
	
	// create and initialize SalPrinter
	SalPrinter* pPrinter = new SalPrinter;
	pPrinter->maPrinterData.Init( pInfoPrinter );
	
	return pPrinter;
}

void SalInstance::DestroyPrinter( SalPrinter* pPrinter )
{
	delete pPrinter;
}

// -----------------------------------------------------------------------

void SalInstance::GetPrinterQueueInfo( ImplPrnQueueList* pList )
{
	if( ! isXprinterInstalled() )
		return;
	
	// determine printer from XSalDefaults
	Config& rXpDefaults = XpDefaults::getXpDefaults();
	rXpDefaults.SetGroup( aImplDevices );
	int	nDevices = rXpDefaults.GetKeyCount();

	// extract printer names and create info list  
	for( int nDevice = 0; nDevice < nDevices; nDevice++ )
	{
		rXpDefaults.SetGroup( aImplDevices );
		String aName( rXpDefaults.GetKeyName( nDevice ), RTL_TEXTENCODING_UTF8 );
		String aValue( rXpDefaults.ReadKey( nDevice ), gsl_getSystemTextEncoding() );
		String aPort( aValue.GetToken( 1, ',' ) );
		String aPPD( aValue.GetToken( 0, ' ' ) );
		String aDriver( aValue.GetToken( 0, ',' ) );
		String aType( aValue.GetToken( 1, ' ' ).GetToken( 0, ',' ) );

		// #75105# check for invalid PPD name, if invalid fall back to SGENPRT
		if( ! PPDParser::getParser( aPPD ) )
			aPPD = String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "SGENPRT" ) );

		// get comment
		String aSection( aPPD );
		aSection.AppendAscii( ",PostScript," );
		aSection += aPort;
		rXpDefaults.SetGroup( ByteString( aSection, gsl_getSystemTextEncoding() ) );
		String aComment( rXpDefaults.ReadKey( "Comment" ), RTL_TEXTENCODING_UTF8 );

		// Neuen Eintrag anlegen
		SalPrinterQueueInfo* pInfo = new SalPrinterQueueInfo;
		pInfo->maPrinterName	= aName;
		pInfo->maDriver			= aDriver;
		pInfo->maLocation		= aPort;
		pInfo->maComment      	= aComment;
		pInfo->mpSysData		= NULL;
		pList->Add( pInfo );
	}
}

// -----------------------------------------------------------------------

void SalInstance::GetPrinterQueueState( SalPrinterQueueInfo* pInfo )
{
	if( ! isXprinterInstalled() )
		return;
}

// -----------------------------------------------------------------------

void SalInstance::DeletePrinterQueueInfo( SalPrinterQueueInfo* pInfo )
{
	delete pInfo;
}

// -----------------------------------------------------------------------

XubString SalInstance::GetDefaultPrinter()
{
	if( ! isXprinterInstalled() )
		return String();

	// Default-Printer-String aus Xpdefaults holen
	XpDefaults::getXpDefaults().SetGroup( aImplWindows );
	String aPrinterName( XpDefaults::getXpDefaults().ReadKey( aImplDevice ), RTL_TEXTENCODING_UTF8 );
	return aPrinterName;
}

// =======================================================================


void SalGraphicsData::Init( ImplSalPrinterData *pPrinter )
{
  xColormap_	= &(pPrinter->GetColormap());
  hDrawable_	= pPrinter->GetDrawable();
  
  bPrinter_		= TRUE;

  nPenPixel_	= GetPixel( nPenColor_ );
  nTextPixel_	= GetPixel( nTextColor_ );
  nBrushPixel_	= GetPixel( nBrushColor_ );
}

// --------------------------------------------------------

inline void writeFeature( FILE* fp, PPDKey* pKey, PPDValue* pValue  )
{
	fprintf( fp, "%%%%BeginFeature: *%s %s\n%s\n%%%%EndFeature\n",
			 ByteString( pKey->getKey(), RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer(),
			 ByteString( pValue->maOption, RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer(),
			 ByteString( pValue->maValue, RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer() );
}

void SalPrinter::xprinterPrologHandler( void* fp, void* pData )
{
	if( pData )
	{
		SalPrinter* pPrinter = (SalPrinter*)pData;
		SalPrinterData* pData = &pPrinter->maPrinterData;
		PPDContext& rContext = pData->mpImplData->maPPDContext;
		
		if( pData->mpImplData->maPPDContext.getParser() )
		{
			for( int i = 0; i < rContext.countValuesModified(); i++ )
			{
				PPDKey* pKey = rContext.getModifiedKey( i );
				if( pKey											&&
					! pKey->getKey().EqualsAscii( "ManualFeed" )	&&
					! pKey->getKey().EqualsAscii( "PageRegion" )	&&
					! pKey->getKey().EqualsAscii( "PageSize" )		&&
					! pKey->getKey().EqualsAscii( "InputSlot" )		&&
					! pKey->getKey().EqualsAscii( "Resolution" )	&&
					pKey->getSetupType() == PPDKey::Prolog
					)
				{
					PPDValue* pValue = rContext.getValue( pKey );
					if( pValue && pValue->meType == eInvocation && pValue->maValue.Len() )
						writeFeature( (FILE*)fp, pKey, pValue );
				}
			}
		}
	}
}

void SalPrinter::xprinterGlobalSetupHandler( void* fp, void* pData )
{
	fprintf( (FILE*)fp, "%%%%BeginSetup\n%%\n" );
	// setup is needed at start of every page by e.g.
	// Lexmark Optra. Does not work without a single line
	// to parse (and be it a comment) either
	if( pData )
	{
		SalPrinter* pPrinter = (SalPrinter*)pData;
		SalPrinterData* pData = &pPrinter->maPrinterData;
		ImplPrinterSetup* pPrinterSetup = pData->mpImplData->mpSetup;
		PPDContext& rContext = pData->mpImplData->maPPDContext;

		if( pData->mbFirstPage )
			pData->mpImplData->mnNewGlobalSetup = ~0;

		if( pData->mpImplData->mnNewGlobalSetup && rContext.getParser() )
		{
			PPDParser* pParser = rContext.getParser();
			// emit features in a ordered way (taken from xprinter)
			
			// set in this order:
			// Resolution, InputSlot, [PageRegion|PageSize]
			PPDValue* pValue	= NULL;
			PPDKey* pKey		= NULL;
			if(( pData->mpImplData->mnNewGlobalSetup & GLOBAL_SETUP_RESOLUTION )	&&
				( pKey = pParser->getKey( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "Resolution" ) ) ) ) &&
				( pValue = rContext.getValue( pKey ) )
				)
			{
				writeFeature( (FILE*)fp, pKey, pValue );
			}

			BOOL bSetInputSlot = FALSE;
			if( pData->mpImplData->mnNewGlobalSetup & GLOBAL_SETUP_PAPERTRAY )
			{
				if(
					( pKey = pParser->getKey( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "ManualFeed" ) ) ) )		&&
					( pValue = rContext.getValue( pKey ) )			&&
					pValue != pKey->getDefaultValue()
					)
				{
					// set manual feed
					bSetInputSlot = TRUE;
					writeFeature( (FILE*)fp, pKey, pValue );
				}
				else if(
					( pKey = pParser->getKey( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "InputSlot" ) ) ) )					&&
					( pValue = pKey->getValue( pParser->getSlot( pPrinterSetup->nPaperBin ) ) )
					)
				{
					// set input slot
					bSetInputSlot = TRUE;
					writeFeature( (FILE*)fp, pKey, pValue );
				}
			}

			// get paper name
			String aPaper = pPrinterSetup->getPaperName( pParser );
			if( ( pData->mpImplData->mnNewGlobalSetup & GLOBAL_SETUP_PAPERSIZE ) && aPaper.Len() )
			{
				if( bSetInputSlot )
					pKey = pParser->getKey( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "PageRegion" ) ) );
				else
					pKey = pParser->getKey( String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "PageSize" ) ) );
				pValue = pKey->getValue( aPaper );
				if( pValue )
					writeFeature( (FILE*)fp, pKey, pValue );
			}
			
			if( pData->mbFirstPage )
			{
				for( int i = 0; i < rContext.countValuesModified(); i++ )
				{
					PPDKey* pKey = rContext.getModifiedKey( i );
					if( pKey							&&
						! pKey->getKey().EqualsAscii( "ManualFeed" )	&&
						! pKey->getKey().EqualsAscii( "PageRegion" )	&&
						! pKey->getKey().EqualsAscii( "PageSize" )		&&
						! pKey->getKey().EqualsAscii( "InputSlot" )		&&
						! pKey->getKey().EqualsAscii( "Resolution" )	&&
						( pKey->getSetupType() == PPDKey::DocumentSetup		||
						  pKey->getSetupType() == PPDKey::AnySetup )
						)
					{
						PPDValue* pValue = rContext.getValue( pKey );
						if( pValue && pValue->meType == eInvocation && pValue->maValue.Len() )
							writeFeature( (FILE*)fp, pKey, pValue );
					}
				}
			}
			pData->mpImplData->mnNewGlobalSetup = 0;
		}
	}
	fprintf( (FILE*)fp, "%%%%EndSetup\n" );
}

void SalPrinter::xprinterPageSetupHandler( void* fp, int nPage, void* pData )
{
	if( pData )
	{
				 
		SalPrinter* pPrinter = (SalPrinter*)pData;
		SalPrinterData* pData = &pPrinter->maPrinterData;
		ImplPrinterSetup* pPrinterSetup = pData->mpImplData->mpSetup;
		PPDContext& rContext = pData->mpImplData->maPPDContext;
		PPDParser* pParser = rContext.getParser();

		if( ! pParser || ! pData || ! pPrinterSetup )
			return;

		fprintf( (FILE*)fp, "%%%%Page: %d %d\n", nPage, nPage );
		// header
		fprintf( (FILE*)fp, "%%%%BeginPageSetup\n/#copies %d def\nsv\n",
				 pPrinterSetup->nCopies );
		// get paper name
		String aPaper = pPrinterSetup->getPaperName( pParser );

		// calculate translation

		int nMarginLeft		= 0,
			nMarginRight	= 0,
			nMarginUpper	= 0,
			nMarginLower	= 0;
		int nWidth			= 0,
			nHeight			= 0;

		pParser->getMargins( aPaper, nMarginLeft, nMarginRight, nMarginUpper, nMarginLower );
		pParser->getPaperDimension( aPaper, nWidth, nHeight );

		int nTranslateX, nTranslateY;
		nTranslateX = 10000 * nMarginLeft / pPrinterSetup->nScale;
		if( pPrinterSetup->eOrientation == ORIENTATION_LANDSCAPE )
		{
			fprintf( (FILE*)fp, "90 rotate\n" );
			nTranslateY = 10000 * nMarginLower / pPrinterSetup->nScale;
		}
		else
		{
			nTranslateY = 10000 * ( nHeight - nMarginLower ) / pPrinterSetup->nScale;
		}
		fprintf( (FILE*)fp,
				 "%d.%d -%d.%d sl\n"
				 "%d.%d %c%d.%d tl\n",
				 pPrinterSetup->nScale / 100, pPrinterSetup->nScale % 100,
				 pPrinterSetup->nScale / 100, pPrinterSetup->nScale % 100,
				 nTranslateX / 100, nTranslateX % 100,
				 pPrinterSetup->eOrientation == ORIENTATION_LANDSCAPE
				 ? ' ' : '-',
				 nTranslateY / 100, nTranslateY % 100
				 );

		for( int i = 0; i < rContext.countValuesModified(); i++ )
		{
			PPDKey* pKey = rContext.getModifiedKey( i );
			if( pKey							&&
				! pKey->getKey().EqualsAscii( "ManualFeed" )	&&
				! pKey->getKey().EqualsAscii( "PageRegion" )	&&
				! pKey->getKey().EqualsAscii( "PageSize" )		&&
				! pKey->getKey().EqualsAscii( "InputSlot" )		&&
				! pKey->getKey().EqualsAscii( "Resolution" )	&&
				pKey->getSetupType() == PPDKey::PageSetup
				)
			{
				PPDValue* pValue = rContext.getValue( pKey );
				if( pValue && pValue->meType == eInvocation && pValue->maValue.Len() )
					writeFeature( (FILE*)fp, pKey, pValue );
			}
		}
		fprintf( (FILE*)fp, "%%%%EndPageSetup\n" );
	}
}

// =======================================================================

#ifdef _USE_NAMESPACE
namespace sal_PrinterExport {
#endif

static void correctPaperInSetup( ImplPrinterSetup* pSetup, const String& rPrinterName )
{
	// check wether printer knows this format
	String aPaper( XSalGetPaperName( pSetup->ePaper ), RTL_TEXTENCODING_ISO_8859_1 );
	String aPPD( getPrinterPPD( rPrinterName ), gsl_getSystemTextEncoding() );
	PPDParser* pParser = NULL;
	if( ! ( pParser = PPDParser::getParser( aPPD ) ) )
	{
		aPPD = String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM( "SGENPRT" ) );
		pParser = PPDParser::getParser( aPPD );
	}
	if( pParser )
	{
		int w = 0, h = 0;
		if( ! ( pParser->getPaperDimension( aPaper, w, h ) && w && h ) )
		{
			// see #75638#
			// the paper does not exist, replace it by something else
			// first search for similar paper names
			// (e.g. apple laser writer has paper formats A4.4Meg instead of A4
			int i;
			for( i = 0; i < pParser->getPaperDimensions(); i++ )
			{
				String aMatch = pParser->getPaperDimension( i );
				if( aMatch.CompareIgnoreCaseToAscii( aPaper, aPaper.Len() ) == COMPARE_EQUAL )
				{
					if( pParser->getPaperDimension( aMatch, w, h ) )
					{
						pSetup->ePaper = PAPER_USER;
						pSetup->nPaperWidth = (int)((double)w * 35.277778);
						pSetup->nPaperHeight = (int)((double)h * 35.277778);

#ifdef DEBUG
						fprintf( stderr, "replacing nonexistent paper \"%s\" by \"%s\" (%d,%d)\n",
								 ByteString( aPaper, RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer(),
								 ByteString( aMatch, RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer(), w, h );
#endif
						return;
					}
				}
			}

			// no replacement found, fall back to default paper
			w = h = 0;
			pParser->getDefaultPaperDimension( w, h );
			if( w & h )
			{
				pSetup->ePaper = PAPER_USER;
				pSetup->nPaperWidth = (int)((double)w * 35.277778);
				pSetup->nPaperHeight = (int)((double)h * 35.277778);
				
#ifdef DEBUG
						fprintf( stderr, "replacing nonexistent paper \"%s\" by \"%s\" (%d,%d)\n",
								 ByteString( aPaper, RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer(),
								 ByteString( pParser->getDefaultPaperDimension(), RTL_TEXTENCODING_ISO_8859_1 ).GetBuffer(), w, h );
#endif
			}
		}
	}
}

void* createSetupFromPrinterName( const String& rPrinterName )
{
	// get PPD and port
	ByteString aPort = getPrinterPort( rPrinterName );
	ByteString aPPD  = getPrinterPPD( rPrinterName );
	// #75105# check for invalid PPD name, if invalid fall back to SGENPRT
	if( ! PPDParser::getParser( String( aPPD, gsl_getSystemTextEncoding() ) ) )
		aPPD = "SGENPRT";

	ByteString aSection = aPPD;
	aSection += ",PostScript,";
	aSection += aPort;

	ImplPrinterSetup* pNewSetup = (ImplPrinterSetup*)XpDefaults::getXpDefaults().readSetupSection( aSection );
	correctPaperInSetup( pNewSetup, rPrinterName );

	return pNewSetup;
}

ImplPrinterSetup* createSetupFromJobsetup( const ImplJobSetup* pJobSetup )
{
	ImplPrinterSetup* pNewSetup = (ImplPrinterSetup*)createSetupFromPrinterName( pJobSetup->maPrinterName );
	ImplPrinterSetup* pOldSetup = (ImplPrinterSetup*)pJobSetup->mpDriverData;

	strncpy( pNewSetup->sPort,
			 getPrinterPort( pJobSetup->maPrinterName ).GetBuffer(),
			 sizeof( pNewSetup->sPort ) );
	strncpy( pNewSetup->cDriverName,
			 ByteString( pJobSetup->maDriver.GetToken(0, ' '), gsl_getSystemTextEncoding() ).GetBuffer(),
			 sizeof( pNewSetup->cDriverName ) );
	// for compatability
	strncpy( pNewSetup->sCompatCommand,
			 pOldSetup ? pOldSetup->sCompatCommand : "lpr",
			 sizeof( pNewSetup->sCompatCommand ) );
	
	pNewSetup->ePaper		= pJobSetup->mePaperFormat;
	if( pNewSetup->ePaper == PAPER_USER )
	{
		pNewSetup->nPaperWidth	= pJobSetup->mnPaperWidth;
		pNewSetup->nPaperHeight	= pJobSetup->mnPaperHeight;
	}
	else
	{
		pNewSetup->nPaperWidth	= 0;
		pNewSetup->nPaperHeight	= 0;
	}
	pNewSetup->eOrientation	= pJobSetup->meOrientation;
	pNewSetup->nPaperBin	= pJobSetup->mnPaperBin;

	correctPaperInSetup( pNewSetup, pJobSetup->maPrinterName );

	return pNewSetup;
}

#ifdef _USE_NAMESPACE
}
#endif

// =======================================================================

// Send a DataChanged Event to every frame, when the printer Configuration
// changed. In other words, when the XpDefaults file has changed.
// to do this check xpdefaults mtime every n secons from a VOS OTimer
// thread and post a userevent to the main thread if that happened.

class ImplPrinterChangedEvent
{
public:
	DECL_STATIC_LINK( ImplPrinterChangedEvent, CallEvent, void* pEvent );
};

IMPL_STATIC_LINK( ImplPrinterChangedEvent, CallEvent, void*, pEvent )
{
	// post data changed event from here
	SalFrame* pFrame = GetSalData()->pFirstFrame_;
	while( pFrame )
	{
		pFrame->maFrameData.Call( SALEVENT_PRINTERCHANGED, NULL );
		pFrame = pFrame->maFrameData.GetNextFrame();
	}
	
	delete (ApplicationEvent*)pEvent;
	return 0;	
}

class PrinterChangedTimer : public OTimer
{
public:
	PrinterChangedTimer();
	~PrinterChangedTimer();
	
	virtual void onShot();
};

PrinterChangedTimer::PrinterChangedTimer() :
		OTimer( TTimeValue( 3, 0 ),
				TTimeValue( 3, 0 ) )
{
}

PrinterChangedTimer::~PrinterChangedTimer()
{
}

void PrinterChangedTimer::onShot()
{
	static time_t nLastModified = 0;
	struct stat aStat;

	ByteString aProfileName( XpDefaults::getProfileName(), gsl_getSystemTextEncoding() );

	if( nLastModified == 0 )
	{
		if( ! stat( aProfileName.GetBuffer(), &aStat ) )
			nLastModified = aStat.st_mtime;
		return;
	}
	
	if( ! stat( aProfileName.GetBuffer(), &aStat ) )
	{
		if( nLastModified != aStat.st_mtime )
		{
			nLastModified = aStat.st_mtime;
			XpDefaults::getXpDefaults().Update();
			// post to main thread that printers have changed
			String aEmpty;
			ApplicationEvent* pEvent = new
				ApplicationEvent( aEmpty, aEmpty, "PRINTERCHANGED", aEmpty );
			Application::PostUserEvent( STATIC_LINK( NULL, ImplPrinterChangedEvent, CallEvent ), pEvent );
		}
	}
}

static PrinterChangedTimer* pPrinterChangedTimer = 0;

void StartPrinterListening()
{
	if( pPrinterChangedTimer == NULL && isXprinterInstalled() )
	{
		pPrinterChangedTimer = new PrinterChangedTimer;
		pPrinterChangedTimer->acquire();
		pPrinterChangedTimer->start();
	}
}

void StopPrinterListening()
{
	if( pPrinterChangedTimer )
	{
		pPrinterChangedTimer->stop();
		pPrinterChangedTimer->release();
		pPrinterChangedTimer = NULL;
	}
}
