/*************************************************************************
 *
 *  $RCSfile: request.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _OSL_DEMONMGR_H_
#       include <osl/demonmgr.h>
#endif
#ifndef _REQUEST_HXX_
#		include "request.hxx"
#endif


/*--------------------------------------------------------------------------*
 * set / get information of a daemon										*
 *--------------------------------------------------------------------------*/


void 
get_info ( char *name, FILE *outfile, oslDaemonInfo *info)
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_Read );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (read)\n", name);
		return;
	}

	entity_handle = osl_openDaemonEntity ( mgr_handle, info->service_name );

	if ( !entity_handle )
	{
		fprintf (outfile, "%s: can't open entity \"%s\"\n", name, info->service_name);
		osl_closeDaemonManager ( mgr_handle );
		return;
	}

	if (osl_getDaemonInfo ( entity_handle, info ) )
	{
		fprintf (outfile, "%s: getting daemon-info\n", name );
		fprintf (outfile, "    service-name = %s\n", info->service_name );
		fprintf (outfile, "    display-name = %s\n", info->display_name );
		fprintf (outfile, "    dependencies = %s\n", info->dependencies );
		fprintf (outfile, "    path         = %s\n", info->path );
		fprintf (outfile, "    autostart    = %s\n", 
			info->startup_mode == osl_Daemon_Automatic ? "automatic" :
			info->startup_mode == osl_Daemon_OnDemand  ? "on-demand" : 
			"unknown" );
	}
	else
	{
		fprintf (outfile, "%s: can't get info for \"%s\"\n", 
				 name, info->service_name);
	}

	osl_closeDaemonEntity  ( entity_handle );
	osl_closeDaemonManager ( mgr_handle );
}


void 
set_info ( char *name, FILE *outfile, oslDaemonInfo *info)
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	entity_handle = osl_openDaemonEntity ( mgr_handle, info->service_name );

	if ( !entity_handle )
	{
		fprintf (outfile, "%s: can't open entity \"%s\"\n", name, info->service_name);
		osl_closeDaemonManager ( mgr_handle );
		return;
	}

	if (osl_setDaemonInfo ( entity_handle, info ) )
	{
		fprintf (outfile, "%s: setting daemon-info\n", name );
		fprintf (outfile, "    service-name = %s\n", info->service_name );
		fprintf (outfile, "    display-name = %s\n", info->display_name );
		fprintf (outfile, "    dependencies = %s\n", info->dependencies );
		fprintf (outfile, "    path         = %s\n", info->path );
		fprintf (outfile, "    autostart    = %s\n", 
			info->startup_mode == osl_Daemon_Automatic ? "automatic" :
			info->startup_mode == osl_Daemon_OnDemand  ? "on-demand" : 
			"unknown" );
	}
	else
	{
		fprintf (outfile, "%s: can't set info for \"%s\"\n", 
				 name, info->service_name);
	}

	osl_closeDaemonEntity  ( entity_handle );
	osl_closeDaemonManager ( mgr_handle );
}


/*--------------------------------------------------------------------------*
 * install / uninstall a daemon 											*
 *--------------------------------------------------------------------------*/


void
daemon_install ( char *name, FILE *outfile, oslDaemonInfo *info)
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	entity_handle = osl_installDaemonEntity ( mgr_handle, info->service_name );

	if ( !entity_handle )
	{
		fprintf (outfile, "%s: can't install entity \"%s\"\n", name, info->service_name);
		osl_closeDaemonManager ( mgr_handle );
		return;
	}
	else
	{
		fprintf (outfile, "%s: successfully installed entity \"%s\"\n", name, info->service_name);
	}

	osl_closeDaemonEntity  ( entity_handle );
	osl_closeDaemonManager ( mgr_handle );
}


void
daemon_uninstall ( char *name, FILE *outfile, oslDaemonInfo *info)
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	entity_handle = osl_openDaemonEntity ( mgr_handle, info->service_name );

	if ( !entity_handle )
	{
		fprintf (outfile, "%s: can't open entity \"%s\"\n", name, info->service_name);
		osl_closeDaemonManager ( mgr_handle );
		return;
	}

	osl_uninstallDaemonEntity ( entity_handle );
	fprintf (outfile, "%s: uninstalled daemon %s\n", name, info->service_name );

	osl_closeDaemonManager ( mgr_handle );
}


/*--------------------------------------------------------------------------*
 * get a list of all daemons												*
 *--------------------------------------------------------------------------*/


void
enum_info ( char *name, FILE *outfile, oslDaemonInfo *info)
{
	osl_DaemonMgrHandle mgr_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	int i = 0;

	while ( osl_enumDaemonInfo ( mgr_handle, i, info ) )
	{
		fprintf (outfile, "%s: getting daemon-info [%i]\n", name, i );
		fprintf (outfile, "    service-name = %s\n", info->service_name );
		fprintf (outfile, "    display-name = %s\n", info->display_name );
		fprintf (outfile, "    dependencies = %s\n", info->dependencies );
		fprintf (outfile, "    path         = %s\n", info->path );
		fprintf (outfile, "    autostart    = %s\n", 
			info->startup_mode == osl_Daemon_Automatic ? "automatic" :
			info->startup_mode == osl_Daemon_OnDemand  ? "on-demand" : 
			"unknown" );
		fprintf (outfile, "\n");
		i++;
	}
	fprintf (outfile, "%s: got info for %i daemons\n", name, i );

	osl_closeDaemonManager ( mgr_handle );
}


/*--------------------------------------------------------------------------*
 * get / retrieve daemon specific properties								*
 *--------------------------------------------------------------------------*/


void 
daemon_proprequest ( char *name, FILE *outfile, oslDaemonInfo *info,
					 char *property_name )
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;
	char property_value[ 128 ];

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	entity_handle = osl_openDaemonEntity ( mgr_handle, info->service_name );

	if ( !entity_handle )
	{
		fprintf (outfile, "%s: can't open entity \"%s\"\n", name, info->service_name);
		osl_closeDaemonManager ( mgr_handle );
		return;
	}
	
	if ( osl_getDaemonProperty (entity_handle, property_name, 
								property_value, sizeof(property_value) ))
	{
		fprintf (outfile, "%s: got property \"%s\" = \"%s\"\n", 
				name, property_name, property_value );
	}
	else
	{
		fprintf (outfile, "%s: can't get property \"%s\" for \"%s\"\n", 
				 name, property_name, info->service_name);
	}

	osl_closeDaemonEntity  ( entity_handle );
	osl_closeDaemonManager ( mgr_handle );
}


void 
daemon_propset ( char *name, FILE *outfile, oslDaemonInfo *info,
				 char *property_name, char *property_value )
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	entity_handle = osl_openDaemonEntity ( mgr_handle, info->service_name );

	if ( !entity_handle )
	{
		fprintf (outfile, "%s: can't open entity \"%s\"\n", name, info->service_name);
		osl_closeDaemonManager ( mgr_handle );
		return;
	}
	
	if ( osl_setDaemonProperty (entity_handle, property_name, property_value ) )
	{
		fprintf (outfile, "%s: set property \"%s\" = \"%s\"\n", name, property_name, property_value ? property_value : "nil" );
	}
	else
	{
		fprintf (outfile, "%s: can't set property \"%s\" for \"%s\"\n", 
				 name, property_name, info->service_name);
	}

	osl_closeDaemonEntity  ( entity_handle );
	osl_closeDaemonManager ( mgr_handle );
}


/*--------------------------------------------------------------------------*
 * get / retrieve global properties 										*
 *--------------------------------------------------------------------------*/


void 
global_propset ( char *name, FILE *outfile, oslDaemonInfo *info,
				 char *property_name, char *property_value )
{
	osl_DaemonMgrHandle mgr_handle;
	osl_DaemonEntityHandle entity_handle;

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	if ( osl_setDaemonMgrProperty (mgr_handle, property_name, property_value ) )
	{
		fprintf (outfile, "%s: set global property \"%s\" = \"%s\"\n", 
				name, property_name, property_value ? property_value : "nil" );
	}
	else
	{
		fprintf (outfile, "%s: can't set global property \"%s\" for \"%s\"\n", 
				 name, property_name, info->service_name);
	}

	osl_closeDaemonManager ( mgr_handle );
}


void
global_proprequest ( char *name, FILE *outfile, oslDaemonInfo *info,
					 char *property_name )
{
	osl_DaemonMgrHandle mgr_handle;
	char property_value[ 128 ];

	mgr_handle = osl_openDaemonManager ( osl_DaemonMgr_ReadWrite );

	if ( !mgr_handle ) 
	{
		fprintf (outfile, "%s: can't open daemon-mgr (write)\n", name);
		return;
	}

	if ( osl_getDaemonMgrProperty (mgr_handle, property_name, 
						  		   property_value, sizeof(property_value) ))
	{
		fprintf (outfile, "%s: got global property \"%s\" = \"%s\"\n", 
				name, property_name, property_value );
	}
	else
	{
		fprintf (outfile, "%s: can't get global property \"%s\" for \"%s\"\n", 
				 name, property_name, info->service_name);
	}

	osl_closeDaemonManager ( mgr_handle );
}


