/*************************************************************************
 *
 *  $RCSfile: simpled.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#ifndef _RUN_HXX_
#	include "run.hxx"
#endif
#ifndef _REQUEST_HXX_
#	include "request.hxx"
#endif
#ifndef _CTRL_HXX_
#	include "ctrl.hxx"
#endif
#ifndef _OSL_DEMONMGR_H_
#	include <osl/demonmgr.h>
#endif
#ifndef _OSL_DAEMON_H_
#	include <osl/daemon.h>
#endif


char*
basename (const char *filename)
{
  char *p = strrchr (filename, '/');
  return p ? p + 1 : (char *) filename;
}


void 
usage ( char **argv )
{
	fprintf (stderr, "\n");	
	fprintf (stderr, "usage:\n");
	fprintf (stderr, "\n");	
	fprintf (stderr, "    %s [-h] [-o file] [-l] [-i] [-u] [-a] [-d name] \n", basename(argv[0]) );
	fprintf (stderr, "            [-n name] [-p path] [-r] [-s] [-R] [-S] [-N name] [-V value]\n" );
	fprintf (stderr, "            [-c start | stop | pause | continue | shutdown | request ]\n" );
	fprintf (stderr, "\n");
	fprintf (stderr, "    -h       : help, show this message\n");
	fprintf (stderr, "    -o file  : write all messages to \"file\" instead to \"stderr\"\n");
	fprintf (stderr, "    -l       : list available daemon info from server\n");
	fprintf (stderr, "    -i       : install, register itself to the daemon-manager\n");
	fprintf (stderr, "    -u       : uninstall\n");
	fprintf (stderr, "    -a       : use startup mode \"automatic\" in negotiation of daemon info\n");
	fprintf (stderr, "               instead of \"on-demand\"\n");
	fprintf (stderr, "    -d name  : use display-name \"name\" in negotiation of daemon info\n");
	fprintf (stderr, "               instead of \"%s\"\n", argv[0] );
	fprintf (stderr, "    -n name  : use service-name \"name\" in negotiation of daemon info\n");
	fprintf (stderr, "               instead of \"%s\"\n", basename(argv[0]) );
	fprintf (stderr, "    -p path  : use \"path\" for negotiation of daemon info\n");
	fprintf (stderr, "               instead of \"%s\"\n", argv[0] );
	fprintf (stderr, "    -N name -V value : use \"name=value\" for negotiation of \n");
	fprintf (stderr, "               properties, instead of \"iam=(null)\"\n" );
	fprintf (stderr, "    -r | -s  : request / specify the daemons information of itself\n");
	fprintf (stderr, "    -R | -S  : request / specify a daemon-specific property name-value pair\n");
	fprintf (stderr, "    -P | -Q  : request / specify a global property name-value pair\n");
	fprintf (stderr, "\n");

	exit (0);
}

#define CMP_EQUAL 0 

oslDaemonCommand
str2cmd ( char* str )
{
	int length;
	if (str == NULL || (length = strlen(str)) == 0)
		return osl_Daemon_CommandUnknown;

	if (CMP_EQUAL == strncmp(str, "start", length))
		return osl_Daemon_CommandStart;
	if (CMP_EQUAL == strncmp(str, "stop", length))
		return osl_Daemon_CommandStop;
	if (CMP_EQUAL == strncmp(str, "shutdown", length))
		return osl_Daemon_CommandShutdown;
	if (CMP_EQUAL == strncmp(str, "pause", length))
		return osl_Daemon_CommandPause;
	if (CMP_EQUAL == strncmp(str, "request", length))
		return osl_Daemon_CommandUnknown;
	if (CMP_EQUAL == strncmp(str, "continue", length))
		return osl_Daemon_CommandContinue;

	return osl_Daemon_CommandUnknown;
}




typedef enum {
	mode_run,
	mode_list,
	mode_inforequest,
	mode_infoset,
	mode_install,
	mode_uninstall,
	mode_daemonproperty_request,
	mode_daemonproperty_set,
	mode_property_request,
	mode_property_set,
	mode_control
} run_mode;




int 
main ( int argc, char **argv )
{
	run_mode          mode = mode_run;
	oslDaemonInfo     info;
	oslDaemonCommand  daemon_ctrl = osl_Daemon_CommandUnknown;
	FILE* 	          outfile = stderr;	

	strncpy ( info.display_name, basename(argv[0]), OSL_DISPLAY_NAMELEN );
	strncpy ( info.service_name, basename(argv[0]), OSL_SERVICE_NAMELEN );
	strncpy ( info.path,		 argv[0],			OSL_PATHLEN );
	memset  ( info.dependencies, 0, 				OSL_DEPENDENCIESLEN );
	info.startup_mode = osl_Daemon_OnDemand;
	info.size 		  = sizeof (oslDaemonInfo);

	char *defaultname = "iam" ; 
	char *defaultvalue = NULL; 

	char *name  = defaultname;
	char *value = defaultvalue;

	/*
	 * parse command-line options, if any
	 */
 
	int ch;
	while ((ch = getopt(argc, argv, "ho:iluac:d:n:p:rsN:V:RSPQ")) != EOF)
	{
		switch(ch) 
		{
		case 'o':

			outfile = fopen ( optarg, "w" );
			if ( outfile == NULL )
				outfile = stderr;
			break;

		case 'i':

			mode = mode_install;
			break;

		case 'l':

			mode = mode_list;
			break;

		case 'u':

			mode = mode_uninstall;
			break;

		case 'c':

			mode = mode_control;
			daemon_ctrl = str2cmd (optarg);
			break;	

		case 'd':

			strncpy ( info.display_name, optarg, OSL_DISPLAY_NAMELEN );
			break;

		case 'n':

			strncpy ( info.service_name, optarg, OSL_SERVICE_NAMELEN );
			break;

		case 'p':

			strncpy ( info.path, optarg, OSL_PATHLEN );
			break;

		case 'r':

			mode = mode_inforequest;
			break;

		case 's':

			mode = mode_infoset;
			break;

		case 'R':

			mode = mode_daemonproperty_request;
			break;

		case 'S':

			mode = mode_daemonproperty_set;
			break;

		case 'P':

			mode = mode_property_request;
			break;

		case 'Q':

			mode = mode_property_set;
			break;

		case 'N':

			name = optarg;
			break;

		case 'V':

			value = optarg;
			break;

		case 'h':
		case '?':
		default :

			usage( argv );
		}
	}

	/*
     * run command-line driven mode 
	 */


	switch ( mode )
	{
		case mode_list:
	
			enum_info ( basename(argv[0]), outfile, &info );
			break;
	
		case mode_inforequest:
	
			get_info ( basename(argv[0]), outfile, &info );
			break;
	
		case mode_infoset:

			set_info ( basename(argv[0]), outfile, &info );
			break;

		case mode_install:

			daemon_install ( basename(argv[0]), outfile, &info );
			break;

		case mode_uninstall:

			daemon_uninstall ( basename(argv[0]), outfile, &info );
			break;

		case mode_daemonproperty_request:

			daemon_proprequest ( basename(argv[0]), outfile, &info, name );
			break;

		case mode_daemonproperty_set:

			daemon_propset ( basename(argv[0]), outfile, &info, name, value );
			break;

		case mode_property_request:

			global_proprequest ( basename(argv[0]), outfile, &info, name );
			break;

		case mode_property_set:

			global_propset ( basename(argv[0]), outfile, &info, name, value );
			break;
		
		case mode_control:

			run_as_master ( basename(argv[0]), outfile, &info, daemon_ctrl );
			break;

		case mode_run:
		default:

			run_as_slave ( basename(argv[0]), outfile, &info);
			break;

	}
}

