/*************************************************************************
 *
 *  $RCSfile: actor.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_DAEMON_H_
#   include <osl/daemon.h>
#endif
#ifndef _OSL_DEMONMGR_H_
#   include <osl/demonmgr.h>
#endif
#ifndef _OSL_DEMONREQ_H_
#   include <osl/demonreq.h>
#endif
#ifndef _VOS_MUTEX_HXX_
#	include <vos/mutex.hxx>
#endif

#ifndef _ACTOR_HXX_
#	include "actor.hxx"
#endif
#ifndef _ENTITY_HXX_
#	include "entity.hxx"
#endif


DmgrActor::DmgrActor ( OStreamPipe &hConnection, 
			  		   DmgrEntity *pReporter )
	: maConnection ( hConnection ), 
	  mpReporter   ( pReporter ) 
{
	mpReporter->SetRequestedState( osl_Daemon_StatusRunning );
}


DmgrActor::~DmgrActor ()
{
}


Boolean
DmgrActor::Control ( oslDaemonCommand eCommand )
{
	sSize_t 		nBytesWritten;
	DmgrRequest	 	aRequest;

	RTC_MEMSET( aRequest );

	oslDaemonStatus ReqState = mpReporter->GetRequestedState();

	switch ( eCommand )
	{
	case osl_Daemon_CommandStart:
		// this can't be handled here really, since the existence of this 
		// (the DmgrActor object) requires a running daemon 
		break;
	case osl_Daemon_CommandStop:
		if ( ReqState == osl_Daemon_StatusStopped )
			return True;
		ReqState = osl_Daemon_StatusStopped;
		break;
	case osl_Daemon_CommandPause:
		if ( ReqState == osl_Daemon_StatusStopped )
			return False;
		if ( ReqState == osl_Daemon_StatusPaused ) 
			return True;
		ReqState = osl_Daemon_StatusPaused;
		break;
	case osl_Daemon_CommandContinue:
		if ( ! (ReqState == osl_Daemon_StatusPaused) )
			return False;
		ReqState = osl_Daemon_StatusRunning;
		break;
	case osl_Daemon_CommandShutdown:
		if ( ReqState == osl_Daemon_StatusStopped )
			return True;
		ReqState = osl_Daemon_StatusStopped;
		break;
	default:
	case osl_Daemon_CommandUnknown:
		return False;
		break;
	}

	aRequest.request = osl_Daemon_CtrlDaemon;
	aRequest.request_data[0] = (Dword)eCommand;
	nBytesWritten = maConnection.write ( (void*) &aRequest, sizeof(aRequest) );
	if ( !nBytesWritten )
		return False;
	if ( ! mpReporter->SetRequestedState( ReqState ))
		return False;

	return True;
}


