/*************************************************************************
 *
 *  $RCSfile: commctor.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:10 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _OSL_DEMONMGR_H_
#	include <osl/demonmgr.h>
#endif 
#ifndef _OSL_DEMONREQ_H_
#	include <osl/demonreq.h>
#endif 

#ifndef _COMMCTOR_HXX_
#	include "commctor.hxx"
#endif
#ifndef _DISPATCH_HXX_
#	include "dispatch.hxx"
#endif
#ifndef _ENTITY_HXX_
#	include "entity.hxx"
#endif
#ifndef _ACTOR_HXX_
#	include "actor.hxx"
#endif

void
standard_reply ( DmgrReply *reply, DmgrRequest *request )
{
	reply->reply    	= request->request;
	reply->uniq_id  	= request->uniq_id;
	reply->reply_id 	= request->request_id;
	reply->reply_size 	= sizeof( DmgrReply );
}



DmgrCommunicator::DmgrCommunicator ( OStreamPipe &hConnection, 
			  			   		     DmgrDispatcher *pDispatcher )
	: maConnection ( hConnection ), 
	  mpDispatcher ( pDispatcher ) 
{
	mnOpenEntities = 0;
}


DmgrCommunicator::~DmgrCommunicator ()
{
}


Boolean
DmgrCommunicator::Communicate ( )
{
	DmgrRequest aRequest;
	DmgrReply   aReply;

	RTC_MEMSET( aRequest );
	RTC_MEMSET( aReply );

	while ( 1 )
	{
		int nBytesRead = maConnection.read ( (void*) &aRequest, sizeof(aRequest) );
		if ( !nBytesRead )
		{
			on_CloseConnection ( aRequest, aReply );
			return False;	// <--| Emergency Exit
		}

		switch ( aRequest.request )
		{
		case osl_Daemon_OpenConnection:
			on_OpenConnection ( aRequest, aReply );
			break;

		case osl_Daemon_CloseConnection:
			on_CloseConnection ( aRequest, aReply );
			return True;	// <--| Normal Exit 

    	case osl_Daemon_EchoRequest:
			on_EchoRequest ( aRequest, aReply );
			break;

    	case osl_Daemon_InstallDaemon:
    	case osl_Daemon_OpenDaemon:
			on_OpenDaemon ( aRequest, aReply );
			break;

    	case osl_Daemon_CloseDaemon:
			on_CloseDaemon ( aRequest, aReply );
			break;

    	case osl_Daemon_UninstallDaemon:
			on_UninstallDaemon ( aRequest, aReply );
			break;

    	case osl_Daemon_EnumDaemonInfo:
			on_EnumDaemonInfo ( aRequest, aReply );
			break;

    	case osl_Daemon_GetDaemonInfo:
			on_GetDaemonInfo ( aRequest, aReply );
			break;

    	case osl_Daemon_SetDaemonInfo:
			on_SetDaemonInfo ( aRequest, aReply );
			break;

    	case osl_Daemon_SetDaemonProperty:
    	case osl_Daemon_SetProperty:
			on_SetProperty ( aRequest, aReply );
			break;

    	case osl_Daemon_GetProperty:
    	case osl_Daemon_GetDaemonProperty:
			on_GetProperty ( aRequest, aReply );
			break;

    	case osl_Daemon_CtrlDaemon:
			on_CtrlDaemon ( aRequest, aReply );
			break;

    	case osl_Daemon_QueryDaemon:
			on_QueryDaemon ( aRequest, aReply );
			break;

    	case osl_Daemon_NOP:
		default:

			break;
		}
	}

	// never reached
	return False;
}


void
DmgrCommunicator::on_OpenConnection ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	// can't be handled here, since the communicator
	// expects an open connection
}

void
DmgrCommunicator::on_CloseConnection ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	if ( mnOpenEntities )
		mpDispatcher->Cleanup ( aRequest.uniq_id );

	mnOpenEntities = 0;
}

void
DmgrCommunicator::on_EchoRequest ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	standard_reply ( &aReply, &aRequest );
    int nBytesWritten = maConnection.write ( (void*) &aReply, sizeof( aReply ) );
}

void
DmgrCommunicator::on_OpenDaemon ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	int 	nNamelen  =  aRequest.request_data[0];
	int 	nEntityId = 0;
	Boolean bSuccess  = False;
	
	if ( nNamelen > 0 )
	{
		char *pName      = new char[ nNamelen ];
 		int   nBytesRead = maConnection.read ( pName, nNamelen );
		if ( ! (nBytesRead == nNamelen) )
			nEntityId = 0;
		else
		if ( aRequest.request == osl_Daemon_OpenDaemon )
			nEntityId = mpDispatcher->OpenEntity ( pName, aRequest.uniq_id );
		else
		if ( aRequest.request == osl_Daemon_InstallDaemon )
			nEntityId = mpDispatcher->NewEntity ( pName, aRequest.uniq_id );

		bSuccess = nEntityId > 0 ? True : False;		
		delete pName;
	}
	
	if ( bSuccess )
		mnOpenEntities += 1;

	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = (Dword)bSuccess;
	aReply.reply_data[1] = (Dword)nEntityId;

	int nBytesWritten = maConnection.write ( &aReply, sizeof( aReply ) );
}

void
DmgrCommunicator::on_CloseDaemon ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	mpDispatcher->CloseEntity ( aRequest.request_data[0],
                                aRequest.uniq_id );
	mnOpenEntities -= 1;
}

void
DmgrCommunicator::on_UninstallDaemon ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	mpDispatcher->DeleteEntity ( aRequest.request_data[0],
                                 aRequest.uniq_id );
	mnOpenEntities -= 1;
}

void
DmgrCommunicator::on_EnumDaemonInfo ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	Boolean bSuccess = mpDispatcher->EnumDaemonInfo ( aRequest.request_data[0],
													  &maBuffer );
	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = (Dword)bSuccess;
	aReply.reply_data[1] = sizeof( maBuffer );

	int nBytesWritten = maConnection.write ( &aReply, sizeof( aReply ) );
	if ( bSuccess )
		nBytesWritten = maConnection.write ( &maBuffer, sizeof(maBuffer) );
}

void
DmgrCommunicator::on_GetDaemonInfo ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	Boolean bSuccess;
	DmgrEntity *pEntity = mpDispatcher->GetEntity ( aRequest.request_data[0],
									  			    aRequest.uniq_id );
	if ( !pEntity )
		bSuccess = False;
	else
		bSuccess = pEntity->GetDaemonInfo ( &maBuffer );

	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = (Dword)bSuccess;
	aReply.reply_data[1] = sizeof( maBuffer );

	int nBytesWritten = maConnection.write ( &aReply, sizeof( aReply ) );
	if ( bSuccess )
		nBytesWritten = maConnection.write ( &maBuffer, sizeof(maBuffer) );
}

void
DmgrCommunicator::on_SetDaemonInfo ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	Boolean bSuccess;
	DmgrEntity *pEntity = NULL;
	int nBytesRead = maConnection.read ( &maBuffer, sizeof(maBuffer) );
	if ( !nBytesRead )
		bSuccess == False;
	else
		pEntity = mpDispatcher->GetEntity ( aRequest.request_data[0],
											aRequest.uniq_id );
	if ( !pEntity )
		bSuccess = False;
	else
		bSuccess = pEntity->SetDaemonInfo ( &maBuffer );

	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = (Dword)bSuccess;

	int nBytesWritten = maConnection.write ( &aReply, sizeof(aReply) );
}

void
DmgrCommunicator::on_SetProperty ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	int 	nNamelen  = aRequest.request_data[1];
	int 	nValuelen = aRequest.request_data[2];
	Boolean bSuccess  = False;

	Boolean     bOnEntity = (aRequest.request == osl_Daemon_SetDaemonProperty);
	DmgrEntity *pEntity;

	if ( bOnEntity )
		pEntity = mpDispatcher->GetEntity ( aRequest.request_data[0], aRequest.uniq_id );

	if ( ( nNamelen > 0) && (nValuelen > 0) )
	{
		char *pName = new char[ nNamelen  ];
		char *pValue= new char[ nValuelen ];

		int   nBytesRead  = maConnection.read ( (void*) pName,  nNamelen );
		      nBytesRead += maConnection.read ( (void*) pValue, nValuelen );
	
		if ( bOnEntity && pEntity )
			bSuccess = pEntity->SetProperty ( pName, pValue );
		else
		if ( bOnEntity )
			bSuccess = False;
		else
			bSuccess = mpDispatcher->SetProperty( pName, pValue);

		delete pName;
		delete pValue;
	}
	else
	if ( nNamelen > 0 )
	{
		char *pName = new char[ nNamelen ];
		int nBytesRead  = maConnection.read ( (void*) pName,  nNamelen );
				
		if ( bOnEntity && pEntity )
			bSuccess = pEntity->SetProperty ( pName, NULL );
		else
		if ( bOnEntity )
			bSuccess = False;
		else
			bSuccess = mpDispatcher->SetProperty( pName, NULL);

		delete pName;
	}

	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = (Dword)bSuccess;

	int nBytesWritten = maConnection.write ( &aReply, sizeof(aReply) );
}

void
DmgrCommunicator::on_GetProperty ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	int 	nNamelen  = aRequest.request_data[1];
	int 	nValuelen = aRequest.request_data[2];
	Boolean bSuccess  = False;

	Boolean     bOnEntity = (aRequest.request == osl_Daemon_GetDaemonProperty);
	DmgrEntity *pEntity;

	if ( bOnEntity )
		pEntity = mpDispatcher->GetEntity ( aRequest.request_data[0], aRequest.uniq_id );

	standard_reply ( &aReply, &aRequest );

	if ( (nNamelen > 0) && (nValuelen > 0) )
	{
		char *pName 	 = new char[ nNamelen  ];
		char *pValue	 = new char[ nValuelen ];
		int   nBytesRead = maConnection.read ( (void*) pName,  nNamelen );

		if ( bOnEntity && pEntity )
			bSuccess = pEntity->GetProperty ( pName, pValue, nValuelen );
		else
		if ( bOnEntity )
			bSuccess = False;
		else
			bSuccess = mpDispatcher->GetProperty( pName, pValue, nValuelen );

		aReply.reply_data[0] = (Dword)bSuccess;
		aReply.reply_data[1] = (Dword)nValuelen;

		int nBytesWritten  = maConnection.write ( &aReply, sizeof(aReply) );
		    nBytesWritten += maConnection.write ( pValue, nValuelen );

		delete pName;
		delete pValue;
	}
	else
	{
		aReply.reply_data[0] = (Dword)False;
		int nBytesWritten  = maConnection.write ( &aReply, sizeof(aReply) );
	}
}

void
DmgrCommunicator::on_CtrlDaemon ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	oslDaemonCommand eCommand = aRequest.request_data[1];
	int 			 nTimeout = aRequest.request_data[2]; // uSeconds
				
	Boolean     bSuccess = False;
	DmgrEntity *pEntity  = mpDispatcher->GetEntity ( aRequest.request_data[0],
						  			  			     aRequest.uniq_id );
	if ( pEntity )
	{
		DmgrActor* pActor;
		if ( eCommand == osl_Daemon_CommandStart )
		{
			bSuccess = mpDispatcher->StartDaemon ( aRequest.request_data[0],
												   aRequest.uniq_id,
												   nTimeout );

			// road works -- wait until the daemon connects
		}
		else
		if ( (pActor = pEntity->GetActor()) )
		{
			bSuccess = pActor->Control ( eCommand );
			// road works -- wait until the daemon connects
		}
	}
	
	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = bSuccess;
	int nBytesWritten = maConnection.write ( &aReply, sizeof( aReply ) );
}

void
DmgrCommunicator::on_QueryDaemon ( DmgrRequest &aRequest, DmgrReply &aReply )
{
	oslDaemonStatus eStatus  = osl_Daemon_StatusUnknown;
	Boolean         bSuccess = False;
	DmgrEntity     *pEntity  = mpDispatcher->GetEntity ( aRequest.request_data[0],
					    	  			                 aRequest.uniq_id );
	if ( pEntity )
	{
		eStatus = pEntity->GetDaemonState();
		bSuccess = eStatus == osl_Daemon_StatusUnknown ? False : True ;
	}	

	standard_reply ( &aReply, &aRequest );
	aReply.reply_data[0] = bSuccess;
	aReply.reply_data[1] = eStatus;
	int nBytesWritten = maConnection.write ( &aReply, sizeof( aReply ) );
}

