/*************************************************************************
 *
 *  $RCSfile: dispatch.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _VOS_MUTEX_HXX_
#   include <vos/mutex.hxx>
#endif
#ifndef _VOS_PROCESS_HXX_
#   include <vos/process.hxx>
#endif
#ifndef _ERROR_HXX_
#	include "error.hxx"
#endif
#ifndef _ENTITY_HXX_
#	include "entity.hxx"
#endif
#ifndef _MTPROFILE_HXX_
#	include "mtprofile.hxx"
#endif
#ifndef _DISPATCH_HXX_
#	include "dispatch.hxx"
#endif

#include <string.h>



DmgrDispatcher::DmgrDispatcher  ( MTProfile *pRepository,
								  Boolean bAutomatic  )
	: mpRepository ( pRepository )
{
	mpEntity      = new Entity[ 32 ];
	mnAllocCount  = 32;
	mnEntityCount =  1;

	int  nIdx = 0;
	int  nEntityId = 1;
	char pServiceName[ 1024 ];
	while ( mpRepository->GetNextService ( &nIdx, pServiceName, 
										   sizeof(pServiceName) ) )
	{
		int id = NewEntity ( pServiceName, 0 );
		if ( bAutomatic && id )
		{
			StartDaemon ( id, 0 /* caller (self) */, 0 /* timeout */ );
		}
	}
}


DmgrDispatcher::~DmgrDispatcher ( )
{
    for ( int i = 0; i < mnEntityCount; i++ )
    {
		delete mpEntity[ i ].entry;
		delete mpEntity[ i ].name;
	}

	free (mpEntity);
}


int
DmgrDispatcher::EntityExists ( char* name )
{
	OGuard Guard( maMutex );
	int    nEntityId = 0;

    for ( int i = 0; i < mnEntityCount; i++ )
    {
		if (   (mpEntity[i].owner >= 0)
			&& (strcmp (name, mpEntity[i].name) == 0) )
			nEntityId = i;
	}

	return nEntityId;
}


Boolean
DmgrDispatcher::AccessAllowed ( int nEntityId, int caller, Boolean do_lock )
{
	OGuard  Guard( maMutex );

	if ( (nEntityId == 0) || !(nEntityId < mnEntityCount) )
		return False;
	
	if ( do_lock )
	{
		if ( mpEntity[ nEntityId ].owner == 0 )
		{
			mpEntity[ nEntityId ].owner	= caller;
			return True;
		}
		return False;
	}
	
	if ( caller == 0 )
		return True;

	return ( mpEntity[ nEntityId ].owner == caller);
}


int
DmgrDispatcher::OpenEntity ( char* name, int caller )
{
	OGuard Guard( maMutex );
	int    nEntityId;

	nEntityId = EntityExists ( name );
	if ( !AccessAllowed (nEntityId, caller, True) )
		nEntityId = 0;

	return nEntityId;
}


int
DmgrDispatcher::EmptyEntity()
{
    OGuard Guard( maMutex );
	int i;

	for ( i = 0; i < mnEntityCount; i++ )
	{
		if ( mpEntity[i].owner == -1 )
			break;
	}

	// enlarge the list of services and return the first new
	if ( i == mnAllocCount /* && i == mnEntityCount */  )
	{
		int 	nNewCount = 2 * mnAllocCount; 
		void* 	pNewPointer;

		pNewPointer = (void*) realloc ( mpEntity, 
									    nNewCount * sizeof (Entity) );
		if (! pNewPointer )
			return 0;

		mpEntity = (Entity*)pNewPointer;
		for (int j = mnAllocCount; j < nNewCount; j++)
			mpEntity[ j ].owner == -1;

		mnEntityCount += 1;
		mnAllocCount   = nNewCount;
	}
	else
	if ( i == mnEntityCount )
	{
		mnEntityCount += 1;
	}

	return i;
} 


int
DmgrDispatcher::NewEntity ( char* name, int caller )
{
	OGuard Guard( maMutex );
	int    nEntityId;

	if ( (Boolean)EntityExists( name ) )
		return 0;

	if (! (nEntityId = EmptyEntity()) )
		return 0;

	int nNamelen = strlen(name) + 1;
	mpEntity[ nEntityId ].owner  = caller;
	mpEntity[ nEntityId ].name   = new char [ strlen(name) + 1 ];
	mpEntity[ nEntityId ].daemon = 0;
	mpEntity[ nEntityId ].entry  = new DmgrEntity ( mpRepository, name );

	strcpy ( mpEntity[ nEntityId ].name, name );

	return nEntityId;
}


DmgrEntity*
DmgrDispatcher::GetEntity  ( int id, int caller )
{
	OGuard Guard( maMutex );

	if ( AccessAllowed ( id, caller, False ) )
		return mpEntity[ id ].entry;

	return NULL;
}


Boolean
DmgrDispatcher::EnumDaemonInfo ( int ordinal, oslDaemonInfo* Info )
{
	OGuard Guard( maMutex );
	int i, j;

	if ( (ordinal < 0) || (ordinal >= mnEntityCount) )
		return False;

	ordinal += 1;
	for (i = 1, j = 0; i < mnEntityCount ; i++)
	{
		if ( mpEntity[i].owner >= 0 )
			j++;
		if ( j == ordinal )
			return mpEntity[i].entry->GetDaemonInfo ( Info );
	}

	return False;
} 


Boolean
DmgrDispatcher::StartDaemon ( int id, int caller, int timeout )
{
	OGuard Guard( maMutex );

	if ( AccessAllowed ( id, caller, False ) )
	{
		oslDaemonInfo Info;
		if (! mpEntity[ id ].entry->GetDaemonInfo( &Info ) )
			return False;
		if ( mpEntity[ id ].entry->IsConnected() )
			return False;

		OProcess::TProcessError eError;
		OProcess::TProcessInfo  aProcessInfo;
		OProcess 				DaemonProcess ( Info.path );

		eError = DaemonProcess.execute ( OProcess::TOption_Detached );
		if ( eError != OProcess::E_None )
			return False;
		
		eError = DaemonProcess.getInfo (OProcess::TData_Identifier, &aProcessInfo) ;
		if ( eError !=  OProcess::E_None )
			return False;

		mpEntity[ id ].entry->IsConnected( True );
		mpEntity[ id ].daemon = aProcessInfo.Ident;
		return True;
	}
	return False;
}


void
DmgrDispatcher::DeleteEntity ( int id, int caller )
{
	OGuard Guard( maMutex );

	if ( AccessAllowed ( id, caller, False ) )
	{
		mpEntity[ id ].entry->MarkForUninstall();

		if ( mpEntity[ id ].entry->IsReadyForUninstall() )
		{
			delete mpEntity[ id ].entry;
			mpEntity[ id ].entry = NULL;
			delete mpEntity[ id ].name;
			mpEntity[ id ].name  = NULL;
			mpEntity[ id ].owner = -1;
		}
		else
		{
			delete mpEntity[ id ].name;
			mpEntity[ id ].name  = NULL;
			mpEntity[ id ].owner = -2; 
		}
	}
}


void
DmgrDispatcher::CloseEntity ( int id, int caller )
{
	OGuard Guard( maMutex );

	if ( AccessAllowed ( id, caller, False ) )
		mpEntity[ id ].owner = 0; 
}


Boolean
DmgrDispatcher::SetProperty ( char* Name, char *Value )
{
	return mpRepository->SetProperty ("Properties", Name, Value);
}


Boolean
DmgrDispatcher::GetProperty ( char* Name, char *Value, size_t Length )
{
	return mpRepository->GetProperty ( "Properties", Name, Value, Length );
}


Boolean
DmgrDispatcher::Cleanup	( int caller )
{
	OGuard  Guard( maMutex );
	Boolean bResult = True;

    for ( int i = 0; i < mnEntityCount; i++ )
    {
		if (mpEntity[i].owner == caller)
		{
			mpEntity[i].owner = 0;
			bResult = False;
		}
	}

	return bResult;
}


int
DmgrDispatcher::ReconnectDaemon( int pid, OStreamPipe &aConnection)
{
    OGuard Guard( maMutex );

	int nEntityId;
	for ( nEntityId = 0; nEntityId < mnEntityCount; nEntityId++ )
	{
		if (   mpEntity[ nEntityId ].daemon == pid 
			&& mpEntity[ nEntityId ].owner != -1 )
			break;
	}

	DmgrEntity* pEntity = GetEntity ( nEntityId, 0 /* caller self */ );
	if ( !pEntity )
		return 0;

	pEntity->NewListener ( aConnection );
	pEntity->NewActor    ( aConnection );

	return nEntityId;
} 


Boolean
DmgrDispatcher::DisconnectDaemon( int nid )
{
    OGuard Guard( maMutex );

	DmgrEntity* pEntity = GetEntity ( nid, 0 /* caller self */ );
	if ( !pEntity )
		return False;

	pEntity->DelListener 	( );
	pEntity->DelActor    	( );
	pEntity->IsConnected 	( False );
	pEntity->SetDaemonState ( osl_Daemon_StatusStopped );
	
	return True;
} 
