/*************************************************************************
 *
 *  $RCSfile: entity.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _ENTITY_HXX_
#	include "entity.hxx"
#endif
#ifndef _ACTOR_HXX_
#	include "actor.hxx"
#endif
#ifndef _LISTENER_HXX_
#	include "listener.hxx"
#endif
#ifndef _MTPROFILE_HXX_
#	include "mtprofile.hxx"
#endif


#include <string.h>


DmgrEntity::DmgrEntity ( MTProfile *pRepository, char* pService )
	: mpRepository ( pRepository )
{
	mbUninstall 	= False;
	mbWriteThrough 	= True;
	mbConnected		= False;
	
	mpDaemonInfo 	= NULL;
	mpActor 		= NULL;
	mpListener 		= NULL;

    meDaemonState	= osl_Daemon_StatusStopped;
    meRequestedState= osl_Daemon_StatusStopped;
    meError			= dmgr_NoError;

	if ( pService )
	{
		mpDaemonInfo = new oslDaemonInfo;
		Boolean bResult = mpRepository->GetInfo ( pService, mpDaemonInfo );
		if ( !bResult )
		{
			mpRepository->NewService ( pService );
			delete mpDaemonInfo;
			mpDaemonInfo = NULL;
		}
	}
}


DmgrEntity::~DmgrEntity ()
{
	OGuard aGuard( maMutex );

	if ( !mbWriteThrough && !mbUninstall && (mpDaemonInfo != NULL) )
		mpRepository->SetInfo ( mpDaemonInfo );

	if ( mbUninstall && (mpDaemonInfo != NULL) )
		mpRepository->DelService ( mpDaemonInfo->service_name );

	if ( mpListener )
		delete mpListener;
	if ( mpActor )
		delete mpActor;
	if ( mpDaemonInfo )
		delete mpDaemonInfo;
}


void
DmgrEntity::MarkForUninstall ()
{
	mbUninstall  = True;
}


Boolean
DmgrEntity::IsMarkedForUninstall ()
{
	return mbUninstall;
}


Boolean
DmgrEntity::IsReadyForUninstall ()
{
	OGuard aGuard( maMutex );

	if (    mbUninstall 
		 && (mpActor 		== NULL) 
		 && (mpListener 	== NULL)
		 && (meDaemonState 	== osl_Daemon_StatusStopped) )
		return True;

	return False;
}


Boolean
DmgrEntity::GetDaemonInfo ( oslDaemonInfo *DaemonInfo )
{
	OGuard aGuard( maMutex );

	if ( mpDaemonInfo != NULL )
		memcpy ( DaemonInfo, mpDaemonInfo, sizeof(oslDaemonInfo) );

	return (mpDaemonInfo != NULL);
}


Boolean
DmgrEntity::SetDaemonInfo ( oslDaemonInfo *DaemonInfo )
{
	OGuard aGuard( maMutex );
	Boolean bResult = True;

	if ( mpDaemonInfo == NULL )
		mpDaemonInfo = new oslDaemonInfo;

	memcpy ( mpDaemonInfo, DaemonInfo, sizeof (oslDaemonInfo) );

	if ( mbWriteThrough )
		bResult = mpRepository->SetInfo ( DaemonInfo );

	return bResult;
}


oslDaemonStatus
DmgrEntity::GetDaemonState ()
{
	OGuard aGuard( maMutex );

	return meDaemonState; 
}


Boolean
DmgrEntity::SetDaemonState ( oslDaemonStatus eState )
{
	OGuard aGuard( maMutex );
	Boolean bState = True;

	meDaemonState = eState;
	if ( mbWriteThrough )
		bState = mpRepository->SetState ( mpDaemonInfo->service_name, eState );

	return bState;
}


oslDaemonStatus
DmgrEntity::GetRequestedState ()
{
	return meRequestedState;
}


Boolean
DmgrEntity::SetRequestedState ( oslDaemonStatus eState )
{
	// OGuard aGuard( maMutex );
	
	meRequestedState = eState;
	return True;
}


void
DmgrEntity::SetError ( dmgrError  eError )
{
	// OGuard aGuard( maMutex );
	
	meError = eError;
}


dmgrError 
DmgrEntity::GetError ()
{
	// OGuard aGuard( maMutex );
	
	return meError;
}


Boolean
DmgrEntity::IsConnected ()
{
	// OGuard aGuard( maMutex );

	return mbConnected;
}


void
DmgrEntity::IsConnected ( Boolean bConn )
{
	// OGuard aGuard( maMutex );

	mbConnected = bConn;
}


DmgrActor*
DmgrEntity::GetActor ()
{
	// OGuard maGuard ( maMutex );

	return mpActor;
}


DmgrListener*
DmgrEntity::GetListener ()
{
	// OGuard maGuard ( maMutex );

	return mpListener;
}


void
DmgrEntity::NewActor ( OStreamPipe &hConnection )
{
	OGuard maGuard ( maMutex );

	if ( mpActor != NULL )
		abort();

	mpActor = new DmgrActor ( hConnection, this );	
}

void
DmgrEntity::NewListener	( OStreamPipe &hConnection )
{
	OGuard maGuard ( maMutex );

	if ( mpListener != NULL )
		abort();

	mpListener = new DmgrListener ( hConnection, this );	
}


void
DmgrEntity::DelActor ()
{
	OGuard maGuard ( maMutex );

	if ( mpActor )
		delete mpActor;
	mpActor = NULL;
}


void
DmgrEntity::DelListener ()
{
	OGuard maGuard ( maMutex );

	if ( mpListener )
		delete mpListener;
	mpListener = NULL;
}


Boolean
DmgrEntity::SetProperty ( char* Name, char *Value )
{
	OGuard maGuard ( maMutex );

	if ( ! mpDaemonInfo )
		return False;

    return mpRepository->SetDaemonProperty ( mpDaemonInfo->service_name,
											 Name, Value);
}


Boolean
DmgrEntity::GetProperty ( char* Name, char *Value, size_t Length )
{
	OGuard maGuard ( maMutex );

	if ( ! mpDaemonInfo )
		return False;

    return mpRepository->GetDaemonProperty ( mpDaemonInfo->service_name,
											 Name, Value, Length );
}

