/*************************************************************************
 *
 *  $RCSfile: main.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>


#ifndef _VOS_PIPE_HXX_
#   include <vos/pipe.hxx>
#endif
#ifndef _VOS_THREAD_HXX_
#   include <vos/thread.hxx>
#endif
#ifndef _OSL_DEMONMGR_H_
#   include <osl/demonmgr.h>
#endif
#ifndef _OSL_DEMONREQ_H_
#   include <osl/demonreq.h>
#endif
#ifndef _ERROR_HXX_
#	include "error.hxx"
#endif
#ifndef _DISPATCH_HXX_
#	include "dispatch.hxx"
#endif
#ifndef _MTPROFILE_HXX_
#	include "mtprofile.hxx"
#endif
#ifndef _RECEPT_HXX_
#	include "recept.hxx"
#endif

#define DMGR_PROFILE "dmgr.rc"

char*
basename (const char *filename)
{
  char *p = strrchr (filename, '/');
  return p ? p + 1 : (char *) filename;
}

void 
usage ( char **argv )
{
	fprintf (stderr, "\n");	
	fprintf (stderr, "usage:\n");
	fprintf (stderr, "\n");	
	fprintf (stderr, "    %s [-d] [-e] [-h] [-i inifile] [-n] [-p pipe] \n", basename(argv[0]) );
	fprintf (stderr, "\n");
	fprintf (stderr, "    -d         : turn on debug, i.e. run in foreground\n");
	fprintf (stderr, "    -e         : run even with an empty inifile\n");
	fprintf (stderr, "    -h         : help, show this message\n");
	fprintf (stderr, "    -i inifile : use \"inifile\" for config instead of default \"\$HOME/%s\"\n", DMGR_PROFILE );
	fprintf (stderr, "    -n         : do not automatically start the registered daemons\n");
	fprintf (stderr, "    -p pipe    : connect to \"pipe\" instead to default \"%s\"\n", OSL_DMGR_PIPE );
	fprintf (stderr, "\n");

	exit (0);
}

extern int GlobalInit;


int 
main ( int argc, char **argv )
{
	Boolean   bDebug = False;
	Boolean   bEmpty = False;
	Boolean	  bAutoStart = True;
	ErrorList aPipeError ( PipeError );
	char      *pInifileName    = DMGR_PROFILE;
	char      *pConnectionName = OSL_DMGR_PIPE;

	GlobalInit = 1;


	// parse command-line options, if any
 
	int ch;
	while ((ch = getopt(argc, argv, "dehi:np:")) != EOF)
	{
		switch(ch) 
		{
		case 'd':
			bDebug = True;
			break;
		case 'e':
			bEmpty = True;
			break;
		case 'i':
			pInifileName = optarg;
			break;
		case 'n':
			bAutoStart = False;
			break;
		case 'p':
			pConnectionName = optarg;
			break;
		case 'h':
		case '?':
		default :
			usage( argv );
		}
	}


	// create pipe to connect

	OPipe aPipe;
	if ( ! aPipe.create ( pConnectionName, OPipe::TOption_Create ))
	{
		fprintf (stderr, "%s: can't create pipe %s\n", 
				 basename(argv[0]), pConnectionName);
				 // aPipeError.Map(ePipeError) );
		exit(-1);
	}


	// create a profile 

	MTProfile *pProfile = new MTProfile( pInifileName );
	if ( bEmpty && !pProfile->IsValid() )
	{
		pProfile->Initial();
	}
	if ( !pProfile->IsValid() )
	{
		fprintf (stderr, "%s: invalid profile \"%s\"%s\n", 
				 basename(argv[0]), pInifileName, 
				 bEmpty ? " even tried to initialize" : "" );
		exit(-1);
	}


	// create a daemon dispatcher

	DmgrDispatcher* pDispatcher = new DmgrDispatcher ( pProfile, bAutoStart );


	// accept connections on the pipe and decide which dispatcher to call

	while ( 1 )
	{
		OPipe::TPipeError ePipeError;
		OStreamPipe *pConnection = new OStreamPipe;
		ePipeError = aPipe.accept( *pConnection );
		if ( ePipeError != OPipe::E_None )
		{
			fprintf (stderr, "%s: can't create pipe %s (%s)\n", 
					 basename(argv[0]), pConnectionName,
					 aPipeError.Map(ePipeError) );
			delete pConnection;
			break;
		}

		DmgrReception *pReceptionist = new DmgrReception ( pConnection, pDispatcher );
		pReceptionist->create();
	}


	// well done

	delete pProfile;
	delete pDispatcher;
}

