/*************************************************************************
 *
 *  $RCSfile: mtprofile.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _MTPROFILE_HXX_
#	include "mtprofile.hxx"
#endif

#include <string.h>


MTProfile::MTProfile (  const char* pszProfile ) 
	: maProfile ( pszProfile, OProfile::TOption_Default) 
{
}

MTProfile::~MTProfile ()
{
}


Boolean
MTProfile::IsValid ()
{
	Boolean bResult;
    char pSectionNr  [ 32 ];
    int  nSectionNr;
	
	sprintf (pSectionNr, "0");
	bResult = maProfile.readString ( "startup", "service_id", pSectionNr, 
									 sizeof(pSectionNr), "0");
    nSectionNr = atoi (pSectionNr);

	return bResult && (nSectionNr > 0);
}


Boolean
MTProfile::Initial ()
{
	Boolean bResult;

    bResult = maProfile.writeString ( "startup", "service_id", "1000" );
	return bResult;
}


Boolean
MTProfile::SetDaemonProperty ( char* ServiceName, char *Name, char *Value )
{
	OGuard aGuard ( maMutex );
	Boolean Result = True;
	char pSectionName [ 64 ];
	char pPropertyName[ 64 ];

	if ( ServiceName  == (char*)NULL )
		return False;

	Result  = maProfile.readString  ( "services", ServiceName, pSectionName, 
									  sizeof(pSectionName), "");

	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

	Result  = maProfile.readString  ( pSectionName, "properties", 
									  pPropertyName, sizeof(pPropertyName), "");

	if ( !Result || ( pPropertyName[0] == '\0' ) )
	{
		sprintf ( pPropertyName, "%s_properties", pSectionName );
		Result  &= maProfile.writeString  ( pSectionName, "properties", 
										    pPropertyName );
	}

    Result &= SetProperty ( pPropertyName, Name, Value );
	return Result;
}


Boolean
MTProfile::GetDaemonProperty ( char* ServiceName, char *Name, char *Value, 
							   size_t Length )
{
	OGuard aGuard ( maMutex );
	Boolean Result = True;
	char pSectionName [ 64 ];
	char pPropertyName[ 64 ];

	if ( ServiceName  == (char*)NULL )
		return False;

	Result  = maProfile.readString  ( "services", ServiceName, pSectionName, 
									  sizeof(pSectionName), "");

	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

	Result  = maProfile.readString  ( pSectionName, "properties", 
									  pPropertyName, sizeof(pPropertyName), "");

	if ( !Result || ( pPropertyName[0] == '\0' ) )
		return False;

    Result &= GetProperty ( pPropertyName, Name, Value, Length );
	return Result;
}


Boolean
MTProfile::SetProperty ( char* Section, char* Name, char* Value )
{
	OGuard aGuard ( maMutex );

	if ( Name  == (char*)NULL )
		return False;
	if ( Value == (char*)NULL )
		return maProfile.removeEntry ( Section, Name );
	else
		return maProfile.writeString ( Section, Name, Value );
}


Boolean
MTProfile::GetProperty ( char* Section, char* Name, char* Value, size_t Length )
{
	OGuard aGuard ( maMutex );

	if ( (Name  == (char*)NULL) || (Value == (char*)NULL) )
		return False;

	Boolean bResult = maProfile.readString ( Section, Name, Value, 
											 Length, "" );
	return ( bResult && (Value[0] != '\0') );
}


Boolean
MTProfile::SetInfo ( oslDaemonInfo* Info )
{
	OGuard aGuard ( maMutex );
	Boolean Result = True;
	char pSectionName[ 64 ];

	if ( Info == (oslDaemonInfo*)NULL )
		return False;

	Result &= maProfile.readString ( "services", Info->service_name, 
									   pSectionName, sizeof(pSectionName), "");
	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

	Result &= maProfile.writeString ( pSectionName, "display_name", Info->display_name );
	Result &= maProfile.writeString ( pSectionName, "path", Info->path );

	// convert the '00' terminated, '0' separated string container to
	// a ',' separated string
	char  dep[ OSL_DEPENDENCIESLEN ];
	char *ptr = (char*)memcpy ( dep, Info->dependencies, OSL_DEPENDENCIESLEN );   
	
	for ( int i = 0 ; i < OSL_DEPENDENCIESLEN ; i++, ptr++ )
		if ( *ptr == (char)0 )
		{
			if (ptr[1] == (char)0)
				break;
			else
				ptr[0] = ',' ;
		}

    Result &= maProfile.writeString ( pSectionName, "dependencies", dep );

	switch ( Info->startup_mode )
	{
	case osl_Daemon_Automatic:
	    Result &= maProfile.writeString ( pSectionName, "startup", "automatic" );
		break;
	case osl_Daemon_OnDemand:
	default:
	    Result &= maProfile.writeString ( pSectionName, "startup", "ondemand" );
		break;
	}

	return Result;
}


Boolean
MTProfile::GetInfo ( char* ServiceName, oslDaemonInfo* Info )
{
	OGuard aGuard ( maMutex );
	Boolean Result = True;
	char pSectionName[ 64 ];

	if ( Info == (oslDaemonInfo*)NULL )
		return False;

	Result  = maProfile.readString ( "services", ServiceName, pSectionName,
									 sizeof(pSectionName), "");
	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

	strncpy ( Info->service_name, ServiceName, OSL_SERVICE_NAMELEN );

	Result &= maProfile.readString ( pSectionName, "display_name", 
									 Info->display_name, OSL_DISPLAY_NAMELEN, "" );
	Result &= maProfile.readString ( pSectionName, "path",
									 Info->path, OSL_PATHLEN, "" );
	Result &= maProfile.readString ( pSectionName, "dependencies",
									 Info->dependencies, OSL_DEPENDENCIESLEN, "" );

	// convert the '0' terminated, ',' separated stringlist to
	// a '0' separated, '00' terminated dependenciy container
	char *ptr = Info->dependencies;
	
	for ( int i = 0 ; i < OSL_DEPENDENCIESLEN - 2 ; i++, ptr++ )
	{
		if ( *ptr == (char)',' )
			*ptr = '\0';
		else
		if ( *ptr == '\0' )
			break;
	}
	ptr[0] = '\0';
	ptr[1] = '\0';

	char startup_mode[ 16 ];
    Result &= maProfile.readString ( pSectionName, "startup", 
									 startup_mode, sizeof(startup_mode), "" );

	Info->startup_mode = osl_Daemon_OnDemand;
	switch ( startup_mode[0] )
	{
	case 'a':
		if ( 0 == strncmp ( startup_mode, "automatic", 10 ) )
			Info->startup_mode = osl_Daemon_Automatic;
		break;
	case 'o':
	    if ( 0 == strncmp ( startup_mode, "ondemand", 9 ) )
			Info->startup_mode = osl_Daemon_OnDemand;
		break;
	default:
		break;
	}

	Info->size = sizeof (oslDaemonInfo);
	
	return Result;
}


Boolean
MTProfile::NewService ( char* ServiceName )
{
	OGuard aGuard ( maMutex );
	Boolean Result = True;

	if ( ServiceName  == (char*)NULL )
		return False;

	char pSectionName[ 64 ];
	char pSectionNr  [ 32 ];
	int  nSectionNr;

    Result &= maProfile.readString ( "startup", "service_id", pSectionNr, 
									 sizeof(pSectionNr), "0" );
	nSectionNr = atoi (pSectionNr);
	sprintf ( pSectionNr,   "%i",            ++nSectionNr);
	sprintf ( pSectionName, "service_id_%i",   nSectionNr);
	Result &= maProfile.writeString ( "startup", "service_id", pSectionNr);
	Result &= maProfile.writeString ( "services", ServiceName, pSectionName);

    Result &= maProfile.writeString ( pSectionName, "display_name", 	"" );
    Result &= maProfile.writeString ( pSectionName, "path", 			"" );
    Result &= maProfile.writeString ( pSectionName, "dependencies", 	"" );
    Result &= maProfile.writeString ( pSectionName, "startup", 			"" );

	return Result;
}


Boolean
MTProfile::GetNextService ( int *Position, char *Service, int Len )
{
	OGuard aGuard ( maMutex );

	static int    nSectionSize = 0;
	static char  *pSection	   = NULL;
		   char  *pPtr;
		   int	  nEntryLen;

	if ( nSectionSize == 0 )
	{
		nSectionSize = maProfile.getSectionEntries ( "services", NULL, 0 );
	    pSection     = new char[ nSectionSize ];
		maProfile.getSectionEntries ( "services", pSection, nSectionSize );
	}

	if ( *Position >= nSectionSize )
		return False;

	pPtr = pSection + *Position;
	nEntryLen = strlen ( pPtr );
	
	if ( nEntryLen == 0 )
		return False;

	*Position += nEntryLen + 1;
	strncpy ( Service, pPtr, Len ); 
	return True;
}




Boolean
MTProfile::DelService ( char* ServiceName )
{
	OGuard aGuard ( maMutex );
	Boolean Result = True;
	char    pSectionName[ 64 ];

	if ( ServiceName  == (char*)NULL )
		return False;

	Result &= maProfile.readString  ( "services", ServiceName, pSectionName, 
									  sizeof(pSectionName), "");
	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

    Result &= maProfile.removeEntry ( "services", ServiceName );

	int   nEntrySize = maProfile.getSectionEntries ( pSectionName, NULL, 0);
	char* pEntries   = new char[ nEntrySize ];
	char* pPtr;
	nEntrySize       = maProfile.getSectionEntries ( pSectionName, pEntries, nEntrySize );

	for ( int i = 0; i < nEntrySize; i += 1 + strlen( pPtr ) )
	{
		pPtr    = pEntries + i;
		Result &= maProfile.removeEntry ( pSectionName, pPtr );
	}
	delete pEntries;

	return Result;
}


const char   
*StateName[] = {
	"StartPending",	"StopPending", 	"PausePending",
    "Running", 		"Stopped", 		"Paused",	
	"Unknown", NULL
};



Boolean
MTProfile::SetState ( char* ServiceName, oslDaemonStatus eStatus )
{
	OGuard aGuard ( maMutex );

	Boolean Result = True;
	char pSectionName[ 64 ];

	if ( ServiceName  == (char*)NULL )
		return False;

	Result  = maProfile.readString  ( "services", ServiceName, pSectionName, 
									  sizeof(pSectionName), "");

	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

    Result &= maProfile.writeString ( pSectionName, "state", 
								     StateName[ eStatus ] );
	return Result;
}


Boolean
MTProfile::GetState ( char* ServiceName, oslDaemonStatus *eStatus )
{
	OGuard  aGuard ( maMutex );
	char    Property[ 16 ];
	Boolean Result = True;
	char pSectionName[ 64 ];

	if ( ServiceName  == (char*)NULL )
		return False;

	Result  = maProfile.readString ( "services", ServiceName, pSectionName, 
									 sizeof(pSectionName), "");
	if ( !Result || ( pSectionName[0] == '\0' ) )
		return False;

    Result &= maProfile.readString ( pSectionName, "state", Property, 
									 sizeof(Property), "" );

	*eStatus = osl_Daemon_StatusUnknown;
	int i = 0;
	while ( StateName[ i ] )
	{
		if ( strcmp ( StateName[i], Property ) == 0 )
		{
			*eStatus = osl_Daemon_StatusUnknown;
			break;
		}
		i++;
	}

	return Result;
}

