/*************************************************************************
 *
 *  $RCSfile: recept.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <unistd.h>

#ifndef _OSL_DEMONMGR_H_
#   include <osl/demonmgr.h>
#endif
#ifndef _OSL_DEMONREQ_H_
#   include <osl/demonreq.h>
#endif
#ifndef _VOS_PIPE_HXX_
#   include <vos/pipe.hxx>
#endif
#ifndef _VOS_THREAD_HXX_
#   include <vos/thread.hxx>
#endif
#ifndef _RECEPT_HXX_
#	include "recept.hxx"
#endif
#ifndef _ENTITY_HXX_
#	include "entity.hxx"
#endif
#ifndef _COMMCTOR_HXX_
#	include "commctor.hxx"
#endif
#ifndef _LISTENER_HXX_
#	include "listener.hxx"
#endif
#ifndef _DISPATCH_HXX_
#	include "dispatch.hxx"
#endif



DmgrReception::DmgrReception  ( OStreamPipe    *pConnection,
                                DmgrDispatcher *pDispatcher )
	: mpDispatcher ( pDispatcher ), 
	  mpConnection ( pConnection )
{
}


DmgrReception::~DmgrReception ()
{
}


void
DmgrReception::run()
{
	sSize_t 	nBytesRead;
	DmgrRequest aRequest;
	DmgrReply	aReply;

	RTC_MEMSET ( aRequest );
	RTC_MEMSET ( aReply   );

	nBytesRead = mpConnection->read ( (void*) &aRequest, sizeof(aRequest) );
	if ( !nBytesRead )
		return;

	switch ( aRequest.request )
	{
		case osl_Daemon_OpenConnection:
		{
		    aReply.reply    	 = osl_Daemon_OpenConnection;
			aReply.uniq_id  	 = aRequest.request_data[0];  // clients pid();
	    	aReply.reply_id 	 = aRequest.request_id;
			aReply.reply_data[0] = (Dword)True;
			aReply.reply_data[1] = (Dword)getpid();
			aReply.reply_data[2] = (Dword)SUPD;

			// aRequest.request_data[1] == Mode; // not evaluated now
			// aRequest.request_data[2] == SUPD;

			sSize_t nBytesWritten;
			nBytesWritten = mpConnection->write( (void*) &aReply, sizeof(aReply) );
			if ( !nBytesWritten )
				return;
			
			// this runs until client closes connection or gets naughty

			DmgrCommunicator aCommunicator ( *mpConnection, mpDispatcher );
			aCommunicator.Communicate();
		}
		break;

		case osl_Reporter_OpenConnection:
		{
			int nEntityId;

		    aReply.reply    	 = osl_Reporter_OpenConnection;
			aReply.uniq_id  	 = aRequest.request_data[0];  // clients pid();
	    	aReply.reply_id 	 = aRequest.request_id;
			aReply.reply_data[1] = (Dword)getpid();
			aReply.reply_data[2] = (Dword)SUPD;

			// aRequest.request_data[1] == Mode; // not evaluated now
			// aRequest.request_data[2] == SUPD;

			nEntityId = mpDispatcher->ReconnectDaemon( aRequest.request_data[0],
													   *mpConnection);

			aReply.reply_data[0] = (Dword)(nEntityId > 0);

			sSize_t nBytesWritten;
			nBytesWritten = mpConnection->write( (void*) &aReply, sizeof(aReply) );
			if ( !nBytesWritten || !nEntityId )
				return;
			
			// this runs until client closes connection or gets naughty

			DmgrListener *pListener;
			DmgrEntity   *pEntity = mpDispatcher->GetEntity ( nEntityId, 0 /* calller (self) */ );
			
			if ( pEntity && (pListener = pEntity->GetListener()) )
				pListener->Listen();
			// else would be a severe error, reconnected but no listener ?

			mpDispatcher->DisconnectDaemon( nEntityId );
			return;
		}

		case osl_Reporter_NOP:
		case osl_Daemon_NOP:

			break;

		default:

			break;

		// this in an error 

	}
}


void
DmgrReception::onTerminated()
{
	delete mpConnection;
}

