/*************************************************************************
 *
 *  $RCSfile: channel.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _VOS_CHANNEL_HXX_
#define _VOS_CHANNEL_HXX_

#ifndef _VOS_TYPES_HXX_
#	include <vos/types.hxx>
#endif
#ifndef _VOS_OBJECT_HXX_
#	include <vos/object.hxx>
#endif
#ifndef _VOS_THREAD_HXX_
#	include <vos/thread.hxx>
#endif
#ifndef _VOS_SOCKET_HXX_
#	include <vos/socket.hxx>
#endif
#ifndef _VOS_PIPE_HXX_
#	include <vos/pipe.hxx>
#endif
#ifndef _VOS_SECURITY_HXX_
#	include <vos/security.hxx>
#endif
#ifndef _VOS_REFERNCE_HXX_
#	include <vos/refernce.hxx>
#endif
#ifndef _VOS_REF_HXX_
#	include <vos/ref.hxx>
#endif
#ifndef _RTL_STRING_HXX_
#	include <rtl/string.hxx>
#endif

#ifndef _OSL_CHANNEL_H_
#	include <osl/channel.h>
#endif

#ifdef _USE_NAMESPACE
namespace vos
{     
#endif


class OChannel : public OReference, public OObject
{
public:

	VOS_DECLARE_CLASSINFO(NAMESPACE_VOS(OChannel));

	static ORef<OChannel> SAL_CALL createChannel(OObject& rResource);

	virtual IStream* SAL_CALL getStream();
	virtual OObject* SAL_CALL getResource();
	virtual void 	 SAL_CALL close();	

protected:

	OObject* m_pResource;	

private:

	OChannel();							
	virtual ~OChannel();
	OChannel(const OChannel& );			// no implementation
	void SAL_CALL operator=(const OChannel& );	// no implementation
};

class IAuthenticationCheck
{
public:
	virtual sal_Bool SAL_CALL check
	(
		const NAMESPACE_RTL(OUString)& userIdent,
		const NAMESPACE_RTL(OUString)& password
	) = 0;
};

class OChannelBroker : public OReference, public OObject
{
public: 

	VOS_DECLARE_CLASSINFO(NAMESPACE_VOS(OChannelBroker));

	enum TError
	{
		NoError = 					osl_Channel_E_None,
		NoManager = 				osl_Channel_E_NoManger,
		InvalidName = 				osl_Channel_E_InvalidName,
		UnsupportedResourceClass =	osl_Channel_E_UnsupportedResourceClass,
		Timeout = 					osl_Channel_E_Timeout,
		ChannelInUse = 				osl_Channel_E_ChannelInUse,
		AuthenticationFailed = 		osl_Channel_E_AuthenticationFailed,
		Unknown = 					osl_Channel_E_Unknown
	};

	static ORef<OChannelBroker> SAL_CALL getGlobalChannelBroker();

	virtual TError SAL_CALL provide(ORef<OChannel>& 			rChannel, 
						   NAMESPACE_RTL(OUString)& resourceName, 
						   const TimeValue* 		pTimeout = 0);

	virtual TError SAL_CALL provide(ORef<OChannel>& 			rChannel, 
						   NAMESPACE_RTL(OUString)& resourceName, 
						   NAMESPACE_RTL(OUString)& userIdent, 
						   NAMESPACE_RTL(OUString)& password, 
						   const TimeValue* 		pTimeout = 0);

	virtual TError SAL_CALL accept(ORef<OChannel>& 			rChannel, 
						  NAMESPACE_RTL(OUString)& 	resourceName, 
						  IAuthenticationCheck*		pAuthCheck,
						  const TimeValue*  		pTimeout = 0);

protected:

	oslChannelBroker m_hBroker;

private:

	OChannelBroker();									
	virtual ~OChannelBroker();
	OChannelBroker(const OChannelBroker& objectSrc);	// no implementation
	void SAL_CALL operator=(const OChannelBroker& objectSrc);	// no implementation
};



#ifdef _USE_NAMESPACE
}     
#endif

#endif // _VOS_CHANNEL_HXX_

