/*************************************************************************
 *
 *  $RCSfile: daemon.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _VOS_DAEMON_HXX_
#define _VOS_DAEMON_HXX_

#ifndef _VOS_IDAEMON_HXX_
#	include <vos/idaemon.hxx>
#endif
#ifndef _VOS_IDAEMMGR_HXX_
#	include <vos/idaemmgr.hxx>
#endif
#ifndef _VOS_THREAD_HXX_
#	include <vos/thread.hxx>
#endif

#ifdef _USE_NAMESPACE
namespace vos
{
#endif


class ODaemon_Impl;


/** Implementation ODaemon.
	A default implemention of a daemon. 
	override the command handler methods on...() to add functionality.
	Instances of this class has to be created by new and hold by reference (ORef - class).

	Overide at least following methods to declare a concrete daemon class:

	- public: virtual void onStart()
	- public: virtual void onStop()
	- public: virtual void onPause()
	- public: virtual void onContinue()
	- public: virtual void onShutdown()

	Optional overwrite following methods to add properties:

	- public: virtual sal_uInt32 getPropertyCount() const
	- public: virtual sal_uInt32 getProperties(...) const
	- public: virtual sal_Bool setProperties(...) 

	For further information and implemetation restrictions see the description 
	of the interfaces IDaemonAccess and IDaemonControl.
*/
class ODaemon : public IDaemonAccess, 
				public IDaemonControl
{

	VOS_DECLARE_CLASSINFO(VOS_NAMESPACE(ODaemon, vos));

public:

	/** Enum TCommandLineOption.
		denotes the command to execute determined by parisng th command line.
	*/
	enum TCommandLineOption
	{
		TOption_Install,
		TOption_Uninstall,
		TOption_ExecuteAsDaemon,
		TOption_ExecuteAsProcess,
		TOption_Start,
		TOption_Stop,
		TOption_Pause,
		TOption_Resume,
		TOption_Unknown
	};

	/** Parse the command line to determine the command to execute by this daemon.
			-i 		-> TOption_Install,
			-u 		-> TOption_Uninstall,
			   		-> TOption_ExecuteAsDaemon,
			-p 		-> TOption_ExecuteAsProcess,
			-start 	-> TOption_Start,
			-stop 	-> TOption_Stop,
			-pause 	-> TOption_Pause,
			-resume -> TOption_Resume,
	*/
	static TCommandLineOption SAL_CALL parseArgs(sal_Int32 argc, sal_Char* argv[]);

	/** Parse the commmand line and execute the resulting command on this daemon.
		This method returns not until the command is complete or - if command is 
		TOption_ExecuteAsDaemon - until the daemon was stopped.
		This method an be used if the calling process is implementing only one daemon
		as followin example:
 
			class MyDaemon : public Daemon
			{
				...
			};

			sal_Int32 main( sal_Int32 argc, sal_Char * argv[] )
			{
				ORef<MyDaemon> rDaemon = new MyDaemon();

				rDaemon->execute(argc, argv);

				return 0;
			}
	*/
	IDaemonManager::TDaemonManagerError	SAL_CALL execute(sal_Int32 argc, sal_Char* argv[]);

	/** Execute this daemon as standalone process, not under control of daemon manager.
	*/
	IDaemonManager::TDaemonManagerError	SAL_CALL executeAsProcess(sal_Bool waitForCompletion = sal_False);

	/** Constructor 
	*/
	ODaemon(const NAMESPACE_RTL(OUString)& Identifier, 
			const NAMESPACE_RTL(OUString)& DisplayName,
			const NAMESPACE_RTL(OUString)& Manufactor,
			sal_uInt32 Version, 
			TDaemonStartupMode Mode,
			const NAMESPACE_RTL(OUString)& Dependencies);

	/** Destructor 
	*/
	virtual ~ODaemon();

	/* Implementation of IDaemonAccess interface.
	*/
	virtual const NAMESPACE_RTL(OUString)& 	SAL_CALL getIdentifier() const;
	virtual const NAMESPACE_RTL(OUString)&	SAL_CALL getDisplayName() const;
	virtual const NAMESPACE_RTL(OUString)&	SAL_CALL getManufactor() const;
	virtual const NAMESPACE_RTL(OUString)&	SAL_CALL getDependencies() const;
	virtual sal_uInt32 			SAL_CALL getVersion() const;
	virtual TDaemonStatus 		SAL_CALL getStatus() const;
	virtual TDaemonStartupMode 	SAL_CALL getMode() const;
	virtual sal_uInt32 			SAL_CALL getPropertyCount() const;
	virtual sal_uInt32 			SAL_CALL getProperties(ODaemonProperty* 	pProperties, 
                                                  sal_uInt32 		count) const;
	virtual sal_Bool			SAL_CALL setProperties(ODaemonProperty* 	    pProperties, 
											  sal_uInt32                count,
								  			  sal_uInt32&               errorCode,
											  NAMESPACE_RTL(OUString)&  errorText);


	/* implementation of IReference	interface.
	*/
	virtual RefCount SAL_CALL acquire();
	virtual RefCount SAL_CALL release();
	virtual RefCount SAL_CALL referenced() const;

	/* Implementation of IDaemonControl	interface.
	*/
	void SAL_CALL setListener(const ORef<IDaemonListener>& rListener);


protected:

	/** Helper method which should be called by the impelementation to send 
		status informations to the daemon manager.
		This method has to be called first in all command handler methods
		to indicate that the command recieved (e.g. setStatus(TStatus_StopPending))
		and at the time the command was cempleted (e.g. setStatus(TStatus_Stopped)).
	*/
	void SAL_CALL setStatus(TDaemonStatus status);

protected:

	ODaemon_Impl* m_pImpl;

private:
	ODaemon(const ODaemon&);
	ODaemon& SAL_CALL operator=(const ODaemon&);
};

#ifdef _USE_NAMESPACE
}
#endif

#endif // _VOS_DAEMON_HXX_

/*************************************************************************
*
*    $Log: daemon.hxx,v $
*    Revision 1.1.1.1  2000/09/18 15:18:12  hr
*    initial import
*
*    Revision 1.18  2000/09/18 10:52:28  willem.vandorp
*    OpenOffice header added.
*
*    Revision 1.17  2000/07/26 16:16:11  willem.vandorp
*    Headers/footers replaced.
*
*    Revision 1.16  2000/05/29 16:29:10  hro
*    SRC591: Explicite SAL_CALL calling convention
*
*    Revision 1.15  2000/05/18 11:21:51  mfe
*    UniCode changes
*
*    Revision 1.14  2000/05/17 13:19:59  mfe
*    UniCode Changes for Daemons
*
*    Revision 1.13  2000/04/11 17:10:15  abi
*    Conversion to Unicode
*
*    Revision 1.12  1999/02/05 08:32:18  jsc
*    #61011# Typanpassung und OString entfernt
*
*    Revision 1.11  1998/11/27 15:21:41  ts
*    #52712# setProperties -> error code
*
*    Revision 1.10  1998/11/26 12:11:17  ts
*    #52712# Dokumentation
*
*    Revision 1.9  1998/11/26 08:08:13  ts
*    #52712# Property Schnittstelle
*
*    Revision 1.8  1998/11/17 18:38:33  ts
*    #52712# Daemonschnittstelle angepasst
*
*    Revision 1.7  1998/11/13 09:02:54  ts
*    #52712# daemon manager eingebaut
*
*    Revision 1.6  1998/11/13 08:53:15  ts
*    #52712# daemon manager eingebaut
*
*    Revision 1.5  1998/11/13 08:14:12  ts
*    #52712# daemon manager eingebaut
*
*************************************************************************/
 
