/*************************************************************************
 *
 *  $RCSfile: idaemmgr.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _VOS_IDAEMMGR_HXX_
#define _VOS_IDAEMMGR_HXX_

#ifndef _VOS_TYPES_HXX_
#	include <vos/types.hxx>
#endif
#ifndef _VOS_IDAEMON_HXX_
#	include <vos/idaemon.hxx>
#endif
#ifndef _VOS_REFERNCE_HXX_
#	include <vos/refernce.hxx>
#endif
#ifndef _VOS_PROFILE_HXX_
#	include <vos/profile.hxx>
#endif
#ifndef _VOS_REF_HXX_
#	include <vos/ref.hxx>
#endif

#ifdef _USE_NAMESPACE
namespace vos
{
#endif


/////////////////////////////////////////////////////////////////////////////
//
// Interface IDaemonManager.
//	This class defines the interface to a manager, which can control other 
//	processes as daemons. The implemetation has to store informations 
//	of installed daemons in a persistent repository to remember 
//	which processes are to start as daemons.
//   	Instances of this interface are hold by reference (ORef - class).
//

class IDaemonManager : public IReference
{
public:

	/** Error codes returns by methods of this interface.
	*/
	enum TDaemonManagerError
	{
		TManagerError_NoError			= osl_Daemon_ManagerError_NoError,
		TManagerError_NotInstalled		= osl_Daemon_ManagerError_NotInstalled,
		TManagerError_AllreadyInstalled = osl_Daemon_ManagerError_AllreadyInstalled,
		TManagerError_AllreadyRunning	= osl_Daemon_ManagerError_AllreadyRunning,
		TManagerError_NotRunning		= osl_Daemon_ManagerError_NotRunning,
		TManagerError_NotPaused			= osl_Daemon_ManagerError_NotPaused,
		TManagerError_NoManager			= osl_Daemon_ManagerError_NoManager,
		TManagerError_Timeout			= osl_Daemon_ManagerError_Timeout,
		TManagerError_Security			= osl_Daemon_ManagerError_Security,
		TManagerError_CircularDep		= osl_Daemon_ManagerError_CircularDep,
		TManagerError_Unknown			= osl_Daemon_ManagerError_Unknown
	};

	/** Destructor.
		Does nothing - only implemented to call destructors of derived classes.
	*/
	virtual ~IDaemonManager() { }

	/** Add a listener to listen on status changes of all registered daemons.
		The listener is called if any daemon changes his actual status or if
		addiitional daemons are installed or daemons are uninstalled.
	*/
	virtual void SAL_CALL addListener(const ORef<IDaemonListener>& rListener) = 0;

	/** Removes a added listener.
	*/
	virtual void SAL_CALL removeListener(const ORef<IDaemonListener>& rListener) = 0;

	/** Query if a daemon with denoted identifer is installed.
	*/
	virtual sal_Bool SAL_CALL isInstalled(const NAMESPACE_RTL(OUString)& identifier) = 0;

	/** Uninstall the daemon with denoted identifer.
		Erases all informations of this daemon of the repository.
	*/
	virtual TDaemonManagerError	SAL_CALL uninstall(const NAMESPACE_RTL(OUString)& identifier) = 0;

	/** Install the calling process as a new daemon.
		Writes the informations of this daemon in the repository. Do not start the daemon.
		If path is not NULL, register the denoted file and not the calling process.
	*/
	virtual TDaemonManagerError	SAL_CALL install(const ORef<IDaemonAccess>& rDaemonAccess, const NAMESPACE_RTL(OUString)& ustrPath ) = 0;

	/** Start the calling process as denoted daemon.
		This method gives control of the calling thread to the daemon manager and returns
		not until the daemon has been stopped. The daemon itself is called while executing 
		this method via the callback methods of the IDaemonControl interface.
	*/
	virtual TDaemonManagerError	SAL_CALL execute(const ORef<IDaemonAccess>& rDaemonAccess,
										const ORef<IDaemonControl>& rDaemonControl) = 0;

	/** Start the calling process as a set of denoted daemons.
		This method gives control of the calling thread to the daemon manager and returns
		not until all daemon have been stopped. The daemons itselfs are called while executing 
		this method via the callback methods of the IDaemonControl interfaces.
	*/
	virtual TDaemonManagerError	SAL_CALL execute(ORef<IDaemonAccess>* prDaemonAccess,
										ORef<IDaemonControl>* prDaemonControls,
										sal_uInt32 numOfControls) = 0;

	/** Get the number of installed daemons.
	*/
	virtual sal_uInt32 SAL_CALL getDaemonCount() const = 0;

	/** Get an access interface to daemon by index.
	*/
	virtual ORef<IDaemonAccess> SAL_CALL getDaemonByIndex(sal_uInt32 index) const = 0;

	/** Get an access interface to daemon by identifier.
	*/
	virtual ORef<IDaemonAccess> SAL_CALL getDaemonByIdentifier(const NAMESPACE_RTL(OUString)& identifier) const = 0;

	/** Start the given daemon.
	*/
	virtual TDaemonManagerError	SAL_CALL startDaemon(const ORef<IDaemonAccess>& rDaemonAccess) = 0;

	/** Stop the given daemon.
	*/
	virtual TDaemonManagerError	SAL_CALL stopDaemon(const ORef<IDaemonAccess>& rDaemonAccess) = 0;

	/** Pause the given daemon.
	*/
	virtual TDaemonManagerError	SAL_CALL pauseDaemon(const ORef<IDaemonAccess>& rDaemonAccess) = 0;

	/** Resume the given daemon.
	*/
	virtual TDaemonManagerError	SAL_CALL resumeDaemon(const ORef<IDaemonAccess>& rDaemonAccess) = 0;

	/** Return a interface to the profile section of the denoted daemon.
		The pointer will be valid until closeProfile() is called.
	*/
	virtual	IUProfile* SAL_CALL openProfile(const NAMESPACE_RTL(OUString)& identifier) = 0;

	/** Close the interface to a profile section.
		The pointer will become invalid.
	*/
	virtual void SAL_CALL closeProfile(IUProfile* pProfile) = 0;

	/** Return a interface to the only instance of the daemon manager.
	*/
	static ORef<IDaemonManager> SAL_CALL getGlobalManager();
};


#ifdef _USE_NAMESPACE
}
#endif

#endif // _VOS_DAEMON_HXX_


