/*************************************************************************
 *
 *  $RCSfile: idaemon.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _VOS_IDAEMON_HXX_
#define _VOS_IDAEMON_HXX_

#ifndef _VOS_TYPES_HXX_
#	include <vos/types.hxx>
#endif
#ifndef _VOS_REFERNCE_HXX_
#	include <vos/refernce.hxx>
#endif
#ifndef _VOS_REF_HXX_
#	include <vos/ref.hxx>
#endif

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

#ifndef _OSL_DAEMON_H_
#	include <osl/daemon.h>
#endif

#ifdef _USE_NAMESPACE
namespace vos
{
#endif

class ODaemonProperty_Impl;

/** class ODaemonProperty.
	Objects of this class represents one daemon property.
	A property has a type, a name and a value; valid types are sal_Bool,
	unsigned long, string an list of this types. 
	The user of this class has only use the get/set-methods, wich match the
	actual type of a property and the actual list size. The get/set-methods 
	are robust against type mismatch, that means the wrong get method 
	returns a default value, the wrong set methods is ignored.
*/

class ODaemonProperty
{
public:

	/** enum TDaemonPropertyType.
		Denotes the type of the property.
	*/
	enum TDaemonPropertyType
	{
		TPropertyType_Boolean 		= osl_Daemon_PropertyType_Boolean,
		TPropertyType_BooleanList 	= osl_Daemon_PropertyType_BooleanList,
		TPropertyType_uLong			= osl_Daemon_PropertyType_uLong,
		TPropertyType_uLongList		= osl_Daemon_PropertyType_uLongList,
		TPropertyType_String		= osl_Daemon_PropertyType_String,
		TPropertyType_StringList	= osl_Daemon_PropertyType_StringList,
		TPropertyType_Invalid		= osl_Daemon_PropertyType_Invalid
	};

	/** default constructor.
		Creates an property with type TPropertyType_Invalid, 
		empty name and empty value.
	*/
	ODaemonProperty();

	/** Constructor.
		Creates a property with the given type, name and list size.
		If type is not a list type, the parameter listSize is ignored.
	*/
	ODaemonProperty(const NAMESPACE_RTL(OUString)& name, TDaemonPropertyType type, sal_uInt32 listSize);

	/** Constructor.
		Creates a property with type TPropertyType_Boolean, 
		the given name and givern value.
	*/
	ODaemonProperty(const NAMESPACE_RTL(OUString)& name, sal_Bool aBool);

	/** Constructor.
		Creates a property with type TPropertyType_uLong, 
		the given name and given value.
	*/
	ODaemonProperty(const NAMESPACE_RTL(OUString)& name, sal_uInt32 aULong);

	/** Constructor.
		Creates a property with type TPropertyType_String, 
		the given name and givern value.
	*/
	ODaemonProperty(const NAMESPACE_RTL(OUString)&name, const NAMESPACE_RTL(OUString)& aString);

	/** Copy constructor.
		Creates a copy of the given property.
	*/
	ODaemonProperty(const ODaemonProperty& rProperty);

	/** Destructor.
	*/
	~ODaemonProperty();

	/** Assignment operator.
		Creates a copy of the given property.
	*/
	ODaemonProperty& SAL_CALL operator=(const ODaemonProperty& rProperty);

	/** Get the name of the property, which was set by constructor.
	 */
	NAMESPACE_RTL(OUString)& SAL_CALL getName() const;

	/** Get the type of the property, wich was set by constructor.
	*/
	TDaemonPropertyType SAL_CALL getType() const;

	/** Get the actual list size of the property.
		If the type is not a list type returns 0.
	*/
	sal_uInt32 SAL_CALL getListSize() const;

	/** Set the actual list size of the property.
		If the type is not a list type this call is ignored.
	*/
	void SAL_CALL setListSize(sal_uInt32 size);

	/** Returns 
		- either the sal_Bool value if the type is TPropertyType_Boolean 
		- or the sal_Bool value on position index if the type is 
		  TPropertyType_BooleanList and 0 <= index < getListSize().
		- Otherwise returns the default value 0.
	*/
	sal_Bool SAL_CALL getBoolean(sal_uInt32 index = 0);

	/** Returns 
		- either the unsigned long value if the type is TPropertyType_uLong 
		- or the unsigned long value on position index if the type is 
		  TPropertyType_uLongList and 0 <= index < getListSize().
		- Otherwise returns the default value 0.
	*/
	sal_uInt32 SAL_CALL getULong(sal_uInt32 index = 0);

	/** Returns 
		- either the String value if the type is TPropertyType_String
		- or the String value on position index if the type is 
		  TPropertyType_StringList and 0 <= index < getListSize().
		- Otherwise returns the default value 0.
	*/
	const NAMESPACE_RTL(OUString) SAL_CALL getString(sal_uInt32 index = 0);

	/** Sets
		- either the sal_Bool value if the type is TPropertyType_Boolean 
		- or the sal_Bool value on position index if the type is 
		  TPropertyType_BooleanList and 0 <= index < getListSize().
		- Otherwise the call is ignored.
	*/
	void SAL_CALL setBoolean(sal_Bool value, sal_uInt32 index = 0);

	/** Sets 
		- either the unsigned long value if the type is TPropertyType_uLong 
		- or the unsigned long value on position index if the type is 
		  TPropertyType_uLongList and 0 <= index < getListSize().
		- Otherwise the call is ignored.
	*/
	void SAL_CALL setULong(sal_uInt32 value, sal_uInt32 index = 0);

	/** Sets 
		- either the String value if the type is TPropertyType_String
		- or the String value on position index if the type is 
		  TPropertyType_StringList and 0 <= index < getListSize().
		- Otherwise the call is ignored.
	*/
	void SAL_CALL setString(const NAMESPACE_RTL(OUString)& value, sal_uInt32 index = 0);

protected:

	ODaemonProperty_Impl* m_pImpl;	
};

/** Interface IDaemonAccess.
	This interface has to be implemented by a concrete daemom class. 
	It is used by the manager and clients of the manager to access properties and 
	status informations of the deamon.
   	Instances of this interface are hold by reference (ORef - class).
*/
class IDaemonAccess : public IReference
{
public:	

	/** Destructor.
		Does nothing - only implemented to call destructors of derived classes.
	*/
	virtual ~IDaemonAccess() { }

	/** Enum TDaemonStatus.
		Represents the actual runtime status of o daemon.
	*/
	enum TDaemonStatus
	{
		TStatus_StartPending = osl_Daemon_StatusStartPending,
		TStatus_StopPending  = osl_Daemon_StatusStopPending,
		TStatus_PausePending = osl_Daemon_StatusPausePending,
		TStatus_Running      = osl_Daemon_StatusRunning,
		TStatus_Stopped      = osl_Daemon_StatusStopped,
		TStatus_Paused       = osl_Daemon_StatusPaused,
		TStatus_Unknown      = osl_Daemon_StatusUnknown
	};

	/** Enum TDaemonStartupMode.
		Denotes if a daemon has to be started by the daemon manager 
		automatic or not.
	*/
	enum TDaemonStartupMode
	{
		TStartupMode_Automatic 	= osl_Daemon_Automatic,
		TStartupMode_OnDemand 	= osl_Daemon_OnDemand,
		TStartupMode_Unknown
	};

	/** Get the unique identifier of the daemon.
	*/
	virtual const NAMESPACE_RTL(OUString)& SAL_CALL getIdentifier() const = 0;

	/** Get the name to display of the daemon.
	*/
	virtual const NAMESPACE_RTL(OUString)& SAL_CALL getDisplayName() const = 0;

	/** Get the manufactor to display of the daemon.
	*/
	virtual const NAMESPACE_RTL(OUString)& SAL_CALL getManufactor() const = 0;

	/** Get the identifiers of daemons on wich this daemon is depend on.
		Returns a list of identifier seperated by '\0' finalized by '\0''\0'.
		On start up of this daemon all depend daemons are started too.
	*/
	virtual const NAMESPACE_RTL(OUString)& SAL_CALL getDependencies() const = 0;

	/** Get the version information of the daemon.
	*/
	virtual sal_uInt32 SAL_CALL getVersion() const = 0;

	/** Get the actual runtime status of the daemon.
	*/
	virtual TDaemonStatus SAL_CALL getStatus() const = 0;

	/** Get the start up mode of the daemon.
	*/
	virtual TDaemonStartupMode SAL_CALL getMode() const = 0;

	/** Get the number of supported properties of the daemon.
	*/
	virtual sal_uInt32 SAL_CALL getPropertyCount() const = 0;

	/** Get the actual properties of the daemon.
	*/
	virtual sal_uInt32 SAL_CALL getProperties(ODaemonProperty* 	pProperties, 
				  			    sal_uInt32 				count) const = 0;

	/** Set the given properties.
		Returns true if the daemon accept the properties, otherwise False.
		If the daemon does not accept the properties, a detailed error message 
		is returned in the parameter errorText. 
	*/
	virtual sal_Bool SAL_CALL setProperties(ODaemonProperty* 		pProperties, 
                                   sal_uInt32 				count,
                                   sal_uInt32&				errorCode,
                                   NAMESPACE_RTL(OUString)&	ustrErrorText) = 0;
};

/** Interface IDaemonListener.
	This interface can be register at daemons and the manager to listen on 
	status events of the running deamons.
   	Instances of this interface are hold by reference (ORef - class).
*/
class IDaemonListener : public IReference
{
public:
	/** Destructor.
		Does nothing - only implemented to call destructors of derived classes.
	*/
	virtual ~IDaemonListener() { }

	/** Callback.
		Called if the actual status of a daemon changes.
		This method has to return immediately.
	*/
	virtual void SAL_CALL statusChanged(const ORef<IDaemonAccess>& rDaemon, 
							   IDaemonAccess::TDaemonStatus status) = 0;
};

/** Interface IDaemonControl.
	This interface has to be implemented by a concrete daemom class. 
	It is only used by the manager and to control (e.g. start and stop) the daemon.
	All command handler methods has to work like this pattern:
	- call the listener that the the command has recieved 
	  (e.g. set status = TStatus_StartPending)
	- do any internal work to start executing this command
	- return immediately (do not wait for completion of command)
	The daemon has to announce the completion of te command by calling the listener
	(e.g. set status = TStatus_Running)
   	Instances of this interface are hold by reference (ORef - class).
*/
class IDaemonControl : public IReference
{
public:

	/** Destructor.
		Does nothing - only implemented to call destructors of derived classes.
	*/
	virtual ~IDaemonControl() { }

	/** Set a listener to listen on status changes of this daemon.
	*/
	virtual void SAL_CALL setListener(const ORef<IDaemonListener>& rListener) = 0;

 	/** Command handler method which is called if the daemon has to start. 
	*/
	virtual void SAL_CALL onStart() = 0;

 	/** Command handler method which is called if the daemon has to stop. 
	*/
	virtual void SAL_CALL onStop() = 0;

 	/** Command handler method which is called if the daemon has to pause. 
	*/
	virtual void SAL_CALL onPause() = 0;

 	/** Command handler method which is called if the daemon has to resume. 
	*/
	virtual void SAL_CALL onContinue() = 0;

 	/** Command handler method which is called if the daemon has to shutdown. 
	*/
	virtual void SAL_CALL onShutdown() = 0;
};


#ifdef _USE_NAMESPACE
}
#endif

#endif // _VOS_DAEMON_HXX_

