/*************************************************************************
 *
 *  $RCSfile: ports.hxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _VOS_PORTS_HXX_
#define _VOS_PORTS_HXX_

#ifndef _VOS_TYPES_HXX_
#	include <vos/types.hxx>
#endif
#ifndef _VOS_OBJECT_HXX_
#	include <vos/object.hxx>
#endif
#ifndef _VOS_ISTREAM_HXX_
#	include <vos/istream.hxx>
#endif
#ifndef _VOS_REF_HXX_
#   include <vos/ref.hxx>
#endif
#ifndef _RTL_USTRING_
#   include <rtl/ustring>
#endif
#ifndef _OSL_PORTS_H_
#	include <osl/ports.h>
#endif

#include <stl/vector>

#ifdef _USE_NAMESPACE
namespace vos
{
#endif


class IPortEnum: public IReference
{
public:
	virtual sal_Bool SAL_CALL getNextPortName( NAMESPACE_RTL(OUString)& strPortName ) = 0;
    virtual void SAL_CALL reset() = 0;
};

typedef NAMESPACE_VOS(ORef) < IPortEnum > OPortEnumRef;

// ---------------------------------------------------------------------------

/**
	Note:
		All functions with the returntype TPortError (or sal_Bool)
		will return an E_NotAvailable (or False), if the port isn't 
		open or	the function isn't supported.
*/

class OPortImpl;

class OPort :	public NAMESPACE_VOS(OObject),
				public NAMESPACE_VOS(IStream)
{
	VOS_DECLARE_CLASSINFO(VOS_NAMESPACE(OPort, vos));

	OPort( const OPort & );
	OPort SAL_CALL operator=( const OPort & );

protected:

	OPortImpl*			m_pPortData;
	
public:

	enum TPortError
	{
		E_None					=	osl_Port_E_None,			
		E_NotAvailable			=	osl_Port_E_NotAvailable,
		E_PortNotFound			=	osl_Port_E_PortNotFound,
		E_InvalidHandle			=  	osl_Port_E_InvalidHandle,
		E_DifferentStructSize	= 	osl_Port_E_DifferentStructSize,
		E_InvalidType			=  	osl_Port_E_InvalidType,
		E_InvalidArgument		=  	osl_Port_E_InvalidArgument,
		E_NoMemory				= 	osl_Port_E_NoMemory,		
		E_Unknown				= 	osl_Port_E_Unknown
	};

public:	
	typedef sal_uInt32			TPortType;
	typedef sal_uInt16			TAccessMode;

public:
						OPort();
	virtual				~OPort();


	/** The open function opens a port for a connection.
		@param pPortName [in]
			The name of the port, like COM1 (win) or /dev/term/a (unx). 
			This must be a null terminated string.
		@return Error value.
	*/
	TPortError SAL_CALL open( const NAMESPACE_RTL(OUString)& strPortName );

	/** 
		The close function closes an open port and frees all resources.
	*/
	TPortError SAL_CALL close();

	/** The isOpen function returns TRUE if the port is open, otherwise FALSE.
	*/
	sal_Bool SAL_CALL isOpen() const;

	/** The isOpen function returns TRUE if the port supports interval timeout, 
		which means the timeout between the arrival of two byte. If interval timeout
		is supported, the set or get timeouts can be used.
	*/
	sal_Bool SAL_CALL isIntervalTimeoutSupported() const;

	/** The isOpen function returns TRUE if the port supports total timeout, 
	*/
	sal_Bool SAL_CALL isTotalTimeoutSupported() const;

	/** The getType function returns Type of the port:
		This value can be one of TPortType constants
	*/	
	TPortType SAL_CALL getType();

	/** The getTimeout function returns the time-out parameters for all read 
		and write operations on a specified communications device. 

		@param ReadIntervalTimeout;
			Specifies the maximum time, in milliseconds, allowed to elapse between the arrival 
			of two characters on the communications line. During a ReadFile operation, the time 
			period begins when the first character is received. If the interval between the arrival 
			of any two characters exceeds this amount, the ReadFile operation is completed and any 
			buffered data is returned. A value of zero indicates that interval time-outs are not used. 
			A value of MAXDWORD, combined with zero values for both the ReadTotalTimeoutConstant
			and ReadTotalTimeoutMultiplier members, specifies that the read operation is to return
			immediately with the characters that have already been received, even if no characters
			have been received. 
		
		@param ReadTotalTimeoutMultiplier; 
			Specifies the multiplier, in milliseconds, used to calculate the total 
			time-out period for read operations. For each read operation, this value is 
			multiplied by the requested number of bytes to be read. 
			
		@param ReadTotalTimeoutConstant;
			Specifies the constant, in milliseconds, used to calculate the total time-out 
			period for read operations. For each read operation, this value is added to the 
			product of the ReadTotalTimeoutMultiplier member and the requested number of bytes. 
			A value of zero for both the ReadTotalTimeoutMultiplier and ReadTotalTimeoutConstant
			members indicates that total time-outs are not used for read operations. 
		@param WriteTotalTimeoutMultiplier; 
			Specifies the multiplier, in milliseconds, used to calculate the total time-out 
			period for write operations. For each write operation, this value is multiplied by 
			the number of bytes to be written.
		
	    @param WriteTotalTimeoutConstant; 
			Specifies the constant, in milliseconds, used to calculate the total time-out period
			for write operations. For each write operation, this value is added to the product of 
			the WriteTotalTimeoutMultiplier member and the number of bytes to be written. 
			A value of zero for both the WriteTotalTimeoutMultiplier and WriteTotalTimeoutConstant
			members indicates that total time-outs are not used for write operations. 
	*/
	TPortError	SAL_CALL getTimeout( 
								sal_uInt32 &ReadIntervalTimeout,
								sal_uInt32 &ReadTotalTimeoutMultiplier,
								sal_uInt32 &ReadTotalTimeoutConstant,
								sal_uInt32 &WriteTotalTimeoutMultiplier,
								sal_uInt32 &WriteTotalTimeoutConstant
								);

	/** The setPortTimeout function set the time-out parameters for all read 
		and write operations on a specified communications device. The meaning of the
		parameters are described in getTimeout();
	*/
	TPortError	SAL_CALL setTimeout( 
								sal_uInt32 ReadIntervalTimeout ,
								sal_uInt32 ReadTotalTimeoutMultiplier,
								sal_uInt32 ReadTotalTimeoutConstant,
								sal_uInt32 WriteTotalTimeoutMultiplier,
								sal_uInt32 WriteTotalTimeoutConstant
								);
						
	TPortError	SAL_CALL setReadIntervalTimeout(			sal_uInt32 ReadIntervalTimeout );
	TPortError	SAL_CALL setReadTotalTimeoutMultiplier(	sal_uInt32 ReadTotalTimeoutMultiplier );
	TPortError	SAL_CALL setReadTotalTimeoutConstant(	sal_uInt32 ReadTotalTimeoutConstant );
	TPortError	SAL_CALL setWriteTotalTimeoutMultiplier( sal_uInt32 WriteTotalTimeoutMultiplier );
	TPortError	SAL_CALL setWriteTotalTimeoutConstant(	sal_uInt32 WriteTotalTimeoutConstant );
						
	TPortError	SAL_CALL getReadIntervalTimeout(			sal_uInt32 &ReadIntervalTimeout );
	TPortError	SAL_CALL getReadTotalTimeoutMultiplier(	sal_uInt32 &ReadTotalTimeoutMultiplier );
	TPortError	SAL_CALL getReadTotalTimeoutConstant(	sal_uInt32 &ReadTotalTimeoutConstant );
	TPortError	SAL_CALL getWriteTotalTimeoutMultiplier( sal_uInt32 &WriteTotalTimeoutMultiplier );
	TPortError	SAL_CALL getWriteTotalTimeoutConstant(	sal_uInt32 &WriteTotalTimeoutConstant );
						
						
	/** The function read tries to fill the contents of pBuffer synchronous from the port.
		The function will block until nNumberOfBytesToRead bytes received, or if a timeout is set
		(with setPortTimeout) until the timeout has reached. If no timeout is set, the function
		may be block forever.

		@return The number of bytes received.
	*/
	virtual	sal_Int32 SAL_CALL read( void* pBuffer, sal_uInt32 nNumberOfBytesToRead ) const;
						
	/** The function write tries to send the contents of pBuffer synchronous to the port.
		The function will block until all bytes are transmitted, or if a timeout is set
		(with setTimeout) until the timeout has reached. If no timeout is set, the function
		may be block forever.

		@return The number of bytes written.
	*/
	virtual	sal_Int32 SAL_CALL write( const void* pBuffer, sal_uInt32 nNumberOfBytesToWrite );

	/**	The function isEof is the implementation from the IStream interface. 
		If the Port is open, this function will always return FALSE. Otherwise TRUE.

	*/
	virtual	sal_Bool SAL_CALL isEof() const;

	/** The transmitChar function transmits a specified character ahead of 
		any pending data in the output buffer of the specified communications device.
		This function is useful for sending an interrupt character (such as a ctrl+c) to a host system. 
	 */
	TPortError SAL_CALL transmitChar( sal_Char cChar );


public:
	/**	This class memberfunction returns the port type of the associated port name.
		For Example win32: getPortTypeByName( "COM1" ) returns OPort::Rs232
	*/
	static TPortType SAL_CALL getPortTypeByName( const NAMESPACE_RTL(OUString)& strName );

    /** This class memberfunction returns a port enumeration interface.
     */
    static OPortEnumRef SAL_CALL enumPortNames();

	/**	This class memberfunction returns a vector of port names, which are
		currently available in the system.

        @param rPortNameVec [out] vector of stings filled with the available port-names.
    */
	static TPortError SAL_CALL getPortNames( NAMESPACE_STD(vector) < NAMESPACE_RTL(OUString) >& rPortNameVec );

public:
	static const TPortType		Rs232;
	static const TPortType		Parallel;
	static const TPortType		Unknown;
	
};



struct SPortState
{
	/**	This struct is usefull, if the OSerialPort::getAndClearError() function
		is used. The OSerialPort::getAndClearError() will use this struct to write
		the current state of the serial port. Note that this is the only way to recieve
		information about the state of a serial port.
	*/
	sal_uInt32	Size;

    sal_Bool CTSHold;
		/** Specifies whether transmission is waiting for the CTS (clear-to-send) signal
			to be sent.	If this member is TRUE, transmission is waiting. */
    sal_Bool DSRHold;
		/** Specifies whether transmission is waiting for the DSR (data-set-ready) 
			signal to be sent. If this member is TRUE, transmission is waiting. */
    sal_Bool RLSDHold;
		/**	Specifies whether transmission is waiting for the RLSD 
			(receive-line-signal-detect) signal to be sent. If this member is TRUE,
			transmission is waiting. */
    sal_Bool XOFFHold;
		/** Specifies whether transmission is waiting because the XOFF character 
			was received. If this member is TRUE, transmission is waiting. **/
    sal_Bool XOFFSent;
		/**	Specifies whether transmission is waiting because the XOFF character was
			transmitted. If this member is TRUE, transmission is waiting. Transmission 
			halts when the XOFF character is transmitted to a system that takes the 
			next character as XON, regardless of the actual character. */
	sal_Bool Txim;
		/** If this member is TRUE, there is a character queued for transmission that
			has come to the communications device by way of the oslTransmitPortChar 
			function. The communications device transmits such a character ahead of other 
			characters in the device's output buffer. 
		*/
    sal_Bool Eof;
		/**	Specifies whether the end-of-file (EOF) character has been received. If this
			member is TRUE, the EOF character has been received. */
	sal_uInt32 BytesInQueue;
		/** Specifies the number of bytes received by the serial provider but not yet 
			read by a recieve operation. */
    sal_uInt32 BytesOutQueue;
		/**	Specifies the number of bytes of user data remaining to be transmitted 
			for all write operations. */
};


class ISerialPortListener;
class OSerialPortImpl;

class OSerialPort : public NAMESPACE_VOS(OPort)
{
	VOS_DECLARE_CLASSINFO(VOS_NAMESPACE(OSerialPort, vos));

protected:
	
	OSerialPortImpl*		m_pSerialData;

private:

					OSerialPort( const OSerialPort & );
	OSerialPort	SAL_CALL operator=( const OSerialPort & );

public:
	typedef 	sal_uInt16	TDatabit;
	typedef 	sal_uInt16	TStopbit;
	typedef 	sal_uInt16	TParityMode;
	typedef 	sal_uInt32	TBaudRate;
	typedef 	sal_uInt16	TPurgeOption;
	typedef 	sal_uInt16	TEvent;
	typedef 	sal_uInt16	TError;
	typedef		sal_uInt8	TDtrControl;
	typedef		sal_uInt8	TRtsControl;

public:
					OSerialPort();
	virtual			~OSerialPort();

public:
	// Listener
	virtual void	SAL_CALL notifyListener( sal_uInt32 nEventMask );	//only for Callback use

	sal_Bool		SAL_CALL setListener( ISerialPortListener *pPortListener);
	void			SAL_CALL removeListener();
	
	TPortError		SAL_CALL filterEvents( TEvent nEventMask );

public:

	// overwrite memberfunction from OPort
	TPortError		SAL_CALL open( const NAMESPACE_RTL(OUString)& pPortName );

public:


	TPortError		SAL_CALL setBreak();
	/** The osl_setPortBreak function suspends character transmission for a 
		specified communications device and places the transmission line in a 
		break state until the ClearCommBreak function is called. 
	*/
	TPortError		SAL_CALL clearBreak();
	/** The clearBreak function restores character transmission for a 
		specified communications device and places the transmission line 
		in a nonbreak state.	
	*/
	TPortError		SAL_CALL getAndClearError( TError &nErrorMask, SPortState *pPortState = NULL); 
	/** The getAndClearError retrieves information about a communications error and
		reports the current status of a communications device. The function is called
		when a communications error occurs, and it clears the device's error flag to enable
		additional input and output (I/O) operations. 
	
		@param pErrorMask [out]
			Specifies the error that occurred. This parameter can be a combination of 
			the OSerialPort::TError constants.

		@param pPortState	[out]
			Pointer to a SPortState structure in which the device's status information is returned.
			If pPortState is NULL, no status information is returned. 
	*/
	TPortError		SAL_CALL purge( TPurgeOption aPurgeMask );
	/** The purge function can discard all characters from the output or input 
		buffer of a specified communications resource. It can also terminate pending 
		read or write operations on the resource. 

		@param nFlags [in]
			Specifies the action to take. This parameter can be a combination of 
			the OSerialPort::TPurgeOption constants.
	*/

	// most useable properties
	
	sal_Bool	SAL_CALL isParityCheckSupported() const;
	sal_Bool	SAL_CALL isBinaryModeSetable() const;
	sal_Bool	SAL_CALL isStopbitsSetable() const;
	sal_Bool	SAL_CALL isParityCheckSetable() const;
	sal_Bool	SAL_CALL isParityModeSetable() const;
	sal_Bool	SAL_CALL isDatabitsSetable() const;
	sal_Bool	SAL_CALL isBaudRateSetable() const;
	sal_Bool	SAL_CALL isHandshakingSetable() const;

	TBaudRate	SAL_CALL getMaxBaudRate() const;
	/** return one of the OSerialPort::TBaudRate constants. 
	*/
	TBaudRate	SAL_CALL getSetableBaudRate() const;
	/** return a bitmask value of OSerialPort::TBaudRate constants, which 
		specifies supported baudrates.
	*/
	TDatabit	SAL_CALL getSetableDatabits() const;
	/** return a bitmask value of OSerialPort::TDatabit constants, which 
		specifies supported databits.
	*/
	TStopbit	SAL_CALL getSetableStopbits() const;
	/** return a bitmask value of OSerialPort::TStopbit constants, which 
		specifies supported stopbits.
	*/
	TParityMode	SAL_CALL getSetableParityMode() const;
	/** return a bitmask value of OSerialPort::TParityMode constants, which 
		specifies supported paritymodes.
	*/
	sal_uInt32	SAL_CALL getBaudRate() const;
	/** Specifies the baud rate at which the communications device operates.
		Note that this is NOT a TBaudRate constant, but a real value like 9600 baud.
	*/
	TPortError	SAL_CALL setBaudRate( sal_uInt32 nBaudrate );
	/** Set the baud rate at which the communications device operates.
		Note that this is NOT a TBaudRate constant, but a real value like 9600 baud.
	*/
	sal_Bool	SAL_CALL isBinaryModeEnabled() const;
	/** Specifies whether binary mode is enabled. For Win32 this can only be TRUE */
	
	TPortError	SAL_CALL enableBinaryMode( sal_Bool bEnable );
	/** Enables the binary mode, if bEnable is TRUE. Win32 only operates in Binary Mode*/
	
	sal_Bool	SAL_CALL isParityCheckingEnabled() const;
	/** returns TRUE, if parity checking is enabled. */

	TPortError	SAL_CALL enableParityChecking( sal_Bool bEnable );
	/** Enables the parity checking in the current selected mode, 
		if bEnable is TRUE.
	*/
	TStopbit	SAL_CALL getStopbits() const;
	/**	returns the one of the OSerialPort::TStopbit constants, which specifies the
		number of stopbits used for the connection.
	*/
	TPortError	SAL_CALL setStopbits( TStopbit aStopbit );
	/**	sets one of the OSerialPort::TStopbit constants, which specifies the
		number of stopbits used for the connection.
	*/
	TParityMode	SAL_CALL getParityMode() const;
	/**	returns the one of the OSerialPort::TParityMode constants, which specifies the
		number of stopbits used for the connection.
	*/
	TPortError	SAL_CALL setParityMode( TParityMode aParityMode );
	/**	sets one of the OSerialPort::TParityMode constants, which specifies the
		parity mode that will be used , if parity checking is enabled.
	*/
	sal_uInt8	SAL_CALL getDatabits() const;
	TPortError	SAL_CALL setDatabits( sal_uInt8 nDatabitSize );

public:
	// extended properties
	
	sal_Bool	SAL_CALL isDtrDsrSupported() const;
	sal_Bool	SAL_CALL isRtsCtsSupported() const;
	sal_Bool	SAL_CALL isRlsdSupported() const;
	sal_Bool	SAL_CALL isXonXoffSupported() const;
	sal_Bool	SAL_CALL isXCharSetable() const;

	sal_uInt32	SAL_CALL getTxQueueMaxSize() const;
	/** Specifies the maximum size, in bytes, of the driver's internal output buffer. 
		A value of zero indicates that no maximum value is imposed by the serial provider. 
	*/
	sal_uInt32	SAL_CALL getRxQueueMaxSize() const;
	/** Specifies the maximum size, in bytes, of the driver's internal input buffer. 
		A value of zero indicates that no maximum value is imposed by the serial provider. 
	*/
	sal_uInt32	SAL_CALL getTxQueueCurrentSize() const;
	/** Specifies the current size, in bytes, of the driver's internal output buffer. 
		A value of zero indicates that no maximum value is imposed by the serial provider. 
	*/
	sal_uInt32	SAL_CALL getRxQueueCurrentSize() const;
	/** Specifies the current size, in bytes, of the driver's internal input buffer. 
		A value of zero indicates that no maximum value is imposed by the serial provider. 
	*/
	sal_Bool	SAL_CALL isOutCtsFlowControlEnabled() const;
	/** returnd whether the CTS (clear-to-send) signal is monitored for output flow control. 
		If this value is TRUE and CTS is turned off, output is suspended until CTS is sent again. 
	*/
	TPortError	SAL_CALL enableOutCtsFlowControl( sal_Bool bEnable );
	/** Enables the CTS (clear-to-send) signal.*/

	sal_Bool	SAL_CALL isOutDsrFlowControlEnabled() const;
	/** Specifies whether the DSR (data-set-ready) signal is monitored for output flow control. 
		If this value is TRUE and DSR is turned off, output is suspended until DSR is sent again. */
	TPortError	SAL_CALL enableOutDsrFlowControl( sal_Bool bEnable );
	/** Enables the DSR (data-set-ready) signal. */

	TDtrControl	SAL_CALL getDtrControlMode() const;
	/** returns the DTR (data-terminal-ready) flow control mode. 
		This value can be one of OSerialPort::TDtrControl constants. 
	*/
	TPortError	SAL_CALL setDtrControlMode( TDtrControl aDtrControlMode );
	/** sets the DTR (data-terminal-ready) flow control mode. 
		This value can be one of OSerialPort::TDtrControl constants. 
	*/
	sal_Bool	SAL_CALL isDtrSensitivityEnabled() const;
	/** returns TRUE if the communications driver is sensitive to the state of the DSR signal. 
		If this value is TRUE, the driver ignores any bytes received,
		unless the DSR modem input line is high.
	*/ 
	TPortError	SAL_CALL enableDtrSensitivity( sal_Bool bEnable );
	/** Enables the reaction to of the communications driver to sensitive, if bEnable is TRUE*/ 

	sal_Bool	SAL_CALL isTxContinueOnXoffEnabled() const;
	/** Specifies whether transmission stops when the input buffer is full and the driver has 
		transmitted the XoffChar character. If this value is TRUE, transmission continues after 
		the input buffer has come within XoffLim bytes of being full and the driver has transmitted 
		the XoffChar character to stop receiving bytes. If this member is FALSE, transmission does 
		not continue until the input buffer is within XonLim bytes of being empty and the driver has 
		transmitted the XonChar character to resume reception. 
	*/
	TPortError	SAL_CALL enableTxContinueOnXoff( sal_Bool bEnable );

	sal_Bool	SAL_CALL isOutXonXoffEnabled() const;
	/** Specifies whether XON/XOFF flow control is used during transmission. If this value is TRUE,
		transmission stops when the XoffChar character is received and starts again when the XonChar 
		character is received. 
	*/
	TPortError	SAL_CALL enableOutXonXoff( sal_Bool bEnable );
	/** Enables the XonXoff for output control. */

	sal_Bool	SAL_CALL isInXonXoffEnabled() const;
	/** Specifies whether XON/XOFF flow control is used during reception. If this member is TRUE, 
		the XoffChar character is sent when the input buffer comes within XoffLim bytes of being full, 
		and the XonChar character is sent when the input buffer comes within XonLim bytes of being empty.
	*/
	TPortError	SAL_CALL enableInXonXoff( sal_Bool bEnable );
	/** Enables the XonXoff for input control. */

	sal_Bool	SAL_CALL isErrorCharReplacementEnabled() const;
	/** Specifies whether bytes received with parity errors are replaced with the character 
		specified by the ErrorChar value. If this value is TRUE and the 
		parity checking is enabled, replacement occurs. 
	*/
	TPortError	SAL_CALL enableErrorCharReplacement( sal_Bool bEnable );
	/** Enables the error sal_Char replacement, if bEnable if TRUE */

	sal_Bool	SAL_CALL isNullByteDiscardEnabled() const;
	/** Specifies whether null bytes are discarded. If this value is TRUE, 
		null bytes are discarded when received. 
	*/
	TPortError	SAL_CALL enableNullByteDiscard( sal_Bool bEnable );
	/** Enables the null bytes discard, if bEnable is TRUE. */
	
	sal_Bool	SAL_CALL isHoldOnErrorEnabled()  const;
	/** Specifies whether read and write operations are terminated if an error occurs.
		If this value is TRUE, the driver terminates all read and write operations with an error 
		status if an error occurs. The driver will not accept any further communications operations 
		until the application has acknowledged the error by calling the getAndClearError() function. 
	*/
	TPortError	SAL_CALL enableHoldOnError( sal_Bool bEnable );
	/** Enables the hold on error, if bEnable is TRUE. */

	TRtsControl	SAL_CALL getRtsControlMode() const;
	/** Specifies the RTS (request-to-send) flow control. 
		If this value is zero, the default is OSerialPort::RtsControl_Handshake. 
		This value can be one of OSerialPort::TRtsControl constants. 
	*/
	TPortError	SAL_CALL setRtsControlMode( TRtsControl aRtsControlMode );
	/** Sets  the RTS (request-to-send) flow control. 
		This value can be one of OSerialPort::TRtsControl constants. 
	*/
	sal_Char	SAL_CALL getErrorChar() const;
	/** returns the error sal_Char, which specifies the value of the 
		character used to replace bytes received with a parity error. 
	*/
	TPortError	SAL_CALL setErrorChar( sal_Char cChar );
	/** sets the error sal_Char, which specifies the value of the 
		character used to replace bytes received with a parity error. 
	*/
	sal_uInt32	SAL_CALL getXonLimit() const;
	/** returns the XOnLimit, which specifies the minimum number of bytes
		allowed in the input buffer before the XON character is sent. 
	*/
	TPortError	SAL_CALL setXonLimit( sal_uInt32 aNumberOfBytes );
	/** sets the XOnLimit, which specifies the minimum number of bytes
		allowed in the input buffer before the XON character is sent. 
	*/
	sal_uInt32	SAL_CALL getXoffLimit() const;
	/** returns the XoffLimitSpecifies the maximum number of bytes allowed in the input buffer 
		before the XOFF character is sent. The maximum number of bytes allowed is calculated 
		by subtracting this value from the size, in bytes, of the input buffer. 
	*/
	TPortError	SAL_CALL setXoffLimit( sal_uInt32 aNumberOfBytes );
	/** sets the XoffLimit */
	sal_Char	SAL_CALL getXonChar() const;
	/** returns the XonChar, which specifies the value of the XON character 
		for both transmission and reception. 
	*/
	TPortError	SAL_CALL setXonChar( sal_Char cChar );
	/** sets the XonChar, which specifies the value of the XON character 
		for both transmission and reception. 
	*/
	sal_Char	SAL_CALL getXoffChar() const;
	/** returns the XoffChar, which specifies the value of the XOFF 
		character for both transmission and reception. 
	*/
	TPortError	SAL_CALL setXoffChar( sal_Char cChar );
	/** sets the XoffChar, which specifies the value of the XOFF 
		character for both transmission and reception. 
	*/
	sal_Char	SAL_CALL getEOFChar() const;
	/** returns the EOFChar, which specifies the value of the character 
		used to signal the end of data. 
	*/
	TPortError	SAL_CALL setEOFChar( sal_Char cChar );
	/** sets the EOFChar, which specifies the value of the character 
		used to signal the end of data. 
	*/
	sal_Char	SAL_CALL getEventChar() const;
	/** returns the EventChar, which specifies the value 
		of the character used to signal an event. 
	*/
	TPortError	SAL_CALL setEventChar( sal_Char cChar );
	/** sets the EventChar, which specifies the value 
		of the character used to signal an event. 
	*/

public:

	// ----------- TDatabit --------------------

	static const TDatabit		Databits_5;
	static const TDatabit		Databits_6;
	static const TDatabit		Databits_7;
	static const TDatabit		Databits_8;
	static const TDatabit		Databits_16;
	static const TDatabit		Databits_16x;
								
	// ----------- TStopbit --------------------
	
	static const TStopbit		Stopbits_10;
	static const TStopbit		Stopbits_15;
	static const TStopbit		Stopbits_20;

	// ----------- TParityMode --------------------

	static const TParityMode	Parity_None;
	static const TParityMode	Parity_Odd;
	static const TParityMode	Parity_Even;
	static const TParityMode	Parity_Mark;
	static const TParityMode	Parity_Space;
	

	// ----------- TBaudRate --------------------

	static const TBaudRate		Baud_075;
	static const TBaudRate		Baud_110;
	static const TBaudRate		Baud_134_5;
	static const TBaudRate		Baud_150;
	static const TBaudRate		Baud_300;
	static const TBaudRate		Baud_600;
	static const TBaudRate		Baud_1200;
	static const TBaudRate		Baud_1800;
	static const TBaudRate		Baud_2400;
	static const TBaudRate		Baud_4800;
	static const TBaudRate		Baud_7200;
	static const TBaudRate		Baud_9600;
	static const TBaudRate		Baud_14400;
	static const TBaudRate		Baud_19200;
	static const TBaudRate		Baud_38400;
	static const TBaudRate		Baud_56k;
	static const TBaudRate		Baud_128k;
	static const TBaudRate		Baud_115200;
	static const TBaudRate		Baud_57600;
	static const TBaudRate		Baud_User;
	/** varriant settings possible */

	// ----------- TPurgeOption --------------------
	
	static const TPurgeOption	Purge_TxAbort;
	/** Terminates all outstanding overlapped write operations and  
		returns immediately, even if the write operations have not been completed. 	*/
	static const TPurgeOption	Purge_RxAbort;
	/** Terminates all outstanding overlapped read operations and returns immediately, 
		even if the read operations have not been completed. */
	static const TPurgeOption	Purge_TxClear;
	/** Clears the output buffer (if the device driver has one). */
	static const TPurgeOption	Purge_RxClear;
	/** Clears the input buffer (if the device driver has one).  */

	// ----------- TEvent --------------------

	static const TEvent			Event_RxChar;
	/** A character was received and placed in the input buffer. */
	static const TEvent			Event_RxFlag;
	/**	The event character was received and placed in the 
		input buffer. The event character is specified with
		get/setEventChar().
	*/
	static const TEvent			Event_TxEmpty;
	/** The last character in the output buffer was sent. */
	static const TEvent			Event_Cts;
	/** The CTS (clear-to-send) signal changed state. */
	static const TEvent			Event_Dsr;
	/** The DSR (data-set-ready) signal changed state. */
	static const TEvent			Event_Rlsd;
	/** The RLSD (receive-line-signal-detect) signal changed state. */
	static const TEvent			Event_Break;
	/** A break was detected on input. */
	static const TEvent			Event_Error;
	/** An error occurded. To find the cause of the error, 
		call getAndClearError().*/					 
	static const TEvent			Event_Ring;
	/** A ring indicator was detected. */
	static const TEvent			Event_RX80Percent;
	/** The receive buffer is 80 percent full. */

	// ----------- TError --------------------
	
	static const TError			Error_None;
	static const TError			Error_RxOverflow;
	/** An input buffer overflow has occurred. 
		There is either no room in the input buffer, 
		or a character was received after the end-of-file (EOF) character. 	*/
	static const TError			Error_Overrun;
	/** A character-buffer overrun has occurred. The next character is lost.  */
	static const TError			Error_RxParity;
	/** The hardware detected a parity error.  */
	static const TError			Error_Frame;
	/** The hardware detected a framing error.  */
	static const TError			Error_TxFull;
	/** The application tried to transmit a character, 
		but the output buffer was full.	*/
	static const TError			Error_Io;
	/** An I/O error occurred during communications with the device.  */
	static const TError			Error_Unknown;
	/** An unmapped error occurred. */

	// ----------- TDtrControl --------------------

	static const TDtrControl	DtrControl_Disable;
	/* Disables the DTR line when the device is opened and leaves it disabled. */
	static const TDtrControl	DtrControl_Enable;
	/* Enables the DTR line when the device is opened and leaves it on. */ 
	static const TDtrControl	DtrControl_Handshake;
	/* Enables DTR handshaking */

	// ----------- TRtsControl --------------------
	
	static const TRtsControl	RtsControl_Disable;
	/** Disables the RTS line when the device is opened and leaves it disabled.*/
	static const TRtsControl	RtsControl_Enable;
	/** Enables the RTS line when the device is opened and leaves it on */
	static const TRtsControl	RtsControl_Handshake;
	/** Enables RTS handshaking. */
	static const TRtsControl	RtsControl_Toggle;
	/** Specifies that the RTS line will be high if bytes are available 
		for transmission. After all buffered bytes have been sent, 
		the RTS line will be low. */


};


class ISerialPortListener
{


public:

	virtual	void SAL_CALL event( OSerialPort::TEvent nEventMask ) = 0;
};


#ifdef WNT 
	/*	in win32 muss die initialisierung hier erfolgen, da es nicht moeglich ist
		auf Daten in einer andeen dll zu zugreifen.
	*/
	const OPort::TPortType	OPort::Rs232		=	OSL_PORT_TYPE_RS232;
	const OPort::TPortType	OPort::Parallel		=	OSL_PORT_TYPE_PARALLELPORT; 
	const OPort::TPortType	OPort::Unknown		=	OSL_PORT_TYPE_UNKNOWN;


	const OSerialPort::TDatabit	OSerialPort::Databits_5		=	OSL_PORT_DATABITS_5;
	const OSerialPort::TDatabit	OSerialPort::Databits_6		=	OSL_PORT_DATABITS_6;
	const OSerialPort::TDatabit	OSerialPort::Databits_7		=	OSL_PORT_DATABITS_7;
	const OSerialPort::TDatabit	OSerialPort::Databits_8		=	OSL_PORT_DATABITS_8;
	const OSerialPort::TDatabit	OSerialPort::Databits_16	=	OSL_PORT_DATABITS_16;
	const OSerialPort::TDatabit	OSerialPort::Databits_16x	=	OSL_PORT_DATABITS_16X;


	const OSerialPort::TStopbit	OSerialPort::Stopbits_10	=	OSL_PORT_STOPBITS_10;
	const OSerialPort::TStopbit	OSerialPort::Stopbits_15	=	OSL_PORT_STOPBITS_15;
	const OSerialPort::TStopbit	OSerialPort::Stopbits_20	=	OSL_PORT_STOPBITS_20;

	const OSerialPort::TParityMode	OSerialPort::Parity_None	=	OSL_PORT_PARITY_NONE;
	const OSerialPort::TParityMode	OSerialPort::Parity_Odd		=	OSL_PORT_PARITY_ODD;
	const OSerialPort::TParityMode	OSerialPort::Parity_Even	=	OSL_PORT_PARITY_EVEN;
	const OSerialPort::TParityMode	OSerialPort::Parity_Mark	=	OSL_PORT_PARITY_MARK;
	const OSerialPort::TParityMode	OSerialPort::Parity_Space	=	OSL_PORT_PARITY_SPACE;


	const OSerialPort::TBaudRate	OSerialPort::Baud_075		=	OSL_PORT_BAUD_075;
	const OSerialPort::TBaudRate	OSerialPort::Baud_110		=	OSL_PORT_BAUD_110;
	const OSerialPort::TBaudRate	OSerialPort::Baud_134_5		=	OSL_PORT_BAUD_134_5;
	const OSerialPort::TBaudRate	OSerialPort::Baud_150		=	OSL_PORT_BAUD_150;
	const OSerialPort::TBaudRate	OSerialPort::Baud_300		=	OSL_PORT_BAUD_300;
	const OSerialPort::TBaudRate	OSerialPort::Baud_600		=	OSL_PORT_BAUD_600;
	const OSerialPort::TBaudRate	OSerialPort::Baud_1200		=	OSL_PORT_BAUD_1200;
	const OSerialPort::TBaudRate	OSerialPort::Baud_1800		=	OSL_PORT_BAUD_1800;
	const OSerialPort::TBaudRate	OSerialPort::Baud_2400		=	OSL_PORT_BAUD_2400;
	const OSerialPort::TBaudRate	OSerialPort::Baud_4800		=	OSL_PORT_BAUD_4800;
	const OSerialPort::TBaudRate	OSerialPort::Baud_7200		=	OSL_PORT_BAUD_7200;
	const OSerialPort::TBaudRate	OSerialPort::Baud_9600		=	OSL_PORT_BAUD_9600;
	const OSerialPort::TBaudRate	OSerialPort::Baud_14400		=	OSL_PORT_BAUD_14400;
	const OSerialPort::TBaudRate	OSerialPort::Baud_19200		=	OSL_PORT_BAUD_19200;
	const OSerialPort::TBaudRate	OSerialPort::Baud_38400		=	OSL_PORT_BAUD_38400;
	const OSerialPort::TBaudRate	OSerialPort::Baud_56k		=	OSL_PORT_BAUD_56000;
	const OSerialPort::TBaudRate	OSerialPort::Baud_128k		=	OSL_PORT_BAUD_128000;
	const OSerialPort::TBaudRate	OSerialPort::Baud_115200	=	OSL_PORT_BAUD_115200;
	const OSerialPort::TBaudRate	OSerialPort::Baud_57600		=	OSL_PORT_BAUD_57600;
	const OSerialPort::TBaudRate	OSerialPort::Baud_User		=	OSL_PORT_BAUD_USER;


	const OSerialPort::TPurgeOption	OSerialPort::Purge_TxAbort	=	OSL_PORT_PURGE_TXABORT;
	const OSerialPort::TPurgeOption	OSerialPort::Purge_RxAbort	=	OSL_PORT_PURGE_RXABORT;
	const OSerialPort::TPurgeOption	OSerialPort::Purge_TxClear	=	OSL_PORT_PURGE_TXCLEAR;
	const OSerialPort::TPurgeOption	OSerialPort::Purge_RxClear	=	OSL_PORT_PURGE_TXCLEAR;

	const OSerialPort::TEvent	OSerialPort::Event_RxChar	=	OSL_PORT_EV_RXCHAR;
	const OSerialPort::TEvent	OSerialPort::Event_RxFlag	=	OSL_PORT_EV_RXFLAG;
	const OSerialPort::TEvent	OSerialPort::Event_TxEmpty	=	OSL_PORT_EV_TXEMPTY;
	const OSerialPort::TEvent	OSerialPort::Event_Cts		=	OSL_PORT_EV_CTS;
	const OSerialPort::TEvent	OSerialPort::Event_Dsr		=	OSL_PORT_EV_DSR;
	const OSerialPort::TEvent	OSerialPort::Event_Rlsd		=	OSL_PORT_EV_RLSD;
	const OSerialPort::TEvent	OSerialPort::Event_Break	=	OSL_PORT_EV_BREAK;
	const OSerialPort::TEvent	OSerialPort::Event_Error	=	OSL_PORT_EV_ERR;
	const OSerialPort::TEvent	OSerialPort::Event_Ring		=	OSL_PORT_EV_RING;
	const OSerialPort::TEvent	OSerialPort::Event_RX80Percent	=	OSL_PORT_EV_RX80FULL;


	const OSerialPort::TError	OSerialPort::Error_None			=	OSL_PORT_E_NONE;
	const OSerialPort::TError	OSerialPort::Error_RxOverflow	=	OSL_PORT_E_RXOVER;
	const OSerialPort::TError	OSerialPort::Error_Overrun		=	OSL_PORT_E_OVERRUN;
	const OSerialPort::TError	OSerialPort::Error_RxParity		=	OSL_PORT_E_RXPARITY;
	const OSerialPort::TError	OSerialPort::Error_Frame		=	OSL_PORT_E_FRAME;
	const OSerialPort::TError	OSerialPort::Error_TxFull		=	OSL_PORT_E_TXFULL;
	const OSerialPort::TError	OSerialPort::Error_Io			=	OSL_PORT_E_IOE;
	const OSerialPort::TError	OSerialPort::Error_Unknown		=	OSL_PORT_E_UNKNOWN;

	const OSerialPort::TDtrControl	OSerialPort::DtrControl_Disable	=	OSL_PORT_DTR_CONTROL_DISABLE;
	const OSerialPort::TDtrControl	OSerialPort::DtrControl_Enable	=	OSL_PORT_DTR_CONTROL_ENABLE;
	const OSerialPort::TDtrControl	OSerialPort::DtrControl_Handshake=	OSL_PORT_DTR_CONTROL_HANDSHAKE;


	const OSerialPort::TRtsControl	OSerialPort::RtsControl_Disable	=	OSL_PORT_RTS_CONTROL_DISABLE;
	const OSerialPort::TRtsControl	OSerialPort::RtsControl_Enable	=	OSL_PORT_RTS_CONTROL_ENABLE;
	const OSerialPort::TRtsControl	OSerialPort::RtsControl_Handshake=	OSL_PORT_RTS_CONTROL_HANDSHAKE;
	const OSerialPort::TRtsControl	OSerialPort::RtsControl_Toggle	=	OSL_PORT_RTS_CONTROL_TOGGLE;

#endif //WNT 

#ifdef _USE_NAMESPACE
}
#endif

#endif // _VOS_PORTS_HXX_

