/*************************************************************************
 *
 *  $RCSfile: channel.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#ifndef _VOS_CHANNEL_HXX_
#	include <vos/channel.hxx>
#endif

#ifdef _USE_NAMESPACE
using namespace vos;
using namespace rtl;
#endif


VOS_IMPLEMENT_CLASSINFO(VOS_CLASSNAME(OChannel, vos), 
						VOS_NAMESPACE(OChannel, vos), 
						VOS_NAMESPACE(OObject, vos), 0);

OChannel::OChannel()
	: m_pResource(NULL)
{
}	

ORef<OChannel> OChannel::createChannel(OObject& rResource)
{
	ORef<OChannel> ret;
	
 	if (rResource.isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OPipe, vos))))
	{
		ret = new OChannel();
		ret->m_pResource = new OStreamPipe((OPipe&)rResource);
	}		
	else if (rResource.isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OSocket, vos))))
	{
		ret = new OChannel();
		ret->m_pResource = new OStreamSocket((OSocket&)rResource);
	}

	return ret;	
}	


OChannel::~OChannel()
{
	if (m_pResource != NULL)
	{
		delete m_pResource;		
	}	
}	

IStream* OChannel::getStream()
{
	IStream* pStream = NULL;		
	
	if (m_pResource != NULL)
	{
		if (m_pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OStreamPipe, vos))))
		{
			pStream = (OStreamPipe*)m_pResource;
		}		
		else if (m_pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OStreamSocket, vos))))
		{
			pStream = (OStreamSocket*)m_pResource;
		}
	}
	
	return pStream;
}	
	
OObject* OChannel::getResource()
{
	return m_pResource;	
}	

void OChannel::close()
{
	if (m_pResource != NULL)
	{
		if (m_pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OStreamPipe, vos))))
		{
			((OStreamPipe*)m_pResource)->close();
		}		
		else if (m_pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OStreamSocket, vos))))
		{
			((OStreamSocket*)m_pResource)->close();
		}
	}
}	



VOS_IMPLEMENT_CLASSINFO(VOS_CLASSNAME(OChannelBroker, vos), 
						VOS_NAMESPACE(OChannelBroker, vos), 
						VOS_NAMESPACE(OObject, vos), 0);

ORef<OChannelBroker> OChannelBroker::getGlobalChannelBroker()
{
	static ORef<OChannelBroker> globalManager = new OChannelBroker();
	
	return globalManager;
}	

OChannelBroker::OChannelBroker()
{
	m_hBroker = osl_channelConnectBroker();
}	

OChannelBroker::~OChannelBroker()
{
	if (m_hBroker != NULL)
		osl_channelDisconnectBroker(m_hBroker);
}	

OChannelBroker::TError OChannelBroker::provide
(
	ORef<OChannel>& rChannel, 
	NAMESPACE_RTL(OUString)& 	resourceName, 
	const TimeValue* pTimeout
)
{
	OChannelBroker::TError error = OChannelBroker::NoManager;

	if (m_hBroker != NULL)
	{
		error = OChannelBroker::UnsupportedResourceClass;
		
		if (rChannel.isValid())
		{
			OObject* pResource = rChannel->getResource();
		
			if (pResource != NULL)
			{
				if (pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OPipe, vos))))
				{
					oslChannelResource Resource;
					
					Resource.Pipe = (oslPipe)(*(OPipe*)pResource);
					
					error = (OChannelBroker::TError)osl_channelProvide(
														m_hBroker,
														Resource, 
														osl_Channel_Type_Pipe,
														resourceName.pData, 
														NULL,
														NULL,
														pTimeout);
				}		
				else if (pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OSocket, vos))))
				{
					oslChannelResource Resource;
					
					Resource.Socket = (oslSocket)(*(OSocket*)pResource);

					error = (OChannelBroker::TError)osl_channelProvide(
														m_hBroker,
														Resource, 
														osl_Channel_Type_Socket,
														resourceName.pData, 
														NULL,
														NULL,
														pTimeout);
				}
			}
		}		
	}
	
	return error;
}

OChannelBroker::TError OChannelBroker::provide
(
	ORef<OChannel>& rChannel, 
	NAMESPACE_RTL(OUString)& 		resourceName, 
	NAMESPACE_RTL(OUString)& 		userIdent, 
	NAMESPACE_RTL(OUString)& 		password, 
	const TimeValue* pTimeout
)
{
	OChannelBroker::TError error = OChannelBroker::NoManager;

	if (m_hBroker != NULL)
	{
		error = OChannelBroker::UnsupportedResourceClass;
		
		if (rChannel.isValid())
		{
			OObject* pResource = rChannel->getResource();
		
			if (pResource != NULL)
			{
				if (pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OPipe, vos))))
				{
					oslChannelResource Resource;
					
					Resource.Pipe = (oslPipe)(*(OPipe*)pResource);
					
					error = (OChannelBroker::TError)osl_channelProvide(
														m_hBroker,
														Resource, 
														osl_Channel_Type_Pipe,
														resourceName.pData, 
														userIdent.pData,
														password.pData,
														pTimeout);
				}		
				else if (pResource->isKindOf(VOS_CLASSINFO(VOS_NAMESPACE(OSocket, vos))))
				{
					oslChannelResource Resource;
					
					Resource.Socket = (oslSocket)(*(OSocket*)pResource);

					error = (OChannelBroker::TError)osl_channelProvide(
														m_hBroker,
														Resource, 
														osl_Channel_Type_Socket,
														resourceName.pData, 
														userIdent.pData,
														password.pData,
														pTimeout);
				}
			}
		}		
	}
	
	return error;
}


static sal_Bool SAL_CALL checkAuthentication
(
	void* pData,
	rtl_uString* pUserIdent, 
	rtl_uString* pPassword
)
{
	IAuthenticationCheck* pAutoCheck = (IAuthenticationCheck*)pData;
	NAMESPACE_RTL(OUString) userIdent( pUserIdent );
	NAMESPACE_RTL(OUString) password( pPassword );
	
	return pAutoCheck->check(userIdent, password);
}	

OChannelBroker::TError OChannelBroker::accept
(
	ORef<OChannel>& 		  rChannel, 
	NAMESPACE_RTL(OUString)&  resourceName, 
  	IAuthenticationCheck*	  pAuthCheck,
	const TimeValue* 		  pTimeout
)
{
	OChannelBroker::TError error = OChannelBroker::NoManager;

	if (m_hBroker != NULL)
	{
		oslChannelResource 	Resource;
		oslChannelType		Type;					

		CheckAuthentication_F check_f = NULL;

		if (pAuthCheck)
			check_f = checkAuthentication;
			
		error = (OChannelBroker::TError)osl_channelAccept(
												m_hBroker,
												&Resource, 
												&Type,
												resourceName.pData, 
												check_f,
												pAuthCheck,
												pTimeout);

		if (error == OChannelBroker::NoError)
		{
			if (Type == osl_Channel_Type_Socket)
			{
				OSocket socket(Resource.Socket);

				rChannel = OChannel::createChannel(socket);
			}			
			else if (Type == osl_Channel_Type_Socket)
			{
				OPipe pipe(Resource.Pipe);
				
				rChannel = OChannel::createChannel(pipe);
			}			
			else
				error = OChannelBroker::UnsupportedResourceClass;
		}
	}
	
	return error;
}

