/*************************************************************************
 *
 *  $RCSfile: daemmgr.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/


#include <sal/config.h>

#include <map>
#include <list>

#include <vos/diagnose.hxx>
#include <vos/object.hxx>
#include <vos/idaemon.hxx>
#include <vos/idaemmgr.hxx>
#include <vos/mutex.hxx>
#include <vos/process.hxx>
#include <vos/timer.hxx>
#include <vos/security.hxx>
#include <rtl/alloc.h>
#include <rtl/ustrbuf.h>
#include <osl/file.hxx>

#ifdef _USE_NAMESPACE
using namespace vos;
using namespace std;
using namespace rtl;
#endif

static void SAL_CALL _IDaemonControl_ControlFunction(void* pthis, oslDaemonCommand control)
{
    IDaemonControl* pThis= (IDaemonControl*)pthis;

    switch (control)
    {
        case osl_Daemon_CommandStart:
            pThis->onStart();
            break;

        case osl_Daemon_CommandStop:
            pThis->onStop();
            break;

        case osl_Daemon_CommandPause:
            pThis->onPause();
            break;

        case osl_Daemon_CommandContinue:
            pThis->onContinue();
            break;

        case osl_Daemon_CommandShutdown:
            pThis->onShutdown();
            break;
        default:
            break;
    }
}

static oslDaemonManagerError SAL_CALL _IDaemonAccess_GetPropCount(void* pthis, sal_uInt32* pCount)
{
    IDaemonAccess* pThis= (IDaemonAccess*)pthis;

    *pCount = pThis->getPropertyCount();

    return osl_Daemon_ManagerError_NoError;
}

static oslDaemonManagerError SAL_CALL _IDaemonAccess_GetProps(void* pthis, oslDaemonProperty** ppProp, sal_uInt32* pCount)
{
    IDaemonAccess* pThis= (IDaemonAccess*)pthis;
    sal_uInt32 i, j;

    *pCount = pThis->getPropertyCount();

    if (*pCount > 0)
    {
        ODaemonProperty* pVOSProps = new ODaemonProperty[*pCount];

        *pCount = pThis->getProperties(pVOSProps, *pCount);
        
        if (*pCount > 0)
        {
            *ppProp = osl_createDaemonProperties(*pCount);
            
            for (i = 0; i < *pCount; i++)
            {
                osl_initDaemonProperty(
                    &((*ppProp)[i]),
                    (oslDaemonPropertyType)pVOSProps[i].getType(),
                    pVOSProps[i].getName().pData,
                    pVOSProps[i].getListSize());
                
                switch (pVOSProps[i].getType())
                {
                    case ODaemonProperty::TPropertyType_Boolean:
                        (*ppProp)[i].Value.aBool = pVOSProps[i].getBoolean();
                        break;
                    case ODaemonProperty::TPropertyType_uLong:
                        (*ppProp)[i].Value.aULong = pVOSProps[i].getULong();
                        break;
                    case ODaemonProperty::TPropertyType_String:
                    {
                        const OUString& rDataString = pVOSProps[i].getString();
                        if ( rDataString.pData != NULL )
                        {
                            (*ppProp)[i].Value.aString = (sal_Unicode*) rtl_allocateMemory(rDataString.getLength() * sizeof(sal_Unicode));
                            memcpy( (*ppProp)[i].Value.aString, pVOSProps[i].getString().pData->buffer,rDataString.getLength() * sizeof(sal_Unicode));
                        }
                        else
                        {
                            (*ppProp)[i].Value.aString = 0;
                        }
                    }
                        break;
                    case ODaemonProperty::TPropertyType_BooleanList:
                        for (j = 0; j < pVOSProps[i].getListSize(); j++)
                        {
                            (*ppProp)[i].Value.aList.Data.pBool[j] = pVOSProps[i].getBoolean(j);
                        }
                        break;
                    case ODaemonProperty::TPropertyType_uLongList:
                        for (j = 0; j < pVOSProps[i].getListSize(); j++)
                        {
                            (*ppProp)[i].Value.aList.Data.pULong[j] = pVOSProps[i].getULong(j);
                        }
                        break;
                    case ODaemonProperty::TPropertyType_StringList:
                    {
                        (*ppProp)[i].Value.aList.Count = pVOSProps[i].getListSize();
                        (*ppProp)[i].Value.aList.Data.pString = (sal_Unicode**)rtl_allocateMemory(pVOSProps[i].getListSize() * sizeof(sal_Unicode*));
                        
                        for (j = 0; j < pVOSProps[i].getListSize(); j++)
                        {
                            (*ppProp)[i].Value.aList.Data.pString[j]
                            = (sal_Unicode*) rtl_allocateMemory( pVOSProps[i].getString(j).getLength() * sizeof(sal_Unicode));
                             memcpy( (*ppProp)[i].Value.aList.Data.pString[j] , pVOSProps[i].getString(j).pData->buffer , pVOSProps[i].getString(j).getLength() * sizeof(sal_Unicode));
                        }
                    }
                    
                        break;
                    default:
                        break;
                }
            }
        }
        else
        {
            *ppProp = NULL;
        }
        
        delete[] pVOSProps;
    }
    else
    {
        *ppProp = NULL;
    }
    
    return osl_Daemon_ManagerError_NoError;
}


static oslDaemonManagerError SAL_CALL _IDaemonAccess_SetProps(void* pthis, oslDaemonProperty* pProps, sal_uInt32 count, sal_uInt32* pErrorCode, rtl_uString** errorBuffer )
{
    oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;
    IDaemonAccess* pThis= (IDaemonAccess*)pthis;
    sal_uInt32 i, j;
    OUString errorText;
    
    if (count > 0)
    {
        ODaemonProperty* pVOSProps = new ODaemonProperty[count];

        for (i = 0; i < count; i++)
        {
            pVOSProps[i] =
                ODaemonProperty(
                    *((OUString*)pProps[i].Name) ,
                    (ODaemonProperty::TDaemonPropertyType)pProps[i].Type,
                    0);
            
            switch (pVOSProps[i].getType())
            {
                case ODaemonProperty::TPropertyType_Boolean:
                    pVOSProps[i].setBoolean(pProps[i].Value.aBool);
                    break;
                case ODaemonProperty::TPropertyType_uLong:
                    pVOSProps[i].setULong(pProps[i].Value.aULong);
                    break;
                case ODaemonProperty::TPropertyType_String:
                    pVOSProps[i].setString(pProps[i].Value.aString);
                    break;
                case ODaemonProperty::TPropertyType_BooleanList:
                    pVOSProps[i].setListSize(pProps[i].Value.aList.Count);
                    for (j = 0; j < pProps[i].Value.aList.Count; j++)
                    {
                        pVOSProps[i].setBoolean(pProps[i].Value.aList.Data.pBool[j], j);
                    }
                    break;
                case ODaemonProperty::TPropertyType_uLongList:
                    pVOSProps[i].setListSize(pProps[i].Value.aList.Count);
                    for (j = 0; j < pProps[i].Value.aList.Count; j++)
                    {
                        pVOSProps[i].setULong(pProps[i].Value.aList.Data.pULong[j], j);
                    }
                    break;
                case ODaemonProperty::TPropertyType_StringList:
                    pVOSProps[i].setListSize(pProps[i].Value.aList.Count);
                    for (j = 0; j < pProps[i].Value.aList.Count; j++)
                    {
                        pVOSProps[i].setString(pProps[i].Value.aList.Data.pString[j], j);
                    }
                    break;
                default:
                    break;
            }
        }

        if (!pThis->setProperties(pVOSProps, count, *pErrorCode, errorText))
            ret = osl_Daemon_ManagerError_Unknown;

/*        if (sizeOfBuffer > 0)
        {
        strncpy(errorBuffer, (const sal_Char*) errorText, sizeOfBuffer);
        errorBuffer[sizeOfBuffer - 1] = '\0';
        }
*/

        *errorBuffer = errorText.pData;
        rtl_uString_acquire( *errorBuffer );
        
        delete[] pVOSProps;
    }
    
    return ret;
}

static sal_Bool checkDaemonSecurity()
{
    static sal_Bool isInit = sal_False;
    static sal_Bool    ret = sal_False;
    
    if (!isInit)
    {
        OUString buffer;
        const OUString dummy = OUString::createFromAscii( "OSL_DAEMON_DEBUG_ENV" );
        
        if (
            ( osl_getEnvironment( dummy.pData,&buffer.pData ) == osl_Process_E_None ) &&
            ( buffer.compareToAscii( "user", ( sal_Int32 ) 4 ) == 0 ) )
        {
            ret = sal_True;
        }
        else
        {
            OSecurity sec;
            ret = sec.isAdministrator();
        }
        
        isInit = sal_True;
    }

    return ret;
}

/*************************************************************************

    class ODaemonAccess_Impl

*************************************************************************/

#ifdef _USE_NAMESPACE
namespace vos
{
#endif

class ODaemonAccess_Impl : public OObject
                         , public IDaemonAccess
{
    VOS_DECLARE_CLASSINFO(VOS_NAMESPACE(ODaemonAccess_Impl, vos));

public:
    ODaemonAccess_Impl(const OUString& identifier);
    ~ODaemonAccess_Impl();

    // implementation of IDaemonAccess
    virtual const OUString&		SAL_CALL getIdentifier() const;
    virtual const OUString&		SAL_CALL getDisplayName() const;
    virtual const OUString&		SAL_CALL getManufactor() const;
    virtual const OUString&		SAL_CALL getDependencies() const;
    virtual sal_uInt32			SAL_CALL getVersion() const;
    virtual TDaemonStatus		SAL_CALL getStatus() const;
    virtual TDaemonStartupMode	SAL_CALL getMode() const;
    virtual sal_uInt32			SAL_CALL getPropertyCount() const;
    virtual sal_uInt32			SAL_CALL getProperties(ODaemonProperty*  pProperties,
											sal_uInt32        count) const;
    virtual sal_Bool            SAL_CALL setProperties(ODaemonProperty*  pProperties,
                                              sal_uInt32        count,
                                              sal_uInt32&       rErrorCode,
                                              OUString&         errorText);

    // implementation of IReference
    virtual RefCount SAL_CALL acquire();
    virtual RefCount SAL_CALL release();
    virtual RefCount SAL_CALL referenced() const;

    void SAL_CALL update();
    
protected:
    OUString       m_identifier;
    oslDaemonInfo  m_info;
    ORefCount      m_refCount;
    OMutex         m_mutex;
};

#ifdef _USE_NAMESPACE
}
#endif

VOS_IMPLEMENT_CLASSINFO(VOS_CLASSNAME(ODaemonAccess_Impl, vos), VOS_NAMESPACE(ODaemonAccess_Impl, vos), VOS_NAMESPACE(OObject, vos), 0);

ODaemonAccess_Impl::ODaemonAccess_Impl(const OUString& identifier)
{
    memset(&m_info, 0, sizeof(m_info));
    m_info.Size = sizeof(m_info);
    
    osl_queryDaemonInfo(identifier.pData, &m_info);
    m_identifier = identifier;
}

ODaemonAccess_Impl::~ODaemonAccess_Impl()
{
}

const OUString& ODaemonAccess_Impl::getIdentifier() const
{
    OGuard guard((OMutex&)m_mutex);
    return m_identifier;
}

const OUString& ODaemonAccess_Impl::getDisplayName() const
{
    OGuard guard((OMutex&)m_mutex);
    return *((OUString*)m_info.DisplayName);
}

const OUString& ODaemonAccess_Impl::getManufactor() const
{
    OGuard guard((OMutex&)m_mutex);
    return *((OUString*)m_info.Manufactor);
}

const OUString& ODaemonAccess_Impl::getDependencies() const
{
    OGuard guard((OMutex&)m_mutex);
    OUString aDepend(m_info.Dependencies);
    
    return *((OUString*) aDepend.pData);
}

sal_uInt32 ODaemonAccess_Impl::getVersion() const
{
    OGuard guard((OMutex&)m_mutex);
    return m_info.Version;
}

IDaemonAccess::TDaemonStatus ODaemonAccess_Impl::getStatus() const
{
    OGuard guard((OMutex&)m_mutex);
    oslDaemonStatus status;
    
    if (
        osl_queryDaemonStatus(m_info.Identifier, &status) ==
        osl_Daemon_ManagerError_NoError
       )
        return (TDaemonStatus) status;
    else
        return TStatus_Unknown;
}

IDaemonAccess::TDaemonStartupMode ODaemonAccess_Impl::getMode() const
{
    OGuard guard((OMutex&)m_mutex);
    return (TDaemonStartupMode)m_info.Mode;
}


sal_uInt32 ODaemonAccess_Impl::getPropertyCount() const
{
    sal_uInt32 count = 0;
    
    if (osl_getDaemonPropertyCount(getIdentifier().pData, &count) == osl_Daemon_ManagerError_NoError)
    {
        return count;
    }
    else
        return 0;
}

sal_uInt32 ODaemonAccess_Impl::getProperties(ODaemonProperty*     pProperties,
                                          sal_uInt32                 count) const
{
    sal_uInt32 oslCount = 0;
    sal_uInt32 i, j;
    oslDaemonProperty* poslProps = NULL;
    
    if (osl_getDaemonProperties(getIdentifier().pData, &poslProps, &oslCount)
        ==
        osl_Daemon_ManagerError_NoError)
    {
        for (i = 0; i < VOS_MIN(count, oslCount); i++)
        {
            pProperties[i] =
                ODaemonProperty(
                    *((OUString*)( poslProps[i].Name )),
                    (ODaemonProperty::TDaemonPropertyType) poslProps[i].Type,
                    poslProps[i].Value.aList.Count);

            switch (poslProps[i].Type)
            {
                case osl_Daemon_PropertyType_Boolean:
                    pProperties[i].setBoolean(poslProps[i].Value.aBool);
                    break;
                case osl_Daemon_PropertyType_uLong:
                    pProperties[i].setULong(poslProps[i].Value.aULong);
                    break;
                case osl_Daemon_PropertyType_String:
                    pProperties[i].setString(poslProps[i].Value.aString);
                    break;
                case osl_Daemon_PropertyType_BooleanList:
                    for (j = 0; j < poslProps[i].Value.aList.Count; j++)
                    {
                        pProperties[i].setBoolean(poslProps[i].Value.aList.Data.pBool[j], j);
                    }
                    break;
                case osl_Daemon_PropertyType_uLongList:
                    for (j = 0; j < poslProps[i].Value.aList.Count; j++)
                    {
                        pProperties[i].setULong(poslProps[i].Value.aList.Data.pULong[j], j);
                    }
                    break;
                case osl_Daemon_PropertyType_StringList:
                    for (j = 0; j < poslProps[i].Value.aList.Count; j++)
                    {
                        pProperties[i].setString(poslProps[i].Value.aList.Data.pString[j], j);
                    }
                    break;
                default:
                    break;
            }
        }
        
        osl_destroyDaemonProperties(poslProps, oslCount);
    }
    
    return VOS_MIN(count, oslCount);
}


sal_Bool ODaemonAccess_Impl::setProperties(ODaemonProperty* pProperties,
                                           sal_uInt32       count,
                                           sal_uInt32&      rErrorCode,
                                           OUString&        errorText)
{
    oslDaemonManagerError ret = osl_Daemon_ManagerError_NoError;

    if (!checkDaemonSecurity())
    {
        errorText = OUString::createFromAscii( "security violation" );
        
        return sal_False;
    }
    
    if (count > 0)
    {
        sal_uInt32 i, j;
        oslDaemonProperty* poslProps = osl_createDaemonProperties(count);
        OUString errorBuffer;
        
        for (i = 0; i < count; i++)
        {
            osl_initDaemonProperty(
                &(poslProps[i]),
                (oslDaemonPropertyType) pProperties[i].getType(),
                pProperties[i].getName().pData,
                pProperties[i].getListSize());
            
            switch (poslProps[i].Type)
            {
                case osl_Daemon_PropertyType_Boolean:
                    poslProps[i].Value.aBool = pProperties[i].getBoolean();
                    break;
                case osl_Daemon_PropertyType_uLong:
                    poslProps[i].Value.aULong = pProperties[i].getULong();
                    break;
                case osl_Daemon_PropertyType_String:
                    {
                        const OUString& rDataString = pProperties[i].getString();
                        if ( rDataString.pData != NULL )
                        {
                            poslProps[i].Value.aString = (sal_Unicode*) rtl_allocateMemory(rDataString.getLength() * sizeof(sal_Unicode));
                            memcpy( poslProps[i].Value.aString, pProperties[i].getString().pData->buffer,rDataString.getLength() * sizeof(sal_Unicode));
                        }
                        else
                        {
                            poslProps[i].Value.aString = 0;
                        }
                    }
                    break;
                case osl_Daemon_PropertyType_BooleanList:
                    for (j = 0; j < poslProps[i].Value.aList.Count; j++)
                    {
                        poslProps[i].Value.aList.Data.pBool[j] = pProperties[i].getBoolean(j);
                    }
                    break;
                case osl_Daemon_PropertyType_uLongList:
                    for (j = 0; j < poslProps[i].Value.aList.Count; j++)
                    {
                        poslProps[i].Value.aList.Data.pULong[j] = pProperties[i].getULong(j);
                    }
                    break;
                case osl_Daemon_PropertyType_StringList:
                    {
                        poslProps[i].Value.aList.Count = pProperties[i].getListSize();
                        poslProps[i].Value.aList.Data.pString = (sal_Unicode**)rtl_allocateMemory(pProperties[i].getListSize() * sizeof(sal_Unicode*));
                        
                        for (j = 0; j < pProperties[i].getListSize(); j++)
                        {
                            poslProps[i].Value.aList.Data.pString[j]
                            = (sal_Unicode*) rtl_allocateMemory( pProperties[i].getString(j).getLength() * sizeof(sal_Unicode));
                            memcpy( poslProps[i].Value.aList.Data.pString[j] , pProperties[i].getString(j).pData->buffer , pProperties[i].getString(j).getLength() * sizeof(sal_Unicode));
                        }
                    }
// obr
//                    for (j = 0; j < poslProps[i].Value.aList.Count; j++)
//                    {
//                        poslProps[i].Value.aList.Data.pString[j] = strdup((const sal_Char*)pProperties[i].getString(j));
//                    }
                    break;
                default:
                    break;
            }
        }
        
        ret = osl_setDaemonProperties(getIdentifier().pData, poslProps, count, &rErrorCode, &errorBuffer.pData );
        
        errorText = errorBuffer;
        
        osl_destroyDaemonProperties(poslProps, count);
    }


    
    return (ret == osl_Daemon_ManagerError_NoError);
}

IReference::RefCount ODaemonAccess_Impl::acquire()
{
    return m_refCount.acquire();
}

IReference::RefCount ODaemonAccess_Impl::release()
{
    IReference::RefCount ret = m_refCount.release();
    
    if (ret == 0)
    {
        delete this;
    }
    
    return ret;
}

IReference::RefCount ODaemonAccess_Impl::referenced() const
{
    return m_refCount.referenced();
}

void ODaemonAccess_Impl::update()
{
    OGuard guard((OMutex&)m_mutex);
    osl_queryDaemonInfo(m_info.Identifier, &m_info);
}

/*************************************************************************

    class ODaemonProfile_Impl

*************************************************************************/

#ifdef _USE_NAMESPACE
namespace vos
{
#endif

class ODaemonProfile_Impl : public OObject
                          , public IUProfile
{

public:

    VOS_DECLARE_CLASSINFO(VOS_NAMESPACE(ODaemonProfile_Impl, vos));

    ODaemonProfile_Impl(const OUString& ustridentifier);
    ~ODaemonProfile_Impl();

    // implementation of IUProfile
    virtual sal_Bool    SAL_CALL readString(const OUString& ustrSection, const OUString& ustrEntry,
                                   OUString& ustrString, const OUString& ustrDefault);

    virtual sal_Bool    SAL_CALL readBool(const OUString& ustrSection, const OUString& ustrEntry,
                                 sal_Bool Default);

    virtual sal_uInt32  SAL_CALL readIdent(const OUString& ustrSection, const OUString& ustrEntry,
                                  sal_uInt32 FirstId, const OUString*& ustrStrings,
                                  sal_uInt32 Default);

    virtual sal_Bool    SAL_CALL writeString(const OUString& ustrSection, const OUString& ustrEntry, const OUString& ustrString);

    virtual sal_Bool    SAL_CALL writeBool(const OUString& ustrSection, const OUString& ustrEntry, sal_Bool Value);

    virtual sal_Bool    SAL_CALL writeIdent(const OUString& ustrSection, const OUString& ustrEntry,
                                sal_uInt32 FirstId, const OUString*& ustrStrings,
                                sal_uInt32 Value);

    virtual sal_Bool    SAL_CALL removeEntry(const OUString& ustrSection, const OUString& ustrEntry);

    virtual sal_Bool  SAL_CALL getSectionEntries(const OUString& ustrSection, OUString& ustrBuffer);

    virtual sal_Bool  SAL_CALL getSections(const OUString& ustrSection, OUString& ustrBuffer);



protected:

    OUString     m_identifier;
};

#ifdef _USE_NAMESPACE
}
#endif

VOS_IMPLEMENT_CLASSINFO(VOS_CLASSNAME(ODaemonProfile_Impl, vos), VOS_NAMESPACE(ODaemonProfile_Impl, vos), VOS_NAMESPACE(OObject, vos), 0);

ODaemonProfile_Impl::ODaemonProfile_Impl(const OUString& identifier)
    : m_identifier(identifier)
{
    
}

ODaemonProfile_Impl::~ODaemonProfile_Impl()
{
    
}

sal_Bool ODaemonProfile_Impl::readString(const OUString& ustrSection,
                                         const OUString& ustrEntry,
                                         OUString& ustrString,
                                         const OUString& ustrDefault)
{
    oslDaemonManagerError error =
        osl_readDaemonProfileString(m_identifier.pData,
                                      ustrSection.pData,
                                      ustrEntry.pData,
                                      &ustrString.pData,
                                      ustrDefault.pData);
    
    return (error == osl_Daemon_ManagerError_NoError);
}

sal_Bool ODaemonProfile_Impl::readBool(const OUString& ustrSection,
                                       const OUString& ustrEntry,
                                       sal_Bool Default)
{
    sal_Bool ret = Default;
    oslDaemonManagerError error =
        osl_readDaemonProfileBool(m_identifier.pData,
                                  ustrSection.pData,
                                  ustrEntry.pData,
                                  &ret,
                                  Default);

    if (error == osl_Daemon_ManagerError_NoError)
        return ret;
    else
        return Default;
}


sal_uInt32 ODaemonProfile_Impl::readIdent(const OUString& ustrSection,
                                        const OUString& ustrEntry,
                                        sal_uInt32 FirstId,
                                        const OUString *& ustrStrings,
                                        sal_uInt32 Default)
{
	sal_uInt32 ret = Default;

	oslDaemonManagerError error = osl_readDaemonProfileIdent(
		m_identifier.pData,	ustrSection.pData, ustrEntry.pData, &ret,
		FirstId, (rtl_uString **)&ustrStrings[0].pData, Default);

	if (error == osl_Daemon_ManagerError_NoError)
		return ret;
	else
		return Default;

}


sal_Bool ODaemonProfile_Impl::writeString(const OUString& ustrSection,
                                          const OUString& ustrEntry,
                                          const OUString& ustrString)
{
    oslDaemonManagerError error =
        osl_writeDaemonProfileString(m_identifier.pData,
                                     ustrSection.pData,
                                     ustrEntry.pData,
                                     ustrString.pData);
    
    return (error == osl_Daemon_ManagerError_NoError);
    
}

sal_Bool ODaemonProfile_Impl::writeBool(const OUString& ustrSection,
                                        const OUString& ustrEntry,
                                        sal_Bool Value)
{
    oslDaemonManagerError error =
        osl_writeDaemonProfileBool(m_identifier.pData,
                                   ustrSection.pData,
                                   ustrEntry.pData,
                                   Value);
    
    return (error == osl_Daemon_ManagerError_NoError);
}

sal_Bool ODaemonProfile_Impl::writeIdent(const OUString& ustrSection,
                                         const OUString& ustrEntry,
                                         sal_uInt32 FirstId,
                                         const OUString *& ustrStrings,
                                         sal_uInt32 Value)
{
    oslDaemonManagerError error =
        osl_writeDaemonProfileIdent(m_identifier.pData,
                                    ustrSection.pData,
                                    ustrEntry.pData,
                                    Value,
                                    FirstId,
                                    (rtl_uString **)&ustrStrings[0].pData );

    return (error == osl_Daemon_ManagerError_NoError);
}


sal_Bool ODaemonProfile_Impl::removeEntry(const OUString& ustrSection,
                                          const OUString& ustrEntry)
{
    oslDaemonManagerError error =
        osl_removeDaemonProfileEntry(m_identifier.pData,
                                     ustrSection.pData,
                                     ustrEntry.pData);
    
    return (error == osl_Daemon_ManagerError_NoError);
}

sal_Bool ODaemonProfile_Impl::getSectionEntries(const OUString& ustrSection,
                                                  OUString& ustrBuffer)
{
    sal_uInt32 requiredSize = 0;
    oslDaemonManagerError error =
        osl_getDaemonProfileEntries(m_identifier.pData,
                                    ustrSection.pData,
                                    ustrBuffer.pData->buffer,
                                    0,
                                    &requiredSize);
    
    if (error == osl_Daemon_ManagerError_NoError)
    {
		sal_Int32	nCapacity = 0;

		rtl_uStringbuffer_ensureCapacity(&ustrBuffer.pData, &nCapacity, requiredSize);

		error = osl_getDaemonProfileEntries(m_identifier.pData,
                                    ustrSection.pData,
                                    ustrBuffer.pData->buffer,
                                    nCapacity,
                                    &requiredSize);
    }

	return (error == osl_Daemon_ManagerError_NoError);

}

sal_Bool ODaemonProfile_Impl::getSections(const OUString& ustrSection,
                                            OUString& ustrBuffer)
{
    sal_uInt32 requiredSize;
    oslDaemonManagerError error =
        osl_getDaemonProfileSections(m_identifier.pData,
                                     ustrBuffer.pData->buffer,
                                     0,
                                     &requiredSize);

    if (error == osl_Daemon_ManagerError_NoError)
    {
		sal_Int32	nCapacity = 0;

		rtl_uStringbuffer_ensureCapacity(&ustrBuffer.pData, &nCapacity, requiredSize);

		error = osl_getDaemonProfileSections(m_identifier.pData,
                                     ustrBuffer.pData->buffer,
                                     nCapacity,
                                     &requiredSize);
    }

	return (error == osl_Daemon_ManagerError_NoError);
}


/*************************************************************************

    class ODaemonManager_VOS_Impl

*************************************************************************/


struct ltOUString
{
  bool operator()(const OUString& s1, const OUString& s2) const
  {
	  return s1 < s2 ? true : false;
  }
};

struct ltPointer
{
  bool operator()(const ORef<IDaemonAccess>& s1, const ORef<IDaemonAccess>& s2) const
  {
    return s1.getBodyPtr() < s2.getBodyPtr();
  }
};

struct DaemonStubEntry_Impl
{
    DaemonStubEntry_Impl(const ORef<IDaemonControl>& _control, oslDaemonHandle _handle)
        : control(_control), handle(_handle) {}

    ORef<IDaemonControl>     control;
    oslDaemonHandle            handle;
};

struct DaemonProxyEntry_Impl
{
    DaemonProxyEntry_Impl(const ORef<ODaemonAccess_Impl>& _access)
        : access(_access), lastStatus(IDaemonAccess::TStatus_Unknown) {}

    ORef<ODaemonAccess_Impl>         access;
    IDaemonAccess::TDaemonStatus     lastStatus;
};

#ifdef _USE_NAMESPACE
namespace vos
{
#endif

class ODaemonManager_Impl;

static TTimeValue pollingTimout(1, 0);

class OManagerTimer_Impl : public OTimer
{
public:

    OManagerTimer_Impl(ODaemonManager_Impl* pManager, const TTimeValue& Time)
        : OTimer(Time, Time)
        , m_pManager(pManager)
    {}

protected:

    virtual void SAL_CALL onShot();
    
    ODaemonManager_Impl* m_pManager;
};

class ODaemonManager_Impl : public OObject
                          ,    public IDaemonManager
                          ,    public IDaemonListener
{
    VOS_DECLARE_CLASSINFO(VOS_NAMESPACE(ODaemonManager_Impl, vos));
    
public:

    ODaemonManager_Impl();
    virtual ~ODaemonManager_Impl();

    // implementation of IDaemonManager
    virtual void SAL_CALL addListener(const ORef<IDaemonListener>& rListener);
    virtual void SAL_CALL removeListener(const ORef<IDaemonListener>& rListener);
    
    virtual sal_Bool                SAL_CALL isInstalled(const OUString& identifier);
    virtual TDaemonManagerError     SAL_CALL uninstall(const OUString& identifier);
    virtual TDaemonManagerError     SAL_CALL install(const ORef<IDaemonAccess>& rDaemonAccess, const OUString& ustrPath);
    virtual TDaemonManagerError     SAL_CALL execute(const ORef<IDaemonAccess>& rDaemonAccess,
                                            const ORef<IDaemonControl>& rDaemonControl);
    virtual TDaemonManagerError     SAL_CALL execute(ORef<IDaemonAccess>* prDaemonAccess,
                                            ORef<IDaemonControl>* prDaemonControls,
                                            sal_uInt32 numOfControls);

    virtual TDaemonManagerError     SAL_CALL startDaemon(const ORef<IDaemonAccess>& rDaemonAccess);
    virtual TDaemonManagerError     SAL_CALL stopDaemon(const ORef<IDaemonAccess>& rDaemonAccess);
    virtual TDaemonManagerError     SAL_CALL pauseDaemon(const ORef<IDaemonAccess>& rDaemonAccess);
    virtual TDaemonManagerError     SAL_CALL resumeDaemon(const ORef<IDaemonAccess>& rDaemonAccess);

    virtual sal_uInt32              SAL_CALL getDaemonCount() const;
    virtual ORef<IDaemonAccess>     SAL_CALL getDaemonByIndex(sal_uInt32 index) const;
    virtual ORef<IDaemonAccess>     SAL_CALL getDaemonByIdentifier(const OUString& identifier) const;

    virtual    IUProfile*           SAL_CALL openProfile(const OUString& identifier);
    virtual void                    SAL_CALL closeProfile(IUProfile* pProfile);

    // implementation of IDaemonListener
    virtual void SAL_CALL statusChanged(const ORef<IDaemonAccess>& rAccess,
                               IDaemonAccess::TDaemonStatus status);

    // implementation of IReference
    virtual RefCount SAL_CALL acquire();
    virtual RefCount SAL_CALL release();
    virtual RefCount SAL_CALL referenced() const;

    void SAL_CALL update();
    
protected:

    void SAL_CALL eraseMaps();
    
    typedef map<OUString, DaemonProxyEntry_Impl*, ltOUString>            DaemonProxyMap;
    typedef map<ORef<IDaemonAccess>, DaemonStubEntry_Impl*, ltPointer>   DaemonStubMap;
    typedef list<ORef<IDaemonListener> >                                 ListenerList;

    DaemonProxyMap              m_daemonProxyMap;
    DaemonStubMap               m_daemonStubMap;
    OMutex                      m_mutex;
    ORefCount                   m_refCount;
    ORef<OManagerTimer_Impl>    m_rTimer;
    ListenerList                m_listenerList;
    TimeValue                   m_lastChange;
};

#ifdef _USE_NAMESPACE
}
#endif

VOS_IMPLEMENT_CLASSINFO(VOS_CLASSNAME(ODaemonManager_Impl, vos), VOS_NAMESPACE(ODaemonManager_Impl, vos), VOS_NAMESPACE(OObject, vos), 0);

void OManagerTimer_Impl::onShot()
{
    m_pManager->update();
}

ODaemonManager_Impl::ODaemonManager_Impl()
    : m_rTimer(new OManagerTimer_Impl(this, pollingTimout))
{
    m_lastChange.Seconds = 0;
    m_lastChange.Nanosec = 0;
}

ODaemonManager_Impl::~ODaemonManager_Impl()
{
    eraseMaps();
}

void ODaemonManager_Impl::addListener(const ORef<IDaemonListener>& rListener)
{
    m_listenerList.push_front(rListener);
    
    if (!m_rTimer->isTicking())
    {
        m_rTimer->start();
    }
}

void ODaemonManager_Impl::removeListener(const ORef<IDaemonListener>& rListener)
{
    ListenerList::iterator iter = m_listenerList.begin();
    
    while ((iter != m_listenerList.end()) && ((*iter) != rListener))
    {
        iter++;
    }
    
    if (iter != m_listenerList.end())
    {
        m_listenerList.erase(iter);
    }
}

sal_Bool ODaemonManager_Impl::isInstalled(const OUString& identifier)
{
    OGuard guard(m_mutex);
    
    sal_Bool ret = sal_False;
    
    if (osl_isDaemonInstalled(identifier.pData, &ret) == osl_Daemon_ManagerError_NoError)
        return ret;
    else
        return sal_False;
}

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::uninstall(const OUString& identifier)
{
    OGuard guard(m_mutex);

    if (!checkDaemonSecurity())
    {
        return TManagerError_Security;
    }

    osl_stopDaemon(identifier.pData);
    
    TDaemonManagerError ret = (TDaemonManagerError) osl_uninstallDaemon(identifier.pData);

    return ret;
}

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::install(const ORef<IDaemonAccess>& rAccess, const OUString& ustrPath)
{
    OGuard guard(m_mutex);

    if (!checkDaemonSecurity())
    {
        return TManagerError_Security;
    }

    TDaemonManagerError ret = TManagerError_NoManager;
    OStartupInfo        startupInfo;
    OUString aPath = ustrPath;
    OUString aUString;
    sal_uInt32 nLength=0;    

    if (aPath.getLength() == 0)
    {
        OUString aUNCPath;
		startupInfo.getExecutableFile( aUNCPath );
		osl::FileBase::getSystemPathFromNormalizedPath(aUNCPath, aPath);
    }
    
    oslDaemonInfo         info;
    memset(&info, 0, sizeof(info));
    
    info.Size    = sizeof(info);
    info.Version = rAccess->getVersion();
    info.Mode    = (oslDaemonStartupMode)rAccess->getMode();
    

    aUString = rAccess->getIdentifier();
    rtl_uString_newFromString(&info.Identifier,aUString.pData);
    

    aUString = rAccess->getDisplayName();
    rtl_uString_newFromString(&info.DisplayName,aUString.pData);
    
    aUString = rAccess->getManufactor();
    rtl_uString_newFromString(&info.Manufactor,aUString.pData);

    rtl_uString_newFromString(&info.Path,aPath.pData);

    if (rAccess->getDependencies())
    {
        aUString = rAccess->getDependencies();
        nLength = aUString.getLength();
        if ( nLength > OSL_DAEMON_DEPENDENCIES_LEN )
        {
            nLength = OSL_DAEMON_DEPENDENCIES_LEN;
        }
        memcpy(info.Path, aUString.pData->buffer, nLength * sizeof(sal_Unicode));
        info.Dependencies[OSL_DAEMON_DEPENDENCIES_LEN - 1] = 0;
    }
    
    ret = (IDaemonManager::TDaemonManagerError) osl_installDaemon(&info);

    rtl_uString_release(info.Identifier);
    rtl_uString_release(info.DisplayName);
    rtl_uString_release(info.Manufactor);
    rtl_uString_release(info.Path);
    
    return ret;
}

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::execute
(
    const ORef<IDaemonAccess>&  rAccess,
    const ORef<IDaemonControl>&  rControl
)
{
    oslDaemonHandle handle = NULL;
     static oslDaemonPropertyAccess propertyAccess =
    {
        _IDaemonAccess_GetPropCount,
        _IDaemonAccess_GetProps,
        _IDaemonAccess_SetProps
    };
    

    TDaemonManagerError ret =  (TDaemonManagerError)
        osl_createDaemon(rAccess->getIdentifier().pData,
                         _IDaemonControl_ControlFunction,
                         rControl.getBodyPtr(),
                         &propertyAccess,
                         rAccess.getBodyPtr(),
                         &handle);


    if (ret == TManagerError_NoError)
    {
        DaemonStubEntry_Impl* pEntry = NULL;
        
        {
            OGuard guard(m_mutex);

            rControl->setListener(ORef<IDaemonListener>(this));
        
            pEntry = new DaemonStubEntry_Impl(rControl, handle);
            
            m_daemonStubMap.insert(DaemonStubMap::value_type(rAccess, pEntry));
        }
        
        ret = (TDaemonManagerError) osl_executeDaemon(handle);

        rControl->setListener(ORef<IDaemonListener>(NULL));
        delete pEntry;
        m_daemonStubMap.erase(m_daemonStubMap.begin());
    }

    return ret;
}

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::execute
(
    ORef<IDaemonAccess>* prDaemonAccess,
    ORef<IDaemonControl>* prDaemonControls,
    sal_uInt32 numOfControls)
{
    return TManagerError_Unknown;
}

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::startDaemon(const ORef<IDaemonAccess>& rDaemonAccess)
{
    {
        OGuard guard(m_mutex);
        
        if (!checkDaemonSecurity())
        {
            return TManagerError_Security;
        }
    }

    TDaemonManagerError ret = (TDaemonManagerError)osl_startDaemon((rDaemonAccess->getIdentifier()).pData);
    
    return ret;
}    

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::stopDaemon(const ORef<IDaemonAccess>& rDaemonAccess)
{
    // caution  'cause of possible deadlock when stopping sadmind out of sdadmin
    {
        OGuard guard(m_mutex);
        
        if (!checkDaemonSecurity())
        {
            return TManagerError_Security;
        }
    }

    TDaemonManagerError ret = (TDaemonManagerError)osl_stopDaemon((rDaemonAccess->getIdentifier()).pData);
    
    return ret;
}    

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::pauseDaemon(const ORef<IDaemonAccess>& rDaemonAccess)
{
    {
        OGuard guard(m_mutex);
        
        if (!checkDaemonSecurity())
        {
            return TManagerError_Security;
        }
    }
    
    
    TDaemonManagerError ret = (TDaemonManagerError)osl_pauseDaemon((rDaemonAccess->getIdentifier()).pData);
    
    return ret;
}    

IDaemonManager::TDaemonManagerError ODaemonManager_Impl::resumeDaemon(const ORef<IDaemonAccess>& rDaemonAccess)
{
    {
        OGuard guard(m_mutex);
        
        if (!checkDaemonSecurity())
        {
            return TManagerError_Security;
        }
    }

    TDaemonManagerError ret = (TDaemonManagerError)osl_continueDaemon((rDaemonAccess->getIdentifier()).pData);
    
    return ret;
}

sal_uInt32 ODaemonManager_Impl::getDaemonCount() const
{
    OGuard guard((OMutex&)m_mutex);
    sal_uInt32 count = 0;
    
    if (osl_queryDaemonCount(&count) == osl_Daemon_ManagerError_NoError)
        return count;
    else
        return 0;
}

ORef<IDaemonAccess> ODaemonManager_Impl::getDaemonByIndex(sal_uInt32 index) const
{
    ODaemonManager_Impl* This = ((ODaemonManager_Impl*)this);
    
    OGuard guard(This->m_mutex);
    ORef<IDaemonAccess> ret;
    OUString buffer;

    if (
        osl_queryDaemonIdentifier(index, &buffer.pData) ==
        osl_Daemon_ManagerError_NoError)
    {
        DaemonProxyMap::const_iterator iter = m_daemonProxyMap.find(buffer);

        if (iter == m_daemonProxyMap.end())
        {
            ORef<ODaemonAccess_Impl> newAccess = new ODaemonAccess_Impl(buffer);
            DaemonProxyEntry_Impl* pImpl = new DaemonProxyEntry_Impl(newAccess.getBodyPtr());
            
            iter = This->m_daemonProxyMap.insert(
                DaemonProxyMap::value_type(buffer, pImpl)).first;

            if (!This->m_rTimer->isTicking())
            {
                This->m_rTimer->start();
            }
        }
        
        ret = (*iter).second->access.getBodyPtr();
    }
    
    return ret;
}

ORef<IDaemonAccess> ODaemonManager_Impl::getDaemonByIdentifier(const OUString& identifier) const
{
    ODaemonManager_Impl* This = ((ODaemonManager_Impl*)this);

    OGuard guard(This->m_mutex);

    ORef<IDaemonAccess> ret;
    
    DaemonProxyMap::const_iterator iter = m_daemonProxyMap.find(identifier);

    if (iter == m_daemonProxyMap.end())
    {
        sal_Bool isInstalled = sal_False;
        
        if (
            (osl_isDaemonInstalled(identifier.pData, &isInstalled) == osl_Daemon_ManagerError_NoError) &&
            (isInstalled)
           )
        {
            ORef<ODaemonAccess_Impl> newAccess = new ODaemonAccess_Impl(identifier);
            DaemonProxyEntry_Impl* pImpl = new DaemonProxyEntry_Impl(newAccess.getBodyPtr());

            iter = This->m_daemonProxyMap.insert(
                DaemonProxyMap::value_type(identifier, pImpl)).first;

            if (!This->m_rTimer->isTicking())
            {
                This->m_rTimer->start();
            }

            ret = (*iter).second->access.getBodyPtr();
        }
    }
    else
    {
        ret = (*iter).second->access.getBodyPtr();
    }
    
    return ret;
}

IUProfile* ODaemonManager_Impl::openProfile(const OUString& identifier)
{
    ODaemonProfile_Impl* pRet = NULL;
    if (isInstalled(identifier))
    {
        pRet = new ODaemonProfile_Impl(identifier);
    }
    
    return pRet;
}

void ODaemonManager_Impl::closeProfile(IUProfile* pProfile)
{
    if (pProfile) delete (ODaemonProfile_Impl*)pProfile;
}

void ODaemonManager_Impl::statusChanged(const ORef<IDaemonAccess>& rAccess,
                                           IDaemonAccess::TDaemonStatus status)
{
    OGuard guard((OMutex&)m_mutex);

    DaemonStubMap::iterator iter = m_daemonStubMap.find(rAccess);

    if (iter != m_daemonStubMap.end())
    {
        osl_setDaemonStatus((*iter).second->handle, (oslDaemonStatus)status);
    }
}

IReference::RefCount ODaemonManager_Impl::acquire()
{
    return m_refCount.acquire();
}

IReference::RefCount ODaemonManager_Impl::release()
{
    RefCount ret = m_refCount.release();
    
    if (ret == 0)
    {
        delete this;
    }
    
    return ret;
}

IReference::RefCount ODaemonManager_Impl::referenced() const
{
    return m_refCount.referenced();
}

typedef pair<ORef<ODaemonAccess_Impl>, IDaemonAccess::TDaemonStatus> ChangedEntry;

typedef list<ChangedEntry> ChangedEntryList;

void ODaemonManager_Impl::update()
{
    ChangedEntryList     changedEntries;
    ListenerList        copyOfListeners;
    sal_Bool            bChanged = sal_False;
    
    if (
        (osl_queryDaemonChanges(&m_lastChange, &bChanged) != osl_Daemon_ManagerError_NoError) ||
        bChanged
       )
    {
        // evaluation of daemon list must be protected against add/remove
        // by another thread
        {
            OGuard guard((OMutex&)m_mutex);
            sal_Bool isInstalled = sal_False;

            if (!m_daemonProxyMap.empty())
            {
                DaemonProxyMap::iterator iter = m_daemonProxyMap.begin();
                // update all entries

                while (iter != m_daemonProxyMap.end())
                {
                    (*iter).second->access->update();
                    iter++;
                }
                
                // check if all cached daemons are valid
                // and call listener if status changed
                iter = m_daemonProxyMap.begin();

                while (iter != m_daemonProxyMap.end())
                {
                    IDaemonAccess::TDaemonStatus newStatus = (*iter).second->access->getStatus();
                    
                    if (newStatus != (*iter).second->lastStatus)
                    {
                        changedEntries.push_back(
                            ChangedEntry((*iter).second->access, newStatus));
                            
                        (*iter).second->lastStatus = newStatus;
                    }
                        
                    if (
                        (
                         osl_isDaemonInstalled(((*iter).second->access->getIdentifier()).pData, &isInstalled) !=
                         osl_Daemon_ManagerError_NoError
                        ) ||
                        !isInstalled
                       )
                    {
                        if ((*iter).second->lastStatus != IDaemonAccess::TStatus_Unknown)
                        {
                            changedEntries.push_back(
                                ChangedEntry((*iter).second->access, IDaemonAccess::TStatus_Unknown));
                        }

                        delete (*iter).second;
                        m_daemonProxyMap.erase(iter);
                        iter = m_daemonProxyMap.begin();
                    }
                    else
                    {
                        iter++;
                    }
                }
            }

            // check for new daemons
            sal_uInt32 daemonCount = 0;
            
            if (osl_queryDaemonCount(&daemonCount) == osl_Daemon_ManagerError_NoError)
            {
                if (m_daemonProxyMap.size() < daemonCount)
                {
                    OUString identifier;
                    sal_uInt32 i;
                    
                    for (i = 0; i < daemonCount; i++)
                    {
                        if (osl_queryDaemonIdentifier(i, &identifier.pData ) ==
                            osl_Daemon_ManagerError_NoError)
                        {
                            if (m_daemonProxyMap.find(identifier) == m_daemonProxyMap.end())
                            {
                                ORef<ODaemonAccess_Impl> newAccess = new ODaemonAccess_Impl(identifier);
                                DaemonProxyEntry_Impl* pImpl = new DaemonProxyEntry_Impl(newAccess.getBodyPtr());

                                pImpl->lastStatus = newAccess->getStatus();
                                
                                m_daemonProxyMap.insert(DaemonProxyMap::value_type(identifier, pImpl));
                                
                                changedEntries.push_back(
                                    ChangedEntry(newAccess, pImpl->lastStatus));
                            }
                        }
                    }
                }
            }
            
            if (!changedEntries.empty())
            {
                copyOfListeners = m_listenerList;
            }
            
            if (m_daemonProxyMap.empty() && m_listenerList.empty())
            {
                // cache is empty -> stop timer until new entry is added
                m_rTimer->stop();
            }
        }
        

        // following code has not to be protected by the mutex because
        // it works on copied data. It must not be protected because of
        // avoiding deadlocks while calling the listeners

        if (!changedEntries.empty() && !copyOfListeners.empty())
        {
            ChangedEntryList::const_iterator entryIter = changedEntries.begin();
            
            while (entryIter != changedEntries.end())
            {
                ListenerList::iterator listenerIter = copyOfListeners.begin();
                
                while (listenerIter != copyOfListeners.end())
                {
                    (*listenerIter)->statusChanged(
                        (*entryIter).first.getBodyPtr(),
                        (*entryIter).second);
                
                    listenerIter++;
                }
                
                entryIter++;
            }
        }    
    }
}    

void ODaemonManager_Impl::eraseMaps()
{
    OGuard guard((OMutex&)m_mutex);
    DaemonStubMap::iterator stubIter = m_daemonStubMap.begin();
    
    while (stubIter != m_daemonStubMap.end())
    {
        osl_endDaemon((*stubIter).second->handle);
        delete (*stubIter).second;
        stubIter++;
    }

    DaemonProxyMap::iterator proxyIter = m_daemonProxyMap.begin();
    
    while (proxyIter != m_daemonProxyMap.end())
    {
        delete (*proxyIter).second;
        proxyIter++;
    }
}

ORef<IDaemonManager> IDaemonManager::getGlobalManager()
{
    static IDaemonManager* pManager = NULL;

    if (pManager == NULL)
    {
        OGuard guard(OMutex::getGlobalMutex());
    
        if (pManager == NULL)
        {
            pManager = new ODaemonManager_Impl();
            
            pManager->acquire();
        }
    }   
    
    return pManager;
}


