/*************************************************************************
 *
 *  $RCSfile: daemon.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>
#include <map>

#include <vos/diagnose.hxx>
#include <vos/daemon.hxx>
#include <vos/timer.hxx>
#include <vos/conditn.hxx>
#include <rtl/ustring.hxx>

#ifdef _USE_NAMESPACE
using namespace vos;
using namespace std;
using namespace rtl;
#endif


/*************************************************************************

	class ODaemon
	
*************************************************************************/

VOS_IMPLEMENT_CLASSINFO(VOS_CLASSNAME(ODaemon, vos), VOS_NAMESPACE(ODaemon, vos), VOS_NAMESPACE(OObject, vos), 0);

#ifdef _USE_NAMESPACE
namespace vos
{
#endif

static TTimeValue pollingTimeout(1, 0);

class ODaemon_Impl
{
public:
	ODaemon_Impl
	(
		const NAMESPACE_RTL(OUString)& Identifier, 
		const NAMESPACE_RTL(OUString)& DisplayName, 
		const NAMESPACE_RTL(OUString)& Manufactor, 
		const NAMESPACE_RTL(OUString)& Dependencies, 
		sal_uInt32 Version, 
		IDaemonAccess::TDaemonStatus Status, 
		IDaemonAccess::TDaemonStartupMode Mode
	) :
		m_identifier( Identifier ),
		m_displayName( DisplayName ),	
		m_manufactor( Manufactor ),
		m_version( Version ),
		m_status( Status ),
		m_mode( Mode )
	{
		if ( Dependencies.getLength() )
			m_dependencies = Dependencies;	
	}
	
	~ODaemon_Impl()
	{
	}
	
	NAMESPACE_RTL(OUString) 				m_identifier;	
	NAMESPACE_RTL(OUString)					m_displayName;	
	NAMESPACE_RTL(OUString)					m_manufactor;	
	NAMESPACE_RTL(OUString)					m_dependencies;
	sal_uInt32 								m_version;
	ORefCount 							m_refCount;
	IDaemonAccess::TDaemonStatus 		m_status;
	ORef<IDaemonListener>				m_rListener;
	IDaemonAccess::TDaemonStartupMode 	m_mode;
	OCondition							m_wasStopped;
};
#ifdef _USE_NAMESPACE
}
#endif

ODaemon::ODaemon
(
	const NAMESPACE_RTL(OUString)& Identifier, 
	const NAMESPACE_RTL(OUString)& DisplayName, 
	const NAMESPACE_RTL(OUString)& Manufactor, 
	sal_uInt32 Version, 
	TDaemonStartupMode Mode,
	const NAMESPACE_RTL(OUString)& Dependencies
)
{
	m_pImpl = new ODaemon_Impl(Identifier, 
							   DisplayName, 
							   Manufactor, 
							   Dependencies,
							   Version, 
							   TStatus_Unknown, 
							   Mode);
}
             
ODaemon::~ODaemon()
{
	delete m_pImpl;
}

ODaemon::TCommandLineOption ODaemon::parseArgs(sal_Int32 argc, sal_Char* argv[])
{
	TCommandLineOption ret = TOption_Unknown;
	
	if (argc <= 1)
	{
		ret = TOption_ExecuteAsDaemon;		
	}

	for (sal_Int32 i = 1; (i < argc) && (ret == TOption_Unknown); i++)
	{
		if (strcmp(argv[i], "-i") == 0) 
		{
			ret = TOption_Install;		
		} 
		else if (strcmp(argv[i], "-u") == 0) 
		{
		ret = TOption_Uninstall;		
		}
		else if (strcmp(argv[i], "-p") == 0) 
		{
			ret = TOption_ExecuteAsProcess;		
		}
		else if (strcmp(argv[i], "-start") == 0) 
		{
			ret = TOption_Start;		
		}
		else if (strcmp(argv[i], "-stop") == 0) 
		{
			ret = TOption_Stop;		
		}
		else if (strcmp(argv[i], "-pause") == 0) 
		{
			ret = TOption_Pause;		
		}
		else if (strcmp(argv[i], "-resume") == 0) 
		{
			ret = TOption_Resume;		
		}
	}

	return ret;	
}	

#ifdef UNX
static sal_Int32 parseArgsForInstallPath(sal_Int32 argc, sal_Char* argv[])
{
	sal_Int32 ret	= -1;

	for (sal_Int32 i = 1; (i < argc) && (ret < 0); i++)
	{
		if ((strcmp(argv[i], "-i") == 0) && ((i + 1) < argc))
		{
			ret	= i + 1;
		}
	}
	
	return ret;	
}	
#endif

IDaemonManager::TDaemonManagerError	ODaemon::execute(sal_Int32 argc, sal_Char* argv[])
{
	IDaemonManager::TDaemonManagerError ret = IDaemonManager::TManagerError_Unknown;
	
	ODaemon::TCommandLineOption mode = ODaemon::parseArgs(argc, argv);
	
	switch (mode)
	{
		case ODaemon::TOption_Install:
			{
#ifdef UNX
				sal_Int32 index = parseArgsForInstallPath(argc, argv);
				
				if (index > 0)
				{
					ret = IDaemonManager::getGlobalManager()->install(ORef<IDaemonAccess>(this), OUString::createFromAscii(argv[index]));
				}
				else
				{
					ret = IDaemonManager::getGlobalManager()->install(ORef<IDaemonAccess>(this),OUString());	
				}
#else
				ret = IDaemonManager::getGlobalManager()->install(ORef<IDaemonAccess>(this), OUString());	
#endif				
			}
			break;
		case ODaemon::TOption_Uninstall:
			ret = IDaemonManager::getGlobalManager()->uninstall(getIdentifier());	
			break;
		case ODaemon::TOption_ExecuteAsDaemon:
			ret = IDaemonManager::getGlobalManager()->execute(ORef<IDaemonAccess>(this), ORef<IDaemonControl>(this));	
			break;
		case ODaemon::TOption_ExecuteAsProcess:
			ret = executeAsProcess(sal_True);
			break;
		case ODaemon::TOption_Start:
			ret = IDaemonManager::getGlobalManager()->startDaemon(ORef<IDaemonAccess>(this));
			break;
		case ODaemon::TOption_Stop:
			ret = IDaemonManager::getGlobalManager()->stopDaemon(ORef<IDaemonAccess>(this));
			break;
		case ODaemon::TOption_Pause:
			ret = IDaemonManager::getGlobalManager()->pauseDaemon(ORef<IDaemonAccess>(this));
			break;
		case ODaemon::TOption_Resume:
			ret = IDaemonManager::getGlobalManager()->resumeDaemon(ORef<IDaemonAccess>(this));
			break;
		default:
			ret = IDaemonManager::TManagerError_NoError;
			break;
	}	

    const OUString& aUString = getDisplayName();
    OString aString = OUStringToOString( aUString,RTL_TEXTENCODING_UTF8);

    switch (ret)
	{
		case IDaemonManager::TManagerError_NoError:
        {
            switch (mode)
            {
                case ODaemon::TOption_Install:
                    fprintf(stderr, "Daemon '%s' installed sucessfully\n", aString.getStr());                    
                    break;
                case ODaemon::TOption_Uninstall:
                    fprintf(stderr, "Daemon '%s' uninstalled sucessfully\n", aString.getStr());
                    break;
                case ODaemon::TOption_ExecuteAsDaemon:
                    break;
                case ODaemon::TOption_ExecuteAsProcess:
                    break;
                case ODaemon::TOption_Start:
                    fprintf(stderr, "Daemon '%s' was started\n", aString.getStr());
                    break;
                case ODaemon::TOption_Stop:
                    fprintf(stderr, "Daemon '%s' was stopped\n", aString.getStr());
                    break;
                case ODaemon::TOption_Pause:
                    fprintf(stderr, "Daemon '%s' was paused\n", aString.getStr());
                    break;
                case ODaemon::TOption_Resume:
                    fprintf(stderr, "Daemon '%s' was resumed\n", aString.getStr());
                    break;
                default:
                    fprintf(stderr, "wrong parameter\n");
                    break;
            }	
        }
			break;
		case IDaemonManager::TManagerError_NotInstalled:
			fprintf(stderr, "Daemon '%s' is not installed\n", aString.getStr());
			break;
		case IDaemonManager::TManagerError_AllreadyInstalled:
			fprintf(stderr, "Daemon '%s' is allready installed\n", aString.getStr());
			break;
		case IDaemonManager::TManagerError_AllreadyRunning:
			fprintf(stderr, "Daemon '%s' is allready running\n", aString.getStr());
			break;
		case IDaemonManager::TManagerError_NotRunning:
			fprintf(stderr, "Daemon '%s' is not running\n", aString.getStr());
			break;
		case IDaemonManager::TManagerError_NotPaused:
			fprintf(stderr, "Daemon '%s' is not paused\n", aString.getStr());
			break;
		case IDaemonManager::TManagerError_NoManager:
			fprintf(stderr, "No daemon manager available\n");
			break;
		case IDaemonManager::TManagerError_Security:
			fprintf(stderr, "security violation\n");
			break;
		default :
			fprintf(stderr, "Unknown error occured\n");
			break;
	}

	return ret;
}	

IDaemonManager::TDaemonManagerError ODaemon::executeAsProcess(sal_Bool waitForCompletion)
{
	onStart();
					
	if (waitForCompletion)
	{
		m_pImpl->m_wasStopped.wait();
	}		

	return IDaemonManager::TManagerError_NoError;
}														  
													  
void ODaemon::setStatus(TDaemonStatus status)
{
	m_pImpl->m_status = status;
	
	if (m_pImpl->m_rListener.isValid())
	{
		m_pImpl->m_rListener->statusChanged(ORef<IDaemonAccess>(this), status);
	}
	
	if (m_pImpl->m_status == TStatus_Stopped)
	{
		m_pImpl->m_wasStopped.set();	
	}	
}	

void ODaemon::setListener(const ORef<IDaemonListener>& rListener)
{
	m_pImpl->m_rListener = rListener;	
}	

const NAMESPACE_RTL(OUString)& ODaemon::getIdentifier() const
{
	return m_pImpl->m_identifier;
}

const NAMESPACE_RTL(OUString)& ODaemon::getDisplayName() const
{
	return m_pImpl->m_displayName;
}

const NAMESPACE_RTL(OUString)& ODaemon::getManufactor() const
{
	return m_pImpl->m_manufactor;
}

sal_uInt32 ODaemon::getVersion() const
{
	return m_pImpl->m_version;
}

const NAMESPACE_RTL(OUString)& ODaemon::getDependencies() const
{
	return m_pImpl->m_dependencies;
}	

IDaemonAccess::TDaemonStartupMode ODaemon::getMode() const
{
	return m_pImpl->m_mode;
}

ODaemon::TDaemonStatus ODaemon::getStatus() const
{
	return m_pImpl->m_status;
}

sal_uInt32 ODaemon::getPropertyCount() const
{
	return 0;	
}	

sal_uInt32 ODaemon::getProperties(ODaemonProperty* 	pProperties, 
			 		  		 sal_uInt32 				count) const
{
	return 0;	
}	
										
sal_Bool	ODaemon::setProperties(ODaemonProperty* 	pProperties, 
			    			   sal_uInt32 				count,
							   sal_uInt32&				rErrorCode,
					  		   NAMESPACE_RTL(OUString)&	ustrErrorText)
{
    ustrErrorText.createFromAscii("daemon has no properties");
    
	return sal_False;	
}						  


IReference::RefCount ODaemon::acquire() 		
{ 
	return m_pImpl->m_refCount.acquire(); 
}

IReference::RefCount ODaemon::release() 		
{ 
	RefCount ret = m_pImpl->m_refCount.release();
	
	if (ret == 0)
	{
		delete this;		
	}
	
	return ret; 
}

IReference::RefCount ODaemon::referenced() const 	
{ 
	return m_pImpl->m_refCount.referenced(); 
}

#ifdef _USE_NAMESPACE
namespace vos
{
#endif

class ODaemonProperty_Impl
{
public:
	ODaemonProperty_Impl(const NAMESPACE_RTL(OUString)& name, ODaemonProperty::TDaemonPropertyType type, sal_uInt32 listSize);
	~ODaemonProperty_Impl();	
	ODaemonProperty_Impl(const ODaemonProperty_Impl & rImpl);

	ODaemonProperty::TDaemonPropertyType 	m_type;
	sal_uInt32			  					m_listSize;
	NAMESPACE_RTL(OUString)					m_name;
		
	union 
	{
		sal_Bool 					m_bool;
		sal_uInt32 					m_uLong;
		NAMESPACE_RTL(OUString)* 	m_pString;
		sal_Bool* 					m_pBool;
		sal_uInt32* 				m_pULong;
	};
};

#ifdef _USE_NAMESPACE
}
#endif

ODaemonProperty_Impl::ODaemonProperty_Impl(const NAMESPACE_RTL(OUString)& name, ODaemonProperty::TDaemonPropertyType type, sal_uInt32 listSize) :
	m_type( type ),
	m_listSize( listSize ),
	m_name( name )
{
	sal_uInt32 i = 0;
	
	switch (m_type)
	{
		case ODaemonProperty::TPropertyType_Boolean:
			m_bool = sal_False;
			break;
		case ODaemonProperty::TPropertyType_BooleanList:
			if (m_listSize)
			{
				m_pBool = new sal_Bool[m_listSize];
				for (i = 0; i < m_listSize; i++)
				{
					m_pBool[i] = sal_False;
				}			
			}
			else
				m_pBool = NULL;
			
			break;
		case ODaemonProperty::TPropertyType_uLong:
			m_uLong = 0;
			break;
		case ODaemonProperty::TPropertyType_uLongList:
			if (m_listSize)
			{
				m_pULong = new sal_uInt32[m_listSize];
				for (i = 0; i < m_listSize; i++)
				{
					m_pULong[i] = 0;
				}			
			}
			else
				m_pULong = NULL;
			break;
		case ODaemonProperty::TPropertyType_String:
			m_pString = new NAMESPACE_RTL(OUString)();		
			break;
		case ODaemonProperty::TPropertyType_StringList:
			if (m_listSize)
			{
				m_pString = new NAMESPACE_RTL(OUString)[m_listSize];
			}
			else
				m_pString = NULL;
			break;
		default:
			m_uLong = 0;
			break;
	}	
}	

ODaemonProperty_Impl::ODaemonProperty_Impl(const ODaemonProperty_Impl & rImpl)
{
	sal_uInt32 i = 0;
	m_type = rImpl.m_type;
	m_listSize = rImpl.m_listSize;
	m_name = rImpl.m_name;

	switch (m_type)
	{
		case ODaemonProperty::TPropertyType_Boolean:
			m_bool = rImpl.m_bool;
			break;
		case ODaemonProperty::TPropertyType_BooleanList:
			if (m_listSize)
			{
				m_pBool = new sal_Bool[m_listSize];
				for (i = 0; i < m_listSize; i++)
				{
					m_pBool[i] = rImpl.m_pBool[i];
				}			
			}
			else
				m_pBool = NULL;
			break;
		case ODaemonProperty::TPropertyType_uLong:
			m_uLong = rImpl.m_uLong;
			break;
		case ODaemonProperty::TPropertyType_uLongList:
			if (m_listSize)
			{
				m_pULong = new sal_uInt32[m_listSize];
				for (i = 0; i < m_listSize; i++)
				{
					m_pULong[i] = rImpl.m_pULong[i];
				}			
			}
			else
				m_pULong = NULL;
			break;
		case ODaemonProperty::TPropertyType_String:
			m_pString = new NAMESPACE_RTL(OUString)(*rImpl.m_pString);		
			break;
		case ODaemonProperty::TPropertyType_StringList:
			if (m_listSize)
			{
				m_pString = new NAMESPACE_RTL(OUString)[m_listSize];
				for (i = 0; i < m_listSize; i++)
				{
					m_pString[i] = rImpl.m_pString[i];
				}			
			}
			else
				m_pString = NULL;
			break;
		default:
			m_uLong = 0;
			break;
	}	
}	

ODaemonProperty_Impl::~ODaemonProperty_Impl()
{
	switch (m_type)
	{
		case ODaemonProperty::TPropertyType_Boolean:
			break;
		case ODaemonProperty::TPropertyType_BooleanList:
			delete[] m_pBool; 
			break;
		case ODaemonProperty::TPropertyType_uLong:
			break;
		case ODaemonProperty::TPropertyType_uLongList:
			delete[] m_pULong; 
			break;
		case ODaemonProperty::TPropertyType_String:
			delete m_pString;
			break;
		case ODaemonProperty::TPropertyType_StringList:
			delete[] m_pString;
			break;
		default:
			break;
	}	
}	

ODaemonProperty::ODaemonProperty()
{
	m_pImpl = new ODaemonProperty_Impl(*new NAMESPACE_RTL(OUString)(), TPropertyType_Invalid, 0);
}

ODaemonProperty::ODaemonProperty(const NAMESPACE_RTL(OUString)& name, ODaemonProperty::TDaemonPropertyType type, sal_uInt32 listSize)
{
	m_pImpl = new ODaemonProperty_Impl(name, type, listSize);
}


ODaemonProperty::ODaemonProperty(const NAMESPACE_RTL(OUString)& name, sal_Bool aBool)
{
	m_pImpl = new ODaemonProperty_Impl(name, TPropertyType_Boolean, 0);
	m_pImpl->m_bool = aBool;	
}

ODaemonProperty::ODaemonProperty(const NAMESPACE_RTL(OUString)& name, sal_uInt32 aULong)
{
	m_pImpl = new ODaemonProperty_Impl(name, TPropertyType_uLong, 0);
	m_pImpl->m_uLong = aULong;	
}

ODaemonProperty::ODaemonProperty(const NAMESPACE_RTL(OUString)& name, const NAMESPACE_RTL(OUString)& aString)
{
	m_pImpl = new ODaemonProperty_Impl(name, TPropertyType_String, 0);
	*(m_pImpl->m_pString) = aString;
}

ODaemonProperty::ODaemonProperty(const ODaemonProperty& rProperty)
{
	m_pImpl = new ODaemonProperty_Impl(*rProperty.m_pImpl);
}

ODaemonProperty::~ODaemonProperty()
{
	delete m_pImpl;
}

ODaemonProperty& ODaemonProperty::operator=(const ODaemonProperty& rProperty)
{
	delete m_pImpl;
	m_pImpl = new ODaemonProperty_Impl(*rProperty.m_pImpl);
	
	return *this;
}

NAMESPACE_RTL(OUString)& ODaemonProperty::getName() const
{
	return m_pImpl->m_name;	
}	

ODaemonProperty::TDaemonPropertyType ODaemonProperty::getType() const
{
	return m_pImpl->m_type;
}

sal_uInt32 ODaemonProperty::getListSize() const
{
	if (
		(m_pImpl->m_type == TPropertyType_BooleanList) ||
		(m_pImpl->m_type == TPropertyType_uLongList) ||
		(m_pImpl->m_type == TPropertyType_StringList)
	   )	
	{
		return m_pImpl->m_listSize;
	}
	else
		return 0;
}

void ODaemonProperty::setListSize(sal_uInt32 size)
{
	switch (m_pImpl->m_type)
	{
		case TPropertyType_BooleanList:
			{
				sal_Bool *pBool = NULL;
				if (size)
				{
					pBool = new sal_Bool[size];
					
					for (sal_uInt32 i = 0; i < VOS_MIN(size, m_pImpl->m_listSize); i++)
					{
						pBool[i] = m_pImpl->m_pBool[i];
					}			
				}
				
				delete[] m_pImpl->m_pBool;				
				m_pImpl->m_pBool = pBool;
			}
			break;
		case TPropertyType_uLongList:
			{
				sal_uInt32 *pULong = NULL;

				if (size)
				{
					pULong = new sal_uInt32[size];
					
					for (sal_uInt32 i = 0; i < VOS_MIN(size, m_pImpl->m_listSize); i++)
					{
						pULong[i] = m_pImpl->m_pULong[i];
					}			
				}

				delete[] m_pImpl->m_pULong;
				m_pImpl->m_pULong = pULong;
				m_pImpl->m_listSize = size;				
			}
			break;
		case TPropertyType_StringList:
			{
				NAMESPACE_RTL(OUString) *pString = NULL;

				if (size)
				{
					pString = new NAMESPACE_RTL(OUString)[size];
					
					for (sal_uInt32 i = 0; i < VOS_MIN(size, m_pImpl->m_listSize); i++)
					{
						pString[i] = m_pImpl->m_pString[i];
					}			
				}
				delete[] m_pImpl->m_pString;
				m_pImpl->m_pString = pString;
				m_pImpl->m_listSize = size;				
			}
			break;
		default:
			break;
	}
}

sal_Bool ODaemonProperty::getBoolean(sal_uInt32 index)
{
	if (m_pImpl->m_type == TPropertyType_Boolean)
	{
		return m_pImpl->m_bool;
	}	
	else if (
			 (m_pImpl->m_type == TPropertyType_BooleanList) && 
			 (index < m_pImpl->m_listSize)
			)
	{
		return m_pImpl->m_pBool[index];
	}
	else
	{
		return sal_False;	
	}
}

sal_uInt32 ODaemonProperty::getULong(sal_uInt32 index)
{
	if (m_pImpl->m_type == TPropertyType_uLong)
	{
		return m_pImpl->m_uLong;
	}	
	else if (
			 (m_pImpl->m_type == TPropertyType_uLongList) &&
			 (index < m_pImpl->m_listSize)
			)
	{
		return m_pImpl->m_pULong[index];
	}
	else
	{
		return 0;	
	}
}

const NAMESPACE_RTL(OUString) ODaemonProperty::getString(sal_uInt32 index)
{
	if (m_pImpl->m_type == TPropertyType_String)
	{
		return *(m_pImpl->m_pString);
	}	
	else if (
			 (m_pImpl->m_type == TPropertyType_StringList) &&
			 (index < m_pImpl->m_listSize)
			) 
	{
		return m_pImpl->m_pString[index];
	}
	else
	{
		return NAMESPACE_RTL(OUString)();
	}
}

void ODaemonProperty::setBoolean(sal_Bool value, sal_uInt32 index)
{
	if (m_pImpl->m_type == TPropertyType_Boolean)
	{
		m_pImpl->m_bool = value;
	}	
	else if (
			 (m_pImpl->m_type == TPropertyType_BooleanList) && 
			 (index < m_pImpl->m_listSize)
			)
	{
		m_pImpl->m_pBool[index] = value;
	}
}

void ODaemonProperty::setULong(sal_uInt32 value, sal_uInt32 index)
{
	if (m_pImpl->m_type == TPropertyType_uLong)
	{
		m_pImpl->m_uLong = value;
	}	
	else if (
			 (m_pImpl->m_type == TPropertyType_uLongList) &&
			 (index < m_pImpl->m_listSize)
			)
	{
		m_pImpl->m_pULong[index] = value;
	}
}

void ODaemonProperty::setString(const NAMESPACE_RTL(OUString)& value, sal_uInt32 index)
{
	if (m_pImpl->m_type == TPropertyType_String)
	{
		*(m_pImpl->m_pString) = value;
	}	
	else if (
			 (m_pImpl->m_type == TPropertyType_StringList) &&
			 (index < m_pImpl->m_listSize)
			) 
	{
		m_pImpl->m_pString[index] = value;
	}
}


