/*************************************************************************
 *
 *  $RCSfile: dynload.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:16 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <osl/util.h>
#ifndef _VOS_DYNLOAD_HXX_
#include <vos/dynload.hxx>
#endif
#ifndef _VOS_MODULE_HXX_
#include <vos/module.hxx>
#endif

#ifndef _RTL_USTRBUF_HXX_
#include <rtl/ustrbuf.hxx>
#endif


#ifdef UNX
#ifdef SCO
#define _IBCS2
#endif
#include <limits.h>
#define _MAX_PATH PATH_MAX
#endif

#ifdef MAC
#include <limits.h>
#endif

#ifdef WIN32
#    define DYNLOAD_CALLTYPE		__cdecl
#elif OS2
#ifdef ICC
#    define DYNLOAD_CALLTYPE		_Optlink
#else /* GCC */
#    define DYNLOAD_CALLTYPE		
#endif
#else
#    define DYNLOAD_CALLTYPE	
#endif


#ifdef UNX
#define REG_PREFIX 		"lib"	
#if defined HPUX
#define DLL_POSTFIX 	".sl"
#elif defined MACOSX 
#define DLL_POSTFIX 	".dylib.framework"
#else
#define DLL_POSTFIX 	".so"
#endif
#else
#define REG_PREFIX 		""		
#define DLL_POSTFIX 	".dll"	
#endif

#if defined WNT
#if defined MIPS
  #define COMPILER_PROCESSOR "mm"
#elif defined ICC
  #define COMPILER_PROCESSOR "cn"
#elif defined ALPHA
  #define COMPILER_PROCESSOR "ma"
#elif defined PPC
  #define COMPILER_PROCESSOR "mp"
#elif defined INTEL
  #define COMPILER_PROCESSOR "mi"
#endif
#elif defined UNX
#ifdef AIX
  #define COMPILER_PROCESSOR "ap"
#elif defined HPUX
  #define COMPILER_PROCESSOR "hr"
#elif defined SOLARIS && defined SPARC
  #define COMPILER_PROCESSOR "ss"
#elif defined SOLARIS && defined INTEL
  #define COMPILER_PROCESSOR "si"
#elif defined SCO
  #define COMPILER_PROCESSOR "ci"
#elif defined NETBSD && defined X86
  #define COMPILER_PROCESSOR "bi"
#elif defined NETBSD && defined ARM32
  #define COMPILER_PROCESSOR "ba"
#elif defined LINUX && defined X86
  #define COMPILER_PROCESSOR "li"
#elif defined SINIX
  #define COMPILER_PROCESSOR "nm"
#elif defined IRIX
  #define COMPILER_PROCESSOR "im"
#elif defined MACOSX
  #define COMPILER_PROCESSOR "mxp"
#endif
#elif defined MAC
  #define COMPILER_PROCESSOR "mp"
#endif



#ifdef _USE_NAMESPACE
namespace vos
{     
#endif

typedef	void*	(DYNLOAD_CALLTYPE *ApiInitFunction) (void);

ORealDynamicLoader::ORealDynamicLoader(ORealDynamicLoader ** ppSetToZeroInDestructor_,
									   const rtl::OUString& moduleName,
						 			   const rtl::OUString& initFunction,
									   void* pApi,
									   OModule* pModule)
	: m_pApi(pApi)
	, m_refCount(1)
	, m_pModule(pModule)
	, ppSetToZeroInDestructor( ppSetToZeroInDestructor_ )
{
	m_strModuleName = new NAMESPACE_RTL(OUString)(moduleName);
	m_strInitFunction = new NAMESPACE_RTL(OUString)(initFunction);
}

SAL_DLLEXPORT ORealDynamicLoader* ORealDynamicLoader::newInstance(ORealDynamicLoader ** ppSetToZeroInDestructor,
													const rtl::OUString& moduleName,
													const rtl::OUString& initFunction,
													sal_Bool bCreateName)
{
	NAMESPACE_RTL( OUString ) dllName;
	if (bCreateName)
	{
		computeModuleName(moduleName, dllName );
	} else
	{
		NAMESPACE_RTL(OUString) name(moduleName);
		
		rtl_uString_newFromString( &dllName.pData, name.pData ); 
	}

	ApiInitFunction initFunc;
	OModule* 		pModule = new OModule();	
	
	if (!(pModule->load(dllName)))
	{
		delete(pModule);
		return NULL;
	}
	
	initFunc = (ApiInitFunction)pModule->getSymbol(initFunction);

	if (initFunc == NULL)
	{
		delete(pModule);
		return NULL;
	}

	return(new ORealDynamicLoader(ppSetToZeroInDestructor, dllName, 
								 initFunction,
								 initFunc(),
								 pModule));
}

SAL_DLLEXPORT ORealDynamicLoader* ORealDynamicLoader::newInstance(ORealDynamicLoader ** ppSetToZeroInDestructor,
																  const rtl::OUString& moduleName,
																  const rtl::OUString& initFunction,
													sal_Bool bCreateName,
													sal_Int32 nVersion,
													sal_Bool bCPP)
{
	rtl::OUString dllName;
	if (bCreateName)
	{
		computeModuleName(moduleName, dllName, nVersion, bCPP);
	} else
	{
		NAMESPACE_RTL(OUString) name(moduleName);
		
		rtl_uString_newFromString( &dllName.pData,name.pData ); 
	}
	
	ApiInitFunction initFunc;
	OModule* 		pModule = new OModule();	
	
	if (!(pModule->load(dllName)))
	{
		delete(pModule);
		return NULL;
	}
	
	initFunc = (ApiInitFunction)pModule->getSymbol(initFunction);

	if (initFunc == NULL)
	{
		delete(pModule);
		return NULL;
	}

	return(new ORealDynamicLoader( ppSetToZeroInDestructor, dllName, 
								 initFunction,
								 initFunc(),
								 pModule));
}

ORealDynamicLoader::~ORealDynamicLoader()
{
	// set the address to zero
	if( ppSetToZeroInDestructor )
		*ppSetToZeroInDestructor = 0;

	if (m_strModuleName) delete(m_strModuleName);
	if (m_strInitFunction) delete(m_strInitFunction);
	if (m_pModule) 
	{
		m_pModule->unload();	
		delete m_pModule;
	}
}	

sal_uInt32 ORealDynamicLoader::acquire()
{ 
	return ++m_refCount; 
}

sal_uInt32 ORealDynamicLoader::release()
{ 
	sal_uInt32 nRet = --m_refCount; 
	if( nRet == 0 )
		delete this;
	return nRet; 
}


void* ORealDynamicLoader::getApi() const
{ 
		return m_pApi;		
}

SAL_DLLEXPORT sal_Int32 ORealDynamicLoader::computeLibraryName(const rtl::OUString& pPureName, 
															   rtl::OUString& pExtName )
{
	NAMESPACE_RTL(OUString) libName = rtl::OUString::createFromAscii( REG_PREFIX );
	
	libName += pPureName;
	
#ifndef OS2   
	libName += rtl::OUString::createFromAscii( DLL_POSTFIX );
#endif
	
	rtl_uString_newFromString( &pExtName.pData, libName.pData ); 
	return libName.getLength();
}	

SAL_DLLEXPORT sal_Int32 ORealDynamicLoader::computeSolarName(const rtl::OUString& rPureName, 
															 rtl::OUString& rExtName )
{
	rtl::OUStringBuffer dllName(32);
	
	dllName.append(rPureName);
	dllName.append(rtl::OUString::valueOf( osl_getSUPD() ));
	dllName.append(rtl::OUString::createFromAscii(COMPILER_PROCESSOR));
	
	rExtName = dllName.makeStringAndClear();

	return rExtName.getLength();
}	


SAL_DLLEXPORT sal_Int32 ORealDynamicLoader::computeModuleName(const rtl::OUString& prefix, 
															  rtl::OUString& moduleName )
{
	if( !computeSolarName( prefix, moduleName ) )
		return 0;
	
	return computeLibraryName(moduleName, moduleName );
}	

SAL_DLLEXPORT sal_Int32 ORealDynamicLoader::computeModuleName(const rtl::OUString& prefix, 
															  rtl::OUString& moduleName, 
															  sal_Int32 nVersion,
															  sal_Bool bCPP)
{
	sal_Int32 nResult = 0;
	
	if(nVersion)
	{
		NAMESPACE_RTL(OUString) dllName(prefix);
		
		dllName += NAMESPACE_RTL(OUString)::valueOf(nVersion);
		
		if(bCPP)
			dllName += rtl::OUString::createFromAscii( COM_STR );

		nResult = computeLibraryName( dllName, moduleName );
	}
	else
		nResult = computeModuleName(prefix, moduleName );

	return nResult;
}	

#ifdef _USE_NAMESPACE
}
#endif

