/*************************************************************************
 *
 *  $RCSfile: raw.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:18:18 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef __STL_USE_NEWALLOC
#define __STL_USE_NEWALLOC 1
#endif
#ifndef __UTILITY__
#include <stl/utility>
#endif
#ifndef __MAP__
#include <stl/map>
#endif

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _VOS_MACROS_HXX_
#include <vos/macros.hxx>
#endif

#ifndef _PROTOCOL_HXX_
#include "protocol.hxx"
#endif

#include <stdio.h>

using namespace com::sun::star::proxy;
using namespace rtl;

/*========================================================================
 *
 * OProtocolHandler internals.
 *
 *======================================================================*/
#ifndef COPYCTOR_API
#define COPYCTOR_API(C) C (const C&); C& operator= (const C&)
#endif

typedef NAMESPACE_STD(less)<sal_uInt32>                       PacketCmp;
typedef NAMESPACE_STD(map)<sal_uInt32, sal_uInt32, PacketCmp> PacketMap;

/*========================================================================
 *
 * OProtocolHandler...Impl interface.
 *
 *======================================================================*/
namespace com
{
namespace sun
{
namespace star
{
namespace proxy
{
namespace raw
{

class OProtocolHandler_Impl : public IProtocolHandler
{
public:
	OProtocolHandler_Impl (const OString &rName);
	virtual ~OProtocolHandler_Impl (void);

	/** IProtocolHandler.
	 */
	virtual void init   (void);
	virtual void update (const sal_uInt8 *pData, sal_uInt32 nSize);
	virtual void final  (void);

private:
	/** Representation.
	 */
	OString    m_aName;
	PacketMap  m_aPacketMap;
	sal_uInt32 m_pHist[256];

	/** Not implemented.
	 */
	COPYCTOR_API (OProtocolHandler_Impl);
};

class OProtocolHandlerFactory_Impl : public IProtocolHandlerFactory
{
public:
	OProtocolHandlerFactory_Impl (void);
	virtual ~OProtocolHandlerFactory_Impl (void);

	/** IProtocolHandlerFactory.
	 */
	virtual IProtocolHandler* createInstance (const OString &rName);

private:
	/** Representation.
	 */
	OUString m_aProtocol;

	/** Not implemented.
	 */
	COPYCTOR_API (OProtocolHandlerFactory_Impl);
};

} // raw
} // proxy
} // star
} // sun
} // com

using namespace com::sun::star::proxy::raw;

/*========================================================================
 *
 * OProtocolHandler_Impl implementation.
 *
 *======================================================================*/
/*
 * OProtocolHandler_Impl.
 */
OProtocolHandler_Impl::OProtocolHandler_Impl (const OString &rName)
	: m_aName (rName)
{
}

/*
 * ~OProtocolHandler_Impl.
 */
OProtocolHandler_Impl::~OProtocolHandler_Impl (void)
{
}

/*
 * init.
 */
void OProtocolHandler_Impl::init (void)
{
	// Packet Data Histogram.
	for (sal_uInt32 i = 0; i < 256; i++)
		m_pHist[i] = 0;

	// Packet Size Histogram.
	m_aPacketMap.clear();
}

/*
 * update.
 */
void OProtocolHandler_Impl::update (const sal_uInt8 *pData, sal_uInt32 nSize)
{
	// Packet Data Histogram.
	for (sal_uInt32 i = 0; i < nSize; i++)
		m_pHist[pData[i]]++;

	// Packet Size Histogram.
	sal_uInt32 n = m_aPacketMap[nSize];
	m_aPacketMap[nSize] = n + 1;
}

/*
 * final.
 */
void OProtocolHandler_Impl::final (void)
{
	// Packet Data Histogram.
	fprintf (
		stdout, "%s (Packet Data Histogram)\n",
		m_aName.pData->buffer);

	for (sal_uInt32 i = 0; i < 256; i++)
		fprintf (stdout, "%d: %d\n", i, m_pHist[i]);

	// Packet Size Histogram.
	PacketMap::const_iterator first, last;
	first = m_aPacketMap.begin();
	last  = m_aPacketMap.end();
	if (first != last)
	{
		fprintf (
			stdout, "%s (Packet Size Histogram)\n",
			m_aName.pData->buffer);

		sal_uInt32 nData = 0, nSize = 0;
		for (PacketMap::const_iterator it = first; it != last; ++it)
		{
			sal_uInt32 k = (*it).first;
			sal_uInt32 n = (*it).second;
			fprintf (stdout, "%d: %d\n", k, n);
			nData += n;
			nSize += n * k;
		}

		fprintf (
			stdout, "%s (Packet Size Histogram) (%d, %d)\n",
			m_aName.pData->buffer, nData, nSize);
	}

	// ...
	fflush (stdout);
}

/*========================================================================
 *
 * OProtocolHandlerFactory_Impl implementation.
 *
 *======================================================================*/
/*
 * The single instance.
 */
static OProtocolHandlerFactory_Impl g_aFactory;

/*
 * OProtocolHandlerFactory_Impl.
 */
OProtocolHandlerFactory_Impl::OProtocolHandlerFactory_Impl (void)
	: m_aProtocol (OUString::createFromAscii ("raw"))
{
	OProtocolHandlerRegistry::registerFactory (m_aProtocol, this);
}

/*
 * ~OProtocolHandlerFactory_Impl.
 */
OProtocolHandlerFactory_Impl::~OProtocolHandlerFactory_Impl (void)
{
	OProtocolHandlerRegistry::deregisterFactory (m_aProtocol, this);
}

/*
 * createInstance.
 */
IProtocolHandler*
OProtocolHandlerFactory_Impl::createInstance (const OString &rName)
{
	return new OProtocolHandler_Impl (rName);
}

