#include <sys/types.h>
#include <memory.h>
#include <stdio.h>

#include <bdb/db.h>

#define DBFFIELDLEN_URL		200
#define DBFFIELDLEN_ID		19


int main( int argc, char* argv[] )
{
	if ( argc != 3 )
	{
		fprintf( stderr, "Usage: DBF2DB dbf-filename berkeley-filename" );
		return 1;
	}
	else
	{
		DB* dbp;
		DBT key, data;
		if ( db_create( &dbp, NULL, 0 ) )
		{
			fprintf( stderr, "Error initializing BDB" );
			return 2;
		}

		FILE* pDBFFile = fopen( argv[1], "r" );
		if ( !pDBFFile )
		{
			fprintf( stderr, "Error open dbf file" );
			return 3;
		}
		
		if ( dbp->open( dbp, argv[2], NULL, DB_BTREE, DB_CREATE|DB_TRUNCATE, 0664 ) )
		{
			fprintf( stderr, "Error creating berkeley database file" );
			return 4;
		}
		
		char nDBFVersion;
		unsigned short nHeaderLen;
		unsigned long nRecords;
		char buffer[256];
		char buffer2[256];
		fread( &nDBFVersion, 1, 1, pDBFFile );
		fread( buffer, 3, 1, pDBFFile );	// Date
		fread( &nRecords, 4, 1, pDBFFile );
		fread( &nHeaderLen, 2, 1, pDBFFile );	// Header Len
//		fread( buffer, 20, 1, pDBFFile );	// Reserved

		fseek( pDBFFile, nHeaderLen, SEEK_SET );

		fprintf( stdout, "DBF-Version: %c\n\n", nDBFVersion+'0' );
		
		for ( unsigned long n = 0; n < nRecords; n++ )
		{
			fread( buffer, 1, 1, pDBFFile ); // Delete mark

			fread( buffer, DBFFIELDLEN_URL, 1, pDBFFile );
			char* pURLStart = buffer+1;
			char* pURLEnd = buffer+DBFFIELDLEN_URL-1;
			// Skip trailing blanks
			while ( *pURLEnd == ' ' )
				pURLEnd--;
			pURLEnd++;

			fread( buffer2, DBFFIELDLEN_ID, 1, pDBFFile );
			char* pIDStart = buffer2;
			char* pIDEnd = pIDStart;
			while ( *pIDEnd )
				pIDEnd++;

			*pURLEnd = 0;
			*pIDEnd = 0;
				
			fprintf( stdout, "ID: %s, URL: %s\n", pIDStart, pURLStart );

			memset(&key, 0, sizeof(key));
			memset(&data, 0, sizeof(data));
			key.data = pIDStart;
			key.size = pIDEnd-pIDStart;
			data.data = pURLStart;
			data.size = pURLEnd-pURLStart;
		
			if ( dbp->put(dbp, NULL, &key, &data, 0) )
			{
				fprintf( stderr, "Error storing into database!" );
				return 5;
			}
		}

		dbp->close(dbp, 0);
	}
}


