/*************************************************************************
 *
 *  $RCSfile: pkgurl.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:31:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _COM_SUN_STAR_XML_SAX_XDOCUMENTHANDLER_HPP_
#include <com/sun/star/xml/sax/XDocumentHandler.hpp>
#endif

#ifndef _GRFMGR_HXX
#include <goodies/grfmgr.hxx>
#endif

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#ifndef _SV_CVTGRF_HXX
#include <vcl/cvtgrf.hxx>
#endif

#include <rtl/strbuf.hxx>
#include <assert.h>

#ifndef _URLPACKER_HXX
#include <pkgurl.hxx>
#endif

#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

#ifndef _XMLOFF_ATTRLIST_HXX
#include <xmloff/attrlist.hxx>
#endif

using namespace ::rtl;
using namespace ::com::sun::star;
using namespace ::com::sun::star::uno;

URLPacker::URLPacker( const ::rtl::OUString& rBase,
				::com::sun::star::uno::Reference <::com::sun::star::xml::sax::XDocumentHandler> xParent,  
				sal_Bool bIAbsolute,
				sal_Bool bIExport) :
	sMainXMLFile(rBase),
	xHandler(xParent),
	b2Absolute(bIAbsolute),
	bExport(bIExport)
{
	INetURLObject Temp(rBase);
	Temp.removeSegment();
	sAbsolutePath = Temp.GetMainURL();
	if (bExport)
	{
		pIDList = new SvStringsSortDtor;
		pGrafList = 0;
	}
	else
	{
		pGrafList = new URLGraphicList;
		pIDList = 0;
	}
}
URLGraphicData::URLGraphicData(ByteString& rID):
	sOldID (rID)
{
}
URLGraphicData::~URLGraphicData()
{
}
GraphicObject* URLGraphicList::Contains( ByteString& rID)
{
	URLGraphicData *pTemp;
	for (pTemp = First();
			pTemp != NULL;
			pTemp = Next())
	{
		if (pTemp->sOldID == rID)
			return &(pTemp->aGrafObj);
	}
	return NULL;
}

void SAL_CALL URLPacker::startDocument(void)
{
	xHandler->startDocument();
}

void SAL_CALL URLPacker::endDocument(void)
{
	xHandler->endDocument();
}

void SAL_CALL URLPacker::endElement(const ::rtl::OUString& aName)
{
	xHandler->endElement(aName);
}
	
void SAL_CALL URLPacker::characters(const ::rtl::OUString& aChars)
{
	xHandler->characters(aChars);
}

void SAL_CALL URLPacker::ignorableWhitespace(const ::rtl::OUString& aWhitespaces)
{
	xHandler->ignorableWhitespace(aWhitespaces);
}
void SAL_CALL URLPacker::processingInstruction(const ::rtl::OUString& aTarget,
			const ::rtl::OUString& aData) 
{
	xHandler->processingInstruction(aTarget, aData);
}
void SAL_CALL URLPacker::setDocumentLocator(const ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XLocator > & xLocator)
{
	xHandler->setDocumentLocator( xLocator);
}
URLPacker::~URLPacker()
{
	URLGraphicData *pTmp;
	if (pGrafList)
		for (pTmp=pGrafList->First();
			pTmp!= NULL;
			pTmp=pGrafList->Next())
		{
			delete pTmp;
			pGrafList->Remove();
		}
	if (bExport)
		delete pIDList;
	else
		delete pGrafList;
}

void SAL_CALL URLPacker::startElement(const OUString& rElementName,
									   const Reference< xml::sax::XAttributeList > & xAttribs) 
{
	sal_Int32 nAttribCount = xAttribs.is() ? xAttribs->getLength() : 0;
	Reference < xml::sax::XAttributeList > xNewAttrList = xAttribs;

	for (sal_Int32 i=0 ; i <nAttribCount ; i++)
	{
		const OUString& rName  = xAttribs->getNameByIndex(i);
		if (rName.equalsAsciiL("href", 4) ||
			rName.equalsAsciiL("xlink:href", 10)  )
		{
			OUString sValue (xAttribs->getValueByIndex(i));
			const OUString& sType  = OUString(xAttribs->getTypeByIndex(i));
			SvXMLAttributeList* pAttrList;

			if (!bExport)
			{ // Import filter
				// Relative to Absolute
				INetURLObject aURLObj( sValue);
				INetURLObject aBaseURL( sAbsolutePath);
					

				if (aURLObj.GetProtocol() == INET_PROT_NOT_VALID)
				{
					// Assume the protocol is the same as the main Document
					aURLObj = INetURLObject(sValue, aBaseURL.GetProtocol());
				}

				if (aURLObj.GetProtocol() == INET_PROT_FILE ) 
				{
					INetURLObject aURLObj;
					
					aBaseURL.GetNewAbsURL(sValue, &aURLObj);
					//sValue = sValue.copy (aURLObj.GetScheme ( INET_PROT_FILE ).Len());
					//aBaseURL.setName(sValue, -1, false);
					OUString sExtension(aURLObj.getExtension());
					if (COMPARE_EQUAL == sExtension.compareToAscii ("jpg") ||
						COMPARE_EQUAL == sExtension.compareToAscii ("gif") )
					{
						aURLObj.removeExtension();
						sValue = aURLObj.getName();
						aURLObj.setExtension(sExtension);

						String sIdentifier (sValue);
						ByteString aUniqueID( sIdentifier, RTL_TEXTENCODING_UTF8 );
						GraphicObject *pGrafObject = NULL;
						
						if ( (pGrafObject = pGrafList->Contains(aUniqueID)) == NULL)
						{
							URLGraphicData* pGrafData = new URLGraphicData(aUniqueID);
							pGrafList->Insert(pGrafData);
							LoadImage(aURLObj, pGrafData->aGraphic, pGrafData->aGrafObj);
							pGrafObject = &pGrafData->aGrafObj;
						}
						sValue = OUString ( RTL_CONSTASCII_USTRINGPARAM("vnd.sun.star.GraphicObject:"));			
						sValue += OUString::createFromAscii( pGrafObject->GetUniqueID().GetBuffer() );
						pAttrList = new SvXMLAttributeList();
						pAttrList->SetAttributeList(xAttribs);
						pAttrList->RemoveAttribute( rName );
						pAttrList->AddAttribute(rName, sType, sValue);
						xNewAttrList = pAttrList;
						break;
					}
				}
			}
			else
			{ // Export Filter
				INetURLObject aURLObj( sAbsolutePath );

				if( 0 == sValue.compareToAscii( RTL_CONSTASCII_STRINGPARAM("vnd.sun.star.GraphicObject:") ) )
				{
					// graphic manager url
					sValue =sValue.copy( sizeof ("vnd.sun.star.GraphicObject:" ) -1 );
					aURLObj.setName(sValue, -1, false);
					SaveImage( aURLObj);
					sValue = aURLObj.GetMainURL();
				}
				if ( !b2Absolute && aURLObj.GetProtocol() == INET_PROT_FILE ) 
				{
					//sValue = INetURLObject::GetScheme(INET_PROT_FILE) + INetURLObject::GetRelURL(sAbsolutePath, aURLObj.GetMainURL());
					sValue = INetURLObject::GetRelURL(sAbsolutePath, aURLObj.GetMainURL());
				}
				pAttrList = new SvXMLAttributeList();
				pAttrList->SetAttributeList(xAttribs);
				pAttrList->RemoveAttribute(rName);
				pAttrList->AddAttribute(rName, sType, sValue);
				xNewAttrList = pAttrList;
				break;
				
				//xHandler->startElement(rElementName, (xml::sax::XAttributeList*)pAttrList);
				//return;
			}
		}
	}
	//xHandler->startElement(rElementName, xAttribs);
	xHandler->startElement(rElementName, xNewAttrList);
}

void URLPacker::SaveImage( INetURLObject& rURLObj )
{	
	BOOL bRet = FALSE;
	ULONG nRet;
	String sIdentifier = String(rURLObj.getName());
	
	ByteString aUniqueID( String(rURLObj.getName()),RTL_TEXTENCODING_UTF8 );
	GraphicObject aGrafObj( aUniqueID );
	Graphic aGraphic( aGrafObj.GetGraphic() );
	
	if( aGraphic.GetType() != GRAPHIC_NONE )
	{
	//	INetURLObject	aURL( CreateUniqueFileName( GetParent(), SGA_OBJ_BMP ) );
		SvFileStream	aOStm;

		if( aGraphic.GetType() == GRAPHIC_BITMAP )
		{
			ULONG nExportFormat;

			if( aGraphic.IsAnimated() )
			{
				rURLObj.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "gif" ) ) );
				nExportFormat = CVT_GIF;
			}
			else
			{
				rURLObj.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "jpg" ) ) );
				nExportFormat = CVT_JPG;
			}
			if (!pIDList->Seek_Entry(&sIdentifier))
			{

				aOStm.Open( rURLObj.PathToFileName(), STREAM_WRITE | STREAM_TRUNC );

				if( aOStm.IsOpen() )
				{
					aOStm.SetVersion( SOFFICE_FILEFORMAT_NOW );
	
					bRet = (nRet = GraphicConverter::Export( aOStm, aGraphic, nExportFormat ) ) == ERRCODE_NONE;
					aOStm.Close();
				}
				pIDList->Insert ( new String(sIdentifier));
			}
		}
		else
		{
			rURLObj.SetExtension( String( RTL_CONSTASCII_USTRINGPARAM( "svm" ) ) );
			if (!pIDList->Seek_Entry(&sIdentifier))
			{
 
				aOStm.Open( rURLObj.PathToFileName(), STREAM_WRITE | STREAM_TRUNC );

				if( aOStm.IsOpen() )
				{
					GDIMetaFile aMtf( aGraphic.GetGDIMetaFile() );
					aOStm.SetVersion( SOFFICE_FILEFORMAT_NOW );
					aMtf.Write( aOStm );
					bRet = ( aOStm.GetError() == ERRCODE_NONE );
					aOStm.Close();
				}
				pIDList->Insert ( new String(sIdentifier));
			}
		}
	}
}

void URLPacker::LoadImage( INetURLObject& rURLObj, Graphic& rGraphic, GraphicObject& rGrafObj)
{	
	BOOL bRet = FALSE;
	ULONG nRet;
	SvFileStream aIStm;
    OUString Temp(rURLObj.PathToFileName());
	aIStm.Open( rURLObj.PathToFileName(), STREAM_STD_READ );

	if( aIStm.IsOpen() )
	{
		nRet = GraphicConverter::Import( aIStm, rGraphic);
		aIStm.Close();
	}

	rGrafObj.SetGraphic(rGraphic);
}
