/*************************************************************************
 *
 *  $RCSfile: xmltkmap.cxx,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/19 00:31:44 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _SVARRAY_HXX
#include <svtools/svarray.hxx>
#endif

#ifndef _XMLOFF_XMLTKMAP_HXX
#include "xmltkmap.hxx"
#endif

using namespace rtl;

class SvXMLTokenMapEntry_Impl
{
	sal_uInt16	nPrefixKey;
	OUString	sLocalName;
	sal_uInt16	nToken;

public:

	sal_uInt16 GetToken() const { return nToken; }

	SvXMLTokenMapEntry_Impl( sal_uInt16 nPrefix, const OUString& rLName,
						     sal_uInt16 nTok=XML_TOK_UNKNOWN ) :
		nPrefixKey( nPrefix ),
		sLocalName( rLName  ),
		nToken( nTok )
	{}

	SvXMLTokenMapEntry_Impl( const SvXMLTokenMapEntry& rEntry ) :
		nPrefixKey( rEntry.nPrefixKey ),
		sLocalName( OUString::createFromAscii(rEntry.pLocalName) ),
		nToken( rEntry.nToken )
	{}

	BOOL operator==( const SvXMLTokenMapEntry_Impl& r ) const
	{
		return nPrefixKey == r.nPrefixKey &&
			   sLocalName == r.sLocalName;
	}

	BOOL operator<( const SvXMLTokenMapEntry_Impl& r ) const
	{
		return nPrefixKey < r.nPrefixKey ||
			   ( nPrefixKey == r.nPrefixKey &&
				 sLocalName < r.sLocalName);
	}
};

typedef SvXMLTokenMapEntry_Impl *SvXMLTokenMapEntry_ImplPtr;
SV_DECL_PTRARR_SORT_DEL( SvXMLTokenMap_Impl, SvXMLTokenMapEntry_ImplPtr, 5, 5 )
SV_IMPL_OP_PTRARR_SORT( SvXMLTokenMap_Impl, SvXMLTokenMapEntry_ImplPtr )

// ---------------------------------------------------------------------

SvXMLTokenMapEntry_Impl *SvXMLTokenMap::_Find( sal_uInt16 nKeyPrefix,
									 		   const OUString& rLName ) const
{
	SvXMLTokenMapEntry_Impl *pRet = 0;
	SvXMLTokenMapEntry_Impl aTst( nKeyPrefix, rLName );

	sal_uInt16 nPos;
	if( pImpl->Seek_Entry( &aTst, &nPos ) )
	{
		pRet = (*pImpl)[nPos];
	}

	return pRet;
}

SvXMLTokenMap::SvXMLTokenMap( SvXMLTokenMapEntry *pMap ) :
	pImpl( new SvXMLTokenMap_Impl )
{
	while( pMap->pLocalName )
	{
		pImpl->Insert( new SvXMLTokenMapEntry_Impl( *pMap ) );
		pMap++;
	}
}

SvXMLTokenMap::~SvXMLTokenMap()
{
	delete pImpl;
}

sal_uInt16 SvXMLTokenMap::Get( sal_uInt16 nKeyPrefix,
							   const OUString& rLName ) const
{
	SvXMLTokenMapEntry_Impl *pEntry = _Find( nKeyPrefix, rLName );
	if( pEntry )
		return pEntry->GetToken();
	else
		return XML_TOK_UNKNOWN;
}


