/*************************************************************************
 *
 *  $RCSfile: file.c,v $
 *
 *  $Revision: 1.1.1.1 $
 *
 *  last change: $Author: hr $ $Date: 2000/09/18 15:17:21 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

/*
 *   to do:
 *   - Fix: reorganize this biest
 *   - Fix: reorganize this biest!
 *   - Fix: reorganize this biest!!
 *   - Fix: reorganize this biest!!!
 *   - Fix: reorganize this biest!!!!
 *   - Fix: reorganize this biest!!!!!
 *   - Fix: reorganize the source, make it more cleanly, properly, reliable
 *   - Fix: acquire and release all floppy handles properly!!!
 *   - Fix: free all char* derived from rtl_ustrings properly
 *          (via oslGetNativePath)
 *   - Fix: free all memory properly
 *   - Fix: check for corresponding struct sizes in exported functions
 *   - check size/use of oslDirectory
 *   - check size/use of oslDirectoryItem
 *   - check size/use of oslFileStatus
 *   - check size/use of oslVolumeDeviceHandle
 *   - check size/use of oslVolumeInfo
 *   - check size/use of oslFileHandle
 *
 *   done:
 *   - Fix: FD check in all Handle uses
 *   - Fix: make rtl_uString thread save with acquire and release!
 *   - Fix: check for relative Path Names
 *   - Fix: remove UNC prefix from filenames (check only)
 *   - FIx: Validate flag as 'exists'
 *   - Fix: truncate in setFileSize
 *   - Fix: convertion to off_t evtly return OVERFLOW
 *   - Fix: osl_getAbsolutePath
 *   - Fix: osl_copyFile
 *   - Fix: osl_moveFile
 *   - Fix: set ValidFields correctly in getFileStatus
 *   - Fix: set ValidFields correctly in getVolumeInformation
 *   - Fix: attributes in getFileStatus
 *   - Fix: osl_getCanonicalName
 *   - Fix: open only a regular File
 *   - Fix: copy move links correctly
 *   - Fix: make proper native path/file names from UNC ones
 */

#include "system.h"
#include "rtl/alloc.h"
#include "osl/file.h"
#include <rtl/string.h>
#include <osl/thread.h>

#if defined(SOLARIS)
#include <sys/mnttab.h>
#include <sys/statvfs.h>
#include <sys/fs/ufs_quota.h>
static const sal_Char* MOUNTTAB="/etc/mnttab";
#endif /* SOLARIS */

#if defined(LINUX)
#include <mntent.h>
#include <sys/vfs.h>
#include <sys/quota.h>
#include <sys/mount.h>
#include <ctype.h>
static const sal_Char* MOUNTTAB="/etc/mtab";
#endif /* LINUX */

#if defined(MACOSX)
#include <ufs/ufs/quota.h>
#include <ctype.h>
static const sal_Char* MOUNTTAB="/etc/mtab";
#endif /* MACOSX */

#include <sys/mman.h>

#if defined(DEBUG)
/*  #define DEBUG_OSL_FILE */
/*  #define TRACE_OSL_FILE */
#endif

/*****************************************************************************/
/* Data Type Definition */
/*****************************************************************************/

#define rtl_uString_newFromAscii( dest, src )	\
	rtl_string2UString( dest, src, rtl_str_getLength( src ), osl_getThreadTextEncoding(), OUSTRING_TO_OSTRING_CVTFLAGS )

/* #define RTL_TEXTENCODING_UTF8	osl_getThreadTextEncoding()*/

#define OSL_MAX_FILE_NAME   1024
#define OSL_MAX_FILE_PATH   1024
#define OSL_MAX_NATIVE_PATH 1024


typedef struct
{
    rtl_uString* osl_FileName;  /* Holds native FileName (Not in UNC Notation!) */
    rtl_uString* osl_DirName;   /* Holds native DirName (Not in UNC Notation!) */
    sal_uInt32   RefCount;
} oslDirectoryItemImpl;


typedef struct
{
    rtl_uString* osl_DirName;   /* Holds native DirName (Not in UNC Notation!) */
    rtl_uString* osl_FileName;  /* Holds native FileName (Not in UNC Notation!) */
    DIR* aDir;
} oslDirectoryImpl;


typedef struct
{
    rtl_uString* filepathname;  /* Holds native Path and FileName (Not in UNC Notation!) */
    int fd;
} oslFileHandleImpl;



typedef struct _oslVolumeDeviceHandleImpl
{
    sal_Char pszMountPoint[PATH_MAX];
    sal_Char pszFilePath[PATH_MAX];
    sal_Char pszDevice[PATH_MAX];
    sal_Char ident[4];
    sal_uInt32   RefCount;
} oslVolumeDeviceHandleImpl;

extern sal_Bool osl_getFullPath(const sal_Char* pszFilename, sal_Char* pszPath, sal_uInt32 MaxLen);

/*****************************************************************************/
/* Static Module Functions Declarations */
/*****************************************************************************/

static oslFileError oslIsMountPoint( sal_Char* pszFileName, sal_Bool* bMountPoint );
static oslFileError oslTranslateFileError( int nErr );
static oslFileError oslChangeFileModes( sal_Char* pszFileName, mode_t nMode, time_t nAcTime, time_t nModTime, uid_t nUID, gid_t nGID );
static oslFileError oslDoCopy( sal_Char* pszSourceFileName, sal_Char* pszDestFileName, mode_t nMode, size_t nSourceSize, int DestFileExists );
static int          oslDoCopyLink( sal_Char* pszSourceFileName, sal_Char* pszDestFileName );
static int          oslDoCopyFile( sal_Char* pszSourceFileName, sal_Char* pszDestFileName, size_t nSourceSize );
static sal_Bool     oslCheckUNC( sal_Char* pszPathName );
static sal_Char*    oslGetBaseName( sal_Char* pszPath );
static sal_Char*    oslGetDirName( sal_Char* pszPath );
static sal_Char*    oslGetNativeName( rtl_uString* strPathName );
static void         oslMakeUniqueName( sal_Char* pszRequested, sal_Char* pszUnique, sal_uInt32 nIndex );
static sal_Char*    oslGetNativePath( sal_Char* pszUNCPath );
static sal_Char*    oslSeparatePathEntry( sal_Char* pszUNCPath, sal_Char cSeparator );
static oslFileError oslCheckForExistence( rtl_uString* strFilePath );
static sal_Char*    oslDoGetAbsolutePath( sal_Char* pszDirBase, sal_Char* pszRelative );


/*#if defined(SOLARIS)*/
/*
 *  'removeable device' aka floppy functions
 */

static oslVolumeDeviceHandle  osl_isFloppyDrive(const sal_Char* pszPath);
static oslFileError   osl_mountFloppy(oslVolumeDeviceHandle hFloppy);
static oslFileError   osl_unmountFloppy(oslVolumeDeviceHandle hFloppy);


#if defined(SOLARIS)
static sal_Bool       osl_isFloppyMounted(sal_Char* pszPath, sal_Char* pszMountPath);
static sal_Bool       osl_getFloppyMountEntry(const sal_Char* pszPath, sal_Bool* bMountPoint, sal_Char* pBuffer);
static sal_Bool       osl_checkFloppyPath(sal_Char* pszPath, sal_Char* pszFilePath, sal_Char* pszDevicePath);
#endif

#if defined(LINUX)
static sal_Bool       osl_isFloppyMounted(oslVolumeDeviceHandleImpl* pDevice);
static sal_Bool       osl_getFloppyMountEntry(const sal_Char* pszPath, oslVolumeDeviceHandleImpl* pItem);
#endif

/*static sal_Char*      osl_getFloppyPath(oslVolumeDeviceHandle hFloppy);*/
/*  static sal_Bool       osl_isFloppyDrivePresent(); */

/*  static sal_Bool       osl_isFloppyInserted(oslVolumeDeviceHandle hFloppy); */

/*#endif*/

static sal_Char*      osl_getBaseName(sal_Char* pszPath);
static sal_Char*      osl_getDirName(sal_Char* pszPath);

#ifdef DEBUG_OSL_FILE
static void osl_printFloppyHandle(oslVolumeDeviceHandleImpl* hFloppy);
#endif


/*****************************************************************************/
/* Exported Module Functions */
/*****************************************************************************/

oslFileError osl_openDirectory( rtl_uString* strDirectoryPath, oslDirectory* pDirectory )
{
    oslDirectoryImpl* pDirImpl=0;
    int nRet=0;
    sal_Bool bRet=sal_False;
    sal_Char* pszUNCPath=0;
    sal_Char* pszDirectoryPath=0;
    sal_Char AbsDirName[OSL_MAX_NATIVE_PATH] = "";
    DIR* pDirStruct=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_openDirectory\n");
#endif

    if ( pDirectory == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [pDirectory == 0]\n");
#endif
        return osl_File_E_INVAL;        
    }
    
    if ( strDirectoryPath == 0)
    {
        *pDirectory=0;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [strDirectoryPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strDirectoryPath);
    pszUNCPath=oslGetNativeName(strDirectoryPath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strDirectoryPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [pszUNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strDirectoryPath);
        rtl_freeMemory(pszUNCPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszDirectoryPath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strDirectoryPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [No Native Path]\n");
#endif
        return osl_File_E_INVAL;
    }


#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Dir to Open is '%s'\n",pszDirectoryPath);
#endif

    if ( (pDirStruct = opendir(pszDirectoryPath)) == 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("opendir");
#endif
        rtl_freeMemory(pszDirectoryPath);
        rtl_uString_release(strDirectoryPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [opendir]\n");
#endif
        return oslTranslateFileError(nRet);;
    }

    AbsDirName[0]=0;
    bRet = osl_getFullPath(pszDirectoryPath,AbsDirName,OSL_MAX_NATIVE_PATH);
#ifdef DEBUG_OSL_FILE
/*    fprintf(stderr,"Abs. Dir Name  : '%s' (bRet == %i)\n",AbsDirName,bRet);*/
#endif

    pDirImpl= (oslDirectoryImpl*) rtl_allocateMemory(sizeof(oslDirectoryImpl));
    if ( pDirImpl == 0 )
    {
        rtl_uString_release(strDirectoryPath);
        rtl_freeMemory(pszDirectoryPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }
    memset(pDirImpl,0,sizeof(oslDirectoryImpl));

    rtl_uString_newFromAscii(&(pDirImpl->osl_DirName),AbsDirName);
    if ( pDirImpl->osl_DirName == 0 )
    {
        rtl_uString_release(strDirectoryPath);
        rtl_freeMemory(pszDirectoryPath);
        rtl_freeMemory(pDirImpl);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectory [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

    pDirImpl->aDir=pDirStruct;
    pDirImpl->osl_FileName=0;

    rtl_uString_release(strDirectoryPath);
    rtl_freeMemory(pszDirectoryPath);
    *pDirectory=pDirImpl;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_openDirectory [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_getNextDirectoryItem( oslDirectory Directory, oslDirectoryItem* pItem, sal_uInt32 uRequested)
{
    oslDirectoryImpl* pDirImpl=0;
    oslDirectoryItemImpl* pDirItemImpl=0;
    DIR* pDirStruct=0;
    struct dirent* de=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getNextDirectoryItem\n");
#endif

    pDirImpl = (oslDirectoryImpl*) Directory;

    if ( pDirImpl == 0 || pItem == 0 )
    {
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNextDirectoryItem [pDir == 0x%08X pItem == 0x%08X Requested == %i]\n",Directory,pItem,uRequested);
#endif
        return osl_File_E_INVAL;
    }

    pDirStruct=pDirImpl->aDir;

    de = readdir( pDirStruct );

    while ( de != 0 && de->d_name[0] == '.' && ( de->d_name[1] == '.' || de->d_name[1] == 0 ) )
    {
        de = readdir( pDirStruct );
    }

    if ( de == 0 )
    {
        *pItem=0;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNextDirectoryItem [NOENT]\n");
#endif
        return osl_File_E_NOENT;
    }
    
    pDirItemImpl = (oslDirectoryItemImpl*) rtl_allocateMemory(sizeof(oslDirectoryItemImpl));
    if ( pDirItemImpl == 0 )
    {
        *pItem=0;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNextDirectoryItem [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }
    memset(pDirItemImpl,0,sizeof(oslDirectoryItemImpl));

    rtl_uString_newFromAscii(&(pDirItemImpl->osl_FileName),de->d_name);
    if ( pDirItemImpl->osl_FileName == 0 )
    {
        *pItem=0;
        rtl_freeMemory(pDirItemImpl);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNextDirectoryItem [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }
    
    rtl_uString_newFromString(&(pDirItemImpl->osl_DirName),pDirImpl->osl_DirName);
    if ( pDirItemImpl->osl_DirName == 0 )
    {
        rtl_uString_release(pDirItemImpl->osl_FileName);
        *pItem=0;
        rtl_freeMemory(pDirItemImpl);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNextDirectoryItem [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

#ifdef DEBUG_OSL_FILE
    {
        sal_Char* p;
        
        p=oslGetNativeName(pDirItemImpl->osl_FileName);
        if ( p != 0 )
        {
            fprintf(stderr,"Entry File is : '%s'\n",p);
            rtl_freeMemory(p);
        }

        p = oslGetNativeName(pDirItemImpl->osl_DirName);
        if ( p != 0 )
        {                
            fprintf(stderr,"Entry Dir  is : '%s'\n",p);
            rtl_freeMemory(p);
        }
    }
#endif

    pDirItemImpl->RefCount=1;    
    *pItem=pDirItemImpl;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getNextDirectoryItem [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_closeDirectory( oslDirectory Directory )
{
    int nRet=0;
    oslDirectoryImpl* pDirImpl=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_closeDirectory\n");
#endif

    pDirImpl=(oslDirectoryImpl*) Directory;

    if ( pDirImpl == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_closeDirectory [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    nRet=closedir(pDirImpl->aDir);
    if ( nRet != 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("closedir");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_closeDirectory [closedir]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    if ( pDirImpl->osl_FileName != 0 )
    {
        rtl_uString_release(pDirImpl->osl_FileName);
    }

    if ( pDirImpl->osl_DirName != 0 )
    {
        rtl_uString_release(pDirImpl->osl_DirName);
    }

    rtl_freeMemory(pDirImpl);
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_closeDirectory [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_getDirectoryItem( rtl_uString* strFilePath, oslDirectoryItem* pItem )
{
    int nRet=0;
    oslDirectoryItemImpl* pDirItemImpl=0;
    sal_Char AbsPath[OSL_MAX_NATIVE_PATH] = "";
    sal_Char* pszUNCPath=0;
    sal_Char* pszFilePath=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getDirectoryItem\n");
#endif

    if ( strFilePath == 0 || pItem == 0 )
    {
        if ( pItem != 0 )
        {    
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    
    rtl_uString_acquire(strFilePath);
    pszUNCPath=oslGetNativeName(strFilePath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strFilePath);
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openDirectoryItem [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ! oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszUNCPath);
        
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszFilePath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strFilePath);
        if ( pItem != 0 )
        {    
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [No Native Path]\n");
#endif
        return osl_File_E_INVAL;
    }
    

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"FileNamePath : '%s'\n",pszFilePath);
#endif

    nRet=access(pszFilePath,F_OK);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("access");
#endif
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszFilePath);
        
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [access]\n");
#endif
        return oslTranslateFileError(nRet);
    }
    
    pDirItemImpl = (oslDirectoryItemImpl*) rtl_allocateMemory(sizeof(oslDirectoryItemImpl));
    if ( pDirItemImpl == 0 )
    {
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszFilePath);

        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }
    memset(pDirItemImpl,0,sizeof(oslDirectoryItemImpl));

    osl_getFullPath(pszFilePath,AbsPath,OSL_MAX_NATIVE_PATH);
#ifdef DEBUG_OSL_FILE
/*      fprintf(stderr,"Abs Path     : '%s'\n",AbsPath); */
/*      fprintf(stderr,"DirName      : '%s'\n",oslGetDirName(AbsPath)); */
#endif

    rtl_uString_newFromAscii(&(pDirItemImpl->osl_DirName),oslGetDirName(AbsPath));
    if ( pDirItemImpl->osl_DirName == 0 )
    {
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pDirItemImpl);
        rtl_freeMemory(pszFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

    osl_getFullPath(pszFilePath,AbsPath,OSL_MAX_NATIVE_PATH);
#ifdef DEBUG_OSL_FILE
/*      fprintf(stderr,"Abs Path     : '%s'\n",AbsPath); */
/*      fprintf(stderr,"FileName     : '%s'\n",oslGetBaseName(AbsPath)); */
#endif

    rtl_uString_newFromAscii(&(pDirItemImpl->osl_FileName),oslGetBaseName(AbsPath));
    if ( pDirItemImpl->osl_FileName == 0 )
    {
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pDirItemImpl);
        rtl_freeMemory(pszFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

#ifdef DEBUG_OSL_FILE
    {
        sal_Char* p;

        p=oslGetNativeName(pDirItemImpl->osl_FileName);
        if ( p != 0 )
        {
            fprintf(stderr,"FileName is '%s'\n",p);
            rtl_freeMemory(p);
        }
    }
#endif

    pDirItemImpl->RefCount=1;
    
    rtl_uString_release(strFilePath);
    rtl_freeMemory(pszFilePath);
    *pItem=pDirItemImpl;
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getDirectoryItem [ok]\n");
#endif

    return osl_File_E_None;
}

oslFileError osl_acquireDirectoryItem( oslDirectoryItem Item )
{
    oslDirectoryItemImpl* pDirItem = (oslDirectoryItemImpl*) Item;

    if ( pDirItem == 0 )
    {
        return osl_File_E_INVAL;
    }
    
    pDirItem->RefCount++;
    
    return osl_File_E_None;
}

oslFileError osl_releaseDirectoryItem( oslDirectoryItem Item )
{
    oslDirectoryItemImpl* pDirItemImpl=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_releaseDirectoryItem\n");
#endif
    
    pDirItemImpl = (oslDirectoryItemImpl*) Item;

    if ( pDirItemImpl == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_releaseDirectoryItem [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    pDirItemImpl->RefCount--;

    if ( pDirItemImpl->RefCount < 0 )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"negativ RefCount in Handle!!!\n");
#endif
    }
    
    if ( pDirItemImpl->RefCount == 0 )
    {
        
        if ( pDirItemImpl->osl_FileName != 0 )
        {
            rtl_uString_release(pDirItemImpl->osl_FileName);
        }

        if ( pDirItemImpl->osl_DirName != 0 )
        {
            rtl_uString_release(pDirItemImpl->osl_DirName);
        }

        rtl_freeMemory(pDirItemImpl);
    }
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_releaseDirectoryItem [ok]\n");
#endif
    
    return osl_File_E_None;
}


oslFileError osl_getFileStatus( oslDirectoryItem Item, oslFileStatus* pStat, sal_uInt32 uFieldMask )
{
    int nRet=0;
    int nSLen=0;
    struct stat aFileStat;
    oslFileError eErr=osl_File_E_invalidError;
    sal_Bool bMountPoint=sal_False;
    rtl_uString* pTmpString=0;
    sal_Char pbuffer[OSL_MAX_NATIVE_PATH] = "";
    /* mfe : HACK!!! */
    sal_Char pszNativePath[OSL_MAX_NATIVE_PATH] = "";
    sal_Char* pszOslDirName=0;
    sal_Char* pszFileName=0;
    rtl_uString* pTmpString2=0;

    oslDirectoryItemImpl* pDirItemImpl = (oslDirectoryItemImpl*) Item;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getFileStatus\n");
#endif

    if ( pDirItemImpl == 0 || pStat == 0 )
    {
        if ( pStat != 0 )
        {    
            pStat->uValidFields=0;
        }        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileStatus [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(pDirItemImpl->osl_DirName);
    rtl_uString_acquire(pDirItemImpl->osl_FileName);
    pszOslDirName=oslGetNativeName(pDirItemImpl->osl_DirName);
    if ( pszOslDirName == 0 )
    {
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileStatus [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    strcpy(pbuffer,pszOslDirName);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Dir is '%s'\n",pbuffer);
#endif

    /* mfe : just another hack */ 
    nSLen = strlen(pbuffer);

/*      fprintf(stderr,"pbuffer[nSLen] is '%c'\n",pbuffer[nSLen]); */
    
    if ( nSLen > 2 && pbuffer[nSLen-1] == '/' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"deleting trailing '/'\n");
#endif
        pbuffer[nSLen-1]=0;
    }
    
    
    if ( !(pszOslDirName[0] == '/' && pszOslDirName[1] == '\0') )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"catting '/'\n");
#endif
        strcat(pbuffer,"/");
    }
    rtl_freeMemory(pszOslDirName);
    
    pszFileName=oslGetNativeName(pDirItemImpl->osl_FileName);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"File Name is '%s'\n",pszFileName);
#endif
    
    if ( pszFileName != 0 )
    {    
        strcat(pbuffer,pszFileName);
    }    
    
#ifdef DEBUG_OSL_FILE
    {
        sal_Char* p;
        p=oslGetNativeName(pDirItemImpl->osl_FileName);
        if ( p != 0 )
        {
            fprintf(stderr,"FileName '%s'\n",p);
            rtl_freeMemory(p);
        }

        p = oslGetNativeName(pDirItemImpl->osl_DirName);
        if ( p != 0 )
        {    
            fprintf(stderr,"DirName  '%s'\n",p);
            rtl_freeMemory(p);
        }
    }
#endif


#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making lstat with '%s'\n",pbuffer);
#endif
    nRet = lstat(pbuffer,&aFileStat);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("lstat");
#endif
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileStatus [lstat]\n");
#endif
        pStat->uValidFields=0;
        return oslTranslateFileError(nRet);
    }

    /* mfe : HACK!!! */    
    strcpy(pszNativePath,pbuffer);
    
    /* Type */
    if ( S_ISDIR(aFileStat.st_mode) ) 
    { 
        eErr=oslIsMountPoint(pbuffer,&bMountPoint); 
        if ( eErr != osl_File_E_None ) 
        { 
            rtl_uString_release(pDirItemImpl->osl_DirName);
            rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_getFileStatus [isMountPoint]\n");
#endif
            pStat->uValidFields=0;
            return eErr; 
        } 

        if ( bMountPoint == True ) 
        {
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"Yup, is mount point\n");
#endif
            pStat->eType=osl_File_Type_Volume;
        }
        else 
        {
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"Oh, is directory\n");
#endif
            pStat->eType=osl_File_Type_Directory;
        } 
    } 
    else if ( S_ISREG(aFileStat.st_mode) ) 
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Regular File\n");
#endif
        pStat->eType=osl_File_Type_Regular;
    }
    else if ( S_ISFIFO(aFileStat.st_mode) ) 
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Fifo File\n");
#endif
        pStat->eType=osl_File_Type_Fifo;
    }
    else if ( S_ISLNK(aFileStat.st_mode) ) 
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Link File\n");
#endif
        pStat->eType=osl_File_Type_Link;
    }
    else if ( S_ISSOCK(aFileStat.st_mode) ) 
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Socket File\n");
#endif
        pStat->eType=osl_File_Type_Socket;
    }
    else if ( S_ISCHR(aFileStat.st_mode) || S_ISBLK(aFileStat.st_mode) ) 
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Special File\n");
#endif
        pStat->eType=osl_File_Type_Special;
    }
    else if ( S_ISSOCK(aFileStat.st_mode) ) 
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Socket File\n");
#endif
        pStat->eType=osl_File_Type_Socket;
    }
    else
    { 
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"OOPS: File Type Unknown\n");
#endif
        pStat->eType=osl_File_Type_Unknown;
    } 
    pStat->uValidFields|=osl_FileStatus_Mask_Type;


    /*
     *   mfe : obsolete 
     *         get the information not of the link
     *         but to the file it points to
     */
/*      if ( S_ISLNK(aFileStat.st_mode) ) */
/*      { */
/*  #ifdef DEBUG_OSL_FILE */
/*          fprintf(stderr,"IS LINK '%s'\n",pbuffer); */
/*  #endif */

/*  #ifdef DEBUG_OSL_FILE */
/*          fprintf(stderr,"making stat with '%s'\n",pbuffer); */
/*  #endif */
/*          nRet = stat(pbuffer,&aFileStat); */
/*          if ( nRet < 0 ) */
/*          { */
/*              nRet=errno; */
/*  #ifdef DEBUG_OSL_FILE */
/*              perror("stat"); */
/*  #endif */
/*              rtl_uString_release(pDirItemImpl->osl_DirName); */
/*              rtl_uString_release(pDirItemImpl->osl_FileName); */
/*  #ifdef TRACE_OSL_FILE */
/*              fprintf(stderr,"Out osl_getFileStatus [stat]\n"); */
/*  #endif */
/*              pStat->uValidFields=0; */
/*              return oslTranslateFileError(nRet); */
/*          } */
/*      } */


    /* File Exists semantic of osl_FileStatus_Mask_Validate */
    if ( !(uFieldMask & ~osl_FileStatus_Mask_Validate) )
    {
        /* FIXME getFileStatus: is this all/enough? */
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(pDirItemImpl->osl_FileName);
        pStat->uValidFields=osl_FileStatus_Mask_Validate;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileStatus [Validate]\n");
#endif
        pStat->uValidFields=0;
        return osl_File_E_None;
    }


    /* FileName */
    rtl_uString_newFromString(&(pStat->strFileName),pDirItemImpl->osl_FileName);
    if ( pStat->strFileName == 0 ) 
    { 
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(pDirItemImpl->osl_FileName);
        pStat->strFileName=0;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getDirectoryItem [NOMEM]\n");
#endif
        pStat->uValidFields=0;
        return osl_File_E_NOMEM;
    }

#ifdef DEBUG_OSL_FILE
    {
        sal_Char* p;
        p = oslGetNativeName(pStat->strFileName);
        if ( p != 0 )
        {
            fprintf(stderr,"strFileName '%s'\n",p);
            rtl_freeMemory(p);
        }
    }
    
#endif

    pStat->uValidFields|=osl_FileStatus_Mask_FileName; 

    /* osl_File_Attribute */
    if ( uFieldMask & osl_FileStatus_Mask_Attributes )
    {
        nRet = access(pbuffer,W_OK);
        if ( nRet < 0 )
        {
            if ( errno == EACCES || errno == EROFS )
            {
                pStat->uAttributes=osl_File_Attribute_ReadOnly;
                pStat->uValidFields|=osl_FileStatus_Mask_Attributes;
            }
            else
            {
                pStat->uAttributes=0;
                pStat->uValidFields&=~osl_FileStatus_Mask_Attributes;
            }
        }
        else
        {
            pStat->uAttributes=0;
            pStat->uValidFields|=osl_FileStatus_Mask_Attributes;
        }
    }


    /* Creation.Time */
    pStat->aCreationTime.Seconds = aFileStat.st_ctime;
    pStat->aCreationTime.Nanosec = 0;
    pStat->uValidFields|=osl_FileStatus_Mask_CreationTime;


    /* Access.Time */ 
    pStat->aAccessTime.Seconds = aFileStat.st_atime;
    pStat->aAccessTime.Nanosec = 0;
    pStat->uValidFields|=osl_FileStatus_Mask_AccessTime;


    /* Modify.Time */ 
    pStat->aModifyTime.Seconds = aFileStat.st_mtime;
    pStat->aModifyTime.Nanosec = 0;
    pStat->uValidFields|=osl_FileStatus_Mask_ModifyTime;
    

    /* FileSize */ 
    if ( pStat->eType == osl_File_Type_Regular )
    {
        pStat->uFileSize=aFileStat.st_size;
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Regular File Size '%llu'\n",pStat->uFileSize);
#endif
    }
    else if ( pStat->eType == osl_File_Type_Volume )
    {
        oslVolumeInfo aVInfo;
        rtl_uString* pString=0;
        sal_Char pTmpBuffer[OSL_MAX_NATIVE_PATH] = "";
        strcpy(pTmpBuffer,"//.");
        strcat(pTmpBuffer,pbuffer);
        rtl_uString_newFromAscii(&pString,pTmpBuffer);

        if ( pString == 0 )
        {
            rtl_uString_release(pDirItemImpl->osl_DirName);
            rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_getFileStatus [NOMEM]\n");
#endif
            pStat->uValidFields=0;
            return osl_File_E_NOMEM;
        }

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"getting size of mount point '%s' (sal)\n",pTmpBuffer);
#endif

        eErr=osl_getVolumeInformation(pString,&aVInfo,osl_VolumeInfo_Mask_FreeSpace);
        if ( eErr != osl_File_E_None )
        {
            rtl_uString_release(pDirItemImpl->osl_DirName);
            rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_getFileStatus [getVolumeInformation]\n");
#endif
            pStat->uValidFields=0;
            return eErr;
        }
        pStat->uFileSize=aVInfo.uFreeSpace;
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Volume Free Size '%llu'\n",pStat->uFileSize);
#endif
        rtl_uString_release(pString);
    }
    else if ( pStat->eType == osl_File_Type_Directory )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Type == osl_File_Type_Directory\n");
#endif  
    }
    else if ( pStat->eType == osl_File_Type_Fifo )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Type == osl_File_Type_Fifo\n");
#endif  
    }
    else if ( pStat->eType == osl_File_Type_Socket )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Type == osl_File_Type_Socket\n");
#endif        
    }
    else if ( pStat->eType == osl_File_Type_Link )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Type == osl_File_Type_Link\n");
#endif
    }
    else if ( pStat->eType == osl_File_Type_Special )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Type == osl_File_Type_Special\n");
#endif  
    }    
    else
    {
        pStat->uFileSize=aFileStat.st_size;
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Unknown Type size '%llu'\n",pStat->uFileSize);
#endif
    }
    pStat->uValidFields|=osl_FileStatus_Mask_FileSize;


    /* fsFilePath */ 
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making strFilePath\n");
#endif

    rtl_uString_newFromAscii(pStat->pstrFilePath,"//.");
    if ( *(pStat->pstrFilePath) == 0 )
    {
/*        rtl_uString_release(*(pStat->strFilePath));*/
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileStatus [NOMEM]\n");
#endif
        pStat->uValidFields=0;
        return osl_File_E_NOMEM;
    }

#ifdef DEBUG_OSL_FILE
    {
        sal_Char* p;
        p = oslGetNativeName(pDirItemImpl->osl_DirName);
        if ( p != 0 )
        {
            fprintf(stderr,"--- pDirItemImpl->osl_DirName '%s'\n",p);
            rtl_freeMemory(p);
        }
    }
#endif
    
    if ( rtl_uString_getStr(pDirItemImpl->osl_DirName)[0] == 0 )
    {
        /* mfe: we should never ever come to this point. If we do, we have a really big problem! */
/*        pStat->uValidFields=0;*/
#ifdef TRACE_OSL_FILE
/*        fprintf(stderr,"Out getFileStatus []\n");*/
#endif
/*        return osl_File_E_NOTDIR;*/

        /* mfe: the following code is 'fall back code' (commented out.) */
        pTmpString2=0;
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"concat '/'\n");
#endif
        rtl_uString_new(&pTmpString2);
        rtl_uString_newFromAscii(&pTmpString2,"/");
        if ( pTmpString2 == 0 )
        {
            rtl_uString_release(*(pStat->pstrFilePath));
            rtl_uString_release(pDirItemImpl->osl_DirName);
            rtl_uString_release(pDirItemImpl->osl_FileName);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out getFileStatus [NOMEM]\n");
#endif
            return osl_File_E_NOMEM;
        }
        rtl_uString_newConcat(&pTmpString,*(pStat->pstrFilePath),pTmpString2);
        rtl_uString_release(pTmpString2);
        pTmpString2=0;
    }
    else
    {
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"concat dirname\n");*/
#endif
        rtl_uString_newConcat(&pTmpString,*(pStat->pstrFilePath),pDirItemImpl->osl_DirName);
    }

    
    rtl_uString_release(*(pStat->pstrFilePath));
    *(pStat->pstrFilePath)=0;
    rtl_uString_newFromAscii(&pTmpString2,"/");
    rtl_uString_newConcat(&pTmpString,pTmpString,pDirItemImpl->osl_FileName);
    rtl_uString_newFromString(pStat->pstrFilePath,pTmpString);
    rtl_uString_release(pTmpString2);
    
    if ( *(pStat->pstrFilePath) == 0 )
    {
        rtl_uString_release(pDirItemImpl->osl_DirName);
        rtl_uString_release(pDirItemImpl->osl_FileName);
        rtl_uString_release(pTmpString);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out getFileStatus [NOMEM]\n");
#endif
        pStat->uValidFields=0;
        return osl_File_E_NOMEM;
    }

    rtl_uString_release(pTmpString);
    pStat->uValidFields|=osl_FileStatus_Mask_FilePath;
#ifdef DEBUG_OSL_FILE
/*    fprintf(stderr,"done ... (%s)\n",rtl_uString_getStr(pStat->strFilePath));*/
#endif


    /* fsFileNativePath */ 
/*    *(pStat->pstrNativePath)=0;*/

    /* mfe : HACK!!! */
    {
        rtl_uString* pTmpString=0;
        rtl_uString* pTmpString2=0;
        
        rtl_uString_newFromAscii(&pTmpString2,"//.");
        rtl_uString_newFromAscii(&pTmpString,pszNativePath);
        rtl_uString_newConcat(&pTmpString,pTmpString2,pTmpString);    
        rtl_uString_newFromString(pStat->pstrFilePath,pTmpString);

        rtl_uString_release(pTmpString);
        rtl_uString_release(pTmpString2);
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Native path is '%s'\n",pbuffer);
    fprintf(stderr,"File path is ");
    debug_ustring(*(pStat->pstrFilePath));
#endif
    
/*      if ( *(pStat->pstrNativePath) == 0 ) */
/*      { */
/*          rtl_uString_release(pDirItemImpl->osl_DirName); */
/*          rtl_uString_release(pDirItemImpl->osl_FileName); */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out getFileStatus [NOMEM]\n"); */
/*  #endif */
/*          pStat->uValidFields=0; */
/*          return osl_File_E_NOMEM; */
/*      } */
    pStat->uValidFields|=osl_FileStatus_Mask_NativePath;


    rtl_uString_release(pDirItemImpl->osl_DirName);
    rtl_uString_release(pDirItemImpl->osl_FileName);

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getFileStatus [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_getVolumeInformation( rtl_uString* strDirectory, oslVolumeInfo* pInfo, sal_uInt32 uFieldMask )
{
    int nRet=0;
    long lRet=0;
    sal_Char* pszDirectory=0;
    sal_Char* pszUNCPath=0;

#if defined(LINUX)
    struct statfs aStatFS;
#endif /* LINUX */

#if defined(SOLARIS)
    struct statvfs aStatVFS;
#endif /* SOLARIS */

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getVolumeInformation\n");
#endif

    if ( pInfo == 0 )
    {
        if ( pInfo != 0 )
        {    
            pInfo->uValidFields=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [pInfo == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( strDirectory == 0 )
    {
        if ( pInfo != 0 )
        {    
            pInfo->uValidFields=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [strDirectory == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strDirectory);
    pszUNCPath=oslGetNativeName(strDirectory);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strDirectory);
        if ( pInfo != 0 )
        {
            pInfo->uValidFields=0;            
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strDirectory);
        rtl_freeMemory(pszUNCPath);
        
        if ( pInfo != 0 )
        {    
            pInfo->uValidFields=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszDirectory=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strDirectory);
        if ( pInfo != 0 )
        {    
            pInfo->uValidFields=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }



#if defined(LINUX)
    memset(&aStatFS,0,sizeof(aStatFS));
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making statfs with '%s'\n",pszDirectory);
#endif
    nRet=statfs(pszDirectory,&aStatFS);
    if ( nRet != 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("statfs");
#endif
        rtl_uString_release(strDirectory);
        rtl_freeMemory(pszDirectory);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [statfs]\n");
#endif
        pInfo->uValidFields=0;
        return oslTranslateFileError(nRet);
    }

    /* FIXME getVolumeInformation: is there any list with ascii strings we can use with fsid?     */
    /*        answer: no! but how can we get the corresponding (name)string? */
    /*                Were we can translate the magic key to the name?       */
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"      block size  : %lu\n",aStatFS.f_bsize);
    fprintf(stderr," #    blocks      : %lu\n",aStatFS.f_blocks);
    fprintf(stderr,"free  blocks      : %lu\n",aStatFS.f_bfree);
    fprintf(stderr,"avail blocks      : %lu\n",aStatFS.f_bavail);
    fprintf(stderr,"fsid              : %lu\n",aStatFS.f_fsid);
    fprintf(stderr,"namelen           : %lu\n",aStatFS.f_namelen);
#endif

    /* FIXME getVolumeInformation: uAttributes how can we determine the kind of storage (harddisk, cdrom, ...)?*/
    pInfo->uValidFields&=~osl_VolumeInfo_Mask_UsedSpace;

    /* uTotalSpace */
    pInfo->uTotalSpace=(sal_uInt64)((sal_uInt64)aStatFS.f_blocks*(sal_uInt64)aStatFS.f_bsize);
    pInfo->uValidFields|=osl_VolumeInfo_Mask_TotalSpace;

    /* uFreeSpace */
    /*
     *  mfe:
     *  'root' has reserved blocks on many file systems the average user can't access
     *  so we handle the free space differently.
     */
    if ( getuid() == 0 )
    {
        pInfo->uFreeSpace=(sal_uInt64)((sal_uInt64)aStatFS.f_bfree*(sal_uInt64)aStatFS.f_bsize);
    }
    else
    {
        pInfo->uFreeSpace=(sal_uInt64)((sal_uInt64)aStatFS.f_bavail*(sal_uInt64)aStatFS.f_bsize);
    }
    pInfo->uValidFields|=osl_VolumeInfo_Mask_FreeSpace;


    /* uUsedSpace */
    pInfo->uUsedSpace=(sal_uInt64)
        (( (sal_uInt64)aStatFS.f_blocks * (sal_uInt64)aStatFS.f_bsize ) - ( (sal_uInt64)aStatFS.f_bfree * (sal_uInt64)aStatFS.f_bsize ));
    pInfo->uValidFields|=osl_VolumeInfo_Mask_UsedSpace;

    /* uMaxNameLength */
    lRet = pathconf(pszDirectory,_PC_NAME_MAX);
    if ( lRet > 0 )
    {
        pInfo->uMaxNameLength=(sal_uInt32)lRet;
    }
    pInfo->uValidFields|=osl_VolumeInfo_Mask_MaxNameLength;
    
    /* uMaxPathLength */
    lRet = pathconf(pszDirectory,_PC_PATH_MAX);
    if ( lRet > 0 )
    {
        pInfo->uMaxPathLength=(sal_uInt32)lRet;
    }
    pInfo->uValidFields|=osl_VolumeInfo_Mask_MaxPathLength;

    /* mfe: how can we get it besides via getmntent(3)? */
    /* FIXME getVolumeInformation: pstrFileSystemName */
    pInfo->uValidFields&=~osl_VolumeInfo_Mask_FileSystemName;

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Total Size (ll)       : '%15llu'\n", pInfo->uTotalSpace );
    fprintf(stderr,"Free  Size (ll)       : '%15llu'\n", pInfo->uFreeSpace  );
    fprintf(stderr,"Used  Size (ll)       : '%15llu'\n", pInfo->uUsedSpace  );
    fprintf(stderr,"Max Name Length(conf) : '%lu'\n", pInfo->uMaxNameLength );
    fprintf(stderr,"Max Path Length(conf) : '%lu'\n", pInfo->uMaxPathLength );
    fprintf(stderr,"FileSystemName        : '%s'\n", oslGetNativeName(*(pInfo->pstrFileSystemName)) );
#endif


    if ( uFieldMask & osl_VolumeInfo_Mask_DeviceHandle )
    {
        /* FIXME: check also entries in mntent for the device and fill it with correct values */        
        *(pInfo->pDeviceHandle)=osl_isFloppyDrive(pszDirectory);
        if ( *(pInfo->pDeviceHandle) != 0 )
        {   
            pInfo->uValidFields |= osl_VolumeInfo_Mask_DeviceHandle;
            pInfo->uAttributes = osl_Volume_Attribute_Removeable;
        }
    }
#ifdef DEBUG_OSL_FILE    
    fprintf(stderr,"Device Handle         : '0X%08x'\n", *(pInfo->pDeviceHandle ));
#endif
    
#endif /* LINUX */

#if defined(SOLARIS)
    memset(&aStatVFS,0,sizeof(aStatVFS));

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making statvfs with '%s'\n",pszDirectory);
#endif

    nRet = statvfs(pszDirectory,&aStatVFS);
    if ( nRet != 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("statvfs");
#endif
        rtl_uString_release(strDirectory);
        rtl_freeMemory(pszDirectory);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeInformation [statvfs]\n");
#endif
        pInfo->uValidFields=0;
        return oslTranslateFileError(nRet);
    }

#ifdef DEBUG_OSL_FILE
/*      fprintf(stderr,"      block size  : %lu\n",aStatVFS.f_bsize); */
/*      fprintf(stderr,"fund. block size  : %lu\n",aStatVFS.f_frsize); */
/*      fprintf(stderr," #    blocks      : %lu\n",aStatVFS.f_blocks); */
/*      fprintf(stderr,"free  blocks      : %lu\n",aStatVFS.f_bfree); */
/*      fprintf(stderr,"avail blocks      : %lu\n",aStatVFS.f_bavail); */
/*      fprintf(stderr,"basetype          : '%s'\n",aStatVFS.f_basetype); */
/*      fprintf(stderr,"namemax           : %lu\n",aStatVFS.f_namemax); */
/*      fprintf(stderr,"fstr              : '%s'\n",aStatVFS.f_fstr); */
#endif

    /* FIXME getVolumeInformation: uAttributes how can we determine the kind of storage (harddisk, cdrom, ...)?*/
    pInfo->uValidFields&=~osl_VolumeInfo_Mask_UsedSpace;

    /* uTotalSpace */
    pInfo->uTotalSpace=(sal_uInt64)((sal_uInt64)aStatVFS.f_blocks*(sal_uInt64)aStatVFS.f_frsize);
    pInfo->uValidFields|=osl_VolumeInfo_Mask_TotalSpace;


    /* uFreeSpace */
    /*
     *  mfe:
     *  'root' has reserved blocks on many file systems the average user can't access
     *  so we handle the free space differently
     */
    if ( getuid() == 0 )
    {
        pInfo->uFreeSpace=(sal_uInt64)((sal_uInt64)aStatVFS.f_bfree*(sal_uInt64)aStatVFS.f_frsize);
    }
    else
    {
        pInfo->uFreeSpace=(sal_uInt64)((sal_uInt64)aStatVFS.f_bavail*(sal_uInt64)aStatVFS.f_frsize);
    }
    pInfo->uValidFields|=osl_VolumeInfo_Mask_FreeSpace;


    /* uUsedSpace */
    pInfo->uUsedSpace=(sal_uInt64)
        ( ( (sal_uInt64)aStatVFS.f_blocks * (sal_uInt64)aStatVFS.f_frsize ) - ( (sal_uInt64)aStatVFS.f_bfree * (sal_uInt64)aStatVFS.f_frsize ) );
    pInfo->uValidFields|=osl_VolumeInfo_Mask_UsedSpace;


    /* uMaxNameLength */
    /*
     *  mfe:
     *  The statvfs.namelen field holds sometimes rubbish
     *  so we can't use it.
     */
    lRet = pathconf(pszDirectory,_PC_NAME_MAX);
    if ( lRet > 0 )
    {
        pInfo->uMaxNameLength=(sal_uInt32)lRet;
    }
    pInfo->uValidFields|=osl_VolumeInfo_Mask_MaxNameLength;


    /* uMaxPathLength */
    lRet = pathconf(pszDirectory,_PC_PATH_MAX);
    if ( lRet > 0 )
    {
        pInfo->uMaxPathLength=(sal_uInt32)lRet;
    }
    pInfo->uValidFields|=osl_VolumeInfo_Mask_MaxPathLength;


    /* FIXME: getVolumeInformation: sFileSystemName */
    if ( uFieldMask & osl_VolumeInfo_Mask_FileSystemName )
    {
        /* FIXME: expects */
        rtl_uString_newFromAscii(pInfo->pstrFileSystemName,aStatVFS.f_basetype);
        pInfo->uValidFields|=osl_VolumeInfo_Mask_FileSystemName;
    }
    
#ifdef DEBUG_OSL_FILE    
    fprintf(stderr,"Total Size (ll)       : '%15llu'\n", pInfo->uTotalSpace );
    fprintf(stderr,"Free  Size (ll)       : '%15llu'\n", pInfo->uFreeSpace  );
    fprintf(stderr,"Used  Size (ll)       : '%15llu'\n", pInfo->uUsedSpace  );
    fprintf(stderr,"Max Name Length(conf) : '%lu'\n", pInfo->uMaxNameLength );
    fprintf(stderr,"Max Path Length(conf) : '%lu'\n", pInfo->uMaxPathLength );
#endif
#ifdef DEBUG_OSL_FILE
    if ( pInfo->uValidFields & osl_VolumeInfo_Mask_FileSystemName )
    {    
        fprintf(stderr,"FileSystemName        : '%s'\n", oslGetNativeName(*(pInfo->pstrFileSystemName )));
    }
#endif

    if ( uFieldMask & osl_VolumeInfo_Mask_DeviceHandle )
    {
        /* FIXME: check also entries in mntent for the device and fill it with correct values */
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"osl_getVolumeInformation : checking DeviceHandle\n");
#endif
        *(pInfo->pDeviceHandle)=osl_isFloppyDrive(pszDirectory);
        
        if ( *(pInfo->pDeviceHandle) != 0 )
        {
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"osl_getVolumeInformation : is DeviceHandle\n");
#endif
            pInfo->uValidFields |= osl_VolumeInfo_Mask_DeviceHandle;
            pInfo->uAttributes = osl_Volume_Attribute_Removeable;
        }
    }
#ifdef DEBUG_OSL_FILE
    if ( pInfo->pDeviceHandle != 0 )
    {    
        fprintf(stderr,"Device Handle         : '0X%08x'\n", *(pInfo->pDeviceHandle ));
    }
#endif
    
#endif /* SOLARIS */

    rtl_uString_release(strDirectory);
    rtl_freeMemory(pszDirectory);
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getVolumeInformation [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_openFile( rtl_uString* strPath, oslFileHandle* pHandle, sal_uInt32 uFlags )
{
    int nRet=0;
    int fd=-1;
    int mode=0;
    int flags=0;
    struct stat aFileStat;
    sal_Char* pszPath=0;
    sal_Char* pszUNCPath=0;
    oslFileHandleImpl* pHandleImpl=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_openFile\n");
#endif

    if ( strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [strPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pHandle == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [pHandle == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);
    pszUNCPath=oslGetNativeName(strPath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [pszPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }
            
    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( (pszPath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

/*      if ( pHandle == 0 ) */
/*      { */
/*          rtl_uString_release(strPath); */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_openFile [pHandle == 0]\n"); */
/*  #endif */
/*          return osl_File_E_INVAL; */
/*      } */


/*      pszFileName=oslGetNativeName(strPath); */
/*      if ( pszFileName == 0 ) */
/*      { */
/*          rtl_uString_release(strPath); */
/*          rtl_freeMemory(pHandleImpl); */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_openFile [INVAL]\n"); */
/*  #endif */
/*          return osl_File_E_INVAL; */
/*      } */

    pHandleImpl=(oslFileHandleImpl*) rtl_allocateMemory(sizeof(oslFileHandleImpl));

    if ( pHandleImpl == 0 )
    {
        rtl_uString_release(strPath);
        rtl_freeMemory(pszPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }
    memset(pHandleImpl,0,sizeof(pHandleImpl));

    mode|= S_IRUSR | S_IRGRP | S_IROTH;
    flags=O_RDONLY;
    if ( uFlags & osl_File_OpenFlag_Write )
    {
        mode |= S_IWUSR | S_IWGRP | S_IWOTH;
        flags = O_RDWR;
    }

    if ( uFlags & osl_File_OpenFlag_Create )
    {
        mode |= S_IWUSR | S_IWGRP | S_IWOTH;
        flags = O_CREAT | O_RDWR;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"File to open '%s'\n",pszPath);
#endif


    nRet= stat(pszPath,&aFileStat);
    if ( nRet < 0 && !( uFlags & osl_File_OpenFlag_Create ) )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("stat");
#endif
        rtl_uString_release(strPath);
        rtl_freeMemory(pHandleImpl);
        rtl_freeMemory(pszPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [stat]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    if ( nRet >= 0 && !S_ISREG(aFileStat.st_mode) )
    {
        rtl_uString_release(strPath);
        rtl_freeMemory(pHandleImpl);
        rtl_freeMemory(pszPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }


    fd = open(pszPath,flags,mode);
    if ( fd < 0 )
    {
        fd=errno;
#ifdef DEBUG_OSL_FILE
        perror("open");
#endif
        rtl_freeMemory(pHandleImpl);
        rtl_uString_release(strPath);
        rtl_freeMemory(pszPath);
       
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [open]\n");
#endif
        return oslTranslateFileError(fd);
    }

    pHandleImpl->fd=fd;
    pHandleImpl->filepathname=0;
    rtl_uString_newFromString(&(pHandleImpl->filepathname), strPath);
    if ( pHandleImpl->filepathname == 0 )
    {
        rtl_freeMemory(pHandleImpl);
        rtl_uString_release(strPath);
        rtl_freeMemory(pszPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_openFile [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

    *pHandle=pHandleImpl;
    rtl_uString_release(strPath);
    rtl_freeMemory(pszPath);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_openFile [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_closeFile( oslFileHandle Handle )
{
    int nRet=0;
    oslFileHandleImpl* pHandleImpl=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_closeFile\n");
#endif

    pHandleImpl = (oslFileHandleImpl*) Handle;

    if ( pHandleImpl == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_closeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pHandleImpl->fd < 0 || pHandleImpl->filepathname == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_closeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    nRet=close(pHandleImpl->fd);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("close");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_closeFile [close]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    rtl_uString_release(pHandleImpl->filepathname);
    rtl_freeMemory(pHandleImpl);

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_closeFile [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_readFile( oslFileHandle Handle, void* pBuffer, sal_uInt64 uBytesRequested, sal_uInt64* pBytesRead )
{
    ssize_t nBytes=0;
    int nRet=0;
    oslFileHandleImpl* pHandleImpl=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_readFile\n");
#endif

    if ( Handle == 0 || pBytesRead == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_readFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pBuffer == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_readFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( uBytesRequested == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_readFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    pHandleImpl = (oslFileHandleImpl*) Handle;
    
    if ( pHandleImpl->fd < 0 || pHandleImpl->filepathname == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_readFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }


    nBytes = read(pHandleImpl->fd,pBuffer,uBytesRequested);
    if ( nBytes < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("read");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_readFile [read]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    *pBytesRead=nBytes;
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_readFile [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_writeFile( oslFileHandle Handle, const void* pBuffer, sal_uInt64 uBytesToWrite, sal_uInt64* pBytesWritten )
{
    ssize_t nBytes=0;
    int nRet=0;
    oslFileHandleImpl* pHandleImpl=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_writeFile\n");
#endif

    pHandleImpl = (oslFileHandleImpl*) Handle;
    if ( pHandleImpl == 0 || pBytesWritten == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_writeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pBuffer == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_writeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( uBytesToWrite == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_writeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pHandleImpl->fd < 0 || pHandleImpl->filepathname == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_writeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }


    nBytes = write(pHandleImpl->fd,pBuffer,uBytesToWrite);
    if ( nBytes < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("write");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_writeFile [write]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    *pBytesWritten=nBytes;
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_writeFile [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_setFilePos( oslFileHandle Handle, sal_uInt32 uHow, sal_uInt64 uPos )
{
    oslFileHandleImpl* pHandleImpl=0;
    int nRet=0;
    off_t nOffset=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_setFilePos\n");
#endif

    pHandleImpl = (oslFileHandleImpl*) Handle;
    if ( pHandleImpl == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFilePos [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pHandleImpl->fd < 0 || pHandleImpl->filepathname == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFilePos [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    /* FIXME setFilePos: Do we have any runtime function to determine LONG_MAX? */
    if ( uPos > LONG_MAX )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFilePos [OVERFLOW]\n");
#endif
        return osl_File_E_OVERFLOW;
    }

    nOffset=(off_t)uPos;
    
    switch(uHow)
    {
        case osl_Pos_Absolut:
            nOffset = lseek(pHandleImpl->fd,nOffset,SEEK_SET);
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_setFilePos Absolut\n");
#endif
            break;

        case osl_Pos_Current:
            nOffset = lseek(pHandleImpl->fd,nOffset,SEEK_CUR);
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_setFilePos Current\n");
#endif
            break;

        case osl_Pos_End:
            nOffset = lseek(pHandleImpl->fd,nOffset,SEEK_END);
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_setFilePos End\n");
#endif
            break;

        default:
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_setFilePos [INVAL]\n");
#endif
            return osl_File_E_INVAL;
            break;
    }

    if ( nOffset < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("lseek");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFilePos [lseek]\n");
#endif
        return oslTranslateFileError(nRet);
    }

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_setFilePos [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_getFilePos( oslFileHandle Handle, sal_uInt64* pPos )
{
    oslFileHandleImpl* pHandleImpl=0;
    off_t nOffset=0;
    int nRet=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getFilePos\n");
#endif

    pHandleImpl = (oslFileHandleImpl*) Handle;
    if ( pHandleImpl == 0 || pPos == 0)
    {
        if ( pPos != 0 )
        {
            *pPos=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFilePos [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pHandleImpl->fd < 0 || pHandleImpl->filepathname == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFilePos [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    nOffset=lseek(pHandleImpl->fd,0,SEEK_CUR);
    if ( nOffset < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("lseek");
#endif
        *pPos=0;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFilePos [lseek]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    *pPos=nOffset;
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getFilePos [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_setFileSize( oslFileHandle Handle, sal_uInt64 uSize )
{
    oslFileHandleImpl* pHandleImpl=0;
    off_t nOffset=0;
    int nRet=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_setFileSize\n");
#endif

    pHandleImpl = (oslFileHandleImpl*) Handle;
    if ( pHandleImpl == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileSize [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pHandleImpl->fd < 0 || pHandleImpl->filepathname == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileSize [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    /* FIXME setFileSize: Do we have any runtime function to determine LONG_MAX? */
    if ( uSize > LONG_MAX )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileSize [OVERFLOW]\n");
#endif
        return osl_File_E_OVERFLOW;
    }

    nOffset=(off_t) uSize;
    nRet=ftruncate(pHandleImpl->fd,nOffset);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("ftruncate");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileSize [ftruncate]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    /* mfe: this is the second alternative for truncating the file (commented out.) */
/*      pszFileName=oslGetNativeName(rtl_uString_getStr(pHandleImpl->filepathname)); */
/*      if ( pszFileName == 0 ) */
/*      { */
/*          rtl_uString_release(pHandleImpl->filepathname); */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_setFileSize [INVAL]\n"); */
/*  #endif */
/*          return osl_File_E_INVAL; */
/*      }  */
/*      nRet=truncate(pszfilename,nOffset); */
/*      if ( nRet < 0 ) */
/*      { */
/*          nRet=errno; */
/*  #ifdef DEBUG_OSL_FILE */
/*          perror("truncate"); */
/*  #endif */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_setFileSize [ftruncate]\n"); */
/*  #endif */
/*          return oslTranslateFileError(nRet); */
/*      } */

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_setFileSize [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_removeFile( rtl_uString* strPath )
{
    int nRet=0;
    sal_Char* pszFileName=0;
    sal_Char* pszUNCPath=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_removeFile\n");
#endif

    if ( strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeFile [strPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);
    pszUNCPath=oslGetNativeName(strPath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeFile [UNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }
            
    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeFile [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszFileName=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    nRet = unlink(pszFileName);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("unlink");
#endif
        rtl_uString_release(strPath);
        rtl_freeMemory(pszFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeFile [unlink]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    rtl_uString_release(strPath);
    rtl_freeMemory(pszFileName);

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_removeFile [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_createDirectory( rtl_uString* strPath )
{
    int nRet=0;
    int mode = S_IRWXU | S_IRWXG | S_IRWXO;
    sal_Char* pszDirectory=0;
    sal_Char* pszUNCPath=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_createDirectory\n");
#endif

    if ( strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_createDirectory [strPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);
    pszUNCPath=oslGetNativeName(strPath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectory [UNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"dir to create '%s'\n",pszUNCPath);
#endif
    
    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectory [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszDirectory=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectory [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Creating dir '%s'\n",pszDirectory);
#endif
    
    nRet = mkdir(pszDirectory,mode);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("mkdir");
#endif
        rtl_uString_release(strPath);
        rtl_freeMemory(pszDirectory);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectory [mkdir]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    rtl_uString_release(strPath);
    rtl_freeMemory(pszDirectory);        
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_createDirectory [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_removeDirectory( rtl_uString* strPath )
{
    int nRet=0;
    sal_Char* pszDirectory=0;
    sal_Char* pszUNCPath=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_removeDirectory\n");
#endif

    if ( strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeDirectory [strPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);
    pszUNCPath=oslGetNativeName(strPath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeDirectory [UNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"dir to remove '%s'\n",pszUNCPath);
#endif
    
    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeDirecotry [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszDirectory=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeDirectory [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"removing dir '%s'\n",pszDirectory);
#endif

    nRet = rmdir(pszDirectory);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("rmdir");
#endif
        rtl_uString_release(strPath);
        rtl_freeMemory(pszDirectory);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_removeDirectory [rmdir]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    rtl_uString_release(strPath);
    rtl_freeMemory(pszDirectory);        
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_removeDirectory [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_copyFile( rtl_uString* strPath, rtl_uString* strDestPath )
{
    sal_Char* pszSourceFileName=0;
    sal_Char* pszDestFileName=0;
    sal_Char* pszUNCPath=0;
    time_t nAcTime=0;
    time_t nModTime=0;
    uid_t nUID=0;
    gid_t nGID=0;
    int nRet=0;
    mode_t nMode=0;
    struct stat aFileStat;
    oslFileError tErr=osl_File_E_invalidError;
    size_t nSourceSize=0;
    int DestFileExists=1;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_copyFile\n");
#endif
    
    if ( strPath == 0 || strDestPath == 0)
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);
    rtl_uString_acquire(strDestPath);

    pszUNCPath=oslGetNativeName(strPath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [pszUNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszSourceFileName=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }

    pszUNCPath=oslGetNativeName(strDestPath);
    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszSourceFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [pszUNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( !oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszUNCPath);
        rtl_freeMemory(pszSourceFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszDestFileName=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszSourceFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }


#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Native Name Source is '%s'\n",pszSourceFileName);
    fprintf(stderr,"Native Name Dest   is '%s'\n",pszDestFileName);
#endif

    /* mfe: does the source file really exists? */
    nRet = lstat(pszSourceFileName,&aFileStat);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("stat");
#endif
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszSourceFileName);        
        rtl_freeMemory(pszDestFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [lstat]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    /* mfe: we do only copy files here! */
    if ( S_ISDIR(aFileStat.st_mode) )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszSourceFileName);        
        rtl_freeMemory(pszDestFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [not regular files]\n");
#endif
        return osl_File_E_ISDIR;
    }

    nSourceSize=(size_t)aFileStat.st_size;
    nMode=aFileStat.st_mode;
    nAcTime=aFileStat.st_atime;
    nModTime=aFileStat.st_mtime;
    nUID=aFileStat.st_uid;
    nGID=aFileStat.st_gid;

    nRet = stat(pszDestFileName,&aFileStat);
    if ( nRet < 0 ) 
    { 
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("stat");
#endif
        if ( nRet == ENOENT )
        {
            DestFileExists=0;
        }
/*        rtl_uString_release(strPath);*/
/*        rtl_uString_release(strDestPath);*/
/*        return oslTranslateFileError(nRet);*/
    } 

    /* mfe : temporarly disabled begin */
#if 0
    /* mfe: the destination file must not be a directory! */
    if ( nRet == 0 && S_ISDIR(aFileStat.st_mode) )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszSourceFileName);        
        rtl_freeMemory(pszDestFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [is directory]\n");
#endif
        return osl_File_E_ISDIR;
    }
    else
    {
        /* mfe: file does not exists or is no dir */
    }
#endif
    /* mfe : temporarly disabled end */
    
    tErr = oslDoCopy(pszSourceFileName,pszDestFileName,nMode,nSourceSize,DestFileExists);
    
    if ( tErr != osl_File_E_None )
    {
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        rtl_freeMemory(pszSourceFileName);        
        rtl_freeMemory(pszDestFileName);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_copyFile [oslDoCopy]\n");
#endif
        return tErr;
    }

    tErr = oslChangeFileModes(pszDestFileName,nMode,nAcTime,nModTime,nUID,nGID);
    
    rtl_uString_release(strPath);
    rtl_uString_release(strDestPath);
    rtl_freeMemory(pszSourceFileName);        
    rtl_freeMemory(pszDestFileName);        
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_copyFile [ok]\n");
#endif
    return tErr;
}

/*
 *  mfe: one of the first new functions for the reorganization
 */

oslFileError osl_psz_moveFile(sal_Char* pszPath, sal_Char* pszDestPath)
{

    int nRet = 0;

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"moving '%s' to '%s'\n",pszPath,pszDestPath);
#endif
    
    nRet = rename(pszPath,pszDestPath);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"rename failed : '%s'\n",strerror(nRet));
#endif
        return oslTranslateFileError(nRet);
    }
    
    return osl_File_E_None;
}


oslFileError oslDoMoveFile(rtl_uString* ustrPath, rtl_uString* ustrDestPath)
{
    sal_Char* pszPath=0;
    sal_Char* pszDestPath=0;
    oslFileError tErr;
    rtl_String* strPath=0;
    rtl_String* strDestPath=0;
    
    if ( ( ustrPath != 0 )  && ( ustrPath->buffer[0] != 0 ) )
    {   
        rtl_uString2String( &strPath,
                            rtl_uString_getStr(ustrPath),
                            rtl_uString_getLength(ustrPath),
                            osl_getThreadTextEncoding(),
                            OUSTRING_TO_OSTRING_CVTFLAGS );
        pszPath = rtl_string_getStr(strPath);

        if ( strncmp(pszPath,"//./",4) != 0 )
        {
            rtl_string_release(strPath);
            return osl_File_E_INVAL;
        }
        pszPath+=3;
    }

    if ( ustrDestPath != 0  && ustrDestPath->buffer[0] != 0 )
    {
        rtl_uString2String( &strDestPath,
                            rtl_uString_getStr(ustrDestPath),
                            rtl_uString_getLength(ustrDestPath),
                            osl_getThreadTextEncoding(),
                            OUSTRING_TO_OSTRING_CVTFLAGS );
        pszDestPath = rtl_string_getStr(strDestPath);

        if ( strncmp(pszDestPath,"//./",4) != 0 )
        {
            rtl_string_release(strPath);
            rtl_string_release(strDestPath);
            return osl_File_E_INVAL;
        }
        pszDestPath+=3;
    }
    

    tErr = osl_psz_moveFile(pszPath,pszDestPath);
    
    if ( strPath != 0 )
    {
        rtl_string_release(strPath);
    }

    if ( strDestPath != 0 )
    {
        rtl_string_release(strDestPath);
    }
    
    return tErr;
}


oslFileError osl_moveFile( rtl_uString* strPath, rtl_uString* strDestPath )
{
    oslFileError tErr=osl_File_E_invalidError;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_moveFile\n");
#endif
    
    if ( strPath == 0 || strDestPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_moveFile [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);
    rtl_uString_acquire(strDestPath);

    tErr = oslDoMoveFile(strPath,strDestPath);
    if ( tErr == osl_File_E_None )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_moveFile [rename succeeded]\n");
#endif
        return tErr;
    }
    
        
    if ( tErr != osl_File_E_XDEV )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_moveFile [something went wrong]\n");
#endif
        return tErr;
    }
    
    tErr=osl_copyFile(strPath,strDestPath);
    if ( tErr != osl_File_E_None )
    {
        oslFileError tErrRemove;
        rtl_uString_release(strPath);
        rtl_uString_release(strDestPath);
        tErrRemove=osl_removeFile(strDestPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_moveFile [copy failed]\n");
#endif
        return tErr;
    }

    tErr=osl_removeFile(strPath);
    
    rtl_uString_release(strPath);
    rtl_uString_release(strDestPath);
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_moveFile [ok]\n");
#endif
    return tErr;
}


oslFileError osl_createDirectoryItemFromHandle( oslFileHandle Handle, oslDirectoryItem* pItem )
{
    oslFileHandleImpl* pHandleImpl=0;
    oslDirectoryItemImpl* pDirItemImpl=0;
    sal_Char pBuffer[OSL_MAX_NATIVE_PATH] = "";
    sal_Char* pszFileName=0;
            
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_createDirectoryItemFromHandle\n");
#endif

    if ( Handle == 0 || pItem == 0 )
    {
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectoryItemFromHandle [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    pHandleImpl = (oslFileHandleImpl*) Handle;
    pDirItemImpl = (oslDirectoryItemImpl*) rtl_allocateMemory(sizeof(oslDirectoryItemImpl));

    if ( pDirItemImpl == 0 )
    {
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectoryItemFromHandle [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

    memset(pDirItemImpl,0,sizeof(pDirItemImpl));
    
    rtl_uString_acquire(pHandleImpl->filepathname);

    pszFileName=oslGetNativeName(pHandleImpl->filepathname);
    if ( pszFileName == 0 )
    {
        if ( pItem != 0 )
        {
            *pItem=0;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectoryItemFromHandle [INVAL]\n");
#endif
        rtl_uString_release(pHandleImpl->filepathname);
        return osl_File_E_INVAL;
    }
            
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"First Copy (basename)\n");
#endif

    strcpy(pBuffer,pszFileName);
        
    rtl_uString_newFromAscii(&(pDirItemImpl->osl_FileName),oslGetBaseName(pBuffer));
    if ( pDirItemImpl->osl_FileName == 0 )
    {
        rtl_freeMemory(pszFileName);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectoryItemFromHandle [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }

/*    pszFileName=oslGetNativeName(pHandleImpl->filepathname);*/
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Second Copy (dirname)\n");
#endif

    strcpy(pBuffer,pszFileName);

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Second Copy (Ok)\n");
#endif

    pDirItemImpl->osl_DirName=0;
    
    rtl_uString_newFromAscii(&(pDirItemImpl->osl_DirName),oslGetDirName(pBuffer));
    if ( pDirItemImpl->osl_DirName == 0 )
    {
        rtl_freeMemory(pszFileName);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_createDirectoryItemFromHandle [NOMEM]\n");
#endif
        return osl_File_E_NOMEM;
    }


#ifdef TRACE_OSL_FILE
    fprintf(stderr,"The Last\n");
#endif

    rtl_freeMemory(pszFileName);
    rtl_uString_release(pHandleImpl->filepathname);
    *pItem=pDirItemImpl;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_createDirectoryItemFromHandle [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_getCanonicalName( rtl_uString* strRequested, rtl_uString** strValid )
{
    int nFD=0;
    int nDone=0;
    sal_Char* pszRequested=0;
    sal_Char* pszRequestedDir=0;
    sal_Char  szRequestedDir[OSL_MAX_FILE_NAME] = "";
    sal_Char  szUnique[OSL_MAX_FILE_NAME] = "";
    sal_uInt32 nIndex=1;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getCanonicalName\n");
#endif
    
    if ( strRequested == 0 || strValid == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getCanonicalName [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    rtl_uString_acquire(strRequested);
    pszRequested = oslGetNativeName(strRequested);

    if ( pszRequested == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getCanonicalName [pszRequested==0]\n");
#endif
        rtl_uString_release(strRequested);
        return osl_File_E_INVAL;        
    }

    nFD = open(pszRequested,O_RDONLY);
    if ( nFD == -1 )
    {
        nFD = errno;
        if ( nFD == ENOENT )
        {
            DIR* pDIR=0;
            
            /* mfe: one of the path entries does not exists ; maybe the file or one of the directories */
            strcpy(szRequestedDir,pszRequested);
            pszRequestedDir=oslGetDirName(szRequestedDir);
            pDIR = opendir(pszRequestedDir);
            if ( pDIR == 0 )
            {
                /* mfe: OK, one of the directories does not exist. Return Error */
                nFD=errno;
                rtl_uString_release(strRequested);
                rtl_freeMemory(pszRequested);
                
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"Out osl_getCanonicalName [no dir]\n");
#endif
                return oslTranslateFileError(nFD);
            }
            /* mfe: file does not exists, everything is OK */
            closedir(pDIR);

            rtl_uString_newFromString(strValid,strRequested);

            rtl_uString_release(strRequested);
#ifdef DEBUG_OSL_FILE
            {
                sal_Char* p;
                p = oslGetNativeName(*strValid);
                if ( p != 0 )
                {
                    fprintf(stderr,"Valid File Name '%s'\n",p);
                    rtl_freeMemory(p);
                }
            }
#endif
            rtl_freeMemory(pszRequested);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_getCanonicalName [ok]\n");
#endif
            return osl_File_E_None;
        }

        /* mfe: some other error occurred */
        rtl_uString_release(strRequested);
        rtl_freeMemory(pszRequested);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getCanonicalName [other error]\n");
#endif
        return oslTranslateFileError(nFD);
    }

    close(nFD);
    
    /* mfe: file exists, make new unique file name */
    while ( nDone == 0 && nIndex < 10000 )
    {
        oslMakeUniqueName(pszRequested,szUnique,nIndex);
        nFD = open(szUnique,O_RDONLY);
        if ( nFD == -1 )
        {
            nFD = errno;
            if ( nFD == ENOENT )
            {
                nDone=1;
            }
        }
        else
        {
            close(nFD);
        }
        nIndex++;
    }

    if ( nIndex == 10000 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getCanonicalName [ok]\n");
#endif
        return osl_File_E_EXIST;
    }

    /* mfe: reuse */
    strcpy(szRequestedDir,"//./");
    strcat(szRequestedDir,szUnique);
    strcpy(szUnique,szRequestedDir);


    rtl_uString_newFromAscii(strValid,szUnique);
    rtl_freeMemory(pszRequested);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getCanonicalName [ok]\n");
#endif
    return osl_File_E_None;
}


oslFileError osl_getAbsolutePath( rtl_uString* strDirBase, rtl_uString* strRelative, rtl_uString** strAbsolute )
{
    sal_Char* pszBaseDir=0;
    sal_Char* pszRelDir=0;
    sal_Char* pszEntry=0;
    sal_Char* pszEntryNext=0;
    sal_Char* pszUNCPath=0;
    sal_Char* pszAbsDir=0;
    int nRet=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getAbsolutePath\n");
#endif
    
    if ( strAbsolute == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [strAbsolute == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( strDirBase == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [strDirBase == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( strRelative == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [strRelative == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strDirBase);
    rtl_uString_acquire(strRelative);
    
    pszBaseDir=oslGetNativeName(strDirBase);
    if ( pszBaseDir == 0 )
    {
        rtl_uString_release(strDirBase);
        rtl_uString_release(strRelative);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [pszUNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

  
    if ( pszBaseDir[0] != '/' )
    {
        rtl_freeMemory(pszBaseDir);
        rtl_uString_release(strDirBase);
        rtl_uString_release(strRelative);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [no heading '/']\n");
#endif
        return osl_File_E_INVAL;
    }

    pszRelDir=oslGetNativeName(strRelative);

    if ( pszRelDir == 0 )
    {
        rtl_uString_release(strRelative);
        rtl_uString_release(strDirBase);
        rtl_freeMemory(pszBaseDir);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [relative path == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pszRelDir[0] == '/' )
    {
        rtl_freeMemory(pszBaseDir);
        rtl_freeMemory(pszRelDir);
        rtl_uString_release(strDirBase);
        rtl_uString_release(strRelative);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [heading '/']\n");
#endif
        return osl_File_E_INVAL;
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"BaseDir is '%s'\n",pszBaseDir);
    fprintf(stderr,"RelDir  is '%s'\n",pszRelDir);
#endif
    
    pszAbsDir=oslDoGetAbsolutePath(pszBaseDir,pszRelDir);
    if ( pszAbsDir == 0 )
    {
        rtl_freeMemory(pszBaseDir);
        rtl_freeMemory(pszRelDir);
        rtl_uString_release(strDirBase);
        rtl_uString_release(strRelative);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getAbsolutePath [pszAbsDir == 0]\n");
#endif
        return osl_File_E_NOENT;
    }

    rtl_uString_newFromAscii(strAbsolute,pszAbsDir);

    rtl_uString_release(strDirBase);
    rtl_uString_release(strRelative);
    
    
    rtl_freeMemory(pszBaseDir);
    rtl_freeMemory(pszRelDir);
    rtl_freeMemory(pszAbsDir);

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getAbsolutePath [ok]\n");
#endif    
    return osl_File_E_None;
}


oslFileError osl_normalizePath( rtl_uString* strSysPath, rtl_uString** strPath )
{
    sal_Char* pszPath=0;
    sal_Char* pszUNCPrefix="//.";
    rtl_uString* strUNCPath=0;
    sal_Char* pszUNCPath=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_normalizePath\n");
#endif

    if ( strSysPath == 0 || strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_normalizePath [strSysPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strSysPath);
        
    pszUNCPath=oslGetNativeName(strSysPath);

    if ( pszUNCPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_normalizePath [pszUNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }
    pszPath=pszUNCPath;
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"pszPath == '%s'\n",pszPath);
#endif
    
    if ( pszPath[0] != '/' )
    {
        rtl_freeMemory(pszPath);
        
        rtl_uString_release(strSysPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_normalizePath [no valid path]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_newFromAscii(&strUNCPath, pszUNCPrefix);

    rtl_uString_newConcat(strPath,strUNCPath,strSysPath);
    rtl_uString_release(strUNCPath);
    
    rtl_uString_release(strSysPath);

    rtl_freeMemory(pszPath);
    
#ifdef DEBUG_OSL_FILE
    {
        sal_Char* p;

        p = oslGetNativeName(*strPath);
        if ( p != 0 )
        {
            fprintf(stderr,"UNC Path is '%s'\n",p);
            rtl_freeMemory(p);
        }
    }
#endif
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_normalizePath [ok]\n");
#endif

    return osl_File_E_None;
}



static void _osl_uString_newFromEncoded( rtl_uString** pstrPath, const sal_Char *pszURL )
{
	rtl_uString*	strTempPath = NULL;
	sal_Unicode*	pszBuffer;
	sal_Unicode*	pSource;
	sal_Unicode*	pDest;
	sal_Unicode cCurrent;

#ifdef TRACE_OSL_FILE
    fprintf(stderr, "_osl_uString_newFromEncoded [input]:\"%s\"\n", pszURL );
#endif

	rtl_string2UString( &strTempPath, pszURL, rtl_str_getLength(pszURL), osl_getThreadTextEncoding(), OUSTRING_TO_OSTRING_CVTFLAGS );

	pSource = rtl_uString_getStr(strTempPath);
	pDest = pszBuffer = rtl_allocateMemory( (rtl_uString_getLength(strTempPath) + 1) * sizeof(sal_Unicode) );

	while ( 0 != (cCurrent = *pSource++) )
	{
		switch( cCurrent )
		{
		case '%':
			{
				sal_Char	cBuffer[256];
				sal_Char*	pEnc = cBuffer;
				rtl_uString*	strEncoded = NULL;

				do
				{
					rtl_String*	strHexBuf = NULL;
					sal_Char*	pEnd;

					rtl_uString2String( &strHexBuf, pSource, 2, osl_getThreadTextEncoding(), OUSTRING_TO_OSTRING_CVTFLAGS );
					*pEnc++ = strtoul( rtl_string_getStr( strHexBuf ), &pEnd, 16 );
					rtl_string_release( strHexBuf );

					pSource += 2;
					if ( '%' == (cCurrent = *pSource) )
						pSource++;
				} while ( '%' == cCurrent  );
				*pEnc = 0;

				rtl_string2UString( &strEncoded, cBuffer, rtl_str_getLength(cBuffer), RTL_TEXTENCODING_UTF8, OUSTRING_TO_OSTRING_CVTFLAGS );

				if ( strEncoded )
				{
					rtl_copyMemory( pDest, rtl_uString_getStr( strEncoded ), rtl_uString_getLength( strEncoded ) * sizeof(sal_Unicode) );
					pDest += rtl_uString_getLength( strEncoded );
					rtl_uString_release( strEncoded );
				}

				break;
			}
		default:
			*pDest++ = cCurrent;
			break;
		}
	}

	*pDest = cCurrent;

	rtl_uString_release( strTempPath );

#ifdef TRACE_OSL_FILE
	{
	rtl_String*	strTemp = NULL;

	rtl_uString2String( &strTemp, pszBuffer, rtl_ustr_getLength(pszBuffer), osl_getThreadTextEncoding(), OUSTRING_TO_OSTRING_CVTFLAGS );
    fprintf(stderr, "_osl_uString_newFromEncoded [output]:\"%s\"\n\n", rtl_string_getStr(strTemp) );
	rtl_string_release( strTemp );
	}
#endif

	rtl_uString_newFromStr( pstrPath, pszBuffer );
	rtl_freeMemory( pszBuffer );
}

oslFileError osl_getNormalizedPathFromFileURL( rtl_uString* strUrlPath, rtl_uString** strPath )
{
    sal_Char* pszUrl = 0;
    sal_Char* pszUrlBegin = 0;    
    sal_Char* pszMethod = 0;
    sal_Char* pszHostNameBegin = 0;
    sal_Char  pszLocalHostName[512] = "";
    sal_Char* pchr = 0;
	sal_Bool  bThreeSlashes = False;
    int nLen = 0;
    int nRet=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getNormalizePathFromFileURL\n");
#endif
    
    if ( strUrlPath == 0 || strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNormalizePathFromFileURL [INVAL]\n");
#endif
        return osl_File_E_INVAL;   
    }

    rtl_uString_acquire(strUrlPath);
    
    pszUrl = oslGetNativeName(strUrlPath);
    if ( pszUrl == 0 )
    {
        rtl_uString_release(strUrlPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNormalizePathFromFileURL [pszUrl == 0]\n");
#endif        
        return osl_File_E_INVAL;  
    }

    pszUrlBegin=pszUrl;
    
    nLen = strlen(pszUrl);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"pszURL == '%s'\n",pszUrl);
#endif
    if ( ( pchr=strstr(pszUrl,":") ) != 0 )
    {
        *pchr=0;
        pszMethod=pszUrl;
        pszUrl=pchr+1;
    }
   
    if ( pszMethod == 0 || strcmp(pszMethod,"file") != 0 )
    {
        rtl_uString_release(strUrlPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNormalizePathFromFileURL [Method != file]\n");
#endif
        return osl_File_E_INVAL;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"pszMethod == '%s'\n",pszMethod);
    fprintf(stderr,"after method pszUrl == '%s'\n",pszUrl);
#endif
    
    if ( *pszUrl != '/' && *(pszUrl+1) != '/' )
    {
        rtl_uString_release(strUrlPath);
        rtl_freeMemory(pszUrlBegin);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNormalizePathFromFileURL [no valid url]\n");
#endif
        return osl_File_E_INVAL;
    }

    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"after valid check pszUrl == '%s'\n",pszUrl);
#endif
    
    pszUrl+=2;
    
    /* check for three slashes in a row, meaning a file URL on local host 
     * without expl. specifying the hostname
     */
    if ( *pszUrl == '/' )
    {
        pszUrl+=1;
    }        
    else
    {
        
/*        pszUrl+=1;*/

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Ok, hostname Path '%s'\n",pszUrl);
#endif
        
        
/*        pszUrl+=1;*/

/*  		if ( *pszUrl == '/' )  */
/*  		{ */
/* 			pszUrl+=2; */
/*  			bThreeSlashes = True; */
/*  		} */
/*  		else	 */
/*  		{	 */
/*			pszUrl+=1;*/
	        pszHostNameBegin=pszUrl;
        
    	    if ( ( pchr=strstr(pszUrl,"@") ) != 0 )
        	{
            	pszUrl=pchr+1;
	            pszHostNameBegin=pszUrl;
	        }

#ifdef DEBUG_OSL_FILE
	        fprintf(stderr,"after name pszUrl == '%s' & hostname == '%s'\n", 
						pszUrl, pszHostNameBegin);
#endif
        
    	    if ( ( pchr=strchr(pszUrl,':') ) != 0 )
	        {
	            *pchr=0;
	            pszUrl=pchr+1;            
	        }

#ifdef DEBUG_OSL_FILE
	        fprintf(stderr,"after port pszUrl == '%s', hostname == '%s'\n",pszUrl,pszHostNameBegin);
#endif
	        
	        if ( (pchr=strchr(pszUrl,'/') ) == 0 )
	        {
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"Out osl_getNormalizePathFromFileURL [no valid url]\n");
#endif
                return osl_File_E_INVAL;
	        }

            pszUrl=pchr+1;
            
#ifdef DEBUG_OSL_FILE
	        fprintf(stderr,"after port pszUrl == '%s'\n",pszUrl);
#endif

/*          if ( *pszUrl != '/' ) */
/*          { */
/*              rtl_uString_release(strUrlPath); */
/*              rtl_freeMemory(pszUrlBegin); */
            
/*  #ifdef TRACE_OSL_FILE */
/*              fprintf(stderr,"Out osl_getNormalizePathFromFileURL [no valid url]\n"); */
/*  #endif */
/*              return osl_File_E_INVAL; */
/*          } */
            
/*  		}	 */
    
/*          if ( ( pchr=strchr(pszUrl,'/') ) == 0 ) */
/*          { */
/*              rtl_uString_release(strUrlPath); */
/*              rtl_freeMemory(pszUrlBegin); */
            
/*  #ifdef TRACE_OSL_FILE */
/*              fprintf(stderr,"Out osl_getNormalizePathFromFileURL [no valid strUrlpath]\n"); */
/*  #endif */
/*              return osl_File_E_INVAL; */
/*          } */

#if 0
		if ( !bThreeSlashes ) 
		{

#ifdef DEBUG_OSL_FILE
	        fprintf(stderr,"before gethostname pszUrl == '%s'\n",pszUrl);
#endif
	        nRet=gethostname(pszLocalHostName,512);
	        if ( nRet < 0 )
	        {
	            nRet=errno;
	            rtl_uString_release(strUrlPath);
	            rtl_freeMemory(pszUrlBegin);
            
#ifdef TRACE_OSL_FILE
	            fprintf(stderr,"Out osl_getNormalizePathFromFileURL [gethostname]\n");
#endif
	            return oslTranslateFileError(nRet);
	        }

#ifdef DEBUG_OSL_FILE
	        fprintf(stderr,"before host cmp == '%s' & '%s'\n",pszHostNameBegin,pszLocalHostName);
#endif
        
	        if ( strcmp(pszLocalHostName,pszHostNameBegin) != 0  )
    	    {
	            rtl_uString_release(strUrlPath);
	            rtl_freeMemory(pszUrlBegin);

#ifdef TRACE_OSL_FILE
    	        fprintf(stderr,"Out osl_getNormalizePathFromFileURL [not local hostname]\n");
#endif
	            return osl_File_E_INVAL;
    	    }
		}
#endif
/* if 0 */        
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"before final step == '%s'\n",pszUrl);    
#endif    
        
    if ( pszUrl != 0 )
    {
        sal_Char* pBuffer = (sal_Char*) rtl_allocateMemory(nLen+8);
        if ( pBuffer == 0 )
        {
            rtl_uString_release(strUrlPath);
            rtl_freeMemory(pszUrlBegin);

#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_getNormalizePathFromFileURL [no mem]\n");
#endif
            return osl_File_E_NOMEM;
        }
        
        strcpy(pBuffer,"//./");
        strcat(pBuffer,pszUrl);

		_osl_uString_newFromEncoded( strPath, pBuffer );

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"returning path '%s'\n",pBuffer);
#endif
        
        rtl_freeMemory(pBuffer);
        rtl_uString_release(strUrlPath);
        rtl_freeMemory(pszUrlBegin);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getNormalizePathFromFileURL [ok]\n");
#endif
        return osl_File_E_None;
    }

    rtl_uString_release(strUrlPath);
    rtl_freeMemory(pszUrlBegin);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getNormalizePathFromFileURL [no path]\n");
#endif
    
    return osl_File_E_INVAL;
}



static void _osl_uString_newEncodedURLFromStr( rtl_uString** pstrEncodedURL, sal_Char *pszURL )
{
	/* Encode non ascii characters within the URL */

	rtl_uString	*strPath = NULL;
	rtl_String	*strUTF8 = NULL;
	sal_Char	*pszEncodedURL;
	sal_Char *	pPathScan;
	sal_Char *	pURLDest;
	sal_Char	cCurrent;

#ifdef TRACE_OSL_FILE
	{
    fprintf(stderr, "_osl_uString_newEncodedURLFromStr [input]:\"%s\"\n", pszURL );
	}
#endif

	/* Create an UNICODE string using system encoding */

	rtl_string2UString( &strPath, pszURL, strlen(pszURL), osl_getThreadTextEncoding(), OUSTRING_TO_OSTRING_CVTFLAGS );

	/* Now create an UTF8 encoded strings due to compatiblity with INetURLObject in TOOLS */

	rtl_uString2String( &strUTF8, rtl_uString_getStr( strPath ), rtl_uString_getLength( strPath ), RTL_TEXTENCODING_UTF8, OUSTRING_TO_OSTRING_CVTFLAGS );

	/* Release the UNICODE string */

	rtl_uString_release( strPath );

	/* Allocate Memory fopr Destination Buffer */

	pszEncodedURL = rtl_allocateMemory( (rtl_string_getLength( strUTF8 ) * 3 + 1)  * sizeof(sal_Char) );

	pURLDest = pszEncodedURL;
	pPathScan = rtl_string_getStr( strUTF8 );

	do
	{
		cCurrent = *pPathScan;

		switch ( cCurrent )
		{
		default:
			if (!( cCurrent >= 'a' && cCurrent <= 'z' || cCurrent >= 'A' && cCurrent <= 'Z' || cCurrent >= '0' && cCurrent <= '9' ))
			{
				sprintf( pURLDest, "%%%02X", (unsigned char)cCurrent );
				pURLDest += 3;
				break;
			}
		case '!':
		case '\'':
		case '(':
		case ')':
		case '*':
		case '-':
		case '.':
		case '_':
		case '~':
		case '$':
		case '&':
		case '+':
		case ',':
		case ':':
		case '=':
		case '@':
		case '/':
		case 0:
			*pURLDest++ = cCurrent;
			break;
		}

		pPathScan++;
	} while ( cCurrent );


	/* Free the UTF8 encoded source buffer */

	rtl_string_release( strUTF8 );

	/* Create a UNICODE string from encoded URL */

#ifdef TRACE_OSL_FILE
	{
    fprintf(stderr, "_osl_uString_newEncodedURLFromStr [output]:\"%s\"\n\n", pszEncodedURL );
	}
#endif
    
	rtl_uString_newFromAscii( pstrEncodedURL, pszEncodedURL );

	/* Release the destination buffer */

	rtl_freeMemory( pszEncodedURL );
}


oslFileError osl_getFileURLFromNormalizedPath( rtl_uString* strNormPath, rtl_uString** strPath )
{
    sal_Char* pszNormPath=0;
    sal_Char* pszFileURL=0;    
    sal_Char* pszUNCPath=0;    
/*    sal_Char  pszHostName[512] = "";*/
    int nLen=0;
    int nRet=0;
    
#ifdef TRACE_OSL_FILE
	{
	rtl_String* strTemp = NULL;
	rtl_uString2String( &strTemp, rtl_uString_getStr(strNormPath), rtl_uString_getLength(strNormPath), osl_getThreadTextEncoding(), OUSTRING_TO_OSTRING_CVTFLAGS );
    fprintf(stderr,"osl_getFileURLFromNormalizedPath [input]:\"%s\"\n", rtl_string_getStr(strTemp) );
	rtl_string_release( strTemp );
	}
#endif

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getFileURLFromNormalizedPath\n");
#endif

    if ( strNormPath == 0 || strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [INVAL]\n");
#endif
        return osl_File_E_INVAL;   
    }

    rtl_uString_acquire(strNormPath);
    
    pszUNCPath=oslGetNativeName(strNormPath);
    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strNormPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [pszNormPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    if ( ! oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strNormPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [! UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszNormPath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strNormPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }


#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Path == '%s'\n",pszNormPath);
#endif
    
    
/*      nRet=gethostname(pszHostName,512); */
/*      if ( nRet < 0 ) */
/*      { */
/*          nRet=errno; */
/*          rtl_uString_release(strNormPath); */
/*          rtl_freeMemory(pszNormPath); */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [gethostname]\n"); */
/*  #endif */
/*          return oslTranslateFileError(nRet); */
/*      } */
    
    nLen=strlen(pszNormPath);

    pszFileURL = (sal_Char*) rtl_allocateMemory(nLen+532);

    if ( pszFileURL == 0 )
    {
        rtl_uString_release(strNormPath);        
        rtl_freeMemory(pszNormPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [pszFileURL == 0]\n");
#endif
        return osl_File_E_NOMEM;
    }

    strcpy(pszFileURL,"file://");
    strcat(pszFileURL,pszNormPath);

	_osl_uString_newEncodedURLFromStr( strPath, pszFileURL );

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Url == '%s'\n",pszFileURL);
#endif
    
    rtl_freeMemory(pszFileURL);
    rtl_freeMemory(pszNormPath);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getFileURLFromNormalizedPath [ok]\n");
#endif

    return osl_File_E_None;
}


oslFileError SAL_CALL osl_getSystemPathFromNormalizedPath( rtl_uString *strNormPath, rtl_uString **strPath)
{
    sal_Char* pszUNCPath=0;
    sal_Char* pszNormPath=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getSystemPathFromNormalizedPath\n");
#endif

    if ( strNormPath == 0 || strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getSystemPathFromNormalizedPath [INVAL]\n");
#endif
        return osl_File_E_INVAL;   
    }

    rtl_uString_acquire(strNormPath);
    
    pszUNCPath=oslGetNativeName(strNormPath);
    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strNormPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getSystemPathFromNormalizedPath [pszNormPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    if ( ! oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strNormPath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getSystemPathFromNormalizedPath [! UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszNormPath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strNormPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getSystemPathFromNormalizedPath [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }


#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Path == '%s'\n",pszNormPath);
#endif

    rtl_uString_newFromAscii(strPath,pszNormPath);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getSystemPathFromNormalizedPath [ok]\n");
#endif

    return osl_File_E_None;
}


oslFileError osl_searchNormalizedPath( rtl_uString* strFilePath, rtl_uString* strSearchPath, rtl_uString** strPath )
{
    sal_Char* pszFilePath=0;
    sal_Char* pszSearchPath=0;
    sal_Char* pszEnvPath=0;
    rtl_uString* strResPath=0;
    sal_Char pszCWD[OSL_MAX_FILE_PATH] = "";
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_searchNormalizedPath\n");
#endif

    if ( strFilePath == 0 || strPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_searchNormalizedPath [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strFilePath);

    if ( strSearchPath != 0 )
    {
        rtl_uString_acquire(strSearchPath);
    }
    
    /* check for file-URL */
    /*      if yes then test for existence */    
    /* check for filePath == UNC */
    /*      if yes then test for existence */
    /* check searchPath != 0 */
    /*      if yes then check filePath in searchPath */
    /* get ENV PATH */
    /* if PATH == 0 */
    /*    check in /usr/local/bin /usr/bin /usr/sbin /bin /sbin */
    /* else */
    /*    check for existence in ENV PATH */
    /* get LD_LIBRARY_PATH */
    /* if LD_LIBRARY_PATH == 0 */
    /*    check in /usr/local/lib /usr/lib /lib */
    /* check for existence in LD_LIBRARY_PATH */
    /* get LD_PRELOAD */
    /* check for existence in LD_PRELOAD */
    /* get LD_RUN_PATH */
    /* check for existence in LD_RUN_PATH */
    /* get LD_LIBRARY_PATH_64 */
    /* check for existence in LD_LIBRARY_PATH_64 */

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Checking for URL\n");    
#endif
    
    /*
     *   Case strFilePath is file-URL
     */

    if ( osl_getNormalizedPathFromFileURL(strFilePath,&strResPath) == osl_File_E_None )
    {
        oslFileError tErr;
        
        tErr=oslCheckForExistence(strResPath);
        if ( tErr == osl_File_E_None )
        {
            rtl_uString_newFromString(strPath,strResPath);
            rtl_uString_release(strResPath);
        }
        
        if ( strFilePath != 0 )
        {    
            rtl_uString_release(strFilePath);
        }
        
        if ( strSearchPath != 0 )
        {    
            rtl_uString_release(strSearchPath);
        }
        
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_searchNormalizedPath [is File-URL]\n");
#endif
        return tErr;
    }

    
    /*
     *   Case strFilePath is UNC Path
     */

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Checking for UNC\n");
#endif
    
/*      if ( pszUNC == 0 ) */
/*      { */
/*          *strPath=0; */
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_searchNormalizedPath [pszUNC == 0]\n"); */
/*  #endif */
/*          return osl_File_E_INVAL; */
/*      } */

/*      if ( oslCheckUNC(pszUNC) ) */
/*      { */
/*          oslFileError tErr; */
        
/*          tErr=oslCheckForExistence(strFilePath); */
/*          if ( tErr == osl_File_E_None ) */
/*          { */
/*              rtl_uString_newFromString(strPath,strFilePath); */
/*          } */
        
/*          rtl_uString_release(strFilePath); */
/*          if ( strSearchPath != 0 ) */
/*          {     */
/*              rtl_uString_release(strSearchPath); */
/*          } */
        
/*  #ifdef TRACE_OSL_FILE */
/*          fprintf(stderr,"Out osl_searchNormalizedPath [is UNC]\n"); */
/*  #endif */
/*          return tErr; */
/*      } */
    
/*      pszFilePath=oslGetNativePath(pszUNC); */
    pszFilePath=oslGetNativeName(strFilePath);
    if ( pszFilePath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_searchNormalizedPath []\n");
#endif
        return osl_File_E_INVAL;        
    }

    
    /*
     *   Case strFilePath is a systemdependent (relative) file or directory
     */

    if ( pszFilePath[0] == '/' )
    {
        sal_Char* pUNCFile = (sal_Char*) malloc(strlen(pszFilePath) + 4 );

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Whuhh: we have an absolute File Path!!!\n");
#endif
        
        strcpy(pUNCFile,"//.");

        strcat(pUNCFile,pszFilePath);
        
        rtl_uString_newFromAscii(strPath,pUNCFile);

        free(pUNCFile);

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Out osl_searchNormalizedPath [absolute FilePath]\n");
#endif
        
        return osl_File_E_None;
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Checking in Search Path\n");
#endif
    
    if ( strSearchPath != 0 )
    {
        
        /* search in SearchPath first */
        pszSearchPath=oslGetNativeName(strSearchPath);
        if ( pszSearchPath != 0 )
        {
            sal_Char* pEntryBegin=0;
            sal_Char* pEntryEnd=0;
            sal_Char* pNewPath=0;
            int nLenPathEntry=0;
            int nLenFilePath=0;
            int nRet=0;

            pEntryBegin=pszSearchPath;
/*            pEntryEnd=pszSearchPath;*/
            nLenFilePath=strlen(pszFilePath);

#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"pszFilePath   '%s'\n",pszFilePath);
            fprintf(stderr,"pszSearchPath '%s'\n",pszSearchPath);
#endif
            while ( pEntryBegin != 0 )
            {
                pEntryEnd=oslSeparatePathEntry(pEntryBegin,';');
                
                if ( pEntryBegin != 0 )
                {
                    /* mfe: we do have some entries in searchpath */            
                    if ( pEntryEnd != 0 )
                    {    
                        *pEntryEnd=0;
                        pEntryEnd++;
                    }
                    
#ifdef DEBUG_OSL_FILE
                    fprintf(stderr,"pszFilePath   '%s'\n",pszFilePath);
                    fprintf(stderr,"Path entry is '%s'\n",pEntryBegin);
#endif
                    nLenPathEntry=strlen(pEntryBegin);
                    pNewPath = (sal_Char*) rtl_allocateMemory(nLenFilePath + nLenPathEntry + 8);
                    strcpy(pNewPath,pEntryBegin);
                    strcat(pNewPath,"/");
                    strcat(pNewPath,pszFilePath);

#ifdef DEBUG_OSL_FILE
                    fprintf(stderr,"testing '%s'\n",pNewPath);
#endif
                    
                    nRet=access(pNewPath,F_OK);
                    if ( nRet < 0 )
                    {
                        /* mfe: some kind of error, next entry please */
#ifdef TRACE_OSL_FILE
                        fprintf(stderr,"Out osl_searchNormalizedPath [access]\n"); 
#endif
                    }
                    else
                    {
                        rtl_uString* strTmp=0;
                        
                        /* mfe: yep, we have it */
                        rtl_uString_newFromAscii(&strTmp,pNewPath);
                        osl_normalizePath(strTmp,strPath);
                        if ( strTmp != 0 )
                        {    
                            rtl_uString_release(strTmp);
                        }
                        
                        rtl_freeMemory(pNewPath);
                        if ( strFilePath != 0 )
                        {    
                            rtl_uString_release(strFilePath);
                        }
                        
                        if ( strSearchPath != 0 )
                        {    
                            rtl_uString_release(strSearchPath);
                        }
                        
                        rtl_freeMemory(pszFilePath);
                        rtl_freeMemory(pszSearchPath);
                
#ifdef TRACE_OSL_FILE
                        fprintf(stderr,"Out osl_searchNormalizedPath [ok]\n");
#endif
                        
                        return osl_File_E_None;
                    }
                }
                else
                {
                    /* mfe: didn't found it */
                    if ( strFilePath != 0 )
                    {    
                        rtl_uString_release(strFilePath);
                    }
                    if ( strSearchPath != 0 )
                    {    
                        rtl_uString_release(strSearchPath);
                    }
                    
                    rtl_freeMemory(pszFilePath);
                    rtl_freeMemory(pszSearchPath);
#ifdef TRACE_OSL_FILE
                  fprintf(stderr,"Out osl_searchNormalizedPath [INVAL]\n");
#endif
                  return osl_File_E_INVAL;
                }
                pEntryBegin=pEntryEnd;
                
            } 
        }
    }
    

    /*
     *   we haven't found it yet
     *   search in 'system paths'
     */

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Checking in System Paths\n");
#endif
    
    /* search in SearchPath first */
    pszEnvPath=getenv("PATH");
    if ( pszEnvPath != 0 )
    {
        sal_Char* pEntryBegin=0;
        sal_Char* pEntryEnd=0;
        sal_Char* pNewPath=0;
        int nLenPathEntry=0;
        int nLenFilePath=0;
        int nRet=0;

        pszSearchPath=strdup(pszEnvPath);
        
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"PATH ENV is '%s'\n",pszSearchPath);
#endif
            
        pEntryBegin=pszSearchPath;
/*            pEntryEnd=pszSearchPath;*/
        nLenFilePath=strlen(pszFilePath);

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"pszFilePath   '%s'\n",pszFilePath);
        fprintf(stderr,"pszSearchPath '%s'\n",pszSearchPath);
#endif
        while ( pEntryBegin != 0 )
        {
            pEntryEnd=oslSeparatePathEntry(pEntryBegin,':');
            
            if ( pEntryBegin != 0 )
            {
                /* mfe: we do have some entries in searchpath */            
                if ( pEntryEnd != 0 )
                {    
                    *pEntryEnd=0;
                    pEntryEnd++;
                }
                
                    
#ifdef DEBUG_OSL_FILE
                    fprintf(stderr,"pszFilePath   '%s'\n",pszFilePath);
                    fprintf(stderr,"Path entry is '%s'\n",pEntryBegin);
#endif
                    nLenPathEntry=strlen(pEntryBegin);
                    pNewPath = (sal_Char*) rtl_allocateMemory(nLenFilePath + nLenPathEntry + 8);
                    strcpy(pNewPath,pEntryBegin);
                    strcat(pNewPath,"/");
                    strcat(pNewPath,pszFilePath);
                    
#ifdef DEBUG_OSL_FILE
                    fprintf(stderr,"testing '%s'\n",pNewPath);
#endif
                    
                    nRet=access(pNewPath,F_OK);
                    if ( nRet < 0 )
                    {
                        /* mfe: some kind of error, next entry please */
#ifdef TRACE_OSL_FILE
                        fprintf(stderr,"Out osl_searchNormalizedPath [access]\n"); 
#endif
                    }
                    else
                    {
                        sal_Char* pszFileAbs=0;

                        pszFileAbs=oslDoGetAbsolutePath(pEntryBegin,pszFilePath);
                        if ( pszFileAbs != 0 )
                        {
                            rtl_uString* strTmp=0;
                            
#ifdef DEBUG_OSL_FILE
                            fprintf(stderr,"Abs File Path is '%s'\n",pszFileAbs);
#endif
                            /* mfe: yep, we have it */
                            rtl_uString_newFromAscii(&strTmp,pszFileAbs);
                            osl_normalizePath(strTmp,strPath);
                            if ( strTmp != 0 )
                            {    
                                rtl_uString_release(strTmp);
                            }
                            
                            
                            rtl_freeMemory(pszFileAbs);
                            rtl_freeMemory(pNewPath);
                            if ( strFilePath != 0 )
                            {
                                rtl_uString_release(strFilePath);
                            }
                            if ( strSearchPath != 0 )
                            {    
                                rtl_uString_release(strSearchPath);
                            }
                            
                            rtl_freeMemory(pszFilePath);
                        
#ifdef TRACE_OSL_FILE
                            fprintf(stderr,"Out osl_searchNormalizedPath [ok]\n");
#endif
                            return osl_File_E_None;
                        }
                        rtl_freeMemory(pszFileAbs);
                    }
                
            }
            else
            {
                /* mfe: didn't found it */
                if ( strFilePath != 0 )
                {    
                    rtl_uString_release(strFilePath);
                }
                if ( strSearchPath != 0 )
                {    
                    rtl_uString_release(strSearchPath);
                }
                
                rtl_freeMemory(pszFilePath);
                rtl_freeMemory(pszSearchPath);
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"Out osl_searchNormalizedPath [INVAL]\n");
#endif
                return osl_File_E_INVAL;
            }
            pEntryBegin=pEntryEnd; 
        }
    }


    rtl_freeMemory(pszSearchPath);
    
    /*
     *   didn't found it yet, use cwd
     */

    pszSearchPath=getcwd(pszCWD,OSL_MAX_FILE_PATH);
    if ( pszSearchPath != 0 )
    {
        int nLenFilePath=0;
        int nLenPathEntry=0;
        int nRet=0;
        sal_Char* pNewPath=0;
        sal_Char* pszFileAbs=0;
        
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"cwd is '%s'\n",pszSearchPath);
#endif

        nLenPathEntry=strlen(pszSearchPath);
        nLenFilePath=strlen(pszFilePath);
        pNewPath = (sal_Char*) rtl_allocateMemory(nLenFilePath + nLenPathEntry + 8);
        strcpy(pNewPath,pszSearchPath);
        strcat(pNewPath,"/");
        strcat(pNewPath,pszFilePath);

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"testing '%s'\n",pNewPath);
#endif
                    
        nRet=access(pNewPath,F_OK);
        if ( nRet < 0 )
        {
            rtl_freeMemory(pNewPath);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_searchNormalizedPath [access]\n"); 
#endif
            return osl_File_E_NOENT;
        }

        pszFileAbs=oslDoGetAbsolutePath(pszSearchPath,pszFilePath);
        if ( pszFileAbs != 0 )
        {
            rtl_uString* strTmp=0;
            
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"Abs File Path is '%s'\n",pszFileAbs);
#endif
                        
            /* mfe: yep, we have it */
            rtl_uString_newFromAscii(&strTmp,pszFileAbs);
            osl_normalizePath(strTmp,strPath);
            if ( strTmp != 0 )
            {    
                rtl_uString_release(strTmp);
            }
            
            
            rtl_freeMemory(pszFileAbs);

            rtl_freeMemory(pNewPath);
            return osl_File_E_None;
        }
        rtl_freeMemory(pszFileAbs);
    }

    if ( strFilePath != 0 )
    {    
        rtl_uString_release(strFilePath);
    }
    if ( strSearchPath != 0 )
    {    
        rtl_uString_release(strSearchPath);
    }
    
    rtl_freeMemory(pszFilePath);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_searchNormalizedPath [ok]\n");
#endif
    return osl_File_E_INVAL;
}


oslFileError osl_setFileAttributes( rtl_uString* strFilePath, sal_uInt64 uAttributes )
{
    sal_Char* pszUNC=0;
    sal_Char* pszFilePath=0;
    int nRet=0;
    mode_t nOldMode=0;
    mode_t nNewMode=0;
    struct stat aFileStat;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_setFileAttributes\n");
#endif
    
    if ( strFilePath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [strFilePath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strFilePath);
    pszUNC=oslGetNativeName(strFilePath);
    if ( pszUNC == 0 )
    {
        rtl_uString_release(strFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [pszFilePath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ! oslCheckUNC(pszUNC) )
    {
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszUNC);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszFilePath=oslGetNativePath(pszUNC) ) == 0 )
    {
        rtl_uString_release(strFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [no native path]\n");
#endif
        return osl_File_E_INVAL;        
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making lstat with '%s'\n",pszFilePath);
#endif
    nRet = lstat(pszFilePath,&aFileStat);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("lstat");
#endif
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [lstat]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    nOldMode=aFileStat.st_mode;
    nNewMode=nOldMode;

    nNewMode&=~S_IRUSR;
    nNewMode&=~S_IWUSR;
    nNewMode&=~S_IXUSR;
    nNewMode&=~S_IRGRP;
    nNewMode&=~S_IWGRP;
    nNewMode&=~S_IXGRP;
    nNewMode&=~S_IROTH;
    nNewMode&=~S_IWOTH;
    nNewMode&=~S_IXOTH;
    
    if ( uAttributes & osl_File_Attribute_OwnRead ) /* ( S_IRUSR & nOldMode )*/
    {
        nNewMode|=S_IRUSR;
    }
    if ( !( uAttributes & osl_File_Attribute_ReadOnly ) && ( uAttributes & osl_File_Attribute_OwnWrite ) ) /* ( S_IWUSR & nOldMode) */
    {
        nNewMode|=S_IWUSR;
    }
    if  ( ( uAttributes & osl_File_Attribute_Executable ) && ( uAttributes & osl_File_Attribute_OwnExe ) ) /* ( S_IXUSR & nOldMode ) */
    {
        nNewMode|=S_IXUSR;
    }
    if ( uAttributes & osl_File_Attribute_GrpRead ) /* ( S_IRGRP & nOldMode ) */
    {
        nNewMode|=S_IRGRP;
    }
    if ( !( uAttributes & osl_File_Attribute_ReadOnly ) && ( uAttributes & osl_File_Attribute_GrpWrite ) ) /* ( S_IWGRP & nOldMode ) */
    {
        nNewMode|=S_IWGRP;
    }
    if ( ( uAttributes & osl_File_Attribute_Executable ) && ( uAttributes & osl_File_Attribute_GrpExe ) ) /* ( S_IXGRP & nOldMode ) */
    {
        nNewMode|=S_IXGRP;
    }
    if ( uAttributes & osl_File_Attribute_OthRead ) /* ( S_IROTH & nOldMode ) */
    {
        nNewMode|=S_IROTH;
    }
    if ( !( uAttributes & osl_File_Attribute_ReadOnly ) && ( uAttributes & osl_File_Attribute_OthWrite ) ) /* ( S_IWOTH & nOldMode ) */
    {
        nNewMode|=S_IWOTH;
    }
    if ( ( uAttributes & osl_File_Attribute_Executable ) && ( uAttributes & osl_File_Attribute_OthExe ) ) /* ( S_IXOTH & nOldMode ) */
    {
        nNewMode|=S_IXOTH;        
    }

    nRet = chmod(pszFilePath,nNewMode);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("chmod");
#endif
        rtl_uString_release(strFilePath);        
        rtl_freeMemory(pszFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [chmod]\n");
#endif
        return oslTranslateFileError(nRet);
    }    

    rtl_uString_release(strFilePath);
    rtl_freeMemory(pszFilePath);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_setFileAttributes [ok]\n");
#endif    
    return osl_File_E_None;
}


oslFileError osl_setFileTime( rtl_uString* strFilePath,
                                       TimeValue* pCreationTime,
                                       TimeValue* pLastAccessTime,
                                       TimeValue* pLastWriteTime )
{
    int nRet=0;
    struct utimbuf aTimeBuffer;
    sal_Char* pszFileName=0;
    sal_Char* pszUNCPath=0;
    struct stat aFileStat;
#ifdef DEBUG_OSL_FILE
    struct tm* pTM=0;
#endif
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_setFileTime\n");
#endif
    
    if ( strFilePath == 0  )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileTime [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strFilePath);

    pszUNCPath=oslGetNativeName(strFilePath);

    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileTime [pszUNCPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ! oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileTime [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszFileName=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strFilePath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileTime [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }

    nRet = lstat(pszFileName,&aFileStat);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("lstat");
#endif
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszFileName);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileAttributes [lstat]\n");
#endif
        return oslTranslateFileError(nRet);
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"File Times are (in localtime):\n");
    pTM=localtime(&aFileStat.st_ctime);
    fprintf(stderr,"CreationTime is '%s'\n",asctime(pTM));
    pTM=localtime(&aFileStat.st_atime);
    fprintf(stderr,"AccessTime   is '%s'\n",asctime(pTM));
    pTM=localtime(&aFileStat.st_mtime);
    fprintf(stderr,"Modification is '%s'\n",asctime(pTM));

    fprintf(stderr,"File Times are (in UTC):\n");    
    fprintf(stderr,"CreationTime is '%s'\n",ctime(&aFileStat.st_ctime));
    fprintf(stderr,"AccessTime   is '%s'\n",ctime(&aTimeBuffer.actime));
    fprintf(stderr,"Modification is '%s'\n",ctime(&aTimeBuffer.modtime));
#endif
    
    if ( pLastAccessTime != 0 )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Changing Access Time!\n");
#endif
        aTimeBuffer.actime=pLastAccessTime->Seconds;
    }
    else
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Access Time unchanged!\n");
#endif
        aTimeBuffer.actime=aFileStat.st_atime;
    }
    
    if ( pLastWriteTime != 0 )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Changing Modification Time!\n");
#endif
        aTimeBuffer.modtime=pLastWriteTime->Seconds;
    }
    else
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Modification Time unchanged!\n");
#endif
        aTimeBuffer.modtime=aFileStat.st_mtime;
    }
    
    /* mfe: Creation time not used here! */
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"File Times are (in localtime):\n");    
    pTM=localtime(&aFileStat.st_ctime);
    fprintf(stderr,"CreationTime now '%s'\n",asctime(pTM));
    pTM=localtime(&aTimeBuffer.actime);
    fprintf(stderr,"AccessTime   now '%s'\n",asctime(pTM));
    pTM=localtime(&aTimeBuffer.modtime);
    fprintf(stderr,"Modification now '%s'\n",asctime(pTM));    

    fprintf(stderr,"File Times are (in UTC):\n");    
    fprintf(stderr,"CreationTime now '%s'\n",ctime(&aFileStat.st_ctime));
    fprintf(stderr,"AccessTime   now '%s'\n",ctime(&aTimeBuffer.actime));
    fprintf(stderr,"Modification now '%s'\n",ctime(&aTimeBuffer.modtime));
#endif
    
    nRet=utime(pszFileName,&aTimeBuffer);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("utime");
#endif
        rtl_uString_release(strFilePath);
        rtl_freeMemory(pszFileName);

#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileTime [utime]\n");
#endif
        return oslTranslateFileError(nRet);
    }

    rtl_uString_release(strFilePath);        
    rtl_freeMemory(pszFileName);

#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_setFileTime [ok]\n");
#endif
    return osl_File_E_None;
}



oslFileError osl_getQuota(oslDirectoryItem Item, sal_uInt32* SoftLimit, sal_uInt32* HardLimit, sal_uInt32* TimeLeft)
{
    int nRet=0;
    sal_Char* pszFileName=0;

#if defined(LINUX)
    struct dqblk aQuota;
    int QuotaCommand;
#endif /* LINUX */

#if defined(SOLARIS)
    int fd;
    struct quotctl aQuotaCtl;
    struct dqblk aQuota;
#endif /* SOLARIS */

#if defined(MACOSX)
    struct dqblk aQuota;
    int QuotaCommand;
#endif /* MACOSX */

    oslDirectoryItemImpl* pDirItemImpl = (oslDirectoryItemImpl*) Item;

    if ( pDirItemImpl == 0 )
    {
        *SoftLimit=0;
        *HardLimit=0;
        *TimeLeft=0;
        return osl_File_E_INVAL;
    }

    if ( pDirItemImpl->osl_FileName == 0 )
    {
        *SoftLimit=0;
        *HardLimit=0;
        *TimeLeft=0;
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(pDirItemImpl->osl_FileName);
    pszFileName=oslGetNativeName(pDirItemImpl->osl_FileName);
    if ( pszFileName == 0 )
    {
        rtl_uString_release(pDirItemImpl->osl_FileName);
        *SoftLimit=0;
        *HardLimit=0;
        *TimeLeft=0;
        return osl_File_E_INVAL;
    }

#if defined(LINUX)
    QuotaCommand = QCMD(Q_GETQUOTA,USRQUOTA);
    nRet = quotactl (QuotaCommand, pszFileName, getuid(), (caddr_t)&aQuota);
#endif /* LINUX */

#if defined(SOLARIS)
    fd = open(pszFileName,O_RDONLY);
    if ( fd < 0 )
    {
        nRet=errno;        
#ifdef DEBUG_OSL_FILE
        perror("open");
#endif
        rtl_uString_release(pDirItemImpl->osl_FileName);
        rtl_freeMemory(pszFileName);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getQuota [open]\n");
#endif        
        return oslTranslateFileError(nRet);
    }

    aQuotaCtl.op=Q_GETQUOTA;
    aQuotaCtl.uid=getuid();
    aQuotaCtl.addr=(char*)&aQuota;
    nRet = ioctl(fd, Q_QUOTACTL, &aQuota);
    close(fd);
#endif /* SOLARIS */

    rtl_uString_release(pDirItemImpl->osl_FileName);

    if ( nRet != 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("quota");
#endif
        *SoftLimit=0;
        *HardLimit=0;
        *TimeLeft=0;
        rtl_freeMemory(pszFileName);
        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getQuota [quota]\n");
#endif        
        return oslTranslateFileError(nRet);
    }

    *SoftLimit=aQuota.dqb_bsoftlimit;
    *HardLimit=aQuota.dqb_bhardlimit;
#if defined(LINUX)
    *TimeLeft=aQuota.dqb_btime;
#endif /* LINUX */

#if defined (SOLARIS)
    *TimeLeft=aQuota.dqb_btimelimit;
#endif /* SOLARIS */

    return osl_File_E_None;
}



/*****************************************************************************/
/*
/*   'Native' implementation Functions */
/*
/*****************************************************************************/

sal_Char* oslDoGetAbsolutePath( sal_Char* pszBaseDir, sal_Char* pszRelDir )
{
    sal_Char* pszAbsDir=0;
    sal_Char* pszEntry=0;
    sal_Char* pszEntryNext=0;
    sal_Char* pszDirBase=0;
    int nRet=0;
    struct stat aFileStat;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslDoGetAbsolutePath\n");
#endif

    if ( pszBaseDir[0] == '.' && pszBaseDir[1] == '\0' )
    {
        sal_Char pszCWD[PATH_MAX + 1];
        pszDirBase=getcwd(pszCWD,PATH_MAX+1);
        pszDirBase = rtl_allocateMemory(strlen(pszCWD)+8);
        if ( pszDirBase == 0 )
        {
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out oslDoGetAbsolutePath [nomem]\n");
#endif
            return 0;            
        }
        strcpy(pszDirBase,pszCWD);
    }
    else
    {
        pszDirBase = rtl_allocateMemory(strlen(pszBaseDir)+8);
        if ( pszDirBase == 0 )
        {
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out oslDoGetAbsolutePath [nomem]\n");
#endif
            return 0;            
        }
        strcpy(pszDirBase,pszBaseDir);        
    }
    
    
    pszAbsDir=malloc(strlen(pszDirBase) + strlen(pszRelDir) + 4 );

    if ( pszAbsDir == 0 )
    {
        rtl_freeMemory(pszDirBase);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslDoGetAbsolutePath [nomem]\n");
#endif
        return 0;
    }
    

    pszAbsDir[0]=0;
    pszEntry=pszDirBase+1;
    pszEntryNext=strchr(pszDirBase+1,'/');
    while ( pszEntry != 0 )
    {
        if ( pszEntryNext != 0 )
        {
            *pszEntryNext=0;
            ++pszEntryNext;
        }
        strcat(pszAbsDir,"/");
        strcat(pszAbsDir,pszEntry);
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"pszEntry == '%s'\n",pszEntry);
#endif
        if ( pszEntryNext != 0 )
        {
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"pszEntryNext == '%s'\n",pszEntryNext);
#endif
        }
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"pszAbsDir == '%s'\n",pszAbsDir);
#endif

        nRet = stat(pszAbsDir,&aFileStat);
        if (nRet < 0 )
        {
            nRet=errno;
#ifdef DEBUG_OSL_FILE
            perror("stat");
#endif
            rtl_freeMemory(pszAbsDir);
            rtl_freeMemory(pszDirBase);
            
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out oslDoGetAbsolutePath [stat]\n");
#endif
            return 0;
            
/*            return oslTranslateFileError(nRet);*/
        }

        if ( ! S_ISDIR(aFileStat.st_mode) )
        {
            rtl_freeMemory(pszAbsDir);
            rtl_freeMemory(pszDirBase);
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out oslDoGetAbsolutePath [not dir]\n");
#endif
            return 0;
            
/*            return osl_File_E_NOTDIR;*/
        }
        pszEntry=pszEntryNext;
        if ( pszEntry != 0 )
        {
            pszEntryNext=strchr(pszEntry,'/');
        }
    }


    pszEntry=pszRelDir;
    pszEntryNext=strchr(pszRelDir+1,'/');
    while ( pszEntry != 0 )
    {
        if ( pszEntryNext != 0 )
        {
            *pszEntryNext=0;
            ++pszEntryNext;
        }
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"pszEntry == '%s'\n",pszEntry);
#endif
        if ( pszEntryNext != 0 )
        {
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"pszEntryNext == '%s'\n",pszEntryNext);
#endif
        }

        if ( strcmp(pszEntry,"..") == 0 || strcmp(pszEntry,".") == 0 || pszEntry[0] == '/' )
        {
            if ( strcmp(pszEntry,"..") == 0 )
            {
				sal_Char* pChar = strrchr(pszAbsDir,'/');
				*pChar = 0;
            }
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"pszAbsDir == '%s'\n",pszAbsDir);
#endif
            
            pszEntry=pszEntryNext;
            if ( pszEntry != 0 )
            {
                pszEntryNext=strchr(pszEntry,'/');
            }
            continue;
        }
        strcat(pszAbsDir,"/");
        strcat(pszAbsDir,pszEntry);
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"pszAbsDir == '%s'\n",pszAbsDir);
#endif

        nRet = stat(pszAbsDir,&aFileStat);
        if ( nRet < 0 )
        {
            nRet=errno;
#ifdef DEBUG_OSL_FILE
            perror("stat");
#endif
/*            rtl_freeMemory(pszBaseDir);*/
            rtl_freeMemory(pszAbsDir);
            rtl_freeMemory(pszDirBase);

#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out oslDoGetAbsolutePath [stat]\n");
#endif
            return 0;
            
/*            return oslTranslateFileError(nRet);*/
        }

        if ( ! S_ISDIR(aFileStat.st_mode) && pszEntryNext != 0 )
        {
/*            rtl_freeMemory(pszBaseDir);*/
            rtl_freeMemory(pszAbsDir);
            rtl_freeMemory(pszDirBase);

#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out oslDoGetAbsolutePath [not dir]\n");
#endif
            return 0;
            
/*            return osl_File_E_NOTDIR;*/
        }

        pszEntry=pszEntryNext;
        if ( pszEntry != 0 )
        {
            pszEntryNext=strchr(pszEntry,'/');
        }
        
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"pszAbsDir == '%s'\n",pszAbsDir);
#endif

    rtl_freeMemory(pszDirBase);
    
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslDoGetAbsolutePath [ok]\n");
#endif    
    return pszAbsDir;

}



/*****************************************************************************/
/* Static Module Functions */
/*****************************************************************************/

static oslFileError oslIsMountPoint(sal_Char* pszFileName, sal_Bool* bMountPoint )
{
    int nRet=0;
    FILE* mntfile=0;
    sal_Char buffer[OSL_MAX_NATIVE_PATH] = "";

#if defined(LINUX)
    struct mntent *pMountEnt=0;
    mntfile = setmntent(MOUNTTAB,"r");
#endif /* LINUX */

#if defined(SOLARIS)
    struct mnttab MountEnt;
    mntfile = fopen(MOUNTTAB,"r");
#endif /* SOLARIS */

    if ( mntfile == 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("mounttab");
#endif
        return oslTranslateFileError(nRet);
    }

    strcpy(buffer,pszFileName);
#ifdef DEBUG_OSL_FILE
/*    fprintf(stderr,"Checking mount of %s\n",buffer);*/
#endif

    /* FIXME isMountPoint: THIS IS NOT THREAD SAFE ! */
#if defined(LINUX)
    pMountEnt=getmntent(mntfile);
    while ( pMountEnt != 0 )
    {
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"mnt_fsname : %s\n",pMountEnt->mnt_fsname);*/
/*        fprintf(stderr,"mnt_dir    : %s\n",pMountEnt->mnt_dir);*/
/*        fprintf(stderr,"mnt_type   : %s\n",pMountEnt->mnt_type);*/
#endif
        if ( strcmp(pMountEnt->mnt_dir,buffer) == 0 )
        {
            fclose(mntfile);
            *bMountPoint=True;
            return osl_File_E_None;
        }
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"=================\n");*/
#endif
        pMountEnt=getmntent(mntfile);
    }
#endif /* LINUX */

#if defined(SOLARIS)
    nRet=getmntent(mntfile,&MountEnt);
    while ( nRet == 0 )
    {
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"mnt_special : %s\n",MountEnt.mnt_special);*/
/*        fprintf(stderr,"mnt_mountp  : %s\n",MountEnt.mnt_mountp);*/
/*        fprintf(stderr,"mnt_type    : %s\n",MountEnt.mnt_fstype);*/
#endif
        if ( strcmp(MountEnt.mnt_mountp,buffer) == 0 )
        {
            fclose(mntfile);
            *bMountPoint=True;
            return osl_File_E_None;
        }
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"=================\n");*/
#endif
        nRet=getmntent(mntfile,&MountEnt);
    }
#endif /* SOLARIS */

    fclose(mntfile);
    *bMountPoint=False;
    return osl_File_E_None;
}


static oslFileError oslChangeFileModes(sal_Char* pszFileName, mode_t nMode, time_t nAcTime, time_t nModTime, uid_t nUID, gid_t nGID)
{
    int nRet=0;
    struct utimbuf aTimeBuffer;

    nRet = chmod(pszFileName,nMode);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("chmod");
#endif
        return oslTranslateFileError(nRet);
    }

    aTimeBuffer.actime=nAcTime;
    aTimeBuffer.modtime=nModTime;
    nRet=utime(pszFileName,&aTimeBuffer);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("utime");
#endif
        return oslTranslateFileError(nRet);
    }

    if ( nUID != getuid() )
    {
        nUID=getuid();
    }

    nRet=chown(pszFileName,nUID,nGID);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("chown");
#endif
        /* mfe: do not return an error here! */
        /* return oslTranslateFileError(nRet);*/
    }

    return osl_File_E_None;
}


static oslFileError oslDoCopy(sal_Char* pszSourceFileName, sal_Char* pszDestFileName, mode_t nMode, size_t nSourceSize, int DestFileExists)
{
    int nRet=0;
    sal_Char pszTmpDestFile[OSL_MAX_NATIVE_PATH] = "";
/*    sal_Char* pC;*/

    if ( DestFileExists )
    {
        strcpy(pszTmpDestFile,pszDestFileName);
        strcat(pszTmpDestFile,".osl-tmp");
        /* FIXME: what if pszTmpDestFile already exists? */
        /*        with getcanonical??? */
        nRet=rename(pszDestFileName,pszTmpDestFile);
    }

    /* mfe: should be S_ISREG */
    if ( !S_ISLNK(nMode) )
    {
        /* copy SourceFile to DestFile */
        nRet = oslDoCopyFile(pszSourceFileName,pszDestFileName,nSourceSize);
    }
    /* mfe: OK redundant at the moment */
    else if ( S_ISLNK(nMode) )
    {
        nRet = oslDoCopyLink(pszSourceFileName,pszDestFileName);
    }
    else
    {
        /* mfe: what to do here? */
        nRet=ENOSYS;
    }

    if ( nRet > 0 && DestFileExists == 1 )
    {
        unlink(pszDestFileName);
        rename(pszTmpDestFile,pszDestFileName);
    }

    if ( nRet > 0 )
    {
        return oslTranslateFileError(nRet);
    }
    
    if ( DestFileExists == 1 )
    {
        unlink(pszTmpDestFile);
    }
    
    return osl_File_E_None;

}


static int oslDoCopyFile(sal_Char* pszSourceFileName, sal_Char* pszDestFileName, size_t nSourceSize)
{
    oslVolumeInfo aVInfo;
    sal_Char pBuffer[OSL_MAX_NATIVE_PATH] = "";
    sal_Char* pC=0;
    rtl_uString* strVolInfo=0;
    oslFileError tErr=osl_File_E_invalidError;
    int SourceFileFD=0;
    int DestFileFD=0;
    int nRet=0;
    void* pSourceFile=0;


    /* mfe: simple check if we have enough space on the destination */
    memset(&aVInfo,0,sizeof(oslVolumeInfo));
    aVInfo.uStructSize=sizeof(oslVolumeInfo);
    strcpy(pBuffer,"//.");
    strcat(pBuffer,pszDestFileName);
    pC=strrchr(pBuffer,'/');
    if ( pC != 0 && pC != pBuffer+3 )
    {
        *pC=0;
    }

/*    rtl_uString_newFromStr(&strVolInfo,pBuffer); */
    rtl_uString_newFromAscii(&strVolInfo,pBuffer);
/*    osl_makeNewUniCode(strVolInfo,pBuffer);*/
    tErr = osl_getVolumeInformation(strVolInfo,&aVInfo,osl_VolumeInfo_Mask_FreeSpace);
    rtl_uString_release(strVolInfo);
    if ( tErr != osl_File_E_None )
    {
        return EIO;
    }
    
    if ( nSourceSize > aVInfo.uFreeSpace )
    {
        return ENOSPC;
    }

    SourceFileFD=open(pszSourceFileName,O_RDONLY);
    if ( SourceFileFD < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("open");
#endif
        return nRet;
    }
    
    DestFileFD=open(pszDestFileName,O_WRONLY|O_CREAT);
    if ( DestFileFD < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("open");
#endif
        close(SourceFileFD);
        return nRet;
    }

    /* FIXME doCopy: fall back code for systems not having mmap */
    /* mmap file -- open dest file -- write once -- fsync it */
    pSourceFile=mmap(0,nSourceSize,PROT_READ,MAP_PRIVATE,SourceFileFD,0);
    if ( pSourceFile == MAP_FAILED )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("mmap");
#endif
        close(SourceFileFD);
        close(DestFileFD);
        return nRet;
    }

    nRet=write(DestFileFD,pSourceFile,nSourceSize);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("write");
#endif
        close(SourceFileFD);
        close(DestFileFD);
        nRet = munmap((char*)pSourceFile,nSourceSize);
        return nRet;
    }

    nRet = fsync(DestFileFD);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("fsync");
#endif
        close(SourceFileFD);
        close(DestFileFD);
        nRet = munmap((char*)pSourceFile,nSourceSize);
        return nRet;
    }

    nRet = munmap((char*)pSourceFile,nSourceSize);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("munmap");
#endif
        close(SourceFileFD);
        return nRet;
    }

    close(SourceFileFD);
    close(DestFileFD);

    return 0;
}


static int oslDoCopyLink(sal_Char* pszSourceFileName, sal_Char* pszDestFileName)
{
    int nRet=0;

    /* mfe: if dest file is symbolic link remove the link and place the file instead (hro says so) */
    /* mfe: if source is a link copy the link and not the file it points to (hro says so) */
    sal_Char pszLinkContent[OSL_MAX_FILE_NAME] = "";
    
    memset(pszLinkContent,0,OSL_MAX_FILE_NAME);

    nRet = readlink(pszSourceFileName,pszLinkContent,OSL_MAX_FILE_NAME);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("readlink");
#endif
        return nRet;
    }
    
    nRet = symlink(pszLinkContent,pszDestFileName);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("symlink");
#endif
        return nRet;
    }
    return 0;
}


static sal_Bool oslCheckUNC(sal_Char* pszPathName)
{
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslCheckUNC [%s]\n",pszPathName);
#endif

    if ( pszPathName == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckUNC [pszPathName == 0]\n");
#endif
        return False;
    }

    if ( strncmp(pszPathName,"//.",3) != 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckUNC [False]\n");
#endif
        return False;
    }

    /* mfe: check for relative FileNames! These are 'verboten'*/
    if ( pszPathName[4] == '.' && ( pszPathName[5] == '.' || !isalnum(pszPathName[5]) ) )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckUNC [pszPathName == .[.]]\n");
#endif
        return 0;
    }
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslCheckUNC [True]\n");
#endif
    return True;
}


static sal_Char* oslGetNativePath(sal_Char* pszUNCPath)
{
    sal_Char* pszUNCPrefix="//./";
    int nLen = 0;
    sal_Char* pszPath=0;
        
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslGetNativePath\n");
#endif

    if ( pszUNCPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativePath [INVAL]\n");
#endif
        return 0;
    }

    nLen = strlen(pszUNCPath);
    pszPath = (sal_Char*) rtl_allocateMemory(nLen+8);

    if ( pszPath == 0 )
    {
        rtl_freeMemory(pszUNCPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativePath [no mem]\n");
#endif
        return 0;
    }
    
    if  ( strncmp(pszUNCPath,pszUNCPrefix,strlen(pszUNCPrefix)) != 0 )
    {
        rtl_freeMemory(pszUNCPath);
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativePath [no native path]\n");
#endif
        return 0;
    }

    strcpy(pszPath,pszUNCPath+3);
    rtl_freeMemory(pszUNCPath);
        
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslGetNativePath [ok]\n");
#endif
    return pszPath;
}


static sal_Char* oslGetNativeName(rtl_uString* pstrPathName)
{
    sal_Char* pC=0;
    sal_Char* pStr=0;
    sal_Unicode* pU=0;
    int nLen=0;
    int index=0;
    int cindex=0;
    
    rtl_String* pStringTmp=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslGetNativeName\n");
#endif
    
    if ( pstrPathName == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativeName [pstrPathName == 0]\n");
#endif
        return 0;
    }

    pU = rtl_uString_getStr(pstrPathName);

    if ( pU == 0 || *pU == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativeName [pU == 0]\n");
#endif
        return 0;        
    }
    
    rtl_uString2String(&pStringTmp, pU, rtl_uString_getLength(pstrPathName),
                       osl_getThreadTextEncoding(), 0);

    
    if ( pStringTmp == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativeName [pStringTmp == 0]\n");
#endif
        return 0;
    }

    pC = rtl_string_getStr(pStringTmp);
    if ( pC == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativeName [pC == 0]\n");
#endif
        return 0;
    }
    
    /* FIXME: convertion from sal_Unicode/rtl_uString to sal_Char !!! */
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Converted File Name is '%s'\n",pC); 
#endif

    nLen=strlen(pC);

#if 0
    nLen = rtl_uString_getLength(pstrPathName);

    fprintf(stderr,"Length is '%i' (%i)\n",nLen,(nLen/2)+8);
    
    pStr=(sal_Char*) rtl_allocateMemory((nLen/2)+8);
#endif
    
    pStr=(sal_Char*) rtl_allocateMemory(nLen+8);
    if ( pStr == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslGetNativeName [no mem]\n");
#endif
        return 0;
    }

/*      pC=(sal_Char*) pU; */
    
/*      for ( index = 0 ; index < ( nLen * sizeof(sal_Unicode)); index++ ) */
/*      { */
/*          fprintf(stderr,"%i == %c \n",index,pC[index]);         */
/*      } */

    
/*      for ( index = 0 ; index <= ( nLen * sizeof(sal_Unicode)) ; index+=sizeof(sal_Unicode) ) */
/*      { */
/*  /*          fprintf(stderr," %c ",pC[index]); */
        
/*          pStr[cindex]=pC[index]; */
/*          ++cindex; */
/*      } */
    
    strcpy(pStr,pC);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Char String is '%s' \n",pStr);
#endif
    
    rtl_string_release(pStringTmp);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslGetNativeName [ok]\n");
#endif
    return pStr;
}

static sal_Char* oslSeparatePathEntry(sal_Char* pszPath, sal_Char cSeparator)
{
    sal_Char* pC=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslSeparatePathEntry\n");
#endif

    pC=strchr(pszPath,cSeparator);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslSeparatePathEntry [ok]\n");
#endif
    return pC;
}


static oslFileError oslCheckForExistence(rtl_uString* strFilePath)
{
    sal_Char* pszUNCPath=0;
    sal_Char* pszFilePath=0;
    int nRet=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslCheckForExistence\n");
#endif

    if ( strFilePath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckForExistence [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strFilePath);

    pszUNCPath=oslGetNativeName(strFilePath);
    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strFilePath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckForExistence [pszNormPath == 0]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    if ( ! oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strFilePath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckForExistence [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszFilePath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strFilePath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckForExistence [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making access test with '%s'\n",pszFilePath);
#endif
    
    nRet=access(pszFilePath,F_OK);
    if ( nRet < 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("access");
#endif
        rtl_freeMemory(pszFilePath);

#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslCheckForExistence [access]\n");
#endif
        return oslTranslateFileError(nRet);
    }
    
    rtl_freeMemory(pszFilePath);
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslCheckForExistence [ok]\n");
#endif
    return osl_File_E_None;
}

static void oslMakeUniqueName(sal_Char* pszRequested, sal_Char* pszUnique, sal_uInt32 nIndex)
{
    sal_Char pszTmpRequested[OSL_MAX_FILE_NAME] = "";
    sal_Char* pC=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  oslMakeUniqueName\n");
#endif
    
    if ( pszUnique == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslMakeUniqueName [pszUnique==0]\n");
#endif
        return;
    }

    if ( pszRequested == 0 )
    {
        pszUnique[0]=0;
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out oslMakeUniqueName [pszRequested==0]\n");
#endif
        return;
    }

    strcpy(pszTmpRequested,pszRequested);
    
    if ( ( pC = strrchr(pszRequested,'.') ) != 0 )
    {
        *pC=0;
        ++pC;
        sprintf(pszUnique,"%s-%lu.%s",pszTmpRequested,nIndex,pC);
    }
    else
    {
        sprintf(pszUnique,"%s-%lu",pszRequested,nIndex);
    }
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out oslMakeUniqueName [%s]\n",pszUnique);
#endif
    return;
}


static sal_Char* oslGetDirName(sal_Char* pszPath)
{
    sal_Char* pC=0;

    if ( (pC=strrchr(pszPath,'/')) == 0 )
    {
        return 0;
    }
    else
    {
        *(pC+1)=0;
    }
    return pszPath;
}


static sal_Char* oslGetBaseName(sal_Char* pszPath)
{
    sal_Char* pC=0;

    if ( (pC=strrchr(pszPath,'/')) == 0 )
    {
        return 0;
    }
    return pC+1;
}


static oslFileError oslTranslateFileError(int nErr)
{
    oslFileError tErr = osl_File_E_invalidError;
#ifdef TRACE_OSL_FILE
/*    fprintf(stderr,"In  oslTranslateFileError (err == %i [%s])\n",nErr,sys_errlist[nErr]);*/
#endif
    
    switch(nErr)
    {
        case 0:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_None");
#endif
            tErr=osl_File_E_None;
            break;
            
        case EPERM:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_PERM");
#endif
            tErr = osl_File_E_PERM;
            break;

        case ENOENT:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOENT");
#endif
            tErr = osl_File_E_NOENT;
            break;

        case ESRCH:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_SRCH");
#endif
            tErr = osl_File_E_SRCH;
            break;

        case EINTR:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_INTR");
#endif
            tErr = osl_File_E_INTR;
            break;

        case EIO:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_IO");
#endif
            tErr = osl_File_E_IO;
            break;

        case ENXIO:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NXIO");
#endif
            tErr = osl_File_E_IO;
            break;
            
        case E2BIG:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_2BIG");
#endif
            tErr = osl_File_E_2BIG;
            break;
            
        case ENOEXEC:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOEXEC");
#endif
            tErr = osl_File_E_NOEXEC;
            break;

        case EBADF:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_BADF");
#endif
            tErr = osl_File_E_BADF;
            break;

        case ECHILD:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_CHILD");
#endif
            tErr = osl_File_E_CHILD;
            break;

        case EAGAIN:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_AGAIN");
#endif
            tErr = osl_File_E_AGAIN;
            break;

        case ENOMEM:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOMEM");
#endif
            tErr = osl_File_E_NOMEM;
            break;

        case EACCES:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_ACCES");
#endif
            tErr = osl_File_E_ACCES;
            break;

        case EFAULT:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_FAULT");
#endif
            tErr = osl_File_E_FAULT;
            break;

        case EBUSY:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_BUSY");
#endif
            tErr = osl_File_E_BUSY;
            break;

        case EEXIST:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_EXIST");
#endif
            tErr = osl_File_E_EXIST;
            break;
            
        case EXDEV:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_XDEV");
#endif
            tErr = osl_File_E_XDEV;
            break;
            
        case ENODEV:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NODEV");
#endif
            tErr = osl_File_E_NODEV;
            break;
            
        case ENOTDIR:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOTDIR");
#endif
            tErr = osl_File_E_NOTDIR;
            break;

        case EISDIR:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_ISDIR");
#endif
            tErr = osl_File_E_ISDIR;
            break;

        case EINVAL:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_INVAL");
#endif
            tErr = osl_File_E_INVAL;
            break;
            
        case ENFILE:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NFILE");
#endif
            tErr = osl_File_E_NFILE;
            break;

        case EMFILE:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_MFILE");
#endif
            tErr = osl_File_E_MFILE;
            break;

        case ENOTTY:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOTTY");
#endif
            tErr = osl_File_E_NOTTY;
            break;

        case EFBIG:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_FBIG");
#endif
            tErr = osl_File_E_FBIG;
            break;

        case ENOSPC:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOSPC");
#endif
            tErr = osl_File_E_NOSPC;
            break;

        case ESPIPE:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_SPIPE");
#endif
            tErr = osl_File_E_SPIPE;
            break;
            
        case EROFS:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_ROFS");
#endif
            tErr = osl_File_E_ROFS;
            break;

        case EMLINK:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_MLINK");
#endif
            tErr = osl_File_E_MLINK;
            break;
            
        case EPIPE:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_PIPE");
#endif
            tErr = osl_File_E_PIPE;
            break;

        case EDOM:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_DOM");
#endif
            tErr = osl_File_E_DOM;
            break;

        case ERANGE:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_RANGE");
#endif
            tErr = osl_File_E_RANGE;
            break;

        case EDEADLK:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_DEADLK");
#endif
            tErr = osl_File_E_DEADLK;
            break;
            
        case ENAMETOOLONG:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NAMETOOLONG");
#endif
            tErr = osl_File_E_NAMETOOLONG;
            break;
            
        case ENOLCK:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOLCK");
#endif
            tErr = osl_File_E_NOLCK;
            break;

        case ENOSYS: 
#ifdef DEBUG_OSL_FILE
           fprintf(stderr,"osl_File_E_NOSYS");
#endif
           tErr = osl_File_E_NOSYS;
            break;

        case ENOTEMPTY:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOTEMPTY");
#endif
            tErr = osl_File_E_NOTEMPTY;
            break;

        case ELOOP:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_LOOP");
#endif
            tErr = osl_File_E_LOOP;
            break;

#ifndef MACOSX
        case EILSEQ:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_ILSEQ");
#endif
            tErr = osl_File_E_ILSEQ;
            break;
#endif /* MACOSX */
            
#ifndef MACOSX
        case ENOLINK:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_NOLINK");
#endif
            tErr = osl_File_E_NOLINK;
            break;
#endif /* MACOSX */
            
#ifndef MACOSX
        case EMULTIHOP:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_MULTIHOP");
#endif
            tErr = osl_File_E_MULTIHOP;
            break;
#endif /* MACOSX */

        case EUSERS:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_USERS");
#endif
            tErr = osl_File_E_USERS;
            break;

        case EOVERFLOW:
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_OVERFLOW");
#endif
            tErr = osl_File_E_OVERFLOW;
            break;
            
        default:
            /* FIXME translateFileError: is this alright? Or add a new one: osl_File_E_Unknown? */
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"osl_File_E_Unknown");
#endif
            tErr = osl_File_E_invalidError;
            break;

    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"\n");
#endif
    
#ifdef TRACE_OSL_FILE
/*    fprintf(stderr,"Out oslTranslateFileError\n");*/
#endif
    return tErr;
}

static sal_Char* osl_getDirName(sal_Char* pszPath)
{
    sal_Char* pC;

    if ( (pC=strrchr(pszPath,'/')) == 0 )
    {
        if ( pszPath[1] == 0 )
        {
            *pC = 0;
        }
        
        return 0;
    }
    else
    {
        *pC=0;
    }
    return pszPath;
}


static sal_Char* osl_getBaseName(sal_Char* pszPath)
{
    sal_Char* pC;

    if ( (pC=strrchr(pszPath,'/')) == 0 )
    {
        return 0;
    }
    return (pC+1);
}



/*************************************************************************
*
*               GENERIC FLOPPY FUNCTIONS
*
*************************************************************************/

oslFileError osl_unmountVolumeDevice( oslVolumeDeviceHandle Handle )
{
    oslFileError tErr = osl_File_E_NOSYS;

    tErr = osl_unmountFloppy(Handle);

    return tErr;
}

oslFileError osl_automountVolumeDevice( oslVolumeDeviceHandle Handle )
{
    oslFileError tErr = osl_File_E_NOSYS;

    tErr = osl_mountFloppy(Handle);

    return tErr;
}


oslFileError osl_getVolumeDeviceMountPath( oslVolumeDeviceHandle Handle, rtl_uString **pstrPath )
{
    oslVolumeDeviceHandleImpl* pItem = (oslVolumeDeviceHandleImpl*) Handle;
    sal_Char Buffer[PATH_MAX] = "";

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getVolumeDeviceMountPath\n");
#endif    
     
    if ( pItem == 0 || pstrPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeDeviceMountPath [INVAL]\n");
#endif     
        return osl_File_E_INVAL;
    }

    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getVolumeDeviceMountPath [INVAL]\n");
#endif     
        return osl_File_E_INVAL;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Handle is:\n");
    osl_printFloppyHandle(pItem);
#endif

    sprintf(Buffer,"//.%s",pItem->pszMountPoint);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Mount Point is: '%s'\n",Buffer);
#endif
    
    rtl_uString_newFromAscii(pstrPath,Buffer);

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getVolumeDeviceMountPath [ok]\n");
#endif     
    
    return osl_File_E_None;
}


oslFileError SAL_CALL osl_acquireVolumeDeviceHandle( oslVolumeDeviceHandle Handle )
{
    oslVolumeDeviceHandleImpl* pItem =(oslVolumeDeviceHandleImpl*) Handle;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_acquireVolumeDeviceHandle\n");
#endif     
    
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_acquireVolumeDeviceHandle [INVAL]\n");
#endif     
        return osl_File_E_INVAL;
    }
    
    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_acquireVolumeDeviceHandle [INVAL]\n");
#endif        
        return osl_File_E_INVAL;
    }

    ++pItem->RefCount;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_acquireVolumeDeviceHandle [ok]\n");
#endif     
    
    return osl_File_E_None;
}


oslFileError osl_releaseVolumeDeviceHandle( oslVolumeDeviceHandle Handle )
{
    oslVolumeDeviceHandleImpl* pItem =(oslVolumeDeviceHandleImpl*) Handle;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_releaseVolumeDeviceHandle\n");
#endif
    
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_releaseVolumeDeviceHandle [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_releaseVolumeDeviceHandle [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    --pItem->RefCount;

    if ( pItem->RefCount < 0 )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"negativ RefCount in Handle!!!\n");
#endif
    }
    
    
    if ( pItem->RefCount == 0 )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"freeing!!!\n");
#endif
        free(pItem);
    }

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_releaseVolumeDeviceHandle [ok]\n");
#endif
    
    return osl_File_E_None;
}


oslFileError osl_getRealPath(rtl_uString* strPath, rtl_uString** strRealPath)
{
    sal_Char* pszPath=0;
    sal_Char* pszUNCPath=0;
    sal_Char pszRealPath[PATH_MAX] = "";
    sal_Char pszRealUNC[PATH_MAX] = "";
    sal_Char* pChr=0;
    int nRet;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getRealPath\n");
#endif
    
    if ( strPath == 0 || strRealPath == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getRealPath [INVAL]\n");
#endif
        return osl_File_E_INVAL;
    }

    rtl_uString_acquire(strPath);

    pszUNCPath=oslGetNativeName(strPath);
    if ( pszUNCPath == 0 )
    {
        rtl_uString_release(strPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getRealPath [no path]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    if ( ! oslCheckUNC(pszUNCPath) )
    {
        rtl_uString_release(strPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getRealPath [!UNC]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( ( pszPath=oslGetNativePath(pszUNCPath) ) == 0 )
    {
        rtl_uString_release(strPath);        
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getRealPath [no native path]\n");
#endif
        return osl_File_E_INVAL;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"making access test with '%s'\n",pszPath);
#endif
    
    
    pChr=realpath(pszPath,pszRealPath);
    
    if ( pChr == 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("access");
#endif
        rtl_freeMemory(pszPath);

#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getRealPath [realpath]\n");
#endif
        return oslTranslateFileError(nRet);
    }
    
    strcpy(pszRealUNC,"//.");
    strcat(pszRealUNC,pszRealPath);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"real path is '%s'\n",pszRealUNC);
#endif
    
    
    rtl_uString_newFromAscii(strRealPath,pszRealUNC);
    
    rtl_freeMemory(pszPath);

    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getRealPath [ok]\n");
#endif
    
    return osl_File_E_None;
}


/*************************************************************************
*
*               SOLARIS FLOPPY FUNCTIONS
*
*************************************************************************/

#if defined(SOLARIS)

static oslVolumeDeviceHandle osl_isFloppyDrive(const sal_Char* pszPath)
{
    oslVolumeDeviceHandleImpl* pItem=0;
    sal_Bool bRet=sal_False;
    sal_Bool bMount=sal_False;
    sal_Char MountPoint[PATH_MAX] = "";
    
    int nRet;
    sal_Char DevBuf[PATH_MAX] = "";
    struct dirent* de=0;
        
#ifdef TRACE_OSL_FILE   
    fprintf(stderr,"In  osl_isFloppyDrive\n");
#endif
    
    bRet=osl_getFloppyMountEntry(pszPath,&bMount,MountPoint);

    if ( bMount == sal_False )
    {
        if ( strncmp("/floppy/floppy",pszPath,strlen("/floppy/floppy")) != 0 )
        {    
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_isFloppyDrive [not mount point]\n");
#endif
            return 0;
        }
        strncpy(MountPoint,pszPath,strlen("/floppy/floppy")+1);
        
        /*        strcpy(MountPoint,pszPath);*/
    }


#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Mount point may be '%s'\n",MountPoint);
#endif


    pItem = (oslVolumeDeviceHandleImpl*) malloc(sizeof(oslVolumeDeviceHandleImpl));
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE   
        fprintf(stderr,"Out osl_isFloppyDrive [NOMEM]\n");
#endif        
        return osl_File_E_NOMEM;
    }
    memset(pItem,0,sizeof(oslVolumeDeviceHandleImpl));
    pItem->ident[0]='O';
    pItem->ident[1]='V';
    pItem->ident[2]='D';
    pItem->ident[3]='H';
    pItem->RefCount=1;

    bRet = osl_checkFloppyPath(MountPoint,pItem->pszFilePath,pItem->pszDevice);

    if ( bRet == sal_False )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"no more floppies\n");
#endif
        free(pItem);
        return 0;
    }

    MountPoint[0]=0;
    
    bRet = osl_isFloppyMounted(pItem->pszFilePath,MountPoint);

    if ( MountPoint[0] == '\0' )
    {
        strncpy(pItem->pszMountPoint, pszPath, strlen("/floppy/floppy")+1);
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"assuming mount point to be '%s'\n",MountPoint);
#endif
    }
    else
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"mount point is '%s'\n",MountPoint);
#endif
        strcpy(pItem->pszMountPoint,MountPoint);
    }
    
    
#ifdef DEBUG_OSL_FILE
    osl_printFloppyHandle(pItem);
    fprintf(stderr,"Out osl_isFloppyDrive [ok]\n");
#endif
    return (oslVolumeDeviceHandle) pItem;
}

#endif /* SOLARIS */


#if defined(SOLARIS)

static oslFileError osl_mountFloppy(oslVolumeDeviceHandle hFloppy)
{
    oslVolumeDeviceHandleImpl* pItem = (oslVolumeDeviceHandleImpl*) hFloppy;
    sal_Char MountPoint[PATH_MAX]= "";
    
    int nRet=0;
    sal_Bool bRet = sal_False;
    sal_Char pszCmd[512] = "";
    sal_Char pszDevice[PATH_MAX]= "";
    sal_Char* pChr=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_mountFloppy\n");
#endif

    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [pItem == 0]\n");
#endif
        
        return osl_File_E_INVAL;
    }

    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [invalid handle]\n");
#endif
        return osl_File_E_INVAL;
    }
    

    strcpy(pszDevice,pItem->pszDevice);

    pChr=osl_getBaseName(pszDevice);
    
    
    sprintf(pszCmd,"eject -q %s > /dev/null 2>&1", pChr);
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"executing '%s'\n",pszCmd);
#endif
    
    nRet = system(pszCmd);    

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"call returned '%i'\n",nRet);
    fprintf(stderr,"exit status is '%i'\n", WEXITSTATUS(nRet));
#endif
    
    switch ( WEXITSTATUS(nRet) )
    {
    case 0:
        /* FIXME: test if floppy is really mounted */
        /* FIXME: test if "device" or path is in Mount Path */
        /* FIXME: does the floppy link show to a valid directory */
        /* FIXME:      and if directory is accessable */
        bRet = osl_isFloppyMounted(pItem->pszFilePath,MountPoint);
        if ( bRet == sal_True )
        {
#ifdef DEBUG_OSL_FILE            
            fprintf(stderr,"checking access of path '%s'\n",pItem->pszFilePath);
#endif
            bRet = osl_checkFloppyPath(MountPoint,pItem->pszFilePath,pItem->pszDevice);
            if ( bRet == sal_True )
            {    
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"seems to be mounted already\n");
#endif
                strcpy(pItem->pszMountPoint,MountPoint);
                
                return osl_File_E_None;
            }
        }
        
        pItem->pszMountPoint[0]=0;
            
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [mount ok]\n");
#endif
        return osl_File_E_BUSY;
        break;

    case 1:
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [no disk]\n");
#endif
        return osl_File_E_BUSY;
        break;

    default:
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [is busy]\n");
#endif
        return osl_File_E_BUSY;
        break;
    }

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_mountFloppy [not reached]\n");
#endif
    
    return osl_File_E_BUSY;
}

#endif /* SOLARIS */


#if defined(SOLARIS)

static oslFileError osl_unmountFloppy(oslVolumeDeviceHandle hFloppy)
{
    oslVolumeDeviceHandleImpl* pItem = (oslVolumeDeviceHandleImpl*) hFloppy;    
    
    int nRet=0;
    sal_Char pszCmd[512] = "";
    sal_Char pszDevice[PATH_MAX]= "";    
    sal_Char MountPoint[PATH_MAX]= "";
    sal_Bool bRet = sal_False;
    sal_Char* pChr=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_unmountFloppy\n");
#endif    

    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [pItem==0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [invalid handle]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    strcpy(pszDevice,pItem->pszDevice);

    pChr=osl_getBaseName(pszDevice);
    
    sprintf(pszCmd,"eject %s > /dev/null 2>&1",pChr);
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"executing '%s'\n",pszCmd);
#endif
    
    nRet = system(pszCmd);
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"call returned  '%i'\n",nRet);
    fprintf(stderr,"exit status is '%i'\n", WEXITSTATUS(nRet));
#endif
    
    switch ( WEXITSTATUS(nRet) )
    {
    case 0:        
        /* FIXME: test if floppy is really mounted */
        /* FIXME: test if device or path is in Mount Path */
        /* FIXME: does the link show to a valid/current directory */
        /* FIXME:    is the dir really THE floppy dir */
        bRet = osl_isFloppyMounted(pItem->pszFilePath,MountPoint);
        if ( bRet == sal_True )
        {
#ifdef DEBUG_OSL_FILE            
            fprintf(stderr,"checking access of path '%s'",pItem->pszFilePath);
#endif
            bRet = osl_checkFloppyPath(pItem->pszMountPoint,pItem->pszFilePath,pItem->pszDevice);
            if ( bRet == sal_True )
            {    
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"seems to be mounted already\n");
#endif
                return osl_File_E_BUSY;
            }
        }
        
        sprintf(pszCmd,"eject %s > /dev/null 2>&1",pszDevice);
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"executing '%s'\n",pszCmd);
#endif
        nRet = system(pszCmd);    
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"call returned  '%i'\n",nRet);
        fprintf(stderr,"exit status is '%i'\n", WEXITSTATUS(nRet));
#endif
        /* FIXME: check for all returns values? */
        if ( WEXITSTATUS(nRet) != 4 )
        {
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_unmountFloppy [unmount error]\n");
#endif    
            return osl_File_E_BUSY;
        }
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [save to eject]\n");
#endif    
        /* FIXME: clear mount point in handle */
        pItem->pszMountPoint[0]=0;
        return osl_File_E_None;
        break;
        
    case 1:
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [no disk]\n");
#endif    
        return osl_File_E_NODEV;
        break;

    case 4:
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [save to unmount]\n");
#endif    
        return osl_File_E_None;
        break;

    default:
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [is busy]\n");
#endif    
        return osl_File_E_BUSY;
        break;
    }

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_unmountFloppy [not reached]\n");
#endif    
    
    return osl_File_E_BUSY;
}

#endif /* SOLARIS */


#if defined(SOLARIS)

static sal_Bool osl_getFloppyMountEntry(const sal_Char* pszPath, sal_Bool* bMountPoint, sal_Char* pBuffer)
{
    
    int nRet;
    FILE* mntfile;
    sal_Char* pChr=0;
    sal_Char buffer[PATH_MAX] = "";
    
    struct mnttab MountEnt;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getFloppyMountEntry\n");
#endif
    
    mntfile = fopen(MOUNTTAB,"r");

    if ( mntfile == 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("mounttab");
#endif
        return sal_False;
    }

    strcpy(buffer,pszPath);
#ifdef DEBUG_OSL_FILE
/*    fprintf(stderr,"Checking mount of '%s'\n",buffer);*/
#endif

    nRet=getmntent(mntfile,&MountEnt);
    while ( nRet == 0 )
    {
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"mnt_special : %s\n",MountEnt.mnt_special);*/
/*        fprintf(stderr,"mnt_mountp  : %s\n",MountEnt.mnt_mountp);*/
/*        fprintf(stderr,"mnt_type    : %s\n",MountEnt.mnt_fstype);*/
#endif
        pChr=buffer;
        
        while ( pChr != 0 )
        {
#ifdef DEBUG_OSL_FILE
/*            fprintf(stderr,"Checking mount of '%s'\n",MountEnt.mnt_mountp);*/
/*            fprintf(stderr,"checking against  '%s'\n",pChr);*/
#endif
            if ( strcmp(MountEnt.mnt_mountp,buffer) == 0 )
            {
                fclose(mntfile);
                *bMountPoint=sal_True;
#ifdef DEBUG_OSL_FILE
/*                fprintf(stderr,"mnt_special : %s\n",MountEnt.mnt_special);*/
/*                fprintf(stderr,"mnt_mountp  : %s\n",MountEnt.mnt_mountp);*/
/*                fprintf(stderr,"mnt_type    : %s\n",MountEnt.mnt_fstype);*/
#endif
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"Out osl_getFloppyMountEntry [found]\n");
#endif
                strcpy(pBuffer,MountEnt.mnt_mountp);
                return sal_True;
            }

            pChr = osl_getDirName(buffer);
        }
        
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"=================\n");*/
#endif
        nRet=getmntent(mntfile,&MountEnt);
        strcpy(buffer,pszPath);
    }

    fclose(mntfile);
    *bMountPoint=sal_False;
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getFloppyMountEntry [not found]\n");
#endif
    
    return sal_False;
}

#endif /* SOLARIS */


#if defined(SOLARIS)
static sal_Bool osl_isFloppyMounted(sal_Char* pszPath, sal_Char* pszMountPoint)
{
  
    sal_Char pLink[PATH_MAX]      = "";
    sal_Char MountPoint[PATH_MAX] = "";
    sal_Char DirName[PATH_MAX]    = "";
    int nRet=0;
    DIR* pDir;
    sal_Char* pLinkName=0;
    sal_Char* pDirName=0;
    
    struct dirent* de=0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_isFloppyMounted\n");
#endif

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"testing '%s'\n",pszPath);
#endif
    
    nRet = readlink(pszPath,pLink,PATH_MAX);
    if ( nRet < 0 )
    {
#ifdef DEBUG_OSL_FILE
        perror("readlink");
        fprintf(stderr,"not existing floppy link : no floppy drive present?\n");
        fprintf(stderr,"Out osl_isFloppyMounted [readlink]\n");
#endif
        pszMountPoint[0]=0;
        return sal_False;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"link is '%s'\n",pLink);
#endif
    
    strcpy(DirName,pszPath);
    
    pLinkName=oslGetBaseName(pLink);
    pDirName=oslGetDirName(DirName);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"linkname is '%s'\n",pLinkName);
    fprintf(stderr,"pDirName is '%s'\n",pDirName);
#endif

    if ( pLinkName[0] == '/' || pDirName[strlen(pDirName) -1 ] == '/' )
    {
        sprintf(MountPoint,"%s%s",pDirName,pLinkName);
    }
    else
    {
        sprintf(MountPoint,"%s/%s",pDirName,pLinkName);
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Link should be '%s'\n",MountPoint);
#endif
    
    /* pLink is mount point? */

    /* check for mounted media */
    pDir = opendir(MountPoint);
    if ( pDir == 0 )
    {
#ifdef DEBUG_OSL_FILE
        perror("opendir");
        fprintf(stderr,"floppy link shows to no valid directory : seems not to be mounted\n");
        fprintf(stderr,"Out osl_isFloppyMounted [opendir]\n");
#endif
        pszMountPoint[0]=0;
        return sal_False;
    }
#ifdef DEBUG_OSL_FILE
    else
    {
        fprintf(stderr,"is openable\n");
    }
#endif

    
    de=readdir(pDir);
    if ( de == 0 )
    {
        /* case not mounted */
#ifdef DEBUG_OSL_FILE
        perror("readdir");
#endif
        closedir(pDir);
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"Seems to be not mounted\n");
        fprintf(stderr,"Out osl_isFloppyMounted [readdir]\n");        
#endif
        pszMountPoint[0]=0;
        return sal_False;
    }
    else
    {
        /* is mounted */
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"is readable i.e. seems to be mounted\n");
        fprintf(stderr,"copying Mountpoint '%s'\n",MountPoint);
#endif
        strcpy(pszMountPoint,MountPoint);
    }
    closedir(pDir);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Out osl_isFloppyMounted [ok]\n");        
#endif
    
    return sal_True;
}

#endif /* SOLARIS */


#if defined(SOLARIS)
static sal_Bool osl_checkFloppyPath(sal_Char* pszPath, sal_Char* pszFilePath, sal_Char* pszDevicePath)
{
    int nRet=0;
    int index=0;
    struct stat buf;
    struct stat linkbuf;
    sal_Char DirBuf[PATH_MAX] = "";

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_checkFloppyPath\n");
#endif

    memset(&buf,0,sizeof(buf));
    
    nRet = stat(pszPath,&buf);
    if ( nRet < 0 )
    {
#ifdef DEBUG_OSL_FILE
        perror("file stat");
#endif
    }
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"inode number %lu\n",buf.st_ino);
#endif
    
    while ( nRet == 0 )
    {
        sprintf(DirBuf,"/floppy/floppy%i",index);

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"checking possible '%s'\n",DirBuf);
#endif
        
        memset(&linkbuf,0,sizeof(linkbuf));
        nRet = stat(DirBuf,&linkbuf);
        if ( nRet < 0 )
        {
#ifdef DEBUG_OSL_FILE
            perror("link stat");
#endif
            if ( buf.st_ino  == 0 && linkbuf.st_ino == 0 )
            {
#ifdef DEBUG_OSL_FILE
                fprintf(stderr,"Assuming path is not floppy!!!\n",DirBuf);
#endif                
#ifdef TRACE_OSL_FILE
                fprintf(stderr,"Out osl_checkFloppyPath [not floppy]\n");
#endif
                return sal_False;
            }
        }

#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"inode number %lu\n",linkbuf.st_ino);
#endif
        if ( linkbuf.st_ino == buf.st_ino )
        {
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"Found File Path '%s'\n",DirBuf);
#endif
            strcpy(pszFilePath,DirBuf);
            sprintf(pszDevicePath,"/dev/diskette%i",index);    
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"copying Device '%s'\n",pszDevicePath);
#endif
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_checkFloppyPath [found]\n");
#endif

            return sal_True;
        }
        ++index;
    }

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"no floppy path found!!!\n",DirBuf);
#endif        

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_checkFloppyPath [no floppy found]\n");
#endif

    
    return sal_False;
}

#endif /* SOLARIS */
    

/*************************************************************************
*
*               LINUX FLOPPY FUNCTIONS
*
*************************************************************************/

#if defined(LINUX)
static oslVolumeDeviceHandle osl_isFloppyDrive(const sal_Char* pszPath)
{
    oslVolumeDeviceHandleImpl* pItem=0;    
    sal_Bool bRet = sal_False;
    
#ifdef TRACE_OSL_FILE   
    fprintf(stderr,"In  osl_isFloppyDrive\n");
#endif


    pItem = (oslVolumeDeviceHandleImpl*) malloc(sizeof(oslVolumeDeviceHandleImpl));
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE   
        fprintf(stderr,"Out osl_isFloppyDrive [NOMEM]\n");
#endif        
        return 0;
    }    
    memset(pItem,0,sizeof(oslVolumeDeviceHandleImpl));
    pItem->ident[0]='O';
    pItem->ident[1]='V';
    pItem->ident[2]='D';
    pItem->ident[3]='H';
    pItem->RefCount=1;

    bRet=osl_getFloppyMountEntry(pszPath,pItem);

    if ( bRet == sal_False )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_isFloppyDrive [not a floppy]\n");
#endif
        free(pItem);
        return 0;
    }

    
#ifdef DEBUG_OSL_FILE
    osl_printFloppyHandle(pItem);
#endif
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_isFloppyDrive [ok]\n");
#endif
    
    return (oslVolumeDeviceHandle) pItem;
}

#endif /* LINUX */


#if defined(LINUX)

static oslFileError osl_mountFloppy(oslVolumeDeviceHandle hFloppy)
{       
    sal_Bool bRet = sal_False;
    oslVolumeDeviceHandleImpl* pItem=0;
    int nRet;
    sal_Char  pszCmd[PATH_MAX] = "";
    sal_Char* pszMountProg = "mount";
    sal_Char* pszSuDo = 0;
    sal_Char* pszTmp = 0;
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_mountFloppy\n");
#endif

    pItem = (oslVolumeDeviceHandleImpl*) hFloppy;
    
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [pItem == 0]\n");
#endif
        
        return osl_File_E_INVAL;
    }

    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_mountFloppy [invalid handle]\n");
#endif
        return osl_File_E_INVAL;
    }
    
    bRet = osl_isFloppyMounted(pItem);
    if ( bRet == sal_True )
    {
#ifdef DEBUG_OSL_FILE
        fprintf(stderr,"detected mounted floppy at '%s'\n",pItem->pszMountPoint);
#endif
        return osl_File_E_BUSY;
    }

    /* mfe: we can't use the mount(2) system call!!!   */
    /*      even if we are root                        */
    /*      since mtab is not updated!!!               */
    /*      but we need it to be updated               */
    /*      some "magic" must be done                  */

/*      nRet = mount(pItem->pszDevice,pItem->pszMountPoint,0,0,0); */
/*      if ( nRet != 0 ) */
/*      { */
/*          nRet=errno; */
/*  #ifdef DEBUG_OSL_FILE */
/*          perror("mount"); */
/*  #endif */
/*      } */

    pszTmp = getenv("SAL_MOUNT_MOUNTPROG");
    if ( pszTmp != 0 )
    {
        pszMountProg=pszTmp;
    }

    pszTmp=getenv("SAL_MOUNT_SU_DO");
    if ( pszTmp != 0 )
    {
        pszSuDo=pszTmp;
    }
    
    if ( pszSuDo != 0 )
    {
        sprintf(pszCmd,"%s %s %s %s",pszSuDo,pszMountProg,pItem->pszDevice,pItem->pszMountPoint);
    }
    else
    {
        sprintf(pszCmd,"%s %s",pszMountProg,pItem->pszMountPoint);        
    }
    
    
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"executing '%s'\n",pszCmd);
#endif
    
    nRet = system(pszCmd);    

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"call returned '%i'\n",nRet);
    fprintf(stderr,"exit status is '%i'\n", WEXITSTATUS(nRet));
#endif
    
    
    switch ( WEXITSTATUS(nRet) )
    {
    case 0:
        nRet=0;
        break;
        
    case 2:
        nRet=EPERM;
        break;

    case 4:
        nRet=ENOENT;
        break;

    case 8:
        nRet=EINTR;
        break;

    case 16:
        nRet=EPERM;
        break;

    case 32:
        nRet=EBUSY;
        break;

    case 64:
        nRet=EAGAIN;
        break;

    default:
        nRet=EBUSY;
        break;
    }
    
    return oslTranslateFileError(nRet);
}
#endif /* LINUX */


#if defined(LINUX)
static oslFileError osl_unmountFloppy(oslVolumeDeviceHandle hFloppy)
{
    oslVolumeDeviceHandleImpl* pItem=0;
    int nRet=0;
    sal_Char pszCmd[PATH_MAX] = "";
    sal_Char* pszTmp = 0;
    sal_Char* pszSuDo = 0;
    sal_Char* pszUmountProg = "umount";
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_unmountFloppy\n");
#endif    

    pItem = (oslVolumeDeviceHandleImpl*) hFloppy;
    
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [pItem==0]\n");
#endif
        return osl_File_E_INVAL;
    }

    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_unmountFloppy [invalid handle]\n");
#endif
        return osl_File_E_INVAL;
    }

    /* mfe: we can't use the umount(2) system call!!!  */
    /*      even if we are root                        */
    /*      since mtab is not updated!!!               */
    /*      but we need it to be updated               */
    /*      some "magic" must be done                  */

/*      nRet=umount(pItem->pszDevice); */
/*      if ( nRet != 0 ) */
/*      { */
/*          nRet = errno; */
        
/*  #ifdef DEBUG_OSL_FILE */
/*          perror("mount"); */
/*  #endif */
/*      } */
    
    
    pszTmp = getenv("SAL_MOUNT_UMOUNTPROG");
    if ( pszTmp != 0 )
    {
        pszUmountProg=pszTmp;
    }

    pszTmp = getenv("SAL_MOUNT_SU_DO");
    if ( pszTmp != 0 )
    {
        pszSuDo=pszTmp;
    }

    if ( pszSuDo != 0 )
    {    
        sprintf(pszCmd,"%s %s %s",pszSuDo,pszUmountProg,pItem->pszMountPoint);        
    }
    else
    {
        sprintf(pszCmd,"%s %s",pszUmountProg,pItem->pszMountPoint);
    }
    

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"executing '%s'\n",pszCmd);
#endif
    
    nRet = system(pszCmd);    

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"call returned '%i'\n",nRet);
    fprintf(stderr,"exit status is '%i'\n", WEXITSTATUS(nRet));
#endif
    
    switch ( WEXITSTATUS(nRet) )
    {
    case 0:
        nRet=0;
        break;

    default:
        nRet=EBUSY;
        break;
    }
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_unmountFloppy [ok]\n");
#endif    

    return oslTranslateFileError(nRet);
    
/*    return osl_File_E_None;*/
}

#endif /* LINUX */

#if defined(LINUX)
static sal_Bool osl_getFloppyMountEntry(const sal_Char* pszPath, oslVolumeDeviceHandleImpl* pItem)
{
    struct mntent *pMountEnt=0;
    sal_Char buffer[PATH_MAX] = "";
    FILE* mntfile=0;
    int nRet=0;
    
    mntfile = setmntent(MOUNTTAB,"r");

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_getFloppyMountEntry\n");
#endif

    strcpy(buffer,pszPath);
#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Checking mount of %s\n",buffer);
#endif

    
    if ( mntfile == 0 )
    {
        nRet=errno;
#ifdef DEBUG_OSL_FILE
        perror("mounttab");
#endif
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_getFloppyMountEntry [mntfile]\n");
#endif
        return sal_False;
    }

    pMountEnt=getmntent(mntfile);
    while ( pMountEnt != 0 )
    {
#ifdef DEBUG_OSL_FILE
/*           fprintf(stderr,"mnt_fsname : %s\n",pMountEnt->mnt_fsname); */
/*           fprintf(stderr,"mnt_dir    : %s\n",pMountEnt->mnt_dir); */
/*        fprintf(stderr,"mnt_type   : %s\n",pMountEnt->mnt_type);*/
#endif
        if ( strcmp(pMountEnt->mnt_dir,buffer) == 0 &&
             strncmp(pMountEnt->mnt_fsname,"/dev/fd",strlen("/dev/fd")) == 0 )
        {
            strcpy(pItem->pszMountPoint,pMountEnt->mnt_dir);
            strcpy(pItem->pszFilePath,pMountEnt->mnt_dir);
            strcpy(pItem->pszDevice,pMountEnt->mnt_fsname);
            
            fclose(mntfile);
#ifdef DEBUG_OSL_FILE
            fprintf(stderr,"Mount Point found '%s'\n",pItem->pszMountPoint);
#endif
#ifdef TRACE_OSL_FILE
            fprintf(stderr,"Out osl_getFloppyMountEntry [found]\n");
#endif
            return sal_True;
        }
#ifdef DEBUG_OSL_FILE
/*        fprintf(stderr,"=================\n");*/
#endif
        pMountEnt=getmntent(mntfile);
    }    

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_getFloppyMountEntry [not found]\n");
#endif

    fclose(mntfile);
    return sal_False;
}

#endif /* LINUX */

#if defined(LINUX)
static sal_Bool osl_isFloppyMounted(oslVolumeDeviceHandleImpl* pDevice)
{
    sal_Char buffer[PATH_MAX] = "";
    oslVolumeDeviceHandleImpl* pItem=0;    
    sal_Bool bRet=0;

#ifdef TRACE_OSL_FILE
    fprintf(stderr,"In  osl_isFloppyMounted\n");
#endif

    pItem = (oslVolumeDeviceHandleImpl*) malloc(sizeof(oslVolumeDeviceHandleImpl));
    if ( pItem == 0 )
    {
#ifdef TRACE_OSL_FILE   
        fprintf(stderr,"Out osl_isFloppyDrive [NOMEM]\n");
#endif        
        return osl_File_E_NOMEM;
    }
    memset(pItem,0,sizeof(oslVolumeDeviceHandleImpl));
    pItem->ident[0]='O';
    pItem->ident[1]='V';
    pItem->ident[2]='D';
    pItem->ident[3]='H';
    pItem->RefCount=1;
    
    strcpy(buffer,pDevice->pszMountPoint);

#ifdef DEBUG_OSL_FILE
    fprintf(stderr,"Checking mount of %s\n",buffer);
#endif

    bRet = osl_getFloppyMountEntry(buffer,pItem);

    if ( bRet == sal_False )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_isFloppyMounted [not mounted]\n");
#endif
        return sal_False;
    }

    if (strcmp(pItem->pszMountPoint, pDevice->pszMountPoint) == 0 &&
        strcmp(pItem->pszDevice,pDevice->pszDevice) == 0)
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Out osl_isFloppyMounted [is mounted]\n");
#endif
        free(pItem);
        return sal_True;
    }
    
#ifdef TRACE_OSL_FILE
    fprintf(stderr,"Out osl_isFloppyMounted [may be EBUSY]\n");
#endif

    free(pItem);
    return sal_False;    
}
#endif /* LINUX */


/*************************************************************************
*
*               MACOSX FLOPPY FUNCTIONS
*
*************************************************************************/

#if defined(MACOSX)
static oslVolumeDeviceHandle osl_isFloppyDrive(const sal_Char* pszPath)
{
    return NULL;
}
#endif /* MACOSX */

#if defined(MACOSX)
static oslFileError osl_mountFloppy(oslVolumeDeviceHandle hFloppy)
{
    return osl_File_E_BUSY;
}
#endif /* MACOSX */

#if defined(MACOSX)
static oslFileError osl_unmountFloppy(oslVolumeDeviceHandle hFloppy)
{
    return osl_File_E_BUSY;
}
#endif /* MACOSX */

#if defined(MACOSX)
static sal_Bool osl_getFloppyMountEntry(const sal_Char* pszPath, oslVolumeDeviceHandleImpl* pItem)
{
    return sal_False;
}
#endif /* MACOSX */

#if defined(MACOSX)
static sal_Bool osl_isFloppyMounted(oslVolumeDeviceHandleImpl* pDevice)
{
    return sal_False;
}
#endif /* MACOSX */


#ifdef DEBUG_OSL_FILE
static void osl_printFloppyHandle(oslVolumeDeviceHandleImpl* pItem)
{
    if (pItem == 0 )
    {
        fprintf(stderr,"NULL Handle\n");
        return;
    }
    if ( pItem->ident[0] != 'O' || pItem->ident[1] != 'V' || pItem->ident[2] != 'D' || pItem->ident[3] != 'H' )
    {
#ifdef TRACE_OSL_FILE
        fprintf(stderr,"Invalid Handle]\n");
#endif
        return;
    }
    

    fprintf(stderr,"MountPoint : '%s'\n",pItem->pszMountPoint);
    fprintf(stderr,"FilePath   : '%s'\n",pItem->pszFilePath);
    fprintf(stderr,"Device     : '%s'\n",pItem->pszDevice);

    return;    
}
#endif

